// VRML97/X3D file "" converted to C++ with white_dune

#include <stddef.h> // for NULL

class X3dSceneGraph;

class X3dNode {
public:
    X3dNode() {
        m_parent = NULL;
        m_protoRoot = NULL;
        m_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
    }
    X3dNode *m_parent;
    X3dNode *m_protoRoot;
    void *m_data;
    virtual int getType() const = 0;
};

typedef void (*X3dCallback)(X3dNode *node, void *dataptr);
typedef bool (*X3dProcessEventCallback)(X3dNode *node, const char *event, void *dataptr);


class X3dColorRGBA : public X3dNode {
public:
    float* color;
    int color_length;
    void* extra_data;

    int getType() const { return 40; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColorRGBA() {
        color = NULL;
        color_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColorRGBA::renderCallback = NULL;
X3dCallback X3dColorRGBA::treeRenderCallback = NULL;
X3dCallback X3dColorRGBA::doWithDataCallback = NULL;
X3dCallback X3dColorRGBA::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColorRGBA::processEventCallback = NULL;

int X3dColorRGBAType = 40;



class X3dVolumeEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    X3dNode* coord;
    float* direction;
    int direction_length;
    int* coordIndex;
    int coordIndex_length;
    bool internal;
    void* extra_data;

    int getType() const { return 323; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dVolumeEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coord = NULL;
        direction = NULL;
        direction_length = 0;
        coordIndex = NULL;
        coordIndex_length = 0;
        internal = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (coord)
                coord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (coord)
                coord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dVolumeEmitter::renderCallback = NULL;
X3dCallback X3dVolumeEmitter::treeRenderCallback = NULL;
X3dCallback X3dVolumeEmitter::doWithDataCallback = NULL;
X3dCallback X3dVolumeEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dVolumeEmitter::processEventCallback = NULL;

int X3dVolumeEmitterType = 323;



class X3dHAnimJoint : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    X3dNode** displacers;
    int displacers_length;
    float* limitOrientation;
    int limitOrientation_length;
    float* llimit;
    int llimit_length;
    const char* name;
    int* skinCoordIndex;
    int skinCoordIndex_length;
    float* skinCoordWeight;
    int skinCoordWeight_length;
    float* stiffness;
    int stiffness_length;
    float* ulimit;
    int ulimit_length;
    void* extra_data;

    int getType() const { return 107; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimJoint() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        displacers = NULL;
        displacers_length = 0;
        limitOrientation = NULL;
        limitOrientation_length = 0;
        llimit = NULL;
        llimit_length = 0;
        name = NULL;
        skinCoordIndex = NULL;
        skinCoordIndex_length = 0;
        skinCoordWeight = NULL;
        skinCoordWeight_length = 0;
        stiffness = NULL;
        stiffness_length = 0;
        ulimit = NULL;
        ulimit_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimJoint::renderCallback = NULL;
X3dCallback X3dHAnimJoint::treeRenderCallback = NULL;
X3dCallback X3dHAnimJoint::doWithDataCallback = NULL;
X3dCallback X3dHAnimJoint::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimJoint::processEventCallback = NULL;

int X3dHAnimJointType = 107;



class X3dWindPhysicsModel : public X3dNode {
public:
    float* direction;
    int direction_length;
    bool enabled;
    float gustiness;
    float speed;
    float turbulence;
    void* extra_data;

    int getType() const { return 328; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dWindPhysicsModel() {
        direction = NULL;
        direction_length = 0;
        enabled = false;
        gustiness = 0.0f;
        speed = 0.0f;
        turbulence = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dWindPhysicsModel::renderCallback = NULL;
X3dCallback X3dWindPhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dWindPhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dWindPhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dWindPhysicsModel::processEventCallback = NULL;

int X3dWindPhysicsModelType = 328;



class X3dTransform : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    void* extra_data;
    int num_route_source;
    X3dNode **route_sources;

    int getType() const { return 289; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTransform() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTransform::renderCallback = NULL;
X3dCallback X3dTransform::treeRenderCallback = NULL;
X3dCallback X3dTransform::doWithDataCallback = NULL;
X3dCallback X3dTransform::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTransform::processEventCallback = NULL;

int X3dTransformType = 289;



class X3dCylinderSensor : public X3dNode {
public:
    bool isActive;
    bool isOver;
    float* rotation_changed;
    int rotation_changed_length;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    bool autoOffset;
    float* axisRotation;
    int axisRotation_length;
    const char* description;
    float diskAngle;
    bool enabled;
    float maxAngle;
    float minAngle;
    float offset;
    void* extra_data;

    int getType() const { return 65; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCylinderSensor() {
        autoOffset = false;
        axisRotation = NULL;
        axisRotation_length = 0;
        description = NULL;
        diskAngle = 0.0f;
        enabled = false;
        maxAngle = 0.0f;
        minAngle = 0.0f;
        offset = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCylinderSensor::renderCallback = NULL;
X3dCallback X3dCylinderSensor::treeRenderCallback = NULL;
X3dCallback X3dCylinderSensor::doWithDataCallback = NULL;
X3dCallback X3dCylinderSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCylinderSensor::processEventCallback = NULL;

int X3dCylinderSensorType = 65;



class X3dCommonSurfaceShader : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 42; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCommonSurfaceShader() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCommonSurfaceShader::renderCallback = NULL;
X3dCallback X3dCommonSurfaceShader::treeRenderCallback = NULL;
X3dCallback X3dCommonSurfaceShader::doWithDataCallback = NULL;
X3dCallback X3dCommonSurfaceShader::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCommonSurfaceShader::processEventCallback = NULL;

int X3dCommonSurfaceShaderType = 42;



class X3dPlaneSensor : public X3dNode {
public:
    bool isOver;
    bool isActive;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    float* translation_changed;
    int translation_changed_length;
    bool autoOffset;
    float* axisRotation;
    int axisRotation_length;
    const char* description;
    bool enabled;
    float* maxPosition;
    int maxPosition_length;
    float* minPosition;
    int minPosition_length;
    float* offset;
    int offset_length;
    void* extra_data;

    int getType() const { return 196; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPlaneSensor() {
        autoOffset = false;
        axisRotation = NULL;
        axisRotation_length = 0;
        description = NULL;
        enabled = false;
        maxPosition = NULL;
        maxPosition_length = 0;
        minPosition = NULL;
        minPosition_length = 0;
        offset = NULL;
        offset_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPlaneSensor::renderCallback = NULL;
X3dCallback X3dPlaneSensor::treeRenderCallback = NULL;
X3dCallback X3dPlaneSensor::doWithDataCallback = NULL;
X3dCallback X3dPlaneSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPlaneSensor::processEventCallback = NULL;

int X3dPlaneSensorType = 196;



class X3dPositionInterpolator2D : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 211; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPositionInterpolator2D() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPositionInterpolator2D::renderCallback = NULL;
X3dCallback X3dPositionInterpolator2D::treeRenderCallback = NULL;
X3dCallback X3dPositionInterpolator2D::doWithDataCallback = NULL;
X3dCallback X3dPositionInterpolator2D::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPositionInterpolator2D::processEventCallback = NULL;

int X3dPositionInterpolator2DType = 211;



class X3dHAnimSite : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    X3dNode** children;
    int children_length;
    const char* name;
    void* extra_data;

    int getType() const { return 109; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimSite() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        children = NULL;
        children_length = 0;
        name = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimSite::renderCallback = NULL;
X3dCallback X3dHAnimSite::treeRenderCallback = NULL;
X3dCallback X3dHAnimSite::doWithDataCallback = NULL;
X3dCallback X3dHAnimSite::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimSite::processEventCallback = NULL;

int X3dHAnimSiteType = 109;



class X3dImageTexture : public X3dNode {
public:
    const char** url;
    int url_length;
    bool repeatS;
    bool repeatT;
    X3dNode* textureProperties;
    X3dNode** effects;
    int effects_length;
    const char* alphaChannel;
    void* extra_data;

    int getType() const { return 112; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dImageTexture() {
        url = NULL;
        url_length = 0;
        repeatS = false;
        repeatT = false;
        textureProperties = NULL;
        effects = NULL;
        effects_length = 0;
        alphaChannel = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dImageTexture::renderCallback = NULL;
X3dCallback X3dImageTexture::treeRenderCallback = NULL;
X3dCallback X3dImageTexture::doWithDataCallback = NULL;
X3dCallback X3dImageTexture::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dImageTexture::processEventCallback = NULL;

int X3dImageTextureType = 112;



class X3dScreenFontStyle : public X3dNode {
public:
    const char** family;
    int family_length;
    bool horizontal;
    const char** justify;
    int justify_length;
    const char* language;
    bool leftToRight;
    float spacing;
    const char* style;
    bool topToBottom;
    float pointSize;
    void* extra_data;

    int getType() const { return 232; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScreenFontStyle() {
        family = NULL;
        family_length = 0;
        horizontal = false;
        justify = NULL;
        justify_length = 0;
        language = NULL;
        leftToRight = false;
        spacing = 0.0f;
        style = NULL;
        topToBottom = false;
        pointSize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScreenFontStyle::renderCallback = NULL;
X3dCallback X3dScreenFontStyle::treeRenderCallback = NULL;
X3dCallback X3dScreenFontStyle::doWithDataCallback = NULL;
X3dCallback X3dScreenFontStyle::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScreenFontStyle::processEventCallback = NULL;

int X3dScreenFontStyleType = 232;



class X3dNavigationInfo : public X3dNode {
public:
    bool set_bind;
    double bindTime;
    bool isBound;
    bool transitionComplete;
    float* avatarSize;
    int avatarSize_length;
    bool headlight;
    float speed;
    double transitionTime;
    const char** transitionType;
    int transitionType_length;
    const char** type;
    int type_length;
    float visibilityLimit;
    const char* blendingSort;
    void* extra_data;

    int getType() const { return 166; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNavigationInfo() {
        avatarSize = NULL;
        avatarSize_length = 0;
        headlight = false;
        speed = 0.0f;
        transitionTime = 0;
        transitionType = NULL;
        transitionType_length = 0;
        type = NULL;
        type_length = 0;
        visibilityLimit = 0.0f;
        blendingSort = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNavigationInfo::renderCallback = NULL;
X3dCallback X3dNavigationInfo::treeRenderCallback = NULL;
X3dCallback X3dNavigationInfo::doWithDataCallback = NULL;
X3dCallback X3dNavigationInfo::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNavigationInfo::processEventCallback = NULL;

int X3dNavigationInfoType = 166;



class X3dRemoteSelectionGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 224; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dRemoteSelectionGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dRemoteSelectionGroup::renderCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::treeRenderCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::doWithDataCallback = NULL;
X3dCallback X3dRemoteSelectionGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dRemoteSelectionGroup::processEventCallback = NULL;

int X3dRemoteSelectionGroupType = 224;



class X3dCoordinateInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 57; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCoordinateInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCoordinateInterpolator::renderCallback = NULL;
X3dCallback X3dCoordinateInterpolator::treeRenderCallback = NULL;
X3dCallback X3dCoordinateInterpolator::doWithDataCallback = NULL;
X3dCallback X3dCoordinateInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCoordinateInterpolator::processEventCallback = NULL;

int X3dCoordinateInterpolatorType = 57;



class X3dDirectionalLight : public X3dNode {
public:
    float ambientIntensity;
    float* color;
    int color_length;
    float* direction;
    int direction_length;
    bool global;
    float intensity;
    bool on;
    float* projectionRectangle;
    int projectionRectangle_length;
    float* projectionLocation;
    int projectionLocation_length;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 67; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dDirectionalLight() {
        ambientIntensity = 0.0f;
        color = NULL;
        color_length = 0;
        direction = NULL;
        direction_length = 0;
        global = false;
        intensity = 0.0f;
        on = false;
        projectionRectangle = NULL;
        projectionRectangle_length = 0;
        projectionLocation = NULL;
        projectionLocation_length = 0;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dDirectionalLight::renderCallback = NULL;
X3dCallback X3dDirectionalLight::treeRenderCallback = NULL;
X3dCallback X3dDirectionalLight::doWithDataCallback = NULL;
X3dCallback X3dDirectionalLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dDirectionalLight::processEventCallback = NULL;

int X3dDirectionalLightType = 67;



class X3dCollision : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    double collideTime;
    bool isActive;
    X3dNode** children;
    int children_length;
    bool enabled;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* proxy;
    void* extra_data;

    int getType() const { return 31; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCollision() {
        children = NULL;
        children_length = 0;
        enabled = false;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        proxy = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (proxy)
                proxy->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (proxy)
                proxy->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCollision::renderCallback = NULL;
X3dCallback X3dCollision::treeRenderCallback = NULL;
X3dCallback X3dCollision::doWithDataCallback = NULL;
X3dCallback X3dCollision::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCollision::processEventCallback = NULL;

int X3dCollisionType = 31;



class X3dNurbsGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float tessellationScale;
    void* extra_data;

    int getType() const { return 172; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNurbsGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        tessellationScale = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNurbsGroup::renderCallback = NULL;
X3dCallback X3dNurbsGroup::treeRenderCallback = NULL;
X3dCallback X3dNurbsGroup::doWithDataCallback = NULL;
X3dCallback X3dNurbsGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNurbsGroup::processEventCallback = NULL;

int X3dNurbsGroupType = 172;



class X3dPointEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float* direction;
    int direction_length;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 197; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        direction = NULL;
        direction_length = 0;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointEmitter::renderCallback = NULL;
X3dCallback X3dPointEmitter::treeRenderCallback = NULL;
X3dCallback X3dPointEmitter::doWithDataCallback = NULL;
X3dCallback X3dPointEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointEmitter::processEventCallback = NULL;

int X3dPointEmitterType = 197;



class X3dProximitySensor : public X3dNode {
public:
    double enterTime;
    double exitTime;
    float* centerOfRotation_changed;
    int centerOfRotation_changed_length;
    bool isActive;
    float* position_changed;
    int position_changed_length;
    float* orientation_changed;
    int orientation_changed_length;
    float* center;
    int center_length;
    bool enabled;
    float* size;
    int size_length;
    void* extra_data;

    int getType() const { return 216; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dProximitySensor() {
        center = NULL;
        center_length = 0;
        enabled = false;
        size = NULL;
        size_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dProximitySensor::renderCallback = NULL;
X3dCallback X3dProximitySensor::treeRenderCallback = NULL;
X3dCallback X3dProximitySensor::doWithDataCallback = NULL;
X3dCallback X3dProximitySensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dProximitySensor::processEventCallback = NULL;

int X3dProximitySensorType = 216;



class X3dHAnimHumanoid : public X3dNode {
public:
    float* rotation;
    int rotation_length;
    float* translation;
    int translation_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* center;
    int center_length;
    float* scale;
    int scale_length;
    float* scaleOrientation;
    int scaleOrientation_length;
    const char** info;
    int info_length;
    X3dNode** joints;
    int joints_length;
    const char* name;
    X3dNode** segments;
    int segments_length;
    X3dNode** sites;
    int sites_length;
    X3dNode** skeleton;
    int skeleton_length;
    X3dNode** skin;
    int skin_length;
    X3dNode* skinCoord;
    X3dNode* skinNormal;
    const char* version;
    X3dNode** viewpoints;
    int viewpoints_length;
    void* extra_data;

    int getType() const { return 106; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimHumanoid() {
        rotation = NULL;
        rotation_length = 0;
        translation = NULL;
        translation_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        center = NULL;
        center_length = 0;
        scale = NULL;
        scale_length = 0;
        scaleOrientation = NULL;
        scaleOrientation_length = 0;
        info = NULL;
        info_length = 0;
        joints = NULL;
        joints_length = 0;
        name = NULL;
        segments = NULL;
        segments_length = 0;
        sites = NULL;
        sites_length = 0;
        skeleton = NULL;
        skeleton_length = 0;
        skin = NULL;
        skin_length = 0;
        skinCoord = NULL;
        skinNormal = NULL;
        version = NULL;
        viewpoints = NULL;
        viewpoints_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (joints)
                for (i = 0; i < joints_length; i++)
                    if (joints[i])
                        joints[i]->treeRender(dataptr);
            if (segments)
                for (i = 0; i < segments_length; i++)
                    if (segments[i])
                        segments[i]->treeRender(dataptr);
            if (sites)
                for (i = 0; i < sites_length; i++)
                    if (sites[i])
                        sites[i]->treeRender(dataptr);
            if (skeleton)
                for (i = 0; i < skeleton_length; i++)
                    if (skeleton[i])
                        skeleton[i]->treeRender(dataptr);
            if (skin)
                for (i = 0; i < skin_length; i++)
                    if (skin[i])
                        skin[i]->treeRender(dataptr);
            if (skinCoord)
                skinCoord->treeRender(dataptr);
            if (skinNormal)
                skinNormal->treeRender(dataptr);
            if (viewpoints)
                for (i = 0; i < viewpoints_length; i++)
                    if (viewpoints[i])
                        viewpoints[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (joints)
                for (i = 0; i < joints_length; i++)
                    if (joints[i])
                        joints[i]->treeDoWithData(dataptr);
            if (segments)
                for (i = 0; i < segments_length; i++)
                    if (segments[i])
                        segments[i]->treeDoWithData(dataptr);
            if (sites)
                for (i = 0; i < sites_length; i++)
                    if (sites[i])
                        sites[i]->treeDoWithData(dataptr);
            if (skeleton)
                for (i = 0; i < skeleton_length; i++)
                    if (skeleton[i])
                        skeleton[i]->treeDoWithData(dataptr);
            if (skin)
                for (i = 0; i < skin_length; i++)
                    if (skin[i])
                        skin[i]->treeDoWithData(dataptr);
            if (skinCoord)
                skinCoord->treeDoWithData(dataptr);
            if (skinNormal)
                skinNormal->treeDoWithData(dataptr);
            if (viewpoints)
                for (i = 0; i < viewpoints_length; i++)
                    if (viewpoints[i])
                        viewpoints[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimHumanoid::renderCallback = NULL;
X3dCallback X3dHAnimHumanoid::treeRenderCallback = NULL;
X3dCallback X3dHAnimHumanoid::doWithDataCallback = NULL;
X3dCallback X3dHAnimHumanoid::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimHumanoid::processEventCallback = NULL;

int X3dHAnimHumanoidType = 106;



class X3dIndexedFaceSet : public X3dNode {
public:
    int* set_colorIndex;
    int set_colorIndex_length;
    int* set_coordIndex;
    int set_coordIndex_length;
    int* set_normalIndex;
    int set_normalIndex_length;
    int* set_texCoordIndex;
    int set_texCoordIndex_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* normal;
    X3dNode* texCoord;
    bool ccw;
    int* colorIndex;
    int colorIndex_length;
    bool colorPerVertex;
    bool convex;
    int* coordIndex;
    int coordIndex_length;
    float creaseAngle;
    int* normalIndex;
    int normalIndex_length;
    bool normalPerVertex;
    bool solid;
    int* texCoordIndex;
    int texCoordIndex_length;
    float* radianceTransfer;
    int radianceTransfer_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 115; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback createNormalsCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dIndexedFaceSet() {
        color = NULL;
        coord = NULL;
        normal = NULL;
        texCoord = NULL;
        ccw = false;
        colorIndex = NULL;
        colorIndex_length = 0;
        colorPerVertex = false;
        convex = false;
        coordIndex = NULL;
        coordIndex_length = 0;
        creaseAngle = 0.0f;
        normalIndex = NULL;
        normalIndex_length = 0;
        normalPerVertex = false;
        solid = false;
        texCoordIndex = NULL;
        texCoordIndex_length = 0;
        radianceTransfer = NULL;
        radianceTransfer_length = 0;
        attrib = NULL;
        attrib_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (normal)
                normal->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void createNormals(void *dataptr) {
        if (createNormalsCallback)
            createNormalsCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (normal)
                normal->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dIndexedFaceSet::renderCallback = NULL;
X3dCallback X3dIndexedFaceSet::treeRenderCallback = NULL;
X3dCallback X3dIndexedFaceSet::createNormalsCallback = NULL;
X3dCallback X3dIndexedFaceSet::doWithDataCallback = NULL;
X3dCallback X3dIndexedFaceSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dIndexedFaceSet::processEventCallback = NULL;

int X3dIndexedFaceSetType = 115;



class X3dSound : public X3dNode {
public:
    float* direction;
    int direction_length;
    float intensity;
    float* location;
    int location_length;
    float maxBack;
    float maxFront;
    float minBack;
    float minFront;
    float priority;
    X3dNode* source;
    bool spatialize;
    void* extra_data;

    int getType() const { return 249; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSound() {
        direction = NULL;
        direction_length = 0;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        maxBack = 0.0f;
        maxFront = 0.0f;
        minBack = 0.0f;
        minFront = 0.0f;
        priority = 0.0f;
        source = NULL;
        spatialize = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (source)
                source->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (source)
                source->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSound::renderCallback = NULL;
X3dCallback X3dSound::treeRenderCallback = NULL;
X3dCallback X3dSound::doWithDataCallback = NULL;
X3dCallback X3dSound::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSound::processEventCallback = NULL;

int X3dSoundType = 249;



class X3dTriangleSet : public X3dNode {
public:
    X3dNode* color;
    X3dNode* coord;
    X3dNode* normal;
    X3dNode* texCoord;
    bool ccw;
    bool colorPerVertex;
    bool normalPerVertex;
    bool solid;
    float* radianceTransfer;
    int radianceTransfer_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* fogCoord;
    void* extra_data;

    int getType() const { return 293; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTriangleSet() {
        color = NULL;
        coord = NULL;
        normal = NULL;
        texCoord = NULL;
        ccw = false;
        colorPerVertex = false;
        normalPerVertex = false;
        solid = false;
        radianceTransfer = NULL;
        radianceTransfer_length = 0;
        attrib = NULL;
        attrib_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (normal)
                normal->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (normal)
                normal->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTriangleSet::renderCallback = NULL;
X3dCallback X3dTriangleSet::treeRenderCallback = NULL;
X3dCallback X3dTriangleSet::doWithDataCallback = NULL;
X3dCallback X3dTriangleSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTriangleSet::processEventCallback = NULL;

int X3dTriangleSetType = 293;



class X3dFontStyle : public X3dNode {
public:
    const char** family;
    int family_length;
    bool horizontal;
    const char** justify;
    int justify_length;
    const char* language;
    bool leftToRight;
    float spacing;
    const char* style;
    bool topToBottom;
    float size;
    void* extra_data;

    int getType() const { return 89; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dFontStyle() {
        family = NULL;
        family_length = 0;
        horizontal = false;
        justify = NULL;
        justify_length = 0;
        language = NULL;
        leftToRight = false;
        spacing = 0.0f;
        style = NULL;
        topToBottom = false;
        size = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dFontStyle::renderCallback = NULL;
X3dCallback X3dFontStyle::treeRenderCallback = NULL;
X3dCallback X3dFontStyle::doWithDataCallback = NULL;
X3dCallback X3dFontStyle::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dFontStyle::processEventCallback = NULL;

int X3dFontStyleType = 89;



class X3dInline : public X3dNode {
public:
    bool load;
    const char** url;
    int url_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 121; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dInline() {
        load = false;
        url = NULL;
        url_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dInline::renderCallback = NULL;
X3dCallback X3dInline::treeRenderCallback = NULL;
X3dCallback X3dInline::doWithDataCallback = NULL;
X3dCallback X3dInline::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dInline::processEventCallback = NULL;

int X3dInlineType = 121;



class X3dMaterial : public X3dNode {
public:
    float ambientIntensity;
    float* diffuseColor;
    int diffuseColor_length;
    float* emissiveColor;
    int emissiveColor_length;
    float shininess;
    float* specularColor;
    int specularColor_length;
    float transparency;
    float mirror;
    float* reflSpecular;
    int reflSpecular_length;
    float* reflDiffuse;
    int reflDiffuse_length;
    float* transSpecular;
    int transSpecular_length;
    float* transDiffuse;
    int transDiffuse_length;
    float reflSpecularExp;
    float transSpecularExp;
    void* extra_data;

    int getType() const { return 147; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dMaterial() {
        ambientIntensity = 0.0f;
        diffuseColor = NULL;
        diffuseColor_length = 0;
        emissiveColor = NULL;
        emissiveColor_length = 0;
        shininess = 0.0f;
        specularColor = NULL;
        specularColor_length = 0;
        transparency = 0.0f;
        mirror = 0.0f;
        reflSpecular = NULL;
        reflSpecular_length = 0;
        reflDiffuse = NULL;
        reflDiffuse_length = 0;
        transSpecular = NULL;
        transSpecular_length = 0;
        transDiffuse = NULL;
        transDiffuse_length = 0;
        reflSpecularExp = 0.0f;
        transSpecularExp = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dMaterial::renderCallback = NULL;
X3dCallback X3dMaterial::treeRenderCallback = NULL;
X3dCallback X3dMaterial::doWithDataCallback = NULL;
X3dCallback X3dMaterial::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dMaterial::processEventCallback = NULL;

int X3dMaterialType = 147;



class X3dConeEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float angle;
    float* direction;
    int direction_length;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 48; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dConeEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        angle = 0.0f;
        direction = NULL;
        direction_length = 0;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dConeEmitter::renderCallback = NULL;
X3dCallback X3dConeEmitter::treeRenderCallback = NULL;
X3dCallback X3dConeEmitter::doWithDataCallback = NULL;
X3dCallback X3dConeEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dConeEmitter::processEventCallback = NULL;

int X3dConeEmitterType = 48;



class X3dOrientationInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 186; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dOrientationInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dOrientationInterpolator::renderCallback = NULL;
X3dCallback X3dOrientationInterpolator::treeRenderCallback = NULL;
X3dCallback X3dOrientationInterpolator::doWithDataCallback = NULL;
X3dCallback X3dOrientationInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dOrientationInterpolator::processEventCallback = NULL;

int X3dOrientationInterpolatorType = 186;



class X3dTimeSensor : public X3dNode {
public:
    double cycleTime;
    double elapsedTime;
    float fraction_changed;
    bool isActive;
    bool isPaused;
    double time;
    double cycleInterval;
    bool enabled;
    bool loop;
    double pauseTime;
    double resumeTime;
    double startTime;
    double stopTime;
    void* extra_data;

    int getType() const { return 284; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTimeSensor() {
        cycleInterval = 0;
        enabled = false;
        loop = false;
        pauseTime = 0;
        resumeTime = 0;
        startTime = 0;
        stopTime = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTimeSensor::renderCallback = NULL;
X3dCallback X3dTimeSensor::treeRenderCallback = NULL;
X3dCallback X3dTimeSensor::doWithDataCallback = NULL;
X3dCallback X3dTimeSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTimeSensor::processEventCallback = NULL;

int X3dTimeSensorType = 284;



class X3dIndexedLineSet : public X3dNode {
public:
    int* set_colorIndex;
    int set_colorIndex_length;
    int* set_coordIndex;
    int set_coordIndex_length;
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    int* colorIndex;
    int colorIndex_length;
    bool colorPerVertex;
    int* coordIndex;
    int coordIndex_length;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 116; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dIndexedLineSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        colorIndex = NULL;
        colorIndex_length = 0;
        colorPerVertex = false;
        coordIndex = NULL;
        coordIndex_length = 0;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dIndexedLineSet::renderCallback = NULL;
X3dCallback X3dIndexedLineSet::treeRenderCallback = NULL;
X3dCallback X3dIndexedLineSet::doWithDataCallback = NULL;
X3dCallback X3dIndexedLineSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dIndexedLineSet::processEventCallback = NULL;

int X3dIndexedLineSetType = 116;



class X3dMultiPart : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 162; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dMultiPart() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dMultiPart::renderCallback = NULL;
X3dCallback X3dMultiPart::treeRenderCallback = NULL;
X3dCallback X3dMultiPart::doWithDataCallback = NULL;
X3dCallback X3dMultiPart::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dMultiPart::processEventCallback = NULL;

int X3dMultiPartType = 162;



class X3dViewport : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* clipBoundary;
    int clipBoundary_length;
    void* extra_data;

    int getType() const { return 318; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dViewport() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        clipBoundary = NULL;
        clipBoundary_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dViewport::renderCallback = NULL;
X3dCallback X3dViewport::treeRenderCallback = NULL;
X3dCallback X3dViewport::doWithDataCallback = NULL;
X3dCallback X3dViewport::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dViewport::processEventCallback = NULL;

int X3dViewportType = 318;



class X3dAnchor : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    const char* description;
    const char** parameter;
    int parameter_length;
    const char** url;
    int url_length;
    void* extra_data;

    int getType() const { return 0; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAnchor() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        description = NULL;
        parameter = NULL;
        parameter_length = 0;
        url = NULL;
        url_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAnchor::renderCallback = NULL;
X3dCallback X3dAnchor::treeRenderCallback = NULL;
X3dCallback X3dAnchor::doWithDataCallback = NULL;
X3dCallback X3dAnchor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAnchor::processEventCallback = NULL;

int X3dAnchorType = 0;



class X3dColor : public X3dNode {
public:
    float* color;
    int color_length;
    void* extra_data;

    int getType() const { return 35; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColor() {
        color = NULL;
        color_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColor::renderCallback = NULL;
X3dCallback X3dColor::treeRenderCallback = NULL;
X3dCallback X3dColor::doWithDataCallback = NULL;
X3dCallback X3dColor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColor::processEventCallback = NULL;

int X3dColorType = 35;



class X3dShape : public X3dNode {
public:
    X3dNode* appearance;
    X3dNode* geometry;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* octreeTriangles;
    const char* shading;
    void* extra_data;

    int getType() const { return 240; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dShape() {
        appearance = NULL;
        geometry = NULL;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        octreeTriangles = NULL;
        shading = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (appearance)
                appearance->treeRender(dataptr);
            if (geometry)
                geometry->treeRender(dataptr);
            if (octreeTriangles)
                octreeTriangles->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (appearance)
                appearance->treeDoWithData(dataptr);
            if (geometry)
                geometry->treeDoWithData(dataptr);
            if (octreeTriangles)
                octreeTriangles->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dShape::renderCallback = NULL;
X3dCallback X3dShape::treeRenderCallback = NULL;
X3dCallback X3dShape::doWithDataCallback = NULL;
X3dCallback X3dShape::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dShape::processEventCallback = NULL;

int X3dShapeType = 240;



class X3dNormalInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 168; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNormalInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNormalInterpolator::renderCallback = NULL;
X3dCallback X3dNormalInterpolator::treeRenderCallback = NULL;
X3dCallback X3dNormalInterpolator::doWithDataCallback = NULL;
X3dCallback X3dNormalInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNormalInterpolator::processEventCallback = NULL;

int X3dNormalInterpolatorType = 168;



class X3dScalarInterpolator : public X3dNode {
public:
    float set_fraction;
    float value_changed;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 230; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScalarInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScalarInterpolator::renderCallback = NULL;
X3dCallback X3dScalarInterpolator::treeRenderCallback = NULL;
X3dCallback X3dScalarInterpolator::doWithDataCallback = NULL;
X3dCallback X3dScalarInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScalarInterpolator::processEventCallback = NULL;

int X3dScalarInterpolatorType = 230;



class X3dHAnimSegment : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    float* centerOfMass;
    int centerOfMass_length;
    X3dNode* coord;
    X3dNode** displacers;
    int displacers_length;
    float mass;
    float* momentsOfInertia;
    int momentsOfInertia_length;
    const char* name;
    void* extra_data;

    int getType() const { return 108; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dHAnimSegment() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        centerOfMass = NULL;
        centerOfMass_length = 0;
        coord = NULL;
        displacers = NULL;
        displacers_length = 0;
        mass = 0.0f;
        momentsOfInertia = NULL;
        momentsOfInertia_length = 0;
        name = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (displacers)
                for (i = 0; i < displacers_length; i++)
                    if (displacers[i])
                        displacers[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dHAnimSegment::renderCallback = NULL;
X3dCallback X3dHAnimSegment::treeRenderCallback = NULL;
X3dCallback X3dHAnimSegment::doWithDataCallback = NULL;
X3dCallback X3dHAnimSegment::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dHAnimSegment::processEventCallback = NULL;

int X3dHAnimSegmentType = 108;



class X3dTextureCoordinate : public X3dNode {
public:
    float* point;
    int point_length;
    void* extra_data;

    int getType() const { return 276; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTextureCoordinate() {
        point = NULL;
        point_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTextureCoordinate::renderCallback = NULL;
X3dCallback X3dTextureCoordinate::treeRenderCallback = NULL;
X3dCallback X3dTextureCoordinate::doWithDataCallback = NULL;
X3dCallback X3dTextureCoordinate::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTextureCoordinate::processEventCallback = NULL;

int X3dTextureCoordinateType = 276;



class X3dNormal : public X3dNode {
public:
    float* vector;
    int vector_length;
    void* extra_data;

    int getType() const { return 167; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dNormal() {
        vector = NULL;
        vector_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dNormal::renderCallback = NULL;
X3dCallback X3dNormal::treeRenderCallback = NULL;
X3dCallback X3dNormal::doWithDataCallback = NULL;
X3dCallback X3dNormal::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dNormal::processEventCallback = NULL;

int X3dNormalType = 167;



class X3dBillboard : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* axisOfRotation;
    int axisOfRotation_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 9; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBillboard() {
        axisOfRotation = NULL;
        axisOfRotation_length = 0;
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBillboard::renderCallback = NULL;
X3dCallback X3dBillboard::treeRenderCallback = NULL;
X3dCallback X3dBillboard::doWithDataCallback = NULL;
X3dCallback X3dBillboard::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBillboard::processEventCallback = NULL;

int X3dBillboardType = 9;



class X3dSpotLight : public X3dNode {
public:
    float ambientIntensity;
    float* attenuation;
    int attenuation_length;
    float beamWidth;
    float* color;
    int color_length;
    float cutOffAngle;
    float* direction;
    int direction_length;
    bool global;
    float intensity;
    float* location;
    int location_length;
    bool on;
    float radius;
    float projectionAngle;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 257; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSpotLight() {
        ambientIntensity = 0.0f;
        attenuation = NULL;
        attenuation_length = 0;
        beamWidth = 0.0f;
        color = NULL;
        color_length = 0;
        cutOffAngle = 0.0f;
        direction = NULL;
        direction_length = 0;
        global = false;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        on = false;
        radius = 0.0f;
        projectionAngle = 0.0f;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSpotLight::renderCallback = NULL;
X3dCallback X3dSpotLight::treeRenderCallback = NULL;
X3dCallback X3dSpotLight::doWithDataCallback = NULL;
X3dCallback X3dSpotLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSpotLight::processEventCallback = NULL;

int X3dSpotLightType = 257;



class X3dColorInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 38; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dColorInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dColorInterpolator::renderCallback = NULL;
X3dCallback X3dColorInterpolator::treeRenderCallback = NULL;
X3dCallback X3dColorInterpolator::doWithDataCallback = NULL;
X3dCallback X3dColorInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dColorInterpolator::processEventCallback = NULL;

int X3dColorInterpolatorType = 38;



class X3dExplosionEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 81; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dExplosionEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dExplosionEmitter::renderCallback = NULL;
X3dCallback X3dExplosionEmitter::treeRenderCallback = NULL;
X3dCallback X3dExplosionEmitter::doWithDataCallback = NULL;
X3dCallback X3dExplosionEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dExplosionEmitter::processEventCallback = NULL;

int X3dExplosionEmitterType = 81;



class X3dLineSet : public X3dNode {
public:
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* fogCoord;
    int* vertexCount;
    int vertexCount_length;
    void* extra_data;
    int glName_number;

    int getType() const { return 143; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dLineSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        fogCoord = NULL;
        vertexCount = NULL;
        vertexCount_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dLineSet::renderCallback = NULL;
X3dCallback X3dLineSet::treeRenderCallback = NULL;
X3dCallback X3dLineSet::doWithDataCallback = NULL;
X3dCallback X3dLineSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dLineSet::processEventCallback = NULL;

int X3dLineSetType = 143;



class X3dPixelTexture : public X3dNode {
public:
    int* image;
    int image_length;
    bool repeatS;
    bool repeatT;
    X3dNode* textureProperties;
    X3dNode** effects;
    int effects_length;
    const char* alphaChannel;
    void* extra_data;

    int getType() const { return 193; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPixelTexture() {
        image = NULL;
        image_length = 0;
        repeatS = false;
        repeatT = false;
        textureProperties = NULL;
        effects = NULL;
        effects_length = 0;
        alphaChannel = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (textureProperties)
                textureProperties->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPixelTexture::renderCallback = NULL;
X3dCallback X3dPixelTexture::treeRenderCallback = NULL;
X3dCallback X3dPixelTexture::doWithDataCallback = NULL;
X3dCallback X3dPixelTexture::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPixelTexture::processEventCallback = NULL;

int X3dPixelTextureType = 193;



class X3dSwitch : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode** children;
    int children_length;
    int whichChoice;
    void* extra_data;

    int getType() const { return 269; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSwitch() {
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        children = NULL;
        children_length = 0;
        whichChoice = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSwitch::renderCallback = NULL;
X3dCallback X3dSwitch::treeRenderCallback = NULL;
X3dCallback X3dSwitch::doWithDataCallback = NULL;
X3dCallback X3dSwitch::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSwitch::processEventCallback = NULL;

int X3dSwitchType = 269;



class X3dBlock : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 10; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBlock() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBlock::renderCallback = NULL;
X3dCallback X3dBlock::treeRenderCallback = NULL;
X3dCallback X3dBlock::doWithDataCallback = NULL;
X3dCallback X3dBlock::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBlock::processEventCallback = NULL;

int X3dBlockType = 10;



class X3dViewpoint : public X3dNode {
public:
    bool set_bind;
    double bindTime;
    bool isBound;
    float* centerOfRotation;
    int centerOfRotation_length;
    const char* description;
    float fieldOfView;
    bool jump;
    float* orientation;
    int orientation_length;
    bool retainUserOffsets;
    float* position;
    int position_length;
    void* extra_data;

    int getType() const { return 316; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dViewpoint() {
        centerOfRotation = NULL;
        centerOfRotation_length = 0;
        description = NULL;
        fieldOfView = 0.0f;
        jump = false;
        orientation = NULL;
        orientation_length = 0;
        retainUserOffsets = false;
        position = NULL;
        position_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dViewpoint::renderCallback = NULL;
X3dCallback X3dViewpoint::treeRenderCallback = NULL;
X3dCallback X3dViewpoint::doWithDataCallback = NULL;
X3dCallback X3dViewpoint::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dViewpoint::processEventCallback = NULL;

int X3dViewpointType = 316;



class X3dWorldInfo : public X3dNode {
public:
    const char** info;
    int info_length;
    const char* title;
    void* extra_data;

    int getType() const { return 330; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dWorldInfo() {
        info = NULL;
        info_length = 0;
        title = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dWorldInfo::renderCallback = NULL;
X3dCallback X3dWorldInfo::treeRenderCallback = NULL;
X3dCallback X3dWorldInfo::doWithDataCallback = NULL;
X3dCallback X3dWorldInfo::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dWorldInfo::processEventCallback = NULL;

int X3dWorldInfoType = 330;



class X3dAudioClip : public X3dNode {
public:
    double duration_changed;
    double elapsedTime;
    bool isActive;
    bool isPaused;
    const char* description;
    bool loop;
    double pauseTime;
    float pitch;
    double resumeTime;
    double startTime;
    double stopTime;
    const char** url;
    int url_length;
    void* extra_data;

    int getType() const { return 5; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAudioClip() {
        description = NULL;
        loop = false;
        pauseTime = 0;
        pitch = 0.0f;
        resumeTime = 0;
        startTime = 0;
        stopTime = 0;
        url = NULL;
        url_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAudioClip::renderCallback = NULL;
X3dCallback X3dAudioClip::treeRenderCallback = NULL;
X3dCallback X3dAudioClip::doWithDataCallback = NULL;
X3dCallback X3dAudioClip::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAudioClip::processEventCallback = NULL;

int X3dAudioClipType = 5;



class X3dPointSet : public X3dNode {
public:
    X3dNode** attrib;
    int attrib_length;
    X3dNode* color;
    X3dNode* coord;
    X3dNode* fogCoord;
    void* extra_data;
    int glName_number;

    int getType() const { return 200; }

    void setGlName(int number){ glName_number = number; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointSet() {
        attrib = NULL;
        attrib_length = 0;
        color = NULL;
        coord = NULL;
        fogCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeRender(dataptr);
            if (color)
                color->treeRender(dataptr);
            if (coord)
                coord->treeRender(dataptr);
            if (fogCoord)
                fogCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (attrib)
                for (i = 0; i < attrib_length; i++)
                    if (attrib[i])
                        attrib[i]->treeDoWithData(dataptr);
            if (color)
                color->treeDoWithData(dataptr);
            if (coord)
                coord->treeDoWithData(dataptr);
            if (fogCoord)
                fogCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointSet::renderCallback = NULL;
X3dCallback X3dPointSet::treeRenderCallback = NULL;
X3dCallback X3dPointSet::doWithDataCallback = NULL;
X3dCallback X3dPointSet::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointSet::processEventCallback = NULL;

int X3dPointSetType = 200;



class X3dBoundedPhysicsModel : public X3dNode {
public:
    bool enabled;
    X3dNode* geometry;
    void* extra_data;

    int getType() const { return 18; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBoundedPhysicsModel() {
        enabled = false;
        geometry = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (geometry)
                geometry->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (geometry)
                geometry->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBoundedPhysicsModel::renderCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dBoundedPhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBoundedPhysicsModel::processEventCallback = NULL;

int X3dBoundedPhysicsModelType = 18;



class X3dGroup : public X3dNode {
public:
    X3dNode** addChildren;
    int addChildren_length;
    X3dNode** removeChildren;
    int removeChildren_length;
    X3dNode** children;
    int children_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    void* extra_data;

    int getType() const { return 104; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dGroup() {
        children = NULL;
        children_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (children)
                for (i = 0; i < children_length; i++)
                    if (children[i])
                        children[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dGroup::renderCallback = NULL;
X3dCallback X3dGroup::treeRenderCallback = NULL;
X3dCallback X3dGroup::doWithDataCallback = NULL;
X3dCallback X3dGroup::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dGroup::processEventCallback = NULL;

int X3dGroupType = 104;



class X3dPolylineEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    X3dNode* coord;
    float* direction;
    int direction_length;
    int* coordIndex;
    int coordIndex_length;
    void* extra_data;

    int getType() const { return 202; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPolylineEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coord = NULL;
        direction = NULL;
        direction_length = 0;
        coordIndex = NULL;
        coordIndex_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (coord)
                coord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (coord)
                coord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPolylineEmitter::renderCallback = NULL;
X3dCallback X3dPolylineEmitter::treeRenderCallback = NULL;
X3dCallback X3dPolylineEmitter::doWithDataCallback = NULL;
X3dCallback X3dPolylineEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPolylineEmitter::processEventCallback = NULL;

int X3dPolylineEmitterType = 202;



class X3dPositionInterpolator : public X3dNode {
public:
    float set_fraction;
    float* value_changed;
    int value_changed_length;
    float* key;
    int key_length;
    float* keyValue;
    int keyValue_length;
    void* extra_data;

    int getType() const { return 210; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPositionInterpolator() {
        key = NULL;
        key_length = 0;
        keyValue = NULL;
        keyValue_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPositionInterpolator::renderCallback = NULL;
X3dCallback X3dPositionInterpolator::treeRenderCallback = NULL;
X3dCallback X3dPositionInterpolator::doWithDataCallback = NULL;
X3dCallback X3dPositionInterpolator::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPositionInterpolator::processEventCallback = NULL;

int X3dPositionInterpolatorType = 210;



class X3dCoordinate : public X3dNode {
public:
    float* point;
    int point_length;
    void* extra_data;

    int getType() const { return 52; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dCoordinate() {
        point = NULL;
        point_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dCoordinate::renderCallback = NULL;
X3dCallback X3dCoordinate::treeRenderCallback = NULL;
X3dCallback X3dCoordinate::doWithDataCallback = NULL;
X3dCallback X3dCoordinate::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dCoordinate::processEventCallback = NULL;

int X3dCoordinateType = 52;



class X3dForcePhysicsModel : public X3dNode {
public:
    bool enabled;
    float* force;
    int force_length;
    void* extra_data;

    int getType() const { return 90; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dForcePhysicsModel() {
        enabled = false;
        force = NULL;
        force_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dForcePhysicsModel::renderCallback = NULL;
X3dCallback X3dForcePhysicsModel::treeRenderCallback = NULL;
X3dCallback X3dForcePhysicsModel::doWithDataCallback = NULL;
X3dCallback X3dForcePhysicsModel::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dForcePhysicsModel::processEventCallback = NULL;

int X3dForcePhysicsModelType = 90;



class X3dSphereSensor : public X3dNode {
public:
    bool isActive;
    bool isOver;
    float* rotation_changed;
    int rotation_changed_length;
    float* trackPoint_changed;
    int trackPoint_changed_length;
    bool autoOffset;
    const char* description;
    bool enabled;
    float* offset;
    int offset_length;
    void* extra_data;

    int getType() const { return 253; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSphereSensor() {
        autoOffset = false;
        description = NULL;
        enabled = false;
        offset = NULL;
        offset_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSphereSensor::renderCallback = NULL;
X3dCallback X3dSphereSensor::treeRenderCallback = NULL;
X3dCallback X3dSphereSensor::doWithDataCallback = NULL;
X3dCallback X3dSphereSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSphereSensor::processEventCallback = NULL;

int X3dSphereSensorType = 253;



class X3dBackground : public X3dNode {
public:
    bool set_bind;
    bool isBound;
    float* groundAngle;
    int groundAngle_length;
    float* groundColor;
    int groundColor_length;
    const char** backUrl;
    int backUrl_length;
    const char** rightUrl;
    int rightUrl_length;
    const char** frontUrl;
    int frontUrl_length;
    const char** leftUrl;
    int leftUrl_length;
    const char** topUrl;
    int topUrl_length;
    const char** bottomUrl;
    int bottomUrl_length;
    float* skyAngle;
    int skyAngle_length;
    float transparency;
    float* skyColor;
    int skyColor_length;
    void* extra_data;

    int getType() const { return 6; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dBackground() {
        groundAngle = NULL;
        groundAngle_length = 0;
        groundColor = NULL;
        groundColor_length = 0;
        backUrl = NULL;
        backUrl_length = 0;
        rightUrl = NULL;
        rightUrl_length = 0;
        frontUrl = NULL;
        frontUrl_length = 0;
        leftUrl = NULL;
        leftUrl_length = 0;
        topUrl = NULL;
        topUrl_length = 0;
        bottomUrl = NULL;
        bottomUrl_length = 0;
        skyAngle = NULL;
        skyAngle_length = 0;
        transparency = 0.0f;
        skyColor = NULL;
        skyColor_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dBackground::renderCallback = NULL;
X3dCallback X3dBackground::treeRenderCallback = NULL;
X3dCallback X3dBackground::doWithDataCallback = NULL;
X3dCallback X3dBackground::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dBackground::processEventCallback = NULL;

int X3dBackgroundType = 6;



class X3dSurfaceEmitter : public X3dNode {
public:
    float speed;
    float variation;
    float mass;
    float surfaceArea;
    int* coordIndex;
    int coordIndex_length;
    X3dNode* surface;
    void* extra_data;

    int getType() const { return 267; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dSurfaceEmitter() {
        speed = 0.0f;
        variation = 0.0f;
        mass = 0.0f;
        surfaceArea = 0.0f;
        coordIndex = NULL;
        coordIndex_length = 0;
        surface = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (surface)
                surface->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (surface)
                surface->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dSurfaceEmitter::renderCallback = NULL;
X3dCallback X3dSurfaceEmitter::treeRenderCallback = NULL;
X3dCallback X3dSurfaceEmitter::doWithDataCallback = NULL;
X3dCallback X3dSurfaceEmitter::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dSurfaceEmitter::processEventCallback = NULL;

int X3dSurfaceEmitterType = 267;



class X3dTouchSensor : public X3dNode {
public:
    float* hitNormal_changed;
    int hitNormal_changed_length;
    float* hitPoint_changed;
    int hitPoint_changed_length;
    float* hitTexCoord_changed;
    int hitTexCoord_changed_length;
    bool isActive;
    bool isOver;
    double touchTime;
    const char* description;
    bool enabled;
    void* extra_data;

    int getType() const { return 288; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTouchSensor() {
        description = NULL;
        enabled = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTouchSensor::renderCallback = NULL;
X3dCallback X3dTouchSensor::treeRenderCallback = NULL;
X3dCallback X3dTouchSensor::doWithDataCallback = NULL;
X3dCallback X3dTouchSensor::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTouchSensor::processEventCallback = NULL;

int X3dTouchSensorType = 288;



class X3dPointLight : public X3dNode {
public:
    float ambientIntensity;
    float* attenuation;
    int attenuation_length;
    float* color;
    int color_length;
    bool global;
    float intensity;
    float* location;
    int location_length;
    bool on;
    float radius;
    bool shadows;
    bool kambiShadows;
    bool kambiShadowsMain;
    float projectionNear;
    float projectionFar;
    float* up;
    int up_length;
    X3dNode* defaultShadowMap;
    X3dNode** effects;
    int effects_length;
    bool shadowVolumes;
    bool shadowVolumesMain;
    void* extra_data;

    int getType() const { return 198; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dPointLight() {
        ambientIntensity = 0.0f;
        attenuation = NULL;
        attenuation_length = 0;
        color = NULL;
        color_length = 0;
        global = false;
        intensity = 0.0f;
        location = NULL;
        location_length = 0;
        on = false;
        radius = 0.0f;
        shadows = false;
        kambiShadows = false;
        kambiShadowsMain = false;
        projectionNear = 0.0f;
        projectionFar = 0.0f;
        up = NULL;
        up_length = 0;
        defaultShadowMap = NULL;
        effects = NULL;
        effects_length = 0;
        shadowVolumes = false;
        shadowVolumesMain = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (defaultShadowMap)
                defaultShadowMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dPointLight::renderCallback = NULL;
X3dCallback X3dPointLight::treeRenderCallback = NULL;
X3dCallback X3dPointLight::doWithDataCallback = NULL;
X3dCallback X3dPointLight::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dPointLight::processEventCallback = NULL;

int X3dPointLightType = 198;



class X3dText : public X3dNode {
public:
    float* lineBounds;
    int lineBounds_length;
    float* origin;
    int origin_length;
    float* textBounds;
    int textBounds_length;
    X3dNode* fontStyle;
    float* length;
    int length_length;
    float maxExtent;
    const char** string;
    int string_length;
    bool solid;
    X3dNode* texCoord;
    void* extra_data;

    int getType() const { return 273; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dText() {
        fontStyle = NULL;
        length = NULL;
        length_length = 0;
        maxExtent = 0.0f;
        string = NULL;
        string_length = 0;
        solid = false;
        texCoord = NULL;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (fontStyle)
                fontStyle->treeRender(dataptr);
            if (texCoord)
                texCoord->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (fontStyle)
                fontStyle->treeDoWithData(dataptr);
            if (texCoord)
                texCoord->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dText::renderCallback = NULL;
X3dCallback X3dText::treeRenderCallback = NULL;
X3dCallback X3dText::doWithDataCallback = NULL;
X3dCallback X3dText::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dText::processEventCallback = NULL;

int X3dTextType = 273;



class X3dAppearance : public X3dNode {
public:
    X3dNode* material;
    X3dNode* texture;
    X3dNode* textureTransform;
    X3dNode* fillProperties;
    X3dNode* lineProperties;
    X3dNode** shaders;
    int shaders_length;
    X3dNode** receiveShadows;
    int receiveShadows_length;
    bool shadowCaster;
    X3dNode* normalMap;
    X3dNode* heightMap;
    float heightMapScale;
    X3dNode** effects;
    int effects_length;
    void* extra_data;

    int getType() const { return 1; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dAppearance() {
        material = NULL;
        texture = NULL;
        textureTransform = NULL;
        fillProperties = NULL;
        lineProperties = NULL;
        shaders = NULL;
        shaders_length = 0;
        receiveShadows = NULL;
        receiveShadows_length = 0;
        shadowCaster = false;
        normalMap = NULL;
        heightMap = NULL;
        heightMapScale = 0.0f;
        effects = NULL;
        effects_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (material)
                material->treeRender(dataptr);
            if (texture)
                texture->treeRender(dataptr);
            if (textureTransform)
                textureTransform->treeRender(dataptr);
            if (fillProperties)
                fillProperties->treeRender(dataptr);
            if (lineProperties)
                lineProperties->treeRender(dataptr);
            if (shaders)
                for (i = 0; i < shaders_length; i++)
                    if (shaders[i])
                        shaders[i]->treeRender(dataptr);
            if (receiveShadows)
                for (i = 0; i < receiveShadows_length; i++)
                    if (receiveShadows[i])
                        receiveShadows[i]->treeRender(dataptr);
            if (normalMap)
                normalMap->treeRender(dataptr);
            if (heightMap)
                heightMap->treeRender(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (material)
                material->treeDoWithData(dataptr);
            if (texture)
                texture->treeDoWithData(dataptr);
            if (textureTransform)
                textureTransform->treeDoWithData(dataptr);
            if (fillProperties)
                fillProperties->treeDoWithData(dataptr);
            if (lineProperties)
                lineProperties->treeDoWithData(dataptr);
            if (shaders)
                for (i = 0; i < shaders_length; i++)
                    if (shaders[i])
                        shaders[i]->treeDoWithData(dataptr);
            if (receiveShadows)
                for (i = 0; i < receiveShadows_length; i++)
                    if (receiveShadows[i])
                        receiveShadows[i]->treeDoWithData(dataptr);
            if (normalMap)
                normalMap->treeDoWithData(dataptr);
            if (heightMap)
                heightMap->treeDoWithData(dataptr);
            if (effects)
                for (i = 0; i < effects_length; i++)
                    if (effects[i])
                        effects[i]->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dAppearance::renderCallback = NULL;
X3dCallback X3dAppearance::treeRenderCallback = NULL;
X3dCallback X3dAppearance::doWithDataCallback = NULL;
X3dCallback X3dAppearance::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dAppearance::processEventCallback = NULL;

int X3dAppearanceType = 1;



class X3dParticleSystem : public X3dNode {
public:
    X3dNode* appearance;
    bool createParticles;
    X3dNode* geometry;
    bool enabled;
    float lifetimeVariation;
    int maxParticles;
    float particleLifetime;
    float* particleSize;
    int particleSize_length;
    float* bboxCenter;
    int bboxCenter_length;
    float* bboxSize;
    int bboxSize_length;
    X3dNode* colorRamp;
    float* colorKey;
    int colorKey_length;
    X3dNode* emitter;
    const char* geometryType;
    X3dNode** physics;
    int physics_length;
    X3dNode* texCoordRamp;
    float* texCoordKey;
    int texCoordKey_length;
    void* extra_data;

    int getType() const { return 191; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dParticleSystem() {
        appearance = NULL;
        createParticles = false;
        geometry = NULL;
        enabled = false;
        lifetimeVariation = 0.0f;
        maxParticles = 0;
        particleLifetime = 0.0f;
        particleSize = NULL;
        particleSize_length = 0;
        bboxCenter = NULL;
        bboxCenter_length = 0;
        bboxSize = NULL;
        bboxSize_length = 0;
        colorRamp = NULL;
        colorKey = NULL;
        colorKey_length = 0;
        emitter = NULL;
        geometryType = NULL;
        physics = NULL;
        physics_length = 0;
        texCoordRamp = NULL;
        texCoordKey = NULL;
        texCoordKey_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            if (appearance)
                appearance->treeRender(dataptr);
            if (geometry)
                geometry->treeRender(dataptr);
            if (colorRamp)
                colorRamp->treeRender(dataptr);
            if (emitter)
                emitter->treeRender(dataptr);
            if (physics)
                for (i = 0; i < physics_length; i++)
                    if (physics[i])
                        physics[i]->treeRender(dataptr);
            if (texCoordRamp)
                texCoordRamp->treeRender(dataptr);
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            if (appearance)
                appearance->treeDoWithData(dataptr);
            if (geometry)
                geometry->treeDoWithData(dataptr);
            if (colorRamp)
                colorRamp->treeDoWithData(dataptr);
            if (emitter)
                emitter->treeDoWithData(dataptr);
            if (physics)
                for (i = 0; i < physics_length; i++)
                    if (physics[i])
                        physics[i]->treeDoWithData(dataptr);
            if (texCoordRamp)
                texCoordRamp->treeDoWithData(dataptr);
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dParticleSystem::renderCallback = NULL;
X3dCallback X3dParticleSystem::treeRenderCallback = NULL;
X3dCallback X3dParticleSystem::doWithDataCallback = NULL;
X3dCallback X3dParticleSystem::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dParticleSystem::processEventCallback = NULL;

int X3dParticleSystemType = 191;



class X3dTextureTransform : public X3dNode {
public:
    float* center;
    int center_length;
    float rotation;
    float* scale;
    int scale_length;
    float* translation;
    int translation_length;
    void* extra_data;

    int getType() const { return 281; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dTextureTransform() {
        center = NULL;
        center_length = 0;
        rotation = 0.0f;
        scale = NULL;
        scale_length = 0;
        translation = NULL;
        translation_length = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dTextureTransform::renderCallback = NULL;
X3dCallback X3dTextureTransform::treeRenderCallback = NULL;
X3dCallback X3dTextureTransform::doWithDataCallback = NULL;
X3dCallback X3dTextureTransform::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dTextureTransform::processEventCallback = NULL;

int X3dTextureTransformType = 281;



class X3dScript_BOX1_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_HIT_CHECK::processEventCallback = NULL;

int X3dScriptType = 234;



class X3dScript_BOX1_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX1_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX1_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX1_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX1_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX1_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_STEER : public X3dNode {
public:
    float* wall_xy_in;
    int wall_xy_in_length;
    float xvalue;
    float yvalue;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_STEER() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_STEER::renderCallback = NULL;
X3dCallback X3dScript_STEER::treeRenderCallback = NULL;
X3dCallback X3dScript_STEER::doWithDataCallback = NULL;
X3dCallback X3dScript_STEER::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_STEER::processEventCallback = NULL;



class X3dScript_ROT_TRANS1 : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* rotation_in;
    int rotation_in_length;
    const char** debug;
    int debug_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* rotation;
    int rotation_length;
    float* quaternion1;
    int quaternion1_length;
    float* quaternion2;
    int quaternion2_length;
    float* quaternion3;
    int quaternion3_length;
    float* quaternion4;
    int quaternion4_length;
    float leng;
    float winkelkorrektur;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROT_TRANS1() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rotation = NULL;
        rotation_length = 0;
        quaternion1 = NULL;
        quaternion1_length = 0;
        quaternion2 = NULL;
        quaternion2_length = 0;
        quaternion3 = NULL;
        quaternion3_length = 0;
        quaternion4 = NULL;
        quaternion4_length = 0;
        leng = 0.0f;
        winkelkorrektur = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROT_TRANS1::renderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::treeRenderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::doWithDataCallback = NULL;
X3dCallback X3dScript_ROT_TRANS1::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROT_TRANS1::processEventCallback = NULL;



class X3dScript_ROT_TRANS2 : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* rotation_in;
    int rotation_in_length;
    const char** debug;
    int debug_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* rotation;
    int rotation_length;
    float* quaternion1;
    int quaternion1_length;
    float* quaternion2;
    int quaternion2_length;
    float* quaternion3;
    int quaternion3_length;
    float* quaternion4;
    int quaternion4_length;
    float leng;
    float winkelkorrektur;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROT_TRANS2() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rotation = NULL;
        rotation_length = 0;
        quaternion1 = NULL;
        quaternion1_length = 0;
        quaternion2 = NULL;
        quaternion2_length = 0;
        quaternion3 = NULL;
        quaternion3_length = 0;
        quaternion4 = NULL;
        quaternion4_length = 0;
        leng = 0.0f;
        winkelkorrektur = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROT_TRANS2::renderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::treeRenderCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::doWithDataCallback = NULL;
X3dCallback X3dScript_ROT_TRANS2::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROT_TRANS2::processEventCallback = NULL;



class X3dScript_WALL_FIRE : public X3dNode {
public:
    bool isactive_in;
    bool fire;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    bool old_isactive;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_WALL_FIRE() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        old_isactive = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_WALL_FIRE::renderCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::treeRenderCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::doWithDataCallback = NULL;
X3dCallback X3dScript_WALL_FIRE::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_WALL_FIRE::processEventCallback = NULL;



class X3dScript_ROLL_TURN : public X3dNode {
public:
    float xvalue_in;
    int rock_and_roll_flag_in;
    float* rotation_out;
    int rotation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int rock_and_roll_flag;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_ROLL_TURN() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        rock_and_roll_flag = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_ROLL_TURN::renderCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::treeRenderCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::doWithDataCallback = NULL;
X3dCallback X3dScript_ROLL_TURN::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_ROLL_TURN::processEventCallback = NULL;



class X3dScript_UP_DOWN_TURN : public X3dNode {
public:
    float yvalue_in;
    float* rotation_out;
    int rotation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_UP_DOWN_TURN() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_UP_DOWN_TURN::renderCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::treeRenderCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::doWithDataCallback = NULL;
X3dCallback X3dScript_UP_DOWN_TURN::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_UP_DOWN_TURN::processEventCallback = NULL;



class X3dScript_FLY_FORWARD : public X3dNode {
public:
    double time_in;
    float* translation_out;
    int translation_out_length;
    double time_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_FLY_FORWARD() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_FLY_FORWARD::renderCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::treeRenderCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::doWithDataCallback = NULL;
X3dCallback X3dScript_FLY_FORWARD::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_FLY_FORWARD::processEventCallback = NULL;



class X3dScript_FLY_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    double time_in;
    double enter_time;
    double exit_time;
    bool fire_in;
    float* translation_out;
    int translation_out_length;
    float* top_translation_out;
    int top_translation_out_length;
    int bullet_choice_out;
    float* bullet_translation_out;
    int bullet_translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* top_translation;
    int top_translation_length;
    float* translation_old;
    int translation_old_length;
    float* bullet_translation_start;
    int bullet_translation_start_length;
    float* bullet_direction;
    int bullet_direction_length;
    float bulletway_length;
    double old_time;
    float factor;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_FLY_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        top_translation = NULL;
        top_translation_length = 0;
        translation_old = NULL;
        translation_old_length = 0;
        bullet_translation_start = NULL;
        bullet_translation_start_length = 0;
        bullet_direction = NULL;
        bullet_direction_length = 0;
        bulletway_length = 0.0f;
        old_time = 0;
        factor = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_FLY_ON::renderCallback = NULL;
X3dCallback X3dScript_FLY_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_FLY_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_FLY_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_FLY_ON::processEventCallback = NULL;



class X3dScript_THING_SCRIPT : public X3dNode {
public:
    bool isover_in;
    bool fire_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_THING_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_THING_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_THING_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_THING_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX2_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX2_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX2_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX2_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX2_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX2_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX2_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX3_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX3_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX3_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX3_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX3_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX3_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX3_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX4_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX4_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX4_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX4_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX4_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX4_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX4_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_BOX5_HIT_CHECK : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* box_position;
    int box_position_length;
    int deleted_out;
    int choice_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int deleted;
    float* box;
    int box_length;
    float boxsize;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_HIT_CHECK() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        deleted = 0;
        box = NULL;
        box_length = 0;
        boxsize = 0.0f;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_HIT_CHECK::renderCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_HIT_CHECK::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_HIT_CHECK::processEventCallback = NULL;



class X3dScript_BOX5_MOVE_ON : public X3dNode {
public:
    float* translation_in;
    int translation_in_length;
    float* translation_out;
    int translation_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    float* step_translation;
    int step_translation_length;
    int step;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_MOVE_ON() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        step_translation = NULL;
        step_translation_length = 0;
        step = 0;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_MOVE_ON::renderCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_MOVE_ON::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_MOVE_ON::processEventCallback = NULL;



class X3dScript_BOX5_HIDE_SCRIPT : public X3dNode {
public:
    bool isover_in;
    float* hide_out;
    int hide_out_length;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_BOX5_HIDE_SCRIPT() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_BOX5_HIDE_SCRIPT::renderCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::treeRenderCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::doWithDataCallback = NULL;
X3dCallback X3dScript_BOX5_HIDE_SCRIPT::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_BOX5_HIDE_SCRIPT::processEventCallback = NULL;



class X3dScript_LOAD_NEW_WORLD : public X3dNode {
public:
    double starttime_in;
    int box1_in;
    int box2_in;
    int box3_in;
    int box4_in;
    int box5_in;
    double starttime_out;
    int plane_choice;
    int choice_out;
    int victory_out;
    const char** url;
    int url_length;
    bool directOutput;
    bool mustEvaluate;
    int* box_in;
    int box_in_length;
    const char** your_url;
    int your_url_length;
    const char** param;
    int param_length;
    int deleted;
    int waitticks;
    bool victory;
    void* extra_data;

    int getType() const { return 234; }

    static X3dCallback renderCallback;
    static X3dCallback treeRenderCallback;
    static X3dCallback doWithDataCallback;
    static X3dCallback treeDoWithDataCallback;
    static X3dProcessEventCallback processEventCallback;

    X3dScript_LOAD_NEW_WORLD() {
        url = NULL;
        url_length = 0;
        directOutput = false;
        mustEvaluate = false;
        box_in = NULL;
        box_in_length = 0;
        your_url = NULL;
        your_url_length = 0;
        param = NULL;
        param_length = 0;
        deleted = 0;
        waitticks = 0;
        victory = false;
        extra_data = NULL;
    }
    virtual void treeRender(void *dataptr) {
        int i;
        if (treeRenderCallback) {
            treeRenderCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeRender(dataptr);
        else {
            render(dataptr);
        }
    }
    virtual void render(void *dataptr) {
        if (renderCallback)
            renderCallback(this, dataptr);
    }
    virtual void treeDoWithData(void *dataptr) {
        int i;
        if (treeDoWithDataCallback) {
            treeDoWithDataCallback(this, dataptr);
            return;
        }
        if (m_protoRoot != NULL)
            m_protoRoot->treeDoWithData(dataptr);
        else {
            doWithData(dataptr);
        }
    }
    virtual void doWithData(void *dataptr) {
        if (doWithDataCallback)
            doWithDataCallback(this, dataptr);
    }
};

X3dCallback X3dScript_LOAD_NEW_WORLD::renderCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::treeRenderCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::doWithDataCallback = NULL;
X3dCallback X3dScript_LOAD_NEW_WORLD::treeDoWithDataCallback = NULL;
X3dProcessEventCallback X3dScript_LOAD_NEW_WORLD::processEventCallback = NULL;




bool X3dScript_BOX1_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_HIT_CHECK *self = (X3dScript_BOX1_HIT_CHECK *)node;
   static bool BOX1_HIT_CHECK_init = false;
   if (!BOX1_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX1_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX1_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_MOVE_ON *self = (X3dScript_BOX1_MOVE_ON *)node;
   static bool BOX1_MOVE_ON_init = false;
   if (!BOX1_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX1_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX1_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX1_HIDE_SCRIPT *self = (X3dScript_BOX1_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_STEERSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_STEER *self = (X3dScript_STEER *)node;
   self->xvalue = -self->wall_xy_in[0] / 1.5;
   self->yvalue = self->wall_xy_in[1] / 1.5;
    return true;
}

bool X3dScript_ROT_TRANS1SendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;
   static bool ROT_TRANS1_init = false;
   if (!ROT_TRANS1_init) {
       self->translation_out[0] = 0;
       self->translation_out[1] = 0;
       self->translation_out[2] = 0;
       ROT_TRANS1_init = true;
   }         

// account quaternion multiplication a b
   extern void quaternionMult(float *c, float *a, float *b);
// make rotation to quaternion with norm 1 
   extern void norm_quaternion_from_rotation(float *quad, float *rot); 

   self->rotation[0] = self->rotation_in[0];
   self->rotation[1] = self->rotation_in[1];
   self->rotation[2] = self->rotation_in[2];
   self->rotation[3] = self->rotation_in[3];
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);
   self->quaternion2[0] = self->translation_in[0];
   self->quaternion2[1] = self->translation_in[1];
   self->quaternion2[2] = self->translation_in[2];
   self->quaternion2[3] = 0;
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);
   self->quaternion4[0] = -self->quaternion1[0];
   self->quaternion4[1] = -self->quaternion1[1];
   self->quaternion4[2] = -self->quaternion1[2];
   self->quaternion4[3] = self->quaternion1[3];
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);
   self->translation_out[0] = self->quaternion2[0];
   self->translation_out[1] = self->quaternion2[1];
   self->translation_out[2] = self->quaternion2[2];
    return true;
}

bool X3dScript_ROT_TRANS2SendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;
   static bool ROT_TRANS1_init = false;
   if (!ROT_TRANS1_init) {
       self->translation_out[0] = 0;
       self->translation_out[1] = 0;
       self->translation_out[2] = 0;
       ROT_TRANS1_init = true;
   }         

// account quaternion multiplication a b
   extern void quaternionMult(float *c, float *a, float *b);
// make rotation to quaternion with norm 1 
   extern void norm_quaternion_from_rotation(float *quad, float *rot); 

   self->rotation[0] = self->rotation_in[0];
   self->rotation[1] = self->rotation_in[1];
   self->rotation[2] = self->rotation_in[2];
   self->rotation[3] = self->rotation_in[3];
   norm_quaternion_from_rotation(self->quaternion1, self->rotation);
   self->quaternion2[0] = self->translation_in[0];
   self->quaternion2[1] = self->translation_in[1];
   self->quaternion2[2] = self->translation_in[2];
   self->quaternion2[3] = 0;
   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);
   self->quaternion4[0] = -self->quaternion1[0];
   self->quaternion4[1] = -self->quaternion1[1];
   self->quaternion4[2] = -self->quaternion1[2];
   self->quaternion4[3] = self->quaternion1[3];
   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);
   self->translation_out[0] = self->quaternion2[0];
   self->translation_out[1] = self->quaternion2[1];
   self->translation_out[2] = self->quaternion2[2];
    return true;
}

bool X3dScript_WALL_FIRESendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_WALL_FIRE *self = (X3dScript_WALL_FIRE *)node;
      static bool WALL_FIRE_init = false;
      if (!WALL_FIRE_init) {
           self->fire = false; 
           WALL_FIRE_init = true;
      }         
      if ((!self->isactive_in) && self->old_isactive)
          self->fire = true;
      else
          self->fire = false;
      self->old_isactive = self->isactive_in;           
    return true;
}

bool X3dScript_ROLL_TURNSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_ROLL_TURN *self = (X3dScript_ROLL_TURN *)node;
   static bool ROLL_TURN_init = false;
   if (!ROLL_TURN_init) {
       self->rotation_out[0]=1;
       self->rotation_out[1]=0;
       self->rotation_out[2]=0;
       self->rotation_out[3]=0;
       ROLL_TURN_init = true;
   }         
   self->rock_and_roll_flag = self->rock_and_roll_flag_in;
   if (self->rock_and_roll_flag==1) {
       self->rotation_out[1]=0;
       self->rotation_out[2]=1;
   } else {
       self->rotation_out[1]=1;
       self->rotation_out[2]=0;
   }
   self->rotation_out[0]=0;
   self->rotation_out[3]=self->xvalue_in * 2.0;
    return true;
}

bool X3dScript_UP_DOWN_TURNSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_UP_DOWN_TURN *self = (X3dScript_UP_DOWN_TURN *)node;
   static bool UP_DOWN_TURN_init = false;
   if (!UP_DOWN_TURN_init) {
       self->rotation_out[0]=0;
       self->rotation_out[1]=0;
       self->rotation_out[2]=0;
       self->rotation_out[3]=0;
       UP_DOWN_TURN_init = true;
   }         
   self->rotation_out[0]=1;
   self->rotation_out[1]=0;
   self->rotation_out[2]=0;
   self->rotation_out[3]=self->yvalue_in * 2.0;
    return true;
}

bool X3dScript_FLY_FORWARDSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_FLY_FORWARD *self = (X3dScript_FLY_FORWARD *)node;
   static bool FLY_FORWARD_init = false;
   if (!FLY_FORWARD_init) {
       self->translation_out[0]=0;
       self->translation_out[1]=0;
       self->translation_out[2]=0;
       FLY_FORWARD_init = true;
   }         
   self->translation_out[0]=0;
   self->translation_out[1]=0;
   self->translation_out[2]=0.15;
   self->time_out = self->time_in;
    return true;
}

bool X3dScript_FLY_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_FLY_ON *self = (X3dScript_FLY_ON *)node;
      static bool FLY_ON_init = false;
      if (!FLY_ON_init) {
           self->bullet_choice_out = -1; 
           FLY_ON_init = true;
      }         
/*
   function exit_time(val)
      {
      brake=TRUE;
      }
   // restart movement
   function enter_time(val)
      {
      brake=FALSE;
      }
*/
      self->factor = 0.06;
      self->old_time = self->time_in;
      if (self->fire_in)
         {
         self->bullet_translation_start = self->top_translation_out;
         // switch on bullet
         self->bullet_choice_out = 0;
         self->bulletway_length = 0;
         }
      self->top_translation[0] = self->translation_old[0] -
                                 self->translation_in[0] * self->factor;
      self->top_translation[1] = self->translation_old[1] -
                                 self->translation_in[1] * self->factor;
      self->top_translation[2] = self->translation_old[2] -
                                 self->translation_in[2] * self->factor;
      if (self->top_translation[2] < -35)
          self->top_translation[2] = 7;
      if (self->top_translation[2] > 7)
          self->top_translation[2] = -35;
      if (self->top_translation[0] < -7)
          self->top_translation[0] = 7;
      if (self->top_translation[0] > 7)
          self->top_translation[0] = -7;
      if (self->top_translation[1] < -7)
          self->top_translation[1] = 7;
      if (self->top_translation[1] > 7)
          self->top_translation[1] = -7;
      self->translation_old = self->top_translation;
      self->translation_out[0] = self->translation_old[0] -
                                 self->translation_in[0] * self->factor;
      self->translation_out[1] = self->translation_old[1] -
                                 self->translation_in[1] * self->factor;
      self->translation_out[2] = self->translation_old[2] - 
                                 self->translation_in[2] * self->factor;
      self->translation_old[0] = self->translation_out[0];
      self->translation_old[1] = self->translation_out[1];
      self->translation_old[2] = self->translation_out[2];
      if (self->bullet_choice_out == 0) 
         {
         if (self->bulletway_length == 0)
            {
            self->bullet_direction = self->translation_in;
            }
         if ((self->bullet_translation_out[2] > 10) ||
             (self->bullet_translation_out[2] < -35))
            {
            self->bulletway_length = 0;
            self->bullet_choice_out = -1;
            self->bullet_translation_out[2] = self->translation_in[2];
            }
         else if (self->bulletway_length < 45)
            {
            self->bullet_translation_out[0] = self->bullet_translation_start[0] -
                                              self->bulletway_length *
                                              self->bullet_direction[0];
            self->bullet_translation_out[1] = self->bullet_translation_start[1] -
                                              self->bulletway_length *
                                              self->bullet_direction[1];
            self->bullet_translation_out[2] = self->bullet_translation_start[2] -
                                              self->bulletway_length *
                                              self->bullet_direction[2];
            self->bulletway_length += 10.0 * self->factor;
            }
         else
            {
            self->bulletway_length = 0;
            self->bullet_choice_out = -1;
            self->bullet_translation_out[2] = self->translation_in[2];
            }
         }
      self->top_translation_out = self->top_translation;   
    return true;
}

bool X3dScript_THING_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_THING_SCRIPT *self = (X3dScript_THING_SCRIPT *)node;
      if (self->isover_in)
         {
         self->hide_out[0]=0.1;
         self->hide_out[1]=0.1;
         self->hide_out[2]=0.1;
         }
      else
         {
         self->hide_out[0]=1.0;
         self->hide_out[1]=1.0;
         self->hide_out[2]=1.0;
         }
      if (self->fire_in)
         {
         self->hide_out[0]=1;
         self->hide_out[1]=1;
         self->hide_out[2]=1;
         }                  
    return true;
}

bool X3dScript_BOX2_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_HIT_CHECK *self = (X3dScript_BOX2_HIT_CHECK *)node;
   static bool BOX2_HIT_CHECK_init = false;
   if (!BOX2_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX2_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX2_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_MOVE_ON *self = (X3dScript_BOX2_MOVE_ON *)node;
   static bool BOX2_MOVE_ON_init = false;
   if (!BOX2_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX2_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX2_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX2_HIDE_SCRIPT *self = (X3dScript_BOX2_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX3_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_HIT_CHECK *self = (X3dScript_BOX3_HIT_CHECK *)node;
   static bool BOX3_HIT_CHECK_init = false;
   if (!BOX3_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 9;
       BOX3_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX3_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_MOVE_ON *self = (X3dScript_BOX3_MOVE_ON *)node;
   static bool BOX3_MOVE_ON_init = false;
   if (!BOX3_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX3_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX3_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX3_HIDE_SCRIPT *self = (X3dScript_BOX3_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX4_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_HIT_CHECK *self = (X3dScript_BOX4_HIT_CHECK *)node;
   static bool BOX4_HIT_CHECK_init = false;
   if (!BOX4_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX4_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX4_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_MOVE_ON *self = (X3dScript_BOX4_MOVE_ON *)node;
   static bool BOX4_MOVE_ON_init = false;
   if (!BOX4_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX4_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX4_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX4_HIDE_SCRIPT *self = (X3dScript_BOX4_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_BOX5_HIT_CHECKSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_HIT_CHECK *self = (X3dScript_BOX5_HIT_CHECK *)node;
   static bool BOX5_HIT_CHECK_init = false;
   if (!BOX5_HIT_CHECK_init) {
       self->choice_out = 0;
       self->deleted_out = 0;
       BOX5_HIT_CHECK_init = true;
   } else if (!self->deleted) 
      {
      self->box[0] = self->box_position[0];
      self->box[1] = self->box_position[1];
      self->box[2] = self->box_position[2];
      if (
          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && 
          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && 
          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && 
          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && 
          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && 
          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)
         )
         {
         self->choice_out = 1;
         self->deleted_out = 1;
         self->deleted = 1;
         }
      else
         self->choice_out = 0;
   }
   else
      self->choice_out = 1;
    return true;
}

bool X3dScript_BOX5_MOVE_ONSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_MOVE_ON *self = (X3dScript_BOX5_MOVE_ON *)node;
   static bool BOX5_MOVE_ON_init = false;
   if (!BOX5_MOVE_ON_init) {
       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;
       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;
       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;
       BOX5_MOVE_ON_init = true;
   }         
   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;
   if ((self->translation_out[0] + self->step_translation[0] < 4) && 
       (self->translation_out[0] + self->step_translation[0] > -4)) 
       self->translation_out[0] += self->step_translation[0];
   if ((self->translation_out[1] + self->step_translation[1] < 3) &&
       (self->translation_out[1] + self->step_translation[1] > -3)) 
       self->translation_out[1] += self->step_translation[1];
   if ((self->translation_out[2] + self->step_translation[2] < -1) && 
       (self->translation_out[2] + self->step_translation[2] > -30)) 
       self->translation_out[2] += self->step_translation[2];
    return true;
}

bool X3dScript_BOX5_HIDE_SCRIPTSendEventsCallback(X3dNode *node, const char *event,void* data)
{

   X3dScript_BOX5_HIDE_SCRIPT *self = (X3dScript_BOX5_HIDE_SCRIPT *)node;
   if (self->isover_in) {             
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 100;
   } else {
       self->hide_out[0] = 0;
       self->hide_out[1] = 0;
       self->hide_out[2] = 0;
   }
    return true;
}

bool X3dScript_LOAD_NEW_WORLDSendEventsCallback(X3dNode *node, const char *event,void* data)
{

      X3dScript_LOAD_NEW_WORLD *self = (X3dScript_LOAD_NEW_WORLD *)node;
      static bool LOAD_NEW_WORLD_init = false;
      if (!LOAD_NEW_WORLD_init) {
          self->box_in[0] = 0;
          self->box_in[1] = 0;
          self->box_in[2] = 0;
          self->box_in[3] = 0;
          self->box_in[4] = 0;
          self->starttime_out = 0;
          self->plane_choice = 0;
          self->deleted = self->waitticks;
          LOAD_NEW_WORLD_init = true;
      }         
      if (!self->victory)      
         if (self->deleted > 0)
            {
            if (self->deleted == self->waitticks)
               {
//               starttime_out=val;
               self->choice_out = 0;
               }
            self->deleted--;
            }         
         else
            self->choice_out = -1;

      if ((self->box1_in == 1) && (self->box_in[0] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[0] = self->box1_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box2_in == 1) && (self->box_in[1] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[1] = self->box2_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box3_in == 1) && (self->box_in[2] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[2] = self->box3_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box4_in == 1) && (self->box_in[3] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[3] = self->box4_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box5_in == 1) && (self->box_in[4] == 0))
          {
          self->deleted = self->waitticks;          
          self->box_in[4] = self->box5_in;
          if (self->plane_choice == 0)
              self->plane_choice = 1;
          else
              self->plane_choice = 0;
          }
      if ((self->box_in[0] + 
           self->box_in[1] + 
           self->box_in[2] + 
           self->box_in[3] +
           self->box_in[4]) == 5)
          {
          self->victory = true;
          self->victory_out = 1;
          }
    return true;
}

class X3dSceneGraph {
public:
    X3dGroup root;
    X3dSwitch Switch2;
    X3dGroup Group_71;
    X3dTransform BOX1m_HIDE;
    X3dSwitch BOX1m_SWITCH;
    X3dTransform BOX1m_MOVE;
    X3dTouchSensor BOX1m_SENSOR;
    X3dGroup Inline_5;
    X3dGroup Group1;
    X3dShape Shape_92;
    X3dAppearance Appearance_93;
    X3dMaterial Material_94;
    X3dIndexedFaceSet IndexedFaceSet_1;
    X3dColor Color1_4;
    X3dCoordinate Coordinate1_9;
    X3dNormal Normal1;
    X3dColor Color1;
    X3dCoordinate Coordinate1;
    X3dNormal Normal_2;
    X3dColor Color_95;
    X3dCoordinate Coordinate_96;
    X3dShape Shape_97;
    X3dAppearance Appearance_98;
    X3dMaterial Material_99;
    X3dImageTexture ImageTexture_100;
    X3dIndexedFaceSet IndexedFaceSet_3;
    X3dCoordinate Coordinate2_9;
    X3dNormal Normal2;
    X3dTextureCoordinate TextureCoordinate1_16;
    X3dCoordinate Coordinate2;
    X3dNormal Normal_4;
    X3dTextureCoordinate TextureCoordinate1;
    X3dCoordinate Coordinate_101;
    X3dTextureCoordinate TextureCoordinate_102;
    X3dWorldInfo WorldInfo1;
    X3dScript_BOX1_HIT_CHECK BOX1m_HITm_CHECK;
    X3dScript_BOX1_MOVE_ON BOX1m_MOVEm_ON;
    X3dScript_BOX1_HIDE_SCRIPT BOX1m_HIDEm_SCRIPT;
    X3dScript_STEER STEER;
    X3dPlaneSensor WALLm_SENSOR;
    X3dTouchSensor WALLm_SENSOR2;
    X3dScript_ROT_TRANS1 ROTm_TRANS1;
    X3dScript_ROT_TRANS2 ROTm_TRANS2;
    X3dBackground Background_72;
    X3dScript_WALL_FIRE WALLm_FIRE;
    X3dTransform THING;
    X3dTransform ROLLm_AXIS;
    X3dTransform UPm_DOWNm_AXIS;
    X3dTouchSensor THINGm_SENSOR;
    X3dTransform HIDEm_THING;
    X3dGroup Inline_22;
    X3dTransform Transform1;
    X3dGroup Inline_10;
    X3dGroup Group2;
    X3dTransform Transform_103;
    X3dShape Shape_104;
    X3dAppearance Appearance_105;
    X3dMaterial A0m_1;
    X3dIndexedFaceSet IndexedFaceSet_6;
    X3dColor Color2_4;
    X3dCoordinate Coordinate3_9;
    X3dNormal Normal3_5;
    X3dColor Color2;
    X3dCoordinate Coordinate3;
    X3dNormal Normal3;
    X3dColor Color_106;
    X3dCoordinate Coordinate_107;
    X3dNormal Normal_108;
    X3dTransform Transform_109;
    X3dShape Shape_110;
    X3dAppearance Appearance_111;
    X3dMaterial A0m_2;
    X3dIndexedFaceSet IndexedFaceSet_7;
    X3dColor Color3_4;
    X3dCoordinate Coordinate4_9;
    X3dNormal Normal4_5;
    X3dColor Color3;
    X3dCoordinate Coordinate4;
    X3dNormal Normal4;
    X3dColor Color_112;
    X3dCoordinate Coordinate_113;
    X3dNormal Normal_114;
    X3dTransform Transform_115;
    X3dShape Shape_116;
    X3dAppearance Appearance_117;
    X3dMaterial Material_118;
    X3dIndexedFaceSet IndexedFaceSet_8;
    X3dCoordinate Coordinate5_9;
    X3dNormal Normal5_5;
    X3dCoordinate Coordinate5;
    X3dNormal Normal5;
    X3dCoordinate Coordinate_119;
    X3dNormal Normal_120;
    X3dTransform Transform_121;
    X3dShape Shape_122;
    X3dAppearance Appearance_123;
    X3dMaterial Material_124;
    X3dIndexedFaceSet IndexedFaceSet_9;
    X3dCoordinate Coordinate6_9;
    X3dNormal Normal6_5;
    X3dCoordinate Coordinate6;
    X3dNormal Normal6;
    X3dCoordinate Coordinate_125;
    X3dNormal Normal_126;
    X3dTransform Transform_127;
    X3dTransform Transform_128;
    X3dTransform Transform_129;
    X3dShape Shape_130;
    X3dAppearance Appearance_131;
    X3dMaterial Material_132;
    X3dIndexedFaceSet IndexedFaceSet_11;
    X3dCoordinate Coordinate7_9;
    X3dNormal Normal7_5;
    X3dTextureCoordinate TextureCoordinate2_16;
    X3dCoordinate Coordinate7;
    X3dNormal Normal7;
    X3dTextureCoordinate TextureCoordinate2;
    X3dCoordinate Coordinate_133;
    X3dNormal Normal_134;
    X3dTextureCoordinate TextureCoordinate_135;
    X3dTransform Transform_136;
    X3dShape Shape_137;
    X3dAppearance Appearance_138;
    X3dMaterial Material_139;
    X3dIndexedFaceSet IndexedFaceSet_12;
    X3dCoordinate Coordinate8_9;
    X3dNormal Normal8_5;
    X3dTextureCoordinate TextureCoordinate3_16;
    X3dCoordinate Coordinate8;
    X3dNormal Normal8;
    X3dTextureCoordinate TextureCoordinate3;
    X3dCoordinate Coordinate_140;
    X3dNormal Normal_141;
    X3dTextureCoordinate TextureCoordinate_142;
    X3dTransform Transform_143;
    X3dTransform Transform_144;
    X3dShape Shape_145;
    X3dAppearance Appearance_146;
    X3dMaterial Material_147;
    X3dIndexedFaceSet IndexedFaceSet_13;
    X3dCoordinate Coordinate9_9;
    X3dNormal Normal9_5;
    X3dTextureCoordinate TextureCoordinate4_16;
    X3dCoordinate Coordinate9;
    X3dNormal Normal9;
    X3dTextureCoordinate TextureCoordinate4;
    X3dCoordinate Coordinate_148;
    X3dNormal Normal_149;
    X3dTextureCoordinate TextureCoordinate_150;
    X3dTransform Transform_151;
    X3dShape Shape_152;
    X3dAppearance Appearance_153;
    X3dMaterial Material_154;
    X3dIndexedFaceSet IndexedFaceSet_14;
    X3dCoordinate Coordinate10_9;
    X3dNormal Normal10_5;
    X3dTextureCoordinate TextureCoordinate5_16;
    X3dCoordinate Coordinate10;
    X3dNormal Normal10;
    X3dTextureCoordinate TextureCoordinate5;
    X3dCoordinate Coordinate_155;
    X3dNormal Normal_156;
    X3dTextureCoordinate TextureCoordinate_157;
    X3dGroup Inline_21;
    X3dGroup Inline_17;
    X3dGroup Group3;
    X3dTransform Transform_158;
    X3dCoordinateInterpolator CORINTm_5;
    X3dTimeSensor TIMERm_4;
    X3dGroup Group_159;
    X3dBackground Background_160;
    X3dShape Shape_161;
    X3dAppearance Appearance_162;
    X3dMaterial Material_163;
    X3dIndexedFaceSet IndexedFaceSet_15;
    X3dCoordinate COORD1_6;
    X3dNormal Normal11;
    X3dCoordinate COORD1_5;
    X3dNormal Normal_16;
    X3dCoordinate COORD1m_4;
    X3dGroup Inline_20;
    X3dGroup Group4;
    X3dTransform Transform_164;
    X3dCoordinateInterpolator CORINTm_6;
    X3dTimeSensor TIMERm_5;
    X3dGroup Group_165;
    X3dBackground Background_166;
    X3dShape Shape_167;
    X3dAppearance Appearance_168;
    X3dMaterial Material_169;
    X3dIndexedFaceSet IndexedFaceSet_18;
    X3dCoordinate COORD1_9;
    X3dNormal Normal12;
    X3dCoordinate COORD1_8;
    X3dNormal Normal_19;
    X3dCoordinate COORD1m_7;
    X3dWorldInfo WorldInfo2;
    X3dSwitch PLANEm_SWITCH;
    X3dGroup Inline_25;
    X3dTransform Transform2;
    X3dShape Shape_170;
    X3dAppearance Appearance_171;
    X3dMaterial Material_172;
    X3dIndexedFaceSet IndexedFaceSet_23;
    X3dColor Color4_4;
    X3dCoordinate Coordinate11_9;
    X3dNormal Normal13;
    X3dColor Color4;
    X3dCoordinate Coordinate11;
    X3dNormal Normal_24;
    X3dColor Color_173;
    X3dCoordinate Coordinate_174;
    X3dTransform Transform_73;
    X3dSwitch BULLETm_SWITCH;
    X3dTransform BULLETm_MOVE;
    X3dTouchSensor DUMMYm_BULLET;
    X3dGroup Inline_28;
    X3dTransform Transform3;
    X3dGroup Inline_27;
    X3dShape Shape1;
    X3dAppearance Appearance_175;
    X3dMaterial Material_176;
    X3dImageTexture ImageTexture_177;
    X3dIndexedFaceSet IndexedFaceSet_26;
    X3dCoordinate Coordinate12_9;
    X3dNormal Normal14_5;
    X3dTextureCoordinate TextureCoordinate6_16;
    X3dCoordinate Coordinate12;
    X3dNormal Normal14;
    X3dTextureCoordinate TextureCoordinate6;
    X3dCoordinate Coordinate_178;
    X3dNormal Normal_179;
    X3dTextureCoordinate TextureCoordinate_180;
    X3dWorldInfo WorldInfo3;
    X3dWorldInfo WorldInfo4;
    X3dTimeSensor TIMER;
    X3dSound SOUND;
    X3dAudioClip SOUNDCLIP;
    X3dNavigationInfo NavigationInfo_74;
    X3dScript_ROLL_TURN ROLLm_TURN;
    X3dScript_UP_DOWN_TURN UPm_DOWNm_TURN;
    X3dScript_FLY_FORWARD FLYm_FORWARD;
    X3dScript_FLY_ON FLYm_ON;
    X3dScript_THING_SCRIPT THINGm_SCRIPT;
    X3dViewpoint Viewpoint_75;
    X3dViewpoint Viewpoint_76;
    X3dWorldInfo WorldInfo_77;
    X3dTransform Transform_78;
    X3dShape Shape_79;
    X3dAppearance Appearance_80;
    X3dMaterial Material_81;
    X3dIndexedFaceSet Text1;
    X3dCoordinate Coordinate13;
    X3dCoordinate Coordinate_181;
    X3dGroup Group_82;
    X3dSwitch Switch1;
    X3dTransform Transform_83;
    X3dShape Shape_84;
    X3dAppearance Appearance_85;
    X3dMaterial Material_86;
    X3dIndexedFaceSet Text1m_5;
    X3dCoordinate Coordinate14;
    X3dNormal Normal15;
    X3dTextureCoordinate TextureCoordinate7;
    X3dCoordinate Coordinate_182;
    X3dNormal Normal_183;
    X3dTextureCoordinate TextureCoordinate_184;
    X3dTouchSensor DUMMYm_TEXT;
    X3dShape Shape_87;
    X3dAppearance Appearance_88;
    X3dMaterial Material_89;
    X3dIndexedFaceSet IndexedFaceSet_29;
    X3dCoordinate COORD_5;
    X3dNormal Normal16_5;
    X3dCoordinate COORD_4;
    X3dNormal Normal16;
    X3dCoordinate COORD;
    X3dNormal Normal_90;
    X3dTransform BOX2m_HIDE;
    X3dSwitch BOX2m_SWITCH;
    X3dTransform BOX2m_MOVE;
    X3dTouchSensor BOX2m_SENSOR;
    X3dGroup Inline_34;
    X3dGroup Group5;
    X3dShape Shape_185;
    X3dAppearance Appearance_186;
    X3dMaterial Material_187;
    X3dIndexedFaceSet IndexedFaceSet_30;
    X3dColor Color5_4;
    X3dCoordinate Coordinate15_9;
    X3dNormal Normal17;
    X3dColor Color5;
    X3dCoordinate Coordinate15;
    X3dNormal Normal_31;
    X3dColor Color_188;
    X3dCoordinate Coordinate_189;
    X3dShape Shape_190;
    X3dAppearance Appearance_191;
    X3dMaterial Material_192;
    X3dImageTexture ImageTexture_193;
    X3dIndexedFaceSet IndexedFaceSet_32;
    X3dCoordinate Coordinate16_9;
    X3dNormal Normal18;
    X3dTextureCoordinate TextureCoordinate8_16;
    X3dCoordinate Coordinate16;
    X3dNormal Normal_33;
    X3dTextureCoordinate TextureCoordinate8;
    X3dCoordinate Coordinate_194;
    X3dTextureCoordinate TextureCoordinate_195;
    X3dWorldInfo WorldInfo5;
    X3dScript_BOX2_HIT_CHECK BOX2m_HITm_CHECK;
    X3dScript_BOX2_MOVE_ON BOX2m_MOVEm_ON;
    X3dScript_BOX2_HIDE_SCRIPT BOX2m_HIDEm_SCRIPT;
    X3dTransform BOX3m_HIDE;
    X3dSwitch BOX3m_SWITCH;
    X3dTransform BOX3m_MOVE;
    X3dTouchSensor BOX3m_SENSOR;
    X3dGroup Inline_39;
    X3dGroup Group6;
    X3dShape Shape_196;
    X3dAppearance Appearance_197;
    X3dMaterial Material_198;
    X3dIndexedFaceSet IndexedFaceSet_35;
    X3dColor Color6_4;
    X3dCoordinate Coordinate17_9;
    X3dNormal Normal19;
    X3dColor Color6;
    X3dCoordinate Coordinate17;
    X3dNormal Normal_36;
    X3dColor Color_199;
    X3dCoordinate Coordinate_200;
    X3dShape Shape_201;
    X3dAppearance Appearance_202;
    X3dMaterial Material_203;
    X3dImageTexture ImageTexture_204;
    X3dIndexedFaceSet IndexedFaceSet_37;
    X3dCoordinate Coordinate18_9;
    X3dNormal Normal20;
    X3dTextureCoordinate TextureCoordinate9_16;
    X3dCoordinate Coordinate18;
    X3dNormal Normal_38;
    X3dTextureCoordinate TextureCoordinate9;
    X3dCoordinate Coordinate_205;
    X3dTextureCoordinate TextureCoordinate_206;
    X3dWorldInfo WorldInfo6;
    X3dScript_BOX3_HIT_CHECK BOX3m_HITm_CHECK;
    X3dScript_BOX3_MOVE_ON BOX3m_MOVEm_ON;
    X3dScript_BOX3_HIDE_SCRIPT BOX3m_HIDEm_SCRIPT;
    X3dTransform BOX4m_HIDE;
    X3dSwitch BOX4m_SWITCH;
    X3dTransform BOX4m_MOVE;
    X3dTouchSensor BOX4m_SENSOR;
    X3dGroup Inline_44;
    X3dGroup Group7;
    X3dShape Shape_207;
    X3dAppearance Appearance_208;
    X3dMaterial Material_209;
    X3dIndexedFaceSet IndexedFaceSet_40;
    X3dColor Color7_4;
    X3dCoordinate Coordinate19_9;
    X3dNormal Normal21;
    X3dColor Color7;
    X3dCoordinate Coordinate19;
    X3dNormal Normal_41;
    X3dColor Color_210;
    X3dCoordinate Coordinate_211;
    X3dShape Shape_212;
    X3dAppearance Appearance_213;
    X3dMaterial Material_214;
    X3dImageTexture ImageTexture_215;
    X3dIndexedFaceSet IndexedFaceSet_42;
    X3dCoordinate Coordinate20_9;
    X3dNormal Normal22;
    X3dTextureCoordinate TextureCoordinate10_16;
    X3dCoordinate Coordinate20;
    X3dNormal Normal_43;
    X3dTextureCoordinate TextureCoordinate10;
    X3dCoordinate Coordinate_216;
    X3dTextureCoordinate TextureCoordinate_217;
    X3dWorldInfo WorldInfo7;
    X3dScript_BOX4_HIT_CHECK BOX4m_HITm_CHECK;
    X3dScript_BOX4_MOVE_ON BOX4m_MOVEm_ON;
    X3dScript_BOX4_HIDE_SCRIPT BOX4m_HIDEm_SCRIPT;
    X3dTransform BOX5m_HIDE;
    X3dSwitch BOX5m_SWITCH;
    X3dTransform BOX5m_MOVE;
    X3dTouchSensor BOX5m_SENSOR;
    X3dGroup Inline_49;
    X3dGroup Group8;
    X3dShape Shape_218;
    X3dAppearance Appearance_219;
    X3dMaterial Material_220;
    X3dIndexedFaceSet IndexedFaceSet_45;
    X3dColor Color8_4;
    X3dCoordinate Coordinate21_9;
    X3dNormal Normal23;
    X3dColor Color8;
    X3dCoordinate Coordinate21;
    X3dNormal Normal_46;
    X3dColor Color_221;
    X3dCoordinate Coordinate_222;
    X3dShape Shape_223;
    X3dAppearance Appearance_224;
    X3dMaterial Material_225;
    X3dImageTexture ImageTexture_226;
    X3dIndexedFaceSet IndexedFaceSet_47;
    X3dCoordinate Coordinate22_9;
    X3dNormal Normal24;
    X3dTextureCoordinate TextureCoordinate11_16;
    X3dCoordinate Coordinate22;
    X3dNormal Normal_48;
    X3dTextureCoordinate TextureCoordinate11;
    X3dCoordinate Coordinate_227;
    X3dTextureCoordinate TextureCoordinate_228;
    X3dWorldInfo WorldInfo8;
    X3dScript_BOX5_HIT_CHECK BOX5m_HITm_CHECK;
    X3dScript_BOX5_MOVE_ON BOX5m_MOVEm_ON;
    X3dScript_BOX5_HIDE_SCRIPT BOX5m_HIDEm_SCRIPT;
    X3dTransform Transform_91;
    X3dGroup Inline_70;
    X3dBackground Background1;
    X3dAnchor Anchor1;
    X3dBillboard Billboard_229;
    X3dShape Shape_230;
    X3dAppearance Appearance_231;
    X3dMaterial Material_232;
    X3dIndexedFaceSet IndexedFaceSet_50;
    X3dCoordinate COORD_8;
    X3dNormal Normal25;
    X3dCoordinate COORD_7;
    X3dNormal Normal_51;
    X3dCoordinate COORDm_6;
    X3dAnchor Anchor2;
    X3dTransform SCHRIFTm_6;
    X3dTransform Transform_233;
    X3dGroup Inline_68;
    X3dTransform Transform4;
    X3dGroup Inline_56;
    X3dGroup Group9;
    X3dTransform Transform_234;
    X3dShape Shape_235;
    X3dAppearance Appearance_236;
    X3dMaterial A0m_3;
    X3dIndexedFaceSet IndexedFaceSet_52;
    X3dColor Color9_4;
    X3dCoordinate Coordinate23_9;
    X3dNormal Normal26_5;
    X3dColor Color9;
    X3dCoordinate Coordinate23;
    X3dNormal Normal26;
    X3dColor Color_237;
    X3dCoordinate Coordinate_238;
    X3dNormal Normal_239;
    X3dTransform Transform_240;
    X3dShape Shape_241;
    X3dAppearance Appearance_242;
    X3dMaterial A0m_4;
    X3dIndexedFaceSet IndexedFaceSet_53;
    X3dColor Color10_4;
    X3dCoordinate Coordinate24_9;
    X3dNormal Normal27_5;
    X3dColor Color10;
    X3dCoordinate Coordinate24;
    X3dNormal Normal27;
    X3dColor Color_243;
    X3dCoordinate Coordinate_244;
    X3dNormal Normal_245;
    X3dTransform Transform_246;
    X3dShape Shape_247;
    X3dAppearance Appearance_248;
    X3dMaterial Material_249;
    X3dIndexedFaceSet IndexedFaceSet_54;
    X3dCoordinate Coordinate25_9;
    X3dNormal Normal28_5;
    X3dCoordinate Coordinate25;
    X3dNormal Normal28;
    X3dCoordinate Coordinate_250;
    X3dNormal Normal_251;
    X3dTransform Transform_252;
    X3dShape Shape_253;
    X3dAppearance Appearance_254;
    X3dMaterial Material_255;
    X3dIndexedFaceSet IndexedFaceSet_55;
    X3dCoordinate Coordinate26_9;
    X3dNormal Normal29_5;
    X3dCoordinate Coordinate26;
    X3dNormal Normal29;
    X3dCoordinate Coordinate_256;
    X3dNormal Normal_257;
    X3dTransform Transform_258;
    X3dTransform Transform_259;
    X3dTransform Transform_260;
    X3dShape Shape_261;
    X3dAppearance Appearance_262;
    X3dMaterial Material_263;
    X3dIndexedFaceSet IndexedFaceSet_57;
    X3dCoordinate Coordinate27_9;
    X3dNormal Normal30_5;
    X3dTextureCoordinate TextureCoordinate12_16;
    X3dCoordinate Coordinate27;
    X3dNormal Normal30;
    X3dTextureCoordinate TextureCoordinate12;
    X3dCoordinate Coordinate_264;
    X3dNormal Normal_265;
    X3dTextureCoordinate TextureCoordinate_266;
    X3dTransform Transform_267;
    X3dShape Shape_268;
    X3dAppearance Appearance_269;
    X3dMaterial Material_270;
    X3dIndexedFaceSet IndexedFaceSet_58;
    X3dCoordinate Coordinate28_9;
    X3dNormal Normal31_5;
    X3dTextureCoordinate TextureCoordinate13_16;
    X3dCoordinate Coordinate28;
    X3dNormal Normal31;
    X3dTextureCoordinate TextureCoordinate13;
    X3dCoordinate Coordinate_271;
    X3dNormal Normal_272;
    X3dTextureCoordinate TextureCoordinate_273;
    X3dTransform Transform_274;
    X3dTransform Transform_275;
    X3dShape Shape_276;
    X3dAppearance Appearance_277;
    X3dMaterial Material_278;
    X3dIndexedFaceSet IndexedFaceSet_59;
    X3dCoordinate Coordinate29_9;
    X3dNormal Normal32_5;
    X3dTextureCoordinate TextureCoordinate14_16;
    X3dCoordinate Coordinate29;
    X3dNormal Normal32;
    X3dTextureCoordinate TextureCoordinate14;
    X3dCoordinate Coordinate_279;
    X3dNormal Normal_280;
    X3dTextureCoordinate TextureCoordinate_281;
    X3dTransform Transform_282;
    X3dShape Shape_283;
    X3dAppearance Appearance_284;
    X3dMaterial Material_285;
    X3dIndexedFaceSet IndexedFaceSet_60;
    X3dCoordinate Coordinate30_9;
    X3dNormal Normal33_5;
    X3dTextureCoordinate TextureCoordinate15_16;
    X3dCoordinate Coordinate30;
    X3dNormal Normal33;
    X3dTextureCoordinate TextureCoordinate15;
    X3dCoordinate Coordinate_286;
    X3dNormal Normal_287;
    X3dTextureCoordinate TextureCoordinate_288;
    X3dGroup Inline_67;
    X3dGroup Inline_63;
    X3dGroup Group10;
    X3dTransform Transform_289;
    X3dCoordinateInterpolator CORINTm_7;
    X3dTimeSensor TIMERm_6;
    X3dGroup Group_290;
    X3dBackground Background_291;
    X3dShape Shape_292;
    X3dAppearance Appearance_293;
    X3dMaterial Material_294;
    X3dIndexedFaceSet IndexedFaceSet_61;
    X3dCoordinate COORD1_12;
    X3dNormal Normal34;
    X3dCoordinate COORD1_11;
    X3dNormal Normal_62;
    X3dCoordinate COORD1m_10;
    X3dGroup Inline_66;
    X3dGroup Group11;
    X3dTransform Transform_295;
    X3dCoordinateInterpolator CORINTm_8;
    X3dTimeSensor TIMERm_7;
    X3dGroup Group_296;
    X3dBackground Background_297;
    X3dShape Shape_298;
    X3dAppearance Appearance_299;
    X3dMaterial Material_300;
    X3dIndexedFaceSet IndexedFaceSet_64;
    X3dCoordinate COORD1_15;
    X3dNormal Normal35;
    X3dCoordinate COORD1_14;
    X3dNormal Normal_65;
    X3dCoordinate COORD1m_13;
    X3dWorldInfo WorldInfo9;
    X3dTransform Transform_301;
    X3dShape Shape_302;
    X3dIndexedFaceSet Text_69;
    X3dCoordinate Coordinate31;
    X3dNormal Normal36;
    X3dTextureCoordinate TextureCoordinate16;
    X3dCoordinate Coordinate_303;
    X3dNormal Normal_304;
    X3dTextureCoordinate TextureCoordinate_305;
    X3dOrientationInterpolator ROTATORm_6;
    X3dTimeSensor TIMERm_8;
    X3dScript_LOAD_NEW_WORLD LOADm_NEWm_WORLD;
public:
    X3dSceneGraph();
    void render(void *data) { root.treeRender(data); }
    void doWithData(void *data) { root.treeDoWithData(data); }
    X3dNode *getNodeFromGlName(int glName);
};

X3dSceneGraph::X3dSceneGraph() {
    root.m_parent = (X3dNode *)NULL;
    Switch2.m_parent = (X3dNode *)&root;
    Group_71.m_parent = (X3dNode *)&Switch2;
    BOX1m_HIDE.route_sources =  NULL;
    BOX1m_HIDE.num_route_source = 0;
    BOX1m_HIDE.m_parent = (X3dNode *)&Group_71;
    BOX1m_SWITCH.m_parent = (X3dNode *)&BOX1m_HIDE;
    BOX1m_MOVE.route_sources =  NULL;
    BOX1m_MOVE.num_route_source = 0;
    BOX1m_MOVE.m_parent = (X3dNode *)&BOX1m_SWITCH;
    BOX1m_SENSOR.m_parent = (X3dNode *)&BOX1m_MOVE;
    BOX1m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX1m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX1m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX1m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX1m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX1m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX1m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX1m_SENSOR.isActive = false;
    BOX1m_SENSOR.isOver = false;
    BOX1m_SENSOR.touchTime = 0;
    Inline_5.m_parent = (X3dNode *)&BOX1m_MOVE;
    Group1.m_parent = (X3dNode *)&Inline_5;
    Shape_92.m_parent = (X3dNode *)NULL;
    Appearance_93.m_parent = (X3dNode *)NULL;
    Material_94.m_parent = (X3dNode *)NULL;
    Material_94.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_94.diffuseColor = m_diffuseColor;
    Material_94.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_94.emissiveColor = m_emissiveColor;
    Material_94.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_94.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_94.specularColor = m_specularColor;
    Material_94.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_94.transparency = 0;
    Appearance_93.material = &Material_94;
    Appearance_93.texture = NULL;
    Appearance_93.textureTransform = NULL;
    IndexedFaceSet_1.setGlName(0);
    IndexedFaceSet_1.m_parent = (X3dNode *)NULL;
    Color1_4.m_parent = (X3dNode *)&IndexedFaceSet_1;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color1_4.color = m_color;
    Color1_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate1_9.m_parent = (X3dNode *)&IndexedFaceSet_1;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate1_9.point = m_point;
    Coordinate1_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal1.m_parent = (X3dNode *)&IndexedFaceSet_1;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal1.vector = m_vector;
    Normal1.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_1.color = &Color1_4;
    IndexedFaceSet_1.coord = &Coordinate1_9;
    IndexedFaceSet_1.normal = &Normal1;
    IndexedFaceSet_1.texCoord = NULL;
    IndexedFaceSet_1.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 1487768880, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet_1.colorIndex = m_colorIndex;
    IndexedFaceSet_1.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_1.colorPerVertex = false;
    IndexedFaceSet_1.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_1.coordIndex = m_coordIndex;
    IndexedFaceSet_1.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_1.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet_1.normalIndex = m_normalIndex;
    IndexedFaceSet_1.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_1.normalPerVertex = true;
    IndexedFaceSet_1.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_1.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_1.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_1.set_colorIndex = NULL;
    IndexedFaceSet_1.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_1.set_coordIndex = NULL;
    IndexedFaceSet_1.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_1.set_normalIndex = NULL;
    IndexedFaceSet_1.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_1.set_texCoordIndex = NULL;
    IndexedFaceSet_1.set_texCoordIndex_length = 0;
    }
    Shape_92.appearance = &Appearance_93;
    Shape_92.geometry = &IndexedFaceSet_1;
    Shape_97.m_parent = (X3dNode *)NULL;
    Appearance_98.m_parent = (X3dNode *)NULL;
    Material_99.m_parent = (X3dNode *)NULL;
    Material_99.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_99.diffuseColor = m_diffuseColor;
    Material_99.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_99.emissiveColor = m_emissiveColor;
    Material_99.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_99.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_99.specularColor = m_specularColor;
    Material_99.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_99.transparency = 0;
    ImageTexture_100.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture_100.url = m_url;
    ImageTexture_100.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_100.repeatS = true;
    ImageTexture_100.repeatT = true;
    Appearance_98.material = &Material_99;
    Appearance_98.texture = &ImageTexture_100;
    Appearance_98.textureTransform = NULL;
    IndexedFaceSet_3.setGlName(1);
    IndexedFaceSet_3.m_parent = (X3dNode *)NULL;
    Coordinate2_9.m_parent = (X3dNode *)&IndexedFaceSet_3;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate2_9.point = m_point;
    Coordinate2_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal2.m_parent = (X3dNode *)&IndexedFaceSet_3;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal2.vector = m_vector;
    Normal2.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate1_16.m_parent = (X3dNode *)&IndexedFaceSet_3;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate1_16.point = m_point;
    TextureCoordinate1_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_3.color = NULL;
    IndexedFaceSet_3.coord = &Coordinate2_9;
    IndexedFaceSet_3.normal = &Normal2;
    IndexedFaceSet_3.texCoord = &TextureCoordinate1_16;
    IndexedFaceSet_3.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet_3.colorIndex = m_colorIndex;
    IndexedFaceSet_3.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_3.colorPerVertex = false;
    IndexedFaceSet_3.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_3.coordIndex = m_coordIndex;
    IndexedFaceSet_3.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_3.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_3.normalIndex = m_normalIndex;
    IndexedFaceSet_3.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_3.normalPerVertex = true;
    IndexedFaceSet_3.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_3.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_3.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_3.set_colorIndex = NULL;
    IndexedFaceSet_3.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_3.set_coordIndex = NULL;
    IndexedFaceSet_3.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_3.set_normalIndex = NULL;
    IndexedFaceSet_3.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_3.set_texCoordIndex = NULL;
    IndexedFaceSet_3.set_texCoordIndex_length = 0;
    }
    Shape_97.appearance = &Appearance_98;
    Shape_97.geometry = &IndexedFaceSet_3;
    {
    static X3dNode* m_children[] = { &Shape_92, &Shape_97 };
    Group1.children = m_children;
    Group1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group1.bboxCenter = m_bboxCenter;
    Group1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group1.bboxSize = m_bboxSize;
    Group1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group1.addChildren = NULL;
    Group1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group1.removeChildren = NULL;
    Group1.removeChildren_length = 0;
    }
    WorldInfo1.m_parent = (X3dNode *)&Inline_5;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo1.info = m_info;
    WorldInfo1.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo1.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group1, &WorldInfo1 };
    Inline_5.children = m_children;
    Inline_5.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_5.bboxCenter = m_bboxCenter;
    Inline_5.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_5.bboxSize = m_bboxSize;
    Inline_5.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_5.addChildren = NULL;
    Inline_5.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_5.removeChildren = NULL;
    Inline_5.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX1m_MOVE.rotation = m_rotation;
    BOX1m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX1m_MOVE.translation = m_translation;
    BOX1m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX1m_MOVE.bboxCenter = m_bboxCenter;
    BOX1m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX1m_MOVE.bboxSize = m_bboxSize;
    BOX1m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX1m_MOVE.center = m_center;
    BOX1m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX1m_MOVE.scale = m_scale;
    BOX1m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX1m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX1m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_SENSOR, &Inline_5 };
    BOX1m_MOVE.children = m_children;
    BOX1m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX1m_MOVE.addChildren = NULL;
    BOX1m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX1m_MOVE.removeChildren = NULL;
    BOX1m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX1m_MOVE };
    BOX1m_SWITCH.children = m_children;
    BOX1m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX1m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX1m_HIDE.rotation = m_rotation;
    BOX1m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX1m_HIDE.translation = m_translation;
    BOX1m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX1m_HIDE.bboxCenter = m_bboxCenter;
    BOX1m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX1m_HIDE.bboxSize = m_bboxSize;
    BOX1m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX1m_HIDE.center = m_center;
    BOX1m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX1m_HIDE.scale = m_scale;
    BOX1m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX1m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX1m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_SWITCH };
    BOX1m_HIDE.children = m_children;
    BOX1m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX1m_HIDE.addChildren = NULL;
    BOX1m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX1m_HIDE.removeChildren = NULL;
    BOX1m_HIDE.removeChildren_length = 0;
    }
    BOX1m_HITm_CHECK.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function initialize()\n         {\n         choice_out=0;\n         deleted_out=0;\n         }\n      function translation_in(val)  \n         {\n         if (!deleted)\n            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)\n                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)\n                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)\n               )\n               {\n               choice_out=1;\n               deleted_out=1;\n               deleted=1;\n               }\n            else\n               choice_out=0;\n         else\n            choice_out=1;\n         }\n         \n      function box_position(val)\n         {\n         box[0]=val[0];\n         box[1]=val[1];\n         box[2]=val[2];\n         }  \n      ", "c++:\n   X3dScript_BOX1_HIT_CHECK *self = (X3dScript_BOX1_HIT_CHECK *)node;\n   static bool BOX1_HIT_CHECK_init = false;\n   if (!BOX1_HIT_CHECK_init) {\n       self->choice_out = 0;\n       self->deleted_out = 0;\n       BOX1_HIT_CHECK_init = true;\n   } else if (!self->deleted) \n      {\n      self->box[0] = self->box_position[0];\n      self->box[1] = self->box_position[1];\n      self->box[2] = self->box_position[2];\n      if (\n          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && \n          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && \n          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && \n          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && \n          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && \n          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)\n         )\n         {\n         self->choice_out = 1;\n         self->deleted_out = 1;\n         self->deleted = 1;\n         }\n      else\n         self->choice_out = 0;\n   }\n   else\n      self->choice_out = 1;\n" };
    BOX1m_HITm_CHECK.url = m_url;
    BOX1m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_HITm_CHECK.directOutput = false;
    BOX1m_HITm_CHECK.mustEvaluate = false;
    BOX1m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX1m_HITm_CHECK.box = m_box;
    BOX1m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX1m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX1m_HITm_CHECK.translation_in = m_translation_in;
    BOX1m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX1m_HITm_CHECK.box_position = m_box_position;
    BOX1m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX1m_HITm_CHECK.deleted_out = 0;
    BOX1m_HITm_CHECK.choice_out = 0;
    BOX1m_MOVEm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function nextsteps()\n         {\n         step_translation[0]=(Math.random()-0.5)*0.04;\n         step_translation[1]=(Math.random()-0.5)*0.04;\n         step_translation[2]=(Math.random()-0.5)*0.04;\n         step=Math.random()*200+1;\n         }\n      function initialize()\n         {\n         translation_out[0]=Math.random()*8-4;\n         translation_out[1]=Math.random()*6-3;\n         translation_out[2]=Math.random()*-30-1;\n         nextsteps();\n         }\n      function translation_in(val)  \n         {\n         step--;\n         if (step<=0)\n            nextsteps();\n         if (   (translation_out[0]+step_translation[0]<4) \n             && (translation_out[0]+step_translation[0]>-4)) \n            translation_out[0]+=step_translation[0];\n         if (   (translation_out[1]+step_translation[1]<3) \n             && (translation_out[1]+step_translation[1]>-3)) \n             translation_out[1]+=step_translation[1];\n         if (   (translation_out[2]+step_translation[2]<-1) \n             && (translation_out[2]+step_translation[2]>-30)) \n            translation_out[2]+=step_translation[2];\n         }\n      ", "c++:\n   X3dScript_BOX1_MOVE_ON *self = (X3dScript_BOX1_MOVE_ON *)node;\n   static bool BOX1_MOVE_ON_init = false;\n   if (!BOX1_MOVE_ON_init) {\n       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;\n       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;\n       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;\n       BOX1_MOVE_ON_init = true;\n   }         \n   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   if ((self->translation_out[0] + self->step_translation[0] < 4) && \n       (self->translation_out[0] + self->step_translation[0] > -4)) \n       self->translation_out[0] += self->step_translation[0];\n   if ((self->translation_out[1] + self->step_translation[1] < 3) &&\n       (self->translation_out[1] + self->step_translation[1] > -3)) \n       self->translation_out[1] += self->step_translation[1];\n   if ((self->translation_out[2] + self->step_translation[2] < -1) && \n       (self->translation_out[2] + self->step_translation[2] > -30)) \n       self->translation_out[2] += self->step_translation[2];\n" };
    BOX1m_MOVEm_ON.url = m_url;
    BOX1m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_MOVEm_ON.directOutput = false;
    BOX1m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.step_translation = m_step_translation;
    BOX1m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX1m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.translation_in = m_translation_in;
    BOX1m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX1m_MOVEm_ON.translation_out = m_translation_out;
    BOX1m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX1m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=100;\n         }\n      else\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=0;\n         }\n      }    \n   ", "c++:\n   X3dScript_BOX1_HIDE_SCRIPT *self = (X3dScript_BOX1_HIDE_SCRIPT *)node;\n   if (self->isover_in) {             \n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 100;\n   } else {\n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 0;\n   }\n" };
    BOX1m_HIDEm_SCRIPT.url = m_url;
    BOX1m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX1m_HIDEm_SCRIPT.directOutput = false;
    BOX1m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX1m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX1m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX1m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    STEER.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()\n      {\n      }\n/*\n   function steer()\n      {\n      xvalue=wall_xy[0];\n      yvalue=wall_xy[1];\n      }\n*/\n   function wall_xy_in(val)\n      {\n      xvalue=val[0]/1.5;\n      yvalue=val[1]/1.5;\n      }\n   ", "c++:\n   X3dScript_STEER *self = (X3dScript_STEER *)node;\n   self->xvalue = -self->wall_xy_in[0] / 1.5;\n   self->yvalue = self->wall_xy_in[1] / 1.5;\n" };
    STEER.url = m_url;
    STEER.url_length = sizeof(m_url) / sizeof(const char*);
    }
    STEER.directOutput = false;
    STEER.mustEvaluate = false;
    {
    static float m_wall_xy_in[] = { 0, 0, 0 };
    STEER.wall_xy_in = m_wall_xy_in;
    STEER.wall_xy_in_length = sizeof(m_wall_xy_in) / sizeof(float);
    }
    STEER.xvalue = 0;
    STEER.yvalue = 0;
    WALLm_SENSOR.m_parent = (X3dNode *)&Group_71;
    WALLm_SENSOR.autoOffset = true;
    WALLm_SENSOR.enabled = true;
    {
    static float m_maxPosition[2] = { 1e4, 1e4 };
    WALLm_SENSOR.maxPosition = m_maxPosition;
    WALLm_SENSOR.maxPosition_length = sizeof(m_maxPosition) / sizeof(float);
    }
    {
    static float m_minPosition[2] = { -1e4, -1e4 };
    WALLm_SENSOR.minPosition = m_minPosition;
    WALLm_SENSOR.minPosition_length = sizeof(m_minPosition) / sizeof(float);
    }
    {
    static float m_offset[] = { 0, 0, 0 };
    WALLm_SENSOR.offset = m_offset;
    WALLm_SENSOR.offset_length = sizeof(m_offset) / sizeof(float);
    }
    WALLm_SENSOR.isActive = false;
    {
    static float m_trackPoint_changed[] = { 0, 0, 0 };
    WALLm_SENSOR.trackPoint_changed = m_trackPoint_changed;
    WALLm_SENSOR.trackPoint_changed_length = sizeof(m_trackPoint_changed) / sizeof(float);
    }
    {
    static float m_translation_changed[] = { 0, 0, 0 };
    WALLm_SENSOR.translation_changed = m_translation_changed;
    WALLm_SENSOR.translation_changed_length = sizeof(m_translation_changed) / sizeof(float);
    }
    WALLm_SENSOR2.m_parent = (X3dNode *)&Group_71;
    WALLm_SENSOR2.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    WALLm_SENSOR2.hitNormal_changed = m_hitNormal_changed;
    WALLm_SENSOR2.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    WALLm_SENSOR2.hitPoint_changed = m_hitPoint_changed;
    WALLm_SENSOR2.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    WALLm_SENSOR2.hitTexCoord_changed = m_hitTexCoord_changed;
    WALLm_SENSOR2.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    WALLm_SENSOR2.isActive = false;
    WALLm_SENSOR2.isOver = false;
    WALLm_SENSOR2.touchTime = 0;
    ROTm_TRANS1.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()\n      {\n      translation_out[0]=0;\n      translation_out[1]=0;\n      translation_out[2]=0;\n      }\n// account vektorproduct c = a x b\n   function vectorproduct(c,a,b)\n      {\n      c[0]=a[1]*b[2]-a[2]*b[1];\n      c[1]=a[2]*b[0]-a[0]*b[2];\n      c[2]=a[0]*b[1]-a[1]*b[0];\n      }\n// account scalarproduct a * b\n   function scalarproduct(a,b)\n      {\n      return(a[0]*b[0]+a[1]*b[1]+a[2]*b[2]);\n      }\n// account quaternion multiplication a b\n   function quaternionmult(c,a,b)\n      {\n      vectorproduct(c,a,b);\n      c[0]=c[0]+a[3]*b[0]+b[3]*a[0];\n      c[1]=c[1]+a[3]*b[1]+b[3]*a[1];\n      c[2]=c[2]+a[3]*b[2]+b[3]*a[2];\n      c[3]=a[3]*b[3]-scalarproduct(a,b);\n      }             \n// make rotation to quaternion with norm 1 \n   function norm_quaternion_from_rotation(quad,rot)  \n      {\n      quad[3]=Math.cos(rot[3]/2.0);\n      winkelkorrektur=Math.sin(rot[3]/2.0);\n      quad[0]=rot[0]*winkelkorrektur;\n      quad[1]=rot[1]*winkelkorrektur;\n      quad[2]=rot[2]*winkelkorrektur;\n      leng=Math.sqrt(quad[0]*quad[0]+quad[1]*quad[1]+quad[2]*quad[2]+\n                     quad[3]*quad[3]);\n      if (leng!=0)\n         {\n         quad[0]=quad[0]/leng;\n         quad[1]=quad[1]/leng;\n         quad[2]=quad[2]/leng;\n         quad[3]=quad[3]/leng;\n         }\n      else\n         {\n         quad[0]=0;\n         quad[1]=0;\n         quad[2]=0;\n         quad[3]=0;\n         }\n      }\n   function rotation_in(val)  \n      {\n      rotation[0]=val[0];\n      rotation[1]=val[1];\n      rotation[2]=val[2];\n      rotation[3]=val[3];\n//            debug = new MFString(rotation);\n      }\n   function translation_in(val)  \n      {\n      norm_quaternion_from_rotation(quaternion1,rotation);\n      quaternion2[0]=val[0];\n      quaternion2[1]=val[1];\n      quaternion2[2]=val[2];\n      quaternion2[3]=0;\n      quaternionmult(quaternion3,quaternion1,quaternion2);\n      quaternion4[0]=-quaternion1[0];\n      quaternion4[1]=-quaternion1[1];\n      quaternion4[2]=-quaternion1[2];\n      quaternion4[3]= quaternion1[3];\n      quaternionmult(quaternion2,quaternion3,quaternion4);\n      translation_out[0]=quaternion2[0];\n      translation_out[1]=quaternion2[1];\n      translation_out[2]=quaternion2[2];\n      }\n   ", "c++:\n   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;\n   static bool ROT_TRANS1_init = false;\n   if (!ROT_TRANS1_init) {\n       self->translation_out[0] = 0;\n       self->translation_out[1] = 0;\n       self->translation_out[2] = 0;\n       ROT_TRANS1_init = true;\n   }         \n\n// account quaternion multiplication a b\n   extern void quaternionMult(float *c, float *a, float *b);\n// make rotation to quaternion with norm 1 \n   extern void norm_quaternion_from_rotation(float *quad, float *rot); \n\n   self->rotation[0] = self->rotation_in[0];\n   self->rotation[1] = self->rotation_in[1];\n   self->rotation[2] = self->rotation_in[2];\n   self->rotation[3] = self->rotation_in[3];\n   norm_quaternion_from_rotation(self->quaternion1, self->rotation);\n   self->quaternion2[0] = self->translation_in[0];\n   self->quaternion2[1] = self->translation_in[1];\n   self->quaternion2[2] = self->translation_in[2];\n   self->quaternion2[3] = 0;\n   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);\n   self->quaternion4[0] = -self->quaternion1[0];\n   self->quaternion4[1] = -self->quaternion1[1];\n   self->quaternion4[2] = -self->quaternion1[2];\n   self->quaternion4[3] = self->quaternion1[3];\n   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);\n   self->translation_out[0] = self->quaternion2[0];\n   self->translation_out[1] = self->quaternion2[1];\n   self->translation_out[2] = self->quaternion2[2];\n" };
    ROTm_TRANS1.url = m_url;
    ROTm_TRANS1.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROTm_TRANS1.directOutput = false;
    ROTm_TRANS1.mustEvaluate = false;
    {
    static float m_rotation[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.rotation = m_rotation;
    ROTm_TRANS1.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_quaternion1[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion1 = m_quaternion1;
    ROTm_TRANS1.quaternion1_length = sizeof(m_quaternion1) / sizeof(float);
    }
    {
    static float m_quaternion2[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion2 = m_quaternion2;
    ROTm_TRANS1.quaternion2_length = sizeof(m_quaternion2) / sizeof(float);
    }
    {
    static float m_quaternion3[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion3 = m_quaternion3;
    ROTm_TRANS1.quaternion3_length = sizeof(m_quaternion3) / sizeof(float);
    }
    {
    static float m_quaternion4[] = { 0, 0, 0, 0 };
    ROTm_TRANS1.quaternion4 = m_quaternion4;
    ROTm_TRANS1.quaternion4_length = sizeof(m_quaternion4) / sizeof(float);
    }
    ROTm_TRANS1.leng = 1;
    ROTm_TRANS1.winkelkorrektur = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    ROTm_TRANS1.translation_in = m_translation_in;
    ROTm_TRANS1.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_rotation_in[] = { 0, 0, 1, 0 };
    ROTm_TRANS1.rotation_in = m_rotation_in;
    ROTm_TRANS1.rotation_in_length = sizeof(m_rotation_in) / sizeof(float);
    }
    {
    static const char* m_debug[1];
    ROTm_TRANS1.debug = NULL;
    ROTm_TRANS1.debug_length = 0;
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    ROTm_TRANS1.translation_out = m_translation_out;
    ROTm_TRANS1.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    ROTm_TRANS2.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()\n      {\n      translation_out[0]=0;\n      translation_out[1]=0;\n      translation_out[2]=0;\n      }\n// account vektorproduct c = a x b\n   function vectorproduct(c,a,b)\n      {\n      c[0]=a[1]*b[2]-a[2]*b[1];\n      c[1]=a[2]*b[0]-a[0]*b[2];\n      c[2]=a[0]*b[1]-a[1]*b[0];\n      }\n// account scalarproduct a * b\n   function scalarproduct(a,b)\n      {\n      return(a[0]*b[0]+a[1]*b[1]+a[2]*b[2]);\n      }\n// account quaternion multiplication a b\n   function quaternionmult(c,a,b)\n      {\n      vectorproduct(c,a,b);\n      c[0]=c[0]+a[3]*b[0]+b[3]*a[0];\n      c[1]=c[1]+a[3]*b[1]+b[3]*a[1];\n      c[2]=c[2]+a[3]*b[2]+b[3]*a[2];\n      c[3]=a[3]*b[3]-scalarproduct(a,b);\n      }             \n// make rotation to quaternion with norm 1 \n   function norm_quaternion_from_rotation(quad,rot)  \n      {\n      quad[3]=Math.cos(rot[3]/2.0);\n      winkelkorrektur=Math.sin(rot[3]/2.0);\n      quad[0]=rot[0]*winkelkorrektur;\n      quad[1]=rot[1]*winkelkorrektur;\n      quad[2]=rot[2]*winkelkorrektur;\n      leng=Math.sqrt(quad[0]*quad[0]+quad[1]*quad[1]+quad[2]*quad[2]+\n                     quad[3]*quad[3]);\n      if (leng!=0)\n         {\n         quad[0]=quad[0]/leng;\n         quad[1]=quad[1]/leng;\n         quad[2]=quad[2]/leng;\n         quad[3]=quad[3]/leng;\n         }\n      else\n         {\n         quad[0]=0;\n         quad[1]=0;\n         quad[2]=0;\n         quad[3]=0;\n         }\n      }\n   function rotation_in(val)  \n      {\n      rotation[0]=val[0];\n      rotation[1]=val[1];\n      rotation[2]=val[2];\n      rotation[3]=val[3];\n//            debug = new MFString(rotation);\n      }\n   function translation_in(val)  \n      {\n      norm_quaternion_from_rotation(quaternion1,rotation);\n      quaternion2[0]=val[0];\n      quaternion2[1]=val[1];\n      quaternion2[2]=val[2];\n      quaternion2[3]=0;\n      quaternionmult(quaternion3,quaternion1,quaternion2);\n      quaternion4[0]=-quaternion1[0];\n      quaternion4[1]=-quaternion1[1];\n      quaternion4[2]=-quaternion1[2];\n      quaternion4[3]= quaternion1[3];\n      quaternionmult(quaternion2,quaternion3,quaternion4);\n      translation_out[0]=quaternion2[0];\n      translation_out[1]=quaternion2[1];\n      translation_out[2]=quaternion2[2];\n      }\n   ", "c++:\n   X3dScript_ROT_TRANS1 *self = (X3dScript_ROT_TRANS1 *)node;\n   static bool ROT_TRANS1_init = false;\n   if (!ROT_TRANS1_init) {\n       self->translation_out[0] = 0;\n       self->translation_out[1] = 0;\n       self->translation_out[2] = 0;\n       ROT_TRANS1_init = true;\n   }         \n\n// account quaternion multiplication a b\n   extern void quaternionMult(float *c, float *a, float *b);\n// make rotation to quaternion with norm 1 \n   extern void norm_quaternion_from_rotation(float *quad, float *rot); \n\n   self->rotation[0] = self->rotation_in[0];\n   self->rotation[1] = self->rotation_in[1];\n   self->rotation[2] = self->rotation_in[2];\n   self->rotation[3] = self->rotation_in[3];\n   norm_quaternion_from_rotation(self->quaternion1, self->rotation);\n   self->quaternion2[0] = self->translation_in[0];\n   self->quaternion2[1] = self->translation_in[1];\n   self->quaternion2[2] = self->translation_in[2];\n   self->quaternion2[3] = 0;\n   quaternionMult(self->quaternion3, self->quaternion2, self->quaternion1);\n   self->quaternion4[0] = -self->quaternion1[0];\n   self->quaternion4[1] = -self->quaternion1[1];\n   self->quaternion4[2] = -self->quaternion1[2];\n   self->quaternion4[3] = self->quaternion1[3];\n   quaternionMult(self->quaternion2, self->quaternion4, self->quaternion3);\n   self->translation_out[0] = self->quaternion2[0];\n   self->translation_out[1] = self->quaternion2[1];\n   self->translation_out[2] = self->quaternion2[2];\n" };
    ROTm_TRANS2.url = m_url;
    ROTm_TRANS2.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROTm_TRANS2.directOutput = false;
    ROTm_TRANS2.mustEvaluate = false;
    {
    static float m_rotation[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.rotation = m_rotation;
    ROTm_TRANS2.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_quaternion1[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion1 = m_quaternion1;
    ROTm_TRANS2.quaternion1_length = sizeof(m_quaternion1) / sizeof(float);
    }
    {
    static float m_quaternion2[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion2 = m_quaternion2;
    ROTm_TRANS2.quaternion2_length = sizeof(m_quaternion2) / sizeof(float);
    }
    {
    static float m_quaternion3[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion3 = m_quaternion3;
    ROTm_TRANS2.quaternion3_length = sizeof(m_quaternion3) / sizeof(float);
    }
    {
    static float m_quaternion4[] = { 0, 0, 0, 0 };
    ROTm_TRANS2.quaternion4 = m_quaternion4;
    ROTm_TRANS2.quaternion4_length = sizeof(m_quaternion4) / sizeof(float);
    }
    ROTm_TRANS2.leng = 1;
    ROTm_TRANS2.winkelkorrektur = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    ROTm_TRANS2.translation_in = m_translation_in;
    ROTm_TRANS2.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_rotation_in[] = { 0, 0, 1, 0 };
    ROTm_TRANS2.rotation_in = m_rotation_in;
    ROTm_TRANS2.rotation_in_length = sizeof(m_rotation_in) / sizeof(float);
    }
    {
    static const char* m_debug[1];
    ROTm_TRANS2.debug = NULL;
    ROTm_TRANS2.debug_length = 0;
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    ROTm_TRANS2.translation_out = m_translation_out;
    ROTm_TRANS2.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    Background_72.m_parent = (X3dNode *)&Group_71;
    {
    static float m_groundAngle[1];
    Background_72.groundAngle = NULL;
    Background_72.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background_72.groundColor = m_groundColor;
    Background_72.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background_72.backUrl = NULL;
    Background_72.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background_72.rightUrl = NULL;
    Background_72.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background_72.frontUrl = NULL;
    Background_72.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background_72.leftUrl = NULL;
    Background_72.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background_72.topUrl = NULL;
    Background_72.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background_72.bottomUrl = NULL;
    Background_72.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background_72.skyAngle = NULL;
    Background_72.skyAngle_length = 0;
    }
    Background_72.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background_72.skyColor = m_skyColor;
    Background_72.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background_72.set_bind = false;
    Background_72.isBound = false;
    WALLm_FIRE.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n\n // field SFBool old_isactive //\n\n // eventOut SFBool fire //\n\n      function initialize()            \n         {            \n         fire=FALSE;\n         }            \n// release button == fire     \n      function isactive_in(val)            \n         {            \n         if ((!val) && old_isactive) \n            fire=TRUE;\n         else \n            fire=FALSE;           \n         old_isactive=val;           \n         }            \n      ", "c++:\n      X3dScript_WALL_FIRE *self = (X3dScript_WALL_FIRE *)node;\n      static bool WALL_FIRE_init = false;\n      if (!WALL_FIRE_init) {\n           self->fire = false; \n           WALL_FIRE_init = true;\n      }         \n      if ((!self->isactive_in) && self->old_isactive)\n          self->fire = true;\n      else\n          self->fire = false;\n      self->old_isactive = self->isactive_in;           \n" };
    WALLm_FIRE.url = m_url;
    WALLm_FIRE.url_length = sizeof(m_url) / sizeof(const char*);
    }
    WALLm_FIRE.directOutput = false;
    WALLm_FIRE.mustEvaluate = false;
    WALLm_FIRE.old_isactive = false;
    WALLm_FIRE.isactive_in = false;
    WALLm_FIRE.fire = false;
    THING.route_sources =  NULL;
    THING.num_route_source = 0;
    THING.m_parent = (X3dNode *)&Group_71;
    ROLLm_AXIS.route_sources =  NULL;
    ROLLm_AXIS.num_route_source = 0;
    ROLLm_AXIS.m_parent = (X3dNode *)&THING;
    UPm_DOWNm_AXIS.route_sources =  NULL;
    UPm_DOWNm_AXIS.num_route_source = 0;
    UPm_DOWNm_AXIS.m_parent = (X3dNode *)&ROLLm_AXIS;
    THINGm_SENSOR.m_parent = (X3dNode *)&UPm_DOWNm_AXIS;
    THINGm_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    THINGm_SENSOR.hitNormal_changed = m_hitNormal_changed;
    THINGm_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    THINGm_SENSOR.hitPoint_changed = m_hitPoint_changed;
    THINGm_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    THINGm_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    THINGm_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    THINGm_SENSOR.isActive = false;
    THINGm_SENSOR.isOver = false;
    THINGm_SENSOR.touchTime = 0;
    HIDEm_THING.route_sources =  NULL;
    HIDEm_THING.num_route_source = 0;
    HIDEm_THING.m_parent = (X3dNode *)&UPm_DOWNm_AXIS;
    Inline_22.m_parent = (X3dNode *)&HIDEm_THING;
    Transform1.route_sources =  NULL;
    Transform1.num_route_source = 0;
    Transform1.m_parent = (X3dNode *)&Inline_22;
    Inline_10.m_parent = (X3dNode *)NULL;
    Group2.m_parent = (X3dNode *)&Inline_10;
    Transform_103.route_sources =  NULL;
    Transform_103.num_route_source = 0;
    Transform_103.m_parent = (X3dNode *)NULL;
    Shape_104.m_parent = (X3dNode *)NULL;
    Appearance_105.m_parent = (X3dNode *)NULL;
    A0m_1.m_parent = (X3dNode *)NULL;
    A0m_1.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_1.diffuseColor = m_diffuseColor;
    A0m_1.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_1.emissiveColor = m_emissiveColor;
    A0m_1.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_1.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_1.specularColor = m_specularColor;
    A0m_1.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_1.transparency = 0;
    Appearance_105.material = &A0m_1;
    Appearance_105.texture = NULL;
    Appearance_105.textureTransform = NULL;
    IndexedFaceSet_6.setGlName(2);
    IndexedFaceSet_6.m_parent = (X3dNode *)NULL;
    Color2_4.m_parent = (X3dNode *)&IndexedFaceSet_6;
    {
    static float m_color[] = { 1, 0.793614, 0.087319, 1, 0.638174, 0.077397, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.754465, 0.084820, 1, 0.754465, 0.084820, 1, 0.775771, 0.086180, 1, 0.756482, 0.084949, 1, 0.756482, 0.084949, 1, 0.777047, 0.086262 };
    Color2_4.color = m_color;
    Color2_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate3_9.m_parent = (X3dNode *)&IndexedFaceSet_6;
    {
    static float m_point[] = { 0.150260, 0.330511, 1.553120, 0.256526, 0.628258, 1.205410, 0.043506, 0.580147, 1.404830, 0.041913, 0.281121, 1.582420, -0.098818, 0.305979, 1.539120, -0.172704, 0.627974, 1.205410, -0.237476, 0.530979, 1.149410, -0.066433, 0.330228, 1.553120, 0.043506, 0.580146, 1.404830, 0.041913, 0.257624, 1.511120, 0.041913, 0.257624, 1.511120, 0.150260, 0.282014, 1.525120, 0.321306, 0.531263, 1.149410, 0.041913, 0.281121, 1.582420, -0.066433, 0.281730, 1.525120, 0.182646, 0.306263, 1.539120, 0.090713, 0.547383, 1.392800, 0.248053, 0.604517, 1.233140, 0.248850, 0.594803, 1.240080, 0.054951, 0.553382, 1.420730, 0.026772, 0.542105, 1.427400, -0.006386, 0.549079, 1.391450, -0.160300, 0.593221, 1.246000, -0.165083, 0.594759, 1.239830, 0.231697, 0.398694, 1.194280, 0.202474, 0.389779, 1.198610, 0.041913, 0.328005, 1.091790, -0.121436, 0.391855, 1.193190, 0.041913, 0.328657, 1.087900, -0.146267, 0.396108, 1.200800 };
    Coordinate3_9.point = m_point;
    Coordinate3_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal3_5.m_parent = (X3dNode *)&IndexedFaceSet_6;
    {
    static float m_vector[] = { 0.240520, -0.178855, 0.954024, 0.241302, -0.186810, 0.952300, 0.240520, -0.178855, 0.954024, 0.240520, -0.178855, 0.954024, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.293132, 0.577824, 0.761704, 0.293132, 0.577824, 0.761704, -0.000257, 0.510630, 0.859801, -0.540597, 0.550255, 0.636376, -0.540597, 0.550255, 0.636376, -0.627971, 0.485889, 0.607918, -0.741730, 0.393026, 0.543477, -0.741730, 0.393026, 0.543477, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, 0.273372, -0.961689, -0.020544, 0.273372, -0.961689, -0.020544, 0.274033, -0.961490, -0.021035, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, -0.237959, -0.155430, 0.958758, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.001166, 0.510636, 0.859796, -0.278860, 0.577795, 0.767066, -0.278860, 0.577795, 0.767066, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.871407, -0.488674, 0.042975, -0.871407, -0.488674, 0.042975, -0.867993, -0.495110, 0.038144, -0.168427, -0.936187, 0.308521, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.947812, -0.246434, 0.202292, 0.949402, -0.233910, 0.209575, 0.949402, -0.233910, 0.209575, 0.947812, -0.246434, 0.202292, 0.539886, 0.550710, 0.636586, 0.539886, 0.550710, 0.636586, 0.765835, 0.369611, 0.526198, 0.765835, 0.369611, 0.526198, 0.765820, 0.369627, 0.526210, 0.741377, 0.393368, 0.543712, 0.741377, 0.393368, 0.543712, 0.502593, 0.598896, 0.623477, 0.502593, 0.598896, 0.623477, -0.489526, 0.601019, 0.631775, -0.489526, 0.601019, 0.631775, 0.578916, -0.803148, -0.140751, 0.578916, -0.803148, -0.140751, 0.344423, -0.915028, -0.209994, 0.344423, -0.915028, -0.209994, 0.439693, -0.885759, -0.148667, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, -0.438743, -0.886217, -0.148742, -0.829634, -0.558259, 0.007340, -0.242173, -0.932158, -0.269134 };
    Normal3_5.vector = m_vector;
    Normal3_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_6.color = &Color2_4;
    IndexedFaceSet_6.coord = &Coordinate3_9;
    IndexedFaceSet_6.normal = &Normal3_5;
    IndexedFaceSet_6.texCoord = NULL;
    IndexedFaceSet_6.ccw = true;
    {
    static int m_colorIndex[] = { 1, 0, 11, 0, -1, 16, 0, 19, 0, 0, -1, 1, 1, 0, -1, 1, 0, 13, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 1, 0, 1, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 7, 0, 0, 9, 2, -1, 5, 0, 6, -1, 0, 0, 8, -1, 0, 3, 8, -1, 10, 0, 4, -1, 15, 0, 12, -1, 0, 14, 17, -1, 0, 18, 0, -1, 0, 23, 0, -1, 22, 0, 1, 25, -1, 1, 26, 24, -1, 1, 27, 29, -1, 0, 0, 21, -1, 28, 1, 0, 20, -1 };
    IndexedFaceSet_6.colorIndex = m_colorIndex;
    IndexedFaceSet_6.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_6.colorPerVertex = true;
    IndexedFaceSet_6.convex = true;
    {
    static int m_coordIndex[] = { 3, 0, 19, 2, -1, 22, 5, 23, 4, 7, -1, 3, 10, 11, -1, 13, 8, 20, 7, -1, 13, 7, 4, -1, 13, 4, 14, -1, 6, 14, 4, -1, 13, 14, 9, -1, 3, 15, 0, -1, 3, 11, 15, -1, 12, 15, 11, -1, 17, 0, 15, 18, 1, -1, 16, 0, 17, -1, 15, 12, 18, -1, 12, 1, 18, -1, 19, 0, 16, -1, 21, 7, 20, -1, 7, 21, 22, -1, 4, 23, 6, -1, 11, 24, 12, -1, 24, 11, 10, 25, -1, 10, 26, 25, -1, 9, 27, 28, -1, 14, 6, 29, -1, 27, 9, 14, 29, -1 };
    IndexedFaceSet_6.coordIndex = m_coordIndex;
    IndexedFaceSet_6.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_6.creaseAngle = 2.942970;
    {
    static int m_normalIndex[] = { 0, 4, 9, 11, -1, 12, 14, 15, 17, 22, -1, 1, 27, 30, -1, 35, 39, 40, 23, -1, 36, 24, 18, -1, 37, 19, 42, -1, 47, 43, 20, -1, 38, 44, 50, -1, 2, 53, 5, -1, 3, 31, 54, -1, 58, 55, 32, -1, 62, 6, 56, 64, 67, -1, 69, 7, 63, -1, 57, 59, 65, -1, 60, 68, 66, -1, 10, 8, 70, -1, 71, 25, 41, -1, 26, 72, 13, -1, 21, 16, 48, -1, 33, 73, 61, -1, 74, 34, 28, 75, -1, 29, 77, 76, -1, 51, 78, 80, -1, 45, 49, 81, -1, 79, 52, 46, 82, -1 };
    IndexedFaceSet_6.normalIndex = m_normalIndex;
    IndexedFaceSet_6.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_6.normalPerVertex = true;
    IndexedFaceSet_6.solid = true;
    {
    static int m_texCoordIndex[1];
    IndexedFaceSet_6.texCoordIndex = NULL;
    IndexedFaceSet_6.texCoordIndex_length = 0;
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_6.set_colorIndex = NULL;
    IndexedFaceSet_6.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_6.set_coordIndex = NULL;
    IndexedFaceSet_6.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_6.set_normalIndex = NULL;
    IndexedFaceSet_6.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_6.set_texCoordIndex = NULL;
    IndexedFaceSet_6.set_texCoordIndex_length = 0;
    }
    Shape_104.appearance = &Appearance_105;
    Shape_104.geometry = &IndexedFaceSet_6;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_103.rotation = m_rotation;
    Transform_103.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -2.980230e-8, 5.005030e-3, -3.011850e-2 };
    Transform_103.translation = m_translation;
    Transform_103.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_103.bboxCenter = m_bboxCenter;
    Transform_103.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_103.bboxSize = m_bboxSize;
    Transform_103.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_103.center = m_center;
    Transform_103.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_103.scale = m_scale;
    Transform_103.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_103.scaleOrientation = m_scaleOrientation;
    Transform_103.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_104 };
    Transform_103.children = m_children;
    Transform_103.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_103.addChildren = NULL;
    Transform_103.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_103.removeChildren = NULL;
    Transform_103.removeChildren_length = 0;
    }
    Transform_109.route_sources =  NULL;
    Transform_109.num_route_source = 0;
    Transform_109.m_parent = (X3dNode *)NULL;
    Shape_110.m_parent = (X3dNode *)NULL;
    Appearance_111.m_parent = (X3dNode *)NULL;
    A0m_2.m_parent = (X3dNode *)NULL;
    A0m_2.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_2.diffuseColor = m_diffuseColor;
    A0m_2.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_2.emissiveColor = m_emissiveColor;
    A0m_2.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_2.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_2.specularColor = m_specularColor;
    A0m_2.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_2.transparency = 0;
    Appearance_111.material = &A0m_2;
    Appearance_111.texture = NULL;
    Appearance_111.textureTransform = NULL;
    IndexedFaceSet_7.setGlName(3);
    IndexedFaceSet_7.m_parent = (X3dNode *)NULL;
    Color3_4.m_parent = (X3dNode *)&IndexedFaceSet_7;
    {
    static float m_color[] = { 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794 };
    Color3_4.color = m_color;
    Color3_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate4_9.m_parent = (X3dNode *)&IndexedFaceSet_7;
    {
    static float m_point[] = { 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180, 0.041613, 0.179757, -1.951840, 0.275296, 0.046906, -1.173840, 0.607257, 0.161629, -1.835830, 0.041613, 0.179757, -1.951840, -0.524042, 0.158129, -1.836490, -0.192141, 0.045295, -1.174140, 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180 };
    Coordinate4_9.point = m_point;
    Coordinate4_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal4_5.m_parent = (X3dNode *)&IndexedFaceSet_7;
    {
    static float m_vector[] = { -0.958763, 0.012792, 0.283919, -0.181027, 0.455152, 0.871817, -0.110099, 0.301903, 0.946960, -0.110099, 0.301903, 0.946960, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.698330, -0.710853, 0.083806, -0.181027, 0.455152, 0.871817, -0.698330, -0.710853, 0.083806, 0.875466, 0.215616, 0.432514, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, 0.003126, -0.985533, -0.169456, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, 0.003126, -0.985533, -0.169456, -0.896416, 0.017432, 0.442870, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112 };
    Normal4_5.vector = m_vector;
    Normal4_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_7.color = &Color3_4;
    IndexedFaceSet_7.coord = &Coordinate4_9;
    IndexedFaceSet_7.normal = &Normal4_5;
    IndexedFaceSet_7.texCoord = NULL;
    IndexedFaceSet_7.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 1, 1, 0, -1, 0, 1, 1, -1, 2, 3, 2, -1, 5, 4, 4, -1 };
    IndexedFaceSet_7.colorIndex = m_colorIndex;
    IndexedFaceSet_7.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_7.colorPerVertex = true;
    IndexedFaceSet_7.convex = true;
    {
    static int m_coordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet_7.coordIndex = m_coordIndex;
    IndexedFaceSet_7.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_7.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 2, 4, 7, -1, 2, 7, 0, -1, 3, 10, 12, -1, 3, 12, 5, -1, 11, 13, 8, -1, 11, 8, 1, -1, 17, 19, 22, -1, 17, 22, 15, -1, 25, 27, 20, -1, 25, 20, 18, -1, 26, 28, 23, -1, 26, 23, 16, -1, 14, 9, 6, -1, 29, 24, 21, -1, 30, 31, 32, -1, 33, 34, 35, -1 };
    IndexedFaceSet_7.normalIndex = m_normalIndex;
    IndexedFaceSet_7.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_7.normalPerVertex = true;
    IndexedFaceSet_7.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet_7.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_7.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_7.set_colorIndex = NULL;
    IndexedFaceSet_7.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_7.set_coordIndex = NULL;
    IndexedFaceSet_7.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_7.set_normalIndex = NULL;
    IndexedFaceSet_7.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_7.set_texCoordIndex = NULL;
    IndexedFaceSet_7.set_texCoordIndex_length = 0;
    }
    Shape_110.appearance = &Appearance_111;
    Shape_110.geometry = &IndexedFaceSet_7;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_109.rotation = m_rotation;
    Transform_109.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0.179843, -9.580490e-2 };
    Transform_109.translation = m_translation;
    Transform_109.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_109.bboxCenter = m_bboxCenter;
    Transform_109.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_109.bboxSize = m_bboxSize;
    Transform_109.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_109.center = m_center;
    Transform_109.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_109.scale = m_scale;
    Transform_109.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_109.scaleOrientation = m_scaleOrientation;
    Transform_109.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_110 };
    Transform_109.children = m_children;
    Transform_109.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_109.addChildren = NULL;
    Transform_109.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_109.removeChildren = NULL;
    Transform_109.removeChildren_length = 0;
    }
    Transform_115.route_sources =  NULL;
    Transform_115.num_route_source = 0;
    Transform_115.m_parent = (X3dNode *)NULL;
    Shape_116.m_parent = (X3dNode *)NULL;
    Appearance_117.m_parent = (X3dNode *)NULL;
    Material_118.m_parent = (X3dNode *)NULL;
    Material_118.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_118.diffuseColor = m_diffuseColor;
    Material_118.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_118.emissiveColor = m_emissiveColor;
    Material_118.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_118.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_118.specularColor = m_specularColor;
    Material_118.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_118.transparency = 6.382980e-2;
    Appearance_117.material = &Material_118;
    Appearance_117.texture = NULL;
    Appearance_117.textureTransform = NULL;
    IndexedFaceSet_8.setGlName(4);
    IndexedFaceSet_8.m_parent = (X3dNode *)NULL;
    Coordinate5_9.m_parent = (X3dNode *)&IndexedFaceSet_8;
    {
    static float m_point[] = { 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.357975, 0.274623, 0.391110, 0.435755, 0.282992, -1.405890, 0.732023, 0.277202, -1.011950, 0.803608, 0.283498, -0.488598, 0.807795, 0.291706, -0.169135, 0.642283, 0.308596, 0.135106, -0.576839, 0.295092, 0.138182, -0.741419, 0.274624, -0.165225, -0.736057, 0.266519, -0.484712, -0.662406, 0.261798, -1.008430, -0.364349, 0.274033, -1.403870, 0.034173, 1.521740, 0.072271, 0.032379, 1.258290, 0.402776, 0.324052, 1.234020, 0.434518, 0.382959, 1.442650, 0.114314, 0.462972, 1.076210, 0.512842, 0.634192, 1.091710, 0.196483, 0.036047, 1.641430, -0.319814, 0.468123, 1.540110, -0.266019, 0.780864, 1.161780, -0.151701, 0.753362, 0.779925, 0.302416, 0.875071, 0.752977, 0.006324, 0.646152, 0.462049, 0.391531, 0.036992, 1.662050, -0.688097, 0.522545, 1.549060, -0.628109, 0.873437, 1.148710, -0.497152, 0.978519, 0.692729, -0.320894, 0.036779, 1.562450, -1.012960, 0.537744, 1.451000, -0.953720, 0.879328, 1.083370, -0.818803, 0.983199, 0.632646, -0.644566, 0.672590, 0.889258, -1.304530, 0.877021, 0.531470, -1.119920, 0.439314, 0.644779, -1.532240, 0.477913, 0.461302, -1.461330, 0.028006, 0.456359, -1.522590, 0.025463, 0.201297, -1.389500, -0.259292, 1.233740, 0.434518, -0.316266, 1.434790, 0.116078, -0.398215, 1.075930, 0.512842, -0.569430, 1.091430, 0.196483, -0.398142, 1.530440, -0.263833, -0.718144, 1.145240, -0.147918, -0.688602, 0.779642, 0.302416, -0.820289, 0.734310, 0.010603, -0.581395, 0.461765, 0.391532, -0.450917, 1.538220, -0.625652, -0.809539, 1.130170, -0.492904, -0.923472, 0.671821, -0.316093, -0.466519, 1.439820, -0.951186, -0.815262, 1.064720, -0.814525, -0.927883, 0.611639, -0.639742, -0.391565, 1.162260, -1.429880, -0.821517, 0.512768, -1.115630, -0.422165, 0.451259, -1.459060, 0.032379, 1.315590, 1.240890, 0.228842, 1.352260, 0.975027, 0.032379, 1.384890, 0.907439, 0.314142, 1.285900, 1.111910, 0.314142, 1.207010, 1.274610, 0.228842, 1.140650, 1.411480, 0.032379, 1.107740, 1.479070, 0.032379, 1.284680, 0.661614, 0.313972, 1.236840, 0.760583, 0.438872, 1.139660, 0.961005, 0.438872, 0.959292, 1.192490, 0.313972, 0.916734, 1.377380, 0.032379, 0.878853, 1.498640, 0.457846, 0.598657, 1.016000, 0.325246, 0.449644, 1.073590, -0.164085, 1.351980, 0.975027, -0.249383, 1.285620, 1.111910, -0.249383, 1.206730, 1.274610, -0.164085, 1.140370, 1.411480, -0.249218, 1.236560, 0.760583, -0.374117, 1.139380, 0.961005, -0.374117, 0.959003, 1.192490, -0.249218, 0.916450, 1.377380, -0.412717, 0.598373, 1.016000, -0.277239, 0.449360, 1.073590, 0.030998, 0.455945, -1.522540, 0.200163, 0.782040, -2.021660, 0.022803, 0.816558, -2.021660, -0.153194, 0.791256, -2.021660, 0.030997, 0.455945, -1.522540, -0.638224, 0.898115, -1.302900, -0.394524, 0.650450, -1.531170, 0.012304, 1.278610, -1.485710, 0.022803, 0.816558, -2.021660, 0.012304, 1.278610, -1.485710, 0.452747, 1.152960, -1.430970 };
    Coordinate5_9.point = m_point;
    Coordinate5_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal5_5.m_parent = (X3dNode *)&IndexedFaceSet_8;
    {
    static float m_vector[] = { -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940694, 0.149539, 0.304520, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.834986, -0.422306, 0.352783, 0.834986, -0.422306, 0.352783, 0.837532, -0.417196, 0.352827, 0.834986, -0.422306, 0.352783, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824726, 0.564327, 0.036905, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983866, 0.093656, 0.152433, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.919028, -0.327107, 0.219973, 0.919028, -0.327107, 0.219973, 0.919030, -0.327639, 0.219171, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857347, 0.447616, -0.254158, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.871918, -0.485933, -0.060240, 0.871918, -0.485933, -0.060240, 0.871819, -0.486088, -0.060419, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.748933, -0.523293, -0.406527, 0.748933, -0.523293, -0.406527, 0.748998, -0.522697, -0.407174, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.687379, -0.282285, -0.669197, 0.682617, -0.274976, -0.677069, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.373882, -0.393399, -0.839911, 0.373882, -0.393399, -0.839911, 0.372714, -0.393753, -0.840264, 0.043708, -0.372748, -0.926903, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.001617, -0.462617, -0.886557, 0.001617, -0.462617, -0.886557, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936815, 0.032489, 0.348312, -0.936814, 0.032493, 0.348315, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.753712, -0.525849, 0.394210, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.858943, 0.498866, 0.115538, -0.859015, 0.498704, 0.115703, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.909057, -0.352626, 0.221969, -0.909016, -0.354059, 0.219844, -0.909016, -0.354059, 0.219844, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.972656, 0.230360, -0.029579, -0.917591, -0.388391, 0.084727, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.926292, 0.088999, -0.366146, -0.914069, 0.304729, -0.267615, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.914069, 0.304729, -0.267615, -0.657343, -0.174205, -0.733180, -0.694040, -0.019618, -0.719669, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.053473, -0.461524, -0.885515, -0.465415, -0.809935, 0.356924, -0.465415, -0.809935, 0.356924, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.301321, -0.764284, 0.570154, 0.512027, -0.788035, 0.341804, 0.512027, -0.788035, 0.341804, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, 0.115620, -0.804235, -0.582956, -0.370219, -0.828569, -0.420014, 0.115620, -0.804235, -0.582956, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.117789, 0.819323, -0.561103, -0.725452, 0.412604, -0.550888, -0.112111, 0.280660, -0.953237, -0.115216, 0.288185, -0.950618, -0.115216, 0.288185, -0.950618, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.694040, -0.019618, -0.719669, 0.113269, -0.848919, -0.516243, 0.113269, -0.848919, -0.516243, 0.109595, 0.280897, -0.953460, 0.110751, 0.283846, -0.952452, 0.109595, 0.280897, -0.953460, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.285260, 0.728679, -0.622619, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.509719, 0.651105, -0.562360, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.046990, 0.842110, -0.537256 };
    Normal5_5.vector = m_vector;
    Normal5_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_8.color = NULL;
    IndexedFaceSet_8.coord = &Coordinate5_9;
    IndexedFaceSet_8.normal = &Normal5_5;
    IndexedFaceSet_8.texCoord = NULL;
    IndexedFaceSet_8.ccw = true;
    {
    static int m_colorIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_8.colorIndex = m_colorIndex;
    IndexedFaceSet_8.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_8.colorPerVertex = true;
    IndexedFaceSet_8.convex = true;
    {
    static int m_coordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_8.coordIndex = m_coordIndex;
    IndexedFaceSet_8.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_8.creaseAngle = 3.104420;
    {
    static int m_normalIndex[] = { 0, 6, 12, -1, 13, 18, 1, -1, 19, 14, 24, -1, 25, 30, 20, -1, 36, 2, 21, -1, 22, 42, 37, -1, 43, 23, 31, -1, 32, 48, 44, -1, 49, 33, 54, -1, 55, 60, 50, -1, 61, 56, 66, -1, 67, 72, 62, -1, 76, 38, 45, -1, 46, 82, 77, -1, 83, 47, 51, -1, 52, 88, 84, -1, 89, 53, 63, -1, 64, 94, 90, -1, 95, 65, 73, -1, 74, 100, 96, -1, 103, 78, 85, -1, 86, 109, 104, -1, 110, 87, 91, -1, 92, 115, 111, -1, 116, 93, 97, -1, 98, 121, 117, -1, 122, 99, 101, -1, 102, 127, 123, -1, 130, 118, 124, -1, 125, 136, 131, -1, 137, 126, 128, -1, 129, 142, 138, -1, 145, 132, 139, -1, 140, 150, 146, -1, 151, 141, 143, -1, 144, 155, 152, -1, 158, 147, 153, -1, 159, 154, 156, -1, 160, 157, 164, -1, 3, 166, 7, -1, 167, 4, 172, -1, 173, 178, 168, -1, 179, 174, 184, -1, 39, 175, 5, -1, 176, 40, 190, -1, 191, 185, 177, -1, 186, 192, 196, -1, 197, 202, 187, -1, 203, 198, 208, -1, 209, 214, 204, -1, 215, 210, 220, -1, 79, 193, 41, -1, 194, 80, 224, -1, 225, 199, 195, -1, 200, 226, 230, -1, 231, 211, 201, -1, 212, 232, 236, -1, 237, 221, 213, -1, 222, 238, 242, -1, 105, 227, 81, -1, 228, 106, 245, -1, 246, 233, 229, -1, 234, 247, 251, -1, 252, 239, 235, -1, 240, 253, 257, -1, 258, 243, 241, -1, 244, 259, 263, -1, 266, 254, 248, -1, 272, 264, 260, -1, 265, 273, 278, -1, 281, 279, 274, -1, 280, 282, 286, -1, 161, 287, 283, -1, 162, 165, 288, -1, 223, 289, 216, -1, 291, 217, 290, -1, 75, 68, 297, -1, 299, 309, 314, -1, 300, 318, 310, -1, 301, 323, 319, -1, 302, 328, 324, -1, 303, 333, 329, -1, 339, 315, 311, -1, 312, 345, 340, -1, 346, 313, 320, -1, 321, 351, 347, -1, 352, 322, 325, -1, 326, 357, 353, -1, 358, 327, 330, -1, 331, 363, 359, -1, 364, 332, 334, -1, 335, 369, 365, -1, 8, 341, 348, -1, 349, 15, 9, -1, 16, 350, 354, -1, 355, 26, 17, -1, 27, 356, 360, -1, 361, 375, 28, -1, 376, 362, 366, -1, 367, 381, 377, -1, 382, 368, 370, -1, 371, 292, 383, -1, 34, 29, 378, -1, 379, 57, 35, -1, 58, 380, 384, -1, 385, 69, 59, -1, 70, 386, 293, -1, 304, 316, 387, -1, 305, 388, 392, -1, 306, 393, 397, -1, 307, 398, 402, -1, 308, 403, 336, -1, 342, 389, 317, -1, 390, 343, 407, -1, 408, 394, 391, -1, 395, 409, 413, -1, 414, 399, 396, -1, 400, 415, 419, -1, 420, 404, 401, -1, 405, 421, 425, -1, 426, 337, 406, -1, 338, 427, 372, -1, 10, 410, 344, -1, 411, 11, 169, -1, 170, 416, 412, -1, 417, 171, 180, -1, 181, 422, 418, -1, 423, 182, 431, -1, 432, 428, 424, -1, 429, 433, 437, -1, 438, 373, 430, -1, 374, 439, 294, -1, 188, 434, 183, -1, 435, 189, 205, -1, 206, 440, 436, -1, 441, 207, 218, -1, 219, 295, 442, -1, 71, 296, 298, -1, 443, 445, 450, -1, 453, 446, 459, -1, 444, 460, 447, -1, 464, 466, 469, -1, 467, 474, 475, -1, 476, 470, 468, -1, 112, 119, 477, -1, 120, 133, 478, -1, 471, 479, 134, -1, 255, 267, 454, -1, 261, 256, 455, -1, 275, 262, 456, -1, 107, 113, 481, -1, 249, 108, 482, -1, 268, 250, 483, -1, 114, 480, 484, -1, 451, 448, 269, -1, 449, 457, 270, -1, 485, 452, 271, -1, 284, 276, 461, -1, 163, 285, 462, -1, 277, 458, 463, -1, 465, 472, 148, -1, 473, 135, 149, -1 };
    IndexedFaceSet_8.normalIndex = m_normalIndex;
    IndexedFaceSet_8.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_8.normalPerVertex = true;
    IndexedFaceSet_8.solid = true;
    {
    static int m_texCoordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_8.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_8.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_8.set_colorIndex = NULL;
    IndexedFaceSet_8.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_8.set_coordIndex = NULL;
    IndexedFaceSet_8.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_8.set_normalIndex = NULL;
    IndexedFaceSet_8.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_8.set_texCoordIndex = NULL;
    IndexedFaceSet_8.set_texCoordIndex_length = 0;
    }
    Shape_116.appearance = &Appearance_117;
    Shape_116.geometry = &IndexedFaceSet_8;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_115.rotation = m_rotation;
    Transform_115.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform_115.translation = m_translation;
    Transform_115.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_115.bboxCenter = m_bboxCenter;
    Transform_115.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_115.bboxSize = m_bboxSize;
    Transform_115.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_115.center = m_center;
    Transform_115.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_115.scale = m_scale;
    Transform_115.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_115.scaleOrientation = m_scaleOrientation;
    Transform_115.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_116 };
    Transform_115.children = m_children;
    Transform_115.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_115.addChildren = NULL;
    Transform_115.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_115.removeChildren = NULL;
    Transform_115.removeChildren_length = 0;
    }
    Transform_121.route_sources =  NULL;
    Transform_121.num_route_source = 0;
    Transform_121.m_parent = (X3dNode *)NULL;
    Shape_122.m_parent = (X3dNode *)NULL;
    Appearance_123.m_parent = (X3dNode *)NULL;
    Material_124.m_parent = (X3dNode *)NULL;
    Material_124.ambientIntensity = 1;
    {
    static float m_diffuseColor[] = { 1, 1, 1 };
    Material_124.diffuseColor = m_diffuseColor;
    Material_124.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0.691489, 0.691489, 0.691489 };
    Material_124.emissiveColor = m_emissiveColor;
    Material_124.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_124.shininess = 1;
    {
    static float m_specularColor[] = { 0.765957, 0.765957, 0.765957 };
    Material_124.specularColor = m_specularColor;
    Material_124.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_124.transparency = 0.372340;
    Appearance_123.material = &Material_124;
    Appearance_123.texture = NULL;
    Appearance_123.textureTransform = NULL;
    IndexedFaceSet_9.setGlName(5);
    IndexedFaceSet_9.m_parent = (X3dNode *)NULL;
    Coordinate6_9.m_parent = (X3dNode *)&IndexedFaceSet_9;
    {
    static float m_point[] = { 0.023775, 0.125771, 0.096370, -0.736057, 0.266519, -0.484712, 0.019864, 0.155999, -0.238411, -0.741419, 0.274624, -0.165225, 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.033239, 0.290582, 0.404469, -0.576839, 0.295092, 0.138182, 0.357975, 0.274623, 0.391110, 0.642283, 0.308596, 0.135106, 0.026537, 0.145128, 0.290011, 0.807795, 0.291706, -0.169135, 0.803608, 0.283498, -0.488598, -0.662406, 0.261798, -1.008430, 0.019557, 0.169360, -0.585595, -0.364349, 0.274033, -1.403870, 0.034997, 0.201297, -1.394460, 0.732023, 0.277202, -1.011950, 0.435755, 0.282992, -1.405890 };
    Coordinate6_9.point = m_point;
    Coordinate6_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal6_5.m_parent = (X3dNode *)&IndexedFaceSet_9;
    {
    static float m_vector[] = { -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.115983, -0.989347, -0.087975, -0.197996, -0.979966, 0.021537, -0.141172, -0.989925, -0.010929, -0.115983, -0.989347, -0.087975, 0.131644, -0.987142, -0.090669, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, 0.127132, -0.991148, -0.038256, 0.157553, -0.987463, -0.009671, -0.228866, -0.968557, 0.097558, -0.229441, -0.968303, 0.098722, -0.229441, -0.968303, 0.098722, 0.003121, -0.997858, 0.065337, 0.003121, -0.997858, 0.065337, 0.057747, -0.997198, -0.047547, 0.057747, -0.997198, -0.047547, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, -0.193939, -0.928966, 0.315294, -0.193939, -0.928966, 0.315294, -0.194333, -0.929016, 0.314902, -0.046282, -0.996765, 0.065709, -0.056059, -0.979739, -0.192273, 0.204185, -0.934007, 0.293153, 0.205250, -0.934031, 0.292331, 0.205250, -0.934031, 0.292331, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.256610, -0.958616, 0.123319, 0.257004, -0.958385, 0.124286, 0.256610, -0.958616, 0.123319, 0.169604, -0.985103, -0.028410, 0.169720, -0.985078, -0.028591, 0.169604, -0.985103, -0.028410, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, -0.178067, -0.983112, -0.042216, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, 0.127132, -0.991148, -0.038256, 0.198666, -0.979446, -0.034880, 0.097470, -0.991352, -0.087874, -0.178067, -0.983112, -0.042216, -0.078733, -0.992819, -0.090062, 0.010487, -0.999174, -0.039251, 0.010487, -0.999174, -0.039251, 0.131753, -0.990507, -0.039208, 0.132081, -0.990482, -0.038739, 0.131753, -0.990507, -0.039208, 0.148313, -0.987028, -0.061478, 0.148313, -0.987028, -0.061478 };
    Normal6_5.vector = m_vector;
    Normal6_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_9.color = NULL;
    IndexedFaceSet_9.coord = &Coordinate6_9;
    IndexedFaceSet_9.normal = &Normal6_5;
    IndexedFaceSet_9.texCoord = NULL;
    IndexedFaceSet_9.ccw = true;
    {
    static int m_colorIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_9.colorIndex = m_colorIndex;
    IndexedFaceSet_9.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_9.colorPerVertex = true;
    IndexedFaceSet_9.convex = true;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_9.coordIndex = m_coordIndex;
    IndexedFaceSet_9.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_9.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 0, 6, 9, -1, 7, 1, 15, -1, 18, 20, 22, -1, 28, 23, 21, -1, 19, 24, 31, -1, 33, 32, 25, -1, 36, 16, 2, -1, 26, 37, 34, -1, 27, 29, 38, -1, 17, 39, 30, -1, 40, 3, 42, -1, 43, 35, 41, -1, 4, 10, 45, -1, 46, 44, 5, -1, 48, 11, 8, -1, 51, 57, 59, -1, 12, 49, 52, -1, 58, 53, 50, -1, 13, 54, 61, -1, 62, 47, 14, -1, 55, 60, 64, -1, 65, 63, 56, -1 };
    IndexedFaceSet_9.normalIndex = m_normalIndex;
    IndexedFaceSet_9.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_9.normalPerVertex = true;
    IndexedFaceSet_9.solid = true;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_9.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_9.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_9.set_colorIndex = NULL;
    IndexedFaceSet_9.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_9.set_coordIndex = NULL;
    IndexedFaceSet_9.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_9.set_normalIndex = NULL;
    IndexedFaceSet_9.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_9.set_texCoordIndex = NULL;
    IndexedFaceSet_9.set_texCoordIndex_length = 0;
    }
    Shape_122.appearance = &Appearance_123;
    Shape_122.geometry = &IndexedFaceSet_9;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_121.rotation = m_rotation;
    Transform_121.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform_121.translation = m_translation;
    Transform_121.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_121.bboxCenter = m_bboxCenter;
    Transform_121.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_121.bboxSize = m_bboxSize;
    Transform_121.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_121.center = m_center;
    Transform_121.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_121.scale = m_scale;
    Transform_121.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_121.scaleOrientation = m_scaleOrientation;
    Transform_121.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_122 };
    Transform_121.children = m_children;
    Transform_121.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_121.addChildren = NULL;
    Transform_121.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_121.removeChildren = NULL;
    Transform_121.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_103, &Transform_109, &Transform_115, &Transform_121 };
    Group2.children = m_children;
    Group2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group2.bboxCenter = m_bboxCenter;
    Group2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group2.bboxSize = m_bboxSize;
    Group2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group2.addChildren = NULL;
    Group2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group2.removeChildren = NULL;
    Group2.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group2 };
    Inline_10.children = m_children;
    Inline_10.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_10.bboxCenter = m_bboxCenter;
    Inline_10.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_10.bboxSize = m_bboxSize;
    Inline_10.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_10.addChildren = NULL;
    Inline_10.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_10.removeChildren = NULL;
    Inline_10.removeChildren_length = 0;
    }
    Transform_127.route_sources =  NULL;
    Transform_127.num_route_source = 0;
    Transform_127.m_parent = (X3dNode *)NULL;
    Transform_128.route_sources =  NULL;
    Transform_128.num_route_source = 0;
    Transform_128.m_parent = (X3dNode *)NULL;
    Transform_129.route_sources =  NULL;
    Transform_129.num_route_source = 0;
    Transform_129.m_parent = (X3dNode *)NULL;
    Shape_130.m_parent = (X3dNode *)NULL;
    Appearance_131.m_parent = (X3dNode *)NULL;
    Material_132.m_parent = (X3dNode *)NULL;
    Material_132.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_132.diffuseColor = m_diffuseColor;
    Material_132.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_132.emissiveColor = m_emissiveColor;
    Material_132.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_132.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_132.specularColor = m_specularColor;
    Material_132.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_132.transparency = 0;
    Appearance_131.material = &Material_132;
    Appearance_131.texture = NULL;
    Appearance_131.textureTransform = NULL;
    IndexedFaceSet_11.setGlName(6);
    IndexedFaceSet_11.m_parent = (X3dNode *)NULL;
    Coordinate7_9.m_parent = (X3dNode *)&IndexedFaceSet_11;
    {
    static float m_point[] = { 0, 0, -1, -0.382683, 0, -0.923880, -0.707107, 0, -0.707107, -0.923880, 0, -0.382683, -1, 0, 0, -0.923880, 0, 0.382684, -0.707107, 0, 0.707107, -0.382683, 0, 0.923880, 0, 0, 1, 0.382683, 0, 0.923880, 0.707107, 0, 0.707107, 0.923880, 0, 0.382683, 1, 0, 0, 0.923879, 0, -0.382684, 0.707107, 0, -0.707107, 0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, -0.363954, 0.309017, -0.878662, -0.672499, 0.309017, -0.672499, -0.878662, 0.309017, -0.363954, -0.951057, 0.309017, 0, -0.878662, 0.309017, 0.363954, -0.672499, 0.309017, 0.672499, -0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, 0.363954, 0.309017, 0.878662, 0.672499, 0.309017, 0.672498, 0.878662, 0.309017, 0.363953, 0.951057, 0.309017, 0, 0.878662, 0.309017, -0.363954, 0.672498, 0.309017, -0.672499, 0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, -0.309597, 0.587785, -0.747434, -0.572061, 0.587785, -0.572061, -0.747434, 0.587785, -0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, 0.309597, -0.572061, 0.587785, 0.572061, -0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, 0.309597, 0.587785, 0.747434, 0.572061, 0.587785, 0.572061, 0.747434, 0.587785, 0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, -0.309598, 0.572061, 0.587785, -0.572062, 0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, -0.224936, 0.809017, -0.543043, -0.415627, 0.809017, -0.415627, -0.543043, 0.809017, -0.224936, -0.587785, 0.809017, 0, -0.543043, 0.809017, 0.224936, -0.415627, 0.809017, 0.415627, -0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, 0.224936, 0.809017, 0.543043, 0.415627, 0.809017, 0.415627, 0.543043, 0.809017, 0.224935, 0.587785, 0.809017, 0, 0.543043, 0.809017, -0.224936, 0.415627, 0.809017, -0.415627, 0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, -0.118256, 0.951057, -0.285494, -0.218508, 0.951057, -0.218508, -0.285494, 0.951057, -0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, 0.118256, -0.218508, 0.951057, 0.218508, -0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, 0.118256, 0.951057, 0.285494, 0.218508, 0.951057, 0.218508, 0.285495, 0.951057, 0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, -0.118256, 0.218508, 0.951057, -0.218508, 0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate7_9.point = m_point;
    Coordinate7_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal7_5.m_parent = (X3dNode *)&IndexedFaceSet_11;
    {
    static float m_vector[] = { -0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, -0.185864, 0.303889, -0.934401, -0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377972, 0.156433, -0.912505, -0.377972, 0.156433, -0.912505, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.698401, 0.156433, -0.698401, -0.698401, 0.156433, -0.698401, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.912505, 0.156434, -0.377972, -0.912505, 0.156434, -0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987689, 0.156434, 0, -0.987689, 0.156434, 0, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.912505, 0.156434, 0.377972, -0.912505, 0.156434, 0.377972, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.698401, 0.156434, 0.698402, -0.698401, 0.156434, 0.698402, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377971, 0.156434, 0.912505, -0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377973, 0.156433, 0.912505, 0.377973, 0.156433, 0.912505, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.698402, 0.156434, 0.698400, 0.698402, 0.156434, 0.698400, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.912505, 0.156435, 0.377972, 0.912505, 0.156435, 0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987688, 0.156434, 0, 0.987688, 0.156434, 0, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.912505, 0.156433, -0.377973, 0.912505, 0.156433, -0.377973, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.698401, 0.156434, -0.698401, 0.698401, 0.156434, -0.698401, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377973, 0.156435, -0.912505, 0.377973, 0.156435, -0.912505, 0.104474, 0.325528, -0.939743, 0.105800, 0.318418, -0.942028, 0.104474, 0.325528, -0.939743, 0.104474, 0.325528, -0.939743, 0.097141, 0.233062, -0.967598, 0.097141, 0.233062, -0.967598, -0.158828, 0.580683, -0.798486, -0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.090247, 0.600810, -0.794282, 0.091515, 0.594767, -0.798672, 0.090247, 0.600810, -0.794282, 0.090247, 0.600810, -0.794282, -0.116055, 0.803815, -0.583449, -0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.067135, 0.818259, -0.570916, 0.068222, 0.813901, -0.576985, 0.067135, 0.818259, -0.570916, 0.067135, 0.818259, -0.570916, -0.061299, 0.949355, -0.308167, -0.061299, 0.949355, -0.308167, 0, 1, 0, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137550, 0.933532, -0.331056, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253769, 0.933532, -0.253218, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331354, 0.933532, -0.136830, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, 0.000389, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331056, 0.933532, 0.137550, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253218, 0.933532, 0.253769, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.136830, 0.933532, 0.331354, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, 0.000389, 0.933532, 0.358494, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137550, 0.933532, 0.331056, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253768, 0.933532, 0.253219, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331354, 0.933532, 0.136830, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, -0.000391, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331056, 0.933532, -0.137550, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253218, 0.933532, -0.253769, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.136830, 0.933532, -0.331354, 0.042033, 0.942951, -0.330266, 0.042048, 0.942867, -0.330504, 0.042033, 0.942951, -0.330266, 0, 1, 0, -0.031102, 0.987211, -0.156357, -0.088569, 0.987211, -0.132553, -0.132553, 0.987211, -0.088569, -0.156357, 0.987211, -0.031102, -0.156357, 0.987211, 0.031102, -0.132553, 0.987211, 0.088569, -0.088569, 0.987211, 0.132553, -0.031102, 0.987211, 0.156357, 0.031102, 0.987211, 0.156357, 0.088569, 0.987211, 0.132553, 0.132553, 0.987211, 0.088570, 0.156357, 0.987211, 0.031101, 0.156357, 0.987211, -0.031102, 0.132553, 0.987211, -0.088569, 0.088569, 0.987211, -0.132553, 0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal7_5.vector = m_vector;
    Normal7_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate2_16.m_parent = (X3dNode *)&IndexedFaceSet_11;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate2_16.point = m_point;
    TextureCoordinate2_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_11.color = NULL;
    IndexedFaceSet_11.coord = &Coordinate7_9;
    IndexedFaceSet_11.normal = &Normal7_5;
    IndexedFaceSet_11.texCoord = &TextureCoordinate2_16;
    IndexedFaceSet_11.ccw = false;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_11.colorIndex = m_colorIndex;
    IndexedFaceSet_11.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_11.colorPerVertex = true;
    IndexedFaceSet_11.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_11.coordIndex = m_coordIndex;
    IndexedFaceSet_11.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_11.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet_11.normalIndex = m_normalIndex;
    IndexedFaceSet_11.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_11.normalPerVertex = true;
    IndexedFaceSet_11.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_11.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_11.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_11.set_colorIndex = NULL;
    IndexedFaceSet_11.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_11.set_coordIndex = NULL;
    IndexedFaceSet_11.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_11.set_normalIndex = NULL;
    IndexedFaceSet_11.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_11.set_texCoordIndex = NULL;
    IndexedFaceSet_11.set_texCoordIndex_length = 0;
    }
    Shape_130.appearance = &Appearance_131;
    Shape_130.geometry = &IndexedFaceSet_11;
    Transform_136.route_sources =  NULL;
    Transform_136.num_route_source = 0;
    Transform_136.m_parent = (X3dNode *)NULL;
    Shape_137.m_parent = (X3dNode *)NULL;
    Appearance_138.m_parent = (X3dNode *)NULL;
    Material_139.m_parent = (X3dNode *)NULL;
    Material_139.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material_139.diffuseColor = m_diffuseColor;
    Material_139.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_139.emissiveColor = m_emissiveColor;
    Material_139.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_139.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material_139.specularColor = m_specularColor;
    Material_139.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_139.transparency = 0;
    Appearance_138.material = &Material_139;
    Appearance_138.texture = NULL;
    Appearance_138.textureTransform = NULL;
    IndexedFaceSet_12.setGlName(7);
    IndexedFaceSet_12.m_parent = (X3dNode *)NULL;
    Coordinate8_9.m_parent = (X3dNode *)&IndexedFaceSet_12;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate8_9.point = m_point;
    Coordinate8_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal8_5.m_parent = (X3dNode *)&IndexedFaceSet_12;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal8_5.vector = m_vector;
    Normal8_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate3_16.m_parent = (X3dNode *)&IndexedFaceSet_12;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate3_16.point = m_point;
    TextureCoordinate3_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_12.color = NULL;
    IndexedFaceSet_12.coord = &Coordinate8_9;
    IndexedFaceSet_12.normal = &Normal8_5;
    IndexedFaceSet_12.texCoord = &TextureCoordinate3_16;
    IndexedFaceSet_12.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_12.colorIndex = m_colorIndex;
    IndexedFaceSet_12.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_12.colorPerVertex = true;
    IndexedFaceSet_12.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_12.coordIndex = m_coordIndex;
    IndexedFaceSet_12.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_12.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet_12.normalIndex = m_normalIndex;
    IndexedFaceSet_12.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_12.normalPerVertex = true;
    IndexedFaceSet_12.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_12.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_12.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_12.set_colorIndex = NULL;
    IndexedFaceSet_12.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_12.set_coordIndex = NULL;
    IndexedFaceSet_12.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_12.set_normalIndex = NULL;
    IndexedFaceSet_12.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_12.set_texCoordIndex = NULL;
    IndexedFaceSet_12.set_texCoordIndex_length = 0;
    }
    Shape_137.appearance = &Appearance_138;
    Shape_137.geometry = &IndexedFaceSet_12;
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 0 };
    Transform_136.rotation = m_rotation;
    Transform_136.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.220000, 0.440000, 0.160000 };
    Transform_136.translation = m_translation;
    Transform_136.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_136.bboxCenter = m_bboxCenter;
    Transform_136.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_136.bboxSize = m_bboxSize;
    Transform_136.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_136.center = m_center;
    Transform_136.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform_136.scale = m_scale;
    Transform_136.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_136.scaleOrientation = m_scaleOrientation;
    Transform_136.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_137 };
    Transform_136.children = m_children;
    Transform_136.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_136.addChildren = NULL;
    Transform_136.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_136.removeChildren = NULL;
    Transform_136.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 1.578764 };
    Transform_129.rotation = m_rotation;
    Transform_129.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_129.translation = m_translation;
    Transform_129.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_129.bboxCenter = m_bboxCenter;
    Transform_129.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_129.bboxSize = m_bboxSize;
    Transform_129.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_129.center = m_center;
    Transform_129.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_129.scale = m_scale;
    Transform_129.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_129.scaleOrientation = m_scaleOrientation;
    Transform_129.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_130, &Transform_136 };
    Transform_129.children = m_children;
    Transform_129.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_129.addChildren = NULL;
    Transform_129.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_129.removeChildren = NULL;
    Transform_129.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.565552, -0.792356, -0.228743, 0.826199 };
    Transform_128.rotation = m_rotation;
    Transform_128.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.109765, 0.715337, 1.294240 };
    Transform_128.translation = m_translation;
    Transform_128.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_128.bboxCenter = m_bboxCenter;
    Transform_128.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_128.bboxSize = m_bboxSize;
    Transform_128.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_128.center = m_center;
    Transform_128.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform_128.scale = m_scale;
    Transform_128.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_128.scaleOrientation = m_scaleOrientation;
    Transform_128.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_129 };
    Transform_128.children = m_children;
    Transform_128.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_128.addChildren = NULL;
    Transform_128.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_128.removeChildren = NULL;
    Transform_128.removeChildren_length = 0;
    }
    Transform_143.route_sources =  NULL;
    Transform_143.num_route_source = 0;
    Transform_143.m_parent = (X3dNode *)NULL;
    Transform_144.route_sources =  NULL;
    Transform_144.num_route_source = 0;
    Transform_144.m_parent = (X3dNode *)NULL;
    Shape_145.m_parent = (X3dNode *)NULL;
    Appearance_146.m_parent = (X3dNode *)NULL;
    Material_147.m_parent = (X3dNode *)NULL;
    Material_147.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_147.diffuseColor = m_diffuseColor;
    Material_147.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_147.emissiveColor = m_emissiveColor;
    Material_147.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_147.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_147.specularColor = m_specularColor;
    Material_147.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_147.transparency = 0;
    Appearance_146.material = &Material_147;
    Appearance_146.texture = NULL;
    Appearance_146.textureTransform = NULL;
    IndexedFaceSet_13.setGlName(8);
    IndexedFaceSet_13.m_parent = (X3dNode *)NULL;
    Coordinate9_9.m_parent = (X3dNode *)&IndexedFaceSet_13;
    {
    static float m_point[] = { 0, 0, -1, 0.382683, 0, -0.923880, 0.707107, 0, -0.707107, 0.923880, 0, -0.382683, 1, 0, 0, 0.923880, 0, 0.382684, 0.707107, 0, 0.707107, 0.382683, 0, 0.923880, 0, 0, 1, -0.382683, 0, 0.923880, -0.707107, 0, 0.707107, -0.923880, 0, 0.382683, -1, 0, 0, -0.923879, 0, -0.382684, -0.707107, 0, -0.707107, -0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, 0.363954, 0.309017, -0.878662, 0.672499, 0.309017, -0.672499, 0.878662, 0.309017, -0.363954, 0.951057, 0.309017, 0, 0.878662, 0.309017, 0.363954, 0.672499, 0.309017, 0.672499, 0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, -0.363954, 0.309017, 0.878662, -0.672499, 0.309017, 0.672498, -0.878662, 0.309017, 0.363953, -0.951057, 0.309017, 0, -0.878662, 0.309017, -0.363954, -0.672498, 0.309017, -0.672499, -0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, 0.309597, 0.587785, -0.747434, 0.572061, 0.587785, -0.572061, 0.747434, 0.587785, -0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, 0.309597, 0.572061, 0.587785, 0.572061, 0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, -0.309597, 0.587785, 0.747434, -0.572061, 0.587785, 0.572061, -0.747434, 0.587785, 0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, -0.309598, -0.572061, 0.587785, -0.572062, -0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, 0.224936, 0.809017, -0.543043, 0.415627, 0.809017, -0.415627, 0.543043, 0.809017, -0.224936, 0.587785, 0.809017, 0, 0.543043, 0.809017, 0.224936, 0.415627, 0.809017, 0.415627, 0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, -0.224936, 0.809017, 0.543043, -0.415627, 0.809017, 0.415627, -0.543043, 0.809017, 0.224935, -0.587785, 0.809017, 0, -0.543043, 0.809017, -0.224936, -0.415627, 0.809017, -0.415627, -0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, 0.118256, 0.951057, -0.285494, 0.218508, 0.951057, -0.218508, 0.285494, 0.951057, -0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, 0.118256, 0.218508, 0.951057, 0.218508, 0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, -0.118256, 0.951057, 0.285494, -0.218508, 0.951057, 0.218508, -0.285495, 0.951057, 0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, -0.118256, -0.218508, 0.951057, -0.218508, -0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate9_9.point = m_point;
    Coordinate9_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal9_5.m_parent = (X3dNode *)&IndexedFaceSet_13;
    {
    static float m_vector[] = { 0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, 0.185864, 0.303889, -0.934401, 0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377972, 0.156433, -0.912505, 0.377972, 0.156433, -0.912505, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.698401, 0.156433, -0.698401, 0.698401, 0.156433, -0.698401, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.912505, 0.156434, -0.377972, 0.912505, 0.156434, -0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987689, 0.156434, 0, 0.987689, 0.156434, 0, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.912505, 0.156434, 0.377972, 0.912505, 0.156434, 0.377972, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.698401, 0.156434, 0.698402, 0.698401, 0.156434, 0.698402, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377971, 0.156434, 0.912505, 0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377973, 0.156433, 0.912505, -0.377973, 0.156433, 0.912505, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.698402, 0.156434, 0.698400, -0.698402, 0.156434, 0.698400, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.912505, 0.156435, 0.377972, -0.912505, 0.156435, 0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987688, 0.156434, 0, -0.987688, 0.156434, 0, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.912505, 0.156433, -0.377973, -0.912505, 0.156433, -0.377973, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.698401, 0.156434, -0.698401, -0.698401, 0.156434, -0.698401, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377973, 0.156435, -0.912505, -0.377973, 0.156435, -0.912505, -0.104474, 0.325528, -0.939743, -0.105800, 0.318418, -0.942028, -0.104474, 0.325528, -0.939743, -0.104474, 0.325528, -0.939743, -0.097141, 0.233062, -0.967598, -0.097141, 0.233062, -0.967598, 0.158828, 0.580683, -0.798486, 0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.090247, 0.600810, -0.794282, -0.091515, 0.594767, -0.798672, -0.090247, 0.600810, -0.794282, -0.090247, 0.600810, -0.794282, 0.116055, 0.803815, -0.583449, 0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.067135, 0.818259, -0.570916, -0.068222, 0.813901, -0.576985, -0.067135, 0.818259, -0.570916, -0.067135, 0.818259, -0.570916, 0.061299, 0.949355, -0.308167, 0.061299, 0.949355, -0.308167, 0, 1, 0, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137550, 0.933532, -0.331056, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253769, 0.933532, -0.253218, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331354, 0.933532, -0.136830, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, 0.000389, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331056, 0.933532, 0.137550, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253218, 0.933532, 0.253769, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.136830, 0.933532, 0.331354, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, -0.000389, 0.933532, 0.358494, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137550, 0.933532, 0.331056, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253768, 0.933532, 0.253219, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331354, 0.933532, 0.136830, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, -0.000391, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331056, 0.933532, -0.137550, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253218, 0.933532, -0.253769, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.136830, 0.933532, -0.331354, -0.042033, 0.942951, -0.330266, -0.042048, 0.942867, -0.330504, -0.042033, 0.942951, -0.330266, 0, 1, 0, 0.031102, 0.987211, -0.156357, 0.088569, 0.987211, -0.132553, 0.132553, 0.987211, -0.088569, 0.156357, 0.987211, -0.031102, 0.156357, 0.987211, 0.031102, 0.132553, 0.987211, 0.088569, 0.088569, 0.987211, 0.132553, 0.031102, 0.987211, 0.156357, -0.031102, 0.987211, 0.156357, -0.088569, 0.987211, 0.132553, -0.132553, 0.987211, 0.088570, -0.156357, 0.987211, 0.031101, -0.156357, 0.987211, -0.031102, -0.132553, 0.987211, -0.088569, -0.088569, 0.987211, -0.132553, -0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal9_5.vector = m_vector;
    Normal9_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate4_16.m_parent = (X3dNode *)&IndexedFaceSet_13;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate4_16.point = m_point;
    TextureCoordinate4_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_13.color = NULL;
    IndexedFaceSet_13.coord = &Coordinate9_9;
    IndexedFaceSet_13.normal = &Normal9_5;
    IndexedFaceSet_13.texCoord = &TextureCoordinate4_16;
    IndexedFaceSet_13.ccw = true;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_13.colorIndex = m_colorIndex;
    IndexedFaceSet_13.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_13.colorPerVertex = true;
    IndexedFaceSet_13.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_13.coordIndex = m_coordIndex;
    IndexedFaceSet_13.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_13.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet_13.normalIndex = m_normalIndex;
    IndexedFaceSet_13.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_13.normalPerVertex = true;
    IndexedFaceSet_13.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_13.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_13.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_13.set_colorIndex = NULL;
    IndexedFaceSet_13.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_13.set_coordIndex = NULL;
    IndexedFaceSet_13.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_13.set_normalIndex = NULL;
    IndexedFaceSet_13.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_13.set_texCoordIndex = NULL;
    IndexedFaceSet_13.set_texCoordIndex_length = 0;
    }
    Shape_145.appearance = &Appearance_146;
    Shape_145.geometry = &IndexedFaceSet_13;
    Transform_151.route_sources =  NULL;
    Transform_151.num_route_source = 0;
    Transform_151.m_parent = (X3dNode *)NULL;
    Shape_152.m_parent = (X3dNode *)NULL;
    Appearance_153.m_parent = (X3dNode *)NULL;
    Material_154.m_parent = (X3dNode *)NULL;
    Material_154.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material_154.diffuseColor = m_diffuseColor;
    Material_154.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_154.emissiveColor = m_emissiveColor;
    Material_154.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_154.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material_154.specularColor = m_specularColor;
    Material_154.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_154.transparency = 0;
    Appearance_153.material = &Material_154;
    Appearance_153.texture = NULL;
    Appearance_153.textureTransform = NULL;
    IndexedFaceSet_14.setGlName(9);
    IndexedFaceSet_14.m_parent = (X3dNode *)NULL;
    Coordinate10_9.m_parent = (X3dNode *)&IndexedFaceSet_14;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate10_9.point = m_point;
    Coordinate10_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal10_5.m_parent = (X3dNode *)&IndexedFaceSet_14;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal10_5.vector = m_vector;
    Normal10_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate5_16.m_parent = (X3dNode *)&IndexedFaceSet_14;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate5_16.point = m_point;
    TextureCoordinate5_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_14.color = NULL;
    IndexedFaceSet_14.coord = &Coordinate10_9;
    IndexedFaceSet_14.normal = &Normal10_5;
    IndexedFaceSet_14.texCoord = &TextureCoordinate5_16;
    IndexedFaceSet_14.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_14.colorIndex = m_colorIndex;
    IndexedFaceSet_14.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_14.colorPerVertex = true;
    IndexedFaceSet_14.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_14.coordIndex = m_coordIndex;
    IndexedFaceSet_14.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_14.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet_14.normalIndex = m_normalIndex;
    IndexedFaceSet_14.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_14.normalPerVertex = true;
    IndexedFaceSet_14.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_14.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_14.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_14.set_colorIndex = NULL;
    IndexedFaceSet_14.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_14.set_coordIndex = NULL;
    IndexedFaceSet_14.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_14.set_normalIndex = NULL;
    IndexedFaceSet_14.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_14.set_texCoordIndex = NULL;
    IndexedFaceSet_14.set_texCoordIndex_length = 0;
    }
    Shape_152.appearance = &Appearance_153;
    Shape_152.geometry = &IndexedFaceSet_14;
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, 0 };
    Transform_151.rotation = m_rotation;
    Transform_151.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.220000, 0.440000, 0.160000 };
    Transform_151.translation = m_translation;
    Transform_151.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_151.bboxCenter = m_bboxCenter;
    Transform_151.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_151.bboxSize = m_bboxSize;
    Transform_151.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_151.center = m_center;
    Transform_151.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform_151.scale = m_scale;
    Transform_151.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_151.scaleOrientation = m_scaleOrientation;
    Transform_151.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_152 };
    Transform_151.children = m_children;
    Transform_151.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_151.addChildren = NULL;
    Transform_151.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_151.removeChildren = NULL;
    Transform_151.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, -1.578764 };
    Transform_144.rotation = m_rotation;
    Transform_144.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_144.translation = m_translation;
    Transform_144.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_144.bboxCenter = m_bboxCenter;
    Transform_144.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_144.bboxSize = m_bboxSize;
    Transform_144.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_144.center = m_center;
    Transform_144.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_144.scale = m_scale;
    Transform_144.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_144.scaleOrientation = m_scaleOrientation;
    Transform_144.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_145, &Transform_151 };
    Transform_144.children = m_children;
    Transform_144.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_144.addChildren = NULL;
    Transform_144.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_144.removeChildren = NULL;
    Transform_144.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.565552, -0.792356, -0.228743, -0.826199 };
    Transform_143.rotation = m_rotation;
    Transform_143.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.109765, 0.715337, 1.294240 };
    Transform_143.translation = m_translation;
    Transform_143.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_143.bboxCenter = m_bboxCenter;
    Transform_143.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_143.bboxSize = m_bboxSize;
    Transform_143.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_143.center = m_center;
    Transform_143.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform_143.scale = m_scale;
    Transform_143.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_143.scaleOrientation = m_scaleOrientation;
    Transform_143.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_144 };
    Transform_143.children = m_children;
    Transform_143.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_143.addChildren = NULL;
    Transform_143.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_143.removeChildren = NULL;
    Transform_143.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_127.rotation = m_rotation;
    Transform_127.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0.120000 };
    Transform_127.translation = m_translation;
    Transform_127.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_127.bboxCenter = m_bboxCenter;
    Transform_127.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_127.bboxSize = m_bboxSize;
    Transform_127.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_127.center = m_center;
    Transform_127.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_127.scale = m_scale;
    Transform_127.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_127.scaleOrientation = m_scaleOrientation;
    Transform_127.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_128, &Transform_143 };
    Transform_127.children = m_children;
    Transform_127.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_127.addChildren = NULL;
    Transform_127.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_127.removeChildren = NULL;
    Transform_127.removeChildren_length = 0;
    }
    Inline_21.m_parent = (X3dNode *)NULL;
    Inline_17.m_parent = (X3dNode *)NULL;
    Group3.m_parent = (X3dNode *)&Inline_17;
    Transform_158.route_sources =  NULL;
    Transform_158.num_route_source = 0;
    Transform_158.m_parent = (X3dNode *)NULL;
    CORINTm_5.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_5.key = m_key;
    CORINTm_5.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.619297, 0.959445, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.779760, 0.966057, -0.506980, 1.645980, 0.964696, -0.929632, 2.257560, 1.170100, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890 };
    CORINTm_5.keyValue = m_keyValue;
    CORINTm_5.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_5.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_5.value_changed = NULL;
    CORINTm_5.value_changed_length = 0;
    }
    TIMERm_4.m_parent = (X3dNode *)NULL;
    TIMERm_4.cycleInterval = 2;
    TIMERm_4.enabled = true;
    TIMERm_4.loop = true;
    TIMERm_4.startTime = 0;
    TIMERm_4.stopTime = 0;
    TIMERm_4.cycleTime = 0;
    TIMERm_4.fraction_changed = 0;
    TIMERm_4.isActive = false;
    TIMERm_4.time = 0;
    Group_159.m_parent = (X3dNode *)NULL;
    Background_160.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background_160.groundAngle = NULL;
    Background_160.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background_160.groundColor = m_groundColor;
    Background_160.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background_160.backUrl = NULL;
    Background_160.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background_160.rightUrl = NULL;
    Background_160.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background_160.frontUrl = NULL;
    Background_160.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background_160.leftUrl = NULL;
    Background_160.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background_160.topUrl = NULL;
    Background_160.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background_160.bottomUrl = NULL;
    Background_160.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background_160.skyAngle = NULL;
    Background_160.skyAngle_length = 0;
    }
    Background_160.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background_160.skyColor = m_skyColor;
    Background_160.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background_160.set_bind = false;
    Background_160.isBound = false;
    Shape_161.m_parent = (X3dNode *)NULL;
    Appearance_162.m_parent = (X3dNode *)NULL;
    Material_163.m_parent = (X3dNode *)NULL;
    Material_163.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_163.diffuseColor = m_diffuseColor;
    Material_163.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_163.emissiveColor = m_emissiveColor;
    Material_163.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_163.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_163.specularColor = m_specularColor;
    Material_163.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_163.transparency = 6.382980e-2;
    Appearance_162.material = &Material_163;
    Appearance_162.texture = NULL;
    Appearance_162.textureTransform = NULL;
    IndexedFaceSet_15.setGlName(10);
    IndexedFaceSet_15.m_parent = (X3dNode *)NULL;
    COORD1_6.m_parent = (X3dNode *)&IndexedFaceSet_15;
    {
    static float m_point[] = { 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.780723, -0.294626, 1.000070, 0.734580, -0.981481, 1.700790, 0.307968, -0.506980, 1.580170, 0.280283, -1.021760, 2.071980, -0.277696, -0.976890 };
    COORD1_6.point = m_point;
    COORD1_6.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal11.m_parent = (X3dNode *)&IndexedFaceSet_15;
    {
    static float m_vector[] = { 0.358806, 0.927849, -0.101757, 0.358806, 0.927849, -0.101757, 0.359718, 0.927470, -0.101994, 0.281824, 0.955978, -0.081744, 0.281824, 0.955978, -0.081744, 0.240191, 0.969864, -0.040901, 0.495155, 0.855218, -0.153047, 0.496190, 0.854566, -0.153340, 0.496190, 0.854566, -0.153340, 0.631845, 0.753624, -0.181169, 0.631845, 0.753624, -0.181169, 0.632281, 0.753233, -0.181277, 0.673713, 0.712478, -0.196177, 0.673713, 0.712478, -0.196177, 0.742001, 0.637272, -0.208134 };
    Normal11.vector = m_vector;
    Normal11.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_15.color = NULL;
    IndexedFaceSet_15.coord = &COORD1_6;
    IndexedFaceSet_15.normal = &Normal11;
    IndexedFaceSet_15.texCoord = NULL;
    IndexedFaceSet_15.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_15.colorIndex = m_colorIndex;
    IndexedFaceSet_15.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_15.colorPerVertex = true;
    IndexedFaceSet_15.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_15.coordIndex = m_coordIndex;
    IndexedFaceSet_15.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_15.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet_15.normalIndex = m_normalIndex;
    IndexedFaceSet_15.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_15.normalPerVertex = true;
    IndexedFaceSet_15.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_15.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_15.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_15.set_colorIndex = NULL;
    IndexedFaceSet_15.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_15.set_coordIndex = NULL;
    IndexedFaceSet_15.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_15.set_normalIndex = NULL;
    IndexedFaceSet_15.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_15.set_texCoordIndex = NULL;
    IndexedFaceSet_15.set_texCoordIndex_length = 0;
    }
    Shape_161.appearance = &Appearance_162;
    Shape_161.geometry = &IndexedFaceSet_15;
    {
    static X3dNode* m_children[] = { &Background_160, &Shape_161 };
    Group_159.children = m_children;
    Group_159.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_159.bboxCenter = m_bboxCenter;
    Group_159.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_159.bboxSize = m_bboxSize;
    Group_159.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_159.addChildren = NULL;
    Group_159.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_159.removeChildren = NULL;
    Group_159.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_158.rotation = m_rotation;
    Transform_158.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_158.translation = m_translation;
    Transform_158.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_158.bboxCenter = m_bboxCenter;
    Transform_158.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_158.bboxSize = m_bboxSize;
    Transform_158.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_158.center = m_center;
    Transform_158.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_158.scale = m_scale;
    Transform_158.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_158.scaleOrientation = m_scaleOrientation;
    Transform_158.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_5, &TIMERm_4, &Group_159 };
    Transform_158.children = m_children;
    Transform_158.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_158.addChildren = NULL;
    Transform_158.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_158.removeChildren = NULL;
    Transform_158.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_158 };
    Group3.children = m_children;
    Group3.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group3.bboxCenter = m_bboxCenter;
    Group3.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group3.bboxSize = m_bboxSize;
    Group3.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group3.addChildren = NULL;
    Group3.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group3.removeChildren = NULL;
    Group3.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group3 };
    Inline_17.children = m_children;
    Inline_17.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_17.bboxCenter = m_bboxCenter;
    Inline_17.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_17.bboxSize = m_bboxSize;
    Inline_17.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_17.addChildren = NULL;
    Inline_17.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_17.removeChildren = NULL;
    Inline_17.removeChildren_length = 0;
    }
    Inline_20.m_parent = (X3dNode *)NULL;
    Group4.m_parent = (X3dNode *)&Inline_20;
    Transform_164.route_sources =  NULL;
    Transform_164.num_route_source = 0;
    Transform_164.m_parent = (X3dNode *)NULL;
    CORINTm_6.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_6.key = m_key;
    CORINTm_6.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.619297, 0.959445, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.779760, 0.966057, -0.506980, -1.645980, 0.964696, -0.929632, -2.257560, 1.170100, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890 };
    CORINTm_6.keyValue = m_keyValue;
    CORINTm_6.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_6.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_6.value_changed = NULL;
    CORINTm_6.value_changed_length = 0;
    }
    TIMERm_5.m_parent = (X3dNode *)NULL;
    TIMERm_5.cycleInterval = 2;
    TIMERm_5.enabled = true;
    TIMERm_5.loop = true;
    TIMERm_5.startTime = 0;
    TIMERm_5.stopTime = 0;
    TIMERm_5.cycleTime = 0;
    TIMERm_5.fraction_changed = 0;
    TIMERm_5.isActive = false;
    TIMERm_5.time = 0;
    Group_165.m_parent = (X3dNode *)NULL;
    Background_166.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background_166.groundAngle = NULL;
    Background_166.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background_166.groundColor = m_groundColor;
    Background_166.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background_166.backUrl = NULL;
    Background_166.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background_166.rightUrl = NULL;
    Background_166.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background_166.frontUrl = NULL;
    Background_166.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background_166.leftUrl = NULL;
    Background_166.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background_166.topUrl = NULL;
    Background_166.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background_166.bottomUrl = NULL;
    Background_166.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background_166.skyAngle = NULL;
    Background_166.skyAngle_length = 0;
    }
    Background_166.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background_166.skyColor = m_skyColor;
    Background_166.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background_166.set_bind = false;
    Background_166.isBound = false;
    Shape_167.m_parent = (X3dNode *)NULL;
    Appearance_168.m_parent = (X3dNode *)NULL;
    Material_169.m_parent = (X3dNode *)NULL;
    Material_169.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_169.diffuseColor = m_diffuseColor;
    Material_169.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_169.emissiveColor = m_emissiveColor;
    Material_169.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_169.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_169.specularColor = m_specularColor;
    Material_169.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_169.transparency = 6.382980e-2;
    Appearance_168.material = &Material_169;
    Appearance_168.texture = NULL;
    Appearance_168.textureTransform = NULL;
    IndexedFaceSet_18.setGlName(11);
    IndexedFaceSet_18.m_parent = (X3dNode *)NULL;
    COORD1_9.m_parent = (X3dNode *)&IndexedFaceSet_18;
    {
    static float m_point[] = { -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.780723, -0.294626, -1.000070, 0.734580, -0.981481, -1.700790, 0.307968, -0.506980, -1.580170, 0.280283, -1.021760, -2.071980, -0.277696, -0.976890 };
    COORD1_9.point = m_point;
    COORD1_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal12.m_parent = (X3dNode *)&IndexedFaceSet_18;
    {
    static float m_vector[] = { 0.358806, -0.927849, 0.101757, 0.358806, -0.927849, 0.101757, 0.359718, -0.927470, 0.101994, 0.281824, -0.955978, 0.081744, 0.281824, -0.955978, 0.081744, 0.240191, -0.969864, 0.040901, 0.495155, -0.855218, 0.153047, 0.496190, -0.854566, 0.153340, 0.496190, -0.854566, 0.153340, 0.631845, -0.753624, 0.181169, 0.631845, -0.753624, 0.181169, 0.632281, -0.753233, 0.181277, 0.673713, -0.712478, 0.196177, 0.673713, -0.712478, 0.196177, 0.742001, -0.637272, 0.208134 };
    Normal12.vector = m_vector;
    Normal12.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_18.color = NULL;
    IndexedFaceSet_18.coord = &COORD1_9;
    IndexedFaceSet_18.normal = &Normal12;
    IndexedFaceSet_18.texCoord = NULL;
    IndexedFaceSet_18.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_18.colorIndex = m_colorIndex;
    IndexedFaceSet_18.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_18.colorPerVertex = true;
    IndexedFaceSet_18.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_18.coordIndex = m_coordIndex;
    IndexedFaceSet_18.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_18.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet_18.normalIndex = m_normalIndex;
    IndexedFaceSet_18.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_18.normalPerVertex = true;
    IndexedFaceSet_18.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_18.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_18.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_18.set_colorIndex = NULL;
    IndexedFaceSet_18.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_18.set_coordIndex = NULL;
    IndexedFaceSet_18.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_18.set_normalIndex = NULL;
    IndexedFaceSet_18.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_18.set_texCoordIndex = NULL;
    IndexedFaceSet_18.set_texCoordIndex_length = 0;
    }
    Shape_167.appearance = &Appearance_168;
    Shape_167.geometry = &IndexedFaceSet_18;
    {
    static X3dNode* m_children[] = { &Background_166, &Shape_167 };
    Group_165.children = m_children;
    Group_165.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_165.bboxCenter = m_bboxCenter;
    Group_165.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_165.bboxSize = m_bboxSize;
    Group_165.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_165.addChildren = NULL;
    Group_165.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_165.removeChildren = NULL;
    Group_165.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_164.rotation = m_rotation;
    Transform_164.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_164.translation = m_translation;
    Transform_164.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_164.bboxCenter = m_bboxCenter;
    Transform_164.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_164.bboxSize = m_bboxSize;
    Transform_164.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_164.center = m_center;
    Transform_164.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_164.scale = m_scale;
    Transform_164.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_164.scaleOrientation = m_scaleOrientation;
    Transform_164.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_6, &TIMERm_5, &Group_165 };
    Transform_164.children = m_children;
    Transform_164.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_164.addChildren = NULL;
    Transform_164.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_164.removeChildren = NULL;
    Transform_164.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_164 };
    Group4.children = m_children;
    Group4.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group4.bboxCenter = m_bboxCenter;
    Group4.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group4.bboxSize = m_bboxSize;
    Group4.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group4.addChildren = NULL;
    Group4.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group4.removeChildren = NULL;
    Group4.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group4 };
    Inline_20.children = m_children;
    Inline_20.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_20.bboxCenter = m_bboxCenter;
    Inline_20.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_20.bboxSize = m_bboxSize;
    Inline_20.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_20.addChildren = NULL;
    Inline_20.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_20.removeChildren = NULL;
    Inline_20.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Inline_17, &Inline_20 };
    Inline_21.children = m_children;
    Inline_21.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_21.bboxCenter = m_bboxCenter;
    Inline_21.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_21.bboxSize = m_bboxSize;
    Inline_21.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_21.addChildren = NULL;
    Inline_21.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_21.removeChildren = NULL;
    Inline_21.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform1.rotation = m_rotation;
    Transform1.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform1.translation = m_translation;
    Transform1.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform1.bboxCenter = m_bboxCenter;
    Transform1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform1.bboxSize = m_bboxSize;
    Transform1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform1.center = m_center;
    Transform1.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform1.scale = m_scale;
    Transform1.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform1.scaleOrientation = m_scaleOrientation;
    Transform1.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_10, &Transform_127, &Inline_21 };
    Transform1.children = m_children;
    Transform1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform1.addChildren = NULL;
    Transform1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform1.removeChildren = NULL;
    Transform1.removeChildren_length = 0;
    }
    WorldInfo2.m_parent = (X3dNode *)&Inline_22;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "p_fly is a modification of the program tux_aqfh by Steve Baker", "tux_aqfh is GPL" };
    WorldInfo2.info = m_info;
    WorldInfo2.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo2.title = "Tux warbird 3D Game";
    {
    static X3dNode* m_children[] = { &Transform1, &WorldInfo2 };
    Inline_22.children = m_children;
    Inline_22.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_22.bboxCenter = m_bboxCenter;
    Inline_22.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_22.bboxSize = m_bboxSize;
    Inline_22.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_22.addChildren = NULL;
    Inline_22.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_22.removeChildren = NULL;
    Inline_22.removeChildren_length = 0;
    }
    PLANEm_SWITCH.m_parent = (X3dNode *)&HIDEm_THING;
    Inline_25.m_parent = (X3dNode *)&PLANEm_SWITCH;
    Transform2.route_sources =  NULL;
    Transform2.num_route_source = 0;
    Transform2.m_parent = (X3dNode *)&Inline_25;
    Shape_170.m_parent = (X3dNode *)NULL;
    Appearance_171.m_parent = (X3dNode *)NULL;
    Material_172.m_parent = (X3dNode *)NULL;
    Material_172.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_172.diffuseColor = m_diffuseColor;
    Material_172.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_172.emissiveColor = m_emissiveColor;
    Material_172.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_172.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_172.specularColor = m_specularColor;
    Material_172.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_172.transparency = 0;
    Appearance_171.material = &Material_172;
    Appearance_171.texture = NULL;
    Appearance_171.textureTransform = NULL;
    IndexedFaceSet_23.setGlName(12);
    IndexedFaceSet_23.m_parent = (X3dNode *)NULL;
    Color4_4.m_parent = (X3dNode *)&IndexedFaceSet_23;
    {
    static float m_color[] = { 1, 0, 0 };
    Color4_4.color = m_color;
    Color4_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate11_9.m_parent = (X3dNode *)&IndexedFaceSet_23;
    {
    static float m_point[] = { 0, 2.145490, -2.315630, 0, 0.876689, -1.765810, 0, 1.553380, -0.553399, 0, 2.201880, -1.427460 };
    Coordinate11_9.point = m_point;
    Coordinate11_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal13.m_parent = (X3dNode *)&IndexedFaceSet_23;
    {
    static float m_vector[] = { -1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0 };
    Normal13.vector = m_vector;
    Normal13.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_23.color = &Color4_4;
    IndexedFaceSet_23.coord = &Coordinate11_9;
    IndexedFaceSet_23.normal = &Normal13;
    IndexedFaceSet_23.texCoord = NULL;
    IndexedFaceSet_23.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet_23.colorIndex = m_colorIndex;
    IndexedFaceSet_23.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_23.colorPerVertex = false;
    IndexedFaceSet_23.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_23.coordIndex = m_coordIndex;
    IndexedFaceSet_23.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_23.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_23.normalIndex = m_normalIndex;
    IndexedFaceSet_23.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_23.normalPerVertex = true;
    IndexedFaceSet_23.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_23.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_23.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_23.set_colorIndex = NULL;
    IndexedFaceSet_23.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_23.set_coordIndex = NULL;
    IndexedFaceSet_23.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_23.set_normalIndex = NULL;
    IndexedFaceSet_23.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_23.set_texCoordIndex = NULL;
    IndexedFaceSet_23.set_texCoordIndex_length = 0;
    }
    Shape_170.appearance = &Appearance_171;
    Shape_170.geometry = &IndexedFaceSet_23;
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform2.rotation = m_rotation;
    Transform2.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform2.translation = m_translation;
    Transform2.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform2.bboxCenter = m_bboxCenter;
    Transform2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform2.bboxSize = m_bboxSize;
    Transform2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform2.center = m_center;
    Transform2.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform2.scale = m_scale;
    Transform2.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform2.scaleOrientation = m_scaleOrientation;
    Transform2.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_170 };
    Transform2.children = m_children;
    Transform2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform2.addChildren = NULL;
    Transform2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform2.removeChildren = NULL;
    Transform2.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform2 };
    Inline_25.children = m_children;
    Inline_25.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_25.bboxCenter = m_bboxCenter;
    Inline_25.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_25.bboxSize = m_bboxSize;
    Inline_25.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_25.addChildren = NULL;
    Inline_25.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_25.removeChildren = NULL;
    Inline_25.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Inline_25 };
    PLANEm_SWITCH.children = m_children;
    PLANEm_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    PLANEm_SWITCH.whichChoice = -1;
    Transform_73.route_sources =  NULL;
    Transform_73.num_route_source = 0;
    Transform_73.m_parent = (X3dNode *)&HIDEm_THING;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_73.rotation = m_rotation;
    Transform_73.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -1.150000 };
    Transform_73.translation = m_translation;
    Transform_73.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_73.bboxCenter = m_bboxCenter;
    Transform_73.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_73.bboxSize = m_bboxSize;
    Transform_73.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_73.center = m_center;
    Transform_73.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_73.scale = m_scale;
    Transform_73.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_73.scaleOrientation = m_scaleOrientation;
    Transform_73.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[1];
    Transform_73.children = NULL;
    Transform_73.children_length = 0;
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_73.addChildren = NULL;
    Transform_73.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_73.removeChildren = NULL;
    Transform_73.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    HIDEm_THING.rotation = m_rotation;
    HIDEm_THING.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    HIDEm_THING.translation = m_translation;
    HIDEm_THING.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    HIDEm_THING.bboxCenter = m_bboxCenter;
    HIDEm_THING.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    HIDEm_THING.bboxSize = m_bboxSize;
    HIDEm_THING.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    HIDEm_THING.center = m_center;
    HIDEm_THING.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    HIDEm_THING.scale = m_scale;
    HIDEm_THING.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    HIDEm_THING.scaleOrientation = m_scaleOrientation;
    HIDEm_THING.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_22, &PLANEm_SWITCH, &Transform_73 };
    HIDEm_THING.children = m_children;
    HIDEm_THING.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    HIDEm_THING.addChildren = NULL;
    HIDEm_THING.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    HIDEm_THING.removeChildren = NULL;
    HIDEm_THING.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    UPm_DOWNm_AXIS.rotation = m_rotation;
    UPm_DOWNm_AXIS.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.translation = m_translation;
    UPm_DOWNm_AXIS.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.bboxCenter = m_bboxCenter;
    UPm_DOWNm_AXIS.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    UPm_DOWNm_AXIS.bboxSize = m_bboxSize;
    UPm_DOWNm_AXIS.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    UPm_DOWNm_AXIS.center = m_center;
    UPm_DOWNm_AXIS.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    UPm_DOWNm_AXIS.scale = m_scale;
    UPm_DOWNm_AXIS.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    UPm_DOWNm_AXIS.scaleOrientation = m_scaleOrientation;
    UPm_DOWNm_AXIS.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &THINGm_SENSOR, &HIDEm_THING };
    UPm_DOWNm_AXIS.children = m_children;
    UPm_DOWNm_AXIS.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    UPm_DOWNm_AXIS.addChildren = NULL;
    UPm_DOWNm_AXIS.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    UPm_DOWNm_AXIS.removeChildren = NULL;
    UPm_DOWNm_AXIS.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    ROLLm_AXIS.rotation = m_rotation;
    ROLLm_AXIS.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    ROLLm_AXIS.translation = m_translation;
    ROLLm_AXIS.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    ROLLm_AXIS.bboxCenter = m_bboxCenter;
    ROLLm_AXIS.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    ROLLm_AXIS.bboxSize = m_bboxSize;
    ROLLm_AXIS.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    ROLLm_AXIS.center = m_center;
    ROLLm_AXIS.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    ROLLm_AXIS.scale = m_scale;
    ROLLm_AXIS.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    ROLLm_AXIS.scaleOrientation = m_scaleOrientation;
    ROLLm_AXIS.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &UPm_DOWNm_AXIS };
    ROLLm_AXIS.children = m_children;
    ROLLm_AXIS.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    ROLLm_AXIS.addChildren = NULL;
    ROLLm_AXIS.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    ROLLm_AXIS.removeChildren = NULL;
    ROLLm_AXIS.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    THING.rotation = m_rotation;
    THING.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    THING.translation = m_translation;
    THING.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    THING.bboxCenter = m_bboxCenter;
    THING.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    THING.bboxSize = m_bboxSize;
    THING.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    THING.center = m_center;
    THING.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    THING.scale = m_scale;
    THING.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    THING.scaleOrientation = m_scaleOrientation;
    THING.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &ROLLm_AXIS };
    THING.children = m_children;
    THING.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    THING.addChildren = NULL;
    THING.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    THING.removeChildren = NULL;
    THING.removeChildren_length = 0;
    }
    BULLETm_SWITCH.m_parent = (X3dNode *)&Group_71;
    BULLETm_MOVE.route_sources =  NULL;
    BULLETm_MOVE.num_route_source = 0;
    BULLETm_MOVE.m_parent = (X3dNode *)&BULLETm_SWITCH;
    DUMMYm_BULLET.m_parent = (X3dNode *)&BULLETm_MOVE;
    DUMMYm_BULLET.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    DUMMYm_BULLET.hitNormal_changed = m_hitNormal_changed;
    DUMMYm_BULLET.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    DUMMYm_BULLET.hitPoint_changed = m_hitPoint_changed;
    DUMMYm_BULLET.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    DUMMYm_BULLET.hitTexCoord_changed = m_hitTexCoord_changed;
    DUMMYm_BULLET.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    DUMMYm_BULLET.isActive = false;
    DUMMYm_BULLET.isOver = false;
    DUMMYm_BULLET.touchTime = 0;
    Inline_28.m_parent = (X3dNode *)&BULLETm_MOVE;
    Transform3.route_sources =  NULL;
    Transform3.num_route_source = 0;
    Transform3.m_parent = (X3dNode *)&Inline_28;
    Inline_27.m_parent = (X3dNode *)NULL;
    Shape1.m_parent = (X3dNode *)&Inline_27;
    Appearance_175.m_parent = (X3dNode *)NULL;
    Material_176.m_parent = (X3dNode *)NULL;
    Material_176.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_176.diffuseColor = m_diffuseColor;
    Material_176.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_176.emissiveColor = m_emissiveColor;
    Material_176.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_176.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_176.specularColor = m_specularColor;
    Material_176.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_176.transparency = 0;
    ImageTexture_177.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "cake.png" };
    ImageTexture_177.url = m_url;
    ImageTexture_177.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_177.repeatS = true;
    ImageTexture_177.repeatT = true;
    Appearance_175.material = &Material_176;
    Appearance_175.texture = &ImageTexture_177;
    Appearance_175.textureTransform = NULL;
    IndexedFaceSet_26.setGlName(13);
    IndexedFaceSet_26.m_parent = (X3dNode *)NULL;
    Coordinate12_9.m_parent = (X3dNode *)&IndexedFaceSet_26;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate12_9.point = m_point;
    Coordinate12_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal14_5.m_parent = (X3dNode *)&IndexedFaceSet_26;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal14_5.vector = m_vector;
    Normal14_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate6_16.m_parent = (X3dNode *)&IndexedFaceSet_26;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate6_16.point = m_point;
    TextureCoordinate6_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_26.color = NULL;
    IndexedFaceSet_26.coord = &Coordinate12_9;
    IndexedFaceSet_26.normal = &Normal14_5;
    IndexedFaceSet_26.texCoord = &TextureCoordinate6_16;
    IndexedFaceSet_26.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1 };
    IndexedFaceSet_26.colorIndex = m_colorIndex;
    IndexedFaceSet_26.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_26.colorPerVertex = false;
    IndexedFaceSet_26.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_26.coordIndex = m_coordIndex;
    IndexedFaceSet_26.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_26.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_26.normalIndex = m_normalIndex;
    IndexedFaceSet_26.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_26.normalPerVertex = true;
    IndexedFaceSet_26.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_26.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_26.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_26.set_colorIndex = NULL;
    IndexedFaceSet_26.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_26.set_coordIndex = NULL;
    IndexedFaceSet_26.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_26.set_normalIndex = NULL;
    IndexedFaceSet_26.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_26.set_texCoordIndex = NULL;
    IndexedFaceSet_26.set_texCoordIndex_length = 0;
    }
    Shape1.appearance = &Appearance_175;
    Shape1.geometry = &IndexedFaceSet_26;
    WorldInfo3.m_parent = (X3dNode *)&Inline_27;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo3.info = m_info;
    WorldInfo3.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo3.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Shape1, &WorldInfo3 };
    Inline_27.children = m_children;
    Inline_27.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_27.bboxCenter = m_bboxCenter;
    Inline_27.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_27.bboxSize = m_bboxSize;
    Inline_27.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_27.addChildren = NULL;
    Inline_27.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_27.removeChildren = NULL;
    Inline_27.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform3.rotation = m_rotation;
    Transform3.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform3.translation = m_translation;
    Transform3.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform3.bboxCenter = m_bboxCenter;
    Transform3.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform3.bboxSize = m_bboxSize;
    Transform3.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform3.center = m_center;
    Transform3.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform3.scale = m_scale;
    Transform3.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform3.scaleOrientation = m_scaleOrientation;
    Transform3.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_27 };
    Transform3.children = m_children;
    Transform3.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform3.addChildren = NULL;
    Transform3.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform3.removeChildren = NULL;
    Transform3.removeChildren_length = 0;
    }
    WorldInfo4.m_parent = (X3dNode *)&Inline_28;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo4.info = m_info;
    WorldInfo4.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo4.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Transform3, &WorldInfo4 };
    Inline_28.children = m_children;
    Inline_28.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_28.bboxCenter = m_bboxCenter;
    Inline_28.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_28.bboxSize = m_bboxSize;
    Inline_28.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_28.addChildren = NULL;
    Inline_28.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_28.removeChildren = NULL;
    Inline_28.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BULLETm_MOVE.rotation = m_rotation;
    BULLETm_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BULLETm_MOVE.translation = m_translation;
    BULLETm_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BULLETm_MOVE.bboxCenter = m_bboxCenter;
    BULLETm_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BULLETm_MOVE.bboxSize = m_bboxSize;
    BULLETm_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BULLETm_MOVE.center = m_center;
    BULLETm_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BULLETm_MOVE.scale = m_scale;
    BULLETm_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BULLETm_MOVE.scaleOrientation = m_scaleOrientation;
    BULLETm_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &DUMMYm_BULLET, &Inline_28 };
    BULLETm_MOVE.children = m_children;
    BULLETm_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BULLETm_MOVE.addChildren = NULL;
    BULLETm_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BULLETm_MOVE.removeChildren = NULL;
    BULLETm_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BULLETm_MOVE };
    BULLETm_SWITCH.children = m_children;
    BULLETm_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BULLETm_SWITCH.whichChoice = -1;
    TIMER.m_parent = (X3dNode *)&Group_71;
    TIMER.cycleInterval = 1;
    TIMER.enabled = true;
    TIMER.loop = true;
    TIMER.startTime = 0;
    TIMER.stopTime = 0;
    TIMER.cycleTime = 0;
    TIMER.fraction_changed = 0;
    TIMER.isActive = false;
    TIMER.time = 0;
    SOUND.m_parent = (X3dNode *)&Group_71;
    SOUNDCLIP.m_parent = (X3dNode *)&SOUND;
    SOUNDCLIP.description = "away";
    SOUNDCLIP.loop = false;
    SOUNDCLIP.pitch = 1;
    SOUNDCLIP.startTime = 0;
    SOUNDCLIP.stopTime = 0;
    {
    static const char* m_url[] = { "multimedia/bubble.wav" };
    SOUNDCLIP.url = m_url;
    SOUNDCLIP.url_length = sizeof(m_url) / sizeof(const char*);
    }
    SOUNDCLIP.duration_changed = 0;
    SOUNDCLIP.isActive = false;
    {
    static float m_direction[] = { 0, 0, 1 };
    SOUND.direction = m_direction;
    SOUND.direction_length = sizeof(m_direction) / sizeof(float);
    }
    SOUND.intensity = 1;
    {
    static float m_location[] = { 0, 0, 0 };
    SOUND.location = m_location;
    SOUND.location_length = sizeof(m_location) / sizeof(float);
    }
    SOUND.maxBack = 500;
    SOUND.maxFront = 500;
    SOUND.minBack = 500;
    SOUND.minFront = 500;
    SOUND.priority = 0;
    SOUND.source = &SOUNDCLIP;
    SOUND.spatialize = true;
    NavigationInfo_74.m_parent = (X3dNode *)&Group_71;
    {
    static float m_avatarSize[] = { 0.250000, 1.600000, 0.750000 };
    NavigationInfo_74.avatarSize = m_avatarSize;
    NavigationInfo_74.avatarSize_length = sizeof(m_avatarSize) / sizeof(float);
    }
    NavigationInfo_74.headlight = true;
    NavigationInfo_74.speed = 1;
    {
    static const char* m_type[] = { "NONE" };
    NavigationInfo_74.type = m_type;
    NavigationInfo_74.type_length = sizeof(m_type) / sizeof(const char*);
    }
    NavigationInfo_74.visibilityLimit = 0;
    NavigationInfo_74.set_bind = false;
    NavigationInfo_74.isBound = false;
    ROLLm_TURN.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()            \n      {            \n      rotation_out[0]=1;            \n      rotation_out[1]=0;            \n      rotation_out[2]=0;            \n      rotation_out[3]=0;            \n      }            \n   function rock_and_roll_flag_in(val)\n      {\n      rock_and_roll_flag=val;\n      }\n   function xvalue_in(val)  \n      {\n      rotation_out[0]=0;\n      if (rock_and_roll_flag==1)\n         {\n         rotation_out[1]=0;\n         rotation_out[2]=1;\n         }\n      else\n         {\n         rotation_out[1]=1;\n         rotation_out[2]=0;\n         }\n      rotation_out[3]=-val/1.0;\n      }\n   ", "c++:\n   X3dScript_ROLL_TURN *self = (X3dScript_ROLL_TURN *)node;\n   static bool ROLL_TURN_init = false;\n   if (!ROLL_TURN_init) {\n       self->rotation_out[0]=1;\n       self->rotation_out[1]=0;\n       self->rotation_out[2]=0;\n       self->rotation_out[3]=0;\n       ROLL_TURN_init = true;\n   }         \n   self->rock_and_roll_flag = self->rock_and_roll_flag_in;\n   if (self->rock_and_roll_flag==1) {\n       self->rotation_out[1]=0;\n       self->rotation_out[2]=1;\n   } else {\n       self->rotation_out[1]=1;\n       self->rotation_out[2]=0;\n   }\n   self->rotation_out[0]=0;\n   self->rotation_out[3]=self->xvalue_in * 2.0;\n" };
    ROLLm_TURN.url = m_url;
    ROLLm_TURN.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ROLLm_TURN.directOutput = false;
    ROLLm_TURN.mustEvaluate = false;
    ROLLm_TURN.rock_and_roll_flag = 0;
    ROLLm_TURN.xvalue_in = 0;
    ROLLm_TURN.rock_and_roll_flag_in = 0;
    {
    static float m_rotation_out[] = { 0, 0, 1, 0 };
    ROLLm_TURN.rotation_out = m_rotation_out;
    ROLLm_TURN.rotation_out_length = sizeof(m_rotation_out) / sizeof(float);
    }
    UPm_DOWNm_TURN.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()            \n      {            \n      rotation_out[0]=0;            \n      rotation_out[1]=0;            \n      rotation_out[2]=0;            \n      rotation_out[3]=0;            \n      }            \n   function yvalue_in(val)  \n      {\n      rotation_out[0]=1;\n      rotation_out[1]=0;\n      rotation_out[2]=0;\n      rotation_out[3]=-val/1.0;\n      }\n   ", "c++:\n   X3dScript_UP_DOWN_TURN *self = (X3dScript_UP_DOWN_TURN *)node;\n   static bool UP_DOWN_TURN_init = false;\n   if (!UP_DOWN_TURN_init) {\n       self->rotation_out[0]=0;\n       self->rotation_out[1]=0;\n       self->rotation_out[2]=0;\n       self->rotation_out[3]=0;\n       UP_DOWN_TURN_init = true;\n   }         \n   self->rotation_out[0]=1;\n   self->rotation_out[1]=0;\n   self->rotation_out[2]=0;\n   self->rotation_out[3]=self->yvalue_in * 2.0;\n" };
    UPm_DOWNm_TURN.url = m_url;
    UPm_DOWNm_TURN.url_length = sizeof(m_url) / sizeof(const char*);
    }
    UPm_DOWNm_TURN.directOutput = false;
    UPm_DOWNm_TURN.mustEvaluate = false;
    UPm_DOWNm_TURN.yvalue_in = 0;
    {
    static float m_rotation_out[] = { 0, 0, 1, 0 };
    UPm_DOWNm_TURN.rotation_out = m_rotation_out;
    UPm_DOWNm_TURN.rotation_out_length = sizeof(m_rotation_out) / sizeof(float);
    }
    FLYm_FORWARD.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function initialize()\n      {\n      translation_out[0]=0;\n      translation_out[1]=0;\n      translation_out[2]=0;\n      }\n   function time_in(val)  \n      {\n      translation_out[0]=0;\n      translation_out[1]=0;\n      translation_out[2]=0.15;\n      time_out=val;\n      }\n   ", "c++:\n   X3dScript_FLY_FORWARD *self = (X3dScript_FLY_FORWARD *)node;\n   static bool FLY_FORWARD_init = false;\n   if (!FLY_FORWARD_init) {\n       self->translation_out[0]=0;\n       self->translation_out[1]=0;\n       self->translation_out[2]=0;\n       FLY_FORWARD_init = true;\n   }         \n   self->translation_out[0]=0;\n   self->translation_out[1]=0;\n   self->translation_out[2]=0.15;\n   self->time_out = self->time_in;\n" };
    FLYm_FORWARD.url = m_url;
    FLYm_FORWARD.url_length = sizeof(m_url) / sizeof(const char*);
    }
    FLYm_FORWARD.directOutput = false;
    FLYm_FORWARD.mustEvaluate = false;
    FLYm_FORWARD.time_in = 0;
    {
    static float m_translation_out[] = { 0, 0, 0 };
    FLYm_FORWARD.translation_out = m_translation_out;
    FLYm_FORWARD.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    FLYm_FORWARD.time_out = 0;
    FLYm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n\n // field SFNode metadata //\n\n // field SFVec3f top_translation //\n\n // field SFVec3f translation_old //\n\n // field SFBool brake //\n\n // field SFVec3f bullet_translation_start //\n\n // field SFVec3f bullet_direction //\n\n // field SFFloat bulletway_length //\n\n // field SFTime old_time //\n\n // field SFFloat factor //\n\n // eventOut SFVec3f translation_out //\n\n // eventOut SFVec3f top_translation_out //\n\n // eventOut SFInt32 bullet_choice_out //\n\n // eventOut SFVec3f bullet_translation_out //\n\n   function initialize()\n      {\n      bullet_choice_out=-1;\n      }\n   // stop movement \n   function exit_time(val)\n      {\n      brake=TRUE;\n      }\n   // restart movement\n   function enter_time(val)\n      {\n      brake=FALSE;\n      }\n   function time_in(val)\n      {\n      factor=5.0/Browser.getCurrentFrameRate();\n      old_time=val;\n      }\n   function fire_in(val)\n      {\n      if (val)\n         {\n         bullet_translation_start=top_translation_out;\n         // switch on bullet\n         bullet_choice_out=0;\n         bulletway_length=0;\n         }\n      }\n   function drive_bullet(val)\n      {\n      if (bulletway_length==0)\n         {\n         bullet_direction=val;\n         }\n      if ((bullet_translation_out[2]>10) ||\n          (bullet_translation_out[2]<-35))\n         {\n         bulletway_length=0;\n         bullet_choice_out=-1;\n         bullet_translation_out[2]=val[2];\n         }\n      else if (bulletway_length<45)\n         {\n         bullet_translation_out[0]=bullet_translation_start[0]-\n                                   bulletway_length*bullet_direction[0];\n         bullet_translation_out[1]=bullet_translation_start[1]-\n                                   bulletway_length*bullet_direction[1];\n         bullet_translation_out[2]=bullet_translation_start[2]-\n                                   bulletway_length*bullet_direction[2];\n         bulletway_length+=10.0*factor;\n         }\n      else\n         {\n         bulletway_length=0;\n         bullet_choice_out=-1;\n         bullet_translation_out[2]=val[2];\n         }\n      }\n   //\n   function translation_in(val)  \n      {\n      top_translation[0]=translation_old[0]-val[0]*factor;\n      top_translation[1]=translation_old[1]-val[1]*factor;\n      top_translation[2]=translation_old[2]-val[2]*factor;\n      if (top_translation[2]<-35)\n          top_translation[2] = 7;\n      if (top_translation[2]>7)\n          top_translation[2] = -35;\n      if (top_translation[0]<-7)\n          top_translation[0] = 7;\n      if (top_translation[0]>7)\n          top_translation[0] = -7;\n      if (top_translation[1]<-7)\n          top_translation[1] = 7;\n      if (top_translation[1]>7)\n          top_translation[1] = -7;\n      translation_old = top_translation;\n      translation_out[0]=translation_old[0]-val[0]*factor;\n      translation_out[1]=translation_old[1]-val[1]*factor;\n      translation_out[2]=translation_old[2]-val[2]*factor;\n      translation_old=translation_out;\n      if (bullet_choice_out==0)\n         drive_bullet(val);\n      top_translation_out=top_translation;\n      }\n   ", "c++:\n      X3dScript_FLY_ON *self = (X3dScript_FLY_ON *)node;\n      static bool FLY_ON_init = false;\n      if (!FLY_ON_init) {\n           self->bullet_choice_out = -1; \n           FLY_ON_init = true;\n      }         \n/*\n   function exit_time(val)\n      {\n      brake=TRUE;\n      }\n   // restart movement\n   function enter_time(val)\n      {\n      brake=FALSE;\n      }\n*/\n      self->factor = 0.06;\n      self->old_time = self->time_in;\n      if (self->fire_in)\n         {\n         self->bullet_translation_start = self->top_translation_out;\n         // switch on bullet\n         self->bullet_choice_out = 0;\n         self->bulletway_length = 0;\n         }\n      self->top_translation[0] = self->translation_old[0] -\n                                 self->translation_in[0] * self->factor;\n      self->top_translation[1] = self->translation_old[1] -\n                                 self->translation_in[1] * self->factor;\n      self->top_translation[2] = self->translation_old[2] -\n                                 self->translation_in[2] * self->factor;\n      if (self->top_translation[2] < -35)\n          self->top_translation[2] = 7;\n      if (self->top_translation[2] > 7)\n          self->top_translation[2] = -35;\n      if (self->top_translation[0] < -7)\n          self->top_translation[0] = 7;\n      if (self->top_translation[0] > 7)\n          self->top_translation[0] = -7;\n      if (self->top_translation[1] < -7)\n          self->top_translation[1] = 7;\n      if (self->top_translation[1] > 7)\n          self->top_translation[1] = -7;\n      self->translation_old = self->top_translation;\n      self->translation_out[0] = self->translation_old[0] -\n                                 self->translation_in[0] * self->factor;\n      self->translation_out[1] = self->translation_old[1] -\n                                 self->translation_in[1] * self->factor;\n      self->translation_out[2] = self->translation_old[2] - \n                                 self->translation_in[2] * self->factor;\n      self->translation_old[0] = self->translation_out[0];\n      self->translation_old[1] = self->translation_out[1];\n      self->translation_old[2] = self->translation_out[2];\n      if (self->bullet_choice_out == 0) \n         {\n         if (self->bulletway_length == 0)\n            {\n            self->bullet_direction = self->translation_in;\n            }\n         if ((self->bullet_translation_out[2] > 10) ||\n             (self->bullet_translation_out[2] < -35))\n            {\n            self->bulletway_length = 0;\n            self->bullet_choice_out = -1;\n            self->bullet_translation_out[2] = self->translation_in[2];\n            }\n         else if (self->bulletway_length < 45)\n            {\n            self->bullet_translation_out[0] = self->bullet_translation_start[0] -\n                                              self->bulletway_length *\n                                              self->bullet_direction[0];\n            self->bullet_translation_out[1] = self->bullet_translation_start[1] -\n                                              self->bulletway_length *\n                                              self->bullet_direction[1];\n            self->bullet_translation_out[2] = self->bullet_translation_start[2] -\n                                              self->bulletway_length *\n                                              self->bullet_direction[2];\n            self->bulletway_length += 10.0 * self->factor;\n            }\n         else\n            {\n            self->bulletway_length = 0;\n            self->bullet_choice_out = -1;\n            self->bullet_translation_out[2] = self->translation_in[2];\n            }\n         }\n      self->top_translation_out = self->top_translation;   \n" };
    FLYm_ON.url = m_url;
    FLYm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    FLYm_ON.directOutput = false;
    FLYm_ON.mustEvaluate = false;
    {
    static float m_top_translation[] = { -1, 0, 0 };
    FLYm_ON.top_translation = m_top_translation;
    FLYm_ON.top_translation_length = sizeof(m_top_translation) / sizeof(float);
    }
    {
    static float m_translation_old[] = { 0, 0, 0 };
    FLYm_ON.translation_old = m_translation_old;
    FLYm_ON.translation_old_length = sizeof(m_translation_old) / sizeof(float);
    }
    {
    static float m_bullet_translation_start[] = { 0, 0, 0 };
    FLYm_ON.bullet_translation_start = m_bullet_translation_start;
    FLYm_ON.bullet_translation_start_length = sizeof(m_bullet_translation_start) / sizeof(float);
    }
    {
    static float m_bullet_direction[] = { 0, 0, 0 };
    FLYm_ON.bullet_direction = m_bullet_direction;
    FLYm_ON.bullet_direction_length = sizeof(m_bullet_direction) / sizeof(float);
    }
    FLYm_ON.bulletway_length = 0;
    FLYm_ON.old_time = 0;
    FLYm_ON.factor = 10;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    FLYm_ON.translation_in = m_translation_in;
    FLYm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    FLYm_ON.time_in = 0;
    FLYm_ON.enter_time = 0;
    FLYm_ON.exit_time = 0;
    FLYm_ON.fire_in = false;
    {
    static float m_translation_out[] = { 0, 0, 0 };
    FLYm_ON.translation_out = m_translation_out;
    FLYm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    {
    static float m_top_translation_out[] = { 0, 0, 0 };
    FLYm_ON.top_translation_out = m_top_translation_out;
    FLYm_ON.top_translation_out_length = sizeof(m_top_translation_out) / sizeof(float);
    }
    FLYm_ON.bullet_choice_out = 0;
    {
    static float m_bullet_translation_out[] = { 0, 0, 0 };
    FLYm_ON.bullet_translation_out = m_bullet_translation_out;
    FLYm_ON.bullet_translation_out_length = sizeof(m_bullet_translation_out) / sizeof(float);
    }
    THINGm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0.1;\n         hide_out[1]=0.1;\n         hide_out[2]=0.1;\n         }\n      else\n         {\n         hide_out[0]=1.0;\n         hide_out[1]=1.0;\n         hide_out[2]=1.0;\n         }\n      }    \n   function fire_in(val)            \n      {\n      if (val)\n         {\n         hide_out[0]=1;\n         hide_out[1]=1;\n         hide_out[2]=1;\n         }                  \n      }\n   ", "c++:\n      X3dScript_THING_SCRIPT *self = (X3dScript_THING_SCRIPT *)node;\n      if (self->isover_in)\n         {\n         self->hide_out[0]=0.1;\n         self->hide_out[1]=0.1;\n         self->hide_out[2]=0.1;\n         }\n      else\n         {\n         self->hide_out[0]=1.0;\n         self->hide_out[1]=1.0;\n         self->hide_out[2]=1.0;\n         }\n      if (self->fire_in)\n         {\n         self->hide_out[0]=1;\n         self->hide_out[1]=1;\n         self->hide_out[2]=1;\n         }                  \n" };
    THINGm_SCRIPT.url = m_url;
    THINGm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    THINGm_SCRIPT.directOutput = false;
    THINGm_SCRIPT.mustEvaluate = false;
    THINGm_SCRIPT.isover_in = false;
    THINGm_SCRIPT.fire_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    THINGm_SCRIPT.hide_out = m_hide_out;
    THINGm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    Viewpoint_75.m_parent = (X3dNode *)&Group_71;
    Viewpoint_75.description = "";
    Viewpoint_75.fieldOfView = 0.785398;
    Viewpoint_75.jump = true;
    {
    static float m_orientation[] = { 0.216404, 6.832186e-2, -0.973910, 0 };
    Viewpoint_75.orientation = m_orientation;
    Viewpoint_75.orientation_length = sizeof(m_orientation) / sizeof(float);
    }
    {
    static float m_position[] = { 1.070576e-4, -3.391865e-4, 10 };
    Viewpoint_75.position = m_position;
    Viewpoint_75.position_length = sizeof(m_position) / sizeof(float);
    }
    Viewpoint_75.set_bind = false;
    Viewpoint_75.bindTime = 0;
    Viewpoint_75.isBound = false;
    Viewpoint_76.m_parent = (X3dNode *)&Group_71;
    Viewpoint_76.description = "";
    Viewpoint_76.fieldOfView = 0.785398;
    Viewpoint_76.jump = true;
    {
    static float m_orientation[] = { 0, 0, 1, 0 };
    Viewpoint_76.orientation = m_orientation;
    Viewpoint_76.orientation_length = sizeof(m_orientation) / sizeof(float);
    }
    {
    static float m_position[] = { 0, 0, 10 };
    Viewpoint_76.position = m_position;
    Viewpoint_76.position_length = sizeof(m_position) / sizeof(float);
    }
    Viewpoint_76.set_bind = false;
    Viewpoint_76.bindTime = 0;
    Viewpoint_76.isBound = false;
    WorldInfo_77.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA" };
    WorldInfo_77.info = m_info;
    WorldInfo_77.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo_77.title = "Tux warbird";
    Transform_78.route_sources =  NULL;
    Transform_78.num_route_source = 0;
    Transform_78.m_parent = (X3dNode *)&Group_71;
    Shape_79.m_parent = (X3dNode *)&Transform_78;
    Appearance_80.m_parent = (X3dNode *)&Shape_79;
    Material_81.m_parent = (X3dNode *)&Appearance_80;
    Material_81.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_81.diffuseColor = m_diffuseColor;
    Material_81.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_81.emissiveColor = m_emissiveColor;
    Material_81.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_81.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_81.specularColor = m_specularColor;
    Material_81.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_81.transparency = 0;
    Appearance_80.material = &Material_81;
    Appearance_80.texture = NULL;
    Appearance_80.textureTransform = NULL;
    Text1.setGlName(14);
    Text1.m_parent = (X3dNode *)&Shape_79;
    Coordinate13.m_parent = (X3dNode *)&Text1;
    {
    static float m_point[1];
    Coordinate13.point = NULL;
    Coordinate13.point_length = 0;
    }
    Text1.color = NULL;
    Text1.coord = &Coordinate13;
    Text1.normal = NULL;
    Text1.texCoord = NULL;
    Text1.ccw = false;
    {
    static int m_colorIndex[1];
    Text1.colorIndex = NULL;
    Text1.colorIndex_length = 0;
    }
    Text1.colorPerVertex = false;
    Text1.convex = false;
    {
    static int m_coordIndex[1];
    Text1.coordIndex = NULL;
    Text1.coordIndex_length = 0;
    }
    Text1.creaseAngle = 0;
    {
    static int m_normalIndex[1];
    Text1.normalIndex = NULL;
    Text1.normalIndex_length = 0;
    }
    Text1.normalPerVertex = false;
    Text1.solid = false;
    {
    static int m_texCoordIndex[1];
    Text1.texCoordIndex = NULL;
    Text1.texCoordIndex_length = 0;
    }
    {
    static int m_set_colorIndex[1];
    Text1.set_colorIndex = NULL;
    Text1.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    Text1.set_coordIndex = NULL;
    Text1.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    Text1.set_normalIndex = NULL;
    Text1.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    Text1.set_texCoordIndex = NULL;
    Text1.set_texCoordIndex_length = 0;
    }
    Shape_79.appearance = &Appearance_80;
    Shape_79.geometry = &Text1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_78.rotation = m_rotation;
    Transform_78.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -5.552411, -2.600564, 0 };
    Transform_78.translation = m_translation;
    Transform_78.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_78.bboxCenter = m_bboxCenter;
    Transform_78.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_78.bboxSize = m_bboxSize;
    Transform_78.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_78.center = m_center;
    Transform_78.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_78.scale = m_scale;
    Transform_78.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_78.scaleOrientation = m_scaleOrientation;
    Transform_78.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_79 };
    Transform_78.children = m_children;
    Transform_78.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_78.addChildren = NULL;
    Transform_78.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_78.removeChildren = NULL;
    Transform_78.removeChildren_length = 0;
    }
    Group_82.m_parent = (X3dNode *)&Group_71;
    Switch1.m_parent = (X3dNode *)&Group_82;
    Transform_83.route_sources =  NULL;
    Transform_83.num_route_source = 0;
    Transform_83.m_parent = (X3dNode *)&Switch1;
    Shape_84.m_parent = (X3dNode *)&Transform_83;
    Appearance_85.m_parent = (X3dNode *)&Shape_84;
    Material_86.m_parent = (X3dNode *)&Appearance_85;
    Material_86.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_86.diffuseColor = m_diffuseColor;
    Material_86.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_86.emissiveColor = m_emissiveColor;
    Material_86.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_86.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_86.specularColor = m_specularColor;
    Material_86.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_86.transparency = 0;
    Appearance_85.material = &Material_86;
    Appearance_85.texture = NULL;
    Appearance_85.textureTransform = NULL;
    Text1m_5.setGlName(15);
    Text1m_5.m_parent = (X3dNode *)&Shape_84;
    Coordinate14.m_parent = (X3dNode *)&Text1m_5;
    {
    static float m_point[] = { 0.473438, 0.367969, 0, 0.490251, 0.360604, 0, 0.473438, 0.367969, 0, 0.473438, 0.367969, 0, 0.490251, 0.360604, 0, 0.490251, 0.360604, 0, 0.490251, 0.360604, 0, 0.506836, 0.349967, 0, 0.490251, 0.360604, 0, 0.490251, 0.360604, 0, 0.506836, 0.349967, 0, 0.506836, 0.349967, 0, 0.506836, 0.349967, 0, 0.523193, 0.336060, 0, 0.506836, 0.349967, 0, 0.506836, 0.349967, 0, 0.523193, 0.336060, 0, 0.523193, 0.336060, 0, 0.523193, 0.336060, 0, 0.539323, 0.318880, 0, 0.523193, 0.336060, 0, 0.523193, 0.336060, 0, 0.539323, 0.318880, 0, 0.539323, 0.318880, 0, 0.539323, 0.318880, 0, 0.555355, 0.298315, 0, 0.539323, 0.318880, 0, 0.539323, 0.318880, 0, 0.555355, 0.298315, 0, 0.555355, 0.298315, 0, 0.555355, 0.298315, 0, 0.571419, 0.274251, 0, 0.555355, 0.298315, 0, 0.555355, 0.298315, 0, 0.571419, 0.274251, 0, 0.571419, 0.274251, 0, 0.571419, 0.274251, 0, 0.587516, 0.246688, 0, 0.571419, 0.274251, 0, 0.571419, 0.274251, 0, 0.587516, 0.246688, 0, 0.587516, 0.246688, 0, 0.587516, 0.246688, 0, 0.603646, 0.215625, 0, 0.587516, 0.246688, 0, 0.587516, 0.246688, 0, 0.603646, 0.215625, 0, 0.603646, 0.215625, 0, 0.603646, 0.215625, 0, 0.710417, 0, 0, 0.603646, 0.215625, 0, 0.603646, 0.215625, 0, 0.710417, 0, 0, 0.710417, 0, 0, 0.710417, 0, 0, 0.597396, 0, 0, 0.710417, 0, 0, 0.710417, 0, 0, 0.597396, 0, 0, 0.597396, 0, 0, 0.597396, 0, 0, 0.497917, 0.202344, 0, 0.597396, 0, 0, 0.597396, 0, 0, 0.497917, 0.202344, 0, 0.497917, 0.202344, 0, 0.497917, 0.202344, 0, 0.478792, 0.238599, 0, 0.497917, 0.202344, 0, 0.497917, 0.202344, 0, 0.478792, 0.238599, 0, 0.478792, 0.238599, 0, 0.478792, 0.238599, 0, 0.459961, 0.268197, 0, 0.478792, 0.238599, 0, 0.478792, 0.238599, 0, 0.459961, 0.268197, 0, 0.459961, 0.268197, 0, 0.459961, 0.268197, 0, 0.441423, 0.291138, 0, 0.459961, 0.268197, 0, 0.459961, 0.268197, 0, 0.441423, 0.291138, 0, 0.441423, 0.291138, 0, 0.441423, 0.291138, 0, 0.423177, 0.307422, 0, 0.441423, 0.291138, 0, 0.441423, 0.291138, 0, 0.423177, 0.307422, 0, 0.423177, 0.307422, 0, 0.423177, 0.307422, 0, 0.403434, 0.318758, 0, 0.423177, 0.307422, 0, 0.423177, 0.307422, 0, 0.403434, 0.318758, 0, 0.403434, 0.318758, 0, 0.403434, 0.318758, 0, 0.380404, 0.326855, 0, 0.403434, 0.318758, 0, 0.403434, 0.318758, 0, 0.380404, 0.326855, 0, 0.380404, 0.326855, 0, 0.380404, 0.326855, 0, 0.354085, 0.331714, 0, 0.380404, 0.326855, 0, 0.380404, 0.326855, 0, 0.354085, 0.331714, 0, 0.354085, 0.331714, 0, 0.354085, 0.331714, 0, 0.324479, 0.333333, 0, 0.354085, 0.331714, 0, 0.354085, 0.331714, 0, 0.324479, 0.333333, 0, 0.324479, 0.333333, 0, 0.324479, 0.333333, 0, 0.209896, 0.333333, 0, 0.324479, 0.333333, 0, 0.324479, 0.333333, 0, 0.209896, 0.333333, 0, 0.209896, 0.333333, 0, 0.209896, 0.333333, 0, 0.209896, 0, 0, 0.209896, 0.333333, 0, 0.209896, 0.333333, 0, 0.209896, 0, 0, 0.209896, 0, 0, 0.209896, 0, 0, 0.104688, 0, 0, 0.209896, 0, 0, 0.209896, 0, 0, 0.104688, 0, 0, 0.104688, 0, 0, 0.104688, 0, 0, 0.104688, 0.783333, 0, 0.104688, 0, 0, 0.104688, 0, 0, 0.104688, 0.783333, 0, 0.104688, 0.783333, 0, 0.104688, 0.783333, 0, 0.342187, 0.783333, 0, 0.104688, 0.783333, 0, 0.104688, 0.783333, 0, 0.342187, 0.783333, 0, 0.342187, 0.783333, 0, 0.342187, 0.783333, 0, 0.404622, 0.779834, 0, 0.342187, 0.783333, 0, 0.342187, 0.783333, 0, 0.404622, 0.779834, 0, 0.404622, 0.779834, 0, 0.404622, 0.779834, 0, 0.458594, 0.769336, 0, 0.404622, 0.779834, 0, 0.404622, 0.779834, 0, 0.458594, 0.769336, 0, 0.458594, 0.769336, 0, 0.458594, 0.769336, 0, 0.504102, 0.751839, 0, 0.458594, 0.769336, 0, 0.458594, 0.769336, 0, 0.504102, 0.751839, 0, 0.504102, 0.751839, 0, 0.504102, 0.751839, 0, 0.541146, 0.727344, 0, 0.504102, 0.751839, 0, 0.504102, 0.751839, 0, 0.541146, 0.727344, 0, 0.541146, 0.727344, 0, 0.541146, 0.727344, 0, 0.569857, 0.695768, 0, 0.541146, 0.727344, 0, 0.541146, 0.727344, 0, 0.569857, 0.695768, 0, 0.569857, 0.695768, 0, 0.569857, 0.695768, 0, 0.590365, 0.657031, 0, 0.569857, 0.695768, 0, 0.569857, 0.695768, 0, 0.590365, 0.657031, 0, 0.590365, 0.657031, 0, 0.590365, 0.657031, 0, 0.602669, 0.611133, 0, 0.590365, 0.657031, 0, 0.590365, 0.657031, 0, 0.602669, 0.611133, 0, 0.602669, 0.611133, 0, 0.602669, 0.611133, 0, 0.606771, 0.558073, 0, 0.602669, 0.611133, 0, 0.602669, 0.611133, 0, 0.606771, 0.558073, 0, 0.606771, 0.558073, 0, 0.606771, 0.558073, 0, 0.604639, 0.522778, 0, 0.606771, 0.558073, 0, 0.606771, 0.558073, 0, 0.604639, 0.522778, 0, 0.604639, 0.522778, 0, 0.604639, 0.522778, 0, 0.598242, 0.490592, 0, 0.604639, 0.522778, 0, 0.604639, 0.522778, 0, 0.598242, 0.490592, 0, 0.598242, 0.490592, 0, 0.598242, 0.490592, 0, 0.587581, 0.461515, 0, 0.598242, 0.490592, 0, 0.598242, 0.490592, 0, 0.587581, 0.461515, 0, 0.587581, 0.461515, 0, 0.587581, 0.461515, 0, 0.572656, 0.435547, 0, 0.587581, 0.461515, 0, 0.587581, 0.461515, 0, 0.572656, 0.435547, 0, 0.572656, 0.435547, 0, 0.572656, 0.435547, 0, 0.553662, 0.413013, 0, 0.572656, 0.435547, 0, 0.572656, 0.435547, 0, 0.553662, 0.413013, 0, 0.553662, 0.413013, 0, 0.553662, 0.413013, 0, 0.530794, 0.394238, 0, 0.553662, 0.413013, 0, 0.553662, 0.413013, 0, 0.530794, 0.394238, 0, 0.530794, 0.394238, 0, 0.530794, 0.394238, 0, 0.504053, 0.379224, 0, 0.530794, 0.394238, 0, 0.530794, 0.394238, 0, 0.504053, 0.379224, 0, 0.504053, 0.379224, 0, 0.104688, 0.783333, 0, 0.209896, 0.700000, 0, 0.342187, 0.783333, 0, 0.209896, 0.700000, 0, 0.104688, 0.783333, 0, 0.342187, 0.783333, 0, 0.209896, 0.700000, 0, 0.104688, 0.783333, 0, 0.209896, 0.416667, 0, 0.104688, 0.783333, 0, 0.209896, 0.700000, 0, 0.209896, 0.416667, 0, 0.104688, 0.783333, 0, 0.104688, 0, 0, 0.209896, 0.416667, 0, 0.104688, 0, 0, 0.104688, 0.783333, 0, 0.209896, 0.416667, 0, 0.104688, 0, 0, 0.209896, 0.333333, 0, 0.209896, 0.416667, 0, 0.209896, 0.333333, 0, 0.104688, 0, 0, 0.209896, 0.416667, 0, 0.104688, 0, 0, 0.209896, 0, 0, 0.209896, 0.333333, 0, 0.209896, 0, 0, 0.104688, 0, 0, 0.209896, 0.333333, 0, 0.209896, 0.416667, 0, 0.209896, 0.333333, 0, 0.324479, 0.333333, 0, 0.209896, 0.333333, 0, 0.209896, 0.416667, 0, 0.324479, 0.333333, 0, 0.209896, 0.416667, 0, 0.324479, 0.333333, 0, 0.342187, 0.416667, 0, 0.324479, 0.333333, 0, 0.209896, 0.416667, 0, 0.342187, 0.416667, 0, 0.342187, 0.416667, 0, 0.324479, 0.333333, 0, 0.354085, 0.331714, 0, 0.324479, 0.333333, 0, 0.342187, 0.416667, 0, 0.354085, 0.331714, 0, 0.342187, 0.416667, 0, 0.354085, 0.331714, 0, 0.377881, 0.418921, 0, 0.354085, 0.331714, 0, 0.342187, 0.416667, 0, 0.377881, 0.418921, 0, 0.377881, 0.418921, 0, 0.354085, 0.331714, 0, 0.380404, 0.326855, 0, 0.354085, 0.331714, 0, 0.377881, 0.418921, 0, 0.380404, 0.326855, 0, 0.377881, 0.418921, 0, 0.380404, 0.326855, 0, 0.408919, 0.425684, 0, 0.380404, 0.326855, 0, 0.377881, 0.418921, 0, 0.408919, 0.425684, 0, 0.380404, 0.326855, 0, 0.403434, 0.318758, 0, 0.408919, 0.425684, 0, 0.403434, 0.318758, 0, 0.380404, 0.326855, 0, 0.408919, 0.425684, 0, 0.473438, 0.367969, 0, 0.408919, 0.425684, 0, 0.403434, 0.318758, 0, 0.408919, 0.425684, 0, 0.473438, 0.367969, 0, 0.403434, 0.318758, 0, 0.435303, 0.436955, 0, 0.408919, 0.425684, 0, 0.473438, 0.367969, 0, 0.408919, 0.425684, 0, 0.435303, 0.436955, 0, 0.473438, 0.367969, 0, 0.457031, 0.452734, 0, 0.435303, 0.436955, 0, 0.473438, 0.367969, 0, 0.435303, 0.436955, 0, 0.457031, 0.452734, 0, 0.473438, 0.367969, 0, 0.457031, 0.452734, 0, 0.473438, 0.367969, 0, 0.504053, 0.379224, 0, 0.473438, 0.367969, 0, 0.457031, 0.452734, 0, 0.504053, 0.379224, 0, 0.457031, 0.452734, 0, 0.504053, 0.379224, 0, 0.530794, 0.394238, 0, 0.504053, 0.379224, 0, 0.457031, 0.452734, 0, 0.530794, 0.394238, 0, 0.474007, 0.472892, 0, 0.457031, 0.452734, 0, 0.530794, 0.394238, 0, 0.457031, 0.452734, 0, 0.474007, 0.472892, 0, 0.530794, 0.394238, 0, 0.474007, 0.472892, 0, 0.530794, 0.394238, 0, 0.553662, 0.413013, 0, 0.530794, 0.394238, 0, 0.474007, 0.472892, 0, 0.553662, 0.413013, 0, 0.474007, 0.472892, 0, 0.553662, 0.413013, 0, 0.572656, 0.435547, 0, 0.553662, 0.413013, 0, 0.474007, 0.472892, 0, 0.572656, 0.435547, 0, 0.486133, 0.497298, 0, 0.474007, 0.472892, 0, 0.572656, 0.435547, 0, 0.474007, 0.472892, 0, 0.486133, 0.497298, 0, 0.572656, 0.435547, 0, 0.486133, 0.497298, 0, 0.572656, 0.435547, 0, 0.587581, 0.461515, 0, 0.572656, 0.435547, 0, 0.486133, 0.497298, 0, 0.587581, 0.461515, 0, 0.598242, 0.490592, 0, 0.486133, 0.497298, 0, 0.587581, 0.461515, 0, 0.486133, 0.497298, 0, 0.598242, 0.490592, 0, 0.587581, 0.461515, 0, 0.598242, 0.490592, 0, 0.493408, 0.525952, 0, 0.486133, 0.497298, 0, 0.493408, 0.525952, 0, 0.598242, 0.490592, 0, 0.486133, 0.497298, 0, 0.493408, 0.525952, 0, 0.598242, 0.490592, 0, 0.604639, 0.522778, 0, 0.598242, 0.490592, 0, 0.493408, 0.525952, 0, 0.604639, 0.522778, 0, 0.495833, 0.558854, 0, 0.493408, 0.525952, 0, 0.604639, 0.522778, 0, 0.493408, 0.525952, 0, 0.495833, 0.558854, 0, 0.604639, 0.522778, 0, 0.495833, 0.558854, 0, 0.604639, 0.522778, 0, 0.606771, 0.558073, 0, 0.604639, 0.522778, 0, 0.495833, 0.558854, 0, 0.606771, 0.558073, 0, 0.606771, 0.558073, 0, 0.493408, 0.591724, 0, 0.495833, 0.558854, 0, 0.493408, 0.591724, 0, 0.606771, 0.558073, 0, 0.495833, 0.558854, 0, 0.602669, 0.611133, 0, 0.493408, 0.591724, 0, 0.606771, 0.558073, 0, 0.493408, 0.591724, 0, 0.602669, 0.611133, 0, 0.606771, 0.558073, 0, 0.602669, 0.611133, 0, 0.486133, 0.620280, 0, 0.493408, 0.591724, 0, 0.486133, 0.620280, 0, 0.602669, 0.611133, 0, 0.493408, 0.591724, 0, 0.590365, 0.657031, 0, 0.486133, 0.620280, 0, 0.602669, 0.611133, 0, 0.486133, 0.620280, 0, 0.590365, 0.657031, 0, 0.602669, 0.611133, 0, 0.569857, 0.695768, 0, 0.486133, 0.620280, 0, 0.590365, 0.657031, 0, 0.486133, 0.620280, 0, 0.569857, 0.695768, 0, 0.590365, 0.657031, 0, 0.474007, 0.644523, 0, 0.486133, 0.620280, 0, 0.569857, 0.695768, 0, 0.486133, 0.620280, 0, 0.474007, 0.644523, 0, 0.569857, 0.695768, 0, 0.541146, 0.727344, 0, 0.474007, 0.644523, 0, 0.569857, 0.695768, 0, 0.474007, 0.644523, 0, 0.541146, 0.727344, 0, 0.569857, 0.695768, 0, 0.457031, 0.664453, 0, 0.474007, 0.644523, 0, 0.541146, 0.727344, 0, 0.474007, 0.644523, 0, 0.457031, 0.664453, 0, 0.541146, 0.727344, 0, 0.504102, 0.751839, 0, 0.457031, 0.664453, 0, 0.541146, 0.727344, 0, 0.457031, 0.664453, 0, 0.504102, 0.751839, 0, 0.541146, 0.727344, 0, 0.435303, 0.680005, 0, 0.457031, 0.664453, 0, 0.504102, 0.751839, 0, 0.457031, 0.664453, 0, 0.435303, 0.680005, 0, 0.504102, 0.751839, 0, 0.458594, 0.769336, 0, 0.435303, 0.680005, 0, 0.504102, 0.751839, 0, 0.435303, 0.680005, 0, 0.458594, 0.769336, 0, 0.504102, 0.751839, 0, 0.408919, 0.691113, 0, 0.435303, 0.680005, 0, 0.458594, 0.769336, 0, 0.435303, 0.680005, 0, 0.408919, 0.691113, 0, 0.458594, 0.769336, 0, 0.404622, 0.779834, 0, 0.408919, 0.691113, 0, 0.458594, 0.769336, 0, 0.408919, 0.691113, 0, 0.404622, 0.779834, 0, 0.458594, 0.769336, 0, 0.404622, 0.779834, 0, 0.377881, 0.697778, 0, 0.408919, 0.691113, 0, 0.377881, 0.697778, 0, 0.404622, 0.779834, 0, 0.408919, 0.691113, 0, 0.342187, 0.783333, 0, 0.377881, 0.697778, 0, 0.404622, 0.779834, 0, 0.377881, 0.697778, 0, 0.342187, 0.783333, 0, 0.404622, 0.779834, 0, 0.342187, 0.783333, 0, 0.342187, 0.700000, 0, 0.377881, 0.697778, 0, 0.342187, 0.700000, 0, 0.342187, 0.783333, 0, 0.377881, 0.697778, 0, 0.209896, 0.700000, 0, 0.342187, 0.700000, 0, 0.342187, 0.783333, 0, 0.342187, 0.700000, 0, 0.209896, 0.700000, 0, 0.342187, 0.783333, 0, 0.473438, 0.367969, 0, 0.403434, 0.318758, 0, 0.423177, 0.307422, 0, 0.403434, 0.318758, 0, 0.473438, 0.367969, 0, 0.423177, 0.307422, 0, 0.473438, 0.367969, 0, 0.423177, 0.307422, 0, 0.441423, 0.291138, 0, 0.423177, 0.307422, 0, 0.473438, 0.367969, 0, 0.441423, 0.291138, 0, 0.490251, 0.360604, 0, 0.473438, 0.367969, 0, 0.441423, 0.291138, 0, 0.473438, 0.367969, 0, 0.490251, 0.360604, 0, 0.441423, 0.291138, 0, 0.490251, 0.360604, 0, 0.441423, 0.291138, 0, 0.506836, 0.349967, 0, 0.441423, 0.291138, 0, 0.490251, 0.360604, 0, 0.506836, 0.349967, 0, 0.506836, 0.349967, 0, 0.441423, 0.291138, 0, 0.523193, 0.336060, 0, 0.441423, 0.291138, 0, 0.506836, 0.349967, 0, 0.523193, 0.336060, 0, 0.441423, 0.291138, 0, 0.459961, 0.268197, 0, 0.523193, 0.336060, 0, 0.459961, 0.268197, 0, 0.441423, 0.291138, 0, 0.523193, 0.336060, 0, 0.523193, 0.336060, 0, 0.459961, 0.268197, 0, 0.539323, 0.318880, 0, 0.459961, 0.268197, 0, 0.523193, 0.336060, 0, 0.539323, 0.318880, 0, 0.539323, 0.318880, 0, 0.459961, 0.268197, 0, 0.555355, 0.298315, 0, 0.459961, 0.268197, 0, 0.539323, 0.318880, 0, 0.555355, 0.298315, 0, 0.459961, 0.268197, 0, 0.478792, 0.238599, 0, 0.555355, 0.298315, 0, 0.478792, 0.238599, 0, 0.459961, 0.268197, 0, 0.555355, 0.298315, 0, 0.555355, 0.298315, 0, 0.478792, 0.238599, 0, 0.571419, 0.274251, 0, 0.478792, 0.238599, 0, 0.555355, 0.298315, 0, 0.571419, 0.274251, 0, 0.478792, 0.238599, 0, 0.497917, 0.202344, 0, 0.571419, 0.274251, 0, 0.497917, 0.202344, 0, 0.478792, 0.238599, 0, 0.571419, 0.274251, 0, 0.571419, 0.274251, 0, 0.497917, 0.202344, 0, 0.587516, 0.246688, 0, 0.497917, 0.202344, 0, 0.571419, 0.274251, 0, 0.587516, 0.246688, 0, 0.587516, 0.246688, 0, 0.497917, 0.202344, 0, 0.603646, 0.215625, 0, 0.497917, 0.202344, 0, 0.587516, 0.246688, 0, 0.603646, 0.215625, 0, 0.497917, 0.202344, 0, 0.597396, 0, 0, 0.603646, 0.215625, 0, 0.597396, 0, 0, 0.497917, 0.202344, 0, 0.603646, 0.215625, 0, 0.603646, 0.215625, 0, 0.597396, 0, 0, 0.710417, 0, 0, 0.597396, 0, 0, 0.603646, 0.215625, 0, 0.710417, 0, 0, 0.209896, 0.700000, 0, 0.209896, 0.416667, 0, 0.209896, 0.700000, 0, 0.209896, 0.700000, 0, 0.209896, 0.416667, 0, 0.209896, 0.416667, 0, 0.209896, 0.416667, 0, 0.342187, 0.416667, 0, 0.209896, 0.416667, 0, 0.209896, 0.416667, 0, 0.342187, 0.416667, 0, 0.342187, 0.416667, 0, 0.342187, 0.416667, 0, 0.377881, 0.418921, 0, 0.342187, 0.416667, 0, 0.342187, 0.416667, 0, 0.377881, 0.418921, 0, 0.377881, 0.418921, 0, 0.377881, 0.418921, 0, 0.408919, 0.425684, 0, 0.377881, 0.418921, 0, 0.377881, 0.418921, 0, 0.408919, 0.425684, 0, 0.408919, 0.425684, 0, 0.408919, 0.425684, 0, 0.435303, 0.436955, 0, 0.408919, 0.425684, 0, 0.408919, 0.425684, 0, 0.435303, 0.436955, 0, 0.435303, 0.436955, 0, 0.435303, 0.436955, 0, 0.457031, 0.452734, 0, 0.435303, 0.436955, 0, 0.435303, 0.436955, 0, 0.457031, 0.452734, 0, 0.457031, 0.452734, 0, 0.457031, 0.452734, 0, 0.474007, 0.472892, 0, 0.457031, 0.452734, 0, 0.457031, 0.452734, 0, 0.474007, 0.472892, 0, 0.474007, 0.472892, 0, 0.474007, 0.472892, 0, 0.486133, 0.497298, 0, 0.474007, 0.472892, 0, 0.474007, 0.472892, 0, 0.486133, 0.497298, 0, 0.486133, 0.497298, 0, 0.486133, 0.497298, 0, 0.493408, 0.525952, 0, 0.486133, 0.497298, 0, 0.486133, 0.497298, 0, 0.493408, 0.525952, 0, 0.493408, 0.525952, 0, 0.493408, 0.525952, 0, 0.495833, 0.558854, 0, 0.493408, 0.525952, 0, 0.493408, 0.525952, 0, 0.495833, 0.558854, 0, 0.495833, 0.558854, 0, 0.495833, 0.558854, 0, 0.493408, 0.591724, 0, 0.495833, 0.558854, 0, 0.495833, 0.558854, 0, 0.493408, 0.591724, 0, 0.493408, 0.591724, 0, 0.493408, 0.591724, 0, 0.486133, 0.620280, 0, 0.493408, 0.591724, 0, 0.493408, 0.591724, 0, 0.486133, 0.620280, 0, 0.486133, 0.620280, 0, 0.486133, 0.620280, 0, 0.474007, 0.644523, 0, 0.486133, 0.620280, 0, 0.486133, 0.620280, 0, 0.474007, 0.644523, 0, 0.474007, 0.644523, 0, 0.474007, 0.644523, 0, 0.457031, 0.664453, 0, 0.474007, 0.644523, 0, 0.474007, 0.644523, 0, 0.457031, 0.664453, 0, 0.457031, 0.664453, 0, 0.457031, 0.664453, 0, 0.435303, 0.680005, 0, 0.457031, 0.664453, 0, 0.457031, 0.664453, 0, 0.435303, 0.680005, 0, 0.435303, 0.680005, 0, 0.435303, 0.680005, 0, 0.408919, 0.691113, 0, 0.435303, 0.680005, 0, 0.435303, 0.680005, 0, 0.408919, 0.691113, 0, 0.408919, 0.691113, 0, 0.408919, 0.691113, 0, 0.377881, 0.697778, 0, 0.408919, 0.691113, 0, 0.408919, 0.691113, 0, 0.377881, 0.697778, 0, 0.377881, 0.697778, 0, 0.377881, 0.697778, 0, 0.342187, 0.700000, 0, 0.377881, 0.697778, 0, 0.377881, 0.697778, 0, 0.342187, 0.700000, 0, 0.342187, 0.700000, 0, 1.282812, 0.318750, 0, 1.282812, 0.266667, 0, 1.282812, 0.318750, 0, 1.282812, 0.318750, 0, 1.282812, 0.266667, 0, 1.282812, 0.266667, 0, 1.282812, 0.266667, 0, 0.842188, 0.266667, 0, 1.282812, 0.266667, 0, 1.282812, 0.266667, 0, 0.842188, 0.266667, 0, 0.842188, 0.266667, 0, 0.842188, 0.266667, 0, 0.848258, 0.220744, 0, 0.842188, 0.266667, 0, 0.842188, 0.266667, 0, 0.848258, 0.220744, 0, 0.848258, 0.220744, 0, 0.848258, 0.220744, 0, 0.860221, 0.180632, 0, 0.848258, 0.220744, 0, 0.848258, 0.220744, 0, 0.860221, 0.180632, 0, 0.860221, 0.180632, 0, 0.860221, 0.180632, 0, 0.878076, 0.146330, 0, 0.860221, 0.180632, 0, 0.860221, 0.180632, 0, 0.878076, 0.146330, 0, 0.878076, 0.146330, 0, 0.878076, 0.146330, 0, 0.901823, 0.117839, 0, 0.878076, 0.146330, 0, 0.878076, 0.146330, 0, 0.901823, 0.117839, 0, 0.901823, 0.117839, 0, 0.901823, 0.117839, 0, 0.931136, 0.095451, 0, 0.901823, 0.117839, 0, 0.901823, 0.117839, 0, 0.931136, 0.095451, 0, 0.931136, 0.095451, 0, 0.931136, 0.095451, 0, 0.965690, 0.079460, 0, 0.931136, 0.095451, 0, 0.931136, 0.095451, 0, 0.965690, 0.079460, 0, 0.965690, 0.079460, 0, 0.965690, 0.079460, 0, 1.005485, 0.069865, 0, 0.965690, 0.079460, 0, 0.965690, 0.079460, 0, 1.005485, 0.069865, 0, 1.005485, 0.069865, 0, 1.005485, 0.069865, 0, 1.050521, 0.066667, 0, 1.005485, 0.069865, 0, 1.005485, 0.069865, 0, 1.050521, 0.066667, 0, 1.050521, 0.066667, 0, 1.050521, 0.066667, 0, 1.077913, 0.067448, 0, 1.050521, 0.066667, 0, 1.050521, 0.066667, 0, 1.077913, 0.067448, 0, 1.077913, 0.067448, 0, 1.077913, 0.067448, 0, 1.104883, 0.069792, 0, 1.077913, 0.067448, 0, 1.077913, 0.067448, 0, 1.104883, 0.069792, 0, 1.104883, 0.069792, 0, 1.104883, 0.069792, 0, 1.131429, 0.073698, 0, 1.104883, 0.069792, 0, 1.104883, 0.069792, 0, 1.131429, 0.073698, 0, 1.131429, 0.073698, 0, 1.131429, 0.073698, 0, 1.157552, 0.079167, 0, 1.131429, 0.073698, 0, 1.131429, 0.073698, 0, 1.157552, 0.079167, 0, 1.157552, 0.079167, 0, 1.157552, 0.079167, 0, 1.183415, 0.086198, 0, 1.157552, 0.079167, 0, 1.157552, 0.079167, 0, 1.183415, 0.086198, 0, 1.183415, 0.086198, 0, 1.183415, 0.086198, 0, 1.209180, 0.094792, 0, 1.183415, 0.086198, 0, 1.183415, 0.086198, 0, 1.209180, 0.094792, 0, 1.209180, 0.094792, 0, 1.209180, 0.094792, 0, 1.234847, 0.104948, 0, 1.209180, 0.094792, 0, 1.209180, 0.094792, 0, 1.234847, 0.104948, 0, 1.234847, 0.104948, 0, 1.234847, 0.104948, 0, 1.260417, 0.116667, 0, 1.234847, 0.104948, 0, 1.234847, 0.104948, 0, 1.260417, 0.116667, 0, 1.260417, 0.116667, 0, 1.260417, 0.116667, 0, 1.260417, 0.033333, 0, 1.260417, 0.116667, 0, 1.260417, 0.116667, 0, 1.260417, 0.033333, 0, 1.260417, 0.033333, 0, 1.260417, 0.033333, 0, 1.234473, 0.021826, 0, 1.260417, 0.033333, 0, 1.260417, 0.033333, 0, 1.234473, 0.021826, 0, 1.234473, 0.021826, 0, 1.234473, 0.021826, 0, 1.208203, 0.011784, 0, 1.234473, 0.021826, 0, 1.234473, 0.021826, 0, 1.208203, 0.011784, 0, 1.208203, 0.011784, 0, 1.208203, 0.011784, 0, 1.181608, 0.003206, 0, 1.208203, 0.011784, 0, 1.208203, 0.011784, 0, 1.181608, 0.003206, 0, 1.181608, 0.003206, 0, 1.181608, 0.003206, 0, 1.154688, -0.003906, 0, 1.181608, 0.003206, 0, 1.181608, 0.003206, 0, 1.154688, -0.003906, 0, 1.154688, -0.003906, 0, 1.154688, -0.003906, 0, 1.127507, -0.009489, 0, 1.154688, -0.003906, 0, 1.154688, -0.003906, 0, 1.127507, -0.009489, 0, 1.127507, -0.009489, 0, 1.127507, -0.009489, 0, 1.100130, -0.013477, 0, 1.127507, -0.009489, 0, 1.127507, -0.009489, 0, 1.100130, -0.013477, 0, 1.100130, -0.013477, 0, 1.100130, -0.013477, 0, 1.072559, -0.015869, 0, 1.100130, -0.013477, 0, 1.100130, -0.013477, 0, 1.072559, -0.015869, 0, 1.072559, -0.015869, 0, 1.072559, -0.015869, 0, 1.044792, -0.016667, 0, 1.072559, -0.015869, 0, 1.072559, -0.015869, 0, 1.044792, -0.016667, 0, 1.044792, -0.016667, 0, 1.044792, -0.016667, 0, 0.978630, -0.011556, 0, 1.044792, -0.016667, 0, 1.044792, -0.016667, 0, 0.978630, -0.011556, 0, 0.978630, -0.011556, 0, 0.978630, -0.011556, 0, 0.919727, 0.003776, 0, 0.978630, -0.011556, 0, 0.978630, -0.011556, 0, 0.919727, 0.003776, 0, 0.919727, 0.003776, 0, 0.919727, 0.003776, 0, 0.868083, 0.029329, 0, 0.919727, 0.003776, 0, 0.919727, 0.003776, 0, 0.868083, 0.029329, 0, 0.868083, 0.029329, 0, 0.868083, 0.029329, 0, 0.823698, 0.065104, 0, 0.868083, 0.029329, 0, 0.868083, 0.029329, 0, 0.823698, 0.065104, 0, 0.823698, 0.065104, 0, 0.823698, 0.065104, 0, 0.788037, 0.109603, 0, 0.823698, 0.065104, 0, 0.823698, 0.065104, 0, 0.788037, 0.109603, 0, 0.788037, 0.109603, 0, 0.788037, 0.109603, 0, 0.762565, 0.161328, 0, 0.788037, 0.109603, 0, 0.788037, 0.109603, 0, 0.762565, 0.161328, 0, 0.762565, 0.161328, 0, 0.762565, 0.161328, 0, 0.747282, 0.220280, 0, 0.762565, 0.161328, 0, 0.762565, 0.161328, 0, 0.747282, 0.220280, 0, 0.747282, 0.220280, 0, 0.747282, 0.220280, 0, 0.742188, 0.286458, 0, 0.747282, 0.220280, 0, 0.747282, 0.220280, 0, 0.742188, 0.286458, 0, 0.742188, 0.286458, 0, 0.742188, 0.286458, 0, 0.747021, 0.354867, 0, 0.742188, 0.286458, 0, 0.742188, 0.286458, 0, 0.747021, 0.354867, 0, 0.747021, 0.354867, 0, 0.747021, 0.354867, 0, 0.761523, 0.415820, 0, 0.747021, 0.354867, 0, 0.747021, 0.354867, 0, 0.761523, 0.415820, 0, 0.761523, 0.415820, 0, 0.761523, 0.415820, 0, 0.785693, 0.469320, 0, 0.761523, 0.415820, 0, 0.761523, 0.415820, 0, 0.785693, 0.469320, 0, 0.785693, 0.469320, 0, 0.785693, 0.469320, 0, 0.819531, 0.515365, 0, 0.785693, 0.469320, 0, 0.785693, 0.469320, 0, 0.819531, 0.515365, 0, 0.819531, 0.515365, 0, 0.819531, 0.515365, 0, 0.861572, 0.552393, 0, 0.819531, 0.515365, 0, 0.819531, 0.515365, 0, 0.861572, 0.552393, 0, 0.861572, 0.552393, 0, 0.861572, 0.552393, 0, 0.910352, 0.578841, 0, 0.861572, 0.552393, 0, 0.861572, 0.552393, 0, 0.910352, 0.578841, 0, 0.910352, 0.578841, 0, 0.910352, 0.578841, 0, 0.965869, 0.594710, 0, 0.910352, 0.578841, 0, 0.910352, 0.578841, 0, 0.965869, 0.594710, 0, 0.965869, 0.594710, 0, 0.965869, 0.594710, 0, 1.028125, 0.600000, 0, 0.965869, 0.594710, 0, 0.965869, 0.594710, 0, 1.028125, 0.600000, 0, 1.028125, 0.600000, 0, 1.028125, 0.600000, 0, 1.083903, 0.595272, 0, 1.028125, 0.600000, 0, 1.028125, 0.600000, 0, 1.083903, 0.595272, 0, 1.083903, 0.595272, 0, 1.083903, 0.595272, 0, 1.133529, 0.581087, 0, 1.083903, 0.595272, 0, 1.083903, 0.595272, 0, 1.133529, 0.581087, 0, 1.133529, 0.581087, 0, 1.133529, 0.581087, 0, 1.177002, 0.557446, 0, 1.133529, 0.581087, 0, 1.133529, 0.581087, 0, 1.177002, 0.557446, 0, 1.177002, 0.557446, 0, 1.177002, 0.557446, 0, 1.214323, 0.524349, 0, 1.177002, 0.557446, 0, 1.177002, 0.557446, 0, 1.214323, 0.524349, 0, 1.214323, 0.524349, 0, 1.214323, 0.524349, 0, 1.244287, 0.483130, 0, 1.214323, 0.524349, 0, 1.214323, 0.524349, 0, 1.244287, 0.483130, 0, 1.244287, 0.483130, 0, 1.244287, 0.483130, 0, 1.265690, 0.435124, 0, 1.244287, 0.483130, 0, 1.244287, 0.483130, 0, 1.265690, 0.435124, 0, 1.265690, 0.435124, 0, 1.265690, 0.435124, 0, 1.278532, 0.380330, 0, 1.265690, 0.435124, 0, 1.265690, 0.435124, 0, 1.278532, 0.380330, 0, 1.278532, 0.380330, 0, 0.747021, 0.354867, 0, 0.742188, 0.286458, 0, 0.845313, 0.349479, 0, 0.742188, 0.286458, 0, 0.747021, 0.354867, 0, 0.845313, 0.349479, 0, 0.761523, 0.415820, 0, 0.747021, 0.354867, 0, 0.845313, 0.349479, 0, 0.747021, 0.354867, 0, 0.761523, 0.415820, 0, 0.845313, 0.349479, 0, 0.761523, 0.415820, 0, 0.845313, 0.349479, 0, 0.851514, 0.387077, 0, 0.845313, 0.349479, 0, 0.761523, 0.415820, 0, 0.851514, 0.387077, 0, 0.862826, 0.420182, 0, 0.761523, 0.415820, 0, 0.851514, 0.387077, 0, 0.761523, 0.415820, 0, 0.862826, 0.420182, 0, 0.851514, 0.387077, 0, 0.862826, 0.420182, 0, 0.785693, 0.469320, 0, 0.761523, 0.415820, 0, 0.785693, 0.469320, 0, 0.862826, 0.420182, 0, 0.761523, 0.415820, 0, 0.785693, 0.469320, 0, 0.862826, 0.420182, 0, 0.879248, 0.448796, 0, 0.862826, 0.420182, 0, 0.785693, 0.469320, 0, 0.879248, 0.448796, 0, 0.819531, 0.515365, 0, 0.785693, 0.469320, 0, 0.879248, 0.448796, 0, 0.785693, 0.469320, 0, 0.819531, 0.515365, 0, 0.879248, 0.448796, 0, 0.819531, 0.515365, 0, 0.879248, 0.448796, 0, 0.900781, 0.472917, 0, 0.879248, 0.448796, 0, 0.819531, 0.515365, 0, 0.900781, 0.472917, 0, 0.861572, 0.552393, 0, 0.819531, 0.515365, 0, 0.900781, 0.472917, 0, 0.819531, 0.515365, 0, 0.861572, 0.552393, 0, 0.900781, 0.472917, 0, 0.861572, 0.552393, 0, 0.900781, 0.472917, 0, 0.926872, 0.492057, 0, 0.900781, 0.472917, 0, 0.861572, 0.552393, 0, 0.926872, 0.492057, 0, 0.910352, 0.578841, 0, 0.861572, 0.552393, 0, 0.926872, 0.492057, 0, 0.861572, 0.552393, 0, 0.910352, 0.578841, 0, 0.926872, 0.492057, 0, 0.910352, 0.578841, 0, 0.926872, 0.492057, 0, 0.956966, 0.505729, 0, 0.926872, 0.492057, 0, 0.910352, 0.578841, 0, 0.956966, 0.505729, 0, 0.910352, 0.578841, 0, 0.956966, 0.505729, 0, 0.965869, 0.594710, 0, 0.956966, 0.505729, 0, 0.910352, 0.578841, 0, 0.965869, 0.594710, 0, 0.965869, 0.594710, 0, 0.956966, 0.505729, 0, 0.991064, 0.513932, 0, 0.956966, 0.505729, 0, 0.965869, 0.594710, 0, 0.991064, 0.513932, 0, 0.965869, 0.594710, 0, 0.991064, 0.513932, 0, 1.028125, 0.600000, 0, 0.991064, 0.513932, 0, 0.965869, 0.594710, 0, 1.028125, 0.600000, 0, 1.028125, 0.600000, 0, 0.991064, 0.513932, 0, 1.029167, 0.516667, 0, 0.991064, 0.513932, 0, 1.028125, 0.600000, 0, 1.029167, 0.516667, 0, 1.028125, 0.600000, 0, 1.029167, 0.516667, 0, 1.062842, 0.513835, 0, 1.029167, 0.516667, 0, 1.028125, 0.600000, 0, 1.062842, 0.513835, 0, 1.028125, 0.600000, 0, 1.062842, 0.513835, 0, 1.083903, 0.595272, 0, 1.062842, 0.513835, 0, 1.028125, 0.600000, 0, 1.083903, 0.595272, 0, 1.083903, 0.595272, 0, 1.062842, 0.513835, 0, 1.093034, 0.505339, 0, 1.062842, 0.513835, 0, 1.083903, 0.595272, 0, 1.093034, 0.505339, 0, 1.083903, 0.595272, 0, 1.093034, 0.505339, 0, 1.133529, 0.581087, 0, 1.093034, 0.505339, 0, 1.083903, 0.595272, 0, 1.133529, 0.581087, 0, 1.093034, 0.505339, 0, 1.119743, 0.491178, 0, 1.133529, 0.581087, 0, 1.119743, 0.491178, 0, 1.093034, 0.505339, 0, 1.133529, 0.581087, 0, 1.133529, 0.581087, 0, 1.119743, 0.491178, 0, 1.177002, 0.557446, 0, 1.119743, 0.491178, 0, 1.133529, 0.581087, 0, 1.177002, 0.557446, 0, 1.119743, 0.491178, 0, 1.142969, 0.471354, 0, 1.177002, 0.557446, 0, 1.142969, 0.471354, 0, 1.119743, 0.491178, 0, 1.177002, 0.557446, 0, 1.177002, 0.557446, 0, 1.142969, 0.471354, 0, 1.214323, 0.524349, 0, 1.142969, 0.471354, 0, 1.177002, 0.557446, 0, 1.214323, 0.524349, 0, 1.142969, 0.471354, 0, 1.161833, 0.446777, 0, 1.214323, 0.524349, 0, 1.161833, 0.446777, 0, 1.142969, 0.471354, 0, 1.214323, 0.524349, 0, 1.214323, 0.524349, 0, 1.161833, 0.446777, 0, 1.244287, 0.483130, 0, 1.161833, 0.446777, 0, 1.214323, 0.524349, 0, 1.244287, 0.483130, 0, 1.161833, 0.446777, 0, 1.175456, 0.418359, 0, 1.244287, 0.483130, 0, 1.175456, 0.418359, 0, 1.161833, 0.446777, 0, 1.244287, 0.483130, 0, 1.244287, 0.483130, 0, 1.175456, 0.418359, 0, 1.265690, 0.435124, 0, 1.175456, 0.418359, 0, 1.244287, 0.483130, 0, 1.265690, 0.435124, 0, 1.175456, 0.418359, 0, 1.183838, 0.386100, 0, 1.265690, 0.435124, 0, 1.183838, 0.386100, 0, 1.175456, 0.418359, 0, 1.265690, 0.435124, 0, 1.265690, 0.435124, 0, 1.183838, 0.386100, 0, 1.278532, 0.380330, 0, 1.183838, 0.386100, 0, 1.265690, 0.435124, 0, 1.278532, 0.380330, 0, 1.183838, 0.386100, 0, 1.186979, 0.350000, 0, 1.278532, 0.380330, 0, 1.186979, 0.350000, 0, 1.183838, 0.386100, 0, 1.278532, 0.380330, 0, 1.278532, 0.380330, 0, 1.186979, 0.350000, 0, 1.282812, 0.318750, 0, 1.186979, 0.350000, 0, 1.278532, 0.380330, 0, 1.282812, 0.318750, 0, 1.282812, 0.266667, 0, 1.282812, 0.318750, 0, 1.186979, 0.350000, 0, 1.282812, 0.318750, 0, 1.282812, 0.266667, 0, 1.186979, 0.350000, 0, 1.186979, 0.350000, 0, 0.842188, 0.266667, 0, 1.282812, 0.266667, 0, 0.842188, 0.266667, 0, 1.186979, 0.350000, 0, 1.282812, 0.266667, 0, 1.186979, 0.350000, 0, 0.845313, 0.349479, 0, 0.842188, 0.266667, 0, 0.845313, 0.349479, 0, 1.186979, 0.350000, 0, 0.842188, 0.266667, 0, 0.742188, 0.286458, 0, 0.842188, 0.266667, 0, 0.845313, 0.349479, 0, 0.842188, 0.266667, 0, 0.742188, 0.286458, 0, 0.845313, 0.349479, 0, 0.742188, 0.286458, 0, 0.747282, 0.220280, 0, 0.842188, 0.266667, 0, 0.747282, 0.220280, 0, 0.742188, 0.286458, 0, 0.842188, 0.266667, 0, 0.842188, 0.266667, 0, 0.747282, 0.220280, 0, 0.848258, 0.220744, 0, 0.747282, 0.220280, 0, 0.842188, 0.266667, 0, 0.848258, 0.220744, 0, 0.747282, 0.220280, 0, 0.762565, 0.161328, 0, 0.848258, 0.220744, 0, 0.762565, 0.161328, 0, 0.747282, 0.220280, 0, 0.848258, 0.220744, 0, 0.848258, 0.220744, 0, 0.762565, 0.161328, 0, 0.860221, 0.180632, 0, 0.762565, 0.161328, 0, 0.848258, 0.220744, 0, 0.860221, 0.180632, 0, 0.762565, 0.161328, 0, 0.788037, 0.109603, 0, 0.860221, 0.180632, 0, 0.788037, 0.109603, 0, 0.762565, 0.161328, 0, 0.860221, 0.180632, 0, 0.860221, 0.180632, 0, 0.788037, 0.109603, 0, 0.878076, 0.146330, 0, 0.788037, 0.109603, 0, 0.860221, 0.180632, 0, 0.878076, 0.146330, 0, 0.788037, 0.109603, 0, 0.823698, 0.065104, 0, 0.878076, 0.146330, 0, 0.823698, 0.065104, 0, 0.788037, 0.109603, 0, 0.878076, 0.146330, 0, 0.878076, 0.146330, 0, 0.823698, 0.065104, 0, 0.901823, 0.117839, 0, 0.823698, 0.065104, 0, 0.878076, 0.146330, 0, 0.901823, 0.117839, 0, 0.823698, 0.065104, 0, 0.868083, 0.029329, 0, 0.901823, 0.117839, 0, 0.868083, 0.029329, 0, 0.823698, 0.065104, 0, 0.901823, 0.117839, 0, 0.901823, 0.117839, 0, 0.868083, 0.029329, 0, 0.931136, 0.095451, 0, 0.868083, 0.029329, 0, 0.901823, 0.117839, 0, 0.931136, 0.095451, 0, 0.931136, 0.095451, 0, 0.868083, 0.029329, 0, 0.919727, 0.003776, 0, 0.868083, 0.029329, 0, 0.931136, 0.095451, 0, 0.919727, 0.003776, 0, 0.965690, 0.079460, 0, 0.931136, 0.095451, 0, 0.919727, 0.003776, 0, 0.931136, 0.095451, 0, 0.965690, 0.079460, 0, 0.919727, 0.003776, 0, 0.965690, 0.079460, 0, 0.919727, 0.003776, 0, 0.978630, -0.011556, 0, 0.919727, 0.003776, 0, 0.965690, 0.079460, 0, 0.978630, -0.011556, 0, 1.005485, 0.069865, 0, 0.965690, 0.079460, 0, 0.978630, -0.011556, 0, 0.965690, 0.079460, 0, 1.005485, 0.069865, 0, 0.978630, -0.011556, 0, 1.005485, 0.069865, 0, 0.978630, -0.011556, 0, 1.044792, -0.016667, 0, 0.978630, -0.011556, 0, 1.005485, 0.069865, 0, 1.044792, -0.016667, 0, 1.050521, 0.066667, 0, 1.005485, 0.069865, 0, 1.044792, -0.016667, 0, 1.005485, 0.069865, 0, 1.050521, 0.066667, 0, 1.044792, -0.016667, 0, 1.050521, 0.066667, 0, 1.044792, -0.016667, 0, 1.072559, -0.015869, 0, 1.044792, -0.016667, 0, 1.050521, 0.066667, 0, 1.072559, -0.015869, 0, 1.050521, 0.066667, 0, 1.072559, -0.015869, 0, 1.077913, 0.067448, 0, 1.072559, -0.015869, 0, 1.050521, 0.066667, 0, 1.077913, 0.067448, 0, 1.077913, 0.067448, 0, 1.072559, -0.015869, 0, 1.100130, -0.013477, 0, 1.072559, -0.015869, 0, 1.077913, 0.067448, 0, 1.100130, -0.013477, 0, 1.077913, 0.067448, 0, 1.100130, -0.013477, 0, 1.104883, 0.069792, 0, 1.100130, -0.013477, 0, 1.077913, 0.067448, 0, 1.104883, 0.069792, 0, 1.104883, 0.069792, 0, 1.100130, -0.013477, 0, 1.127507, -0.009489, 0, 1.100130, -0.013477, 0, 1.104883, 0.069792, 0, 1.127507, -0.009489, 0, 1.104883, 0.069792, 0, 1.127507, -0.009489, 0, 1.131429, 0.073698, 0, 1.127507, -0.009489, 0, 1.104883, 0.069792, 0, 1.131429, 0.073698, 0, 1.131429, 0.073698, 0, 1.127507, -0.009489, 0, 1.154688, -0.003906, 0, 1.127507, -0.009489, 0, 1.131429, 0.073698, 0, 1.154688, -0.003906, 0, 1.131429, 0.073698, 0, 1.154688, -0.003906, 0, 1.157552, 0.079167, 0, 1.154688, -0.003906, 0, 1.131429, 0.073698, 0, 1.157552, 0.079167, 0, 1.157552, 0.079167, 0, 1.154688, -0.003906, 0, 1.181608, 0.003206, 0, 1.154688, -0.003906, 0, 1.157552, 0.079167, 0, 1.181608, 0.003206, 0, 1.183415, 0.086198, 0, 1.157552, 0.079167, 0, 1.181608, 0.003206, 0, 1.157552, 0.079167, 0, 1.183415, 0.086198, 0, 1.181608, 0.003206, 0, 1.208203, 0.011784, 0, 1.183415, 0.086198, 0, 1.181608, 0.003206, 0, 1.183415, 0.086198, 0, 1.208203, 0.011784, 0, 1.181608, 0.003206, 0, 1.183415, 0.086198, 0, 1.208203, 0.011784, 0, 1.234473, 0.021826, 0, 1.208203, 0.011784, 0, 1.183415, 0.086198, 0, 1.234473, 0.021826, 0, 1.209180, 0.094792, 0, 1.183415, 0.086198, 0, 1.234473, 0.021826, 0, 1.183415, 0.086198, 0, 1.209180, 0.094792, 0, 1.234473, 0.021826, 0, 1.209180, 0.094792, 0, 1.234473, 0.021826, 0, 1.260417, 0.033333, 0, 1.234473, 0.021826, 0, 1.209180, 0.094792, 0, 1.260417, 0.033333, 0, 1.234847, 0.104948, 0, 1.209180, 0.094792, 0, 1.260417, 0.033333, 0, 1.209180, 0.094792, 0, 1.234847, 0.104948, 0, 1.260417, 0.033333, 0, 1.234847, 0.104948, 0, 1.260417, 0.033333, 0, 1.260417, 0.116667, 0, 1.260417, 0.033333, 0, 1.234847, 0.104948, 0, 1.260417, 0.116667, 0, 1.186979, 0.350000, 0, 1.183838, 0.386100, 0, 1.186979, 0.350000, 0, 1.186979, 0.350000, 0, 1.183838, 0.386100, 0, 1.183838, 0.386100, 0, 1.183838, 0.386100, 0, 1.175456, 0.418359, 0, 1.183838, 0.386100, 0, 1.183838, 0.386100, 0, 1.175456, 0.418359, 0, 1.175456, 0.418359, 0, 1.175456, 0.418359, 0, 1.161833, 0.446777, 0, 1.175456, 0.418359, 0, 1.175456, 0.418359, 0, 1.161833, 0.446777, 0, 1.161833, 0.446777, 0, 1.161833, 0.446777, 0, 1.142969, 0.471354, 0, 1.161833, 0.446777, 0, 1.161833, 0.446777, 0, 1.142969, 0.471354, 0, 1.142969, 0.471354, 0, 1.142969, 0.471354, 0, 1.119743, 0.491178, 0, 1.142969, 0.471354, 0, 1.142969, 0.471354, 0, 1.119743, 0.491178, 0, 1.119743, 0.491178, 0, 1.119743, 0.491178, 0, 1.093034, 0.505339, 0, 1.119743, 0.491178, 0, 1.119743, 0.491178, 0, 1.093034, 0.505339, 0, 1.093034, 0.505339, 0, 1.093034, 0.505339, 0, 1.062842, 0.513835, 0, 1.093034, 0.505339, 0, 1.093034, 0.505339, 0, 1.062842, 0.513835, 0, 1.062842, 0.513835, 0, 1.062842, 0.513835, 0, 1.029167, 0.516667, 0, 1.062842, 0.513835, 0, 1.062842, 0.513835, 0, 1.029167, 0.516667, 0, 1.029167, 0.516667, 0, 1.029167, 0.516667, 0, 0.991064, 0.513932, 0, 1.029167, 0.516667, 0, 1.029167, 0.516667, 0, 0.991064, 0.513932, 0, 0.991064, 0.513932, 0, 0.991064, 0.513932, 0, 0.956966, 0.505729, 0, 0.991064, 0.513932, 0, 0.991064, 0.513932, 0, 0.956966, 0.505729, 0, 0.956966, 0.505729, 0, 0.956966, 0.505729, 0, 0.926872, 0.492057, 0, 0.956966, 0.505729, 0, 0.956966, 0.505729, 0, 0.926872, 0.492057, 0, 0.926872, 0.492057, 0, 0.926872, 0.492057, 0, 0.900781, 0.472917, 0, 0.926872, 0.492057, 0, 0.926872, 0.492057, 0, 0.900781, 0.472917, 0, 0.900781, 0.472917, 0, 0.900781, 0.472917, 0, 0.879248, 0.448796, 0, 0.900781, 0.472917, 0, 0.900781, 0.472917, 0, 0.879248, 0.448796, 0, 0.879248, 0.448796, 0, 0.879248, 0.448796, 0, 0.862826, 0.420182, 0, 0.879248, 0.448796, 0, 0.879248, 0.448796, 0, 0.862826, 0.420182, 0, 0.862826, 0.420182, 0, 0.862826, 0.420182, 0, 0.851514, 0.387077, 0, 0.862826, 0.420182, 0, 0.862826, 0.420182, 0, 0.851514, 0.387077, 0, 0.851514, 0.387077, 0, 0.851514, 0.387077, 0, 0.845313, 0.349479, 0, 0.851514, 0.387077, 0, 0.851514, 0.387077, 0, 0.845313, 0.349479, 0, 0.845313, 0.349479, 0, 1.805729, 0.550000, 0, 1.805729, 0.466667, 0, 1.805729, 0.550000, 0, 1.805729, 0.550000, 0, 1.805729, 0.466667, 0, 1.805729, 0.466667, 0, 1.805729, 0.466667, 0, 1.785221, 0.478385, 0, 1.805729, 0.466667, 0, 1.805729, 0.466667, 0, 1.785221, 0.478385, 0, 1.785221, 0.478385, 0, 1.785221, 0.478385, 0, 1.764323, 0.488542, 0, 1.785221, 0.478385, 0, 1.785221, 0.478385, 0, 1.764323, 0.488542, 0, 1.764323, 0.488542, 0, 1.764323, 0.488542, 0, 1.743034, 0.497135, 0, 1.764323, 0.488542, 0, 1.764323, 0.488542, 0, 1.743034, 0.497135, 0, 1.743034, 0.497135, 0, 1.743034, 0.497135, 0, 1.721354, 0.504167, 0, 1.743034, 0.497135, 0, 1.743034, 0.497135, 0, 1.721354, 0.504167, 0, 1.721354, 0.504167, 0, 1.721354, 0.504167, 0, 1.699284, 0.509635, 0, 1.721354, 0.504167, 0, 1.721354, 0.504167, 0, 1.699284, 0.509635, 0, 1.699284, 0.509635, 0, 1.699284, 0.509635, 0, 1.676823, 0.513542, 0, 1.699284, 0.509635, 0, 1.699284, 0.509635, 0, 1.676823, 0.513542, 0, 1.676823, 0.513542, 0, 1.676823, 0.513542, 0, 1.653971, 0.515885, 0, 1.676823, 0.513542, 0, 1.676823, 0.513542, 0, 1.653971, 0.515885, 0, 1.653971, 0.515885, 0, 1.653971, 0.515885, 0, 1.630729, 0.516667, 0, 1.653971, 0.515885, 0, 1.653971, 0.515885, 0, 1.630729, 0.516667, 0, 1.630729, 0.516667, 0, 1.630729, 0.516667, 0, 1.597282, 0.515251, 0, 1.630729, 0.516667, 0, 1.630729, 0.516667, 0, 1.597282, 0.515251, 0, 1.597282, 0.515251, 0, 1.597282, 0.515251, 0, 1.568294, 0.511003, 0, 1.597282, 0.515251, 0, 1.597282, 0.515251, 0, 1.568294, 0.511003, 0, 1.568294, 0.511003, 0, 1.568294, 0.511003, 0, 1.543766, 0.503923, 0, 1.568294, 0.511003, 0, 1.568294, 0.511003, 0, 1.543766, 0.503923, 0, 1.543766, 0.503923, 0, 1.543766, 0.503923, 0, 1.523698, 0.494010, 0, 1.543766, 0.503923, 0, 1.543766, 0.503923, 0, 1.523698, 0.494010, 0, 1.523698, 0.494010, 0, 1.523698, 0.494010, 0, 1.508089, 0.481266, 0, 1.523698, 0.494010, 0, 1.523698, 0.494010, 0, 1.508089, 0.481266, 0, 1.508089, 0.481266, 0, 1.508089, 0.481266, 0, 1.496940, 0.465690, 0, 1.508089, 0.481266, 0, 1.508089, 0.481266, 0, 1.496940, 0.465690, 0, 1.496940, 0.465690, 0, 1.496940, 0.465690, 0, 1.490251, 0.447282, 0, 1.496940, 0.465690, 0, 1.496940, 0.465690, 0, 1.490251, 0.447282, 0, 1.490251, 0.447282, 0, 1.490251, 0.447282, 0, 1.488021, 0.426042, 0, 1.490251, 0.447282, 0, 1.490251, 0.447282, 0, 1.488021, 0.426042, 0, 1.488021, 0.426042, 0, 1.488021, 0.426042, 0, 1.489616, 0.409766, 0, 1.488021, 0.426042, 0, 1.488021, 0.426042, 0, 1.489616, 0.409766, 0, 1.489616, 0.409766, 0, 1.489616, 0.409766, 0, 1.494401, 0.395313, 0, 1.489616, 0.409766, 0, 1.489616, 0.409766, 0, 1.494401, 0.395313, 0, 1.494401, 0.395313, 0, 1.494401, 0.395313, 0, 1.502376, 0.382682, 0, 1.494401, 0.395313, 0, 1.494401, 0.395313, 0, 1.502376, 0.382682, 0, 1.502376, 0.382682, 0, 1.502376, 0.382682, 0, 1.513542, 0.371875, 0, 1.502376, 0.382682, 0, 1.502376, 0.382682, 0, 1.513542, 0.371875, 0, 1.513542, 0.371875, 0, 1.513542, 0.371875, 0, 1.529525, 0.362109, 0, 1.513542, 0.371875, 0, 1.513542, 0.371875, 0, 1.529525, 0.362109, 0, 1.529525, 0.362109, 0, 1.529525, 0.362109, 0, 1.551953, 0.352604, 0, 1.529525, 0.362109, 0, 1.529525, 0.362109, 0, 1.551953, 0.352604, 0, 1.551953, 0.352604, 0, 1.551953, 0.352604, 0, 1.580827, 0.343359, 0, 1.551953, 0.352604, 0, 1.551953, 0.352604, 0, 1.580827, 0.343359, 0, 1.580827, 0.343359, 0, 1.580827, 0.343359, 0, 1.616146, 0.334375, 0, 1.580827, 0.343359, 0, 1.580827, 0.343359, 0, 1.616146, 0.334375, 0, 1.616146, 0.334375, 0, 1.616146, 0.334375, 0, 1.648958, 0.327083, 0, 1.616146, 0.334375, 0, 1.616146, 0.334375, 0, 1.648958, 0.327083, 0, 1.648958, 0.327083, 0, 1.648958, 0.327083, 0, 1.696305, 0.315177, 0, 1.648958, 0.327083, 0, 1.648958, 0.327083, 0, 1.696305, 0.315177, 0, 1.696305, 0.315177, 0, 1.696305, 0.315177, 0, 1.736263, 0.301074, 0, 1.696305, 0.315177, 0, 1.696305, 0.315177, 0, 1.736263, 0.301074, 0, 1.736263, 0.301074, 0, 1.736263, 0.301074, 0, 1.768831, 0.284774, 0, 1.736263, 0.301074, 0, 1.736263, 0.301074, 0, 1.768831, 0.284774, 0, 1.768831, 0.284774, 0, 1.768831, 0.284774, 0, 1.794010, 0.266276, 0, 1.768831, 0.284774, 0, 1.768831, 0.284774, 0, 1.794010, 0.266276, 0, 1.794010, 0.266276, 0, 1.794010, 0.266276, 0, 1.812809, 0.244735, 0, 1.794010, 0.266276, 0, 1.794010, 0.266276, 0, 1.812809, 0.244735, 0, 1.812809, 0.244735, 0, 1.812809, 0.244735, 0, 1.826237, 0.219303, 0, 1.812809, 0.244735, 0, 1.812809, 0.244735, 0, 1.826237, 0.219303, 0, 1.826237, 0.219303, 0, 1.826237, 0.219303, 0, 1.834294, 0.189982, 0, 1.826237, 0.219303, 0, 1.826237, 0.219303, 0, 1.834294, 0.189982, 0, 1.834294, 0.189982, 0, 1.834294, 0.189982, 0, 1.836979, 0.156771, 0, 1.834294, 0.189982, 0, 1.834294, 0.189982, 0, 1.836979, 0.156771, 0, 1.836979, 0.156771, 0, 1.836979, 0.156771, 0, 1.832959, 0.118872, 0, 1.836979, 0.156771, 0, 1.836979, 0.156771, 0, 1.832959, 0.118872, 0, 1.832959, 0.118872, 0, 1.832959, 0.118872, 0, 1.820898, 0.085124, 0, 1.832959, 0.118872, 0, 1.832959, 0.118872, 0, 1.820898, 0.085124, 0, 1.820898, 0.085124, 0, 1.820898, 0.085124, 0, 1.800797, 0.055526, 0, 1.820898, 0.085124, 0, 1.820898, 0.085124, 0, 1.800797, 0.055526, 0, 1.800797, 0.055526, 0, 1.800797, 0.055526, 0, 1.772656, 0.030078, 0, 1.800797, 0.055526, 0, 1.800797, 0.055526, 0, 1.772656, 0.030078, 0, 1.772656, 0.030078, 0, 1.772656, 0.030078, 0, 1.737484, 0.009627, 0, 1.772656, 0.030078, 0, 1.772656, 0.030078, 0, 1.737484, 0.009627, 0, 1.737484, 0.009627, 0, 1.737484, 0.009627, 0, 1.696289, -0.004980, 0, 1.737484, 0.009627, 0, 1.737484, 0.009627, 0, 1.696289, -0.004980, 0, 1.696289, -0.004980, 0, 1.696289, -0.004980, 0, 1.649072, -0.013745, 0, 1.696289, -0.004980, 0, 1.696289, -0.004980, 0, 1.649072, -0.013745, 0, 1.649072, -0.013745, 0, 1.649072, -0.013745, 0, 1.595833, -0.016667, 0, 1.649072, -0.013745, 0, 1.649072, -0.013745, 0, 1.595833, -0.016667, 0, 1.595833, -0.016667, 0, 1.595833, -0.016667, 0, 1.572152, -0.016146, 0, 1.595833, -0.016667, 0, 1.595833, -0.016667, 0, 1.572152, -0.016146, 0, 1.572152, -0.016146, 0, 1.572152, -0.016146, 0, 1.547982, -0.014583, 0, 1.572152, -0.016146, 0, 1.572152, -0.016146, 0, 1.547982, -0.014583, 0, 1.547982, -0.014583, 0, 1.547982, -0.014583, 0, 1.523324, -0.011979, 0, 1.547982, -0.014583, 0, 1.547982, -0.014583, 0, 1.523324, -0.011979, 0, 1.523324, -0.011979, 0, 1.523324, -0.011979, 0, 1.498177, -0.008333, 0, 1.523324, -0.011979, 0, 1.523324, -0.011979, 0, 1.498177, -0.008333, 0, 1.498177, -0.008333, 0, 1.498177, -0.008333, 0, 1.472445, -0.003646, 0, 1.498177, -0.008333, 0, 1.498177, -0.008333, 0, 1.472445, -0.003646, 0, 1.472445, -0.003646, 0, 1.472445, -0.003646, 0, 1.446029, 0.002083, 0, 1.472445, -0.003646, 0, 1.472445, -0.003646, 0, 1.446029, 0.002083, 0, 1.446029, 0.002083, 0, 1.446029, 0.002083, 0, 1.418929, 0.008854, 0, 1.446029, 0.002083, 0, 1.446029, 0.002083, 0, 1.418929, 0.008854, 0, 1.418929, 0.008854, 0, 1.418929, 0.008854, 0, 1.391146, 0.016667, 0, 1.418929, 0.008854, 0, 1.418929, 0.008854, 0, 1.391146, 0.016667, 0, 1.391146, 0.016667, 0, 1.391146, 0.016667, 0, 1.391146, 0.116667, 0, 1.391146, 0.016667, 0, 1.391146, 0.016667, 0, 1.391146, 0.116667, 0, 1.391146, 0.116667, 0, 1.391146, 0.116667, 0, 1.417611, 0.104948, 0, 1.391146, 0.116667, 0, 1.391146, 0.116667, 0, 1.417611, 0.104948, 0, 1.417611, 0.104948, 0, 1.417611, 0.104948, 0, 1.443880, 0.094792, 0, 1.417611, 0.104948, 0, 1.417611, 0.104948, 0, 1.443880, 0.094792, 0, 1.443880, 0.094792, 0, 1.443880, 0.094792, 0, 1.469954, 0.086198, 0, 1.443880, 0.094792, 0, 1.443880, 0.094792, 0, 1.469954, 0.086198, 0, 1.469954, 0.086198, 0, 1.469954, 0.086198, 0, 1.495833, 0.079167, 0, 1.469954, 0.086198, 0, 1.469954, 0.086198, 0, 1.495833, 0.079167, 0, 1.495833, 0.079167, 0, 1.495833, 0.079167, 0, 1.521550, 0.073698, 0, 1.495833, 0.079167, 0, 1.495833, 0.079167, 0, 1.521550, 0.073698, 0, 1.521550, 0.073698, 0, 1.521550, 0.073698, 0, 1.547135, 0.069792, 0, 1.521550, 0.073698, 0, 1.521550, 0.073698, 0, 1.547135, 0.069792, 0, 1.547135, 0.069792, 0, 1.547135, 0.069792, 0, 1.572591, 0.067448, 0, 1.547135, 0.069792, 0, 1.547135, 0.069792, 0, 1.572591, 0.067448, 0, 1.572591, 0.067448, 0, 1.572591, 0.067448, 0, 1.597917, 0.066667, 0, 1.572591, 0.067448, 0, 1.572591, 0.067448, 0, 1.597917, 0.066667, 0, 1.597917, 0.066667, 0, 1.597917, 0.066667, 0, 1.629818, 0.068058, 0, 1.597917, 0.066667, 0, 1.597917, 0.066667, 0, 1.629818, 0.068058, 0, 1.629818, 0.068058, 0, 1.629818, 0.068058, 0, 1.657812, 0.072233, 0, 1.629818, 0.068058, 0, 1.629818, 0.068058, 0, 1.657812, 0.072233, 0, 1.657812, 0.072233, 0, 1.657812, 0.072233, 0, 1.681901, 0.079191, 0, 1.657812, 0.072233, 0, 1.657812, 0.072233, 0, 1.681901, 0.079191, 0, 1.681901, 0.079191, 0, 1.681901, 0.079191, 0, 1.702083, 0.088932, 0, 1.681901, 0.079191, 0, 1.681901, 0.079191, 0, 1.702083, 0.088932, 0, 1.702083, 0.088932, 0, 1.702083, 0.088932, 0, 1.718034, 0.101213, 0, 1.702083, 0.088932, 0, 1.702083, 0.088932, 0, 1.718034, 0.101213, 0, 1.718034, 0.101213, 0, 1.718034, 0.101213, 0, 1.729427, 0.115788, 0, 1.718034, 0.101213, 0, 1.718034, 0.101213, 0, 1.729427, 0.115788, 0, 1.729427, 0.115788, 0, 1.729427, 0.115788, 0, 1.736263, 0.132658, 0, 1.729427, 0.115788, 0, 1.729427, 0.115788, 0, 1.736263, 0.132658, 0, 1.736263, 0.132658, 0, 1.736263, 0.132658, 0, 1.738542, 0.151823, 0, 1.736263, 0.132658, 0, 1.736263, 0.132658, 0, 1.738542, 0.151823, 0, 1.738542, 0.151823, 0, 1.738542, 0.151823, 0, 1.736898, 0.169482, 0, 1.738542, 0.151823, 0, 1.738542, 0.151823, 0, 1.736898, 0.169482, 0, 1.736898, 0.169482, 0, 1.736898, 0.169482, 0, 1.731966, 0.184961, 0, 1.736898, 0.169482, 0, 1.736898, 0.169482, 0, 1.731966, 0.184961, 0, 1.731966, 0.184961, 0, 1.731966, 0.184961, 0, 1.723747, 0.198258, 0, 1.731966, 0.184961, 0, 1.731966, 0.184961, 0, 1.723747, 0.198258, 0, 1.723747, 0.198258, 0, 1.723747, 0.198258, 0, 1.712240, 0.209375, 0, 1.723747, 0.198258, 0, 1.723747, 0.198258, 0, 1.712240, 0.209375, 0, 1.712240, 0.209375, 0, 1.712240, 0.209375, 0, 1.695166, 0.219303, 0, 1.712240, 0.209375, 0, 1.712240, 0.209375, 0, 1.695166, 0.219303, 0, 1.695166, 0.219303, 0, 1.695166, 0.219303, 0, 1.670247, 0.229036, 0, 1.695166, 0.219303, 0, 1.695166, 0.219303, 0, 1.670247, 0.229036, 0, 1.670247, 0.229036, 0, 1.670247, 0.229036, 0, 1.637484, 0.238574, 0, 1.670247, 0.229036, 0, 1.670247, 0.229036, 0, 1.637484, 0.238574, 0, 1.637484, 0.238574, 0, 1.637484, 0.238574, 0, 1.596875, 0.247917, 0, 1.637484, 0.238574, 0, 1.637484, 0.238574, 0, 1.596875, 0.247917, 0, 1.596875, 0.247917, 0, 1.596875, 0.247917, 0, 1.563542, 0.255729, 0, 1.596875, 0.247917, 0, 1.596875, 0.247917, 0, 1.563542, 0.255729, 0, 1.563542, 0.255729, 0, 1.563542, 0.255729, 0, 1.522103, 0.266642, 0, 1.563542, 0.255729, 0, 1.563542, 0.255729, 0, 1.522103, 0.266642, 0, 1.522103, 0.266642, 0, 1.522103, 0.266642, 0, 1.486849, 0.280111, 0, 1.522103, 0.266642, 0, 1.522103, 0.266642, 0, 1.486849, 0.280111, 0, 1.486849, 0.280111, 0, 1.486849, 0.280111, 0, 1.457780, 0.296134, 0, 1.486849, 0.280111, 0, 1.486849, 0.280111, 0, 1.457780, 0.296134, 0, 1.457780, 0.296134, 0, 1.457780, 0.296134, 0, 1.434896, 0.314714, 0, 1.457780, 0.296134, 0, 1.457780, 0.296134, 0, 1.434896, 0.314714, 0, 1.434896, 0.314714, 0, 1.434896, 0.314714, 0, 1.417578, 0.336434, 0, 1.434896, 0.314714, 0, 1.434896, 0.314714, 0, 1.417578, 0.336434, 0, 1.417578, 0.336434, 0, 1.417578, 0.336434, 0, 1.405208, 0.361882, 0, 1.417578, 0.336434, 0, 1.417578, 0.336434, 0, 1.405208, 0.361882, 0, 1.405208, 0.361882, 0, 1.405208, 0.361882, 0, 1.397786, 0.391056, 0, 1.405208, 0.361882, 0, 1.405208, 0.361882, 0, 1.397786, 0.391056, 0, 1.397786, 0.391056, 0, 1.397786, 0.391056, 0, 1.395313, 0.423958, 0, 1.397786, 0.391056, 0, 1.397786, 0.391056, 0, 1.395313, 0.423958, 0, 1.395313, 0.423958, 0, 1.395313, 0.423958, 0, 1.398958, 0.463631, 0, 1.395313, 0.423958, 0, 1.395313, 0.423958, 0, 1.398958, 0.463631, 0, 1.398958, 0.463631, 0, 1.398958, 0.463631, 0, 1.409896, 0.498535, 0, 1.398958, 0.463631, 0, 1.398958, 0.463631, 0, 1.409896, 0.498535, 0, 1.409896, 0.498535, 0, 1.409896, 0.498535, 0, 1.428125, 0.528670, 0, 1.409896, 0.498535, 0, 1.409896, 0.498535, 0, 1.428125, 0.528670, 0, 1.428125, 0.528670, 0, 1.428125, 0.528670, 0, 1.453646, 0.554036, 0, 1.428125, 0.528670, 0, 1.428125, 0.528670, 0, 1.453646, 0.554036, 0, 1.453646, 0.554036, 0, 1.453646, 0.554036, 0, 1.485872, 0.574145, 0, 1.453646, 0.554036, 0, 1.453646, 0.554036, 0, 1.485872, 0.574145, 0, 1.485872, 0.574145, 0, 1.485872, 0.574145, 0, 1.524219, 0.588509, 0, 1.485872, 0.574145, 0, 1.485872, 0.574145, 0, 1.524219, 0.588509, 0, 1.524219, 0.588509, 0, 1.524219, 0.588509, 0, 1.568685, 0.597127, 0, 1.524219, 0.588509, 0, 1.524219, 0.588509, 0, 1.568685, 0.597127, 0, 1.568685, 0.597127, 0, 1.568685, 0.597127, 0, 1.619271, 0.600000, 0, 1.568685, 0.597127, 0, 1.568685, 0.597127, 0, 1.619271, 0.600000, 0, 1.619271, 0.600000, 0, 1.619271, 0.600000, 0, 1.645443, 0.599219, 0, 1.619271, 0.600000, 0, 1.619271, 0.600000, 0, 1.645443, 0.599219, 0, 1.645443, 0.599219, 0, 1.645443, 0.599219, 0, 1.670833, 0.596875, 0, 1.645443, 0.599219, 0, 1.645443, 0.599219, 0, 1.670833, 0.596875, 0, 1.670833, 0.596875, 0, 1.670833, 0.596875, 0, 1.695443, 0.592969, 0, 1.670833, 0.596875, 0, 1.670833, 0.596875, 0, 1.695443, 0.592969, 0, 1.695443, 0.592969, 0, 1.695443, 0.592969, 0, 1.719271, 0.587500, 0, 1.695443, 0.592969, 0, 1.695443, 0.592969, 0, 1.719271, 0.587500, 0, 1.719271, 0.587500, 0, 1.719271, 0.587500, 0, 1.742253, 0.580469, 0, 1.719271, 0.587500, 0, 1.719271, 0.587500, 0, 1.742253, 0.580469, 0, 1.742253, 0.580469, 0, 1.742253, 0.580469, 0, 1.764323, 0.571875, 0, 1.742253, 0.580469, 0, 1.742253, 0.580469, 0, 1.764323, 0.571875, 0, 1.764323, 0.571875, 0, 1.764323, 0.571875, 0, 1.785482, 0.561719, 0, 1.764323, 0.571875, 0, 1.764323, 0.571875, 0, 1.785482, 0.561719, 0, 1.785482, 0.561719, 0, 1.391146, 0.116667, 0, 1.391146, 0.016667, 0, 1.417611, 0.104948, 0, 1.391146, 0.016667, 0, 1.391146, 0.116667, 0, 1.417611, 0.104948, 0, 1.443880, 0.094792, 0, 1.417611, 0.104948, 0, 1.391146, 0.016667, 0, 1.417611, 0.104948, 0, 1.443880, 0.094792, 0, 1.391146, 0.016667, 0, 1.443880, 0.094792, 0, 1.391146, 0.016667, 0, 1.418929, 0.008854, 0, 1.391146, 0.016667, 0, 1.443880, 0.094792, 0, 1.418929, 0.008854, 0, 1.469954, 0.086198, 0, 1.443880, 0.094792, 0, 1.418929, 0.008854, 0, 1.443880, 0.094792, 0, 1.469954, 0.086198, 0, 1.418929, 0.008854, 0, 1.469954, 0.086198, 0, 1.418929, 0.008854, 0, 1.446029, 0.002083, 0, 1.418929, 0.008854, 0, 1.469954, 0.086198, 0, 1.446029, 0.002083, 0, 1.469954, 0.086198, 0, 1.446029, 0.002083, 0, 1.472445, -0.003646, 0, 1.446029, 0.002083, 0, 1.469954, 0.086198, 0, 1.472445, -0.003646, 0, 1.469954, 0.086198, 0, 1.472445, -0.003646, 0, 1.495833, 0.079167, 0, 1.472445, -0.003646, 0, 1.469954, 0.086198, 0, 1.495833, 0.079167, 0, 1.495833, 0.079167, 0, 1.472445, -0.003646, 0, 1.498177, -0.008333, 0, 1.472445, -0.003646, 0, 1.495833, 0.079167, 0, 1.498177, -0.008333, 0, 1.495833, 0.079167, 0, 1.498177, -0.008333, 0, 1.521550, 0.073698, 0, 1.498177, -0.008333, 0, 1.495833, 0.079167, 0, 1.521550, 0.073698, 0, 1.521550, 0.073698, 0, 1.498177, -0.008333, 0, 1.523324, -0.011979, 0, 1.498177, -0.008333, 0, 1.521550, 0.073698, 0, 1.523324, -0.011979, 0, 1.521550, 0.073698, 0, 1.523324, -0.011979, 0, 1.547135, 0.069792, 0, 1.523324, -0.011979, 0, 1.521550, 0.073698, 0, 1.547135, 0.069792, 0, 1.547135, 0.069792, 0, 1.523324, -0.011979, 0, 1.547982, -0.014583, 0, 1.523324, -0.011979, 0, 1.547135, 0.069792, 0, 1.547982, -0.014583, 0, 1.547135, 0.069792, 0, 1.547982, -0.014583, 0, 1.572591, 0.067448, 0, 1.547982, -0.014583, 0, 1.547135, 0.069792, 0, 1.572591, 0.067448, 0, 1.547982, -0.014583, 0, 1.572152, -0.016146, 0, 1.572591, 0.067448, 0, 1.572152, -0.016146, 0, 1.547982, -0.014583, 0, 1.572591, 0.067448, 0, 1.572591, 0.067448, 0, 1.572152, -0.016146, 0, 1.597917, 0.066667, 0, 1.572152, -0.016146, 0, 1.572591, 0.067448, 0, 1.597917, 0.066667, 0, 1.572152, -0.016146, 0, 1.595833, -0.016667, 0, 1.597917, 0.066667, 0, 1.595833, -0.016667, 0, 1.572152, -0.016146, 0, 1.597917, 0.066667, 0, 1.629818, 0.068058, 0, 1.597917, 0.066667, 0, 1.595833, -0.016667, 0, 1.597917, 0.066667, 0, 1.629818, 0.068058, 0, 1.595833, -0.016667, 0, 1.649072, -0.013745, 0, 1.629818, 0.068058, 0, 1.595833, -0.016667, 0, 1.629818, 0.068058, 0, 1.649072, -0.013745, 0, 1.595833, -0.016667, 0, 1.657812, 0.072233, 0, 1.629818, 0.068058, 0, 1.649072, -0.013745, 0, 1.629818, 0.068058, 0, 1.657812, 0.072233, 0, 1.649072, -0.013745, 0, 1.696289, -0.004980, 0, 1.657812, 0.072233, 0, 1.649072, -0.013745, 0, 1.657812, 0.072233, 0, 1.696289, -0.004980, 0, 1.649072, -0.013745, 0, 1.681901, 0.079191, 0, 1.657812, 0.072233, 0, 1.696289, -0.004980, 0, 1.657812, 0.072233, 0, 1.681901, 0.079191, 0, 1.696289, -0.004980, 0, 1.681901, 0.079191, 0, 1.696289, -0.004980, 0, 1.737484, 0.009627, 0, 1.696289, -0.004980, 0, 1.681901, 0.079191, 0, 1.737484, 0.009627, 0, 1.702083, 0.088932, 0, 1.681901, 0.079191, 0, 1.737484, 0.009627, 0, 1.681901, 0.079191, 0, 1.702083, 0.088932, 0, 1.737484, 0.009627, 0, 1.702083, 0.088932, 0, 1.737484, 0.009627, 0, 1.772656, 0.030078, 0, 1.737484, 0.009627, 0, 1.702083, 0.088932, 0, 1.772656, 0.030078, 0, 1.718034, 0.101213, 0, 1.702083, 0.088932, 0, 1.772656, 0.030078, 0, 1.702083, 0.088932, 0, 1.718034, 0.101213, 0, 1.772656, 0.030078, 0, 1.718034, 0.101213, 0, 1.772656, 0.030078, 0, 1.800797, 0.055526, 0, 1.772656, 0.030078, 0, 1.718034, 0.101213, 0, 1.800797, 0.055526, 0, 1.729427, 0.115788, 0, 1.718034, 0.101213, 0, 1.800797, 0.055526, 0, 1.718034, 0.101213, 0, 1.729427, 0.115788, 0, 1.800797, 0.055526, 0, 1.729427, 0.115788, 0, 1.800797, 0.055526, 0, 1.820898, 0.085124, 0, 1.800797, 0.055526, 0, 1.729427, 0.115788, 0, 1.820898, 0.085124, 0, 1.736263, 0.132658, 0, 1.729427, 0.115788, 0, 1.820898, 0.085124, 0, 1.729427, 0.115788, 0, 1.736263, 0.132658, 0, 1.820898, 0.085124, 0, 1.736263, 0.132658, 0, 1.820898, 0.085124, 0, 1.832959, 0.118872, 0, 1.820898, 0.085124, 0, 1.736263, 0.132658, 0, 1.832959, 0.118872, 0, 1.738542, 0.151823, 0, 1.736263, 0.132658, 0, 1.832959, 0.118872, 0, 1.736263, 0.132658, 0, 1.738542, 0.151823, 0, 1.832959, 0.118872, 0, 1.738542, 0.151823, 0, 1.832959, 0.118872, 0, 1.836979, 0.156771, 0, 1.832959, 0.118872, 0, 1.738542, 0.151823, 0, 1.836979, 0.156771, 0, 1.836979, 0.156771, 0, 1.736898, 0.169482, 0, 1.738542, 0.151823, 0, 1.736898, 0.169482, 0, 1.836979, 0.156771, 0, 1.738542, 0.151823, 0, 1.834294, 0.189982, 0, 1.736898, 0.169482, 0, 1.836979, 0.156771, 0, 1.736898, 0.169482, 0, 1.834294, 0.189982, 0, 1.836979, 0.156771, 0, 1.826237, 0.219303, 0, 1.736898, 0.169482, 0, 1.834294, 0.189982, 0, 1.736898, 0.169482, 0, 1.826237, 0.219303, 0, 1.834294, 0.189982, 0, 1.731966, 0.184961, 0, 1.736898, 0.169482, 0, 1.826237, 0.219303, 0, 1.736898, 0.169482, 0, 1.731966, 0.184961, 0, 1.826237, 0.219303, 0, 1.812809, 0.244735, 0, 1.731966, 0.184961, 0, 1.826237, 0.219303, 0, 1.731966, 0.184961, 0, 1.812809, 0.244735, 0, 1.826237, 0.219303, 0, 1.723747, 0.198258, 0, 1.731966, 0.184961, 0, 1.812809, 0.244735, 0, 1.731966, 0.184961, 0, 1.723747, 0.198258, 0, 1.812809, 0.244735, 0, 1.794010, 0.266276, 0, 1.723747, 0.198258, 0, 1.812809, 0.244735, 0, 1.723747, 0.198258, 0, 1.794010, 0.266276, 0, 1.812809, 0.244735, 0, 1.768831, 0.284774, 0, 1.723747, 0.198258, 0, 1.794010, 0.266276, 0, 1.723747, 0.198258, 0, 1.768831, 0.284774, 0, 1.794010, 0.266276, 0, 1.712240, 0.209375, 0, 1.723747, 0.198258, 0, 1.768831, 0.284774, 0, 1.723747, 0.198258, 0, 1.712240, 0.209375, 0, 1.768831, 0.284774, 0, 1.736263, 0.301074, 0, 1.712240, 0.209375, 0, 1.768831, 0.284774, 0, 1.712240, 0.209375, 0, 1.736263, 0.301074, 0, 1.768831, 0.284774, 0, 1.695166, 0.219303, 0, 1.712240, 0.209375, 0, 1.736263, 0.301074, 0, 1.712240, 0.209375, 0, 1.695166, 0.219303, 0, 1.736263, 0.301074, 0, 1.696305, 0.315177, 0, 1.695166, 0.219303, 0, 1.736263, 0.301074, 0, 1.695166, 0.219303, 0, 1.696305, 0.315177, 0, 1.736263, 0.301074, 0, 1.670247, 0.229036, 0, 1.695166, 0.219303, 0, 1.696305, 0.315177, 0, 1.695166, 0.219303, 0, 1.670247, 0.229036, 0, 1.696305, 0.315177, 0, 1.696305, 0.315177, 0, 1.637484, 0.238574, 0, 1.670247, 0.229036, 0, 1.637484, 0.238574, 0, 1.696305, 0.315177, 0, 1.670247, 0.229036, 0, 1.696305, 0.315177, 0, 1.648958, 0.327083, 0, 1.637484, 0.238574, 0, 1.648958, 0.327083, 0, 1.696305, 0.315177, 0, 1.637484, 0.238574, 0, 1.596875, 0.247917, 0, 1.637484, 0.238574, 0, 1.648958, 0.327083, 0, 1.637484, 0.238574, 0, 1.596875, 0.247917, 0, 1.648958, 0.327083, 0, 1.616146, 0.334375, 0, 1.596875, 0.247917, 0, 1.648958, 0.327083, 0, 1.596875, 0.247917, 0, 1.616146, 0.334375, 0, 1.648958, 0.327083, 0, 1.563542, 0.255729, 0, 1.596875, 0.247917, 0, 1.616146, 0.334375, 0, 1.596875, 0.247917, 0, 1.563542, 0.255729, 0, 1.616146, 0.334375, 0, 1.580827, 0.343359, 0, 1.563542, 0.255729, 0, 1.616146, 0.334375, 0, 1.563542, 0.255729, 0, 1.580827, 0.343359, 0, 1.616146, 0.334375, 0, 1.522103, 0.266642, 0, 1.563542, 0.255729, 0, 1.580827, 0.343359, 0, 1.563542, 0.255729, 0, 1.522103, 0.266642, 0, 1.580827, 0.343359, 0, 1.551953, 0.352604, 0, 1.522103, 0.266642, 0, 1.580827, 0.343359, 0, 1.522103, 0.266642, 0, 1.551953, 0.352604, 0, 1.580827, 0.343359, 0, 1.551953, 0.352604, 0, 1.529525, 0.362109, 0, 1.522103, 0.266642, 0, 1.529525, 0.362109, 0, 1.551953, 0.352604, 0, 1.522103, 0.266642, 0, 1.529525, 0.362109, 0, 1.486849, 0.280111, 0, 1.522103, 0.266642, 0, 1.486849, 0.280111, 0, 1.529525, 0.362109, 0, 1.522103, 0.266642, 0, 1.529525, 0.362109, 0, 1.513542, 0.371875, 0, 1.486849, 0.280111, 0, 1.513542, 0.371875, 0, 1.529525, 0.362109, 0, 1.486849, 0.280111, 0, 1.513542, 0.371875, 0, 1.457780, 0.296134, 0, 1.486849, 0.280111, 0, 1.457780, 0.296134, 0, 1.513542, 0.371875, 0, 1.486849, 0.280111, 0, 1.434896, 0.314714, 0, 1.457780, 0.296134, 0, 1.513542, 0.371875, 0, 1.457780, 0.296134, 0, 1.434896, 0.314714, 0, 1.513542, 0.371875, 0, 1.513542, 0.371875, 0, 1.502376, 0.382682, 0, 1.434896, 0.314714, 0, 1.502376, 0.382682, 0, 1.513542, 0.371875, 0, 1.434896, 0.314714, 0, 1.502376, 0.382682, 0, 1.417578, 0.336434, 0, 1.434896, 0.314714, 0, 1.417578, 0.336434, 0, 1.502376, 0.382682, 0, 1.434896, 0.314714, 0, 1.502376, 0.382682, 0, 1.494401, 0.395313, 0, 1.417578, 0.336434, 0, 1.494401, 0.395313, 0, 1.502376, 0.382682, 0, 1.417578, 0.336434, 0, 1.494401, 0.395313, 0, 1.405208, 0.361882, 0, 1.417578, 0.336434, 0, 1.405208, 0.361882, 0, 1.494401, 0.395313, 0, 1.417578, 0.336434, 0, 1.494401, 0.395313, 0, 1.489616, 0.409766, 0, 1.405208, 0.361882, 0, 1.489616, 0.409766, 0, 1.494401, 0.395313, 0, 1.405208, 0.361882, 0, 1.489616, 0.409766, 0, 1.397786, 0.391056, 0, 1.405208, 0.361882, 0, 1.397786, 0.391056, 0, 1.489616, 0.409766, 0, 1.405208, 0.361882, 0, 1.395313, 0.423958, 0, 1.397786, 0.391056, 0, 1.489616, 0.409766, 0, 1.397786, 0.391056, 0, 1.395313, 0.423958, 0, 1.489616, 0.409766, 0, 1.488021, 0.426042, 0, 1.395313, 0.423958, 0, 1.489616, 0.409766, 0, 1.395313, 0.423958, 0, 1.488021, 0.426042, 0, 1.489616, 0.409766, 0, 1.398958, 0.463631, 0, 1.395313, 0.423958, 0, 1.488021, 0.426042, 0, 1.395313, 0.423958, 0, 1.398958, 0.463631, 0, 1.488021, 0.426042, 0, 1.398958, 0.463631, 0, 1.488021, 0.426042, 0, 1.490251, 0.447282, 0, 1.488021, 0.426042, 0, 1.398958, 0.463631, 0, 1.490251, 0.447282, 0, 1.409896, 0.498535, 0, 1.398958, 0.463631, 0, 1.490251, 0.447282, 0, 1.398958, 0.463631, 0, 1.409896, 0.498535, 0, 1.490251, 0.447282, 0, 1.409896, 0.498535, 0, 1.490251, 0.447282, 0, 1.496940, 0.465690, 0, 1.490251, 0.447282, 0, 1.409896, 0.498535, 0, 1.496940, 0.465690, 0, 1.428125, 0.528670, 0, 1.409896, 0.498535, 0, 1.496940, 0.465690, 0, 1.409896, 0.498535, 0, 1.428125, 0.528670, 0, 1.496940, 0.465690, 0, 1.428125, 0.528670, 0, 1.496940, 0.465690, 0, 1.508089, 0.481266, 0, 1.496940, 0.465690, 0, 1.428125, 0.528670, 0, 1.508089, 0.481266, 0, 1.453646, 0.554036, 0, 1.428125, 0.528670, 0, 1.508089, 0.481266, 0, 1.428125, 0.528670, 0, 1.453646, 0.554036, 0, 1.508089, 0.481266, 0, 1.453646, 0.554036, 0, 1.508089, 0.481266, 0, 1.523698, 0.494010, 0, 1.508089, 0.481266, 0, 1.453646, 0.554036, 0, 1.523698, 0.494010, 0, 1.485872, 0.574145, 0, 1.453646, 0.554036, 0, 1.523698, 0.494010, 0, 1.453646, 0.554036, 0, 1.485872, 0.574145, 0, 1.523698, 0.494010, 0, 1.485872, 0.574145, 0, 1.523698, 0.494010, 0, 1.543766, 0.503923, 0, 1.523698, 0.494010, 0, 1.485872, 0.574145, 0, 1.543766, 0.503923, 0, 1.524219, 0.588509, 0, 1.485872, 0.574145, 0, 1.543766, 0.503923, 0, 1.485872, 0.574145, 0, 1.524219, 0.588509, 0, 1.543766, 0.503923, 0, 1.524219, 0.588509, 0, 1.543766, 0.503923, 0, 1.568294, 0.511003, 0, 1.543766, 0.503923, 0, 1.524219, 0.588509, 0, 1.568294, 0.511003, 0, 1.524219, 0.588509, 0, 1.568294, 0.511003, 0, 1.568685, 0.597127, 0, 1.568294, 0.511003, 0, 1.524219, 0.588509, 0, 1.568685, 0.597127, 0, 1.568685, 0.597127, 0, 1.568294, 0.511003, 0, 1.597282, 0.515251, 0, 1.568294, 0.511003, 0, 1.568685, 0.597127, 0, 1.597282, 0.515251, 0, 1.619271, 0.600000, 0, 1.568685, 0.597127, 0, 1.597282, 0.515251, 0, 1.568685, 0.597127, 0, 1.619271, 0.600000, 0, 1.597282, 0.515251, 0, 1.619271, 0.600000, 0, 1.597282, 0.515251, 0, 1.630729, 0.516667, 0, 1.597282, 0.515251, 0, 1.619271, 0.600000, 0, 1.630729, 0.516667, 0, 1.630729, 0.516667, 0, 1.645443, 0.599219, 0, 1.619271, 0.600000, 0, 1.645443, 0.599219, 0, 1.630729, 0.516667, 0, 1.619271, 0.600000, 0, 1.645443, 0.599219, 0, 1.630729, 0.516667, 0, 1.653971, 0.515885, 0, 1.630729, 0.516667, 0, 1.645443, 0.599219, 0, 1.653971, 0.515885, 0, 1.670833, 0.596875, 0, 1.645443, 0.599219, 0, 1.653971, 0.515885, 0, 1.645443, 0.599219, 0, 1.670833, 0.596875, 0, 1.653971, 0.515885, 0, 1.670833, 0.596875, 0, 1.653971, 0.515885, 0, 1.676823, 0.513542, 0, 1.653971, 0.515885, 0, 1.670833, 0.596875, 0, 1.676823, 0.513542, 0, 1.670833, 0.596875, 0, 1.676823, 0.513542, 0, 1.695443, 0.592969, 0, 1.676823, 0.513542, 0, 1.670833, 0.596875, 0, 1.695443, 0.592969, 0, 1.695443, 0.592969, 0, 1.676823, 0.513542, 0, 1.699284, 0.509635, 0, 1.676823, 0.513542, 0, 1.695443, 0.592969, 0, 1.699284, 0.509635, 0, 1.695443, 0.592969, 0, 1.699284, 0.509635, 0, 1.719271, 0.587500, 0, 1.699284, 0.509635, 0, 1.695443, 0.592969, 0, 1.719271, 0.587500, 0, 1.742253, 0.580469, 0, 1.719271, 0.587500, 0, 1.699284, 0.509635, 0, 1.719271, 0.587500, 0, 1.742253, 0.580469, 0, 1.699284, 0.509635, 0, 1.742253, 0.580469, 0, 1.699284, 0.509635, 0, 1.721354, 0.504167, 0, 1.699284, 0.509635, 0, 1.742253, 0.580469, 0, 1.721354, 0.504167, 0, 1.742253, 0.580469, 0, 1.721354, 0.504167, 0, 1.764323, 0.571875, 0, 1.721354, 0.504167, 0, 1.742253, 0.580469, 0, 1.764323, 0.571875, 0, 1.721354, 0.504167, 0, 1.743034, 0.497135, 0, 1.764323, 0.571875, 0, 1.743034, 0.497135, 0, 1.721354, 0.504167, 0, 1.764323, 0.571875, 0, 1.764323, 0.571875, 0, 1.743034, 0.497135, 0, 1.785482, 0.561719, 0, 1.743034, 0.497135, 0, 1.764323, 0.571875, 0, 1.785482, 0.561719, 0, 1.743034, 0.497135, 0, 1.764323, 0.488542, 0, 1.785482, 0.561719, 0, 1.764323, 0.488542, 0, 1.743034, 0.497135, 0, 1.785482, 0.561719, 0, 1.785482, 0.561719, 0, 1.764323, 0.488542, 0, 1.805729, 0.550000, 0, 1.764323, 0.488542, 0, 1.785482, 0.561719, 0, 1.805729, 0.550000, 0, 1.764323, 0.488542, 0, 1.785221, 0.478385, 0, 1.805729, 0.550000, 0, 1.785221, 0.478385, 0, 1.764323, 0.488542, 0, 1.805729, 0.550000, 0, 1.785221, 0.478385, 0, 1.805729, 0.466667, 0, 1.805729, 0.550000, 0, 1.805729, 0.466667, 0, 1.785221, 0.478385, 0, 1.805729, 0.550000, 0, 1.983854, 0.583333, 0, 2.079688, 0.583333, 0, 1.983854, 0.583333, 0, 1.983854, 0.583333, 0, 2.079688, 0.583333, 0, 2.079688, 0.583333, 0, 2.079688, 0.583333, 0, 2.079688, 0, 0, 2.079688, 0.583333, 0, 2.079688, 0.583333, 0, 2.079688, 0, 0, 2.079688, 0, 0, 2.079688, 0, 0, 1.983854, 0, 0, 2.079688, 0, 0, 2.079688, 0, 0, 1.983854, 0, 0, 1.983854, 0, 0, 1.983854, 0.583333, 0, 2.079688, 0, 0, 2.079688, 0.583333, 0, 2.079688, 0, 0, 1.983854, 0.583333, 0, 2.079688, 0.583333, 0, 1.983854, 0.583333, 0, 1.983854, 0, 0, 2.079688, 0, 0, 1.983854, 0, 0, 1.983854, 0.583333, 0, 2.079688, 0, 0, 1.983854, 0.816667, 0, 2.079688, 0.816667, 0, 1.983854, 0.816667, 0, 1.983854, 0.816667, 0, 2.079688, 0.816667, 0, 2.079688, 0.816667, 0, 2.079688, 0.816667, 0, 2.079688, 0.700000, 0, 2.079688, 0.816667, 0, 2.079688, 0.816667, 0, 2.079688, 0.700000, 0, 2.079688, 0.700000, 0, 2.079688, 0.700000, 0, 1.983854, 0.700000, 0, 2.079688, 0.700000, 0, 2.079688, 0.700000, 0, 1.983854, 0.700000, 0, 1.983854, 0.700000, 0, 1.983854, 0.816667, 0, 2.079688, 0.700000, 0, 2.079688, 0.816667, 0, 2.079688, 0.700000, 0, 1.983854, 0.816667, 0, 2.079688, 0.816667, 0, 1.983854, 0.816667, 0, 1.983854, 0.700000, 0, 2.079688, 0.700000, 0, 1.983854, 0.700000, 0, 1.983854, 0.816667, 0, 2.079688, 0.700000, 0, 2.655729, 0.550000, 0, 2.655729, 0.466667, 0, 2.655729, 0.550000, 0, 2.655729, 0.550000, 0, 2.655729, 0.466667, 0, 2.655729, 0.466667, 0, 2.655729, 0.466667, 0, 2.635221, 0.478385, 0, 2.655729, 0.466667, 0, 2.655729, 0.466667, 0, 2.635221, 0.478385, 0, 2.635221, 0.478385, 0, 2.635221, 0.478385, 0, 2.614323, 0.488542, 0, 2.635221, 0.478385, 0, 2.635221, 0.478385, 0, 2.614323, 0.488542, 0, 2.614323, 0.488542, 0, 2.614323, 0.488542, 0, 2.593034, 0.497135, 0, 2.614323, 0.488542, 0, 2.614323, 0.488542, 0, 2.593034, 0.497135, 0, 2.593034, 0.497135, 0, 2.593034, 0.497135, 0, 2.571354, 0.504167, 0, 2.593034, 0.497135, 0, 2.593034, 0.497135, 0, 2.571354, 0.504167, 0, 2.571354, 0.504167, 0, 2.571354, 0.504167, 0, 2.549284, 0.509635, 0, 2.571354, 0.504167, 0, 2.571354, 0.504167, 0, 2.549284, 0.509635, 0, 2.549284, 0.509635, 0, 2.549284, 0.509635, 0, 2.526823, 0.513542, 0, 2.549284, 0.509635, 0, 2.549284, 0.509635, 0, 2.526823, 0.513542, 0, 2.526823, 0.513542, 0, 2.526823, 0.513542, 0, 2.503971, 0.515885, 0, 2.526823, 0.513542, 0, 2.526823, 0.513542, 0, 2.503971, 0.515885, 0, 2.503971, 0.515885, 0, 2.503971, 0.515885, 0, 2.480729, 0.516667, 0, 2.503971, 0.515885, 0, 2.503971, 0.515885, 0, 2.480729, 0.516667, 0, 2.480729, 0.516667, 0, 2.480729, 0.516667, 0, 2.447282, 0.515251, 0, 2.480729, 0.516667, 0, 2.480729, 0.516667, 0, 2.447282, 0.515251, 0, 2.447282, 0.515251, 0, 2.447282, 0.515251, 0, 2.418294, 0.511003, 0, 2.447282, 0.515251, 0, 2.447282, 0.515251, 0, 2.418294, 0.511003, 0, 2.418294, 0.511003, 0, 2.418294, 0.511003, 0, 2.393766, 0.503923, 0, 2.418294, 0.511003, 0, 2.418294, 0.511003, 0, 2.393766, 0.503923, 0, 2.393766, 0.503923, 0, 2.393766, 0.503923, 0, 2.373698, 0.494010, 0, 2.393766, 0.503923, 0, 2.393766, 0.503923, 0, 2.373698, 0.494010, 0, 2.373698, 0.494010, 0, 2.373698, 0.494010, 0, 2.358089, 0.481266, 0, 2.373698, 0.494010, 0, 2.373698, 0.494010, 0, 2.358089, 0.481266, 0, 2.358089, 0.481266, 0, 2.358089, 0.481266, 0, 2.346940, 0.465690, 0, 2.358089, 0.481266, 0, 2.358089, 0.481266, 0, 2.346940, 0.465690, 0, 2.346940, 0.465690, 0, 2.346940, 0.465690, 0, 2.340251, 0.447282, 0, 2.346940, 0.465690, 0, 2.346940, 0.465690, 0, 2.340251, 0.447282, 0, 2.340251, 0.447282, 0, 2.340251, 0.447282, 0, 2.338021, 0.426042, 0, 2.340251, 0.447282, 0, 2.340251, 0.447282, 0, 2.338021, 0.426042, 0, 2.338021, 0.426042, 0, 2.338021, 0.426042, 0, 2.339616, 0.409766, 0, 2.338021, 0.426042, 0, 2.338021, 0.426042, 0, 2.339616, 0.409766, 0, 2.339616, 0.409766, 0, 2.339616, 0.409766, 0, 2.344401, 0.395313, 0, 2.339616, 0.409766, 0, 2.339616, 0.409766, 0, 2.344401, 0.395313, 0, 2.344401, 0.395313, 0, 2.344401, 0.395313, 0, 2.352376, 0.382682, 0, 2.344401, 0.395313, 0, 2.344401, 0.395313, 0, 2.352376, 0.382682, 0, 2.352376, 0.382682, 0, 2.352376, 0.382682, 0, 2.363542, 0.371875, 0, 2.352376, 0.382682, 0, 2.352376, 0.382682, 0, 2.363542, 0.371875, 0, 2.363542, 0.371875, 0, 2.363542, 0.371875, 0, 2.379525, 0.362109, 0, 2.363542, 0.371875, 0, 2.363542, 0.371875, 0, 2.379525, 0.362109, 0, 2.379525, 0.362109, 0, 2.379525, 0.362109, 0, 2.401953, 0.352604, 0, 2.379525, 0.362109, 0, 2.379525, 0.362109, 0, 2.401953, 0.352604, 0, 2.401953, 0.352604, 0, 2.401953, 0.352604, 0, 2.430827, 0.343359, 0, 2.401953, 0.352604, 0, 2.401953, 0.352604, 0, 2.430827, 0.343359, 0, 2.430827, 0.343359, 0, 2.430827, 0.343359, 0, 2.466146, 0.334375, 0, 2.430827, 0.343359, 0, 2.430827, 0.343359, 0, 2.466146, 0.334375, 0, 2.466146, 0.334375, 0, 2.466146, 0.334375, 0, 2.498958, 0.327083, 0, 2.466146, 0.334375, 0, 2.466146, 0.334375, 0, 2.498958, 0.327083, 0, 2.498958, 0.327083, 0, 2.498958, 0.327083, 0, 2.546305, 0.315177, 0, 2.498958, 0.327083, 0, 2.498958, 0.327083, 0, 2.546305, 0.315177, 0, 2.546305, 0.315177, 0, 2.546305, 0.315177, 0, 2.586263, 0.301074, 0, 2.546305, 0.315177, 0, 2.546305, 0.315177, 0, 2.586263, 0.301074, 0, 2.586263, 0.301074, 0, 2.586263, 0.301074, 0, 2.618831, 0.284774, 0, 2.586263, 0.301074, 0, 2.586263, 0.301074, 0, 2.618831, 0.284774, 0, 2.618831, 0.284774, 0, 2.618831, 0.284774, 0, 2.644010, 0.266276, 0, 2.618831, 0.284774, 0, 2.618831, 0.284774, 0, 2.644010, 0.266276, 0, 2.644010, 0.266276, 0, 2.644010, 0.266276, 0, 2.662809, 0.244735, 0, 2.644010, 0.266276, 0, 2.644010, 0.266276, 0, 2.662809, 0.244735, 0, 2.662809, 0.244735, 0, 2.662809, 0.244735, 0, 2.676237, 0.219303, 0, 2.662809, 0.244735, 0, 2.662809, 0.244735, 0, 2.676237, 0.219303, 0, 2.676237, 0.219303, 0, 2.676237, 0.219303, 0, 2.684294, 0.189982, 0, 2.676237, 0.219303, 0, 2.676237, 0.219303, 0, 2.684294, 0.189982, 0, 2.684294, 0.189982, 0, 2.684294, 0.189982, 0, 2.686979, 0.156771, 0, 2.684294, 0.189982, 0, 2.684294, 0.189982, 0, 2.686979, 0.156771, 0, 2.686979, 0.156771, 0, 2.686979, 0.156771, 0, 2.682959, 0.118872, 0, 2.686979, 0.156771, 0, 2.686979, 0.156771, 0, 2.682959, 0.118872, 0, 2.682959, 0.118872, 0, 2.682959, 0.118872, 0, 2.670898, 0.085124, 0, 2.682959, 0.118872, 0, 2.682959, 0.118872, 0, 2.670898, 0.085124, 0, 2.670898, 0.085124, 0, 2.670898, 0.085124, 0, 2.650798, 0.055526, 0, 2.670898, 0.085124, 0, 2.670898, 0.085124, 0, 2.650798, 0.055526, 0, 2.650798, 0.055526, 0, 2.650798, 0.055526, 0, 2.622656, 0.030078, 0, 2.650798, 0.055526, 0, 2.650798, 0.055526, 0, 2.622656, 0.030078, 0, 2.622656, 0.030078, 0, 2.622656, 0.030078, 0, 2.587484, 0.009627, 0, 2.622656, 0.030078, 0, 2.622656, 0.030078, 0, 2.587484, 0.009627, 0, 2.587484, 0.009627, 0, 2.587484, 0.009627, 0, 2.546289, -0.004980, 0, 2.587484, 0.009627, 0, 2.587484, 0.009627, 0, 2.546289, -0.004980, 0, 2.546289, -0.004980, 0, 2.546289, -0.004980, 0, 2.499072, -0.013745, 0, 2.546289, -0.004980, 0, 2.546289, -0.004980, 0, 2.499072, -0.013745, 0, 2.499072, -0.013745, 0, 2.499072, -0.013745, 0, 2.445833, -0.016667, 0, 2.499072, -0.013745, 0, 2.499072, -0.013745, 0, 2.445833, -0.016667, 0, 2.445833, -0.016667, 0, 2.445833, -0.016667, 0, 2.422152, -0.016146, 0, 2.445833, -0.016667, 0, 2.445833, -0.016667, 0, 2.422152, -0.016146, 0, 2.422152, -0.016146, 0, 2.422152, -0.016146, 0, 2.397982, -0.014583, 0, 2.422152, -0.016146, 0, 2.422152, -0.016146, 0, 2.397982, -0.014583, 0, 2.397982, -0.014583, 0, 2.397982, -0.014583, 0, 2.373324, -0.011979, 0, 2.397982, -0.014583, 0, 2.397982, -0.014583, 0, 2.373324, -0.011979, 0, 2.373324, -0.011979, 0, 2.373324, -0.011979, 0, 2.348177, -0.008333, 0, 2.373324, -0.011979, 0, 2.373324, -0.011979, 0, 2.348177, -0.008333, 0, 2.348177, -0.008333, 0, 2.348177, -0.008333, 0, 2.322445, -0.003646, 0, 2.348177, -0.008333, 0, 2.348177, -0.008333, 0, 2.322445, -0.003646, 0, 2.322445, -0.003646, 0, 2.322445, -0.003646, 0, 2.296029, 0.002083, 0, 2.322445, -0.003646, 0, 2.322445, -0.003646, 0, 2.296029, 0.002083, 0, 2.296029, 0.002083, 0, 2.296029, 0.002083, 0, 2.268929, 0.008854, 0, 2.296029, 0.002083, 0, 2.296029, 0.002083, 0, 2.268929, 0.008854, 0, 2.268929, 0.008854, 0, 2.268929, 0.008854, 0, 2.241146, 0.016667, 0, 2.268929, 0.008854, 0, 2.268929, 0.008854, 0, 2.241146, 0.016667, 0, 2.241146, 0.016667, 0, 2.241146, 0.016667, 0, 2.241146, 0.116667, 0, 2.241146, 0.016667, 0, 2.241146, 0.016667, 0, 2.241146, 0.116667, 0, 2.241146, 0.116667, 0, 2.241146, 0.116667, 0, 2.267611, 0.104948, 0, 2.241146, 0.116667, 0, 2.241146, 0.116667, 0, 2.267611, 0.104948, 0, 2.267611, 0.104948, 0, 2.267611, 0.104948, 0, 2.293880, 0.094792, 0, 2.267611, 0.104948, 0, 2.267611, 0.104948, 0, 2.293880, 0.094792, 0, 2.293880, 0.094792, 0, 2.293880, 0.094792, 0, 2.319955, 0.086198, 0, 2.293880, 0.094792, 0, 2.293880, 0.094792, 0, 2.319955, 0.086198, 0, 2.319955, 0.086198, 0, 2.319955, 0.086198, 0, 2.345834, 0.079167, 0, 2.319955, 0.086198, 0, 2.319955, 0.086198, 0, 2.345834, 0.079167, 0, 2.345834, 0.079167, 0, 2.345834, 0.079167, 0, 2.371549, 0.073698, 0, 2.345834, 0.079167, 0, 2.345834, 0.079167, 0, 2.371549, 0.073698, 0, 2.371549, 0.073698, 0, 2.371549, 0.073698, 0, 2.397135, 0.069792, 0, 2.371549, 0.073698, 0, 2.371549, 0.073698, 0, 2.397135, 0.069792, 0, 2.397135, 0.069792, 0, 2.397135, 0.069792, 0, 2.422591, 0.067448, 0, 2.397135, 0.069792, 0, 2.397135, 0.069792, 0, 2.422591, 0.067448, 0, 2.422591, 0.067448, 0, 2.422591, 0.067448, 0, 2.447917, 0.066667, 0, 2.422591, 0.067448, 0, 2.422591, 0.067448, 0, 2.447917, 0.066667, 0, 2.447917, 0.066667, 0, 2.447917, 0.066667, 0, 2.479818, 0.068058, 0, 2.447917, 0.066667, 0, 2.447917, 0.066667, 0, 2.479818, 0.068058, 0, 2.479818, 0.068058, 0, 2.479818, 0.068058, 0, 2.507813, 0.072233, 0, 2.479818, 0.068058, 0, 2.479818, 0.068058, 0, 2.507813, 0.072233, 0, 2.507813, 0.072233, 0, 2.507813, 0.072233, 0, 2.531901, 0.079191, 0, 2.507813, 0.072233, 0, 2.507813, 0.072233, 0, 2.531901, 0.079191, 0, 2.531901, 0.079191, 0, 2.531901, 0.079191, 0, 2.552083, 0.088932, 0, 2.531901, 0.079191, 0, 2.531901, 0.079191, 0, 2.552083, 0.088932, 0, 2.552083, 0.088932, 0, 2.552083, 0.088932, 0, 2.568034, 0.101213, 0, 2.552083, 0.088932, 0, 2.552083, 0.088932, 0, 2.568034, 0.101213, 0, 2.568034, 0.101213, 0, 2.568034, 0.101213, 0, 2.579427, 0.115788, 0, 2.568034, 0.101213, 0, 2.568034, 0.101213, 0, 2.579427, 0.115788, 0, 2.579427, 0.115788, 0, 2.579427, 0.115788, 0, 2.586263, 0.132658, 0, 2.579427, 0.115788, 0, 2.579427, 0.115788, 0, 2.586263, 0.132658, 0, 2.586263, 0.132658, 0, 2.586263, 0.132658, 0, 2.588542, 0.151823, 0, 2.586263, 0.132658, 0, 2.586263, 0.132658, 0, 2.588542, 0.151823, 0, 2.588542, 0.151823, 0, 2.588542, 0.151823, 0, 2.586898, 0.169482, 0, 2.588542, 0.151823, 0, 2.588542, 0.151823, 0, 2.586898, 0.169482, 0, 2.586898, 0.169482, 0, 2.586898, 0.169482, 0, 2.581966, 0.184961, 0, 2.586898, 0.169482, 0, 2.586898, 0.169482, 0, 2.581966, 0.184961, 0, 2.581966, 0.184961, 0, 2.581966, 0.184961, 0, 2.573747, 0.198258, 0, 2.581966, 0.184961, 0, 2.581966, 0.184961, 0, 2.573747, 0.198258, 0, 2.573747, 0.198258, 0, 2.573747, 0.198258, 0, 2.562240, 0.209375, 0, 2.573747, 0.198258, 0, 2.573747, 0.198258, 0, 2.562240, 0.209375, 0, 2.562240, 0.209375, 0, 2.562240, 0.209375, 0, 2.545166, 0.219303, 0, 2.562240, 0.209375, 0, 2.562240, 0.209375, 0, 2.545166, 0.219303, 0, 2.545166, 0.219303, 0, 2.545166, 0.219303, 0, 2.520247, 0.229036, 0, 2.545166, 0.219303, 0, 2.545166, 0.219303, 0, 2.520247, 0.229036, 0, 2.520247, 0.229036, 0, 2.520247, 0.229036, 0, 2.487484, 0.238574, 0, 2.520247, 0.229036, 0, 2.520247, 0.229036, 0, 2.487484, 0.238574, 0, 2.487484, 0.238574, 0, 2.487484, 0.238574, 0, 2.446875, 0.247917, 0, 2.487484, 0.238574, 0, 2.487484, 0.238574, 0, 2.446875, 0.247917, 0, 2.446875, 0.247917, 0, 2.446875, 0.247917, 0, 2.413542, 0.255729, 0, 2.446875, 0.247917, 0, 2.446875, 0.247917, 0, 2.413542, 0.255729, 0, 2.413542, 0.255729, 0, 2.413542, 0.255729, 0, 2.372103, 0.266642, 0, 2.413542, 0.255729, 0, 2.413542, 0.255729, 0, 2.372103, 0.266642, 0, 2.372103, 0.266642, 0, 2.372103, 0.266642, 0, 2.336849, 0.280111, 0, 2.372103, 0.266642, 0, 2.372103, 0.266642, 0, 2.336849, 0.280111, 0, 2.336849, 0.280111, 0, 2.336849, 0.280111, 0, 2.307780, 0.296134, 0, 2.336849, 0.280111, 0, 2.336849, 0.280111, 0, 2.307780, 0.296134, 0, 2.307780, 0.296134, 0, 2.307780, 0.296134, 0, 2.284896, 0.314714, 0, 2.307780, 0.296134, 0, 2.307780, 0.296134, 0, 2.284896, 0.314714, 0, 2.284896, 0.314714, 0, 2.284896, 0.314714, 0, 2.267578, 0.336434, 0, 2.284896, 0.314714, 0, 2.284896, 0.314714, 0, 2.267578, 0.336434, 0, 2.267578, 0.336434, 0, 2.267578, 0.336434, 0, 2.255208, 0.361882, 0, 2.267578, 0.336434, 0, 2.267578, 0.336434, 0, 2.255208, 0.361882, 0, 2.255208, 0.361882, 0, 2.255208, 0.361882, 0, 2.247787, 0.391056, 0, 2.255208, 0.361882, 0, 2.255208, 0.361882, 0, 2.247787, 0.391056, 0, 2.247787, 0.391056, 0, 2.247787, 0.391056, 0, 2.245312, 0.423958, 0, 2.247787, 0.391056, 0, 2.247787, 0.391056, 0, 2.245312, 0.423958, 0, 2.245312, 0.423958, 0, 2.245312, 0.423958, 0, 2.248958, 0.463631, 0, 2.245312, 0.423958, 0, 2.245312, 0.423958, 0, 2.248958, 0.463631, 0, 2.248958, 0.463631, 0, 2.248958, 0.463631, 0, 2.259896, 0.498535, 0, 2.248958, 0.463631, 0, 2.248958, 0.463631, 0, 2.259896, 0.498535, 0, 2.259896, 0.498535, 0, 2.259896, 0.498535, 0, 2.278125, 0.528670, 0, 2.259896, 0.498535, 0, 2.259896, 0.498535, 0, 2.278125, 0.528670, 0, 2.278125, 0.528670, 0, 2.278125, 0.528670, 0, 2.303646, 0.554036, 0, 2.278125, 0.528670, 0, 2.278125, 0.528670, 0, 2.303646, 0.554036, 0, 2.303646, 0.554036, 0, 2.303646, 0.554036, 0, 2.335872, 0.574145, 0, 2.303646, 0.554036, 0, 2.303646, 0.554036, 0, 2.335872, 0.574145, 0, 2.335872, 0.574145, 0, 2.335872, 0.574145, 0, 2.374219, 0.588509, 0, 2.335872, 0.574145, 0, 2.335872, 0.574145, 0, 2.374219, 0.588509, 0, 2.374219, 0.588509, 0, 2.374219, 0.588509, 0, 2.418685, 0.597127, 0, 2.374219, 0.588509, 0, 2.374219, 0.588509, 0, 2.418685, 0.597127, 0, 2.418685, 0.597127, 0, 2.418685, 0.597127, 0, 2.469271, 0.600000, 0, 2.418685, 0.597127, 0, 2.418685, 0.597127, 0, 2.469271, 0.600000, 0, 2.469271, 0.600000, 0, 2.469271, 0.600000, 0, 2.495443, 0.599219, 0, 2.469271, 0.600000, 0, 2.469271, 0.600000, 0, 2.495443, 0.599219, 0, 2.495443, 0.599219, 0, 2.495443, 0.599219, 0, 2.520833, 0.596875, 0, 2.495443, 0.599219, 0, 2.495443, 0.599219, 0, 2.520833, 0.596875, 0, 2.520833, 0.596875, 0, 2.520833, 0.596875, 0, 2.545443, 0.592969, 0, 2.520833, 0.596875, 0, 2.520833, 0.596875, 0, 2.545443, 0.592969, 0, 2.545443, 0.592969, 0, 2.545443, 0.592969, 0, 2.569271, 0.587500, 0, 2.545443, 0.592969, 0, 2.545443, 0.592969, 0, 2.569271, 0.587500, 0, 2.569271, 0.587500, 0, 2.569271, 0.587500, 0, 2.592252, 0.580469, 0, 2.569271, 0.587500, 0, 2.569271, 0.587500, 0, 2.592252, 0.580469, 0, 2.592252, 0.580469, 0, 2.592252, 0.580469, 0, 2.614323, 0.571875, 0, 2.592252, 0.580469, 0, 2.592252, 0.580469, 0, 2.614323, 0.571875, 0, 2.614323, 0.571875, 0, 2.614323, 0.571875, 0, 2.635482, 0.561719, 0, 2.614323, 0.571875, 0, 2.614323, 0.571875, 0, 2.635482, 0.561719, 0, 2.635482, 0.561719, 0, 2.241146, 0.116667, 0, 2.241146, 0.016667, 0, 2.267611, 0.104948, 0, 2.241146, 0.016667, 0, 2.241146, 0.116667, 0, 2.267611, 0.104948, 0, 2.293880, 0.094792, 0, 2.267611, 0.104948, 0, 2.241146, 0.016667, 0, 2.267611, 0.104948, 0, 2.293880, 0.094792, 0, 2.241146, 0.016667, 0, 2.293880, 0.094792, 0, 2.241146, 0.016667, 0, 2.268929, 0.008854, 0, 2.241146, 0.016667, 0, 2.293880, 0.094792, 0, 2.268929, 0.008854, 0, 2.319955, 0.086198, 0, 2.293880, 0.094792, 0, 2.268929, 0.008854, 0, 2.293880, 0.094792, 0, 2.319955, 0.086198, 0, 2.268929, 0.008854, 0, 2.319955, 0.086198, 0, 2.268929, 0.008854, 0, 2.296029, 0.002083, 0, 2.268929, 0.008854, 0, 2.319955, 0.086198, 0, 2.296029, 0.002083, 0, 2.319955, 0.086198, 0, 2.296029, 0.002083, 0, 2.322445, -0.003646, 0, 2.296029, 0.002083, 0, 2.319955, 0.086198, 0, 2.322445, -0.003646, 0, 2.319955, 0.086198, 0, 2.322445, -0.003646, 0, 2.345834, 0.079167, 0, 2.322445, -0.003646, 0, 2.319955, 0.086198, 0, 2.345834, 0.079167, 0, 2.345834, 0.079167, 0, 2.322445, -0.003646, 0, 2.348177, -0.008333, 0, 2.322445, -0.003646, 0, 2.345834, 0.079167, 0, 2.348177, -0.008333, 0, 2.345834, 0.079167, 0, 2.348177, -0.008333, 0, 2.371549, 0.073698, 0, 2.348177, -0.008333, 0, 2.345834, 0.079167, 0, 2.371549, 0.073698, 0, 2.371549, 0.073698, 0, 2.348177, -0.008333, 0, 2.373324, -0.011979, 0, 2.348177, -0.008333, 0, 2.371549, 0.073698, 0, 2.373324, -0.011979, 0, 2.371549, 0.073698, 0, 2.373324, -0.011979, 0, 2.397135, 0.069792, 0, 2.373324, -0.011979, 0, 2.371549, 0.073698, 0, 2.397135, 0.069792, 0, 2.397135, 0.069792, 0, 2.373324, -0.011979, 0, 2.397982, -0.014583, 0, 2.373324, -0.011979, 0, 2.397135, 0.069792, 0, 2.397982, -0.014583, 0, 2.397135, 0.069792, 0, 2.397982, -0.014583, 0, 2.422591, 0.067448, 0, 2.397982, -0.014583, 0, 2.397135, 0.069792, 0, 2.422591, 0.067448, 0, 2.397982, -0.014583, 0, 2.422152, -0.016146, 0, 2.422591, 0.067448, 0, 2.422152, -0.016146, 0, 2.397982, -0.014583, 0, 2.422591, 0.067448, 0, 2.422591, 0.067448, 0, 2.422152, -0.016146, 0, 2.447917, 0.066667, 0, 2.422152, -0.016146, 0, 2.422591, 0.067448, 0, 2.447917, 0.066667, 0, 2.422152, -0.016146, 0, 2.445833, -0.016667, 0, 2.447917, 0.066667, 0, 2.445833, -0.016667, 0, 2.422152, -0.016146, 0, 2.447917, 0.066667, 0, 2.479818, 0.068058, 0, 2.447917, 0.066667, 0, 2.445833, -0.016667, 0, 2.447917, 0.066667, 0, 2.479818, 0.068058, 0, 2.445833, -0.016667, 0, 2.499072, -0.013745, 0, 2.479818, 0.068058, 0, 2.445833, -0.016667, 0, 2.479818, 0.068058, 0, 2.499072, -0.013745, 0, 2.445833, -0.016667, 0, 2.507813, 0.072233, 0, 2.479818, 0.068058, 0, 2.499072, -0.013745, 0, 2.479818, 0.068058, 0, 2.507813, 0.072233, 0, 2.499072, -0.013745, 0, 2.546289, -0.004980, 0, 2.507813, 0.072233, 0, 2.499072, -0.013745, 0, 2.507813, 0.072233, 0, 2.546289, -0.004980, 0, 2.499072, -0.013745, 0, 2.531901, 0.079191, 0, 2.507813, 0.072233, 0, 2.546289, -0.004980, 0, 2.507813, 0.072233, 0, 2.531901, 0.079191, 0, 2.546289, -0.004980, 0, 2.531901, 0.079191, 0, 2.546289, -0.004980, 0, 2.587484, 0.009627, 0, 2.546289, -0.004980, 0, 2.531901, 0.079191, 0, 2.587484, 0.009627, 0, 2.552083, 0.088932, 0, 2.531901, 0.079191, 0, 2.587484, 0.009627, 0, 2.531901, 0.079191, 0, 2.552083, 0.088932, 0, 2.587484, 0.009627, 0, 2.552083, 0.088932, 0, 2.587484, 0.009627, 0, 2.622656, 0.030078, 0, 2.587484, 0.009627, 0, 2.552083, 0.088932, 0, 2.622656, 0.030078, 0, 2.568034, 0.101213, 0, 2.552083, 0.088932, 0, 2.622656, 0.030078, 0, 2.552083, 0.088932, 0, 2.568034, 0.101213, 0, 2.622656, 0.030078, 0, 2.568034, 0.101213, 0, 2.622656, 0.030078, 0, 2.650798, 0.055526, 0, 2.622656, 0.030078, 0, 2.568034, 0.101213, 0, 2.650798, 0.055526, 0, 2.579427, 0.115788, 0, 2.568034, 0.101213, 0, 2.650798, 0.055526, 0, 2.568034, 0.101213, 0, 2.579427, 0.115788, 0, 2.650798, 0.055526, 0, 2.579427, 0.115788, 0, 2.650798, 0.055526, 0, 2.670898, 0.085124, 0, 2.650798, 0.055526, 0, 2.579427, 0.115788, 0, 2.670898, 0.085124, 0, 2.586263, 0.132658, 0, 2.579427, 0.115788, 0, 2.670898, 0.085124, 0, 2.579427, 0.115788, 0, 2.586263, 0.132658, 0, 2.670898, 0.085124, 0, 2.586263, 0.132658, 0, 2.670898, 0.085124, 0, 2.682959, 0.118872, 0, 2.670898, 0.085124, 0, 2.586263, 0.132658, 0, 2.682959, 0.118872, 0, 2.588542, 0.151823, 0, 2.586263, 0.132658, 0, 2.682959, 0.118872, 0, 2.586263, 0.132658, 0, 2.588542, 0.151823, 0, 2.682959, 0.118872, 0, 2.588542, 0.151823, 0, 2.682959, 0.118872, 0, 2.686979, 0.156771, 0, 2.682959, 0.118872, 0, 2.588542, 0.151823, 0, 2.686979, 0.156771, 0, 2.686979, 0.156771, 0, 2.586898, 0.169482, 0, 2.588542, 0.151823, 0, 2.586898, 0.169482, 0, 2.686979, 0.156771, 0, 2.588542, 0.151823, 0, 2.684294, 0.189982, 0, 2.586898, 0.169482, 0, 2.686979, 0.156771, 0, 2.586898, 0.169482, 0, 2.684294, 0.189982, 0, 2.686979, 0.156771, 0, 2.676237, 0.219303, 0, 2.586898, 0.169482, 0, 2.684294, 0.189982, 0, 2.586898, 0.169482, 0, 2.676237, 0.219303, 0, 2.684294, 0.189982, 0, 2.581966, 0.184961, 0, 2.586898, 0.169482, 0, 2.676237, 0.219303, 0, 2.586898, 0.169482, 0, 2.581966, 0.184961, 0, 2.676237, 0.219303, 0, 2.662809, 0.244735, 0, 2.581966, 0.184961, 0, 2.676237, 0.219303, 0, 2.581966, 0.184961, 0, 2.662809, 0.244735, 0, 2.676237, 0.219303, 0, 2.573747, 0.198258, 0, 2.581966, 0.184961, 0, 2.662809, 0.244735, 0, 2.581966, 0.184961, 0, 2.573747, 0.198258, 0, 2.662809, 0.244735, 0, 2.644010, 0.266276, 0, 2.573747, 0.198258, 0, 2.662809, 0.244735, 0, 2.573747, 0.198258, 0, 2.644010, 0.266276, 0, 2.662809, 0.244735, 0, 2.618831, 0.284774, 0, 2.573747, 0.198258, 0, 2.644010, 0.266276, 0, 2.573747, 0.198258, 0, 2.618831, 0.284774, 0, 2.644010, 0.266276, 0, 2.562240, 0.209375, 0, 2.573747, 0.198258, 0, 2.618831, 0.284774, 0, 2.573747, 0.198258, 0, 2.562240, 0.209375, 0, 2.618831, 0.284774, 0, 2.586263, 0.301074, 0, 2.562240, 0.209375, 0, 2.618831, 0.284774, 0, 2.562240, 0.209375, 0, 2.586263, 0.301074, 0, 2.618831, 0.284774, 0, 2.545166, 0.219303, 0, 2.562240, 0.209375, 0, 2.586263, 0.301074, 0, 2.562240, 0.209375, 0, 2.545166, 0.219303, 0, 2.586263, 0.301074, 0, 2.546305, 0.315177, 0, 2.545166, 0.219303, 0, 2.586263, 0.301074, 0, 2.545166, 0.219303, 0, 2.546305, 0.315177, 0, 2.586263, 0.301074, 0, 2.520247, 0.229036, 0, 2.545166, 0.219303, 0, 2.546305, 0.315177, 0, 2.545166, 0.219303, 0, 2.520247, 0.229036, 0, 2.546305, 0.315177, 0, 2.546305, 0.315177, 0, 2.487484, 0.238574, 0, 2.520247, 0.229036, 0, 2.487484, 0.238574, 0, 2.546305, 0.315177, 0, 2.520247, 0.229036, 0, 2.546305, 0.315177, 0, 2.498958, 0.327083, 0, 2.487484, 0.238574, 0, 2.498958, 0.327083, 0, 2.546305, 0.315177, 0, 2.487484, 0.238574, 0, 2.446875, 0.247917, 0, 2.487484, 0.238574, 0, 2.498958, 0.327083, 0, 2.487484, 0.238574, 0, 2.446875, 0.247917, 0, 2.498958, 0.327083, 0, 2.466146, 0.334375, 0, 2.446875, 0.247917, 0, 2.498958, 0.327083, 0, 2.446875, 0.247917, 0, 2.466146, 0.334375, 0, 2.498958, 0.327083, 0, 2.413542, 0.255729, 0, 2.446875, 0.247917, 0, 2.466146, 0.334375, 0, 2.446875, 0.247917, 0, 2.413542, 0.255729, 0, 2.466146, 0.334375, 0, 2.430827, 0.343359, 0, 2.413542, 0.255729, 0, 2.466146, 0.334375, 0, 2.413542, 0.255729, 0, 2.430827, 0.343359, 0, 2.466146, 0.334375, 0, 2.372103, 0.266642, 0, 2.413542, 0.255729, 0, 2.430827, 0.343359, 0, 2.413542, 0.255729, 0, 2.372103, 0.266642, 0, 2.430827, 0.343359, 0, 2.401953, 0.352604, 0, 2.372103, 0.266642, 0, 2.430827, 0.343359, 0, 2.372103, 0.266642, 0, 2.401953, 0.352604, 0, 2.430827, 0.343359, 0, 2.401953, 0.352604, 0, 2.379525, 0.362109, 0, 2.372103, 0.266642, 0, 2.379525, 0.362109, 0, 2.401953, 0.352604, 0, 2.372103, 0.266642, 0, 2.379525, 0.362109, 0, 2.336849, 0.280111, 0, 2.372103, 0.266642, 0, 2.336849, 0.280111, 0, 2.379525, 0.362109, 0, 2.372103, 0.266642, 0, 2.379525, 0.362109, 0, 2.363542, 0.371875, 0, 2.336849, 0.280111, 0, 2.363542, 0.371875, 0, 2.379525, 0.362109, 0, 2.336849, 0.280111, 0, 2.363542, 0.371875, 0, 2.307780, 0.296134, 0, 2.336849, 0.280111, 0, 2.307780, 0.296134, 0, 2.363542, 0.371875, 0, 2.336849, 0.280111, 0, 2.284896, 0.314714, 0, 2.307780, 0.296134, 0, 2.363542, 0.371875, 0, 2.307780, 0.296134, 0, 2.284896, 0.314714, 0, 2.363542, 0.371875, 0, 2.363542, 0.371875, 0, 2.352376, 0.382682, 0, 2.284896, 0.314714, 0, 2.352376, 0.382682, 0, 2.363542, 0.371875, 0, 2.284896, 0.314714, 0, 2.352376, 0.382682, 0, 2.267578, 0.336434, 0, 2.284896, 0.314714, 0, 2.267578, 0.336434, 0, 2.352376, 0.382682, 0, 2.284896, 0.314714, 0, 2.352376, 0.382682, 0, 2.344401, 0.395313, 0, 2.267578, 0.336434, 0, 2.344401, 0.395313, 0, 2.352376, 0.382682, 0, 2.267578, 0.336434, 0, 2.344401, 0.395313, 0, 2.255208, 0.361882, 0, 2.267578, 0.336434, 0, 2.255208, 0.361882, 0, 2.344401, 0.395313, 0, 2.267578, 0.336434, 0, 2.344401, 0.395313, 0, 2.339616, 0.409766, 0, 2.255208, 0.361882, 0, 2.339616, 0.409766, 0, 2.344401, 0.395313, 0, 2.255208, 0.361882, 0, 2.339616, 0.409766, 0, 2.247787, 0.391056, 0, 2.255208, 0.361882, 0, 2.247787, 0.391056, 0, 2.339616, 0.409766, 0, 2.255208, 0.361882, 0, 2.245312, 0.423958, 0, 2.247787, 0.391056, 0, 2.339616, 0.409766, 0, 2.247787, 0.391056, 0, 2.245312, 0.423958, 0, 2.339616, 0.409766, 0, 2.338021, 0.426042, 0, 2.245312, 0.423958, 0, 2.339616, 0.409766, 0, 2.245312, 0.423958, 0, 2.338021, 0.426042, 0, 2.339616, 0.409766, 0, 2.248958, 0.463631, 0, 2.245312, 0.423958, 0, 2.338021, 0.426042, 0, 2.245312, 0.423958, 0, 2.248958, 0.463631, 0, 2.338021, 0.426042, 0, 2.248958, 0.463631, 0, 2.338021, 0.426042, 0, 2.340251, 0.447282, 0, 2.338021, 0.426042, 0, 2.248958, 0.463631, 0, 2.340251, 0.447282, 0, 2.259896, 0.498535, 0, 2.248958, 0.463631, 0, 2.340251, 0.447282, 0, 2.248958, 0.463631, 0, 2.259896, 0.498535, 0, 2.340251, 0.447282, 0, 2.259896, 0.498535, 0, 2.340251, 0.447282, 0, 2.346940, 0.465690, 0, 2.340251, 0.447282, 0, 2.259896, 0.498535, 0, 2.346940, 0.465690, 0, 2.278125, 0.528670, 0, 2.259896, 0.498535, 0, 2.346940, 0.465690, 0, 2.259896, 0.498535, 0, 2.278125, 0.528670, 0, 2.346940, 0.465690, 0, 2.278125, 0.528670, 0, 2.346940, 0.465690, 0, 2.358089, 0.481266, 0, 2.346940, 0.465690, 0, 2.278125, 0.528670, 0, 2.358089, 0.481266, 0, 2.303646, 0.554036, 0, 2.278125, 0.528670, 0, 2.358089, 0.481266, 0, 2.278125, 0.528670, 0, 2.303646, 0.554036, 0, 2.358089, 0.481266, 0, 2.303646, 0.554036, 0, 2.358089, 0.481266, 0, 2.373698, 0.494010, 0, 2.358089, 0.481266, 0, 2.303646, 0.554036, 0, 2.373698, 0.494010, 0, 2.335872, 0.574145, 0, 2.303646, 0.554036, 0, 2.373698, 0.494010, 0, 2.303646, 0.554036, 0, 2.335872, 0.574145, 0, 2.373698, 0.494010, 0, 2.335872, 0.574145, 0, 2.373698, 0.494010, 0, 2.393766, 0.503923, 0, 2.373698, 0.494010, 0, 2.335872, 0.574145, 0, 2.393766, 0.503923, 0, 2.374219, 0.588509, 0, 2.335872, 0.574145, 0, 2.393766, 0.503923, 0, 2.335872, 0.574145, 0, 2.374219, 0.588509, 0, 2.393766, 0.503923, 0, 2.374219, 0.588509, 0, 2.393766, 0.503923, 0, 2.418294, 0.511003, 0, 2.393766, 0.503923, 0, 2.374219, 0.588509, 0, 2.418294, 0.511003, 0, 2.374219, 0.588509, 0, 2.418294, 0.511003, 0, 2.418685, 0.597127, 0, 2.418294, 0.511003, 0, 2.374219, 0.588509, 0, 2.418685, 0.597127, 0, 2.418685, 0.597127, 0, 2.418294, 0.511003, 0, 2.447282, 0.515251, 0, 2.418294, 0.511003, 0, 2.418685, 0.597127, 0, 2.447282, 0.515251, 0, 2.469271, 0.600000, 0, 2.418685, 0.597127, 0, 2.447282, 0.515251, 0, 2.418685, 0.597127, 0, 2.469271, 0.600000, 0, 2.447282, 0.515251, 0, 2.469271, 0.600000, 0, 2.447282, 0.515251, 0, 2.480729, 0.516667, 0, 2.447282, 0.515251, 0, 2.469271, 0.600000, 0, 2.480729, 0.516667, 0, 2.480729, 0.516667, 0, 2.495443, 0.599219, 0, 2.469271, 0.600000, 0, 2.495443, 0.599219, 0, 2.480729, 0.516667, 0, 2.469271, 0.600000, 0, 2.495443, 0.599219, 0, 2.480729, 0.516667, 0, 2.503971, 0.515885, 0, 2.480729, 0.516667, 0, 2.495443, 0.599219, 0, 2.503971, 0.515885, 0, 2.520833, 0.596875, 0, 2.495443, 0.599219, 0, 2.503971, 0.515885, 0, 2.495443, 0.599219, 0, 2.520833, 0.596875, 0, 2.503971, 0.515885, 0, 2.520833, 0.596875, 0, 2.503971, 0.515885, 0, 2.526823, 0.513542, 0, 2.503971, 0.515885, 0, 2.520833, 0.596875, 0, 2.526823, 0.513542, 0, 2.520833, 0.596875, 0, 2.526823, 0.513542, 0, 2.545443, 0.592969, 0, 2.526823, 0.513542, 0, 2.520833, 0.596875, 0, 2.545443, 0.592969, 0, 2.545443, 0.592969, 0, 2.526823, 0.513542, 0, 2.549284, 0.509635, 0, 2.526823, 0.513542, 0, 2.545443, 0.592969, 0, 2.549284, 0.509635, 0, 2.545443, 0.592969, 0, 2.549284, 0.509635, 0, 2.569271, 0.587500, 0, 2.549284, 0.509635, 0, 2.545443, 0.592969, 0, 2.569271, 0.587500, 0, 2.592252, 0.580469, 0, 2.569271, 0.587500, 0, 2.549284, 0.509635, 0, 2.569271, 0.587500, 0, 2.592252, 0.580469, 0, 2.549284, 0.509635, 0, 2.592252, 0.580469, 0, 2.549284, 0.509635, 0, 2.571354, 0.504167, 0, 2.549284, 0.509635, 0, 2.592252, 0.580469, 0, 2.571354, 0.504167, 0, 2.592252, 0.580469, 0, 2.571354, 0.504167, 0, 2.614323, 0.571875, 0, 2.571354, 0.504167, 0, 2.592252, 0.580469, 0, 2.614323, 0.571875, 0, 2.571354, 0.504167, 0, 2.593034, 0.497135, 0, 2.614323, 0.571875, 0, 2.593034, 0.497135, 0, 2.571354, 0.504167, 0, 2.614323, 0.571875, 0, 2.614323, 0.571875, 0, 2.593034, 0.497135, 0, 2.635482, 0.561719, 0, 2.593034, 0.497135, 0, 2.614323, 0.571875, 0, 2.635482, 0.561719, 0, 2.593034, 0.497135, 0, 2.614323, 0.488542, 0, 2.635482, 0.561719, 0, 2.614323, 0.488542, 0, 2.593034, 0.497135, 0, 2.635482, 0.561719, 0, 2.635482, 0.561719, 0, 2.614323, 0.488542, 0, 2.655729, 0.550000, 0, 2.614323, 0.488542, 0, 2.635482, 0.561719, 0, 2.655729, 0.550000, 0, 2.614323, 0.488542, 0, 2.635221, 0.478385, 0, 2.655729, 0.550000, 0, 2.635221, 0.478385, 0, 2.614323, 0.488542, 0, 2.655729, 0.550000, 0, 2.635221, 0.478385, 0, 2.655729, 0.466667, 0, 2.655729, 0.550000, 0, 2.655729, 0.466667, 0, 2.635221, 0.478385, 0, 2.655729, 0.550000, 0, 2.928646, 0.750000, 0, 2.928646, 0.583333, 0, 2.928646, 0.750000, 0, 2.928646, 0.750000, 0, 2.928646, 0.583333, 0, 2.928646, 0.583333, 0, 2.928646, 0.583333, 0, 3.126042, 0.583333, 0, 2.928646, 0.583333, 0, 2.928646, 0.583333, 0, 3.126042, 0.583333, 0, 3.126042, 0.583333, 0, 3.126042, 0.583333, 0, 3.126042, 0.500000, 0, 3.126042, 0.583333, 0, 3.126042, 0.583333, 0, 3.126042, 0.500000, 0, 3.126042, 0.500000, 0, 3.126042, 0.500000, 0, 2.928646, 0.500000, 0, 3.126042, 0.500000, 0, 3.126042, 0.500000, 0, 2.928646, 0.500000, 0, 2.928646, 0.500000, 0, 2.928646, 0.500000, 0, 2.928646, 0.192188, 0, 2.928646, 0.500000, 0, 2.928646, 0.500000, 0, 2.928646, 0.192188, 0, 2.928646, 0.192188, 0, 2.928646, 0.192188, 0, 2.929867, 0.160645, 0, 2.928646, 0.192188, 0, 2.928646, 0.192188, 0, 2.929867, 0.160645, 0, 2.929867, 0.160645, 0, 2.929867, 0.160645, 0, 2.933529, 0.135286, 0, 2.929867, 0.160645, 0, 2.929867, 0.160645, 0, 2.933529, 0.135286, 0, 2.933529, 0.135286, 0, 2.933529, 0.135286, 0, 2.939632, 0.116113, 0, 2.933529, 0.135286, 0, 2.933529, 0.135286, 0, 2.939632, 0.116113, 0, 2.939632, 0.116113, 0, 2.939632, 0.116113, 0, 2.948177, 0.103125, 0, 2.939632, 0.116113, 0, 2.939632, 0.116113, 0, 2.948177, 0.103125, 0, 2.948177, 0.103125, 0, 2.948177, 0.103125, 0, 2.960465, 0.094466, 0, 2.948177, 0.103125, 0, 2.948177, 0.103125, 0, 2.960465, 0.094466, 0, 2.960465, 0.094466, 0, 2.960465, 0.094466, 0, 2.977799, 0.088281, 0, 2.960465, 0.094466, 0, 2.960465, 0.094466, 0, 2.977799, 0.088281, 0, 2.977799, 0.088281, 0, 2.977799, 0.088281, 0, 3.000179, 0.084570, 0, 2.977799, 0.088281, 0, 2.977799, 0.088281, 0, 3.000179, 0.084570, 0, 3.000179, 0.084570, 0, 3.000179, 0.084570, 0, 3.027604, 0.083333, 0, 3.000179, 0.084570, 0, 3.000179, 0.084570, 0, 3.027604, 0.083333, 0, 3.027604, 0.083333, 0, 3.027604, 0.083333, 0, 3.126042, 0.083333, 0, 3.027604, 0.083333, 0, 3.027604, 0.083333, 0, 3.126042, 0.083333, 0, 3.126042, 0.083333, 0, 3.126042, 0.083333, 0, 3.126042, 0, 0, 3.126042, 0.083333, 0, 3.126042, 0.083333, 0, 3.126042, 0, 0, 3.126042, 0, 0, 3.126042, 0, 0, 3.027604, 0, 0, 3.126042, 0, 0, 3.126042, 0, 0, 3.027604, 0, 0, 3.027604, 0, 0, 3.027604, 0, 0, 2.976432, 0.002588, 0, 3.027604, 0, 0, 3.027604, 0, 0, 2.976432, 0.002588, 0, 2.976432, 0.002588, 0, 2.976432, 0.002588, 0, 2.933854, 0.010352, 0, 2.976432, 0.002588, 0, 2.976432, 0.002588, 0, 2.933854, 0.010352, 0, 2.933854, 0.010352, 0, 2.933854, 0.010352, 0, 2.899870, 0.023291, 0, 2.933854, 0.010352, 0, 2.933854, 0.010352, 0, 2.899870, 0.023291, 0, 2.899870, 0.023291, 0, 2.899870, 0.023291, 0, 2.874479, 0.041406, 0, 2.899870, 0.023291, 0, 2.899870, 0.023291, 0, 2.874479, 0.041406, 0, 2.874479, 0.041406, 0, 2.874479, 0.041406, 0, 2.856022, 0.066357, 0, 2.874479, 0.041406, 0, 2.874479, 0.041406, 0, 2.856022, 0.066357, 0, 2.856022, 0.066357, 0, 2.856022, 0.066357, 0, 2.842839, 0.099805, 0, 2.856022, 0.066357, 0, 2.856022, 0.066357, 0, 2.842839, 0.099805, 0, 2.842839, 0.099805, 0, 2.842839, 0.099805, 0, 2.834928, 0.141748, 0, 2.842839, 0.099805, 0, 2.842839, 0.099805, 0, 2.834928, 0.141748, 0, 2.834928, 0.141748, 0, 2.834928, 0.141748, 0, 2.832292, 0.192188, 0, 2.834928, 0.141748, 0, 2.834928, 0.141748, 0, 2.832292, 0.192188, 0, 2.832292, 0.192188, 0, 2.832292, 0.192188, 0, 2.832292, 0.500000, 0, 2.832292, 0.192188, 0, 2.832292, 0.192188, 0, 2.832292, 0.500000, 0, 2.832292, 0.500000, 0, 2.832292, 0.500000, 0, 2.761979, 0.500000, 0, 2.832292, 0.500000, 0, 2.832292, 0.500000, 0, 2.761979, 0.500000, 0, 2.761979, 0.500000, 0, 2.761979, 0.500000, 0, 2.761979, 0.583333, 0, 2.761979, 0.500000, 0, 2.761979, 0.500000, 0, 2.761979, 0.583333, 0, 2.761979, 0.583333, 0, 2.761979, 0.583333, 0, 2.832292, 0.583333, 0, 2.761979, 0.583333, 0, 2.761979, 0.583333, 0, 2.832292, 0.583333, 0, 2.832292, 0.583333, 0, 2.832292, 0.583333, 0, 2.832292, 0.750000, 0, 2.832292, 0.583333, 0, 2.832292, 0.583333, 0, 2.832292, 0.750000, 0, 2.832292, 0.750000, 0, 2.761979, 0.583333, 0, 2.832292, 0.500000, 0, 2.832292, 0.583333, 0, 2.832292, 0.500000, 0, 2.761979, 0.583333, 0, 2.832292, 0.583333, 0, 2.761979, 0.583333, 0, 2.761979, 0.500000, 0, 2.832292, 0.500000, 0, 2.761979, 0.500000, 0, 2.761979, 0.583333, 0, 2.832292, 0.500000, 0, 2.832292, 0.583333, 0, 2.832292, 0.500000, 0, 2.928646, 0.500000, 0, 2.832292, 0.500000, 0, 2.832292, 0.583333, 0, 2.928646, 0.500000, 0, 2.832292, 0.583333, 0, 2.928646, 0.500000, 0, 2.928646, 0.583333, 0, 2.928646, 0.500000, 0, 2.832292, 0.583333, 0, 2.928646, 0.583333, 0, 2.832292, 0.750000, 0, 2.832292, 0.583333, 0, 2.928646, 0.583333, 0, 2.832292, 0.583333, 0, 2.832292, 0.750000, 0, 2.928646, 0.583333, 0, 2.832292, 0.750000, 0, 2.928646, 0.583333, 0, 2.928646, 0.750000, 0, 2.928646, 0.583333, 0, 2.832292, 0.750000, 0, 2.928646, 0.750000, 0, 2.928646, 0.583333, 0, 2.928646, 0.500000, 0, 3.126042, 0.500000, 0, 2.928646, 0.500000, 0, 2.928646, 0.583333, 0, 3.126042, 0.500000, 0, 2.928646, 0.583333, 0, 3.126042, 0.500000, 0, 3.126042, 0.583333, 0, 3.126042, 0.500000, 0, 2.928646, 0.583333, 0, 3.126042, 0.583333, 0, 2.832292, 0.500000, 0, 2.928646, 0.192188, 0, 2.928646, 0.500000, 0, 2.928646, 0.192188, 0, 2.832292, 0.500000, 0, 2.928646, 0.500000, 0, 2.832292, 0.500000, 0, 2.832292, 0.192188, 0, 2.928646, 0.192188, 0, 2.832292, 0.192188, 0, 2.832292, 0.500000, 0, 2.928646, 0.192188, 0, 2.928646, 0.192188, 0, 2.832292, 0.192188, 0, 2.929867, 0.160645, 0, 2.832292, 0.192188, 0, 2.928646, 0.192188, 0, 2.929867, 0.160645, 0, 2.832292, 0.192188, 0, 2.834928, 0.141748, 0, 2.929867, 0.160645, 0, 2.834928, 0.141748, 0, 2.832292, 0.192188, 0, 2.929867, 0.160645, 0, 2.929867, 0.160645, 0, 2.834928, 0.141748, 0, 2.933529, 0.135286, 0, 2.834928, 0.141748, 0, 2.929867, 0.160645, 0, 2.933529, 0.135286, 0, 2.834928, 0.141748, 0, 2.842839, 0.099805, 0, 2.933529, 0.135286, 0, 2.842839, 0.099805, 0, 2.834928, 0.141748, 0, 2.933529, 0.135286, 0, 2.933529, 0.135286, 0, 2.842839, 0.099805, 0, 2.939632, 0.116113, 0, 2.842839, 0.099805, 0, 2.933529, 0.135286, 0, 2.939632, 0.116113, 0, 2.939632, 0.116113, 0, 2.842839, 0.099805, 0, 2.856022, 0.066357, 0, 2.842839, 0.099805, 0, 2.939632, 0.116113, 0, 2.856022, 0.066357, 0, 2.948177, 0.103125, 0, 2.939632, 0.116113, 0, 2.856022, 0.066357, 0, 2.939632, 0.116113, 0, 2.948177, 0.103125, 0, 2.856022, 0.066357, 0, 2.856022, 0.066357, 0, 2.874479, 0.041406, 0, 2.948177, 0.103125, 0, 2.874479, 0.041406, 0, 2.856022, 0.066357, 0, 2.948177, 0.103125, 0, 2.948177, 0.103125, 0, 2.874479, 0.041406, 0, 2.899870, 0.023291, 0, 2.874479, 0.041406, 0, 2.948177, 0.103125, 0, 2.899870, 0.023291, 0, 2.960465, 0.094466, 0, 2.948177, 0.103125, 0, 2.899870, 0.023291, 0, 2.948177, 0.103125, 0, 2.960465, 0.094466, 0, 2.899870, 0.023291, 0, 2.899870, 0.023291, 0, 2.933854, 0.010352, 0, 2.960465, 0.094466, 0, 2.933854, 0.010352, 0, 2.899870, 0.023291, 0, 2.960465, 0.094466, 0, 2.960465, 0.094466, 0, 2.933854, 0.010352, 0, 2.977799, 0.088281, 0, 2.933854, 0.010352, 0, 2.960465, 0.094466, 0, 2.977799, 0.088281, 0, 2.933854, 0.010352, 0, 2.976432, 0.002588, 0, 2.977799, 0.088281, 0, 2.976432, 0.002588, 0, 2.933854, 0.010352, 0, 2.977799, 0.088281, 0, 2.977799, 0.088281, 0, 2.976432, 0.002588, 0, 3.000179, 0.084570, 0, 2.976432, 0.002588, 0, 2.977799, 0.088281, 0, 3.000179, 0.084570, 0, 3.000179, 0.084570, 0, 2.976432, 0.002588, 0, 3.027604, 0, 0, 2.976432, 0.002588, 0, 3.000179, 0.084570, 0, 3.027604, 0, 0, 3.027604, 0.083333, 0, 3.000179, 0.084570, 0, 3.027604, 0, 0, 3.000179, 0.084570, 0, 3.027604, 0.083333, 0, 3.027604, 0, 0, 3.027604, 0.083333, 0, 3.027604, 0, 0, 3.126042, 0, 0, 3.027604, 0, 0, 3.027604, 0.083333, 0, 3.126042, 0, 0, 3.027604, 0.083333, 0, 3.126042, 0, 0, 3.126042, 0.083333, 0, 3.126042, 0, 0, 3.027604, 0.083333, 0, 3.126042, 0.083333, 0, 3.515625, 0.283333, 0, 3.462012, 0.281755, 0, 3.515625, 0.283333, 0, 3.515625, 0.283333, 0, 3.462012, 0.281755, 0, 3.462012, 0.281755, 0, 3.462012, 0.281755, 0, 3.417318, 0.277018, 0, 3.462012, 0.281755, 0, 3.462012, 0.281755, 0, 3.417318, 0.277018, 0, 3.417318, 0.277018, 0, 3.417318, 0.277018, 0, 3.381543, 0.269124, 0, 3.417318, 0.277018, 0, 3.417318, 0.277018, 0, 3.381543, 0.269124, 0, 3.381543, 0.269124, 0, 3.381543, 0.269124, 0, 3.354687, 0.258073, 0, 3.381543, 0.269124, 0, 3.381543, 0.269124, 0, 3.354687, 0.258073, 0, 3.354687, 0.258073, 0, 3.354687, 0.258073, 0, 3.335091, 0.243229, 0, 3.354687, 0.258073, 0, 3.354687, 0.258073, 0, 3.335091, 0.243229, 0, 3.335091, 0.243229, 0, 3.335091, 0.243229, 0, 3.321094, 0.223958, 0, 3.335091, 0.243229, 0, 3.335091, 0.243229, 0, 3.321094, 0.223958, 0, 3.321094, 0.223958, 0, 3.321094, 0.223958, 0, 3.312695, 0.200260, 0, 3.321094, 0.223958, 0, 3.321094, 0.223958, 0, 3.312695, 0.200260, 0, 3.312695, 0.200260, 0, 3.312695, 0.200260, 0, 3.309896, 0.172135, 0, 3.312695, 0.200260, 0, 3.312695, 0.200260, 0, 3.309896, 0.172135, 0, 3.309896, 0.172135, 0, 3.309896, 0.172135, 0, 3.311995, 0.149056, 0, 3.309896, 0.172135, 0, 3.309896, 0.172135, 0, 3.311995, 0.149056, 0, 3.311995, 0.149056, 0, 3.311995, 0.149056, 0, 3.318294, 0.128516, 0, 3.311995, 0.149056, 0, 3.311995, 0.149056, 0, 3.318294, 0.128516, 0, 3.318294, 0.128516, 0, 3.318294, 0.128516, 0, 3.328792, 0.110514, 0, 3.318294, 0.128516, 0, 3.318294, 0.128516, 0, 3.328792, 0.110514, 0, 3.328792, 0.110514, 0, 3.328792, 0.110514, 0, 3.343489, 0.095052, 0, 3.328792, 0.110514, 0, 3.328792, 0.110514, 0, 3.343489, 0.095052, 0, 3.343489, 0.095052, 0, 3.343489, 0.095052, 0, 3.361800, 0.082633, 0, 3.343489, 0.095052, 0, 3.343489, 0.095052, 0, 3.361800, 0.082633, 0, 3.361800, 0.082633, 0, 3.361800, 0.082633, 0, 3.383138, 0.073763, 0, 3.361800, 0.082633, 0, 3.361800, 0.082633, 0, 3.383138, 0.073763, 0, 3.383138, 0.073763, 0, 3.383138, 0.073763, 0, 3.407503, 0.068441, 0, 3.383138, 0.073763, 0, 3.383138, 0.073763, 0, 3.407503, 0.068441, 0, 3.407503, 0.068441, 0, 3.407503, 0.068441, 0, 3.434896, 0.066667, 0, 3.407503, 0.068441, 0, 3.407503, 0.068441, 0, 3.434896, 0.066667, 0, 3.434896, 0.066667, 0, 3.434896, 0.066667, 0, 3.472770, 0.070011, 0, 3.434896, 0.066667, 0, 3.434896, 0.066667, 0, 3.472770, 0.070011, 0, 3.472770, 0.070011, 0, 3.472770, 0.070011, 0, 3.506706, 0.080046, 0, 3.472770, 0.070011, 0, 3.472770, 0.070011, 0, 3.506706, 0.080046, 0, 3.506706, 0.080046, 0, 3.506706, 0.080046, 0, 3.536702, 0.096769, 0, 3.506706, 0.080046, 0, 3.506706, 0.080046, 0, 3.536702, 0.096769, 0, 3.536702, 0.096769, 0, 3.536702, 0.096769, 0, 3.562761, 0.120182, 0, 3.536702, 0.096769, 0, 3.536702, 0.096769, 0, 3.562761, 0.120182, 0, 3.562761, 0.120182, 0, 3.562761, 0.120182, 0, 3.583838, 0.149146, 0, 3.562761, 0.120182, 0, 3.562761, 0.120182, 0, 3.583838, 0.149146, 0, 3.583838, 0.149146, 0, 3.583838, 0.149146, 0, 3.598893, 0.182520, 0, 3.583838, 0.149146, 0, 3.583838, 0.149146, 0, 3.598893, 0.182520, 0, 3.598893, 0.182520, 0, 3.598893, 0.182520, 0, 3.607927, 0.220304, 0, 3.598893, 0.182520, 0, 3.598893, 0.182520, 0, 3.607927, 0.220304, 0, 3.607927, 0.220304, 0, 3.607927, 0.220304, 0, 3.610938, 0.262500, 0, 3.607927, 0.220304, 0, 3.607927, 0.220304, 0, 3.610938, 0.262500, 0, 3.610938, 0.262500, 0, 3.610938, 0.262500, 0, 3.610938, 0.283333, 0, 3.610938, 0.262500, 0, 3.610938, 0.262500, 0, 3.610938, 0.283333, 0, 3.610938, 0.283333, 0, 3.706771, 0.327344, 0, 3.706771, 0, 0, 3.706771, 0.327344, 0, 3.706771, 0.327344, 0, 3.706771, 0, 0, 3.706771, 0, 0, 3.706771, 0, 0, 3.610938, 0, 0, 3.706771, 0, 0, 3.706771, 0, 0, 3.610938, 0, 0, 3.610938, 0, 0, 3.610938, 0, 0, 3.610938, 0.083333, 0, 3.610938, 0, 0, 3.610938, 0, 0, 3.610938, 0.083333, 0, 3.610938, 0.083333, 0, 3.610938, 0.083333, 0, 3.593522, 0.059367, 0, 3.610938, 0.083333, 0, 3.610938, 0.083333, 0, 3.593522, 0.059367, 0, 3.593522, 0.059367, 0, 3.593522, 0.059367, 0, 3.574089, 0.038770, 0, 3.593522, 0.059367, 0, 3.593522, 0.059367, 0, 3.574089, 0.038770, 0, 3.574089, 0.038770, 0, 3.574089, 0.038770, 0, 3.552637, 0.021541, 0, 3.574089, 0.038770, 0, 3.574089, 0.038770, 0, 3.552637, 0.021541, 0, 3.552637, 0.021541, 0, 3.552637, 0.021541, 0, 3.529167, 0.007682, 0, 3.552637, 0.021541, 0, 3.552637, 0.021541, 0, 3.529167, 0.007682, 0, 3.529167, 0.007682, 0, 3.529167, 0.007682, 0, 3.503320, -0.002970, 0, 3.529167, 0.007682, 0, 3.529167, 0.007682, 0, 3.503320, -0.002970, 0, 3.503320, -0.002970, 0, 3.503320, -0.002970, 0, 3.474740, -0.010579, 0, 3.503320, -0.002970, 0, 3.503320, -0.002970, 0, 3.474740, -0.010579, 0, 3.474740, -0.010579, 0, 3.474740, -0.010579, 0, 3.443425, -0.015145, 0, 3.474740, -0.010579, 0, 3.474740, -0.010579, 0, 3.443425, -0.015145, 0, 3.443425, -0.015145, 0, 3.443425, -0.015145, 0, 3.409375, -0.016667, 0, 3.443425, -0.015145, 0, 3.443425, -0.015145, 0, 3.409375, -0.016667, 0, 3.409375, -0.016667, 0, 3.409375, -0.016667, 0, 3.366878, -0.013525, 0, 3.409375, -0.016667, 0, 3.409375, -0.016667, 0, 3.366878, -0.013525, 0, 3.366878, -0.013525, 0, 3.366878, -0.013525, 0, 3.328972, -0.004102, 0, 3.366878, -0.013525, 0, 3.366878, -0.013525, 0, 3.328972, -0.004102, 0, 3.328972, -0.004102, 0, 3.328972, -0.004102, 0, 3.295654, 0.011605, 0, 3.328972, -0.004102, 0, 3.328972, -0.004102, 0, 3.295654, 0.011605, 0, 3.295654, 0.011605, 0, 3.295654, 0.011605, 0, 3.266927, 0.033594, 0, 3.295654, 0.011605, 0, 3.295654, 0.011605, 0, 3.266927, 0.033594, 0, 3.266927, 0.033594, 0, 3.266927, 0.033594, 0, 3.243799, 0.060856, 0, 3.266927, 0.033594, 0, 3.266927, 0.033594, 0, 3.243799, 0.060856, 0, 3.243799, 0.060856, 0, 3.243799, 0.060856, 0, 3.227278, 0.092383, 0, 3.243799, 0.060856, 0, 3.243799, 0.060856, 0, 3.227278, 0.092383, 0, 3.227278, 0.092383, 0, 3.227278, 0.092383, 0, 3.217366, 0.128174, 0, 3.227278, 0.092383, 0, 3.227278, 0.092383, 0, 3.217366, 0.128174, 0, 3.217366, 0.128174, 0, 3.217366, 0.128174, 0, 3.214062, 0.168229, 0, 3.217366, 0.128174, 0, 3.217366, 0.128174, 0, 3.214062, 0.168229, 0, 3.214062, 0.168229, 0, 3.214062, 0.168229, 0, 3.218180, 0.214421, 0, 3.214062, 0.168229, 0, 3.214062, 0.168229, 0, 3.218180, 0.214421, 0, 3.218180, 0.214421, 0, 3.218180, 0.214421, 0, 3.230534, 0.254557, 0, 3.218180, 0.214421, 0, 3.218180, 0.214421, 0, 3.230534, 0.254557, 0, 3.230534, 0.254557, 0, 3.230534, 0.254557, 0, 3.251123, 0.288639, 0, 3.230534, 0.254557, 0, 3.230534, 0.254557, 0, 3.251123, 0.288639, 0, 3.251123, 0.288639, 0, 3.251123, 0.288639, 0, 3.279948, 0.316667, 0, 3.251123, 0.288639, 0, 3.251123, 0.288639, 0, 3.279948, 0.316667, 0, 3.279948, 0.316667, 0, 3.279948, 0.316667, 0, 3.316943, 0.338542, 0, 3.279948, 0.316667, 0, 3.279948, 0.316667, 0, 3.316943, 0.338542, 0, 3.316943, 0.338542, 0, 3.316943, 0.338542, 0, 3.362044, 0.354167, 0, 3.316943, 0.338542, 0, 3.316943, 0.338542, 0, 3.362044, 0.354167, 0, 3.362044, 0.354167, 0, 3.362044, 0.354167, 0, 3.415251, 0.363542, 0, 3.362044, 0.354167, 0, 3.362044, 0.354167, 0, 3.415251, 0.363542, 0, 3.415251, 0.363542, 0, 3.415251, 0.363542, 0, 3.476563, 0.366667, 0, 3.415251, 0.363542, 0, 3.415251, 0.363542, 0, 3.476563, 0.366667, 0, 3.476563, 0.366667, 0, 3.476563, 0.366667, 0, 3.610938, 0.366667, 0, 3.476563, 0.366667, 0, 3.476563, 0.366667, 0, 3.610938, 0.366667, 0, 3.610938, 0.366667, 0, 3.610938, 0.366667, 0, 3.610938, 0.376042, 0, 3.610938, 0.366667, 0, 3.610938, 0.366667, 0, 3.610938, 0.376042, 0, 3.610938, 0.376042, 0, 3.610938, 0.376042, 0, 3.608220, 0.407731, 0, 3.610938, 0.376042, 0, 3.610938, 0.376042, 0, 3.608220, 0.407731, 0, 3.608220, 0.407731, 0, 3.608220, 0.407731, 0, 3.600065, 0.435612, 0, 3.608220, 0.407731, 0, 3.608220, 0.407731, 0, 3.600065, 0.435612, 0, 3.600065, 0.435612, 0, 3.600065, 0.435612, 0, 3.586475, 0.459684, 0, 3.600065, 0.435612, 0, 3.600065, 0.435612, 0, 3.586475, 0.459684, 0, 3.586475, 0.459684, 0, 3.586475, 0.459684, 0, 3.567448, 0.479948, 0, 3.586475, 0.459684, 0, 3.586475, 0.459684, 0, 3.567448, 0.479948, 0, 3.567448, 0.479948, 0, 3.567448, 0.479948, 0, 3.543506, 0.496012, 0, 3.567448, 0.479948, 0, 3.567448, 0.479948, 0, 3.543506, 0.496012, 0, 3.543506, 0.496012, 0, 3.543506, 0.496012, 0, 3.515169, 0.507487, 0, 3.543506, 0.496012, 0, 3.543506, 0.496012, 0, 3.515169, 0.507487, 0, 3.515169, 0.507487, 0, 3.515169, 0.507487, 0, 3.482438, 0.514372, 0, 3.515169, 0.507487, 0, 3.515169, 0.507487, 0, 3.482438, 0.514372, 0, 3.482438, 0.514372, 0, 3.482438, 0.514372, 0, 3.445313, 0.516667, 0, 3.482438, 0.514372, 0, 3.482438, 0.514372, 0, 3.445313, 0.516667, 0, 3.445313, 0.516667, 0, 3.445313, 0.516667, 0, 3.420475, 0.515885, 0, 3.445313, 0.516667, 0, 3.445313, 0.516667, 0, 3.420475, 0.515885, 0, 3.420475, 0.515885, 0, 3.420475, 0.515885, 0, 3.395964, 0.513542, 0, 3.420475, 0.515885, 0, 3.420475, 0.515885, 0, 3.395964, 0.513542, 0, 3.395964, 0.513542, 0, 3.395964, 0.513542, 0, 3.371777, 0.509635, 0, 3.395964, 0.513542, 0, 3.395964, 0.513542, 0, 3.371777, 0.509635, 0, 3.371777, 0.509635, 0, 3.371777, 0.509635, 0, 3.347917, 0.504167, 0, 3.371777, 0.509635, 0, 3.371777, 0.509635, 0, 3.347917, 0.504167, 0, 3.347917, 0.504167, 0, 3.347917, 0.504167, 0, 3.324446, 0.497135, 0, 3.347917, 0.504167, 0, 3.347917, 0.504167, 0, 3.324446, 0.497135, 0, 3.324446, 0.497135, 0, 3.324446, 0.497135, 0, 3.301432, 0.488542, 0, 3.324446, 0.497135, 0, 3.324446, 0.497135, 0, 3.301432, 0.488542, 0, 3.301432, 0.488542, 0, 3.301432, 0.488542, 0, 3.278874, 0.478385, 0, 3.301432, 0.488542, 0, 3.301432, 0.488542, 0, 3.278874, 0.478385, 0, 3.278874, 0.478385, 0, 3.278874, 0.478385, 0, 3.256771, 0.466667, 0, 3.278874, 0.478385, 0, 3.278874, 0.478385, 0, 3.256771, 0.466667, 0, 3.256771, 0.466667, 0, 3.256771, 0.466667, 0, 3.256771, 0.550000, 0, 3.256771, 0.466667, 0, 3.256771, 0.466667, 0, 3.256771, 0.550000, 0, 3.256771, 0.550000, 0, 3.256771, 0.550000, 0, 3.282878, 0.561719, 0, 3.256771, 0.550000, 0, 3.256771, 0.550000, 0, 3.282878, 0.561719, 0, 3.282878, 0.561719, 0, 3.282878, 0.561719, 0, 3.308594, 0.571875, 0, 3.282878, 0.561719, 0, 3.282878, 0.561719, 0, 3.308594, 0.571875, 0, 3.308594, 0.571875, 0, 3.308594, 0.571875, 0, 3.333919, 0.580469, 0, 3.308594, 0.571875, 0, 3.308594, 0.571875, 0, 3.333919, 0.580469, 0, 3.333919, 0.580469, 0, 3.333919, 0.580469, 0, 3.358854, 0.587500, 0, 3.333919, 0.580469, 0, 3.333919, 0.580469, 0, 3.358854, 0.587500, 0, 3.358854, 0.587500, 0, 3.358854, 0.587500, 0, 3.383431, 0.592969, 0, 3.358854, 0.587500, 0, 3.358854, 0.587500, 0, 3.383431, 0.592969, 0, 3.383431, 0.592969, 0, 3.383431, 0.592969, 0, 3.407682, 0.596875, 0, 3.383431, 0.592969, 0, 3.383431, 0.592969, 0, 3.407682, 0.596875, 0, 3.407682, 0.596875, 0, 3.407682, 0.596875, 0, 3.431608, 0.599219, 0, 3.407682, 0.596875, 0, 3.407682, 0.596875, 0, 3.431608, 0.599219, 0, 3.431608, 0.599219, 0, 3.431608, 0.599219, 0, 3.455208, 0.600000, 0, 3.431608, 0.599219, 0, 3.431608, 0.599219, 0, 3.455208, 0.600000, 0, 3.455208, 0.600000, 0, 3.455208, 0.600000, 0, 3.514486, 0.595776, 0, 3.455208, 0.600000, 0, 3.455208, 0.600000, 0, 3.514486, 0.595776, 0, 3.514486, 0.595776, 0, 3.514486, 0.595776, 0, 3.565755, 0.583106, 0, 3.514486, 0.595776, 0, 3.514486, 0.595776, 0, 3.565755, 0.583106, 0, 3.565755, 0.583106, 0, 3.565755, 0.583106, 0, 3.609017, 0.561987, 0, 3.565755, 0.583106, 0, 3.565755, 0.583106, 0, 3.609017, 0.561987, 0, 3.609017, 0.561987, 0, 3.609017, 0.561987, 0, 3.644271, 0.532422, 0, 3.609017, 0.561987, 0, 3.609017, 0.561987, 0, 3.644271, 0.532422, 0, 3.644271, 0.532422, 0, 3.644271, 0.532422, 0, 3.671614, 0.494263, 0, 3.644271, 0.532422, 0, 3.644271, 0.532422, 0, 3.671614, 0.494263, 0, 3.671614, 0.494263, 0, 3.671614, 0.494263, 0, 3.691146, 0.447363, 0, 3.671614, 0.494263, 0, 3.671614, 0.494263, 0, 3.691146, 0.447363, 0, 3.691146, 0.447363, 0, 3.691146, 0.447363, 0, 3.702864, 0.391724, 0, 3.691146, 0.447363, 0, 3.691146, 0.447363, 0, 3.702864, 0.391724, 0, 3.702864, 0.391724, 0, 3.218180, 0.214421, 0, 3.214062, 0.168229, 0, 3.309896, 0.172135, 0, 3.214062, 0.168229, 0, 3.218180, 0.214421, 0, 3.309896, 0.172135, 0, 3.218180, 0.214421, 0, 3.309896, 0.172135, 0, 3.312695, 0.200260, 0, 3.309896, 0.172135, 0, 3.218180, 0.214421, 0, 3.312695, 0.200260, 0, 3.230534, 0.254557, 0, 3.218180, 0.214421, 0, 3.312695, 0.200260, 0, 3.218180, 0.214421, 0, 3.230534, 0.254557, 0, 3.312695, 0.200260, 0, 3.230534, 0.254557, 0, 3.312695, 0.200260, 0, 3.321094, 0.223958, 0, 3.312695, 0.200260, 0, 3.230534, 0.254557, 0, 3.321094, 0.223958, 0, 3.251123, 0.288639, 0, 3.230534, 0.254557, 0, 3.321094, 0.223958, 0, 3.230534, 0.254557, 0, 3.251123, 0.288639, 0, 3.321094, 0.223958, 0, 3.251123, 0.288639, 0, 3.321094, 0.223958, 0, 3.335091, 0.243229, 0, 3.321094, 0.223958, 0, 3.251123, 0.288639, 0, 3.335091, 0.243229, 0, 3.279948, 0.316667, 0, 3.251123, 0.288639, 0, 3.335091, 0.243229, 0, 3.251123, 0.288639, 0, 3.279948, 0.316667, 0, 3.335091, 0.243229, 0, 3.279948, 0.316667, 0, 3.335091, 0.243229, 0, 3.354687, 0.258073, 0, 3.335091, 0.243229, 0, 3.279948, 0.316667, 0, 3.354687, 0.258073, 0, 3.316943, 0.338542, 0, 3.279948, 0.316667, 0, 3.354687, 0.258073, 0, 3.279948, 0.316667, 0, 3.316943, 0.338542, 0, 3.354687, 0.258073, 0, 3.381543, 0.269124, 0, 3.316943, 0.338542, 0, 3.354687, 0.258073, 0, 3.316943, 0.338542, 0, 3.381543, 0.269124, 0, 3.354687, 0.258073, 0, 3.362044, 0.354167, 0, 3.316943, 0.338542, 0, 3.381543, 0.269124, 0, 3.316943, 0.338542, 0, 3.362044, 0.354167, 0, 3.381543, 0.269124, 0, 3.417318, 0.277018, 0, 3.362044, 0.354167, 0, 3.381543, 0.269124, 0, 3.362044, 0.354167, 0, 3.417318, 0.277018, 0, 3.381543, 0.269124, 0, 3.415251, 0.363542, 0, 3.362044, 0.354167, 0, 3.417318, 0.277018, 0, 3.362044, 0.354167, 0, 3.415251, 0.363542, 0, 3.417318, 0.277018, 0, 3.415251, 0.363542, 0, 3.417318, 0.277018, 0, 3.462012, 0.281755, 0, 3.417318, 0.277018, 0, 3.415251, 0.363542, 0, 3.462012, 0.281755, 0, 3.415251, 0.363542, 0, 3.462012, 0.281755, 0, 3.476563, 0.366667, 0, 3.462012, 0.281755, 0, 3.415251, 0.363542, 0, 3.476563, 0.366667, 0, 3.476563, 0.366667, 0, 3.462012, 0.281755, 0, 3.515625, 0.283333, 0, 3.462012, 0.281755, 0, 3.476563, 0.366667, 0, 3.515625, 0.283333, 0, 3.476563, 0.366667, 0, 3.515625, 0.283333, 0, 3.610938, 0.366667, 0, 3.515625, 0.283333, 0, 3.476563, 0.366667, 0, 3.610938, 0.366667, 0, 3.515625, 0.283333, 0, 3.610938, 0.283333, 0, 3.610938, 0.366667, 0, 3.610938, 0.283333, 0, 3.515625, 0.283333, 0, 3.610938, 0.366667, 0, 3.610938, 0.366667, 0, 3.610938, 0.283333, 0, 3.706771, 0.327344, 0, 3.610938, 0.283333, 0, 3.610938, 0.366667, 0, 3.706771, 0.327344, 0, 3.706771, 0.327344, 0, 3.702864, 0.391724, 0, 3.610938, 0.366667, 0, 3.702864, 0.391724, 0, 3.706771, 0.327344, 0, 3.610938, 0.366667, 0, 3.702864, 0.391724, 0, 3.610938, 0.376042, 0, 3.610938, 0.366667, 0, 3.610938, 0.376042, 0, 3.702864, 0.391724, 0, 3.610938, 0.366667, 0, 3.608220, 0.407731, 0, 3.610938, 0.376042, 0, 3.702864, 0.391724, 0, 3.610938, 0.376042, 0, 3.608220, 0.407731, 0, 3.702864, 0.391724, 0, 3.691146, 0.447363, 0, 3.608220, 0.407731, 0, 3.702864, 0.391724, 0, 3.608220, 0.407731, 0, 3.691146, 0.447363, 0, 3.702864, 0.391724, 0, 3.600065, 0.435612, 0, 3.608220, 0.407731, 0, 3.691146, 0.447363, 0, 3.608220, 0.407731, 0, 3.600065, 0.435612, 0, 3.691146, 0.447363, 0, 3.671614, 0.494263, 0, 3.600065, 0.435612, 0, 3.691146, 0.447363, 0, 3.600065, 0.435612, 0, 3.671614, 0.494263, 0, 3.691146, 0.447363, 0, 3.586475, 0.459684, 0, 3.600065, 0.435612, 0, 3.671614, 0.494263, 0, 3.600065, 0.435612, 0, 3.586475, 0.459684, 0, 3.671614, 0.494263, 0, 3.644271, 0.532422, 0, 3.586475, 0.459684, 0, 3.671614, 0.494263, 0, 3.586475, 0.459684, 0, 3.644271, 0.532422, 0, 3.671614, 0.494263, 0, 3.567448, 0.479948, 0, 3.586475, 0.459684, 0, 3.644271, 0.532422, 0, 3.586475, 0.459684, 0, 3.567448, 0.479948, 0, 3.644271, 0.532422, 0, 3.609017, 0.561987, 0, 3.567448, 0.479948, 0, 3.644271, 0.532422, 0, 3.567448, 0.479948, 0, 3.609017, 0.561987, 0, 3.644271, 0.532422, 0, 3.543506, 0.496012, 0, 3.567448, 0.479948, 0, 3.609017, 0.561987, 0, 3.567448, 0.479948, 0, 3.543506, 0.496012, 0, 3.609017, 0.561987, 0, 3.565755, 0.583106, 0, 3.543506, 0.496012, 0, 3.609017, 0.561987, 0, 3.543506, 0.496012, 0, 3.565755, 0.583106, 0, 3.609017, 0.561987, 0, 3.515169, 0.507487, 0, 3.543506, 0.496012, 0, 3.565755, 0.583106, 0, 3.543506, 0.496012, 0, 3.515169, 0.507487, 0, 3.565755, 0.583106, 0, 3.514486, 0.595776, 0, 3.515169, 0.507487, 0, 3.565755, 0.583106, 0, 3.515169, 0.507487, 0, 3.514486, 0.595776, 0, 3.565755, 0.583106, 0, 3.514486, 0.595776, 0, 3.482438, 0.514372, 0, 3.515169, 0.507487, 0, 3.482438, 0.514372, 0, 3.514486, 0.595776, 0, 3.515169, 0.507487, 0, 3.514486, 0.595776, 0, 3.455208, 0.600000, 0, 3.482438, 0.514372, 0, 3.455208, 0.600000, 0, 3.514486, 0.595776, 0, 3.482438, 0.514372, 0, 3.445313, 0.516667, 0, 3.482438, 0.514372, 0, 3.455208, 0.600000, 0, 3.482438, 0.514372, 0, 3.445313, 0.516667, 0, 3.455208, 0.600000, 0, 3.455208, 0.600000, 0, 3.431608, 0.599219, 0, 3.445313, 0.516667, 0, 3.431608, 0.599219, 0, 3.455208, 0.600000, 0, 3.445313, 0.516667, 0, 3.431608, 0.599219, 0, 3.420475, 0.515885, 0, 3.445313, 0.516667, 0, 3.420475, 0.515885, 0, 3.431608, 0.599219, 0, 3.445313, 0.516667, 0, 3.407682, 0.596875, 0, 3.420475, 0.515885, 0, 3.431608, 0.599219, 0, 3.420475, 0.515885, 0, 3.407682, 0.596875, 0, 3.431608, 0.599219, 0, 3.407682, 0.596875, 0, 3.395964, 0.513542, 0, 3.420475, 0.515885, 0, 3.395964, 0.513542, 0, 3.407682, 0.596875, 0, 3.420475, 0.515885, 0, 3.383431, 0.592969, 0, 3.395964, 0.513542, 0, 3.407682, 0.596875, 0, 3.395964, 0.513542, 0, 3.383431, 0.592969, 0, 3.407682, 0.596875, 0, 3.383431, 0.592969, 0, 3.358854, 0.587500, 0, 3.395964, 0.513542, 0, 3.358854, 0.587500, 0, 3.383431, 0.592969, 0, 3.395964, 0.513542, 0, 3.395964, 0.513542, 0, 3.358854, 0.587500, 0, 3.371777, 0.509635, 0, 3.358854, 0.587500, 0, 3.395964, 0.513542, 0, 3.371777, 0.509635, 0, 3.358854, 0.587500, 0, 3.333919, 0.580469, 0, 3.371777, 0.509635, 0, 3.333919, 0.580469, 0, 3.358854, 0.587500, 0, 3.371777, 0.509635, 0, 3.333919, 0.580469, 0, 3.347917, 0.504167, 0, 3.371777, 0.509635, 0, 3.347917, 0.504167, 0, 3.333919, 0.580469, 0, 3.371777, 0.509635, 0, 3.333919, 0.580469, 0, 3.308594, 0.571875, 0, 3.347917, 0.504167, 0, 3.308594, 0.571875, 0, 3.333919, 0.580469, 0, 3.347917, 0.504167, 0, 3.308594, 0.571875, 0, 3.324446, 0.497135, 0, 3.347917, 0.504167, 0, 3.324446, 0.497135, 0, 3.308594, 0.571875, 0, 3.347917, 0.504167, 0, 3.308594, 0.571875, 0, 3.282878, 0.561719, 0, 3.324446, 0.497135, 0, 3.282878, 0.561719, 0, 3.308594, 0.571875, 0, 3.324446, 0.497135, 0, 3.282878, 0.561719, 0, 3.301432, 0.488542, 0, 3.324446, 0.497135, 0, 3.301432, 0.488542, 0, 3.282878, 0.561719, 0, 3.324446, 0.497135, 0, 3.282878, 0.561719, 0, 3.256771, 0.550000, 0, 3.301432, 0.488542, 0, 3.256771, 0.550000, 0, 3.282878, 0.561719, 0, 3.301432, 0.488542, 0, 3.256771, 0.550000, 0, 3.278874, 0.478385, 0, 3.301432, 0.488542, 0, 3.278874, 0.478385, 0, 3.256771, 0.550000, 0, 3.301432, 0.488542, 0, 3.256771, 0.550000, 0, 3.256771, 0.466667, 0, 3.278874, 0.478385, 0, 3.256771, 0.466667, 0, 3.256771, 0.550000, 0, 3.278874, 0.478385, 0, 3.706771, 0.327344, 0, 3.610938, 0.283333, 0, 3.610938, 0.262500, 0, 3.610938, 0.283333, 0, 3.706771, 0.327344, 0, 3.610938, 0.262500, 0, 3.706771, 0.327344, 0, 3.610938, 0.262500, 0, 3.607927, 0.220304, 0, 3.610938, 0.262500, 0, 3.706771, 0.327344, 0, 3.607927, 0.220304, 0, 3.706771, 0, 0, 3.706771, 0.327344, 0, 3.607927, 0.220304, 0, 3.706771, 0.327344, 0, 3.706771, 0, 0, 3.607927, 0.220304, 0, 3.706771, 0, 0, 3.607927, 0.220304, 0, 3.598893, 0.182520, 0, 3.607927, 0.220304, 0, 3.706771, 0, 0, 3.598893, 0.182520, 0, 3.610938, 0.083333, 0, 3.706771, 0, 0, 3.598893, 0.182520, 0, 3.706771, 0, 0, 3.610938, 0.083333, 0, 3.598893, 0.182520, 0, 3.610938, 0.083333, 0, 3.610938, 0, 0, 3.706771, 0, 0, 3.610938, 0, 0, 3.610938, 0.083333, 0, 3.706771, 0, 0, 3.598893, 0.182520, 0, 3.583838, 0.149146, 0, 3.610938, 0.083333, 0, 3.583838, 0.149146, 0, 3.598893, 0.182520, 0, 3.610938, 0.083333, 0, 3.583838, 0.149146, 0, 3.562761, 0.120182, 0, 3.610938, 0.083333, 0, 3.562761, 0.120182, 0, 3.583838, 0.149146, 0, 3.610938, 0.083333, 0, 3.562761, 0.120182, 0, 3.593522, 0.059367, 0, 3.610938, 0.083333, 0, 3.593522, 0.059367, 0, 3.562761, 0.120182, 0, 3.610938, 0.083333, 0, 3.562761, 0.120182, 0, 3.536702, 0.096769, 0, 3.593522, 0.059367, 0, 3.536702, 0.096769, 0, 3.562761, 0.120182, 0, 3.593522, 0.059367, 0, 3.536702, 0.096769, 0, 3.574089, 0.038770, 0, 3.593522, 0.059367, 0, 3.574089, 0.038770, 0, 3.536702, 0.096769, 0, 3.593522, 0.059367, 0, 3.574089, 0.038770, 0, 3.536702, 0.096769, 0, 3.552637, 0.021541, 0, 3.536702, 0.096769, 0, 3.574089, 0.038770, 0, 3.552637, 0.021541, 0, 3.536702, 0.096769, 0, 3.506706, 0.080046, 0, 3.552637, 0.021541, 0, 3.506706, 0.080046, 0, 3.536702, 0.096769, 0, 3.552637, 0.021541, 0, 3.506706, 0.080046, 0, 3.529167, 0.007682, 0, 3.552637, 0.021541, 0, 3.529167, 0.007682, 0, 3.506706, 0.080046, 0, 3.552637, 0.021541, 0, 3.529167, 0.007682, 0, 3.506706, 0.080046, 0, 3.503320, -0.002970, 0, 3.506706, 0.080046, 0, 3.529167, 0.007682, 0, 3.503320, -0.002970, 0, 3.506706, 0.080046, 0, 3.472770, 0.070011, 0, 3.503320, -0.002970, 0, 3.472770, 0.070011, 0, 3.506706, 0.080046, 0, 3.503320, -0.002970, 0, 3.472770, 0.070011, 0, 3.474740, -0.010579, 0, 3.503320, -0.002970, 0, 3.474740, -0.010579, 0, 3.472770, 0.070011, 0, 3.503320, -0.002970, 0, 3.472770, 0.070011, 0, 3.434896, 0.066667, 0, 3.474740, -0.010579, 0, 3.434896, 0.066667, 0, 3.472770, 0.070011, 0, 3.474740, -0.010579, 0, 3.434896, 0.066667, 0, 3.443425, -0.015145, 0, 3.474740, -0.010579, 0, 3.443425, -0.015145, 0, 3.434896, 0.066667, 0, 3.474740, -0.010579, 0, 3.434896, 0.066667, 0, 3.409375, -0.016667, 0, 3.443425, -0.015145, 0, 3.409375, -0.016667, 0, 3.434896, 0.066667, 0, 3.443425, -0.015145, 0, 3.407503, 0.068441, 0, 3.409375, -0.016667, 0, 3.434896, 0.066667, 0, 3.409375, -0.016667, 0, 3.407503, 0.068441, 0, 3.434896, 0.066667, 0, 3.407503, 0.068441, 0, 3.366878, -0.013525, 0, 3.409375, -0.016667, 0, 3.366878, -0.013525, 0, 3.407503, 0.068441, 0, 3.409375, -0.016667, 0, 3.383138, 0.073763, 0, 3.366878, -0.013525, 0, 3.407503, 0.068441, 0, 3.366878, -0.013525, 0, 3.383138, 0.073763, 0, 3.407503, 0.068441, 0, 3.328972, -0.004102, 0, 3.366878, -0.013525, 0, 3.383138, 0.073763, 0, 3.366878, -0.013525, 0, 3.328972, -0.004102, 0, 3.383138, 0.073763, 0, 3.361800, 0.082633, 0, 3.328972, -0.004102, 0, 3.383138, 0.073763, 0, 3.328972, -0.004102, 0, 3.361800, 0.082633, 0, 3.383138, 0.073763, 0, 3.295654, 0.011605, 0, 3.328972, -0.004102, 0, 3.361800, 0.082633, 0, 3.328972, -0.004102, 0, 3.295654, 0.011605, 0, 3.361800, 0.082633, 0, 3.361800, 0.082633, 0, 3.343489, 0.095052, 0, 3.295654, 0.011605, 0, 3.343489, 0.095052, 0, 3.361800, 0.082633, 0, 3.295654, 0.011605, 0, 3.343489, 0.095052, 0, 3.266927, 0.033594, 0, 3.295654, 0.011605, 0, 3.266927, 0.033594, 0, 3.343489, 0.095052, 0, 3.295654, 0.011605, 0, 3.343489, 0.095052, 0, 3.328792, 0.110514, 0, 3.266927, 0.033594, 0, 3.328792, 0.110514, 0, 3.343489, 0.095052, 0, 3.266927, 0.033594, 0, 3.328792, 0.110514, 0, 3.243799, 0.060856, 0, 3.266927, 0.033594, 0, 3.243799, 0.060856, 0, 3.328792, 0.110514, 0, 3.266927, 0.033594, 0, 3.328792, 0.110514, 0, 3.318294, 0.128516, 0, 3.243799, 0.060856, 0, 3.318294, 0.128516, 0, 3.328792, 0.110514, 0, 3.243799, 0.060856, 0, 3.318294, 0.128516, 0, 3.227278, 0.092383, 0, 3.243799, 0.060856, 0, 3.227278, 0.092383, 0, 3.318294, 0.128516, 0, 3.243799, 0.060856, 0, 3.217366, 0.128174, 0, 3.227278, 0.092383, 0, 3.318294, 0.128516, 0, 3.227278, 0.092383, 0, 3.217366, 0.128174, 0, 3.318294, 0.128516, 0, 3.311995, 0.149056, 0, 3.217366, 0.128174, 0, 3.318294, 0.128516, 0, 3.217366, 0.128174, 0, 3.311995, 0.149056, 0, 3.318294, 0.128516, 0, 3.214062, 0.168229, 0, 3.217366, 0.128174, 0, 3.311995, 0.149056, 0, 3.217366, 0.128174, 0, 3.214062, 0.168229, 0, 3.311995, 0.149056, 0, 3.309896, 0.172135, 0, 3.214062, 0.168229, 0, 3.311995, 0.149056, 0, 3.214062, 0.168229, 0, 3.309896, 0.172135, 0, 3.311995, 0.149056, 0, 4.385417, 0.357292, 0, 4.385417, 0, 0, 4.385417, 0.357292, 0, 4.385417, 0.357292, 0, 4.385417, 0, 0, 4.385417, 0, 0, 4.385417, 0, 0, 4.289583, 0, 0, 4.385417, 0, 0, 4.385417, 0, 0, 4.289583, 0, 0, 4.289583, 0, 0, 4.289583, 0, 0, 4.289583, 0.354167, 0, 4.289583, 0, 0, 4.289583, 0, 0, 4.289583, 0.354167, 0, 4.289583, 0.354167, 0, 4.289583, 0.354167, 0, 4.287565, 0.392358, 0, 4.289583, 0.354167, 0, 4.289583, 0.354167, 0, 4.287565, 0.392358, 0, 4.287565, 0.392358, 0, 4.287565, 0.392358, 0, 4.281511, 0.425423, 0, 4.287565, 0.392358, 0, 4.287565, 0.392358, 0, 4.281511, 0.425423, 0, 4.281511, 0.425423, 0, 4.281511, 0.425423, 0, 4.271419, 0.453361, 0, 4.281511, 0.425423, 0, 4.281511, 0.425423, 0, 4.271419, 0.453361, 0, 4.271419, 0.453361, 0, 4.271419, 0.453361, 0, 4.257292, 0.476172, 0, 4.271419, 0.453361, 0, 4.271419, 0.453361, 0, 4.257292, 0.476172, 0, 4.257292, 0.476172, 0, 4.257292, 0.476172, 0, 4.239128, 0.493888, 0, 4.257292, 0.476172, 0, 4.257292, 0.476172, 0, 4.239128, 0.493888, 0, 4.239128, 0.493888, 0, 4.239128, 0.493888, 0, 4.216927, 0.506543, 0, 4.239128, 0.493888, 0, 4.239128, 0.493888, 0, 4.216927, 0.506543, 0, 4.216927, 0.506543, 0, 4.216927, 0.506543, 0, 4.190691, 0.514136, 0, 4.216927, 0.506543, 0, 4.216927, 0.506543, 0, 4.190691, 0.514136, 0, 4.190691, 0.514136, 0, 4.190691, 0.514136, 0, 4.160417, 0.516667, 0, 4.190691, 0.514136, 0, 4.190691, 0.514136, 0, 4.160417, 0.516667, 0, 4.160417, 0.516667, 0, 4.160417, 0.516667, 0, 4.123665, 0.513615, 0, 4.160417, 0.516667, 0, 4.160417, 0.516667, 0, 4.123665, 0.513615, 0, 4.123665, 0.513615, 0, 4.123665, 0.513615, 0, 4.091016, 0.504460, 0, 4.123665, 0.513615, 0, 4.123665, 0.513615, 0, 4.091016, 0.504460, 0, 4.091016, 0.504460, 0, 4.091016, 0.504460, 0, 4.062468, 0.489201, 0, 4.091016, 0.504460, 0, 4.091016, 0.504460, 0, 4.062468, 0.489201, 0, 4.062468, 0.489201, 0, 4.062468, 0.489201, 0, 4.038021, 0.467839, 0, 4.062468, 0.489201, 0, 4.062468, 0.489201, 0, 4.038021, 0.467839, 0, 4.038021, 0.467839, 0, 4.038021, 0.467839, 0, 4.018425, 0.441203, 0, 4.038021, 0.467839, 0, 4.038021, 0.467839, 0, 4.018425, 0.441203, 0, 4.018425, 0.441203, 0, 4.018425, 0.441203, 0, 4.004427, 0.410124, 0, 4.018425, 0.441203, 0, 4.018425, 0.441203, 0, 4.004427, 0.410124, 0, 4.004427, 0.410124, 0, 4.004427, 0.410124, 0, 3.996029, 0.374601, 0, 4.004427, 0.410124, 0, 4.004427, 0.410124, 0, 3.996029, 0.374601, 0, 3.996029, 0.374601, 0, 3.996029, 0.374601, 0, 3.993229, 0.334635, 0, 3.996029, 0.374601, 0, 3.996029, 0.374601, 0, 3.993229, 0.334635, 0, 3.993229, 0.334635, 0, 3.993229, 0.334635, 0, 3.993229, 0, 0, 3.993229, 0.334635, 0, 3.993229, 0.334635, 0, 3.993229, 0, 0, 3.993229, 0, 0, 3.993229, 0, 0, 3.896875, 0, 0, 3.993229, 0, 0, 3.993229, 0, 0, 3.896875, 0, 0, 3.896875, 0, 0, 3.896875, 0, 0, 3.896875, 0.583333, 0, 3.896875, 0, 0, 3.896875, 0, 0, 3.896875, 0.583333, 0, 3.896875, 0.583333, 0, 3.896875, 0.583333, 0, 3.993229, 0.583333, 0, 3.896875, 0.583333, 0, 3.896875, 0.583333, 0, 3.993229, 0.583333, 0, 3.993229, 0.583333, 0, 3.993229, 0.583333, 0, 3.993229, 0.500000, 0, 3.993229, 0.583333, 0, 3.993229, 0.583333, 0, 3.993229, 0.500000, 0, 3.993229, 0.500000, 0, 3.993229, 0.500000, 0, 4.011182, 0.523543, 0, 3.993229, 0.500000, 0, 3.993229, 0.500000, 0, 4.011182, 0.523543, 0, 4.011182, 0.523543, 0, 4.011182, 0.523543, 0, 4.030664, 0.543913, 0, 4.011182, 0.523543, 0, 4.011182, 0.523543, 0, 4.030664, 0.543913, 0, 4.030664, 0.543913, 0, 4.030664, 0.543913, 0, 4.051677, 0.561108, 0, 4.030664, 0.543913, 0, 4.030664, 0.543913, 0, 4.051677, 0.561108, 0, 4.051677, 0.561108, 0, 4.051677, 0.561108, 0, 4.074219, 0.575130, 0, 4.051677, 0.561108, 0, 4.051677, 0.561108, 0, 4.074219, 0.575130, 0, 4.074219, 0.575130, 0, 4.074219, 0.575130, 0, 4.098421, 0.586011, 0, 4.074219, 0.575130, 0, 4.074219, 0.575130, 0, 4.098421, 0.586011, 0, 4.098421, 0.586011, 0, 4.098421, 0.586011, 0, 4.124414, 0.593783, 0, 4.098421, 0.586011, 0, 4.098421, 0.586011, 0, 4.124414, 0.593783, 0, 4.124414, 0.593783, 0, 4.124414, 0.593783, 0, 4.152197, 0.598446, 0, 4.124414, 0.593783, 0, 4.124414, 0.593783, 0, 4.152197, 0.598446, 0, 4.152197, 0.598446, 0, 4.152197, 0.598446, 0, 4.181771, 0.600000, 0, 4.152197, 0.598446, 0, 4.152197, 0.598446, 0, 4.181771, 0.600000, 0, 4.181771, 0.600000, 0, 4.181771, 0.600000, 0, 4.228971, 0.596151, 0, 4.181771, 0.600000, 0, 4.181771, 0.600000, 0, 4.228971, 0.596151, 0, 4.228971, 0.596151, 0, 4.228971, 0.596151, 0, 4.270052, 0.584603, 0, 4.228971, 0.596151, 0, 4.228971, 0.596151, 0, 4.270052, 0.584603, 0, 4.270052, 0.584603, 0, 4.270052, 0.584603, 0, 4.305013, 0.565356, 0, 4.270052, 0.584603, 0, 4.270052, 0.584603, 0, 4.305013, 0.565356, 0, 4.305013, 0.565356, 0, 4.305013, 0.565356, 0, 4.333854, 0.538411, 0, 4.305013, 0.565356, 0, 4.305013, 0.565356, 0, 4.333854, 0.538411, 0, 4.333854, 0.538411, 0, 4.333854, 0.538411, 0, 4.356413, 0.503996, 0, 4.333854, 0.538411, 0, 4.333854, 0.538411, 0, 4.356413, 0.503996, 0, 4.356413, 0.503996, 0, 4.356413, 0.503996, 0, 4.372526, 0.462337, 0, 4.356413, 0.503996, 0, 4.356413, 0.503996, 0, 4.372526, 0.462337, 0, 4.372526, 0.462337, 0, 4.372526, 0.462337, 0, 4.382194, 0.413436, 0, 4.372526, 0.462337, 0, 4.372526, 0.462337, 0, 4.382194, 0.413436, 0, 4.382194, 0.413436, 0, 3.993229, 0.583333, 0, 3.896875, 0.583333, 0, 3.993229, 0.500000, 0, 3.896875, 0.583333, 0, 3.993229, 0.583333, 0, 3.993229, 0.500000, 0, 3.896875, 0.583333, 0, 4.004427, 0.410124, 0, 3.993229, 0.500000, 0, 4.004427, 0.410124, 0, 3.896875, 0.583333, 0, 3.993229, 0.500000, 0, 4.004427, 0.410124, 0, 3.896875, 0.583333, 0, 3.996029, 0.374601, 0, 3.896875, 0.583333, 0, 4.004427, 0.410124, 0, 3.996029, 0.374601, 0, 3.996029, 0.374601, 0, 3.896875, 0.583333, 0, 3.993229, 0.334635, 0, 3.896875, 0.583333, 0, 3.996029, 0.374601, 0, 3.993229, 0.334635, 0, 3.896875, 0.583333, 0, 3.896875, 0, 0, 3.993229, 0.334635, 0, 3.896875, 0, 0, 3.896875, 0.583333, 0, 3.993229, 0.334635, 0, 3.896875, 0, 0, 3.993229, 0, 0, 3.993229, 0.334635, 0, 3.993229, 0, 0, 3.896875, 0, 0, 3.993229, 0.334635, 0, 3.993229, 0.500000, 0, 4.004427, 0.410124, 0, 4.018425, 0.441203, 0, 4.004427, 0.410124, 0, 3.993229, 0.500000, 0, 4.018425, 0.441203, 0, 3.993229, 0.500000, 0, 4.018425, 0.441203, 0, 4.038021, 0.467839, 0, 4.018425, 0.441203, 0, 3.993229, 0.500000, 0, 4.038021, 0.467839, 0, 4.011182, 0.523543, 0, 3.993229, 0.500000, 0, 4.038021, 0.467839, 0, 3.993229, 0.500000, 0, 4.011182, 0.523543, 0, 4.038021, 0.467839, 0, 4.011182, 0.523543, 0, 4.038021, 0.467839, 0, 4.062468, 0.489201, 0, 4.038021, 0.467839, 0, 4.011182, 0.523543, 0, 4.062468, 0.489201, 0, 4.030664, 0.543913, 0, 4.011182, 0.523543, 0, 4.062468, 0.489201, 0, 4.011182, 0.523543, 0, 4.030664, 0.543913, 0, 4.062468, 0.489201, 0, 4.091016, 0.504460, 0, 4.030664, 0.543913, 0, 4.062468, 0.489201, 0, 4.030664, 0.543913, 0, 4.091016, 0.504460, 0, 4.062468, 0.489201, 0, 4.091016, 0.504460, 0, 4.051677, 0.561108, 0, 4.030664, 0.543913, 0, 4.051677, 0.561108, 0, 4.091016, 0.504460, 0, 4.030664, 0.543913, 0, 4.074219, 0.575130, 0, 4.051677, 0.561108, 0, 4.091016, 0.504460, 0, 4.051677, 0.561108, 0, 4.074219, 0.575130, 0, 4.091016, 0.504460, 0, 4.074219, 0.575130, 0, 4.091016, 0.504460, 0, 4.123665, 0.513615, 0, 4.091016, 0.504460, 0, 4.074219, 0.575130, 0, 4.123665, 0.513615, 0, 4.098421, 0.586011, 0, 4.074219, 0.575130, 0, 4.123665, 0.513615, 0, 4.074219, 0.575130, 0, 4.098421, 0.586011, 0, 4.123665, 0.513615, 0, 4.098421, 0.586011, 0, 4.123665, 0.513615, 0, 4.124414, 0.593783, 0, 4.123665, 0.513615, 0, 4.098421, 0.586011, 0, 4.124414, 0.593783, 0, 4.124414, 0.593783, 0, 4.123665, 0.513615, 0, 4.160417, 0.516667, 0, 4.123665, 0.513615, 0, 4.124414, 0.593783, 0, 4.160417, 0.516667, 0, 4.152197, 0.598446, 0, 4.124414, 0.593783, 0, 4.160417, 0.516667, 0, 4.124414, 0.593783, 0, 4.152197, 0.598446, 0, 4.160417, 0.516667, 0, 4.181771, 0.600000, 0, 4.152197, 0.598446, 0, 4.160417, 0.516667, 0, 4.152197, 0.598446, 0, 4.181771, 0.600000, 0, 4.160417, 0.516667, 0, 4.181771, 0.600000, 0, 4.160417, 0.516667, 0, 4.190691, 0.514136, 0, 4.160417, 0.516667, 0, 4.181771, 0.600000, 0, 4.190691, 0.514136, 0, 4.228971, 0.596151, 0, 4.181771, 0.600000, 0, 4.190691, 0.514136, 0, 4.181771, 0.600000, 0, 4.228971, 0.596151, 0, 4.190691, 0.514136, 0, 4.190691, 0.514136, 0, 4.216927, 0.506543, 0, 4.228971, 0.596151, 0, 4.216927, 0.506543, 0, 4.190691, 0.514136, 0, 4.228971, 0.596151, 0, 4.228971, 0.596151, 0, 4.216927, 0.506543, 0, 4.270052, 0.584603, 0, 4.216927, 0.506543, 0, 4.228971, 0.596151, 0, 4.270052, 0.584603, 0, 4.216927, 0.506543, 0, 4.239128, 0.493888, 0, 4.270052, 0.584603, 0, 4.239128, 0.493888, 0, 4.216927, 0.506543, 0, 4.270052, 0.584603, 0, 4.270052, 0.584603, 0, 4.239128, 0.493888, 0, 4.305013, 0.565356, 0, 4.239128, 0.493888, 0, 4.270052, 0.584603, 0, 4.305013, 0.565356, 0, 4.239128, 0.493888, 0, 4.257292, 0.476172, 0, 4.305013, 0.565356, 0, 4.257292, 0.476172, 0, 4.239128, 0.493888, 0, 4.305013, 0.565356, 0, 4.305013, 0.565356, 0, 4.257292, 0.476172, 0, 4.333854, 0.538411, 0, 4.257292, 0.476172, 0, 4.305013, 0.565356, 0, 4.333854, 0.538411, 0, 4.333854, 0.538411, 0, 4.257292, 0.476172, 0, 4.356413, 0.503996, 0, 4.257292, 0.476172, 0, 4.333854, 0.538411, 0, 4.356413, 0.503996, 0, 4.356413, 0.503996, 0, 4.257292, 0.476172, 0, 4.271419, 0.453361, 0, 4.257292, 0.476172, 0, 4.356413, 0.503996, 0, 4.271419, 0.453361, 0, 4.356413, 0.503996, 0, 4.271419, 0.453361, 0, 4.372526, 0.462337, 0, 4.271419, 0.453361, 0, 4.356413, 0.503996, 0, 4.372526, 0.462337, 0, 4.271419, 0.453361, 0, 4.281511, 0.425423, 0, 4.372526, 0.462337, 0, 4.281511, 0.425423, 0, 4.271419, 0.453361, 0, 4.372526, 0.462337, 0, 4.372526, 0.462337, 0, 4.281511, 0.425423, 0, 4.382194, 0.413436, 0, 4.281511, 0.425423, 0, 4.372526, 0.462337, 0, 4.382194, 0.413436, 0, 4.281511, 0.425423, 0, 4.287565, 0.392358, 0, 4.382194, 0.413436, 0, 4.287565, 0.392358, 0, 4.281511, 0.425423, 0, 4.382194, 0.413436, 0, 4.382194, 0.413436, 0, 4.287565, 0.392358, 0, 4.385417, 0.357292, 0, 4.287565, 0.392358, 0, 4.382194, 0.413436, 0, 4.385417, 0.357292, 0, 4.385417, 0.357292, 0, 4.287565, 0.392358, 0, 4.289583, 0.354167, 0, 4.287565, 0.392358, 0, 4.385417, 0.357292, 0, 4.289583, 0.354167, 0, 4.289583, 0.354167, 0, 4.385417, 0, 0, 4.385417, 0.357292, 0, 4.385417, 0, 0, 4.289583, 0.354167, 0, 4.385417, 0.357292, 0, 4.289583, 0.354167, 0, 4.289583, 0, 0, 4.385417, 0, 0, 4.289583, 0, 0, 4.289583, 0.354167, 0, 4.385417, 0, 0, 5.003646, 0.550000, 0, 5.003646, 0.466667, 0, 5.003646, 0.550000, 0, 5.003646, 0.550000, 0, 5.003646, 0.466667, 0, 5.003646, 0.466667, 0, 5.003646, 0.466667, 0, 4.983317, 0.478385, 0, 5.003646, 0.466667, 0, 5.003646, 0.466667, 0, 4.983317, 0.478385, 0, 4.983317, 0.478385, 0, 4.983317, 0.478385, 0, 4.962956, 0.488542, 0, 4.983317, 0.478385, 0, 4.983317, 0.478385, 0, 4.962956, 0.488542, 0, 4.962956, 0.488542, 0, 4.962956, 0.488542, 0, 4.942562, 0.497135, 0, 4.962956, 0.488542, 0, 4.962956, 0.488542, 0, 4.942562, 0.497135, 0, 4.942562, 0.497135, 0, 4.942562, 0.497135, 0, 4.922136, 0.504167, 0, 4.942562, 0.497135, 0, 4.942562, 0.497135, 0, 4.922136, 0.504167, 0, 4.922136, 0.504167, 0, 4.922136, 0.504167, 0, 4.901644, 0.509635, 0, 4.922136, 0.504167, 0, 4.922136, 0.504167, 0, 4.901644, 0.509635, 0, 4.901644, 0.509635, 0, 4.901644, 0.509635, 0, 4.881055, 0.513542, 0, 4.901644, 0.509635, 0, 4.901644, 0.509635, 0, 4.881055, 0.513542, 0, 4.881055, 0.513542, 0, 4.881055, 0.513542, 0, 4.860368, 0.515885, 0, 4.881055, 0.513542, 0, 4.881055, 0.513542, 0, 4.860368, 0.515885, 0, 4.860368, 0.515885, 0, 4.860368, 0.515885, 0, 4.839583, 0.516667, 0, 4.860368, 0.515885, 0, 4.860368, 0.515885, 0, 4.839583, 0.516667, 0, 4.839583, 0.516667, 0, 4.839583, 0.516667, 0, 4.795573, 0.512972, 0, 4.839583, 0.516667, 0, 4.839583, 0.516667, 0, 4.795573, 0.512972, 0, 4.795573, 0.512972, 0, 4.795573, 0.512972, 0, 4.756771, 0.501888, 0, 4.795573, 0.512972, 0, 4.795573, 0.512972, 0, 4.756771, 0.501888, 0, 4.756771, 0.501888, 0, 4.756771, 0.501888, 0, 4.723177, 0.483415, 0, 4.756771, 0.501888, 0, 4.756771, 0.501888, 0, 4.723177, 0.483415, 0, 4.723177, 0.483415, 0, 4.723177, 0.483415, 0, 4.694792, 0.457552, 0, 4.723177, 0.483415, 0, 4.723177, 0.483415, 0, 4.694792, 0.457552, 0, 4.694792, 0.457552, 0, 4.694792, 0.457552, 0, 4.672233, 0.425016, 0, 4.694792, 0.457552, 0, 4.694792, 0.457552, 0, 4.672233, 0.425016, 0, 4.672233, 0.425016, 0, 4.672233, 0.425016, 0, 4.656120, 0.386523, 0, 4.672233, 0.425016, 0, 4.672233, 0.425016, 0, 4.656120, 0.386523, 0, 4.656120, 0.386523, 0, 4.656120, 0.386523, 0, 4.646452, 0.342074, 0, 4.656120, 0.386523, 0, 4.656120, 0.386523, 0, 4.646452, 0.342074, 0, 4.646452, 0.342074, 0, 4.646452, 0.342074, 0, 4.643229, 0.291667, 0, 4.646452, 0.342074, 0, 4.646452, 0.342074, 0, 4.643229, 0.291667, 0, 4.643229, 0.291667, 0, 4.643229, 0.291667, 0, 4.646452, 0.241260, 0, 4.643229, 0.291667, 0, 4.643229, 0.291667, 0, 4.646452, 0.241260, 0, 4.646452, 0.241260, 0, 4.646452, 0.241260, 0, 4.656120, 0.196810, 0, 4.646452, 0.241260, 0, 4.646452, 0.241260, 0, 4.656120, 0.196810, 0, 4.656120, 0.196810, 0, 4.656120, 0.196810, 0, 4.672233, 0.158317, 0, 4.656120, 0.196810, 0, 4.656120, 0.196810, 0, 4.672233, 0.158317, 0, 4.672233, 0.158317, 0, 4.672233, 0.158317, 0, 4.694792, 0.125781, 0, 4.672233, 0.158317, 0, 4.672233, 0.158317, 0, 4.694792, 0.125781, 0, 4.694792, 0.125781, 0, 4.694792, 0.125781, 0, 4.723177, 0.099919, 0, 4.694792, 0.125781, 0, 4.694792, 0.125781, 0, 4.723177, 0.099919, 0, 4.723177, 0.099919, 0, 4.723177, 0.099919, 0, 4.756771, 0.081445, 0, 4.723177, 0.099919, 0, 4.723177, 0.099919, 0, 4.756771, 0.081445, 0, 4.756771, 0.081445, 0, 4.756771, 0.081445, 0, 4.795573, 0.070361, 0, 4.756771, 0.081445, 0, 4.756771, 0.081445, 0, 4.795573, 0.070361, 0, 4.795573, 0.070361, 0, 4.795573, 0.070361, 0, 4.839583, 0.066667, 0, 4.795573, 0.070361, 0, 4.795573, 0.070361, 0, 4.839583, 0.066667, 0, 4.839583, 0.066667, 0, 4.839583, 0.066667, 0, 4.860368, 0.067448, 0, 4.839583, 0.066667, 0, 4.839583, 0.066667, 0, 4.860368, 0.067448, 0, 4.860368, 0.067448, 0, 4.860368, 0.067448, 0, 4.881055, 0.069792, 0, 4.860368, 0.067448, 0, 4.860368, 0.067448, 0, 4.881055, 0.069792, 0, 4.881055, 0.069792, 0, 4.881055, 0.069792, 0, 4.901644, 0.073698, 0, 4.881055, 0.069792, 0, 4.881055, 0.069792, 0, 4.901644, 0.073698, 0, 4.901644, 0.073698, 0, 4.901644, 0.073698, 0, 4.922136, 0.079167, 0, 4.901644, 0.073698, 0, 4.901644, 0.073698, 0, 4.922136, 0.079167, 0, 4.922136, 0.079167, 0, 4.922136, 0.079167, 0, 4.942562, 0.086198, 0, 4.922136, 0.079167, 0, 4.922136, 0.079167, 0, 4.942562, 0.086198, 0, 4.942562, 0.086198, 0, 4.942562, 0.086198, 0, 4.962956, 0.094792, 0, 4.942562, 0.086198, 0, 4.942562, 0.086198, 0, 4.962956, 0.094792, 0, 4.962956, 0.094792, 0, 4.962956, 0.094792, 0, 4.983317, 0.104948, 0, 4.962956, 0.094792, 0, 4.962956, 0.094792, 0, 4.983317, 0.104948, 0, 4.983317, 0.104948, 0, 4.983317, 0.104948, 0, 5.003646, 0.116667, 0, 4.983317, 0.104948, 0, 4.983317, 0.104948, 0, 5.003646, 0.116667, 0, 5.003646, 0.116667, 0, 5.003646, 0.116667, 0, 5.003646, 0.033333, 0, 5.003646, 0.116667, 0, 5.003646, 0.116667, 0, 5.003646, 0.033333, 0, 5.003646, 0.033333, 0, 5.003646, 0.033333, 0, 4.983415, 0.021615, 0, 5.003646, 0.033333, 0, 5.003646, 0.033333, 0, 4.983415, 0.021615, 0, 4.983415, 0.021615, 0, 4.983415, 0.021615, 0, 4.962825, 0.011458, 0, 4.983415, 0.021615, 0, 4.983415, 0.021615, 0, 4.962825, 0.011458, 0, 4.962825, 0.011458, 0, 4.962825, 0.011458, 0, 4.941878, 0.002865, 0, 4.962825, 0.011458, 0, 4.962825, 0.011458, 0, 4.941878, 0.002865, 0, 4.941878, 0.002865, 0, 4.941878, 0.002865, 0, 4.920573, -0.004167, 0, 4.941878, 0.002865, 0, 4.941878, 0.002865, 0, 4.920573, -0.004167, 0, 4.920573, -0.004167, 0, 4.920573, -0.004167, 0, 4.898747, -0.009635, 0, 4.920573, -0.004167, 0, 4.920573, -0.004167, 0, 4.898747, -0.009635, 0, 4.898747, -0.009635, 0, 4.898747, -0.009635, 0, 4.876237, -0.013542, 0, 4.898747, -0.009635, 0, 4.898747, -0.009635, 0, 4.876237, -0.013542, 0, 4.876237, -0.013542, 0, 4.876237, -0.013542, 0, 4.853044, -0.015885, 0, 4.876237, -0.013542, 0, 4.876237, -0.013542, 0, 4.853044, -0.015885, 0, 4.853044, -0.015885, 0, 4.853044, -0.015885, 0, 4.829167, -0.016667, 0, 4.853044, -0.015885, 0, 4.853044, -0.015885, 0, 4.829167, -0.016667, 0, 4.829167, -0.016667, 0, 4.829167, -0.016667, 0, 4.766667, -0.011458, 0, 4.829167, -0.016667, 0, 4.829167, -0.016667, 0, 4.766667, -0.011458, 0, 4.766667, -0.011458, 0, 4.766667, -0.011458, 0, 4.710938, 0.004167, 0, 4.766667, -0.011458, 0, 4.766667, -0.011458, 0, 4.710938, 0.004167, 0, 4.710938, 0.004167, 0, 4.710938, 0.004167, 0, 4.661979, 0.030208, 0, 4.710938, 0.004167, 0, 4.710938, 0.004167, 0, 4.661979, 0.030208, 0, 4.661979, 0.030208, 0, 4.661979, 0.030208, 0, 4.619792, 0.066667, 0, 4.661979, 0.030208, 0, 4.661979, 0.030208, 0, 4.619792, 0.066667, 0, 4.619792, 0.066667, 0, 4.619792, 0.066667, 0, 4.585840, 0.111979, 0, 4.619792, 0.066667, 0, 4.619792, 0.066667, 0, 4.585840, 0.111979, 0, 4.585840, 0.111979, 0, 4.585840, 0.111979, 0, 4.561589, 0.164583, 0, 4.585840, 0.111979, 0, 4.585840, 0.111979, 0, 4.561589, 0.164583, 0, 4.561589, 0.164583, 0, 4.561589, 0.164583, 0, 4.547038, 0.224479, 0, 4.561589, 0.164583, 0, 4.561589, 0.164583, 0, 4.547038, 0.224479, 0, 4.547038, 0.224479, 0, 4.547038, 0.224479, 0, 4.542188, 0.291667, 0, 4.547038, 0.224479, 0, 4.547038, 0.224479, 0, 4.542188, 0.291667, 0, 4.542188, 0.291667, 0, 4.542188, 0.291667, 0, 4.547087, 0.359701, 0, 4.542188, 0.291667, 0, 4.542188, 0.291667, 0, 4.547087, 0.359701, 0, 4.547087, 0.359701, 0, 4.547087, 0.359701, 0, 4.561784, 0.420052, 0, 4.547087, 0.359701, 0, 4.547087, 0.359701, 0, 4.561784, 0.420052, 0, 4.561784, 0.420052, 0, 4.561784, 0.420052, 0, 4.586279, 0.472721, 0, 4.561784, 0.420052, 0, 4.561784, 0.420052, 0, 4.586279, 0.472721, 0, 4.586279, 0.472721, 0, 4.586279, 0.472721, 0, 4.620573, 0.517708, 0, 4.586279, 0.472721, 0, 4.586279, 0.472721, 0, 4.620573, 0.517708, 0, 4.620573, 0.517708, 0, 4.620573, 0.517708, 0, 4.663395, 0.553711, 0, 4.620573, 0.517708, 0, 4.620573, 0.517708, 0, 4.663395, 0.553711, 0, 4.663395, 0.553711, 0, 4.663395, 0.553711, 0, 4.713477, 0.579427, 0, 4.663395, 0.553711, 0, 4.663395, 0.553711, 0, 4.713477, 0.579427, 0, 4.713477, 0.579427, 0, 4.713477, 0.579427, 0, 4.770817, 0.594857, 0, 4.713477, 0.579427, 0, 4.713477, 0.579427, 0, 4.770817, 0.594857, 0, 4.770817, 0.594857, 0, 4.770817, 0.594857, 0, 4.835417, 0.600000, 0, 4.770817, 0.594857, 0, 4.770817, 0.594857, 0, 4.835417, 0.600000, 0, 4.835417, 0.600000, 0, 4.835417, 0.600000, 0, 4.857422, 0.599219, 0, 4.835417, 0.600000, 0, 4.835417, 0.600000, 0, 4.857422, 0.599219, 0, 4.857422, 0.599219, 0, 4.857422, 0.599219, 0, 4.879167, 0.596875, 0, 4.857422, 0.599219, 0, 4.857422, 0.599219, 0, 4.879167, 0.596875, 0, 4.879167, 0.596875, 0, 4.879167, 0.596875, 0, 4.900651, 0.592969, 0, 4.879167, 0.596875, 0, 4.879167, 0.596875, 0, 4.900651, 0.592969, 0, 4.900651, 0.592969, 0, 4.900651, 0.592969, 0, 4.921875, 0.587500, 0, 4.900651, 0.592969, 0, 4.900651, 0.592969, 0, 4.921875, 0.587500, 0, 4.921875, 0.587500, 0, 4.921875, 0.587500, 0, 4.942806, 0.580469, 0, 4.921875, 0.587500, 0, 4.921875, 0.587500, 0, 4.942806, 0.580469, 0, 4.942806, 0.580469, 0, 4.942806, 0.580469, 0, 4.963411, 0.571875, 0, 4.942806, 0.580469, 0, 4.942806, 0.580469, 0, 4.963411, 0.571875, 0, 4.963411, 0.571875, 0, 4.963411, 0.571875, 0, 4.983692, 0.561719, 0, 4.963411, 0.571875, 0, 4.963411, 0.571875, 0, 4.983692, 0.561719, 0, 4.983692, 0.561719, 0, 4.646452, 0.342074, 0, 4.547087, 0.359701, 0, 4.542188, 0.291667, 0, 4.547087, 0.359701, 0, 4.646452, 0.342074, 0, 4.542188, 0.291667, 0, 4.656120, 0.386523, 0, 4.547087, 0.359701, 0, 4.646452, 0.342074, 0, 4.547087, 0.359701, 0, 4.656120, 0.386523, 0, 4.646452, 0.342074, 0, 4.656120, 0.386523, 0, 4.561784, 0.420052, 0, 4.547087, 0.359701, 0, 4.561784, 0.420052, 0, 4.656120, 0.386523, 0, 4.547087, 0.359701, 0, 4.672233, 0.425016, 0, 4.561784, 0.420052, 0, 4.656120, 0.386523, 0, 4.561784, 0.420052, 0, 4.672233, 0.425016, 0, 4.656120, 0.386523, 0, 4.672233, 0.425016, 0, 4.586279, 0.472721, 0, 4.561784, 0.420052, 0, 4.586279, 0.472721, 0, 4.672233, 0.425016, 0, 4.561784, 0.420052, 0, 4.620573, 0.517708, 0, 4.586279, 0.472721, 0, 4.672233, 0.425016, 0, 4.586279, 0.472721, 0, 4.620573, 0.517708, 0, 4.672233, 0.425016, 0, 4.620573, 0.517708, 0, 4.672233, 0.425016, 0, 4.694792, 0.457552, 0, 4.672233, 0.425016, 0, 4.620573, 0.517708, 0, 4.694792, 0.457552, 0, 4.663395, 0.553711, 0, 4.620573, 0.517708, 0, 4.694792, 0.457552, 0, 4.620573, 0.517708, 0, 4.663395, 0.553711, 0, 4.694792, 0.457552, 0, 4.663395, 0.553711, 0, 4.694792, 0.457552, 0, 4.723177, 0.483415, 0, 4.694792, 0.457552, 0, 4.663395, 0.553711, 0, 4.723177, 0.483415, 0, 4.713477, 0.579427, 0, 4.663395, 0.553711, 0, 4.723177, 0.483415, 0, 4.663395, 0.553711, 0, 4.713477, 0.579427, 0, 4.723177, 0.483415, 0, 4.713477, 0.579427, 0, 4.723177, 0.483415, 0, 4.756771, 0.501888, 0, 4.723177, 0.483415, 0, 4.713477, 0.579427, 0, 4.756771, 0.501888, 0, 4.713477, 0.579427, 0, 4.756771, 0.501888, 0, 4.770817, 0.594857, 0, 4.756771, 0.501888, 0, 4.713477, 0.579427, 0, 4.770817, 0.594857, 0, 4.770817, 0.594857, 0, 4.756771, 0.501888, 0, 4.795573, 0.512972, 0, 4.756771, 0.501888, 0, 4.770817, 0.594857, 0, 4.795573, 0.512972, 0, 4.835417, 0.600000, 0, 4.770817, 0.594857, 0, 4.795573, 0.512972, 0, 4.770817, 0.594857, 0, 4.835417, 0.600000, 0, 4.795573, 0.512972, 0, 4.795573, 0.512972, 0, 4.839583, 0.516667, 0, 4.835417, 0.600000, 0, 4.839583, 0.516667, 0, 4.795573, 0.512972, 0, 4.835417, 0.600000, 0, 4.857422, 0.599219, 0, 4.835417, 0.600000, 0, 4.839583, 0.516667, 0, 4.835417, 0.600000, 0, 4.857422, 0.599219, 0, 4.839583, 0.516667, 0, 4.857422, 0.599219, 0, 4.839583, 0.516667, 0, 4.860368, 0.515885, 0, 4.839583, 0.516667, 0, 4.857422, 0.599219, 0, 4.860368, 0.515885, 0, 4.857422, 0.599219, 0, 4.860368, 0.515885, 0, 4.879167, 0.596875, 0, 4.860368, 0.515885, 0, 4.857422, 0.599219, 0, 4.879167, 0.596875, 0, 4.879167, 0.596875, 0, 4.860368, 0.515885, 0, 4.881055, 0.513542, 0, 4.860368, 0.515885, 0, 4.879167, 0.596875, 0, 4.881055, 0.513542, 0, 4.900651, 0.592969, 0, 4.879167, 0.596875, 0, 4.881055, 0.513542, 0, 4.879167, 0.596875, 0, 4.900651, 0.592969, 0, 4.881055, 0.513542, 0, 4.900651, 0.592969, 0, 4.881055, 0.513542, 0, 4.901644, 0.509635, 0, 4.881055, 0.513542, 0, 4.900651, 0.592969, 0, 4.901644, 0.509635, 0, 4.900651, 0.592969, 0, 4.901644, 0.509635, 0, 4.921875, 0.587500, 0, 4.901644, 0.509635, 0, 4.900651, 0.592969, 0, 4.921875, 0.587500, 0, 4.942806, 0.580469, 0, 4.921875, 0.587500, 0, 4.901644, 0.509635, 0, 4.921875, 0.587500, 0, 4.942806, 0.580469, 0, 4.901644, 0.509635, 0, 4.942806, 0.580469, 0, 4.901644, 0.509635, 0, 4.922136, 0.504167, 0, 4.901644, 0.509635, 0, 4.942806, 0.580469, 0, 4.922136, 0.504167, 0, 4.942806, 0.580469, 0, 4.922136, 0.504167, 0, 4.963411, 0.571875, 0, 4.922136, 0.504167, 0, 4.942806, 0.580469, 0, 4.963411, 0.571875, 0, 4.922136, 0.504167, 0, 4.942562, 0.497135, 0, 4.963411, 0.571875, 0, 4.942562, 0.497135, 0, 4.922136, 0.504167, 0, 4.963411, 0.571875, 0, 4.963411, 0.571875, 0, 4.942562, 0.497135, 0, 4.983692, 0.561719, 0, 4.942562, 0.497135, 0, 4.963411, 0.571875, 0, 4.983692, 0.561719, 0, 4.942562, 0.497135, 0, 4.962956, 0.488542, 0, 4.983692, 0.561719, 0, 4.962956, 0.488542, 0, 4.942562, 0.497135, 0, 4.983692, 0.561719, 0, 4.983692, 0.561719, 0, 4.962956, 0.488542, 0, 5.003646, 0.550000, 0, 4.962956, 0.488542, 0, 4.983692, 0.561719, 0, 5.003646, 0.550000, 0, 4.962956, 0.488542, 0, 4.983317, 0.478385, 0, 5.003646, 0.550000, 0, 4.983317, 0.478385, 0, 4.962956, 0.488542, 0, 5.003646, 0.550000, 0, 4.983317, 0.478385, 0, 5.003646, 0.466667, 0, 5.003646, 0.550000, 0, 5.003646, 0.466667, 0, 4.983317, 0.478385, 0, 5.003646, 0.550000, 0, 4.646452, 0.342074, 0, 4.542188, 0.291667, 0, 4.643229, 0.291667, 0, 4.542188, 0.291667, 0, 4.646452, 0.342074, 0, 4.643229, 0.291667, 0, 4.643229, 0.291667, 0, 4.542188, 0.291667, 0, 4.646452, 0.241260, 0, 4.542188, 0.291667, 0, 4.643229, 0.291667, 0, 4.646452, 0.241260, 0, 4.542188, 0.291667, 0, 4.547038, 0.224479, 0, 4.646452, 0.241260, 0, 4.547038, 0.224479, 0, 4.542188, 0.291667, 0, 4.646452, 0.241260, 0, 4.646452, 0.241260, 0, 4.547038, 0.224479, 0, 4.656120, 0.196810, 0, 4.547038, 0.224479, 0, 4.646452, 0.241260, 0, 4.656120, 0.196810, 0, 4.547038, 0.224479, 0, 4.561589, 0.164583, 0, 4.656120, 0.196810, 0, 4.561589, 0.164583, 0, 4.547038, 0.224479, 0, 4.656120, 0.196810, 0, 4.656120, 0.196810, 0, 4.561589, 0.164583, 0, 4.585840, 0.111979, 0, 4.561589, 0.164583, 0, 4.656120, 0.196810, 0, 4.585840, 0.111979, 0, 4.672233, 0.158317, 0, 4.656120, 0.196810, 0, 4.585840, 0.111979, 0, 4.656120, 0.196810, 0, 4.672233, 0.158317, 0, 4.585840, 0.111979, 0, 4.672233, 0.158317, 0, 4.585840, 0.111979, 0, 4.619792, 0.066667, 0, 4.585840, 0.111979, 0, 4.672233, 0.158317, 0, 4.619792, 0.066667, 0, 4.694792, 0.125781, 0, 4.672233, 0.158317, 0, 4.619792, 0.066667, 0, 4.672233, 0.158317, 0, 4.694792, 0.125781, 0, 4.619792, 0.066667, 0, 4.619792, 0.066667, 0, 4.661979, 0.030208, 0, 4.694792, 0.125781, 0, 4.661979, 0.030208, 0, 4.619792, 0.066667, 0, 4.694792, 0.125781, 0, 4.694792, 0.125781, 0, 4.661979, 0.030208, 0, 4.723177, 0.099919, 0, 4.661979, 0.030208, 0, 4.694792, 0.125781, 0, 4.723177, 0.099919, 0, 4.723177, 0.099919, 0, 4.661979, 0.030208, 0, 4.710938, 0.004167, 0, 4.661979, 0.030208, 0, 4.723177, 0.099919, 0, 4.710938, 0.004167, 0, 4.756771, 0.081445, 0, 4.723177, 0.099919, 0, 4.710938, 0.004167, 0, 4.723177, 0.099919, 0, 4.756771, 0.081445, 0, 4.710938, 0.004167, 0, 4.756771, 0.081445, 0, 4.710938, 0.004167, 0, 4.766667, -0.011458, 0, 4.710938, 0.004167, 0, 4.756771, 0.081445, 0, 4.766667, -0.011458, 0, 4.795573, 0.070361, 0, 4.756771, 0.081445, 0, 4.766667, -0.011458, 0, 4.756771, 0.081445, 0, 4.795573, 0.070361, 0, 4.766667, -0.011458, 0, 4.795573, 0.070361, 0, 4.766667, -0.011458, 0, 4.829167, -0.016667, 0, 4.766667, -0.011458, 0, 4.795573, 0.070361, 0, 4.829167, -0.016667, 0, 4.839583, 0.066667, 0, 4.795573, 0.070361, 0, 4.829167, -0.016667, 0, 4.795573, 0.070361, 0, 4.839583, 0.066667, 0, 4.829167, -0.016667, 0, 4.839583, 0.066667, 0, 4.829167, -0.016667, 0, 4.853044, -0.015885, 0, 4.829167, -0.016667, 0, 4.839583, 0.066667, 0, 4.853044, -0.015885, 0, 4.839583, 0.066667, 0, 4.853044, -0.015885, 0, 4.860368, 0.067448, 0, 4.853044, -0.015885, 0, 4.839583, 0.066667, 0, 4.860368, 0.067448, 0, 4.860368, 0.067448, 0, 4.853044, -0.015885, 0, 4.876237, -0.013542, 0, 4.853044, -0.015885, 0, 4.860368, 0.067448, 0, 4.876237, -0.013542, 0, 4.881055, 0.069792, 0, 4.860368, 0.067448, 0, 4.876237, -0.013542, 0, 4.860368, 0.067448, 0, 4.881055, 0.069792, 0, 4.876237, -0.013542, 0, 4.898747, -0.009635, 0, 4.881055, 0.069792, 0, 4.876237, -0.013542, 0, 4.881055, 0.069792, 0, 4.898747, -0.009635, 0, 4.876237, -0.013542, 0, 4.901644, 0.073698, 0, 4.881055, 0.069792, 0, 4.898747, -0.009635, 0, 4.881055, 0.069792, 0, 4.901644, 0.073698, 0, 4.898747, -0.009635, 0, 4.920573, -0.004167, 0, 4.901644, 0.073698, 0, 4.898747, -0.009635, 0, 4.901644, 0.073698, 0, 4.920573, -0.004167, 0, 4.898747, -0.009635, 0, 4.901644, 0.073698, 0, 4.920573, -0.004167, 0, 4.941878, 0.002865, 0, 4.920573, -0.004167, 0, 4.901644, 0.073698, 0, 4.941878, 0.002865, 0, 4.922136, 0.079167, 0, 4.901644, 0.073698, 0, 4.941878, 0.002865, 0, 4.901644, 0.073698, 0, 4.922136, 0.079167, 0, 4.941878, 0.002865, 0, 4.922136, 0.079167, 0, 4.941878, 0.002865, 0, 4.962825, 0.011458, 0, 4.941878, 0.002865, 0, 4.922136, 0.079167, 0, 4.962825, 0.011458, 0, 4.942562, 0.086198, 0, 4.922136, 0.079167, 0, 4.962825, 0.011458, 0, 4.922136, 0.079167, 0, 4.942562, 0.086198, 0, 4.962825, 0.011458, 0, 4.942562, 0.086198, 0, 4.962825, 0.011458, 0, 4.983415, 0.021615, 0, 4.962825, 0.011458, 0, 4.942562, 0.086198, 0, 4.983415, 0.021615, 0, 4.962956, 0.094792, 0, 4.942562, 0.086198, 0, 4.983415, 0.021615, 0, 4.942562, 0.086198, 0, 4.962956, 0.094792, 0, 4.983415, 0.021615, 0, 4.962956, 0.094792, 0, 4.983415, 0.021615, 0, 5.003646, 0.033333, 0, 4.983415, 0.021615, 0, 4.962956, 0.094792, 0, 5.003646, 0.033333, 0, 4.983317, 0.104948, 0, 4.962956, 0.094792, 0, 5.003646, 0.033333, 0, 4.962956, 0.094792, 0, 4.983317, 0.104948, 0, 5.003646, 0.033333, 0, 4.983317, 0.104948, 0, 5.003646, 0.033333, 0, 5.003646, 0.116667, 0, 5.003646, 0.033333, 0, 4.983317, 0.104948, 0, 5.003646, 0.116667, 0, 5.666146, 0.318750, 0, 5.666146, 0.266667, 0, 5.666146, 0.318750, 0, 5.666146, 0.318750, 0, 5.666146, 0.266667, 0, 5.666146, 0.266667, 0, 5.666146, 0.266667, 0, 5.225521, 0.266667, 0, 5.666146, 0.266667, 0, 5.666146, 0.266667, 0, 5.225521, 0.266667, 0, 5.225521, 0.266667, 0, 5.225521, 0.266667, 0, 5.231592, 0.220744, 0, 5.225521, 0.266667, 0, 5.225521, 0.266667, 0, 5.231592, 0.220744, 0, 5.231592, 0.220744, 0, 5.231592, 0.220744, 0, 5.243555, 0.180632, 0, 5.231592, 0.220744, 0, 5.231592, 0.220744, 0, 5.243555, 0.180632, 0, 5.243555, 0.180632, 0, 5.243555, 0.180632, 0, 5.261409, 0.146330, 0, 5.243555, 0.180632, 0, 5.243555, 0.180632, 0, 5.261409, 0.146330, 0, 5.261409, 0.146330, 0, 5.261409, 0.146330, 0, 5.285156, 0.117839, 0, 5.261409, 0.146330, 0, 5.261409, 0.146330, 0, 5.285156, 0.117839, 0, 5.285156, 0.117839, 0, 5.285156, 0.117839, 0, 5.314470, 0.095451, 0, 5.285156, 0.117839, 0, 5.285156, 0.117839, 0, 5.314470, 0.095451, 0, 5.314470, 0.095451, 0, 5.314470, 0.095451, 0, 5.349023, 0.079460, 0, 5.314470, 0.095451, 0, 5.314470, 0.095451, 0, 5.349023, 0.079460, 0, 5.349023, 0.079460, 0, 5.349023, 0.079460, 0, 5.388818, 0.069865, 0, 5.349023, 0.079460, 0, 5.349023, 0.079460, 0, 5.388818, 0.069865, 0, 5.388818, 0.069865, 0, 5.388818, 0.069865, 0, 5.433855, 0.066667, 0, 5.388818, 0.069865, 0, 5.388818, 0.069865, 0, 5.433855, 0.066667, 0, 5.433855, 0.066667, 0, 5.433855, 0.066667, 0, 5.461247, 0.067448, 0, 5.433855, 0.066667, 0, 5.433855, 0.066667, 0, 5.461247, 0.067448, 0, 5.461247, 0.067448, 0, 5.461247, 0.067448, 0, 5.488216, 0.069792, 0, 5.461247, 0.067448, 0, 5.461247, 0.067448, 0, 5.488216, 0.069792, 0, 5.488216, 0.069792, 0, 5.488216, 0.069792, 0, 5.514762, 0.073698, 0, 5.488216, 0.069792, 0, 5.488216, 0.069792, 0, 5.514762, 0.073698, 0, 5.514762, 0.073698, 0, 5.514762, 0.073698, 0, 5.540885, 0.079167, 0, 5.514762, 0.073698, 0, 5.514762, 0.073698, 0, 5.540885, 0.079167, 0, 5.540885, 0.079167, 0, 5.540885, 0.079167, 0, 5.566748, 0.086198, 0, 5.540885, 0.079167, 0, 5.540885, 0.079167, 0, 5.566748, 0.086198, 0, 5.566748, 0.086198, 0, 5.566748, 0.086198, 0, 5.592513, 0.094792, 0, 5.566748, 0.086198, 0, 5.566748, 0.086198, 0, 5.592513, 0.094792, 0, 5.592513, 0.094792, 0, 5.592513, 0.094792, 0, 5.618181, 0.104948, 0, 5.592513, 0.094792, 0, 5.592513, 0.094792, 0, 5.618181, 0.104948, 0, 5.618181, 0.104948, 0, 5.618181, 0.104948, 0, 5.643750, 0.116667, 0, 5.618181, 0.104948, 0, 5.618181, 0.104948, 0, 5.643750, 0.116667, 0, 5.643750, 0.116667, 0, 5.643750, 0.116667, 0, 5.643750, 0.033333, 0, 5.643750, 0.116667, 0, 5.643750, 0.116667, 0, 5.643750, 0.033333, 0, 5.643750, 0.033333, 0, 5.643750, 0.033333, 0, 5.617806, 0.021826, 0, 5.643750, 0.033333, 0, 5.643750, 0.033333, 0, 5.617806, 0.021826, 0, 5.617806, 0.021826, 0, 5.617806, 0.021826, 0, 5.591537, 0.011784, 0, 5.617806, 0.021826, 0, 5.617806, 0.021826, 0, 5.591537, 0.011784, 0, 5.591537, 0.011784, 0, 5.591537, 0.011784, 0, 5.564941, 0.003206, 0, 5.591537, 0.011784, 0, 5.591537, 0.011784, 0, 5.564941, 0.003206, 0, 5.564941, 0.003206, 0, 5.564941, 0.003206, 0, 5.538021, -0.003906, 0, 5.564941, 0.003206, 0, 5.564941, 0.003206, 0, 5.538021, -0.003906, 0, 5.538021, -0.003906, 0, 5.538021, -0.003906, 0, 5.510840, -0.009489, 0, 5.538021, -0.003906, 0, 5.538021, -0.003906, 0, 5.510840, -0.009489, 0, 5.510840, -0.009489, 0, 5.510840, -0.009489, 0, 5.483464, -0.013477, 0, 5.510840, -0.009489, 0, 5.510840, -0.009489, 0, 5.483464, -0.013477, 0, 5.483464, -0.013477, 0, 5.483464, -0.013477, 0, 5.455892, -0.015869, 0, 5.483464, -0.013477, 0, 5.483464, -0.013477, 0, 5.455892, -0.015869, 0, 5.455892, -0.015869, 0, 5.455892, -0.015869, 0, 5.428125, -0.016667, 0, 5.455892, -0.015869, 0, 5.455892, -0.015869, 0, 5.428125, -0.016667, 0, 5.428125, -0.016667, 0, 5.428125, -0.016667, 0, 5.361963, -0.011556, 0, 5.428125, -0.016667, 0, 5.428125, -0.016667, 0, 5.361963, -0.011556, 0, 5.361963, -0.011556, 0, 5.361963, -0.011556, 0, 5.303060, 0.003776, 0, 5.361963, -0.011556, 0, 5.361963, -0.011556, 0, 5.303060, 0.003776, 0, 5.303060, 0.003776, 0, 5.303060, 0.003776, 0, 5.251416, 0.029329, 0, 5.303060, 0.003776, 0, 5.303060, 0.003776, 0, 5.251416, 0.029329, 0, 5.251416, 0.029329, 0, 5.251416, 0.029329, 0, 5.207031, 0.065104, 0, 5.251416, 0.029329, 0, 5.251416, 0.029329, 0, 5.207031, 0.065104, 0, 5.207031, 0.065104, 0, 5.207031, 0.065104, 0, 5.171371, 0.109603, 0, 5.207031, 0.065104, 0, 5.207031, 0.065104, 0, 5.171371, 0.109603, 0, 5.171371, 0.109603, 0, 5.171371, 0.109603, 0, 5.145898, 0.161328, 0, 5.171371, 0.109603, 0, 5.171371, 0.109603, 0, 5.145898, 0.161328, 0, 5.145898, 0.161328, 0, 5.145898, 0.161328, 0, 5.130615, 0.220280, 0, 5.145898, 0.161328, 0, 5.145898, 0.161328, 0, 5.130615, 0.220280, 0, 5.130615, 0.220280, 0, 5.130615, 0.220280, 0, 5.125521, 0.286458, 0, 5.130615, 0.220280, 0, 5.130615, 0.220280, 0, 5.125521, 0.286458, 0, 5.125521, 0.286458, 0, 5.125521, 0.286458, 0, 5.130355, 0.354867, 0, 5.125521, 0.286458, 0, 5.125521, 0.286458, 0, 5.130355, 0.354867, 0, 5.130355, 0.354867, 0, 5.130355, 0.354867, 0, 5.144857, 0.415820, 0, 5.130355, 0.354867, 0, 5.130355, 0.354867, 0, 5.144857, 0.415820, 0, 5.144857, 0.415820, 0, 5.144857, 0.415820, 0, 5.169027, 0.469320, 0, 5.144857, 0.415820, 0, 5.144857, 0.415820, 0, 5.169027, 0.469320, 0, 5.169027, 0.469320, 0, 5.169027, 0.469320, 0, 5.202865, 0.515365, 0, 5.169027, 0.469320, 0, 5.169027, 0.469320, 0, 5.202865, 0.515365, 0, 5.202865, 0.515365, 0, 5.202865, 0.515365, 0, 5.244905, 0.552393, 0, 5.202865, 0.515365, 0, 5.202865, 0.515365, 0, 5.244905, 0.552393, 0, 5.244905, 0.552393, 0, 5.244905, 0.552393, 0, 5.293685, 0.578841, 0, 5.244905, 0.552393, 0, 5.244905, 0.552393, 0, 5.293685, 0.578841, 0, 5.293685, 0.578841, 0, 5.293685, 0.578841, 0, 5.349203, 0.594710, 0, 5.293685, 0.578841, 0, 5.293685, 0.578841, 0, 5.349203, 0.594710, 0, 5.349203, 0.594710, 0, 5.349203, 0.594710, 0, 5.411458, 0.600000, 0, 5.349203, 0.594710, 0, 5.349203, 0.594710, 0, 5.411458, 0.600000, 0, 5.411458, 0.600000, 0, 5.411458, 0.600000, 0, 5.467237, 0.595272, 0, 5.411458, 0.600000, 0, 5.411458, 0.600000, 0, 5.467237, 0.595272, 0, 5.467237, 0.595272, 0, 5.467237, 0.595272, 0, 5.516862, 0.581087, 0, 5.467237, 0.595272, 0, 5.467237, 0.595272, 0, 5.516862, 0.581087, 0, 5.516862, 0.581087, 0, 5.516862, 0.581087, 0, 5.560335, 0.557446, 0, 5.516862, 0.581087, 0, 5.516862, 0.581087, 0, 5.560335, 0.557446, 0, 5.560335, 0.557446, 0, 5.560335, 0.557446, 0, 5.597656, 0.524349, 0, 5.560335, 0.557446, 0, 5.560335, 0.557446, 0, 5.597656, 0.524349, 0, 5.597656, 0.524349, 0, 5.597656, 0.524349, 0, 5.627620, 0.483130, 0, 5.597656, 0.524349, 0, 5.597656, 0.524349, 0, 5.627620, 0.483130, 0, 5.627620, 0.483130, 0, 5.627620, 0.483130, 0, 5.649024, 0.435124, 0, 5.627620, 0.483130, 0, 5.627620, 0.483130, 0, 5.649024, 0.435124, 0, 5.649024, 0.435124, 0, 5.649024, 0.435124, 0, 5.661865, 0.380330, 0, 5.649024, 0.435124, 0, 5.649024, 0.435124, 0, 5.661865, 0.380330, 0, 5.661865, 0.380330, 0, 5.130355, 0.354867, 0, 5.125521, 0.286458, 0, 5.228646, 0.349479, 0, 5.125521, 0.286458, 0, 5.130355, 0.354867, 0, 5.228646, 0.349479, 0, 5.144857, 0.415820, 0, 5.130355, 0.354867, 0, 5.228646, 0.349479, 0, 5.130355, 0.354867, 0, 5.144857, 0.415820, 0, 5.228646, 0.349479, 0, 5.144857, 0.415820, 0, 5.228646, 0.349479, 0, 5.234847, 0.387077, 0, 5.228646, 0.349479, 0, 5.144857, 0.415820, 0, 5.234847, 0.387077, 0, 5.246159, 0.420182, 0, 5.144857, 0.415820, 0, 5.234847, 0.387077, 0, 5.144857, 0.415820, 0, 5.246159, 0.420182, 0, 5.234847, 0.387077, 0, 5.246159, 0.420182, 0, 5.169027, 0.469320, 0, 5.144857, 0.415820, 0, 5.169027, 0.469320, 0, 5.246159, 0.420182, 0, 5.144857, 0.415820, 0, 5.169027, 0.469320, 0, 5.246159, 0.420182, 0, 5.262581, 0.448796, 0, 5.246159, 0.420182, 0, 5.169027, 0.469320, 0, 5.262581, 0.448796, 0, 5.202865, 0.515365, 0, 5.169027, 0.469320, 0, 5.262581, 0.448796, 0, 5.169027, 0.469320, 0, 5.202865, 0.515365, 0, 5.262581, 0.448796, 0, 5.202865, 0.515365, 0, 5.262581, 0.448796, 0, 5.284114, 0.472917, 0, 5.262581, 0.448796, 0, 5.202865, 0.515365, 0, 5.284114, 0.472917, 0, 5.244905, 0.552393, 0, 5.202865, 0.515365, 0, 5.284114, 0.472917, 0, 5.202865, 0.515365, 0, 5.244905, 0.552393, 0, 5.284114, 0.472917, 0, 5.244905, 0.552393, 0, 5.284114, 0.472917, 0, 5.310205, 0.492057, 0, 5.284114, 0.472917, 0, 5.244905, 0.552393, 0, 5.310205, 0.492057, 0, 5.293685, 0.578841, 0, 5.244905, 0.552393, 0, 5.310205, 0.492057, 0, 5.244905, 0.552393, 0, 5.293685, 0.578841, 0, 5.310205, 0.492057, 0, 5.293685, 0.578841, 0, 5.310205, 0.492057, 0, 5.340300, 0.505729, 0, 5.310205, 0.492057, 0, 5.293685, 0.578841, 0, 5.340300, 0.505729, 0, 5.293685, 0.578841, 0, 5.340300, 0.505729, 0, 5.349203, 0.594710, 0, 5.340300, 0.505729, 0, 5.293685, 0.578841, 0, 5.349203, 0.594710, 0, 5.349203, 0.594710, 0, 5.340300, 0.505729, 0, 5.374398, 0.513932, 0, 5.340300, 0.505729, 0, 5.349203, 0.594710, 0, 5.374398, 0.513932, 0, 5.349203, 0.594710, 0, 5.374398, 0.513932, 0, 5.411458, 0.600000, 0, 5.374398, 0.513932, 0, 5.349203, 0.594710, 0, 5.411458, 0.600000, 0, 5.411458, 0.600000, 0, 5.374398, 0.513932, 0, 5.412500, 0.516667, 0, 5.374398, 0.513932, 0, 5.411458, 0.600000, 0, 5.412500, 0.516667, 0, 5.411458, 0.600000, 0, 5.412500, 0.516667, 0, 5.446175, 0.513835, 0, 5.412500, 0.516667, 0, 5.411458, 0.600000, 0, 5.446175, 0.513835, 0, 5.411458, 0.600000, 0, 5.446175, 0.513835, 0, 5.467237, 0.595272, 0, 5.446175, 0.513835, 0, 5.411458, 0.600000, 0, 5.467237, 0.595272, 0, 5.467237, 0.595272, 0, 5.446175, 0.513835, 0, 5.476367, 0.505339, 0, 5.446175, 0.513835, 0, 5.467237, 0.595272, 0, 5.476367, 0.505339, 0, 5.467237, 0.595272, 0, 5.476367, 0.505339, 0, 5.516862, 0.581087, 0, 5.476367, 0.505339, 0, 5.467237, 0.595272, 0, 5.516862, 0.581087, 0, 5.476367, 0.505339, 0, 5.503076, 0.491178, 0, 5.516862, 0.581087, 0, 5.503076, 0.491178, 0, 5.476367, 0.505339, 0, 5.516862, 0.581087, 0, 5.516862, 0.581087, 0, 5.503076, 0.491178, 0, 5.560335, 0.557446, 0, 5.503076, 0.491178, 0, 5.516862, 0.581087, 0, 5.560335, 0.557446, 0, 5.503076, 0.491178, 0, 5.526302, 0.471354, 0, 5.560335, 0.557446, 0, 5.526302, 0.471354, 0, 5.503076, 0.491178, 0, 5.560335, 0.557446, 0, 5.560335, 0.557446, 0, 5.526302, 0.471354, 0, 5.597656, 0.524349, 0, 5.526302, 0.471354, 0, 5.560335, 0.557446, 0, 5.597656, 0.524349, 0, 5.526302, 0.471354, 0, 5.545166, 0.446777, 0, 5.597656, 0.524349, 0, 5.545166, 0.446777, 0, 5.526302, 0.471354, 0, 5.597656, 0.524349, 0, 5.597656, 0.524349, 0, 5.545166, 0.446777, 0, 5.627620, 0.483130, 0, 5.545166, 0.446777, 0, 5.597656, 0.524349, 0, 5.627620, 0.483130, 0, 5.545166, 0.446777, 0, 5.558789, 0.418359, 0, 5.627620, 0.483130, 0, 5.558789, 0.418359, 0, 5.545166, 0.446777, 0, 5.627620, 0.483130, 0, 5.627620, 0.483130, 0, 5.558789, 0.418359, 0, 5.649024, 0.435124, 0, 5.558789, 0.418359, 0, 5.627620, 0.483130, 0, 5.649024, 0.435124, 0, 5.558789, 0.418359, 0, 5.567171, 0.386100, 0, 5.649024, 0.435124, 0, 5.567171, 0.386100, 0, 5.558789, 0.418359, 0, 5.649024, 0.435124, 0, 5.649024, 0.435124, 0, 5.567171, 0.386100, 0, 5.661865, 0.380330, 0, 5.567171, 0.386100, 0, 5.649024, 0.435124, 0, 5.661865, 0.380330, 0, 5.567171, 0.386100, 0, 5.570313, 0.350000, 0, 5.661865, 0.380330, 0, 5.570313, 0.350000, 0, 5.567171, 0.386100, 0, 5.661865, 0.380330, 0, 5.661865, 0.380330, 0, 5.570313, 0.350000, 0, 5.666146, 0.318750, 0, 5.570313, 0.350000, 0, 5.661865, 0.380330, 0, 5.666146, 0.318750, 0, 5.666146, 0.266667, 0, 5.666146, 0.318750, 0, 5.570313, 0.350000, 0, 5.666146, 0.318750, 0, 5.666146, 0.266667, 0, 5.570313, 0.350000, 0, 5.570313, 0.350000, 0, 5.225521, 0.266667, 0, 5.666146, 0.266667, 0, 5.225521, 0.266667, 0, 5.570313, 0.350000, 0, 5.666146, 0.266667, 0, 5.570313, 0.350000, 0, 5.228646, 0.349479, 0, 5.225521, 0.266667, 0, 5.228646, 0.349479, 0, 5.570313, 0.350000, 0, 5.225521, 0.266667, 0, 5.125521, 0.286458, 0, 5.225521, 0.266667, 0, 5.228646, 0.349479, 0, 5.225521, 0.266667, 0, 5.125521, 0.286458, 0, 5.228646, 0.349479, 0, 5.125521, 0.286458, 0, 5.130615, 0.220280, 0, 5.225521, 0.266667, 0, 5.130615, 0.220280, 0, 5.125521, 0.286458, 0, 5.225521, 0.266667, 0, 5.225521, 0.266667, 0, 5.130615, 0.220280, 0, 5.231592, 0.220744, 0, 5.130615, 0.220280, 0, 5.225521, 0.266667, 0, 5.231592, 0.220744, 0, 5.130615, 0.220280, 0, 5.145898, 0.161328, 0, 5.231592, 0.220744, 0, 5.145898, 0.161328, 0, 5.130615, 0.220280, 0, 5.231592, 0.220744, 0, 5.231592, 0.220744, 0, 5.145898, 0.161328, 0, 5.243555, 0.180632, 0, 5.145898, 0.161328, 0, 5.231592, 0.220744, 0, 5.243555, 0.180632, 0, 5.145898, 0.161328, 0, 5.171371, 0.109603, 0, 5.243555, 0.180632, 0, 5.171371, 0.109603, 0, 5.145898, 0.161328, 0, 5.243555, 0.180632, 0, 5.243555, 0.180632, 0, 5.171371, 0.109603, 0, 5.261409, 0.146330, 0, 5.171371, 0.109603, 0, 5.243555, 0.180632, 0, 5.261409, 0.146330, 0, 5.171371, 0.109603, 0, 5.207031, 0.065104, 0, 5.261409, 0.146330, 0, 5.207031, 0.065104, 0, 5.171371, 0.109603, 0, 5.261409, 0.146330, 0, 5.261409, 0.146330, 0, 5.207031, 0.065104, 0, 5.285156, 0.117839, 0, 5.207031, 0.065104, 0, 5.261409, 0.146330, 0, 5.285156, 0.117839, 0, 5.207031, 0.065104, 0, 5.251416, 0.029329, 0, 5.285156, 0.117839, 0, 5.251416, 0.029329, 0, 5.207031, 0.065104, 0, 5.285156, 0.117839, 0, 5.285156, 0.117839, 0, 5.251416, 0.029329, 0, 5.314470, 0.095451, 0, 5.251416, 0.029329, 0, 5.285156, 0.117839, 0, 5.314470, 0.095451, 0, 5.314470, 0.095451, 0, 5.251416, 0.029329, 0, 5.303060, 0.003776, 0, 5.251416, 0.029329, 0, 5.314470, 0.095451, 0, 5.303060, 0.003776, 0, 5.349023, 0.079460, 0, 5.314470, 0.095451, 0, 5.303060, 0.003776, 0, 5.314470, 0.095451, 0, 5.349023, 0.079460, 0, 5.303060, 0.003776, 0, 5.349023, 0.079460, 0, 5.303060, 0.003776, 0, 5.361963, -0.011556, 0, 5.303060, 0.003776, 0, 5.349023, 0.079460, 0, 5.361963, -0.011556, 0, 5.388818, 0.069865, 0, 5.349023, 0.079460, 0, 5.361963, -0.011556, 0, 5.349023, 0.079460, 0, 5.388818, 0.069865, 0, 5.361963, -0.011556, 0, 5.388818, 0.069865, 0, 5.361963, -0.011556, 0, 5.428125, -0.016667, 0, 5.361963, -0.011556, 0, 5.388818, 0.069865, 0, 5.428125, -0.016667, 0, 5.433855, 0.066667, 0, 5.388818, 0.069865, 0, 5.428125, -0.016667, 0, 5.388818, 0.069865, 0, 5.433855, 0.066667, 0, 5.428125, -0.016667, 0, 5.433855, 0.066667, 0, 5.428125, -0.016667, 0, 5.455892, -0.015869, 0, 5.428125, -0.016667, 0, 5.433855, 0.066667, 0, 5.455892, -0.015869, 0, 5.433855, 0.066667, 0, 5.455892, -0.015869, 0, 5.461247, 0.067448, 0, 5.455892, -0.015869, 0, 5.433855, 0.066667, 0, 5.461247, 0.067448, 0, 5.461247, 0.067448, 0, 5.455892, -0.015869, 0, 5.483464, -0.013477, 0, 5.455892, -0.015869, 0, 5.461247, 0.067448, 0, 5.483464, -0.013477, 0, 5.461247, 0.067448, 0, 5.483464, -0.013477, 0, 5.488216, 0.069792, 0, 5.483464, -0.013477, 0, 5.461247, 0.067448, 0, 5.488216, 0.069792, 0, 5.488216, 0.069792, 0, 5.483464, -0.013477, 0, 5.510840, -0.009489, 0, 5.483464, -0.013477, 0, 5.488216, 0.069792, 0, 5.510840, -0.009489, 0, 5.488216, 0.069792, 0, 5.510840, -0.009489, 0, 5.514762, 0.073698, 0, 5.510840, -0.009489, 0, 5.488216, 0.069792, 0, 5.514762, 0.073698, 0, 5.514762, 0.073698, 0, 5.510840, -0.009489, 0, 5.538021, -0.003906, 0, 5.510840, -0.009489, 0, 5.514762, 0.073698, 0, 5.538021, -0.003906, 0, 5.514762, 0.073698, 0, 5.538021, -0.003906, 0, 5.540885, 0.079167, 0, 5.538021, -0.003906, 0, 5.514762, 0.073698, 0, 5.540885, 0.079167, 0, 5.540885, 0.079167, 0, 5.538021, -0.003906, 0, 5.564941, 0.003206, 0, 5.538021, -0.003906, 0, 5.540885, 0.079167, 0, 5.564941, 0.003206, 0, 5.566748, 0.086198, 0, 5.540885, 0.079167, 0, 5.564941, 0.003206, 0, 5.540885, 0.079167, 0, 5.566748, 0.086198, 0, 5.564941, 0.003206, 0, 5.591537, 0.011784, 0, 5.566748, 0.086198, 0, 5.564941, 0.003206, 0, 5.566748, 0.086198, 0, 5.591537, 0.011784, 0, 5.564941, 0.003206, 0, 5.566748, 0.086198, 0, 5.591537, 0.011784, 0, 5.617806, 0.021826, 0, 5.591537, 0.011784, 0, 5.566748, 0.086198, 0, 5.617806, 0.021826, 0, 5.592513, 0.094792, 0, 5.566748, 0.086198, 0, 5.617806, 0.021826, 0, 5.566748, 0.086198, 0, 5.592513, 0.094792, 0, 5.617806, 0.021826, 0, 5.592513, 0.094792, 0, 5.617806, 0.021826, 0, 5.643750, 0.033333, 0, 5.617806, 0.021826, 0, 5.592513, 0.094792, 0, 5.643750, 0.033333, 0, 5.618181, 0.104948, 0, 5.592513, 0.094792, 0, 5.643750, 0.033333, 0, 5.592513, 0.094792, 0, 5.618181, 0.104948, 0, 5.643750, 0.033333, 0, 5.618181, 0.104948, 0, 5.643750, 0.033333, 0, 5.643750, 0.116667, 0, 5.643750, 0.033333, 0, 5.618181, 0.104948, 0, 5.643750, 0.116667, 0, 5.570313, 0.350000, 0, 5.567171, 0.386100, 0, 5.570313, 0.350000, 0, 5.570313, 0.350000, 0, 5.567171, 0.386100, 0, 5.567171, 0.386100, 0, 5.567171, 0.386100, 0, 5.558789, 0.418359, 0, 5.567171, 0.386100, 0, 5.567171, 0.386100, 0, 5.558789, 0.418359, 0, 5.558789, 0.418359, 0, 5.558789, 0.418359, 0, 5.545166, 0.446777, 0, 5.558789, 0.418359, 0, 5.558789, 0.418359, 0, 5.545166, 0.446777, 0, 5.545166, 0.446777, 0, 5.545166, 0.446777, 0, 5.526302, 0.471354, 0, 5.545166, 0.446777, 0, 5.545166, 0.446777, 0, 5.526302, 0.471354, 0, 5.526302, 0.471354, 0, 5.526302, 0.471354, 0, 5.503076, 0.491178, 0, 5.526302, 0.471354, 0, 5.526302, 0.471354, 0, 5.503076, 0.491178, 0, 5.503076, 0.491178, 0, 5.503076, 0.491178, 0, 5.476367, 0.505339, 0, 5.503076, 0.491178, 0, 5.503076, 0.491178, 0, 5.476367, 0.505339, 0, 5.476367, 0.505339, 0, 5.476367, 0.505339, 0, 5.446175, 0.513835, 0, 5.476367, 0.505339, 0, 5.476367, 0.505339, 0, 5.446175, 0.513835, 0, 5.446175, 0.513835, 0, 5.446175, 0.513835, 0, 5.412500, 0.516667, 0, 5.446175, 0.513835, 0, 5.446175, 0.513835, 0, 5.412500, 0.516667, 0, 5.412500, 0.516667, 0, 5.412500, 0.516667, 0, 5.374398, 0.513932, 0, 5.412500, 0.516667, 0, 5.412500, 0.516667, 0, 5.374398, 0.513932, 0, 5.374398, 0.513932, 0, 5.374398, 0.513932, 0, 5.340300, 0.505729, 0, 5.374398, 0.513932, 0, 5.374398, 0.513932, 0, 5.340300, 0.505729, 0, 5.340300, 0.505729, 0, 5.340300, 0.505729, 0, 5.310205, 0.492057, 0, 5.340300, 0.505729, 0, 5.340300, 0.505729, 0, 5.310205, 0.492057, 0, 5.310205, 0.492057, 0, 5.310205, 0.492057, 0, 5.284114, 0.472917, 0, 5.310205, 0.492057, 0, 5.310205, 0.492057, 0, 5.284114, 0.472917, 0, 5.284114, 0.472917, 0, 5.284114, 0.472917, 0, 5.262581, 0.448796, 0, 5.284114, 0.472917, 0, 5.284114, 0.472917, 0, 5.262581, 0.448796, 0, 5.262581, 0.448796, 0, 5.262581, 0.448796, 0, 5.246159, 0.420182, 0, 5.262581, 0.448796, 0, 5.262581, 0.448796, 0, 5.246159, 0.420182, 0, 5.246159, 0.420182, 0, 5.246159, 0.420182, 0, 5.234847, 0.387077, 0, 5.246159, 0.420182, 0, 5.246159, 0.420182, 0, 5.234847, 0.387077, 0, 5.234847, 0.387077, 0, 5.234847, 0.387077, 0, 5.228646, 0.349479, 0, 5.234847, 0.387077, 0, 5.234847, 0.387077, 0, 5.228646, 0.349479, 0, 5.228646, 0.349479, 0, 6.150521, 0.583333, 0, 6.246355, 0.583333, 0, 6.150521, 0.583333, 0, 6.150521, 0.583333, 0, 6.246355, 0.583333, 0, 6.246355, 0.583333, 0, 6.246355, 0.583333, 0, 6.246355, 0, 0, 6.246355, 0.583333, 0, 6.246355, 0.583333, 0, 6.246355, 0, 0, 6.246355, 0, 0, 6.246355, 0, 0, 6.150521, 0, 0, 6.246355, 0, 0, 6.246355, 0, 0, 6.150521, 0, 0, 6.150521, 0, 0, 6.150521, 0.583333, 0, 6.246355, 0, 0, 6.246355, 0.583333, 0, 6.246355, 0, 0, 6.150521, 0.583333, 0, 6.246355, 0.583333, 0, 6.150521, 0.583333, 0, 6.150521, 0, 0, 6.246355, 0, 0, 6.150521, 0, 0, 6.150521, 0.583333, 0, 6.246355, 0, 0, 6.150521, 0.816667, 0, 6.246355, 0.816667, 0, 6.150521, 0.816667, 0, 6.150521, 0.816667, 0, 6.246355, 0.816667, 0, 6.246355, 0.816667, 0, 6.246355, 0.816667, 0, 6.246355, 0.700000, 0, 6.246355, 0.816667, 0, 6.246355, 0.816667, 0, 6.246355, 0.700000, 0, 6.246355, 0.700000, 0, 6.246355, 0.700000, 0, 6.150521, 0.700000, 0, 6.246355, 0.700000, 0, 6.246355, 0.700000, 0, 6.150521, 0.700000, 0, 6.150521, 0.700000, 0, 6.150521, 0.816667, 0, 6.246355, 0.700000, 0, 6.246355, 0.816667, 0, 6.246355, 0.700000, 0, 6.150521, 0.816667, 0, 6.246355, 0.816667, 0, 6.150521, 0.816667, 0, 6.150521, 0.700000, 0, 6.246355, 0.700000, 0, 6.150521, 0.700000, 0, 6.150521, 0.816667, 0, 6.246355, 0.700000, 0, 6.822396, 0.550000, 0, 6.822396, 0.466667, 0, 6.822396, 0.550000, 0, 6.822396, 0.550000, 0, 6.822396, 0.466667, 0, 6.822396, 0.466667, 0, 6.822396, 0.466667, 0, 6.801888, 0.478385, 0, 6.822396, 0.466667, 0, 6.822396, 0.466667, 0, 6.801888, 0.478385, 0, 6.801888, 0.478385, 0, 6.801888, 0.478385, 0, 6.780990, 0.488542, 0, 6.801888, 0.478385, 0, 6.801888, 0.478385, 0, 6.780990, 0.488542, 0, 6.780990, 0.488542, 0, 6.780990, 0.488542, 0, 6.759701, 0.497135, 0, 6.780990, 0.488542, 0, 6.780990, 0.488542, 0, 6.759701, 0.497135, 0, 6.759701, 0.497135, 0, 6.759701, 0.497135, 0, 6.738021, 0.504167, 0, 6.759701, 0.497135, 0, 6.759701, 0.497135, 0, 6.738021, 0.504167, 0, 6.738021, 0.504167, 0, 6.738021, 0.504167, 0, 6.715951, 0.509635, 0, 6.738021, 0.504167, 0, 6.738021, 0.504167, 0, 6.715951, 0.509635, 0, 6.715951, 0.509635, 0, 6.715951, 0.509635, 0, 6.693490, 0.513542, 0, 6.715951, 0.509635, 0, 6.715951, 0.509635, 0, 6.693490, 0.513542, 0, 6.693490, 0.513542, 0, 6.693490, 0.513542, 0, 6.670639, 0.515885, 0, 6.693490, 0.513542, 0, 6.693490, 0.513542, 0, 6.670639, 0.515885, 0, 6.670639, 0.515885, 0, 6.670639, 0.515885, 0, 6.647396, 0.516667, 0, 6.670639, 0.515885, 0, 6.670639, 0.515885, 0, 6.647396, 0.516667, 0, 6.647396, 0.516667, 0, 6.647396, 0.516667, 0, 6.613949, 0.515251, 0, 6.647396, 0.516667, 0, 6.647396, 0.516667, 0, 6.613949, 0.515251, 0, 6.613949, 0.515251, 0, 6.613949, 0.515251, 0, 6.584961, 0.511003, 0, 6.613949, 0.515251, 0, 6.613949, 0.515251, 0, 6.584961, 0.511003, 0, 6.584961, 0.511003, 0, 6.584961, 0.511003, 0, 6.560433, 0.503923, 0, 6.584961, 0.511003, 0, 6.584961, 0.511003, 0, 6.560433, 0.503923, 0, 6.560433, 0.503923, 0, 6.560433, 0.503923, 0, 6.540365, 0.494010, 0, 6.560433, 0.503923, 0, 6.560433, 0.503923, 0, 6.540365, 0.494010, 0, 6.540365, 0.494010, 0, 6.540365, 0.494010, 0, 6.524756, 0.481266, 0, 6.540365, 0.494010, 0, 6.540365, 0.494010, 0, 6.524756, 0.481266, 0, 6.524756, 0.481266, 0, 6.524756, 0.481266, 0, 6.513607, 0.465690, 0, 6.524756, 0.481266, 0, 6.524756, 0.481266, 0, 6.513607, 0.465690, 0, 6.513607, 0.465690, 0, 6.513607, 0.465690, 0, 6.506917, 0.447282, 0, 6.513607, 0.465690, 0, 6.513607, 0.465690, 0, 6.506917, 0.447282, 0, 6.506917, 0.447282, 0, 6.506917, 0.447282, 0, 6.504688, 0.426042, 0, 6.506917, 0.447282, 0, 6.506917, 0.447282, 0, 6.504688, 0.426042, 0, 6.504688, 0.426042, 0, 6.504688, 0.426042, 0, 6.506283, 0.409766, 0, 6.504688, 0.426042, 0, 6.504688, 0.426042, 0, 6.506283, 0.409766, 0, 6.506283, 0.409766, 0, 6.506283, 0.409766, 0, 6.511068, 0.395313, 0, 6.506283, 0.409766, 0, 6.506283, 0.409766, 0, 6.511068, 0.395313, 0, 6.511068, 0.395313, 0, 6.511068, 0.395313, 0, 6.519043, 0.382682, 0, 6.511068, 0.395313, 0, 6.511068, 0.395313, 0, 6.519043, 0.382682, 0, 6.519043, 0.382682, 0, 6.519043, 0.382682, 0, 6.530209, 0.371875, 0, 6.519043, 0.382682, 0, 6.519043, 0.382682, 0, 6.530209, 0.371875, 0, 6.530209, 0.371875, 0, 6.530209, 0.371875, 0, 6.546192, 0.362109, 0, 6.530209, 0.371875, 0, 6.530209, 0.371875, 0, 6.546192, 0.362109, 0, 6.546192, 0.362109, 0, 6.546192, 0.362109, 0, 6.568620, 0.352604, 0, 6.546192, 0.362109, 0, 6.546192, 0.362109, 0, 6.568620, 0.352604, 0, 6.568620, 0.352604, 0, 6.568620, 0.352604, 0, 6.597494, 0.343359, 0, 6.568620, 0.352604, 0, 6.568620, 0.352604, 0, 6.597494, 0.343359, 0, 6.597494, 0.343359, 0, 6.597494, 0.343359, 0, 6.632813, 0.334375, 0, 6.597494, 0.343359, 0, 6.597494, 0.343359, 0, 6.632813, 0.334375, 0, 6.632813, 0.334375, 0, 6.632813, 0.334375, 0, 6.665625, 0.327083, 0, 6.632813, 0.334375, 0, 6.632813, 0.334375, 0, 6.665625, 0.327083, 0, 6.665625, 0.327083, 0, 6.665625, 0.327083, 0, 6.712972, 0.315177, 0, 6.665625, 0.327083, 0, 6.665625, 0.327083, 0, 6.712972, 0.315177, 0, 6.712972, 0.315177, 0, 6.712972, 0.315177, 0, 6.752930, 0.301074, 0, 6.712972, 0.315177, 0, 6.712972, 0.315177, 0, 6.752930, 0.301074, 0, 6.752930, 0.301074, 0, 6.752930, 0.301074, 0, 6.785498, 0.284774, 0, 6.752930, 0.301074, 0, 6.752930, 0.301074, 0, 6.785498, 0.284774, 0, 6.785498, 0.284774, 0, 6.785498, 0.284774, 0, 6.810678, 0.266276, 0, 6.785498, 0.284774, 0, 6.785498, 0.284774, 0, 6.810678, 0.266276, 0, 6.810678, 0.266276, 0, 6.810678, 0.266276, 0, 6.829476, 0.244735, 0, 6.810678, 0.266276, 0, 6.810678, 0.266276, 0, 6.829476, 0.244735, 0, 6.829476, 0.244735, 0, 6.829476, 0.244735, 0, 6.842904, 0.219303, 0, 6.829476, 0.244735, 0, 6.829476, 0.244735, 0, 6.842904, 0.219303, 0, 6.842904, 0.219303, 0, 6.842904, 0.219303, 0, 6.850961, 0.189982, 0, 6.842904, 0.219303, 0, 6.842904, 0.219303, 0, 6.850961, 0.189982, 0, 6.850961, 0.189982, 0, 6.850961, 0.189982, 0, 6.853646, 0.156771, 0, 6.850961, 0.189982, 0, 6.850961, 0.189982, 0, 6.853646, 0.156771, 0, 6.853646, 0.156771, 0, 6.853646, 0.156771, 0, 6.849626, 0.118872, 0, 6.853646, 0.156771, 0, 6.853646, 0.156771, 0, 6.849626, 0.118872, 0, 6.849626, 0.118872, 0, 6.849626, 0.118872, 0, 6.837565, 0.085124, 0, 6.849626, 0.118872, 0, 6.849626, 0.118872, 0, 6.837565, 0.085124, 0, 6.837565, 0.085124, 0, 6.837565, 0.085124, 0, 6.817464, 0.055526, 0, 6.837565, 0.085124, 0, 6.837565, 0.085124, 0, 6.817464, 0.055526, 0, 6.817464, 0.055526, 0, 6.817464, 0.055526, 0, 6.789323, 0.030078, 0, 6.817464, 0.055526, 0, 6.817464, 0.055526, 0, 6.789323, 0.030078, 0, 6.789323, 0.030078, 0, 6.789323, 0.030078, 0, 6.754150, 0.009627, 0, 6.789323, 0.030078, 0, 6.789323, 0.030078, 0, 6.754150, 0.009627, 0, 6.754150, 0.009627, 0, 6.754150, 0.009627, 0, 6.712955, -0.004980, 0, 6.754150, 0.009627, 0, 6.754150, 0.009627, 0, 6.712955, -0.004980, 0, 6.712955, -0.004980, 0, 6.712955, -0.004980, 0, 6.665739, -0.013745, 0, 6.712955, -0.004980, 0, 6.712955, -0.004980, 0, 6.665739, -0.013745, 0, 6.665739, -0.013745, 0, 6.665739, -0.013745, 0, 6.612500, -0.016667, 0, 6.665739, -0.013745, 0, 6.665739, -0.013745, 0, 6.612500, -0.016667, 0, 6.612500, -0.016667, 0, 6.612500, -0.016667, 0, 6.588819, -0.016146, 0, 6.612500, -0.016667, 0, 6.612500, -0.016667, 0, 6.588819, -0.016146, 0, 6.588819, -0.016146, 0, 6.588819, -0.016146, 0, 6.564649, -0.014583, 0, 6.588819, -0.016146, 0, 6.588819, -0.016146, 0, 6.564649, -0.014583, 0, 6.564649, -0.014583, 0, 6.564649, -0.014583, 0, 6.539990, -0.011979, 0, 6.564649, -0.014583, 0, 6.564649, -0.014583, 0, 6.539990, -0.011979, 0, 6.539990, -0.011979, 0, 6.539990, -0.011979, 0, 6.514844, -0.008333, 0, 6.539990, -0.011979, 0, 6.539990, -0.011979, 0, 6.514844, -0.008333, 0, 6.514844, -0.008333, 0, 6.514844, -0.008333, 0, 6.489111, -0.003646, 0, 6.514844, -0.008333, 0, 6.514844, -0.008333, 0, 6.489111, -0.003646, 0, 6.489111, -0.003646, 0, 6.489111, -0.003646, 0, 6.462696, 0.002083, 0, 6.489111, -0.003646, 0, 6.489111, -0.003646, 0, 6.462696, 0.002083, 0, 6.462696, 0.002083, 0, 6.462696, 0.002083, 0, 6.435596, 0.008854, 0, 6.462696, 0.002083, 0, 6.462696, 0.002083, 0, 6.435596, 0.008854, 0, 6.435596, 0.008854, 0, 6.435596, 0.008854, 0, 6.407813, 0.016667, 0, 6.435596, 0.008854, 0, 6.435596, 0.008854, 0, 6.407813, 0.016667, 0, 6.407813, 0.016667, 0, 6.407813, 0.016667, 0, 6.407813, 0.116667, 0, 6.407813, 0.016667, 0, 6.407813, 0.016667, 0, 6.407813, 0.116667, 0, 6.407813, 0.116667, 0, 6.407813, 0.116667, 0, 6.434278, 0.104948, 0, 6.407813, 0.116667, 0, 6.407813, 0.116667, 0, 6.434278, 0.104948, 0, 6.434278, 0.104948, 0, 6.434278, 0.104948, 0, 6.460547, 0.094792, 0, 6.434278, 0.104948, 0, 6.434278, 0.104948, 0, 6.460547, 0.094792, 0, 6.460547, 0.094792, 0, 6.460547, 0.094792, 0, 6.486621, 0.086198, 0, 6.460547, 0.094792, 0, 6.460547, 0.094792, 0, 6.486621, 0.086198, 0, 6.486621, 0.086198, 0, 6.486621, 0.086198, 0, 6.512500, 0.079167, 0, 6.486621, 0.086198, 0, 6.486621, 0.086198, 0, 6.512500, 0.079167, 0, 6.512500, 0.079167, 0, 6.512500, 0.079167, 0, 6.538217, 0.073698, 0, 6.512500, 0.079167, 0, 6.512500, 0.079167, 0, 6.538217, 0.073698, 0, 6.538217, 0.073698, 0, 6.538217, 0.073698, 0, 6.563802, 0.069792, 0, 6.538217, 0.073698, 0, 6.538217, 0.073698, 0, 6.563802, 0.069792, 0, 6.563802, 0.069792, 0, 6.563802, 0.069792, 0, 6.589258, 0.067448, 0, 6.563802, 0.069792, 0, 6.563802, 0.069792, 0, 6.589258, 0.067448, 0, 6.589258, 0.067448, 0, 6.589258, 0.067448, 0, 6.614583, 0.066667, 0, 6.589258, 0.067448, 0, 6.589258, 0.067448, 0, 6.614583, 0.066667, 0, 6.614583, 0.066667, 0, 6.614583, 0.066667, 0, 6.646484, 0.068058, 0, 6.614583, 0.066667, 0, 6.614583, 0.066667, 0, 6.646484, 0.068058, 0, 6.646484, 0.068058, 0, 6.646484, 0.068058, 0, 6.674479, 0.072233, 0, 6.646484, 0.068058, 0, 6.646484, 0.068058, 0, 6.674479, 0.072233, 0, 6.674479, 0.072233, 0, 6.674479, 0.072233, 0, 6.698568, 0.079191, 0, 6.674479, 0.072233, 0, 6.674479, 0.072233, 0, 6.698568, 0.079191, 0, 6.698568, 0.079191, 0, 6.698568, 0.079191, 0, 6.718750, 0.088932, 0, 6.698568, 0.079191, 0, 6.698568, 0.079191, 0, 6.718750, 0.088932, 0, 6.718750, 0.088932, 0, 6.718750, 0.088932, 0, 6.734701, 0.101213, 0, 6.718750, 0.088932, 0, 6.718750, 0.088932, 0, 6.734701, 0.101213, 0, 6.734701, 0.101213, 0, 6.734701, 0.101213, 0, 6.746094, 0.115788, 0, 6.734701, 0.101213, 0, 6.734701, 0.101213, 0, 6.746094, 0.115788, 0, 6.746094, 0.115788, 0, 6.746094, 0.115788, 0, 6.752930, 0.132658, 0, 6.746094, 0.115788, 0, 6.746094, 0.115788, 0, 6.752930, 0.132658, 0, 6.752930, 0.132658, 0, 6.752930, 0.132658, 0, 6.755208, 0.151823, 0, 6.752930, 0.132658, 0, 6.752930, 0.132658, 0, 6.755208, 0.151823, 0, 6.755208, 0.151823, 0, 6.755208, 0.151823, 0, 6.753564, 0.169482, 0, 6.755208, 0.151823, 0, 6.755208, 0.151823, 0, 6.753564, 0.169482, 0, 6.753564, 0.169482, 0, 6.753564, 0.169482, 0, 6.748633, 0.184961, 0, 6.753564, 0.169482, 0, 6.753564, 0.169482, 0, 6.748633, 0.184961, 0, 6.748633, 0.184961, 0, 6.748633, 0.184961, 0, 6.740414, 0.198258, 0, 6.748633, 0.184961, 0, 6.748633, 0.184961, 0, 6.740414, 0.198258, 0, 6.740414, 0.198258, 0, 6.740414, 0.198258, 0, 6.728906, 0.209375, 0, 6.740414, 0.198258, 0, 6.740414, 0.198258, 0, 6.728906, 0.209375, 0, 6.728906, 0.209375, 0, 6.728906, 0.209375, 0, 6.711833, 0.219303, 0, 6.728906, 0.209375, 0, 6.728906, 0.209375, 0, 6.711833, 0.219303, 0, 6.711833, 0.219303, 0, 6.711833, 0.219303, 0, 6.686914, 0.229036, 0, 6.711833, 0.219303, 0, 6.711833, 0.219303, 0, 6.686914, 0.229036, 0, 6.686914, 0.229036, 0, 6.686914, 0.229036, 0, 6.654150, 0.238574, 0, 6.686914, 0.229036, 0, 6.686914, 0.229036, 0, 6.654150, 0.238574, 0, 6.654150, 0.238574, 0, 6.654150, 0.238574, 0, 6.613542, 0.247917, 0, 6.654150, 0.238574, 0, 6.654150, 0.238574, 0, 6.613542, 0.247917, 0, 6.613542, 0.247917, 0, 6.613542, 0.247917, 0, 6.580209, 0.255729, 0, 6.613542, 0.247917, 0, 6.613542, 0.247917, 0, 6.580209, 0.255729, 0, 6.580209, 0.255729, 0, 6.580209, 0.255729, 0, 6.538770, 0.266642, 0, 6.580209, 0.255729, 0, 6.580209, 0.255729, 0, 6.538770, 0.266642, 0, 6.538770, 0.266642, 0, 6.538770, 0.266642, 0, 6.503516, 0.280111, 0, 6.538770, 0.266642, 0, 6.538770, 0.266642, 0, 6.503516, 0.280111, 0, 6.503516, 0.280111, 0, 6.503516, 0.280111, 0, 6.474447, 0.296134, 0, 6.503516, 0.280111, 0, 6.503516, 0.280111, 0, 6.474447, 0.296134, 0, 6.474447, 0.296134, 0, 6.474447, 0.296134, 0, 6.451562, 0.314714, 0, 6.474447, 0.296134, 0, 6.474447, 0.296134, 0, 6.451562, 0.314714, 0, 6.451562, 0.314714, 0, 6.451562, 0.314714, 0, 6.434245, 0.336434, 0, 6.451562, 0.314714, 0, 6.451562, 0.314714, 0, 6.434245, 0.336434, 0, 6.434245, 0.336434, 0, 6.434245, 0.336434, 0, 6.421875, 0.361882, 0, 6.434245, 0.336434, 0, 6.434245, 0.336434, 0, 6.421875, 0.361882, 0, 6.421875, 0.361882, 0, 6.421875, 0.361882, 0, 6.414453, 0.391056, 0, 6.421875, 0.361882, 0, 6.421875, 0.361882, 0, 6.414453, 0.391056, 0, 6.414453, 0.391056, 0, 6.414453, 0.391056, 0, 6.411979, 0.423958, 0, 6.414453, 0.391056, 0, 6.414453, 0.391056, 0, 6.411979, 0.423958, 0, 6.411979, 0.423958, 0, 6.411979, 0.423958, 0, 6.415625, 0.463631, 0, 6.411979, 0.423958, 0, 6.411979, 0.423958, 0, 6.415625, 0.463631, 0, 6.415625, 0.463631, 0, 6.415625, 0.463631, 0, 6.426563, 0.498535, 0, 6.415625, 0.463631, 0, 6.415625, 0.463631, 0, 6.426563, 0.498535, 0, 6.426563, 0.498535, 0, 6.426563, 0.498535, 0, 6.444791, 0.528670, 0, 6.426563, 0.498535, 0, 6.426563, 0.498535, 0, 6.444791, 0.528670, 0, 6.444791, 0.528670, 0, 6.444791, 0.528670, 0, 6.470313, 0.554036, 0, 6.444791, 0.528670, 0, 6.444791, 0.528670, 0, 6.470313, 0.554036, 0, 6.470313, 0.554036, 0, 6.470313, 0.554036, 0, 6.502539, 0.574145, 0, 6.470313, 0.554036, 0, 6.470313, 0.554036, 0, 6.502539, 0.574145, 0, 6.502539, 0.574145, 0, 6.502539, 0.574145, 0, 6.540885, 0.588509, 0, 6.502539, 0.574145, 0, 6.502539, 0.574145, 0, 6.540885, 0.588509, 0, 6.540885, 0.588509, 0, 6.540885, 0.588509, 0, 6.585351, 0.597127, 0, 6.540885, 0.588509, 0, 6.540885, 0.588509, 0, 6.585351, 0.597127, 0, 6.585351, 0.597127, 0, 6.585351, 0.597127, 0, 6.635938, 0.600000, 0, 6.585351, 0.597127, 0, 6.585351, 0.597127, 0, 6.635938, 0.600000, 0, 6.635938, 0.600000, 0, 6.635938, 0.600000, 0, 6.662109, 0.599219, 0, 6.635938, 0.600000, 0, 6.635938, 0.600000, 0, 6.662109, 0.599219, 0, 6.662109, 0.599219, 0, 6.662109, 0.599219, 0, 6.687500, 0.596875, 0, 6.662109, 0.599219, 0, 6.662109, 0.599219, 0, 6.687500, 0.596875, 0, 6.687500, 0.596875, 0, 6.687500, 0.596875, 0, 6.712110, 0.592969, 0, 6.687500, 0.596875, 0, 6.687500, 0.596875, 0, 6.712110, 0.592969, 0, 6.712110, 0.592969, 0, 6.712110, 0.592969, 0, 6.735938, 0.587500, 0, 6.712110, 0.592969, 0, 6.712110, 0.592969, 0, 6.735938, 0.587500, 0, 6.735938, 0.587500, 0, 6.735938, 0.587500, 0, 6.758920, 0.580469, 0, 6.735938, 0.587500, 0, 6.735938, 0.587500, 0, 6.758920, 0.580469, 0, 6.758920, 0.580469, 0, 6.758920, 0.580469, 0, 6.780990, 0.571875, 0, 6.758920, 0.580469, 0, 6.758920, 0.580469, 0, 6.780990, 0.571875, 0, 6.780990, 0.571875, 0, 6.780990, 0.571875, 0, 6.802148, 0.561719, 0, 6.780990, 0.571875, 0, 6.780990, 0.571875, 0, 6.802148, 0.561719, 0, 6.802148, 0.561719, 0, 6.407813, 0.116667, 0, 6.407813, 0.016667, 0, 6.434278, 0.104948, 0, 6.407813, 0.016667, 0, 6.407813, 0.116667, 0, 6.434278, 0.104948, 0, 6.460547, 0.094792, 0, 6.434278, 0.104948, 0, 6.407813, 0.016667, 0, 6.434278, 0.104948, 0, 6.460547, 0.094792, 0, 6.407813, 0.016667, 0, 6.460547, 0.094792, 0, 6.407813, 0.016667, 0, 6.435596, 0.008854, 0, 6.407813, 0.016667, 0, 6.460547, 0.094792, 0, 6.435596, 0.008854, 0, 6.486621, 0.086198, 0, 6.460547, 0.094792, 0, 6.435596, 0.008854, 0, 6.460547, 0.094792, 0, 6.486621, 0.086198, 0, 6.435596, 0.008854, 0, 6.486621, 0.086198, 0, 6.435596, 0.008854, 0, 6.462696, 0.002083, 0, 6.435596, 0.008854, 0, 6.486621, 0.086198, 0, 6.462696, 0.002083, 0, 6.486621, 0.086198, 0, 6.462696, 0.002083, 0, 6.489111, -0.003646, 0, 6.462696, 0.002083, 0, 6.486621, 0.086198, 0, 6.489111, -0.003646, 0, 6.486621, 0.086198, 0, 6.489111, -0.003646, 0, 6.512500, 0.079167, 0, 6.489111, -0.003646, 0, 6.486621, 0.086198, 0, 6.512500, 0.079167, 0, 6.512500, 0.079167, 0, 6.489111, -0.003646, 0, 6.514844, -0.008333, 0, 6.489111, -0.003646, 0, 6.512500, 0.079167, 0, 6.514844, -0.008333, 0, 6.512500, 0.079167, 0, 6.514844, -0.008333, 0, 6.538217, 0.073698, 0, 6.514844, -0.008333, 0, 6.512500, 0.079167, 0, 6.538217, 0.073698, 0, 6.538217, 0.073698, 0, 6.514844, -0.008333, 0, 6.539990, -0.011979, 0, 6.514844, -0.008333, 0, 6.538217, 0.073698, 0, 6.539990, -0.011979, 0, 6.538217, 0.073698, 0, 6.539990, -0.011979, 0, 6.563802, 0.069792, 0, 6.539990, -0.011979, 0, 6.538217, 0.073698, 0, 6.563802, 0.069792, 0, 6.563802, 0.069792, 0, 6.539990, -0.011979, 0, 6.564649, -0.014583, 0, 6.539990, -0.011979, 0, 6.563802, 0.069792, 0, 6.564649, -0.014583, 0, 6.563802, 0.069792, 0, 6.564649, -0.014583, 0, 6.589258, 0.067448, 0, 6.564649, -0.014583, 0, 6.563802, 0.069792, 0, 6.589258, 0.067448, 0, 6.564649, -0.014583, 0, 6.588819, -0.016146, 0, 6.589258, 0.067448, 0, 6.588819, -0.016146, 0, 6.564649, -0.014583, 0, 6.589258, 0.067448, 0, 6.589258, 0.067448, 0, 6.588819, -0.016146, 0, 6.614583, 0.066667, 0, 6.588819, -0.016146, 0, 6.589258, 0.067448, 0, 6.614583, 0.066667, 0, 6.588819, -0.016146, 0, 6.612500, -0.016667, 0, 6.614583, 0.066667, 0, 6.612500, -0.016667, 0, 6.588819, -0.016146, 0, 6.614583, 0.066667, 0, 6.646484, 0.068058, 0, 6.614583, 0.066667, 0, 6.612500, -0.016667, 0, 6.614583, 0.066667, 0, 6.646484, 0.068058, 0, 6.612500, -0.016667, 0, 6.665739, -0.013745, 0, 6.646484, 0.068058, 0, 6.612500, -0.016667, 0, 6.646484, 0.068058, 0, 6.665739, -0.013745, 0, 6.612500, -0.016667, 0, 6.674479, 0.072233, 0, 6.646484, 0.068058, 0, 6.665739, -0.013745, 0, 6.646484, 0.068058, 0, 6.674479, 0.072233, 0, 6.665739, -0.013745, 0, 6.712955, -0.004980, 0, 6.674479, 0.072233, 0, 6.665739, -0.013745, 0, 6.674479, 0.072233, 0, 6.712955, -0.004980, 0, 6.665739, -0.013745, 0, 6.698568, 0.079191, 0, 6.674479, 0.072233, 0, 6.712955, -0.004980, 0, 6.674479, 0.072233, 0, 6.698568, 0.079191, 0, 6.712955, -0.004980, 0, 6.698568, 0.079191, 0, 6.712955, -0.004980, 0, 6.754150, 0.009627, 0, 6.712955, -0.004980, 0, 6.698568, 0.079191, 0, 6.754150, 0.009627, 0, 6.718750, 0.088932, 0, 6.698568, 0.079191, 0, 6.754150, 0.009627, 0, 6.698568, 0.079191, 0, 6.718750, 0.088932, 0, 6.754150, 0.009627, 0, 6.718750, 0.088932, 0, 6.754150, 0.009627, 0, 6.789323, 0.030078, 0, 6.754150, 0.009627, 0, 6.718750, 0.088932, 0, 6.789323, 0.030078, 0, 6.734701, 0.101213, 0, 6.718750, 0.088932, 0, 6.789323, 0.030078, 0, 6.718750, 0.088932, 0, 6.734701, 0.101213, 0, 6.789323, 0.030078, 0, 6.734701, 0.101213, 0, 6.789323, 0.030078, 0, 6.817464, 0.055526, 0, 6.789323, 0.030078, 0, 6.734701, 0.101213, 0, 6.817464, 0.055526, 0, 6.746094, 0.115788, 0, 6.734701, 0.101213, 0, 6.817464, 0.055526, 0, 6.734701, 0.101213, 0, 6.746094, 0.115788, 0, 6.817464, 0.055526, 0, 6.746094, 0.115788, 0, 6.817464, 0.055526, 0, 6.837565, 0.085124, 0, 6.817464, 0.055526, 0, 6.746094, 0.115788, 0, 6.837565, 0.085124, 0, 6.752930, 0.132658, 0, 6.746094, 0.115788, 0, 6.837565, 0.085124, 0, 6.746094, 0.115788, 0, 6.752930, 0.132658, 0, 6.837565, 0.085124, 0, 6.752930, 0.132658, 0, 6.837565, 0.085124, 0, 6.849626, 0.118872, 0, 6.837565, 0.085124, 0, 6.752930, 0.132658, 0, 6.849626, 0.118872, 0, 6.755208, 0.151823, 0, 6.752930, 0.132658, 0, 6.849626, 0.118872, 0, 6.752930, 0.132658, 0, 6.755208, 0.151823, 0, 6.849626, 0.118872, 0, 6.755208, 0.151823, 0, 6.849626, 0.118872, 0, 6.853646, 0.156771, 0, 6.849626, 0.118872, 0, 6.755208, 0.151823, 0, 6.853646, 0.156771, 0, 6.853646, 0.156771, 0, 6.753564, 0.169482, 0, 6.755208, 0.151823, 0, 6.753564, 0.169482, 0, 6.853646, 0.156771, 0, 6.755208, 0.151823, 0, 6.850961, 0.189982, 0, 6.753564, 0.169482, 0, 6.853646, 0.156771, 0, 6.753564, 0.169482, 0, 6.850961, 0.189982, 0, 6.853646, 0.156771, 0, 6.842904, 0.219303, 0, 6.753564, 0.169482, 0, 6.850961, 0.189982, 0, 6.753564, 0.169482, 0, 6.842904, 0.219303, 0, 6.850961, 0.189982, 0, 6.748633, 0.184961, 0, 6.753564, 0.169482, 0, 6.842904, 0.219303, 0, 6.753564, 0.169482, 0, 6.748633, 0.184961, 0, 6.842904, 0.219303, 0, 6.829476, 0.244735, 0, 6.748633, 0.184961, 0, 6.842904, 0.219303, 0, 6.748633, 0.184961, 0, 6.829476, 0.244735, 0, 6.842904, 0.219303, 0, 6.740414, 0.198258, 0, 6.748633, 0.184961, 0, 6.829476, 0.244735, 0, 6.748633, 0.184961, 0, 6.740414, 0.198258, 0, 6.829476, 0.244735, 0, 6.810678, 0.266276, 0, 6.740414, 0.198258, 0, 6.829476, 0.244735, 0, 6.740414, 0.198258, 0, 6.810678, 0.266276, 0, 6.829476, 0.244735, 0, 6.785498, 0.284774, 0, 6.740414, 0.198258, 0, 6.810678, 0.266276, 0, 6.740414, 0.198258, 0, 6.785498, 0.284774, 0, 6.810678, 0.266276, 0, 6.728906, 0.209375, 0, 6.740414, 0.198258, 0, 6.785498, 0.284774, 0, 6.740414, 0.198258, 0, 6.728906, 0.209375, 0, 6.785498, 0.284774, 0, 6.752930, 0.301074, 0, 6.728906, 0.209375, 0, 6.785498, 0.284774, 0, 6.728906, 0.209375, 0, 6.752930, 0.301074, 0, 6.785498, 0.284774, 0, 6.711833, 0.219303, 0, 6.728906, 0.209375, 0, 6.752930, 0.301074, 0, 6.728906, 0.209375, 0, 6.711833, 0.219303, 0, 6.752930, 0.301074, 0, 6.712972, 0.315177, 0, 6.711833, 0.219303, 0, 6.752930, 0.301074, 0, 6.711833, 0.219303, 0, 6.712972, 0.315177, 0, 6.752930, 0.301074, 0, 6.686914, 0.229036, 0, 6.711833, 0.219303, 0, 6.712972, 0.315177, 0, 6.711833, 0.219303, 0, 6.686914, 0.229036, 0, 6.712972, 0.315177, 0, 6.712972, 0.315177, 0, 6.654150, 0.238574, 0, 6.686914, 0.229036, 0, 6.654150, 0.238574, 0, 6.712972, 0.315177, 0, 6.686914, 0.229036, 0, 6.712972, 0.315177, 0, 6.665625, 0.327083, 0, 6.654150, 0.238574, 0, 6.665625, 0.327083, 0, 6.712972, 0.315177, 0, 6.654150, 0.238574, 0, 6.613542, 0.247917, 0, 6.654150, 0.238574, 0, 6.665625, 0.327083, 0, 6.654150, 0.238574, 0, 6.613542, 0.247917, 0, 6.665625, 0.327083, 0, 6.632813, 0.334375, 0, 6.613542, 0.247917, 0, 6.665625, 0.327083, 0, 6.613542, 0.247917, 0, 6.632813, 0.334375, 0, 6.665625, 0.327083, 0, 6.580209, 0.255729, 0, 6.613542, 0.247917, 0, 6.632813, 0.334375, 0, 6.613542, 0.247917, 0, 6.580209, 0.255729, 0, 6.632813, 0.334375, 0, 6.597494, 0.343359, 0, 6.580209, 0.255729, 0, 6.632813, 0.334375, 0, 6.580209, 0.255729, 0, 6.597494, 0.343359, 0, 6.632813, 0.334375, 0, 6.538770, 0.266642, 0, 6.580209, 0.255729, 0, 6.597494, 0.343359, 0, 6.580209, 0.255729, 0, 6.538770, 0.266642, 0, 6.597494, 0.343359, 0, 6.568620, 0.352604, 0, 6.538770, 0.266642, 0, 6.597494, 0.343359, 0, 6.538770, 0.266642, 0, 6.568620, 0.352604, 0, 6.597494, 0.343359, 0, 6.568620, 0.352604, 0, 6.546192, 0.362109, 0, 6.538770, 0.266642, 0, 6.546192, 0.362109, 0, 6.568620, 0.352604, 0, 6.538770, 0.266642, 0, 6.546192, 0.362109, 0, 6.503516, 0.280111, 0, 6.538770, 0.266642, 0, 6.503516, 0.280111, 0, 6.546192, 0.362109, 0, 6.538770, 0.266642, 0, 6.546192, 0.362109, 0, 6.530209, 0.371875, 0, 6.503516, 0.280111, 0, 6.530209, 0.371875, 0, 6.546192, 0.362109, 0, 6.503516, 0.280111, 0, 6.530209, 0.371875, 0, 6.474447, 0.296134, 0, 6.503516, 0.280111, 0, 6.474447, 0.296134, 0, 6.530209, 0.371875, 0, 6.503516, 0.280111, 0, 6.451562, 0.314714, 0, 6.474447, 0.296134, 0, 6.530209, 0.371875, 0, 6.474447, 0.296134, 0, 6.451562, 0.314714, 0, 6.530209, 0.371875, 0, 6.530209, 0.371875, 0, 6.519043, 0.382682, 0, 6.451562, 0.314714, 0, 6.519043, 0.382682, 0, 6.530209, 0.371875, 0, 6.451562, 0.314714, 0, 6.519043, 0.382682, 0, 6.434245, 0.336434, 0, 6.451562, 0.314714, 0, 6.434245, 0.336434, 0, 6.519043, 0.382682, 0, 6.451562, 0.314714, 0, 6.519043, 0.382682, 0, 6.511068, 0.395313, 0, 6.434245, 0.336434, 0, 6.511068, 0.395313, 0, 6.519043, 0.382682, 0, 6.434245, 0.336434, 0, 6.511068, 0.395313, 0, 6.421875, 0.361882, 0, 6.434245, 0.336434, 0, 6.421875, 0.361882, 0, 6.511068, 0.395313, 0, 6.434245, 0.336434, 0, 6.511068, 0.395313, 0, 6.506283, 0.409766, 0, 6.421875, 0.361882, 0, 6.506283, 0.409766, 0, 6.511068, 0.395313, 0, 6.421875, 0.361882, 0, 6.506283, 0.409766, 0, 6.414453, 0.391056, 0, 6.421875, 0.361882, 0, 6.414453, 0.391056, 0, 6.506283, 0.409766, 0, 6.421875, 0.361882, 0, 6.411979, 0.423958, 0, 6.414453, 0.391056, 0, 6.506283, 0.409766, 0, 6.414453, 0.391056, 0, 6.411979, 0.423958, 0, 6.506283, 0.409766, 0, 6.504688, 0.426042, 0, 6.411979, 0.423958, 0, 6.506283, 0.409766, 0, 6.411979, 0.423958, 0, 6.504688, 0.426042, 0, 6.506283, 0.409766, 0, 6.415625, 0.463631, 0, 6.411979, 0.423958, 0, 6.504688, 0.426042, 0, 6.411979, 0.423958, 0, 6.415625, 0.463631, 0, 6.504688, 0.426042, 0, 6.415625, 0.463631, 0, 6.504688, 0.426042, 0, 6.506917, 0.447282, 0, 6.504688, 0.426042, 0, 6.415625, 0.463631, 0, 6.506917, 0.447282, 0, 6.426563, 0.498535, 0, 6.415625, 0.463631, 0, 6.506917, 0.447282, 0, 6.415625, 0.463631, 0, 6.426563, 0.498535, 0, 6.506917, 0.447282, 0, 6.426563, 0.498535, 0, 6.506917, 0.447282, 0, 6.513607, 0.465690, 0, 6.506917, 0.447282, 0, 6.426563, 0.498535, 0, 6.513607, 0.465690, 0, 6.444791, 0.528670, 0, 6.426563, 0.498535, 0, 6.513607, 0.465690, 0, 6.426563, 0.498535, 0, 6.444791, 0.528670, 0, 6.513607, 0.465690, 0, 6.444791, 0.528670, 0, 6.513607, 0.465690, 0, 6.524756, 0.481266, 0, 6.513607, 0.465690, 0, 6.444791, 0.528670, 0, 6.524756, 0.481266, 0, 6.470313, 0.554036, 0, 6.444791, 0.528670, 0, 6.524756, 0.481266, 0, 6.444791, 0.528670, 0, 6.470313, 0.554036, 0, 6.524756, 0.481266, 0, 6.470313, 0.554036, 0, 6.524756, 0.481266, 0, 6.540365, 0.494010, 0, 6.524756, 0.481266, 0, 6.470313, 0.554036, 0, 6.540365, 0.494010, 0, 6.502539, 0.574145, 0, 6.470313, 0.554036, 0, 6.540365, 0.494010, 0, 6.470313, 0.554036, 0, 6.502539, 0.574145, 0, 6.540365, 0.494010, 0, 6.502539, 0.574145, 0, 6.540365, 0.494010, 0, 6.560433, 0.503923, 0, 6.540365, 0.494010, 0, 6.502539, 0.574145, 0, 6.560433, 0.503923, 0, 6.540885, 0.588509, 0, 6.502539, 0.574145, 0, 6.560433, 0.503923, 0, 6.502539, 0.574145, 0, 6.540885, 0.588509, 0, 6.560433, 0.503923, 0, 6.540885, 0.588509, 0, 6.560433, 0.503923, 0, 6.584961, 0.511003, 0, 6.560433, 0.503923, 0, 6.540885, 0.588509, 0, 6.584961, 0.511003, 0, 6.540885, 0.588509, 0, 6.584961, 0.511003, 0, 6.585351, 0.597127, 0, 6.584961, 0.511003, 0, 6.540885, 0.588509, 0, 6.585351, 0.597127, 0, 6.585351, 0.597127, 0, 6.584961, 0.511003, 0, 6.613949, 0.515251, 0, 6.584961, 0.511003, 0, 6.585351, 0.597127, 0, 6.613949, 0.515251, 0, 6.635938, 0.600000, 0, 6.585351, 0.597127, 0, 6.613949, 0.515251, 0, 6.585351, 0.597127, 0, 6.635938, 0.600000, 0, 6.613949, 0.515251, 0, 6.635938, 0.600000, 0, 6.613949, 0.515251, 0, 6.647396, 0.516667, 0, 6.613949, 0.515251, 0, 6.635938, 0.600000, 0, 6.647396, 0.516667, 0, 6.647396, 0.516667, 0, 6.662109, 0.599219, 0, 6.635938, 0.600000, 0, 6.662109, 0.599219, 0, 6.647396, 0.516667, 0, 6.635938, 0.600000, 0, 6.662109, 0.599219, 0, 6.647396, 0.516667, 0, 6.670639, 0.515885, 0, 6.647396, 0.516667, 0, 6.662109, 0.599219, 0, 6.670639, 0.515885, 0, 6.687500, 0.596875, 0, 6.662109, 0.599219, 0, 6.670639, 0.515885, 0, 6.662109, 0.599219, 0, 6.687500, 0.596875, 0, 6.670639, 0.515885, 0, 6.687500, 0.596875, 0, 6.670639, 0.515885, 0, 6.693490, 0.513542, 0, 6.670639, 0.515885, 0, 6.687500, 0.596875, 0, 6.693490, 0.513542, 0, 6.687500, 0.596875, 0, 6.693490, 0.513542, 0, 6.712110, 0.592969, 0, 6.693490, 0.513542, 0, 6.687500, 0.596875, 0, 6.712110, 0.592969, 0, 6.712110, 0.592969, 0, 6.693490, 0.513542, 0, 6.715951, 0.509635, 0, 6.693490, 0.513542, 0, 6.712110, 0.592969, 0, 6.715951, 0.509635, 0, 6.712110, 0.592969, 0, 6.715951, 0.509635, 0, 6.735938, 0.587500, 0, 6.715951, 0.509635, 0, 6.712110, 0.592969, 0, 6.735938, 0.587500, 0, 6.758920, 0.580469, 0, 6.735938, 0.587500, 0, 6.715951, 0.509635, 0, 6.735938, 0.587500, 0, 6.758920, 0.580469, 0, 6.715951, 0.509635, 0, 6.758920, 0.580469, 0, 6.715951, 0.509635, 0, 6.738021, 0.504167, 0, 6.715951, 0.509635, 0, 6.758920, 0.580469, 0, 6.738021, 0.504167, 0, 6.758920, 0.580469, 0, 6.738021, 0.504167, 0, 6.780990, 0.571875, 0, 6.738021, 0.504167, 0, 6.758920, 0.580469, 0, 6.780990, 0.571875, 0, 6.738021, 0.504167, 0, 6.759701, 0.497135, 0, 6.780990, 0.571875, 0, 6.759701, 0.497135, 0, 6.738021, 0.504167, 0, 6.780990, 0.571875, 0, 6.780990, 0.571875, 0, 6.759701, 0.497135, 0, 6.802148, 0.561719, 0, 6.759701, 0.497135, 0, 6.780990, 0.571875, 0, 6.802148, 0.561719, 0, 6.759701, 0.497135, 0, 6.780990, 0.488542, 0, 6.802148, 0.561719, 0, 6.780990, 0.488542, 0, 6.759701, 0.497135, 0, 6.802148, 0.561719, 0, 6.802148, 0.561719, 0, 6.780990, 0.488542, 0, 6.822396, 0.550000, 0, 6.780990, 0.488542, 0, 6.802148, 0.561719, 0, 6.822396, 0.550000, 0, 6.780990, 0.488542, 0, 6.801888, 0.478385, 0, 6.822396, 0.550000, 0, 6.801888, 0.478385, 0, 6.780990, 0.488542, 0, 6.822396, 0.550000, 0, 6.801888, 0.478385, 0, 6.822396, 0.466667, 0, 6.822396, 0.550000, 0, 6.822396, 0.466667, 0, 6.801888, 0.478385, 0, 6.822396, 0.550000, 0, 7.629167, 0.816667, 0, 7.629167, 0.733333, 0, 7.629167, 0.816667, 0, 7.629167, 0.816667, 0, 7.629167, 0.733333, 0, 7.629167, 0.733333, 0, 7.629167, 0.733333, 0, 7.537500, 0.733333, 0, 7.629167, 0.733333, 0, 7.629167, 0.733333, 0, 7.537500, 0.733333, 0, 7.537500, 0.733333, 0, 7.537500, 0.733333, 0, 7.513688, 0.732007, 0, 7.537500, 0.733333, 0, 7.537500, 0.733333, 0, 7.513688, 0.732007, 0, 7.513688, 0.732007, 0, 7.513688, 0.732007, 0, 7.493815, 0.728027, 0, 7.513688, 0.732007, 0, 7.513688, 0.732007, 0, 7.493815, 0.728027, 0, 7.493815, 0.728027, 0, 7.493815, 0.728027, 0, 7.477881, 0.721395, 0, 7.493815, 0.728027, 0, 7.493815, 0.728027, 0, 7.477881, 0.721395, 0, 7.477881, 0.721395, 0, 7.477881, 0.721395, 0, 7.465885, 0.712109, 0, 7.477881, 0.721395, 0, 7.477881, 0.721395, 0, 7.465885, 0.712109, 0, 7.465885, 0.712109, 0, 7.465885, 0.712109, 0, 7.457113, 0.699373, 0, 7.465885, 0.712109, 0, 7.465885, 0.712109, 0, 7.457113, 0.699373, 0, 7.457113, 0.699373, 0, 7.457113, 0.699373, 0, 7.450846, 0.682389, 0, 7.457113, 0.699373, 0, 7.457113, 0.699373, 0, 7.450846, 0.682389, 0, 7.450846, 0.682389, 0, 7.450846, 0.682389, 0, 7.447086, 0.661157, 0, 7.450846, 0.682389, 0, 7.450846, 0.682389, 0, 7.447086, 0.661157, 0, 7.447086, 0.661157, 0, 7.447086, 0.661157, 0, 7.445834, 0.635677, 0, 7.447086, 0.661157, 0, 7.447086, 0.661157, 0, 7.445834, 0.635677, 0, 7.445834, 0.635677, 0, 7.445834, 0.635677, 0, 7.445834, 0.583333, 0, 7.445834, 0.635677, 0, 7.445834, 0.635677, 0, 7.445834, 0.583333, 0, 7.445834, 0.583333, 0, 7.445834, 0.583333, 0, 7.603646, 0.583333, 0, 7.445834, 0.583333, 0, 7.445834, 0.583333, 0, 7.603646, 0.583333, 0, 7.603646, 0.583333, 0, 7.603646, 0.583333, 0, 7.603646, 0.500000, 0, 7.603646, 0.583333, 0, 7.603646, 0.583333, 0, 7.603646, 0.500000, 0, 7.603646, 0.500000, 0, 7.603646, 0.500000, 0, 7.445834, 0.500000, 0, 7.603646, 0.500000, 0, 7.603646, 0.500000, 0, 7.445834, 0.500000, 0, 7.445834, 0.500000, 0, 7.445834, 0.500000, 0, 7.445834, 0, 0, 7.445834, 0.500000, 0, 7.445834, 0.500000, 0, 7.445834, 0, 0, 7.445834, 0, 0, 7.445834, 0, 0, 7.349479, 0, 0, 7.445834, 0, 0, 7.445834, 0, 0, 7.349479, 0, 0, 7.349479, 0, 0, 7.349479, 0, 0, 7.349479, 0.500000, 0, 7.349479, 0, 0, 7.349479, 0, 0, 7.349479, 0.500000, 0, 7.349479, 0.500000, 0, 7.349479, 0.500000, 0, 7.257813, 0.500000, 0, 7.349479, 0.500000, 0, 7.349479, 0.500000, 0, 7.257813, 0.500000, 0, 7.257813, 0.500000, 0, 7.257813, 0.500000, 0, 7.257813, 0.583333, 0, 7.257813, 0.500000, 0, 7.257813, 0.500000, 0, 7.257813, 0.583333, 0, 7.257813, 0.583333, 0, 7.257813, 0.583333, 0, 7.349479, 0.583333, 0, 7.257813, 0.583333, 0, 7.257813, 0.583333, 0, 7.349479, 0.583333, 0, 7.349479, 0.583333, 0, 7.349479, 0.583333, 0, 7.349479, 0.624740, 0, 7.349479, 0.583333, 0, 7.349479, 0.583333, 0, 7.349479, 0.624740, 0, 7.349479, 0.624740, 0, 7.349479, 0.624740, 0, 7.352311, 0.671468, 0, 7.349479, 0.624740, 0, 7.349479, 0.624740, 0, 7.352311, 0.671468, 0, 7.352311, 0.671468, 0, 7.352311, 0.671468, 0, 7.360807, 0.711393, 0, 7.352311, 0.671468, 0, 7.352311, 0.671468, 0, 7.360807, 0.711393, 0, 7.360807, 0.711393, 0, 7.360807, 0.711393, 0, 7.374967, 0.744515, 0, 7.360807, 0.711393, 0, 7.360807, 0.711393, 0, 7.374967, 0.744515, 0, 7.374967, 0.744515, 0, 7.374967, 0.744515, 0, 7.394792, 0.770833, 0, 7.374967, 0.744515, 0, 7.374967, 0.744515, 0, 7.394792, 0.770833, 0, 7.394792, 0.770833, 0, 7.394792, 0.770833, 0, 7.420768, 0.790885, 0, 7.394792, 0.770833, 0, 7.394792, 0.770833, 0, 7.420768, 0.790885, 0, 7.420768, 0.790885, 0, 7.420768, 0.790885, 0, 7.453385, 0.805208, 0, 7.420768, 0.790885, 0, 7.420768, 0.790885, 0, 7.453385, 0.805208, 0, 7.453385, 0.805208, 0, 7.453385, 0.805208, 0, 7.492643, 0.813802, 0, 7.453385, 0.805208, 0, 7.453385, 0.805208, 0, 7.492643, 0.813802, 0, 7.492643, 0.813802, 0, 7.492643, 0.813802, 0, 7.538541, 0.816667, 0, 7.492643, 0.813802, 0, 7.492643, 0.813802, 0, 7.538541, 0.816667, 0, 7.538541, 0.816667, 0, 7.257813, 0.583333, 0, 7.349479, 0.500000, 0, 7.349479, 0.583333, 0, 7.349479, 0.500000, 0, 7.257813, 0.583333, 0, 7.349479, 0.583333, 0, 7.257813, 0.583333, 0, 7.257813, 0.500000, 0, 7.349479, 0.500000, 0, 7.257813, 0.500000, 0, 7.257813, 0.583333, 0, 7.349479, 0.500000, 0, 7.349479, 0.583333, 0, 7.349479, 0.500000, 0, 7.445834, 0.500000, 0, 7.349479, 0.500000, 0, 7.349479, 0.583333, 0, 7.445834, 0.500000, 0, 7.349479, 0.583333, 0, 7.445834, 0.500000, 0, 7.445834, 0.583333, 0, 7.445834, 0.500000, 0, 7.349479, 0.583333, 0, 7.445834, 0.583333, 0, 7.349479, 0.624740, 0, 7.349479, 0.583333, 0, 7.445834, 0.583333, 0, 7.349479, 0.583333, 0, 7.349479, 0.624740, 0, 7.445834, 0.583333, 0, 7.349479, 0.624740, 0, 7.445834, 0.583333, 0, 7.445834, 0.635677, 0, 7.445834, 0.583333, 0, 7.349479, 0.624740, 0, 7.445834, 0.635677, 0, 7.352311, 0.671468, 0, 7.349479, 0.624740, 0, 7.445834, 0.635677, 0, 7.349479, 0.624740, 0, 7.352311, 0.671468, 0, 7.445834, 0.635677, 0, 7.352311, 0.671468, 0, 7.445834, 0.635677, 0, 7.447086, 0.661157, 0, 7.445834, 0.635677, 0, 7.352311, 0.671468, 0, 7.447086, 0.661157, 0, 7.450846, 0.682389, 0, 7.352311, 0.671468, 0, 7.447086, 0.661157, 0, 7.352311, 0.671468, 0, 7.450846, 0.682389, 0, 7.447086, 0.661157, 0, 7.450846, 0.682389, 0, 7.360807, 0.711393, 0, 7.352311, 0.671468, 0, 7.360807, 0.711393, 0, 7.450846, 0.682389, 0, 7.352311, 0.671468, 0, 7.457113, 0.699373, 0, 7.360807, 0.711393, 0, 7.450846, 0.682389, 0, 7.360807, 0.711393, 0, 7.457113, 0.699373, 0, 7.450846, 0.682389, 0, 7.457113, 0.699373, 0, 7.374967, 0.744515, 0, 7.360807, 0.711393, 0, 7.374967, 0.744515, 0, 7.457113, 0.699373, 0, 7.360807, 0.711393, 0, 7.394792, 0.770833, 0, 7.374967, 0.744515, 0, 7.457113, 0.699373, 0, 7.374967, 0.744515, 0, 7.394792, 0.770833, 0, 7.457113, 0.699373, 0, 7.394792, 0.770833, 0, 7.457113, 0.699373, 0, 7.465885, 0.712109, 0, 7.457113, 0.699373, 0, 7.394792, 0.770833, 0, 7.465885, 0.712109, 0, 7.420768, 0.790885, 0, 7.394792, 0.770833, 0, 7.465885, 0.712109, 0, 7.394792, 0.770833, 0, 7.420768, 0.790885, 0, 7.465885, 0.712109, 0, 7.420768, 0.790885, 0, 7.465885, 0.712109, 0, 7.477881, 0.721395, 0, 7.465885, 0.712109, 0, 7.420768, 0.790885, 0, 7.477881, 0.721395, 0, 7.453385, 0.805208, 0, 7.420768, 0.790885, 0, 7.477881, 0.721395, 0, 7.420768, 0.790885, 0, 7.453385, 0.805208, 0, 7.477881, 0.721395, 0, 7.453385, 0.805208, 0, 7.477881, 0.721395, 0, 7.493815, 0.728027, 0, 7.477881, 0.721395, 0, 7.453385, 0.805208, 0, 7.493815, 0.728027, 0, 7.492643, 0.813802, 0, 7.453385, 0.805208, 0, 7.493815, 0.728027, 0, 7.453385, 0.805208, 0, 7.492643, 0.813802, 0, 7.493815, 0.728027, 0, 7.492643, 0.813802, 0, 7.493815, 0.728027, 0, 7.513688, 0.732007, 0, 7.493815, 0.728027, 0, 7.492643, 0.813802, 0, 7.513688, 0.732007, 0, 7.492643, 0.813802, 0, 7.513688, 0.732007, 0, 7.538541, 0.816667, 0, 7.513688, 0.732007, 0, 7.492643, 0.813802, 0, 7.538541, 0.816667, 0, 7.513688, 0.732007, 0, 7.537500, 0.733333, 0, 7.538541, 0.816667, 0, 7.537500, 0.733333, 0, 7.513688, 0.732007, 0, 7.538541, 0.816667, 0, 7.538541, 0.816667, 0, 7.537500, 0.733333, 0, 7.629167, 0.733333, 0, 7.537500, 0.733333, 0, 7.538541, 0.816667, 0, 7.629167, 0.733333, 0, 7.538541, 0.816667, 0, 7.629167, 0.733333, 0, 7.629167, 0.816667, 0, 7.629167, 0.733333, 0, 7.538541, 0.816667, 0, 7.629167, 0.816667, 0, 7.445834, 0.583333, 0, 7.445834, 0.500000, 0, 7.603646, 0.500000, 0, 7.445834, 0.500000, 0, 7.445834, 0.583333, 0, 7.603646, 0.500000, 0, 7.445834, 0.583333, 0, 7.603646, 0.500000, 0, 7.603646, 0.583333, 0, 7.603646, 0.500000, 0, 7.445834, 0.583333, 0, 7.603646, 0.583333, 0, 7.349479, 0.500000, 0, 7.445834, 0, 0, 7.445834, 0.500000, 0, 7.445834, 0, 0, 7.349479, 0.500000, 0, 7.445834, 0.500000, 0, 7.349479, 0.500000, 0, 7.349479, 0, 0, 7.445834, 0, 0, 7.349479, 0, 0, 7.349479, 0.500000, 0, 7.445834, 0, 0, 7.707292, 0.225781, 0, 7.707292, 0.583333, 0, 7.707292, 0.225781, 0, 7.707292, 0.225781, 0, 7.707292, 0.583333, 0, 7.707292, 0.583333, 0, 7.707292, 0.583333, 0, 7.803125, 0.583333, 0, 7.707292, 0.583333, 0, 7.707292, 0.583333, 0, 7.803125, 0.583333, 0, 7.803125, 0.583333, 0, 7.803125, 0.583333, 0, 7.803125, 0.229427, 0, 7.803125, 0.583333, 0, 7.803125, 0.583333, 0, 7.803125, 0.229427, 0, 7.803125, 0.229427, 0, 7.803125, 0.229427, 0, 7.805143, 0.191227, 0, 7.803125, 0.229427, 0, 7.803125, 0.229427, 0, 7.805143, 0.191227, 0, 7.805143, 0.191227, 0, 7.805143, 0.191227, 0, 7.811198, 0.158138, 0, 7.805143, 0.191227, 0, 7.805143, 0.191227, 0, 7.811198, 0.158138, 0, 7.811198, 0.158138, 0, 7.811198, 0.158138, 0, 7.821289, 0.130160, 0, 7.811198, 0.158138, 0, 7.811198, 0.158138, 0, 7.821289, 0.130160, 0, 7.821289, 0.130160, 0, 7.821289, 0.130160, 0, 7.835416, 0.107292, 0, 7.821289, 0.130160, 0, 7.821289, 0.130160, 0, 7.835416, 0.107292, 0, 7.835416, 0.107292, 0, 7.835416, 0.107292, 0, 7.853580, 0.089518, 0, 7.835416, 0.107292, 0, 7.835416, 0.107292, 0, 7.853580, 0.089518, 0, 7.853580, 0.089518, 0, 7.853580, 0.089518, 0, 7.875782, 0.076823, 0, 7.853580, 0.089518, 0, 7.853580, 0.089518, 0, 7.875782, 0.076823, 0, 7.875782, 0.076823, 0, 7.875782, 0.076823, 0, 7.902018, 0.069206, 0, 7.875782, 0.076823, 0, 7.875782, 0.076823, 0, 7.902018, 0.069206, 0, 7.902018, 0.069206, 0, 7.902018, 0.069206, 0, 7.932292, 0.066667, 0, 7.902018, 0.069206, 0, 7.902018, 0.069206, 0, 7.932292, 0.066667, 0, 7.932292, 0.066667, 0, 7.932292, 0.066667, 0, 7.969059, 0.069718, 0, 7.932292, 0.066667, 0, 7.932292, 0.066667, 0, 7.969059, 0.069718, 0, 7.969059, 0.069718, 0, 7.969059, 0.069718, 0, 8.001758, 0.078874, 0, 7.969059, 0.069718, 0, 7.969059, 0.069718, 0, 8.001758, 0.078874, 0, 8.001758, 0.078874, 0, 8.001758, 0.078874, 0, 8.030387, 0.094132, 0, 8.001758, 0.078874, 0, 8.001758, 0.078874, 0, 8.030387, 0.094132, 0, 8.030387, 0.094132, 0, 8.030387, 0.094132, 0, 8.054948, 0.115495, 0, 8.030387, 0.094132, 0, 8.030387, 0.094132, 0, 8.054948, 0.115495, 0, 8.054948, 0.115495, 0, 8.054948, 0.115495, 0, 8.074658, 0.142114, 0, 8.054948, 0.115495, 0, 8.054948, 0.115495, 0, 8.074658, 0.142114, 0, 8.074658, 0.142114, 0, 8.074658, 0.142114, 0, 8.088737, 0.173145, 0, 8.074658, 0.142114, 0, 8.074658, 0.142114, 0, 8.088737, 0.173145, 0, 8.088737, 0.173145, 0, 8.088737, 0.173145, 0, 8.097184, 0.208586, 0, 8.088737, 0.173145, 0, 8.088737, 0.173145, 0, 8.097184, 0.208586, 0, 8.097184, 0.208586, 0, 8.097184, 0.208586, 0, 8.100000, 0.248438, 0, 8.097184, 0.208586, 0, 8.097184, 0.208586, 0, 8.100000, 0.248438, 0, 8.100000, 0.248438, 0, 8.100000, 0.248438, 0, 8.100000, 0.583333, 0, 8.100000, 0.248438, 0, 8.100000, 0.248438, 0, 8.100000, 0.583333, 0, 8.100000, 0.583333, 0, 8.100000, 0.583333, 0, 8.195834, 0.583333, 0, 8.100000, 0.583333, 0, 8.100000, 0.583333, 0, 8.195834, 0.583333, 0, 8.195834, 0.583333, 0, 8.195834, 0.583333, 0, 8.195834, 0, 0, 8.195834, 0.583333, 0, 8.195834, 0.583333, 0, 8.195834, 0, 0, 8.195834, 0, 0, 8.195834, 0, 0, 8.100000, 0, 0, 8.195834, 0, 0, 8.195834, 0, 0, 8.100000, 0, 0, 8.100000, 0, 0, 8.100000, 0, 0, 8.100000, 0.083333, 0, 8.100000, 0, 0, 8.100000, 0, 0, 8.100000, 0.083333, 0, 8.100000, 0.083333, 0, 8.100000, 0.083333, 0, 8.081852, 0.059578, 0, 8.100000, 0.083333, 0, 8.100000, 0.083333, 0, 8.081852, 0.059578, 0, 8.081852, 0.059578, 0, 8.081852, 0.059578, 0, 8.062304, 0.039095, 0, 8.081852, 0.059578, 0, 8.081852, 0.059578, 0, 8.062304, 0.039095, 0, 8.062304, 0.039095, 0, 8.062304, 0.039095, 0, 8.041358, 0.021883, 0, 8.062304, 0.039095, 0, 8.062304, 0.039095, 0, 8.041358, 0.021883, 0, 8.041358, 0.021883, 0, 8.041358, 0.021883, 0, 8.019011, 0.007943, 0, 8.041358, 0.021883, 0, 8.041358, 0.021883, 0, 8.019011, 0.007943, 0, 8.019011, 0.007943, 0, 8.019011, 0.007943, 0, 7.995036, -0.002824, 0, 8.019011, 0.007943, 0, 8.019011, 0.007943, 0, 7.995036, -0.002824, 0, 7.995036, -0.002824, 0, 7.995036, -0.002824, 0, 7.969205, -0.010514, 0, 7.995036, -0.002824, 0, 7.995036, -0.002824, 0, 7.969205, -0.010514, 0, 7.969205, -0.010514, 0, 7.969205, -0.010514, 0, 7.941520, -0.015129, 0, 7.969205, -0.010514, 0, 7.969205, -0.010514, 0, 7.941520, -0.015129, 0, 7.941520, -0.015129, 0, 7.941520, -0.015129, 0, 7.911979, -0.016667, 0, 7.941520, -0.015129, 0, 7.941520, -0.015129, 0, 7.911979, -0.016667, 0, 7.911979, -0.016667, 0, 7.911979, -0.016667, 0, 7.864746, -0.012809, 0, 7.911979, -0.016667, 0, 7.911979, -0.016667, 0, 7.864746, -0.012809, 0, 7.864746, -0.012809, 0, 7.864746, -0.012809, 0, 7.823568, -0.001237, 0, 7.864746, -0.012809, 0, 7.864746, -0.012809, 0, 7.823568, -0.001237, 0, 7.823568, -0.001237, 0, 7.823568, -0.001237, 0, 7.788444, 0.018050, 0, 7.823568, -0.001237, 0, 7.823568, -0.001237, 0, 7.788444, 0.018050, 0, 7.788444, 0.018050, 0, 7.788444, 0.018050, 0, 7.759375, 0.045052, 0, 7.788444, 0.018050, 0, 7.788444, 0.018050, 0, 7.759375, 0.045052, 0, 7.759375, 0.045052, 0, 7.759375, 0.045052, 0, 7.736588, 0.079492, 0, 7.759375, 0.045052, 0, 7.759375, 0.045052, 0, 7.736588, 0.079492, 0, 7.736588, 0.079492, 0, 7.736588, 0.079492, 0, 7.720313, 0.121094, 0, 7.736588, 0.079492, 0, 7.736588, 0.079492, 0, 7.720313, 0.121094, 0, 7.720313, 0.121094, 0, 7.720313, 0.121094, 0, 7.710547, 0.169857, 0, 7.720313, 0.121094, 0, 7.720313, 0.121094, 0, 7.710547, 0.169857, 0, 7.710547, 0.169857, 0, 7.707292, 0.583333, 0, 7.803125, 0.229427, 0, 7.803125, 0.583333, 0, 7.803125, 0.229427, 0, 7.707292, 0.583333, 0, 7.803125, 0.583333, 0, 7.707292, 0.583333, 0, 7.707292, 0.225781, 0, 7.803125, 0.229427, 0, 7.707292, 0.225781, 0, 7.707292, 0.583333, 0, 7.803125, 0.229427, 0, 7.803125, 0.229427, 0, 7.707292, 0.225781, 0, 7.805143, 0.191227, 0, 7.707292, 0.225781, 0, 7.803125, 0.229427, 0, 7.805143, 0.191227, 0, 7.707292, 0.225781, 0, 7.710547, 0.169857, 0, 7.805143, 0.191227, 0, 7.710547, 0.169857, 0, 7.707292, 0.225781, 0, 7.805143, 0.191227, 0, 7.805143, 0.191227, 0, 7.710547, 0.169857, 0, 7.811198, 0.158138, 0, 7.710547, 0.169857, 0, 7.805143, 0.191227, 0, 7.811198, 0.158138, 0, 7.710547, 0.169857, 0, 7.720313, 0.121094, 0, 7.811198, 0.158138, 0, 7.720313, 0.121094, 0, 7.710547, 0.169857, 0, 7.811198, 0.158138, 0, 7.811198, 0.158138, 0, 7.720313, 0.121094, 0, 7.821289, 0.130160, 0, 7.720313, 0.121094, 0, 7.811198, 0.158138, 0, 7.821289, 0.130160, 0, 7.720313, 0.121094, 0, 7.736588, 0.079492, 0, 7.821289, 0.130160, 0, 7.736588, 0.079492, 0, 7.720313, 0.121094, 0, 7.821289, 0.130160, 0, 7.821289, 0.130160, 0, 7.736588, 0.079492, 0, 7.835416, 0.107292, 0, 7.736588, 0.079492, 0, 7.821289, 0.130160, 0, 7.835416, 0.107292, 0, 7.736588, 0.079492, 0, 7.759375, 0.045052, 0, 7.835416, 0.107292, 0, 7.759375, 0.045052, 0, 7.736588, 0.079492, 0, 7.835416, 0.107292, 0, 7.835416, 0.107292, 0, 7.759375, 0.045052, 0, 7.788444, 0.018050, 0, 7.759375, 0.045052, 0, 7.835416, 0.107292, 0, 7.788444, 0.018050, 0, 7.853580, 0.089518, 0, 7.835416, 0.107292, 0, 7.788444, 0.018050, 0, 7.835416, 0.107292, 0, 7.853580, 0.089518, 0, 7.788444, 0.018050, 0, 7.853580, 0.089518, 0, 7.788444, 0.018050, 0, 7.823568, -0.001237, 0, 7.788444, 0.018050, 0, 7.853580, 0.089518, 0, 7.823568, -0.001237, 0, 7.875782, 0.076823, 0, 7.853580, 0.089518, 0, 7.823568, -0.001237, 0, 7.853580, 0.089518, 0, 7.875782, 0.076823, 0, 7.823568, -0.001237, 0, 7.823568, -0.001237, 0, 7.864746, -0.012809, 0, 7.875782, 0.076823, 0, 7.864746, -0.012809, 0, 7.823568, -0.001237, 0, 7.875782, 0.076823, 0, 7.875782, 0.076823, 0, 7.864746, -0.012809, 0, 7.902018, 0.069206, 0, 7.864746, -0.012809, 0, 7.875782, 0.076823, 0, 7.902018, 0.069206, 0, 7.902018, 0.069206, 0, 7.864746, -0.012809, 0, 7.911979, -0.016667, 0, 7.864746, -0.012809, 0, 7.902018, 0.069206, 0, 7.911979, -0.016667, 0, 7.902018, 0.069206, 0, 7.911979, -0.016667, 0, 7.932292, 0.066667, 0, 7.911979, -0.016667, 0, 7.902018, 0.069206, 0, 7.932292, 0.066667, 0, 7.932292, 0.066667, 0, 7.911979, -0.016667, 0, 7.941520, -0.015129, 0, 7.911979, -0.016667, 0, 7.932292, 0.066667, 0, 7.941520, -0.015129, 0, 7.932292, 0.066667, 0, 7.941520, -0.015129, 0, 7.969205, -0.010514, 0, 7.941520, -0.015129, 0, 7.932292, 0.066667, 0, 7.969205, -0.010514, 0, 7.969059, 0.069718, 0, 7.932292, 0.066667, 0, 7.969205, -0.010514, 0, 7.932292, 0.066667, 0, 7.969059, 0.069718, 0, 7.969205, -0.010514, 0, 7.969059, 0.069718, 0, 7.969205, -0.010514, 0, 7.995036, -0.002824, 0, 7.969205, -0.010514, 0, 7.969059, 0.069718, 0, 7.995036, -0.002824, 0, 7.969059, 0.069718, 0, 7.995036, -0.002824, 0, 8.019011, 0.007943, 0, 7.995036, -0.002824, 0, 7.969059, 0.069718, 0, 8.019011, 0.007943, 0, 8.001758, 0.078874, 0, 7.969059, 0.069718, 0, 8.019011, 0.007943, 0, 7.969059, 0.069718, 0, 8.001758, 0.078874, 0, 8.019011, 0.007943, 0, 8.001758, 0.078874, 0, 8.019011, 0.007943, 0, 8.041358, 0.021883, 0, 8.019011, 0.007943, 0, 8.001758, 0.078874, 0, 8.041358, 0.021883, 0, 8.001758, 0.078874, 0, 8.041358, 0.021883, 0, 8.062304, 0.039095, 0, 8.041358, 0.021883, 0, 8.001758, 0.078874, 0, 8.062304, 0.039095, 0, 8.030387, 0.094132, 0, 8.001758, 0.078874, 0, 8.062304, 0.039095, 0, 8.001758, 0.078874, 0, 8.030387, 0.094132, 0, 8.062304, 0.039095, 0, 8.030387, 0.094132, 0, 8.062304, 0.039095, 0, 8.081852, 0.059578, 0, 8.062304, 0.039095, 0, 8.030387, 0.094132, 0, 8.081852, 0.059578, 0, 8.054948, 0.115495, 0, 8.030387, 0.094132, 0, 8.081852, 0.059578, 0, 8.030387, 0.094132, 0, 8.054948, 0.115495, 0, 8.081852, 0.059578, 0, 8.054948, 0.115495, 0, 8.081852, 0.059578, 0, 8.100000, 0.083333, 0, 8.081852, 0.059578, 0, 8.054948, 0.115495, 0, 8.100000, 0.083333, 0, 8.074658, 0.142114, 0, 8.054948, 0.115495, 0, 8.100000, 0.083333, 0, 8.054948, 0.115495, 0, 8.074658, 0.142114, 0, 8.100000, 0.083333, 0, 8.088737, 0.173145, 0, 8.074658, 0.142114, 0, 8.100000, 0.083333, 0, 8.074658, 0.142114, 0, 8.088737, 0.173145, 0, 8.100000, 0.083333, 0, 8.100000, 0.083333, 0, 8.195834, 0, 0, 8.088737, 0.173145, 0, 8.195834, 0, 0, 8.100000, 0.083333, 0, 8.088737, 0.173145, 0, 8.100000, 0.083333, 0, 8.100000, 0, 0, 8.195834, 0, 0, 8.100000, 0, 0, 8.100000, 0.083333, 0, 8.195834, 0, 0, 8.195834, 0, 0, 8.097184, 0.208586, 0, 8.088737, 0.173145, 0, 8.097184, 0.208586, 0, 8.195834, 0, 0, 8.088737, 0.173145, 0, 8.100000, 0.248438, 0, 8.097184, 0.208586, 0, 8.195834, 0, 0, 8.097184, 0.208586, 0, 8.100000, 0.248438, 0, 8.195834, 0, 0, 8.195834, 0.583333, 0, 8.100000, 0.248438, 0, 8.195834, 0, 0, 8.100000, 0.248438, 0, 8.195834, 0.583333, 0, 8.195834, 0, 0, 8.195834, 0.583333, 0, 8.100000, 0.583333, 0, 8.100000, 0.248438, 0, 8.100000, 0.583333, 0, 8.195834, 0.583333, 0, 8.100000, 0.248438, 0, 8.495313, 0.750000, 0, 8.495313, 0.583333, 0, 8.495313, 0.750000, 0, 8.495313, 0.750000, 0, 8.495313, 0.583333, 0, 8.495313, 0.583333, 0, 8.495313, 0.583333, 0, 8.692709, 0.583333, 0, 8.495313, 0.583333, 0, 8.495313, 0.583333, 0, 8.692709, 0.583333, 0, 8.692709, 0.583333, 0, 8.692709, 0.583333, 0, 8.692709, 0.500000, 0, 8.692709, 0.583333, 0, 8.692709, 0.583333, 0, 8.692709, 0.500000, 0, 8.692709, 0.500000, 0, 8.692709, 0.500000, 0, 8.495313, 0.500000, 0, 8.692709, 0.500000, 0, 8.692709, 0.500000, 0, 8.495313, 0.500000, 0, 8.495313, 0.500000, 0, 8.495313, 0.500000, 0, 8.495313, 0.192188, 0, 8.495313, 0.500000, 0, 8.495313, 0.500000, 0, 8.495313, 0.192188, 0, 8.495313, 0.192188, 0, 8.495313, 0.192188, 0, 8.496533, 0.160645, 0, 8.495313, 0.192188, 0, 8.495313, 0.192188, 0, 8.496533, 0.160645, 0, 8.496533, 0.160645, 0, 8.496533, 0.160645, 0, 8.500196, 0.135286, 0, 8.496533, 0.160645, 0, 8.496533, 0.160645, 0, 8.500196, 0.135286, 0, 8.500196, 0.135286, 0, 8.500196, 0.135286, 0, 8.506299, 0.116113, 0, 8.500196, 0.135286, 0, 8.500196, 0.135286, 0, 8.506299, 0.116113, 0, 8.506299, 0.116113, 0, 8.506299, 0.116113, 0, 8.514844, 0.103125, 0, 8.506299, 0.116113, 0, 8.506299, 0.116113, 0, 8.514844, 0.103125, 0, 8.514844, 0.103125, 0, 8.514844, 0.103125, 0, 8.527133, 0.094466, 0, 8.514844, 0.103125, 0, 8.514844, 0.103125, 0, 8.527133, 0.094466, 0, 8.527133, 0.094466, 0, 8.527133, 0.094466, 0, 8.544467, 0.088281, 0, 8.527133, 0.094466, 0, 8.527133, 0.094466, 0, 8.544467, 0.088281, 0, 8.544467, 0.088281, 0, 8.544467, 0.088281, 0, 8.566846, 0.084570, 0, 8.544467, 0.088281, 0, 8.544467, 0.088281, 0, 8.566846, 0.084570, 0, 8.566846, 0.084570, 0, 8.566846, 0.084570, 0, 8.594272, 0.083333, 0, 8.566846, 0.084570, 0, 8.566846, 0.084570, 0, 8.594272, 0.083333, 0, 8.594272, 0.083333, 0, 8.594272, 0.083333, 0, 8.692709, 0.083333, 0, 8.594272, 0.083333, 0, 8.594272, 0.083333, 0, 8.692709, 0.083333, 0, 8.692709, 0.083333, 0, 8.692709, 0.083333, 0, 8.692709, 0, 0, 8.692709, 0.083333, 0, 8.692709, 0.083333, 0, 8.692709, 0, 0, 8.692709, 0, 0, 8.692709, 0, 0, 8.594272, 0, 0, 8.692709, 0, 0, 8.692709, 0, 0, 8.594272, 0, 0, 8.594272, 0, 0, 8.594272, 0, 0, 8.543099, 0.002588, 0, 8.594272, 0, 0, 8.594272, 0, 0, 8.543099, 0.002588, 0, 8.543099, 0.002588, 0, 8.543099, 0.002588, 0, 8.500522, 0.010352, 0, 8.543099, 0.002588, 0, 8.543099, 0.002588, 0, 8.500522, 0.010352, 0, 8.500522, 0.010352, 0, 8.500522, 0.010352, 0, 8.466537, 0.023291, 0, 8.500522, 0.010352, 0, 8.500522, 0.010352, 0, 8.466537, 0.023291, 0, 8.466537, 0.023291, 0, 8.466537, 0.023291, 0, 8.441146, 0.041406, 0, 8.466537, 0.023291, 0, 8.466537, 0.023291, 0, 8.441146, 0.041406, 0, 8.441146, 0.041406, 0, 8.441146, 0.041406, 0, 8.422689, 0.066357, 0, 8.441146, 0.041406, 0, 8.441146, 0.041406, 0, 8.422689, 0.066357, 0, 8.422689, 0.066357, 0, 8.422689, 0.066357, 0, 8.409506, 0.099805, 0, 8.422689, 0.066357, 0, 8.422689, 0.066357, 0, 8.409506, 0.099805, 0, 8.409506, 0.099805, 0, 8.409506, 0.099805, 0, 8.401595, 0.141748, 0, 8.409506, 0.099805, 0, 8.409506, 0.099805, 0, 8.401595, 0.141748, 0, 8.401595, 0.141748, 0, 8.401595, 0.141748, 0, 8.398959, 0.192188, 0, 8.401595, 0.141748, 0, 8.401595, 0.141748, 0, 8.398959, 0.192188, 0, 8.398959, 0.192188, 0, 8.398959, 0.192188, 0, 8.398959, 0.500000, 0, 8.398959, 0.192188, 0, 8.398959, 0.192188, 0, 8.398959, 0.500000, 0, 8.398959, 0.500000, 0, 8.398959, 0.500000, 0, 8.328647, 0.500000, 0, 8.398959, 0.500000, 0, 8.398959, 0.500000, 0, 8.328647, 0.500000, 0, 8.328647, 0.500000, 0, 8.328647, 0.500000, 0, 8.328647, 0.583333, 0, 8.328647, 0.500000, 0, 8.328647, 0.500000, 0, 8.328647, 0.583333, 0, 8.328647, 0.583333, 0, 8.328647, 0.583333, 0, 8.398959, 0.583333, 0, 8.328647, 0.583333, 0, 8.328647, 0.583333, 0, 8.398959, 0.583333, 0, 8.398959, 0.583333, 0, 8.398959, 0.583333, 0, 8.398959, 0.750000, 0, 8.398959, 0.583333, 0, 8.398959, 0.583333, 0, 8.398959, 0.750000, 0, 8.398959, 0.750000, 0, 8.328647, 0.583333, 0, 8.398959, 0.500000, 0, 8.398959, 0.583333, 0, 8.398959, 0.500000, 0, 8.328647, 0.583333, 0, 8.398959, 0.583333, 0, 8.328647, 0.583333, 0, 8.328647, 0.500000, 0, 8.398959, 0.500000, 0, 8.328647, 0.500000, 0, 8.328647, 0.583333, 0, 8.398959, 0.500000, 0, 8.398959, 0.583333, 0, 8.398959, 0.500000, 0, 8.495313, 0.500000, 0, 8.398959, 0.500000, 0, 8.398959, 0.583333, 0, 8.495313, 0.500000, 0, 8.398959, 0.583333, 0, 8.495313, 0.500000, 0, 8.495313, 0.583333, 0, 8.495313, 0.500000, 0, 8.398959, 0.583333, 0, 8.495313, 0.583333, 0, 8.398959, 0.750000, 0, 8.398959, 0.583333, 0, 8.495313, 0.583333, 0, 8.398959, 0.583333, 0, 8.398959, 0.750000, 0, 8.495313, 0.583333, 0, 8.398959, 0.750000, 0, 8.495313, 0.583333, 0, 8.495313, 0.750000, 0, 8.495313, 0.583333, 0, 8.398959, 0.750000, 0, 8.495313, 0.750000, 0, 8.495313, 0.583333, 0, 8.495313, 0.500000, 0, 8.692709, 0.500000, 0, 8.495313, 0.500000, 0, 8.495313, 0.583333, 0, 8.692709, 0.500000, 0, 8.495313, 0.583333, 0, 8.692709, 0.500000, 0, 8.692709, 0.583333, 0, 8.692709, 0.500000, 0, 8.495313, 0.583333, 0, 8.692709, 0.583333, 0, 8.398959, 0.500000, 0, 8.495313, 0.192188, 0, 8.495313, 0.500000, 0, 8.495313, 0.192188, 0, 8.398959, 0.500000, 0, 8.495313, 0.500000, 0, 8.398959, 0.500000, 0, 8.398959, 0.192188, 0, 8.495313, 0.192188, 0, 8.398959, 0.192188, 0, 8.398959, 0.500000, 0, 8.495313, 0.192188, 0, 8.495313, 0.192188, 0, 8.398959, 0.192188, 0, 8.496533, 0.160645, 0, 8.398959, 0.192188, 0, 8.495313, 0.192188, 0, 8.496533, 0.160645, 0, 8.398959, 0.192188, 0, 8.401595, 0.141748, 0, 8.496533, 0.160645, 0, 8.401595, 0.141748, 0, 8.398959, 0.192188, 0, 8.496533, 0.160645, 0, 8.496533, 0.160645, 0, 8.401595, 0.141748, 0, 8.500196, 0.135286, 0, 8.401595, 0.141748, 0, 8.496533, 0.160645, 0, 8.500196, 0.135286, 0, 8.401595, 0.141748, 0, 8.409506, 0.099805, 0, 8.500196, 0.135286, 0, 8.409506, 0.099805, 0, 8.401595, 0.141748, 0, 8.500196, 0.135286, 0, 8.500196, 0.135286, 0, 8.409506, 0.099805, 0, 8.506299, 0.116113, 0, 8.409506, 0.099805, 0, 8.500196, 0.135286, 0, 8.506299, 0.116113, 0, 8.506299, 0.116113, 0, 8.409506, 0.099805, 0, 8.422689, 0.066357, 0, 8.409506, 0.099805, 0, 8.506299, 0.116113, 0, 8.422689, 0.066357, 0, 8.514844, 0.103125, 0, 8.506299, 0.116113, 0, 8.422689, 0.066357, 0, 8.506299, 0.116113, 0, 8.514844, 0.103125, 0, 8.422689, 0.066357, 0, 8.422689, 0.066357, 0, 8.441146, 0.041406, 0, 8.514844, 0.103125, 0, 8.441146, 0.041406, 0, 8.422689, 0.066357, 0, 8.514844, 0.103125, 0, 8.514844, 0.103125, 0, 8.441146, 0.041406, 0, 8.466537, 0.023291, 0, 8.441146, 0.041406, 0, 8.514844, 0.103125, 0, 8.466537, 0.023291, 0, 8.527133, 0.094466, 0, 8.514844, 0.103125, 0, 8.466537, 0.023291, 0, 8.514844, 0.103125, 0, 8.527133, 0.094466, 0, 8.466537, 0.023291, 0, 8.466537, 0.023291, 0, 8.500522, 0.010352, 0, 8.527133, 0.094466, 0, 8.500522, 0.010352, 0, 8.466537, 0.023291, 0, 8.527133, 0.094466, 0, 8.527133, 0.094466, 0, 8.500522, 0.010352, 0, 8.544467, 0.088281, 0, 8.500522, 0.010352, 0, 8.527133, 0.094466, 0, 8.544467, 0.088281, 0, 8.500522, 0.010352, 0, 8.543099, 0.002588, 0, 8.544467, 0.088281, 0, 8.543099, 0.002588, 0, 8.500522, 0.010352, 0, 8.544467, 0.088281, 0, 8.544467, 0.088281, 0, 8.543099, 0.002588, 0, 8.566846, 0.084570, 0, 8.543099, 0.002588, 0, 8.544467, 0.088281, 0, 8.566846, 0.084570, 0, 8.566846, 0.084570, 0, 8.543099, 0.002588, 0, 8.594272, 0, 0, 8.543099, 0.002588, 0, 8.566846, 0.084570, 0, 8.594272, 0, 0, 8.594272, 0.083333, 0, 8.566846, 0.084570, 0, 8.594272, 0, 0, 8.566846, 0.084570, 0, 8.594272, 0.083333, 0, 8.594272, 0, 0, 8.594272, 0.083333, 0, 8.594272, 0, 0, 8.692709, 0, 0, 8.594272, 0, 0, 8.594272, 0.083333, 0, 8.692709, 0, 0, 8.594272, 0.083333, 0, 8.692709, 0, 0, 8.692709, 0.083333, 0, 8.692709, 0, 0, 8.594272, 0.083333, 0, 8.692709, 0.083333, 0, 8.817187, 0.583333, 0, 8.913021, 0.583333, 0, 8.817187, 0.583333, 0, 8.817187, 0.583333, 0, 8.913021, 0.583333, 0, 8.913021, 0.583333, 0, 8.913021, 0.583333, 0, 8.913021, 0, 0, 8.913021, 0.583333, 0, 8.913021, 0.583333, 0, 8.913021, 0, 0, 8.913021, 0, 0, 8.913021, 0, 0, 8.817187, 0, 0, 8.913021, 0, 0, 8.913021, 0, 0, 8.817187, 0, 0, 8.817187, 0, 0, 8.817187, 0.583333, 0, 8.913021, 0, 0, 8.913021, 0.583333, 0, 8.913021, 0, 0, 8.817187, 0.583333, 0, 8.913021, 0.583333, 0, 8.817187, 0.583333, 0, 8.817187, 0, 0, 8.913021, 0, 0, 8.817187, 0, 0, 8.817187, 0.583333, 0, 8.913021, 0, 0, 8.817187, 0.816667, 0, 8.913021, 0.816667, 0, 8.817187, 0.816667, 0, 8.817187, 0.816667, 0, 8.913021, 0.816667, 0, 8.913021, 0.816667, 0, 8.913021, 0.816667, 0, 8.913021, 0.700000, 0, 8.913021, 0.816667, 0, 8.913021, 0.816667, 0, 8.913021, 0.700000, 0, 8.913021, 0.700000, 0, 8.913021, 0.700000, 0, 8.817187, 0.700000, 0, 8.913021, 0.700000, 0, 8.913021, 0.700000, 0, 8.817187, 0.700000, 0, 8.817187, 0.700000, 0, 8.817187, 0.816667, 0, 8.913021, 0.700000, 0, 8.913021, 0.816667, 0, 8.913021, 0.700000, 0, 8.817187, 0.816667, 0, 8.913021, 0.816667, 0, 8.817187, 0.816667, 0, 8.817187, 0.700000, 0, 8.913021, 0.700000, 0, 8.817187, 0.700000, 0, 8.817187, 0.816667, 0, 8.913021, 0.700000, 0, 9.117188, 0.816667, 0, 9.213021, 0.816667, 0, 9.117188, 0.816667, 0, 9.117188, 0.816667, 0, 9.213021, 0.816667, 0, 9.213021, 0.816667, 0, 9.213021, 0.816667, 0, 9.213021, 0, 0, 9.213021, 0.816667, 0, 9.213021, 0.816667, 0, 9.213021, 0, 0, 9.213021, 0, 0, 9.213021, 0, 0, 9.117188, 0, 0, 9.213021, 0, 0, 9.213021, 0, 0, 9.117188, 0, 0, 9.117188, 0, 0, 9.117188, 0.816667, 0, 9.213021, 0, 0, 9.213021, 0.816667, 0, 9.213021, 0, 0, 9.117188, 0.816667, 0, 9.213021, 0.816667, 0, 9.117188, 0.816667, 0, 9.117188, 0, 0, 9.213021, 0, 0, 9.117188, 0, 0, 9.117188, 0.816667, 0, 9.213021, 0, 0, 9.916146, 0.318750, 0, 9.916146, 0.266667, 0, 9.916146, 0.318750, 0, 9.916146, 0.318750, 0, 9.916146, 0.266667, 0, 9.916146, 0.266667, 0, 9.916146, 0.266667, 0, 9.475521, 0.266667, 0, 9.916146, 0.266667, 0, 9.916146, 0.266667, 0, 9.475521, 0.266667, 0, 9.475521, 0.266667, 0, 9.475521, 0.266667, 0, 9.481592, 0.220744, 0, 9.475521, 0.266667, 0, 9.475521, 0.266667, 0, 9.481592, 0.220744, 0, 9.481592, 0.220744, 0, 9.481592, 0.220744, 0, 9.493555, 0.180632, 0, 9.481592, 0.220744, 0, 9.481592, 0.220744, 0, 9.493555, 0.180632, 0, 9.493555, 0.180632, 0, 9.493555, 0.180632, 0, 9.511410, 0.146330, 0, 9.493555, 0.180632, 0, 9.493555, 0.180632, 0, 9.511410, 0.146330, 0, 9.511410, 0.146330, 0, 9.511410, 0.146330, 0, 9.535156, 0.117839, 0, 9.511410, 0.146330, 0, 9.511410, 0.146330, 0, 9.535156, 0.117839, 0, 9.535156, 0.117839, 0, 9.535156, 0.117839, 0, 9.564469, 0.095451, 0, 9.535156, 0.117839, 0, 9.535156, 0.117839, 0, 9.564469, 0.095451, 0, 9.564469, 0.095451, 0, 9.564469, 0.095451, 0, 9.599024, 0.079460, 0, 9.564469, 0.095451, 0, 9.564469, 0.095451, 0, 9.599024, 0.079460, 0, 9.599024, 0.079460, 0, 9.599024, 0.079460, 0, 9.638819, 0.069865, 0, 9.599024, 0.079460, 0, 9.599024, 0.079460, 0, 9.638819, 0.069865, 0, 9.638819, 0.069865, 0, 9.638819, 0.069865, 0, 9.683854, 0.066667, 0, 9.638819, 0.069865, 0, 9.638819, 0.069865, 0, 9.683854, 0.066667, 0, 9.683854, 0.066667, 0, 9.683854, 0.066667, 0, 9.711246, 0.067448, 0, 9.683854, 0.066667, 0, 9.683854, 0.066667, 0, 9.711246, 0.067448, 0, 9.711246, 0.067448, 0, 9.711246, 0.067448, 0, 9.738216, 0.069792, 0, 9.711246, 0.067448, 0, 9.711246, 0.067448, 0, 9.738216, 0.069792, 0, 9.738216, 0.069792, 0, 9.738216, 0.069792, 0, 9.764762, 0.073698, 0, 9.738216, 0.069792, 0, 9.738216, 0.069792, 0, 9.764762, 0.073698, 0, 9.764762, 0.073698, 0, 9.764762, 0.073698, 0, 9.790885, 0.079167, 0, 9.764762, 0.073698, 0, 9.764762, 0.073698, 0, 9.790885, 0.079167, 0, 9.790885, 0.079167, 0, 9.790885, 0.079167, 0, 9.816749, 0.086198, 0, 9.790885, 0.079167, 0, 9.790885, 0.079167, 0, 9.816749, 0.086198, 0, 9.816749, 0.086198, 0, 9.816749, 0.086198, 0, 9.842513, 0.094792, 0, 9.816749, 0.086198, 0, 9.816749, 0.086198, 0, 9.842513, 0.094792, 0, 9.842513, 0.094792, 0, 9.842513, 0.094792, 0, 9.868180, 0.104948, 0, 9.842513, 0.094792, 0, 9.842513, 0.094792, 0, 9.868180, 0.104948, 0, 9.868180, 0.104948, 0, 9.868180, 0.104948, 0, 9.893750, 0.116667, 0, 9.868180, 0.104948, 0, 9.868180, 0.104948, 0, 9.893750, 0.116667, 0, 9.893750, 0.116667, 0, 9.893750, 0.116667, 0, 9.893750, 0.033333, 0, 9.893750, 0.116667, 0, 9.893750, 0.116667, 0, 9.893750, 0.033333, 0, 9.893750, 0.033333, 0, 9.893750, 0.033333, 0, 9.867806, 0.021826, 0, 9.893750, 0.033333, 0, 9.893750, 0.033333, 0, 9.867806, 0.021826, 0, 9.867806, 0.021826, 0, 9.867806, 0.021826, 0, 9.841537, 0.011784, 0, 9.867806, 0.021826, 0, 9.867806, 0.021826, 0, 9.841537, 0.011784, 0, 9.841537, 0.011784, 0, 9.841537, 0.011784, 0, 9.814941, 0.003206, 0, 9.841537, 0.011784, 0, 9.841537, 0.011784, 0, 9.814941, 0.003206, 0, 9.814941, 0.003206, 0, 9.814941, 0.003206, 0, 9.788021, -0.003906, 0, 9.814941, 0.003206, 0, 9.814941, 0.003206, 0, 9.788021, -0.003906, 0, 9.788021, -0.003906, 0, 9.788021, -0.003906, 0, 9.760840, -0.009489, 0, 9.788021, -0.003906, 0, 9.788021, -0.003906, 0, 9.760840, -0.009489, 0, 9.760840, -0.009489, 0, 9.760840, -0.009489, 0, 9.733463, -0.013477, 0, 9.760840, -0.009489, 0, 9.760840, -0.009489, 0, 9.733463, -0.013477, 0, 9.733463, -0.013477, 0, 9.733463, -0.013477, 0, 9.705893, -0.015869, 0, 9.733463, -0.013477, 0, 9.733463, -0.013477, 0, 9.705893, -0.015869, 0, 9.705893, -0.015869, 0, 9.705893, -0.015869, 0, 9.678125, -0.016667, 0, 9.705893, -0.015869, 0, 9.705893, -0.015869, 0, 9.678125, -0.016667, 0, 9.678125, -0.016667, 0, 9.678125, -0.016667, 0, 9.611963, -0.011556, 0, 9.678125, -0.016667, 0, 9.678125, -0.016667, 0, 9.611963, -0.011556, 0, 9.611963, -0.011556, 0, 9.611963, -0.011556, 0, 9.553061, 0.003776, 0, 9.611963, -0.011556, 0, 9.611963, -0.011556, 0, 9.553061, 0.003776, 0, 9.553061, 0.003776, 0, 9.553061, 0.003776, 0, 9.501416, 0.029329, 0, 9.553061, 0.003776, 0, 9.553061, 0.003776, 0, 9.501416, 0.029329, 0, 9.501416, 0.029329, 0, 9.501416, 0.029329, 0, 9.457031, 0.065104, 0, 9.501416, 0.029329, 0, 9.501416, 0.029329, 0, 9.457031, 0.065104, 0, 9.457031, 0.065104, 0, 9.457031, 0.065104, 0, 9.421371, 0.109603, 0, 9.457031, 0.065104, 0, 9.457031, 0.065104, 0, 9.421371, 0.109603, 0, 9.421371, 0.109603, 0, 9.421371, 0.109603, 0, 9.395899, 0.161328, 0, 9.421371, 0.109603, 0, 9.421371, 0.109603, 0, 9.395899, 0.161328, 0, 9.395899, 0.161328, 0, 9.395899, 0.161328, 0, 9.380615, 0.220280, 0, 9.395899, 0.161328, 0, 9.395899, 0.161328, 0, 9.380615, 0.220280, 0, 9.380615, 0.220280, 0, 9.380615, 0.220280, 0, 9.375522, 0.286458, 0, 9.380615, 0.220280, 0, 9.380615, 0.220280, 0, 9.375522, 0.286458, 0, 9.375522, 0.286458, 0, 9.375522, 0.286458, 0, 9.380355, 0.354867, 0, 9.375522, 0.286458, 0, 9.375522, 0.286458, 0, 9.380355, 0.354867, 0, 9.380355, 0.354867, 0, 9.380355, 0.354867, 0, 9.394857, 0.415820, 0, 9.380355, 0.354867, 0, 9.380355, 0.354867, 0, 9.394857, 0.415820, 0, 9.394857, 0.415820, 0, 9.394857, 0.415820, 0, 9.419027, 0.469320, 0, 9.394857, 0.415820, 0, 9.394857, 0.415820, 0, 9.419027, 0.469320, 0, 9.419027, 0.469320, 0, 9.419027, 0.469320, 0, 9.452865, 0.515365, 0, 9.419027, 0.469320, 0, 9.419027, 0.469320, 0, 9.452865, 0.515365, 0, 9.452865, 0.515365, 0, 9.452865, 0.515365, 0, 9.494906, 0.552393, 0, 9.452865, 0.515365, 0, 9.452865, 0.515365, 0, 9.494906, 0.552393, 0, 9.494906, 0.552393, 0, 9.494906, 0.552393, 0, 9.543685, 0.578841, 0, 9.494906, 0.552393, 0, 9.494906, 0.552393, 0, 9.543685, 0.578841, 0, 9.543685, 0.578841, 0, 9.543685, 0.578841, 0, 9.599203, 0.594710, 0, 9.543685, 0.578841, 0, 9.543685, 0.578841, 0, 9.599203, 0.594710, 0, 9.599203, 0.594710, 0, 9.599203, 0.594710, 0, 9.661459, 0.600000, 0, 9.599203, 0.594710, 0, 9.599203, 0.594710, 0, 9.661459, 0.600000, 0, 9.661459, 0.600000, 0, 9.661459, 0.600000, 0, 9.717237, 0.595272, 0, 9.661459, 0.600000, 0, 9.661459, 0.600000, 0, 9.717237, 0.595272, 0, 9.717237, 0.595272, 0, 9.717237, 0.595272, 0, 9.766862, 0.581087, 0, 9.717237, 0.595272, 0, 9.717237, 0.595272, 0, 9.766862, 0.581087, 0, 9.766862, 0.581087, 0, 9.766862, 0.581087, 0, 9.810336, 0.557446, 0, 9.766862, 0.581087, 0, 9.766862, 0.581087, 0, 9.810336, 0.557446, 0, 9.810336, 0.557446, 0, 9.810336, 0.557446, 0, 9.847656, 0.524349, 0, 9.810336, 0.557446, 0, 9.810336, 0.557446, 0, 9.847656, 0.524349, 0, 9.847656, 0.524349, 0, 9.847656, 0.524349, 0, 9.877621, 0.483130, 0, 9.847656, 0.524349, 0, 9.847656, 0.524349, 0, 9.877621, 0.483130, 0, 9.877621, 0.483130, 0, 9.877621, 0.483130, 0, 9.899024, 0.435124, 0, 9.877621, 0.483130, 0, 9.877621, 0.483130, 0, 9.899024, 0.435124, 0, 9.899024, 0.435124, 0, 9.899024, 0.435124, 0, 9.911865, 0.380330, 0, 9.899024, 0.435124, 0, 9.899024, 0.435124, 0, 9.911865, 0.380330, 0, 9.911865, 0.380330, 0, 9.380355, 0.354867, 0, 9.375522, 0.286458, 0, 9.478646, 0.349479, 0, 9.375522, 0.286458, 0, 9.380355, 0.354867, 0, 9.478646, 0.349479, 0, 9.394857, 0.415820, 0, 9.380355, 0.354867, 0, 9.478646, 0.349479, 0, 9.380355, 0.354867, 0, 9.394857, 0.415820, 0, 9.478646, 0.349479, 0, 9.394857, 0.415820, 0, 9.478646, 0.349479, 0, 9.484847, 0.387077, 0, 9.478646, 0.349479, 0, 9.394857, 0.415820, 0, 9.484847, 0.387077, 0, 9.496159, 0.420182, 0, 9.394857, 0.415820, 0, 9.484847, 0.387077, 0, 9.394857, 0.415820, 0, 9.496159, 0.420182, 0, 9.484847, 0.387077, 0, 9.496159, 0.420182, 0, 9.419027, 0.469320, 0, 9.394857, 0.415820, 0, 9.419027, 0.469320, 0, 9.496159, 0.420182, 0, 9.394857, 0.415820, 0, 9.419027, 0.469320, 0, 9.496159, 0.420182, 0, 9.512582, 0.448796, 0, 9.496159, 0.420182, 0, 9.419027, 0.469320, 0, 9.512582, 0.448796, 0, 9.452865, 0.515365, 0, 9.419027, 0.469320, 0, 9.512582, 0.448796, 0, 9.419027, 0.469320, 0, 9.452865, 0.515365, 0, 9.512582, 0.448796, 0, 9.452865, 0.515365, 0, 9.512582, 0.448796, 0, 9.534115, 0.472917, 0, 9.512582, 0.448796, 0, 9.452865, 0.515365, 0, 9.534115, 0.472917, 0, 9.494906, 0.552393, 0, 9.452865, 0.515365, 0, 9.534115, 0.472917, 0, 9.452865, 0.515365, 0, 9.494906, 0.552393, 0, 9.534115, 0.472917, 0, 9.494906, 0.552393, 0, 9.534115, 0.472917, 0, 9.560205, 0.492057, 0, 9.534115, 0.472917, 0, 9.494906, 0.552393, 0, 9.560205, 0.492057, 0, 9.543685, 0.578841, 0, 9.494906, 0.552393, 0, 9.560205, 0.492057, 0, 9.494906, 0.552393, 0, 9.543685, 0.578841, 0, 9.560205, 0.492057, 0, 9.543685, 0.578841, 0, 9.560205, 0.492057, 0, 9.590300, 0.505729, 0, 9.560205, 0.492057, 0, 9.543685, 0.578841, 0, 9.590300, 0.505729, 0, 9.543685, 0.578841, 0, 9.590300, 0.505729, 0, 9.599203, 0.594710, 0, 9.590300, 0.505729, 0, 9.543685, 0.578841, 0, 9.599203, 0.594710, 0, 9.599203, 0.594710, 0, 9.590300, 0.505729, 0, 9.624398, 0.513932, 0, 9.590300, 0.505729, 0, 9.599203, 0.594710, 0, 9.624398, 0.513932, 0, 9.599203, 0.594710, 0, 9.624398, 0.513932, 0, 9.661459, 0.600000, 0, 9.624398, 0.513932, 0, 9.599203, 0.594710, 0, 9.661459, 0.600000, 0, 9.661459, 0.600000, 0, 9.624398, 0.513932, 0, 9.662500, 0.516667, 0, 9.624398, 0.513932, 0, 9.661459, 0.600000, 0, 9.662500, 0.516667, 0, 9.661459, 0.600000, 0, 9.662500, 0.516667, 0, 9.696176, 0.513835, 0, 9.662500, 0.516667, 0, 9.661459, 0.600000, 0, 9.696176, 0.513835, 0, 9.661459, 0.600000, 0, 9.696176, 0.513835, 0, 9.717237, 0.595272, 0, 9.696176, 0.513835, 0, 9.661459, 0.600000, 0, 9.717237, 0.595272, 0, 9.717237, 0.595272, 0, 9.696176, 0.513835, 0, 9.726367, 0.505339, 0, 9.696176, 0.513835, 0, 9.717237, 0.595272, 0, 9.726367, 0.505339, 0, 9.717237, 0.595272, 0, 9.726367, 0.505339, 0, 9.766862, 0.581087, 0, 9.726367, 0.505339, 0, 9.717237, 0.595272, 0, 9.766862, 0.581087, 0, 9.726367, 0.505339, 0, 9.753077, 0.491178, 0, 9.766862, 0.581087, 0, 9.753077, 0.491178, 0, 9.726367, 0.505339, 0, 9.766862, 0.581087, 0, 9.766862, 0.581087, 0, 9.753077, 0.491178, 0, 9.810336, 0.557446, 0, 9.753077, 0.491178, 0, 9.766862, 0.581087, 0, 9.810336, 0.557446, 0, 9.753077, 0.491178, 0, 9.776302, 0.471354, 0, 9.810336, 0.557446, 0, 9.776302, 0.471354, 0, 9.753077, 0.491178, 0, 9.810336, 0.557446, 0, 9.810336, 0.557446, 0, 9.776302, 0.471354, 0, 9.847656, 0.524349, 0, 9.776302, 0.471354, 0, 9.810336, 0.557446, 0, 9.847656, 0.524349, 0, 9.776302, 0.471354, 0, 9.795166, 0.446777, 0, 9.847656, 0.524349, 0, 9.795166, 0.446777, 0, 9.776302, 0.471354, 0, 9.847656, 0.524349, 0, 9.847656, 0.524349, 0, 9.795166, 0.446777, 0, 9.877621, 0.483130, 0, 9.795166, 0.446777, 0, 9.847656, 0.524349, 0, 9.877621, 0.483130, 0, 9.795166, 0.446777, 0, 9.808789, 0.418359, 0, 9.877621, 0.483130, 0, 9.808789, 0.418359, 0, 9.795166, 0.446777, 0, 9.877621, 0.483130, 0, 9.877621, 0.483130, 0, 9.808789, 0.418359, 0, 9.899024, 0.435124, 0, 9.808789, 0.418359, 0, 9.877621, 0.483130, 0, 9.899024, 0.435124, 0, 9.808789, 0.418359, 0, 9.817172, 0.386100, 0, 9.899024, 0.435124, 0, 9.817172, 0.386100, 0, 9.808789, 0.418359, 0, 9.899024, 0.435124, 0, 9.899024, 0.435124, 0, 9.817172, 0.386100, 0, 9.911865, 0.380330, 0, 9.817172, 0.386100, 0, 9.899024, 0.435124, 0, 9.911865, 0.380330, 0, 9.817172, 0.386100, 0, 9.820313, 0.350000, 0, 9.911865, 0.380330, 0, 9.820313, 0.350000, 0, 9.817172, 0.386100, 0, 9.911865, 0.380330, 0, 9.911865, 0.380330, 0, 9.820313, 0.350000, 0, 9.916146, 0.318750, 0, 9.820313, 0.350000, 0, 9.911865, 0.380330, 0, 9.916146, 0.318750, 0, 9.916146, 0.266667, 0, 9.916146, 0.318750, 0, 9.820313, 0.350000, 0, 9.916146, 0.318750, 0, 9.916146, 0.266667, 0, 9.820313, 0.350000, 0, 9.820313, 0.350000, 0, 9.475521, 0.266667, 0, 9.916146, 0.266667, 0, 9.475521, 0.266667, 0, 9.820313, 0.350000, 0, 9.916146, 0.266667, 0, 9.820313, 0.350000, 0, 9.478646, 0.349479, 0, 9.475521, 0.266667, 0, 9.478646, 0.349479, 0, 9.820313, 0.350000, 0, 9.475521, 0.266667, 0, 9.375522, 0.286458, 0, 9.475521, 0.266667, 0, 9.478646, 0.349479, 0, 9.475521, 0.266667, 0, 9.375522, 0.286458, 0, 9.478646, 0.349479, 0, 9.375522, 0.286458, 0, 9.380615, 0.220280, 0, 9.475521, 0.266667, 0, 9.380615, 0.220280, 0, 9.375522, 0.286458, 0, 9.475521, 0.266667, 0, 9.475521, 0.266667, 0, 9.380615, 0.220280, 0, 9.481592, 0.220744, 0, 9.380615, 0.220280, 0, 9.475521, 0.266667, 0, 9.481592, 0.220744, 0, 9.380615, 0.220280, 0, 9.395899, 0.161328, 0, 9.481592, 0.220744, 0, 9.395899, 0.161328, 0, 9.380615, 0.220280, 0, 9.481592, 0.220744, 0, 9.481592, 0.220744, 0, 9.395899, 0.161328, 0, 9.493555, 0.180632, 0, 9.395899, 0.161328, 0, 9.481592, 0.220744, 0, 9.493555, 0.180632, 0, 9.395899, 0.161328, 0, 9.421371, 0.109603, 0, 9.493555, 0.180632, 0, 9.421371, 0.109603, 0, 9.395899, 0.161328, 0, 9.493555, 0.180632, 0, 9.493555, 0.180632, 0, 9.421371, 0.109603, 0, 9.511410, 0.146330, 0, 9.421371, 0.109603, 0, 9.493555, 0.180632, 0, 9.511410, 0.146330, 0, 9.421371, 0.109603, 0, 9.457031, 0.065104, 0, 9.511410, 0.146330, 0, 9.457031, 0.065104, 0, 9.421371, 0.109603, 0, 9.511410, 0.146330, 0, 9.511410, 0.146330, 0, 9.457031, 0.065104, 0, 9.535156, 0.117839, 0, 9.457031, 0.065104, 0, 9.511410, 0.146330, 0, 9.535156, 0.117839, 0, 9.457031, 0.065104, 0, 9.501416, 0.029329, 0, 9.535156, 0.117839, 0, 9.501416, 0.029329, 0, 9.457031, 0.065104, 0, 9.535156, 0.117839, 0, 9.535156, 0.117839, 0, 9.501416, 0.029329, 0, 9.564469, 0.095451, 0, 9.501416, 0.029329, 0, 9.535156, 0.117839, 0, 9.564469, 0.095451, 0, 9.564469, 0.095451, 0, 9.501416, 0.029329, 0, 9.553061, 0.003776, 0, 9.501416, 0.029329, 0, 9.564469, 0.095451, 0, 9.553061, 0.003776, 0, 9.599024, 0.079460, 0, 9.564469, 0.095451, 0, 9.553061, 0.003776, 0, 9.564469, 0.095451, 0, 9.599024, 0.079460, 0, 9.553061, 0.003776, 0, 9.599024, 0.079460, 0, 9.553061, 0.003776, 0, 9.611963, -0.011556, 0, 9.553061, 0.003776, 0, 9.599024, 0.079460, 0, 9.611963, -0.011556, 0, 9.638819, 0.069865, 0, 9.599024, 0.079460, 0, 9.611963, -0.011556, 0, 9.599024, 0.079460, 0, 9.638819, 0.069865, 0, 9.611963, -0.011556, 0, 9.638819, 0.069865, 0, 9.611963, -0.011556, 0, 9.678125, -0.016667, 0, 9.611963, -0.011556, 0, 9.638819, 0.069865, 0, 9.678125, -0.016667, 0, 9.683854, 0.066667, 0, 9.638819, 0.069865, 0, 9.678125, -0.016667, 0, 9.638819, 0.069865, 0, 9.683854, 0.066667, 0, 9.678125, -0.016667, 0, 9.683854, 0.066667, 0, 9.678125, -0.016667, 0, 9.705893, -0.015869, 0, 9.678125, -0.016667, 0, 9.683854, 0.066667, 0, 9.705893, -0.015869, 0, 9.683854, 0.066667, 0, 9.705893, -0.015869, 0, 9.711246, 0.067448, 0, 9.705893, -0.015869, 0, 9.683854, 0.066667, 0, 9.711246, 0.067448, 0, 9.711246, 0.067448, 0, 9.705893, -0.015869, 0, 9.733463, -0.013477, 0, 9.705893, -0.015869, 0, 9.711246, 0.067448, 0, 9.733463, -0.013477, 0, 9.711246, 0.067448, 0, 9.733463, -0.013477, 0, 9.738216, 0.069792, 0, 9.733463, -0.013477, 0, 9.711246, 0.067448, 0, 9.738216, 0.069792, 0, 9.738216, 0.069792, 0, 9.733463, -0.013477, 0, 9.760840, -0.009489, 0, 9.733463, -0.013477, 0, 9.738216, 0.069792, 0, 9.760840, -0.009489, 0, 9.738216, 0.069792, 0, 9.760840, -0.009489, 0, 9.764762, 0.073698, 0, 9.760840, -0.009489, 0, 9.738216, 0.069792, 0, 9.764762, 0.073698, 0, 9.764762, 0.073698, 0, 9.760840, -0.009489, 0, 9.788021, -0.003906, 0, 9.760840, -0.009489, 0, 9.764762, 0.073698, 0, 9.788021, -0.003906, 0, 9.764762, 0.073698, 0, 9.788021, -0.003906, 0, 9.790885, 0.079167, 0, 9.788021, -0.003906, 0, 9.764762, 0.073698, 0, 9.790885, 0.079167, 0, 9.790885, 0.079167, 0, 9.788021, -0.003906, 0, 9.814941, 0.003206, 0, 9.788021, -0.003906, 0, 9.790885, 0.079167, 0, 9.814941, 0.003206, 0, 9.816749, 0.086198, 0, 9.790885, 0.079167, 0, 9.814941, 0.003206, 0, 9.790885, 0.079167, 0, 9.816749, 0.086198, 0, 9.814941, 0.003206, 0, 9.841537, 0.011784, 0, 9.816749, 0.086198, 0, 9.814941, 0.003206, 0, 9.816749, 0.086198, 0, 9.841537, 0.011784, 0, 9.814941, 0.003206, 0, 9.816749, 0.086198, 0, 9.841537, 0.011784, 0, 9.867806, 0.021826, 0, 9.841537, 0.011784, 0, 9.816749, 0.086198, 0, 9.867806, 0.021826, 0, 9.842513, 0.094792, 0, 9.816749, 0.086198, 0, 9.867806, 0.021826, 0, 9.816749, 0.086198, 0, 9.842513, 0.094792, 0, 9.867806, 0.021826, 0, 9.842513, 0.094792, 0, 9.867806, 0.021826, 0, 9.893750, 0.033333, 0, 9.867806, 0.021826, 0, 9.842513, 0.094792, 0, 9.893750, 0.033333, 0, 9.868180, 0.104948, 0, 9.842513, 0.094792, 0, 9.893750, 0.033333, 0, 9.842513, 0.094792, 0, 9.868180, 0.104948, 0, 9.893750, 0.033333, 0, 9.868180, 0.104948, 0, 9.893750, 0.033333, 0, 9.893750, 0.116667, 0, 9.893750, 0.033333, 0, 9.868180, 0.104948, 0, 9.893750, 0.116667, 0, 9.820313, 0.350000, 0, 9.817172, 0.386100, 0, 9.820313, 0.350000, 0, 9.820313, 0.350000, 0, 9.817172, 0.386100, 0, 9.817172, 0.386100, 0, 9.817172, 0.386100, 0, 9.808789, 0.418359, 0, 9.817172, 0.386100, 0, 9.817172, 0.386100, 0, 9.808789, 0.418359, 0, 9.808789, 0.418359, 0, 9.808789, 0.418359, 0, 9.795166, 0.446777, 0, 9.808789, 0.418359, 0, 9.808789, 0.418359, 0, 9.795166, 0.446777, 0, 9.795166, 0.446777, 0, 9.795166, 0.446777, 0, 9.776302, 0.471354, 0, 9.795166, 0.446777, 0, 9.795166, 0.446777, 0, 9.776302, 0.471354, 0, 9.776302, 0.471354, 0, 9.776302, 0.471354, 0, 9.753077, 0.491178, 0, 9.776302, 0.471354, 0, 9.776302, 0.471354, 0, 9.753077, 0.491178, 0, 9.753077, 0.491178, 0, 9.753077, 0.491178, 0, 9.726367, 0.505339, 0, 9.753077, 0.491178, 0, 9.753077, 0.491178, 0, 9.726367, 0.505339, 0, 9.726367, 0.505339, 0, 9.726367, 0.505339, 0, 9.696176, 0.513835, 0, 9.726367, 0.505339, 0, 9.726367, 0.505339, 0, 9.696176, 0.513835, 0, 9.696176, 0.513835, 0, 9.696176, 0.513835, 0, 9.662500, 0.516667, 0, 9.696176, 0.513835, 0, 9.696176, 0.513835, 0, 9.662500, 0.516667, 0, 9.662500, 0.516667, 0, 9.662500, 0.516667, 0, 9.624398, 0.513932, 0, 9.662500, 0.516667, 0, 9.662500, 0.516667, 0, 9.624398, 0.513932, 0, 9.624398, 0.513932, 0, 9.624398, 0.513932, 0, 9.590300, 0.505729, 0, 9.624398, 0.513932, 0, 9.624398, 0.513932, 0, 9.590300, 0.505729, 0, 9.590300, 0.505729, 0, 9.590300, 0.505729, 0, 9.560205, 0.492057, 0, 9.590300, 0.505729, 0, 9.590300, 0.505729, 0, 9.560205, 0.492057, 0, 9.560205, 0.492057, 0, 9.560205, 0.492057, 0, 9.534115, 0.472917, 0, 9.560205, 0.492057, 0, 9.560205, 0.492057, 0, 9.534115, 0.472917, 0, 9.534115, 0.472917, 0, 9.534115, 0.472917, 0, 9.512582, 0.448796, 0, 9.534115, 0.472917, 0, 9.534115, 0.472917, 0, 9.512582, 0.448796, 0, 9.512582, 0.448796, 0, 9.512582, 0.448796, 0, 9.496159, 0.420182, 0, 9.512582, 0.448796, 0, 9.512582, 0.448796, 0, 9.496159, 0.420182, 0, 9.496159, 0.420182, 0, 9.496159, 0.420182, 0, 9.484847, 0.387077, 0, 9.496159, 0.420182, 0, 9.496159, 0.420182, 0, 9.484847, 0.387077, 0, 9.484847, 0.387077, 0, 9.484847, 0.387077, 0, 9.478646, 0.349479, 0, 9.484847, 0.387077, 0, 9.484847, 0.387077, 0, 9.478646, 0.349479, 0, 9.478646, 0.349479, 0, 10.080729, 0.133333, 0, 10.190625, 0.133333, 0, 10.080729, 0.133333, 0, 10.080729, 0.133333, 0, 10.190625, 0.133333, 0, 10.190625, 0.133333, 0, 10.190625, 0.133333, 0, 10.190625, 0, 0, 10.190625, 0.133333, 0, 10.190625, 0.133333, 0, 10.190625, 0, 0, 10.190625, 0, 0, 10.190625, 0, 0, 10.080729, 0, 0, 10.190625, 0, 0, 10.190625, 0, 0, 10.080729, 0, 0, 10.080729, 0, 0, 10.080729, 0.133333, 0, 10.190625, 0, 0, 10.190625, 0.133333, 0, 10.190625, 0, 0, 10.080729, 0.133333, 0, 10.190625, 0.133333, 0, 10.080729, 0.133333, 0, 10.080729, 0, 0, 10.190625, 0, 0, 10.080729, 0, 0, 10.080729, 0.133333, 0, 10.190625, 0, 0, 10.631250, 0.783333, 0, 10.744271, 0.783333, 0, 10.631250, 0.783333, 0, 10.631250, 0.783333, 0, 10.744271, 0.783333, 0, 10.744271, 0.783333, 0, 10.744271, 0.783333, 0, 10.959896, 0.461198, 0, 10.744271, 0.783333, 0, 10.744271, 0.783333, 0, 10.959896, 0.461198, 0, 10.959896, 0.461198, 0, 10.959896, 0.461198, 0, 11.173959, 0.783333, 0, 10.959896, 0.461198, 0, 10.959896, 0.461198, 0, 11.173959, 0.783333, 0, 11.173959, 0.783333, 0, 11.173959, 0.783333, 0, 11.286979, 0.783333, 0, 11.173959, 0.783333, 0, 11.173959, 0.783333, 0, 11.286979, 0.783333, 0, 11.286979, 0.783333, 0, 11.286979, 0.783333, 0, 11.011979, 0.372917, 0, 11.286979, 0.783333, 0, 11.286979, 0.783333, 0, 11.011979, 0.372917, 0, 11.011979, 0.372917, 0, 11.011979, 0.372917, 0, 11.011979, 0, 0, 11.011979, 0.372917, 0, 11.011979, 0.372917, 0, 11.011979, 0, 0, 11.011979, 0, 0, 11.011979, 0, 0, 10.906250, 0, 0, 11.011979, 0, 0, 11.011979, 0, 0, 10.906250, 0, 0, 10.906250, 0, 0, 10.906250, 0, 0, 10.906250, 0.372917, 0, 10.906250, 0, 0, 10.906250, 0, 0, 10.906250, 0.372917, 0, 10.906250, 0.372917, 0, 10.744271, 0.783333, 0, 10.631250, 0.783333, 0, 10.906250, 0.372917, 0, 10.631250, 0.783333, 0, 10.744271, 0.783333, 0, 10.906250, 0.372917, 0, 10.959896, 0.461198, 0, 10.744271, 0.783333, 0, 10.906250, 0.372917, 0, 10.744271, 0.783333, 0, 10.959896, 0.461198, 0, 10.906250, 0.372917, 0, 10.959896, 0.461198, 0, 10.906250, 0.372917, 0, 11.011979, 0.372917, 0, 10.906250, 0.372917, 0, 10.959896, 0.461198, 0, 11.011979, 0.372917, 0, 10.959896, 0.461198, 0, 11.011979, 0.372917, 0, 11.173959, 0.783333, 0, 11.011979, 0.372917, 0, 10.959896, 0.461198, 0, 11.173959, 0.783333, 0, 11.286979, 0.783333, 0, 11.173959, 0.783333, 0, 11.011979, 0.372917, 0, 11.173959, 0.783333, 0, 11.286979, 0.783333, 0, 11.011979, 0.372917, 0, 10.906250, 0.372917, 0, 11.011979, 0, 0, 11.011979, 0.372917, 0, 11.011979, 0, 0, 10.906250, 0.372917, 0, 11.011979, 0.372917, 0, 10.906250, 0.372917, 0, 10.906250, 0, 0, 11.011979, 0, 0, 10.906250, 0, 0, 10.906250, 0.372917, 0, 11.011979, 0, 0, 11.476563, 0.516667, 0, 11.440040, 0.512907, 0, 11.476563, 0.516667, 0, 11.476563, 0.516667, 0, 11.440040, 0.512907, 0, 11.440040, 0.512907, 0, 11.440040, 0.512907, 0, 11.407553, 0.501628, 0, 11.440040, 0.512907, 0, 11.440040, 0.512907, 0, 11.407553, 0.501628, 0, 11.407553, 0.501628, 0, 11.407553, 0.501628, 0, 11.379102, 0.482829, 0, 11.407553, 0.501628, 0, 11.407553, 0.501628, 0, 11.379102, 0.482829, 0, 11.379102, 0.482829, 0, 11.379102, 0.482829, 0, 11.354688, 0.456510, 0, 11.379102, 0.482829, 0, 11.379102, 0.482829, 0, 11.354688, 0.456510, 0, 11.354688, 0.456510, 0, 11.354688, 0.456510, 0, 11.335092, 0.423649, 0, 11.354688, 0.456510, 0, 11.354688, 0.456510, 0, 11.335092, 0.423649, 0, 11.335092, 0.423649, 0, 11.335092, 0.423649, 0, 11.321094, 0.385221, 0, 11.335092, 0.423649, 0, 11.335092, 0.423649, 0, 11.321094, 0.385221, 0, 11.321094, 0.385221, 0, 11.321094, 0.385221, 0, 11.312696, 0.341227, 0, 11.321094, 0.385221, 0, 11.321094, 0.385221, 0, 11.312696, 0.341227, 0, 11.312696, 0.341227, 0, 11.312696, 0.341227, 0, 11.309896, 0.291667, 0, 11.312696, 0.341227, 0, 11.312696, 0.341227, 0, 11.309896, 0.291667, 0, 11.309896, 0.291667, 0, 11.309896, 0.291667, 0, 11.312679, 0.242106, 0, 11.309896, 0.291667, 0, 11.309896, 0.291667, 0, 11.312679, 0.242106, 0, 11.312679, 0.242106, 0, 11.312679, 0.242106, 0, 11.321029, 0.198112, 0, 11.312679, 0.242106, 0, 11.312679, 0.242106, 0, 11.321029, 0.198112, 0, 11.321029, 0.198112, 0, 11.321029, 0.198112, 0, 11.334945, 0.159684, 0, 11.321029, 0.198112, 0, 11.321029, 0.198112, 0, 11.334945, 0.159684, 0, 11.334945, 0.159684, 0, 11.334945, 0.159684, 0, 11.354427, 0.126823, 0, 11.334945, 0.159684, 0, 11.334945, 0.159684, 0, 11.354427, 0.126823, 0, 11.354427, 0.126823, 0, 11.354427, 0.126823, 0, 11.378760, 0.100505, 0, 11.354427, 0.126823, 0, 11.354427, 0.126823, 0, 11.378760, 0.100505, 0, 11.378760, 0.100505, 0, 11.378760, 0.100505, 0, 11.407227, 0.081706, 0, 11.378760, 0.100505, 0, 11.378760, 0.100505, 0, 11.407227, 0.081706, 0, 11.407227, 0.081706, 0, 11.407227, 0.081706, 0, 11.439828, 0.070426, 0, 11.407227, 0.081706, 0, 11.407227, 0.081706, 0, 11.439828, 0.070426, 0, 11.439828, 0.070426, 0, 11.439828, 0.070426, 0, 11.476563, 0.066667, 0, 11.439828, 0.070426, 0, 11.439828, 0.070426, 0, 11.476563, 0.066667, 0, 11.476563, 0.066667, 0, 11.476563, 0.066667, 0, 11.512858, 0.070443, 0, 11.476563, 0.066667, 0, 11.476563, 0.066667, 0, 11.512858, 0.070443, 0, 11.512858, 0.070443, 0, 11.512858, 0.070443, 0, 11.545182, 0.081771, 0, 11.512858, 0.070443, 0, 11.512858, 0.070443, 0, 11.545182, 0.081771, 0, 11.545182, 0.081771, 0, 11.545182, 0.081771, 0, 11.573535, 0.100651, 0, 11.545182, 0.081771, 0, 11.545182, 0.081771, 0, 11.573535, 0.100651, 0, 11.573535, 0.100651, 0, 11.573535, 0.100651, 0, 11.597917, 0.127083, 0, 11.573535, 0.100651, 0, 11.573535, 0.100651, 0, 11.597917, 0.127083, 0, 11.597917, 0.127083, 0, 11.597917, 0.127083, 0, 11.617514, 0.160026, 0, 11.597917, 0.127083, 0, 11.597917, 0.127083, 0, 11.617514, 0.160026, 0, 11.617514, 0.160026, 0, 11.617514, 0.160026, 0, 11.631511, 0.198437, 0, 11.617514, 0.160026, 0, 11.617514, 0.160026, 0, 11.631511, 0.198437, 0, 11.631511, 0.198437, 0, 11.631511, 0.198437, 0, 11.639909, 0.242318, 0, 11.631511, 0.198437, 0, 11.631511, 0.198437, 0, 11.639909, 0.242318, 0, 11.639909, 0.242318, 0, 11.639909, 0.242318, 0, 11.642709, 0.291667, 0, 11.639909, 0.242318, 0, 11.639909, 0.242318, 0, 11.642709, 0.291667, 0, 11.642709, 0.291667, 0, 11.642709, 0.291667, 0, 11.639909, 0.340804, 0, 11.642709, 0.291667, 0, 11.642709, 0.291667, 0, 11.639909, 0.340804, 0, 11.639909, 0.340804, 0, 11.639909, 0.340804, 0, 11.631511, 0.384570, 0, 11.639909, 0.340804, 0, 11.639909, 0.340804, 0, 11.631511, 0.384570, 0, 11.631511, 0.384570, 0, 11.631511, 0.384570, 0, 11.617514, 0.422965, 0, 11.631511, 0.384570, 0, 11.631511, 0.384570, 0, 11.617514, 0.422965, 0, 11.617514, 0.422965, 0, 11.617514, 0.422965, 0, 11.597917, 0.455990, 0, 11.617514, 0.422965, 0, 11.617514, 0.422965, 0, 11.597917, 0.455990, 0, 11.597917, 0.455990, 0, 11.597917, 0.455990, 0, 11.573535, 0.482536, 0, 11.597917, 0.455990, 0, 11.597917, 0.455990, 0, 11.573535, 0.482536, 0, 11.573535, 0.482536, 0, 11.573535, 0.482536, 0, 11.545182, 0.501497, 0, 11.573535, 0.482536, 0, 11.573535, 0.482536, 0, 11.545182, 0.501497, 0, 11.545182, 0.501497, 0, 11.545182, 0.501497, 0, 11.512858, 0.512874, 0, 11.545182, 0.501497, 0, 11.545182, 0.501497, 0, 11.512858, 0.512874, 0, 11.512858, 0.512874, 0, 11.476563, 0.600000, 0, 11.535709, 0.594889, 0, 11.476563, 0.600000, 0, 11.476563, 0.600000, 0, 11.535709, 0.594889, 0, 11.535709, 0.594889, 0, 11.535709, 0.594889, 0, 11.588151, 0.579557, 0, 11.535709, 0.594889, 0, 11.535709, 0.594889, 0, 11.588151, 0.579557, 0, 11.588151, 0.579557, 0, 11.588151, 0.579557, 0, 11.633887, 0.554004, 0, 11.588151, 0.579557, 0, 11.588151, 0.579557, 0, 11.633887, 0.554004, 0, 11.633887, 0.554004, 0, 11.633887, 0.554004, 0, 11.672916, 0.518229, 0, 11.633887, 0.554004, 0, 11.633887, 0.554004, 0, 11.672916, 0.518229, 0, 11.672916, 0.518229, 0, 11.672916, 0.518229, 0, 11.704135, 0.473405, 0, 11.672916, 0.518229, 0, 11.672916, 0.518229, 0, 11.704135, 0.473405, 0, 11.704135, 0.473405, 0, 11.704135, 0.473405, 0, 11.726432, 0.420703, 0, 11.704135, 0.473405, 0, 11.704135, 0.473405, 0, 11.726432, 0.420703, 0, 11.726432, 0.420703, 0, 11.726432, 0.420703, 0, 11.739811, 0.360124, 0, 11.726432, 0.420703, 0, 11.726432, 0.420703, 0, 11.739811, 0.360124, 0, 11.739811, 0.360124, 0, 11.739811, 0.360124, 0, 11.744271, 0.291667, 0, 11.739811, 0.360124, 0, 11.739811, 0.360124, 0, 11.744271, 0.291667, 0, 11.744271, 0.291667, 0, 11.744271, 0.291667, 0, 11.739811, 0.223421, 0, 11.744271, 0.291667, 0, 11.744271, 0.291667, 0, 11.739811, 0.223421, 0, 11.739811, 0.223421, 0, 11.739811, 0.223421, 0, 11.726432, 0.162956, 0, 11.739811, 0.223421, 0, 11.739811, 0.223421, 0, 11.726432, 0.162956, 0, 11.726432, 0.162956, 0, 11.726432, 0.162956, 0, 11.704135, 0.110270, 0, 11.726432, 0.162956, 0, 11.726432, 0.162956, 0, 11.704135, 0.110270, 0, 11.704135, 0.110270, 0, 11.704135, 0.110270, 0, 11.672916, 0.065365, 0, 11.704135, 0.110270, 0, 11.704135, 0.110270, 0, 11.672916, 0.065365, 0, 11.672916, 0.065365, 0, 11.672916, 0.065365, 0, 11.633887, 0.029476, 0, 11.672916, 0.065365, 0, 11.672916, 0.065365, 0, 11.633887, 0.029476, 0, 11.633887, 0.029476, 0, 11.633887, 0.029476, 0, 11.588151, 0.003841, 0, 11.633887, 0.029476, 0, 11.633887, 0.029476, 0, 11.588151, 0.003841, 0, 11.588151, 0.003841, 0, 11.588151, 0.003841, 0, 11.535709, -0.011540, 0, 11.588151, 0.003841, 0, 11.588151, 0.003841, 0, 11.535709, -0.011540, 0, 11.535709, -0.011540, 0, 11.535709, -0.011540, 0, 11.476563, -0.016667, 0, 11.535709, -0.011540, 0, 11.535709, -0.011540, 0, 11.476563, -0.016667, 0, 11.476563, -0.016667, 0, 11.476563, -0.016667, 0, 11.417204, -0.011540, 0, 11.476563, -0.016667, 0, 11.476563, -0.016667, 0, 11.417204, -0.011540, 0, 11.417204, -0.011540, 0, 11.417204, -0.011540, 0, 11.364649, 0.003841, 0, 11.417204, -0.011540, 0, 11.417204, -0.011540, 0, 11.364649, 0.003841, 0, 11.364649, 0.003841, 0, 11.364649, 0.003841, 0, 11.318896, 0.029476, 0, 11.364649, 0.003841, 0, 11.364649, 0.003841, 0, 11.318896, 0.029476, 0, 11.318896, 0.029476, 0, 11.318896, 0.029476, 0, 11.279948, 0.065365, 0, 11.318896, 0.029476, 0, 11.318896, 0.029476, 0, 11.279948, 0.065365, 0, 11.279948, 0.065365, 0, 11.279948, 0.065365, 0, 11.248844, 0.110270, 0, 11.279948, 0.065365, 0, 11.279948, 0.065365, 0, 11.248844, 0.110270, 0, 11.248844, 0.110270, 0, 11.248844, 0.110270, 0, 11.226627, 0.162956, 0, 11.248844, 0.110270, 0, 11.248844, 0.110270, 0, 11.226627, 0.162956, 0, 11.226627, 0.162956, 0, 11.226627, 0.162956, 0, 11.213298, 0.223421, 0, 11.226627, 0.162956, 0, 11.226627, 0.162956, 0, 11.213298, 0.223421, 0, 11.213298, 0.223421, 0, 11.213298, 0.223421, 0, 11.208854, 0.291667, 0, 11.213298, 0.223421, 0, 11.213298, 0.223421, 0, 11.208854, 0.291667, 0, 11.208854, 0.291667, 0, 11.208854, 0.291667, 0, 11.213298, 0.360124, 0, 11.208854, 0.291667, 0, 11.208854, 0.291667, 0, 11.213298, 0.360124, 0, 11.213298, 0.360124, 0, 11.213298, 0.360124, 0, 11.226627, 0.420703, 0, 11.213298, 0.360124, 0, 11.213298, 0.360124, 0, 11.226627, 0.420703, 0, 11.226627, 0.420703, 0, 11.226627, 0.420703, 0, 11.248844, 0.473405, 0, 11.226627, 0.420703, 0, 11.226627, 0.420703, 0, 11.248844, 0.473405, 0, 11.248844, 0.473405, 0, 11.248844, 0.473405, 0, 11.279948, 0.518229, 0, 11.248844, 0.473405, 0, 11.248844, 0.473405, 0, 11.279948, 0.518229, 0, 11.279948, 0.518229, 0, 11.279948, 0.518229, 0, 11.318896, 0.554004, 0, 11.279948, 0.518229, 0, 11.279948, 0.518229, 0, 11.318896, 0.554004, 0, 11.318896, 0.554004, 0, 11.318896, 0.554004, 0, 11.364649, 0.579557, 0, 11.318896, 0.554004, 0, 11.318896, 0.554004, 0, 11.364649, 0.579557, 0, 11.364649, 0.579557, 0, 11.364649, 0.579557, 0, 11.417204, 0.594889, 0, 11.364649, 0.579557, 0, 11.364649, 0.579557, 0, 11.417204, 0.594889, 0, 11.417204, 0.594889, 0, 11.312696, 0.341227, 0, 11.213298, 0.360124, 0, 11.208854, 0.291667, 0, 11.213298, 0.360124, 0, 11.312696, 0.341227, 0, 11.208854, 0.291667, 0, 11.321094, 0.385221, 0, 11.213298, 0.360124, 0, 11.312696, 0.341227, 0, 11.213298, 0.360124, 0, 11.321094, 0.385221, 0, 11.312696, 0.341227, 0, 11.321094, 0.385221, 0, 11.226627, 0.420703, 0, 11.213298, 0.360124, 0, 11.226627, 0.420703, 0, 11.321094, 0.385221, 0, 11.213298, 0.360124, 0, 11.226627, 0.420703, 0, 11.321094, 0.385221, 0, 11.335092, 0.423649, 0, 11.321094, 0.385221, 0, 11.226627, 0.420703, 0, 11.335092, 0.423649, 0, 11.248844, 0.473405, 0, 11.226627, 0.420703, 0, 11.335092, 0.423649, 0, 11.226627, 0.420703, 0, 11.248844, 0.473405, 0, 11.335092, 0.423649, 0, 11.354688, 0.456510, 0, 11.248844, 0.473405, 0, 11.335092, 0.423649, 0, 11.248844, 0.473405, 0, 11.354688, 0.456510, 0, 11.335092, 0.423649, 0, 11.354688, 0.456510, 0, 11.279948, 0.518229, 0, 11.248844, 0.473405, 0, 11.279948, 0.518229, 0, 11.354688, 0.456510, 0, 11.248844, 0.473405, 0, 11.318896, 0.554004, 0, 11.279948, 0.518229, 0, 11.354688, 0.456510, 0, 11.279948, 0.518229, 0, 11.318896, 0.554004, 0, 11.354688, 0.456510, 0, 11.318896, 0.554004, 0, 11.354688, 0.456510, 0, 11.379102, 0.482829, 0, 11.354688, 0.456510, 0, 11.318896, 0.554004, 0, 11.379102, 0.482829, 0, 11.364649, 0.579557, 0, 11.318896, 0.554004, 0, 11.379102, 0.482829, 0, 11.318896, 0.554004, 0, 11.364649, 0.579557, 0, 11.379102, 0.482829, 0, 11.364649, 0.579557, 0, 11.379102, 0.482829, 0, 11.407553, 0.501628, 0, 11.379102, 0.482829, 0, 11.364649, 0.579557, 0, 11.407553, 0.501628, 0, 11.364649, 0.579557, 0, 11.407553, 0.501628, 0, 11.417204, 0.594889, 0, 11.407553, 0.501628, 0, 11.364649, 0.579557, 0, 11.417204, 0.594889, 0, 11.417204, 0.594889, 0, 11.407553, 0.501628, 0, 11.440040, 0.512907, 0, 11.407553, 0.501628, 0, 11.417204, 0.594889, 0, 11.440040, 0.512907, 0, 11.417204, 0.594889, 0, 11.440040, 0.512907, 0, 11.476563, 0.600000, 0, 11.440040, 0.512907, 0, 11.417204, 0.594889, 0, 11.476563, 0.600000, 0, 11.440040, 0.512907, 0, 11.476563, 0.516667, 0, 11.476563, 0.600000, 0, 11.476563, 0.516667, 0, 11.440040, 0.512907, 0, 11.476563, 0.600000, 0, 11.476563, 0.600000, 0, 11.476563, 0.516667, 0, 11.512858, 0.512874, 0, 11.476563, 0.516667, 0, 11.476563, 0.600000, 0, 11.512858, 0.512874, 0, 11.476563, 0.600000, 0, 11.512858, 0.512874, 0, 11.535709, 0.594889, 0, 11.512858, 0.512874, 0, 11.476563, 0.600000, 0, 11.535709, 0.594889, 0, 11.535709, 0.594889, 0, 11.512858, 0.512874, 0, 11.545182, 0.501497, 0, 11.512858, 0.512874, 0, 11.535709, 0.594889, 0, 11.545182, 0.501497, 0, 11.535709, 0.594889, 0, 11.545182, 0.501497, 0, 11.588151, 0.579557, 0, 11.545182, 0.501497, 0, 11.535709, 0.594889, 0, 11.588151, 0.579557, 0, 11.545182, 0.501497, 0, 11.573535, 0.482536, 0, 11.588151, 0.579557, 0, 11.573535, 0.482536, 0, 11.545182, 0.501497, 0, 11.588151, 0.579557, 0, 11.588151, 0.579557, 0, 11.573535, 0.482536, 0, 11.633887, 0.554004, 0, 11.573535, 0.482536, 0, 11.588151, 0.579557, 0, 11.633887, 0.554004, 0, 11.573535, 0.482536, 0, 11.597917, 0.455990, 0, 11.633887, 0.554004, 0, 11.597917, 0.455990, 0, 11.573535, 0.482536, 0, 11.633887, 0.554004, 0, 11.633887, 0.554004, 0, 11.597917, 0.455990, 0, 11.672916, 0.518229, 0, 11.597917, 0.455990, 0, 11.633887, 0.554004, 0, 11.672916, 0.518229, 0, 11.672916, 0.518229, 0, 11.597917, 0.455990, 0, 11.704135, 0.473405, 0, 11.597917, 0.455990, 0, 11.672916, 0.518229, 0, 11.704135, 0.473405, 0, 11.704135, 0.473405, 0, 11.597917, 0.455990, 0, 11.617514, 0.422965, 0, 11.597917, 0.455990, 0, 11.704135, 0.473405, 0, 11.617514, 0.422965, 0, 11.704135, 0.473405, 0, 11.617514, 0.422965, 0, 11.726432, 0.420703, 0, 11.617514, 0.422965, 0, 11.704135, 0.473405, 0, 11.726432, 0.420703, 0, 11.617514, 0.422965, 0, 11.631511, 0.384570, 0, 11.726432, 0.420703, 0, 11.631511, 0.384570, 0, 11.617514, 0.422965, 0, 11.726432, 0.420703, 0, 11.726432, 0.420703, 0, 11.631511, 0.384570, 0, 11.739811, 0.360124, 0, 11.631511, 0.384570, 0, 11.726432, 0.420703, 0, 11.739811, 0.360124, 0, 11.631511, 0.384570, 0, 11.639909, 0.340804, 0, 11.739811, 0.360124, 0, 11.639909, 0.340804, 0, 11.631511, 0.384570, 0, 11.739811, 0.360124, 0, 11.739811, 0.360124, 0, 11.639909, 0.340804, 0, 11.744271, 0.291667, 0, 11.639909, 0.340804, 0, 11.739811, 0.360124, 0, 11.744271, 0.291667, 0, 11.744271, 0.291667, 0, 11.639909, 0.340804, 0, 11.642709, 0.291667, 0, 11.639909, 0.340804, 0, 11.744271, 0.291667, 0, 11.642709, 0.291667, 0, 11.744271, 0.291667, 0, 11.642709, 0.291667, 0, 11.639909, 0.242318, 0, 11.642709, 0.291667, 0, 11.744271, 0.291667, 0, 11.639909, 0.242318, 0, 11.744271, 0.291667, 0, 11.639909, 0.242318, 0, 11.739811, 0.223421, 0, 11.639909, 0.242318, 0, 11.744271, 0.291667, 0, 11.739811, 0.223421, 0, 11.639909, 0.242318, 0, 11.631511, 0.198437, 0, 11.739811, 0.223421, 0, 11.631511, 0.198437, 0, 11.639909, 0.242318, 0, 11.739811, 0.223421, 0, 11.631511, 0.198437, 0, 11.726432, 0.162956, 0, 11.739811, 0.223421, 0, 11.726432, 0.162956, 0, 11.631511, 0.198437, 0, 11.739811, 0.223421, 0, 11.631511, 0.198437, 0, 11.617514, 0.160026, 0, 11.726432, 0.162956, 0, 11.617514, 0.160026, 0, 11.631511, 0.198437, 0, 11.726432, 0.162956, 0, 11.617514, 0.160026, 0, 11.704135, 0.110270, 0, 11.726432, 0.162956, 0, 11.704135, 0.110270, 0, 11.617514, 0.160026, 0, 11.726432, 0.162956, 0, 11.617514, 0.160026, 0, 11.597917, 0.127083, 0, 11.704135, 0.110270, 0, 11.597917, 0.127083, 0, 11.617514, 0.160026, 0, 11.704135, 0.110270, 0, 11.597917, 0.127083, 0, 11.672916, 0.065365, 0, 11.704135, 0.110270, 0, 11.672916, 0.065365, 0, 11.597917, 0.127083, 0, 11.704135, 0.110270, 0, 11.672916, 0.065365, 0, 11.597917, 0.127083, 0, 11.633887, 0.029476, 0, 11.597917, 0.127083, 0, 11.672916, 0.065365, 0, 11.633887, 0.029476, 0, 11.597917, 0.127083, 0, 11.573535, 0.100651, 0, 11.633887, 0.029476, 0, 11.573535, 0.100651, 0, 11.597917, 0.127083, 0, 11.633887, 0.029476, 0, 11.633887, 0.029476, 0, 11.573535, 0.100651, 0, 11.588151, 0.003841, 0, 11.573535, 0.100651, 0, 11.633887, 0.029476, 0, 11.588151, 0.003841, 0, 11.573535, 0.100651, 0, 11.545182, 0.081771, 0, 11.588151, 0.003841, 0, 11.545182, 0.081771, 0, 11.573535, 0.100651, 0, 11.588151, 0.003841, 0, 11.588151, 0.003841, 0, 11.545182, 0.081771, 0, 11.535709, -0.011540, 0, 11.545182, 0.081771, 0, 11.588151, 0.003841, 0, 11.535709, -0.011540, 0, 11.545182, 0.081771, 0, 11.512858, 0.070443, 0, 11.535709, -0.011540, 0, 11.512858, 0.070443, 0, 11.545182, 0.081771, 0, 11.535709, -0.011540, 0, 11.535709, -0.011540, 0, 11.512858, 0.070443, 0, 11.476563, -0.016667, 0, 11.512858, 0.070443, 0, 11.535709, -0.011540, 0, 11.476563, -0.016667, 0, 11.512858, 0.070443, 0, 11.476563, 0.066667, 0, 11.476563, -0.016667, 0, 11.476563, 0.066667, 0, 11.512858, 0.070443, 0, 11.476563, -0.016667, 0, 11.476563, 0.066667, 0, 11.439828, 0.070426, 0, 11.476563, -0.016667, 0, 11.439828, 0.070426, 0, 11.476563, 0.066667, 0, 11.476563, -0.016667, 0, 11.439828, 0.070426, 0, 11.417204, -0.011540, 0, 11.476563, -0.016667, 0, 11.417204, -0.011540, 0, 11.439828, 0.070426, 0, 11.476563, -0.016667, 0, 11.439828, 0.070426, 0, 11.407227, 0.081706, 0, 11.417204, -0.011540, 0, 11.407227, 0.081706, 0, 11.439828, 0.070426, 0, 11.417204, -0.011540, 0, 11.407227, 0.081706, 0, 11.364649, 0.003841, 0, 11.417204, -0.011540, 0, 11.364649, 0.003841, 0, 11.407227, 0.081706, 0, 11.417204, -0.011540, 0, 11.407227, 0.081706, 0, 11.378760, 0.100505, 0, 11.364649, 0.003841, 0, 11.378760, 0.100505, 0, 11.407227, 0.081706, 0, 11.364649, 0.003841, 0, 11.378760, 0.100505, 0, 11.318896, 0.029476, 0, 11.364649, 0.003841, 0, 11.318896, 0.029476, 0, 11.378760, 0.100505, 0, 11.364649, 0.003841, 0, 11.378760, 0.100505, 0, 11.354427, 0.126823, 0, 11.318896, 0.029476, 0, 11.354427, 0.126823, 0, 11.378760, 0.100505, 0, 11.318896, 0.029476, 0, 11.354427, 0.126823, 0, 11.279948, 0.065365, 0, 11.318896, 0.029476, 0, 11.279948, 0.065365, 0, 11.354427, 0.126823, 0, 11.318896, 0.029476, 0, 11.248844, 0.110270, 0, 11.279948, 0.065365, 0, 11.354427, 0.126823, 0, 11.279948, 0.065365, 0, 11.248844, 0.110270, 0, 11.354427, 0.126823, 0, 11.334945, 0.159684, 0, 11.248844, 0.110270, 0, 11.354427, 0.126823, 0, 11.248844, 0.110270, 0, 11.334945, 0.159684, 0, 11.354427, 0.126823, 0, 11.226627, 0.162956, 0, 11.248844, 0.110270, 0, 11.334945, 0.159684, 0, 11.248844, 0.110270, 0, 11.226627, 0.162956, 0, 11.334945, 0.159684, 0, 11.321029, 0.198112, 0, 11.226627, 0.162956, 0, 11.334945, 0.159684, 0, 11.226627, 0.162956, 0, 11.321029, 0.198112, 0, 11.334945, 0.159684, 0, 11.213298, 0.223421, 0, 11.226627, 0.162956, 0, 11.321029, 0.198112, 0, 11.226627, 0.162956, 0, 11.213298, 0.223421, 0, 11.321029, 0.198112, 0, 11.312679, 0.242106, 0, 11.213298, 0.223421, 0, 11.321029, 0.198112, 0, 11.213298, 0.223421, 0, 11.312679, 0.242106, 0, 11.321029, 0.198112, 0, 11.208854, 0.291667, 0, 11.213298, 0.223421, 0, 11.312679, 0.242106, 0, 11.213298, 0.223421, 0, 11.208854, 0.291667, 0, 11.312679, 0.242106, 0, 11.309896, 0.291667, 0, 11.208854, 0.291667, 0, 11.312679, 0.242106, 0, 11.208854, 0.291667, 0, 11.309896, 0.291667, 0, 11.312679, 0.242106, 0, 11.312696, 0.341227, 0, 11.208854, 0.291667, 0, 11.309896, 0.291667, 0, 11.208854, 0.291667, 0, 11.312696, 0.341227, 0, 11.309896, 0.291667, 0, 11.890625, 0.225781, 0, 11.890625, 0.583333, 0, 11.890625, 0.225781, 0, 11.890625, 0.225781, 0, 11.890625, 0.583333, 0, 11.890625, 0.583333, 0, 11.890625, 0.583333, 0, 11.986459, 0.583333, 0, 11.890625, 0.583333, 0, 11.890625, 0.583333, 0, 11.986459, 0.583333, 0, 11.986459, 0.583333, 0, 11.986459, 0.583333, 0, 11.986459, 0.229427, 0, 11.986459, 0.583333, 0, 11.986459, 0.583333, 0, 11.986459, 0.229427, 0, 11.986459, 0.229427, 0, 11.986459, 0.229427, 0, 11.988477, 0.191227, 0, 11.986459, 0.229427, 0, 11.986459, 0.229427, 0, 11.988477, 0.191227, 0, 11.988477, 0.191227, 0, 11.988477, 0.191227, 0, 11.994532, 0.158138, 0, 11.988477, 0.191227, 0, 11.988477, 0.191227, 0, 11.994532, 0.158138, 0, 11.994532, 0.158138, 0, 11.994532, 0.158138, 0, 12.004622, 0.130160, 0, 11.994532, 0.158138, 0, 11.994532, 0.158138, 0, 12.004622, 0.130160, 0, 12.004622, 0.130160, 0, 12.004622, 0.130160, 0, 12.018750, 0.107292, 0, 12.004622, 0.130160, 0, 12.004622, 0.130160, 0, 12.018750, 0.107292, 0, 12.018750, 0.107292, 0, 12.018750, 0.107292, 0, 12.036914, 0.089518, 0, 12.018750, 0.107292, 0, 12.018750, 0.107292, 0, 12.036914, 0.089518, 0, 12.036914, 0.089518, 0, 12.036914, 0.089518, 0, 12.059115, 0.076823, 0, 12.036914, 0.089518, 0, 12.036914, 0.089518, 0, 12.059115, 0.076823, 0, 12.059115, 0.076823, 0, 12.059115, 0.076823, 0, 12.085352, 0.069206, 0, 12.059115, 0.076823, 0, 12.059115, 0.076823, 0, 12.085352, 0.069206, 0, 12.085352, 0.069206, 0, 12.085352, 0.069206, 0, 12.115625, 0.066667, 0, 12.085352, 0.069206, 0, 12.085352, 0.069206, 0, 12.115625, 0.066667, 0, 12.115625, 0.066667, 0, 12.115625, 0.066667, 0, 12.152392, 0.069718, 0, 12.115625, 0.066667, 0, 12.115625, 0.066667, 0, 12.152392, 0.069718, 0, 12.152392, 0.069718, 0, 12.152392, 0.069718, 0, 12.185092, 0.078874, 0, 12.152392, 0.069718, 0, 12.152392, 0.069718, 0, 12.185092, 0.078874, 0, 12.185092, 0.078874, 0, 12.185092, 0.078874, 0, 12.213721, 0.094132, 0, 12.185092, 0.078874, 0, 12.185092, 0.078874, 0, 12.213721, 0.094132, 0, 12.213721, 0.094132, 0, 12.213721, 0.094132, 0, 12.238281, 0.115495, 0, 12.213721, 0.094132, 0, 12.213721, 0.094132, 0, 12.238281, 0.115495, 0, 12.238281, 0.115495, 0, 12.238281, 0.115495, 0, 12.257992, 0.142114, 0, 12.238281, 0.115495, 0, 12.238281, 0.115495, 0, 12.257992, 0.142114, 0, 12.257992, 0.142114, 0, 12.257992, 0.142114, 0, 12.272071, 0.173145, 0, 12.257992, 0.142114, 0, 12.257992, 0.142114, 0, 12.272071, 0.173145, 0, 12.272071, 0.173145, 0, 12.272071, 0.173145, 0, 12.280518, 0.208586, 0, 12.272071, 0.173145, 0, 12.272071, 0.173145, 0, 12.280518, 0.208586, 0, 12.280518, 0.208586, 0, 12.280518, 0.208586, 0, 12.283334, 0.248438, 0, 12.280518, 0.208586, 0, 12.280518, 0.208586, 0, 12.283334, 0.248438, 0, 12.283334, 0.248438, 0, 12.283334, 0.248438, 0, 12.283334, 0.583333, 0, 12.283334, 0.248438, 0, 12.283334, 0.248438, 0, 12.283334, 0.583333, 0, 12.283334, 0.583333, 0, 12.283334, 0.583333, 0, 12.379167, 0.583333, 0, 12.283334, 0.583333, 0, 12.283334, 0.583333, 0, 12.379167, 0.583333, 0, 12.379167, 0.583333, 0, 12.379167, 0.583333, 0, 12.379167, 0, 0, 12.379167, 0.583333, 0, 12.379167, 0.583333, 0, 12.379167, 0, 0, 12.379167, 0, 0, 12.379167, 0, 0, 12.283334, 0, 0, 12.379167, 0, 0, 12.379167, 0, 0, 12.283334, 0, 0, 12.283334, 0, 0, 12.283334, 0, 0, 12.283334, 0.083333, 0, 12.283334, 0, 0, 12.283334, 0, 0, 12.283334, 0.083333, 0, 12.283334, 0.083333, 0, 12.283334, 0.083333, 0, 12.265185, 0.059578, 0, 12.283334, 0.083333, 0, 12.283334, 0.083333, 0, 12.265185, 0.059578, 0, 12.265185, 0.059578, 0, 12.265185, 0.059578, 0, 12.245639, 0.039095, 0, 12.265185, 0.059578, 0, 12.265185, 0.059578, 0, 12.245639, 0.039095, 0, 12.245639, 0.039095, 0, 12.245639, 0.039095, 0, 12.224691, 0.021883, 0, 12.245639, 0.039095, 0, 12.245639, 0.039095, 0, 12.224691, 0.021883, 0, 12.224691, 0.021883, 0, 12.224691, 0.021883, 0, 12.202344, 0.007943, 0, 12.224691, 0.021883, 0, 12.224691, 0.021883, 0, 12.202344, 0.007943, 0, 12.202344, 0.007943, 0, 12.202344, 0.007943, 0, 12.178370, -0.002824, 0, 12.202344, 0.007943, 0, 12.202344, 0.007943, 0, 12.178370, -0.002824, 0, 12.178370, -0.002824, 0, 12.178370, -0.002824, 0, 12.152539, -0.010514, 0, 12.178370, -0.002824, 0, 12.178370, -0.002824, 0, 12.152539, -0.010514, 0, 12.152539, -0.010514, 0, 12.152539, -0.010514, 0, 12.124854, -0.015129, 0, 12.152539, -0.010514, 0, 12.152539, -0.010514, 0, 12.124854, -0.015129, 0, 12.124854, -0.015129, 0, 12.124854, -0.015129, 0, 12.095313, -0.016667, 0, 12.124854, -0.015129, 0, 12.124854, -0.015129, 0, 12.095313, -0.016667, 0, 12.095313, -0.016667, 0, 12.095313, -0.016667, 0, 12.048079, -0.012809, 0, 12.095313, -0.016667, 0, 12.095313, -0.016667, 0, 12.048079, -0.012809, 0, 12.048079, -0.012809, 0, 12.048079, -0.012809, 0, 12.006901, -0.001237, 0, 12.048079, -0.012809, 0, 12.048079, -0.012809, 0, 12.006901, -0.001237, 0, 12.006901, -0.001237, 0, 12.006901, -0.001237, 0, 11.971778, 0.018050, 0, 12.006901, -0.001237, 0, 12.006901, -0.001237, 0, 11.971778, 0.018050, 0, 11.971778, 0.018050, 0, 11.971778, 0.018050, 0, 11.942709, 0.045052, 0, 11.971778, 0.018050, 0, 11.971778, 0.018050, 0, 11.942709, 0.045052, 0, 11.942709, 0.045052, 0, 11.942709, 0.045052, 0, 11.919922, 0.079492, 0, 11.942709, 0.045052, 0, 11.942709, 0.045052, 0, 11.919922, 0.079492, 0, 11.919922, 0.079492, 0, 11.919922, 0.079492, 0, 11.903646, 0.121094, 0, 11.919922, 0.079492, 0, 11.919922, 0.079492, 0, 11.903646, 0.121094, 0, 11.903646, 0.121094, 0, 11.903646, 0.121094, 0, 11.893881, 0.169857, 0, 11.903646, 0.121094, 0, 11.903646, 0.121094, 0, 11.893881, 0.169857, 0, 11.893881, 0.169857, 0, 11.890625, 0.583333, 0, 11.986459, 0.229427, 0, 11.986459, 0.583333, 0, 11.986459, 0.229427, 0, 11.890625, 0.583333, 0, 11.986459, 0.583333, 0, 11.890625, 0.583333, 0, 11.890625, 0.225781, 0, 11.986459, 0.229427, 0, 11.890625, 0.225781, 0, 11.890625, 0.583333, 0, 11.986459, 0.229427, 0, 11.986459, 0.229427, 0, 11.890625, 0.225781, 0, 11.988477, 0.191227, 0, 11.890625, 0.225781, 0, 11.986459, 0.229427, 0, 11.988477, 0.191227, 0, 11.890625, 0.225781, 0, 11.893881, 0.169857, 0, 11.988477, 0.191227, 0, 11.893881, 0.169857, 0, 11.890625, 0.225781, 0, 11.988477, 0.191227, 0, 11.988477, 0.191227, 0, 11.893881, 0.169857, 0, 11.994532, 0.158138, 0, 11.893881, 0.169857, 0, 11.988477, 0.191227, 0, 11.994532, 0.158138, 0, 11.893881, 0.169857, 0, 11.903646, 0.121094, 0, 11.994532, 0.158138, 0, 11.903646, 0.121094, 0, 11.893881, 0.169857, 0, 11.994532, 0.158138, 0, 11.994532, 0.158138, 0, 11.903646, 0.121094, 0, 12.004622, 0.130160, 0, 11.903646, 0.121094, 0, 11.994532, 0.158138, 0, 12.004622, 0.130160, 0, 11.903646, 0.121094, 0, 11.919922, 0.079492, 0, 12.004622, 0.130160, 0, 11.919922, 0.079492, 0, 11.903646, 0.121094, 0, 12.004622, 0.130160, 0, 12.004622, 0.130160, 0, 11.919922, 0.079492, 0, 12.018750, 0.107292, 0, 11.919922, 0.079492, 0, 12.004622, 0.130160, 0, 12.018750, 0.107292, 0, 11.919922, 0.079492, 0, 11.942709, 0.045052, 0, 12.018750, 0.107292, 0, 11.942709, 0.045052, 0, 11.919922, 0.079492, 0, 12.018750, 0.107292, 0, 12.018750, 0.107292, 0, 11.942709, 0.045052, 0, 11.971778, 0.018050, 0, 11.942709, 0.045052, 0, 12.018750, 0.107292, 0, 11.971778, 0.018050, 0, 12.036914, 0.089518, 0, 12.018750, 0.107292, 0, 11.971778, 0.018050, 0, 12.018750, 0.107292, 0, 12.036914, 0.089518, 0, 11.971778, 0.018050, 0, 12.036914, 0.089518, 0, 11.971778, 0.018050, 0, 12.006901, -0.001237, 0, 11.971778, 0.018050, 0, 12.036914, 0.089518, 0, 12.006901, -0.001237, 0, 12.059115, 0.076823, 0, 12.036914, 0.089518, 0, 12.006901, -0.001237, 0, 12.036914, 0.089518, 0, 12.059115, 0.076823, 0, 12.006901, -0.001237, 0, 12.006901, -0.001237, 0, 12.048079, -0.012809, 0, 12.059115, 0.076823, 0, 12.048079, -0.012809, 0, 12.006901, -0.001237, 0, 12.059115, 0.076823, 0, 12.059115, 0.076823, 0, 12.048079, -0.012809, 0, 12.085352, 0.069206, 0, 12.048079, -0.012809, 0, 12.059115, 0.076823, 0, 12.085352, 0.069206, 0, 12.085352, 0.069206, 0, 12.048079, -0.012809, 0, 12.095313, -0.016667, 0, 12.048079, -0.012809, 0, 12.085352, 0.069206, 0, 12.095313, -0.016667, 0, 12.085352, 0.069206, 0, 12.095313, -0.016667, 0, 12.115625, 0.066667, 0, 12.095313, -0.016667, 0, 12.085352, 0.069206, 0, 12.115625, 0.066667, 0, 12.115625, 0.066667, 0, 12.095313, -0.016667, 0, 12.124854, -0.015129, 0, 12.095313, -0.016667, 0, 12.115625, 0.066667, 0, 12.124854, -0.015129, 0, 12.115625, 0.066667, 0, 12.124854, -0.015129, 0, 12.152539, -0.010514, 0, 12.124854, -0.015129, 0, 12.115625, 0.066667, 0, 12.152539, -0.010514, 0, 12.152392, 0.069718, 0, 12.115625, 0.066667, 0, 12.152539, -0.010514, 0, 12.115625, 0.066667, 0, 12.152392, 0.069718, 0, 12.152539, -0.010514, 0, 12.152392, 0.069718, 0, 12.152539, -0.010514, 0, 12.178370, -0.002824, 0, 12.152539, -0.010514, 0, 12.152392, 0.069718, 0, 12.178370, -0.002824, 0, 12.152392, 0.069718, 0, 12.178370, -0.002824, 0, 12.202344, 0.007943, 0, 12.178370, -0.002824, 0, 12.152392, 0.069718, 0, 12.202344, 0.007943, 0, 12.185092, 0.078874, 0, 12.152392, 0.069718, 0, 12.202344, 0.007943, 0, 12.152392, 0.069718, 0, 12.185092, 0.078874, 0, 12.202344, 0.007943, 0, 12.185092, 0.078874, 0, 12.202344, 0.007943, 0, 12.224691, 0.021883, 0, 12.202344, 0.007943, 0, 12.185092, 0.078874, 0, 12.224691, 0.021883, 0, 12.185092, 0.078874, 0, 12.224691, 0.021883, 0, 12.245639, 0.039095, 0, 12.224691, 0.021883, 0, 12.185092, 0.078874, 0, 12.245639, 0.039095, 0, 12.213721, 0.094132, 0, 12.185092, 0.078874, 0, 12.245639, 0.039095, 0, 12.185092, 0.078874, 0, 12.213721, 0.094132, 0, 12.245639, 0.039095, 0, 12.213721, 0.094132, 0, 12.245639, 0.039095, 0, 12.265185, 0.059578, 0, 12.245639, 0.039095, 0, 12.213721, 0.094132, 0, 12.265185, 0.059578, 0, 12.238281, 0.115495, 0, 12.213721, 0.094132, 0, 12.265185, 0.059578, 0, 12.213721, 0.094132, 0, 12.238281, 0.115495, 0, 12.265185, 0.059578, 0, 12.238281, 0.115495, 0, 12.265185, 0.059578, 0, 12.283334, 0.083333, 0, 12.265185, 0.059578, 0, 12.238281, 0.115495, 0, 12.283334, 0.083333, 0, 12.257992, 0.142114, 0, 12.238281, 0.115495, 0, 12.283334, 0.083333, 0, 12.238281, 0.115495, 0, 12.257992, 0.142114, 0, 12.283334, 0.083333, 0, 12.272071, 0.173145, 0, 12.257992, 0.142114, 0, 12.283334, 0.083333, 0, 12.257992, 0.142114, 0, 12.272071, 0.173145, 0, 12.283334, 0.083333, 0, 12.283334, 0.083333, 0, 12.379167, 0, 0, 12.272071, 0.173145, 0, 12.379167, 0, 0, 12.283334, 0.083333, 0, 12.272071, 0.173145, 0, 12.283334, 0.083333, 0, 12.283334, 0, 0, 12.379167, 0, 0, 12.283334, 0, 0, 12.283334, 0.083333, 0, 12.379167, 0, 0, 12.379167, 0, 0, 12.280518, 0.208586, 0, 12.272071, 0.173145, 0, 12.280518, 0.208586, 0, 12.379167, 0, 0, 12.272071, 0.173145, 0, 12.283334, 0.248438, 0, 12.280518, 0.208586, 0, 12.379167, 0, 0, 12.280518, 0.208586, 0, 12.283334, 0.248438, 0, 12.379167, 0, 0, 12.379167, 0.583333, 0, 12.283334, 0.248438, 0, 12.379167, 0, 0, 12.283334, 0.248438, 0, 12.379167, 0.583333, 0, 12.379167, 0, 0, 12.379167, 0.583333, 0, 12.283334, 0.583333, 0, 12.283334, 0.248438, 0, 12.283334, 0.583333, 0, 12.379167, 0.583333, 0, 12.283334, 0.248438, 0, 12.861458, 0.583333, 0, 12.957293, 0.583333, 0, 12.861458, 0.583333, 0, 12.861458, 0.583333, 0, 12.957293, 0.583333, 0, 12.957293, 0.583333, 0, 12.957293, 0.583333, 0, 13.077083, 0.128125, 0, 12.957293, 0.583333, 0, 12.957293, 0.583333, 0, 13.077083, 0.128125, 0, 13.077083, 0.128125, 0, 13.077083, 0.128125, 0, 13.196355, 0.583333, 0, 13.077083, 0.128125, 0, 13.077083, 0.128125, 0, 13.196355, 0.583333, 0, 13.196355, 0.583333, 0, 13.196355, 0.583333, 0, 13.309375, 0.583333, 0, 13.196355, 0.583333, 0, 13.196355, 0.583333, 0, 13.309375, 0.583333, 0, 13.309375, 0.583333, 0, 13.309375, 0.583333, 0, 13.429168, 0.128125, 0, 13.309375, 0.583333, 0, 13.309375, 0.583333, 0, 13.429168, 0.128125, 0, 13.429168, 0.128125, 0, 13.429168, 0.128125, 0, 13.548438, 0.583333, 0, 13.429168, 0.128125, 0, 13.429168, 0.128125, 0, 13.548438, 0.583333, 0, 13.548438, 0.583333, 0, 13.548438, 0.583333, 0, 13.644271, 0.583333, 0, 13.548438, 0.583333, 0, 13.548438, 0.583333, 0, 13.644271, 0.583333, 0, 13.644271, 0.583333, 0, 13.644271, 0.583333, 0, 13.491668, 0, 0, 13.644271, 0.583333, 0, 13.644271, 0.583333, 0, 13.491668, 0, 0, 13.491668, 0, 0, 13.491668, 0, 0, 13.378646, 0, 0, 13.491668, 0, 0, 13.491668, 0, 0, 13.378646, 0, 0, 13.378646, 0, 0, 13.378646, 0, 0, 13.253125, 0.478125, 0, 13.378646, 0, 0, 13.378646, 0, 0, 13.253125, 0.478125, 0, 13.253125, 0.478125, 0, 13.253125, 0.478125, 0, 13.127083, 0, 0, 13.253125, 0.478125, 0, 13.253125, 0.478125, 0, 13.127083, 0, 0, 13.127083, 0, 0, 13.127083, 0, 0, 13.014063, 0, 0, 13.127083, 0, 0, 13.127083, 0, 0, 13.014063, 0, 0, 13.014063, 0, 0, 12.957293, 0.583333, 0, 12.861458, 0.583333, 0, 13.077083, 0.128125, 0, 12.861458, 0.583333, 0, 12.957293, 0.583333, 0, 13.077083, 0.128125, 0, 12.861458, 0.583333, 0, 13.014063, 0, 0, 13.077083, 0.128125, 0, 13.014063, 0, 0, 12.861458, 0.583333, 0, 13.077083, 0.128125, 0, 13.077083, 0.128125, 0, 13.014063, 0, 0, 13.127083, 0, 0, 13.014063, 0, 0, 13.077083, 0.128125, 0, 13.127083, 0, 0, 13.253125, 0.478125, 0, 13.077083, 0.128125, 0, 13.127083, 0, 0, 13.077083, 0.128125, 0, 13.253125, 0.478125, 0, 13.127083, 0, 0, 13.253125, 0.478125, 0, 13.196355, 0.583333, 0, 13.077083, 0.128125, 0, 13.196355, 0.583333, 0, 13.253125, 0.478125, 0, 13.077083, 0.128125, 0, 13.196355, 0.583333, 0, 13.253125, 0.478125, 0, 13.309375, 0.583333, 0, 13.253125, 0.478125, 0, 13.196355, 0.583333, 0, 13.309375, 0.583333, 0, 13.309375, 0.583333, 0, 13.253125, 0.478125, 0, 13.429168, 0.128125, 0, 13.253125, 0.478125, 0, 13.309375, 0.583333, 0, 13.429168, 0.128125, 0, 13.429168, 0.128125, 0, 13.253125, 0.478125, 0, 13.378646, 0, 0, 13.253125, 0.478125, 0, 13.429168, 0.128125, 0, 13.378646, 0, 0, 13.429168, 0.128125, 0, 13.378646, 0, 0, 13.491668, 0, 0, 13.378646, 0, 0, 13.429168, 0.128125, 0, 13.491668, 0, 0, 13.491668, 0, 0, 13.644271, 0.583333, 0, 13.429168, 0.128125, 0, 13.644271, 0.583333, 0, 13.491668, 0, 0, 13.429168, 0.128125, 0, 13.644271, 0.583333, 0, 13.548438, 0.583333, 0, 13.429168, 0.128125, 0, 13.548438, 0.583333, 0, 13.644271, 0.583333, 0, 13.429168, 0.128125, 0, 13.783854, 0.583333, 0, 13.879687, 0.583333, 0, 13.783854, 0.583333, 0, 13.783854, 0.583333, 0, 13.879687, 0.583333, 0, 13.879687, 0.583333, 0, 13.879687, 0.583333, 0, 13.879687, 0, 0, 13.879687, 0.583333, 0, 13.879687, 0.583333, 0, 13.879687, 0, 0, 13.879687, 0, 0, 13.879687, 0, 0, 13.783854, 0, 0, 13.879687, 0, 0, 13.879687, 0, 0, 13.783854, 0, 0, 13.783854, 0, 0, 13.783854, 0.583333, 0, 13.879687, 0, 0, 13.879687, 0.583333, 0, 13.879687, 0, 0, 13.783854, 0.583333, 0, 13.879687, 0.583333, 0, 13.783854, 0.583333, 0, 13.783854, 0, 0, 13.879687, 0, 0, 13.783854, 0, 0, 13.783854, 0.583333, 0, 13.879687, 0, 0, 13.783854, 0.816667, 0, 13.879687, 0.816667, 0, 13.783854, 0.816667, 0, 13.783854, 0.816667, 0, 13.879687, 0.816667, 0, 13.879687, 0.816667, 0, 13.879687, 0.816667, 0, 13.879687, 0.700000, 0, 13.879687, 0.816667, 0, 13.879687, 0.816667, 0, 13.879687, 0.700000, 0, 13.879687, 0.700000, 0, 13.879687, 0.700000, 0, 13.783854, 0.700000, 0, 13.879687, 0.700000, 0, 13.879687, 0.700000, 0, 13.783854, 0.700000, 0, 13.783854, 0.700000, 0, 13.783854, 0.816667, 0, 13.879687, 0.700000, 0, 13.879687, 0.816667, 0, 13.879687, 0.700000, 0, 13.783854, 0.816667, 0, 13.879687, 0.816667, 0, 13.783854, 0.816667, 0, 13.783854, 0.700000, 0, 13.879687, 0.700000, 0, 13.783854, 0.700000, 0, 13.783854, 0.816667, 0, 13.879687, 0.700000, 0, 14.083855, 0.816667, 0, 14.179688, 0.816667, 0, 14.083855, 0.816667, 0, 14.083855, 0.816667, 0, 14.179688, 0.816667, 0, 14.179688, 0.816667, 0, 14.179688, 0.816667, 0, 14.179688, 0, 0, 14.179688, 0.816667, 0, 14.179688, 0.816667, 0, 14.179688, 0, 0, 14.179688, 0, 0, 14.179688, 0, 0, 14.083855, 0, 0, 14.179688, 0, 0, 14.179688, 0, 0, 14.083855, 0, 0, 14.083855, 0, 0, 14.083855, 0.816667, 0, 14.179688, 0, 0, 14.179688, 0.816667, 0, 14.179688, 0, 0, 14.083855, 0.816667, 0, 14.179688, 0.816667, 0, 14.083855, 0.816667, 0, 14.083855, 0, 0, 14.179688, 0, 0, 14.083855, 0, 0, 14.083855, 0.816667, 0, 14.179688, 0, 0, 14.383855, 0.816667, 0, 14.479688, 0.816667, 0, 14.383855, 0.816667, 0, 14.383855, 0.816667, 0, 14.479688, 0.816667, 0, 14.479688, 0.816667, 0, 14.479688, 0.816667, 0, 14.479688, 0, 0, 14.479688, 0.816667, 0, 14.479688, 0.816667, 0, 14.479688, 0, 0, 14.479688, 0, 0, 14.479688, 0, 0, 14.383855, 0, 0, 14.479688, 0, 0, 14.479688, 0, 0, 14.383855, 0, 0, 14.383855, 0, 0, 14.383855, 0.816667, 0, 14.479688, 0, 0, 14.479688, 0.816667, 0, 14.479688, 0, 0, 14.383855, 0.816667, 0, 14.479688, 0.816667, 0, 14.383855, 0.816667, 0, 14.383855, 0, 0, 14.479688, 0, 0, 14.383855, 0, 0, 14.383855, 0.816667, 0, 14.479688, 0, 0, 15.435938, 0.291667, 0, 15.433220, 0.341439, 0, 15.435938, 0.291667, 0, 15.435938, 0.291667, 0, 15.433220, 0.341439, 0, 15.433220, 0.341439, 0, 15.433220, 0.341439, 0, 15.425066, 0.385547, 0, 15.433220, 0.341439, 0, 15.433220, 0.341439, 0, 15.425066, 0.385547, 0, 15.425066, 0.385547, 0, 15.425066, 0.385547, 0, 15.411475, 0.423991, 0, 15.425066, 0.385547, 0, 15.425066, 0.385547, 0, 15.411475, 0.423991, 0, 15.411475, 0.423991, 0, 15.411475, 0.423991, 0, 15.392448, 0.456771, 0, 15.411475, 0.423991, 0, 15.411475, 0.423991, 0, 15.392448, 0.456771, 0, 15.392448, 0.456771, 0, 15.392448, 0.456771, 0, 15.368670, 0.482975, 0, 15.392448, 0.456771, 0, 15.392448, 0.456771, 0, 15.368670, 0.482975, 0, 15.368670, 0.482975, 0, 15.368670, 0.482975, 0, 15.340820, 0.501693, 0, 15.368670, 0.482975, 0, 15.368670, 0.482975, 0, 15.340820, 0.501693, 0, 15.340820, 0.501693, 0, 15.340820, 0.501693, 0, 15.308904, 0.512923, 0, 15.340820, 0.501693, 0, 15.340820, 0.501693, 0, 15.308904, 0.512923, 0, 15.308904, 0.512923, 0, 15.308904, 0.512923, 0, 15.272918, 0.516667, 0, 15.308904, 0.512923, 0, 15.308904, 0.512923, 0, 15.272918, 0.516667, 0, 15.272918, 0.516667, 0, 15.272918, 0.516667, 0, 15.236931, 0.512923, 0, 15.272918, 0.516667, 0, 15.272918, 0.516667, 0, 15.236931, 0.512923, 0, 15.236931, 0.512923, 0, 15.236931, 0.512923, 0, 15.205012, 0.501693, 0, 15.236931, 0.512923, 0, 15.236931, 0.512923, 0, 15.205012, 0.501693, 0, 15.205012, 0.501693, 0, 15.205012, 0.501693, 0, 15.177165, 0.482975, 0, 15.205012, 0.501693, 0, 15.205012, 0.501693, 0, 15.177165, 0.482975, 0, 15.177165, 0.482975, 0, 15.177165, 0.482975, 0, 15.153386, 0.456771, 0, 15.177165, 0.482975, 0, 15.177165, 0.482975, 0, 15.153386, 0.456771, 0, 15.153386, 0.456771, 0, 15.153386, 0.456771, 0, 15.134358, 0.423991, 0, 15.153386, 0.456771, 0, 15.153386, 0.456771, 0, 15.134358, 0.423991, 0, 15.134358, 0.423991, 0, 15.134358, 0.423991, 0, 15.120769, 0.385547, 0, 15.134358, 0.423991, 0, 15.134358, 0.423991, 0, 15.120769, 0.385547, 0, 15.120769, 0.385547, 0, 15.120769, 0.385547, 0, 15.112615, 0.341439, 0, 15.120769, 0.385547, 0, 15.120769, 0.385547, 0, 15.112615, 0.341439, 0, 15.112615, 0.341439, 0, 15.112615, 0.341439, 0, 15.109896, 0.291667, 0, 15.112615, 0.341439, 0, 15.112615, 0.341439, 0, 15.109896, 0.291667, 0, 15.109896, 0.291667, 0, 15.109896, 0.291667, 0, 15.112615, 0.241895, 0, 15.109896, 0.291667, 0, 15.109896, 0.291667, 0, 15.112615, 0.241895, 0, 15.112615, 0.241895, 0, 15.112615, 0.241895, 0, 15.120769, 0.197786, 0, 15.112615, 0.241895, 0, 15.112615, 0.241895, 0, 15.120769, 0.197786, 0, 15.120769, 0.197786, 0, 15.120769, 0.197786, 0, 15.134358, 0.159342, 0, 15.120769, 0.197786, 0, 15.120769, 0.197786, 0, 15.134358, 0.159342, 0, 15.134358, 0.159342, 0, 15.134358, 0.159342, 0, 15.153386, 0.126563, 0, 15.134358, 0.159342, 0, 15.134358, 0.159342, 0, 15.153386, 0.126563, 0, 15.153386, 0.126563, 0, 15.153386, 0.126563, 0, 15.177165, 0.100358, 0, 15.153386, 0.126563, 0, 15.153386, 0.126563, 0, 15.177165, 0.100358, 0, 15.177165, 0.100358, 0, 15.177165, 0.100358, 0, 15.205012, 0.081641, 0, 15.177165, 0.100358, 0, 15.177165, 0.100358, 0, 15.205012, 0.081641, 0, 15.205012, 0.081641, 0, 15.205012, 0.081641, 0, 15.236931, 0.070410, 0, 15.205012, 0.081641, 0, 15.205012, 0.081641, 0, 15.236931, 0.070410, 0, 15.236931, 0.070410, 0, 15.236931, 0.070410, 0, 15.272918, 0.066667, 0, 15.236931, 0.070410, 0, 15.236931, 0.070410, 0, 15.272918, 0.066667, 0, 15.272918, 0.066667, 0, 15.272918, 0.066667, 0, 15.308904, 0.070410, 0, 15.272918, 0.066667, 0, 15.272918, 0.066667, 0, 15.308904, 0.070410, 0, 15.308904, 0.070410, 0, 15.308904, 0.070410, 0, 15.340820, 0.081641, 0, 15.308904, 0.070410, 0, 15.308904, 0.070410, 0, 15.340820, 0.081641, 0, 15.340820, 0.081641, 0, 15.340820, 0.081641, 0, 15.368670, 0.100358, 0, 15.340820, 0.081641, 0, 15.340820, 0.081641, 0, 15.368670, 0.100358, 0, 15.368670, 0.100358, 0, 15.368670, 0.100358, 0, 15.392448, 0.126563, 0, 15.368670, 0.100358, 0, 15.368670, 0.100358, 0, 15.392448, 0.126563, 0, 15.392448, 0.126563, 0, 15.392448, 0.126563, 0, 15.411475, 0.159342, 0, 15.392448, 0.126563, 0, 15.392448, 0.126563, 0, 15.411475, 0.159342, 0, 15.411475, 0.159342, 0, 15.411475, 0.159342, 0, 15.425066, 0.197786, 0, 15.411475, 0.159342, 0, 15.411475, 0.159342, 0, 15.425066, 0.197786, 0, 15.425066, 0.197786, 0, 15.425066, 0.197786, 0, 15.433220, 0.241895, 0, 15.425066, 0.197786, 0, 15.425066, 0.197786, 0, 15.433220, 0.241895, 0, 15.433220, 0.241895, 0, 15.109896, 0.500000, 0, 15.125994, 0.523755, 0, 15.109896, 0.500000, 0, 15.109896, 0.500000, 0, 15.125994, 0.523755, 0, 15.125994, 0.523755, 0, 15.125994, 0.523755, 0, 15.144075, 0.544238, 0, 15.125994, 0.523755, 0, 15.125994, 0.523755, 0, 15.144075, 0.544238, 0, 15.144075, 0.544238, 0, 15.144075, 0.544238, 0, 15.164144, 0.561450, 0, 15.144075, 0.544238, 0, 15.144075, 0.544238, 0, 15.164144, 0.561450, 0, 15.164144, 0.561450, 0, 15.164144, 0.561450, 0, 15.186198, 0.575391, 0, 15.164144, 0.561450, 0, 15.164144, 0.561450, 0, 15.186198, 0.575391, 0, 15.186198, 0.575391, 0, 15.186198, 0.575391, 0, 15.210368, 0.586157, 0, 15.186198, 0.575391, 0, 15.186198, 0.575391, 0, 15.210368, 0.586157, 0, 15.210368, 0.586157, 0, 15.210368, 0.586157, 0, 15.236785, 0.593848, 0, 15.210368, 0.586157, 0, 15.210368, 0.586157, 0, 15.236785, 0.593848, 0, 15.236785, 0.593848, 0, 15.236785, 0.593848, 0, 15.265447, 0.598462, 0, 15.236785, 0.593848, 0, 15.236785, 0.593848, 0, 15.265447, 0.598462, 0, 15.265447, 0.598462, 0, 15.265447, 0.598462, 0, 15.296355, 0.600000, 0, 15.265447, 0.598462, 0, 15.265447, 0.598462, 0, 15.296355, 0.600000, 0, 15.296355, 0.600000, 0, 15.296355, 0.600000, 0, 15.346990, 0.594694, 0, 15.296355, 0.600000, 0, 15.296355, 0.600000, 0, 15.346990, 0.594694, 0, 15.346990, 0.594694, 0, 15.346990, 0.594694, 0, 15.392644, 0.578776, 0, 15.346990, 0.594694, 0, 15.346990, 0.594694, 0, 15.392644, 0.578776, 0, 15.392644, 0.578776, 0, 15.392644, 0.578776, 0, 15.433318, 0.552246, 0, 15.392644, 0.578776, 0, 15.392644, 0.578776, 0, 15.433318, 0.552246, 0, 15.433318, 0.552246, 0, 15.433318, 0.552246, 0, 15.469011, 0.515104, 0, 15.433318, 0.552246, 0, 15.433318, 0.552246, 0, 15.469011, 0.515104, 0, 15.469011, 0.515104, 0, 15.469011, 0.515104, 0, 15.498064, 0.469303, 0, 15.469011, 0.515104, 0, 15.469011, 0.515104, 0, 15.498064, 0.469303, 0, 15.498064, 0.469303, 0, 15.498064, 0.469303, 0, 15.518816, 0.416797, 0, 15.498064, 0.469303, 0, 15.498064, 0.469303, 0, 15.518816, 0.416797, 0, 15.518816, 0.416797, 0, 15.518816, 0.416797, 0, 15.531267, 0.357585, 0, 15.518816, 0.416797, 0, 15.518816, 0.416797, 0, 15.531267, 0.357585, 0, 15.531267, 0.357585, 0, 15.531267, 0.357585, 0, 15.535418, 0.291667, 0, 15.531267, 0.357585, 0, 15.531267, 0.357585, 0, 15.535418, 0.291667, 0, 15.535418, 0.291667, 0, 15.535418, 0.291667, 0, 15.531267, 0.225749, 0, 15.535418, 0.291667, 0, 15.535418, 0.291667, 0, 15.531267, 0.225749, 0, 15.531267, 0.225749, 0, 15.531267, 0.225749, 0, 15.518816, 0.166536, 0, 15.531267, 0.225749, 0, 15.531267, 0.225749, 0, 15.518816, 0.166536, 0, 15.518816, 0.166536, 0, 15.518816, 0.166536, 0, 15.498064, 0.114030, 0, 15.518816, 0.166536, 0, 15.518816, 0.166536, 0, 15.498064, 0.114030, 0, 15.498064, 0.114030, 0, 15.498064, 0.114030, 0, 15.469011, 0.068229, 0, 15.498064, 0.114030, 0, 15.498064, 0.114030, 0, 15.469011, 0.068229, 0, 15.469011, 0.068229, 0, 15.469011, 0.068229, 0, 15.433318, 0.031087, 0, 15.469011, 0.068229, 0, 15.469011, 0.068229, 0, 15.433318, 0.031087, 0, 15.433318, 0.031087, 0, 15.433318, 0.031087, 0, 15.392644, 0.004557, 0, 15.433318, 0.031087, 0, 15.433318, 0.031087, 0, 15.392644, 0.004557, 0, 15.392644, 0.004557, 0, 15.392644, 0.004557, 0, 15.346990, -0.011361, 0, 15.392644, 0.004557, 0, 15.392644, 0.004557, 0, 15.346990, -0.011361, 0, 15.346990, -0.011361, 0, 15.346990, -0.011361, 0, 15.296355, -0.016667, 0, 15.346990, -0.011361, 0, 15.346990, -0.011361, 0, 15.296355, -0.016667, 0, 15.296355, -0.016667, 0, 15.296355, -0.016667, 0, 15.265447, -0.015129, 0, 15.296355, -0.016667, 0, 15.296355, -0.016667, 0, 15.265447, -0.015129, 0, 15.265447, -0.015129, 0, 15.265447, -0.015129, 0, 15.236785, -0.010514, 0, 15.265447, -0.015129, 0, 15.265447, -0.015129, 0, 15.236785, -0.010514, 0, 15.236785, -0.010514, 0, 15.236785, -0.010514, 0, 15.210368, -0.002824, 0, 15.236785, -0.010514, 0, 15.236785, -0.010514, 0, 15.210368, -0.002824, 0, 15.210368, -0.002824, 0, 15.210368, -0.002824, 0, 15.186198, 0.007943, 0, 15.210368, -0.002824, 0, 15.210368, -0.002824, 0, 15.186198, 0.007943, 0, 15.186198, 0.007943, 0, 15.186198, 0.007943, 0, 15.164144, 0.021883, 0, 15.186198, 0.007943, 0, 15.186198, 0.007943, 0, 15.164144, 0.021883, 0, 15.164144, 0.021883, 0, 15.164144, 0.021883, 0, 15.144075, 0.039095, 0, 15.164144, 0.021883, 0, 15.164144, 0.021883, 0, 15.144075, 0.039095, 0, 15.144075, 0.039095, 0, 15.144075, 0.039095, 0, 15.125994, 0.059578, 0, 15.144075, 0.039095, 0, 15.144075, 0.039095, 0, 15.125994, 0.059578, 0, 15.125994, 0.059578, 0, 15.125994, 0.059578, 0, 15.109896, 0.083333, 0, 15.125994, 0.059578, 0, 15.125994, 0.059578, 0, 15.109896, 0.083333, 0, 15.109896, 0.083333, 0, 15.109896, 0.083333, 0, 15.109896, 0, 0, 15.109896, 0.083333, 0, 15.109896, 0.083333, 0, 15.109896, 0, 0, 15.109896, 0, 0, 15.109896, 0, 0, 15.013542, 0, 0, 15.109896, 0, 0, 15.109896, 0, 0, 15.013542, 0, 0, 15.013542, 0, 0, 15.013542, 0, 0, 15.013542, 0.816667, 0, 15.013542, 0, 0, 15.013542, 0, 0, 15.013542, 0.816667, 0, 15.013542, 0.816667, 0, 15.013542, 0.816667, 0, 15.109896, 0.816667, 0, 15.013542, 0.816667, 0, 15.013542, 0.816667, 0, 15.109896, 0.816667, 0, 15.109896, 0.816667, 0, 15.109896, 0.816667, 0, 15.013542, 0.816667, 0, 15.109896, 0.500000, 0, 15.013542, 0.816667, 0, 15.109896, 0.816667, 0, 15.109896, 0.500000, 0, 15.013542, 0.816667, 0, 15.013542, 0, 0, 15.109896, 0.500000, 0, 15.013542, 0, 0, 15.013542, 0.816667, 0, 15.109896, 0.500000, 0, 15.109896, 0.500000, 0, 15.013542, 0, 0, 15.112615, 0.341439, 0, 15.013542, 0, 0, 15.109896, 0.500000, 0, 15.112615, 0.341439, 0, 15.109896, 0.500000, 0, 15.112615, 0.341439, 0, 15.120769, 0.385547, 0, 15.112615, 0.341439, 0, 15.109896, 0.500000, 0, 15.120769, 0.385547, 0, 15.109896, 0.500000, 0, 15.120769, 0.385547, 0, 15.134358, 0.423991, 0, 15.120769, 0.385547, 0, 15.109896, 0.500000, 0, 15.134358, 0.423991, 0, 15.109896, 0.500000, 0, 15.134358, 0.423991, 0, 15.153386, 0.456771, 0, 15.134358, 0.423991, 0, 15.109896, 0.500000, 0, 15.153386, 0.456771, 0, 15.109896, 0.500000, 0, 15.153386, 0.456771, 0, 15.177165, 0.482975, 0, 15.153386, 0.456771, 0, 15.109896, 0.500000, 0, 15.177165, 0.482975, 0, 15.125994, 0.523755, 0, 15.109896, 0.500000, 0, 15.177165, 0.482975, 0, 15.109896, 0.500000, 0, 15.125994, 0.523755, 0, 15.177165, 0.482975, 0, 15.144075, 0.544238, 0, 15.125994, 0.523755, 0, 15.177165, 0.482975, 0, 15.125994, 0.523755, 0, 15.144075, 0.544238, 0, 15.177165, 0.482975, 0, 15.144075, 0.544238, 0, 15.177165, 0.482975, 0, 15.205012, 0.501693, 0, 15.177165, 0.482975, 0, 15.144075, 0.544238, 0, 15.205012, 0.501693, 0, 15.164144, 0.561450, 0, 15.144075, 0.544238, 0, 15.205012, 0.501693, 0, 15.144075, 0.544238, 0, 15.164144, 0.561450, 0, 15.205012, 0.501693, 0, 15.186198, 0.575391, 0, 15.164144, 0.561450, 0, 15.205012, 0.501693, 0, 15.164144, 0.561450, 0, 15.186198, 0.575391, 0, 15.205012, 0.501693, 0, 15.186198, 0.575391, 0, 15.205012, 0.501693, 0, 15.236931, 0.512923, 0, 15.205012, 0.501693, 0, 15.186198, 0.575391, 0, 15.236931, 0.512923, 0, 15.210368, 0.586157, 0, 15.186198, 0.575391, 0, 15.236931, 0.512923, 0, 15.186198, 0.575391, 0, 15.210368, 0.586157, 0, 15.236931, 0.512923, 0, 15.236785, 0.593848, 0, 15.210368, 0.586157, 0, 15.236931, 0.512923, 0, 15.210368, 0.586157, 0, 15.236785, 0.593848, 0, 15.236931, 0.512923, 0, 15.236785, 0.593848, 0, 15.236931, 0.512923, 0, 15.272918, 0.516667, 0, 15.236931, 0.512923, 0, 15.236785, 0.593848, 0, 15.272918, 0.516667, 0, 15.265447, 0.598462, 0, 15.236785, 0.593848, 0, 15.272918, 0.516667, 0, 15.236785, 0.593848, 0, 15.265447, 0.598462, 0, 15.272918, 0.516667, 0, 15.296355, 0.600000, 0, 15.265447, 0.598462, 0, 15.272918, 0.516667, 0, 15.265447, 0.598462, 0, 15.296355, 0.600000, 0, 15.272918, 0.516667, 0, 15.296355, 0.600000, 0, 15.272918, 0.516667, 0, 15.308904, 0.512923, 0, 15.272918, 0.516667, 0, 15.296355, 0.600000, 0, 15.308904, 0.512923, 0, 15.346990, 0.594694, 0, 15.296355, 0.600000, 0, 15.308904, 0.512923, 0, 15.296355, 0.600000, 0, 15.346990, 0.594694, 0, 15.308904, 0.512923, 0, 15.308904, 0.512923, 0, 15.340820, 0.501693, 0, 15.346990, 0.594694, 0, 15.340820, 0.501693, 0, 15.308904, 0.512923, 0, 15.346990, 0.594694, 0, 15.346990, 0.594694, 0, 15.340820, 0.501693, 0, 15.392644, 0.578776, 0, 15.340820, 0.501693, 0, 15.346990, 0.594694, 0, 15.392644, 0.578776, 0, 15.340820, 0.501693, 0, 15.368670, 0.482975, 0, 15.392644, 0.578776, 0, 15.368670, 0.482975, 0, 15.340820, 0.501693, 0, 15.392644, 0.578776, 0, 15.392644, 0.578776, 0, 15.368670, 0.482975, 0, 15.433318, 0.552246, 0, 15.368670, 0.482975, 0, 15.392644, 0.578776, 0, 15.433318, 0.552246, 0, 15.368670, 0.482975, 0, 15.392448, 0.456771, 0, 15.433318, 0.552246, 0, 15.392448, 0.456771, 0, 15.368670, 0.482975, 0, 15.433318, 0.552246, 0, 15.433318, 0.552246, 0, 15.392448, 0.456771, 0, 15.469011, 0.515104, 0, 15.392448, 0.456771, 0, 15.433318, 0.552246, 0, 15.469011, 0.515104, 0, 15.469011, 0.515104, 0, 15.392448, 0.456771, 0, 15.498064, 0.469303, 0, 15.392448, 0.456771, 0, 15.469011, 0.515104, 0, 15.498064, 0.469303, 0, 15.498064, 0.469303, 0, 15.392448, 0.456771, 0, 15.411475, 0.423991, 0, 15.392448, 0.456771, 0, 15.498064, 0.469303, 0, 15.411475, 0.423991, 0, 15.498064, 0.469303, 0, 15.411475, 0.423991, 0, 15.518816, 0.416797, 0, 15.411475, 0.423991, 0, 15.498064, 0.469303, 0, 15.518816, 0.416797, 0, 15.518816, 0.416797, 0, 15.411475, 0.423991, 0, 15.425066, 0.385547, 0, 15.411475, 0.423991, 0, 15.518816, 0.416797, 0, 15.425066, 0.385547, 0, 15.518816, 0.416797, 0, 15.425066, 0.385547, 0, 15.531267, 0.357585, 0, 15.425066, 0.385547, 0, 15.518816, 0.416797, 0, 15.531267, 0.357585, 0, 15.531267, 0.357585, 0, 15.425066, 0.385547, 0, 15.433220, 0.341439, 0, 15.425066, 0.385547, 0, 15.531267, 0.357585, 0, 15.433220, 0.341439, 0, 15.531267, 0.357585, 0, 15.433220, 0.341439, 0, 15.535418, 0.291667, 0, 15.433220, 0.341439, 0, 15.531267, 0.357585, 0, 15.535418, 0.291667, 0, 15.535418, 0.291667, 0, 15.433220, 0.341439, 0, 15.435938, 0.291667, 0, 15.433220, 0.341439, 0, 15.535418, 0.291667, 0, 15.435938, 0.291667, 0, 15.535418, 0.291667, 0, 15.435938, 0.291667, 0, 15.433220, 0.241895, 0, 15.435938, 0.291667, 0, 15.535418, 0.291667, 0, 15.433220, 0.241895, 0, 15.535418, 0.291667, 0, 15.433220, 0.241895, 0, 15.531267, 0.225749, 0, 15.433220, 0.241895, 0, 15.535418, 0.291667, 0, 15.531267, 0.225749, 0, 15.433220, 0.241895, 0, 15.425066, 0.197786, 0, 15.531267, 0.225749, 0, 15.425066, 0.197786, 0, 15.433220, 0.241895, 0, 15.531267, 0.225749, 0, 15.425066, 0.197786, 0, 15.518816, 0.166536, 0, 15.531267, 0.225749, 0, 15.518816, 0.166536, 0, 15.425066, 0.197786, 0, 15.531267, 0.225749, 0, 15.425066, 0.197786, 0, 15.411475, 0.159342, 0, 15.518816, 0.166536, 0, 15.411475, 0.159342, 0, 15.425066, 0.197786, 0, 15.518816, 0.166536, 0, 15.411475, 0.159342, 0, 15.498064, 0.114030, 0, 15.518816, 0.166536, 0, 15.498064, 0.114030, 0, 15.411475, 0.159342, 0, 15.518816, 0.166536, 0, 15.411475, 0.159342, 0, 15.392448, 0.126563, 0, 15.498064, 0.114030, 0, 15.392448, 0.126563, 0, 15.411475, 0.159342, 0, 15.498064, 0.114030, 0, 15.392448, 0.126563, 0, 15.469011, 0.068229, 0, 15.498064, 0.114030, 0, 15.469011, 0.068229, 0, 15.392448, 0.126563, 0, 15.498064, 0.114030, 0, 15.469011, 0.068229, 0, 15.392448, 0.126563, 0, 15.433318, 0.031087, 0, 15.392448, 0.126563, 0, 15.469011, 0.068229, 0, 15.433318, 0.031087, 0, 15.392448, 0.126563, 0, 15.368670, 0.100358, 0, 15.433318, 0.031087, 0, 15.368670, 0.100358, 0, 15.392448, 0.126563, 0, 15.433318, 0.031087, 0, 15.433318, 0.031087, 0, 15.368670, 0.100358, 0, 15.392644, 0.004557, 0, 15.368670, 0.100358, 0, 15.433318, 0.031087, 0, 15.392644, 0.004557, 0, 15.368670, 0.100358, 0, 15.340820, 0.081641, 0, 15.392644, 0.004557, 0, 15.340820, 0.081641, 0, 15.368670, 0.100358, 0, 15.392644, 0.004557, 0, 15.392644, 0.004557, 0, 15.340820, 0.081641, 0, 15.346990, -0.011361, 0, 15.340820, 0.081641, 0, 15.392644, 0.004557, 0, 15.346990, -0.011361, 0, 15.340820, 0.081641, 0, 15.308904, 0.070410, 0, 15.346990, -0.011361, 0, 15.308904, 0.070410, 0, 15.340820, 0.081641, 0, 15.346990, -0.011361, 0, 15.308904, 0.070410, 0, 15.296355, -0.016667, 0, 15.346990, -0.011361, 0, 15.296355, -0.016667, 0, 15.308904, 0.070410, 0, 15.346990, -0.011361, 0, 15.272918, 0.066667, 0, 15.296355, -0.016667, 0, 15.308904, 0.070410, 0, 15.296355, -0.016667, 0, 15.272918, 0.066667, 0, 15.308904, 0.070410, 0, 15.272918, 0.066667, 0, 15.265447, -0.015129, 0, 15.296355, -0.016667, 0, 15.265447, -0.015129, 0, 15.272918, 0.066667, 0, 15.296355, -0.016667, 0, 15.236785, -0.010514, 0, 15.265447, -0.015129, 0, 15.272918, 0.066667, 0, 15.265447, -0.015129, 0, 15.236785, -0.010514, 0, 15.272918, 0.066667, 0, 15.236931, 0.070410, 0, 15.236785, -0.010514, 0, 15.272918, 0.066667, 0, 15.236785, -0.010514, 0, 15.236931, 0.070410, 0, 15.272918, 0.066667, 0, 15.210368, -0.002824, 0, 15.236785, -0.010514, 0, 15.236931, 0.070410, 0, 15.236785, -0.010514, 0, 15.210368, -0.002824, 0, 15.236931, 0.070410, 0, 15.186198, 0.007943, 0, 15.210368, -0.002824, 0, 15.236931, 0.070410, 0, 15.210368, -0.002824, 0, 15.186198, 0.007943, 0, 15.236931, 0.070410, 0, 15.205012, 0.081641, 0, 15.186198, 0.007943, 0, 15.236931, 0.070410, 0, 15.186198, 0.007943, 0, 15.205012, 0.081641, 0, 15.236931, 0.070410, 0, 15.164144, 0.021883, 0, 15.186198, 0.007943, 0, 15.205012, 0.081641, 0, 15.186198, 0.007943, 0, 15.164144, 0.021883, 0, 15.205012, 0.081641, 0, 15.144075, 0.039095, 0, 15.164144, 0.021883, 0, 15.205012, 0.081641, 0, 15.164144, 0.021883, 0, 15.144075, 0.039095, 0, 15.205012, 0.081641, 0, 15.177165, 0.100358, 0, 15.144075, 0.039095, 0, 15.205012, 0.081641, 0, 15.144075, 0.039095, 0, 15.177165, 0.100358, 0, 15.205012, 0.081641, 0, 15.125994, 0.059578, 0, 15.144075, 0.039095, 0, 15.177165, 0.100358, 0, 15.144075, 0.039095, 0, 15.125994, 0.059578, 0, 15.177165, 0.100358, 0, 15.109896, 0.083333, 0, 15.125994, 0.059578, 0, 15.177165, 0.100358, 0, 15.125994, 0.059578, 0, 15.109896, 0.083333, 0, 15.177165, 0.100358, 0, 15.153386, 0.126563, 0, 15.109896, 0.083333, 0, 15.177165, 0.100358, 0, 15.109896, 0.083333, 0, 15.153386, 0.126563, 0, 15.177165, 0.100358, 0, 15.134358, 0.159342, 0, 15.109896, 0.083333, 0, 15.153386, 0.126563, 0, 15.109896, 0.083333, 0, 15.134358, 0.159342, 0, 15.153386, 0.126563, 0, 15.120769, 0.197786, 0, 15.109896, 0.083333, 0, 15.134358, 0.159342, 0, 15.109896, 0.083333, 0, 15.120769, 0.197786, 0, 15.134358, 0.159342, 0, 15.120769, 0.197786, 0, 15.013542, 0, 0, 15.109896, 0.083333, 0, 15.013542, 0, 0, 15.120769, 0.197786, 0, 15.109896, 0.083333, 0, 15.120769, 0.197786, 0, 15.112615, 0.241895, 0, 15.013542, 0, 0, 15.112615, 0.241895, 0, 15.120769, 0.197786, 0, 15.013542, 0, 0, 15.109896, 0.291667, 0, 15.013542, 0, 0, 15.112615, 0.241895, 0, 15.013542, 0, 0, 15.109896, 0.291667, 0, 15.112615, 0.241895, 0, 15.112615, 0.341439, 0, 15.013542, 0, 0, 15.109896, 0.291667, 0, 15.013542, 0, 0, 15.112615, 0.341439, 0, 15.109896, 0.291667, 0, 15.013542, 0, 0, 15.109896, 0, 0, 15.109896, 0.083333, 0, 15.109896, 0, 0, 15.013542, 0, 0, 15.109896, 0.083333, 0, 16.199480, 0.318750, 0, 16.199480, 0.266667, 0, 16.199480, 0.318750, 0, 16.199480, 0.318750, 0, 16.199480, 0.266667, 0, 16.199480, 0.266667, 0, 16.199480, 0.266667, 0, 15.758855, 0.266667, 0, 16.199480, 0.266667, 0, 16.199480, 0.266667, 0, 15.758855, 0.266667, 0, 15.758855, 0.266667, 0, 15.758855, 0.266667, 0, 15.764925, 0.220744, 0, 15.758855, 0.266667, 0, 15.758855, 0.266667, 0, 15.764925, 0.220744, 0, 15.764925, 0.220744, 0, 15.764925, 0.220744, 0, 15.776888, 0.180632, 0, 15.764925, 0.220744, 0, 15.764925, 0.220744, 0, 15.776888, 0.180632, 0, 15.776888, 0.180632, 0, 15.776888, 0.180632, 0, 15.794744, 0.146330, 0, 15.776888, 0.180632, 0, 15.776888, 0.180632, 0, 15.794744, 0.146330, 0, 15.794744, 0.146330, 0, 15.794744, 0.146330, 0, 15.818489, 0.117839, 0, 15.794744, 0.146330, 0, 15.794744, 0.146330, 0, 15.818489, 0.117839, 0, 15.818489, 0.117839, 0, 15.818489, 0.117839, 0, 15.847803, 0.095451, 0, 15.818489, 0.117839, 0, 15.818489, 0.117839, 0, 15.847803, 0.095451, 0, 15.847803, 0.095451, 0, 15.847803, 0.095451, 0, 15.882357, 0.079460, 0, 15.847803, 0.095451, 0, 15.847803, 0.095451, 0, 15.882357, 0.079460, 0, 15.882357, 0.079460, 0, 15.882357, 0.079460, 0, 15.922152, 0.069865, 0, 15.882357, 0.079460, 0, 15.882357, 0.079460, 0, 15.922152, 0.069865, 0, 15.922152, 0.069865, 0, 15.922152, 0.069865, 0, 15.967188, 0.066667, 0, 15.922152, 0.069865, 0, 15.922152, 0.069865, 0, 15.967188, 0.066667, 0, 15.967188, 0.066667, 0, 15.967188, 0.066667, 0, 15.994580, 0.067448, 0, 15.967188, 0.066667, 0, 15.967188, 0.066667, 0, 15.994580, 0.067448, 0, 15.994580, 0.067448, 0, 15.994580, 0.067448, 0, 16.021551, 0.069792, 0, 15.994580, 0.067448, 0, 15.994580, 0.067448, 0, 16.021551, 0.069792, 0, 16.021551, 0.069792, 0, 16.021551, 0.069792, 0, 16.048096, 0.073698, 0, 16.021551, 0.069792, 0, 16.021551, 0.069792, 0, 16.048096, 0.073698, 0, 16.048096, 0.073698, 0, 16.048096, 0.073698, 0, 16.074219, 0.079167, 0, 16.048096, 0.073698, 0, 16.048096, 0.073698, 0, 16.074219, 0.079167, 0, 16.074219, 0.079167, 0, 16.074219, 0.079167, 0, 16.100080, 0.086198, 0, 16.074219, 0.079167, 0, 16.074219, 0.079167, 0, 16.100080, 0.086198, 0, 16.100080, 0.086198, 0, 16.100080, 0.086198, 0, 16.125847, 0.094792, 0, 16.100080, 0.086198, 0, 16.100080, 0.086198, 0, 16.125847, 0.094792, 0, 16.125847, 0.094792, 0, 16.125847, 0.094792, 0, 16.151514, 0.104948, 0, 16.125847, 0.094792, 0, 16.125847, 0.094792, 0, 16.151514, 0.104948, 0, 16.151514, 0.104948, 0, 16.151514, 0.104948, 0, 16.177084, 0.116667, 0, 16.151514, 0.104948, 0, 16.151514, 0.104948, 0, 16.177084, 0.116667, 0, 16.177084, 0.116667, 0, 16.177084, 0.116667, 0, 16.177084, 0.033333, 0, 16.177084, 0.116667, 0, 16.177084, 0.116667, 0, 16.177084, 0.033333, 0, 16.177084, 0.033333, 0, 16.177084, 0.033333, 0, 16.151140, 0.021826, 0, 16.177084, 0.033333, 0, 16.177084, 0.033333, 0, 16.151140, 0.021826, 0, 16.151140, 0.021826, 0, 16.151140, 0.021826, 0, 16.124870, 0.011784, 0, 16.151140, 0.021826, 0, 16.151140, 0.021826, 0, 16.124870, 0.011784, 0, 16.124870, 0.011784, 0, 16.124870, 0.011784, 0, 16.098274, 0.003206, 0, 16.124870, 0.011784, 0, 16.124870, 0.011784, 0, 16.098274, 0.003206, 0, 16.098274, 0.003206, 0, 16.098274, 0.003206, 0, 16.071356, -0.003906, 0, 16.098274, 0.003206, 0, 16.098274, 0.003206, 0, 16.071356, -0.003906, 0, 16.071356, -0.003906, 0, 16.071356, -0.003906, 0, 16.044172, -0.009489, 0, 16.071356, -0.003906, 0, 16.071356, -0.003906, 0, 16.044172, -0.009489, 0, 16.044172, -0.009489, 0, 16.044172, -0.009489, 0, 16.016798, -0.013477, 0, 16.044172, -0.009489, 0, 16.044172, -0.009489, 0, 16.016798, -0.013477, 0, 16.016798, -0.013477, 0, 16.016798, -0.013477, 0, 15.989226, -0.015869, 0, 16.016798, -0.013477, 0, 16.016798, -0.013477, 0, 15.989226, -0.015869, 0, 15.989226, -0.015869, 0, 15.989226, -0.015869, 0, 15.961458, -0.016667, 0, 15.989226, -0.015869, 0, 15.989226, -0.015869, 0, 15.961458, -0.016667, 0, 15.961458, -0.016667, 0, 15.961458, -0.016667, 0, 15.895296, -0.011556, 0, 15.961458, -0.016667, 0, 15.961458, -0.016667, 0, 15.895296, -0.011556, 0, 15.895296, -0.011556, 0, 15.895296, -0.011556, 0, 15.836394, 0.003776, 0, 15.895296, -0.011556, 0, 15.895296, -0.011556, 0, 15.836394, 0.003776, 0, 15.836394, 0.003776, 0, 15.836394, 0.003776, 0, 15.784749, 0.029329, 0, 15.836394, 0.003776, 0, 15.836394, 0.003776, 0, 15.784749, 0.029329, 0, 15.784749, 0.029329, 0, 15.784749, 0.029329, 0, 15.740364, 0.065104, 0, 15.784749, 0.029329, 0, 15.784749, 0.029329, 0, 15.740364, 0.065104, 0, 15.740364, 0.065104, 0, 15.740364, 0.065104, 0, 15.704704, 0.109603, 0, 15.740364, 0.065104, 0, 15.740364, 0.065104, 0, 15.704704, 0.109603, 0, 15.704704, 0.109603, 0, 15.704704, 0.109603, 0, 15.679232, 0.161328, 0, 15.704704, 0.109603, 0, 15.704704, 0.109603, 0, 15.679232, 0.161328, 0, 15.679232, 0.161328, 0, 15.679232, 0.161328, 0, 15.663948, 0.220280, 0, 15.679232, 0.161328, 0, 15.679232, 0.161328, 0, 15.663948, 0.220280, 0, 15.663948, 0.220280, 0, 15.663948, 0.220280, 0, 15.658854, 0.286458, 0, 15.663948, 0.220280, 0, 15.663948, 0.220280, 0, 15.658854, 0.286458, 0, 15.658854, 0.286458, 0, 15.658854, 0.286458, 0, 15.663689, 0.354867, 0, 15.658854, 0.286458, 0, 15.658854, 0.286458, 0, 15.663689, 0.354867, 0, 15.663689, 0.354867, 0, 15.663689, 0.354867, 0, 15.678191, 0.415820, 0, 15.663689, 0.354867, 0, 15.663689, 0.354867, 0, 15.678191, 0.415820, 0, 15.678191, 0.415820, 0, 15.678191, 0.415820, 0, 15.702361, 0.469320, 0, 15.678191, 0.415820, 0, 15.678191, 0.415820, 0, 15.702361, 0.469320, 0, 15.702361, 0.469320, 0, 15.702361, 0.469320, 0, 15.736198, 0.515365, 0, 15.702361, 0.469320, 0, 15.702361, 0.469320, 0, 15.736198, 0.515365, 0, 15.736198, 0.515365, 0, 15.736198, 0.515365, 0, 15.778239, 0.552393, 0, 15.736198, 0.515365, 0, 15.736198, 0.515365, 0, 15.778239, 0.552393, 0, 15.778239, 0.552393, 0, 15.778239, 0.552393, 0, 15.827019, 0.578841, 0, 15.778239, 0.552393, 0, 15.778239, 0.552393, 0, 15.827019, 0.578841, 0, 15.827019, 0.578841, 0, 15.827019, 0.578841, 0, 15.882537, 0.594710, 0, 15.827019, 0.578841, 0, 15.827019, 0.578841, 0, 15.882537, 0.594710, 0, 15.882537, 0.594710, 0, 15.882537, 0.594710, 0, 15.944793, 0.600000, 0, 15.882537, 0.594710, 0, 15.882537, 0.594710, 0, 15.944793, 0.600000, 0, 15.944793, 0.600000, 0, 15.944793, 0.600000, 0, 16.000570, 0.595272, 0, 15.944793, 0.600000, 0, 15.944793, 0.600000, 0, 16.000570, 0.595272, 0, 16.000570, 0.595272, 0, 16.000570, 0.595272, 0, 16.050196, 0.581087, 0, 16.000570, 0.595272, 0, 16.000570, 0.595272, 0, 16.050196, 0.581087, 0, 16.050196, 0.581087, 0, 16.050196, 0.581087, 0, 16.093670, 0.557446, 0, 16.050196, 0.581087, 0, 16.050196, 0.581087, 0, 16.093670, 0.557446, 0, 16.093670, 0.557446, 0, 16.093670, 0.557446, 0, 16.130989, 0.524349, 0, 16.093670, 0.557446, 0, 16.093670, 0.557446, 0, 16.130989, 0.524349, 0, 16.130989, 0.524349, 0, 16.130989, 0.524349, 0, 16.160955, 0.483130, 0, 16.130989, 0.524349, 0, 16.130989, 0.524349, 0, 16.160955, 0.483130, 0, 16.160955, 0.483130, 0, 16.160955, 0.483130, 0, 16.182356, 0.435124, 0, 16.160955, 0.483130, 0, 16.160955, 0.483130, 0, 16.182356, 0.435124, 0, 16.182356, 0.435124, 0, 16.182356, 0.435124, 0, 16.195198, 0.380330, 0, 16.182356, 0.435124, 0, 16.182356, 0.435124, 0, 16.195198, 0.380330, 0, 16.195198, 0.380330, 0, 15.663689, 0.354867, 0, 15.658854, 0.286458, 0, 15.761980, 0.349479, 0, 15.658854, 0.286458, 0, 15.663689, 0.354867, 0, 15.761980, 0.349479, 0, 15.678191, 0.415820, 0, 15.663689, 0.354867, 0, 15.761980, 0.349479, 0, 15.663689, 0.354867, 0, 15.678191, 0.415820, 0, 15.761980, 0.349479, 0, 15.678191, 0.415820, 0, 15.761980, 0.349479, 0, 15.768181, 0.387077, 0, 15.761980, 0.349479, 0, 15.678191, 0.415820, 0, 15.768181, 0.387077, 0, 15.779492, 0.420182, 0, 15.678191, 0.415820, 0, 15.768181, 0.387077, 0, 15.678191, 0.415820, 0, 15.779492, 0.420182, 0, 15.768181, 0.387077, 0, 15.779492, 0.420182, 0, 15.702361, 0.469320, 0, 15.678191, 0.415820, 0, 15.702361, 0.469320, 0, 15.779492, 0.420182, 0, 15.678191, 0.415820, 0, 15.702361, 0.469320, 0, 15.779492, 0.420182, 0, 15.795916, 0.448796, 0, 15.779492, 0.420182, 0, 15.702361, 0.469320, 0, 15.795916, 0.448796, 0, 15.736198, 0.515365, 0, 15.702361, 0.469320, 0, 15.795916, 0.448796, 0, 15.702361, 0.469320, 0, 15.736198, 0.515365, 0, 15.795916, 0.448796, 0, 15.736198, 0.515365, 0, 15.795916, 0.448796, 0, 15.817449, 0.472917, 0, 15.795916, 0.448796, 0, 15.736198, 0.515365, 0, 15.817449, 0.472917, 0, 15.778239, 0.552393, 0, 15.736198, 0.515365, 0, 15.817449, 0.472917, 0, 15.736198, 0.515365, 0, 15.778239, 0.552393, 0, 15.817449, 0.472917, 0, 15.778239, 0.552393, 0, 15.817449, 0.472917, 0, 15.843538, 0.492057, 0, 15.817449, 0.472917, 0, 15.778239, 0.552393, 0, 15.843538, 0.492057, 0, 15.827019, 0.578841, 0, 15.778239, 0.552393, 0, 15.843538, 0.492057, 0, 15.778239, 0.552393, 0, 15.827019, 0.578841, 0, 15.843538, 0.492057, 0, 15.827019, 0.578841, 0, 15.843538, 0.492057, 0, 15.873633, 0.505729, 0, 15.843538, 0.492057, 0, 15.827019, 0.578841, 0, 15.873633, 0.505729, 0, 15.827019, 0.578841, 0, 15.873633, 0.505729, 0, 15.882537, 0.594710, 0, 15.873633, 0.505729, 0, 15.827019, 0.578841, 0, 15.882537, 0.594710, 0, 15.882537, 0.594710, 0, 15.873633, 0.505729, 0, 15.907732, 0.513932, 0, 15.873633, 0.505729, 0, 15.882537, 0.594710, 0, 15.907732, 0.513932, 0, 15.882537, 0.594710, 0, 15.907732, 0.513932, 0, 15.944793, 0.600000, 0, 15.907732, 0.513932, 0, 15.882537, 0.594710, 0, 15.944793, 0.600000, 0, 15.944793, 0.600000, 0, 15.907732, 0.513932, 0, 15.945833, 0.516667, 0, 15.907732, 0.513932, 0, 15.944793, 0.600000, 0, 15.945833, 0.516667, 0, 15.944793, 0.600000, 0, 15.945833, 0.516667, 0, 15.979509, 0.513835, 0, 15.945833, 0.516667, 0, 15.944793, 0.600000, 0, 15.979509, 0.513835, 0, 15.944793, 0.600000, 0, 15.979509, 0.513835, 0, 16.000570, 0.595272, 0, 15.979509, 0.513835, 0, 15.944793, 0.600000, 0, 16.000570, 0.595272, 0, 16.000570, 0.595272, 0, 15.979509, 0.513835, 0, 16.009701, 0.505339, 0, 15.979509, 0.513835, 0, 16.000570, 0.595272, 0, 16.009701, 0.505339, 0, 16.000570, 0.595272, 0, 16.009701, 0.505339, 0, 16.050196, 0.581087, 0, 16.009701, 0.505339, 0, 16.000570, 0.595272, 0, 16.050196, 0.581087, 0, 16.009701, 0.505339, 0, 16.036409, 0.491178, 0, 16.050196, 0.581087, 0, 16.036409, 0.491178, 0, 16.009701, 0.505339, 0, 16.050196, 0.581087, 0, 16.050196, 0.581087, 0, 16.036409, 0.491178, 0, 16.093670, 0.557446, 0, 16.036409, 0.491178, 0, 16.050196, 0.581087, 0, 16.093670, 0.557446, 0, 16.036409, 0.491178, 0, 16.059637, 0.471354, 0, 16.093670, 0.557446, 0, 16.059637, 0.471354, 0, 16.036409, 0.491178, 0, 16.093670, 0.557446, 0, 16.093670, 0.557446, 0, 16.059637, 0.471354, 0, 16.130989, 0.524349, 0, 16.059637, 0.471354, 0, 16.093670, 0.557446, 0, 16.130989, 0.524349, 0, 16.059637, 0.471354, 0, 16.078499, 0.446777, 0, 16.130989, 0.524349, 0, 16.078499, 0.446777, 0, 16.059637, 0.471354, 0, 16.130989, 0.524349, 0, 16.130989, 0.524349, 0, 16.078499, 0.446777, 0, 16.160955, 0.483130, 0, 16.078499, 0.446777, 0, 16.130989, 0.524349, 0, 16.160955, 0.483130, 0, 16.078499, 0.446777, 0, 16.092123, 0.418359, 0, 16.160955, 0.483130, 0, 16.092123, 0.418359, 0, 16.078499, 0.446777, 0, 16.160955, 0.483130, 0, 16.160955, 0.483130, 0, 16.092123, 0.418359, 0, 16.182356, 0.435124, 0, 16.092123, 0.418359, 0, 16.160955, 0.483130, 0, 16.182356, 0.435124, 0, 16.092123, 0.418359, 0, 16.100506, 0.386100, 0, 16.182356, 0.435124, 0, 16.100506, 0.386100, 0, 16.092123, 0.418359, 0, 16.182356, 0.435124, 0, 16.182356, 0.435124, 0, 16.100506, 0.386100, 0, 16.195198, 0.380330, 0, 16.100506, 0.386100, 0, 16.182356, 0.435124, 0, 16.195198, 0.380330, 0, 16.100506, 0.386100, 0, 16.103645, 0.350000, 0, 16.195198, 0.380330, 0, 16.103645, 0.350000, 0, 16.100506, 0.386100, 0, 16.195198, 0.380330, 0, 16.195198, 0.380330, 0, 16.103645, 0.350000, 0, 16.199480, 0.318750, 0, 16.103645, 0.350000, 0, 16.195198, 0.380330, 0, 16.199480, 0.318750, 0, 16.199480, 0.266667, 0, 16.199480, 0.318750, 0, 16.103645, 0.350000, 0, 16.199480, 0.318750, 0, 16.199480, 0.266667, 0, 16.103645, 0.350000, 0, 16.103645, 0.350000, 0, 15.758855, 0.266667, 0, 16.199480, 0.266667, 0, 15.758855, 0.266667, 0, 16.103645, 0.350000, 0, 16.199480, 0.266667, 0, 16.103645, 0.350000, 0, 15.761980, 0.349479, 0, 15.758855, 0.266667, 0, 15.761980, 0.349479, 0, 16.103645, 0.350000, 0, 15.758855, 0.266667, 0, 15.658854, 0.286458, 0, 15.758855, 0.266667, 0, 15.761980, 0.349479, 0, 15.758855, 0.266667, 0, 15.658854, 0.286458, 0, 15.761980, 0.349479, 0, 15.658854, 0.286458, 0, 15.663948, 0.220280, 0, 15.758855, 0.266667, 0, 15.663948, 0.220280, 0, 15.658854, 0.286458, 0, 15.758855, 0.266667, 0, 15.758855, 0.266667, 0, 15.663948, 0.220280, 0, 15.764925, 0.220744, 0, 15.663948, 0.220280, 0, 15.758855, 0.266667, 0, 15.764925, 0.220744, 0, 15.663948, 0.220280, 0, 15.679232, 0.161328, 0, 15.764925, 0.220744, 0, 15.679232, 0.161328, 0, 15.663948, 0.220280, 0, 15.764925, 0.220744, 0, 15.764925, 0.220744, 0, 15.679232, 0.161328, 0, 15.776888, 0.180632, 0, 15.679232, 0.161328, 0, 15.764925, 0.220744, 0, 15.776888, 0.180632, 0, 15.679232, 0.161328, 0, 15.704704, 0.109603, 0, 15.776888, 0.180632, 0, 15.704704, 0.109603, 0, 15.679232, 0.161328, 0, 15.776888, 0.180632, 0, 15.776888, 0.180632, 0, 15.704704, 0.109603, 0, 15.794744, 0.146330, 0, 15.704704, 0.109603, 0, 15.776888, 0.180632, 0, 15.794744, 0.146330, 0, 15.704704, 0.109603, 0, 15.740364, 0.065104, 0, 15.794744, 0.146330, 0, 15.740364, 0.065104, 0, 15.704704, 0.109603, 0, 15.794744, 0.146330, 0, 15.794744, 0.146330, 0, 15.740364, 0.065104, 0, 15.818489, 0.117839, 0, 15.740364, 0.065104, 0, 15.794744, 0.146330, 0, 15.818489, 0.117839, 0, 15.740364, 0.065104, 0, 15.784749, 0.029329, 0, 15.818489, 0.117839, 0, 15.784749, 0.029329, 0, 15.740364, 0.065104, 0, 15.818489, 0.117839, 0, 15.818489, 0.117839, 0, 15.784749, 0.029329, 0, 15.847803, 0.095451, 0, 15.784749, 0.029329, 0, 15.818489, 0.117839, 0, 15.847803, 0.095451, 0, 15.847803, 0.095451, 0, 15.784749, 0.029329, 0, 15.836394, 0.003776, 0, 15.784749, 0.029329, 0, 15.847803, 0.095451, 0, 15.836394, 0.003776, 0, 15.882357, 0.079460, 0, 15.847803, 0.095451, 0, 15.836394, 0.003776, 0, 15.847803, 0.095451, 0, 15.882357, 0.079460, 0, 15.836394, 0.003776, 0, 15.882357, 0.079460, 0, 15.836394, 0.003776, 0, 15.895296, -0.011556, 0, 15.836394, 0.003776, 0, 15.882357, 0.079460, 0, 15.895296, -0.011556, 0, 15.922152, 0.069865, 0, 15.882357, 0.079460, 0, 15.895296, -0.011556, 0, 15.882357, 0.079460, 0, 15.922152, 0.069865, 0, 15.895296, -0.011556, 0, 15.922152, 0.069865, 0, 15.895296, -0.011556, 0, 15.961458, -0.016667, 0, 15.895296, -0.011556, 0, 15.922152, 0.069865, 0, 15.961458, -0.016667, 0, 15.967188, 0.066667, 0, 15.922152, 0.069865, 0, 15.961458, -0.016667, 0, 15.922152, 0.069865, 0, 15.967188, 0.066667, 0, 15.961458, -0.016667, 0, 15.967188, 0.066667, 0, 15.961458, -0.016667, 0, 15.989226, -0.015869, 0, 15.961458, -0.016667, 0, 15.967188, 0.066667, 0, 15.989226, -0.015869, 0, 15.967188, 0.066667, 0, 15.989226, -0.015869, 0, 15.994580, 0.067448, 0, 15.989226, -0.015869, 0, 15.967188, 0.066667, 0, 15.994580, 0.067448, 0, 15.994580, 0.067448, 0, 15.989226, -0.015869, 0, 16.016798, -0.013477, 0, 15.989226, -0.015869, 0, 15.994580, 0.067448, 0, 16.016798, -0.013477, 0, 15.994580, 0.067448, 0, 16.016798, -0.013477, 0, 16.021551, 0.069792, 0, 16.016798, -0.013477, 0, 15.994580, 0.067448, 0, 16.021551, 0.069792, 0, 16.021551, 0.069792, 0, 16.016798, -0.013477, 0, 16.044172, -0.009489, 0, 16.016798, -0.013477, 0, 16.021551, 0.069792, 0, 16.044172, -0.009489, 0, 16.021551, 0.069792, 0, 16.044172, -0.009489, 0, 16.048096, 0.073698, 0, 16.044172, -0.009489, 0, 16.021551, 0.069792, 0, 16.048096, 0.073698, 0, 16.048096, 0.073698, 0, 16.044172, -0.009489, 0, 16.071356, -0.003906, 0, 16.044172, -0.009489, 0, 16.048096, 0.073698, 0, 16.071356, -0.003906, 0, 16.048096, 0.073698, 0, 16.071356, -0.003906, 0, 16.074219, 0.079167, 0, 16.071356, -0.003906, 0, 16.048096, 0.073698, 0, 16.074219, 0.079167, 0, 16.074219, 0.079167, 0, 16.071356, -0.003906, 0, 16.098274, 0.003206, 0, 16.071356, -0.003906, 0, 16.074219, 0.079167, 0, 16.098274, 0.003206, 0, 16.100080, 0.086198, 0, 16.074219, 0.079167, 0, 16.098274, 0.003206, 0, 16.074219, 0.079167, 0, 16.100080, 0.086198, 0, 16.098274, 0.003206, 0, 16.124870, 0.011784, 0, 16.100080, 0.086198, 0, 16.098274, 0.003206, 0, 16.100080, 0.086198, 0, 16.124870, 0.011784, 0, 16.098274, 0.003206, 0, 16.100080, 0.086198, 0, 16.124870, 0.011784, 0, 16.151140, 0.021826, 0, 16.124870, 0.011784, 0, 16.100080, 0.086198, 0, 16.151140, 0.021826, 0, 16.125847, 0.094792, 0, 16.100080, 0.086198, 0, 16.151140, 0.021826, 0, 16.100080, 0.086198, 0, 16.125847, 0.094792, 0, 16.151140, 0.021826, 0, 16.125847, 0.094792, 0, 16.151140, 0.021826, 0, 16.177084, 0.033333, 0, 16.151140, 0.021826, 0, 16.125847, 0.094792, 0, 16.177084, 0.033333, 0, 16.151514, 0.104948, 0, 16.125847, 0.094792, 0, 16.177084, 0.033333, 0, 16.125847, 0.094792, 0, 16.151514, 0.104948, 0, 16.177084, 0.033333, 0, 16.151514, 0.104948, 0, 16.177084, 0.033333, 0, 16.177084, 0.116667, 0, 16.177084, 0.033333, 0, 16.151514, 0.104948, 0, 16.177084, 0.116667, 0, 16.103645, 0.350000, 0, 16.100506, 0.386100, 0, 16.103645, 0.350000, 0, 16.103645, 0.350000, 0, 16.100506, 0.386100, 0, 16.100506, 0.386100, 0, 16.100506, 0.386100, 0, 16.092123, 0.418359, 0, 16.100506, 0.386100, 0, 16.100506, 0.386100, 0, 16.092123, 0.418359, 0, 16.092123, 0.418359, 0, 16.092123, 0.418359, 0, 16.078499, 0.446777, 0, 16.092123, 0.418359, 0, 16.092123, 0.418359, 0, 16.078499, 0.446777, 0, 16.078499, 0.446777, 0, 16.078499, 0.446777, 0, 16.059637, 0.471354, 0, 16.078499, 0.446777, 0, 16.078499, 0.446777, 0, 16.059637, 0.471354, 0, 16.059637, 0.471354, 0, 16.059637, 0.471354, 0, 16.036409, 0.491178, 0, 16.059637, 0.471354, 0, 16.059637, 0.471354, 0, 16.036409, 0.491178, 0, 16.036409, 0.491178, 0, 16.036409, 0.491178, 0, 16.009701, 0.505339, 0, 16.036409, 0.491178, 0, 16.036409, 0.491178, 0, 16.009701, 0.505339, 0, 16.009701, 0.505339, 0, 16.009701, 0.505339, 0, 15.979509, 0.513835, 0, 16.009701, 0.505339, 0, 16.009701, 0.505339, 0, 15.979509, 0.513835, 0, 15.979509, 0.513835, 0, 15.979509, 0.513835, 0, 15.945833, 0.516667, 0, 15.979509, 0.513835, 0, 15.979509, 0.513835, 0, 15.945833, 0.516667, 0, 15.945833, 0.516667, 0, 15.945833, 0.516667, 0, 15.907732, 0.513932, 0, 15.945833, 0.516667, 0, 15.945833, 0.516667, 0, 15.907732, 0.513932, 0, 15.907732, 0.513932, 0, 15.907732, 0.513932, 0, 15.873633, 0.505729, 0, 15.907732, 0.513932, 0, 15.907732, 0.513932, 0, 15.873633, 0.505729, 0, 15.873633, 0.505729, 0, 15.873633, 0.505729, 0, 15.843538, 0.492057, 0, 15.873633, 0.505729, 0, 15.873633, 0.505729, 0, 15.843538, 0.492057, 0, 15.843538, 0.492057, 0, 15.843538, 0.492057, 0, 15.817449, 0.472917, 0, 15.843538, 0.492057, 0, 15.843538, 0.492057, 0, 15.817449, 0.472917, 0, 15.817449, 0.472917, 0, 15.817449, 0.472917, 0, 15.795916, 0.448796, 0, 15.817449, 0.472917, 0, 15.817449, 0.472917, 0, 15.795916, 0.448796, 0, 15.795916, 0.448796, 0, 15.795916, 0.448796, 0, 15.779492, 0.420182, 0, 15.795916, 0.448796, 0, 15.795916, 0.448796, 0, 15.779492, 0.420182, 0, 15.779492, 0.420182, 0, 15.779492, 0.420182, 0, 15.768181, 0.387077, 0, 15.779492, 0.420182, 0, 15.779492, 0.420182, 0, 15.768181, 0.387077, 0, 15.768181, 0.387077, 0, 15.768181, 0.387077, 0, 15.761980, 0.349479, 0, 15.768181, 0.387077, 0, 15.768181, 0.387077, 0, 15.761980, 0.349479, 0, 15.761980, 0.349479, 0, 16.948957, 0.283333, 0, 16.895346, 0.281755, 0, 16.948957, 0.283333, 0, 16.948957, 0.283333, 0, 16.895346, 0.281755, 0, 16.895346, 0.281755, 0, 16.895346, 0.281755, 0, 16.850653, 0.277018, 0, 16.895346, 0.281755, 0, 16.895346, 0.281755, 0, 16.850653, 0.277018, 0, 16.850653, 0.277018, 0, 16.850653, 0.277018, 0, 16.814877, 0.269124, 0, 16.850653, 0.277018, 0, 16.850653, 0.277018, 0, 16.814877, 0.269124, 0, 16.814877, 0.269124, 0, 16.814877, 0.269124, 0, 16.788021, 0.258073, 0, 16.814877, 0.269124, 0, 16.814877, 0.269124, 0, 16.788021, 0.258073, 0, 16.788021, 0.258073, 0, 16.788021, 0.258073, 0, 16.768425, 0.243229, 0, 16.788021, 0.258073, 0, 16.788021, 0.258073, 0, 16.768425, 0.243229, 0, 16.768425, 0.243229, 0, 16.768425, 0.243229, 0, 16.754427, 0.223958, 0, 16.768425, 0.243229, 0, 16.768425, 0.243229, 0, 16.754427, 0.223958, 0, 16.754427, 0.223958, 0, 16.754427, 0.223958, 0, 16.746029, 0.200260, 0, 16.754427, 0.223958, 0, 16.754427, 0.223958, 0, 16.746029, 0.200260, 0, 16.746029, 0.200260, 0, 16.746029, 0.200260, 0, 16.743231, 0.172135, 0, 16.746029, 0.200260, 0, 16.746029, 0.200260, 0, 16.743231, 0.172135, 0, 16.743231, 0.172135, 0, 16.743231, 0.172135, 0, 16.745329, 0.149056, 0, 16.743231, 0.172135, 0, 16.743231, 0.172135, 0, 16.745329, 0.149056, 0, 16.745329, 0.149056, 0, 16.745329, 0.149056, 0, 16.751629, 0.128516, 0, 16.745329, 0.149056, 0, 16.745329, 0.149056, 0, 16.751629, 0.128516, 0, 16.751629, 0.128516, 0, 16.751629, 0.128516, 0, 16.762127, 0.110514, 0, 16.751629, 0.128516, 0, 16.751629, 0.128516, 0, 16.762127, 0.110514, 0, 16.762127, 0.110514, 0, 16.762127, 0.110514, 0, 16.776823, 0.095052, 0, 16.762127, 0.110514, 0, 16.762127, 0.110514, 0, 16.776823, 0.095052, 0, 16.776823, 0.095052, 0, 16.776823, 0.095052, 0, 16.795134, 0.082633, 0, 16.776823, 0.095052, 0, 16.776823, 0.095052, 0, 16.795134, 0.082633, 0, 16.795134, 0.082633, 0, 16.795134, 0.082633, 0, 16.816473, 0.073763, 0, 16.795134, 0.082633, 0, 16.795134, 0.082633, 0, 16.816473, 0.073763, 0, 16.816473, 0.073763, 0, 16.816473, 0.073763, 0, 16.840837, 0.068441, 0, 16.816473, 0.073763, 0, 16.816473, 0.073763, 0, 16.840837, 0.068441, 0, 16.840837, 0.068441, 0, 16.840837, 0.068441, 0, 16.868231, 0.066667, 0, 16.840837, 0.068441, 0, 16.840837, 0.068441, 0, 16.868231, 0.066667, 0, 16.868231, 0.066667, 0, 16.868231, 0.066667, 0, 16.906103, 0.070011, 0, 16.868231, 0.066667, 0, 16.868231, 0.066667, 0, 16.906103, 0.070011, 0, 16.906103, 0.070011, 0, 16.906103, 0.070011, 0, 16.940039, 0.080046, 0, 16.906103, 0.070011, 0, 16.906103, 0.070011, 0, 16.940039, 0.080046, 0, 16.940039, 0.080046, 0, 16.940039, 0.080046, 0, 16.970037, 0.096769, 0, 16.940039, 0.080046, 0, 16.940039, 0.080046, 0, 16.970037, 0.096769, 0, 16.970037, 0.096769, 0, 16.970037, 0.096769, 0, 16.996094, 0.120182, 0, 16.970037, 0.096769, 0, 16.970037, 0.096769, 0, 16.996094, 0.120182, 0, 16.996094, 0.120182, 0, 16.996094, 0.120182, 0, 17.017172, 0.149146, 0, 16.996094, 0.120182, 0, 16.996094, 0.120182, 0, 17.017172, 0.149146, 0, 17.017172, 0.149146, 0, 17.017172, 0.149146, 0, 17.032227, 0.182520, 0, 17.017172, 0.149146, 0, 17.017172, 0.149146, 0, 17.032227, 0.182520, 0, 17.032227, 0.182520, 0, 17.032227, 0.182520, 0, 17.041260, 0.220304, 0, 17.032227, 0.182520, 0, 17.032227, 0.182520, 0, 17.041260, 0.220304, 0, 17.041260, 0.220304, 0, 17.041260, 0.220304, 0, 17.044271, 0.262500, 0, 17.041260, 0.220304, 0, 17.041260, 0.220304, 0, 17.044271, 0.262500, 0, 17.044271, 0.262500, 0, 17.044271, 0.262500, 0, 17.044271, 0.283333, 0, 17.044271, 0.262500, 0, 17.044271, 0.262500, 0, 17.044271, 0.283333, 0, 17.044271, 0.283333, 0, 17.140104, 0.327344, 0, 17.140104, 0, 0, 17.140104, 0.327344, 0, 17.140104, 0.327344, 0, 17.140104, 0, 0, 17.140104, 0, 0, 17.140104, 0, 0, 17.044271, 0, 0, 17.140104, 0, 0, 17.140104, 0, 0, 17.044271, 0, 0, 17.044271, 0, 0, 17.044271, 0, 0, 17.044271, 0.083333, 0, 17.044271, 0, 0, 17.044271, 0, 0, 17.044271, 0.083333, 0, 17.044271, 0.083333, 0, 17.044271, 0.083333, 0, 17.026855, 0.059367, 0, 17.044271, 0.083333, 0, 17.044271, 0.083333, 0, 17.026855, 0.059367, 0, 17.026855, 0.059367, 0, 17.026855, 0.059367, 0, 17.007421, 0.038770, 0, 17.026855, 0.059367, 0, 17.026855, 0.059367, 0, 17.007421, 0.038770, 0, 17.007421, 0.038770, 0, 17.007421, 0.038770, 0, 16.985970, 0.021541, 0, 17.007421, 0.038770, 0, 17.007421, 0.038770, 0, 16.985970, 0.021541, 0, 16.985970, 0.021541, 0, 16.985970, 0.021541, 0, 16.962500, 0.007682, 0, 16.985970, 0.021541, 0, 16.985970, 0.021541, 0, 16.962500, 0.007682, 0, 16.962500, 0.007682, 0, 16.962500, 0.007682, 0, 16.936653, -0.002970, 0, 16.962500, 0.007682, 0, 16.962500, 0.007682, 0, 16.936653, -0.002970, 0, 16.936653, -0.002970, 0, 16.936653, -0.002970, 0, 16.908073, -0.010579, 0, 16.936653, -0.002970, 0, 16.936653, -0.002970, 0, 16.908073, -0.010579, 0, 16.908073, -0.010579, 0, 16.908073, -0.010579, 0, 16.876759, -0.015145, 0, 16.908073, -0.010579, 0, 16.908073, -0.010579, 0, 16.876759, -0.015145, 0, 16.876759, -0.015145, 0, 16.876759, -0.015145, 0, 16.842709, -0.016667, 0, 16.876759, -0.015145, 0, 16.876759, -0.015145, 0, 16.842709, -0.016667, 0, 16.842709, -0.016667, 0, 16.842709, -0.016667, 0, 16.800213, -0.013525, 0, 16.842709, -0.016667, 0, 16.842709, -0.016667, 0, 16.800213, -0.013525, 0, 16.800213, -0.013525, 0, 16.800213, -0.013525, 0, 16.762304, -0.004102, 0, 16.800213, -0.013525, 0, 16.800213, -0.013525, 0, 16.762304, -0.004102, 0, 16.762304, -0.004102, 0, 16.762304, -0.004102, 0, 16.728987, 0.011605, 0, 16.762304, -0.004102, 0, 16.762304, -0.004102, 0, 16.728987, 0.011605, 0, 16.728987, 0.011605, 0, 16.728987, 0.011605, 0, 16.700262, 0.033594, 0, 16.728987, 0.011605, 0, 16.728987, 0.011605, 0, 16.700262, 0.033594, 0, 16.700262, 0.033594, 0, 16.700262, 0.033594, 0, 16.677132, 0.060856, 0, 16.700262, 0.033594, 0, 16.700262, 0.033594, 0, 16.677132, 0.060856, 0, 16.677132, 0.060856, 0, 16.677132, 0.060856, 0, 16.660612, 0.092383, 0, 16.677132, 0.060856, 0, 16.677132, 0.060856, 0, 16.660612, 0.092383, 0, 16.660612, 0.092383, 0, 16.660612, 0.092383, 0, 16.650702, 0.128174, 0, 16.660612, 0.092383, 0, 16.660612, 0.092383, 0, 16.650702, 0.128174, 0, 16.650702, 0.128174, 0, 16.650702, 0.128174, 0, 16.647396, 0.168229, 0, 16.650702, 0.128174, 0, 16.650702, 0.128174, 0, 16.647396, 0.168229, 0, 16.647396, 0.168229, 0, 16.647396, 0.168229, 0, 16.651514, 0.214421, 0, 16.647396, 0.168229, 0, 16.647396, 0.168229, 0, 16.651514, 0.214421, 0, 16.651514, 0.214421, 0, 16.651514, 0.214421, 0, 16.663868, 0.254557, 0, 16.651514, 0.214421, 0, 16.651514, 0.214421, 0, 16.663868, 0.254557, 0, 16.663868, 0.254557, 0, 16.663868, 0.254557, 0, 16.684456, 0.288639, 0, 16.663868, 0.254557, 0, 16.663868, 0.254557, 0, 16.684456, 0.288639, 0, 16.684456, 0.288639, 0, 16.684456, 0.288639, 0, 16.713282, 0.316667, 0, 16.684456, 0.288639, 0, 16.684456, 0.288639, 0, 16.713282, 0.316667, 0, 16.713282, 0.316667, 0, 16.713282, 0.316667, 0, 16.750277, 0.338542, 0, 16.713282, 0.316667, 0, 16.713282, 0.316667, 0, 16.750277, 0.338542, 0, 16.750277, 0.338542, 0, 16.750277, 0.338542, 0, 16.795378, 0.354167, 0, 16.750277, 0.338542, 0, 16.750277, 0.338542, 0, 16.795378, 0.354167, 0, 16.795378, 0.354167, 0, 16.795378, 0.354167, 0, 16.848585, 0.363542, 0, 16.795378, 0.354167, 0, 16.795378, 0.354167, 0, 16.848585, 0.363542, 0, 16.848585, 0.363542, 0, 16.848585, 0.363542, 0, 16.909895, 0.366667, 0, 16.848585, 0.363542, 0, 16.848585, 0.363542, 0, 16.909895, 0.366667, 0, 16.909895, 0.366667, 0, 16.909895, 0.366667, 0, 17.044271, 0.366667, 0, 16.909895, 0.366667, 0, 16.909895, 0.366667, 0, 17.044271, 0.366667, 0, 17.044271, 0.366667, 0, 17.044271, 0.366667, 0, 17.044271, 0.376042, 0, 17.044271, 0.366667, 0, 17.044271, 0.366667, 0, 17.044271, 0.376042, 0, 17.044271, 0.376042, 0, 17.044271, 0.376042, 0, 17.041553, 0.407731, 0, 17.044271, 0.376042, 0, 17.044271, 0.376042, 0, 17.041553, 0.407731, 0, 17.041553, 0.407731, 0, 17.041553, 0.407731, 0, 17.033400, 0.435612, 0, 17.041553, 0.407731, 0, 17.041553, 0.407731, 0, 17.033400, 0.435612, 0, 17.033400, 0.435612, 0, 17.033400, 0.435612, 0, 17.019808, 0.459684, 0, 17.033400, 0.435612, 0, 17.033400, 0.435612, 0, 17.019808, 0.459684, 0, 17.019808, 0.459684, 0, 17.019808, 0.459684, 0, 17.000782, 0.479948, 0, 17.019808, 0.459684, 0, 17.019808, 0.459684, 0, 17.000782, 0.479948, 0, 17.000782, 0.479948, 0, 17.000782, 0.479948, 0, 16.976839, 0.496012, 0, 17.000782, 0.479948, 0, 17.000782, 0.479948, 0, 16.976839, 0.496012, 0, 16.976839, 0.496012, 0, 16.976839, 0.496012, 0, 16.948503, 0.507487, 0, 16.976839, 0.496012, 0, 16.976839, 0.496012, 0, 16.948503, 0.507487, 0, 16.948503, 0.507487, 0, 16.948503, 0.507487, 0, 16.915771, 0.514372, 0, 16.948503, 0.507487, 0, 16.948503, 0.507487, 0, 16.915771, 0.514372, 0, 16.915771, 0.514372, 0, 16.915771, 0.514372, 0, 16.878645, 0.516667, 0, 16.915771, 0.514372, 0, 16.915771, 0.514372, 0, 16.878645, 0.516667, 0, 16.878645, 0.516667, 0, 16.878645, 0.516667, 0, 16.853809, 0.515885, 0, 16.878645, 0.516667, 0, 16.878645, 0.516667, 0, 16.853809, 0.515885, 0, 16.853809, 0.515885, 0, 16.853809, 0.515885, 0, 16.829298, 0.513542, 0, 16.853809, 0.515885, 0, 16.853809, 0.515885, 0, 16.829298, 0.513542, 0, 16.829298, 0.513542, 0, 16.829298, 0.513542, 0, 16.805111, 0.509635, 0, 16.829298, 0.513542, 0, 16.829298, 0.513542, 0, 16.805111, 0.509635, 0, 16.805111, 0.509635, 0, 16.805111, 0.509635, 0, 16.781250, 0.504167, 0, 16.805111, 0.509635, 0, 16.805111, 0.509635, 0, 16.781250, 0.504167, 0, 16.781250, 0.504167, 0, 16.781250, 0.504167, 0, 16.757780, 0.497135, 0, 16.781250, 0.504167, 0, 16.781250, 0.504167, 0, 16.757780, 0.497135, 0, 16.757780, 0.497135, 0, 16.757780, 0.497135, 0, 16.734766, 0.488542, 0, 16.757780, 0.497135, 0, 16.757780, 0.497135, 0, 16.734766, 0.488542, 0, 16.734766, 0.488542, 0, 16.734766, 0.488542, 0, 16.712208, 0.478385, 0, 16.734766, 0.488542, 0, 16.734766, 0.488542, 0, 16.712208, 0.478385, 0, 16.712208, 0.478385, 0, 16.712208, 0.478385, 0, 16.690105, 0.466667, 0, 16.712208, 0.478385, 0, 16.712208, 0.478385, 0, 16.690105, 0.466667, 0, 16.690105, 0.466667, 0, 16.690105, 0.466667, 0, 16.690105, 0.550000, 0, 16.690105, 0.466667, 0, 16.690105, 0.466667, 0, 16.690105, 0.550000, 0, 16.690105, 0.550000, 0, 16.690105, 0.550000, 0, 16.716211, 0.561719, 0, 16.690105, 0.550000, 0, 16.690105, 0.550000, 0, 16.716211, 0.561719, 0, 16.716211, 0.561719, 0, 16.716211, 0.561719, 0, 16.741926, 0.571875, 0, 16.716211, 0.561719, 0, 16.716211, 0.561719, 0, 16.741926, 0.571875, 0, 16.741926, 0.571875, 0, 16.741926, 0.571875, 0, 16.767254, 0.580469, 0, 16.741926, 0.571875, 0, 16.741926, 0.571875, 0, 16.767254, 0.580469, 0, 16.767254, 0.580469, 0, 16.767254, 0.580469, 0, 16.792189, 0.587500, 0, 16.767254, 0.580469, 0, 16.767254, 0.580469, 0, 16.792189, 0.587500, 0, 16.792189, 0.587500, 0, 16.792189, 0.587500, 0, 16.816765, 0.592969, 0, 16.792189, 0.587500, 0, 16.792189, 0.587500, 0, 16.816765, 0.592969, 0, 16.816765, 0.592969, 0, 16.816765, 0.592969, 0, 16.841017, 0.596875, 0, 16.816765, 0.592969, 0, 16.816765, 0.592969, 0, 16.841017, 0.596875, 0, 16.841017, 0.596875, 0, 16.841017, 0.596875, 0, 16.864943, 0.599219, 0, 16.841017, 0.596875, 0, 16.841017, 0.596875, 0, 16.864943, 0.599219, 0, 16.864943, 0.599219, 0, 16.864943, 0.599219, 0, 16.888542, 0.600000, 0, 16.864943, 0.599219, 0, 16.864943, 0.599219, 0, 16.888542, 0.600000, 0, 16.888542, 0.600000, 0, 16.888542, 0.600000, 0, 16.947821, 0.595776, 0, 16.888542, 0.600000, 0, 16.888542, 0.600000, 0, 16.947821, 0.595776, 0, 16.947821, 0.595776, 0, 16.947821, 0.595776, 0, 16.999090, 0.583106, 0, 16.947821, 0.595776, 0, 16.947821, 0.595776, 0, 16.999090, 0.583106, 0, 16.999090, 0.583106, 0, 16.999090, 0.583106, 0, 17.042351, 0.561987, 0, 16.999090, 0.583106, 0, 16.999090, 0.583106, 0, 17.042351, 0.561987, 0, 17.042351, 0.561987, 0, 17.042351, 0.561987, 0, 17.077604, 0.532422, 0, 17.042351, 0.561987, 0, 17.042351, 0.561987, 0, 17.077604, 0.532422, 0, 17.077604, 0.532422, 0, 17.077604, 0.532422, 0, 17.104948, 0.494263, 0, 17.077604, 0.532422, 0, 17.077604, 0.532422, 0, 17.104948, 0.494263, 0, 17.104948, 0.494263, 0, 17.104948, 0.494263, 0, 17.124479, 0.447363, 0, 17.104948, 0.494263, 0, 17.104948, 0.494263, 0, 17.124479, 0.447363, 0, 17.124479, 0.447363, 0, 17.124479, 0.447363, 0, 17.136198, 0.391724, 0, 17.124479, 0.447363, 0, 17.124479, 0.447363, 0, 17.136198, 0.391724, 0, 17.136198, 0.391724, 0, 16.651514, 0.214421, 0, 16.647396, 0.168229, 0, 16.743231, 0.172135, 0, 16.647396, 0.168229, 0, 16.651514, 0.214421, 0, 16.743231, 0.172135, 0, 16.651514, 0.214421, 0, 16.743231, 0.172135, 0, 16.746029, 0.200260, 0, 16.743231, 0.172135, 0, 16.651514, 0.214421, 0, 16.746029, 0.200260, 0, 16.663868, 0.254557, 0, 16.651514, 0.214421, 0, 16.746029, 0.200260, 0, 16.651514, 0.214421, 0, 16.663868, 0.254557, 0, 16.746029, 0.200260, 0, 16.663868, 0.254557, 0, 16.746029, 0.200260, 0, 16.754427, 0.223958, 0, 16.746029, 0.200260, 0, 16.663868, 0.254557, 0, 16.754427, 0.223958, 0, 16.684456, 0.288639, 0, 16.663868, 0.254557, 0, 16.754427, 0.223958, 0, 16.663868, 0.254557, 0, 16.684456, 0.288639, 0, 16.754427, 0.223958, 0, 16.684456, 0.288639, 0, 16.754427, 0.223958, 0, 16.768425, 0.243229, 0, 16.754427, 0.223958, 0, 16.684456, 0.288639, 0, 16.768425, 0.243229, 0, 16.713282, 0.316667, 0, 16.684456, 0.288639, 0, 16.768425, 0.243229, 0, 16.684456, 0.288639, 0, 16.713282, 0.316667, 0, 16.768425, 0.243229, 0, 16.713282, 0.316667, 0, 16.768425, 0.243229, 0, 16.788021, 0.258073, 0, 16.768425, 0.243229, 0, 16.713282, 0.316667, 0, 16.788021, 0.258073, 0, 16.750277, 0.338542, 0, 16.713282, 0.316667, 0, 16.788021, 0.258073, 0, 16.713282, 0.316667, 0, 16.750277, 0.338542, 0, 16.788021, 0.258073, 0, 16.814877, 0.269124, 0, 16.750277, 0.338542, 0, 16.788021, 0.258073, 0, 16.750277, 0.338542, 0, 16.814877, 0.269124, 0, 16.788021, 0.258073, 0, 16.795378, 0.354167, 0, 16.750277, 0.338542, 0, 16.814877, 0.269124, 0, 16.750277, 0.338542, 0, 16.795378, 0.354167, 0, 16.814877, 0.269124, 0, 16.850653, 0.277018, 0, 16.795378, 0.354167, 0, 16.814877, 0.269124, 0, 16.795378, 0.354167, 0, 16.850653, 0.277018, 0, 16.814877, 0.269124, 0, 16.848585, 0.363542, 0, 16.795378, 0.354167, 0, 16.850653, 0.277018, 0, 16.795378, 0.354167, 0, 16.848585, 0.363542, 0, 16.850653, 0.277018, 0, 16.848585, 0.363542, 0, 16.850653, 0.277018, 0, 16.895346, 0.281755, 0, 16.850653, 0.277018, 0, 16.848585, 0.363542, 0, 16.895346, 0.281755, 0, 16.848585, 0.363542, 0, 16.895346, 0.281755, 0, 16.909895, 0.366667, 0, 16.895346, 0.281755, 0, 16.848585, 0.363542, 0, 16.909895, 0.366667, 0, 16.909895, 0.366667, 0, 16.895346, 0.281755, 0, 16.948957, 0.283333, 0, 16.895346, 0.281755, 0, 16.909895, 0.366667, 0, 16.948957, 0.283333, 0, 16.909895, 0.366667, 0, 16.948957, 0.283333, 0, 17.044271, 0.366667, 0, 16.948957, 0.283333, 0, 16.909895, 0.366667, 0, 17.044271, 0.366667, 0, 16.948957, 0.283333, 0, 17.044271, 0.283333, 0, 17.044271, 0.366667, 0, 17.044271, 0.283333, 0, 16.948957, 0.283333, 0, 17.044271, 0.366667, 0, 17.044271, 0.366667, 0, 17.044271, 0.283333, 0, 17.140104, 0.327344, 0, 17.044271, 0.283333, 0, 17.044271, 0.366667, 0, 17.140104, 0.327344, 0, 17.140104, 0.327344, 0, 17.136198, 0.391724, 0, 17.044271, 0.366667, 0, 17.136198, 0.391724, 0, 17.140104, 0.327344, 0, 17.044271, 0.366667, 0, 17.136198, 0.391724, 0, 17.044271, 0.376042, 0, 17.044271, 0.366667, 0, 17.044271, 0.376042, 0, 17.136198, 0.391724, 0, 17.044271, 0.366667, 0, 17.041553, 0.407731, 0, 17.044271, 0.376042, 0, 17.136198, 0.391724, 0, 17.044271, 0.376042, 0, 17.041553, 0.407731, 0, 17.136198, 0.391724, 0, 17.124479, 0.447363, 0, 17.041553, 0.407731, 0, 17.136198, 0.391724, 0, 17.041553, 0.407731, 0, 17.124479, 0.447363, 0, 17.136198, 0.391724, 0, 17.033400, 0.435612, 0, 17.041553, 0.407731, 0, 17.124479, 0.447363, 0, 17.041553, 0.407731, 0, 17.033400, 0.435612, 0, 17.124479, 0.447363, 0, 17.104948, 0.494263, 0, 17.033400, 0.435612, 0, 17.124479, 0.447363, 0, 17.033400, 0.435612, 0, 17.104948, 0.494263, 0, 17.124479, 0.447363, 0, 17.019808, 0.459684, 0, 17.033400, 0.435612, 0, 17.104948, 0.494263, 0, 17.033400, 0.435612, 0, 17.019808, 0.459684, 0, 17.104948, 0.494263, 0, 17.077604, 0.532422, 0, 17.019808, 0.459684, 0, 17.104948, 0.494263, 0, 17.019808, 0.459684, 0, 17.077604, 0.532422, 0, 17.104948, 0.494263, 0, 17.000782, 0.479948, 0, 17.019808, 0.459684, 0, 17.077604, 0.532422, 0, 17.019808, 0.459684, 0, 17.000782, 0.479948, 0, 17.077604, 0.532422, 0, 17.042351, 0.561987, 0, 17.000782, 0.479948, 0, 17.077604, 0.532422, 0, 17.000782, 0.479948, 0, 17.042351, 0.561987, 0, 17.077604, 0.532422, 0, 16.976839, 0.496012, 0, 17.000782, 0.479948, 0, 17.042351, 0.561987, 0, 17.000782, 0.479948, 0, 16.976839, 0.496012, 0, 17.042351, 0.561987, 0, 16.999090, 0.583106, 0, 16.976839, 0.496012, 0, 17.042351, 0.561987, 0, 16.976839, 0.496012, 0, 16.999090, 0.583106, 0, 17.042351, 0.561987, 0, 16.948503, 0.507487, 0, 16.976839, 0.496012, 0, 16.999090, 0.583106, 0, 16.976839, 0.496012, 0, 16.948503, 0.507487, 0, 16.999090, 0.583106, 0, 16.947821, 0.595776, 0, 16.948503, 0.507487, 0, 16.999090, 0.583106, 0, 16.948503, 0.507487, 0, 16.947821, 0.595776, 0, 16.999090, 0.583106, 0, 16.947821, 0.595776, 0, 16.915771, 0.514372, 0, 16.948503, 0.507487, 0, 16.915771, 0.514372, 0, 16.947821, 0.595776, 0, 16.948503, 0.507487, 0, 16.947821, 0.595776, 0, 16.888542, 0.600000, 0, 16.915771, 0.514372, 0, 16.888542, 0.600000, 0, 16.947821, 0.595776, 0, 16.915771, 0.514372, 0, 16.878645, 0.516667, 0, 16.915771, 0.514372, 0, 16.888542, 0.600000, 0, 16.915771, 0.514372, 0, 16.878645, 0.516667, 0, 16.888542, 0.600000, 0, 16.888542, 0.600000, 0, 16.864943, 0.599219, 0, 16.878645, 0.516667, 0, 16.864943, 0.599219, 0, 16.888542, 0.600000, 0, 16.878645, 0.516667, 0, 16.864943, 0.599219, 0, 16.853809, 0.515885, 0, 16.878645, 0.516667, 0, 16.853809, 0.515885, 0, 16.864943, 0.599219, 0, 16.878645, 0.516667, 0, 16.841017, 0.596875, 0, 16.853809, 0.515885, 0, 16.864943, 0.599219, 0, 16.853809, 0.515885, 0, 16.841017, 0.596875, 0, 16.864943, 0.599219, 0, 16.841017, 0.596875, 0, 16.829298, 0.513542, 0, 16.853809, 0.515885, 0, 16.829298, 0.513542, 0, 16.841017, 0.596875, 0, 16.853809, 0.515885, 0, 16.816765, 0.592969, 0, 16.829298, 0.513542, 0, 16.841017, 0.596875, 0, 16.829298, 0.513542, 0, 16.816765, 0.592969, 0, 16.841017, 0.596875, 0, 16.816765, 0.592969, 0, 16.792189, 0.587500, 0, 16.829298, 0.513542, 0, 16.792189, 0.587500, 0, 16.816765, 0.592969, 0, 16.829298, 0.513542, 0, 16.829298, 0.513542, 0, 16.792189, 0.587500, 0, 16.805111, 0.509635, 0, 16.792189, 0.587500, 0, 16.829298, 0.513542, 0, 16.805111, 0.509635, 0, 16.792189, 0.587500, 0, 16.767254, 0.580469, 0, 16.805111, 0.509635, 0, 16.767254, 0.580469, 0, 16.792189, 0.587500, 0, 16.805111, 0.509635, 0, 16.767254, 0.580469, 0, 16.781250, 0.504167, 0, 16.805111, 0.509635, 0, 16.781250, 0.504167, 0, 16.767254, 0.580469, 0, 16.805111, 0.509635, 0, 16.767254, 0.580469, 0, 16.741926, 0.571875, 0, 16.781250, 0.504167, 0, 16.741926, 0.571875, 0, 16.767254, 0.580469, 0, 16.781250, 0.504167, 0, 16.741926, 0.571875, 0, 16.757780, 0.497135, 0, 16.781250, 0.504167, 0, 16.757780, 0.497135, 0, 16.741926, 0.571875, 0, 16.781250, 0.504167, 0, 16.741926, 0.571875, 0, 16.716211, 0.561719, 0, 16.757780, 0.497135, 0, 16.716211, 0.561719, 0, 16.741926, 0.571875, 0, 16.757780, 0.497135, 0, 16.716211, 0.561719, 0, 16.734766, 0.488542, 0, 16.757780, 0.497135, 0, 16.734766, 0.488542, 0, 16.716211, 0.561719, 0, 16.757780, 0.497135, 0, 16.716211, 0.561719, 0, 16.690105, 0.550000, 0, 16.734766, 0.488542, 0, 16.690105, 0.550000, 0, 16.716211, 0.561719, 0, 16.734766, 0.488542, 0, 16.690105, 0.550000, 0, 16.712208, 0.478385, 0, 16.734766, 0.488542, 0, 16.712208, 0.478385, 0, 16.690105, 0.550000, 0, 16.734766, 0.488542, 0, 16.690105, 0.550000, 0, 16.690105, 0.466667, 0, 16.712208, 0.478385, 0, 16.690105, 0.466667, 0, 16.690105, 0.550000, 0, 16.712208, 0.478385, 0, 17.140104, 0.327344, 0, 17.044271, 0.283333, 0, 17.044271, 0.262500, 0, 17.044271, 0.283333, 0, 17.140104, 0.327344, 0, 17.044271, 0.262500, 0, 17.140104, 0.327344, 0, 17.044271, 0.262500, 0, 17.041260, 0.220304, 0, 17.044271, 0.262500, 0, 17.140104, 0.327344, 0, 17.041260, 0.220304, 0, 17.140104, 0, 0, 17.140104, 0.327344, 0, 17.041260, 0.220304, 0, 17.140104, 0.327344, 0, 17.140104, 0, 0, 17.041260, 0.220304, 0, 17.140104, 0, 0, 17.041260, 0.220304, 0, 17.032227, 0.182520, 0, 17.041260, 0.220304, 0, 17.140104, 0, 0, 17.032227, 0.182520, 0, 17.044271, 0.083333, 0, 17.140104, 0, 0, 17.032227, 0.182520, 0, 17.140104, 0, 0, 17.044271, 0.083333, 0, 17.032227, 0.182520, 0, 17.044271, 0.083333, 0, 17.044271, 0, 0, 17.140104, 0, 0, 17.044271, 0, 0, 17.044271, 0.083333, 0, 17.140104, 0, 0, 17.032227, 0.182520, 0, 17.017172, 0.149146, 0, 17.044271, 0.083333, 0, 17.017172, 0.149146, 0, 17.032227, 0.182520, 0, 17.044271, 0.083333, 0, 17.017172, 0.149146, 0, 16.996094, 0.120182, 0, 17.044271, 0.083333, 0, 16.996094, 0.120182, 0, 17.017172, 0.149146, 0, 17.044271, 0.083333, 0, 16.996094, 0.120182, 0, 17.026855, 0.059367, 0, 17.044271, 0.083333, 0, 17.026855, 0.059367, 0, 16.996094, 0.120182, 0, 17.044271, 0.083333, 0, 16.996094, 0.120182, 0, 16.970037, 0.096769, 0, 17.026855, 0.059367, 0, 16.970037, 0.096769, 0, 16.996094, 0.120182, 0, 17.026855, 0.059367, 0, 16.970037, 0.096769, 0, 17.007421, 0.038770, 0, 17.026855, 0.059367, 0, 17.007421, 0.038770, 0, 16.970037, 0.096769, 0, 17.026855, 0.059367, 0, 17.007421, 0.038770, 0, 16.970037, 0.096769, 0, 16.985970, 0.021541, 0, 16.970037, 0.096769, 0, 17.007421, 0.038770, 0, 16.985970, 0.021541, 0, 16.970037, 0.096769, 0, 16.940039, 0.080046, 0, 16.985970, 0.021541, 0, 16.940039, 0.080046, 0, 16.970037, 0.096769, 0, 16.985970, 0.021541, 0, 16.940039, 0.080046, 0, 16.962500, 0.007682, 0, 16.985970, 0.021541, 0, 16.962500, 0.007682, 0, 16.940039, 0.080046, 0, 16.985970, 0.021541, 0, 16.962500, 0.007682, 0, 16.940039, 0.080046, 0, 16.936653, -0.002970, 0, 16.940039, 0.080046, 0, 16.962500, 0.007682, 0, 16.936653, -0.002970, 0, 16.940039, 0.080046, 0, 16.906103, 0.070011, 0, 16.936653, -0.002970, 0, 16.906103, 0.070011, 0, 16.940039, 0.080046, 0, 16.936653, -0.002970, 0, 16.906103, 0.070011, 0, 16.908073, -0.010579, 0, 16.936653, -0.002970, 0, 16.908073, -0.010579, 0, 16.906103, 0.070011, 0, 16.936653, -0.002970, 0, 16.906103, 0.070011, 0, 16.868231, 0.066667, 0, 16.908073, -0.010579, 0, 16.868231, 0.066667, 0, 16.906103, 0.070011, 0, 16.908073, -0.010579, 0, 16.868231, 0.066667, 0, 16.876759, -0.015145, 0, 16.908073, -0.010579, 0, 16.876759, -0.015145, 0, 16.868231, 0.066667, 0, 16.908073, -0.010579, 0, 16.868231, 0.066667, 0, 16.842709, -0.016667, 0, 16.876759, -0.015145, 0, 16.842709, -0.016667, 0, 16.868231, 0.066667, 0, 16.876759, -0.015145, 0, 16.840837, 0.068441, 0, 16.842709, -0.016667, 0, 16.868231, 0.066667, 0, 16.842709, -0.016667, 0, 16.840837, 0.068441, 0, 16.868231, 0.066667, 0, 16.840837, 0.068441, 0, 16.800213, -0.013525, 0, 16.842709, -0.016667, 0, 16.800213, -0.013525, 0, 16.840837, 0.068441, 0, 16.842709, -0.016667, 0, 16.816473, 0.073763, 0, 16.800213, -0.013525, 0, 16.840837, 0.068441, 0, 16.800213, -0.013525, 0, 16.816473, 0.073763, 0, 16.840837, 0.068441, 0, 16.762304, -0.004102, 0, 16.800213, -0.013525, 0, 16.816473, 0.073763, 0, 16.800213, -0.013525, 0, 16.762304, -0.004102, 0, 16.816473, 0.073763, 0, 16.795134, 0.082633, 0, 16.762304, -0.004102, 0, 16.816473, 0.073763, 0, 16.762304, -0.004102, 0, 16.795134, 0.082633, 0, 16.816473, 0.073763, 0, 16.728987, 0.011605, 0, 16.762304, -0.004102, 0, 16.795134, 0.082633, 0, 16.762304, -0.004102, 0, 16.728987, 0.011605, 0, 16.795134, 0.082633, 0, 16.795134, 0.082633, 0, 16.776823, 0.095052, 0, 16.728987, 0.011605, 0, 16.776823, 0.095052, 0, 16.795134, 0.082633, 0, 16.728987, 0.011605, 0, 16.776823, 0.095052, 0, 16.700262, 0.033594, 0, 16.728987, 0.011605, 0, 16.700262, 0.033594, 0, 16.776823, 0.095052, 0, 16.728987, 0.011605, 0, 16.776823, 0.095052, 0, 16.762127, 0.110514, 0, 16.700262, 0.033594, 0, 16.762127, 0.110514, 0, 16.776823, 0.095052, 0, 16.700262, 0.033594, 0, 16.762127, 0.110514, 0, 16.677132, 0.060856, 0, 16.700262, 0.033594, 0, 16.677132, 0.060856, 0, 16.762127, 0.110514, 0, 16.700262, 0.033594, 0, 16.762127, 0.110514, 0, 16.751629, 0.128516, 0, 16.677132, 0.060856, 0, 16.751629, 0.128516, 0, 16.762127, 0.110514, 0, 16.677132, 0.060856, 0, 16.751629, 0.128516, 0, 16.660612, 0.092383, 0, 16.677132, 0.060856, 0, 16.660612, 0.092383, 0, 16.751629, 0.128516, 0, 16.677132, 0.060856, 0, 16.650702, 0.128174, 0, 16.660612, 0.092383, 0, 16.751629, 0.128516, 0, 16.660612, 0.092383, 0, 16.650702, 0.128174, 0, 16.751629, 0.128516, 0, 16.745329, 0.149056, 0, 16.650702, 0.128174, 0, 16.751629, 0.128516, 0, 16.650702, 0.128174, 0, 16.745329, 0.149056, 0, 16.751629, 0.128516, 0, 16.647396, 0.168229, 0, 16.650702, 0.128174, 0, 16.745329, 0.149056, 0, 16.650702, 0.128174, 0, 16.647396, 0.168229, 0, 16.745329, 0.149056, 0, 16.743231, 0.172135, 0, 16.647396, 0.168229, 0, 16.745329, 0.149056, 0, 16.647396, 0.168229, 0, 16.743231, 0.172135, 0, 16.745329, 0.149056, 0, 17.705730, 0.550000, 0, 17.705730, 0.466667, 0, 17.705730, 0.550000, 0, 17.705730, 0.550000, 0, 17.705730, 0.466667, 0, 17.705730, 0.466667, 0, 17.705730, 0.466667, 0, 17.685223, 0.478385, 0, 17.705730, 0.466667, 0, 17.705730, 0.466667, 0, 17.685223, 0.478385, 0, 17.685223, 0.478385, 0, 17.685223, 0.478385, 0, 17.664324, 0.488542, 0, 17.685223, 0.478385, 0, 17.685223, 0.478385, 0, 17.664324, 0.488542, 0, 17.664324, 0.488542, 0, 17.664324, 0.488542, 0, 17.643034, 0.497135, 0, 17.664324, 0.488542, 0, 17.664324, 0.488542, 0, 17.643034, 0.497135, 0, 17.643034, 0.497135, 0, 17.643034, 0.497135, 0, 17.621355, 0.504167, 0, 17.643034, 0.497135, 0, 17.643034, 0.497135, 0, 17.621355, 0.504167, 0, 17.621355, 0.504167, 0, 17.621355, 0.504167, 0, 17.599285, 0.509635, 0, 17.621355, 0.504167, 0, 17.621355, 0.504167, 0, 17.599285, 0.509635, 0, 17.599285, 0.509635, 0, 17.599285, 0.509635, 0, 17.576824, 0.513542, 0, 17.599285, 0.509635, 0, 17.599285, 0.509635, 0, 17.576824, 0.513542, 0, 17.576824, 0.513542, 0, 17.576824, 0.513542, 0, 17.553972, 0.515885, 0, 17.576824, 0.513542, 0, 17.576824, 0.513542, 0, 17.553972, 0.515885, 0, 17.553972, 0.515885, 0, 17.553972, 0.515885, 0, 17.530729, 0.516667, 0, 17.553972, 0.515885, 0, 17.553972, 0.515885, 0, 17.530729, 0.516667, 0, 17.530729, 0.516667, 0, 17.530729, 0.516667, 0, 17.497282, 0.515251, 0, 17.530729, 0.516667, 0, 17.530729, 0.516667, 0, 17.497282, 0.515251, 0, 17.497282, 0.515251, 0, 17.497282, 0.515251, 0, 17.468294, 0.511003, 0, 17.497282, 0.515251, 0, 17.497282, 0.515251, 0, 17.468294, 0.511003, 0, 17.468294, 0.511003, 0, 17.468294, 0.511003, 0, 17.443768, 0.503923, 0, 17.468294, 0.511003, 0, 17.468294, 0.511003, 0, 17.443768, 0.503923, 0, 17.443768, 0.503923, 0, 17.443768, 0.503923, 0, 17.423698, 0.494010, 0, 17.443768, 0.503923, 0, 17.443768, 0.503923, 0, 17.423698, 0.494010, 0, 17.423698, 0.494010, 0, 17.423698, 0.494010, 0, 17.408089, 0.481266, 0, 17.423698, 0.494010, 0, 17.423698, 0.494010, 0, 17.408089, 0.481266, 0, 17.408089, 0.481266, 0, 17.408089, 0.481266, 0, 17.396940, 0.465690, 0, 17.408089, 0.481266, 0, 17.408089, 0.481266, 0, 17.396940, 0.465690, 0, 17.396940, 0.465690, 0, 17.396940, 0.465690, 0, 17.390251, 0.447282, 0, 17.396940, 0.465690, 0, 17.396940, 0.465690, 0, 17.390251, 0.447282, 0, 17.390251, 0.447282, 0, 17.390251, 0.447282, 0, 17.388021, 0.426042, 0, 17.390251, 0.447282, 0, 17.390251, 0.447282, 0, 17.388021, 0.426042, 0, 17.388021, 0.426042, 0, 17.388021, 0.426042, 0, 17.389616, 0.409766, 0, 17.388021, 0.426042, 0, 17.388021, 0.426042, 0, 17.389616, 0.409766, 0, 17.389616, 0.409766, 0, 17.389616, 0.409766, 0, 17.394402, 0.395313, 0, 17.389616, 0.409766, 0, 17.389616, 0.409766, 0, 17.394402, 0.395313, 0, 17.394402, 0.395313, 0, 17.394402, 0.395313, 0, 17.402376, 0.382682, 0, 17.394402, 0.395313, 0, 17.394402, 0.395313, 0, 17.402376, 0.382682, 0, 17.402376, 0.382682, 0, 17.402376, 0.382682, 0, 17.413542, 0.371875, 0, 17.402376, 0.382682, 0, 17.402376, 0.382682, 0, 17.413542, 0.371875, 0, 17.413542, 0.371875, 0, 17.413542, 0.371875, 0, 17.429525, 0.362109, 0, 17.413542, 0.371875, 0, 17.413542, 0.371875, 0, 17.429525, 0.362109, 0, 17.429525, 0.362109, 0, 17.429525, 0.362109, 0, 17.451954, 0.352604, 0, 17.429525, 0.362109, 0, 17.429525, 0.362109, 0, 17.451954, 0.352604, 0, 17.451954, 0.352604, 0, 17.451954, 0.352604, 0, 17.480827, 0.343359, 0, 17.451954, 0.352604, 0, 17.451954, 0.352604, 0, 17.480827, 0.343359, 0, 17.480827, 0.343359, 0, 17.480827, 0.343359, 0, 17.516146, 0.334375, 0, 17.480827, 0.343359, 0, 17.480827, 0.343359, 0, 17.516146, 0.334375, 0, 17.516146, 0.334375, 0, 17.516146, 0.334375, 0, 17.548958, 0.327083, 0, 17.516146, 0.334375, 0, 17.516146, 0.334375, 0, 17.548958, 0.327083, 0, 17.548958, 0.327083, 0, 17.548958, 0.327083, 0, 17.596306, 0.315177, 0, 17.548958, 0.327083, 0, 17.548958, 0.327083, 0, 17.596306, 0.315177, 0, 17.596306, 0.315177, 0, 17.596306, 0.315177, 0, 17.636263, 0.301074, 0, 17.596306, 0.315177, 0, 17.596306, 0.315177, 0, 17.636263, 0.301074, 0, 17.636263, 0.301074, 0, 17.636263, 0.301074, 0, 17.668831, 0.284774, 0, 17.636263, 0.301074, 0, 17.636263, 0.301074, 0, 17.668831, 0.284774, 0, 17.668831, 0.284774, 0, 17.668831, 0.284774, 0, 17.694012, 0.266276, 0, 17.668831, 0.284774, 0, 17.668831, 0.284774, 0, 17.694012, 0.266276, 0, 17.694012, 0.266276, 0, 17.694012, 0.266276, 0, 17.712811, 0.244735, 0, 17.694012, 0.266276, 0, 17.694012, 0.266276, 0, 17.712811, 0.244735, 0, 17.712811, 0.244735, 0, 17.712811, 0.244735, 0, 17.726238, 0.219303, 0, 17.712811, 0.244735, 0, 17.712811, 0.244735, 0, 17.726238, 0.219303, 0, 17.726238, 0.219303, 0, 17.726238, 0.219303, 0, 17.734295, 0.189982, 0, 17.726238, 0.219303, 0, 17.726238, 0.219303, 0, 17.734295, 0.189982, 0, 17.734295, 0.189982, 0, 17.734295, 0.189982, 0, 17.736980, 0.156771, 0, 17.734295, 0.189982, 0, 17.734295, 0.189982, 0, 17.736980, 0.156771, 0, 17.736980, 0.156771, 0, 17.736980, 0.156771, 0, 17.732960, 0.118872, 0, 17.736980, 0.156771, 0, 17.736980, 0.156771, 0, 17.732960, 0.118872, 0, 17.732960, 0.118872, 0, 17.732960, 0.118872, 0, 17.720900, 0.085124, 0, 17.732960, 0.118872, 0, 17.732960, 0.118872, 0, 17.720900, 0.085124, 0, 17.720900, 0.085124, 0, 17.720900, 0.085124, 0, 17.700798, 0.055526, 0, 17.720900, 0.085124, 0, 17.720900, 0.085124, 0, 17.700798, 0.055526, 0, 17.700798, 0.055526, 0, 17.700798, 0.055526, 0, 17.672657, 0.030078, 0, 17.700798, 0.055526, 0, 17.700798, 0.055526, 0, 17.672657, 0.030078, 0, 17.672657, 0.030078, 0, 17.672657, 0.030078, 0, 17.637484, 0.009627, 0, 17.672657, 0.030078, 0, 17.672657, 0.030078, 0, 17.637484, 0.009627, 0, 17.637484, 0.009627, 0, 17.637484, 0.009627, 0, 17.596289, -0.004980, 0, 17.637484, 0.009627, 0, 17.637484, 0.009627, 0, 17.596289, -0.004980, 0, 17.596289, -0.004980, 0, 17.596289, -0.004980, 0, 17.549072, -0.013745, 0, 17.596289, -0.004980, 0, 17.596289, -0.004980, 0, 17.549072, -0.013745, 0, 17.549072, -0.013745, 0, 17.549072, -0.013745, 0, 17.495832, -0.016667, 0, 17.549072, -0.013745, 0, 17.549072, -0.013745, 0, 17.495832, -0.016667, 0, 17.495832, -0.016667, 0, 17.495832, -0.016667, 0, 17.472151, -0.016146, 0, 17.495832, -0.016667, 0, 17.495832, -0.016667, 0, 17.472151, -0.016146, 0, 17.472151, -0.016146, 0, 17.472151, -0.016146, 0, 17.447981, -0.014583, 0, 17.472151, -0.016146, 0, 17.472151, -0.016146, 0, 17.447981, -0.014583, 0, 17.447981, -0.014583, 0, 17.447981, -0.014583, 0, 17.423323, -0.011979, 0, 17.447981, -0.014583, 0, 17.447981, -0.014583, 0, 17.423323, -0.011979, 0, 17.423323, -0.011979, 0, 17.423323, -0.011979, 0, 17.398176, -0.008333, 0, 17.423323, -0.011979, 0, 17.423323, -0.011979, 0, 17.398176, -0.008333, 0, 17.398176, -0.008333, 0, 17.398176, -0.008333, 0, 17.372444, -0.003646, 0, 17.398176, -0.008333, 0, 17.398176, -0.008333, 0, 17.372444, -0.003646, 0, 17.372444, -0.003646, 0, 17.372444, -0.003646, 0, 17.346029, 0.002083, 0, 17.372444, -0.003646, 0, 17.372444, -0.003646, 0, 17.346029, 0.002083, 0, 17.346029, 0.002083, 0, 17.346029, 0.002083, 0, 17.318930, 0.008854, 0, 17.346029, 0.002083, 0, 17.346029, 0.002083, 0, 17.318930, 0.008854, 0, 17.318930, 0.008854, 0, 17.318930, 0.008854, 0, 17.291145, 0.016667, 0, 17.318930, 0.008854, 0, 17.318930, 0.008854, 0, 17.291145, 0.016667, 0, 17.291145, 0.016667, 0, 17.291145, 0.016667, 0, 17.291145, 0.116667, 0, 17.291145, 0.016667, 0, 17.291145, 0.016667, 0, 17.291145, 0.116667, 0, 17.291145, 0.116667, 0, 17.291145, 0.116667, 0, 17.317610, 0.104948, 0, 17.291145, 0.116667, 0, 17.291145, 0.116667, 0, 17.317610, 0.104948, 0, 17.317610, 0.104948, 0, 17.317610, 0.104948, 0, 17.343880, 0.094792, 0, 17.317610, 0.104948, 0, 17.317610, 0.104948, 0, 17.343880, 0.094792, 0, 17.343880, 0.094792, 0, 17.343880, 0.094792, 0, 17.369955, 0.086198, 0, 17.343880, 0.094792, 0, 17.343880, 0.094792, 0, 17.369955, 0.086198, 0, 17.369955, 0.086198, 0, 17.369955, 0.086198, 0, 17.395834, 0.079167, 0, 17.369955, 0.086198, 0, 17.369955, 0.086198, 0, 17.395834, 0.079167, 0, 17.395834, 0.079167, 0, 17.395834, 0.079167, 0, 17.421551, 0.073698, 0, 17.395834, 0.079167, 0, 17.395834, 0.079167, 0, 17.421551, 0.073698, 0, 17.421551, 0.073698, 0, 17.421551, 0.073698, 0, 17.447136, 0.069792, 0, 17.421551, 0.073698, 0, 17.421551, 0.073698, 0, 17.447136, 0.069792, 0, 17.447136, 0.069792, 0, 17.447136, 0.069792, 0, 17.472591, 0.067448, 0, 17.447136, 0.069792, 0, 17.447136, 0.069792, 0, 17.472591, 0.067448, 0, 17.472591, 0.067448, 0, 17.472591, 0.067448, 0, 17.497917, 0.066667, 0, 17.472591, 0.067448, 0, 17.472591, 0.067448, 0, 17.497917, 0.066667, 0, 17.497917, 0.066667, 0, 17.497917, 0.066667, 0, 17.529818, 0.068058, 0, 17.497917, 0.066667, 0, 17.497917, 0.066667, 0, 17.529818, 0.068058, 0, 17.529818, 0.068058, 0, 17.529818, 0.068058, 0, 17.557814, 0.072233, 0, 17.529818, 0.068058, 0, 17.529818, 0.068058, 0, 17.557814, 0.072233, 0, 17.557814, 0.072233, 0, 17.557814, 0.072233, 0, 17.581902, 0.079191, 0, 17.557814, 0.072233, 0, 17.557814, 0.072233, 0, 17.581902, 0.079191, 0, 17.581902, 0.079191, 0, 17.581902, 0.079191, 0, 17.602083, 0.088932, 0, 17.581902, 0.079191, 0, 17.581902, 0.079191, 0, 17.602083, 0.088932, 0, 17.602083, 0.088932, 0, 17.602083, 0.088932, 0, 17.618034, 0.101213, 0, 17.602083, 0.088932, 0, 17.602083, 0.088932, 0, 17.618034, 0.101213, 0, 17.618034, 0.101213, 0, 17.618034, 0.101213, 0, 17.629427, 0.115788, 0, 17.618034, 0.101213, 0, 17.618034, 0.101213, 0, 17.629427, 0.115788, 0, 17.629427, 0.115788, 0, 17.629427, 0.115788, 0, 17.636263, 0.132658, 0, 17.629427, 0.115788, 0, 17.629427, 0.115788, 0, 17.636263, 0.132658, 0, 17.636263, 0.132658, 0, 17.636263, 0.132658, 0, 17.638542, 0.151823, 0, 17.636263, 0.132658, 0, 17.636263, 0.132658, 0, 17.638542, 0.151823, 0, 17.638542, 0.151823, 0, 17.638542, 0.151823, 0, 17.636898, 0.169482, 0, 17.638542, 0.151823, 0, 17.638542, 0.151823, 0, 17.636898, 0.169482, 0, 17.636898, 0.169482, 0, 17.636898, 0.169482, 0, 17.631966, 0.184961, 0, 17.636898, 0.169482, 0, 17.636898, 0.169482, 0, 17.631966, 0.184961, 0, 17.631966, 0.184961, 0, 17.631966, 0.184961, 0, 17.623747, 0.198258, 0, 17.631966, 0.184961, 0, 17.631966, 0.184961, 0, 17.623747, 0.198258, 0, 17.623747, 0.198258, 0, 17.623747, 0.198258, 0, 17.612240, 0.209375, 0, 17.623747, 0.198258, 0, 17.623747, 0.198258, 0, 17.612240, 0.209375, 0, 17.612240, 0.209375, 0, 17.612240, 0.209375, 0, 17.595167, 0.219303, 0, 17.612240, 0.209375, 0, 17.612240, 0.209375, 0, 17.595167, 0.219303, 0, 17.595167, 0.219303, 0, 17.595167, 0.219303, 0, 17.570248, 0.229036, 0, 17.595167, 0.219303, 0, 17.595167, 0.219303, 0, 17.570248, 0.229036, 0, 17.570248, 0.229036, 0, 17.570248, 0.229036, 0, 17.537483, 0.238574, 0, 17.570248, 0.229036, 0, 17.570248, 0.229036, 0, 17.537483, 0.238574, 0, 17.537483, 0.238574, 0, 17.537483, 0.238574, 0, 17.496876, 0.247917, 0, 17.537483, 0.238574, 0, 17.537483, 0.238574, 0, 17.496876, 0.247917, 0, 17.496876, 0.247917, 0, 17.496876, 0.247917, 0, 17.463543, 0.255729, 0, 17.496876, 0.247917, 0, 17.496876, 0.247917, 0, 17.463543, 0.255729, 0, 17.463543, 0.255729, 0, 17.463543, 0.255729, 0, 17.422102, 0.266642, 0, 17.463543, 0.255729, 0, 17.463543, 0.255729, 0, 17.422102, 0.266642, 0, 17.422102, 0.266642, 0, 17.422102, 0.266642, 0, 17.386848, 0.280111, 0, 17.422102, 0.266642, 0, 17.422102, 0.266642, 0, 17.386848, 0.280111, 0, 17.386848, 0.280111, 0, 17.386848, 0.280111, 0, 17.357780, 0.296134, 0, 17.386848, 0.280111, 0, 17.386848, 0.280111, 0, 17.357780, 0.296134, 0, 17.357780, 0.296134, 0, 17.357780, 0.296134, 0, 17.334896, 0.314714, 0, 17.357780, 0.296134, 0, 17.357780, 0.296134, 0, 17.334896, 0.314714, 0, 17.334896, 0.314714, 0, 17.334896, 0.314714, 0, 17.317579, 0.336434, 0, 17.334896, 0.314714, 0, 17.334896, 0.314714, 0, 17.317579, 0.336434, 0, 17.317579, 0.336434, 0, 17.317579, 0.336434, 0, 17.305208, 0.361882, 0, 17.317579, 0.336434, 0, 17.317579, 0.336434, 0, 17.305208, 0.361882, 0, 17.305208, 0.361882, 0, 17.305208, 0.361882, 0, 17.297787, 0.391056, 0, 17.305208, 0.361882, 0, 17.305208, 0.361882, 0, 17.297787, 0.391056, 0, 17.297787, 0.391056, 0, 17.297787, 0.391056, 0, 17.295313, 0.423958, 0, 17.297787, 0.391056, 0, 17.297787, 0.391056, 0, 17.295313, 0.423958, 0, 17.295313, 0.423958, 0, 17.295313, 0.423958, 0, 17.298958, 0.463631, 0, 17.295313, 0.423958, 0, 17.295313, 0.423958, 0, 17.298958, 0.463631, 0, 17.298958, 0.463631, 0, 17.298958, 0.463631, 0, 17.309896, 0.498535, 0, 17.298958, 0.463631, 0, 17.298958, 0.463631, 0, 17.309896, 0.498535, 0, 17.309896, 0.498535, 0, 17.309896, 0.498535, 0, 17.328125, 0.528670, 0, 17.309896, 0.498535, 0, 17.309896, 0.498535, 0, 17.328125, 0.528670, 0, 17.328125, 0.528670, 0, 17.328125, 0.528670, 0, 17.353645, 0.554036, 0, 17.328125, 0.528670, 0, 17.328125, 0.528670, 0, 17.353645, 0.554036, 0, 17.353645, 0.554036, 0, 17.353645, 0.554036, 0, 17.385872, 0.574145, 0, 17.353645, 0.554036, 0, 17.353645, 0.554036, 0, 17.385872, 0.574145, 0, 17.385872, 0.574145, 0, 17.385872, 0.574145, 0, 17.424219, 0.588509, 0, 17.385872, 0.574145, 0, 17.385872, 0.574145, 0, 17.424219, 0.588509, 0, 17.424219, 0.588509, 0, 17.424219, 0.588509, 0, 17.468685, 0.597127, 0, 17.424219, 0.588509, 0, 17.424219, 0.588509, 0, 17.468685, 0.597127, 0, 17.468685, 0.597127, 0, 17.468685, 0.597127, 0, 17.519270, 0.600000, 0, 17.468685, 0.597127, 0, 17.468685, 0.597127, 0, 17.519270, 0.600000, 0, 17.519270, 0.600000, 0, 17.519270, 0.600000, 0, 17.545443, 0.599219, 0, 17.519270, 0.600000, 0, 17.519270, 0.600000, 0, 17.545443, 0.599219, 0, 17.545443, 0.599219, 0, 17.545443, 0.599219, 0, 17.570833, 0.596875, 0, 17.545443, 0.599219, 0, 17.545443, 0.599219, 0, 17.570833, 0.596875, 0, 17.570833, 0.596875, 0, 17.570833, 0.596875, 0, 17.595442, 0.592969, 0, 17.570833, 0.596875, 0, 17.570833, 0.596875, 0, 17.595442, 0.592969, 0, 17.595442, 0.592969, 0, 17.595442, 0.592969, 0, 17.619270, 0.587500, 0, 17.595442, 0.592969, 0, 17.595442, 0.592969, 0, 17.619270, 0.587500, 0, 17.619270, 0.587500, 0, 17.619270, 0.587500, 0, 17.642254, 0.580469, 0, 17.619270, 0.587500, 0, 17.619270, 0.587500, 0, 17.642254, 0.580469, 0, 17.642254, 0.580469, 0, 17.642254, 0.580469, 0, 17.664324, 0.571875, 0, 17.642254, 0.580469, 0, 17.642254, 0.580469, 0, 17.664324, 0.571875, 0, 17.664324, 0.571875, 0, 17.664324, 0.571875, 0, 17.685482, 0.561719, 0, 17.664324, 0.571875, 0, 17.664324, 0.571875, 0, 17.685482, 0.561719, 0, 17.685482, 0.561719, 0, 17.291145, 0.116667, 0, 17.291145, 0.016667, 0, 17.317610, 0.104948, 0, 17.291145, 0.016667, 0, 17.291145, 0.116667, 0, 17.317610, 0.104948, 0, 17.343880, 0.094792, 0, 17.317610, 0.104948, 0, 17.291145, 0.016667, 0, 17.317610, 0.104948, 0, 17.343880, 0.094792, 0, 17.291145, 0.016667, 0, 17.343880, 0.094792, 0, 17.291145, 0.016667, 0, 17.318930, 0.008854, 0, 17.291145, 0.016667, 0, 17.343880, 0.094792, 0, 17.318930, 0.008854, 0, 17.369955, 0.086198, 0, 17.343880, 0.094792, 0, 17.318930, 0.008854, 0, 17.343880, 0.094792, 0, 17.369955, 0.086198, 0, 17.318930, 0.008854, 0, 17.369955, 0.086198, 0, 17.318930, 0.008854, 0, 17.346029, 0.002083, 0, 17.318930, 0.008854, 0, 17.369955, 0.086198, 0, 17.346029, 0.002083, 0, 17.369955, 0.086198, 0, 17.346029, 0.002083, 0, 17.372444, -0.003646, 0, 17.346029, 0.002083, 0, 17.369955, 0.086198, 0, 17.372444, -0.003646, 0, 17.369955, 0.086198, 0, 17.372444, -0.003646, 0, 17.395834, 0.079167, 0, 17.372444, -0.003646, 0, 17.369955, 0.086198, 0, 17.395834, 0.079167, 0, 17.395834, 0.079167, 0, 17.372444, -0.003646, 0, 17.398176, -0.008333, 0, 17.372444, -0.003646, 0, 17.395834, 0.079167, 0, 17.398176, -0.008333, 0, 17.395834, 0.079167, 0, 17.398176, -0.008333, 0, 17.421551, 0.073698, 0, 17.398176, -0.008333, 0, 17.395834, 0.079167, 0, 17.421551, 0.073698, 0, 17.421551, 0.073698, 0, 17.398176, -0.008333, 0, 17.423323, -0.011979, 0, 17.398176, -0.008333, 0, 17.421551, 0.073698, 0, 17.423323, -0.011979, 0, 17.421551, 0.073698, 0, 17.423323, -0.011979, 0, 17.447136, 0.069792, 0, 17.423323, -0.011979, 0, 17.421551, 0.073698, 0, 17.447136, 0.069792, 0, 17.447136, 0.069792, 0, 17.423323, -0.011979, 0, 17.447981, -0.014583, 0, 17.423323, -0.011979, 0, 17.447136, 0.069792, 0, 17.447981, -0.014583, 0, 17.447136, 0.069792, 0, 17.447981, -0.014583, 0, 17.472591, 0.067448, 0, 17.447981, -0.014583, 0, 17.447136, 0.069792, 0, 17.472591, 0.067448, 0, 17.447981, -0.014583, 0, 17.472151, -0.016146, 0, 17.472591, 0.067448, 0, 17.472151, -0.016146, 0, 17.447981, -0.014583, 0, 17.472591, 0.067448, 0, 17.472591, 0.067448, 0, 17.472151, -0.016146, 0, 17.497917, 0.066667, 0, 17.472151, -0.016146, 0, 17.472591, 0.067448, 0, 17.497917, 0.066667, 0, 17.472151, -0.016146, 0, 17.495832, -0.016667, 0, 17.497917, 0.066667, 0, 17.495832, -0.016667, 0, 17.472151, -0.016146, 0, 17.497917, 0.066667, 0, 17.529818, 0.068058, 0, 17.497917, 0.066667, 0, 17.495832, -0.016667, 0, 17.497917, 0.066667, 0, 17.529818, 0.068058, 0, 17.495832, -0.016667, 0, 17.549072, -0.013745, 0, 17.529818, 0.068058, 0, 17.495832, -0.016667, 0, 17.529818, 0.068058, 0, 17.549072, -0.013745, 0, 17.495832, -0.016667, 0, 17.557814, 0.072233, 0, 17.529818, 0.068058, 0, 17.549072, -0.013745, 0, 17.529818, 0.068058, 0, 17.557814, 0.072233, 0, 17.549072, -0.013745, 0, 17.596289, -0.004980, 0, 17.557814, 0.072233, 0, 17.549072, -0.013745, 0, 17.557814, 0.072233, 0, 17.596289, -0.004980, 0, 17.549072, -0.013745, 0, 17.581902, 0.079191, 0, 17.557814, 0.072233, 0, 17.596289, -0.004980, 0, 17.557814, 0.072233, 0, 17.581902, 0.079191, 0, 17.596289, -0.004980, 0, 17.581902, 0.079191, 0, 17.596289, -0.004980, 0, 17.637484, 0.009627, 0, 17.596289, -0.004980, 0, 17.581902, 0.079191, 0, 17.637484, 0.009627, 0, 17.602083, 0.088932, 0, 17.581902, 0.079191, 0, 17.637484, 0.009627, 0, 17.581902, 0.079191, 0, 17.602083, 0.088932, 0, 17.637484, 0.009627, 0, 17.602083, 0.088932, 0, 17.637484, 0.009627, 0, 17.672657, 0.030078, 0, 17.637484, 0.009627, 0, 17.602083, 0.088932, 0, 17.672657, 0.030078, 0, 17.618034, 0.101213, 0, 17.602083, 0.088932, 0, 17.672657, 0.030078, 0, 17.602083, 0.088932, 0, 17.618034, 0.101213, 0, 17.672657, 0.030078, 0, 17.618034, 0.101213, 0, 17.672657, 0.030078, 0, 17.700798, 0.055526, 0, 17.672657, 0.030078, 0, 17.618034, 0.101213, 0, 17.700798, 0.055526, 0, 17.629427, 0.115788, 0, 17.618034, 0.101213, 0, 17.700798, 0.055526, 0, 17.618034, 0.101213, 0, 17.629427, 0.115788, 0, 17.700798, 0.055526, 0, 17.629427, 0.115788, 0, 17.700798, 0.055526, 0, 17.720900, 0.085124, 0, 17.700798, 0.055526, 0, 17.629427, 0.115788, 0, 17.720900, 0.085124, 0, 17.636263, 0.132658, 0, 17.629427, 0.115788, 0, 17.720900, 0.085124, 0, 17.629427, 0.115788, 0, 17.636263, 0.132658, 0, 17.720900, 0.085124, 0, 17.636263, 0.132658, 0, 17.720900, 0.085124, 0, 17.732960, 0.118872, 0, 17.720900, 0.085124, 0, 17.636263, 0.132658, 0, 17.732960, 0.118872, 0, 17.638542, 0.151823, 0, 17.636263, 0.132658, 0, 17.732960, 0.118872, 0, 17.636263, 0.132658, 0, 17.638542, 0.151823, 0, 17.732960, 0.118872, 0, 17.638542, 0.151823, 0, 17.732960, 0.118872, 0, 17.736980, 0.156771, 0, 17.732960, 0.118872, 0, 17.638542, 0.151823, 0, 17.736980, 0.156771, 0, 17.736980, 0.156771, 0, 17.636898, 0.169482, 0, 17.638542, 0.151823, 0, 17.636898, 0.169482, 0, 17.736980, 0.156771, 0, 17.638542, 0.151823, 0, 17.734295, 0.189982, 0, 17.636898, 0.169482, 0, 17.736980, 0.156771, 0, 17.636898, 0.169482, 0, 17.734295, 0.189982, 0, 17.736980, 0.156771, 0, 17.726238, 0.219303, 0, 17.636898, 0.169482, 0, 17.734295, 0.189982, 0, 17.636898, 0.169482, 0, 17.726238, 0.219303, 0, 17.734295, 0.189982, 0, 17.631966, 0.184961, 0, 17.636898, 0.169482, 0, 17.726238, 0.219303, 0, 17.636898, 0.169482, 0, 17.631966, 0.184961, 0, 17.726238, 0.219303, 0, 17.712811, 0.244735, 0, 17.631966, 0.184961, 0, 17.726238, 0.219303, 0, 17.631966, 0.184961, 0, 17.712811, 0.244735, 0, 17.726238, 0.219303, 0, 17.623747, 0.198258, 0, 17.631966, 0.184961, 0, 17.712811, 0.244735, 0, 17.631966, 0.184961, 0, 17.623747, 0.198258, 0, 17.712811, 0.244735, 0, 17.694012, 0.266276, 0, 17.623747, 0.198258, 0, 17.712811, 0.244735, 0, 17.623747, 0.198258, 0, 17.694012, 0.266276, 0, 17.712811, 0.244735, 0, 17.668831, 0.284774, 0, 17.623747, 0.198258, 0, 17.694012, 0.266276, 0, 17.623747, 0.198258, 0, 17.668831, 0.284774, 0, 17.694012, 0.266276, 0, 17.612240, 0.209375, 0, 17.623747, 0.198258, 0, 17.668831, 0.284774, 0, 17.623747, 0.198258, 0, 17.612240, 0.209375, 0, 17.668831, 0.284774, 0, 17.636263, 0.301074, 0, 17.612240, 0.209375, 0, 17.668831, 0.284774, 0, 17.612240, 0.209375, 0, 17.636263, 0.301074, 0, 17.668831, 0.284774, 0, 17.595167, 0.219303, 0, 17.612240, 0.209375, 0, 17.636263, 0.301074, 0, 17.612240, 0.209375, 0, 17.595167, 0.219303, 0, 17.636263, 0.301074, 0, 17.596306, 0.315177, 0, 17.595167, 0.219303, 0, 17.636263, 0.301074, 0, 17.595167, 0.219303, 0, 17.596306, 0.315177, 0, 17.636263, 0.301074, 0, 17.570248, 0.229036, 0, 17.595167, 0.219303, 0, 17.596306, 0.315177, 0, 17.595167, 0.219303, 0, 17.570248, 0.229036, 0, 17.596306, 0.315177, 0, 17.596306, 0.315177, 0, 17.537483, 0.238574, 0, 17.570248, 0.229036, 0, 17.537483, 0.238574, 0, 17.596306, 0.315177, 0, 17.570248, 0.229036, 0, 17.596306, 0.315177, 0, 17.548958, 0.327083, 0, 17.537483, 0.238574, 0, 17.548958, 0.327083, 0, 17.596306, 0.315177, 0, 17.537483, 0.238574, 0, 17.496876, 0.247917, 0, 17.537483, 0.238574, 0, 17.548958, 0.327083, 0, 17.537483, 0.238574, 0, 17.496876, 0.247917, 0, 17.548958, 0.327083, 0, 17.516146, 0.334375, 0, 17.496876, 0.247917, 0, 17.548958, 0.327083, 0, 17.496876, 0.247917, 0, 17.516146, 0.334375, 0, 17.548958, 0.327083, 0, 17.463543, 0.255729, 0, 17.496876, 0.247917, 0, 17.516146, 0.334375, 0, 17.496876, 0.247917, 0, 17.463543, 0.255729, 0, 17.516146, 0.334375, 0, 17.480827, 0.343359, 0, 17.463543, 0.255729, 0, 17.516146, 0.334375, 0, 17.463543, 0.255729, 0, 17.480827, 0.343359, 0, 17.516146, 0.334375, 0, 17.422102, 0.266642, 0, 17.463543, 0.255729, 0, 17.480827, 0.343359, 0, 17.463543, 0.255729, 0, 17.422102, 0.266642, 0, 17.480827, 0.343359, 0, 17.451954, 0.352604, 0, 17.422102, 0.266642, 0, 17.480827, 0.343359, 0, 17.422102, 0.266642, 0, 17.451954, 0.352604, 0, 17.480827, 0.343359, 0, 17.451954, 0.352604, 0, 17.429525, 0.362109, 0, 17.422102, 0.266642, 0, 17.429525, 0.362109, 0, 17.451954, 0.352604, 0, 17.422102, 0.266642, 0, 17.429525, 0.362109, 0, 17.386848, 0.280111, 0, 17.422102, 0.266642, 0, 17.386848, 0.280111, 0, 17.429525, 0.362109, 0, 17.422102, 0.266642, 0, 17.429525, 0.362109, 0, 17.413542, 0.371875, 0, 17.386848, 0.280111, 0, 17.413542, 0.371875, 0, 17.429525, 0.362109, 0, 17.386848, 0.280111, 0, 17.413542, 0.371875, 0, 17.357780, 0.296134, 0, 17.386848, 0.280111, 0, 17.357780, 0.296134, 0, 17.413542, 0.371875, 0, 17.386848, 0.280111, 0, 17.334896, 0.314714, 0, 17.357780, 0.296134, 0, 17.413542, 0.371875, 0, 17.357780, 0.296134, 0, 17.334896, 0.314714, 0, 17.413542, 0.371875, 0, 17.413542, 0.371875, 0, 17.402376, 0.382682, 0, 17.334896, 0.314714, 0, 17.402376, 0.382682, 0, 17.413542, 0.371875, 0, 17.334896, 0.314714, 0, 17.402376, 0.382682, 0, 17.317579, 0.336434, 0, 17.334896, 0.314714, 0, 17.317579, 0.336434, 0, 17.402376, 0.382682, 0, 17.334896, 0.314714, 0, 17.402376, 0.382682, 0, 17.394402, 0.395313, 0, 17.317579, 0.336434, 0, 17.394402, 0.395313, 0, 17.402376, 0.382682, 0, 17.317579, 0.336434, 0, 17.394402, 0.395313, 0, 17.305208, 0.361882, 0, 17.317579, 0.336434, 0, 17.305208, 0.361882, 0, 17.394402, 0.395313, 0, 17.317579, 0.336434, 0, 17.394402, 0.395313, 0, 17.389616, 0.409766, 0, 17.305208, 0.361882, 0, 17.389616, 0.409766, 0, 17.394402, 0.395313, 0, 17.305208, 0.361882, 0, 17.389616, 0.409766, 0, 17.297787, 0.391056, 0, 17.305208, 0.361882, 0, 17.297787, 0.391056, 0, 17.389616, 0.409766, 0, 17.305208, 0.361882, 0, 17.295313, 0.423958, 0, 17.297787, 0.391056, 0, 17.389616, 0.409766, 0, 17.297787, 0.391056, 0, 17.295313, 0.423958, 0, 17.389616, 0.409766, 0, 17.388021, 0.426042, 0, 17.295313, 0.423958, 0, 17.389616, 0.409766, 0, 17.295313, 0.423958, 0, 17.388021, 0.426042, 0, 17.389616, 0.409766, 0, 17.298958, 0.463631, 0, 17.295313, 0.423958, 0, 17.388021, 0.426042, 0, 17.295313, 0.423958, 0, 17.298958, 0.463631, 0, 17.388021, 0.426042, 0, 17.298958, 0.463631, 0, 17.388021, 0.426042, 0, 17.390251, 0.447282, 0, 17.388021, 0.426042, 0, 17.298958, 0.463631, 0, 17.390251, 0.447282, 0, 17.309896, 0.498535, 0, 17.298958, 0.463631, 0, 17.390251, 0.447282, 0, 17.298958, 0.463631, 0, 17.309896, 0.498535, 0, 17.390251, 0.447282, 0, 17.309896, 0.498535, 0, 17.390251, 0.447282, 0, 17.396940, 0.465690, 0, 17.390251, 0.447282, 0, 17.309896, 0.498535, 0, 17.396940, 0.465690, 0, 17.328125, 0.528670, 0, 17.309896, 0.498535, 0, 17.396940, 0.465690, 0, 17.309896, 0.498535, 0, 17.328125, 0.528670, 0, 17.396940, 0.465690, 0, 17.328125, 0.528670, 0, 17.396940, 0.465690, 0, 17.408089, 0.481266, 0, 17.396940, 0.465690, 0, 17.328125, 0.528670, 0, 17.408089, 0.481266, 0, 17.353645, 0.554036, 0, 17.328125, 0.528670, 0, 17.408089, 0.481266, 0, 17.328125, 0.528670, 0, 17.353645, 0.554036, 0, 17.408089, 0.481266, 0, 17.353645, 0.554036, 0, 17.408089, 0.481266, 0, 17.423698, 0.494010, 0, 17.408089, 0.481266, 0, 17.353645, 0.554036, 0, 17.423698, 0.494010, 0, 17.385872, 0.574145, 0, 17.353645, 0.554036, 0, 17.423698, 0.494010, 0, 17.353645, 0.554036, 0, 17.385872, 0.574145, 0, 17.423698, 0.494010, 0, 17.385872, 0.574145, 0, 17.423698, 0.494010, 0, 17.443768, 0.503923, 0, 17.423698, 0.494010, 0, 17.385872, 0.574145, 0, 17.443768, 0.503923, 0, 17.424219, 0.588509, 0, 17.385872, 0.574145, 0, 17.443768, 0.503923, 0, 17.385872, 0.574145, 0, 17.424219, 0.588509, 0, 17.443768, 0.503923, 0, 17.424219, 0.588509, 0, 17.443768, 0.503923, 0, 17.468294, 0.511003, 0, 17.443768, 0.503923, 0, 17.424219, 0.588509, 0, 17.468294, 0.511003, 0, 17.424219, 0.588509, 0, 17.468294, 0.511003, 0, 17.468685, 0.597127, 0, 17.468294, 0.511003, 0, 17.424219, 0.588509, 0, 17.468685, 0.597127, 0, 17.468685, 0.597127, 0, 17.468294, 0.511003, 0, 17.497282, 0.515251, 0, 17.468294, 0.511003, 0, 17.468685, 0.597127, 0, 17.497282, 0.515251, 0, 17.519270, 0.600000, 0, 17.468685, 0.597127, 0, 17.497282, 0.515251, 0, 17.468685, 0.597127, 0, 17.519270, 0.600000, 0, 17.497282, 0.515251, 0, 17.519270, 0.600000, 0, 17.497282, 0.515251, 0, 17.530729, 0.516667, 0, 17.497282, 0.515251, 0, 17.519270, 0.600000, 0, 17.530729, 0.516667, 0, 17.530729, 0.516667, 0, 17.545443, 0.599219, 0, 17.519270, 0.600000, 0, 17.545443, 0.599219, 0, 17.530729, 0.516667, 0, 17.519270, 0.600000, 0, 17.545443, 0.599219, 0, 17.530729, 0.516667, 0, 17.553972, 0.515885, 0, 17.530729, 0.516667, 0, 17.545443, 0.599219, 0, 17.553972, 0.515885, 0, 17.570833, 0.596875, 0, 17.545443, 0.599219, 0, 17.553972, 0.515885, 0, 17.545443, 0.599219, 0, 17.570833, 0.596875, 0, 17.553972, 0.515885, 0, 17.570833, 0.596875, 0, 17.553972, 0.515885, 0, 17.576824, 0.513542, 0, 17.553972, 0.515885, 0, 17.570833, 0.596875, 0, 17.576824, 0.513542, 0, 17.570833, 0.596875, 0, 17.576824, 0.513542, 0, 17.595442, 0.592969, 0, 17.576824, 0.513542, 0, 17.570833, 0.596875, 0, 17.595442, 0.592969, 0, 17.595442, 0.592969, 0, 17.576824, 0.513542, 0, 17.599285, 0.509635, 0, 17.576824, 0.513542, 0, 17.595442, 0.592969, 0, 17.599285, 0.509635, 0, 17.595442, 0.592969, 0, 17.599285, 0.509635, 0, 17.619270, 0.587500, 0, 17.599285, 0.509635, 0, 17.595442, 0.592969, 0, 17.619270, 0.587500, 0, 17.642254, 0.580469, 0, 17.619270, 0.587500, 0, 17.599285, 0.509635, 0, 17.619270, 0.587500, 0, 17.642254, 0.580469, 0, 17.599285, 0.509635, 0, 17.642254, 0.580469, 0, 17.599285, 0.509635, 0, 17.621355, 0.504167, 0, 17.599285, 0.509635, 0, 17.642254, 0.580469, 0, 17.621355, 0.504167, 0, 17.642254, 0.580469, 0, 17.621355, 0.504167, 0, 17.664324, 0.571875, 0, 17.621355, 0.504167, 0, 17.642254, 0.580469, 0, 17.664324, 0.571875, 0, 17.621355, 0.504167, 0, 17.643034, 0.497135, 0, 17.664324, 0.571875, 0, 17.643034, 0.497135, 0, 17.621355, 0.504167, 0, 17.664324, 0.571875, 0, 17.664324, 0.571875, 0, 17.643034, 0.497135, 0, 17.685482, 0.561719, 0, 17.643034, 0.497135, 0, 17.664324, 0.571875, 0, 17.685482, 0.561719, 0, 17.643034, 0.497135, 0, 17.664324, 0.488542, 0, 17.685482, 0.561719, 0, 17.664324, 0.488542, 0, 17.643034, 0.497135, 0, 17.685482, 0.561719, 0, 17.685482, 0.561719, 0, 17.664324, 0.488542, 0, 17.705730, 0.550000, 0, 17.664324, 0.488542, 0, 17.685482, 0.561719, 0, 17.705730, 0.550000, 0, 17.664324, 0.488542, 0, 17.685223, 0.478385, 0, 17.705730, 0.550000, 0, 17.685223, 0.478385, 0, 17.664324, 0.488542, 0, 17.705730, 0.550000, 0, 17.685223, 0.478385, 0, 17.705730, 0.466667, 0, 17.705730, 0.550000, 0, 17.705730, 0.466667, 0, 17.685223, 0.478385, 0, 17.705730, 0.550000, 0, 18.255730, 0.550000, 0, 18.255730, 0.466667, 0, 18.255730, 0.550000, 0, 18.255730, 0.550000, 0, 18.255730, 0.466667, 0, 18.255730, 0.466667, 0, 18.255730, 0.466667, 0, 18.235222, 0.478385, 0, 18.255730, 0.466667, 0, 18.255730, 0.466667, 0, 18.235222, 0.478385, 0, 18.235222, 0.478385, 0, 18.235222, 0.478385, 0, 18.214323, 0.488542, 0, 18.235222, 0.478385, 0, 18.235222, 0.478385, 0, 18.214323, 0.488542, 0, 18.214323, 0.488542, 0, 18.214323, 0.488542, 0, 18.193035, 0.497135, 0, 18.214323, 0.488542, 0, 18.214323, 0.488542, 0, 18.193035, 0.497135, 0, 18.193035, 0.497135, 0, 18.193035, 0.497135, 0, 18.171354, 0.504167, 0, 18.193035, 0.497135, 0, 18.193035, 0.497135, 0, 18.171354, 0.504167, 0, 18.171354, 0.504167, 0, 18.171354, 0.504167, 0, 18.149284, 0.509635, 0, 18.171354, 0.504167, 0, 18.171354, 0.504167, 0, 18.149284, 0.509635, 0, 18.149284, 0.509635, 0, 18.149284, 0.509635, 0, 18.126823, 0.513542, 0, 18.149284, 0.509635, 0, 18.149284, 0.509635, 0, 18.126823, 0.513542, 0, 18.126823, 0.513542, 0, 18.126823, 0.513542, 0, 18.103971, 0.515885, 0, 18.126823, 0.513542, 0, 18.126823, 0.513542, 0, 18.103971, 0.515885, 0, 18.103971, 0.515885, 0, 18.103971, 0.515885, 0, 18.080730, 0.516667, 0, 18.103971, 0.515885, 0, 18.103971, 0.515885, 0, 18.080730, 0.516667, 0, 18.080730, 0.516667, 0, 18.080730, 0.516667, 0, 18.047283, 0.515251, 0, 18.080730, 0.516667, 0, 18.080730, 0.516667, 0, 18.047283, 0.515251, 0, 18.047283, 0.515251, 0, 18.047283, 0.515251, 0, 18.018293, 0.511003, 0, 18.047283, 0.515251, 0, 18.047283, 0.515251, 0, 18.018293, 0.511003, 0, 18.018293, 0.511003, 0, 18.018293, 0.511003, 0, 17.993767, 0.503923, 0, 18.018293, 0.511003, 0, 18.018293, 0.511003, 0, 17.993767, 0.503923, 0, 17.993767, 0.503923, 0, 17.993767, 0.503923, 0, 17.973700, 0.494010, 0, 17.993767, 0.503923, 0, 17.993767, 0.503923, 0, 17.973700, 0.494010, 0, 17.973700, 0.494010, 0, 17.973700, 0.494010, 0, 17.958090, 0.481266, 0, 17.973700, 0.494010, 0, 17.973700, 0.494010, 0, 17.958090, 0.481266, 0, 17.958090, 0.481266, 0, 17.958090, 0.481266, 0, 17.946941, 0.465690, 0, 17.958090, 0.481266, 0, 17.958090, 0.481266, 0, 17.946941, 0.465690, 0, 17.946941, 0.465690, 0, 17.946941, 0.465690, 0, 17.940252, 0.447282, 0, 17.946941, 0.465690, 0, 17.946941, 0.465690, 0, 17.940252, 0.447282, 0, 17.940252, 0.447282, 0, 17.940252, 0.447282, 0, 17.938021, 0.426042, 0, 17.940252, 0.447282, 0, 17.940252, 0.447282, 0, 17.938021, 0.426042, 0, 17.938021, 0.426042, 0, 17.938021, 0.426042, 0, 17.939617, 0.409766, 0, 17.938021, 0.426042, 0, 17.938021, 0.426042, 0, 17.939617, 0.409766, 0, 17.939617, 0.409766, 0, 17.939617, 0.409766, 0, 17.944403, 0.395313, 0, 17.939617, 0.409766, 0, 17.939617, 0.409766, 0, 17.944403, 0.395313, 0, 17.944403, 0.395313, 0, 17.944403, 0.395313, 0, 17.952375, 0.382682, 0, 17.944403, 0.395313, 0, 17.944403, 0.395313, 0, 17.952375, 0.382682, 0, 17.952375, 0.382682, 0, 17.952375, 0.382682, 0, 17.963543, 0.371875, 0, 17.952375, 0.382682, 0, 17.952375, 0.382682, 0, 17.963543, 0.371875, 0, 17.963543, 0.371875, 0, 17.963543, 0.371875, 0, 17.979525, 0.362109, 0, 17.963543, 0.371875, 0, 17.963543, 0.371875, 0, 17.979525, 0.362109, 0, 17.979525, 0.362109, 0, 17.979525, 0.362109, 0, 18.001953, 0.352604, 0, 17.979525, 0.362109, 0, 17.979525, 0.362109, 0, 18.001953, 0.352604, 0, 18.001953, 0.352604, 0, 18.001953, 0.352604, 0, 18.030828, 0.343359, 0, 18.001953, 0.352604, 0, 18.001953, 0.352604, 0, 18.030828, 0.343359, 0, 18.030828, 0.343359, 0, 18.030828, 0.343359, 0, 18.066145, 0.334375, 0, 18.030828, 0.343359, 0, 18.030828, 0.343359, 0, 18.066145, 0.334375, 0, 18.066145, 0.334375, 0, 18.066145, 0.334375, 0, 18.098959, 0.327083, 0, 18.066145, 0.334375, 0, 18.066145, 0.334375, 0, 18.098959, 0.327083, 0, 18.098959, 0.327083, 0, 18.098959, 0.327083, 0, 18.146307, 0.315177, 0, 18.098959, 0.327083, 0, 18.098959, 0.327083, 0, 18.146307, 0.315177, 0, 18.146307, 0.315177, 0, 18.146307, 0.315177, 0, 18.186262, 0.301074, 0, 18.146307, 0.315177, 0, 18.146307, 0.315177, 0, 18.186262, 0.301074, 0, 18.186262, 0.301074, 0, 18.186262, 0.301074, 0, 18.218832, 0.284774, 0, 18.186262, 0.301074, 0, 18.186262, 0.301074, 0, 18.218832, 0.284774, 0, 18.218832, 0.284774, 0, 18.218832, 0.284774, 0, 18.244011, 0.266276, 0, 18.218832, 0.284774, 0, 18.218832, 0.284774, 0, 18.244011, 0.266276, 0, 18.244011, 0.266276, 0, 18.244011, 0.266276, 0, 18.262810, 0.244735, 0, 18.244011, 0.266276, 0, 18.244011, 0.266276, 0, 18.262810, 0.244735, 0, 18.262810, 0.244735, 0, 18.262810, 0.244735, 0, 18.276237, 0.219303, 0, 18.262810, 0.244735, 0, 18.262810, 0.244735, 0, 18.276237, 0.219303, 0, 18.276237, 0.219303, 0, 18.276237, 0.219303, 0, 18.284294, 0.189982, 0, 18.276237, 0.219303, 0, 18.276237, 0.219303, 0, 18.284294, 0.189982, 0, 18.284294, 0.189982, 0, 18.284294, 0.189982, 0, 18.286980, 0.156771, 0, 18.284294, 0.189982, 0, 18.284294, 0.189982, 0, 18.286980, 0.156771, 0, 18.286980, 0.156771, 0, 18.286980, 0.156771, 0, 18.282959, 0.118872, 0, 18.286980, 0.156771, 0, 18.286980, 0.156771, 0, 18.282959, 0.118872, 0, 18.282959, 0.118872, 0, 18.282959, 0.118872, 0, 18.270899, 0.085124, 0, 18.282959, 0.118872, 0, 18.282959, 0.118872, 0, 18.270899, 0.085124, 0, 18.270899, 0.085124, 0, 18.270899, 0.085124, 0, 18.250799, 0.055526, 0, 18.270899, 0.085124, 0, 18.270899, 0.085124, 0, 18.250799, 0.055526, 0, 18.250799, 0.055526, 0, 18.250799, 0.055526, 0, 18.222656, 0.030078, 0, 18.250799, 0.055526, 0, 18.250799, 0.055526, 0, 18.222656, 0.030078, 0, 18.222656, 0.030078, 0, 18.222656, 0.030078, 0, 18.187483, 0.009627, 0, 18.222656, 0.030078, 0, 18.222656, 0.030078, 0, 18.187483, 0.009627, 0, 18.187483, 0.009627, 0, 18.187483, 0.009627, 0, 18.146290, -0.004980, 0, 18.187483, 0.009627, 0, 18.187483, 0.009627, 0, 18.146290, -0.004980, 0, 18.146290, -0.004980, 0, 18.146290, -0.004980, 0, 18.099073, -0.013745, 0, 18.146290, -0.004980, 0, 18.146290, -0.004980, 0, 18.099073, -0.013745, 0, 18.099073, -0.013745, 0, 18.099073, -0.013745, 0, 18.045834, -0.016667, 0, 18.099073, -0.013745, 0, 18.099073, -0.013745, 0, 18.045834, -0.016667, 0, 18.045834, -0.016667, 0, 18.045834, -0.016667, 0, 18.022152, -0.016146, 0, 18.045834, -0.016667, 0, 18.045834, -0.016667, 0, 18.022152, -0.016146, 0, 18.022152, -0.016146, 0, 18.022152, -0.016146, 0, 17.997982, -0.014583, 0, 18.022152, -0.016146, 0, 18.022152, -0.016146, 0, 17.997982, -0.014583, 0, 17.997982, -0.014583, 0, 17.997982, -0.014583, 0, 17.973324, -0.011979, 0, 17.997982, -0.014583, 0, 17.997982, -0.014583, 0, 17.973324, -0.011979, 0, 17.973324, -0.011979, 0, 17.973324, -0.011979, 0, 17.948177, -0.008333, 0, 17.973324, -0.011979, 0, 17.973324, -0.011979, 0, 17.948177, -0.008333, 0, 17.948177, -0.008333, 0, 17.948177, -0.008333, 0, 17.922445, -0.003646, 0, 17.948177, -0.008333, 0, 17.948177, -0.008333, 0, 17.922445, -0.003646, 0, 17.922445, -0.003646, 0, 17.922445, -0.003646, 0, 17.896029, 0.002083, 0, 17.922445, -0.003646, 0, 17.922445, -0.003646, 0, 17.896029, 0.002083, 0, 17.896029, 0.002083, 0, 17.896029, 0.002083, 0, 17.868929, 0.008854, 0, 17.896029, 0.002083, 0, 17.896029, 0.002083, 0, 17.868929, 0.008854, 0, 17.868929, 0.008854, 0, 17.868929, 0.008854, 0, 17.841146, 0.016667, 0, 17.868929, 0.008854, 0, 17.868929, 0.008854, 0, 17.841146, 0.016667, 0, 17.841146, 0.016667, 0, 17.841146, 0.016667, 0, 17.841146, 0.116667, 0, 17.841146, 0.016667, 0, 17.841146, 0.016667, 0, 17.841146, 0.116667, 0, 17.841146, 0.116667, 0, 17.841146, 0.116667, 0, 17.867611, 0.104948, 0, 17.841146, 0.116667, 0, 17.841146, 0.116667, 0, 17.867611, 0.104948, 0, 17.867611, 0.104948, 0, 17.867611, 0.104948, 0, 17.893881, 0.094792, 0, 17.867611, 0.104948, 0, 17.867611, 0.104948, 0, 17.893881, 0.094792, 0, 17.893881, 0.094792, 0, 17.893881, 0.094792, 0, 17.919954, 0.086198, 0, 17.893881, 0.094792, 0, 17.893881, 0.094792, 0, 17.919954, 0.086198, 0, 17.919954, 0.086198, 0, 17.919954, 0.086198, 0, 17.945833, 0.079167, 0, 17.919954, 0.086198, 0, 17.919954, 0.086198, 0, 17.945833, 0.079167, 0, 17.945833, 0.079167, 0, 17.945833, 0.079167, 0, 17.971550, 0.073698, 0, 17.945833, 0.079167, 0, 17.945833, 0.079167, 0, 17.971550, 0.073698, 0, 17.971550, 0.073698, 0, 17.971550, 0.073698, 0, 17.997137, 0.069792, 0, 17.971550, 0.073698, 0, 17.971550, 0.073698, 0, 17.997137, 0.069792, 0, 17.997137, 0.069792, 0, 17.997137, 0.069792, 0, 18.022591, 0.067448, 0, 17.997137, 0.069792, 0, 17.997137, 0.069792, 0, 18.022591, 0.067448, 0, 18.022591, 0.067448, 0, 18.022591, 0.067448, 0, 18.047918, 0.066667, 0, 18.022591, 0.067448, 0, 18.022591, 0.067448, 0, 18.047918, 0.066667, 0, 18.047918, 0.066667, 0, 18.047918, 0.066667, 0, 18.079817, 0.068058, 0, 18.047918, 0.066667, 0, 18.047918, 0.066667, 0, 18.079817, 0.068058, 0, 18.079817, 0.068058, 0, 18.079817, 0.068058, 0, 18.107813, 0.072233, 0, 18.079817, 0.068058, 0, 18.079817, 0.068058, 0, 18.107813, 0.072233, 0, 18.107813, 0.072233, 0, 18.107813, 0.072233, 0, 18.131903, 0.079191, 0, 18.107813, 0.072233, 0, 18.107813, 0.072233, 0, 18.131903, 0.079191, 0, 18.131903, 0.079191, 0, 18.131903, 0.079191, 0, 18.152082, 0.088932, 0, 18.131903, 0.079191, 0, 18.131903, 0.079191, 0, 18.152082, 0.088932, 0, 18.152082, 0.088932, 0, 18.152082, 0.088932, 0, 18.168036, 0.101213, 0, 18.152082, 0.088932, 0, 18.152082, 0.088932, 0, 18.168036, 0.101213, 0, 18.168036, 0.101213, 0, 18.168036, 0.101213, 0, 18.179426, 0.115788, 0, 18.168036, 0.101213, 0, 18.168036, 0.101213, 0, 18.179426, 0.115788, 0, 18.179426, 0.115788, 0, 18.179426, 0.115788, 0, 18.186262, 0.132658, 0, 18.179426, 0.115788, 0, 18.179426, 0.115788, 0, 18.186262, 0.132658, 0, 18.186262, 0.132658, 0, 18.186262, 0.132658, 0, 18.188543, 0.151823, 0, 18.186262, 0.132658, 0, 18.186262, 0.132658, 0, 18.188543, 0.151823, 0, 18.188543, 0.151823, 0, 18.188543, 0.151823, 0, 18.186897, 0.169482, 0, 18.188543, 0.151823, 0, 18.188543, 0.151823, 0, 18.186897, 0.169482, 0, 18.186897, 0.169482, 0, 18.186897, 0.169482, 0, 18.181967, 0.184961, 0, 18.186897, 0.169482, 0, 18.186897, 0.169482, 0, 18.181967, 0.184961, 0, 18.181967, 0.184961, 0, 18.181967, 0.184961, 0, 18.173748, 0.198258, 0, 18.181967, 0.184961, 0, 18.181967, 0.184961, 0, 18.173748, 0.198258, 0, 18.173748, 0.198258, 0, 18.173748, 0.198258, 0, 18.162239, 0.209375, 0, 18.173748, 0.198258, 0, 18.173748, 0.198258, 0, 18.162239, 0.209375, 0, 18.162239, 0.209375, 0, 18.162239, 0.209375, 0, 18.145166, 0.219303, 0, 18.162239, 0.209375, 0, 18.162239, 0.209375, 0, 18.145166, 0.219303, 0, 18.145166, 0.219303, 0, 18.145166, 0.219303, 0, 18.120247, 0.229036, 0, 18.145166, 0.219303, 0, 18.145166, 0.219303, 0, 18.120247, 0.229036, 0, 18.120247, 0.229036, 0, 18.120247, 0.229036, 0, 18.087484, 0.238574, 0, 18.120247, 0.229036, 0, 18.120247, 0.229036, 0, 18.087484, 0.238574, 0, 18.087484, 0.238574, 0, 18.087484, 0.238574, 0, 18.046875, 0.247917, 0, 18.087484, 0.238574, 0, 18.087484, 0.238574, 0, 18.046875, 0.247917, 0, 18.046875, 0.247917, 0, 18.046875, 0.247917, 0, 18.013542, 0.255729, 0, 18.046875, 0.247917, 0, 18.046875, 0.247917, 0, 18.013542, 0.255729, 0, 18.013542, 0.255729, 0, 18.013542, 0.255729, 0, 17.972103, 0.266642, 0, 18.013542, 0.255729, 0, 18.013542, 0.255729, 0, 17.972103, 0.266642, 0, 17.972103, 0.266642, 0, 17.972103, 0.266642, 0, 17.936850, 0.280111, 0, 17.972103, 0.266642, 0, 17.972103, 0.266642, 0, 17.936850, 0.280111, 0, 17.936850, 0.280111, 0, 17.936850, 0.280111, 0, 17.907782, 0.296134, 0, 17.936850, 0.280111, 0, 17.936850, 0.280111, 0, 17.907782, 0.296134, 0, 17.907782, 0.296134, 0, 17.907782, 0.296134, 0, 17.884895, 0.314714, 0, 17.907782, 0.296134, 0, 17.907782, 0.296134, 0, 17.884895, 0.314714, 0, 17.884895, 0.314714, 0, 17.884895, 0.314714, 0, 17.867579, 0.336434, 0, 17.884895, 0.314714, 0, 17.884895, 0.314714, 0, 17.867579, 0.336434, 0, 17.867579, 0.336434, 0, 17.867579, 0.336434, 0, 17.855207, 0.361882, 0, 17.867579, 0.336434, 0, 17.867579, 0.336434, 0, 17.855207, 0.361882, 0, 17.855207, 0.361882, 0, 17.855207, 0.361882, 0, 17.847786, 0.391056, 0, 17.855207, 0.361882, 0, 17.855207, 0.361882, 0, 17.847786, 0.391056, 0, 17.847786, 0.391056, 0, 17.847786, 0.391056, 0, 17.845312, 0.423958, 0, 17.847786, 0.391056, 0, 17.847786, 0.391056, 0, 17.845312, 0.423958, 0, 17.845312, 0.423958, 0, 17.845312, 0.423958, 0, 17.848959, 0.463631, 0, 17.845312, 0.423958, 0, 17.845312, 0.423958, 0, 17.848959, 0.463631, 0, 17.848959, 0.463631, 0, 17.848959, 0.463631, 0, 17.859896, 0.498535, 0, 17.848959, 0.463631, 0, 17.848959, 0.463631, 0, 17.859896, 0.498535, 0, 17.859896, 0.498535, 0, 17.859896, 0.498535, 0, 17.878126, 0.528670, 0, 17.859896, 0.498535, 0, 17.859896, 0.498535, 0, 17.878126, 0.528670, 0, 17.878126, 0.528670, 0, 17.878126, 0.528670, 0, 17.903646, 0.554036, 0, 17.878126, 0.528670, 0, 17.878126, 0.528670, 0, 17.903646, 0.554036, 0, 17.903646, 0.554036, 0, 17.903646, 0.554036, 0, 17.935873, 0.574145, 0, 17.903646, 0.554036, 0, 17.903646, 0.554036, 0, 17.935873, 0.574145, 0, 17.935873, 0.574145, 0, 17.935873, 0.574145, 0, 17.974218, 0.588509, 0, 17.935873, 0.574145, 0, 17.935873, 0.574145, 0, 17.974218, 0.588509, 0, 17.974218, 0.588509, 0, 17.974218, 0.588509, 0, 18.018684, 0.597127, 0, 17.974218, 0.588509, 0, 17.974218, 0.588509, 0, 18.018684, 0.597127, 0, 18.018684, 0.597127, 0, 18.018684, 0.597127, 0, 18.069271, 0.600000, 0, 18.018684, 0.597127, 0, 18.018684, 0.597127, 0, 18.069271, 0.600000, 0, 18.069271, 0.600000, 0, 18.069271, 0.600000, 0, 18.095442, 0.599219, 0, 18.069271, 0.600000, 0, 18.069271, 0.600000, 0, 18.095442, 0.599219, 0, 18.095442, 0.599219, 0, 18.095442, 0.599219, 0, 18.120832, 0.596875, 0, 18.095442, 0.599219, 0, 18.095442, 0.599219, 0, 18.120832, 0.596875, 0, 18.120832, 0.596875, 0, 18.120832, 0.596875, 0, 18.145443, 0.592969, 0, 18.120832, 0.596875, 0, 18.120832, 0.596875, 0, 18.145443, 0.592969, 0, 18.145443, 0.592969, 0, 18.145443, 0.592969, 0, 18.169271, 0.587500, 0, 18.145443, 0.592969, 0, 18.145443, 0.592969, 0, 18.169271, 0.587500, 0, 18.169271, 0.587500, 0, 18.169271, 0.587500, 0, 18.192253, 0.580469, 0, 18.169271, 0.587500, 0, 18.169271, 0.587500, 0, 18.192253, 0.580469, 0, 18.192253, 0.580469, 0, 18.192253, 0.580469, 0, 18.214323, 0.571875, 0, 18.192253, 0.580469, 0, 18.192253, 0.580469, 0, 18.214323, 0.571875, 0, 18.214323, 0.571875, 0, 18.214323, 0.571875, 0, 18.235481, 0.561719, 0, 18.214323, 0.571875, 0, 18.214323, 0.571875, 0, 18.235481, 0.561719, 0, 18.235481, 0.561719, 0, 17.841146, 0.116667, 0, 17.841146, 0.016667, 0, 17.867611, 0.104948, 0, 17.841146, 0.016667, 0, 17.841146, 0.116667, 0, 17.867611, 0.104948, 0, 17.893881, 0.094792, 0, 17.867611, 0.104948, 0, 17.841146, 0.016667, 0, 17.867611, 0.104948, 0, 17.893881, 0.094792, 0, 17.841146, 0.016667, 0, 17.893881, 0.094792, 0, 17.841146, 0.016667, 0, 17.868929, 0.008854, 0, 17.841146, 0.016667, 0, 17.893881, 0.094792, 0, 17.868929, 0.008854, 0, 17.919954, 0.086198, 0, 17.893881, 0.094792, 0, 17.868929, 0.008854, 0, 17.893881, 0.094792, 0, 17.919954, 0.086198, 0, 17.868929, 0.008854, 0, 17.919954, 0.086198, 0, 17.868929, 0.008854, 0, 17.896029, 0.002083, 0, 17.868929, 0.008854, 0, 17.919954, 0.086198, 0, 17.896029, 0.002083, 0, 17.919954, 0.086198, 0, 17.896029, 0.002083, 0, 17.922445, -0.003646, 0, 17.896029, 0.002083, 0, 17.919954, 0.086198, 0, 17.922445, -0.003646, 0, 17.919954, 0.086198, 0, 17.922445, -0.003646, 0, 17.945833, 0.079167, 0, 17.922445, -0.003646, 0, 17.919954, 0.086198, 0, 17.945833, 0.079167, 0, 17.945833, 0.079167, 0, 17.922445, -0.003646, 0, 17.948177, -0.008333, 0, 17.922445, -0.003646, 0, 17.945833, 0.079167, 0, 17.948177, -0.008333, 0, 17.945833, 0.079167, 0, 17.948177, -0.008333, 0, 17.971550, 0.073698, 0, 17.948177, -0.008333, 0, 17.945833, 0.079167, 0, 17.971550, 0.073698, 0, 17.971550, 0.073698, 0, 17.948177, -0.008333, 0, 17.973324, -0.011979, 0, 17.948177, -0.008333, 0, 17.971550, 0.073698, 0, 17.973324, -0.011979, 0, 17.971550, 0.073698, 0, 17.973324, -0.011979, 0, 17.997137, 0.069792, 0, 17.973324, -0.011979, 0, 17.971550, 0.073698, 0, 17.997137, 0.069792, 0, 17.997137, 0.069792, 0, 17.973324, -0.011979, 0, 17.997982, -0.014583, 0, 17.973324, -0.011979, 0, 17.997137, 0.069792, 0, 17.997982, -0.014583, 0, 17.997137, 0.069792, 0, 17.997982, -0.014583, 0, 18.022591, 0.067448, 0, 17.997982, -0.014583, 0, 17.997137, 0.069792, 0, 18.022591, 0.067448, 0, 17.997982, -0.014583, 0, 18.022152, -0.016146, 0, 18.022591, 0.067448, 0, 18.022152, -0.016146, 0, 17.997982, -0.014583, 0, 18.022591, 0.067448, 0, 18.022591, 0.067448, 0, 18.022152, -0.016146, 0, 18.047918, 0.066667, 0, 18.022152, -0.016146, 0, 18.022591, 0.067448, 0, 18.047918, 0.066667, 0, 18.022152, -0.016146, 0, 18.045834, -0.016667, 0, 18.047918, 0.066667, 0, 18.045834, -0.016667, 0, 18.022152, -0.016146, 0, 18.047918, 0.066667, 0, 18.079817, 0.068058, 0, 18.047918, 0.066667, 0, 18.045834, -0.016667, 0, 18.047918, 0.066667, 0, 18.079817, 0.068058, 0, 18.045834, -0.016667, 0, 18.099073, -0.013745, 0, 18.079817, 0.068058, 0, 18.045834, -0.016667, 0, 18.079817, 0.068058, 0, 18.099073, -0.013745, 0, 18.045834, -0.016667, 0, 18.107813, 0.072233, 0, 18.079817, 0.068058, 0, 18.099073, -0.013745, 0, 18.079817, 0.068058, 0, 18.107813, 0.072233, 0, 18.099073, -0.013745, 0, 18.146290, -0.004980, 0, 18.107813, 0.072233, 0, 18.099073, -0.013745, 0, 18.107813, 0.072233, 0, 18.146290, -0.004980, 0, 18.099073, -0.013745, 0, 18.131903, 0.079191, 0, 18.107813, 0.072233, 0, 18.146290, -0.004980, 0, 18.107813, 0.072233, 0, 18.131903, 0.079191, 0, 18.146290, -0.004980, 0, 18.131903, 0.079191, 0, 18.146290, -0.004980, 0, 18.187483, 0.009627, 0, 18.146290, -0.004980, 0, 18.131903, 0.079191, 0, 18.187483, 0.009627, 0, 18.152082, 0.088932, 0, 18.131903, 0.079191, 0, 18.187483, 0.009627, 0, 18.131903, 0.079191, 0, 18.152082, 0.088932, 0, 18.187483, 0.009627, 0, 18.152082, 0.088932, 0, 18.187483, 0.009627, 0, 18.222656, 0.030078, 0, 18.187483, 0.009627, 0, 18.152082, 0.088932, 0, 18.222656, 0.030078, 0, 18.168036, 0.101213, 0, 18.152082, 0.088932, 0, 18.222656, 0.030078, 0, 18.152082, 0.088932, 0, 18.168036, 0.101213, 0, 18.222656, 0.030078, 0, 18.168036, 0.101213, 0, 18.222656, 0.030078, 0, 18.250799, 0.055526, 0, 18.222656, 0.030078, 0, 18.168036, 0.101213, 0, 18.250799, 0.055526, 0, 18.179426, 0.115788, 0, 18.168036, 0.101213, 0, 18.250799, 0.055526, 0, 18.168036, 0.101213, 0, 18.179426, 0.115788, 0, 18.250799, 0.055526, 0, 18.179426, 0.115788, 0, 18.250799, 0.055526, 0, 18.270899, 0.085124, 0, 18.250799, 0.055526, 0, 18.179426, 0.115788, 0, 18.270899, 0.085124, 0, 18.186262, 0.132658, 0, 18.179426, 0.115788, 0, 18.270899, 0.085124, 0, 18.179426, 0.115788, 0, 18.186262, 0.132658, 0, 18.270899, 0.085124, 0, 18.186262, 0.132658, 0, 18.270899, 0.085124, 0, 18.282959, 0.118872, 0, 18.270899, 0.085124, 0, 18.186262, 0.132658, 0, 18.282959, 0.118872, 0, 18.188543, 0.151823, 0, 18.186262, 0.132658, 0, 18.282959, 0.118872, 0, 18.186262, 0.132658, 0, 18.188543, 0.151823, 0, 18.282959, 0.118872, 0, 18.188543, 0.151823, 0, 18.282959, 0.118872, 0, 18.286980, 0.156771, 0, 18.282959, 0.118872, 0, 18.188543, 0.151823, 0, 18.286980, 0.156771, 0, 18.286980, 0.156771, 0, 18.186897, 0.169482, 0, 18.188543, 0.151823, 0, 18.186897, 0.169482, 0, 18.286980, 0.156771, 0, 18.188543, 0.151823, 0, 18.284294, 0.189982, 0, 18.186897, 0.169482, 0, 18.286980, 0.156771, 0, 18.186897, 0.169482, 0, 18.284294, 0.189982, 0, 18.286980, 0.156771, 0, 18.276237, 0.219303, 0, 18.186897, 0.169482, 0, 18.284294, 0.189982, 0, 18.186897, 0.169482, 0, 18.276237, 0.219303, 0, 18.284294, 0.189982, 0, 18.181967, 0.184961, 0, 18.186897, 0.169482, 0, 18.276237, 0.219303, 0, 18.186897, 0.169482, 0, 18.181967, 0.184961, 0, 18.276237, 0.219303, 0, 18.262810, 0.244735, 0, 18.181967, 0.184961, 0, 18.276237, 0.219303, 0, 18.181967, 0.184961, 0, 18.262810, 0.244735, 0, 18.276237, 0.219303, 0, 18.173748, 0.198258, 0, 18.181967, 0.184961, 0, 18.262810, 0.244735, 0, 18.181967, 0.184961, 0, 18.173748, 0.198258, 0, 18.262810, 0.244735, 0, 18.244011, 0.266276, 0, 18.173748, 0.198258, 0, 18.262810, 0.244735, 0, 18.173748, 0.198258, 0, 18.244011, 0.266276, 0, 18.262810, 0.244735, 0, 18.218832, 0.284774, 0, 18.173748, 0.198258, 0, 18.244011, 0.266276, 0, 18.173748, 0.198258, 0, 18.218832, 0.284774, 0, 18.244011, 0.266276, 0, 18.162239, 0.209375, 0, 18.173748, 0.198258, 0, 18.218832, 0.284774, 0, 18.173748, 0.198258, 0, 18.162239, 0.209375, 0, 18.218832, 0.284774, 0, 18.186262, 0.301074, 0, 18.162239, 0.209375, 0, 18.218832, 0.284774, 0, 18.162239, 0.209375, 0, 18.186262, 0.301074, 0, 18.218832, 0.284774, 0, 18.145166, 0.219303, 0, 18.162239, 0.209375, 0, 18.186262, 0.301074, 0, 18.162239, 0.209375, 0, 18.145166, 0.219303, 0, 18.186262, 0.301074, 0, 18.146307, 0.315177, 0, 18.145166, 0.219303, 0, 18.186262, 0.301074, 0, 18.145166, 0.219303, 0, 18.146307, 0.315177, 0, 18.186262, 0.301074, 0, 18.120247, 0.229036, 0, 18.145166, 0.219303, 0, 18.146307, 0.315177, 0, 18.145166, 0.219303, 0, 18.120247, 0.229036, 0, 18.146307, 0.315177, 0, 18.146307, 0.315177, 0, 18.087484, 0.238574, 0, 18.120247, 0.229036, 0, 18.087484, 0.238574, 0, 18.146307, 0.315177, 0, 18.120247, 0.229036, 0, 18.146307, 0.315177, 0, 18.098959, 0.327083, 0, 18.087484, 0.238574, 0, 18.098959, 0.327083, 0, 18.146307, 0.315177, 0, 18.087484, 0.238574, 0, 18.046875, 0.247917, 0, 18.087484, 0.238574, 0, 18.098959, 0.327083, 0, 18.087484, 0.238574, 0, 18.046875, 0.247917, 0, 18.098959, 0.327083, 0, 18.066145, 0.334375, 0, 18.046875, 0.247917, 0, 18.098959, 0.327083, 0, 18.046875, 0.247917, 0, 18.066145, 0.334375, 0, 18.098959, 0.327083, 0, 18.013542, 0.255729, 0, 18.046875, 0.247917, 0, 18.066145, 0.334375, 0, 18.046875, 0.247917, 0, 18.013542, 0.255729, 0, 18.066145, 0.334375, 0, 18.030828, 0.343359, 0, 18.013542, 0.255729, 0, 18.066145, 0.334375, 0, 18.013542, 0.255729, 0, 18.030828, 0.343359, 0, 18.066145, 0.334375, 0, 17.972103, 0.266642, 0, 18.013542, 0.255729, 0, 18.030828, 0.343359, 0, 18.013542, 0.255729, 0, 17.972103, 0.266642, 0, 18.030828, 0.343359, 0, 18.001953, 0.352604, 0, 17.972103, 0.266642, 0, 18.030828, 0.343359, 0, 17.972103, 0.266642, 0, 18.001953, 0.352604, 0, 18.030828, 0.343359, 0, 18.001953, 0.352604, 0, 17.979525, 0.362109, 0, 17.972103, 0.266642, 0, 17.979525, 0.362109, 0, 18.001953, 0.352604, 0, 17.972103, 0.266642, 0, 17.979525, 0.362109, 0, 17.936850, 0.280111, 0, 17.972103, 0.266642, 0, 17.936850, 0.280111, 0, 17.979525, 0.362109, 0, 17.972103, 0.266642, 0, 17.979525, 0.362109, 0, 17.963543, 0.371875, 0, 17.936850, 0.280111, 0, 17.963543, 0.371875, 0, 17.979525, 0.362109, 0, 17.936850, 0.280111, 0, 17.963543, 0.371875, 0, 17.907782, 0.296134, 0, 17.936850, 0.280111, 0, 17.907782, 0.296134, 0, 17.963543, 0.371875, 0, 17.936850, 0.280111, 0, 17.884895, 0.314714, 0, 17.907782, 0.296134, 0, 17.963543, 0.371875, 0, 17.907782, 0.296134, 0, 17.884895, 0.314714, 0, 17.963543, 0.371875, 0, 17.963543, 0.371875, 0, 17.952375, 0.382682, 0, 17.884895, 0.314714, 0, 17.952375, 0.382682, 0, 17.963543, 0.371875, 0, 17.884895, 0.314714, 0, 17.952375, 0.382682, 0, 17.867579, 0.336434, 0, 17.884895, 0.314714, 0, 17.867579, 0.336434, 0, 17.952375, 0.382682, 0, 17.884895, 0.314714, 0, 17.952375, 0.382682, 0, 17.944403, 0.395313, 0, 17.867579, 0.336434, 0, 17.944403, 0.395313, 0, 17.952375, 0.382682, 0, 17.867579, 0.336434, 0, 17.944403, 0.395313, 0, 17.855207, 0.361882, 0, 17.867579, 0.336434, 0, 17.855207, 0.361882, 0, 17.944403, 0.395313, 0, 17.867579, 0.336434, 0, 17.944403, 0.395313, 0, 17.939617, 0.409766, 0, 17.855207, 0.361882, 0, 17.939617, 0.409766, 0, 17.944403, 0.395313, 0, 17.855207, 0.361882, 0, 17.939617, 0.409766, 0, 17.847786, 0.391056, 0, 17.855207, 0.361882, 0, 17.847786, 0.391056, 0, 17.939617, 0.409766, 0, 17.855207, 0.361882, 0, 17.845312, 0.423958, 0, 17.847786, 0.391056, 0, 17.939617, 0.409766, 0, 17.847786, 0.391056, 0, 17.845312, 0.423958, 0, 17.939617, 0.409766, 0, 17.938021, 0.426042, 0, 17.845312, 0.423958, 0, 17.939617, 0.409766, 0, 17.845312, 0.423958, 0, 17.938021, 0.426042, 0, 17.939617, 0.409766, 0, 17.848959, 0.463631, 0, 17.845312, 0.423958, 0, 17.938021, 0.426042, 0, 17.845312, 0.423958, 0, 17.848959, 0.463631, 0, 17.938021, 0.426042, 0, 17.848959, 0.463631, 0, 17.938021, 0.426042, 0, 17.940252, 0.447282, 0, 17.938021, 0.426042, 0, 17.848959, 0.463631, 0, 17.940252, 0.447282, 0, 17.859896, 0.498535, 0, 17.848959, 0.463631, 0, 17.940252, 0.447282, 0, 17.848959, 0.463631, 0, 17.859896, 0.498535, 0, 17.940252, 0.447282, 0, 17.859896, 0.498535, 0, 17.940252, 0.447282, 0, 17.946941, 0.465690, 0, 17.940252, 0.447282, 0, 17.859896, 0.498535, 0, 17.946941, 0.465690, 0, 17.878126, 0.528670, 0, 17.859896, 0.498535, 0, 17.946941, 0.465690, 0, 17.859896, 0.498535, 0, 17.878126, 0.528670, 0, 17.946941, 0.465690, 0, 17.878126, 0.528670, 0, 17.946941, 0.465690, 0, 17.958090, 0.481266, 0, 17.946941, 0.465690, 0, 17.878126, 0.528670, 0, 17.958090, 0.481266, 0, 17.903646, 0.554036, 0, 17.878126, 0.528670, 0, 17.958090, 0.481266, 0, 17.878126, 0.528670, 0, 17.903646, 0.554036, 0, 17.958090, 0.481266, 0, 17.903646, 0.554036, 0, 17.958090, 0.481266, 0, 17.973700, 0.494010, 0, 17.958090, 0.481266, 0, 17.903646, 0.554036, 0, 17.973700, 0.494010, 0, 17.935873, 0.574145, 0, 17.903646, 0.554036, 0, 17.973700, 0.494010, 0, 17.903646, 0.554036, 0, 17.935873, 0.574145, 0, 17.973700, 0.494010, 0, 17.935873, 0.574145, 0, 17.973700, 0.494010, 0, 17.993767, 0.503923, 0, 17.973700, 0.494010, 0, 17.935873, 0.574145, 0, 17.993767, 0.503923, 0, 17.974218, 0.588509, 0, 17.935873, 0.574145, 0, 17.993767, 0.503923, 0, 17.935873, 0.574145, 0, 17.974218, 0.588509, 0, 17.993767, 0.503923, 0, 17.974218, 0.588509, 0, 17.993767, 0.503923, 0, 18.018293, 0.511003, 0, 17.993767, 0.503923, 0, 17.974218, 0.588509, 0, 18.018293, 0.511003, 0, 17.974218, 0.588509, 0, 18.018293, 0.511003, 0, 18.018684, 0.597127, 0, 18.018293, 0.511003, 0, 17.974218, 0.588509, 0, 18.018684, 0.597127, 0, 18.018684, 0.597127, 0, 18.018293, 0.511003, 0, 18.047283, 0.515251, 0, 18.018293, 0.511003, 0, 18.018684, 0.597127, 0, 18.047283, 0.515251, 0, 18.069271, 0.600000, 0, 18.018684, 0.597127, 0, 18.047283, 0.515251, 0, 18.018684, 0.597127, 0, 18.069271, 0.600000, 0, 18.047283, 0.515251, 0, 18.069271, 0.600000, 0, 18.047283, 0.515251, 0, 18.080730, 0.516667, 0, 18.047283, 0.515251, 0, 18.069271, 0.600000, 0, 18.080730, 0.516667, 0, 18.080730, 0.516667, 0, 18.095442, 0.599219, 0, 18.069271, 0.600000, 0, 18.095442, 0.599219, 0, 18.080730, 0.516667, 0, 18.069271, 0.600000, 0, 18.095442, 0.599219, 0, 18.080730, 0.516667, 0, 18.103971, 0.515885, 0, 18.080730, 0.516667, 0, 18.095442, 0.599219, 0, 18.103971, 0.515885, 0, 18.120832, 0.596875, 0, 18.095442, 0.599219, 0, 18.103971, 0.515885, 0, 18.095442, 0.599219, 0, 18.120832, 0.596875, 0, 18.103971, 0.515885, 0, 18.120832, 0.596875, 0, 18.103971, 0.515885, 0, 18.126823, 0.513542, 0, 18.103971, 0.515885, 0, 18.120832, 0.596875, 0, 18.126823, 0.513542, 0, 18.120832, 0.596875, 0, 18.126823, 0.513542, 0, 18.145443, 0.592969, 0, 18.126823, 0.513542, 0, 18.120832, 0.596875, 0, 18.145443, 0.592969, 0, 18.145443, 0.592969, 0, 18.126823, 0.513542, 0, 18.149284, 0.509635, 0, 18.126823, 0.513542, 0, 18.145443, 0.592969, 0, 18.149284, 0.509635, 0, 18.145443, 0.592969, 0, 18.149284, 0.509635, 0, 18.169271, 0.587500, 0, 18.149284, 0.509635, 0, 18.145443, 0.592969, 0, 18.169271, 0.587500, 0, 18.192253, 0.580469, 0, 18.169271, 0.587500, 0, 18.149284, 0.509635, 0, 18.169271, 0.587500, 0, 18.192253, 0.580469, 0, 18.149284, 0.509635, 0, 18.192253, 0.580469, 0, 18.149284, 0.509635, 0, 18.171354, 0.504167, 0, 18.149284, 0.509635, 0, 18.192253, 0.580469, 0, 18.171354, 0.504167, 0, 18.192253, 0.580469, 0, 18.171354, 0.504167, 0, 18.214323, 0.571875, 0, 18.171354, 0.504167, 0, 18.192253, 0.580469, 0, 18.214323, 0.571875, 0, 18.171354, 0.504167, 0, 18.193035, 0.497135, 0, 18.214323, 0.571875, 0, 18.193035, 0.497135, 0, 18.171354, 0.504167, 0, 18.214323, 0.571875, 0, 18.214323, 0.571875, 0, 18.193035, 0.497135, 0, 18.235481, 0.561719, 0, 18.193035, 0.497135, 0, 18.214323, 0.571875, 0, 18.235481, 0.561719, 0, 18.193035, 0.497135, 0, 18.214323, 0.488542, 0, 18.235481, 0.561719, 0, 18.214323, 0.488542, 0, 18.193035, 0.497135, 0, 18.235481, 0.561719, 0, 18.235481, 0.561719, 0, 18.214323, 0.488542, 0, 18.255730, 0.550000, 0, 18.214323, 0.488542, 0, 18.235481, 0.561719, 0, 18.255730, 0.550000, 0, 18.214323, 0.488542, 0, 18.235222, 0.478385, 0, 18.255730, 0.550000, 0, 18.235222, 0.478385, 0, 18.214323, 0.488542, 0, 18.255730, 0.550000, 0, 18.235222, 0.478385, 0, 18.255730, 0.466667, 0, 18.255730, 0.550000, 0, 18.255730, 0.466667, 0, 18.235222, 0.478385, 0, 18.255730, 0.550000, 0, 18.433855, 0.583333, 0, 18.529688, 0.583333, 0, 18.433855, 0.583333, 0, 18.433855, 0.583333, 0, 18.529688, 0.583333, 0, 18.529688, 0.583333, 0, 18.529688, 0.583333, 0, 18.529688, 0, 0, 18.529688, 0.583333, 0, 18.529688, 0.583333, 0, 18.529688, 0, 0, 18.529688, 0, 0, 18.529688, 0, 0, 18.433855, 0, 0, 18.529688, 0, 0, 18.529688, 0, 0, 18.433855, 0, 0, 18.433855, 0, 0, 18.433855, 0.583333, 0, 18.529688, 0, 0, 18.529688, 0.583333, 0, 18.529688, 0, 0, 18.433855, 0.583333, 0, 18.529688, 0.583333, 0, 18.433855, 0.583333, 0, 18.433855, 0, 0, 18.529688, 0, 0, 18.433855, 0, 0, 18.433855, 0.583333, 0, 18.529688, 0, 0, 18.433855, 0.816667, 0, 18.529688, 0.816667, 0, 18.433855, 0.816667, 0, 18.433855, 0.816667, 0, 18.529688, 0.816667, 0, 18.529688, 0.816667, 0, 18.529688, 0.816667, 0, 18.529688, 0.700000, 0, 18.529688, 0.816667, 0, 18.529688, 0.816667, 0, 18.529688, 0.700000, 0, 18.529688, 0.700000, 0, 18.529688, 0.700000, 0, 18.433855, 0.700000, 0, 18.529688, 0.700000, 0, 18.529688, 0.700000, 0, 18.433855, 0.700000, 0, 18.433855, 0.700000, 0, 18.433855, 0.816667, 0, 18.529688, 0.700000, 0, 18.529688, 0.816667, 0, 18.529688, 0.700000, 0, 18.433855, 0.816667, 0, 18.529688, 0.816667, 0, 18.433855, 0.816667, 0, 18.433855, 0.700000, 0, 18.529688, 0.700000, 0, 18.433855, 0.700000, 0, 18.433855, 0.816667, 0, 18.529688, 0.700000, 0, 19.188021, 0.478385, 0, 19.206869, 0.507471, 0, 19.188021, 0.478385, 0, 19.188021, 0.478385, 0, 19.206869, 0.507471, 0, 19.206869, 0.507471, 0, 19.206869, 0.507471, 0, 19.227474, 0.532487, 0, 19.206869, 0.507471, 0, 19.206869, 0.507471, 0, 19.227474, 0.532487, 0, 19.227474, 0.532487, 0, 19.227474, 0.532487, 0, 19.249838, 0.553434, 0, 19.227474, 0.532487, 0, 19.227474, 0.532487, 0, 19.249838, 0.553434, 0, 19.249838, 0.553434, 0, 19.249838, 0.553434, 0, 19.273958, 0.570313, 0, 19.249838, 0.553434, 0, 19.249838, 0.553434, 0, 19.273958, 0.570313, 0, 19.273958, 0.570313, 0, 19.273958, 0.570313, 0, 19.300066, 0.583301, 0, 19.273958, 0.570313, 0, 19.273958, 0.570313, 0, 19.300066, 0.583301, 0, 19.300066, 0.583301, 0, 19.300066, 0.583301, 0, 19.328386, 0.592578, 0, 19.300066, 0.583301, 0, 19.300066, 0.583301, 0, 19.328386, 0.592578, 0, 19.328386, 0.592578, 0, 19.328386, 0.592578, 0, 19.358919, 0.598145, 0, 19.328386, 0.592578, 0, 19.328386, 0.592578, 0, 19.358919, 0.598145, 0, 19.358919, 0.598145, 0, 19.358919, 0.598145, 0, 19.391668, 0.600000, 0, 19.358919, 0.598145, 0, 19.358919, 0.598145, 0, 19.391668, 0.600000, 0, 19.391668, 0.600000, 0, 19.391668, 0.600000, 0, 19.434637, 0.596053, 0, 19.391668, 0.600000, 0, 19.391668, 0.600000, 0, 19.434637, 0.596053, 0, 19.434637, 0.596053, 0, 19.434637, 0.596053, 0, 19.472395, 0.584212, 0, 19.434637, 0.596053, 0, 19.434637, 0.596053, 0, 19.472395, 0.584212, 0, 19.472395, 0.584212, 0, 19.472395, 0.584212, 0, 19.504950, 0.564478, 0, 19.472395, 0.584212, 0, 19.472395, 0.584212, 0, 19.504950, 0.564478, 0, 19.504950, 0.564478, 0, 19.504950, 0.564478, 0, 19.532293, 0.536849, 0, 19.504950, 0.564478, 0, 19.504950, 0.564478, 0, 19.532293, 0.536849, 0, 19.532293, 0.536849, 0, 19.532293, 0.536849, 0, 19.553938, 0.501945, 0, 19.532293, 0.536849, 0, 19.532293, 0.536849, 0, 19.553938, 0.501945, 0, 19.553938, 0.501945, 0, 19.553938, 0.501945, 0, 19.569403, 0.460384, 0, 19.553938, 0.501945, 0, 19.553938, 0.501945, 0, 19.569403, 0.460384, 0, 19.569403, 0.460384, 0, 19.569403, 0.460384, 0, 19.578680, 0.412166, 0, 19.569403, 0.460384, 0, 19.569403, 0.460384, 0, 19.578680, 0.412166, 0, 19.578680, 0.412166, 0, 19.578680, 0.412166, 0, 19.581770, 0.357292, 0, 19.578680, 0.412166, 0, 19.578680, 0.412166, 0, 19.581770, 0.357292, 0, 19.581770, 0.357292, 0, 19.581770, 0.357292, 0, 19.581770, 0, 0, 19.581770, 0.357292, 0, 19.581770, 0.357292, 0, 19.581770, 0, 0, 19.581770, 0, 0, 19.581770, 0, 0, 19.485418, 0, 0, 19.581770, 0, 0, 19.581770, 0, 0, 19.485418, 0, 0, 19.485418, 0, 0, 19.485418, 0, 0, 19.485418, 0.354167, 0, 19.485418, 0, 0, 19.485418, 0, 0, 19.485418, 0.354167, 0, 19.485418, 0.354167, 0, 19.485418, 0.354167, 0, 19.483562, 0.392782, 0, 19.485418, 0.354167, 0, 19.485418, 0.354167, 0, 19.483562, 0.392782, 0, 19.483562, 0.392782, 0, 19.483562, 0.392782, 0, 19.477995, 0.426074, 0, 19.483562, 0.392782, 0, 19.483562, 0.392782, 0, 19.477995, 0.426074, 0, 19.477995, 0.426074, 0, 19.477995, 0.426074, 0, 19.468718, 0.454045, 0, 19.477995, 0.426074, 0, 19.477995, 0.426074, 0, 19.468718, 0.454045, 0, 19.468718, 0.454045, 0, 19.468718, 0.454045, 0, 19.455730, 0.476693, 0, 19.468718, 0.454045, 0, 19.468718, 0.454045, 0, 19.455730, 0.476693, 0, 19.455730, 0.476693, 0, 19.455730, 0.476693, 0, 19.438932, 0.494181, 0, 19.455730, 0.476693, 0, 19.455730, 0.476693, 0, 19.438932, 0.494181, 0, 19.438932, 0.494181, 0, 19.438932, 0.494181, 0, 19.418230, 0.506673, 0, 19.438932, 0.494181, 0, 19.438932, 0.494181, 0, 19.418230, 0.506673, 0, 19.418230, 0.506673, 0, 19.418230, 0.506673, 0, 19.393621, 0.514168, 0, 19.418230, 0.506673, 0, 19.418230, 0.506673, 0, 19.393621, 0.514168, 0, 19.393621, 0.514168, 0, 19.393621, 0.514168, 0, 19.365105, 0.516667, 0, 19.393621, 0.514168, 0, 19.393621, 0.514168, 0, 19.365105, 0.516667, 0, 19.365105, 0.516667, 0, 19.365105, 0.516667, 0, 19.329817, 0.513615, 0, 19.365105, 0.516667, 0, 19.365105, 0.516667, 0, 19.329817, 0.513615, 0, 19.329817, 0.513615, 0, 19.329817, 0.513615, 0, 19.298437, 0.504460, 0, 19.329817, 0.513615, 0, 19.329817, 0.513615, 0, 19.298437, 0.504460, 0, 19.298437, 0.504460, 0, 19.298437, 0.504460, 0, 19.270964, 0.489201, 0, 19.298437, 0.504460, 0, 19.298437, 0.504460, 0, 19.270964, 0.489201, 0, 19.270964, 0.489201, 0, 19.270964, 0.489201, 0, 19.247396, 0.467839, 0, 19.270964, 0.489201, 0, 19.270964, 0.489201, 0, 19.247396, 0.467839, 0, 19.247396, 0.467839, 0, 19.247396, 0.467839, 0, 19.228483, 0.441203, 0, 19.247396, 0.467839, 0, 19.247396, 0.467839, 0, 19.228483, 0.441203, 0, 19.228483, 0.441203, 0, 19.228483, 0.441203, 0, 19.214973, 0.410124, 0, 19.228483, 0.441203, 0, 19.228483, 0.441203, 0, 19.214973, 0.410124, 0, 19.214973, 0.410124, 0, 19.214973, 0.410124, 0, 19.206869, 0.374601, 0, 19.214973, 0.410124, 0, 19.214973, 0.410124, 0, 19.206869, 0.374601, 0, 19.206869, 0.374601, 0, 19.206869, 0.374601, 0, 19.204168, 0.334635, 0, 19.206869, 0.374601, 0, 19.206869, 0.374601, 0, 19.204168, 0.334635, 0, 19.204168, 0.334635, 0, 19.204168, 0.334635, 0, 19.204168, 0, 0, 19.204168, 0.334635, 0, 19.204168, 0.334635, 0, 19.204168, 0, 0, 19.204168, 0, 0, 19.204168, 0, 0, 19.107813, 0, 0, 19.204168, 0, 0, 19.204168, 0, 0, 19.107813, 0, 0, 19.107813, 0, 0, 19.107813, 0, 0, 19.107813, 0.354167, 0, 19.107813, 0, 0, 19.107813, 0, 0, 19.107813, 0.354167, 0, 19.107813, 0.354167, 0, 19.107813, 0.354167, 0, 19.105957, 0.392993, 0, 19.107813, 0.354167, 0, 19.107813, 0.354167, 0, 19.105957, 0.392993, 0, 19.105957, 0.392993, 0, 19.105957, 0.392993, 0, 19.100391, 0.426400, 0, 19.105957, 0.392993, 0, 19.105957, 0.392993, 0, 19.100391, 0.426400, 0, 19.100391, 0.426400, 0, 19.100391, 0.426400, 0, 19.091114, 0.454386, 0, 19.100391, 0.426400, 0, 19.100391, 0.426400, 0, 19.091114, 0.454386, 0, 19.091114, 0.454386, 0, 19.091114, 0.454386, 0, 19.078125, 0.476953, 0, 19.091114, 0.454386, 0, 19.091114, 0.454386, 0, 19.078125, 0.476953, 0, 19.078125, 0.476953, 0, 19.078125, 0.476953, 0, 19.061262, 0.494328, 0, 19.078125, 0.476953, 0, 19.078125, 0.476953, 0, 19.061262, 0.494328, 0, 19.061262, 0.494328, 0, 19.061262, 0.494328, 0, 19.040365, 0.506738, 0, 19.061262, 0.494328, 0, 19.061262, 0.494328, 0, 19.040365, 0.506738, 0, 19.040365, 0.506738, 0, 19.040365, 0.506738, 0, 19.015430, 0.514185, 0, 19.040365, 0.506738, 0, 19.040365, 0.506738, 0, 19.015430, 0.514185, 0, 19.015430, 0.514185, 0, 19.015430, 0.514185, 0, 18.986458, 0.516667, 0, 19.015430, 0.514185, 0, 19.015430, 0.514185, 0, 18.986458, 0.516667, 0, 18.986458, 0.516667, 0, 18.986458, 0.516667, 0, 18.951628, 0.513599, 0, 18.986458, 0.516667, 0, 18.986458, 0.516667, 0, 18.951628, 0.513599, 0, 18.951628, 0.513599, 0, 18.951628, 0.513599, 0, 18.920574, 0.504395, 0, 18.951628, 0.513599, 0, 18.951628, 0.513599, 0, 18.920574, 0.504395, 0, 18.920574, 0.504395, 0, 18.920574, 0.504395, 0, 18.893293, 0.489054, 0, 18.920574, 0.504395, 0, 18.920574, 0.504395, 0, 18.893293, 0.489054, 0, 18.893293, 0.489054, 0, 18.893293, 0.489054, 0, 18.869793, 0.467578, 0, 18.893293, 0.489054, 0, 18.893293, 0.489054, 0, 18.869793, 0.467578, 0, 18.869793, 0.467578, 0, 18.869793, 0.467578, 0, 18.850880, 0.440861, 0, 18.869793, 0.467578, 0, 18.869793, 0.467578, 0, 18.850880, 0.440861, 0, 18.850880, 0.440861, 0, 18.850880, 0.440861, 0, 18.837370, 0.409798, 0, 18.850880, 0.440861, 0, 18.850880, 0.440861, 0, 18.837370, 0.409798, 0, 18.837370, 0.409798, 0, 18.837370, 0.409798, 0, 18.829266, 0.374390, 0, 18.837370, 0.409798, 0, 18.837370, 0.409798, 0, 18.829266, 0.374390, 0, 18.829266, 0.374390, 0, 18.829266, 0.374390, 0, 18.826563, 0.334635, 0, 18.829266, 0.374390, 0, 18.829266, 0.374390, 0, 18.826563, 0.334635, 0, 18.826563, 0.334635, 0, 18.826563, 0.334635, 0, 18.826563, 0, 0, 18.826563, 0.334635, 0, 18.826563, 0.334635, 0, 18.826563, 0, 0, 18.826563, 0, 0, 18.826563, 0, 0, 18.730207, 0, 0, 18.826563, 0, 0, 18.826563, 0, 0, 18.730207, 0, 0, 18.730207, 0, 0, 18.730207, 0, 0, 18.730207, 0.583333, 0, 18.730207, 0, 0, 18.730207, 0, 0, 18.730207, 0.583333, 0, 18.730207, 0.583333, 0, 18.730207, 0.583333, 0, 18.826563, 0.583333, 0, 18.730207, 0.583333, 0, 18.730207, 0.583333, 0, 18.826563, 0.583333, 0, 18.826563, 0.583333, 0, 18.826563, 0.583333, 0, 18.826563, 0.500000, 0, 18.826563, 0.583333, 0, 18.826563, 0.583333, 0, 18.826563, 0.500000, 0, 18.826563, 0.500000, 0, 18.826563, 0.500000, 0, 18.843782, 0.523966, 0, 18.826563, 0.500000, 0, 18.826563, 0.500000, 0, 18.843782, 0.523966, 0, 18.843782, 0.523966, 0, 18.843782, 0.523966, 0, 18.862631, 0.544564, 0, 18.843782, 0.523966, 0, 18.843782, 0.523966, 0, 18.862631, 0.544564, 0, 18.862631, 0.544564, 0, 18.862631, 0.544564, 0, 18.883106, 0.561792, 0, 18.862631, 0.544564, 0, 18.862631, 0.544564, 0, 18.883106, 0.561792, 0, 18.883106, 0.561792, 0, 18.883106, 0.561792, 0, 18.905209, 0.575651, 0, 18.883106, 0.561792, 0, 18.883106, 0.561792, 0, 18.905209, 0.575651, 0, 18.905209, 0.575651, 0, 18.905209, 0.575651, 0, 18.929199, 0.586304, 0, 18.905209, 0.575651, 0, 18.905209, 0.575651, 0, 18.929199, 0.586304, 0, 18.929199, 0.586304, 0, 18.929199, 0.586304, 0, 18.955339, 0.593913, 0, 18.929199, 0.586304, 0, 18.929199, 0.586304, 0, 18.955339, 0.593913, 0, 18.955339, 0.593913, 0, 18.955339, 0.593913, 0, 18.983625, 0.598478, 0, 18.955339, 0.593913, 0, 18.955339, 0.593913, 0, 18.983625, 0.598478, 0, 18.983625, 0.598478, 0, 18.983625, 0.598478, 0, 19.014063, 0.600000, 0, 18.983625, 0.598478, 0, 18.983625, 0.598478, 0, 19.014063, 0.600000, 0, 19.014063, 0.600000, 0, 19.014063, 0.600000, 0, 19.044645, 0.598055, 0, 19.014063, 0.600000, 0, 19.014063, 0.600000, 0, 19.044645, 0.598055, 0, 19.044645, 0.598055, 0, 19.044645, 0.598055, 0, 19.072851, 0.592220, 0, 19.044645, 0.598055, 0, 19.044645, 0.598055, 0, 19.072851, 0.592220, 0, 19.072851, 0.592220, 0, 19.072851, 0.592220, 0, 19.098682, 0.582495, 0, 19.072851, 0.592220, 0, 19.072851, 0.592220, 0, 19.098682, 0.582495, 0, 19.098682, 0.582495, 0, 19.098682, 0.582495, 0, 19.122137, 0.568880, 0, 19.098682, 0.582495, 0, 19.098682, 0.582495, 0, 19.122137, 0.568880, 0, 19.122137, 0.568880, 0, 19.122137, 0.568880, 0, 19.142952, 0.551554, 0, 19.122137, 0.568880, 0, 19.122137, 0.568880, 0, 19.142952, 0.551554, 0, 19.142952, 0.551554, 0, 19.142952, 0.551554, 0, 19.160872, 0.530697, 0, 19.142952, 0.551554, 0, 19.142952, 0.551554, 0, 19.160872, 0.530697, 0, 19.160872, 0.530697, 0, 19.160872, 0.530697, 0, 19.175896, 0.506307, 0, 19.160872, 0.530697, 0, 19.160872, 0.530697, 0, 19.175896, 0.506307, 0, 19.175896, 0.506307, 0, 18.826563, 0.583333, 0, 18.730207, 0.583333, 0, 18.826563, 0.500000, 0, 18.730207, 0.583333, 0, 18.826563, 0.583333, 0, 18.826563, 0.500000, 0, 18.837370, 0.409798, 0, 18.826563, 0.500000, 0, 18.730207, 0.583333, 0, 18.826563, 0.500000, 0, 18.837370, 0.409798, 0, 18.730207, 0.583333, 0, 18.826563, 0.500000, 0, 18.837370, 0.409798, 0, 18.850880, 0.440861, 0, 18.837370, 0.409798, 0, 18.826563, 0.500000, 0, 18.850880, 0.440861, 0, 18.826563, 0.500000, 0, 18.850880, 0.440861, 0, 18.869793, 0.467578, 0, 18.850880, 0.440861, 0, 18.826563, 0.500000, 0, 18.869793, 0.467578, 0, 18.843782, 0.523966, 0, 18.826563, 0.500000, 0, 18.869793, 0.467578, 0, 18.826563, 0.500000, 0, 18.843782, 0.523966, 0, 18.869793, 0.467578, 0, 18.843782, 0.523966, 0, 18.869793, 0.467578, 0, 18.893293, 0.489054, 0, 18.869793, 0.467578, 0, 18.843782, 0.523966, 0, 18.893293, 0.489054, 0, 18.862631, 0.544564, 0, 18.843782, 0.523966, 0, 18.893293, 0.489054, 0, 18.843782, 0.523966, 0, 18.862631, 0.544564, 0, 18.893293, 0.489054, 0, 18.862631, 0.544564, 0, 18.893293, 0.489054, 0, 18.920574, 0.504395, 0, 18.893293, 0.489054, 0, 18.862631, 0.544564, 0, 18.920574, 0.504395, 0, 18.883106, 0.561792, 0, 18.862631, 0.544564, 0, 18.920574, 0.504395, 0, 18.862631, 0.544564, 0, 18.883106, 0.561792, 0, 18.920574, 0.504395, 0, 18.905209, 0.575651, 0, 18.883106, 0.561792, 0, 18.920574, 0.504395, 0, 18.883106, 0.561792, 0, 18.905209, 0.575651, 0, 18.920574, 0.504395, 0, 18.905209, 0.575651, 0, 18.920574, 0.504395, 0, 18.951628, 0.513599, 0, 18.920574, 0.504395, 0, 18.905209, 0.575651, 0, 18.951628, 0.513599, 0, 18.929199, 0.586304, 0, 18.905209, 0.575651, 0, 18.951628, 0.513599, 0, 18.905209, 0.575651, 0, 18.929199, 0.586304, 0, 18.951628, 0.513599, 0, 18.929199, 0.586304, 0, 18.951628, 0.513599, 0, 18.955339, 0.593913, 0, 18.951628, 0.513599, 0, 18.929199, 0.586304, 0, 18.955339, 0.593913, 0, 18.955339, 0.593913, 0, 18.951628, 0.513599, 0, 18.986458, 0.516667, 0, 18.951628, 0.513599, 0, 18.955339, 0.593913, 0, 18.986458, 0.516667, 0, 18.983625, 0.598478, 0, 18.955339, 0.593913, 0, 18.986458, 0.516667, 0, 18.955339, 0.593913, 0, 18.983625, 0.598478, 0, 18.986458, 0.516667, 0, 19.014063, 0.600000, 0, 18.983625, 0.598478, 0, 18.986458, 0.516667, 0, 18.983625, 0.598478, 0, 19.014063, 0.600000, 0, 18.986458, 0.516667, 0, 19.014063, 0.600000, 0, 18.986458, 0.516667, 0, 19.015430, 0.514185, 0, 18.986458, 0.516667, 0, 19.014063, 0.600000, 0, 19.015430, 0.514185, 0, 19.044645, 0.598055, 0, 19.014063, 0.600000, 0, 19.015430, 0.514185, 0, 19.014063, 0.600000, 0, 19.044645, 0.598055, 0, 19.015430, 0.514185, 0, 19.044645, 0.598055, 0, 19.015430, 0.514185, 0, 19.040365, 0.506738, 0, 19.015430, 0.514185, 0, 19.044645, 0.598055, 0, 19.040365, 0.506738, 0, 19.072851, 0.592220, 0, 19.044645, 0.598055, 0, 19.040365, 0.506738, 0, 19.044645, 0.598055, 0, 19.072851, 0.592220, 0, 19.040365, 0.506738, 0, 19.072851, 0.592220, 0, 19.040365, 0.506738, 0, 19.098682, 0.582495, 0, 19.040365, 0.506738, 0, 19.072851, 0.592220, 0, 19.098682, 0.582495, 0, 19.040365, 0.506738, 0, 19.061262, 0.494328, 0, 19.098682, 0.582495, 0, 19.061262, 0.494328, 0, 19.040365, 0.506738, 0, 19.098682, 0.582495, 0, 19.098682, 0.582495, 0, 19.061262, 0.494328, 0, 19.122137, 0.568880, 0, 19.061262, 0.494328, 0, 19.098682, 0.582495, 0, 19.122137, 0.568880, 0, 19.122137, 0.568880, 0, 19.061262, 0.494328, 0, 19.142952, 0.551554, 0, 19.061262, 0.494328, 0, 19.122137, 0.568880, 0, 19.142952, 0.551554, 0, 19.061262, 0.494328, 0, 19.078125, 0.476953, 0, 19.142952, 0.551554, 0, 19.078125, 0.476953, 0, 19.061262, 0.494328, 0, 19.142952, 0.551554, 0, 19.142952, 0.551554, 0, 19.078125, 0.476953, 0, 19.160872, 0.530697, 0, 19.078125, 0.476953, 0, 19.142952, 0.551554, 0, 19.160872, 0.530697, 0, 19.160872, 0.530697, 0, 19.078125, 0.476953, 0, 19.175896, 0.506307, 0, 19.078125, 0.476953, 0, 19.160872, 0.530697, 0, 19.175896, 0.506307, 0, 19.078125, 0.476953, 0, 19.091114, 0.454386, 0, 19.175896, 0.506307, 0, 19.091114, 0.454386, 0, 19.078125, 0.476953, 0, 19.175896, 0.506307, 0, 19.175896, 0.506307, 0, 19.091114, 0.454386, 0, 19.188021, 0.478385, 0, 19.091114, 0.454386, 0, 19.175896, 0.506307, 0, 19.188021, 0.478385, 0, 19.091114, 0.454386, 0, 19.100391, 0.426400, 0, 19.188021, 0.478385, 0, 19.100391, 0.426400, 0, 19.091114, 0.454386, 0, 19.188021, 0.478385, 0, 19.188021, 0.478385, 0, 19.100391, 0.426400, 0, 19.214973, 0.410124, 0, 19.100391, 0.426400, 0, 19.188021, 0.478385, 0, 19.214973, 0.410124, 0, 19.188021, 0.478385, 0, 19.214973, 0.410124, 0, 19.228483, 0.441203, 0, 19.214973, 0.410124, 0, 19.188021, 0.478385, 0, 19.228483, 0.441203, 0, 19.188021, 0.478385, 0, 19.228483, 0.441203, 0, 19.247396, 0.467839, 0, 19.228483, 0.441203, 0, 19.188021, 0.478385, 0, 19.247396, 0.467839, 0, 19.206869, 0.507471, 0, 19.188021, 0.478385, 0, 19.247396, 0.467839, 0, 19.188021, 0.478385, 0, 19.206869, 0.507471, 0, 19.247396, 0.467839, 0, 19.270964, 0.489201, 0, 19.206869, 0.507471, 0, 19.247396, 0.467839, 0, 19.206869, 0.507471, 0, 19.270964, 0.489201, 0, 19.247396, 0.467839, 0, 19.270964, 0.489201, 0, 19.227474, 0.532487, 0, 19.206869, 0.507471, 0, 19.227474, 0.532487, 0, 19.270964, 0.489201, 0, 19.206869, 0.507471, 0, 19.249838, 0.553434, 0, 19.227474, 0.532487, 0, 19.270964, 0.489201, 0, 19.227474, 0.532487, 0, 19.249838, 0.553434, 0, 19.270964, 0.489201, 0, 19.249838, 0.553434, 0, 19.270964, 0.489201, 0, 19.298437, 0.504460, 0, 19.270964, 0.489201, 0, 19.249838, 0.553434, 0, 19.298437, 0.504460, 0, 19.273958, 0.570313, 0, 19.249838, 0.553434, 0, 19.298437, 0.504460, 0, 19.249838, 0.553434, 0, 19.273958, 0.570313, 0, 19.298437, 0.504460, 0, 19.273958, 0.570313, 0, 19.298437, 0.504460, 0, 19.300066, 0.583301, 0, 19.298437, 0.504460, 0, 19.273958, 0.570313, 0, 19.300066, 0.583301, 0, 19.300066, 0.583301, 0, 19.298437, 0.504460, 0, 19.329817, 0.513615, 0, 19.298437, 0.504460, 0, 19.300066, 0.583301, 0, 19.329817, 0.513615, 0, 19.328386, 0.592578, 0, 19.300066, 0.583301, 0, 19.329817, 0.513615, 0, 19.300066, 0.583301, 0, 19.328386, 0.592578, 0, 19.329817, 0.513615, 0, 19.328386, 0.592578, 0, 19.329817, 0.513615, 0, 19.365105, 0.516667, 0, 19.329817, 0.513615, 0, 19.328386, 0.592578, 0, 19.365105, 0.516667, 0, 19.358919, 0.598145, 0, 19.328386, 0.592578, 0, 19.365105, 0.516667, 0, 19.328386, 0.592578, 0, 19.358919, 0.598145, 0, 19.365105, 0.516667, 0, 19.391668, 0.600000, 0, 19.358919, 0.598145, 0, 19.365105, 0.516667, 0, 19.358919, 0.598145, 0, 19.391668, 0.600000, 0, 19.365105, 0.516667, 0, 19.391668, 0.600000, 0, 19.365105, 0.516667, 0, 19.393621, 0.514168, 0, 19.365105, 0.516667, 0, 19.391668, 0.600000, 0, 19.393621, 0.514168, 0, 19.434637, 0.596053, 0, 19.391668, 0.600000, 0, 19.393621, 0.514168, 0, 19.391668, 0.600000, 0, 19.434637, 0.596053, 0, 19.393621, 0.514168, 0, 19.393621, 0.514168, 0, 19.418230, 0.506673, 0, 19.434637, 0.596053, 0, 19.418230, 0.506673, 0, 19.393621, 0.514168, 0, 19.434637, 0.596053, 0, 19.434637, 0.596053, 0, 19.418230, 0.506673, 0, 19.472395, 0.584212, 0, 19.418230, 0.506673, 0, 19.434637, 0.596053, 0, 19.472395, 0.584212, 0, 19.418230, 0.506673, 0, 19.438932, 0.494181, 0, 19.472395, 0.584212, 0, 19.438932, 0.494181, 0, 19.418230, 0.506673, 0, 19.472395, 0.584212, 0, 19.472395, 0.584212, 0, 19.438932, 0.494181, 0, 19.504950, 0.564478, 0, 19.438932, 0.494181, 0, 19.472395, 0.584212, 0, 19.504950, 0.564478, 0, 19.438932, 0.494181, 0, 19.455730, 0.476693, 0, 19.504950, 0.564478, 0, 19.455730, 0.476693, 0, 19.438932, 0.494181, 0, 19.504950, 0.564478, 0, 19.504950, 0.564478, 0, 19.455730, 0.476693, 0, 19.532293, 0.536849, 0, 19.455730, 0.476693, 0, 19.504950, 0.564478, 0, 19.532293, 0.536849, 0, 19.532293, 0.536849, 0, 19.455730, 0.476693, 0, 19.553938, 0.501945, 0, 19.455730, 0.476693, 0, 19.532293, 0.536849, 0, 19.553938, 0.501945, 0, 19.553938, 0.501945, 0, 19.455730, 0.476693, 0, 19.468718, 0.454045, 0, 19.455730, 0.476693, 0, 19.553938, 0.501945, 0, 19.468718, 0.454045, 0, 19.553938, 0.501945, 0, 19.468718, 0.454045, 0, 19.569403, 0.460384, 0, 19.468718, 0.454045, 0, 19.553938, 0.501945, 0, 19.569403, 0.460384, 0, 19.468718, 0.454045, 0, 19.477995, 0.426074, 0, 19.569403, 0.460384, 0, 19.477995, 0.426074, 0, 19.468718, 0.454045, 0, 19.569403, 0.460384, 0, 19.569403, 0.460384, 0, 19.477995, 0.426074, 0, 19.578680, 0.412166, 0, 19.477995, 0.426074, 0, 19.569403, 0.460384, 0, 19.578680, 0.412166, 0, 19.477995, 0.426074, 0, 19.483562, 0.392782, 0, 19.578680, 0.412166, 0, 19.483562, 0.392782, 0, 19.477995, 0.426074, 0, 19.578680, 0.412166, 0, 19.578680, 0.412166, 0, 19.483562, 0.392782, 0, 19.581770, 0.357292, 0, 19.483562, 0.392782, 0, 19.578680, 0.412166, 0, 19.581770, 0.357292, 0, 19.581770, 0.357292, 0, 19.483562, 0.392782, 0, 19.485418, 0.354167, 0, 19.483562, 0.392782, 0, 19.581770, 0.357292, 0, 19.485418, 0.354167, 0, 19.485418, 0.354167, 0, 19.581770, 0, 0, 19.581770, 0.357292, 0, 19.581770, 0, 0, 19.485418, 0.354167, 0, 19.581770, 0.357292, 0, 19.485418, 0.354167, 0, 19.485418, 0, 0, 19.581770, 0, 0, 19.485418, 0, 0, 19.485418, 0.354167, 0, 19.581770, 0, 0, 19.105957, 0.392993, 0, 19.214973, 0.410124, 0, 19.100391, 0.426400, 0, 19.214973, 0.410124, 0, 19.105957, 0.392993, 0, 19.100391, 0.426400, 0, 19.214973, 0.410124, 0, 19.105957, 0.392993, 0, 19.206869, 0.374601, 0, 19.105957, 0.392993, 0, 19.214973, 0.410124, 0, 19.206869, 0.374601, 0, 19.105957, 0.392993, 0, 19.107813, 0.354167, 0, 19.206869, 0.374601, 0, 19.107813, 0.354167, 0, 19.105957, 0.392993, 0, 19.206869, 0.374601, 0, 19.107813, 0.354167, 0, 19.204168, 0.334635, 0, 19.206869, 0.374601, 0, 19.204168, 0.334635, 0, 19.107813, 0.354167, 0, 19.206869, 0.374601, 0, 19.204168, 0.334635, 0, 19.107813, 0.354167, 0, 19.107813, 0, 0, 19.107813, 0.354167, 0, 19.204168, 0.334635, 0, 19.107813, 0, 0, 19.107813, 0, 0, 19.204168, 0, 0, 19.204168, 0.334635, 0, 19.204168, 0, 0, 19.107813, 0, 0, 19.204168, 0.334635, 0, 18.837370, 0.409798, 0, 18.730207, 0.583333, 0, 18.829266, 0.374390, 0, 18.730207, 0.583333, 0, 18.837370, 0.409798, 0, 18.829266, 0.374390, 0, 18.826563, 0.334635, 0, 18.829266, 0.374390, 0, 18.730207, 0.583333, 0, 18.829266, 0.374390, 0, 18.826563, 0.334635, 0, 18.730207, 0.583333, 0, 18.826563, 0.334635, 0, 18.730207, 0.583333, 0, 18.730207, 0, 0, 18.730207, 0.583333, 0, 18.826563, 0.334635, 0, 18.730207, 0, 0, 18.730207, 0, 0, 18.826563, 0, 0, 18.826563, 0.334635, 0, 18.826563, 0, 0, 18.730207, 0, 0, 18.826563, 0.334635, 0, 19.767187, 0.583333, 0, 19.863020, 0.583333, 0, 19.767187, 0.583333, 0, 19.767187, 0.583333, 0, 19.863020, 0.583333, 0, 19.863020, 0.583333, 0, 19.863020, 0.583333, 0, 19.863020, 0, 0, 19.863020, 0.583333, 0, 19.863020, 0.583333, 0, 19.863020, 0, 0, 19.863020, 0, 0, 19.863020, 0, 0, 19.767187, 0, 0, 19.863020, 0, 0, 19.863020, 0, 0, 19.767187, 0, 0, 19.767187, 0, 0, 19.767187, 0.583333, 0, 19.863020, 0, 0, 19.863020, 0.583333, 0, 19.863020, 0, 0, 19.767187, 0.583333, 0, 19.863020, 0.583333, 0, 19.767187, 0.583333, 0, 19.767187, 0, 0, 19.863020, 0, 0, 19.767187, 0, 0, 19.767187, 0.583333, 0, 19.863020, 0, 0, 19.767187, 0.816667, 0, 19.863020, 0.816667, 0, 19.767187, 0.816667, 0, 19.767187, 0.816667, 0, 19.863020, 0.816667, 0, 19.863020, 0.816667, 0, 19.863020, 0.816667, 0, 19.863020, 0.700000, 0, 19.863020, 0.816667, 0, 19.863020, 0.816667, 0, 19.863020, 0.700000, 0, 19.863020, 0.700000, 0, 19.863020, 0.700000, 0, 19.767187, 0.700000, 0, 19.863020, 0.700000, 0, 19.863020, 0.700000, 0, 19.767187, 0.700000, 0, 19.767187, 0.700000, 0, 19.767187, 0.816667, 0, 19.863020, 0.700000, 0, 19.863020, 0.816667, 0, 19.863020, 0.700000, 0, 19.767187, 0.816667, 0, 19.863020, 0.816667, 0, 19.767187, 0.816667, 0, 19.767187, 0.700000, 0, 19.863020, 0.700000, 0, 19.767187, 0.700000, 0, 19.767187, 0.816667, 0, 19.863020, 0.700000, 0, 20.067188, 0.816667, 0, 20.163021, 0.816667, 0, 20.067188, 0.816667, 0, 20.067188, 0.816667, 0, 20.163021, 0.816667, 0, 20.163021, 0.816667, 0, 20.163021, 0.816667, 0, 20.163021, 0, 0, 20.163021, 0.816667, 0, 20.163021, 0.816667, 0, 20.163021, 0, 0, 20.163021, 0, 0, 20.163021, 0, 0, 20.067188, 0, 0, 20.163021, 0, 0, 20.163021, 0, 0, 20.067188, 0, 0, 20.067188, 0, 0, 20.067188, 0.816667, 0, 20.163021, 0, 0, 20.163021, 0.816667, 0, 20.163021, 0, 0, 20.067188, 0.816667, 0, 20.163021, 0.816667, 0, 20.067188, 0.816667, 0, 20.067188, 0, 0, 20.163021, 0, 0, 20.067188, 0, 0, 20.067188, 0.816667, 0, 20.163021, 0, 0, 20.632292, 0.283333, 0, 20.578680, 0.281755, 0, 20.632292, 0.283333, 0, 20.632292, 0.283333, 0, 20.578680, 0.281755, 0, 20.578680, 0.281755, 0, 20.578680, 0.281755, 0, 20.533985, 0.277018, 0, 20.578680, 0.281755, 0, 20.578680, 0.281755, 0, 20.533985, 0.277018, 0, 20.533985, 0.277018, 0, 20.533985, 0.277018, 0, 20.498211, 0.269124, 0, 20.533985, 0.277018, 0, 20.533985, 0.277018, 0, 20.498211, 0.269124, 0, 20.498211, 0.269124, 0, 20.498211, 0.269124, 0, 20.471355, 0.258073, 0, 20.498211, 0.269124, 0, 20.498211, 0.269124, 0, 20.471355, 0.258073, 0, 20.471355, 0.258073, 0, 20.471355, 0.258073, 0, 20.451757, 0.243229, 0, 20.471355, 0.258073, 0, 20.471355, 0.258073, 0, 20.451757, 0.243229, 0, 20.451757, 0.243229, 0, 20.451757, 0.243229, 0, 20.437761, 0.223958, 0, 20.451757, 0.243229, 0, 20.451757, 0.243229, 0, 20.437761, 0.223958, 0, 20.437761, 0.223958, 0, 20.437761, 0.223958, 0, 20.429363, 0.200260, 0, 20.437761, 0.223958, 0, 20.437761, 0.223958, 0, 20.429363, 0.200260, 0, 20.429363, 0.200260, 0, 20.429363, 0.200260, 0, 20.426563, 0.172135, 0, 20.429363, 0.200260, 0, 20.429363, 0.200260, 0, 20.426563, 0.172135, 0, 20.426563, 0.172135, 0, 20.426563, 0.172135, 0, 20.428663, 0.149056, 0, 20.426563, 0.172135, 0, 20.426563, 0.172135, 0, 20.428663, 0.149056, 0, 20.428663, 0.149056, 0, 20.428663, 0.149056, 0, 20.434961, 0.128516, 0, 20.428663, 0.149056, 0, 20.428663, 0.149056, 0, 20.434961, 0.128516, 0, 20.434961, 0.128516, 0, 20.434961, 0.128516, 0, 20.445459, 0.110514, 0, 20.434961, 0.128516, 0, 20.434961, 0.128516, 0, 20.445459, 0.110514, 0, 20.445459, 0.110514, 0, 20.445459, 0.110514, 0, 20.460157, 0.095052, 0, 20.445459, 0.110514, 0, 20.445459, 0.110514, 0, 20.460157, 0.095052, 0, 20.460157, 0.095052, 0, 20.460157, 0.095052, 0, 20.478468, 0.082633, 0, 20.460157, 0.095052, 0, 20.460157, 0.095052, 0, 20.478468, 0.082633, 0, 20.478468, 0.082633, 0, 20.478468, 0.082633, 0, 20.499805, 0.073763, 0, 20.478468, 0.082633, 0, 20.478468, 0.082633, 0, 20.499805, 0.073763, 0, 20.499805, 0.073763, 0, 20.499805, 0.073763, 0, 20.524170, 0.068441, 0, 20.499805, 0.073763, 0, 20.499805, 0.073763, 0, 20.524170, 0.068441, 0, 20.524170, 0.068441, 0, 20.524170, 0.068441, 0, 20.551563, 0.066667, 0, 20.524170, 0.068441, 0, 20.524170, 0.068441, 0, 20.551563, 0.066667, 0, 20.551563, 0.066667, 0, 20.551563, 0.066667, 0, 20.589437, 0.070011, 0, 20.551563, 0.066667, 0, 20.551563, 0.066667, 0, 20.589437, 0.070011, 0, 20.589437, 0.070011, 0, 20.589437, 0.070011, 0, 20.623373, 0.080046, 0, 20.589437, 0.070011, 0, 20.589437, 0.070011, 0, 20.623373, 0.080046, 0, 20.623373, 0.080046, 0, 20.623373, 0.080046, 0, 20.653370, 0.096769, 0, 20.623373, 0.080046, 0, 20.623373, 0.080046, 0, 20.653370, 0.096769, 0, 20.653370, 0.096769, 0, 20.653370, 0.096769, 0, 20.679426, 0.120182, 0, 20.653370, 0.096769, 0, 20.653370, 0.096769, 0, 20.679426, 0.120182, 0, 20.679426, 0.120182, 0, 20.679426, 0.120182, 0, 20.700506, 0.149146, 0, 20.679426, 0.120182, 0, 20.679426, 0.120182, 0, 20.700506, 0.149146, 0, 20.700506, 0.149146, 0, 20.700506, 0.149146, 0, 20.715559, 0.182520, 0, 20.700506, 0.149146, 0, 20.700506, 0.149146, 0, 20.715559, 0.182520, 0, 20.715559, 0.182520, 0, 20.715559, 0.182520, 0, 20.724592, 0.220304, 0, 20.715559, 0.182520, 0, 20.715559, 0.182520, 0, 20.724592, 0.220304, 0, 20.724592, 0.220304, 0, 20.724592, 0.220304, 0, 20.727606, 0.262500, 0, 20.724592, 0.220304, 0, 20.724592, 0.220304, 0, 20.727606, 0.262500, 0, 20.727606, 0.262500, 0, 20.727606, 0.262500, 0, 20.727606, 0.283333, 0, 20.727606, 0.262500, 0, 20.727606, 0.262500, 0, 20.727606, 0.283333, 0, 20.727606, 0.283333, 0, 20.823439, 0.327344, 0, 20.823439, 0, 0, 20.823439, 0.327344, 0, 20.823439, 0.327344, 0, 20.823439, 0, 0, 20.823439, 0, 0, 20.823439, 0, 0, 20.727606, 0, 0, 20.823439, 0, 0, 20.823439, 0, 0, 20.727606, 0, 0, 20.727606, 0, 0, 20.727606, 0, 0, 20.727606, 0.083333, 0, 20.727606, 0, 0, 20.727606, 0, 0, 20.727606, 0.083333, 0, 20.727606, 0.083333, 0, 20.727606, 0.083333, 0, 20.710188, 0.059367, 0, 20.727606, 0.083333, 0, 20.727606, 0.083333, 0, 20.710188, 0.059367, 0, 20.710188, 0.059367, 0, 20.710188, 0.059367, 0, 20.690756, 0.038770, 0, 20.710188, 0.059367, 0, 20.710188, 0.059367, 0, 20.690756, 0.038770, 0, 20.690756, 0.038770, 0, 20.690756, 0.038770, 0, 20.669304, 0.021541, 0, 20.690756, 0.038770, 0, 20.690756, 0.038770, 0, 20.669304, 0.021541, 0, 20.669304, 0.021541, 0, 20.669304, 0.021541, 0, 20.645834, 0.007682, 0, 20.669304, 0.021541, 0, 20.669304, 0.021541, 0, 20.645834, 0.007682, 0, 20.645834, 0.007682, 0, 20.645834, 0.007682, 0, 20.619987, -0.002970, 0, 20.645834, 0.007682, 0, 20.645834, 0.007682, 0, 20.619987, -0.002970, 0, 20.619987, -0.002970, 0, 20.619987, -0.002970, 0, 20.591406, -0.010579, 0, 20.619987, -0.002970, 0, 20.619987, -0.002970, 0, 20.591406, -0.010579, 0, 20.591406, -0.010579, 0, 20.591406, -0.010579, 0, 20.560091, -0.015145, 0, 20.591406, -0.010579, 0, 20.591406, -0.010579, 0, 20.560091, -0.015145, 0, 20.560091, -0.015145, 0, 20.560091, -0.015145, 0, 20.526043, -0.016667, 0, 20.560091, -0.015145, 0, 20.560091, -0.015145, 0, 20.526043, -0.016667, 0, 20.526043, -0.016667, 0, 20.526043, -0.016667, 0, 20.483545, -0.013525, 0, 20.526043, -0.016667, 0, 20.526043, -0.016667, 0, 20.483545, -0.013525, 0, 20.483545, -0.013525, 0, 20.483545, -0.013525, 0, 20.445639, -0.004102, 0, 20.483545, -0.013525, 0, 20.483545, -0.013525, 0, 20.445639, -0.004102, 0, 20.445639, -0.004102, 0, 20.445639, -0.004102, 0, 20.412321, 0.011605, 0, 20.445639, -0.004102, 0, 20.445639, -0.004102, 0, 20.412321, 0.011605, 0, 20.412321, 0.011605, 0, 20.412321, 0.011605, 0, 20.383595, 0.033594, 0, 20.412321, 0.011605, 0, 20.412321, 0.011605, 0, 20.383595, 0.033594, 0, 20.383595, 0.033594, 0, 20.383595, 0.033594, 0, 20.360466, 0.060856, 0, 20.383595, 0.033594, 0, 20.383595, 0.033594, 0, 20.360466, 0.060856, 0, 20.360466, 0.060856, 0, 20.360466, 0.060856, 0, 20.343946, 0.092383, 0, 20.360466, 0.060856, 0, 20.360466, 0.060856, 0, 20.343946, 0.092383, 0, 20.343946, 0.092383, 0, 20.343946, 0.092383, 0, 20.334034, 0.128174, 0, 20.343946, 0.092383, 0, 20.343946, 0.092383, 0, 20.334034, 0.128174, 0, 20.334034, 0.128174, 0, 20.334034, 0.128174, 0, 20.330730, 0.168229, 0, 20.334034, 0.128174, 0, 20.334034, 0.128174, 0, 20.330730, 0.168229, 0, 20.330730, 0.168229, 0, 20.330730, 0.168229, 0, 20.334846, 0.214421, 0, 20.330730, 0.168229, 0, 20.330730, 0.168229, 0, 20.334846, 0.214421, 0, 20.334846, 0.214421, 0, 20.334846, 0.214421, 0, 20.347200, 0.254557, 0, 20.334846, 0.214421, 0, 20.334846, 0.214421, 0, 20.347200, 0.254557, 0, 20.347200, 0.254557, 0, 20.347200, 0.254557, 0, 20.367790, 0.288639, 0, 20.347200, 0.254557, 0, 20.347200, 0.254557, 0, 20.367790, 0.288639, 0, 20.367790, 0.288639, 0, 20.367790, 0.288639, 0, 20.396614, 0.316667, 0, 20.367790, 0.288639, 0, 20.367790, 0.288639, 0, 20.396614, 0.316667, 0, 20.396614, 0.316667, 0, 20.396614, 0.316667, 0, 20.433611, 0.338542, 0, 20.396614, 0.316667, 0, 20.396614, 0.316667, 0, 20.433611, 0.338542, 0, 20.433611, 0.338542, 0, 20.433611, 0.338542, 0, 20.478712, 0.354167, 0, 20.433611, 0.338542, 0, 20.433611, 0.338542, 0, 20.478712, 0.354167, 0, 20.478712, 0.354167, 0, 20.478712, 0.354167, 0, 20.531918, 0.363542, 0, 20.478712, 0.354167, 0, 20.478712, 0.354167, 0, 20.531918, 0.363542, 0, 20.531918, 0.363542, 0, 20.531918, 0.363542, 0, 20.593229, 0.366667, 0, 20.531918, 0.363542, 0, 20.531918, 0.363542, 0, 20.593229, 0.366667, 0, 20.593229, 0.366667, 0, 20.593229, 0.366667, 0, 20.727606, 0.366667, 0, 20.593229, 0.366667, 0, 20.593229, 0.366667, 0, 20.727606, 0.366667, 0, 20.727606, 0.366667, 0, 20.727606, 0.366667, 0, 20.727606, 0.376042, 0, 20.727606, 0.366667, 0, 20.727606, 0.366667, 0, 20.727606, 0.376042, 0, 20.727606, 0.376042, 0, 20.727606, 0.376042, 0, 20.724886, 0.407731, 0, 20.727606, 0.376042, 0, 20.727606, 0.376042, 0, 20.724886, 0.407731, 0, 20.724886, 0.407731, 0, 20.724886, 0.407731, 0, 20.716732, 0.435612, 0, 20.724886, 0.407731, 0, 20.724886, 0.407731, 0, 20.716732, 0.435612, 0, 20.716732, 0.435612, 0, 20.716732, 0.435612, 0, 20.703142, 0.459684, 0, 20.716732, 0.435612, 0, 20.716732, 0.435612, 0, 20.703142, 0.459684, 0, 20.703142, 0.459684, 0, 20.703142, 0.459684, 0, 20.684114, 0.479948, 0, 20.703142, 0.459684, 0, 20.703142, 0.459684, 0, 20.684114, 0.479948, 0, 20.684114, 0.479948, 0, 20.684114, 0.479948, 0, 20.660173, 0.496012, 0, 20.684114, 0.479948, 0, 20.684114, 0.479948, 0, 20.660173, 0.496012, 0, 20.660173, 0.496012, 0, 20.660173, 0.496012, 0, 20.631836, 0.507487, 0, 20.660173, 0.496012, 0, 20.660173, 0.496012, 0, 20.631836, 0.507487, 0, 20.631836, 0.507487, 0, 20.631836, 0.507487, 0, 20.599104, 0.514372, 0, 20.631836, 0.507487, 0, 20.631836, 0.507487, 0, 20.599104, 0.514372, 0, 20.599104, 0.514372, 0, 20.599104, 0.514372, 0, 20.561979, 0.516667, 0, 20.599104, 0.514372, 0, 20.599104, 0.514372, 0, 20.561979, 0.516667, 0, 20.561979, 0.516667, 0, 20.561979, 0.516667, 0, 20.537142, 0.515885, 0, 20.561979, 0.516667, 0, 20.561979, 0.516667, 0, 20.537142, 0.515885, 0, 20.537142, 0.515885, 0, 20.537142, 0.515885, 0, 20.512630, 0.513542, 0, 20.537142, 0.515885, 0, 20.537142, 0.515885, 0, 20.512630, 0.513542, 0, 20.512630, 0.513542, 0, 20.512630, 0.513542, 0, 20.488445, 0.509635, 0, 20.512630, 0.513542, 0, 20.512630, 0.513542, 0, 20.488445, 0.509635, 0, 20.488445, 0.509635, 0, 20.488445, 0.509635, 0, 20.464582, 0.504167, 0, 20.488445, 0.509635, 0, 20.488445, 0.509635, 0, 20.464582, 0.504167, 0, 20.464582, 0.504167, 0, 20.464582, 0.504167, 0, 20.441114, 0.497135, 0, 20.464582, 0.504167, 0, 20.464582, 0.504167, 0, 20.441114, 0.497135, 0, 20.441114, 0.497135, 0, 20.441114, 0.497135, 0, 20.418098, 0.488542, 0, 20.441114, 0.497135, 0, 20.441114, 0.497135, 0, 20.418098, 0.488542, 0, 20.418098, 0.488542, 0, 20.418098, 0.488542, 0, 20.395540, 0.478385, 0, 20.418098, 0.488542, 0, 20.418098, 0.488542, 0, 20.395540, 0.478385, 0, 20.395540, 0.478385, 0, 20.395540, 0.478385, 0, 20.373438, 0.466667, 0, 20.395540, 0.478385, 0, 20.395540, 0.478385, 0, 20.373438, 0.466667, 0, 20.373438, 0.466667, 0, 20.373438, 0.466667, 0, 20.373438, 0.550000, 0, 20.373438, 0.466667, 0, 20.373438, 0.466667, 0, 20.373438, 0.550000, 0, 20.373438, 0.550000, 0, 20.373438, 0.550000, 0, 20.399544, 0.561719, 0, 20.373438, 0.550000, 0, 20.373438, 0.550000, 0, 20.399544, 0.561719, 0, 20.399544, 0.561719, 0, 20.399544, 0.561719, 0, 20.425261, 0.571875, 0, 20.399544, 0.561719, 0, 20.399544, 0.561719, 0, 20.425261, 0.571875, 0, 20.425261, 0.571875, 0, 20.425261, 0.571875, 0, 20.450586, 0.580469, 0, 20.425261, 0.571875, 0, 20.425261, 0.571875, 0, 20.450586, 0.580469, 0, 20.450586, 0.580469, 0, 20.450586, 0.580469, 0, 20.475521, 0.587500, 0, 20.450586, 0.580469, 0, 20.450586, 0.580469, 0, 20.475521, 0.587500, 0, 20.475521, 0.587500, 0, 20.475521, 0.587500, 0, 20.500097, 0.592969, 0, 20.475521, 0.587500, 0, 20.475521, 0.587500, 0, 20.500097, 0.592969, 0, 20.500097, 0.592969, 0, 20.500097, 0.592969, 0, 20.524349, 0.596875, 0, 20.500097, 0.592969, 0, 20.500097, 0.592969, 0, 20.524349, 0.596875, 0, 20.524349, 0.596875, 0, 20.524349, 0.596875, 0, 20.548275, 0.599219, 0, 20.524349, 0.596875, 0, 20.524349, 0.596875, 0, 20.548275, 0.599219, 0, 20.548275, 0.599219, 0, 20.548275, 0.599219, 0, 20.571875, 0.600000, 0, 20.548275, 0.599219, 0, 20.548275, 0.599219, 0, 20.571875, 0.600000, 0, 20.571875, 0.600000, 0, 20.571875, 0.600000, 0, 20.631153, 0.595776, 0, 20.571875, 0.600000, 0, 20.571875, 0.600000, 0, 20.631153, 0.595776, 0, 20.631153, 0.595776, 0, 20.631153, 0.595776, 0, 20.682423, 0.583106, 0, 20.631153, 0.595776, 0, 20.631153, 0.595776, 0, 20.682423, 0.583106, 0, 20.682423, 0.583106, 0, 20.682423, 0.583106, 0, 20.725683, 0.561987, 0, 20.682423, 0.583106, 0, 20.682423, 0.583106, 0, 20.725683, 0.561987, 0, 20.725683, 0.561987, 0, 20.725683, 0.561987, 0, 20.760939, 0.532422, 0, 20.725683, 0.561987, 0, 20.725683, 0.561987, 0, 20.760939, 0.532422, 0, 20.760939, 0.532422, 0, 20.760939, 0.532422, 0, 20.788282, 0.494263, 0, 20.760939, 0.532422, 0, 20.760939, 0.532422, 0, 20.788282, 0.494263, 0, 20.788282, 0.494263, 0, 20.788282, 0.494263, 0, 20.807814, 0.447363, 0, 20.788282, 0.494263, 0, 20.788282, 0.494263, 0, 20.807814, 0.447363, 0, 20.807814, 0.447363, 0, 20.807814, 0.447363, 0, 20.819532, 0.391724, 0, 20.807814, 0.447363, 0, 20.807814, 0.447363, 0, 20.819532, 0.391724, 0, 20.819532, 0.391724, 0, 20.334846, 0.214421, 0, 20.330730, 0.168229, 0, 20.426563, 0.172135, 0, 20.330730, 0.168229, 0, 20.334846, 0.214421, 0, 20.426563, 0.172135, 0, 20.334846, 0.214421, 0, 20.426563, 0.172135, 0, 20.429363, 0.200260, 0, 20.426563, 0.172135, 0, 20.334846, 0.214421, 0, 20.429363, 0.200260, 0, 20.347200, 0.254557, 0, 20.334846, 0.214421, 0, 20.429363, 0.200260, 0, 20.334846, 0.214421, 0, 20.347200, 0.254557, 0, 20.429363, 0.200260, 0, 20.347200, 0.254557, 0, 20.429363, 0.200260, 0, 20.437761, 0.223958, 0, 20.429363, 0.200260, 0, 20.347200, 0.254557, 0, 20.437761, 0.223958, 0, 20.367790, 0.288639, 0, 20.347200, 0.254557, 0, 20.437761, 0.223958, 0, 20.347200, 0.254557, 0, 20.367790, 0.288639, 0, 20.437761, 0.223958, 0, 20.367790, 0.288639, 0, 20.437761, 0.223958, 0, 20.451757, 0.243229, 0, 20.437761, 0.223958, 0, 20.367790, 0.288639, 0, 20.451757, 0.243229, 0, 20.396614, 0.316667, 0, 20.367790, 0.288639, 0, 20.451757, 0.243229, 0, 20.367790, 0.288639, 0, 20.396614, 0.316667, 0, 20.451757, 0.243229, 0, 20.396614, 0.316667, 0, 20.451757, 0.243229, 0, 20.471355, 0.258073, 0, 20.451757, 0.243229, 0, 20.396614, 0.316667, 0, 20.471355, 0.258073, 0, 20.433611, 0.338542, 0, 20.396614, 0.316667, 0, 20.471355, 0.258073, 0, 20.396614, 0.316667, 0, 20.433611, 0.338542, 0, 20.471355, 0.258073, 0, 20.498211, 0.269124, 0, 20.433611, 0.338542, 0, 20.471355, 0.258073, 0, 20.433611, 0.338542, 0, 20.498211, 0.269124, 0, 20.471355, 0.258073, 0, 20.478712, 0.354167, 0, 20.433611, 0.338542, 0, 20.498211, 0.269124, 0, 20.433611, 0.338542, 0, 20.478712, 0.354167, 0, 20.498211, 0.269124, 0, 20.533985, 0.277018, 0, 20.478712, 0.354167, 0, 20.498211, 0.269124, 0, 20.478712, 0.354167, 0, 20.533985, 0.277018, 0, 20.498211, 0.269124, 0, 20.531918, 0.363542, 0, 20.478712, 0.354167, 0, 20.533985, 0.277018, 0, 20.478712, 0.354167, 0, 20.531918, 0.363542, 0, 20.533985, 0.277018, 0, 20.531918, 0.363542, 0, 20.533985, 0.277018, 0, 20.578680, 0.281755, 0, 20.533985, 0.277018, 0, 20.531918, 0.363542, 0, 20.578680, 0.281755, 0, 20.531918, 0.363542, 0, 20.578680, 0.281755, 0, 20.593229, 0.366667, 0, 20.578680, 0.281755, 0, 20.531918, 0.363542, 0, 20.593229, 0.366667, 0, 20.593229, 0.366667, 0, 20.578680, 0.281755, 0, 20.632292, 0.283333, 0, 20.578680, 0.281755, 0, 20.593229, 0.366667, 0, 20.632292, 0.283333, 0, 20.593229, 0.366667, 0, 20.632292, 0.283333, 0, 20.727606, 0.366667, 0, 20.632292, 0.283333, 0, 20.593229, 0.366667, 0, 20.727606, 0.366667, 0, 20.632292, 0.283333, 0, 20.727606, 0.283333, 0, 20.727606, 0.366667, 0, 20.727606, 0.283333, 0, 20.632292, 0.283333, 0, 20.727606, 0.366667, 0, 20.727606, 0.366667, 0, 20.727606, 0.283333, 0, 20.823439, 0.327344, 0, 20.727606, 0.283333, 0, 20.727606, 0.366667, 0, 20.823439, 0.327344, 0, 20.823439, 0.327344, 0, 20.819532, 0.391724, 0, 20.727606, 0.366667, 0, 20.819532, 0.391724, 0, 20.823439, 0.327344, 0, 20.727606, 0.366667, 0, 20.819532, 0.391724, 0, 20.727606, 0.376042, 0, 20.727606, 0.366667, 0, 20.727606, 0.376042, 0, 20.819532, 0.391724, 0, 20.727606, 0.366667, 0, 20.724886, 0.407731, 0, 20.727606, 0.376042, 0, 20.819532, 0.391724, 0, 20.727606, 0.376042, 0, 20.724886, 0.407731, 0, 20.819532, 0.391724, 0, 20.807814, 0.447363, 0, 20.724886, 0.407731, 0, 20.819532, 0.391724, 0, 20.724886, 0.407731, 0, 20.807814, 0.447363, 0, 20.819532, 0.391724, 0, 20.716732, 0.435612, 0, 20.724886, 0.407731, 0, 20.807814, 0.447363, 0, 20.724886, 0.407731, 0, 20.716732, 0.435612, 0, 20.807814, 0.447363, 0, 20.788282, 0.494263, 0, 20.716732, 0.435612, 0, 20.807814, 0.447363, 0, 20.716732, 0.435612, 0, 20.788282, 0.494263, 0, 20.807814, 0.447363, 0, 20.703142, 0.459684, 0, 20.716732, 0.435612, 0, 20.788282, 0.494263, 0, 20.716732, 0.435612, 0, 20.703142, 0.459684, 0, 20.788282, 0.494263, 0, 20.760939, 0.532422, 0, 20.703142, 0.459684, 0, 20.788282, 0.494263, 0, 20.703142, 0.459684, 0, 20.760939, 0.532422, 0, 20.788282, 0.494263, 0, 20.684114, 0.479948, 0, 20.703142, 0.459684, 0, 20.760939, 0.532422, 0, 20.703142, 0.459684, 0, 20.684114, 0.479948, 0, 20.760939, 0.532422, 0, 20.725683, 0.561987, 0, 20.684114, 0.479948, 0, 20.760939, 0.532422, 0, 20.684114, 0.479948, 0, 20.725683, 0.561987, 0, 20.760939, 0.532422, 0, 20.660173, 0.496012, 0, 20.684114, 0.479948, 0, 20.725683, 0.561987, 0, 20.684114, 0.479948, 0, 20.660173, 0.496012, 0, 20.725683, 0.561987, 0, 20.682423, 0.583106, 0, 20.660173, 0.496012, 0, 20.725683, 0.561987, 0, 20.660173, 0.496012, 0, 20.682423, 0.583106, 0, 20.725683, 0.561987, 0, 20.631836, 0.507487, 0, 20.660173, 0.496012, 0, 20.682423, 0.583106, 0, 20.660173, 0.496012, 0, 20.631836, 0.507487, 0, 20.682423, 0.583106, 0, 20.631153, 0.595776, 0, 20.631836, 0.507487, 0, 20.682423, 0.583106, 0, 20.631836, 0.507487, 0, 20.631153, 0.595776, 0, 20.682423, 0.583106, 0, 20.631153, 0.595776, 0, 20.599104, 0.514372, 0, 20.631836, 0.507487, 0, 20.599104, 0.514372, 0, 20.631153, 0.595776, 0, 20.631836, 0.507487, 0, 20.631153, 0.595776, 0, 20.571875, 0.600000, 0, 20.599104, 0.514372, 0, 20.571875, 0.600000, 0, 20.631153, 0.595776, 0, 20.599104, 0.514372, 0, 20.561979, 0.516667, 0, 20.599104, 0.514372, 0, 20.571875, 0.600000, 0, 20.599104, 0.514372, 0, 20.561979, 0.516667, 0, 20.571875, 0.600000, 0, 20.571875, 0.600000, 0, 20.548275, 0.599219, 0, 20.561979, 0.516667, 0, 20.548275, 0.599219, 0, 20.571875, 0.600000, 0, 20.561979, 0.516667, 0, 20.548275, 0.599219, 0, 20.537142, 0.515885, 0, 20.561979, 0.516667, 0, 20.537142, 0.515885, 0, 20.548275, 0.599219, 0, 20.561979, 0.516667, 0, 20.524349, 0.596875, 0, 20.537142, 0.515885, 0, 20.548275, 0.599219, 0, 20.537142, 0.515885, 0, 20.524349, 0.596875, 0, 20.548275, 0.599219, 0, 20.524349, 0.596875, 0, 20.512630, 0.513542, 0, 20.537142, 0.515885, 0, 20.512630, 0.513542, 0, 20.524349, 0.596875, 0, 20.537142, 0.515885, 0, 20.500097, 0.592969, 0, 20.512630, 0.513542, 0, 20.524349, 0.596875, 0, 20.512630, 0.513542, 0, 20.500097, 0.592969, 0, 20.524349, 0.596875, 0, 20.500097, 0.592969, 0, 20.475521, 0.587500, 0, 20.512630, 0.513542, 0, 20.475521, 0.587500, 0, 20.500097, 0.592969, 0, 20.512630, 0.513542, 0, 20.512630, 0.513542, 0, 20.475521, 0.587500, 0, 20.488445, 0.509635, 0, 20.475521, 0.587500, 0, 20.512630, 0.513542, 0, 20.488445, 0.509635, 0, 20.475521, 0.587500, 0, 20.450586, 0.580469, 0, 20.488445, 0.509635, 0, 20.450586, 0.580469, 0, 20.475521, 0.587500, 0, 20.488445, 0.509635, 0, 20.450586, 0.580469, 0, 20.464582, 0.504167, 0, 20.488445, 0.509635, 0, 20.464582, 0.504167, 0, 20.450586, 0.580469, 0, 20.488445, 0.509635, 0, 20.450586, 0.580469, 0, 20.425261, 0.571875, 0, 20.464582, 0.504167, 0, 20.425261, 0.571875, 0, 20.450586, 0.580469, 0, 20.464582, 0.504167, 0, 20.425261, 0.571875, 0, 20.441114, 0.497135, 0, 20.464582, 0.504167, 0, 20.441114, 0.497135, 0, 20.425261, 0.571875, 0, 20.464582, 0.504167, 0, 20.425261, 0.571875, 0, 20.399544, 0.561719, 0, 20.441114, 0.497135, 0, 20.399544, 0.561719, 0, 20.425261, 0.571875, 0, 20.441114, 0.497135, 0, 20.399544, 0.561719, 0, 20.418098, 0.488542, 0, 20.441114, 0.497135, 0, 20.418098, 0.488542, 0, 20.399544, 0.561719, 0, 20.441114, 0.497135, 0, 20.399544, 0.561719, 0, 20.373438, 0.550000, 0, 20.418098, 0.488542, 0, 20.373438, 0.550000, 0, 20.399544, 0.561719, 0, 20.418098, 0.488542, 0, 20.373438, 0.550000, 0, 20.395540, 0.478385, 0, 20.418098, 0.488542, 0, 20.395540, 0.478385, 0, 20.373438, 0.550000, 0, 20.418098, 0.488542, 0, 20.373438, 0.550000, 0, 20.373438, 0.466667, 0, 20.395540, 0.478385, 0, 20.373438, 0.466667, 0, 20.373438, 0.550000, 0, 20.395540, 0.478385, 0, 20.823439, 0.327344, 0, 20.727606, 0.283333, 0, 20.727606, 0.262500, 0, 20.727606, 0.283333, 0, 20.823439, 0.327344, 0, 20.727606, 0.262500, 0, 20.823439, 0.327344, 0, 20.727606, 0.262500, 0, 20.724592, 0.220304, 0, 20.727606, 0.262500, 0, 20.823439, 0.327344, 0, 20.724592, 0.220304, 0, 20.823439, 0, 0, 20.823439, 0.327344, 0, 20.724592, 0.220304, 0, 20.823439, 0.327344, 0, 20.823439, 0, 0, 20.724592, 0.220304, 0, 20.823439, 0, 0, 20.724592, 0.220304, 0, 20.715559, 0.182520, 0, 20.724592, 0.220304, 0, 20.823439, 0, 0, 20.715559, 0.182520, 0, 20.727606, 0.083333, 0, 20.823439, 0, 0, 20.715559, 0.182520, 0, 20.823439, 0, 0, 20.727606, 0.083333, 0, 20.715559, 0.182520, 0, 20.727606, 0.083333, 0, 20.727606, 0, 0, 20.823439, 0, 0, 20.727606, 0, 0, 20.727606, 0.083333, 0, 20.823439, 0, 0, 20.715559, 0.182520, 0, 20.700506, 0.149146, 0, 20.727606, 0.083333, 0, 20.700506, 0.149146, 0, 20.715559, 0.182520, 0, 20.727606, 0.083333, 0, 20.700506, 0.149146, 0, 20.679426, 0.120182, 0, 20.727606, 0.083333, 0, 20.679426, 0.120182, 0, 20.700506, 0.149146, 0, 20.727606, 0.083333, 0, 20.679426, 0.120182, 0, 20.710188, 0.059367, 0, 20.727606, 0.083333, 0, 20.710188, 0.059367, 0, 20.679426, 0.120182, 0, 20.727606, 0.083333, 0, 20.679426, 0.120182, 0, 20.653370, 0.096769, 0, 20.710188, 0.059367, 0, 20.653370, 0.096769, 0, 20.679426, 0.120182, 0, 20.710188, 0.059367, 0, 20.653370, 0.096769, 0, 20.690756, 0.038770, 0, 20.710188, 0.059367, 0, 20.690756, 0.038770, 0, 20.653370, 0.096769, 0, 20.710188, 0.059367, 0, 20.690756, 0.038770, 0, 20.653370, 0.096769, 0, 20.669304, 0.021541, 0, 20.653370, 0.096769, 0, 20.690756, 0.038770, 0, 20.669304, 0.021541, 0, 20.653370, 0.096769, 0, 20.623373, 0.080046, 0, 20.669304, 0.021541, 0, 20.623373, 0.080046, 0, 20.653370, 0.096769, 0, 20.669304, 0.021541, 0, 20.623373, 0.080046, 0, 20.645834, 0.007682, 0, 20.669304, 0.021541, 0, 20.645834, 0.007682, 0, 20.623373, 0.080046, 0, 20.669304, 0.021541, 0, 20.645834, 0.007682, 0, 20.623373, 0.080046, 0, 20.619987, -0.002970, 0, 20.623373, 0.080046, 0, 20.645834, 0.007682, 0, 20.619987, -0.002970, 0, 20.623373, 0.080046, 0, 20.589437, 0.070011, 0, 20.619987, -0.002970, 0, 20.589437, 0.070011, 0, 20.623373, 0.080046, 0, 20.619987, -0.002970, 0, 20.589437, 0.070011, 0, 20.591406, -0.010579, 0, 20.619987, -0.002970, 0, 20.591406, -0.010579, 0, 20.589437, 0.070011, 0, 20.619987, -0.002970, 0, 20.589437, 0.070011, 0, 20.551563, 0.066667, 0, 20.591406, -0.010579, 0, 20.551563, 0.066667, 0, 20.589437, 0.070011, 0, 20.591406, -0.010579, 0, 20.551563, 0.066667, 0, 20.560091, -0.015145, 0, 20.591406, -0.010579, 0, 20.560091, -0.015145, 0, 20.551563, 0.066667, 0, 20.591406, -0.010579, 0, 20.551563, 0.066667, 0, 20.526043, -0.016667, 0, 20.560091, -0.015145, 0, 20.526043, -0.016667, 0, 20.551563, 0.066667, 0, 20.560091, -0.015145, 0, 20.524170, 0.068441, 0, 20.526043, -0.016667, 0, 20.551563, 0.066667, 0, 20.526043, -0.016667, 0, 20.524170, 0.068441, 0, 20.551563, 0.066667, 0, 20.524170, 0.068441, 0, 20.483545, -0.013525, 0, 20.526043, -0.016667, 0, 20.483545, -0.013525, 0, 20.524170, 0.068441, 0, 20.526043, -0.016667, 0, 20.499805, 0.073763, 0, 20.483545, -0.013525, 0, 20.524170, 0.068441, 0, 20.483545, -0.013525, 0, 20.499805, 0.073763, 0, 20.524170, 0.068441, 0, 20.445639, -0.004102, 0, 20.483545, -0.013525, 0, 20.499805, 0.073763, 0, 20.483545, -0.013525, 0, 20.445639, -0.004102, 0, 20.499805, 0.073763, 0, 20.478468, 0.082633, 0, 20.445639, -0.004102, 0, 20.499805, 0.073763, 0, 20.445639, -0.004102, 0, 20.478468, 0.082633, 0, 20.499805, 0.073763, 0, 20.412321, 0.011605, 0, 20.445639, -0.004102, 0, 20.478468, 0.082633, 0, 20.445639, -0.004102, 0, 20.412321, 0.011605, 0, 20.478468, 0.082633, 0, 20.478468, 0.082633, 0, 20.460157, 0.095052, 0, 20.412321, 0.011605, 0, 20.460157, 0.095052, 0, 20.478468, 0.082633, 0, 20.412321, 0.011605, 0, 20.460157, 0.095052, 0, 20.383595, 0.033594, 0, 20.412321, 0.011605, 0, 20.383595, 0.033594, 0, 20.460157, 0.095052, 0, 20.412321, 0.011605, 0, 20.460157, 0.095052, 0, 20.445459, 0.110514, 0, 20.383595, 0.033594, 0, 20.445459, 0.110514, 0, 20.460157, 0.095052, 0, 20.383595, 0.033594, 0, 20.445459, 0.110514, 0, 20.360466, 0.060856, 0, 20.383595, 0.033594, 0, 20.360466, 0.060856, 0, 20.445459, 0.110514, 0, 20.383595, 0.033594, 0, 20.445459, 0.110514, 0, 20.434961, 0.128516, 0, 20.360466, 0.060856, 0, 20.434961, 0.128516, 0, 20.445459, 0.110514, 0, 20.360466, 0.060856, 0, 20.434961, 0.128516, 0, 20.343946, 0.092383, 0, 20.360466, 0.060856, 0, 20.343946, 0.092383, 0, 20.434961, 0.128516, 0, 20.360466, 0.060856, 0, 20.334034, 0.128174, 0, 20.343946, 0.092383, 0, 20.434961, 0.128516, 0, 20.343946, 0.092383, 0, 20.334034, 0.128174, 0, 20.434961, 0.128516, 0, 20.428663, 0.149056, 0, 20.334034, 0.128174, 0, 20.434961, 0.128516, 0, 20.334034, 0.128174, 0, 20.428663, 0.149056, 0, 20.434961, 0.128516, 0, 20.330730, 0.168229, 0, 20.334034, 0.128174, 0, 20.428663, 0.149056, 0, 20.334034, 0.128174, 0, 20.330730, 0.168229, 0, 20.428663, 0.149056, 0, 20.426563, 0.172135, 0, 20.330730, 0.168229, 0, 20.428663, 0.149056, 0, 20.330730, 0.168229, 0, 20.426563, 0.172135, 0, 20.428663, 0.149056, 0, 21.111980, 0.750000, 0, 21.111980, 0.583333, 0, 21.111980, 0.750000, 0, 21.111980, 0.750000, 0, 21.111980, 0.583333, 0, 21.111980, 0.583333, 0, 21.111980, 0.583333, 0, 21.309376, 0.583333, 0, 21.111980, 0.583333, 0, 21.111980, 0.583333, 0, 21.309376, 0.583333, 0, 21.309376, 0.583333, 0, 21.309376, 0.583333, 0, 21.309376, 0.500000, 0, 21.309376, 0.583333, 0, 21.309376, 0.583333, 0, 21.309376, 0.500000, 0, 21.309376, 0.500000, 0, 21.309376, 0.500000, 0, 21.111980, 0.500000, 0, 21.309376, 0.500000, 0, 21.309376, 0.500000, 0, 21.111980, 0.500000, 0, 21.111980, 0.500000, 0, 21.111980, 0.500000, 0, 21.111980, 0.192188, 0, 21.111980, 0.500000, 0, 21.111980, 0.500000, 0, 21.111980, 0.192188, 0, 21.111980, 0.192188, 0, 21.111980, 0.192188, 0, 21.113201, 0.160645, 0, 21.111980, 0.192188, 0, 21.111980, 0.192188, 0, 21.113201, 0.160645, 0, 21.113201, 0.160645, 0, 21.113201, 0.160645, 0, 21.116863, 0.135286, 0, 21.113201, 0.160645, 0, 21.113201, 0.160645, 0, 21.116863, 0.135286, 0, 21.116863, 0.135286, 0, 21.116863, 0.135286, 0, 21.122967, 0.116113, 0, 21.116863, 0.135286, 0, 21.116863, 0.135286, 0, 21.122967, 0.116113, 0, 21.122967, 0.116113, 0, 21.122967, 0.116113, 0, 21.131512, 0.103125, 0, 21.122967, 0.116113, 0, 21.122967, 0.116113, 0, 21.131512, 0.103125, 0, 21.131512, 0.103125, 0, 21.131512, 0.103125, 0, 21.143799, 0.094466, 0, 21.131512, 0.103125, 0, 21.131512, 0.103125, 0, 21.143799, 0.094466, 0, 21.143799, 0.094466, 0, 21.143799, 0.094466, 0, 21.161133, 0.088281, 0, 21.143799, 0.094466, 0, 21.143799, 0.094466, 0, 21.161133, 0.088281, 0, 21.161133, 0.088281, 0, 21.161133, 0.088281, 0, 21.183514, 0.084570, 0, 21.161133, 0.088281, 0, 21.161133, 0.088281, 0, 21.183514, 0.084570, 0, 21.183514, 0.084570, 0, 21.183514, 0.084570, 0, 21.210938, 0.083333, 0, 21.183514, 0.084570, 0, 21.183514, 0.084570, 0, 21.210938, 0.083333, 0, 21.210938, 0.083333, 0, 21.210938, 0.083333, 0, 21.309376, 0.083333, 0, 21.210938, 0.083333, 0, 21.210938, 0.083333, 0, 21.309376, 0.083333, 0, 21.309376, 0.083333, 0, 21.309376, 0.083333, 0, 21.309376, 0, 0, 21.309376, 0.083333, 0, 21.309376, 0.083333, 0, 21.309376, 0, 0, 21.309376, 0, 0, 21.309376, 0, 0, 21.210938, 0, 0, 21.309376, 0, 0, 21.309376, 0, 0, 21.210938, 0, 0, 21.210938, 0, 0, 21.210938, 0, 0, 21.159765, 0.002588, 0, 21.210938, 0, 0, 21.210938, 0, 0, 21.159765, 0.002588, 0, 21.159765, 0.002588, 0, 21.159765, 0.002588, 0, 21.117188, 0.010352, 0, 21.159765, 0.002588, 0, 21.159765, 0.002588, 0, 21.117188, 0.010352, 0, 21.117188, 0.010352, 0, 21.117188, 0.010352, 0, 21.083204, 0.023291, 0, 21.117188, 0.010352, 0, 21.117188, 0.010352, 0, 21.083204, 0.023291, 0, 21.083204, 0.023291, 0, 21.083204, 0.023291, 0, 21.057814, 0.041406, 0, 21.083204, 0.023291, 0, 21.083204, 0.023291, 0, 21.057814, 0.041406, 0, 21.057814, 0.041406, 0, 21.057814, 0.041406, 0, 21.039356, 0.066357, 0, 21.057814, 0.041406, 0, 21.057814, 0.041406, 0, 21.039356, 0.066357, 0, 21.039356, 0.066357, 0, 21.039356, 0.066357, 0, 21.026173, 0.099805, 0, 21.039356, 0.066357, 0, 21.039356, 0.066357, 0, 21.026173, 0.099805, 0, 21.026173, 0.099805, 0, 21.026173, 0.099805, 0, 21.018263, 0.141748, 0, 21.026173, 0.099805, 0, 21.026173, 0.099805, 0, 21.018263, 0.141748, 0, 21.018263, 0.141748, 0, 21.018263, 0.141748, 0, 21.015625, 0.192188, 0, 21.018263, 0.141748, 0, 21.018263, 0.141748, 0, 21.015625, 0.192188, 0, 21.015625, 0.192188, 0, 21.015625, 0.192188, 0, 21.015625, 0.500000, 0, 21.015625, 0.192188, 0, 21.015625, 0.192188, 0, 21.015625, 0.500000, 0, 21.015625, 0.500000, 0, 21.015625, 0.500000, 0, 20.945313, 0.500000, 0, 21.015625, 0.500000, 0, 21.015625, 0.500000, 0, 20.945313, 0.500000, 0, 20.945313, 0.500000, 0, 20.945313, 0.500000, 0, 20.945313, 0.583333, 0, 20.945313, 0.500000, 0, 20.945313, 0.500000, 0, 20.945313, 0.583333, 0, 20.945313, 0.583333, 0, 20.945313, 0.583333, 0, 21.015625, 0.583333, 0, 20.945313, 0.583333, 0, 20.945313, 0.583333, 0, 21.015625, 0.583333, 0, 21.015625, 0.583333, 0, 21.015625, 0.583333, 0, 21.015625, 0.750000, 0, 21.015625, 0.583333, 0, 21.015625, 0.583333, 0, 21.015625, 0.750000, 0, 21.015625, 0.750000, 0, 20.945313, 0.583333, 0, 21.015625, 0.500000, 0, 21.015625, 0.583333, 0, 21.015625, 0.500000, 0, 20.945313, 0.583333, 0, 21.015625, 0.583333, 0, 20.945313, 0.583333, 0, 20.945313, 0.500000, 0, 21.015625, 0.500000, 0, 20.945313, 0.500000, 0, 20.945313, 0.583333, 0, 21.015625, 0.500000, 0, 21.015625, 0.583333, 0, 21.015625, 0.500000, 0, 21.111980, 0.500000, 0, 21.015625, 0.500000, 0, 21.015625, 0.583333, 0, 21.111980, 0.500000, 0, 21.015625, 0.583333, 0, 21.111980, 0.500000, 0, 21.111980, 0.583333, 0, 21.111980, 0.500000, 0, 21.015625, 0.583333, 0, 21.111980, 0.583333, 0, 21.015625, 0.750000, 0, 21.015625, 0.583333, 0, 21.111980, 0.583333, 0, 21.015625, 0.583333, 0, 21.015625, 0.750000, 0, 21.111980, 0.583333, 0, 21.015625, 0.750000, 0, 21.111980, 0.583333, 0, 21.111980, 0.750000, 0, 21.111980, 0.583333, 0, 21.015625, 0.750000, 0, 21.111980, 0.750000, 0, 21.111980, 0.583333, 0, 21.111980, 0.500000, 0, 21.309376, 0.500000, 0, 21.111980, 0.500000, 0, 21.111980, 0.583333, 0, 21.309376, 0.500000, 0, 21.111980, 0.583333, 0, 21.309376, 0.500000, 0, 21.309376, 0.583333, 0, 21.309376, 0.500000, 0, 21.111980, 0.583333, 0, 21.309376, 0.583333, 0, 21.015625, 0.500000, 0, 21.111980, 0.192188, 0, 21.111980, 0.500000, 0, 21.111980, 0.192188, 0, 21.015625, 0.500000, 0, 21.111980, 0.500000, 0, 21.015625, 0.500000, 0, 21.015625, 0.192188, 0, 21.111980, 0.192188, 0, 21.015625, 0.192188, 0, 21.015625, 0.500000, 0, 21.111980, 0.192188, 0, 21.111980, 0.192188, 0, 21.015625, 0.192188, 0, 21.113201, 0.160645, 0, 21.015625, 0.192188, 0, 21.111980, 0.192188, 0, 21.113201, 0.160645, 0, 21.015625, 0.192188, 0, 21.018263, 0.141748, 0, 21.113201, 0.160645, 0, 21.018263, 0.141748, 0, 21.015625, 0.192188, 0, 21.113201, 0.160645, 0, 21.113201, 0.160645, 0, 21.018263, 0.141748, 0, 21.116863, 0.135286, 0, 21.018263, 0.141748, 0, 21.113201, 0.160645, 0, 21.116863, 0.135286, 0, 21.018263, 0.141748, 0, 21.026173, 0.099805, 0, 21.116863, 0.135286, 0, 21.026173, 0.099805, 0, 21.018263, 0.141748, 0, 21.116863, 0.135286, 0, 21.116863, 0.135286, 0, 21.026173, 0.099805, 0, 21.122967, 0.116113, 0, 21.026173, 0.099805, 0, 21.116863, 0.135286, 0, 21.122967, 0.116113, 0, 21.122967, 0.116113, 0, 21.026173, 0.099805, 0, 21.039356, 0.066357, 0, 21.026173, 0.099805, 0, 21.122967, 0.116113, 0, 21.039356, 0.066357, 0, 21.131512, 0.103125, 0, 21.122967, 0.116113, 0, 21.039356, 0.066357, 0, 21.122967, 0.116113, 0, 21.131512, 0.103125, 0, 21.039356, 0.066357, 0, 21.039356, 0.066357, 0, 21.057814, 0.041406, 0, 21.131512, 0.103125, 0, 21.057814, 0.041406, 0, 21.039356, 0.066357, 0, 21.131512, 0.103125, 0, 21.131512, 0.103125, 0, 21.057814, 0.041406, 0, 21.083204, 0.023291, 0, 21.057814, 0.041406, 0, 21.131512, 0.103125, 0, 21.083204, 0.023291, 0, 21.143799, 0.094466, 0, 21.131512, 0.103125, 0, 21.083204, 0.023291, 0, 21.131512, 0.103125, 0, 21.143799, 0.094466, 0, 21.083204, 0.023291, 0, 21.083204, 0.023291, 0, 21.117188, 0.010352, 0, 21.143799, 0.094466, 0, 21.117188, 0.010352, 0, 21.083204, 0.023291, 0, 21.143799, 0.094466, 0, 21.143799, 0.094466, 0, 21.117188, 0.010352, 0, 21.161133, 0.088281, 0, 21.117188, 0.010352, 0, 21.143799, 0.094466, 0, 21.161133, 0.088281, 0, 21.117188, 0.010352, 0, 21.159765, 0.002588, 0, 21.161133, 0.088281, 0, 21.159765, 0.002588, 0, 21.117188, 0.010352, 0, 21.161133, 0.088281, 0, 21.161133, 0.088281, 0, 21.159765, 0.002588, 0, 21.183514, 0.084570, 0, 21.159765, 0.002588, 0, 21.161133, 0.088281, 0, 21.183514, 0.084570, 0, 21.183514, 0.084570, 0, 21.159765, 0.002588, 0, 21.210938, 0, 0, 21.159765, 0.002588, 0, 21.183514, 0.084570, 0, 21.210938, 0, 0, 21.210938, 0.083333, 0, 21.183514, 0.084570, 0, 21.210938, 0, 0, 21.183514, 0.084570, 0, 21.210938, 0.083333, 0, 21.210938, 0, 0, 21.210938, 0.083333, 0, 21.210938, 0, 0, 21.309376, 0, 0, 21.210938, 0, 0, 21.210938, 0.083333, 0, 21.309376, 0, 0, 21.210938, 0.083333, 0, 21.309376, 0, 0, 21.309376, 0.083333, 0, 21.309376, 0, 0, 21.210938, 0.083333, 0, 21.309376, 0.083333, 0, 21.447395, 0.133333, 0, 21.557293, 0.133333, 0, 21.447395, 0.133333, 0, 21.447395, 0.133333, 0, 21.557293, 0.133333, 0, 21.557293, 0.133333, 0, 21.557293, 0.133333, 0, 21.557293, 0, 0, 21.557293, 0.133333, 0, 21.557293, 0.133333, 0, 21.557293, 0, 0, 21.557293, 0, 0, 21.557293, 0, 0, 21.447395, 0, 0, 21.557293, 0, 0, 21.557293, 0, 0, 21.447395, 0, 0, 21.447395, 0, 0, 21.447395, 0.133333, 0, 21.557293, 0, 0, 21.557293, 0.133333, 0, 21.557293, 0, 0, 21.447395, 0.133333, 0, 21.557293, 0.133333, 0, 21.447395, 0.133333, 0, 21.447395, 0, 0, 21.557293, 0, 0, 21.447395, 0, 0, 21.447395, 0.133333, 0, 21.557293, 0, 0, 21.780729, 0.133333, 0, 21.890625, 0.133333, 0, 21.780729, 0.133333, 0, 21.780729, 0.133333, 0, 21.890625, 0.133333, 0, 21.890625, 0.133333, 0, 21.890625, 0.133333, 0, 21.890625, 0, 0, 21.890625, 0.133333, 0, 21.890625, 0.133333, 0, 21.890625, 0, 0, 21.890625, 0, 0, 21.890625, 0, 0, 21.780729, 0, 0, 21.890625, 0, 0, 21.890625, 0, 0, 21.780729, 0, 0, 21.780729, 0, 0, 21.780729, 0.133333, 0, 21.890625, 0, 0, 21.890625, 0.133333, 0, 21.890625, 0, 0, 21.780729, 0.133333, 0, 21.890625, 0.133333, 0, 21.780729, 0.133333, 0, 21.780729, 0, 0, 21.890625, 0, 0, 21.780729, 0, 0, 21.780729, 0.133333, 0, 21.890625, 0, 0, 22.114063, 0.133333, 0, 22.223959, 0.133333, 0, 22.114063, 0.133333, 0, 22.114063, 0.133333, 0, 22.223959, 0.133333, 0, 22.223959, 0.133333, 0, 22.223959, 0.133333, 0, 22.223959, 0, 0, 22.223959, 0.133333, 0, 22.223959, 0.133333, 0, 22.223959, 0, 0, 22.223959, 0, 0, 22.223959, 0, 0, 22.114063, 0, 0, 22.223959, 0, 0, 22.223959, 0, 0, 22.114063, 0, 0, 22.114063, 0, 0, 22.114063, 0.133333, 0, 22.223959, 0, 0, 22.223959, 0.133333, 0, 22.223959, 0, 0, 22.114063, 0.133333, 0, 22.223959, 0.133333, 0, 22.114063, 0.133333, 0, 22.114063, 0, 0, 22.223959, 0, 0, 22.114063, 0, 0, 22.114063, 0.133333, 0, 22.223959, 0, 0, 0.634896, -0.893490, 0, 0.634896, -0.683333, 0, 0.634896, -0.893490, 0, 0.634896, -0.893490, 0, 0.634896, -0.683333, 0, 0.634896, -0.683333, 0, 0.634896, -0.683333, 0, 0.463021, -0.683333, 0, 0.634896, -0.683333, 0, 0.634896, -0.683333, 0, 0.463021, -0.683333, 0, 0.463021, -0.683333, 0, 0.463021, -0.683333, 0, 0.463021, -0.600000, 0, 0.463021, -0.683333, 0, 0.463021, -0.683333, 0, 0.463021, -0.600000, 0, 0.463021, -0.600000, 0, 0.463021, -0.600000, 0, 0.739062, -0.600000, 0, 0.463021, -0.600000, 0, 0.463021, -0.600000, 0, 0.739062, -0.600000, 0, 0.739062, -0.600000, 0, 0.739062, -0.600000, 0, 0.739062, -0.932292, 0, 0.739062, -0.600000, 0, 0.739062, -0.600000, 0, 0.739062, -0.932292, 0, 0.739062, -0.932292, 0, 0.739062, -0.932292, 0, 0.707812, -0.951855, 0, 0.739062, -0.932292, 0, 0.739062, -0.932292, 0, 0.707812, -0.951855, 0, 0.707812, -0.951855, 0, 0.707812, -0.951855, 0, 0.675000, -0.968880, 0, 0.707812, -0.951855, 0, 0.707812, -0.951855, 0, 0.675000, -0.968880, 0, 0.675000, -0.968880, 0, 0.675000, -0.968880, 0, 0.640625, -0.983366, 0, 0.675000, -0.968880, 0, 0.675000, -0.968880, 0, 0.640625, -0.983366, 0, 0.640625, -0.983366, 0, 0.640625, -0.983366, 0, 0.604688, -0.995313, 0, 0.640625, -0.983366, 0, 0.640625, -0.983366, 0, 0.604688, -0.995313, 0, 0.604688, -0.995313, 0, 0.604688, -0.995313, 0, 0.567350, -1.004655, 0, 0.604688, -0.995313, 0, 0.604688, -0.995313, 0, 0.567350, -1.004655, 0, 0.567350, -1.004655, 0, 0.567350, -1.004655, 0, 0.528776, -1.011328, 0, 0.567350, -1.004655, 0, 0.567350, -1.004655, 0, 0.528776, -1.011328, 0, 0.528776, -1.011328, 0, 0.528776, -1.011328, 0, 0.488965, -1.015332, 0, 0.528776, -1.011328, 0, 0.528776, -1.011328, 0, 0.488965, -1.015332, 0, 0.488965, -1.015332, 0, 0.488965, -1.015332, 0, 0.447917, -1.016667, 0, 0.488965, -1.015332, 0, 0.488965, -1.015332, 0, 0.447917, -1.016667, 0, 0.447917, -1.016667, 0, 0.447917, -1.016667, 0, 0.361735, -1.009928, 0, 0.447917, -1.016667, 0, 0.447917, -1.016667, 0, 0.361735, -1.009928, 0, 0.361735, -1.009928, 0, 0.361735, -1.009928, 0, 0.285482, -0.989714, 0, 0.361735, -1.009928, 0, 0.361735, -1.009928, 0, 0.285482, -0.989714, 0, 0.285482, -0.989714, 0, 0.285482, -0.989714, 0, 0.219157, -0.956022, 0, 0.285482, -0.989714, 0, 0.285482, -0.989714, 0, 0.219157, -0.956022, 0, 0.219157, -0.956022, 0, 0.219157, -0.956022, 0, 0.162760, -0.908854, 0, 0.219157, -0.956022, 0, 0.219157, -0.956022, 0, 0.162760, -0.908854, 0, 0.162760, -0.908854, 0, 0.162760, -0.908854, 0, 0.117757, -0.849658, 0, 0.162760, -0.908854, 0, 0.162760, -0.908854, 0, 0.117757, -0.849658, 0, 0.117757, -0.849658, 0, 0.117757, -0.849658, 0, 0.085612, -0.779883, 0, 0.117757, -0.849658, 0, 0.117757, -0.849658, 0, 0.085612, -0.779883, 0, 0.085612, -0.779883, 0, 0.085612, -0.779883, 0, 0.066325, -0.699528, 0, 0.085612, -0.779883, 0, 0.085612, -0.779883, 0, 0.066325, -0.699528, 0, 0.066325, -0.699528, 0, 0.066325, -0.699528, 0, 0.059896, -0.608594, 0, 0.066325, -0.699528, 0, 0.066325, -0.699528, 0, 0.059896, -0.608594, 0, 0.059896, -0.608594, 0, 0.059896, -0.608594, 0, 0.066325, -0.517432, 0, 0.059896, -0.608594, 0, 0.059896, -0.608594, 0, 0.066325, -0.517432, 0, 0.066325, -0.517432, 0, 0.066325, -0.517432, 0, 0.085612, -0.436914, 0, 0.066325, -0.517432, 0, 0.066325, -0.517432, 0, 0.085612, -0.436914, 0, 0.085612, -0.436914, 0, 0.085612, -0.436914, 0, 0.117757, -0.367041, 0, 0.085612, -0.436914, 0, 0.085612, -0.436914, 0, 0.117757, -0.367041, 0, 0.117757, -0.367041, 0, 0.117757, -0.367041, 0, 0.162760, -0.307813, 0, 0.117757, -0.367041, 0, 0.117757, -0.367041, 0, 0.162760, -0.307813, 0, 0.162760, -0.307813, 0, 0.162760, -0.307813, 0, 0.219157, -0.260644, 0, 0.162760, -0.307813, 0, 0.162760, -0.307813, 0, 0.219157, -0.260644, 0, 0.219157, -0.260644, 0, 0.219157, -0.260644, 0, 0.285482, -0.226953, 0, 0.219157, -0.260644, 0, 0.219157, -0.260644, 0, 0.285482, -0.226953, 0, 0.285482, -0.226953, 0, 0.285482, -0.226953, 0, 0.361735, -0.206738, 0, 0.285482, -0.226953, 0, 0.285482, -0.226953, 0, 0.361735, -0.206738, 0, 0.361735, -0.206738, 0, 0.361735, -0.206738, 0, 0.447917, -0.200000, 0, 0.361735, -0.206738, 0, 0.361735, -0.206738, 0, 0.447917, -0.200000, 0, 0.447917, -0.200000, 0, 0.447917, -0.200000, 0, 0.485465, -0.201058, 0, 0.447917, -0.200000, 0, 0.447917, -0.200000, 0, 0.485465, -0.201058, 0, 0.485465, -0.201058, 0, 0.485465, -0.201058, 0, 0.522070, -0.204232, 0, 0.485465, -0.201058, 0, 0.485465, -0.201058, 0, 0.522070, -0.204232, 0, 0.522070, -0.204232, 0, 0.522070, -0.204232, 0, 0.557731, -0.209521, 0, 0.522070, -0.204232, 0, 0.522070, -0.204232, 0, 0.557731, -0.209521, 0, 0.557731, -0.209521, 0, 0.557731, -0.209521, 0, 0.592448, -0.216927, 0, 0.557731, -0.209521, 0, 0.557731, -0.209521, 0, 0.592448, -0.216927, 0, 0.592448, -0.216927, 0, 0.592448, -0.216927, 0, 0.626025, -0.226383, 0, 0.592448, -0.216927, 0, 0.592448, -0.216927, 0, 0.626025, -0.226383, 0, 0.626025, -0.226383, 0, 0.626025, -0.226383, 0, 0.658268, -0.237826, 0, 0.626025, -0.226383, 0, 0.626025, -0.226383, 0, 0.658268, -0.237826, 0, 0.658268, -0.237826, 0, 0.658268, -0.237826, 0, 0.689176, -0.251253, 0, 0.658268, -0.237826, 0, 0.658268, -0.237826, 0, 0.689176, -0.251253, 0, 0.689176, -0.251253, 0, 0.689176, -0.251253, 0, 0.718750, -0.266667, 0, 0.689176, -0.251253, 0, 0.689176, -0.251253, 0, 0.718750, -0.266667, 0, 0.718750, -0.266667, 0, 0.718750, -0.266667, 0, 0.718750, -0.383333, 0, 0.718750, -0.266667, 0, 0.718750, -0.266667, 0, 0.718750, -0.383333, 0, 0.718750, -0.383333, 0, 0.718750, -0.383333, 0, 0.689128, -0.360002, 0, 0.718750, -0.383333, 0, 0.718750, -0.383333, 0, 0.689128, -0.360002, 0, 0.689128, -0.360002, 0, 0.689128, -0.360002, 0, 0.658594, -0.339746, 0, 0.689128, -0.360002, 0, 0.689128, -0.360002, 0, 0.658594, -0.339746, 0, 0.658594, -0.339746, 0, 0.658594, -0.339746, 0, 0.627148, -0.322567, 0, 0.658594, -0.339746, 0, 0.658594, -0.339746, 0, 0.627148, -0.322567, 0, 0.627148, -0.322567, 0, 0.627148, -0.322567, 0, 0.594792, -0.308464, 0, 0.627148, -0.322567, 0, 0.627148, -0.322567, 0, 0.594792, -0.308464, 0, 0.594792, -0.308464, 0, 0.594792, -0.308464, 0, 0.561556, -0.297469, 0, 0.594792, -0.308464, 0, 0.594792, -0.308464, 0, 0.561556, -0.297469, 0, 0.561556, -0.297469, 0, 0.561556, -0.297469, 0, 0.527474, -0.289616, 0, 0.561556, -0.297469, 0, 0.561556, -0.297469, 0, 0.527474, -0.289616, 0, 0.527474, -0.289616, 0, 0.527474, -0.289616, 0, 0.492546, -0.284904, 0, 0.527474, -0.289616, 0, 0.527474, -0.289616, 0, 0.492546, -0.284904, 0, 0.492546, -0.284904, 0, 0.492546, -0.284904, 0, 0.456771, -0.283333, 0, 0.492546, -0.284904, 0, 0.492546, -0.284904, 0, 0.456771, -0.283333, 0, 0.456771, -0.283333, 0, 0.456771, -0.283333, 0, 0.389860, -0.288444, 0, 0.456771, -0.283333, 0, 0.456771, -0.283333, 0, 0.389860, -0.288444, 0, 0.389860, -0.288444, 0, 0.389860, -0.288444, 0, 0.331836, -0.303776, 0, 0.389860, -0.288444, 0, 0.389860, -0.288444, 0, 0.331836, -0.303776, 0, 0.331836, -0.303776, 0, 0.331836, -0.303776, 0, 0.282699, -0.329329, 0, 0.331836, -0.303776, 0, 0.331836, -0.303776, 0, 0.282699, -0.329329, 0, 0.282699, -0.329329, 0, 0.282699, -0.329329, 0, 0.242448, -0.365104, 0, 0.282699, -0.329329, 0, 0.282699, -0.329329, 0, 0.242448, -0.365104, 0, 0.242448, -0.365104, 0, 0.242448, -0.365104, 0, 0.211117, -0.410986, 0, 0.242448, -0.365104, 0, 0.242448, -0.365104, 0, 0.211117, -0.410986, 0, 0.211117, -0.410986, 0, 0.211117, -0.410986, 0, 0.188737, -0.466862, 0, 0.211117, -0.410986, 0, 0.211117, -0.410986, 0, 0.188737, -0.466862, 0, 0.188737, -0.466862, 0, 0.188737, -0.466862, 0, 0.175309, -0.532731, 0, 0.188737, -0.466862, 0, 0.188737, -0.466862, 0, 0.175309, -0.532731, 0, 0.175309, -0.532731, 0, 0.175309, -0.532731, 0, 0.170833, -0.608594, 0, 0.175309, -0.532731, 0, 0.175309, -0.532731, 0, 0.170833, -0.608594, 0, 0.170833, -0.608594, 0, 0.170833, -0.608594, 0, 0.175309, -0.684229, 0, 0.170833, -0.608594, 0, 0.170833, -0.608594, 0, 0.175309, -0.684229, 0, 0.175309, -0.684229, 0, 0.175309, -0.684229, 0, 0.188737, -0.749935, 0, 0.175309, -0.684229, 0, 0.175309, -0.684229, 0, 0.188737, -0.749935, 0, 0.188737, -0.749935, 0, 0.188737, -0.749935, 0, 0.211117, -0.805713, 0, 0.188737, -0.749935, 0, 0.188737, -0.749935, 0, 0.211117, -0.805713, 0, 0.211117, -0.805713, 0, 0.211117, -0.805713, 0, 0.242448, -0.851563, 0, 0.211117, -0.805713, 0, 0.211117, -0.805713, 0, 0.242448, -0.851563, 0, 0.242448, -0.851563, 0, 0.242448, -0.851563, 0, 0.282699, -0.887337, 0, 0.242448, -0.851563, 0, 0.242448, -0.851563, 0, 0.282699, -0.887337, 0, 0.282699, -0.887337, 0, 0.282699, -0.887337, 0, 0.331836, -0.912891, 0, 0.282699, -0.887337, 0, 0.282699, -0.887337, 0, 0.331836, -0.912891, 0, 0.331836, -0.912891, 0, 0.331836, -0.912891, 0, 0.389860, -0.928223, 0, 0.331836, -0.912891, 0, 0.331836, -0.912891, 0, 0.389860, -0.928223, 0, 0.389860, -0.928223, 0, 0.389860, -0.928223, 0, 0.456771, -0.933333, 0, 0.389860, -0.928223, 0, 0.389860, -0.928223, 0, 0.456771, -0.933333, 0, 0.456771, -0.933333, 0, 0.456771, -0.933333, 0, 0.483887, -0.932731, 0, 0.456771, -0.933333, 0, 0.456771, -0.933333, 0, 0.483887, -0.932731, 0, 0.483887, -0.932731, 0, 0.483887, -0.932731, 0, 0.509505, -0.930924, 0, 0.483887, -0.932731, 0, 0.483887, -0.932731, 0, 0.509505, -0.930924, 0, 0.509505, -0.930924, 0, 0.509505, -0.930924, 0, 0.533626, -0.927913, 0, 0.509505, -0.930924, 0, 0.509505, -0.930924, 0, 0.533626, -0.927913, 0, 0.533626, -0.927913, 0, 0.533626, -0.927913, 0, 0.556250, -0.923698, 0, 0.533626, -0.927913, 0, 0.533626, -0.927913, 0, 0.556250, -0.923698, 0, 0.556250, -0.923698, 0, 0.556250, -0.923698, 0, 0.577572, -0.918197, 0, 0.556250, -0.923698, 0, 0.556250, -0.923698, 0, 0.577572, -0.918197, 0, 0.577572, -0.918197, 0, 0.577572, -0.918197, 0, 0.597786, -0.911328, 0, 0.577572, -0.918197, 0, 0.577572, -0.918197, 0, 0.597786, -0.911328, 0, 0.597786, -0.911328, 0, 0.597786, -0.911328, 0, 0.616895, -0.903092, 0, 0.597786, -0.911328, 0, 0.597786, -0.911328, 0, 0.616895, -0.903092, 0, 0.616895, -0.903092, 0, 0.175309, -0.532731, 0, 0.066325, -0.517432, 0, 0.059896, -0.608594, 0, 0.066325, -0.517432, 0, 0.175309, -0.532731, 0, 0.059896, -0.608594, 0, 0.085612, -0.436914, 0, 0.066325, -0.517432, 0, 0.175309, -0.532731, 0, 0.066325, -0.517432, 0, 0.085612, -0.436914, 0, 0.175309, -0.532731, 0, 0.085612, -0.436914, 0, 0.175309, -0.532731, 0, 0.188737, -0.466862, 0, 0.175309, -0.532731, 0, 0.085612, -0.436914, 0, 0.188737, -0.466862, 0, 0.117757, -0.367041, 0, 0.085612, -0.436914, 0, 0.188737, -0.466862, 0, 0.085612, -0.436914, 0, 0.117757, -0.367041, 0, 0.188737, -0.466862, 0, 0.117757, -0.367041, 0, 0.188737, -0.466862, 0, 0.211117, -0.410986, 0, 0.188737, -0.466862, 0, 0.117757, -0.367041, 0, 0.211117, -0.410986, 0, 0.162760, -0.307813, 0, 0.117757, -0.367041, 0, 0.211117, -0.410986, 0, 0.117757, -0.367041, 0, 0.162760, -0.307813, 0, 0.211117, -0.410986, 0, 0.162760, -0.307813, 0, 0.211117, -0.410986, 0, 0.242448, -0.365104, 0, 0.211117, -0.410986, 0, 0.162760, -0.307813, 0, 0.242448, -0.365104, 0, 0.219157, -0.260644, 0, 0.162760, -0.307813, 0, 0.242448, -0.365104, 0, 0.162760, -0.307813, 0, 0.219157, -0.260644, 0, 0.242448, -0.365104, 0, 0.219157, -0.260644, 0, 0.242448, -0.365104, 0, 0.282699, -0.329329, 0, 0.242448, -0.365104, 0, 0.219157, -0.260644, 0, 0.282699, -0.329329, 0, 0.219157, -0.260644, 0, 0.282699, -0.329329, 0, 0.285482, -0.226953, 0, 0.282699, -0.329329, 0, 0.219157, -0.260644, 0, 0.285482, -0.226953, 0, 0.285482, -0.226953, 0, 0.282699, -0.329329, 0, 0.331836, -0.303776, 0, 0.282699, -0.329329, 0, 0.285482, -0.226953, 0, 0.331836, -0.303776, 0, 0.285482, -0.226953, 0, 0.331836, -0.303776, 0, 0.361735, -0.206738, 0, 0.331836, -0.303776, 0, 0.285482, -0.226953, 0, 0.361735, -0.206738, 0, 0.361735, -0.206738, 0, 0.331836, -0.303776, 0, 0.389860, -0.288444, 0, 0.331836, -0.303776, 0, 0.361735, -0.206738, 0, 0.389860, -0.288444, 0, 0.447917, -0.200000, 0, 0.361735, -0.206738, 0, 0.389860, -0.288444, 0, 0.361735, -0.206738, 0, 0.447917, -0.200000, 0, 0.389860, -0.288444, 0, 0.389860, -0.288444, 0, 0.456771, -0.283333, 0, 0.447917, -0.200000, 0, 0.456771, -0.283333, 0, 0.389860, -0.288444, 0, 0.447917, -0.200000, 0, 0.485465, -0.201058, 0, 0.447917, -0.200000, 0, 0.456771, -0.283333, 0, 0.447917, -0.200000, 0, 0.485465, -0.201058, 0, 0.456771, -0.283333, 0, 0.485465, -0.201058, 0, 0.456771, -0.283333, 0, 0.492546, -0.284904, 0, 0.456771, -0.283333, 0, 0.485465, -0.201058, 0, 0.492546, -0.284904, 0, 0.485465, -0.201058, 0, 0.492546, -0.284904, 0, 0.522070, -0.204232, 0, 0.492546, -0.284904, 0, 0.485465, -0.201058, 0, 0.522070, -0.204232, 0, 0.522070, -0.204232, 0, 0.492546, -0.284904, 0, 0.527474, -0.289616, 0, 0.492546, -0.284904, 0, 0.522070, -0.204232, 0, 0.527474, -0.289616, 0, 0.522070, -0.204232, 0, 0.527474, -0.289616, 0, 0.557731, -0.209521, 0, 0.527474, -0.289616, 0, 0.522070, -0.204232, 0, 0.557731, -0.209521, 0, 0.557731, -0.209521, 0, 0.527474, -0.289616, 0, 0.561556, -0.297469, 0, 0.527474, -0.289616, 0, 0.557731, -0.209521, 0, 0.561556, -0.297469, 0, 0.557731, -0.209521, 0, 0.561556, -0.297469, 0, 0.592448, -0.216927, 0, 0.561556, -0.297469, 0, 0.557731, -0.209521, 0, 0.592448, -0.216927, 0, 0.592448, -0.216927, 0, 0.561556, -0.297469, 0, 0.594792, -0.308464, 0, 0.561556, -0.297469, 0, 0.592448, -0.216927, 0, 0.594792, -0.308464, 0, 0.592448, -0.216927, 0, 0.594792, -0.308464, 0, 0.626025, -0.226383, 0, 0.594792, -0.308464, 0, 0.592448, -0.216927, 0, 0.626025, -0.226383, 0, 0.626025, -0.226383, 0, 0.594792, -0.308464, 0, 0.658268, -0.237826, 0, 0.594792, -0.308464, 0, 0.626025, -0.226383, 0, 0.658268, -0.237826, 0, 0.594792, -0.308464, 0, 0.627148, -0.322567, 0, 0.658268, -0.237826, 0, 0.627148, -0.322567, 0, 0.594792, -0.308464, 0, 0.658268, -0.237826, 0, 0.658268, -0.237826, 0, 0.627148, -0.322567, 0, 0.689176, -0.251253, 0, 0.627148, -0.322567, 0, 0.658268, -0.237826, 0, 0.689176, -0.251253, 0, 0.627148, -0.322567, 0, 0.658594, -0.339746, 0, 0.689176, -0.251253, 0, 0.658594, -0.339746, 0, 0.627148, -0.322567, 0, 0.689176, -0.251253, 0, 0.689176, -0.251253, 0, 0.658594, -0.339746, 0, 0.718750, -0.266667, 0, 0.658594, -0.339746, 0, 0.689176, -0.251253, 0, 0.718750, -0.266667, 0, 0.658594, -0.339746, 0, 0.689128, -0.360002, 0, 0.718750, -0.266667, 0, 0.689128, -0.360002, 0, 0.658594, -0.339746, 0, 0.718750, -0.266667, 0, 0.689128, -0.360002, 0, 0.718750, -0.383333, 0, 0.718750, -0.266667, 0, 0.718750, -0.383333, 0, 0.689128, -0.360002, 0, 0.718750, -0.266667, 0, 0.175309, -0.532731, 0, 0.059896, -0.608594, 0, 0.170833, -0.608594, 0, 0.059896, -0.608594, 0, 0.175309, -0.532731, 0, 0.170833, -0.608594, 0, 0.170833, -0.608594, 0, 0.059896, -0.608594, 0, 0.175309, -0.684229, 0, 0.059896, -0.608594, 0, 0.170833, -0.608594, 0, 0.175309, -0.684229, 0, 0.059896, -0.608594, 0, 0.066325, -0.699528, 0, 0.175309, -0.684229, 0, 0.066325, -0.699528, 0, 0.059896, -0.608594, 0, 0.175309, -0.684229, 0, 0.175309, -0.684229, 0, 0.066325, -0.699528, 0, 0.085612, -0.779883, 0, 0.066325, -0.699528, 0, 0.175309, -0.684229, 0, 0.085612, -0.779883, 0, 0.188737, -0.749935, 0, 0.175309, -0.684229, 0, 0.085612, -0.779883, 0, 0.175309, -0.684229, 0, 0.188737, -0.749935, 0, 0.085612, -0.779883, 0, 0.188737, -0.749935, 0, 0.085612, -0.779883, 0, 0.117757, -0.849658, 0, 0.085612, -0.779883, 0, 0.188737, -0.749935, 0, 0.117757, -0.849658, 0, 0.211117, -0.805713, 0, 0.188737, -0.749935, 0, 0.117757, -0.849658, 0, 0.188737, -0.749935, 0, 0.211117, -0.805713, 0, 0.117757, -0.849658, 0, 0.117757, -0.849658, 0, 0.162760, -0.908854, 0, 0.211117, -0.805713, 0, 0.162760, -0.908854, 0, 0.117757, -0.849658, 0, 0.211117, -0.805713, 0, 0.211117, -0.805713, 0, 0.162760, -0.908854, 0, 0.242448, -0.851563, 0, 0.162760, -0.908854, 0, 0.211117, -0.805713, 0, 0.242448, -0.851563, 0, 0.162760, -0.908854, 0, 0.219157, -0.956022, 0, 0.242448, -0.851563, 0, 0.219157, -0.956022, 0, 0.162760, -0.908854, 0, 0.242448, -0.851563, 0, 0.242448, -0.851563, 0, 0.219157, -0.956022, 0, 0.282699, -0.887337, 0, 0.219157, -0.956022, 0, 0.242448, -0.851563, 0, 0.282699, -0.887337, 0, 0.282699, -0.887337, 0, 0.219157, -0.956022, 0, 0.285482, -0.989714, 0, 0.219157, -0.956022, 0, 0.282699, -0.887337, 0, 0.285482, -0.989714, 0, 0.282699, -0.887337, 0, 0.285482, -0.989714, 0, 0.331836, -0.912891, 0, 0.285482, -0.989714, 0, 0.282699, -0.887337, 0, 0.331836, -0.912891, 0, 0.331836, -0.912891, 0, 0.285482, -0.989714, 0, 0.361735, -1.009928, 0, 0.285482, -0.989714, 0, 0.331836, -0.912891, 0, 0.361735, -1.009928, 0, 0.389860, -0.928223, 0, 0.331836, -0.912891, 0, 0.361735, -1.009928, 0, 0.331836, -0.912891, 0, 0.389860, -0.928223, 0, 0.361735, -1.009928, 0, 0.389860, -0.928223, 0, 0.361735, -1.009928, 0, 0.447917, -1.016667, 0, 0.361735, -1.009928, 0, 0.389860, -0.928223, 0, 0.447917, -1.016667, 0, 0.456771, -0.933333, 0, 0.389860, -0.928223, 0, 0.447917, -1.016667, 0, 0.389860, -0.928223, 0, 0.456771, -0.933333, 0, 0.447917, -1.016667, 0, 0.456771, -0.933333, 0, 0.447917, -1.016667, 0, 0.488965, -1.015332, 0, 0.447917, -1.016667, 0, 0.456771, -0.933333, 0, 0.488965, -1.015332, 0, 0.483887, -0.932731, 0, 0.456771, -0.933333, 0, 0.488965, -1.015332, 0, 0.456771, -0.933333, 0, 0.483887, -0.932731, 0, 0.488965, -1.015332, 0, 0.509505, -0.930924, 0, 0.483887, -0.932731, 0, 0.488965, -1.015332, 0, 0.483887, -0.932731, 0, 0.509505, -0.930924, 0, 0.488965, -1.015332, 0, 0.528776, -1.011328, 0, 0.509505, -0.930924, 0, 0.488965, -1.015332, 0, 0.509505, -0.930924, 0, 0.528776, -1.011328, 0, 0.488965, -1.015332, 0, 0.509505, -0.930924, 0, 0.528776, -1.011328, 0, 0.533626, -0.927913, 0, 0.528776, -1.011328, 0, 0.509505, -0.930924, 0, 0.533626, -0.927913, 0, 0.533626, -0.927913, 0, 0.528776, -1.011328, 0, 0.567350, -1.004655, 0, 0.528776, -1.011328, 0, 0.533626, -0.927913, 0, 0.567350, -1.004655, 0, 0.556250, -0.923698, 0, 0.533626, -0.927913, 0, 0.567350, -1.004655, 0, 0.533626, -0.927913, 0, 0.556250, -0.923698, 0, 0.567350, -1.004655, 0, 0.556250, -0.923698, 0, 0.567350, -1.004655, 0, 0.604688, -0.995313, 0, 0.567350, -1.004655, 0, 0.556250, -0.923698, 0, 0.604688, -0.995313, 0, 0.577572, -0.918197, 0, 0.556250, -0.923698, 0, 0.604688, -0.995313, 0, 0.556250, -0.923698, 0, 0.577572, -0.918197, 0, 0.604688, -0.995313, 0, 0.597786, -0.911328, 0, 0.577572, -0.918197, 0, 0.604688, -0.995313, 0, 0.577572, -0.918197, 0, 0.597786, -0.911328, 0, 0.604688, -0.995313, 0, 0.640625, -0.983366, 0, 0.597786, -0.911328, 0, 0.604688, -0.995313, 0, 0.597786, -0.911328, 0, 0.640625, -0.983366, 0, 0.604688, -0.995313, 0, 0.616895, -0.903092, 0, 0.597786, -0.911328, 0, 0.640625, -0.983366, 0, 0.597786, -0.911328, 0, 0.616895, -0.903092, 0, 0.640625, -0.983366, 0, 0.616895, -0.903092, 0, 0.640625, -0.983366, 0, 0.675000, -0.968880, 0, 0.640625, -0.983366, 0, 0.616895, -0.903092, 0, 0.675000, -0.968880, 0, 0.634896, -0.893490, 0, 0.616895, -0.903092, 0, 0.675000, -0.968880, 0, 0.616895, -0.903092, 0, 0.634896, -0.893490, 0, 0.675000, -0.968880, 0, 0.634896, -0.893490, 0, 0.675000, -0.968880, 0, 0.707812, -0.951855, 0, 0.675000, -0.968880, 0, 0.634896, -0.893490, 0, 0.707812, -0.951855, 0, 0.739062, -0.932292, 0, 0.634896, -0.893490, 0, 0.707812, -0.951855, 0, 0.634896, -0.893490, 0, 0.739062, -0.932292, 0, 0.707812, -0.951855, 0, 0.739062, -0.932292, 0, 0.634896, -0.683333, 0, 0.634896, -0.893490, 0, 0.634896, -0.683333, 0, 0.739062, -0.932292, 0, 0.634896, -0.893490, 0, 0.634896, -0.683333, 0, 0.739062, -0.932292, 0, 0.739062, -0.600000, 0, 0.739062, -0.932292, 0, 0.634896, -0.683333, 0, 0.739062, -0.600000, 0, 0.463021, -0.600000, 0, 0.634896, -0.683333, 0, 0.739062, -0.600000, 0, 0.634896, -0.683333, 0, 0.463021, -0.600000, 0, 0.739062, -0.600000, 0, 0.463021, -0.600000, 0, 0.463021, -0.683333, 0, 0.634896, -0.683333, 0, 0.463021, -0.683333, 0, 0.463021, -0.600000, 0, 0.634896, -0.683333, 0, 1.432813, -0.681250, 0, 1.432813, -0.733333, 0, 1.432813, -0.681250, 0, 1.432813, -0.681250, 0, 1.432813, -0.733333, 0, 1.432813, -0.733333, 0, 1.432813, -0.733333, 0, 0.992188, -0.733333, 0, 1.432813, -0.733333, 0, 1.432813, -0.733333, 0, 0.992188, -0.733333, 0, 0.992188, -0.733333, 0, 0.992188, -0.733333, 0, 0.998259, -0.779256, 0, 0.992188, -0.733333, 0, 0.992188, -0.733333, 0, 0.998259, -0.779256, 0, 0.998259, -0.779256, 0, 0.998259, -0.779256, 0, 1.010221, -0.819368, 0, 0.998259, -0.779256, 0, 0.998259, -0.779256, 0, 1.010221, -0.819368, 0, 1.010221, -0.819368, 0, 1.010221, -0.819368, 0, 1.028076, -0.853670, 0, 1.010221, -0.819368, 0, 1.010221, -0.819368, 0, 1.028076, -0.853670, 0, 1.028076, -0.853670, 0, 1.028076, -0.853670, 0, 1.051823, -0.882161, 0, 1.028076, -0.853670, 0, 1.028076, -0.853670, 0, 1.051823, -0.882161, 0, 1.051823, -0.882161, 0, 1.051823, -0.882161, 0, 1.081136, -0.904549, 0, 1.051823, -0.882161, 0, 1.051823, -0.882161, 0, 1.081136, -0.904549, 0, 1.081136, -0.904549, 0, 1.081136, -0.904549, 0, 1.115690, -0.920540, 0, 1.081136, -0.904549, 0, 1.081136, -0.904549, 0, 1.115690, -0.920540, 0, 1.115690, -0.920540, 0, 1.115690, -0.920540, 0, 1.155485, -0.930135, 0, 1.115690, -0.920540, 0, 1.115690, -0.920540, 0, 1.155485, -0.930135, 0, 1.155485, -0.930135, 0, 1.155485, -0.930135, 0, 1.200521, -0.933333, 0, 1.155485, -0.930135, 0, 1.155485, -0.930135, 0, 1.200521, -0.933333, 0, 1.200521, -0.933333, 0, 1.200521, -0.933333, 0, 1.227913, -0.932552, 0, 1.200521, -0.933333, 0, 1.200521, -0.933333, 0, 1.227913, -0.932552, 0, 1.227913, -0.932552, 0, 1.227913, -0.932552, 0, 1.254883, -0.930208, 0, 1.227913, -0.932552, 0, 1.227913, -0.932552, 0, 1.254883, -0.930208, 0, 1.254883, -0.930208, 0, 1.254883, -0.930208, 0, 1.281429, -0.926302, 0, 1.254883, -0.930208, 0, 1.254883, -0.930208, 0, 1.281429, -0.926302, 0, 1.281429, -0.926302, 0, 1.281429, -0.926302, 0, 1.307552, -0.920833, 0, 1.281429, -0.926302, 0, 1.281429, -0.926302, 0, 1.307552, -0.920833, 0, 1.307552, -0.920833, 0, 1.307552, -0.920833, 0, 1.333415, -0.913802, 0, 1.307552, -0.920833, 0, 1.307552, -0.920833, 0, 1.333415, -0.913802, 0, 1.333415, -0.913802, 0, 1.333415, -0.913802, 0, 1.359180, -0.905208, 0, 1.333415, -0.913802, 0, 1.333415, -0.913802, 0, 1.359180, -0.905208, 0, 1.359180, -0.905208, 0, 1.359180, -0.905208, 0, 1.384847, -0.895052, 0, 1.359180, -0.905208, 0, 1.359180, -0.905208, 0, 1.384847, -0.895052, 0, 1.384847, -0.895052, 0, 1.384847, -0.895052, 0, 1.410417, -0.883333, 0, 1.384847, -0.895052, 0, 1.384847, -0.895052, 0, 1.410417, -0.883333, 0, 1.410417, -0.883333, 0, 1.410417, -0.883333, 0, 1.410417, -0.966667, 0, 1.410417, -0.883333, 0, 1.410417, -0.883333, 0, 1.410417, -0.966667, 0, 1.410417, -0.966667, 0, 1.410417, -0.966667, 0, 1.384473, -0.978174, 0, 1.410417, -0.966667, 0, 1.410417, -0.966667, 0, 1.384473, -0.978174, 0, 1.384473, -0.978174, 0, 1.384473, -0.978174, 0, 1.358203, -0.988216, 0, 1.384473, -0.978174, 0, 1.384473, -0.978174, 0, 1.358203, -0.988216, 0, 1.358203, -0.988216, 0, 1.358203, -0.988216, 0, 1.331608, -0.996794, 0, 1.358203, -0.988216, 0, 1.358203, -0.988216, 0, 1.331608, -0.996794, 0, 1.331608, -0.996794, 0, 1.331608, -0.996794, 0, 1.304688, -1.003906, 0, 1.331608, -0.996794, 0, 1.331608, -0.996794, 0, 1.304688, -1.003906, 0, 1.304688, -1.003906, 0, 1.304688, -1.003906, 0, 1.277507, -1.009489, 0, 1.304688, -1.003906, 0, 1.304688, -1.003906, 0, 1.277507, -1.009489, 0, 1.277507, -1.009489, 0, 1.277507, -1.009489, 0, 1.250130, -1.013477, 0, 1.277507, -1.009489, 0, 1.277507, -1.009489, 0, 1.250130, -1.013477, 0, 1.250130, -1.013477, 0, 1.250130, -1.013477, 0, 1.222559, -1.015869, 0, 1.250130, -1.013477, 0, 1.250130, -1.013477, 0, 1.222559, -1.015869, 0, 1.222559, -1.015869, 0, 1.222559, -1.015869, 0, 1.194792, -1.016667, 0, 1.222559, -1.015869, 0, 1.222559, -1.015869, 0, 1.194792, -1.016667, 0, 1.194792, -1.016667, 0, 1.194792, -1.016667, 0, 1.128630, -1.011556, 0, 1.194792, -1.016667, 0, 1.194792, -1.016667, 0, 1.128630, -1.011556, 0, 1.128630, -1.011556, 0, 1.128630, -1.011556, 0, 1.069727, -0.996224, 0, 1.128630, -1.011556, 0, 1.128630, -1.011556, 0, 1.069727, -0.996224, 0, 1.069727, -0.996224, 0, 1.069727, -0.996224, 0, 1.018083, -0.970671, 0, 1.069727, -0.996224, 0, 1.069727, -0.996224, 0, 1.018083, -0.970671, 0, 1.018083, -0.970671, 0, 1.018083, -0.970671, 0, 0.973698, -0.934896, 0, 1.018083, -0.970671, 0, 1.018083, -0.970671, 0, 0.973698, -0.934896, 0, 0.973698, -0.934896, 0, 0.973698, -0.934896, 0, 0.938037, -0.890397, 0, 0.973698, -0.934896, 0, 0.973698, -0.934896, 0, 0.938037, -0.890397, 0, 0.938037, -0.890397, 0, 0.938037, -0.890397, 0, 0.912565, -0.838672, 0, 0.938037, -0.890397, 0, 0.938037, -0.890397, 0, 0.912565, -0.838672, 0, 0.912565, -0.838672, 0, 0.912565, -0.838672, 0, 0.897282, -0.779720, 0, 0.912565, -0.838672, 0, 0.912565, -0.838672, 0, 0.897282, -0.779720, 0, 0.897282, -0.779720, 0, 0.897282, -0.779720, 0, 0.892188, -0.713542, 0, 0.897282, -0.779720, 0, 0.897282, -0.779720, 0, 0.892188, -0.713542, 0, 0.892188, -0.713542, 0, 0.892188, -0.713542, 0, 0.897021, -0.645133, 0, 0.892188, -0.713542, 0, 0.892188, -0.713542, 0, 0.897021, -0.645133, 0, 0.897021, -0.645133, 0, 0.897021, -0.645133, 0, 0.911523, -0.584180, 0, 0.897021, -0.645133, 0, 0.897021, -0.645133, 0, 0.911523, -0.584180, 0, 0.911523, -0.584180, 0, 0.911523, -0.584180, 0, 0.935693, -0.530680, 0, 0.911523, -0.584180, 0, 0.911523, -0.584180, 0, 0.935693, -0.530680, 0, 0.935693, -0.530680, 0, 0.935693, -0.530680, 0, 0.969531, -0.484635, 0, 0.935693, -0.530680, 0, 0.935693, -0.530680, 0, 0.969531, -0.484635, 0, 0.969531, -0.484635, 0, 0.969531, -0.484635, 0, 1.011572, -0.447607, 0, 0.969531, -0.484635, 0, 0.969531, -0.484635, 0, 1.011572, -0.447607, 0, 1.011572, -0.447607, 0, 1.011572, -0.447607, 0, 1.060352, -0.421159, 0, 1.011572, -0.447607, 0, 1.011572, -0.447607, 0, 1.060352, -0.421159, 0, 1.060352, -0.421159, 0, 1.060352, -0.421159, 0, 1.115869, -0.405290, 0, 1.060352, -0.421159, 0, 1.060352, -0.421159, 0, 1.115869, -0.405290, 0, 1.115869, -0.405290, 0, 1.115869, -0.405290, 0, 1.178125, -0.400000, 0, 1.115869, -0.405290, 0, 1.115869, -0.405290, 0, 1.178125, -0.400000, 0, 1.178125, -0.400000, 0, 1.178125, -0.400000, 0, 1.233903, -0.404728, 0, 1.178125, -0.400000, 0, 1.178125, -0.400000, 0, 1.233903, -0.404728, 0, 1.233903, -0.404728, 0, 1.233903, -0.404728, 0, 1.283529, -0.418913, 0, 1.233903, -0.404728, 0, 1.233903, -0.404728, 0, 1.283529, -0.418913, 0, 1.283529, -0.418913, 0, 1.283529, -0.418913, 0, 1.327002, -0.442554, 0, 1.283529, -0.418913, 0, 1.283529, -0.418913, 0, 1.327002, -0.442554, 0, 1.327002, -0.442554, 0, 1.327002, -0.442554, 0, 1.364323, -0.475651, 0, 1.327002, -0.442554, 0, 1.327002, -0.442554, 0, 1.364323, -0.475651, 0, 1.364323, -0.475651, 0, 1.364323, -0.475651, 0, 1.394287, -0.516870, 0, 1.364323, -0.475651, 0, 1.364323, -0.475651, 0, 1.394287, -0.516870, 0, 1.394287, -0.516870, 0, 1.394287, -0.516870, 0, 1.415690, -0.564876, 0, 1.394287, -0.516870, 0, 1.394287, -0.516870, 0, 1.415690, -0.564876, 0, 1.415690, -0.564876, 0, 1.415690, -0.564876, 0, 1.428532, -0.619670, 0, 1.415690, -0.564876, 0, 1.415690, -0.564876, 0, 1.428532, -0.619670, 0, 1.428532, -0.619670, 0, 0.897021, -0.645133, 0, 0.892188, -0.713542, 0, 0.995313, -0.650521, 0, 0.892188, -0.713542, 0, 0.897021, -0.645133, 0, 0.995313, -0.650521, 0, 0.911523, -0.584180, 0, 0.897021, -0.645133, 0, 0.995313, -0.650521, 0, 0.897021, -0.645133, 0, 0.911523, -0.584180, 0, 0.995313, -0.650521, 0, 0.911523, -0.584180, 0, 0.995313, -0.650521, 0, 1.001514, -0.612923, 0, 0.995313, -0.650521, 0, 0.911523, -0.584180, 0, 1.001514, -0.612923, 0, 1.012825, -0.579818, 0, 0.911523, -0.584180, 0, 1.001514, -0.612923, 0, 0.911523, -0.584180, 0, 1.012825, -0.579818, 0, 1.001514, -0.612923, 0, 1.012825, -0.579818, 0, 0.935693, -0.530680, 0, 0.911523, -0.584180, 0, 0.935693, -0.530680, 0, 1.012825, -0.579818, 0, 0.911523, -0.584180, 0, 0.935693, -0.530680, 0, 1.012825, -0.579818, 0, 1.029248, -0.551204, 0, 1.012825, -0.579818, 0, 0.935693, -0.530680, 0, 1.029248, -0.551204, 0, 0.969531, -0.484635, 0, 0.935693, -0.530680, 0, 1.029248, -0.551204, 0, 0.935693, -0.530680, 0, 0.969531, -0.484635, 0, 1.029248, -0.551204, 0, 0.969531, -0.484635, 0, 1.029248, -0.551204, 0, 1.050781, -0.527083, 0, 1.029248, -0.551204, 0, 0.969531, -0.484635, 0, 1.050781, -0.527083, 0, 1.011572, -0.447607, 0, 0.969531, -0.484635, 0, 1.050781, -0.527083, 0, 0.969531, -0.484635, 0, 1.011572, -0.447607, 0, 1.050781, -0.527083, 0, 1.011572, -0.447607, 0, 1.050781, -0.527083, 0, 1.076872, -0.507943, 0, 1.050781, -0.527083, 0, 1.011572, -0.447607, 0, 1.076872, -0.507943, 0, 1.060352, -0.421159, 0, 1.011572, -0.447607, 0, 1.076872, -0.507943, 0, 1.011572, -0.447607, 0, 1.060352, -0.421159, 0, 1.076872, -0.507943, 0, 1.060352, -0.421159, 0, 1.076872, -0.507943, 0, 1.106966, -0.494271, 0, 1.076872, -0.507943, 0, 1.060352, -0.421159, 0, 1.106966, -0.494271, 0, 1.060352, -0.421159, 0, 1.106966, -0.494271, 0, 1.115869, -0.405290, 0, 1.106966, -0.494271, 0, 1.060352, -0.421159, 0, 1.115869, -0.405290, 0, 1.115869, -0.405290, 0, 1.106966, -0.494271, 0, 1.141065, -0.486068, 0, 1.106966, -0.494271, 0, 1.115869, -0.405290, 0, 1.141065, -0.486068, 0, 1.115869, -0.405290, 0, 1.141065, -0.486068, 0, 1.178125, -0.400000, 0, 1.141065, -0.486068, 0, 1.115869, -0.405290, 0, 1.178125, -0.400000, 0, 1.178125, -0.400000, 0, 1.141065, -0.486068, 0, 1.179167, -0.483333, 0, 1.141065, -0.486068, 0, 1.178125, -0.400000, 0, 1.179167, -0.483333, 0, 1.178125, -0.400000, 0, 1.179167, -0.483333, 0, 1.212842, -0.486165, 0, 1.179167, -0.483333, 0, 1.178125, -0.400000, 0, 1.212842, -0.486165, 0, 1.178125, -0.400000, 0, 1.212842, -0.486165, 0, 1.233903, -0.404728, 0, 1.212842, -0.486165, 0, 1.178125, -0.400000, 0, 1.233903, -0.404728, 0, 1.233903, -0.404728, 0, 1.212842, -0.486165, 0, 1.243034, -0.494661, 0, 1.212842, -0.486165, 0, 1.233903, -0.404728, 0, 1.243034, -0.494661, 0, 1.233903, -0.404728, 0, 1.243034, -0.494661, 0, 1.283529, -0.418913, 0, 1.243034, -0.494661, 0, 1.233903, -0.404728, 0, 1.283529, -0.418913, 0, 1.243034, -0.494661, 0, 1.269743, -0.508822, 0, 1.283529, -0.418913, 0, 1.269743, -0.508822, 0, 1.243034, -0.494661, 0, 1.283529, -0.418913, 0, 1.283529, -0.418913, 0, 1.269743, -0.508822, 0, 1.327002, -0.442554, 0, 1.269743, -0.508822, 0, 1.283529, -0.418913, 0, 1.327002, -0.442554, 0, 1.269743, -0.508822, 0, 1.292969, -0.528646, 0, 1.327002, -0.442554, 0, 1.292969, -0.528646, 0, 1.269743, -0.508822, 0, 1.327002, -0.442554, 0, 1.327002, -0.442554, 0, 1.292969, -0.528646, 0, 1.364323, -0.475651, 0, 1.292969, -0.528646, 0, 1.327002, -0.442554, 0, 1.364323, -0.475651, 0, 1.292969, -0.528646, 0, 1.311833, -0.553223, 0, 1.364323, -0.475651, 0, 1.311833, -0.553223, 0, 1.292969, -0.528646, 0, 1.364323, -0.475651, 0, 1.364323, -0.475651, 0, 1.311833, -0.553223, 0, 1.394287, -0.516870, 0, 1.311833, -0.553223, 0, 1.364323, -0.475651, 0, 1.394287, -0.516870, 0, 1.311833, -0.553223, 0, 1.325456, -0.581641, 0, 1.394287, -0.516870, 0, 1.325456, -0.581641, 0, 1.311833, -0.553223, 0, 1.394287, -0.516870, 0, 1.394287, -0.516870, 0, 1.325456, -0.581641, 0, 1.415690, -0.564876, 0, 1.325456, -0.581641, 0, 1.394287, -0.516870, 0, 1.415690, -0.564876, 0, 1.325456, -0.581641, 0, 1.333838, -0.613900, 0, 1.415690, -0.564876, 0, 1.333838, -0.613900, 0, 1.325456, -0.581641, 0, 1.415690, -0.564876, 0, 1.415690, -0.564876, 0, 1.333838, -0.613900, 0, 1.428532, -0.619670, 0, 1.333838, -0.613900, 0, 1.415690, -0.564876, 0, 1.428532, -0.619670, 0, 1.333838, -0.613900, 0, 1.336979, -0.650000, 0, 1.428532, -0.619670, 0, 1.336979, -0.650000, 0, 1.333838, -0.613900, 0, 1.428532, -0.619670, 0, 1.428532, -0.619670, 0, 1.336979, -0.650000, 0, 1.432813, -0.681250, 0, 1.336979, -0.650000, 0, 1.428532, -0.619670, 0, 1.432813, -0.681250, 0, 1.432813, -0.733333, 0, 1.432813, -0.681250, 0, 1.336979, -0.650000, 0, 1.432813, -0.681250, 0, 1.432813, -0.733333, 0, 1.336979, -0.650000, 0, 1.336979, -0.650000, 0, 0.992188, -0.733333, 0, 1.432813, -0.733333, 0, 0.992188, -0.733333, 0, 1.336979, -0.650000, 0, 1.432813, -0.733333, 0, 1.336979, -0.650000, 0, 0.995313, -0.650521, 0, 0.992188, -0.733333, 0, 0.995313, -0.650521, 0, 1.336979, -0.650000, 0, 0.992188, -0.733333, 0, 0.892188, -0.713542, 0, 0.992188, -0.733333, 0, 0.995313, -0.650521, 0, 0.992188, -0.733333, 0, 0.892188, -0.713542, 0, 0.995313, -0.650521, 0, 0.892188, -0.713542, 0, 0.897282, -0.779720, 0, 0.992188, -0.733333, 0, 0.897282, -0.779720, 0, 0.892188, -0.713542, 0, 0.992188, -0.733333, 0, 0.992188, -0.733333, 0, 0.897282, -0.779720, 0, 0.998259, -0.779256, 0, 0.897282, -0.779720, 0, 0.992188, -0.733333, 0, 0.998259, -0.779256, 0, 0.897282, -0.779720, 0, 0.912565, -0.838672, 0, 0.998259, -0.779256, 0, 0.912565, -0.838672, 0, 0.897282, -0.779720, 0, 0.998259, -0.779256, 0, 0.998259, -0.779256, 0, 0.912565, -0.838672, 0, 1.010221, -0.819368, 0, 0.912565, -0.838672, 0, 0.998259, -0.779256, 0, 1.010221, -0.819368, 0, 0.912565, -0.838672, 0, 0.938037, -0.890397, 0, 1.010221, -0.819368, 0, 0.938037, -0.890397, 0, 0.912565, -0.838672, 0, 1.010221, -0.819368, 0, 1.010221, -0.819368, 0, 0.938037, -0.890397, 0, 1.028076, -0.853670, 0, 0.938037, -0.890397, 0, 1.010221, -0.819368, 0, 1.028076, -0.853670, 0, 0.938037, -0.890397, 0, 0.973698, -0.934896, 0, 1.028076, -0.853670, 0, 0.973698, -0.934896, 0, 0.938037, -0.890397, 0, 1.028076, -0.853670, 0, 1.028076, -0.853670, 0, 0.973698, -0.934896, 0, 1.051823, -0.882161, 0, 0.973698, -0.934896, 0, 1.028076, -0.853670, 0, 1.051823, -0.882161, 0, 0.973698, -0.934896, 0, 1.018083, -0.970671, 0, 1.051823, -0.882161, 0, 1.018083, -0.970671, 0, 0.973698, -0.934896, 0, 1.051823, -0.882161, 0, 1.051823, -0.882161, 0, 1.018083, -0.970671, 0, 1.081136, -0.904549, 0, 1.018083, -0.970671, 0, 1.051823, -0.882161, 0, 1.081136, -0.904549, 0, 1.081136, -0.904549, 0, 1.018083, -0.970671, 0, 1.069727, -0.996224, 0, 1.018083, -0.970671, 0, 1.081136, -0.904549, 0, 1.069727, -0.996224, 0, 1.115690, -0.920540, 0, 1.081136, -0.904549, 0, 1.069727, -0.996224, 0, 1.081136, -0.904549, 0, 1.115690, -0.920540, 0, 1.069727, -0.996224, 0, 1.115690, -0.920540, 0, 1.069727, -0.996224, 0, 1.128630, -1.011556, 0, 1.069727, -0.996224, 0, 1.115690, -0.920540, 0, 1.128630, -1.011556, 0, 1.155485, -0.930135, 0, 1.115690, -0.920540, 0, 1.128630, -1.011556, 0, 1.115690, -0.920540, 0, 1.155485, -0.930135, 0, 1.128630, -1.011556, 0, 1.155485, -0.930135, 0, 1.128630, -1.011556, 0, 1.194792, -1.016667, 0, 1.128630, -1.011556, 0, 1.155485, -0.930135, 0, 1.194792, -1.016667, 0, 1.200521, -0.933333, 0, 1.155485, -0.930135, 0, 1.194792, -1.016667, 0, 1.155485, -0.930135, 0, 1.200521, -0.933333, 0, 1.194792, -1.016667, 0, 1.200521, -0.933333, 0, 1.194792, -1.016667, 0, 1.222559, -1.015869, 0, 1.194792, -1.016667, 0, 1.200521, -0.933333, 0, 1.222559, -1.015869, 0, 1.200521, -0.933333, 0, 1.222559, -1.015869, 0, 1.227913, -0.932552, 0, 1.222559, -1.015869, 0, 1.200521, -0.933333, 0, 1.227913, -0.932552, 0, 1.227913, -0.932552, 0, 1.222559, -1.015869, 0, 1.250130, -1.013477, 0, 1.222559, -1.015869, 0, 1.227913, -0.932552, 0, 1.250130, -1.013477, 0, 1.227913, -0.932552, 0, 1.250130, -1.013477, 0, 1.254883, -0.930208, 0, 1.250130, -1.013477, 0, 1.227913, -0.932552, 0, 1.254883, -0.930208, 0, 1.254883, -0.930208, 0, 1.250130, -1.013477, 0, 1.277507, -1.009489, 0, 1.250130, -1.013477, 0, 1.254883, -0.930208, 0, 1.277507, -1.009489, 0, 1.254883, -0.930208, 0, 1.277507, -1.009489, 0, 1.281429, -0.926302, 0, 1.277507, -1.009489, 0, 1.254883, -0.930208, 0, 1.281429, -0.926302, 0, 1.281429, -0.926302, 0, 1.277507, -1.009489, 0, 1.304688, -1.003906, 0, 1.277507, -1.009489, 0, 1.281429, -0.926302, 0, 1.304688, -1.003906, 0, 1.281429, -0.926302, 0, 1.304688, -1.003906, 0, 1.307552, -0.920833, 0, 1.304688, -1.003906, 0, 1.281429, -0.926302, 0, 1.307552, -0.920833, 0, 1.307552, -0.920833, 0, 1.304688, -1.003906, 0, 1.331608, -0.996794, 0, 1.304688, -1.003906, 0, 1.307552, -0.920833, 0, 1.331608, -0.996794, 0, 1.333415, -0.913802, 0, 1.307552, -0.920833, 0, 1.331608, -0.996794, 0, 1.307552, -0.920833, 0, 1.333415, -0.913802, 0, 1.331608, -0.996794, 0, 1.358203, -0.988216, 0, 1.333415, -0.913802, 0, 1.331608, -0.996794, 0, 1.333415, -0.913802, 0, 1.358203, -0.988216, 0, 1.331608, -0.996794, 0, 1.333415, -0.913802, 0, 1.358203, -0.988216, 0, 1.384473, -0.978174, 0, 1.358203, -0.988216, 0, 1.333415, -0.913802, 0, 1.384473, -0.978174, 0, 1.359180, -0.905208, 0, 1.333415, -0.913802, 0, 1.384473, -0.978174, 0, 1.333415, -0.913802, 0, 1.359180, -0.905208, 0, 1.384473, -0.978174, 0, 1.359180, -0.905208, 0, 1.384473, -0.978174, 0, 1.410417, -0.966667, 0, 1.384473, -0.978174, 0, 1.359180, -0.905208, 0, 1.410417, -0.966667, 0, 1.384847, -0.895052, 0, 1.359180, -0.905208, 0, 1.410417, -0.966667, 0, 1.359180, -0.905208, 0, 1.384847, -0.895052, 0, 1.410417, -0.966667, 0, 1.384847, -0.895052, 0, 1.410417, -0.966667, 0, 1.410417, -0.883333, 0, 1.410417, -0.966667, 0, 1.384847, -0.895052, 0, 1.410417, -0.883333, 0, 1.336979, -0.650000, 0, 1.333838, -0.613900, 0, 1.336979, -0.650000, 0, 1.336979, -0.650000, 0, 1.333838, -0.613900, 0, 1.333838, -0.613900, 0, 1.333838, -0.613900, 0, 1.325456, -0.581641, 0, 1.333838, -0.613900, 0, 1.333838, -0.613900, 0, 1.325456, -0.581641, 0, 1.325456, -0.581641, 0, 1.325456, -0.581641, 0, 1.311833, -0.553223, 0, 1.325456, -0.581641, 0, 1.325456, -0.581641, 0, 1.311833, -0.553223, 0, 1.311833, -0.553223, 0, 1.311833, -0.553223, 0, 1.292969, -0.528646, 0, 1.311833, -0.553223, 0, 1.311833, -0.553223, 0, 1.292969, -0.528646, 0, 1.292969, -0.528646, 0, 1.292969, -0.528646, 0, 1.269743, -0.508822, 0, 1.292969, -0.528646, 0, 1.292969, -0.528646, 0, 1.269743, -0.508822, 0, 1.269743, -0.508822, 0, 1.269743, -0.508822, 0, 1.243034, -0.494661, 0, 1.269743, -0.508822, 0, 1.269743, -0.508822, 0, 1.243034, -0.494661, 0, 1.243034, -0.494661, 0, 1.243034, -0.494661, 0, 1.212842, -0.486165, 0, 1.243034, -0.494661, 0, 1.243034, -0.494661, 0, 1.212842, -0.486165, 0, 1.212842, -0.486165, 0, 1.212842, -0.486165, 0, 1.179167, -0.483333, 0, 1.212842, -0.486165, 0, 1.212842, -0.486165, 0, 1.179167, -0.483333, 0, 1.179167, -0.483333, 0, 1.179167, -0.483333, 0, 1.141065, -0.486068, 0, 1.179167, -0.483333, 0, 1.179167, -0.483333, 0, 1.141065, -0.486068, 0, 1.141065, -0.486068, 0, 1.141065, -0.486068, 0, 1.106966, -0.494271, 0, 1.141065, -0.486068, 0, 1.141065, -0.486068, 0, 1.106966, -0.494271, 0, 1.106966, -0.494271, 0, 1.106966, -0.494271, 0, 1.076872, -0.507943, 0, 1.106966, -0.494271, 0, 1.106966, -0.494271, 0, 1.076872, -0.507943, 0, 1.076872, -0.507943, 0, 1.076872, -0.507943, 0, 1.050781, -0.527083, 0, 1.076872, -0.507943, 0, 1.076872, -0.507943, 0, 1.050781, -0.527083, 0, 1.050781, -0.527083, 0, 1.050781, -0.527083, 0, 1.029248, -0.551204, 0, 1.050781, -0.527083, 0, 1.050781, -0.527083, 0, 1.029248, -0.551204, 0, 1.029248, -0.551204, 0, 1.029248, -0.551204, 0, 1.012825, -0.579818, 0, 1.029248, -0.551204, 0, 1.029248, -0.551204, 0, 1.012825, -0.579818, 0, 1.012825, -0.579818, 0, 1.012825, -0.579818, 0, 1.001514, -0.612923, 0, 1.012825, -0.579818, 0, 1.012825, -0.579818, 0, 1.001514, -0.612923, 0, 1.001514, -0.612923, 0, 1.001514, -0.612923, 0, 0.995313, -0.650521, 0, 1.001514, -0.612923, 0, 1.001514, -0.612923, 0, 0.995313, -0.650521, 0, 0.995313, -0.650521, 0, 2.068750, -0.642708, 0, 2.068750, -1, 0, 2.068750, -0.642708, 0, 2.068750, -0.642708, 0, 2.068750, -1, 0, 2.068750, -1, 0, 2.068750, -1, 0, 1.972917, -1, 0, 2.068750, -1, 0, 2.068750, -1, 0, 1.972917, -1, 0, 1.972917, -1, 0, 1.972917, -1, 0, 1.972917, -0.645833, 0, 1.972917, -1, 0, 1.972917, -1, 0, 1.972917, -0.645833, 0, 1.972917, -0.645833, 0, 1.972917, -0.645833, 0, 1.970899, -0.607642, 0, 1.972917, -0.645833, 0, 1.972917, -0.645833, 0, 1.970899, -0.607642, 0, 1.970899, -0.607642, 0, 1.970899, -0.607642, 0, 1.964844, -0.574577, 0, 1.970899, -0.607642, 0, 1.970899, -0.607642, 0, 1.964844, -0.574577, 0, 1.964844, -0.574577, 0, 1.964844, -0.574577, 0, 1.954753, -0.546639, 0, 1.964844, -0.574577, 0, 1.964844, -0.574577, 0, 1.954753, -0.546639, 0, 1.954753, -0.546639, 0, 1.954753, -0.546639, 0, 1.940625, -0.523828, 0, 1.954753, -0.546639, 0, 1.954753, -0.546639, 0, 1.940625, -0.523828, 0, 1.940625, -0.523828, 0, 1.940625, -0.523828, 0, 1.922461, -0.506112, 0, 1.940625, -0.523828, 0, 1.940625, -0.523828, 0, 1.922461, -0.506112, 0, 1.922461, -0.506112, 0, 1.922461, -0.506112, 0, 1.900260, -0.493457, 0, 1.922461, -0.506112, 0, 1.922461, -0.506112, 0, 1.900260, -0.493457, 0, 1.900260, -0.493457, 0, 1.900260, -0.493457, 0, 1.874023, -0.485864, 0, 1.900260, -0.493457, 0, 1.900260, -0.493457, 0, 1.874023, -0.485864, 0, 1.874023, -0.485864, 0, 1.874023, -0.485864, 0, 1.843750, -0.483333, 0, 1.874023, -0.485864, 0, 1.874023, -0.485864, 0, 1.843750, -0.483333, 0, 1.843750, -0.483333, 0, 1.843750, -0.483333, 0, 1.806999, -0.486385, 0, 1.843750, -0.483333, 0, 1.843750, -0.483333, 0, 1.806999, -0.486385, 0, 1.806999, -0.486385, 0, 1.806999, -0.486385, 0, 1.774349, -0.495540, 0, 1.806999, -0.486385, 0, 1.806999, -0.486385, 0, 1.774349, -0.495540, 0, 1.774349, -0.495540, 0, 1.774349, -0.495540, 0, 1.745801, -0.510799, 0, 1.774349, -0.495540, 0, 1.774349, -0.495540, 0, 1.745801, -0.510799, 0, 1.745801, -0.510799, 0, 1.745801, -0.510799, 0, 1.721354, -0.532161, 0, 1.745801, -0.510799, 0, 1.745801, -0.510799, 0, 1.721354, -0.532161, 0, 1.721354, -0.532161, 0, 1.721354, -0.532161, 0, 1.701758, -0.558797, 0, 1.721354, -0.532161, 0, 1.721354, -0.532161, 0, 1.701758, -0.558797, 0, 1.701758, -0.558797, 0, 1.701758, -0.558797, 0, 1.687760, -0.589876, 0, 1.701758, -0.558797, 0, 1.701758, -0.558797, 0, 1.687760, -0.589876, 0, 1.687760, -0.589876, 0, 1.687760, -0.589876, 0, 1.679362, -0.625399, 0, 1.687760, -0.589876, 0, 1.687760, -0.589876, 0, 1.679362, -0.625399, 0, 1.679362, -0.625399, 0, 1.679362, -0.625399, 0, 1.676563, -0.665365, 0, 1.679362, -0.625399, 0, 1.679362, -0.625399, 0, 1.676563, -0.665365, 0, 1.676563, -0.665365, 0, 1.676563, -0.665365, 0, 1.676563, -1, 0, 1.676563, -0.665365, 0, 1.676563, -0.665365, 0, 1.676563, -1, 0, 1.676563, -1, 0, 1.676563, -1, 0, 1.580208, -1, 0, 1.676563, -1, 0, 1.676563, -1, 0, 1.580208, -1, 0, 1.580208, -1, 0, 1.580208, -1, 0, 1.580208, -0.416667, 0, 1.580208, -1, 0, 1.580208, -1, 0, 1.580208, -0.416667, 0, 1.580208, -0.416667, 0, 1.580208, -0.416667, 0, 1.676563, -0.416667, 0, 1.580208, -0.416667, 0, 1.580208, -0.416667, 0, 1.676563, -0.416667, 0, 1.676563, -0.416667, 0, 1.676563, -0.416667, 0, 1.676563, -0.500000, 0, 1.676563, -0.416667, 0, 1.676563, -0.416667, 0, 1.676563, -0.500000, 0, 1.676563, -0.500000, 0, 1.676563, -0.500000, 0, 1.694515, -0.476457, 0, 1.676563, -0.500000, 0, 1.676563, -0.500000, 0, 1.694515, -0.476457, 0, 1.694515, -0.476457, 0, 1.694515, -0.476457, 0, 1.713997, -0.456087, 0, 1.694515, -0.476457, 0, 1.694515, -0.476457, 0, 1.713997, -0.456087, 0, 1.713997, -0.456087, 0, 1.713997, -0.456087, 0, 1.735010, -0.438892, 0, 1.713997, -0.456087, 0, 1.713997, -0.456087, 0, 1.735010, -0.438892, 0, 1.735010, -0.438892, 0, 1.735010, -0.438892, 0, 1.757552, -0.424870, 0, 1.735010, -0.438892, 0, 1.735010, -0.438892, 0, 1.757552, -0.424870, 0, 1.757552, -0.424870, 0, 1.757552, -0.424870, 0, 1.781754, -0.413989, 0, 1.757552, -0.424870, 0, 1.757552, -0.424870, 0, 1.781754, -0.413989, 0, 1.781754, -0.413989, 0, 1.781754, -0.413989, 0, 1.807747, -0.406217, 0, 1.781754, -0.413989, 0, 1.781754, -0.413989, 0, 1.807747, -0.406217, 0, 1.807747, -0.406217, 0, 1.807747, -0.406217, 0, 1.835531, -0.401554, 0, 1.807747, -0.406217, 0, 1.807747, -0.406217, 0, 1.835531, -0.401554, 0, 1.835531, -0.401554, 0, 1.835531, -0.401554, 0, 1.865104, -0.400000, 0, 1.835531, -0.401554, 0, 1.835531, -0.401554, 0, 1.865104, -0.400000, 0, 1.865104, -0.400000, 0, 1.865104, -0.400000, 0, 1.912305, -0.403849, 0, 1.865104, -0.400000, 0, 1.865104, -0.400000, 0, 1.912305, -0.403849, 0, 1.912305, -0.403849, 0, 1.912305, -0.403849, 0, 1.953385, -0.415397, 0, 1.912305, -0.403849, 0, 1.912305, -0.403849, 0, 1.953385, -0.415397, 0, 1.953385, -0.415397, 0, 1.953385, -0.415397, 0, 1.988346, -0.434644, 0, 1.953385, -0.415397, 0, 1.953385, -0.415397, 0, 1.988346, -0.434644, 0, 1.988346, -0.434644, 0, 1.988346, -0.434644, 0, 2.017188, -0.461589, 0, 1.988346, -0.434644, 0, 1.988346, -0.434644, 0, 2.017188, -0.461589, 0, 2.017188, -0.461589, 0, 2.017188, -0.461589, 0, 2.039746, -0.496004, 0, 2.017188, -0.461589, 0, 2.017188, -0.461589, 0, 2.039746, -0.496004, 0, 2.039746, -0.496004, 0, 2.039746, -0.496004, 0, 2.055859, -0.537663, 0, 2.039746, -0.496004, 0, 2.039746, -0.496004, 0, 2.055859, -0.537663, 0, 2.055859, -0.537663, 0, 2.055859, -0.537663, 0, 2.065527, -0.586564, 0, 2.055859, -0.537663, 0, 2.055859, -0.537663, 0, 2.065527, -0.586564, 0, 2.065527, -0.586564, 0, 1.676563, -0.416667, 0, 1.580208, -0.416667, 0, 1.676563, -0.500000, 0, 1.580208, -0.416667, 0, 1.676563, -0.416667, 0, 1.676563, -0.500000, 0, 1.580208, -0.416667, 0, 1.687760, -0.589876, 0, 1.676563, -0.500000, 0, 1.687760, -0.589876, 0, 1.580208, -0.416667, 0, 1.676563, -0.500000, 0, 1.687760, -0.589876, 0, 1.580208, -0.416667, 0, 1.679362, -0.625399, 0, 1.580208, -0.416667, 0, 1.687760, -0.589876, 0, 1.679362, -0.625399, 0, 1.679362, -0.625399, 0, 1.580208, -0.416667, 0, 1.676563, -0.665365, 0, 1.580208, -0.416667, 0, 1.679362, -0.625399, 0, 1.676563, -0.665365, 0, 1.580208, -0.416667, 0, 1.580208, -1, 0, 1.676563, -0.665365, 0, 1.580208, -1, 0, 1.580208, -0.416667, 0, 1.676563, -0.665365, 0, 1.580208, -1, 0, 1.676563, -1, 0, 1.676563, -0.665365, 0, 1.676563, -1, 0, 1.580208, -1, 0, 1.676563, -0.665365, 0, 1.676563, -0.500000, 0, 1.687760, -0.589876, 0, 1.701758, -0.558797, 0, 1.687760, -0.589876, 0, 1.676563, -0.500000, 0, 1.701758, -0.558797, 0, 1.676563, -0.500000, 0, 1.701758, -0.558797, 0, 1.721354, -0.532161, 0, 1.701758, -0.558797, 0, 1.676563, -0.500000, 0, 1.721354, -0.532161, 0, 1.694515, -0.476457, 0, 1.676563, -0.500000, 0, 1.721354, -0.532161, 0, 1.676563, -0.500000, 0, 1.694515, -0.476457, 0, 1.721354, -0.532161, 0, 1.694515, -0.476457, 0, 1.721354, -0.532161, 0, 1.745801, -0.510799, 0, 1.721354, -0.532161, 0, 1.694515, -0.476457, 0, 1.745801, -0.510799, 0, 1.713997, -0.456087, 0, 1.694515, -0.476457, 0, 1.745801, -0.510799, 0, 1.694515, -0.476457, 0, 1.713997, -0.456087, 0, 1.745801, -0.510799, 0, 1.774349, -0.495540, 0, 1.713997, -0.456087, 0, 1.745801, -0.510799, 0, 1.713997, -0.456087, 0, 1.774349, -0.495540, 0, 1.745801, -0.510799, 0, 1.774349, -0.495540, 0, 1.735010, -0.438892, 0, 1.713997, -0.456087, 0, 1.735010, -0.438892, 0, 1.774349, -0.495540, 0, 1.713997, -0.456087, 0, 1.757552, -0.424870, 0, 1.735010, -0.438892, 0, 1.774349, -0.495540, 0, 1.735010, -0.438892, 0, 1.757552, -0.424870, 0, 1.774349, -0.495540, 0, 1.757552, -0.424870, 0, 1.774349, -0.495540, 0, 1.806999, -0.486385, 0, 1.774349, -0.495540, 0, 1.757552, -0.424870, 0, 1.806999, -0.486385, 0, 1.781754, -0.413989, 0, 1.757552, -0.424870, 0, 1.806999, -0.486385, 0, 1.757552, -0.424870, 0, 1.781754, -0.413989, 0, 1.806999, -0.486385, 0, 1.781754, -0.413989, 0, 1.806999, -0.486385, 0, 1.807747, -0.406217, 0, 1.806999, -0.486385, 0, 1.781754, -0.413989, 0, 1.807747, -0.406217, 0, 1.807747, -0.406217, 0, 1.806999, -0.486385, 0, 1.843750, -0.483333, 0, 1.806999, -0.486385, 0, 1.807747, -0.406217, 0, 1.843750, -0.483333, 0, 1.835531, -0.401554, 0, 1.807747, -0.406217, 0, 1.843750, -0.483333, 0, 1.807747, -0.406217, 0, 1.835531, -0.401554, 0, 1.843750, -0.483333, 0, 1.865104, -0.400000, 0, 1.835531, -0.401554, 0, 1.843750, -0.483333, 0, 1.835531, -0.401554, 0, 1.865104, -0.400000, 0, 1.843750, -0.483333, 0, 1.865104, -0.400000, 0, 1.843750, -0.483333, 0, 1.874023, -0.485864, 0, 1.843750, -0.483333, 0, 1.865104, -0.400000, 0, 1.874023, -0.485864, 0, 1.912305, -0.403849, 0, 1.865104, -0.400000, 0, 1.874023, -0.485864, 0, 1.865104, -0.400000, 0, 1.912305, -0.403849, 0, 1.874023, -0.485864, 0, 1.874023, -0.485864, 0, 1.900260, -0.493457, 0, 1.912305, -0.403849, 0, 1.900260, -0.493457, 0, 1.874023, -0.485864, 0, 1.912305, -0.403849, 0, 1.912305, -0.403849, 0, 1.900260, -0.493457, 0, 1.953385, -0.415397, 0, 1.900260, -0.493457, 0, 1.912305, -0.403849, 0, 1.953385, -0.415397, 0, 1.900260, -0.493457, 0, 1.922461, -0.506112, 0, 1.953385, -0.415397, 0, 1.922461, -0.506112, 0, 1.900260, -0.493457, 0, 1.953385, -0.415397, 0, 1.953385, -0.415397, 0, 1.922461, -0.506112, 0, 1.988346, -0.434644, 0, 1.922461, -0.506112, 0, 1.953385, -0.415397, 0, 1.988346, -0.434644, 0, 1.922461, -0.506112, 0, 1.940625, -0.523828, 0, 1.988346, -0.434644, 0, 1.940625, -0.523828, 0, 1.922461, -0.506112, 0, 1.988346, -0.434644, 0, 1.988346, -0.434644, 0, 1.940625, -0.523828, 0, 2.017188, -0.461589, 0, 1.940625, -0.523828, 0, 1.988346, -0.434644, 0, 2.017188, -0.461589, 0, 2.017188, -0.461589, 0, 1.940625, -0.523828, 0, 2.039746, -0.496004, 0, 1.940625, -0.523828, 0, 2.017188, -0.461589, 0, 2.039746, -0.496004, 0, 2.039746, -0.496004, 0, 1.940625, -0.523828, 0, 1.954753, -0.546639, 0, 1.940625, -0.523828, 0, 2.039746, -0.496004, 0, 1.954753, -0.546639, 0, 2.039746, -0.496004, 0, 1.954753, -0.546639, 0, 2.055859, -0.537663, 0, 1.954753, -0.546639, 0, 2.039746, -0.496004, 0, 2.055859, -0.537663, 0, 1.954753, -0.546639, 0, 1.964844, -0.574577, 0, 2.055859, -0.537663, 0, 1.964844, -0.574577, 0, 1.954753, -0.546639, 0, 2.055859, -0.537663, 0, 2.055859, -0.537663, 0, 1.964844, -0.574577, 0, 2.065527, -0.586564, 0, 1.964844, -0.574577, 0, 2.055859, -0.537663, 0, 2.065527, -0.586564, 0, 1.964844, -0.574577, 0, 1.970899, -0.607642, 0, 2.065527, -0.586564, 0, 1.970899, -0.607642, 0, 1.964844, -0.574577, 0, 2.065527, -0.586564, 0, 2.065527, -0.586564, 0, 1.970899, -0.607642, 0, 2.068750, -0.642708, 0, 1.970899, -0.607642, 0, 2.065527, -0.586564, 0, 2.068750, -0.642708, 0, 2.068750, -0.642708, 0, 1.970899, -0.607642, 0, 1.972917, -0.645833, 0, 1.970899, -0.607642, 0, 2.068750, -0.642708, 0, 1.972917, -0.645833, 0, 1.972917, -0.645833, 0, 2.068750, -1, 0, 2.068750, -0.642708, 0, 2.068750, -1, 0, 1.972917, -0.645833, 0, 2.068750, -0.642708, 0, 1.972917, -0.645833, 0, 1.972917, -1, 0, 2.068750, -1, 0, 1.972917, -1, 0, 1.972917, -0.645833, 0, 2.068750, -1, 0, 2.766146, -0.681250, 0, 2.766146, -0.733333, 0, 2.766146, -0.681250, 0, 2.766146, -0.681250, 0, 2.766146, -0.733333, 0, 2.766146, -0.733333, 0, 2.766146, -0.733333, 0, 2.325521, -0.733333, 0, 2.766146, -0.733333, 0, 2.766146, -0.733333, 0, 2.325521, -0.733333, 0, 2.325521, -0.733333, 0, 2.325521, -0.733333, 0, 2.331592, -0.779256, 0, 2.325521, -0.733333, 0, 2.325521, -0.733333, 0, 2.331592, -0.779256, 0, 2.331592, -0.779256, 0, 2.331592, -0.779256, 0, 2.343555, -0.819368, 0, 2.331592, -0.779256, 0, 2.331592, -0.779256, 0, 2.343555, -0.819368, 0, 2.343555, -0.819368, 0, 2.343555, -0.819368, 0, 2.361410, -0.853670, 0, 2.343555, -0.819368, 0, 2.343555, -0.819368, 0, 2.361410, -0.853670, 0, 2.361410, -0.853670, 0, 2.361410, -0.853670, 0, 2.385156, -0.882161, 0, 2.361410, -0.853670, 0, 2.361410, -0.853670, 0, 2.385156, -0.882161, 0, 2.385156, -0.882161, 0, 2.385156, -0.882161, 0, 2.414469, -0.904549, 0, 2.385156, -0.882161, 0, 2.385156, -0.882161, 0, 2.414469, -0.904549, 0, 2.414469, -0.904549, 0, 2.414469, -0.904549, 0, 2.449023, -0.920540, 0, 2.414469, -0.904549, 0, 2.414469, -0.904549, 0, 2.449023, -0.920540, 0, 2.449023, -0.920540, 0, 2.449023, -0.920540, 0, 2.488818, -0.930135, 0, 2.449023, -0.920540, 0, 2.449023, -0.920540, 0, 2.488818, -0.930135, 0, 2.488818, -0.930135, 0, 2.488818, -0.930135, 0, 2.533854, -0.933333, 0, 2.488818, -0.930135, 0, 2.488818, -0.930135, 0, 2.533854, -0.933333, 0, 2.533854, -0.933333, 0, 2.533854, -0.933333, 0, 2.561247, -0.932552, 0, 2.533854, -0.933333, 0, 2.533854, -0.933333, 0, 2.561247, -0.932552, 0, 2.561247, -0.932552, 0, 2.561247, -0.932552, 0, 2.588216, -0.930208, 0, 2.561247, -0.932552, 0, 2.561247, -0.932552, 0, 2.588216, -0.930208, 0, 2.588216, -0.930208, 0, 2.588216, -0.930208, 0, 2.614762, -0.926302, 0, 2.588216, -0.930208, 0, 2.588216, -0.930208, 0, 2.614762, -0.926302, 0, 2.614762, -0.926302, 0, 2.614762, -0.926302, 0, 2.640885, -0.920833, 0, 2.614762, -0.926302, 0, 2.614762, -0.926302, 0, 2.640885, -0.920833, 0, 2.640885, -0.920833, 0, 2.640885, -0.920833, 0, 2.666748, -0.913802, 0, 2.640885, -0.920833, 0, 2.640885, -0.920833, 0, 2.666748, -0.913802, 0, 2.666748, -0.913802, 0, 2.666748, -0.913802, 0, 2.692513, -0.905208, 0, 2.666748, -0.913802, 0, 2.666748, -0.913802, 0, 2.692513, -0.905208, 0, 2.692513, -0.905208, 0, 2.692513, -0.905208, 0, 2.718180, -0.895052, 0, 2.692513, -0.905208, 0, 2.692513, -0.905208, 0, 2.718180, -0.895052, 0, 2.718180, -0.895052, 0, 2.718180, -0.895052, 0, 2.743750, -0.883333, 0, 2.718180, -0.895052, 0, 2.718180, -0.895052, 0, 2.743750, -0.883333, 0, 2.743750, -0.883333, 0, 2.743750, -0.883333, 0, 2.743750, -0.966667, 0, 2.743750, -0.883333, 0, 2.743750, -0.883333, 0, 2.743750, -0.966667, 0, 2.743750, -0.966667, 0, 2.743750, -0.966667, 0, 2.717806, -0.978174, 0, 2.743750, -0.966667, 0, 2.743750, -0.966667, 0, 2.717806, -0.978174, 0, 2.717806, -0.978174, 0, 2.717806, -0.978174, 0, 2.691537, -0.988216, 0, 2.717806, -0.978174, 0, 2.717806, -0.978174, 0, 2.691537, -0.988216, 0, 2.691537, -0.988216, 0, 2.691537, -0.988216, 0, 2.664942, -0.996794, 0, 2.691537, -0.988216, 0, 2.691537, -0.988216, 0, 2.664942, -0.996794, 0, 2.664942, -0.996794, 0, 2.664942, -0.996794, 0, 2.638021, -1.003906, 0, 2.664942, -0.996794, 0, 2.664942, -0.996794, 0, 2.638021, -1.003906, 0, 2.638021, -1.003906, 0, 2.638021, -1.003906, 0, 2.610840, -1.009489, 0, 2.638021, -1.003906, 0, 2.638021, -1.003906, 0, 2.610840, -1.009489, 0, 2.610840, -1.009489, 0, 2.610840, -1.009489, 0, 2.583463, -1.013477, 0, 2.610840, -1.009489, 0, 2.610840, -1.009489, 0, 2.583463, -1.013477, 0, 2.583463, -1.013477, 0, 2.583463, -1.013477, 0, 2.555892, -1.015869, 0, 2.583463, -1.013477, 0, 2.583463, -1.013477, 0, 2.555892, -1.015869, 0, 2.555892, -1.015869, 0, 2.555892, -1.015869, 0, 2.528125, -1.016667, 0, 2.555892, -1.015869, 0, 2.555892, -1.015869, 0, 2.528125, -1.016667, 0, 2.528125, -1.016667, 0, 2.528125, -1.016667, 0, 2.461963, -1.011556, 0, 2.528125, -1.016667, 0, 2.528125, -1.016667, 0, 2.461963, -1.011556, 0, 2.461963, -1.011556, 0, 2.461963, -1.011556, 0, 2.403060, -0.996224, 0, 2.461963, -1.011556, 0, 2.461963, -1.011556, 0, 2.403060, -0.996224, 0, 2.403060, -0.996224, 0, 2.403060, -0.996224, 0, 2.351416, -0.970671, 0, 2.403060, -0.996224, 0, 2.403060, -0.996224, 0, 2.351416, -0.970671, 0, 2.351416, -0.970671, 0, 2.351416, -0.970671, 0, 2.307031, -0.934896, 0, 2.351416, -0.970671, 0, 2.351416, -0.970671, 0, 2.307031, -0.934896, 0, 2.307031, -0.934896, 0, 2.307031, -0.934896, 0, 2.271371, -0.890397, 0, 2.307031, -0.934896, 0, 2.307031, -0.934896, 0, 2.271371, -0.890397, 0, 2.271371, -0.890397, 0, 2.271371, -0.890397, 0, 2.245898, -0.838672, 0, 2.271371, -0.890397, 0, 2.271371, -0.890397, 0, 2.245898, -0.838672, 0, 2.245898, -0.838672, 0, 2.245898, -0.838672, 0, 2.230615, -0.779720, 0, 2.245898, -0.838672, 0, 2.245898, -0.838672, 0, 2.230615, -0.779720, 0, 2.230615, -0.779720, 0, 2.230615, -0.779720, 0, 2.225521, -0.713542, 0, 2.230615, -0.779720, 0, 2.230615, -0.779720, 0, 2.225521, -0.713542, 0, 2.225521, -0.713542, 0, 2.225521, -0.713542, 0, 2.230355, -0.645133, 0, 2.225521, -0.713542, 0, 2.225521, -0.713542, 0, 2.230355, -0.645133, 0, 2.230355, -0.645133, 0, 2.230355, -0.645133, 0, 2.244857, -0.584180, 0, 2.230355, -0.645133, 0, 2.230355, -0.645133, 0, 2.244857, -0.584180, 0, 2.244857, -0.584180, 0, 2.244857, -0.584180, 0, 2.269027, -0.530680, 0, 2.244857, -0.584180, 0, 2.244857, -0.584180, 0, 2.269027, -0.530680, 0, 2.269027, -0.530680, 0, 2.269027, -0.530680, 0, 2.302865, -0.484635, 0, 2.269027, -0.530680, 0, 2.269027, -0.530680, 0, 2.302865, -0.484635, 0, 2.302865, -0.484635, 0, 2.302865, -0.484635, 0, 2.344906, -0.447607, 0, 2.302865, -0.484635, 0, 2.302865, -0.484635, 0, 2.344906, -0.447607, 0, 2.344906, -0.447607, 0, 2.344906, -0.447607, 0, 2.393685, -0.421159, 0, 2.344906, -0.447607, 0, 2.344906, -0.447607, 0, 2.393685, -0.421159, 0, 2.393685, -0.421159, 0, 2.393685, -0.421159, 0, 2.449203, -0.405290, 0, 2.393685, -0.421159, 0, 2.393685, -0.421159, 0, 2.449203, -0.405290, 0, 2.449203, -0.405290, 0, 2.449203, -0.405290, 0, 2.511458, -0.400000, 0, 2.449203, -0.405290, 0, 2.449203, -0.405290, 0, 2.511458, -0.400000, 0, 2.511458, -0.400000, 0, 2.511458, -0.400000, 0, 2.567236, -0.404728, 0, 2.511458, -0.400000, 0, 2.511458, -0.400000, 0, 2.567236, -0.404728, 0, 2.567236, -0.404728, 0, 2.567236, -0.404728, 0, 2.616862, -0.418913, 0, 2.567236, -0.404728, 0, 2.567236, -0.404728, 0, 2.616862, -0.418913, 0, 2.616862, -0.418913, 0, 2.616862, -0.418913, 0, 2.660335, -0.442554, 0, 2.616862, -0.418913, 0, 2.616862, -0.418913, 0, 2.660335, -0.442554, 0, 2.660335, -0.442554, 0, 2.660335, -0.442554, 0, 2.697656, -0.475651, 0, 2.660335, -0.442554, 0, 2.660335, -0.442554, 0, 2.697656, -0.475651, 0, 2.697656, -0.475651, 0, 2.697656, -0.475651, 0, 2.727621, -0.516870, 0, 2.697656, -0.475651, 0, 2.697656, -0.475651, 0, 2.727621, -0.516870, 0, 2.727621, -0.516870, 0, 2.727621, -0.516870, 0, 2.749023, -0.564876, 0, 2.727621, -0.516870, 0, 2.727621, -0.516870, 0, 2.749023, -0.564876, 0, 2.749023, -0.564876, 0, 2.749023, -0.564876, 0, 2.761865, -0.619670, 0, 2.749023, -0.564876, 0, 2.749023, -0.564876, 0, 2.761865, -0.619670, 0, 2.761865, -0.619670, 0, 2.230355, -0.645133, 0, 2.225521, -0.713542, 0, 2.328646, -0.650521, 0, 2.225521, -0.713542, 0, 2.230355, -0.645133, 0, 2.328646, -0.650521, 0, 2.244857, -0.584180, 0, 2.230355, -0.645133, 0, 2.328646, -0.650521, 0, 2.230355, -0.645133, 0, 2.244857, -0.584180, 0, 2.328646, -0.650521, 0, 2.244857, -0.584180, 0, 2.328646, -0.650521, 0, 2.334847, -0.612923, 0, 2.328646, -0.650521, 0, 2.244857, -0.584180, 0, 2.334847, -0.612923, 0, 2.346159, -0.579818, 0, 2.244857, -0.584180, 0, 2.334847, -0.612923, 0, 2.244857, -0.584180, 0, 2.346159, -0.579818, 0, 2.334847, -0.612923, 0, 2.346159, -0.579818, 0, 2.269027, -0.530680, 0, 2.244857, -0.584180, 0, 2.269027, -0.530680, 0, 2.346159, -0.579818, 0, 2.244857, -0.584180, 0, 2.269027, -0.530680, 0, 2.346159, -0.579818, 0, 2.362581, -0.551204, 0, 2.346159, -0.579818, 0, 2.269027, -0.530680, 0, 2.362581, -0.551204, 0, 2.302865, -0.484635, 0, 2.269027, -0.530680, 0, 2.362581, -0.551204, 0, 2.269027, -0.530680, 0, 2.302865, -0.484635, 0, 2.362581, -0.551204, 0, 2.302865, -0.484635, 0, 2.362581, -0.551204, 0, 2.384115, -0.527083, 0, 2.362581, -0.551204, 0, 2.302865, -0.484635, 0, 2.384115, -0.527083, 0, 2.344906, -0.447607, 0, 2.302865, -0.484635, 0, 2.384115, -0.527083, 0, 2.302865, -0.484635, 0, 2.344906, -0.447607, 0, 2.384115, -0.527083, 0, 2.344906, -0.447607, 0, 2.384115, -0.527083, 0, 2.410205, -0.507943, 0, 2.384115, -0.527083, 0, 2.344906, -0.447607, 0, 2.410205, -0.507943, 0, 2.393685, -0.421159, 0, 2.344906, -0.447607, 0, 2.410205, -0.507943, 0, 2.344906, -0.447607, 0, 2.393685, -0.421159, 0, 2.410205, -0.507943, 0, 2.393685, -0.421159, 0, 2.410205, -0.507943, 0, 2.440300, -0.494271, 0, 2.410205, -0.507943, 0, 2.393685, -0.421159, 0, 2.440300, -0.494271, 0, 2.393685, -0.421159, 0, 2.440300, -0.494271, 0, 2.449203, -0.405290, 0, 2.440300, -0.494271, 0, 2.393685, -0.421159, 0, 2.449203, -0.405290, 0, 2.449203, -0.405290, 0, 2.440300, -0.494271, 0, 2.474398, -0.486068, 0, 2.440300, -0.494271, 0, 2.449203, -0.405290, 0, 2.474398, -0.486068, 0, 2.449203, -0.405290, 0, 2.474398, -0.486068, 0, 2.511458, -0.400000, 0, 2.474398, -0.486068, 0, 2.449203, -0.405290, 0, 2.511458, -0.400000, 0, 2.511458, -0.400000, 0, 2.474398, -0.486068, 0, 2.512500, -0.483333, 0, 2.474398, -0.486068, 0, 2.511458, -0.400000, 0, 2.512500, -0.483333, 0, 2.511458, -0.400000, 0, 2.512500, -0.483333, 0, 2.546175, -0.486165, 0, 2.512500, -0.483333, 0, 2.511458, -0.400000, 0, 2.546175, -0.486165, 0, 2.511458, -0.400000, 0, 2.546175, -0.486165, 0, 2.567236, -0.404728, 0, 2.546175, -0.486165, 0, 2.511458, -0.400000, 0, 2.567236, -0.404728, 0, 2.567236, -0.404728, 0, 2.546175, -0.486165, 0, 2.576367, -0.494661, 0, 2.546175, -0.486165, 0, 2.567236, -0.404728, 0, 2.576367, -0.494661, 0, 2.567236, -0.404728, 0, 2.576367, -0.494661, 0, 2.616862, -0.418913, 0, 2.576367, -0.494661, 0, 2.567236, -0.404728, 0, 2.616862, -0.418913, 0, 2.576367, -0.494661, 0, 2.603076, -0.508822, 0, 2.616862, -0.418913, 0, 2.603076, -0.508822, 0, 2.576367, -0.494661, 0, 2.616862, -0.418913, 0, 2.616862, -0.418913, 0, 2.603076, -0.508822, 0, 2.660335, -0.442554, 0, 2.603076, -0.508822, 0, 2.616862, -0.418913, 0, 2.660335, -0.442554, 0, 2.603076, -0.508822, 0, 2.626302, -0.528646, 0, 2.660335, -0.442554, 0, 2.626302, -0.528646, 0, 2.603076, -0.508822, 0, 2.660335, -0.442554, 0, 2.660335, -0.442554, 0, 2.626302, -0.528646, 0, 2.697656, -0.475651, 0, 2.626302, -0.528646, 0, 2.660335, -0.442554, 0, 2.697656, -0.475651, 0, 2.626302, -0.528646, 0, 2.645166, -0.553223, 0, 2.697656, -0.475651, 0, 2.645166, -0.553223, 0, 2.626302, -0.528646, 0, 2.697656, -0.475651, 0, 2.697656, -0.475651, 0, 2.645166, -0.553223, 0, 2.727621, -0.516870, 0, 2.645166, -0.553223, 0, 2.697656, -0.475651, 0, 2.727621, -0.516870, 0, 2.645166, -0.553223, 0, 2.658789, -0.581641, 0, 2.727621, -0.516870, 0, 2.658789, -0.581641, 0, 2.645166, -0.553223, 0, 2.727621, -0.516870, 0, 2.727621, -0.516870, 0, 2.658789, -0.581641, 0, 2.749023, -0.564876, 0, 2.658789, -0.581641, 0, 2.727621, -0.516870, 0, 2.749023, -0.564876, 0, 2.658789, -0.581641, 0, 2.667171, -0.613900, 0, 2.749023, -0.564876, 0, 2.667171, -0.613900, 0, 2.658789, -0.581641, 0, 2.749023, -0.564876, 0, 2.749023, -0.564876, 0, 2.667171, -0.613900, 0, 2.761865, -0.619670, 0, 2.667171, -0.613900, 0, 2.749023, -0.564876, 0, 2.761865, -0.619670, 0, 2.667171, -0.613900, 0, 2.670313, -0.650000, 0, 2.761865, -0.619670, 0, 2.670313, -0.650000, 0, 2.667171, -0.613900, 0, 2.761865, -0.619670, 0, 2.761865, -0.619670, 0, 2.670313, -0.650000, 0, 2.766146, -0.681250, 0, 2.670313, -0.650000, 0, 2.761865, -0.619670, 0, 2.766146, -0.681250, 0, 2.766146, -0.733333, 0, 2.766146, -0.681250, 0, 2.670313, -0.650000, 0, 2.766146, -0.681250, 0, 2.766146, -0.733333, 0, 2.670313, -0.650000, 0, 2.670313, -0.650000, 0, 2.325521, -0.733333, 0, 2.766146, -0.733333, 0, 2.325521, -0.733333, 0, 2.670313, -0.650000, 0, 2.766146, -0.733333, 0, 2.670313, -0.650000, 0, 2.328646, -0.650521, 0, 2.325521, -0.733333, 0, 2.328646, -0.650521, 0, 2.670313, -0.650000, 0, 2.325521, -0.733333, 0, 2.225521, -0.713542, 0, 2.325521, -0.733333, 0, 2.328646, -0.650521, 0, 2.325521, -0.733333, 0, 2.225521, -0.713542, 0, 2.328646, -0.650521, 0, 2.225521, -0.713542, 0, 2.230615, -0.779720, 0, 2.325521, -0.733333, 0, 2.230615, -0.779720, 0, 2.225521, -0.713542, 0, 2.325521, -0.733333, 0, 2.325521, -0.733333, 0, 2.230615, -0.779720, 0, 2.331592, -0.779256, 0, 2.230615, -0.779720, 0, 2.325521, -0.733333, 0, 2.331592, -0.779256, 0, 2.230615, -0.779720, 0, 2.245898, -0.838672, 0, 2.331592, -0.779256, 0, 2.245898, -0.838672, 0, 2.230615, -0.779720, 0, 2.331592, -0.779256, 0, 2.331592, -0.779256, 0, 2.245898, -0.838672, 0, 2.343555, -0.819368, 0, 2.245898, -0.838672, 0, 2.331592, -0.779256, 0, 2.343555, -0.819368, 0, 2.245898, -0.838672, 0, 2.271371, -0.890397, 0, 2.343555, -0.819368, 0, 2.271371, -0.890397, 0, 2.245898, -0.838672, 0, 2.343555, -0.819368, 0, 2.343555, -0.819368, 0, 2.271371, -0.890397, 0, 2.361410, -0.853670, 0, 2.271371, -0.890397, 0, 2.343555, -0.819368, 0, 2.361410, -0.853670, 0, 2.271371, -0.890397, 0, 2.307031, -0.934896, 0, 2.361410, -0.853670, 0, 2.307031, -0.934896, 0, 2.271371, -0.890397, 0, 2.361410, -0.853670, 0, 2.361410, -0.853670, 0, 2.307031, -0.934896, 0, 2.385156, -0.882161, 0, 2.307031, -0.934896, 0, 2.361410, -0.853670, 0, 2.385156, -0.882161, 0, 2.307031, -0.934896, 0, 2.351416, -0.970671, 0, 2.385156, -0.882161, 0, 2.351416, -0.970671, 0, 2.307031, -0.934896, 0, 2.385156, -0.882161, 0, 2.385156, -0.882161, 0, 2.351416, -0.970671, 0, 2.414469, -0.904549, 0, 2.351416, -0.970671, 0, 2.385156, -0.882161, 0, 2.414469, -0.904549, 0, 2.414469, -0.904549, 0, 2.351416, -0.970671, 0, 2.403060, -0.996224, 0, 2.351416, -0.970671, 0, 2.414469, -0.904549, 0, 2.403060, -0.996224, 0, 2.449023, -0.920540, 0, 2.414469, -0.904549, 0, 2.403060, -0.996224, 0, 2.414469, -0.904549, 0, 2.449023, -0.920540, 0, 2.403060, -0.996224, 0, 2.449023, -0.920540, 0, 2.403060, -0.996224, 0, 2.461963, -1.011556, 0, 2.403060, -0.996224, 0, 2.449023, -0.920540, 0, 2.461963, -1.011556, 0, 2.488818, -0.930135, 0, 2.449023, -0.920540, 0, 2.461963, -1.011556, 0, 2.449023, -0.920540, 0, 2.488818, -0.930135, 0, 2.461963, -1.011556, 0, 2.488818, -0.930135, 0, 2.461963, -1.011556, 0, 2.528125, -1.016667, 0, 2.461963, -1.011556, 0, 2.488818, -0.930135, 0, 2.528125, -1.016667, 0, 2.533854, -0.933333, 0, 2.488818, -0.930135, 0, 2.528125, -1.016667, 0, 2.488818, -0.930135, 0, 2.533854, -0.933333, 0, 2.528125, -1.016667, 0, 2.533854, -0.933333, 0, 2.528125, -1.016667, 0, 2.555892, -1.015869, 0, 2.528125, -1.016667, 0, 2.533854, -0.933333, 0, 2.555892, -1.015869, 0, 2.533854, -0.933333, 0, 2.555892, -1.015869, 0, 2.561247, -0.932552, 0, 2.555892, -1.015869, 0, 2.533854, -0.933333, 0, 2.561247, -0.932552, 0, 2.561247, -0.932552, 0, 2.555892, -1.015869, 0, 2.583463, -1.013477, 0, 2.555892, -1.015869, 0, 2.561247, -0.932552, 0, 2.583463, -1.013477, 0, 2.561247, -0.932552, 0, 2.583463, -1.013477, 0, 2.588216, -0.930208, 0, 2.583463, -1.013477, 0, 2.561247, -0.932552, 0, 2.588216, -0.930208, 0, 2.588216, -0.930208, 0, 2.583463, -1.013477, 0, 2.610840, -1.009489, 0, 2.583463, -1.013477, 0, 2.588216, -0.930208, 0, 2.610840, -1.009489, 0, 2.588216, -0.930208, 0, 2.610840, -1.009489, 0, 2.614762, -0.926302, 0, 2.610840, -1.009489, 0, 2.588216, -0.930208, 0, 2.614762, -0.926302, 0, 2.614762, -0.926302, 0, 2.610840, -1.009489, 0, 2.638021, -1.003906, 0, 2.610840, -1.009489, 0, 2.614762, -0.926302, 0, 2.638021, -1.003906, 0, 2.614762, -0.926302, 0, 2.638021, -1.003906, 0, 2.640885, -0.920833, 0, 2.638021, -1.003906, 0, 2.614762, -0.926302, 0, 2.640885, -0.920833, 0, 2.640885, -0.920833, 0, 2.638021, -1.003906, 0, 2.664942, -0.996794, 0, 2.638021, -1.003906, 0, 2.640885, -0.920833, 0, 2.664942, -0.996794, 0, 2.666748, -0.913802, 0, 2.640885, -0.920833, 0, 2.664942, -0.996794, 0, 2.640885, -0.920833, 0, 2.666748, -0.913802, 0, 2.664942, -0.996794, 0, 2.691537, -0.988216, 0, 2.666748, -0.913802, 0, 2.664942, -0.996794, 0, 2.666748, -0.913802, 0, 2.691537, -0.988216, 0, 2.664942, -0.996794, 0, 2.666748, -0.913802, 0, 2.691537, -0.988216, 0, 2.717806, -0.978174, 0, 2.691537, -0.988216, 0, 2.666748, -0.913802, 0, 2.717806, -0.978174, 0, 2.692513, -0.905208, 0, 2.666748, -0.913802, 0, 2.717806, -0.978174, 0, 2.666748, -0.913802, 0, 2.692513, -0.905208, 0, 2.717806, -0.978174, 0, 2.692513, -0.905208, 0, 2.717806, -0.978174, 0, 2.743750, -0.966667, 0, 2.717806, -0.978174, 0, 2.692513, -0.905208, 0, 2.743750, -0.966667, 0, 2.718180, -0.895052, 0, 2.692513, -0.905208, 0, 2.743750, -0.966667, 0, 2.692513, -0.905208, 0, 2.718180, -0.895052, 0, 2.743750, -0.966667, 0, 2.718180, -0.895052, 0, 2.743750, -0.966667, 0, 2.743750, -0.883333, 0, 2.743750, -0.966667, 0, 2.718180, -0.895052, 0, 2.743750, -0.883333, 0, 2.670313, -0.650000, 0, 2.667171, -0.613900, 0, 2.670313, -0.650000, 0, 2.670313, -0.650000, 0, 2.667171, -0.613900, 0, 2.667171, -0.613900, 0, 2.667171, -0.613900, 0, 2.658789, -0.581641, 0, 2.667171, -0.613900, 0, 2.667171, -0.613900, 0, 2.658789, -0.581641, 0, 2.658789, -0.581641, 0, 2.658789, -0.581641, 0, 2.645166, -0.553223, 0, 2.658789, -0.581641, 0, 2.658789, -0.581641, 0, 2.645166, -0.553223, 0, 2.645166, -0.553223, 0, 2.645166, -0.553223, 0, 2.626302, -0.528646, 0, 2.645166, -0.553223, 0, 2.645166, -0.553223, 0, 2.626302, -0.528646, 0, 2.626302, -0.528646, 0, 2.626302, -0.528646, 0, 2.603076, -0.508822, 0, 2.626302, -0.528646, 0, 2.626302, -0.528646, 0, 2.603076, -0.508822, 0, 2.603076, -0.508822, 0, 2.603076, -0.508822, 0, 2.576367, -0.494661, 0, 2.603076, -0.508822, 0, 2.603076, -0.508822, 0, 2.576367, -0.494661, 0, 2.576367, -0.494661, 0, 2.576367, -0.494661, 0, 2.546175, -0.486165, 0, 2.576367, -0.494661, 0, 2.576367, -0.494661, 0, 2.546175, -0.486165, 0, 2.546175, -0.486165, 0, 2.546175, -0.486165, 0, 2.512500, -0.483333, 0, 2.546175, -0.486165, 0, 2.546175, -0.486165, 0, 2.512500, -0.483333, 0, 2.512500, -0.483333, 0, 2.512500, -0.483333, 0, 2.474398, -0.486068, 0, 2.512500, -0.483333, 0, 2.512500, -0.483333, 0, 2.474398, -0.486068, 0, 2.474398, -0.486068, 0, 2.474398, -0.486068, 0, 2.440300, -0.494271, 0, 2.474398, -0.486068, 0, 2.474398, -0.486068, 0, 2.440300, -0.494271, 0, 2.440300, -0.494271, 0, 2.440300, -0.494271, 0, 2.410205, -0.507943, 0, 2.440300, -0.494271, 0, 2.440300, -0.494271, 0, 2.410205, -0.507943, 0, 2.410205, -0.507943, 0, 2.410205, -0.507943, 0, 2.384115, -0.527083, 0, 2.410205, -0.507943, 0, 2.410205, -0.507943, 0, 2.384115, -0.527083, 0, 2.384115, -0.527083, 0, 2.384115, -0.527083, 0, 2.362581, -0.551204, 0, 2.384115, -0.527083, 0, 2.384115, -0.527083, 0, 2.362581, -0.551204, 0, 2.362581, -0.551204, 0, 2.362581, -0.551204, 0, 2.346159, -0.579818, 0, 2.362581, -0.551204, 0, 2.362581, -0.551204, 0, 2.346159, -0.579818, 0, 2.346159, -0.579818, 0, 2.346159, -0.579818, 0, 2.334847, -0.612923, 0, 2.346159, -0.579818, 0, 2.346159, -0.579818, 0, 2.334847, -0.612923, 0, 2.334847, -0.612923, 0, 2.334847, -0.612923, 0, 2.328646, -0.650521, 0, 2.334847, -0.612923, 0, 2.334847, -0.612923, 0, 2.328646, -0.650521, 0, 2.328646, -0.650521, 0, 3.255208, -0.500000, 0, 3.246956, -0.495988, 0, 3.255208, -0.500000, 0, 3.255208, -0.500000, 0, 3.246956, -0.495988, 0, 3.246956, -0.495988, 0, 3.246956, -0.495988, 0, 3.238347, -0.492546, 0, 3.246956, -0.495988, 0, 3.246956, -0.495988, 0, 3.238347, -0.492546, 0, 3.238347, -0.492546, 0, 3.238347, -0.492546, 0, 3.229378, -0.489673, 0, 3.238347, -0.492546, 0, 3.238347, -0.492546, 0, 3.229378, -0.489673, 0, 3.229378, -0.489673, 0, 3.229378, -0.489673, 0, 3.220052, -0.487370, 0, 3.229378, -0.489673, 0, 3.229378, -0.489673, 0, 3.220052, -0.487370, 0, 3.220052, -0.487370, 0, 3.220052, -0.487370, 0, 3.210303, -0.485604, 0, 3.220052, -0.487370, 0, 3.220052, -0.487370, 0, 3.210303, -0.485604, 0, 3.210303, -0.485604, 0, 3.210303, -0.485604, 0, 3.200065, -0.484342, 0, 3.210303, -0.485604, 0, 3.210303, -0.485604, 0, 3.200065, -0.484342, 0, 3.200065, -0.484342, 0, 3.200065, -0.484342, 0, 3.189339, -0.483586, 0, 3.200065, -0.484342, 0, 3.200065, -0.484342, 0, 3.189339, -0.483586, 0, 3.189339, -0.483586, 0, 3.189339, -0.483586, 0, 3.178125, -0.483333, 0, 3.189339, -0.483586, 0, 3.189339, -0.483586, 0, 3.178125, -0.483333, 0, 3.178125, -0.483333, 0, 3.178125, -0.483333, 0, 3.139860, -0.486646, 0, 3.178125, -0.483333, 0, 3.178125, -0.483333, 0, 3.139860, -0.486646, 0, 3.139860, -0.486646, 0, 3.139860, -0.486646, 0, 3.106315, -0.496582, 0, 3.139860, -0.486646, 0, 3.139860, -0.486646, 0, 3.106315, -0.496582, 0, 3.106315, -0.496582, 0, 3.106315, -0.496582, 0, 3.077490, -0.513143, 0, 3.106315, -0.496582, 0, 3.106315, -0.496582, 0, 3.077490, -0.513143, 0, 3.077490, -0.513143, 0, 3.077490, -0.513143, 0, 3.053385, -0.536328, 0, 3.077490, -0.513143, 0, 3.077490, -0.513143, 0, 3.053385, -0.536328, 0, 3.053385, -0.536328, 0, 3.053385, -0.536328, 0, 3.034359, -0.565698, 0, 3.053385, -0.536328, 0, 3.053385, -0.536328, 0, 3.034359, -0.565698, 0, 3.034359, -0.565698, 0, 3.034359, -0.565698, 0, 3.020768, -0.600814, 0, 3.034359, -0.565698, 0, 3.034359, -0.565698, 0, 3.020768, -0.600814, 0, 3.020768, -0.600814, 0, 3.020768, -0.600814, 0, 3.012614, -0.641675, 0, 3.020768, -0.600814, 0, 3.020768, -0.600814, 0, 3.012614, -0.641675, 0, 3.012614, -0.641675, 0, 3.012614, -0.641675, 0, 3.009896, -0.688281, 0, 3.012614, -0.641675, 0, 3.012614, -0.641675, 0, 3.009896, -0.688281, 0, 3.009896, -0.688281, 0, 3.009896, -0.688281, 0, 3.009896, -1, 0, 3.009896, -0.688281, 0, 3.009896, -0.688281, 0, 3.009896, -1, 0, 3.009896, -1, 0, 3.009896, -1, 0, 2.913542, -1, 0, 3.009896, -1, 0, 3.009896, -1, 0, 2.913542, -1, 0, 2.913542, -1, 0, 2.913542, -1, 0, 2.913542, -0.416667, 0, 2.913542, -1, 0, 2.913542, -1, 0, 2.913542, -0.416667, 0, 2.913542, -0.416667, 0, 2.913542, -0.416667, 0, 3.009896, -0.416667, 0, 2.913542, -0.416667, 0, 2.913542, -0.416667, 0, 3.009896, -0.416667, 0, 3.009896, -0.416667, 0, 3.009896, -0.416667, 0, 3.009896, -0.500000, 0, 3.009896, -0.416667, 0, 3.009896, -0.416667, 0, 3.009896, -0.500000, 0, 3.009896, -0.500000, 0, 3.009896, -0.500000, 0, 3.026139, -0.476245, 0, 3.009896, -0.500000, 0, 3.009896, -0.500000, 0, 3.026139, -0.476245, 0, 3.026139, -0.476245, 0, 3.026139, -0.476245, 0, 3.044662, -0.455762, 0, 3.026139, -0.476245, 0, 3.026139, -0.476245, 0, 3.044662, -0.455762, 0, 3.044662, -0.455762, 0, 3.044662, -0.455762, 0, 3.065462, -0.438550, 0, 3.044662, -0.455762, 0, 3.044662, -0.455762, 0, 3.065462, -0.438550, 0, 3.065462, -0.438550, 0, 3.065462, -0.438550, 0, 3.088542, -0.424609, 0, 3.065462, -0.438550, 0, 3.065462, -0.438550, 0, 3.088542, -0.424609, 0, 3.088542, -0.424609, 0, 3.088542, -0.424609, 0, 3.114063, -0.413843, 0, 3.088542, -0.424609, 0, 3.088542, -0.424609, 0, 3.114063, -0.413843, 0, 3.114063, -0.413843, 0, 3.114063, -0.413843, 0, 3.142188, -0.406152, 0, 3.114063, -0.413843, 0, 3.114063, -0.413843, 0, 3.142188, -0.406152, 0, 3.142188, -0.406152, 0, 3.142188, -0.406152, 0, 3.172917, -0.401538, 0, 3.142188, -0.406152, 0, 3.142188, -0.406152, 0, 3.172917, -0.401538, 0, 3.172917, -0.401538, 0, 3.172917, -0.401538, 0, 3.206250, -0.400000, 0, 3.172917, -0.401538, 0, 3.172917, -0.401538, 0, 3.206250, -0.400000, 0, 3.206250, -0.400000, 0, 3.206250, -0.400000, 0, 3.211328, -0.400000, 0, 3.206250, -0.400000, 0, 3.206250, -0.400000, 0, 3.211328, -0.400000, 0, 3.211328, -0.400000, 0, 3.211328, -0.400000, 0, 3.216667, -0.400000, 0, 3.211328, -0.400000, 0, 3.211328, -0.400000, 0, 3.216667, -0.400000, 0, 3.216667, -0.400000, 0, 3.216667, -0.400000, 0, 3.222266, -0.400000, 0, 3.216667, -0.400000, 0, 3.216667, -0.400000, 0, 3.222266, -0.400000, 0, 3.222266, -0.400000, 0, 3.222266, -0.400000, 0, 3.228125, -0.400000, 0, 3.222266, -0.400000, 0, 3.222266, -0.400000, 0, 3.228125, -0.400000, 0, 3.228125, -0.400000, 0, 3.228125, -0.400000, 0, 3.234277, -0.400000, 0, 3.228125, -0.400000, 0, 3.228125, -0.400000, 0, 3.234277, -0.400000, 0, 3.234277, -0.400000, 0, 3.234277, -0.400000, 0, 3.240755, -0.400000, 0, 3.234277, -0.400000, 0, 3.234277, -0.400000, 0, 3.240755, -0.400000, 0, 3.240755, -0.400000, 0, 3.240755, -0.400000, 0, 3.247559, -0.400000, 0, 3.240755, -0.400000, 0, 3.240755, -0.400000, 0, 3.247559, -0.400000, 0, 3.247559, -0.400000, 0, 3.247559, -0.400000, 0, 3.254688, -0.400000, 0, 3.247559, -0.400000, 0, 3.247559, -0.400000, 0, 3.254688, -0.400000, 0, 3.254688, -0.400000, 0, 3.009896, -0.416667, 0, 2.913542, -0.416667, 0, 3.009896, -0.500000, 0, 2.913542, -0.416667, 0, 3.009896, -0.416667, 0, 3.009896, -0.500000, 0, 3.020768, -0.600814, 0, 3.009896, -0.500000, 0, 2.913542, -0.416667, 0, 3.009896, -0.500000, 0, 3.020768, -0.600814, 0, 2.913542, -0.416667, 0, 3.009896, -0.500000, 0, 3.020768, -0.600814, 0, 3.034359, -0.565698, 0, 3.020768, -0.600814, 0, 3.009896, -0.500000, 0, 3.034359, -0.565698, 0, 3.009896, -0.500000, 0, 3.034359, -0.565698, 0, 3.053385, -0.536328, 0, 3.034359, -0.565698, 0, 3.009896, -0.500000, 0, 3.053385, -0.536328, 0, 3.026139, -0.476245, 0, 3.009896, -0.500000, 0, 3.053385, -0.536328, 0, 3.009896, -0.500000, 0, 3.026139, -0.476245, 0, 3.053385, -0.536328, 0, 3.026139, -0.476245, 0, 3.053385, -0.536328, 0, 3.077490, -0.513143, 0, 3.053385, -0.536328, 0, 3.026139, -0.476245, 0, 3.077490, -0.513143, 0, 3.044662, -0.455762, 0, 3.026139, -0.476245, 0, 3.077490, -0.513143, 0, 3.026139, -0.476245, 0, 3.044662, -0.455762, 0, 3.077490, -0.513143, 0, 3.044662, -0.455762, 0, 3.077490, -0.513143, 0, 3.106315, -0.496582, 0, 3.077490, -0.513143, 0, 3.044662, -0.455762, 0, 3.106315, -0.496582, 0, 3.065462, -0.438550, 0, 3.044662, -0.455762, 0, 3.106315, -0.496582, 0, 3.044662, -0.455762, 0, 3.065462, -0.438550, 0, 3.106315, -0.496582, 0, 3.088542, -0.424609, 0, 3.065462, -0.438550, 0, 3.106315, -0.496582, 0, 3.065462, -0.438550, 0, 3.088542, -0.424609, 0, 3.106315, -0.496582, 0, 3.088542, -0.424609, 0, 3.106315, -0.496582, 0, 3.139860, -0.486646, 0, 3.106315, -0.496582, 0, 3.088542, -0.424609, 0, 3.139860, -0.486646, 0, 3.114063, -0.413843, 0, 3.088542, -0.424609, 0, 3.139860, -0.486646, 0, 3.088542, -0.424609, 0, 3.114063, -0.413843, 0, 3.139860, -0.486646, 0, 3.114063, -0.413843, 0, 3.139860, -0.486646, 0, 3.142188, -0.406152, 0, 3.139860, -0.486646, 0, 3.114063, -0.413843, 0, 3.142188, -0.406152, 0, 3.142188, -0.406152, 0, 3.139860, -0.486646, 0, 3.178125, -0.483333, 0, 3.139860, -0.486646, 0, 3.142188, -0.406152, 0, 3.178125, -0.483333, 0, 3.172917, -0.401538, 0, 3.142188, -0.406152, 0, 3.178125, -0.483333, 0, 3.142188, -0.406152, 0, 3.172917, -0.401538, 0, 3.178125, -0.483333, 0, 3.172917, -0.401538, 0, 3.178125, -0.483333, 0, 3.189339, -0.483586, 0, 3.178125, -0.483333, 0, 3.172917, -0.401538, 0, 3.189339, -0.483586, 0, 3.206250, -0.400000, 0, 3.172917, -0.401538, 0, 3.189339, -0.483586, 0, 3.172917, -0.401538, 0, 3.206250, -0.400000, 0, 3.189339, -0.483586, 0, 3.189339, -0.483586, 0, 3.200065, -0.484342, 0, 3.206250, -0.400000, 0, 3.200065, -0.484342, 0, 3.189339, -0.483586, 0, 3.206250, -0.400000, 0, 3.211328, -0.400000, 0, 3.206250, -0.400000, 0, 3.200065, -0.484342, 0, 3.206250, -0.400000, 0, 3.211328, -0.400000, 0, 3.200065, -0.484342, 0, 3.211328, -0.400000, 0, 3.200065, -0.484342, 0, 3.210303, -0.485604, 0, 3.200065, -0.484342, 0, 3.211328, -0.400000, 0, 3.210303, -0.485604, 0, 3.216667, -0.400000, 0, 3.211328, -0.400000, 0, 3.210303, -0.485604, 0, 3.211328, -0.400000, 0, 3.216667, -0.400000, 0, 3.210303, -0.485604, 0, 3.210303, -0.485604, 0, 3.222266, -0.400000, 0, 3.216667, -0.400000, 0, 3.222266, -0.400000, 0, 3.210303, -0.485604, 0, 3.216667, -0.400000, 0, 3.222266, -0.400000, 0, 3.210303, -0.485604, 0, 3.220052, -0.487370, 0, 3.210303, -0.485604, 0, 3.222266, -0.400000, 0, 3.220052, -0.487370, 0, 3.228125, -0.400000, 0, 3.222266, -0.400000, 0, 3.220052, -0.487370, 0, 3.222266, -0.400000, 0, 3.228125, -0.400000, 0, 3.220052, -0.487370, 0, 3.220052, -0.487370, 0, 3.234277, -0.400000, 0, 3.228125, -0.400000, 0, 3.234277, -0.400000, 0, 3.220052, -0.487370, 0, 3.228125, -0.400000, 0, 3.234277, -0.400000, 0, 3.220052, -0.487370, 0, 3.229378, -0.489673, 0, 3.220052, -0.487370, 0, 3.234277, -0.400000, 0, 3.229378, -0.489673, 0, 3.240755, -0.400000, 0, 3.234277, -0.400000, 0, 3.229378, -0.489673, 0, 3.234277, -0.400000, 0, 3.240755, -0.400000, 0, 3.229378, -0.489673, 0, 3.247559, -0.400000, 0, 3.240755, -0.400000, 0, 3.229378, -0.489673, 0, 3.240755, -0.400000, 0, 3.247559, -0.400000, 0, 3.229378, -0.489673, 0, 3.238347, -0.492546, 0, 3.247559, -0.400000, 0, 3.229378, -0.489673, 0, 3.247559, -0.400000, 0, 3.238347, -0.492546, 0, 3.229378, -0.489673, 0, 3.254688, -0.400000, 0, 3.247559, -0.400000, 0, 3.238347, -0.492546, 0, 3.247559, -0.400000, 0, 3.254688, -0.400000, 0, 3.238347, -0.492546, 0, 3.246956, -0.495988, 0, 3.254688, -0.400000, 0, 3.238347, -0.492546, 0, 3.254688, -0.400000, 0, 3.246956, -0.495988, 0, 3.238347, -0.492546, 0, 3.255208, -0.500000, 0, 3.254688, -0.400000, 0, 3.246956, -0.495988, 0, 3.254688, -0.400000, 0, 3.255208, -0.500000, 0, 3.246956, -0.495988, 0, 3.020768, -0.600814, 0, 2.913542, -0.416667, 0, 3.012614, -0.641675, 0, 2.913542, -0.416667, 0, 3.020768, -0.600814, 0, 3.012614, -0.641675, 0, 3.009896, -0.688281, 0, 3.012614, -0.641675, 0, 2.913542, -0.416667, 0, 3.012614, -0.641675, 0, 3.009896, -0.688281, 0, 2.913542, -0.416667, 0, 3.009896, -0.688281, 0, 2.913542, -0.416667, 0, 2.913542, -1, 0, 2.913542, -0.416667, 0, 3.009896, -0.688281, 0, 2.913542, -1, 0, 2.913542, -1, 0, 3.009896, -1, 0, 3.009896, -0.688281, 0, 3.009896, -1, 0, 2.913542, -1, 0, 3.009896, -0.688281, 0, 3.615625, -0.716667, 0, 3.562012, -0.718245, 0, 3.615625, -0.716667, 0, 3.615625, -0.716667, 0, 3.562012, -0.718245, 0, 3.562012, -0.718245, 0, 3.562012, -0.718245, 0, 3.517318, -0.722982, 0, 3.562012, -0.718245, 0, 3.562012, -0.718245, 0, 3.517318, -0.722982, 0, 3.517318, -0.722982, 0, 3.517318, -0.722982, 0, 3.481543, -0.730876, 0, 3.517318, -0.722982, 0, 3.517318, -0.722982, 0, 3.481543, -0.730876, 0, 3.481543, -0.730876, 0, 3.481543, -0.730876, 0, 3.454688, -0.741927, 0, 3.481543, -0.730876, 0, 3.481543, -0.730876, 0, 3.454688, -0.741927, 0, 3.454688, -0.741927, 0, 3.454688, -0.741927, 0, 3.435091, -0.756771, 0, 3.454688, -0.741927, 0, 3.454688, -0.741927, 0, 3.435091, -0.756771, 0, 3.435091, -0.756771, 0, 3.435091, -0.756771, 0, 3.421094, -0.776042, 0, 3.435091, -0.756771, 0, 3.435091, -0.756771, 0, 3.421094, -0.776042, 0, 3.421094, -0.776042, 0, 3.421094, -0.776042, 0, 3.412695, -0.799740, 0, 3.421094, -0.776042, 0, 3.421094, -0.776042, 0, 3.412695, -0.799740, 0, 3.412695, -0.799740, 0, 3.412695, -0.799740, 0, 3.409896, -0.827865, 0, 3.412695, -0.799740, 0, 3.412695, -0.799740, 0, 3.409896, -0.827865, 0, 3.409896, -0.827865, 0, 3.409896, -0.827865, 0, 3.411995, -0.850944, 0, 3.409896, -0.827865, 0, 3.409896, -0.827865, 0, 3.411995, -0.850944, 0, 3.411995, -0.850944, 0, 3.411995, -0.850944, 0, 3.418294, -0.871484, 0, 3.411995, -0.850944, 0, 3.411995, -0.850944, 0, 3.418294, -0.871484, 0, 3.418294, -0.871484, 0, 3.418294, -0.871484, 0, 3.428792, -0.889486, 0, 3.418294, -0.871484, 0, 3.418294, -0.871484, 0, 3.428792, -0.889486, 0, 3.428792, -0.889486, 0, 3.428792, -0.889486, 0, 3.443490, -0.904948, 0, 3.428792, -0.889486, 0, 3.428792, -0.889486, 0, 3.443490, -0.904948, 0, 3.443490, -0.904948, 0, 3.443490, -0.904948, 0, 3.461800, -0.917367, 0, 3.443490, -0.904948, 0, 3.443490, -0.904948, 0, 3.461800, -0.917367, 0, 3.461800, -0.917367, 0, 3.461800, -0.917367, 0, 3.483138, -0.926237, 0, 3.461800, -0.917367, 0, 3.461800, -0.917367, 0, 3.483138, -0.926237, 0, 3.483138, -0.926237, 0, 3.483138, -0.926237, 0, 3.507503, -0.931559, 0, 3.483138, -0.926237, 0, 3.483138, -0.926237, 0, 3.507503, -0.931559, 0, 3.507503, -0.931559, 0, 3.507503, -0.931559, 0, 3.534896, -0.933333, 0, 3.507503, -0.931559, 0, 3.507503, -0.931559, 0, 3.534896, -0.933333, 0, 3.534896, -0.933333, 0, 3.534896, -0.933333, 0, 3.572770, -0.929989, 0, 3.534896, -0.933333, 0, 3.534896, -0.933333, 0, 3.572770, -0.929989, 0, 3.572770, -0.929989, 0, 3.572770, -0.929989, 0, 3.606706, -0.919954, 0, 3.572770, -0.929989, 0, 3.572770, -0.929989, 0, 3.606706, -0.919954, 0, 3.606706, -0.919954, 0, 3.606706, -0.919954, 0, 3.636702, -0.903231, 0, 3.606706, -0.919954, 0, 3.606706, -0.919954, 0, 3.636702, -0.903231, 0, 3.636702, -0.903231, 0, 3.636702, -0.903231, 0, 3.662760, -0.879818, 0, 3.636702, -0.903231, 0, 3.636702, -0.903231, 0, 3.662760, -0.879818, 0, 3.662760, -0.879818, 0, 3.662760, -0.879818, 0, 3.683838, -0.850855, 0, 3.662760, -0.879818, 0, 3.662760, -0.879818, 0, 3.683838, -0.850855, 0, 3.683838, -0.850855, 0, 3.683838, -0.850855, 0, 3.698893, -0.817480, 0, 3.683838, -0.850855, 0, 3.683838, -0.850855, 0, 3.698893, -0.817480, 0, 3.698893, -0.817480, 0, 3.698893, -0.817480, 0, 3.707927, -0.779696, 0, 3.698893, -0.817480, 0, 3.698893, -0.817480, 0, 3.707927, -0.779696, 0, 3.707927, -0.779696, 0, 3.707927, -0.779696, 0, 3.710938, -0.737500, 0, 3.707927, -0.779696, 0, 3.707927, -0.779696, 0, 3.710938, -0.737500, 0, 3.710938, -0.737500, 0, 3.710938, -0.737500, 0, 3.710938, -0.716667, 0, 3.710938, -0.737500, 0, 3.710938, -0.737500, 0, 3.710938, -0.716667, 0, 3.710938, -0.716667, 0, 3.806771, -0.672656, 0, 3.806771, -1, 0, 3.806771, -0.672656, 0, 3.806771, -0.672656, 0, 3.806771, -1, 0, 3.806771, -1, 0, 3.806771, -1, 0, 3.710938, -1, 0, 3.806771, -1, 0, 3.806771, -1, 0, 3.710938, -1, 0, 3.710938, -1, 0, 3.710938, -1, 0, 3.710938, -0.916667, 0, 3.710938, -1, 0, 3.710938, -1, 0, 3.710938, -0.916667, 0, 3.710938, -0.916667, 0, 3.710938, -0.916667, 0, 3.693522, -0.940633, 0, 3.710938, -0.916667, 0, 3.710938, -0.916667, 0, 3.693522, -0.940633, 0, 3.693522, -0.940633, 0, 3.693522, -0.940633, 0, 3.674089, -0.961230, 0, 3.693522, -0.940633, 0, 3.693522, -0.940633, 0, 3.674089, -0.961230, 0, 3.674089, -0.961230, 0, 3.674089, -0.961230, 0, 3.652637, -0.978459, 0, 3.674089, -0.961230, 0, 3.674089, -0.961230, 0, 3.652637, -0.978459, 0, 3.652637, -0.978459, 0, 3.652637, -0.978459, 0, 3.629167, -0.992318, 0, 3.652637, -0.978459, 0, 3.652637, -0.978459, 0, 3.629167, -0.992318, 0, 3.629167, -0.992318, 0, 3.629167, -0.992318, 0, 3.603320, -1.002970, 0, 3.629167, -0.992318, 0, 3.629167, -0.992318, 0, 3.603320, -1.002970, 0, 3.603320, -1.002970, 0, 3.603320, -1.002970, 0, 3.574739, -1.010579, 0, 3.603320, -1.002970, 0, 3.603320, -1.002970, 0, 3.574739, -1.010579, 0, 3.574739, -1.010579, 0, 3.574739, -1.010579, 0, 3.543425, -1.015145, 0, 3.574739, -1.010579, 0, 3.574739, -1.010579, 0, 3.543425, -1.015145, 0, 3.543425, -1.015145, 0, 3.543425, -1.015145, 0, 3.509375, -1.016667, 0, 3.543425, -1.015145, 0, 3.543425, -1.015145, 0, 3.509375, -1.016667, 0, 3.509375, -1.016667, 0, 3.509375, -1.016667, 0, 3.466878, -1.013525, 0, 3.509375, -1.016667, 0, 3.509375, -1.016667, 0, 3.466878, -1.013525, 0, 3.466878, -1.013525, 0, 3.466878, -1.013525, 0, 3.428972, -1.004102, 0, 3.466878, -1.013525, 0, 3.466878, -1.013525, 0, 3.428972, -1.004102, 0, 3.428972, -1.004102, 0, 3.428972, -1.004102, 0, 3.395654, -0.988395, 0, 3.428972, -1.004102, 0, 3.428972, -1.004102, 0, 3.395654, -0.988395, 0, 3.395654, -0.988395, 0, 3.395654, -0.988395, 0, 3.366927, -0.966406, 0, 3.395654, -0.988395, 0, 3.395654, -0.988395, 0, 3.366927, -0.966406, 0, 3.366927, -0.966406, 0, 3.366927, -0.966406, 0, 3.343799, -0.939144, 0, 3.366927, -0.966406, 0, 3.366927, -0.966406, 0, 3.343799, -0.939144, 0, 3.343799, -0.939144, 0, 3.343799, -0.939144, 0, 3.327279, -0.907617, 0, 3.343799, -0.939144, 0, 3.343799, -0.939144, 0, 3.327279, -0.907617, 0, 3.327279, -0.907617, 0, 3.327279, -0.907617, 0, 3.317366, -0.871826, 0, 3.327279, -0.907617, 0, 3.327279, -0.907617, 0, 3.317366, -0.871826, 0, 3.317366, -0.871826, 0, 3.317366, -0.871826, 0, 3.314063, -0.831771, 0, 3.317366, -0.871826, 0, 3.317366, -0.871826, 0, 3.314063, -0.831771, 0, 3.314063, -0.831771, 0, 3.314063, -0.831771, 0, 3.318181, -0.785579, 0, 3.314063, -0.831771, 0, 3.314063, -0.831771, 0, 3.318181, -0.785579, 0, 3.318181, -0.785579, 0, 3.318181, -0.785579, 0, 3.330534, -0.745443, 0, 3.318181, -0.785579, 0, 3.318181, -0.785579, 0, 3.330534, -0.745443, 0, 3.330534, -0.745443, 0, 3.330534, -0.745443, 0, 3.351123, -0.711361, 0, 3.330534, -0.745443, 0, 3.330534, -0.745443, 0, 3.351123, -0.711361, 0, 3.351123, -0.711361, 0, 3.351123, -0.711361, 0, 3.379948, -0.683333, 0, 3.351123, -0.711361, 0, 3.351123, -0.711361, 0, 3.379948, -0.683333, 0, 3.379948, -0.683333, 0, 3.379948, -0.683333, 0, 3.416943, -0.661458, 0, 3.379948, -0.683333, 0, 3.379948, -0.683333, 0, 3.416943, -0.661458, 0, 3.416943, -0.661458, 0, 3.416943, -0.661458, 0, 3.462044, -0.645833, 0, 3.416943, -0.661458, 0, 3.416943, -0.661458, 0, 3.462044, -0.645833, 0, 3.462044, -0.645833, 0, 3.462044, -0.645833, 0, 3.515251, -0.636458, 0, 3.462044, -0.645833, 0, 3.462044, -0.645833, 0, 3.515251, -0.636458, 0, 3.515251, -0.636458, 0, 3.515251, -0.636458, 0, 3.576563, -0.633333, 0, 3.515251, -0.636458, 0, 3.515251, -0.636458, 0, 3.576563, -0.633333, 0, 3.576563, -0.633333, 0, 3.576563, -0.633333, 0, 3.710938, -0.633333, 0, 3.576563, -0.633333, 0, 3.576563, -0.633333, 0, 3.710938, -0.633333, 0, 3.710938, -0.633333, 0, 3.710938, -0.633333, 0, 3.710938, -0.623958, 0, 3.710938, -0.633333, 0, 3.710938, -0.633333, 0, 3.710938, -0.623958, 0, 3.710938, -0.623958, 0, 3.710938, -0.623958, 0, 3.708220, -0.592269, 0, 3.710938, -0.623958, 0, 3.710938, -0.623958, 0, 3.708220, -0.592269, 0, 3.708220, -0.592269, 0, 3.708220, -0.592269, 0, 3.700065, -0.564388, 0, 3.708220, -0.592269, 0, 3.708220, -0.592269, 0, 3.700065, -0.564388, 0, 3.700065, -0.564388, 0, 3.700065, -0.564388, 0, 3.686475, -0.540316, 0, 3.700065, -0.564388, 0, 3.700065, -0.564388, 0, 3.686475, -0.540316, 0, 3.686475, -0.540316, 0, 3.686475, -0.540316, 0, 3.667448, -0.520052, 0, 3.686475, -0.540316, 0, 3.686475, -0.540316, 0, 3.667448, -0.520052, 0, 3.667448, -0.520052, 0, 3.667448, -0.520052, 0, 3.643506, -0.503988, 0, 3.667448, -0.520052, 0, 3.667448, -0.520052, 0, 3.643506, -0.503988, 0, 3.643506, -0.503988, 0, 3.643506, -0.503988, 0, 3.615169, -0.492513, 0, 3.643506, -0.503988, 0, 3.643506, -0.503988, 0, 3.615169, -0.492513, 0, 3.615169, -0.492513, 0, 3.615169, -0.492513, 0, 3.582438, -0.485628, 0, 3.615169, -0.492513, 0, 3.615169, -0.492513, 0, 3.582438, -0.485628, 0, 3.582438, -0.485628, 0, 3.582438, -0.485628, 0, 3.545313, -0.483333, 0, 3.582438, -0.485628, 0, 3.582438, -0.485628, 0, 3.545313, -0.483333, 0, 3.545313, -0.483333, 0, 3.545313, -0.483333, 0, 3.520475, -0.484115, 0, 3.545313, -0.483333, 0, 3.545313, -0.483333, 0, 3.520475, -0.484115, 0, 3.520475, -0.484115, 0, 3.520475, -0.484115, 0, 3.495964, -0.486458, 0, 3.520475, -0.484115, 0, 3.520475, -0.484115, 0, 3.495964, -0.486458, 0, 3.495964, -0.486458, 0, 3.495964, -0.486458, 0, 3.471777, -0.490365, 0, 3.495964, -0.486458, 0, 3.495964, -0.486458, 0, 3.471777, -0.490365, 0, 3.471777, -0.490365, 0, 3.471777, -0.490365, 0, 3.447917, -0.495833, 0, 3.471777, -0.490365, 0, 3.471777, -0.490365, 0, 3.447917, -0.495833, 0, 3.447917, -0.495833, 0, 3.447917, -0.495833, 0, 3.424447, -0.502865, 0, 3.447917, -0.495833, 0, 3.447917, -0.495833, 0, 3.424447, -0.502865, 0, 3.424447, -0.502865, 0, 3.424447, -0.502865, 0, 3.401433, -0.511458, 0, 3.424447, -0.502865, 0, 3.424447, -0.502865, 0, 3.401433, -0.511458, 0, 3.401433, -0.511458, 0, 3.401433, -0.511458, 0, 3.378874, -0.521615, 0, 3.401433, -0.511458, 0, 3.401433, -0.511458, 0, 3.378874, -0.521615, 0, 3.378874, -0.521615, 0, 3.378874, -0.521615, 0, 3.356771, -0.533333, 0, 3.378874, -0.521615, 0, 3.378874, -0.521615, 0, 3.356771, -0.533333, 0, 3.356771, -0.533333, 0, 3.356771, -0.533333, 0, 3.356771, -0.450000, 0, 3.356771, -0.533333, 0, 3.356771, -0.533333, 0, 3.356771, -0.450000, 0, 3.356771, -0.450000, 0, 3.356771, -0.450000, 0, 3.382878, -0.438281, 0, 3.356771, -0.450000, 0, 3.356771, -0.450000, 0, 3.382878, -0.438281, 0, 3.382878, -0.438281, 0, 3.382878, -0.438281, 0, 3.408594, -0.428125, 0, 3.382878, -0.438281, 0, 3.382878, -0.438281, 0, 3.408594, -0.428125, 0, 3.408594, -0.428125, 0, 3.408594, -0.428125, 0, 3.433919, -0.419531, 0, 3.408594, -0.428125, 0, 3.408594, -0.428125, 0, 3.433919, -0.419531, 0, 3.433919, -0.419531, 0, 3.433919, -0.419531, 0, 3.458854, -0.412500, 0, 3.433919, -0.419531, 0, 3.433919, -0.419531, 0, 3.458854, -0.412500, 0, 3.458854, -0.412500, 0, 3.458854, -0.412500, 0, 3.483431, -0.407031, 0, 3.458854, -0.412500, 0, 3.458854, -0.412500, 0, 3.483431, -0.407031, 0, 3.483431, -0.407031, 0, 3.483431, -0.407031, 0, 3.507683, -0.403125, 0, 3.483431, -0.407031, 0, 3.483431, -0.407031, 0, 3.507683, -0.403125, 0, 3.507683, -0.403125, 0, 3.507683, -0.403125, 0, 3.531608, -0.400781, 0, 3.507683, -0.403125, 0, 3.507683, -0.403125, 0, 3.531608, -0.400781, 0, 3.531608, -0.400781, 0, 3.531608, -0.400781, 0, 3.555208, -0.400000, 0, 3.531608, -0.400781, 0, 3.531608, -0.400781, 0, 3.555208, -0.400000, 0, 3.555208, -0.400000, 0, 3.555208, -0.400000, 0, 3.614486, -0.404224, 0, 3.555208, -0.400000, 0, 3.555208, -0.400000, 0, 3.614486, -0.404224, 0, 3.614486, -0.404224, 0, 3.614486, -0.404224, 0, 3.665755, -0.416894, 0, 3.614486, -0.404224, 0, 3.614486, -0.404224, 0, 3.665755, -0.416894, 0, 3.665755, -0.416894, 0, 3.665755, -0.416894, 0, 3.709017, -0.438013, 0, 3.665755, -0.416894, 0, 3.665755, -0.416894, 0, 3.709017, -0.438013, 0, 3.709017, -0.438013, 0, 3.709017, -0.438013, 0, 3.744271, -0.467578, 0, 3.709017, -0.438013, 0, 3.709017, -0.438013, 0, 3.744271, -0.467578, 0, 3.744271, -0.467578, 0, 3.744271, -0.467578, 0, 3.771615, -0.505737, 0, 3.744271, -0.467578, 0, 3.744271, -0.467578, 0, 3.771615, -0.505737, 0, 3.771615, -0.505737, 0, 3.771615, -0.505737, 0, 3.791146, -0.552637, 0, 3.771615, -0.505737, 0, 3.771615, -0.505737, 0, 3.791146, -0.552637, 0, 3.791146, -0.552637, 0, 3.791146, -0.552637, 0, 3.802865, -0.608276, 0, 3.791146, -0.552637, 0, 3.791146, -0.552637, 0, 3.802865, -0.608276, 0, 3.802865, -0.608276, 0, 3.318181, -0.785579, 0, 3.314063, -0.831771, 0, 3.409896, -0.827865, 0, 3.314063, -0.831771, 0, 3.318181, -0.785579, 0, 3.409896, -0.827865, 0, 3.318181, -0.785579, 0, 3.409896, -0.827865, 0, 3.412695, -0.799740, 0, 3.409896, -0.827865, 0, 3.318181, -0.785579, 0, 3.412695, -0.799740, 0, 3.330534, -0.745443, 0, 3.318181, -0.785579, 0, 3.412695, -0.799740, 0, 3.318181, -0.785579, 0, 3.330534, -0.745443, 0, 3.412695, -0.799740, 0, 3.330534, -0.745443, 0, 3.412695, -0.799740, 0, 3.421094, -0.776042, 0, 3.412695, -0.799740, 0, 3.330534, -0.745443, 0, 3.421094, -0.776042, 0, 3.351123, -0.711361, 0, 3.330534, -0.745443, 0, 3.421094, -0.776042, 0, 3.330534, -0.745443, 0, 3.351123, -0.711361, 0, 3.421094, -0.776042, 0, 3.351123, -0.711361, 0, 3.421094, -0.776042, 0, 3.435091, -0.756771, 0, 3.421094, -0.776042, 0, 3.351123, -0.711361, 0, 3.435091, -0.756771, 0, 3.379948, -0.683333, 0, 3.351123, -0.711361, 0, 3.435091, -0.756771, 0, 3.351123, -0.711361, 0, 3.379948, -0.683333, 0, 3.435091, -0.756771, 0, 3.379948, -0.683333, 0, 3.435091, -0.756771, 0, 3.454688, -0.741927, 0, 3.435091, -0.756771, 0, 3.379948, -0.683333, 0, 3.454688, -0.741927, 0, 3.416943, -0.661458, 0, 3.379948, -0.683333, 0, 3.454688, -0.741927, 0, 3.379948, -0.683333, 0, 3.416943, -0.661458, 0, 3.454688, -0.741927, 0, 3.481543, -0.730876, 0, 3.416943, -0.661458, 0, 3.454688, -0.741927, 0, 3.416943, -0.661458, 0, 3.481543, -0.730876, 0, 3.454688, -0.741927, 0, 3.462044, -0.645833, 0, 3.416943, -0.661458, 0, 3.481543, -0.730876, 0, 3.416943, -0.661458, 0, 3.462044, -0.645833, 0, 3.481543, -0.730876, 0, 3.517318, -0.722982, 0, 3.462044, -0.645833, 0, 3.481543, -0.730876, 0, 3.462044, -0.645833, 0, 3.517318, -0.722982, 0, 3.481543, -0.730876, 0, 3.515251, -0.636458, 0, 3.462044, -0.645833, 0, 3.517318, -0.722982, 0, 3.462044, -0.645833, 0, 3.515251, -0.636458, 0, 3.517318, -0.722982, 0, 3.515251, -0.636458, 0, 3.517318, -0.722982, 0, 3.562012, -0.718245, 0, 3.517318, -0.722982, 0, 3.515251, -0.636458, 0, 3.562012, -0.718245, 0, 3.515251, -0.636458, 0, 3.562012, -0.718245, 0, 3.576563, -0.633333, 0, 3.562012, -0.718245, 0, 3.515251, -0.636458, 0, 3.576563, -0.633333, 0, 3.576563, -0.633333, 0, 3.562012, -0.718245, 0, 3.615625, -0.716667, 0, 3.562012, -0.718245, 0, 3.576563, -0.633333, 0, 3.615625, -0.716667, 0, 3.576563, -0.633333, 0, 3.615625, -0.716667, 0, 3.710938, -0.633333, 0, 3.615625, -0.716667, 0, 3.576563, -0.633333, 0, 3.710938, -0.633333, 0, 3.615625, -0.716667, 0, 3.710938, -0.716667, 0, 3.710938, -0.633333, 0, 3.710938, -0.716667, 0, 3.615625, -0.716667, 0, 3.710938, -0.633333, 0, 3.710938, -0.633333, 0, 3.710938, -0.716667, 0, 3.806771, -0.672656, 0, 3.710938, -0.716667, 0, 3.710938, -0.633333, 0, 3.806771, -0.672656, 0, 3.806771, -0.672656, 0, 3.802865, -0.608276, 0, 3.710938, -0.633333, 0, 3.802865, -0.608276, 0, 3.806771, -0.672656, 0, 3.710938, -0.633333, 0, 3.802865, -0.608276, 0, 3.710938, -0.623958, 0, 3.710938, -0.633333, 0, 3.710938, -0.623958, 0, 3.802865, -0.608276, 0, 3.710938, -0.633333, 0, 3.708220, -0.592269, 0, 3.710938, -0.623958, 0, 3.802865, -0.608276, 0, 3.710938, -0.623958, 0, 3.708220, -0.592269, 0, 3.802865, -0.608276, 0, 3.791146, -0.552637, 0, 3.708220, -0.592269, 0, 3.802865, -0.608276, 0, 3.708220, -0.592269, 0, 3.791146, -0.552637, 0, 3.802865, -0.608276, 0, 3.700065, -0.564388, 0, 3.708220, -0.592269, 0, 3.791146, -0.552637, 0, 3.708220, -0.592269, 0, 3.700065, -0.564388, 0, 3.791146, -0.552637, 0, 3.771615, -0.505737, 0, 3.700065, -0.564388, 0, 3.791146, -0.552637, 0, 3.700065, -0.564388, 0, 3.771615, -0.505737, 0, 3.791146, -0.552637, 0, 3.686475, -0.540316, 0, 3.700065, -0.564388, 0, 3.771615, -0.505737, 0, 3.700065, -0.564388, 0, 3.686475, -0.540316, 0, 3.771615, -0.505737, 0, 3.744271, -0.467578, 0, 3.686475, -0.540316, 0, 3.771615, -0.505737, 0, 3.686475, -0.540316, 0, 3.744271, -0.467578, 0, 3.771615, -0.505737, 0, 3.667448, -0.520052, 0, 3.686475, -0.540316, 0, 3.744271, -0.467578, 0, 3.686475, -0.540316, 0, 3.667448, -0.520052, 0, 3.744271, -0.467578, 0, 3.709017, -0.438013, 0, 3.667448, -0.520052, 0, 3.744271, -0.467578, 0, 3.667448, -0.520052, 0, 3.709017, -0.438013, 0, 3.744271, -0.467578, 0, 3.643506, -0.503988, 0, 3.667448, -0.520052, 0, 3.709017, -0.438013, 0, 3.667448, -0.520052, 0, 3.643506, -0.503988, 0, 3.709017, -0.438013, 0, 3.665755, -0.416894, 0, 3.643506, -0.503988, 0, 3.709017, -0.438013, 0, 3.643506, -0.503988, 0, 3.665755, -0.416894, 0, 3.709017, -0.438013, 0, 3.615169, -0.492513, 0, 3.643506, -0.503988, 0, 3.665755, -0.416894, 0, 3.643506, -0.503988, 0, 3.615169, -0.492513, 0, 3.665755, -0.416894, 0, 3.614486, -0.404224, 0, 3.615169, -0.492513, 0, 3.665755, -0.416894, 0, 3.615169, -0.492513, 0, 3.614486, -0.404224, 0, 3.665755, -0.416894, 0, 3.614486, -0.404224, 0, 3.582438, -0.485628, 0, 3.615169, -0.492513, 0, 3.582438, -0.485628, 0, 3.614486, -0.404224, 0, 3.615169, -0.492513, 0, 3.614486, -0.404224, 0, 3.555208, -0.400000, 0, 3.582438, -0.485628, 0, 3.555208, -0.400000, 0, 3.614486, -0.404224, 0, 3.582438, -0.485628, 0, 3.545313, -0.483333, 0, 3.582438, -0.485628, 0, 3.555208, -0.400000, 0, 3.582438, -0.485628, 0, 3.545313, -0.483333, 0, 3.555208, -0.400000, 0, 3.555208, -0.400000, 0, 3.531608, -0.400781, 0, 3.545313, -0.483333, 0, 3.531608, -0.400781, 0, 3.555208, -0.400000, 0, 3.545313, -0.483333, 0, 3.531608, -0.400781, 0, 3.520475, -0.484115, 0, 3.545313, -0.483333, 0, 3.520475, -0.484115, 0, 3.531608, -0.400781, 0, 3.545313, -0.483333, 0, 3.507683, -0.403125, 0, 3.520475, -0.484115, 0, 3.531608, -0.400781, 0, 3.520475, -0.484115, 0, 3.507683, -0.403125, 0, 3.531608, -0.400781, 0, 3.507683, -0.403125, 0, 3.495964, -0.486458, 0, 3.520475, -0.484115, 0, 3.495964, -0.486458, 0, 3.507683, -0.403125, 0, 3.520475, -0.484115, 0, 3.483431, -0.407031, 0, 3.495964, -0.486458, 0, 3.507683, -0.403125, 0, 3.495964, -0.486458, 0, 3.483431, -0.407031, 0, 3.507683, -0.403125, 0, 3.483431, -0.407031, 0, 3.458854, -0.412500, 0, 3.495964, -0.486458, 0, 3.458854, -0.412500, 0, 3.483431, -0.407031, 0, 3.495964, -0.486458, 0, 3.495964, -0.486458, 0, 3.458854, -0.412500, 0, 3.471777, -0.490365, 0, 3.458854, -0.412500, 0, 3.495964, -0.486458, 0, 3.471777, -0.490365, 0, 3.458854, -0.412500, 0, 3.433919, -0.419531, 0, 3.471777, -0.490365, 0, 3.433919, -0.419531, 0, 3.458854, -0.412500, 0, 3.471777, -0.490365, 0, 3.433919, -0.419531, 0, 3.447917, -0.495833, 0, 3.471777, -0.490365, 0, 3.447917, -0.495833, 0, 3.433919, -0.419531, 0, 3.471777, -0.490365, 0, 3.433919, -0.419531, 0, 3.408594, -0.428125, 0, 3.447917, -0.495833, 0, 3.408594, -0.428125, 0, 3.433919, -0.419531, 0, 3.447917, -0.495833, 0, 3.408594, -0.428125, 0, 3.424447, -0.502865, 0, 3.447917, -0.495833, 0, 3.424447, -0.502865, 0, 3.408594, -0.428125, 0, 3.447917, -0.495833, 0, 3.408594, -0.428125, 0, 3.382878, -0.438281, 0, 3.424447, -0.502865, 0, 3.382878, -0.438281, 0, 3.408594, -0.428125, 0, 3.424447, -0.502865, 0, 3.382878, -0.438281, 0, 3.401433, -0.511458, 0, 3.424447, -0.502865, 0, 3.401433, -0.511458, 0, 3.382878, -0.438281, 0, 3.424447, -0.502865, 0, 3.382878, -0.438281, 0, 3.356771, -0.450000, 0, 3.401433, -0.511458, 0, 3.356771, -0.450000, 0, 3.382878, -0.438281, 0, 3.401433, -0.511458, 0, 3.356771, -0.450000, 0, 3.378874, -0.521615, 0, 3.401433, -0.511458, 0, 3.378874, -0.521615, 0, 3.356771, -0.450000, 0, 3.401433, -0.511458, 0, 3.356771, -0.450000, 0, 3.356771, -0.533333, 0, 3.378874, -0.521615, 0, 3.356771, -0.533333, 0, 3.356771, -0.450000, 0, 3.378874, -0.521615, 0, 3.806771, -0.672656, 0, 3.710938, -0.716667, 0, 3.710938, -0.737500, 0, 3.710938, -0.716667, 0, 3.806771, -0.672656, 0, 3.710938, -0.737500, 0, 3.806771, -0.672656, 0, 3.710938, -0.737500, 0, 3.707927, -0.779696, 0, 3.710938, -0.737500, 0, 3.806771, -0.672656, 0, 3.707927, -0.779696, 0, 3.806771, -1, 0, 3.806771, -0.672656, 0, 3.707927, -0.779696, 0, 3.806771, -0.672656, 0, 3.806771, -1, 0, 3.707927, -0.779696, 0, 3.806771, -1, 0, 3.707927, -0.779696, 0, 3.698893, -0.817480, 0, 3.707927, -0.779696, 0, 3.806771, -1, 0, 3.698893, -0.817480, 0, 3.710938, -0.916667, 0, 3.806771, -1, 0, 3.698893, -0.817480, 0, 3.806771, -1, 0, 3.710938, -0.916667, 0, 3.698893, -0.817480, 0, 3.710938, -0.916667, 0, 3.710938, -1, 0, 3.806771, -1, 0, 3.710938, -1, 0, 3.710938, -0.916667, 0, 3.806771, -1, 0, 3.698893, -0.817480, 0, 3.683838, -0.850855, 0, 3.710938, -0.916667, 0, 3.683838, -0.850855, 0, 3.698893, -0.817480, 0, 3.710938, -0.916667, 0, 3.683838, -0.850855, 0, 3.662760, -0.879818, 0, 3.710938, -0.916667, 0, 3.662760, -0.879818, 0, 3.683838, -0.850855, 0, 3.710938, -0.916667, 0, 3.662760, -0.879818, 0, 3.693522, -0.940633, 0, 3.710938, -0.916667, 0, 3.693522, -0.940633, 0, 3.662760, -0.879818, 0, 3.710938, -0.916667, 0, 3.662760, -0.879818, 0, 3.636702, -0.903231, 0, 3.693522, -0.940633, 0, 3.636702, -0.903231, 0, 3.662760, -0.879818, 0, 3.693522, -0.940633, 0, 3.636702, -0.903231, 0, 3.674089, -0.961230, 0, 3.693522, -0.940633, 0, 3.674089, -0.961230, 0, 3.636702, -0.903231, 0, 3.693522, -0.940633, 0, 3.674089, -0.961230, 0, 3.636702, -0.903231, 0, 3.652637, -0.978459, 0, 3.636702, -0.903231, 0, 3.674089, -0.961230, 0, 3.652637, -0.978459, 0, 3.636702, -0.903231, 0, 3.606706, -0.919954, 0, 3.652637, -0.978459, 0, 3.606706, -0.919954, 0, 3.636702, -0.903231, 0, 3.652637, -0.978459, 0, 3.606706, -0.919954, 0, 3.629167, -0.992318, 0, 3.652637, -0.978459, 0, 3.629167, -0.992318, 0, 3.606706, -0.919954, 0, 3.652637, -0.978459, 0, 3.629167, -0.992318, 0, 3.606706, -0.919954, 0, 3.603320, -1.002970, 0, 3.606706, -0.919954, 0, 3.629167, -0.992318, 0, 3.603320, -1.002970, 0, 3.606706, -0.919954, 0, 3.572770, -0.929989, 0, 3.603320, -1.002970, 0, 3.572770, -0.929989, 0, 3.606706, -0.919954, 0, 3.603320, -1.002970, 0, 3.572770, -0.929989, 0, 3.574739, -1.010579, 0, 3.603320, -1.002970, 0, 3.574739, -1.010579, 0, 3.572770, -0.929989, 0, 3.603320, -1.002970, 0, 3.572770, -0.929989, 0, 3.534896, -0.933333, 0, 3.574739, -1.010579, 0, 3.534896, -0.933333, 0, 3.572770, -0.929989, 0, 3.574739, -1.010579, 0, 3.534896, -0.933333, 0, 3.543425, -1.015145, 0, 3.574739, -1.010579, 0, 3.543425, -1.015145, 0, 3.534896, -0.933333, 0, 3.574739, -1.010579, 0, 3.534896, -0.933333, 0, 3.509375, -1.016667, 0, 3.543425, -1.015145, 0, 3.509375, -1.016667, 0, 3.534896, -0.933333, 0, 3.543425, -1.015145, 0, 3.507503, -0.931559, 0, 3.509375, -1.016667, 0, 3.534896, -0.933333, 0, 3.509375, -1.016667, 0, 3.507503, -0.931559, 0, 3.534896, -0.933333, 0, 3.507503, -0.931559, 0, 3.466878, -1.013525, 0, 3.509375, -1.016667, 0, 3.466878, -1.013525, 0, 3.507503, -0.931559, 0, 3.509375, -1.016667, 0, 3.483138, -0.926237, 0, 3.466878, -1.013525, 0, 3.507503, -0.931559, 0, 3.466878, -1.013525, 0, 3.483138, -0.926237, 0, 3.507503, -0.931559, 0, 3.428972, -1.004102, 0, 3.466878, -1.013525, 0, 3.483138, -0.926237, 0, 3.466878, -1.013525, 0, 3.428972, -1.004102, 0, 3.483138, -0.926237, 0, 3.461800, -0.917367, 0, 3.428972, -1.004102, 0, 3.483138, -0.926237, 0, 3.428972, -1.004102, 0, 3.461800, -0.917367, 0, 3.483138, -0.926237, 0, 3.395654, -0.988395, 0, 3.428972, -1.004102, 0, 3.461800, -0.917367, 0, 3.428972, -1.004102, 0, 3.395654, -0.988395, 0, 3.461800, -0.917367, 0, 3.461800, -0.917367, 0, 3.443490, -0.904948, 0, 3.395654, -0.988395, 0, 3.443490, -0.904948, 0, 3.461800, -0.917367, 0, 3.395654, -0.988395, 0, 3.443490, -0.904948, 0, 3.366927, -0.966406, 0, 3.395654, -0.988395, 0, 3.366927, -0.966406, 0, 3.443490, -0.904948, 0, 3.395654, -0.988395, 0, 3.443490, -0.904948, 0, 3.428792, -0.889486, 0, 3.366927, -0.966406, 0, 3.428792, -0.889486, 0, 3.443490, -0.904948, 0, 3.366927, -0.966406, 0, 3.428792, -0.889486, 0, 3.343799, -0.939144, 0, 3.366927, -0.966406, 0, 3.343799, -0.939144, 0, 3.428792, -0.889486, 0, 3.366927, -0.966406, 0, 3.428792, -0.889486, 0, 3.418294, -0.871484, 0, 3.343799, -0.939144, 0, 3.418294, -0.871484, 0, 3.428792, -0.889486, 0, 3.343799, -0.939144, 0, 3.418294, -0.871484, 0, 3.327279, -0.907617, 0, 3.343799, -0.939144, 0, 3.327279, -0.907617, 0, 3.418294, -0.871484, 0, 3.343799, -0.939144, 0, 3.317366, -0.871826, 0, 3.327279, -0.907617, 0, 3.418294, -0.871484, 0, 3.327279, -0.907617, 0, 3.317366, -0.871826, 0, 3.418294, -0.871484, 0, 3.411995, -0.850944, 0, 3.317366, -0.871826, 0, 3.418294, -0.871484, 0, 3.317366, -0.871826, 0, 3.411995, -0.850944, 0, 3.418294, -0.871484, 0, 3.314063, -0.831771, 0, 3.317366, -0.871826, 0, 3.411995, -0.850944, 0, 3.317366, -0.871826, 0, 3.314063, -0.831771, 0, 3.411995, -0.850944, 0, 3.409896, -0.827865, 0, 3.314063, -0.831771, 0, 3.411995, -0.850944, 0, 3.314063, -0.831771, 0, 3.409896, -0.827865, 0, 3.411995, -0.850944, 0, 4.000521, -0.183333, 0, 4.096354, -0.183333, 0, 4.000521, -0.183333, 0, 4.000521, -0.183333, 0, 4.096354, -0.183333, 0, 4.096354, -0.183333, 0, 4.096354, -0.183333, 0, 4.096354, -1, 0, 4.096354, -0.183333, 0, 4.096354, -0.183333, 0, 4.096354, -1, 0, 4.096354, -1, 0, 4.096354, -1, 0, 4.000521, -1, 0, 4.096354, -1, 0, 4.096354, -1, 0, 4.000521, -1, 0, 4.000521, -1, 0, 4.000521, -0.183333, 0, 4.096354, -1, 0, 4.096354, -0.183333, 0, 4.096354, -1, 0, 4.000521, -0.183333, 0, 4.096354, -0.183333, 0, 4.000521, -0.183333, 0, 4.000521, -1, 0, 4.096354, -1, 0, 4.000521, -1, 0, 4.000521, -0.183333, 0, 4.096354, -1, 0, 4.743229, -0.300000, 0, 4.743229, -0.600000, 0, 4.743229, -0.300000, 0, 4.743229, -0.300000, 0, 4.743229, -0.600000, 0, 4.743229, -0.600000, 0, 4.743229, -0.600000, 0, 4.875521, -0.600000, 0, 4.743229, -0.600000, 0, 4.743229, -0.600000, 0, 4.875521, -0.600000, 0, 4.875521, -0.600000, 0, 4.875521, -0.600000, 0, 4.910156, -0.597559, 0, 4.875521, -0.600000, 0, 4.875521, -0.600000, 0, 4.910156, -0.597559, 0, 4.910156, -0.597559, 0, 4.910156, -0.597559, 0, 4.940625, -0.590234, 0, 4.910156, -0.597559, 0, 4.910156, -0.597559, 0, 4.940625, -0.590234, 0, 4.940625, -0.590234, 0, 4.940625, -0.590234, 0, 4.966927, -0.578027, 0, 4.940625, -0.590234, 0, 4.940625, -0.590234, 0, 4.966927, -0.578027, 0, 4.966927, -0.578027, 0, 4.966927, -0.578027, 0, 4.989063, -0.560938, 0, 4.966927, -0.578027, 0, 4.966927, -0.578027, 0, 4.989063, -0.560938, 0, 4.989063, -0.560938, 0, 4.989063, -0.560938, 0, 5.006608, -0.539339, 0, 4.989063, -0.560938, 0, 4.989063, -0.560938, 0, 5.006608, -0.539339, 0, 5.006608, -0.539339, 0, 5.006608, -0.539339, 0, 5.019141, -0.513607, 0, 5.006608, -0.539339, 0, 5.006608, -0.539339, 0, 5.019141, -0.513607, 0, 5.019141, -0.513607, 0, 5.019141, -0.513607, 0, 5.026660, -0.483740, 0, 5.019141, -0.513607, 0, 5.019141, -0.513607, 0, 5.026660, -0.483740, 0, 5.026660, -0.483740, 0, 5.026660, -0.483740, 0, 5.029167, -0.449740, 0, 5.026660, -0.483740, 0, 5.026660, -0.483740, 0, 5.029167, -0.449740, 0, 5.029167, -0.449740, 0, 5.029167, -0.449740, 0, 5.026660, -0.415967, 0, 5.029167, -0.449740, 0, 5.029167, -0.449740, 0, 5.026660, -0.415967, 0, 5.026660, -0.415967, 0, 5.026660, -0.415967, 0, 5.019141, -0.386263, 0, 5.026660, -0.415967, 0, 5.026660, -0.415967, 0, 5.019141, -0.386263, 0, 5.019141, -0.386263, 0, 5.019141, -0.386263, 0, 5.006608, -0.360628, 0, 5.019141, -0.386263, 0, 5.019141, -0.386263, 0, 5.006608, -0.360628, 0, 5.006608, -0.360628, 0, 5.006608, -0.360628, 0, 4.989063, -0.339063, 0, 5.006608, -0.360628, 0, 5.006608, -0.360628, 0, 4.989063, -0.339063, 0, 4.989063, -0.339063, 0, 4.989063, -0.339063, 0, 4.966927, -0.321973, 0, 4.989063, -0.339063, 0, 4.989063, -0.339063, 0, 4.966927, -0.321973, 0, 4.966927, -0.321973, 0, 4.966927, -0.321973, 0, 4.940625, -0.309766, 0, 4.966927, -0.321973, 0, 4.966927, -0.321973, 0, 4.940625, -0.309766, 0, 4.940625, -0.309766, 0, 4.940625, -0.309766, 0, 4.910156, -0.302441, 0, 4.940625, -0.309766, 0, 4.940625, -0.309766, 0, 4.910156, -0.302441, 0, 4.910156, -0.302441, 0, 4.910156, -0.302441, 0, 4.875521, -0.300000, 0, 4.910156, -0.302441, 0, 4.910156, -0.302441, 0, 4.875521, -0.300000, 0, 4.875521, -0.300000, 0, 4.638021, -0.216667, 0, 4.875521, -0.216667, 0, 4.638021, -0.216667, 0, 4.638021, -0.216667, 0, 4.875521, -0.216667, 0, 4.875521, -0.216667, 0, 4.875521, -0.216667, 0, 4.936898, -0.220378, 0, 4.875521, -0.216667, 0, 4.875521, -0.216667, 0, 4.936898, -0.220378, 0, 4.936898, -0.220378, 0, 4.936898, -0.220378, 0, 4.990300, -0.231510, 0, 4.936898, -0.220378, 0, 4.936898, -0.220378, 0, 4.990300, -0.231510, 0, 4.990300, -0.231510, 0, 4.990300, -0.231510, 0, 5.035726, -0.250065, 0, 4.990300, -0.231510, 0, 4.990300, -0.231510, 0, 5.035726, -0.250065, 0, 5.035726, -0.250065, 0, 5.035726, -0.250065, 0, 5.073177, -0.276042, 0, 5.035726, -0.250065, 0, 5.035726, -0.250065, 0, 5.073177, -0.276042, 0, 5.073177, -0.276042, 0, 5.073177, -0.276042, 0, 5.102458, -0.309163, 0, 5.073177, -0.276042, 0, 5.073177, -0.276042, 0, 5.102458, -0.309163, 0, 5.102458, -0.309163, 0, 5.102458, -0.309163, 0, 5.123373, -0.349154, 0, 5.102458, -0.309163, 0, 5.102458, -0.309163, 0, 5.123373, -0.349154, 0, 5.123373, -0.349154, 0, 5.123373, -0.349154, 0, 5.135921, -0.396012, 0, 5.123373, -0.349154, 0, 5.123373, -0.349154, 0, 5.135921, -0.396012, 0, 5.135921, -0.396012, 0, 5.135921, -0.396012, 0, 5.140104, -0.449740, 0, 5.135921, -0.396012, 0, 5.135921, -0.396012, 0, 5.140104, -0.449740, 0, 5.140104, -0.449740, 0, 5.140104, -0.449740, 0, 5.135921, -0.503906, 0, 5.140104, -0.449740, 0, 5.140104, -0.449740, 0, 5.135921, -0.503906, 0, 5.135921, -0.503906, 0, 5.135921, -0.503906, 0, 5.123373, -0.551042, 0, 5.135921, -0.503906, 0, 5.135921, -0.503906, 0, 5.123373, -0.551042, 0, 5.123373, -0.551042, 0, 5.123373, -0.551042, 0, 5.102458, -0.591146, 0, 5.123373, -0.551042, 0, 5.123373, -0.551042, 0, 5.102458, -0.591146, 0, 5.102458, -0.591146, 0, 5.102458, -0.591146, 0, 5.073177, -0.624219, 0, 5.102458, -0.591146, 0, 5.102458, -0.591146, 0, 5.073177, -0.624219, 0, 5.073177, -0.624219, 0, 5.073177, -0.624219, 0, 5.035726, -0.650081, 0, 5.073177, -0.624219, 0, 5.073177, -0.624219, 0, 5.035726, -0.650081, 0, 5.035726, -0.650081, 0, 5.035726, -0.650081, 0, 4.990300, -0.668555, 0, 5.035726, -0.650081, 0, 5.035726, -0.650081, 0, 4.990300, -0.668555, 0, 4.990300, -0.668555, 0, 4.990300, -0.668555, 0, 4.936898, -0.679639, 0, 4.990300, -0.668555, 0, 4.990300, -0.668555, 0, 4.936898, -0.679639, 0, 4.936898, -0.679639, 0, 4.936898, -0.679639, 0, 4.875521, -0.683333, 0, 4.936898, -0.679639, 0, 4.936898, -0.679639, 0, 4.875521, -0.683333, 0, 4.875521, -0.683333, 0, 4.875521, -0.683333, 0, 4.743229, -0.683333, 0, 4.875521, -0.683333, 0, 4.875521, -0.683333, 0, 4.743229, -0.683333, 0, 4.743229, -0.683333, 0, 4.743229, -0.683333, 0, 4.743229, -1, 0, 4.743229, -0.683333, 0, 4.743229, -0.683333, 0, 4.743229, -1, 0, 4.743229, -1, 0, 4.743229, -1, 0, 4.638021, -1, 0, 4.743229, -1, 0, 4.743229, -1, 0, 4.638021, -1, 0, 4.638021, -1, 0, 4.638021, -0.216667, 0, 4.743229, -0.300000, 0, 4.875521, -0.216667, 0, 4.743229, -0.300000, 0, 4.638021, -0.216667, 0, 4.875521, -0.216667, 0, 4.743229, -0.300000, 0, 4.638021, -0.216667, 0, 4.743229, -0.600000, 0, 4.638021, -0.216667, 0, 4.743229, -0.300000, 0, 4.743229, -0.600000, 0, 4.638021, -0.216667, 0, 4.638021, -1, 0, 4.743229, -0.600000, 0, 4.638021, -1, 0, 4.638021, -0.216667, 0, 4.743229, -0.600000, 0, 4.638021, -1, 0, 4.743229, -0.683333, 0, 4.743229, -0.600000, 0, 4.743229, -0.683333, 0, 4.638021, -1, 0, 4.743229, -0.600000, 0, 4.638021, -1, 0, 4.743229, -1, 0, 4.743229, -0.683333, 0, 4.743229, -1, 0, 4.638021, -1, 0, 4.743229, -0.683333, 0, 4.743229, -0.600000, 0, 4.743229, -0.683333, 0, 4.875521, -0.683333, 0, 4.743229, -0.683333, 0, 4.743229, -0.600000, 0, 4.875521, -0.683333, 0, 4.743229, -0.600000, 0, 4.875521, -0.683333, 0, 4.875521, -0.600000, 0, 4.875521, -0.683333, 0, 4.743229, -0.600000, 0, 4.875521, -0.600000, 0, 4.910156, -0.597559, 0, 4.875521, -0.600000, 0, 4.875521, -0.683333, 0, 4.875521, -0.600000, 0, 4.910156, -0.597559, 0, 4.875521, -0.683333, 0, 4.936898, -0.679639, 0, 4.910156, -0.597559, 0, 4.875521, -0.683333, 0, 4.910156, -0.597559, 0, 4.936898, -0.679639, 0, 4.875521, -0.683333, 0, 4.910156, -0.597559, 0, 4.936898, -0.679639, 0, 4.940625, -0.590234, 0, 4.936898, -0.679639, 0, 4.910156, -0.597559, 0, 4.940625, -0.590234, 0, 4.940625, -0.590234, 0, 4.936898, -0.679639, 0, 4.990300, -0.668555, 0, 4.936898, -0.679639, 0, 4.940625, -0.590234, 0, 4.990300, -0.668555, 0, 4.966927, -0.578027, 0, 4.940625, -0.590234, 0, 4.990300, -0.668555, 0, 4.940625, -0.590234, 0, 4.966927, -0.578027, 0, 4.990300, -0.668555, 0, 4.966927, -0.578027, 0, 4.990300, -0.668555, 0, 5.035726, -0.650081, 0, 4.990300, -0.668555, 0, 4.966927, -0.578027, 0, 5.035726, -0.650081, 0, 4.989063, -0.560938, 0, 4.966927, -0.578027, 0, 5.035726, -0.650081, 0, 4.966927, -0.578027, 0, 4.989063, -0.560938, 0, 5.035726, -0.650081, 0, 4.989063, -0.560938, 0, 5.035726, -0.650081, 0, 5.073177, -0.624219, 0, 5.035726, -0.650081, 0, 4.989063, -0.560938, 0, 5.073177, -0.624219, 0, 5.006608, -0.539339, 0, 4.989063, -0.560938, 0, 5.073177, -0.624219, 0, 4.989063, -0.560938, 0, 5.006608, -0.539339, 0, 5.073177, -0.624219, 0, 5.006608, -0.539339, 0, 5.073177, -0.624219, 0, 5.102458, -0.591146, 0, 5.073177, -0.624219, 0, 5.006608, -0.539339, 0, 5.102458, -0.591146, 0, 5.019141, -0.513607, 0, 5.006608, -0.539339, 0, 5.102458, -0.591146, 0, 5.006608, -0.539339, 0, 5.019141, -0.513607, 0, 5.102458, -0.591146, 0, 5.123373, -0.551042, 0, 5.019141, -0.513607, 0, 5.102458, -0.591146, 0, 5.019141, -0.513607, 0, 5.123373, -0.551042, 0, 5.102458, -0.591146, 0, 5.019141, -0.513607, 0, 5.123373, -0.551042, 0, 5.135921, -0.503906, 0, 5.123373, -0.551042, 0, 5.019141, -0.513607, 0, 5.135921, -0.503906, 0, 5.026660, -0.483740, 0, 5.019141, -0.513607, 0, 5.135921, -0.503906, 0, 5.019141, -0.513607, 0, 5.026660, -0.483740, 0, 5.135921, -0.503906, 0, 5.140104, -0.449740, 0, 5.026660, -0.483740, 0, 5.135921, -0.503906, 0, 5.026660, -0.483740, 0, 5.140104, -0.449740, 0, 5.135921, -0.503906, 0, 5.140104, -0.449740, 0, 5.029167, -0.449740, 0, 5.026660, -0.483740, 0, 5.029167, -0.449740, 0, 5.140104, -0.449740, 0, 5.026660, -0.483740, 0, 5.140104, -0.449740, 0, 5.026660, -0.415967, 0, 5.029167, -0.449740, 0, 5.026660, -0.415967, 0, 5.140104, -0.449740, 0, 5.029167, -0.449740, 0, 5.135921, -0.396012, 0, 5.026660, -0.415967, 0, 5.140104, -0.449740, 0, 5.026660, -0.415967, 0, 5.135921, -0.396012, 0, 5.140104, -0.449740, 0, 5.135921, -0.396012, 0, 5.019141, -0.386263, 0, 5.026660, -0.415967, 0, 5.019141, -0.386263, 0, 5.135921, -0.396012, 0, 5.026660, -0.415967, 0, 5.123373, -0.349154, 0, 5.019141, -0.386263, 0, 5.135921, -0.396012, 0, 5.019141, -0.386263, 0, 5.123373, -0.349154, 0, 5.135921, -0.396012, 0, 5.102458, -0.309163, 0, 5.019141, -0.386263, 0, 5.123373, -0.349154, 0, 5.019141, -0.386263, 0, 5.102458, -0.309163, 0, 5.123373, -0.349154, 0, 5.006608, -0.360628, 0, 5.019141, -0.386263, 0, 5.102458, -0.309163, 0, 5.019141, -0.386263, 0, 5.006608, -0.360628, 0, 5.102458, -0.309163, 0, 5.073177, -0.276042, 0, 5.006608, -0.360628, 0, 5.102458, -0.309163, 0, 5.006608, -0.360628, 0, 5.073177, -0.276042, 0, 5.102458, -0.309163, 0, 4.989063, -0.339063, 0, 5.006608, -0.360628, 0, 5.073177, -0.276042, 0, 5.006608, -0.360628, 0, 4.989063, -0.339063, 0, 5.073177, -0.276042, 0, 5.035726, -0.250065, 0, 4.989063, -0.339063, 0, 5.073177, -0.276042, 0, 4.989063, -0.339063, 0, 5.035726, -0.250065, 0, 5.073177, -0.276042, 0, 4.966927, -0.321973, 0, 4.989063, -0.339063, 0, 5.035726, -0.250065, 0, 4.989063, -0.339063, 0, 4.966927, -0.321973, 0, 5.035726, -0.250065, 0, 4.990300, -0.231510, 0, 4.966927, -0.321973, 0, 5.035726, -0.250065, 0, 4.966927, -0.321973, 0, 4.990300, -0.231510, 0, 5.035726, -0.250065, 0, 4.940625, -0.309766, 0, 4.966927, -0.321973, 0, 4.990300, -0.231510, 0, 4.966927, -0.321973, 0, 4.940625, -0.309766, 0, 4.990300, -0.231510, 0, 4.936898, -0.220378, 0, 4.940625, -0.309766, 0, 4.990300, -0.231510, 0, 4.940625, -0.309766, 0, 4.936898, -0.220378, 0, 4.990300, -0.231510, 0, 4.936898, -0.220378, 0, 4.910156, -0.302441, 0, 4.940625, -0.309766, 0, 4.910156, -0.302441, 0, 4.936898, -0.220378, 0, 4.940625, -0.309766, 0, 4.875521, -0.216667, 0, 4.910156, -0.302441, 0, 4.936898, -0.220378, 0, 4.910156, -0.302441, 0, 4.875521, -0.216667, 0, 4.936898, -0.220378, 0, 4.875521, -0.216667, 0, 4.875521, -0.300000, 0, 4.910156, -0.302441, 0, 4.875521, -0.300000, 0, 4.875521, -0.216667, 0, 4.910156, -0.302441, 0, 4.743229, -0.300000, 0, 4.875521, -0.300000, 0, 4.875521, -0.216667, 0, 4.875521, -0.300000, 0, 4.743229, -0.300000, 0, 4.875521, -0.216667, 0, 5.605208, -0.500000, 0, 5.596956, -0.495988, 0, 5.605208, -0.500000, 0, 5.605208, -0.500000, 0, 5.596956, -0.495988, 0, 5.596956, -0.495988, 0, 5.596956, -0.495988, 0, 5.588346, -0.492546, 0, 5.596956, -0.495988, 0, 5.596956, -0.495988, 0, 5.588346, -0.492546, 0, 5.588346, -0.492546, 0, 5.588346, -0.492546, 0, 5.579378, -0.489673, 0, 5.588346, -0.492546, 0, 5.588346, -0.492546, 0, 5.579378, -0.489673, 0, 5.579378, -0.489673, 0, 5.579378, -0.489673, 0, 5.570052, -0.487370, 0, 5.579378, -0.489673, 0, 5.579378, -0.489673, 0, 5.570052, -0.487370, 0, 5.570052, -0.487370, 0, 5.570052, -0.487370, 0, 5.560303, -0.485604, 0, 5.570052, -0.487370, 0, 5.570052, -0.487370, 0, 5.560303, -0.485604, 0, 5.560303, -0.485604, 0, 5.560303, -0.485604, 0, 5.550066, -0.484342, 0, 5.560303, -0.485604, 0, 5.560303, -0.485604, 0, 5.550066, -0.484342, 0, 5.550066, -0.484342, 0, 5.550066, -0.484342, 0, 5.539339, -0.483586, 0, 5.550066, -0.484342, 0, 5.550066, -0.484342, 0, 5.539339, -0.483586, 0, 5.539339, -0.483586, 0, 5.539339, -0.483586, 0, 5.528125, -0.483333, 0, 5.539339, -0.483586, 0, 5.539339, -0.483586, 0, 5.528125, -0.483333, 0, 5.528125, -0.483333, 0, 5.528125, -0.483333, 0, 5.489860, -0.486646, 0, 5.528125, -0.483333, 0, 5.528125, -0.483333, 0, 5.489860, -0.486646, 0, 5.489860, -0.486646, 0, 5.489860, -0.486646, 0, 5.456316, -0.496582, 0, 5.489860, -0.486646, 0, 5.489860, -0.486646, 0, 5.456316, -0.496582, 0, 5.456316, -0.496582, 0, 5.456316, -0.496582, 0, 5.427490, -0.513143, 0, 5.456316, -0.496582, 0, 5.456316, -0.496582, 0, 5.427490, -0.513143, 0, 5.427490, -0.513143, 0, 5.427490, -0.513143, 0, 5.403386, -0.536328, 0, 5.427490, -0.513143, 0, 5.427490, -0.513143, 0, 5.403386, -0.536328, 0, 5.403386, -0.536328, 0, 5.403386, -0.536328, 0, 5.384359, -0.565698, 0, 5.403386, -0.536328, 0, 5.403386, -0.536328, 0, 5.384359, -0.565698, 0, 5.384359, -0.565698, 0, 5.384359, -0.565698, 0, 5.370769, -0.600814, 0, 5.384359, -0.565698, 0, 5.384359, -0.565698, 0, 5.370769, -0.600814, 0, 5.370769, -0.600814, 0, 5.370769, -0.600814, 0, 5.362614, -0.641675, 0, 5.370769, -0.600814, 0, 5.370769, -0.600814, 0, 5.362614, -0.641675, 0, 5.362614, -0.641675, 0, 5.362614, -0.641675, 0, 5.359896, -0.688281, 0, 5.362614, -0.641675, 0, 5.362614, -0.641675, 0, 5.359896, -0.688281, 0, 5.359896, -0.688281, 0, 5.359896, -0.688281, 0, 5.359896, -1, 0, 5.359896, -0.688281, 0, 5.359896, -0.688281, 0, 5.359896, -1, 0, 5.359896, -1, 0, 5.359896, -1, 0, 5.263542, -1, 0, 5.359896, -1, 0, 5.359896, -1, 0, 5.263542, -1, 0, 5.263542, -1, 0, 5.263542, -1, 0, 5.263542, -0.416667, 0, 5.263542, -1, 0, 5.263542, -1, 0, 5.263542, -0.416667, 0, 5.263542, -0.416667, 0, 5.263542, -0.416667, 0, 5.359896, -0.416667, 0, 5.263542, -0.416667, 0, 5.263542, -0.416667, 0, 5.359896, -0.416667, 0, 5.359896, -0.416667, 0, 5.359896, -0.416667, 0, 5.359896, -0.500000, 0, 5.359896, -0.416667, 0, 5.359896, -0.416667, 0, 5.359896, -0.500000, 0, 5.359896, -0.500000, 0, 5.359896, -0.500000, 0, 5.376140, -0.476245, 0, 5.359896, -0.500000, 0, 5.359896, -0.500000, 0, 5.376140, -0.476245, 0, 5.376140, -0.476245, 0, 5.376140, -0.476245, 0, 5.394661, -0.455762, 0, 5.376140, -0.476245, 0, 5.376140, -0.476245, 0, 5.394661, -0.455762, 0, 5.394661, -0.455762, 0, 5.394661, -0.455762, 0, 5.415462, -0.438550, 0, 5.394661, -0.455762, 0, 5.394661, -0.455762, 0, 5.415462, -0.438550, 0, 5.415462, -0.438550, 0, 5.415462, -0.438550, 0, 5.438542, -0.424609, 0, 5.415462, -0.438550, 0, 5.415462, -0.438550, 0, 5.438542, -0.424609, 0, 5.438542, -0.424609, 0, 5.438542, -0.424609, 0, 5.464063, -0.413843, 0, 5.438542, -0.424609, 0, 5.438542, -0.424609, 0, 5.464063, -0.413843, 0, 5.464063, -0.413843, 0, 5.464063, -0.413843, 0, 5.492188, -0.406152, 0, 5.464063, -0.413843, 0, 5.464063, -0.413843, 0, 5.492188, -0.406152, 0, 5.492188, -0.406152, 0, 5.492188, -0.406152, 0, 5.522917, -0.401538, 0, 5.492188, -0.406152, 0, 5.492188, -0.406152, 0, 5.522917, -0.401538, 0, 5.522917, -0.401538, 0, 5.522917, -0.401538, 0, 5.556250, -0.400000, 0, 5.522917, -0.401538, 0, 5.522917, -0.401538, 0, 5.556250, -0.400000, 0, 5.556250, -0.400000, 0, 5.556250, -0.400000, 0, 5.561328, -0.400000, 0, 5.556250, -0.400000, 0, 5.556250, -0.400000, 0, 5.561328, -0.400000, 0, 5.561328, -0.400000, 0, 5.561328, -0.400000, 0, 5.566667, -0.400000, 0, 5.561328, -0.400000, 0, 5.561328, -0.400000, 0, 5.566667, -0.400000, 0, 5.566667, -0.400000, 0, 5.566667, -0.400000, 0, 5.572266, -0.400000, 0, 5.566667, -0.400000, 0, 5.566667, -0.400000, 0, 5.572266, -0.400000, 0, 5.572266, -0.400000, 0, 5.572266, -0.400000, 0, 5.578125, -0.400000, 0, 5.572266, -0.400000, 0, 5.572266, -0.400000, 0, 5.578125, -0.400000, 0, 5.578125, -0.400000, 0, 5.578125, -0.400000, 0, 5.584278, -0.400000, 0, 5.578125, -0.400000, 0, 5.578125, -0.400000, 0, 5.584278, -0.400000, 0, 5.584278, -0.400000, 0, 5.584278, -0.400000, 0, 5.590755, -0.400000, 0, 5.584278, -0.400000, 0, 5.584278, -0.400000, 0, 5.590755, -0.400000, 0, 5.590755, -0.400000, 0, 5.590755, -0.400000, 0, 5.597558, -0.400000, 0, 5.590755, -0.400000, 0, 5.590755, -0.400000, 0, 5.597558, -0.400000, 0, 5.597558, -0.400000, 0, 5.597558, -0.400000, 0, 5.604688, -0.400000, 0, 5.597558, -0.400000, 0, 5.597558, -0.400000, 0, 5.604688, -0.400000, 0, 5.604688, -0.400000, 0, 5.359896, -0.416667, 0, 5.263542, -0.416667, 0, 5.359896, -0.500000, 0, 5.263542, -0.416667, 0, 5.359896, -0.416667, 0, 5.359896, -0.500000, 0, 5.370769, -0.600814, 0, 5.359896, -0.500000, 0, 5.263542, -0.416667, 0, 5.359896, -0.500000, 0, 5.370769, -0.600814, 0, 5.263542, -0.416667, 0, 5.359896, -0.500000, 0, 5.370769, -0.600814, 0, 5.384359, -0.565698, 0, 5.370769, -0.600814, 0, 5.359896, -0.500000, 0, 5.384359, -0.565698, 0, 5.359896, -0.500000, 0, 5.384359, -0.565698, 0, 5.403386, -0.536328, 0, 5.384359, -0.565698, 0, 5.359896, -0.500000, 0, 5.403386, -0.536328, 0, 5.376140, -0.476245, 0, 5.359896, -0.500000, 0, 5.403386, -0.536328, 0, 5.359896, -0.500000, 0, 5.376140, -0.476245, 0, 5.403386, -0.536328, 0, 5.376140, -0.476245, 0, 5.403386, -0.536328, 0, 5.427490, -0.513143, 0, 5.403386, -0.536328, 0, 5.376140, -0.476245, 0, 5.427490, -0.513143, 0, 5.394661, -0.455762, 0, 5.376140, -0.476245, 0, 5.427490, -0.513143, 0, 5.376140, -0.476245, 0, 5.394661, -0.455762, 0, 5.427490, -0.513143, 0, 5.394661, -0.455762, 0, 5.427490, -0.513143, 0, 5.456316, -0.496582, 0, 5.427490, -0.513143, 0, 5.394661, -0.455762, 0, 5.456316, -0.496582, 0, 5.415462, -0.438550, 0, 5.394661, -0.455762, 0, 5.456316, -0.496582, 0, 5.394661, -0.455762, 0, 5.415462, -0.438550, 0, 5.456316, -0.496582, 0, 5.438542, -0.424609, 0, 5.415462, -0.438550, 0, 5.456316, -0.496582, 0, 5.415462, -0.438550, 0, 5.438542, -0.424609, 0, 5.456316, -0.496582, 0, 5.438542, -0.424609, 0, 5.456316, -0.496582, 0, 5.489860, -0.486646, 0, 5.456316, -0.496582, 0, 5.438542, -0.424609, 0, 5.489860, -0.486646, 0, 5.464063, -0.413843, 0, 5.438542, -0.424609, 0, 5.489860, -0.486646, 0, 5.438542, -0.424609, 0, 5.464063, -0.413843, 0, 5.489860, -0.486646, 0, 5.464063, -0.413843, 0, 5.489860, -0.486646, 0, 5.492188, -0.406152, 0, 5.489860, -0.486646, 0, 5.464063, -0.413843, 0, 5.492188, -0.406152, 0, 5.492188, -0.406152, 0, 5.489860, -0.486646, 0, 5.528125, -0.483333, 0, 5.489860, -0.486646, 0, 5.492188, -0.406152, 0, 5.528125, -0.483333, 0, 5.522917, -0.401538, 0, 5.492188, -0.406152, 0, 5.528125, -0.483333, 0, 5.492188, -0.406152, 0, 5.522917, -0.401538, 0, 5.528125, -0.483333, 0, 5.522917, -0.401538, 0, 5.528125, -0.483333, 0, 5.539339, -0.483586, 0, 5.528125, -0.483333, 0, 5.522917, -0.401538, 0, 5.539339, -0.483586, 0, 5.556250, -0.400000, 0, 5.522917, -0.401538, 0, 5.539339, -0.483586, 0, 5.522917, -0.401538, 0, 5.556250, -0.400000, 0, 5.539339, -0.483586, 0, 5.539339, -0.483586, 0, 5.550066, -0.484342, 0, 5.556250, -0.400000, 0, 5.550066, -0.484342, 0, 5.539339, -0.483586, 0, 5.556250, -0.400000, 0, 5.561328, -0.400000, 0, 5.556250, -0.400000, 0, 5.550066, -0.484342, 0, 5.556250, -0.400000, 0, 5.561328, -0.400000, 0, 5.550066, -0.484342, 0, 5.561328, -0.400000, 0, 5.550066, -0.484342, 0, 5.560303, -0.485604, 0, 5.550066, -0.484342, 0, 5.561328, -0.400000, 0, 5.560303, -0.485604, 0, 5.566667, -0.400000, 0, 5.561328, -0.400000, 0, 5.560303, -0.485604, 0, 5.561328, -0.400000, 0, 5.566667, -0.400000, 0, 5.560303, -0.485604, 0, 5.560303, -0.485604, 0, 5.572266, -0.400000, 0, 5.566667, -0.400000, 0, 5.572266, -0.400000, 0, 5.560303, -0.485604, 0, 5.566667, -0.400000, 0, 5.572266, -0.400000, 0, 5.560303, -0.485604, 0, 5.570052, -0.487370, 0, 5.560303, -0.485604, 0, 5.572266, -0.400000, 0, 5.570052, -0.487370, 0, 5.578125, -0.400000, 0, 5.572266, -0.400000, 0, 5.570052, -0.487370, 0, 5.572266, -0.400000, 0, 5.578125, -0.400000, 0, 5.570052, -0.487370, 0, 5.570052, -0.487370, 0, 5.584278, -0.400000, 0, 5.578125, -0.400000, 0, 5.584278, -0.400000, 0, 5.570052, -0.487370, 0, 5.578125, -0.400000, 0, 5.584278, -0.400000, 0, 5.570052, -0.487370, 0, 5.579378, -0.489673, 0, 5.570052, -0.487370, 0, 5.584278, -0.400000, 0, 5.579378, -0.489673, 0, 5.590755, -0.400000, 0, 5.584278, -0.400000, 0, 5.579378, -0.489673, 0, 5.584278, -0.400000, 0, 5.590755, -0.400000, 0, 5.579378, -0.489673, 0, 5.597558, -0.400000, 0, 5.590755, -0.400000, 0, 5.579378, -0.489673, 0, 5.590755, -0.400000, 0, 5.597558, -0.400000, 0, 5.579378, -0.489673, 0, 5.588346, -0.492546, 0, 5.597558, -0.400000, 0, 5.579378, -0.489673, 0, 5.597558, -0.400000, 0, 5.588346, -0.492546, 0, 5.579378, -0.489673, 0, 5.604688, -0.400000, 0, 5.597558, -0.400000, 0, 5.588346, -0.492546, 0, 5.597558, -0.400000, 0, 5.604688, -0.400000, 0, 5.588346, -0.492546, 0, 5.596956, -0.495988, 0, 5.604688, -0.400000, 0, 5.588346, -0.492546, 0, 5.604688, -0.400000, 0, 5.596956, -0.495988, 0, 5.588346, -0.492546, 0, 5.605208, -0.500000, 0, 5.604688, -0.400000, 0, 5.596956, -0.495988, 0, 5.604688, -0.400000, 0, 5.605208, -0.500000, 0, 5.596956, -0.495988, 0, 5.370769, -0.600814, 0, 5.263542, -0.416667, 0, 5.362614, -0.641675, 0, 5.263542, -0.416667, 0, 5.370769, -0.600814, 0, 5.362614, -0.641675, 0, 5.359896, -0.688281, 0, 5.362614, -0.641675, 0, 5.263542, -0.416667, 0, 5.362614, -0.641675, 0, 5.359896, -0.688281, 0, 5.263542, -0.416667, 0, 5.359896, -0.688281, 0, 5.263542, -0.416667, 0, 5.263542, -1, 0, 5.263542, -0.416667, 0, 5.359896, -0.688281, 0, 5.263542, -1, 0, 5.263542, -1, 0, 5.359896, -1, 0, 5.359896, -0.688281, 0, 5.359896, -1, 0, 5.263542, -1, 0, 5.359896, -0.688281, 0, 5.909896, -0.483333, 0, 5.873373, -0.487093, 0, 5.909896, -0.483333, 0, 5.909896, -0.483333, 0, 5.873373, -0.487093, 0, 5.873373, -0.487093, 0, 5.873373, -0.487093, 0, 5.840886, -0.498372, 0, 5.873373, -0.487093, 0, 5.873373, -0.487093, 0, 5.840886, -0.498372, 0, 5.840886, -0.498372, 0, 5.840886, -0.498372, 0, 5.812435, -0.517171, 0, 5.840886, -0.498372, 0, 5.840886, -0.498372, 0, 5.812435, -0.517171, 0, 5.812435, -0.517171, 0, 5.812435, -0.517171, 0, 5.788021, -0.543490, 0, 5.812435, -0.517171, 0, 5.812435, -0.517171, 0, 5.788021, -0.543490, 0, 5.788021, -0.543490, 0, 5.788021, -0.543490, 0, 5.768425, -0.576351, 0, 5.788021, -0.543490, 0, 5.788021, -0.543490, 0, 5.768425, -0.576351, 0, 5.768425, -0.576351, 0, 5.768425, -0.576351, 0, 5.754427, -0.614779, 0, 5.768425, -0.576351, 0, 5.768425, -0.576351, 0, 5.754427, -0.614779, 0, 5.754427, -0.614779, 0, 5.754427, -0.614779, 0, 5.746028, -0.658773, 0, 5.754427, -0.614779, 0, 5.754427, -0.614779, 0, 5.746028, -0.658773, 0, 5.746028, -0.658773, 0, 5.746028, -0.658773, 0, 5.743229, -0.708333, 0, 5.746028, -0.658773, 0, 5.746028, -0.658773, 0, 5.743229, -0.708333, 0, 5.743229, -0.708333, 0, 5.743229, -0.708333, 0, 5.746013, -0.757894, 0, 5.743229, -0.708333, 0, 5.743229, -0.708333, 0, 5.746013, -0.757894, 0, 5.746013, -0.757894, 0, 5.746013, -0.757894, 0, 5.754362, -0.801888, 0, 5.746013, -0.757894, 0, 5.746013, -0.757894, 0, 5.754362, -0.801888, 0, 5.754362, -0.801888, 0, 5.754362, -0.801888, 0, 5.768278, -0.840316, 0, 5.754362, -0.801888, 0, 5.754362, -0.801888, 0, 5.768278, -0.840316, 0, 5.768278, -0.840316, 0, 5.768278, -0.840316, 0, 5.787761, -0.873177, 0, 5.768278, -0.840316, 0, 5.768278, -0.840316, 0, 5.787761, -0.873177, 0, 5.787761, -0.873177, 0, 5.787761, -0.873177, 0, 5.812093, -0.899495, 0, 5.787761, -0.873177, 0, 5.787761, -0.873177, 0, 5.812093, -0.899495, 0, 5.812093, -0.899495, 0, 5.812093, -0.899495, 0, 5.840560, -0.918294, 0, 5.812093, -0.899495, 0, 5.812093, -0.899495, 0, 5.840560, -0.918294, 0, 5.840560, -0.918294, 0, 5.840560, -0.918294, 0, 5.873161, -0.929574, 0, 5.840560, -0.918294, 0, 5.840560, -0.918294, 0, 5.873161, -0.929574, 0, 5.873161, -0.929574, 0, 5.873161, -0.929574, 0, 5.909896, -0.933333, 0, 5.873161, -0.929574, 0, 5.873161, -0.929574, 0, 5.909896, -0.933333, 0, 5.909896, -0.933333, 0, 5.909896, -0.933333, 0, 5.946191, -0.929557, 0, 5.909896, -0.933333, 0, 5.909896, -0.933333, 0, 5.946191, -0.929557, 0, 5.946191, -0.929557, 0, 5.946191, -0.929557, 0, 5.978516, -0.918229, 0, 5.946191, -0.929557, 0, 5.946191, -0.929557, 0, 5.978516, -0.918229, 0, 5.978516, -0.918229, 0, 5.978516, -0.918229, 0, 6.006868, -0.899349, 0, 5.978516, -0.918229, 0, 5.978516, -0.918229, 0, 6.006868, -0.899349, 0, 6.006868, -0.899349, 0, 6.006868, -0.899349, 0, 6.031250, -0.872917, 0, 6.006868, -0.899349, 0, 6.006868, -0.899349, 0, 6.031250, -0.872917, 0, 6.031250, -0.872917, 0, 6.031250, -0.872917, 0, 6.050847, -0.839974, 0, 6.031250, -0.872917, 0, 6.031250, -0.872917, 0, 6.050847, -0.839974, 0, 6.050847, -0.839974, 0, 6.050847, -0.839974, 0, 6.064844, -0.801562, 0, 6.050847, -0.839974, 0, 6.050847, -0.839974, 0, 6.064844, -0.801562, 0, 6.064844, -0.801562, 0, 6.064844, -0.801562, 0, 6.073242, -0.757682, 0, 6.064844, -0.801562, 0, 6.064844, -0.801562, 0, 6.073242, -0.757682, 0, 6.073242, -0.757682, 0, 6.073242, -0.757682, 0, 6.076042, -0.708333, 0, 6.073242, -0.757682, 0, 6.073242, -0.757682, 0, 6.076042, -0.708333, 0, 6.076042, -0.708333, 0, 6.076042, -0.708333, 0, 6.073242, -0.659196, 0, 6.076042, -0.708333, 0, 6.076042, -0.708333, 0, 6.073242, -0.659196, 0, 6.073242, -0.659196, 0, 6.073242, -0.659196, 0, 6.064844, -0.615430, 0, 6.073242, -0.659196, 0, 6.073242, -0.659196, 0, 6.064844, -0.615430, 0, 6.064844, -0.615430, 0, 6.064844, -0.615430, 0, 6.050847, -0.577034, 0, 6.064844, -0.615430, 0, 6.064844, -0.615430, 0, 6.050847, -0.577034, 0, 6.050847, -0.577034, 0, 6.050847, -0.577034, 0, 6.031250, -0.544010, 0, 6.050847, -0.577034, 0, 6.050847, -0.577034, 0, 6.031250, -0.544010, 0, 6.031250, -0.544010, 0, 6.031250, -0.544010, 0, 6.006868, -0.517464, 0, 6.031250, -0.544010, 0, 6.031250, -0.544010, 0, 6.006868, -0.517464, 0, 6.006868, -0.517464, 0, 6.006868, -0.517464, 0, 5.978516, -0.498503, 0, 6.006868, -0.517464, 0, 6.006868, -0.517464, 0, 5.978516, -0.498503, 0, 5.978516, -0.498503, 0, 5.978516, -0.498503, 0, 5.946191, -0.487126, 0, 5.978516, -0.498503, 0, 5.978516, -0.498503, 0, 5.946191, -0.487126, 0, 5.946191, -0.487126, 0, 5.909896, -0.400000, 0, 5.969043, -0.405111, 0, 5.909896, -0.400000, 0, 5.909896, -0.400000, 0, 5.969043, -0.405111, 0, 5.969043, -0.405111, 0, 5.969043, -0.405111, 0, 6.021484, -0.420443, 0, 5.969043, -0.405111, 0, 5.969043, -0.405111, 0, 6.021484, -0.420443, 0, 6.021484, -0.420443, 0, 6.021484, -0.420443, 0, 6.067220, -0.445996, 0, 6.021484, -0.420443, 0, 6.021484, -0.420443, 0, 6.067220, -0.445996, 0, 6.067220, -0.445996, 0, 6.067220, -0.445996, 0, 6.106250, -0.481771, 0, 6.067220, -0.445996, 0, 6.067220, -0.445996, 0, 6.106250, -0.481771, 0, 6.106250, -0.481771, 0, 6.106250, -0.481771, 0, 6.137468, -0.526595, 0, 6.106250, -0.481771, 0, 6.106250, -0.481771, 0, 6.137468, -0.526595, 0, 6.137468, -0.526595, 0, 6.137468, -0.526595, 0, 6.159766, -0.579297, 0, 6.137468, -0.526595, 0, 6.137468, -0.526595, 0, 6.159766, -0.579297, 0, 6.159766, -0.579297, 0, 6.159766, -0.579297, 0, 6.173145, -0.639876, 0, 6.159766, -0.579297, 0, 6.159766, -0.579297, 0, 6.173145, -0.639876, 0, 6.173145, -0.639876, 0, 6.173145, -0.639876, 0, 6.177604, -0.708333, 0, 6.173145, -0.639876, 0, 6.173145, -0.639876, 0, 6.177604, -0.708333, 0, 6.177604, -0.708333, 0, 6.177604, -0.708333, 0, 6.173145, -0.776579, 0, 6.177604, -0.708333, 0, 6.177604, -0.708333, 0, 6.173145, -0.776579, 0, 6.173145, -0.776579, 0, 6.173145, -0.776579, 0, 6.159766, -0.837044, 0, 6.173145, -0.776579, 0, 6.173145, -0.776579, 0, 6.159766, -0.837044, 0, 6.159766, -0.837044, 0, 6.159766, -0.837044, 0, 6.137468, -0.889730, 0, 6.159766, -0.837044, 0, 6.159766, -0.837044, 0, 6.137468, -0.889730, 0, 6.137468, -0.889730, 0, 6.137468, -0.889730, 0, 6.106250, -0.934635, 0, 6.137468, -0.889730, 0, 6.137468, -0.889730, 0, 6.106250, -0.934635, 0, 6.106250, -0.934635, 0, 6.106250, -0.934635, 0, 6.067220, -0.970524, 0, 6.106250, -0.934635, 0, 6.106250, -0.934635, 0, 6.067220, -0.970524, 0, 6.067220, -0.970524, 0, 6.067220, -0.970524, 0, 6.021484, -0.996159, 0, 6.067220, -0.970524, 0, 6.067220, -0.970524, 0, 6.021484, -0.996159, 0, 6.021484, -0.996159, 0, 6.021484, -0.996159, 0, 5.969043, -1.011540, 0, 6.021484, -0.996159, 0, 6.021484, -0.996159, 0, 5.969043, -1.011540, 0, 5.969043, -1.011540, 0, 5.969043, -1.011540, 0, 5.909896, -1.016667, 0, 5.969043, -1.011540, 0, 5.969043, -1.011540, 0, 5.909896, -1.016667, 0, 5.909896, -1.016667, 0, 5.909896, -1.016667, 0, 5.850537, -1.011540, 0, 5.909896, -1.016667, 0, 5.909896, -1.016667, 0, 5.850537, -1.011540, 0, 5.850537, -1.011540, 0, 5.850537, -1.011540, 0, 5.797982, -0.996159, 0, 5.850537, -1.011540, 0, 5.850537, -1.011540, 0, 5.797982, -0.996159, 0, 5.797982, -0.996159, 0, 5.797982, -0.996159, 0, 5.752230, -0.970524, 0, 5.797982, -0.996159, 0, 5.797982, -0.996159, 0, 5.752230, -0.970524, 0, 5.752230, -0.970524, 0, 5.752230, -0.970524, 0, 5.713281, -0.934635, 0, 5.752230, -0.970524, 0, 5.752230, -0.970524, 0, 5.713281, -0.934635, 0, 5.713281, -0.934635, 0, 5.713281, -0.934635, 0, 5.682178, -0.889730, 0, 5.713281, -0.934635, 0, 5.713281, -0.934635, 0, 5.682178, -0.889730, 0, 5.682178, -0.889730, 0, 5.682178, -0.889730, 0, 5.659961, -0.837044, 0, 5.682178, -0.889730, 0, 5.682178, -0.889730, 0, 5.659961, -0.837044, 0, 5.659961, -0.837044, 0, 5.659961, -0.837044, 0, 5.646631, -0.776579, 0, 5.659961, -0.837044, 0, 5.659961, -0.837044, 0, 5.646631, -0.776579, 0, 5.646631, -0.776579, 0, 5.646631, -0.776579, 0, 5.642188, -0.708333, 0, 5.646631, -0.776579, 0, 5.646631, -0.776579, 0, 5.642188, -0.708333, 0, 5.642188, -0.708333, 0, 5.642188, -0.708333, 0, 5.646631, -0.639876, 0, 5.642188, -0.708333, 0, 5.642188, -0.708333, 0, 5.646631, -0.639876, 0, 5.646631, -0.639876, 0, 5.646631, -0.639876, 0, 5.659961, -0.579297, 0, 5.646631, -0.639876, 0, 5.646631, -0.639876, 0, 5.659961, -0.579297, 0, 5.659961, -0.579297, 0, 5.659961, -0.579297, 0, 5.682178, -0.526595, 0, 5.659961, -0.579297, 0, 5.659961, -0.579297, 0, 5.682178, -0.526595, 0, 5.682178, -0.526595, 0, 5.682178, -0.526595, 0, 5.713281, -0.481771, 0, 5.682178, -0.526595, 0, 5.682178, -0.526595, 0, 5.713281, -0.481771, 0, 5.713281, -0.481771, 0, 5.713281, -0.481771, 0, 5.752230, -0.445996, 0, 5.713281, -0.481771, 0, 5.713281, -0.481771, 0, 5.752230, -0.445996, 0, 5.752230, -0.445996, 0, 5.752230, -0.445996, 0, 5.797982, -0.420443, 0, 5.752230, -0.445996, 0, 5.752230, -0.445996, 0, 5.797982, -0.420443, 0, 5.797982, -0.420443, 0, 5.797982, -0.420443, 0, 5.850537, -0.405111, 0, 5.797982, -0.420443, 0, 5.797982, -0.420443, 0, 5.850537, -0.405111, 0, 5.850537, -0.405111, 0, 5.746028, -0.658773, 0, 5.646631, -0.639876, 0, 5.642188, -0.708333, 0, 5.646631, -0.639876, 0, 5.746028, -0.658773, 0, 5.642188, -0.708333, 0, 5.754427, -0.614779, 0, 5.646631, -0.639876, 0, 5.746028, -0.658773, 0, 5.646631, -0.639876, 0, 5.754427, -0.614779, 0, 5.746028, -0.658773, 0, 5.754427, -0.614779, 0, 5.659961, -0.579297, 0, 5.646631, -0.639876, 0, 5.659961, -0.579297, 0, 5.754427, -0.614779, 0, 5.646631, -0.639876, 0, 5.659961, -0.579297, 0, 5.754427, -0.614779, 0, 5.768425, -0.576351, 0, 5.754427, -0.614779, 0, 5.659961, -0.579297, 0, 5.768425, -0.576351, 0, 5.682178, -0.526595, 0, 5.659961, -0.579297, 0, 5.768425, -0.576351, 0, 5.659961, -0.579297, 0, 5.682178, -0.526595, 0, 5.768425, -0.576351, 0, 5.788021, -0.543490, 0, 5.682178, -0.526595, 0, 5.768425, -0.576351, 0, 5.682178, -0.526595, 0, 5.788021, -0.543490, 0, 5.768425, -0.576351, 0, 5.788021, -0.543490, 0, 5.713281, -0.481771, 0, 5.682178, -0.526595, 0, 5.713281, -0.481771, 0, 5.788021, -0.543490, 0, 5.682178, -0.526595, 0, 5.752230, -0.445996, 0, 5.713281, -0.481771, 0, 5.788021, -0.543490, 0, 5.713281, -0.481771, 0, 5.752230, -0.445996, 0, 5.788021, -0.543490, 0, 5.752230, -0.445996, 0, 5.788021, -0.543490, 0, 5.812435, -0.517171, 0, 5.788021, -0.543490, 0, 5.752230, -0.445996, 0, 5.812435, -0.517171, 0, 5.797982, -0.420443, 0, 5.752230, -0.445996, 0, 5.812435, -0.517171, 0, 5.752230, -0.445996, 0, 5.797982, -0.420443, 0, 5.812435, -0.517171, 0, 5.797982, -0.420443, 0, 5.812435, -0.517171, 0, 5.840886, -0.498372, 0, 5.812435, -0.517171, 0, 5.797982, -0.420443, 0, 5.840886, -0.498372, 0, 5.797982, -0.420443, 0, 5.840886, -0.498372, 0, 5.850537, -0.405111, 0, 5.840886, -0.498372, 0, 5.797982, -0.420443, 0, 5.850537, -0.405111, 0, 5.850537, -0.405111, 0, 5.840886, -0.498372, 0, 5.873373, -0.487093, 0, 5.840886, -0.498372, 0, 5.850537, -0.405111, 0, 5.873373, -0.487093, 0, 5.850537, -0.405111, 0, 5.873373, -0.487093, 0, 5.909896, -0.400000, 0, 5.873373, -0.487093, 0, 5.850537, -0.405111, 0, 5.909896, -0.400000, 0, 5.873373, -0.487093, 0, 5.909896, -0.483333, 0, 5.909896, -0.400000, 0, 5.909896, -0.483333, 0, 5.873373, -0.487093, 0, 5.909896, -0.400000, 0, 5.909896, -0.400000, 0, 5.909896, -0.483333, 0, 5.946191, -0.487126, 0, 5.909896, -0.483333, 0, 5.909896, -0.400000, 0, 5.946191, -0.487126, 0, 5.909896, -0.400000, 0, 5.946191, -0.487126, 0, 5.969043, -0.405111, 0, 5.946191, -0.487126, 0, 5.909896, -0.400000, 0, 5.969043, -0.405111, 0, 5.969043, -0.405111, 0, 5.946191, -0.487126, 0, 5.978516, -0.498503, 0, 5.946191, -0.487126, 0, 5.969043, -0.405111, 0, 5.978516, -0.498503, 0, 5.969043, -0.405111, 0, 5.978516, -0.498503, 0, 6.021484, -0.420443, 0, 5.978516, -0.498503, 0, 5.969043, -0.405111, 0, 6.021484, -0.420443, 0, 5.978516, -0.498503, 0, 6.006868, -0.517464, 0, 6.021484, -0.420443, 0, 6.006868, -0.517464, 0, 5.978516, -0.498503, 0, 6.021484, -0.420443, 0, 6.021484, -0.420443, 0, 6.006868, -0.517464, 0, 6.067220, -0.445996, 0, 6.006868, -0.517464, 0, 6.021484, -0.420443, 0, 6.067220, -0.445996, 0, 6.006868, -0.517464, 0, 6.031250, -0.544010, 0, 6.067220, -0.445996, 0, 6.031250, -0.544010, 0, 6.006868, -0.517464, 0, 6.067220, -0.445996, 0, 6.067220, -0.445996, 0, 6.031250, -0.544010, 0, 6.106250, -0.481771, 0, 6.031250, -0.544010, 0, 6.067220, -0.445996, 0, 6.106250, -0.481771, 0, 6.106250, -0.481771, 0, 6.031250, -0.544010, 0, 6.137468, -0.526595, 0, 6.031250, -0.544010, 0, 6.106250, -0.481771, 0, 6.137468, -0.526595, 0, 6.137468, -0.526595, 0, 6.031250, -0.544010, 0, 6.050847, -0.577034, 0, 6.031250, -0.544010, 0, 6.137468, -0.526595, 0, 6.050847, -0.577034, 0, 6.137468, -0.526595, 0, 6.050847, -0.577034, 0, 6.159766, -0.579297, 0, 6.050847, -0.577034, 0, 6.137468, -0.526595, 0, 6.159766, -0.579297, 0, 6.050847, -0.577034, 0, 6.064844, -0.615430, 0, 6.159766, -0.579297, 0, 6.064844, -0.615430, 0, 6.050847, -0.577034, 0, 6.159766, -0.579297, 0, 6.159766, -0.579297, 0, 6.064844, -0.615430, 0, 6.173145, -0.639876, 0, 6.064844, -0.615430, 0, 6.159766, -0.579297, 0, 6.173145, -0.639876, 0, 6.064844, -0.615430, 0, 6.073242, -0.659196, 0, 6.173145, -0.639876, 0, 6.073242, -0.659196, 0, 6.064844, -0.615430, 0, 6.173145, -0.639876, 0, 6.173145, -0.639876, 0, 6.073242, -0.659196, 0, 6.177604, -0.708333, 0, 6.073242, -0.659196, 0, 6.173145, -0.639876, 0, 6.177604, -0.708333, 0, 6.177604, -0.708333, 0, 6.073242, -0.659196, 0, 6.076042, -0.708333, 0, 6.073242, -0.659196, 0, 6.177604, -0.708333, 0, 6.076042, -0.708333, 0, 6.177604, -0.708333, 0, 6.076042, -0.708333, 0, 6.073242, -0.757682, 0, 6.076042, -0.708333, 0, 6.177604, -0.708333, 0, 6.073242, -0.757682, 0, 6.177604, -0.708333, 0, 6.073242, -0.757682, 0, 6.173145, -0.776579, 0, 6.073242, -0.757682, 0, 6.177604, -0.708333, 0, 6.173145, -0.776579, 0, 6.073242, -0.757682, 0, 6.064844, -0.801562, 0, 6.173145, -0.776579, 0, 6.064844, -0.801562, 0, 6.073242, -0.757682, 0, 6.173145, -0.776579, 0, 6.064844, -0.801562, 0, 6.159766, -0.837044, 0, 6.173145, -0.776579, 0, 6.159766, -0.837044, 0, 6.064844, -0.801562, 0, 6.173145, -0.776579, 0, 6.064844, -0.801562, 0, 6.050847, -0.839974, 0, 6.159766, -0.837044, 0, 6.050847, -0.839974, 0, 6.064844, -0.801562, 0, 6.159766, -0.837044, 0, 6.050847, -0.839974, 0, 6.137468, -0.889730, 0, 6.159766, -0.837044, 0, 6.137468, -0.889730, 0, 6.050847, -0.839974, 0, 6.159766, -0.837044, 0, 6.050847, -0.839974, 0, 6.031250, -0.872917, 0, 6.137468, -0.889730, 0, 6.031250, -0.872917, 0, 6.050847, -0.839974, 0, 6.137468, -0.889730, 0, 6.031250, -0.872917, 0, 6.106250, -0.934635, 0, 6.137468, -0.889730, 0, 6.106250, -0.934635, 0, 6.031250, -0.872917, 0, 6.137468, -0.889730, 0, 6.106250, -0.934635, 0, 6.031250, -0.872917, 0, 6.067220, -0.970524, 0, 6.031250, -0.872917, 0, 6.106250, -0.934635, 0, 6.067220, -0.970524, 0, 6.031250, -0.872917, 0, 6.006868, -0.899349, 0, 6.067220, -0.970524, 0, 6.006868, -0.899349, 0, 6.031250, -0.872917, 0, 6.067220, -0.970524, 0, 6.067220, -0.970524, 0, 6.006868, -0.899349, 0, 6.021484, -0.996159, 0, 6.006868, -0.899349, 0, 6.067220, -0.970524, 0, 6.021484, -0.996159, 0, 6.006868, -0.899349, 0, 5.978516, -0.918229, 0, 6.021484, -0.996159, 0, 5.978516, -0.918229, 0, 6.006868, -0.899349, 0, 6.021484, -0.996159, 0, 6.021484, -0.996159, 0, 5.978516, -0.918229, 0, 5.969043, -1.011540, 0, 5.978516, -0.918229, 0, 6.021484, -0.996159, 0, 5.969043, -1.011540, 0, 5.978516, -0.918229, 0, 5.946191, -0.929557, 0, 5.969043, -1.011540, 0, 5.946191, -0.929557, 0, 5.978516, -0.918229, 0, 5.969043, -1.011540, 0, 5.969043, -1.011540, 0, 5.946191, -0.929557, 0, 5.909896, -1.016667, 0, 5.946191, -0.929557, 0, 5.969043, -1.011540, 0, 5.909896, -1.016667, 0, 5.946191, -0.929557, 0, 5.909896, -0.933333, 0, 5.909896, -1.016667, 0, 5.909896, -0.933333, 0, 5.946191, -0.929557, 0, 5.909896, -1.016667, 0, 5.909896, -0.933333, 0, 5.873161, -0.929574, 0, 5.909896, -1.016667, 0, 5.873161, -0.929574, 0, 5.909896, -0.933333, 0, 5.909896, -1.016667, 0, 5.873161, -0.929574, 0, 5.850537, -1.011540, 0, 5.909896, -1.016667, 0, 5.850537, -1.011540, 0, 5.873161, -0.929574, 0, 5.909896, -1.016667, 0, 5.873161, -0.929574, 0, 5.840560, -0.918294, 0, 5.850537, -1.011540, 0, 5.840560, -0.918294, 0, 5.873161, -0.929574, 0, 5.850537, -1.011540, 0, 5.840560, -0.918294, 0, 5.797982, -0.996159, 0, 5.850537, -1.011540, 0, 5.797982, -0.996159, 0, 5.840560, -0.918294, 0, 5.850537, -1.011540, 0, 5.840560, -0.918294, 0, 5.812093, -0.899495, 0, 5.797982, -0.996159, 0, 5.812093, -0.899495, 0, 5.840560, -0.918294, 0, 5.797982, -0.996159, 0, 5.812093, -0.899495, 0, 5.752230, -0.970524, 0, 5.797982, -0.996159, 0, 5.752230, -0.970524, 0, 5.812093, -0.899495, 0, 5.797982, -0.996159, 0, 5.812093, -0.899495, 0, 5.787761, -0.873177, 0, 5.752230, -0.970524, 0, 5.787761, -0.873177, 0, 5.812093, -0.899495, 0, 5.752230, -0.970524, 0, 5.787761, -0.873177, 0, 5.713281, -0.934635, 0, 5.752230, -0.970524, 0, 5.713281, -0.934635, 0, 5.787761, -0.873177, 0, 5.752230, -0.970524, 0, 5.682178, -0.889730, 0, 5.713281, -0.934635, 0, 5.787761, -0.873177, 0, 5.713281, -0.934635, 0, 5.682178, -0.889730, 0, 5.787761, -0.873177, 0, 5.768278, -0.840316, 0, 5.682178, -0.889730, 0, 5.787761, -0.873177, 0, 5.682178, -0.889730, 0, 5.768278, -0.840316, 0, 5.787761, -0.873177, 0, 5.659961, -0.837044, 0, 5.682178, -0.889730, 0, 5.768278, -0.840316, 0, 5.682178, -0.889730, 0, 5.659961, -0.837044, 0, 5.768278, -0.840316, 0, 5.754362, -0.801888, 0, 5.659961, -0.837044, 0, 5.768278, -0.840316, 0, 5.659961, -0.837044, 0, 5.754362, -0.801888, 0, 5.768278, -0.840316, 0, 5.646631, -0.776579, 0, 5.659961, -0.837044, 0, 5.754362, -0.801888, 0, 5.659961, -0.837044, 0, 5.646631, -0.776579, 0, 5.754362, -0.801888, 0, 5.746013, -0.757894, 0, 5.646631, -0.776579, 0, 5.754362, -0.801888, 0, 5.646631, -0.776579, 0, 5.746013, -0.757894, 0, 5.754362, -0.801888, 0, 5.642188, -0.708333, 0, 5.646631, -0.776579, 0, 5.746013, -0.757894, 0, 5.646631, -0.776579, 0, 5.642188, -0.708333, 0, 5.746013, -0.757894, 0, 5.743229, -0.708333, 0, 5.642188, -0.708333, 0, 5.746013, -0.757894, 0, 5.642188, -0.708333, 0, 5.743229, -0.708333, 0, 5.746013, -0.757894, 0, 5.746028, -0.658773, 0, 5.642188, -0.708333, 0, 5.743229, -0.708333, 0, 5.642188, -0.708333, 0, 5.746028, -0.658773, 0, 5.743229, -0.708333, 0, 6.428646, -0.250000, 0, 6.428646, -0.416667, 0, 6.428646, -0.250000, 0, 6.428646, -0.250000, 0, 6.428646, -0.416667, 0, 6.428646, -0.416667, 0, 6.428646, -0.416667, 0, 6.626041, -0.416667, 0, 6.428646, -0.416667, 0, 6.428646, -0.416667, 0, 6.626041, -0.416667, 0, 6.626041, -0.416667, 0, 6.626041, -0.416667, 0, 6.626041, -0.500000, 0, 6.626041, -0.416667, 0, 6.626041, -0.416667, 0, 6.626041, -0.500000, 0, 6.626041, -0.500000, 0, 6.626041, -0.500000, 0, 6.428646, -0.500000, 0, 6.626041, -0.500000, 0, 6.626041, -0.500000, 0, 6.428646, -0.500000, 0, 6.428646, -0.500000, 0, 6.428646, -0.500000, 0, 6.428646, -0.807813, 0, 6.428646, -0.500000, 0, 6.428646, -0.500000, 0, 6.428646, -0.807813, 0, 6.428646, -0.807813, 0, 6.428646, -0.807813, 0, 6.429867, -0.839355, 0, 6.428646, -0.807813, 0, 6.428646, -0.807813, 0, 6.429867, -0.839355, 0, 6.429867, -0.839355, 0, 6.429867, -0.839355, 0, 6.433529, -0.864714, 0, 6.429867, -0.839355, 0, 6.429867, -0.839355, 0, 6.433529, -0.864714, 0, 6.433529, -0.864714, 0, 6.433529, -0.864714, 0, 6.439632, -0.883887, 0, 6.433529, -0.864714, 0, 6.433529, -0.864714, 0, 6.439632, -0.883887, 0, 6.439632, -0.883887, 0, 6.439632, -0.883887, 0, 6.448177, -0.896875, 0, 6.439632, -0.883887, 0, 6.439632, -0.883887, 0, 6.448177, -0.896875, 0, 6.448177, -0.896875, 0, 6.448177, -0.896875, 0, 6.460465, -0.905534, 0, 6.448177, -0.896875, 0, 6.448177, -0.896875, 0, 6.460465, -0.905534, 0, 6.460465, -0.905534, 0, 6.460465, -0.905534, 0, 6.477799, -0.911719, 0, 6.460465, -0.905534, 0, 6.460465, -0.905534, 0, 6.477799, -0.911719, 0, 6.477799, -0.911719, 0, 6.477799, -0.911719, 0, 6.500179, -0.915430, 0, 6.477799, -0.911719, 0, 6.477799, -0.911719, 0, 6.500179, -0.915430, 0, 6.500179, -0.915430, 0, 6.500179, -0.915430, 0, 6.527604, -0.916667, 0, 6.500179, -0.915430, 0, 6.500179, -0.915430, 0, 6.527604, -0.916667, 0, 6.527604, -0.916667, 0, 6.527604, -0.916667, 0, 6.626041, -0.916667, 0, 6.527604, -0.916667, 0, 6.527604, -0.916667, 0, 6.626041, -0.916667, 0, 6.626041, -0.916667, 0, 6.626041, -0.916667, 0, 6.626041, -1, 0, 6.626041, -0.916667, 0, 6.626041, -0.916667, 0, 6.626041, -1, 0, 6.626041, -1, 0, 6.626041, -1, 0, 6.527604, -1, 0, 6.626041, -1, 0, 6.626041, -1, 0, 6.527604, -1, 0, 6.527604, -1, 0, 6.527604, -1, 0, 6.476432, -0.997412, 0, 6.527604, -1, 0, 6.527604, -1, 0, 6.476432, -0.997412, 0, 6.476432, -0.997412, 0, 6.476432, -0.997412, 0, 6.433854, -0.989648, 0, 6.476432, -0.997412, 0, 6.476432, -0.997412, 0, 6.433854, -0.989648, 0, 6.433854, -0.989648, 0, 6.433854, -0.989648, 0, 6.399870, -0.976709, 0, 6.433854, -0.989648, 0, 6.433854, -0.989648, 0, 6.399870, -0.976709, 0, 6.399870, -0.976709, 0, 6.399870, -0.976709, 0, 6.374479, -0.958594, 0, 6.399870, -0.976709, 0, 6.399870, -0.976709, 0, 6.374479, -0.958594, 0, 6.374479, -0.958594, 0, 6.374479, -0.958594, 0, 6.356022, -0.933643, 0, 6.374479, -0.958594, 0, 6.374479, -0.958594, 0, 6.356022, -0.933643, 0, 6.356022, -0.933643, 0, 6.356022, -0.933643, 0, 6.342839, -0.900195, 0, 6.356022, -0.933643, 0, 6.356022, -0.933643, 0, 6.342839, -0.900195, 0, 6.342839, -0.900195, 0, 6.342839, -0.900195, 0, 6.334929, -0.858252, 0, 6.342839, -0.900195, 0, 6.342839, -0.900195, 0, 6.334929, -0.858252, 0, 6.334929, -0.858252, 0, 6.334929, -0.858252, 0, 6.332292, -0.807813, 0, 6.334929, -0.858252, 0, 6.334929, -0.858252, 0, 6.332292, -0.807813, 0, 6.332292, -0.807813, 0, 6.332292, -0.807813, 0, 6.332292, -0.500000, 0, 6.332292, -0.807813, 0, 6.332292, -0.807813, 0, 6.332292, -0.500000, 0, 6.332292, -0.500000, 0, 6.332292, -0.500000, 0, 6.261980, -0.500000, 0, 6.332292, -0.500000, 0, 6.332292, -0.500000, 0, 6.261980, -0.500000, 0, 6.261980, -0.500000, 0, 6.261980, -0.500000, 0, 6.261980, -0.416667, 0, 6.261980, -0.500000, 0, 6.261980, -0.500000, 0, 6.261980, -0.416667, 0, 6.261980, -0.416667, 0, 6.261980, -0.416667, 0, 6.332292, -0.416667, 0, 6.261980, -0.416667, 0, 6.261980, -0.416667, 0, 6.332292, -0.416667, 0, 6.332292, -0.416667, 0, 6.332292, -0.416667, 0, 6.332292, -0.250000, 0, 6.332292, -0.416667, 0, 6.332292, -0.416667, 0, 6.332292, -0.250000, 0, 6.332292, -0.250000, 0, 6.261980, -0.416667, 0, 6.332292, -0.500000, 0, 6.332292, -0.416667, 0, 6.332292, -0.500000, 0, 6.261980, -0.416667, 0, 6.332292, -0.416667, 0, 6.261980, -0.416667, 0, 6.261980, -0.500000, 0, 6.332292, -0.500000, 0, 6.261980, -0.500000, 0, 6.261980, -0.416667, 0, 6.332292, -0.500000, 0, 6.332292, -0.416667, 0, 6.332292, -0.500000, 0, 6.428646, -0.500000, 0, 6.332292, -0.500000, 0, 6.332292, -0.416667, 0, 6.428646, -0.500000, 0, 6.332292, -0.416667, 0, 6.428646, -0.500000, 0, 6.428646, -0.416667, 0, 6.428646, -0.500000, 0, 6.332292, -0.416667, 0, 6.428646, -0.416667, 0, 6.332292, -0.250000, 0, 6.332292, -0.416667, 0, 6.428646, -0.416667, 0, 6.332292, -0.416667, 0, 6.332292, -0.250000, 0, 6.428646, -0.416667, 0, 6.332292, -0.250000, 0, 6.428646, -0.416667, 0, 6.428646, -0.250000, 0, 6.428646, -0.416667, 0, 6.332292, -0.250000, 0, 6.428646, -0.250000, 0, 6.428646, -0.416667, 0, 6.428646, -0.500000, 0, 6.626041, -0.500000, 0, 6.428646, -0.500000, 0, 6.428646, -0.416667, 0, 6.626041, -0.500000, 0, 6.428646, -0.416667, 0, 6.626041, -0.500000, 0, 6.626041, -0.416667, 0, 6.626041, -0.500000, 0, 6.428646, -0.416667, 0, 6.626041, -0.416667, 0, 6.332292, -0.500000, 0, 6.428646, -0.807813, 0, 6.428646, -0.500000, 0, 6.428646, -0.807813, 0, 6.332292, -0.500000, 0, 6.428646, -0.500000, 0, 6.332292, -0.500000, 0, 6.332292, -0.807813, 0, 6.428646, -0.807813, 0, 6.332292, -0.807813, 0, 6.332292, -0.500000, 0, 6.428646, -0.807813, 0, 6.428646, -0.807813, 0, 6.332292, -0.807813, 0, 6.429867, -0.839355, 0, 6.332292, -0.807813, 0, 6.428646, -0.807813, 0, 6.429867, -0.839355, 0, 6.332292, -0.807813, 0, 6.334929, -0.858252, 0, 6.429867, -0.839355, 0, 6.334929, -0.858252, 0, 6.332292, -0.807813, 0, 6.429867, -0.839355, 0, 6.429867, -0.839355, 0, 6.334929, -0.858252, 0, 6.433529, -0.864714, 0, 6.334929, -0.858252, 0, 6.429867, -0.839355, 0, 6.433529, -0.864714, 0, 6.334929, -0.858252, 0, 6.342839, -0.900195, 0, 6.433529, -0.864714, 0, 6.342839, -0.900195, 0, 6.334929, -0.858252, 0, 6.433529, -0.864714, 0, 6.433529, -0.864714, 0, 6.342839, -0.900195, 0, 6.439632, -0.883887, 0, 6.342839, -0.900195, 0, 6.433529, -0.864714, 0, 6.439632, -0.883887, 0, 6.439632, -0.883887, 0, 6.342839, -0.900195, 0, 6.356022, -0.933643, 0, 6.342839, -0.900195, 0, 6.439632, -0.883887, 0, 6.356022, -0.933643, 0, 6.448177, -0.896875, 0, 6.439632, -0.883887, 0, 6.356022, -0.933643, 0, 6.439632, -0.883887, 0, 6.448177, -0.896875, 0, 6.356022, -0.933643, 0, 6.356022, -0.933643, 0, 6.374479, -0.958594, 0, 6.448177, -0.896875, 0, 6.374479, -0.958594, 0, 6.356022, -0.933643, 0, 6.448177, -0.896875, 0, 6.448177, -0.896875, 0, 6.374479, -0.958594, 0, 6.399870, -0.976709, 0, 6.374479, -0.958594, 0, 6.448177, -0.896875, 0, 6.399870, -0.976709, 0, 6.460465, -0.905534, 0, 6.448177, -0.896875, 0, 6.399870, -0.976709, 0, 6.448177, -0.896875, 0, 6.460465, -0.905534, 0, 6.399870, -0.976709, 0, 6.399870, -0.976709, 0, 6.433854, -0.989648, 0, 6.460465, -0.905534, 0, 6.433854, -0.989648, 0, 6.399870, -0.976709, 0, 6.460465, -0.905534, 0, 6.460465, -0.905534, 0, 6.433854, -0.989648, 0, 6.477799, -0.911719, 0, 6.433854, -0.989648, 0, 6.460465, -0.905534, 0, 6.477799, -0.911719, 0, 6.433854, -0.989648, 0, 6.476432, -0.997412, 0, 6.477799, -0.911719, 0, 6.476432, -0.997412, 0, 6.433854, -0.989648, 0, 6.477799, -0.911719, 0, 6.477799, -0.911719, 0, 6.476432, -0.997412, 0, 6.500179, -0.915430, 0, 6.476432, -0.997412, 0, 6.477799, -0.911719, 0, 6.500179, -0.915430, 0, 6.500179, -0.915430, 0, 6.476432, -0.997412, 0, 6.527604, -1, 0, 6.476432, -0.997412, 0, 6.500179, -0.915430, 0, 6.527604, -1, 0, 6.527604, -0.916667, 0, 6.500179, -0.915430, 0, 6.527604, -1, 0, 6.500179, -0.915430, 0, 6.527604, -0.916667, 0, 6.527604, -1, 0, 6.527604, -0.916667, 0, 6.527604, -1, 0, 6.626041, -1, 0, 6.527604, -1, 0, 6.527604, -0.916667, 0, 6.626041, -1, 0, 6.527604, -0.916667, 0, 6.626041, -1, 0, 6.626041, -0.916667, 0, 6.626041, -1, 0, 6.527604, -0.916667, 0, 6.626041, -0.916667, 0, 7.249479, -0.681250, 0, 7.249479, -0.733333, 0, 7.249479, -0.681250, 0, 7.249479, -0.681250, 0, 7.249479, -0.733333, 0, 7.249479, -0.733333, 0, 7.249479, -0.733333, 0, 6.808854, -0.733333, 0, 7.249479, -0.733333, 0, 7.249479, -0.733333, 0, 6.808854, -0.733333, 0, 6.808854, -0.733333, 0, 6.808854, -0.733333, 0, 6.814926, -0.779256, 0, 6.808854, -0.733333, 0, 6.808854, -0.733333, 0, 6.814926, -0.779256, 0, 6.814926, -0.779256, 0, 6.814926, -0.779256, 0, 6.826889, -0.819368, 0, 6.814926, -0.779256, 0, 6.814926, -0.779256, 0, 6.826889, -0.819368, 0, 6.826889, -0.819368, 0, 6.826889, -0.819368, 0, 6.844743, -0.853670, 0, 6.826889, -0.819368, 0, 6.826889, -0.819368, 0, 6.844743, -0.853670, 0, 6.844743, -0.853670, 0, 6.844743, -0.853670, 0, 6.868490, -0.882161, 0, 6.844743, -0.853670, 0, 6.844743, -0.853670, 0, 6.868490, -0.882161, 0, 6.868490, -0.882161, 0, 6.868490, -0.882161, 0, 6.897803, -0.904549, 0, 6.868490, -0.882161, 0, 6.868490, -0.882161, 0, 6.897803, -0.904549, 0, 6.897803, -0.904549, 0, 6.897803, -0.904549, 0, 6.932357, -0.920540, 0, 6.897803, -0.904549, 0, 6.897803, -0.904549, 0, 6.932357, -0.920540, 0, 6.932357, -0.920540, 0, 6.932357, -0.920540, 0, 6.972152, -0.930135, 0, 6.932357, -0.920540, 0, 6.932357, -0.920540, 0, 6.972152, -0.930135, 0, 6.972152, -0.930135, 0, 6.972152, -0.930135, 0, 7.017188, -0.933333, 0, 6.972152, -0.930135, 0, 6.972152, -0.930135, 0, 7.017188, -0.933333, 0, 7.017188, -0.933333, 0, 7.017188, -0.933333, 0, 7.044580, -0.932552, 0, 7.017188, -0.933333, 0, 7.017188, -0.933333, 0, 7.044580, -0.932552, 0, 7.044580, -0.932552, 0, 7.044580, -0.932552, 0, 7.071549, -0.930208, 0, 7.044580, -0.932552, 0, 7.044580, -0.932552, 0, 7.071549, -0.930208, 0, 7.071549, -0.930208, 0, 7.071549, -0.930208, 0, 7.098096, -0.926302, 0, 7.071549, -0.930208, 0, 7.071549, -0.930208, 0, 7.098096, -0.926302, 0, 7.098096, -0.926302, 0, 7.098096, -0.926302, 0, 7.124219, -0.920833, 0, 7.098096, -0.926302, 0, 7.098096, -0.926302, 0, 7.124219, -0.920833, 0, 7.124219, -0.920833, 0, 7.124219, -0.920833, 0, 7.150082, -0.913802, 0, 7.124219, -0.920833, 0, 7.124219, -0.920833, 0, 7.150082, -0.913802, 0, 7.150082, -0.913802, 0, 7.150082, -0.913802, 0, 7.175846, -0.905208, 0, 7.150082, -0.913802, 0, 7.150082, -0.913802, 0, 7.175846, -0.905208, 0, 7.175846, -0.905208, 0, 7.175846, -0.905208, 0, 7.201514, -0.895052, 0, 7.175846, -0.905208, 0, 7.175846, -0.905208, 0, 7.201514, -0.895052, 0, 7.201514, -0.895052, 0, 7.201514, -0.895052, 0, 7.227084, -0.883333, 0, 7.201514, -0.895052, 0, 7.201514, -0.895052, 0, 7.227084, -0.883333, 0, 7.227084, -0.883333, 0, 7.227084, -0.883333, 0, 7.227084, -0.966667, 0, 7.227084, -0.883333, 0, 7.227084, -0.883333, 0, 7.227084, -0.966667, 0, 7.227084, -0.966667, 0, 7.227084, -0.966667, 0, 7.201139, -0.978174, 0, 7.227084, -0.966667, 0, 7.227084, -0.966667, 0, 7.201139, -0.978174, 0, 7.201139, -0.978174, 0, 7.201139, -0.978174, 0, 7.174870, -0.988216, 0, 7.201139, -0.978174, 0, 7.201139, -0.978174, 0, 7.174870, -0.988216, 0, 7.174870, -0.988216, 0, 7.174870, -0.988216, 0, 7.148275, -0.996794, 0, 7.174870, -0.988216, 0, 7.174870, -0.988216, 0, 7.148275, -0.996794, 0, 7.148275, -0.996794, 0, 7.148275, -0.996794, 0, 7.121354, -1.003906, 0, 7.148275, -0.996794, 0, 7.148275, -0.996794, 0, 7.121354, -1.003906, 0, 7.121354, -1.003906, 0, 7.121354, -1.003906, 0, 7.094173, -1.009489, 0, 7.121354, -1.003906, 0, 7.121354, -1.003906, 0, 7.094173, -1.009489, 0, 7.094173, -1.009489, 0, 7.094173, -1.009489, 0, 7.066797, -1.013477, 0, 7.094173, -1.009489, 0, 7.094173, -1.009489, 0, 7.066797, -1.013477, 0, 7.066797, -1.013477, 0, 7.066797, -1.013477, 0, 7.039225, -1.015869, 0, 7.066797, -1.013477, 0, 7.066797, -1.013477, 0, 7.039225, -1.015869, 0, 7.039225, -1.015869, 0, 7.039225, -1.015869, 0, 7.011459, -1.016667, 0, 7.039225, -1.015869, 0, 7.039225, -1.015869, 0, 7.011459, -1.016667, 0, 7.011459, -1.016667, 0, 7.011459, -1.016667, 0, 6.945297, -1.011556, 0, 7.011459, -1.016667, 0, 7.011459, -1.016667, 0, 6.945297, -1.011556, 0, 6.945297, -1.011556, 0, 6.945297, -1.011556, 0, 6.886393, -0.996224, 0, 6.945297, -1.011556, 0, 6.945297, -1.011556, 0, 6.886393, -0.996224, 0, 6.886393, -0.996224, 0, 6.886393, -0.996224, 0, 6.834750, -0.970671, 0, 6.886393, -0.996224, 0, 6.886393, -0.996224, 0, 6.834750, -0.970671, 0, 6.834750, -0.970671, 0, 6.834750, -0.970671, 0, 6.790365, -0.934896, 0, 6.834750, -0.970671, 0, 6.834750, -0.970671, 0, 6.790365, -0.934896, 0, 6.790365, -0.934896, 0, 6.790365, -0.934896, 0, 6.754704, -0.890397, 0, 6.790365, -0.934896, 0, 6.790365, -0.934896, 0, 6.754704, -0.890397, 0, 6.754704, -0.890397, 0, 6.754704, -0.890397, 0, 6.729232, -0.838672, 0, 6.754704, -0.890397, 0, 6.754704, -0.890397, 0, 6.729232, -0.838672, 0, 6.729232, -0.838672, 0, 6.729232, -0.838672, 0, 6.713949, -0.779720, 0, 6.729232, -0.838672, 0, 6.729232, -0.838672, 0, 6.713949, -0.779720, 0, 6.713949, -0.779720, 0, 6.713949, -0.779720, 0, 6.708854, -0.713542, 0, 6.713949, -0.779720, 0, 6.713949, -0.779720, 0, 6.708854, -0.713542, 0, 6.708854, -0.713542, 0, 6.708854, -0.713542, 0, 6.713688, -0.645133, 0, 6.708854, -0.713542, 0, 6.708854, -0.713542, 0, 6.713688, -0.645133, 0, 6.713688, -0.645133, 0, 6.713688, -0.645133, 0, 6.728190, -0.584180, 0, 6.713688, -0.645133, 0, 6.713688, -0.645133, 0, 6.728190, -0.584180, 0, 6.728190, -0.584180, 0, 6.728190, -0.584180, 0, 6.752360, -0.530680, 0, 6.728190, -0.584180, 0, 6.728190, -0.584180, 0, 6.752360, -0.530680, 0, 6.752360, -0.530680, 0, 6.752360, -0.530680, 0, 6.786198, -0.484635, 0, 6.752360, -0.530680, 0, 6.752360, -0.530680, 0, 6.786198, -0.484635, 0, 6.786198, -0.484635, 0, 6.786198, -0.484635, 0, 6.828239, -0.447607, 0, 6.786198, -0.484635, 0, 6.786198, -0.484635, 0, 6.828239, -0.447607, 0, 6.828239, -0.447607, 0, 6.828239, -0.447607, 0, 6.877018, -0.421159, 0, 6.828239, -0.447607, 0, 6.828239, -0.447607, 0, 6.877018, -0.421159, 0, 6.877018, -0.421159, 0, 6.877018, -0.421159, 0, 6.932536, -0.405290, 0, 6.877018, -0.421159, 0, 6.877018, -0.421159, 0, 6.932536, -0.405290, 0, 6.932536, -0.405290, 0, 6.932536, -0.405290, 0, 6.994792, -0.400000, 0, 6.932536, -0.405290, 0, 6.932536, -0.405290, 0, 6.994792, -0.400000, 0, 6.994792, -0.400000, 0, 6.994792, -0.400000, 0, 7.050570, -0.404728, 0, 6.994792, -0.400000, 0, 6.994792, -0.400000, 0, 7.050570, -0.404728, 0, 7.050570, -0.404728, 0, 7.050570, -0.404728, 0, 7.100195, -0.418913, 0, 7.050570, -0.404728, 0, 7.050570, -0.404728, 0, 7.100195, -0.418913, 0, 7.100195, -0.418913, 0, 7.100195, -0.418913, 0, 7.143669, -0.442554, 0, 7.100195, -0.418913, 0, 7.100195, -0.418913, 0, 7.143669, -0.442554, 0, 7.143669, -0.442554, 0, 7.143669, -0.442554, 0, 7.180990, -0.475651, 0, 7.143669, -0.442554, 0, 7.143669, -0.442554, 0, 7.180990, -0.475651, 0, 7.180990, -0.475651, 0, 7.180990, -0.475651, 0, 7.210954, -0.516870, 0, 7.180990, -0.475651, 0, 7.180990, -0.475651, 0, 7.210954, -0.516870, 0, 7.210954, -0.516870, 0, 7.210954, -0.516870, 0, 7.232357, -0.564876, 0, 7.210954, -0.516870, 0, 7.210954, -0.516870, 0, 7.232357, -0.564876, 0, 7.232357, -0.564876, 0, 7.232357, -0.564876, 0, 7.245199, -0.619670, 0, 7.232357, -0.564876, 0, 7.232357, -0.564876, 0, 7.245199, -0.619670, 0, 7.245199, -0.619670, 0, 6.713688, -0.645133, 0, 6.708854, -0.713542, 0, 6.811979, -0.650521, 0, 6.708854, -0.713542, 0, 6.713688, -0.645133, 0, 6.811979, -0.650521, 0, 6.728190, -0.584180, 0, 6.713688, -0.645133, 0, 6.811979, -0.650521, 0, 6.713688, -0.645133, 0, 6.728190, -0.584180, 0, 6.811979, -0.650521, 0, 6.728190, -0.584180, 0, 6.811979, -0.650521, 0, 6.818180, -0.612923, 0, 6.811979, -0.650521, 0, 6.728190, -0.584180, 0, 6.818180, -0.612923, 0, 6.829492, -0.579818, 0, 6.728190, -0.584180, 0, 6.818180, -0.612923, 0, 6.728190, -0.584180, 0, 6.829492, -0.579818, 0, 6.818180, -0.612923, 0, 6.829492, -0.579818, 0, 6.752360, -0.530680, 0, 6.728190, -0.584180, 0, 6.752360, -0.530680, 0, 6.829492, -0.579818, 0, 6.728190, -0.584180, 0, 6.752360, -0.530680, 0, 6.829492, -0.579818, 0, 6.845914, -0.551204, 0, 6.829492, -0.579818, 0, 6.752360, -0.530680, 0, 6.845914, -0.551204, 0, 6.786198, -0.484635, 0, 6.752360, -0.530680, 0, 6.845914, -0.551204, 0, 6.752360, -0.530680, 0, 6.786198, -0.484635, 0, 6.845914, -0.551204, 0, 6.786198, -0.484635, 0, 6.845914, -0.551204, 0, 6.867448, -0.527083, 0, 6.845914, -0.551204, 0, 6.786198, -0.484635, 0, 6.867448, -0.527083, 0, 6.828239, -0.447607, 0, 6.786198, -0.484635, 0, 6.867448, -0.527083, 0, 6.786198, -0.484635, 0, 6.828239, -0.447607, 0, 6.867448, -0.527083, 0, 6.828239, -0.447607, 0, 6.867448, -0.527083, 0, 6.893539, -0.507943, 0, 6.867448, -0.527083, 0, 6.828239, -0.447607, 0, 6.893539, -0.507943, 0, 6.877018, -0.421159, 0, 6.828239, -0.447607, 0, 6.893539, -0.507943, 0, 6.828239, -0.447607, 0, 6.877018, -0.421159, 0, 6.893539, -0.507943, 0, 6.877018, -0.421159, 0, 6.893539, -0.507943, 0, 6.923633, -0.494271, 0, 6.893539, -0.507943, 0, 6.877018, -0.421159, 0, 6.923633, -0.494271, 0, 6.877018, -0.421159, 0, 6.923633, -0.494271, 0, 6.932536, -0.405290, 0, 6.923633, -0.494271, 0, 6.877018, -0.421159, 0, 6.932536, -0.405290, 0, 6.932536, -0.405290, 0, 6.923633, -0.494271, 0, 6.957731, -0.486068, 0, 6.923633, -0.494271, 0, 6.932536, -0.405290, 0, 6.957731, -0.486068, 0, 6.932536, -0.405290, 0, 6.957731, -0.486068, 0, 6.994792, -0.400000, 0, 6.957731, -0.486068, 0, 6.932536, -0.405290, 0, 6.994792, -0.400000, 0, 6.994792, -0.400000, 0, 6.957731, -0.486068, 0, 6.995834, -0.483333, 0, 6.957731, -0.486068, 0, 6.994792, -0.400000, 0, 6.995834, -0.483333, 0, 6.994792, -0.400000, 0, 6.995834, -0.483333, 0, 7.029508, -0.486165, 0, 6.995834, -0.483333, 0, 6.994792, -0.400000, 0, 7.029508, -0.486165, 0, 6.994792, -0.400000, 0, 7.029508, -0.486165, 0, 7.050570, -0.404728, 0, 7.029508, -0.486165, 0, 6.994792, -0.400000, 0, 7.050570, -0.404728, 0, 7.050570, -0.404728, 0, 7.029508, -0.486165, 0, 7.059701, -0.494661, 0, 7.029508, -0.486165, 0, 7.050570, -0.404728, 0, 7.059701, -0.494661, 0, 7.050570, -0.404728, 0, 7.059701, -0.494661, 0, 7.100195, -0.418913, 0, 7.059701, -0.494661, 0, 7.050570, -0.404728, 0, 7.100195, -0.418913, 0, 7.059701, -0.494661, 0, 7.086410, -0.508822, 0, 7.100195, -0.418913, 0, 7.086410, -0.508822, 0, 7.059701, -0.494661, 0, 7.100195, -0.418913, 0, 7.100195, -0.418913, 0, 7.086410, -0.508822, 0, 7.143669, -0.442554, 0, 7.086410, -0.508822, 0, 7.100195, -0.418913, 0, 7.143669, -0.442554, 0, 7.086410, -0.508822, 0, 7.109635, -0.528646, 0, 7.143669, -0.442554, 0, 7.109635, -0.528646, 0, 7.086410, -0.508822, 0, 7.143669, -0.442554, 0, 7.143669, -0.442554, 0, 7.109635, -0.528646, 0, 7.180990, -0.475651, 0, 7.109635, -0.528646, 0, 7.143669, -0.442554, 0, 7.180990, -0.475651, 0, 7.109635, -0.528646, 0, 7.128500, -0.553223, 0, 7.180990, -0.475651, 0, 7.128500, -0.553223, 0, 7.109635, -0.528646, 0, 7.180990, -0.475651, 0, 7.180990, -0.475651, 0, 7.128500, -0.553223, 0, 7.210954, -0.516870, 0, 7.128500, -0.553223, 0, 7.180990, -0.475651, 0, 7.210954, -0.516870, 0, 7.128500, -0.553223, 0, 7.142123, -0.581641, 0, 7.210954, -0.516870, 0, 7.142123, -0.581641, 0, 7.128500, -0.553223, 0, 7.210954, -0.516870, 0, 7.210954, -0.516870, 0, 7.142123, -0.581641, 0, 7.232357, -0.564876, 0, 7.142123, -0.581641, 0, 7.210954, -0.516870, 0, 7.232357, -0.564876, 0, 7.142123, -0.581641, 0, 7.150505, -0.613900, 0, 7.232357, -0.564876, 0, 7.150505, -0.613900, 0, 7.142123, -0.581641, 0, 7.232357, -0.564876, 0, 7.232357, -0.564876, 0, 7.150505, -0.613900, 0, 7.245199, -0.619670, 0, 7.150505, -0.613900, 0, 7.232357, -0.564876, 0, 7.245199, -0.619670, 0, 7.150505, -0.613900, 0, 7.153646, -0.650000, 0, 7.245199, -0.619670, 0, 7.153646, -0.650000, 0, 7.150505, -0.613900, 0, 7.245199, -0.619670, 0, 7.245199, -0.619670, 0, 7.153646, -0.650000, 0, 7.249479, -0.681250, 0, 7.153646, -0.650000, 0, 7.245199, -0.619670, 0, 7.249479, -0.681250, 0, 7.249479, -0.733333, 0, 7.249479, -0.681250, 0, 7.153646, -0.650000, 0, 7.249479, -0.681250, 0, 7.249479, -0.733333, 0, 7.153646, -0.650000, 0, 7.153646, -0.650000, 0, 6.808854, -0.733333, 0, 7.249479, -0.733333, 0, 6.808854, -0.733333, 0, 7.153646, -0.650000, 0, 7.249479, -0.733333, 0, 7.153646, -0.650000, 0, 6.811979, -0.650521, 0, 6.808854, -0.733333, 0, 6.811979, -0.650521, 0, 7.153646, -0.650000, 0, 6.808854, -0.733333, 0, 6.708854, -0.713542, 0, 6.808854, -0.733333, 0, 6.811979, -0.650521, 0, 6.808854, -0.733333, 0, 6.708854, -0.713542, 0, 6.811979, -0.650521, 0, 6.708854, -0.713542, 0, 6.713949, -0.779720, 0, 6.808854, -0.733333, 0, 6.713949, -0.779720, 0, 6.708854, -0.713542, 0, 6.808854, -0.733333, 0, 6.808854, -0.733333, 0, 6.713949, -0.779720, 0, 6.814926, -0.779256, 0, 6.713949, -0.779720, 0, 6.808854, -0.733333, 0, 6.814926, -0.779256, 0, 6.713949, -0.779720, 0, 6.729232, -0.838672, 0, 6.814926, -0.779256, 0, 6.729232, -0.838672, 0, 6.713949, -0.779720, 0, 6.814926, -0.779256, 0, 6.814926, -0.779256, 0, 6.729232, -0.838672, 0, 6.826889, -0.819368, 0, 6.729232, -0.838672, 0, 6.814926, -0.779256, 0, 6.826889, -0.819368, 0, 6.729232, -0.838672, 0, 6.754704, -0.890397, 0, 6.826889, -0.819368, 0, 6.754704, -0.890397, 0, 6.729232, -0.838672, 0, 6.826889, -0.819368, 0, 6.826889, -0.819368, 0, 6.754704, -0.890397, 0, 6.844743, -0.853670, 0, 6.754704, -0.890397, 0, 6.826889, -0.819368, 0, 6.844743, -0.853670, 0, 6.754704, -0.890397, 0, 6.790365, -0.934896, 0, 6.844743, -0.853670, 0, 6.790365, -0.934896, 0, 6.754704, -0.890397, 0, 6.844743, -0.853670, 0, 6.844743, -0.853670, 0, 6.790365, -0.934896, 0, 6.868490, -0.882161, 0, 6.790365, -0.934896, 0, 6.844743, -0.853670, 0, 6.868490, -0.882161, 0, 6.790365, -0.934896, 0, 6.834750, -0.970671, 0, 6.868490, -0.882161, 0, 6.834750, -0.970671, 0, 6.790365, -0.934896, 0, 6.868490, -0.882161, 0, 6.868490, -0.882161, 0, 6.834750, -0.970671, 0, 6.897803, -0.904549, 0, 6.834750, -0.970671, 0, 6.868490, -0.882161, 0, 6.897803, -0.904549, 0, 6.897803, -0.904549, 0, 6.834750, -0.970671, 0, 6.886393, -0.996224, 0, 6.834750, -0.970671, 0, 6.897803, -0.904549, 0, 6.886393, -0.996224, 0, 6.932357, -0.920540, 0, 6.897803, -0.904549, 0, 6.886393, -0.996224, 0, 6.897803, -0.904549, 0, 6.932357, -0.920540, 0, 6.886393, -0.996224, 0, 6.932357, -0.920540, 0, 6.886393, -0.996224, 0, 6.945297, -1.011556, 0, 6.886393, -0.996224, 0, 6.932357, -0.920540, 0, 6.945297, -1.011556, 0, 6.972152, -0.930135, 0, 6.932357, -0.920540, 0, 6.945297, -1.011556, 0, 6.932357, -0.920540, 0, 6.972152, -0.930135, 0, 6.945297, -1.011556, 0, 6.972152, -0.930135, 0, 6.945297, -1.011556, 0, 7.011459, -1.016667, 0, 6.945297, -1.011556, 0, 6.972152, -0.930135, 0, 7.011459, -1.016667, 0, 7.017188, -0.933333, 0, 6.972152, -0.930135, 0, 7.011459, -1.016667, 0, 6.972152, -0.930135, 0, 7.017188, -0.933333, 0, 7.011459, -1.016667, 0, 7.017188, -0.933333, 0, 7.011459, -1.016667, 0, 7.039225, -1.015869, 0, 7.011459, -1.016667, 0, 7.017188, -0.933333, 0, 7.039225, -1.015869, 0, 7.017188, -0.933333, 0, 7.039225, -1.015869, 0, 7.044580, -0.932552, 0, 7.039225, -1.015869, 0, 7.017188, -0.933333, 0, 7.044580, -0.932552, 0, 7.044580, -0.932552, 0, 7.039225, -1.015869, 0, 7.066797, -1.013477, 0, 7.039225, -1.015869, 0, 7.044580, -0.932552, 0, 7.066797, -1.013477, 0, 7.044580, -0.932552, 0, 7.066797, -1.013477, 0, 7.071549, -0.930208, 0, 7.066797, -1.013477, 0, 7.044580, -0.932552, 0, 7.071549, -0.930208, 0, 7.071549, -0.930208, 0, 7.066797, -1.013477, 0, 7.094173, -1.009489, 0, 7.066797, -1.013477, 0, 7.071549, -0.930208, 0, 7.094173, -1.009489, 0, 7.071549, -0.930208, 0, 7.094173, -1.009489, 0, 7.098096, -0.926302, 0, 7.094173, -1.009489, 0, 7.071549, -0.930208, 0, 7.098096, -0.926302, 0, 7.098096, -0.926302, 0, 7.094173, -1.009489, 0, 7.121354, -1.003906, 0, 7.094173, -1.009489, 0, 7.098096, -0.926302, 0, 7.121354, -1.003906, 0, 7.098096, -0.926302, 0, 7.121354, -1.003906, 0, 7.124219, -0.920833, 0, 7.121354, -1.003906, 0, 7.098096, -0.926302, 0, 7.124219, -0.920833, 0, 7.124219, -0.920833, 0, 7.121354, -1.003906, 0, 7.148275, -0.996794, 0, 7.121354, -1.003906, 0, 7.124219, -0.920833, 0, 7.148275, -0.996794, 0, 7.150082, -0.913802, 0, 7.124219, -0.920833, 0, 7.148275, -0.996794, 0, 7.124219, -0.920833, 0, 7.150082, -0.913802, 0, 7.148275, -0.996794, 0, 7.174870, -0.988216, 0, 7.150082, -0.913802, 0, 7.148275, -0.996794, 0, 7.150082, -0.913802, 0, 7.174870, -0.988216, 0, 7.148275, -0.996794, 0, 7.150082, -0.913802, 0, 7.174870, -0.988216, 0, 7.201139, -0.978174, 0, 7.174870, -0.988216, 0, 7.150082, -0.913802, 0, 7.201139, -0.978174, 0, 7.175846, -0.905208, 0, 7.150082, -0.913802, 0, 7.201139, -0.978174, 0, 7.150082, -0.913802, 0, 7.175846, -0.905208, 0, 7.201139, -0.978174, 0, 7.175846, -0.905208, 0, 7.201139, -0.978174, 0, 7.227084, -0.966667, 0, 7.201139, -0.978174, 0, 7.175846, -0.905208, 0, 7.227084, -0.966667, 0, 7.201514, -0.895052, 0, 7.175846, -0.905208, 0, 7.227084, -0.966667, 0, 7.175846, -0.905208, 0, 7.201514, -0.895052, 0, 7.227084, -0.966667, 0, 7.201514, -0.895052, 0, 7.227084, -0.966667, 0, 7.227084, -0.883333, 0, 7.227084, -0.966667, 0, 7.201514, -0.895052, 0, 7.227084, -0.883333, 0, 7.153646, -0.650000, 0, 7.150505, -0.613900, 0, 7.153646, -0.650000, 0, 7.153646, -0.650000, 0, 7.150505, -0.613900, 0, 7.150505, -0.613900, 0, 7.150505, -0.613900, 0, 7.142123, -0.581641, 0, 7.150505, -0.613900, 0, 7.150505, -0.613900, 0, 7.142123, -0.581641, 0, 7.142123, -0.581641, 0, 7.142123, -0.581641, 0, 7.128500, -0.553223, 0, 7.142123, -0.581641, 0, 7.142123, -0.581641, 0, 7.128500, -0.553223, 0, 7.128500, -0.553223, 0, 7.128500, -0.553223, 0, 7.109635, -0.528646, 0, 7.128500, -0.553223, 0, 7.128500, -0.553223, 0, 7.109635, -0.528646, 0, 7.109635, -0.528646, 0, 7.109635, -0.528646, 0, 7.086410, -0.508822, 0, 7.109635, -0.528646, 0, 7.109635, -0.528646, 0, 7.086410, -0.508822, 0, 7.086410, -0.508822, 0, 7.086410, -0.508822, 0, 7.059701, -0.494661, 0, 7.086410, -0.508822, 0, 7.086410, -0.508822, 0, 7.059701, -0.494661, 0, 7.059701, -0.494661, 0, 7.059701, -0.494661, 0, 7.029508, -0.486165, 0, 7.059701, -0.494661, 0, 7.059701, -0.494661, 0, 7.029508, -0.486165, 0, 7.029508, -0.486165, 0, 7.029508, -0.486165, 0, 6.995834, -0.483333, 0, 7.029508, -0.486165, 0, 7.029508, -0.486165, 0, 6.995834, -0.483333, 0, 6.995834, -0.483333, 0, 6.995834, -0.483333, 0, 6.957731, -0.486068, 0, 6.995834, -0.483333, 0, 6.995834, -0.483333, 0, 6.957731, -0.486068, 0, 6.957731, -0.486068, 0, 6.957731, -0.486068, 0, 6.923633, -0.494271, 0, 6.957731, -0.486068, 0, 6.957731, -0.486068, 0, 6.923633, -0.494271, 0, 6.923633, -0.494271, 0, 6.923633, -0.494271, 0, 6.893539, -0.507943, 0, 6.923633, -0.494271, 0, 6.923633, -0.494271, 0, 6.893539, -0.507943, 0, 6.893539, -0.507943, 0, 6.893539, -0.507943, 0, 6.867448, -0.527083, 0, 6.893539, -0.507943, 0, 6.893539, -0.507943, 0, 6.867448, -0.527083, 0, 6.867448, -0.527083, 0, 6.867448, -0.527083, 0, 6.845914, -0.551204, 0, 6.867448, -0.527083, 0, 6.867448, -0.527083, 0, 6.845914, -0.551204, 0, 6.845914, -0.551204, 0, 6.845914, -0.551204, 0, 6.829492, -0.579818, 0, 6.845914, -0.551204, 0, 6.845914, -0.551204, 0, 6.829492, -0.579818, 0, 6.829492, -0.579818, 0, 6.829492, -0.579818, 0, 6.818180, -0.612923, 0, 6.829492, -0.579818, 0, 6.829492, -0.579818, 0, 6.818180, -0.612923, 0, 6.818180, -0.612923, 0, 6.818180, -0.612923, 0, 6.811979, -0.650521, 0, 6.818180, -0.612923, 0, 6.818180, -0.612923, 0, 6.811979, -0.650521, 0, 6.811979, -0.650521, 0, 7.820313, -0.450000, 0, 7.820313, -0.533333, 0, 7.820313, -0.450000, 0, 7.820313, -0.450000, 0, 7.820313, -0.533333, 0, 7.820313, -0.533333, 0, 7.820313, -0.533333, 0, 7.799984, -0.521615, 0, 7.820313, -0.533333, 0, 7.820313, -0.533333, 0, 7.799984, -0.521615, 0, 7.799984, -0.521615, 0, 7.799984, -0.521615, 0, 7.779623, -0.511458, 0, 7.799984, -0.521615, 0, 7.799984, -0.521615, 0, 7.779623, -0.511458, 0, 7.779623, -0.511458, 0, 7.779623, -0.511458, 0, 7.759229, -0.502865, 0, 7.779623, -0.511458, 0, 7.779623, -0.511458, 0, 7.759229, -0.502865, 0, 7.759229, -0.502865, 0, 7.759229, -0.502865, 0, 7.738802, -0.495833, 0, 7.759229, -0.502865, 0, 7.759229, -0.502865, 0, 7.738802, -0.495833, 0, 7.738802, -0.495833, 0, 7.738802, -0.495833, 0, 7.718311, -0.490365, 0, 7.738802, -0.495833, 0, 7.738802, -0.495833, 0, 7.718311, -0.490365, 0, 7.718311, -0.490365, 0, 7.718311, -0.490365, 0, 7.697721, -0.486458, 0, 7.718311, -0.490365, 0, 7.718311, -0.490365, 0, 7.697721, -0.486458, 0, 7.697721, -0.486458, 0, 7.697721, -0.486458, 0, 7.677035, -0.484115, 0, 7.697721, -0.486458, 0, 7.697721, -0.486458, 0, 7.677035, -0.484115, 0, 7.677035, -0.484115, 0, 7.677035, -0.484115, 0, 7.656250, -0.483333, 0, 7.677035, -0.484115, 0, 7.677035, -0.484115, 0, 7.656250, -0.483333, 0, 7.656250, -0.483333, 0, 7.656250, -0.483333, 0, 7.612240, -0.487028, 0, 7.656250, -0.483333, 0, 7.656250, -0.483333, 0, 7.612240, -0.487028, 0, 7.612240, -0.487028, 0, 7.612240, -0.487028, 0, 7.573438, -0.498112, 0, 7.612240, -0.487028, 0, 7.612240, -0.487028, 0, 7.573438, -0.498112, 0, 7.573438, -0.498112, 0, 7.573438, -0.498112, 0, 7.539844, -0.516585, 0, 7.573438, -0.498112, 0, 7.573438, -0.498112, 0, 7.539844, -0.516585, 0, 7.539844, -0.516585, 0, 7.539844, -0.516585, 0, 7.511459, -0.542448, 0, 7.539844, -0.516585, 0, 7.539844, -0.516585, 0, 7.511459, -0.542448, 0, 7.511459, -0.542448, 0, 7.511459, -0.542448, 0, 7.488900, -0.574984, 0, 7.511459, -0.542448, 0, 7.511459, -0.542448, 0, 7.488900, -0.574984, 0, 7.488900, -0.574984, 0, 7.488900, -0.574984, 0, 7.472787, -0.613477, 0, 7.488900, -0.574984, 0, 7.488900, -0.574984, 0, 7.472787, -0.613477, 0, 7.472787, -0.613477, 0, 7.472787, -0.613477, 0, 7.463119, -0.657926, 0, 7.472787, -0.613477, 0, 7.472787, -0.613477, 0, 7.463119, -0.657926, 0, 7.463119, -0.657926, 0, 7.463119, -0.657926, 0, 7.459896, -0.708333, 0, 7.463119, -0.657926, 0, 7.463119, -0.657926, 0, 7.459896, -0.708333, 0, 7.459896, -0.708333, 0, 7.459896, -0.708333, 0, 7.463119, -0.758740, 0, 7.459896, -0.708333, 0, 7.459896, -0.708333, 0, 7.463119, -0.758740, 0, 7.463119, -0.758740, 0, 7.463119, -0.758740, 0, 7.472787, -0.803190, 0, 7.463119, -0.758740, 0, 7.463119, -0.758740, 0, 7.472787, -0.803190, 0, 7.472787, -0.803190, 0, 7.472787, -0.803190, 0, 7.488900, -0.841683, 0, 7.472787, -0.803190, 0, 7.472787, -0.803190, 0, 7.488900, -0.841683, 0, 7.488900, -0.841683, 0, 7.488900, -0.841683, 0, 7.511459, -0.874219, 0, 7.488900, -0.841683, 0, 7.488900, -0.841683, 0, 7.511459, -0.874219, 0, 7.511459, -0.874219, 0, 7.511459, -0.874219, 0, 7.539844, -0.900081, 0, 7.511459, -0.874219, 0, 7.511459, -0.874219, 0, 7.539844, -0.900081, 0, 7.539844, -0.900081, 0, 7.539844, -0.900081, 0, 7.573438, -0.918555, 0, 7.539844, -0.900081, 0, 7.539844, -0.900081, 0, 7.573438, -0.918555, 0, 7.573438, -0.918555, 0, 7.573438, -0.918555, 0, 7.612240, -0.929639, 0, 7.573438, -0.918555, 0, 7.573438, -0.918555, 0, 7.612240, -0.929639, 0, 7.612240, -0.929639, 0, 7.612240, -0.929639, 0, 7.656250, -0.933333, 0, 7.612240, -0.929639, 0, 7.612240, -0.929639, 0, 7.656250, -0.933333, 0, 7.656250, -0.933333, 0, 7.656250, -0.933333, 0, 7.677035, -0.932552, 0, 7.656250, -0.933333, 0, 7.656250, -0.933333, 0, 7.677035, -0.932552, 0, 7.677035, -0.932552, 0, 7.677035, -0.932552, 0, 7.697721, -0.930208, 0, 7.677035, -0.932552, 0, 7.677035, -0.932552, 0, 7.697721, -0.930208, 0, 7.697721, -0.930208, 0, 7.697721, -0.930208, 0, 7.718311, -0.926302, 0, 7.697721, -0.930208, 0, 7.697721, -0.930208, 0, 7.718311, -0.926302, 0, 7.718311, -0.926302, 0, 7.718311, -0.926302, 0, 7.738802, -0.920833, 0, 7.718311, -0.926302, 0, 7.718311, -0.926302, 0, 7.738802, -0.920833, 0, 7.738802, -0.920833, 0, 7.738802, -0.920833, 0, 7.759229, -0.913802, 0, 7.738802, -0.920833, 0, 7.738802, -0.920833, 0, 7.759229, -0.913802, 0, 7.759229, -0.913802, 0, 7.759229, -0.913802, 0, 7.779623, -0.905208, 0, 7.759229, -0.913802, 0, 7.759229, -0.913802, 0, 7.779623, -0.905208, 0, 7.779623, -0.905208, 0, 7.779623, -0.905208, 0, 7.799984, -0.895052, 0, 7.779623, -0.905208, 0, 7.779623, -0.905208, 0, 7.799984, -0.895052, 0, 7.799984, -0.895052, 0, 7.799984, -0.895052, 0, 7.820313, -0.883333, 0, 7.799984, -0.895052, 0, 7.799984, -0.895052, 0, 7.820313, -0.883333, 0, 7.820313, -0.883333, 0, 7.820313, -0.883333, 0, 7.820313, -0.966667, 0, 7.820313, -0.883333, 0, 7.820313, -0.883333, 0, 7.820313, -0.966667, 0, 7.820313, -0.966667, 0, 7.820313, -0.966667, 0, 7.800082, -0.978385, 0, 7.820313, -0.966667, 0, 7.820313, -0.966667, 0, 7.800082, -0.978385, 0, 7.800082, -0.978385, 0, 7.800082, -0.978385, 0, 7.779492, -0.988542, 0, 7.800082, -0.978385, 0, 7.800082, -0.978385, 0, 7.779492, -0.988542, 0, 7.779492, -0.988542, 0, 7.779492, -0.988542, 0, 7.758545, -0.997135, 0, 7.779492, -0.988542, 0, 7.779492, -0.988542, 0, 7.758545, -0.997135, 0, 7.758545, -0.997135, 0, 7.758545, -0.997135, 0, 7.737240, -1.004167, 0, 7.758545, -0.997135, 0, 7.758545, -0.997135, 0, 7.737240, -1.004167, 0, 7.737240, -1.004167, 0, 7.737240, -1.004167, 0, 7.715414, -1.009635, 0, 7.737240, -1.004167, 0, 7.737240, -1.004167, 0, 7.715414, -1.009635, 0, 7.715414, -1.009635, 0, 7.715414, -1.009635, 0, 7.692904, -1.013542, 0, 7.715414, -1.009635, 0, 7.715414, -1.009635, 0, 7.692904, -1.013542, 0, 7.692904, -1.013542, 0, 7.692904, -1.013542, 0, 7.669711, -1.015885, 0, 7.692904, -1.013542, 0, 7.692904, -1.013542, 0, 7.669711, -1.015885, 0, 7.669711, -1.015885, 0, 7.669711, -1.015885, 0, 7.645833, -1.016667, 0, 7.669711, -1.015885, 0, 7.669711, -1.015885, 0, 7.645833, -1.016667, 0, 7.645833, -1.016667, 0, 7.645833, -1.016667, 0, 7.583333, -1.011458, 0, 7.645833, -1.016667, 0, 7.645833, -1.016667, 0, 7.583333, -1.011458, 0, 7.583333, -1.011458, 0, 7.583333, -1.011458, 0, 7.527604, -0.995833, 0, 7.583333, -1.011458, 0, 7.583333, -1.011458, 0, 7.527604, -0.995833, 0, 7.527604, -0.995833, 0, 7.527604, -0.995833, 0, 7.478646, -0.969792, 0, 7.527604, -0.995833, 0, 7.527604, -0.995833, 0, 7.478646, -0.969792, 0, 7.478646, -0.969792, 0, 7.478646, -0.969792, 0, 7.436459, -0.933333, 0, 7.478646, -0.969792, 0, 7.478646, -0.969792, 0, 7.436459, -0.933333, 0, 7.436459, -0.933333, 0, 7.436459, -0.933333, 0, 7.402506, -0.888021, 0, 7.436459, -0.933333, 0, 7.436459, -0.933333, 0, 7.402506, -0.888021, 0, 7.402506, -0.888021, 0, 7.402506, -0.888021, 0, 7.378255, -0.835417, 0, 7.402506, -0.888021, 0, 7.402506, -0.888021, 0, 7.378255, -0.835417, 0, 7.378255, -0.835417, 0, 7.378255, -0.835417, 0, 7.363705, -0.775521, 0, 7.378255, -0.835417, 0, 7.378255, -0.835417, 0, 7.363705, -0.775521, 0, 7.363705, -0.775521, 0, 7.363705, -0.775521, 0, 7.358854, -0.708333, 0, 7.363705, -0.775521, 0, 7.363705, -0.775521, 0, 7.358854, -0.708333, 0, 7.358854, -0.708333, 0, 7.358854, -0.708333, 0, 7.363754, -0.640299, 0, 7.358854, -0.708333, 0, 7.358854, -0.708333, 0, 7.363754, -0.640299, 0, 7.363754, -0.640299, 0, 7.363754, -0.640299, 0, 7.378451, -0.579948, 0, 7.363754, -0.640299, 0, 7.363754, -0.640299, 0, 7.378451, -0.579948, 0, 7.378451, -0.579948, 0, 7.378451, -0.579948, 0, 7.402946, -0.527279, 0, 7.378451, -0.579948, 0, 7.378451, -0.579948, 0, 7.402946, -0.527279, 0, 7.402946, -0.527279, 0, 7.402946, -0.527279, 0, 7.437240, -0.482292, 0, 7.402946, -0.527279, 0, 7.402946, -0.527279, 0, 7.437240, -0.482292, 0, 7.437240, -0.482292, 0, 7.437240, -0.482292, 0, 7.480062, -0.446289, 0, 7.437240, -0.482292, 0, 7.437240, -0.482292, 0, 7.480062, -0.446289, 0, 7.480062, -0.446289, 0, 7.480062, -0.446289, 0, 7.530143, -0.420573, 0, 7.480062, -0.446289, 0, 7.480062, -0.446289, 0, 7.530143, -0.420573, 0, 7.530143, -0.420573, 0, 7.530143, -0.420573, 0, 7.587484, -0.405143, 0, 7.530143, -0.420573, 0, 7.530143, -0.420573, 0, 7.587484, -0.405143, 0, 7.587484, -0.405143, 0, 7.587484, -0.405143, 0, 7.652084, -0.400000, 0, 7.587484, -0.405143, 0, 7.587484, -0.405143, 0, 7.652084, -0.400000, 0, 7.652084, -0.400000, 0, 7.652084, -0.400000, 0, 7.674088, -0.400781, 0, 7.652084, -0.400000, 0, 7.652084, -0.400000, 0, 7.674088, -0.400781, 0, 7.674088, -0.400781, 0, 7.674088, -0.400781, 0, 7.695834, -0.403125, 0, 7.674088, -0.400781, 0, 7.674088, -0.400781, 0, 7.695834, -0.403125, 0, 7.695834, -0.403125, 0, 7.695834, -0.403125, 0, 7.717318, -0.407031, 0, 7.695834, -0.403125, 0, 7.695834, -0.403125, 0, 7.717318, -0.407031, 0, 7.717318, -0.407031, 0, 7.717318, -0.407031, 0, 7.738542, -0.412500, 0, 7.717318, -0.407031, 0, 7.717318, -0.407031, 0, 7.738542, -0.412500, 0, 7.738542, -0.412500, 0, 7.738542, -0.412500, 0, 7.759473, -0.419531, 0, 7.738542, -0.412500, 0, 7.738542, -0.412500, 0, 7.759473, -0.419531, 0, 7.759473, -0.419531, 0, 7.759473, -0.419531, 0, 7.780078, -0.428125, 0, 7.759473, -0.419531, 0, 7.759473, -0.419531, 0, 7.780078, -0.428125, 0, 7.780078, -0.428125, 0, 7.780078, -0.428125, 0, 7.800358, -0.438281, 0, 7.780078, -0.428125, 0, 7.780078, -0.428125, 0, 7.800358, -0.438281, 0, 7.800358, -0.438281, 0, 7.463119, -0.657926, 0, 7.363754, -0.640299, 0, 7.358854, -0.708333, 0, 7.363754, -0.640299, 0, 7.463119, -0.657926, 0, 7.358854, -0.708333, 0, 7.472787, -0.613477, 0, 7.363754, -0.640299, 0, 7.463119, -0.657926, 0, 7.363754, -0.640299, 0, 7.472787, -0.613477, 0, 7.463119, -0.657926, 0, 7.472787, -0.613477, 0, 7.378451, -0.579948, 0, 7.363754, -0.640299, 0, 7.378451, -0.579948, 0, 7.472787, -0.613477, 0, 7.363754, -0.640299, 0, 7.488900, -0.574984, 0, 7.378451, -0.579948, 0, 7.472787, -0.613477, 0, 7.378451, -0.579948, 0, 7.488900, -0.574984, 0, 7.472787, -0.613477, 0, 7.488900, -0.574984, 0, 7.402946, -0.527279, 0, 7.378451, -0.579948, 0, 7.402946, -0.527279, 0, 7.488900, -0.574984, 0, 7.378451, -0.579948, 0, 7.437240, -0.482292, 0, 7.402946, -0.527279, 0, 7.488900, -0.574984, 0, 7.402946, -0.527279, 0, 7.437240, -0.482292, 0, 7.488900, -0.574984, 0, 7.437240, -0.482292, 0, 7.488900, -0.574984, 0, 7.511459, -0.542448, 0, 7.488900, -0.574984, 0, 7.437240, -0.482292, 0, 7.511459, -0.542448, 0, 7.480062, -0.446289, 0, 7.437240, -0.482292, 0, 7.511459, -0.542448, 0, 7.437240, -0.482292, 0, 7.480062, -0.446289, 0, 7.511459, -0.542448, 0, 7.480062, -0.446289, 0, 7.511459, -0.542448, 0, 7.539844, -0.516585, 0, 7.511459, -0.542448, 0, 7.480062, -0.446289, 0, 7.539844, -0.516585, 0, 7.530143, -0.420573, 0, 7.480062, -0.446289, 0, 7.539844, -0.516585, 0, 7.480062, -0.446289, 0, 7.530143, -0.420573, 0, 7.539844, -0.516585, 0, 7.530143, -0.420573, 0, 7.539844, -0.516585, 0, 7.573438, -0.498112, 0, 7.539844, -0.516585, 0, 7.530143, -0.420573, 0, 7.573438, -0.498112, 0, 7.530143, -0.420573, 0, 7.573438, -0.498112, 0, 7.587484, -0.405143, 0, 7.573438, -0.498112, 0, 7.530143, -0.420573, 0, 7.587484, -0.405143, 0, 7.587484, -0.405143, 0, 7.573438, -0.498112, 0, 7.612240, -0.487028, 0, 7.573438, -0.498112, 0, 7.587484, -0.405143, 0, 7.612240, -0.487028, 0, 7.652084, -0.400000, 0, 7.587484, -0.405143, 0, 7.612240, -0.487028, 0, 7.587484, -0.405143, 0, 7.652084, -0.400000, 0, 7.612240, -0.487028, 0, 7.612240, -0.487028, 0, 7.656250, -0.483333, 0, 7.652084, -0.400000, 0, 7.656250, -0.483333, 0, 7.612240, -0.487028, 0, 7.652084, -0.400000, 0, 7.674088, -0.400781, 0, 7.652084, -0.400000, 0, 7.656250, -0.483333, 0, 7.652084, -0.400000, 0, 7.674088, -0.400781, 0, 7.656250, -0.483333, 0, 7.674088, -0.400781, 0, 7.656250, -0.483333, 0, 7.677035, -0.484115, 0, 7.656250, -0.483333, 0, 7.674088, -0.400781, 0, 7.677035, -0.484115, 0, 7.674088, -0.400781, 0, 7.677035, -0.484115, 0, 7.695834, -0.403125, 0, 7.677035, -0.484115, 0, 7.674088, -0.400781, 0, 7.695834, -0.403125, 0, 7.695834, -0.403125, 0, 7.677035, -0.484115, 0, 7.697721, -0.486458, 0, 7.677035, -0.484115, 0, 7.695834, -0.403125, 0, 7.697721, -0.486458, 0, 7.717318, -0.407031, 0, 7.695834, -0.403125, 0, 7.697721, -0.486458, 0, 7.695834, -0.403125, 0, 7.717318, -0.407031, 0, 7.697721, -0.486458, 0, 7.717318, -0.407031, 0, 7.697721, -0.486458, 0, 7.718311, -0.490365, 0, 7.697721, -0.486458, 0, 7.717318, -0.407031, 0, 7.718311, -0.490365, 0, 7.717318, -0.407031, 0, 7.718311, -0.490365, 0, 7.738542, -0.412500, 0, 7.718311, -0.490365, 0, 7.717318, -0.407031, 0, 7.738542, -0.412500, 0, 7.759473, -0.419531, 0, 7.738542, -0.412500, 0, 7.718311, -0.490365, 0, 7.738542, -0.412500, 0, 7.759473, -0.419531, 0, 7.718311, -0.490365, 0, 7.759473, -0.419531, 0, 7.718311, -0.490365, 0, 7.738802, -0.495833, 0, 7.718311, -0.490365, 0, 7.759473, -0.419531, 0, 7.738802, -0.495833, 0, 7.759473, -0.419531, 0, 7.738802, -0.495833, 0, 7.780078, -0.428125, 0, 7.738802, -0.495833, 0, 7.759473, -0.419531, 0, 7.780078, -0.428125, 0, 7.738802, -0.495833, 0, 7.759229, -0.502865, 0, 7.780078, -0.428125, 0, 7.759229, -0.502865, 0, 7.738802, -0.495833, 0, 7.780078, -0.428125, 0, 7.780078, -0.428125, 0, 7.759229, -0.502865, 0, 7.800358, -0.438281, 0, 7.759229, -0.502865, 0, 7.780078, -0.428125, 0, 7.800358, -0.438281, 0, 7.759229, -0.502865, 0, 7.779623, -0.511458, 0, 7.800358, -0.438281, 0, 7.779623, -0.511458, 0, 7.759229, -0.502865, 0, 7.800358, -0.438281, 0, 7.800358, -0.438281, 0, 7.779623, -0.511458, 0, 7.820313, -0.450000, 0, 7.779623, -0.511458, 0, 7.800358, -0.438281, 0, 7.820313, -0.450000, 0, 7.779623, -0.511458, 0, 7.799984, -0.521615, 0, 7.820313, -0.450000, 0, 7.799984, -0.521615, 0, 7.779623, -0.511458, 0, 7.820313, -0.450000, 0, 7.799984, -0.521615, 0, 7.820313, -0.533333, 0, 7.820313, -0.450000, 0, 7.820313, -0.533333, 0, 7.799984, -0.521615, 0, 7.820313, -0.450000, 0, 7.463119, -0.657926, 0, 7.358854, -0.708333, 0, 7.459896, -0.708333, 0, 7.358854, -0.708333, 0, 7.463119, -0.657926, 0, 7.459896, -0.708333, 0, 7.459896, -0.708333, 0, 7.358854, -0.708333, 0, 7.463119, -0.758740, 0, 7.358854, -0.708333, 0, 7.459896, -0.708333, 0, 7.463119, -0.758740, 0, 7.358854, -0.708333, 0, 7.363705, -0.775521, 0, 7.463119, -0.758740, 0, 7.363705, -0.775521, 0, 7.358854, -0.708333, 0, 7.463119, -0.758740, 0, 7.463119, -0.758740, 0, 7.363705, -0.775521, 0, 7.472787, -0.803190, 0, 7.363705, -0.775521, 0, 7.463119, -0.758740, 0, 7.472787, -0.803190, 0, 7.363705, -0.775521, 0, 7.378255, -0.835417, 0, 7.472787, -0.803190, 0, 7.378255, -0.835417, 0, 7.363705, -0.775521, 0, 7.472787, -0.803190, 0, 7.472787, -0.803190, 0, 7.378255, -0.835417, 0, 7.402506, -0.888021, 0, 7.378255, -0.835417, 0, 7.472787, -0.803190, 0, 7.402506, -0.888021, 0, 7.488900, -0.841683, 0, 7.472787, -0.803190, 0, 7.402506, -0.888021, 0, 7.472787, -0.803190, 0, 7.488900, -0.841683, 0, 7.402506, -0.888021, 0, 7.488900, -0.841683, 0, 7.402506, -0.888021, 0, 7.436459, -0.933333, 0, 7.402506, -0.888021, 0, 7.488900, -0.841683, 0, 7.436459, -0.933333, 0, 7.511459, -0.874219, 0, 7.488900, -0.841683, 0, 7.436459, -0.933333, 0, 7.488900, -0.841683, 0, 7.511459, -0.874219, 0, 7.436459, -0.933333, 0, 7.436459, -0.933333, 0, 7.478646, -0.969792, 0, 7.511459, -0.874219, 0, 7.478646, -0.969792, 0, 7.436459, -0.933333, 0, 7.511459, -0.874219, 0, 7.511459, -0.874219, 0, 7.478646, -0.969792, 0, 7.539844, -0.900081, 0, 7.478646, -0.969792, 0, 7.511459, -0.874219, 0, 7.539844, -0.900081, 0, 7.539844, -0.900081, 0, 7.478646, -0.969792, 0, 7.527604, -0.995833, 0, 7.478646, -0.969792, 0, 7.539844, -0.900081, 0, 7.527604, -0.995833, 0, 7.573438, -0.918555, 0, 7.539844, -0.900081, 0, 7.527604, -0.995833, 0, 7.539844, -0.900081, 0, 7.573438, -0.918555, 0, 7.527604, -0.995833, 0, 7.573438, -0.918555, 0, 7.527604, -0.995833, 0, 7.583333, -1.011458, 0, 7.527604, -0.995833, 0, 7.573438, -0.918555, 0, 7.583333, -1.011458, 0, 7.612240, -0.929639, 0, 7.573438, -0.918555, 0, 7.583333, -1.011458, 0, 7.573438, -0.918555, 0, 7.612240, -0.929639, 0, 7.583333, -1.011458, 0, 7.612240, -0.929639, 0, 7.583333, -1.011458, 0, 7.645833, -1.016667, 0, 7.583333, -1.011458, 0, 7.612240, -0.929639, 0, 7.645833, -1.016667, 0, 7.656250, -0.933333, 0, 7.612240, -0.929639, 0, 7.645833, -1.016667, 0, 7.612240, -0.929639, 0, 7.656250, -0.933333, 0, 7.645833, -1.016667, 0, 7.656250, -0.933333, 0, 7.645833, -1.016667, 0, 7.669711, -1.015885, 0, 7.645833, -1.016667, 0, 7.656250, -0.933333, 0, 7.669711, -1.015885, 0, 7.656250, -0.933333, 0, 7.669711, -1.015885, 0, 7.677035, -0.932552, 0, 7.669711, -1.015885, 0, 7.656250, -0.933333, 0, 7.677035, -0.932552, 0, 7.677035, -0.932552, 0, 7.669711, -1.015885, 0, 7.692904, -1.013542, 0, 7.669711, -1.015885, 0, 7.677035, -0.932552, 0, 7.692904, -1.013542, 0, 7.697721, -0.930208, 0, 7.677035, -0.932552, 0, 7.692904, -1.013542, 0, 7.677035, -0.932552, 0, 7.697721, -0.930208, 0, 7.692904, -1.013542, 0, 7.715414, -1.009635, 0, 7.697721, -0.930208, 0, 7.692904, -1.013542, 0, 7.697721, -0.930208, 0, 7.715414, -1.009635, 0, 7.692904, -1.013542, 0, 7.718311, -0.926302, 0, 7.697721, -0.930208, 0, 7.715414, -1.009635, 0, 7.697721, -0.930208, 0, 7.718311, -0.926302, 0, 7.715414, -1.009635, 0, 7.737240, -1.004167, 0, 7.718311, -0.926302, 0, 7.715414, -1.009635, 0, 7.718311, -0.926302, 0, 7.737240, -1.004167, 0, 7.715414, -1.009635, 0, 7.718311, -0.926302, 0, 7.737240, -1.004167, 0, 7.758545, -0.997135, 0, 7.737240, -1.004167, 0, 7.718311, -0.926302, 0, 7.758545, -0.997135, 0, 7.738802, -0.920833, 0, 7.718311, -0.926302, 0, 7.758545, -0.997135, 0, 7.718311, -0.926302, 0, 7.738802, -0.920833, 0, 7.758545, -0.997135, 0, 7.738802, -0.920833, 0, 7.758545, -0.997135, 0, 7.779492, -0.988542, 0, 7.758545, -0.997135, 0, 7.738802, -0.920833, 0, 7.779492, -0.988542, 0, 7.759229, -0.913802, 0, 7.738802, -0.920833, 0, 7.779492, -0.988542, 0, 7.738802, -0.920833, 0, 7.759229, -0.913802, 0, 7.779492, -0.988542, 0, 7.759229, -0.913802, 0, 7.779492, -0.988542, 0, 7.800082, -0.978385, 0, 7.779492, -0.988542, 0, 7.759229, -0.913802, 0, 7.800082, -0.978385, 0, 7.779623, -0.905208, 0, 7.759229, -0.913802, 0, 7.800082, -0.978385, 0, 7.759229, -0.913802, 0, 7.779623, -0.905208, 0, 7.800082, -0.978385, 0, 7.779623, -0.905208, 0, 7.800082, -0.978385, 0, 7.820313, -0.966667, 0, 7.800082, -0.978385, 0, 7.779623, -0.905208, 0, 7.820313, -0.966667, 0, 7.799984, -0.895052, 0, 7.779623, -0.905208, 0, 7.820313, -0.966667, 0, 7.779623, -0.905208, 0, 7.799984, -0.895052, 0, 7.820313, -0.966667, 0, 7.799984, -0.895052, 0, 7.820313, -0.966667, 0, 7.820313, -0.883333, 0, 7.820313, -0.966667, 0, 7.799984, -0.895052, 0, 7.820313, -0.883333, 0, 8.078647, -0.250000, 0, 8.078647, -0.416667, 0, 8.078647, -0.250000, 0, 8.078647, -0.250000, 0, 8.078647, -0.416667, 0, 8.078647, -0.416667, 0, 8.078647, -0.416667, 0, 8.276042, -0.416667, 0, 8.078647, -0.416667, 0, 8.078647, -0.416667, 0, 8.276042, -0.416667, 0, 8.276042, -0.416667, 0, 8.276042, -0.416667, 0, 8.276042, -0.500000, 0, 8.276042, -0.416667, 0, 8.276042, -0.416667, 0, 8.276042, -0.500000, 0, 8.276042, -0.500000, 0, 8.276042, -0.500000, 0, 8.078647, -0.500000, 0, 8.276042, -0.500000, 0, 8.276042, -0.500000, 0, 8.078647, -0.500000, 0, 8.078647, -0.500000, 0, 8.078647, -0.500000, 0, 8.078647, -0.807813, 0, 8.078647, -0.500000, 0, 8.078647, -0.500000, 0, 8.078647, -0.807813, 0, 8.078647, -0.807813, 0, 8.078647, -0.807813, 0, 8.079867, -0.839355, 0, 8.078647, -0.807813, 0, 8.078647, -0.807813, 0, 8.079867, -0.839355, 0, 8.079867, -0.839355, 0, 8.079867, -0.839355, 0, 8.083529, -0.864714, 0, 8.079867, -0.839355, 0, 8.079867, -0.839355, 0, 8.083529, -0.864714, 0, 8.083529, -0.864714, 0, 8.083529, -0.864714, 0, 8.089633, -0.883887, 0, 8.083529, -0.864714, 0, 8.083529, -0.864714, 0, 8.089633, -0.883887, 0, 8.089633, -0.883887, 0, 8.089633, -0.883887, 0, 8.098178, -0.896875, 0, 8.089633, -0.883887, 0, 8.089633, -0.883887, 0, 8.098178, -0.896875, 0, 8.098178, -0.896875, 0, 8.098178, -0.896875, 0, 8.110465, -0.905534, 0, 8.098178, -0.896875, 0, 8.098178, -0.896875, 0, 8.110465, -0.905534, 0, 8.110465, -0.905534, 0, 8.110465, -0.905534, 0, 8.127799, -0.911719, 0, 8.110465, -0.905534, 0, 8.110465, -0.905534, 0, 8.127799, -0.911719, 0, 8.127799, -0.911719, 0, 8.127799, -0.911719, 0, 8.150180, -0.915430, 0, 8.127799, -0.911719, 0, 8.127799, -0.911719, 0, 8.150180, -0.915430, 0, 8.150180, -0.915430, 0, 8.150180, -0.915430, 0, 8.177604, -0.916667, 0, 8.150180, -0.915430, 0, 8.150180, -0.915430, 0, 8.177604, -0.916667, 0, 8.177604, -0.916667, 0, 8.177604, -0.916667, 0, 8.276042, -0.916667, 0, 8.177604, -0.916667, 0, 8.177604, -0.916667, 0, 8.276042, -0.916667, 0, 8.276042, -0.916667, 0, 8.276042, -0.916667, 0, 8.276042, -1, 0, 8.276042, -0.916667, 0, 8.276042, -0.916667, 0, 8.276042, -1, 0, 8.276042, -1, 0, 8.276042, -1, 0, 8.177604, -1, 0, 8.276042, -1, 0, 8.276042, -1, 0, 8.177604, -1, 0, 8.177604, -1, 0, 8.177604, -1, 0, 8.126432, -0.997412, 0, 8.177604, -1, 0, 8.177604, -1, 0, 8.126432, -0.997412, 0, 8.126432, -0.997412, 0, 8.126432, -0.997412, 0, 8.083854, -0.989648, 0, 8.126432, -0.997412, 0, 8.126432, -0.997412, 0, 8.083854, -0.989648, 0, 8.083854, -0.989648, 0, 8.083854, -0.989648, 0, 8.049870, -0.976709, 0, 8.083854, -0.989648, 0, 8.083854, -0.989648, 0, 8.049870, -0.976709, 0, 8.049870, -0.976709, 0, 8.049870, -0.976709, 0, 8.024479, -0.958594, 0, 8.049870, -0.976709, 0, 8.049870, -0.976709, 0, 8.024479, -0.958594, 0, 8.024479, -0.958594, 0, 8.024479, -0.958594, 0, 8.006022, -0.933643, 0, 8.024479, -0.958594, 0, 8.024479, -0.958594, 0, 8.006022, -0.933643, 0, 8.006022, -0.933643, 0, 8.006022, -0.933643, 0, 7.992838, -0.900195, 0, 8.006022, -0.933643, 0, 8.006022, -0.933643, 0, 7.992838, -0.900195, 0, 7.992838, -0.900195, 0, 7.992838, -0.900195, 0, 7.984928, -0.858252, 0, 7.992838, -0.900195, 0, 7.992838, -0.900195, 0, 7.984928, -0.858252, 0, 7.984928, -0.858252, 0, 7.984928, -0.858252, 0, 7.982292, -0.807813, 0, 7.984928, -0.858252, 0, 7.984928, -0.858252, 0, 7.982292, -0.807813, 0, 7.982292, -0.807813, 0, 7.982292, -0.807813, 0, 7.982292, -0.500000, 0, 7.982292, -0.807813, 0, 7.982292, -0.807813, 0, 7.982292, -0.500000, 0, 7.982292, -0.500000, 0, 7.982292, -0.500000, 0, 7.911979, -0.500000, 0, 7.982292, -0.500000, 0, 7.982292, -0.500000, 0, 7.911979, -0.500000, 0, 7.911979, -0.500000, 0, 7.911979, -0.500000, 0, 7.911979, -0.416667, 0, 7.911979, -0.500000, 0, 7.911979, -0.500000, 0, 7.911979, -0.416667, 0, 7.911979, -0.416667, 0, 7.911979, -0.416667, 0, 7.982292, -0.416667, 0, 7.911979, -0.416667, 0, 7.911979, -0.416667, 0, 7.982292, -0.416667, 0, 7.982292, -0.416667, 0, 7.982292, -0.416667, 0, 7.982292, -0.250000, 0, 7.982292, -0.416667, 0, 7.982292, -0.416667, 0, 7.982292, -0.250000, 0, 7.982292, -0.250000, 0, 7.911979, -0.416667, 0, 7.982292, -0.500000, 0, 7.982292, -0.416667, 0, 7.982292, -0.500000, 0, 7.911979, -0.416667, 0, 7.982292, -0.416667, 0, 7.911979, -0.416667, 0, 7.911979, -0.500000, 0, 7.982292, -0.500000, 0, 7.911979, -0.500000, 0, 7.911979, -0.416667, 0, 7.982292, -0.500000, 0, 7.982292, -0.416667, 0, 7.982292, -0.500000, 0, 8.078647, -0.500000, 0, 7.982292, -0.500000, 0, 7.982292, -0.416667, 0, 8.078647, -0.500000, 0, 7.982292, -0.416667, 0, 8.078647, -0.500000, 0, 8.078647, -0.416667, 0, 8.078647, -0.500000, 0, 7.982292, -0.416667, 0, 8.078647, -0.416667, 0, 7.982292, -0.250000, 0, 7.982292, -0.416667, 0, 8.078647, -0.416667, 0, 7.982292, -0.416667, 0, 7.982292, -0.250000, 0, 8.078647, -0.416667, 0, 7.982292, -0.250000, 0, 8.078647, -0.416667, 0, 8.078647, -0.250000, 0, 8.078647, -0.416667, 0, 7.982292, -0.250000, 0, 8.078647, -0.250000, 0, 8.078647, -0.416667, 0, 8.078647, -0.500000, 0, 8.276042, -0.500000, 0, 8.078647, -0.500000, 0, 8.078647, -0.416667, 0, 8.276042, -0.500000, 0, 8.078647, -0.416667, 0, 8.276042, -0.500000, 0, 8.276042, -0.416667, 0, 8.276042, -0.500000, 0, 8.078647, -0.416667, 0, 8.276042, -0.416667, 0, 7.982292, -0.500000, 0, 8.078647, -0.807813, 0, 8.078647, -0.500000, 0, 8.078647, -0.807813, 0, 7.982292, -0.500000, 0, 8.078647, -0.500000, 0, 7.982292, -0.500000, 0, 7.982292, -0.807813, 0, 8.078647, -0.807813, 0, 7.982292, -0.807813, 0, 7.982292, -0.500000, 0, 8.078647, -0.807813, 0, 8.078647, -0.807813, 0, 7.982292, -0.807813, 0, 8.079867, -0.839355, 0, 7.982292, -0.807813, 0, 8.078647, -0.807813, 0, 8.079867, -0.839355, 0, 7.982292, -0.807813, 0, 7.984928, -0.858252, 0, 8.079867, -0.839355, 0, 7.984928, -0.858252, 0, 7.982292, -0.807813, 0, 8.079867, -0.839355, 0, 8.079867, -0.839355, 0, 7.984928, -0.858252, 0, 8.083529, -0.864714, 0, 7.984928, -0.858252, 0, 8.079867, -0.839355, 0, 8.083529, -0.864714, 0, 7.984928, -0.858252, 0, 7.992838, -0.900195, 0, 8.083529, -0.864714, 0, 7.992838, -0.900195, 0, 7.984928, -0.858252, 0, 8.083529, -0.864714, 0, 8.083529, -0.864714, 0, 7.992838, -0.900195, 0, 8.089633, -0.883887, 0, 7.992838, -0.900195, 0, 8.083529, -0.864714, 0, 8.089633, -0.883887, 0, 8.089633, -0.883887, 0, 7.992838, -0.900195, 0, 8.006022, -0.933643, 0, 7.992838, -0.900195, 0, 8.089633, -0.883887, 0, 8.006022, -0.933643, 0, 8.098178, -0.896875, 0, 8.089633, -0.883887, 0, 8.006022, -0.933643, 0, 8.089633, -0.883887, 0, 8.098178, -0.896875, 0, 8.006022, -0.933643, 0, 8.006022, -0.933643, 0, 8.024479, -0.958594, 0, 8.098178, -0.896875, 0, 8.024479, -0.958594, 0, 8.006022, -0.933643, 0, 8.098178, -0.896875, 0, 8.098178, -0.896875, 0, 8.024479, -0.958594, 0, 8.049870, -0.976709, 0, 8.024479, -0.958594, 0, 8.098178, -0.896875, 0, 8.049870, -0.976709, 0, 8.110465, -0.905534, 0, 8.098178, -0.896875, 0, 8.049870, -0.976709, 0, 8.098178, -0.896875, 0, 8.110465, -0.905534, 0, 8.049870, -0.976709, 0, 8.049870, -0.976709, 0, 8.083854, -0.989648, 0, 8.110465, -0.905534, 0, 8.083854, -0.989648, 0, 8.049870, -0.976709, 0, 8.110465, -0.905534, 0, 8.110465, -0.905534, 0, 8.083854, -0.989648, 0, 8.127799, -0.911719, 0, 8.083854, -0.989648, 0, 8.110465, -0.905534, 0, 8.127799, -0.911719, 0, 8.083854, -0.989648, 0, 8.126432, -0.997412, 0, 8.127799, -0.911719, 0, 8.126432, -0.997412, 0, 8.083854, -0.989648, 0, 8.127799, -0.911719, 0, 8.127799, -0.911719, 0, 8.126432, -0.997412, 0, 8.150180, -0.915430, 0, 8.126432, -0.997412, 0, 8.127799, -0.911719, 0, 8.150180, -0.915430, 0, 8.150180, -0.915430, 0, 8.126432, -0.997412, 0, 8.177604, -1, 0, 8.126432, -0.997412, 0, 8.150180, -0.915430, 0, 8.177604, -1, 0, 8.177604, -0.916667, 0, 8.150180, -0.915430, 0, 8.177604, -1, 0, 8.150180, -0.915430, 0, 8.177604, -0.916667, 0, 8.177604, -1, 0, 8.177604, -0.916667, 0, 8.177604, -1, 0, 8.276042, -1, 0, 8.177604, -1, 0, 8.177604, -0.916667, 0, 8.276042, -1, 0, 8.177604, -0.916667, 0, 8.276042, -1, 0, 8.276042, -0.916667, 0, 8.276042, -1, 0, 8.177604, -0.916667, 0, 8.276042, -0.916667, 0, 8.400521, -0.416667, 0, 8.496354, -0.416667, 0, 8.400521, -0.416667, 0, 8.400521, -0.416667, 0, 8.496354, -0.416667, 0, 8.496354, -0.416667, 0, 8.496354, -0.416667, 0, 8.496354, -1, 0, 8.496354, -0.416667, 0, 8.496354, -0.416667, 0, 8.496354, -1, 0, 8.496354, -1, 0, 8.496354, -1, 0, 8.400521, -1, 0, 8.496354, -1, 0, 8.496354, -1, 0, 8.400521, -1, 0, 8.400521, -1, 0, 8.400521, -0.416667, 0, 8.496354, -1, 0, 8.496354, -0.416667, 0, 8.496354, -1, 0, 8.400521, -0.416667, 0, 8.496354, -0.416667, 0, 8.400521, -0.416667, 0, 8.400521, -1, 0, 8.496354, -1, 0, 8.400521, -1, 0, 8.400521, -0.416667, 0, 8.496354, -1, 0, 8.400521, -0.183333, 0, 8.496354, -0.183333, 0, 8.400521, -0.183333, 0, 8.400521, -0.183333, 0, 8.496354, -0.183333, 0, 8.496354, -0.183333, 0, 8.496354, -0.183333, 0, 8.496354, -0.300000, 0, 8.496354, -0.183333, 0, 8.496354, -0.183333, 0, 8.496354, -0.300000, 0, 8.496354, -0.300000, 0, 8.496354, -0.300000, 0, 8.400521, -0.300000, 0, 8.496354, -0.300000, 0, 8.496354, -0.300000, 0, 8.400521, -0.300000, 0, 8.400521, -0.300000, 0, 8.400521, -0.183333, 0, 8.496354, -0.300000, 0, 8.496354, -0.183333, 0, 8.496354, -0.300000, 0, 8.400521, -0.183333, 0, 8.496354, -0.183333, 0, 8.400521, -0.183333, 0, 8.400521, -0.300000, 0, 8.496354, -0.300000, 0, 8.400521, -0.300000, 0, 8.400521, -0.183333, 0, 8.496354, -0.300000, 0, 8.926562, -0.483333, 0, 8.890039, -0.487093, 0, 8.926562, -0.483333, 0, 8.926562, -0.483333, 0, 8.890039, -0.487093, 0, 8.890039, -0.487093, 0, 8.890039, -0.487093, 0, 8.857553, -0.498372, 0, 8.890039, -0.487093, 0, 8.890039, -0.487093, 0, 8.857553, -0.498372, 0, 8.857553, -0.498372, 0, 8.857553, -0.498372, 0, 8.829102, -0.517171, 0, 8.857553, -0.498372, 0, 8.857553, -0.498372, 0, 8.829102, -0.517171, 0, 8.829102, -0.517171, 0, 8.829102, -0.517171, 0, 8.804688, -0.543490, 0, 8.829102, -0.517171, 0, 8.829102, -0.517171, 0, 8.804688, -0.543490, 0, 8.804688, -0.543490, 0, 8.804688, -0.543490, 0, 8.785091, -0.576351, 0, 8.804688, -0.543490, 0, 8.804688, -0.543490, 0, 8.785091, -0.576351, 0, 8.785091, -0.576351, 0, 8.785091, -0.576351, 0, 8.771094, -0.614779, 0, 8.785091, -0.576351, 0, 8.785091, -0.576351, 0, 8.771094, -0.614779, 0, 8.771094, -0.614779, 0, 8.771094, -0.614779, 0, 8.762695, -0.658773, 0, 8.771094, -0.614779, 0, 8.771094, -0.614779, 0, 8.762695, -0.658773, 0, 8.762695, -0.658773, 0, 8.762695, -0.658773, 0, 8.759896, -0.708333, 0, 8.762695, -0.658773, 0, 8.762695, -0.658773, 0, 8.759896, -0.708333, 0, 8.759896, -0.708333, 0, 8.759896, -0.708333, 0, 8.762679, -0.757894, 0, 8.759896, -0.708333, 0, 8.759896, -0.708333, 0, 8.762679, -0.757894, 0, 8.762679, -0.757894, 0, 8.762679, -0.757894, 0, 8.771029, -0.801888, 0, 8.762679, -0.757894, 0, 8.762679, -0.757894, 0, 8.771029, -0.801888, 0, 8.771029, -0.801888, 0, 8.771029, -0.801888, 0, 8.784945, -0.840316, 0, 8.771029, -0.801888, 0, 8.771029, -0.801888, 0, 8.784945, -0.840316, 0, 8.784945, -0.840316, 0, 8.784945, -0.840316, 0, 8.804427, -0.873177, 0, 8.784945, -0.840316, 0, 8.784945, -0.840316, 0, 8.804427, -0.873177, 0, 8.804427, -0.873177, 0, 8.804427, -0.873177, 0, 8.828760, -0.899495, 0, 8.804427, -0.873177, 0, 8.804427, -0.873177, 0, 8.828760, -0.899495, 0, 8.828760, -0.899495, 0, 8.828760, -0.899495, 0, 8.857226, -0.918294, 0, 8.828760, -0.899495, 0, 8.828760, -0.899495, 0, 8.857226, -0.918294, 0, 8.857226, -0.918294, 0, 8.857226, -0.918294, 0, 8.889828, -0.929574, 0, 8.857226, -0.918294, 0, 8.857226, -0.918294, 0, 8.889828, -0.929574, 0, 8.889828, -0.929574, 0, 8.889828, -0.929574, 0, 8.926562, -0.933333, 0, 8.889828, -0.929574, 0, 8.889828, -0.929574, 0, 8.926562, -0.933333, 0, 8.926562, -0.933333, 0, 8.926562, -0.933333, 0, 8.962858, -0.929557, 0, 8.926562, -0.933333, 0, 8.926562, -0.933333, 0, 8.962858, -0.929557, 0, 8.962858, -0.929557, 0, 8.962858, -0.929557, 0, 8.995182, -0.918229, 0, 8.962858, -0.929557, 0, 8.962858, -0.929557, 0, 8.995182, -0.918229, 0, 8.995182, -0.918229, 0, 8.995182, -0.918229, 0, 9.023536, -0.899349, 0, 8.995182, -0.918229, 0, 8.995182, -0.918229, 0, 9.023536, -0.899349, 0, 9.023536, -0.899349, 0, 9.023536, -0.899349, 0, 9.047916, -0.872917, 0, 9.023536, -0.899349, 0, 9.023536, -0.899349, 0, 9.047916, -0.872917, 0, 9.047916, -0.872917, 0, 9.047916, -0.872917, 0, 9.067513, -0.839974, 0, 9.047916, -0.872917, 0, 9.047916, -0.872917, 0, 9.067513, -0.839974, 0, 9.067513, -0.839974, 0, 9.067513, -0.839974, 0, 9.081511, -0.801562, 0, 9.067513, -0.839974, 0, 9.067513, -0.839974, 0, 9.081511, -0.801562, 0, 9.081511, -0.801562, 0, 9.081511, -0.801562, 0, 9.089909, -0.757682, 0, 9.081511, -0.801562, 0, 9.081511, -0.801562, 0, 9.089909, -0.757682, 0, 9.089909, -0.757682, 0, 9.089909, -0.757682, 0, 9.092709, -0.708333, 0, 9.089909, -0.757682, 0, 9.089909, -0.757682, 0, 9.092709, -0.708333, 0, 9.092709, -0.708333, 0, 9.092709, -0.708333, 0, 9.089909, -0.659196, 0, 9.092709, -0.708333, 0, 9.092709, -0.708333, 0, 9.089909, -0.659196, 0, 9.089909, -0.659196, 0, 9.089909, -0.659196, 0, 9.081511, -0.615430, 0, 9.089909, -0.659196, 0, 9.089909, -0.659196, 0, 9.081511, -0.615430, 0, 9.081511, -0.615430, 0, 9.081511, -0.615430, 0, 9.067513, -0.577034, 0, 9.081511, -0.615430, 0, 9.081511, -0.615430, 0, 9.067513, -0.577034, 0, 9.067513, -0.577034, 0, 9.067513, -0.577034, 0, 9.047916, -0.544010, 0, 9.067513, -0.577034, 0, 9.067513, -0.577034, 0, 9.047916, -0.544010, 0, 9.047916, -0.544010, 0, 9.047916, -0.544010, 0, 9.023536, -0.517464, 0, 9.047916, -0.544010, 0, 9.047916, -0.544010, 0, 9.023536, -0.517464, 0, 9.023536, -0.517464, 0, 9.023536, -0.517464, 0, 8.995182, -0.498503, 0, 9.023536, -0.517464, 0, 9.023536, -0.517464, 0, 8.995182, -0.498503, 0, 8.995182, -0.498503, 0, 8.995182, -0.498503, 0, 8.962858, -0.487126, 0, 8.995182, -0.498503, 0, 8.995182, -0.498503, 0, 8.962858, -0.487126, 0, 8.962858, -0.487126, 0, 8.926562, -0.400000, 0, 8.985709, -0.405111, 0, 8.926562, -0.400000, 0, 8.926562, -0.400000, 0, 8.985709, -0.405111, 0, 8.985709, -0.405111, 0, 8.985709, -0.405111, 0, 9.038151, -0.420443, 0, 8.985709, -0.405111, 0, 8.985709, -0.405111, 0, 9.038151, -0.420443, 0, 9.038151, -0.420443, 0, 9.038151, -0.420443, 0, 9.083887, -0.445996, 0, 9.038151, -0.420443, 0, 9.038151, -0.420443, 0, 9.083887, -0.445996, 0, 9.083887, -0.445996, 0, 9.083887, -0.445996, 0, 9.122916, -0.481771, 0, 9.083887, -0.445996, 0, 9.083887, -0.445996, 0, 9.122916, -0.481771, 0, 9.122916, -0.481771, 0, 9.122916, -0.481771, 0, 9.154135, -0.526595, 0, 9.122916, -0.481771, 0, 9.122916, -0.481771, 0, 9.154135, -0.526595, 0, 9.154135, -0.526595, 0, 9.154135, -0.526595, 0, 9.176433, -0.579297, 0, 9.154135, -0.526595, 0, 9.154135, -0.526595, 0, 9.176433, -0.579297, 0, 9.176433, -0.579297, 0, 9.176433, -0.579297, 0, 9.189811, -0.639876, 0, 9.176433, -0.579297, 0, 9.176433, -0.579297, 0, 9.189811, -0.639876, 0, 9.189811, -0.639876, 0, 9.189811, -0.639876, 0, 9.194271, -0.708333, 0, 9.189811, -0.639876, 0, 9.189811, -0.639876, 0, 9.194271, -0.708333, 0, 9.194271, -0.708333, 0, 9.194271, -0.708333, 0, 9.189811, -0.776579, 0, 9.194271, -0.708333, 0, 9.194271, -0.708333, 0, 9.189811, -0.776579, 0, 9.189811, -0.776579, 0, 9.189811, -0.776579, 0, 9.176433, -0.837044, 0, 9.189811, -0.776579, 0, 9.189811, -0.776579, 0, 9.176433, -0.837044, 0, 9.176433, -0.837044, 0, 9.176433, -0.837044, 0, 9.154135, -0.889730, 0, 9.176433, -0.837044, 0, 9.176433, -0.837044, 0, 9.154135, -0.889730, 0, 9.154135, -0.889730, 0, 9.154135, -0.889730, 0, 9.122916, -0.934635, 0, 9.154135, -0.889730, 0, 9.154135, -0.889730, 0, 9.122916, -0.934635, 0, 9.122916, -0.934635, 0, 9.122916, -0.934635, 0, 9.083887, -0.970524, 0, 9.122916, -0.934635, 0, 9.122916, -0.934635, 0, 9.083887, -0.970524, 0, 9.083887, -0.970524, 0, 9.083887, -0.970524, 0, 9.038151, -0.996159, 0, 9.083887, -0.970524, 0, 9.083887, -0.970524, 0, 9.038151, -0.996159, 0, 9.038151, -0.996159, 0, 9.038151, -0.996159, 0, 8.985709, -1.011540, 0, 9.038151, -0.996159, 0, 9.038151, -0.996159, 0, 8.985709, -1.011540, 0, 8.985709, -1.011540, 0, 8.985709, -1.011540, 0, 8.926562, -1.016667, 0, 8.985709, -1.011540, 0, 8.985709, -1.011540, 0, 8.926562, -1.016667, 0, 8.926562, -1.016667, 0, 8.926562, -1.016667, 0, 8.867204, -1.011540, 0, 8.926562, -1.016667, 0, 8.926562, -1.016667, 0, 8.867204, -1.011540, 0, 8.867204, -1.011540, 0, 8.867204, -1.011540, 0, 8.814649, -0.996159, 0, 8.867204, -1.011540, 0, 8.867204, -1.011540, 0, 8.814649, -0.996159, 0, 8.814649, -0.996159, 0, 8.814649, -0.996159, 0, 8.768897, -0.970524, 0, 8.814649, -0.996159, 0, 8.814649, -0.996159, 0, 8.768897, -0.970524, 0, 8.768897, -0.970524, 0, 8.768897, -0.970524, 0, 8.729948, -0.934635, 0, 8.768897, -0.970524, 0, 8.768897, -0.970524, 0, 8.729948, -0.934635, 0, 8.729948, -0.934635, 0, 8.729948, -0.934635, 0, 8.698844, -0.889730, 0, 8.729948, -0.934635, 0, 8.729948, -0.934635, 0, 8.698844, -0.889730, 0, 8.698844, -0.889730, 0, 8.698844, -0.889730, 0, 8.676627, -0.837044, 0, 8.698844, -0.889730, 0, 8.698844, -0.889730, 0, 8.676627, -0.837044, 0, 8.676627, -0.837044, 0, 8.676627, -0.837044, 0, 8.663298, -0.776579, 0, 8.676627, -0.837044, 0, 8.676627, -0.837044, 0, 8.663298, -0.776579, 0, 8.663298, -0.776579, 0, 8.663298, -0.776579, 0, 8.658854, -0.708333, 0, 8.663298, -0.776579, 0, 8.663298, -0.776579, 0, 8.658854, -0.708333, 0, 8.658854, -0.708333, 0, 8.658854, -0.708333, 0, 8.663298, -0.639876, 0, 8.658854, -0.708333, 0, 8.658854, -0.708333, 0, 8.663298, -0.639876, 0, 8.663298, -0.639876, 0, 8.663298, -0.639876, 0, 8.676627, -0.579297, 0, 8.663298, -0.639876, 0, 8.663298, -0.639876, 0, 8.676627, -0.579297, 0, 8.676627, -0.579297, 0, 8.676627, -0.579297, 0, 8.698844, -0.526595, 0, 8.676627, -0.579297, 0, 8.676627, -0.579297, 0, 8.698844, -0.526595, 0, 8.698844, -0.526595, 0, 8.698844, -0.526595, 0, 8.729948, -0.481771, 0, 8.698844, -0.526595, 0, 8.698844, -0.526595, 0, 8.729948, -0.481771, 0, 8.729948, -0.481771, 0, 8.729948, -0.481771, 0, 8.768897, -0.445996, 0, 8.729948, -0.481771, 0, 8.729948, -0.481771, 0, 8.768897, -0.445996, 0, 8.768897, -0.445996, 0, 8.768897, -0.445996, 0, 8.814649, -0.420443, 0, 8.768897, -0.445996, 0, 8.768897, -0.445996, 0, 8.814649, -0.420443, 0, 8.814649, -0.420443, 0, 8.814649, -0.420443, 0, 8.867204, -0.405111, 0, 8.814649, -0.420443, 0, 8.814649, -0.420443, 0, 8.867204, -0.405111, 0, 8.867204, -0.405111, 0, 8.762695, -0.658773, 0, 8.663298, -0.639876, 0, 8.658854, -0.708333, 0, 8.663298, -0.639876, 0, 8.762695, -0.658773, 0, 8.658854, -0.708333, 0, 8.771094, -0.614779, 0, 8.663298, -0.639876, 0, 8.762695, -0.658773, 0, 8.663298, -0.639876, 0, 8.771094, -0.614779, 0, 8.762695, -0.658773, 0, 8.771094, -0.614779, 0, 8.676627, -0.579297, 0, 8.663298, -0.639876, 0, 8.676627, -0.579297, 0, 8.771094, -0.614779, 0, 8.663298, -0.639876, 0, 8.676627, -0.579297, 0, 8.771094, -0.614779, 0, 8.785091, -0.576351, 0, 8.771094, -0.614779, 0, 8.676627, -0.579297, 0, 8.785091, -0.576351, 0, 8.698844, -0.526595, 0, 8.676627, -0.579297, 0, 8.785091, -0.576351, 0, 8.676627, -0.579297, 0, 8.698844, -0.526595, 0, 8.785091, -0.576351, 0, 8.804688, -0.543490, 0, 8.698844, -0.526595, 0, 8.785091, -0.576351, 0, 8.698844, -0.526595, 0, 8.804688, -0.543490, 0, 8.785091, -0.576351, 0, 8.804688, -0.543490, 0, 8.729948, -0.481771, 0, 8.698844, -0.526595, 0, 8.729948, -0.481771, 0, 8.804688, -0.543490, 0, 8.698844, -0.526595, 0, 8.768897, -0.445996, 0, 8.729948, -0.481771, 0, 8.804688, -0.543490, 0, 8.729948, -0.481771, 0, 8.768897, -0.445996, 0, 8.804688, -0.543490, 0, 8.768897, -0.445996, 0, 8.804688, -0.543490, 0, 8.829102, -0.517171, 0, 8.804688, -0.543490, 0, 8.768897, -0.445996, 0, 8.829102, -0.517171, 0, 8.814649, -0.420443, 0, 8.768897, -0.445996, 0, 8.829102, -0.517171, 0, 8.768897, -0.445996, 0, 8.814649, -0.420443, 0, 8.829102, -0.517171, 0, 8.814649, -0.420443, 0, 8.829102, -0.517171, 0, 8.857553, -0.498372, 0, 8.829102, -0.517171, 0, 8.814649, -0.420443, 0, 8.857553, -0.498372, 0, 8.814649, -0.420443, 0, 8.857553, -0.498372, 0, 8.867204, -0.405111, 0, 8.857553, -0.498372, 0, 8.814649, -0.420443, 0, 8.867204, -0.405111, 0, 8.867204, -0.405111, 0, 8.857553, -0.498372, 0, 8.890039, -0.487093, 0, 8.857553, -0.498372, 0, 8.867204, -0.405111, 0, 8.890039, -0.487093, 0, 8.867204, -0.405111, 0, 8.890039, -0.487093, 0, 8.926562, -0.400000, 0, 8.890039, -0.487093, 0, 8.867204, -0.405111, 0, 8.926562, -0.400000, 0, 8.890039, -0.487093, 0, 8.926562, -0.483333, 0, 8.926562, -0.400000, 0, 8.926562, -0.483333, 0, 8.890039, -0.487093, 0, 8.926562, -0.400000, 0, 8.926562, -0.400000, 0, 8.926562, -0.483333, 0, 8.962858, -0.487126, 0, 8.926562, -0.483333, 0, 8.926562, -0.400000, 0, 8.962858, -0.487126, 0, 8.926562, -0.400000, 0, 8.962858, -0.487126, 0, 8.985709, -0.405111, 0, 8.962858, -0.487126, 0, 8.926562, -0.400000, 0, 8.985709, -0.405111, 0, 8.985709, -0.405111, 0, 8.962858, -0.487126, 0, 8.995182, -0.498503, 0, 8.962858, -0.487126, 0, 8.985709, -0.405111, 0, 8.995182, -0.498503, 0, 8.985709, -0.405111, 0, 8.995182, -0.498503, 0, 9.038151, -0.420443, 0, 8.995182, -0.498503, 0, 8.985709, -0.405111, 0, 9.038151, -0.420443, 0, 8.995182, -0.498503, 0, 9.023536, -0.517464, 0, 9.038151, -0.420443, 0, 9.023536, -0.517464, 0, 8.995182, -0.498503, 0, 9.038151, -0.420443, 0, 9.038151, -0.420443, 0, 9.023536, -0.517464, 0, 9.083887, -0.445996, 0, 9.023536, -0.517464, 0, 9.038151, -0.420443, 0, 9.083887, -0.445996, 0, 9.023536, -0.517464, 0, 9.047916, -0.544010, 0, 9.083887, -0.445996, 0, 9.047916, -0.544010, 0, 9.023536, -0.517464, 0, 9.083887, -0.445996, 0, 9.083887, -0.445996, 0, 9.047916, -0.544010, 0, 9.122916, -0.481771, 0, 9.047916, -0.544010, 0, 9.083887, -0.445996, 0, 9.122916, -0.481771, 0, 9.122916, -0.481771, 0, 9.047916, -0.544010, 0, 9.154135, -0.526595, 0, 9.047916, -0.544010, 0, 9.122916, -0.481771, 0, 9.154135, -0.526595, 0, 9.154135, -0.526595, 0, 9.047916, -0.544010, 0, 9.067513, -0.577034, 0, 9.047916, -0.544010, 0, 9.154135, -0.526595, 0, 9.067513, -0.577034, 0, 9.154135, -0.526595, 0, 9.067513, -0.577034, 0, 9.176433, -0.579297, 0, 9.067513, -0.577034, 0, 9.154135, -0.526595, 0, 9.176433, -0.579297, 0, 9.067513, -0.577034, 0, 9.081511, -0.615430, 0, 9.176433, -0.579297, 0, 9.081511, -0.615430, 0, 9.067513, -0.577034, 0, 9.176433, -0.579297, 0, 9.176433, -0.579297, 0, 9.081511, -0.615430, 0, 9.189811, -0.639876, 0, 9.081511, -0.615430, 0, 9.176433, -0.579297, 0, 9.189811, -0.639876, 0, 9.081511, -0.615430, 0, 9.089909, -0.659196, 0, 9.189811, -0.639876, 0, 9.089909, -0.659196, 0, 9.081511, -0.615430, 0, 9.189811, -0.639876, 0, 9.189811, -0.639876, 0, 9.089909, -0.659196, 0, 9.194271, -0.708333, 0, 9.089909, -0.659196, 0, 9.189811, -0.639876, 0, 9.194271, -0.708333, 0, 9.194271, -0.708333, 0, 9.089909, -0.659196, 0, 9.092709, -0.708333, 0, 9.089909, -0.659196, 0, 9.194271, -0.708333, 0, 9.092709, -0.708333, 0, 9.194271, -0.708333, 0, 9.092709, -0.708333, 0, 9.089909, -0.757682, 0, 9.092709, -0.708333, 0, 9.194271, -0.708333, 0, 9.089909, -0.757682, 0, 9.194271, -0.708333, 0, 9.089909, -0.757682, 0, 9.189811, -0.776579, 0, 9.089909, -0.757682, 0, 9.194271, -0.708333, 0, 9.189811, -0.776579, 0, 9.089909, -0.757682, 0, 9.081511, -0.801562, 0, 9.189811, -0.776579, 0, 9.081511, -0.801562, 0, 9.089909, -0.757682, 0, 9.189811, -0.776579, 0, 9.081511, -0.801562, 0, 9.176433, -0.837044, 0, 9.189811, -0.776579, 0, 9.176433, -0.837044, 0, 9.081511, -0.801562, 0, 9.189811, -0.776579, 0, 9.081511, -0.801562, 0, 9.067513, -0.839974, 0, 9.176433, -0.837044, 0, 9.067513, -0.839974, 0, 9.081511, -0.801562, 0, 9.176433, -0.837044, 0, 9.067513, -0.839974, 0, 9.154135, -0.889730, 0, 9.176433, -0.837044, 0, 9.154135, -0.889730, 0, 9.067513, -0.839974, 0, 9.176433, -0.837044, 0, 9.067513, -0.839974, 0, 9.047916, -0.872917, 0, 9.154135, -0.889730, 0, 9.047916, -0.872917, 0, 9.067513, -0.839974, 0, 9.154135, -0.889730, 0, 9.047916, -0.872917, 0, 9.122916, -0.934635, 0, 9.154135, -0.889730, 0, 9.122916, -0.934635, 0, 9.047916, -0.872917, 0, 9.154135, -0.889730, 0, 9.122916, -0.934635, 0, 9.047916, -0.872917, 0, 9.083887, -0.970524, 0, 9.047916, -0.872917, 0, 9.122916, -0.934635, 0, 9.083887, -0.970524, 0, 9.047916, -0.872917, 0, 9.023536, -0.899349, 0, 9.083887, -0.970524, 0, 9.023536, -0.899349, 0, 9.047916, -0.872917, 0, 9.083887, -0.970524, 0, 9.083887, -0.970524, 0, 9.023536, -0.899349, 0, 9.038151, -0.996159, 0, 9.023536, -0.899349, 0, 9.083887, -0.970524, 0, 9.038151, -0.996159, 0, 9.023536, -0.899349, 0, 8.995182, -0.918229, 0, 9.038151, -0.996159, 0, 8.995182, -0.918229, 0, 9.023536, -0.899349, 0, 9.038151, -0.996159, 0, 9.038151, -0.996159, 0, 8.995182, -0.918229, 0, 8.985709, -1.011540, 0, 8.995182, -0.918229, 0, 9.038151, -0.996159, 0, 8.985709, -1.011540, 0, 8.995182, -0.918229, 0, 8.962858, -0.929557, 0, 8.985709, -1.011540, 0, 8.962858, -0.929557, 0, 8.995182, -0.918229, 0, 8.985709, -1.011540, 0, 8.985709, -1.011540, 0, 8.962858, -0.929557, 0, 8.926562, -1.016667, 0, 8.962858, -0.929557, 0, 8.985709, -1.011540, 0, 8.926562, -1.016667, 0, 8.962858, -0.929557, 0, 8.926562, -0.933333, 0, 8.926562, -1.016667, 0, 8.926562, -0.933333, 0, 8.962858, -0.929557, 0, 8.926562, -1.016667, 0, 8.926562, -0.933333, 0, 8.889828, -0.929574, 0, 8.926562, -1.016667, 0, 8.889828, -0.929574, 0, 8.926562, -0.933333, 0, 8.926562, -1.016667, 0, 8.889828, -0.929574, 0, 8.867204, -1.011540, 0, 8.926562, -1.016667, 0, 8.867204, -1.011540, 0, 8.889828, -0.929574, 0, 8.926562, -1.016667, 0, 8.889828, -0.929574, 0, 8.857226, -0.918294, 0, 8.867204, -1.011540, 0, 8.857226, -0.918294, 0, 8.889828, -0.929574, 0, 8.867204, -1.011540, 0, 8.857226, -0.918294, 0, 8.814649, -0.996159, 0, 8.867204, -1.011540, 0, 8.814649, -0.996159, 0, 8.857226, -0.918294, 0, 8.867204, -1.011540, 0, 8.857226, -0.918294, 0, 8.828760, -0.899495, 0, 8.814649, -0.996159, 0, 8.828760, -0.899495, 0, 8.857226, -0.918294, 0, 8.814649, -0.996159, 0, 8.828760, -0.899495, 0, 8.768897, -0.970524, 0, 8.814649, -0.996159, 0, 8.768897, -0.970524, 0, 8.828760, -0.899495, 0, 8.814649, -0.996159, 0, 8.828760, -0.899495, 0, 8.804427, -0.873177, 0, 8.768897, -0.970524, 0, 8.804427, -0.873177, 0, 8.828760, -0.899495, 0, 8.768897, -0.970524, 0, 8.804427, -0.873177, 0, 8.729948, -0.934635, 0, 8.768897, -0.970524, 0, 8.729948, -0.934635, 0, 8.804427, -0.873177, 0, 8.768897, -0.970524, 0, 8.698844, -0.889730, 0, 8.729948, -0.934635, 0, 8.804427, -0.873177, 0, 8.729948, -0.934635, 0, 8.698844, -0.889730, 0, 8.804427, -0.873177, 0, 8.784945, -0.840316, 0, 8.698844, -0.889730, 0, 8.804427, -0.873177, 0, 8.698844, -0.889730, 0, 8.784945, -0.840316, 0, 8.804427, -0.873177, 0, 8.676627, -0.837044, 0, 8.698844, -0.889730, 0, 8.784945, -0.840316, 0, 8.698844, -0.889730, 0, 8.676627, -0.837044, 0, 8.784945, -0.840316, 0, 8.771029, -0.801888, 0, 8.676627, -0.837044, 0, 8.784945, -0.840316, 0, 8.676627, -0.837044, 0, 8.771029, -0.801888, 0, 8.784945, -0.840316, 0, 8.663298, -0.776579, 0, 8.676627, -0.837044, 0, 8.771029, -0.801888, 0, 8.676627, -0.837044, 0, 8.663298, -0.776579, 0, 8.771029, -0.801888, 0, 8.762679, -0.757894, 0, 8.663298, -0.776579, 0, 8.771029, -0.801888, 0, 8.663298, -0.776579, 0, 8.762679, -0.757894, 0, 8.771029, -0.801888, 0, 8.658854, -0.708333, 0, 8.663298, -0.776579, 0, 8.762679, -0.757894, 0, 8.663298, -0.776579, 0, 8.658854, -0.708333, 0, 8.762679, -0.757894, 0, 8.759896, -0.708333, 0, 8.658854, -0.708333, 0, 8.762679, -0.757894, 0, 8.658854, -0.708333, 0, 8.759896, -0.708333, 0, 8.762679, -0.757894, 0, 8.762695, -0.658773, 0, 8.658854, -0.708333, 0, 8.759896, -0.708333, 0, 8.658854, -0.708333, 0, 8.762695, -0.658773, 0, 8.759896, -0.708333, 0, 9.835417, -0.642708, 0, 9.835417, -1, 0, 9.835417, -0.642708, 0, 9.835417, -0.642708, 0, 9.835417, -1, 0, 9.835417, -1, 0, 9.835417, -1, 0, 9.739584, -1, 0, 9.835417, -1, 0, 9.835417, -1, 0, 9.739584, -1, 0, 9.739584, -1, 0, 9.739584, -1, 0, 9.739584, -0.645833, 0, 9.739584, -1, 0, 9.739584, -1, 0, 9.739584, -0.645833, 0, 9.739584, -0.645833, 0, 9.739584, -0.645833, 0, 9.737565, -0.607642, 0, 9.739584, -0.645833, 0, 9.739584, -0.645833, 0, 9.737565, -0.607642, 0, 9.737565, -0.607642, 0, 9.737565, -0.607642, 0, 9.731510, -0.574577, 0, 9.737565, -0.607642, 0, 9.737565, -0.607642, 0, 9.731510, -0.574577, 0, 9.731510, -0.574577, 0, 9.731510, -0.574577, 0, 9.721419, -0.546639, 0, 9.731510, -0.574577, 0, 9.731510, -0.574577, 0, 9.721419, -0.546639, 0, 9.721419, -0.546639, 0, 9.721419, -0.546639, 0, 9.707292, -0.523828, 0, 9.721419, -0.546639, 0, 9.721419, -0.546639, 0, 9.707292, -0.523828, 0, 9.707292, -0.523828, 0, 9.707292, -0.523828, 0, 9.689128, -0.506112, 0, 9.707292, -0.523828, 0, 9.707292, -0.523828, 0, 9.689128, -0.506112, 0, 9.689128, -0.506112, 0, 9.689128, -0.506112, 0, 9.666927, -0.493457, 0, 9.689128, -0.506112, 0, 9.689128, -0.506112, 0, 9.666927, -0.493457, 0, 9.666927, -0.493457, 0, 9.666927, -0.493457, 0, 9.640690, -0.485864, 0, 9.666927, -0.493457, 0, 9.666927, -0.493457, 0, 9.640690, -0.485864, 0, 9.640690, -0.485864, 0, 9.640690, -0.485864, 0, 9.610416, -0.483333, 0, 9.640690, -0.485864, 0, 9.640690, -0.485864, 0, 9.610416, -0.483333, 0, 9.610416, -0.483333, 0, 9.610416, -0.483333, 0, 9.573666, -0.486385, 0, 9.610416, -0.483333, 0, 9.610416, -0.483333, 0, 9.573666, -0.486385, 0, 9.573666, -0.486385, 0, 9.573666, -0.486385, 0, 9.541016, -0.495540, 0, 9.573666, -0.486385, 0, 9.573666, -0.486385, 0, 9.541016, -0.495540, 0, 9.541016, -0.495540, 0, 9.541016, -0.495540, 0, 9.512467, -0.510799, 0, 9.541016, -0.495540, 0, 9.541016, -0.495540, 0, 9.512467, -0.510799, 0, 9.512467, -0.510799, 0, 9.512467, -0.510799, 0, 9.488021, -0.532161, 0, 9.512467, -0.510799, 0, 9.512467, -0.510799, 0, 9.488021, -0.532161, 0, 9.488021, -0.532161, 0, 9.488021, -0.532161, 0, 9.468425, -0.558797, 0, 9.488021, -0.532161, 0, 9.488021, -0.532161, 0, 9.468425, -0.558797, 0, 9.468425, -0.558797, 0, 9.468425, -0.558797, 0, 9.454428, -0.589876, 0, 9.468425, -0.558797, 0, 9.468425, -0.558797, 0, 9.454428, -0.589876, 0, 9.454428, -0.589876, 0, 9.454428, -0.589876, 0, 9.446029, -0.625399, 0, 9.454428, -0.589876, 0, 9.454428, -0.589876, 0, 9.446029, -0.625399, 0, 9.446029, -0.625399, 0, 9.446029, -0.625399, 0, 9.443229, -0.665365, 0, 9.446029, -0.625399, 0, 9.446029, -0.625399, 0, 9.443229, -0.665365, 0, 9.443229, -0.665365, 0, 9.443229, -0.665365, 0, 9.443229, -1, 0, 9.443229, -0.665365, 0, 9.443229, -0.665365, 0, 9.443229, -1, 0, 9.443229, -1, 0, 9.443229, -1, 0, 9.346875, -1, 0, 9.443229, -1, 0, 9.443229, -1, 0, 9.346875, -1, 0, 9.346875, -1, 0, 9.346875, -1, 0, 9.346875, -0.416667, 0, 9.346875, -1, 0, 9.346875, -1, 0, 9.346875, -0.416667, 0, 9.346875, -0.416667, 0, 9.346875, -0.416667, 0, 9.443229, -0.416667, 0, 9.346875, -0.416667, 0, 9.346875, -0.416667, 0, 9.443229, -0.416667, 0, 9.443229, -0.416667, 0, 9.443229, -0.416667, 0, 9.443229, -0.500000, 0, 9.443229, -0.416667, 0, 9.443229, -0.416667, 0, 9.443229, -0.500000, 0, 9.443229, -0.500000, 0, 9.443229, -0.500000, 0, 9.461182, -0.476457, 0, 9.443229, -0.500000, 0, 9.443229, -0.500000, 0, 9.461182, -0.476457, 0, 9.461182, -0.476457, 0, 9.461182, -0.476457, 0, 9.480664, -0.456087, 0, 9.461182, -0.476457, 0, 9.461182, -0.476457, 0, 9.480664, -0.456087, 0, 9.480664, -0.456087, 0, 9.480664, -0.456087, 0, 9.501677, -0.438892, 0, 9.480664, -0.456087, 0, 9.480664, -0.456087, 0, 9.501677, -0.438892, 0, 9.501677, -0.438892, 0, 9.501677, -0.438892, 0, 9.524219, -0.424870, 0, 9.501677, -0.438892, 0, 9.501677, -0.438892, 0, 9.524219, -0.424870, 0, 9.524219, -0.424870, 0, 9.524219, -0.424870, 0, 9.548422, -0.413989, 0, 9.524219, -0.424870, 0, 9.524219, -0.424870, 0, 9.548422, -0.413989, 0, 9.548422, -0.413989, 0, 9.548422, -0.413989, 0, 9.574414, -0.406217, 0, 9.548422, -0.413989, 0, 9.548422, -0.413989, 0, 9.574414, -0.406217, 0, 9.574414, -0.406217, 0, 9.574414, -0.406217, 0, 9.602198, -0.401554, 0, 9.574414, -0.406217, 0, 9.574414, -0.406217, 0, 9.602198, -0.401554, 0, 9.602198, -0.401554, 0, 9.602198, -0.401554, 0, 9.631771, -0.400000, 0, 9.602198, -0.401554, 0, 9.602198, -0.401554, 0, 9.631771, -0.400000, 0, 9.631771, -0.400000, 0, 9.631771, -0.400000, 0, 9.678971, -0.403849, 0, 9.631771, -0.400000, 0, 9.631771, -0.400000, 0, 9.678971, -0.403849, 0, 9.678971, -0.403849, 0, 9.678971, -0.403849, 0, 9.720053, -0.415397, 0, 9.678971, -0.403849, 0, 9.678971, -0.403849, 0, 9.720053, -0.415397, 0, 9.720053, -0.415397, 0, 9.720053, -0.415397, 0, 9.755013, -0.434644, 0, 9.720053, -0.415397, 0, 9.720053, -0.415397, 0, 9.755013, -0.434644, 0, 9.755013, -0.434644, 0, 9.755013, -0.434644, 0, 9.783854, -0.461589, 0, 9.755013, -0.434644, 0, 9.755013, -0.434644, 0, 9.783854, -0.461589, 0, 9.783854, -0.461589, 0, 9.783854, -0.461589, 0, 9.806413, -0.496004, 0, 9.783854, -0.461589, 0, 9.783854, -0.461589, 0, 9.806413, -0.496004, 0, 9.806413, -0.496004, 0, 9.806413, -0.496004, 0, 9.822526, -0.537663, 0, 9.806413, -0.496004, 0, 9.806413, -0.496004, 0, 9.822526, -0.537663, 0, 9.822526, -0.537663, 0, 9.822526, -0.537663, 0, 9.832194, -0.586564, 0, 9.822526, -0.537663, 0, 9.822526, -0.537663, 0, 9.832194, -0.586564, 0, 9.832194, -0.586564, 0, 9.443229, -0.416667, 0, 9.346875, -0.416667, 0, 9.443229, -0.500000, 0, 9.346875, -0.416667, 0, 9.443229, -0.416667, 0, 9.443229, -0.500000, 0, 9.346875, -0.416667, 0, 9.454428, -0.589876, 0, 9.443229, -0.500000, 0, 9.454428, -0.589876, 0, 9.346875, -0.416667, 0, 9.443229, -0.500000, 0, 9.454428, -0.589876, 0, 9.346875, -0.416667, 0, 9.446029, -0.625399, 0, 9.346875, -0.416667, 0, 9.454428, -0.589876, 0, 9.446029, -0.625399, 0, 9.446029, -0.625399, 0, 9.346875, -0.416667, 0, 9.443229, -0.665365, 0, 9.346875, -0.416667, 0, 9.446029, -0.625399, 0, 9.443229, -0.665365, 0, 9.346875, -0.416667, 0, 9.346875, -1, 0, 9.443229, -0.665365, 0, 9.346875, -1, 0, 9.346875, -0.416667, 0, 9.443229, -0.665365, 0, 9.346875, -1, 0, 9.443229, -1, 0, 9.443229, -0.665365, 0, 9.443229, -1, 0, 9.346875, -1, 0, 9.443229, -0.665365, 0, 9.443229, -0.500000, 0, 9.454428, -0.589876, 0, 9.468425, -0.558797, 0, 9.454428, -0.589876, 0, 9.443229, -0.500000, 0, 9.468425, -0.558797, 0, 9.443229, -0.500000, 0, 9.468425, -0.558797, 0, 9.488021, -0.532161, 0, 9.468425, -0.558797, 0, 9.443229, -0.500000, 0, 9.488021, -0.532161, 0, 9.461182, -0.476457, 0, 9.443229, -0.500000, 0, 9.488021, -0.532161, 0, 9.443229, -0.500000, 0, 9.461182, -0.476457, 0, 9.488021, -0.532161, 0, 9.461182, -0.476457, 0, 9.488021, -0.532161, 0, 9.512467, -0.510799, 0, 9.488021, -0.532161, 0, 9.461182, -0.476457, 0, 9.512467, -0.510799, 0, 9.480664, -0.456087, 0, 9.461182, -0.476457, 0, 9.512467, -0.510799, 0, 9.461182, -0.476457, 0, 9.480664, -0.456087, 0, 9.512467, -0.510799, 0, 9.541016, -0.495540, 0, 9.480664, -0.456087, 0, 9.512467, -0.510799, 0, 9.480664, -0.456087, 0, 9.541016, -0.495540, 0, 9.512467, -0.510799, 0, 9.541016, -0.495540, 0, 9.501677, -0.438892, 0, 9.480664, -0.456087, 0, 9.501677, -0.438892, 0, 9.541016, -0.495540, 0, 9.480664, -0.456087, 0, 9.524219, -0.424870, 0, 9.501677, -0.438892, 0, 9.541016, -0.495540, 0, 9.501677, -0.438892, 0, 9.524219, -0.424870, 0, 9.541016, -0.495540, 0, 9.524219, -0.424870, 0, 9.541016, -0.495540, 0, 9.573666, -0.486385, 0, 9.541016, -0.495540, 0, 9.524219, -0.424870, 0, 9.573666, -0.486385, 0, 9.548422, -0.413989, 0, 9.524219, -0.424870, 0, 9.573666, -0.486385, 0, 9.524219, -0.424870, 0, 9.548422, -0.413989, 0, 9.573666, -0.486385, 0, 9.548422, -0.413989, 0, 9.573666, -0.486385, 0, 9.574414, -0.406217, 0, 9.573666, -0.486385, 0, 9.548422, -0.413989, 0, 9.574414, -0.406217, 0, 9.574414, -0.406217, 0, 9.573666, -0.486385, 0, 9.610416, -0.483333, 0, 9.573666, -0.486385, 0, 9.574414, -0.406217, 0, 9.610416, -0.483333, 0, 9.602198, -0.401554, 0, 9.574414, -0.406217, 0, 9.610416, -0.483333, 0, 9.574414, -0.406217, 0, 9.602198, -0.401554, 0, 9.610416, -0.483333, 0, 9.631771, -0.400000, 0, 9.602198, -0.401554, 0, 9.610416, -0.483333, 0, 9.602198, -0.401554, 0, 9.631771, -0.400000, 0, 9.610416, -0.483333, 0, 9.631771, -0.400000, 0, 9.610416, -0.483333, 0, 9.640690, -0.485864, 0, 9.610416, -0.483333, 0, 9.631771, -0.400000, 0, 9.640690, -0.485864, 0, 9.678971, -0.403849, 0, 9.631771, -0.400000, 0, 9.640690, -0.485864, 0, 9.631771, -0.400000, 0, 9.678971, -0.403849, 0, 9.640690, -0.485864, 0, 9.640690, -0.485864, 0, 9.666927, -0.493457, 0, 9.678971, -0.403849, 0, 9.666927, -0.493457, 0, 9.640690, -0.485864, 0, 9.678971, -0.403849, 0, 9.678971, -0.403849, 0, 9.666927, -0.493457, 0, 9.720053, -0.415397, 0, 9.666927, -0.493457, 0, 9.678971, -0.403849, 0, 9.720053, -0.415397, 0, 9.666927, -0.493457, 0, 9.689128, -0.506112, 0, 9.720053, -0.415397, 0, 9.689128, -0.506112, 0, 9.666927, -0.493457, 0, 9.720053, -0.415397, 0, 9.720053, -0.415397, 0, 9.689128, -0.506112, 0, 9.755013, -0.434644, 0, 9.689128, -0.506112, 0, 9.720053, -0.415397, 0, 9.755013, -0.434644, 0, 9.689128, -0.506112, 0, 9.707292, -0.523828, 0, 9.755013, -0.434644, 0, 9.707292, -0.523828, 0, 9.689128, -0.506112, 0, 9.755013, -0.434644, 0, 9.755013, -0.434644, 0, 9.707292, -0.523828, 0, 9.783854, -0.461589, 0, 9.707292, -0.523828, 0, 9.755013, -0.434644, 0, 9.783854, -0.461589, 0, 9.783854, -0.461589, 0, 9.707292, -0.523828, 0, 9.806413, -0.496004, 0, 9.707292, -0.523828, 0, 9.783854, -0.461589, 0, 9.806413, -0.496004, 0, 9.806413, -0.496004, 0, 9.707292, -0.523828, 0, 9.721419, -0.546639, 0, 9.707292, -0.523828, 0, 9.806413, -0.496004, 0, 9.721419, -0.546639, 0, 9.806413, -0.496004, 0, 9.721419, -0.546639, 0, 9.822526, -0.537663, 0, 9.721419, -0.546639, 0, 9.806413, -0.496004, 0, 9.822526, -0.537663, 0, 9.721419, -0.546639, 0, 9.731510, -0.574577, 0, 9.822526, -0.537663, 0, 9.731510, -0.574577, 0, 9.721419, -0.546639, 0, 9.822526, -0.537663, 0, 9.822526, -0.537663, 0, 9.731510, -0.574577, 0, 9.832194, -0.586564, 0, 9.731510, -0.574577, 0, 9.822526, -0.537663, 0, 9.832194, -0.586564, 0, 9.731510, -0.574577, 0, 9.737565, -0.607642, 0, 9.832194, -0.586564, 0, 9.737565, -0.607642, 0, 9.731510, -0.574577, 0, 9.832194, -0.586564, 0, 9.832194, -0.586564, 0, 9.737565, -0.607642, 0, 9.835417, -0.642708, 0, 9.737565, -0.607642, 0, 9.832194, -0.586564, 0, 9.835417, -0.642708, 0, 9.835417, -0.642708, 0, 9.737565, -0.607642, 0, 9.739584, -0.645833, 0, 9.737565, -0.607642, 0, 9.835417, -0.642708, 0, 9.739584, -0.645833, 0, 9.739584, -0.645833, 0, 9.835417, -1, 0, 9.835417, -0.642708, 0, 9.835417, -1, 0, 9.739584, -0.645833, 0, 9.835417, -0.642708, 0, 9.739584, -0.645833, 0, 9.739584, -1, 0, 9.835417, -1, 0, 9.739584, -1, 0, 9.739584, -0.645833, 0, 9.835417, -1, 0, 10.371354, -0.216667, 0, 10.818229, -0.216667, 0, 10.371354, -0.216667, 0, 10.371354, -0.216667, 0, 10.818229, -0.216667, 0, 10.818229, -0.216667, 0, 10.818229, -0.216667, 0, 10.818229, -0.300000, 0, 10.818229, -0.216667, 0, 10.818229, -0.216667, 0, 10.818229, -0.300000, 0, 10.818229, -0.300000, 0, 10.818229, -0.300000, 0, 10.476563, -0.300000, 0, 10.818229, -0.300000, 0, 10.818229, -0.300000, 0, 10.476563, -0.300000, 0, 10.476563, -0.300000, 0, 10.476563, -0.300000, 0, 10.476563, -0.533333, 0, 10.476563, -0.300000, 0, 10.476563, -0.300000, 0, 10.476563, -0.533333, 0, 10.476563, -0.533333, 0, 10.476563, -0.533333, 0, 10.784896, -0.533333, 0, 10.476563, -0.533333, 0, 10.476563, -0.533333, 0, 10.784896, -0.533333, 0, 10.784896, -0.533333, 0, 10.784896, -0.533333, 0, 10.784896, -0.616667, 0, 10.784896, -0.533333, 0, 10.784896, -0.533333, 0, 10.784896, -0.616667, 0, 10.784896, -0.616667, 0, 10.784896, -0.616667, 0, 10.476563, -0.616667, 0, 10.784896, -0.616667, 0, 10.784896, -0.616667, 0, 10.476563, -0.616667, 0, 10.476563, -0.616667, 0, 10.476563, -0.616667, 0, 10.476563, -1, 0, 10.476563, -0.616667, 0, 10.476563, -0.616667, 0, 10.476563, -1, 0, 10.476563, -1, 0, 10.476563, -1, 0, 10.371354, -1, 0, 10.476563, -1, 0, 10.476563, -1, 0, 10.371354, -1, 0, 10.371354, -1, 0, 10.371354, -0.216667, 0, 10.476563, -0.300000, 0, 10.818229, -0.216667, 0, 10.476563, -0.300000, 0, 10.371354, -0.216667, 0, 10.818229, -0.216667, 0, 10.476563, -0.300000, 0, 10.371354, -0.216667, 0, 10.476563, -0.533333, 0, 10.371354, -0.216667, 0, 10.476563, -0.300000, 0, 10.476563, -0.533333, 0, 10.476563, -0.533333, 0, 10.371354, -0.216667, 0, 10.476563, -0.616667, 0, 10.371354, -0.216667, 0, 10.476563, -0.533333, 0, 10.476563, -0.616667, 0, 10.476563, -0.533333, 0, 10.476563, -0.616667, 0, 10.784896, -0.616667, 0, 10.476563, -0.616667, 0, 10.476563, -0.533333, 0, 10.784896, -0.616667, 0, 10.476563, -0.533333, 0, 10.784896, -0.616667, 0, 10.784896, -0.533333, 0, 10.784896, -0.616667, 0, 10.476563, -0.533333, 0, 10.784896, -0.533333, 0, 10.371354, -0.216667, 0, 10.371354, -1, 0, 10.476563, -0.616667, 0, 10.371354, -1, 0, 10.371354, -0.216667, 0, 10.476563, -0.616667, 0, 10.371354, -1, 0, 10.476563, -1, 0, 10.476563, -0.616667, 0, 10.476563, -1, 0, 10.371354, -1, 0, 10.476563, -0.616667, 0, 10.476563, -0.300000, 0, 10.818229, -0.300000, 0, 10.818229, -0.216667, 0, 10.818229, -0.300000, 0, 10.476563, -0.300000, 0, 10.818229, -0.216667, 0, 11.148959, -0.716667, 0, 11.095345, -0.718245, 0, 11.148959, -0.716667, 0, 11.148959, -0.716667, 0, 11.095345, -0.718245, 0, 11.095345, -0.718245, 0, 11.095345, -0.718245, 0, 11.050651, -0.722982, 0, 11.095345, -0.718245, 0, 11.095345, -0.718245, 0, 11.050651, -0.722982, 0, 11.050651, -0.722982, 0, 11.050651, -0.722982, 0, 11.014876, -0.730876, 0, 11.050651, -0.722982, 0, 11.050651, -0.722982, 0, 11.014876, -0.730876, 0, 11.014876, -0.730876, 0, 11.014876, -0.730876, 0, 10.988021, -0.741927, 0, 11.014876, -0.730876, 0, 11.014876, -0.730876, 0, 10.988021, -0.741927, 0, 10.988021, -0.741927, 0, 10.988021, -0.741927, 0, 10.968425, -0.756771, 0, 10.988021, -0.741927, 0, 10.988021, -0.741927, 0, 10.968425, -0.756771, 0, 10.968425, -0.756771, 0, 10.968425, -0.756771, 0, 10.954428, -0.776042, 0, 10.968425, -0.756771, 0, 10.968425, -0.756771, 0, 10.954428, -0.776042, 0, 10.954428, -0.776042, 0, 10.954428, -0.776042, 0, 10.946029, -0.799740, 0, 10.954428, -0.776042, 0, 10.954428, -0.776042, 0, 10.946029, -0.799740, 0, 10.946029, -0.799740, 0, 10.946029, -0.799740, 0, 10.943229, -0.827865, 0, 10.946029, -0.799740, 0, 10.946029, -0.799740, 0, 10.943229, -0.827865, 0, 10.943229, -0.827865, 0, 10.943229, -0.827865, 0, 10.945329, -0.850944, 0, 10.943229, -0.827865, 0, 10.943229, -0.827865, 0, 10.945329, -0.850944, 0, 10.945329, -0.850944, 0, 10.945329, -0.850944, 0, 10.951628, -0.871484, 0, 10.945329, -0.850944, 0, 10.945329, -0.850944, 0, 10.951628, -0.871484, 0, 10.951628, -0.871484, 0, 10.951628, -0.871484, 0, 10.962126, -0.889486, 0, 10.951628, -0.871484, 0, 10.951628, -0.871484, 0, 10.962126, -0.889486, 0, 10.962126, -0.889486, 0, 10.962126, -0.889486, 0, 10.976823, -0.904948, 0, 10.962126, -0.889486, 0, 10.962126, -0.889486, 0, 10.976823, -0.904948, 0, 10.976823, -0.904948, 0, 10.976823, -0.904948, 0, 10.995133, -0.917367, 0, 10.976823, -0.904948, 0, 10.976823, -0.904948, 0, 10.995133, -0.917367, 0, 10.995133, -0.917367, 0, 10.995133, -0.917367, 0, 11.016471, -0.926237, 0, 10.995133, -0.917367, 0, 10.995133, -0.917367, 0, 11.016471, -0.926237, 0, 11.016471, -0.926237, 0, 11.016471, -0.926237, 0, 11.040836, -0.931559, 0, 11.016471, -0.926237, 0, 11.016471, -0.926237, 0, 11.040836, -0.931559, 0, 11.040836, -0.931559, 0, 11.040836, -0.931559, 0, 11.068229, -0.933333, 0, 11.040836, -0.931559, 0, 11.040836, -0.931559, 0, 11.068229, -0.933333, 0, 11.068229, -0.933333, 0, 11.068229, -0.933333, 0, 11.106104, -0.929989, 0, 11.068229, -0.933333, 0, 11.068229, -0.933333, 0, 11.106104, -0.929989, 0, 11.106104, -0.929989, 0, 11.106104, -0.929989, 0, 11.140039, -0.919954, 0, 11.106104, -0.929989, 0, 11.106104, -0.929989, 0, 11.140039, -0.919954, 0, 11.140039, -0.919954, 0, 11.140039, -0.919954, 0, 11.170035, -0.903231, 0, 11.140039, -0.919954, 0, 11.140039, -0.919954, 0, 11.170035, -0.903231, 0, 11.170035, -0.903231, 0, 11.170035, -0.903231, 0, 11.196094, -0.879818, 0, 11.170035, -0.903231, 0, 11.170035, -0.903231, 0, 11.196094, -0.879818, 0, 11.196094, -0.879818, 0, 11.196094, -0.879818, 0, 11.217172, -0.850855, 0, 11.196094, -0.879818, 0, 11.196094, -0.879818, 0, 11.217172, -0.850855, 0, 11.217172, -0.850855, 0, 11.217172, -0.850855, 0, 11.232226, -0.817480, 0, 11.217172, -0.850855, 0, 11.217172, -0.850855, 0, 11.232226, -0.817480, 0, 11.232226, -0.817480, 0, 11.232226, -0.817480, 0, 11.241260, -0.779696, 0, 11.232226, -0.817480, 0, 11.232226, -0.817480, 0, 11.241260, -0.779696, 0, 11.241260, -0.779696, 0, 11.241260, -0.779696, 0, 11.244271, -0.737500, 0, 11.241260, -0.779696, 0, 11.241260, -0.779696, 0, 11.244271, -0.737500, 0, 11.244271, -0.737500, 0, 11.244271, -0.737500, 0, 11.244271, -0.716667, 0, 11.244271, -0.737500, 0, 11.244271, -0.737500, 0, 11.244271, -0.716667, 0, 11.244271, -0.716667, 0, 11.340104, -0.672656, 0, 11.340104, -1, 0, 11.340104, -0.672656, 0, 11.340104, -0.672656, 0, 11.340104, -1, 0, 11.340104, -1, 0, 11.340104, -1, 0, 11.244271, -1, 0, 11.340104, -1, 0, 11.340104, -1, 0, 11.244271, -1, 0, 11.244271, -1, 0, 11.244271, -1, 0, 11.244271, -0.916667, 0, 11.244271, -1, 0, 11.244271, -1, 0, 11.244271, -0.916667, 0, 11.244271, -0.916667, 0, 11.244271, -0.916667, 0, 11.226855, -0.940633, 0, 11.244271, -0.916667, 0, 11.244271, -0.916667, 0, 11.226855, -0.940633, 0, 11.226855, -0.940633, 0, 11.226855, -0.940633, 0, 11.207422, -0.961230, 0, 11.226855, -0.940633, 0, 11.226855, -0.940633, 0, 11.207422, -0.961230, 0, 11.207422, -0.961230, 0, 11.207422, -0.961230, 0, 11.185970, -0.978459, 0, 11.207422, -0.961230, 0, 11.207422, -0.961230, 0, 11.185970, -0.978459, 0, 11.185970, -0.978459, 0, 11.185970, -0.978459, 0, 11.162500, -0.992318, 0, 11.185970, -0.978459, 0, 11.185970, -0.978459, 0, 11.162500, -0.992318, 0, 11.162500, -0.992318, 0, 11.162500, -0.992318, 0, 11.136654, -1.002970, 0, 11.162500, -0.992318, 0, 11.162500, -0.992318, 0, 11.136654, -1.002970, 0, 11.136654, -1.002970, 0, 11.136654, -1.002970, 0, 11.108073, -1.010579, 0, 11.136654, -1.002970, 0, 11.136654, -1.002970, 0, 11.108073, -1.010579, 0, 11.108073, -1.010579, 0, 11.108073, -1.010579, 0, 11.076758, -1.015145, 0, 11.108073, -1.010579, 0, 11.108073, -1.010579, 0, 11.076758, -1.015145, 0, 11.076758, -1.015145, 0, 11.076758, -1.015145, 0, 11.042708, -1.016667, 0, 11.076758, -1.015145, 0, 11.076758, -1.015145, 0, 11.042708, -1.016667, 0, 11.042708, -1.016667, 0, 11.042708, -1.016667, 0, 11.000212, -1.013525, 0, 11.042708, -1.016667, 0, 11.042708, -1.016667, 0, 11.000212, -1.013525, 0, 11.000212, -1.013525, 0, 11.000212, -1.013525, 0, 10.962305, -1.004102, 0, 11.000212, -1.013525, 0, 11.000212, -1.013525, 0, 10.962305, -1.004102, 0, 10.962305, -1.004102, 0, 10.962305, -1.004102, 0, 10.928988, -0.988395, 0, 10.962305, -1.004102, 0, 10.962305, -1.004102, 0, 10.928988, -0.988395, 0, 10.928988, -0.988395, 0, 10.928988, -0.988395, 0, 10.900260, -0.966406, 0, 10.928988, -0.988395, 0, 10.928988, -0.988395, 0, 10.900260, -0.966406, 0, 10.900260, -0.966406, 0, 10.900260, -0.966406, 0, 10.877132, -0.939144, 0, 10.900260, -0.966406, 0, 10.900260, -0.966406, 0, 10.877132, -0.939144, 0, 10.877132, -0.939144, 0, 10.877132, -0.939144, 0, 10.860612, -0.907617, 0, 10.877132, -0.939144, 0, 10.877132, -0.939144, 0, 10.860612, -0.907617, 0, 10.860612, -0.907617, 0, 10.860612, -0.907617, 0, 10.850699, -0.871826, 0, 10.860612, -0.907617, 0, 10.860612, -0.907617, 0, 10.850699, -0.871826, 0, 10.850699, -0.871826, 0, 10.850699, -0.871826, 0, 10.847396, -0.831771, 0, 10.850699, -0.871826, 0, 10.850699, -0.871826, 0, 10.847396, -0.831771, 0, 10.847396, -0.831771, 0, 10.847396, -0.831771, 0, 10.851514, -0.785579, 0, 10.847396, -0.831771, 0, 10.847396, -0.831771, 0, 10.851514, -0.785579, 0, 10.851514, -0.785579, 0, 10.851514, -0.785579, 0, 10.863868, -0.745443, 0, 10.851514, -0.785579, 0, 10.851514, -0.785579, 0, 10.863868, -0.745443, 0, 10.863868, -0.745443, 0, 10.863868, -0.745443, 0, 10.884457, -0.711361, 0, 10.863868, -0.745443, 0, 10.863868, -0.745443, 0, 10.884457, -0.711361, 0, 10.884457, -0.711361, 0, 10.884457, -0.711361, 0, 10.913281, -0.683333, 0, 10.884457, -0.711361, 0, 10.884457, -0.711361, 0, 10.913281, -0.683333, 0, 10.913281, -0.683333, 0, 10.913281, -0.683333, 0, 10.950277, -0.661458, 0, 10.913281, -0.683333, 0, 10.913281, -0.683333, 0, 10.950277, -0.661458, 0, 10.950277, -0.661458, 0, 10.950277, -0.661458, 0, 10.995378, -0.645833, 0, 10.950277, -0.661458, 0, 10.950277, -0.661458, 0, 10.995378, -0.645833, 0, 10.995378, -0.645833, 0, 10.995378, -0.645833, 0, 11.048584, -0.636458, 0, 10.995378, -0.645833, 0, 10.995378, -0.645833, 0, 11.048584, -0.636458, 0, 11.048584, -0.636458, 0, 11.048584, -0.636458, 0, 11.109897, -0.633333, 0, 11.048584, -0.636458, 0, 11.048584, -0.636458, 0, 11.109897, -0.633333, 0, 11.109897, -0.633333, 0, 11.109897, -0.633333, 0, 11.244271, -0.633333, 0, 11.109897, -0.633333, 0, 11.109897, -0.633333, 0, 11.244271, -0.633333, 0, 11.244271, -0.633333, 0, 11.244271, -0.633333, 0, 11.244271, -0.623958, 0, 11.244271, -0.633333, 0, 11.244271, -0.633333, 0, 11.244271, -0.623958, 0, 11.244271, -0.623958, 0, 11.244271, -0.623958, 0, 11.241553, -0.592269, 0, 11.244271, -0.623958, 0, 11.244271, -0.623958, 0, 11.241553, -0.592269, 0, 11.241553, -0.592269, 0, 11.241553, -0.592269, 0, 11.233398, -0.564388, 0, 11.241553, -0.592269, 0, 11.241553, -0.592269, 0, 11.233398, -0.564388, 0, 11.233398, -0.564388, 0, 11.233398, -0.564388, 0, 11.219809, -0.540316, 0, 11.233398, -0.564388, 0, 11.233398, -0.564388, 0, 11.219809, -0.540316, 0, 11.219809, -0.540316, 0, 11.219809, -0.540316, 0, 11.200782, -0.520052, 0, 11.219809, -0.540316, 0, 11.219809, -0.540316, 0, 11.200782, -0.520052, 0, 11.200782, -0.520052, 0, 11.200782, -0.520052, 0, 11.176840, -0.503988, 0, 11.200782, -0.520052, 0, 11.200782, -0.520052, 0, 11.176840, -0.503988, 0, 11.176840, -0.503988, 0, 11.176840, -0.503988, 0, 11.148502, -0.492513, 0, 11.176840, -0.503988, 0, 11.176840, -0.503988, 0, 11.148502, -0.492513, 0, 11.148502, -0.492513, 0, 11.148502, -0.492513, 0, 11.115771, -0.485628, 0, 11.148502, -0.492513, 0, 11.148502, -0.492513, 0, 11.115771, -0.485628, 0, 11.115771, -0.485628, 0, 11.115771, -0.485628, 0, 11.078647, -0.483333, 0, 11.115771, -0.485628, 0, 11.115771, -0.485628, 0, 11.078647, -0.483333, 0, 11.078647, -0.483333, 0, 11.078647, -0.483333, 0, 11.053809, -0.484115, 0, 11.078647, -0.483333, 0, 11.078647, -0.483333, 0, 11.053809, -0.484115, 0, 11.053809, -0.484115, 0, 11.053809, -0.484115, 0, 11.029297, -0.486458, 0, 11.053809, -0.484115, 0, 11.053809, -0.484115, 0, 11.029297, -0.486458, 0, 11.029297, -0.486458, 0, 11.029297, -0.486458, 0, 11.005111, -0.490365, 0, 11.029297, -0.486458, 0, 11.029297, -0.486458, 0, 11.005111, -0.490365, 0, 11.005111, -0.490365, 0, 11.005111, -0.490365, 0, 10.981250, -0.495833, 0, 11.005111, -0.490365, 0, 11.005111, -0.490365, 0, 10.981250, -0.495833, 0, 10.981250, -0.495833, 0, 10.981250, -0.495833, 0, 10.957780, -0.502865, 0, 10.981250, -0.495833, 0, 10.981250, -0.495833, 0, 10.957780, -0.502865, 0, 10.957780, -0.502865, 0, 10.957780, -0.502865, 0, 10.934766, -0.511458, 0, 10.957780, -0.502865, 0, 10.957780, -0.502865, 0, 10.934766, -0.511458, 0, 10.934766, -0.511458, 0, 10.934766, -0.511458, 0, 10.912208, -0.521615, 0, 10.934766, -0.511458, 0, 10.934766, -0.511458, 0, 10.912208, -0.521615, 0, 10.912208, -0.521615, 0, 10.912208, -0.521615, 0, 10.890104, -0.533333, 0, 10.912208, -0.521615, 0, 10.912208, -0.521615, 0, 10.890104, -0.533333, 0, 10.890104, -0.533333, 0, 10.890104, -0.533333, 0, 10.890104, -0.450000, 0, 10.890104, -0.533333, 0, 10.890104, -0.533333, 0, 10.890104, -0.450000, 0, 10.890104, -0.450000, 0, 10.890104, -0.450000, 0, 10.916211, -0.438281, 0, 10.890104, -0.450000, 0, 10.890104, -0.450000, 0, 10.916211, -0.438281, 0, 10.916211, -0.438281, 0, 10.916211, -0.438281, 0, 10.941928, -0.428125, 0, 10.916211, -0.438281, 0, 10.916211, -0.438281, 0, 10.941928, -0.428125, 0, 10.941928, -0.428125, 0, 10.941928, -0.428125, 0, 10.967253, -0.419531, 0, 10.941928, -0.428125, 0, 10.941928, -0.428125, 0, 10.967253, -0.419531, 0, 10.967253, -0.419531, 0, 10.967253, -0.419531, 0, 10.992188, -0.412500, 0, 10.967253, -0.419531, 0, 10.967253, -0.419531, 0, 10.992188, -0.412500, 0, 10.992188, -0.412500, 0, 10.992188, -0.412500, 0, 11.016765, -0.407031, 0, 10.992188, -0.412500, 0, 10.992188, -0.412500, 0, 11.016765, -0.407031, 0, 11.016765, -0.407031, 0, 11.016765, -0.407031, 0, 11.041016, -0.403125, 0, 11.016765, -0.407031, 0, 11.016765, -0.407031, 0, 11.041016, -0.403125, 0, 11.041016, -0.403125, 0, 11.041016, -0.403125, 0, 11.064941, -0.400781, 0, 11.041016, -0.403125, 0, 11.041016, -0.403125, 0, 11.064941, -0.400781, 0, 11.064941, -0.400781, 0, 11.064941, -0.400781, 0, 11.088542, -0.400000, 0, 11.064941, -0.400781, 0, 11.064941, -0.400781, 0, 11.088542, -0.400000, 0, 11.088542, -0.400000, 0, 11.088542, -0.400000, 0, 11.147819, -0.404224, 0, 11.088542, -0.400000, 0, 11.088542, -0.400000, 0, 11.147819, -0.404224, 0, 11.147819, -0.404224, 0, 11.147819, -0.404224, 0, 11.199088, -0.416894, 0, 11.147819, -0.404224, 0, 11.147819, -0.404224, 0, 11.199088, -0.416894, 0, 11.199088, -0.416894, 0, 11.199088, -0.416894, 0, 11.242351, -0.438013, 0, 11.199088, -0.416894, 0, 11.199088, -0.416894, 0, 11.242351, -0.438013, 0, 11.242351, -0.438013, 0, 11.242351, -0.438013, 0, 11.277604, -0.467578, 0, 11.242351, -0.438013, 0, 11.242351, -0.438013, 0, 11.277604, -0.467578, 0, 11.277604, -0.467578, 0, 11.277604, -0.467578, 0, 11.304948, -0.505737, 0, 11.277604, -0.467578, 0, 11.277604, -0.467578, 0, 11.304948, -0.505737, 0, 11.304948, -0.505737, 0, 11.304948, -0.505737, 0, 11.324479, -0.552637, 0, 11.304948, -0.505737, 0, 11.304948, -0.505737, 0, 11.324479, -0.552637, 0, 11.324479, -0.552637, 0, 11.324479, -0.552637, 0, 11.336198, -0.608276, 0, 11.324479, -0.552637, 0, 11.324479, -0.552637, 0, 11.336198, -0.608276, 0, 11.336198, -0.608276, 0, 10.851514, -0.785579, 0, 10.847396, -0.831771, 0, 10.943229, -0.827865, 0, 10.847396, -0.831771, 0, 10.851514, -0.785579, 0, 10.943229, -0.827865, 0, 10.851514, -0.785579, 0, 10.943229, -0.827865, 0, 10.946029, -0.799740, 0, 10.943229, -0.827865, 0, 10.851514, -0.785579, 0, 10.946029, -0.799740, 0, 10.863868, -0.745443, 0, 10.851514, -0.785579, 0, 10.946029, -0.799740, 0, 10.851514, -0.785579, 0, 10.863868, -0.745443, 0, 10.946029, -0.799740, 0, 10.863868, -0.745443, 0, 10.946029, -0.799740, 0, 10.954428, -0.776042, 0, 10.946029, -0.799740, 0, 10.863868, -0.745443, 0, 10.954428, -0.776042, 0, 10.884457, -0.711361, 0, 10.863868, -0.745443, 0, 10.954428, -0.776042, 0, 10.863868, -0.745443, 0, 10.884457, -0.711361, 0, 10.954428, -0.776042, 0, 10.884457, -0.711361, 0, 10.954428, -0.776042, 0, 10.968425, -0.756771, 0, 10.954428, -0.776042, 0, 10.884457, -0.711361, 0, 10.968425, -0.756771, 0, 10.913281, -0.683333, 0, 10.884457, -0.711361, 0, 10.968425, -0.756771, 0, 10.884457, -0.711361, 0, 10.913281, -0.683333, 0, 10.968425, -0.756771, 0, 10.913281, -0.683333, 0, 10.968425, -0.756771, 0, 10.988021, -0.741927, 0, 10.968425, -0.756771, 0, 10.913281, -0.683333, 0, 10.988021, -0.741927, 0, 10.950277, -0.661458, 0, 10.913281, -0.683333, 0, 10.988021, -0.741927, 0, 10.913281, -0.683333, 0, 10.950277, -0.661458, 0, 10.988021, -0.741927, 0, 11.014876, -0.730876, 0, 10.950277, -0.661458, 0, 10.988021, -0.741927, 0, 10.950277, -0.661458, 0, 11.014876, -0.730876, 0, 10.988021, -0.741927, 0, 10.995378, -0.645833, 0, 10.950277, -0.661458, 0, 11.014876, -0.730876, 0, 10.950277, -0.661458, 0, 10.995378, -0.645833, 0, 11.014876, -0.730876, 0, 11.050651, -0.722982, 0, 10.995378, -0.645833, 0, 11.014876, -0.730876, 0, 10.995378, -0.645833, 0, 11.050651, -0.722982, 0, 11.014876, -0.730876, 0, 11.048584, -0.636458, 0, 10.995378, -0.645833, 0, 11.050651, -0.722982, 0, 10.995378, -0.645833, 0, 11.048584, -0.636458, 0, 11.050651, -0.722982, 0, 11.048584, -0.636458, 0, 11.050651, -0.722982, 0, 11.095345, -0.718245, 0, 11.050651, -0.722982, 0, 11.048584, -0.636458, 0, 11.095345, -0.718245, 0, 11.048584, -0.636458, 0, 11.095345, -0.718245, 0, 11.109897, -0.633333, 0, 11.095345, -0.718245, 0, 11.048584, -0.636458, 0, 11.109897, -0.633333, 0, 11.109897, -0.633333, 0, 11.095345, -0.718245, 0, 11.148959, -0.716667, 0, 11.095345, -0.718245, 0, 11.109897, -0.633333, 0, 11.148959, -0.716667, 0, 11.109897, -0.633333, 0, 11.148959, -0.716667, 0, 11.244271, -0.633333, 0, 11.148959, -0.716667, 0, 11.109897, -0.633333, 0, 11.244271, -0.633333, 0, 11.148959, -0.716667, 0, 11.244271, -0.716667, 0, 11.244271, -0.633333, 0, 11.244271, -0.716667, 0, 11.148959, -0.716667, 0, 11.244271, -0.633333, 0, 11.244271, -0.633333, 0, 11.244271, -0.716667, 0, 11.340104, -0.672656, 0, 11.244271, -0.716667, 0, 11.244271, -0.633333, 0, 11.340104, -0.672656, 0, 11.340104, -0.672656, 0, 11.336198, -0.608276, 0, 11.244271, -0.633333, 0, 11.336198, -0.608276, 0, 11.340104, -0.672656, 0, 11.244271, -0.633333, 0, 11.336198, -0.608276, 0, 11.244271, -0.623958, 0, 11.244271, -0.633333, 0, 11.244271, -0.623958, 0, 11.336198, -0.608276, 0, 11.244271, -0.633333, 0, 11.241553, -0.592269, 0, 11.244271, -0.623958, 0, 11.336198, -0.608276, 0, 11.244271, -0.623958, 0, 11.241553, -0.592269, 0, 11.336198, -0.608276, 0, 11.324479, -0.552637, 0, 11.241553, -0.592269, 0, 11.336198, -0.608276, 0, 11.241553, -0.592269, 0, 11.324479, -0.552637, 0, 11.336198, -0.608276, 0, 11.233398, -0.564388, 0, 11.241553, -0.592269, 0, 11.324479, -0.552637, 0, 11.241553, -0.592269, 0, 11.233398, -0.564388, 0, 11.324479, -0.552637, 0, 11.304948, -0.505737, 0, 11.233398, -0.564388, 0, 11.324479, -0.552637, 0, 11.233398, -0.564388, 0, 11.304948, -0.505737, 0, 11.324479, -0.552637, 0, 11.219809, -0.540316, 0, 11.233398, -0.564388, 0, 11.304948, -0.505737, 0, 11.233398, -0.564388, 0, 11.219809, -0.540316, 0, 11.304948, -0.505737, 0, 11.277604, -0.467578, 0, 11.219809, -0.540316, 0, 11.304948, -0.505737, 0, 11.219809, -0.540316, 0, 11.277604, -0.467578, 0, 11.304948, -0.505737, 0, 11.200782, -0.520052, 0, 11.219809, -0.540316, 0, 11.277604, -0.467578, 0, 11.219809, -0.540316, 0, 11.200782, -0.520052, 0, 11.277604, -0.467578, 0, 11.242351, -0.438013, 0, 11.200782, -0.520052, 0, 11.277604, -0.467578, 0, 11.200782, -0.520052, 0, 11.242351, -0.438013, 0, 11.277604, -0.467578, 0, 11.176840, -0.503988, 0, 11.200782, -0.520052, 0, 11.242351, -0.438013, 0, 11.200782, -0.520052, 0, 11.176840, -0.503988, 0, 11.242351, -0.438013, 0, 11.199088, -0.416894, 0, 11.176840, -0.503988, 0, 11.242351, -0.438013, 0, 11.176840, -0.503988, 0, 11.199088, -0.416894, 0, 11.242351, -0.438013, 0, 11.148502, -0.492513, 0, 11.176840, -0.503988, 0, 11.199088, -0.416894, 0, 11.176840, -0.503988, 0, 11.148502, -0.492513, 0, 11.199088, -0.416894, 0, 11.147819, -0.404224, 0, 11.148502, -0.492513, 0, 11.199088, -0.416894, 0, 11.148502, -0.492513, 0, 11.147819, -0.404224, 0, 11.199088, -0.416894, 0, 11.147819, -0.404224, 0, 11.115771, -0.485628, 0, 11.148502, -0.492513, 0, 11.115771, -0.485628, 0, 11.147819, -0.404224, 0, 11.148502, -0.492513, 0, 11.147819, -0.404224, 0, 11.088542, -0.400000, 0, 11.115771, -0.485628, 0, 11.088542, -0.400000, 0, 11.147819, -0.404224, 0, 11.115771, -0.485628, 0, 11.078647, -0.483333, 0, 11.115771, -0.485628, 0, 11.088542, -0.400000, 0, 11.115771, -0.485628, 0, 11.078647, -0.483333, 0, 11.088542, -0.400000, 0, 11.088542, -0.400000, 0, 11.064941, -0.400781, 0, 11.078647, -0.483333, 0, 11.064941, -0.400781, 0, 11.088542, -0.400000, 0, 11.078647, -0.483333, 0, 11.064941, -0.400781, 0, 11.053809, -0.484115, 0, 11.078647, -0.483333, 0, 11.053809, -0.484115, 0, 11.064941, -0.400781, 0, 11.078647, -0.483333, 0, 11.041016, -0.403125, 0, 11.053809, -0.484115, 0, 11.064941, -0.400781, 0, 11.053809, -0.484115, 0, 11.041016, -0.403125, 0, 11.064941, -0.400781, 0, 11.041016, -0.403125, 0, 11.029297, -0.486458, 0, 11.053809, -0.484115, 0, 11.029297, -0.486458, 0, 11.041016, -0.403125, 0, 11.053809, -0.484115, 0, 11.016765, -0.407031, 0, 11.029297, -0.486458, 0, 11.041016, -0.403125, 0, 11.029297, -0.486458, 0, 11.016765, -0.407031, 0, 11.041016, -0.403125, 0, 11.016765, -0.407031, 0, 10.992188, -0.412500, 0, 11.029297, -0.486458, 0, 10.992188, -0.412500, 0, 11.016765, -0.407031, 0, 11.029297, -0.486458, 0, 11.029297, -0.486458, 0, 10.992188, -0.412500, 0, 11.005111, -0.490365, 0, 10.992188, -0.412500, 0, 11.029297, -0.486458, 0, 11.005111, -0.490365, 0, 10.992188, -0.412500, 0, 10.967253, -0.419531, 0, 11.005111, -0.490365, 0, 10.967253, -0.419531, 0, 10.992188, -0.412500, 0, 11.005111, -0.490365, 0, 10.967253, -0.419531, 0, 10.981250, -0.495833, 0, 11.005111, -0.490365, 0, 10.981250, -0.495833, 0, 10.967253, -0.419531, 0, 11.005111, -0.490365, 0, 10.967253, -0.419531, 0, 10.941928, -0.428125, 0, 10.981250, -0.495833, 0, 10.941928, -0.428125, 0, 10.967253, -0.419531, 0, 10.981250, -0.495833, 0, 10.941928, -0.428125, 0, 10.957780, -0.502865, 0, 10.981250, -0.495833, 0, 10.957780, -0.502865, 0, 10.941928, -0.428125, 0, 10.981250, -0.495833, 0, 10.941928, -0.428125, 0, 10.916211, -0.438281, 0, 10.957780, -0.502865, 0, 10.916211, -0.438281, 0, 10.941928, -0.428125, 0, 10.957780, -0.502865, 0, 10.916211, -0.438281, 0, 10.934766, -0.511458, 0, 10.957780, -0.502865, 0, 10.934766, -0.511458, 0, 10.916211, -0.438281, 0, 10.957780, -0.502865, 0, 10.916211, -0.438281, 0, 10.890104, -0.450000, 0, 10.934766, -0.511458, 0, 10.890104, -0.450000, 0, 10.916211, -0.438281, 0, 10.934766, -0.511458, 0, 10.890104, -0.450000, 0, 10.912208, -0.521615, 0, 10.934766, -0.511458, 0, 10.912208, -0.521615, 0, 10.890104, -0.450000, 0, 10.934766, -0.511458, 0, 10.890104, -0.450000, 0, 10.890104, -0.533333, 0, 10.912208, -0.521615, 0, 10.890104, -0.533333, 0, 10.890104, -0.450000, 0, 10.912208, -0.521615, 0, 11.340104, -0.672656, 0, 11.244271, -0.716667, 0, 11.244271, -0.737500, 0, 11.244271, -0.716667, 0, 11.340104, -0.672656, 0, 11.244271, -0.737500, 0, 11.340104, -0.672656, 0, 11.244271, -0.737500, 0, 11.241260, -0.779696, 0, 11.244271, -0.737500, 0, 11.340104, -0.672656, 0, 11.241260, -0.779696, 0, 11.340104, -1, 0, 11.340104, -0.672656, 0, 11.241260, -0.779696, 0, 11.340104, -0.672656, 0, 11.340104, -1, 0, 11.241260, -0.779696, 0, 11.340104, -1, 0, 11.241260, -0.779696, 0, 11.232226, -0.817480, 0, 11.241260, -0.779696, 0, 11.340104, -1, 0, 11.232226, -0.817480, 0, 11.244271, -0.916667, 0, 11.340104, -1, 0, 11.232226, -0.817480, 0, 11.340104, -1, 0, 11.244271, -0.916667, 0, 11.232226, -0.817480, 0, 11.244271, -0.916667, 0, 11.244271, -1, 0, 11.340104, -1, 0, 11.244271, -1, 0, 11.244271, -0.916667, 0, 11.340104, -1, 0, 11.232226, -0.817480, 0, 11.217172, -0.850855, 0, 11.244271, -0.916667, 0, 11.217172, -0.850855, 0, 11.232226, -0.817480, 0, 11.244271, -0.916667, 0, 11.217172, -0.850855, 0, 11.196094, -0.879818, 0, 11.244271, -0.916667, 0, 11.196094, -0.879818, 0, 11.217172, -0.850855, 0, 11.244271, -0.916667, 0, 11.196094, -0.879818, 0, 11.226855, -0.940633, 0, 11.244271, -0.916667, 0, 11.226855, -0.940633, 0, 11.196094, -0.879818, 0, 11.244271, -0.916667, 0, 11.196094, -0.879818, 0, 11.170035, -0.903231, 0, 11.226855, -0.940633, 0, 11.170035, -0.903231, 0, 11.196094, -0.879818, 0, 11.226855, -0.940633, 0, 11.170035, -0.903231, 0, 11.207422, -0.961230, 0, 11.226855, -0.940633, 0, 11.207422, -0.961230, 0, 11.170035, -0.903231, 0, 11.226855, -0.940633, 0, 11.207422, -0.961230, 0, 11.170035, -0.903231, 0, 11.185970, -0.978459, 0, 11.170035, -0.903231, 0, 11.207422, -0.961230, 0, 11.185970, -0.978459, 0, 11.170035, -0.903231, 0, 11.140039, -0.919954, 0, 11.185970, -0.978459, 0, 11.140039, -0.919954, 0, 11.170035, -0.903231, 0, 11.185970, -0.978459, 0, 11.140039, -0.919954, 0, 11.162500, -0.992318, 0, 11.185970, -0.978459, 0, 11.162500, -0.992318, 0, 11.140039, -0.919954, 0, 11.185970, -0.978459, 0, 11.162500, -0.992318, 0, 11.140039, -0.919954, 0, 11.136654, -1.002970, 0, 11.140039, -0.919954, 0, 11.162500, -0.992318, 0, 11.136654, -1.002970, 0, 11.140039, -0.919954, 0, 11.106104, -0.929989, 0, 11.136654, -1.002970, 0, 11.106104, -0.929989, 0, 11.140039, -0.919954, 0, 11.136654, -1.002970, 0, 11.106104, -0.929989, 0, 11.108073, -1.010579, 0, 11.136654, -1.002970, 0, 11.108073, -1.010579, 0, 11.106104, -0.929989, 0, 11.136654, -1.002970, 0, 11.106104, -0.929989, 0, 11.068229, -0.933333, 0, 11.108073, -1.010579, 0, 11.068229, -0.933333, 0, 11.106104, -0.929989, 0, 11.108073, -1.010579, 0, 11.068229, -0.933333, 0, 11.076758, -1.015145, 0, 11.108073, -1.010579, 0, 11.076758, -1.015145, 0, 11.068229, -0.933333, 0, 11.108073, -1.010579, 0, 11.068229, -0.933333, 0, 11.042708, -1.016667, 0, 11.076758, -1.015145, 0, 11.042708, -1.016667, 0, 11.068229, -0.933333, 0, 11.076758, -1.015145, 0, 11.040836, -0.931559, 0, 11.042708, -1.016667, 0, 11.068229, -0.933333, 0, 11.042708, -1.016667, 0, 11.040836, -0.931559, 0, 11.068229, -0.933333, 0, 11.040836, -0.931559, 0, 11.000212, -1.013525, 0, 11.042708, -1.016667, 0, 11.000212, -1.013525, 0, 11.040836, -0.931559, 0, 11.042708, -1.016667, 0, 11.016471, -0.926237, 0, 11.000212, -1.013525, 0, 11.040836, -0.931559, 0, 11.000212, -1.013525, 0, 11.016471, -0.926237, 0, 11.040836, -0.931559, 0, 10.962305, -1.004102, 0, 11.000212, -1.013525, 0, 11.016471, -0.926237, 0, 11.000212, -1.013525, 0, 10.962305, -1.004102, 0, 11.016471, -0.926237, 0, 10.995133, -0.917367, 0, 10.962305, -1.004102, 0, 11.016471, -0.926237, 0, 10.962305, -1.004102, 0, 10.995133, -0.917367, 0, 11.016471, -0.926237, 0, 10.928988, -0.988395, 0, 10.962305, -1.004102, 0, 10.995133, -0.917367, 0, 10.962305, -1.004102, 0, 10.928988, -0.988395, 0, 10.995133, -0.917367, 0, 10.995133, -0.917367, 0, 10.976823, -0.904948, 0, 10.928988, -0.988395, 0, 10.976823, -0.904948, 0, 10.995133, -0.917367, 0, 10.928988, -0.988395, 0, 10.976823, -0.904948, 0, 10.900260, -0.966406, 0, 10.928988, -0.988395, 0, 10.900260, -0.966406, 0, 10.976823, -0.904948, 0, 10.928988, -0.988395, 0, 10.976823, -0.904948, 0, 10.962126, -0.889486, 0, 10.900260, -0.966406, 0, 10.962126, -0.889486, 0, 10.976823, -0.904948, 0, 10.900260, -0.966406, 0, 10.962126, -0.889486, 0, 10.877132, -0.939144, 0, 10.900260, -0.966406, 0, 10.877132, -0.939144, 0, 10.962126, -0.889486, 0, 10.900260, -0.966406, 0, 10.962126, -0.889486, 0, 10.951628, -0.871484, 0, 10.877132, -0.939144, 0, 10.951628, -0.871484, 0, 10.962126, -0.889486, 0, 10.877132, -0.939144, 0, 10.951628, -0.871484, 0, 10.860612, -0.907617, 0, 10.877132, -0.939144, 0, 10.860612, -0.907617, 0, 10.951628, -0.871484, 0, 10.877132, -0.939144, 0, 10.850699, -0.871826, 0, 10.860612, -0.907617, 0, 10.951628, -0.871484, 0, 10.860612, -0.907617, 0, 10.850699, -0.871826, 0, 10.951628, -0.871484, 0, 10.945329, -0.850944, 0, 10.850699, -0.871826, 0, 10.951628, -0.871484, 0, 10.850699, -0.871826, 0, 10.945329, -0.850944, 0, 10.951628, -0.871484, 0, 10.847396, -0.831771, 0, 10.850699, -0.871826, 0, 10.945329, -0.850944, 0, 10.850699, -0.871826, 0, 10.847396, -0.831771, 0, 10.945329, -0.850944, 0, 10.943229, -0.827865, 0, 10.847396, -0.831771, 0, 10.945329, -0.850944, 0, 10.847396, -0.831771, 0, 10.943229, -0.827865, 0, 10.945329, -0.850944, 0, 11.523959, -0.774219, 0, 11.523959, -0.416667, 0, 11.523959, -0.774219, 0, 11.523959, -0.774219, 0, 11.523959, -0.416667, 0, 11.523959, -0.416667, 0, 11.523959, -0.416667, 0, 11.619792, -0.416667, 0, 11.523959, -0.416667, 0, 11.523959, -0.416667, 0, 11.619792, -0.416667, 0, 11.619792, -0.416667, 0, 11.619792, -0.416667, 0, 11.619792, -0.770573, 0, 11.619792, -0.416667, 0, 11.619792, -0.416667, 0, 11.619792, -0.770573, 0, 11.619792, -0.770573, 0, 11.619792, -0.770573, 0, 11.621810, -0.808773, 0, 11.619792, -0.770573, 0, 11.619792, -0.770573, 0, 11.621810, -0.808773, 0, 11.621810, -0.808773, 0, 11.621810, -0.808773, 0, 11.627865, -0.841862, 0, 11.621810, -0.808773, 0, 11.621810, -0.808773, 0, 11.627865, -0.841862, 0, 11.627865, -0.841862, 0, 11.627865, -0.841862, 0, 11.637956, -0.869841, 0, 11.627865, -0.841862, 0, 11.627865, -0.841862, 0, 11.637956, -0.869841, 0, 11.637956, -0.869841, 0, 11.637956, -0.869841, 0, 11.652083, -0.892708, 0, 11.637956, -0.869841, 0, 11.637956, -0.869841, 0, 11.652083, -0.892708, 0, 11.652083, -0.892708, 0, 11.652083, -0.892708, 0, 11.670248, -0.910482, 0, 11.652083, -0.892708, 0, 11.652083, -0.892708, 0, 11.670248, -0.910482, 0, 11.670248, -0.910482, 0, 11.670248, -0.910482, 0, 11.692448, -0.923177, 0, 11.670248, -0.910482, 0, 11.670248, -0.910482, 0, 11.692448, -0.923177, 0, 11.692448, -0.923177, 0, 11.692448, -0.923177, 0, 11.718685, -0.930794, 0, 11.692448, -0.923177, 0, 11.692448, -0.923177, 0, 11.718685, -0.930794, 0, 11.718685, -0.930794, 0, 11.718685, -0.930794, 0, 11.748959, -0.933333, 0, 11.718685, -0.930794, 0, 11.718685, -0.930794, 0, 11.748959, -0.933333, 0, 11.748959, -0.933333, 0, 11.748959, -0.933333, 0, 11.785727, -0.930282, 0, 11.748959, -0.933333, 0, 11.748959, -0.933333, 0, 11.785727, -0.930282, 0, 11.785727, -0.930282, 0, 11.785727, -0.930282, 0, 11.818424, -0.921126, 0, 11.785727, -0.930282, 0, 11.785727, -0.930282, 0, 11.818424, -0.921126, 0, 11.818424, -0.921126, 0, 11.818424, -0.921126, 0, 11.847054, -0.905868, 0, 11.818424, -0.921126, 0, 11.818424, -0.921126, 0, 11.847054, -0.905868, 0, 11.847054, -0.905868, 0, 11.847054, -0.905868, 0, 11.871615, -0.884505, 0, 11.847054, -0.905868, 0, 11.847054, -0.905868, 0, 11.871615, -0.884505, 0, 11.871615, -0.884505, 0, 11.871615, -0.884505, 0, 11.891325, -0.857886, 0, 11.871615, -0.884505, 0, 11.871615, -0.884505, 0, 11.891325, -0.857886, 0, 11.891325, -0.857886, 0, 11.891325, -0.857886, 0, 11.905404, -0.826855, 0, 11.891325, -0.857886, 0, 11.891325, -0.857886, 0, 11.905404, -0.826855, 0, 11.905404, -0.826855, 0, 11.905404, -0.826855, 0, 11.913852, -0.791414, 0, 11.905404, -0.826855, 0, 11.905404, -0.826855, 0, 11.913852, -0.791414, 0, 11.913852, -0.791414, 0, 11.913852, -0.791414, 0, 11.916667, -0.751562, 0, 11.913852, -0.791414, 0, 11.913852, -0.791414, 0, 11.916667, -0.751562, 0, 11.916667, -0.751562, 0, 11.916667, -0.751562, 0, 11.916667, -0.416667, 0, 11.916667, -0.751562, 0, 11.916667, -0.751562, 0, 11.916667, -0.416667, 0, 11.916667, -0.416667, 0, 11.916667, -0.416667, 0, 12.012500, -0.416667, 0, 11.916667, -0.416667, 0, 11.916667, -0.416667, 0, 12.012500, -0.416667, 0, 12.012500, -0.416667, 0, 12.012500, -0.416667, 0, 12.012500, -1, 0, 12.012500, -0.416667, 0, 12.012500, -0.416667, 0, 12.012500, -1, 0, 12.012500, -1, 0, 12.012500, -1, 0, 11.916667, -1, 0, 12.012500, -1, 0, 12.012500, -1, 0, 11.916667, -1, 0, 11.916667, -1, 0, 11.916667, -1, 0, 11.916667, -0.916667, 0, 11.916667, -1, 0, 11.916667, -1, 0, 11.916667, -0.916667, 0, 11.916667, -0.916667, 0, 11.916667, -0.916667, 0, 11.898520, -0.940422, 0, 11.916667, -0.916667, 0, 11.916667, -0.916667, 0, 11.898520, -0.940422, 0, 11.898520, -0.940422, 0, 11.898520, -0.940422, 0, 11.878971, -0.960905, 0, 11.898520, -0.940422, 0, 11.898520, -0.940422, 0, 11.878971, -0.960905, 0, 11.878971, -0.960905, 0, 11.878971, -0.960905, 0, 11.858024, -0.978117, 0, 11.878971, -0.960905, 0, 11.878971, -0.960905, 0, 11.858024, -0.978117, 0, 11.858024, -0.978117, 0, 11.858024, -0.978117, 0, 11.835677, -0.992057, 0, 11.858024, -0.978117, 0, 11.858024, -0.978117, 0, 11.835677, -0.992057, 0, 11.835677, -0.992057, 0, 11.835677, -0.992057, 0, 11.811703, -1.002824, 0, 11.835677, -0.992057, 0, 11.835677, -0.992057, 0, 11.811703, -1.002824, 0, 11.811703, -1.002824, 0, 11.811703, -1.002824, 0, 11.785872, -1.010514, 0, 11.811703, -1.002824, 0, 11.811703, -1.002824, 0, 11.785872, -1.010514, 0, 11.785872, -1.010514, 0, 11.785872, -1.010514, 0, 11.758187, -1.015129, 0, 11.785872, -1.010514, 0, 11.785872, -1.010514, 0, 11.758187, -1.015129, 0, 11.758187, -1.015129, 0, 11.758187, -1.015129, 0, 11.728646, -1.016667, 0, 11.758187, -1.015129, 0, 11.758187, -1.015129, 0, 11.728646, -1.016667, 0, 11.728646, -1.016667, 0, 11.728646, -1.016667, 0, 11.681413, -1.012809, 0, 11.728646, -1.016667, 0, 11.728646, -1.016667, 0, 11.681413, -1.012809, 0, 11.681413, -1.012809, 0, 11.681413, -1.012809, 0, 11.640235, -1.001237, 0, 11.681413, -1.012809, 0, 11.681413, -1.012809, 0, 11.640235, -1.001237, 0, 11.640235, -1.001237, 0, 11.640235, -1.001237, 0, 11.605111, -0.981950, 0, 11.640235, -1.001237, 0, 11.640235, -1.001237, 0, 11.605111, -0.981950, 0, 11.605111, -0.981950, 0, 11.605111, -0.981950, 0, 11.576041, -0.954948, 0, 11.605111, -0.981950, 0, 11.605111, -0.981950, 0, 11.576041, -0.954948, 0, 11.576041, -0.954948, 0, 11.576041, -0.954948, 0, 11.553256, -0.920508, 0, 11.576041, -0.954948, 0, 11.576041, -0.954948, 0, 11.553256, -0.920508, 0, 11.553256, -0.920508, 0, 11.553256, -0.920508, 0, 11.536979, -0.878906, 0, 11.553256, -0.920508, 0, 11.553256, -0.920508, 0, 11.536979, -0.878906, 0, 11.536979, -0.878906, 0, 11.536979, -0.878906, 0, 11.527213, -0.830143, 0, 11.536979, -0.878906, 0, 11.536979, -0.878906, 0, 11.527213, -0.830143, 0, 11.527213, -0.830143, 0, 11.523959, -0.416667, 0, 11.619792, -0.770573, 0, 11.619792, -0.416667, 0, 11.619792, -0.770573, 0, 11.523959, -0.416667, 0, 11.619792, -0.416667, 0, 11.523959, -0.416667, 0, 11.523959, -0.774219, 0, 11.619792, -0.770573, 0, 11.523959, -0.774219, 0, 11.523959, -0.416667, 0, 11.619792, -0.770573, 0, 11.619792, -0.770573, 0, 11.523959, -0.774219, 0, 11.621810, -0.808773, 0, 11.523959, -0.774219, 0, 11.619792, -0.770573, 0, 11.621810, -0.808773, 0, 11.523959, -0.774219, 0, 11.527213, -0.830143, 0, 11.621810, -0.808773, 0, 11.527213, -0.830143, 0, 11.523959, -0.774219, 0, 11.621810, -0.808773, 0, 11.621810, -0.808773, 0, 11.527213, -0.830143, 0, 11.627865, -0.841862, 0, 11.527213, -0.830143, 0, 11.621810, -0.808773, 0, 11.627865, -0.841862, 0, 11.527213, -0.830143, 0, 11.536979, -0.878906, 0, 11.627865, -0.841862, 0, 11.536979, -0.878906, 0, 11.527213, -0.830143, 0, 11.627865, -0.841862, 0, 11.627865, -0.841862, 0, 11.536979, -0.878906, 0, 11.637956, -0.869841, 0, 11.536979, -0.878906, 0, 11.627865, -0.841862, 0, 11.637956, -0.869841, 0, 11.536979, -0.878906, 0, 11.553256, -0.920508, 0, 11.637956, -0.869841, 0, 11.553256, -0.920508, 0, 11.536979, -0.878906, 0, 11.637956, -0.869841, 0, 11.637956, -0.869841, 0, 11.553256, -0.920508, 0, 11.652083, -0.892708, 0, 11.553256, -0.920508, 0, 11.637956, -0.869841, 0, 11.652083, -0.892708, 0, 11.553256, -0.920508, 0, 11.576041, -0.954948, 0, 11.652083, -0.892708, 0, 11.576041, -0.954948, 0, 11.553256, -0.920508, 0, 11.652083, -0.892708, 0, 11.652083, -0.892708, 0, 11.576041, -0.954948, 0, 11.605111, -0.981950, 0, 11.576041, -0.954948, 0, 11.652083, -0.892708, 0, 11.605111, -0.981950, 0, 11.670248, -0.910482, 0, 11.652083, -0.892708, 0, 11.605111, -0.981950, 0, 11.652083, -0.892708, 0, 11.670248, -0.910482, 0, 11.605111, -0.981950, 0, 11.670248, -0.910482, 0, 11.605111, -0.981950, 0, 11.640235, -1.001237, 0, 11.605111, -0.981950, 0, 11.670248, -0.910482, 0, 11.640235, -1.001237, 0, 11.692448, -0.923177, 0, 11.670248, -0.910482, 0, 11.640235, -1.001237, 0, 11.670248, -0.910482, 0, 11.692448, -0.923177, 0, 11.640235, -1.001237, 0, 11.640235, -1.001237, 0, 11.681413, -1.012809, 0, 11.692448, -0.923177, 0, 11.681413, -1.012809, 0, 11.640235, -1.001237, 0, 11.692448, -0.923177, 0, 11.692448, -0.923177, 0, 11.681413, -1.012809, 0, 11.718685, -0.930794, 0, 11.681413, -1.012809, 0, 11.692448, -0.923177, 0, 11.718685, -0.930794, 0, 11.718685, -0.930794, 0, 11.681413, -1.012809, 0, 11.728646, -1.016667, 0, 11.681413, -1.012809, 0, 11.718685, -0.930794, 0, 11.728646, -1.016667, 0, 11.718685, -0.930794, 0, 11.728646, -1.016667, 0, 11.748959, -0.933333, 0, 11.728646, -1.016667, 0, 11.718685, -0.930794, 0, 11.748959, -0.933333, 0, 11.748959, -0.933333, 0, 11.728646, -1.016667, 0, 11.758187, -1.015129, 0, 11.728646, -1.016667, 0, 11.748959, -0.933333, 0, 11.758187, -1.015129, 0, 11.748959, -0.933333, 0, 11.758187, -1.015129, 0, 11.785872, -1.010514, 0, 11.758187, -1.015129, 0, 11.748959, -0.933333, 0, 11.785872, -1.010514, 0, 11.785727, -0.930282, 0, 11.748959, -0.933333, 0, 11.785872, -1.010514, 0, 11.748959, -0.933333, 0, 11.785727, -0.930282, 0, 11.785872, -1.010514, 0, 11.785727, -0.930282, 0, 11.785872, -1.010514, 0, 11.811703, -1.002824, 0, 11.785872, -1.010514, 0, 11.785727, -0.930282, 0, 11.811703, -1.002824, 0, 11.785727, -0.930282, 0, 11.811703, -1.002824, 0, 11.835677, -0.992057, 0, 11.811703, -1.002824, 0, 11.785727, -0.930282, 0, 11.835677, -0.992057, 0, 11.818424, -0.921126, 0, 11.785727, -0.930282, 0, 11.835677, -0.992057, 0, 11.785727, -0.930282, 0, 11.818424, -0.921126, 0, 11.835677, -0.992057, 0, 11.818424, -0.921126, 0, 11.835677, -0.992057, 0, 11.858024, -0.978117, 0, 11.835677, -0.992057, 0, 11.818424, -0.921126, 0, 11.858024, -0.978117, 0, 11.818424, -0.921126, 0, 11.858024, -0.978117, 0, 11.878971, -0.960905, 0, 11.858024, -0.978117, 0, 11.818424, -0.921126, 0, 11.878971, -0.960905, 0, 11.847054, -0.905868, 0, 11.818424, -0.921126, 0, 11.878971, -0.960905, 0, 11.818424, -0.921126, 0, 11.847054, -0.905868, 0, 11.878971, -0.960905, 0, 11.847054, -0.905868, 0, 11.878971, -0.960905, 0, 11.898520, -0.940422, 0, 11.878971, -0.960905, 0, 11.847054, -0.905868, 0, 11.898520, -0.940422, 0, 11.871615, -0.884505, 0, 11.847054, -0.905868, 0, 11.898520, -0.940422, 0, 11.847054, -0.905868, 0, 11.871615, -0.884505, 0, 11.898520, -0.940422, 0, 11.871615, -0.884505, 0, 11.898520, -0.940422, 0, 11.916667, -0.916667, 0, 11.898520, -0.940422, 0, 11.871615, -0.884505, 0, 11.916667, -0.916667, 0, 11.891325, -0.857886, 0, 11.871615, -0.884505, 0, 11.916667, -0.916667, 0, 11.871615, -0.884505, 0, 11.891325, -0.857886, 0, 11.916667, -0.916667, 0, 11.905404, -0.826855, 0, 11.891325, -0.857886, 0, 11.916667, -0.916667, 0, 11.891325, -0.857886, 0, 11.905404, -0.826855, 0, 11.916667, -0.916667, 0, 11.916667, -0.916667, 0, 12.012500, -1, 0, 11.905404, -0.826855, 0, 12.012500, -1, 0, 11.916667, -0.916667, 0, 11.905404, -0.826855, 0, 11.916667, -0.916667, 0, 11.916667, -1, 0, 12.012500, -1, 0, 11.916667, -1, 0, 11.916667, -0.916667, 0, 12.012500, -1, 0, 12.012500, -1, 0, 11.913852, -0.791414, 0, 11.905404, -0.826855, 0, 11.913852, -0.791414, 0, 12.012500, -1, 0, 11.905404, -0.826855, 0, 11.916667, -0.751562, 0, 11.913852, -0.791414, 0, 12.012500, -1, 0, 11.913852, -0.791414, 0, 11.916667, -0.751562, 0, 12.012500, -1, 0, 12.012500, -0.416667, 0, 11.916667, -0.751562, 0, 12.012500, -1, 0, 11.916667, -0.751562, 0, 12.012500, -0.416667, 0, 12.012500, -1, 0, 12.012500, -0.416667, 0, 11.916667, -0.416667, 0, 11.916667, -0.751562, 0, 11.916667, -0.416667, 0, 12.012500, -0.416667, 0, 11.916667, -0.751562, 0, 12.217188, -0.183333, 0, 12.313022, -0.183333, 0, 12.217188, -0.183333, 0, 12.217188, -0.183333, 0, 12.313022, -0.183333, 0, 12.313022, -0.183333, 0, 12.313022, -0.183333, 0, 12.313022, -1, 0, 12.313022, -0.183333, 0, 12.313022, -0.183333, 0, 12.313022, -1, 0, 12.313022, -1, 0, 12.313022, -1, 0, 12.217188, -1, 0, 12.313022, -1, 0, 12.313022, -1, 0, 12.217188, -1, 0, 12.217188, -1, 0, 12.217188, -0.183333, 0, 12.313022, -1, 0, 12.313022, -0.183333, 0, 12.313022, -1, 0, 12.217188, -0.183333, 0, 12.313022, -0.183333, 0, 12.217188, -0.183333, 0, 12.217188, -1, 0, 12.313022, -1, 0, 12.217188, -1, 0, 12.217188, -0.183333, 0, 12.313022, -1, 0, 12.611979, -0.250000, 0, 12.611979, -0.416667, 0, 12.611979, -0.250000, 0, 12.611979, -0.250000, 0, 12.611979, -0.416667, 0, 12.611979, -0.416667, 0, 12.611979, -0.416667, 0, 12.809375, -0.416667, 0, 12.611979, -0.416667, 0, 12.611979, -0.416667, 0, 12.809375, -0.416667, 0, 12.809375, -0.416667, 0, 12.809375, -0.416667, 0, 12.809375, -0.500000, 0, 12.809375, -0.416667, 0, 12.809375, -0.416667, 0, 12.809375, -0.500000, 0, 12.809375, -0.500000, 0, 12.809375, -0.500000, 0, 12.611979, -0.500000, 0, 12.809375, -0.500000, 0, 12.809375, -0.500000, 0, 12.611979, -0.500000, 0, 12.611979, -0.500000, 0, 12.611979, -0.500000, 0, 12.611979, -0.807813, 0, 12.611979, -0.500000, 0, 12.611979, -0.500000, 0, 12.611979, -0.807813, 0, 12.611979, -0.807813, 0, 12.611979, -0.807813, 0, 12.613200, -0.839355, 0, 12.611979, -0.807813, 0, 12.611979, -0.807813, 0, 12.613200, -0.839355, 0, 12.613200, -0.839355, 0, 12.613200, -0.839355, 0, 12.616862, -0.864714, 0, 12.613200, -0.839355, 0, 12.613200, -0.839355, 0, 12.616862, -0.864714, 0, 12.616862, -0.864714, 0, 12.616862, -0.864714, 0, 12.622966, -0.883887, 0, 12.616862, -0.864714, 0, 12.616862, -0.864714, 0, 12.622966, -0.883887, 0, 12.622966, -0.883887, 0, 12.622966, -0.883887, 0, 12.631511, -0.896875, 0, 12.622966, -0.883887, 0, 12.622966, -0.883887, 0, 12.631511, -0.896875, 0, 12.631511, -0.896875, 0, 12.631511, -0.896875, 0, 12.643799, -0.905534, 0, 12.631511, -0.896875, 0, 12.631511, -0.896875, 0, 12.643799, -0.905534, 0, 12.643799, -0.905534, 0, 12.643799, -0.905534, 0, 12.661133, -0.911719, 0, 12.643799, -0.905534, 0, 12.643799, -0.905534, 0, 12.661133, -0.911719, 0, 12.661133, -0.911719, 0, 12.661133, -0.911719, 0, 12.683513, -0.915430, 0, 12.661133, -0.911719, 0, 12.661133, -0.911719, 0, 12.683513, -0.915430, 0, 12.683513, -0.915430, 0, 12.683513, -0.915430, 0, 12.710938, -0.916667, 0, 12.683513, -0.915430, 0, 12.683513, -0.915430, 0, 12.710938, -0.916667, 0, 12.710938, -0.916667, 0, 12.710938, -0.916667, 0, 12.809375, -0.916667, 0, 12.710938, -0.916667, 0, 12.710938, -0.916667, 0, 12.809375, -0.916667, 0, 12.809375, -0.916667, 0, 12.809375, -0.916667, 0, 12.809375, -1, 0, 12.809375, -0.916667, 0, 12.809375, -0.916667, 0, 12.809375, -1, 0, 12.809375, -1, 0, 12.809375, -1, 0, 12.710938, -1, 0, 12.809375, -1, 0, 12.809375, -1, 0, 12.710938, -1, 0, 12.710938, -1, 0, 12.710938, -1, 0, 12.659766, -0.997412, 0, 12.710938, -1, 0, 12.710938, -1, 0, 12.659766, -0.997412, 0, 12.659766, -0.997412, 0, 12.659766, -0.997412, 0, 12.617188, -0.989648, 0, 12.659766, -0.997412, 0, 12.659766, -0.997412, 0, 12.617188, -0.989648, 0, 12.617188, -0.989648, 0, 12.617188, -0.989648, 0, 12.583203, -0.976709, 0, 12.617188, -0.989648, 0, 12.617188, -0.989648, 0, 12.583203, -0.976709, 0, 12.583203, -0.976709, 0, 12.583203, -0.976709, 0, 12.557813, -0.958594, 0, 12.583203, -0.976709, 0, 12.583203, -0.976709, 0, 12.557813, -0.958594, 0, 12.557813, -0.958594, 0, 12.557813, -0.958594, 0, 12.539355, -0.933643, 0, 12.557813, -0.958594, 0, 12.557813, -0.958594, 0, 12.539355, -0.933643, 0, 12.539355, -0.933643, 0, 12.539355, -0.933643, 0, 12.526172, -0.900195, 0, 12.539355, -0.933643, 0, 12.539355, -0.933643, 0, 12.526172, -0.900195, 0, 12.526172, -0.900195, 0, 12.526172, -0.900195, 0, 12.518262, -0.858252, 0, 12.526172, -0.900195, 0, 12.526172, -0.900195, 0, 12.518262, -0.858252, 0, 12.518262, -0.858252, 0, 12.518262, -0.858252, 0, 12.515625, -0.807813, 0, 12.518262, -0.858252, 0, 12.518262, -0.858252, 0, 12.515625, -0.807813, 0, 12.515625, -0.807813, 0, 12.515625, -0.807813, 0, 12.515625, -0.500000, 0, 12.515625, -0.807813, 0, 12.515625, -0.807813, 0, 12.515625, -0.500000, 0, 12.515625, -0.500000, 0, 12.515625, -0.500000, 0, 12.445313, -0.500000, 0, 12.515625, -0.500000, 0, 12.515625, -0.500000, 0, 12.445313, -0.500000, 0, 12.445313, -0.500000, 0, 12.445313, -0.500000, 0, 12.445313, -0.416667, 0, 12.445313, -0.500000, 0, 12.445313, -0.500000, 0, 12.445313, -0.416667, 0, 12.445313, -0.416667, 0, 12.445313, -0.416667, 0, 12.515625, -0.416667, 0, 12.445313, -0.416667, 0, 12.445313, -0.416667, 0, 12.515625, -0.416667, 0, 12.515625, -0.416667, 0, 12.515625, -0.416667, 0, 12.515625, -0.250000, 0, 12.515625, -0.416667, 0, 12.515625, -0.416667, 0, 12.515625, -0.250000, 0, 12.515625, -0.250000, 0, 12.445313, -0.416667, 0, 12.515625, -0.500000, 0, 12.515625, -0.416667, 0, 12.515625, -0.500000, 0, 12.445313, -0.416667, 0, 12.515625, -0.416667, 0, 12.445313, -0.416667, 0, 12.445313, -0.500000, 0, 12.515625, -0.500000, 0, 12.445313, -0.500000, 0, 12.445313, -0.416667, 0, 12.515625, -0.500000, 0, 12.515625, -0.416667, 0, 12.515625, -0.500000, 0, 12.611979, -0.500000, 0, 12.515625, -0.500000, 0, 12.515625, -0.416667, 0, 12.611979, -0.500000, 0, 12.515625, -0.416667, 0, 12.611979, -0.500000, 0, 12.611979, -0.416667, 0, 12.611979, -0.500000, 0, 12.515625, -0.416667, 0, 12.611979, -0.416667, 0, 12.515625, -0.250000, 0, 12.515625, -0.416667, 0, 12.611979, -0.416667, 0, 12.515625, -0.416667, 0, 12.515625, -0.250000, 0, 12.611979, -0.416667, 0, 12.515625, -0.250000, 0, 12.611979, -0.416667, 0, 12.611979, -0.250000, 0, 12.611979, -0.416667, 0, 12.515625, -0.250000, 0, 12.611979, -0.250000, 0, 12.611979, -0.416667, 0, 12.611979, -0.500000, 0, 12.809375, -0.500000, 0, 12.611979, -0.500000, 0, 12.611979, -0.416667, 0, 12.809375, -0.500000, 0, 12.611979, -0.416667, 0, 12.809375, -0.500000, 0, 12.809375, -0.416667, 0, 12.809375, -0.500000, 0, 12.611979, -0.416667, 0, 12.809375, -0.416667, 0, 12.515625, -0.500000, 0, 12.611979, -0.807813, 0, 12.611979, -0.500000, 0, 12.611979, -0.807813, 0, 12.515625, -0.500000, 0, 12.611979, -0.500000, 0, 12.515625, -0.500000, 0, 12.515625, -0.807813, 0, 12.611979, -0.807813, 0, 12.515625, -0.807813, 0, 12.515625, -0.500000, 0, 12.611979, -0.807813, 0, 12.611979, -0.807813, 0, 12.515625, -0.807813, 0, 12.613200, -0.839355, 0, 12.515625, -0.807813, 0, 12.611979, -0.807813, 0, 12.613200, -0.839355, 0, 12.515625, -0.807813, 0, 12.518262, -0.858252, 0, 12.613200, -0.839355, 0, 12.518262, -0.858252, 0, 12.515625, -0.807813, 0, 12.613200, -0.839355, 0, 12.613200, -0.839355, 0, 12.518262, -0.858252, 0, 12.616862, -0.864714, 0, 12.518262, -0.858252, 0, 12.613200, -0.839355, 0, 12.616862, -0.864714, 0, 12.518262, -0.858252, 0, 12.526172, -0.900195, 0, 12.616862, -0.864714, 0, 12.526172, -0.900195, 0, 12.518262, -0.858252, 0, 12.616862, -0.864714, 0, 12.616862, -0.864714, 0, 12.526172, -0.900195, 0, 12.622966, -0.883887, 0, 12.526172, -0.900195, 0, 12.616862, -0.864714, 0, 12.622966, -0.883887, 0, 12.622966, -0.883887, 0, 12.526172, -0.900195, 0, 12.539355, -0.933643, 0, 12.526172, -0.900195, 0, 12.622966, -0.883887, 0, 12.539355, -0.933643, 0, 12.631511, -0.896875, 0, 12.622966, -0.883887, 0, 12.539355, -0.933643, 0, 12.622966, -0.883887, 0, 12.631511, -0.896875, 0, 12.539355, -0.933643, 0, 12.539355, -0.933643, 0, 12.557813, -0.958594, 0, 12.631511, -0.896875, 0, 12.557813, -0.958594, 0, 12.539355, -0.933643, 0, 12.631511, -0.896875, 0, 12.631511, -0.896875, 0, 12.557813, -0.958594, 0, 12.583203, -0.976709, 0, 12.557813, -0.958594, 0, 12.631511, -0.896875, 0, 12.583203, -0.976709, 0, 12.643799, -0.905534, 0, 12.631511, -0.896875, 0, 12.583203, -0.976709, 0, 12.631511, -0.896875, 0, 12.643799, -0.905534, 0, 12.583203, -0.976709, 0, 12.583203, -0.976709, 0, 12.617188, -0.989648, 0, 12.643799, -0.905534, 0, 12.617188, -0.989648, 0, 12.583203, -0.976709, 0, 12.643799, -0.905534, 0, 12.643799, -0.905534, 0, 12.617188, -0.989648, 0, 12.661133, -0.911719, 0, 12.617188, -0.989648, 0, 12.643799, -0.905534, 0, 12.661133, -0.911719, 0, 12.617188, -0.989648, 0, 12.659766, -0.997412, 0, 12.661133, -0.911719, 0, 12.659766, -0.997412, 0, 12.617188, -0.989648, 0, 12.661133, -0.911719, 0, 12.661133, -0.911719, 0, 12.659766, -0.997412, 0, 12.683513, -0.915430, 0, 12.659766, -0.997412, 0, 12.661133, -0.911719, 0, 12.683513, -0.915430, 0, 12.683513, -0.915430, 0, 12.659766, -0.997412, 0, 12.710938, -1, 0, 12.659766, -0.997412, 0, 12.683513, -0.915430, 0, 12.710938, -1, 0, 12.710938, -0.916667, 0, 12.683513, -0.915430, 0, 12.710938, -1, 0, 12.683513, -0.915430, 0, 12.710938, -0.916667, 0, 12.710938, -1, 0, 12.710938, -0.916667, 0, 12.710938, -1, 0, 12.809375, -1, 0, 12.710938, -1, 0, 12.710938, -0.916667, 0, 12.809375, -1, 0, 12.710938, -0.916667, 0, 12.809375, -1, 0, 12.809375, -0.916667, 0, 12.809375, -1, 0, 12.710938, -0.916667, 0, 12.809375, -0.916667, 0, 13.267188, -0.416667, 0, 13.363021, -0.416667, 0, 13.267188, -0.416667, 0, 13.267188, -0.416667, 0, 13.363021, -0.416667, 0, 13.363021, -0.416667, 0, 13.363021, -0.416667, 0, 13.363021, -1, 0, 13.363021, -0.416667, 0, 13.363021, -0.416667, 0, 13.363021, -1, 0, 13.363021, -1, 0, 13.363021, -1, 0, 13.267188, -1, 0, 13.363021, -1, 0, 13.363021, -1, 0, 13.267188, -1, 0, 13.267188, -1, 0, 13.267188, -0.416667, 0, 13.363021, -1, 0, 13.363021, -0.416667, 0, 13.363021, -1, 0, 13.267188, -0.416667, 0, 13.363021, -0.416667, 0, 13.267188, -0.416667, 0, 13.267188, -1, 0, 13.363021, -1, 0, 13.267188, -1, 0, 13.267188, -0.416667, 0, 13.363021, -1, 0, 13.267188, -0.183333, 0, 13.363021, -0.183333, 0, 13.267188, -0.183333, 0, 13.267188, -0.183333, 0, 13.363021, -0.183333, 0, 13.363021, -0.183333, 0, 13.363021, -0.183333, 0, 13.363021, -0.300000, 0, 13.363021, -0.183333, 0, 13.363021, -0.183333, 0, 13.363021, -0.300000, 0, 13.363021, -0.300000, 0, 13.363021, -0.300000, 0, 13.267188, -0.300000, 0, 13.363021, -0.300000, 0, 13.363021, -0.300000, 0, 13.267188, -0.300000, 0, 13.267188, -0.300000, 0, 13.267188, -0.183333, 0, 13.363021, -0.300000, 0, 13.363021, -0.183333, 0, 13.363021, -0.300000, 0, 13.267188, -0.183333, 0, 13.363021, -0.183333, 0, 13.267188, -0.183333, 0, 13.267188, -0.300000, 0, 13.363021, -0.300000, 0, 13.267188, -0.300000, 0, 13.267188, -0.183333, 0, 13.363021, -0.300000, 0, 14.052083, -0.642708, 0, 14.052083, -1, 0, 14.052083, -0.642708, 0, 14.052083, -0.642708, 0, 14.052083, -1, 0, 14.052083, -1, 0, 14.052083, -1, 0, 13.956250, -1, 0, 14.052083, -1, 0, 14.052083, -1, 0, 13.956250, -1, 0, 13.956250, -1, 0, 13.956250, -1, 0, 13.956250, -0.645833, 0, 13.956250, -1, 0, 13.956250, -1, 0, 13.956250, -0.645833, 0, 13.956250, -0.645833, 0, 13.956250, -0.645833, 0, 13.954231, -0.607642, 0, 13.956250, -0.645833, 0, 13.956250, -0.645833, 0, 13.954231, -0.607642, 0, 13.954231, -0.607642, 0, 13.954231, -0.607642, 0, 13.948176, -0.574577, 0, 13.954231, -0.607642, 0, 13.954231, -0.607642, 0, 13.948176, -0.574577, 0, 13.948176, -0.574577, 0, 13.948176, -0.574577, 0, 13.938087, -0.546639, 0, 13.948176, -0.574577, 0, 13.948176, -0.574577, 0, 13.938087, -0.546639, 0, 13.938087, -0.546639, 0, 13.938087, -0.546639, 0, 13.923958, -0.523828, 0, 13.938087, -0.546639, 0, 13.938087, -0.546639, 0, 13.923958, -0.523828, 0, 13.923958, -0.523828, 0, 13.923958, -0.523828, 0, 13.905794, -0.506112, 0, 13.923958, -0.523828, 0, 13.923958, -0.523828, 0, 13.905794, -0.506112, 0, 13.905794, -0.506112, 0, 13.905794, -0.506112, 0, 13.883594, -0.493457, 0, 13.905794, -0.506112, 0, 13.905794, -0.506112, 0, 13.883594, -0.493457, 0, 13.883594, -0.493457, 0, 13.883594, -0.493457, 0, 13.857356, -0.485864, 0, 13.883594, -0.493457, 0, 13.883594, -0.493457, 0, 13.857356, -0.485864, 0, 13.857356, -0.485864, 0, 13.857356, -0.485864, 0, 13.827083, -0.483333, 0, 13.857356, -0.485864, 0, 13.857356, -0.485864, 0, 13.827083, -0.483333, 0, 13.827083, -0.483333, 0, 13.827083, -0.483333, 0, 13.790332, -0.486385, 0, 13.827083, -0.483333, 0, 13.827083, -0.483333, 0, 13.790332, -0.486385, 0, 13.790332, -0.486385, 0, 13.790332, -0.486385, 0, 13.757683, -0.495540, 0, 13.790332, -0.486385, 0, 13.790332, -0.486385, 0, 13.757683, -0.495540, 0, 13.757683, -0.495540, 0, 13.757683, -0.495540, 0, 13.729134, -0.510799, 0, 13.757683, -0.495540, 0, 13.757683, -0.495540, 0, 13.729134, -0.510799, 0, 13.729134, -0.510799, 0, 13.729134, -0.510799, 0, 13.704688, -0.532161, 0, 13.729134, -0.510799, 0, 13.729134, -0.510799, 0, 13.704688, -0.532161, 0, 13.704688, -0.532161, 0, 13.704688, -0.532161, 0, 13.685091, -0.558797, 0, 13.704688, -0.532161, 0, 13.704688, -0.532161, 0, 13.685091, -0.558797, 0, 13.685091, -0.558797, 0, 13.685091, -0.558797, 0, 13.671094, -0.589876, 0, 13.685091, -0.558797, 0, 13.685091, -0.558797, 0, 13.671094, -0.589876, 0, 13.671094, -0.589876, 0, 13.671094, -0.589876, 0, 13.662696, -0.625399, 0, 13.671094, -0.589876, 0, 13.671094, -0.589876, 0, 13.662696, -0.625399, 0, 13.662696, -0.625399, 0, 13.662696, -0.625399, 0, 13.659896, -0.665365, 0, 13.662696, -0.625399, 0, 13.662696, -0.625399, 0, 13.659896, -0.665365, 0, 13.659896, -0.665365, 0, 13.659896, -0.665365, 0, 13.659896, -1, 0, 13.659896, -0.665365, 0, 13.659896, -0.665365, 0, 13.659896, -1, 0, 13.659896, -1, 0, 13.659896, -1, 0, 13.563542, -1, 0, 13.659896, -1, 0, 13.659896, -1, 0, 13.563542, -1, 0, 13.563542, -1, 0, 13.563542, -1, 0, 13.563542, -0.416667, 0, 13.563542, -1, 0, 13.563542, -1, 0, 13.563542, -0.416667, 0, 13.563542, -0.416667, 0, 13.563542, -0.416667, 0, 13.659896, -0.416667, 0, 13.563542, -0.416667, 0, 13.563542, -0.416667, 0, 13.659896, -0.416667, 0, 13.659896, -0.416667, 0, 13.659896, -0.416667, 0, 13.659896, -0.500000, 0, 13.659896, -0.416667, 0, 13.659896, -0.416667, 0, 13.659896, -0.500000, 0, 13.659896, -0.500000, 0, 13.659896, -0.500000, 0, 13.677849, -0.476457, 0, 13.659896, -0.500000, 0, 13.659896, -0.500000, 0, 13.677849, -0.476457, 0, 13.677849, -0.476457, 0, 13.677849, -0.476457, 0, 13.697331, -0.456087, 0, 13.677849, -0.476457, 0, 13.677849, -0.476457, 0, 13.697331, -0.456087, 0, 13.697331, -0.456087, 0, 13.697331, -0.456087, 0, 13.718343, -0.438892, 0, 13.697331, -0.456087, 0, 13.697331, -0.456087, 0, 13.718343, -0.438892, 0, 13.718343, -0.438892, 0, 13.718343, -0.438892, 0, 13.740886, -0.424870, 0, 13.718343, -0.438892, 0, 13.718343, -0.438892, 0, 13.740886, -0.424870, 0, 13.740886, -0.424870, 0, 13.740886, -0.424870, 0, 13.765088, -0.413989, 0, 13.740886, -0.424870, 0, 13.740886, -0.424870, 0, 13.765088, -0.413989, 0, 13.765088, -0.413989, 0, 13.765088, -0.413989, 0, 13.791081, -0.406217, 0, 13.765088, -0.413989, 0, 13.765088, -0.413989, 0, 13.791081, -0.406217, 0, 13.791081, -0.406217, 0, 13.791081, -0.406217, 0, 13.818865, -0.401554, 0, 13.791081, -0.406217, 0, 13.791081, -0.406217, 0, 13.818865, -0.401554, 0, 13.818865, -0.401554, 0, 13.818865, -0.401554, 0, 13.848437, -0.400000, 0, 13.818865, -0.401554, 0, 13.818865, -0.401554, 0, 13.848437, -0.400000, 0, 13.848437, -0.400000, 0, 13.848437, -0.400000, 0, 13.895638, -0.403849, 0, 13.848437, -0.400000, 0, 13.848437, -0.400000, 0, 13.895638, -0.403849, 0, 13.895638, -0.403849, 0, 13.895638, -0.403849, 0, 13.936719, -0.415397, 0, 13.895638, -0.403849, 0, 13.895638, -0.403849, 0, 13.936719, -0.415397, 0, 13.936719, -0.415397, 0, 13.936719, -0.415397, 0, 13.971680, -0.434644, 0, 13.936719, -0.415397, 0, 13.936719, -0.415397, 0, 13.971680, -0.434644, 0, 13.971680, -0.434644, 0, 13.971680, -0.434644, 0, 14.000521, -0.461589, 0, 13.971680, -0.434644, 0, 13.971680, -0.434644, 0, 14.000521, -0.461589, 0, 14.000521, -0.461589, 0, 14.000521, -0.461589, 0, 14.023079, -0.496004, 0, 14.000521, -0.461589, 0, 14.000521, -0.461589, 0, 14.023079, -0.496004, 0, 14.023079, -0.496004, 0, 14.023079, -0.496004, 0, 14.039192, -0.537663, 0, 14.023079, -0.496004, 0, 14.023079, -0.496004, 0, 14.039192, -0.537663, 0, 14.039192, -0.537663, 0, 14.039192, -0.537663, 0, 14.048861, -0.586564, 0, 14.039192, -0.537663, 0, 14.039192, -0.537663, 0, 14.048861, -0.586564, 0, 14.048861, -0.586564, 0, 13.659896, -0.416667, 0, 13.563542, -0.416667, 0, 13.659896, -0.500000, 0, 13.563542, -0.416667, 0, 13.659896, -0.416667, 0, 13.659896, -0.500000, 0, 13.563542, -0.416667, 0, 13.671094, -0.589876, 0, 13.659896, -0.500000, 0, 13.671094, -0.589876, 0, 13.563542, -0.416667, 0, 13.659896, -0.500000, 0, 13.671094, -0.589876, 0, 13.563542, -0.416667, 0, 13.662696, -0.625399, 0, 13.563542, -0.416667, 0, 13.671094, -0.589876, 0, 13.662696, -0.625399, 0, 13.662696, -0.625399, 0, 13.563542, -0.416667, 0, 13.659896, -0.665365, 0, 13.563542, -0.416667, 0, 13.662696, -0.625399, 0, 13.659896, -0.665365, 0, 13.563542, -0.416667, 0, 13.563542, -1, 0, 13.659896, -0.665365, 0, 13.563542, -1, 0, 13.563542, -0.416667, 0, 13.659896, -0.665365, 0, 13.563542, -1, 0, 13.659896, -1, 0, 13.659896, -0.665365, 0, 13.659896, -1, 0, 13.563542, -1, 0, 13.659896, -0.665365, 0, 13.659896, -0.500000, 0, 13.671094, -0.589876, 0, 13.685091, -0.558797, 0, 13.671094, -0.589876, 0, 13.659896, -0.500000, 0, 13.685091, -0.558797, 0, 13.659896, -0.500000, 0, 13.685091, -0.558797, 0, 13.704688, -0.532161, 0, 13.685091, -0.558797, 0, 13.659896, -0.500000, 0, 13.704688, -0.532161, 0, 13.677849, -0.476457, 0, 13.659896, -0.500000, 0, 13.704688, -0.532161, 0, 13.659896, -0.500000, 0, 13.677849, -0.476457, 0, 13.704688, -0.532161, 0, 13.677849, -0.476457, 0, 13.704688, -0.532161, 0, 13.729134, -0.510799, 0, 13.704688, -0.532161, 0, 13.677849, -0.476457, 0, 13.729134, -0.510799, 0, 13.697331, -0.456087, 0, 13.677849, -0.476457, 0, 13.729134, -0.510799, 0, 13.677849, -0.476457, 0, 13.697331, -0.456087, 0, 13.729134, -0.510799, 0, 13.757683, -0.495540, 0, 13.697331, -0.456087, 0, 13.729134, -0.510799, 0, 13.697331, -0.456087, 0, 13.757683, -0.495540, 0, 13.729134, -0.510799, 0, 13.757683, -0.495540, 0, 13.718343, -0.438892, 0, 13.697331, -0.456087, 0, 13.718343, -0.438892, 0, 13.757683, -0.495540, 0, 13.697331, -0.456087, 0, 13.740886, -0.424870, 0, 13.718343, -0.438892, 0, 13.757683, -0.495540, 0, 13.718343, -0.438892, 0, 13.740886, -0.424870, 0, 13.757683, -0.495540, 0, 13.740886, -0.424870, 0, 13.757683, -0.495540, 0, 13.790332, -0.486385, 0, 13.757683, -0.495540, 0, 13.740886, -0.424870, 0, 13.790332, -0.486385, 0, 13.765088, -0.413989, 0, 13.740886, -0.424870, 0, 13.790332, -0.486385, 0, 13.740886, -0.424870, 0, 13.765088, -0.413989, 0, 13.790332, -0.486385, 0, 13.765088, -0.413989, 0, 13.790332, -0.486385, 0, 13.791081, -0.406217, 0, 13.790332, -0.486385, 0, 13.765088, -0.413989, 0, 13.791081, -0.406217, 0, 13.791081, -0.406217, 0, 13.790332, -0.486385, 0, 13.827083, -0.483333, 0, 13.790332, -0.486385, 0, 13.791081, -0.406217, 0, 13.827083, -0.483333, 0, 13.818865, -0.401554, 0, 13.791081, -0.406217, 0, 13.827083, -0.483333, 0, 13.791081, -0.406217, 0, 13.818865, -0.401554, 0, 13.827083, -0.483333, 0, 13.848437, -0.400000, 0, 13.818865, -0.401554, 0, 13.827083, -0.483333, 0, 13.818865, -0.401554, 0, 13.848437, -0.400000, 0, 13.827083, -0.483333, 0, 13.848437, -0.400000, 0, 13.827083, -0.483333, 0, 13.857356, -0.485864, 0, 13.827083, -0.483333, 0, 13.848437, -0.400000, 0, 13.857356, -0.485864, 0, 13.895638, -0.403849, 0, 13.848437, -0.400000, 0, 13.857356, -0.485864, 0, 13.848437, -0.400000, 0, 13.895638, -0.403849, 0, 13.857356, -0.485864, 0, 13.857356, -0.485864, 0, 13.883594, -0.493457, 0, 13.895638, -0.403849, 0, 13.883594, -0.493457, 0, 13.857356, -0.485864, 0, 13.895638, -0.403849, 0, 13.895638, -0.403849, 0, 13.883594, -0.493457, 0, 13.936719, -0.415397, 0, 13.883594, -0.493457, 0, 13.895638, -0.403849, 0, 13.936719, -0.415397, 0, 13.883594, -0.493457, 0, 13.905794, -0.506112, 0, 13.936719, -0.415397, 0, 13.905794, -0.506112, 0, 13.883594, -0.493457, 0, 13.936719, -0.415397, 0, 13.936719, -0.415397, 0, 13.905794, -0.506112, 0, 13.971680, -0.434644, 0, 13.905794, -0.506112, 0, 13.936719, -0.415397, 0, 13.971680, -0.434644, 0, 13.905794, -0.506112, 0, 13.923958, -0.523828, 0, 13.971680, -0.434644, 0, 13.923958, -0.523828, 0, 13.905794, -0.506112, 0, 13.971680, -0.434644, 0, 13.971680, -0.434644, 0, 13.923958, -0.523828, 0, 14.000521, -0.461589, 0, 13.923958, -0.523828, 0, 13.971680, -0.434644, 0, 14.000521, -0.461589, 0, 14.000521, -0.461589, 0, 13.923958, -0.523828, 0, 14.023079, -0.496004, 0, 13.923958, -0.523828, 0, 14.000521, -0.461589, 0, 14.023079, -0.496004, 0, 14.023079, -0.496004, 0, 13.923958, -0.523828, 0, 13.938087, -0.546639, 0, 13.923958, -0.523828, 0, 14.023079, -0.496004, 0, 13.938087, -0.546639, 0, 14.023079, -0.496004, 0, 13.938087, -0.546639, 0, 14.039192, -0.537663, 0, 13.938087, -0.546639, 0, 14.023079, -0.496004, 0, 14.039192, -0.537663, 0, 13.938087, -0.546639, 0, 13.948176, -0.574577, 0, 14.039192, -0.537663, 0, 13.948176, -0.574577, 0, 13.938087, -0.546639, 0, 14.039192, -0.537663, 0, 14.039192, -0.537663, 0, 13.948176, -0.574577, 0, 14.048861, -0.586564, 0, 13.948176, -0.574577, 0, 14.039192, -0.537663, 0, 14.048861, -0.586564, 0, 13.948176, -0.574577, 0, 13.954231, -0.607642, 0, 14.048861, -0.586564, 0, 13.954231, -0.607642, 0, 13.948176, -0.574577, 0, 14.048861, -0.586564, 0, 14.048861, -0.586564, 0, 13.954231, -0.607642, 0, 14.052083, -0.642708, 0, 13.954231, -0.607642, 0, 14.048861, -0.586564, 0, 14.052083, -0.642708, 0, 14.052083, -0.642708, 0, 13.954231, -0.607642, 0, 13.956250, -0.645833, 0, 13.954231, -0.607642, 0, 14.052083, -0.642708, 0, 13.956250, -0.645833, 0, 13.956250, -0.645833, 0, 14.052083, -1, 0, 14.052083, -0.642708, 0, 14.052083, -1, 0, 13.956250, -0.645833, 0, 14.052083, -0.642708, 0, 13.956250, -0.645833, 0, 13.956250, -1, 0, 14.052083, -1, 0, 13.956250, -1, 0, 13.956250, -0.645833, 0, 14.052083, -1, 0, 14.588020, -0.216667, 0, 14.744792, -0.216667, 0, 14.588020, -0.216667, 0, 14.588020, -0.216667, 0, 14.744792, -0.216667, 0, 14.744792, -0.216667, 0, 14.744792, -0.216667, 0, 14.943230, -0.750260, 0, 14.744792, -0.216667, 0, 14.744792, -0.216667, 0, 14.943230, -0.750260, 0, 14.943230, -0.750260, 0, 14.943230, -0.750260, 0, 15.142708, -0.216667, 0, 14.943230, -0.750260, 0, 14.943230, -0.750260, 0, 15.142708, -0.216667, 0, 15.142708, -0.216667, 0, 15.142708, -0.216667, 0, 15.299479, -0.216667, 0, 15.142708, -0.216667, 0, 15.142708, -0.216667, 0, 15.299479, -0.216667, 0, 15.299479, -0.216667, 0, 15.299479, -0.216667, 0, 15.299479, -1, 0, 15.299479, -0.216667, 0, 15.299479, -0.216667, 0, 15.299479, -1, 0, 15.299479, -1, 0, 15.299479, -1, 0, 15.196876, -1, 0, 15.299479, -1, 0, 15.299479, -1, 0, 15.196876, -1, 0, 15.196876, -1, 0, 15.196876, -1, 0, 15.196876, -0.312240, 0, 15.196876, -1, 0, 15.196876, -1, 0, 15.196876, -0.312240, 0, 15.196876, -0.312240, 0, 15.196876, -0.312240, 0, 14.996355, -0.850000, 0, 15.196876, -0.312240, 0, 15.196876, -0.312240, 0, 14.996355, -0.850000, 0, 14.996355, -0.850000, 0, 14.996355, -0.850000, 0, 14.890625, -0.850000, 0, 14.996355, -0.850000, 0, 14.996355, -0.850000, 0, 14.890625, -0.850000, 0, 14.890625, -0.850000, 0, 14.890625, -0.850000, 0, 14.690104, -0.312240, 0, 14.890625, -0.850000, 0, 14.890625, -0.850000, 0, 14.690104, -0.312240, 0, 14.690104, -0.312240, 0, 14.690104, -0.312240, 0, 14.690104, -1, 0, 14.690104, -0.312240, 0, 14.690104, -0.312240, 0, 14.690104, -1, 0, 14.690104, -1, 0, 14.690104, -1, 0, 14.588020, -1, 0, 14.690104, -1, 0, 14.690104, -1, 0, 14.588020, -1, 0, 14.588020, -1, 0, 14.588020, -0.216667, 0, 14.690104, -0.312240, 0, 14.744792, -0.216667, 0, 14.690104, -0.312240, 0, 14.588020, -0.216667, 0, 14.744792, -0.216667, 0, 14.588020, -0.216667, 0, 14.588020, -1, 0, 14.690104, -0.312240, 0, 14.588020, -1, 0, 14.588020, -0.216667, 0, 14.690104, -0.312240, 0, 14.690104, -1, 0, 14.690104, -0.312240, 0, 14.588020, -1, 0, 14.690104, -0.312240, 0, 14.690104, -1, 0, 14.588020, -1, 0, 14.744792, -0.216667, 0, 14.690104, -0.312240, 0, 14.943230, -0.750260, 0, 14.690104, -0.312240, 0, 14.744792, -0.216667, 0, 14.943230, -0.750260, 0, 14.690104, -0.312240, 0, 14.890625, -0.850000, 0, 14.943230, -0.750260, 0, 14.890625, -0.850000, 0, 14.690104, -0.312240, 0, 14.943230, -0.750260, 0, 14.943230, -0.750260, 0, 14.890625, -0.850000, 0, 14.996355, -0.850000, 0, 14.890625, -0.850000, 0, 14.943230, -0.750260, 0, 14.996355, -0.850000, 0, 14.943230, -0.750260, 0, 14.996355, -0.850000, 0, 15.196876, -0.312240, 0, 14.996355, -0.850000, 0, 14.943230, -0.750260, 0, 15.196876, -0.312240, 0, 15.196876, -0.312240, 0, 15.142708, -0.216667, 0, 14.943230, -0.750260, 0, 15.142708, -0.216667, 0, 15.196876, -0.312240, 0, 14.943230, -0.750260, 0, 15.142708, -0.216667, 0, 15.196876, -0.312240, 0, 15.299479, -0.216667, 0, 15.196876, -0.312240, 0, 15.142708, -0.216667, 0, 15.299479, -0.216667, 0, 15.196876, -0.312240, 0, 15.299479, -1, 0, 15.299479, -0.216667, 0, 15.299479, -1, 0, 15.196876, -0.312240, 0, 15.299479, -0.216667, 0, 15.299479, -1, 0, 15.196876, -0.312240, 0, 15.196876, -1, 0, 15.196876, -0.312240, 0, 15.299479, -1, 0, 15.196876, -1, 0, 15.970833, -0.233333, 0, 15.970833, -0.333333, 0, 15.970833, -0.233333, 0, 15.970833, -0.233333, 0, 15.970833, -0.333333, 0, 15.970833, -0.333333, 0, 15.970833, -0.333333, 0, 15.941309, -0.321509, 0, 15.970833, -0.333333, 0, 15.970833, -0.333333, 0, 15.941309, -0.321509, 0, 15.941309, -0.321509, 0, 15.941309, -0.321509, 0, 15.912630, -0.311296, 0, 15.941309, -0.321509, 0, 15.941309, -0.321509, 0, 15.912630, -0.311296, 0, 15.912630, -0.311296, 0, 15.912630, -0.311296, 0, 15.884798, -0.302694, 0, 15.912630, -0.311296, 0, 15.912630, -0.311296, 0, 15.884798, -0.302694, 0, 15.884798, -0.302694, 0, 15.884798, -0.302694, 0, 15.857813, -0.295703, 0, 15.884798, -0.302694, 0, 15.884798, -0.302694, 0, 15.857813, -0.295703, 0, 15.857813, -0.295703, 0, 15.857813, -0.295703, 0, 15.831477, -0.290291, 0, 15.857813, -0.295703, 0, 15.857813, -0.295703, 0, 15.831477, -0.290291, 0, 15.831477, -0.290291, 0, 15.831477, -0.290291, 0, 15.805598, -0.286426, 0, 15.831477, -0.290291, 0, 15.831477, -0.290291, 0, 15.805598, -0.286426, 0, 15.805598, -0.286426, 0, 15.805598, -0.286426, 0, 15.780176, -0.284106, 0, 15.805598, -0.286426, 0, 15.805598, -0.286426, 0, 15.780176, -0.284106, 0, 15.780176, -0.284106, 0, 15.780176, -0.284106, 0, 15.755208, -0.283333, 0, 15.780176, -0.284106, 0, 15.780176, -0.284106, 0, 15.755208, -0.283333, 0, 15.755208, -0.283333, 0, 15.755208, -0.283333, 0, 15.714698, -0.285392, 0, 15.755208, -0.283333, 0, 15.755208, -0.283333, 0, 15.714698, -0.285392, 0, 15.714698, -0.285392, 0, 15.714698, -0.285392, 0, 15.679102, -0.291569, 0, 15.714698, -0.285392, 0, 15.714698, -0.285392, 0, 15.679102, -0.291569, 0, 15.679102, -0.291569, 0, 15.679102, -0.291569, 0, 15.648421, -0.301864, 0, 15.679102, -0.291569, 0, 15.679102, -0.291569, 0, 15.648421, -0.301864, 0, 15.648421, -0.301864, 0, 15.648421, -0.301864, 0, 15.622657, -0.316276, 0, 15.648421, -0.301864, 0, 15.648421, -0.301864, 0, 15.622657, -0.316276, 0, 15.622657, -0.316276, 0, 15.622657, -0.316276, 0, 15.602263, -0.334481, 0, 15.622657, -0.316276, 0, 15.622657, -0.316276, 0, 15.602263, -0.334481, 0, 15.602263, -0.334481, 0, 15.602263, -0.334481, 0, 15.587695, -0.356152, 0, 15.602263, -0.334481, 0, 15.602263, -0.334481, 0, 15.587695, -0.356152, 0, 15.587695, -0.356152, 0, 15.587695, -0.356152, 0, 15.578956, -0.381291, 0, 15.587695, -0.356152, 0, 15.587695, -0.356152, 0, 15.578956, -0.381291, 0, 15.578956, -0.381291, 0, 15.578956, -0.381291, 0, 15.576042, -0.409896, 0, 15.578956, -0.381291, 0, 15.578956, -0.381291, 0, 15.576042, -0.409896, 0, 15.576042, -0.409896, 0, 15.576042, -0.409896, 0, 15.577979, -0.433740, 0, 15.576042, -0.409896, 0, 15.576042, -0.409896, 0, 15.577979, -0.433740, 0, 15.577979, -0.433740, 0, 15.577979, -0.433740, 0, 15.583789, -0.454492, 0, 15.577979, -0.433740, 0, 15.577979, -0.433740, 0, 15.583789, -0.454492, 0, 15.583789, -0.454492, 0, 15.583789, -0.454492, 0, 15.593474, -0.472152, 0, 15.583789, -0.454492, 0, 15.583789, -0.454492, 0, 15.593474, -0.472152, 0, 15.593474, -0.472152, 0, 15.593474, -0.472152, 0, 15.607032, -0.486719, 0, 15.593474, -0.472152, 0, 15.593474, -0.472152, 0, 15.607032, -0.486719, 0, 15.607032, -0.486719, 0, 15.607032, -0.486719, 0, 15.625994, -0.499105, 0, 15.607032, -0.486719, 0, 15.607032, -0.486719, 0, 15.625994, -0.499105, 0, 15.625994, -0.499105, 0, 15.625994, -0.499105, 0, 15.651888, -0.510221, 0, 15.625994, -0.499105, 0, 15.625994, -0.499105, 0, 15.651888, -0.510221, 0, 15.651888, -0.510221, 0, 15.651888, -0.510221, 0, 15.684717, -0.520068, 0, 15.651888, -0.510221, 0, 15.651888, -0.510221, 0, 15.684717, -0.520068, 0, 15.684717, -0.520068, 0, 15.684717, -0.520068, 0, 15.724480, -0.528646, 0, 15.684717, -0.520068, 0, 15.684717, -0.520068, 0, 15.724480, -0.528646, 0, 15.724480, -0.528646, 0, 15.724480, -0.528646, 0, 15.788020, -0.541927, 0, 15.724480, -0.528646, 0, 15.724480, -0.528646, 0, 15.788020, -0.541927, 0, 15.788020, -0.541927, 0, 15.788020, -0.541927, 0, 15.843018, -0.555713, 0, 15.788020, -0.541927, 0, 15.788020, -0.541927, 0, 15.843018, -0.555713, 0, 15.843018, -0.555713, 0, 15.843018, -0.555713, 0, 15.890300, -0.573893, 0, 15.843018, -0.555713, 0, 15.843018, -0.555713, 0, 15.890300, -0.573893, 0, 15.890300, -0.573893, 0, 15.890300, -0.573893, 0, 15.929866, -0.596468, 0, 15.890300, -0.573893, 0, 15.890300, -0.573893, 0, 15.929866, -0.596468, 0, 15.929866, -0.596468, 0, 15.929866, -0.596468, 0, 15.961719, -0.623438, 0, 15.929866, -0.596468, 0, 15.929866, -0.596468, 0, 15.961719, -0.623438, 0, 15.961719, -0.623438, 0, 15.961719, -0.623438, 0, 15.986214, -0.655078, 0, 15.961719, -0.623438, 0, 15.961719, -0.623438, 0, 15.986214, -0.655078, 0, 15.986214, -0.655078, 0, 15.986214, -0.655078, 0, 16.003712, -0.691667, 0, 15.986214, -0.655078, 0, 15.986214, -0.655078, 0, 16.003712, -0.691667, 0, 16.003712, -0.691667, 0, 16.003712, -0.691667, 0, 16.014210, -0.733203, 0, 16.003712, -0.691667, 0, 16.003712, -0.691667, 0, 16.014210, -0.733203, 0, 16.014210, -0.733203, 0, 16.014210, -0.733203, 0, 16.017708, -0.779688, 0, 16.014210, -0.733203, 0, 16.014210, -0.733203, 0, 16.017708, -0.779688, 0, 16.017708, -0.779688, 0, 16.017708, -0.779688, 0, 16.012972, -0.834489, 0, 16.017708, -0.779688, 0, 16.017708, -0.779688, 0, 16.012972, -0.834489, 0, 16.012972, -0.834489, 0, 16.012972, -0.834489, 0, 15.998763, -0.882227, 0, 16.012972, -0.834489, 0, 16.012972, -0.834489, 0, 15.998763, -0.882227, 0, 15.998763, -0.882227, 0, 15.998763, -0.882227, 0, 15.975081, -0.922900, 0, 15.998763, -0.882227, 0, 15.998763, -0.882227, 0, 15.975081, -0.922900, 0, 15.975081, -0.922900, 0, 15.975081, -0.922900, 0, 15.941927, -0.956510, 0, 15.975081, -0.922900, 0, 15.975081, -0.922900, 0, 15.941927, -0.956510, 0, 15.941927, -0.956510, 0, 15.941927, -0.956510, 0, 15.899627, -0.982829, 0, 15.941927, -0.956510, 0, 15.941927, -0.956510, 0, 15.899627, -0.982829, 0, 15.899627, -0.982829, 0, 15.899627, -0.982829, 0, 15.848503, -1.001628, 0, 15.899627, -0.982829, 0, 15.899627, -0.982829, 0, 15.848503, -1.001628, 0, 15.848503, -1.001628, 0, 15.848503, -1.001628, 0, 15.788558, -1.012907, 0, 15.848503, -1.001628, 0, 15.848503, -1.001628, 0, 15.788558, -1.012907, 0, 15.788558, -1.012907, 0, 15.788558, -1.012907, 0, 15.719792, -1.016667, 0, 15.788558, -1.012907, 0, 15.788558, -1.012907, 0, 15.719792, -1.016667, 0, 15.719792, -1.016667, 0, 15.719792, -1.016667, 0, 15.691749, -1.015877, 0, 15.719792, -1.016667, 0, 15.719792, -1.016667, 0, 15.691749, -1.015877, 0, 15.691749, -1.015877, 0, 15.691749, -1.015877, 0, 15.662826, -1.013509, 0, 15.691749, -1.015877, 0, 15.691749, -1.015877, 0, 15.662826, -1.013509, 0, 15.662826, -1.013509, 0, 15.662826, -1.013509, 0, 15.633025, -1.009562, 0, 15.662826, -1.013509, 0, 15.662826, -1.013509, 0, 15.633025, -1.009562, 0, 15.633025, -1.009562, 0, 15.633025, -1.009562, 0, 15.602344, -1.004036, 0, 15.633025, -1.009562, 0, 15.633025, -1.009562, 0, 15.602344, -1.004036, 0, 15.602344, -1.004036, 0, 15.602344, -1.004036, 0, 15.570948, -0.996965, 0, 15.602344, -1.004036, 0, 15.602344, -1.004036, 0, 15.570948, -0.996965, 0, 15.570948, -0.996965, 0, 15.570948, -0.996965, 0, 15.538997, -0.988379, 0, 15.570948, -0.996965, 0, 15.570948, -0.996965, 0, 15.538997, -0.988379, 0, 15.538997, -0.988379, 0, 15.538997, -0.988379, 0, 15.506495, -0.978280, 0, 15.538997, -0.988379, 0, 15.538997, -0.988379, 0, 15.506495, -0.978280, 0, 15.506495, -0.978280, 0, 15.506495, -0.978280, 0, 15.473437, -0.966667, 0, 15.506495, -0.978280, 0, 15.506495, -0.978280, 0, 15.473437, -0.966667, 0, 15.473437, -0.966667, 0, 15.473437, -0.966667, 0, 15.473437, -0.850000, 0, 15.473437, -0.966667, 0, 15.473437, -0.966667, 0, 15.473437, -0.850000, 0, 15.473437, -0.850000, 0, 15.473437, -0.850000, 0, 15.505305, -0.869425, 0, 15.473437, -0.850000, 0, 15.473437, -0.850000, 0, 15.505305, -0.869425, 0, 15.505305, -0.869425, 0, 15.505305, -0.869425, 0, 15.536849, -0.886296, 0, 15.505305, -0.869425, 0, 15.505305, -0.869425, 0, 15.536849, -0.886296, 0, 15.536849, -0.886296, 0, 15.536849, -0.886296, 0, 15.568067, -0.900610, 0, 15.536849, -0.886296, 0, 15.536849, -0.886296, 0, 15.568067, -0.900610, 0, 15.568067, -0.900610, 0, 15.568067, -0.900610, 0, 15.598958, -0.912370, 0, 15.568067, -0.900610, 0, 15.568067, -0.900610, 0, 15.598958, -0.912370, 0, 15.598958, -0.912370, 0, 15.598958, -0.912370, 0, 15.629558, -0.921541, 0, 15.598958, -0.912370, 0, 15.598958, -0.912370, 0, 15.629558, -0.921541, 0, 15.629558, -0.921541, 0, 15.629558, -0.921541, 0, 15.659896, -0.928092, 0, 15.629558, -0.921541, 0, 15.629558, -0.921541, 0, 15.659896, -0.928092, 0, 15.659896, -0.928092, 0, 15.659896, -0.928092, 0, 15.689974, -0.932023, 0, 15.659896, -0.928092, 0, 15.659896, -0.928092, 0, 15.689974, -0.932023, 0, 15.689974, -0.932023, 0, 15.689974, -0.932023, 0, 15.719792, -0.933333, 0, 15.689974, -0.932023, 0, 15.689974, -0.932023, 0, 15.719792, -0.933333, 0, 15.719792, -0.933333, 0, 15.719792, -0.933333, 0, 15.762273, -0.930998, 0, 15.719792, -0.933333, 0, 15.719792, -0.933333, 0, 15.762273, -0.930998, 0, 15.762273, -0.930998, 0, 15.762273, -0.930998, 0, 15.799609, -0.923991, 0, 15.762273, -0.930998, 0, 15.762273, -0.930998, 0, 15.799609, -0.923991, 0, 15.799609, -0.923991, 0, 15.799609, -0.923991, 0, 15.831804, -0.912313, 0, 15.799609, -0.923991, 0, 15.799609, -0.923991, 0, 15.831804, -0.912313, 0, 15.831804, -0.912313, 0, 15.831804, -0.912313, 0, 15.858855, -0.895964, 0, 15.831804, -0.912313, 0, 15.831804, -0.912313, 0, 15.858855, -0.895964, 0, 15.858855, -0.895964, 0, 15.858855, -0.895964, 0, 15.880274, -0.875301, 0, 15.858855, -0.895964, 0, 15.858855, -0.895964, 0, 15.880274, -0.875301, 0, 15.880274, -0.875301, 0, 15.880274, -0.875301, 0, 15.895574, -0.850684, 0, 15.880274, -0.875301, 0, 15.880274, -0.875301, 0, 15.895574, -0.850684, 0, 15.895574, -0.850684, 0, 15.895574, -0.850684, 0, 15.904754, -0.822111, 0, 15.895574, -0.850684, 0, 15.895574, -0.850684, 0, 15.904754, -0.822111, 0, 15.904754, -0.822111, 0, 15.904754, -0.822111, 0, 15.907813, -0.789583, 0, 15.904754, -0.822111, 0, 15.904754, -0.822111, 0, 15.907813, -0.789583, 0, 15.907813, -0.789583, 0, 15.907813, -0.789583, 0, 15.905616, -0.761019, 0, 15.907813, -0.789583, 0, 15.907813, -0.789583, 0, 15.905616, -0.761019, 0, 15.905616, -0.761019, 0, 15.905616, -0.761019, 0, 15.899024, -0.735742, 0, 15.905616, -0.761019, 0, 15.905616, -0.761019, 0, 15.899024, -0.735742, 0, 15.899024, -0.735742, 0, 15.899024, -0.735742, 0, 15.888038, -0.713753, 0, 15.899024, -0.735742, 0, 15.899024, -0.735742, 0, 15.888038, -0.713753, 0, 15.888038, -0.713753, 0, 15.888038, -0.713753, 0, 15.872657, -0.695052, 0, 15.888038, -0.713753, 0, 15.888038, -0.713753, 0, 15.872657, -0.695052, 0, 15.872657, -0.695052, 0, 15.872657, -0.695052, 0, 15.852263, -0.679069, 0, 15.872657, -0.695052, 0, 15.872657, -0.695052, 0, 15.852263, -0.679069, 0, 15.852263, -0.679069, 0, 15.852263, -0.679069, 0, 15.826238, -0.665234, 0, 15.852263, -0.679069, 0, 15.852263, -0.679069, 0, 15.826238, -0.665234, 0, 15.826238, -0.665234, 0, 15.826238, -0.665234, 0, 15.794580, -0.653548, 0, 15.826238, -0.665234, 0, 15.826238, -0.665234, 0, 15.794580, -0.653548, 0, 15.794580, -0.653548, 0, 15.794580, -0.653548, 0, 15.757293, -0.644010, 0, 15.794580, -0.653548, 0, 15.794580, -0.653548, 0, 15.757293, -0.644010, 0, 15.757293, -0.644010, 0, 15.757293, -0.644010, 0, 15.693230, -0.631250, 0, 15.757293, -0.644010, 0, 15.757293, -0.644010, 0, 15.693230, -0.631250, 0, 15.693230, -0.631250, 0, 15.693230, -0.631250, 0, 15.638445, -0.618001, 0, 15.693230, -0.631250, 0, 15.693230, -0.631250, 0, 15.638445, -0.618001, 0, 15.638445, -0.618001, 0, 15.638445, -0.618001, 0, 15.591797, -0.601432, 0, 15.638445, -0.618001, 0, 15.638445, -0.618001, 0, 15.591797, -0.601432, 0, 15.591797, -0.601432, 0, 15.591797, -0.601432, 0, 15.553288, -0.581543, 0, 15.591797, -0.601432, 0, 15.591797, -0.601432, 0, 15.553288, -0.581543, 0, 15.553288, -0.581543, 0, 15.553288, -0.581543, 0, 15.522918, -0.558333, 0, 15.553288, -0.581543, 0, 15.553288, -0.581543, 0, 15.522918, -0.558333, 0, 15.522918, -0.558333, 0, 15.522918, -0.558333, 0, 15.499903, -0.531055, 0, 15.522918, -0.558333, 0, 15.522918, -0.558333, 0, 15.499903, -0.531055, 0, 15.499903, -0.531055, 0, 15.499903, -0.531055, 0, 15.483464, -0.498958, 0, 15.499903, -0.531055, 0, 15.499903, -0.531055, 0, 15.483464, -0.498958, 0, 15.483464, -0.498958, 0, 15.483464, -0.498958, 0, 15.473601, -0.462044, 0, 15.483464, -0.498958, 0, 15.483464, -0.498958, 0, 15.473601, -0.462044, 0, 15.473601, -0.462044, 0, 15.473601, -0.462044, 0, 15.470313, -0.420312, 0, 15.473601, -0.462044, 0, 15.473601, -0.462044, 0, 15.470313, -0.420312, 0, 15.470313, -0.420312, 0, 15.470313, -0.420312, 0, 15.474854, -0.371851, 0, 15.470313, -0.420312, 0, 15.470313, -0.420312, 0, 15.474854, -0.371851, 0, 15.474854, -0.371851, 0, 15.474854, -0.371851, 0, 15.488477, -0.328809, 0, 15.474854, -0.371851, 0, 15.474854, -0.371851, 0, 15.488477, -0.328809, 0, 15.488477, -0.328809, 0, 15.488477, -0.328809, 0, 15.511182, -0.291187, 0, 15.488477, -0.328809, 0, 15.488477, -0.328809, 0, 15.511182, -0.291187, 0, 15.511182, -0.291187, 0, 15.511182, -0.291187, 0, 15.542969, -0.258984, 0, 15.511182, -0.291187, 0, 15.511182, -0.291187, 0, 15.542969, -0.258984, 0, 15.542969, -0.258984, 0, 15.542969, -0.258984, 0, 15.582732, -0.233179, 0, 15.542969, -0.258984, 0, 15.542969, -0.258984, 0, 15.582732, -0.233179, 0, 15.582732, -0.233179, 0, 15.582732, -0.233179, 0, 15.629363, -0.214746, 0, 15.582732, -0.233179, 0, 15.582732, -0.233179, 0, 15.629363, -0.214746, 0, 15.629363, -0.214746, 0, 15.629363, -0.214746, 0, 15.682861, -0.203687, 0, 15.629363, -0.214746, 0, 15.629363, -0.214746, 0, 15.682861, -0.203687, 0, 15.682861, -0.203687, 0, 15.682861, -0.203687, 0, 15.743230, -0.200000, 0, 15.682861, -0.203687, 0, 15.682861, -0.203687, 0, 15.743230, -0.200000, 0, 15.743230, -0.200000, 0, 15.743230, -0.200000, 0, 15.770703, -0.200521, 0, 15.743230, -0.200000, 0, 15.743230, -0.200000, 0, 15.770703, -0.200521, 0, 15.770703, -0.200521, 0, 15.770703, -0.200521, 0, 15.798438, -0.202083, 0, 15.770703, -0.200521, 0, 15.770703, -0.200521, 0, 15.798438, -0.202083, 0, 15.798438, -0.202083, 0, 15.798438, -0.202083, 0, 15.826433, -0.204687, 0, 15.798438, -0.202083, 0, 15.798438, -0.202083, 0, 15.826433, -0.204687, 0, 15.826433, -0.204687, 0, 15.826433, -0.204687, 0, 15.854688, -0.208333, 0, 15.826433, -0.204687, 0, 15.826433, -0.204687, 0, 15.854688, -0.208333, 0, 15.854688, -0.208333, 0, 15.854688, -0.208333, 0, 15.883235, -0.213021, 0, 15.854688, -0.208333, 0, 15.854688, -0.208333, 0, 15.883235, -0.213021, 0, 15.883235, -0.213021, 0, 15.883235, -0.213021, 0, 15.912109, -0.218750, 0, 15.883235, -0.213021, 0, 15.883235, -0.213021, 0, 15.912109, -0.218750, 0, 15.912109, -0.218750, 0, 15.912109, -0.218750, 0, 15.941309, -0.225521, 0, 15.912109, -0.218750, 0, 15.912109, -0.218750, 0, 15.941309, -0.225521, 0, 15.941309, -0.225521, 0, 15.474854, -0.371851, 0, 15.470313, -0.420312, 0, 15.576042, -0.409896, 0, 15.470313, -0.420312, 0, 15.474854, -0.371851, 0, 15.576042, -0.409896, 0, 15.474854, -0.371851, 0, 15.576042, -0.409896, 0, 15.578956, -0.381291, 0, 15.576042, -0.409896, 0, 15.474854, -0.371851, 0, 15.578956, -0.381291, 0, 15.488477, -0.328809, 0, 15.474854, -0.371851, 0, 15.578956, -0.381291, 0, 15.474854, -0.371851, 0, 15.488477, -0.328809, 0, 15.578956, -0.381291, 0, 15.488477, -0.328809, 0, 15.578956, -0.381291, 0, 15.587695, -0.356152, 0, 15.578956, -0.381291, 0, 15.488477, -0.328809, 0, 15.587695, -0.356152, 0, 15.511182, -0.291187, 0, 15.488477, -0.328809, 0, 15.587695, -0.356152, 0, 15.488477, -0.328809, 0, 15.511182, -0.291187, 0, 15.587695, -0.356152, 0, 15.511182, -0.291187, 0, 15.587695, -0.356152, 0, 15.602263, -0.334481, 0, 15.587695, -0.356152, 0, 15.511182, -0.291187, 0, 15.602263, -0.334481, 0, 15.542969, -0.258984, 0, 15.511182, -0.291187, 0, 15.602263, -0.334481, 0, 15.511182, -0.291187, 0, 15.542969, -0.258984, 0, 15.602263, -0.334481, 0, 15.542969, -0.258984, 0, 15.602263, -0.334481, 0, 15.622657, -0.316276, 0, 15.602263, -0.334481, 0, 15.542969, -0.258984, 0, 15.622657, -0.316276, 0, 15.582732, -0.233179, 0, 15.542969, -0.258984, 0, 15.622657, -0.316276, 0, 15.542969, -0.258984, 0, 15.582732, -0.233179, 0, 15.622657, -0.316276, 0, 15.582732, -0.233179, 0, 15.622657, -0.316276, 0, 15.648421, -0.301864, 0, 15.622657, -0.316276, 0, 15.582732, -0.233179, 0, 15.648421, -0.301864, 0, 15.629363, -0.214746, 0, 15.582732, -0.233179, 0, 15.648421, -0.301864, 0, 15.582732, -0.233179, 0, 15.629363, -0.214746, 0, 15.648421, -0.301864, 0, 15.629363, -0.214746, 0, 15.648421, -0.301864, 0, 15.679102, -0.291569, 0, 15.648421, -0.301864, 0, 15.629363, -0.214746, 0, 15.679102, -0.291569, 0, 15.629363, -0.214746, 0, 15.679102, -0.291569, 0, 15.682861, -0.203687, 0, 15.679102, -0.291569, 0, 15.629363, -0.214746, 0, 15.682861, -0.203687, 0, 15.682861, -0.203687, 0, 15.679102, -0.291569, 0, 15.714698, -0.285392, 0, 15.679102, -0.291569, 0, 15.682861, -0.203687, 0, 15.714698, -0.285392, 0, 15.743230, -0.200000, 0, 15.682861, -0.203687, 0, 15.714698, -0.285392, 0, 15.682861, -0.203687, 0, 15.743230, -0.200000, 0, 15.714698, -0.285392, 0, 15.743230, -0.200000, 0, 15.714698, -0.285392, 0, 15.755208, -0.283333, 0, 15.714698, -0.285392, 0, 15.743230, -0.200000, 0, 15.755208, -0.283333, 0, 15.755208, -0.283333, 0, 15.770703, -0.200521, 0, 15.743230, -0.200000, 0, 15.770703, -0.200521, 0, 15.755208, -0.283333, 0, 15.743230, -0.200000, 0, 15.770703, -0.200521, 0, 15.755208, -0.283333, 0, 15.780176, -0.284106, 0, 15.755208, -0.283333, 0, 15.770703, -0.200521, 0, 15.780176, -0.284106, 0, 15.770703, -0.200521, 0, 15.780176, -0.284106, 0, 15.798438, -0.202083, 0, 15.780176, -0.284106, 0, 15.770703, -0.200521, 0, 15.798438, -0.202083, 0, 15.798438, -0.202083, 0, 15.780176, -0.284106, 0, 15.805598, -0.286426, 0, 15.780176, -0.284106, 0, 15.798438, -0.202083, 0, 15.805598, -0.286426, 0, 15.826433, -0.204687, 0, 15.798438, -0.202083, 0, 15.805598, -0.286426, 0, 15.798438, -0.202083, 0, 15.826433, -0.204687, 0, 15.805598, -0.286426, 0, 15.826433, -0.204687, 0, 15.805598, -0.286426, 0, 15.831477, -0.290291, 0, 15.805598, -0.286426, 0, 15.826433, -0.204687, 0, 15.831477, -0.290291, 0, 15.826433, -0.204687, 0, 15.831477, -0.290291, 0, 15.854688, -0.208333, 0, 15.831477, -0.290291, 0, 15.826433, -0.204687, 0, 15.854688, -0.208333, 0, 15.854688, -0.208333, 0, 15.831477, -0.290291, 0, 15.857813, -0.295703, 0, 15.831477, -0.290291, 0, 15.854688, -0.208333, 0, 15.857813, -0.295703, 0, 15.854688, -0.208333, 0, 15.857813, -0.295703, 0, 15.883235, -0.213021, 0, 15.857813, -0.295703, 0, 15.854688, -0.208333, 0, 15.883235, -0.213021, 0, 15.883235, -0.213021, 0, 15.857813, -0.295703, 0, 15.884798, -0.302694, 0, 15.857813, -0.295703, 0, 15.883235, -0.213021, 0, 15.884798, -0.302694, 0, 15.883235, -0.213021, 0, 15.884798, -0.302694, 0, 15.912109, -0.218750, 0, 15.884798, -0.302694, 0, 15.883235, -0.213021, 0, 15.912109, -0.218750, 0, 15.912109, -0.218750, 0, 15.884798, -0.302694, 0, 15.912630, -0.311296, 0, 15.884798, -0.302694, 0, 15.912109, -0.218750, 0, 15.912630, -0.311296, 0, 15.912109, -0.218750, 0, 15.912630, -0.311296, 0, 15.941309, -0.225521, 0, 15.912630, -0.311296, 0, 15.912109, -0.218750, 0, 15.941309, -0.225521, 0, 15.941309, -0.225521, 0, 15.912630, -0.311296, 0, 15.941309, -0.321509, 0, 15.912630, -0.311296, 0, 15.941309, -0.225521, 0, 15.941309, -0.321509, 0, 15.970833, -0.233333, 0, 15.941309, -0.225521, 0, 15.941309, -0.321509, 0, 15.941309, -0.225521, 0, 15.970833, -0.233333, 0, 15.941309, -0.321509, 0, 15.941309, -0.321509, 0, 15.970833, -0.333333, 0, 15.970833, -0.233333, 0, 15.970833, -0.333333, 0, 15.941309, -0.321509, 0, 15.970833, -0.233333, 0, 15.576042, -0.409896, 0, 15.470313, -0.420312, 0, 15.577979, -0.433740, 0, 15.470313, -0.420312, 0, 15.576042, -0.409896, 0, 15.577979, -0.433740, 0, 15.470313, -0.420312, 0, 15.473601, -0.462044, 0, 15.577979, -0.433740, 0, 15.473601, -0.462044, 0, 15.470313, -0.420312, 0, 15.577979, -0.433740, 0, 15.577979, -0.433740, 0, 15.473601, -0.462044, 0, 15.583789, -0.454492, 0, 15.473601, -0.462044, 0, 15.577979, -0.433740, 0, 15.583789, -0.454492, 0, 15.473601, -0.462044, 0, 15.483464, -0.498958, 0, 15.583789, -0.454492, 0, 15.483464, -0.498958, 0, 15.473601, -0.462044, 0, 15.583789, -0.454492, 0, 15.583789, -0.454492, 0, 15.483464, -0.498958, 0, 15.593474, -0.472152, 0, 15.483464, -0.498958, 0, 15.583789, -0.454492, 0, 15.593474, -0.472152, 0, 15.483464, -0.498958, 0, 15.499903, -0.531055, 0, 15.593474, -0.472152, 0, 15.499903, -0.531055, 0, 15.483464, -0.498958, 0, 15.593474, -0.472152, 0, 15.593474, -0.472152, 0, 15.499903, -0.531055, 0, 15.522918, -0.558333, 0, 15.499903, -0.531055, 0, 15.593474, -0.472152, 0, 15.522918, -0.558333, 0, 15.607032, -0.486719, 0, 15.593474, -0.472152, 0, 15.522918, -0.558333, 0, 15.593474, -0.472152, 0, 15.607032, -0.486719, 0, 15.522918, -0.558333, 0, 15.607032, -0.486719, 0, 15.522918, -0.558333, 0, 15.553288, -0.581543, 0, 15.522918, -0.558333, 0, 15.607032, -0.486719, 0, 15.553288, -0.581543, 0, 15.625994, -0.499105, 0, 15.607032, -0.486719, 0, 15.553288, -0.581543, 0, 15.607032, -0.486719, 0, 15.625994, -0.499105, 0, 15.553288, -0.581543, 0, 15.553288, -0.581543, 0, 15.591797, -0.601432, 0, 15.625994, -0.499105, 0, 15.591797, -0.601432, 0, 15.553288, -0.581543, 0, 15.625994, -0.499105, 0, 15.625994, -0.499105, 0, 15.591797, -0.601432, 0, 15.651888, -0.510221, 0, 15.591797, -0.601432, 0, 15.625994, -0.499105, 0, 15.651888, -0.510221, 0, 15.591797, -0.601432, 0, 15.638445, -0.618001, 0, 15.651888, -0.510221, 0, 15.638445, -0.618001, 0, 15.591797, -0.601432, 0, 15.651888, -0.510221, 0, 15.651888, -0.510221, 0, 15.638445, -0.618001, 0, 15.684717, -0.520068, 0, 15.638445, -0.618001, 0, 15.651888, -0.510221, 0, 15.684717, -0.520068, 0, 15.684717, -0.520068, 0, 15.638445, -0.618001, 0, 15.693230, -0.631250, 0, 15.638445, -0.618001, 0, 15.684717, -0.520068, 0, 15.693230, -0.631250, 0, 15.684717, -0.520068, 0, 15.693230, -0.631250, 0, 15.724480, -0.528646, 0, 15.693230, -0.631250, 0, 15.684717, -0.520068, 0, 15.724480, -0.528646, 0, 15.724480, -0.528646, 0, 15.693230, -0.631250, 0, 15.757293, -0.644010, 0, 15.693230, -0.631250, 0, 15.724480, -0.528646, 0, 15.757293, -0.644010, 0, 15.724480, -0.528646, 0, 15.757293, -0.644010, 0, 15.788020, -0.541927, 0, 15.757293, -0.644010, 0, 15.724480, -0.528646, 0, 15.788020, -0.541927, 0, 15.788020, -0.541927, 0, 15.757293, -0.644010, 0, 15.794580, -0.653548, 0, 15.757293, -0.644010, 0, 15.788020, -0.541927, 0, 15.794580, -0.653548, 0, 15.788020, -0.541927, 0, 15.794580, -0.653548, 0, 15.843018, -0.555713, 0, 15.794580, -0.653548, 0, 15.788020, -0.541927, 0, 15.843018, -0.555713, 0, 15.794580, -0.653548, 0, 15.826238, -0.665234, 0, 15.843018, -0.555713, 0, 15.826238, -0.665234, 0, 15.794580, -0.653548, 0, 15.843018, -0.555713, 0, 15.843018, -0.555713, 0, 15.826238, -0.665234, 0, 15.890300, -0.573893, 0, 15.826238, -0.665234, 0, 15.843018, -0.555713, 0, 15.890300, -0.573893, 0, 15.826238, -0.665234, 0, 15.852263, -0.679069, 0, 15.890300, -0.573893, 0, 15.852263, -0.679069, 0, 15.826238, -0.665234, 0, 15.890300, -0.573893, 0, 15.890300, -0.573893, 0, 15.852263, -0.679069, 0, 15.929866, -0.596468, 0, 15.852263, -0.679069, 0, 15.890300, -0.573893, 0, 15.929866, -0.596468, 0, 15.852263, -0.679069, 0, 15.872657, -0.695052, 0, 15.929866, -0.596468, 0, 15.872657, -0.695052, 0, 15.852263, -0.679069, 0, 15.929866, -0.596468, 0, 15.929866, -0.596468, 0, 15.872657, -0.695052, 0, 15.961719, -0.623438, 0, 15.872657, -0.695052, 0, 15.929866, -0.596468, 0, 15.961719, -0.623438, 0, 15.872657, -0.695052, 0, 15.888038, -0.713753, 0, 15.961719, -0.623438, 0, 15.888038, -0.713753, 0, 15.872657, -0.695052, 0, 15.961719, -0.623438, 0, 15.961719, -0.623438, 0, 15.888038, -0.713753, 0, 15.986214, -0.655078, 0, 15.888038, -0.713753, 0, 15.961719, -0.623438, 0, 15.986214, -0.655078, 0, 15.986214, -0.655078, 0, 15.888038, -0.713753, 0, 16.003712, -0.691667, 0, 15.888038, -0.713753, 0, 15.986214, -0.655078, 0, 16.003712, -0.691667, 0, 16.003712, -0.691667, 0, 15.888038, -0.713753, 0, 15.899024, -0.735742, 0, 15.888038, -0.713753, 0, 16.003712, -0.691667, 0, 15.899024, -0.735742, 0, 16.003712, -0.691667, 0, 15.899024, -0.735742, 0, 16.014210, -0.733203, 0, 15.899024, -0.735742, 0, 16.003712, -0.691667, 0, 16.014210, -0.733203, 0, 16.014210, -0.733203, 0, 15.899024, -0.735742, 0, 15.905616, -0.761019, 0, 15.899024, -0.735742, 0, 16.014210, -0.733203, 0, 15.905616, -0.761019, 0, 16.014210, -0.733203, 0, 15.905616, -0.761019, 0, 16.017708, -0.779688, 0, 15.905616, -0.761019, 0, 16.014210, -0.733203, 0, 16.017708, -0.779688, 0, 15.905616, -0.761019, 0, 15.907813, -0.789583, 0, 16.017708, -0.779688, 0, 15.907813, -0.789583, 0, 15.905616, -0.761019, 0, 16.017708, -0.779688, 0, 15.907813, -0.789583, 0, 16.012972, -0.834489, 0, 16.017708, -0.779688, 0, 16.012972, -0.834489, 0, 15.907813, -0.789583, 0, 16.017708, -0.779688, 0, 15.907813, -0.789583, 0, 15.904754, -0.822111, 0, 16.012972, -0.834489, 0, 15.904754, -0.822111, 0, 15.907813, -0.789583, 0, 16.012972, -0.834489, 0, 16.012972, -0.834489, 0, 15.904754, -0.822111, 0, 15.998763, -0.882227, 0, 15.904754, -0.822111, 0, 16.012972, -0.834489, 0, 15.998763, -0.882227, 0, 15.904754, -0.822111, 0, 15.895574, -0.850684, 0, 15.998763, -0.882227, 0, 15.895574, -0.850684, 0, 15.904754, -0.822111, 0, 15.998763, -0.882227, 0, 15.895574, -0.850684, 0, 15.975081, -0.922900, 0, 15.998763, -0.882227, 0, 15.975081, -0.922900, 0, 15.895574, -0.850684, 0, 15.998763, -0.882227, 0, 15.895574, -0.850684, 0, 15.880274, -0.875301, 0, 15.975081, -0.922900, 0, 15.880274, -0.875301, 0, 15.895574, -0.850684, 0, 15.975081, -0.922900, 0, 15.880274, -0.875301, 0, 15.941927, -0.956510, 0, 15.975081, -0.922900, 0, 15.941927, -0.956510, 0, 15.880274, -0.875301, 0, 15.975081, -0.922900, 0, 15.880274, -0.875301, 0, 15.858855, -0.895964, 0, 15.941927, -0.956510, 0, 15.858855, -0.895964, 0, 15.880274, -0.875301, 0, 15.941927, -0.956510, 0, 15.858855, -0.895964, 0, 15.899627, -0.982829, 0, 15.941927, -0.956510, 0, 15.899627, -0.982829, 0, 15.858855, -0.895964, 0, 15.941927, -0.956510, 0, 15.858855, -0.895964, 0, 15.831804, -0.912313, 0, 15.899627, -0.982829, 0, 15.831804, -0.912313, 0, 15.858855, -0.895964, 0, 15.899627, -0.982829, 0, 15.831804, -0.912313, 0, 15.848503, -1.001628, 0, 15.899627, -0.982829, 0, 15.848503, -1.001628, 0, 15.831804, -0.912313, 0, 15.899627, -0.982829, 0, 15.831804, -0.912313, 0, 15.799609, -0.923991, 0, 15.848503, -1.001628, 0, 15.799609, -0.923991, 0, 15.831804, -0.912313, 0, 15.848503, -1.001628, 0, 15.799609, -0.923991, 0, 15.788558, -1.012907, 0, 15.848503, -1.001628, 0, 15.788558, -1.012907, 0, 15.799609, -0.923991, 0, 15.848503, -1.001628, 0, 15.762273, -0.930998, 0, 15.788558, -1.012907, 0, 15.799609, -0.923991, 0, 15.788558, -1.012907, 0, 15.762273, -0.930998, 0, 15.799609, -0.923991, 0, 15.788558, -1.012907, 0, 15.762273, -0.930998, 0, 15.719792, -1.016667, 0, 15.762273, -0.930998, 0, 15.788558, -1.012907, 0, 15.719792, -1.016667, 0, 15.762273, -0.930998, 0, 15.719792, -0.933333, 0, 15.719792, -1.016667, 0, 15.719792, -0.933333, 0, 15.762273, -0.930998, 0, 15.719792, -1.016667, 0, 15.691749, -1.015877, 0, 15.719792, -1.016667, 0, 15.719792, -0.933333, 0, 15.719792, -1.016667, 0, 15.691749, -1.015877, 0, 15.719792, -0.933333, 0, 15.689974, -0.932023, 0, 15.691749, -1.015877, 0, 15.719792, -0.933333, 0, 15.691749, -1.015877, 0, 15.689974, -0.932023, 0, 15.719792, -0.933333, 0, 15.689974, -0.932023, 0, 15.662826, -1.013509, 0, 15.691749, -1.015877, 0, 15.662826, -1.013509, 0, 15.689974, -0.932023, 0, 15.691749, -1.015877, 0, 15.659896, -0.928092, 0, 15.662826, -1.013509, 0, 15.689974, -0.932023, 0, 15.662826, -1.013509, 0, 15.659896, -0.928092, 0, 15.689974, -0.932023, 0, 15.659896, -0.928092, 0, 15.633025, -1.009562, 0, 15.662826, -1.013509, 0, 15.633025, -1.009562, 0, 15.659896, -0.928092, 0, 15.662826, -1.013509, 0, 15.629558, -0.921541, 0, 15.633025, -1.009562, 0, 15.659896, -0.928092, 0, 15.633025, -1.009562, 0, 15.629558, -0.921541, 0, 15.659896, -0.928092, 0, 15.629558, -0.921541, 0, 15.602344, -1.004036, 0, 15.633025, -1.009562, 0, 15.602344, -1.004036, 0, 15.629558, -0.921541, 0, 15.633025, -1.009562, 0, 15.598958, -0.912370, 0, 15.602344, -1.004036, 0, 15.629558, -0.921541, 0, 15.602344, -1.004036, 0, 15.598958, -0.912370, 0, 15.629558, -0.921541, 0, 15.598958, -0.912370, 0, 15.570948, -0.996965, 0, 15.602344, -1.004036, 0, 15.570948, -0.996965, 0, 15.598958, -0.912370, 0, 15.602344, -1.004036, 0, 15.568067, -0.900610, 0, 15.570948, -0.996965, 0, 15.598958, -0.912370, 0, 15.570948, -0.996965, 0, 15.568067, -0.900610, 0, 15.598958, -0.912370, 0, 15.568067, -0.900610, 0, 15.538997, -0.988379, 0, 15.570948, -0.996965, 0, 15.538997, -0.988379, 0, 15.568067, -0.900610, 0, 15.570948, -0.996965, 0, 15.568067, -0.900610, 0, 15.506495, -0.978280, 0, 15.538997, -0.988379, 0, 15.506495, -0.978280, 0, 15.568067, -0.900610, 0, 15.538997, -0.988379, 0, 15.568067, -0.900610, 0, 15.536849, -0.886296, 0, 15.506495, -0.978280, 0, 15.536849, -0.886296, 0, 15.568067, -0.900610, 0, 15.506495, -0.978280, 0, 15.473437, -0.966667, 0, 15.506495, -0.978280, 0, 15.536849, -0.886296, 0, 15.506495, -0.978280, 0, 15.473437, -0.966667, 0, 15.536849, -0.886296, 0, 15.505305, -0.869425, 0, 15.473437, -0.966667, 0, 15.536849, -0.886296, 0, 15.473437, -0.966667, 0, 15.505305, -0.869425, 0, 15.536849, -0.886296, 0, 15.473437, -0.850000, 0, 15.473437, -0.966667, 0, 15.505305, -0.869425, 0, 15.473437, -0.966667, 0, 15.473437, -0.850000, 0, 15.505305, -0.869425, 0, 16.293230, -0.633333, 0, 16.293230, -0.916667, 0, 16.293230, -0.633333, 0, 16.293230, -0.633333, 0, 16.293230, -0.916667, 0, 16.293230, -0.916667, 0, 16.293230, -0.916667, 0, 16.461981, -0.916667, 0, 16.293230, -0.916667, 0, 16.293230, -0.916667, 0, 16.461981, -0.916667, 0, 16.461981, -0.916667, 0, 16.461981, -0.916667, 0, 16.501677, -0.914478, 0, 16.461981, -0.916667, 0, 16.461981, -0.916667, 0, 16.501677, -0.914478, 0, 16.501677, -0.914478, 0, 16.501677, -0.914478, 0, 16.535872, -0.907910, 0, 16.501677, -0.914478, 0, 16.501677, -0.914478, 0, 16.535872, -0.907910, 0, 16.535872, -0.907910, 0, 16.535872, -0.907910, 0, 16.564568, -0.896964, 0, 16.535872, -0.907910, 0, 16.535872, -0.907910, 0, 16.564568, -0.896964, 0, 16.564568, -0.896964, 0, 16.564568, -0.896964, 0, 16.587761, -0.881641, 0, 16.564568, -0.896964, 0, 16.564568, -0.896964, 0, 16.587761, -0.881641, 0, 16.587761, -0.881641, 0, 16.587761, -0.881641, 0, 16.605648, -0.861825, 0, 16.587761, -0.881641, 0, 16.587761, -0.881641, 0, 16.605648, -0.861825, 0, 16.605648, -0.861825, 0, 16.605648, -0.861825, 0, 16.618425, -0.837402, 0, 16.605648, -0.861825, 0, 16.605648, -0.861825, 0, 16.618425, -0.837402, 0, 16.618425, -0.837402, 0, 16.618425, -0.837402, 0, 16.626091, -0.808374, 0, 16.618425, -0.837402, 0, 16.618425, -0.837402, 0, 16.626091, -0.808374, 0, 16.626091, -0.808374, 0, 16.626091, -0.808374, 0, 16.628645, -0.774740, 0, 16.626091, -0.808374, 0, 16.626091, -0.808374, 0, 16.628645, -0.774740, 0, 16.628645, -0.774740, 0, 16.628645, -0.774740, 0, 16.626091, -0.740910, 0, 16.628645, -0.774740, 0, 16.628645, -0.774740, 0, 16.626091, -0.740910, 0, 16.626091, -0.740910, 0, 16.626091, -0.740910, 0, 16.618425, -0.711816, 0, 16.626091, -0.740910, 0, 16.626091, -0.740910, 0, 16.618425, -0.711816, 0, 16.618425, -0.711816, 0, 16.618425, -0.711816, 0, 16.605648, -0.687459, 0, 16.618425, -0.711816, 0, 16.618425, -0.711816, 0, 16.605648, -0.687459, 0, 16.605648, -0.687459, 0, 16.605648, -0.687459, 0, 16.587761, -0.667839, 0, 16.605648, -0.687459, 0, 16.605648, -0.687459, 0, 16.587761, -0.667839, 0, 16.587761, -0.667839, 0, 16.587761, -0.667839, 0, 16.564568, -0.652743, 0, 16.587761, -0.667839, 0, 16.587761, -0.667839, 0, 16.564568, -0.652743, 0, 16.564568, -0.652743, 0, 16.564568, -0.652743, 0, 16.535872, -0.641960, 0, 16.564568, -0.652743, 0, 16.564568, -0.652743, 0, 16.535872, -0.641960, 0, 16.535872, -0.641960, 0, 16.535872, -0.641960, 0, 16.501677, -0.635490, 0, 16.535872, -0.641960, 0, 16.535872, -0.641960, 0, 16.501677, -0.635490, 0, 16.501677, -0.635490, 0, 16.501677, -0.635490, 0, 16.461981, -0.633333, 0, 16.501677, -0.635490, 0, 16.501677, -0.635490, 0, 16.461981, -0.633333, 0, 16.461981, -0.633333, 0, 16.293230, -0.300000, 0, 16.293230, -0.550000, 0, 16.293230, -0.300000, 0, 16.293230, -0.300000, 0, 16.293230, -0.550000, 0, 16.293230, -0.550000, 0, 16.293230, -0.550000, 0, 16.448957, -0.550000, 0, 16.293230, -0.550000, 0, 16.293230, -0.550000, 0, 16.448957, -0.550000, 0, 16.448957, -0.550000, 0, 16.448957, -0.550000, 0, 16.485043, -0.548071, 0, 16.448957, -0.550000, 0, 16.448957, -0.550000, 0, 16.485043, -0.548071, 0, 16.485043, -0.548071, 0, 16.485043, -0.548071, 0, 16.516211, -0.542285, 0, 16.485043, -0.548071, 0, 16.485043, -0.548071, 0, 16.516211, -0.542285, 0, 16.516211, -0.542285, 0, 16.516211, -0.542285, 0, 16.542463, -0.532642, 0, 16.516211, -0.542285, 0, 16.516211, -0.542285, 0, 16.542463, -0.532642, 0, 16.542463, -0.532642, 0, 16.542463, -0.532642, 0, 16.563803, -0.519141, 0, 16.542463, -0.532642, 0, 16.542463, -0.532642, 0, 16.563803, -0.519141, 0, 16.563803, -0.519141, 0, 16.563803, -0.519141, 0, 16.580322, -0.501685, 0, 16.563803, -0.519141, 0, 16.563803, -0.519141, 0, 16.580322, -0.501685, 0, 16.580322, -0.501685, 0, 16.580322, -0.501685, 0, 16.592123, -0.480176, 0, 16.580322, -0.501685, 0, 16.580322, -0.501685, 0, 16.592123, -0.480176, 0, 16.592123, -0.480176, 0, 16.592123, -0.480176, 0, 16.599203, -0.454614, 0, 16.592123, -0.480176, 0, 16.592123, -0.480176, 0, 16.599203, -0.454614, 0, 16.599203, -0.454614, 0, 16.599203, -0.454614, 0, 16.601563, -0.425000, 0, 16.599203, -0.454614, 0, 16.599203, -0.454614, 0, 16.601563, -0.425000, 0, 16.601563, -0.425000, 0, 16.601563, -0.425000, 0, 16.599203, -0.395597, 0, 16.601563, -0.425000, 0, 16.601563, -0.425000, 0, 16.599203, -0.395597, 0, 16.599203, -0.395597, 0, 16.599203, -0.395597, 0, 16.592123, -0.370150, 0, 16.599203, -0.395597, 0, 16.599203, -0.395597, 0, 16.592123, -0.370150, 0, 16.592123, -0.370150, 0, 16.592123, -0.370150, 0, 16.580322, -0.348657, 0, 16.592123, -0.370150, 0, 16.592123, -0.370150, 0, 16.580322, -0.348657, 0, 16.580322, -0.348657, 0, 16.580322, -0.348657, 0, 16.563803, -0.331120, 0, 16.580322, -0.348657, 0, 16.580322, -0.348657, 0, 16.563803, -0.331120, 0, 16.563803, -0.331120, 0, 16.563803, -0.331120, 0, 16.542463, -0.317505, 0, 16.563803, -0.331120, 0, 16.563803, -0.331120, 0, 16.542463, -0.317505, 0, 16.542463, -0.317505, 0, 16.542463, -0.317505, 0, 16.516211, -0.307780, 0, 16.542463, -0.317505, 0, 16.542463, -0.317505, 0, 16.516211, -0.307780, 0, 16.516211, -0.307780, 0, 16.516211, -0.307780, 0, 16.485043, -0.301945, 0, 16.516211, -0.307780, 0, 16.516211, -0.307780, 0, 16.485043, -0.301945, 0, 16.485043, -0.301945, 0, 16.485043, -0.301945, 0, 16.448957, -0.300000, 0, 16.485043, -0.301945, 0, 16.485043, -0.301945, 0, 16.448957, -0.300000, 0, 16.448957, -0.300000, 0, 16.188021, -0.216667, 0, 16.456770, -0.216667, 0, 16.188021, -0.216667, 0, 16.188021, -0.216667, 0, 16.456770, -0.216667, 0, 16.456770, -0.216667, 0, 16.456770, -0.216667, 0, 16.513477, -0.219889, 0, 16.456770, -0.216667, 0, 16.456770, -0.216667, 0, 16.513477, -0.219889, 0, 16.513477, -0.219889, 0, 16.513477, -0.219889, 0, 16.563282, -0.229557, 0, 16.513477, -0.219889, 0, 16.513477, -0.219889, 0, 16.563282, -0.229557, 0, 16.563282, -0.229557, 0, 16.563282, -0.229557, 0, 16.606184, -0.245671, 0, 16.563282, -0.229557, 0, 16.563282, -0.229557, 0, 16.606184, -0.245671, 0, 16.606184, -0.245671, 0, 16.606184, -0.245671, 0, 16.642187, -0.268229, 0, 16.606184, -0.245671, 0, 16.606184, -0.245671, 0, 16.642187, -0.268229, 0, 16.642187, -0.268229, 0, 16.642187, -0.268229, 0, 16.670671, -0.296745, 0, 16.642187, -0.268229, 0, 16.642187, -0.268229, 0, 16.670671, -0.296745, 0, 16.670671, -0.296745, 0, 16.670671, -0.296745, 0, 16.691015, -0.330729, 0, 16.670671, -0.296745, 0, 16.670671, -0.296745, 0, 16.691015, -0.330729, 0, 16.691015, -0.330729, 0, 16.691015, -0.330729, 0, 16.703222, -0.370182, 0, 16.691015, -0.330729, 0, 16.691015, -0.330729, 0, 16.703222, -0.370182, 0, 16.703222, -0.370182, 0, 16.703222, -0.370182, 0, 16.707293, -0.415104, 0, 16.703222, -0.370182, 0, 16.703222, -0.370182, 0, 16.707293, -0.415104, 0, 16.707293, -0.415104, 0, 16.707293, -0.415104, 0, 16.705208, -0.449959, 0, 16.707293, -0.415104, 0, 16.707293, -0.415104, 0, 16.705208, -0.449959, 0, 16.705208, -0.449959, 0, 16.705208, -0.449959, 0, 16.698957, -0.481087, 0, 16.705208, -0.449959, 0, 16.705208, -0.449959, 0, 16.698957, -0.481087, 0, 16.698957, -0.481087, 0, 16.698957, -0.481087, 0, 16.688543, -0.508488, 0, 16.698957, -0.481087, 0, 16.698957, -0.481087, 0, 16.688543, -0.508488, 0, 16.688543, -0.508488, 0, 16.688543, -0.508488, 0, 16.673958, -0.532161, 0, 16.688543, -0.508488, 0, 16.688543, -0.508488, 0, 16.673958, -0.532161, 0, 16.673958, -0.532161, 0, 16.673958, -0.532161, 0, 16.655340, -0.551912, 0, 16.673958, -0.532161, 0, 16.673958, -0.532161, 0, 16.655340, -0.551912, 0, 16.655340, -0.551912, 0, 16.655340, -0.551912, 0, 16.632813, -0.567546, 0, 16.655340, -0.551912, 0, 16.655340, -0.551912, 0, 16.632813, -0.567546, 0, 16.632813, -0.567546, 0, 16.632813, -0.567546, 0, 16.606380, -0.579061, 0, 16.632813, -0.567546, 0, 16.632813, -0.567546, 0, 16.606380, -0.579061, 0, 16.606380, -0.579061, 0, 16.606380, -0.579061, 0, 16.576042, -0.586458, 0, 16.606380, -0.579061, 0, 16.606380, -0.579061, 0, 16.576042, -0.586458, 0, 16.576042, -0.586458, 0, 16.576042, -0.586458, 0, 16.612679, -0.596899, 0, 16.576042, -0.586458, 0, 16.576042, -0.586458, 0, 16.612679, -0.596899, 0, 16.612679, -0.596899, 0, 16.612679, -0.596899, 0, 16.644987, -0.611816, 0, 16.612679, -0.596899, 0, 16.612679, -0.596899, 0, 16.644987, -0.611816, 0, 16.644987, -0.611816, 0, 16.644987, -0.611816, 0, 16.672966, -0.631209, 0, 16.644987, -0.611816, 0, 16.644987, -0.611816, 0, 16.672966, -0.631209, 0, 16.672966, -0.631209, 0, 16.672966, -0.631209, 0, 16.696615, -0.655078, 0, 16.672966, -0.631209, 0, 16.672966, -0.631209, 0, 16.696615, -0.655078, 0, 16.696615, -0.655078, 0, 16.696615, -0.655078, 0, 16.715414, -0.682788, 0, 16.696615, -0.655078, 0, 16.696615, -0.655078, 0, 16.715414, -0.682788, 0, 16.715414, -0.682788, 0, 16.715414, -0.682788, 0, 16.728842, -0.713704, 0, 16.715414, -0.682788, 0, 16.715414, -0.682788, 0, 16.728842, -0.713704, 0, 16.728842, -0.713704, 0, 16.728842, -0.713704, 0, 16.736898, -0.747827, 0, 16.728842, -0.713704, 0, 16.728842, -0.713704, 0, 16.736898, -0.747827, 0, 16.736898, -0.747827, 0, 16.736898, -0.747827, 0, 16.739584, -0.785156, 0, 16.736898, -0.747827, 0, 16.736898, -0.747827, 0, 16.739584, -0.785156, 0, 16.739584, -0.785156, 0, 16.739584, -0.785156, 0, 16.735157, -0.833659, 0, 16.739584, -0.785156, 0, 16.739584, -0.785156, 0, 16.735157, -0.833659, 0, 16.735157, -0.833659, 0, 16.735157, -0.833659, 0, 16.721876, -0.876302, 0, 16.735157, -0.833659, 0, 16.735157, -0.833659, 0, 16.721876, -0.876302, 0, 16.721876, -0.876302, 0, 16.721876, -0.876302, 0, 16.699739, -0.913086, 0, 16.721876, -0.876302, 0, 16.721876, -0.876302, 0, 16.699739, -0.913086, 0, 16.699739, -0.913086, 0, 16.699739, -0.913086, 0, 16.668751, -0.944010, 0, 16.699739, -0.913086, 0, 16.699739, -0.913086, 0, 16.668751, -0.944010, 0, 16.668751, -0.944010, 0, 16.668751, -0.944010, 0, 16.629591, -0.968506, 0, 16.668751, -0.944010, 0, 16.668751, -0.944010, 0, 16.629591, -0.968506, 0, 16.629591, -0.968506, 0, 16.629591, -0.968506, 0, 16.582943, -0.986003, 0, 16.629591, -0.968506, 0, 16.629591, -0.968506, 0, 16.582943, -0.986003, 0, 16.582943, -0.986003, 0, 16.582943, -0.986003, 0, 16.528809, -0.996501, 0, 16.582943, -0.986003, 0, 16.582943, -0.986003, 0, 16.528809, -0.996501, 0, 16.528809, -0.996501, 0, 16.528809, -0.996501, 0, 16.467188, -1, 0, 16.528809, -0.996501, 0, 16.528809, -0.996501, 0, 16.467188, -1, 0, 16.467188, -1, 0, 16.467188, -1, 0, 16.188021, -1, 0, 16.467188, -1, 0, 16.467188, -1, 0, 16.188021, -1, 0, 16.188021, -1, 0, 16.188021, -0.216667, 0, 16.293230, -0.300000, 0, 16.456770, -0.216667, 0, 16.293230, -0.300000, 0, 16.188021, -0.216667, 0, 16.456770, -0.216667, 0, 16.293230, -0.300000, 0, 16.188021, -0.216667, 0, 16.293230, -0.550000, 0, 16.188021, -0.216667, 0, 16.293230, -0.300000, 0, 16.293230, -0.550000, 0, 16.293230, -0.550000, 0, 16.188021, -0.216667, 0, 16.293230, -0.633333, 0, 16.188021, -0.216667, 0, 16.293230, -0.550000, 0, 16.293230, -0.633333, 0, 16.448957, -0.550000, 0, 16.293230, -0.550000, 0, 16.293230, -0.633333, 0, 16.293230, -0.550000, 0, 16.448957, -0.550000, 0, 16.293230, -0.633333, 0, 16.461981, -0.633333, 0, 16.448957, -0.550000, 0, 16.293230, -0.633333, 0, 16.448957, -0.550000, 0, 16.461981, -0.633333, 0, 16.293230, -0.633333, 0, 16.448957, -0.550000, 0, 16.461981, -0.633333, 0, 16.485043, -0.548071, 0, 16.461981, -0.633333, 0, 16.448957, -0.550000, 0, 16.485043, -0.548071, 0, 16.485043, -0.548071, 0, 16.461981, -0.633333, 0, 16.501677, -0.635490, 0, 16.461981, -0.633333, 0, 16.485043, -0.548071, 0, 16.501677, -0.635490, 0, 16.501677, -0.635490, 0, 16.516211, -0.542285, 0, 16.485043, -0.548071, 0, 16.516211, -0.542285, 0, 16.501677, -0.635490, 0, 16.485043, -0.548071, 0, 16.576042, -0.586458, 0, 16.516211, -0.542285, 0, 16.501677, -0.635490, 0, 16.516211, -0.542285, 0, 16.576042, -0.586458, 0, 16.501677, -0.635490, 0, 16.542463, -0.532642, 0, 16.516211, -0.542285, 0, 16.576042, -0.586458, 0, 16.516211, -0.542285, 0, 16.542463, -0.532642, 0, 16.576042, -0.586458, 0, 16.563803, -0.519141, 0, 16.542463, -0.532642, 0, 16.576042, -0.586458, 0, 16.542463, -0.532642, 0, 16.563803, -0.519141, 0, 16.576042, -0.586458, 0, 16.563803, -0.519141, 0, 16.576042, -0.586458, 0, 16.606380, -0.579061, 0, 16.576042, -0.586458, 0, 16.563803, -0.519141, 0, 16.606380, -0.579061, 0, 16.580322, -0.501685, 0, 16.563803, -0.519141, 0, 16.606380, -0.579061, 0, 16.563803, -0.519141, 0, 16.580322, -0.501685, 0, 16.606380, -0.579061, 0, 16.580322, -0.501685, 0, 16.606380, -0.579061, 0, 16.632813, -0.567546, 0, 16.606380, -0.579061, 0, 16.580322, -0.501685, 0, 16.632813, -0.567546, 0, 16.580322, -0.501685, 0, 16.632813, -0.567546, 0, 16.655340, -0.551912, 0, 16.632813, -0.567546, 0, 16.580322, -0.501685, 0, 16.655340, -0.551912, 0, 16.592123, -0.480176, 0, 16.580322, -0.501685, 0, 16.655340, -0.551912, 0, 16.580322, -0.501685, 0, 16.592123, -0.480176, 0, 16.655340, -0.551912, 0, 16.592123, -0.480176, 0, 16.655340, -0.551912, 0, 16.673958, -0.532161, 0, 16.655340, -0.551912, 0, 16.592123, -0.480176, 0, 16.673958, -0.532161, 0, 16.592123, -0.480176, 0, 16.673958, -0.532161, 0, 16.688543, -0.508488, 0, 16.673958, -0.532161, 0, 16.592123, -0.480176, 0, 16.688543, -0.508488, 0, 16.599203, -0.454614, 0, 16.592123, -0.480176, 0, 16.688543, -0.508488, 0, 16.592123, -0.480176, 0, 16.599203, -0.454614, 0, 16.688543, -0.508488, 0, 16.599203, -0.454614, 0, 16.688543, -0.508488, 0, 16.698957, -0.481087, 0, 16.688543, -0.508488, 0, 16.599203, -0.454614, 0, 16.698957, -0.481087, 0, 16.599203, -0.454614, 0, 16.698957, -0.481087, 0, 16.705208, -0.449959, 0, 16.698957, -0.481087, 0, 16.599203, -0.454614, 0, 16.705208, -0.449959, 0, 16.601563, -0.425000, 0, 16.599203, -0.454614, 0, 16.705208, -0.449959, 0, 16.599203, -0.454614, 0, 16.601563, -0.425000, 0, 16.705208, -0.449959, 0, 16.601563, -0.425000, 0, 16.705208, -0.449959, 0, 16.707293, -0.415104, 0, 16.705208, -0.449959, 0, 16.601563, -0.425000, 0, 16.707293, -0.415104, 0, 16.599203, -0.395597, 0, 16.601563, -0.425000, 0, 16.707293, -0.415104, 0, 16.601563, -0.425000, 0, 16.599203, -0.395597, 0, 16.707293, -0.415104, 0, 16.703222, -0.370182, 0, 16.599203, -0.395597, 0, 16.707293, -0.415104, 0, 16.599203, -0.395597, 0, 16.703222, -0.370182, 0, 16.707293, -0.415104, 0, 16.703222, -0.370182, 0, 16.592123, -0.370150, 0, 16.599203, -0.395597, 0, 16.592123, -0.370150, 0, 16.703222, -0.370182, 0, 16.599203, -0.395597, 0, 16.691015, -0.330729, 0, 16.592123, -0.370150, 0, 16.703222, -0.370182, 0, 16.592123, -0.370150, 0, 16.691015, -0.330729, 0, 16.703222, -0.370182, 0, 16.670671, -0.296745, 0, 16.592123, -0.370150, 0, 16.691015, -0.330729, 0, 16.592123, -0.370150, 0, 16.670671, -0.296745, 0, 16.691015, -0.330729, 0, 16.580322, -0.348657, 0, 16.592123, -0.370150, 0, 16.670671, -0.296745, 0, 16.592123, -0.370150, 0, 16.580322, -0.348657, 0, 16.670671, -0.296745, 0, 16.642187, -0.268229, 0, 16.580322, -0.348657, 0, 16.670671, -0.296745, 0, 16.580322, -0.348657, 0, 16.642187, -0.268229, 0, 16.670671, -0.296745, 0, 16.563803, -0.331120, 0, 16.580322, -0.348657, 0, 16.642187, -0.268229, 0, 16.580322, -0.348657, 0, 16.563803, -0.331120, 0, 16.642187, -0.268229, 0, 16.606184, -0.245671, 0, 16.563803, -0.331120, 0, 16.642187, -0.268229, 0, 16.563803, -0.331120, 0, 16.606184, -0.245671, 0, 16.642187, -0.268229, 0, 16.542463, -0.317505, 0, 16.563803, -0.331120, 0, 16.606184, -0.245671, 0, 16.563803, -0.331120, 0, 16.542463, -0.317505, 0, 16.606184, -0.245671, 0, 16.563282, -0.229557, 0, 16.542463, -0.317505, 0, 16.606184, -0.245671, 0, 16.542463, -0.317505, 0, 16.563282, -0.229557, 0, 16.606184, -0.245671, 0, 16.516211, -0.307780, 0, 16.542463, -0.317505, 0, 16.563282, -0.229557, 0, 16.542463, -0.317505, 0, 16.516211, -0.307780, 0, 16.563282, -0.229557, 0, 16.513477, -0.219889, 0, 16.516211, -0.307780, 0, 16.563282, -0.229557, 0, 16.516211, -0.307780, 0, 16.513477, -0.219889, 0, 16.563282, -0.229557, 0, 16.513477, -0.219889, 0, 16.485043, -0.301945, 0, 16.516211, -0.307780, 0, 16.485043, -0.301945, 0, 16.513477, -0.219889, 0, 16.516211, -0.307780, 0, 16.456770, -0.216667, 0, 16.485043, -0.301945, 0, 16.513477, -0.219889, 0, 16.485043, -0.301945, 0, 16.456770, -0.216667, 0, 16.513477, -0.219889, 0, 16.456770, -0.216667, 0, 16.448957, -0.300000, 0, 16.485043, -0.301945, 0, 16.448957, -0.300000, 0, 16.456770, -0.216667, 0, 16.485043, -0.301945, 0, 16.293230, -0.300000, 0, 16.448957, -0.300000, 0, 16.456770, -0.216667, 0, 16.448957, -0.300000, 0, 16.293230, -0.300000, 0, 16.456770, -0.216667, 0, 16.576042, -0.586458, 0, 16.501677, -0.635490, 0, 16.535872, -0.641960, 0, 16.501677, -0.635490, 0, 16.576042, -0.586458, 0, 16.535872, -0.641960, 0, 16.535872, -0.641960, 0, 16.564568, -0.652743, 0, 16.576042, -0.586458, 0, 16.564568, -0.652743, 0, 16.535872, -0.641960, 0, 16.576042, -0.586458, 0, 16.576042, -0.586458, 0, 16.564568, -0.652743, 0, 16.612679, -0.596899, 0, 16.564568, -0.652743, 0, 16.576042, -0.586458, 0, 16.612679, -0.596899, 0, 16.564568, -0.652743, 0, 16.587761, -0.667839, 0, 16.612679, -0.596899, 0, 16.587761, -0.667839, 0, 16.564568, -0.652743, 0, 16.612679, -0.596899, 0, 16.612679, -0.596899, 0, 16.587761, -0.667839, 0, 16.644987, -0.611816, 0, 16.587761, -0.667839, 0, 16.612679, -0.596899, 0, 16.644987, -0.611816, 0, 16.587761, -0.667839, 0, 16.605648, -0.687459, 0, 16.644987, -0.611816, 0, 16.605648, -0.687459, 0, 16.587761, -0.667839, 0, 16.644987, -0.611816, 0, 16.644987, -0.611816, 0, 16.605648, -0.687459, 0, 16.672966, -0.631209, 0, 16.605648, -0.687459, 0, 16.644987, -0.611816, 0, 16.672966, -0.631209, 0, 16.672966, -0.631209, 0, 16.605648, -0.687459, 0, 16.696615, -0.655078, 0, 16.605648, -0.687459, 0, 16.672966, -0.631209, 0, 16.696615, -0.655078, 0, 16.605648, -0.687459, 0, 16.618425, -0.711816, 0, 16.696615, -0.655078, 0, 16.618425, -0.711816, 0, 16.605648, -0.687459, 0, 16.696615, -0.655078, 0, 16.696615, -0.655078, 0, 16.618425, -0.711816, 0, 16.715414, -0.682788, 0, 16.618425, -0.711816, 0, 16.696615, -0.655078, 0, 16.715414, -0.682788, 0, 16.715414, -0.682788, 0, 16.618425, -0.711816, 0, 16.626091, -0.740910, 0, 16.618425, -0.711816, 0, 16.715414, -0.682788, 0, 16.626091, -0.740910, 0, 16.728842, -0.713704, 0, 16.715414, -0.682788, 0, 16.626091, -0.740910, 0, 16.715414, -0.682788, 0, 16.728842, -0.713704, 0, 16.626091, -0.740910, 0, 16.728842, -0.713704, 0, 16.626091, -0.740910, 0, 16.736898, -0.747827, 0, 16.626091, -0.740910, 0, 16.728842, -0.713704, 0, 16.736898, -0.747827, 0, 16.626091, -0.740910, 0, 16.628645, -0.774740, 0, 16.736898, -0.747827, 0, 16.628645, -0.774740, 0, 16.626091, -0.740910, 0, 16.736898, -0.747827, 0, 16.736898, -0.747827, 0, 16.628645, -0.774740, 0, 16.739584, -0.785156, 0, 16.628645, -0.774740, 0, 16.736898, -0.747827, 0, 16.739584, -0.785156, 0, 16.628645, -0.774740, 0, 16.626091, -0.808374, 0, 16.739584, -0.785156, 0, 16.626091, -0.808374, 0, 16.628645, -0.774740, 0, 16.739584, -0.785156, 0, 16.626091, -0.808374, 0, 16.735157, -0.833659, 0, 16.739584, -0.785156, 0, 16.735157, -0.833659, 0, 16.626091, -0.808374, 0, 16.739584, -0.785156, 0, 16.626091, -0.808374, 0, 16.618425, -0.837402, 0, 16.735157, -0.833659, 0, 16.618425, -0.837402, 0, 16.626091, -0.808374, 0, 16.735157, -0.833659, 0, 16.618425, -0.837402, 0, 16.721876, -0.876302, 0, 16.735157, -0.833659, 0, 16.721876, -0.876302, 0, 16.618425, -0.837402, 0, 16.735157, -0.833659, 0, 16.618425, -0.837402, 0, 16.699739, -0.913086, 0, 16.721876, -0.876302, 0, 16.699739, -0.913086, 0, 16.618425, -0.837402, 0, 16.721876, -0.876302, 0, 16.618425, -0.837402, 0, 16.605648, -0.861825, 0, 16.699739, -0.913086, 0, 16.605648, -0.861825, 0, 16.618425, -0.837402, 0, 16.699739, -0.913086, 0, 16.605648, -0.861825, 0, 16.668751, -0.944010, 0, 16.699739, -0.913086, 0, 16.668751, -0.944010, 0, 16.605648, -0.861825, 0, 16.699739, -0.913086, 0, 16.605648, -0.861825, 0, 16.587761, -0.881641, 0, 16.668751, -0.944010, 0, 16.587761, -0.881641, 0, 16.605648, -0.861825, 0, 16.668751, -0.944010, 0, 16.587761, -0.881641, 0, 16.629591, -0.968506, 0, 16.668751, -0.944010, 0, 16.629591, -0.968506, 0, 16.587761, -0.881641, 0, 16.668751, -0.944010, 0, 16.587761, -0.881641, 0, 16.564568, -0.896964, 0, 16.629591, -0.968506, 0, 16.564568, -0.896964, 0, 16.587761, -0.881641, 0, 16.629591, -0.968506, 0, 16.564568, -0.896964, 0, 16.582943, -0.986003, 0, 16.629591, -0.968506, 0, 16.582943, -0.986003, 0, 16.564568, -0.896964, 0, 16.629591, -0.968506, 0, 16.564568, -0.896964, 0, 16.535872, -0.907910, 0, 16.582943, -0.986003, 0, 16.535872, -0.907910, 0, 16.564568, -0.896964, 0, 16.582943, -0.986003, 0, 16.535872, -0.907910, 0, 16.528809, -0.996501, 0, 16.582943, -0.986003, 0, 16.528809, -0.996501, 0, 16.535872, -0.907910, 0, 16.582943, -0.986003, 0, 16.501677, -0.914478, 0, 16.528809, -0.996501, 0, 16.535872, -0.907910, 0, 16.528809, -0.996501, 0, 16.501677, -0.914478, 0, 16.535872, -0.907910, 0, 16.528809, -0.996501, 0, 16.501677, -0.914478, 0, 16.467188, -1, 0, 16.501677, -0.914478, 0, 16.528809, -0.996501, 0, 16.467188, -1, 0, 16.501677, -0.914478, 0, 16.461981, -0.916667, 0, 16.467188, -1, 0, 16.461981, -0.916667, 0, 16.501677, -0.914478, 0, 16.467188, -1, 0, 16.461981, -0.916667, 0, 16.293230, -0.916667, 0, 16.467188, -1, 0, 16.293230, -0.916667, 0, 16.461981, -0.916667, 0, 16.467188, -1, 0, 16.293230, -0.916667, 0, 16.188021, -1, 0, 16.467188, -1, 0, 16.188021, -1, 0, 16.293230, -0.916667, 0, 16.467188, -1, 0, 16.188021, -1, 0, 16.293230, -0.916667, 0, 16.293230, -0.633333, 0, 16.293230, -0.916667, 0, 16.188021, -1, 0, 16.293230, -0.633333, 0, 16.188021, -0.216667, 0, 16.188021, -1, 0, 16.293230, -0.633333, 0, 16.188021, -1, 0, 16.188021, -0.216667, 0, 16.293230, -0.633333, 0, 16.921354, -0.216667, 0, 17.026564, -0.216667, 0, 16.921354, -0.216667, 0, 16.921354, -0.216667, 0, 17.026564, -0.216667, 0, 17.026564, -0.216667, 0, 17.026564, -0.216667, 0, 17.026564, -1, 0, 17.026564, -0.216667, 0, 17.026564, -0.216667, 0, 17.026564, -1, 0, 17.026564, -1, 0, 17.026564, -1, 0, 16.921354, -1, 0, 17.026564, -1, 0, 17.026564, -1, 0, 16.921354, -1, 0, 16.921354, -1, 0, 16.921354, -0.216667, 0, 17.026564, -1, 0, 17.026564, -0.216667, 0, 17.026564, -1, 0, 16.921354, -0.216667, 0, 17.026564, -0.216667, 0, 16.921354, -0.216667, 0, 16.921354, -1, 0, 17.026564, -1, 0, 16.921354, -1, 0, 16.921354, -0.216667, 0, 17.026564, -1, 0, 17.238020, -0.216667, 0, 17.343229, -0.216667, 0, 17.238020, -0.216667, 0, 17.238020, -0.216667, 0, 17.343229, -0.216667, 0, 17.343229, -0.216667, 0, 17.343229, -0.216667, 0, 17.343229, -0.916667, 0, 17.343229, -0.216667, 0, 17.343229, -0.216667, 0, 17.343229, -0.916667, 0, 17.343229, -0.916667, 0, 17.343229, -0.916667, 0, 17.721876, -0.916667, 0, 17.343229, -0.916667, 0, 17.343229, -0.916667, 0, 17.721876, -0.916667, 0, 17.721876, -0.916667, 0, 17.721876, -0.916667, 0, 17.721876, -1, 0, 17.721876, -0.916667, 0, 17.721876, -0.916667, 0, 17.721876, -1, 0, 17.721876, -1, 0, 17.721876, -1, 0, 17.238020, -1, 0, 17.721876, -1, 0, 17.721876, -1, 0, 17.238020, -1, 0, 17.238020, -1, 0, 17.238020, -0.216667, 0, 17.343229, -0.916667, 0, 17.343229, -0.216667, 0, 17.343229, -0.916667, 0, 17.238020, -0.216667, 0, 17.343229, -0.216667, 0, 17.238020, -0.216667, 0, 17.238020, -1, 0, 17.343229, -0.916667, 0, 17.238020, -1, 0, 17.238020, -0.216667, 0, 17.343229, -0.916667, 0, 17.343229, -0.916667, 0, 17.238020, -1, 0, 17.721876, -1, 0, 17.238020, -1, 0, 17.343229, -0.916667, 0, 17.721876, -1, 0, 17.343229, -0.916667, 0, 17.721876, -1, 0, 17.721876, -0.916667, 0, 17.721876, -1, 0, 17.343229, -0.916667, 0, 17.721876, -0.916667, 0, 17.838020, -0.216667, 0, 17.943230, -0.216667, 0, 17.838020, -0.216667, 0, 17.838020, -0.216667, 0, 17.943230, -0.216667, 0, 17.943230, -0.216667, 0, 17.943230, -0.216667, 0, 17.943230, -0.916667, 0, 17.943230, -0.216667, 0, 17.943230, -0.216667, 0, 17.943230, -0.916667, 0, 17.943230, -0.916667, 0, 17.943230, -0.916667, 0, 18.321875, -0.916667, 0, 17.943230, -0.916667, 0, 17.943230, -0.916667, 0, 18.321875, -0.916667, 0, 18.321875, -0.916667, 0, 18.321875, -0.916667, 0, 18.321875, -1, 0, 18.321875, -0.916667, 0, 18.321875, -0.916667, 0, 18.321875, -1, 0, 18.321875, -1, 0, 18.321875, -1, 0, 17.838020, -1, 0, 18.321875, -1, 0, 18.321875, -1, 0, 17.838020, -1, 0, 17.838020, -1, 0, 17.838020, -0.216667, 0, 17.943230, -0.916667, 0, 17.943230, -0.216667, 0, 17.943230, -0.916667, 0, 17.838020, -0.216667, 0, 17.943230, -0.216667, 0, 17.838020, -0.216667, 0, 17.838020, -1, 0, 17.943230, -0.916667, 0, 17.838020, -1, 0, 17.838020, -0.216667, 0, 17.943230, -0.916667, 0, 17.943230, -0.916667, 0, 17.838020, -1, 0, 18.321875, -1, 0, 17.838020, -1, 0, 17.943230, -0.916667, 0, 18.321875, -1, 0, 17.943230, -0.916667, 0, 18.321875, -1, 0, 18.321875, -0.916667, 0, 18.321875, -1, 0, 17.943230, -0.916667, 0, 18.321875, -0.916667, 0, 18.766146, -0.571615, 0, 18.801840, -0.582170, 0, 18.766146, -0.571615, 0, 18.766146, -0.571615, 0, 18.801840, -0.582170, 0, 18.801840, -0.582170, 0, 18.801840, -0.582170, 0, 18.833399, -0.597168, 0, 18.801840, -0.582170, 0, 18.801840, -0.582170, 0, 18.833399, -0.597168, 0, 18.833399, -0.597168, 0, 18.833399, -0.597168, 0, 18.860823, -0.616610, 0, 18.833399, -0.597168, 0, 18.833399, -0.597168, 0, 18.860823, -0.616610, 0, 18.860823, -0.616610, 0, 18.860823, -0.616610, 0, 18.884115, -0.640495, 0, 18.860823, -0.616610, 0, 18.860823, -0.616610, 0, 18.884115, -0.640495, 0, 18.884115, -0.640495, 0, 18.884115, -0.640495, 0, 18.902685, -0.668140, 0, 18.884115, -0.640495, 0, 18.884115, -0.640495, 0, 18.902685, -0.668140, 0, 18.902685, -0.668140, 0, 18.902685, -0.668140, 0, 18.915951, -0.698861, 0, 18.902685, -0.668140, 0, 18.902685, -0.668140, 0, 18.915951, -0.698861, 0, 18.915951, -0.698861, 0, 18.915951, -0.698861, 0, 18.923910, -0.732658, 0, 18.915951, -0.698861, 0, 18.915951, -0.698861, 0, 18.923910, -0.732658, 0, 18.923910, -0.732658, 0, 18.923910, -0.732658, 0, 18.926563, -0.769531, 0, 18.923910, -0.732658, 0, 18.923910, -0.732658, 0, 18.926563, -0.769531, 0, 18.926563, -0.769531, 0, 18.926563, -0.769531, 0, 18.921614, -0.825179, 0, 18.926563, -0.769531, 0, 18.926563, -0.769531, 0, 18.921614, -0.825179, 0, 18.921614, -0.825179, 0, 18.921614, -0.825179, 0, 18.906771, -0.874154, 0, 18.921614, -0.825179, 0, 18.921614, -0.825179, 0, 18.906771, -0.874154, 0, 18.906771, -0.874154, 0, 18.906771, -0.874154, 0, 18.882032, -0.916455, 0, 18.906771, -0.874154, 0, 18.906771, -0.874154, 0, 18.882032, -0.916455, 0, 18.882032, -0.916455, 0, 18.882032, -0.916455, 0, 18.847395, -0.952083, 0, 18.882032, -0.916455, 0, 18.882032, -0.916455, 0, 18.847395, -0.952083, 0, 18.847395, -0.952083, 0, 18.847395, -0.952083, 0, 18.803646, -0.980339, 0, 18.847395, -0.952083, 0, 18.847395, -0.952083, 0, 18.803646, -0.980339, 0, 18.803646, -0.980339, 0, 18.803646, -0.980339, 0, 18.751562, -1.000521, 0, 18.803646, -0.980339, 0, 18.803646, -0.980339, 0, 18.751562, -1.000521, 0, 18.751562, -1.000521, 0, 18.751562, -1.000521, 0, 18.691145, -1.012630, 0, 18.751562, -1.000521, 0, 18.751562, -1.000521, 0, 18.691145, -1.012630, 0, 18.691145, -1.012630, 0, 18.691145, -1.012630, 0, 18.622396, -1.016667, 0, 18.691145, -1.012630, 0, 18.691145, -1.012630, 0, 18.622396, -1.016667, 0, 18.622396, -1.016667, 0, 18.622396, -1.016667, 0, 18.597738, -1.016146, 0, 18.622396, -1.016667, 0, 18.622396, -1.016667, 0, 18.597738, -1.016146, 0, 18.597738, -1.016146, 0, 18.597738, -1.016146, 0, 18.572721, -1.014583, 0, 18.597738, -1.016146, 0, 18.597738, -1.016146, 0, 18.572721, -1.014583, 0, 18.572721, -1.014583, 0, 18.572721, -1.014583, 0, 18.547346, -1.011979, 0, 18.572721, -1.014583, 0, 18.572721, -1.014583, 0, 18.547346, -1.011979, 0, 18.547346, -1.011979, 0, 18.547346, -1.011979, 0, 18.521614, -1.008333, 0, 18.547346, -1.011979, 0, 18.547346, -1.011979, 0, 18.521614, -1.008333, 0, 18.521614, -1.008333, 0, 18.521614, -1.008333, 0, 18.495491, -1.003646, 0, 18.521614, -1.008333, 0, 18.521614, -1.008333, 0, 18.495491, -1.003646, 0, 18.495491, -1.003646, 0, 18.495491, -1.003646, 0, 18.468946, -0.997917, 0, 18.495491, -1.003646, 0, 18.495491, -1.003646, 0, 18.468946, -0.997917, 0, 18.468946, -0.997917, 0, 18.468946, -0.997917, 0, 18.441977, -0.991146, 0, 18.468946, -0.997917, 0, 18.468946, -0.997917, 0, 18.441977, -0.991146, 0, 18.441977, -0.991146, 0, 18.441977, -0.991146, 0, 18.414583, -0.983333, 0, 18.441977, -0.991146, 0, 18.441977, -0.991146, 0, 18.414583, -0.983333, 0, 18.414583, -0.983333, 0, 18.414583, -0.983333, 0, 18.414583, -0.883333, 0, 18.414583, -0.983333, 0, 18.414583, -0.983333, 0, 18.414583, -0.883333, 0, 18.414583, -0.883333, 0, 18.414583, -0.883333, 0, 18.436979, -0.894946, 0, 18.414583, -0.883333, 0, 18.414583, -0.883333, 0, 18.436979, -0.894946, 0, 18.436979, -0.894946, 0, 18.436979, -0.894946, 0, 18.460417, -0.905046, 0, 18.436979, -0.894946, 0, 18.436979, -0.894946, 0, 18.460417, -0.905046, 0, 18.460417, -0.905046, 0, 18.460417, -0.905046, 0, 18.484896, -0.913631, 0, 18.460417, -0.905046, 0, 18.460417, -0.905046, 0, 18.484896, -0.913631, 0, 18.484896, -0.913631, 0, 18.484896, -0.913631, 0, 18.510418, -0.920703, 0, 18.484896, -0.913631, 0, 18.484896, -0.913631, 0, 18.510418, -0.920703, 0, 18.510418, -0.920703, 0, 18.510418, -0.920703, 0, 18.536751, -0.926229, 0, 18.510418, -0.920703, 0, 18.510418, -0.920703, 0, 18.536751, -0.926229, 0, 18.536751, -0.926229, 0, 18.536751, -0.926229, 0, 18.563673, -0.930176, 0, 18.536751, -0.926229, 0, 18.536751, -0.926229, 0, 18.563673, -0.930176, 0, 18.563673, -0.930176, 0, 18.563673, -0.930176, 0, 18.591179, -0.932544, 0, 18.563673, -0.930176, 0, 18.563673, -0.930176, 0, 18.591179, -0.932544, 0, 18.591179, -0.932544, 0, 18.591179, -0.932544, 0, 18.619270, -0.933333, 0, 18.591179, -0.932544, 0, 18.591179, -0.932544, 0, 18.619270, -0.933333, 0, 18.619270, -0.933333, 0, 18.619270, -0.933333, 0, 18.665804, -0.930745, 0, 18.619270, -0.933333, 0, 18.619270, -0.933333, 0, 18.665804, -0.930745, 0, 18.665804, -0.930745, 0, 18.665804, -0.930745, 0, 18.706446, -0.922982, 0, 18.665804, -0.930745, 0, 18.665804, -0.930745, 0, 18.706446, -0.922982, 0, 18.706446, -0.922982, 0, 18.706446, -0.922982, 0, 18.741194, -0.910042, 0, 18.706446, -0.922982, 0, 18.706446, -0.922982, 0, 18.741194, -0.910042, 0, 18.741194, -0.910042, 0, 18.741194, -0.910042, 0, 18.770052, -0.891927, 0, 18.741194, -0.910042, 0, 18.741194, -0.910042, 0, 18.770052, -0.891927, 0, 18.770052, -0.891927, 0, 18.770052, -0.891927, 0, 18.792725, -0.868880, 0, 18.770052, -0.891927, 0, 18.770052, -0.891927, 0, 18.792725, -0.868880, 0, 18.792725, -0.868880, 0, 18.792725, -0.868880, 0, 18.808920, -0.841146, 0, 18.792725, -0.868880, 0, 18.792725, -0.868880, 0, 18.808920, -0.841146, 0, 18.808920, -0.841146, 0, 18.808920, -0.841146, 0, 18.818636, -0.808724, 0, 18.808920, -0.841146, 0, 18.808920, -0.841146, 0, 18.818636, -0.808724, 0, 18.818636, -0.808724, 0, 18.818636, -0.808724, 0, 18.821875, -0.771615, 0, 18.818636, -0.808724, 0, 18.818636, -0.808724, 0, 18.821875, -0.771615, 0, 18.821875, -0.771615, 0, 18.821875, -0.771615, 0, 18.818865, -0.737256, 0, 18.821875, -0.771615, 0, 18.821875, -0.771615, 0, 18.818865, -0.737256, 0, 18.818865, -0.737256, 0, 18.818865, -0.737256, 0, 18.809832, -0.706836, 0, 18.818865, -0.737256, 0, 18.818865, -0.737256, 0, 18.809832, -0.706836, 0, 18.809832, -0.706836, 0, 18.809832, -0.706836, 0, 18.794775, -0.680355, 0, 18.809832, -0.706836, 0, 18.809832, -0.706836, 0, 18.794775, -0.680355, 0, 18.794775, -0.680355, 0, 18.794775, -0.680355, 0, 18.773699, -0.657812, 0, 18.794775, -0.680355, 0, 18.794775, -0.680355, 0, 18.773699, -0.657812, 0, 18.773699, -0.657812, 0, 18.773699, -0.657812, 0, 18.747250, -0.639811, 0, 18.773699, -0.657812, 0, 18.773699, -0.657812, 0, 18.747250, -0.639811, 0, 18.747250, -0.639811, 0, 18.747250, -0.639811, 0, 18.716082, -0.626953, 0, 18.747250, -0.639811, 0, 18.747250, -0.639811, 0, 18.716082, -0.626953, 0, 18.716082, -0.626953, 0, 18.716082, -0.626953, 0, 18.680193, -0.619238, 0, 18.716082, -0.626953, 0, 18.716082, -0.626953, 0, 18.680193, -0.619238, 0, 18.680193, -0.619238, 0, 18.680193, -0.619238, 0, 18.639584, -0.616667, 0, 18.680193, -0.619238, 0, 18.680193, -0.619238, 0, 18.639584, -0.616667, 0, 18.639584, -0.616667, 0, 18.639584, -0.616667, 0, 18.548958, -0.616667, 0, 18.639584, -0.616667, 0, 18.639584, -0.616667, 0, 18.548958, -0.616667, 0, 18.548958, -0.616667, 0, 18.548958, -0.616667, 0, 18.548958, -0.533333, 0, 18.548958, -0.616667, 0, 18.548958, -0.616667, 0, 18.548958, -0.533333, 0, 18.548958, -0.533333, 0, 18.548958, -0.533333, 0, 18.643751, -0.533333, 0, 18.548958, -0.533333, 0, 18.548958, -0.533333, 0, 18.643751, -0.533333, 0, 18.643751, -0.533333, 0, 18.643751, -0.533333, 0, 18.680273, -0.531348, 0, 18.643751, -0.533333, 0, 18.643751, -0.533333, 0, 18.680273, -0.531348, 0, 18.680273, -0.531348, 0, 18.680273, -0.531348, 0, 18.712240, -0.525391, 0, 18.680273, -0.531348, 0, 18.680273, -0.531348, 0, 18.712240, -0.525391, 0, 18.712240, -0.525391, 0, 18.712240, -0.525391, 0, 18.739649, -0.515462, 0, 18.712240, -0.525391, 0, 18.712240, -0.525391, 0, 18.739649, -0.515462, 0, 18.739649, -0.515462, 0, 18.739649, -0.515462, 0, 18.762501, -0.501562, 0, 18.739649, -0.515462, 0, 18.739649, -0.515462, 0, 18.762501, -0.501562, 0, 18.762501, -0.501562, 0, 18.762501, -0.501562, 0, 18.780500, -0.483936, 0, 18.762501, -0.501562, 0, 18.762501, -0.501562, 0, 18.780500, -0.483936, 0, 18.780500, -0.483936, 0, 18.780500, -0.483936, 0, 18.793360, -0.462826, 0, 18.780500, -0.483936, 0, 18.780500, -0.483936, 0, 18.793360, -0.462826, 0, 18.793360, -0.462826, 0, 18.793360, -0.462826, 0, 18.801075, -0.438232, 0, 18.793360, -0.462826, 0, 18.793360, -0.462826, 0, 18.801075, -0.438232, 0, 18.801075, -0.438232, 0, 18.801075, -0.438232, 0, 18.803646, -0.410156, 0, 18.801075, -0.438232, 0, 18.801075, -0.438232, 0, 18.803646, -0.410156, 0, 18.803646, -0.410156, 0, 18.803646, -0.410156, 0, 18.800993, -0.381331, 0, 18.803646, -0.410156, 0, 18.803646, -0.410156, 0, 18.800993, -0.381331, 0, 18.800993, -0.381331, 0, 18.800993, -0.381331, 0, 18.793036, -0.356055, 0, 18.800993, -0.381331, 0, 18.800993, -0.381331, 0, 18.793036, -0.356055, 0, 18.793036, -0.356055, 0, 18.793036, -0.356055, 0, 18.779768, -0.334326, 0, 18.793036, -0.356055, 0, 18.793036, -0.356055, 0, 18.779768, -0.334326, 0, 18.779768, -0.334326, 0, 18.779768, -0.334326, 0, 18.761198, -0.316146, 0, 18.779768, -0.334326, 0, 18.779768, -0.334326, 0, 18.761198, -0.316146, 0, 18.761198, -0.316146, 0, 18.761198, -0.316146, 0, 18.737679, -0.301790, 0, 18.761198, -0.316146, 0, 18.761198, -0.316146, 0, 18.737679, -0.301790, 0, 18.737679, -0.301790, 0, 18.737679, -0.301790, 0, 18.709570, -0.291536, 0, 18.737679, -0.301790, 0, 18.737679, -0.301790, 0, 18.709570, -0.291536, 0, 18.709570, -0.291536, 0, 18.709570, -0.291536, 0, 18.676872, -0.285384, 0, 18.709570, -0.291536, 0, 18.709570, -0.291536, 0, 18.676872, -0.285384, 0, 18.676872, -0.285384, 0, 18.676872, -0.285384, 0, 18.639584, -0.283333, 0, 18.676872, -0.285384, 0, 18.676872, -0.285384, 0, 18.639584, -0.283333, 0, 18.639584, -0.283333, 0, 18.639584, -0.283333, 0, 18.617579, -0.284090, 0, 18.639584, -0.283333, 0, 18.639584, -0.283333, 0, 18.617579, -0.284090, 0, 18.617579, -0.284090, 0, 18.617579, -0.284090, 0, 18.594793, -0.286361, 0, 18.617579, -0.284090, 0, 18.617579, -0.284090, 0, 18.594793, -0.286361, 0, 18.594793, -0.286361, 0, 18.594793, -0.286361, 0, 18.571224, -0.290145, 0, 18.594793, -0.286361, 0, 18.594793, -0.286361, 0, 18.571224, -0.290145, 0, 18.571224, -0.290145, 0, 18.571224, -0.290145, 0, 18.546875, -0.295443, 0, 18.571224, -0.290145, 0, 18.571224, -0.290145, 0, 18.546875, -0.295443, 0, 18.546875, -0.295443, 0, 18.546875, -0.295443, 0, 18.521517, -0.302352, 0, 18.546875, -0.295443, 0, 18.546875, -0.295443, 0, 18.521517, -0.302352, 0, 18.521517, -0.302352, 0, 18.521517, -0.302352, 0, 18.494923, -0.310970, 0, 18.521517, -0.302352, 0, 18.521517, -0.302352, 0, 18.494923, -0.310970, 0, 18.494923, -0.310970, 0, 18.494923, -0.310970, 0, 18.467091, -0.321297, 0, 18.494923, -0.310970, 0, 18.494923, -0.310970, 0, 18.467091, -0.321297, 0, 18.467091, -0.321297, 0, 18.467091, -0.321297, 0, 18.438021, -0.333333, 0, 18.467091, -0.321297, 0, 18.467091, -0.321297, 0, 18.438021, -0.333333, 0, 18.438021, -0.333333, 0, 18.438021, -0.333333, 0, 18.438021, -0.233333, 0, 18.438021, -0.333333, 0, 18.438021, -0.333333, 0, 18.438021, -0.233333, 0, 18.438021, -0.233333, 0, 18.438021, -0.233333, 0, 18.467497, -0.225521, 0, 18.438021, -0.233333, 0, 18.438021, -0.233333, 0, 18.467497, -0.225521, 0, 18.467497, -0.225521, 0, 18.467497, -0.225521, 0, 18.496029, -0.218750, 0, 18.467497, -0.225521, 0, 18.467497, -0.225521, 0, 18.496029, -0.218750, 0, 18.496029, -0.218750, 0, 18.496029, -0.218750, 0, 18.523617, -0.213021, 0, 18.496029, -0.218750, 0, 18.496029, -0.218750, 0, 18.523617, -0.213021, 0, 18.523617, -0.213021, 0, 18.523617, -0.213021, 0, 18.550261, -0.208333, 0, 18.523617, -0.213021, 0, 18.523617, -0.213021, 0, 18.550261, -0.208333, 0, 18.550261, -0.208333, 0, 18.550261, -0.208333, 0, 18.576057, -0.204687, 0, 18.550261, -0.208333, 0, 18.550261, -0.208333, 0, 18.576057, -0.204687, 0, 18.576057, -0.204687, 0, 18.576057, -0.204687, 0, 18.601107, -0.202083, 0, 18.576057, -0.204687, 0, 18.576057, -0.204687, 0, 18.601107, -0.202083, 0, 18.601107, -0.202083, 0, 18.601107, -0.202083, 0, 18.625408, -0.200521, 0, 18.601107, -0.202083, 0, 18.601107, -0.202083, 0, 18.625408, -0.200521, 0, 18.625408, -0.200521, 0, 18.625408, -0.200521, 0, 18.648958, -0.200000, 0, 18.625408, -0.200521, 0, 18.625408, -0.200521, 0, 18.648958, -0.200000, 0, 18.648958, -0.200000, 0, 18.648958, -0.200000, 0, 18.705730, -0.203394, 0, 18.648958, -0.200000, 0, 18.648958, -0.200000, 0, 18.705730, -0.203394, 0, 18.705730, -0.203394, 0, 18.705730, -0.203394, 0, 18.756250, -0.213574, 0, 18.705730, -0.203394, 0, 18.705730, -0.203394, 0, 18.756250, -0.213574, 0, 18.756250, -0.213574, 0, 18.756250, -0.213574, 0, 18.800520, -0.230542, 0, 18.756250, -0.213574, 0, 18.756250, -0.213574, 0, 18.800520, -0.230542, 0, 18.800520, -0.230542, 0, 18.800520, -0.230542, 0, 18.838543, -0.254297, 0, 18.800520, -0.230542, 0, 18.800520, -0.230542, 0, 18.838543, -0.254297, 0, 18.838543, -0.254297, 0, 18.838543, -0.254297, 0, 18.869076, -0.283830, 0, 18.838543, -0.254297, 0, 18.838543, -0.254297, 0, 18.869076, -0.283830, 0, 18.869076, -0.283830, 0, 18.869076, -0.283830, 0, 18.890886, -0.318132, 0, 18.869076, -0.283830, 0, 18.869076, -0.283830, 0, 18.890886, -0.318132, 0, 18.890886, -0.318132, 0, 18.890886, -0.318132, 0, 18.903973, -0.357202, 0, 18.890886, -0.318132, 0, 18.890886, -0.318132, 0, 18.903973, -0.357202, 0, 18.903973, -0.357202, 0, 18.903973, -0.357202, 0, 18.908333, -0.401042, 0, 18.903973, -0.357202, 0, 18.903973, -0.357202, 0, 18.908333, -0.401042, 0, 18.908333, -0.401042, 0, 18.908333, -0.401042, 0, 18.906023, -0.432072, 0, 18.908333, -0.401042, 0, 18.908333, -0.401042, 0, 18.906023, -0.432072, 0, 18.906023, -0.432072, 0, 18.906023, -0.432072, 0, 18.899090, -0.460579, 0, 18.906023, -0.432072, 0, 18.906023, -0.432072, 0, 18.899090, -0.460579, 0, 18.899090, -0.460579, 0, 18.899090, -0.460579, 0, 18.887533, -0.486564, 0, 18.899090, -0.460579, 0, 18.899090, -0.460579, 0, 18.887533, -0.486564, 0, 18.887533, -0.486564, 0, 18.887533, -0.486564, 0, 18.871355, -0.510026, 0, 18.887533, -0.486564, 0, 18.887533, -0.486564, 0, 18.871355, -0.510026, 0, 18.871355, -0.510026, 0, 18.871355, -0.510026, 0, 18.850912, -0.530526, 0, 18.871355, -0.510026, 0, 18.871355, -0.510026, 0, 18.850912, -0.530526, 0, 18.850912, -0.530526, 0, 18.850912, -0.530526, 0, 18.826563, -0.547624, 0, 18.850912, -0.530526, 0, 18.850912, -0.530526, 0, 18.826563, -0.547624, 0, 18.826563, -0.547624, 0, 18.826563, -0.547624, 0, 18.798307, -0.561320, 0, 18.826563, -0.547624, 0, 18.826563, -0.547624, 0, 18.798307, -0.561320, 0, 18.798307, -0.561320, 0, 18.414583, -0.883333, 0, 18.414583, -0.983333, 0, 18.436979, -0.894946, 0, 18.414583, -0.983333, 0, 18.414583, -0.883333, 0, 18.436979, -0.894946, 0, 18.436979, -0.894946, 0, 18.414583, -0.983333, 0, 18.460417, -0.905046, 0, 18.414583, -0.983333, 0, 18.436979, -0.894946, 0, 18.460417, -0.905046, 0, 18.414583, -0.983333, 0, 18.441977, -0.991146, 0, 18.460417, -0.905046, 0, 18.441977, -0.991146, 0, 18.414583, -0.983333, 0, 18.460417, -0.905046, 0, 18.460417, -0.905046, 0, 18.441977, -0.991146, 0, 18.484896, -0.913631, 0, 18.441977, -0.991146, 0, 18.460417, -0.905046, 0, 18.484896, -0.913631, 0, 18.441977, -0.991146, 0, 18.468946, -0.997917, 0, 18.484896, -0.913631, 0, 18.468946, -0.997917, 0, 18.441977, -0.991146, 0, 18.484896, -0.913631, 0, 18.484896, -0.913631, 0, 18.468946, -0.997917, 0, 18.510418, -0.920703, 0, 18.468946, -0.997917, 0, 18.484896, -0.913631, 0, 18.510418, -0.920703, 0, 18.468946, -0.997917, 0, 18.495491, -1.003646, 0, 18.510418, -0.920703, 0, 18.495491, -1.003646, 0, 18.468946, -0.997917, 0, 18.510418, -0.920703, 0, 18.510418, -0.920703, 0, 18.495491, -1.003646, 0, 18.536751, -0.926229, 0, 18.495491, -1.003646, 0, 18.510418, -0.920703, 0, 18.536751, -0.926229, 0, 18.495491, -1.003646, 0, 18.521614, -1.008333, 0, 18.536751, -0.926229, 0, 18.521614, -1.008333, 0, 18.495491, -1.003646, 0, 18.536751, -0.926229, 0, 18.536751, -0.926229, 0, 18.521614, -1.008333, 0, 18.547346, -1.011979, 0, 18.521614, -1.008333, 0, 18.536751, -0.926229, 0, 18.547346, -1.011979, 0, 18.536751, -0.926229, 0, 18.547346, -1.011979, 0, 18.563673, -0.930176, 0, 18.547346, -1.011979, 0, 18.536751, -0.926229, 0, 18.563673, -0.930176, 0, 18.563673, -0.930176, 0, 18.547346, -1.011979, 0, 18.572721, -1.014583, 0, 18.547346, -1.011979, 0, 18.563673, -0.930176, 0, 18.572721, -1.014583, 0, 18.563673, -0.930176, 0, 18.572721, -1.014583, 0, 18.591179, -0.932544, 0, 18.572721, -1.014583, 0, 18.563673, -0.930176, 0, 18.591179, -0.932544, 0, 18.591179, -0.932544, 0, 18.572721, -1.014583, 0, 18.597738, -1.016146, 0, 18.572721, -1.014583, 0, 18.591179, -0.932544, 0, 18.597738, -1.016146, 0, 18.591179, -0.932544, 0, 18.597738, -1.016146, 0, 18.619270, -0.933333, 0, 18.597738, -1.016146, 0, 18.591179, -0.932544, 0, 18.619270, -0.933333, 0, 18.619270, -0.933333, 0, 18.597738, -1.016146, 0, 18.622396, -1.016667, 0, 18.597738, -1.016146, 0, 18.619270, -0.933333, 0, 18.622396, -1.016667, 0, 18.665804, -0.930745, 0, 18.619270, -0.933333, 0, 18.622396, -1.016667, 0, 18.619270, -0.933333, 0, 18.665804, -0.930745, 0, 18.622396, -1.016667, 0, 18.691145, -1.012630, 0, 18.665804, -0.930745, 0, 18.622396, -1.016667, 0, 18.665804, -0.930745, 0, 18.691145, -1.012630, 0, 18.622396, -1.016667, 0, 18.665804, -0.930745, 0, 18.691145, -1.012630, 0, 18.706446, -0.922982, 0, 18.691145, -1.012630, 0, 18.665804, -0.930745, 0, 18.706446, -0.922982, 0, 18.706446, -0.922982, 0, 18.691145, -1.012630, 0, 18.751562, -1.000521, 0, 18.691145, -1.012630, 0, 18.706446, -0.922982, 0, 18.751562, -1.000521, 0, 18.741194, -0.910042, 0, 18.706446, -0.922982, 0, 18.751562, -1.000521, 0, 18.706446, -0.922982, 0, 18.741194, -0.910042, 0, 18.751562, -1.000521, 0, 18.741194, -0.910042, 0, 18.751562, -1.000521, 0, 18.803646, -0.980339, 0, 18.751562, -1.000521, 0, 18.741194, -0.910042, 0, 18.803646, -0.980339, 0, 18.770052, -0.891927, 0, 18.741194, -0.910042, 0, 18.803646, -0.980339, 0, 18.741194, -0.910042, 0, 18.770052, -0.891927, 0, 18.803646, -0.980339, 0, 18.770052, -0.891927, 0, 18.803646, -0.980339, 0, 18.847395, -0.952083, 0, 18.803646, -0.980339, 0, 18.770052, -0.891927, 0, 18.847395, -0.952083, 0, 18.792725, -0.868880, 0, 18.770052, -0.891927, 0, 18.847395, -0.952083, 0, 18.770052, -0.891927, 0, 18.792725, -0.868880, 0, 18.847395, -0.952083, 0, 18.792725, -0.868880, 0, 18.847395, -0.952083, 0, 18.882032, -0.916455, 0, 18.847395, -0.952083, 0, 18.792725, -0.868880, 0, 18.882032, -0.916455, 0, 18.808920, -0.841146, 0, 18.792725, -0.868880, 0, 18.882032, -0.916455, 0, 18.792725, -0.868880, 0, 18.808920, -0.841146, 0, 18.882032, -0.916455, 0, 18.808920, -0.841146, 0, 18.882032, -0.916455, 0, 18.906771, -0.874154, 0, 18.882032, -0.916455, 0, 18.808920, -0.841146, 0, 18.906771, -0.874154, 0, 18.818636, -0.808724, 0, 18.808920, -0.841146, 0, 18.906771, -0.874154, 0, 18.808920, -0.841146, 0, 18.818636, -0.808724, 0, 18.906771, -0.874154, 0, 18.921614, -0.825179, 0, 18.818636, -0.808724, 0, 18.906771, -0.874154, 0, 18.818636, -0.808724, 0, 18.921614, -0.825179, 0, 18.906771, -0.874154, 0, 18.821875, -0.771615, 0, 18.818636, -0.808724, 0, 18.921614, -0.825179, 0, 18.818636, -0.808724, 0, 18.821875, -0.771615, 0, 18.921614, -0.825179, 0, 18.821875, -0.771615, 0, 18.921614, -0.825179, 0, 18.926563, -0.769531, 0, 18.921614, -0.825179, 0, 18.821875, -0.771615, 0, 18.926563, -0.769531, 0, 18.923910, -0.732658, 0, 18.821875, -0.771615, 0, 18.926563, -0.769531, 0, 18.821875, -0.771615, 0, 18.923910, -0.732658, 0, 18.926563, -0.769531, 0, 18.818865, -0.737256, 0, 18.821875, -0.771615, 0, 18.923910, -0.732658, 0, 18.821875, -0.771615, 0, 18.818865, -0.737256, 0, 18.923910, -0.732658, 0, 18.915951, -0.698861, 0, 18.818865, -0.737256, 0, 18.923910, -0.732658, 0, 18.818865, -0.737256, 0, 18.915951, -0.698861, 0, 18.923910, -0.732658, 0, 18.809832, -0.706836, 0, 18.818865, -0.737256, 0, 18.915951, -0.698861, 0, 18.818865, -0.737256, 0, 18.809832, -0.706836, 0, 18.915951, -0.698861, 0, 18.902685, -0.668140, 0, 18.809832, -0.706836, 0, 18.915951, -0.698861, 0, 18.809832, -0.706836, 0, 18.902685, -0.668140, 0, 18.915951, -0.698861, 0, 18.884115, -0.640495, 0, 18.809832, -0.706836, 0, 18.902685, -0.668140, 0, 18.809832, -0.706836, 0, 18.884115, -0.640495, 0, 18.902685, -0.668140, 0, 18.794775, -0.680355, 0, 18.809832, -0.706836, 0, 18.884115, -0.640495, 0, 18.809832, -0.706836, 0, 18.794775, -0.680355, 0, 18.884115, -0.640495, 0, 18.860823, -0.616610, 0, 18.794775, -0.680355, 0, 18.884115, -0.640495, 0, 18.794775, -0.680355, 0, 18.860823, -0.616610, 0, 18.884115, -0.640495, 0, 18.860823, -0.616610, 0, 18.833399, -0.597168, 0, 18.794775, -0.680355, 0, 18.833399, -0.597168, 0, 18.860823, -0.616610, 0, 18.794775, -0.680355, 0, 18.833399, -0.597168, 0, 18.773699, -0.657812, 0, 18.794775, -0.680355, 0, 18.773699, -0.657812, 0, 18.833399, -0.597168, 0, 18.794775, -0.680355, 0, 18.801840, -0.582170, 0, 18.773699, -0.657812, 0, 18.833399, -0.597168, 0, 18.773699, -0.657812, 0, 18.801840, -0.582170, 0, 18.833399, -0.597168, 0, 18.747250, -0.639811, 0, 18.773699, -0.657812, 0, 18.801840, -0.582170, 0, 18.773699, -0.657812, 0, 18.747250, -0.639811, 0, 18.801840, -0.582170, 0, 18.766146, -0.571615, 0, 18.747250, -0.639811, 0, 18.801840, -0.582170, 0, 18.747250, -0.639811, 0, 18.766146, -0.571615, 0, 18.801840, -0.582170, 0, 18.716082, -0.626953, 0, 18.747250, -0.639811, 0, 18.766146, -0.571615, 0, 18.747250, -0.639811, 0, 18.716082, -0.626953, 0, 18.766146, -0.571615, 0, 18.766146, -0.571615, 0, 18.712240, -0.525391, 0, 18.716082, -0.626953, 0, 18.712240, -0.525391, 0, 18.766146, -0.571615, 0, 18.716082, -0.626953, 0, 18.739649, -0.515462, 0, 18.712240, -0.525391, 0, 18.766146, -0.571615, 0, 18.712240, -0.525391, 0, 18.739649, -0.515462, 0, 18.766146, -0.571615, 0, 18.762501, -0.501562, 0, 18.739649, -0.515462, 0, 18.766146, -0.571615, 0, 18.739649, -0.515462, 0, 18.762501, -0.501562, 0, 18.766146, -0.571615, 0, 18.762501, -0.501562, 0, 18.766146, -0.571615, 0, 18.798307, -0.561320, 0, 18.766146, -0.571615, 0, 18.762501, -0.501562, 0, 18.798307, -0.561320, 0, 18.826563, -0.547624, 0, 18.762501, -0.501562, 0, 18.798307, -0.561320, 0, 18.762501, -0.501562, 0, 18.826563, -0.547624, 0, 18.798307, -0.561320, 0, 18.826563, -0.547624, 0, 18.780500, -0.483936, 0, 18.762501, -0.501562, 0, 18.780500, -0.483936, 0, 18.826563, -0.547624, 0, 18.762501, -0.501562, 0, 18.780500, -0.483936, 0, 18.826563, -0.547624, 0, 18.850912, -0.530526, 0, 18.826563, -0.547624, 0, 18.780500, -0.483936, 0, 18.850912, -0.530526, 0, 18.793360, -0.462826, 0, 18.780500, -0.483936, 0, 18.850912, -0.530526, 0, 18.780500, -0.483936, 0, 18.793360, -0.462826, 0, 18.850912, -0.530526, 0, 18.793360, -0.462826, 0, 18.850912, -0.530526, 0, 18.871355, -0.510026, 0, 18.850912, -0.530526, 0, 18.793360, -0.462826, 0, 18.871355, -0.510026, 0, 18.887533, -0.486564, 0, 18.793360, -0.462826, 0, 18.871355, -0.510026, 0, 18.793360, -0.462826, 0, 18.887533, -0.486564, 0, 18.871355, -0.510026, 0, 18.887533, -0.486564, 0, 18.801075, -0.438232, 0, 18.793360, -0.462826, 0, 18.801075, -0.438232, 0, 18.887533, -0.486564, 0, 18.793360, -0.462826, 0, 18.801075, -0.438232, 0, 18.887533, -0.486564, 0, 18.899090, -0.460579, 0, 18.887533, -0.486564, 0, 18.801075, -0.438232, 0, 18.899090, -0.460579, 0, 18.906023, -0.432072, 0, 18.801075, -0.438232, 0, 18.899090, -0.460579, 0, 18.801075, -0.438232, 0, 18.906023, -0.432072, 0, 18.899090, -0.460579, 0, 18.906023, -0.432072, 0, 18.803646, -0.410156, 0, 18.801075, -0.438232, 0, 18.803646, -0.410156, 0, 18.906023, -0.432072, 0, 18.801075, -0.438232, 0, 18.803646, -0.410156, 0, 18.906023, -0.432072, 0, 18.908333, -0.401042, 0, 18.906023, -0.432072, 0, 18.803646, -0.410156, 0, 18.908333, -0.401042, 0, 18.908333, -0.401042, 0, 18.800993, -0.381331, 0, 18.803646, -0.410156, 0, 18.800993, -0.381331, 0, 18.908333, -0.401042, 0, 18.803646, -0.410156, 0, 18.903973, -0.357202, 0, 18.800993, -0.381331, 0, 18.908333, -0.401042, 0, 18.800993, -0.381331, 0, 18.903973, -0.357202, 0, 18.908333, -0.401042, 0, 18.890886, -0.318132, 0, 18.800993, -0.381331, 0, 18.903973, -0.357202, 0, 18.800993, -0.381331, 0, 18.890886, -0.318132, 0, 18.903973, -0.357202, 0, 18.793036, -0.356055, 0, 18.800993, -0.381331, 0, 18.890886, -0.318132, 0, 18.800993, -0.381331, 0, 18.793036, -0.356055, 0, 18.890886, -0.318132, 0, 18.869076, -0.283830, 0, 18.793036, -0.356055, 0, 18.890886, -0.318132, 0, 18.793036, -0.356055, 0, 18.869076, -0.283830, 0, 18.890886, -0.318132, 0, 18.779768, -0.334326, 0, 18.793036, -0.356055, 0, 18.869076, -0.283830, 0, 18.793036, -0.356055, 0, 18.779768, -0.334326, 0, 18.869076, -0.283830, 0, 18.838543, -0.254297, 0, 18.779768, -0.334326, 0, 18.869076, -0.283830, 0, 18.779768, -0.334326, 0, 18.838543, -0.254297, 0, 18.869076, -0.283830, 0, 18.761198, -0.316146, 0, 18.779768, -0.334326, 0, 18.838543, -0.254297, 0, 18.779768, -0.334326, 0, 18.761198, -0.316146, 0, 18.838543, -0.254297, 0, 18.800520, -0.230542, 0, 18.761198, -0.316146, 0, 18.838543, -0.254297, 0, 18.761198, -0.316146, 0, 18.800520, -0.230542, 0, 18.838543, -0.254297, 0, 18.737679, -0.301790, 0, 18.761198, -0.316146, 0, 18.800520, -0.230542, 0, 18.761198, -0.316146, 0, 18.737679, -0.301790, 0, 18.800520, -0.230542, 0, 18.756250, -0.213574, 0, 18.737679, -0.301790, 0, 18.800520, -0.230542, 0, 18.737679, -0.301790, 0, 18.756250, -0.213574, 0, 18.800520, -0.230542, 0, 18.709570, -0.291536, 0, 18.737679, -0.301790, 0, 18.756250, -0.213574, 0, 18.737679, -0.301790, 0, 18.709570, -0.291536, 0, 18.756250, -0.213574, 0, 18.705730, -0.203394, 0, 18.709570, -0.291536, 0, 18.756250, -0.213574, 0, 18.709570, -0.291536, 0, 18.705730, -0.203394, 0, 18.756250, -0.213574, 0, 18.705730, -0.203394, 0, 18.676872, -0.285384, 0, 18.709570, -0.291536, 0, 18.676872, -0.285384, 0, 18.705730, -0.203394, 0, 18.709570, -0.291536, 0, 18.705730, -0.203394, 0, 18.648958, -0.200000, 0, 18.676872, -0.285384, 0, 18.648958, -0.200000, 0, 18.705730, -0.203394, 0, 18.676872, -0.285384, 0, 18.639584, -0.283333, 0, 18.676872, -0.285384, 0, 18.648958, -0.200000, 0, 18.676872, -0.285384, 0, 18.639584, -0.283333, 0, 18.648958, -0.200000, 0, 18.648958, -0.200000, 0, 18.625408, -0.200521, 0, 18.639584, -0.283333, 0, 18.625408, -0.200521, 0, 18.648958, -0.200000, 0, 18.639584, -0.283333, 0, 18.625408, -0.200521, 0, 18.617579, -0.284090, 0, 18.639584, -0.283333, 0, 18.617579, -0.284090, 0, 18.625408, -0.200521, 0, 18.639584, -0.283333, 0, 18.601107, -0.202083, 0, 18.617579, -0.284090, 0, 18.625408, -0.200521, 0, 18.617579, -0.284090, 0, 18.601107, -0.202083, 0, 18.625408, -0.200521, 0, 18.601107, -0.202083, 0, 18.594793, -0.286361, 0, 18.617579, -0.284090, 0, 18.594793, -0.286361, 0, 18.601107, -0.202083, 0, 18.617579, -0.284090, 0, 18.576057, -0.204687, 0, 18.594793, -0.286361, 0, 18.601107, -0.202083, 0, 18.594793, -0.286361, 0, 18.576057, -0.204687, 0, 18.601107, -0.202083, 0, 18.576057, -0.204687, 0, 18.571224, -0.290145, 0, 18.594793, -0.286361, 0, 18.571224, -0.290145, 0, 18.576057, -0.204687, 0, 18.594793, -0.286361, 0, 18.550261, -0.208333, 0, 18.571224, -0.290145, 0, 18.576057, -0.204687, 0, 18.571224, -0.290145, 0, 18.550261, -0.208333, 0, 18.576057, -0.204687, 0, 18.550261, -0.208333, 0, 18.546875, -0.295443, 0, 18.571224, -0.290145, 0, 18.546875, -0.295443, 0, 18.550261, -0.208333, 0, 18.571224, -0.290145, 0, 18.523617, -0.213021, 0, 18.546875, -0.295443, 0, 18.550261, -0.208333, 0, 18.546875, -0.295443, 0, 18.523617, -0.213021, 0, 18.550261, -0.208333, 0, 18.523617, -0.213021, 0, 18.521517, -0.302352, 0, 18.546875, -0.295443, 0, 18.521517, -0.302352, 0, 18.523617, -0.213021, 0, 18.546875, -0.295443, 0, 18.496029, -0.218750, 0, 18.521517, -0.302352, 0, 18.523617, -0.213021, 0, 18.521517, -0.302352, 0, 18.496029, -0.218750, 0, 18.523617, -0.213021, 0, 18.521517, -0.302352, 0, 18.496029, -0.218750, 0, 18.494923, -0.310970, 0, 18.496029, -0.218750, 0, 18.521517, -0.302352, 0, 18.494923, -0.310970, 0, 18.496029, -0.218750, 0, 18.467497, -0.225521, 0, 18.494923, -0.310970, 0, 18.467497, -0.225521, 0, 18.496029, -0.218750, 0, 18.494923, -0.310970, 0, 18.494923, -0.310970, 0, 18.467497, -0.225521, 0, 18.467091, -0.321297, 0, 18.467497, -0.225521, 0, 18.494923, -0.310970, 0, 18.467091, -0.321297, 0, 18.467497, -0.225521, 0, 18.438021, -0.233333, 0, 18.467091, -0.321297, 0, 18.438021, -0.233333, 0, 18.467497, -0.225521, 0, 18.467091, -0.321297, 0, 18.438021, -0.233333, 0, 18.438021, -0.333333, 0, 18.467091, -0.321297, 0, 18.438021, -0.333333, 0, 18.438021, -0.233333, 0, 18.467091, -0.321297, 0, 18.712240, -0.525391, 0, 18.680193, -0.619238, 0, 18.716082, -0.626953, 0, 18.680193, -0.619238, 0, 18.712240, -0.525391, 0, 18.716082, -0.626953, 0, 18.680193, -0.619238, 0, 18.712240, -0.525391, 0, 18.680273, -0.531348, 0, 18.712240, -0.525391, 0, 18.680193, -0.619238, 0, 18.680273, -0.531348, 0, 18.680193, -0.619238, 0, 18.680273, -0.531348, 0, 18.643751, -0.533333, 0, 18.680273, -0.531348, 0, 18.680193, -0.619238, 0, 18.643751, -0.533333, 0, 18.643751, -0.533333, 0, 18.639584, -0.616667, 0, 18.680193, -0.619238, 0, 18.639584, -0.616667, 0, 18.643751, -0.533333, 0, 18.680193, -0.619238, 0, 18.548958, -0.533333, 0, 18.639584, -0.616667, 0, 18.643751, -0.533333, 0, 18.639584, -0.616667, 0, 18.548958, -0.533333, 0, 18.643751, -0.533333, 0, 18.548958, -0.533333, 0, 18.548958, -0.616667, 0, 18.639584, -0.616667, 0, 18.548958, -0.616667, 0, 18.548958, -0.533333, 0, 18.639584, -0.616667, 0, 19.221355, -0.916667, 0, 19.588543, -0.916667, 0, 19.221355, -0.916667, 0, 19.221355, -0.916667, 0, 19.588543, -0.916667, 0, 19.588543, -0.916667, 0, 19.588543, -0.916667, 0, 19.588543, -1, 0, 19.588543, -0.916667, 0, 19.588543, -0.916667, 0, 19.588543, -1, 0, 19.588543, -1, 0, 19.588543, -1, 0, 19.094793, -1, 0, 19.588543, -1, 0, 19.588543, -1, 0, 19.094793, -1, 0, 19.094793, -1, 0, 19.094793, -1, 0, 19.094793, -0.916667, 0, 19.094793, -1, 0, 19.094793, -1, 0, 19.094793, -0.916667, 0, 19.094793, -0.916667, 0, 19.094793, -0.916667, 0, 19.127459, -0.882023, 0, 19.094793, -0.916667, 0, 19.094793, -0.916667, 0, 19.127459, -0.882023, 0, 19.127459, -0.882023, 0, 19.127459, -0.882023, 0, 19.165560, -0.841895, 0, 19.127459, -0.882023, 0, 19.127459, -0.882023, 0, 19.165560, -0.841895, 0, 19.165560, -0.841895, 0, 19.165560, -0.841895, 0, 19.209099, -0.796281, 0, 19.165560, -0.841895, 0, 19.165560, -0.841895, 0, 19.209099, -0.796281, 0, 19.209099, -0.796281, 0, 19.209099, -0.796281, 0, 19.258074, -0.745182, 0, 19.209099, -0.796281, 0, 19.209099, -0.796281, 0, 19.258074, -0.745182, 0, 19.258074, -0.745182, 0, 19.258074, -0.745182, 0, 19.304964, -0.696134, 0, 19.258074, -0.745182, 0, 19.258074, -0.745182, 0, 19.304964, -0.696134, 0, 19.304964, -0.696134, 0, 19.304964, -0.696134, 0, 19.342253, -0.656673, 0, 19.304964, -0.696134, 0, 19.304964, -0.696134, 0, 19.342253, -0.656673, 0, 19.342253, -0.656673, 0, 19.342253, -0.656673, 0, 19.369940, -0.626799, 0, 19.342253, -0.656673, 0, 19.342253, -0.656673, 0, 19.369940, -0.626799, 0, 19.369940, -0.626799, 0, 19.369940, -0.626799, 0, 19.388021, -0.606510, 0, 19.369940, -0.626799, 0, 19.369940, -0.626799, 0, 19.388021, -0.606510, 0, 19.388021, -0.606510, 0, 19.388021, -0.606510, 0, 19.411377, -0.578345, 0, 19.388021, -0.606510, 0, 19.388021, -0.606510, 0, 19.411377, -0.578345, 0, 19.411377, -0.578345, 0, 19.411377, -0.578345, 0, 19.430925, -0.552441, 0, 19.411377, -0.578345, 0, 19.411377, -0.578345, 0, 19.430925, -0.552441, 0, 19.430925, -0.552441, 0, 19.430925, -0.552441, 0, 19.446663, -0.528800, 0, 19.430925, -0.552441, 0, 19.430925, -0.552441, 0, 19.446663, -0.528800, 0, 19.446663, -0.528800, 0, 19.446663, -0.528800, 0, 19.458593, -0.507422, 0, 19.446663, -0.528800, 0, 19.446663, -0.528800, 0, 19.458593, -0.507422, 0, 19.458593, -0.507422, 0, 19.458593, -0.507422, 0, 19.467367, -0.487248, 0, 19.458593, -0.507422, 0, 19.458593, -0.507422, 0, 19.467367, -0.487248, 0, 19.467367, -0.487248, 0, 19.467367, -0.487248, 0, 19.473633, -0.467220, 0, 19.467367, -0.487248, 0, 19.467367, -0.487248, 0, 19.473633, -0.467220, 0, 19.473633, -0.467220, 0, 19.473633, -0.467220, 0, 19.477392, -0.447339, 0, 19.473633, -0.467220, 0, 19.473633, -0.467220, 0, 19.477392, -0.447339, 0, 19.477392, -0.447339, 0, 19.477392, -0.447339, 0, 19.478645, -0.427604, 0, 19.477392, -0.447339, 0, 19.477392, -0.447339, 0, 19.478645, -0.427604, 0, 19.478645, -0.427604, 0, 19.478645, -0.427604, 0, 19.475927, -0.397176, 0, 19.478645, -0.427604, 0, 19.478645, -0.427604, 0, 19.475927, -0.397176, 0, 19.475927, -0.397176, 0, 19.475927, -0.397176, 0, 19.467773, -0.369694, 0, 19.475927, -0.397176, 0, 19.475927, -0.397176, 0, 19.467773, -0.369694, 0, 19.467773, -0.369694, 0, 19.467773, -0.369694, 0, 19.454184, -0.345158, 0, 19.467773, -0.369694, 0, 19.467773, -0.369694, 0, 19.454184, -0.345158, 0, 19.454184, -0.345158, 0, 19.454184, -0.345158, 0, 19.435156, -0.323568, 0, 19.454184, -0.345158, 0, 19.454184, -0.345158, 0, 19.435156, -0.323568, 0, 19.435156, -0.323568, 0, 19.435156, -0.323568, 0, 19.411768, -0.305965, 0, 19.435156, -0.323568, 0, 19.435156, -0.323568, 0, 19.411768, -0.305965, 0, 19.411768, -0.305965, 0, 19.411768, -0.305965, 0, 19.385092, -0.293392, 0, 19.411768, -0.305965, 0, 19.411768, -0.305965, 0, 19.385092, -0.293392, 0, 19.385092, -0.293392, 0, 19.385092, -0.293392, 0, 19.355127, -0.285848, 0, 19.385092, -0.293392, 0, 19.385092, -0.293392, 0, 19.355127, -0.285848, 0, 19.355127, -0.285848, 0, 19.355127, -0.285848, 0, 19.321875, -0.283333, 0, 19.355127, -0.285848, 0, 19.355127, -0.285848, 0, 19.321875, -0.283333, 0, 19.321875, -0.283333, 0, 19.321875, -0.283333, 0, 19.296795, -0.284627, 0, 19.321875, -0.283333, 0, 19.321875, -0.283333, 0, 19.296795, -0.284627, 0, 19.296795, -0.284627, 0, 19.296795, -0.284627, 0, 19.271029, -0.288509, 0, 19.296795, -0.284627, 0, 19.296795, -0.284627, 0, 19.271029, -0.288509, 0, 19.271029, -0.288509, 0, 19.271029, -0.288509, 0, 19.244581, -0.294979, 0, 19.271029, -0.288509, 0, 19.271029, -0.288509, 0, 19.244581, -0.294979, 0, 19.244581, -0.294979, 0, 19.244581, -0.294979, 0, 19.217449, -0.304036, 0, 19.244581, -0.294979, 0, 19.244581, -0.294979, 0, 19.217449, -0.304036, 0, 19.217449, -0.304036, 0, 19.217449, -0.304036, 0, 19.189503, -0.315714, 0, 19.217449, -0.304036, 0, 19.217449, -0.304036, 0, 19.189503, -0.315714, 0, 19.189503, -0.315714, 0, 19.189503, -0.315714, 0, 19.160612, -0.330046, 0, 19.189503, -0.315714, 0, 19.189503, -0.315714, 0, 19.160612, -0.330046, 0, 19.160612, -0.330046, 0, 19.160612, -0.330046, 0, 19.130779, -0.347030, 0, 19.160612, -0.330046, 0, 19.160612, -0.330046, 0, 19.130779, -0.347030, 0, 19.130779, -0.347030, 0, 19.130779, -0.347030, 0, 19.100000, -0.366667, 0, 19.130779, -0.347030, 0, 19.130779, -0.347030, 0, 19.100000, -0.366667, 0, 19.100000, -0.366667, 0, 19.100000, -0.366667, 0, 19.100000, -0.250000, 0, 19.100000, -0.366667, 0, 19.100000, -0.366667, 0, 19.100000, -0.250000, 0, 19.100000, -0.250000, 0, 19.100000, -0.250000, 0, 19.131250, -0.238387, 0, 19.100000, -0.250000, 0, 19.100000, -0.250000, 0, 19.131250, -0.238387, 0, 19.131250, -0.238387, 0, 19.131250, -0.238387, 0, 19.161459, -0.228288, 0, 19.131250, -0.238387, 0, 19.131250, -0.238387, 0, 19.161459, -0.228288, 0, 19.161459, -0.228288, 0, 19.161459, -0.228288, 0, 19.190626, -0.219702, 0, 19.161459, -0.228288, 0, 19.161459, -0.228288, 0, 19.190626, -0.219702, 0, 19.190626, -0.219702, 0, 19.190626, -0.219702, 0, 19.218750, -0.212630, 0, 19.190626, -0.219702, 0, 19.190626, -0.219702, 0, 19.218750, -0.212630, 0, 19.218750, -0.212630, 0, 19.218750, -0.212630, 0, 19.245770, -0.207105, 0, 19.218750, -0.212630, 0, 19.218750, -0.212630, 0, 19.245770, -0.207105, 0, 19.245770, -0.207105, 0, 19.245770, -0.207105, 0, 19.271614, -0.203158, 0, 19.245770, -0.207105, 0, 19.245770, -0.207105, 0, 19.271614, -0.203158, 0, 19.271614, -0.203158, 0, 19.271614, -0.203158, 0, 19.296289, -0.200789, 0, 19.271614, -0.203158, 0, 19.271614, -0.203158, 0, 19.296289, -0.200789, 0, 19.296289, -0.200789, 0, 19.296289, -0.200789, 0, 19.319792, -0.200000, 0, 19.296289, -0.200789, 0, 19.296289, -0.200789, 0, 19.319792, -0.200000, 0, 19.319792, -0.200000, 0, 19.319792, -0.200000, 0, 19.377150, -0.203849, 0, 19.319792, -0.200000, 0, 19.319792, -0.200000, 0, 19.377150, -0.203849, 0, 19.377150, -0.203849, 0, 19.377150, -0.203849, 0, 19.428387, -0.215397, 0, 19.377150, -0.203849, 0, 19.377150, -0.203849, 0, 19.428387, -0.215397, 0, 19.428387, -0.215397, 0, 19.428387, -0.215397, 0, 19.473503, -0.234644, 0, 19.428387, -0.215397, 0, 19.428387, -0.215397, 0, 19.473503, -0.234644, 0, 19.473503, -0.234644, 0, 19.473503, -0.234644, 0, 19.512501, -0.261589, 0, 19.473503, -0.234644, 0, 19.473503, -0.234644, 0, 19.512501, -0.261589, 0, 19.512501, -0.261589, 0, 19.512501, -0.261589, 0, 19.543945, -0.294963, 0, 19.512501, -0.261589, 0, 19.512501, -0.261589, 0, 19.543945, -0.294963, 0, 19.543945, -0.294963, 0, 19.543945, -0.294963, 0, 19.566406, -0.333496, 0, 19.543945, -0.294963, 0, 19.543945, -0.294963, 0, 19.566406, -0.333496, 0, 19.566406, -0.333496, 0, 19.566406, -0.333496, 0, 19.579884, -0.377189, 0, 19.566406, -0.333496, 0, 19.566406, -0.333496, 0, 19.579884, -0.377189, 0, 19.579884, -0.377189, 0, 19.579884, -0.377189, 0, 19.584375, -0.426042, 0, 19.579884, -0.377189, 0, 19.579884, -0.377189, 0, 19.584375, -0.426042, 0, 19.584375, -0.426042, 0, 19.584375, -0.426042, 0, 19.583252, -0.450195, 0, 19.584375, -0.426042, 0, 19.584375, -0.426042, 0, 19.583252, -0.450195, 0, 19.583252, -0.450195, 0, 19.583252, -0.450195, 0, 19.579884, -0.473698, 0, 19.583252, -0.450195, 0, 19.583252, -0.450195, 0, 19.579884, -0.473698, 0, 19.579884, -0.473698, 0, 19.579884, -0.473698, 0, 19.574268, -0.496549, 0, 19.579884, -0.473698, 0, 19.579884, -0.473698, 0, 19.574268, -0.496549, 0, 19.574268, -0.496549, 0, 19.574268, -0.496549, 0, 19.566406, -0.518750, 0, 19.574268, -0.496549, 0, 19.574268, -0.496549, 0, 19.566406, -0.518750, 0, 19.566406, -0.518750, 0, 19.566406, -0.518750, 0, 19.555584, -0.541618, 0, 19.566406, -0.518750, 0, 19.566406, -0.518750, 0, 19.555584, -0.541618, 0, 19.555584, -0.541618, 0, 19.555584, -0.541618, 0, 19.541082, -0.566471, 0, 19.555584, -0.541618, 0, 19.555584, -0.541618, 0, 19.541082, -0.566471, 0, 19.541082, -0.566471, 0, 19.541082, -0.566471, 0, 19.522902, -0.593311, 0, 19.541082, -0.566471, 0, 19.541082, -0.566471, 0, 19.522902, -0.593311, 0, 19.522902, -0.593311, 0, 19.522902, -0.593311, 0, 19.501043, -0.622135, 0, 19.522902, -0.593311, 0, 19.522902, -0.593311, 0, 19.501043, -0.622135, 0, 19.501043, -0.622135, 0, 19.501043, -0.622135, 0, 19.490984, -0.633447, 0, 19.501043, -0.622135, 0, 19.501043, -0.622135, 0, 19.490984, -0.633447, 0, 19.490984, -0.633447, 0, 19.490984, -0.633447, 0, 19.473829, -0.652018, 0, 19.490984, -0.633447, 0, 19.490984, -0.633447, 0, 19.473829, -0.652018, 0, 19.473829, -0.652018, 0, 19.473829, -0.652018, 0, 19.449577, -0.677848, 0, 19.473829, -0.652018, 0, 19.473829, -0.652018, 0, 19.449577, -0.677848, 0, 19.449577, -0.677848, 0, 19.449577, -0.677848, 0, 19.418230, -0.710938, 0, 19.449577, -0.677848, 0, 19.449577, -0.677848, 0, 19.418230, -0.710938, 0, 19.418230, -0.710938, 0, 19.418230, -0.710938, 0, 19.379753, -0.751335, 0, 19.418230, -0.710938, 0, 19.418230, -0.710938, 0, 19.379753, -0.751335, 0, 19.379753, -0.751335, 0, 19.379753, -0.751335, 0, 19.334114, -0.799089, 0, 19.379753, -0.751335, 0, 19.379753, -0.751335, 0, 19.334114, -0.799089, 0, 19.334114, -0.799089, 0, 19.334114, -0.799089, 0, 19.281317, -0.854199, 0, 19.334114, -0.799089, 0, 19.334114, -0.799089, 0, 19.281317, -0.854199, 0, 19.281317, -0.854199, 0, 19.127459, -0.882023, 0, 19.094793, -0.916667, 0, 19.221355, -0.916667, 0, 19.094793, -0.916667, 0, 19.127459, -0.882023, 0, 19.221355, -0.916667, 0, 19.165560, -0.841895, 0, 19.127459, -0.882023, 0, 19.221355, -0.916667, 0, 19.127459, -0.882023, 0, 19.165560, -0.841895, 0, 19.221355, -0.916667, 0, 19.165560, -0.841895, 0, 19.221355, -0.916667, 0, 19.281317, -0.854199, 0, 19.221355, -0.916667, 0, 19.165560, -0.841895, 0, 19.281317, -0.854199, 0, 19.209099, -0.796281, 0, 19.165560, -0.841895, 0, 19.281317, -0.854199, 0, 19.165560, -0.841895, 0, 19.209099, -0.796281, 0, 19.281317, -0.854199, 0, 19.258074, -0.745182, 0, 19.209099, -0.796281, 0, 19.281317, -0.854199, 0, 19.209099, -0.796281, 0, 19.258074, -0.745182, 0, 19.281317, -0.854199, 0, 19.258074, -0.745182, 0, 19.281317, -0.854199, 0, 19.334114, -0.799089, 0, 19.281317, -0.854199, 0, 19.258074, -0.745182, 0, 19.334114, -0.799089, 0, 19.304964, -0.696134, 0, 19.258074, -0.745182, 0, 19.334114, -0.799089, 0, 19.258074, -0.745182, 0, 19.304964, -0.696134, 0, 19.334114, -0.799089, 0, 19.304964, -0.696134, 0, 19.334114, -0.799089, 0, 19.379753, -0.751335, 0, 19.334114, -0.799089, 0, 19.304964, -0.696134, 0, 19.379753, -0.751335, 0, 19.342253, -0.656673, 0, 19.304964, -0.696134, 0, 19.379753, -0.751335, 0, 19.304964, -0.696134, 0, 19.342253, -0.656673, 0, 19.379753, -0.751335, 0, 19.342253, -0.656673, 0, 19.379753, -0.751335, 0, 19.418230, -0.710938, 0, 19.379753, -0.751335, 0, 19.342253, -0.656673, 0, 19.418230, -0.710938, 0, 19.369940, -0.626799, 0, 19.342253, -0.656673, 0, 19.418230, -0.710938, 0, 19.342253, -0.656673, 0, 19.369940, -0.626799, 0, 19.418230, -0.710938, 0, 19.369940, -0.626799, 0, 19.418230, -0.710938, 0, 19.449577, -0.677848, 0, 19.418230, -0.710938, 0, 19.369940, -0.626799, 0, 19.449577, -0.677848, 0, 19.388021, -0.606510, 0, 19.369940, -0.626799, 0, 19.449577, -0.677848, 0, 19.369940, -0.626799, 0, 19.388021, -0.606510, 0, 19.449577, -0.677848, 0, 19.388021, -0.606510, 0, 19.449577, -0.677848, 0, 19.473829, -0.652018, 0, 19.449577, -0.677848, 0, 19.388021, -0.606510, 0, 19.473829, -0.652018, 0, 19.411377, -0.578345, 0, 19.388021, -0.606510, 0, 19.473829, -0.652018, 0, 19.388021, -0.606510, 0, 19.411377, -0.578345, 0, 19.473829, -0.652018, 0, 19.411377, -0.578345, 0, 19.473829, -0.652018, 0, 19.490984, -0.633447, 0, 19.473829, -0.652018, 0, 19.411377, -0.578345, 0, 19.490984, -0.633447, 0, 19.501043, -0.622135, 0, 19.411377, -0.578345, 0, 19.490984, -0.633447, 0, 19.411377, -0.578345, 0, 19.501043, -0.622135, 0, 19.490984, -0.633447, 0, 19.501043, -0.622135, 0, 19.430925, -0.552441, 0, 19.411377, -0.578345, 0, 19.430925, -0.552441, 0, 19.501043, -0.622135, 0, 19.411377, -0.578345, 0, 19.430925, -0.552441, 0, 19.501043, -0.622135, 0, 19.522902, -0.593311, 0, 19.501043, -0.622135, 0, 19.430925, -0.552441, 0, 19.522902, -0.593311, 0, 19.446663, -0.528800, 0, 19.430925, -0.552441, 0, 19.522902, -0.593311, 0, 19.430925, -0.552441, 0, 19.446663, -0.528800, 0, 19.522902, -0.593311, 0, 19.446663, -0.528800, 0, 19.522902, -0.593311, 0, 19.541082, -0.566471, 0, 19.522902, -0.593311, 0, 19.446663, -0.528800, 0, 19.541082, -0.566471, 0, 19.458593, -0.507422, 0, 19.446663, -0.528800, 0, 19.541082, -0.566471, 0, 19.446663, -0.528800, 0, 19.458593, -0.507422, 0, 19.541082, -0.566471, 0, 19.458593, -0.507422, 0, 19.541082, -0.566471, 0, 19.555584, -0.541618, 0, 19.541082, -0.566471, 0, 19.458593, -0.507422, 0, 19.555584, -0.541618, 0, 19.467367, -0.487248, 0, 19.458593, -0.507422, 0, 19.555584, -0.541618, 0, 19.458593, -0.507422, 0, 19.467367, -0.487248, 0, 19.555584, -0.541618, 0, 19.467367, -0.487248, 0, 19.555584, -0.541618, 0, 19.566406, -0.518750, 0, 19.555584, -0.541618, 0, 19.467367, -0.487248, 0, 19.566406, -0.518750, 0, 19.473633, -0.467220, 0, 19.467367, -0.487248, 0, 19.566406, -0.518750, 0, 19.467367, -0.487248, 0, 19.473633, -0.467220, 0, 19.566406, -0.518750, 0, 19.473633, -0.467220, 0, 19.566406, -0.518750, 0, 19.574268, -0.496549, 0, 19.566406, -0.518750, 0, 19.473633, -0.467220, 0, 19.574268, -0.496549, 0, 19.473633, -0.467220, 0, 19.574268, -0.496549, 0, 19.579884, -0.473698, 0, 19.574268, -0.496549, 0, 19.473633, -0.467220, 0, 19.579884, -0.473698, 0, 19.477392, -0.447339, 0, 19.473633, -0.467220, 0, 19.579884, -0.473698, 0, 19.473633, -0.467220, 0, 19.477392, -0.447339, 0, 19.579884, -0.473698, 0, 19.477392, -0.447339, 0, 19.579884, -0.473698, 0, 19.583252, -0.450195, 0, 19.579884, -0.473698, 0, 19.477392, -0.447339, 0, 19.583252, -0.450195, 0, 19.478645, -0.427604, 0, 19.477392, -0.447339, 0, 19.583252, -0.450195, 0, 19.477392, -0.447339, 0, 19.478645, -0.427604, 0, 19.583252, -0.450195, 0, 19.478645, -0.427604, 0, 19.583252, -0.450195, 0, 19.584375, -0.426042, 0, 19.583252, -0.450195, 0, 19.478645, -0.427604, 0, 19.584375, -0.426042, 0, 19.584375, -0.426042, 0, 19.475927, -0.397176, 0, 19.478645, -0.427604, 0, 19.475927, -0.397176, 0, 19.584375, -0.426042, 0, 19.478645, -0.427604, 0, 19.579884, -0.377189, 0, 19.475927, -0.397176, 0, 19.584375, -0.426042, 0, 19.475927, -0.397176, 0, 19.579884, -0.377189, 0, 19.584375, -0.426042, 0, 19.566406, -0.333496, 0, 19.475927, -0.397176, 0, 19.579884, -0.377189, 0, 19.475927, -0.397176, 0, 19.566406, -0.333496, 0, 19.579884, -0.377189, 0, 19.467773, -0.369694, 0, 19.475927, -0.397176, 0, 19.566406, -0.333496, 0, 19.475927, -0.397176, 0, 19.467773, -0.369694, 0, 19.566406, -0.333496, 0, 19.543945, -0.294963, 0, 19.467773, -0.369694, 0, 19.566406, -0.333496, 0, 19.467773, -0.369694, 0, 19.543945, -0.294963, 0, 19.566406, -0.333496, 0, 19.454184, -0.345158, 0, 19.467773, -0.369694, 0, 19.543945, -0.294963, 0, 19.467773, -0.369694, 0, 19.454184, -0.345158, 0, 19.543945, -0.294963, 0, 19.512501, -0.261589, 0, 19.454184, -0.345158, 0, 19.543945, -0.294963, 0, 19.454184, -0.345158, 0, 19.512501, -0.261589, 0, 19.543945, -0.294963, 0, 19.435156, -0.323568, 0, 19.454184, -0.345158, 0, 19.512501, -0.261589, 0, 19.454184, -0.345158, 0, 19.435156, -0.323568, 0, 19.512501, -0.261589, 0, 19.473503, -0.234644, 0, 19.435156, -0.323568, 0, 19.512501, -0.261589, 0, 19.435156, -0.323568, 0, 19.473503, -0.234644, 0, 19.512501, -0.261589, 0, 19.411768, -0.305965, 0, 19.435156, -0.323568, 0, 19.473503, -0.234644, 0, 19.435156, -0.323568, 0, 19.411768, -0.305965, 0, 19.473503, -0.234644, 0, 19.428387, -0.215397, 0, 19.411768, -0.305965, 0, 19.473503, -0.234644, 0, 19.411768, -0.305965, 0, 19.428387, -0.215397, 0, 19.473503, -0.234644, 0, 19.385092, -0.293392, 0, 19.411768, -0.305965, 0, 19.428387, -0.215397, 0, 19.411768, -0.305965, 0, 19.385092, -0.293392, 0, 19.428387, -0.215397, 0, 19.377150, -0.203849, 0, 19.385092, -0.293392, 0, 19.428387, -0.215397, 0, 19.385092, -0.293392, 0, 19.377150, -0.203849, 0, 19.428387, -0.215397, 0, 19.377150, -0.203849, 0, 19.355127, -0.285848, 0, 19.385092, -0.293392, 0, 19.355127, -0.285848, 0, 19.377150, -0.203849, 0, 19.385092, -0.293392, 0, 19.355127, -0.285848, 0, 19.377150, -0.203849, 0, 19.319792, -0.200000, 0, 19.377150, -0.203849, 0, 19.355127, -0.285848, 0, 19.319792, -0.200000, 0, 19.319792, -0.200000, 0, 19.321875, -0.283333, 0, 19.355127, -0.285848, 0, 19.321875, -0.283333, 0, 19.319792, -0.200000, 0, 19.355127, -0.285848, 0, 19.319792, -0.200000, 0, 19.296289, -0.200789, 0, 19.321875, -0.283333, 0, 19.296289, -0.200789, 0, 19.319792, -0.200000, 0, 19.321875, -0.283333, 0, 19.296289, -0.200789, 0, 19.296795, -0.284627, 0, 19.321875, -0.283333, 0, 19.296795, -0.284627, 0, 19.296289, -0.200789, 0, 19.321875, -0.283333, 0, 19.296289, -0.200789, 0, 19.271614, -0.203158, 0, 19.296795, -0.284627, 0, 19.271614, -0.203158, 0, 19.296289, -0.200789, 0, 19.296795, -0.284627, 0, 19.271614, -0.203158, 0, 19.271029, -0.288509, 0, 19.296795, -0.284627, 0, 19.271029, -0.288509, 0, 19.271614, -0.203158, 0, 19.296795, -0.284627, 0, 19.245770, -0.207105, 0, 19.271029, -0.288509, 0, 19.271614, -0.203158, 0, 19.271029, -0.288509, 0, 19.245770, -0.207105, 0, 19.271614, -0.203158, 0, 19.245770, -0.207105, 0, 19.244581, -0.294979, 0, 19.271029, -0.288509, 0, 19.244581, -0.294979, 0, 19.245770, -0.207105, 0, 19.271029, -0.288509, 0, 19.218750, -0.212630, 0, 19.244581, -0.294979, 0, 19.245770, -0.207105, 0, 19.244581, -0.294979, 0, 19.218750, -0.212630, 0, 19.245770, -0.207105, 0, 19.244581, -0.294979, 0, 19.218750, -0.212630, 0, 19.217449, -0.304036, 0, 19.218750, -0.212630, 0, 19.244581, -0.294979, 0, 19.217449, -0.304036, 0, 19.218750, -0.212630, 0, 19.190626, -0.219702, 0, 19.217449, -0.304036, 0, 19.190626, -0.219702, 0, 19.218750, -0.212630, 0, 19.217449, -0.304036, 0, 19.190626, -0.219702, 0, 19.161459, -0.228288, 0, 19.217449, -0.304036, 0, 19.161459, -0.228288, 0, 19.190626, -0.219702, 0, 19.217449, -0.304036, 0, 19.161459, -0.228288, 0, 19.189503, -0.315714, 0, 19.217449, -0.304036, 0, 19.189503, -0.315714, 0, 19.161459, -0.228288, 0, 19.217449, -0.304036, 0, 19.161459, -0.228288, 0, 19.131250, -0.238387, 0, 19.189503, -0.315714, 0, 19.131250, -0.238387, 0, 19.161459, -0.228288, 0, 19.189503, -0.315714, 0, 19.131250, -0.238387, 0, 19.160612, -0.330046, 0, 19.189503, -0.315714, 0, 19.160612, -0.330046, 0, 19.131250, -0.238387, 0, 19.189503, -0.315714, 0, 19.131250, -0.238387, 0, 19.100000, -0.250000, 0, 19.160612, -0.330046, 0, 19.100000, -0.250000, 0, 19.131250, -0.238387, 0, 19.160612, -0.330046, 0, 19.100000, -0.250000, 0, 19.130779, -0.347030, 0, 19.160612, -0.330046, 0, 19.130779, -0.347030, 0, 19.100000, -0.250000, 0, 19.160612, -0.330046, 0, 19.100000, -0.250000, 0, 19.100000, -0.366667, 0, 19.130779, -0.347030, 0, 19.100000, -0.366667, 0, 19.100000, -0.250000, 0, 19.130779, -0.347030, 0, 19.221355, -0.916667, 0, 19.094793, -0.916667, 0, 19.094793, -1, 0, 19.094793, -0.916667, 0, 19.221355, -0.916667, 0, 19.094793, -1, 0, 19.588543, -1, 0, 19.221355, -0.916667, 0, 19.094793, -1, 0, 19.221355, -0.916667, 0, 19.588543, -1, 0, 19.094793, -1, 0, 19.221355, -0.916667, 0, 19.588543, -1, 0, 19.588543, -0.916667, 0, 19.588543, -1, 0, 19.221355, -0.916667, 0, 19.588543, -0.916667, 0, 19.814062, -0.866667, 0, 19.923958, -0.866667, 0, 19.814062, -0.866667, 0, 19.814062, -0.866667, 0, 19.923958, -0.866667, 0, 19.923958, -0.866667, 0, 19.923958, -0.866667, 0, 19.923958, -1, 0, 19.923958, -0.866667, 0, 19.923958, -0.866667, 0, 19.923958, -1, 0, 19.923958, -1, 0, 19.923958, -1, 0, 19.814062, -1, 0, 19.923958, -1, 0, 19.923958, -1, 0, 19.814062, -1, 0, 19.814062, -1, 0, 19.814062, -0.866667, 0, 19.923958, -1, 0, 19.923958, -0.866667, 0, 19.923958, -1, 0, 19.814062, -0.866667, 0, 19.923958, -0.866667, 0, 19.814062, -0.866667, 0, 19.814062, -1, 0, 19.923958, -1, 0, 19.814062, -1, 0, 19.814062, -0.866667, 0, 19.923958, -1, 0, 20.243231, -0.300000, 0, 20.243231, -0.916667, 0, 20.243231, -0.300000, 0, 20.243231, -0.300000, 0, 20.243231, -0.916667, 0, 20.243231, -0.916667, 0, 20.243231, -0.916667, 0, 20.370314, -0.916667, 0, 20.243231, -0.916667, 0, 20.243231, -0.916667, 0, 20.370314, -0.916667, 0, 20.370314, -0.916667, 0, 20.370314, -0.916667, 0, 20.445395, -0.912020, 0, 20.370314, -0.916667, 0, 20.370314, -0.916667, 0, 20.445395, -0.912020, 0, 20.445395, -0.912020, 0, 20.445395, -0.912020, 0, 20.509701, -0.898079, 0, 20.445395, -0.912020, 0, 20.445395, -0.912020, 0, 20.509701, -0.898079, 0, 20.509701, -0.898079, 0, 20.509701, -0.898079, 0, 20.563232, -0.874845, 0, 20.509701, -0.898079, 0, 20.509701, -0.898079, 0, 20.563232, -0.874845, 0, 20.563232, -0.874845, 0, 20.563232, -0.874845, 0, 20.605989, -0.842318, 0, 20.563232, -0.874845, 0, 20.563232, -0.874845, 0, 20.605989, -0.842318, 0, 20.605989, -0.842318, 0, 20.605989, -0.842318, 0, 20.638689, -0.799764, 0, 20.605989, -0.842318, 0, 20.605989, -0.842318, 0, 20.638689, -0.799764, 0, 20.638689, -0.799764, 0, 20.638689, -0.799764, 0, 20.662045, -0.746452, 0, 20.638689, -0.799764, 0, 20.638689, -0.799764, 0, 20.662045, -0.746452, 0, 20.662045, -0.746452, 0, 20.662045, -0.746452, 0, 20.676058, -0.682381, 0, 20.662045, -0.746452, 0, 20.662045, -0.746452, 0, 20.676058, -0.682381, 0, 20.676058, -0.682381, 0, 20.676058, -0.682381, 0, 20.680731, -0.607552, 0, 20.676058, -0.682381, 0, 20.676058, -0.682381, 0, 20.680731, -0.607552, 0, 20.680731, -0.607552, 0, 20.680731, -0.607552, 0, 20.676058, -0.533195, 0, 20.680731, -0.607552, 0, 20.680731, -0.607552, 0, 20.676058, -0.533195, 0, 20.676058, -0.533195, 0, 20.676058, -0.533195, 0, 20.662045, -0.469499, 0, 20.676058, -0.533195, 0, 20.676058, -0.533195, 0, 20.662045, -0.469499, 0, 20.662045, -0.469499, 0, 20.662045, -0.469499, 0, 20.638689, -0.416463, 0, 20.662045, -0.469499, 0, 20.662045, -0.469499, 0, 20.638689, -0.416463, 0, 20.638689, -0.416463, 0, 20.638689, -0.416463, 0, 20.605989, -0.374089, 0, 20.638689, -0.416463, 0, 20.638689, -0.416463, 0, 20.605989, -0.374089, 0, 20.605989, -0.374089, 0, 20.605989, -0.374089, 0, 20.563232, -0.341675, 0, 20.605989, -0.374089, 0, 20.605989, -0.374089, 0, 20.563232, -0.341675, 0, 20.563232, -0.341675, 0, 20.563232, -0.341675, 0, 20.509701, -0.318522, 0, 20.563232, -0.341675, 0, 20.563232, -0.341675, 0, 20.509701, -0.318522, 0, 20.509701, -0.318522, 0, 20.509701, -0.318522, 0, 20.445395, -0.304631, 0, 20.509701, -0.318522, 0, 20.509701, -0.318522, 0, 20.445395, -0.304631, 0, 20.445395, -0.304631, 0, 20.445395, -0.304631, 0, 20.370314, -0.300000, 0, 20.445395, -0.304631, 0, 20.445395, -0.304631, 0, 20.370314, -0.300000, 0, 20.370314, -0.300000, 0, 20.138021, -0.216667, 0, 20.354168, -0.216667, 0, 20.138021, -0.216667, 0, 20.138021, -0.216667, 0, 20.354168, -0.216667, 0, 20.354168, -0.216667, 0, 20.354168, -0.216667, 0, 20.459669, -0.222583, 0, 20.354168, -0.216667, 0, 20.354168, -0.216667, 0, 20.459669, -0.222583, 0, 20.459669, -0.222583, 0, 20.459669, -0.222583, 0, 20.550131, -0.240332, 0, 20.459669, -0.222583, 0, 20.459669, -0.222583, 0, 20.550131, -0.240332, 0, 20.550131, -0.240332, 0, 20.550131, -0.240332, 0, 20.625555, -0.269914, 0, 20.550131, -0.240332, 0, 20.550131, -0.240332, 0, 20.625555, -0.269914, 0, 20.625555, -0.269914, 0, 20.625555, -0.269914, 0, 20.685938, -0.311328, 0, 20.625555, -0.269914, 0, 20.625555, -0.269914, 0, 20.685938, -0.311328, 0, 20.685938, -0.311328, 0, 20.685938, -0.311328, 0, 20.732195, -0.365340, 0, 20.685938, -0.311328, 0, 20.685938, -0.311328, 0, 20.732195, -0.365340, 0, 20.732195, -0.365340, 0, 20.732195, -0.365340, 0, 20.765234, -0.432715, 0, 20.732195, -0.365340, 0, 20.732195, -0.365340, 0, 20.765234, -0.432715, 0, 20.765234, -0.432715, 0, 20.765234, -0.432715, 0, 20.785059, -0.513452, 0, 20.765234, -0.432715, 0, 20.765234, -0.432715, 0, 20.785059, -0.513452, 0, 20.785059, -0.513452, 0, 20.785059, -0.513452, 0, 20.791668, -0.607552, 0, 20.785059, -0.513452, 0, 20.785059, -0.513452, 0, 20.791668, -0.607552, 0, 20.791668, -0.607552, 0, 20.791668, -0.607552, 0, 20.785027, -0.702124, 0, 20.791668, -0.607552, 0, 20.791668, -0.607552, 0, 20.785027, -0.702124, 0, 20.785027, -0.702124, 0, 20.785027, -0.702124, 0, 20.765104, -0.783236, 0, 20.785027, -0.702124, 0, 20.785027, -0.702124, 0, 20.765104, -0.783236, 0, 20.765104, -0.783236, 0, 20.765104, -0.783236, 0, 20.731901, -0.850887, 0, 20.765104, -0.783236, 0, 20.765104, -0.783236, 0, 20.731901, -0.850887, 0, 20.731901, -0.850887, 0, 20.731901, -0.850887, 0, 20.685417, -0.905078, 0, 20.731901, -0.850887, 0, 20.731901, -0.850887, 0, 20.685417, -0.905078, 0, 20.685417, -0.905078, 0, 20.685417, -0.905078, 0, 20.624870, -0.946606, 0, 20.685417, -0.905078, 0, 20.685417, -0.905078, 0, 20.624870, -0.946606, 0, 20.624870, -0.946606, 0, 20.624870, -0.946606, 0, 20.549480, -0.976270, 0, 20.624870, -0.946606, 0, 20.624870, -0.946606, 0, 20.549480, -0.976270, 0, 20.549480, -0.976270, 0, 20.549480, -0.976270, 0, 20.459246, -0.994067, 0, 20.549480, -0.976270, 0, 20.549480, -0.976270, 0, 20.459246, -0.994067, 0, 20.459246, -0.994067, 0, 20.459246, -0.994067, 0, 20.354168, -1, 0, 20.459246, -0.994067, 0, 20.459246, -0.994067, 0, 20.354168, -1, 0, 20.354168, -1, 0, 20.354168, -1, 0, 20.138021, -1, 0, 20.354168, -1, 0, 20.354168, -1, 0, 20.138021, -1, 0, 20.138021, -1, 0, 20.138021, -0.216667, 0, 20.243231, -0.300000, 0, 20.354168, -0.216667, 0, 20.243231, -0.300000, 0, 20.138021, -0.216667, 0, 20.354168, -0.216667, 0, 20.138021, -0.216667, 0, 20.138021, -1, 0, 20.243231, -0.300000, 0, 20.138021, -1, 0, 20.138021, -0.216667, 0, 20.243231, -0.300000, 0, 20.138021, -1, 0, 20.243231, -0.916667, 0, 20.243231, -0.300000, 0, 20.243231, -0.916667, 0, 20.138021, -1, 0, 20.243231, -0.300000, 0, 20.243231, -0.916667, 0, 20.138021, -1, 0, 20.354168, -1, 0, 20.138021, -1, 0, 20.243231, -0.916667, 0, 20.354168, -1, 0, 20.243231, -0.916667, 0, 20.354168, -1, 0, 20.370314, -0.916667, 0, 20.354168, -1, 0, 20.243231, -0.916667, 0, 20.370314, -0.916667, 0, 20.370314, -0.916667, 0, 20.354168, -1, 0, 20.459246, -0.994067, 0, 20.354168, -1, 0, 20.370314, -0.916667, 0, 20.459246, -0.994067, 0, 20.445395, -0.912020, 0, 20.370314, -0.916667, 0, 20.459246, -0.994067, 0, 20.370314, -0.916667, 0, 20.445395, -0.912020, 0, 20.459246, -0.994067, 0, 20.509701, -0.898079, 0, 20.445395, -0.912020, 0, 20.459246, -0.994067, 0, 20.445395, -0.912020, 0, 20.509701, -0.898079, 0, 20.459246, -0.994067, 0, 20.549480, -0.976270, 0, 20.509701, -0.898079, 0, 20.459246, -0.994067, 0, 20.509701, -0.898079, 0, 20.549480, -0.976270, 0, 20.459246, -0.994067, 0, 20.563232, -0.874845, 0, 20.509701, -0.898079, 0, 20.549480, -0.976270, 0, 20.509701, -0.898079, 0, 20.563232, -0.874845, 0, 20.549480, -0.976270, 0, 20.624870, -0.946606, 0, 20.563232, -0.874845, 0, 20.549480, -0.976270, 0, 20.563232, -0.874845, 0, 20.624870, -0.946606, 0, 20.549480, -0.976270, 0, 20.605989, -0.842318, 0, 20.563232, -0.874845, 0, 20.624870, -0.946606, 0, 20.563232, -0.874845, 0, 20.605989, -0.842318, 0, 20.624870, -0.946606, 0, 20.605989, -0.842318, 0, 20.624870, -0.946606, 0, 20.685417, -0.905078, 0, 20.624870, -0.946606, 0, 20.605989, -0.842318, 0, 20.685417, -0.905078, 0, 20.638689, -0.799764, 0, 20.605989, -0.842318, 0, 20.685417, -0.905078, 0, 20.605989, -0.842318, 0, 20.638689, -0.799764, 0, 20.685417, -0.905078, 0, 20.638689, -0.799764, 0, 20.685417, -0.905078, 0, 20.731901, -0.850887, 0, 20.685417, -0.905078, 0, 20.638689, -0.799764, 0, 20.731901, -0.850887, 0, 20.662045, -0.746452, 0, 20.638689, -0.799764, 0, 20.731901, -0.850887, 0, 20.638689, -0.799764, 0, 20.662045, -0.746452, 0, 20.731901, -0.850887, 0, 20.765104, -0.783236, 0, 20.662045, -0.746452, 0, 20.731901, -0.850887, 0, 20.662045, -0.746452, 0, 20.765104, -0.783236, 0, 20.731901, -0.850887, 0, 20.662045, -0.746452, 0, 20.765104, -0.783236, 0, 20.785027, -0.702124, 0, 20.765104, -0.783236, 0, 20.662045, -0.746452, 0, 20.785027, -0.702124, 0, 20.676058, -0.682381, 0, 20.662045, -0.746452, 0, 20.785027, -0.702124, 0, 20.662045, -0.746452, 0, 20.676058, -0.682381, 0, 20.785027, -0.702124, 0, 20.791668, -0.607552, 0, 20.676058, -0.682381, 0, 20.785027, -0.702124, 0, 20.676058, -0.682381, 0, 20.791668, -0.607552, 0, 20.785027, -0.702124, 0, 20.791668, -0.607552, 0, 20.680731, -0.607552, 0, 20.676058, -0.682381, 0, 20.680731, -0.607552, 0, 20.791668, -0.607552, 0, 20.676058, -0.682381, 0, 20.791668, -0.607552, 0, 20.676058, -0.533195, 0, 20.680731, -0.607552, 0, 20.676058, -0.533195, 0, 20.791668, -0.607552, 0, 20.680731, -0.607552, 0, 20.785059, -0.513452, 0, 20.676058, -0.533195, 0, 20.791668, -0.607552, 0, 20.676058, -0.533195, 0, 20.785059, -0.513452, 0, 20.791668, -0.607552, 0, 20.785059, -0.513452, 0, 20.662045, -0.469499, 0, 20.676058, -0.533195, 0, 20.662045, -0.469499, 0, 20.785059, -0.513452, 0, 20.676058, -0.533195, 0, 20.765234, -0.432715, 0, 20.662045, -0.469499, 0, 20.785059, -0.513452, 0, 20.662045, -0.469499, 0, 20.765234, -0.432715, 0, 20.785059, -0.513452, 0, 20.732195, -0.365340, 0, 20.662045, -0.469499, 0, 20.765234, -0.432715, 0, 20.662045, -0.469499, 0, 20.732195, -0.365340, 0, 20.765234, -0.432715, 0, 20.638689, -0.416463, 0, 20.662045, -0.469499, 0, 20.732195, -0.365340, 0, 20.662045, -0.469499, 0, 20.638689, -0.416463, 0, 20.732195, -0.365340, 0, 20.685938, -0.311328, 0, 20.638689, -0.416463, 0, 20.732195, -0.365340, 0, 20.638689, -0.416463, 0, 20.685938, -0.311328, 0, 20.732195, -0.365340, 0, 20.605989, -0.374089, 0, 20.638689, -0.416463, 0, 20.685938, -0.311328, 0, 20.638689, -0.416463, 0, 20.605989, -0.374089, 0, 20.685938, -0.311328, 0, 20.625555, -0.269914, 0, 20.605989, -0.374089, 0, 20.685938, -0.311328, 0, 20.605989, -0.374089, 0, 20.625555, -0.269914, 0, 20.685938, -0.311328, 0, 20.563232, -0.341675, 0, 20.605989, -0.374089, 0, 20.625555, -0.269914, 0, 20.605989, -0.374089, 0, 20.563232, -0.341675, 0, 20.625555, -0.269914, 0, 20.550131, -0.240332, 0, 20.563232, -0.341675, 0, 20.625555, -0.269914, 0, 20.563232, -0.341675, 0, 20.550131, -0.240332, 0, 20.625555, -0.269914, 0, 20.550131, -0.240332, 0, 20.509701, -0.318522, 0, 20.563232, -0.341675, 0, 20.509701, -0.318522, 0, 20.550131, -0.240332, 0, 20.563232, -0.341675, 0, 20.459669, -0.222583, 0, 20.509701, -0.318522, 0, 20.550131, -0.240332, 0, 20.509701, -0.318522, 0, 20.459669, -0.222583, 0, 20.550131, -0.240332, 0, 20.459669, -0.222583, 0, 20.445395, -0.304631, 0, 20.509701, -0.318522, 0, 20.445395, -0.304631, 0, 20.459669, -0.222583, 0, 20.509701, -0.318522, 0, 20.459669, -0.222583, 0, 20.370314, -0.300000, 0, 20.445395, -0.304631, 0, 20.370314, -0.300000, 0, 20.459669, -0.222583, 0, 20.445395, -0.304631, 0, 20.459669, -0.222583, 0, 20.354168, -0.216667, 0, 20.370314, -0.300000, 0, 20.354168, -0.216667, 0, 20.459669, -0.222583, 0, 20.370314, -0.300000, 0, 20.354168, -0.216667, 0, 20.243231, -0.300000, 0, 20.370314, -0.300000, 0, 20.243231, -0.300000, 0, 20.354168, -0.216667, 0, 20.370314, -0.300000, 0, 20.954687, -0.216667, 0, 21.059896, -0.216667, 0, 20.954687, -0.216667, 0, 20.954687, -0.216667, 0, 21.059896, -0.216667, 0, 21.059896, -0.216667, 0, 21.059896, -0.216667, 0, 21.059896, -0.916667, 0, 21.059896, -0.216667, 0, 21.059896, -0.216667, 0, 21.059896, -0.916667, 0, 21.059896, -0.916667, 0, 21.059896, -0.916667, 0, 21.438543, -0.916667, 0, 21.059896, -0.916667, 0, 21.059896, -0.916667, 0, 21.438543, -0.916667, 0, 21.438543, -0.916667, 0, 21.438543, -0.916667, 0, 21.438543, -1, 0, 21.438543, -0.916667, 0, 21.438543, -0.916667, 0, 21.438543, -1, 0, 21.438543, -1, 0, 21.438543, -1, 0, 20.954687, -1, 0, 21.438543, -1, 0, 21.438543, -1, 0, 20.954687, -1, 0, 20.954687, -1, 0, 20.954687, -0.216667, 0, 21.059896, -0.916667, 0, 21.059896, -0.216667, 0, 21.059896, -0.916667, 0, 20.954687, -0.216667, 0, 21.059896, -0.216667, 0, 20.954687, -0.216667, 0, 20.954687, -1, 0, 21.059896, -0.916667, 0, 20.954687, -1, 0, 20.954687, -0.216667, 0, 21.059896, -0.916667, 0, 21.059896, -0.916667, 0, 20.954687, -1, 0, 21.438543, -1, 0, 20.954687, -1, 0, 21.059896, -0.916667, 0, 21.438543, -1, 0, 21.059896, -0.916667, 0, 21.438543, -1, 0, 21.438543, -0.916667, 0, 21.438543, -1, 0, 21.059896, -0.916667, 0, 21.438543, -0.916667, 0, 21.554688, -0.216667, 0, 21.659895, -0.216667, 0, 21.554688, -0.216667, 0, 21.554688, -0.216667, 0, 21.659895, -0.216667, 0, 21.659895, -0.216667, 0, 21.659895, -0.216667, 0, 21.659895, -0.916667, 0, 21.659895, -0.216667, 0, 21.659895, -0.216667, 0, 21.659895, -0.916667, 0, 21.659895, -0.916667, 0, 21.659895, -0.916667, 0, 22.038542, -0.916667, 0, 21.659895, -0.916667, 0, 21.659895, -0.916667, 0, 22.038542, -0.916667, 0, 22.038542, -0.916667, 0, 22.038542, -0.916667, 0, 22.038542, -1, 0, 22.038542, -0.916667, 0, 22.038542, -0.916667, 0, 22.038542, -1, 0, 22.038542, -1, 0, 22.038542, -1, 0, 21.554688, -1, 0, 22.038542, -1, 0, 22.038542, -1, 0, 21.554688, -1, 0, 21.554688, -1, 0, 21.554688, -0.216667, 0, 21.659895, -0.916667, 0, 21.659895, -0.216667, 0, 21.659895, -0.916667, 0, 21.554688, -0.216667, 0, 21.659895, -0.216667, 0, 21.554688, -0.216667, 0, 21.554688, -1, 0, 21.659895, -0.916667, 0, 21.554688, -1, 0, 21.554688, -0.216667, 0, 21.659895, -0.916667, 0, 21.659895, -0.916667, 0, 21.554688, -1, 0, 22.038542, -1, 0, 21.554688, -1, 0, 21.659895, -0.916667, 0, 22.038542, -1, 0, 21.659895, -0.916667, 0, 22.038542, -1, 0, 22.038542, -0.916667, 0, 22.038542, -1, 0, 21.659895, -0.916667, 0, 22.038542, -0.916667, 0 };
    Coordinate14.point = m_point;
    Coordinate14.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal15.m_parent = (X3dNode *)&Text1m_5;
    {
    static float m_vector[] = { 0.805553, 0.592524, 0, 0.805553, 0.592524, 0, 0.805553, 0.592524, 0, 0.805553, 0.592524, 0, 0.805553, 0.592524, 0, 0.805553, 0.592524, 0, 0.822890, 0.568201, 0, 0.822890, 0.568201, 0, 0.822890, 0.568201, 0, 0.822890, 0.568201, 0, 0.822890, 0.568201, 0, 0.822890, 0.568201, 0, 0.841383, 0.540440, 0, 0.841383, 0.540440, 0, 0.841383, 0.540440, 0, 0.841383, 0.540440, 0, 0.841383, 0.540440, 0, 0.841383, 0.540440, 0, 0.860794, 0.508953, 0, 0.860794, 0.508953, 0, 0.860794, 0.508953, 0, 0.860794, 0.508953, 0, 0.860794, 0.508953, 0, 0.860794, 0.508953, 0, 0.880949, 0.473212, 0, 0.880949, 0.473212, 0, 0.880949, 0.473212, 0, 0.880949, 0.473212, 0, 0.880949, 0.473212, 0, 0.880949, 0.473212, 0, 0.901541, 0.432693, 0, 0.901541, 0.432693, 0, 0.901541, 0.432693, 0, 0.901541, 0.432693, 0, 0.901541, 0.432693, 0, 0.901541, 0.432693, 0, 0.922021, 0.387140, 0, 0.922021, 0.387140, 0, 0.922021, 0.387140, 0, 0.922021, 0.387140, 0, 0.922021, 0.387140, 0, 0.922021, 0.387140, 0, 0.941724, 0.336388, 0, 0.941724, 0.336388, 0, 0.941724, 0.336388, 0, 0.941724, 0.336388, 0, 0.941724, 0.336388, 0, 0.941724, 0.336388, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.926424, 0.376481, 0, 0.926424, 0.376481, 0, 0.926424, 0.376481, 0, 0.926424, 0.376481, 0, 0.926424, 0.376481, 0, 0.926424, 0.376481, 0, 0.895023, 0.446021, 0, 0.895023, 0.446021, 0, 0.895023, 0.446021, 0, 0.895023, 0.446021, 0, 0.895023, 0.446021, 0, 0.895023, 0.446021, 0, 0.863872, 0.503711, 0, 0.863872, 0.503711, 0, 0.863872, 0.503711, 0, 0.863872, 0.503711, 0, 0.863872, 0.503711, 0, 0.863872, 0.503711, 0, 0.834784, 0.550577, 0, 0.834784, 0.550577, 0, 0.834784, 0.550577, 0, 0.834784, 0.550577, 0, 0.834784, 0.550577, 0, 0.834784, 0.550577, 0, 0.809048, 0.587742, 0, 0.809048, 0.587742, 0, 0.809048, 0.587742, 0, 0.809048, 0.587742, 0, 0.809048, 0.587742, 0, 0.809048, 0.587742, 0, 0.784639, 0.619953, 0, 0.784639, 0.619953, 0, 0.784639, 0.619953, 0, 0.784639, 0.619953, 0, 0.784639, 0.619953, 0, 0.784639, 0.619953, 0, 0.758473, 0.651705, 0, 0.758473, 0.651705, 0, 0.758473, 0.651705, 0, 0.758473, 0.651705, 0, 0.758473, 0.651705, 0, 0.758473, 0.651705, 0, 0.729785, 0.683677, 0, 0.729785, 0.683677, 0, 0.729785, 0.683677, 0, 0.729785, 0.683677, 0, 0.729785, 0.683677, 0, 0.729785, 0.683677, 0, 0.697526, 0.716560, 0, 0.697526, 0.716560, 0, 0.697526, 0.716560, 0, 0.697526, 0.716560, 0, 0.697526, 0.716560, 0, 0.697526, 0.716560, 0, 0.532848, 0.846211, 0, 0.532848, 0.846211, 0, 0.532848, 0.846211, 0, 0.532848, 0.846211, 0, 0.532848, 0.846211, 0, 0.532848, 0.846211, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.132466, 0.991188, 0, 0.132466, 0.991188, 0, 0.132466, 0.991188, 0, 0.132466, 0.991188, 0, 0.132466, 0.991188, 0, 0.132466, 0.991188, 0, 0.400307, 0.916381, 0, 0.400307, 0.916381, 0, 0.400307, 0.916381, 0, 0.400307, 0.916381, 0, 0.400307, 0.916381, 0, 0.400307, 0.916381, 0, 0.460554, 0.887632, 0, 0.460554, 0.887632, 0, 0.460554, 0.887632, 0, 0.460554, 0.887632, 0, 0.460554, 0.887632, 0, 0.460554, 0.887632, 0, 0.512024, 0.858971, 0, 0.512024, 0.858971, 0, 0.512024, 0.858971, 0, 0.512024, 0.858971, 0, 0.512024, 0.858971, 0, 0.512024, 0.858971, 0, 0.556897, 0.830581, 0, 0.556897, 0.830581, 0, 0.556897, 0.830581, 0, 0.556897, 0.830581, 0, 0.556897, 0.830581, 0, 0.556897, 0.830581, 0, 0.596916, 0.802304, 0, 0.596916, 0.802304, 0, 0.596916, 0.802304, 0, 0.596916, 0.802304, 0, 0.596916, 0.802304, 0, 0.596916, 0.802304, 0, 0.633632, 0.773635, 0, 0.633632, 0.773635, 0, 0.633632, 0.773635, 0, 0.633632, 0.773635, 0, 0.633632, 0.773635, 0, 0.633632, 0.773635, 0, 0.668362, 0.743836, 0, 0.668362, 0.743836, 0, 0.668362, 0.743836, 0, 0.668362, 0.743836, 0, 0.668362, 0.743836, 0, 0.668362, 0.743836, 0, 0.702159, 0.712020, 0, 0.702159, 0.712020, 0, 0.702159, 0.712020, 0, 0.702159, 0.712020, 0, 0.702159, 0.712020, 0, 0.702159, 0.712020, 0, 0.736025, 0.676954, 0, 0.736025, 0.676954, 0, 0.736025, 0.676954, 0, 0.736025, 0.676954, 0, 0.736025, 0.676954, 0, 0.736025, 0.676954, 0, 0.756458, 0.654043, 0, 0.756458, 0.654043, 0, 0.756458, 0.654043, 0, 0.756458, 0.654043, 0, 0.756458, 0.654043, 0, 0.756458, 0.654043, 0, 0.773246, 0.634106, 0, 0.773246, 0.634106, 0, 0.773246, 0.634106, 0, 0.773246, 0.634106, 0, 0.773246, 0.634106, 0, 0.773246, 0.634106, 0, 0.786420, 0.617693, 0, 0.786420, 0.617693, 0, 0.786420, 0.617693, 0, 0.786420, 0.617693, 0, 0.786420, 0.617693, 0, 0.786420, 0.617693, 0, 0.795942, 0.605372, 0, 0.795942, 0.605372, 0, 0.795942, 0.605372, 0, 0.795942, 0.605372, 0, 0.795942, 0.605372, 0, 0.795942, 0.605372, 0, 0.801550, 0.597928, 0, 0.801550, 0.597928, 0, 0.801550, 0.597928, 0, 0.801550, 0.597928, 0, 0.801550, 0.597928, 0, 0.801550, 0.597928, 0, 0.802792, 0.596260, 0, 0.802792, 0.596260, 0, 0.802792, 0.596260, 0, 0.802792, 0.596260, 0, 0.802792, 0.596260, 0, 0.802792, 0.596260, 0, 0.799098, 0.601201, 0, 0.799098, 0.601201, 0, 0.799098, 0.601201, 0, 0.799098, 0.601201, 0, 0.799098, 0.601201, 0, 0.799098, 0.601201, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.449891, 0.893084, 0, 0.449891, 0.893084, 0, 0.449891, 0.893084, 0, 0.449891, 0.893084, 0, 0.449891, 0.893084, 0, 0.449891, 0.893084, 0, 0.634657, 0.772794, 0, 0.634657, 0.772794, 0, 0.634657, 0.772794, 0, 0.634657, 0.772794, 0, 0.634657, 0.772794, 0, 0.634657, 0.772794, 0, 0.669799, 0.742543, 0, 0.669799, 0.742543, 0, 0.669799, 0.742543, 0, 0.669799, 0.742543, 0, 0.669799, 0.742543, 0, 0.669799, 0.742543, 0, 0.692764, 0.721165, 0, 0.692764, 0.721165, 0, 0.692764, 0.721165, 0, 0.692764, 0.721165, 0, 0.692764, 0.721165, 0, 0.692764, 0.721165, 0, 0.705766, 0.708445, 0, 0.705766, 0.708445, 0, 0.705766, 0.708445, 0, 0.705766, 0.708445, 0, 0.705766, 0.708445, 0, 0.705766, 0.708445, 0, 0.710439, 0.703759, 0, 0.710439, 0.703759, 0, 0.710439, 0.703759, 0, 0.710439, 0.703759, 0, 0.710439, 0.703759, 0, 0.710439, 0.703759, 0, 0.707939, 0.706274, 0, 0.707939, 0.706274, 0, 0.707939, 0.706274, 0, 0.707939, 0.706274, 0, 0.707939, 0.706274, 0, 0.707939, 0.706274, 0, 0.699034, 0.715089, 0, 0.699034, 0.715089, 0, 0.699034, 0.715089, 0, 0.699034, 0.715089, 0, 0.699034, 0.715089, 0, 0.699034, 0.715089, 0, 0.684183, 0.729310, 0, 0.684183, 0.729310, 0, 0.684183, 0.729310, 0, 0.684183, 0.729310, 0, 0.684183, 0.729310, 0, 0.684183, 0.729310, 0, 0.663671, 0.748024, 0, 0.663671, 0.748024, 0, 0.663671, 0.748024, 0, 0.663671, 0.748024, 0, 0.663671, 0.748024, 0, 0.663671, 0.748024, 0, 0.640418, 0.768026, 0, 0.640418, 0.768026, 0, 0.640418, 0.768026, 0, 0.640418, 0.768026, 0, 0.640418, 0.768026, 0, 0.640418, 0.768026, 0, 0.616856, 0.787076, 0, 0.616856, 0.787076, 0, 0.616856, 0.787076, 0, 0.616856, 0.787076, 0, 0.616856, 0.787076, 0, 0.616856, 0.787076, 0, 0.592466, 0.805595, 0, 0.592466, 0.805595, 0, 0.592466, 0.805595, 0, 0.592466, 0.805595, 0, 0.592466, 0.805595, 0, 0.592466, 0.805595, 0, 0.566714, 0.823915, 0, 0.566714, 0.823915, 0, 0.566714, 0.823915, 0, 0.566714, 0.823915, 0, 0.566714, 0.823915, 0, 0.566714, 0.823915, 0, 0.539141, 0.842215, 0, 0.539141, 0.842215, 0, 0.539141, 0.842215, 0, 0.539141, 0.842215, 0, 0.539141, 0.842215, 0, 0.539141, 0.842215, 0, 0.509222, 0.860635, 0, 0.509222, 0.860635, 0, 0.509222, 0.860635, 0, 0.509222, 0.860635, 0, 0.509222, 0.860635, 0, 0.509222, 0.860635, 0, 0.476203, 0.879335, 0, 0.476203, 0.879335, 0, 0.476203, 0.879335, 0, 0.476203, 0.879335, 0, 0.476203, 0.879335, 0, 0.476203, 0.879335, 0, 0.439174, 0.898402, 0, 0.439174, 0.898402, 0, 0.439174, 0.898402, 0, 0.439174, 0.898402, 0, 0.439174, 0.898402, 0, 0.439174, 0.898402, 0, 0.979070, 0.203526, 0, 0.979070, 0.203526, 0, 0.979070, 0.203526, 0, 0.979070, 0.203526, 0, 0.979070, 0.203526, 0, 0.979070, 0.203526, 0, 0.953351, 0.301865, 0, 0.953351, 0.301865, 0, 0.953351, 0.301865, 0, 0.953351, 0.301865, 0, 0.953351, 0.301865, 0, 0.953351, 0.301865, 0, 0.967768, 0.251844, 0, 0.967768, 0.251844, 0, 0.967768, 0.251844, 0, 0.967768, 0.251844, 0, 0.967768, 0.251844, 0, 0.967768, 0.251844, 0, 0.978657, 0.205501, 0, 0.978657, 0.205501, 0, 0.978657, 0.205501, 0, 0.978657, 0.205501, 0, 0.978657, 0.205501, 0, 0.978657, 0.205501, 0, 0.986397, 0.164381, 0, 0.986397, 0.164381, 0, 0.986397, 0.164381, 0, 0.986397, 0.164381, 0, 0.986397, 0.164381, 0, 0.986397, 0.164381, 0, 0.991571, 0.129566, 0, 0.991571, 0.129566, 0, 0.991571, 0.129566, 0, 0.991571, 0.129566, 0, 0.991571, 0.129566, 0, 0.991571, 0.129566, 0, 0.994787, 0.101976, 0, 0.994787, 0.101976, 0, 0.994787, 0.101976, 0, 0.994787, 0.101976, 0, 0.994787, 0.101976, 0, 0.994787, 0.101976, 0, 0.996632, 0.082006, 0, 0.996632, 0.082006, 0, 0.996632, 0.082006, 0, 0.996632, 0.082006, 0, 0.996632, 0.082006, 0, 0.996632, 0.082006, 0, 0.997595, 0.069317, 0, 0.997595, 0.069317, 0, 0.997595, 0.069317, 0, 0.997595, 0.069317, 0, 0.997595, 0.069317, 0, 0.997595, 0.069317, 0, 0.997992, 0.063333, 0, 0.997992, 0.063333, 0, 0.997992, 0.063333, 0, 0.997992, 0.063333, 0, 0.997992, 0.063333, 0, 0.997992, 0.063333, 0, 0.998048, 0.062451, 0, 0.998048, 0.062451, 0, 0.998048, 0.062451, 0, 0.998048, 0.062451, 0, 0.998048, 0.062451, 0, 0.998048, 0.062451, 0, 0.998011, 0.063041, 0, 0.998011, 0.063041, 0, 0.998011, 0.063041, 0, 0.998011, 0.063041, 0, 0.998011, 0.063041, 0, 0.998011, 0.063041, 0, 0.997885, 0.064999, 0, 0.997885, 0.064999, 0, 0.997885, 0.064999, 0, 0.997885, 0.064999, 0, 0.997885, 0.064999, 0, 0.997885, 0.064999, 0, 0.997670, 0.068232, 0, 0.997670, 0.068232, 0, 0.997670, 0.068232, 0, 0.997670, 0.068232, 0, 0.997670, 0.068232, 0, 0.997670, 0.068232, 0, 0.997358, 0.072646, 0, 0.997358, 0.072646, 0, 0.997358, 0.072646, 0, 0.997358, 0.072646, 0, 0.997358, 0.072646, 0, 0.997358, 0.072646, 0, 0.996941, 0.078154, 0, 0.996941, 0.078154, 0, 0.996941, 0.078154, 0, 0.996941, 0.078154, 0, 0.996941, 0.078154, 0, 0.996941, 0.078154, 0, 0.996408, 0.084683, 0, 0.996408, 0.084683, 0, 0.996408, 0.084683, 0, 0.996408, 0.084683, 0, 0.996408, 0.084683, 0, 0.996408, 0.084683, 0, 0.995744, 0.092168, 0, 0.995744, 0.092168, 0, 0.995744, 0.092168, 0, 0.995744, 0.092168, 0, 0.995744, 0.092168, 0, 0.995744, 0.092168, 0, 0.999650, 0.026437, 0, 0.999650, 0.026437, 0, 0.999650, 0.026437, 0, 0.999650, 0.026437, 0, 0.999650, 0.026437, 0, 0.999650, 0.026437, 0, 0.999844, 0.017678, 0, 0.999844, 0.017678, 0, 0.999844, 0.017678, 0, 0.999844, 0.017678, 0, 0.999844, 0.017678, 0, 0.999844, 0.017678, 0, 0.999952, 0.009753, 0, 0.999952, 0.009753, 0, 0.999952, 0.009753, 0, 0.999952, 0.009753, 0, 0.999952, 0.009753, 0, 0.999952, 0.009753, 0, 0.999996, 0.002714, 0, 0.999996, 0.002714, 0, 0.999996, 0.002714, 0, 0.999996, 0.002714, 0, 0.999996, 0.002714, 0, 0.999996, 0.002714, 0, 0.999994, -0.003383, 0, 0.999994, -0.003383, 0, 0.999994, -0.003383, 0, 0.999994, -0.003383, 0, 0.999994, -0.003383, 0, 0.999994, -0.003383, 0, 0.999965, -0.008416, 0, 0.999965, -0.008416, 0, 0.999965, -0.008416, 0, 0.999965, -0.008416, 0, 0.999965, -0.008416, 0, 0.999965, -0.008416, 0, 0.999925, -0.012249, 0, 0.999925, -0.012249, 0, 0.999925, -0.012249, 0, 0.999925, -0.012249, 0, 0.999925, -0.012249, 0, 0.999925, -0.012249, 0, 0.999891, -0.014794, 0, 0.999891, -0.014794, 0, 0.999891, -0.014794, 0, 0.999891, -0.014794, 0, 0.999891, -0.014794, 0, 0.999891, -0.014794, 0, 0.999873, -0.015950, 0, 0.999873, -0.015950, 0, 0.999873, -0.015950, 0, 0.999873, -0.015950, 0, 0.999873, -0.015950, 0, 0.999873, -0.015950, 0, 0.999930, -0.011808, 0, 0.999930, -0.011808, 0, 0.999930, -0.011808, 0, 0.999930, -0.011808, 0, 0.999930, -0.011808, 0, 0.999930, -0.011808, 0, 0.999992, 0.004106, 0, 0.999992, 0.004106, 0, 0.999992, 0.004106, 0, 0.999992, 0.004106, 0, 0.999992, 0.004106, 0, 0.999992, 0.004106, 0, 0.999430, 0.033767, 0, 0.999430, 0.033767, 0, 0.999430, 0.033767, 0, 0.999430, 0.033767, 0, 0.999430, 0.033767, 0, 0.999430, 0.033767, 0, 0.996891, 0.078793, 0, 0.996891, 0.078793, 0, 0.996891, 0.078793, 0, 0.996891, 0.078793, 0, 0.996891, 0.078793, 0, 0.996891, 0.078793, 0, 0.990466, 0.137757, 0, 0.990466, 0.137757, 0, 0.990466, 0.137757, 0, 0.990466, 0.137757, 0, 0.990466, 0.137757, 0, 0.990466, 0.137757, 0, 0.978346, 0.206979, 0, 0.978346, 0.206979, 0, 0.978346, 0.206979, 0, 0.978346, 0.206979, 0, 0.978346, 0.206979, 0, 0.978346, 0.206979, 0, 0.959195, 0.282747, 0, 0.959195, 0.282747, 0, 0.959195, 0.282747, 0, 0.959195, 0.282747, 0, 0.959195, 0.282747, 0, 0.959195, 0.282747, 0, 0.932923, 0.360076, 0, 0.932923, 0.360076, 0, 0.932923, 0.360076, 0, 0.932923, 0.360076, 0, 0.932923, 0.360076, 0, 0.932923, 0.360076, 0, 0.903263, 0.429088, 0, 0.903263, 0.429088, 0, 0.903263, 0.429088, 0, 0.903263, 0.429088, 0, 0.903263, 0.429088, 0, 0.903263, 0.429088, 0, 0.877680, 0.479246, 0, 0.877680, 0.479246, 0, 0.877680, 0.479246, 0, 0.877680, 0.479246, 0, 0.877680, 0.479246, 0, 0.877680, 0.479246, 0, 0.858502, 0.512810, 0, 0.858502, 0.512810, 0, 0.858502, 0.512810, 0, 0.858502, 0.512810, 0, 0.858502, 0.512810, 0, 0.858502, 0.512810, 0, 0.846529, 0.532342, 0, 0.846529, 0.532342, 0, 0.846529, 0.532342, 0, 0.846529, 0.532342, 0, 0.846529, 0.532342, 0, 0.846529, 0.532342, 0, 0.841834, 0.539737, 0, 0.841834, 0.539737, 0, 0.841834, 0.539737, 0, 0.841834, 0.539737, 0, 0.841834, 0.539737, 0, 0.841834, 0.539737, 0, 0.843860, 0.536563, 0, 0.843860, 0.536563, 0, 0.843860, 0.536563, 0, 0.843860, 0.536563, 0, 0.843860, 0.536563, 0, 0.843860, 0.536563, 0, 0.851529, 0.524308, 0, 0.851529, 0.524308, 0, 0.851529, 0.524308, 0, 0.851529, 0.524308, 0, 0.851529, 0.524308, 0, 0.851529, 0.524308, 0, 0.863684, 0.504034, 0, 0.863684, 0.504034, 0, 0.863684, 0.504034, 0, 0.863684, 0.504034, 0, 0.863684, 0.504034, 0, 0.863684, 0.504034, 0, 0.876514, 0.481375, 0, 0.876514, 0.481375, 0, 0.876514, 0.481375, 0, 0.876514, 0.481375, 0, 0.876514, 0.481375, 0, 0.876514, 0.481375, 0, 0.889884, 0.456186, 0, 0.889884, 0.456186, 0, 0.889884, 0.456186, 0, 0.889884, 0.456186, 0, 0.889884, 0.456186, 0, 0.889884, 0.456186, 0, 0.903762, 0.428036, 0, 0.903762, 0.428036, 0, 0.903762, 0.428036, 0, 0.903762, 0.428036, 0, 0.903762, 0.428036, 0, 0.903762, 0.428036, 0, 0.918067, 0.396425, 0, 0.918067, 0.396425, 0, 0.918067, 0.396425, 0, 0.918067, 0.396425, 0, 0.918067, 0.396425, 0, 0.918067, 0.396425, 0, 0.932197, 0.361952, 0, 0.932197, 0.361952, 0, 0.932197, 0.361952, 0, 0.932197, 0.361952, 0, 0.932197, 0.361952, 0, 0.932197, 0.361952, 0, 0.945677, 0.325108, 0, 0.945677, 0.325108, 0, 0.945677, 0.325108, 0, 0.945677, 0.325108, 0, 0.945677, 0.325108, 0, 0.945677, 0.325108, 0, 0.958490, 0.285126, 0, 0.958490, 0.285126, 0, 0.958490, 0.285126, 0, 0.958490, 0.285126, 0, 0.958490, 0.285126, 0, 0.958490, 0.285126, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.950714, 0.310069, 0, 0.950714, 0.310069, 0, 0.950714, 0.310069, 0, 0.950714, 0.310069, 0, 0.950714, 0.310069, 0, 0.950714, 0.310069, 0, 0.942109, 0.335308, 0, 0.942109, 0.335308, 0, 0.942109, 0.335308, 0, 0.942109, 0.335308, 0, 0.942109, 0.335308, 0, 0.942109, 0.335308, 0, 0.933367, 0.358922, 0, 0.933367, 0.358922, 0, 0.933367, 0.358922, 0, 0.933367, 0.358922, 0, 0.933367, 0.358922, 0, 0.933367, 0.358922, 0, 0.924473, 0.381248, 0, 0.924473, 0.381248, 0, 0.924473, 0.381248, 0, 0.924473, 0.381248, 0, 0.924473, 0.381248, 0, 0.924473, 0.381248, 0, 0.915769, 0.401705, 0, 0.915769, 0.401705, 0, 0.915769, 0.401705, 0, 0.915769, 0.401705, 0, 0.915769, 0.401705, 0, 0.915769, 0.401705, 0, 0.907687, 0.419648, 0, 0.907687, 0.419648, 0, 0.907687, 0.419648, 0, 0.907687, 0.419648, 0, 0.907687, 0.419648, 0, 0.907687, 0.419648, 0, 0.900307, 0.435256, 0, 0.900307, 0.435256, 0, 0.900307, 0.435256, 0, 0.900307, 0.435256, 0, 0.900307, 0.435256, 0, 0.900307, 0.435256, 0, 0.893702, 0.448660, 0, 0.893702, 0.448660, 0, 0.893702, 0.448660, 0, 0.893702, 0.448660, 0, 0.893702, 0.448660, 0, 0.893702, 0.448660, 0, 0.887737, 0.460350, 0, 0.887737, 0.460350, 0, 0.887737, 0.460350, 0, 0.887737, 0.460350, 0, 0.887737, 0.460350, 0, 0.887737, 0.460350, 0, 0.884131, 0.467238, 0, 0.884131, 0.467238, 0, 0.884131, 0.467238, 0, 0.884131, 0.467238, 0, 0.884131, 0.467238, 0, 0.884131, 0.467238, 0, 0.883251, 0.468900, 0, 0.883251, 0.468900, 0, 0.883251, 0.468900, 0, 0.883251, 0.468900, 0, 0.883251, 0.468900, 0, 0.883251, 0.468900, 0, 0.885395, 0.464839, 0, 0.885395, 0.464839, 0, 0.885395, 0.464839, 0, 0.885395, 0.464839, 0, 0.885395, 0.464839, 0, 0.885395, 0.464839, 0, 0.890680, 0.454631, 0, 0.890680, 0.454631, 0, 0.890680, 0.454631, 0, 0.890680, 0.454631, 0, 0.890680, 0.454631, 0, 0.890680, 0.454631, 0, 0.899059, 0.437827, 0, 0.899059, 0.437827, 0, 0.899059, 0.437827, 0, 0.899059, 0.437827, 0, 0.899059, 0.437827, 0, 0.899059, 0.437827, 0, 0.910356, 0.413825, 0, 0.910356, 0.413825, 0, 0.910356, 0.413825, 0, 0.910356, 0.413825, 0, 0.910356, 0.413825, 0, 0.910356, 0.413825, 0, 0.924135, 0.382067, 0, 0.924135, 0.382067, 0, 0.924135, 0.382067, 0, 0.924135, 0.382067, 0, 0.924135, 0.382067, 0, 0.924135, 0.382067, 0, 0.968190, 0.250216, 0, 0.968190, 0.250216, 0, 0.968190, 0.250216, 0, 0.968190, 0.250216, 0, 0.968190, 0.250216, 0, 0.968190, 0.250216, 0, 0.965921, 0.258838, 0, 0.965921, 0.258838, 0, 0.965921, 0.258838, 0, 0.965921, 0.258838, 0, 0.965921, 0.258838, 0, 0.965921, 0.258838, 0, 0.963736, 0.266859, 0, 0.963736, 0.266859, 0, 0.963736, 0.266859, 0, 0.963736, 0.266859, 0, 0.963736, 0.266859, 0, 0.963736, 0.266859, 0, 0.961651, 0.274275, 0, 0.961651, 0.274275, 0, 0.961651, 0.274275, 0, 0.961651, 0.274275, 0, 0.961651, 0.274275, 0, 0.961651, 0.274275, 0, 0.959684, 0.281081, 0, 0.959684, 0.281081, 0, 0.959684, 0.281081, 0, 0.959684, 0.281081, 0, 0.959684, 0.281081, 0, 0.959684, 0.281081, 0, 0.957850, 0.287270, 0, 0.957850, 0.287270, 0, 0.957850, 0.287270, 0, 0.957850, 0.287270, 0, 0.957850, 0.287270, 0, 0.957850, 0.287270, 0, 0.956163, 0.292833, 0, 0.956163, 0.292833, 0, 0.956163, 0.292833, 0, 0.956163, 0.292833, 0, 0.956163, 0.292833, 0, 0.956163, 0.292833, 0, 0.954641, 0.297759, 0, 0.954641, 0.297759, 0, 0.954641, 0.297759, 0, 0.954641, 0.297759, 0, 0.954641, 0.297759, 0, 0.954641, 0.297759, 0, 0.953297, 0.302035, 0, 0.953297, 0.302035, 0, 0.953297, 0.302035, 0, 0.953297, 0.302035, 0, 0.953297, 0.302035, 0, 0.953297, 0.302035, 0, 0.951709, 0.307002, 0, 0.951709, 0.307002, 0, 0.951709, 0.307002, 0, 0.951709, 0.307002, 0, 0.951709, 0.307002, 0, 0.951709, 0.307002, 0, 0.950801, 0.309803, 0, 0.950801, 0.309803, 0, 0.950801, 0.309803, 0, 0.950801, 0.309803, 0, 0.950801, 0.309803, 0, 0.950801, 0.309803, 0, 0.950635, 0.310310, 0, 0.950635, 0.310310, 0, 0.950635, 0.310310, 0, 0.950635, 0.310310, 0, 0.950635, 0.310310, 0, 0.950635, 0.310310, 0, 0.951253, 0.308413, 0, 0.951253, 0.308413, 0, 0.951253, 0.308413, 0, 0.951253, 0.308413, 0, 0.951253, 0.308413, 0, 0.951253, 0.308413, 0, 0.952666, 0.304018, 0, 0.952666, 0.304018, 0, 0.952666, 0.304018, 0, 0.952666, 0.304018, 0, 0.952666, 0.304018, 0, 0.952666, 0.304018, 0, 0.954861, 0.297052, 0, 0.954861, 0.297052, 0, 0.954861, 0.297052, 0, 0.954861, 0.297052, 0, 0.954861, 0.297052, 0, 0.954861, 0.297052, 0, 0.957790, 0.287470, 0, 0.957790, 0.287470, 0, 0.957790, 0.287470, 0, 0.957790, 0.287470, 0, 0.957790, 0.287470, 0, 0.957790, 0.287470, 0, 0.961371, 0.275254, 0, 0.961371, 0.275254, 0, 0.961371, 0.275254, 0, 0.961371, 0.275254, 0, 0.961371, 0.275254, 0, 0.961371, 0.275254, 0, 0.964185, 0.265229, 0, 0.964185, 0.265229, 0, 0.964185, 0.265229, 0, 0.964185, 0.265229, 0, 0.964185, 0.265229, 0, 0.964185, 0.265229, 0, 0.966748, 0.255733, 0, 0.966748, 0.255733, 0, 0.966748, 0.255733, 0, 0.966748, 0.255733, 0, 0.966748, 0.255733, 0, 0.966748, 0.255733, 0, 0.969057, 0.246836, 0, 0.969057, 0.246836, 0, 0.969057, 0.246836, 0, 0.969057, 0.246836, 0, 0.969057, 0.246836, 0, 0.969057, 0.246836, 0, 0.971117, 0.238602, 0, 0.971117, 0.238602, 0, 0.971117, 0.238602, 0, 0.971117, 0.238602, 0, 0.971117, 0.238602, 0, 0.971117, 0.238602, 0, 0.973101, 0.230378, 0, 0.973101, 0.230378, 0, 0.973101, 0.230378, 0, 0.973101, 0.230378, 0, 0.973101, 0.230378, 0, 0.973101, 0.230378, 0, 0.975148, 0.221554, 0, 0.975148, 0.221554, 0, 0.975148, 0.221554, 0, 0.975148, 0.221554, 0, 0.975148, 0.221554, 0, 0.975148, 0.221554, 0, 0.977215, 0.212253, 0, 0.977215, 0.212253, 0, 0.977215, 0.212253, 0, 0.977215, 0.212253, 0, 0.977215, 0.212253, 0, 0.977215, 0.212253, 0, 0.979260, 0.202606, 0, 0.979260, 0.202606, 0, 0.979260, 0.202606, 0, 0.979260, 0.202606, 0, 0.979260, 0.202606, 0, 0.979260, 0.202606, 0, 0.980889, 0.194567, 0, 0.980889, 0.194567, 0, 0.980889, 0.194567, 0, 0.980889, 0.194567, 0, 0.980889, 0.194567, 0, 0.980889, 0.194567, 0, 0.983173, 0.182676, 0, 0.983173, 0.182676, 0, 0.983173, 0.182676, 0, 0.983173, 0.182676, 0, 0.983173, 0.182676, 0, 0.983173, 0.182676, 0, 0.985296, 0.170854, 0, 0.985296, 0.170854, 0, 0.985296, 0.170854, 0, 0.985296, 0.170854, 0, 0.985296, 0.170854, 0, 0.985296, 0.170854, 0, 0.987287, 0.158949, 0, 0.987287, 0.158949, 0, 0.987287, 0.158949, 0, 0.987287, 0.158949, 0, 0.987287, 0.158949, 0, 0.987287, 0.158949, 0, 0.989164, 0.146817, 0, 0.989164, 0.146817, 0, 0.989164, 0.146817, 0, 0.989164, 0.146817, 0, 0.989164, 0.146817, 0, 0.989164, 0.146817, 0, 0.991010, 0.133789, 0, 0.991010, 0.133789, 0, 0.991010, 0.133789, 0, 0.991010, 0.133789, 0, 0.991010, 0.133789, 0, 0.991010, 0.133789, 0, 0.992867, 0.119228, 0, 0.992867, 0.119228, 0, 0.992867, 0.119228, 0, 0.992867, 0.119228, 0, 0.992867, 0.119228, 0, 0.992867, 0.119228, 0, 0.994679, 0.103021, 0, 0.994679, 0.103021, 0, 0.994679, 0.103021, 0, 0.994679, 0.103021, 0, 0.994679, 0.103021, 0, 0.994679, 0.103021, 0, 0.996378, 0.085033, 0, 0.996378, 0.085033, 0, 0.996378, 0.085033, 0, 0.996378, 0.085033, 0, 0.996378, 0.085033, 0, 0.996378, 0.085033, 0, 0.997904, 0.064717, 0, 0.997904, 0.064717, 0, 0.997904, 0.064717, 0, 0.997904, 0.064717, 0, 0.997904, 0.064717, 0, 0.997904, 0.064717, 0, 0.998909, 0.046697, 0, 0.998909, 0.046697, 0, 0.998909, 0.046697, 0, 0.998909, 0.046697, 0, 0.998909, 0.046697, 0, 0.998909, 0.046697, 0, 0.999525, 0.030819, 0, 0.999525, 0.030819, 0, 0.999525, 0.030819, 0, 0.999525, 0.030819, 0, 0.999525, 0.030819, 0, 0.999525, 0.030819, 0, 0.999856, 0.016965, 0, 0.999856, 0.016965, 0, 0.999856, 0.016965, 0, 0.999856, 0.016965, 0, 0.999856, 0.016965, 0, 0.999856, 0.016965, 0, 0.999985, 0.005541, 0, 0.999985, 0.005541, 0, 0.999985, 0.005541, 0, 0.999985, 0.005541, 0, 0.999985, 0.005541, 0, 0.999985, 0.005541, 0, 0.999996, -0.002936, 0, 0.999996, -0.002936, 0, 0.999996, -0.002936, 0, 0.999996, -0.002936, 0, 0.999996, -0.002936, 0, 0.999996, -0.002936, 0, 0.999965, -0.008335, 0, 0.999965, -0.008335, 0, 0.999965, -0.008335, 0, 0.999965, -0.008335, 0, 0.999965, -0.008335, 0, 0.999965, -0.008335, 0, 0.999945, -0.010443, 0, 0.999945, -0.010443, 0, 0.999945, -0.010443, 0, 0.999945, -0.010443, 0, 0.999945, -0.010443, 0, 0.999945, -0.010443, 0, 0.999947, -0.010269, 0, 0.999947, -0.010269, 0, 0.999947, -0.010269, 0, 0.999947, -0.010269, 0, 0.999947, -0.010269, 0, 0.999947, -0.010269, 0, 0.999956, -0.009420, 0, 0.999956, -0.009420, 0, 0.999956, -0.009420, 0, 0.999956, -0.009420, 0, 0.999956, -0.009420, 0, 0.999956, -0.009420, 0, 0.999969, -0.007864, 0, 0.999969, -0.007864, 0, 0.999969, -0.007864, 0, 0.999969, -0.007864, 0, 0.999969, -0.007864, 0, 0.999969, -0.007864, 0, 0.999985, -0.005562, 0, 0.999985, -0.005562, 0, 0.999985, -0.005562, 0, 0.999985, -0.005562, 0, 0.999985, -0.005562, 0, 0.999985, -0.005562, 0, 0.999997, -0.002476, 0, 0.999997, -0.002476, 0, 0.999997, -0.002476, 0, 0.999997, -0.002476, 0, 0.999997, -0.002476, 0, 0.999997, -0.002476, 0, 0.999999, 0.001441, 0, 0.999999, 0.001441, 0, 0.999999, 0.001441, 0, 0.999999, 0.001441, 0, 0.999999, 0.001441, 0, 0.999999, 0.001441, 0, 0.999981, 0.006240, 0, 0.999981, 0.006240, 0, 0.999981, 0.006240, 0, 0.999981, 0.006240, 0, 0.999981, 0.006240, 0, 0.999981, 0.006240, 0, 0.999928, 0.011980, 0, 0.999928, 0.011980, 0, 0.999928, 0.011980, 0, 0.999928, 0.011980, 0, 0.999928, 0.011980, 0, 0.999928, 0.011980, 0, 0.996502, 0.083570, 0, 0.996502, 0.083570, 0, 0.996502, 0.083570, 0, 0.996502, 0.083570, 0, 0.996502, 0.083570, 0, 0.996502, 0.083570, 0, 0.997271, 0.073830, 0, 0.997271, 0.073830, 0, 0.997271, 0.073830, 0, 0.997271, 0.073830, 0, 0.997271, 0.073830, 0, 0.997271, 0.073830, 0, 0.997852, 0.065510, 0, 0.997852, 0.065510, 0, 0.997852, 0.065510, 0, 0.997852, 0.065510, 0, 0.997852, 0.065510, 0, 0.997852, 0.065510, 0, 0.998285, 0.058539, 0, 0.998285, 0.058539, 0, 0.998285, 0.058539, 0, 0.998285, 0.058539, 0, 0.998285, 0.058539, 0, 0.998285, 0.058539, 0, 0.998602, 0.052851, 0, 0.998602, 0.052851, 0, 0.998602, 0.052851, 0, 0.998602, 0.052851, 0, 0.998602, 0.052851, 0, 0.998602, 0.052851, 0, 0.998829, 0.048379, 0, 0.998829, 0.048379, 0, 0.998829, 0.048379, 0, 0.998829, 0.048379, 0, 0.998829, 0.048379, 0, 0.998829, 0.048379, 0, 0.998984, 0.045064, 0, 0.998984, 0.045064, 0, 0.998984, 0.045064, 0, 0.998984, 0.045064, 0, 0.998984, 0.045064, 0, 0.998984, 0.045064, 0, 0.999082, 0.042850, 0, 0.999082, 0.042850, 0, 0.999082, 0.042850, 0, 0.999082, 0.042850, 0, 0.999082, 0.042850, 0, 0.999082, 0.042850, 0, 0.999131, 0.041685, 0, 0.999131, 0.041685, 0, 0.999131, 0.041685, 0, 0.999131, 0.041685, 0, 0.999131, 0.041685, 0, 0.999131, 0.041685, 0, 0.999129, 0.041722, 0, 0.999129, 0.041722, 0, 0.999129, 0.041722, 0, 0.999129, 0.041722, 0, 0.999129, 0.041722, 0, 0.999129, 0.041722, 0, 0.999052, 0.043530, 0, 0.999052, 0.043530, 0, 0.999052, 0.043530, 0, 0.999052, 0.043530, 0, 0.999052, 0.043530, 0, 0.999052, 0.043530, 0, 0.998893, 0.047032, 0, 0.998893, 0.047032, 0, 0.998893, 0.047032, 0, 0.998893, 0.047032, 0, 0.998893, 0.047032, 0, 0.998893, 0.047032, 0, 0.998638, 0.052178, 0, 0.998638, 0.052178, 0, 0.998638, 0.052178, 0, 0.998638, 0.052178, 0, 0.998638, 0.052178, 0, 0.998638, 0.052178, 0, 0.998269, 0.058810, 0, 0.998269, 0.058810, 0, 0.998269, 0.058810, 0, 0.998269, 0.058810, 0, 0.998269, 0.058810, 0, 0.998269, 0.058810, 0, 0.997766, 0.066802, 0, 0.997766, 0.066802, 0, 0.997766, 0.066802, 0, 0.997766, 0.066802, 0, 0.997766, 0.066802, 0, 0.997766, 0.066802, 0, 0.997094, 0.076182, 0, 0.997094, 0.076182, 0, 0.997094, 0.076182, 0, 0.997094, 0.076182, 0, 0.997094, 0.076182, 0, 0.997094, 0.076182, 0, 0.996209, 0.086997, 0, 0.996209, 0.086997, 0, 0.996209, 0.086997, 0, 0.996209, 0.086997, 0, 0.996209, 0.086997, 0, 0.996209, 0.086997, 0, 0.995273, 0.097116, 0, 0.995273, 0.097116, 0, 0.995273, 0.097116, 0, 0.995273, 0.097116, 0, 0.995273, 0.097116, 0, 0.995273, 0.097116, 0, 0.994346, 0.106189, 0, 0.994346, 0.106189, 0, 0.994346, 0.106189, 0, 0.994346, 0.106189, 0, 0.994346, 0.106189, 0, 0.994346, 0.106189, 0, 0.993451, 0.114263, 0, 0.993451, 0.114263, 0, 0.993451, 0.114263, 0, 0.993451, 0.114263, 0, 0.993451, 0.114263, 0, 0.993451, 0.114263, 0, 0.992606, 0.121377, 0, 0.992606, 0.121377, 0, 0.992606, 0.121377, 0, 0.992606, 0.121377, 0, 0.992606, 0.121377, 0, 0.992606, 0.121377, 0, 0.991735, 0.128301, 0, 0.991735, 0.128301, 0, 0.991735, 0.128301, 0, 0.991735, 0.128301, 0, 0.991735, 0.128301, 0, 0.991735, 0.128301, 0, 0.990729, 0.135856, 0, 0.990729, 0.135856, 0, 0.990729, 0.135856, 0, 0.990729, 0.135856, 0, 0.990729, 0.135856, 0, 0.990729, 0.135856, 0, 0.989552, 0.144173, 0, 0.989552, 0.144173, 0, 0.989552, 0.144173, 0, 0.989552, 0.144173, 0, 0.989552, 0.144173, 0, 0.989552, 0.144173, 0, 0.988162, 0.153413, 0, 0.988162, 0.153413, 0, 0.988162, 0.153413, 0, 0.988162, 0.153413, 0, 0.988162, 0.153413, 0, 0.988162, 0.153413, 0, 0.986887, 0.161413, 0, 0.986887, 0.161413, 0, 0.986887, 0.161413, 0, 0.986887, 0.161413, 0, 0.986887, 0.161413, 0, 0.986887, 0.161413, 0, 0.985000, 0.172553, 0, 0.985000, 0.172553, 0, 0.985000, 0.172553, 0, 0.985000, 0.172553, 0, 0.985000, 0.172553, 0, 0.985000, 0.172553, 0, 0.982713, 0.185135, 0, 0.982713, 0.185135, 0, 0.982713, 0.185135, 0, 0.982713, 0.185135, 0, 0.982713, 0.185135, 0, 0.982713, 0.185135, 0, 0.979984, 0.199075, 0, 0.979984, 0.199075, 0, 0.979984, 0.199075, 0, 0.979984, 0.199075, 0, 0.979984, 0.199075, 0, 0.979984, 0.199075, 0, 0.976782, 0.214236, 0, 0.976782, 0.214236, 0, 0.976782, 0.214236, 0, 0.976782, 0.214236, 0, 0.976782, 0.214236, 0, 0.976782, 0.214236, 0, 0.972974, 0.230916, 0, 0.972974, 0.230916, 0, 0.972974, 0.230916, 0, 0.972974, 0.230916, 0, 0.972974, 0.230916, 0, 0.972974, 0.230916, 0, 0.968403, 0.249392, 0, 0.968403, 0.249392, 0, 0.968403, 0.249392, 0, 0.968403, 0.249392, 0, 0.968403, 0.249392, 0, 0.968403, 0.249392, 0, 0.963022, 0.269423, 0, 0.963022, 0.269423, 0, 0.963022, 0.269423, 0, 0.963022, 0.269423, 0, 0.963022, 0.269423, 0, 0.963022, 0.269423, 0, 0.956808, 0.290721, 0, 0.956808, 0.290721, 0, 0.956808, 0.290721, 0, 0.956808, 0.290721, 0, 0.956808, 0.290721, 0, 0.956808, 0.290721, 0, 0.949229, 0.314586, 0, 0.949229, 0.314586, 0, 0.949229, 0.314586, 0, 0.949229, 0.314586, 0, 0.949229, 0.314586, 0, 0.949229, 0.314586, 0, 0.942796, 0.333370, 0, 0.942796, 0.333370, 0, 0.942796, 0.333370, 0, 0.942796, 0.333370, 0, 0.942796, 0.333370, 0, 0.942796, 0.333370, 0, 0.937805, 0.347161, 0, 0.937805, 0.347161, 0, 0.937805, 0.347161, 0, 0.937805, 0.347161, 0, 0.937805, 0.347161, 0, 0.937805, 0.347161, 0, 0.934431, 0.356145, 0, 0.934431, 0.356145, 0, 0.934431, 0.356145, 0, 0.934431, 0.356145, 0, 0.934431, 0.356145, 0, 0.934431, 0.356145, 0, 0.932786, 0.360431, 0, 0.932786, 0.360431, 0, 0.932786, 0.360431, 0, 0.932786, 0.360431, 0, 0.932786, 0.360431, 0, 0.932786, 0.360431, 0, 0.932879, 0.360190, 0, 0.932879, 0.360190, 0, 0.932879, 0.360190, 0, 0.932879, 0.360190, 0, 0.932879, 0.360190, 0, 0.932879, 0.360190, 0, 0.934580, 0.355752, 0, 0.934580, 0.355752, 0, 0.934580, 0.355752, 0, 0.934580, 0.355752, 0, 0.934580, 0.355752, 0, 0.934580, 0.355752, 0, 0.937698, 0.347452, 0, 0.937698, 0.347452, 0, 0.937698, 0.347452, 0, 0.937698, 0.347452, 0, 0.937698, 0.347452, 0, 0.937698, 0.347452, 0, 0.939633, 0.342185, 0, 0.939633, 0.342185, 0, 0.939633, 0.342185, 0, 0.939633, 0.342185, 0, 0.939633, 0.342185, 0, 0.939633, 0.342185, 0, 0.941715, 0.336411, 0, 0.941715, 0.336411, 0, 0.941715, 0.336411, 0, 0.941715, 0.336411, 0, 0.941715, 0.336411, 0, 0.941715, 0.336411, 0, 0.943934, 0.330134, 0, 0.943934, 0.330134, 0, 0.943934, 0.330134, 0, 0.943934, 0.330134, 0, 0.943934, 0.330134, 0, 0.943934, 0.330134, 0, 0.946277, 0.323357, 0, 0.946277, 0.323357, 0, 0.946277, 0.323357, 0, 0.946277, 0.323357, 0, 0.946277, 0.323357, 0, 0.946277, 0.323357, 0, 0.948729, 0.316090, 0, 0.948729, 0.316090, 0, 0.948729, 0.316090, 0, 0.948729, 0.316090, 0, 0.948729, 0.316090, 0, 0.948729, 0.316090, 0, 0.951276, 0.308340, 0, 0.951276, 0.308340, 0, 0.951276, 0.308340, 0, 0.951276, 0.308340, 0, 0.951276, 0.308340, 0, 0.951276, 0.308340, 0, 0.953907, 0.300102, 0, 0.953907, 0.300102, 0, 0.953907, 0.300102, 0, 0.953907, 0.300102, 0, 0.953907, 0.300102, 0, 0.953907, 0.300102, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.962841, 0.270068, 0, 0.962841, 0.270068, 0, 0.962841, 0.270068, 0, 0.962841, 0.270068, 0, 0.962841, 0.270068, 0, 0.962841, 0.270068, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.930805, 0.365515, 0, 0.930805, 0.365515, 0, 0.930805, 0.365515, 0, 0.930805, 0.365515, 0, 0.930805, 0.365515, 0, 0.930805, 0.365515, 0, 0.947754, 0.319003, 0, 0.947754, 0.319003, 0, 0.947754, 0.319003, 0, 0.947754, 0.319003, 0, 0.947754, 0.319003, 0, 0.947754, 0.319003, 0, 0.943018, 0.332742, 0, 0.943018, 0.332742, 0, 0.943018, 0.332742, 0, 0.943018, 0.332742, 0, 0.943018, 0.332742, 0, 0.943018, 0.332742, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.984910, 0.173069, 0, 0.984910, 0.173069, 0, 0.984910, 0.173069, 0, 0.984910, 0.173069, 0, 0.984910, 0.173069, 0, 0.984910, 0.173069, 0, 0.983919, 0.178616, 0, 0.983919, 0.178616, 0, 0.983919, 0.178616, 0, 0.983919, 0.178616, 0, 0.983919, 0.178616, 0, 0.983919, 0.178616, 0, 0.982984, 0.183691, 0, 0.982984, 0.183691, 0, 0.982984, 0.183691, 0, 0.982984, 0.183691, 0, 0.982984, 0.183691, 0, 0.982984, 0.183691, 0, 0.982113, 0.188290, 0, 0.982113, 0.188290, 0, 0.982113, 0.188290, 0, 0.982113, 0.188290, 0, 0.982113, 0.188290, 0, 0.982113, 0.188290, 0, 0.981315, 0.192407, 0, 0.981315, 0.192407, 0, 0.981315, 0.192407, 0, 0.981315, 0.192407, 0, 0.981315, 0.192407, 0, 0.981315, 0.192407, 0, 0.980597, 0.196034, 0, 0.980597, 0.196034, 0, 0.980597, 0.196034, 0, 0.980597, 0.196034, 0, 0.980597, 0.196034, 0, 0.980597, 0.196034, 0, 0.979966, 0.199164, 0, 0.979966, 0.199164, 0, 0.979966, 0.199164, 0, 0.979966, 0.199164, 0, 0.979966, 0.199164, 0, 0.979966, 0.199164, 0, 0.979429, 0.201789, 0, 0.979429, 0.201789, 0, 0.979429, 0.201789, 0, 0.979429, 0.201789, 0, 0.979429, 0.201789, 0, 0.979429, 0.201789, 0, 0.978992, 0.203897, 0, 0.978992, 0.203897, 0, 0.978992, 0.203897, 0, 0.978992, 0.203897, 0, 0.978992, 0.203897, 0, 0.978992, 0.203897, 0, 0.978547, 0.206023, 0, 0.978547, 0.206023, 0, 0.978547, 0.206023, 0, 0.978547, 0.206023, 0, 0.978547, 0.206023, 0, 0.978547, 0.206023, 0, 0.978396, 0.206742, 0, 0.978396, 0.206742, 0, 0.978396, 0.206742, 0, 0.978396, 0.206742, 0, 0.978396, 0.206742, 0, 0.978396, 0.206742, 0, 0.978552, 0.205999, 0, 0.978552, 0.205999, 0, 0.978552, 0.205999, 0, 0.978552, 0.205999, 0, 0.978552, 0.205999, 0, 0.978552, 0.205999, 0, 0.979022, 0.203753, 0, 0.979022, 0.203753, 0, 0.979022, 0.203753, 0, 0.979022, 0.203753, 0, 0.979022, 0.203753, 0, 0.979022, 0.203753, 0, 0.979802, 0.199969, 0, 0.979802, 0.199969, 0, 0.979802, 0.199969, 0, 0.979802, 0.199969, 0, 0.979802, 0.199969, 0, 0.979802, 0.199969, 0, 0.980877, 0.194630, 0, 0.980877, 0.194630, 0, 0.980877, 0.194630, 0, 0.980877, 0.194630, 0, 0.980877, 0.194630, 0, 0.980877, 0.194630, 0, 0.982221, 0.187728, 0, 0.982221, 0.187728, 0, 0.982221, 0.187728, 0, 0.982221, 0.187728, 0, 0.982221, 0.187728, 0, 0.982221, 0.187728, 0, 0.983800, 0.179271, 0, 0.983800, 0.179271, 0, 0.983800, 0.179271, 0, 0.983800, 0.179271, 0, 0.983800, 0.179271, 0, 0.983800, 0.179271, 0, 0.985007, 0.172516, 0, 0.985007, 0.172516, 0, 0.985007, 0.172516, 0, 0.985007, 0.172516, 0, 0.985007, 0.172516, 0, 0.985007, 0.172516, 0, 0.986080, 0.166273, 0, 0.986080, 0.166273, 0, 0.986080, 0.166273, 0, 0.986080, 0.166273, 0, 0.986080, 0.166273, 0, 0.986080, 0.166273, 0, 0.987025, 0.160568, 0, 0.987025, 0.160568, 0, 0.987025, 0.160568, 0, 0.987025, 0.160568, 0, 0.987025, 0.160568, 0, 0.987025, 0.160568, 0, 0.987848, 0.155426, 0, 0.987848, 0.155426, 0, 0.987848, 0.155426, 0, 0.987848, 0.155426, 0, 0.987848, 0.155426, 0, 0.987848, 0.155426, 0, 0.988618, 0.150445, 0, 0.988618, 0.150445, 0, 0.988618, 0.150445, 0, 0.988618, 0.150445, 0, 0.988618, 0.150445, 0, 0.988618, 0.150445, 0, 0.989396, 0.145242, 0, 0.989396, 0.145242, 0, 0.989396, 0.145242, 0, 0.989396, 0.145242, 0, 0.989396, 0.145242, 0, 0.989396, 0.145242, 0, 0.990171, 0.139864, 0, 0.990171, 0.139864, 0, 0.990171, 0.139864, 0, 0.990171, 0.139864, 0, 0.990171, 0.139864, 0, 0.990171, 0.139864, 0, 0.990933, 0.134357, 0, 0.990933, 0.134357, 0, 0.990933, 0.134357, 0, 0.990933, 0.134357, 0, 0.990933, 0.134357, 0, 0.990933, 0.134357, 0, 0.991543, 0.129781, 0, 0.991543, 0.129781, 0, 0.991543, 0.129781, 0, 0.991543, 0.129781, 0, 0.991543, 0.129781, 0, 0.991543, 0.129781, 0, 0.992426, 0.122841, 0, 0.992426, 0.122841, 0, 0.992426, 0.122841, 0, 0.992426, 0.122841, 0, 0.992426, 0.122841, 0, 0.992426, 0.122841, 0, 0.993292, 0.115632, 0, 0.993292, 0.115632, 0, 0.993292, 0.115632, 0, 0.993292, 0.115632, 0, 0.993292, 0.115632, 0, 0.993292, 0.115632, 0, 0.994140, 0.108104, 0, 0.994140, 0.108104, 0, 0.994140, 0.108104, 0, 0.994140, 0.108104, 0, 0.994140, 0.108104, 0, 0.994140, 0.108104, 0, 0.994967, 0.100202, 0, 0.994967, 0.100202, 0, 0.994967, 0.100202, 0, 0.994967, 0.100202, 0, 0.994967, 0.100202, 0, 0.994967, 0.100202, 0, 0.995803, 0.091523, 0, 0.995803, 0.091523, 0, 0.995803, 0.091523, 0, 0.995803, 0.091523, 0, 0.995803, 0.091523, 0, 0.995803, 0.091523, 0, 0.996659, 0.081671, 0, 0.996659, 0.081671, 0, 0.996659, 0.081671, 0, 0.996659, 0.081671, 0, 0.996659, 0.081671, 0, 0.996659, 0.081671, 0, 0.997505, 0.070599, 0, 0.997505, 0.070599, 0, 0.997505, 0.070599, 0, 0.997505, 0.070599, 0, 0.997505, 0.070599, 0, 0.997505, 0.070599, 0, 0.998302, 0.058246, 0, 0.998302, 0.058246, 0, 0.998302, 0.058246, 0, 0.998302, 0.058246, 0, 0.998302, 0.058246, 0, 0.998302, 0.058246, 0, 0.999020, 0.044263, 0, 0.999020, 0.044263, 0, 0.999020, 0.044263, 0, 0.999020, 0.044263, 0, 0.999020, 0.044263, 0, 0.999020, 0.044263, 0, 0.999493, 0.031855, 0, 0.999493, 0.031855, 0, 0.999493, 0.031855, 0, 0.999493, 0.031855, 0, 0.999493, 0.031855, 0, 0.999493, 0.031855, 0, 0.999781, 0.020942, 0, 0.999781, 0.020942, 0, 0.999781, 0.020942, 0, 0.999781, 0.020942, 0, 0.999781, 0.020942, 0, 0.999781, 0.020942, 0, 0.999934, 0.011468, 0, 0.999934, 0.011468, 0, 0.999934, 0.011468, 0, 0.999934, 0.011468, 0, 0.999934, 0.011468, 0, 0.999934, 0.011468, 0, 0.999993, 0.003721, 0, 0.999993, 0.003721, 0, 0.999993, 0.003721, 0, 0.999993, 0.003721, 0, 0.999993, 0.003721, 0, 0.999993, 0.003721, 0, 0.999998, -0.001956, 0, 0.999998, -0.001956, 0, 0.999998, -0.001956, 0, 0.999998, -0.001956, 0, 0.999998, -0.001956, 0, 0.999998, -0.001956, 0, 0.999985, -0.005500, 0, 0.999985, -0.005500, 0, 0.999985, -0.005500, 0, 0.999985, -0.005500, 0, 0.999985, -0.005500, 0, 0.999985, -0.005500, 0, 0.999977, -0.006814, 0, 0.999977, -0.006814, 0, 0.999977, -0.006814, 0, 0.999977, -0.006814, 0, 0.999977, -0.006814, 0, 0.999977, -0.006814, 0, 0.999978, -0.006666, 0, 0.999978, -0.006666, 0, 0.999978, -0.006666, 0, 0.999978, -0.006666, 0, 0.999978, -0.006666, 0, 0.999978, -0.006666, 0, 0.999982, -0.006081, 0, 0.999982, -0.006081, 0, 0.999982, -0.006081, 0, 0.999982, -0.006081, 0, 0.999982, -0.006081, 0, 0.999982, -0.006081, 0, 0.999987, -0.005047, 0, 0.999987, -0.005047, 0, 0.999987, -0.005047, 0, 0.999987, -0.005047, 0, 0.999987, -0.005047, 0, 0.999987, -0.005047, 0, 0.999994, -0.003549, 0, 0.999994, -0.003549, 0, 0.999994, -0.003549, 0, 0.999994, -0.003549, 0, 0.999994, -0.003549, 0, 0.999994, -0.003549, 0, 0.999999, -0.001570, 0, 0.999999, -0.001570, 0, 0.999999, -0.001570, 0, 0.999999, -0.001570, 0, 0.999999, -0.001570, 0, 0.999999, -0.001570, 0, 1.000000, 0.000907, 0, 1.000000, 0.000907, 0, 1.000000, 0.000907, 0, 1.000000, 0.000907, 0, 1.000000, 0.000907, 0, 1.000000, 0.000907, 0, 0.999992, 0.003902, 0, 0.999992, 0.003902, 0, 0.999992, 0.003902, 0, 0.999992, 0.003902, 0, 0.999992, 0.003902, 0, 0.999992, 0.003902, 0, 0.999972, 0.007436, 0, 0.999972, 0.007436, 0, 0.999972, 0.007436, 0, 0.999972, 0.007436, 0, 0.999972, 0.007436, 0, 0.999972, 0.007436, 0, 0.998648, 0.051986, 0, 0.998648, 0.051986, 0, 0.998648, 0.051986, 0, 0.998648, 0.051986, 0, 0.998648, 0.051986, 0, 0.998648, 0.051986, 0, 0.998931, 0.046232, 0, 0.998931, 0.046232, 0, 0.998931, 0.046232, 0, 0.998931, 0.046232, 0, 0.998931, 0.046232, 0, 0.998931, 0.046232, 0, 0.999147, 0.041288, 0, 0.999147, 0.041288, 0, 0.999147, 0.041288, 0, 0.999147, 0.041288, 0, 0.999147, 0.041288, 0, 0.999147, 0.041288, 0, 0.999310, 0.037129, 0, 0.999310, 0.037129, 0, 0.999310, 0.037129, 0, 0.999310, 0.037129, 0, 0.999310, 0.037129, 0, 0.999310, 0.037129, 0, 0.999431, 0.033729, 0, 0.999431, 0.033729, 0, 0.999431, 0.033729, 0, 0.999431, 0.033729, 0, 0.999431, 0.033729, 0, 0.999431, 0.033729, 0, 0.999518, 0.031061, 0, 0.999518, 0.031061, 0, 0.999518, 0.031061, 0, 0.999518, 0.031061, 0, 0.999518, 0.031061, 0, 0.999518, 0.031061, 0, 0.999576, 0.029102, 0, 0.999576, 0.029102, 0, 0.999576, 0.029102, 0, 0.999576, 0.029102, 0, 0.999576, 0.029102, 0, 0.999576, 0.029102, 0, 0.999613, 0.027830, 0, 0.999613, 0.027830, 0, 0.999613, 0.027830, 0, 0.999613, 0.027830, 0, 0.999613, 0.027830, 0, 0.999613, 0.027830, 0, 0.999629, 0.027224, 0, 0.999629, 0.027224, 0, 0.999629, 0.027224, 0, 0.999629, 0.027224, 0, 0.999629, 0.027224, 0, 0.999629, 0.027224, 0, 0.999624, 0.027435, 0, 0.999624, 0.027435, 0, 0.999624, 0.027435, 0, 0.999624, 0.027435, 0, 0.999624, 0.027435, 0, 0.999624, 0.027435, 0, 0.999585, 0.028791, 0, 0.999585, 0.028791, 0, 0.999585, 0.028791, 0, 0.999585, 0.028791, 0, 0.999585, 0.028791, 0, 0.999585, 0.028791, 0, 0.999511, 0.031262, 0, 0.999511, 0.031262, 0, 0.999511, 0.031262, 0, 0.999511, 0.031262, 0, 0.999511, 0.031262, 0, 0.999511, 0.031262, 0, 0.999393, 0.034826, 0, 0.999393, 0.034826, 0, 0.999393, 0.034826, 0, 0.999393, 0.034826, 0, 0.999393, 0.034826, 0, 0.999393, 0.034826, 0, 0.999224, 0.039382, 0, 0.999224, 0.039382, 0, 0.999224, 0.039382, 0, 0.999224, 0.039382, 0, 0.999224, 0.039382, 0, 0.999224, 0.039382, 0, 0.998994, 0.044844, 0, 0.998994, 0.044844, 0, 0.998994, 0.044844, 0, 0.998994, 0.044844, 0, 0.998994, 0.044844, 0, 0.998994, 0.044844, 0, 0.998687, 0.051226, 0, 0.998687, 0.051226, 0, 0.998687, 0.051226, 0, 0.998687, 0.051226, 0, 0.998687, 0.051226, 0, 0.998687, 0.051226, 0, 0.998284, 0.058551, 0, 0.998284, 0.058551, 0, 0.998284, 0.058551, 0, 0.998284, 0.058551, 0, 0.998284, 0.058551, 0, 0.998284, 0.058551, 0, 0.997861, 0.065376, 0, 0.997861, 0.065376, 0, 0.997861, 0.065376, 0, 0.997861, 0.065376, 0, 0.997861, 0.065376, 0, 0.997861, 0.065376, 0, 0.997444, 0.071453, 0, 0.997444, 0.071453, 0, 0.997444, 0.071453, 0, 0.997444, 0.071453, 0, 0.997444, 0.071453, 0, 0.997444, 0.071453, 0, 0.997046, 0.076804, 0, 0.997046, 0.076804, 0, 0.997046, 0.076804, 0, 0.997046, 0.076804, 0, 0.997046, 0.076804, 0, 0.997046, 0.076804, 0, 0.996678, 0.081444, 0, 0.996678, 0.081444, 0, 0.996678, 0.081444, 0, 0.996678, 0.081444, 0, 0.996678, 0.081444, 0, 0.996678, 0.081444, 0, 0.996308, 0.085847, 0, 0.996308, 0.085847, 0, 0.996308, 0.085847, 0, 0.996308, 0.085847, 0, 0.996308, 0.085847, 0, 0.996308, 0.085847, 0, 0.995896, 0.090506, 0, 0.995896, 0.090506, 0, 0.995896, 0.090506, 0, 0.995896, 0.090506, 0, 0.995896, 0.090506, 0, 0.995896, 0.090506, 0, 0.995432, 0.095472, 0, 0.995432, 0.095472, 0, 0.995432, 0.095472, 0, 0.995432, 0.095472, 0, 0.995432, 0.095472, 0, 0.995432, 0.095472, 0, 0.994906, 0.100804, 0, 0.994906, 0.100804, 0, 0.994906, 0.100804, 0, 0.994906, 0.100804, 0, 0.994906, 0.100804, 0, 0.994906, 0.100804, 0, 0.994434, 0.105366, 0, 0.994434, 0.105366, 0, 0.994434, 0.105366, 0, 0.994434, 0.105366, 0, 0.994434, 0.105366, 0, 0.994434, 0.105366, 0, 0.993742, 0.111704, 0, 0.993742, 0.111704, 0, 0.993742, 0.111704, 0, 0.993742, 0.111704, 0, 0.993742, 0.111704, 0, 0.993742, 0.111704, 0, 0.992892, 0.119015, 0, 0.992892, 0.119015, 0, 0.992892, 0.119015, 0, 0.992892, 0.119015, 0, 0.992892, 0.119015, 0, 0.992892, 0.119015, 0, 0.991867, 0.127276, 0, 0.991867, 0.127276, 0, 0.991867, 0.127276, 0, 0.991867, 0.127276, 0, 0.991867, 0.127276, 0, 0.991867, 0.127276, 0, 0.990647, 0.136448, 0, 0.990647, 0.136448, 0, 0.990647, 0.136448, 0, 0.990647, 0.136448, 0, 0.990647, 0.136448, 0, 0.990647, 0.136448, 0, 0.989172, 0.146761, 0, 0.989172, 0.146761, 0, 0.989172, 0.146761, 0, 0.989172, 0.146761, 0, 0.989172, 0.146761, 0, 0.989172, 0.146761, 0, 0.987369, 0.158438, 0, 0.987369, 0.158438, 0, 0.987369, 0.158438, 0, 0.987369, 0.158438, 0, 0.987369, 0.158438, 0, 0.987369, 0.158438, 0, 0.985202, 0.171399, 0, 0.985202, 0.171399, 0, 0.985202, 0.171399, 0, 0.985202, 0.171399, 0, 0.985202, 0.171399, 0, 0.985202, 0.171399, 0, 0.982637, 0.185541, 0, 0.982637, 0.185541, 0, 0.982637, 0.185541, 0, 0.982637, 0.185541, 0, 0.982637, 0.185541, 0, 0.982637, 0.185541, 0, 0.979405, 0.201908, 0, 0.979405, 0.201908, 0, 0.979405, 0.201908, 0, 0.979405, 0.201908, 0, 0.979405, 0.201908, 0, 0.979405, 0.201908, 0, 0.976521, 0.215421, 0, 0.976521, 0.215421, 0, 0.976521, 0.215421, 0, 0.976521, 0.215421, 0, 0.976521, 0.215421, 0, 0.976521, 0.215421, 0, 0.974114, 0.226057, 0, 0.974114, 0.226057, 0, 0.974114, 0.226057, 0, 0.974114, 0.226057, 0, 0.974114, 0.226057, 0, 0.974114, 0.226057, 0, 0.972276, 0.233836, 0, 0.972276, 0.233836, 0, 0.972276, 0.233836, 0, 0.972276, 0.233836, 0, 0.972276, 0.233836, 0, 0.972276, 0.233836, 0, 0.971096, 0.238690, 0, 0.971096, 0.238690, 0, 0.971096, 0.238690, 0, 0.971096, 0.238690, 0, 0.971096, 0.238690, 0, 0.971096, 0.238690, 0, 0.970626, 0.240594, 0, 0.970626, 0.240594, 0, 0.970626, 0.240594, 0, 0.970626, 0.240594, 0, 0.970626, 0.240594, 0, 0.970626, 0.240594, 0, 0.970851, 0.239685, 0, 0.970851, 0.239685, 0, 0.970851, 0.239685, 0, 0.970851, 0.239685, 0, 0.970851, 0.239685, 0, 0.970851, 0.239685, 0, 0.971725, 0.236116, 0, 0.971725, 0.236116, 0, 0.971725, 0.236116, 0, 0.971725, 0.236116, 0, 0.971725, 0.236116, 0, 0.971725, 0.236116, 0, 0.972360, 0.233488, 0, 0.972360, 0.233488, 0, 0.972360, 0.233488, 0, 0.972360, 0.233488, 0, 0.972360, 0.233488, 0, 0.972360, 0.233488, 0, 0.973095, 0.230406, 0, 0.973095, 0.230406, 0, 0.973095, 0.230406, 0, 0.973095, 0.230406, 0, 0.973095, 0.230406, 0, 0.973095, 0.230406, 0, 0.973923, 0.226878, 0, 0.973923, 0.226878, 0, 0.973923, 0.226878, 0, 0.973923, 0.226878, 0, 0.973923, 0.226878, 0, 0.973923, 0.226878, 0, 0.974839, 0.222911, 0, 0.974839, 0.222911, 0, 0.974839, 0.222911, 0, 0.974839, 0.222911, 0, 0.974839, 0.222911, 0, 0.974839, 0.222911, 0, 0.975834, 0.218513, 0, 0.975834, 0.218513, 0, 0.975834, 0.218513, 0, 0.975834, 0.218513, 0, 0.975834, 0.218513, 0, 0.975834, 0.218513, 0, 0.976901, 0.213694, 0, 0.976901, 0.213694, 0, 0.976901, 0.213694, 0, 0.976901, 0.213694, 0, 0.976901, 0.213694, 0, 0.976901, 0.213694, 0, 0.978032, 0.208455, 0, 0.978032, 0.208455, 0, 0.978032, 0.208455, 0, 0.978032, 0.208455, 0, 0.978032, 0.208455, 0, 0.978032, 0.208455, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.980735, 0.195345, 0, 0.980735, 0.195345, 0, 0.980735, 0.195345, 0, 0.980735, 0.195345, 0, 0.980735, 0.195345, 0, 0.980735, 0.195345, 0, 0.983031, 0.183438, 0, 0.983031, 0.183438, 0, 0.983031, 0.183438, 0, 0.983031, 0.183438, 0, 0.983031, 0.183438, 0, 0.983031, 0.183438, 0, 0.987449, 0.157939, 0, 0.987449, 0.157939, 0, 0.987449, 0.157939, 0, 0.987449, 0.157939, 0, 0.987449, 0.157939, 0, 0.987449, 0.157939, 0, 0.985737, 0.168292, 0, 0.985737, 0.168292, 0, 0.985737, 0.168292, 0, 0.985737, 0.168292, 0, 0.985737, 0.168292, 0, 0.985737, 0.168292, 0, 0.997854, 0.065482, 0, 0.997854, 0.065482, 0, 0.997854, 0.065482, 0, 0.997854, 0.065482, 0, 0.997854, 0.065482, 0, 0.997854, 0.065482, 0, 0.998500, 0.054748, 0, 0.998500, 0.054748, 0, 0.998500, 0.054748, 0, 0.998500, 0.054748, 0, 0.998500, 0.054748, 0, 0.998500, 0.054748, 0, 0.998938, 0.046068, 0, 0.998938, 0.046068, 0, 0.998938, 0.046068, 0, 0.998938, 0.046068, 0, 0.998938, 0.046068, 0, 0.998938, 0.046068, 0, 0.999221, 0.039468, 0, 0.999221, 0.039468, 0, 0.999221, 0.039468, 0, 0.999221, 0.039468, 0, 0.999221, 0.039468, 0, 0.999221, 0.039468, 0, 0.999389, 0.034958, 0, 0.999389, 0.034958, 0, 0.999389, 0.034958, 0, 0.999389, 0.034958, 0, 0.999389, 0.034958, 0, 0.999389, 0.034958, 0, 0.999491, 0.031893, 0, 0.999491, 0.031893, 0, 0.999491, 0.031893, 0, 0.999491, 0.031893, 0, 0.999491, 0.031893, 0, 0.999491, 0.031893, 0, 0.999561, 0.029633, 0, 0.999561, 0.029633, 0, 0.999561, 0.029633, 0, 0.999561, 0.029633, 0, 0.999561, 0.029633, 0, 0.999561, 0.029633, 0, 0.999603, 0.028177, 0, 0.999603, 0.028177, 0, 0.999603, 0.028177, 0, 0.999603, 0.028177, 0, 0.999603, 0.028177, 0, 0.999603, 0.028177, 0, 0.999621, 0.027514, 0, 0.999621, 0.027514, 0, 0.999621, 0.027514, 0, 0.999621, 0.027514, 0, 0.999621, 0.027514, 0, 0.999621, 0.027514, 0, 0.999645, 0.026648, 0, 0.999645, 0.026648, 0, 0.999645, 0.026648, 0, 0.999645, 0.026648, 0, 0.999645, 0.026648, 0, 0.999645, 0.026648, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1.000000, 0.000869, 0, 1.000000, 0.000869, 0, 1.000000, 0.000869, 0, 1.000000, 0.000869, 0, 1.000000, 0.000869, 0, 1.000000, 0.000869, 0, 0.999994, 0.003528, 0, 0.999994, 0.003528, 0, 0.999994, 0.003528, 0, 0.999994, 0.003528, 0, 0.999994, 0.003528, 0, 0.999994, 0.003528, 0, 0.999968, 0.008031, 0, 0.999968, 0.008031, 0, 0.999968, 0.008031, 0, 0.999968, 0.008031, 0, 0.999968, 0.008031, 0, 0.999968, 0.008031, 0, 0.999896, 0.014403, 0, 0.999896, 0.014403, 0, 0.999896, 0.014403, 0, 0.999896, 0.014403, 0, 0.999896, 0.014403, 0, 0.999896, 0.014403, 0, 0.999730, 0.023228, 0, 0.999730, 0.023228, 0, 0.999730, 0.023228, 0, 0.999730, 0.023228, 0, 0.999730, 0.023228, 0, 0.999730, 0.023228, 0, 0.999384, 0.035086, 0, 0.999384, 0.035086, 0, 0.999384, 0.035086, 0, 0.999384, 0.035086, 0, 0.999384, 0.035086, 0, 0.999384, 0.035086, 0, 0.998752, 0.049938, 0, 0.998752, 0.049938, 0, 0.998752, 0.049938, 0, 0.998752, 0.049938, 0, 0.998752, 0.049938, 0, 0.998752, 0.049938, 0, 0.997706, 0.067700, 0, 0.997706, 0.067700, 0, 0.997706, 0.067700, 0, 0.997706, 0.067700, 0, 0.997706, 0.067700, 0, 0.997706, 0.067700, 0, 0.984773, 0.173847, 0, 0.984773, 0.173847, 0, 0.984773, 0.173847, 0, 0.984773, 0.173847, 0, 0.984773, 0.173847, 0, 0.984773, 0.173847, 0, 0.984006, 0.178134, 0, 0.984006, 0.178134, 0, 0.984006, 0.178134, 0, 0.984006, 0.178134, 0, 0.984006, 0.178134, 0, 0.984006, 0.178134, 0, 0.978416, 0.206643, 0, 0.978416, 0.206643, 0, 0.978416, 0.206643, 0, 0.978416, 0.206643, 0, 0.978416, 0.206643, 0, 0.978416, 0.206643, 0, 0.979442, 0.201724, 0, 0.979442, 0.201724, 0, 0.979442, 0.201724, 0, 0.979442, 0.201724, 0, 0.979442, 0.201724, 0, 0.979442, 0.201724, 0, 0.966682, 0.255981, 0, 0.966682, 0.255981, 0, 0.966682, 0.255981, 0, 0.966682, 0.255981, 0, 0.966682, 0.255981, 0, 0.966682, 0.255981, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.996705, 0.081116, 0, 0.996705, 0.081116, 0, 0.996705, 0.081116, 0, 0.996705, 0.081116, 0, 0.996705, 0.081116, 0, 0.996705, 0.081116, 0, 0.996731, 0.080798, 0, 0.996731, 0.080798, 0, 0.996731, 0.080798, 0, 0.996731, 0.080798, 0, 0.996731, 0.080798, 0, 0.996731, 0.080798, 0, 0.996848, 0.079335, 0, 0.996848, 0.079335, 0, 0.996848, 0.079335, 0, 0.996848, 0.079335, 0, 0.996848, 0.079335, 0, 0.996848, 0.079335, 0, 0.997054, 0.076702, 0, 0.997054, 0.076702, 0, 0.997054, 0.076702, 0, 0.997054, 0.076702, 0, 0.997054, 0.076702, 0, 0.997054, 0.076702, 0, 0.997351, 0.072737, 0, 0.997351, 0.072737, 0, 0.997351, 0.072737, 0, 0.997351, 0.072737, 0, 0.997351, 0.072737, 0, 0.997351, 0.072737, 0, 0.997734, 0.067282, 0, 0.997734, 0.067282, 0, 0.997734, 0.067282, 0, 0.997734, 0.067282, 0, 0.997734, 0.067282, 0, 0.997734, 0.067282, 0, 0.998178, 0.060342, 0, 0.998178, 0.060342, 0, 0.998178, 0.060342, 0, 0.998178, 0.060342, 0, 0.998178, 0.060342, 0, 0.998178, 0.060342, 0, 0.998650, 0.051936, 0, 0.998650, 0.051936, 0, 0.998650, 0.051936, 0, 0.998650, 0.051936, 0, 0.998650, 0.051936, 0, 0.998650, 0.051936, 0, 0.998989, 0.044959, 0, 0.998989, 0.044959, 0, 0.998989, 0.044959, 0, 0.998989, 0.044959, 0, 0.998989, 0.044959, 0, 0.998989, 0.044959, 0, 0.999251, 0.038700, 0, 0.999251, 0.038700, 0, 0.999251, 0.038700, 0, 0.999251, 0.038700, 0, 0.999251, 0.038700, 0, 0.999251, 0.038700, 0, 0.999449, 0.033181, 0, 0.999449, 0.033181, 0, 0.999449, 0.033181, 0, 0.999449, 0.033181, 0, 0.999449, 0.033181, 0, 0.999449, 0.033181, 0, 0.999596, 0.028418, 0, 0.999596, 0.028418, 0, 0.999596, 0.028418, 0, 0.999596, 0.028418, 0, 0.999596, 0.028418, 0, 0.999596, 0.028418, 0, 0.999698, 0.024573, 0, 0.999698, 0.024573, 0, 0.999698, 0.024573, 0, 0.999698, 0.024573, 0, 0.999698, 0.024573, 0, 0.999698, 0.024573, 0, 0.999762, 0.021798, 0, 0.999762, 0.021798, 0, 0.999762, 0.021798, 0, 0.999762, 0.021798, 0, 0.999762, 0.021798, 0, 0.999762, 0.021798, 0, 0.999798, 0.020081, 0, 0.999798, 0.020081, 0, 0.999798, 0.020081, 0, 0.999798, 0.020081, 0, 0.999798, 0.020081, 0, 0.999798, 0.020081, 0, 0.999812, 0.019405, 0, 0.999812, 0.019405, 0, 0.999812, 0.019405, 0, 0.999812, 0.019405, 0, 0.999812, 0.019405, 0, 0.999812, 0.019405, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999740, 0.022820, 0, 0.999740, 0.022820, 0, 0.999740, 0.022820, 0, 0.999740, 0.022820, 0, 0.999740, 0.022820, 0, 0.999740, 0.022820, 0, 0.999626, 0.027351, 0, 0.999626, 0.027351, 0, 0.999626, 0.027351, 0, 0.999626, 0.027351, 0, 0.999626, 0.027351, 0, 0.999626, 0.027351, 0, 0.999432, 0.033714, 0, 0.999432, 0.033714, 0, 0.999432, 0.033714, 0, 0.999432, 0.033714, 0, 0.999432, 0.033714, 0, 0.999432, 0.033714, 0, 0.999135, 0.041580, 0, 0.999135, 0.041580, 0, 0.999135, 0.041580, 0, 0.999135, 0.041580, 0, 0.999135, 0.041580, 0, 0.999135, 0.041580, 0, 0.998716, 0.050650, 0, 0.998716, 0.050650, 0, 0.998716, 0.050650, 0, 0.998716, 0.050650, 0, 0.998716, 0.050650, 0, 0.998716, 0.050650, 0, 0.998141, 0.060948, 0, 0.998141, 0.060948, 0, 0.998141, 0.060948, 0, 0.998141, 0.060948, 0, 0.998141, 0.060948, 0, 0.998141, 0.060948, 0, 0.997368, 0.072504, 0, 0.997368, 0.072504, 0, 0.997368, 0.072504, 0, 0.997368, 0.072504, 0, 0.997368, 0.072504, 0, 0.997368, 0.072504, 0, 0.996936, 0.078225, 0, 0.996936, 0.078225, 0, 0.996936, 0.078225, 0, 0.996936, 0.078225, 0, 0.996936, 0.078225, 0, 0.996936, 0.078225, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999734, 0.023072, 0, 0.999734, 0.023072, 0, 0.999734, 0.023072, 0, 0.999734, 0.023072, 0, 0.999734, 0.023072, 0, 0.999734, 0.023072, 0, 0.999864, 0.016518, 0, 0.999864, 0.016518, 0, 0.999864, 0.016518, 0, 0.999864, 0.016518, 0, 0.999864, 0.016518, 0, 0.999864, 0.016518, 0, 0.999941, 0.010847, 0, 0.999941, 0.010847, 0, 0.999941, 0.010847, 0, 0.999941, 0.010847, 0, 0.999941, 0.010847, 0, 0.999941, 0.010847, 0, 0.999982, 0.006063, 0, 0.999982, 0.006063, 0, 0.999982, 0.006063, 0, 0.999982, 0.006063, 0, 0.999982, 0.006063, 0, 0.999982, 0.006063, 0, 0.999998, 0.002177, 0, 0.999998, 0.002177, 0, 0.999998, 0.002177, 0, 0.999998, 0.002177, 0, 0.999998, 0.002177, 0, 0.999998, 0.002177, 0, 1.000000, -0.000848, 0, 1.000000, -0.000848, 0, 1.000000, -0.000848, 0, 1.000000, -0.000848, 0, 1.000000, -0.000848, 0, 1.000000, -0.000848, 0, 0.999995, -0.003045, 0, 0.999995, -0.003045, 0, 0.999995, -0.003045, 0, 0.999995, -0.003045, 0, 0.999995, -0.003045, 0, 0.999995, -0.003045, 0, 0.999990, -0.004398, 0, 0.999990, -0.004398, 0, 0.999990, -0.004398, 0, 0.999990, -0.004398, 0, 0.999990, -0.004398, 0, 0.999990, -0.004398, 0, 0.999988, -0.004888, 0, 0.999988, -0.004888, 0, 0.999988, -0.004888, 0, 0.999988, -0.004888, 0, 0.999988, -0.004888, 0, 0.999988, -0.004888, 0, 0.999992, -0.004017, 0, 0.999992, -0.004017, 0, 0.999992, -0.004017, 0, 0.999992, -0.004017, 0, 0.999992, -0.004017, 0, 0.999992, -0.004017, 0, 0.999999, -0.001232, 0, 0.999999, -0.001232, 0, 0.999999, -0.001232, 0, 0.999999, -0.001232, 0, 0.999999, -0.001232, 0, 0.999999, -0.001232, 0, 0.999994, 0.003521, 0, 0.999994, 0.003521, 0, 0.999994, 0.003521, 0, 0.999994, 0.003521, 0, 0.999994, 0.003521, 0, 0.999994, 0.003521, 0, 0.999947, 0.010282, 0, 0.999947, 0.010282, 0, 0.999947, 0.010282, 0, 0.999947, 0.010282, 0, 0.999947, 0.010282, 0, 0.999947, 0.010282, 0, 0.999824, 0.018757, 0, 0.999824, 0.018757, 0, 0.999824, 0.018757, 0, 0.999824, 0.018757, 0, 0.999824, 0.018757, 0, 0.999824, 0.018757, 0, 0.999591, 0.028614, 0, 0.999591, 0.028614, 0, 0.999591, 0.028614, 0, 0.999591, 0.028614, 0, 0.999591, 0.028614, 0, 0.999591, 0.028614, 0, 0.999207, 0.039807, 0, 0.999207, 0.039807, 0, 0.999207, 0.039807, 0, 0.999207, 0.039807, 0, 0.999207, 0.039807, 0, 0.999207, 0.039807, 0, 0.998633, 0.052270, 0, 0.998633, 0.052270, 0, 0.998633, 0.052270, 0, 0.998633, 0.052270, 0, 0.998633, 0.052270, 0, 0.998633, 0.052270, 0, 0.997788, 0.066480, 0, 0.997788, 0.066480, 0, 0.997788, 0.066480, 0, 0.997788, 0.066480, 0, 0.997788, 0.066480, 0, 0.997788, 0.066480, 0, 0.996910, 0.078554, 0, 0.996910, 0.078554, 0, 0.996910, 0.078554, 0, 0.996910, 0.078554, 0, 0.996910, 0.078554, 0, 0.996910, 0.078554, 0, 0.996082, 0.088434, 0, 0.996082, 0.088434, 0, 0.996082, 0.088434, 0, 0.996082, 0.088434, 0, 0.996082, 0.088434, 0, 0.996082, 0.088434, 0, 0.995372, 0.096099, 0, 0.995372, 0.096099, 0, 0.995372, 0.096099, 0, 0.995372, 0.096099, 0, 0.995372, 0.096099, 0, 0.995372, 0.096099, 0, 0.994832, 0.101537, 0, 0.994832, 0.101537, 0, 0.994832, 0.101537, 0, 0.994832, 0.101537, 0, 0.994832, 0.101537, 0, 0.994832, 0.101537, 0, 0.994497, 0.104763, 0, 0.994497, 0.104763, 0, 0.994497, 0.104763, 0, 0.994497, 0.104763, 0, 0.994497, 0.104763, 0, 0.994497, 0.104763, 0, 0.994382, 0.105849, 0, 0.994382, 0.105849, 0, 0.994382, 0.105849, 0, 0.994382, 0.105849, 0, 0.994382, 0.105849, 0, 0.994382, 0.105849, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994884, 0.101024, 0, 0.994884, 0.101024, 0, 0.994884, 0.101024, 0, 0.994884, 0.101024, 0, 0.994884, 0.101024, 0, 0.994884, 0.101024, 0, 0.994621, 0.103579, 0, 0.994621, 0.103579, 0, 0.994621, 0.103579, 0, 0.994621, 0.103579, 0, 0.994621, 0.103579, 0, 0.994621, 0.103579, 0, 0.993676, 0.112286, 0, 0.993676, 0.112286, 0, 0.993676, 0.112286, 0, 0.993676, 0.112286, 0, 0.993676, 0.112286, 0, 0.993676, 0.112286, 0, 0.992759, 0.120125, 0, 0.992759, 0.120125, 0, 0.992759, 0.120125, 0, 0.992759, 0.120125, 0, 0.992759, 0.120125, 0, 0.992759, 0.120125, 0, 0.991886, 0.127132, 0, 0.991886, 0.127132, 0, 0.991886, 0.127132, 0, 0.991886, 0.127132, 0, 0.991886, 0.127132, 0, 0.991886, 0.127132, 0, 0.991071, 0.133334, 0, 0.991071, 0.133334, 0, 0.991071, 0.133334, 0, 0.991071, 0.133334, 0, 0.991071, 0.133334, 0, 0.991071, 0.133334, 0, 0.990345, 0.138626, 0, 0.990345, 0.138626, 0, 0.990345, 0.138626, 0, 0.990345, 0.138626, 0, 0.990345, 0.138626, 0, 0.990345, 0.138626, 0, 0.989739, 0.142889, 0, 0.989739, 0.142889, 0, 0.989739, 0.142889, 0, 0.989739, 0.142889, 0, 0.989739, 0.142889, 0, 0.989739, 0.142889, 0, 0.989267, 0.146119, 0, 0.989267, 0.146119, 0, 0.989267, 0.146119, 0, 0.989267, 0.146119, 0, 0.989267, 0.146119, 0, 0.989267, 0.146119, 0, 0.988942, 0.148304, 0, 0.988942, 0.148304, 0, 0.988942, 0.148304, 0, 0.988942, 0.148304, 0, 0.988942, 0.148304, 0, 0.988942, 0.148304, 0, 0.988817, 0.149136, 0, 0.988817, 0.149136, 0, 0.988817, 0.149136, 0, 0.988817, 0.149136, 0, 0.988817, 0.149136, 0, 0.988817, 0.149136, 0, 0.988759, 0.149521, 0, 0.988759, 0.149521, 0, 0.988759, 0.149521, 0, 0.988759, 0.149521, 0, 0.988759, 0.149521, 0, 0.988759, 0.149521, 0, 0.988769, 0.149450, 0, 0.988769, 0.149450, 0, 0.988769, 0.149450, 0, 0.988769, 0.149450, 0, 0.988769, 0.149450, 0, 0.988769, 0.149450, 0, 0.988850, 0.148912, 0, 0.988850, 0.148912, 0, 0.988850, 0.148912, 0, 0.988850, 0.148912, 0, 0.988850, 0.148912, 0, 0.988850, 0.148912, 0, 0.989003, 0.147895, 0, 0.989003, 0.147895, 0, 0.989003, 0.147895, 0, 0.989003, 0.147895, 0, 0.989003, 0.147895, 0, 0.989003, 0.147895, 0, 0.989228, 0.146385, 0, 0.989228, 0.146385, 0, 0.989228, 0.146385, 0, 0.989228, 0.146385, 0, 0.989228, 0.146385, 0, 0.989228, 0.146385, 0, 0.989524, 0.144371, 0, 0.989524, 0.144371, 0, 0.989524, 0.144371, 0, 0.989524, 0.144371, 0, 0.989524, 0.144371, 0, 0.989524, 0.144371, 0, 0.989889, 0.141842, 0, 0.989889, 0.141842, 0, 0.989889, 0.141842, 0, 0.989889, 0.141842, 0, 0.989889, 0.141842, 0, 0.989889, 0.141842, 0, 0.986038, 0.166521, 0, 0.986038, 0.166521, 0, 0.986038, 0.166521, 0, 0.986038, 0.166521, 0, 0.986038, 0.166521, 0, 0.986038, 0.166521, 0, 0.985675, 0.168655, 0, 0.985675, 0.168655, 0, 0.985675, 0.168655, 0, 0.985675, 0.168655, 0, 0.985675, 0.168655, 0, 0.985675, 0.168655, 0, 0.985389, 0.170320, 0, 0.985389, 0.170320, 0, 0.985389, 0.170320, 0, 0.985389, 0.170320, 0, 0.985389, 0.170320, 0, 0.985389, 0.170320, 0, 0.985179, 0.171530, 0, 0.985179, 0.171530, 0, 0.985179, 0.171530, 0, 0.985179, 0.171530, 0, 0.985179, 0.171530, 0, 0.985179, 0.171530, 0, 0.985045, 0.172295, 0, 0.985045, 0.172295, 0, 0.985045, 0.172295, 0, 0.985045, 0.172295, 0, 0.985045, 0.172295, 0, 0.985045, 0.172295, 0, 0.984988, 0.172626, 0, 0.984988, 0.172626, 0, 0.984988, 0.172626, 0, 0.984988, 0.172626, 0, 0.984988, 0.172626, 0, 0.984988, 0.172626, 0, 0.985004, 0.172529, 0, 0.985004, 0.172529, 0, 0.985004, 0.172529, 0, 0.985004, 0.172529, 0, 0.985004, 0.172529, 0, 0.985004, 0.172529, 0, 0.985094, 0.172015, 0, 0.985094, 0.172015, 0, 0.985094, 0.172015, 0, 0.985094, 0.172015, 0, 0.985094, 0.172015, 0, 0.985094, 0.172015, 0, 0.985255, 0.171090, 0, 0.985255, 0.171090, 0, 0.985255, 0.171090, 0, 0.985255, 0.171090, 0, 0.985255, 0.171090, 0, 0.985255, 0.171090, 0, 0.985934, 0.167136, 0, 0.985934, 0.167136, 0, 0.985934, 0.167136, 0, 0.985934, 0.167136, 0, 0.985934, 0.167136, 0, 0.985934, 0.167136, 0, 0.986891, 0.161386, 0, 0.986891, 0.161386, 0, 0.986891, 0.161386, 0, 0.986891, 0.161386, 0, 0.986891, 0.161386, 0, 0.986891, 0.161386, 0, 0.988092, 0.153863, 0, 0.988092, 0.153863, 0, 0.988092, 0.153863, 0, 0.988092, 0.153863, 0, 0.988092, 0.153863, 0, 0.988092, 0.153863, 0, 0.989496, 0.144564, 0, 0.989496, 0.144564, 0, 0.989496, 0.144564, 0, 0.989496, 0.144564, 0, 0.989496, 0.144564, 0, 0.989496, 0.144564, 0, 0.991060, 0.133414, 0, 0.991060, 0.133414, 0, 0.991060, 0.133414, 0, 0.991060, 0.133414, 0, 0.991060, 0.133414, 0, 0.991060, 0.133414, 0, 0.992735, 0.120319, 0, 0.992735, 0.120319, 0, 0.992735, 0.120319, 0, 0.992735, 0.120319, 0, 0.992735, 0.120319, 0, 0.992735, 0.120319, 0, 0.994451, 0.105202, 0, 0.994451, 0.105202, 0, 0.994451, 0.105202, 0, 0.994451, 0.105202, 0, 0.994451, 0.105202, 0, 0.994451, 0.105202, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.996609, 0.082284, 0, 0.996609, 0.082284, 0, 0.996609, 0.082284, 0, 0.996609, 0.082284, 0, 0.996609, 0.082284, 0, 0.996609, 0.082284, 0, 0.995839, 0.091130, 0, 0.995839, 0.091130, 0, 0.995839, 0.091130, 0, 0.995839, 0.091130, 0, 0.995839, 0.091130, 0, 0.995839, 0.091130, 0, 0.995100, 0.098876, 0, 0.995100, 0.098876, 0, 0.995100, 0.098876, 0, 0.995100, 0.098876, 0, 0.995100, 0.098876, 0, 0.995100, 0.098876, 0, 0.994415, 0.105545, 0, 0.994415, 0.105545, 0, 0.994415, 0.105545, 0, 0.994415, 0.105545, 0, 0.994415, 0.105545, 0, 0.994415, 0.105545, 0, 0.993803, 0.111155, 0, 0.993803, 0.111155, 0, 0.993803, 0.111155, 0, 0.993803, 0.111155, 0, 0.993803, 0.111155, 0, 0.993803, 0.111155, 0, 0.993281, 0.115724, 0, 0.993281, 0.115724, 0, 0.993281, 0.115724, 0, 0.993281, 0.115724, 0, 0.993281, 0.115724, 0, 0.993281, 0.115724, 0, 0.992863, 0.119264, 0, 0.992863, 0.119264, 0, 0.992863, 0.119264, 0, 0.992863, 0.119264, 0, 0.992863, 0.119264, 0, 0.992863, 0.119264, 0, 0.992558, 0.121772, 0, 0.992558, 0.121772, 0, 0.992558, 0.121772, 0, 0.992558, 0.121772, 0, 0.992558, 0.121772, 0, 0.992558, 0.121772, 0, 0.992377, 0.123240, 0, 0.992377, 0.123240, 0, 0.992377, 0.123240, 0, 0.992377, 0.123240, 0, 0.992377, 0.123240, 0, 0.992377, 0.123240, 0, 0.992332, 0.123598, 0, 0.992332, 0.123598, 0, 0.992332, 0.123598, 0, 0.992332, 0.123598, 0, 0.992332, 0.123598, 0, 0.992332, 0.123598, 0, 0.992483, 0.122382, 0, 0.992483, 0.122382, 0, 0.992483, 0.122382, 0, 0.992483, 0.122382, 0, 0.992483, 0.122382, 0, 0.992483, 0.122382, 0, 0.992827, 0.119556, 0, 0.992827, 0.119556, 0, 0.992827, 0.119556, 0, 0.992827, 0.119556, 0, 0.992827, 0.119556, 0, 0.992827, 0.119556, 0, 0.993355, 0.115089, 0, 0.993355, 0.115089, 0, 0.993355, 0.115089, 0, 0.993355, 0.115089, 0, 0.993355, 0.115089, 0, 0.993355, 0.115089, 0, 0.994026, 0.109139, 0, 0.994026, 0.109139, 0, 0.994026, 0.109139, 0, 0.994026, 0.109139, 0, 0.994026, 0.109139, 0, 0.994026, 0.109139, 0, 0.994796, 0.101885, 0, 0.994796, 0.101885, 0, 0.994796, 0.101885, 0, 0.994796, 0.101885, 0, 0.994796, 0.101885, 0, 0.994796, 0.101885, 0, 0.995635, 0.093334, 0, 0.995635, 0.093334, 0, 0.995635, 0.093334, 0, 0.995635, 0.093334, 0, 0.995635, 0.093334, 0, 0.995635, 0.093334, 0, 0.996507, 0.083508, 0, 0.996507, 0.083508, 0, 0.996507, 0.083508, 0, 0.996507, 0.083508, 0, 0.996507, 0.083508, 0, 0.996507, 0.083508, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.988981, 0.148043, 0, 0.988981, 0.148043, 0, 0.988981, 0.148043, 0, 0.988981, 0.148043, 0, 0.988981, 0.148043, 0, 0.988981, 0.148043, 0, 0.989498, 0.144546, 0, 0.989498, 0.144546, 0, 0.989498, 0.144546, 0, 0.989498, 0.144546, 0, 0.989498, 0.144546, 0, 0.989498, 0.144546, 0, 0.992252, 0.124242, 0, 0.992252, 0.124242, 0, 0.992252, 0.124242, 0, 0.992252, 0.124242, 0, 0.992252, 0.124242, 0, 0.992252, 0.124242, 0, 0.991589, 0.129423, 0, 0.991589, 0.129423, 0, 0.991589, 0.129423, 0, 0.991589, 0.129423, 0, 0.991589, 0.129423, 0, 0.991589, 0.129423, 0, 0.991018, 0.133732, 0, 0.991018, 0.133732, 0, 0.991018, 0.133732, 0, 0.991018, 0.133732, 0, 0.991018, 0.133732, 0, 0.991018, 0.133732, 0, 0.990546, 0.137179, 0, 0.990546, 0.137179, 0, 0.990546, 0.137179, 0, 0.990546, 0.137179, 0, 0.990546, 0.137179, 0, 0.990546, 0.137179, 0, 0.990183, 0.139778, 0, 0.990183, 0.139778, 0, 0.990183, 0.139778, 0, 0.990183, 0.139778, 0, 0.990183, 0.139778, 0, 0.990183, 0.139778, 0, 0.989932, 0.141545, 0, 0.989932, 0.141545, 0, 0.989932, 0.141545, 0, 0.989932, 0.141545, 0, 0.989932, 0.141545, 0, 0.989932, 0.141545, 0, 0.989795, 0.142499, 0, 0.989795, 0.142499, 0, 0.989795, 0.142499, 0, 0.989795, 0.142499, 0, 0.989795, 0.142499, 0, 0.989795, 0.142499, 0, 0.989773, 0.142653, 0, 0.989773, 0.142653, 0, 0.989773, 0.142653, 0, 0.989773, 0.142653, 0, 0.989773, 0.142653, 0, 0.989773, 0.142653, 0, 0.989863, 0.142025, 0, 0.989863, 0.142025, 0, 0.989863, 0.142025, 0, 0.989863, 0.142025, 0, 0.989863, 0.142025, 0, 0.989863, 0.142025, 0, 0.990210, 0.139588, 0, 0.990210, 0.139588, 0, 0.990210, 0.139588, 0, 0.990210, 0.139588, 0, 0.990210, 0.139588, 0, 0.990210, 0.139588, 0, 0.990758, 0.135642, 0, 0.990758, 0.135642, 0, 0.990758, 0.135642, 0, 0.990758, 0.135642, 0, 0.990758, 0.135642, 0, 0.990758, 0.135642, 0, 0.991487, 0.130207, 0, 0.991487, 0.130207, 0, 0.991487, 0.130207, 0, 0.991487, 0.130207, 0, 0.991487, 0.130207, 0, 0.991487, 0.130207, 0, 0.992371, 0.123286, 0, 0.992371, 0.123286, 0, 0.992371, 0.123286, 0, 0.992371, 0.123286, 0, 0.992371, 0.123286, 0, 0.992371, 0.123286, 0, 0.993374, 0.114924, 0, 0.993374, 0.114924, 0, 0.993374, 0.114924, 0, 0.993374, 0.114924, 0, 0.993374, 0.114924, 0, 0.993374, 0.114924, 0, 0.994456, 0.105151, 0, 0.994456, 0.105151, 0, 0.994456, 0.105151, 0, 0.994456, 0.105151, 0, 0.994456, 0.105151, 0, 0.994456, 0.105151, 0, 0.995579, 0.093927, 0, 0.995579, 0.093927, 0, 0.995579, 0.093927, 0, 0.995579, 0.093927, 0, 0.995579, 0.093927, 0, 0.995579, 0.093927, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.995679, 0.092862, 0, 0.995679, 0.092862, 0, 0.995679, 0.092862, 0, 0.995679, 0.092862, 0, 0.995679, 0.092862, 0, 0.995679, 0.092862, 0, 0.995424, 0.095558, 0, 0.995424, 0.095558, 0, 0.995424, 0.095558, 0, 0.995424, 0.095558, 0, 0.995424, 0.095558, 0, 0.995424, 0.095558, 0, 0.995190, 0.097964, 0, 0.995190, 0.097964, 0, 0.995190, 0.097964, 0, 0.995190, 0.097964, 0, 0.995190, 0.097964, 0, 0.995190, 0.097964, 0, 0.994980, 0.100078, 0, 0.994980, 0.100078, 0, 0.994980, 0.100078, 0, 0.994980, 0.100078, 0, 0.994980, 0.100078, 0, 0.994980, 0.100078, 0, 0.994795, 0.101895, 0, 0.994795, 0.101895, 0, 0.994795, 0.101895, 0, 0.994795, 0.101895, 0, 0.994795, 0.101895, 0, 0.994795, 0.101895, 0, 0.994638, 0.103415, 0, 0.994638, 0.103415, 0, 0.994638, 0.103415, 0, 0.994638, 0.103415, 0, 0.994638, 0.103415, 0, 0.994638, 0.103415, 0, 0.994511, 0.104634, 0, 0.994511, 0.104634, 0, 0.994511, 0.104634, 0, 0.994511, 0.104634, 0, 0.994511, 0.104634, 0, 0.994511, 0.104634, 0, 0.994414, 0.105548, 0, 0.994414, 0.105548, 0, 0.994414, 0.105548, 0, 0.994414, 0.105548, 0, 0.994414, 0.105548, 0, 0.994414, 0.105548, 0, 0.994350, 0.106155, 0, 0.994350, 0.106155, 0, 0.994350, 0.106155, 0, 0.994350, 0.106155, 0, 0.994350, 0.106155, 0, 0.994350, 0.106155, 0, 0.994328, 0.106361, 0, 0.994328, 0.106361, 0, 0.994328, 0.106361, 0, 0.994328, 0.106361, 0, 0.994328, 0.106361, 0, 0.994328, 0.106361, 0, 0.994480, 0.104928, 0, 0.994480, 0.104928, 0, 0.994480, 0.104928, 0, 0.994480, 0.104928, 0, 0.994480, 0.104928, 0, 0.994480, 0.104928, 0, 0.994803, 0.101818, 0, 0.994803, 0.101818, 0, 0.994803, 0.101818, 0, 0.994803, 0.101818, 0, 0.994803, 0.101818, 0, 0.994803, 0.101818, 0, 0.995284, 0.097000, 0, 0.995284, 0.097000, 0, 0.995284, 0.097000, 0, 0.995284, 0.097000, 0, 0.995284, 0.097000, 0, 0.995284, 0.097000, 0, 0.995888, 0.090592, 0, 0.995888, 0.090592, 0, 0.995888, 0.090592, 0, 0.995888, 0.090592, 0, 0.995888, 0.090592, 0, 0.995888, 0.090592, 0, 0.996572, 0.082730, 0, 0.996572, 0.082730, 0, 0.996572, 0.082730, 0, 0.996572, 0.082730, 0, 0.996572, 0.082730, 0, 0.996572, 0.082730, 0, 0.997301, 0.073422, 0, 0.997301, 0.073422, 0, 0.997301, 0.073422, 0, 0.997301, 0.073422, 0, 0.997301, 0.073422, 0, 0.997301, 0.073422, 0, 0.998033, 0.062692, 0, 0.998033, 0.062692, 0, 0.998033, 0.062692, 0, 0.998033, 0.062692, 0, 0.998033, 0.062692, 0, 0.998033, 0.062692, 0, 0.998655, 0.051854, 0, 0.998655, 0.051854, 0, 0.998655, 0.051854, 0, 0.998655, 0.051854, 0, 0.998655, 0.051854, 0, 0.998655, 0.051854, 0, 0.999108, 0.042231, 0, 0.999108, 0.042231, 0, 0.999108, 0.042231, 0, 0.999108, 0.042231, 0, 0.999108, 0.042231, 0, 0.999108, 0.042231, 0, 0.999426, 0.033865, 0, 0.999426, 0.033865, 0, 0.999426, 0.033865, 0, 0.999426, 0.033865, 0, 0.999426, 0.033865, 0, 0.999426, 0.033865, 0, 0.999641, 0.026782, 0, 0.999641, 0.026782, 0, 0.999641, 0.026782, 0, 0.999641, 0.026782, 0, 0.999641, 0.026782, 0, 0.999641, 0.026782, 0, 0.999776, 0.021150, 0, 0.999776, 0.021150, 0, 0.999776, 0.021150, 0, 0.999776, 0.021150, 0, 0.999776, 0.021150, 0, 0.999776, 0.021150, 0, 0.999853, 0.017119, 0, 0.999853, 0.017119, 0, 0.999853, 0.017119, 0, 0.999853, 0.017119, 0, 0.999853, 0.017119, 0, 0.999853, 0.017119, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999905, 0.013774, 0, 0.999905, 0.013774, 0, 0.999905, 0.013774, 0, 0.999905, 0.013774, 0, 0.999905, 0.013774, 0, 0.999905, 0.013774, 0, 0.999904, 0.013876, 0, 0.999904, 0.013876, 0, 0.999904, 0.013876, 0, 0.999904, 0.013876, 0, 0.999904, 0.013876, 0, 0.999904, 0.013876, 0, 0.999898, 0.014297, 0, 0.999898, 0.014297, 0, 0.999898, 0.014297, 0, 0.999898, 0.014297, 0, 0.999898, 0.014297, 0, 0.999898, 0.014297, 0, 0.999887, 0.015034, 0, 0.999887, 0.015034, 0, 0.999887, 0.015034, 0, 0.999887, 0.015034, 0, 0.999887, 0.015034, 0, 0.999887, 0.015034, 0, 0.999871, 0.016082, 0, 0.999871, 0.016082, 0, 0.999871, 0.016082, 0, 0.999871, 0.016082, 0, 0.999871, 0.016082, 0, 0.999871, 0.016082, 0, 0.999848, 0.017437, 0, 0.999848, 0.017437, 0, 0.999848, 0.017437, 0, 0.999848, 0.017437, 0, 0.999848, 0.017437, 0, 0.999848, 0.017437, 0, 0.999818, 0.019096, 0, 0.999818, 0.019096, 0, 0.999818, 0.019096, 0, 0.999818, 0.019096, 0, 0.999818, 0.019096, 0, 0.999818, 0.019096, 0, 0.999778, 0.021055, 0, 0.999778, 0.021055, 0, 0.999778, 0.021055, 0, 0.999778, 0.021055, 0, 0.999778, 0.021055, 0, 0.999778, 0.021055, 0, 0.999728, 0.023310, 0, 0.999728, 0.023310, 0, 0.999728, 0.023310, 0, 0.999728, 0.023310, 0, 0.999728, 0.023310, 0, 0.999728, 0.023310, 0, 0.999978, 0.006662, 0, 0.999978, 0.006662, 0, 0.999978, 0.006662, 0, 0.999978, 0.006662, 0, 0.999978, 0.006662, 0, 0.999978, 0.006662, 0, 0.999991, 0.004337, 0, 0.999991, 0.004337, 0, 0.999991, 0.004337, 0, 0.999991, 0.004337, 0, 0.999991, 0.004337, 0, 0.999991, 0.004337, 0, 0.999997, 0.002309, 0, 0.999997, 0.002309, 0, 0.999997, 0.002309, 0, 0.999997, 0.002309, 0, 0.999997, 0.002309, 0, 0.999997, 0.002309, 0, 1.000000, 0.000580, 0, 1.000000, 0.000580, 0, 1.000000, 0.000580, 0, 1.000000, 0.000580, 0, 1.000000, 0.000580, 0, 1.000000, 0.000580, 0, 1.000000, -0.000847, 0, 1.000000, -0.000847, 0, 1.000000, -0.000847, 0, 1.000000, -0.000847, 0, 1.000000, -0.000847, 0, 1.000000, -0.000847, 0, 0.999998, -0.001967, 0, 0.999998, -0.001967, 0, 0.999998, -0.001967, 0, 0.999998, -0.001967, 0, 0.999998, -0.001967, 0, 0.999998, -0.001967, 0, 0.999996, -0.002777, 0, 0.999996, -0.002777, 0, 0.999996, -0.002777, 0, 0.999996, -0.002777, 0, 0.999996, -0.002777, 0, 0.999996, -0.002777, 0, 0.999995, -0.003273, 0, 0.999995, -0.003273, 0, 0.999995, -0.003273, 0, 0.999995, -0.003273, 0, 0.999995, -0.003273, 0, 0.999995, -0.003273, 0, 0.999994, -0.003451, 0, 0.999994, -0.003451, 0, 0.999994, -0.003451, 0, 0.999994, -0.003451, 0, 0.999994, -0.003451, 0, 0.999994, -0.003451, 0, 0.999997, -0.002404, 0, 0.999997, -0.002404, 0, 0.999997, -0.002404, 0, 0.999997, -0.002404, 0, 0.999997, -0.002404, 0, 0.999997, -0.002404, 0, 1.000000, 0.000884, 0, 1.000000, 0.000884, 0, 1.000000, 0.000884, 0, 1.000000, 0.000884, 0, 1.000000, 0.000884, 0, 1.000000, 0.000884, 0, 0.999979, 0.006480, 0, 0.999979, 0.006480, 0, 0.999979, 0.006480, 0, 0.999979, 0.006480, 0, 0.999979, 0.006480, 0, 0.999979, 0.006480, 0, 0.999896, 0.014429, 0, 0.999896, 0.014429, 0, 0.999896, 0.014429, 0, 0.999896, 0.014429, 0, 0.999896, 0.014429, 0, 0.999896, 0.014429, 0, 0.999702, 0.024411, 0, 0.999702, 0.024411, 0, 0.999702, 0.024411, 0, 0.999702, 0.024411, 0, 0.999702, 0.024411, 0, 0.999702, 0.024411, 0, 0.999350, 0.036057, 0, 0.999350, 0.036057, 0, 0.999350, 0.036057, 0, 0.999350, 0.036057, 0, 0.999350, 0.036057, 0, 0.999350, 0.036057, 0, 0.998784, 0.049308, 0, 0.998784, 0.049308, 0, 0.998784, 0.049308, 0, 0.998784, 0.049308, 0, 0.998784, 0.049308, 0, 0.998784, 0.049308, 0, 0.997945, 0.064081, 0, 0.997945, 0.064081, 0, 0.997945, 0.064081, 0, 0.997945, 0.064081, 0, 0.997945, 0.064081, 0, 0.997945, 0.064081, 0, 0.996886, 0.078859, 0, 0.996886, 0.078859, 0, 0.996886, 0.078859, 0, 0.996886, 0.078859, 0, 0.996886, 0.078859, 0, 0.996886, 0.078859, 0, 0.995787, 0.091693, 0, 0.995787, 0.091693, 0, 0.995787, 0.091693, 0, 0.995787, 0.091693, 0, 0.995787, 0.091693, 0, 0.995787, 0.091693, 0, 0.994730, 0.102530, 0, 0.994730, 0.102530, 0, 0.994730, 0.102530, 0, 0.994730, 0.102530, 0, 0.994730, 0.102530, 0, 0.994730, 0.102530, 0, 0.993782, 0.111347, 0, 0.993782, 0.111347, 0, 0.993782, 0.111347, 0, 0.993782, 0.111347, 0, 0.993782, 0.111347, 0, 0.993782, 0.111347, 0, 0.993025, 0.117907, 0, 0.993025, 0.117907, 0, 0.993025, 0.117907, 0, 0.993025, 0.117907, 0, 0.993025, 0.117907, 0, 0.993025, 0.117907, 0, 0.992529, 0.122011, 0, 0.992529, 0.122011, 0, 0.992529, 0.122011, 0, 0.992529, 0.122011, 0, 0.992529, 0.122011, 0, 0.992529, 0.122011, 0, 0.992316, 0.123728, 0, 0.992316, 0.123728, 0, 0.992316, 0.123728, 0, 0.992316, 0.123728, 0, 0.992316, 0.123728, 0, 0.992316, 0.123728, 0, 0.992389, 0.123140, 0, 0.992389, 0.123140, 0, 0.992389, 0.123140, 0, 0.992389, 0.123140, 0, 0.992389, 0.123140, 0, 0.992389, 0.123140, 0, 0.992477, 0.122433, 0, 0.992477, 0.122433, 0, 0.992477, 0.122433, 0, 0.992477, 0.122433, 0, 0.992477, 0.122433, 0, 0.992477, 0.122433, 0, 0.992601, 0.121426, 0, 0.992601, 0.121426, 0, 0.992601, 0.121426, 0, 0.992601, 0.121426, 0, 0.992601, 0.121426, 0, 0.992601, 0.121426, 0, 0.992759, 0.120122, 0, 0.992759, 0.120122, 0, 0.992759, 0.120122, 0, 0.992759, 0.120122, 0, 0.992759, 0.120122, 0, 0.992759, 0.120122, 0, 0.992951, 0.118524, 0, 0.992951, 0.118524, 0, 0.992951, 0.118524, 0, 0.992951, 0.118524, 0, 0.992951, 0.118524, 0, 0.992951, 0.118524, 0, 0.993175, 0.116636, 0, 0.993175, 0.116636, 0, 0.993175, 0.116636, 0, 0.993175, 0.116636, 0, 0.993175, 0.116636, 0, 0.993175, 0.116636, 0, 0.993428, 0.114461, 0, 0.993428, 0.114461, 0, 0.993428, 0.114461, 0, 0.993428, 0.114461, 0, 0.993428, 0.114461, 0, 0.993428, 0.114461, 0, 0.993708, 0.112002, 0, 0.993708, 0.112002, 0, 0.993708, 0.112002, 0, 0.993708, 0.112002, 0, 0.993708, 0.112002, 0, 0.993708, 0.112002, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998894, 0.047011, 0, 0.998894, 0.047011, 0, 0.998894, 0.047011, 0, 0.998894, 0.047011, 0, 0.998894, 0.047011, 0, 0.998894, 0.047011, 0, 0.998700, 0.050965, 0, 0.998700, 0.050965, 0, 0.998700, 0.050965, 0, 0.998700, 0.050965, 0, 0.998700, 0.050965, 0, 0.998700, 0.050965, 0, 0.999111, 0.042157, 0, 0.999111, 0.042157, 0, 0.999111, 0.042157, 0, 0.999111, 0.042157, 0, 0.999111, 0.042157, 0, 0.999111, 0.042157, 0, 0.999407, 0.034428, 0, 0.999407, 0.034428, 0, 0.999407, 0.034428, 0, 0.999407, 0.034428, 0, 0.999407, 0.034428, 0, 0.999407, 0.034428, 0, 0.999613, 0.027801, 0, 0.999613, 0.027801, 0, 0.999613, 0.027801, 0, 0.999613, 0.027801, 0, 0.999613, 0.027801, 0, 0.999613, 0.027801, 0, 0.999752, 0.022291, 0, 0.999752, 0.022291, 0, 0.999752, 0.022291, 0, 0.999752, 0.022291, 0, 0.999752, 0.022291, 0, 0.999752, 0.022291, 0, 0.999839, 0.017958, 0, 0.999839, 0.017958, 0, 0.999839, 0.017958, 0, 0.999839, 0.017958, 0, 0.999839, 0.017958, 0, 0.999839, 0.017958, 0, 0.999890, 0.014853, 0, 0.999890, 0.014853, 0, 0.999890, 0.014853, 0, 0.999890, 0.014853, 0, 0.999890, 0.014853, 0, 0.999890, 0.014853, 0, 0.999916, 0.012964, 0, 0.999916, 0.012964, 0, 0.999916, 0.012964, 0, 0.999916, 0.012964, 0, 0.999916, 0.012964, 0, 0.999916, 0.012964, 0, 0.999925, 0.012268, 0, 0.999925, 0.012268, 0, 0.999925, 0.012268, 0, 0.999925, 0.012268, 0, 0.999925, 0.012268, 0, 0.999925, 0.012268, 0, 0.999924, 0.012349, 0, 0.999924, 0.012349, 0, 0.999924, 0.012349, 0, 0.999924, 0.012349, 0, 0.999924, 0.012349, 0, 0.999924, 0.012349, 0, 0.999919, 0.012716, 0, 0.999919, 0.012716, 0, 0.999919, 0.012716, 0, 0.999919, 0.012716, 0, 0.999919, 0.012716, 0, 0.999919, 0.012716, 0, 0.999911, 0.013363, 0, 0.999911, 0.013363, 0, 0.999911, 0.013363, 0, 0.999911, 0.013363, 0, 0.999911, 0.013363, 0, 0.999911, 0.013363, 0, 0.999898, 0.014286, 0, 0.999898, 0.014286, 0, 0.999898, 0.014286, 0, 0.999898, 0.014286, 0, 0.999898, 0.014286, 0, 0.999898, 0.014286, 0, 0.999880, 0.015483, 0, 0.999880, 0.015483, 0, 0.999880, 0.015483, 0, 0.999880, 0.015483, 0, 0.999880, 0.015483, 0, 0.999880, 0.015483, 0, 0.999856, 0.016947, 0, 0.999856, 0.016947, 0, 0.999856, 0.016947, 0, 0.999856, 0.016947, 0, 0.999856, 0.016947, 0, 0.999856, 0.016947, 0, 0.999826, 0.018677, 0, 0.999826, 0.018677, 0, 0.999826, 0.018677, 0, 0.999826, 0.018677, 0, 0.999826, 0.018677, 0, 0.999826, 0.018677, 0, 0.999786, 0.020667, 0, 0.999786, 0.020667, 0, 0.999786, 0.020667, 0, 0.999786, 0.020667, 0, 0.999786, 0.020667, 0, 0.999786, 0.020667, 0, 0.999983, 0.005906, 0, 0.999983, 0.005906, 0, 0.999983, 0.005906, 0, 0.999983, 0.005906, 0, 0.999983, 0.005906, 0, 0.999983, 0.005906, 0, 0.999992, 0.003885, 0, 0.999992, 0.003885, 0, 0.999992, 0.003885, 0, 0.999992, 0.003885, 0, 0.999992, 0.003885, 0, 0.999992, 0.003885, 0, 0.999998, 0.002107, 0, 0.999998, 0.002107, 0, 0.999998, 0.002107, 0, 0.999998, 0.002107, 0, 0.999998, 0.002107, 0, 0.999998, 0.002107, 0, 1.000000, 0.000576, 0, 1.000000, 0.000576, 0, 1.000000, 0.000576, 0, 1.000000, 0.000576, 0, 1.000000, 0.000576, 0, 1.000000, 0.000576, 0, 1.000000, -0.000705, 0, 1.000000, -0.000705, 0, 1.000000, -0.000705, 0, 1.000000, -0.000705, 0, 1.000000, -0.000705, 0, 1.000000, -0.000705, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999997, -0.002458, 0, 0.999997, -0.002458, 0, 0.999997, -0.002458, 0, 0.999997, -0.002458, 0, 0.999997, -0.002458, 0, 0.999997, -0.002458, 0, 0.999996, -0.002909, 0, 0.999996, -0.002909, 0, 0.999996, -0.002909, 0, 0.999996, -0.002909, 0, 0.999996, -0.002909, 0, 0.999996, -0.002909, 0, 0.999995, -0.003070, 0, 0.999995, -0.003070, 0, 0.999995, -0.003070, 0, 0.999995, -0.003070, 0, 0.999995, -0.003070, 0, 0.999995, -0.003070, 0, 0.999998, -0.002155, 0, 0.999998, -0.002155, 0, 0.999998, -0.002155, 0, 0.999998, -0.002155, 0, 0.999998, -0.002155, 0, 0.999998, -0.002155, 0, 1.000000, 0.000712, 0, 1.000000, 0.000712, 0, 1.000000, 0.000712, 0, 1.000000, 0.000712, 0, 1.000000, 0.000712, 0, 1.000000, 0.000712, 0, 0.999984, 0.005585, 0, 0.999984, 0.005585, 0, 0.999984, 0.005585, 0, 0.999984, 0.005585, 0, 0.999984, 0.005585, 0, 0.999984, 0.005585, 0, 0.999922, 0.012502, 0, 0.999922, 0.012502, 0, 0.999922, 0.012502, 0, 0.999922, 0.012502, 0, 0.999922, 0.012502, 0, 0.999922, 0.012502, 0, 0.999776, 0.021189, 0, 0.999776, 0.021189, 0, 0.999776, 0.021189, 0, 0.999776, 0.021189, 0, 0.999776, 0.021189, 0, 0.999776, 0.021189, 0, 0.999509, 0.031335, 0, 0.999509, 0.031335, 0, 0.999509, 0.031335, 0, 0.999509, 0.031335, 0, 0.999509, 0.031335, 0, 0.999509, 0.031335, 0, 0.999080, 0.042895, 0, 0.999080, 0.042895, 0, 0.999080, 0.042895, 0, 0.999080, 0.042895, 0, 0.999080, 0.042895, 0, 0.999080, 0.042895, 0, 0.998442, 0.055802, 0, 0.998442, 0.055802, 0, 0.998442, 0.055802, 0, 0.998442, 0.055802, 0, 0.998442, 0.055802, 0, 0.998442, 0.055802, 0, 0.997616, 0.069005, 0, 0.997616, 0.069005, 0, 0.997616, 0.069005, 0, 0.997616, 0.069005, 0, 0.997616, 0.069005, 0, 0.997616, 0.069005, 0, 0.996750, 0.080560, 0, 0.996750, 0.080560, 0, 0.996750, 0.080560, 0, 0.996750, 0.080560, 0, 0.996750, 0.080560, 0, 0.996750, 0.080560, 0, 0.995903, 0.090423, 0, 0.995903, 0.090423, 0, 0.995903, 0.090423, 0, 0.995903, 0.090423, 0, 0.995903, 0.090423, 0, 0.995903, 0.090423, 0, 0.995130, 0.098572, 0, 0.995130, 0.098572, 0, 0.995130, 0.098572, 0, 0.995130, 0.098572, 0, 0.995130, 0.098572, 0, 0.995130, 0.098572, 0, 0.994500, 0.104741, 0, 0.994500, 0.104741, 0, 0.994500, 0.104741, 0, 0.994500, 0.104741, 0, 0.994500, 0.104741, 0, 0.994500, 0.104741, 0, 0.994075, 0.108698, 0, 0.994075, 0.108698, 0, 0.994075, 0.108698, 0, 0.994075, 0.108698, 0, 0.994075, 0.108698, 0, 0.994075, 0.108698, 0, 0.993877, 0.110497, 0, 0.993877, 0.110497, 0, 0.993877, 0.110497, 0, 0.993877, 0.110497, 0, 0.993877, 0.110497, 0, 0.993877, 0.110497, 0, 0.993909, 0.110201, 0, 0.993909, 0.110201, 0, 0.993909, 0.110201, 0, 0.993909, 0.110201, 0, 0.993909, 0.110201, 0, 0.993909, 0.110201, 0, 0.994125, 0.108240, 0, 0.994125, 0.108240, 0, 0.994125, 0.108240, 0, 0.994125, 0.108240, 0, 0.994125, 0.108240, 0, 0.994125, 0.108240, 0, 0.994499, 0.104750, 0, 0.994499, 0.104750, 0, 0.994499, 0.104750, 0, 0.994499, 0.104750, 0, 0.994499, 0.104750, 0, 0.994499, 0.104750, 0, 0.995012, 0.099754, 0, 0.995012, 0.099754, 0, 0.995012, 0.099754, 0, 0.995012, 0.099754, 0, 0.995012, 0.099754, 0, 0.995012, 0.099754, 0, 0.995641, 0.093265, 0, 0.995641, 0.093265, 0, 0.995641, 0.093265, 0, 0.995641, 0.093265, 0, 0.995641, 0.093265, 0, 0.995641, 0.093265, 0, 0.996335, 0.085535, 0, 0.996335, 0.085535, 0, 0.996335, 0.085535, 0, 0.996335, 0.085535, 0, 0.996335, 0.085535, 0, 0.996335, 0.085535, 0, 0.997047, 0.076799, 0, 0.997047, 0.076799, 0, 0.997047, 0.076799, 0, 0.997047, 0.076799, 0, 0.997047, 0.076799, 0, 0.997047, 0.076799, 0, 0.997751, 0.067023, 0, 0.997751, 0.067023, 0, 0.997751, 0.067023, 0, 0.997751, 0.067023, 0, 0.997751, 0.067023, 0, 0.997751, 0.067023, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997604, 0.069187, 0, 0.997604, 0.069187, 0, 0.997604, 0.069187, 0, 0.997604, 0.069187, 0, 0.997604, 0.069187, 0, 0.997604, 0.069187, 0, 0.997180, 0.075049, 0, 0.997180, 0.075049, 0, 0.997180, 0.075049, 0, 0.997180, 0.075049, 0, 0.997180, 0.075049, 0, 0.997180, 0.075049, 0, 0.996770, 0.080310, 0, 0.996770, 0.080310, 0, 0.996770, 0.080310, 0, 0.996770, 0.080310, 0, 0.996770, 0.080310, 0, 0.996770, 0.080310, 0, 0.996382, 0.084984, 0, 0.996382, 0.084984, 0, 0.996382, 0.084984, 0, 0.996382, 0.084984, 0, 0.996382, 0.084984, 0, 0.996382, 0.084984, 0, 0.996040, 0.088902, 0, 0.996040, 0.088902, 0, 0.996040, 0.088902, 0, 0.996040, 0.088902, 0, 0.996040, 0.088902, 0, 0.996040, 0.088902, 0, 0.995770, 0.091886, 0, 0.995770, 0.091886, 0, 0.995770, 0.091886, 0, 0.995770, 0.091886, 0, 0.995770, 0.091886, 0, 0.995770, 0.091886, 0, 0.995579, 0.093931, 0, 0.995579, 0.093931, 0, 0.995579, 0.093931, 0, 0.995579, 0.093931, 0, 0.995579, 0.093931, 0, 0.995579, 0.093931, 0, 0.995475, 0.095026, 0, 0.995475, 0.095026, 0, 0.995475, 0.095026, 0, 0.995475, 0.095026, 0, 0.995475, 0.095026, 0, 0.995475, 0.095026, 0, 0.995459, 0.095192, 0, 0.995459, 0.095192, 0, 0.995459, 0.095192, 0, 0.995459, 0.095192, 0, 0.995459, 0.095192, 0, 0.995459, 0.095192, 0, 0.995546, 0.094279, 0, 0.995546, 0.094279, 0, 0.995546, 0.094279, 0, 0.995546, 0.094279, 0, 0.995546, 0.094279, 0, 0.995546, 0.094279, 0, 0.995734, 0.092267, 0, 0.995734, 0.092267, 0, 0.995734, 0.092267, 0, 0.995734, 0.092267, 0, 0.995734, 0.092267, 0, 0.995734, 0.092267, 0, 0.996019, 0.089142, 0, 0.996019, 0.089142, 0, 0.996019, 0.089142, 0, 0.996019, 0.089142, 0, 0.996019, 0.089142, 0, 0.996019, 0.089142, 0, 0.996383, 0.084972, 0, 0.996383, 0.084972, 0, 0.996383, 0.084972, 0, 0.996383, 0.084972, 0, 0.996383, 0.084972, 0, 0.996383, 0.084972, 0, 0.996808, 0.079838, 0, 0.996808, 0.079838, 0, 0.996808, 0.079838, 0, 0.996808, 0.079838, 0, 0.996808, 0.079838, 0, 0.996808, 0.079838, 0, 0.997277, 0.073741, 0, 0.997277, 0.073741, 0, 0.997277, 0.073741, 0, 0.997277, 0.073741, 0, 0.997277, 0.073741, 0, 0.997277, 0.073741, 0, 0.997774, 0.066691, 0, 0.997774, 0.066691, 0, 0.997774, 0.066691, 0, 0.997774, 0.066691, 0, 0.997774, 0.066691, 0, 0.997774, 0.066691, 0, 0.995668, 0.092983, 0, 0.995668, 0.092983, 0, 0.995668, 0.092983, 0, 0.995668, 0.092983, 0, 0.995668, 0.092983, 0, 0.995668, 0.092983, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.991561, 0.129640, 0, 0.991561, 0.129640, 0, 0.991561, 0.129640, 0, 0.991561, 0.129640, 0, 0.991561, 0.129640, 0, 0.991561, 0.129640, 0, 0.993779, 0.111368, 0, 0.993779, 0.111368, 0, 0.993779, 0.111368, 0, 0.993779, 0.111368, 0, 0.993779, 0.111368, 0, 0.993779, 0.111368, 0, 0.993586, 0.113081, 0, 0.993586, 0.113081, 0, 0.993586, 0.113081, 0, 0.993586, 0.113081, 0, 0.993586, 0.113081, 0, 0.993586, 0.113081, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997669, 0.068243, 0, 0.997669, 0.068243, 0, 0.997669, 0.068243, 0, 0.997669, 0.068243, 0, 0.997669, 0.068243, 0, 0.997669, 0.068243, 0, 0.997536, 0.070158, 0, 0.997536, 0.070158, 0, 0.997536, 0.070158, 0, 0.997536, 0.070158, 0, 0.997536, 0.070158, 0, 0.997536, 0.070158, 0, 0.997415, 0.071860, 0, 0.997415, 0.071860, 0, 0.997415, 0.071860, 0, 0.997415, 0.071860, 0, 0.997415, 0.071860, 0, 0.997415, 0.071860, 0, 0.997307, 0.073346, 0, 0.997307, 0.073346, 0, 0.997307, 0.073346, 0, 0.997307, 0.073346, 0, 0.997307, 0.073346, 0, 0.997307, 0.073346, 0, 0.997212, 0.074616, 0, 0.997212, 0.074616, 0, 0.997212, 0.074616, 0, 0.997212, 0.074616, 0, 0.997212, 0.074616, 0, 0.997212, 0.074616, 0, 0.997133, 0.075667, 0, 0.997133, 0.075667, 0, 0.997133, 0.075667, 0, 0.997133, 0.075667, 0, 0.997133, 0.075667, 0, 0.997133, 0.075667, 0, 0.997070, 0.076498, 0, 0.997070, 0.076498, 0, 0.997070, 0.076498, 0, 0.997070, 0.076498, 0, 0.997070, 0.076498, 0, 0.997070, 0.076498, 0, 0.997023, 0.077106, 0, 0.997023, 0.077106, 0, 0.997023, 0.077106, 0, 0.997023, 0.077106, 0, 0.997023, 0.077106, 0, 0.997023, 0.077106, 0, 0.996993, 0.077491, 0, 0.996993, 0.077491, 0, 0.996993, 0.077491, 0, 0.996993, 0.077491, 0, 0.996993, 0.077491, 0, 0.996993, 0.077491, 0, 0.996979, 0.077668, 0, 0.996979, 0.077668, 0, 0.996979, 0.077668, 0, 0.996979, 0.077668, 0, 0.996979, 0.077668, 0, 0.996979, 0.077668, 0, 0.997003, 0.077369, 0, 0.997003, 0.077369, 0, 0.997003, 0.077369, 0, 0.997003, 0.077369, 0, 0.997003, 0.077369, 0, 0.997003, 0.077369, 0, 0.997063, 0.076587, 0, 0.997063, 0.076587, 0, 0.997063, 0.076587, 0, 0.997063, 0.076587, 0, 0.997063, 0.076587, 0, 0.997063, 0.076587, 0, 0.997160, 0.075318, 0, 0.997160, 0.075318, 0, 0.997160, 0.075318, 0, 0.997160, 0.075318, 0, 0.997160, 0.075318, 0, 0.997160, 0.075318, 0, 0.997291, 0.073560, 0, 0.997291, 0.073560, 0, 0.997291, 0.073560, 0, 0.997291, 0.073560, 0, 0.997291, 0.073560, 0, 0.997291, 0.073560, 0, 0.997454, 0.071313, 0, 0.997454, 0.071313, 0, 0.997454, 0.071313, 0, 0.997454, 0.071313, 0, 0.997454, 0.071313, 0, 0.997454, 0.071313, 0, 0.997646, 0.068578, 0, 0.997646, 0.068578, 0, 0.997646, 0.068578, 0, 0.997646, 0.068578, 0, 0.997646, 0.068578, 0, 0.997646, 0.068578, 0, 0.997862, 0.065358, 0, 0.997862, 0.065358, 0, 0.997862, 0.065358, 0, 0.997862, 0.065358, 0, 0.997862, 0.065358, 0, 0.997862, 0.065358, 0, 0.998023, 0.062855, 0, 0.998023, 0.062855, 0, 0.998023, 0.062855, 0, 0.998023, 0.062855, 0, 0.998023, 0.062855, 0, 0.998023, 0.062855, 0, 0.998162, 0.060602, 0, 0.998162, 0.060602, 0, 0.998162, 0.060602, 0, 0.998162, 0.060602, 0, 0.998162, 0.060602, 0, 0.998162, 0.060602, 0, 0.998281, 0.058601, 0, 0.998281, 0.058601, 0, 0.998281, 0.058601, 0, 0.998281, 0.058601, 0, 0.998281, 0.058601, 0, 0.998281, 0.058601, 0, 0.998382, 0.056855, 0, 0.998382, 0.056855, 0, 0.998382, 0.056855, 0, 0.998382, 0.056855, 0, 0.998382, 0.056855, 0, 0.998382, 0.056855, 0, 0.998474, 0.055232, 0, 0.998474, 0.055232, 0, 0.998474, 0.055232, 0, 0.998474, 0.055232, 0, 0.998474, 0.055232, 0, 0.998474, 0.055232, 0, 0.998562, 0.053603, 0, 0.998562, 0.053603, 0, 0.998562, 0.053603, 0, 0.998562, 0.053603, 0, 0.998562, 0.053603, 0, 0.998562, 0.053603, 0, 0.998648, 0.051974, 0, 0.998648, 0.051974, 0, 0.998648, 0.051974, 0, 0.998648, 0.051974, 0, 0.998648, 0.051974, 0, 0.998648, 0.051974, 0, 0.998732, 0.050348, 0, 0.998732, 0.050348, 0, 0.998732, 0.050348, 0, 0.998732, 0.050348, 0, 0.998732, 0.050348, 0, 0.998732, 0.050348, 0, 0.998798, 0.049011, 0, 0.998798, 0.049011, 0, 0.998798, 0.049011, 0, 0.998798, 0.049011, 0, 0.998798, 0.049011, 0, 0.998798, 0.049011, 0, 0.998900, 0.046899, 0, 0.998900, 0.046899, 0, 0.998900, 0.046899, 0, 0.998900, 0.046899, 0, 0.998900, 0.046899, 0, 0.998900, 0.046899, 0, 0.999008, 0.044540, 0, 0.999008, 0.044540, 0, 0.999008, 0.044540, 0, 0.999008, 0.044540, 0, 0.999008, 0.044540, 0, 0.999008, 0.044540, 0, 0.999121, 0.041931, 0, 0.999121, 0.041931, 0, 0.999121, 0.041931, 0, 0.999121, 0.041931, 0, 0.999121, 0.041931, 0, 0.999121, 0.041931, 0, 0.999237, 0.039067, 0, 0.999237, 0.039067, 0, 0.999237, 0.039067, 0, 0.999237, 0.039067, 0, 0.999237, 0.039067, 0, 0.999237, 0.039067, 0, 0.999359, 0.035812, 0, 0.999359, 0.035812, 0, 0.999359, 0.035812, 0, 0.999359, 0.035812, 0, 0.999359, 0.035812, 0, 0.999359, 0.035812, 0, 0.999487, 0.032032, 0, 0.999487, 0.032032, 0, 0.999487, 0.032032, 0, 0.999487, 0.032032, 0, 0.999487, 0.032032, 0, 0.999487, 0.032032, 0, 0.999616, 0.027720, 0, 0.999616, 0.027720, 0, 0.999616, 0.027720, 0, 0.999616, 0.027720, 0, 0.999616, 0.027720, 0, 0.999616, 0.027720, 0, 0.999739, 0.022868, 0, 0.999739, 0.022868, 0, 0.999739, 0.022868, 0, 0.999739, 0.022868, 0, 0.999739, 0.022868, 0, 0.999739, 0.022868, 0, 0.999849, 0.017352, 0, 0.999849, 0.017352, 0, 0.999849, 0.017352, 0, 0.999849, 0.017352, 0, 0.999849, 0.017352, 0, 0.999849, 0.017352, 0, 0.999923, 0.012448, 0, 0.999923, 0.012448, 0, 0.999923, 0.012448, 0, 0.999923, 0.012448, 0, 0.999923, 0.012448, 0, 0.999923, 0.012448, 0, 0.999967, 0.008144, 0, 0.999967, 0.008144, 0, 0.999967, 0.008144, 0, 0.999967, 0.008144, 0, 0.999967, 0.008144, 0, 0.999967, 0.008144, 0, 0.999990, 0.004430, 0, 0.999990, 0.004430, 0, 0.999990, 0.004430, 0, 0.999990, 0.004430, 0, 0.999990, 0.004430, 0, 0.999990, 0.004430, 0, 0.999999, 0.001425, 0, 0.999999, 0.001425, 0, 0.999999, 0.001425, 0, 0.999999, 0.001425, 0, 0.999999, 0.001425, 0, 0.999999, 0.001425, 0, 1.000000, -0.000742, 0, 1.000000, -0.000742, 0, 1.000000, -0.000742, 0, 1.000000, -0.000742, 0, 1.000000, -0.000742, 0, 1.000000, -0.000742, 0, 0.999998, -0.002062, 0, 0.999998, -0.002062, 0, 0.999998, -0.002062, 0, 0.999998, -0.002062, 0, 0.999998, -0.002062, 0, 0.999998, -0.002062, 0, 0.999997, -0.002520, 0, 0.999997, -0.002520, 0, 0.999997, -0.002520, 0, 0.999997, -0.002520, 0, 0.999997, -0.002520, 0, 0.999997, -0.002520, 0, 0.999997, -0.002450, 0, 0.999997, -0.002450, 0, 0.999997, -0.002450, 0, 0.999997, -0.002450, 0, 0.999997, -0.002450, 0, 0.999997, -0.002450, 0, 0.999998, -0.002221, 0, 0.999998, -0.002221, 0, 0.999998, -0.002221, 0, 0.999998, -0.002221, 0, 0.999998, -0.002221, 0, 0.999998, -0.002221, 0, 0.999998, -0.001832, 0, 0.999998, -0.001832, 0, 0.999998, -0.001832, 0, 0.999998, -0.001832, 0, 0.999998, -0.001832, 0, 0.999998, -0.001832, 0, 0.999999, -0.001279, 0, 0.999999, -0.001279, 0, 0.999999, -0.001279, 0, 0.999999, -0.001279, 0, 0.999999, -0.001279, 0, 0.999999, -0.001279, 0, 1.000000, -0.000562, 0, 1.000000, -0.000562, 0, 1.000000, -0.000562, 0, 1.000000, -0.000562, 0, 1.000000, -0.000562, 0, 1.000000, -0.000562, 0, 1.000000, 0.000322, 0, 1.000000, 0.000322, 0, 1.000000, 0.000322, 0, 1.000000, 0.000322, 0, 1.000000, 0.000322, 0, 1.000000, 0.000322, 0, 0.999999, 0.001376, 0, 0.999999, 0.001376, 0, 0.999999, 0.001376, 0, 0.999999, 0.001376, 0, 0.999999, 0.001376, 0, 0.999999, 0.001376, 0, 0.999997, 0.002601, 0, 0.999997, 0.002601, 0, 0.999997, 0.002601, 0, 0.999997, 0.002601, 0, 0.999997, 0.002601, 0, 0.999997, 0.002601, 0, 0.999834, 0.018204, 0, 0.999834, 0.018204, 0, 0.999834, 0.018204, 0, 0.999834, 0.018204, 0, 0.999834, 0.018204, 0, 0.999834, 0.018204, 0, 0.999867, 0.016309, 0, 0.999867, 0.016309, 0, 0.999867, 0.016309, 0, 0.999867, 0.016309, 0, 0.999867, 0.016309, 0, 0.999867, 0.016309, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999892, 0.014671, 0, 0.999912, 0.013287, 0, 0.999912, 0.013287, 0, 0.999912, 0.013287, 0, 0.999912, 0.013287, 0, 0.999912, 0.013287, 0, 0.999912, 0.013287, 0, 0.999926, 0.012155, 0, 0.999926, 0.012155, 0, 0.999926, 0.012155, 0, 0.999926, 0.012155, 0, 0.999926, 0.012155, 0, 0.999926, 0.012155, 0, 0.999937, 0.011271, 0, 0.999937, 0.011271, 0, 0.999937, 0.011271, 0, 0.999937, 0.011271, 0, 0.999937, 0.011271, 0, 0.999937, 0.011271, 0, 0.999943, 0.010632, 0, 0.999943, 0.010632, 0, 0.999943, 0.010632, 0, 0.999943, 0.010632, 0, 0.999943, 0.010632, 0, 0.999943, 0.010632, 0, 0.999948, 0.010236, 0, 0.999948, 0.010236, 0, 0.999948, 0.010236, 0, 0.999948, 0.010236, 0, 0.999948, 0.010236, 0, 0.999948, 0.010236, 0, 0.999949, 0.010078, 0, 0.999949, 0.010078, 0, 0.999949, 0.010078, 0, 0.999949, 0.010078, 0, 0.999949, 0.010078, 0, 0.999949, 0.010078, 0, 0.999948, 0.010239, 0, 0.999948, 0.010239, 0, 0.999948, 0.010239, 0, 0.999948, 0.010239, 0, 0.999948, 0.010239, 0, 0.999948, 0.010239, 0, 0.999941, 0.010822, 0, 0.999941, 0.010822, 0, 0.999941, 0.010822, 0, 0.999941, 0.010822, 0, 0.999941, 0.010822, 0, 0.999941, 0.010822, 0, 0.999930, 0.011821, 0, 0.999930, 0.011821, 0, 0.999930, 0.011821, 0, 0.999930, 0.011821, 0, 0.999930, 0.011821, 0, 0.999930, 0.011821, 0, 0.999912, 0.013235, 0, 0.999912, 0.013235, 0, 0.999912, 0.013235, 0, 0.999912, 0.013235, 0, 0.999912, 0.013235, 0, 0.999912, 0.013235, 0, 0.999887, 0.015027, 0, 0.999887, 0.015027, 0, 0.999887, 0.015027, 0, 0.999887, 0.015027, 0, 0.999887, 0.015027, 0, 0.999887, 0.015027, 0, 0.999853, 0.017161, 0, 0.999853, 0.017161, 0, 0.999853, 0.017161, 0, 0.999853, 0.017161, 0, 0.999853, 0.017161, 0, 0.999853, 0.017161, 0, 0.999807, 0.019641, 0, 0.999807, 0.019641, 0, 0.999807, 0.019641, 0, 0.999807, 0.019641, 0, 0.999807, 0.019641, 0, 0.999807, 0.019641, 0, 0.999748, 0.022469, 0, 0.999748, 0.022469, 0, 0.999748, 0.022469, 0, 0.999748, 0.022469, 0, 0.999748, 0.022469, 0, 0.999748, 0.022469, 0, 0.999685, 0.025087, 0, 0.999685, 0.025087, 0, 0.999685, 0.025087, 0, 0.999685, 0.025087, 0, 0.999685, 0.025087, 0, 0.999685, 0.025087, 0, 0.999625, 0.027397, 0, 0.999625, 0.027397, 0, 0.999625, 0.027397, 0, 0.999625, 0.027397, 0, 0.999625, 0.027397, 0, 0.999625, 0.027397, 0, 0.999568, 0.029401, 0, 0.999568, 0.029401, 0, 0.999568, 0.029401, 0, 0.999568, 0.029401, 0, 0.999568, 0.029401, 0, 0.999568, 0.029401, 0, 0.999516, 0.031101, 0, 0.999516, 0.031101, 0, 0.999516, 0.031101, 0, 0.999516, 0.031101, 0, 0.999516, 0.031101, 0, 0.999516, 0.031101, 0, 0.999467, 0.032657, 0, 0.999467, 0.032657, 0, 0.999467, 0.032657, 0, 0.999467, 0.032657, 0, 0.999467, 0.032657, 0, 0.999467, 0.032657, 0, 0.999414, 0.034231, 0, 0.999414, 0.034231, 0, 0.999414, 0.034231, 0, 0.999414, 0.034231, 0, 0.999414, 0.034231, 0, 0.999414, 0.034231, 0, 0.999358, 0.035830, 0, 0.999358, 0.035830, 0, 0.999358, 0.035830, 0, 0.999358, 0.035830, 0, 0.999358, 0.035830, 0, 0.999358, 0.035830, 0, 0.999298, 0.037460, 0, 0.999298, 0.037460, 0, 0.999298, 0.037460, 0, 0.999298, 0.037460, 0, 0.999298, 0.037460, 0, 0.999298, 0.037460, 0, 0.999246, 0.038834, 0, 0.999246, 0.038834, 0, 0.999246, 0.038834, 0, 0.999246, 0.038834, 0, 0.999246, 0.038834, 0, 0.999246, 0.038834, 0, 0.999170, 0.040745, 0, 0.999170, 0.040745, 0, 0.999170, 0.040745, 0, 0.999170, 0.040745, 0, 0.999170, 0.040745, 0, 0.999170, 0.040745, 0, 0.999074, 0.043031, 0, 0.999074, 0.043031, 0, 0.999074, 0.043031, 0, 0.999074, 0.043031, 0, 0.999074, 0.043031, 0, 0.999074, 0.043031, 0, 0.998956, 0.045691, 0, 0.998956, 0.045691, 0, 0.998956, 0.045691, 0, 0.998956, 0.045691, 0, 0.998956, 0.045691, 0, 0.998956, 0.045691, 0, 0.998812, 0.048723, 0, 0.998812, 0.048723, 0, 0.998812, 0.048723, 0, 0.998812, 0.048723, 0, 0.998812, 0.048723, 0, 0.998812, 0.048723, 0, 0.998636, 0.052217, 0, 0.998636, 0.052217, 0, 0.998636, 0.052217, 0, 0.998636, 0.052217, 0, 0.998636, 0.052217, 0, 0.998636, 0.052217, 0, 0.998416, 0.056262, 0, 0.998416, 0.056262, 0, 0.998416, 0.056262, 0, 0.998416, 0.056262, 0, 0.998416, 0.056262, 0, 0.998416, 0.056262, 0, 0.998147, 0.060852, 0, 0.998147, 0.060852, 0, 0.998147, 0.060852, 0, 0.998147, 0.060852, 0, 0.998147, 0.060852, 0, 0.998147, 0.060852, 0, 0.997821, 0.065976, 0, 0.997821, 0.065976, 0, 0.997821, 0.065976, 0, 0.997821, 0.065976, 0, 0.997821, 0.065976, 0, 0.997821, 0.065976, 0, 0.997399, 0.072078, 0, 0.997399, 0.072078, 0, 0.997399, 0.072078, 0, 0.997399, 0.072078, 0, 0.997399, 0.072078, 0, 0.997399, 0.072078, 0, 0.997005, 0.077342, 0, 0.997005, 0.077342, 0, 0.997005, 0.077342, 0, 0.997005, 0.077342, 0, 0.997005, 0.077342, 0, 0.997005, 0.077342, 0, 0.996652, 0.081756, 0, 0.996652, 0.081756, 0, 0.996652, 0.081756, 0, 0.996652, 0.081756, 0, 0.996652, 0.081756, 0, 0.996652, 0.081756, 0, 0.996354, 0.085315, 0, 0.996354, 0.085315, 0, 0.996354, 0.085315, 0, 0.996354, 0.085315, 0, 0.996354, 0.085315, 0, 0.996354, 0.085315, 0, 0.996125, 0.087953, 0, 0.996125, 0.087953, 0, 0.996125, 0.087953, 0, 0.996125, 0.087953, 0, 0.996125, 0.087953, 0, 0.996125, 0.087953, 0, 0.995977, 0.089612, 0, 0.995977, 0.089612, 0, 0.995977, 0.089612, 0, 0.995977, 0.089612, 0, 0.995977, 0.089612, 0, 0.995977, 0.089612, 0, 0.995914, 0.090305, 0, 0.995914, 0.090305, 0, 0.995914, 0.090305, 0, 0.995914, 0.090305, 0, 0.995914, 0.090305, 0, 0.995914, 0.090305, 0, 0.995937, 0.090049, 0, 0.995937, 0.090049, 0, 0.995937, 0.090049, 0, 0.995937, 0.090049, 0, 0.995937, 0.090049, 0, 0.995937, 0.090049, 0, 0.995979, 0.089583, 0, 0.995979, 0.089583, 0, 0.995979, 0.089583, 0, 0.995979, 0.089583, 0, 0.995979, 0.089583, 0, 0.995979, 0.089583, 0, 0.996041, 0.088899, 0, 0.996041, 0.088899, 0, 0.996041, 0.088899, 0, 0.996041, 0.088899, 0, 0.996041, 0.088899, 0, 0.996041, 0.088899, 0, 0.996120, 0.088000, 0, 0.996120, 0.088000, 0, 0.996120, 0.088000, 0, 0.996120, 0.088000, 0, 0.996120, 0.088000, 0, 0.996120, 0.088000, 0, 0.996218, 0.086889, 0, 0.996218, 0.086889, 0, 0.996218, 0.086889, 0, 0.996218, 0.086889, 0, 0.996218, 0.086889, 0, 0.996218, 0.086889, 0, 0.996332, 0.085567, 0, 0.996332, 0.085567, 0, 0.996332, 0.085567, 0, 0.996332, 0.085567, 0, 0.996332, 0.085567, 0, 0.996332, 0.085567, 0, 0.996463, 0.084037, 0, 0.996463, 0.084037, 0, 0.996463, 0.084037, 0, 0.996463, 0.084037, 0, 0.996463, 0.084037, 0, 0.996463, 0.084037, 0, 0.996608, 0.082299, 0, 0.996608, 0.082299, 0, 0.996608, 0.082299, 0, 0.996608, 0.082299, 0, 0.996608, 0.082299, 0, 0.996608, 0.082299, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.995412, 0.095681, 0, 0.995412, 0.095681, 0, 0.995412, 0.095681, 0, 0.995412, 0.095681, 0, 0.995412, 0.095681, 0, 0.995412, 0.095681, 0, 0.995301, 0.096834, 0, 0.995301, 0.096834, 0, 0.995301, 0.096834, 0, 0.995301, 0.096834, 0, 0.995301, 0.096834, 0, 0.995301, 0.096834, 0, 0.995288, 0.096964, 0, 0.995288, 0.096964, 0, 0.995288, 0.096964, 0, 0.995288, 0.096964, 0, 0.995288, 0.096964, 0, 0.995288, 0.096964, 0, 0.995314, 0.096695, 0, 0.995314, 0.096695, 0, 0.995314, 0.096695, 0, 0.995314, 0.096695, 0, 0.995314, 0.096695, 0, 0.995314, 0.096695, 0, 0.995379, 0.096025, 0, 0.995379, 0.096025, 0, 0.995379, 0.096025, 0, 0.995379, 0.096025, 0, 0.995379, 0.096025, 0, 0.995379, 0.096025, 0, 0.995482, 0.094951, 0, 0.995482, 0.094951, 0, 0.995482, 0.094951, 0, 0.995482, 0.094951, 0, 0.995482, 0.094951, 0, 0.995482, 0.094951, 0, 0.995631, 0.093376, 0, 0.995631, 0.093376, 0, 0.995631, 0.093376, 0, 0.995631, 0.093376, 0, 0.995631, 0.093376, 0, 0.995631, 0.093376, 0, 0.995832, 0.091204, 0, 0.995832, 0.091204, 0, 0.995832, 0.091204, 0, 0.995832, 0.091204, 0, 0.995832, 0.091204, 0, 0.995832, 0.091204, 0, 0.996082, 0.088433, 0, 0.996082, 0.088433, 0, 0.996082, 0.088433, 0, 0.996082, 0.088433, 0, 0.996082, 0.088433, 0, 0.996082, 0.088433, 0, 0.996376, 0.085064, 0, 0.996376, 0.085064, 0, 0.996376, 0.085064, 0, 0.996376, 0.085064, 0, 0.996376, 0.085064, 0, 0.996376, 0.085064, 0, 0.996945, 0.078104, 0, 0.996945, 0.078104, 0, 0.996945, 0.078104, 0, 0.996945, 0.078104, 0, 0.996945, 0.078104, 0, 0.996945, 0.078104, 0, 0.997070, 0.076493, 0, 0.997070, 0.076493, 0, 0.997070, 0.076493, 0, 0.997070, 0.076493, 0, 0.997070, 0.076493, 0, 0.997070, 0.076493, 0, 0.997845, 0.065616, 0, 0.997845, 0.065616, 0, 0.997845, 0.065616, 0, 0.997845, 0.065616, 0, 0.997845, 0.065616, 0, 0.997845, 0.065616, 0, 0.997753, 0.067001, 0, 0.997753, 0.067001, 0, 0.997753, 0.067001, 0, 0.997753, 0.067001, 0, 0.997753, 0.067001, 0, 0.997753, 0.067001, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.997694, 0.067875, 0, 0.997694, 0.067875, 0, 0.997694, 0.067875, 0, 0.997694, 0.067875, 0, 0.997694, 0.067875, 0, 0.997694, 0.067875, 0, 0.997635, 0.068728, 0, 0.997635, 0.068728, 0, 0.997635, 0.068728, 0, 0.997635, 0.068728, 0, 0.997635, 0.068728, 0, 0.997635, 0.068728, 0, 0.996786, 0.080115, 0, 0.996786, 0.080115, 0, 0.996786, 0.080115, 0, 0.996786, 0.080115, 0, 0.996786, 0.080115, 0, 0.996786, 0.080115, 0, 0.996865, 0.079122, 0, 0.996865, 0.079122, 0, 0.996865, 0.079122, 0, 0.996865, 0.079122, 0, 0.996865, 0.079122, 0, 0.996865, 0.079122, 0, 0.996407, 0.084699, 0, 0.996407, 0.084699, 0, 0.996407, 0.084699, 0, 0.996407, 0.084699, 0, 0.996407, 0.084699, 0, 0.996407, 0.084699, 0, 0.995856, 0.090949, 0, 0.995856, 0.090949, 0, 0.995856, 0.090949, 0, 0.995856, 0.090949, 0, 0.995856, 0.090949, 0, 0.995856, 0.090949, 0, 0.995362, 0.096198, 0, 0.995362, 0.096198, 0, 0.995362, 0.096198, 0, 0.995362, 0.096198, 0, 0.995362, 0.096198, 0, 0.995362, 0.096198, 0, 0.994943, 0.100441, 0, 0.994943, 0.100441, 0, 0.994943, 0.100441, 0, 0.994943, 0.100441, 0, 0.994943, 0.100441, 0, 0.994943, 0.100441, 0, 0.994611, 0.103678, 0, 0.994611, 0.103678, 0, 0.994611, 0.103678, 0, 0.994611, 0.103678, 0, 0.994611, 0.103678, 0, 0.994611, 0.103678, 0, 0.994369, 0.105977, 0, 0.994369, 0.105977, 0, 0.994369, 0.105977, 0, 0.994369, 0.105977, 0, 0.994369, 0.105977, 0, 0.994369, 0.105977, 0, 0.994215, 0.107408, 0, 0.994215, 0.107408, 0, 0.994215, 0.107408, 0, 0.994215, 0.107408, 0, 0.994215, 0.107408, 0, 0.994215, 0.107408, 0, 0.994153, 0.107978, 0, 0.994153, 0.107978, 0, 0.994153, 0.107978, 0, 0.994153, 0.107978, 0, 0.994153, 0.107978, 0, 0.994153, 0.107978, 0, 0.994183, 0.107702, 0, 0.994183, 0.107702, 0, 0.994183, 0.107702, 0, 0.994183, 0.107702, 0, 0.994183, 0.107702, 0, 0.994183, 0.107702, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997148, 0.075470, 0, 0.997148, 0.075470, 0, 0.997148, 0.075470, 0, 0.997148, 0.075470, 0, 0.997148, 0.075470, 0, 0.997148, 0.075470, 0, 0.997217, 0.074548, 0, 0.997217, 0.074548, 0, 0.997217, 0.074548, 0, 0.997217, 0.074548, 0, 0.997217, 0.074548, 0, 0.997217, 0.074548, 0, 0.999568, 0.029389, 0, 0.999568, 0.029389, 0, 0.999568, 0.029389, 0, 0.999568, 0.029389, 0, 0.999568, 0.029389, 0, 0.999568, 0.029389, 0, 0.999700, 0.024493, 0, 0.999700, 0.024493, 0, 0.999700, 0.024493, 0, 0.999700, 0.024493, 0, 0.999700, 0.024493, 0, 0.999700, 0.024493, 0, 0.999795, 0.020241, 0, 0.999795, 0.020241, 0, 0.999795, 0.020241, 0, 0.999795, 0.020241, 0, 0.999795, 0.020241, 0, 0.999795, 0.020241, 0, 0.999862, 0.016639, 0, 0.999862, 0.016639, 0, 0.999862, 0.016639, 0, 0.999862, 0.016639, 0, 0.999862, 0.016639, 0, 0.999862, 0.016639, 0, 0.999906, 0.013692, 0, 0.999906, 0.013692, 0, 0.999906, 0.013692, 0, 0.999906, 0.013692, 0, 0.999906, 0.013692, 0, 0.999906, 0.013692, 0, 0.999935, 0.011398, 0, 0.999935, 0.011398, 0, 0.999935, 0.011398, 0, 0.999935, 0.011398, 0, 0.999935, 0.011398, 0, 0.999935, 0.011398, 0, 0.999952, 0.009754, 0, 0.999952, 0.009754, 0, 0.999952, 0.009754, 0, 0.999952, 0.009754, 0, 0.999952, 0.009754, 0, 0.999952, 0.009754, 0, 0.999962, 0.008758, 0, 0.999962, 0.008758, 0, 0.999962, 0.008758, 0, 0.999962, 0.008758, 0, 0.999962, 0.008758, 0, 0.999962, 0.008758, 0, 0.999965, 0.008404, 0, 0.999965, 0.008404, 0, 0.999965, 0.008404, 0, 0.999965, 0.008404, 0, 0.999965, 0.008404, 0, 0.999965, 0.008404, 0, 0.999962, 0.008748, 0, 0.999962, 0.008748, 0, 0.999962, 0.008748, 0, 0.999962, 0.008748, 0, 0.999962, 0.008748, 0, 0.999962, 0.008748, 0, 0.999951, 0.009857, 0, 0.999951, 0.009857, 0, 0.999951, 0.009857, 0, 0.999951, 0.009857, 0, 0.999951, 0.009857, 0, 0.999951, 0.009857, 0, 0.999931, 0.011721, 0, 0.999931, 0.011721, 0, 0.999931, 0.011721, 0, 0.999931, 0.011721, 0, 0.999931, 0.011721, 0, 0.999931, 0.011721, 0, 0.999897, 0.014337, 0, 0.999897, 0.014337, 0, 0.999897, 0.014337, 0, 0.999897, 0.014337, 0, 0.999897, 0.014337, 0, 0.999897, 0.014337, 0, 0.999845, 0.017597, 0, 0.999845, 0.017597, 0, 0.999845, 0.017597, 0, 0.999845, 0.017597, 0, 0.999845, 0.017597, 0, 0.999845, 0.017597, 0, 0.999771, 0.021401, 0, 0.999771, 0.021401, 0, 0.999771, 0.021401, 0, 0.999771, 0.021401, 0, 0.999771, 0.021401, 0, 0.999771, 0.021401, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999530, 0.030657, 0, 0.999530, 0.030657, 0, 0.999530, 0.030657, 0, 0.999530, 0.030657, 0, 0.999530, 0.030657, 0, 0.999530, 0.030657, 0, 0.997417, 0.071830, 0, 0.997417, 0.071830, 0, 0.997417, 0.071830, 0, 0.997417, 0.071830, 0, 0.997417, 0.071830, 0, 0.997417, 0.071830, 0, 0.997477, 0.070995, 0, 0.997477, 0.070995, 0, 0.997477, 0.070995, 0, 0.997477, 0.070995, 0, 0.997477, 0.070995, 0, 0.997477, 0.070995, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999947, 0.010288, 0, 0.999947, 0.010288, 0, 0.999947, 0.010288, 0, 0.999947, 0.010288, 0, 0.999947, 0.010288, 0, 0.999947, 0.010288, 0, 0.999973, 0.007372, 0, 0.999973, 0.007372, 0, 0.999973, 0.007372, 0, 0.999973, 0.007372, 0, 0.999973, 0.007372, 0, 0.999973, 0.007372, 0, 0.999988, 0.004849, 0, 0.999988, 0.004849, 0, 0.999988, 0.004849, 0, 0.999988, 0.004849, 0, 0.999988, 0.004849, 0, 0.999988, 0.004849, 0, 0.999996, 0.002721, 0, 0.999996, 0.002721, 0, 0.999996, 0.002721, 0, 0.999996, 0.002721, 0, 0.999996, 0.002721, 0, 0.999996, 0.002721, 0, 1.000000, 0.000990, 0, 1.000000, 0.000990, 0, 1.000000, 0.000990, 0, 1.000000, 0.000990, 0, 1.000000, 0.000990, 0, 1.000000, 0.000990, 0, 1.000000, -0.000353, 0, 1.000000, -0.000353, 0, 1.000000, -0.000353, 0, 1.000000, -0.000353, 0, 1.000000, -0.000353, 0, 1.000000, -0.000353, 0, 0.999999, -0.001319, 0, 0.999999, -0.001319, 0, 0.999999, -0.001319, 0, 0.999999, -0.001319, 0, 0.999999, -0.001319, 0, 0.999999, -0.001319, 0, 0.999998, -0.001905, 0, 0.999998, -0.001905, 0, 0.999998, -0.001905, 0, 0.999998, -0.001905, 0, 0.999998, -0.001905, 0, 0.999998, -0.001905, 0, 0.999998, -0.002107, 0, 0.999998, -0.002107, 0, 0.999998, -0.002107, 0, 0.999998, -0.002107, 0, 0.999998, -0.002107, 0, 0.999998, -0.002107, 0, 0.999999, -0.001629, 0, 0.999999, -0.001629, 0, 0.999999, -0.001629, 0, 0.999999, -0.001629, 0, 0.999999, -0.001629, 0, 0.999999, -0.001629, 0, 1, -0.000158, 0, 1, -0.000158, 0, 1, -0.000158, 0, 1, -0.000158, 0, 1, -0.000158, 0, 1, -0.000158, 0, 0.999997, 0.002318, 0, 0.999997, 0.002318, 0, 0.999997, 0.002318, 0, 0.999997, 0.002318, 0, 0.999997, 0.002318, 0, 0.999997, 0.002318, 0, 0.999983, 0.005806, 0, 0.999983, 0.005806, 0, 0.999983, 0.005806, 0, 0.999983, 0.005806, 0, 0.999983, 0.005806, 0, 0.999983, 0.005806, 0, 0.999947, 0.010274, 0, 0.999947, 0.010274, 0, 0.999947, 0.010274, 0, 0.999947, 0.010274, 0, 0.999947, 0.010274, 0, 0.999947, 0.010274, 0, 0.999877, 0.015683, 0, 0.999877, 0.015683, 0, 0.999877, 0.015683, 0, 0.999877, 0.015683, 0, 0.999877, 0.015683, 0, 0.999877, 0.015683, 0, 0.999757, 0.022024, 0, 0.999757, 0.022024, 0, 0.999757, 0.022024, 0, 0.999757, 0.022024, 0, 0.999757, 0.022024, 0, 0.999757, 0.022024, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997651, 0.068504, 0, 0.997651, 0.068504, 0, 0.997651, 0.068504, 0, 0.997651, 0.068504, 0, 0.997651, 0.068504, 0, 0.997651, 0.068504, 0, 0.997756, 0.066955, 0, 0.997756, 0.066955, 0, 0.997756, 0.066955, 0, 0.997756, 0.066955, 0, 0.997756, 0.066955, 0, 0.997756, 0.066955, 0, 0.998350, 0.057425, 0, 0.998350, 0.057425, 0, 0.998350, 0.057425, 0, 0.998350, 0.057425, 0, 0.998350, 0.057425, 0, 0.998350, 0.057425, 0, 0.998272, 0.058754, 0, 0.998272, 0.058754, 0, 0.998272, 0.058754, 0, 0.998272, 0.058754, 0, 0.998272, 0.058754, 0, 0.998272, 0.058754, 0, 0.999744, 0.022617, 0, 0.999744, 0.022617, 0, 0.999744, 0.022617, 0, 0.999744, 0.022617, 0, 0.999744, 0.022617, 0, 0.999744, 0.022617, 0, 0.999821, 0.018904, 0, 0.999821, 0.018904, 0, 0.999821, 0.018904, 0, 0.999821, 0.018904, 0, 0.999821, 0.018904, 0, 0.999821, 0.018904, 0, 0.999873, 0.015914, 0, 0.999873, 0.015914, 0, 0.999873, 0.015914, 0, 0.999873, 0.015914, 0, 0.999873, 0.015914, 0, 0.999873, 0.015914, 0, 0.999907, 0.013649, 0, 0.999907, 0.013649, 0, 0.999907, 0.013649, 0, 0.999907, 0.013649, 0, 0.999907, 0.013649, 0, 0.999907, 0.013649, 0, 0.999927, 0.012110, 0, 0.999927, 0.012110, 0, 0.999927, 0.012110, 0, 0.999927, 0.012110, 0, 0.999927, 0.012110, 0, 0.999927, 0.012110, 0, 0.999939, 0.011078, 0, 0.999939, 0.011078, 0, 0.999939, 0.011078, 0, 0.999939, 0.011078, 0, 0.999939, 0.011078, 0, 0.999939, 0.011078, 0, 0.999947, 0.010331, 0, 0.999947, 0.010331, 0, 0.999947, 0.010331, 0, 0.999947, 0.010331, 0, 0.999947, 0.010331, 0, 0.999947, 0.010331, 0, 0.999951, 0.009871, 0, 0.999951, 0.009871, 0, 0.999951, 0.009871, 0, 0.999951, 0.009871, 0, 0.999951, 0.009871, 0, 0.999951, 0.009871, 0, 0.999953, 0.009696, 0, 0.999953, 0.009696, 0, 0.999953, 0.009696, 0, 0.999953, 0.009696, 0, 0.999953, 0.009696, 0, 0.999953, 0.009696, 0, 0.999954, 0.009586, 0, 0.999954, 0.009586, 0, 0.999954, 0.009586, 0, 0.999954, 0.009586, 0, 0.999954, 0.009586, 0, 0.999954, 0.009586, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1.000000, 0.000303, 0, 1.000000, 0.000303, 0, 1.000000, 0.000303, 0, 1.000000, 0.000303, 0, 1.000000, 0.000303, 0, 1.000000, 0.000303, 0, 0.999999, 0.001218, 0, 0.999999, 0.001218, 0, 0.999999, 0.001218, 0, 0.999999, 0.001218, 0, 0.999999, 0.001218, 0, 0.999999, 0.001218, 0, 0.999996, 0.002751, 0, 0.999996, 0.002751, 0, 0.999996, 0.002751, 0, 0.999996, 0.002751, 0, 0.999996, 0.002751, 0, 0.999996, 0.002751, 0, 0.999988, 0.004905, 0, 0.999988, 0.004905, 0, 0.999988, 0.004905, 0, 0.999988, 0.004905, 0, 0.999988, 0.004905, 0, 0.999988, 0.004905, 0, 0.999969, 0.007878, 0, 0.999969, 0.007878, 0, 0.999969, 0.007878, 0, 0.999969, 0.007878, 0, 0.999969, 0.007878, 0, 0.999969, 0.007878, 0, 0.999930, 0.011867, 0, 0.999930, 0.011867, 0, 0.999930, 0.011867, 0, 0.999930, 0.011867, 0, 0.999930, 0.011867, 0, 0.999930, 0.011867, 0, 0.999858, 0.016869, 0, 0.999858, 0.016869, 0, 0.999858, 0.016869, 0, 0.999858, 0.016869, 0, 0.999858, 0.016869, 0, 0.999858, 0.016869, 0, 0.999738, 0.022876, 0, 0.999738, 0.022876, 0, 0.999738, 0.022876, 0, 0.999738, 0.022876, 0, 0.999738, 0.022876, 0, 0.999738, 0.022876, 0, 0.998233, 0.059426, 0, 0.998233, 0.059426, 0, 0.998233, 0.059426, 0, 0.998233, 0.059426, 0, 0.998233, 0.059426, 0, 0.998233, 0.059426, 0, 0.998203, 0.059926, 0, 0.998203, 0.059926, 0, 0.998203, 0.059926, 0, 0.998203, 0.059926, 0, 0.998203, 0.059926, 0, 0.998203, 0.059926, 0, 0.997556, 0.069868, 0, 0.997556, 0.069868, 0, 0.997556, 0.069868, 0, 0.997556, 0.069868, 0, 0.997556, 0.069868, 0, 0.997556, 0.069868, 0, 0.997597, 0.069286, 0, 0.997597, 0.069286, 0, 0.997597, 0.069286, 0, 0.997597, 0.069286, 0, 0.997597, 0.069286, 0, 0.997597, 0.069286, 0, 0.996037, 0.088943, 0, 0.996037, 0.088943, 0, 0.996037, 0.088943, 0, 0.996037, 0.088943, 0, 0.996037, 0.088943, 0, 0.996037, 0.088943, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997865, 0.065308, 0, 0.997865, 0.065308, 0, 0.997865, 0.065308, 0, 0.997865, 0.065308, 0, 0.997865, 0.065308, 0, 0.997865, 0.065308, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.995829, 0.091244, 0, 0.995829, 0.091244, 0, 0.995829, 0.091244, 0, 0.995829, 0.091244, 0, 0.995829, 0.091244, 0, 0.995829, 0.091244, 0, 0.996930, 0.078296, 0, 0.996930, 0.078296, 0, 0.996930, 0.078296, 0, 0.996930, 0.078296, 0, 0.996930, 0.078296, 0, 0.996930, 0.078296, 0, 0.996863, 0.079141, 0, 0.996863, 0.079141, 0, 0.996863, 0.079141, 0, 0.996863, 0.079141, 0, 0.996863, 0.079141, 0, 0.996863, 0.079141, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.996094, 0.088296, 0, 0.996094, 0.088296, 0, 0.996094, 0.088296, 0, 0.996094, 0.088296, 0, 0.996094, 0.088296, 0, 0.996094, 0.088296, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999639, 0.026882, 0, 0.999639, 0.026882, 0, 0.999639, 0.026882, 0, 0.999639, 0.026882, 0, 0.999639, 0.026882, 0, 0.999639, 0.026882, 0, 0.999604, 0.028132, 0, 0.999604, 0.028132, 0, 0.999604, 0.028132, 0, 0.999604, 0.028132, 0, 0.999604, 0.028132, 0, 0.999604, 0.028132, 0, 0.999729, 0.023275, 0, 0.999729, 0.023275, 0, 0.999729, 0.023275, 0, 0.999729, 0.023275, 0, 0.999729, 0.023275, 0, 0.999729, 0.023275, 0, 0.999819, 0.019023, 0, 0.999819, 0.019023, 0, 0.999819, 0.019023, 0, 0.999819, 0.019023, 0, 0.999819, 0.019023, 0, 0.999819, 0.019023, 0, 0.999882, 0.015383, 0, 0.999882, 0.015383, 0, 0.999882, 0.015383, 0, 0.999882, 0.015383, 0, 0.999882, 0.015383, 0, 0.999882, 0.015383, 0, 0.999924, 0.012357, 0, 0.999924, 0.012357, 0, 0.999924, 0.012357, 0, 0.999924, 0.012357, 0, 0.999924, 0.012357, 0, 0.999924, 0.012357, 0, 0.999950, 0.009979, 0, 0.999950, 0.009979, 0, 0.999950, 0.009979, 0, 0.999950, 0.009979, 0, 0.999950, 0.009979, 0, 0.999950, 0.009979, 0, 0.999966, 0.008278, 0, 0.999966, 0.008278, 0, 0.999966, 0.008278, 0, 0.999966, 0.008278, 0, 0.999966, 0.008278, 0, 0.999966, 0.008278, 0, 0.999974, 0.007248, 0, 0.999974, 0.007248, 0, 0.999974, 0.007248, 0, 0.999974, 0.007248, 0, 0.999974, 0.007248, 0, 0.999974, 0.007248, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006945, 0, 0.999976, 0.006945, 0, 0.999976, 0.006945, 0, 0.999976, 0.006945, 0, 0.999976, 0.006945, 0, 0.999976, 0.006945, 0, 0.999974, 0.007167, 0, 0.999974, 0.007167, 0, 0.999974, 0.007167, 0, 0.999974, 0.007167, 0, 0.999974, 0.007167, 0, 0.999974, 0.007167, 0, 0.999972, 0.007547, 0, 0.999972, 0.007547, 0, 0.999972, 0.007547, 0, 0.999972, 0.007547, 0, 0.999972, 0.007547, 0, 0.999972, 0.007547, 0, 0.999967, 0.008085, 0, 0.999967, 0.008085, 0, 0.999967, 0.008085, 0, 0.999967, 0.008085, 0, 0.999967, 0.008085, 0, 0.999967, 0.008085, 0, 0.999961, 0.008780, 0, 0.999961, 0.008780, 0, 0.999961, 0.008780, 0, 0.999961, 0.008780, 0, 0.999961, 0.008780, 0, 0.999961, 0.008780, 0, 0.999954, 0.009630, 0, 0.999954, 0.009630, 0, 0.999954, 0.009630, 0, 0.999954, 0.009630, 0, 0.999954, 0.009630, 0, 0.999954, 0.009630, 0, 0.999943, 0.010634, 0, 0.999943, 0.010634, 0, 0.999943, 0.010634, 0, 0.999943, 0.010634, 0, 0.999943, 0.010634, 0, 0.999943, 0.010634, 0, 0.999931, 0.011791, 0, 0.999931, 0.011791, 0, 0.999931, 0.011791, 0, 0.999931, 0.011791, 0, 0.999931, 0.011791, 0, 0.999931, 0.011791, 0, 0.999994, 0.003369, 0, 0.999994, 0.003369, 0, 0.999994, 0.003369, 0, 0.999994, 0.003369, 0, 0.999994, 0.003369, 0, 0.999994, 0.003369, 0, 0.999998, 0.002212, 0, 0.999998, 0.002212, 0, 0.999998, 0.002212, 0, 0.999998, 0.002212, 0, 0.999998, 0.002212, 0, 0.999998, 0.002212, 0, 0.999999, 0.001197, 0, 0.999999, 0.001197, 0, 0.999999, 0.001197, 0, 0.999999, 0.001197, 0, 0.999999, 0.001197, 0, 0.999999, 0.001197, 0, 1, 0.000327, 0, 1, 0.000327, 0, 1, 0.000327, 0, 1, 0.000327, 0, 1, 0.000327, 0, 1, 0.000327, 0, 1.000000, -0.000399, 0, 1.000000, -0.000399, 0, 1.000000, -0.000399, 0, 1.000000, -0.000399, 0, 1.000000, -0.000399, 0, 1.000000, -0.000399, 0, 1.000000, -0.000972, 0, 1.000000, -0.000972, 0, 1.000000, -0.000972, 0, 1.000000, -0.000972, 0, 1.000000, -0.000972, 0, 1.000000, -0.000972, 0, 0.999999, -0.001385, 0, 0.999999, -0.001385, 0, 0.999999, -0.001385, 0, 0.999999, -0.001385, 0, 0.999999, -0.001385, 0, 0.999999, -0.001385, 0, 0.999999, -0.001635, 0, 0.999999, -0.001635, 0, 0.999999, -0.001635, 0, 0.999999, -0.001635, 0, 0.999999, -0.001635, 0, 0.999999, -0.001635, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001722, 0, 0.999999, -0.001202, 0, 0.999999, -0.001202, 0, 0.999999, -0.001202, 0, 0.999999, -0.001202, 0, 0.999999, -0.001202, 0, 0.999999, -0.001202, 0, 1.000000, 0.000395, 0, 1.000000, 0.000395, 0, 1.000000, 0.000395, 0, 1.000000, 0.000395, 0, 1.000000, 0.000395, 0, 1.000000, 0.000395, 0, 0.999995, 0.003087, 0, 0.999995, 0.003087, 0, 0.999995, 0.003087, 0, 0.999995, 0.003087, 0, 0.999995, 0.003087, 0, 0.999995, 0.003087, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999976, 0.006884, 0, 0.999932, 0.011633, 0, 0.999932, 0.011633, 0, 0.999932, 0.011633, 0, 0.999932, 0.011633, 0, 0.999932, 0.011633, 0, 0.999932, 0.011633, 0, 0.999853, 0.017168, 0, 0.999853, 0.017168, 0, 0.999853, 0.017168, 0, 0.999853, 0.017168, 0, 0.999853, 0.017168, 0, 0.999853, 0.017168, 0, 0.999724, 0.023476, 0, 0.999724, 0.023476, 0, 0.999724, 0.023476, 0, 0.999724, 0.023476, 0, 0.999724, 0.023476, 0, 0.999724, 0.023476, 0, 0.999534, 0.030540, 0, 0.999534, 0.030540, 0, 0.999534, 0.030540, 0, 0.999534, 0.030540, 0, 0.999534, 0.030540, 0, 0.999534, 0.030540, 0, 0.999285, 0.037804, 0, 0.999285, 0.037804, 0, 0.999285, 0.037804, 0, 0.999285, 0.037804, 0, 0.999285, 0.037804, 0, 0.999285, 0.037804, 0, 0.999022, 0.044217, 0, 0.999022, 0.044217, 0, 0.999022, 0.044217, 0, 0.999022, 0.044217, 0, 0.999022, 0.044217, 0, 0.999022, 0.044217, 0, 0.998761, 0.049765, 0, 0.998761, 0.049765, 0, 0.998761, 0.049765, 0, 0.998761, 0.049765, 0, 0.998761, 0.049765, 0, 0.998761, 0.049765, 0, 0.998517, 0.054439, 0, 0.998517, 0.054439, 0, 0.998517, 0.054439, 0, 0.998517, 0.054439, 0, 0.998517, 0.054439, 0, 0.998517, 0.054439, 0, 0.998312, 0.058080, 0, 0.998312, 0.058080, 0, 0.998312, 0.058080, 0, 0.998312, 0.058080, 0, 0.998312, 0.058080, 0, 0.998312, 0.058080, 0, 0.998166, 0.060540, 0, 0.998166, 0.060540, 0, 0.998166, 0.060540, 0, 0.998166, 0.060540, 0, 0.998166, 0.060540, 0, 0.998166, 0.060540, 0, 0.998086, 0.061836, 0, 0.998086, 0.061836, 0, 0.998086, 0.061836, 0, 0.998086, 0.061836, 0, 0.998086, 0.061836, 0, 0.998086, 0.061836, 0, 0.998077, 0.061983, 0, 0.998077, 0.061983, 0, 0.998077, 0.061983, 0, 0.998077, 0.061983, 0, 0.998077, 0.061983, 0, 0.998077, 0.061983, 0, 0.998129, 0.061145, 0, 0.998129, 0.061145, 0, 0.998129, 0.061145, 0, 0.998129, 0.061145, 0, 0.998129, 0.061145, 0, 0.998129, 0.061145, 0, 0.998235, 0.059391, 0, 0.998235, 0.059391, 0, 0.998235, 0.059391, 0, 0.998235, 0.059391, 0, 0.998235, 0.059391, 0, 0.998235, 0.059391, 0, 0.998389, 0.056731, 0, 0.998389, 0.056731, 0, 0.998389, 0.056731, 0, 0.998389, 0.056731, 0, 0.998389, 0.056731, 0, 0.998389, 0.056731, 0, 0.998585, 0.053171, 0, 0.998585, 0.053171, 0, 0.998585, 0.053171, 0, 0.998585, 0.053171, 0, 0.998585, 0.053171, 0, 0.998585, 0.053171, 0, 0.998806, 0.048853, 0, 0.998806, 0.048853, 0, 0.998806, 0.048853, 0, 0.998806, 0.048853, 0, 0.998806, 0.048853, 0, 0.998806, 0.048853, 0, 0.999035, 0.043914, 0, 0.999035, 0.043914, 0, 0.999035, 0.043914, 0, 0.999035, 0.043914, 0, 0.999035, 0.043914, 0, 0.999035, 0.043914, 0, 0.999265, 0.038343, 0, 0.999265, 0.038343, 0, 0.999265, 0.038343, 0, 0.999265, 0.038343, 0, 0.999265, 0.038343, 0, 0.999265, 0.038343, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999228, 0.039299, 0, 0.999228, 0.039299, 0, 0.999228, 0.039299, 0, 0.999228, 0.039299, 0, 0.999228, 0.039299, 0, 0.999228, 0.039299, 0, 0.999092, 0.042613, 0, 0.999092, 0.042613, 0, 0.999092, 0.042613, 0, 0.999092, 0.042613, 0, 0.999092, 0.042613, 0, 0.999092, 0.042613, 0, 0.998961, 0.045565, 0, 0.998961, 0.045565, 0, 0.998961, 0.045565, 0, 0.998961, 0.045565, 0, 0.998961, 0.045565, 0, 0.998961, 0.045565, 0, 0.998840, 0.048158, 0, 0.998840, 0.048158, 0, 0.998840, 0.048158, 0, 0.998840, 0.048158, 0, 0.998840, 0.048158, 0, 0.998840, 0.048158, 0, 0.998734, 0.050298, 0, 0.998734, 0.050298, 0, 0.998734, 0.050298, 0, 0.998734, 0.050298, 0, 0.998734, 0.050298, 0, 0.998734, 0.050298, 0, 0.998653, 0.051886, 0, 0.998653, 0.051886, 0, 0.998653, 0.051886, 0, 0.998653, 0.051886, 0, 0.998653, 0.051886, 0, 0.998653, 0.051886, 0, 0.998599, 0.052919, 0, 0.998599, 0.052919, 0, 0.998599, 0.052919, 0, 0.998599, 0.052919, 0, 0.998599, 0.052919, 0, 0.998599, 0.052919, 0, 0.998573, 0.053395, 0, 0.998573, 0.053395, 0, 0.998573, 0.053395, 0, 0.998573, 0.053395, 0, 0.998573, 0.053395, 0, 0.998573, 0.053395, 0, 0.998577, 0.053323, 0, 0.998577, 0.053323, 0, 0.998577, 0.053323, 0, 0.998577, 0.053323, 0, 0.998577, 0.053323, 0, 0.998577, 0.053323, 0, 0.998613, 0.052660, 0, 0.998613, 0.052660, 0, 0.998613, 0.052660, 0, 0.998613, 0.052660, 0, 0.998613, 0.052660, 0, 0.998613, 0.052660, 0, 0.998678, 0.051401, 0, 0.998678, 0.051401, 0, 0.998678, 0.051401, 0, 0.998678, 0.051401, 0, 0.998678, 0.051401, 0, 0.998678, 0.051401, 0, 0.998772, 0.049542, 0, 0.998772, 0.049542, 0, 0.998772, 0.049542, 0, 0.998772, 0.049542, 0, 0.998772, 0.049542, 0, 0.998772, 0.049542, 0, 0.998889, 0.047127, 0, 0.998889, 0.047127, 0, 0.998889, 0.047127, 0, 0.998889, 0.047127, 0, 0.998889, 0.047127, 0, 0.998889, 0.047127, 0, 0.999023, 0.044204, 0, 0.999023, 0.044204, 0, 0.999023, 0.044204, 0, 0.999023, 0.044204, 0, 0.999023, 0.044204, 0, 0.999023, 0.044204, 0, 0.999168, 0.040776, 0, 0.999168, 0.040776, 0, 0.999168, 0.040776, 0, 0.999168, 0.040776, 0, 0.999168, 0.040776, 0, 0.999168, 0.040776, 0, 0.999321, 0.036845, 0, 0.999321, 0.036845, 0, 0.999321, 0.036845, 0, 0.999321, 0.036845, 0, 0.999321, 0.036845, 0, 0.999321, 0.036845, 0, 0.999914, 0.013083, 0, 0.999914, 0.013083, 0, 0.999914, 0.013083, 0, 0.999914, 0.013083, 0, 0.999914, 0.013083, 0, 0.999914, 0.013083, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997353, 0.072714, 0, 0.997353, 0.072714, 0, 0.997353, 0.072714, 0, 0.997353, 0.072714, 0, 0.997353, 0.072714, 0, 0.997353, 0.072714, 0, 0.999116, 0.042043, 0, 0.999116, 0.042043, 0, 0.999116, 0.042043, 0, 0.999116, 0.042043, 0, 0.999116, 0.042043, 0, 0.999116, 0.042043, 0, 0.997552, 0.069932, 0, 0.997552, 0.069932, 0, 0.997552, 0.069932, 0, 0.997552, 0.069932, 0, 0.997552, 0.069932, 0, 0.997552, 0.069932, 0, 0.997600, 0.069235, 0, 0.997600, 0.069235, 0, 0.997600, 0.069235, 0, 0.997600, 0.069235, 0, 0.997600, 0.069235, 0, 0.997600, 0.069235, 0, 0.999427, 0.033845, 0, 0.999427, 0.033845, 0, 0.999427, 0.033845, 0, 0.999427, 0.033845, 0, 0.999427, 0.033845, 0, 0.999427, 0.033845, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999416, 0.034173, 0, 0.999416, 0.034173, 0, 0.999416, 0.034173, 0, 0.999416, 0.034173, 0, 0.999416, 0.034173, 0, 0.999416, 0.034173, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998996, 0.044789, 0, 0.998996, 0.044789, 0, 0.998996, 0.044789, 0, 0.998996, 0.044789, 0, 0.998996, 0.044789, 0, 0.998996, 0.044789, 0, 0.999035, 0.043931, 0, 0.999035, 0.043931, 0, 0.999035, 0.043931, 0, 0.999035, 0.043931, 0, 0.999035, 0.043931, 0, 0.999035, 0.043931, 0, 0.999101, 0.042393, 0, 0.999101, 0.042393, 0, 0.999101, 0.042393, 0, 0.999101, 0.042393, 0, 0.999101, 0.042393, 0, 0.999101, 0.042393, 0, 0.999193, 0.040172, 0, 0.999193, 0.040172, 0, 0.999193, 0.040172, 0, 0.999193, 0.040172, 0, 0.999193, 0.040172, 0, 0.999193, 0.040172, 0, 0.999302, 0.037349, 0, 0.999302, 0.037349, 0, 0.999302, 0.037349, 0, 0.999302, 0.037349, 0, 0.999302, 0.037349, 0, 0.999302, 0.037349, 0, 0.999422, 0.034007, 0, 0.999422, 0.034007, 0, 0.999422, 0.034007, 0, 0.999422, 0.034007, 0, 0.999422, 0.034007, 0, 0.999422, 0.034007, 0, 0.999545, 0.030150, 0, 0.999545, 0.030150, 0, 0.999545, 0.030150, 0, 0.999545, 0.030150, 0, 0.999545, 0.030150, 0, 0.999545, 0.030150, 0, 0.999668, 0.025780, 0, 0.999668, 0.025780, 0, 0.999668, 0.025780, 0, 0.999668, 0.025780, 0, 0.999668, 0.025780, 0, 0.999668, 0.025780, 0, 0.999771, 0.021396, 0, 0.999771, 0.021396, 0, 0.999771, 0.021396, 0, 0.999771, 0.021396, 0, 0.999771, 0.021396, 0, 0.999771, 0.021396, 0, 0.999847, 0.017497, 0, 0.999847, 0.017497, 0, 0.999847, 0.017497, 0, 0.999847, 0.017497, 0, 0.999847, 0.017497, 0, 0.999847, 0.017497, 0, 0.999901, 0.014086, 0, 0.999901, 0.014086, 0, 0.999901, 0.014086, 0, 0.999901, 0.014086, 0, 0.999901, 0.014086, 0, 0.999901, 0.014086, 0, 0.999938, 0.011169, 0, 0.999938, 0.011169, 0, 0.999938, 0.011169, 0, 0.999938, 0.011169, 0, 0.999938, 0.011169, 0, 0.999938, 0.011169, 0, 0.999961, 0.008832, 0, 0.999961, 0.008832, 0, 0.999961, 0.008832, 0, 0.999961, 0.008832, 0, 0.999961, 0.008832, 0, 0.999961, 0.008832, 0, 0.999974, 0.007162, 0, 0.999974, 0.007162, 0, 0.999974, 0.007162, 0, 0.999974, 0.007162, 0, 0.999974, 0.007162, 0, 0.999974, 0.007162, 0, 0.999981, 0.006156, 0, 0.999981, 0.006156, 0, 0.999981, 0.006156, 0, 0.999981, 0.006156, 0, 0.999981, 0.006156, 0, 0.999981, 0.006156, 0, 0.999983, 0.005809, 0, 0.999983, 0.005809, 0, 0.999983, 0.005809, 0, 0.999983, 0.005809, 0, 0.999983, 0.005809, 0, 0.999983, 0.005809, 0, 0.999981, 0.006118, 0, 0.999981, 0.006118, 0, 0.999981, 0.006118, 0, 0.999981, 0.006118, 0, 0.999981, 0.006118, 0, 0.999981, 0.006118, 0, 0.999975, 0.007083, 0, 0.999975, 0.007083, 0, 0.999975, 0.007083, 0, 0.999975, 0.007083, 0, 0.999975, 0.007083, 0, 0.999975, 0.007083, 0, 0.999962, 0.008696, 0, 0.999962, 0.008696, 0, 0.999962, 0.008696, 0, 0.999962, 0.008696, 0, 0.999962, 0.008696, 0, 0.999962, 0.008696, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999905, 0.013773, 0, 0.999905, 0.013773, 0, 0.999905, 0.013773, 0, 0.999905, 0.013773, 0, 0.999905, 0.013773, 0, 0.999905, 0.013773, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999783, 0.020813, 0, 0.999783, 0.020813, 0, 0.999783, 0.020813, 0, 0.999783, 0.020813, 0, 0.999783, 0.020813, 0, 0.999783, 0.020813, 0, 0.999686, 0.025044, 0, 0.999686, 0.025044, 0, 0.999686, 0.025044, 0, 0.999686, 0.025044, 0, 0.999686, 0.025044, 0, 0.999686, 0.025044, 0, 0.999572, 0.029266, 0, 0.999572, 0.029266, 0, 0.999572, 0.029266, 0, 0.999572, 0.029266, 0, 0.999572, 0.029266, 0, 0.999572, 0.029266, 0, 0.999454, 0.033045, 0, 0.999454, 0.033045, 0, 0.999454, 0.033045, 0, 0.999454, 0.033045, 0, 0.999454, 0.033045, 0, 0.999454, 0.033045, 0, 0.999338, 0.036383, 0, 0.999338, 0.036383, 0, 0.999338, 0.036383, 0, 0.999338, 0.036383, 0, 0.999338, 0.036383, 0, 0.999338, 0.036383, 0, 0.999228, 0.039286, 0, 0.999228, 0.039286, 0, 0.999228, 0.039286, 0, 0.999228, 0.039286, 0, 0.999228, 0.039286, 0, 0.999228, 0.039286, 0, 0.999132, 0.041657, 0, 0.999132, 0.041657, 0, 0.999132, 0.041657, 0, 0.999132, 0.041657, 0, 0.999132, 0.041657, 0, 0.999132, 0.041657, 0, 0.999058, 0.043397, 0, 0.999058, 0.043397, 0, 0.999058, 0.043397, 0, 0.999058, 0.043397, 0, 0.999058, 0.043397, 0, 0.999058, 0.043397, 0, 0.999009, 0.044504, 0, 0.999009, 0.044504, 0, 0.999009, 0.044504, 0, 0.999009, 0.044504, 0, 0.999009, 0.044504, 0, 0.999009, 0.044504, 0, 0.998673, 0.051501, 0, 0.998673, 0.051501, 0, 0.998673, 0.051501, 0, 0.998673, 0.051501, 0, 0.998673, 0.051501, 0, 0.998673, 0.051501, 0, 0.998752, 0.049950, 0, 0.998752, 0.049950, 0, 0.998752, 0.049950, 0, 0.998752, 0.049950, 0, 0.998752, 0.049950, 0, 0.998752, 0.049950, 0, 0.998868, 0.047566, 0, 0.998868, 0.047566, 0, 0.998868, 0.047566, 0, 0.998868, 0.047566, 0, 0.998868, 0.047566, 0, 0.998868, 0.047566, 0, 0.999016, 0.044352, 0, 0.999016, 0.044352, 0, 0.999016, 0.044352, 0, 0.999016, 0.044352, 0, 0.999016, 0.044352, 0, 0.999016, 0.044352, 0, 0.999183, 0.040415, 0, 0.999183, 0.040415, 0, 0.999183, 0.040415, 0, 0.999183, 0.040415, 0, 0.999183, 0.040415, 0, 0.999183, 0.040415, 0, 0.999357, 0.035853, 0, 0.999357, 0.035853, 0, 0.999357, 0.035853, 0, 0.999357, 0.035853, 0, 0.999357, 0.035853, 0, 0.999357, 0.035853, 0, 0.999530, 0.030661, 0, 0.999530, 0.030661, 0, 0.999530, 0.030661, 0, 0.999530, 0.030661, 0, 0.999530, 0.030661, 0, 0.999530, 0.030661, 0, 0.999692, 0.024827, 0, 0.999692, 0.024827, 0, 0.999692, 0.024827, 0, 0.999692, 0.024827, 0, 0.999692, 0.024827, 0, 0.999692, 0.024827, 0, 0.999819, 0.019028, 0, 0.999819, 0.019028, 0, 0.999819, 0.019028, 0, 0.999819, 0.019028, 0, 0.999819, 0.019028, 0, 0.999819, 0.019028, 0, 0.999904, 0.013895, 0, 0.999904, 0.013895, 0, 0.999904, 0.013895, 0, 0.999904, 0.013895, 0, 0.999904, 0.013895, 0, 0.999904, 0.013895, 0, 0.999956, 0.009421, 0, 0.999956, 0.009421, 0, 0.999956, 0.009421, 0, 0.999956, 0.009421, 0, 0.999956, 0.009421, 0, 0.999956, 0.009421, 0, 0.999984, 0.005600, 0, 0.999984, 0.005600, 0, 0.999984, 0.005600, 0, 0.999984, 0.005600, 0, 0.999984, 0.005600, 0, 0.999984, 0.005600, 0, 0.999997, 0.002534, 0, 0.999997, 0.002534, 0, 0.999997, 0.002534, 0, 0.999997, 0.002534, 0, 0.999997, 0.002534, 0, 0.999997, 0.002534, 0, 1.000000, 0.000331, 0, 1.000000, 0.000331, 0, 1.000000, 0.000331, 0, 1.000000, 0.000331, 0, 1.000000, 0.000331, 0, 1.000000, 0.000331, 0, 1.000000, -0.001000, 0, 1.000000, -0.001000, 0, 1.000000, -0.001000, 0, 1.000000, -0.001000, 0, 1.000000, -0.001000, 0, 1.000000, -0.001000, 0, 0.999999, -0.001452, 0, 0.999999, -0.001452, 0, 0.999999, -0.001452, 0, 0.999999, -0.001452, 0, 0.999999, -0.001452, 0, 0.999999, -0.001452, 0, 1.000000, -0.001011, 0, 1.000000, -0.001011, 0, 1.000000, -0.001011, 0, 1.000000, -0.001011, 0, 1.000000, -0.001011, 0, 1.000000, -0.001011, 0, 1.000000, 0.000338, 0, 1.000000, 0.000338, 0, 1.000000, 0.000338, 0, 1.000000, 0.000338, 0, 1.000000, 0.000338, 0, 1.000000, 0.000338, 0, 0.999997, 0.002604, 0, 0.999997, 0.002604, 0, 0.999997, 0.002604, 0, 0.999997, 0.002604, 0, 0.999997, 0.002604, 0, 0.999997, 0.002604, 0, 0.999983, 0.005795, 0, 0.999983, 0.005795, 0, 0.999983, 0.005795, 0, 0.999983, 0.005795, 0, 0.999983, 0.005795, 0, 0.999983, 0.005795, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999895, 0.014514, 0, 0.999895, 0.014514, 0, 0.999895, 0.014514, 0, 0.999895, 0.014514, 0, 0.999895, 0.014514, 0, 0.999895, 0.014514, 0, 0.999802, 0.019921, 0, 0.999802, 0.019921, 0, 0.999802, 0.019921, 0, 0.999802, 0.019921, 0, 0.999802, 0.019921, 0, 0.999802, 0.019921, 0, 0.999662, 0.026012, 0, 0.999662, 0.026012, 0, 0.999662, 0.026012, 0, 0.999662, 0.026012, 0, 0.999662, 0.026012, 0, 0.999662, 0.026012, 0, 0.999485, 0.032099, 0, 0.999485, 0.032099, 0, 0.999485, 0.032099, 0, 0.999485, 0.032099, 0, 0.999485, 0.032099, 0, 0.999485, 0.032099, 0, 0.999299, 0.037447, 0, 0.999299, 0.037447, 0, 0.999299, 0.037447, 0, 0.999299, 0.037447, 0, 0.999299, 0.037447, 0, 0.999299, 0.037447, 0, 0.999116, 0.042048, 0, 0.999116, 0.042048, 0, 0.999116, 0.042048, 0, 0.999116, 0.042048, 0, 0.999116, 0.042048, 0, 0.999116, 0.042048, 0, 0.998946, 0.045894, 0, 0.998946, 0.045894, 0, 0.998946, 0.045894, 0, 0.998946, 0.045894, 0, 0.998946, 0.045894, 0, 0.998946, 0.045894, 0, 0.998804, 0.048887, 0, 0.998804, 0.048887, 0, 0.998804, 0.048887, 0, 0.998804, 0.048887, 0, 0.998804, 0.048887, 0, 0.998804, 0.048887, 0, 0.998702, 0.050930, 0, 0.998702, 0.050930, 0, 0.998702, 0.050930, 0, 0.998702, 0.050930, 0, 0.998702, 0.050930, 0, 0.998702, 0.050930, 0, 0.998645, 0.052034, 0, 0.998645, 0.052034, 0, 0.998645, 0.052034, 0, 0.998645, 0.052034, 0, 0.998645, 0.052034, 0, 0.998645, 0.052034, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998799, 0.048999, 0, 0.998799, 0.048999, 0, 0.998799, 0.048999, 0, 0.998799, 0.048999, 0, 0.998799, 0.048999, 0, 0.998799, 0.048999, 0, 0.998818, 0.048609, 0, 0.998818, 0.048609, 0, 0.998818, 0.048609, 0, 0.998818, 0.048609, 0, 0.998818, 0.048609, 0, 0.998818, 0.048609, 0, 0.999817, 0.019137, 0, 0.999817, 0.019137, 0, 0.999817, 0.019137, 0, 0.999817, 0.019137, 0, 0.999817, 0.019137, 0, 0.999817, 0.019137, 0, 0.999873, 0.015949, 0, 0.999873, 0.015949, 0, 0.999873, 0.015949, 0, 0.999873, 0.015949, 0, 0.999873, 0.015949, 0, 0.999873, 0.015949, 0, 0.999913, 0.013183, 0, 0.999913, 0.013183, 0, 0.999913, 0.013183, 0, 0.999913, 0.013183, 0, 0.999913, 0.013183, 0, 0.999913, 0.013183, 0, 0.999941, 0.010842, 0, 0.999941, 0.010842, 0, 0.999941, 0.010842, 0, 0.999941, 0.010842, 0, 0.999941, 0.010842, 0, 0.999941, 0.010842, 0, 0.999960, 0.008927, 0, 0.999960, 0.008927, 0, 0.999960, 0.008927, 0, 0.999960, 0.008927, 0, 0.999960, 0.008927, 0, 0.999960, 0.008927, 0, 0.999972, 0.007437, 0, 0.999972, 0.007437, 0, 0.999972, 0.007437, 0, 0.999972, 0.007437, 0, 0.999972, 0.007437, 0, 0.999972, 0.007437, 0, 0.999980, 0.006370, 0, 0.999980, 0.006370, 0, 0.999980, 0.006370, 0, 0.999980, 0.006370, 0, 0.999980, 0.006370, 0, 0.999980, 0.006370, 0, 0.999984, 0.005726, 0, 0.999984, 0.005726, 0, 0.999984, 0.005726, 0, 0.999984, 0.005726, 0, 0.999984, 0.005726, 0, 0.999984, 0.005726, 0, 0.999985, 0.005502, 0, 0.999985, 0.005502, 0, 0.999985, 0.005502, 0, 0.999985, 0.005502, 0, 0.999985, 0.005502, 0, 0.999985, 0.005502, 0, 0.999984, 0.005737, 0, 0.999984, 0.005737, 0, 0.999984, 0.005737, 0, 0.999984, 0.005737, 0, 0.999984, 0.005737, 0, 0.999984, 0.005737, 0, 0.999979, 0.006473, 0, 0.999979, 0.006473, 0, 0.999979, 0.006473, 0, 0.999979, 0.006473, 0, 0.999979, 0.006473, 0, 0.999979, 0.006473, 0, 0.999970, 0.007707, 0, 0.999970, 0.007707, 0, 0.999970, 0.007707, 0, 0.999970, 0.007707, 0, 0.999970, 0.007707, 0, 0.999970, 0.007707, 0, 0.999956, 0.009437, 0, 0.999956, 0.009437, 0, 0.999956, 0.009437, 0, 0.999956, 0.009437, 0, 0.999956, 0.009437, 0, 0.999956, 0.009437, 0, 0.999933, 0.011593, 0, 0.999933, 0.011593, 0, 0.999933, 0.011593, 0, 0.999933, 0.011593, 0, 0.999933, 0.011593, 0, 0.999933, 0.011593, 0, 0.999901, 0.014107, 0, 0.999901, 0.014107, 0, 0.999901, 0.014107, 0, 0.999901, 0.014107, 0, 0.999901, 0.014107, 0, 0.999901, 0.014107, 0, 0.999856, 0.016983, 0, 0.999856, 0.016983, 0, 0.999856, 0.016983, 0, 0.999856, 0.016983, 0, 0.999856, 0.016983, 0, 0.999856, 0.016983, 0, 0.999796, 0.020221, 0, 0.999796, 0.020221, 0, 0.999796, 0.020221, 0, 0.999796, 0.020221, 0, 0.999796, 0.020221, 0, 0.999796, 0.020221, 0, 0.998874, 0.047436, 0, 0.998874, 0.047436, 0, 0.998874, 0.047436, 0, 0.998874, 0.047436, 0, 0.998874, 0.047436, 0, 0.998874, 0.047436, 0, 0.998892, 0.047070, 0, 0.998892, 0.047070, 0, 0.998892, 0.047070, 0, 0.998892, 0.047070, 0, 0.998892, 0.047070, 0, 0.998892, 0.047070, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999977, 0.006784, 0, 0.999977, 0.006784, 0, 0.999977, 0.006784, 0, 0.999977, 0.006784, 0, 0.999977, 0.006784, 0, 0.999977, 0.006784, 0, 0.999988, 0.004857, 0, 0.999988, 0.004857, 0, 0.999988, 0.004857, 0, 0.999988, 0.004857, 0, 0.999988, 0.004857, 0, 0.999988, 0.004857, 0, 0.999995, 0.003193, 0, 0.999995, 0.003193, 0, 0.999995, 0.003193, 0, 0.999995, 0.003193, 0, 0.999995, 0.003193, 0, 0.999995, 0.003193, 0, 0.999998, 0.001790, 0, 0.999998, 0.001790, 0, 0.999998, 0.001790, 0, 0.999998, 0.001790, 0, 0.999998, 0.001790, 0, 0.999998, 0.001790, 0, 1.000000, 0.000651, 0, 1.000000, 0.000651, 0, 1.000000, 0.000651, 0, 1.000000, 0.000651, 0, 1.000000, 0.000651, 0, 1.000000, 0.000651, 0, 1, -0.000232, 0, 1, -0.000232, 0, 1, -0.000232, 0, 1, -0.000232, 0, 1, -0.000232, 0, 1, -0.000232, 0, 1.000000, -0.000865, 0, 1.000000, -0.000865, 0, 1.000000, -0.000865, 0, 1.000000, -0.000865, 0, 1.000000, -0.000865, 0, 1.000000, -0.000865, 0, 0.999999, -0.001248, 0, 0.999999, -0.001248, 0, 0.999999, -0.001248, 0, 0.999999, -0.001248, 0, 0.999999, -0.001248, 0, 0.999999, -0.001248, 0, 0.999999, -0.001378, 0, 0.999999, -0.001378, 0, 0.999999, -0.001378, 0, 0.999999, -0.001378, 0, 0.999999, -0.001378, 0, 0.999999, -0.001378, 0, 0.999999, -0.001063, 0, 0.999999, -0.001063, 0, 0.999999, -0.001063, 0, 0.999999, -0.001063, 0, 0.999999, -0.001063, 0, 0.999999, -0.001063, 0, 1, -0.000103, 0, 1, -0.000103, 0, 1, -0.000103, 0, 1, -0.000103, 0, 1, -0.000103, 0, 1, -0.000103, 0, 0.999999, 0.001508, 0, 0.999999, 0.001508, 0, 0.999999, 0.001508, 0, 0.999999, 0.001508, 0, 0.999999, 0.001508, 0, 0.999999, 0.001508, 0, 0.999993, 0.003772, 0, 0.999993, 0.003772, 0, 0.999993, 0.003772, 0, 0.999993, 0.003772, 0, 0.999993, 0.003772, 0, 0.999993, 0.003772, 0, 0.999978, 0.006669, 0, 0.999978, 0.006669, 0, 0.999978, 0.006669, 0, 0.999978, 0.006669, 0, 0.999978, 0.006669, 0, 0.999978, 0.006669, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999898, 0.014280, 0, 0.999898, 0.014280, 0, 0.999898, 0.014280, 0, 0.999898, 0.014280, 0, 0.999898, 0.014280, 0, 0.999898, 0.014280, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998988, 0.044974, 0, 0.998988, 0.044974, 0, 0.998988, 0.044974, 0, 0.998988, 0.044974, 0, 0.998988, 0.044974, 0, 0.998988, 0.044974, 0, 0.999952, 0.009797, 0, 0.999952, 0.009797, 0, 0.999952, 0.009797, 0, 0.999952, 0.009797, 0, 0.999952, 0.009797, 0, 0.999952, 0.009797, 0, 0.999024, 0.044161, 0, 0.999024, 0.044161, 0, 0.999024, 0.044161, 0, 0.999024, 0.044161, 0, 0.999024, 0.044161, 0, 0.999024, 0.044161, 0, 0.999041, 0.043787, 0, 0.999041, 0.043787, 0, 0.999041, 0.043787, 0, 0.999041, 0.043787, 0, 0.999041, 0.043787, 0, 0.999041, 0.043787, 0, 0.999955, 0.009540, 0, 0.999955, 0.009540, 0, 0.999955, 0.009540, 0, 0.999955, 0.009540, 0, 0.999955, 0.009540, 0, 0.999955, 0.009540, 0, 0.999074, 0.043016, 0, 0.999074, 0.043016, 0, 0.999074, 0.043016, 0, 0.999074, 0.043016, 0, 0.999074, 0.043016, 0, 0.999074, 0.043016, 0, 0.999087, 0.042714, 0, 0.999087, 0.042714, 0, 0.999087, 0.042714, 0, 0.999087, 0.042714, 0, 0.999087, 0.042714, 0, 0.999087, 0.042714, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999350, 0.036053, 0, 0.999350, 0.036053, 0, 0.999350, 0.036053, 0, 0.999350, 0.036053, 0, 0.999350, 0.036053, 0, 0.999350, 0.036053, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999118, 0.041991, 0, 0.999118, 0.041991, 0, 0.999118, 0.041991, 0, 0.999118, 0.041991, 0, 0.999118, 0.041991, 0, 0.999118, 0.041991, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998273, 0.058737, 0, 0.998273, 0.058737, 0, 0.998273, 0.058737, 0, 0.998273, 0.058737, 0, 0.998273, 0.058737, 0, 0.998273, 0.058737, 0, 0.998731, 0.050369, 0, 0.998731, 0.050369, 0, 0.998731, 0.050369, 0, 0.998731, 0.050369, 0, 0.998731, 0.050369, 0, 0.998731, 0.050369, 0, 0.998713, 0.050719, 0, 0.998713, 0.050719, 0, 0.998713, 0.050719, 0, 0.998713, 0.050719, 0, 0.998713, 0.050719, 0, 0.998713, 0.050719, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998346, 0.057499, 0, 0.998346, 0.057499, 0, 0.998346, 0.057499, 0, 0.998346, 0.057499, 0, 0.998346, 0.057499, 0, 0.998346, 0.057499, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998413, 0.056311, 0, 0.998413, 0.056311, 0, 0.998413, 0.056311, 0, 0.998413, 0.056311, 0, 0.998413, 0.056311, 0, 0.998413, 0.056311, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999755, 0.022118, 0, 0.999755, 0.022118, 0, 0.999755, 0.022118, 0, 0.999755, 0.022118, 0, 0.999755, 0.022118, 0, 0.999755, 0.022118, 0, 0.999688, 0.024987, 0, 0.999688, 0.024987, 0, 0.999688, 0.024987, 0, 0.999688, 0.024987, 0, 0.999688, 0.024987, 0, 0.999688, 0.024987, 0, 0.999622, 0.027501, 0, 0.999622, 0.027501, 0, 0.999622, 0.027501, 0, 0.999622, 0.027501, 0, 0.999622, 0.027501, 0, 0.999622, 0.027501, 0, 0.999560, 0.029662, 0, 0.999560, 0.029662, 0, 0.999560, 0.029662, 0, 0.999560, 0.029662, 0, 0.999560, 0.029662, 0, 0.999560, 0.029662, 0, 0.999507, 0.031410, 0, 0.999507, 0.031410, 0, 0.999507, 0.031410, 0, 0.999507, 0.031410, 0, 0.999507, 0.031410, 0, 0.999507, 0.031410, 0, 0.999466, 0.032686, 0, 0.999466, 0.032686, 0, 0.999466, 0.032686, 0, 0.999466, 0.032686, 0, 0.999466, 0.032686, 0, 0.999466, 0.032686, 0, 0.999439, 0.033486, 0, 0.999439, 0.033486, 0, 0.999439, 0.033486, 0, 0.999439, 0.033486, 0, 0.999439, 0.033486, 0, 0.999439, 0.033486, 0, 0.999428, 0.033810, 0, 0.999428, 0.033810, 0, 0.999428, 0.033810, 0, 0.999428, 0.033810, 0, 0.999428, 0.033810, 0, 0.999428, 0.033810, 0, 0.999434, 0.033644, 0, 0.999434, 0.033644, 0, 0.999434, 0.033644, 0, 0.999434, 0.033644, 0, 0.999434, 0.033644, 0, 0.999434, 0.033644, 0, 0.999456, 0.032977, 0, 0.999456, 0.032977, 0, 0.999456, 0.032977, 0, 0.999456, 0.032977, 0, 0.999456, 0.032977, 0, 0.999456, 0.032977, 0, 0.999494, 0.031806, 0, 0.999494, 0.031806, 0, 0.999494, 0.031806, 0, 0.999494, 0.031806, 0, 0.999494, 0.031806, 0, 0.999494, 0.031806, 0, 0.999546, 0.030129, 0, 0.999546, 0.030129, 0, 0.999546, 0.030129, 0, 0.999546, 0.030129, 0, 0.999546, 0.030129, 0, 0.999546, 0.030129, 0, 0.999608, 0.028004, 0, 0.999608, 0.028004, 0, 0.999608, 0.028004, 0, 0.999608, 0.028004, 0, 0.999608, 0.028004, 0, 0.999608, 0.028004, 0, 0.999675, 0.025490, 0, 0.999675, 0.025490, 0, 0.999675, 0.025490, 0, 0.999675, 0.025490, 0, 0.999675, 0.025490, 0, 0.999675, 0.025490, 0, 0.999745, 0.022587, 0, 0.999745, 0.022587, 0, 0.999745, 0.022587, 0, 0.999745, 0.022587, 0, 0.999745, 0.022587, 0, 0.999745, 0.022587, 0, 0.999814, 0.019299, 0, 0.999814, 0.019299, 0, 0.999814, 0.019299, 0, 0.999814, 0.019299, 0, 0.999814, 0.019299, 0, 0.999814, 0.019299, 0, 0.999872, 0.016004, 0, 0.999872, 0.016004, 0, 0.999872, 0.016004, 0, 0.999872, 0.016004, 0, 0.999872, 0.016004, 0, 0.999872, 0.016004, 0, 0.999914, 0.013079, 0, 0.999914, 0.013079, 0, 0.999914, 0.013079, 0, 0.999914, 0.013079, 0, 0.999914, 0.013079, 0, 0.999914, 0.013079, 0, 0.999945, 0.010528, 0, 0.999945, 0.010528, 0, 0.999945, 0.010528, 0, 0.999945, 0.010528, 0, 0.999945, 0.010528, 0, 0.999945, 0.010528, 0, 0.999965, 0.008352, 0, 0.999965, 0.008352, 0, 0.999965, 0.008352, 0, 0.999965, 0.008352, 0, 0.999965, 0.008352, 0, 0.999965, 0.008352, 0, 0.999978, 0.006612, 0, 0.999978, 0.006612, 0, 0.999978, 0.006612, 0, 0.999978, 0.006612, 0, 0.999978, 0.006612, 0, 0.999978, 0.006612, 0, 0.999986, 0.005369, 0, 0.999986, 0.005369, 0, 0.999986, 0.005369, 0, 0.999986, 0.005369, 0, 0.999986, 0.005369, 0, 0.999986, 0.005369, 0, 0.999989, 0.004621, 0, 0.999989, 0.004621, 0, 0.999989, 0.004621, 0, 0.999989, 0.004621, 0, 0.999989, 0.004621, 0, 0.999989, 0.004621, 0, 0.999990, 0.004365, 0, 0.999990, 0.004365, 0, 0.999990, 0.004365, 0, 0.999990, 0.004365, 0, 0.999990, 0.004365, 0, 0.999990, 0.004365, 0, 0.999989, 0.004599, 0, 0.999989, 0.004599, 0, 0.999989, 0.004599, 0, 0.999989, 0.004599, 0, 0.999989, 0.004599, 0, 0.999989, 0.004599, 0, 0.999986, 0.005322, 0, 0.999986, 0.005322, 0, 0.999986, 0.005322, 0, 0.999986, 0.005322, 0, 0.999986, 0.005322, 0, 0.999986, 0.005322, 0, 0.999979, 0.006530, 0, 0.999979, 0.006530, 0, 0.999979, 0.006530, 0, 0.999979, 0.006530, 0, 0.999979, 0.006530, 0, 0.999979, 0.006530, 0, 0.999966, 0.008222, 0, 0.999966, 0.008222, 0, 0.999966, 0.008222, 0, 0.999966, 0.008222, 0, 0.999966, 0.008222, 0, 0.999966, 0.008222, 0, 0.999947, 0.010339, 0, 0.999947, 0.010339, 0, 0.999947, 0.010339, 0, 0.999947, 0.010339, 0, 0.999947, 0.010339, 0, 0.999947, 0.010339, 0, 0.999918, 0.012821, 0, 0.999918, 0.012821, 0, 0.999918, 0.012821, 0, 0.999918, 0.012821, 0, 0.999918, 0.012821, 0, 0.999918, 0.012821, 0, 0.999877, 0.015672, 0, 0.999877, 0.015672, 0, 0.999877, 0.015672, 0, 0.999877, 0.015672, 0, 0.999877, 0.015672, 0, 0.999877, 0.015672, 0, 0.999401, 0.034605, 0, 0.999401, 0.034605, 0, 0.999401, 0.034605, 0, 0.999401, 0.034605, 0, 0.999401, 0.034605, 0, 0.999401, 0.034605, 0, 0.999355, 0.035914, 0, 0.999355, 0.035914, 0, 0.999355, 0.035914, 0, 0.999355, 0.035914, 0, 0.999355, 0.035914, 0, 0.999355, 0.035914, 0, 0.999315, 0.037000, 0, 0.999315, 0.037000, 0, 0.999315, 0.037000, 0, 0.999315, 0.037000, 0, 0.999315, 0.037000, 0, 0.999315, 0.037000, 0, 0.999283, 0.037862, 0, 0.999283, 0.037862, 0, 0.999283, 0.037862, 0, 0.999283, 0.037862, 0, 0.999283, 0.037862, 0, 0.999283, 0.037862, 0, 0.999258, 0.038508, 0, 0.999258, 0.038508, 0, 0.999258, 0.038508, 0, 0.999258, 0.038508, 0, 0.999258, 0.038508, 0, 0.999258, 0.038508, 0, 0.999241, 0.038945, 0, 0.999241, 0.038945, 0, 0.999241, 0.038945, 0, 0.999241, 0.038945, 0, 0.999241, 0.038945, 0, 0.999241, 0.038945, 0, 0.999232, 0.039174, 0, 0.999232, 0.039174, 0, 0.999232, 0.039174, 0, 0.999232, 0.039174, 0, 0.999232, 0.039174, 0, 0.999232, 0.039174, 0, 0.999232, 0.039195, 0, 0.999232, 0.039195, 0, 0.999232, 0.039195, 0, 0.999232, 0.039195, 0, 0.999232, 0.039195, 0, 0.999232, 0.039195, 0, 0.999250, 0.038721, 0, 0.999250, 0.038721, 0, 0.999250, 0.038721, 0, 0.999250, 0.038721, 0, 0.999250, 0.038721, 0, 0.999250, 0.038721, 0, 0.999294, 0.037574, 0, 0.999294, 0.037574, 0, 0.999294, 0.037574, 0, 0.999294, 0.037574, 0, 0.999294, 0.037574, 0, 0.999294, 0.037574, 0, 0.999360, 0.035760, 0, 0.999360, 0.035760, 0, 0.999360, 0.035760, 0, 0.999360, 0.035760, 0, 0.999360, 0.035760, 0, 0.999360, 0.035760, 0, 0.999446, 0.033281, 0, 0.999446, 0.033281, 0, 0.999446, 0.033281, 0, 0.999446, 0.033281, 0, 0.999446, 0.033281, 0, 0.999446, 0.033281, 0, 0.999542, 0.030268, 0, 0.999542, 0.030268, 0, 0.999542, 0.030268, 0, 0.999542, 0.030268, 0, 0.999542, 0.030268, 0, 0.999542, 0.030268, 0, 0.999640, 0.026848, 0, 0.999640, 0.026848, 0, 0.999640, 0.026848, 0, 0.999640, 0.026848, 0, 0.999640, 0.026848, 0, 0.999640, 0.026848, 0, 0.999735, 0.023017, 0, 0.999735, 0.023017, 0, 0.999735, 0.023017, 0, 0.999735, 0.023017, 0, 0.999735, 0.023017, 0, 0.999735, 0.023017, 0, 0.999824, 0.018771, 0, 0.999824, 0.018771, 0, 0.999824, 0.018771, 0, 0.999824, 0.018771, 0, 0.999824, 0.018771, 0, 0.999824, 0.018771, 0, 0.999894, 0.014534, 0, 0.999894, 0.014534, 0, 0.999894, 0.014534, 0, 0.999894, 0.014534, 0, 0.999894, 0.014534, 0, 0.999894, 0.014534, 0, 0.999942, 0.010731, 0, 0.999942, 0.010731, 0, 0.999942, 0.010731, 0, 0.999942, 0.010731, 0, 0.999942, 0.010731, 0, 0.999942, 0.010731, 0, 0.999973, 0.007357, 0, 0.999973, 0.007357, 0, 0.999973, 0.007357, 0, 0.999973, 0.007357, 0, 0.999973, 0.007357, 0, 0.999973, 0.007357, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999998, 0.002014, 0, 0.999998, 0.002014, 0, 0.999998, 0.002014, 0, 0.999998, 0.002014, 0, 0.999998, 0.002014, 0, 0.999998, 0.002014, 0, 1, 0.000296, 0, 1, 0.000296, 0, 1, 0.000296, 0, 1, 0.000296, 0, 1, 0.000296, 0, 1, 0.000296, 0, 1.000000, -0.000740, 0, 1.000000, -0.000740, 0, 1.000000, -0.000740, 0, 1.000000, -0.000740, 0, 1.000000, -0.000740, 0, 1.000000, -0.000740, 0, 0.999999, -0.001090, 0, 0.999999, -0.001090, 0, 0.999999, -0.001090, 0, 0.999999, -0.001090, 0, 0.999999, -0.001090, 0, 0.999999, -0.001090, 0, 1.000000, -0.000991, 0, 1.000000, -0.000991, 0, 1.000000, -0.000991, 0, 1.000000, -0.000991, 0, 1.000000, -0.000991, 0, 1.000000, -0.000991, 0, 1.000000, -0.000690, 0, 1.000000, -0.000690, 0, 1.000000, -0.000690, 0, 1.000000, -0.000690, 0, 1.000000, -0.000690, 0, 1.000000, -0.000690, 0, 1, -0.000186, 0, 1, -0.000186, 0, 1, -0.000186, 0, 1, -0.000186, 0, 1, -0.000186, 0, 1, -0.000186, 0, 1.000000, 0.000523, 0, 1.000000, 0.000523, 0, 1.000000, 0.000523, 0, 1.000000, 0.000523, 0, 1.000000, 0.000523, 0, 1.000000, 0.000523, 0, 0.999999, 0.001443, 0, 0.999999, 0.001443, 0, 0.999999, 0.001443, 0, 0.999999, 0.001443, 0, 0.999999, 0.001443, 0, 0.999999, 0.001443, 0, 0.999997, 0.002582, 0, 0.999997, 0.002582, 0, 0.999997, 0.002582, 0, 0.999997, 0.002582, 0, 0.999997, 0.002582, 0, 0.999997, 0.002582, 0, 0.999992, 0.003939, 0, 0.999992, 0.003939, 0, 0.999992, 0.003939, 0, 0.999992, 0.003939, 0, 0.999992, 0.003939, 0, 0.999992, 0.003939, 0, 0.999985, 0.005515, 0, 0.999985, 0.005515, 0, 0.999985, 0.005515, 0, 0.999985, 0.005515, 0, 0.999985, 0.005515, 0, 0.999985, 0.005515, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.998524, 0.054315, 0, 0.998524, 0.054315, 0, 0.998524, 0.054315, 0, 0.998524, 0.054315, 0, 0.998524, 0.054315, 0, 0.998524, 0.054315, 0, 0.998543, 0.053970, 0, 0.998543, 0.053970, 0, 0.998543, 0.053970, 0, 0.998543, 0.053970, 0, 0.998543, 0.053970, 0, 0.998543, 0.053970, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999865, 0.016459, 0, 0.999865, 0.016459, 0, 0.999865, 0.016459, 0, 0.999865, 0.016459, 0, 0.999865, 0.016459, 0, 0.999865, 0.016459, 0, 0.999857, 0.016919, 0, 0.999857, 0.016919, 0, 0.999857, 0.016919, 0, 0.999857, 0.016919, 0, 0.999857, 0.016919, 0, 0.999857, 0.016919, 0, 0.999902, 0.014001, 0, 0.999902, 0.014001, 0, 0.999902, 0.014001, 0, 0.999902, 0.014001, 0, 0.999902, 0.014001, 0, 0.999902, 0.014001, 0, 0.999934, 0.011448, 0, 0.999934, 0.011448, 0, 0.999934, 0.011448, 0, 0.999934, 0.011448, 0, 0.999934, 0.011448, 0, 0.999934, 0.011448, 0, 0.999957, 0.009264, 0, 0.999957, 0.009264, 0, 0.999957, 0.009264, 0, 0.999957, 0.009264, 0, 0.999957, 0.009264, 0, 0.999957, 0.009264, 0, 0.999972, 0.007449, 0, 0.999972, 0.007449, 0, 0.999972, 0.007449, 0, 0.999972, 0.007449, 0, 0.999972, 0.007449, 0, 0.999972, 0.007449, 0, 0.999982, 0.006023, 0, 0.999982, 0.006023, 0, 0.999982, 0.006023, 0, 0.999982, 0.006023, 0, 0.999982, 0.006023, 0, 0.999982, 0.006023, 0, 0.999987, 0.005003, 0, 0.999987, 0.005003, 0, 0.999987, 0.005003, 0, 0.999987, 0.005003, 0, 0.999987, 0.005003, 0, 0.999987, 0.005003, 0, 0.999990, 0.004388, 0, 0.999990, 0.004388, 0, 0.999990, 0.004388, 0, 0.999990, 0.004388, 0, 0.999990, 0.004388, 0, 0.999990, 0.004388, 0, 0.999991, 0.004175, 0, 0.999991, 0.004175, 0, 0.999991, 0.004175, 0, 0.999991, 0.004175, 0, 0.999991, 0.004175, 0, 0.999991, 0.004175, 0, 0.999991, 0.004217, 0, 0.999991, 0.004217, 0, 0.999991, 0.004217, 0, 0.999991, 0.004217, 0, 0.999991, 0.004217, 0, 0.999991, 0.004217, 0, 0.999991, 0.004356, 0, 0.999991, 0.004356, 0, 0.999991, 0.004356, 0, 0.999991, 0.004356, 0, 0.999991, 0.004356, 0, 0.999991, 0.004356, 0, 0.999989, 0.004592, 0, 0.999989, 0.004592, 0, 0.999989, 0.004592, 0, 0.999989, 0.004592, 0, 0.999989, 0.004592, 0, 0.999989, 0.004592, 0, 0.999988, 0.004925, 0, 0.999988, 0.004925, 0, 0.999988, 0.004925, 0, 0.999988, 0.004925, 0, 0.999988, 0.004925, 0, 0.999988, 0.004925, 0, 0.999986, 0.005354, 0, 0.999986, 0.005354, 0, 0.999986, 0.005354, 0, 0.999986, 0.005354, 0, 0.999986, 0.005354, 0, 0.999986, 0.005354, 0, 0.999983, 0.005878, 0, 0.999983, 0.005878, 0, 0.999983, 0.005878, 0, 0.999983, 0.005878, 0, 0.999983, 0.005878, 0, 0.999983, 0.005878, 0, 0.999979, 0.006498, 0, 0.999979, 0.006498, 0, 0.999979, 0.006498, 0, 0.999979, 0.006498, 0, 0.999979, 0.006498, 0, 0.999979, 0.006498, 0, 0.999974, 0.007212, 0, 0.999974, 0.007212, 0, 0.999974, 0.007212, 0, 0.999974, 0.007212, 0, 0.999974, 0.007212, 0, 0.999974, 0.007212, 0, 0.999998, 0.002061, 0, 0.999998, 0.002061, 0, 0.999998, 0.002061, 0, 0.999998, 0.002061, 0, 0.999998, 0.002061, 0, 0.999998, 0.002061, 0, 0.999999, 0.001351, 0, 0.999999, 0.001351, 0, 0.999999, 0.001351, 0, 0.999999, 0.001351, 0, 0.999999, 0.001351, 0, 0.999999, 0.001351, 0, 1.000000, 0.000731, 0, 1.000000, 0.000731, 0, 1.000000, 0.000731, 0, 1.000000, 0.000731, 0, 1.000000, 0.000731, 0, 1.000000, 0.000731, 0, 1, 0.000199, 0, 1, 0.000199, 0, 1, 0.000199, 0, 1, 0.000199, 0, 1, 0.000199, 0, 1, 0.000199, 0, 1, -0.000243, 0, 1, -0.000243, 0, 1, -0.000243, 0, 1, -0.000243, 0, 1, -0.000243, 0, 1, -0.000243, 0, 1.000000, -0.000591, 0, 1.000000, -0.000591, 0, 1.000000, -0.000591, 0, 1.000000, -0.000591, 0, 1.000000, -0.000591, 0, 1.000000, -0.000591, 0, 1.000000, -0.000841, 0, 1.000000, -0.000841, 0, 1.000000, -0.000841, 0, 1.000000, -0.000841, 0, 1.000000, -0.000841, 0, 1.000000, -0.000841, 0, 1.000000, -0.000992, 0, 1.000000, -0.000992, 0, 1.000000, -0.000992, 0, 1.000000, -0.000992, 0, 1.000000, -0.000992, 0, 1.000000, -0.000992, 0, 0.999999, -0.001044, 0, 0.999999, -0.001044, 0, 0.999999, -0.001044, 0, 0.999999, -0.001044, 0, 0.999999, -0.001044, 0, 0.999999, -0.001044, 0, 1.000000, -0.000727, 0, 1.000000, -0.000727, 0, 1.000000, -0.000727, 0, 1.000000, -0.000727, 0, 1.000000, -0.000727, 0, 1.000000, -0.000727, 0, 1, 0.000238, 0, 1, 0.000238, 0, 1, 0.000238, 0, 1, 0.000238, 0, 1, 0.000238, 0, 1, 0.000238, 0, 0.999998, 0.001858, 0, 0.999998, 0.001858, 0, 0.999998, 0.001858, 0, 0.999998, 0.001858, 0, 0.999998, 0.001858, 0, 0.999998, 0.001858, 0, 0.999991, 0.004136, 0, 0.999991, 0.004136, 0, 0.999991, 0.004136, 0, 0.999991, 0.004136, 0, 0.999991, 0.004136, 0, 0.999991, 0.004136, 0, 0.999976, 0.006979, 0, 0.999976, 0.006979, 0, 0.999976, 0.006979, 0, 0.999976, 0.006979, 0, 0.999976, 0.006979, 0, 0.999976, 0.006979, 0, 0.999947, 0.010289, 0, 0.999947, 0.010289, 0, 0.999947, 0.010289, 0, 0.999947, 0.010289, 0, 0.999947, 0.010289, 0, 0.999947, 0.010289, 0, 0.999901, 0.014061, 0, 0.999901, 0.014061, 0, 0.999901, 0.014061, 0, 0.999901, 0.014061, 0, 0.999901, 0.014061, 0, 0.999901, 0.014061, 0, 0.999833, 0.018291, 0, 0.999833, 0.018291, 0, 0.999833, 0.018291, 0, 0.999833, 0.018291, 0, 0.999833, 0.018291, 0, 0.999833, 0.018291, 0, 0.999744, 0.022650, 0, 0.999744, 0.022650, 0, 0.999744, 0.022650, 0, 0.999744, 0.022650, 0, 0.999744, 0.022650, 0, 0.999744, 0.022650, 0, 0.999649, 0.026513, 0, 0.999649, 0.026513, 0, 0.999649, 0.026513, 0, 0.999649, 0.026513, 0, 0.999649, 0.026513, 0, 0.999649, 0.026513, 0, 0.999554, 0.029875, 0, 0.999554, 0.029875, 0, 0.999554, 0.029875, 0, 0.999554, 0.029875, 0, 0.999554, 0.029875, 0, 0.999554, 0.029875, 0, 0.999464, 0.032733, 0, 0.999464, 0.032733, 0, 0.999464, 0.032733, 0, 0.999464, 0.032733, 0, 0.999464, 0.032733, 0, 0.999464, 0.032733, 0, 0.999388, 0.034988, 0, 0.999388, 0.034988, 0, 0.999388, 0.034988, 0, 0.999388, 0.034988, 0, 0.999388, 0.034988, 0, 0.999388, 0.034988, 0, 0.999332, 0.036549, 0, 0.999332, 0.036549, 0, 0.999332, 0.036549, 0, 0.999332, 0.036549, 0, 0.999332, 0.036549, 0, 0.999332, 0.036549, 0, 0.999300, 0.037418, 0, 0.999300, 0.037418, 0, 0.999300, 0.037418, 0, 0.999300, 0.037418, 0, 0.999300, 0.037418, 0, 0.999300, 0.037418, 0, 0.999293, 0.037603, 0, 0.999293, 0.037603, 0, 0.999293, 0.037603, 0, 0.999293, 0.037603, 0, 0.999293, 0.037603, 0, 0.999293, 0.037603, 0, 0.999309, 0.037177, 0, 0.999309, 0.037177, 0, 0.999309, 0.037177, 0, 0.999309, 0.037177, 0, 0.999309, 0.037177, 0, 0.999309, 0.037177, 0, 0.999345, 0.036181, 0, 0.999345, 0.036181, 0, 0.999345, 0.036181, 0, 0.999345, 0.036181, 0, 0.999345, 0.036181, 0, 0.999345, 0.036181, 0, 0.999401, 0.034617, 0, 0.999401, 0.034617, 0, 0.999401, 0.034617, 0, 0.999401, 0.034617, 0, 0.999401, 0.034617, 0, 0.999401, 0.034617, 0, 0.999472, 0.032489, 0, 0.999472, 0.032489, 0, 0.999472, 0.032489, 0, 0.999472, 0.032489, 0, 0.999472, 0.032489, 0, 0.999472, 0.032489, 0, 0.999554, 0.029882, 0, 0.999554, 0.029882, 0, 0.999554, 0.029882, 0, 0.999554, 0.029882, 0, 0.999554, 0.029882, 0, 0.999554, 0.029882, 0, 0.999639, 0.026879, 0, 0.999639, 0.026879, 0, 0.999639, 0.026879, 0, 0.999639, 0.026879, 0, 0.999639, 0.026879, 0, 0.999639, 0.026879, 0, 0.999724, 0.023478, 0, 0.999724, 0.023478, 0, 0.999724, 0.023478, 0, 0.999724, 0.023478, 0, 0.999724, 0.023478, 0, 0.999724, 0.023478, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999713, 0.023974, 0, 0.999713, 0.023974, 0, 0.999713, 0.023974, 0, 0.999713, 0.023974, 0, 0.999713, 0.023974, 0, 0.999713, 0.023974, 0, 0.999662, 0.025989, 0, 0.999662, 0.025989, 0, 0.999662, 0.025989, 0, 0.999662, 0.025989, 0, 0.999662, 0.025989, 0, 0.999662, 0.025989, 0, 0.999614, 0.027777, 0, 0.999614, 0.027777, 0, 0.999614, 0.027777, 0, 0.999614, 0.027777, 0, 0.999614, 0.027777, 0, 0.999614, 0.027777, 0, 0.999570, 0.029338, 0, 0.999570, 0.029338, 0, 0.999570, 0.029338, 0, 0.999570, 0.029338, 0, 0.999570, 0.029338, 0, 0.999570, 0.029338, 0, 0.999531, 0.030615, 0, 0.999531, 0.030615, 0, 0.999531, 0.030615, 0, 0.999531, 0.030615, 0, 0.999531, 0.030615, 0, 0.999531, 0.030615, 0, 0.999502, 0.031549, 0, 0.999502, 0.031549, 0, 0.999502, 0.031549, 0, 0.999502, 0.031549, 0, 0.999502, 0.031549, 0, 0.999502, 0.031549, 0, 0.999483, 0.032139, 0, 0.999483, 0.032139, 0, 0.999483, 0.032139, 0, 0.999483, 0.032139, 0, 0.999483, 0.032139, 0, 0.999483, 0.032139, 0, 0.999476, 0.032384, 0, 0.999476, 0.032384, 0, 0.999476, 0.032384, 0, 0.999476, 0.032384, 0, 0.999476, 0.032384, 0, 0.999476, 0.032384, 0, 0.999479, 0.032290, 0, 0.999479, 0.032290, 0, 0.999479, 0.032290, 0, 0.999479, 0.032290, 0, 0.999479, 0.032290, 0, 0.999479, 0.032290, 0, 0.999493, 0.031844, 0, 0.999493, 0.031844, 0, 0.999493, 0.031844, 0, 0.999493, 0.031844, 0, 0.999493, 0.031844, 0, 0.999493, 0.031844, 0, 0.999518, 0.031042, 0, 0.999518, 0.031042, 0, 0.999518, 0.031042, 0, 0.999518, 0.031042, 0, 0.999518, 0.031042, 0, 0.999518, 0.031042, 0, 0.999553, 0.029885, 0, 0.999553, 0.029885, 0, 0.999553, 0.029885, 0, 0.999553, 0.029885, 0, 0.999553, 0.029885, 0, 0.999553, 0.029885, 0, 0.999597, 0.028401, 0, 0.999597, 0.028401, 0, 0.999597, 0.028401, 0, 0.999597, 0.028401, 0, 0.999597, 0.028401, 0, 0.999597, 0.028401, 0, 0.999646, 0.026619, 0, 0.999646, 0.026619, 0, 0.999646, 0.026619, 0, 0.999646, 0.026619, 0, 0.999646, 0.026619, 0, 0.999646, 0.026619, 0, 0.999699, 0.024541, 0, 0.999699, 0.024541, 0, 0.999699, 0.024541, 0, 0.999699, 0.024541, 0, 0.999699, 0.024541, 0, 0.999699, 0.024541, 0, 0.999754, 0.022167, 0, 0.999754, 0.022167, 0, 0.999754, 0.022167, 0, 0.999754, 0.022167, 0, 0.999754, 0.022167, 0, 0.999754, 0.022167, 0, 0.999861, 0.016674, 0, 0.999861, 0.016674, 0, 0.999861, 0.016674, 0, 0.999861, 0.016674, 0, 0.999861, 0.016674, 0, 0.999861, 0.016674, 0, 0.999865, 0.016437, 0, 0.999865, 0.016437, 0, 0.999865, 0.016437, 0, 0.999865, 0.016437, 0, 0.999865, 0.016437, 0, 0.999865, 0.016437, 0, 0.999872, 0.016003, 0, 0.999872, 0.016003, 0, 0.999872, 0.016003, 0, 0.999872, 0.016003, 0, 0.999872, 0.016003, 0, 0.999872, 0.016003, 0, 0.999882, 0.015371, 0, 0.999882, 0.015371, 0, 0.999882, 0.015371, 0, 0.999882, 0.015371, 0, 0.999882, 0.015371, 0, 0.999882, 0.015371, 0, 0.999895, 0.014504, 0, 0.999895, 0.014504, 0, 0.999895, 0.014504, 0, 0.999895, 0.014504, 0, 0.999895, 0.014504, 0, 0.999895, 0.014504, 0, 0.999911, 0.013366, 0, 0.999911, 0.013366, 0, 0.999911, 0.013366, 0, 0.999911, 0.013366, 0, 0.999911, 0.013366, 0, 0.999911, 0.013366, 0, 0.999929, 0.011958, 0, 0.999929, 0.011958, 0, 0.999929, 0.011958, 0, 0.999929, 0.011958, 0, 0.999929, 0.011958, 0, 0.999929, 0.011958, 0, 0.999947, 0.010280, 0, 0.999947, 0.010280, 0, 0.999947, 0.010280, 0, 0.999947, 0.010280, 0, 0.999947, 0.010280, 0, 0.999947, 0.010280, 0, 0.999960, 0.008901, 0, 0.999960, 0.008901, 0, 0.999960, 0.008901, 0, 0.999960, 0.008901, 0, 0.999960, 0.008901, 0, 0.999960, 0.008901, 0, 0.999971, 0.007672, 0, 0.999971, 0.007672, 0, 0.999971, 0.007672, 0, 0.999971, 0.007672, 0, 0.999971, 0.007672, 0, 0.999971, 0.007672, 0, 0.999978, 0.006593, 0, 0.999978, 0.006593, 0, 0.999978, 0.006593, 0, 0.999978, 0.006593, 0, 0.999978, 0.006593, 0, 0.999978, 0.006593, 0, 0.999984, 0.005666, 0, 0.999984, 0.005666, 0, 0.999984, 0.005666, 0, 0.999984, 0.005666, 0, 0.999984, 0.005666, 0, 0.999984, 0.005666, 0, 0.999988, 0.004920, 0, 0.999988, 0.004920, 0, 0.999988, 0.004920, 0, 0.999988, 0.004920, 0, 0.999988, 0.004920, 0, 0.999988, 0.004920, 0, 0.999990, 0.004386, 0, 0.999990, 0.004386, 0, 0.999990, 0.004386, 0, 0.999990, 0.004386, 0, 0.999990, 0.004386, 0, 0.999990, 0.004386, 0, 0.999992, 0.004064, 0, 0.999992, 0.004064, 0, 0.999992, 0.004064, 0, 0.999992, 0.004064, 0, 0.999992, 0.004064, 0, 0.999992, 0.004064, 0, 0.999992, 0.003952, 0, 0.999992, 0.003952, 0, 0.999992, 0.003952, 0, 0.999992, 0.003952, 0, 0.999992, 0.003952, 0, 0.999992, 0.003952, 0, 0.999991, 0.004141, 0, 0.999991, 0.004141, 0, 0.999991, 0.004141, 0, 0.999991, 0.004141, 0, 0.999991, 0.004141, 0, 0.999991, 0.004141, 0, 0.999989, 0.004725, 0, 0.999989, 0.004725, 0, 0.999989, 0.004725, 0, 0.999989, 0.004725, 0, 0.999989, 0.004725, 0, 0.999989, 0.004725, 0, 0.999984, 0.005702, 0, 0.999984, 0.005702, 0, 0.999984, 0.005702, 0, 0.999984, 0.005702, 0, 0.999984, 0.005702, 0, 0.999984, 0.005702, 0, 0.999975, 0.007071, 0, 0.999975, 0.007071, 0, 0.999975, 0.007071, 0, 0.999975, 0.007071, 0, 0.999975, 0.007071, 0, 0.999975, 0.007071, 0, 0.999962, 0.008764, 0, 0.999962, 0.008764, 0, 0.999962, 0.008764, 0, 0.999962, 0.008764, 0, 0.999962, 0.008764, 0, 0.999962, 0.008764, 0, 0.999943, 0.010716, 0, 0.999943, 0.010716, 0, 0.999943, 0.010716, 0, 0.999943, 0.010716, 0, 0.999943, 0.010716, 0, 0.999943, 0.010716, 0, 0.999916, 0.012927, 0, 0.999916, 0.012927, 0, 0.999916, 0.012927, 0, 0.999916, 0.012927, 0, 0.999916, 0.012927, 0, 0.999916, 0.012927, 0, 0.999881, 0.015399, 0, 0.999881, 0.015399, 0, 0.999881, 0.015399, 0, 0.999881, 0.015399, 0, 0.999881, 0.015399, 0, 0.999881, 0.015399, 0, 0.999862, 0.016621, 0, 0.999862, 0.016621, 0, 0.999862, 0.016621, 0, 0.999862, 0.016621, 0, 0.999862, 0.016621, 0, 0.999862, 0.016621, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999988, 0.004889, 0, 0.999988, 0.004889, 0, 0.999988, 0.004889, 0, 0.999988, 0.004889, 0, 0.999988, 0.004889, 0, 0.999988, 0.004889, 0, 0.999994, 0.003487, 0, 0.999994, 0.003487, 0, 0.999994, 0.003487, 0, 0.999994, 0.003487, 0, 0.999994, 0.003487, 0, 0.999994, 0.003487, 0, 0.999997, 0.002280, 0, 0.999997, 0.002280, 0, 0.999997, 0.002280, 0, 0.999997, 0.002280, 0, 0.999997, 0.002280, 0, 0.999997, 0.002280, 0, 0.999999, 0.001268, 0, 0.999999, 0.001268, 0, 0.999999, 0.001268, 0, 0.999999, 0.001268, 0, 0.999999, 0.001268, 0, 0.999999, 0.001268, 0, 1.000000, 0.000453, 0, 1.000000, 0.000453, 0, 1.000000, 0.000453, 0, 1.000000, 0.000453, 0, 1.000000, 0.000453, 0, 1.000000, 0.000453, 0, 1, -0.000175, 0, 1, -0.000175, 0, 1, -0.000175, 0, 1, -0.000175, 0, 1, -0.000175, 0, 1, -0.000175, 0, 1.000000, -0.000626, 0, 1.000000, -0.000626, 0, 1.000000, -0.000626, 0, 1.000000, -0.000626, 0, 1.000000, -0.000626, 0, 1.000000, -0.000626, 0, 1.000000, -0.000897, 0, 1.000000, -0.000897, 0, 1.000000, -0.000897, 0, 1.000000, -0.000897, 0, 1.000000, -0.000897, 0, 1.000000, -0.000897, 0, 1.000000, -0.000990, 0, 1.000000, -0.000990, 0, 1.000000, -0.000990, 0, 1.000000, -0.000990, 0, 1.000000, -0.000990, 0, 1.000000, -0.000990, 0, 1.000000, -0.000805, 0, 1.000000, -0.000805, 0, 1.000000, -0.000805, 0, 1.000000, -0.000805, 0, 1.000000, -0.000805, 0, 1.000000, -0.000805, 0, 1, -0.000245, 0, 1, -0.000245, 0, 1, -0.000245, 0, 1, -0.000245, 0, 1, -0.000245, 0, 1, -0.000245, 0, 1.000000, 0.000694, 0, 1.000000, 0.000694, 0, 1.000000, 0.000694, 0, 1.000000, 0.000694, 0, 1.000000, 0.000694, 0, 1.000000, 0.000694, 0, 0.999998, 0.002012, 0, 0.999998, 0.002012, 0, 0.999998, 0.002012, 0, 0.999998, 0.002012, 0, 0.999998, 0.002012, 0, 0.999998, 0.002012, 0, 0.999993, 0.003649, 0, 0.999993, 0.003649, 0, 0.999993, 0.003649, 0, 0.999993, 0.003649, 0, 0.999993, 0.003649, 0, 0.999993, 0.003649, 0, 0.999985, 0.005545, 0, 0.999985, 0.005545, 0, 0.999985, 0.005545, 0, 0.999985, 0.005545, 0, 0.999985, 0.005545, 0, 0.999985, 0.005545, 0, 0.999970, 0.007698, 0, 0.999970, 0.007698, 0, 0.999970, 0.007698, 0, 0.999970, 0.007698, 0, 0.999970, 0.007698, 0, 0.999970, 0.007698, 0, 0.999949, 0.010105, 0, 0.999949, 0.010105, 0, 0.999949, 0.010105, 0, 0.999949, 0.010105, 0, 0.999949, 0.010105, 0, 0.999949, 0.010105, 0, 0.999917, 0.012876, 0, 0.999917, 0.012876, 0, 0.999917, 0.012876, 0, 0.999917, 0.012876, 0, 0.999917, 0.012876, 0, 0.999917, 0.012876, 0, 0.999883, 0.015274, 0, 0.999883, 0.015274, 0, 0.999883, 0.015274, 0, 0.999883, 0.015274, 0, 0.999883, 0.015274, 0, 0.999883, 0.015274, 0, 0.999850, 0.017297, 0, 0.999850, 0.017297, 0, 0.999850, 0.017297, 0, 0.999850, 0.017297, 0, 0.999850, 0.017297, 0, 0.999850, 0.017297, 0, 0.999821, 0.018944, 0, 0.999821, 0.018944, 0, 0.999821, 0.018944, 0, 0.999821, 0.018944, 0, 0.999821, 0.018944, 0, 0.999821, 0.018944, 0, 0.999796, 0.020207, 0, 0.999796, 0.020207, 0, 0.999796, 0.020207, 0, 0.999796, 0.020207, 0, 0.999796, 0.020207, 0, 0.999796, 0.020207, 0, 0.999778, 0.021082, 0, 0.999778, 0.021082, 0, 0.999778, 0.021082, 0, 0.999778, 0.021082, 0, 0.999778, 0.021082, 0, 0.999778, 0.021082, 0, 0.999767, 0.021572, 0, 0.999767, 0.021572, 0, 0.999767, 0.021572, 0, 0.999767, 0.021572, 0, 0.999767, 0.021572, 0, 0.999767, 0.021572, 0, 0.999765, 0.021678, 0, 0.999765, 0.021678, 0, 0.999765, 0.021678, 0, 0.999765, 0.021678, 0, 0.999765, 0.021678, 0, 0.999765, 0.021678, 0, 0.999769, 0.021508, 0, 0.999769, 0.021508, 0, 0.999769, 0.021508, 0, 0.999769, 0.021508, 0, 0.999769, 0.021508, 0, 0.999769, 0.021508, 0, 0.999757, 0.022057, 0, 0.999757, 0.022057, 0, 0.999757, 0.022057, 0, 0.999757, 0.022057, 0, 0.999757, 0.022057, 0, 0.999757, 0.022057, 0, 0.999714, 0.023919, 0, 0.999714, 0.023919, 0, 0.999714, 0.023919, 0, 0.999714, 0.023919, 0, 0.999714, 0.023919, 0, 0.999714, 0.023919, 0, 0.999673, 0.025566, 0, 0.999673, 0.025566, 0, 0.999673, 0.025566, 0, 0.999673, 0.025566, 0, 0.999673, 0.025566, 0, 0.999673, 0.025566, 0, 0.999635, 0.026999, 0, 0.999635, 0.026999, 0, 0.999635, 0.026999, 0, 0.999635, 0.026999, 0, 0.999635, 0.026999, 0, 0.999635, 0.026999, 0, 0.999602, 0.028220, 0, 0.999602, 0.028220, 0, 0.999602, 0.028220, 0, 0.999602, 0.028220, 0, 0.999602, 0.028220, 0, 0.999602, 0.028220, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999552, 0.029929, 0, 0.999552, 0.029929, 0, 0.999552, 0.029929, 0, 0.999552, 0.029929, 0, 0.999552, 0.029929, 0, 0.999552, 0.029929, 0, 0.999538, 0.030394, 0, 0.999538, 0.030394, 0, 0.999538, 0.030394, 0, 0.999538, 0.030394, 0, 0.999538, 0.030394, 0, 0.999538, 0.030394, 0, 0.999532, 0.030596, 0, 0.999532, 0.030596, 0, 0.999532, 0.030596, 0, 0.999532, 0.030596, 0, 0.999532, 0.030596, 0, 0.999532, 0.030596, 0, 0.999532, 0.030595, 0, 0.999532, 0.030595, 0, 0.999532, 0.030595, 0, 0.999532, 0.030595, 0, 0.999532, 0.030595, 0, 0.999532, 0.030595, 0, 0.999535, 0.030501, 0, 0.999535, 0.030501, 0, 0.999535, 0.030501, 0, 0.999535, 0.030501, 0, 0.999535, 0.030501, 0, 0.999535, 0.030501, 0, 0.999541, 0.030312, 0, 0.999541, 0.030312, 0, 0.999541, 0.030312, 0, 0.999541, 0.030312, 0, 0.999541, 0.030312, 0, 0.999541, 0.030312, 0, 0.999549, 0.030030, 0, 0.999549, 0.030030, 0, 0.999549, 0.030030, 0, 0.999549, 0.030030, 0, 0.999549, 0.030030, 0, 0.999549, 0.030030, 0, 0.999560, 0.029653, 0, 0.999560, 0.029653, 0, 0.999560, 0.029653, 0, 0.999560, 0.029653, 0, 0.999560, 0.029653, 0, 0.999560, 0.029653, 0, 0.999574, 0.029181, 0, 0.999574, 0.029181, 0, 0.999574, 0.029181, 0, 0.999574, 0.029181, 0, 0.999574, 0.029181, 0, 0.999574, 0.029181, 0, 0.999591, 0.028613, 0, 0.999591, 0.028613, 0, 0.999591, 0.028613, 0, 0.999591, 0.028613, 0, 0.999591, 0.028613, 0, 0.999591, 0.028613, 0, 0.999609, 0.027950, 0, 0.999609, 0.027950, 0, 0.999609, 0.027950, 0, 0.999609, 0.027950, 0, 0.999609, 0.027950, 0, 0.999609, 0.027950, 0, 0.999457, 0.032936, 0, 0.999457, 0.032936, 0, 0.999457, 0.032936, 0, 0.999457, 0.032936, 0, 0.999457, 0.032936, 0, 0.999457, 0.032936, 0, 0.999436, 0.033584, 0, 0.999436, 0.033584, 0, 0.999436, 0.033584, 0, 0.999436, 0.033584, 0, 0.999436, 0.033584, 0, 0.999436, 0.033584, 0, 0.999417, 0.034138, 0, 0.999417, 0.034138, 0, 0.999417, 0.034138, 0, 0.999417, 0.034138, 0, 0.999417, 0.034138, 0, 0.999417, 0.034138, 0, 0.999401, 0.034598, 0, 0.999401, 0.034598, 0, 0.999401, 0.034598, 0, 0.999401, 0.034598, 0, 0.999401, 0.034598, 0, 0.999401, 0.034598, 0, 0.999389, 0.034965, 0, 0.999389, 0.034965, 0, 0.999389, 0.034965, 0, 0.999389, 0.034965, 0, 0.999389, 0.034965, 0, 0.999389, 0.034965, 0, 0.999379, 0.035239, 0, 0.999379, 0.035239, 0, 0.999379, 0.035239, 0, 0.999379, 0.035239, 0, 0.999379, 0.035239, 0, 0.999379, 0.035239, 0, 0.999373, 0.035420, 0, 0.999373, 0.035420, 0, 0.999373, 0.035420, 0, 0.999373, 0.035420, 0, 0.999373, 0.035420, 0, 0.999373, 0.035420, 0, 0.999369, 0.035508, 0, 0.999369, 0.035508, 0, 0.999369, 0.035508, 0, 0.999369, 0.035508, 0, 0.999369, 0.035508, 0, 0.999369, 0.035508, 0, 0.999370, 0.035505, 0, 0.999370, 0.035505, 0, 0.999370, 0.035505, 0, 0.999370, 0.035505, 0, 0.999370, 0.035505, 0, 0.999370, 0.035505, 0, 0.999383, 0.035132, 0, 0.999383, 0.035132, 0, 0.999383, 0.035132, 0, 0.999383, 0.035132, 0, 0.999383, 0.035132, 0, 0.999383, 0.035132, 0, 0.999412, 0.034282, 0, 0.999412, 0.034282, 0, 0.999412, 0.034282, 0, 0.999412, 0.034282, 0, 0.999412, 0.034282, 0, 0.999412, 0.034282, 0, 0.999457, 0.032958, 0, 0.999457, 0.032958, 0, 0.999457, 0.032958, 0, 0.999457, 0.032958, 0, 0.999457, 0.032958, 0, 0.999457, 0.032958, 0, 0.999514, 0.031161, 0, 0.999514, 0.031161, 0, 0.999514, 0.031161, 0, 0.999514, 0.031161, 0, 0.999514, 0.031161, 0, 0.999514, 0.031161, 0, 0.999583, 0.028884, 0, 0.999583, 0.028884, 0, 0.999583, 0.028884, 0, 0.999583, 0.028884, 0, 0.999583, 0.028884, 0, 0.999583, 0.028884, 0, 0.999659, 0.026115, 0, 0.999659, 0.026115, 0, 0.999659, 0.026115, 0, 0.999659, 0.026115, 0, 0.999659, 0.026115, 0, 0.999659, 0.026115, 0, 0.999739, 0.022853, 0, 0.999739, 0.022853, 0, 0.999739, 0.022853, 0, 0.999739, 0.022853, 0, 0.999739, 0.022853, 0, 0.999739, 0.022853, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999653, 0.026348, 0, 0.999653, 0.026348, 0, 0.999653, 0.026348, 0, 0.999653, 0.026348, 0, 0.999653, 0.026348, 0, 0.999653, 0.026348, 0, 0.999634, 0.027040, 0, 0.999634, 0.027040, 0, 0.999634, 0.027040, 0, 0.999634, 0.027040, 0, 0.999634, 0.027040, 0, 0.999634, 0.027040, 0, 0.999618, 0.027646, 0, 0.999618, 0.027646, 0, 0.999618, 0.027646, 0, 0.999618, 0.027646, 0, 0.999618, 0.027646, 0, 0.999618, 0.027646, 0, 0.999603, 0.028166, 0, 0.999603, 0.028166, 0, 0.999603, 0.028166, 0, 0.999603, 0.028166, 0, 0.999603, 0.028166, 0, 0.999603, 0.028166, 0, 0.999591, 0.028599, 0, 0.999591, 0.028599, 0, 0.999591, 0.028599, 0, 0.999591, 0.028599, 0, 0.999591, 0.028599, 0, 0.999591, 0.028599, 0, 0.999581, 0.028946, 0, 0.999581, 0.028946, 0, 0.999581, 0.028946, 0, 0.999581, 0.028946, 0, 0.999581, 0.028946, 0, 0.999581, 0.028946, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999573, 0.029205, 0, 0.999568, 0.029376, 0, 0.999568, 0.029376, 0, 0.999568, 0.029376, 0, 0.999568, 0.029376, 0, 0.999568, 0.029376, 0, 0.999568, 0.029376, 0, 0.999566, 0.029459, 0, 0.999566, 0.029459, 0, 0.999566, 0.029459, 0, 0.999566, 0.029459, 0, 0.999566, 0.029459, 0, 0.999566, 0.029459, 0, 0.999567, 0.029435, 0, 0.999567, 0.029435, 0, 0.999567, 0.029435, 0, 0.999567, 0.029435, 0, 0.999567, 0.029435, 0, 0.999567, 0.029435, 0, 0.999572, 0.029241, 0, 0.999572, 0.029241, 0, 0.999572, 0.029241, 0, 0.999572, 0.029241, 0, 0.999572, 0.029241, 0, 0.999572, 0.029241, 0, 0.999583, 0.028876, 0, 0.999583, 0.028876, 0, 0.999583, 0.028876, 0, 0.999583, 0.028876, 0, 0.999583, 0.028876, 0, 0.999583, 0.028876, 0, 0.999598, 0.028341, 0, 0.999598, 0.028341, 0, 0.999598, 0.028341, 0, 0.999598, 0.028341, 0, 0.999598, 0.028341, 0, 0.999598, 0.028341, 0, 0.999618, 0.027636, 0, 0.999618, 0.027636, 0, 0.999618, 0.027636, 0, 0.999618, 0.027636, 0, 0.999618, 0.027636, 0, 0.999618, 0.027636, 0, 0.999642, 0.026759, 0, 0.999642, 0.026759, 0, 0.999642, 0.026759, 0, 0.999642, 0.026759, 0, 0.999642, 0.026759, 0, 0.999642, 0.026759, 0, 0.999669, 0.025712, 0, 0.999669, 0.025712, 0, 0.999669, 0.025712, 0, 0.999669, 0.025712, 0, 0.999669, 0.025712, 0, 0.999669, 0.025712, 0, 0.999700, 0.024495, 0, 0.999700, 0.024495, 0, 0.999700, 0.024495, 0, 0.999700, 0.024495, 0, 0.999700, 0.024495, 0, 0.999700, 0.024495, 0, 0.999723, 0.023557, 0, 0.999723, 0.023557, 0, 0.999723, 0.023557, 0, 0.999723, 0.023557, 0, 0.999723, 0.023557, 0, 0.999723, 0.023557, 0, 0.999742, 0.022721, 0, 0.999742, 0.022721, 0, 0.999742, 0.022721, 0, 0.999742, 0.022721, 0, 0.999742, 0.022721, 0, 0.999742, 0.022721, 0, 0.999758, 0.021985, 0, 0.999758, 0.021985, 0, 0.999758, 0.021985, 0, 0.999758, 0.021985, 0, 0.999758, 0.021985, 0, 0.999758, 0.021985, 0, 0.999772, 0.021351, 0, 0.999772, 0.021351, 0, 0.999772, 0.021351, 0, 0.999772, 0.021351, 0, 0.999772, 0.021351, 0, 0.999772, 0.021351, 0, 0.999784, 0.020771, 0, 0.999784, 0.020771, 0, 0.999784, 0.020771, 0, 0.999784, 0.020771, 0, 0.999784, 0.020771, 0, 0.999784, 0.020771, 0, 0.999796, 0.020200, 0, 0.999796, 0.020200, 0, 0.999796, 0.020200, 0, 0.999796, 0.020200, 0, 0.999796, 0.020200, 0, 0.999796, 0.020200, 0, 0.999807, 0.019638, 0, 0.999807, 0.019638, 0, 0.999807, 0.019638, 0, 0.999807, 0.019638, 0, 0.999807, 0.019638, 0, 0.999807, 0.019638, 0, 0.999818, 0.019086, 0, 0.999818, 0.019086, 0, 0.999818, 0.019086, 0, 0.999818, 0.019086, 0, 0.999818, 0.019086, 0, 0.999818, 0.019086, 0, 0.999826, 0.018635, 0, 0.999826, 0.018635, 0, 0.999826, 0.018635, 0, 0.999826, 0.018635, 0, 0.999826, 0.018635, 0, 0.999826, 0.018635, 0, 0.999840, 0.017909, 0, 0.999840, 0.017909, 0, 0.999840, 0.017909, 0, 0.999840, 0.017909, 0, 0.999840, 0.017909, 0, 0.999840, 0.017909, 0, 0.999854, 0.017069, 0, 0.999854, 0.017069, 0, 0.999854, 0.017069, 0, 0.999854, 0.017069, 0, 0.999854, 0.017069, 0, 0.999854, 0.017069, 0, 0.999870, 0.016115, 0, 0.999870, 0.016115, 0, 0.999870, 0.016115, 0, 0.999870, 0.016115, 0, 0.999870, 0.016115, 0, 0.999870, 0.016115, 0, 0.999887, 0.015047, 0, 0.999887, 0.015047, 0, 0.999887, 0.015047, 0, 0.999887, 0.015047, 0, 0.999887, 0.015047, 0, 0.999887, 0.015047, 0, 0.999905, 0.013816, 0, 0.999905, 0.013816, 0, 0.999905, 0.013816, 0, 0.999905, 0.013816, 0, 0.999905, 0.013816, 0, 0.999905, 0.013816, 0, 0.999924, 0.012371, 0, 0.999924, 0.012371, 0, 0.999924, 0.012371, 0, 0.999924, 0.012371, 0, 0.999924, 0.012371, 0, 0.999924, 0.012371, 0, 0.999943, 0.010712, 0, 0.999943, 0.010712, 0, 0.999943, 0.010712, 0, 0.999943, 0.010712, 0, 0.999943, 0.010712, 0, 0.999943, 0.010712, 0, 0.999961, 0.008838, 0, 0.999961, 0.008838, 0, 0.999961, 0.008838, 0, 0.999961, 0.008838, 0, 0.999961, 0.008838, 0, 0.999961, 0.008838, 0, 0.999978, 0.006703, 0, 0.999978, 0.006703, 0, 0.999978, 0.006703, 0, 0.999978, 0.006703, 0, 0.999978, 0.006703, 0, 0.999978, 0.006703, 0, 0.999988, 0.004804, 0, 0.999988, 0.004804, 0, 0.999988, 0.004804, 0, 0.999988, 0.004804, 0, 0.999988, 0.004804, 0, 0.999988, 0.004804, 0, 0.999995, 0.003137, 0, 0.999995, 0.003137, 0, 0.999995, 0.003137, 0, 0.999995, 0.003137, 0, 0.999995, 0.003137, 0, 0.999995, 0.003137, 0, 0.999999, 0.001702, 0, 0.999999, 0.001702, 0, 0.999999, 0.001702, 0, 0.999999, 0.001702, 0, 0.999999, 0.001702, 0, 0.999999, 0.001702, 0, 1.000000, 0.000546, 0, 1.000000, 0.000546, 0, 1.000000, 0.000546, 0, 1.000000, 0.000546, 0, 1.000000, 0.000546, 0, 1.000000, 0.000546, 0, 1, -0.000283, 0, 1, -0.000283, 0, 1, -0.000283, 0, 1, -0.000283, 0, 1, -0.000283, 0, 1, -0.000283, 0, 1.000000, -0.000783, 0, 1.000000, -0.000783, 0, 1.000000, -0.000783, 0, 1.000000, -0.000783, 0, 1.000000, -0.000783, 0, 1.000000, -0.000783, 0, 1.000000, -0.000953, 0, 1.000000, -0.000953, 0, 1.000000, -0.000953, 0, 1.000000, -0.000953, 0, 1.000000, -0.000953, 0, 1.000000, -0.000953, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000836, 0, 1.000000, -0.000836, 0, 1.000000, -0.000836, 0, 1.000000, -0.000836, 0, 1.000000, -0.000836, 0, 1.000000, -0.000836, 0, 1.000000, -0.000688, 0, 1.000000, -0.000688, 0, 1.000000, -0.000688, 0, 1.000000, -0.000688, 0, 1.000000, -0.000688, 0, 1.000000, -0.000688, 0, 1.000000, -0.000479, 0, 1.000000, -0.000479, 0, 1.000000, -0.000479, 0, 1.000000, -0.000479, 0, 1.000000, -0.000479, 0, 1.000000, -0.000479, 0, 1, -0.000210, 0, 1, -0.000210, 0, 1, -0.000210, 0, 1, -0.000210, 0, 1, -0.000210, 0, 1, -0.000210, 0, 1, 0.000120, 0, 1, 0.000120, 0, 1, 0.000120, 0, 1, 0.000120, 0, 1, 0.000120, 0, 1, 0.000120, 0, 1.000000, 0.000511, 0, 1.000000, 0.000511, 0, 1.000000, 0.000511, 0, 1.000000, 0.000511, 0, 1.000000, 0.000511, 0, 1.000000, 0.000511, 0, 1.000000, 0.000964, 0, 1.000000, 0.000964, 0, 1.000000, 0.000964, 0, 1.000000, 0.000964, 0, 1.000000, 0.000964, 0, 1.000000, 0.000964, 0, 0.999977, 0.006747, 0, 0.999977, 0.006747, 0, 0.999977, 0.006747, 0, 0.999977, 0.006747, 0, 0.999977, 0.006747, 0, 0.999977, 0.006747, 0, 0.999982, 0.006060, 0, 0.999982, 0.006060, 0, 0.999982, 0.006060, 0, 0.999982, 0.006060, 0, 0.999982, 0.006060, 0, 0.999982, 0.006060, 0, 0.999985, 0.005465, 0, 0.999985, 0.005465, 0, 0.999985, 0.005465, 0, 0.999985, 0.005465, 0, 0.999985, 0.005465, 0, 0.999985, 0.005465, 0, 0.999988, 0.004962, 0, 0.999988, 0.004962, 0, 0.999988, 0.004962, 0, 0.999988, 0.004962, 0, 0.999988, 0.004962, 0, 0.999988, 0.004962, 0, 0.999990, 0.004551, 0, 0.999990, 0.004551, 0, 0.999990, 0.004551, 0, 0.999990, 0.004551, 0, 0.999990, 0.004551, 0, 0.999990, 0.004551, 0, 0.999991, 0.004230, 0, 0.999991, 0.004230, 0, 0.999991, 0.004230, 0, 0.999991, 0.004230, 0, 0.999991, 0.004230, 0, 0.999991, 0.004230, 0, 0.999992, 0.004000, 0, 0.999992, 0.004000, 0, 0.999992, 0.004000, 0, 0.999992, 0.004000, 0, 0.999992, 0.004000, 0, 0.999992, 0.004000, 0, 0.999993, 0.003860, 0, 0.999993, 0.003860, 0, 0.999993, 0.003860, 0, 0.999993, 0.003860, 0, 0.999993, 0.003860, 0, 0.999993, 0.003860, 0, 0.999993, 0.003810, 0, 0.999993, 0.003810, 0, 0.999993, 0.003810, 0, 0.999993, 0.003810, 0, 0.999993, 0.003810, 0, 0.999993, 0.003810, 0, 0.999992, 0.003882, 0, 0.999992, 0.003882, 0, 0.999992, 0.003882, 0, 0.999992, 0.003882, 0, 0.999992, 0.003882, 0, 0.999992, 0.003882, 0, 0.999992, 0.004114, 0, 0.999992, 0.004114, 0, 0.999992, 0.004114, 0, 0.999992, 0.004114, 0, 0.999992, 0.004114, 0, 0.999992, 0.004114, 0, 0.999990, 0.004504, 0, 0.999990, 0.004504, 0, 0.999990, 0.004504, 0, 0.999990, 0.004504, 0, 0.999990, 0.004504, 0, 0.999990, 0.004504, 0, 0.999987, 0.005052, 0, 0.999987, 0.005052, 0, 0.999987, 0.005052, 0, 0.999987, 0.005052, 0, 0.999987, 0.005052, 0, 0.999987, 0.005052, 0, 0.999983, 0.005745, 0, 0.999983, 0.005745, 0, 0.999983, 0.005745, 0, 0.999983, 0.005745, 0, 0.999983, 0.005745, 0, 0.999983, 0.005745, 0, 0.999978, 0.006568, 0, 0.999978, 0.006568, 0, 0.999978, 0.006568, 0, 0.999978, 0.006568, 0, 0.999978, 0.006568, 0, 0.999978, 0.006568, 0, 0.999972, 0.007522, 0, 0.999972, 0.007522, 0, 0.999972, 0.007522, 0, 0.999972, 0.007522, 0, 0.999972, 0.007522, 0, 0.999972, 0.007522, 0, 0.999963, 0.008607, 0, 0.999963, 0.008607, 0, 0.999963, 0.008607, 0, 0.999963, 0.008607, 0, 0.999963, 0.008607, 0, 0.999963, 0.008607, 0, 0.999954, 0.009609, 0, 0.999954, 0.009609, 0, 0.999954, 0.009609, 0, 0.999954, 0.009609, 0, 0.999954, 0.009609, 0, 0.999954, 0.009609, 0, 0.999945, 0.010490, 0, 0.999945, 0.010490, 0, 0.999945, 0.010490, 0, 0.999945, 0.010490, 0, 0.999945, 0.010490, 0, 0.999945, 0.010490, 0, 0.999937, 0.011249, 0, 0.999937, 0.011249, 0, 0.999937, 0.011249, 0, 0.999937, 0.011249, 0, 0.999937, 0.011249, 0, 0.999937, 0.011249, 0, 0.999929, 0.011887, 0, 0.999929, 0.011887, 0, 0.999929, 0.011887, 0, 0.999929, 0.011887, 0, 0.999929, 0.011887, 0, 0.999929, 0.011887, 0, 0.999922, 0.012463, 0, 0.999922, 0.012463, 0, 0.999922, 0.012463, 0, 0.999922, 0.012463, 0, 0.999922, 0.012463, 0, 0.999922, 0.012463, 0, 0.999915, 0.013034, 0, 0.999915, 0.013034, 0, 0.999915, 0.013034, 0, 0.999915, 0.013034, 0, 0.999915, 0.013034, 0, 0.999915, 0.013034, 0, 0.999907, 0.013602, 0, 0.999907, 0.013602, 0, 0.999907, 0.013602, 0, 0.999907, 0.013602, 0, 0.999907, 0.013602, 0, 0.999907, 0.013602, 0, 0.999900, 0.014168, 0, 0.999900, 0.014168, 0, 0.999900, 0.014168, 0, 0.999900, 0.014168, 0, 0.999900, 0.014168, 0, 0.999900, 0.014168, 0, 0.999893, 0.014642, 0, 0.999893, 0.014642, 0, 0.999893, 0.014642, 0, 0.999893, 0.014642, 0, 0.999893, 0.014642, 0, 0.999893, 0.014642, 0, 0.999883, 0.015303, 0, 0.999883, 0.015303, 0, 0.999883, 0.015303, 0, 0.999883, 0.015303, 0, 0.999883, 0.015303, 0, 0.999883, 0.015303, 0, 0.999870, 0.016108, 0, 0.999870, 0.016108, 0, 0.999870, 0.016108, 0, 0.999870, 0.016108, 0, 0.999870, 0.016108, 0, 0.999870, 0.016108, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999855, 0.017058, 0, 0.999835, 0.018152, 0, 0.999835, 0.018152, 0, 0.999835, 0.018152, 0, 0.999835, 0.018152, 0, 0.999835, 0.018152, 0, 0.999835, 0.018152, 0, 0.999811, 0.019424, 0, 0.999811, 0.019424, 0, 0.999811, 0.019424, 0, 0.999811, 0.019424, 0, 0.999811, 0.019424, 0, 0.999811, 0.019424, 0, 0.999781, 0.020907, 0, 0.999781, 0.020907, 0, 0.999781, 0.020907, 0, 0.999781, 0.020907, 0, 0.999781, 0.020907, 0, 0.999781, 0.020907, 0, 0.999745, 0.022602, 0, 0.999745, 0.022602, 0, 0.999745, 0.022602, 0, 0.999745, 0.022602, 0, 0.999745, 0.022602, 0, 0.999745, 0.022602, 0, 0.999700, 0.024506, 0, 0.999700, 0.024506, 0, 0.999700, 0.024506, 0, 0.999700, 0.024506, 0, 0.999700, 0.024506, 0, 0.999700, 0.024506, 0, 0.999641, 0.026791, 0, 0.999641, 0.026791, 0, 0.999641, 0.026791, 0, 0.999641, 0.026791, 0, 0.999641, 0.026791, 0, 0.999641, 0.026791, 0, 0.999586, 0.028789, 0, 0.999586, 0.028789, 0, 0.999586, 0.028789, 0, 0.999586, 0.028789, 0, 0.999586, 0.028789, 0, 0.999586, 0.028789, 0, 0.999535, 0.030495, 0, 0.999535, 0.030495, 0, 0.999535, 0.030495, 0, 0.999535, 0.030495, 0, 0.999535, 0.030495, 0, 0.999535, 0.030495, 0, 0.999491, 0.031910, 0, 0.999491, 0.031910, 0, 0.999491, 0.031910, 0, 0.999491, 0.031910, 0, 0.999491, 0.031910, 0, 0.999491, 0.031910, 0, 0.999455, 0.033006, 0, 0.999455, 0.033006, 0, 0.999455, 0.033006, 0, 0.999455, 0.033006, 0, 0.999455, 0.033006, 0, 0.999455, 0.033006, 0, 0.999430, 0.033756, 0, 0.999430, 0.033756, 0, 0.999430, 0.033756, 0, 0.999430, 0.033756, 0, 0.999430, 0.033756, 0, 0.999430, 0.033756, 0, 0.999416, 0.034163, 0, 0.999416, 0.034163, 0, 0.999416, 0.034163, 0, 0.999416, 0.034163, 0, 0.999416, 0.034163, 0, 0.999416, 0.034163, 0, 0.999414, 0.034228, 0, 0.999414, 0.034228, 0, 0.999414, 0.034228, 0, 0.999414, 0.034228, 0, 0.999414, 0.034228, 0, 0.999414, 0.034228, 0, 0.999417, 0.034132, 0, 0.999417, 0.034132, 0, 0.999417, 0.034132, 0, 0.999417, 0.034132, 0, 0.999417, 0.034132, 0, 0.999417, 0.034132, 0, 0.999424, 0.033950, 0, 0.999424, 0.033950, 0, 0.999424, 0.033950, 0, 0.999424, 0.033950, 0, 0.999424, 0.033950, 0, 0.999424, 0.033950, 0, 0.999433, 0.033681, 0, 0.999433, 0.033681, 0, 0.999433, 0.033681, 0, 0.999433, 0.033681, 0, 0.999433, 0.033681, 0, 0.999433, 0.033681, 0, 0.999445, 0.033326, 0, 0.999445, 0.033326, 0, 0.999445, 0.033326, 0, 0.999445, 0.033326, 0, 0.999445, 0.033326, 0, 0.999445, 0.033326, 0, 0.999459, 0.032884, 0, 0.999459, 0.032884, 0, 0.999459, 0.032884, 0, 0.999459, 0.032884, 0, 0.999459, 0.032884, 0, 0.999459, 0.032884, 0, 0.999476, 0.032358, 0, 0.999476, 0.032358, 0, 0.999476, 0.032358, 0, 0.999476, 0.032358, 0, 0.999476, 0.032358, 0, 0.999476, 0.032358, 0, 0.999496, 0.031746, 0, 0.999496, 0.031746, 0, 0.999496, 0.031746, 0, 0.999496, 0.031746, 0, 0.999496, 0.031746, 0, 0.999496, 0.031746, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999673, 0.025554, 0, 0.999673, 0.025554, 0, 0.999673, 0.025554, 0, 0.999673, 0.025554, 0, 0.999673, 0.025554, 0, 0.999673, 0.025554, 0, 0.999656, 0.026225, 0, 0.999656, 0.026225, 0, 0.999656, 0.026225, 0, 0.999656, 0.026225, 0, 0.999656, 0.026225, 0, 0.999656, 0.026225, 0, 0.999641, 0.026812, 0, 0.999641, 0.026812, 0, 0.999641, 0.026812, 0, 0.999641, 0.026812, 0, 0.999641, 0.026812, 0, 0.999641, 0.026812, 0, 0.999627, 0.027315, 0, 0.999627, 0.027315, 0, 0.999627, 0.027315, 0, 0.999627, 0.027315, 0, 0.999627, 0.027315, 0, 0.999627, 0.027315, 0, 0.999615, 0.027734, 0, 0.999615, 0.027734, 0, 0.999615, 0.027734, 0, 0.999615, 0.027734, 0, 0.999615, 0.027734, 0, 0.999615, 0.027734, 0, 0.999606, 0.028069, 0, 0.999606, 0.028069, 0, 0.999606, 0.028069, 0, 0.999606, 0.028069, 0, 0.999606, 0.028069, 0, 0.999606, 0.028069, 0, 0.999599, 0.028319, 0, 0.999599, 0.028319, 0, 0.999599, 0.028319, 0, 0.999599, 0.028319, 0, 0.999599, 0.028319, 0, 0.999599, 0.028319, 0, 0.999594, 0.028484, 0, 0.999594, 0.028484, 0, 0.999594, 0.028484, 0, 0.999594, 0.028484, 0, 0.999594, 0.028484, 0, 0.999594, 0.028484, 0, 0.999592, 0.028564, 0, 0.999592, 0.028564, 0, 0.999592, 0.028564, 0, 0.999592, 0.028564, 0, 0.999592, 0.028564, 0, 0.999592, 0.028564, 0, 0.999593, 0.028538, 0, 0.999593, 0.028538, 0, 0.999593, 0.028538, 0, 0.999593, 0.028538, 0, 0.999593, 0.028538, 0, 0.999593, 0.028538, 0, 0.999598, 0.028349, 0, 0.999598, 0.028349, 0, 0.999598, 0.028349, 0, 0.999598, 0.028349, 0, 0.999598, 0.028349, 0, 0.999598, 0.028349, 0, 0.999608, 0.027994, 0, 0.999608, 0.027994, 0, 0.999608, 0.027994, 0, 0.999608, 0.027994, 0, 0.999608, 0.027994, 0, 0.999608, 0.027994, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999641, 0.026790, 0, 0.999641, 0.026790, 0, 0.999641, 0.026790, 0, 0.999641, 0.026790, 0, 0.999641, 0.026790, 0, 0.999641, 0.026790, 0, 0.999664, 0.025939, 0, 0.999664, 0.025939, 0, 0.999664, 0.025939, 0, 0.999664, 0.025939, 0, 0.999664, 0.025939, 0, 0.999664, 0.025939, 0, 0.999689, 0.024924, 0, 0.999689, 0.024924, 0, 0.999689, 0.024924, 0, 0.999689, 0.024924, 0, 0.999689, 0.024924, 0, 0.999689, 0.024924, 0, 0.999718, 0.023744, 0, 0.999718, 0.023744, 0, 0.999718, 0.023744, 0, 0.999718, 0.023744, 0, 0.999718, 0.023744, 0, 0.999718, 0.023744, 0, 0.999739, 0.022835, 0, 0.999739, 0.022835, 0, 0.999739, 0.022835, 0, 0.999739, 0.022835, 0, 0.999739, 0.022835, 0, 0.999739, 0.022835, 0, 0.999757, 0.022025, 0, 0.999757, 0.022025, 0, 0.999757, 0.022025, 0, 0.999757, 0.022025, 0, 0.999757, 0.022025, 0, 0.999757, 0.022025, 0, 0.999773, 0.021312, 0, 0.999773, 0.021312, 0, 0.999773, 0.021312, 0, 0.999773, 0.021312, 0, 0.999773, 0.021312, 0, 0.999773, 0.021312, 0, 0.999786, 0.020697, 0, 0.999786, 0.020697, 0, 0.999786, 0.020697, 0, 0.999786, 0.020697, 0, 0.999786, 0.020697, 0, 0.999786, 0.020697, 0, 0.999797, 0.020136, 0, 0.999797, 0.020136, 0, 0.999797, 0.020136, 0, 0.999797, 0.020136, 0, 0.999797, 0.020136, 0, 0.999797, 0.020136, 0, 0.999808, 0.019583, 0, 0.999808, 0.019583, 0, 0.999808, 0.019583, 0, 0.999808, 0.019583, 0, 0.999808, 0.019583, 0, 0.999808, 0.019583, 0, 0.999819, 0.019039, 0, 0.999819, 0.019039, 0, 0.999819, 0.019039, 0, 0.999819, 0.019039, 0, 0.999819, 0.019039, 0, 0.999819, 0.019039, 0, 0.999829, 0.018505, 0, 0.999829, 0.018505, 0, 0.999829, 0.018505, 0, 0.999829, 0.018505, 0, 0.999829, 0.018505, 0, 0.999829, 0.018505, 0, 0.999837, 0.018069, 0, 0.999837, 0.018069, 0, 0.999837, 0.018069, 0, 0.999837, 0.018069, 0, 0.999837, 0.018069, 0, 0.999837, 0.018069, 0, 0.999849, 0.017366, 0, 0.999849, 0.017366, 0, 0.999849, 0.017366, 0, 0.999849, 0.017366, 0, 0.999849, 0.017366, 0, 0.999849, 0.017366, 0, 0.999863, 0.016553, 0, 0.999863, 0.016553, 0, 0.999863, 0.016553, 0, 0.999863, 0.016553, 0, 0.999863, 0.016553, 0, 0.999863, 0.016553, 0, 0.999878, 0.015629, 0, 0.999878, 0.015629, 0, 0.999878, 0.015629, 0, 0.999878, 0.015629, 0, 0.999878, 0.015629, 0, 0.999878, 0.015629, 0, 0.999894, 0.014594, 0, 0.999894, 0.014594, 0, 0.999894, 0.014594, 0, 0.999894, 0.014594, 0, 0.999894, 0.014594, 0, 0.999894, 0.014594, 0, 0.999910, 0.013400, 0, 0.999910, 0.013400, 0, 0.999910, 0.013400, 0, 0.999910, 0.013400, 0, 0.999910, 0.013400, 0, 0.999910, 0.013400, 0, 0.999928, 0.011999, 0, 0.999928, 0.011999, 0, 0.999928, 0.011999, 0, 0.999928, 0.011999, 0, 0.999928, 0.011999, 0, 0.999928, 0.011999, 0, 0.999946, 0.010390, 0, 0.999946, 0.010390, 0, 0.999946, 0.010390, 0, 0.999946, 0.010390, 0, 0.999946, 0.010390, 0, 0.999946, 0.010390, 0, 0.999963, 0.008572, 0, 0.999963, 0.008572, 0, 0.999963, 0.008572, 0, 0.999963, 0.008572, 0, 0.999963, 0.008572, 0, 0.999963, 0.008572, 0, 0.999979, 0.006502, 0, 0.999979, 0.006502, 0, 0.999979, 0.006502, 0, 0.999979, 0.006502, 0, 0.999979, 0.006502, 0, 0.999979, 0.006502, 0, 0.999989, 0.004659, 0, 0.999989, 0.004659, 0, 0.999989, 0.004659, 0, 0.999989, 0.004659, 0, 0.999989, 0.004659, 0, 0.999989, 0.004659, 0, 0.999995, 0.003042, 0, 0.999995, 0.003042, 0, 0.999995, 0.003042, 0, 0.999995, 0.003042, 0, 0.999995, 0.003042, 0, 0.999995, 0.003042, 0, 0.999999, 0.001651, 0, 0.999999, 0.001651, 0, 0.999999, 0.001651, 0, 0.999999, 0.001651, 0, 0.999999, 0.001651, 0, 0.999999, 0.001651, 0, 1.000000, 0.000529, 0, 1.000000, 0.000529, 0, 1.000000, 0.000529, 0, 1.000000, 0.000529, 0, 1.000000, 0.000529, 0, 1.000000, 0.000529, 0, 1, -0.000274, 0, 1, -0.000274, 0, 1, -0.000274, 0, 1, -0.000274, 0, 1, -0.000274, 0, 1, -0.000274, 0, 1.000000, -0.000759, 0, 1.000000, -0.000759, 0, 1.000000, -0.000759, 0, 1.000000, -0.000759, 0, 1.000000, -0.000759, 0, 1.000000, -0.000759, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000924, 0, 1.000000, -0.000896, 0, 1.000000, -0.000896, 0, 1.000000, -0.000896, 0, 1.000000, -0.000896, 0, 1.000000, -0.000896, 0, 1.000000, -0.000896, 0, 1.000000, -0.000810, 0, 1.000000, -0.000810, 0, 1.000000, -0.000810, 0, 1.000000, -0.000810, 0, 1.000000, -0.000810, 0, 1.000000, -0.000810, 0, 1.000000, -0.000666, 0, 1.000000, -0.000666, 0, 1.000000, -0.000666, 0, 1.000000, -0.000666, 0, 1.000000, -0.000666, 0, 1.000000, -0.000666, 0, 1.000000, -0.000464, 0, 1.000000, -0.000464, 0, 1.000000, -0.000464, 0, 1.000000, -0.000464, 0, 1.000000, -0.000464, 0, 1.000000, -0.000464, 0, 1, -0.000203, 0, 1, -0.000203, 0, 1, -0.000203, 0, 1, -0.000203, 0, 1, -0.000203, 0, 1, -0.000203, 0, 1, 0.000116, 0, 1, 0.000116, 0, 1, 0.000116, 0, 1, 0.000116, 0, 1, 0.000116, 0, 1, 0.000116, 0, 1.000000, 0.000496, 0, 1.000000, 0.000496, 0, 1.000000, 0.000496, 0, 1.000000, 0.000496, 0, 1.000000, 0.000496, 0, 1.000000, 0.000496, 0, 1.000000, 0.000934, 0, 1.000000, 0.000934, 0, 1.000000, 0.000934, 0, 1.000000, 0.000934, 0, 1.000000, 0.000934, 0, 1.000000, 0.000934, 0, 0.999979, 0.006539, 0, 0.999979, 0.006539, 0, 0.999979, 0.006539, 0, 0.999979, 0.006539, 0, 0.999979, 0.006539, 0, 0.999979, 0.006539, 0, 0.999983, 0.005874, 0, 0.999983, 0.005874, 0, 0.999983, 0.005874, 0, 0.999983, 0.005874, 0, 0.999983, 0.005874, 0, 0.999983, 0.005874, 0, 0.999986, 0.005297, 0, 0.999986, 0.005297, 0, 0.999986, 0.005297, 0, 0.999986, 0.005297, 0, 0.999986, 0.005297, 0, 0.999986, 0.005297, 0, 0.999988, 0.004810, 0, 0.999988, 0.004810, 0, 0.999988, 0.004810, 0, 0.999988, 0.004810, 0, 0.999988, 0.004810, 0, 0.999988, 0.004810, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999990, 0.004411, 0, 0.999992, 0.004101, 0, 0.999992, 0.004101, 0, 0.999992, 0.004101, 0, 0.999992, 0.004101, 0, 0.999992, 0.004101, 0, 0.999992, 0.004101, 0, 0.999992, 0.003878, 0, 0.999992, 0.003878, 0, 0.999992, 0.003878, 0, 0.999992, 0.003878, 0, 0.999992, 0.003878, 0, 0.999992, 0.003878, 0, 0.999993, 0.003742, 0, 0.999993, 0.003742, 0, 0.999993, 0.003742, 0, 0.999993, 0.003742, 0, 0.999993, 0.003742, 0, 0.999993, 0.003742, 0, 0.999993, 0.003694, 0, 0.999993, 0.003694, 0, 0.999993, 0.003694, 0, 0.999993, 0.003694, 0, 0.999993, 0.003694, 0, 0.999993, 0.003694, 0, 0.999993, 0.003764, 0, 0.999993, 0.003764, 0, 0.999993, 0.003764, 0, 0.999993, 0.003764, 0, 0.999993, 0.003764, 0, 0.999993, 0.003764, 0, 0.999992, 0.003989, 0, 0.999992, 0.003989, 0, 0.999992, 0.003989, 0, 0.999992, 0.003989, 0, 0.999992, 0.003989, 0, 0.999992, 0.003989, 0, 0.999990, 0.004367, 0, 0.999990, 0.004367, 0, 0.999990, 0.004367, 0, 0.999990, 0.004367, 0, 0.999990, 0.004367, 0, 0.999990, 0.004367, 0, 0.999988, 0.004899, 0, 0.999988, 0.004899, 0, 0.999988, 0.004899, 0, 0.999988, 0.004899, 0, 0.999988, 0.004899, 0, 0.999988, 0.004899, 0, 0.999985, 0.005571, 0, 0.999985, 0.005571, 0, 0.999985, 0.005571, 0, 0.999985, 0.005571, 0, 0.999985, 0.005571, 0, 0.999985, 0.005571, 0, 0.999980, 0.006369, 0, 0.999980, 0.006369, 0, 0.999980, 0.006369, 0, 0.999980, 0.006369, 0, 0.999980, 0.006369, 0, 0.999980, 0.006369, 0, 0.999973, 0.007294, 0, 0.999973, 0.007294, 0, 0.999973, 0.007294, 0, 0.999973, 0.007294, 0, 0.999973, 0.007294, 0, 0.999973, 0.007294, 0, 0.999965, 0.008347, 0, 0.999965, 0.008347, 0, 0.999965, 0.008347, 0, 0.999965, 0.008347, 0, 0.999965, 0.008347, 0, 0.999965, 0.008347, 0, 0.999957, 0.009319, 0, 0.999957, 0.009319, 0, 0.999957, 0.009319, 0, 0.999957, 0.009319, 0, 0.999957, 0.009319, 0, 0.999957, 0.009319, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999948, 0.010172, 0, 0.999941, 0.010908, 0, 0.999941, 0.010908, 0, 0.999941, 0.010908, 0, 0.999941, 0.010908, 0, 0.999941, 0.010908, 0, 0.999941, 0.010908, 0, 0.999934, 0.011527, 0, 0.999934, 0.011527, 0, 0.999934, 0.011527, 0, 0.999934, 0.011527, 0, 0.999934, 0.011527, 0, 0.999934, 0.011527, 0, 0.999927, 0.012085, 0, 0.999927, 0.012085, 0, 0.999927, 0.012085, 0, 0.999927, 0.012085, 0, 0.999927, 0.012085, 0, 0.999927, 0.012085, 0, 0.999920, 0.012639, 0, 0.999920, 0.012639, 0, 0.999920, 0.012639, 0, 0.999920, 0.012639, 0, 0.999920, 0.012639, 0, 0.999920, 0.012639, 0, 0.999913, 0.013189, 0, 0.999913, 0.013189, 0, 0.999913, 0.013189, 0, 0.999913, 0.013189, 0, 0.999913, 0.013189, 0, 0.999913, 0.013189, 0, 0.999906, 0.013736, 0, 0.999906, 0.013736, 0, 0.999906, 0.013736, 0, 0.999906, 0.013736, 0, 0.999906, 0.013736, 0, 0.999906, 0.013736, 0, 0.999899, 0.014195, 0, 0.999899, 0.014195, 0, 0.999899, 0.014195, 0, 0.999899, 0.014195, 0, 0.999899, 0.014195, 0, 0.999899, 0.014195, 0, 0.999890, 0.014835, 0, 0.999890, 0.014835, 0, 0.999890, 0.014835, 0, 0.999890, 0.014835, 0, 0.999890, 0.014835, 0, 0.999890, 0.014835, 0, 0.999878, 0.015615, 0, 0.999878, 0.015615, 0, 0.999878, 0.015615, 0, 0.999878, 0.015615, 0, 0.999878, 0.015615, 0, 0.999878, 0.015615, 0, 0.999863, 0.016534, 0, 0.999863, 0.016534, 0, 0.999863, 0.016534, 0, 0.999863, 0.016534, 0, 0.999863, 0.016534, 0, 0.999863, 0.016534, 0, 0.999845, 0.017594, 0, 0.999845, 0.017594, 0, 0.999845, 0.017594, 0, 0.999845, 0.017594, 0, 0.999845, 0.017594, 0, 0.999845, 0.017594, 0, 0.999823, 0.018826, 0, 0.999823, 0.018826, 0, 0.999823, 0.018826, 0, 0.999823, 0.018826, 0, 0.999823, 0.018826, 0, 0.999823, 0.018826, 0, 0.999795, 0.020263, 0, 0.999795, 0.020263, 0, 0.999795, 0.020263, 0, 0.999795, 0.020263, 0, 0.999795, 0.020263, 0, 0.999795, 0.020263, 0, 0.999760, 0.021905, 0, 0.999760, 0.021905, 0, 0.999760, 0.021905, 0, 0.999760, 0.021905, 0, 0.999760, 0.021905, 0, 0.999760, 0.021905, 0, 0.999718, 0.023751, 0, 0.999718, 0.023751, 0, 0.999718, 0.023751, 0, 0.999718, 0.023751, 0, 0.999718, 0.023751, 0, 0.999718, 0.023751, 0, 0.999663, 0.025966, 0, 0.999663, 0.025966, 0, 0.999663, 0.025966, 0, 0.999663, 0.025966, 0, 0.999663, 0.025966, 0, 0.999663, 0.025966, 0, 0.999611, 0.027903, 0, 0.999611, 0.027903, 0, 0.999611, 0.027903, 0, 0.999611, 0.027903, 0, 0.999611, 0.027903, 0, 0.999611, 0.027903, 0, 0.999563, 0.029558, 0, 0.999563, 0.029558, 0, 0.999563, 0.029558, 0, 0.999563, 0.029558, 0, 0.999563, 0.029558, 0, 0.999563, 0.029558, 0, 0.999522, 0.030931, 0, 0.999522, 0.030931, 0, 0.999522, 0.030931, 0, 0.999522, 0.030931, 0, 0.999522, 0.030931, 0, 0.999522, 0.030931, 0, 0.999488, 0.031995, 0, 0.999488, 0.031995, 0, 0.999488, 0.031995, 0, 0.999488, 0.031995, 0, 0.999488, 0.031995, 0, 0.999488, 0.031995, 0, 0.999464, 0.032724, 0, 0.999464, 0.032724, 0, 0.999464, 0.032724, 0, 0.999464, 0.032724, 0, 0.999464, 0.032724, 0, 0.999464, 0.032724, 0, 0.999451, 0.033121, 0, 0.999451, 0.033121, 0, 0.999451, 0.033121, 0, 0.999451, 0.033121, 0, 0.999451, 0.033121, 0, 0.999451, 0.033121, 0, 0.999449, 0.033187, 0, 0.999449, 0.033187, 0, 0.999449, 0.033187, 0, 0.999449, 0.033187, 0, 0.999449, 0.033187, 0, 0.999449, 0.033187, 0, 0.999452, 0.033096, 0, 0.999452, 0.033096, 0, 0.999452, 0.033096, 0, 0.999452, 0.033096, 0, 0.999452, 0.033096, 0, 0.999452, 0.033096, 0, 0.999458, 0.032921, 0, 0.999458, 0.032921, 0, 0.999458, 0.032921, 0, 0.999458, 0.032921, 0, 0.999458, 0.032921, 0, 0.999458, 0.032921, 0, 0.999466, 0.032661, 0, 0.999466, 0.032661, 0, 0.999466, 0.032661, 0, 0.999466, 0.032661, 0, 0.999466, 0.032661, 0, 0.999466, 0.032661, 0, 0.999478, 0.032318, 0, 0.999478, 0.032318, 0, 0.999478, 0.032318, 0, 0.999478, 0.032318, 0, 0.999478, 0.032318, 0, 0.999478, 0.032318, 0, 0.999491, 0.031891, 0, 0.999491, 0.031891, 0, 0.999491, 0.031891, 0, 0.999491, 0.031891, 0, 0.999491, 0.031891, 0, 0.999491, 0.031891, 0, 0.999508, 0.031382, 0, 0.999508, 0.031382, 0, 0.999508, 0.031382, 0, 0.999508, 0.031382, 0, 0.999508, 0.031382, 0, 0.999508, 0.031382, 0, 0.999526, 0.030789, 0, 0.999526, 0.030789, 0, 0.999526, 0.030789, 0, 0.999526, 0.030789, 0, 0.999526, 0.030789, 0, 0.999526, 0.030789, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999505, 0.031465, 0, 0.999505, 0.031465, 0, 0.999505, 0.031465, 0, 0.999505, 0.031465, 0, 0.999505, 0.031465, 0, 0.999505, 0.031465, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999030, 0.044031, 0, 0.999030, 0.044031, 0, 0.999030, 0.044031, 0, 0.999030, 0.044031, 0, 0.999030, 0.044031, 0, 0.999030, 0.044031, 0, 0.999287, 0.037750, 0, 0.999287, 0.037750, 0, 0.999287, 0.037750, 0, 0.999287, 0.037750, 0, 0.999287, 0.037750, 0, 0.999287, 0.037750, 0, 0.999280, 0.037946, 0, 0.999280, 0.037946, 0, 0.999280, 0.037946, 0, 0.999280, 0.037946, 0, 0.999280, 0.037946, 0, 0.999280, 0.037946, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999651, 0.026412, 0, 0.999651, 0.026412, 0, 0.999651, 0.026412, 0, 0.999651, 0.026412, 0, 0.999651, 0.026412, 0, 0.999651, 0.026412, 0, 0.999617, 0.027683, 0, 0.999617, 0.027683, 0, 0.999617, 0.027683, 0, 0.999617, 0.027683, 0, 0.999617, 0.027683, 0, 0.999617, 0.027683, 0, 0.999587, 0.028738, 0, 0.999587, 0.028738, 0, 0.999587, 0.028738, 0, 0.999587, 0.028738, 0, 0.999587, 0.028738, 0, 0.999587, 0.028738, 0, 0.999563, 0.029577, 0, 0.999563, 0.029577, 0, 0.999563, 0.029577, 0, 0.999563, 0.029577, 0, 0.999563, 0.029577, 0, 0.999563, 0.029577, 0, 0.999544, 0.030209, 0, 0.999544, 0.030209, 0, 0.999544, 0.030209, 0, 0.999544, 0.030209, 0, 0.999544, 0.030209, 0, 0.999544, 0.030209, 0, 0.999530, 0.030644, 0, 0.999530, 0.030644, 0, 0.999530, 0.030644, 0, 0.999530, 0.030644, 0, 0.999530, 0.030644, 0, 0.999530, 0.030644, 0, 0.999523, 0.030883, 0, 0.999523, 0.030883, 0, 0.999523, 0.030883, 0, 0.999523, 0.030883, 0, 0.999523, 0.030883, 0, 0.999523, 0.030883, 0, 0.999522, 0.030926, 0, 0.999522, 0.030926, 0, 0.999522, 0.030926, 0, 0.999522, 0.030926, 0, 0.999522, 0.030926, 0, 0.999522, 0.030926, 0, 0.999530, 0.030655, 0, 0.999530, 0.030655, 0, 0.999530, 0.030655, 0, 0.999530, 0.030655, 0, 0.999530, 0.030655, 0, 0.999530, 0.030655, 0, 0.999550, 0.029989, 0, 0.999550, 0.029989, 0, 0.999550, 0.029989, 0, 0.999550, 0.029989, 0, 0.999550, 0.029989, 0, 0.999550, 0.029989, 0, 0.999582, 0.028928, 0, 0.999582, 0.028928, 0, 0.999582, 0.028928, 0, 0.999582, 0.028928, 0, 0.999582, 0.028928, 0, 0.999582, 0.028928, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999623, 0.027475, 0, 0.999671, 0.025661, 0, 0.999671, 0.025661, 0, 0.999671, 0.025661, 0, 0.999671, 0.025661, 0, 0.999671, 0.025661, 0, 0.999671, 0.025661, 0, 0.999723, 0.023519, 0, 0.999723, 0.023519, 0, 0.999723, 0.023519, 0, 0.999723, 0.023519, 0, 0.999723, 0.023519, 0, 0.999723, 0.023519, 0, 0.999779, 0.021047, 0, 0.999779, 0.021047, 0, 0.999779, 0.021047, 0, 0.999779, 0.021047, 0, 0.999779, 0.021047, 0, 0.999779, 0.021047, 0, 0.999834, 0.018243, 0, 0.999834, 0.018243, 0, 0.999834, 0.018243, 0, 0.999834, 0.018243, 0, 0.999834, 0.018243, 0, 0.999834, 0.018243, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999835, 0.018173, 0, 0.999835, 0.018173, 0, 0.999835, 0.018173, 0, 0.999835, 0.018173, 0, 0.999835, 0.018173, 0, 0.999835, 0.018173, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999797, 0.020156, 0, 0.999761, 0.021869, 0, 0.999761, 0.021869, 0, 0.999761, 0.021869, 0, 0.999761, 0.021869, 0, 0.999761, 0.021869, 0, 0.999761, 0.021869, 0, 0.999728, 0.023315, 0, 0.999728, 0.023315, 0, 0.999728, 0.023315, 0, 0.999728, 0.023315, 0, 0.999728, 0.023315, 0, 0.999728, 0.023315, 0, 0.999700, 0.024494, 0, 0.999700, 0.024494, 0, 0.999700, 0.024494, 0, 0.999700, 0.024494, 0, 0.999700, 0.024494, 0, 0.999700, 0.024494, 0, 0.999677, 0.025414, 0, 0.999677, 0.025414, 0, 0.999677, 0.025414, 0, 0.999677, 0.025414, 0, 0.999677, 0.025414, 0, 0.999677, 0.025414, 0, 0.999660, 0.026084, 0, 0.999660, 0.026084, 0, 0.999660, 0.026084, 0, 0.999660, 0.026084, 0, 0.999660, 0.026084, 0, 0.999660, 0.026084, 0, 0.999649, 0.026503, 0, 0.999649, 0.026503, 0, 0.999649, 0.026503, 0, 0.999649, 0.026503, 0, 0.999649, 0.026503, 0, 0.999649, 0.026503, 0, 0.999644, 0.026671, 0, 0.999644, 0.026671, 0, 0.999644, 0.026671, 0, 0.999644, 0.026671, 0, 0.999644, 0.026671, 0, 0.999644, 0.026671, 0, 0.999647, 0.026562, 0, 0.999647, 0.026562, 0, 0.999647, 0.026562, 0, 0.999647, 0.026562, 0, 0.999647, 0.026562, 0, 0.999647, 0.026562, 0, 0.999659, 0.026131, 0, 0.999659, 0.026131, 0, 0.999659, 0.026131, 0, 0.999659, 0.026131, 0, 0.999659, 0.026131, 0, 0.999659, 0.026131, 0, 0.999678, 0.025377, 0, 0.999678, 0.025377, 0, 0.999678, 0.025377, 0, 0.999678, 0.025377, 0, 0.999678, 0.025377, 0, 0.999678, 0.025377, 0, 0.999705, 0.024299, 0, 0.999705, 0.024299, 0, 0.999705, 0.024299, 0, 0.999705, 0.024299, 0, 0.999705, 0.024299, 0, 0.999705, 0.024299, 0, 0.999737, 0.022939, 0, 0.999737, 0.022939, 0, 0.999737, 0.022939, 0, 0.999737, 0.022939, 0, 0.999737, 0.022939, 0, 0.999737, 0.022939, 0, 0.999772, 0.021339, 0, 0.999772, 0.021339, 0, 0.999772, 0.021339, 0, 0.999772, 0.021339, 0, 0.999772, 0.021339, 0, 0.999772, 0.021339, 0, 0.999810, 0.019500, 0, 0.999810, 0.019500, 0, 0.999810, 0.019500, 0, 0.999810, 0.019500, 0, 0.999810, 0.019500, 0, 0.999810, 0.019500, 0, 0.999848, 0.017423, 0, 0.999848, 0.017423, 0, 0.999848, 0.017423, 0, 0.999848, 0.017423, 0, 0.999848, 0.017423, 0, 0.999848, 0.017423, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999828, 0.018532, 0, 0.999828, 0.018532, 0, 0.999828, 0.018532, 0, 0.999828, 0.018532, 0, 0.999828, 0.018532, 0, 0.999828, 0.018532, 0, 0.999789, 0.020565, 0, 0.999789, 0.020565, 0, 0.999789, 0.020565, 0, 0.999789, 0.020565, 0, 0.999789, 0.020565, 0, 0.999789, 0.020565, 0, 0.999751, 0.022319, 0, 0.999751, 0.022319, 0, 0.999751, 0.022319, 0, 0.999751, 0.022319, 0, 0.999751, 0.022319, 0, 0.999751, 0.022319, 0, 0.999717, 0.023794, 0, 0.999717, 0.023794, 0, 0.999717, 0.023794, 0, 0.999717, 0.023794, 0, 0.999717, 0.023794, 0, 0.999717, 0.023794, 0, 0.999688, 0.024992, 0, 0.999688, 0.024992, 0, 0.999688, 0.024992, 0, 0.999688, 0.024992, 0, 0.999688, 0.024992, 0, 0.999688, 0.024992, 0, 0.999664, 0.025925, 0, 0.999664, 0.025925, 0, 0.999664, 0.025925, 0, 0.999664, 0.025925, 0, 0.999664, 0.025925, 0, 0.999664, 0.025925, 0, 0.999646, 0.026604, 0, 0.999646, 0.026604, 0, 0.999646, 0.026604, 0, 0.999646, 0.026604, 0, 0.999646, 0.026604, 0, 0.999646, 0.026604, 0, 0.999635, 0.027031, 0, 0.999635, 0.027031, 0, 0.999635, 0.027031, 0, 0.999635, 0.027031, 0, 0.999635, 0.027031, 0, 0.999635, 0.027031, 0, 0.999630, 0.027202, 0, 0.999630, 0.027202, 0, 0.999630, 0.027202, 0, 0.999630, 0.027202, 0, 0.999630, 0.027202, 0, 0.999630, 0.027202, 0, 0.999633, 0.027091, 0, 0.999633, 0.027091, 0, 0.999633, 0.027091, 0, 0.999633, 0.027091, 0, 0.999633, 0.027091, 0, 0.999633, 0.027091, 0, 0.999645, 0.026649, 0, 0.999645, 0.026649, 0, 0.999645, 0.026649, 0, 0.999645, 0.026649, 0, 0.999645, 0.026649, 0, 0.999645, 0.026649, 0, 0.999665, 0.025876, 0, 0.999665, 0.025876, 0, 0.999665, 0.025876, 0, 0.999665, 0.025876, 0, 0.999665, 0.025876, 0, 0.999665, 0.025876, 0, 0.999693, 0.024772, 0, 0.999693, 0.024772, 0, 0.999693, 0.024772, 0, 0.999693, 0.024772, 0, 0.999693, 0.024772, 0, 0.999693, 0.024772, 0, 0.999727, 0.023380, 0, 0.999727, 0.023380, 0, 0.999727, 0.023380, 0, 0.999727, 0.023380, 0, 0.999727, 0.023380, 0, 0.999727, 0.023380, 0, 0.999763, 0.021749, 0, 0.999763, 0.021749, 0, 0.999763, 0.021749, 0, 0.999763, 0.021749, 0, 0.999763, 0.021749, 0, 0.999763, 0.021749, 0, 0.999802, 0.019879, 0, 0.999802, 0.019879, 0, 0.999802, 0.019879, 0, 0.999802, 0.019879, 0, 0.999802, 0.019879, 0, 0.999802, 0.019879, 0, 0.999842, 0.017772, 0, 0.999842, 0.017772, 0, 0.999842, 0.017772, 0, 0.999842, 0.017772, 0, 0.999842, 0.017772, 0, 0.999842, 0.017772, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999515, 0.031129, 0, 0.999515, 0.031129, 0, 0.999515, 0.031129, 0, 0.999515, 0.031129, 0, 0.999515, 0.031129, 0, 0.999515, 0.031129, 0, 0.999520, 0.030970, 0, 0.999520, 0.030970, 0, 0.999520, 0.030970, 0, 0.999520, 0.030970, 0, 0.999520, 0.030970, 0, 0.999520, 0.030970, 0, 0.999648, 0.026549, 0, 0.999648, 0.026549, 0, 0.999648, 0.026549, 0, 0.999648, 0.026549, 0, 0.999648, 0.026549, 0, 0.999648, 0.026549, 0, 0.999614, 0.027795, 0, 0.999614, 0.027795, 0, 0.999614, 0.027795, 0, 0.999614, 0.027795, 0, 0.999614, 0.027795, 0, 0.999614, 0.027795, 0, 0.999584, 0.028858, 0, 0.999584, 0.028858, 0, 0.999584, 0.028858, 0, 0.999584, 0.028858, 0, 0.999584, 0.028858, 0, 0.999584, 0.028858, 0, 0.999558, 0.029738, 0, 0.999558, 0.029738, 0, 0.999558, 0.029738, 0, 0.999558, 0.029738, 0, 0.999558, 0.029738, 0, 0.999558, 0.029738, 0, 0.999537, 0.030435, 0, 0.999537, 0.030435, 0, 0.999537, 0.030435, 0, 0.999537, 0.030435, 0, 0.999537, 0.030435, 0, 0.999537, 0.030435, 0, 0.999521, 0.030959, 0, 0.999521, 0.030959, 0, 0.999521, 0.030959, 0, 0.999521, 0.030959, 0, 0.999521, 0.030959, 0, 0.999521, 0.030959, 0, 0.999510, 0.031317, 0, 0.999510, 0.031317, 0, 0.999510, 0.031317, 0, 0.999510, 0.031317, 0, 0.999510, 0.031317, 0, 0.999510, 0.031317, 0, 0.999503, 0.031510, 0, 0.999503, 0.031510, 0, 0.999503, 0.031510, 0, 0.999503, 0.031510, 0, 0.999503, 0.031510, 0, 0.999503, 0.031510, 0, 0.999503, 0.031540, 0, 0.999503, 0.031540, 0, 0.999503, 0.031540, 0, 0.999503, 0.031540, 0, 0.999503, 0.031540, 0, 0.999503, 0.031540, 0, 0.999507, 0.031387, 0, 0.999507, 0.031387, 0, 0.999507, 0.031387, 0, 0.999507, 0.031387, 0, 0.999507, 0.031387, 0, 0.999507, 0.031387, 0, 0.999518, 0.031035, 0, 0.999518, 0.031035, 0, 0.999518, 0.031035, 0, 0.999518, 0.031035, 0, 0.999518, 0.031035, 0, 0.999518, 0.031035, 0, 0.999535, 0.030485, 0, 0.999535, 0.030485, 0, 0.999535, 0.030485, 0, 0.999535, 0.030485, 0, 0.999535, 0.030485, 0, 0.999535, 0.030485, 0, 0.999558, 0.029737, 0, 0.999558, 0.029737, 0, 0.999558, 0.029737, 0, 0.999558, 0.029737, 0, 0.999558, 0.029737, 0, 0.999558, 0.029737, 0, 0.999585, 0.028800, 0, 0.999585, 0.028800, 0, 0.999585, 0.028800, 0, 0.999585, 0.028800, 0, 0.999585, 0.028800, 0, 0.999585, 0.028800, 0, 0.999617, 0.027686, 0, 0.999617, 0.027686, 0, 0.999617, 0.027686, 0, 0.999617, 0.027686, 0, 0.999617, 0.027686, 0, 0.999617, 0.027686, 0, 0.999652, 0.026394, 0, 0.999652, 0.026394, 0, 0.999652, 0.026394, 0, 0.999652, 0.026394, 0, 0.999652, 0.026394, 0, 0.999652, 0.026394, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999569, 0.029355, 0, 0.999569, 0.029355, 0, 0.999569, 0.029355, 0, 0.999569, 0.029355, 0, 0.999569, 0.029355, 0, 0.999569, 0.029355, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999156, 0.041080, 0, 0.999156, 0.041080, 0, 0.999156, 0.041080, 0, 0.999156, 0.041080, 0, 0.999156, 0.041080, 0, 0.999156, 0.041080, 0, 0.999380, 0.035220, 0, 0.999380, 0.035220, 0, 0.999380, 0.035220, 0, 0.999380, 0.035220, 0, 0.999380, 0.035220, 0, 0.999380, 0.035220, 0, 0.999374, 0.035390, 0, 0.999374, 0.035390, 0, 0.999374, 0.035390, 0, 0.999374, 0.035390, 0, 0.999374, 0.035390, 0, 0.999374, 0.035390, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999181, 0.040470, 0, 0.999181, 0.040470, 0, 0.999181, 0.040470, 0, 0.999181, 0.040470, 0, 0.999181, 0.040470, 0, 0.999181, 0.040470, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999906, 0.013690, 0, 0.999906, 0.013690, 0, 0.999906, 0.013690, 0, 0.999906, 0.013690, 0, 0.999906, 0.013690, 0, 0.999906, 0.013690, 0, 0.999909, 0.013489, 0, 0.999909, 0.013489, 0, 0.999909, 0.013489, 0, 0.999909, 0.013489, 0, 0.999909, 0.013489, 0, 0.999909, 0.013489, 0, 0.999914, 0.013128, 0, 0.999914, 0.013128, 0, 0.999914, 0.013128, 0, 0.999914, 0.013128, 0, 0.999914, 0.013128, 0, 0.999914, 0.013128, 0, 0.999921, 0.012606, 0, 0.999921, 0.012606, 0, 0.999921, 0.012606, 0, 0.999921, 0.012606, 0, 0.999921, 0.012606, 0, 0.999921, 0.012606, 0, 0.999929, 0.011892, 0, 0.999929, 0.011892, 0, 0.999929, 0.011892, 0, 0.999929, 0.011892, 0, 0.999929, 0.011892, 0, 0.999929, 0.011892, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999940, 0.010957, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999952, 0.009802, 0, 0.999965, 0.008427, 0, 0.999965, 0.008427, 0, 0.999965, 0.008427, 0, 0.999965, 0.008427, 0, 0.999965, 0.008427, 0, 0.999965, 0.008427, 0, 0.999973, 0.007296, 0, 0.999973, 0.007296, 0, 0.999973, 0.007296, 0, 0.999973, 0.007296, 0, 0.999973, 0.007296, 0, 0.999973, 0.007296, 0, 0.999980, 0.006289, 0, 0.999980, 0.006289, 0, 0.999980, 0.006289, 0, 0.999980, 0.006289, 0, 0.999980, 0.006289, 0, 0.999980, 0.006289, 0, 0.999985, 0.005405, 0, 0.999985, 0.005405, 0, 0.999985, 0.005405, 0, 0.999985, 0.005405, 0, 0.999985, 0.005405, 0, 0.999985, 0.005405, 0, 0.999989, 0.004646, 0, 0.999989, 0.004646, 0, 0.999989, 0.004646, 0, 0.999989, 0.004646, 0, 0.999989, 0.004646, 0, 0.999989, 0.004646, 0, 0.999992, 0.004035, 0, 0.999992, 0.004035, 0, 0.999992, 0.004035, 0, 0.999992, 0.004035, 0, 0.999992, 0.004035, 0, 0.999992, 0.004035, 0, 0.999994, 0.003598, 0, 0.999994, 0.003598, 0, 0.999994, 0.003598, 0, 0.999994, 0.003598, 0, 0.999994, 0.003598, 0, 0.999994, 0.003598, 0, 0.999994, 0.003335, 0, 0.999994, 0.003335, 0, 0.999994, 0.003335, 0, 0.999994, 0.003335, 0, 0.999994, 0.003335, 0, 0.999994, 0.003335, 0, 0.999995, 0.003244, 0, 0.999995, 0.003244, 0, 0.999995, 0.003244, 0, 0.999995, 0.003244, 0, 0.999995, 0.003244, 0, 0.999995, 0.003244, 0, 0.999994, 0.003400, 0, 0.999994, 0.003400, 0, 0.999994, 0.003400, 0, 0.999994, 0.003400, 0, 0.999994, 0.003400, 0, 0.999994, 0.003400, 0, 0.999992, 0.003881, 0, 0.999992, 0.003881, 0, 0.999992, 0.003881, 0, 0.999992, 0.003881, 0, 0.999992, 0.003881, 0, 0.999992, 0.003881, 0, 0.999989, 0.004685, 0, 0.999989, 0.004685, 0, 0.999989, 0.004685, 0, 0.999989, 0.004685, 0, 0.999989, 0.004685, 0, 0.999989, 0.004685, 0, 0.999983, 0.005812, 0, 0.999983, 0.005812, 0, 0.999983, 0.005812, 0, 0.999983, 0.005812, 0, 0.999983, 0.005812, 0, 0.999983, 0.005812, 0, 0.999974, 0.007205, 0, 0.999974, 0.007205, 0, 0.999974, 0.007205, 0, 0.999974, 0.007205, 0, 0.999974, 0.007205, 0, 0.999974, 0.007205, 0, 0.999961, 0.008810, 0, 0.999961, 0.008810, 0, 0.999961, 0.008810, 0, 0.999961, 0.008810, 0, 0.999961, 0.008810, 0, 0.999961, 0.008810, 0, 0.999943, 0.010629, 0, 0.999943, 0.010629, 0, 0.999943, 0.010629, 0, 0.999943, 0.010629, 0, 0.999943, 0.010629, 0, 0.999943, 0.010629, 0, 0.999920, 0.012663, 0, 0.999920, 0.012663, 0, 0.999920, 0.012663, 0, 0.999920, 0.012663, 0, 0.999920, 0.012663, 0, 0.999920, 0.012663, 0, 0.999907, 0.013668, 0, 0.999907, 0.013668, 0, 0.999907, 0.013668, 0, 0.999907, 0.013668, 0, 0.999907, 0.013668, 0, 0.999907, 0.013668, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999992, 0.004020, 0, 0.999992, 0.004020, 0, 0.999992, 0.004020, 0, 0.999992, 0.004020, 0, 0.999992, 0.004020, 0, 0.999992, 0.004020, 0, 0.999996, 0.002867, 0, 0.999996, 0.002867, 0, 0.999996, 0.002867, 0, 0.999996, 0.002867, 0, 0.999996, 0.002867, 0, 0.999996, 0.002867, 0, 0.999998, 0.001874, 0, 0.999998, 0.001874, 0, 0.999998, 0.001874, 0, 0.999998, 0.001874, 0, 0.999998, 0.001874, 0, 0.999998, 0.001874, 0, 0.999999, 0.001042, 0, 0.999999, 0.001042, 0, 0.999999, 0.001042, 0, 0.999999, 0.001042, 0, 0.999999, 0.001042, 0, 0.999999, 0.001042, 0, 1.000000, 0.000372, 0, 1.000000, 0.000372, 0, 1.000000, 0.000372, 0, 1.000000, 0.000372, 0, 1.000000, 0.000372, 0, 1.000000, 0.000372, 0, 1, -0.000144, 0, 1, -0.000144, 0, 1, -0.000144, 0, 1, -0.000144, 0, 1, -0.000144, 0, 1, -0.000144, 0, 1.000000, -0.000514, 0, 1.000000, -0.000514, 0, 1.000000, -0.000514, 0, 1.000000, -0.000514, 0, 1.000000, -0.000514, 0, 1.000000, -0.000514, 0, 1.000000, -0.000737, 0, 1.000000, -0.000737, 0, 1.000000, -0.000737, 0, 1.000000, -0.000737, 0, 1.000000, -0.000737, 0, 1.000000, -0.000737, 0, 1.000000, -0.000812, 0, 1.000000, -0.000812, 0, 1.000000, -0.000812, 0, 1.000000, -0.000812, 0, 1.000000, -0.000812, 0, 1.000000, -0.000812, 0, 1.000000, -0.000660, 0, 1.000000, -0.000660, 0, 1.000000, -0.000660, 0, 1.000000, -0.000660, 0, 1.000000, -0.000660, 0, 1.000000, -0.000660, 0, 1, -0.000201, 0, 1, -0.000201, 0, 1, -0.000201, 0, 1, -0.000201, 0, 1, -0.000201, 0, 1, -0.000201, 0, 1.000000, 0.000569, 0, 1.000000, 0.000569, 0, 1.000000, 0.000569, 0, 1.000000, 0.000569, 0, 1.000000, 0.000569, 0, 1.000000, 0.000569, 0, 0.999999, 0.001648, 0, 0.999999, 0.001648, 0, 0.999999, 0.001648, 0, 0.999999, 0.001648, 0, 0.999999, 0.001648, 0, 0.999999, 0.001648, 0, 0.999996, 0.002989, 0, 0.999996, 0.002989, 0, 0.999996, 0.002989, 0, 0.999996, 0.002989, 0, 0.999996, 0.002989, 0, 0.999996, 0.002989, 0, 0.999990, 0.004541, 0, 0.999990, 0.004541, 0, 0.999990, 0.004541, 0, 0.999990, 0.004541, 0, 0.999990, 0.004541, 0, 0.999990, 0.004541, 0, 0.999980, 0.006303, 0, 0.999980, 0.006303, 0, 0.999980, 0.006303, 0, 0.999980, 0.006303, 0, 0.999980, 0.006303, 0, 0.999980, 0.006303, 0, 0.999966, 0.008274, 0, 0.999966, 0.008274, 0, 0.999966, 0.008274, 0, 0.999966, 0.008274, 0, 0.999966, 0.008274, 0, 0.999966, 0.008274, 0, 0.999944, 0.010544, 0, 0.999944, 0.010544, 0, 0.999944, 0.010544, 0, 0.999944, 0.010544, 0, 0.999944, 0.010544, 0, 0.999944, 0.010544, 0, 0.999922, 0.012510, 0, 0.999922, 0.012510, 0, 0.999922, 0.012510, 0, 0.999922, 0.012510, 0, 0.999922, 0.012510, 0, 0.999922, 0.012510, 0, 0.999900, 0.014170, 0, 0.999900, 0.014170, 0, 0.999900, 0.014170, 0, 0.999900, 0.014170, 0, 0.999900, 0.014170, 0, 0.999900, 0.014170, 0, 0.999880, 0.015524, 0, 0.999880, 0.015524, 0, 0.999880, 0.015524, 0, 0.999880, 0.015524, 0, 0.999880, 0.015524, 0, 0.999880, 0.015524, 0, 0.999863, 0.016566, 0, 0.999863, 0.016566, 0, 0.999863, 0.016566, 0, 0.999863, 0.016566, 0, 0.999863, 0.016566, 0, 0.999863, 0.016566, 0, 0.999851, 0.017292, 0, 0.999851, 0.017292, 0, 0.999851, 0.017292, 0, 0.999851, 0.017292, 0, 0.999851, 0.017292, 0, 0.999851, 0.017292, 0, 0.999843, 0.017703, 0, 0.999843, 0.017703, 0, 0.999843, 0.017703, 0, 0.999843, 0.017703, 0, 0.999843, 0.017703, 0, 0.999843, 0.017703, 0, 0.999842, 0.017802, 0, 0.999842, 0.017802, 0, 0.999842, 0.017802, 0, 0.999842, 0.017802, 0, 0.999842, 0.017802, 0, 0.999842, 0.017802, 0, 0.999844, 0.017687, 0, 0.999844, 0.017687, 0, 0.999844, 0.017687, 0, 0.999844, 0.017687, 0, 0.999844, 0.017687, 0, 0.999844, 0.017687, 0, 0.999835, 0.018139, 0, 0.999835, 0.018139, 0, 0.999835, 0.018139, 0, 0.999835, 0.018139, 0, 0.999835, 0.018139, 0, 0.999835, 0.018139, 0, 0.999807, 0.019670, 0, 0.999807, 0.019670, 0, 0.999807, 0.019670, 0, 0.999807, 0.019670, 0, 0.999807, 0.019670, 0, 0.999807, 0.019670, 0, 0.999779, 0.021022, 0, 0.999779, 0.021022, 0, 0.999779, 0.021022, 0, 0.999779, 0.021022, 0, 0.999779, 0.021022, 0, 0.999779, 0.021022, 0, 0.999754, 0.022198, 0, 0.999754, 0.022198, 0, 0.999754, 0.022198, 0, 0.999754, 0.022198, 0, 0.999754, 0.022198, 0, 0.999754, 0.022198, 0, 0.999731, 0.023197, 0, 0.999731, 0.023197, 0, 0.999731, 0.023197, 0, 0.999731, 0.023197, 0, 0.999731, 0.023197, 0, 0.999731, 0.023197, 0, 0.999712, 0.024001, 0, 0.999712, 0.024001, 0, 0.999712, 0.024001, 0, 0.999712, 0.024001, 0, 0.999712, 0.024001, 0, 0.999712, 0.024001, 0, 0.999698, 0.024590, 0, 0.999698, 0.024590, 0, 0.999698, 0.024590, 0, 0.999698, 0.024590, 0, 0.999698, 0.024590, 0, 0.999698, 0.024590, 0, 0.999688, 0.024963, 0, 0.999688, 0.024963, 0, 0.999688, 0.024963, 0, 0.999688, 0.024963, 0, 0.999688, 0.024963, 0, 0.999688, 0.024963, 0, 0.999684, 0.025119, 0, 0.999684, 0.025119, 0, 0.999684, 0.025119, 0, 0.999684, 0.025119, 0, 0.999684, 0.025119, 0, 0.999684, 0.025119, 0, 0.999685, 0.025112, 0, 0.999685, 0.025112, 0, 0.999685, 0.025112, 0, 0.999685, 0.025112, 0, 0.999685, 0.025112, 0, 0.999685, 0.025112, 0, 0.999687, 0.025028, 0, 0.999687, 0.025028, 0, 0.999687, 0.025028, 0, 0.999687, 0.025028, 0, 0.999687, 0.025028, 0, 0.999687, 0.025028, 0, 0.999691, 0.024867, 0, 0.999691, 0.024867, 0, 0.999691, 0.024867, 0, 0.999691, 0.024867, 0, 0.999691, 0.024867, 0, 0.999691, 0.024867, 0, 0.999697, 0.024629, 0, 0.999697, 0.024629, 0, 0.999697, 0.024629, 0, 0.999697, 0.024629, 0, 0.999697, 0.024629, 0, 0.999697, 0.024629, 0, 0.999704, 0.024313, 0, 0.999704, 0.024313, 0, 0.999704, 0.024313, 0, 0.999704, 0.024313, 0, 0.999704, 0.024313, 0, 0.999704, 0.024313, 0, 0.999714, 0.023920, 0, 0.999714, 0.023920, 0, 0.999714, 0.023920, 0, 0.999714, 0.023920, 0, 0.999714, 0.023920, 0, 0.999714, 0.023920, 0, 0.999725, 0.023449, 0, 0.999725, 0.023449, 0, 0.999725, 0.023449, 0, 0.999725, 0.023449, 0, 0.999725, 0.023449, 0, 0.999725, 0.023449, 0, 0.999738, 0.022900, 0, 0.999738, 0.022900, 0, 0.999738, 0.022900, 0, 0.999738, 0.022900, 0, 0.999738, 0.022900, 0, 0.999738, 0.022900, 0, 0.999636, 0.026986, 0, 0.999636, 0.026986, 0, 0.999636, 0.026986, 0, 0.999636, 0.026986, 0, 0.999636, 0.026986, 0, 0.999636, 0.026986, 0, 0.999621, 0.027525, 0, 0.999621, 0.027525, 0, 0.999621, 0.027525, 0, 0.999621, 0.027525, 0, 0.999621, 0.027525, 0, 0.999621, 0.027525, 0, 0.999608, 0.027987, 0, 0.999608, 0.027987, 0, 0.999608, 0.027987, 0, 0.999608, 0.027987, 0, 0.999608, 0.027987, 0, 0.999608, 0.027987, 0, 0.999597, 0.028373, 0, 0.999597, 0.028373, 0, 0.999597, 0.028373, 0, 0.999597, 0.028373, 0, 0.999597, 0.028373, 0, 0.999597, 0.028373, 0, 0.999589, 0.028681, 0, 0.999589, 0.028681, 0, 0.999589, 0.028681, 0, 0.999589, 0.028681, 0, 0.999589, 0.028681, 0, 0.999589, 0.028681, 0, 0.999582, 0.028913, 0, 0.999582, 0.028913, 0, 0.999582, 0.028913, 0, 0.999582, 0.028913, 0, 0.999582, 0.028913, 0, 0.999582, 0.028913, 0, 0.999577, 0.029069, 0, 0.999577, 0.029069, 0, 0.999577, 0.029069, 0, 0.999577, 0.029069, 0, 0.999577, 0.029069, 0, 0.999577, 0.029069, 0, 0.999575, 0.029149, 0, 0.999575, 0.029149, 0, 0.999575, 0.029149, 0, 0.999575, 0.029149, 0, 0.999575, 0.029149, 0, 0.999575, 0.029149, 0, 0.999575, 0.029154, 0, 0.999575, 0.029154, 0, 0.999575, 0.029154, 0, 0.999575, 0.029154, 0, 0.999575, 0.029154, 0, 0.999575, 0.029154, 0, 0.999583, 0.028865, 0, 0.999583, 0.028865, 0, 0.999583, 0.028865, 0, 0.999583, 0.028865, 0, 0.999583, 0.028865, 0, 0.999583, 0.028865, 0, 0.999603, 0.028182, 0, 0.999603, 0.028182, 0, 0.999603, 0.028182, 0, 0.999603, 0.028182, 0, 0.999603, 0.028182, 0, 0.999603, 0.028182, 0, 0.999633, 0.027106, 0, 0.999633, 0.027106, 0, 0.999633, 0.027106, 0, 0.999633, 0.027106, 0, 0.999633, 0.027106, 0, 0.999633, 0.027106, 0, 0.999671, 0.025637, 0, 0.999671, 0.025637, 0, 0.999671, 0.025637, 0, 0.999671, 0.025637, 0, 0.999671, 0.025637, 0, 0.999671, 0.025637, 0, 0.999717, 0.023769, 0, 0.999717, 0.023769, 0, 0.999717, 0.023769, 0, 0.999717, 0.023769, 0, 0.999717, 0.023769, 0, 0.999717, 0.023769, 0, 0.999769, 0.021495, 0, 0.999769, 0.021495, 0, 0.999769, 0.021495, 0, 0.999769, 0.021495, 0, 0.999769, 0.021495, 0, 0.999769, 0.021495, 0, 0.999823, 0.018812, 0, 0.999823, 0.018812, 0, 0.999823, 0.018812, 0, 0.999823, 0.018812, 0, 0.999823, 0.018812, 0, 0.999823, 0.018812, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999619, 0.027620, 0, 0.999619, 0.027620, 0, 0.999619, 0.027620, 0, 0.999619, 0.027620, 0, 0.999619, 0.027620, 0, 0.999619, 0.027620, 0, 0.999626, 0.027364, 0, 0.999626, 0.027364, 0, 0.999626, 0.027364, 0, 0.999626, 0.027364, 0, 0.999626, 0.027364, 0, 0.999626, 0.027364, 0, 0.999725, 0.023457, 0, 0.999725, 0.023457, 0, 0.999725, 0.023457, 0, 0.999725, 0.023457, 0, 0.999725, 0.023457, 0, 0.999725, 0.023457, 0, 0.999720, 0.023677, 0, 0.999720, 0.023677, 0, 0.999720, 0.023677, 0, 0.999720, 0.023677, 0, 0.999720, 0.023677, 0, 0.999720, 0.023677, 0, 0.999959, 0.009103, 0, 0.999959, 0.009103, 0, 0.999959, 0.009103, 0, 0.999959, 0.009103, 0, 0.999959, 0.009103, 0, 0.999959, 0.009103, 0, 0.999971, 0.007609, 0, 0.999971, 0.007609, 0, 0.999971, 0.007609, 0, 0.999971, 0.007609, 0, 0.999971, 0.007609, 0, 0.999971, 0.007609, 0, 0.999979, 0.006406, 0, 0.999979, 0.006406, 0, 0.999979, 0.006406, 0, 0.999979, 0.006406, 0, 0.999979, 0.006406, 0, 0.999979, 0.006406, 0, 0.999985, 0.005497, 0, 0.999985, 0.005497, 0, 0.999985, 0.005497, 0, 0.999985, 0.005497, 0, 0.999985, 0.005497, 0, 0.999985, 0.005497, 0, 0.999988, 0.004880, 0, 0.999988, 0.004880, 0, 0.999988, 0.004880, 0, 0.999988, 0.004880, 0, 0.999988, 0.004880, 0, 0.999988, 0.004880, 0, 0.999990, 0.004468, 0, 0.999990, 0.004468, 0, 0.999990, 0.004468, 0, 0.999990, 0.004468, 0, 0.999990, 0.004468, 0, 0.999990, 0.004468, 0, 0.999991, 0.004172, 0, 0.999991, 0.004172, 0, 0.999991, 0.004172, 0, 0.999991, 0.004172, 0, 0.999991, 0.004172, 0, 0.999991, 0.004172, 0, 0.999992, 0.003992, 0, 0.999992, 0.003992, 0, 0.999992, 0.003992, 0, 0.999992, 0.003992, 0, 0.999992, 0.003992, 0, 0.999992, 0.003992, 0, 0.999992, 0.003929, 0, 0.999992, 0.003929, 0, 0.999992, 0.003929, 0, 0.999992, 0.003929, 0, 0.999992, 0.003929, 0, 0.999992, 0.003929, 0, 0.999992, 0.003911, 0, 0.999992, 0.003911, 0, 0.999992, 0.003911, 0, 0.999992, 0.003911, 0, 0.999992, 0.003911, 0, 0.999992, 0.003911, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.000122, 0, 1, 0.000122, 0, 1, 0.000122, 0, 1, 0.000122, 0, 1, 0.000122, 0, 1, 0.000122, 0, 1.000000, 0.000490, 0, 1.000000, 0.000490, 0, 1.000000, 0.000490, 0, 1.000000, 0.000490, 0, 1.000000, 0.000490, 0, 1.000000, 0.000490, 0, 0.999999, 0.001105, 0, 0.999999, 0.001105, 0, 0.999999, 0.001105, 0, 0.999999, 0.001105, 0, 0.999999, 0.001105, 0, 0.999999, 0.001105, 0, 0.999998, 0.001966, 0, 0.999998, 0.001966, 0, 0.999998, 0.001966, 0, 0.999998, 0.001966, 0, 0.999998, 0.001966, 0, 0.999998, 0.001966, 0, 0.999995, 0.003154, 0, 0.999995, 0.003154, 0, 0.999995, 0.003154, 0, 0.999995, 0.003154, 0, 0.999995, 0.003154, 0, 0.999995, 0.003154, 0, 0.999989, 0.004747, 0, 0.999989, 0.004747, 0, 0.999989, 0.004747, 0, 0.999989, 0.004747, 0, 0.999989, 0.004747, 0, 0.999989, 0.004747, 0, 0.999977, 0.006744, 0, 0.999977, 0.006744, 0, 0.999977, 0.006744, 0, 0.999977, 0.006744, 0, 0.999977, 0.006744, 0, 0.999977, 0.006744, 0, 0.999958, 0.009145, 0, 0.999958, 0.009145, 0, 0.999958, 0.009145, 0, 0.999958, 0.009145, 0, 0.999958, 0.009145, 0, 0.999958, 0.009145, 0, 0.999717, 0.023785, 0, 0.999717, 0.023785, 0, 0.999717, 0.023785, 0, 0.999717, 0.023785, 0, 0.999717, 0.023785, 0, 0.999717, 0.023785, 0, 0.999715, 0.023865, 0, 0.999715, 0.023865, 0, 0.999715, 0.023865, 0, 0.999715, 0.023865, 0, 0.999715, 0.023865, 0, 0.999715, 0.023865, 0, 0.999612, 0.027840, 0, 0.999612, 0.027840, 0, 0.999612, 0.027840, 0, 0.999612, 0.027840, 0, 0.999612, 0.027840, 0, 0.999612, 0.027840, 0, 0.999615, 0.027746, 0, 0.999615, 0.027746, 0, 0.999615, 0.027746, 0, 0.999615, 0.027746, 0, 0.999615, 0.027746, 0, 0.999615, 0.027746, 0, 0.999364, 0.035665, 0, 0.999364, 0.035665, 0, 0.999364, 0.035665, 0, 0.999364, 0.035665, 0, 0.999364, 0.035665, 0, 0.999364, 0.035665, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999981, 0.006185, 0, 0.999981, 0.006185, 0, 0.999981, 0.006185, 0, 0.999981, 0.006185, 0, 0.999981, 0.006185, 0, 0.999981, 0.006185, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999981, 0.006091, 0, 0.999981, 0.006091, 0, 0.999981, 0.006091, 0, 0.999981, 0.006091, 0, 0.999981, 0.006091, 0, 0.999981, 0.006091, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999982, 0.005999, 0, 0.999982, 0.005999, 0, 0.999982, 0.005999, 0, 0.999982, 0.005999, 0, 0.999982, 0.005999, 0, 0.999982, 0.005999, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.680665, -0.732595, 0, 0.680665, -0.732595, 0, 0.680665, -0.732595, 0, 0.680665, -0.732595, 0, 0.680665, -0.732595, 0, 0.680665, -0.732595, 0, 0.560946, -0.827852, 0, 0.560946, -0.827852, 0, 0.560946, -0.827852, 0, 0.560946, -0.827852, 0, 0.560946, -0.827852, 0, 0.560946, -0.827852, 0, 0.610939, -0.791678, 0, 0.610939, -0.791678, 0, 0.610939, -0.791678, 0, 0.610939, -0.791678, 0, 0.610939, -0.791678, 0, 0.610939, -0.791678, 0, 0.776365, -0.630284, 0, 0.776365, -0.630284, 0, 0.776365, -0.630284, 0, 0.776365, -0.630284, 0, 0.776365, -0.630284, 0, 0.776365, -0.630284, 0, 0.621219, -0.783637, 0, 0.621219, -0.783637, 0, 0.621219, -0.783637, 0, 0.621219, -0.783637, 0, 0.621219, -0.783637, 0, 0.621219, -0.783637, 0, 0.596715, -0.802453, 0, 0.596715, -0.802453, 0, 0.596715, -0.802453, 0, 0.596715, -0.802453, 0, 0.596715, -0.802453, 0, 0.596715, -0.802453, 0, 0.571633, -0.820509, 0, 0.571633, -0.820509, 0, 0.571633, -0.820509, 0, 0.571633, -0.820509, 0, 0.571633, -0.820509, 0, 0.571633, -0.820509, 0, 0.545849, -0.837884, 0, 0.545849, -0.837884, 0, 0.545849, -0.837884, 0, 0.545849, -0.837884, 0, 0.545849, -0.837884, 0, 0.545849, -0.837884, 0, 0.519223, -0.854639, 0, 0.519223, -0.854639, 0, 0.519223, -0.854639, 0, 0.519223, -0.854639, 0, 0.519223, -0.854639, 0, 0.519223, -0.854639, 0, 0.491730, -0.870748, 0, 0.491730, -0.870748, 0, 0.491730, -0.870748, 0, 0.491730, -0.870748, 0, 0.491730, -0.870748, 0, 0.491730, -0.870748, 0, 0.463342, -0.886180, 0, 0.463342, -0.886180, 0, 0.463342, -0.886180, 0, 0.463342, -0.886180, 0, 0.463342, -0.886180, 0, 0.463342, -0.886180, 0, 0.433889, -0.900967, 0, 0.433889, -0.900967, 0, 0.433889, -0.900967, 0, 0.433889, -0.900967, 0, 0.433889, -0.900967, 0, 0.433889, -0.900967, 0, 0.403178, -0.915121, 0, 0.403178, -0.915121, 0, 0.403178, -0.915121, 0, 0.403178, -0.915121, 0, 0.403178, -0.915121, 0, 0.403178, -0.915121, 0, 0.337201, -0.941433, 0, 0.337201, -0.941433, 0, 0.337201, -0.941433, 0, 0.337201, -0.941433, 0, 0.337201, -0.941433, 0, 0.337201, -0.941433, 0, 0.277149, -0.960827, 0, 0.277149, -0.960827, 0, 0.277149, -0.960827, 0, 0.277149, -0.960827, 0, 0.277149, -0.960827, 0, 0.277149, -0.960827, 0, 0.223443, -0.974717, 0, 0.223443, -0.974717, 0, 0.223443, -0.974717, 0, 0.223443, -0.974717, 0, 0.223443, -0.974717, 0, 0.223443, -0.974717, 0, 0.176279, -0.984340, 0, 0.176279, -0.984340, 0, 0.176279, -0.984340, 0, 0.176279, -0.984340, 0, 0.176279, -0.984340, 0, 0.176279, -0.984340, 0, 0.137281, -0.990532, 0, 0.137281, -0.990532, 0, 0.137281, -0.990532, 0, 0.137281, -0.990532, 0, 0.137281, -0.990532, 0, 0.137281, -0.990532, 0, 0.109120, -0.994029, 0, 0.109120, -0.994029, 0, 0.109120, -0.994029, 0, 0.109120, -0.994029, 0, 0.109120, -0.994029, 0, 0.109120, -0.994029, 0, 0.094390, -0.995535, 0, 0.094390, -0.995535, 0, 0.094390, -0.995535, 0, 0.094390, -0.995535, 0, 0.094390, -0.995535, 0, 0.094390, -0.995535, 0, 0.097944, -0.995192, 0, 0.097944, -0.995192, 0, 0.097944, -0.995192, 0, 0.097944, -0.995192, 0, 0.097944, -0.995192, 0, 0.097944, -0.995192, 0, 0.127141, -0.991885, 0, 0.127141, -0.991885, 0, 0.127141, -0.991885, 0, 0.127141, -0.991885, 0, 0.127141, -0.991885, 0, 0.127141, -0.991885, 0, 0.192290, -0.981338, 0, 0.192290, -0.981338, 0, 0.192290, -0.981338, 0, 0.192290, -0.981338, 0, 0.192290, -0.981338, 0, 0.192290, -0.981338, 0, 0.305491, -0.952195, 0, 0.305491, -0.952195, 0, 0.305491, -0.952195, 0, 0.305491, -0.952195, 0, 0.305491, -0.952195, 0, 0.305491, -0.952195, 0, 0.467441, -0.884024, 0, 0.467441, -0.884024, 0, 0.467441, -0.884024, 0, 0.467441, -0.884024, 0, 0.467441, -0.884024, 0, 0.467441, -0.884024, 0, 0.643563, -0.765393, 0, 0.643563, -0.765393, 0, 0.643563, -0.765393, 0, 0.643563, -0.765393, 0, 0.643563, -0.765393, 0, 0.643563, -0.765393, 0, 0.782781, -0.622297, 0, 0.782781, -0.622297, 0, 0.782781, -0.622297, 0, 0.782781, -0.622297, 0, 0.782781, -0.622297, 0, 0.782781, -0.622297, 0, 0.868210, -0.496198, 0, 0.868210, -0.496198, 0, 0.868210, -0.496198, 0, 0.868210, -0.496198, 0, 0.868210, -0.496198, 0, 0.868210, -0.496198, 0, 0.913110, -0.407714, 0, 0.913110, -0.407714, 0, 0.913110, -0.407714, 0, 0.913110, -0.407714, 0, 0.913110, -0.407714, 0, 0.913110, -0.407714, 0, 0.923899, -0.382637, 0, 0.923899, -0.382637, 0, 0.923899, -0.382637, 0, 0.923899, -0.382637, 0, 0.923899, -0.382637, 0, 0.923899, -0.382637, 0, 0.931277, -0.364312, 0, 0.931277, -0.364312, 0, 0.931277, -0.364312, 0, 0.931277, -0.364312, 0, 0.931277, -0.364312, 0, 0.931277, -0.364312, 0, 0.936124, -0.351671, 0, 0.936124, -0.351671, 0, 0.936124, -0.351671, 0, 0.936124, -0.351671, 0, 0.936124, -0.351671, 0, 0.936124, -0.351671, 0, 0.939032, -0.343830, 0, 0.939032, -0.343830, 0, 0.939032, -0.343830, 0, 0.939032, -0.343830, 0, 0.939032, -0.343830, 0, 0.939032, -0.343830, 0, 0.940401, -0.340068, 0, 0.940401, -0.340068, 0, 0.940401, -0.340068, 0, 0.940401, -0.340068, 0, 0.940401, -0.340068, 0, 0.940401, -0.340068, 0, 0.940500, -0.339793, 0, 0.940500, -0.339793, 0, 0.940500, -0.339793, 0, 0.940500, -0.339793, 0, 0.940500, -0.339793, 0, 0.940500, -0.339793, 0, 0.939511, -0.342518, 0, 0.939511, -0.342518, 0, 0.939511, -0.342518, 0, 0.939511, -0.342518, 0, 0.939511, -0.342518, 0, 0.939511, -0.342518, 0, 0.937552, -0.347845, 0, 0.937552, -0.347845, 0, 0.937552, -0.347845, 0, 0.937552, -0.347845, 0, 0.937552, -0.347845, 0, 0.937552, -0.347845, 0, 0.882353, -0.470588, 0, 0.882353, -0.470588, 0, 0.882353, -0.470588, 0, 0.882353, -0.470588, 0, 0.882353, -0.470588, 0, 0.882353, -0.470588, 0, 0.886344, -0.463028, 0, 0.886344, -0.463028, 0, 0.886344, -0.463028, 0, 0.886344, -0.463028, 0, 0.886344, -0.463028, 0, 0.886344, -0.463028, 0, 0.888716, -0.458458, 0, 0.888716, -0.458458, 0, 0.888716, -0.458458, 0, 0.888716, -0.458458, 0, 0.888716, -0.458458, 0, 0.888716, -0.458458, 0, 0.889269, -0.457385, 0, 0.889269, -0.457385, 0, 0.889269, -0.457385, 0, 0.889269, -0.457385, 0, 0.889269, -0.457385, 0, 0.889269, -0.457385, 0, 0.887722, -0.460380, 0, 0.887722, -0.460380, 0, 0.887722, -0.460380, 0, 0.887722, -0.460380, 0, 0.887722, -0.460380, 0, 0.887722, -0.460380, 0, 0.883674, -0.468102, 0, 0.883674, -0.468102, 0, 0.883674, -0.468102, 0, 0.883674, -0.468102, 0, 0.883674, -0.468102, 0, 0.883674, -0.468102, 0, 0.876563, -0.481287, 0, 0.876563, -0.481287, 0, 0.876563, -0.481287, 0, 0.876563, -0.481287, 0, 0.876563, -0.481287, 0, 0.876563, -0.481287, 0, 0.865620, -0.500702, 0, 0.865620, -0.500702, 0, 0.865620, -0.500702, 0, 0.865620, -0.500702, 0, 0.865620, -0.500702, 0, 0.865620, -0.500702, 0, 0.849790, -0.527122, 0, 0.849790, -0.527122, 0, 0.849790, -0.527122, 0, 0.849790, -0.527122, 0, 0.849790, -0.527122, 0, 0.849790, -0.527122, 0, 0.803894, -0.594773, 0, 0.803894, -0.594773, 0, 0.803894, -0.594773, 0, 0.803894, -0.594773, 0, 0.803894, -0.594773, 0, 0.803894, -0.594773, 0, 0.737605, -0.675233, 0, 0.737605, -0.675233, 0, 0.737605, -0.675233, 0, 0.737605, -0.675233, 0, 0.737605, -0.675233, 0, 0.737605, -0.675233, 0, 0.651344, -0.758783, 0, 0.651344, -0.758783, 0, 0.651344, -0.758783, 0, 0.651344, -0.758783, 0, 0.651344, -0.758783, 0, 0.651344, -0.758783, 0, 0.553191, -0.833054, 0, 0.553191, -0.833054, 0, 0.553191, -0.833054, 0, 0.553191, -0.833054, 0, 0.553191, -0.833054, 0, 0.553191, -0.833054, 0, 0.456924, -0.889506, 0, 0.456924, -0.889506, 0, 0.456924, -0.889506, 0, 0.456924, -0.889506, 0, 0.456924, -0.889506, 0, 0.456924, -0.889506, 0, 0.374799, -0.927106, 0, 0.374799, -0.927106, 0, 0.374799, -0.927106, 0, 0.374799, -0.927106, 0, 0.374799, -0.927106, 0, 0.374799, -0.927106, 0, 0.312586, -0.949889, 0, 0.312586, -0.949889, 0, 0.312586, -0.949889, 0, 0.312586, -0.949889, 0, 0.312586, -0.949889, 0, 0.312586, -0.949889, 0, 0.270256, -0.962788, 0, 0.270256, -0.962788, 0, 0.270256, -0.962788, 0, 0.270256, -0.962788, 0, 0.270256, -0.962788, 0, 0.270256, -0.962788, 0, 0.248197, -0.968710, 0, 0.248197, -0.968710, 0, 0.248197, -0.968710, 0, 0.248197, -0.968710, 0, 0.248197, -0.968710, 0, 0.248197, -0.968710, 0, 0.244061, -0.969760, 0, 0.244061, -0.969760, 0, 0.244061, -0.969760, 0, 0.244061, -0.969760, 0, 0.244061, -0.969760, 0, 0.244061, -0.969760, 0, 0.253468, -0.967344, 0, 0.253468, -0.967344, 0, 0.253468, -0.967344, 0, 0.253468, -0.967344, 0, 0.253468, -0.967344, 0, 0.253468, -0.967344, 0, 0.273828, -0.961779, 0, 0.273828, -0.961779, 0, 0.273828, -0.961779, 0, 0.273828, -0.961779, 0, 0.273828, -0.961779, 0, 0.273828, -0.961779, 0, 0.303559, -0.952813, 0, 0.303559, -0.952813, 0, 0.303559, -0.952813, 0, 0.303559, -0.952813, 0, 0.303559, -0.952813, 0, 0.303559, -0.952813, 0, 0.341630, -0.939835, 0, 0.341630, -0.939835, 0, 0.341630, -0.939835, 0, 0.341630, -0.939835, 0, 0.341630, -0.939835, 0, 0.341630, -0.939835, 0, 0.387238, -0.921980, 0, 0.387238, -0.921980, 0, 0.387238, -0.921980, 0, 0.387238, -0.921980, 0, 0.387238, -0.921980, 0, 0.387238, -0.921980, 0, 0.439579, -0.898204, 0, 0.439579, -0.898204, 0, 0.439579, -0.898204, 0, 0.439579, -0.898204, 0, 0.439579, -0.898204, 0, 0.439579, -0.898204, 0, 0.460503, -0.887658, 0, 0.460503, -0.887658, 0, 0.460503, -0.887658, 0, 0.460503, -0.887658, 0, 0.460503, -0.887658, 0, 0.460503, -0.887658, 0, 0.480107, -0.877210, 0, 0.480107, -0.877210, 0, 0.480107, -0.877210, 0, 0.480107, -0.877210, 0, 0.480107, -0.877210, 0, 0.480107, -0.877210, 0, 0.498525, -0.866876, 0, 0.498525, -0.866876, 0, 0.498525, -0.866876, 0, 0.498525, -0.866876, 0, 0.498525, -0.866876, 0, 0.498525, -0.866876, 0, 0.515880, -0.856661, 0, 0.515880, -0.856661, 0, 0.515880, -0.856661, 0, 0.515880, -0.856661, 0, 0.515880, -0.856661, 0, 0.515880, -0.856661, 0, 0.532449, -0.846462, 0, 0.532449, -0.846462, 0, 0.532449, -0.846462, 0, 0.532449, -0.846462, 0, 0.532449, -0.846462, 0, 0.532449, -0.846462, 0, 0.548482, -0.836163, 0, 0.548482, -0.836163, 0, 0.548482, -0.836163, 0, 0.548482, -0.836163, 0, 0.548482, -0.836163, 0, 0.548482, -0.836163, 0, 0.564054, -0.825738, 0, 0.564054, -0.825738, 0, 0.564054, -0.825738, 0, 0.564054, -0.825738, 0, 0.564054, -0.825738, 0, 0.564054, -0.825738, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.890181, -0.455607, 0, 0.890181, -0.455607, 0, 0.890181, -0.455607, 0, 0.890181, -0.455607, 0, 0.890181, -0.455607, 0, 0.890181, -0.455607, 0, 0.804185, -0.594379, 0, 0.804185, -0.594379, 0, 0.804185, -0.594379, 0, 0.804185, -0.594379, 0, 0.804185, -0.594379, 0, 0.804185, -0.594379, 0, 0.788267, -0.615333, 0, 0.788267, -0.615333, 0, 0.788267, -0.615333, 0, 0.788267, -0.615333, 0, 0.788267, -0.615333, 0, 0.788267, -0.615333, 0, 0.776654, -0.629927, 0, 0.776654, -0.629927, 0, 0.776654, -0.629927, 0, 0.776654, -0.629927, 0, 0.776654, -0.629927, 0, 0.776654, -0.629927, 0, 0.769346, -0.638832, 0, 0.769346, -0.638832, 0, 0.769346, -0.638832, 0, 0.769346, -0.638832, 0, 0.769346, -0.638832, 0, 0.769346, -0.638832, 0, 0.766196, -0.642607, 0, 0.766196, -0.642607, 0, 0.766196, -0.642607, 0, 0.766196, -0.642607, 0, 0.766196, -0.642607, 0, 0.766196, -0.642607, 0, 0.766963, -0.641691, 0, 0.766963, -0.641691, 0, 0.766963, -0.641691, 0, 0.766963, -0.641691, 0, 0.766963, -0.641691, 0, 0.766963, -0.641691, 0, 0.771341, -0.636423, 0, 0.771341, -0.636423, 0, 0.771341, -0.636423, 0, 0.771341, -0.636423, 0, 0.771341, -0.636423, 0, 0.771341, -0.636423, 0, 0.778975, -0.627055, 0, 0.778975, -0.627055, 0, 0.778975, -0.627055, 0, 0.778975, -0.627055, 0, 0.778975, -0.627055, 0, 0.778975, -0.627055, 0, 0.789481, -0.613775, 0, 0.789481, -0.613775, 0, 0.789481, -0.613775, 0, 0.789481, -0.613775, 0, 0.789481, -0.613775, 0, 0.789481, -0.613775, 0, 0.796369, -0.604811, 0, 0.796369, -0.604811, 0, 0.796369, -0.604811, 0, 0.796369, -0.604811, 0, 0.796369, -0.604811, 0, 0.796369, -0.604811, 0, 0.803353, -0.595503, 0, 0.803353, -0.595503, 0, 0.803353, -0.595503, 0, 0.803353, -0.595503, 0, 0.803353, -0.595503, 0, 0.803353, -0.595503, 0, 0.810431, -0.585834, 0, 0.810431, -0.585834, 0, 0.810431, -0.585834, 0, 0.810431, -0.585834, 0, 0.810431, -0.585834, 0, 0.810431, -0.585834, 0, 0.817599, -0.575788, 0, 0.817599, -0.575788, 0, 0.817599, -0.575788, 0, 0.817599, -0.575788, 0, 0.817599, -0.575788, 0, 0.817599, -0.575788, 0, 0.824884, -0.565301, 0, 0.824884, -0.565301, 0, 0.824884, -0.565301, 0, 0.824884, -0.565301, 0, 0.824884, -0.565301, 0, 0.824884, -0.565301, 0, 0.832308, -0.554314, 0, 0.832308, -0.554314, 0, 0.832308, -0.554314, 0, 0.832308, -0.554314, 0, 0.832308, -0.554314, 0, 0.832308, -0.554314, 0, 0.839854, -0.542813, 0, 0.839854, -0.542813, 0, 0.839854, -0.542813, 0, 0.839854, -0.542813, 0, 0.839854, -0.542813, 0, 0.839854, -0.542813, 0, 0.847506, -0.530786, 0, 0.847506, -0.530786, 0, 0.847506, -0.530786, 0, 0.847506, -0.530786, 0, 0.847506, -0.530786, 0, 0.847506, -0.530786, 0, 0.824859, -0.565339, 0, 0.824859, -0.565339, 0, 0.824859, -0.565339, 0, 0.824859, -0.565339, 0, 0.824859, -0.565339, 0, 0.824859, -0.565339, 0, 0.816718, -0.577037, 0, 0.816718, -0.577037, 0, 0.816718, -0.577037, 0, 0.816718, -0.577037, 0, 0.816718, -0.577037, 0, 0.816718, -0.577037, 0, 0.808614, -0.588340, 0, 0.808614, -0.588340, 0, 0.808614, -0.588340, 0, 0.808614, -0.588340, 0, 0.808614, -0.588340, 0, 0.808614, -0.588340, 0, 0.800552, -0.599264, 0, 0.800552, -0.599264, 0, 0.800552, -0.599264, 0, 0.800552, -0.599264, 0, 0.800552, -0.599264, 0, 0.800552, -0.599264, 0, 0.792536, -0.609825, 0, 0.792536, -0.609825, 0, 0.792536, -0.609825, 0, 0.792536, -0.609825, 0, 0.792536, -0.609825, 0, 0.792536, -0.609825, 0, 0.784605, -0.619997, 0, 0.784605, -0.619997, 0, 0.784605, -0.619997, 0, 0.784605, -0.619997, 0, 0.784605, -0.619997, 0, 0.784605, -0.619997, 0, 0.776799, -0.629749, 0, 0.776799, -0.629749, 0, 0.776799, -0.629749, 0, 0.776799, -0.629749, 0, 0.776799, -0.629749, 0, 0.776799, -0.629749, 0, 0.769127, -0.639096, 0, 0.769127, -0.639096, 0, 0.769127, -0.639096, 0, 0.769127, -0.639096, 0, 0.769127, -0.639096, 0, 0.769127, -0.639096, 0, 0.761595, -0.648053, 0, 0.761595, -0.648053, 0, 0.761595, -0.648053, 0, 0.761595, -0.648053, 0, 0.761595, -0.648053, 0, 0.761595, -0.648053, 0, 0.744674, -0.667428, 0, 0.744674, -0.667428, 0, 0.744674, -0.667428, 0, 0.744674, -0.667428, 0, 0.744674, -0.667428, 0, 0.744674, -0.667428, 0, 0.731801, -0.681518, 0, 0.731801, -0.681518, 0, 0.731801, -0.681518, 0, 0.731801, -0.681518, 0, 0.731801, -0.681518, 0, 0.731801, -0.681518, 0, 0.723759, -0.690053, 0, 0.723759, -0.690053, 0, 0.723759, -0.690053, 0, 0.723759, -0.690053, 0, 0.723759, -0.690053, 0, 0.723759, -0.690053, 0, 0.721333, -0.692588, 0, 0.721333, -0.692588, 0, 0.721333, -0.692588, 0, 0.721333, -0.692588, 0, 0.721333, -0.692588, 0, 0.721333, -0.692588, 0, 0.725284, -0.688449, 0, 0.725284, -0.688449, 0, 0.725284, -0.688449, 0, 0.725284, -0.688449, 0, 0.725284, -0.688449, 0, 0.725284, -0.688449, 0, 0.736288, -0.676668, 0, 0.736288, -0.676668, 0, 0.736288, -0.676668, 0, 0.736288, -0.676668, 0, 0.736288, -0.676668, 0, 0.736288, -0.676668, 0, 0.754824, -0.655927, 0, 0.754824, -0.655927, 0, 0.754824, -0.655927, 0, 0.754824, -0.655927, 0, 0.754824, -0.655927, 0, 0.754824, -0.655927, 0, 0.780958, -0.624584, 0, 0.780958, -0.624584, 0, 0.780958, -0.624584, 0, 0.780958, -0.624584, 0, 0.780958, -0.624584, 0, 0.780958, -0.624584, 0, 0.811844, -0.583874, 0, 0.811844, -0.583874, 0, 0.811844, -0.583874, 0, 0.811844, -0.583874, 0, 0.811844, -0.583874, 0, 0.811844, -0.583874, 0, 0.841934, -0.539581, 0, 0.841934, -0.539581, 0, 0.841934, -0.539581, 0, 0.841934, -0.539581, 0, 0.841934, -0.539581, 0, 0.841934, -0.539581, 0, 0.869841, -0.493332, 0, 0.869841, -0.493332, 0, 0.869841, -0.493332, 0, 0.869841, -0.493332, 0, 0.869841, -0.493332, 0, 0.869841, -0.493332, 0, 0.894475, -0.447118, 0, 0.894475, -0.447118, 0, 0.894475, -0.447118, 0, 0.894475, -0.447118, 0, 0.894475, -0.447118, 0, 0.894475, -0.447118, 0, 0.914475, -0.404643, 0, 0.914475, -0.404643, 0, 0.914475, -0.404643, 0, 0.914475, -0.404643, 0, 0.914475, -0.404643, 0, 0.914475, -0.404643, 0, 0.929375, -0.369137, 0, 0.929375, -0.369137, 0, 0.929375, -0.369137, 0, 0.929375, -0.369137, 0, 0.929375, -0.369137, 0, 0.929375, -0.369137, 0, 0.939923, -0.341385, 0, 0.939923, -0.341385, 0, 0.939923, -0.341385, 0, 0.939923, -0.341385, 0, 0.939923, -0.341385, 0, 0.939923, -0.341385, 0, 0.946911, -0.321497, 0, 0.946911, -0.321497, 0, 0.946911, -0.321497, 0, 0.946911, -0.321497, 0, 0.946911, -0.321497, 0, 0.946911, -0.321497, 0, 0.950191, -0.311669, 0, 0.950191, -0.311669, 0, 0.950191, -0.311669, 0, 0.950191, -0.311669, 0, 0.950191, -0.311669, 0, 0.950191, -0.311669, 0, 0.950649, -0.310269, 0, 0.950649, -0.310269, 0, 0.950649, -0.310269, 0, 0.950649, -0.310269, 0, 0.950649, -0.310269, 0, 0.950649, -0.310269, 0, 0.948636, -0.316369, 0, 0.948636, -0.316369, 0, 0.948636, -0.316369, 0, 0.948636, -0.316369, 0, 0.948636, -0.316369, 0, 0.948636, -0.316369, 0, 0.944260, -0.329202, 0, 0.944260, -0.329202, 0, 0.944260, -0.329202, 0, 0.944260, -0.329202, 0, 0.944260, -0.329202, 0, 0.944260, -0.329202, 0, 0.937646, -0.347591, 0, 0.937646, -0.347591, 0, 0.937646, -0.347591, 0, 0.937646, -0.347591, 0, 0.937646, -0.347591, 0, 0.937646, -0.347591, 0, 0.928793, -0.370599, 0, 0.928793, -0.370599, 0, 0.928793, -0.370599, 0, 0.928793, -0.370599, 0, 0.928793, -0.370599, 0, 0.928793, -0.370599, 0, 0.917406, -0.397953, 0, 0.917406, -0.397953, 0, 0.917406, -0.397953, 0, 0.917406, -0.397953, 0, 0.917406, -0.397953, 0, 0.917406, -0.397953, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.908404, -0.418094, 0, 0.908404, -0.418094, 0, 0.908404, -0.418094, 0, 0.908404, -0.418094, 0, 0.908404, -0.418094, 0, 0.908404, -0.418094, 0, 0.915712, -0.401836, 0, 0.915712, -0.401836, 0, 0.915712, -0.401836, 0, 0.915712, -0.401836, 0, 0.915712, -0.401836, 0, 0.915712, -0.401836, 0, 0.921416, -0.388577, 0, 0.921416, -0.388577, 0, 0.921416, -0.388577, 0, 0.921416, -0.388577, 0, 0.921416, -0.388577, 0, 0.921416, -0.388577, 0, 0.925621, -0.378451, 0, 0.925621, -0.378451, 0, 0.925621, -0.378451, 0, 0.925621, -0.378451, 0, 0.925621, -0.378451, 0, 0.925621, -0.378451, 0, 0.928244, -0.371973, 0, 0.928244, -0.371973, 0, 0.928244, -0.371973, 0, 0.928244, -0.371973, 0, 0.928244, -0.371973, 0, 0.928244, -0.371973, 0, 0.929133, -0.369746, 0, 0.929133, -0.369746, 0, 0.929133, -0.369746, 0, 0.929133, -0.369746, 0, 0.929133, -0.369746, 0, 0.929133, -0.369746, 0, 0.928205, -0.372069, 0, 0.928205, -0.372069, 0, 0.928205, -0.372069, 0, 0.928205, -0.372069, 0, 0.928205, -0.372069, 0, 0.928205, -0.372069, 0, 0.925286, -0.379269, 0, 0.925286, -0.379269, 0, 0.925286, -0.379269, 0, 0.925286, -0.379269, 0, 0.925286, -0.379269, 0, 0.925286, -0.379269, 0, 0.920007, -0.391902, 0, 0.920007, -0.391902, 0, 0.920007, -0.391902, 0, 0.920007, -0.391902, 0, 0.920007, -0.391902, 0, 0.920007, -0.391902, 0, 0.913110, -0.407712, 0, 0.913110, -0.407712, 0, 0.913110, -0.407712, 0, 0.913110, -0.407712, 0, 0.913110, -0.407712, 0, 0.913110, -0.407712, 0, 0.904437, -0.426608, 0, 0.904437, -0.426608, 0, 0.904437, -0.426608, 0, 0.904437, -0.426608, 0, 0.904437, -0.426608, 0, 0.904437, -0.426608, 0, 0.893851, -0.448365, 0, 0.893851, -0.448365, 0, 0.893851, -0.448365, 0, 0.893851, -0.448365, 0, 0.893851, -0.448365, 0, 0.893851, -0.448365, 0, 0.881544, -0.472103, 0, 0.881544, -0.472103, 0, 0.881544, -0.472103, 0, 0.881544, -0.472103, 0, 0.881544, -0.472103, 0, 0.881544, -0.472103, 0, 0.867852, -0.496824, 0, 0.867852, -0.496824, 0, 0.867852, -0.496824, 0, 0.867852, -0.496824, 0, 0.867852, -0.496824, 0, 0.867852, -0.496824, 0, 0.852946, -0.522000, 0, 0.852946, -0.522000, 0, 0.852946, -0.522000, 0, 0.852946, -0.522000, 0, 0.852946, -0.522000, 0, 0.852946, -0.522000, 0, 0.837070, -0.547096, 0, 0.837070, -0.547096, 0, 0.837070, -0.547096, 0, 0.837070, -0.547096, 0, 0.837070, -0.547096, 0, 0.837070, -0.547096, 0, 0.900331, -0.435205, 0, 0.900331, -0.435205, 0, 0.900331, -0.435205, 0, 0.900331, -0.435205, 0, 0.900331, -0.435205, 0, 0.900331, -0.435205, 0, 0.891965, -0.452105, 0, 0.891965, -0.452105, 0, 0.891965, -0.452105, 0, 0.891965, -0.452105, 0, 0.891965, -0.452105, 0, 0.891965, -0.452105, 0, 0.950376, -0.311105, 0, 0.950376, -0.311105, 0, 0.950376, -0.311105, 0, 0.950376, -0.311105, 0, 0.950376, -0.311105, 0, 0.950376, -0.311105, 0, 0.955614, -0.294622, 0, 0.955614, -0.294622, 0, 0.955614, -0.294622, 0, 0.955614, -0.294622, 0, 0.955614, -0.294622, 0, 0.955614, -0.294622, 0, 0.959803, -0.280674, 0, 0.959803, -0.280674, 0, 0.959803, -0.280674, 0, 0.959803, -0.280674, 0, 0.959803, -0.280674, 0, 0.959803, -0.280674, 0, 0.963052, -0.269314, 0, 0.963052, -0.269314, 0, 0.963052, -0.269314, 0, 0.963052, -0.269314, 0, 0.963052, -0.269314, 0, 0.963052, -0.269314, 0, 0.965447, -0.260601, 0, 0.965447, -0.260601, 0, 0.965447, -0.260601, 0, 0.965447, -0.260601, 0, 0.965447, -0.260601, 0, 0.965447, -0.260601, 0, 0.967050, -0.254588, 0, 0.967050, -0.254588, 0, 0.967050, -0.254588, 0, 0.967050, -0.254588, 0, 0.967050, -0.254588, 0, 0.967050, -0.254588, 0, 0.967898, -0.251343, 0, 0.967898, -0.251343, 0, 0.967898, -0.251343, 0, 0.967898, -0.251343, 0, 0.967898, -0.251343, 0, 0.967898, -0.251343, 0, 0.967996, -0.250965, 0, 0.967996, -0.250965, 0, 0.967996, -0.250965, 0, 0.967996, -0.250965, 0, 0.967996, -0.250965, 0, 0.967996, -0.250965, 0, 0.967315, -0.253579, 0, 0.967315, -0.253579, 0, 0.967315, -0.253579, 0, 0.967315, -0.253579, 0, 0.967315, -0.253579, 0, 0.967315, -0.253579, 0, 0.965631, -0.259916, 0, 0.965631, -0.259916, 0, 0.965631, -0.259916, 0, 0.965631, -0.259916, 0, 0.965631, -0.259916, 0, 0.965631, -0.259916, 0, 0.963144, -0.268987, 0, 0.963144, -0.268987, 0, 0.963144, -0.268987, 0, 0.963144, -0.268987, 0, 0.963144, -0.268987, 0, 0.963144, -0.268987, 0, 0.959762, -0.280814, 0, 0.959762, -0.280814, 0, 0.959762, -0.280814, 0, 0.959762, -0.280814, 0, 0.959762, -0.280814, 0, 0.959762, -0.280814, 0, 0.955386, -0.295360, 0, 0.955386, -0.295360, 0, 0.955386, -0.295360, 0, 0.955386, -0.295360, 0, 0.955386, -0.295360, 0, 0.955386, -0.295360, 0, 0.950090, -0.311976, 0, 0.950090, -0.311976, 0, 0.950090, -0.311976, 0, 0.950090, -0.311976, 0, 0.950090, -0.311976, 0, 0.950090, -0.311976, 0, 0.944005, -0.329932, 0, 0.944005, -0.329932, 0, 0.944005, -0.329932, 0, 0.944005, -0.329932, 0, 0.944005, -0.329932, 0, 0.944005, -0.329932, 0, 0.937127, -0.348989, 0, 0.937127, -0.348989, 0, 0.937127, -0.348989, 0, 0.937127, -0.348989, 0, 0.937127, -0.348989, 0, 0.937127, -0.348989, 0, 0.929479, -0.368875, 0, 0.929479, -0.368875, 0, 0.929479, -0.368875, 0, 0.929479, -0.368875, 0, 0.929479, -0.368875, 0, 0.929479, -0.368875, 0, 0.858832, -0.512258, 0, 0.858832, -0.512258, 0, 0.858832, -0.512258, 0, 0.858832, -0.512258, 0, 0.858832, -0.512258, 0, 0.858832, -0.512258, 0, 0.845012, -0.534747, 0, 0.845012, -0.534747, 0, 0.845012, -0.534747, 0, 0.845012, -0.534747, 0, 0.845012, -0.534747, 0, 0.845012, -0.534747, 0, 0.966951, -0.254964, 0, 0.966951, -0.254964, 0, 0.966951, -0.254964, 0, 0.966951, -0.254964, 0, 0.966951, -0.254964, 0, 0.966951, -0.254964, 0, 0.970479, -0.241188, 0, 0.970479, -0.241188, 0, 0.970479, -0.241188, 0, 0.970479, -0.241188, 0, 0.970479, -0.241188, 0, 0.970479, -0.241188, 0, 0.958292, -0.285791, 0, 0.958292, -0.285791, 0, 0.958292, -0.285791, 0, 0.958292, -0.285791, 0, 0.958292, -0.285791, 0, 0.958292, -0.285791, 0, 0.962670, -0.270679, 0, 0.962670, -0.270679, 0, 0.962670, -0.270679, 0, 0.962670, -0.270679, 0, 0.962670, -0.270679, 0, 0.962670, -0.270679, 0, 0.966372, -0.257147, 0, 0.966372, -0.257147, 0, 0.966372, -0.257147, 0, 0.966372, -0.257147, 0, 0.966372, -0.257147, 0, 0.966372, -0.257147, 0, 0.969463, -0.245237, 0, 0.969463, -0.245237, 0, 0.969463, -0.245237, 0, 0.969463, -0.245237, 0, 0.969463, -0.245237, 0, 0.969463, -0.245237, 0, 0.972002, -0.234971, 0, 0.972002, -0.234971, 0, 0.972002, -0.234971, 0, 0.972002, -0.234971, 0, 0.972002, -0.234971, 0, 0.972002, -0.234971, 0, 0.974053, -0.226320, 0, 0.974053, -0.226320, 0, 0.974053, -0.226320, 0, 0.974053, -0.226320, 0, 0.974053, -0.226320, 0, 0.974053, -0.226320, 0, 0.975671, -0.219242, 0, 0.975671, -0.219242, 0, 0.975671, -0.219242, 0, 0.975671, -0.219242, 0, 0.975671, -0.219242, 0, 0.975671, -0.219242, 0, 0.976896, -0.213713, 0, 0.976896, -0.213713, 0, 0.976896, -0.213713, 0, 0.976896, -0.213713, 0, 0.976896, -0.213713, 0, 0.976896, -0.213713, 0, 0.977766, -0.209697, 0, 0.977766, -0.209697, 0, 0.977766, -0.209697, 0, 0.977766, -0.209697, 0, 0.977766, -0.209697, 0, 0.977766, -0.209697, 0, 0.978420, -0.206627, 0, 0.978420, -0.206627, 0, 0.978420, -0.206627, 0, 0.978420, -0.206627, 0, 0.978420, -0.206627, 0, 0.978420, -0.206627, 0, 0.978128, -0.208004, 0, 0.978128, -0.208004, 0, 0.978128, -0.208004, 0, 0.978128, -0.208004, 0, 0.978128, -0.208004, 0, 0.978128, -0.208004, 0, 0.976932, -0.213553, 0, 0.976932, -0.213553, 0, 0.976932, -0.213553, 0, 0.976932, -0.213553, 0, 0.976932, -0.213553, 0, 0.976932, -0.213553, 0, 0.974804, -0.223062, 0, 0.974804, -0.223062, 0, 0.974804, -0.223062, 0, 0.974804, -0.223062, 0, 0.974804, -0.223062, 0, 0.974804, -0.223062, 0, 0.971684, -0.236284, 0, 0.971684, -0.236284, 0, 0.971684, -0.236284, 0, 0.971684, -0.236284, 0, 0.971684, -0.236284, 0, 0.971684, -0.236284, 0, 0.967462, -0.253017, 0, 0.967462, -0.253017, 0, 0.967462, -0.253017, 0, 0.967462, -0.253017, 0, 0.967462, -0.253017, 0, 0.967462, -0.253017, 0, 0.961964, -0.273176, 0, 0.961964, -0.273176, 0, 0.961964, -0.273176, 0, 0.961964, -0.273176, 0, 0.961964, -0.273176, 0, 0.961964, -0.273176, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.966609, -0.256258, 0, 0.966609, -0.256258, 0, 0.966609, -0.256258, 0, 0.966609, -0.256258, 0, 0.966609, -0.256258, 0, 0.966609, -0.256258, 0, 0.953705, -0.300743, 0, 0.953705, -0.300743, 0, 0.953705, -0.300743, 0, 0.953705, -0.300743, 0, 0.953705, -0.300743, 0, 0.953705, -0.300743, 0, 0.948432, -0.316981, 0, 0.948432, -0.316981, 0, 0.948432, -0.316981, 0, 0.948432, -0.316981, 0, 0.948432, -0.316981, 0, 0.948432, -0.316981, 0, 0.943968, -0.330036, 0, 0.943968, -0.330036, 0, 0.943968, -0.330036, 0, 0.943968, -0.330036, 0, 0.943968, -0.330036, 0, 0.943968, -0.330036, 0, 0.940434, -0.339975, 0, 0.940434, -0.339975, 0, 0.940434, -0.339975, 0, 0.940434, -0.339975, 0, 0.940434, -0.339975, 0, 0.940434, -0.339975, 0, 0.937906, -0.346889, 0, 0.937906, -0.346889, 0, 0.937906, -0.346889, 0, 0.937906, -0.346889, 0, 0.937906, -0.346889, 0, 0.937906, -0.346889, 0, 0.936441, -0.350825, 0, 0.936441, -0.350825, 0, 0.936441, -0.350825, 0, 0.936441, -0.350825, 0, 0.936441, -0.350825, 0, 0.936441, -0.350825, 0, 0.936058, -0.351846, 0, 0.936058, -0.351846, 0, 0.936058, -0.351846, 0, 0.936058, -0.351846, 0, 0.936058, -0.351846, 0, 0.936058, -0.351846, 0, 0.936721, -0.350077, 0, 0.936721, -0.350077, 0, 0.936721, -0.350077, 0, 0.936721, -0.350077, 0, 0.936721, -0.350077, 0, 0.936721, -0.350077, 0, 0.938366, -0.345643, 0, 0.938366, -0.345643, 0, 0.938366, -0.345643, 0, 0.938366, -0.345643, 0, 0.938366, -0.345643, 0, 0.938366, -0.345643, 0, 0.939653, -0.342129, 0, 0.939653, -0.342129, 0, 0.939653, -0.342129, 0, 0.939653, -0.342129, 0, 0.939653, -0.342129, 0, 0.939653, -0.342129, 0, 0.941067, -0.338221, 0, 0.941067, -0.338221, 0, 0.941067, -0.338221, 0, 0.941067, -0.338221, 0, 0.941067, -0.338221, 0, 0.941067, -0.338221, 0, 0.942600, -0.333924, 0, 0.942600, -0.333924, 0, 0.942600, -0.333924, 0, 0.942600, -0.333924, 0, 0.942600, -0.333924, 0, 0.942600, -0.333924, 0, 0.944245, -0.329243, 0, 0.944245, -0.329243, 0, 0.944245, -0.329243, 0, 0.944245, -0.329243, 0, 0.944245, -0.329243, 0, 0.944245, -0.329243, 0, 0.946002, -0.324162, 0, 0.946002, -0.324162, 0, 0.946002, -0.324162, 0, 0.946002, -0.324162, 0, 0.946002, -0.324162, 0, 0.946002, -0.324162, 0, 0.947867, -0.318668, 0, 0.947867, -0.318668, 0, 0.947867, -0.318668, 0, 0.947867, -0.318668, 0, 0.947867, -0.318668, 0, 0.947867, -0.318668, 0, 0.949831, -0.312764, 0, 0.949831, -0.312764, 0, 0.949831, -0.312764, 0, 0.949831, -0.312764, 0, 0.949831, -0.312764, 0, 0.949831, -0.312764, 0, 0.951886, -0.306454, 0, 0.951886, -0.306454, 0, 0.951886, -0.306454, 0, 0.951886, -0.306454, 0, 0.951886, -0.306454, 0, 0.951886, -0.306454, 0, 0.943175, -0.332296, 0, 0.943175, -0.332296, 0, 0.943175, -0.332296, 0, 0.943175, -0.332296, 0, 0.943175, -0.332296, 0, 0.943175, -0.332296, 0, 0.940913, -0.338647, 0, 0.940913, -0.338647, 0, 0.940913, -0.338647, 0, 0.940913, -0.338647, 0, 0.940913, -0.338647, 0, 0.940913, -0.338647, 0, 0.938727, -0.344660, 0, 0.938727, -0.344660, 0, 0.938727, -0.344660, 0, 0.938727, -0.344660, 0, 0.938727, -0.344660, 0, 0.938727, -0.344660, 0, 0.936625, -0.350335, 0, 0.936625, -0.350335, 0, 0.936625, -0.350335, 0, 0.936625, -0.350335, 0, 0.936625, -0.350335, 0, 0.936625, -0.350335, 0, 0.934612, -0.355669, 0, 0.934612, -0.355669, 0, 0.934612, -0.355669, 0, 0.934612, -0.355669, 0, 0.934612, -0.355669, 0, 0.934612, -0.355669, 0, 0.932707, -0.360634, 0, 0.932707, -0.360634, 0, 0.932707, -0.360634, 0, 0.932707, -0.360634, 0, 0.932707, -0.360634, 0, 0.932707, -0.360634, 0, 0.930930, -0.365198, 0, 0.930930, -0.365198, 0, 0.930930, -0.365198, 0, 0.930930, -0.365198, 0, 0.930930, -0.365198, 0, 0.930930, -0.365198, 0, 0.929288, -0.369356, 0, 0.929288, -0.369356, 0, 0.929288, -0.369356, 0, 0.929288, -0.369356, 0, 0.929288, -0.369356, 0, 0.929288, -0.369356, 0, 0.927790, -0.373104, 0, 0.927790, -0.373104, 0, 0.927790, -0.373104, 0, 0.927790, -0.373104, 0, 0.927790, -0.373104, 0, 0.927790, -0.373104, 0, 0.924968, -0.380045, 0, 0.924968, -0.380045, 0, 0.924968, -0.380045, 0, 0.924968, -0.380045, 0, 0.924968, -0.380045, 0, 0.924968, -0.380045, 0, 0.923765, -0.382960, 0, 0.923765, -0.382960, 0, 0.923765, -0.382960, 0, 0.923765, -0.382960, 0, 0.923765, -0.382960, 0, 0.923765, -0.382960, 0, 0.924340, -0.381570, 0, 0.924340, -0.381570, 0, 0.924340, -0.381570, 0, 0.924340, -0.381570, 0, 0.924340, -0.381570, 0, 0.924340, -0.381570, 0, 0.926793, -0.375572, 0, 0.926793, -0.375572, 0, 0.926793, -0.375572, 0, 0.926793, -0.375572, 0, 0.926793, -0.375572, 0, 0.926793, -0.375572, 0, 0.931020, -0.364968, 0, 0.931020, -0.364968, 0, 0.931020, -0.364968, 0, 0.931020, -0.364968, 0, 0.931020, -0.364968, 0, 0.931020, -0.364968, 0, 0.936814, -0.349829, 0, 0.936814, -0.349829, 0, 0.936814, -0.349829, 0, 0.936814, -0.349829, 0, 0.936814, -0.349829, 0, 0.936814, -0.349829, 0, 0.943990, -0.329975, 0, 0.943990, -0.329975, 0, 0.943990, -0.329975, 0, 0.943990, -0.329975, 0, 0.943990, -0.329975, 0, 0.943990, -0.329975, 0, 0.952253, -0.305309, 0, 0.952253, -0.305309, 0, 0.952253, -0.305309, 0, 0.952253, -0.305309, 0, 0.952253, -0.305309, 0, 0.952253, -0.305309, 0, 0.960621, -0.277861, 0, 0.960621, -0.277861, 0, 0.960621, -0.277861, 0, 0.960621, -0.277861, 0, 0.960621, -0.277861, 0, 0.960621, -0.277861, 0, 0.967768, -0.251843, 0, 0.967768, -0.251843, 0, 0.967768, -0.251843, 0, 0.967768, -0.251843, 0, 0.967768, -0.251843, 0, 0.967768, -0.251843, 0, 0.973723, -0.227735, 0, 0.973723, -0.227735, 0, 0.973723, -0.227735, 0, 0.973723, -0.227735, 0, 0.973723, -0.227735, 0, 0.973723, -0.227735, 0, 0.978565, -0.205938, 0, 0.978565, -0.205938, 0, 0.978565, -0.205938, 0, 0.978565, -0.205938, 0, 0.978565, -0.205938, 0, 0.978565, -0.205938, 0, 0.982265, -0.187500, 0, 0.982265, -0.187500, 0, 0.982265, -0.187500, 0, 0.982265, -0.187500, 0, 0.982265, -0.187500, 0, 0.982265, -0.187500, 0, 0.984872, -0.173284, 0, 0.984872, -0.173284, 0, 0.984872, -0.173284, 0, 0.984872, -0.173284, 0, 0.984872, -0.173284, 0, 0.984872, -0.173284, 0, 0.986583, -0.163258, 0, 0.986583, -0.163258, 0, 0.986583, -0.163258, 0, 0.986583, -0.163258, 0, 0.986583, -0.163258, 0, 0.986583, -0.163258, 0, 0.987553, -0.157288, 0, 0.987553, -0.157288, 0, 0.987553, -0.157288, 0, 0.987553, -0.157288, 0, 0.987553, -0.157288, 0, 0.987553, -0.157288, 0, 0.987800, -0.155728, 0, 0.987800, -0.155728, 0, 0.987800, -0.155728, 0, 0.987800, -0.155728, 0, 0.987800, -0.155728, 0, 0.987800, -0.155728, 0, 0.987428, -0.158070, 0, 0.987428, -0.158070, 0, 0.987428, -0.158070, 0, 0.987428, -0.158070, 0, 0.987428, -0.158070, 0, 0.987428, -0.158070, 0, 0.986444, -0.164098, 0, 0.986444, -0.164098, 0, 0.986444, -0.164098, 0, 0.986444, -0.164098, 0, 0.986444, -0.164098, 0, 0.986444, -0.164098, 0, 0.984809, -0.173642, 0, 0.984809, -0.173642, 0, 0.984809, -0.173642, 0, 0.984809, -0.173642, 0, 0.984809, -0.173642, 0, 0.984809, -0.173642, 0, 0.982515, -0.186182, 0, 0.982515, -0.186182, 0, 0.982515, -0.186182, 0, 0.982515, -0.186182, 0, 0.982515, -0.186182, 0, 0.982515, -0.186182, 0, 0.979534, -0.201277, 0, 0.979534, -0.201277, 0, 0.979534, -0.201277, 0, 0.979534, -0.201277, 0, 0.979534, -0.201277, 0, 0.979534, -0.201277, 0, 0.975742, -0.218924, 0, 0.975742, -0.218924, 0, 0.975742, -0.218924, 0, 0.975742, -0.218924, 0, 0.975742, -0.218924, 0, 0.975742, -0.218924, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.974519, -0.224304, 0, 0.974519, -0.224304, 0, 0.974519, -0.224304, 0, 0.974519, -0.224304, 0, 0.974519, -0.224304, 0, 0.974519, -0.224304, 0, 0.976898, -0.213708, 0, 0.976898, -0.213708, 0, 0.976898, -0.213708, 0, 0.976898, -0.213708, 0, 0.976898, -0.213708, 0, 0.976898, -0.213708, 0, 0.978822, -0.204715, 0, 0.978822, -0.204715, 0, 0.978822, -0.204715, 0, 0.978822, -0.204715, 0, 0.978822, -0.204715, 0, 0.978822, -0.204715, 0, 0.980337, -0.197331, 0, 0.980337, -0.197331, 0, 0.980337, -0.197331, 0, 0.980337, -0.197331, 0, 0.980337, -0.197331, 0, 0.980337, -0.197331, 0, 0.981426, -0.191839, 0, 0.981426, -0.191839, 0, 0.981426, -0.191839, 0, 0.981426, -0.191839, 0, 0.981426, -0.191839, 0, 0.981426, -0.191839, 0, 0.982063, -0.188556, 0, 0.982063, -0.188556, 0, 0.982063, -0.188556, 0, 0.982063, -0.188556, 0, 0.982063, -0.188556, 0, 0.982063, -0.188556, 0, 0.982255, -0.187551, 0, 0.982255, -0.187551, 0, 0.982255, -0.187551, 0, 0.982255, -0.187551, 0, 0.982255, -0.187551, 0, 0.982255, -0.187551, 0, 0.981995, -0.188908, 0, 0.981995, -0.188908, 0, 0.981995, -0.188908, 0, 0.981995, -0.188908, 0, 0.981995, -0.188908, 0, 0.981995, -0.188908, 0, 0.981247, -0.192755, 0, 0.981247, -0.192755, 0, 0.981247, -0.192755, 0, 0.981247, -0.192755, 0, 0.981247, -0.192755, 0, 0.981247, -0.192755, 0, 0.980098, -0.198514, 0, 0.980098, -0.198514, 0, 0.980098, -0.198514, 0, 0.980098, -0.198514, 0, 0.980098, -0.198514, 0, 0.980098, -0.198514, 0, 0.978506, -0.206217, 0, 0.978506, -0.206217, 0, 0.978506, -0.206217, 0, 0.978506, -0.206217, 0, 0.978506, -0.206217, 0, 0.978506, -0.206217, 0, 0.976422, -0.215869, 0, 0.976422, -0.215869, 0, 0.976422, -0.215869, 0, 0.976422, -0.215869, 0, 0.976422, -0.215869, 0, 0.976422, -0.215869, 0, 0.973847, -0.227204, 0, 0.973847, -0.227204, 0, 0.973847, -0.227204, 0, 0.973847, -0.227204, 0, 0.973847, -0.227204, 0, 0.973847, -0.227204, 0, 0.970793, -0.239917, 0, 0.970793, -0.239917, 0, 0.970793, -0.239917, 0, 0.970793, -0.239917, 0, 0.970793, -0.239917, 0, 0.970793, -0.239917, 0, 0.967228, -0.253908, 0, 0.967228, -0.253908, 0, 0.967228, -0.253908, 0, 0.967228, -0.253908, 0, 0.967228, -0.253908, 0, 0.967228, -0.253908, 0, 0.963125, -0.269055, 0, 0.963125, -0.269055, 0, 0.963125, -0.269055, 0, 0.963125, -0.269055, 0, 0.963125, -0.269055, 0, 0.963125, -0.269055, 0, 0.988533, -0.151003, 0, 0.988533, -0.151003, 0, 0.988533, -0.151003, 0, 0.988533, -0.151003, 0, 0.988533, -0.151003, 0, 0.988533, -0.151003, 0, 0.988630, -0.150368, 0, 0.988630, -0.150368, 0, 0.988630, -0.150368, 0, 0.988630, -0.150368, 0, 0.988630, -0.150368, 0, 0.988630, -0.150368, 0, 0.988699, -0.149917, 0, 0.988699, -0.149917, 0, 0.988699, -0.149917, 0, 0.988699, -0.149917, 0, 0.988699, -0.149917, 0, 0.988699, -0.149917, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988752, -0.149563, 0, 0.988752, -0.149563, 0, 0.988752, -0.149563, 0, 0.988752, -0.149563, 0, 0.988752, -0.149563, 0, 0.988752, -0.149563, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988739, -0.149650, 0, 0.988699, -0.149912, 0, 0.988699, -0.149912, 0, 0.988699, -0.149912, 0, 0.988699, -0.149912, 0, 0.988699, -0.149912, 0, 0.988699, -0.149912, 0, 0.988632, -0.150352, 0, 0.988632, -0.150352, 0, 0.988632, -0.150352, 0, 0.988632, -0.150352, 0, 0.988632, -0.150352, 0, 0.988632, -0.150352, 0, 0.988201, -0.153161, 0, 0.988201, -0.153161, 0, 0.988201, -0.153161, 0, 0.988201, -0.153161, 0, 0.988201, -0.153161, 0, 0.988201, -0.153161, 0, 0.987462, -0.157858, 0, 0.987462, -0.157858, 0, 0.987462, -0.157858, 0, 0.987462, -0.157858, 0, 0.987462, -0.157858, 0, 0.987462, -0.157858, 0, 0.986382, -0.164470, 0, 0.986382, -0.164470, 0, 0.986382, -0.164470, 0, 0.986382, -0.164470, 0, 0.986382, -0.164470, 0, 0.986382, -0.164470, 0, 0.984922, -0.173002, 0, 0.984922, -0.173002, 0, 0.984922, -0.173002, 0, 0.984922, -0.173002, 0, 0.984922, -0.173002, 0, 0.984922, -0.173002, 0, 0.983062, -0.183273, 0, 0.983062, -0.183273, 0, 0.983062, -0.183273, 0, 0.983062, -0.183273, 0, 0.983062, -0.183273, 0, 0.983062, -0.183273, 0, 0.980789, -0.195073, 0, 0.980789, -0.195073, 0, 0.980789, -0.195073, 0, 0.980789, -0.195073, 0, 0.980789, -0.195073, 0, 0.980789, -0.195073, 0, 0.978060, -0.208323, 0, 0.978060, -0.208323, 0, 0.978060, -0.208323, 0, 0.978060, -0.208323, 0, 0.978060, -0.208323, 0, 0.978060, -0.208323, 0, 0.974837, -0.222919, 0, 0.974837, -0.222919, 0, 0.974837, -0.222919, 0, 0.974837, -0.222919, 0, 0.974837, -0.222919, 0, 0.974837, -0.222919, 0, 0.948995, -0.315292, 0, 0.948995, -0.315292, 0, 0.948995, -0.315292, 0, 0.948995, -0.315292, 0, 0.948995, -0.315292, 0, 0.948995, -0.315292, 0, 0.945839, -0.324636, 0, 0.945839, -0.324636, 0, 0.945839, -0.324636, 0, 0.945839, -0.324636, 0, 0.945839, -0.324636, 0, 0.945839, -0.324636, 0, 0.989928, -0.141570, 0, 0.989928, -0.141570, 0, 0.989928, -0.141570, 0, 0.989928, -0.141570, 0, 0.989928, -0.141570, 0, 0.989928, -0.141570, 0, 0.990554, -0.137125, 0, 0.990554, -0.137125, 0, 0.990554, -0.137125, 0, 0.990554, -0.137125, 0, 0.990554, -0.137125, 0, 0.990554, -0.137125, 0, 0.986481, -0.163873, 0, 0.986481, -0.163873, 0, 0.986481, -0.163873, 0, 0.986481, -0.163873, 0, 0.986481, -0.163873, 0, 0.986481, -0.163873, 0, 0.987842, -0.155464, 0, 0.987842, -0.155464, 0, 0.987842, -0.155464, 0, 0.987842, -0.155464, 0, 0.987842, -0.155464, 0, 0.987842, -0.155464, 0, 0.988981, -0.148043, 0, 0.988981, -0.148043, 0, 0.988981, -0.148043, 0, 0.988981, -0.148043, 0, 0.988981, -0.148043, 0, 0.988981, -0.148043, 0, 0.989921, -0.141620, 0, 0.989921, -0.141620, 0, 0.989921, -0.141620, 0, 0.989921, -0.141620, 0, 0.989921, -0.141620, 0, 0.989921, -0.141620, 0, 0.990682, -0.136198, 0, 0.990682, -0.136198, 0, 0.990682, -0.136198, 0, 0.990682, -0.136198, 0, 0.990682, -0.136198, 0, 0.990682, -0.136198, 0, 0.991285, -0.131737, 0, 0.991285, -0.131737, 0, 0.991285, -0.131737, 0, 0.991285, -0.131737, 0, 0.991285, -0.131737, 0, 0.991285, -0.131737, 0, 0.991749, -0.128191, 0, 0.991749, -0.128191, 0, 0.991749, -0.128191, 0, 0.991749, -0.128191, 0, 0.991749, -0.128191, 0, 0.991749, -0.128191, 0, 0.992087, -0.125550, 0, 0.992087, -0.125550, 0, 0.992087, -0.125550, 0, 0.992087, -0.125550, 0, 0.992087, -0.125550, 0, 0.992087, -0.125550, 0, 0.992308, -0.123797, 0, 0.992308, -0.123797, 0, 0.992308, -0.123797, 0, 0.992308, -0.123797, 0, 0.992308, -0.123797, 0, 0.992308, -0.123797, 0, 0.992332, -0.123604, 0, 0.992332, -0.123604, 0, 0.992332, -0.123604, 0, 0.992332, -0.123604, 0, 0.992332, -0.123604, 0, 0.992332, -0.123604, 0, 0.992357, -0.123402, 0, 0.992357, -0.123402, 0, 0.992357, -0.123402, 0, 0.992357, -0.123402, 0, 0.992357, -0.123402, 0, 0.992357, -0.123402, 0, 0.992383, -0.123191, 0, 0.992383, -0.123191, 0, 0.992383, -0.123191, 0, 0.992383, -0.123191, 0, 0.992383, -0.123191, 0, 0.992383, -0.123191, 0, 0.992410, -0.122970, 0, 0.992410, -0.122970, 0, 0.992410, -0.122970, 0, 0.992410, -0.122970, 0, 0.992410, -0.122970, 0, 0.992410, -0.122970, 0, 0.992439, -0.122740, 0, 0.992439, -0.122740, 0, 0.992439, -0.122740, 0, 0.992439, -0.122740, 0, 0.992439, -0.122740, 0, 0.992439, -0.122740, 0, 0.992469, -0.122498, 0, 0.992469, -0.122498, 0, 0.992469, -0.122498, 0, 0.992469, -0.122498, 0, 0.992469, -0.122498, 0, 0.992469, -0.122498, 0, 0.992500, -0.122246, 0, 0.992500, -0.122246, 0, 0.992500, -0.122246, 0, 0.992500, -0.122246, 0, 0.992500, -0.122246, 0, 0.992500, -0.122246, 0, 0.992532, -0.121982, 0, 0.992532, -0.121982, 0, 0.992532, -0.121982, 0, 0.992532, -0.121982, 0, 0.992532, -0.121982, 0, 0.992532, -0.121982, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.980270, -0.197662, 0, 0.980270, -0.197662, 0, 0.980270, -0.197662, 0, 0.980270, -0.197662, 0, 0.980270, -0.197662, 0, 0.980270, -0.197662, 0, 0.979521, -0.201340, 0, 0.979521, -0.201340, 0, 0.979521, -0.201340, 0, 0.979521, -0.201340, 0, 0.979521, -0.201340, 0, 0.979521, -0.201340, 0, 0.978668, -0.205450, 0, 0.978668, -0.205450, 0, 0.978668, -0.205450, 0, 0.978668, -0.205450, 0, 0.978668, -0.205450, 0, 0.978668, -0.205450, 0, 0.977707, -0.209972, 0, 0.977707, -0.209972, 0, 0.977707, -0.209972, 0, 0.977707, -0.209972, 0, 0.977707, -0.209972, 0, 0.977707, -0.209972, 0, 0.976582, -0.215147, 0, 0.976582, -0.215147, 0, 0.976582, -0.215147, 0, 0.976582, -0.215147, 0, 0.976582, -0.215147, 0, 0.976582, -0.215147, 0, 0.975224, -0.221220, 0, 0.975224, -0.221220, 0, 0.975224, -0.221220, 0, 0.975224, -0.221220, 0, 0.975224, -0.221220, 0, 0.975224, -0.221220, 0, 0.973623, -0.228161, 0, 0.973623, -0.228161, 0, 0.973623, -0.228161, 0, 0.973623, -0.228161, 0, 0.973623, -0.228161, 0, 0.973623, -0.228161, 0, 0.971770, -0.235929, 0, 0.971770, -0.235929, 0, 0.971770, -0.235929, 0, 0.971770, -0.235929, 0, 0.971770, -0.235929, 0, 0.971770, -0.235929, 0, 0.970280, -0.241986, 0, 0.970280, -0.241986, 0, 0.970280, -0.241986, 0, 0.970280, -0.241986, 0, 0.970280, -0.241986, 0, 0.970280, -0.241986, 0, 0.969004, -0.247045, 0, 0.969004, -0.247045, 0, 0.969004, -0.247045, 0, 0.969004, -0.247045, 0, 0.969004, -0.247045, 0, 0.969004, -0.247045, 0, 0.967960, -0.251105, 0, 0.967960, -0.251105, 0, 0.967960, -0.251105, 0, 0.967960, -0.251105, 0, 0.967960, -0.251105, 0, 0.967960, -0.251105, 0, 0.967160, -0.254169, 0, 0.967160, -0.254169, 0, 0.967160, -0.254169, 0, 0.967160, -0.254169, 0, 0.967160, -0.254169, 0, 0.967160, -0.254169, 0, 0.966636, -0.256155, 0, 0.966636, -0.256155, 0, 0.966636, -0.256155, 0, 0.966636, -0.256155, 0, 0.966636, -0.256155, 0, 0.966636, -0.256155, 0, 0.966414, -0.256989, 0, 0.966414, -0.256989, 0, 0.966414, -0.256989, 0, 0.966414, -0.256989, 0, 0.966414, -0.256989, 0, 0.966414, -0.256989, 0, 0.966493, -0.256691, 0, 0.966493, -0.256691, 0, 0.966493, -0.256691, 0, 0.966493, -0.256691, 0, 0.966493, -0.256691, 0, 0.966493, -0.256691, 0, 0.966866, -0.255286, 0, 0.966866, -0.255286, 0, 0.966866, -0.255286, 0, 0.966866, -0.255286, 0, 0.966866, -0.255286, 0, 0.966866, -0.255286, 0, 0.967752, -0.251905, 0, 0.967752, -0.251905, 0, 0.967752, -0.251905, 0, 0.967752, -0.251905, 0, 0.967752, -0.251905, 0, 0.967752, -0.251905, 0, 0.968976, -0.247155, 0, 0.968976, -0.247155, 0, 0.968976, -0.247155, 0, 0.968976, -0.247155, 0, 0.968976, -0.247155, 0, 0.968976, -0.247155, 0, 0.970515, -0.241042, 0, 0.970515, -0.241042, 0, 0.970515, -0.241042, 0, 0.970515, -0.241042, 0, 0.970515, -0.241042, 0, 0.970515, -0.241042, 0, 0.972342, -0.233563, 0, 0.972342, -0.233563, 0, 0.972342, -0.233563, 0, 0.972342, -0.233563, 0, 0.972342, -0.233563, 0, 0.972342, -0.233563, 0, 0.974348, -0.225045, 0, 0.974348, -0.225045, 0, 0.974348, -0.225045, 0, 0.974348, -0.225045, 0, 0.974348, -0.225045, 0, 0.974348, -0.225045, 0, 0.976438, -0.215799, 0, 0.976438, -0.215799, 0, 0.976438, -0.215799, 0, 0.976438, -0.215799, 0, 0.976438, -0.215799, 0, 0.976438, -0.215799, 0, 0.978599, -0.205778, 0, 0.978599, -0.205778, 0, 0.978599, -0.205778, 0, 0.978599, -0.205778, 0, 0.978599, -0.205778, 0, 0.978599, -0.205778, 0, 0.980818, -0.194925, 0, 0.980818, -0.194925, 0, 0.980818, -0.194925, 0, 0.980818, -0.194925, 0, 0.980818, -0.194925, 0, 0.980818, -0.194925, 0, 0.981858, -0.189619, 0, 0.981858, -0.189619, 0, 0.981858, -0.189619, 0, 0.981858, -0.189619, 0, 0.981858, -0.189619, 0, 0.981858, -0.189619, 0, 0.967186, -0.254070, 0, 0.967186, -0.254070, 0, 0.967186, -0.254070, 0, 0.967186, -0.254070, 0, 0.967186, -0.254070, 0, 0.967186, -0.254070, 0, 0.965557, -0.260192, 0, 0.965557, -0.260192, 0, 0.965557, -0.260192, 0, 0.965557, -0.260192, 0, 0.965557, -0.260192, 0, 0.965557, -0.260192, 0, 0.970820, -0.239810, 0, 0.970820, -0.239810, 0, 0.970820, -0.239810, 0, 0.970820, -0.239810, 0, 0.970820, -0.239810, 0, 0.970820, -0.239810, 0, 0.969068, -0.246794, 0, 0.969068, -0.246794, 0, 0.969068, -0.246794, 0, 0.969068, -0.246794, 0, 0.969068, -0.246794, 0, 0.969068, -0.246794, 0, 0.967439, -0.253105, 0, 0.967439, -0.253105, 0, 0.967439, -0.253105, 0, 0.967439, -0.253105, 0, 0.967439, -0.253105, 0, 0.967439, -0.253105, 0, 0.965943, -0.258754, 0, 0.965943, -0.258754, 0, 0.965943, -0.258754, 0, 0.965943, -0.258754, 0, 0.965943, -0.258754, 0, 0.965943, -0.258754, 0, 0.964592, -0.263747, 0, 0.964592, -0.263747, 0, 0.964592, -0.263747, 0, 0.964592, -0.263747, 0, 0.964592, -0.263747, 0, 0.964592, -0.263747, 0, 0.963377, -0.268152, 0, 0.963377, -0.268152, 0, 0.963377, -0.268152, 0, 0.963377, -0.268152, 0, 0.963377, -0.268152, 0, 0.963377, -0.268152, 0, 0.962286, -0.272039, 0, 0.962286, -0.272039, 0, 0.962286, -0.272039, 0, 0.962286, -0.272039, 0, 0.962286, -0.272039, 0, 0.962286, -0.272039, 0, 0.961328, -0.275408, 0, 0.961328, -0.275408, 0, 0.961328, -0.275408, 0, 0.961328, -0.275408, 0, 0.961328, -0.275408, 0, 0.961328, -0.275408, 0, 0.960506, -0.278259, 0, 0.960506, -0.278259, 0, 0.960506, -0.278259, 0, 0.960506, -0.278259, 0, 0.960506, -0.278259, 0, 0.960506, -0.278259, 0, 0.959825, -0.280600, 0, 0.959825, -0.280600, 0, 0.959825, -0.280600, 0, 0.959825, -0.280600, 0, 0.959825, -0.280600, 0, 0.959825, -0.280600, 0, 0.959700, -0.281028, 0, 0.959700, -0.281028, 0, 0.959700, -0.281028, 0, 0.959700, -0.281028, 0, 0.959700, -0.281028, 0, 0.959700, -0.281028, 0, 0.960152, -0.279478, 0, 0.960152, -0.279478, 0, 0.960152, -0.279478, 0, 0.960152, -0.279478, 0, 0.960152, -0.279478, 0, 0.960152, -0.279478, 0, 0.961189, -0.275889, 0, 0.961189, -0.275889, 0, 0.961189, -0.275889, 0, 0.961189, -0.275889, 0, 0.961189, -0.275889, 0, 0.961189, -0.275889, 0, 0.962748, -0.270399, 0, 0.962748, -0.270399, 0, 0.962748, -0.270399, 0, 0.962748, -0.270399, 0, 0.962748, -0.270399, 0, 0.962748, -0.270399, 0, 0.964751, -0.263165, 0, 0.964751, -0.263165, 0, 0.964751, -0.263165, 0, 0.964751, -0.263165, 0, 0.964751, -0.263165, 0, 0.964751, -0.263165, 0, 0.967158, -0.254176, 0, 0.967158, -0.254176, 0, 0.967158, -0.254176, 0, 0.967158, -0.254176, 0, 0.967158, -0.254176, 0, 0.967158, -0.254176, 0, 0.969918, -0.243432, 0, 0.969918, -0.243432, 0, 0.969918, -0.243432, 0, 0.969918, -0.243432, 0, 0.969918, -0.243432, 0, 0.969918, -0.243432, 0, 0.973100, -0.230382, 0, 0.973100, -0.230382, 0, 0.973100, -0.230382, 0, 0.973100, -0.230382, 0, 0.973100, -0.230382, 0, 0.973100, -0.230382, 0, 0.975856, -0.218417, 0, 0.975856, -0.218417, 0, 0.975856, -0.218417, 0, 0.975856, -0.218417, 0, 0.975856, -0.218417, 0, 0.975856, -0.218417, 0, 0.978204, -0.207648, 0, 0.978204, -0.207648, 0, 0.978204, -0.207648, 0, 0.978204, -0.207648, 0, 0.978204, -0.207648, 0, 0.978204, -0.207648, 0, 0.980169, -0.198163, 0, 0.980169, -0.198163, 0, 0.980169, -0.198163, 0, 0.980169, -0.198163, 0, 0.980169, -0.198163, 0, 0.980169, -0.198163, 0, 0.981774, -0.190054, 0, 0.981774, -0.190054, 0, 0.981774, -0.190054, 0, 0.981774, -0.190054, 0, 0.981774, -0.190054, 0, 0.981774, -0.190054, 0, 0.983042, -0.183383, 0, 0.983042, -0.183383, 0, 0.983042, -0.183383, 0, 0.983042, -0.183383, 0, 0.983042, -0.183383, 0, 0.983042, -0.183383, 0, 0.984002, -0.178160, 0, 0.984002, -0.178160, 0, 0.984002, -0.178160, 0, 0.984002, -0.178160, 0, 0.984002, -0.178160, 0, 0.984002, -0.178160, 0, 0.984681, -0.174366, 0, 0.984681, -0.174366, 0, 0.984681, -0.174366, 0, 0.984681, -0.174366, 0, 0.984681, -0.174366, 0, 0.984681, -0.174366, 0, 0.985747, -0.168234, 0, 0.985747, -0.168234, 0, 0.985747, -0.168234, 0, 0.985747, -0.168234, 0, 0.985747, -0.168234, 0, 0.985747, -0.168234, 0, 0.986157, -0.165813, 0, 0.986157, -0.165813, 0, 0.986157, -0.165813, 0, 0.986157, -0.165813, 0, 0.986157, -0.165813, 0, 0.986157, -0.165813, 0, 0.987484, -0.157719, 0, 0.987484, -0.157719, 0, 0.987484, -0.157719, 0, 0.987484, -0.157719, 0, 0.987484, -0.157719, 0, 0.987484, -0.157719, 0, 0.988566, -0.150790, 0, 0.988566, -0.150790, 0, 0.988566, -0.150790, 0, 0.988566, -0.150790, 0, 0.988566, -0.150790, 0, 0.988566, -0.150790, 0, 0.989429, -0.145018, 0, 0.989429, -0.145018, 0, 0.989429, -0.145018, 0, 0.989429, -0.145018, 0, 0.989429, -0.145018, 0, 0.989429, -0.145018, 0, 0.990095, -0.140398, 0, 0.990095, -0.140398, 0, 0.990095, -0.140398, 0, 0.990095, -0.140398, 0, 0.990095, -0.140398, 0, 0.990095, -0.140398, 0, 0.990568, -0.137020, 0, 0.990568, -0.137020, 0, 0.990568, -0.137020, 0, 0.990568, -0.137020, 0, 0.990568, -0.137020, 0, 0.990568, -0.137020, 0, 0.990847, -0.134988, 0, 0.990847, -0.134988, 0, 0.990847, -0.134988, 0, 0.990847, -0.134988, 0, 0.990847, -0.134988, 0, 0.990847, -0.134988, 0, 0.990937, -0.134329, 0, 0.990937, -0.134329, 0, 0.990937, -0.134329, 0, 0.990937, -0.134329, 0, 0.990937, -0.134329, 0, 0.990937, -0.134329, 0, 0.990835, -0.135081, 0, 0.990835, -0.135081, 0, 0.990835, -0.135081, 0, 0.990835, -0.135081, 0, 0.990835, -0.135081, 0, 0.990835, -0.135081, 0, 0.990677, -0.136232, 0, 0.990677, -0.136232, 0, 0.990677, -0.136232, 0, 0.990677, -0.136232, 0, 0.990677, -0.136232, 0, 0.990677, -0.136232, 0, 0.990457, -0.137821, 0, 0.990457, -0.137821, 0, 0.990457, -0.137821, 0, 0.990457, -0.137821, 0, 0.990457, -0.137821, 0, 0.990457, -0.137821, 0, 0.990172, -0.139855, 0, 0.990172, -0.139855, 0, 0.990172, -0.139855, 0, 0.990172, -0.139855, 0, 0.990172, -0.139855, 0, 0.990172, -0.139855, 0, 0.989818, -0.142342, 0, 0.989818, -0.142342, 0, 0.989818, -0.142342, 0, 0.989818, -0.142342, 0, 0.989818, -0.142342, 0, 0.989818, -0.142342, 0, 0.989389, -0.145287, 0, 0.989389, -0.145287, 0, 0.989389, -0.145287, 0, 0.989389, -0.145287, 0, 0.989389, -0.145287, 0, 0.989389, -0.145287, 0, 0.988883, -0.148694, 0, 0.988883, -0.148694, 0, 0.988883, -0.148694, 0, 0.988883, -0.148694, 0, 0.988883, -0.148694, 0, 0.988883, -0.148694, 0, 0.988293, -0.152568, 0, 0.988293, -0.152568, 0, 0.988293, -0.152568, 0, 0.988293, -0.152568, 0, 0.988293, -0.152568, 0, 0.988293, -0.152568, 0, 0.987612, -0.156915, 0, 0.987612, -0.156915, 0, 0.987612, -0.156915, 0, 0.987612, -0.156915, 0, 0.987612, -0.156915, 0, 0.987612, -0.156915, 0, 0.991134, -0.132869, 0, 0.991134, -0.132869, 0, 0.991134, -0.132869, 0, 0.991134, -0.132869, 0, 0.991134, -0.132869, 0, 0.991134, -0.132869, 0, 0.991711, -0.128485, 0, 0.991711, -0.128485, 0, 0.991711, -0.128485, 0, 0.991711, -0.128485, 0, 0.991711, -0.128485, 0, 0.991711, -0.128485, 0, 0.992204, -0.124622, 0, 0.992204, -0.124622, 0, 0.992204, -0.124622, 0, 0.992204, -0.124622, 0, 0.992204, -0.124622, 0, 0.992204, -0.124622, 0, 0.992619, -0.121271, 0, 0.992619, -0.121271, 0, 0.992619, -0.121271, 0, 0.992619, -0.121271, 0, 0.992619, -0.121271, 0, 0.992619, -0.121271, 0, 0.992964, -0.118420, 0, 0.992964, -0.118420, 0, 0.992964, -0.118420, 0, 0.992964, -0.118420, 0, 0.992964, -0.118420, 0, 0.992964, -0.118420, 0, 0.993242, -0.116058, 0, 0.993242, -0.116058, 0, 0.993242, -0.116058, 0, 0.993242, -0.116058, 0, 0.993242, -0.116058, 0, 0.993242, -0.116058, 0, 0.993461, -0.114175, 0, 0.993461, -0.114175, 0, 0.993461, -0.114175, 0, 0.993461, -0.114175, 0, 0.993461, -0.114175, 0, 0.993461, -0.114175, 0, 0.993622, -0.112760, 0, 0.993622, -0.112760, 0, 0.993622, -0.112760, 0, 0.993622, -0.112760, 0, 0.993622, -0.112760, 0, 0.993622, -0.112760, 0, 0.993730, -0.111806, 0, 0.993730, -0.111806, 0, 0.993730, -0.111806, 0, 0.993730, -0.111806, 0, 0.993730, -0.111806, 0, 0.993730, -0.111806, 0, 0.993805, -0.111141, 0, 0.993805, -0.111141, 0, 0.993805, -0.111141, 0, 0.993805, -0.111141, 0, 0.993805, -0.111141, 0, 0.993805, -0.111141, 0, 0.993595, -0.112998, 0, 0.993595, -0.112998, 0, 0.993595, -0.112998, 0, 0.993595, -0.112998, 0, 0.993595, -0.112998, 0, 0.993595, -0.112998, 0, 0.993099, -0.117279, 0, 0.993099, -0.117279, 0, 0.993099, -0.117279, 0, 0.993099, -0.117279, 0, 0.993099, -0.117279, 0, 0.993099, -0.117279, 0, 0.992293, -0.123916, 0, 0.992293, -0.123916, 0, 0.992293, -0.123916, 0, 0.992293, -0.123916, 0, 0.992293, -0.123916, 0, 0.992293, -0.123916, 0, 0.991129, -0.132901, 0, 0.991129, -0.132901, 0, 0.991129, -0.132901, 0, 0.991129, -0.132901, 0, 0.991129, -0.132901, 0, 0.991129, -0.132901, 0, 0.989542, -0.144246, 0, 0.989542, -0.144246, 0, 0.989542, -0.144246, 0, 0.989542, -0.144246, 0, 0.989542, -0.144246, 0, 0.989542, -0.144246, 0, 0.987448, -0.157944, 0, 0.987448, -0.157944, 0, 0.987448, -0.157944, 0, 0.987448, -0.157944, 0, 0.987448, -0.157944, 0, 0.987448, -0.157944, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999000, -0.044710, 0, 0.999000, -0.044710, 0, 0.999000, -0.044710, 0, 0.999000, -0.044710, 0, 0.999000, -0.044710, 0, 0.999000, -0.044710, 0, 0.971472, -0.237155, 0, 0.971472, -0.237155, 0, 0.971472, -0.237155, 0, 0.971472, -0.237155, 0, 0.971472, -0.237155, 0, 0.971472, -0.237155, 0, 0.970150, -0.242506, 0, 0.970150, -0.242506, 0, 0.970150, -0.242506, 0, 0.970150, -0.242506, 0, 0.970150, -0.242506, 0, 0.970150, -0.242506, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.992094, -0.125496, 0, 0.992094, -0.125496, 0, 0.992094, -0.125496, 0, 0.992094, -0.125496, 0, 0.992094, -0.125496, 0, 0.992094, -0.125496, 0, 0.992513, -0.122142, 0, 0.992513, -0.122142, 0, 0.992513, -0.122142, 0, 0.992513, -0.122142, 0, 0.992513, -0.122142, 0, 0.992513, -0.122142, 0, 0.992676, -0.120807, 0, 0.992676, -0.120807, 0, 0.992676, -0.120807, 0, 0.992676, -0.120807, 0, 0.992676, -0.120807, 0, 0.992676, -0.120807, 0, 0.992940, -0.118622, 0, 0.992940, -0.118622, 0, 0.992940, -0.118622, 0, 0.992940, -0.118622, 0, 0.992940, -0.118622, 0, 0.992940, -0.118622, 0, 0.993296, -0.115595, 0, 0.993296, -0.115595, 0, 0.993296, -0.115595, 0, 0.993296, -0.115595, 0, 0.993296, -0.115595, 0, 0.993296, -0.115595, 0, 0.993739, -0.111729, 0, 0.993739, -0.111729, 0, 0.993739, -0.111729, 0, 0.993739, -0.111729, 0, 0.993739, -0.111729, 0, 0.993739, -0.111729, 0, 0.994248, -0.107106, 0, 0.994248, -0.107106, 0, 0.994248, -0.107106, 0, 0.994248, -0.107106, 0, 0.994248, -0.107106, 0, 0.994248, -0.107106, 0, 0.994805, -0.101798, 0, 0.994805, -0.101798, 0, 0.994805, -0.101798, 0, 0.994805, -0.101798, 0, 0.994805, -0.101798, 0, 0.994805, -0.101798, 0, 0.995401, -0.095792, 0, 0.995401, -0.095792, 0, 0.995401, -0.095792, 0, 0.995401, -0.095792, 0, 0.995401, -0.095792, 0, 0.995401, -0.095792, 0, 0.996025, -0.089071, 0, 0.996025, -0.089071, 0, 0.996025, -0.089071, 0, 0.996025, -0.089071, 0, 0.996025, -0.089071, 0, 0.996025, -0.089071, 0, 0.996594, -0.082470, 0, 0.996594, -0.082470, 0, 0.996594, -0.082470, 0, 0.996594, -0.082470, 0, 0.996594, -0.082470, 0, 0.996594, -0.082470, 0, 0.997052, -0.076731, 0, 0.997052, -0.076731, 0, 0.997052, -0.076731, 0, 0.997052, -0.076731, 0, 0.997052, -0.076731, 0, 0.997052, -0.076731, 0, 0.997416, -0.071844, 0, 0.997416, -0.071844, 0, 0.997416, -0.071844, 0, 0.997416, -0.071844, 0, 0.997416, -0.071844, 0, 0.997416, -0.071844, 0, 0.997699, -0.067805, 0, 0.997699, -0.067805, 0, 0.997699, -0.067805, 0, 0.997699, -0.067805, 0, 0.997699, -0.067805, 0, 0.997699, -0.067805, 0, 0.997906, -0.064688, 0, 0.997906, -0.064688, 0, 0.997906, -0.064688, 0, 0.997906, -0.064688, 0, 0.997906, -0.064688, 0, 0.997906, -0.064688, 0, 0.998040, -0.062575, 0, 0.998040, -0.062575, 0, 0.998040, -0.062575, 0, 0.998040, -0.062575, 0, 0.998040, -0.062575, 0, 0.998040, -0.062575, 0, 0.998108, -0.061479, 0, 0.998108, -0.061479, 0, 0.998108, -0.061479, 0, 0.998108, -0.061479, 0, 0.998108, -0.061479, 0, 0.998108, -0.061479, 0, 0.998112, -0.061416, 0, 0.998112, -0.061416, 0, 0.998112, -0.061416, 0, 0.998112, -0.061416, 0, 0.998112, -0.061416, 0, 0.998112, -0.061416, 0, 0.999014, -0.044396, 0, 0.999014, -0.044396, 0, 0.999014, -0.044396, 0, 0.999014, -0.044396, 0, 0.999014, -0.044396, 0, 0.999014, -0.044396, 0, 0.999005, -0.044594, 0, 0.999005, -0.044594, 0, 0.999005, -0.044594, 0, 0.999005, -0.044594, 0, 0.999005, -0.044594, 0, 0.999005, -0.044594, 0, 0.998926, -0.046342, 0, 0.998926, -0.046342, 0, 0.998926, -0.046342, 0, 0.998926, -0.046342, 0, 0.998926, -0.046342, 0, 0.998926, -0.046342, 0, 0.998769, -0.049597, 0, 0.998769, -0.049597, 0, 0.998769, -0.049597, 0, 0.998769, -0.049597, 0, 0.998769, -0.049597, 0, 0.998769, -0.049597, 0, 0.998523, -0.054332, 0, 0.998523, -0.054332, 0, 0.998523, -0.054332, 0, 0.998523, -0.054332, 0, 0.998523, -0.054332, 0, 0.998523, -0.054332, 0, 0.998169, -0.060480, 0, 0.998169, -0.060480, 0, 0.998169, -0.060480, 0, 0.998169, -0.060480, 0, 0.998169, -0.060480, 0, 0.998169, -0.060480, 0, 0.997686, -0.067991, 0, 0.997686, -0.067991, 0, 0.997686, -0.067991, 0, 0.997686, -0.067991, 0, 0.997686, -0.067991, 0, 0.997686, -0.067991, 0, 0.997041, -0.076878, 0, 0.997041, -0.076878, 0, 0.997041, -0.076878, 0, 0.997041, -0.076878, 0, 0.997041, -0.076878, 0, 0.997041, -0.076878, 0, 0.996194, -0.087163, 0, 0.996194, -0.087163, 0, 0.996194, -0.087163, 0, 0.996194, -0.087163, 0, 0.996194, -0.087163, 0, 0.996194, -0.087163, 0, 0.995221, -0.097645, 0, 0.995221, -0.097645, 0, 0.995221, -0.097645, 0, 0.995221, -0.097645, 0, 0.995221, -0.097645, 0, 0.995221, -0.097645, 0, 0.994266, -0.106938, 0, 0.994266, -0.106938, 0, 0.994266, -0.106938, 0, 0.994266, -0.106938, 0, 0.994266, -0.106938, 0, 0.994266, -0.106938, 0, 0.993356, -0.115085, 0, 0.993356, -0.115085, 0, 0.993356, -0.115085, 0, 0.993356, -0.115085, 0, 0.993356, -0.115085, 0, 0.993356, -0.115085, 0, 0.992515, -0.122122, 0, 0.992515, -0.122122, 0, 0.992515, -0.122122, 0, 0.992515, -0.122122, 0, 0.992515, -0.122122, 0, 0.992515, -0.122122, 0, 0.991770, -0.128031, 0, 0.991770, -0.128031, 0, 0.991770, -0.128031, 0, 0.991770, -0.128031, 0, 0.991770, -0.128031, 0, 0.991770, -0.128031, 0, 0.991145, -0.132785, 0, 0.991145, -0.132785, 0, 0.991145, -0.132785, 0, 0.991145, -0.132785, 0, 0.991145, -0.132785, 0, 0.991145, -0.132785, 0, 0.990657, -0.136379, 0, 0.990657, -0.136379, 0, 0.990657, -0.136379, 0, 0.990657, -0.136379, 0, 0.990657, -0.136379, 0, 0.990657, -0.136379, 0, 0.990321, -0.138799, 0, 0.990321, -0.138799, 0, 0.990321, -0.138799, 0, 0.990321, -0.138799, 0, 0.990321, -0.138799, 0, 0.990321, -0.138799, 0, 0.989781, -0.142593, 0, 0.989781, -0.142593, 0, 0.989781, -0.142593, 0, 0.989781, -0.142593, 0, 0.989781, -0.142593, 0, 0.989781, -0.142593, 0, 0.978491, -0.206292, 0, 0.978491, -0.206292, 0, 0.978491, -0.206292, 0, 0.978491, -0.206292, 0, 0.978491, -0.206292, 0, 0.978491, -0.206292, 0, 0.977537, -0.210766, 0, 0.977537, -0.210766, 0, 0.977537, -0.210766, 0, 0.977537, -0.210766, 0, 0.977537, -0.210766, 0, 0.977537, -0.210766, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.996096, -0.088272, 0, 0.996096, -0.088272, 0, 0.996096, -0.088272, 0, 0.996096, -0.088272, 0, 0.996096, -0.088272, 0, 0.996096, -0.088272, 0, 0.996138, -0.087798, 0, 0.996138, -0.087798, 0, 0.996138, -0.087798, 0, 0.996138, -0.087798, 0, 0.996138, -0.087798, 0, 0.996138, -0.087798, 0, 0.996171, -0.087429, 0, 0.996171, -0.087429, 0, 0.996171, -0.087429, 0, 0.996171, -0.087429, 0, 0.996171, -0.087429, 0, 0.996171, -0.087429, 0, 0.996194, -0.087165, 0, 0.996194, -0.087165, 0, 0.996194, -0.087165, 0, 0.996194, -0.087165, 0, 0.996194, -0.087165, 0, 0.996194, -0.087165, 0, 0.996208, -0.087003, 0, 0.996208, -0.087003, 0, 0.996208, -0.087003, 0, 0.996208, -0.087003, 0, 0.996208, -0.087003, 0, 0.996208, -0.087003, 0, 0.996214, -0.086937, 0, 0.996214, -0.086937, 0, 0.996214, -0.086937, 0, 0.996214, -0.086937, 0, 0.996214, -0.086937, 0, 0.996214, -0.086937, 0, 0.996211, -0.086969, 0, 0.996211, -0.086969, 0, 0.996211, -0.086969, 0, 0.996211, -0.086969, 0, 0.996211, -0.086969, 0, 0.996211, -0.086969, 0, 0.996200, -0.087099, 0, 0.996200, -0.087099, 0, 0.996200, -0.087099, 0, 0.996200, -0.087099, 0, 0.996200, -0.087099, 0, 0.996200, -0.087099, 0, 0.996094, -0.088298, 0, 0.996094, -0.088298, 0, 0.996094, -0.088298, 0, 0.996094, -0.088298, 0, 0.996094, -0.088298, 0, 0.996094, -0.088298, 0, 0.995884, -0.090636, 0, 0.995884, -0.090636, 0, 0.995884, -0.090636, 0, 0.995884, -0.090636, 0, 0.995884, -0.090636, 0, 0.995884, -0.090636, 0, 0.995560, -0.094125, 0, 0.995560, -0.094125, 0, 0.995560, -0.094125, 0, 0.995560, -0.094125, 0, 0.995560, -0.094125, 0, 0.995560, -0.094125, 0, 0.995110, -0.098772, 0, 0.995110, -0.098772, 0, 0.995110, -0.098772, 0, 0.995110, -0.098772, 0, 0.995110, -0.098772, 0, 0.995110, -0.098772, 0, 0.994526, -0.104488, 0, 0.994526, -0.104488, 0, 0.994526, -0.104488, 0, 0.994526, -0.104488, 0, 0.994526, -0.104488, 0, 0.994526, -0.104488, 0, 0.993801, -0.111174, 0, 0.993801, -0.111174, 0, 0.993801, -0.111174, 0, 0.993801, -0.111174, 0, 0.993801, -0.111174, 0, 0.993801, -0.111174, 0, 0.992917, -0.118810, 0, 0.992917, -0.118810, 0, 0.992917, -0.118810, 0, 0.992917, -0.118810, 0, 0.992917, -0.118810, 0, 0.992917, -0.118810, 0, 0.991856, -0.127367, 0, 0.991856, -0.127367, 0, 0.991856, -0.127367, 0, 0.991856, -0.127367, 0, 0.991856, -0.127367, 0, 0.991856, -0.127367, 0, 0.983037, -0.183406, 0, 0.983037, -0.183406, 0, 0.983037, -0.183406, 0, 0.983037, -0.183406, 0, 0.983037, -0.183406, 0, 0.983037, -0.183406, 0, 0.982427, -0.186648, 0, 0.982427, -0.186648, 0, 0.982427, -0.186648, 0, 0.982427, -0.186648, 0, 0.982427, -0.186648, 0, 0.982427, -0.186648, 0, 0.996881, -0.078914, 0, 0.996881, -0.078914, 0, 0.996881, -0.078914, 0, 0.996881, -0.078914, 0, 0.996881, -0.078914, 0, 0.996881, -0.078914, 0, 0.996992, -0.077504, 0, 0.996992, -0.077504, 0, 0.996992, -0.077504, 0, 0.996992, -0.077504, 0, 0.996992, -0.077504, 0, 0.996992, -0.077504, 0, 0.995677, -0.092882, 0, 0.995677, -0.092882, 0, 0.995677, -0.092882, 0, 0.995677, -0.092882, 0, 0.995677, -0.092882, 0, 0.995677, -0.092882, 0, 0.996099, -0.088239, 0, 0.996099, -0.088239, 0, 0.996099, -0.088239, 0, 0.996099, -0.088239, 0, 0.996099, -0.088239, 0, 0.996099, -0.088239, 0, 0.996450, -0.084184, 0, 0.996450, -0.084184, 0, 0.996450, -0.084184, 0, 0.996450, -0.084184, 0, 0.996450, -0.084184, 0, 0.996450, -0.084184, 0, 0.996737, -0.080717, 0, 0.996737, -0.080717, 0, 0.996737, -0.080717, 0, 0.996737, -0.080717, 0, 0.996737, -0.080717, 0, 0.996737, -0.080717, 0, 0.996966, -0.077837, 0, 0.996966, -0.077837, 0, 0.996966, -0.077837, 0, 0.996966, -0.077837, 0, 0.996966, -0.077837, 0, 0.996966, -0.077837, 0, 0.997144, -0.075523, 0, 0.997144, -0.075523, 0, 0.997144, -0.075523, 0, 0.997144, -0.075523, 0, 0.997144, -0.075523, 0, 0.997144, -0.075523, 0, 0.997277, -0.073750, 0, 0.997277, -0.073750, 0, 0.997277, -0.073750, 0, 0.997277, -0.073750, 0, 0.997277, -0.073750, 0, 0.997277, -0.073750, 0, 0.997368, -0.072513, 0, 0.997368, -0.072513, 0, 0.997368, -0.072513, 0, 0.997368, -0.072513, 0, 0.997368, -0.072513, 0, 0.997368, -0.072513, 0, 0.997419, -0.071805, 0, 0.997419, -0.071805, 0, 0.997419, -0.071805, 0, 0.997419, -0.071805, 0, 0.997419, -0.071805, 0, 0.997419, -0.071805, 0, 0.997423, -0.071740, 0, 0.997423, -0.071740, 0, 0.997423, -0.071740, 0, 0.997423, -0.071740, 0, 0.997423, -0.071740, 0, 0.997423, -0.071740, 0, 0.997428, -0.071671, 0, 0.997428, -0.071671, 0, 0.997428, -0.071671, 0, 0.997428, -0.071671, 0, 0.997428, -0.071671, 0, 0.997428, -0.071671, 0, 0.997433, -0.071600, 0, 0.997433, -0.071600, 0, 0.997433, -0.071600, 0, 0.997433, -0.071600, 0, 0.997433, -0.071600, 0, 0.997433, -0.071600, 0, 0.997439, -0.071525, 0, 0.997439, -0.071525, 0, 0.997439, -0.071525, 0, 0.997439, -0.071525, 0, 0.997439, -0.071525, 0, 0.997439, -0.071525, 0, 0.997444, -0.071447, 0, 0.997444, -0.071447, 0, 0.997444, -0.071447, 0, 0.997444, -0.071447, 0, 0.997444, -0.071447, 0, 0.997444, -0.071447, 0, 0.997450, -0.071364, 0, 0.997450, -0.071364, 0, 0.997450, -0.071364, 0, 0.997450, -0.071364, 0, 0.997450, -0.071364, 0, 0.997450, -0.071364, 0, 0.997456, -0.071278, 0, 0.997456, -0.071278, 0, 0.997456, -0.071278, 0, 0.997456, -0.071278, 0, 0.997456, -0.071278, 0, 0.997456, -0.071278, 0, 0.997463, -0.071188, 0, 0.997463, -0.071188, 0, 0.997463, -0.071188, 0, 0.997463, -0.071188, 0, 0.997463, -0.071188, 0, 0.997463, -0.071188, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.996579, -0.082649, 0, 0.996579, -0.082649, 0, 0.996579, -0.082649, 0, 0.996579, -0.082649, 0, 0.996579, -0.082649, 0, 0.996579, -0.082649, 0, 0.996380, -0.085016, 0, 0.996380, -0.085016, 0, 0.996380, -0.085016, 0, 0.996380, -0.085016, 0, 0.996380, -0.085016, 0, 0.996380, -0.085016, 0, 0.996065, -0.088627, 0, 0.996065, -0.088627, 0, 0.996065, -0.088627, 0, 0.996065, -0.088627, 0, 0.996065, -0.088627, 0, 0.996065, -0.088627, 0, 0.995620, -0.093488, 0, 0.995620, -0.093488, 0, 0.995620, -0.093488, 0, 0.995620, -0.093488, 0, 0.995620, -0.093488, 0, 0.995620, -0.093488, 0, 0.995046, -0.099420, 0, 0.995046, -0.099420, 0, 0.995046, -0.099420, 0, 0.995046, -0.099420, 0, 0.995046, -0.099420, 0, 0.995046, -0.099420, 0, 0.994341, -0.106231, 0, 0.994341, -0.106231, 0, 0.994341, -0.106231, 0, 0.994341, -0.106231, 0, 0.994341, -0.106231, 0, 0.994341, -0.106231, 0, 0.993492, -0.113902, 0, 0.993492, -0.113902, 0, 0.993492, -0.113902, 0, 0.993492, -0.113902, 0, 0.993492, -0.113902, 0, 0.993492, -0.113902, 0, 0.992480, -0.122406, 0, 0.992480, -0.122406, 0, 0.992480, -0.122406, 0, 0.992480, -0.122406, 0, 0.992480, -0.122406, 0, 0.992480, -0.122406, 0, 0.991413, -0.130767, 0, 0.991413, -0.130767, 0, 0.991413, -0.130767, 0, 0.991413, -0.130767, 0, 0.991413, -0.130767, 0, 0.991413, -0.130767, 0, 0.990430, -0.138019, 0, 0.990430, -0.138019, 0, 0.990430, -0.138019, 0, 0.990430, -0.138019, 0, 0.990430, -0.138019, 0, 0.990430, -0.138019, 0, 0.989555, -0.144157, 0, 0.989555, -0.144157, 0, 0.989555, -0.144157, 0, 0.989555, -0.144157, 0, 0.989555, -0.144157, 0, 0.989555, -0.144157, 0, 0.988810, -0.149178, 0, 0.988810, -0.149178, 0, 0.988810, -0.149178, 0, 0.988810, -0.149178, 0, 0.988810, -0.149178, 0, 0.988810, -0.149178, 0, 0.988235, -0.152942, 0, 0.988235, -0.152942, 0, 0.988235, -0.152942, 0, 0.988235, -0.152942, 0, 0.988235, -0.152942, 0, 0.988235, -0.152942, 0, 0.987864, -0.155319, 0, 0.987864, -0.155319, 0, 0.987864, -0.155319, 0, 0.987864, -0.155319, 0, 0.987864, -0.155319, 0, 0.987864, -0.155319, 0, 0.987705, -0.156329, 0, 0.987705, -0.156329, 0, 0.987705, -0.156329, 0, 0.987705, -0.156329, 0, 0.987705, -0.156329, 0, 0.987705, -0.156329, 0, 0.987758, -0.155994, 0, 0.987758, -0.155994, 0, 0.987758, -0.155994, 0, 0.987758, -0.155994, 0, 0.987758, -0.155994, 0, 0.987758, -0.155994, 0, 0.988000, -0.154452, 0, 0.988000, -0.154452, 0, 0.988000, -0.154452, 0, 0.988000, -0.154452, 0, 0.988000, -0.154452, 0, 0.988000, -0.154452, 0, 0.988410, -0.151808, 0, 0.988410, -0.151808, 0, 0.988410, -0.151808, 0, 0.988410, -0.151808, 0, 0.988410, -0.151808, 0, 0.988410, -0.151808, 0, 0.988977, -0.148070, 0, 0.988977, -0.148070, 0, 0.988977, -0.148070, 0, 0.988977, -0.148070, 0, 0.988977, -0.148070, 0, 0.988977, -0.148070, 0, 0.989688, -0.143240, 0, 0.989688, -0.143240, 0, 0.989688, -0.143240, 0, 0.989688, -0.143240, 0, 0.989688, -0.143240, 0, 0.989688, -0.143240, 0, 0.990502, -0.137501, 0, 0.990502, -0.137501, 0, 0.990502, -0.137501, 0, 0.990502, -0.137501, 0, 0.990502, -0.137501, 0, 0.990502, -0.137501, 0, 0.991379, -0.131026, 0, 0.991379, -0.131026, 0, 0.991379, -0.131026, 0, 0.991379, -0.131026, 0, 0.991379, -0.131026, 0, 0.991379, -0.131026, 0, 0.992307, -0.123798, 0, 0.992307, -0.123798, 0, 0.992307, -0.123798, 0, 0.992307, -0.123798, 0, 0.992307, -0.123798, 0, 0.992307, -0.123798, 0, 0.993273, -0.115794, 0, 0.993273, -0.115794, 0, 0.993273, -0.115794, 0, 0.993273, -0.115794, 0, 0.993273, -0.115794, 0, 0.993273, -0.115794, 0, 0.994161, -0.107907, 0, 0.994161, -0.107907, 0, 0.994161, -0.107907, 0, 0.994161, -0.107907, 0, 0.994161, -0.107907, 0, 0.994161, -0.107907, 0, 0.994891, -0.100956, 0, 0.994891, -0.100956, 0, 0.994891, -0.100956, 0, 0.994891, -0.100956, 0, 0.994891, -0.100956, 0, 0.994891, -0.100956, 0, 0.995484, -0.094934, 0, 0.995484, -0.094934, 0, 0.995484, -0.094934, 0, 0.995484, -0.094934, 0, 0.995484, -0.094934, 0, 0.995484, -0.094934, 0, 0.995957, -0.089834, 0, 0.995957, -0.089834, 0, 0.995957, -0.089834, 0, 0.995957, -0.089834, 0, 0.995957, -0.089834, 0, 0.995957, -0.089834, 0, 0.996310, -0.085828, 0, 0.996310, -0.085828, 0, 0.996310, -0.085828, 0, 0.996310, -0.085828, 0, 0.996310, -0.085828, 0, 0.996310, -0.085828, 0, 0.996542, -0.083094, 0, 0.996542, -0.083094, 0, 0.996542, -0.083094, 0, 0.996542, -0.083094, 0, 0.996542, -0.083094, 0, 0.996542, -0.083094, 0, 0.996661, -0.081649, 0, 0.996661, -0.081649, 0, 0.996661, -0.081649, 0, 0.996661, -0.081649, 0, 0.996661, -0.081649, 0, 0.996661, -0.081649, 0, 0.997705, -0.067713, 0, 0.997705, -0.067713, 0, 0.997705, -0.067713, 0, 0.997705, -0.067713, 0, 0.997705, -0.067713, 0, 0.997705, -0.067713, 0, 0.997571, -0.069654, 0, 0.997571, -0.069654, 0, 0.997571, -0.069654, 0, 0.997571, -0.069654, 0, 0.997571, -0.069654, 0, 0.997571, -0.069654, 0, 0.997309, -0.073311, 0, 0.997309, -0.073311, 0, 0.997309, -0.073311, 0, 0.997309, -0.073311, 0, 0.997309, -0.073311, 0, 0.997309, -0.073311, 0, 0.996902, -0.078654, 0, 0.996902, -0.078654, 0, 0.996902, -0.078654, 0, 0.996902, -0.078654, 0, 0.996902, -0.078654, 0, 0.996902, -0.078654, 0, 0.996339, -0.085486, 0, 0.996339, -0.085486, 0, 0.996339, -0.085486, 0, 0.996339, -0.085486, 0, 0.996339, -0.085486, 0, 0.996339, -0.085486, 0, 0.995607, -0.093632, 0, 0.995607, -0.093632, 0, 0.995607, -0.093632, 0, 0.995607, -0.093632, 0, 0.995607, -0.093632, 0, 0.995607, -0.093632, 0, 0.994671, -0.103102, 0, 0.994671, -0.103102, 0, 0.994671, -0.103102, 0, 0.994671, -0.103102, 0, 0.994671, -0.103102, 0, 0.994671, -0.103102, 0, 0.993491, -0.113915, 0, 0.993491, -0.113915, 0, 0.993491, -0.113915, 0, 0.993491, -0.113915, 0, 0.993491, -0.113915, 0, 0.993491, -0.113915, 0, 0.992180, -0.124816, 0, 0.992180, -0.124816, 0, 0.992180, -0.124816, 0, 0.992180, -0.124816, 0, 0.992180, -0.124816, 0, 0.992180, -0.124816, 0, 0.990893, -0.134651, 0, 0.990893, -0.134651, 0, 0.990893, -0.134651, 0, 0.990893, -0.134651, 0, 0.990893, -0.134651, 0, 0.990893, -0.134651, 0, 0.989655, -0.143467, 0, 0.989655, -0.143467, 0, 0.989655, -0.143467, 0, 0.989655, -0.143467, 0, 0.989655, -0.143467, 0, 0.989655, -0.143467, 0, 0.988488, -0.151300, 0, 0.988488, -0.151300, 0, 0.988488, -0.151300, 0, 0.988488, -0.151300, 0, 0.988488, -0.151300, 0, 0.988488, -0.151300, 0, 0.987446, -0.157954, 0, 0.987446, -0.157954, 0, 0.987446, -0.157954, 0, 0.987446, -0.157954, 0, 0.987446, -0.157954, 0, 0.987446, -0.157954, 0, 0.986590, -0.163216, 0, 0.986590, -0.163216, 0, 0.986590, -0.163216, 0, 0.986590, -0.163216, 0, 0.986590, -0.163216, 0, 0.986590, -0.163216, 0, 0.985943, -0.167082, 0, 0.985943, -0.167082, 0, 0.985943, -0.167082, 0, 0.985943, -0.167082, 0, 0.985943, -0.167082, 0, 0.985943, -0.167082, 0, 0.985524, -0.169538, 0, 0.985524, -0.169538, 0, 0.985524, -0.169538, 0, 0.985524, -0.169538, 0, 0.985524, -0.169538, 0, 0.985524, -0.169538, 0, 0.985380, -0.170369, 0, 0.985380, -0.170369, 0, 0.985380, -0.170369, 0, 0.985380, -0.170369, 0, 0.985380, -0.170369, 0, 0.985380, -0.170369, 0, 0.985559, -0.169330, 0, 0.985559, -0.169330, 0, 0.985559, -0.169330, 0, 0.985559, -0.169330, 0, 0.985559, -0.169330, 0, 0.985559, -0.169330, 0, 0.986063, -0.166370, 0, 0.986063, -0.166370, 0, 0.986063, -0.166370, 0, 0.986063, -0.166370, 0, 0.986063, -0.166370, 0, 0.986063, -0.166370, 0, 0.986882, -0.161444, 0, 0.986882, -0.161444, 0, 0.986882, -0.161444, 0, 0.986882, -0.161444, 0, 0.986882, -0.161444, 0, 0.986882, -0.161444, 0, 0.987962, -0.154698, 0, 0.987962, -0.154698, 0, 0.987962, -0.154698, 0, 0.987962, -0.154698, 0, 0.987962, -0.154698, 0, 0.987962, -0.154698, 0, 0.989241, -0.146297, 0, 0.989241, -0.146297, 0, 0.989241, -0.146297, 0, 0.989241, -0.146297, 0, 0.989241, -0.146297, 0, 0.989241, -0.146297, 0, 0.990675, -0.136247, 0, 0.990675, -0.136247, 0, 0.990675, -0.136247, 0, 0.990675, -0.136247, 0, 0.990675, -0.136247, 0, 0.990675, -0.136247, 0, 0.992212, -0.124565, 0, 0.992212, -0.124565, 0, 0.992212, -0.124565, 0, 0.992212, -0.124565, 0, 0.992212, -0.124565, 0, 0.992212, -0.124565, 0, 0.993640, -0.112599, 0, 0.993640, -0.112599, 0, 0.993640, -0.112599, 0, 0.993640, -0.112599, 0, 0.993640, -0.112599, 0, 0.993640, -0.112599, 0, 0.994803, -0.101818, 0, 0.994803, -0.101818, 0, 0.994803, -0.101818, 0, 0.994803, -0.101818, 0, 0.994803, -0.101818, 0, 0.994803, -0.101818, 0, 0.995733, -0.092279, 0, 0.995733, -0.092279, 0, 0.995733, -0.092279, 0, 0.995733, -0.092279, 0, 0.995733, -0.092279, 0, 0.995733, -0.092279, 0, 0.996464, -0.084027, 0, 0.996464, -0.084027, 0, 0.996464, -0.084027, 0, 0.996464, -0.084027, 0, 0.996464, -0.084027, 0, 0.996464, -0.084027, 0, 0.997008, -0.077302, 0, 0.997008, -0.077302, 0, 0.997008, -0.077302, 0, 0.997008, -0.077302, 0, 0.997008, -0.077302, 0, 0.997008, -0.077302, 0, 0.997381, -0.072325, 0, 0.997381, -0.072325, 0, 0.997381, -0.072325, 0, 0.997381, -0.072325, 0, 0.997381, -0.072325, 0, 0.997381, -0.072325, 0, 0.997611, -0.069078, 0, 0.997611, -0.069078, 0, 0.997611, -0.069078, 0, 0.997611, -0.069078, 0, 0.997611, -0.069078, 0, 0.997611, -0.069078, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997906, -0.064678, 0, 0.997906, -0.064678, 0, 0.997906, -0.064678, 0, 0.997906, -0.064678, 0, 0.997906, -0.064678, 0, 0.997906, -0.064678, 0, 0.998029, -0.062759, 0, 0.998029, -0.062759, 0, 0.998029, -0.062759, 0, 0.998029, -0.062759, 0, 0.998029, -0.062759, 0, 0.998029, -0.062759, 0, 0.997165, -0.075246, 0, 0.997165, -0.075246, 0, 0.997165, -0.075246, 0, 0.997165, -0.075246, 0, 0.997165, -0.075246, 0, 0.997165, -0.075246, 0, 0.996989, -0.077543, 0, 0.996989, -0.077543, 0, 0.996989, -0.077543, 0, 0.996989, -0.077543, 0, 0.996989, -0.077543, 0, 0.996989, -0.077543, 0, 0.992197, -0.124678, 0, 0.992197, -0.124678, 0, 0.992197, -0.124678, 0, 0.992197, -0.124678, 0, 0.992197, -0.124678, 0, 0.992197, -0.124678, 0, 0.991587, -0.129442, 0, 0.991587, -0.129442, 0, 0.991587, -0.129442, 0, 0.991587, -0.129442, 0, 0.991587, -0.129442, 0, 0.991587, -0.129442, 0, 0.991088, -0.133209, 0, 0.991088, -0.133209, 0, 0.991088, -0.133209, 0, 0.991088, -0.133209, 0, 0.991088, -0.133209, 0, 0.991088, -0.133209, 0, 0.990711, -0.135982, 0, 0.990711, -0.135982, 0, 0.990711, -0.135982, 0, 0.990711, -0.135982, 0, 0.990711, -0.135982, 0, 0.990711, -0.135982, 0, 0.990465, -0.137764, 0, 0.990465, -0.137764, 0, 0.990465, -0.137764, 0, 0.990465, -0.137764, 0, 0.990465, -0.137764, 0, 0.990465, -0.137764, 0, 0.990319, -0.138809, 0, 0.990319, -0.138809, 0, 0.990319, -0.138809, 0, 0.990319, -0.138809, 0, 0.990319, -0.138809, 0, 0.990319, -0.138809, 0, 0.990240, -0.139371, 0, 0.990240, -0.139371, 0, 0.990240, -0.139371, 0, 0.990240, -0.139371, 0, 0.990240, -0.139371, 0, 0.990240, -0.139371, 0, 0.990228, -0.139455, 0, 0.990228, -0.139455, 0, 0.990228, -0.139455, 0, 0.990228, -0.139455, 0, 0.990228, -0.139455, 0, 0.990228, -0.139455, 0, 0.990283, -0.139065, 0, 0.990283, -0.139065, 0, 0.990283, -0.139065, 0, 0.990283, -0.139065, 0, 0.990283, -0.139065, 0, 0.990283, -0.139065, 0, 0.990566, -0.137038, 0, 0.990566, -0.137038, 0, 0.990566, -0.137038, 0, 0.990566, -0.137038, 0, 0.990566, -0.137038, 0, 0.990566, -0.137038, 0, 0.988803, -0.149230, 0, 0.988803, -0.149230, 0, 0.988803, -0.149230, 0, 0.988803, -0.149230, 0, 0.988803, -0.149230, 0, 0.988803, -0.149230, 0, 0.988468, -0.151429, 0, 0.988468, -0.151429, 0, 0.988468, -0.151429, 0, 0.988468, -0.151429, 0, 0.988468, -0.151429, 0, 0.988468, -0.151429, 0, 0.988348, -0.152212, 0, 0.988348, -0.152212, 0, 0.988348, -0.152212, 0, 0.988348, -0.152212, 0, 0.988348, -0.152212, 0, 0.988348, -0.152212, 0, 0.988376, -0.152031, 0, 0.988376, -0.152031, 0, 0.988376, -0.152031, 0, 0.988376, -0.152031, 0, 0.988376, -0.152031, 0, 0.988376, -0.152031, 0, 0.988554, -0.150867, 0, 0.988554, -0.150867, 0, 0.988554, -0.150867, 0, 0.988554, -0.150867, 0, 0.988554, -0.150867, 0, 0.988554, -0.150867, 0, 0.988881, -0.148708, 0, 0.988881, -0.148708, 0, 0.988881, -0.148708, 0, 0.988881, -0.148708, 0, 0.988881, -0.148708, 0, 0.988881, -0.148708, 0, 0.989383, -0.145331, 0, 0.989383, -0.145331, 0, 0.989383, -0.145331, 0, 0.989383, -0.145331, 0, 0.989383, -0.145331, 0, 0.989383, -0.145331, 0, 0.990079, -0.140515, 0, 0.990079, -0.140515, 0, 0.990079, -0.140515, 0, 0.990079, -0.140515, 0, 0.990079, -0.140515, 0, 0.990079, -0.140515, 0, 0.990947, -0.134253, 0, 0.990947, -0.134253, 0, 0.990947, -0.134253, 0, 0.990947, -0.134253, 0, 0.990947, -0.134253, 0, 0.990947, -0.134253, 0, 0.991961, -0.126545, 0, 0.991961, -0.126545, 0, 0.991961, -0.126545, 0, 0.991961, -0.126545, 0, 0.991961, -0.126545, 0, 0.991961, -0.126545, 0, 0.996897, -0.078715, 0, 0.996897, -0.078715, 0, 0.996897, -0.078715, 0, 0.996897, -0.078715, 0, 0.996897, -0.078715, 0, 0.996897, -0.078715, 0, 0.996827, -0.079594, 0, 0.996827, -0.079594, 0, 0.996827, -0.079594, 0, 0.996827, -0.079594, 0, 0.996827, -0.079594, 0, 0.996827, -0.079594, 0, 0.997794, -0.066392, 0, 0.997794, -0.066392, 0, 0.997794, -0.066392, 0, 0.997794, -0.066392, 0, 0.997794, -0.066392, 0, 0.997794, -0.066392, 0, 0.997842, -0.065658, 0, 0.997842, -0.065658, 0, 0.997842, -0.065658, 0, 0.997842, -0.065658, 0, 0.997842, -0.065658, 0, 0.997842, -0.065658, 0, 0.999222, -0.039449, 0, 0.999222, -0.039449, 0, 0.999222, -0.039449, 0, 0.999222, -0.039449, 0, 0.999222, -0.039449, 0, 0.999222, -0.039449, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.994923, -0.100643, 0, 0.994923, -0.100643, 0, 0.994923, -0.100643, 0, 0.994923, -0.100643, 0, 0.994923, -0.100643, 0, 0.994923, -0.100643, 0, 0.994250, -0.107084, 0, 0.994250, -0.107084, 0, 0.994250, -0.107084, 0, 0.994250, -0.107084, 0, 0.994250, -0.107084, 0, 0.994250, -0.107084, 0, 0.993526, -0.113605, 0, 0.993526, -0.113605, 0, 0.993526, -0.113605, 0, 0.993526, -0.113605, 0, 0.993526, -0.113605, 0, 0.993526, -0.113605, 0, 0.992874, -0.119166, 0, 0.992874, -0.119166, 0, 0.992874, -0.119166, 0, 0.992874, -0.119166, 0, 0.992874, -0.119166, 0, 0.992874, -0.119166, 0, 0.992312, -0.123760, 0, 0.992312, -0.123760, 0, 0.992312, -0.123760, 0, 0.992312, -0.123760, 0, 0.992312, -0.123760, 0, 0.992312, -0.123760, 0, 0.991853, -0.127390, 0, 0.991853, -0.127390, 0, 0.991853, -0.127390, 0, 0.991853, -0.127390, 0, 0.991853, -0.127390, 0, 0.991853, -0.127390, 0, 0.991511, -0.130023, 0, 0.991511, -0.130023, 0, 0.991511, -0.130023, 0, 0.991511, -0.130023, 0, 0.991511, -0.130023, 0, 0.991511, -0.130023, 0, 0.991298, -0.131633, 0, 0.991298, -0.131633, 0, 0.991298, -0.131633, 0, 0.991298, -0.131633, 0, 0.991298, -0.131633, 0, 0.991298, -0.131633, 0, 0.991218, -0.132236, 0, 0.991218, -0.132236, 0, 0.991218, -0.132236, 0, 0.991218, -0.132236, 0, 0.991218, -0.132236, 0, 0.991218, -0.132236, 0, 0.991270, -0.131846, 0, 0.991270, -0.131846, 0, 0.991270, -0.131846, 0, 0.991270, -0.131846, 0, 0.991270, -0.131846, 0, 0.991270, -0.131846, 0, 0.991351, -0.131234, 0, 0.991351, -0.131234, 0, 0.991351, -0.131234, 0, 0.991351, -0.131234, 0, 0.991351, -0.131234, 0, 0.991351, -0.131234, 0, 0.991459, -0.130419, 0, 0.991459, -0.130419, 0, 0.991459, -0.130419, 0, 0.991459, -0.130419, 0, 0.991459, -0.130419, 0, 0.991459, -0.130419, 0, 0.991592, -0.129403, 0, 0.991592, -0.129403, 0, 0.991592, -0.129403, 0, 0.991592, -0.129403, 0, 0.991592, -0.129403, 0, 0.991592, -0.129403, 0, 0.991750, -0.128188, 0, 0.991750, -0.128188, 0, 0.991750, -0.128188, 0, 0.991750, -0.128188, 0, 0.991750, -0.128188, 0, 0.991750, -0.128188, 0, 0.991932, -0.126772, 0, 0.991932, -0.126772, 0, 0.991932, -0.126772, 0, 0.991932, -0.126772, 0, 0.991932, -0.126772, 0, 0.991932, -0.126772, 0, 0.992137, -0.125155, 0, 0.992137, -0.125155, 0, 0.992137, -0.125155, 0, 0.992137, -0.125155, 0, 0.992137, -0.125155, 0, 0.992137, -0.125155, 0, 0.992365, -0.123338, 0, 0.992365, -0.123338, 0, 0.992365, -0.123338, 0, 0.992365, -0.123338, 0, 0.992365, -0.123338, 0, 0.992365, -0.123338, 0, 0.992613, -0.121323, 0, 0.992613, -0.121323, 0, 0.992613, -0.121323, 0, 0.992613, -0.121323, 0, 0.992613, -0.121323, 0, 0.992613, -0.121323, 0, 0.991173, -0.132575, 0, 0.991173, -0.132575, 0, 0.991173, -0.132575, 0, 0.991173, -0.132575, 0, 0.991173, -0.132575, 0, 0.991173, -0.132575, 0, 0.990900, -0.134600, 0, 0.990900, -0.134600, 0, 0.990900, -0.134600, 0, 0.990900, -0.134600, 0, 0.990900, -0.134600, 0, 0.990900, -0.134600, 0, 0.990648, -0.136445, 0, 0.990648, -0.136445, 0, 0.990648, -0.136445, 0, 0.990648, -0.136445, 0, 0.990648, -0.136445, 0, 0.990648, -0.136445, 0, 0.990417, -0.138109, 0, 0.990417, -0.138109, 0, 0.990417, -0.138109, 0, 0.990417, -0.138109, 0, 0.990417, -0.138109, 0, 0.990417, -0.138109, 0, 0.990209, -0.139591, 0, 0.990209, -0.139591, 0, 0.990209, -0.139591, 0, 0.990209, -0.139591, 0, 0.990209, -0.139591, 0, 0.990209, -0.139591, 0, 0.990027, -0.140879, 0, 0.990027, -0.140879, 0, 0.990027, -0.140879, 0, 0.990027, -0.140879, 0, 0.990027, -0.140879, 0, 0.990027, -0.140879, 0, 0.989872, -0.141961, 0, 0.989872, -0.141961, 0, 0.989872, -0.141961, 0, 0.989872, -0.141961, 0, 0.989872, -0.141961, 0, 0.989872, -0.141961, 0, 0.989746, -0.142836, 0, 0.989746, -0.142836, 0, 0.989746, -0.142836, 0, 0.989746, -0.142836, 0, 0.989746, -0.142836, 0, 0.989746, -0.142836, 0, 0.989650, -0.143500, 0, 0.989650, -0.143500, 0, 0.989650, -0.143500, 0, 0.989650, -0.143500, 0, 0.989650, -0.143500, 0, 0.989650, -0.143500, 0, 0.989559, -0.144126, 0, 0.989559, -0.144126, 0, 0.989559, -0.144126, 0, 0.989559, -0.144126, 0, 0.989559, -0.144126, 0, 0.989559, -0.144126, 0, 0.989697, -0.143175, 0, 0.989697, -0.143175, 0, 0.989697, -0.143175, 0, 0.989697, -0.143175, 0, 0.989697, -0.143175, 0, 0.989697, -0.143175, 0, 0.990065, -0.140609, 0, 0.990065, -0.140609, 0, 0.990065, -0.140609, 0, 0.990065, -0.140609, 0, 0.990065, -0.140609, 0, 0.990065, -0.140609, 0, 0.990655, -0.136393, 0, 0.990655, -0.136393, 0, 0.990655, -0.136393, 0, 0.990655, -0.136393, 0, 0.990655, -0.136393, 0, 0.990655, -0.136393, 0, 0.991424, -0.130688, 0, 0.991424, -0.130688, 0, 0.991424, -0.130688, 0, 0.991424, -0.130688, 0, 0.991424, -0.130688, 0, 0.991424, -0.130688, 0, 0.992323, -0.123674, 0, 0.992323, -0.123674, 0, 0.992323, -0.123674, 0, 0.992323, -0.123674, 0, 0.992323, -0.123674, 0, 0.992323, -0.123674, 0, 0.993324, -0.115359, 0, 0.993324, -0.115359, 0, 0.993324, -0.115359, 0, 0.993324, -0.115359, 0, 0.993324, -0.115359, 0, 0.993324, -0.115359, 0, 0.994392, -0.105762, 0, 0.994392, -0.105762, 0, 0.994392, -0.105762, 0, 0.994392, -0.105762, 0, 0.994392, -0.105762, 0, 0.994392, -0.105762, 0, 0.995415, -0.095652, 0, 0.995415, -0.095652, 0, 0.995415, -0.095652, 0, 0.995415, -0.095652, 0, 0.995415, -0.095652, 0, 0.995415, -0.095652, 0, 0.996252, -0.086500, 0, 0.996252, -0.086500, 0, 0.996252, -0.086500, 0, 0.996252, -0.086500, 0, 0.996252, -0.086500, 0, 0.996252, -0.086500, 0, 0.996926, -0.078350, 0, 0.996926, -0.078350, 0, 0.996926, -0.078350, 0, 0.996926, -0.078350, 0, 0.996926, -0.078350, 0, 0.996926, -0.078350, 0, 0.997460, -0.071233, 0, 0.997460, -0.071233, 0, 0.997460, -0.071233, 0, 0.997460, -0.071233, 0, 0.997460, -0.071233, 0, 0.997460, -0.071233, 0, 0.997858, -0.065412, 0, 0.997858, -0.065412, 0, 0.997858, -0.065412, 0, 0.997858, -0.065412, 0, 0.997858, -0.065412, 0, 0.997858, -0.065412, 0, 0.998130, -0.061127, 0, 0.998130, -0.061127, 0, 0.998130, -0.061127, 0, 0.998130, -0.061127, 0, 0.998130, -0.061127, 0, 0.998130, -0.061127, 0, 0.998295, -0.058362, 0, 0.998295, -0.058362, 0, 0.998295, -0.058362, 0, 0.998295, -0.058362, 0, 0.998295, -0.058362, 0, 0.998295, -0.058362, 0, 0.998369, -0.057092, 0, 0.998369, -0.057092, 0, 0.998369, -0.057092, 0, 0.998369, -0.057092, 0, 0.998369, -0.057092, 0, 0.998369, -0.057092, 0, 0.998357, -0.057309, 0, 0.998357, -0.057309, 0, 0.998357, -0.057309, 0, 0.998357, -0.057309, 0, 0.998357, -0.057309, 0, 0.998357, -0.057309, 0, 0.998264, -0.058898, 0, 0.998264, -0.058898, 0, 0.998264, -0.058898, 0, 0.998264, -0.058898, 0, 0.998264, -0.058898, 0, 0.998264, -0.058898, 0, 0.998087, -0.061832, 0, 0.998087, -0.061832, 0, 0.998087, -0.061832, 0, 0.998087, -0.061832, 0, 0.998087, -0.061832, 0, 0.998087, -0.061832, 0, 0.997814, -0.066093, 0, 0.997814, -0.066093, 0, 0.997814, -0.066093, 0, 0.997814, -0.066093, 0, 0.997814, -0.066093, 0, 0.997814, -0.066093, 0, 0.997441, -0.071495, 0, 0.997441, -0.071495, 0, 0.997441, -0.071495, 0, 0.997441, -0.071495, 0, 0.997441, -0.071495, 0, 0.997441, -0.071495, 0, 0.996964, -0.077867, 0, 0.996964, -0.077867, 0, 0.996964, -0.077867, 0, 0.996964, -0.077867, 0, 0.996964, -0.077867, 0, 0.996964, -0.077867, 0, 0.996362, -0.085217, 0, 0.996362, -0.085217, 0, 0.996362, -0.085217, 0, 0.996362, -0.085217, 0, 0.996362, -0.085217, 0, 0.996362, -0.085217, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.996335, -0.085539, 0, 0.996335, -0.085539, 0, 0.996335, -0.085539, 0, 0.996335, -0.085539, 0, 0.996335, -0.085539, 0, 0.996335, -0.085539, 0, 0.996700, -0.081169, 0, 0.996700, -0.081169, 0, 0.996700, -0.081169, 0, 0.996700, -0.081169, 0, 0.996700, -0.081169, 0, 0.996700, -0.081169, 0, 0.997002, -0.077375, 0, 0.997002, -0.077375, 0, 0.997002, -0.077375, 0, 0.997002, -0.077375, 0, 0.997002, -0.077375, 0, 0.997002, -0.077375, 0, 0.997247, -0.074152, 0, 0.997247, -0.074152, 0, 0.997247, -0.074152, 0, 0.997247, -0.074152, 0, 0.997247, -0.074152, 0, 0.997247, -0.074152, 0, 0.997432, -0.071618, 0, 0.997432, -0.071618, 0, 0.997432, -0.071618, 0, 0.997432, -0.071618, 0, 0.997432, -0.071618, 0, 0.997432, -0.071618, 0, 0.997554, -0.069897, 0, 0.997554, -0.069897, 0, 0.997554, -0.069897, 0, 0.997554, -0.069897, 0, 0.997554, -0.069897, 0, 0.997554, -0.069897, 0, 0.997617, -0.068996, 0, 0.997617, -0.068996, 0, 0.997617, -0.068996, 0, 0.997617, -0.068996, 0, 0.997617, -0.068996, 0, 0.997617, -0.068996, 0, 0.997622, -0.068924, 0, 0.997622, -0.068924, 0, 0.997622, -0.068924, 0, 0.997622, -0.068924, 0, 0.997622, -0.068924, 0, 0.997622, -0.068924, 0, 0.997569, -0.069690, 0, 0.997569, -0.069690, 0, 0.997569, -0.069690, 0, 0.997569, -0.069690, 0, 0.997569, -0.069690, 0, 0.997569, -0.069690, 0, 0.997461, -0.071208, 0, 0.997461, -0.071208, 0, 0.997461, -0.071208, 0, 0.997461, -0.071208, 0, 0.997461, -0.071208, 0, 0.997461, -0.071208, 0, 0.997296, -0.073485, 0, 0.997296, -0.073485, 0, 0.997296, -0.073485, 0, 0.997296, -0.073485, 0, 0.997296, -0.073485, 0, 0.997296, -0.073485, 0, 0.997068, -0.076526, 0, 0.997068, -0.076526, 0, 0.997068, -0.076526, 0, 0.997068, -0.076526, 0, 0.997068, -0.076526, 0, 0.997068, -0.076526, 0, 0.996774, -0.080256, 0, 0.996774, -0.080256, 0, 0.996774, -0.080256, 0, 0.996774, -0.080256, 0, 0.996774, -0.080256, 0, 0.996774, -0.080256, 0, 0.996415, -0.084595, 0, 0.996415, -0.084595, 0, 0.996415, -0.084595, 0, 0.996415, -0.084595, 0, 0.996415, -0.084595, 0, 0.996415, -0.084595, 0, 0.995984, -0.089534, 0, 0.995984, -0.089534, 0, 0.995984, -0.089534, 0, 0.995984, -0.089534, 0, 0.995984, -0.089534, 0, 0.995984, -0.089534, 0, 0.995471, -0.095064, 0, 0.995471, -0.095064, 0, 0.995471, -0.095064, 0, 0.995471, -0.095064, 0, 0.995471, -0.095064, 0, 0.995471, -0.095064, 0, 0.997683, -0.068040, 0, 0.997683, -0.068040, 0, 0.997683, -0.068040, 0, 0.997683, -0.068040, 0, 0.997683, -0.068040, 0, 0.997683, -0.068040, 0, 0.997771, -0.066725, 0, 0.997771, -0.066725, 0, 0.997771, -0.066725, 0, 0.997771, -0.066725, 0, 0.997771, -0.066725, 0, 0.997771, -0.066725, 0, 0.997846, -0.065602, 0, 0.997846, -0.065602, 0, 0.997846, -0.065602, 0, 0.997846, -0.065602, 0, 0.997846, -0.065602, 0, 0.997846, -0.065602, 0, 0.997907, -0.064673, 0, 0.997907, -0.064673, 0, 0.997907, -0.064673, 0, 0.997907, -0.064673, 0, 0.997907, -0.064673, 0, 0.997907, -0.064673, 0, 0.997954, -0.063940, 0, 0.997954, -0.063940, 0, 0.997954, -0.063940, 0, 0.997954, -0.063940, 0, 0.997954, -0.063940, 0, 0.997954, -0.063940, 0, 0.997988, -0.063405, 0, 0.997988, -0.063405, 0, 0.997988, -0.063405, 0, 0.997988, -0.063405, 0, 0.997988, -0.063405, 0, 0.997988, -0.063405, 0, 0.998009, -0.063069, 0, 0.998009, -0.063069, 0, 0.998009, -0.063069, 0, 0.998009, -0.063069, 0, 0.998009, -0.063069, 0, 0.998009, -0.063069, 0, 0.998018, -0.062935, 0, 0.998018, -0.062935, 0, 0.998018, -0.062935, 0, 0.998018, -0.062935, 0, 0.998018, -0.062935, 0, 0.998018, -0.062935, 0, 0.998013, -0.063004, 0, 0.998013, -0.063004, 0, 0.998013, -0.063004, 0, 0.998013, -0.063004, 0, 0.998013, -0.063004, 0, 0.998013, -0.063004, 0, 0.997960, -0.063849, 0, 0.997960, -0.063849, 0, 0.997960, -0.063849, 0, 0.997960, -0.063849, 0, 0.997960, -0.063849, 0, 0.997960, -0.063849, 0, 0.997844, -0.065629, 0, 0.997844, -0.065629, 0, 0.997844, -0.065629, 0, 0.997844, -0.065629, 0, 0.997844, -0.065629, 0, 0.997844, -0.065629, 0, 0.997661, -0.068354, 0, 0.997661, -0.068354, 0, 0.997661, -0.068354, 0, 0.997661, -0.068354, 0, 0.997661, -0.068354, 0, 0.997661, -0.068354, 0, 0.997403, -0.072028, 0, 0.997403, -0.072028, 0, 0.997403, -0.072028, 0, 0.997403, -0.072028, 0, 0.997403, -0.072028, 0, 0.997403, -0.072028, 0, 0.997066, -0.076553, 0, 0.997066, -0.076553, 0, 0.997066, -0.076553, 0, 0.997066, -0.076553, 0, 0.997066, -0.076553, 0, 0.997066, -0.076553, 0, 0.996647, -0.081819, 0, 0.996647, -0.081819, 0, 0.996647, -0.081819, 0, 0.996647, -0.081819, 0, 0.996647, -0.081819, 0, 0.996647, -0.081819, 0, 0.996137, -0.087816, 0, 0.996137, -0.087816, 0, 0.996137, -0.087816, 0, 0.996137, -0.087816, 0, 0.996137, -0.087816, 0, 0.996137, -0.087816, 0, 0.995522, -0.094527, 0, 0.995522, -0.094527, 0, 0.995522, -0.094527, 0, 0.995522, -0.094527, 0, 0.995522, -0.094527, 0, 0.995522, -0.094527, 0, 0.994872, -0.101144, 0, 0.994872, -0.101144, 0, 0.994872, -0.101144, 0, 0.994872, -0.101144, 0, 0.994872, -0.101144, 0, 0.994872, -0.101144, 0, 0.994273, -0.106867, 0, 0.994273, -0.106867, 0, 0.994273, -0.106867, 0, 0.994273, -0.106867, 0, 0.994273, -0.106867, 0, 0.994273, -0.106867, 0, 0.993743, -0.111688, 0, 0.993743, -0.111688, 0, 0.993743, -0.111688, 0, 0.993743, -0.111688, 0, 0.993743, -0.111688, 0, 0.993743, -0.111688, 0, 0.993295, -0.115604, 0, 0.993295, -0.115604, 0, 0.993295, -0.115604, 0, 0.993295, -0.115604, 0, 0.993295, -0.115604, 0, 0.993295, -0.115604, 0, 0.992950, -0.118535, 0, 0.992950, -0.118535, 0, 0.992950, -0.118535, 0, 0.992950, -0.118535, 0, 0.992950, -0.118535, 0, 0.992950, -0.118535, 0, 0.992725, -0.120404, 0, 0.992725, -0.120404, 0, 0.992725, -0.120404, 0, 0.992725, -0.120404, 0, 0.992725, -0.120404, 0, 0.992725, -0.120404, 0, 0.992625, -0.121224, 0, 0.992625, -0.121224, 0, 0.992625, -0.121224, 0, 0.992625, -0.121224, 0, 0.992625, -0.121224, 0, 0.992625, -0.121224, 0, 0.992651, -0.121009, 0, 0.992651, -0.121009, 0, 0.992651, -0.121009, 0, 0.992651, -0.121009, 0, 0.992651, -0.121009, 0, 0.992651, -0.121009, 0, 0.992703, -0.120587, 0, 0.992703, -0.120587, 0, 0.992703, -0.120587, 0, 0.992703, -0.120587, 0, 0.992703, -0.120587, 0, 0.992703, -0.120587, 0, 0.992778, -0.119969, 0, 0.992778, -0.119969, 0, 0.992778, -0.119969, 0, 0.992778, -0.119969, 0, 0.992778, -0.119969, 0, 0.992778, -0.119969, 0, 0.992875, -0.119159, 0, 0.992875, -0.119159, 0, 0.992875, -0.119159, 0, 0.992875, -0.119159, 0, 0.992875, -0.119159, 0, 0.992875, -0.119159, 0, 0.992995, -0.118156, 0, 0.992995, -0.118156, 0, 0.992995, -0.118156, 0, 0.992995, -0.118156, 0, 0.992995, -0.118156, 0, 0.992995, -0.118156, 0, 0.993136, -0.116961, 0, 0.993136, -0.116961, 0, 0.993136, -0.116961, 0, 0.993136, -0.116961, 0, 0.993136, -0.116961, 0, 0.993136, -0.116961, 0, 0.993299, -0.115577, 0, 0.993299, -0.115577, 0, 0.993299, -0.115577, 0, 0.993299, -0.115577, 0, 0.993299, -0.115577, 0, 0.993299, -0.115577, 0, 0.993481, -0.114002, 0, 0.993481, -0.114002, 0, 0.993481, -0.114002, 0, 0.993481, -0.114002, 0, 0.993481, -0.114002, 0, 0.993481, -0.114002, 0, 0.993681, -0.112240, 0, 0.993681, -0.112240, 0, 0.993681, -0.112240, 0, 0.993681, -0.112240, 0, 0.993681, -0.112240, 0, 0.993681, -0.112240, 0, 0.992447, -0.122676, 0, 0.992447, -0.122676, 0, 0.992447, -0.122676, 0, 0.992447, -0.122676, 0, 0.992447, -0.122676, 0, 0.992447, -0.122676, 0, 0.992225, -0.124457, 0, 0.992225, -0.124457, 0, 0.992225, -0.124457, 0, 0.992225, -0.124457, 0, 0.992225, -0.124457, 0, 0.992225, -0.124457, 0, 0.992023, -0.126057, 0, 0.992023, -0.126057, 0, 0.992023, -0.126057, 0, 0.992023, -0.126057, 0, 0.992023, -0.126057, 0, 0.992023, -0.126057, 0, 0.991842, -0.127472, 0, 0.991842, -0.127472, 0, 0.991842, -0.127472, 0, 0.991842, -0.127472, 0, 0.991842, -0.127472, 0, 0.991842, -0.127472, 0, 0.991683, -0.128704, 0, 0.991683, -0.128704, 0, 0.991683, -0.128704, 0, 0.991683, -0.128704, 0, 0.991683, -0.128704, 0, 0.991683, -0.128704, 0, 0.991546, -0.129753, 0, 0.991546, -0.129753, 0, 0.991546, -0.129753, 0, 0.991546, -0.129753, 0, 0.991546, -0.129753, 0, 0.991546, -0.129753, 0, 0.991432, -0.130621, 0, 0.991432, -0.130621, 0, 0.991432, -0.130621, 0, 0.991432, -0.130621, 0, 0.991432, -0.130621, 0, 0.991432, -0.130621, 0, 0.991342, -0.131307, 0, 0.991342, -0.131307, 0, 0.991342, -0.131307, 0, 0.991342, -0.131307, 0, 0.991342, -0.131307, 0, 0.991342, -0.131307, 0, 0.991275, -0.131810, 0, 0.991275, -0.131810, 0, 0.991275, -0.131810, 0, 0.991275, -0.131810, 0, 0.991275, -0.131810, 0, 0.991275, -0.131810, 0, 0.991222, -0.132208, 0, 0.991222, -0.132208, 0, 0.991222, -0.132208, 0, 0.991222, -0.132208, 0, 0.991222, -0.132208, 0, 0.991222, -0.132208, 0, 0.991363, -0.131148, 0, 0.991363, -0.131148, 0, 0.991363, -0.131148, 0, 0.991363, -0.131148, 0, 0.991363, -0.131148, 0, 0.991363, -0.131148, 0, 0.991697, -0.128598, 0, 0.991697, -0.128598, 0, 0.991697, -0.128598, 0, 0.991697, -0.128598, 0, 0.991697, -0.128598, 0, 0.991697, -0.128598, 0, 0.992216, -0.124531, 0, 0.992216, -0.124531, 0, 0.992216, -0.124531, 0, 0.992216, -0.124531, 0, 0.992216, -0.124531, 0, 0.992216, -0.124531, 0, 0.992881, -0.119108, 0, 0.992881, -0.119108, 0, 0.992881, -0.119108, 0, 0.992881, -0.119108, 0, 0.992881, -0.119108, 0, 0.992881, -0.119108, 0, 0.993651, -0.112508, 0, 0.993651, -0.112508, 0, 0.993651, -0.112508, 0, 0.993651, -0.112508, 0, 0.993651, -0.112508, 0, 0.993651, -0.112508, 0, 0.994500, -0.104737, 0, 0.994500, -0.104737, 0, 0.994500, -0.104737, 0, 0.994500, -0.104737, 0, 0.994500, -0.104737, 0, 0.994500, -0.104737, 0, 0.995399, -0.095813, 0, 0.995399, -0.095813, 0, 0.995399, -0.095813, 0, 0.995399, -0.095813, 0, 0.995399, -0.095813, 0, 0.995399, -0.095813, 0, 0.996241, -0.086626, 0, 0.996241, -0.086626, 0, 0.996241, -0.086626, 0, 0.996241, -0.086626, 0, 0.996241, -0.086626, 0, 0.996241, -0.086626, 0, 0.996925, -0.078359, 0, 0.996925, -0.078359, 0, 0.996925, -0.078359, 0, 0.996925, -0.078359, 0, 0.996925, -0.078359, 0, 0.996925, -0.078359, 0, 0.997473, -0.071046, 0, 0.997473, -0.071046, 0, 0.997473, -0.071046, 0, 0.997473, -0.071046, 0, 0.997473, -0.071046, 0, 0.997473, -0.071046, 0, 0.997904, -0.064712, 0, 0.997904, -0.064712, 0, 0.997904, -0.064712, 0, 0.997904, -0.064712, 0, 0.997904, -0.064712, 0, 0.997904, -0.064712, 0, 0.998225, -0.059558, 0, 0.998225, -0.059558, 0, 0.998225, -0.059558, 0, 0.998225, -0.059558, 0, 0.998225, -0.059558, 0, 0.998225, -0.059558, 0, 0.998444, -0.055765, 0, 0.998444, -0.055765, 0, 0.998444, -0.055765, 0, 0.998444, -0.055765, 0, 0.998444, -0.055765, 0, 0.998444, -0.055765, 0, 0.998577, -0.053320, 0, 0.998577, -0.053320, 0, 0.998577, -0.053320, 0, 0.998577, -0.053320, 0, 0.998577, -0.053320, 0, 0.998577, -0.053320, 0, 0.998637, -0.052202, 0, 0.998637, -0.052202, 0, 0.998637, -0.052202, 0, 0.998637, -0.052202, 0, 0.998637, -0.052202, 0, 0.998637, -0.052202, 0, 0.998639, -0.052154, 0, 0.998639, -0.052154, 0, 0.998639, -0.052154, 0, 0.998639, -0.052154, 0, 0.998639, -0.052154, 0, 0.998639, -0.052154, 0, 0.998631, -0.052311, 0, 0.998631, -0.052311, 0, 0.998631, -0.052311, 0, 0.998631, -0.052311, 0, 0.998631, -0.052311, 0, 0.998631, -0.052311, 0, 0.998612, -0.052669, 0, 0.998612, -0.052669, 0, 0.998612, -0.052669, 0, 0.998612, -0.052669, 0, 0.998612, -0.052669, 0, 0.998612, -0.052669, 0, 0.998582, -0.053229, 0, 0.998582, -0.053229, 0, 0.998582, -0.053229, 0, 0.998582, -0.053229, 0, 0.998582, -0.053229, 0, 0.998582, -0.053229, 0, 0.998542, -0.053988, 0, 0.998542, -0.053988, 0, 0.998542, -0.053988, 0, 0.998542, -0.053988, 0, 0.998542, -0.053988, 0, 0.998542, -0.053988, 0, 0.998489, -0.054945, 0, 0.998489, -0.054945, 0, 0.998489, -0.054945, 0, 0.998489, -0.054945, 0, 0.998489, -0.054945, 0, 0.998489, -0.054945, 0, 0.998425, -0.056099, 0, 0.998425, -0.056099, 0, 0.998425, -0.056099, 0, 0.998425, -0.056099, 0, 0.998425, -0.056099, 0, 0.998425, -0.056099, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998673, -0.051508, 0, 0.998673, -0.051508, 0, 0.998673, -0.051508, 0, 0.998673, -0.051508, 0, 0.998673, -0.051508, 0, 0.998673, -0.051508, 0, 0.998735, -0.050282, 0, 0.998735, -0.050282, 0, 0.998735, -0.050282, 0, 0.998735, -0.050282, 0, 0.998735, -0.050282, 0, 0.998735, -0.050282, 0, 0.998180, -0.060305, 0, 0.998180, -0.060305, 0, 0.998180, -0.060305, 0, 0.998180, -0.060305, 0, 0.998180, -0.060305, 0, 0.998180, -0.060305, 0, 0.998090, -0.061773, 0, 0.998090, -0.061773, 0, 0.998090, -0.061773, 0, 0.998090, -0.061773, 0, 0.998090, -0.061773, 0, 0.998090, -0.061773, 0, 0.995038, -0.099497, 0, 0.995038, -0.099497, 0, 0.995038, -0.099497, 0, 0.995038, -0.099497, 0, 0.995038, -0.099497, 0, 0.995038, -0.099497, 0, 0.994648, -0.103326, 0, 0.994648, -0.103326, 0, 0.994648, -0.103326, 0, 0.994648, -0.103326, 0, 0.994648, -0.103326, 0, 0.994648, -0.103326, 0, 0.994327, -0.106365, 0, 0.994327, -0.106365, 0, 0.994327, -0.106365, 0, 0.994327, -0.106365, 0, 0.994327, -0.106365, 0, 0.994327, -0.106365, 0, 0.994084, -0.108615, 0, 0.994084, -0.108615, 0, 0.994084, -0.108615, 0, 0.994084, -0.108615, 0, 0.994084, -0.108615, 0, 0.994084, -0.108615, 0, 0.993923, -0.110077, 0, 0.993923, -0.110077, 0, 0.993923, -0.110077, 0, 0.993923, -0.110077, 0, 0.993923, -0.110077, 0, 0.993923, -0.110077, 0, 0.993825, -0.110961, 0, 0.993825, -0.110961, 0, 0.993825, -0.110961, 0, 0.993825, -0.110961, 0, 0.993825, -0.110961, 0, 0.993825, -0.110961, 0, 0.993767, -0.111474, 0, 0.993767, -0.111474, 0, 0.993767, -0.111474, 0, 0.993767, -0.111474, 0, 0.993767, -0.111474, 0, 0.993767, -0.111474, 0, 0.993751, -0.111618, 0, 0.993751, -0.111618, 0, 0.993751, -0.111618, 0, 0.993751, -0.111618, 0, 0.993751, -0.111618, 0, 0.993751, -0.111618, 0, 0.993776, -0.111397, 0, 0.993776, -0.111397, 0, 0.993776, -0.111397, 0, 0.993776, -0.111397, 0, 0.993776, -0.111397, 0, 0.993776, -0.111397, 0, 0.993922, -0.110088, 0, 0.993922, -0.110088, 0, 0.993922, -0.110088, 0, 0.993922, -0.110088, 0, 0.993922, -0.110088, 0, 0.993922, -0.110088, 0, 0.992779, -0.119958, 0, 0.992779, -0.119958, 0, 0.992779, -0.119958, 0, 0.992779, -0.119958, 0, 0.992779, -0.119958, 0, 0.992779, -0.119958, 0, 0.992606, -0.121381, 0, 0.992606, -0.121381, 0, 0.992606, -0.121381, 0, 0.992606, -0.121381, 0, 0.992606, -0.121381, 0, 0.992606, -0.121381, 0, 0.992552, -0.121823, 0, 0.992552, -0.121823, 0, 0.992552, -0.121823, 0, 0.992552, -0.121823, 0, 0.992552, -0.121823, 0, 0.992552, -0.121823, 0, 0.992590, -0.121516, 0, 0.992590, -0.121516, 0, 0.992590, -0.121516, 0, 0.992590, -0.121516, 0, 0.992590, -0.121516, 0, 0.992590, -0.121516, 0, 0.992720, -0.120449, 0, 0.992720, -0.120449, 0, 0.992720, -0.120449, 0, 0.992720, -0.120449, 0, 0.992720, -0.120449, 0, 0.992720, -0.120449, 0, 0.992940, -0.118615, 0, 0.992940, -0.118615, 0, 0.992940, -0.118615, 0, 0.992940, -0.118615, 0, 0.992940, -0.118615, 0, 0.992940, -0.118615, 0, 0.993269, -0.115833, 0, 0.993269, -0.115833, 0, 0.993269, -0.115833, 0, 0.993269, -0.115833, 0, 0.993269, -0.115833, 0, 0.993269, -0.115833, 0, 0.993717, -0.111918, 0, 0.993717, -0.111918, 0, 0.993717, -0.111918, 0, 0.993717, -0.111918, 0, 0.993717, -0.111918, 0, 0.993717, -0.111918, 0, 0.994273, -0.106868, 0, 0.994273, -0.106868, 0, 0.994273, -0.106868, 0, 0.994273, -0.106868, 0, 0.994273, -0.106868, 0, 0.994273, -0.106868, 0, 0.994918, -0.100686, 0, 0.994918, -0.100686, 0, 0.994918, -0.100686, 0, 0.994918, -0.100686, 0, 0.994918, -0.100686, 0, 0.994918, -0.100686, 0, 0.998044, -0.062516, 0, 0.998044, -0.062516, 0, 0.998044, -0.062516, 0, 0.998044, -0.062516, 0, 0.998044, -0.062516, 0, 0.998044, -0.062516, 0, 0.998009, -0.063070, 0, 0.998009, -0.063070, 0, 0.998009, -0.063070, 0, 0.998009, -0.063070, 0, 0.998009, -0.063070, 0, 0.998009, -0.063070, 0, 0.998616, -0.052590, 0, 0.998616, -0.052590, 0, 0.998616, -0.052590, 0, 0.998616, -0.052590, 0, 0.998616, -0.052590, 0, 0.998616, -0.052590, 0, 0.998640, -0.052128, 0, 0.998640, -0.052128, 0, 0.998640, -0.052128, 0, 0.998640, -0.052128, 0, 0.998640, -0.052128, 0, 0.998640, -0.052128, 0, 0.999510, -0.031304, 0, 0.999510, -0.031304, 0, 0.999510, -0.031304, 0, 0.999510, -0.031304, 0, 0.999510, -0.031304, 0, 0.999510, -0.031304, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998800, -0.048982, 0, 0.998800, -0.048982, 0, 0.998800, -0.048982, 0, 0.998800, -0.048982, 0, 0.998800, -0.048982, 0, 0.998800, -0.048982, 0, 0.993145, -0.116891, 0, 0.993145, -0.116891, 0, 0.993145, -0.116891, 0, 0.993145, -0.116891, 0, 0.993145, -0.116891, 0, 0.993145, -0.116891, 0, 0.992989, -0.118206, 0, 0.992989, -0.118206, 0, 0.992989, -0.118206, 0, 0.992989, -0.118206, 0, 0.992989, -0.118206, 0, 0.992989, -0.118206, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999767, -0.021573, 0, 0.999767, -0.021573, 0, 0.999767, -0.021573, 0, 0.999767, -0.021573, 0, 0.999767, -0.021573, 0, 0.999767, -0.021573, 0, 0.999377, -0.035287, 0, 0.999377, -0.035287, 0, 0.999377, -0.035287, 0, 0.999377, -0.035287, 0, 0.999377, -0.035287, 0, 0.999377, -0.035287, 0, 0.999363, -0.035689, 0, 0.999363, -0.035689, 0, 0.999363, -0.035689, 0, 0.999363, -0.035689, 0, 0.999363, -0.035689, 0, 0.999363, -0.035689, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998502, -0.054709, 0, 0.998502, -0.054709, 0, 0.998502, -0.054709, 0, 0.998502, -0.054709, 0, 0.998502, -0.054709, 0, 0.998502, -0.054709, 0, 0.998421, -0.056176, 0, 0.998421, -0.056176, 0, 0.998421, -0.056176, 0, 0.998421, -0.056176, 0, 0.998421, -0.056176, 0, 0.998421, -0.056176, 0, 0.998289, -0.058476, 0, 0.998289, -0.058476, 0, 0.998289, -0.058476, 0, 0.998289, -0.058476, 0, 0.998289, -0.058476, 0, 0.998289, -0.058476, 0, 0.998100, -0.061610, 0, 0.998100, -0.061610, 0, 0.998100, -0.061610, 0, 0.998100, -0.061610, 0, 0.998100, -0.061610, 0, 0.998100, -0.061610, 0, 0.997855, -0.065465, 0, 0.997855, -0.065465, 0, 0.997855, -0.065465, 0, 0.997855, -0.065465, 0, 0.997855, -0.065465, 0, 0.997855, -0.065465, 0, 0.997553, -0.069920, 0, 0.997553, -0.069920, 0, 0.997553, -0.069920, 0, 0.997553, -0.069920, 0, 0.997553, -0.069920, 0, 0.997553, -0.069920, 0, 0.997186, -0.074968, 0, 0.997186, -0.074968, 0, 0.997186, -0.074968, 0, 0.997186, -0.074968, 0, 0.997186, -0.074968, 0, 0.997186, -0.074968, 0, 0.996747, -0.080598, 0, 0.996747, -0.080598, 0, 0.996747, -0.080598, 0, 0.996747, -0.080598, 0, 0.996747, -0.080598, 0, 0.996747, -0.080598, 0, 0.996280, -0.086169, 0, 0.996280, -0.086169, 0, 0.996280, -0.086169, 0, 0.996280, -0.086169, 0, 0.996280, -0.086169, 0, 0.996280, -0.086169, 0, 0.995847, -0.091045, 0, 0.995847, -0.091045, 0, 0.995847, -0.091045, 0, 0.995847, -0.091045, 0, 0.995847, -0.091045, 0, 0.995847, -0.091045, 0, 0.995456, -0.095219, 0, 0.995456, -0.095219, 0, 0.995456, -0.095219, 0, 0.995456, -0.095219, 0, 0.995456, -0.095219, 0, 0.995456, -0.095219, 0, 0.995118, -0.098691, 0, 0.995118, -0.098691, 0, 0.995118, -0.098691, 0, 0.995118, -0.098691, 0, 0.995118, -0.098691, 0, 0.995118, -0.098691, 0, 0.994850, -0.101358, 0, 0.994850, -0.101358, 0, 0.994850, -0.101358, 0, 0.994850, -0.101358, 0, 0.994850, -0.101358, 0, 0.994850, -0.101358, 0, 0.994668, -0.103125, 0, 0.994668, -0.103125, 0, 0.994668, -0.103125, 0, 0.994668, -0.103125, 0, 0.994668, -0.103125, 0, 0.994668, -0.103125, 0, 0.994577, -0.103999, 0, 0.994577, -0.103999, 0, 0.994577, -0.103999, 0, 0.994577, -0.103999, 0, 0.994577, -0.103999, 0, 0.994577, -0.103999, 0, 0.994578, -0.103990, 0, 0.994578, -0.103990, 0, 0.994578, -0.103990, 0, 0.994578, -0.103990, 0, 0.994578, -0.103990, 0, 0.994578, -0.103990, 0, 0.994665, -0.103159, 0, 0.994665, -0.103159, 0, 0.994665, -0.103159, 0, 0.994665, -0.103159, 0, 0.994665, -0.103159, 0, 0.994665, -0.103159, 0, 0.994830, -0.101552, 0, 0.994830, -0.101552, 0, 0.994830, -0.101552, 0, 0.994830, -0.101552, 0, 0.994830, -0.101552, 0, 0.994830, -0.101552, 0, 0.995070, -0.099176, 0, 0.995070, -0.099176, 0, 0.995070, -0.099176, 0, 0.995070, -0.099176, 0, 0.995070, -0.099176, 0, 0.995070, -0.099176, 0, 0.995378, -0.096031, 0, 0.995378, -0.096031, 0, 0.995378, -0.096031, 0, 0.995378, -0.096031, 0, 0.995378, -0.096031, 0, 0.995378, -0.096031, 0, 0.995737, -0.092241, 0, 0.995737, -0.092241, 0, 0.995737, -0.092241, 0, 0.995737, -0.092241, 0, 0.995737, -0.092241, 0, 0.995737, -0.092241, 0, 0.996127, -0.087921, 0, 0.996127, -0.087921, 0, 0.996127, -0.087921, 0, 0.996127, -0.087921, 0, 0.996127, -0.087921, 0, 0.996127, -0.087921, 0, 0.996544, -0.083066, 0, 0.996544, -0.083066, 0, 0.996544, -0.083066, 0, 0.996544, -0.083066, 0, 0.996544, -0.083066, 0, 0.996544, -0.083066, 0, 0.996979, -0.077666, 0, 0.996979, -0.077666, 0, 0.996979, -0.077666, 0, 0.996979, -0.077666, 0, 0.996979, -0.077666, 0, 0.996979, -0.077666, 0, 0.997381, -0.072330, 0, 0.997381, -0.072330, 0, 0.997381, -0.072330, 0, 0.997381, -0.072330, 0, 0.997381, -0.072330, 0, 0.997381, -0.072330, 0, 0.997712, -0.067612, 0, 0.997712, -0.067612, 0, 0.997712, -0.067612, 0, 0.997712, -0.067612, 0, 0.997712, -0.067612, 0, 0.997712, -0.067612, 0, 0.997981, -0.063509, 0, 0.997981, -0.063509, 0, 0.997981, -0.063509, 0, 0.997981, -0.063509, 0, 0.997981, -0.063509, 0, 0.997981, -0.063509, 0, 0.998197, -0.060017, 0, 0.998197, -0.060017, 0, 0.998197, -0.060017, 0, 0.998197, -0.060017, 0, 0.998197, -0.060017, 0, 0.998197, -0.060017, 0, 0.998360, -0.057252, 0, 0.998360, -0.057252, 0, 0.998360, -0.057252, 0, 0.998360, -0.057252, 0, 0.998360, -0.057252, 0, 0.998360, -0.057252, 0, 0.998468, -0.055334, 0, 0.998468, -0.055334, 0, 0.998468, -0.055334, 0, 0.998468, -0.055334, 0, 0.998468, -0.055334, 0, 0.998468, -0.055334, 0, 0.998526, -0.054269, 0, 0.998526, -0.054269, 0, 0.998526, -0.054269, 0, 0.998526, -0.054269, 0, 0.998526, -0.054269, 0, 0.998526, -0.054269, 0, 0.998985, -0.045038, 0, 0.998985, -0.045038, 0, 0.998985, -0.045038, 0, 0.998985, -0.045038, 0, 0.998985, -0.045038, 0, 0.998985, -0.045038, 0, 0.998920, -0.046468, 0, 0.998920, -0.046468, 0, 0.998920, -0.046468, 0, 0.998920, -0.046468, 0, 0.998920, -0.046468, 0, 0.998920, -0.046468, 0, 0.998797, -0.049038, 0, 0.998797, -0.049038, 0, 0.998797, -0.049038, 0, 0.998797, -0.049038, 0, 0.998797, -0.049038, 0, 0.998797, -0.049038, 0, 0.998609, -0.052735, 0, 0.998609, -0.052735, 0, 0.998609, -0.052735, 0, 0.998609, -0.052735, 0, 0.998609, -0.052735, 0, 0.998609, -0.052735, 0, 0.998349, -0.057430, 0, 0.998349, -0.057430, 0, 0.998349, -0.057430, 0, 0.998349, -0.057430, 0, 0.998349, -0.057430, 0, 0.998349, -0.057430, 0, 0.998013, -0.063003, 0, 0.998013, -0.063003, 0, 0.998013, -0.063003, 0, 0.998013, -0.063003, 0, 0.998013, -0.063003, 0, 0.998013, -0.063003, 0, 0.997585, -0.069461, 0, 0.997585, -0.069461, 0, 0.997585, -0.069461, 0, 0.997585, -0.069461, 0, 0.997585, -0.069461, 0, 0.997585, -0.069461, 0, 0.997046, -0.076813, 0, 0.997046, -0.076813, 0, 0.997046, -0.076813, 0, 0.997046, -0.076813, 0, 0.997046, -0.076813, 0, 0.997046, -0.076813, 0, 0.996449, -0.084204, 0, 0.996449, -0.084204, 0, 0.996449, -0.084204, 0, 0.996449, -0.084204, 0, 0.996449, -0.084204, 0, 0.996449, -0.084204, 0, 0.995866, -0.090840, 0, 0.995866, -0.090840, 0, 0.995866, -0.090840, 0, 0.995866, -0.090840, 0, 0.995866, -0.090840, 0, 0.995866, -0.090840, 0, 0.995310, -0.096738, 0, 0.995310, -0.096738, 0, 0.995310, -0.096738, 0, 0.995310, -0.096738, 0, 0.995310, -0.096738, 0, 0.995310, -0.096738, 0, 0.994793, -0.101916, 0, 0.994793, -0.101916, 0, 0.994793, -0.101916, 0, 0.994793, -0.101916, 0, 0.994793, -0.101916, 0, 0.994793, -0.101916, 0, 0.994341, -0.106236, 0, 0.994341, -0.106236, 0, 0.994341, -0.106236, 0, 0.994341, -0.106236, 0, 0.994341, -0.106236, 0, 0.994341, -0.106236, 0, 0.993981, -0.109554, 0, 0.993981, -0.109554, 0, 0.993981, -0.109554, 0, 0.993981, -0.109554, 0, 0.993981, -0.109554, 0, 0.993981, -0.109554, 0, 0.993723, -0.111865, 0, 0.993723, -0.111865, 0, 0.993723, -0.111865, 0, 0.993723, -0.111865, 0, 0.993723, -0.111865, 0, 0.993723, -0.111865, 0, 0.993577, -0.113161, 0, 0.993577, -0.113161, 0, 0.993577, -0.113161, 0, 0.993577, -0.113161, 0, 0.993577, -0.113161, 0, 0.993577, -0.113161, 0, 0.993556, -0.113341, 0, 0.993556, -0.113341, 0, 0.993556, -0.113341, 0, 0.993556, -0.113341, 0, 0.993556, -0.113341, 0, 0.993556, -0.113341, 0, 0.993675, -0.112297, 0, 0.993675, -0.112297, 0, 0.993675, -0.112297, 0, 0.993675, -0.112297, 0, 0.993675, -0.112297, 0, 0.993675, -0.112297, 0, 0.993931, -0.110006, 0, 0.993931, -0.110006, 0, 0.993931, -0.110006, 0, 0.993931, -0.110006, 0, 0.993931, -0.110006, 0, 0.993931, -0.110006, 0, 0.994318, -0.106452, 0, 0.994318, -0.106452, 0, 0.994318, -0.106452, 0, 0.994318, -0.106452, 0, 0.994318, -0.106452, 0, 0.994318, -0.106452, 0, 0.994810, -0.101751, 0, 0.994810, -0.101751, 0, 0.994810, -0.101751, 0, 0.994810, -0.101751, 0, 0.994810, -0.101751, 0, 0.994810, -0.101751, 0, 0.995379, -0.096025, 0, 0.995379, -0.096025, 0, 0.995379, -0.096025, 0, 0.995379, -0.096025, 0, 0.995379, -0.096025, 0, 0.995379, -0.096025, 0, 0.996006, -0.089282, 0, 0.996006, -0.089282, 0, 0.996006, -0.089282, 0, 0.996006, -0.089282, 0, 0.996006, -0.089282, 0, 0.996006, -0.089282, 0, 0.996671, -0.081532, 0, 0.996671, -0.081532, 0, 0.996671, -0.081532, 0, 0.996671, -0.081532, 0, 0.996671, -0.081532, 0, 0.996671, -0.081532, 0, 0.997283, -0.073660, 0, 0.997283, -0.073660, 0, 0.997283, -0.073660, 0, 0.997283, -0.073660, 0, 0.997283, -0.073660, 0, 0.997283, -0.073660, 0, 0.997779, -0.066617, 0, 0.997779, -0.066617, 0, 0.997779, -0.066617, 0, 0.997779, -0.066617, 0, 0.997779, -0.066617, 0, 0.997779, -0.066617, 0, 0.998173, -0.060426, 0, 0.998173, -0.060426, 0, 0.998173, -0.060426, 0, 0.998173, -0.060426, 0, 0.998173, -0.060426, 0, 0.998173, -0.060426, 0, 0.998481, -0.055102, 0, 0.998481, -0.055102, 0, 0.998481, -0.055102, 0, 0.998481, -0.055102, 0, 0.998481, -0.055102, 0, 0.998481, -0.055102, 0, 0.998709, -0.050795, 0, 0.998709, -0.050795, 0, 0.998709, -0.050795, 0, 0.998709, -0.050795, 0, 0.998709, -0.050795, 0, 0.998709, -0.050795, 0, 0.998864, -0.047644, 0, 0.998864, -0.047644, 0, 0.998864, -0.047644, 0, 0.998864, -0.047644, 0, 0.998864, -0.047644, 0, 0.998864, -0.047644, 0, 0.998958, -0.045639, 0, 0.998958, -0.045639, 0, 0.998958, -0.045639, 0, 0.998958, -0.045639, 0, 0.998958, -0.045639, 0, 0.998958, -0.045639, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.994871, -0.101152, 0, 0.994871, -0.101152, 0, 0.994871, -0.101152, 0, 0.994871, -0.101152, 0, 0.994871, -0.101152, 0, 0.994871, -0.101152, 0, 0.994770, -0.102137, 0, 0.994770, -0.102137, 0, 0.994770, -0.102137, 0, 0.994770, -0.102137, 0, 0.994770, -0.102137, 0, 0.994770, -0.102137, 0, 0.997809, -0.066165, 0, 0.997809, -0.066165, 0, 0.997809, -0.066165, 0, 0.997809, -0.066165, 0, 0.997809, -0.066165, 0, 0.997809, -0.066165, 0, 0.998059, -0.062281, 0, 0.998059, -0.062281, 0, 0.998059, -0.062281, 0, 0.998059, -0.062281, 0, 0.998059, -0.062281, 0, 0.998059, -0.062281, 0, 0.998262, -0.058940, 0, 0.998262, -0.058940, 0, 0.998262, -0.058940, 0, 0.998262, -0.058940, 0, 0.998262, -0.058940, 0, 0.998262, -0.058940, 0, 0.998423, -0.056142, 0, 0.998423, -0.056142, 0, 0.998423, -0.056142, 0, 0.998423, -0.056142, 0, 0.998423, -0.056142, 0, 0.998423, -0.056142, 0, 0.998547, -0.053884, 0, 0.998547, -0.053884, 0, 0.998547, -0.053884, 0, 0.998547, -0.053884, 0, 0.998547, -0.053884, 0, 0.998547, -0.053884, 0, 0.998639, -0.052164, 0, 0.998639, -0.052164, 0, 0.998639, -0.052164, 0, 0.998639, -0.052164, 0, 0.998639, -0.052164, 0, 0.998639, -0.052164, 0, 0.998700, -0.050980, 0, 0.998700, -0.050980, 0, 0.998700, -0.050980, 0, 0.998700, -0.050980, 0, 0.998700, -0.050980, 0, 0.998700, -0.050980, 0, 0.998733, -0.050333, 0, 0.998733, -0.050333, 0, 0.998733, -0.050333, 0, 0.998733, -0.050333, 0, 0.998733, -0.050333, 0, 0.998733, -0.050333, 0, 0.998738, -0.050229, 0, 0.998738, -0.050229, 0, 0.998738, -0.050229, 0, 0.998738, -0.050229, 0, 0.998738, -0.050229, 0, 0.998738, -0.050229, 0, 0.998712, -0.050739, 0, 0.998712, -0.050739, 0, 0.998712, -0.050739, 0, 0.998712, -0.050739, 0, 0.998712, -0.050739, 0, 0.998712, -0.050739, 0, 0.998654, -0.051868, 0, 0.998654, -0.051868, 0, 0.998654, -0.051868, 0, 0.998654, -0.051868, 0, 0.998654, -0.051868, 0, 0.998654, -0.051868, 0, 0.998561, -0.053621, 0, 0.998561, -0.053621, 0, 0.998561, -0.053621, 0, 0.998561, -0.053621, 0, 0.998561, -0.053621, 0, 0.998561, -0.053621, 0, 0.998431, -0.056000, 0, 0.998431, -0.056000, 0, 0.998431, -0.056000, 0, 0.998431, -0.056000, 0, 0.998431, -0.056000, 0, 0.998431, -0.056000, 0, 0.998263, -0.058914, 0, 0.998263, -0.058914, 0, 0.998263, -0.058914, 0, 0.998263, -0.058914, 0, 0.998263, -0.058914, 0, 0.998263, -0.058914, 0, 0.998059, -0.062270, 0, 0.998059, -0.062270, 0, 0.998059, -0.062270, 0, 0.998059, -0.062270, 0, 0.998059, -0.062270, 0, 0.998059, -0.062270, 0, 0.997815, -0.066063, 0, 0.997815, -0.066063, 0, 0.997815, -0.066063, 0, 0.997815, -0.066063, 0, 0.997815, -0.066063, 0, 0.997815, -0.066063, 0, 0.997527, -0.070285, 0, 0.997527, -0.070285, 0, 0.997527, -0.070285, 0, 0.997527, -0.070285, 0, 0.997527, -0.070285, 0, 0.997527, -0.070285, 0, 0.994440, -0.105307, 0, 0.994440, -0.105307, 0, 0.994440, -0.105307, 0, 0.994440, -0.105307, 0, 0.994440, -0.105307, 0, 0.994440, -0.105307, 0, 0.994326, -0.106381, 0, 0.994326, -0.106381, 0, 0.994326, -0.106381, 0, 0.994326, -0.106381, 0, 0.994326, -0.106381, 0, 0.994326, -0.106381, 0, 0.999008, -0.044534, 0, 0.999008, -0.044534, 0, 0.999008, -0.044534, 0, 0.999008, -0.044534, 0, 0.999008, -0.044534, 0, 0.999008, -0.044534, 0, 0.999028, -0.044080, 0, 0.999028, -0.044080, 0, 0.999028, -0.044080, 0, 0.999028, -0.044080, 0, 0.999028, -0.044080, 0, 0.999028, -0.044080, 0, 0.998601, -0.052874, 0, 0.998601, -0.052874, 0, 0.998601, -0.052874, 0, 0.998601, -0.052874, 0, 0.998601, -0.052874, 0, 0.998601, -0.052874, 0, 0.998734, -0.050295, 0, 0.998734, -0.050295, 0, 0.998734, -0.050295, 0, 0.998734, -0.050295, 0, 0.998734, -0.050295, 0, 0.998734, -0.050295, 0, 0.998845, -0.048052, 0, 0.998845, -0.048052, 0, 0.998845, -0.048052, 0, 0.998845, -0.048052, 0, 0.998845, -0.048052, 0, 0.998845, -0.048052, 0, 0.998935, -0.046142, 0, 0.998935, -0.046142, 0, 0.998935, -0.046142, 0, 0.998935, -0.046142, 0, 0.998935, -0.046142, 0, 0.998935, -0.046142, 0, 0.999007, -0.044565, 0, 0.999007, -0.044565, 0, 0.999007, -0.044565, 0, 0.999007, -0.044565, 0, 0.999007, -0.044565, 0, 0.999007, -0.044565, 0, 0.999061, -0.043316, 0, 0.999061, -0.043316, 0, 0.999061, -0.043316, 0, 0.999061, -0.043316, 0, 0.999061, -0.043316, 0, 0.999061, -0.043316, 0, 0.999101, -0.042389, 0, 0.999101, -0.042389, 0, 0.999101, -0.042389, 0, 0.999101, -0.042389, 0, 0.999101, -0.042389, 0, 0.999101, -0.042389, 0, 0.999127, -0.041782, 0, 0.999127, -0.041782, 0, 0.999127, -0.041782, 0, 0.999127, -0.041782, 0, 0.999127, -0.041782, 0, 0.999127, -0.041782, 0, 0.999139, -0.041493, 0, 0.999139, -0.041493, 0, 0.999139, -0.041493, 0, 0.999139, -0.041493, 0, 0.999139, -0.041493, 0, 0.999139, -0.041493, 0, 0.999131, -0.041688, 0, 0.999131, -0.041688, 0, 0.999131, -0.041688, 0, 0.999131, -0.041688, 0, 0.999131, -0.041688, 0, 0.999131, -0.041688, 0, 0.999088, -0.042697, 0, 0.999088, -0.042697, 0, 0.999088, -0.042697, 0, 0.999088, -0.042697, 0, 0.999088, -0.042697, 0, 0.999088, -0.042697, 0, 0.999009, -0.044512, 0, 0.999009, -0.044512, 0, 0.999009, -0.044512, 0, 0.999009, -0.044512, 0, 0.999009, -0.044512, 0, 0.999009, -0.044512, 0, 0.998889, -0.047126, 0, 0.998889, -0.047126, 0, 0.998889, -0.047126, 0, 0.998889, -0.047126, 0, 0.998889, -0.047126, 0, 0.998889, -0.047126, 0, 0.998723, -0.050515, 0, 0.998723, -0.050515, 0, 0.998723, -0.050515, 0, 0.998723, -0.050515, 0, 0.998723, -0.050515, 0, 0.998723, -0.050515, 0, 0.998505, -0.054656, 0, 0.998505, -0.054656, 0, 0.998505, -0.054656, 0, 0.998505, -0.054656, 0, 0.998505, -0.054656, 0, 0.998505, -0.054656, 0, 0.998225, -0.059552, 0, 0.998225, -0.059552, 0, 0.998225, -0.059552, 0, 0.998225, -0.059552, 0, 0.998225, -0.059552, 0, 0.998225, -0.059552, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999799, -0.020024, 0, 0.999799, -0.020024, 0, 0.999799, -0.020024, 0, 0.999799, -0.020024, 0, 0.999799, -0.020024, 0, 0.999799, -0.020024, 0, 0.999616, -0.027720, 0, 0.999616, -0.027720, 0, 0.999616, -0.027720, 0, 0.999616, -0.027720, 0, 0.999616, -0.027720, 0, 0.999616, -0.027720, 0, 0.999590, -0.028624, 0, 0.999590, -0.028624, 0, 0.999590, -0.028624, 0, 0.999590, -0.028624, 0, 0.999590, -0.028624, 0, 0.999590, -0.028624, 0, 0.998707, -0.050841, 0, 0.998707, -0.050841, 0, 0.998707, -0.050841, 0, 0.998707, -0.050841, 0, 0.998707, -0.050841, 0, 0.998707, -0.050841, 0, 0.998780, -0.049392, 0, 0.998780, -0.049392, 0, 0.998780, -0.049392, 0, 0.998780, -0.049392, 0, 0.998780, -0.049392, 0, 0.998780, -0.049392, 0, 0.998369, -0.057085, 0, 0.998369, -0.057085, 0, 0.998369, -0.057085, 0, 0.998369, -0.057085, 0, 0.998369, -0.057085, 0, 0.998369, -0.057085, 0, 0.998272, -0.058760, 0, 0.998272, -0.058760, 0, 0.998272, -0.058760, 0, 0.998272, -0.058760, 0, 0.998272, -0.058760, 0, 0.998272, -0.058760, 0, 0.995475, -0.095019, 0, 0.995475, -0.095019, 0, 0.995475, -0.095019, 0, 0.995475, -0.095019, 0, 0.995475, -0.095019, 0, 0.995475, -0.095019, 0, 0.995384, -0.095974, 0, 0.995384, -0.095974, 0, 0.995384, -0.095974, 0, 0.995384, -0.095974, 0, 0.995384, -0.095974, 0, 0.995384, -0.095974, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997911, -0.064599, 0, 0.997911, -0.064599, 0, 0.997911, -0.064599, 0, 0.997911, -0.064599, 0, 0.997911, -0.064599, 0, 0.997911, -0.064599, 0, 0.997867, -0.065285, 0, 0.997867, -0.065285, 0, 0.997867, -0.065285, 0, 0.997867, -0.065285, 0, 0.997867, -0.065285, 0, 0.997867, -0.065285, 0, 0.997806, -0.066208, 0, 0.997806, -0.066208, 0, 0.997806, -0.066208, 0, 0.997806, -0.066208, 0, 0.997806, -0.066208, 0, 0.997806, -0.066208, 0, 0.997728, -0.067368, 0, 0.997728, -0.067368, 0, 0.997728, -0.067368, 0, 0.997728, -0.067368, 0, 0.997728, -0.067368, 0, 0.997728, -0.067368, 0, 0.997628, -0.068832, 0, 0.997628, -0.068832, 0, 0.997628, -0.068832, 0, 0.997628, -0.068832, 0, 0.997628, -0.068832, 0, 0.997628, -0.068832, 0, 0.997500, -0.070666, 0, 0.997500, -0.070666, 0, 0.997500, -0.070666, 0, 0.997500, -0.070666, 0, 0.997500, -0.070666, 0, 0.997500, -0.070666, 0, 0.997342, -0.072868, 0, 0.997342, -0.072868, 0, 0.997342, -0.072868, 0, 0.997342, -0.072868, 0, 0.997342, -0.072868, 0, 0.997342, -0.072868, 0, 0.997151, -0.075435, 0, 0.997151, -0.075435, 0, 0.997151, -0.075435, 0, 0.997151, -0.075435, 0, 0.997151, -0.075435, 0, 0.997151, -0.075435, 0, 0.996992, -0.077511, 0, 0.996992, -0.077511, 0, 0.996992, -0.077511, 0, 0.996992, -0.077511, 0, 0.996992, -0.077511, 0, 0.996992, -0.077511, 0, 0.996849, -0.079325, 0, 0.996849, -0.079325, 0, 0.996849, -0.079325, 0, 0.996849, -0.079325, 0, 0.996849, -0.079325, 0, 0.996849, -0.079325, 0, 0.996724, -0.080876, 0, 0.996724, -0.080876, 0, 0.996724, -0.080876, 0, 0.996724, -0.080876, 0, 0.996724, -0.080876, 0, 0.996724, -0.080876, 0, 0.996619, -0.082163, 0, 0.996619, -0.082163, 0, 0.996619, -0.082163, 0, 0.996619, -0.082163, 0, 0.996619, -0.082163, 0, 0.996619, -0.082163, 0, 0.996538, -0.083145, 0, 0.996538, -0.083145, 0, 0.996538, -0.083145, 0, 0.996538, -0.083145, 0, 0.996538, -0.083145, 0, 0.996538, -0.083145, 0, 0.996484, -0.083782, 0, 0.996484, -0.083782, 0, 0.996484, -0.083782, 0, 0.996484, -0.083782, 0, 0.996484, -0.083782, 0, 0.996484, -0.083782, 0, 0.996459, -0.084075, 0, 0.996459, -0.084075, 0, 0.996459, -0.084075, 0, 0.996459, -0.084075, 0, 0.996459, -0.084075, 0, 0.996459, -0.084075, 0, 0.996463, -0.084027, 0, 0.996463, -0.084027, 0, 0.996463, -0.084027, 0, 0.996463, -0.084027, 0, 0.996463, -0.084027, 0, 0.996463, -0.084027, 0, 0.996512, -0.083445, 0, 0.996512, -0.083445, 0, 0.996512, -0.083445, 0, 0.996512, -0.083445, 0, 0.996512, -0.083445, 0, 0.996512, -0.083445, 0, 0.996608, -0.082301, 0, 0.996608, -0.082301, 0, 0.996608, -0.082301, 0, 0.996608, -0.082301, 0, 0.996608, -0.082301, 0, 0.996608, -0.082301, 0, 0.996747, -0.080599, 0, 0.996747, -0.080599, 0, 0.996747, -0.080599, 0, 0.996747, -0.080599, 0, 0.996747, -0.080599, 0, 0.996747, -0.080599, 0, 0.996927, -0.078341, 0, 0.996927, -0.078341, 0, 0.996927, -0.078341, 0, 0.996927, -0.078341, 0, 0.996927, -0.078341, 0, 0.996927, -0.078341, 0, 0.997136, -0.075636, 0, 0.997136, -0.075636, 0, 0.997136, -0.075636, 0, 0.997136, -0.075636, 0, 0.997136, -0.075636, 0, 0.997136, -0.075636, 0, 0.997362, -0.072588, 0, 0.997362, -0.072588, 0, 0.997362, -0.072588, 0, 0.997362, -0.072588, 0, 0.997362, -0.072588, 0, 0.997362, -0.072588, 0, 0.997603, -0.069194, 0, 0.997603, -0.069194, 0, 0.997603, -0.069194, 0, 0.997603, -0.069194, 0, 0.997603, -0.069194, 0, 0.997603, -0.069194, 0, 0.997856, -0.065448, 0, 0.997856, -0.065448, 0, 0.997856, -0.065448, 0, 0.997856, -0.065448, 0, 0.997856, -0.065448, 0, 0.997856, -0.065448, 0, 0.997975, -0.063607, 0, 0.997975, -0.063607, 0, 0.997975, -0.063607, 0, 0.997975, -0.063607, 0, 0.997975, -0.063607, 0, 0.997975, -0.063607, 0, 0.996134, -0.087842, 0, 0.996134, -0.087842, 0, 0.996134, -0.087842, 0, 0.996134, -0.087842, 0, 0.996134, -0.087842, 0, 0.996134, -0.087842, 0, 0.996069, -0.088585, 0, 0.996069, -0.088585, 0, 0.996069, -0.088585, 0, 0.996069, -0.088585, 0, 0.996069, -0.088585, 0, 0.996069, -0.088585, 0, 0.996693, -0.081253, 0, 0.996693, -0.081253, 0, 0.996693, -0.081253, 0, 0.996693, -0.081253, 0, 0.996693, -0.081253, 0, 0.996693, -0.081253, 0, 0.996508, -0.083492, 0, 0.996508, -0.083492, 0, 0.996508, -0.083492, 0, 0.996508, -0.083492, 0, 0.996508, -0.083492, 0, 0.996508, -0.083492, 0, 0.996342, -0.085454, 0, 0.996342, -0.085454, 0, 0.996342, -0.085454, 0, 0.996342, -0.085454, 0, 0.996342, -0.085454, 0, 0.996342, -0.085454, 0, 0.996196, -0.087139, 0, 0.996196, -0.087139, 0, 0.996196, -0.087139, 0, 0.996196, -0.087139, 0, 0.996196, -0.087139, 0, 0.996196, -0.087139, 0, 0.996072, -0.088548, 0, 0.996072, -0.088548, 0, 0.996072, -0.088548, 0, 0.996072, -0.088548, 0, 0.996072, -0.088548, 0, 0.996072, -0.088548, 0, 0.995969, -0.089697, 0, 0.995969, -0.089697, 0, 0.995969, -0.089697, 0, 0.995969, -0.089697, 0, 0.995969, -0.089697, 0, 0.995969, -0.089697, 0, 0.995887, -0.090603, 0, 0.995887, -0.090603, 0, 0.995887, -0.090603, 0, 0.995887, -0.090603, 0, 0.995887, -0.090603, 0, 0.995887, -0.090603, 0, 0.995827, -0.091264, 0, 0.995827, -0.091264, 0, 0.995827, -0.091264, 0, 0.995827, -0.091264, 0, 0.995827, -0.091264, 0, 0.995827, -0.091264, 0, 0.995789, -0.091679, 0, 0.995789, -0.091679, 0, 0.995789, -0.091679, 0, 0.995789, -0.091679, 0, 0.995789, -0.091679, 0, 0.995789, -0.091679, 0, 0.995782, -0.091748, 0, 0.995782, -0.091748, 0, 0.995782, -0.091748, 0, 0.995782, -0.091748, 0, 0.995782, -0.091748, 0, 0.995782, -0.091748, 0, 0.995831, -0.091214, 0, 0.995831, -0.091214, 0, 0.995831, -0.091214, 0, 0.995831, -0.091214, 0, 0.995831, -0.091214, 0, 0.995831, -0.091214, 0, 0.995935, -0.090070, 0, 0.995935, -0.090070, 0, 0.995935, -0.090070, 0, 0.995935, -0.090070, 0, 0.995935, -0.090070, 0, 0.995935, -0.090070, 0, 0.996093, -0.088313, 0, 0.996093, -0.088313, 0, 0.996093, -0.088313, 0, 0.996093, -0.088313, 0, 0.996093, -0.088313, 0, 0.996093, -0.088313, 0, 0.996293, -0.086021, 0, 0.996293, -0.086021, 0, 0.996293, -0.086021, 0, 0.996293, -0.086021, 0, 0.996293, -0.086021, 0, 0.996293, -0.086021, 0, 0.996526, -0.083279, 0, 0.996526, -0.083279, 0, 0.996526, -0.083279, 0, 0.996526, -0.083279, 0, 0.996526, -0.083279, 0, 0.996526, -0.083279, 0, 0.996788, -0.080089, 0, 0.996788, -0.080089, 0, 0.996788, -0.080089, 0, 0.996788, -0.080089, 0, 0.996788, -0.080089, 0, 0.996788, -0.080089, 0, 0.997073, -0.076455, 0, 0.997073, -0.076455, 0, 0.997073, -0.076455, 0, 0.997073, -0.076455, 0, 0.997073, -0.076455, 0, 0.997073, -0.076455, 0, 0.997390, -0.072205, 0, 0.997390, -0.072205, 0, 0.997390, -0.072205, 0, 0.997390, -0.072205, 0, 0.997390, -0.072205, 0, 0.997390, -0.072205, 0, 0.997654, -0.068456, 0, 0.997654, -0.068456, 0, 0.997654, -0.068456, 0, 0.997654, -0.068456, 0, 0.997654, -0.068456, 0, 0.997654, -0.068456, 0, 0.997871, -0.065217, 0, 0.997871, -0.065217, 0, 0.997871, -0.065217, 0, 0.997871, -0.065217, 0, 0.997871, -0.065217, 0, 0.997871, -0.065217, 0, 0.998046, -0.062492, 0, 0.998046, -0.062492, 0, 0.998046, -0.062492, 0, 0.998046, -0.062492, 0, 0.998046, -0.062492, 0, 0.998046, -0.062492, 0, 0.998181, -0.060296, 0, 0.998181, -0.060296, 0, 0.998181, -0.060296, 0, 0.998181, -0.060296, 0, 0.998181, -0.060296, 0, 0.998181, -0.060296, 0, 0.998279, -0.058636, 0, 0.998279, -0.058636, 0, 0.998279, -0.058636, 0, 0.998279, -0.058636, 0, 0.998279, -0.058636, 0, 0.998279, -0.058636, 0, 0.998345, -0.057510, 0, 0.998345, -0.057510, 0, 0.998345, -0.057510, 0, 0.998345, -0.057510, 0, 0.998345, -0.057510, 0, 0.998345, -0.057510, 0, 0.998379, -0.056914, 0, 0.998379, -0.056914, 0, 0.998379, -0.056914, 0, 0.998379, -0.056914, 0, 0.998379, -0.056914, 0, 0.998379, -0.056914, 0, 0.998418, -0.056236, 0, 0.998418, -0.056236, 0, 0.998418, -0.056236, 0, 0.998418, -0.056236, 0, 0.998418, -0.056236, 0, 0.998418, -0.056236, 0, 0.998464, -0.055406, 0, 0.998464, -0.055406, 0, 0.998464, -0.055406, 0, 0.998464, -0.055406, 0, 0.998464, -0.055406, 0, 0.998464, -0.055406, 0, 0.998615, -0.052613, 0, 0.998615, -0.052613, 0, 0.998615, -0.052613, 0, 0.998615, -0.052613, 0, 0.998615, -0.052613, 0, 0.998615, -0.052613, 0, 0.998740, -0.050179, 0, 0.998740, -0.050179, 0, 0.998740, -0.050179, 0, 0.998740, -0.050179, 0, 0.998740, -0.050179, 0, 0.998740, -0.050179, 0, 0.998842, -0.048102, 0, 0.998842, -0.048102, 0, 0.998842, -0.048102, 0, 0.998842, -0.048102, 0, 0.998842, -0.048102, 0, 0.998842, -0.048102, 0, 0.998924, -0.046380, 0, 0.998924, -0.046380, 0, 0.998924, -0.046380, 0, 0.998924, -0.046380, 0, 0.998924, -0.046380, 0, 0.998924, -0.046380, 0, 0.998985, -0.045046, 0, 0.998985, -0.045046, 0, 0.998985, -0.045046, 0, 0.998985, -0.045046, 0, 0.998985, -0.045046, 0, 0.998985, -0.045046, 0, 0.999026, -0.044134, 0, 0.999026, -0.044134, 0, 0.999026, -0.044134, 0, 0.999026, -0.044134, 0, 0.999026, -0.044134, 0, 0.999026, -0.044134, 0, 0.999047, -0.043647, 0, 0.999047, -0.043647, 0, 0.999047, -0.043647, 0, 0.999047, -0.043647, 0, 0.999047, -0.043647, 0, 0.999047, -0.043647, 0, 0.999050, -0.043586, 0, 0.999050, -0.043586, 0, 0.999050, -0.043586, 0, 0.999050, -0.043586, 0, 0.999050, -0.043586, 0, 0.999050, -0.043586, 0, 0.999042, -0.043754, 0, 0.999042, -0.043754, 0, 0.999042, -0.043754, 0, 0.999042, -0.043754, 0, 0.999042, -0.043754, 0, 0.999042, -0.043754, 0, 0.999029, -0.044063, 0, 0.999029, -0.044063, 0, 0.999029, -0.044063, 0, 0.999029, -0.044063, 0, 0.999029, -0.044063, 0, 0.999029, -0.044063, 0, 0.999009, -0.044514, 0, 0.999009, -0.044514, 0, 0.999009, -0.044514, 0, 0.999009, -0.044514, 0, 0.999009, -0.044514, 0, 0.999009, -0.044514, 0, 0.998982, -0.045107, 0, 0.998982, -0.045107, 0, 0.998982, -0.045107, 0, 0.998982, -0.045107, 0, 0.998982, -0.045107, 0, 0.998982, -0.045107, 0, 0.998949, -0.045843, 0, 0.998949, -0.045843, 0, 0.998949, -0.045843, 0, 0.998949, -0.045843, 0, 0.998949, -0.045843, 0, 0.998949, -0.045843, 0, 0.998908, -0.046723, 0, 0.998908, -0.046723, 0, 0.998908, -0.046723, 0, 0.998908, -0.046723, 0, 0.998908, -0.046723, 0, 0.998908, -0.046723, 0, 0.998860, -0.047746, 0, 0.998860, -0.047746, 0, 0.998860, -0.047746, 0, 0.998860, -0.047746, 0, 0.998860, -0.047746, 0, 0.998860, -0.047746, 0, 0.998803, -0.048916, 0, 0.998803, -0.048916, 0, 0.998803, -0.048916, 0, 0.998803, -0.048916, 0, 0.998803, -0.048916, 0, 0.998803, -0.048916, 0, 0.999147, -0.041287, 0, 0.999147, -0.041287, 0, 0.999147, -0.041287, 0, 0.999147, -0.041287, 0, 0.999147, -0.041287, 0, 0.999147, -0.041287, 0, 0.999195, -0.040117, 0, 0.999195, -0.040117, 0, 0.999195, -0.040117, 0, 0.999195, -0.040117, 0, 0.999195, -0.040117, 0, 0.999195, -0.040117, 0, 0.999235, -0.039097, 0, 0.999235, -0.039097, 0, 0.999235, -0.039097, 0, 0.999235, -0.039097, 0, 0.999235, -0.039097, 0, 0.999235, -0.039097, 0, 0.999269, -0.038225, 0, 0.999269, -0.038225, 0, 0.999269, -0.038225, 0, 0.999269, -0.038225, 0, 0.999269, -0.038225, 0, 0.999269, -0.038225, 0, 0.999297, -0.037500, 0, 0.999297, -0.037500, 0, 0.999297, -0.037500, 0, 0.999297, -0.037500, 0, 0.999297, -0.037500, 0, 0.999297, -0.037500, 0, 0.999318, -0.036921, 0, 0.999318, -0.036921, 0, 0.999318, -0.036921, 0, 0.999318, -0.036921, 0, 0.999318, -0.036921, 0, 0.999318, -0.036921, 0, 0.999334, -0.036487, 0, 0.999334, -0.036487, 0, 0.999334, -0.036487, 0, 0.999334, -0.036487, 0, 0.999334, -0.036487, 0, 0.999334, -0.036487, 0, 0.999345, -0.036197, 0, 0.999345, -0.036197, 0, 0.999345, -0.036197, 0, 0.999345, -0.036197, 0, 0.999345, -0.036197, 0, 0.999345, -0.036197, 0, 0.999350, -0.036050, 0, 0.999350, -0.036050, 0, 0.999350, -0.036050, 0, 0.999350, -0.036050, 0, 0.999350, -0.036050, 0, 0.999350, -0.036050, 0, 0.999343, -0.036237, 0, 0.999343, -0.036237, 0, 0.999343, -0.036237, 0, 0.999343, -0.036237, 0, 0.999343, -0.036237, 0, 0.999343, -0.036237, 0, 0.999308, -0.037200, 0, 0.999308, -0.037200, 0, 0.999308, -0.037200, 0, 0.999308, -0.037200, 0, 0.999308, -0.037200, 0, 0.999308, -0.037200, 0, 0.999242, -0.038931, 0, 0.999242, -0.038931, 0, 0.999242, -0.038931, 0, 0.999242, -0.038931, 0, 0.999242, -0.038931, 0, 0.999242, -0.038931, 0, 0.999142, -0.041425, 0, 0.999142, -0.041425, 0, 0.999142, -0.041425, 0, 0.999142, -0.041425, 0, 0.999142, -0.041425, 0, 0.999142, -0.041425, 0, 0.999001, -0.044691, 0, 0.999001, -0.044691, 0, 0.999001, -0.044691, 0, 0.999001, -0.044691, 0, 0.999001, -0.044691, 0, 0.999001, -0.044691, 0, 0.998811, -0.048742, 0, 0.998811, -0.048742, 0, 0.998811, -0.048742, 0, 0.998811, -0.048742, 0, 0.998811, -0.048742, 0, 0.998811, -0.048742, 0, 0.998564, -0.053581, 0, 0.998564, -0.053581, 0, 0.998564, -0.053581, 0, 0.998564, -0.053581, 0, 0.998564, -0.053581, 0, 0.998564, -0.053581, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999347, -0.036133, 0, 0.999347, -0.036133, 0, 0.999347, -0.036133, 0, 0.999347, -0.036133, 0, 0.999347, -0.036133, 0, 0.999347, -0.036133, 0, 0.999358, -0.035835, 0, 0.999358, -0.035835, 0, 0.999358, -0.035835, 0, 0.999358, -0.035835, 0, 0.999358, -0.035835, 0, 0.999358, -0.035835, 0, 0.997808, -0.066170, 0, 0.997808, -0.066170, 0, 0.997808, -0.066170, 0, 0.997808, -0.066170, 0, 0.997808, -0.066170, 0, 0.997808, -0.066170, 0, 0.997587, -0.069423, 0, 0.997587, -0.069423, 0, 0.997587, -0.069423, 0, 0.997587, -0.069423, 0, 0.997587, -0.069423, 0, 0.997587, -0.069423, 0, 0.997389, -0.072211, 0, 0.997389, -0.072211, 0, 0.997389, -0.072211, 0, 0.997389, -0.072211, 0, 0.997389, -0.072211, 0, 0.997389, -0.072211, 0, 0.997218, -0.074534, 0, 0.997218, -0.074534, 0, 0.997218, -0.074534, 0, 0.997218, -0.074534, 0, 0.997218, -0.074534, 0, 0.997218, -0.074534, 0, 0.997078, -0.076390, 0, 0.997078, -0.076390, 0, 0.997078, -0.076390, 0, 0.997078, -0.076390, 0, 0.997078, -0.076390, 0, 0.997078, -0.076390, 0, 0.996971, -0.077781, 0, 0.996971, -0.077781, 0, 0.996971, -0.077781, 0, 0.996971, -0.077781, 0, 0.996971, -0.077781, 0, 0.996971, -0.077781, 0, 0.996898, -0.078710, 0, 0.996898, -0.078710, 0, 0.996898, -0.078710, 0, 0.996898, -0.078710, 0, 0.996898, -0.078710, 0, 0.996898, -0.078710, 0, 0.996860, -0.079179, 0, 0.996860, -0.079179, 0, 0.996860, -0.079179, 0, 0.996860, -0.079179, 0, 0.996860, -0.079179, 0, 0.996860, -0.079179, 0, 0.996860, -0.079190, 0, 0.996860, -0.079190, 0, 0.996860, -0.079190, 0, 0.996860, -0.079190, 0, 0.996860, -0.079190, 0, 0.996860, -0.079190, 0, 0.996899, -0.078688, 0, 0.996899, -0.078688, 0, 0.996899, -0.078688, 0, 0.996899, -0.078688, 0, 0.996899, -0.078688, 0, 0.996899, -0.078688, 0, 0.996976, -0.077704, 0, 0.996976, -0.077704, 0, 0.996976, -0.077704, 0, 0.996976, -0.077704, 0, 0.996976, -0.077704, 0, 0.996976, -0.077704, 0, 0.997089, -0.076241, 0, 0.997089, -0.076241, 0, 0.997089, -0.076241, 0, 0.997089, -0.076241, 0, 0.997089, -0.076241, 0, 0.997089, -0.076241, 0, 0.997236, -0.074300, 0, 0.997236, -0.074300, 0, 0.997236, -0.074300, 0, 0.997236, -0.074300, 0, 0.997236, -0.074300, 0, 0.997236, -0.074300, 0, 0.997408, -0.071957, 0, 0.997408, -0.071957, 0, 0.997408, -0.071957, 0, 0.997408, -0.071957, 0, 0.997408, -0.071957, 0, 0.997408, -0.071957, 0, 0.997597, -0.069285, 0, 0.997597, -0.069285, 0, 0.997597, -0.069285, 0, 0.997597, -0.069285, 0, 0.997597, -0.069285, 0, 0.997597, -0.069285, 0, 0.997801, -0.066282, 0, 0.997801, -0.066282, 0, 0.997801, -0.066282, 0, 0.997801, -0.066282, 0, 0.997801, -0.066282, 0, 0.997801, -0.066282, 0, 0.998017, -0.062943, 0, 0.998017, -0.062943, 0, 0.998017, -0.062943, 0, 0.998017, -0.062943, 0, 0.998017, -0.062943, 0, 0.998017, -0.062943, 0, 0.999389, -0.034944, 0, 0.999389, -0.034944, 0, 0.999389, -0.034944, 0, 0.999389, -0.034944, 0, 0.999389, -0.034944, 0, 0.999389, -0.034944, 0, 0.999399, -0.034665, 0, 0.999399, -0.034665, 0, 0.999399, -0.034665, 0, 0.999399, -0.034665, 0, 0.999399, -0.034665, 0, 0.999399, -0.034665, 0, 0.996553, -0.082960, 0, 0.996553, -0.082960, 0, 0.996553, -0.082960, 0, 0.996553, -0.082960, 0, 0.996553, -0.082960, 0, 0.996553, -0.082960, 0, 0.996498, -0.083622, 0, 0.996498, -0.083622, 0, 0.996498, -0.083622, 0, 0.996498, -0.083622, 0, 0.996498, -0.083622, 0, 0.996498, -0.083622, 0, 0.997055, -0.076697, 0, 0.997055, -0.076697, 0, 0.997055, -0.076697, 0, 0.997055, -0.076697, 0, 0.997055, -0.076697, 0, 0.997055, -0.076697, 0, 0.996891, -0.078791, 0, 0.996891, -0.078791, 0, 0.996891, -0.078791, 0, 0.996891, -0.078791, 0, 0.996891, -0.078791, 0, 0.996891, -0.078791, 0, 0.996744, -0.080628, 0, 0.996744, -0.080628, 0, 0.996744, -0.080628, 0, 0.996744, -0.080628, 0, 0.996744, -0.080628, 0, 0.996744, -0.080628, 0, 0.996615, -0.082206, 0, 0.996615, -0.082206, 0, 0.996615, -0.082206, 0, 0.996615, -0.082206, 0, 0.996615, -0.082206, 0, 0.996615, -0.082206, 0, 0.996506, -0.083526, 0, 0.996506, -0.083526, 0, 0.996506, -0.083526, 0, 0.996506, -0.083526, 0, 0.996506, -0.083526, 0, 0.996506, -0.083526, 0, 0.996415, -0.084597, 0, 0.996415, -0.084597, 0, 0.996415, -0.084597, 0, 0.996415, -0.084597, 0, 0.996415, -0.084597, 0, 0.996415, -0.084597, 0, 0.996345, -0.085426, 0, 0.996345, -0.085426, 0, 0.996345, -0.085426, 0, 0.996345, -0.085426, 0, 0.996345, -0.085426, 0, 0.996345, -0.085426, 0, 0.996294, -0.086014, 0, 0.996294, -0.086014, 0, 0.996294, -0.086014, 0, 0.996294, -0.086014, 0, 0.996294, -0.086014, 0, 0.996294, -0.086014, 0, 0.996264, -0.086359, 0, 0.996264, -0.086359, 0, 0.996264, -0.086359, 0, 0.996264, -0.086359, 0, 0.996264, -0.086359, 0, 0.996264, -0.086359, 0, 0.996262, -0.086379, 0, 0.996262, -0.086379, 0, 0.996262, -0.086379, 0, 0.996262, -0.086379, 0, 0.996262, -0.086379, 0, 0.996262, -0.086379, 0, 0.996321, -0.085699, 0, 0.996321, -0.085699, 0, 0.996321, -0.085699, 0, 0.996321, -0.085699, 0, 0.996321, -0.085699, 0, 0.996321, -0.085699, 0, 0.996439, -0.084312, 0, 0.996439, -0.084312, 0, 0.996439, -0.084312, 0, 0.996439, -0.084312, 0, 0.996439, -0.084312, 0, 0.996439, -0.084312, 0, 0.996615, -0.082214, 0, 0.996615, -0.082214, 0, 0.996615, -0.082214, 0, 0.996615, -0.082214, 0, 0.996615, -0.082214, 0, 0.996615, -0.082214, 0, 0.996841, -0.079423, 0, 0.996841, -0.079423, 0, 0.996841, -0.079423, 0, 0.996841, -0.079423, 0, 0.996841, -0.079423, 0, 0.996841, -0.079423, 0, 0.997111, -0.075962, 0, 0.997111, -0.075962, 0, 0.997111, -0.075962, 0, 0.997111, -0.075962, 0, 0.997111, -0.075962, 0, 0.997111, -0.075962, 0, 0.997417, -0.071830, 0, 0.997417, -0.071830, 0, 0.997417, -0.071830, 0, 0.997417, -0.071830, 0, 0.997417, -0.071830, 0, 0.997417, -0.071830, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999889, -0.014888, 0, 0.999889, -0.014888, 0, 0.999889, -0.014888, 0, 0.999889, -0.014888, 0, 0.999889, -0.014888, 0, 0.999889, -0.014888, 0, 0.996718, -0.080948, 0, 0.996718, -0.080948, 0, 0.996718, -0.080948, 0, 0.996718, -0.080948, 0, 0.996718, -0.080948, 0, 0.996718, -0.080948, 0, 0.996667, -0.081579, 0, 0.996667, -0.081579, 0, 0.996667, -0.081579, 0, 0.996667, -0.081579, 0, 0.996667, -0.081579, 0, 0.996667, -0.081579, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999455, -0.033019, 0, 0.999455, -0.033019, 0, 0.999455, -0.033019, 0, 0.999455, -0.033019, 0, 0.999455, -0.033019, 0, 0.999455, -0.033019, 0, 0.999471, -0.032511, 0, 0.999471, -0.032511, 0, 0.999471, -0.032511, 0, 0.999471, -0.032511, 0, 0.999471, -0.032511, 0, 0.999471, -0.032511, 0, 0.999239, -0.039004, 0, 0.999239, -0.039004, 0, 0.999239, -0.039004, 0, 0.999239, -0.039004, 0, 0.999239, -0.039004, 0, 0.999239, -0.039004, 0, 0.999215, -0.039614, 0, 0.999215, -0.039614, 0, 0.999215, -0.039614, 0, 0.999215, -0.039614, 0, 0.999215, -0.039614, 0, 0.999215, -0.039614, 0, 0.997955, -0.063920, 0, 0.997955, -0.063920, 0, 0.997955, -0.063920, 0, 0.997955, -0.063920, 0, 0.997955, -0.063920, 0, 0.997955, -0.063920, 0, 0.997793, -0.066399, 0, 0.997793, -0.066399, 0, 0.997793, -0.066399, 0, 0.997793, -0.066399, 0, 0.997793, -0.066399, 0, 0.997793, -0.066399, 0, 0.997660, -0.068376, 0, 0.997660, -0.068376, 0, 0.997660, -0.068376, 0, 0.997660, -0.068376, 0, 0.997660, -0.068376, 0, 0.997660, -0.068376, 0, 0.997557, -0.069851, 0, 0.997557, -0.069851, 0, 0.997557, -0.069851, 0, 0.997557, -0.069851, 0, 0.997557, -0.069851, 0, 0.997557, -0.069851, 0, 0.997489, -0.070825, 0, 0.997489, -0.070825, 0, 0.997489, -0.070825, 0, 0.997489, -0.070825, 0, 0.997489, -0.070825, 0, 0.997489, -0.070825, 0, 0.997445, -0.071436, 0, 0.997445, -0.071436, 0, 0.997445, -0.071436, 0, 0.997445, -0.071436, 0, 0.997445, -0.071436, 0, 0.997445, -0.071436, 0, 0.997417, -0.071823, 0, 0.997417, -0.071823, 0, 0.997417, -0.071823, 0, 0.997417, -0.071823, 0, 0.997417, -0.071823, 0, 0.997417, -0.071823, 0, 0.997406, -0.071988, 0, 0.997406, -0.071988, 0, 0.997406, -0.071988, 0, 0.997406, -0.071988, 0, 0.997406, -0.071988, 0, 0.997406, -0.071988, 0, 0.997410, -0.071930, 0, 0.997410, -0.071930, 0, 0.997410, -0.071930, 0, 0.997410, -0.071930, 0, 0.997410, -0.071930, 0, 0.997410, -0.071930, 0, 0.997449, -0.071380, 0, 0.997449, -0.071380, 0, 0.997449, -0.071380, 0, 0.997449, -0.071380, 0, 0.997449, -0.071380, 0, 0.997449, -0.071380, 0, 0.996967, -0.077831, 0, 0.996967, -0.077831, 0, 0.996967, -0.077831, 0, 0.996967, -0.077831, 0, 0.996967, -0.077831, 0, 0.996967, -0.077831, 0, 0.996920, -0.078430, 0, 0.996920, -0.078430, 0, 0.996920, -0.078430, 0, 0.996920, -0.078430, 0, 0.996920, -0.078430, 0, 0.996920, -0.078430, 0, 0.996911, -0.078543, 0, 0.996911, -0.078543, 0, 0.996911, -0.078543, 0, 0.996911, -0.078543, 0, 0.996911, -0.078543, 0, 0.996911, -0.078543, 0, 0.996938, -0.078196, 0, 0.996938, -0.078196, 0, 0.996938, -0.078196, 0, 0.996938, -0.078196, 0, 0.996938, -0.078196, 0, 0.996938, -0.078196, 0, 0.997001, -0.077387, 0, 0.997001, -0.077387, 0, 0.997001, -0.077387, 0, 0.997001, -0.077387, 0, 0.997001, -0.077387, 0, 0.997001, -0.077387, 0, 0.997099, -0.076113, 0, 0.997099, -0.076113, 0, 0.997099, -0.076113, 0, 0.997099, -0.076113, 0, 0.997099, -0.076113, 0, 0.997099, -0.076113, 0, 0.997240, -0.074251, 0, 0.997240, -0.074251, 0, 0.997240, -0.074251, 0, 0.997240, -0.074251, 0, 0.997240, -0.074251, 0, 0.997240, -0.074251, 0, 0.997428, -0.071680, 0, 0.997428, -0.071680, 0, 0.997428, -0.071680, 0, 0.997428, -0.071680, 0, 0.997428, -0.071680, 0, 0.997428, -0.071680, 0, 0.997658, -0.068399, 0, 0.997658, -0.068399, 0, 0.997658, -0.068399, 0, 0.997658, -0.068399, 0, 0.997658, -0.068399, 0, 0.997658, -0.068399, 0, 0.997924, -0.064410, 0, 0.997924, -0.064410, 0, 0.997924, -0.064410, 0, 0.997924, -0.064410, 0, 0.997924, -0.064410, 0, 0.997924, -0.064410, 0, 0.999203, -0.039918, 0, 0.999203, -0.039918, 0, 0.999203, -0.039918, 0, 0.999203, -0.039918, 0, 0.999203, -0.039918, 0, 0.999203, -0.039918, 0, 0.999194, -0.040143, 0, 0.999194, -0.040143, 0, 0.999194, -0.040143, 0, 0.999194, -0.040143, 0, 0.999194, -0.040143, 0, 0.999194, -0.040143, 0, 0.999440, -0.033461, 0, 0.999440, -0.033461, 0, 0.999440, -0.033461, 0, 0.999440, -0.033461, 0, 0.999440, -0.033461, 0, 0.999440, -0.033461, 0, 0.999446, -0.033273, 0, 0.999446, -0.033273, 0, 0.999446, -0.033273, 0, 0.999446, -0.033273, 0, 0.999446, -0.033273, 0, 0.999446, -0.033273, 0, 0.999801, -0.019971, 0, 0.999801, -0.019971, 0, 0.999801, -0.019971, 0, 0.999801, -0.019971, 0, 0.999801, -0.019971, 0, 0.999801, -0.019971, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999514, -0.031165, 0, 0.999514, -0.031165, 0, 0.999514, -0.031165, 0, 0.999514, -0.031165, 0, 0.999514, -0.031165, 0, 0.999514, -0.031165, 0, 0.997212, -0.074625, 0, 0.997212, -0.074625, 0, 0.997212, -0.074625, 0, 0.997212, -0.074625, 0, 0.997212, -0.074625, 0, 0.997212, -0.074625, 0, 0.997171, -0.075161, 0, 0.997171, -0.075161, 0, 0.997171, -0.075161, 0, 0.997171, -0.075161, 0, 0.997171, -0.075161, 0, 0.997171, -0.075161, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999906, -0.013718, 0, 0.999906, -0.013718, 0, 0.999906, -0.013718, 0, 0.999906, -0.013718, 0, 0.999906, -0.013718, 0, 0.999906, -0.013718, 0, 0.999748, -0.022444, 0, 0.999748, -0.022444, 0, 0.999748, -0.022444, 0, 0.999748, -0.022444, 0, 0.999748, -0.022444, 0, 0.999748, -0.022444, 0, 0.999744, -0.022606, 0, 0.999744, -0.022606, 0, 0.999744, -0.022606, 0, 0.999744, -0.022606, 0, 0.999744, -0.022606, 0, 0.999744, -0.022606, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997477, -0.070984, 0, 0.997477, -0.070984, 0, 0.997477, -0.070984, 0, 0.997477, -0.070984, 0, 0.997477, -0.070984, 0, 0.997477, -0.070984, 0, 0.997443, -0.071469, 0, 0.997443, -0.071469, 0, 0.997443, -0.071469, 0, 0.997443, -0.071469, 0, 0.997443, -0.071469, 0, 0.997443, -0.071469, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.999053, -0.043504, 0, 0.999053, -0.043504, 0, 0.999053, -0.043504, 0, 0.999053, -0.043504, 0, 0.999053, -0.043504, 0, 0.999053, -0.043504, 0, 0.999153, -0.041159, 0, 0.999153, -0.041159, 0, 0.999153, -0.041159, 0, 0.999153, -0.041159, 0, 0.999153, -0.041159, 0, 0.999153, -0.041159, 0, 0.999232, -0.039189, 0, 0.999232, -0.039189, 0, 0.999232, -0.039189, 0, 0.999232, -0.039189, 0, 0.999232, -0.039189, 0, 0.999232, -0.039189, 0, 0.999293, -0.037594, 0, 0.999293, -0.037594, 0, 0.999293, -0.037594, 0, 0.999293, -0.037594, 0, 0.999293, -0.037594, 0, 0.999293, -0.037594, 0, 0.999338, -0.036372, 0, 0.999338, -0.036372, 0, 0.999338, -0.036372, 0, 0.999338, -0.036372, 0, 0.999338, -0.036372, 0, 0.999338, -0.036372, 0, 0.999369, -0.035520, 0, 0.999369, -0.035520, 0, 0.999369, -0.035520, 0, 0.999369, -0.035520, 0, 0.999369, -0.035520, 0, 0.999369, -0.035520, 0, 0.999386, -0.035040, 0, 0.999386, -0.035040, 0, 0.999386, -0.035040, 0, 0.999386, -0.035040, 0, 0.999386, -0.035040, 0, 0.999386, -0.035040, 0, 0.999390, -0.034934, 0, 0.999390, -0.034934, 0, 0.999390, -0.034934, 0, 0.999390, -0.034934, 0, 0.999390, -0.034934, 0, 0.999390, -0.034934, 0, 0.999379, -0.035248, 0, 0.999379, -0.035248, 0, 0.999379, -0.035248, 0, 0.999379, -0.035248, 0, 0.999379, -0.035248, 0, 0.999379, -0.035248, 0, 0.999352, -0.035996, 0, 0.999352, -0.035996, 0, 0.999352, -0.035996, 0, 0.999352, -0.035996, 0, 0.999352, -0.035996, 0, 0.999352, -0.035996, 0, 0.999309, -0.037180, 0, 0.999309, -0.037180, 0, 0.999309, -0.037180, 0, 0.999309, -0.037180, 0, 0.999309, -0.037180, 0, 0.999309, -0.037180, 0, 0.999247, -0.038801, 0, 0.999247, -0.038801, 0, 0.999247, -0.038801, 0, 0.999247, -0.038801, 0, 0.999247, -0.038801, 0, 0.999247, -0.038801, 0, 0.999167, -0.040799, 0, 0.999167, -0.040799, 0, 0.999167, -0.040799, 0, 0.999167, -0.040799, 0, 0.999167, -0.040799, 0, 0.999167, -0.040799, 0, 0.999070, -0.043108, 0, 0.999070, -0.043108, 0, 0.999070, -0.043108, 0, 0.999070, -0.043108, 0, 0.999070, -0.043108, 0, 0.999070, -0.043108, 0, 0.998954, -0.045726, 0, 0.998954, -0.045726, 0, 0.998954, -0.045726, 0, 0.998954, -0.045726, 0, 0.998954, -0.045726, 0, 0.998954, -0.045726, 0, 0.998816, -0.048652, 0, 0.998816, -0.048652, 0, 0.998816, -0.048652, 0, 0.998816, -0.048652, 0, 0.998816, -0.048652, 0, 0.998816, -0.048652, 0, 0.997331, -0.073012, 0, 0.997331, -0.073012, 0, 0.997331, -0.073012, 0, 0.997331, -0.073012, 0, 0.997331, -0.073012, 0, 0.997331, -0.073012, 0, 0.997293, -0.073527, 0, 0.997293, -0.073527, 0, 0.997293, -0.073527, 0, 0.997293, -0.073527, 0, 0.997293, -0.073527, 0, 0.997293, -0.073527, 0, 0.999529, -0.030705, 0, 0.999529, -0.030705, 0, 0.999529, -0.030705, 0, 0.999529, -0.030705, 0, 0.999529, -0.030705, 0, 0.999529, -0.030705, 0, 0.999535, -0.030489, 0, 0.999535, -0.030489, 0, 0.999535, -0.030489, 0, 0.999535, -0.030489, 0, 0.999535, -0.030489, 0, 0.999535, -0.030489, 0, 0.999331, -0.036579, 0, 0.999331, -0.036579, 0, 0.999331, -0.036579, 0, 0.999331, -0.036579, 0, 0.999331, -0.036579, 0, 0.999331, -0.036579, 0, 0.999394, -0.034813, 0, 0.999394, -0.034813, 0, 0.999394, -0.034813, 0, 0.999394, -0.034813, 0, 0.999394, -0.034813, 0, 0.999394, -0.034813, 0, 0.999446, -0.033279, 0, 0.999446, -0.033279, 0, 0.999446, -0.033279, 0, 0.999446, -0.033279, 0, 0.999446, -0.033279, 0, 0.999446, -0.033279, 0, 0.999489, -0.031977, 0, 0.999489, -0.031977, 0, 0.999489, -0.031977, 0, 0.999489, -0.031977, 0, 0.999489, -0.031977, 0, 0.999489, -0.031977, 0, 0.999522, -0.030905, 0, 0.999522, -0.030905, 0, 0.999522, -0.030905, 0, 0.999522, -0.030905, 0, 0.999522, -0.030905, 0, 0.999522, -0.030905, 0, 0.999548, -0.030062, 0, 0.999548, -0.030062, 0, 0.999548, -0.030062, 0, 0.999548, -0.030062, 0, 0.999548, -0.030062, 0, 0.999548, -0.030062, 0, 0.999566, -0.029442, 0, 0.999566, -0.029442, 0, 0.999566, -0.029442, 0, 0.999566, -0.029442, 0, 0.999566, -0.029442, 0, 0.999566, -0.029442, 0, 0.999578, -0.029046, 0, 0.999578, -0.029046, 0, 0.999578, -0.029046, 0, 0.999578, -0.029046, 0, 0.999578, -0.029046, 0, 0.999578, -0.029046, 0, 0.999583, -0.028872, 0, 0.999583, -0.028872, 0, 0.999583, -0.028872, 0, 0.999583, -0.028872, 0, 0.999583, -0.028872, 0, 0.999583, -0.028872, 0, 0.999578, -0.029051, 0, 0.999578, -0.029051, 0, 0.999578, -0.029051, 0, 0.999578, -0.029051, 0, 0.999578, -0.029051, 0, 0.999578, -0.029051, 0, 0.999556, -0.029793, 0, 0.999556, -0.029793, 0, 0.999556, -0.029793, 0, 0.999556, -0.029793, 0, 0.999556, -0.029793, 0, 0.999556, -0.029793, 0, 0.999516, -0.031094, 0, 0.999516, -0.031094, 0, 0.999516, -0.031094, 0, 0.999516, -0.031094, 0, 0.999516, -0.031094, 0, 0.999516, -0.031094, 0, 0.999457, -0.032951, 0, 0.999457, -0.032951, 0, 0.999457, -0.032951, 0, 0.999457, -0.032951, 0, 0.999457, -0.032951, 0, 0.999457, -0.032951, 0, 0.999375, -0.035348, 0, 0.999375, -0.035348, 0, 0.999375, -0.035348, 0, 0.999375, -0.035348, 0, 0.999375, -0.035348, 0, 0.999375, -0.035348, 0, 0.999267, -0.038269, 0, 0.999267, -0.038269, 0, 0.999267, -0.038269, 0, 0.999267, -0.038269, 0, 0.999267, -0.038269, 0, 0.999267, -0.038269, 0, 0.999130, -0.041715, 0, 0.999130, -0.041715, 0, 0.999130, -0.041715, 0, 0.999130, -0.041715, 0, 0.999130, -0.041715, 0, 0.999130, -0.041715, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999892, -0.014693, 0, 0.999892, -0.014693, 0, 0.999892, -0.014693, 0, 0.999892, -0.014693, 0, 0.999892, -0.014693, 0, 0.999892, -0.014693, 0, 0.998742, -0.050144, 0, 0.998742, -0.050144, 0, 0.998742, -0.050144, 0, 0.998742, -0.050144, 0, 0.998742, -0.050144, 0, 0.998742, -0.050144, 0, 0.999898, -0.014307, 0, 0.999898, -0.014307, 0, 0.999898, -0.014307, 0, 0.999898, -0.014307, 0, 0.999898, -0.014307, 0, 0.999898, -0.014307, 0, 0.999900, -0.014160, 0, 0.999900, -0.014160, 0, 0.999900, -0.014160, 0, 0.999900, -0.014160, 0, 0.999900, -0.014160, 0, 0.999900, -0.014160, 0, 0.997871, -0.065223, 0, 0.997871, -0.065223, 0, 0.997871, -0.065223, 0, 0.997871, -0.065223, 0, 0.997871, -0.065223, 0, 0.997871, -0.065223, 0, 0.997842, -0.065661, 0, 0.997842, -0.065661, 0, 0.997842, -0.065661, 0, 0.997842, -0.065661, 0, 0.997842, -0.065661, 0, 0.997842, -0.065661, 0, 0.999789, -0.020542, 0, 0.999789, -0.020542, 0, 0.999789, -0.020542, 0, 0.999789, -0.020542, 0, 0.999789, -0.020542, 0, 0.999789, -0.020542, 0, 0.998398, -0.056590, 0, 0.998398, -0.056590, 0, 0.998398, -0.056590, 0, 0.998398, -0.056590, 0, 0.998398, -0.056590, 0, 0.998398, -0.056590, 0, 0.998375, -0.056990, 0, 0.998375, -0.056990, 0, 0.998375, -0.056990, 0, 0.998375, -0.056990, 0, 0.998375, -0.056990, 0, 0.998375, -0.056990, 0, 0.999774, -0.021250, 0, 0.999774, -0.021250, 0, 0.999774, -0.021250, 0, 0.999774, -0.021250, 0, 0.999774, -0.021250, 0, 0.999774, -0.021250, 0, 0.997691, -0.067916, 0, 0.997691, -0.067916, 0, 0.997691, -0.067916, 0, 0.997691, -0.067916, 0, 0.997691, -0.067916, 0, 0.997691, -0.067916, 0, 0.997659, -0.068389, 0, 0.997659, -0.068389, 0, 0.997659, -0.068389, 0, 0.997659, -0.068389, 0, 0.997659, -0.068389, 0, 0.997659, -0.068389, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999782, -0.020867, 0, 0.999782, -0.020867, 0, 0.999782, -0.020867, 0, 0.999782, -0.020867, 0, 0.999782, -0.020867, 0, 0.999782, -0.020867, 0, 0.999797, -0.020164, 0, 0.999797, -0.020164, 0, 0.999797, -0.020164, 0, 0.999797, -0.020164, 0, 0.999797, -0.020164, 0, 0.999797, -0.020164, 0, 0.999809, -0.019559, 0, 0.999809, -0.019559, 0, 0.999809, -0.019559, 0, 0.999809, -0.019559, 0, 0.999809, -0.019559, 0, 0.999809, -0.019559, 0, 0.999819, -0.019052, 0, 0.999819, -0.019052, 0, 0.999819, -0.019052, 0, 0.999819, -0.019052, 0, 0.999819, -0.019052, 0, 0.999819, -0.019052, 0, 0.999826, -0.018644, 0, 0.999826, -0.018644, 0, 0.999826, -0.018644, 0, 0.999826, -0.018644, 0, 0.999826, -0.018644, 0, 0.999826, -0.018644, 0, 0.999832, -0.018333, 0, 0.999832, -0.018333, 0, 0.999832, -0.018333, 0, 0.999832, -0.018333, 0, 0.999832, -0.018333, 0, 0.999832, -0.018333, 0, 0.999836, -0.018119, 0, 0.999836, -0.018119, 0, 0.999836, -0.018119, 0, 0.999836, -0.018119, 0, 0.999836, -0.018119, 0, 0.999836, -0.018119, 0, 0.999838, -0.018001, 0, 0.999838, -0.018001, 0, 0.999838, -0.018001, 0, 0.999838, -0.018001, 0, 0.999838, -0.018001, 0, 0.999838, -0.018001, 0, 0.999838, -0.017981, 0, 0.999838, -0.017981, 0, 0.999838, -0.017981, 0, 0.999838, -0.017981, 0, 0.999838, -0.017981, 0, 0.999838, -0.017981, 0, 0.999835, -0.018158, 0, 0.999835, -0.018158, 0, 0.999835, -0.018158, 0, 0.999835, -0.018158, 0, 0.999835, -0.018158, 0, 0.999835, -0.018158, 0, 0.999827, -0.018593, 0, 0.999827, -0.018593, 0, 0.999827, -0.018593, 0, 0.999827, -0.018593, 0, 0.999827, -0.018593, 0, 0.999827, -0.018593, 0, 0.999814, -0.019287, 0, 0.999814, -0.019287, 0, 0.999814, -0.019287, 0, 0.999814, -0.019287, 0, 0.999814, -0.019287, 0, 0.999814, -0.019287, 0, 0.999795, -0.020241, 0, 0.999795, -0.020241, 0, 0.999795, -0.020241, 0, 0.999795, -0.020241, 0, 0.999795, -0.020241, 0, 0.999795, -0.020241, 0, 0.999770, -0.021433, 0, 0.999770, -0.021433, 0, 0.999770, -0.021433, 0, 0.999770, -0.021433, 0, 0.999770, -0.021433, 0, 0.999770, -0.021433, 0, 0.999739, -0.022842, 0, 0.999739, -0.022842, 0, 0.999739, -0.022842, 0, 0.999739, -0.022842, 0, 0.999739, -0.022842, 0, 0.999739, -0.022842, 0, 0.999701, -0.024467, 0, 0.999701, -0.024467, 0, 0.999701, -0.024467, 0, 0.999701, -0.024467, 0, 0.999701, -0.024467, 0, 0.999701, -0.024467, 0, 0.999654, -0.026307, 0, 0.999654, -0.026307, 0, 0.999654, -0.026307, 0, 0.999654, -0.026307, 0, 0.999654, -0.026307, 0, 0.999654, -0.026307, 0, 0.999613, -0.027832, 0, 0.999613, -0.027832, 0, 0.999613, -0.027832, 0, 0.999613, -0.027832, 0, 0.999613, -0.027832, 0, 0.999613, -0.027832, 0, 0.999575, -0.029152, 0, 0.999575, -0.029152, 0, 0.999575, -0.029152, 0, 0.999575, -0.029152, 0, 0.999575, -0.029152, 0, 0.999575, -0.029152, 0, 0.999542, -0.030265, 0, 0.999542, -0.030265, 0, 0.999542, -0.030265, 0, 0.999542, -0.030265, 0, 0.999542, -0.030265, 0, 0.999542, -0.030265, 0, 0.999514, -0.031171, 0, 0.999514, -0.031171, 0, 0.999514, -0.031171, 0, 0.999514, -0.031171, 0, 0.999514, -0.031171, 0, 0.999514, -0.031171, 0, 0.999490, -0.031924, 0, 0.999490, -0.031924, 0, 0.999490, -0.031924, 0, 0.999490, -0.031924, 0, 0.999490, -0.031924, 0, 0.999490, -0.031924, 0, 0.999469, -0.032581, 0, 0.999469, -0.032581, 0, 0.999469, -0.032581, 0, 0.999469, -0.032581, 0, 0.999469, -0.032581, 0, 0.999469, -0.032581, 0, 0.999451, -0.033139, 0, 0.999451, -0.033139, 0, 0.999451, -0.033139, 0, 0.999451, -0.033139, 0, 0.999451, -0.033139, 0, 0.999451, -0.033139, 0, 0.999435, -0.033600, 0, 0.999435, -0.033600, 0, 0.999435, -0.033600, 0, 0.999435, -0.033600, 0, 0.999435, -0.033600, 0, 0.999435, -0.033600, 0, 0.999411, -0.034305, 0, 0.999411, -0.034305, 0, 0.999411, -0.034305, 0, 0.999411, -0.034305, 0, 0.999411, -0.034305, 0, 0.999411, -0.034305, 0, 0.999385, -0.035055, 0, 0.999385, -0.035055, 0, 0.999385, -0.035055, 0, 0.999385, -0.035055, 0, 0.999385, -0.035055, 0, 0.999385, -0.035055, 0, 0.999348, -0.036092, 0, 0.999348, -0.036092, 0, 0.999348, -0.036092, 0, 0.999348, -0.036092, 0, 0.999348, -0.036092, 0, 0.999348, -0.036092, 0, 0.999300, -0.037417, 0, 0.999300, -0.037417, 0, 0.999300, -0.037417, 0, 0.999300, -0.037417, 0, 0.999300, -0.037417, 0, 0.999300, -0.037417, 0, 0.999238, -0.039029, 0, 0.999238, -0.039029, 0, 0.999238, -0.039029, 0, 0.999238, -0.039029, 0, 0.999238, -0.039029, 0, 0.999238, -0.039029, 0, 0.999162, -0.040943, 0, 0.999162, -0.040943, 0, 0.999162, -0.040943, 0, 0.999162, -0.040943, 0, 0.999162, -0.040943, 0, 0.999162, -0.040943, 0, 0.999067, -0.043179, 0, 0.999067, -0.043179, 0, 0.999067, -0.043179, 0, 0.999067, -0.043179, 0, 0.999067, -0.043179, 0, 0.999067, -0.043179, 0, 0.998954, -0.045737, 0, 0.998954, -0.045737, 0, 0.998954, -0.045737, 0, 0.998954, -0.045737, 0, 0.998954, -0.045737, 0, 0.998954, -0.045737, 0, 0.998817, -0.048619, 0, 0.998817, -0.048619, 0, 0.998817, -0.048619, 0, 0.998817, -0.048619, 0, 0.998817, -0.048619, 0, 0.998817, -0.048619, 0, 0.998645, -0.052043, 0, 0.998645, -0.052043, 0, 0.998645, -0.052043, 0, 0.998645, -0.052043, 0, 0.998645, -0.052043, 0, 0.998645, -0.052043, 0, 0.998483, -0.055060, 0, 0.998483, -0.055060, 0, 0.998483, -0.055060, 0, 0.998483, -0.055060, 0, 0.998483, -0.055060, 0, 0.998483, -0.055060, 0, 0.998335, -0.057675, 0, 0.998335, -0.057675, 0, 0.998335, -0.057675, 0, 0.998335, -0.057675, 0, 0.998335, -0.057675, 0, 0.998335, -0.057675, 0, 0.998205, -0.059892, 0, 0.998205, -0.059892, 0, 0.998205, -0.059892, 0, 0.998205, -0.059892, 0, 0.998205, -0.059892, 0, 0.998205, -0.059892, 0, 0.998095, -0.061697, 0, 0.998095, -0.061697, 0, 0.998095, -0.061697, 0, 0.998095, -0.061697, 0, 0.998095, -0.061697, 0, 0.998095, -0.061697, 0, 0.998009, -0.063074, 0, 0.998009, -0.063074, 0, 0.998009, -0.063074, 0, 0.998009, -0.063074, 0, 0.998009, -0.063074, 0, 0.998009, -0.063074, 0, 0.997948, -0.064023, 0, 0.997948, -0.064023, 0, 0.997948, -0.064023, 0, 0.997948, -0.064023, 0, 0.997948, -0.064023, 0, 0.997948, -0.064023, 0, 0.997915, -0.064539, 0, 0.997915, -0.064539, 0, 0.997915, -0.064539, 0, 0.997915, -0.064539, 0, 0.997915, -0.064539, 0, 0.997915, -0.064539, 0, 0.997911, -0.064604, 0, 0.997911, -0.064604, 0, 0.997911, -0.064604, 0, 0.997911, -0.064604, 0, 0.997911, -0.064604, 0, 0.997911, -0.064604, 0, 0.997913, -0.064573, 0, 0.997913, -0.064573, 0, 0.997913, -0.064573, 0, 0.997913, -0.064573, 0, 0.997913, -0.064573, 0, 0.997913, -0.064573, 0, 0.997921, -0.064445, 0, 0.997921, -0.064445, 0, 0.997921, -0.064445, 0, 0.997921, -0.064445, 0, 0.997921, -0.064445, 0, 0.997921, -0.064445, 0, 0.997936, -0.064219, 0, 0.997936, -0.064219, 0, 0.997936, -0.064219, 0, 0.997936, -0.064219, 0, 0.997936, -0.064219, 0, 0.997936, -0.064219, 0, 0.997957, -0.063896, 0, 0.997957, -0.063896, 0, 0.997957, -0.063896, 0, 0.997957, -0.063896, 0, 0.997957, -0.063896, 0, 0.997957, -0.063896, 0, 0.997983, -0.063478, 0, 0.997983, -0.063478, 0, 0.997983, -0.063478, 0, 0.997983, -0.063478, 0, 0.997983, -0.063478, 0, 0.997983, -0.063478, 0, 0.998016, -0.062963, 0, 0.998016, -0.062963, 0, 0.998016, -0.062963, 0, 0.998016, -0.062963, 0, 0.998016, -0.062963, 0, 0.998016, -0.062963, 0, 0.998054, -0.062351, 0, 0.998054, -0.062351, 0, 0.998054, -0.062351, 0, 0.998054, -0.062351, 0, 0.998054, -0.062351, 0, 0.998054, -0.062351, 0, 0.998495, -0.054850, 0, 0.998495, -0.054850, 0, 0.998495, -0.054850, 0, 0.998495, -0.054850, 0, 0.998495, -0.054850, 0, 0.998495, -0.054850, 0, 0.998432, -0.055985, 0, 0.998432, -0.055985, 0, 0.998432, -0.055985, 0, 0.998432, -0.055985, 0, 0.998432, -0.055985, 0, 0.998432, -0.055985, 0, 0.998377, -0.056952, 0, 0.998377, -0.056952, 0, 0.998377, -0.056952, 0, 0.998377, -0.056952, 0, 0.998377, -0.056952, 0, 0.998377, -0.056952, 0, 0.998331, -0.057753, 0, 0.998331, -0.057753, 0, 0.998331, -0.057753, 0, 0.998331, -0.057753, 0, 0.998331, -0.057753, 0, 0.998331, -0.057753, 0, 0.998294, -0.058389, 0, 0.998294, -0.058389, 0, 0.998294, -0.058389, 0, 0.998294, -0.058389, 0, 0.998294, -0.058389, 0, 0.998294, -0.058389, 0, 0.998266, -0.058859, 0, 0.998266, -0.058859, 0, 0.998266, -0.058859, 0, 0.998266, -0.058859, 0, 0.998266, -0.058859, 0, 0.998266, -0.058859, 0, 0.998248, -0.059162, 0, 0.998248, -0.059162, 0, 0.998248, -0.059162, 0, 0.998248, -0.059162, 0, 0.998248, -0.059162, 0, 0.998248, -0.059162, 0, 0.998240, -0.059298, 0, 0.998240, -0.059298, 0, 0.998240, -0.059298, 0, 0.998240, -0.059298, 0, 0.998240, -0.059298, 0, 0.998240, -0.059298, 0, 0.998242, -0.059269, 0, 0.998242, -0.059269, 0, 0.998242, -0.059269, 0, 0.998242, -0.059269, 0, 0.998242, -0.059269, 0, 0.998242, -0.059269, 0, 0.998260, -0.058962, 0, 0.998260, -0.058962, 0, 0.998260, -0.058962, 0, 0.998260, -0.058962, 0, 0.998260, -0.058962, 0, 0.998260, -0.058962, 0, 0.998294, -0.058382, 0, 0.998294, -0.058382, 0, 0.998294, -0.058382, 0, 0.998294, -0.058382, 0, 0.998294, -0.058382, 0, 0.998294, -0.058382, 0, 0.998344, -0.057530, 0, 0.998344, -0.057530, 0, 0.998344, -0.057530, 0, 0.998344, -0.057530, 0, 0.998344, -0.057530, 0, 0.998344, -0.057530, 0, 0.998408, -0.056406, 0, 0.998408, -0.056406, 0, 0.998408, -0.056406, 0, 0.998408, -0.056406, 0, 0.998408, -0.056406, 0, 0.998408, -0.056406, 0, 0.998484, -0.055035, 0, 0.998484, -0.055035, 0, 0.998484, -0.055035, 0, 0.998484, -0.055035, 0, 0.998484, -0.055035, 0, 0.998484, -0.055035, 0, 0.998571, -0.053441, 0, 0.998571, -0.053441, 0, 0.998571, -0.053441, 0, 0.998571, -0.053441, 0, 0.998571, -0.053441, 0, 0.998571, -0.053441, 0, 0.998667, -0.051621, 0, 0.998667, -0.051621, 0, 0.998667, -0.051621, 0, 0.998667, -0.051621, 0, 0.998667, -0.051621, 0, 0.998667, -0.051621, 0, 0.998770, -0.049574, 0, 0.998770, -0.049574, 0, 0.998770, -0.049574, 0, 0.998770, -0.049574, 0, 0.998770, -0.049574, 0, 0.998770, -0.049574, 0, 0.998857, -0.047791, 0, 0.998857, -0.047791, 0, 0.998857, -0.047791, 0, 0.998857, -0.047791, 0, 0.998857, -0.047791, 0, 0.998857, -0.047791, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998931, -0.046226, 0, 0.998992, -0.044879, 0, 0.998992, -0.044879, 0, 0.998992, -0.044879, 0, 0.998992, -0.044879, 0, 0.998992, -0.044879, 0, 0.998992, -0.044879, 0, 0.999043, -0.043747, 0, 0.999043, -0.043747, 0, 0.999043, -0.043747, 0, 0.999043, -0.043747, 0, 0.999043, -0.043747, 0, 0.999043, -0.043747, 0, 0.999084, -0.042798, 0, 0.999084, -0.042798, 0, 0.999084, -0.042798, 0, 0.999084, -0.042798, 0, 0.999084, -0.042798, 0, 0.999084, -0.042798, 0, 0.999118, -0.041997, 0, 0.999118, -0.041997, 0, 0.999118, -0.041997, 0, 0.999118, -0.041997, 0, 0.999118, -0.041997, 0, 0.999118, -0.041997, 0, 0.999145, -0.041343, 0, 0.999145, -0.041343, 0, 0.999145, -0.041343, 0, 0.999145, -0.041343, 0, 0.999145, -0.041343, 0, 0.999145, -0.041343, 0, 0.999166, -0.040837, 0, 0.999166, -0.040837, 0, 0.999166, -0.040837, 0, 0.999166, -0.040837, 0, 0.999166, -0.040837, 0, 0.999166, -0.040837, 0, 0.999192, -0.040192, 0, 0.999192, -0.040192, 0, 0.999192, -0.040192, 0, 0.999192, -0.040192, 0, 0.999192, -0.040192, 0, 0.999192, -0.040192, 0, 0.999220, -0.039487, 0, 0.999220, -0.039487, 0, 0.999220, -0.039487, 0, 0.999220, -0.039487, 0, 0.999220, -0.039487, 0, 0.999220, -0.039487, 0, 0.999257, -0.038545, 0, 0.999257, -0.038545, 0, 0.999257, -0.038545, 0, 0.999257, -0.038545, 0, 0.999257, -0.038545, 0, 0.999257, -0.038545, 0, 0.999302, -0.037364, 0, 0.999302, -0.037364, 0, 0.999302, -0.037364, 0, 0.999302, -0.037364, 0, 0.999302, -0.037364, 0, 0.999302, -0.037364, 0, 0.999354, -0.035945, 0, 0.999354, -0.035945, 0, 0.999354, -0.035945, 0, 0.999354, -0.035945, 0, 0.999354, -0.035945, 0, 0.999354, -0.035945, 0, 0.999414, -0.034242, 0, 0.999414, -0.034242, 0, 0.999414, -0.034242, 0, 0.999414, -0.034242, 0, 0.999414, -0.034242, 0, 0.999414, -0.034242, 0, 0.999481, -0.032209, 0, 0.999481, -0.032209, 0, 0.999481, -0.032209, 0, 0.999481, -0.032209, 0, 0.999481, -0.032209, 0, 0.999481, -0.032209, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999631, -0.027159, 0, 0.999631, -0.027159, 0, 0.999631, -0.027159, 0, 0.999631, -0.027159, 0, 0.999631, -0.027159, 0, 0.999631, -0.027159, 0, 0.999711, -0.024022, 0, 0.999711, -0.024022, 0, 0.999711, -0.024022, 0, 0.999711, -0.024022, 0, 0.999711, -0.024022, 0, 0.999711, -0.024022, 0, 0.999775, -0.021224, 0, 0.999775, -0.021224, 0, 0.999775, -0.021224, 0, 0.999775, -0.021224, 0, 0.999775, -0.021224, 0, 0.999775, -0.021224, 0, 0.999824, -0.018769, 0, 0.999824, -0.018769, 0, 0.999824, -0.018769, 0, 0.999824, -0.018769, 0, 0.999824, -0.018769, 0, 0.999824, -0.018769, 0, 0.999861, -0.016660, 0, 0.999861, -0.016660, 0, 0.999861, -0.016660, 0, 0.999861, -0.016660, 0, 0.999861, -0.016660, 0, 0.999861, -0.016660, 0, 0.999888, -0.014962, 0, 0.999888, -0.014962, 0, 0.999888, -0.014962, 0, 0.999888, -0.014962, 0, 0.999888, -0.014962, 0, 0.999888, -0.014962, 0, 0.999906, -0.013739, 0, 0.999906, -0.013739, 0, 0.999906, -0.013739, 0, 0.999906, -0.013739, 0, 0.999906, -0.013739, 0, 0.999906, -0.013739, 0, 0.999916, -0.012987, 0, 0.999916, -0.012987, 0, 0.999916, -0.012987, 0, 0.999916, -0.012987, 0, 0.999916, -0.012987, 0, 0.999916, -0.012987, 0, 0.999919, -0.012703, 0, 0.999919, -0.012703, 0, 0.999919, -0.012703, 0, 0.999919, -0.012703, 0, 0.999919, -0.012703, 0, 0.999919, -0.012703, 0, 0.999919, -0.012714, 0, 0.999919, -0.012714, 0, 0.999919, -0.012714, 0, 0.999919, -0.012714, 0, 0.999919, -0.012714, 0, 0.999919, -0.012714, 0, 0.999918, -0.012790, 0, 0.999918, -0.012790, 0, 0.999918, -0.012790, 0, 0.999918, -0.012790, 0, 0.999918, -0.012790, 0, 0.999918, -0.012790, 0, 0.999916, -0.012932, 0, 0.999916, -0.012932, 0, 0.999916, -0.012932, 0, 0.999916, -0.012932, 0, 0.999916, -0.012932, 0, 0.999916, -0.012932, 0, 0.999914, -0.013139, 0, 0.999914, -0.013139, 0, 0.999914, -0.013139, 0, 0.999914, -0.013139, 0, 0.999914, -0.013139, 0, 0.999914, -0.013139, 0, 0.999910, -0.013410, 0, 0.999910, -0.013410, 0, 0.999910, -0.013410, 0, 0.999910, -0.013410, 0, 0.999910, -0.013410, 0, 0.999910, -0.013410, 0, 0.999906, -0.013746, 0, 0.999906, -0.013746, 0, 0.999906, -0.013746, 0, 0.999906, -0.013746, 0, 0.999906, -0.013746, 0, 0.999906, -0.013746, 0, 0.999900, -0.014146, 0, 0.999900, -0.014146, 0, 0.999900, -0.014146, 0, 0.999900, -0.014146, 0, 0.999900, -0.014146, 0, 0.999900, -0.014146, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998421, -0.056172, 0, 0.998421, -0.056172, 0, 0.998421, -0.056172, 0, 0.998421, -0.056172, 0, 0.998421, -0.056172, 0, 0.998421, -0.056172, 0, 0.998453, -0.055598, 0, 0.998453, -0.055598, 0, 0.998453, -0.055598, 0, 0.998453, -0.055598, 0, 0.998453, -0.055598, 0, 0.998453, -0.055598, 0, 0.998468, -0.055332, 0, 0.998468, -0.055332, 0, 0.998468, -0.055332, 0, 0.998468, -0.055332, 0, 0.998468, -0.055332, 0, 0.998468, -0.055332, 0, 0.998496, -0.054823, 0, 0.998496, -0.054823, 0, 0.998496, -0.054823, 0, 0.998496, -0.054823, 0, 0.998496, -0.054823, 0, 0.998496, -0.054823, 0, 0.998537, -0.054070, 0, 0.998537, -0.054070, 0, 0.998537, -0.054070, 0, 0.998537, -0.054070, 0, 0.998537, -0.054070, 0, 0.998537, -0.054070, 0, 0.998591, -0.053075, 0, 0.998591, -0.053075, 0, 0.998591, -0.053075, 0, 0.998591, -0.053075, 0, 0.998591, -0.053075, 0, 0.998591, -0.053075, 0, 0.998656, -0.051830, 0, 0.998656, -0.051830, 0, 0.998656, -0.051830, 0, 0.998656, -0.051830, 0, 0.998656, -0.051830, 0, 0.998656, -0.051830, 0, 0.998733, -0.050326, 0, 0.998733, -0.050326, 0, 0.998733, -0.050326, 0, 0.998733, -0.050326, 0, 0.998733, -0.050326, 0, 0.998733, -0.050326, 0, 0.998820, -0.048563, 0, 0.998820, -0.048563, 0, 0.998820, -0.048563, 0, 0.998820, -0.048563, 0, 0.998820, -0.048563, 0, 0.998820, -0.048563, 0, 0.998916, -0.046540, 0, 0.998916, -0.046540, 0, 0.998916, -0.046540, 0, 0.998916, -0.046540, 0, 0.998916, -0.046540, 0, 0.998916, -0.046540, 0, 0.999009, -0.044519, 0, 0.999009, -0.044519, 0, 0.999009, -0.044519, 0, 0.999009, -0.044519, 0, 0.999009, -0.044519, 0, 0.999009, -0.044519, 0, 0.999084, -0.042794, 0, 0.999084, -0.042794, 0, 0.999084, -0.042794, 0, 0.999084, -0.042794, 0, 0.999084, -0.042794, 0, 0.999084, -0.042794, 0, 0.999144, -0.041364, 0, 0.999144, -0.041364, 0, 0.999144, -0.041364, 0, 0.999144, -0.041364, 0, 0.999144, -0.041364, 0, 0.999144, -0.041364, 0, 0.999191, -0.040228, 0, 0.999191, -0.040228, 0, 0.999191, -0.040228, 0, 0.999191, -0.040228, 0, 0.999191, -0.040228, 0, 0.999191, -0.040228, 0, 0.999225, -0.039375, 0, 0.999225, -0.039375, 0, 0.999225, -0.039375, 0, 0.999225, -0.039375, 0, 0.999225, -0.039375, 0, 0.999225, -0.039375, 0, 0.999247, -0.038793, 0, 0.999247, -0.038793, 0, 0.999247, -0.038793, 0, 0.999247, -0.038793, 0, 0.999247, -0.038793, 0, 0.999247, -0.038793, 0, 0.999259, -0.038482, 0, 0.999259, -0.038482, 0, 0.999259, -0.038482, 0, 0.999259, -0.038482, 0, 0.999259, -0.038482, 0, 0.999259, -0.038482, 0, 0.999261, -0.038444, 0, 0.999261, -0.038444, 0, 0.999261, -0.038444, 0, 0.999261, -0.038444, 0, 0.999261, -0.038444, 0, 0.999261, -0.038444, 0, 0.999431, -0.033737, 0, 0.999431, -0.033737, 0, 0.999431, -0.033737, 0, 0.999431, -0.033737, 0, 0.999431, -0.033737, 0, 0.999431, -0.033737, 0, 0.999442, -0.033418, 0, 0.999442, -0.033418, 0, 0.999442, -0.033418, 0, 0.999442, -0.033418, 0, 0.999442, -0.033418, 0, 0.999442, -0.033418, 0, 0.999448, -0.033228, 0, 0.999448, -0.033228, 0, 0.999448, -0.033228, 0, 0.999448, -0.033228, 0, 0.999448, -0.033228, 0, 0.999448, -0.033228, 0, 0.999461, -0.032816, 0, 0.999461, -0.032816, 0, 0.999461, -0.032816, 0, 0.999461, -0.032816, 0, 0.999461, -0.032816, 0, 0.999461, -0.032816, 0, 0.999482, -0.032182, 0, 0.999482, -0.032182, 0, 0.999482, -0.032182, 0, 0.999482, -0.032182, 0, 0.999482, -0.032182, 0, 0.999482, -0.032182, 0, 0.999509, -0.031326, 0, 0.999509, -0.031326, 0, 0.999509, -0.031326, 0, 0.999509, -0.031326, 0, 0.999509, -0.031326, 0, 0.999509, -0.031326, 0, 0.999543, -0.030244, 0, 0.999543, -0.030244, 0, 0.999543, -0.030244, 0, 0.999543, -0.030244, 0, 0.999543, -0.030244, 0, 0.999543, -0.030244, 0, 0.999582, -0.028928, 0, 0.999582, -0.028928, 0, 0.999582, -0.028928, 0, 0.999582, -0.028928, 0, 0.999582, -0.028928, 0, 0.999582, -0.028928, 0, 0.999625, -0.027377, 0, 0.999625, -0.027377, 0, 0.999625, -0.027377, 0, 0.999625, -0.027377, 0, 0.999625, -0.027377, 0, 0.999625, -0.027377, 0, 0.999672, -0.025592, 0, 0.999672, -0.025592, 0, 0.999672, -0.025592, 0, 0.999672, -0.025592, 0, 0.999672, -0.025592, 0, 0.999672, -0.025592, 0, 0.999716, -0.023826, 0, 0.999716, -0.023826, 0, 0.999716, -0.023826, 0, 0.999716, -0.023826, 0, 0.999716, -0.023826, 0, 0.999716, -0.023826, 0, 0.999751, -0.022303, 0, 0.999751, -0.022303, 0, 0.999751, -0.022303, 0, 0.999751, -0.022303, 0, 0.999751, -0.022303, 0, 0.999751, -0.022303, 0, 0.999779, -0.021024, 0, 0.999779, -0.021024, 0, 0.999779, -0.021024, 0, 0.999779, -0.021024, 0, 0.999779, -0.021024, 0, 0.999779, -0.021024, 0, 0.999800, -0.019987, 0, 0.999800, -0.019987, 0, 0.999800, -0.019987, 0, 0.999800, -0.019987, 0, 0.999800, -0.019987, 0, 0.999800, -0.019987, 0, 0.999816, -0.019190, 0, 0.999816, -0.019190, 0, 0.999816, -0.019190, 0, 0.999816, -0.019190, 0, 0.999816, -0.019190, 0, 0.999816, -0.019190, 0, 0.999826, -0.018632, 0, 0.999826, -0.018632, 0, 0.999826, -0.018632, 0, 0.999826, -0.018632, 0, 0.999826, -0.018632, 0, 0.999826, -0.018632, 0, 0.999832, -0.018313, 0, 0.999832, -0.018313, 0, 0.999832, -0.018313, 0, 0.999832, -0.018313, 0, 0.999832, -0.018313, 0, 0.999832, -0.018313, 0, 0.999834, -0.018235, 0, 0.999834, -0.018235, 0, 0.999834, -0.018235, 0, 0.999834, -0.018235, 0, 0.999834, -0.018235, 0, 0.999834, -0.018235, 0, 0.999913, -0.013165, 0, 0.999913, -0.013165, 0, 0.999913, -0.013165, 0, 0.999913, -0.013165, 0, 0.999913, -0.013165, 0, 0.999913, -0.013165, 0, 0.999911, -0.013315, 0, 0.999911, -0.013315, 0, 0.999911, -0.013315, 0, 0.999911, -0.013315, 0, 0.999911, -0.013315, 0, 0.999911, -0.013315, 0, 0.999904, -0.013858, 0, 0.999904, -0.013858, 0, 0.999904, -0.013858, 0, 0.999904, -0.013858, 0, 0.999904, -0.013858, 0, 0.999904, -0.013858, 0, 0.999891, -0.014792, 0, 0.999891, -0.014792, 0, 0.999891, -0.014792, 0, 0.999891, -0.014792, 0, 0.999891, -0.014792, 0, 0.999891, -0.014792, 0, 0.999870, -0.016115, 0, 0.999870, -0.016115, 0, 0.999870, -0.016115, 0, 0.999870, -0.016115, 0, 0.999870, -0.016115, 0, 0.999870, -0.016115, 0, 0.999842, -0.017798, 0, 0.999842, -0.017798, 0, 0.999842, -0.017798, 0, 0.999842, -0.017798, 0, 0.999842, -0.017798, 0, 0.999842, -0.017798, 0, 0.999804, -0.019811, 0, 0.999804, -0.019811, 0, 0.999804, -0.019811, 0, 0.999804, -0.019811, 0, 0.999804, -0.019811, 0, 0.999804, -0.019811, 0, 0.999755, -0.022157, 0, 0.999755, -0.022157, 0, 0.999755, -0.022157, 0, 0.999755, -0.022157, 0, 0.999755, -0.022157, 0, 0.999755, -0.022157, 0, 0.999691, -0.024838, 0, 0.999691, -0.024838, 0, 0.999691, -0.024838, 0, 0.999691, -0.024838, 0, 0.999691, -0.024838, 0, 0.999691, -0.024838, 0, 0.999637, -0.026926, 0, 0.999637, -0.026926, 0, 0.999637, -0.026926, 0, 0.999637, -0.026926, 0, 0.999637, -0.026926, 0, 0.999637, -0.026926, 0, 0.999585, -0.028797, 0, 0.999585, -0.028797, 0, 0.999585, -0.028797, 0, 0.999585, -0.028797, 0, 0.999585, -0.028797, 0, 0.999585, -0.028797, 0, 0.999536, -0.030455, 0, 0.999536, -0.030455, 0, 0.999536, -0.030455, 0, 0.999536, -0.030455, 0, 0.999536, -0.030455, 0, 0.999536, -0.030455, 0, 0.999491, -0.031899, 0, 0.999491, -0.031899, 0, 0.999491, -0.031899, 0, 0.999491, -0.031899, 0, 0.999491, -0.031899, 0, 0.999491, -0.031899, 0, 0.999451, -0.033119, 0, 0.999451, -0.033119, 0, 0.999451, -0.033119, 0, 0.999451, -0.033119, 0, 0.999451, -0.033119, 0, 0.999451, -0.033119, 0, 0.999418, -0.034102, 0, 0.999418, -0.034102, 0, 0.999418, -0.034102, 0, 0.999418, -0.034102, 0, 0.999418, -0.034102, 0, 0.999418, -0.034102, 0, 0.999393, -0.034849, 0, 0.999393, -0.034849, 0, 0.999393, -0.034849, 0, 0.999393, -0.034849, 0, 0.999393, -0.034849, 0, 0.999393, -0.034849, 0, 0.999375, -0.035358, 0, 0.999375, -0.035358, 0, 0.999375, -0.035358, 0, 0.999375, -0.035358, 0, 0.999375, -0.035358, 0, 0.999375, -0.035358, 0, 0.999355, -0.035907, 0, 0.999355, -0.035907, 0, 0.999355, -0.035907, 0, 0.999355, -0.035907, 0, 0.999355, -0.035907, 0, 0.999355, -0.035907, 0, 0.999325, -0.036732, 0, 0.999325, -0.036732, 0, 0.999325, -0.036732, 0, 0.999325, -0.036732, 0, 0.999325, -0.036732, 0, 0.999325, -0.036732, 0, 0.999284, -0.037831, 0, 0.999284, -0.037831, 0, 0.999284, -0.037831, 0, 0.999284, -0.037831, 0, 0.999284, -0.037831, 0, 0.999284, -0.037831, 0, 0.999231, -0.039204, 0, 0.999231, -0.039204, 0, 0.999231, -0.039204, 0, 0.999231, -0.039204, 0, 0.999231, -0.039204, 0, 0.999231, -0.039204, 0, 0.999167, -0.040814, 0, 0.999167, -0.040814, 0, 0.999167, -0.040814, 0, 0.999167, -0.040814, 0, 0.999167, -0.040814, 0, 0.999167, -0.040814, 0, 0.999091, -0.042624, 0, 0.999091, -0.042624, 0, 0.999091, -0.042624, 0, 0.999091, -0.042624, 0, 0.999091, -0.042624, 0, 0.999091, -0.042624, 0, 0.999003, -0.044637, 0, 0.999003, -0.044637, 0, 0.999003, -0.044637, 0, 0.999003, -0.044637, 0, 0.999003, -0.044637, 0, 0.999003, -0.044637, 0, 0.998902, -0.046853, 0, 0.998902, -0.046853, 0, 0.998902, -0.046853, 0, 0.998902, -0.046853, 0, 0.998902, -0.046853, 0, 0.998902, -0.046853, 0, 0.998762, -0.049753, 0, 0.998762, -0.049753, 0, 0.998762, -0.049753, 0, 0.998762, -0.049753, 0, 0.998762, -0.049753, 0, 0.998762, -0.049753, 0, 0.998630, -0.052333, 0, 0.998630, -0.052333, 0, 0.998630, -0.052333, 0, 0.998630, -0.052333, 0, 0.998630, -0.052333, 0, 0.998630, -0.052333, 0, 0.998509, -0.054595, 0, 0.998509, -0.054595, 0, 0.998509, -0.054595, 0, 0.998509, -0.054595, 0, 0.998509, -0.054595, 0, 0.998509, -0.054595, 0, 0.998400, -0.056543, 0, 0.998400, -0.056543, 0, 0.998400, -0.056543, 0, 0.998400, -0.056543, 0, 0.998400, -0.056543, 0, 0.998400, -0.056543, 0, 0.998308, -0.058141, 0, 0.998308, -0.058141, 0, 0.998308, -0.058141, 0, 0.998308, -0.058141, 0, 0.998308, -0.058141, 0, 0.998308, -0.058141, 0, 0.998237, -0.059354, 0, 0.998237, -0.059354, 0, 0.998237, -0.059354, 0, 0.998237, -0.059354, 0, 0.998237, -0.059354, 0, 0.998237, -0.059354, 0, 0.998188, -0.060179, 0, 0.998188, -0.060179, 0, 0.998188, -0.060179, 0, 0.998188, -0.060179, 0, 0.998188, -0.060179, 0, 0.998188, -0.060179, 0, 0.998161, -0.060615, 0, 0.998161, -0.060615, 0, 0.998161, -0.060615, 0, 0.998161, -0.060615, 0, 0.998161, -0.060615, 0, 0.998161, -0.060615, 0, 0.998097, -0.061657, 0, 0.998097, -0.061657, 0, 0.998097, -0.061657, 0, 0.998097, -0.061657, 0, 0.998097, -0.061657, 0, 0.998097, -0.061657, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999919, -0.012724, 0, 0.999919, -0.012724, 0, 0.999919, -0.012724, 0, 0.999919, -0.012724, 0, 0.999919, -0.012724, 0, 0.999919, -0.012724, 0, 0.998280, -0.058631, 0, 0.998280, -0.058631, 0, 0.998280, -0.058631, 0, 0.998280, -0.058631, 0, 0.998280, -0.058631, 0, 0.998280, -0.058631, 0, 0.998258, -0.058994, 0, 0.998258, -0.058994, 0, 0.998258, -0.058994, 0, 0.998258, -0.058994, 0, 0.998258, -0.058994, 0, 0.998258, -0.058994, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999922, -0.012492, 0, 0.999922, -0.012492, 0, 0.999922, -0.012492, 0, 0.999922, -0.012492, 0, 0.999922, -0.012492, 0, 0.999922, -0.012492, 0, 0.998606, -0.052781, 0, 0.998606, -0.052781, 0, 0.998606, -0.052781, 0, 0.998606, -0.052781, 0, 0.998606, -0.052781, 0, 0.998606, -0.052781, 0, 0.998665, -0.051656, 0, 0.998665, -0.051656, 0, 0.998665, -0.051656, 0, 0.998665, -0.051656, 0, 0.998665, -0.051656, 0, 0.998665, -0.051656, 0, 0.998412, -0.056338, 0, 0.998412, -0.056338, 0, 0.998412, -0.056338, 0, 0.998412, -0.056338, 0, 0.998412, -0.056338, 0, 0.998412, -0.056338, 0, 0.998322, -0.057914, 0, 0.998322, -0.057914, 0, 0.998322, -0.057914, 0, 0.998322, -0.057914, 0, 0.998322, -0.057914, 0, 0.998322, -0.057914, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999927, -0.012074, 0, 0.999927, -0.012074, 0, 0.999927, -0.012074, 0, 0.999927, -0.012074, 0, 0.999927, -0.012074, 0, 0.999927, -0.012074, 0, 0.998698, -0.051021, 0, 0.998698, -0.051021, 0, 0.998698, -0.051021, 0, 0.998698, -0.051021, 0, 0.998698, -0.051021, 0, 0.998698, -0.051021, 0, 0.998751, -0.049969, 0, 0.998751, -0.049969, 0, 0.998751, -0.049969, 0, 0.998751, -0.049969, 0, 0.998751, -0.049969, 0, 0.998751, -0.049969, 0, 0.998514, -0.054498, 0, 0.998514, -0.054498, 0, 0.998514, -0.054498, 0, 0.998514, -0.054498, 0, 0.998514, -0.054498, 0, 0.998514, -0.054498, 0, 0.998432, -0.055972, 0, 0.998432, -0.055972, 0, 0.998432, -0.055972, 0, 0.998432, -0.055972, 0, 0.998432, -0.055972, 0, 0.998432, -0.055972, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999521, -0.030949, 0, 0.999521, -0.030949, 0, 0.999521, -0.030949, 0, 0.999521, -0.030949, 0, 0.999521, -0.030949, 0, 0.999521, -0.030949, 0, 0.999498, -0.031692, 0, 0.999498, -0.031692, 0, 0.999498, -0.031692, 0, 0.999498, -0.031692, 0, 0.999498, -0.031692, 0, 0.999498, -0.031692, 0, 0.999466, -0.032675, 0, 0.999466, -0.032675, 0, 0.999466, -0.032675, 0, 0.999466, -0.032675, 0, 0.999466, -0.032675, 0, 0.999466, -0.032675, 0, 0.999425, -0.033898, 0, 0.999425, -0.033898, 0, 0.999425, -0.033898, 0, 0.999425, -0.033898, 0, 0.999425, -0.033898, 0, 0.999425, -0.033898, 0, 0.999376, -0.035324, 0, 0.999376, -0.035324, 0, 0.999376, -0.035324, 0, 0.999376, -0.035324, 0, 0.999376, -0.035324, 0, 0.999376, -0.035324, 0, 0.999318, -0.036920, 0, 0.999318, -0.036920, 0, 0.999318, -0.036920, 0, 0.999318, -0.036920, 0, 0.999318, -0.036920, 0, 0.999318, -0.036920, 0, 0.999251, -0.038687, 0, 0.999251, -0.038687, 0, 0.999251, -0.038687, 0, 0.999251, -0.038687, 0, 0.999251, -0.038687, 0, 0.999251, -0.038687, 0, 0.999175, -0.040625, 0, 0.999175, -0.040625, 0, 0.999175, -0.040625, 0, 0.999175, -0.040625, 0, 0.999175, -0.040625, 0, 0.999175, -0.040625, 0, 0.999050, -0.043569, 0, 0.999050, -0.043569, 0, 0.999050, -0.043569, 0, 0.999050, -0.043569, 0, 0.999050, -0.043569, 0, 0.999050, -0.043569, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998824, -0.048479, 0, 0.998824, -0.048479, 0, 0.998824, -0.048479, 0, 0.998824, -0.048479, 0, 0.998824, -0.048479, 0, 0.998824, -0.048479, 0, 0.998727, -0.050451, 0, 0.998727, -0.050451, 0, 0.998727, -0.050451, 0, 0.998727, -0.050451, 0, 0.998727, -0.050451, 0, 0.998727, -0.050451, 0, 0.998644, -0.052065, 0, 0.998644, -0.052065, 0, 0.998644, -0.052065, 0, 0.998644, -0.052065, 0, 0.998644, -0.052065, 0, 0.998644, -0.052065, 0, 0.998580, -0.053281, 0, 0.998580, -0.053281, 0, 0.998580, -0.053281, 0, 0.998580, -0.053281, 0, 0.998580, -0.053281, 0, 0.998580, -0.053281, 0, 0.998536, -0.054098, 0, 0.998536, -0.054098, 0, 0.998536, -0.054098, 0, 0.998536, -0.054098, 0, 0.998536, -0.054098, 0, 0.998536, -0.054098, 0, 0.998513, -0.054513, 0, 0.998513, -0.054513, 0, 0.998513, -0.054513, 0, 0.998513, -0.054513, 0, 0.998513, -0.054513, 0, 0.998513, -0.054513, 0, 0.998511, -0.054557, 0, 0.998511, -0.054557, 0, 0.998511, -0.054557, 0, 0.998511, -0.054557, 0, 0.998511, -0.054557, 0, 0.998511, -0.054557, 0, 0.998511, -0.054546, 0, 0.998511, -0.054546, 0, 0.998511, -0.054546, 0, 0.998511, -0.054546, 0, 0.998511, -0.054546, 0, 0.998511, -0.054546, 0, 0.998515, -0.054481, 0, 0.998515, -0.054481, 0, 0.998515, -0.054481, 0, 0.998515, -0.054481, 0, 0.998515, -0.054481, 0, 0.998515, -0.054481, 0, 0.998521, -0.054360, 0, 0.998521, -0.054360, 0, 0.998521, -0.054360, 0, 0.998521, -0.054360, 0, 0.998521, -0.054360, 0, 0.998521, -0.054360, 0, 0.998531, -0.054185, 0, 0.998531, -0.054185, 0, 0.998531, -0.054185, 0, 0.998531, -0.054185, 0, 0.998531, -0.054185, 0, 0.998531, -0.054185, 0, 0.998543, -0.053953, 0, 0.998543, -0.053953, 0, 0.998543, -0.053953, 0, 0.998543, -0.053953, 0, 0.998543, -0.053953, 0, 0.998543, -0.053953, 0, 0.998559, -0.053667, 0, 0.998559, -0.053667, 0, 0.998559, -0.053667, 0, 0.998559, -0.053667, 0, 0.998559, -0.053667, 0, 0.998559, -0.053667, 0, 0.998577, -0.053324, 0, 0.998577, -0.053324, 0, 0.998577, -0.053324, 0, 0.998577, -0.053324, 0, 0.998577, -0.053324, 0, 0.998577, -0.053324, 0, 0.998851, -0.047914, 0, 0.998851, -0.047914, 0, 0.998851, -0.047914, 0, 0.998851, -0.047914, 0, 0.998851, -0.047914, 0, 0.998851, -0.047914, 0, 0.998824, -0.048484, 0, 0.998824, -0.048484, 0, 0.998824, -0.048484, 0, 0.998824, -0.048484, 0, 0.998824, -0.048484, 0, 0.998824, -0.048484, 0, 0.998800, -0.048967, 0, 0.998800, -0.048967, 0, 0.998800, -0.048967, 0, 0.998800, -0.048967, 0, 0.998800, -0.048967, 0, 0.998800, -0.048967, 0, 0.998781, -0.049366, 0, 0.998781, -0.049366, 0, 0.998781, -0.049366, 0, 0.998781, -0.049366, 0, 0.998781, -0.049366, 0, 0.998781, -0.049366, 0, 0.998765, -0.049678, 0, 0.998765, -0.049678, 0, 0.998765, -0.049678, 0, 0.998765, -0.049678, 0, 0.998765, -0.049678, 0, 0.998765, -0.049678, 0, 0.998754, -0.049905, 0, 0.998754, -0.049905, 0, 0.998754, -0.049905, 0, 0.998754, -0.049905, 0, 0.998754, -0.049905, 0, 0.998754, -0.049905, 0, 0.998747, -0.050045, 0, 0.998747, -0.050045, 0, 0.998747, -0.050045, 0, 0.998747, -0.050045, 0, 0.998747, -0.050045, 0, 0.998747, -0.050045, 0, 0.998744, -0.050098, 0, 0.998744, -0.050098, 0, 0.998744, -0.050098, 0, 0.998744, -0.050098, 0, 0.998744, -0.050098, 0, 0.998744, -0.050098, 0, 0.998746, -0.050064, 0, 0.998746, -0.050064, 0, 0.998746, -0.050064, 0, 0.998746, -0.050064, 0, 0.998746, -0.050064, 0, 0.998746, -0.050064, 0, 0.998759, -0.049802, 0, 0.998759, -0.049802, 0, 0.998759, -0.049802, 0, 0.998759, -0.049802, 0, 0.998759, -0.049802, 0, 0.998759, -0.049802, 0, 0.998785, -0.049280, 0, 0.998785, -0.049280, 0, 0.998785, -0.049280, 0, 0.998785, -0.049280, 0, 0.998785, -0.049280, 0, 0.998785, -0.049280, 0, 0.998823, -0.048501, 0, 0.998823, -0.048501, 0, 0.998823, -0.048501, 0, 0.998823, -0.048501, 0, 0.998823, -0.048501, 0, 0.998823, -0.048501, 0, 0.998873, -0.047465, 0, 0.998873, -0.047465, 0, 0.998873, -0.047465, 0, 0.998873, -0.047465, 0, 0.998873, -0.047465, 0, 0.998873, -0.047465, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.998933, -0.046186, 0, 0.999002, -0.044676, 0, 0.999002, -0.044676, 0, 0.999002, -0.044676, 0, 0.999002, -0.044676, 0, 0.999002, -0.044676, 0, 0.999002, -0.044676, 0, 0.999078, -0.042935, 0, 0.999078, -0.042935, 0, 0.999078, -0.042935, 0, 0.999078, -0.042935, 0, 0.999078, -0.042935, 0, 0.999078, -0.042935, 0, 0.999161, -0.040961, 0, 0.999161, -0.040961, 0, 0.999161, -0.040961, 0, 0.999161, -0.040961, 0, 0.999161, -0.040961, 0, 0.999161, -0.040961, 0, 0.999234, -0.039146, 0, 0.999234, -0.039146, 0, 0.999234, -0.039146, 0, 0.999234, -0.039146, 0, 0.999234, -0.039146, 0, 0.999234, -0.039146, 0, 0.999295, -0.037551, 0, 0.999295, -0.037551, 0, 0.999295, -0.037551, 0, 0.999295, -0.037551, 0, 0.999295, -0.037551, 0, 0.999295, -0.037551, 0, 0.999345, -0.036175, 0, 0.999345, -0.036175, 0, 0.999345, -0.036175, 0, 0.999345, -0.036175, 0, 0.999345, -0.036175, 0, 0.999345, -0.036175, 0, 0.999387, -0.035018, 0, 0.999387, -0.035018, 0, 0.999387, -0.035018, 0, 0.999387, -0.035018, 0, 0.999387, -0.035018, 0, 0.999387, -0.035018, 0, 0.999418, -0.034108, 0, 0.999418, -0.034108, 0, 0.999418, -0.034108, 0, 0.999418, -0.034108, 0, 0.999418, -0.034108, 0, 0.999418, -0.034108, 0, 0.999439, -0.033479, 0, 0.999439, -0.033479, 0, 0.999439, -0.033479, 0, 0.999439, -0.033479, 0, 0.999439, -0.033479, 0, 0.999439, -0.033479, 0, 0.999451, -0.033131, 0, 0.999451, -0.033131, 0, 0.999451, -0.033131, 0, 0.999451, -0.033131, 0, 0.999451, -0.033131, 0, 0.999451, -0.033131, 0, 0.999453, -0.033066, 0, 0.999453, -0.033066, 0, 0.999453, -0.033066, 0, 0.999453, -0.033066, 0, 0.999453, -0.033066, 0, 0.999453, -0.033066, 0, 0.999448, -0.033227, 0, 0.999448, -0.033227, 0, 0.999448, -0.033227, 0, 0.999448, -0.033227, 0, 0.999448, -0.033227, 0, 0.999448, -0.033227, 0, 0.999587, -0.028741, 0, 0.999587, -0.028741, 0, 0.999587, -0.028741, 0, 0.999587, -0.028741, 0, 0.999587, -0.028741, 0, 0.999587, -0.028741, 0, 0.999591, -0.028595, 0, 0.999591, -0.028595, 0, 0.999591, -0.028595, 0, 0.999591, -0.028595, 0, 0.999591, -0.028595, 0, 0.999591, -0.028595, 0, 0.999596, -0.028433, 0, 0.999596, -0.028433, 0, 0.999596, -0.028433, 0, 0.999596, -0.028433, 0, 0.999596, -0.028433, 0, 0.999596, -0.028433, 0, 0.999606, -0.028066, 0, 0.999606, -0.028066, 0, 0.999606, -0.028066, 0, 0.999606, -0.028066, 0, 0.999606, -0.028066, 0, 0.999606, -0.028066, 0, 0.999622, -0.027496, 0, 0.999622, -0.027496, 0, 0.999622, -0.027496, 0, 0.999622, -0.027496, 0, 0.999622, -0.027496, 0, 0.999622, -0.027496, 0, 0.999643, -0.026723, 0, 0.999643, -0.026723, 0, 0.999643, -0.026723, 0, 0.999643, -0.026723, 0, 0.999643, -0.026723, 0, 0.999643, -0.026723, 0, 0.999668, -0.025759, 0, 0.999668, -0.025759, 0, 0.999668, -0.025759, 0, 0.999668, -0.025759, 0, 0.999668, -0.025759, 0, 0.999668, -0.025759, 0, 0.999697, -0.024620, 0, 0.999697, -0.024620, 0, 0.999697, -0.024620, 0, 0.999697, -0.024620, 0, 0.999697, -0.024620, 0, 0.999697, -0.024620, 0, 0.999729, -0.023303, 0, 0.999729, -0.023303, 0, 0.999729, -0.023303, 0, 0.999729, -0.023303, 0, 0.999729, -0.023303, 0, 0.999729, -0.023303, 0, 0.999762, -0.021807, 0, 0.999762, -0.021807, 0, 0.999762, -0.021807, 0, 0.999762, -0.021807, 0, 0.999762, -0.021807, 0, 0.999762, -0.021807, 0, 0.999794, -0.020278, 0, 0.999794, -0.020278, 0, 0.999794, -0.020278, 0, 0.999794, -0.020278, 0, 0.999794, -0.020278, 0, 0.999794, -0.020278, 0, 0.999821, -0.018943, 0, 0.999821, -0.018943, 0, 0.999821, -0.018943, 0, 0.999821, -0.018943, 0, 0.999821, -0.018943, 0, 0.999821, -0.018943, 0, 0.999842, -0.017800, 0, 0.999842, -0.017800, 0, 0.999842, -0.017800, 0, 0.999842, -0.017800, 0, 0.999842, -0.017800, 0, 0.999842, -0.017800, 0, 0.999858, -0.016849, 0, 0.999858, -0.016849, 0, 0.999858, -0.016849, 0, 0.999858, -0.016849, 0, 0.999858, -0.016849, 0, 0.999858, -0.016849, 0, 0.999870, -0.016104, 0, 0.999870, -0.016104, 0, 0.999870, -0.016104, 0, 0.999870, -0.016104, 0, 0.999870, -0.016104, 0, 0.999870, -0.016104, 0, 0.999879, -0.015580, 0, 0.999879, -0.015580, 0, 0.999879, -0.015580, 0, 0.999879, -0.015580, 0, 0.999879, -0.015580, 0, 0.999879, -0.015580, 0, 0.999883, -0.015278, 0, 0.999883, -0.015278, 0, 0.999883, -0.015278, 0, 0.999883, -0.015278, 0, 0.999883, -0.015278, 0, 0.999883, -0.015278, 0, 0.999884, -0.015199, 0, 0.999884, -0.015199, 0, 0.999884, -0.015199, 0, 0.999884, -0.015199, 0, 0.999884, -0.015199, 0, 0.999884, -0.015199, 0, 0.999884, -0.015257, 0, 0.999884, -0.015257, 0, 0.999884, -0.015257, 0, 0.999884, -0.015257, 0, 0.999884, -0.015257, 0, 0.999884, -0.015257, 0, 0.999881, -0.015398, 0, 0.999881, -0.015398, 0, 0.999881, -0.015398, 0, 0.999881, -0.015398, 0, 0.999881, -0.015398, 0, 0.999881, -0.015398, 0, 0.999878, -0.015621, 0, 0.999878, -0.015621, 0, 0.999878, -0.015621, 0, 0.999878, -0.015621, 0, 0.999878, -0.015621, 0, 0.999878, -0.015621, 0, 0.999873, -0.015927, 0, 0.999873, -0.015927, 0, 0.999873, -0.015927, 0, 0.999873, -0.015927, 0, 0.999873, -0.015927, 0, 0.999873, -0.015927, 0, 0.999867, -0.016322, 0, 0.999867, -0.016322, 0, 0.999867, -0.016322, 0, 0.999867, -0.016322, 0, 0.999867, -0.016322, 0, 0.999867, -0.016322, 0, 0.999859, -0.016811, 0, 0.999859, -0.016811, 0, 0.999859, -0.016811, 0, 0.999859, -0.016811, 0, 0.999859, -0.016811, 0, 0.999859, -0.016811, 0, 0.999849, -0.017396, 0, 0.999849, -0.017396, 0, 0.999849, -0.017396, 0, 0.999849, -0.017396, 0, 0.999849, -0.017396, 0, 0.999849, -0.017396, 0, 0.999837, -0.018076, 0, 0.999837, -0.018076, 0, 0.999837, -0.018076, 0, 0.999837, -0.018076, 0, 0.999837, -0.018076, 0, 0.999837, -0.018076, 0, 0.999920, -0.012654, 0, 0.999920, -0.012654, 0, 0.999920, -0.012654, 0, 0.999920, -0.012654, 0, 0.999920, -0.012654, 0, 0.999920, -0.012654, 0, 0.999925, -0.012211, 0, 0.999925, -0.012211, 0, 0.999925, -0.012211, 0, 0.999925, -0.012211, 0, 0.999925, -0.012211, 0, 0.999925, -0.012211, 0, 0.999930, -0.011826, 0, 0.999930, -0.011826, 0, 0.999930, -0.011826, 0, 0.999930, -0.011826, 0, 0.999930, -0.011826, 0, 0.999930, -0.011826, 0, 0.999934, -0.011499, 0, 0.999934, -0.011499, 0, 0.999934, -0.011499, 0, 0.999934, -0.011499, 0, 0.999934, -0.011499, 0, 0.999934, -0.011499, 0, 0.999937, -0.011230, 0, 0.999937, -0.011230, 0, 0.999937, -0.011230, 0, 0.999937, -0.011230, 0, 0.999937, -0.011230, 0, 0.999937, -0.011230, 0, 0.999939, -0.011018, 0, 0.999939, -0.011018, 0, 0.999939, -0.011018, 0, 0.999939, -0.011018, 0, 0.999939, -0.011018, 0, 0.999939, -0.011018, 0, 0.999941, -0.010863, 0, 0.999941, -0.010863, 0, 0.999941, -0.010863, 0, 0.999941, -0.010863, 0, 0.999941, -0.010863, 0, 0.999941, -0.010863, 0, 0.999942, -0.010765, 0, 0.999942, -0.010765, 0, 0.999942, -0.010765, 0, 0.999942, -0.010765, 0, 0.999942, -0.010765, 0, 0.999942, -0.010765, 0, 0.999943, -0.010724, 0, 0.999943, -0.010724, 0, 0.999943, -0.010724, 0, 0.999943, -0.010724, 0, 0.999943, -0.010724, 0, 0.999943, -0.010724, 0, 0.999941, -0.010873, 0, 0.999941, -0.010873, 0, 0.999941, -0.010873, 0, 0.999941, -0.010873, 0, 0.999941, -0.010873, 0, 0.999941, -0.010873, 0, 0.999935, -0.011386, 0, 0.999935, -0.011386, 0, 0.999935, -0.011386, 0, 0.999935, -0.011386, 0, 0.999935, -0.011386, 0, 0.999935, -0.011386, 0, 0.999925, -0.012262, 0, 0.999925, -0.012262, 0, 0.999925, -0.012262, 0, 0.999925, -0.012262, 0, 0.999925, -0.012262, 0, 0.999925, -0.012262, 0, 0.999909, -0.013498, 0, 0.999909, -0.013498, 0, 0.999909, -0.013498, 0, 0.999909, -0.013498, 0, 0.999909, -0.013498, 0, 0.999909, -0.013498, 0, 0.999887, -0.015040, 0, 0.999887, -0.015040, 0, 0.999887, -0.015040, 0, 0.999887, -0.015040, 0, 0.999887, -0.015040, 0, 0.999887, -0.015040, 0, 0.999858, -0.016838, 0, 0.999858, -0.016838, 0, 0.999858, -0.016838, 0, 0.999858, -0.016838, 0, 0.999858, -0.016838, 0, 0.999858, -0.016838, 0, 0.999822, -0.018892, 0, 0.999822, -0.018892, 0, 0.999822, -0.018892, 0, 0.999822, -0.018892, 0, 0.999822, -0.018892, 0, 0.999822, -0.018892, 0, 0.999775, -0.021205, 0, 0.999775, -0.021205, 0, 0.999775, -0.021205, 0, 0.999775, -0.021205, 0, 0.999775, -0.021205, 0, 0.999775, -0.021205, 0, 0.999739, -0.022848, 0, 0.999739, -0.022848, 0, 0.999739, -0.022848, 0, 0.999739, -0.022848, 0, 0.999739, -0.022848, 0, 0.999739, -0.022848, 0, 0.999703, -0.024363, 0, 0.999703, -0.024363, 0, 0.999703, -0.024363, 0, 0.999703, -0.024363, 0, 0.999703, -0.024363, 0, 0.999703, -0.024363, 0, 0.999668, -0.025753, 0, 0.999668, -0.025753, 0, 0.999668, -0.025753, 0, 0.999668, -0.025753, 0, 0.999668, -0.025753, 0, 0.999668, -0.025753, 0, 0.999635, -0.027017, 0, 0.999635, -0.027017, 0, 0.999635, -0.027017, 0, 0.999635, -0.027017, 0, 0.999635, -0.027017, 0, 0.999635, -0.027017, 0, 0.999604, -0.028132, 0, 0.999604, -0.028132, 0, 0.999604, -0.028132, 0, 0.999604, -0.028132, 0, 0.999604, -0.028132, 0, 0.999604, -0.028132, 0, 0.999577, -0.029076, 0, 0.999577, -0.029076, 0, 0.999577, -0.029076, 0, 0.999577, -0.029076, 0, 0.999577, -0.029076, 0, 0.999577, -0.029076, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0.999555, -0.029847, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998907, -0.046745, 0, 0.998907, -0.046745, 0, 0.998907, -0.046745, 0, 0.998907, -0.046745, 0, 0.998907, -0.046745, 0, 0.998907, -0.046745, 0, 0.998699, -0.050984, 0, 0.998699, -0.050984, 0, 0.998699, -0.050984, 0, 0.998699, -0.050984, 0, 0.998699, -0.050984, 0, 0.998699, -0.050984, 0, 0.998632, -0.052299, 0, 0.998632, -0.052299, 0, 0.998632, -0.052299, 0, 0.998632, -0.052299, 0, 0.998632, -0.052299, 0, 0.998632, -0.052299, 0, 0.998850, -0.047951, 0, 0.998850, -0.047951, 0, 0.998850, -0.047951, 0, 0.998850, -0.047951, 0, 0.998850, -0.047951, 0, 0.998850, -0.047951, 0, 0.998939, -0.046064, 0, 0.998939, -0.046064, 0, 0.998939, -0.046064, 0, 0.998939, -0.046064, 0, 0.998939, -0.046064, 0, 0.998939, -0.046064, 0, 0.999037, -0.043885, 0, 0.999037, -0.043885, 0, 0.999037, -0.043885, 0, 0.999037, -0.043885, 0, 0.999037, -0.043885, 0, 0.999037, -0.043885, 0, 0.999142, -0.041418, 0, 0.999142, -0.041418, 0, 0.999142, -0.041418, 0, 0.999142, -0.041418, 0, 0.999142, -0.041418, 0, 0.999142, -0.041418, 0, 0.999252, -0.038666, 0, 0.999252, -0.038666, 0, 0.999252, -0.038666, 0, 0.999252, -0.038666, 0, 0.999252, -0.038666, 0, 0.999252, -0.038666, 0, 0.999350, -0.036036, 0, 0.999350, -0.036036, 0, 0.999350, -0.036036, 0, 0.999350, -0.036036, 0, 0.999350, -0.036036, 0, 0.999350, -0.036036, 0, 0.999424, -0.033931, 0, 0.999424, -0.033931, 0, 0.999424, -0.033931, 0, 0.999424, -0.033931, 0, 0.999424, -0.033931, 0, 0.999424, -0.033931, 0, 0.999477, -0.032342, 0, 0.999477, -0.032342, 0, 0.999477, -0.032342, 0, 0.999477, -0.032342, 0, 0.999477, -0.032342, 0, 0.999477, -0.032342, 0, 0.999511, -0.031267, 0, 0.999511, -0.031267, 0, 0.999511, -0.031267, 0, 0.999511, -0.031267, 0, 0.999511, -0.031267, 0, 0.999511, -0.031267, 0, 0.999556, -0.029781, 0, 0.999556, -0.029781, 0, 0.999556, -0.029781, 0, 0.999556, -0.029781, 0, 0.999556, -0.029781, 0, 0.999556, -0.029781, 0, 0.999596, -0.028420, 0, 0.999596, -0.028420, 0, 0.999596, -0.028420, 0, 0.999596, -0.028420, 0, 0.999596, -0.028420, 0, 0.999596, -0.028420, 0, 0.999631, -0.027182, 0, 0.999631, -0.027182, 0, 0.999631, -0.027182, 0, 0.999631, -0.027182, 0, 0.999631, -0.027182, 0, 0.999631, -0.027182, 0, 0.999660, -0.026068, 0, 0.999660, -0.026068, 0, 0.999660, -0.026068, 0, 0.999660, -0.026068, 0, 0.999660, -0.026068, 0, 0.999660, -0.026068, 0, 0.999687, -0.025021, 0, 0.999687, -0.025021, 0, 0.999687, -0.025021, 0, 0.999687, -0.025021, 0, 0.999687, -0.025021, 0, 0.999687, -0.025021, 0, 0.999712, -0.023986, 0, 0.999712, -0.023986, 0, 0.999712, -0.023986, 0, 0.999712, -0.023986, 0, 0.999712, -0.023986, 0, 0.999712, -0.023986, 0, 0.999736, -0.022961, 0, 0.999736, -0.022961, 0, 0.999736, -0.022961, 0, 0.999736, -0.022961, 0, 0.999736, -0.022961, 0, 0.999736, -0.022961, 0, 0.999759, -0.021947, 0, 0.999759, -0.021947, 0, 0.999759, -0.021947, 0, 0.999759, -0.021947, 0, 0.999759, -0.021947, 0, 0.999759, -0.021947, 0, 0.999792, -0.020389, 0, 0.999792, -0.020389, 0, 0.999792, -0.020389, 0, 0.999792, -0.020389, 0, 0.999792, -0.020389, 0, 0.999792, -0.020389, 0, 0.999820, -0.018987, 0, 0.999820, -0.018987, 0, 0.999820, -0.018987, 0, 0.999820, -0.018987, 0, 0.999820, -0.018987, 0, 0.999820, -0.018987, 0, 0.999843, -0.017739, 0, 0.999843, -0.017739, 0, 0.999843, -0.017739, 0, 0.999843, -0.017739, 0, 0.999843, -0.017739, 0, 0.999843, -0.017739, 0, 0.999861, -0.016646, 0, 0.999861, -0.016646, 0, 0.999861, -0.016646, 0, 0.999861, -0.016646, 0, 0.999861, -0.016646, 0, 0.999861, -0.016646, 0, 0.999876, -0.015760, 0, 0.999876, -0.015760, 0, 0.999876, -0.015760, 0, 0.999876, -0.015760, 0, 0.999876, -0.015760, 0, 0.999876, -0.015760, 0, 0.999885, -0.015133, 0, 0.999885, -0.015133, 0, 0.999885, -0.015133, 0, 0.999885, -0.015133, 0, 0.999885, -0.015133, 0, 0.999885, -0.015133, 0, 0.999891, -0.014767, 0, 0.999891, -0.014767, 0, 0.999891, -0.014767, 0, 0.999891, -0.014767, 0, 0.999891, -0.014767, 0, 0.999891, -0.014767, 0, 0.999893, -0.014662, 0, 0.999893, -0.014662, 0, 0.999893, -0.014662, 0, 0.999893, -0.014662, 0, 0.999893, -0.014662, 0, 0.999893, -0.014662, 0, 0.999891, -0.014748, 0, 0.999891, -0.014748, 0, 0.999891, -0.014748, 0, 0.999891, -0.014748, 0, 0.999891, -0.014748, 0, 0.999891, -0.014748, 0, 0.999888, -0.014969, 0, 0.999888, -0.014969, 0, 0.999888, -0.014969, 0, 0.999888, -0.014969, 0, 0.999888, -0.014969, 0, 0.999888, -0.014969, 0, 0.999883, -0.015326, 0, 0.999883, -0.015326, 0, 0.999883, -0.015326, 0, 0.999883, -0.015326, 0, 0.999883, -0.015326, 0, 0.999883, -0.015326, 0, 0.999875, -0.015819, 0, 0.999875, -0.015819, 0, 0.999875, -0.015819, 0, 0.999875, -0.015819, 0, 0.999875, -0.015819, 0, 0.999875, -0.015819, 0, 0.999865, -0.016450, 0, 0.999865, -0.016450, 0, 0.999865, -0.016450, 0, 0.999865, -0.016450, 0, 0.999865, -0.016450, 0, 0.999865, -0.016450, 0, 0.999852, -0.017223, 0, 0.999852, -0.017223, 0, 0.999852, -0.017223, 0, 0.999852, -0.017223, 0, 0.999852, -0.017223, 0, 0.999852, -0.017223, 0, 0.999836, -0.018137, 0, 0.999836, -0.018137, 0, 0.999836, -0.018137, 0, 0.999836, -0.018137, 0, 0.999836, -0.018137, 0, 0.999836, -0.018137, 0, 0.999816, -0.019194, 0, 0.999816, -0.019194, 0, 0.999816, -0.019194, 0, 0.999816, -0.019194, 0, 0.999816, -0.019194, 0, 0.999816, -0.019194, 0, 0.999914, -0.013088, 0, 0.999914, -0.013088, 0, 0.999914, -0.013088, 0, 0.999914, -0.013088, 0, 0.999914, -0.013088, 0, 0.999914, -0.013088, 0, 0.999922, -0.012460, 0, 0.999922, -0.012460, 0, 0.999922, -0.012460, 0, 0.999922, -0.012460, 0, 0.999922, -0.012460, 0, 0.999922, -0.012460, 0, 0.999929, -0.011913, 0, 0.999929, -0.011913, 0, 0.999929, -0.011913, 0, 0.999929, -0.011913, 0, 0.999929, -0.011913, 0, 0.999929, -0.011913, 0, 0.999934, -0.011448, 0, 0.999934, -0.011448, 0, 0.999934, -0.011448, 0, 0.999934, -0.011448, 0, 0.999934, -0.011448, 0, 0.999934, -0.011448, 0, 0.999939, -0.011063, 0, 0.999939, -0.011063, 0, 0.999939, -0.011063, 0, 0.999939, -0.011063, 0, 0.999939, -0.011063, 0, 0.999939, -0.011063, 0, 0.999942, -0.010760, 0, 0.999942, -0.010760, 0, 0.999942, -0.010760, 0, 0.999942, -0.010760, 0, 0.999942, -0.010760, 0, 0.999942, -0.010760, 0, 0.999945, -0.010541, 0, 0.999945, -0.010541, 0, 0.999945, -0.010541, 0, 0.999945, -0.010541, 0, 0.999945, -0.010541, 0, 0.999945, -0.010541, 0, 0.999946, -0.010405, 0, 0.999946, -0.010405, 0, 0.999946, -0.010405, 0, 0.999946, -0.010405, 0, 0.999946, -0.010405, 0, 0.999946, -0.010405, 0, 0.999946, -0.010352, 0, 0.999946, -0.010352, 0, 0.999946, -0.010352, 0, 0.999946, -0.010352, 0, 0.999946, -0.010352, 0, 0.999946, -0.010352, 0, 0.999945, -0.010520, 0, 0.999945, -0.010520, 0, 0.999945, -0.010520, 0, 0.999945, -0.010520, 0, 0.999945, -0.010520, 0, 0.999945, -0.010520, 0, 0.999939, -0.011086, 0, 0.999939, -0.011086, 0, 0.999939, -0.011086, 0, 0.999939, -0.011086, 0, 0.999939, -0.011086, 0, 0.999939, -0.011086, 0, 0.999927, -0.012049, 0, 0.999927, -0.012049, 0, 0.999927, -0.012049, 0, 0.999927, -0.012049, 0, 0.999927, -0.012049, 0, 0.999927, -0.012049, 0, 0.999910, -0.013405, 0, 0.999910, -0.013405, 0, 0.999910, -0.013405, 0, 0.999910, -0.013405, 0, 0.999910, -0.013405, 0, 0.999910, -0.013405, 0, 0.999886, -0.015091, 0, 0.999886, -0.015091, 0, 0.999886, -0.015091, 0, 0.999886, -0.015091, 0, 0.999886, -0.015091, 0, 0.999886, -0.015091, 0, 0.999855, -0.017042, 0, 0.999855, -0.017042, 0, 0.999855, -0.017042, 0, 0.999855, -0.017042, 0, 0.999855, -0.017042, 0, 0.999855, -0.017042, 0, 0.999815, -0.019261, 0, 0.999815, -0.019261, 0, 0.999815, -0.019261, 0, 0.999815, -0.019261, 0, 0.999815, -0.019261, 0, 0.999815, -0.019261, 0, 0.999763, -0.021749, 0, 0.999763, -0.021749, 0, 0.999763, -0.021749, 0, 0.999763, -0.021749, 0, 0.999763, -0.021749, 0, 0.999763, -0.021749, 0, 0.999736, -0.022983, 0, 0.999736, -0.022983, 0, 0.999736, -0.022983, 0, 0.999736, -0.022983, 0, 0.999736, -0.022983, 0, 0.999736, -0.022983, 0, 0.999708, -0.024186, 0, 0.999708, -0.024186, 0, 0.999708, -0.024186, 0, 0.999708, -0.024186, 0, 0.999708, -0.024186, 0, 0.999708, -0.024186, 0, 0.999678, -0.025359, 0, 0.999678, -0.025359, 0, 0.999678, -0.025359, 0, 0.999678, -0.025359, 0, 0.999678, -0.025359, 0, 0.999678, -0.025359, 0, 0.999649, -0.026503, 0, 0.999649, -0.026503, 0, 0.999649, -0.026503, 0, 0.999649, -0.026503, 0, 0.999649, -0.026503, 0, 0.999649, -0.026503, 0, 0.999617, -0.027686, 0, 0.999617, -0.027686, 0, 0.999617, -0.027686, 0, 0.999617, -0.027686, 0, 0.999617, -0.027686, 0, 0.999617, -0.027686, 0, 0.999580, -0.028977, 0, 0.999580, -0.028977, 0, 0.999580, -0.028977, 0, 0.999580, -0.028977, 0, 0.999580, -0.028977, 0, 0.999580, -0.028977, 0, 0.999539, -0.030376, 0, 0.999539, -0.030376, 0, 0.999539, -0.030376, 0, 0.999539, -0.030376, 0, 0.999539, -0.030376, 0, 0.999539, -0.030376, 0, 0.999492, -0.031886, 0, 0.999492, -0.031886, 0, 0.999492, -0.031886, 0, 0.999492, -0.031886, 0, 0.999492, -0.031886, 0, 0.999492, -0.031886, 0, 0.999472, -0.032482, 0, 0.999472, -0.032482, 0, 0.999472, -0.032482, 0, 0.999472, -0.032482, 0, 0.999472, -0.032482, 0, 0.999472, -0.032482, 0, 0.999440, -0.033463, 0, 0.999440, -0.033463, 0, 0.999440, -0.033463, 0, 0.999440, -0.033463, 0, 0.999440, -0.033463, 0, 0.999440, -0.033463, 0, 0.999393, -0.034830, 0, 0.999393, -0.034830, 0, 0.999393, -0.034830, 0, 0.999393, -0.034830, 0, 0.999393, -0.034830, 0, 0.999393, -0.034830, 0, 0.999330, -0.036587, 0, 0.999330, -0.036587, 0, 0.999330, -0.036587, 0, 0.999330, -0.036587, 0, 0.999330, -0.036587, 0, 0.999330, -0.036587, 0, 0.999249, -0.038740, 0, 0.999249, -0.038740, 0, 0.999249, -0.038740, 0, 0.999249, -0.038740, 0, 0.999249, -0.038740, 0, 0.999249, -0.038740, 0, 0.999147, -0.041295, 0, 0.999147, -0.041295, 0, 0.999147, -0.041295, 0, 0.999147, -0.041295, 0, 0.999147, -0.041295, 0, 0.999147, -0.041295, 0, 0.999020, -0.044259, 0, 0.999020, -0.044259, 0, 0.999020, -0.044259, 0, 0.999020, -0.044259, 0, 0.999020, -0.044259, 0, 0.999020, -0.044259, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999055, -0.043458, 0, 0.999055, -0.043458, 0, 0.999055, -0.043458, 0, 0.999055, -0.043458, 0, 0.999055, -0.043458, 0, 0.999055, -0.043458, 0, 0.998743, -0.050128, 0, 0.998743, -0.050128, 0, 0.998743, -0.050128, 0, 0.998743, -0.050128, 0, 0.998743, -0.050128, 0, 0.998743, -0.050128, 0, 0.998729, -0.050405, 0, 0.998729, -0.050405, 0, 0.998729, -0.050405, 0, 0.998729, -0.050405, 0, 0.998729, -0.050405, 0, 0.998729, -0.050405, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998976, -0.045236, 0, 0.998976, -0.045236, 0, 0.998976, -0.045236, 0, 0.998976, -0.045236, 0, 0.998976, -0.045236, 0, 0.998976, -0.045236, 0, 0.998989, -0.044955, 0, 0.998989, -0.044955, 0, 0.998989, -0.044955, 0, 0.998989, -0.044955, 0, 0.998989, -0.044955, 0, 0.998989, -0.044955, 0, 0.999007, -0.044563, 0, 0.999007, -0.044563, 0, 0.999007, -0.044563, 0, 0.999007, -0.044563, 0, 0.999007, -0.044563, 0, 0.999007, -0.044563, 0, 0.999043, -0.043746, 0, 0.999043, -0.043746, 0, 0.999043, -0.043746, 0, 0.999043, -0.043746, 0, 0.999043, -0.043746, 0, 0.999043, -0.043746, 0, 0.999096, -0.042506, 0, 0.999096, -0.042506, 0, 0.999096, -0.042506, 0, 0.999096, -0.042506, 0, 0.999096, -0.042506, 0, 0.999096, -0.042506, 0, 0.999166, -0.040843, 0, 0.999166, -0.040843, 0, 0.999166, -0.040843, 0, 0.999166, -0.040843, 0, 0.999166, -0.040843, 0, 0.999166, -0.040843, 0, 0.999250, -0.038722, 0, 0.999250, -0.038722, 0, 0.999250, -0.038722, 0, 0.999250, -0.038722, 0, 0.999250, -0.038722, 0, 0.999250, -0.038722, 0, 0.999348, -0.036103, 0, 0.999348, -0.036103, 0, 0.999348, -0.036103, 0, 0.999348, -0.036103, 0, 0.999348, -0.036103, 0, 0.999348, -0.036103, 0, 0.999456, -0.032985, 0, 0.999456, -0.032985, 0, 0.999456, -0.032985, 0, 0.999456, -0.032985, 0, 0.999456, -0.032985, 0, 0.999456, -0.032985, 0, 0.999569, -0.029365, 0, 0.999569, -0.029365, 0, 0.999569, -0.029365, 0, 0.999569, -0.029365, 0, 0.999569, -0.029365, 0, 0.999569, -0.029365, 0, 0.999668, -0.025779, 0, 0.999668, -0.025779, 0, 0.999668, -0.025779, 0, 0.999668, -0.025779, 0, 0.999668, -0.025779, 0, 0.999668, -0.025779, 0, 0.999742, -0.022717, 0, 0.999742, -0.022717, 0, 0.999742, -0.022717, 0, 0.999742, -0.022717, 0, 0.999742, -0.022717, 0, 0.999742, -0.022717, 0, 0.999796, -0.020175, 0, 0.999796, -0.020175, 0, 0.999796, -0.020175, 0, 0.999796, -0.020175, 0, 0.999796, -0.020175, 0, 0.999796, -0.020175, 0, 0.999835, -0.018151, 0, 0.999835, -0.018151, 0, 0.999835, -0.018151, 0, 0.999835, -0.018151, 0, 0.999835, -0.018151, 0, 0.999835, -0.018151, 0, 0.999862, -0.016614, 0, 0.999862, -0.016614, 0, 0.999862, -0.016614, 0, 0.999862, -0.016614, 0, 0.999862, -0.016614, 0, 0.999862, -0.016614, 0, 0.999879, -0.015528, 0, 0.999879, -0.015528, 0, 0.999879, -0.015528, 0, 0.999879, -0.015528, 0, 0.999879, -0.015528, 0, 0.999879, -0.015528, 0, 0.999889, -0.014898, 0, 0.999889, -0.014898, 0, 0.999889, -0.014898, 0, 0.999889, -0.014898, 0, 0.999889, -0.014898, 0, 0.999889, -0.014898, 0, 0.999892, -0.014726, 0, 0.999892, -0.014726, 0, 0.999892, -0.014726, 0, 0.999892, -0.014726, 0, 0.999892, -0.014726, 0, 0.999892, -0.014726, 0, 0.999943, -0.010644, 0, 0.999943, -0.010644, 0, 0.999943, -0.010644, 0, 0.999943, -0.010644, 0, 0.999943, -0.010644, 0, 0.999943, -0.010644, 0, 0.999941, -0.010878, 0, 0.999941, -0.010878, 0, 0.999941, -0.010878, 0, 0.999941, -0.010878, 0, 0.999941, -0.010878, 0, 0.999941, -0.010878, 0, 0.999932, -0.011694, 0, 0.999932, -0.011694, 0, 0.999932, -0.011694, 0, 0.999932, -0.011694, 0, 0.999932, -0.011694, 0, 0.999932, -0.011694, 0, 0.999914, -0.013085, 0, 0.999914, -0.013085, 0, 0.999914, -0.013085, 0, 0.999914, -0.013085, 0, 0.999914, -0.013085, 0, 0.999914, -0.013085, 0, 0.999887, -0.015049, 0, 0.999887, -0.015049, 0, 0.999887, -0.015049, 0, 0.999887, -0.015049, 0, 0.999887, -0.015049, 0, 0.999887, -0.015049, 0, 0.999845, -0.017619, 0, 0.999845, -0.017619, 0, 0.999845, -0.017619, 0, 0.999845, -0.017619, 0, 0.999845, -0.017619, 0, 0.999845, -0.017619, 0, 0.999783, -0.020834, 0, 0.999783, -0.020834, 0, 0.999783, -0.020834, 0, 0.999783, -0.020834, 0, 0.999783, -0.020834, 0, 0.999783, -0.020834, 0, 0.999695, -0.024695, 0, 0.999695, -0.024695, 0, 0.999695, -0.024695, 0, 0.999695, -0.024695, 0, 0.999695, -0.024695, 0, 0.999695, -0.024695, 0, 0.999573, -0.029208, 0, 0.999573, -0.029208, 0, 0.999573, -0.029208, 0, 0.999573, -0.029208, 0, 0.999573, -0.029208, 0, 0.999573, -0.029208, 0, 0.999430, -0.033761, 0, 0.999430, -0.033761, 0, 0.999430, -0.033761, 0, 0.999430, -0.033761, 0, 0.999430, -0.033761, 0, 0.999430, -0.033761, 0, 0.999289, -0.037692, 0, 0.999289, -0.037692, 0, 0.999289, -0.037692, 0, 0.999289, -0.037692, 0, 0.999289, -0.037692, 0, 0.999289, -0.037692, 0, 0.999159, -0.041008, 0, 0.999159, -0.041008, 0, 0.999159, -0.041008, 0, 0.999159, -0.041008, 0, 0.999159, -0.041008, 0, 0.999159, -0.041008, 0, 0.999044, -0.043713, 0, 0.999044, -0.043713, 0, 0.999044, -0.043713, 0, 0.999044, -0.043713, 0, 0.999044, -0.043713, 0, 0.999044, -0.043713, 0, 0.998948, -0.045848, 0, 0.998948, -0.045848, 0, 0.998948, -0.045848, 0, 0.998948, -0.045848, 0, 0.998948, -0.045848, 0, 0.998948, -0.045848, 0, 0.998873, -0.047455, 0, 0.998873, -0.047455, 0, 0.998873, -0.047455, 0, 0.998873, -0.047455, 0, 0.998873, -0.047455, 0, 0.998873, -0.047455, 0, 0.998822, -0.048530, 0, 0.998822, -0.048530, 0, 0.998822, -0.048530, 0, 0.998822, -0.048530, 0, 0.998822, -0.048530, 0, 0.998822, -0.048530, 0, 0.998795, -0.049071, 0, 0.998795, -0.049071, 0, 0.998795, -0.049071, 0, 0.998795, -0.049071, 0, 0.998795, -0.049071, 0, 0.998795, -0.049071, 0, 0.998769, -0.049596, 0, 0.998769, -0.049596, 0, 0.998769, -0.049596, 0, 0.998769, -0.049596, 0, 0.998769, -0.049596, 0, 0.998769, -0.049596, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999947, -0.010288, 0, 0.999947, -0.010288, 0, 0.999947, -0.010288, 0, 0.999947, -0.010288, 0, 0.999947, -0.010288, 0, 0.999947, -0.010288, 0, 0.999054, -0.043485, 0, 0.999054, -0.043485, 0, 0.999054, -0.043485, 0, 0.999054, -0.043485, 0, 0.999054, -0.043485, 0, 0.999054, -0.043485, 0, 0.999087, -0.042719, 0, 0.999087, -0.042719, 0, 0.999087, -0.042719, 0, 0.999087, -0.042719, 0, 0.999087, -0.042719, 0, 0.999087, -0.042719, 0, 0.998914, -0.046594, 0, 0.998914, -0.046594, 0, 0.998914, -0.046594, 0, 0.998914, -0.046594, 0, 0.998914, -0.046594, 0, 0.998914, -0.046594, 0, 0.998863, -0.047668, 0, 0.998863, -0.047668, 0, 0.998863, -0.047668, 0, 0.998863, -0.047668, 0, 0.998863, -0.047668, 0, 0.998863, -0.047668, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.999950, -0.010003, 0, 0.999950, -0.010003, 0, 0.999950, -0.010003, 0, 0.999950, -0.010003, 0, 0.999950, -0.010003, 0, 0.999950, -0.010003, 0, 0.999106, -0.042283, 0, 0.999106, -0.042283, 0, 0.999106, -0.042283, 0, 0.999106, -0.042283, 0, 0.999106, -0.042283, 0, 0.999106, -0.042283, 0, 0.999136, -0.041558, 0, 0.999136, -0.041558, 0, 0.999136, -0.041558, 0, 0.999136, -0.041558, 0, 0.999136, -0.041558, 0, 0.999136, -0.041558, 0, 0.998972, -0.045328, 0, 0.998972, -0.045328, 0, 0.998972, -0.045328, 0, 0.998972, -0.045328, 0, 0.998972, -0.045328, 0, 0.998972, -0.045328, 0, 0.998926, -0.046344, 0, 0.998926, -0.046344, 0, 0.998926, -0.046344, 0, 0.998926, -0.046344, 0, 0.998926, -0.046344, 0, 0.998926, -0.046344, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal15.vector = m_vector;
    Normal15.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate7.m_parent = (X3dNode *)&Text1m_5;
    {
    static float m_point[] = { 0.018658, 0.755256, 0.019417, 0.751238, 0.018658, 0.755256, 0.018658, 0.755256, 0.019417, 0.751238, 0.019417, 0.751238, 0.019417, 0.751238, 0.020165, 0.745437, 0.019417, 0.751238, 0.019417, 0.751238, 0.020165, 0.745437, 0.020165, 0.745437, 0.020165, 0.745437, 0.020903, 0.737851, 0.020165, 0.745437, 0.020165, 0.745437, 0.020903, 0.737851, 0.020903, 0.737851, 0.020903, 0.737851, 0.021631, 0.728480, 0.020903, 0.737851, 0.020903, 0.737851, 0.021631, 0.728480, 0.021631, 0.728480, 0.021631, 0.728480, 0.022354, 0.717263, 0.021631, 0.728480, 0.021631, 0.728480, 0.022354, 0.717263, 0.022354, 0.717263, 0.022354, 0.717263, 0.023079, 0.704137, 0.022354, 0.717263, 0.022354, 0.717263, 0.023079, 0.704137, 0.023079, 0.704137, 0.023079, 0.704137, 0.023805, 0.689102, 0.023079, 0.704137, 0.023079, 0.704137, 0.023805, 0.689102, 0.023805, 0.689102, 0.023805, 0.689102, 0.024533, 0.672159, 0.023805, 0.689102, 0.023805, 0.689102, 0.024533, 0.672159, 0.024533, 0.672159, 0.024533, 0.672159, 0.029350, 0.554545, 0.024533, 0.672159, 0.024533, 0.672159, 0.029350, 0.554545, 0.029350, 0.554545, 0.029350, 0.554545, 0.024251, 0.554545, 0.029350, 0.554545, 0.029350, 0.554545, 0.024251, 0.554545, 0.024251, 0.554545, 0.024251, 0.554545, 0.019763, 0.664915, 0.024251, 0.554545, 0.024251, 0.554545, 0.019763, 0.664915, 0.019763, 0.664915, 0.019763, 0.664915, 0.018900, 0.684690, 0.019763, 0.664915, 0.019763, 0.664915, 0.018900, 0.684690, 0.018900, 0.684690, 0.018900, 0.684690, 0.018050, 0.700834, 0.018900, 0.684690, 0.018900, 0.684690, 0.018050, 0.700834, 0.018050, 0.700834, 0.018050, 0.700834, 0.017214, 0.713348, 0.018050, 0.700834, 0.018050, 0.700834, 0.017214, 0.713348, 0.017214, 0.713348, 0.017214, 0.713348, 0.016391, 0.722230, 0.017214, 0.713348, 0.017214, 0.713348, 0.016391, 0.722230, 0.016391, 0.722230, 0.016391, 0.722230, 0.015500, 0.728414, 0.016391, 0.722230, 0.016391, 0.722230, 0.015500, 0.728414, 0.015500, 0.728414, 0.015500, 0.728414, 0.014461, 0.732830, 0.015500, 0.728414, 0.015500, 0.728414, 0.014461, 0.732830, 0.014461, 0.732830, 0.014461, 0.732830, 0.013273, 0.735480, 0.014461, 0.732830, 0.014461, 0.732830, 0.013273, 0.735480, 0.013273, 0.735480, 0.013273, 0.735480, 0.011937, 0.736364, 0.013273, 0.735480, 0.013273, 0.735480, 0.011937, 0.736364, 0.011937, 0.736364, 0.011937, 0.736364, 0.006768, 0.736364, 0.011937, 0.736364, 0.011937, 0.736364, 0.006768, 0.736364, 0.006768, 0.736364, 0.006768, 0.736364, 0.006768, 0.554545, 0.006768, 0.736364, 0.006768, 0.736364, 0.006768, 0.554545, 0.006768, 0.554545, 0.006768, 0.554545, 0.002021, 0.554545, 0.006768, 0.554545, 0.006768, 0.554545, 0.002021, 0.554545, 0.002021, 0.554545, 0.002021, 0.554545, 0.002021, 0.981818, 0.002021, 0.554545, 0.002021, 0.554545, 0.002021, 0.981818, 0.002021, 0.981818, 0.002021, 0.981818, 0.012736, 0.981818, 0.002021, 0.981818, 0.002021, 0.981818, 0.012736, 0.981818, 0.012736, 0.981818, 0.012736, 0.981818, 0.015553, 0.979909, 0.012736, 0.981818, 0.012736, 0.981818, 0.015553, 0.979909, 0.015553, 0.979909, 0.015553, 0.979909, 0.017988, 0.974183, 0.015553, 0.979909, 0.015553, 0.979909, 0.017988, 0.974183, 0.017988, 0.974183, 0.017988, 0.974183, 0.020042, 0.964639, 0.017988, 0.974183, 0.017988, 0.974183, 0.020042, 0.964639, 0.020042, 0.964639, 0.020042, 0.964639, 0.021713, 0.951278, 0.020042, 0.964639, 0.020042, 0.964639, 0.021713, 0.951278, 0.021713, 0.951278, 0.021713, 0.951278, 0.023008, 0.934055, 0.021713, 0.951278, 0.021713, 0.951278, 0.023008, 0.934055, 0.023008, 0.934055, 0.023008, 0.934055, 0.023934, 0.912926, 0.023008, 0.934055, 0.023008, 0.934055, 0.023934, 0.912926, 0.023934, 0.912926, 0.023934, 0.912926, 0.024489, 0.887891, 0.023934, 0.912926, 0.023934, 0.912926, 0.024489, 0.887891, 0.024489, 0.887891, 0.024489, 0.887891, 0.024674, 0.858949, 0.024489, 0.887891, 0.024489, 0.887891, 0.024674, 0.858949, 0.024674, 0.858949, 0.024674, 0.858949, 0.024578, 0.839697, 0.024674, 0.858949, 0.024674, 0.858949, 0.024578, 0.839697, 0.024578, 0.839697, 0.024578, 0.839697, 0.024289, 0.822141, 0.024578, 0.839697, 0.024578, 0.839697, 0.024289, 0.822141, 0.024289, 0.822141, 0.024289, 0.822141, 0.023808, 0.806281, 0.024289, 0.822141, 0.024289, 0.822141, 0.023808, 0.806281, 0.023808, 0.806281, 0.023808, 0.806281, 0.023135, 0.792116, 0.023808, 0.806281, 0.023808, 0.806281, 0.023135, 0.792116, 0.023135, 0.792116, 0.023135, 0.792116, 0.022278, 0.779825, 0.023135, 0.792116, 0.023135, 0.792116, 0.022278, 0.779825, 0.022278, 0.779825, 0.022278, 0.779825, 0.021246, 0.769584, 0.022278, 0.779825, 0.022278, 0.779825, 0.021246, 0.769584, 0.021246, 0.769584, 0.021246, 0.769584, 0.020040, 0.761395, 0.021246, 0.769584, 0.021246, 0.769584, 0.020040, 0.761395, 0.020040, 0.761395, 0.002021, 0.981818, 0.006768, 0.936364, 0.012736, 0.981818, 0.006768, 0.936364, 0.002021, 0.981818, 0.012736, 0.981818, 0.006768, 0.936364, 0.002021, 0.981818, 0.006768, 0.781818, 0.002021, 0.981818, 0.006768, 0.936364, 0.006768, 0.781818, 0.002021, 0.981818, 0.002021, 0.554545, 0.006768, 0.781818, 0.002021, 0.554545, 0.002021, 0.981818, 0.006768, 0.781818, 0.002021, 0.554545, 0.006768, 0.736364, 0.006768, 0.781818, 0.006768, 0.736364, 0.002021, 0.554545, 0.006768, 0.781818, 0.002021, 0.554545, 0.006768, 0.554545, 0.006768, 0.736364, 0.006768, 0.554545, 0.002021, 0.554545, 0.006768, 0.736364, 0.006768, 0.781818, 0.006768, 0.736364, 0.011937, 0.736364, 0.006768, 0.736364, 0.006768, 0.781818, 0.011937, 0.736364, 0.006768, 0.781818, 0.011937, 0.736364, 0.012736, 0.781818, 0.011937, 0.736364, 0.006768, 0.781818, 0.012736, 0.781818, 0.012736, 0.781818, 0.011937, 0.736364, 0.013273, 0.735480, 0.011937, 0.736364, 0.012736, 0.781818, 0.013273, 0.735480, 0.012736, 0.781818, 0.013273, 0.735480, 0.014347, 0.783048, 0.013273, 0.735480, 0.012736, 0.781818, 0.014347, 0.783048, 0.014347, 0.783048, 0.013273, 0.735480, 0.014461, 0.732830, 0.013273, 0.735480, 0.014347, 0.783048, 0.014461, 0.732830, 0.014347, 0.783048, 0.014461, 0.732830, 0.015747, 0.786736, 0.014461, 0.732830, 0.014347, 0.783048, 0.015747, 0.786736, 0.014461, 0.732830, 0.015500, 0.728414, 0.015747, 0.786736, 0.015500, 0.728414, 0.014461, 0.732830, 0.015747, 0.786736, 0.018658, 0.755256, 0.015747, 0.786736, 0.015500, 0.728414, 0.015747, 0.786736, 0.018658, 0.755256, 0.015500, 0.728414, 0.016938, 0.792884, 0.015747, 0.786736, 0.018658, 0.755256, 0.015747, 0.786736, 0.016938, 0.792884, 0.018658, 0.755256, 0.017918, 0.801491, 0.016938, 0.792884, 0.018658, 0.755256, 0.016938, 0.792884, 0.017918, 0.801491, 0.018658, 0.755256, 0.017918, 0.801491, 0.018658, 0.755256, 0.020040, 0.761395, 0.018658, 0.755256, 0.017918, 0.801491, 0.020040, 0.761395, 0.017918, 0.801491, 0.020040, 0.761395, 0.021246, 0.769584, 0.020040, 0.761395, 0.017918, 0.801491, 0.021246, 0.769584, 0.018684, 0.812487, 0.017918, 0.801491, 0.021246, 0.769584, 0.017918, 0.801491, 0.018684, 0.812487, 0.021246, 0.769584, 0.018684, 0.812487, 0.021246, 0.769584, 0.022278, 0.779825, 0.021246, 0.769584, 0.018684, 0.812487, 0.022278, 0.779825, 0.018684, 0.812487, 0.022278, 0.779825, 0.023135, 0.792116, 0.022278, 0.779825, 0.018684, 0.812487, 0.023135, 0.792116, 0.019231, 0.825799, 0.018684, 0.812487, 0.023135, 0.792116, 0.018684, 0.812487, 0.019231, 0.825799, 0.023135, 0.792116, 0.019231, 0.825799, 0.023135, 0.792116, 0.023808, 0.806281, 0.023135, 0.792116, 0.019231, 0.825799, 0.023808, 0.806281, 0.024289, 0.822141, 0.019231, 0.825799, 0.023808, 0.806281, 0.019231, 0.825799, 0.024289, 0.822141, 0.023808, 0.806281, 0.024289, 0.822141, 0.019559, 0.841428, 0.019231, 0.825799, 0.019559, 0.841428, 0.024289, 0.822141, 0.019231, 0.825799, 0.019559, 0.841428, 0.024289, 0.822141, 0.024578, 0.839697, 0.024289, 0.822141, 0.019559, 0.841428, 0.024578, 0.839697, 0.019669, 0.859375, 0.019559, 0.841428, 0.024578, 0.839697, 0.019559, 0.841428, 0.019669, 0.859375, 0.024578, 0.839697, 0.019669, 0.859375, 0.024578, 0.839697, 0.024674, 0.858949, 0.024578, 0.839697, 0.019669, 0.859375, 0.024674, 0.858949, 0.024674, 0.858949, 0.019559, 0.877304, 0.019669, 0.859375, 0.019559, 0.877304, 0.024674, 0.858949, 0.019669, 0.859375, 0.024489, 0.887891, 0.019559, 0.877304, 0.024674, 0.858949, 0.019559, 0.877304, 0.024489, 0.887891, 0.024674, 0.858949, 0.024489, 0.887891, 0.019231, 0.892880, 0.019559, 0.877304, 0.019231, 0.892880, 0.024489, 0.887891, 0.019559, 0.877304, 0.023934, 0.912926, 0.019231, 0.892880, 0.024489, 0.887891, 0.019231, 0.892880, 0.023934, 0.912926, 0.024489, 0.887891, 0.023008, 0.934055, 0.019231, 0.892880, 0.023934, 0.912926, 0.019231, 0.892880, 0.023008, 0.934055, 0.023934, 0.912926, 0.018684, 0.906103, 0.019231, 0.892880, 0.023008, 0.934055, 0.019231, 0.892880, 0.018684, 0.906103, 0.023008, 0.934055, 0.021713, 0.951278, 0.018684, 0.906103, 0.023008, 0.934055, 0.018684, 0.906103, 0.021713, 0.951278, 0.023008, 0.934055, 0.017918, 0.916974, 0.018684, 0.906103, 0.021713, 0.951278, 0.018684, 0.906103, 0.017918, 0.916974, 0.021713, 0.951278, 0.020042, 0.964639, 0.017918, 0.916974, 0.021713, 0.951278, 0.017918, 0.916974, 0.020042, 0.964639, 0.021713, 0.951278, 0.016938, 0.925457, 0.017918, 0.916974, 0.020042, 0.964639, 0.017918, 0.916974, 0.016938, 0.925457, 0.020042, 0.964639, 0.017988, 0.974183, 0.016938, 0.925457, 0.020042, 0.964639, 0.016938, 0.925457, 0.017988, 0.974183, 0.020042, 0.964639, 0.015747, 0.931516, 0.016938, 0.925457, 0.017988, 0.974183, 0.016938, 0.925457, 0.015747, 0.931516, 0.017988, 0.974183, 0.015553, 0.979909, 0.015747, 0.931516, 0.017988, 0.974183, 0.015747, 0.931516, 0.015553, 0.979909, 0.017988, 0.974183, 0.015553, 0.979909, 0.014347, 0.935152, 0.015747, 0.931516, 0.014347, 0.935152, 0.015553, 0.979909, 0.015747, 0.931516, 0.012736, 0.981818, 0.014347, 0.935152, 0.015553, 0.979909, 0.014347, 0.935152, 0.012736, 0.981818, 0.015553, 0.979909, 0.012736, 0.981818, 0.012736, 0.936364, 0.014347, 0.935152, 0.012736, 0.936364, 0.012736, 0.981818, 0.014347, 0.935152, 0.006768, 0.936364, 0.012736, 0.936364, 0.012736, 0.981818, 0.012736, 0.936364, 0.006768, 0.936364, 0.012736, 0.981818, 0.018658, 0.755256, 0.015500, 0.728414, 0.016391, 0.722230, 0.015500, 0.728414, 0.018658, 0.755256, 0.016391, 0.722230, 0.018658, 0.755256, 0.016391, 0.722230, 0.017214, 0.713348, 0.016391, 0.722230, 0.018658, 0.755256, 0.017214, 0.713348, 0.019417, 0.751238, 0.018658, 0.755256, 0.017214, 0.713348, 0.018658, 0.755256, 0.019417, 0.751238, 0.017214, 0.713348, 0.019417, 0.751238, 0.017214, 0.713348, 0.020165, 0.745437, 0.017214, 0.713348, 0.019417, 0.751238, 0.020165, 0.745437, 0.020165, 0.745437, 0.017214, 0.713348, 0.020903, 0.737851, 0.017214, 0.713348, 0.020165, 0.745437, 0.020903, 0.737851, 0.017214, 0.713348, 0.018050, 0.700834, 0.020903, 0.737851, 0.018050, 0.700834, 0.017214, 0.713348, 0.020903, 0.737851, 0.020903, 0.737851, 0.018050, 0.700834, 0.021631, 0.728480, 0.018050, 0.700834, 0.020903, 0.737851, 0.021631, 0.728480, 0.021631, 0.728480, 0.018050, 0.700834, 0.022354, 0.717263, 0.018050, 0.700834, 0.021631, 0.728480, 0.022354, 0.717263, 0.018050, 0.700834, 0.018900, 0.684690, 0.022354, 0.717263, 0.018900, 0.684690, 0.018050, 0.700834, 0.022354, 0.717263, 0.022354, 0.717263, 0.018900, 0.684690, 0.023079, 0.704137, 0.018900, 0.684690, 0.022354, 0.717263, 0.023079, 0.704137, 0.018900, 0.684690, 0.019763, 0.664915, 0.023079, 0.704137, 0.019763, 0.664915, 0.018900, 0.684690, 0.023079, 0.704137, 0.023079, 0.704137, 0.019763, 0.664915, 0.023805, 0.689102, 0.019763, 0.664915, 0.023079, 0.704137, 0.023805, 0.689102, 0.023805, 0.689102, 0.019763, 0.664915, 0.024533, 0.672159, 0.019763, 0.664915, 0.023805, 0.689102, 0.024533, 0.672159, 0.019763, 0.664915, 0.024251, 0.554545, 0.024533, 0.672159, 0.024251, 0.554545, 0.019763, 0.664915, 0.024533, 0.672159, 0.024533, 0.672159, 0.024251, 0.554545, 0.029350, 0.554545, 0.024251, 0.554545, 0.024533, 0.672159, 0.029350, 0.554545, 0.006768, 0.936364, 0.006768, 0.781818, 0.006768, 0.936364, 0.006768, 0.936364, 0.006768, 0.781818, 0.006768, 0.781818, 0.006768, 0.781818, 0.012736, 0.781818, 0.006768, 0.781818, 0.006768, 0.781818, 0.012736, 0.781818, 0.012736, 0.781818, 0.012736, 0.781818, 0.014347, 0.783048, 0.012736, 0.781818, 0.012736, 0.781818, 0.014347, 0.783048, 0.014347, 0.783048, 0.014347, 0.783048, 0.015747, 0.786736, 0.014347, 0.783048, 0.014347, 0.783048, 0.015747, 0.786736, 0.015747, 0.786736, 0.015747, 0.786736, 0.016938, 0.792884, 0.015747, 0.786736, 0.015747, 0.786736, 0.016938, 0.792884, 0.016938, 0.792884, 0.016938, 0.792884, 0.017918, 0.801491, 0.016938, 0.792884, 0.016938, 0.792884, 0.017918, 0.801491, 0.017918, 0.801491, 0.017918, 0.801491, 0.018684, 0.812487, 0.017918, 0.801491, 0.017918, 0.801491, 0.018684, 0.812487, 0.018684, 0.812487, 0.018684, 0.812487, 0.019231, 0.825799, 0.018684, 0.812487, 0.018684, 0.812487, 0.019231, 0.825799, 0.019231, 0.825799, 0.019231, 0.825799, 0.019559, 0.841428, 0.019231, 0.825799, 0.019231, 0.825799, 0.019559, 0.841428, 0.019559, 0.841428, 0.019559, 0.841428, 0.019669, 0.859375, 0.019559, 0.841428, 0.019559, 0.841428, 0.019669, 0.859375, 0.019669, 0.859375, 0.019669, 0.859375, 0.019559, 0.877304, 0.019669, 0.859375, 0.019669, 0.859375, 0.019559, 0.877304, 0.019559, 0.877304, 0.019559, 0.877304, 0.019231, 0.892880, 0.019559, 0.877304, 0.019559, 0.877304, 0.019231, 0.892880, 0.019231, 0.892880, 0.019231, 0.892880, 0.018684, 0.906103, 0.019231, 0.892880, 0.019231, 0.892880, 0.018684, 0.906103, 0.018684, 0.906103, 0.018684, 0.906103, 0.017918, 0.916974, 0.018684, 0.906103, 0.018684, 0.906103, 0.017918, 0.916974, 0.017918, 0.916974, 0.017918, 0.916974, 0.016938, 0.925457, 0.017918, 0.916974, 0.017918, 0.916974, 0.016938, 0.925457, 0.016938, 0.925457, 0.016938, 0.925457, 0.015747, 0.931516, 0.016938, 0.925457, 0.016938, 0.925457, 0.015747, 0.931516, 0.015747, 0.931516, 0.015747, 0.931516, 0.014347, 0.935152, 0.015747, 0.931516, 0.015747, 0.931516, 0.014347, 0.935152, 0.014347, 0.935152, 0.014347, 0.935152, 0.012736, 0.936364, 0.014347, 0.935152, 0.014347, 0.935152, 0.012736, 0.936364, 0.012736, 0.936364, 0.055176, 0.728409, 0.055176, 0.700000, 0.055176, 0.728409, 0.055176, 0.728409, 0.055176, 0.700000, 0.055176, 0.700000, 0.055176, 0.700000, 0.035296, 0.700000, 0.055176, 0.700000, 0.055176, 0.700000, 0.035296, 0.700000, 0.035296, 0.700000, 0.035296, 0.700000, 0.035569, 0.674951, 0.035296, 0.700000, 0.035296, 0.700000, 0.035569, 0.674951, 0.035569, 0.674951, 0.035569, 0.674951, 0.036109, 0.653072, 0.035569, 0.674951, 0.035569, 0.674951, 0.036109, 0.653072, 0.036109, 0.653072, 0.036109, 0.653072, 0.036915, 0.634362, 0.036109, 0.653072, 0.036109, 0.653072, 0.036915, 0.634362, 0.036915, 0.634362, 0.036915, 0.634362, 0.037986, 0.618821, 0.036915, 0.634362, 0.036915, 0.634362, 0.037986, 0.618821, 0.037986, 0.618821, 0.037986, 0.618821, 0.039309, 0.606610, 0.037986, 0.618821, 0.037986, 0.618821, 0.039309, 0.606610, 0.039309, 0.606610, 0.039309, 0.606610, 0.040868, 0.597887, 0.039309, 0.606610, 0.039309, 0.606610, 0.040868, 0.597887, 0.040868, 0.597887, 0.040868, 0.597887, 0.042663, 0.592654, 0.040868, 0.597887, 0.040868, 0.597887, 0.042663, 0.592654, 0.042663, 0.592654, 0.042663, 0.592654, 0.044695, 0.590909, 0.042663, 0.592654, 0.042663, 0.592654, 0.044695, 0.590909, 0.044695, 0.590909, 0.044695, 0.590909, 0.045931, 0.591335, 0.044695, 0.590909, 0.044695, 0.590909, 0.045931, 0.591335, 0.045931, 0.591335, 0.045931, 0.591335, 0.047148, 0.592614, 0.045931, 0.591335, 0.045931, 0.591335, 0.047148, 0.592614, 0.047148, 0.592614, 0.047148, 0.592614, 0.048346, 0.594744, 0.047148, 0.592614, 0.047148, 0.592614, 0.048346, 0.594744, 0.048346, 0.594744, 0.048346, 0.594744, 0.049524, 0.597727, 0.048346, 0.594744, 0.048346, 0.594744, 0.049524, 0.597727, 0.049524, 0.597727, 0.049524, 0.597727, 0.050691, 0.601563, 0.049524, 0.597727, 0.049524, 0.597727, 0.050691, 0.601563, 0.050691, 0.601563, 0.050691, 0.601563, 0.051853, 0.606250, 0.050691, 0.601563, 0.050691, 0.601563, 0.051853, 0.606250, 0.051853, 0.606250, 0.051853, 0.606250, 0.053012, 0.611790, 0.051853, 0.606250, 0.051853, 0.606250, 0.053012, 0.611790, 0.053012, 0.611790, 0.053012, 0.611790, 0.054165, 0.618182, 0.053012, 0.611790, 0.053012, 0.611790, 0.054165, 0.618182, 0.054165, 0.618182, 0.054165, 0.618182, 0.054165, 0.572727, 0.054165, 0.618182, 0.054165, 0.618182, 0.054165, 0.572727, 0.054165, 0.572727, 0.054165, 0.572727, 0.052995, 0.566451, 0.054165, 0.572727, 0.054165, 0.572727, 0.052995, 0.566451, 0.052995, 0.566451, 0.052995, 0.566451, 0.051809, 0.560973, 0.052995, 0.566451, 0.052995, 0.566451, 0.051809, 0.560973, 0.051809, 0.560973, 0.051809, 0.560973, 0.050610, 0.556294, 0.051809, 0.560973, 0.051809, 0.560973, 0.050610, 0.556294, 0.050610, 0.556294, 0.050610, 0.556294, 0.049395, 0.552415, 0.050610, 0.556294, 0.050610, 0.556294, 0.049395, 0.552415, 0.049395, 0.552415, 0.049395, 0.552415, 0.048169, 0.549370, 0.049395, 0.552415, 0.049395, 0.552415, 0.048169, 0.549370, 0.048169, 0.549370, 0.048169, 0.549370, 0.046933, 0.547195, 0.048169, 0.549370, 0.048169, 0.549370, 0.046933, 0.547195, 0.046933, 0.547195, 0.046933, 0.547195, 0.045689, 0.545890, 0.046933, 0.547195, 0.046933, 0.547195, 0.045689, 0.545890, 0.045689, 0.545890, 0.045689, 0.545890, 0.044437, 0.545455, 0.045689, 0.545890, 0.045689, 0.545890, 0.044437, 0.545455, 0.044437, 0.545455, 0.044437, 0.545455, 0.041452, 0.548242, 0.044437, 0.545455, 0.044437, 0.545455, 0.041452, 0.548242, 0.041452, 0.548242, 0.041452, 0.548242, 0.038794, 0.556605, 0.041452, 0.548242, 0.041452, 0.548242, 0.038794, 0.556605, 0.038794, 0.556605, 0.038794, 0.556605, 0.036464, 0.570543, 0.038794, 0.556605, 0.038794, 0.556605, 0.036464, 0.570543, 0.036464, 0.570543, 0.036464, 0.570543, 0.034461, 0.590057, 0.036464, 0.570543, 0.036464, 0.570543, 0.034461, 0.590057, 0.034461, 0.590057, 0.034461, 0.590057, 0.032852, 0.614329, 0.034461, 0.590057, 0.034461, 0.590057, 0.032852, 0.614329, 0.032852, 0.614329, 0.032852, 0.614329, 0.031703, 0.642543, 0.032852, 0.614329, 0.032852, 0.614329, 0.031703, 0.642543, 0.031703, 0.642543, 0.031703, 0.642543, 0.031014, 0.674698, 0.031703, 0.642543, 0.031703, 0.642543, 0.031014, 0.674698, 0.031014, 0.674698, 0.031014, 0.674698, 0.030784, 0.710795, 0.031014, 0.674698, 0.031014, 0.674698, 0.030784, 0.710795, 0.030784, 0.710795, 0.030784, 0.710795, 0.031002, 0.748109, 0.030784, 0.710795, 0.030784, 0.710795, 0.031002, 0.748109, 0.031002, 0.748109, 0.031002, 0.748109, 0.031656, 0.781357, 0.031002, 0.748109, 0.031002, 0.748109, 0.031656, 0.781357, 0.031656, 0.781357, 0.031656, 0.781357, 0.032747, 0.810538, 0.031656, 0.781357, 0.031656, 0.781357, 0.032747, 0.810538, 0.032747, 0.810538, 0.032747, 0.810538, 0.034273, 0.835653, 0.032747, 0.810538, 0.032747, 0.810538, 0.034273, 0.835653, 0.034273, 0.835653, 0.034273, 0.835653, 0.036170, 0.855850, 0.034273, 0.835653, 0.034273, 0.835653, 0.036170, 0.855850, 0.036170, 0.855850, 0.036170, 0.855850, 0.038371, 0.870277, 0.036170, 0.855850, 0.036170, 0.855850, 0.038371, 0.870277, 0.038371, 0.870277, 0.038371, 0.870277, 0.040876, 0.878933, 0.038371, 0.870277, 0.038371, 0.870277, 0.040876, 0.878933, 0.040876, 0.878933, 0.040876, 0.878933, 0.043685, 0.881818, 0.040876, 0.878933, 0.040876, 0.878933, 0.043685, 0.881818, 0.043685, 0.881818, 0.043685, 0.881818, 0.046201, 0.879239, 0.043685, 0.881818, 0.043685, 0.881818, 0.046201, 0.879239, 0.046201, 0.879239, 0.046201, 0.879239, 0.048440, 0.871502, 0.046201, 0.879239, 0.046201, 0.879239, 0.048440, 0.871502, 0.048440, 0.871502, 0.048440, 0.871502, 0.050402, 0.858607, 0.048440, 0.871502, 0.048440, 0.871502, 0.050402, 0.858607, 0.050402, 0.858607, 0.050402, 0.858607, 0.052086, 0.840554, 0.050402, 0.858607, 0.050402, 0.858607, 0.052086, 0.840554, 0.052086, 0.840554, 0.052086, 0.840554, 0.053437, 0.818071, 0.052086, 0.840554, 0.052086, 0.840554, 0.053437, 0.818071, 0.053437, 0.818071, 0.053437, 0.818071, 0.054403, 0.791886, 0.053437, 0.818071, 0.053437, 0.818071, 0.054403, 0.791886, 0.054403, 0.791886, 0.054403, 0.791886, 0.054983, 0.761998, 0.054403, 0.791886, 0.054403, 0.791886, 0.054983, 0.761998, 0.054983, 0.761998, 0.031002, 0.748109, 0.030784, 0.710795, 0.035436, 0.745170, 0.030784, 0.710795, 0.031002, 0.748109, 0.035436, 0.745170, 0.031656, 0.781357, 0.031002, 0.748109, 0.035436, 0.745170, 0.031002, 0.748109, 0.031656, 0.781357, 0.035436, 0.745170, 0.031656, 0.781357, 0.035436, 0.745170, 0.035716, 0.765678, 0.035436, 0.745170, 0.031656, 0.781357, 0.035716, 0.765678, 0.036227, 0.783736, 0.031656, 0.781357, 0.035716, 0.765678, 0.031656, 0.781357, 0.036227, 0.783736, 0.035716, 0.765678, 0.036227, 0.783736, 0.032747, 0.810538, 0.031656, 0.781357, 0.032747, 0.810538, 0.036227, 0.783736, 0.031656, 0.781357, 0.032747, 0.810538, 0.036227, 0.783736, 0.036968, 0.799343, 0.036227, 0.783736, 0.032747, 0.810538, 0.036968, 0.799343, 0.034273, 0.835653, 0.032747, 0.810538, 0.036968, 0.799343, 0.032747, 0.810538, 0.034273, 0.835653, 0.036968, 0.799343, 0.034273, 0.835653, 0.036968, 0.799343, 0.037939, 0.812500, 0.036968, 0.799343, 0.034273, 0.835653, 0.037939, 0.812500, 0.036170, 0.855850, 0.034273, 0.835653, 0.037939, 0.812500, 0.034273, 0.835653, 0.036170, 0.855850, 0.037939, 0.812500, 0.036170, 0.855850, 0.037939, 0.812500, 0.039116, 0.822940, 0.037939, 0.812500, 0.036170, 0.855850, 0.039116, 0.822940, 0.038371, 0.870277, 0.036170, 0.855850, 0.039116, 0.822940, 0.036170, 0.855850, 0.038371, 0.870277, 0.039116, 0.822940, 0.038371, 0.870277, 0.039116, 0.822940, 0.040474, 0.830398, 0.039116, 0.822940, 0.038371, 0.870277, 0.040474, 0.830398, 0.038371, 0.870277, 0.040474, 0.830398, 0.040876, 0.878933, 0.040474, 0.830398, 0.038371, 0.870277, 0.040876, 0.878933, 0.040876, 0.878933, 0.040474, 0.830398, 0.042013, 0.834872, 0.040474, 0.830398, 0.040876, 0.878933, 0.042013, 0.834872, 0.040876, 0.878933, 0.042013, 0.834872, 0.043685, 0.881818, 0.042013, 0.834872, 0.040876, 0.878933, 0.043685, 0.881818, 0.043685, 0.881818, 0.042013, 0.834872, 0.043732, 0.836364, 0.042013, 0.834872, 0.043685, 0.881818, 0.043732, 0.836364, 0.043685, 0.881818, 0.043732, 0.836364, 0.045251, 0.834819, 0.043732, 0.836364, 0.043685, 0.881818, 0.045251, 0.834819, 0.043685, 0.881818, 0.045251, 0.834819, 0.046201, 0.879239, 0.045251, 0.834819, 0.043685, 0.881818, 0.046201, 0.879239, 0.046201, 0.879239, 0.045251, 0.834819, 0.046613, 0.830185, 0.045251, 0.834819, 0.046201, 0.879239, 0.046613, 0.830185, 0.046201, 0.879239, 0.046613, 0.830185, 0.048440, 0.871502, 0.046613, 0.830185, 0.046201, 0.879239, 0.048440, 0.871502, 0.046613, 0.830185, 0.047818, 0.822461, 0.048440, 0.871502, 0.047818, 0.822461, 0.046613, 0.830185, 0.048440, 0.871502, 0.048440, 0.871502, 0.047818, 0.822461, 0.050402, 0.858607, 0.047818, 0.822461, 0.048440, 0.871502, 0.050402, 0.858607, 0.047818, 0.822461, 0.048866, 0.811648, 0.050402, 0.858607, 0.048866, 0.811648, 0.047818, 0.822461, 0.050402, 0.858607, 0.050402, 0.858607, 0.048866, 0.811648, 0.052086, 0.840554, 0.048866, 0.811648, 0.050402, 0.858607, 0.052086, 0.840554, 0.048866, 0.811648, 0.049717, 0.798242, 0.052086, 0.840554, 0.049717, 0.798242, 0.048866, 0.811648, 0.052086, 0.840554, 0.052086, 0.840554, 0.049717, 0.798242, 0.053437, 0.818071, 0.049717, 0.798242, 0.052086, 0.840554, 0.053437, 0.818071, 0.049717, 0.798242, 0.050332, 0.782741, 0.053437, 0.818071, 0.050332, 0.782741, 0.049717, 0.798242, 0.053437, 0.818071, 0.053437, 0.818071, 0.050332, 0.782741, 0.054403, 0.791886, 0.050332, 0.782741, 0.053437, 0.818071, 0.054403, 0.791886, 0.050332, 0.782741, 0.050710, 0.765146, 0.054403, 0.791886, 0.050710, 0.765146, 0.050332, 0.782741, 0.054403, 0.791886, 0.054403, 0.791886, 0.050710, 0.765146, 0.054983, 0.761998, 0.050710, 0.765146, 0.054403, 0.791886, 0.054983, 0.761998, 0.050710, 0.765146, 0.050852, 0.745455, 0.054983, 0.761998, 0.050852, 0.745455, 0.050710, 0.765146, 0.054983, 0.761998, 0.054983, 0.761998, 0.050852, 0.745455, 0.055176, 0.728409, 0.050852, 0.745455, 0.054983, 0.761998, 0.055176, 0.728409, 0.055176, 0.700000, 0.055176, 0.728409, 0.050852, 0.745455, 0.055176, 0.728409, 0.055176, 0.700000, 0.050852, 0.745455, 0.050852, 0.745455, 0.035296, 0.700000, 0.055176, 0.700000, 0.035296, 0.700000, 0.050852, 0.745455, 0.055176, 0.700000, 0.050852, 0.745455, 0.035436, 0.745170, 0.035296, 0.700000, 0.035436, 0.745170, 0.050852, 0.745455, 0.035296, 0.700000, 0.030784, 0.710795, 0.035296, 0.700000, 0.035436, 0.745170, 0.035296, 0.700000, 0.030784, 0.710795, 0.035436, 0.745170, 0.030784, 0.710795, 0.031014, 0.674698, 0.035296, 0.700000, 0.031014, 0.674698, 0.030784, 0.710795, 0.035296, 0.700000, 0.035296, 0.700000, 0.031014, 0.674698, 0.035569, 0.674951, 0.031014, 0.674698, 0.035296, 0.700000, 0.035569, 0.674951, 0.031014, 0.674698, 0.031703, 0.642543, 0.035569, 0.674951, 0.031703, 0.642543, 0.031014, 0.674698, 0.035569, 0.674951, 0.035569, 0.674951, 0.031703, 0.642543, 0.036109, 0.653072, 0.031703, 0.642543, 0.035569, 0.674951, 0.036109, 0.653072, 0.031703, 0.642543, 0.032852, 0.614329, 0.036109, 0.653072, 0.032852, 0.614329, 0.031703, 0.642543, 0.036109, 0.653072, 0.036109, 0.653072, 0.032852, 0.614329, 0.036915, 0.634362, 0.032852, 0.614329, 0.036109, 0.653072, 0.036915, 0.634362, 0.032852, 0.614329, 0.034461, 0.590057, 0.036915, 0.634362, 0.034461, 0.590057, 0.032852, 0.614329, 0.036915, 0.634362, 0.036915, 0.634362, 0.034461, 0.590057, 0.037986, 0.618821, 0.034461, 0.590057, 0.036915, 0.634362, 0.037986, 0.618821, 0.034461, 0.590057, 0.036464, 0.570543, 0.037986, 0.618821, 0.036464, 0.570543, 0.034461, 0.590057, 0.037986, 0.618821, 0.037986, 0.618821, 0.036464, 0.570543, 0.039309, 0.606610, 0.036464, 0.570543, 0.037986, 0.618821, 0.039309, 0.606610, 0.039309, 0.606610, 0.036464, 0.570543, 0.038794, 0.556605, 0.036464, 0.570543, 0.039309, 0.606610, 0.038794, 0.556605, 0.040868, 0.597887, 0.039309, 0.606610, 0.038794, 0.556605, 0.039309, 0.606610, 0.040868, 0.597887, 0.038794, 0.556605, 0.040868, 0.597887, 0.038794, 0.556605, 0.041452, 0.548242, 0.038794, 0.556605, 0.040868, 0.597887, 0.041452, 0.548242, 0.042663, 0.592654, 0.040868, 0.597887, 0.041452, 0.548242, 0.040868, 0.597887, 0.042663, 0.592654, 0.041452, 0.548242, 0.042663, 0.592654, 0.041452, 0.548242, 0.044437, 0.545455, 0.041452, 0.548242, 0.042663, 0.592654, 0.044437, 0.545455, 0.044695, 0.590909, 0.042663, 0.592654, 0.044437, 0.545455, 0.042663, 0.592654, 0.044695, 0.590909, 0.044437, 0.545455, 0.044695, 0.590909, 0.044437, 0.545455, 0.045689, 0.545890, 0.044437, 0.545455, 0.044695, 0.590909, 0.045689, 0.545890, 0.044695, 0.590909, 0.045689, 0.545890, 0.045931, 0.591335, 0.045689, 0.545890, 0.044695, 0.590909, 0.045931, 0.591335, 0.045931, 0.591335, 0.045689, 0.545890, 0.046933, 0.547195, 0.045689, 0.545890, 0.045931, 0.591335, 0.046933, 0.547195, 0.045931, 0.591335, 0.046933, 0.547195, 0.047148, 0.592614, 0.046933, 0.547195, 0.045931, 0.591335, 0.047148, 0.592614, 0.047148, 0.592614, 0.046933, 0.547195, 0.048169, 0.549370, 0.046933, 0.547195, 0.047148, 0.592614, 0.048169, 0.549370, 0.047148, 0.592614, 0.048169, 0.549370, 0.048346, 0.594744, 0.048169, 0.549370, 0.047148, 0.592614, 0.048346, 0.594744, 0.048346, 0.594744, 0.048169, 0.549370, 0.049395, 0.552415, 0.048169, 0.549370, 0.048346, 0.594744, 0.049395, 0.552415, 0.048346, 0.594744, 0.049395, 0.552415, 0.049524, 0.597727, 0.049395, 0.552415, 0.048346, 0.594744, 0.049524, 0.597727, 0.049524, 0.597727, 0.049395, 0.552415, 0.050610, 0.556294, 0.049395, 0.552415, 0.049524, 0.597727, 0.050610, 0.556294, 0.050691, 0.601563, 0.049524, 0.597727, 0.050610, 0.556294, 0.049524, 0.597727, 0.050691, 0.601563, 0.050610, 0.556294, 0.051809, 0.560973, 0.050691, 0.601563, 0.050610, 0.556294, 0.050691, 0.601563, 0.051809, 0.560973, 0.050610, 0.556294, 0.050691, 0.601563, 0.051809, 0.560973, 0.052995, 0.566451, 0.051809, 0.560973, 0.050691, 0.601563, 0.052995, 0.566451, 0.051853, 0.606250, 0.050691, 0.601563, 0.052995, 0.566451, 0.050691, 0.601563, 0.051853, 0.606250, 0.052995, 0.566451, 0.051853, 0.606250, 0.052995, 0.566451, 0.054165, 0.572727, 0.052995, 0.566451, 0.051853, 0.606250, 0.054165, 0.572727, 0.053012, 0.611790, 0.051853, 0.606250, 0.054165, 0.572727, 0.051853, 0.606250, 0.053012, 0.611790, 0.054165, 0.572727, 0.053012, 0.611790, 0.054165, 0.572727, 0.054165, 0.618182, 0.054165, 0.572727, 0.053012, 0.611790, 0.054165, 0.618182, 0.050852, 0.745455, 0.050710, 0.765146, 0.050852, 0.745455, 0.050852, 0.745455, 0.050710, 0.765146, 0.050710, 0.765146, 0.050710, 0.765146, 0.050332, 0.782741, 0.050710, 0.765146, 0.050710, 0.765146, 0.050332, 0.782741, 0.050332, 0.782741, 0.050332, 0.782741, 0.049717, 0.798242, 0.050332, 0.782741, 0.050332, 0.782741, 0.049717, 0.798242, 0.049717, 0.798242, 0.049717, 0.798242, 0.048866, 0.811648, 0.049717, 0.798242, 0.049717, 0.798242, 0.048866, 0.811648, 0.048866, 0.811648, 0.048866, 0.811648, 0.047818, 0.822461, 0.048866, 0.811648, 0.048866, 0.811648, 0.047818, 0.822461, 0.047818, 0.822461, 0.047818, 0.822461, 0.046613, 0.830185, 0.047818, 0.822461, 0.047818, 0.822461, 0.046613, 0.830185, 0.046613, 0.830185, 0.046613, 0.830185, 0.045251, 0.834819, 0.046613, 0.830185, 0.046613, 0.830185, 0.045251, 0.834819, 0.045251, 0.834819, 0.045251, 0.834819, 0.043732, 0.836364, 0.045251, 0.834819, 0.045251, 0.834819, 0.043732, 0.836364, 0.043732, 0.836364, 0.043732, 0.836364, 0.042013, 0.834872, 0.043732, 0.836364, 0.043732, 0.836364, 0.042013, 0.834872, 0.042013, 0.834872, 0.042013, 0.834872, 0.040474, 0.830398, 0.042013, 0.834872, 0.042013, 0.834872, 0.040474, 0.830398, 0.040474, 0.830398, 0.040474, 0.830398, 0.039116, 0.822940, 0.040474, 0.830398, 0.040474, 0.830398, 0.039116, 0.822940, 0.039116, 0.822940, 0.039116, 0.822940, 0.037939, 0.812500, 0.039116, 0.822940, 0.039116, 0.822940, 0.037939, 0.812500, 0.037939, 0.812500, 0.037939, 0.812500, 0.036968, 0.799343, 0.037939, 0.812500, 0.037939, 0.812500, 0.036968, 0.799343, 0.036968, 0.799343, 0.036968, 0.799343, 0.036227, 0.783736, 0.036968, 0.799343, 0.036968, 0.799343, 0.036227, 0.783736, 0.036227, 0.783736, 0.036227, 0.783736, 0.035716, 0.765678, 0.036227, 0.783736, 0.036227, 0.783736, 0.035716, 0.765678, 0.035716, 0.765678, 0.035716, 0.765678, 0.035436, 0.745170, 0.035716, 0.765678, 0.035716, 0.765678, 0.035436, 0.745170, 0.035436, 0.745170, 0.078769, 0.854545, 0.078769, 0.809091, 0.078769, 0.854545, 0.078769, 0.854545, 0.078769, 0.809091, 0.078769, 0.809091, 0.078769, 0.809091, 0.077843, 0.815483, 0.078769, 0.809091, 0.078769, 0.809091, 0.077843, 0.815483, 0.077843, 0.815483, 0.077843, 0.815483, 0.076900, 0.821023, 0.077843, 0.815483, 0.077843, 0.815483, 0.076900, 0.821023, 0.076900, 0.821023, 0.076900, 0.821023, 0.075940, 0.825710, 0.076900, 0.821023, 0.076900, 0.821023, 0.075940, 0.825710, 0.075940, 0.825710, 0.075940, 0.825710, 0.074962, 0.829545, 0.075940, 0.825710, 0.075940, 0.825710, 0.074962, 0.829545, 0.074962, 0.829545, 0.074962, 0.829545, 0.073966, 0.832528, 0.074962, 0.829545, 0.074962, 0.829545, 0.073966, 0.832528, 0.073966, 0.832528, 0.073966, 0.832528, 0.072953, 0.834659, 0.073966, 0.832528, 0.073966, 0.832528, 0.072953, 0.834659, 0.072953, 0.834659, 0.072953, 0.834659, 0.071922, 0.835937, 0.072953, 0.834659, 0.072953, 0.834659, 0.071922, 0.835937, 0.071922, 0.835937, 0.071922, 0.835937, 0.070873, 0.836364, 0.071922, 0.835937, 0.071922, 0.835937, 0.070873, 0.836364, 0.070873, 0.836364, 0.070873, 0.836364, 0.069364, 0.835591, 0.070873, 0.836364, 0.070873, 0.836364, 0.069364, 0.835591, 0.069364, 0.835591, 0.069364, 0.835591, 0.068056, 0.833274, 0.069364, 0.835591, 0.069364, 0.835591, 0.068056, 0.833274, 0.068056, 0.833274, 0.068056, 0.833274, 0.066949, 0.829412, 0.068056, 0.833274, 0.068056, 0.833274, 0.066949, 0.829412, 0.066949, 0.829412, 0.066949, 0.829412, 0.066044, 0.824006, 0.066949, 0.829412, 0.066949, 0.829412, 0.066044, 0.824006, 0.066044, 0.824006, 0.066044, 0.824006, 0.065340, 0.817054, 0.066044, 0.824006, 0.066044, 0.824006, 0.065340, 0.817054, 0.065340, 0.817054, 0.065340, 0.817054, 0.064837, 0.808558, 0.065340, 0.817054, 0.065340, 0.817054, 0.064837, 0.808558, 0.064837, 0.808558, 0.064837, 0.808558, 0.064535, 0.798517, 0.064837, 0.808558, 0.064837, 0.808558, 0.064535, 0.798517, 0.064535, 0.798517, 0.064535, 0.798517, 0.064434, 0.786932, 0.064535, 0.798517, 0.064535, 0.798517, 0.064434, 0.786932, 0.064434, 0.786932, 0.064434, 0.786932, 0.064506, 0.778054, 0.064434, 0.786932, 0.064434, 0.786932, 0.064506, 0.778054, 0.064506, 0.778054, 0.064506, 0.778054, 0.064722, 0.770170, 0.064506, 0.778054, 0.064506, 0.778054, 0.064722, 0.770170, 0.064722, 0.770170, 0.064722, 0.770170, 0.065082, 0.763281, 0.064722, 0.770170, 0.064722, 0.770170, 0.065082, 0.763281, 0.065082, 0.763281, 0.065082, 0.763281, 0.065586, 0.757386, 0.065082, 0.763281, 0.065082, 0.763281, 0.065586, 0.757386, 0.065586, 0.757386, 0.065586, 0.757386, 0.066307, 0.752060, 0.065586, 0.757386, 0.065586, 0.757386, 0.066307, 0.752060, 0.066307, 0.752060, 0.066307, 0.752060, 0.067319, 0.746875, 0.066307, 0.752060, 0.066307, 0.752060, 0.067319, 0.746875, 0.067319, 0.746875, 0.067319, 0.746875, 0.068621, 0.741832, 0.067319, 0.746875, 0.067319, 0.746875, 0.068621, 0.741832, 0.068621, 0.741832, 0.068621, 0.741832, 0.070215, 0.736932, 0.068621, 0.741832, 0.068621, 0.741832, 0.070215, 0.736932, 0.070215, 0.736932, 0.070215, 0.736932, 0.071695, 0.732955, 0.070215, 0.736932, 0.070215, 0.736932, 0.071695, 0.732955, 0.071695, 0.732955, 0.071695, 0.732955, 0.073832, 0.726460, 0.071695, 0.732955, 0.071695, 0.732955, 0.073832, 0.726460, 0.073832, 0.726460, 0.073832, 0.726460, 0.075634, 0.718768, 0.073832, 0.726460, 0.073832, 0.726460, 0.075634, 0.718768, 0.075634, 0.718768, 0.075634, 0.718768, 0.077104, 0.709877, 0.075634, 0.718768, 0.075634, 0.718768, 0.077104, 0.709877, 0.077104, 0.709877, 0.077104, 0.709877, 0.078240, 0.699787, 0.077104, 0.709877, 0.077104, 0.709877, 0.078240, 0.699787, 0.078240, 0.699787, 0.078240, 0.699787, 0.079088, 0.688037, 0.078240, 0.699787, 0.078240, 0.699787, 0.079088, 0.688037, 0.079088, 0.688037, 0.079088, 0.688037, 0.079694, 0.674165, 0.079088, 0.688037, 0.079088, 0.688037, 0.079694, 0.674165, 0.079694, 0.674165, 0.079694, 0.674165, 0.080057, 0.658172, 0.079694, 0.674165, 0.079694, 0.674165, 0.080057, 0.658172, 0.080057, 0.658172, 0.080057, 0.658172, 0.080179, 0.640057, 0.080057, 0.658172, 0.080057, 0.658172, 0.080179, 0.640057, 0.080179, 0.640057, 0.080179, 0.640057, 0.079997, 0.619385, 0.080179, 0.640057, 0.080179, 0.640057, 0.079997, 0.619385, 0.079997, 0.619385, 0.079997, 0.619385, 0.079453, 0.600977, 0.079997, 0.619385, 0.079997, 0.619385, 0.079453, 0.600977, 0.079453, 0.600977, 0.079453, 0.600977, 0.078546, 0.584832, 0.079453, 0.600977, 0.079453, 0.600977, 0.078546, 0.584832, 0.078546, 0.584832, 0.078546, 0.584832, 0.077276, 0.570952, 0.078546, 0.584832, 0.078546, 0.584832, 0.077276, 0.570952, 0.077276, 0.570952, 0.077276, 0.570952, 0.075690, 0.559797, 0.077276, 0.570952, 0.077276, 0.570952, 0.075690, 0.559797, 0.075690, 0.559797, 0.075690, 0.559797, 0.073831, 0.551829, 0.075690, 0.559797, 0.075690, 0.559797, 0.073831, 0.551829, 0.073831, 0.551829, 0.073831, 0.551829, 0.071701, 0.547048, 0.073831, 0.551829, 0.073831, 0.551829, 0.071701, 0.547048, 0.071701, 0.547048, 0.071701, 0.547048, 0.069299, 0.545455, 0.071701, 0.547048, 0.071701, 0.547048, 0.069299, 0.545455, 0.069299, 0.545455, 0.069299, 0.545455, 0.068230, 0.545739, 0.069299, 0.545455, 0.069299, 0.545455, 0.068230, 0.545739, 0.068230, 0.545739, 0.068230, 0.545739, 0.067140, 0.546591, 0.068230, 0.545739, 0.068230, 0.545739, 0.067140, 0.546591, 0.067140, 0.546591, 0.067140, 0.546591, 0.066027, 0.548011, 0.067140, 0.546591, 0.067140, 0.546591, 0.066027, 0.548011, 0.066027, 0.548011, 0.066027, 0.548011, 0.064892, 0.550000, 0.066027, 0.548011, 0.066027, 0.548011, 0.064892, 0.550000, 0.064892, 0.550000, 0.064892, 0.550000, 0.063731, 0.552557, 0.064892, 0.550000, 0.064892, 0.550000, 0.063731, 0.552557, 0.063731, 0.552557, 0.063731, 0.552557, 0.062540, 0.555682, 0.063731, 0.552557, 0.063731, 0.552557, 0.062540, 0.555682, 0.062540, 0.555682, 0.062540, 0.555682, 0.061317, 0.559375, 0.062540, 0.555682, 0.062540, 0.555682, 0.061317, 0.559375, 0.061317, 0.559375, 0.061317, 0.559375, 0.060063, 0.563636, 0.061317, 0.559375, 0.061317, 0.559375, 0.060063, 0.563636, 0.060063, 0.563636, 0.060063, 0.563636, 0.060063, 0.618182, 0.060063, 0.563636, 0.060063, 0.563636, 0.060063, 0.618182, 0.060063, 0.618182, 0.060063, 0.618182, 0.061257, 0.611790, 0.060063, 0.618182, 0.060063, 0.618182, 0.061257, 0.611790, 0.061257, 0.611790, 0.061257, 0.611790, 0.062443, 0.606250, 0.061257, 0.611790, 0.061257, 0.611790, 0.062443, 0.606250, 0.062443, 0.606250, 0.062443, 0.606250, 0.063619, 0.601563, 0.062443, 0.606250, 0.062443, 0.606250, 0.063619, 0.601563, 0.063619, 0.601563, 0.063619, 0.601563, 0.064787, 0.597727, 0.063619, 0.601563, 0.063619, 0.601563, 0.064787, 0.597727, 0.064787, 0.597727, 0.064787, 0.597727, 0.065947, 0.594744, 0.064787, 0.597727, 0.064787, 0.597727, 0.065947, 0.594744, 0.065947, 0.594744, 0.065947, 0.594744, 0.067101, 0.592614, 0.065947, 0.594744, 0.065947, 0.594744, 0.067101, 0.592614, 0.067101, 0.592614, 0.067101, 0.592614, 0.068250, 0.591335, 0.067101, 0.592614, 0.067101, 0.592614, 0.068250, 0.591335, 0.068250, 0.591335, 0.068250, 0.591335, 0.069393, 0.590909, 0.068250, 0.591335, 0.068250, 0.591335, 0.069393, 0.590909, 0.069393, 0.590909, 0.069393, 0.590909, 0.070832, 0.591668, 0.069393, 0.590909, 0.069393, 0.590909, 0.070832, 0.591668, 0.070832, 0.591668, 0.070832, 0.591668, 0.072095, 0.593945, 0.070832, 0.591668, 0.070832, 0.591668, 0.072095, 0.593945, 0.072095, 0.593945, 0.072095, 0.593945, 0.073182, 0.597741, 0.072095, 0.593945, 0.072095, 0.593945, 0.073182, 0.597741, 0.073182, 0.597741, 0.073182, 0.597741, 0.074092, 0.603054, 0.073182, 0.597741, 0.073182, 0.597741, 0.074092, 0.603054, 0.074092, 0.603054, 0.074092, 0.603054, 0.074812, 0.609752, 0.074092, 0.603054, 0.074092, 0.603054, 0.074812, 0.609752, 0.074812, 0.609752, 0.074812, 0.609752, 0.075326, 0.617702, 0.074812, 0.609752, 0.074812, 0.609752, 0.075326, 0.617702, 0.075326, 0.617702, 0.075326, 0.617702, 0.075634, 0.626904, 0.075326, 0.617702, 0.075326, 0.617702, 0.075634, 0.626904, 0.075634, 0.626904, 0.075634, 0.626904, 0.075737, 0.637358, 0.075634, 0.626904, 0.075634, 0.626904, 0.075737, 0.637358, 0.075737, 0.637358, 0.075737, 0.637358, 0.075663, 0.646990, 0.075737, 0.637358, 0.075737, 0.637358, 0.075663, 0.646990, 0.075663, 0.646990, 0.075663, 0.646990, 0.075441, 0.655433, 0.075663, 0.646990, 0.075663, 0.646990, 0.075441, 0.655433, 0.075441, 0.655433, 0.075441, 0.655433, 0.075070, 0.662686, 0.075441, 0.655433, 0.075441, 0.655433, 0.075070, 0.662686, 0.075070, 0.662686, 0.075070, 0.662686, 0.074551, 0.668750, 0.075070, 0.662686, 0.075070, 0.662686, 0.074551, 0.668750, 0.074551, 0.668750, 0.074551, 0.668750, 0.073780, 0.674165, 0.074551, 0.668750, 0.074551, 0.668750, 0.073780, 0.674165, 0.073780, 0.674165, 0.073780, 0.674165, 0.072656, 0.679474, 0.073780, 0.674165, 0.073780, 0.674165, 0.072656, 0.679474, 0.072656, 0.679474, 0.072656, 0.679474, 0.071178, 0.684677, 0.072656, 0.679474, 0.072656, 0.679474, 0.071178, 0.684677, 0.071178, 0.684677, 0.071178, 0.684677, 0.069346, 0.689773, 0.071178, 0.684677, 0.071178, 0.684677, 0.069346, 0.689773, 0.069346, 0.689773, 0.069346, 0.689773, 0.067842, 0.694034, 0.069346, 0.689773, 0.069346, 0.689773, 0.067842, 0.694034, 0.067842, 0.694034, 0.067842, 0.694034, 0.065972, 0.699987, 0.067842, 0.694034, 0.067842, 0.694034, 0.065972, 0.699987, 0.065972, 0.699987, 0.065972, 0.699987, 0.064381, 0.707333, 0.065972, 0.699987, 0.065972, 0.699987, 0.064381, 0.707333, 0.064381, 0.707333, 0.064381, 0.707333, 0.063070, 0.716073, 0.064381, 0.707333, 0.064381, 0.707333, 0.063070, 0.716073, 0.063070, 0.716073, 0.063070, 0.716073, 0.062037, 0.726207, 0.063070, 0.716073, 0.063070, 0.716073, 0.062037, 0.726207, 0.062037, 0.726207, 0.062037, 0.726207, 0.061256, 0.738055, 0.062037, 0.726207, 0.062037, 0.726207, 0.061256, 0.738055, 0.061256, 0.738055, 0.061256, 0.738055, 0.060698, 0.751935, 0.061256, 0.738055, 0.061256, 0.738055, 0.060698, 0.751935, 0.060698, 0.751935, 0.060698, 0.751935, 0.060363, 0.767849, 0.060698, 0.751935, 0.060698, 0.751935, 0.060363, 0.767849, 0.060363, 0.767849, 0.060363, 0.767849, 0.060251, 0.785795, 0.060363, 0.767849, 0.060363, 0.767849, 0.060251, 0.785795, 0.060251, 0.785795, 0.060251, 0.785795, 0.060416, 0.807435, 0.060251, 0.785795, 0.060251, 0.785795, 0.060416, 0.807435, 0.060416, 0.807435, 0.060416, 0.807435, 0.060909, 0.826474, 0.060416, 0.807435, 0.060416, 0.807435, 0.060909, 0.826474, 0.060909, 0.826474, 0.060909, 0.826474, 0.061732, 0.842911, 0.060909, 0.826474, 0.060909, 0.826474, 0.061732, 0.842911, 0.061732, 0.842911, 0.061732, 0.842911, 0.062883, 0.856747, 0.061732, 0.842911, 0.061732, 0.842911, 0.062883, 0.856747, 0.062883, 0.856747, 0.062883, 0.856747, 0.064337, 0.867716, 0.062883, 0.856747, 0.062883, 0.856747, 0.064337, 0.867716, 0.064337, 0.867716, 0.064337, 0.867716, 0.066067, 0.875550, 0.064337, 0.867716, 0.064337, 0.867716, 0.066067, 0.875550, 0.066067, 0.875550, 0.066067, 0.875550, 0.068074, 0.880251, 0.066067, 0.875550, 0.066067, 0.875550, 0.068074, 0.880251, 0.068074, 0.880251, 0.068074, 0.880251, 0.070356, 0.881818, 0.068074, 0.880251, 0.068074, 0.880251, 0.070356, 0.881818, 0.070356, 0.881818, 0.070356, 0.881818, 0.071537, 0.881392, 0.070356, 0.881818, 0.070356, 0.881818, 0.071537, 0.881392, 0.071537, 0.881392, 0.071537, 0.881392, 0.072682, 0.880114, 0.071537, 0.881392, 0.071537, 0.881392, 0.072682, 0.880114, 0.072682, 0.880114, 0.072682, 0.880114, 0.073793, 0.877983, 0.072682, 0.880114, 0.072682, 0.880114, 0.073793, 0.877983, 0.073793, 0.877983, 0.073793, 0.877983, 0.074868, 0.875000, 0.073793, 0.877983, 0.073793, 0.877983, 0.074868, 0.875000, 0.074868, 0.875000, 0.074868, 0.875000, 0.075905, 0.871165, 0.074868, 0.875000, 0.074868, 0.875000, 0.075905, 0.871165, 0.075905, 0.871165, 0.075905, 0.871165, 0.076900, 0.866477, 0.075905, 0.871165, 0.075905, 0.871165, 0.076900, 0.866477, 0.076900, 0.866477, 0.076900, 0.866477, 0.077855, 0.860937, 0.076900, 0.866477, 0.076900, 0.866477, 0.077855, 0.860937, 0.077855, 0.860937, 0.060063, 0.618182, 0.060063, 0.563636, 0.061257, 0.611790, 0.060063, 0.563636, 0.060063, 0.618182, 0.061257, 0.611790, 0.062443, 0.606250, 0.061257, 0.611790, 0.060063, 0.563636, 0.061257, 0.611790, 0.062443, 0.606250, 0.060063, 0.563636, 0.062443, 0.606250, 0.060063, 0.563636, 0.061317, 0.559375, 0.060063, 0.563636, 0.062443, 0.606250, 0.061317, 0.559375, 0.063619, 0.601563, 0.062443, 0.606250, 0.061317, 0.559375, 0.062443, 0.606250, 0.063619, 0.601563, 0.061317, 0.559375, 0.063619, 0.601563, 0.061317, 0.559375, 0.062540, 0.555682, 0.061317, 0.559375, 0.063619, 0.601563, 0.062540, 0.555682, 0.063619, 0.601563, 0.062540, 0.555682, 0.063731, 0.552557, 0.062540, 0.555682, 0.063619, 0.601563, 0.063731, 0.552557, 0.063619, 0.601563, 0.063731, 0.552557, 0.064787, 0.597727, 0.063731, 0.552557, 0.063619, 0.601563, 0.064787, 0.597727, 0.064787, 0.597727, 0.063731, 0.552557, 0.064892, 0.550000, 0.063731, 0.552557, 0.064787, 0.597727, 0.064892, 0.550000, 0.064787, 0.597727, 0.064892, 0.550000, 0.065947, 0.594744, 0.064892, 0.550000, 0.064787, 0.597727, 0.065947, 0.594744, 0.065947, 0.594744, 0.064892, 0.550000, 0.066027, 0.548011, 0.064892, 0.550000, 0.065947, 0.594744, 0.066027, 0.548011, 0.065947, 0.594744, 0.066027, 0.548011, 0.067101, 0.592614, 0.066027, 0.548011, 0.065947, 0.594744, 0.067101, 0.592614, 0.067101, 0.592614, 0.066027, 0.548011, 0.067140, 0.546591, 0.066027, 0.548011, 0.067101, 0.592614, 0.067140, 0.546591, 0.067101, 0.592614, 0.067140, 0.546591, 0.068250, 0.591335, 0.067140, 0.546591, 0.067101, 0.592614, 0.068250, 0.591335, 0.067140, 0.546591, 0.068230, 0.545739, 0.068250, 0.591335, 0.068230, 0.545739, 0.067140, 0.546591, 0.068250, 0.591335, 0.068250, 0.591335, 0.068230, 0.545739, 0.069393, 0.590909, 0.068230, 0.545739, 0.068250, 0.591335, 0.069393, 0.590909, 0.068230, 0.545739, 0.069299, 0.545455, 0.069393, 0.590909, 0.069299, 0.545455, 0.068230, 0.545739, 0.069393, 0.590909, 0.070832, 0.591668, 0.069393, 0.590909, 0.069299, 0.545455, 0.069393, 0.590909, 0.070832, 0.591668, 0.069299, 0.545455, 0.071701, 0.547048, 0.070832, 0.591668, 0.069299, 0.545455, 0.070832, 0.591668, 0.071701, 0.547048, 0.069299, 0.545455, 0.072095, 0.593945, 0.070832, 0.591668, 0.071701, 0.547048, 0.070832, 0.591668, 0.072095, 0.593945, 0.071701, 0.547048, 0.073831, 0.551829, 0.072095, 0.593945, 0.071701, 0.547048, 0.072095, 0.593945, 0.073831, 0.551829, 0.071701, 0.547048, 0.073182, 0.597741, 0.072095, 0.593945, 0.073831, 0.551829, 0.072095, 0.593945, 0.073182, 0.597741, 0.073831, 0.551829, 0.073182, 0.597741, 0.073831, 0.551829, 0.075690, 0.559797, 0.073831, 0.551829, 0.073182, 0.597741, 0.075690, 0.559797, 0.074092, 0.603054, 0.073182, 0.597741, 0.075690, 0.559797, 0.073182, 0.597741, 0.074092, 0.603054, 0.075690, 0.559797, 0.074092, 0.603054, 0.075690, 0.559797, 0.077276, 0.570952, 0.075690, 0.559797, 0.074092, 0.603054, 0.077276, 0.570952, 0.074812, 0.609752, 0.074092, 0.603054, 0.077276, 0.570952, 0.074092, 0.603054, 0.074812, 0.609752, 0.077276, 0.570952, 0.074812, 0.609752, 0.077276, 0.570952, 0.078546, 0.584832, 0.077276, 0.570952, 0.074812, 0.609752, 0.078546, 0.584832, 0.075326, 0.617702, 0.074812, 0.609752, 0.078546, 0.584832, 0.074812, 0.609752, 0.075326, 0.617702, 0.078546, 0.584832, 0.075326, 0.617702, 0.078546, 0.584832, 0.079453, 0.600977, 0.078546, 0.584832, 0.075326, 0.617702, 0.079453, 0.600977, 0.075634, 0.626904, 0.075326, 0.617702, 0.079453, 0.600977, 0.075326, 0.617702, 0.075634, 0.626904, 0.079453, 0.600977, 0.075634, 0.626904, 0.079453, 0.600977, 0.079997, 0.619385, 0.079453, 0.600977, 0.075634, 0.626904, 0.079997, 0.619385, 0.075737, 0.637358, 0.075634, 0.626904, 0.079997, 0.619385, 0.075634, 0.626904, 0.075737, 0.637358, 0.079997, 0.619385, 0.075737, 0.637358, 0.079997, 0.619385, 0.080179, 0.640057, 0.079997, 0.619385, 0.075737, 0.637358, 0.080179, 0.640057, 0.080179, 0.640057, 0.075663, 0.646990, 0.075737, 0.637358, 0.075663, 0.646990, 0.080179, 0.640057, 0.075737, 0.637358, 0.080057, 0.658172, 0.075663, 0.646990, 0.080179, 0.640057, 0.075663, 0.646990, 0.080057, 0.658172, 0.080179, 0.640057, 0.079694, 0.674165, 0.075663, 0.646990, 0.080057, 0.658172, 0.075663, 0.646990, 0.079694, 0.674165, 0.080057, 0.658172, 0.075441, 0.655433, 0.075663, 0.646990, 0.079694, 0.674165, 0.075663, 0.646990, 0.075441, 0.655433, 0.079694, 0.674165, 0.079088, 0.688037, 0.075441, 0.655433, 0.079694, 0.674165, 0.075441, 0.655433, 0.079088, 0.688037, 0.079694, 0.674165, 0.075070, 0.662686, 0.075441, 0.655433, 0.079088, 0.688037, 0.075441, 0.655433, 0.075070, 0.662686, 0.079088, 0.688037, 0.078240, 0.699787, 0.075070, 0.662686, 0.079088, 0.688037, 0.075070, 0.662686, 0.078240, 0.699787, 0.079088, 0.688037, 0.077104, 0.709877, 0.075070, 0.662686, 0.078240, 0.699787, 0.075070, 0.662686, 0.077104, 0.709877, 0.078240, 0.699787, 0.074551, 0.668750, 0.075070, 0.662686, 0.077104, 0.709877, 0.075070, 0.662686, 0.074551, 0.668750, 0.077104, 0.709877, 0.075634, 0.718768, 0.074551, 0.668750, 0.077104, 0.709877, 0.074551, 0.668750, 0.075634, 0.718768, 0.077104, 0.709877, 0.073780, 0.674165, 0.074551, 0.668750, 0.075634, 0.718768, 0.074551, 0.668750, 0.073780, 0.674165, 0.075634, 0.718768, 0.073832, 0.726460, 0.073780, 0.674165, 0.075634, 0.718768, 0.073780, 0.674165, 0.073832, 0.726460, 0.075634, 0.718768, 0.072656, 0.679474, 0.073780, 0.674165, 0.073832, 0.726460, 0.073780, 0.674165, 0.072656, 0.679474, 0.073832, 0.726460, 0.073832, 0.726460, 0.071178, 0.684677, 0.072656, 0.679474, 0.071178, 0.684677, 0.073832, 0.726460, 0.072656, 0.679474, 0.073832, 0.726460, 0.071695, 0.732955, 0.071178, 0.684677, 0.071695, 0.732955, 0.073832, 0.726460, 0.071178, 0.684677, 0.069346, 0.689773, 0.071178, 0.684677, 0.071695, 0.732955, 0.071178, 0.684677, 0.069346, 0.689773, 0.071695, 0.732955, 0.070215, 0.736932, 0.069346, 0.689773, 0.071695, 0.732955, 0.069346, 0.689773, 0.070215, 0.736932, 0.071695, 0.732955, 0.067842, 0.694034, 0.069346, 0.689773, 0.070215, 0.736932, 0.069346, 0.689773, 0.067842, 0.694034, 0.070215, 0.736932, 0.068621, 0.741832, 0.067842, 0.694034, 0.070215, 0.736932, 0.067842, 0.694034, 0.068621, 0.741832, 0.070215, 0.736932, 0.065972, 0.699987, 0.067842, 0.694034, 0.068621, 0.741832, 0.067842, 0.694034, 0.065972, 0.699987, 0.068621, 0.741832, 0.067319, 0.746875, 0.065972, 0.699987, 0.068621, 0.741832, 0.065972, 0.699987, 0.067319, 0.746875, 0.068621, 0.741832, 0.067319, 0.746875, 0.066307, 0.752060, 0.065972, 0.699987, 0.066307, 0.752060, 0.067319, 0.746875, 0.065972, 0.699987, 0.066307, 0.752060, 0.064381, 0.707333, 0.065972, 0.699987, 0.064381, 0.707333, 0.066307, 0.752060, 0.065972, 0.699987, 0.066307, 0.752060, 0.065586, 0.757386, 0.064381, 0.707333, 0.065586, 0.757386, 0.066307, 0.752060, 0.064381, 0.707333, 0.065586, 0.757386, 0.063070, 0.716073, 0.064381, 0.707333, 0.063070, 0.716073, 0.065586, 0.757386, 0.064381, 0.707333, 0.062037, 0.726207, 0.063070, 0.716073, 0.065586, 0.757386, 0.063070, 0.716073, 0.062037, 0.726207, 0.065586, 0.757386, 0.065586, 0.757386, 0.065082, 0.763281, 0.062037, 0.726207, 0.065082, 0.763281, 0.065586, 0.757386, 0.062037, 0.726207, 0.065082, 0.763281, 0.061256, 0.738055, 0.062037, 0.726207, 0.061256, 0.738055, 0.065082, 0.763281, 0.062037, 0.726207, 0.065082, 0.763281, 0.064722, 0.770170, 0.061256, 0.738055, 0.064722, 0.770170, 0.065082, 0.763281, 0.061256, 0.738055, 0.064722, 0.770170, 0.060698, 0.751935, 0.061256, 0.738055, 0.060698, 0.751935, 0.064722, 0.770170, 0.061256, 0.738055, 0.064722, 0.770170, 0.064506, 0.778054, 0.060698, 0.751935, 0.064506, 0.778054, 0.064722, 0.770170, 0.060698, 0.751935, 0.064506, 0.778054, 0.060363, 0.767849, 0.060698, 0.751935, 0.060363, 0.767849, 0.064506, 0.778054, 0.060698, 0.751935, 0.060251, 0.785795, 0.060363, 0.767849, 0.064506, 0.778054, 0.060363, 0.767849, 0.060251, 0.785795, 0.064506, 0.778054, 0.064434, 0.786932, 0.060251, 0.785795, 0.064506, 0.778054, 0.060251, 0.785795, 0.064434, 0.786932, 0.064506, 0.778054, 0.060416, 0.807435, 0.060251, 0.785795, 0.064434, 0.786932, 0.060251, 0.785795, 0.060416, 0.807435, 0.064434, 0.786932, 0.060416, 0.807435, 0.064434, 0.786932, 0.064535, 0.798517, 0.064434, 0.786932, 0.060416, 0.807435, 0.064535, 0.798517, 0.060909, 0.826474, 0.060416, 0.807435, 0.064535, 0.798517, 0.060416, 0.807435, 0.060909, 0.826474, 0.064535, 0.798517, 0.060909, 0.826474, 0.064535, 0.798517, 0.064837, 0.808558, 0.064535, 0.798517, 0.060909, 0.826474, 0.064837, 0.808558, 0.061732, 0.842911, 0.060909, 0.826474, 0.064837, 0.808558, 0.060909, 0.826474, 0.061732, 0.842911, 0.064837, 0.808558, 0.061732, 0.842911, 0.064837, 0.808558, 0.065340, 0.817054, 0.064837, 0.808558, 0.061732, 0.842911, 0.065340, 0.817054, 0.062883, 0.856747, 0.061732, 0.842911, 0.065340, 0.817054, 0.061732, 0.842911, 0.062883, 0.856747, 0.065340, 0.817054, 0.062883, 0.856747, 0.065340, 0.817054, 0.066044, 0.824006, 0.065340, 0.817054, 0.062883, 0.856747, 0.066044, 0.824006, 0.064337, 0.867716, 0.062883, 0.856747, 0.066044, 0.824006, 0.062883, 0.856747, 0.064337, 0.867716, 0.066044, 0.824006, 0.064337, 0.867716, 0.066044, 0.824006, 0.066949, 0.829412, 0.066044, 0.824006, 0.064337, 0.867716, 0.066949, 0.829412, 0.066067, 0.875550, 0.064337, 0.867716, 0.066949, 0.829412, 0.064337, 0.867716, 0.066067, 0.875550, 0.066949, 0.829412, 0.066067, 0.875550, 0.066949, 0.829412, 0.068056, 0.833274, 0.066949, 0.829412, 0.066067, 0.875550, 0.068056, 0.833274, 0.066067, 0.875550, 0.068056, 0.833274, 0.068074, 0.880251, 0.068056, 0.833274, 0.066067, 0.875550, 0.068074, 0.880251, 0.068074, 0.880251, 0.068056, 0.833274, 0.069364, 0.835591, 0.068056, 0.833274, 0.068074, 0.880251, 0.069364, 0.835591, 0.070356, 0.881818, 0.068074, 0.880251, 0.069364, 0.835591, 0.068074, 0.880251, 0.070356, 0.881818, 0.069364, 0.835591, 0.070356, 0.881818, 0.069364, 0.835591, 0.070873, 0.836364, 0.069364, 0.835591, 0.070356, 0.881818, 0.070873, 0.836364, 0.070873, 0.836364, 0.071537, 0.881392, 0.070356, 0.881818, 0.071537, 0.881392, 0.070873, 0.836364, 0.070356, 0.881818, 0.071537, 0.881392, 0.070873, 0.836364, 0.071922, 0.835937, 0.070873, 0.836364, 0.071537, 0.881392, 0.071922, 0.835937, 0.072682, 0.880114, 0.071537, 0.881392, 0.071922, 0.835937, 0.071537, 0.881392, 0.072682, 0.880114, 0.071922, 0.835937, 0.072682, 0.880114, 0.071922, 0.835937, 0.072953, 0.834659, 0.071922, 0.835937, 0.072682, 0.880114, 0.072953, 0.834659, 0.072682, 0.880114, 0.072953, 0.834659, 0.073793, 0.877983, 0.072953, 0.834659, 0.072682, 0.880114, 0.073793, 0.877983, 0.073793, 0.877983, 0.072953, 0.834659, 0.073966, 0.832528, 0.072953, 0.834659, 0.073793, 0.877983, 0.073966, 0.832528, 0.073793, 0.877983, 0.073966, 0.832528, 0.074868, 0.875000, 0.073966, 0.832528, 0.073793, 0.877983, 0.074868, 0.875000, 0.075905, 0.871165, 0.074868, 0.875000, 0.073966, 0.832528, 0.074868, 0.875000, 0.075905, 0.871165, 0.073966, 0.832528, 0.075905, 0.871165, 0.073966, 0.832528, 0.074962, 0.829545, 0.073966, 0.832528, 0.075905, 0.871165, 0.074962, 0.829545, 0.075905, 0.871165, 0.074962, 0.829545, 0.076900, 0.866477, 0.074962, 0.829545, 0.075905, 0.871165, 0.076900, 0.866477, 0.074962, 0.829545, 0.075940, 0.825710, 0.076900, 0.866477, 0.075940, 0.825710, 0.074962, 0.829545, 0.076900, 0.866477, 0.076900, 0.866477, 0.075940, 0.825710, 0.077855, 0.860937, 0.075940, 0.825710, 0.076900, 0.866477, 0.077855, 0.860937, 0.075940, 0.825710, 0.076900, 0.821023, 0.077855, 0.860937, 0.076900, 0.821023, 0.075940, 0.825710, 0.077855, 0.860937, 0.077855, 0.860937, 0.076900, 0.821023, 0.078769, 0.854545, 0.076900, 0.821023, 0.077855, 0.860937, 0.078769, 0.854545, 0.076900, 0.821023, 0.077843, 0.815483, 0.078769, 0.854545, 0.077843, 0.815483, 0.076900, 0.821023, 0.078769, 0.854545, 0.077843, 0.815483, 0.078769, 0.809091, 0.078769, 0.854545, 0.078769, 0.809091, 0.077843, 0.815483, 0.078769, 0.854545, 0.086805, 0.872727, 0.091129, 0.872727, 0.086805, 0.872727, 0.086805, 0.872727, 0.091129, 0.872727, 0.091129, 0.872727, 0.091129, 0.872727, 0.091129, 0.554545, 0.091129, 0.872727, 0.091129, 0.872727, 0.091129, 0.554545, 0.091129, 0.554545, 0.091129, 0.554545, 0.086805, 0.554545, 0.091129, 0.554545, 0.091129, 0.554545, 0.086805, 0.554545, 0.086805, 0.554545, 0.086805, 0.872727, 0.091129, 0.554545, 0.091129, 0.872727, 0.091129, 0.554545, 0.086805, 0.872727, 0.091129, 0.872727, 0.086805, 0.872727, 0.086805, 0.554545, 0.091129, 0.554545, 0.086805, 0.554545, 0.086805, 0.872727, 0.091129, 0.554545, 0.086805, 1, 0.091129, 1, 0.086805, 1, 0.086805, 1, 0.091129, 1, 0.091129, 1, 0.091129, 1, 0.091129, 0.936364, 0.091129, 1, 0.091129, 1, 0.091129, 0.936364, 0.091129, 0.936364, 0.091129, 0.936364, 0.086805, 0.936364, 0.091129, 0.936364, 0.091129, 0.936364, 0.086805, 0.936364, 0.086805, 0.936364, 0.086805, 1, 0.091129, 0.936364, 0.091129, 1, 0.091129, 0.936364, 0.086805, 1, 0.091129, 1, 0.086805, 1, 0.086805, 0.936364, 0.091129, 0.936364, 0.086805, 0.936364, 0.086805, 1, 0.091129, 0.936364, 0.117119, 0.854545, 0.117119, 0.809091, 0.117119, 0.854545, 0.117119, 0.854545, 0.117119, 0.809091, 0.117119, 0.809091, 0.117119, 0.809091, 0.116194, 0.815483, 0.117119, 0.809091, 0.117119, 0.809091, 0.116194, 0.815483, 0.116194, 0.815483, 0.116194, 0.815483, 0.115251, 0.821023, 0.116194, 0.815483, 0.116194, 0.815483, 0.115251, 0.821023, 0.115251, 0.821023, 0.115251, 0.821023, 0.114290, 0.825710, 0.115251, 0.821023, 0.115251, 0.821023, 0.114290, 0.825710, 0.114290, 0.825710, 0.114290, 0.825710, 0.113312, 0.829545, 0.114290, 0.825710, 0.114290, 0.825710, 0.113312, 0.829545, 0.113312, 0.829545, 0.113312, 0.829545, 0.112316, 0.832528, 0.113312, 0.829545, 0.113312, 0.829545, 0.112316, 0.832528, 0.112316, 0.832528, 0.112316, 0.832528, 0.111303, 0.834659, 0.112316, 0.832528, 0.112316, 0.832528, 0.111303, 0.834659, 0.111303, 0.834659, 0.111303, 0.834659, 0.110272, 0.835937, 0.111303, 0.834659, 0.111303, 0.834659, 0.110272, 0.835937, 0.110272, 0.835937, 0.110272, 0.835937, 0.109223, 0.836364, 0.110272, 0.835937, 0.110272, 0.835937, 0.109223, 0.836364, 0.109223, 0.836364, 0.109223, 0.836364, 0.107714, 0.835591, 0.109223, 0.836364, 0.109223, 0.836364, 0.107714, 0.835591, 0.107714, 0.835591, 0.107714, 0.835591, 0.106406, 0.833274, 0.107714, 0.835591, 0.107714, 0.835591, 0.106406, 0.833274, 0.106406, 0.833274, 0.106406, 0.833274, 0.105300, 0.829412, 0.106406, 0.833274, 0.106406, 0.833274, 0.105300, 0.829412, 0.105300, 0.829412, 0.105300, 0.829412, 0.104394, 0.824006, 0.105300, 0.829412, 0.105300, 0.829412, 0.104394, 0.824006, 0.104394, 0.824006, 0.104394, 0.824006, 0.103690, 0.817054, 0.104394, 0.824006, 0.104394, 0.824006, 0.103690, 0.817054, 0.103690, 0.817054, 0.103690, 0.817054, 0.103187, 0.808558, 0.103690, 0.817054, 0.103690, 0.817054, 0.103187, 0.808558, 0.103187, 0.808558, 0.103187, 0.808558, 0.102885, 0.798517, 0.103187, 0.808558, 0.103187, 0.808558, 0.102885, 0.798517, 0.102885, 0.798517, 0.102885, 0.798517, 0.102785, 0.786932, 0.102885, 0.798517, 0.102885, 0.798517, 0.102785, 0.786932, 0.102785, 0.786932, 0.102785, 0.786932, 0.102857, 0.778054, 0.102785, 0.786932, 0.102785, 0.786932, 0.102857, 0.778054, 0.102857, 0.778054, 0.102857, 0.778054, 0.103073, 0.770170, 0.102857, 0.778054, 0.102857, 0.778054, 0.103073, 0.770170, 0.103073, 0.770170, 0.103073, 0.770170, 0.103432, 0.763281, 0.103073, 0.770170, 0.103073, 0.770170, 0.103432, 0.763281, 0.103432, 0.763281, 0.103432, 0.763281, 0.103936, 0.757386, 0.103432, 0.763281, 0.103432, 0.763281, 0.103936, 0.757386, 0.103936, 0.757386, 0.103936, 0.757386, 0.104657, 0.752060, 0.103936, 0.757386, 0.103936, 0.757386, 0.104657, 0.752060, 0.104657, 0.752060, 0.104657, 0.752060, 0.105669, 0.746875, 0.104657, 0.752060, 0.104657, 0.752060, 0.105669, 0.746875, 0.105669, 0.746875, 0.105669, 0.746875, 0.106972, 0.741832, 0.105669, 0.746875, 0.105669, 0.746875, 0.106972, 0.741832, 0.106972, 0.741832, 0.106972, 0.741832, 0.108565, 0.736932, 0.106972, 0.741832, 0.106972, 0.741832, 0.108565, 0.736932, 0.108565, 0.736932, 0.108565, 0.736932, 0.110046, 0.732955, 0.108565, 0.736932, 0.108565, 0.736932, 0.110046, 0.732955, 0.110046, 0.732955, 0.110046, 0.732955, 0.112182, 0.726460, 0.110046, 0.732955, 0.110046, 0.732955, 0.112182, 0.726460, 0.112182, 0.726460, 0.112182, 0.726460, 0.113985, 0.718768, 0.112182, 0.726460, 0.112182, 0.726460, 0.113985, 0.718768, 0.113985, 0.718768, 0.113985, 0.718768, 0.115454, 0.709877, 0.113985, 0.718768, 0.113985, 0.718768, 0.115454, 0.709877, 0.115454, 0.709877, 0.115454, 0.709877, 0.116590, 0.699787, 0.115454, 0.709877, 0.115454, 0.709877, 0.116590, 0.699787, 0.116590, 0.699787, 0.116590, 0.699787, 0.117438, 0.688037, 0.116590, 0.699787, 0.116590, 0.699787, 0.117438, 0.688037, 0.117438, 0.688037, 0.117438, 0.688037, 0.118044, 0.674165, 0.117438, 0.688037, 0.117438, 0.688037, 0.118044, 0.674165, 0.118044, 0.674165, 0.118044, 0.674165, 0.118408, 0.658172, 0.118044, 0.674165, 0.118044, 0.674165, 0.118408, 0.658172, 0.118408, 0.658172, 0.118408, 0.658172, 0.118529, 0.640057, 0.118408, 0.658172, 0.118408, 0.658172, 0.118529, 0.640057, 0.118529, 0.640057, 0.118529, 0.640057, 0.118348, 0.619385, 0.118529, 0.640057, 0.118529, 0.640057, 0.118348, 0.619385, 0.118348, 0.619385, 0.118348, 0.619385, 0.117803, 0.600977, 0.118348, 0.619385, 0.118348, 0.619385, 0.117803, 0.600977, 0.117803, 0.600977, 0.117803, 0.600977, 0.116897, 0.584832, 0.117803, 0.600977, 0.117803, 0.600977, 0.116897, 0.584832, 0.116897, 0.584832, 0.116897, 0.584832, 0.115627, 0.570952, 0.116897, 0.584832, 0.116897, 0.584832, 0.115627, 0.570952, 0.115627, 0.570952, 0.115627, 0.570952, 0.114040, 0.559797, 0.115627, 0.570952, 0.115627, 0.570952, 0.114040, 0.559797, 0.114040, 0.559797, 0.114040, 0.559797, 0.112181, 0.551829, 0.114040, 0.559797, 0.114040, 0.559797, 0.112181, 0.551829, 0.112181, 0.551829, 0.112181, 0.551829, 0.110051, 0.547048, 0.112181, 0.551829, 0.112181, 0.551829, 0.110051, 0.547048, 0.110051, 0.547048, 0.110051, 0.547048, 0.107649, 0.545455, 0.110051, 0.547048, 0.110051, 0.547048, 0.107649, 0.545455, 0.107649, 0.545455, 0.107649, 0.545455, 0.106580, 0.545739, 0.107649, 0.545455, 0.107649, 0.545455, 0.106580, 0.545739, 0.106580, 0.545739, 0.106580, 0.545739, 0.105490, 0.546591, 0.106580, 0.545739, 0.106580, 0.545739, 0.105490, 0.546591, 0.105490, 0.546591, 0.105490, 0.546591, 0.104377, 0.548011, 0.105490, 0.546591, 0.105490, 0.546591, 0.104377, 0.548011, 0.104377, 0.548011, 0.104377, 0.548011, 0.103243, 0.550000, 0.104377, 0.548011, 0.104377, 0.548011, 0.103243, 0.550000, 0.103243, 0.550000, 0.103243, 0.550000, 0.102082, 0.552557, 0.103243, 0.550000, 0.103243, 0.550000, 0.102082, 0.552557, 0.102082, 0.552557, 0.102082, 0.552557, 0.100890, 0.555682, 0.102082, 0.552557, 0.102082, 0.552557, 0.100890, 0.555682, 0.100890, 0.555682, 0.100890, 0.555682, 0.099667, 0.559375, 0.100890, 0.555682, 0.100890, 0.555682, 0.099667, 0.559375, 0.099667, 0.559375, 0.099667, 0.559375, 0.098414, 0.563636, 0.099667, 0.559375, 0.099667, 0.559375, 0.098414, 0.563636, 0.098414, 0.563636, 0.098414, 0.563636, 0.098414, 0.618182, 0.098414, 0.563636, 0.098414, 0.563636, 0.098414, 0.618182, 0.098414, 0.618182, 0.098414, 0.618182, 0.099608, 0.611790, 0.098414, 0.618182, 0.098414, 0.618182, 0.099608, 0.611790, 0.099608, 0.611790, 0.099608, 0.611790, 0.100793, 0.606250, 0.099608, 0.611790, 0.099608, 0.611790, 0.100793, 0.606250, 0.100793, 0.606250, 0.100793, 0.606250, 0.101970, 0.601563, 0.100793, 0.606250, 0.100793, 0.606250, 0.101970, 0.601563, 0.101970, 0.601563, 0.101970, 0.601563, 0.103137, 0.597727, 0.101970, 0.601563, 0.101970, 0.601563, 0.103137, 0.597727, 0.103137, 0.597727, 0.103137, 0.597727, 0.104297, 0.594744, 0.103137, 0.597727, 0.103137, 0.597727, 0.104297, 0.594744, 0.104297, 0.594744, 0.104297, 0.594744, 0.105452, 0.592614, 0.104297, 0.594744, 0.104297, 0.594744, 0.105452, 0.592614, 0.105452, 0.592614, 0.105452, 0.592614, 0.106600, 0.591335, 0.105452, 0.592614, 0.105452, 0.592614, 0.106600, 0.591335, 0.106600, 0.591335, 0.106600, 0.591335, 0.107743, 0.590909, 0.106600, 0.591335, 0.106600, 0.591335, 0.107743, 0.590909, 0.107743, 0.590909, 0.107743, 0.590909, 0.109182, 0.591668, 0.107743, 0.590909, 0.107743, 0.590909, 0.109182, 0.591668, 0.109182, 0.591668, 0.109182, 0.591668, 0.110445, 0.593945, 0.109182, 0.591668, 0.109182, 0.591668, 0.110445, 0.593945, 0.110445, 0.593945, 0.110445, 0.593945, 0.111532, 0.597741, 0.110445, 0.593945, 0.110445, 0.593945, 0.111532, 0.597741, 0.111532, 0.597741, 0.111532, 0.597741, 0.112443, 0.603054, 0.111532, 0.597741, 0.111532, 0.597741, 0.112443, 0.603054, 0.112443, 0.603054, 0.112443, 0.603054, 0.113162, 0.609752, 0.112443, 0.603054, 0.112443, 0.603054, 0.113162, 0.609752, 0.113162, 0.609752, 0.113162, 0.609752, 0.113676, 0.617702, 0.113162, 0.609752, 0.113162, 0.609752, 0.113676, 0.617702, 0.113676, 0.617702, 0.113676, 0.617702, 0.113985, 0.626904, 0.113676, 0.617702, 0.113676, 0.617702, 0.113985, 0.626904, 0.113985, 0.626904, 0.113985, 0.626904, 0.114088, 0.637358, 0.113985, 0.626904, 0.113985, 0.626904, 0.114088, 0.637358, 0.114088, 0.637358, 0.114088, 0.637358, 0.114013, 0.646990, 0.114088, 0.637358, 0.114088, 0.637358, 0.114013, 0.646990, 0.114013, 0.646990, 0.114013, 0.646990, 0.113791, 0.655433, 0.114013, 0.646990, 0.114013, 0.646990, 0.113791, 0.655433, 0.113791, 0.655433, 0.113791, 0.655433, 0.113420, 0.662686, 0.113791, 0.655433, 0.113791, 0.655433, 0.113420, 0.662686, 0.113420, 0.662686, 0.113420, 0.662686, 0.112901, 0.668750, 0.113420, 0.662686, 0.113420, 0.662686, 0.112901, 0.668750, 0.112901, 0.668750, 0.112901, 0.668750, 0.112131, 0.674165, 0.112901, 0.668750, 0.112901, 0.668750, 0.112131, 0.674165, 0.112131, 0.674165, 0.112131, 0.674165, 0.111006, 0.679474, 0.112131, 0.674165, 0.112131, 0.674165, 0.111006, 0.679474, 0.111006, 0.679474, 0.111006, 0.679474, 0.109528, 0.684677, 0.111006, 0.679474, 0.111006, 0.679474, 0.109528, 0.684677, 0.109528, 0.684677, 0.109528, 0.684677, 0.107696, 0.689773, 0.109528, 0.684677, 0.109528, 0.684677, 0.107696, 0.689773, 0.107696, 0.689773, 0.107696, 0.689773, 0.106192, 0.694034, 0.107696, 0.689773, 0.107696, 0.689773, 0.106192, 0.694034, 0.106192, 0.694034, 0.106192, 0.694034, 0.104322, 0.699987, 0.106192, 0.694034, 0.106192, 0.694034, 0.104322, 0.699987, 0.104322, 0.699987, 0.104322, 0.699987, 0.102732, 0.707333, 0.104322, 0.699987, 0.104322, 0.699987, 0.102732, 0.707333, 0.102732, 0.707333, 0.102732, 0.707333, 0.101420, 0.716073, 0.102732, 0.707333, 0.102732, 0.707333, 0.101420, 0.716073, 0.101420, 0.716073, 0.101420, 0.716073, 0.100388, 0.726207, 0.101420, 0.716073, 0.101420, 0.716073, 0.100388, 0.726207, 0.100388, 0.726207, 0.100388, 0.726207, 0.099606, 0.738055, 0.100388, 0.726207, 0.100388, 0.726207, 0.099606, 0.738055, 0.099606, 0.738055, 0.099606, 0.738055, 0.099048, 0.751935, 0.099606, 0.738055, 0.099606, 0.738055, 0.099048, 0.751935, 0.099048, 0.751935, 0.099048, 0.751935, 0.098713, 0.767849, 0.099048, 0.751935, 0.099048, 0.751935, 0.098713, 0.767849, 0.098713, 0.767849, 0.098713, 0.767849, 0.098602, 0.785795, 0.098713, 0.767849, 0.098713, 0.767849, 0.098602, 0.785795, 0.098602, 0.785795, 0.098602, 0.785795, 0.098766, 0.807435, 0.098602, 0.785795, 0.098602, 0.785795, 0.098766, 0.807435, 0.098766, 0.807435, 0.098766, 0.807435, 0.099260, 0.826474, 0.098766, 0.807435, 0.098766, 0.807435, 0.099260, 0.826474, 0.099260, 0.826474, 0.099260, 0.826474, 0.100082, 0.842911, 0.099260, 0.826474, 0.099260, 0.826474, 0.100082, 0.842911, 0.100082, 0.842911, 0.100082, 0.842911, 0.101234, 0.856747, 0.100082, 0.842911, 0.100082, 0.842911, 0.101234, 0.856747, 0.101234, 0.856747, 0.101234, 0.856747, 0.102688, 0.867716, 0.101234, 0.856747, 0.101234, 0.856747, 0.102688, 0.867716, 0.102688, 0.867716, 0.102688, 0.867716, 0.104418, 0.875550, 0.102688, 0.867716, 0.102688, 0.867716, 0.104418, 0.875550, 0.104418, 0.875550, 0.104418, 0.875550, 0.106424, 0.880251, 0.104418, 0.875550, 0.104418, 0.875550, 0.106424, 0.880251, 0.106424, 0.880251, 0.106424, 0.880251, 0.108706, 0.881818, 0.106424, 0.880251, 0.106424, 0.880251, 0.108706, 0.881818, 0.108706, 0.881818, 0.108706, 0.881818, 0.109887, 0.881392, 0.108706, 0.881818, 0.108706, 0.881818, 0.109887, 0.881392, 0.109887, 0.881392, 0.109887, 0.881392, 0.111033, 0.880114, 0.109887, 0.881392, 0.109887, 0.881392, 0.111033, 0.880114, 0.111033, 0.880114, 0.111033, 0.880114, 0.112143, 0.877983, 0.111033, 0.880114, 0.111033, 0.880114, 0.112143, 0.877983, 0.112143, 0.877983, 0.112143, 0.877983, 0.113218, 0.875000, 0.112143, 0.877983, 0.112143, 0.877983, 0.113218, 0.875000, 0.113218, 0.875000, 0.113218, 0.875000, 0.114255, 0.871165, 0.113218, 0.875000, 0.113218, 0.875000, 0.114255, 0.871165, 0.114255, 0.871165, 0.114255, 0.871165, 0.115251, 0.866477, 0.114255, 0.871165, 0.114255, 0.871165, 0.115251, 0.866477, 0.115251, 0.866477, 0.115251, 0.866477, 0.116206, 0.860937, 0.115251, 0.866477, 0.115251, 0.866477, 0.116206, 0.860937, 0.116206, 0.860937, 0.098414, 0.618182, 0.098414, 0.563636, 0.099608, 0.611790, 0.098414, 0.563636, 0.098414, 0.618182, 0.099608, 0.611790, 0.100793, 0.606250, 0.099608, 0.611790, 0.098414, 0.563636, 0.099608, 0.611790, 0.100793, 0.606250, 0.098414, 0.563636, 0.100793, 0.606250, 0.098414, 0.563636, 0.099667, 0.559375, 0.098414, 0.563636, 0.100793, 0.606250, 0.099667, 0.559375, 0.101970, 0.601563, 0.100793, 0.606250, 0.099667, 0.559375, 0.100793, 0.606250, 0.101970, 0.601563, 0.099667, 0.559375, 0.101970, 0.601563, 0.099667, 0.559375, 0.100890, 0.555682, 0.099667, 0.559375, 0.101970, 0.601563, 0.100890, 0.555682, 0.101970, 0.601563, 0.100890, 0.555682, 0.102082, 0.552557, 0.100890, 0.555682, 0.101970, 0.601563, 0.102082, 0.552557, 0.101970, 0.601563, 0.102082, 0.552557, 0.103137, 0.597727, 0.102082, 0.552557, 0.101970, 0.601563, 0.103137, 0.597727, 0.103137, 0.597727, 0.102082, 0.552557, 0.103243, 0.550000, 0.102082, 0.552557, 0.103137, 0.597727, 0.103243, 0.550000, 0.103137, 0.597727, 0.103243, 0.550000, 0.104297, 0.594744, 0.103243, 0.550000, 0.103137, 0.597727, 0.104297, 0.594744, 0.104297, 0.594744, 0.103243, 0.550000, 0.104377, 0.548011, 0.103243, 0.550000, 0.104297, 0.594744, 0.104377, 0.548011, 0.104297, 0.594744, 0.104377, 0.548011, 0.105452, 0.592614, 0.104377, 0.548011, 0.104297, 0.594744, 0.105452, 0.592614, 0.105452, 0.592614, 0.104377, 0.548011, 0.105490, 0.546591, 0.104377, 0.548011, 0.105452, 0.592614, 0.105490, 0.546591, 0.105452, 0.592614, 0.105490, 0.546591, 0.106600, 0.591335, 0.105490, 0.546591, 0.105452, 0.592614, 0.106600, 0.591335, 0.105490, 0.546591, 0.106580, 0.545739, 0.106600, 0.591335, 0.106580, 0.545739, 0.105490, 0.546591, 0.106600, 0.591335, 0.106600, 0.591335, 0.106580, 0.545739, 0.107743, 0.590909, 0.106580, 0.545739, 0.106600, 0.591335, 0.107743, 0.590909, 0.106580, 0.545739, 0.107649, 0.545455, 0.107743, 0.590909, 0.107649, 0.545455, 0.106580, 0.545739, 0.107743, 0.590909, 0.109182, 0.591668, 0.107743, 0.590909, 0.107649, 0.545455, 0.107743, 0.590909, 0.109182, 0.591668, 0.107649, 0.545455, 0.110051, 0.547048, 0.109182, 0.591668, 0.107649, 0.545455, 0.109182, 0.591668, 0.110051, 0.547048, 0.107649, 0.545455, 0.110445, 0.593945, 0.109182, 0.591668, 0.110051, 0.547048, 0.109182, 0.591668, 0.110445, 0.593945, 0.110051, 0.547048, 0.112181, 0.551829, 0.110445, 0.593945, 0.110051, 0.547048, 0.110445, 0.593945, 0.112181, 0.551829, 0.110051, 0.547048, 0.111532, 0.597741, 0.110445, 0.593945, 0.112181, 0.551829, 0.110445, 0.593945, 0.111532, 0.597741, 0.112181, 0.551829, 0.111532, 0.597741, 0.112181, 0.551829, 0.114040, 0.559797, 0.112181, 0.551829, 0.111532, 0.597741, 0.114040, 0.559797, 0.112443, 0.603054, 0.111532, 0.597741, 0.114040, 0.559797, 0.111532, 0.597741, 0.112443, 0.603054, 0.114040, 0.559797, 0.112443, 0.603054, 0.114040, 0.559797, 0.115627, 0.570952, 0.114040, 0.559797, 0.112443, 0.603054, 0.115627, 0.570952, 0.113162, 0.609752, 0.112443, 0.603054, 0.115627, 0.570952, 0.112443, 0.603054, 0.113162, 0.609752, 0.115627, 0.570952, 0.113162, 0.609752, 0.115627, 0.570952, 0.116897, 0.584832, 0.115627, 0.570952, 0.113162, 0.609752, 0.116897, 0.584832, 0.113676, 0.617702, 0.113162, 0.609752, 0.116897, 0.584832, 0.113162, 0.609752, 0.113676, 0.617702, 0.116897, 0.584832, 0.113676, 0.617702, 0.116897, 0.584832, 0.117803, 0.600977, 0.116897, 0.584832, 0.113676, 0.617702, 0.117803, 0.600977, 0.113985, 0.626904, 0.113676, 0.617702, 0.117803, 0.600977, 0.113676, 0.617702, 0.113985, 0.626904, 0.117803, 0.600977, 0.113985, 0.626904, 0.117803, 0.600977, 0.118348, 0.619385, 0.117803, 0.600977, 0.113985, 0.626904, 0.118348, 0.619385, 0.114088, 0.637358, 0.113985, 0.626904, 0.118348, 0.619385, 0.113985, 0.626904, 0.114088, 0.637358, 0.118348, 0.619385, 0.114088, 0.637358, 0.118348, 0.619385, 0.118529, 0.640057, 0.118348, 0.619385, 0.114088, 0.637358, 0.118529, 0.640057, 0.118529, 0.640057, 0.114013, 0.646990, 0.114088, 0.637358, 0.114013, 0.646990, 0.118529, 0.640057, 0.114088, 0.637358, 0.118408, 0.658172, 0.114013, 0.646990, 0.118529, 0.640057, 0.114013, 0.646990, 0.118408, 0.658172, 0.118529, 0.640057, 0.118044, 0.674165, 0.114013, 0.646990, 0.118408, 0.658172, 0.114013, 0.646990, 0.118044, 0.674165, 0.118408, 0.658172, 0.113791, 0.655433, 0.114013, 0.646990, 0.118044, 0.674165, 0.114013, 0.646990, 0.113791, 0.655433, 0.118044, 0.674165, 0.117438, 0.688037, 0.113791, 0.655433, 0.118044, 0.674165, 0.113791, 0.655433, 0.117438, 0.688037, 0.118044, 0.674165, 0.113420, 0.662686, 0.113791, 0.655433, 0.117438, 0.688037, 0.113791, 0.655433, 0.113420, 0.662686, 0.117438, 0.688037, 0.116590, 0.699787, 0.113420, 0.662686, 0.117438, 0.688037, 0.113420, 0.662686, 0.116590, 0.699787, 0.117438, 0.688037, 0.115454, 0.709877, 0.113420, 0.662686, 0.116590, 0.699787, 0.113420, 0.662686, 0.115454, 0.709877, 0.116590, 0.699787, 0.112901, 0.668750, 0.113420, 0.662686, 0.115454, 0.709877, 0.113420, 0.662686, 0.112901, 0.668750, 0.115454, 0.709877, 0.113985, 0.718768, 0.112901, 0.668750, 0.115454, 0.709877, 0.112901, 0.668750, 0.113985, 0.718768, 0.115454, 0.709877, 0.112131, 0.674165, 0.112901, 0.668750, 0.113985, 0.718768, 0.112901, 0.668750, 0.112131, 0.674165, 0.113985, 0.718768, 0.112182, 0.726460, 0.112131, 0.674165, 0.113985, 0.718768, 0.112131, 0.674165, 0.112182, 0.726460, 0.113985, 0.718768, 0.111006, 0.679474, 0.112131, 0.674165, 0.112182, 0.726460, 0.112131, 0.674165, 0.111006, 0.679474, 0.112182, 0.726460, 0.112182, 0.726460, 0.109528, 0.684677, 0.111006, 0.679474, 0.109528, 0.684677, 0.112182, 0.726460, 0.111006, 0.679474, 0.112182, 0.726460, 0.110046, 0.732955, 0.109528, 0.684677, 0.110046, 0.732955, 0.112182, 0.726460, 0.109528, 0.684677, 0.107696, 0.689773, 0.109528, 0.684677, 0.110046, 0.732955, 0.109528, 0.684677, 0.107696, 0.689773, 0.110046, 0.732955, 0.108565, 0.736932, 0.107696, 0.689773, 0.110046, 0.732955, 0.107696, 0.689773, 0.108565, 0.736932, 0.110046, 0.732955, 0.106192, 0.694034, 0.107696, 0.689773, 0.108565, 0.736932, 0.107696, 0.689773, 0.106192, 0.694034, 0.108565, 0.736932, 0.106972, 0.741832, 0.106192, 0.694034, 0.108565, 0.736932, 0.106192, 0.694034, 0.106972, 0.741832, 0.108565, 0.736932, 0.104322, 0.699987, 0.106192, 0.694034, 0.106972, 0.741832, 0.106192, 0.694034, 0.104322, 0.699987, 0.106972, 0.741832, 0.105669, 0.746875, 0.104322, 0.699987, 0.106972, 0.741832, 0.104322, 0.699987, 0.105669, 0.746875, 0.106972, 0.741832, 0.105669, 0.746875, 0.104657, 0.752060, 0.104322, 0.699987, 0.104657, 0.752060, 0.105669, 0.746875, 0.104322, 0.699987, 0.104657, 0.752060, 0.102732, 0.707333, 0.104322, 0.699987, 0.102732, 0.707333, 0.104657, 0.752060, 0.104322, 0.699987, 0.104657, 0.752060, 0.103936, 0.757386, 0.102732, 0.707333, 0.103936, 0.757386, 0.104657, 0.752060, 0.102732, 0.707333, 0.103936, 0.757386, 0.101420, 0.716073, 0.102732, 0.707333, 0.101420, 0.716073, 0.103936, 0.757386, 0.102732, 0.707333, 0.100388, 0.726207, 0.101420, 0.716073, 0.103936, 0.757386, 0.101420, 0.716073, 0.100388, 0.726207, 0.103936, 0.757386, 0.103936, 0.757386, 0.103432, 0.763281, 0.100388, 0.726207, 0.103432, 0.763281, 0.103936, 0.757386, 0.100388, 0.726207, 0.103432, 0.763281, 0.099606, 0.738055, 0.100388, 0.726207, 0.099606, 0.738055, 0.103432, 0.763281, 0.100388, 0.726207, 0.103432, 0.763281, 0.103073, 0.770170, 0.099606, 0.738055, 0.103073, 0.770170, 0.103432, 0.763281, 0.099606, 0.738055, 0.103073, 0.770170, 0.099048, 0.751935, 0.099606, 0.738055, 0.099048, 0.751935, 0.103073, 0.770170, 0.099606, 0.738055, 0.103073, 0.770170, 0.102857, 0.778054, 0.099048, 0.751935, 0.102857, 0.778054, 0.103073, 0.770170, 0.099048, 0.751935, 0.102857, 0.778054, 0.098713, 0.767849, 0.099048, 0.751935, 0.098713, 0.767849, 0.102857, 0.778054, 0.099048, 0.751935, 0.098602, 0.785795, 0.098713, 0.767849, 0.102857, 0.778054, 0.098713, 0.767849, 0.098602, 0.785795, 0.102857, 0.778054, 0.102785, 0.786932, 0.098602, 0.785795, 0.102857, 0.778054, 0.098602, 0.785795, 0.102785, 0.786932, 0.102857, 0.778054, 0.098766, 0.807435, 0.098602, 0.785795, 0.102785, 0.786932, 0.098602, 0.785795, 0.098766, 0.807435, 0.102785, 0.786932, 0.098766, 0.807435, 0.102785, 0.786932, 0.102885, 0.798517, 0.102785, 0.786932, 0.098766, 0.807435, 0.102885, 0.798517, 0.099260, 0.826474, 0.098766, 0.807435, 0.102885, 0.798517, 0.098766, 0.807435, 0.099260, 0.826474, 0.102885, 0.798517, 0.099260, 0.826474, 0.102885, 0.798517, 0.103187, 0.808558, 0.102885, 0.798517, 0.099260, 0.826474, 0.103187, 0.808558, 0.100082, 0.842911, 0.099260, 0.826474, 0.103187, 0.808558, 0.099260, 0.826474, 0.100082, 0.842911, 0.103187, 0.808558, 0.100082, 0.842911, 0.103187, 0.808558, 0.103690, 0.817054, 0.103187, 0.808558, 0.100082, 0.842911, 0.103690, 0.817054, 0.101234, 0.856747, 0.100082, 0.842911, 0.103690, 0.817054, 0.100082, 0.842911, 0.101234, 0.856747, 0.103690, 0.817054, 0.101234, 0.856747, 0.103690, 0.817054, 0.104394, 0.824006, 0.103690, 0.817054, 0.101234, 0.856747, 0.104394, 0.824006, 0.102688, 0.867716, 0.101234, 0.856747, 0.104394, 0.824006, 0.101234, 0.856747, 0.102688, 0.867716, 0.104394, 0.824006, 0.102688, 0.867716, 0.104394, 0.824006, 0.105300, 0.829412, 0.104394, 0.824006, 0.102688, 0.867716, 0.105300, 0.829412, 0.104418, 0.875550, 0.102688, 0.867716, 0.105300, 0.829412, 0.102688, 0.867716, 0.104418, 0.875550, 0.105300, 0.829412, 0.104418, 0.875550, 0.105300, 0.829412, 0.106406, 0.833274, 0.105300, 0.829412, 0.104418, 0.875550, 0.106406, 0.833274, 0.104418, 0.875550, 0.106406, 0.833274, 0.106424, 0.880251, 0.106406, 0.833274, 0.104418, 0.875550, 0.106424, 0.880251, 0.106424, 0.880251, 0.106406, 0.833274, 0.107714, 0.835591, 0.106406, 0.833274, 0.106424, 0.880251, 0.107714, 0.835591, 0.108706, 0.881818, 0.106424, 0.880251, 0.107714, 0.835591, 0.106424, 0.880251, 0.108706, 0.881818, 0.107714, 0.835591, 0.108706, 0.881818, 0.107714, 0.835591, 0.109223, 0.836364, 0.107714, 0.835591, 0.108706, 0.881818, 0.109223, 0.836364, 0.109223, 0.836364, 0.109887, 0.881392, 0.108706, 0.881818, 0.109887, 0.881392, 0.109223, 0.836364, 0.108706, 0.881818, 0.109887, 0.881392, 0.109223, 0.836364, 0.110272, 0.835937, 0.109223, 0.836364, 0.109887, 0.881392, 0.110272, 0.835937, 0.111033, 0.880114, 0.109887, 0.881392, 0.110272, 0.835937, 0.109887, 0.881392, 0.111033, 0.880114, 0.110272, 0.835937, 0.111033, 0.880114, 0.110272, 0.835937, 0.111303, 0.834659, 0.110272, 0.835937, 0.111033, 0.880114, 0.111303, 0.834659, 0.111033, 0.880114, 0.111303, 0.834659, 0.112143, 0.877983, 0.111303, 0.834659, 0.111033, 0.880114, 0.112143, 0.877983, 0.112143, 0.877983, 0.111303, 0.834659, 0.112316, 0.832528, 0.111303, 0.834659, 0.112143, 0.877983, 0.112316, 0.832528, 0.112143, 0.877983, 0.112316, 0.832528, 0.113218, 0.875000, 0.112316, 0.832528, 0.112143, 0.877983, 0.113218, 0.875000, 0.114255, 0.871165, 0.113218, 0.875000, 0.112316, 0.832528, 0.113218, 0.875000, 0.114255, 0.871165, 0.112316, 0.832528, 0.114255, 0.871165, 0.112316, 0.832528, 0.113312, 0.829545, 0.112316, 0.832528, 0.114255, 0.871165, 0.113312, 0.829545, 0.114255, 0.871165, 0.113312, 0.829545, 0.115251, 0.866477, 0.113312, 0.829545, 0.114255, 0.871165, 0.115251, 0.866477, 0.113312, 0.829545, 0.114290, 0.825710, 0.115251, 0.866477, 0.114290, 0.825710, 0.113312, 0.829545, 0.115251, 0.866477, 0.115251, 0.866477, 0.114290, 0.825710, 0.116206, 0.860937, 0.114290, 0.825710, 0.115251, 0.866477, 0.116206, 0.860937, 0.114290, 0.825710, 0.115251, 0.821023, 0.116206, 0.860937, 0.115251, 0.821023, 0.114290, 0.825710, 0.116206, 0.860937, 0.116206, 0.860937, 0.115251, 0.821023, 0.117119, 0.854545, 0.115251, 0.821023, 0.116206, 0.860937, 0.117119, 0.854545, 0.115251, 0.821023, 0.116194, 0.815483, 0.117119, 0.854545, 0.116194, 0.815483, 0.115251, 0.821023, 0.117119, 0.854545, 0.116194, 0.815483, 0.117119, 0.809091, 0.117119, 0.854545, 0.117119, 0.809091, 0.116194, 0.815483, 0.117119, 0.854545, 0.129432, 0.963636, 0.129432, 0.872727, 0.129432, 0.963636, 0.129432, 0.963636, 0.129432, 0.872727, 0.129432, 0.872727, 0.129432, 0.872727, 0.138339, 0.872727, 0.129432, 0.872727, 0.129432, 0.872727, 0.138339, 0.872727, 0.138339, 0.872727, 0.138339, 0.872727, 0.138339, 0.827273, 0.138339, 0.872727, 0.138339, 0.872727, 0.138339, 0.827273, 0.138339, 0.827273, 0.138339, 0.827273, 0.129432, 0.827273, 0.138339, 0.827273, 0.138339, 0.827273, 0.129432, 0.827273, 0.129432, 0.827273, 0.129432, 0.827273, 0.129432, 0.659375, 0.129432, 0.827273, 0.129432, 0.827273, 0.129432, 0.659375, 0.129432, 0.659375, 0.129432, 0.659375, 0.129488, 0.642170, 0.129432, 0.659375, 0.129432, 0.659375, 0.129488, 0.642170, 0.129488, 0.642170, 0.129488, 0.642170, 0.129653, 0.628338, 0.129488, 0.642170, 0.129488, 0.642170, 0.129653, 0.628338, 0.129653, 0.628338, 0.129653, 0.628338, 0.129928, 0.617880, 0.129653, 0.628338, 0.129653, 0.628338, 0.129928, 0.617880, 0.129928, 0.617880, 0.129928, 0.617880, 0.130314, 0.610795, 0.129928, 0.617880, 0.129928, 0.617880, 0.130314, 0.610795, 0.130314, 0.610795, 0.130314, 0.610795, 0.130868, 0.606072, 0.130314, 0.610795, 0.130314, 0.610795, 0.130868, 0.606072, 0.130868, 0.606072, 0.130868, 0.606072, 0.131650, 0.602699, 0.130868, 0.606072, 0.130868, 0.606072, 0.131650, 0.602699, 0.131650, 0.602699, 0.131650, 0.602699, 0.132660, 0.600675, 0.131650, 0.602699, 0.131650, 0.602699, 0.132660, 0.600675, 0.132660, 0.600675, 0.132660, 0.600675, 0.133897, 0.600000, 0.132660, 0.600675, 0.132660, 0.600675, 0.133897, 0.600000, 0.133897, 0.600000, 0.133897, 0.600000, 0.138339, 0.600000, 0.133897, 0.600000, 0.133897, 0.600000, 0.138339, 0.600000, 0.138339, 0.600000, 0.138339, 0.600000, 0.138339, 0.554545, 0.138339, 0.600000, 0.138339, 0.600000, 0.138339, 0.554545, 0.138339, 0.554545, 0.138339, 0.554545, 0.133897, 0.554545, 0.138339, 0.554545, 0.138339, 0.554545, 0.133897, 0.554545, 0.133897, 0.554545, 0.133897, 0.554545, 0.131589, 0.555957, 0.133897, 0.554545, 0.133897, 0.554545, 0.131589, 0.555957, 0.131589, 0.555957, 0.131589, 0.555957, 0.129667, 0.560192, 0.131589, 0.555957, 0.131589, 0.555957, 0.129667, 0.560192, 0.129667, 0.560192, 0.129667, 0.560192, 0.128134, 0.567250, 0.129667, 0.560192, 0.129667, 0.560192, 0.128134, 0.567250, 0.128134, 0.567250, 0.128134, 0.567250, 0.126989, 0.577131, 0.128134, 0.567250, 0.128134, 0.567250, 0.126989, 0.577131, 0.126989, 0.577131, 0.126989, 0.577131, 0.126156, 0.590740, 0.126989, 0.577131, 0.126989, 0.577131, 0.126156, 0.590740, 0.126156, 0.590740, 0.126156, 0.590740, 0.125561, 0.608984, 0.126156, 0.590740, 0.126156, 0.590740, 0.125561, 0.608984, 0.125561, 0.608984, 0.125561, 0.608984, 0.125204, 0.631863, 0.125561, 0.608984, 0.125561, 0.608984, 0.125204, 0.631863, 0.125204, 0.631863, 0.125204, 0.631863, 0.125085, 0.659375, 0.125204, 0.631863, 0.125204, 0.631863, 0.125085, 0.659375, 0.125085, 0.659375, 0.125085, 0.659375, 0.125085, 0.827273, 0.125085, 0.659375, 0.125085, 0.659375, 0.125085, 0.827273, 0.125085, 0.827273, 0.125085, 0.827273, 0.121913, 0.827273, 0.125085, 0.827273, 0.125085, 0.827273, 0.121913, 0.827273, 0.121913, 0.827273, 0.121913, 0.827273, 0.121913, 0.872727, 0.121913, 0.827273, 0.121913, 0.827273, 0.121913, 0.872727, 0.121913, 0.872727, 0.121913, 0.872727, 0.125085, 0.872727, 0.121913, 0.872727, 0.121913, 0.872727, 0.125085, 0.872727, 0.125085, 0.872727, 0.125085, 0.872727, 0.125085, 0.963636, 0.125085, 0.872727, 0.125085, 0.872727, 0.125085, 0.963636, 0.125085, 0.963636, 0.121913, 0.872727, 0.125085, 0.827273, 0.125085, 0.872727, 0.125085, 0.827273, 0.121913, 0.872727, 0.125085, 0.872727, 0.121913, 0.872727, 0.121913, 0.827273, 0.125085, 0.827273, 0.121913, 0.827273, 0.121913, 0.872727, 0.125085, 0.827273, 0.125085, 0.872727, 0.125085, 0.827273, 0.129432, 0.827273, 0.125085, 0.827273, 0.125085, 0.872727, 0.129432, 0.827273, 0.125085, 0.872727, 0.129432, 0.827273, 0.129432, 0.872727, 0.129432, 0.827273, 0.125085, 0.872727, 0.129432, 0.872727, 0.125085, 0.963636, 0.125085, 0.872727, 0.129432, 0.872727, 0.125085, 0.872727, 0.125085, 0.963636, 0.129432, 0.872727, 0.125085, 0.963636, 0.129432, 0.872727, 0.129432, 0.963636, 0.129432, 0.872727, 0.125085, 0.963636, 0.129432, 0.963636, 0.129432, 0.872727, 0.129432, 0.827273, 0.138339, 0.827273, 0.129432, 0.827273, 0.129432, 0.872727, 0.138339, 0.827273, 0.129432, 0.872727, 0.138339, 0.827273, 0.138339, 0.872727, 0.138339, 0.827273, 0.129432, 0.872727, 0.138339, 0.872727, 0.125085, 0.827273, 0.129432, 0.659375, 0.129432, 0.827273, 0.129432, 0.659375, 0.125085, 0.827273, 0.129432, 0.827273, 0.125085, 0.827273, 0.125085, 0.659375, 0.129432, 0.659375, 0.125085, 0.659375, 0.125085, 0.827273, 0.129432, 0.659375, 0.129432, 0.659375, 0.125085, 0.659375, 0.129488, 0.642170, 0.125085, 0.659375, 0.129432, 0.659375, 0.129488, 0.642170, 0.125085, 0.659375, 0.125204, 0.631863, 0.129488, 0.642170, 0.125204, 0.631863, 0.125085, 0.659375, 0.129488, 0.642170, 0.129488, 0.642170, 0.125204, 0.631863, 0.129653, 0.628338, 0.125204, 0.631863, 0.129488, 0.642170, 0.129653, 0.628338, 0.125204, 0.631863, 0.125561, 0.608984, 0.129653, 0.628338, 0.125561, 0.608984, 0.125204, 0.631863, 0.129653, 0.628338, 0.129653, 0.628338, 0.125561, 0.608984, 0.129928, 0.617880, 0.125561, 0.608984, 0.129653, 0.628338, 0.129928, 0.617880, 0.129928, 0.617880, 0.125561, 0.608984, 0.126156, 0.590740, 0.125561, 0.608984, 0.129928, 0.617880, 0.126156, 0.590740, 0.130314, 0.610795, 0.129928, 0.617880, 0.126156, 0.590740, 0.129928, 0.617880, 0.130314, 0.610795, 0.126156, 0.590740, 0.126156, 0.590740, 0.126989, 0.577131, 0.130314, 0.610795, 0.126989, 0.577131, 0.126156, 0.590740, 0.130314, 0.610795, 0.130314, 0.610795, 0.126989, 0.577131, 0.128134, 0.567250, 0.126989, 0.577131, 0.130314, 0.610795, 0.128134, 0.567250, 0.130868, 0.606072, 0.130314, 0.610795, 0.128134, 0.567250, 0.130314, 0.610795, 0.130868, 0.606072, 0.128134, 0.567250, 0.128134, 0.567250, 0.129667, 0.560192, 0.130868, 0.606072, 0.129667, 0.560192, 0.128134, 0.567250, 0.130868, 0.606072, 0.130868, 0.606072, 0.129667, 0.560192, 0.131650, 0.602699, 0.129667, 0.560192, 0.130868, 0.606072, 0.131650, 0.602699, 0.129667, 0.560192, 0.131589, 0.555957, 0.131650, 0.602699, 0.131589, 0.555957, 0.129667, 0.560192, 0.131650, 0.602699, 0.131650, 0.602699, 0.131589, 0.555957, 0.132660, 0.600675, 0.131589, 0.555957, 0.131650, 0.602699, 0.132660, 0.600675, 0.132660, 0.600675, 0.131589, 0.555957, 0.133897, 0.554545, 0.131589, 0.555957, 0.132660, 0.600675, 0.133897, 0.554545, 0.133897, 0.600000, 0.132660, 0.600675, 0.133897, 0.554545, 0.132660, 0.600675, 0.133897, 0.600000, 0.133897, 0.554545, 0.133897, 0.600000, 0.133897, 0.554545, 0.138339, 0.554545, 0.133897, 0.554545, 0.133897, 0.600000, 0.138339, 0.554545, 0.133897, 0.600000, 0.138339, 0.554545, 0.138339, 0.600000, 0.138339, 0.554545, 0.133897, 0.600000, 0.138339, 0.600000, 0.155916, 0.709091, 0.153497, 0.708230, 0.155916, 0.709091, 0.155916, 0.709091, 0.153497, 0.708230, 0.153497, 0.708230, 0.153497, 0.708230, 0.151480, 0.705646, 0.153497, 0.708230, 0.153497, 0.708230, 0.151480, 0.705646, 0.151480, 0.705646, 0.151480, 0.705646, 0.149866, 0.701341, 0.151480, 0.705646, 0.151480, 0.705646, 0.149866, 0.701341, 0.149866, 0.701341, 0.149866, 0.701341, 0.148655, 0.695313, 0.149866, 0.701341, 0.149866, 0.701341, 0.148655, 0.695313, 0.148655, 0.695313, 0.148655, 0.695313, 0.147771, 0.687216, 0.148655, 0.695313, 0.148655, 0.695313, 0.147771, 0.687216, 0.147771, 0.687216, 0.147771, 0.687216, 0.147139, 0.676705, 0.147771, 0.687216, 0.147771, 0.687216, 0.147139, 0.676705, 0.147139, 0.676705, 0.147139, 0.676705, 0.146760, 0.663778, 0.147139, 0.676705, 0.147139, 0.676705, 0.146760, 0.663778, 0.146760, 0.663778, 0.146760, 0.663778, 0.146634, 0.648437, 0.146760, 0.663778, 0.146760, 0.663778, 0.146634, 0.648437, 0.146634, 0.648437, 0.146634, 0.648437, 0.146729, 0.635849, 0.146634, 0.648437, 0.146634, 0.648437, 0.146729, 0.635849, 0.146729, 0.635849, 0.146729, 0.635849, 0.147013, 0.624645, 0.146729, 0.635849, 0.146729, 0.635849, 0.147013, 0.624645, 0.147013, 0.624645, 0.147013, 0.624645, 0.147486, 0.614826, 0.147013, 0.624645, 0.147013, 0.624645, 0.147486, 0.614826, 0.147486, 0.614826, 0.147486, 0.614826, 0.148149, 0.606392, 0.147486, 0.614826, 0.147486, 0.614826, 0.148149, 0.606392, 0.148149, 0.606392, 0.148149, 0.606392, 0.148976, 0.599618, 0.148149, 0.606392, 0.148149, 0.606392, 0.148976, 0.599618, 0.148976, 0.599618, 0.148976, 0.599618, 0.149938, 0.594780, 0.148976, 0.599618, 0.148976, 0.599618, 0.149938, 0.594780, 0.149938, 0.594780, 0.149938, 0.594780, 0.151038, 0.591877, 0.149938, 0.594780, 0.149938, 0.594780, 0.151038, 0.591877, 0.151038, 0.591877, 0.151038, 0.591877, 0.152274, 0.590909, 0.151038, 0.591877, 0.151038, 0.591877, 0.152274, 0.590909, 0.152274, 0.590909, 0.152274, 0.590909, 0.153982, 0.592733, 0.152274, 0.590909, 0.152274, 0.590909, 0.153982, 0.592733, 0.153982, 0.592733, 0.153982, 0.592733, 0.155513, 0.598207, 0.153982, 0.592733, 0.153982, 0.592733, 0.155513, 0.598207, 0.155513, 0.598207, 0.155513, 0.598207, 0.156867, 0.607329, 0.155513, 0.598207, 0.155513, 0.598207, 0.156867, 0.607329, 0.156867, 0.607329, 0.156867, 0.607329, 0.158043, 0.620099, 0.156867, 0.607329, 0.156867, 0.607329, 0.158043, 0.620099, 0.158043, 0.620099, 0.158043, 0.620099, 0.158994, 0.635898, 0.158043, 0.620099, 0.158043, 0.620099, 0.158994, 0.635898, 0.158994, 0.635898, 0.158994, 0.635898, 0.159673, 0.654102, 0.158994, 0.635898, 0.158994, 0.635898, 0.159673, 0.654102, 0.159673, 0.654102, 0.159673, 0.654102, 0.160080, 0.674711, 0.159673, 0.654102, 0.159673, 0.654102, 0.160080, 0.674711, 0.160080, 0.674711, 0.160080, 0.674711, 0.160216, 0.697727, 0.160080, 0.674711, 0.160080, 0.674711, 0.160216, 0.697727, 0.160216, 0.697727, 0.160216, 0.697727, 0.160216, 0.709091, 0.160216, 0.697727, 0.160216, 0.697727, 0.160216, 0.709091, 0.160216, 0.709091, 0.164540, 0.733097, 0.164540, 0.554545, 0.164540, 0.733097, 0.164540, 0.733097, 0.164540, 0.554545, 0.164540, 0.554545, 0.164540, 0.554545, 0.160216, 0.554545, 0.164540, 0.554545, 0.164540, 0.554545, 0.160216, 0.554545, 0.160216, 0.554545, 0.160216, 0.554545, 0.160216, 0.600000, 0.160216, 0.554545, 0.160216, 0.554545, 0.160216, 0.600000, 0.160216, 0.600000, 0.160216, 0.600000, 0.159430, 0.586927, 0.160216, 0.600000, 0.160216, 0.600000, 0.159430, 0.586927, 0.159430, 0.586927, 0.159430, 0.586927, 0.158554, 0.575692, 0.159430, 0.586927, 0.159430, 0.586927, 0.158554, 0.575692, 0.158554, 0.575692, 0.158554, 0.575692, 0.157586, 0.566295, 0.158554, 0.575692, 0.158554, 0.575692, 0.157586, 0.566295, 0.157586, 0.566295, 0.157586, 0.566295, 0.156527, 0.558736, 0.157586, 0.566295, 0.157586, 0.566295, 0.156527, 0.558736, 0.156527, 0.558736, 0.156527, 0.558736, 0.155361, 0.552925, 0.156527, 0.558736, 0.156527, 0.558736, 0.155361, 0.552925, 0.155361, 0.552925, 0.155361, 0.552925, 0.154071, 0.548775, 0.155361, 0.552925, 0.155361, 0.552925, 0.154071, 0.548775, 0.154071, 0.548775, 0.154071, 0.548775, 0.152658, 0.546285, 0.154071, 0.548775, 0.154071, 0.548775, 0.152658, 0.546285, 0.152658, 0.546285, 0.152658, 0.546285, 0.151122, 0.545455, 0.152658, 0.546285, 0.152658, 0.546285, 0.151122, 0.545455, 0.151122, 0.545455, 0.151122, 0.545455, 0.149205, 0.547168, 0.151122, 0.545455, 0.151122, 0.545455, 0.149205, 0.547168, 0.149205, 0.547168, 0.149205, 0.547168, 0.147494, 0.552308, 0.149205, 0.547168, 0.149205, 0.547168, 0.147494, 0.552308, 0.147494, 0.552308, 0.147494, 0.552308, 0.145991, 0.560875, 0.147494, 0.552308, 0.147494, 0.552308, 0.145991, 0.560875, 0.145991, 0.560875, 0.145991, 0.560875, 0.144695, 0.572869, 0.145991, 0.560875, 0.145991, 0.560875, 0.144695, 0.572869, 0.144695, 0.572869, 0.144695, 0.572869, 0.143652, 0.587740, 0.144695, 0.572869, 0.144695, 0.572869, 0.143652, 0.587740, 0.143652, 0.587740, 0.143652, 0.587740, 0.142906, 0.604936, 0.143652, 0.587740, 0.143652, 0.587740, 0.142906, 0.604936, 0.142906, 0.604936, 0.142906, 0.604936, 0.142459, 0.624458, 0.142906, 0.604936, 0.142906, 0.604936, 0.142459, 0.624458, 0.142459, 0.624458, 0.142459, 0.624458, 0.142310, 0.646307, 0.142459, 0.624458, 0.142459, 0.624458, 0.142310, 0.646307, 0.142310, 0.646307, 0.142310, 0.646307, 0.142496, 0.671502, 0.142310, 0.646307, 0.142310, 0.646307, 0.142496, 0.671502, 0.142496, 0.671502, 0.142496, 0.671502, 0.143053, 0.693395, 0.142496, 0.671502, 0.142496, 0.671502, 0.143053, 0.693395, 0.143053, 0.693395, 0.143053, 0.693395, 0.143982, 0.711985, 0.143053, 0.693395, 0.143053, 0.693395, 0.143982, 0.711985, 0.143982, 0.711985, 0.143982, 0.711985, 0.145283, 0.727273, 0.143982, 0.711985, 0.143982, 0.711985, 0.145283, 0.727273, 0.145283, 0.727273, 0.145283, 0.727273, 0.146952, 0.739205, 0.145283, 0.727273, 0.145283, 0.727273, 0.146952, 0.739205, 0.146952, 0.739205, 0.146952, 0.739205, 0.148987, 0.747727, 0.146952, 0.739205, 0.146952, 0.739205, 0.148987, 0.747727, 0.148987, 0.747727, 0.148987, 0.747727, 0.151387, 0.752841, 0.148987, 0.747727, 0.148987, 0.747727, 0.151387, 0.752841, 0.151387, 0.752841, 0.151387, 0.752841, 0.154153, 0.754545, 0.151387, 0.752841, 0.151387, 0.752841, 0.154153, 0.754545, 0.154153, 0.754545, 0.154153, 0.754545, 0.160216, 0.754545, 0.154153, 0.754545, 0.154153, 0.754545, 0.160216, 0.754545, 0.160216, 0.754545, 0.160216, 0.754545, 0.160216, 0.759659, 0.160216, 0.754545, 0.160216, 0.754545, 0.160216, 0.759659, 0.160216, 0.759659, 0.160216, 0.759659, 0.160094, 0.776944, 0.160216, 0.759659, 0.160216, 0.759659, 0.160094, 0.776944, 0.160094, 0.776944, 0.160094, 0.776944, 0.159726, 0.792152, 0.160094, 0.776944, 0.160094, 0.776944, 0.159726, 0.792152, 0.159726, 0.792152, 0.159726, 0.792152, 0.159112, 0.805282, 0.159726, 0.792152, 0.159726, 0.792152, 0.159112, 0.805282, 0.159112, 0.805282, 0.159112, 0.805282, 0.158254, 0.816335, 0.159112, 0.805282, 0.159112, 0.805282, 0.158254, 0.816335, 0.158254, 0.816335, 0.158254, 0.816335, 0.157174, 0.825098, 0.158254, 0.816335, 0.158254, 0.816335, 0.157174, 0.825098, 0.157174, 0.825098, 0.157174, 0.825098, 0.155895, 0.831357, 0.157174, 0.825098, 0.157174, 0.825098, 0.155895, 0.831357, 0.155895, 0.831357, 0.155895, 0.831357, 0.154419, 0.835112, 0.155895, 0.831357, 0.155895, 0.831357, 0.154419, 0.835112, 0.154419, 0.835112, 0.154419, 0.835112, 0.152744, 0.836364, 0.154419, 0.835112, 0.154419, 0.835112, 0.152744, 0.836364, 0.152744, 0.836364, 0.152744, 0.836364, 0.151623, 0.835937, 0.152744, 0.836364, 0.152744, 0.836364, 0.151623, 0.835937, 0.151623, 0.835937, 0.151623, 0.835937, 0.150517, 0.834659, 0.151623, 0.835937, 0.151623, 0.835937, 0.150517, 0.834659, 0.150517, 0.834659, 0.150517, 0.834659, 0.149426, 0.832528, 0.150517, 0.834659, 0.150517, 0.834659, 0.149426, 0.832528, 0.149426, 0.832528, 0.149426, 0.832528, 0.148349, 0.829545, 0.149426, 0.832528, 0.149426, 0.832528, 0.148349, 0.829545, 0.148349, 0.829545, 0.148349, 0.829545, 0.147290, 0.825710, 0.148349, 0.829545, 0.148349, 0.829545, 0.147290, 0.825710, 0.147290, 0.825710, 0.147290, 0.825710, 0.146252, 0.821023, 0.147290, 0.825710, 0.147290, 0.825710, 0.146252, 0.821023, 0.146252, 0.821023, 0.146252, 0.821023, 0.145234, 0.815483, 0.146252, 0.821023, 0.146252, 0.821023, 0.145234, 0.815483, 0.145234, 0.815483, 0.145234, 0.815483, 0.144237, 0.809091, 0.145234, 0.815483, 0.145234, 0.815483, 0.144237, 0.809091, 0.144237, 0.809091, 0.144237, 0.809091, 0.144237, 0.854545, 0.144237, 0.809091, 0.144237, 0.809091, 0.144237, 0.854545, 0.144237, 0.854545, 0.144237, 0.854545, 0.145415, 0.860937, 0.144237, 0.854545, 0.144237, 0.854545, 0.145415, 0.860937, 0.145415, 0.860937, 0.145415, 0.860937, 0.146575, 0.866477, 0.145415, 0.860937, 0.145415, 0.860937, 0.146575, 0.866477, 0.146575, 0.866477, 0.146575, 0.866477, 0.147718, 0.871165, 0.146575, 0.866477, 0.146575, 0.866477, 0.147718, 0.871165, 0.147718, 0.871165, 0.147718, 0.871165, 0.148843, 0.875000, 0.147718, 0.871165, 0.147718, 0.871165, 0.148843, 0.875000, 0.148843, 0.875000, 0.148843, 0.875000, 0.149952, 0.877983, 0.148843, 0.875000, 0.148843, 0.875000, 0.149952, 0.877983, 0.149952, 0.877983, 0.149952, 0.877983, 0.151046, 0.880114, 0.149952, 0.877983, 0.149952, 0.877983, 0.151046, 0.880114, 0.151046, 0.880114, 0.151046, 0.880114, 0.152125, 0.881392, 0.151046, 0.880114, 0.151046, 0.880114, 0.152125, 0.881392, 0.152125, 0.881392, 0.152125, 0.881392, 0.153190, 0.881818, 0.152125, 0.881392, 0.152125, 0.881392, 0.153190, 0.881818, 0.153190, 0.881818, 0.153190, 0.881818, 0.155864, 0.879514, 0.153190, 0.881818, 0.153190, 0.881818, 0.155864, 0.879514, 0.155864, 0.879514, 0.155864, 0.879514, 0.158178, 0.872603, 0.155864, 0.879514, 0.155864, 0.879514, 0.158178, 0.872603, 0.158178, 0.872603, 0.158178, 0.872603, 0.160130, 0.861084, 0.158178, 0.872603, 0.158178, 0.872603, 0.160130, 0.861084, 0.160130, 0.861084, 0.160130, 0.861084, 0.161720, 0.844957, 0.160130, 0.861084, 0.160130, 0.861084, 0.161720, 0.844957, 0.161720, 0.844957, 0.161720, 0.844957, 0.162954, 0.824143, 0.161720, 0.844957, 0.161720, 0.844957, 0.162954, 0.824143, 0.162954, 0.824143, 0.162954, 0.824143, 0.163835, 0.798562, 0.162954, 0.824143, 0.162954, 0.824143, 0.163835, 0.798562, 0.163835, 0.798562, 0.163835, 0.798562, 0.164364, 0.768213, 0.163835, 0.798562, 0.163835, 0.798562, 0.164364, 0.768213, 0.164364, 0.768213, 0.142496, 0.671502, 0.142310, 0.646307, 0.146634, 0.648437, 0.142310, 0.646307, 0.142496, 0.671502, 0.146634, 0.648437, 0.142496, 0.671502, 0.146634, 0.648437, 0.146760, 0.663778, 0.146634, 0.648437, 0.142496, 0.671502, 0.146760, 0.663778, 0.143053, 0.693395, 0.142496, 0.671502, 0.146760, 0.663778, 0.142496, 0.671502, 0.143053, 0.693395, 0.146760, 0.663778, 0.143053, 0.693395, 0.146760, 0.663778, 0.147139, 0.676705, 0.146760, 0.663778, 0.143053, 0.693395, 0.147139, 0.676705, 0.143982, 0.711985, 0.143053, 0.693395, 0.147139, 0.676705, 0.143053, 0.693395, 0.143982, 0.711985, 0.147139, 0.676705, 0.143982, 0.711985, 0.147139, 0.676705, 0.147771, 0.687216, 0.147139, 0.676705, 0.143982, 0.711985, 0.147771, 0.687216, 0.145283, 0.727273, 0.143982, 0.711985, 0.147771, 0.687216, 0.143982, 0.711985, 0.145283, 0.727273, 0.147771, 0.687216, 0.145283, 0.727273, 0.147771, 0.687216, 0.148655, 0.695313, 0.147771, 0.687216, 0.145283, 0.727273, 0.148655, 0.695313, 0.146952, 0.739205, 0.145283, 0.727273, 0.148655, 0.695313, 0.145283, 0.727273, 0.146952, 0.739205, 0.148655, 0.695313, 0.149866, 0.701341, 0.146952, 0.739205, 0.148655, 0.695313, 0.146952, 0.739205, 0.149866, 0.701341, 0.148655, 0.695313, 0.148987, 0.747727, 0.146952, 0.739205, 0.149866, 0.701341, 0.146952, 0.739205, 0.148987, 0.747727, 0.149866, 0.701341, 0.151480, 0.705646, 0.148987, 0.747727, 0.149866, 0.701341, 0.148987, 0.747727, 0.151480, 0.705646, 0.149866, 0.701341, 0.151387, 0.752841, 0.148987, 0.747727, 0.151480, 0.705646, 0.148987, 0.747727, 0.151387, 0.752841, 0.151480, 0.705646, 0.151387, 0.752841, 0.151480, 0.705646, 0.153497, 0.708230, 0.151480, 0.705646, 0.151387, 0.752841, 0.153497, 0.708230, 0.151387, 0.752841, 0.153497, 0.708230, 0.154153, 0.754545, 0.153497, 0.708230, 0.151387, 0.752841, 0.154153, 0.754545, 0.154153, 0.754545, 0.153497, 0.708230, 0.155916, 0.709091, 0.153497, 0.708230, 0.154153, 0.754545, 0.155916, 0.709091, 0.154153, 0.754545, 0.155916, 0.709091, 0.160216, 0.754545, 0.155916, 0.709091, 0.154153, 0.754545, 0.160216, 0.754545, 0.155916, 0.709091, 0.160216, 0.709091, 0.160216, 0.754545, 0.160216, 0.709091, 0.155916, 0.709091, 0.160216, 0.754545, 0.160216, 0.754545, 0.160216, 0.709091, 0.164540, 0.733097, 0.160216, 0.709091, 0.160216, 0.754545, 0.164540, 0.733097, 0.164540, 0.733097, 0.164364, 0.768213, 0.160216, 0.754545, 0.164364, 0.768213, 0.164540, 0.733097, 0.160216, 0.754545, 0.164364, 0.768213, 0.160216, 0.759659, 0.160216, 0.754545, 0.160216, 0.759659, 0.164364, 0.768213, 0.160216, 0.754545, 0.160094, 0.776944, 0.160216, 0.759659, 0.164364, 0.768213, 0.160216, 0.759659, 0.160094, 0.776944, 0.164364, 0.768213, 0.163835, 0.798562, 0.160094, 0.776944, 0.164364, 0.768213, 0.160094, 0.776944, 0.163835, 0.798562, 0.164364, 0.768213, 0.159726, 0.792152, 0.160094, 0.776944, 0.163835, 0.798562, 0.160094, 0.776944, 0.159726, 0.792152, 0.163835, 0.798562, 0.162954, 0.824143, 0.159726, 0.792152, 0.163835, 0.798562, 0.159726, 0.792152, 0.162954, 0.824143, 0.163835, 0.798562, 0.159112, 0.805282, 0.159726, 0.792152, 0.162954, 0.824143, 0.159726, 0.792152, 0.159112, 0.805282, 0.162954, 0.824143, 0.161720, 0.844957, 0.159112, 0.805282, 0.162954, 0.824143, 0.159112, 0.805282, 0.161720, 0.844957, 0.162954, 0.824143, 0.158254, 0.816335, 0.159112, 0.805282, 0.161720, 0.844957, 0.159112, 0.805282, 0.158254, 0.816335, 0.161720, 0.844957, 0.160130, 0.861084, 0.158254, 0.816335, 0.161720, 0.844957, 0.158254, 0.816335, 0.160130, 0.861084, 0.161720, 0.844957, 0.157174, 0.825098, 0.158254, 0.816335, 0.160130, 0.861084, 0.158254, 0.816335, 0.157174, 0.825098, 0.160130, 0.861084, 0.158178, 0.872603, 0.157174, 0.825098, 0.160130, 0.861084, 0.157174, 0.825098, 0.158178, 0.872603, 0.160130, 0.861084, 0.155895, 0.831357, 0.157174, 0.825098, 0.158178, 0.872603, 0.157174, 0.825098, 0.155895, 0.831357, 0.158178, 0.872603, 0.155864, 0.879514, 0.155895, 0.831357, 0.158178, 0.872603, 0.155895, 0.831357, 0.155864, 0.879514, 0.158178, 0.872603, 0.155864, 0.879514, 0.154419, 0.835112, 0.155895, 0.831357, 0.154419, 0.835112, 0.155864, 0.879514, 0.155895, 0.831357, 0.155864, 0.879514, 0.153190, 0.881818, 0.154419, 0.835112, 0.153190, 0.881818, 0.155864, 0.879514, 0.154419, 0.835112, 0.152744, 0.836364, 0.154419, 0.835112, 0.153190, 0.881818, 0.154419, 0.835112, 0.152744, 0.836364, 0.153190, 0.881818, 0.153190, 0.881818, 0.152125, 0.881392, 0.152744, 0.836364, 0.152125, 0.881392, 0.153190, 0.881818, 0.152744, 0.836364, 0.152125, 0.881392, 0.151623, 0.835937, 0.152744, 0.836364, 0.151623, 0.835937, 0.152125, 0.881392, 0.152744, 0.836364, 0.151046, 0.880114, 0.151623, 0.835937, 0.152125, 0.881392, 0.151623, 0.835937, 0.151046, 0.880114, 0.152125, 0.881392, 0.151046, 0.880114, 0.150517, 0.834659, 0.151623, 0.835937, 0.150517, 0.834659, 0.151046, 0.880114, 0.151623, 0.835937, 0.149952, 0.877983, 0.150517, 0.834659, 0.151046, 0.880114, 0.150517, 0.834659, 0.149952, 0.877983, 0.151046, 0.880114, 0.149952, 0.877983, 0.148843, 0.875000, 0.150517, 0.834659, 0.148843, 0.875000, 0.149952, 0.877983, 0.150517, 0.834659, 0.150517, 0.834659, 0.148843, 0.875000, 0.149426, 0.832528, 0.148843, 0.875000, 0.150517, 0.834659, 0.149426, 0.832528, 0.148843, 0.875000, 0.147718, 0.871165, 0.149426, 0.832528, 0.147718, 0.871165, 0.148843, 0.875000, 0.149426, 0.832528, 0.147718, 0.871165, 0.148349, 0.829545, 0.149426, 0.832528, 0.148349, 0.829545, 0.147718, 0.871165, 0.149426, 0.832528, 0.147718, 0.871165, 0.146575, 0.866477, 0.148349, 0.829545, 0.146575, 0.866477, 0.147718, 0.871165, 0.148349, 0.829545, 0.146575, 0.866477, 0.147290, 0.825710, 0.148349, 0.829545, 0.147290, 0.825710, 0.146575, 0.866477, 0.148349, 0.829545, 0.146575, 0.866477, 0.145415, 0.860937, 0.147290, 0.825710, 0.145415, 0.860937, 0.146575, 0.866477, 0.147290, 0.825710, 0.145415, 0.860937, 0.146252, 0.821023, 0.147290, 0.825710, 0.146252, 0.821023, 0.145415, 0.860937, 0.147290, 0.825710, 0.145415, 0.860937, 0.144237, 0.854545, 0.146252, 0.821023, 0.144237, 0.854545, 0.145415, 0.860937, 0.146252, 0.821023, 0.144237, 0.854545, 0.145234, 0.815483, 0.146252, 0.821023, 0.145234, 0.815483, 0.144237, 0.854545, 0.146252, 0.821023, 0.144237, 0.854545, 0.144237, 0.809091, 0.145234, 0.815483, 0.144237, 0.809091, 0.144237, 0.854545, 0.145234, 0.815483, 0.164540, 0.733097, 0.160216, 0.709091, 0.160216, 0.697727, 0.160216, 0.709091, 0.164540, 0.733097, 0.160216, 0.697727, 0.164540, 0.733097, 0.160216, 0.697727, 0.160080, 0.674711, 0.160216, 0.697727, 0.164540, 0.733097, 0.160080, 0.674711, 0.164540, 0.554545, 0.164540, 0.733097, 0.160080, 0.674711, 0.164540, 0.733097, 0.164540, 0.554545, 0.160080, 0.674711, 0.164540, 0.554545, 0.160080, 0.674711, 0.159673, 0.654102, 0.160080, 0.674711, 0.164540, 0.554545, 0.159673, 0.654102, 0.160216, 0.600000, 0.164540, 0.554545, 0.159673, 0.654102, 0.164540, 0.554545, 0.160216, 0.600000, 0.159673, 0.654102, 0.160216, 0.600000, 0.160216, 0.554545, 0.164540, 0.554545, 0.160216, 0.554545, 0.160216, 0.600000, 0.164540, 0.554545, 0.159673, 0.654102, 0.158994, 0.635898, 0.160216, 0.600000, 0.158994, 0.635898, 0.159673, 0.654102, 0.160216, 0.600000, 0.158994, 0.635898, 0.158043, 0.620099, 0.160216, 0.600000, 0.158043, 0.620099, 0.158994, 0.635898, 0.160216, 0.600000, 0.158043, 0.620099, 0.159430, 0.586927, 0.160216, 0.600000, 0.159430, 0.586927, 0.158043, 0.620099, 0.160216, 0.600000, 0.158043, 0.620099, 0.156867, 0.607329, 0.159430, 0.586927, 0.156867, 0.607329, 0.158043, 0.620099, 0.159430, 0.586927, 0.156867, 0.607329, 0.158554, 0.575692, 0.159430, 0.586927, 0.158554, 0.575692, 0.156867, 0.607329, 0.159430, 0.586927, 0.158554, 0.575692, 0.156867, 0.607329, 0.157586, 0.566295, 0.156867, 0.607329, 0.158554, 0.575692, 0.157586, 0.566295, 0.156867, 0.607329, 0.155513, 0.598207, 0.157586, 0.566295, 0.155513, 0.598207, 0.156867, 0.607329, 0.157586, 0.566295, 0.155513, 0.598207, 0.156527, 0.558736, 0.157586, 0.566295, 0.156527, 0.558736, 0.155513, 0.598207, 0.157586, 0.566295, 0.156527, 0.558736, 0.155513, 0.598207, 0.155361, 0.552925, 0.155513, 0.598207, 0.156527, 0.558736, 0.155361, 0.552925, 0.155513, 0.598207, 0.153982, 0.592733, 0.155361, 0.552925, 0.153982, 0.592733, 0.155513, 0.598207, 0.155361, 0.552925, 0.153982, 0.592733, 0.154071, 0.548775, 0.155361, 0.552925, 0.154071, 0.548775, 0.153982, 0.592733, 0.155361, 0.552925, 0.153982, 0.592733, 0.152274, 0.590909, 0.154071, 0.548775, 0.152274, 0.590909, 0.153982, 0.592733, 0.154071, 0.548775, 0.152274, 0.590909, 0.152658, 0.546285, 0.154071, 0.548775, 0.152658, 0.546285, 0.152274, 0.590909, 0.154071, 0.548775, 0.152274, 0.590909, 0.151122, 0.545455, 0.152658, 0.546285, 0.151122, 0.545455, 0.152274, 0.590909, 0.152658, 0.546285, 0.151038, 0.591877, 0.151122, 0.545455, 0.152274, 0.590909, 0.151122, 0.545455, 0.151038, 0.591877, 0.152274, 0.590909, 0.151038, 0.591877, 0.149205, 0.547168, 0.151122, 0.545455, 0.149205, 0.547168, 0.151038, 0.591877, 0.151122, 0.545455, 0.149938, 0.594780, 0.149205, 0.547168, 0.151038, 0.591877, 0.149205, 0.547168, 0.149938, 0.594780, 0.151038, 0.591877, 0.147494, 0.552308, 0.149205, 0.547168, 0.149938, 0.594780, 0.149205, 0.547168, 0.147494, 0.552308, 0.149938, 0.594780, 0.148976, 0.599618, 0.147494, 0.552308, 0.149938, 0.594780, 0.147494, 0.552308, 0.148976, 0.599618, 0.149938, 0.594780, 0.145991, 0.560875, 0.147494, 0.552308, 0.148976, 0.599618, 0.147494, 0.552308, 0.145991, 0.560875, 0.148976, 0.599618, 0.148976, 0.599618, 0.148149, 0.606392, 0.145991, 0.560875, 0.148149, 0.606392, 0.148976, 0.599618, 0.145991, 0.560875, 0.148149, 0.606392, 0.144695, 0.572869, 0.145991, 0.560875, 0.144695, 0.572869, 0.148149, 0.606392, 0.145991, 0.560875, 0.148149, 0.606392, 0.147486, 0.614826, 0.144695, 0.572869, 0.147486, 0.614826, 0.148149, 0.606392, 0.144695, 0.572869, 0.147486, 0.614826, 0.143652, 0.587740, 0.144695, 0.572869, 0.143652, 0.587740, 0.147486, 0.614826, 0.144695, 0.572869, 0.147486, 0.614826, 0.147013, 0.624645, 0.143652, 0.587740, 0.147013, 0.624645, 0.147486, 0.614826, 0.143652, 0.587740, 0.147013, 0.624645, 0.142906, 0.604936, 0.143652, 0.587740, 0.142906, 0.604936, 0.147013, 0.624645, 0.143652, 0.587740, 0.142459, 0.624458, 0.142906, 0.604936, 0.147013, 0.624645, 0.142906, 0.604936, 0.142459, 0.624458, 0.147013, 0.624645, 0.146729, 0.635849, 0.142459, 0.624458, 0.147013, 0.624645, 0.142459, 0.624458, 0.146729, 0.635849, 0.147013, 0.624645, 0.142310, 0.646307, 0.142459, 0.624458, 0.146729, 0.635849, 0.142459, 0.624458, 0.142310, 0.646307, 0.146729, 0.635849, 0.146634, 0.648437, 0.142310, 0.646307, 0.146729, 0.635849, 0.142310, 0.646307, 0.146634, 0.648437, 0.146729, 0.635849, 0.195159, 0.749432, 0.195159, 0.554545, 0.195159, 0.749432, 0.195159, 0.749432, 0.195159, 0.554545, 0.195159, 0.554545, 0.195159, 0.554545, 0.190835, 0.554545, 0.195159, 0.554545, 0.195159, 0.554545, 0.190835, 0.554545, 0.190835, 0.554545, 0.190835, 0.554545, 0.190835, 0.747727, 0.190835, 0.554545, 0.190835, 0.554545, 0.190835, 0.747727, 0.190835, 0.747727, 0.190835, 0.747727, 0.190744, 0.768559, 0.190835, 0.747727, 0.190835, 0.747727, 0.190744, 0.768559, 0.190744, 0.768559, 0.190744, 0.768559, 0.190471, 0.786594, 0.190744, 0.768559, 0.190744, 0.768559, 0.190471, 0.786594, 0.190471, 0.786594, 0.190471, 0.786594, 0.190016, 0.801833, 0.190471, 0.786594, 0.190471, 0.786594, 0.190016, 0.801833, 0.190016, 0.801833, 0.190016, 0.801833, 0.189378, 0.814276, 0.190016, 0.801833, 0.190016, 0.801833, 0.189378, 0.814276, 0.189378, 0.814276, 0.189378, 0.814276, 0.188559, 0.823939, 0.189378, 0.814276, 0.189378, 0.814276, 0.188559, 0.823939, 0.188559, 0.823939, 0.188559, 0.823939, 0.187557, 0.830842, 0.188559, 0.823939, 0.188559, 0.823939, 0.187557, 0.830842, 0.187557, 0.830842, 0.187557, 0.830842, 0.186374, 0.834983, 0.187557, 0.830842, 0.187557, 0.830842, 0.186374, 0.834983, 0.186374, 0.834983, 0.186374, 0.834983, 0.185008, 0.836364, 0.186374, 0.834983, 0.186374, 0.834983, 0.185008, 0.836364, 0.185008, 0.836364, 0.185008, 0.836364, 0.183349, 0.834699, 0.185008, 0.836364, 0.185008, 0.836364, 0.183349, 0.834699, 0.183349, 0.834699, 0.183349, 0.834699, 0.181876, 0.829705, 0.183349, 0.834699, 0.183349, 0.834699, 0.181876, 0.829705, 0.181876, 0.829705, 0.181876, 0.829705, 0.180588, 0.821382, 0.181876, 0.829705, 0.181876, 0.829705, 0.180588, 0.821382, 0.180588, 0.821382, 0.180588, 0.821382, 0.179485, 0.809730, 0.180588, 0.821382, 0.180588, 0.821382, 0.179485, 0.809730, 0.179485, 0.809730, 0.179485, 0.809730, 0.178601, 0.795201, 0.179485, 0.809730, 0.179485, 0.809730, 0.178601, 0.795201, 0.178601, 0.795201, 0.178601, 0.795201, 0.177970, 0.778249, 0.178601, 0.795201, 0.178601, 0.795201, 0.177970, 0.778249, 0.177970, 0.778249, 0.177970, 0.778249, 0.177591, 0.758873, 0.177970, 0.778249, 0.177970, 0.778249, 0.177591, 0.758873, 0.177591, 0.758873, 0.177591, 0.758873, 0.177464, 0.737074, 0.177591, 0.758873, 0.177591, 0.758873, 0.177464, 0.737074, 0.177464, 0.737074, 0.177464, 0.737074, 0.177464, 0.554545, 0.177464, 0.737074, 0.177464, 0.737074, 0.177464, 0.554545, 0.177464, 0.554545, 0.177464, 0.554545, 0.173117, 0.554545, 0.177464, 0.554545, 0.177464, 0.554545, 0.173117, 0.554545, 0.173117, 0.554545, 0.173117, 0.554545, 0.173117, 0.872727, 0.173117, 0.554545, 0.173117, 0.554545, 0.173117, 0.872727, 0.173117, 0.872727, 0.173117, 0.872727, 0.177464, 0.872727, 0.173117, 0.872727, 0.173117, 0.872727, 0.177464, 0.872727, 0.177464, 0.872727, 0.177464, 0.872727, 0.177464, 0.827273, 0.177464, 0.872727, 0.177464, 0.872727, 0.177464, 0.827273, 0.177464, 0.827273, 0.177464, 0.827273, 0.178274, 0.840115, 0.177464, 0.827273, 0.177464, 0.827273, 0.178274, 0.840115, 0.178274, 0.840115, 0.178274, 0.840115, 0.179153, 0.851225, 0.178274, 0.840115, 0.178274, 0.840115, 0.179153, 0.851225, 0.179153, 0.851225, 0.179153, 0.851225, 0.180101, 0.860605, 0.179153, 0.851225, 0.179153, 0.851225, 0.180101, 0.860605, 0.180101, 0.860605, 0.180101, 0.860605, 0.181119, 0.868253, 0.180101, 0.860605, 0.180101, 0.860605, 0.181119, 0.868253, 0.181119, 0.868253, 0.181119, 0.868253, 0.182211, 0.874188, 0.181119, 0.868253, 0.181119, 0.868253, 0.182211, 0.874188, 0.182211, 0.874188, 0.182211, 0.874188, 0.183383, 0.878427, 0.182211, 0.874188, 0.182211, 0.874188, 0.183383, 0.878427, 0.183383, 0.878427, 0.183383, 0.878427, 0.184637, 0.880970, 0.183383, 0.878427, 0.183383, 0.878427, 0.184637, 0.880970, 0.184637, 0.880970, 0.184637, 0.880970, 0.185971, 0.881818, 0.184637, 0.880970, 0.184637, 0.880970, 0.185971, 0.881818, 0.185971, 0.881818, 0.185971, 0.881818, 0.188101, 0.879719, 0.185971, 0.881818, 0.185971, 0.881818, 0.188101, 0.879719, 0.188101, 0.879719, 0.188101, 0.879719, 0.189954, 0.873420, 0.188101, 0.879719, 0.188101, 0.879719, 0.189954, 0.873420, 0.189954, 0.873420, 0.189954, 0.873420, 0.191532, 0.862922, 0.189954, 0.873420, 0.189954, 0.873420, 0.191532, 0.862922, 0.191532, 0.862922, 0.191532, 0.862922, 0.192833, 0.848224, 0.191532, 0.862922, 0.191532, 0.862922, 0.192833, 0.848224, 0.192833, 0.848224, 0.192833, 0.848224, 0.193851, 0.829452, 0.192833, 0.848224, 0.192833, 0.848224, 0.193851, 0.829452, 0.193851, 0.829452, 0.193851, 0.829452, 0.194578, 0.806729, 0.193851, 0.829452, 0.193851, 0.829452, 0.194578, 0.806729, 0.194578, 0.806729, 0.194578, 0.806729, 0.195014, 0.780056, 0.194578, 0.806729, 0.194578, 0.806729, 0.195014, 0.780056, 0.195014, 0.780056, 0.177464, 0.872727, 0.173117, 0.872727, 0.177464, 0.827273, 0.173117, 0.872727, 0.177464, 0.872727, 0.177464, 0.827273, 0.173117, 0.872727, 0.177970, 0.778249, 0.177464, 0.827273, 0.177970, 0.778249, 0.173117, 0.872727, 0.177464, 0.827273, 0.177970, 0.778249, 0.173117, 0.872727, 0.177591, 0.758873, 0.173117, 0.872727, 0.177970, 0.778249, 0.177591, 0.758873, 0.177591, 0.758873, 0.173117, 0.872727, 0.177464, 0.737074, 0.173117, 0.872727, 0.177591, 0.758873, 0.177464, 0.737074, 0.173117, 0.872727, 0.173117, 0.554545, 0.177464, 0.737074, 0.173117, 0.554545, 0.173117, 0.872727, 0.177464, 0.737074, 0.173117, 0.554545, 0.177464, 0.554545, 0.177464, 0.737074, 0.177464, 0.554545, 0.173117, 0.554545, 0.177464, 0.737074, 0.177464, 0.827273, 0.177970, 0.778249, 0.178601, 0.795201, 0.177970, 0.778249, 0.177464, 0.827273, 0.178601, 0.795201, 0.177464, 0.827273, 0.178601, 0.795201, 0.179485, 0.809730, 0.178601, 0.795201, 0.177464, 0.827273, 0.179485, 0.809730, 0.178274, 0.840115, 0.177464, 0.827273, 0.179485, 0.809730, 0.177464, 0.827273, 0.178274, 0.840115, 0.179485, 0.809730, 0.178274, 0.840115, 0.179485, 0.809730, 0.180588, 0.821382, 0.179485, 0.809730, 0.178274, 0.840115, 0.180588, 0.821382, 0.179153, 0.851225, 0.178274, 0.840115, 0.180588, 0.821382, 0.178274, 0.840115, 0.179153, 0.851225, 0.180588, 0.821382, 0.181876, 0.829705, 0.179153, 0.851225, 0.180588, 0.821382, 0.179153, 0.851225, 0.181876, 0.829705, 0.180588, 0.821382, 0.181876, 0.829705, 0.180101, 0.860605, 0.179153, 0.851225, 0.180101, 0.860605, 0.181876, 0.829705, 0.179153, 0.851225, 0.181119, 0.868253, 0.180101, 0.860605, 0.181876, 0.829705, 0.180101, 0.860605, 0.181119, 0.868253, 0.181876, 0.829705, 0.181119, 0.868253, 0.181876, 0.829705, 0.183349, 0.834699, 0.181876, 0.829705, 0.181119, 0.868253, 0.183349, 0.834699, 0.182211, 0.874188, 0.181119, 0.868253, 0.183349, 0.834699, 0.181119, 0.868253, 0.182211, 0.874188, 0.183349, 0.834699, 0.182211, 0.874188, 0.183349, 0.834699, 0.183383, 0.878427, 0.183349, 0.834699, 0.182211, 0.874188, 0.183383, 0.878427, 0.183383, 0.878427, 0.183349, 0.834699, 0.185008, 0.836364, 0.183349, 0.834699, 0.183383, 0.878427, 0.185008, 0.836364, 0.184637, 0.880970, 0.183383, 0.878427, 0.185008, 0.836364, 0.183383, 0.878427, 0.184637, 0.880970, 0.185008, 0.836364, 0.185971, 0.881818, 0.184637, 0.880970, 0.185008, 0.836364, 0.184637, 0.880970, 0.185971, 0.881818, 0.185008, 0.836364, 0.185971, 0.881818, 0.185008, 0.836364, 0.186374, 0.834983, 0.185008, 0.836364, 0.185971, 0.881818, 0.186374, 0.834983, 0.188101, 0.879719, 0.185971, 0.881818, 0.186374, 0.834983, 0.185971, 0.881818, 0.188101, 0.879719, 0.186374, 0.834983, 0.186374, 0.834983, 0.187557, 0.830842, 0.188101, 0.879719, 0.187557, 0.830842, 0.186374, 0.834983, 0.188101, 0.879719, 0.188101, 0.879719, 0.187557, 0.830842, 0.189954, 0.873420, 0.187557, 0.830842, 0.188101, 0.879719, 0.189954, 0.873420, 0.187557, 0.830842, 0.188559, 0.823939, 0.189954, 0.873420, 0.188559, 0.823939, 0.187557, 0.830842, 0.189954, 0.873420, 0.189954, 0.873420, 0.188559, 0.823939, 0.191532, 0.862922, 0.188559, 0.823939, 0.189954, 0.873420, 0.191532, 0.862922, 0.188559, 0.823939, 0.189378, 0.814276, 0.191532, 0.862922, 0.189378, 0.814276, 0.188559, 0.823939, 0.191532, 0.862922, 0.191532, 0.862922, 0.189378, 0.814276, 0.192833, 0.848224, 0.189378, 0.814276, 0.191532, 0.862922, 0.192833, 0.848224, 0.192833, 0.848224, 0.189378, 0.814276, 0.193851, 0.829452, 0.189378, 0.814276, 0.192833, 0.848224, 0.193851, 0.829452, 0.193851, 0.829452, 0.189378, 0.814276, 0.190016, 0.801833, 0.189378, 0.814276, 0.193851, 0.829452, 0.190016, 0.801833, 0.193851, 0.829452, 0.190016, 0.801833, 0.194578, 0.806729, 0.190016, 0.801833, 0.193851, 0.829452, 0.194578, 0.806729, 0.190016, 0.801833, 0.190471, 0.786594, 0.194578, 0.806729, 0.190471, 0.786594, 0.190016, 0.801833, 0.194578, 0.806729, 0.194578, 0.806729, 0.190471, 0.786594, 0.195014, 0.780056, 0.190471, 0.786594, 0.194578, 0.806729, 0.195014, 0.780056, 0.190471, 0.786594, 0.190744, 0.768559, 0.195014, 0.780056, 0.190744, 0.768559, 0.190471, 0.786594, 0.195014, 0.780056, 0.195014, 0.780056, 0.190744, 0.768559, 0.195159, 0.749432, 0.190744, 0.768559, 0.195014, 0.780056, 0.195159, 0.749432, 0.195159, 0.749432, 0.190744, 0.768559, 0.190835, 0.747727, 0.190744, 0.768559, 0.195159, 0.749432, 0.190835, 0.747727, 0.190835, 0.747727, 0.195159, 0.554545, 0.195159, 0.749432, 0.195159, 0.554545, 0.190835, 0.747727, 0.195159, 0.749432, 0.190835, 0.747727, 0.190835, 0.554545, 0.195159, 0.554545, 0.190835, 0.554545, 0.190835, 0.747727, 0.195159, 0.554545, 0.223053, 0.854545, 0.223053, 0.809091, 0.223053, 0.854545, 0.223053, 0.854545, 0.223053, 0.809091, 0.223053, 0.809091, 0.223053, 0.809091, 0.222135, 0.815483, 0.223053, 0.809091, 0.223053, 0.809091, 0.222135, 0.815483, 0.222135, 0.815483, 0.222135, 0.815483, 0.221217, 0.821023, 0.222135, 0.815483, 0.222135, 0.815483, 0.221217, 0.821023, 0.221217, 0.821023, 0.221217, 0.821023, 0.220297, 0.825710, 0.221217, 0.821023, 0.221217, 0.821023, 0.220297, 0.825710, 0.220297, 0.825710, 0.220297, 0.825710, 0.219375, 0.829545, 0.220297, 0.825710, 0.220297, 0.825710, 0.219375, 0.829545, 0.219375, 0.829545, 0.219375, 0.829545, 0.218450, 0.832528, 0.219375, 0.829545, 0.219375, 0.829545, 0.218450, 0.832528, 0.218450, 0.832528, 0.218450, 0.832528, 0.217521, 0.834659, 0.218450, 0.832528, 0.218450, 0.832528, 0.217521, 0.834659, 0.217521, 0.834659, 0.217521, 0.834659, 0.216588, 0.835937, 0.217521, 0.834659, 0.217521, 0.834659, 0.216588, 0.835937, 0.216588, 0.835937, 0.216588, 0.835937, 0.215650, 0.836364, 0.216588, 0.835937, 0.216588, 0.835937, 0.215650, 0.836364, 0.215650, 0.836364, 0.215650, 0.836364, 0.213665, 0.834348, 0.215650, 0.836364, 0.215650, 0.836364, 0.213665, 0.834348, 0.213665, 0.834348, 0.213665, 0.834348, 0.211914, 0.828303, 0.213665, 0.834348, 0.213665, 0.834348, 0.211914, 0.828303, 0.211914, 0.828303, 0.211914, 0.828303, 0.210398, 0.818226, 0.211914, 0.828303, 0.211914, 0.828303, 0.210398, 0.818226, 0.210398, 0.818226, 0.210398, 0.818226, 0.209118, 0.804119, 0.210398, 0.818226, 0.210398, 0.818226, 0.209118, 0.804119, 0.209118, 0.804119, 0.209118, 0.804119, 0.208100, 0.786372, 0.209118, 0.804119, 0.209118, 0.804119, 0.208100, 0.786372, 0.208100, 0.786372, 0.208100, 0.786372, 0.207373, 0.765376, 0.208100, 0.786372, 0.208100, 0.786372, 0.207373, 0.765376, 0.207373, 0.765376, 0.207373, 0.765376, 0.206937, 0.741131, 0.207373, 0.765376, 0.207373, 0.765376, 0.206937, 0.741131, 0.206937, 0.741131, 0.206937, 0.741131, 0.206791, 0.713636, 0.206937, 0.741131, 0.206937, 0.741131, 0.206791, 0.713636, 0.206791, 0.713636, 0.206791, 0.713636, 0.206937, 0.686142, 0.206791, 0.713636, 0.206791, 0.713636, 0.206937, 0.686142, 0.206937, 0.686142, 0.206937, 0.686142, 0.207373, 0.661896, 0.206937, 0.686142, 0.206937, 0.686142, 0.207373, 0.661896, 0.207373, 0.661896, 0.207373, 0.661896, 0.208100, 0.640900, 0.207373, 0.661896, 0.207373, 0.661896, 0.208100, 0.640900, 0.208100, 0.640900, 0.208100, 0.640900, 0.209118, 0.623153, 0.208100, 0.640900, 0.208100, 0.640900, 0.209118, 0.623153, 0.209118, 0.623153, 0.209118, 0.623153, 0.210398, 0.609047, 0.209118, 0.623153, 0.209118, 0.623153, 0.210398, 0.609047, 0.210398, 0.609047, 0.210398, 0.609047, 0.211914, 0.598970, 0.210398, 0.609047, 0.210398, 0.609047, 0.211914, 0.598970, 0.211914, 0.598970, 0.211914, 0.598970, 0.213665, 0.592924, 0.211914, 0.598970, 0.211914, 0.598970, 0.213665, 0.592924, 0.213665, 0.592924, 0.213665, 0.592924, 0.215650, 0.590909, 0.213665, 0.592924, 0.213665, 0.592924, 0.215650, 0.590909, 0.215650, 0.590909, 0.215650, 0.590909, 0.216588, 0.591335, 0.215650, 0.590909, 0.215650, 0.590909, 0.216588, 0.591335, 0.216588, 0.591335, 0.216588, 0.591335, 0.217521, 0.592614, 0.216588, 0.591335, 0.216588, 0.591335, 0.217521, 0.592614, 0.217521, 0.592614, 0.217521, 0.592614, 0.218450, 0.594744, 0.217521, 0.592614, 0.217521, 0.592614, 0.218450, 0.594744, 0.218450, 0.594744, 0.218450, 0.594744, 0.219375, 0.597727, 0.218450, 0.594744, 0.218450, 0.594744, 0.219375, 0.597727, 0.219375, 0.597727, 0.219375, 0.597727, 0.220297, 0.601563, 0.219375, 0.597727, 0.219375, 0.597727, 0.220297, 0.601563, 0.220297, 0.601563, 0.220297, 0.601563, 0.221217, 0.606250, 0.220297, 0.601563, 0.220297, 0.601563, 0.221217, 0.606250, 0.221217, 0.606250, 0.221217, 0.606250, 0.222135, 0.611790, 0.221217, 0.606250, 0.221217, 0.606250, 0.222135, 0.611790, 0.222135, 0.611790, 0.222135, 0.611790, 0.223053, 0.618182, 0.222135, 0.611790, 0.222135, 0.611790, 0.223053, 0.618182, 0.223053, 0.618182, 0.223053, 0.618182, 0.223053, 0.572727, 0.223053, 0.618182, 0.223053, 0.618182, 0.223053, 0.572727, 0.223053, 0.572727, 0.223053, 0.572727, 0.222140, 0.566335, 0.223053, 0.572727, 0.223053, 0.572727, 0.222140, 0.566335, 0.222140, 0.566335, 0.222140, 0.566335, 0.221211, 0.560795, 0.222140, 0.566335, 0.222140, 0.566335, 0.221211, 0.560795, 0.221211, 0.560795, 0.221211, 0.560795, 0.220266, 0.556108, 0.221211, 0.560795, 0.221211, 0.560795, 0.220266, 0.556108, 0.220266, 0.556108, 0.220266, 0.556108, 0.219304, 0.552273, 0.220266, 0.556108, 0.220266, 0.556108, 0.219304, 0.552273, 0.219304, 0.552273, 0.219304, 0.552273, 0.218320, 0.549290, 0.219304, 0.552273, 0.219304, 0.552273, 0.218320, 0.549290, 0.218320, 0.549290, 0.218320, 0.549290, 0.217304, 0.547159, 0.218320, 0.549290, 0.218320, 0.549290, 0.217304, 0.547159, 0.217304, 0.547159, 0.217304, 0.547159, 0.216258, 0.545881, 0.217304, 0.547159, 0.217304, 0.547159, 0.216258, 0.545881, 0.216258, 0.545881, 0.216258, 0.545881, 0.215180, 0.545455, 0.216258, 0.545881, 0.216258, 0.545881, 0.215180, 0.545455, 0.215180, 0.545455, 0.215180, 0.545455, 0.212360, 0.548295, 0.215180, 0.545455, 0.215180, 0.545455, 0.212360, 0.548295, 0.212360, 0.548295, 0.212360, 0.548295, 0.209846, 0.556818, 0.212360, 0.548295, 0.212360, 0.548295, 0.209846, 0.556818, 0.209846, 0.556818, 0.209846, 0.556818, 0.207637, 0.571023, 0.209846, 0.556818, 0.209846, 0.556818, 0.207637, 0.571023, 0.207637, 0.571023, 0.207637, 0.571023, 0.205734, 0.590909, 0.207637, 0.571023, 0.207637, 0.571023, 0.205734, 0.590909, 0.205734, 0.590909, 0.205734, 0.590909, 0.204202, 0.615625, 0.205734, 0.590909, 0.205734, 0.590909, 0.204202, 0.615625, 0.204202, 0.615625, 0.204202, 0.615625, 0.203108, 0.644318, 0.204202, 0.615625, 0.204202, 0.615625, 0.203108, 0.644318, 0.203108, 0.644318, 0.203108, 0.644318, 0.202451, 0.676989, 0.203108, 0.644318, 0.203108, 0.644318, 0.202451, 0.676989, 0.202451, 0.676989, 0.202451, 0.676989, 0.202232, 0.713636, 0.202451, 0.676989, 0.202451, 0.676989, 0.202232, 0.713636, 0.202232, 0.713636, 0.202232, 0.713636, 0.202453, 0.750746, 0.202232, 0.713636, 0.202232, 0.713636, 0.202453, 0.750746, 0.202453, 0.750746, 0.202453, 0.750746, 0.203117, 0.783665, 0.202453, 0.750746, 0.202453, 0.750746, 0.203117, 0.783665, 0.203117, 0.783665, 0.203117, 0.783665, 0.204222, 0.812393, 0.203117, 0.783665, 0.203117, 0.783665, 0.204222, 0.812393, 0.204222, 0.812393, 0.204222, 0.812393, 0.205769, 0.836932, 0.204222, 0.812393, 0.204222, 0.812393, 0.205769, 0.836932, 0.205769, 0.836932, 0.205769, 0.836932, 0.207701, 0.856570, 0.205769, 0.836932, 0.205769, 0.836932, 0.207701, 0.856570, 0.207701, 0.856570, 0.207701, 0.856570, 0.209961, 0.870597, 0.207701, 0.856570, 0.207701, 0.856570, 0.209961, 0.870597, 0.209961, 0.870597, 0.209961, 0.870597, 0.212548, 0.879013, 0.209961, 0.870597, 0.209961, 0.870597, 0.212548, 0.879013, 0.212548, 0.879013, 0.212548, 0.879013, 0.215462, 0.881818, 0.212548, 0.879013, 0.212548, 0.879013, 0.215462, 0.881818, 0.215462, 0.881818, 0.215462, 0.881818, 0.216455, 0.881392, 0.215462, 0.881818, 0.215462, 0.881818, 0.216455, 0.881392, 0.216455, 0.881392, 0.216455, 0.881392, 0.217436, 0.880114, 0.216455, 0.881392, 0.216455, 0.881392, 0.217436, 0.880114, 0.217436, 0.880114, 0.217436, 0.880114, 0.218406, 0.877983, 0.217436, 0.880114, 0.217436, 0.880114, 0.218406, 0.877983, 0.218406, 0.877983, 0.218406, 0.877983, 0.219363, 0.875000, 0.218406, 0.877983, 0.218406, 0.877983, 0.219363, 0.875000, 0.219363, 0.875000, 0.219363, 0.875000, 0.220308, 0.871165, 0.219363, 0.875000, 0.219363, 0.875000, 0.220308, 0.871165, 0.220308, 0.871165, 0.220308, 0.871165, 0.221237, 0.866477, 0.220308, 0.871165, 0.220308, 0.871165, 0.221237, 0.866477, 0.221237, 0.866477, 0.221237, 0.866477, 0.222152, 0.860937, 0.221237, 0.866477, 0.221237, 0.866477, 0.222152, 0.860937, 0.222152, 0.860937, 0.206937, 0.741131, 0.202453, 0.750746, 0.202232, 0.713636, 0.202453, 0.750746, 0.206937, 0.741131, 0.202232, 0.713636, 0.207373, 0.765376, 0.202453, 0.750746, 0.206937, 0.741131, 0.202453, 0.750746, 0.207373, 0.765376, 0.206937, 0.741131, 0.207373, 0.765376, 0.203117, 0.783665, 0.202453, 0.750746, 0.203117, 0.783665, 0.207373, 0.765376, 0.202453, 0.750746, 0.208100, 0.786372, 0.203117, 0.783665, 0.207373, 0.765376, 0.203117, 0.783665, 0.208100, 0.786372, 0.207373, 0.765376, 0.208100, 0.786372, 0.204222, 0.812393, 0.203117, 0.783665, 0.204222, 0.812393, 0.208100, 0.786372, 0.203117, 0.783665, 0.205769, 0.836932, 0.204222, 0.812393, 0.208100, 0.786372, 0.204222, 0.812393, 0.205769, 0.836932, 0.208100, 0.786372, 0.205769, 0.836932, 0.208100, 0.786372, 0.209118, 0.804119, 0.208100, 0.786372, 0.205769, 0.836932, 0.209118, 0.804119, 0.207701, 0.856570, 0.205769, 0.836932, 0.209118, 0.804119, 0.205769, 0.836932, 0.207701, 0.856570, 0.209118, 0.804119, 0.207701, 0.856570, 0.209118, 0.804119, 0.210398, 0.818226, 0.209118, 0.804119, 0.207701, 0.856570, 0.210398, 0.818226, 0.209961, 0.870597, 0.207701, 0.856570, 0.210398, 0.818226, 0.207701, 0.856570, 0.209961, 0.870597, 0.210398, 0.818226, 0.209961, 0.870597, 0.210398, 0.818226, 0.211914, 0.828303, 0.210398, 0.818226, 0.209961, 0.870597, 0.211914, 0.828303, 0.209961, 0.870597, 0.211914, 0.828303, 0.212548, 0.879013, 0.211914, 0.828303, 0.209961, 0.870597, 0.212548, 0.879013, 0.212548, 0.879013, 0.211914, 0.828303, 0.213665, 0.834348, 0.211914, 0.828303, 0.212548, 0.879013, 0.213665, 0.834348, 0.215462, 0.881818, 0.212548, 0.879013, 0.213665, 0.834348, 0.212548, 0.879013, 0.215462, 0.881818, 0.213665, 0.834348, 0.213665, 0.834348, 0.215650, 0.836364, 0.215462, 0.881818, 0.215650, 0.836364, 0.213665, 0.834348, 0.215462, 0.881818, 0.216455, 0.881392, 0.215462, 0.881818, 0.215650, 0.836364, 0.215462, 0.881818, 0.216455, 0.881392, 0.215650, 0.836364, 0.216455, 0.881392, 0.215650, 0.836364, 0.216588, 0.835937, 0.215650, 0.836364, 0.216455, 0.881392, 0.216588, 0.835937, 0.216455, 0.881392, 0.216588, 0.835937, 0.217436, 0.880114, 0.216588, 0.835937, 0.216455, 0.881392, 0.217436, 0.880114, 0.217436, 0.880114, 0.216588, 0.835937, 0.217521, 0.834659, 0.216588, 0.835937, 0.217436, 0.880114, 0.217521, 0.834659, 0.218406, 0.877983, 0.217436, 0.880114, 0.217521, 0.834659, 0.217436, 0.880114, 0.218406, 0.877983, 0.217521, 0.834659, 0.218406, 0.877983, 0.217521, 0.834659, 0.218450, 0.832528, 0.217521, 0.834659, 0.218406, 0.877983, 0.218450, 0.832528, 0.218406, 0.877983, 0.218450, 0.832528, 0.219363, 0.875000, 0.218450, 0.832528, 0.218406, 0.877983, 0.219363, 0.875000, 0.220308, 0.871165, 0.219363, 0.875000, 0.218450, 0.832528, 0.219363, 0.875000, 0.220308, 0.871165, 0.218450, 0.832528, 0.220308, 0.871165, 0.218450, 0.832528, 0.219375, 0.829545, 0.218450, 0.832528, 0.220308, 0.871165, 0.219375, 0.829545, 0.220308, 0.871165, 0.219375, 0.829545, 0.221237, 0.866477, 0.219375, 0.829545, 0.220308, 0.871165, 0.221237, 0.866477, 0.219375, 0.829545, 0.220297, 0.825710, 0.221237, 0.866477, 0.220297, 0.825710, 0.219375, 0.829545, 0.221237, 0.866477, 0.221237, 0.866477, 0.220297, 0.825710, 0.222152, 0.860937, 0.220297, 0.825710, 0.221237, 0.866477, 0.222152, 0.860937, 0.220297, 0.825710, 0.221217, 0.821023, 0.222152, 0.860937, 0.221217, 0.821023, 0.220297, 0.825710, 0.222152, 0.860937, 0.222152, 0.860937, 0.221217, 0.821023, 0.223053, 0.854545, 0.221217, 0.821023, 0.222152, 0.860937, 0.223053, 0.854545, 0.221217, 0.821023, 0.222135, 0.815483, 0.223053, 0.854545, 0.222135, 0.815483, 0.221217, 0.821023, 0.223053, 0.854545, 0.222135, 0.815483, 0.223053, 0.809091, 0.223053, 0.854545, 0.223053, 0.809091, 0.222135, 0.815483, 0.223053, 0.854545, 0.206937, 0.741131, 0.202232, 0.713636, 0.206791, 0.713636, 0.202232, 0.713636, 0.206937, 0.741131, 0.206791, 0.713636, 0.206791, 0.713636, 0.202232, 0.713636, 0.206937, 0.686142, 0.202232, 0.713636, 0.206791, 0.713636, 0.206937, 0.686142, 0.202232, 0.713636, 0.202451, 0.676989, 0.206937, 0.686142, 0.202451, 0.676989, 0.202232, 0.713636, 0.206937, 0.686142, 0.206937, 0.686142, 0.202451, 0.676989, 0.207373, 0.661896, 0.202451, 0.676989, 0.206937, 0.686142, 0.207373, 0.661896, 0.202451, 0.676989, 0.203108, 0.644318, 0.207373, 0.661896, 0.203108, 0.644318, 0.202451, 0.676989, 0.207373, 0.661896, 0.207373, 0.661896, 0.203108, 0.644318, 0.204202, 0.615625, 0.203108, 0.644318, 0.207373, 0.661896, 0.204202, 0.615625, 0.208100, 0.640900, 0.207373, 0.661896, 0.204202, 0.615625, 0.207373, 0.661896, 0.208100, 0.640900, 0.204202, 0.615625, 0.208100, 0.640900, 0.204202, 0.615625, 0.205734, 0.590909, 0.204202, 0.615625, 0.208100, 0.640900, 0.205734, 0.590909, 0.209118, 0.623153, 0.208100, 0.640900, 0.205734, 0.590909, 0.208100, 0.640900, 0.209118, 0.623153, 0.205734, 0.590909, 0.205734, 0.590909, 0.207637, 0.571023, 0.209118, 0.623153, 0.207637, 0.571023, 0.205734, 0.590909, 0.209118, 0.623153, 0.209118, 0.623153, 0.207637, 0.571023, 0.210398, 0.609047, 0.207637, 0.571023, 0.209118, 0.623153, 0.210398, 0.609047, 0.210398, 0.609047, 0.207637, 0.571023, 0.209846, 0.556818, 0.207637, 0.571023, 0.210398, 0.609047, 0.209846, 0.556818, 0.211914, 0.598970, 0.210398, 0.609047, 0.209846, 0.556818, 0.210398, 0.609047, 0.211914, 0.598970, 0.209846, 0.556818, 0.211914, 0.598970, 0.209846, 0.556818, 0.212360, 0.548295, 0.209846, 0.556818, 0.211914, 0.598970, 0.212360, 0.548295, 0.213665, 0.592924, 0.211914, 0.598970, 0.212360, 0.548295, 0.211914, 0.598970, 0.213665, 0.592924, 0.212360, 0.548295, 0.213665, 0.592924, 0.212360, 0.548295, 0.215180, 0.545455, 0.212360, 0.548295, 0.213665, 0.592924, 0.215180, 0.545455, 0.215650, 0.590909, 0.213665, 0.592924, 0.215180, 0.545455, 0.213665, 0.592924, 0.215650, 0.590909, 0.215180, 0.545455, 0.215650, 0.590909, 0.215180, 0.545455, 0.216258, 0.545881, 0.215180, 0.545455, 0.215650, 0.590909, 0.216258, 0.545881, 0.215650, 0.590909, 0.216258, 0.545881, 0.216588, 0.591335, 0.216258, 0.545881, 0.215650, 0.590909, 0.216588, 0.591335, 0.216588, 0.591335, 0.216258, 0.545881, 0.217304, 0.547159, 0.216258, 0.545881, 0.216588, 0.591335, 0.217304, 0.547159, 0.217521, 0.592614, 0.216588, 0.591335, 0.217304, 0.547159, 0.216588, 0.591335, 0.217521, 0.592614, 0.217304, 0.547159, 0.218320, 0.549290, 0.217521, 0.592614, 0.217304, 0.547159, 0.217521, 0.592614, 0.218320, 0.549290, 0.217304, 0.547159, 0.218450, 0.594744, 0.217521, 0.592614, 0.218320, 0.549290, 0.217521, 0.592614, 0.218450, 0.594744, 0.218320, 0.549290, 0.219304, 0.552273, 0.218450, 0.594744, 0.218320, 0.549290, 0.218450, 0.594744, 0.219304, 0.552273, 0.218320, 0.549290, 0.218450, 0.594744, 0.219304, 0.552273, 0.220266, 0.556108, 0.219304, 0.552273, 0.218450, 0.594744, 0.220266, 0.556108, 0.219375, 0.597727, 0.218450, 0.594744, 0.220266, 0.556108, 0.218450, 0.594744, 0.219375, 0.597727, 0.220266, 0.556108, 0.219375, 0.597727, 0.220266, 0.556108, 0.221211, 0.560795, 0.220266, 0.556108, 0.219375, 0.597727, 0.221211, 0.560795, 0.220297, 0.601563, 0.219375, 0.597727, 0.221211, 0.560795, 0.219375, 0.597727, 0.220297, 0.601563, 0.221211, 0.560795, 0.220297, 0.601563, 0.221211, 0.560795, 0.222140, 0.566335, 0.221211, 0.560795, 0.220297, 0.601563, 0.222140, 0.566335, 0.221217, 0.606250, 0.220297, 0.601563, 0.222140, 0.566335, 0.220297, 0.601563, 0.221217, 0.606250, 0.222140, 0.566335, 0.221217, 0.606250, 0.222140, 0.566335, 0.223053, 0.572727, 0.222140, 0.566335, 0.221217, 0.606250, 0.223053, 0.572727, 0.222135, 0.611790, 0.221217, 0.606250, 0.223053, 0.572727, 0.221217, 0.606250, 0.222135, 0.611790, 0.223053, 0.572727, 0.222135, 0.611790, 0.223053, 0.572727, 0.223053, 0.618182, 0.223053, 0.572727, 0.222135, 0.611790, 0.223053, 0.618182, 0.252943, 0.728409, 0.252943, 0.700000, 0.252943, 0.728409, 0.252943, 0.728409, 0.252943, 0.700000, 0.252943, 0.700000, 0.252943, 0.700000, 0.233063, 0.700000, 0.252943, 0.700000, 0.252943, 0.700000, 0.233063, 0.700000, 0.233063, 0.700000, 0.233063, 0.700000, 0.233337, 0.674951, 0.233063, 0.700000, 0.233063, 0.700000, 0.233337, 0.674951, 0.233337, 0.674951, 0.233337, 0.674951, 0.233877, 0.653072, 0.233337, 0.674951, 0.233337, 0.674951, 0.233877, 0.653072, 0.233877, 0.653072, 0.233877, 0.653072, 0.234682, 0.634362, 0.233877, 0.653072, 0.233877, 0.653072, 0.234682, 0.634362, 0.234682, 0.634362, 0.234682, 0.634362, 0.235754, 0.618821, 0.234682, 0.634362, 0.234682, 0.634362, 0.235754, 0.618821, 0.235754, 0.618821, 0.235754, 0.618821, 0.237076, 0.606610, 0.235754, 0.618821, 0.235754, 0.618821, 0.237076, 0.606610, 0.237076, 0.606610, 0.237076, 0.606610, 0.238635, 0.597887, 0.237076, 0.606610, 0.237076, 0.606610, 0.238635, 0.597887, 0.238635, 0.597887, 0.238635, 0.597887, 0.240431, 0.592654, 0.238635, 0.597887, 0.238635, 0.597887, 0.240431, 0.592654, 0.240431, 0.592654, 0.240431, 0.592654, 0.242463, 0.590909, 0.240431, 0.592654, 0.240431, 0.592654, 0.242463, 0.590909, 0.242463, 0.590909, 0.242463, 0.590909, 0.243699, 0.591335, 0.242463, 0.590909, 0.242463, 0.590909, 0.243699, 0.591335, 0.243699, 0.591335, 0.243699, 0.591335, 0.244915, 0.592614, 0.243699, 0.591335, 0.243699, 0.591335, 0.244915, 0.592614, 0.244915, 0.592614, 0.244915, 0.592614, 0.246113, 0.594744, 0.244915, 0.592614, 0.244915, 0.592614, 0.246113, 0.594744, 0.246113, 0.594744, 0.246113, 0.594744, 0.247292, 0.597727, 0.246113, 0.594744, 0.246113, 0.594744, 0.247292, 0.597727, 0.247292, 0.597727, 0.247292, 0.597727, 0.248459, 0.601563, 0.247292, 0.597727, 0.247292, 0.597727, 0.248459, 0.601563, 0.248459, 0.601563, 0.248459, 0.601563, 0.249621, 0.606250, 0.248459, 0.601563, 0.248459, 0.601563, 0.249621, 0.606250, 0.249621, 0.606250, 0.249621, 0.606250, 0.250779, 0.611790, 0.249621, 0.606250, 0.249621, 0.606250, 0.250779, 0.611790, 0.250779, 0.611790, 0.250779, 0.611790, 0.251933, 0.618182, 0.250779, 0.611790, 0.250779, 0.611790, 0.251933, 0.618182, 0.251933, 0.618182, 0.251933, 0.618182, 0.251933, 0.572727, 0.251933, 0.618182, 0.251933, 0.618182, 0.251933, 0.572727, 0.251933, 0.572727, 0.251933, 0.572727, 0.250762, 0.566451, 0.251933, 0.572727, 0.251933, 0.572727, 0.250762, 0.566451, 0.250762, 0.566451, 0.250762, 0.566451, 0.249577, 0.560973, 0.250762, 0.566451, 0.250762, 0.566451, 0.249577, 0.560973, 0.249577, 0.560973, 0.249577, 0.560973, 0.248377, 0.556294, 0.249577, 0.560973, 0.249577, 0.560973, 0.248377, 0.556294, 0.248377, 0.556294, 0.248377, 0.556294, 0.247162, 0.552415, 0.248377, 0.556294, 0.248377, 0.556294, 0.247162, 0.552415, 0.247162, 0.552415, 0.247162, 0.552415, 0.245936, 0.549370, 0.247162, 0.552415, 0.247162, 0.552415, 0.245936, 0.549370, 0.245936, 0.549370, 0.245936, 0.549370, 0.244701, 0.547195, 0.245936, 0.549370, 0.245936, 0.549370, 0.244701, 0.547195, 0.244701, 0.547195, 0.244701, 0.547195, 0.243457, 0.545890, 0.244701, 0.547195, 0.244701, 0.547195, 0.243457, 0.545890, 0.243457, 0.545890, 0.243457, 0.545890, 0.242204, 0.545455, 0.243457, 0.545890, 0.243457, 0.545890, 0.242204, 0.545455, 0.242204, 0.545455, 0.242204, 0.545455, 0.239219, 0.548242, 0.242204, 0.545455, 0.242204, 0.545455, 0.239219, 0.548242, 0.239219, 0.548242, 0.239219, 0.548242, 0.236562, 0.556605, 0.239219, 0.548242, 0.239219, 0.548242, 0.236562, 0.556605, 0.236562, 0.556605, 0.236562, 0.556605, 0.234231, 0.570543, 0.236562, 0.556605, 0.236562, 0.556605, 0.234231, 0.570543, 0.234231, 0.570543, 0.234231, 0.570543, 0.232229, 0.590057, 0.234231, 0.570543, 0.234231, 0.570543, 0.232229, 0.590057, 0.232229, 0.590057, 0.232229, 0.590057, 0.230620, 0.614329, 0.232229, 0.590057, 0.232229, 0.590057, 0.230620, 0.614329, 0.230620, 0.614329, 0.230620, 0.614329, 0.229471, 0.642543, 0.230620, 0.614329, 0.230620, 0.614329, 0.229471, 0.642543, 0.229471, 0.642543, 0.229471, 0.642543, 0.228781, 0.674698, 0.229471, 0.642543, 0.229471, 0.642543, 0.228781, 0.674698, 0.228781, 0.674698, 0.228781, 0.674698, 0.228551, 0.710795, 0.228781, 0.674698, 0.228781, 0.674698, 0.228551, 0.710795, 0.228551, 0.710795, 0.228551, 0.710795, 0.228769, 0.748109, 0.228551, 0.710795, 0.228551, 0.710795, 0.228769, 0.748109, 0.228769, 0.748109, 0.228769, 0.748109, 0.229424, 0.781357, 0.228769, 0.748109, 0.228769, 0.748109, 0.229424, 0.781357, 0.229424, 0.781357, 0.229424, 0.781357, 0.230514, 0.810538, 0.229424, 0.781357, 0.229424, 0.781357, 0.230514, 0.810538, 0.230514, 0.810538, 0.230514, 0.810538, 0.232041, 0.835653, 0.230514, 0.810538, 0.230514, 0.810538, 0.232041, 0.835653, 0.232041, 0.835653, 0.232041, 0.835653, 0.233938, 0.855850, 0.232041, 0.835653, 0.232041, 0.835653, 0.233938, 0.855850, 0.233938, 0.855850, 0.233938, 0.855850, 0.236139, 0.870277, 0.233938, 0.855850, 0.233938, 0.855850, 0.236139, 0.870277, 0.236139, 0.870277, 0.236139, 0.870277, 0.238643, 0.878933, 0.236139, 0.870277, 0.236139, 0.870277, 0.238643, 0.878933, 0.238643, 0.878933, 0.238643, 0.878933, 0.241452, 0.881818, 0.238643, 0.878933, 0.238643, 0.878933, 0.241452, 0.881818, 0.241452, 0.881818, 0.241452, 0.881818, 0.243969, 0.879239, 0.241452, 0.881818, 0.241452, 0.881818, 0.243969, 0.879239, 0.243969, 0.879239, 0.243969, 0.879239, 0.246208, 0.871502, 0.243969, 0.879239, 0.243969, 0.879239, 0.246208, 0.871502, 0.246208, 0.871502, 0.246208, 0.871502, 0.248169, 0.858607, 0.246208, 0.871502, 0.246208, 0.871502, 0.248169, 0.858607, 0.248169, 0.858607, 0.248169, 0.858607, 0.249853, 0.840554, 0.248169, 0.858607, 0.248169, 0.858607, 0.249853, 0.840554, 0.249853, 0.840554, 0.249853, 0.840554, 0.251205, 0.818071, 0.249853, 0.840554, 0.249853, 0.840554, 0.251205, 0.818071, 0.251205, 0.818071, 0.251205, 0.818071, 0.252171, 0.791886, 0.251205, 0.818071, 0.251205, 0.818071, 0.252171, 0.791886, 0.252171, 0.791886, 0.252171, 0.791886, 0.252750, 0.761998, 0.252171, 0.791886, 0.252171, 0.791886, 0.252750, 0.761998, 0.252750, 0.761998, 0.228769, 0.748109, 0.228551, 0.710795, 0.233204, 0.745170, 0.228551, 0.710795, 0.228769, 0.748109, 0.233204, 0.745170, 0.229424, 0.781357, 0.228769, 0.748109, 0.233204, 0.745170, 0.228769, 0.748109, 0.229424, 0.781357, 0.233204, 0.745170, 0.229424, 0.781357, 0.233204, 0.745170, 0.233484, 0.765678, 0.233204, 0.745170, 0.229424, 0.781357, 0.233484, 0.765678, 0.233994, 0.783736, 0.229424, 0.781357, 0.233484, 0.765678, 0.229424, 0.781357, 0.233994, 0.783736, 0.233484, 0.765678, 0.233994, 0.783736, 0.230514, 0.810538, 0.229424, 0.781357, 0.230514, 0.810538, 0.233994, 0.783736, 0.229424, 0.781357, 0.230514, 0.810538, 0.233994, 0.783736, 0.234735, 0.799343, 0.233994, 0.783736, 0.230514, 0.810538, 0.234735, 0.799343, 0.232041, 0.835653, 0.230514, 0.810538, 0.234735, 0.799343, 0.230514, 0.810538, 0.232041, 0.835653, 0.234735, 0.799343, 0.232041, 0.835653, 0.234735, 0.799343, 0.235707, 0.812500, 0.234735, 0.799343, 0.232041, 0.835653, 0.235707, 0.812500, 0.233938, 0.855850, 0.232041, 0.835653, 0.235707, 0.812500, 0.232041, 0.835653, 0.233938, 0.855850, 0.235707, 0.812500, 0.233938, 0.855850, 0.235707, 0.812500, 0.236884, 0.822940, 0.235707, 0.812500, 0.233938, 0.855850, 0.236884, 0.822940, 0.236139, 0.870277, 0.233938, 0.855850, 0.236884, 0.822940, 0.233938, 0.855850, 0.236139, 0.870277, 0.236884, 0.822940, 0.236139, 0.870277, 0.236884, 0.822940, 0.238242, 0.830398, 0.236884, 0.822940, 0.236139, 0.870277, 0.238242, 0.830398, 0.236139, 0.870277, 0.238242, 0.830398, 0.238643, 0.878933, 0.238242, 0.830398, 0.236139, 0.870277, 0.238643, 0.878933, 0.238643, 0.878933, 0.238242, 0.830398, 0.239780, 0.834872, 0.238242, 0.830398, 0.238643, 0.878933, 0.239780, 0.834872, 0.238643, 0.878933, 0.239780, 0.834872, 0.241452, 0.881818, 0.239780, 0.834872, 0.238643, 0.878933, 0.241452, 0.881818, 0.241452, 0.881818, 0.239780, 0.834872, 0.241499, 0.836364, 0.239780, 0.834872, 0.241452, 0.881818, 0.241499, 0.836364, 0.241452, 0.881818, 0.241499, 0.836364, 0.243019, 0.834819, 0.241499, 0.836364, 0.241452, 0.881818, 0.243019, 0.834819, 0.241452, 0.881818, 0.243019, 0.834819, 0.243969, 0.879239, 0.243019, 0.834819, 0.241452, 0.881818, 0.243969, 0.879239, 0.243969, 0.879239, 0.243019, 0.834819, 0.244381, 0.830185, 0.243019, 0.834819, 0.243969, 0.879239, 0.244381, 0.830185, 0.243969, 0.879239, 0.244381, 0.830185, 0.246208, 0.871502, 0.244381, 0.830185, 0.243969, 0.879239, 0.246208, 0.871502, 0.244381, 0.830185, 0.245586, 0.822461, 0.246208, 0.871502, 0.245586, 0.822461, 0.244381, 0.830185, 0.246208, 0.871502, 0.246208, 0.871502, 0.245586, 0.822461, 0.248169, 0.858607, 0.245586, 0.822461, 0.246208, 0.871502, 0.248169, 0.858607, 0.245586, 0.822461, 0.246634, 0.811648, 0.248169, 0.858607, 0.246634, 0.811648, 0.245586, 0.822461, 0.248169, 0.858607, 0.248169, 0.858607, 0.246634, 0.811648, 0.249853, 0.840554, 0.246634, 0.811648, 0.248169, 0.858607, 0.249853, 0.840554, 0.246634, 0.811648, 0.247485, 0.798242, 0.249853, 0.840554, 0.247485, 0.798242, 0.246634, 0.811648, 0.249853, 0.840554, 0.249853, 0.840554, 0.247485, 0.798242, 0.251205, 0.818071, 0.247485, 0.798242, 0.249853, 0.840554, 0.251205, 0.818071, 0.247485, 0.798242, 0.248100, 0.782741, 0.251205, 0.818071, 0.248100, 0.782741, 0.247485, 0.798242, 0.251205, 0.818071, 0.251205, 0.818071, 0.248100, 0.782741, 0.252171, 0.791886, 0.248100, 0.782741, 0.251205, 0.818071, 0.252171, 0.791886, 0.248100, 0.782741, 0.248478, 0.765146, 0.252171, 0.791886, 0.248478, 0.765146, 0.248100, 0.782741, 0.252171, 0.791886, 0.252171, 0.791886, 0.248478, 0.765146, 0.252750, 0.761998, 0.248478, 0.765146, 0.252171, 0.791886, 0.252750, 0.761998, 0.248478, 0.765146, 0.248619, 0.745455, 0.252750, 0.761998, 0.248619, 0.745455, 0.248478, 0.765146, 0.252750, 0.761998, 0.252750, 0.761998, 0.248619, 0.745455, 0.252943, 0.728409, 0.248619, 0.745455, 0.252750, 0.761998, 0.252943, 0.728409, 0.252943, 0.700000, 0.252943, 0.728409, 0.248619, 0.745455, 0.252943, 0.728409, 0.252943, 0.700000, 0.248619, 0.745455, 0.248619, 0.745455, 0.233063, 0.700000, 0.252943, 0.700000, 0.233063, 0.700000, 0.248619, 0.745455, 0.252943, 0.700000, 0.248619, 0.745455, 0.233204, 0.745170, 0.233063, 0.700000, 0.233204, 0.745170, 0.248619, 0.745455, 0.233063, 0.700000, 0.228551, 0.710795, 0.233063, 0.700000, 0.233204, 0.745170, 0.233063, 0.700000, 0.228551, 0.710795, 0.233204, 0.745170, 0.228551, 0.710795, 0.228781, 0.674698, 0.233063, 0.700000, 0.228781, 0.674698, 0.228551, 0.710795, 0.233063, 0.700000, 0.233063, 0.700000, 0.228781, 0.674698, 0.233337, 0.674951, 0.228781, 0.674698, 0.233063, 0.700000, 0.233337, 0.674951, 0.228781, 0.674698, 0.229471, 0.642543, 0.233337, 0.674951, 0.229471, 0.642543, 0.228781, 0.674698, 0.233337, 0.674951, 0.233337, 0.674951, 0.229471, 0.642543, 0.233877, 0.653072, 0.229471, 0.642543, 0.233337, 0.674951, 0.233877, 0.653072, 0.229471, 0.642543, 0.230620, 0.614329, 0.233877, 0.653072, 0.230620, 0.614329, 0.229471, 0.642543, 0.233877, 0.653072, 0.233877, 0.653072, 0.230620, 0.614329, 0.234682, 0.634362, 0.230620, 0.614329, 0.233877, 0.653072, 0.234682, 0.634362, 0.230620, 0.614329, 0.232229, 0.590057, 0.234682, 0.634362, 0.232229, 0.590057, 0.230620, 0.614329, 0.234682, 0.634362, 0.234682, 0.634362, 0.232229, 0.590057, 0.235754, 0.618821, 0.232229, 0.590057, 0.234682, 0.634362, 0.235754, 0.618821, 0.232229, 0.590057, 0.234231, 0.570543, 0.235754, 0.618821, 0.234231, 0.570543, 0.232229, 0.590057, 0.235754, 0.618821, 0.235754, 0.618821, 0.234231, 0.570543, 0.237076, 0.606610, 0.234231, 0.570543, 0.235754, 0.618821, 0.237076, 0.606610, 0.237076, 0.606610, 0.234231, 0.570543, 0.236562, 0.556605, 0.234231, 0.570543, 0.237076, 0.606610, 0.236562, 0.556605, 0.238635, 0.597887, 0.237076, 0.606610, 0.236562, 0.556605, 0.237076, 0.606610, 0.238635, 0.597887, 0.236562, 0.556605, 0.238635, 0.597887, 0.236562, 0.556605, 0.239219, 0.548242, 0.236562, 0.556605, 0.238635, 0.597887, 0.239219, 0.548242, 0.240431, 0.592654, 0.238635, 0.597887, 0.239219, 0.548242, 0.238635, 0.597887, 0.240431, 0.592654, 0.239219, 0.548242, 0.240431, 0.592654, 0.239219, 0.548242, 0.242204, 0.545455, 0.239219, 0.548242, 0.240431, 0.592654, 0.242204, 0.545455, 0.242463, 0.590909, 0.240431, 0.592654, 0.242204, 0.545455, 0.240431, 0.592654, 0.242463, 0.590909, 0.242204, 0.545455, 0.242463, 0.590909, 0.242204, 0.545455, 0.243457, 0.545890, 0.242204, 0.545455, 0.242463, 0.590909, 0.243457, 0.545890, 0.242463, 0.590909, 0.243457, 0.545890, 0.243699, 0.591335, 0.243457, 0.545890, 0.242463, 0.590909, 0.243699, 0.591335, 0.243699, 0.591335, 0.243457, 0.545890, 0.244701, 0.547195, 0.243457, 0.545890, 0.243699, 0.591335, 0.244701, 0.547195, 0.243699, 0.591335, 0.244701, 0.547195, 0.244915, 0.592614, 0.244701, 0.547195, 0.243699, 0.591335, 0.244915, 0.592614, 0.244915, 0.592614, 0.244701, 0.547195, 0.245936, 0.549370, 0.244701, 0.547195, 0.244915, 0.592614, 0.245936, 0.549370, 0.244915, 0.592614, 0.245936, 0.549370, 0.246113, 0.594744, 0.245936, 0.549370, 0.244915, 0.592614, 0.246113, 0.594744, 0.246113, 0.594744, 0.245936, 0.549370, 0.247162, 0.552415, 0.245936, 0.549370, 0.246113, 0.594744, 0.247162, 0.552415, 0.246113, 0.594744, 0.247162, 0.552415, 0.247292, 0.597727, 0.247162, 0.552415, 0.246113, 0.594744, 0.247292, 0.597727, 0.247292, 0.597727, 0.247162, 0.552415, 0.248377, 0.556294, 0.247162, 0.552415, 0.247292, 0.597727, 0.248377, 0.556294, 0.248459, 0.601563, 0.247292, 0.597727, 0.248377, 0.556294, 0.247292, 0.597727, 0.248459, 0.601563, 0.248377, 0.556294, 0.249577, 0.560973, 0.248459, 0.601563, 0.248377, 0.556294, 0.248459, 0.601563, 0.249577, 0.560973, 0.248377, 0.556294, 0.248459, 0.601563, 0.249577, 0.560973, 0.250762, 0.566451, 0.249577, 0.560973, 0.248459, 0.601563, 0.250762, 0.566451, 0.249621, 0.606250, 0.248459, 0.601563, 0.250762, 0.566451, 0.248459, 0.601563, 0.249621, 0.606250, 0.250762, 0.566451, 0.249621, 0.606250, 0.250762, 0.566451, 0.251933, 0.572727, 0.250762, 0.566451, 0.249621, 0.606250, 0.251933, 0.572727, 0.250779, 0.611790, 0.249621, 0.606250, 0.251933, 0.572727, 0.249621, 0.606250, 0.250779, 0.611790, 0.251933, 0.572727, 0.250779, 0.611790, 0.251933, 0.572727, 0.251933, 0.618182, 0.251933, 0.572727, 0.250779, 0.611790, 0.251933, 0.618182, 0.248619, 0.745455, 0.248478, 0.765146, 0.248619, 0.745455, 0.248619, 0.745455, 0.248478, 0.765146, 0.248478, 0.765146, 0.248478, 0.765146, 0.248100, 0.782741, 0.248478, 0.765146, 0.248478, 0.765146, 0.248100, 0.782741, 0.248100, 0.782741, 0.248100, 0.782741, 0.247485, 0.798242, 0.248100, 0.782741, 0.248100, 0.782741, 0.247485, 0.798242, 0.247485, 0.798242, 0.247485, 0.798242, 0.246634, 0.811648, 0.247485, 0.798242, 0.247485, 0.798242, 0.246634, 0.811648, 0.246634, 0.811648, 0.246634, 0.811648, 0.245586, 0.822461, 0.246634, 0.811648, 0.246634, 0.811648, 0.245586, 0.822461, 0.245586, 0.822461, 0.245586, 0.822461, 0.244381, 0.830185, 0.245586, 0.822461, 0.245586, 0.822461, 0.244381, 0.830185, 0.244381, 0.830185, 0.244381, 0.830185, 0.243019, 0.834819, 0.244381, 0.830185, 0.244381, 0.830185, 0.243019, 0.834819, 0.243019, 0.834819, 0.243019, 0.834819, 0.241499, 0.836364, 0.243019, 0.834819, 0.243019, 0.834819, 0.241499, 0.836364, 0.241499, 0.836364, 0.241499, 0.836364, 0.239780, 0.834872, 0.241499, 0.836364, 0.241499, 0.836364, 0.239780, 0.834872, 0.239780, 0.834872, 0.239780, 0.834872, 0.238242, 0.830398, 0.239780, 0.834872, 0.239780, 0.834872, 0.238242, 0.830398, 0.238242, 0.830398, 0.238242, 0.830398, 0.236884, 0.822940, 0.238242, 0.830398, 0.238242, 0.830398, 0.236884, 0.822940, 0.236884, 0.822940, 0.236884, 0.822940, 0.235707, 0.812500, 0.236884, 0.822940, 0.236884, 0.822940, 0.235707, 0.812500, 0.235707, 0.812500, 0.235707, 0.812500, 0.234735, 0.799343, 0.235707, 0.812500, 0.235707, 0.812500, 0.234735, 0.799343, 0.234735, 0.799343, 0.234735, 0.799343, 0.233994, 0.783736, 0.234735, 0.799343, 0.234735, 0.799343, 0.233994, 0.783736, 0.233994, 0.783736, 0.233994, 0.783736, 0.233484, 0.765678, 0.233994, 0.783736, 0.233994, 0.783736, 0.233484, 0.765678, 0.233484, 0.765678, 0.233484, 0.765678, 0.233204, 0.745170, 0.233484, 0.765678, 0.233484, 0.765678, 0.233204, 0.745170, 0.233204, 0.745170, 0.274797, 0.872727, 0.279121, 0.872727, 0.274797, 0.872727, 0.274797, 0.872727, 0.279121, 0.872727, 0.279121, 0.872727, 0.279121, 0.872727, 0.279121, 0.554545, 0.279121, 0.872727, 0.279121, 0.872727, 0.279121, 0.554545, 0.279121, 0.554545, 0.279121, 0.554545, 0.274797, 0.554545, 0.279121, 0.554545, 0.279121, 0.554545, 0.274797, 0.554545, 0.274797, 0.554545, 0.274797, 0.872727, 0.279121, 0.554545, 0.279121, 0.872727, 0.279121, 0.554545, 0.274797, 0.872727, 0.279121, 0.872727, 0.274797, 0.872727, 0.274797, 0.554545, 0.279121, 0.554545, 0.274797, 0.554545, 0.274797, 0.872727, 0.279121, 0.554545, 0.274797, 1, 0.279121, 1, 0.274797, 1, 0.274797, 1, 0.279121, 1, 0.279121, 1, 0.279121, 1, 0.279121, 0.936364, 0.279121, 1, 0.279121, 1, 0.279121, 0.936364, 0.279121, 0.936364, 0.279121, 0.936364, 0.274797, 0.936364, 0.279121, 0.936364, 0.279121, 0.936364, 0.274797, 0.936364, 0.274797, 0.936364, 0.274797, 1, 0.279121, 0.936364, 0.279121, 1, 0.279121, 0.936364, 0.274797, 1, 0.279121, 1, 0.274797, 1, 0.274797, 0.936364, 0.279121, 0.936364, 0.274797, 0.936364, 0.274797, 1, 0.279121, 0.936364, 0.305111, 0.854545, 0.305111, 0.809091, 0.305111, 0.854545, 0.305111, 0.854545, 0.305111, 0.809091, 0.305111, 0.809091, 0.305111, 0.809091, 0.304186, 0.815483, 0.305111, 0.809091, 0.305111, 0.809091, 0.304186, 0.815483, 0.304186, 0.815483, 0.304186, 0.815483, 0.303243, 0.821023, 0.304186, 0.815483, 0.304186, 0.815483, 0.303243, 0.821023, 0.303243, 0.821023, 0.303243, 0.821023, 0.302282, 0.825710, 0.303243, 0.821023, 0.303243, 0.821023, 0.302282, 0.825710, 0.302282, 0.825710, 0.302282, 0.825710, 0.301304, 0.829545, 0.302282, 0.825710, 0.302282, 0.825710, 0.301304, 0.829545, 0.301304, 0.829545, 0.301304, 0.829545, 0.300308, 0.832528, 0.301304, 0.829545, 0.301304, 0.829545, 0.300308, 0.832528, 0.300308, 0.832528, 0.300308, 0.832528, 0.299295, 0.834659, 0.300308, 0.832528, 0.300308, 0.832528, 0.299295, 0.834659, 0.299295, 0.834659, 0.299295, 0.834659, 0.298264, 0.835937, 0.299295, 0.834659, 0.299295, 0.834659, 0.298264, 0.835937, 0.298264, 0.835937, 0.298264, 0.835937, 0.297215, 0.836364, 0.298264, 0.835937, 0.298264, 0.835937, 0.297215, 0.836364, 0.297215, 0.836364, 0.297215, 0.836364, 0.295706, 0.835591, 0.297215, 0.836364, 0.297215, 0.836364, 0.295706, 0.835591, 0.295706, 0.835591, 0.295706, 0.835591, 0.294398, 0.833274, 0.295706, 0.835591, 0.295706, 0.835591, 0.294398, 0.833274, 0.294398, 0.833274, 0.294398, 0.833274, 0.293292, 0.829412, 0.294398, 0.833274, 0.294398, 0.833274, 0.293292, 0.829412, 0.293292, 0.829412, 0.293292, 0.829412, 0.292386, 0.824006, 0.293292, 0.829412, 0.293292, 0.829412, 0.292386, 0.824006, 0.292386, 0.824006, 0.292386, 0.824006, 0.291682, 0.817054, 0.292386, 0.824006, 0.292386, 0.824006, 0.291682, 0.817054, 0.291682, 0.817054, 0.291682, 0.817054, 0.291179, 0.808558, 0.291682, 0.817054, 0.291682, 0.817054, 0.291179, 0.808558, 0.291179, 0.808558, 0.291179, 0.808558, 0.290877, 0.798517, 0.291179, 0.808558, 0.291179, 0.808558, 0.290877, 0.798517, 0.290877, 0.798517, 0.290877, 0.798517, 0.290777, 0.786932, 0.290877, 0.798517, 0.290877, 0.798517, 0.290777, 0.786932, 0.290777, 0.786932, 0.290777, 0.786932, 0.290849, 0.778054, 0.290777, 0.786932, 0.290777, 0.786932, 0.290849, 0.778054, 0.290849, 0.778054, 0.290849, 0.778054, 0.291065, 0.770170, 0.290849, 0.778054, 0.290849, 0.778054, 0.291065, 0.770170, 0.291065, 0.770170, 0.291065, 0.770170, 0.291424, 0.763281, 0.291065, 0.770170, 0.291065, 0.770170, 0.291424, 0.763281, 0.291424, 0.763281, 0.291424, 0.763281, 0.291928, 0.757386, 0.291424, 0.763281, 0.291424, 0.763281, 0.291928, 0.757386, 0.291928, 0.757386, 0.291928, 0.757386, 0.292649, 0.752060, 0.291928, 0.757386, 0.291928, 0.757386, 0.292649, 0.752060, 0.292649, 0.752060, 0.292649, 0.752060, 0.293661, 0.746875, 0.292649, 0.752060, 0.292649, 0.752060, 0.293661, 0.746875, 0.293661, 0.746875, 0.293661, 0.746875, 0.294964, 0.741832, 0.293661, 0.746875, 0.293661, 0.746875, 0.294964, 0.741832, 0.294964, 0.741832, 0.294964, 0.741832, 0.296557, 0.736932, 0.294964, 0.741832, 0.294964, 0.741832, 0.296557, 0.736932, 0.296557, 0.736932, 0.296557, 0.736932, 0.298038, 0.732955, 0.296557, 0.736932, 0.296557, 0.736932, 0.298038, 0.732955, 0.298038, 0.732955, 0.298038, 0.732955, 0.300174, 0.726460, 0.298038, 0.732955, 0.298038, 0.732955, 0.300174, 0.726460, 0.300174, 0.726460, 0.300174, 0.726460, 0.301977, 0.718768, 0.300174, 0.726460, 0.300174, 0.726460, 0.301977, 0.718768, 0.301977, 0.718768, 0.301977, 0.718768, 0.303446, 0.709877, 0.301977, 0.718768, 0.301977, 0.718768, 0.303446, 0.709877, 0.303446, 0.709877, 0.303446, 0.709877, 0.304582, 0.699787, 0.303446, 0.709877, 0.303446, 0.709877, 0.304582, 0.699787, 0.304582, 0.699787, 0.304582, 0.699787, 0.305430, 0.688037, 0.304582, 0.699787, 0.304582, 0.699787, 0.305430, 0.688037, 0.305430, 0.688037, 0.305430, 0.688037, 0.306036, 0.674165, 0.305430, 0.688037, 0.305430, 0.688037, 0.306036, 0.674165, 0.306036, 0.674165, 0.306036, 0.674165, 0.306400, 0.658172, 0.306036, 0.674165, 0.306036, 0.674165, 0.306400, 0.658172, 0.306400, 0.658172, 0.306400, 0.658172, 0.306521, 0.640057, 0.306400, 0.658172, 0.306400, 0.658172, 0.306521, 0.640057, 0.306521, 0.640057, 0.306521, 0.640057, 0.306340, 0.619385, 0.306521, 0.640057, 0.306521, 0.640057, 0.306340, 0.619385, 0.306340, 0.619385, 0.306340, 0.619385, 0.305795, 0.600977, 0.306340, 0.619385, 0.306340, 0.619385, 0.305795, 0.600977, 0.305795, 0.600977, 0.305795, 0.600977, 0.304889, 0.584832, 0.305795, 0.600977, 0.305795, 0.600977, 0.304889, 0.584832, 0.304889, 0.584832, 0.304889, 0.584832, 0.303619, 0.570952, 0.304889, 0.584832, 0.304889, 0.584832, 0.303619, 0.570952, 0.303619, 0.570952, 0.303619, 0.570952, 0.302032, 0.559797, 0.303619, 0.570952, 0.303619, 0.570952, 0.302032, 0.559797, 0.302032, 0.559797, 0.302032, 0.559797, 0.300173, 0.551829, 0.302032, 0.559797, 0.302032, 0.559797, 0.300173, 0.551829, 0.300173, 0.551829, 0.300173, 0.551829, 0.298043, 0.547048, 0.300173, 0.551829, 0.300173, 0.551829, 0.298043, 0.547048, 0.298043, 0.547048, 0.298043, 0.547048, 0.295641, 0.545455, 0.298043, 0.547048, 0.298043, 0.547048, 0.295641, 0.545455, 0.295641, 0.545455, 0.295641, 0.545455, 0.294572, 0.545739, 0.295641, 0.545455, 0.295641, 0.545455, 0.294572, 0.545739, 0.294572, 0.545739, 0.294572, 0.545739, 0.293482, 0.546591, 0.294572, 0.545739, 0.294572, 0.545739, 0.293482, 0.546591, 0.293482, 0.546591, 0.293482, 0.546591, 0.292369, 0.548011, 0.293482, 0.546591, 0.293482, 0.546591, 0.292369, 0.548011, 0.292369, 0.548011, 0.292369, 0.548011, 0.291235, 0.550000, 0.292369, 0.548011, 0.292369, 0.548011, 0.291235, 0.550000, 0.291235, 0.550000, 0.291235, 0.550000, 0.290074, 0.552557, 0.291235, 0.550000, 0.291235, 0.550000, 0.290074, 0.552557, 0.290074, 0.552557, 0.290074, 0.552557, 0.288882, 0.555682, 0.290074, 0.552557, 0.290074, 0.552557, 0.288882, 0.555682, 0.288882, 0.555682, 0.288882, 0.555682, 0.287659, 0.559375, 0.288882, 0.555682, 0.288882, 0.555682, 0.287659, 0.559375, 0.287659, 0.559375, 0.287659, 0.559375, 0.286406, 0.563636, 0.287659, 0.559375, 0.287659, 0.559375, 0.286406, 0.563636, 0.286406, 0.563636, 0.286406, 0.563636, 0.286406, 0.618182, 0.286406, 0.563636, 0.286406, 0.563636, 0.286406, 0.618182, 0.286406, 0.618182, 0.286406, 0.618182, 0.287600, 0.611790, 0.286406, 0.618182, 0.286406, 0.618182, 0.287600, 0.611790, 0.287600, 0.611790, 0.287600, 0.611790, 0.288785, 0.606250, 0.287600, 0.611790, 0.287600, 0.611790, 0.288785, 0.606250, 0.288785, 0.606250, 0.288785, 0.606250, 0.289962, 0.601563, 0.288785, 0.606250, 0.288785, 0.606250, 0.289962, 0.601563, 0.289962, 0.601563, 0.289962, 0.601563, 0.291129, 0.597727, 0.289962, 0.601563, 0.289962, 0.601563, 0.291129, 0.597727, 0.291129, 0.597727, 0.291129, 0.597727, 0.292289, 0.594744, 0.291129, 0.597727, 0.291129, 0.597727, 0.292289, 0.594744, 0.292289, 0.594744, 0.292289, 0.594744, 0.293444, 0.592614, 0.292289, 0.594744, 0.292289, 0.594744, 0.293444, 0.592614, 0.293444, 0.592614, 0.293444, 0.592614, 0.294592, 0.591335, 0.293444, 0.592614, 0.293444, 0.592614, 0.294592, 0.591335, 0.294592, 0.591335, 0.294592, 0.591335, 0.295735, 0.590909, 0.294592, 0.591335, 0.294592, 0.591335, 0.295735, 0.590909, 0.295735, 0.590909, 0.295735, 0.590909, 0.297174, 0.591668, 0.295735, 0.590909, 0.295735, 0.590909, 0.297174, 0.591668, 0.297174, 0.591668, 0.297174, 0.591668, 0.298437, 0.593945, 0.297174, 0.591668, 0.297174, 0.591668, 0.298437, 0.593945, 0.298437, 0.593945, 0.298437, 0.593945, 0.299524, 0.597741, 0.298437, 0.593945, 0.298437, 0.593945, 0.299524, 0.597741, 0.299524, 0.597741, 0.299524, 0.597741, 0.300435, 0.603054, 0.299524, 0.597741, 0.299524, 0.597741, 0.300435, 0.603054, 0.300435, 0.603054, 0.300435, 0.603054, 0.301154, 0.609752, 0.300435, 0.603054, 0.300435, 0.603054, 0.301154, 0.609752, 0.301154, 0.609752, 0.301154, 0.609752, 0.301668, 0.617702, 0.301154, 0.609752, 0.301154, 0.609752, 0.301668, 0.617702, 0.301668, 0.617702, 0.301668, 0.617702, 0.301977, 0.626904, 0.301668, 0.617702, 0.301668, 0.617702, 0.301977, 0.626904, 0.301977, 0.626904, 0.301977, 0.626904, 0.302080, 0.637358, 0.301977, 0.626904, 0.301977, 0.626904, 0.302080, 0.637358, 0.302080, 0.637358, 0.302080, 0.637358, 0.302005, 0.646990, 0.302080, 0.637358, 0.302080, 0.637358, 0.302005, 0.646990, 0.302005, 0.646990, 0.302005, 0.646990, 0.301783, 0.655433, 0.302005, 0.646990, 0.302005, 0.646990, 0.301783, 0.655433, 0.301783, 0.655433, 0.301783, 0.655433, 0.301412, 0.662686, 0.301783, 0.655433, 0.301783, 0.655433, 0.301412, 0.662686, 0.301412, 0.662686, 0.301412, 0.662686, 0.300893, 0.668750, 0.301412, 0.662686, 0.301412, 0.662686, 0.300893, 0.668750, 0.300893, 0.668750, 0.300893, 0.668750, 0.300123, 0.674165, 0.300893, 0.668750, 0.300893, 0.668750, 0.300123, 0.674165, 0.300123, 0.674165, 0.300123, 0.674165, 0.298998, 0.679474, 0.300123, 0.674165, 0.300123, 0.674165, 0.298998, 0.679474, 0.298998, 0.679474, 0.298998, 0.679474, 0.297520, 0.684677, 0.298998, 0.679474, 0.298998, 0.679474, 0.297520, 0.684677, 0.297520, 0.684677, 0.297520, 0.684677, 0.295688, 0.689773, 0.297520, 0.684677, 0.297520, 0.684677, 0.295688, 0.689773, 0.295688, 0.689773, 0.295688, 0.689773, 0.294184, 0.694034, 0.295688, 0.689773, 0.295688, 0.689773, 0.294184, 0.694034, 0.294184, 0.694034, 0.294184, 0.694034, 0.292314, 0.699987, 0.294184, 0.694034, 0.294184, 0.694034, 0.292314, 0.699987, 0.292314, 0.699987, 0.292314, 0.699987, 0.290724, 0.707333, 0.292314, 0.699987, 0.292314, 0.699987, 0.290724, 0.707333, 0.290724, 0.707333, 0.290724, 0.707333, 0.289412, 0.716073, 0.290724, 0.707333, 0.290724, 0.707333, 0.289412, 0.716073, 0.289412, 0.716073, 0.289412, 0.716073, 0.288380, 0.726207, 0.289412, 0.716073, 0.289412, 0.716073, 0.288380, 0.726207, 0.288380, 0.726207, 0.288380, 0.726207, 0.287598, 0.738055, 0.288380, 0.726207, 0.288380, 0.726207, 0.287598, 0.738055, 0.287598, 0.738055, 0.287598, 0.738055, 0.287040, 0.751935, 0.287598, 0.738055, 0.287598, 0.738055, 0.287040, 0.751935, 0.287040, 0.751935, 0.287040, 0.751935, 0.286705, 0.767849, 0.287040, 0.751935, 0.287040, 0.751935, 0.286705, 0.767849, 0.286705, 0.767849, 0.286705, 0.767849, 0.286594, 0.785795, 0.286705, 0.767849, 0.286705, 0.767849, 0.286594, 0.785795, 0.286594, 0.785795, 0.286594, 0.785795, 0.286758, 0.807435, 0.286594, 0.785795, 0.286594, 0.785795, 0.286758, 0.807435, 0.286758, 0.807435, 0.286758, 0.807435, 0.287252, 0.826474, 0.286758, 0.807435, 0.286758, 0.807435, 0.287252, 0.826474, 0.287252, 0.826474, 0.287252, 0.826474, 0.288074, 0.842911, 0.287252, 0.826474, 0.287252, 0.826474, 0.288074, 0.842911, 0.288074, 0.842911, 0.288074, 0.842911, 0.289226, 0.856747, 0.288074, 0.842911, 0.288074, 0.842911, 0.289226, 0.856747, 0.289226, 0.856747, 0.289226, 0.856747, 0.290680, 0.867716, 0.289226, 0.856747, 0.289226, 0.856747, 0.290680, 0.867716, 0.290680, 0.867716, 0.290680, 0.867716, 0.292410, 0.875550, 0.290680, 0.867716, 0.290680, 0.867716, 0.292410, 0.875550, 0.292410, 0.875550, 0.292410, 0.875550, 0.294416, 0.880251, 0.292410, 0.875550, 0.292410, 0.875550, 0.294416, 0.880251, 0.294416, 0.880251, 0.294416, 0.880251, 0.296698, 0.881818, 0.294416, 0.880251, 0.294416, 0.880251, 0.296698, 0.881818, 0.296698, 0.881818, 0.296698, 0.881818, 0.297879, 0.881392, 0.296698, 0.881818, 0.296698, 0.881818, 0.297879, 0.881392, 0.297879, 0.881392, 0.297879, 0.881392, 0.299025, 0.880114, 0.297879, 0.881392, 0.297879, 0.881392, 0.299025, 0.880114, 0.299025, 0.880114, 0.299025, 0.880114, 0.300135, 0.877983, 0.299025, 0.880114, 0.299025, 0.880114, 0.300135, 0.877983, 0.300135, 0.877983, 0.300135, 0.877983, 0.301210, 0.875000, 0.300135, 0.877983, 0.300135, 0.877983, 0.301210, 0.875000, 0.301210, 0.875000, 0.301210, 0.875000, 0.302247, 0.871165, 0.301210, 0.875000, 0.301210, 0.875000, 0.302247, 0.871165, 0.302247, 0.871165, 0.302247, 0.871165, 0.303243, 0.866477, 0.302247, 0.871165, 0.302247, 0.871165, 0.303243, 0.866477, 0.303243, 0.866477, 0.303243, 0.866477, 0.304197, 0.860937, 0.303243, 0.866477, 0.303243, 0.866477, 0.304197, 0.860937, 0.304197, 0.860937, 0.286406, 0.618182, 0.286406, 0.563636, 0.287600, 0.611790, 0.286406, 0.563636, 0.286406, 0.618182, 0.287600, 0.611790, 0.288785, 0.606250, 0.287600, 0.611790, 0.286406, 0.563636, 0.287600, 0.611790, 0.288785, 0.606250, 0.286406, 0.563636, 0.288785, 0.606250, 0.286406, 0.563636, 0.287659, 0.559375, 0.286406, 0.563636, 0.288785, 0.606250, 0.287659, 0.559375, 0.289962, 0.601563, 0.288785, 0.606250, 0.287659, 0.559375, 0.288785, 0.606250, 0.289962, 0.601563, 0.287659, 0.559375, 0.289962, 0.601563, 0.287659, 0.559375, 0.288882, 0.555682, 0.287659, 0.559375, 0.289962, 0.601563, 0.288882, 0.555682, 0.289962, 0.601563, 0.288882, 0.555682, 0.290074, 0.552557, 0.288882, 0.555682, 0.289962, 0.601563, 0.290074, 0.552557, 0.289962, 0.601563, 0.290074, 0.552557, 0.291129, 0.597727, 0.290074, 0.552557, 0.289962, 0.601563, 0.291129, 0.597727, 0.291129, 0.597727, 0.290074, 0.552557, 0.291235, 0.550000, 0.290074, 0.552557, 0.291129, 0.597727, 0.291235, 0.550000, 0.291129, 0.597727, 0.291235, 0.550000, 0.292289, 0.594744, 0.291235, 0.550000, 0.291129, 0.597727, 0.292289, 0.594744, 0.292289, 0.594744, 0.291235, 0.550000, 0.292369, 0.548011, 0.291235, 0.550000, 0.292289, 0.594744, 0.292369, 0.548011, 0.292289, 0.594744, 0.292369, 0.548011, 0.293444, 0.592614, 0.292369, 0.548011, 0.292289, 0.594744, 0.293444, 0.592614, 0.293444, 0.592614, 0.292369, 0.548011, 0.293482, 0.546591, 0.292369, 0.548011, 0.293444, 0.592614, 0.293482, 0.546591, 0.293444, 0.592614, 0.293482, 0.546591, 0.294592, 0.591335, 0.293482, 0.546591, 0.293444, 0.592614, 0.294592, 0.591335, 0.293482, 0.546591, 0.294572, 0.545739, 0.294592, 0.591335, 0.294572, 0.545739, 0.293482, 0.546591, 0.294592, 0.591335, 0.294592, 0.591335, 0.294572, 0.545739, 0.295735, 0.590909, 0.294572, 0.545739, 0.294592, 0.591335, 0.295735, 0.590909, 0.294572, 0.545739, 0.295641, 0.545455, 0.295735, 0.590909, 0.295641, 0.545455, 0.294572, 0.545739, 0.295735, 0.590909, 0.297174, 0.591668, 0.295735, 0.590909, 0.295641, 0.545455, 0.295735, 0.590909, 0.297174, 0.591668, 0.295641, 0.545455, 0.298043, 0.547048, 0.297174, 0.591668, 0.295641, 0.545455, 0.297174, 0.591668, 0.298043, 0.547048, 0.295641, 0.545455, 0.298437, 0.593945, 0.297174, 0.591668, 0.298043, 0.547048, 0.297174, 0.591668, 0.298437, 0.593945, 0.298043, 0.547048, 0.300173, 0.551829, 0.298437, 0.593945, 0.298043, 0.547048, 0.298437, 0.593945, 0.300173, 0.551829, 0.298043, 0.547048, 0.299524, 0.597741, 0.298437, 0.593945, 0.300173, 0.551829, 0.298437, 0.593945, 0.299524, 0.597741, 0.300173, 0.551829, 0.299524, 0.597741, 0.300173, 0.551829, 0.302032, 0.559797, 0.300173, 0.551829, 0.299524, 0.597741, 0.302032, 0.559797, 0.300435, 0.603054, 0.299524, 0.597741, 0.302032, 0.559797, 0.299524, 0.597741, 0.300435, 0.603054, 0.302032, 0.559797, 0.300435, 0.603054, 0.302032, 0.559797, 0.303619, 0.570952, 0.302032, 0.559797, 0.300435, 0.603054, 0.303619, 0.570952, 0.301154, 0.609752, 0.300435, 0.603054, 0.303619, 0.570952, 0.300435, 0.603054, 0.301154, 0.609752, 0.303619, 0.570952, 0.301154, 0.609752, 0.303619, 0.570952, 0.304889, 0.584832, 0.303619, 0.570952, 0.301154, 0.609752, 0.304889, 0.584832, 0.301668, 0.617702, 0.301154, 0.609752, 0.304889, 0.584832, 0.301154, 0.609752, 0.301668, 0.617702, 0.304889, 0.584832, 0.301668, 0.617702, 0.304889, 0.584832, 0.305795, 0.600977, 0.304889, 0.584832, 0.301668, 0.617702, 0.305795, 0.600977, 0.301977, 0.626904, 0.301668, 0.617702, 0.305795, 0.600977, 0.301668, 0.617702, 0.301977, 0.626904, 0.305795, 0.600977, 0.301977, 0.626904, 0.305795, 0.600977, 0.306340, 0.619385, 0.305795, 0.600977, 0.301977, 0.626904, 0.306340, 0.619385, 0.302080, 0.637358, 0.301977, 0.626904, 0.306340, 0.619385, 0.301977, 0.626904, 0.302080, 0.637358, 0.306340, 0.619385, 0.302080, 0.637358, 0.306340, 0.619385, 0.306521, 0.640057, 0.306340, 0.619385, 0.302080, 0.637358, 0.306521, 0.640057, 0.306521, 0.640057, 0.302005, 0.646990, 0.302080, 0.637358, 0.302005, 0.646990, 0.306521, 0.640057, 0.302080, 0.637358, 0.306400, 0.658172, 0.302005, 0.646990, 0.306521, 0.640057, 0.302005, 0.646990, 0.306400, 0.658172, 0.306521, 0.640057, 0.306036, 0.674165, 0.302005, 0.646990, 0.306400, 0.658172, 0.302005, 0.646990, 0.306036, 0.674165, 0.306400, 0.658172, 0.301783, 0.655433, 0.302005, 0.646990, 0.306036, 0.674165, 0.302005, 0.646990, 0.301783, 0.655433, 0.306036, 0.674165, 0.305430, 0.688037, 0.301783, 0.655433, 0.306036, 0.674165, 0.301783, 0.655433, 0.305430, 0.688037, 0.306036, 0.674165, 0.301412, 0.662686, 0.301783, 0.655433, 0.305430, 0.688037, 0.301783, 0.655433, 0.301412, 0.662686, 0.305430, 0.688037, 0.304582, 0.699787, 0.301412, 0.662686, 0.305430, 0.688037, 0.301412, 0.662686, 0.304582, 0.699787, 0.305430, 0.688037, 0.303446, 0.709877, 0.301412, 0.662686, 0.304582, 0.699787, 0.301412, 0.662686, 0.303446, 0.709877, 0.304582, 0.699787, 0.300893, 0.668750, 0.301412, 0.662686, 0.303446, 0.709877, 0.301412, 0.662686, 0.300893, 0.668750, 0.303446, 0.709877, 0.301977, 0.718768, 0.300893, 0.668750, 0.303446, 0.709877, 0.300893, 0.668750, 0.301977, 0.718768, 0.303446, 0.709877, 0.300123, 0.674165, 0.300893, 0.668750, 0.301977, 0.718768, 0.300893, 0.668750, 0.300123, 0.674165, 0.301977, 0.718768, 0.300174, 0.726460, 0.300123, 0.674165, 0.301977, 0.718768, 0.300123, 0.674165, 0.300174, 0.726460, 0.301977, 0.718768, 0.298998, 0.679474, 0.300123, 0.674165, 0.300174, 0.726460, 0.300123, 0.674165, 0.298998, 0.679474, 0.300174, 0.726460, 0.300174, 0.726460, 0.297520, 0.684677, 0.298998, 0.679474, 0.297520, 0.684677, 0.300174, 0.726460, 0.298998, 0.679474, 0.300174, 0.726460, 0.298038, 0.732955, 0.297520, 0.684677, 0.298038, 0.732955, 0.300174, 0.726460, 0.297520, 0.684677, 0.295688, 0.689773, 0.297520, 0.684677, 0.298038, 0.732955, 0.297520, 0.684677, 0.295688, 0.689773, 0.298038, 0.732955, 0.296557, 0.736932, 0.295688, 0.689773, 0.298038, 0.732955, 0.295688, 0.689773, 0.296557, 0.736932, 0.298038, 0.732955, 0.294184, 0.694034, 0.295688, 0.689773, 0.296557, 0.736932, 0.295688, 0.689773, 0.294184, 0.694034, 0.296557, 0.736932, 0.294964, 0.741832, 0.294184, 0.694034, 0.296557, 0.736932, 0.294184, 0.694034, 0.294964, 0.741832, 0.296557, 0.736932, 0.292314, 0.699987, 0.294184, 0.694034, 0.294964, 0.741832, 0.294184, 0.694034, 0.292314, 0.699987, 0.294964, 0.741832, 0.293661, 0.746875, 0.292314, 0.699987, 0.294964, 0.741832, 0.292314, 0.699987, 0.293661, 0.746875, 0.294964, 0.741832, 0.293661, 0.746875, 0.292649, 0.752060, 0.292314, 0.699987, 0.292649, 0.752060, 0.293661, 0.746875, 0.292314, 0.699987, 0.292649, 0.752060, 0.290724, 0.707333, 0.292314, 0.699987, 0.290724, 0.707333, 0.292649, 0.752060, 0.292314, 0.699987, 0.292649, 0.752060, 0.291928, 0.757386, 0.290724, 0.707333, 0.291928, 0.757386, 0.292649, 0.752060, 0.290724, 0.707333, 0.291928, 0.757386, 0.289412, 0.716073, 0.290724, 0.707333, 0.289412, 0.716073, 0.291928, 0.757386, 0.290724, 0.707333, 0.288380, 0.726207, 0.289412, 0.716073, 0.291928, 0.757386, 0.289412, 0.716073, 0.288380, 0.726207, 0.291928, 0.757386, 0.291928, 0.757386, 0.291424, 0.763281, 0.288380, 0.726207, 0.291424, 0.763281, 0.291928, 0.757386, 0.288380, 0.726207, 0.291424, 0.763281, 0.287598, 0.738055, 0.288380, 0.726207, 0.287598, 0.738055, 0.291424, 0.763281, 0.288380, 0.726207, 0.291424, 0.763281, 0.291065, 0.770170, 0.287598, 0.738055, 0.291065, 0.770170, 0.291424, 0.763281, 0.287598, 0.738055, 0.291065, 0.770170, 0.287040, 0.751935, 0.287598, 0.738055, 0.287040, 0.751935, 0.291065, 0.770170, 0.287598, 0.738055, 0.291065, 0.770170, 0.290849, 0.778054, 0.287040, 0.751935, 0.290849, 0.778054, 0.291065, 0.770170, 0.287040, 0.751935, 0.290849, 0.778054, 0.286705, 0.767849, 0.287040, 0.751935, 0.286705, 0.767849, 0.290849, 0.778054, 0.287040, 0.751935, 0.286594, 0.785795, 0.286705, 0.767849, 0.290849, 0.778054, 0.286705, 0.767849, 0.286594, 0.785795, 0.290849, 0.778054, 0.290777, 0.786932, 0.286594, 0.785795, 0.290849, 0.778054, 0.286594, 0.785795, 0.290777, 0.786932, 0.290849, 0.778054, 0.286758, 0.807435, 0.286594, 0.785795, 0.290777, 0.786932, 0.286594, 0.785795, 0.286758, 0.807435, 0.290777, 0.786932, 0.286758, 0.807435, 0.290777, 0.786932, 0.290877, 0.798517, 0.290777, 0.786932, 0.286758, 0.807435, 0.290877, 0.798517, 0.287252, 0.826474, 0.286758, 0.807435, 0.290877, 0.798517, 0.286758, 0.807435, 0.287252, 0.826474, 0.290877, 0.798517, 0.287252, 0.826474, 0.290877, 0.798517, 0.291179, 0.808558, 0.290877, 0.798517, 0.287252, 0.826474, 0.291179, 0.808558, 0.288074, 0.842911, 0.287252, 0.826474, 0.291179, 0.808558, 0.287252, 0.826474, 0.288074, 0.842911, 0.291179, 0.808558, 0.288074, 0.842911, 0.291179, 0.808558, 0.291682, 0.817054, 0.291179, 0.808558, 0.288074, 0.842911, 0.291682, 0.817054, 0.289226, 0.856747, 0.288074, 0.842911, 0.291682, 0.817054, 0.288074, 0.842911, 0.289226, 0.856747, 0.291682, 0.817054, 0.289226, 0.856747, 0.291682, 0.817054, 0.292386, 0.824006, 0.291682, 0.817054, 0.289226, 0.856747, 0.292386, 0.824006, 0.290680, 0.867716, 0.289226, 0.856747, 0.292386, 0.824006, 0.289226, 0.856747, 0.290680, 0.867716, 0.292386, 0.824006, 0.290680, 0.867716, 0.292386, 0.824006, 0.293292, 0.829412, 0.292386, 0.824006, 0.290680, 0.867716, 0.293292, 0.829412, 0.292410, 0.875550, 0.290680, 0.867716, 0.293292, 0.829412, 0.290680, 0.867716, 0.292410, 0.875550, 0.293292, 0.829412, 0.292410, 0.875550, 0.293292, 0.829412, 0.294398, 0.833274, 0.293292, 0.829412, 0.292410, 0.875550, 0.294398, 0.833274, 0.292410, 0.875550, 0.294398, 0.833274, 0.294416, 0.880251, 0.294398, 0.833274, 0.292410, 0.875550, 0.294416, 0.880251, 0.294416, 0.880251, 0.294398, 0.833274, 0.295706, 0.835591, 0.294398, 0.833274, 0.294416, 0.880251, 0.295706, 0.835591, 0.296698, 0.881818, 0.294416, 0.880251, 0.295706, 0.835591, 0.294416, 0.880251, 0.296698, 0.881818, 0.295706, 0.835591, 0.296698, 0.881818, 0.295706, 0.835591, 0.297215, 0.836364, 0.295706, 0.835591, 0.296698, 0.881818, 0.297215, 0.836364, 0.297215, 0.836364, 0.297879, 0.881392, 0.296698, 0.881818, 0.297879, 0.881392, 0.297215, 0.836364, 0.296698, 0.881818, 0.297879, 0.881392, 0.297215, 0.836364, 0.298264, 0.835937, 0.297215, 0.836364, 0.297879, 0.881392, 0.298264, 0.835937, 0.299025, 0.880114, 0.297879, 0.881392, 0.298264, 0.835937, 0.297879, 0.881392, 0.299025, 0.880114, 0.298264, 0.835937, 0.299025, 0.880114, 0.298264, 0.835937, 0.299295, 0.834659, 0.298264, 0.835937, 0.299025, 0.880114, 0.299295, 0.834659, 0.299025, 0.880114, 0.299295, 0.834659, 0.300135, 0.877983, 0.299295, 0.834659, 0.299025, 0.880114, 0.300135, 0.877983, 0.300135, 0.877983, 0.299295, 0.834659, 0.300308, 0.832528, 0.299295, 0.834659, 0.300135, 0.877983, 0.300308, 0.832528, 0.300135, 0.877983, 0.300308, 0.832528, 0.301210, 0.875000, 0.300308, 0.832528, 0.300135, 0.877983, 0.301210, 0.875000, 0.302247, 0.871165, 0.301210, 0.875000, 0.300308, 0.832528, 0.301210, 0.875000, 0.302247, 0.871165, 0.300308, 0.832528, 0.302247, 0.871165, 0.300308, 0.832528, 0.301304, 0.829545, 0.300308, 0.832528, 0.302247, 0.871165, 0.301304, 0.829545, 0.302247, 0.871165, 0.301304, 0.829545, 0.303243, 0.866477, 0.301304, 0.829545, 0.302247, 0.871165, 0.303243, 0.866477, 0.301304, 0.829545, 0.302282, 0.825710, 0.303243, 0.866477, 0.302282, 0.825710, 0.301304, 0.829545, 0.303243, 0.866477, 0.303243, 0.866477, 0.302282, 0.825710, 0.304197, 0.860937, 0.302282, 0.825710, 0.303243, 0.866477, 0.304197, 0.860937, 0.302282, 0.825710, 0.303243, 0.821023, 0.304197, 0.860937, 0.303243, 0.821023, 0.302282, 0.825710, 0.304197, 0.860937, 0.304197, 0.860937, 0.303243, 0.821023, 0.305111, 0.854545, 0.303243, 0.821023, 0.304197, 0.860937, 0.305111, 0.854545, 0.303243, 0.821023, 0.304186, 0.815483, 0.305111, 0.854545, 0.304186, 0.815483, 0.303243, 0.821023, 0.305111, 0.854545, 0.304186, 0.815483, 0.305111, 0.809091, 0.305111, 0.854545, 0.305111, 0.809091, 0.304186, 0.815483, 0.305111, 0.854545, 0.341511, 1, 0.341511, 0.954545, 0.341511, 1, 0.341511, 1, 0.341511, 0.954545, 0.341511, 0.954545, 0.341511, 0.954545, 0.337375, 0.954545, 0.341511, 0.954545, 0.341511, 0.954545, 0.337375, 0.954545, 0.337375, 0.954545, 0.337375, 0.954545, 0.336301, 0.953822, 0.337375, 0.954545, 0.337375, 0.954545, 0.336301, 0.953822, 0.336301, 0.953822, 0.336301, 0.953822, 0.335404, 0.951651, 0.336301, 0.953822, 0.336301, 0.953822, 0.335404, 0.951651, 0.335404, 0.951651, 0.335404, 0.951651, 0.334685, 0.948034, 0.335404, 0.951651, 0.335404, 0.951651, 0.334685, 0.948034, 0.334685, 0.948034, 0.334685, 0.948034, 0.334144, 0.942969, 0.334685, 0.948034, 0.334685, 0.948034, 0.334144, 0.942969, 0.334144, 0.942969, 0.334144, 0.942969, 0.333748, 0.936022, 0.334144, 0.942969, 0.334144, 0.942969, 0.333748, 0.936022, 0.333748, 0.936022, 0.333748, 0.936022, 0.333465, 0.926758, 0.333748, 0.936022, 0.333748, 0.936022, 0.333465, 0.926758, 0.333465, 0.926758, 0.333465, 0.926758, 0.333296, 0.915177, 0.333465, 0.926758, 0.333465, 0.926758, 0.333296, 0.915177, 0.333296, 0.915177, 0.333296, 0.915177, 0.333239, 0.901278, 0.333296, 0.915177, 0.333296, 0.915177, 0.333239, 0.901278, 0.333239, 0.901278, 0.333239, 0.901278, 0.333239, 0.872727, 0.333239, 0.901278, 0.333239, 0.901278, 0.333239, 0.872727, 0.333239, 0.872727, 0.333239, 0.872727, 0.340360, 0.872727, 0.333239, 0.872727, 0.333239, 0.872727, 0.340360, 0.872727, 0.340360, 0.872727, 0.340360, 0.872727, 0.340360, 0.827273, 0.340360, 0.872727, 0.340360, 0.872727, 0.340360, 0.827273, 0.340360, 0.827273, 0.340360, 0.827273, 0.333239, 0.827273, 0.340360, 0.827273, 0.340360, 0.827273, 0.333239, 0.827273, 0.333239, 0.827273, 0.333239, 0.827273, 0.333239, 0.554545, 0.333239, 0.827273, 0.333239, 0.827273, 0.333239, 0.554545, 0.333239, 0.554545, 0.333239, 0.554545, 0.328892, 0.554545, 0.333239, 0.554545, 0.333239, 0.554545, 0.328892, 0.554545, 0.328892, 0.554545, 0.328892, 0.554545, 0.328892, 0.827273, 0.328892, 0.554545, 0.328892, 0.554545, 0.328892, 0.827273, 0.328892, 0.827273, 0.328892, 0.827273, 0.324756, 0.827273, 0.328892, 0.827273, 0.328892, 0.827273, 0.324756, 0.827273, 0.324756, 0.827273, 0.324756, 0.827273, 0.324756, 0.872727, 0.324756, 0.827273, 0.324756, 0.827273, 0.324756, 0.872727, 0.324756, 0.872727, 0.324756, 0.872727, 0.328892, 0.872727, 0.324756, 0.872727, 0.324756, 0.872727, 0.328892, 0.872727, 0.328892, 0.872727, 0.328892, 0.872727, 0.328892, 0.895312, 0.328892, 0.872727, 0.328892, 0.872727, 0.328892, 0.895312, 0.328892, 0.895312, 0.328892, 0.895312, 0.329020, 0.920801, 0.328892, 0.895312, 0.328892, 0.895312, 0.329020, 0.920801, 0.329020, 0.920801, 0.329020, 0.920801, 0.329403, 0.942578, 0.329020, 0.920801, 0.329020, 0.920801, 0.329403, 0.942578, 0.329403, 0.942578, 0.329403, 0.942578, 0.330042, 0.960644, 0.329403, 0.942578, 0.329403, 0.942578, 0.330042, 0.960644, 0.330042, 0.960644, 0.330042, 0.960644, 0.330936, 0.975000, 0.330042, 0.960644, 0.330042, 0.960644, 0.330936, 0.975000, 0.330936, 0.975000, 0.330936, 0.975000, 0.332108, 0.985937, 0.330936, 0.975000, 0.330936, 0.975000, 0.332108, 0.985937, 0.332108, 0.985937, 0.332108, 0.985937, 0.333580, 0.993750, 0.332108, 0.985937, 0.332108, 0.985937, 0.333580, 0.993750, 0.333580, 0.993750, 0.333580, 0.993750, 0.335351, 0.998437, 0.333580, 0.993750, 0.333580, 0.993750, 0.335351, 0.998437, 0.335351, 0.998437, 0.335351, 0.998437, 0.337422, 1, 0.335351, 0.998437, 0.335351, 0.998437, 0.337422, 1, 0.337422, 1, 0.324756, 0.872727, 0.328892, 0.827273, 0.328892, 0.872727, 0.328892, 0.827273, 0.324756, 0.872727, 0.328892, 0.872727, 0.324756, 0.872727, 0.324756, 0.827273, 0.328892, 0.827273, 0.324756, 0.827273, 0.324756, 0.872727, 0.328892, 0.827273, 0.328892, 0.872727, 0.328892, 0.827273, 0.333239, 0.827273, 0.328892, 0.827273, 0.328892, 0.872727, 0.333239, 0.827273, 0.328892, 0.872727, 0.333239, 0.827273, 0.333239, 0.872727, 0.333239, 0.827273, 0.328892, 0.872727, 0.333239, 0.872727, 0.328892, 0.895312, 0.328892, 0.872727, 0.333239, 0.872727, 0.328892, 0.872727, 0.328892, 0.895312, 0.333239, 0.872727, 0.328892, 0.895312, 0.333239, 0.872727, 0.333239, 0.901278, 0.333239, 0.872727, 0.328892, 0.895312, 0.333239, 0.901278, 0.329020, 0.920801, 0.328892, 0.895312, 0.333239, 0.901278, 0.328892, 0.895312, 0.329020, 0.920801, 0.333239, 0.901278, 0.329020, 0.920801, 0.333239, 0.901278, 0.333296, 0.915177, 0.333239, 0.901278, 0.329020, 0.920801, 0.333296, 0.915177, 0.333465, 0.926758, 0.329020, 0.920801, 0.333296, 0.915177, 0.329020, 0.920801, 0.333465, 0.926758, 0.333296, 0.915177, 0.333465, 0.926758, 0.329403, 0.942578, 0.329020, 0.920801, 0.329403, 0.942578, 0.333465, 0.926758, 0.329020, 0.920801, 0.333748, 0.936022, 0.329403, 0.942578, 0.333465, 0.926758, 0.329403, 0.942578, 0.333748, 0.936022, 0.333465, 0.926758, 0.333748, 0.936022, 0.330042, 0.960644, 0.329403, 0.942578, 0.330042, 0.960644, 0.333748, 0.936022, 0.329403, 0.942578, 0.330936, 0.975000, 0.330042, 0.960644, 0.333748, 0.936022, 0.330042, 0.960644, 0.330936, 0.975000, 0.333748, 0.936022, 0.330936, 0.975000, 0.333748, 0.936022, 0.334144, 0.942969, 0.333748, 0.936022, 0.330936, 0.975000, 0.334144, 0.942969, 0.332108, 0.985937, 0.330936, 0.975000, 0.334144, 0.942969, 0.330936, 0.975000, 0.332108, 0.985937, 0.334144, 0.942969, 0.332108, 0.985937, 0.334144, 0.942969, 0.334685, 0.948034, 0.334144, 0.942969, 0.332108, 0.985937, 0.334685, 0.948034, 0.333580, 0.993750, 0.332108, 0.985937, 0.334685, 0.948034, 0.332108, 0.985937, 0.333580, 0.993750, 0.334685, 0.948034, 0.333580, 0.993750, 0.334685, 0.948034, 0.335404, 0.951651, 0.334685, 0.948034, 0.333580, 0.993750, 0.335404, 0.951651, 0.335351, 0.998437, 0.333580, 0.993750, 0.335404, 0.951651, 0.333580, 0.993750, 0.335351, 0.998437, 0.335404, 0.951651, 0.335351, 0.998437, 0.335404, 0.951651, 0.336301, 0.953822, 0.335404, 0.951651, 0.335351, 0.998437, 0.336301, 0.953822, 0.335351, 0.998437, 0.336301, 0.953822, 0.337422, 1, 0.336301, 0.953822, 0.335351, 0.998437, 0.337422, 1, 0.336301, 0.953822, 0.337375, 0.954545, 0.337422, 1, 0.337375, 0.954545, 0.336301, 0.953822, 0.337422, 1, 0.337422, 1, 0.337375, 0.954545, 0.341511, 0.954545, 0.337375, 0.954545, 0.337422, 1, 0.341511, 0.954545, 0.337422, 1, 0.341511, 0.954545, 0.341511, 1, 0.341511, 0.954545, 0.337422, 1, 0.341511, 1, 0.333239, 0.872727, 0.333239, 0.827273, 0.340360, 0.827273, 0.333239, 0.827273, 0.333239, 0.872727, 0.340360, 0.827273, 0.333239, 0.872727, 0.340360, 0.827273, 0.340360, 0.872727, 0.340360, 0.827273, 0.333239, 0.872727, 0.340360, 0.872727, 0.328892, 0.827273, 0.333239, 0.554545, 0.333239, 0.827273, 0.333239, 0.554545, 0.328892, 0.827273, 0.333239, 0.827273, 0.328892, 0.827273, 0.328892, 0.554545, 0.333239, 0.554545, 0.328892, 0.554545, 0.328892, 0.827273, 0.333239, 0.554545, 0.345036, 0.677699, 0.345036, 0.872727, 0.345036, 0.677699, 0.345036, 0.677699, 0.345036, 0.872727, 0.345036, 0.872727, 0.345036, 0.872727, 0.349360, 0.872727, 0.345036, 0.872727, 0.345036, 0.872727, 0.349360, 0.872727, 0.349360, 0.872727, 0.349360, 0.872727, 0.349360, 0.679688, 0.349360, 0.872727, 0.349360, 0.872727, 0.349360, 0.679688, 0.349360, 0.679688, 0.349360, 0.679688, 0.349451, 0.658851, 0.349360, 0.679688, 0.349360, 0.679688, 0.349451, 0.658851, 0.349451, 0.658851, 0.349451, 0.658851, 0.349724, 0.640803, 0.349451, 0.658851, 0.349451, 0.658851, 0.349724, 0.640803, 0.349724, 0.640803, 0.349724, 0.640803, 0.350179, 0.625542, 0.349724, 0.640803, 0.349724, 0.640803, 0.350179, 0.625542, 0.350179, 0.625542, 0.350179, 0.625542, 0.350817, 0.613068, 0.350179, 0.625542, 0.350179, 0.625542, 0.350817, 0.613068, 0.350817, 0.613068, 0.350817, 0.613068, 0.351636, 0.603374, 0.350817, 0.613068, 0.350817, 0.613068, 0.351636, 0.603374, 0.351636, 0.603374, 0.351636, 0.603374, 0.352638, 0.596449, 0.351636, 0.603374, 0.351636, 0.603374, 0.352638, 0.596449, 0.352638, 0.596449, 0.352638, 0.596449, 0.353822, 0.592294, 0.352638, 0.596449, 0.352638, 0.596449, 0.353822, 0.592294, 0.353822, 0.592294, 0.353822, 0.592294, 0.355187, 0.590909, 0.353822, 0.592294, 0.353822, 0.592294, 0.355187, 0.590909, 0.355187, 0.590909, 0.355187, 0.590909, 0.356846, 0.592574, 0.355187, 0.590909, 0.355187, 0.590909, 0.356846, 0.592574, 0.356846, 0.592574, 0.356846, 0.592574, 0.358322, 0.597567, 0.356846, 0.592574, 0.356846, 0.592574, 0.358322, 0.597567, 0.358322, 0.597567, 0.358322, 0.597567, 0.359613, 0.605890, 0.358322, 0.597567, 0.358322, 0.597567, 0.359613, 0.605890, 0.359613, 0.605890, 0.359613, 0.605890, 0.360721, 0.617543, 0.359613, 0.605890, 0.359613, 0.605890, 0.360721, 0.617543, 0.360721, 0.617543, 0.360721, 0.617543, 0.361611, 0.632062, 0.360721, 0.617543, 0.360721, 0.617543, 0.361611, 0.632062, 0.361611, 0.632062, 0.361611, 0.632062, 0.362246, 0.648988, 0.361611, 0.632062, 0.361611, 0.632062, 0.362246, 0.648988, 0.362246, 0.648988, 0.362246, 0.648988, 0.362627, 0.668319, 0.362246, 0.648988, 0.362246, 0.648988, 0.362627, 0.668319, 0.362627, 0.668319, 0.362627, 0.668319, 0.362754, 0.690057, 0.362627, 0.668319, 0.362627, 0.668319, 0.362754, 0.690057, 0.362754, 0.690057, 0.362754, 0.690057, 0.362754, 0.872727, 0.362754, 0.690057, 0.362754, 0.690057, 0.362754, 0.872727, 0.362754, 0.872727, 0.362754, 0.872727, 0.367078, 0.872727, 0.362754, 0.872727, 0.362754, 0.872727, 0.367078, 0.872727, 0.367078, 0.872727, 0.367078, 0.872727, 0.367078, 0.554545, 0.367078, 0.872727, 0.367078, 0.872727, 0.367078, 0.554545, 0.367078, 0.554545, 0.367078, 0.554545, 0.362754, 0.554545, 0.367078, 0.554545, 0.367078, 0.554545, 0.362754, 0.554545, 0.362754, 0.554545, 0.362754, 0.554545, 0.362754, 0.600000, 0.362754, 0.554545, 0.362754, 0.554545, 0.362754, 0.600000, 0.362754, 0.600000, 0.362754, 0.600000, 0.361935, 0.587043, 0.362754, 0.600000, 0.362754, 0.600000, 0.361935, 0.587043, 0.361935, 0.587043, 0.361935, 0.587043, 0.361053, 0.575870, 0.361935, 0.587043, 0.361935, 0.587043, 0.361053, 0.575870, 0.361053, 0.575870, 0.361053, 0.575870, 0.360108, 0.566482, 0.361053, 0.575870, 0.361053, 0.575870, 0.360108, 0.566482, 0.360108, 0.566482, 0.360108, 0.566482, 0.359100, 0.558878, 0.360108, 0.566482, 0.360108, 0.566482, 0.359100, 0.558878, 0.359100, 0.558878, 0.359100, 0.558878, 0.358018, 0.553005, 0.359100, 0.558878, 0.359100, 0.558878, 0.358018, 0.553005, 0.358018, 0.553005, 0.358018, 0.553005, 0.356853, 0.548810, 0.358018, 0.553005, 0.358018, 0.553005, 0.356853, 0.548810, 0.356853, 0.548810, 0.356853, 0.548810, 0.355604, 0.546293, 0.356853, 0.548810, 0.356853, 0.548810, 0.355604, 0.546293, 0.355604, 0.546293, 0.355604, 0.546293, 0.354271, 0.545455, 0.355604, 0.546293, 0.355604, 0.546293, 0.354271, 0.545455, 0.354271, 0.545455, 0.354271, 0.545455, 0.352140, 0.547559, 0.354271, 0.545455, 0.354271, 0.545455, 0.352140, 0.547559, 0.352140, 0.547559, 0.352140, 0.547559, 0.350282, 0.553871, 0.352140, 0.547559, 0.352140, 0.547559, 0.350282, 0.553871, 0.350282, 0.553871, 0.350282, 0.553871, 0.348697, 0.564391, 0.350282, 0.553871, 0.350282, 0.553871, 0.348697, 0.564391, 0.348697, 0.564391, 0.348697, 0.564391, 0.347386, 0.579119, 0.348697, 0.564391, 0.348697, 0.564391, 0.347386, 0.579119, 0.347386, 0.579119, 0.347386, 0.579119, 0.346358, 0.597905, 0.347386, 0.579119, 0.347386, 0.579119, 0.346358, 0.597905, 0.346358, 0.597905, 0.346358, 0.597905, 0.345623, 0.620597, 0.346358, 0.597905, 0.346358, 0.597905, 0.345623, 0.620597, 0.345623, 0.620597, 0.345623, 0.620597, 0.345183, 0.647195, 0.345623, 0.620597, 0.345623, 0.620597, 0.345183, 0.647195, 0.345183, 0.647195, 0.345036, 0.872727, 0.349360, 0.679688, 0.349360, 0.872727, 0.349360, 0.679688, 0.345036, 0.872727, 0.349360, 0.872727, 0.345036, 0.872727, 0.345036, 0.677699, 0.349360, 0.679688, 0.345036, 0.677699, 0.345036, 0.872727, 0.349360, 0.679688, 0.349360, 0.679688, 0.345036, 0.677699, 0.349451, 0.658851, 0.345036, 0.677699, 0.349360, 0.679688, 0.349451, 0.658851, 0.345036, 0.677699, 0.345183, 0.647195, 0.349451, 0.658851, 0.345183, 0.647195, 0.345036, 0.677699, 0.349451, 0.658851, 0.349451, 0.658851, 0.345183, 0.647195, 0.349724, 0.640803, 0.345183, 0.647195, 0.349451, 0.658851, 0.349724, 0.640803, 0.345183, 0.647195, 0.345623, 0.620597, 0.349724, 0.640803, 0.345623, 0.620597, 0.345183, 0.647195, 0.349724, 0.640803, 0.349724, 0.640803, 0.345623, 0.620597, 0.350179, 0.625542, 0.345623, 0.620597, 0.349724, 0.640803, 0.350179, 0.625542, 0.345623, 0.620597, 0.346358, 0.597905, 0.350179, 0.625542, 0.346358, 0.597905, 0.345623, 0.620597, 0.350179, 0.625542, 0.350179, 0.625542, 0.346358, 0.597905, 0.350817, 0.613068, 0.346358, 0.597905, 0.350179, 0.625542, 0.350817, 0.613068, 0.346358, 0.597905, 0.347386, 0.579119, 0.350817, 0.613068, 0.347386, 0.579119, 0.346358, 0.597905, 0.350817, 0.613068, 0.350817, 0.613068, 0.347386, 0.579119, 0.348697, 0.564391, 0.347386, 0.579119, 0.350817, 0.613068, 0.348697, 0.564391, 0.351636, 0.603374, 0.350817, 0.613068, 0.348697, 0.564391, 0.350817, 0.613068, 0.351636, 0.603374, 0.348697, 0.564391, 0.351636, 0.603374, 0.348697, 0.564391, 0.350282, 0.553871, 0.348697, 0.564391, 0.351636, 0.603374, 0.350282, 0.553871, 0.352638, 0.596449, 0.351636, 0.603374, 0.350282, 0.553871, 0.351636, 0.603374, 0.352638, 0.596449, 0.350282, 0.553871, 0.350282, 0.553871, 0.352140, 0.547559, 0.352638, 0.596449, 0.352140, 0.547559, 0.350282, 0.553871, 0.352638, 0.596449, 0.352638, 0.596449, 0.352140, 0.547559, 0.353822, 0.592294, 0.352140, 0.547559, 0.352638, 0.596449, 0.353822, 0.592294, 0.353822, 0.592294, 0.352140, 0.547559, 0.354271, 0.545455, 0.352140, 0.547559, 0.353822, 0.592294, 0.354271, 0.545455, 0.353822, 0.592294, 0.354271, 0.545455, 0.355187, 0.590909, 0.354271, 0.545455, 0.353822, 0.592294, 0.355187, 0.590909, 0.355187, 0.590909, 0.354271, 0.545455, 0.355604, 0.546293, 0.354271, 0.545455, 0.355187, 0.590909, 0.355604, 0.546293, 0.355187, 0.590909, 0.355604, 0.546293, 0.356853, 0.548810, 0.355604, 0.546293, 0.355187, 0.590909, 0.356853, 0.548810, 0.356846, 0.592574, 0.355187, 0.590909, 0.356853, 0.548810, 0.355187, 0.590909, 0.356846, 0.592574, 0.356853, 0.548810, 0.356846, 0.592574, 0.356853, 0.548810, 0.358018, 0.553005, 0.356853, 0.548810, 0.356846, 0.592574, 0.358018, 0.553005, 0.356846, 0.592574, 0.358018, 0.553005, 0.359100, 0.558878, 0.358018, 0.553005, 0.356846, 0.592574, 0.359100, 0.558878, 0.358322, 0.597567, 0.356846, 0.592574, 0.359100, 0.558878, 0.356846, 0.592574, 0.358322, 0.597567, 0.359100, 0.558878, 0.358322, 0.597567, 0.359100, 0.558878, 0.360108, 0.566482, 0.359100, 0.558878, 0.358322, 0.597567, 0.360108, 0.566482, 0.358322, 0.597567, 0.360108, 0.566482, 0.361053, 0.575870, 0.360108, 0.566482, 0.358322, 0.597567, 0.361053, 0.575870, 0.359613, 0.605890, 0.358322, 0.597567, 0.361053, 0.575870, 0.358322, 0.597567, 0.359613, 0.605890, 0.361053, 0.575870, 0.359613, 0.605890, 0.361053, 0.575870, 0.361935, 0.587043, 0.361053, 0.575870, 0.359613, 0.605890, 0.361935, 0.587043, 0.360721, 0.617543, 0.359613, 0.605890, 0.361935, 0.587043, 0.359613, 0.605890, 0.360721, 0.617543, 0.361935, 0.587043, 0.360721, 0.617543, 0.361935, 0.587043, 0.362754, 0.600000, 0.361935, 0.587043, 0.360721, 0.617543, 0.362754, 0.600000, 0.361611, 0.632062, 0.360721, 0.617543, 0.362754, 0.600000, 0.360721, 0.617543, 0.361611, 0.632062, 0.362754, 0.600000, 0.362246, 0.648988, 0.361611, 0.632062, 0.362754, 0.600000, 0.361611, 0.632062, 0.362246, 0.648988, 0.362754, 0.600000, 0.362754, 0.600000, 0.367078, 0.554545, 0.362246, 0.648988, 0.367078, 0.554545, 0.362754, 0.600000, 0.362246, 0.648988, 0.362754, 0.600000, 0.362754, 0.554545, 0.367078, 0.554545, 0.362754, 0.554545, 0.362754, 0.600000, 0.367078, 0.554545, 0.367078, 0.554545, 0.362627, 0.668319, 0.362246, 0.648988, 0.362627, 0.668319, 0.367078, 0.554545, 0.362246, 0.648988, 0.362754, 0.690057, 0.362627, 0.668319, 0.367078, 0.554545, 0.362627, 0.668319, 0.362754, 0.690057, 0.367078, 0.554545, 0.367078, 0.872727, 0.362754, 0.690057, 0.367078, 0.554545, 0.362754, 0.690057, 0.367078, 0.872727, 0.367078, 0.554545, 0.367078, 0.872727, 0.362754, 0.872727, 0.362754, 0.690057, 0.362754, 0.872727, 0.367078, 0.872727, 0.362754, 0.690057, 0.380590, 0.963636, 0.380590, 0.872727, 0.380590, 0.963636, 0.380590, 0.963636, 0.380590, 0.872727, 0.380590, 0.872727, 0.380590, 0.872727, 0.389496, 0.872727, 0.380590, 0.872727, 0.380590, 0.872727, 0.389496, 0.872727, 0.389496, 0.872727, 0.389496, 0.872727, 0.389496, 0.827273, 0.389496, 0.872727, 0.389496, 0.872727, 0.389496, 0.827273, 0.389496, 0.827273, 0.389496, 0.827273, 0.380590, 0.827273, 0.389496, 0.827273, 0.389496, 0.827273, 0.380590, 0.827273, 0.380590, 0.827273, 0.380590, 0.827273, 0.380590, 0.659375, 0.380590, 0.827273, 0.380590, 0.827273, 0.380590, 0.659375, 0.380590, 0.659375, 0.380590, 0.659375, 0.380645, 0.642170, 0.380590, 0.659375, 0.380590, 0.659375, 0.380645, 0.642170, 0.380645, 0.642170, 0.380645, 0.642170, 0.380810, 0.628338, 0.380645, 0.642170, 0.380645, 0.642170, 0.380810, 0.628338, 0.380810, 0.628338, 0.380810, 0.628338, 0.381086, 0.617880, 0.380810, 0.628338, 0.380810, 0.628338, 0.381086, 0.617880, 0.381086, 0.617880, 0.381086, 0.617880, 0.381471, 0.610795, 0.381086, 0.617880, 0.381086, 0.617880, 0.381471, 0.610795, 0.381471, 0.610795, 0.381471, 0.610795, 0.382026, 0.606072, 0.381471, 0.610795, 0.381471, 0.610795, 0.382026, 0.606072, 0.382026, 0.606072, 0.382026, 0.606072, 0.382808, 0.602699, 0.382026, 0.606072, 0.382026, 0.606072, 0.382808, 0.602699, 0.382808, 0.602699, 0.382808, 0.602699, 0.383817, 0.600675, 0.382808, 0.602699, 0.382808, 0.602699, 0.383817, 0.600675, 0.383817, 0.600675, 0.383817, 0.600675, 0.385055, 0.600000, 0.383817, 0.600675, 0.383817, 0.600675, 0.385055, 0.600000, 0.385055, 0.600000, 0.385055, 0.600000, 0.389496, 0.600000, 0.385055, 0.600000, 0.385055, 0.600000, 0.389496, 0.600000, 0.389496, 0.600000, 0.389496, 0.600000, 0.389496, 0.554545, 0.389496, 0.600000, 0.389496, 0.600000, 0.389496, 0.554545, 0.389496, 0.554545, 0.389496, 0.554545, 0.385055, 0.554545, 0.389496, 0.554545, 0.389496, 0.554545, 0.385055, 0.554545, 0.385055, 0.554545, 0.385055, 0.554545, 0.382746, 0.555957, 0.385055, 0.554545, 0.385055, 0.554545, 0.382746, 0.555957, 0.382746, 0.555957, 0.382746, 0.555957, 0.380825, 0.560192, 0.382746, 0.555957, 0.382746, 0.555957, 0.380825, 0.560192, 0.380825, 0.560192, 0.380825, 0.560192, 0.379292, 0.567250, 0.380825, 0.560192, 0.380825, 0.560192, 0.379292, 0.567250, 0.379292, 0.567250, 0.379292, 0.567250, 0.378146, 0.577131, 0.379292, 0.567250, 0.379292, 0.567250, 0.378146, 0.577131, 0.378146, 0.577131, 0.378146, 0.577131, 0.377313, 0.590740, 0.378146, 0.577131, 0.378146, 0.577131, 0.377313, 0.590740, 0.377313, 0.590740, 0.377313, 0.590740, 0.376718, 0.608984, 0.377313, 0.590740, 0.377313, 0.590740, 0.376718, 0.608984, 0.376718, 0.608984, 0.376718, 0.608984, 0.376361, 0.631863, 0.376718, 0.608984, 0.376718, 0.608984, 0.376361, 0.631863, 0.376361, 0.631863, 0.376361, 0.631863, 0.376243, 0.659375, 0.376361, 0.631863, 0.376361, 0.631863, 0.376243, 0.659375, 0.376243, 0.659375, 0.376243, 0.659375, 0.376243, 0.827273, 0.376243, 0.659375, 0.376243, 0.659375, 0.376243, 0.827273, 0.376243, 0.827273, 0.376243, 0.827273, 0.373070, 0.827273, 0.376243, 0.827273, 0.376243, 0.827273, 0.373070, 0.827273, 0.373070, 0.827273, 0.373070, 0.827273, 0.373070, 0.872727, 0.373070, 0.827273, 0.373070, 0.827273, 0.373070, 0.872727, 0.373070, 0.872727, 0.373070, 0.872727, 0.376243, 0.872727, 0.373070, 0.872727, 0.373070, 0.872727, 0.376243, 0.872727, 0.376243, 0.872727, 0.376243, 0.872727, 0.376243, 0.963636, 0.376243, 0.872727, 0.376243, 0.872727, 0.376243, 0.963636, 0.376243, 0.963636, 0.373070, 0.872727, 0.376243, 0.827273, 0.376243, 0.872727, 0.376243, 0.827273, 0.373070, 0.872727, 0.376243, 0.872727, 0.373070, 0.872727, 0.373070, 0.827273, 0.376243, 0.827273, 0.373070, 0.827273, 0.373070, 0.872727, 0.376243, 0.827273, 0.376243, 0.872727, 0.376243, 0.827273, 0.380590, 0.827273, 0.376243, 0.827273, 0.376243, 0.872727, 0.380590, 0.827273, 0.376243, 0.872727, 0.380590, 0.827273, 0.380590, 0.872727, 0.380590, 0.827273, 0.376243, 0.872727, 0.380590, 0.872727, 0.376243, 0.963636, 0.376243, 0.872727, 0.380590, 0.872727, 0.376243, 0.872727, 0.376243, 0.963636, 0.380590, 0.872727, 0.376243, 0.963636, 0.380590, 0.872727, 0.380590, 0.963636, 0.380590, 0.872727, 0.376243, 0.963636, 0.380590, 0.963636, 0.380590, 0.872727, 0.380590, 0.827273, 0.389496, 0.827273, 0.380590, 0.827273, 0.380590, 0.872727, 0.389496, 0.827273, 0.380590, 0.872727, 0.389496, 0.827273, 0.389496, 0.872727, 0.389496, 0.827273, 0.380590, 0.872727, 0.389496, 0.872727, 0.376243, 0.827273, 0.380590, 0.659375, 0.380590, 0.827273, 0.380590, 0.659375, 0.376243, 0.827273, 0.380590, 0.827273, 0.376243, 0.827273, 0.376243, 0.659375, 0.380590, 0.659375, 0.376243, 0.659375, 0.376243, 0.827273, 0.380590, 0.659375, 0.380590, 0.659375, 0.376243, 0.659375, 0.380645, 0.642170, 0.376243, 0.659375, 0.380590, 0.659375, 0.380645, 0.642170, 0.376243, 0.659375, 0.376361, 0.631863, 0.380645, 0.642170, 0.376361, 0.631863, 0.376243, 0.659375, 0.380645, 0.642170, 0.380645, 0.642170, 0.376361, 0.631863, 0.380810, 0.628338, 0.376361, 0.631863, 0.380645, 0.642170, 0.380810, 0.628338, 0.376361, 0.631863, 0.376718, 0.608984, 0.380810, 0.628338, 0.376718, 0.608984, 0.376361, 0.631863, 0.380810, 0.628338, 0.380810, 0.628338, 0.376718, 0.608984, 0.381086, 0.617880, 0.376718, 0.608984, 0.380810, 0.628338, 0.381086, 0.617880, 0.381086, 0.617880, 0.376718, 0.608984, 0.377313, 0.590740, 0.376718, 0.608984, 0.381086, 0.617880, 0.377313, 0.590740, 0.381471, 0.610795, 0.381086, 0.617880, 0.377313, 0.590740, 0.381086, 0.617880, 0.381471, 0.610795, 0.377313, 0.590740, 0.377313, 0.590740, 0.378146, 0.577131, 0.381471, 0.610795, 0.378146, 0.577131, 0.377313, 0.590740, 0.381471, 0.610795, 0.381471, 0.610795, 0.378146, 0.577131, 0.379292, 0.567250, 0.378146, 0.577131, 0.381471, 0.610795, 0.379292, 0.567250, 0.382026, 0.606072, 0.381471, 0.610795, 0.379292, 0.567250, 0.381471, 0.610795, 0.382026, 0.606072, 0.379292, 0.567250, 0.379292, 0.567250, 0.380825, 0.560192, 0.382026, 0.606072, 0.380825, 0.560192, 0.379292, 0.567250, 0.382026, 0.606072, 0.382026, 0.606072, 0.380825, 0.560192, 0.382808, 0.602699, 0.380825, 0.560192, 0.382026, 0.606072, 0.382808, 0.602699, 0.380825, 0.560192, 0.382746, 0.555957, 0.382808, 0.602699, 0.382746, 0.555957, 0.380825, 0.560192, 0.382808, 0.602699, 0.382808, 0.602699, 0.382746, 0.555957, 0.383817, 0.600675, 0.382746, 0.555957, 0.382808, 0.602699, 0.383817, 0.600675, 0.383817, 0.600675, 0.382746, 0.555957, 0.385055, 0.554545, 0.382746, 0.555957, 0.383817, 0.600675, 0.385055, 0.554545, 0.385055, 0.600000, 0.383817, 0.600675, 0.385055, 0.554545, 0.383817, 0.600675, 0.385055, 0.600000, 0.385055, 0.554545, 0.385055, 0.600000, 0.385055, 0.554545, 0.389496, 0.554545, 0.385055, 0.554545, 0.385055, 0.600000, 0.389496, 0.554545, 0.385055, 0.600000, 0.389496, 0.554545, 0.389496, 0.600000, 0.389496, 0.554545, 0.385055, 0.600000, 0.389496, 0.600000, 0.395112, 0.872727, 0.399436, 0.872727, 0.395112, 0.872727, 0.395112, 0.872727, 0.399436, 0.872727, 0.399436, 0.872727, 0.399436, 0.872727, 0.399436, 0.554545, 0.399436, 0.872727, 0.399436, 0.872727, 0.399436, 0.554545, 0.399436, 0.554545, 0.399436, 0.554545, 0.395112, 0.554545, 0.399436, 0.554545, 0.399436, 0.554545, 0.395112, 0.554545, 0.395112, 0.554545, 0.395112, 0.872727, 0.399436, 0.554545, 0.399436, 0.872727, 0.399436, 0.554545, 0.395112, 0.872727, 0.399436, 0.872727, 0.395112, 0.872727, 0.395112, 0.554545, 0.399436, 0.554545, 0.395112, 0.554545, 0.395112, 0.872727, 0.399436, 0.554545, 0.395112, 1, 0.399436, 1, 0.395112, 1, 0.395112, 1, 0.399436, 1, 0.399436, 1, 0.399436, 1, 0.399436, 0.936364, 0.399436, 1, 0.399436, 1, 0.399436, 0.936364, 0.399436, 0.936364, 0.399436, 0.936364, 0.395112, 0.936364, 0.399436, 0.936364, 0.399436, 0.936364, 0.395112, 0.936364, 0.395112, 0.936364, 0.395112, 1, 0.399436, 0.936364, 0.399436, 1, 0.399436, 0.936364, 0.395112, 1, 0.399436, 1, 0.395112, 1, 0.395112, 0.936364, 0.399436, 0.936364, 0.395112, 0.936364, 0.395112, 1, 0.399436, 0.936364, 0.408648, 1, 0.412971, 1, 0.408648, 1, 0.408648, 1, 0.412971, 1, 0.412971, 1, 0.412971, 1, 0.412971, 0.554545, 0.412971, 1, 0.412971, 1, 0.412971, 0.554545, 0.412971, 0.554545, 0.412971, 0.554545, 0.408648, 0.554545, 0.412971, 0.554545, 0.412971, 0.554545, 0.408648, 0.554545, 0.408648, 0.554545, 0.408648, 1, 0.412971, 0.554545, 0.412971, 1, 0.412971, 0.554545, 0.408648, 1, 0.412971, 1, 0.408648, 1, 0.408648, 0.554545, 0.412971, 0.554545, 0.408648, 0.554545, 0.408648, 1, 0.412971, 0.554545, 0.444695, 0.728409, 0.444695, 0.700000, 0.444695, 0.728409, 0.444695, 0.728409, 0.444695, 0.700000, 0.444695, 0.700000, 0.444695, 0.700000, 0.424815, 0.700000, 0.444695, 0.700000, 0.444695, 0.700000, 0.424815, 0.700000, 0.424815, 0.700000, 0.424815, 0.700000, 0.425089, 0.674951, 0.424815, 0.700000, 0.424815, 0.700000, 0.425089, 0.674951, 0.425089, 0.674951, 0.425089, 0.674951, 0.425629, 0.653072, 0.425089, 0.674951, 0.425089, 0.674951, 0.425629, 0.653072, 0.425629, 0.653072, 0.425629, 0.653072, 0.426434, 0.634362, 0.425629, 0.653072, 0.425629, 0.653072, 0.426434, 0.634362, 0.426434, 0.634362, 0.426434, 0.634362, 0.427506, 0.618821, 0.426434, 0.634362, 0.426434, 0.634362, 0.427506, 0.618821, 0.427506, 0.618821, 0.427506, 0.618821, 0.428828, 0.606610, 0.427506, 0.618821, 0.427506, 0.618821, 0.428828, 0.606610, 0.428828, 0.606610, 0.428828, 0.606610, 0.430387, 0.597887, 0.428828, 0.606610, 0.428828, 0.606610, 0.430387, 0.597887, 0.430387, 0.597887, 0.430387, 0.597887, 0.432183, 0.592654, 0.430387, 0.597887, 0.430387, 0.597887, 0.432183, 0.592654, 0.432183, 0.592654, 0.432183, 0.592654, 0.434215, 0.590909, 0.432183, 0.592654, 0.432183, 0.592654, 0.434215, 0.590909, 0.434215, 0.590909, 0.434215, 0.590909, 0.435450, 0.591335, 0.434215, 0.590909, 0.434215, 0.590909, 0.435450, 0.591335, 0.435450, 0.591335, 0.435450, 0.591335, 0.436667, 0.592614, 0.435450, 0.591335, 0.435450, 0.591335, 0.436667, 0.592614, 0.436667, 0.592614, 0.436667, 0.592614, 0.437865, 0.594744, 0.436667, 0.592614, 0.436667, 0.592614, 0.437865, 0.594744, 0.437865, 0.594744, 0.437865, 0.594744, 0.439044, 0.597727, 0.437865, 0.594744, 0.437865, 0.594744, 0.439044, 0.597727, 0.439044, 0.597727, 0.439044, 0.597727, 0.440210, 0.601563, 0.439044, 0.597727, 0.439044, 0.597727, 0.440210, 0.601563, 0.440210, 0.601563, 0.440210, 0.601563, 0.441373, 0.606250, 0.440210, 0.601563, 0.440210, 0.601563, 0.441373, 0.606250, 0.441373, 0.606250, 0.441373, 0.606250, 0.442531, 0.611790, 0.441373, 0.606250, 0.441373, 0.606250, 0.442531, 0.611790, 0.442531, 0.611790, 0.442531, 0.611790, 0.443685, 0.618182, 0.442531, 0.611790, 0.442531, 0.611790, 0.443685, 0.618182, 0.443685, 0.618182, 0.443685, 0.618182, 0.443685, 0.572727, 0.443685, 0.618182, 0.443685, 0.618182, 0.443685, 0.572727, 0.443685, 0.572727, 0.443685, 0.572727, 0.442514, 0.566451, 0.443685, 0.572727, 0.443685, 0.572727, 0.442514, 0.566451, 0.442514, 0.566451, 0.442514, 0.566451, 0.441329, 0.560973, 0.442514, 0.566451, 0.442514, 0.566451, 0.441329, 0.560973, 0.441329, 0.560973, 0.441329, 0.560973, 0.440129, 0.556294, 0.441329, 0.560973, 0.441329, 0.560973, 0.440129, 0.556294, 0.440129, 0.556294, 0.440129, 0.556294, 0.438914, 0.552415, 0.440129, 0.556294, 0.440129, 0.556294, 0.438914, 0.552415, 0.438914, 0.552415, 0.438914, 0.552415, 0.437688, 0.549370, 0.438914, 0.552415, 0.438914, 0.552415, 0.437688, 0.549370, 0.437688, 0.549370, 0.437688, 0.549370, 0.436453, 0.547195, 0.437688, 0.549370, 0.437688, 0.549370, 0.436453, 0.547195, 0.436453, 0.547195, 0.436453, 0.547195, 0.435209, 0.545890, 0.436453, 0.547195, 0.436453, 0.547195, 0.435209, 0.545890, 0.435209, 0.545890, 0.435209, 0.545890, 0.433956, 0.545455, 0.435209, 0.545890, 0.435209, 0.545890, 0.433956, 0.545455, 0.433956, 0.545455, 0.433956, 0.545455, 0.430971, 0.548242, 0.433956, 0.545455, 0.433956, 0.545455, 0.430971, 0.548242, 0.430971, 0.548242, 0.430971, 0.548242, 0.428313, 0.556605, 0.430971, 0.548242, 0.430971, 0.548242, 0.428313, 0.556605, 0.428313, 0.556605, 0.428313, 0.556605, 0.425983, 0.570543, 0.428313, 0.556605, 0.428313, 0.556605, 0.425983, 0.570543, 0.425983, 0.570543, 0.425983, 0.570543, 0.423981, 0.590057, 0.425983, 0.570543, 0.425983, 0.570543, 0.423981, 0.590057, 0.423981, 0.590057, 0.423981, 0.590057, 0.422372, 0.614329, 0.423981, 0.590057, 0.423981, 0.590057, 0.422372, 0.614329, 0.422372, 0.614329, 0.422372, 0.614329, 0.421223, 0.642543, 0.422372, 0.614329, 0.422372, 0.614329, 0.421223, 0.642543, 0.421223, 0.642543, 0.421223, 0.642543, 0.420533, 0.674698, 0.421223, 0.642543, 0.421223, 0.642543, 0.420533, 0.674698, 0.420533, 0.674698, 0.420533, 0.674698, 0.420303, 0.710795, 0.420533, 0.674698, 0.420533, 0.674698, 0.420303, 0.710795, 0.420303, 0.710795, 0.420303, 0.710795, 0.420521, 0.748109, 0.420303, 0.710795, 0.420303, 0.710795, 0.420521, 0.748109, 0.420521, 0.748109, 0.420521, 0.748109, 0.421176, 0.781357, 0.420521, 0.748109, 0.420521, 0.748109, 0.421176, 0.781357, 0.421176, 0.781357, 0.421176, 0.781357, 0.422266, 0.810538, 0.421176, 0.781357, 0.421176, 0.781357, 0.422266, 0.810538, 0.422266, 0.810538, 0.422266, 0.810538, 0.423793, 0.835653, 0.422266, 0.810538, 0.422266, 0.810538, 0.423793, 0.835653, 0.423793, 0.835653, 0.423793, 0.835653, 0.425690, 0.855850, 0.423793, 0.835653, 0.423793, 0.835653, 0.425690, 0.855850, 0.425690, 0.855850, 0.425690, 0.855850, 0.427890, 0.870277, 0.425690, 0.855850, 0.425690, 0.855850, 0.427890, 0.870277, 0.427890, 0.870277, 0.427890, 0.870277, 0.430395, 0.878933, 0.427890, 0.870277, 0.427890, 0.870277, 0.430395, 0.878933, 0.430395, 0.878933, 0.430395, 0.878933, 0.433204, 0.881818, 0.430395, 0.878933, 0.430395, 0.878933, 0.433204, 0.881818, 0.433204, 0.881818, 0.433204, 0.881818, 0.435721, 0.879239, 0.433204, 0.881818, 0.433204, 0.881818, 0.435721, 0.879239, 0.435721, 0.879239, 0.435721, 0.879239, 0.437960, 0.871502, 0.435721, 0.879239, 0.435721, 0.879239, 0.437960, 0.871502, 0.437960, 0.871502, 0.437960, 0.871502, 0.439921, 0.858607, 0.437960, 0.871502, 0.437960, 0.871502, 0.439921, 0.858607, 0.439921, 0.858607, 0.439921, 0.858607, 0.441605, 0.840554, 0.439921, 0.858607, 0.439921, 0.858607, 0.441605, 0.840554, 0.441605, 0.840554, 0.441605, 0.840554, 0.442957, 0.818071, 0.441605, 0.840554, 0.441605, 0.840554, 0.442957, 0.818071, 0.442957, 0.818071, 0.442957, 0.818071, 0.443923, 0.791886, 0.442957, 0.818071, 0.442957, 0.818071, 0.443923, 0.791886, 0.443923, 0.791886, 0.443923, 0.791886, 0.444502, 0.761998, 0.443923, 0.791886, 0.443923, 0.791886, 0.444502, 0.761998, 0.444502, 0.761998, 0.420521, 0.748109, 0.420303, 0.710795, 0.424956, 0.745170, 0.420303, 0.710795, 0.420521, 0.748109, 0.424956, 0.745170, 0.421176, 0.781357, 0.420521, 0.748109, 0.424956, 0.745170, 0.420521, 0.748109, 0.421176, 0.781357, 0.424956, 0.745170, 0.421176, 0.781357, 0.424956, 0.745170, 0.425236, 0.765678, 0.424956, 0.745170, 0.421176, 0.781357, 0.425236, 0.765678, 0.425746, 0.783736, 0.421176, 0.781357, 0.425236, 0.765678, 0.421176, 0.781357, 0.425746, 0.783736, 0.425236, 0.765678, 0.425746, 0.783736, 0.422266, 0.810538, 0.421176, 0.781357, 0.422266, 0.810538, 0.425746, 0.783736, 0.421176, 0.781357, 0.422266, 0.810538, 0.425746, 0.783736, 0.426487, 0.799343, 0.425746, 0.783736, 0.422266, 0.810538, 0.426487, 0.799343, 0.423793, 0.835653, 0.422266, 0.810538, 0.426487, 0.799343, 0.422266, 0.810538, 0.423793, 0.835653, 0.426487, 0.799343, 0.423793, 0.835653, 0.426487, 0.799343, 0.427459, 0.812500, 0.426487, 0.799343, 0.423793, 0.835653, 0.427459, 0.812500, 0.425690, 0.855850, 0.423793, 0.835653, 0.427459, 0.812500, 0.423793, 0.835653, 0.425690, 0.855850, 0.427459, 0.812500, 0.425690, 0.855850, 0.427459, 0.812500, 0.428636, 0.822940, 0.427459, 0.812500, 0.425690, 0.855850, 0.428636, 0.822940, 0.427890, 0.870277, 0.425690, 0.855850, 0.428636, 0.822940, 0.425690, 0.855850, 0.427890, 0.870277, 0.428636, 0.822940, 0.427890, 0.870277, 0.428636, 0.822940, 0.429994, 0.830398, 0.428636, 0.822940, 0.427890, 0.870277, 0.429994, 0.830398, 0.427890, 0.870277, 0.429994, 0.830398, 0.430395, 0.878933, 0.429994, 0.830398, 0.427890, 0.870277, 0.430395, 0.878933, 0.430395, 0.878933, 0.429994, 0.830398, 0.431532, 0.834872, 0.429994, 0.830398, 0.430395, 0.878933, 0.431532, 0.834872, 0.430395, 0.878933, 0.431532, 0.834872, 0.433204, 0.881818, 0.431532, 0.834872, 0.430395, 0.878933, 0.433204, 0.881818, 0.433204, 0.881818, 0.431532, 0.834872, 0.433251, 0.836364, 0.431532, 0.834872, 0.433204, 0.881818, 0.433251, 0.836364, 0.433204, 0.881818, 0.433251, 0.836364, 0.434770, 0.834819, 0.433251, 0.836364, 0.433204, 0.881818, 0.434770, 0.834819, 0.433204, 0.881818, 0.434770, 0.834819, 0.435721, 0.879239, 0.434770, 0.834819, 0.433204, 0.881818, 0.435721, 0.879239, 0.435721, 0.879239, 0.434770, 0.834819, 0.436133, 0.830185, 0.434770, 0.834819, 0.435721, 0.879239, 0.436133, 0.830185, 0.435721, 0.879239, 0.436133, 0.830185, 0.437960, 0.871502, 0.436133, 0.830185, 0.435721, 0.879239, 0.437960, 0.871502, 0.436133, 0.830185, 0.437338, 0.822461, 0.437960, 0.871502, 0.437338, 0.822461, 0.436133, 0.830185, 0.437960, 0.871502, 0.437960, 0.871502, 0.437338, 0.822461, 0.439921, 0.858607, 0.437338, 0.822461, 0.437960, 0.871502, 0.439921, 0.858607, 0.437338, 0.822461, 0.438386, 0.811648, 0.439921, 0.858607, 0.438386, 0.811648, 0.437338, 0.822461, 0.439921, 0.858607, 0.439921, 0.858607, 0.438386, 0.811648, 0.441605, 0.840554, 0.438386, 0.811648, 0.439921, 0.858607, 0.441605, 0.840554, 0.438386, 0.811648, 0.439237, 0.798242, 0.441605, 0.840554, 0.439237, 0.798242, 0.438386, 0.811648, 0.441605, 0.840554, 0.441605, 0.840554, 0.439237, 0.798242, 0.442957, 0.818071, 0.439237, 0.798242, 0.441605, 0.840554, 0.442957, 0.818071, 0.439237, 0.798242, 0.439851, 0.782741, 0.442957, 0.818071, 0.439851, 0.782741, 0.439237, 0.798242, 0.442957, 0.818071, 0.442957, 0.818071, 0.439851, 0.782741, 0.443923, 0.791886, 0.439851, 0.782741, 0.442957, 0.818071, 0.443923, 0.791886, 0.439851, 0.782741, 0.440230, 0.765146, 0.443923, 0.791886, 0.440230, 0.765146, 0.439851, 0.782741, 0.443923, 0.791886, 0.443923, 0.791886, 0.440230, 0.765146, 0.444502, 0.761998, 0.440230, 0.765146, 0.443923, 0.791886, 0.444502, 0.761998, 0.440230, 0.765146, 0.440371, 0.745455, 0.444502, 0.761998, 0.440371, 0.745455, 0.440230, 0.765146, 0.444502, 0.761998, 0.444502, 0.761998, 0.440371, 0.745455, 0.444695, 0.728409, 0.440371, 0.745455, 0.444502, 0.761998, 0.444695, 0.728409, 0.444695, 0.700000, 0.444695, 0.728409, 0.440371, 0.745455, 0.444695, 0.728409, 0.444695, 0.700000, 0.440371, 0.745455, 0.440371, 0.745455, 0.424815, 0.700000, 0.444695, 0.700000, 0.424815, 0.700000, 0.440371, 0.745455, 0.444695, 0.700000, 0.440371, 0.745455, 0.424956, 0.745170, 0.424815, 0.700000, 0.424956, 0.745170, 0.440371, 0.745455, 0.424815, 0.700000, 0.420303, 0.710795, 0.424815, 0.700000, 0.424956, 0.745170, 0.424815, 0.700000, 0.420303, 0.710795, 0.424956, 0.745170, 0.420303, 0.710795, 0.420533, 0.674698, 0.424815, 0.700000, 0.420533, 0.674698, 0.420303, 0.710795, 0.424815, 0.700000, 0.424815, 0.700000, 0.420533, 0.674698, 0.425089, 0.674951, 0.420533, 0.674698, 0.424815, 0.700000, 0.425089, 0.674951, 0.420533, 0.674698, 0.421223, 0.642543, 0.425089, 0.674951, 0.421223, 0.642543, 0.420533, 0.674698, 0.425089, 0.674951, 0.425089, 0.674951, 0.421223, 0.642543, 0.425629, 0.653072, 0.421223, 0.642543, 0.425089, 0.674951, 0.425629, 0.653072, 0.421223, 0.642543, 0.422372, 0.614329, 0.425629, 0.653072, 0.422372, 0.614329, 0.421223, 0.642543, 0.425629, 0.653072, 0.425629, 0.653072, 0.422372, 0.614329, 0.426434, 0.634362, 0.422372, 0.614329, 0.425629, 0.653072, 0.426434, 0.634362, 0.422372, 0.614329, 0.423981, 0.590057, 0.426434, 0.634362, 0.423981, 0.590057, 0.422372, 0.614329, 0.426434, 0.634362, 0.426434, 0.634362, 0.423981, 0.590057, 0.427506, 0.618821, 0.423981, 0.590057, 0.426434, 0.634362, 0.427506, 0.618821, 0.423981, 0.590057, 0.425983, 0.570543, 0.427506, 0.618821, 0.425983, 0.570543, 0.423981, 0.590057, 0.427506, 0.618821, 0.427506, 0.618821, 0.425983, 0.570543, 0.428828, 0.606610, 0.425983, 0.570543, 0.427506, 0.618821, 0.428828, 0.606610, 0.428828, 0.606610, 0.425983, 0.570543, 0.428313, 0.556605, 0.425983, 0.570543, 0.428828, 0.606610, 0.428313, 0.556605, 0.430387, 0.597887, 0.428828, 0.606610, 0.428313, 0.556605, 0.428828, 0.606610, 0.430387, 0.597887, 0.428313, 0.556605, 0.430387, 0.597887, 0.428313, 0.556605, 0.430971, 0.548242, 0.428313, 0.556605, 0.430387, 0.597887, 0.430971, 0.548242, 0.432183, 0.592654, 0.430387, 0.597887, 0.430971, 0.548242, 0.430387, 0.597887, 0.432183, 0.592654, 0.430971, 0.548242, 0.432183, 0.592654, 0.430971, 0.548242, 0.433956, 0.545455, 0.430971, 0.548242, 0.432183, 0.592654, 0.433956, 0.545455, 0.434215, 0.590909, 0.432183, 0.592654, 0.433956, 0.545455, 0.432183, 0.592654, 0.434215, 0.590909, 0.433956, 0.545455, 0.434215, 0.590909, 0.433956, 0.545455, 0.435209, 0.545890, 0.433956, 0.545455, 0.434215, 0.590909, 0.435209, 0.545890, 0.434215, 0.590909, 0.435209, 0.545890, 0.435450, 0.591335, 0.435209, 0.545890, 0.434215, 0.590909, 0.435450, 0.591335, 0.435450, 0.591335, 0.435209, 0.545890, 0.436453, 0.547195, 0.435209, 0.545890, 0.435450, 0.591335, 0.436453, 0.547195, 0.435450, 0.591335, 0.436453, 0.547195, 0.436667, 0.592614, 0.436453, 0.547195, 0.435450, 0.591335, 0.436667, 0.592614, 0.436667, 0.592614, 0.436453, 0.547195, 0.437688, 0.549370, 0.436453, 0.547195, 0.436667, 0.592614, 0.437688, 0.549370, 0.436667, 0.592614, 0.437688, 0.549370, 0.437865, 0.594744, 0.437688, 0.549370, 0.436667, 0.592614, 0.437865, 0.594744, 0.437865, 0.594744, 0.437688, 0.549370, 0.438914, 0.552415, 0.437688, 0.549370, 0.437865, 0.594744, 0.438914, 0.552415, 0.437865, 0.594744, 0.438914, 0.552415, 0.439044, 0.597727, 0.438914, 0.552415, 0.437865, 0.594744, 0.439044, 0.597727, 0.439044, 0.597727, 0.438914, 0.552415, 0.440129, 0.556294, 0.438914, 0.552415, 0.439044, 0.597727, 0.440129, 0.556294, 0.440210, 0.601563, 0.439044, 0.597727, 0.440129, 0.556294, 0.439044, 0.597727, 0.440210, 0.601563, 0.440129, 0.556294, 0.441329, 0.560973, 0.440210, 0.601563, 0.440129, 0.556294, 0.440210, 0.601563, 0.441329, 0.560973, 0.440129, 0.556294, 0.440210, 0.601563, 0.441329, 0.560973, 0.442514, 0.566451, 0.441329, 0.560973, 0.440210, 0.601563, 0.442514, 0.566451, 0.441373, 0.606250, 0.440210, 0.601563, 0.442514, 0.566451, 0.440210, 0.601563, 0.441373, 0.606250, 0.442514, 0.566451, 0.441373, 0.606250, 0.442514, 0.566451, 0.443685, 0.572727, 0.442514, 0.566451, 0.441373, 0.606250, 0.443685, 0.572727, 0.442531, 0.611790, 0.441373, 0.606250, 0.443685, 0.572727, 0.441373, 0.606250, 0.442531, 0.611790, 0.443685, 0.572727, 0.442531, 0.611790, 0.443685, 0.572727, 0.443685, 0.618182, 0.443685, 0.572727, 0.442531, 0.611790, 0.443685, 0.618182, 0.440371, 0.745455, 0.440230, 0.765146, 0.440371, 0.745455, 0.440371, 0.745455, 0.440230, 0.765146, 0.440230, 0.765146, 0.440230, 0.765146, 0.439851, 0.782741, 0.440230, 0.765146, 0.440230, 0.765146, 0.439851, 0.782741, 0.439851, 0.782741, 0.439851, 0.782741, 0.439237, 0.798242, 0.439851, 0.782741, 0.439851, 0.782741, 0.439237, 0.798242, 0.439237, 0.798242, 0.439237, 0.798242, 0.438386, 0.811648, 0.439237, 0.798242, 0.439237, 0.798242, 0.438386, 0.811648, 0.438386, 0.811648, 0.438386, 0.811648, 0.437338, 0.822461, 0.438386, 0.811648, 0.438386, 0.811648, 0.437338, 0.822461, 0.437338, 0.822461, 0.437338, 0.822461, 0.436133, 0.830185, 0.437338, 0.822461, 0.437338, 0.822461, 0.436133, 0.830185, 0.436133, 0.830185, 0.436133, 0.830185, 0.434770, 0.834819, 0.436133, 0.830185, 0.436133, 0.830185, 0.434770, 0.834819, 0.434770, 0.834819, 0.434770, 0.834819, 0.433251, 0.836364, 0.434770, 0.834819, 0.434770, 0.834819, 0.433251, 0.836364, 0.433251, 0.836364, 0.433251, 0.836364, 0.431532, 0.834872, 0.433251, 0.836364, 0.433251, 0.836364, 0.431532, 0.834872, 0.431532, 0.834872, 0.431532, 0.834872, 0.429994, 0.830398, 0.431532, 0.834872, 0.431532, 0.834872, 0.429994, 0.830398, 0.429994, 0.830398, 0.429994, 0.830398, 0.428636, 0.822940, 0.429994, 0.830398, 0.429994, 0.830398, 0.428636, 0.822940, 0.428636, 0.822940, 0.428636, 0.822940, 0.427459, 0.812500, 0.428636, 0.822940, 0.428636, 0.822940, 0.427459, 0.812500, 0.427459, 0.812500, 0.427459, 0.812500, 0.426487, 0.799343, 0.427459, 0.812500, 0.427459, 0.812500, 0.426487, 0.799343, 0.426487, 0.799343, 0.426487, 0.799343, 0.425746, 0.783736, 0.426487, 0.799343, 0.426487, 0.799343, 0.425746, 0.783736, 0.425746, 0.783736, 0.425746, 0.783736, 0.425236, 0.765678, 0.425746, 0.783736, 0.425746, 0.783736, 0.425236, 0.765678, 0.425236, 0.765678, 0.425236, 0.765678, 0.424956, 0.745170, 0.425236, 0.765678, 0.425236, 0.765678, 0.424956, 0.745170, 0.424956, 0.745170, 0.452121, 0.627273, 0.457079, 0.627273, 0.452121, 0.627273, 0.452121, 0.627273, 0.457079, 0.627273, 0.457079, 0.627273, 0.457079, 0.627273, 0.457079, 0.554545, 0.457079, 0.627273, 0.457079, 0.627273, 0.457079, 0.554545, 0.457079, 0.554545, 0.457079, 0.554545, 0.452121, 0.554545, 0.457079, 0.554545, 0.457079, 0.554545, 0.452121, 0.554545, 0.452121, 0.554545, 0.452121, 0.627273, 0.457079, 0.554545, 0.457079, 0.627273, 0.457079, 0.554545, 0.452121, 0.627273, 0.457079, 0.627273, 0.452121, 0.627273, 0.452121, 0.554545, 0.457079, 0.554545, 0.452121, 0.554545, 0.452121, 0.627273, 0.457079, 0.554545, 0.476959, 0.981818, 0.482059, 0.981818, 0.476959, 0.981818, 0.476959, 0.981818, 0.482059, 0.981818, 0.482059, 0.981818, 0.482059, 0.981818, 0.491787, 0.806108, 0.482059, 0.981818, 0.482059, 0.981818, 0.491787, 0.806108, 0.491787, 0.806108, 0.491787, 0.806108, 0.501445, 0.981818, 0.491787, 0.806108, 0.491787, 0.806108, 0.501445, 0.981818, 0.501445, 0.981818, 0.501445, 0.981818, 0.506544, 0.981818, 0.501445, 0.981818, 0.501445, 0.981818, 0.506544, 0.981818, 0.506544, 0.981818, 0.506544, 0.981818, 0.494137, 0.757955, 0.506544, 0.981818, 0.506544, 0.981818, 0.494137, 0.757955, 0.494137, 0.757955, 0.494137, 0.757955, 0.494137, 0.554545, 0.494137, 0.757955, 0.494137, 0.757955, 0.494137, 0.554545, 0.494137, 0.554545, 0.494137, 0.554545, 0.489367, 0.554545, 0.494137, 0.554545, 0.494137, 0.554545, 0.489367, 0.554545, 0.489367, 0.554545, 0.489367, 0.554545, 0.489367, 0.757955, 0.489367, 0.554545, 0.489367, 0.554545, 0.489367, 0.757955, 0.489367, 0.757955, 0.482059, 0.981818, 0.476959, 0.981818, 0.489367, 0.757955, 0.476959, 0.981818, 0.482059, 0.981818, 0.489367, 0.757955, 0.491787, 0.806108, 0.482059, 0.981818, 0.489367, 0.757955, 0.482059, 0.981818, 0.491787, 0.806108, 0.489367, 0.757955, 0.491787, 0.806108, 0.489367, 0.757955, 0.494137, 0.757955, 0.489367, 0.757955, 0.491787, 0.806108, 0.494137, 0.757955, 0.491787, 0.806108, 0.494137, 0.757955, 0.501445, 0.981818, 0.494137, 0.757955, 0.491787, 0.806108, 0.501445, 0.981818, 0.506544, 0.981818, 0.501445, 0.981818, 0.494137, 0.757955, 0.501445, 0.981818, 0.506544, 0.981818, 0.494137, 0.757955, 0.489367, 0.757955, 0.494137, 0.554545, 0.494137, 0.757955, 0.494137, 0.554545, 0.489367, 0.757955, 0.494137, 0.757955, 0.489367, 0.757955, 0.489367, 0.554545, 0.494137, 0.554545, 0.489367, 0.554545, 0.489367, 0.757955, 0.494137, 0.554545, 0.515098, 0.836364, 0.513450, 0.834313, 0.515098, 0.836364, 0.515098, 0.836364, 0.513450, 0.834313, 0.513450, 0.834313, 0.513450, 0.834313, 0.511985, 0.828161, 0.513450, 0.834313, 0.513450, 0.834313, 0.511985, 0.828161, 0.511985, 0.828161, 0.511985, 0.828161, 0.510701, 0.817907, 0.511985, 0.828161, 0.511985, 0.828161, 0.510701, 0.817907, 0.510701, 0.817907, 0.510701, 0.817907, 0.509599, 0.803551, 0.510701, 0.817907, 0.510701, 0.817907, 0.509599, 0.803551, 0.509599, 0.803551, 0.509599, 0.803551, 0.508715, 0.785627, 0.509599, 0.803551, 0.509599, 0.803551, 0.508715, 0.785627, 0.508715, 0.785627, 0.508715, 0.785627, 0.508084, 0.764666, 0.508715, 0.785627, 0.508715, 0.785627, 0.508084, 0.764666, 0.508084, 0.764666, 0.508084, 0.764666, 0.507705, 0.740669, 0.508084, 0.764666, 0.508084, 0.764666, 0.507705, 0.740669, 0.507705, 0.740669, 0.507705, 0.740669, 0.507578, 0.713636, 0.507705, 0.740669, 0.507705, 0.740669, 0.507578, 0.713636, 0.507578, 0.713636, 0.507578, 0.713636, 0.507704, 0.686603, 0.507578, 0.713636, 0.507578, 0.713636, 0.507704, 0.686603, 0.507704, 0.686603, 0.507704, 0.686603, 0.508081, 0.662607, 0.507704, 0.686603, 0.507704, 0.686603, 0.508081, 0.662607, 0.508081, 0.662607, 0.508081, 0.662607, 0.508709, 0.641646, 0.508081, 0.662607, 0.508081, 0.662607, 0.508709, 0.641646, 0.508709, 0.641646, 0.508709, 0.641646, 0.509588, 0.623722, 0.508709, 0.641646, 0.508709, 0.641646, 0.509588, 0.623722, 0.509588, 0.623722, 0.509588, 0.623722, 0.510686, 0.609366, 0.509588, 0.623722, 0.509588, 0.623722, 0.510686, 0.609366, 0.510686, 0.609366, 0.510686, 0.609366, 0.511970, 0.599112, 0.510686, 0.609366, 0.510686, 0.609366, 0.511970, 0.599112, 0.511970, 0.599112, 0.511970, 0.599112, 0.513441, 0.592960, 0.511970, 0.599112, 0.511970, 0.599112, 0.513441, 0.592960, 0.513441, 0.592960, 0.513441, 0.592960, 0.515098, 0.590909, 0.513441, 0.592960, 0.513441, 0.592960, 0.515098, 0.590909, 0.515098, 0.590909, 0.515098, 0.590909, 0.516736, 0.592969, 0.515098, 0.590909, 0.515098, 0.590909, 0.516736, 0.592969, 0.516736, 0.592969, 0.516736, 0.592969, 0.518194, 0.599148, 0.516736, 0.592969, 0.516736, 0.592969, 0.518194, 0.599148, 0.518194, 0.599148, 0.518194, 0.599148, 0.519473, 0.609446, 0.518194, 0.599148, 0.518194, 0.599148, 0.519473, 0.609446, 0.519473, 0.609446, 0.519473, 0.609446, 0.520573, 0.623864, 0.519473, 0.609446, 0.519473, 0.609446, 0.520573, 0.623864, 0.520573, 0.623864, 0.520573, 0.623864, 0.521458, 0.641832, 0.520573, 0.623864, 0.520573, 0.623864, 0.521458, 0.641832, 0.521458, 0.641832, 0.521458, 0.641832, 0.522089, 0.662784, 0.521458, 0.641832, 0.521458, 0.641832, 0.522089, 0.662784, 0.522089, 0.662784, 0.522089, 0.662784, 0.522468, 0.686719, 0.522089, 0.662784, 0.522089, 0.662784, 0.522468, 0.686719, 0.522468, 0.686719, 0.522468, 0.686719, 0.522594, 0.713636, 0.522468, 0.686719, 0.522468, 0.686719, 0.522594, 0.713636, 0.522594, 0.713636, 0.522594, 0.713636, 0.522468, 0.740439, 0.522594, 0.713636, 0.522594, 0.713636, 0.522468, 0.740439, 0.522468, 0.740439, 0.522468, 0.740439, 0.522089, 0.764311, 0.522468, 0.740439, 0.522468, 0.740439, 0.522089, 0.764311, 0.522089, 0.764311, 0.522089, 0.764311, 0.521458, 0.785254, 0.522089, 0.764311, 0.522089, 0.764311, 0.521458, 0.785254, 0.521458, 0.785254, 0.521458, 0.785254, 0.520573, 0.803267, 0.521458, 0.785254, 0.521458, 0.785254, 0.520573, 0.803267, 0.520573, 0.803267, 0.520573, 0.803267, 0.519473, 0.817747, 0.520573, 0.803267, 0.520573, 0.803267, 0.519473, 0.817747, 0.519473, 0.817747, 0.519473, 0.817747, 0.518194, 0.828090, 0.519473, 0.817747, 0.519473, 0.817747, 0.518194, 0.828090, 0.518194, 0.828090, 0.518194, 0.828090, 0.516736, 0.834295, 0.518194, 0.828090, 0.518194, 0.828090, 0.516736, 0.834295, 0.516736, 0.834295, 0.515098, 0.881818, 0.517767, 0.879031, 0.515098, 0.881818, 0.515098, 0.881818, 0.517767, 0.879031, 0.517767, 0.879031, 0.517767, 0.879031, 0.520133, 0.870668, 0.517767, 0.879031, 0.517767, 0.879031, 0.520133, 0.870668, 0.520133, 0.870668, 0.520133, 0.870668, 0.522196, 0.856729, 0.520133, 0.870668, 0.520133, 0.870668, 0.522196, 0.856729, 0.522196, 0.856729, 0.522196, 0.856729, 0.523957, 0.837216, 0.522196, 0.856729, 0.522196, 0.856729, 0.523957, 0.837216, 0.523957, 0.837216, 0.523957, 0.837216, 0.525366, 0.812766, 0.523957, 0.837216, 0.523957, 0.837216, 0.525366, 0.812766, 0.525366, 0.812766, 0.525366, 0.812766, 0.526372, 0.784020, 0.525366, 0.812766, 0.525366, 0.812766, 0.526372, 0.784020, 0.526372, 0.784020, 0.526372, 0.784020, 0.526975, 0.750977, 0.526372, 0.784020, 0.526372, 0.784020, 0.526975, 0.750977, 0.526975, 0.750977, 0.526975, 0.750977, 0.527177, 0.713636, 0.526975, 0.750977, 0.526975, 0.750977, 0.527177, 0.713636, 0.527177, 0.713636, 0.527177, 0.713636, 0.526975, 0.676412, 0.527177, 0.713636, 0.527177, 0.713636, 0.526975, 0.676412, 0.526975, 0.676412, 0.526975, 0.676412, 0.526372, 0.643430, 0.526975, 0.676412, 0.526975, 0.676412, 0.526372, 0.643430, 0.526372, 0.643430, 0.526372, 0.643430, 0.525366, 0.614693, 0.526372, 0.643430, 0.526372, 0.643430, 0.525366, 0.614693, 0.525366, 0.614693, 0.525366, 0.614693, 0.523957, 0.590199, 0.525366, 0.614693, 0.525366, 0.614693, 0.523957, 0.590199, 0.523957, 0.590199, 0.523957, 0.590199, 0.522196, 0.570623, 0.523957, 0.590199, 0.523957, 0.590199, 0.522196, 0.570623, 0.522196, 0.570623, 0.522196, 0.570623, 0.520133, 0.556641, 0.522196, 0.570623, 0.522196, 0.570623, 0.520133, 0.556641, 0.520133, 0.556641, 0.520133, 0.556641, 0.517767, 0.548251, 0.520133, 0.556641, 0.520133, 0.556641, 0.517767, 0.548251, 0.517767, 0.548251, 0.517767, 0.548251, 0.515098, 0.545455, 0.517767, 0.548251, 0.517767, 0.548251, 0.515098, 0.545455, 0.515098, 0.545455, 0.515098, 0.545455, 0.512420, 0.548251, 0.515098, 0.545455, 0.515098, 0.545455, 0.512420, 0.548251, 0.512420, 0.548251, 0.512420, 0.548251, 0.510049, 0.556641, 0.512420, 0.548251, 0.512420, 0.548251, 0.510049, 0.556641, 0.510049, 0.556641, 0.510049, 0.556641, 0.507985, 0.570623, 0.510049, 0.556641, 0.510049, 0.556641, 0.507985, 0.570623, 0.507985, 0.570623, 0.507985, 0.570623, 0.506227, 0.590199, 0.507985, 0.570623, 0.507985, 0.570623, 0.506227, 0.590199, 0.506227, 0.590199, 0.506227, 0.590199, 0.504824, 0.614693, 0.506227, 0.590199, 0.506227, 0.590199, 0.504824, 0.614693, 0.504824, 0.614693, 0.504824, 0.614693, 0.503822, 0.643430, 0.504824, 0.614693, 0.504824, 0.614693, 0.503822, 0.643430, 0.503822, 0.643430, 0.503822, 0.643430, 0.503220, 0.676412, 0.503822, 0.643430, 0.503822, 0.643430, 0.503220, 0.676412, 0.503220, 0.676412, 0.503220, 0.676412, 0.503020, 0.713636, 0.503220, 0.676412, 0.503220, 0.676412, 0.503020, 0.713636, 0.503020, 0.713636, 0.503020, 0.713636, 0.503220, 0.750977, 0.503020, 0.713636, 0.503020, 0.713636, 0.503220, 0.750977, 0.503220, 0.750977, 0.503220, 0.750977, 0.503822, 0.784020, 0.503220, 0.750977, 0.503220, 0.750977, 0.503822, 0.784020, 0.503822, 0.784020, 0.503822, 0.784020, 0.504824, 0.812766, 0.503822, 0.784020, 0.503822, 0.784020, 0.504824, 0.812766, 0.504824, 0.812766, 0.504824, 0.812766, 0.506227, 0.837216, 0.504824, 0.812766, 0.504824, 0.812766, 0.506227, 0.837216, 0.506227, 0.837216, 0.506227, 0.837216, 0.507985, 0.856729, 0.506227, 0.837216, 0.506227, 0.837216, 0.507985, 0.856729, 0.507985, 0.856729, 0.507985, 0.856729, 0.510049, 0.870668, 0.507985, 0.856729, 0.507985, 0.856729, 0.510049, 0.870668, 0.510049, 0.870668, 0.510049, 0.870668, 0.512420, 0.879031, 0.510049, 0.870668, 0.510049, 0.870668, 0.512420, 0.879031, 0.512420, 0.879031, 0.507705, 0.740669, 0.503220, 0.750977, 0.503020, 0.713636, 0.503220, 0.750977, 0.507705, 0.740669, 0.503020, 0.713636, 0.508084, 0.764666, 0.503220, 0.750977, 0.507705, 0.740669, 0.503220, 0.750977, 0.508084, 0.764666, 0.507705, 0.740669, 0.508084, 0.764666, 0.503822, 0.784020, 0.503220, 0.750977, 0.503822, 0.784020, 0.508084, 0.764666, 0.503220, 0.750977, 0.503822, 0.784020, 0.508084, 0.764666, 0.508715, 0.785627, 0.508084, 0.764666, 0.503822, 0.784020, 0.508715, 0.785627, 0.504824, 0.812766, 0.503822, 0.784020, 0.508715, 0.785627, 0.503822, 0.784020, 0.504824, 0.812766, 0.508715, 0.785627, 0.509599, 0.803551, 0.504824, 0.812766, 0.508715, 0.785627, 0.504824, 0.812766, 0.509599, 0.803551, 0.508715, 0.785627, 0.509599, 0.803551, 0.506227, 0.837216, 0.504824, 0.812766, 0.506227, 0.837216, 0.509599, 0.803551, 0.504824, 0.812766, 0.507985, 0.856729, 0.506227, 0.837216, 0.509599, 0.803551, 0.506227, 0.837216, 0.507985, 0.856729, 0.509599, 0.803551, 0.507985, 0.856729, 0.509599, 0.803551, 0.510701, 0.817907, 0.509599, 0.803551, 0.507985, 0.856729, 0.510701, 0.817907, 0.510049, 0.870668, 0.507985, 0.856729, 0.510701, 0.817907, 0.507985, 0.856729, 0.510049, 0.870668, 0.510701, 0.817907, 0.510049, 0.870668, 0.510701, 0.817907, 0.511985, 0.828161, 0.510701, 0.817907, 0.510049, 0.870668, 0.511985, 0.828161, 0.510049, 0.870668, 0.511985, 0.828161, 0.512420, 0.879031, 0.511985, 0.828161, 0.510049, 0.870668, 0.512420, 0.879031, 0.512420, 0.879031, 0.511985, 0.828161, 0.513450, 0.834313, 0.511985, 0.828161, 0.512420, 0.879031, 0.513450, 0.834313, 0.512420, 0.879031, 0.513450, 0.834313, 0.515098, 0.881818, 0.513450, 0.834313, 0.512420, 0.879031, 0.515098, 0.881818, 0.513450, 0.834313, 0.515098, 0.836364, 0.515098, 0.881818, 0.515098, 0.836364, 0.513450, 0.834313, 0.515098, 0.881818, 0.515098, 0.881818, 0.515098, 0.836364, 0.516736, 0.834295, 0.515098, 0.836364, 0.515098, 0.881818, 0.516736, 0.834295, 0.515098, 0.881818, 0.516736, 0.834295, 0.517767, 0.879031, 0.516736, 0.834295, 0.515098, 0.881818, 0.517767, 0.879031, 0.517767, 0.879031, 0.516736, 0.834295, 0.518194, 0.828090, 0.516736, 0.834295, 0.517767, 0.879031, 0.518194, 0.828090, 0.517767, 0.879031, 0.518194, 0.828090, 0.520133, 0.870668, 0.518194, 0.828090, 0.517767, 0.879031, 0.520133, 0.870668, 0.518194, 0.828090, 0.519473, 0.817747, 0.520133, 0.870668, 0.519473, 0.817747, 0.518194, 0.828090, 0.520133, 0.870668, 0.520133, 0.870668, 0.519473, 0.817747, 0.522196, 0.856729, 0.519473, 0.817747, 0.520133, 0.870668, 0.522196, 0.856729, 0.519473, 0.817747, 0.520573, 0.803267, 0.522196, 0.856729, 0.520573, 0.803267, 0.519473, 0.817747, 0.522196, 0.856729, 0.522196, 0.856729, 0.520573, 0.803267, 0.523957, 0.837216, 0.520573, 0.803267, 0.522196, 0.856729, 0.523957, 0.837216, 0.523957, 0.837216, 0.520573, 0.803267, 0.525366, 0.812766, 0.520573, 0.803267, 0.523957, 0.837216, 0.525366, 0.812766, 0.525366, 0.812766, 0.520573, 0.803267, 0.521458, 0.785254, 0.520573, 0.803267, 0.525366, 0.812766, 0.521458, 0.785254, 0.525366, 0.812766, 0.521458, 0.785254, 0.526372, 0.784020, 0.521458, 0.785254, 0.525366, 0.812766, 0.526372, 0.784020, 0.521458, 0.785254, 0.522089, 0.764311, 0.526372, 0.784020, 0.522089, 0.764311, 0.521458, 0.785254, 0.526372, 0.784020, 0.526372, 0.784020, 0.522089, 0.764311, 0.526975, 0.750977, 0.522089, 0.764311, 0.526372, 0.784020, 0.526975, 0.750977, 0.522089, 0.764311, 0.522468, 0.740439, 0.526975, 0.750977, 0.522468, 0.740439, 0.522089, 0.764311, 0.526975, 0.750977, 0.526975, 0.750977, 0.522468, 0.740439, 0.527177, 0.713636, 0.522468, 0.740439, 0.526975, 0.750977, 0.527177, 0.713636, 0.527177, 0.713636, 0.522468, 0.740439, 0.522594, 0.713636, 0.522468, 0.740439, 0.527177, 0.713636, 0.522594, 0.713636, 0.527177, 0.713636, 0.522594, 0.713636, 0.522468, 0.686719, 0.522594, 0.713636, 0.527177, 0.713636, 0.522468, 0.686719, 0.527177, 0.713636, 0.522468, 0.686719, 0.526975, 0.676412, 0.522468, 0.686719, 0.527177, 0.713636, 0.526975, 0.676412, 0.522468, 0.686719, 0.522089, 0.662784, 0.526975, 0.676412, 0.522089, 0.662784, 0.522468, 0.686719, 0.526975, 0.676412, 0.522089, 0.662784, 0.526372, 0.643430, 0.526975, 0.676412, 0.526372, 0.643430, 0.522089, 0.662784, 0.526975, 0.676412, 0.522089, 0.662784, 0.521458, 0.641832, 0.526372, 0.643430, 0.521458, 0.641832, 0.522089, 0.662784, 0.526372, 0.643430, 0.521458, 0.641832, 0.525366, 0.614693, 0.526372, 0.643430, 0.525366, 0.614693, 0.521458, 0.641832, 0.526372, 0.643430, 0.521458, 0.641832, 0.520573, 0.623864, 0.525366, 0.614693, 0.520573, 0.623864, 0.521458, 0.641832, 0.525366, 0.614693, 0.520573, 0.623864, 0.523957, 0.590199, 0.525366, 0.614693, 0.523957, 0.590199, 0.520573, 0.623864, 0.525366, 0.614693, 0.523957, 0.590199, 0.520573, 0.623864, 0.522196, 0.570623, 0.520573, 0.623864, 0.523957, 0.590199, 0.522196, 0.570623, 0.520573, 0.623864, 0.519473, 0.609446, 0.522196, 0.570623, 0.519473, 0.609446, 0.520573, 0.623864, 0.522196, 0.570623, 0.522196, 0.570623, 0.519473, 0.609446, 0.520133, 0.556641, 0.519473, 0.609446, 0.522196, 0.570623, 0.520133, 0.556641, 0.519473, 0.609446, 0.518194, 0.599148, 0.520133, 0.556641, 0.518194, 0.599148, 0.519473, 0.609446, 0.520133, 0.556641, 0.520133, 0.556641, 0.518194, 0.599148, 0.517767, 0.548251, 0.518194, 0.599148, 0.520133, 0.556641, 0.517767, 0.548251, 0.518194, 0.599148, 0.516736, 0.592969, 0.517767, 0.548251, 0.516736, 0.592969, 0.518194, 0.599148, 0.517767, 0.548251, 0.517767, 0.548251, 0.516736, 0.592969, 0.515098, 0.545455, 0.516736, 0.592969, 0.517767, 0.548251, 0.515098, 0.545455, 0.516736, 0.592969, 0.515098, 0.590909, 0.515098, 0.545455, 0.515098, 0.590909, 0.516736, 0.592969, 0.515098, 0.545455, 0.515098, 0.590909, 0.513441, 0.592960, 0.515098, 0.545455, 0.513441, 0.592960, 0.515098, 0.590909, 0.515098, 0.545455, 0.513441, 0.592960, 0.512420, 0.548251, 0.515098, 0.545455, 0.512420, 0.548251, 0.513441, 0.592960, 0.515098, 0.545455, 0.513441, 0.592960, 0.511970, 0.599112, 0.512420, 0.548251, 0.511970, 0.599112, 0.513441, 0.592960, 0.512420, 0.548251, 0.511970, 0.599112, 0.510049, 0.556641, 0.512420, 0.548251, 0.510049, 0.556641, 0.511970, 0.599112, 0.512420, 0.548251, 0.511970, 0.599112, 0.510686, 0.609366, 0.510049, 0.556641, 0.510686, 0.609366, 0.511970, 0.599112, 0.510049, 0.556641, 0.510686, 0.609366, 0.507985, 0.570623, 0.510049, 0.556641, 0.507985, 0.570623, 0.510686, 0.609366, 0.510049, 0.556641, 0.510686, 0.609366, 0.509588, 0.623722, 0.507985, 0.570623, 0.509588, 0.623722, 0.510686, 0.609366, 0.507985, 0.570623, 0.509588, 0.623722, 0.506227, 0.590199, 0.507985, 0.570623, 0.506227, 0.590199, 0.509588, 0.623722, 0.507985, 0.570623, 0.504824, 0.614693, 0.506227, 0.590199, 0.509588, 0.623722, 0.506227, 0.590199, 0.504824, 0.614693, 0.509588, 0.623722, 0.508709, 0.641646, 0.504824, 0.614693, 0.509588, 0.623722, 0.504824, 0.614693, 0.508709, 0.641646, 0.509588, 0.623722, 0.503822, 0.643430, 0.504824, 0.614693, 0.508709, 0.641646, 0.504824, 0.614693, 0.503822, 0.643430, 0.508709, 0.641646, 0.508081, 0.662607, 0.503822, 0.643430, 0.508709, 0.641646, 0.503822, 0.643430, 0.508081, 0.662607, 0.508709, 0.641646, 0.503220, 0.676412, 0.503822, 0.643430, 0.508081, 0.662607, 0.503822, 0.643430, 0.503220, 0.676412, 0.508081, 0.662607, 0.507704, 0.686603, 0.503220, 0.676412, 0.508081, 0.662607, 0.503220, 0.676412, 0.507704, 0.686603, 0.508081, 0.662607, 0.503020, 0.713636, 0.503220, 0.676412, 0.507704, 0.686603, 0.503220, 0.676412, 0.503020, 0.713636, 0.507704, 0.686603, 0.507578, 0.713636, 0.503020, 0.713636, 0.507704, 0.686603, 0.503020, 0.713636, 0.507578, 0.713636, 0.507704, 0.686603, 0.507705, 0.740669, 0.503020, 0.713636, 0.507578, 0.713636, 0.503020, 0.713636, 0.507705, 0.740669, 0.507578, 0.713636, 0.533780, 0.677699, 0.533780, 0.872727, 0.533780, 0.677699, 0.533780, 0.677699, 0.533780, 0.872727, 0.533780, 0.872727, 0.533780, 0.872727, 0.538104, 0.872727, 0.533780, 0.872727, 0.533780, 0.872727, 0.538104, 0.872727, 0.538104, 0.872727, 0.538104, 0.872727, 0.538104, 0.679688, 0.538104, 0.872727, 0.538104, 0.872727, 0.538104, 0.679688, 0.538104, 0.679688, 0.538104, 0.679688, 0.538195, 0.658851, 0.538104, 0.679688, 0.538104, 0.679688, 0.538195, 0.658851, 0.538195, 0.658851, 0.538195, 0.658851, 0.538468, 0.640803, 0.538195, 0.658851, 0.538195, 0.658851, 0.538468, 0.640803, 0.538468, 0.640803, 0.538468, 0.640803, 0.538923, 0.625542, 0.538468, 0.640803, 0.538468, 0.640803, 0.538923, 0.625542, 0.538923, 0.625542, 0.538923, 0.625542, 0.539561, 0.613068, 0.538923, 0.625542, 0.538923, 0.625542, 0.539561, 0.613068, 0.539561, 0.613068, 0.539561, 0.613068, 0.540380, 0.603374, 0.539561, 0.613068, 0.539561, 0.613068, 0.540380, 0.603374, 0.540380, 0.603374, 0.540380, 0.603374, 0.541382, 0.596449, 0.540380, 0.603374, 0.540380, 0.603374, 0.541382, 0.596449, 0.541382, 0.596449, 0.541382, 0.596449, 0.542566, 0.592294, 0.541382, 0.596449, 0.541382, 0.596449, 0.542566, 0.592294, 0.542566, 0.592294, 0.542566, 0.592294, 0.543931, 0.590909, 0.542566, 0.592294, 0.542566, 0.592294, 0.543931, 0.590909, 0.543931, 0.590909, 0.543931, 0.590909, 0.545590, 0.592574, 0.543931, 0.590909, 0.543931, 0.590909, 0.545590, 0.592574, 0.545590, 0.592574, 0.545590, 0.592574, 0.547066, 0.597567, 0.545590, 0.592574, 0.545590, 0.592574, 0.547066, 0.597567, 0.547066, 0.597567, 0.547066, 0.597567, 0.548357, 0.605890, 0.547066, 0.597567, 0.547066, 0.597567, 0.548357, 0.605890, 0.548357, 0.605890, 0.548357, 0.605890, 0.549465, 0.617543, 0.548357, 0.605890, 0.548357, 0.605890, 0.549465, 0.617543, 0.549465, 0.617543, 0.549465, 0.617543, 0.550355, 0.632062, 0.549465, 0.617543, 0.549465, 0.617543, 0.550355, 0.632062, 0.550355, 0.632062, 0.550355, 0.632062, 0.550990, 0.648988, 0.550355, 0.632062, 0.550355, 0.632062, 0.550990, 0.648988, 0.550990, 0.648988, 0.550990, 0.648988, 0.551371, 0.668319, 0.550990, 0.648988, 0.550990, 0.648988, 0.551371, 0.668319, 0.551371, 0.668319, 0.551371, 0.668319, 0.551498, 0.690057, 0.551371, 0.668319, 0.551371, 0.668319, 0.551498, 0.690057, 0.551498, 0.690057, 0.551498, 0.690057, 0.551498, 0.872727, 0.551498, 0.690057, 0.551498, 0.690057, 0.551498, 0.872727, 0.551498, 0.872727, 0.551498, 0.872727, 0.555822, 0.872727, 0.551498, 0.872727, 0.551498, 0.872727, 0.555822, 0.872727, 0.555822, 0.872727, 0.555822, 0.872727, 0.555822, 0.554545, 0.555822, 0.872727, 0.555822, 0.872727, 0.555822, 0.554545, 0.555822, 0.554545, 0.555822, 0.554545, 0.551498, 0.554545, 0.555822, 0.554545, 0.555822, 0.554545, 0.551498, 0.554545, 0.551498, 0.554545, 0.551498, 0.554545, 0.551498, 0.600000, 0.551498, 0.554545, 0.551498, 0.554545, 0.551498, 0.600000, 0.551498, 0.600000, 0.551498, 0.600000, 0.550679, 0.587043, 0.551498, 0.600000, 0.551498, 0.600000, 0.550679, 0.587043, 0.550679, 0.587043, 0.550679, 0.587043, 0.549797, 0.575870, 0.550679, 0.587043, 0.550679, 0.587043, 0.549797, 0.575870, 0.549797, 0.575870, 0.549797, 0.575870, 0.548852, 0.566482, 0.549797, 0.575870, 0.549797, 0.575870, 0.548852, 0.566482, 0.548852, 0.566482, 0.548852, 0.566482, 0.547844, 0.558878, 0.548852, 0.566482, 0.548852, 0.566482, 0.547844, 0.558878, 0.547844, 0.558878, 0.547844, 0.558878, 0.546762, 0.553005, 0.547844, 0.558878, 0.547844, 0.558878, 0.546762, 0.553005, 0.546762, 0.553005, 0.546762, 0.553005, 0.545597, 0.548810, 0.546762, 0.553005, 0.546762, 0.553005, 0.545597, 0.548810, 0.545597, 0.548810, 0.545597, 0.548810, 0.544348, 0.546293, 0.545597, 0.548810, 0.545597, 0.548810, 0.544348, 0.546293, 0.544348, 0.546293, 0.544348, 0.546293, 0.543015, 0.545455, 0.544348, 0.546293, 0.544348, 0.546293, 0.543015, 0.545455, 0.543015, 0.545455, 0.543015, 0.545455, 0.540884, 0.547559, 0.543015, 0.545455, 0.543015, 0.545455, 0.540884, 0.547559, 0.540884, 0.547559, 0.540884, 0.547559, 0.539026, 0.553871, 0.540884, 0.547559, 0.540884, 0.547559, 0.539026, 0.553871, 0.539026, 0.553871, 0.539026, 0.553871, 0.537441, 0.564391, 0.539026, 0.553871, 0.539026, 0.553871, 0.537441, 0.564391, 0.537441, 0.564391, 0.537441, 0.564391, 0.536130, 0.579119, 0.537441, 0.564391, 0.537441, 0.564391, 0.536130, 0.579119, 0.536130, 0.579119, 0.536130, 0.579119, 0.535102, 0.597905, 0.536130, 0.579119, 0.536130, 0.579119, 0.535102, 0.597905, 0.535102, 0.597905, 0.535102, 0.597905, 0.534367, 0.620597, 0.535102, 0.597905, 0.535102, 0.597905, 0.534367, 0.620597, 0.534367, 0.620597, 0.534367, 0.620597, 0.533927, 0.647195, 0.534367, 0.620597, 0.534367, 0.620597, 0.533927, 0.647195, 0.533927, 0.647195, 0.533780, 0.872727, 0.538104, 0.679688, 0.538104, 0.872727, 0.538104, 0.679688, 0.533780, 0.872727, 0.538104, 0.872727, 0.533780, 0.872727, 0.533780, 0.677699, 0.538104, 0.679688, 0.533780, 0.677699, 0.533780, 0.872727, 0.538104, 0.679688, 0.538104, 0.679688, 0.533780, 0.677699, 0.538195, 0.658851, 0.533780, 0.677699, 0.538104, 0.679688, 0.538195, 0.658851, 0.533780, 0.677699, 0.533927, 0.647195, 0.538195, 0.658851, 0.533927, 0.647195, 0.533780, 0.677699, 0.538195, 0.658851, 0.538195, 0.658851, 0.533927, 0.647195, 0.538468, 0.640803, 0.533927, 0.647195, 0.538195, 0.658851, 0.538468, 0.640803, 0.533927, 0.647195, 0.534367, 0.620597, 0.538468, 0.640803, 0.534367, 0.620597, 0.533927, 0.647195, 0.538468, 0.640803, 0.538468, 0.640803, 0.534367, 0.620597, 0.538923, 0.625542, 0.534367, 0.620597, 0.538468, 0.640803, 0.538923, 0.625542, 0.534367, 0.620597, 0.535102, 0.597905, 0.538923, 0.625542, 0.535102, 0.597905, 0.534367, 0.620597, 0.538923, 0.625542, 0.538923, 0.625542, 0.535102, 0.597905, 0.539561, 0.613068, 0.535102, 0.597905, 0.538923, 0.625542, 0.539561, 0.613068, 0.535102, 0.597905, 0.536130, 0.579119, 0.539561, 0.613068, 0.536130, 0.579119, 0.535102, 0.597905, 0.539561, 0.613068, 0.539561, 0.613068, 0.536130, 0.579119, 0.537441, 0.564391, 0.536130, 0.579119, 0.539561, 0.613068, 0.537441, 0.564391, 0.540380, 0.603374, 0.539561, 0.613068, 0.537441, 0.564391, 0.539561, 0.613068, 0.540380, 0.603374, 0.537441, 0.564391, 0.540380, 0.603374, 0.537441, 0.564391, 0.539026, 0.553871, 0.537441, 0.564391, 0.540380, 0.603374, 0.539026, 0.553871, 0.541382, 0.596449, 0.540380, 0.603374, 0.539026, 0.553871, 0.540380, 0.603374, 0.541382, 0.596449, 0.539026, 0.553871, 0.539026, 0.553871, 0.540884, 0.547559, 0.541382, 0.596449, 0.540884, 0.547559, 0.539026, 0.553871, 0.541382, 0.596449, 0.541382, 0.596449, 0.540884, 0.547559, 0.542566, 0.592294, 0.540884, 0.547559, 0.541382, 0.596449, 0.542566, 0.592294, 0.542566, 0.592294, 0.540884, 0.547559, 0.543015, 0.545455, 0.540884, 0.547559, 0.542566, 0.592294, 0.543015, 0.545455, 0.542566, 0.592294, 0.543015, 0.545455, 0.543931, 0.590909, 0.543015, 0.545455, 0.542566, 0.592294, 0.543931, 0.590909, 0.543931, 0.590909, 0.543015, 0.545455, 0.544348, 0.546293, 0.543015, 0.545455, 0.543931, 0.590909, 0.544348, 0.546293, 0.543931, 0.590909, 0.544348, 0.546293, 0.545597, 0.548810, 0.544348, 0.546293, 0.543931, 0.590909, 0.545597, 0.548810, 0.545590, 0.592574, 0.543931, 0.590909, 0.545597, 0.548810, 0.543931, 0.590909, 0.545590, 0.592574, 0.545597, 0.548810, 0.545590, 0.592574, 0.545597, 0.548810, 0.546762, 0.553005, 0.545597, 0.548810, 0.545590, 0.592574, 0.546762, 0.553005, 0.545590, 0.592574, 0.546762, 0.553005, 0.547844, 0.558878, 0.546762, 0.553005, 0.545590, 0.592574, 0.547844, 0.558878, 0.547066, 0.597567, 0.545590, 0.592574, 0.547844, 0.558878, 0.545590, 0.592574, 0.547066, 0.597567, 0.547844, 0.558878, 0.547066, 0.597567, 0.547844, 0.558878, 0.548852, 0.566482, 0.547844, 0.558878, 0.547066, 0.597567, 0.548852, 0.566482, 0.547066, 0.597567, 0.548852, 0.566482, 0.549797, 0.575870, 0.548852, 0.566482, 0.547066, 0.597567, 0.549797, 0.575870, 0.548357, 0.605890, 0.547066, 0.597567, 0.549797, 0.575870, 0.547066, 0.597567, 0.548357, 0.605890, 0.549797, 0.575870, 0.548357, 0.605890, 0.549797, 0.575870, 0.550679, 0.587043, 0.549797, 0.575870, 0.548357, 0.605890, 0.550679, 0.587043, 0.549465, 0.617543, 0.548357, 0.605890, 0.550679, 0.587043, 0.548357, 0.605890, 0.549465, 0.617543, 0.550679, 0.587043, 0.549465, 0.617543, 0.550679, 0.587043, 0.551498, 0.600000, 0.550679, 0.587043, 0.549465, 0.617543, 0.551498, 0.600000, 0.550355, 0.632062, 0.549465, 0.617543, 0.551498, 0.600000, 0.549465, 0.617543, 0.550355, 0.632062, 0.551498, 0.600000, 0.550990, 0.648988, 0.550355, 0.632062, 0.551498, 0.600000, 0.550355, 0.632062, 0.550990, 0.648988, 0.551498, 0.600000, 0.551498, 0.600000, 0.555822, 0.554545, 0.550990, 0.648988, 0.555822, 0.554545, 0.551498, 0.600000, 0.550990, 0.648988, 0.551498, 0.600000, 0.551498, 0.554545, 0.555822, 0.554545, 0.551498, 0.554545, 0.551498, 0.600000, 0.555822, 0.554545, 0.555822, 0.554545, 0.551371, 0.668319, 0.550990, 0.648988, 0.551371, 0.668319, 0.555822, 0.554545, 0.550990, 0.648988, 0.551498, 0.690057, 0.551371, 0.668319, 0.555822, 0.554545, 0.551371, 0.668319, 0.551498, 0.690057, 0.555822, 0.554545, 0.555822, 0.872727, 0.551498, 0.690057, 0.555822, 0.554545, 0.551498, 0.690057, 0.555822, 0.872727, 0.555822, 0.554545, 0.555822, 0.872727, 0.551498, 0.872727, 0.551498, 0.690057, 0.551498, 0.872727, 0.555822, 0.872727, 0.551498, 0.690057, 0.577582, 0.872727, 0.581906, 0.872727, 0.577582, 0.872727, 0.577582, 0.872727, 0.581906, 0.872727, 0.581906, 0.872727, 0.581906, 0.872727, 0.587310, 0.624432, 0.581906, 0.872727, 0.581906, 0.872727, 0.587310, 0.624432, 0.587310, 0.624432, 0.587310, 0.624432, 0.592692, 0.872727, 0.587310, 0.624432, 0.587310, 0.624432, 0.592692, 0.872727, 0.592692, 0.872727, 0.592692, 0.872727, 0.597791, 0.872727, 0.592692, 0.872727, 0.592692, 0.872727, 0.597791, 0.872727, 0.597791, 0.872727, 0.597791, 0.872727, 0.603196, 0.624432, 0.597791, 0.872727, 0.597791, 0.872727, 0.603196, 0.624432, 0.603196, 0.624432, 0.603196, 0.624432, 0.608577, 0.872727, 0.603196, 0.624432, 0.603196, 0.624432, 0.608577, 0.872727, 0.608577, 0.872727, 0.608577, 0.872727, 0.612901, 0.872727, 0.608577, 0.872727, 0.608577, 0.872727, 0.612901, 0.872727, 0.612901, 0.872727, 0.612901, 0.872727, 0.606016, 0.554545, 0.612901, 0.872727, 0.612901, 0.872727, 0.606016, 0.554545, 0.606016, 0.554545, 0.606016, 0.554545, 0.600917, 0.554545, 0.606016, 0.554545, 0.606016, 0.554545, 0.600917, 0.554545, 0.600917, 0.554545, 0.600917, 0.554545, 0.595253, 0.815341, 0.600917, 0.554545, 0.600917, 0.554545, 0.595253, 0.815341, 0.595253, 0.815341, 0.595253, 0.815341, 0.589566, 0.554545, 0.595253, 0.815341, 0.595253, 0.815341, 0.589566, 0.554545, 0.589566, 0.554545, 0.589566, 0.554545, 0.584467, 0.554545, 0.589566, 0.554545, 0.589566, 0.554545, 0.584467, 0.554545, 0.584467, 0.554545, 0.581906, 0.872727, 0.577582, 0.872727, 0.587310, 0.624432, 0.577582, 0.872727, 0.581906, 0.872727, 0.587310, 0.624432, 0.577582, 0.872727, 0.584467, 0.554545, 0.587310, 0.624432, 0.584467, 0.554545, 0.577582, 0.872727, 0.587310, 0.624432, 0.587310, 0.624432, 0.584467, 0.554545, 0.589566, 0.554545, 0.584467, 0.554545, 0.587310, 0.624432, 0.589566, 0.554545, 0.595253, 0.815341, 0.587310, 0.624432, 0.589566, 0.554545, 0.587310, 0.624432, 0.595253, 0.815341, 0.589566, 0.554545, 0.595253, 0.815341, 0.592692, 0.872727, 0.587310, 0.624432, 0.592692, 0.872727, 0.595253, 0.815341, 0.587310, 0.624432, 0.592692, 0.872727, 0.595253, 0.815341, 0.597791, 0.872727, 0.595253, 0.815341, 0.592692, 0.872727, 0.597791, 0.872727, 0.597791, 0.872727, 0.595253, 0.815341, 0.603196, 0.624432, 0.595253, 0.815341, 0.597791, 0.872727, 0.603196, 0.624432, 0.603196, 0.624432, 0.595253, 0.815341, 0.600917, 0.554545, 0.595253, 0.815341, 0.603196, 0.624432, 0.600917, 0.554545, 0.603196, 0.624432, 0.600917, 0.554545, 0.606016, 0.554545, 0.600917, 0.554545, 0.603196, 0.624432, 0.606016, 0.554545, 0.606016, 0.554545, 0.612901, 0.872727, 0.603196, 0.624432, 0.612901, 0.872727, 0.606016, 0.554545, 0.603196, 0.624432, 0.612901, 0.872727, 0.608577, 0.872727, 0.603196, 0.624432, 0.608577, 0.872727, 0.612901, 0.872727, 0.603196, 0.624432, 0.619199, 0.872727, 0.623523, 0.872727, 0.619199, 0.872727, 0.619199, 0.872727, 0.623523, 0.872727, 0.623523, 0.872727, 0.623523, 0.872727, 0.623523, 0.554545, 0.623523, 0.872727, 0.623523, 0.872727, 0.623523, 0.554545, 0.623523, 0.554545, 0.623523, 0.554545, 0.619199, 0.554545, 0.623523, 0.554545, 0.623523, 0.554545, 0.619199, 0.554545, 0.619199, 0.554545, 0.619199, 0.872727, 0.623523, 0.554545, 0.623523, 0.872727, 0.623523, 0.554545, 0.619199, 0.872727, 0.623523, 0.872727, 0.619199, 0.872727, 0.619199, 0.554545, 0.623523, 0.554545, 0.619199, 0.554545, 0.619199, 0.872727, 0.623523, 0.554545, 0.619199, 1, 0.623523, 1, 0.619199, 1, 0.619199, 1, 0.623523, 1, 0.623523, 1, 0.623523, 1, 0.623523, 0.936364, 0.623523, 1, 0.623523, 1, 0.623523, 0.936364, 0.623523, 0.936364, 0.623523, 0.936364, 0.619199, 0.936364, 0.623523, 0.936364, 0.623523, 0.936364, 0.619199, 0.936364, 0.619199, 0.936364, 0.619199, 1, 0.623523, 0.936364, 0.623523, 1, 0.623523, 0.936364, 0.619199, 1, 0.623523, 1, 0.619199, 1, 0.619199, 0.936364, 0.623523, 0.936364, 0.619199, 0.936364, 0.619199, 1, 0.623523, 0.936364, 0.632734, 1, 0.637058, 1, 0.632734, 1, 0.632734, 1, 0.637058, 1, 0.637058, 1, 0.637058, 1, 0.637058, 0.554545, 0.637058, 1, 0.637058, 1, 0.637058, 0.554545, 0.637058, 0.554545, 0.637058, 0.554545, 0.632734, 0.554545, 0.637058, 0.554545, 0.637058, 0.554545, 0.632734, 0.554545, 0.632734, 0.554545, 0.632734, 1, 0.637058, 0.554545, 0.637058, 1, 0.637058, 0.554545, 0.632734, 1, 0.637058, 1, 0.632734, 1, 0.632734, 0.554545, 0.637058, 0.554545, 0.632734, 0.554545, 0.632734, 1, 0.637058, 0.554545, 0.646270, 1, 0.650593, 1, 0.646270, 1, 0.646270, 1, 0.650593, 1, 0.650593, 1, 0.650593, 1, 0.650593, 0.554545, 0.650593, 1, 0.650593, 1, 0.650593, 0.554545, 0.650593, 0.554545, 0.650593, 0.554545, 0.646270, 0.554545, 0.650593, 0.554545, 0.650593, 0.554545, 0.646270, 0.554545, 0.646270, 0.554545, 0.646270, 1, 0.650593, 0.554545, 0.650593, 1, 0.650593, 0.554545, 0.646270, 1, 0.650593, 1, 0.646270, 1, 0.646270, 0.554545, 0.650593, 0.554545, 0.646270, 0.554545, 0.646270, 1, 0.650593, 0.554545, 0.693738, 0.713636, 0.693615, 0.740785, 0.693738, 0.713636, 0.693738, 0.713636, 0.693615, 0.740785, 0.693615, 0.740785, 0.693615, 0.740785, 0.693247, 0.764844, 0.693615, 0.740785, 0.693615, 0.740785, 0.693247, 0.764844, 0.693247, 0.764844, 0.693247, 0.764844, 0.692634, 0.785813, 0.693247, 0.764844, 0.693247, 0.764844, 0.692634, 0.785813, 0.692634, 0.785813, 0.692634, 0.785813, 0.691775, 0.803693, 0.692634, 0.785813, 0.692634, 0.785813, 0.691775, 0.803693, 0.691775, 0.803693, 0.691775, 0.803693, 0.690703, 0.817986, 0.691775, 0.803693, 0.691775, 0.803693, 0.690703, 0.817986, 0.690703, 0.817986, 0.690703, 0.817986, 0.689446, 0.828196, 0.690703, 0.817986, 0.690703, 0.817986, 0.689446, 0.828196, 0.689446, 0.828196, 0.689446, 0.828196, 0.688006, 0.834322, 0.689446, 0.828196, 0.689446, 0.828196, 0.688006, 0.834322, 0.688006, 0.834322, 0.688006, 0.834322, 0.686382, 0.836364, 0.688006, 0.834322, 0.688006, 0.834322, 0.686382, 0.836364, 0.686382, 0.836364, 0.686382, 0.836364, 0.684759, 0.834322, 0.686382, 0.836364, 0.686382, 0.836364, 0.684759, 0.834322, 0.684759, 0.834322, 0.684759, 0.834322, 0.683319, 0.828196, 0.684759, 0.834322, 0.684759, 0.834322, 0.683319, 0.828196, 0.683319, 0.828196, 0.683319, 0.828196, 0.682062, 0.817986, 0.683319, 0.828196, 0.683319, 0.828196, 0.682062, 0.817986, 0.682062, 0.817986, 0.682062, 0.817986, 0.680989, 0.803693, 0.682062, 0.817986, 0.682062, 0.817986, 0.680989, 0.803693, 0.680989, 0.803693, 0.680989, 0.803693, 0.680131, 0.785813, 0.680989, 0.803693, 0.680989, 0.803693, 0.680131, 0.785813, 0.680131, 0.785813, 0.680131, 0.785813, 0.679518, 0.764844, 0.680131, 0.785813, 0.680131, 0.785813, 0.679518, 0.764844, 0.679518, 0.764844, 0.679518, 0.764844, 0.679150, 0.740785, 0.679518, 0.764844, 0.679518, 0.764844, 0.679150, 0.740785, 0.679150, 0.740785, 0.679150, 0.740785, 0.679027, 0.713636, 0.679150, 0.740785, 0.679150, 0.740785, 0.679027, 0.713636, 0.679027, 0.713636, 0.679027, 0.713636, 0.679150, 0.686488, 0.679027, 0.713636, 0.679027, 0.713636, 0.679150, 0.686488, 0.679150, 0.686488, 0.679150, 0.686488, 0.679518, 0.662429, 0.679150, 0.686488, 0.679150, 0.686488, 0.679518, 0.662429, 0.679518, 0.662429, 0.679518, 0.662429, 0.680131, 0.641459, 0.679518, 0.662429, 0.679518, 0.662429, 0.680131, 0.641459, 0.680131, 0.641459, 0.680131, 0.641459, 0.680989, 0.623580, 0.680131, 0.641459, 0.680131, 0.641459, 0.680989, 0.623580, 0.680989, 0.623580, 0.680989, 0.623580, 0.682062, 0.609286, 0.680989, 0.623580, 0.680989, 0.623580, 0.682062, 0.609286, 0.682062, 0.609286, 0.682062, 0.609286, 0.683319, 0.599077, 0.682062, 0.609286, 0.682062, 0.609286, 0.683319, 0.599077, 0.683319, 0.599077, 0.683319, 0.599077, 0.684759, 0.592951, 0.683319, 0.599077, 0.683319, 0.599077, 0.684759, 0.592951, 0.684759, 0.592951, 0.684759, 0.592951, 0.686382, 0.590909, 0.684759, 0.592951, 0.684759, 0.592951, 0.686382, 0.590909, 0.686382, 0.590909, 0.686382, 0.590909, 0.688006, 0.592951, 0.686382, 0.590909, 0.686382, 0.590909, 0.688006, 0.592951, 0.688006, 0.592951, 0.688006, 0.592951, 0.689446, 0.599077, 0.688006, 0.592951, 0.688006, 0.592951, 0.689446, 0.599077, 0.689446, 0.599077, 0.689446, 0.599077, 0.690703, 0.609286, 0.689446, 0.599077, 0.689446, 0.599077, 0.690703, 0.609286, 0.690703, 0.609286, 0.690703, 0.609286, 0.691775, 0.623580, 0.690703, 0.609286, 0.690703, 0.609286, 0.691775, 0.623580, 0.691775, 0.623580, 0.691775, 0.623580, 0.692634, 0.641459, 0.691775, 0.623580, 0.691775, 0.623580, 0.692634, 0.641459, 0.692634, 0.641459, 0.692634, 0.641459, 0.693247, 0.662429, 0.692634, 0.641459, 0.692634, 0.641459, 0.693247, 0.662429, 0.693247, 0.662429, 0.693247, 0.662429, 0.693615, 0.686488, 0.693247, 0.662429, 0.693247, 0.662429, 0.693615, 0.686488, 0.693615, 0.686488, 0.679027, 0.827273, 0.679753, 0.840230, 0.679027, 0.827273, 0.679027, 0.827273, 0.679753, 0.840230, 0.679753, 0.840230, 0.679753, 0.840230, 0.680569, 0.851403, 0.679753, 0.840230, 0.679753, 0.840230, 0.680569, 0.851403, 0.680569, 0.851403, 0.680569, 0.851403, 0.681475, 0.860791, 0.680569, 0.851403, 0.680569, 0.851403, 0.681475, 0.860791, 0.681475, 0.860791, 0.681475, 0.860791, 0.682470, 0.868395, 0.681475, 0.860791, 0.681475, 0.860791, 0.682470, 0.868395, 0.682470, 0.868395, 0.682470, 0.868395, 0.683560, 0.874268, 0.682470, 0.868395, 0.682470, 0.868395, 0.683560, 0.874268, 0.683560, 0.874268, 0.683560, 0.874268, 0.684752, 0.878462, 0.683560, 0.874268, 0.683560, 0.874268, 0.684752, 0.878462, 0.684752, 0.878462, 0.684752, 0.878462, 0.686045, 0.880979, 0.684752, 0.878462, 0.684752, 0.878462, 0.686045, 0.880979, 0.686045, 0.880979, 0.686045, 0.880979, 0.687440, 0.881818, 0.686045, 0.880979, 0.686045, 0.880979, 0.687440, 0.881818, 0.687440, 0.881818, 0.687440, 0.881818, 0.689724, 0.878924, 0.687440, 0.881818, 0.687440, 0.881818, 0.689724, 0.878924, 0.689724, 0.878924, 0.689724, 0.878924, 0.691784, 0.870242, 0.689724, 0.878924, 0.689724, 0.878924, 0.691784, 0.870242, 0.691784, 0.870242, 0.691784, 0.870242, 0.693619, 0.855771, 0.691784, 0.870242, 0.691784, 0.870242, 0.693619, 0.855771, 0.693619, 0.855771, 0.693619, 0.855771, 0.695230, 0.835511, 0.693619, 0.855771, 0.693619, 0.855771, 0.695230, 0.835511, 0.695230, 0.835511, 0.695230, 0.835511, 0.696541, 0.810529, 0.695230, 0.835511, 0.695230, 0.835511, 0.696541, 0.810529, 0.696541, 0.810529, 0.696541, 0.810529, 0.697477, 0.781889, 0.696541, 0.810529, 0.696541, 0.810529, 0.697477, 0.781889, 0.697477, 0.781889, 0.697477, 0.781889, 0.698039, 0.749592, 0.697477, 0.781889, 0.697477, 0.781889, 0.698039, 0.749592, 0.698039, 0.749592, 0.698039, 0.749592, 0.698226, 0.713636, 0.698039, 0.749592, 0.698039, 0.749592, 0.698226, 0.713636, 0.698226, 0.713636, 0.698226, 0.713636, 0.698039, 0.677681, 0.698226, 0.713636, 0.698226, 0.713636, 0.698039, 0.677681, 0.698039, 0.677681, 0.698039, 0.677681, 0.697477, 0.645383, 0.698039, 0.677681, 0.698039, 0.677681, 0.697477, 0.645383, 0.697477, 0.645383, 0.697477, 0.645383, 0.696541, 0.616744, 0.697477, 0.645383, 0.697477, 0.645383, 0.696541, 0.616744, 0.696541, 0.616744, 0.696541, 0.616744, 0.695230, 0.591761, 0.696541, 0.616744, 0.696541, 0.616744, 0.695230, 0.591761, 0.695230, 0.591761, 0.695230, 0.591761, 0.693619, 0.571502, 0.695230, 0.591761, 0.695230, 0.591761, 0.693619, 0.571502, 0.693619, 0.571502, 0.693619, 0.571502, 0.691784, 0.557031, 0.693619, 0.571502, 0.693619, 0.571502, 0.691784, 0.557031, 0.691784, 0.557031, 0.691784, 0.557031, 0.689724, 0.548349, 0.691784, 0.557031, 0.691784, 0.557031, 0.689724, 0.548349, 0.689724, 0.548349, 0.689724, 0.548349, 0.687440, 0.545455, 0.689724, 0.548349, 0.689724, 0.548349, 0.687440, 0.545455, 0.687440, 0.545455, 0.687440, 0.545455, 0.686045, 0.546293, 0.687440, 0.545455, 0.687440, 0.545455, 0.686045, 0.546293, 0.686045, 0.546293, 0.686045, 0.546293, 0.684752, 0.548810, 0.686045, 0.546293, 0.686045, 0.546293, 0.684752, 0.548810, 0.684752, 0.548810, 0.684752, 0.548810, 0.683560, 0.553005, 0.684752, 0.548810, 0.684752, 0.548810, 0.683560, 0.553005, 0.683560, 0.553005, 0.683560, 0.553005, 0.682470, 0.558878, 0.683560, 0.553005, 0.683560, 0.553005, 0.682470, 0.558878, 0.682470, 0.558878, 0.682470, 0.558878, 0.681475, 0.566482, 0.682470, 0.558878, 0.682470, 0.558878, 0.681475, 0.566482, 0.681475, 0.566482, 0.681475, 0.566482, 0.680569, 0.575870, 0.681475, 0.566482, 0.681475, 0.566482, 0.680569, 0.575870, 0.680569, 0.575870, 0.680569, 0.575870, 0.679753, 0.587043, 0.680569, 0.575870, 0.680569, 0.575870, 0.679753, 0.587043, 0.679753, 0.587043, 0.679753, 0.587043, 0.679027, 0.600000, 0.679753, 0.587043, 0.679753, 0.587043, 0.679027, 0.600000, 0.679027, 0.600000, 0.679027, 0.600000, 0.679027, 0.554545, 0.679027, 0.600000, 0.679027, 0.600000, 0.679027, 0.554545, 0.679027, 0.554545, 0.679027, 0.554545, 0.674680, 0.554545, 0.679027, 0.554545, 0.679027, 0.554545, 0.674680, 0.554545, 0.674680, 0.554545, 0.674680, 0.554545, 0.674680, 1, 0.674680, 0.554545, 0.674680, 0.554545, 0.674680, 1, 0.674680, 1, 0.674680, 1, 0.679027, 1, 0.674680, 1, 0.674680, 1, 0.679027, 1, 0.679027, 1, 0.679027, 1, 0.674680, 1, 0.679027, 0.827273, 0.674680, 1, 0.679027, 1, 0.679027, 0.827273, 0.674680, 1, 0.674680, 0.554545, 0.679027, 0.827273, 0.674680, 0.554545, 0.674680, 1, 0.679027, 0.827273, 0.679027, 0.827273, 0.674680, 0.554545, 0.679150, 0.740785, 0.674680, 0.554545, 0.679027, 0.827273, 0.679150, 0.740785, 0.679027, 0.827273, 0.679150, 0.740785, 0.679518, 0.764844, 0.679150, 0.740785, 0.679027, 0.827273, 0.679518, 0.764844, 0.679027, 0.827273, 0.679518, 0.764844, 0.680131, 0.785813, 0.679518, 0.764844, 0.679027, 0.827273, 0.680131, 0.785813, 0.679027, 0.827273, 0.680131, 0.785813, 0.680989, 0.803693, 0.680131, 0.785813, 0.679027, 0.827273, 0.680989, 0.803693, 0.679027, 0.827273, 0.680989, 0.803693, 0.682062, 0.817986, 0.680989, 0.803693, 0.679027, 0.827273, 0.682062, 0.817986, 0.679753, 0.840230, 0.679027, 0.827273, 0.682062, 0.817986, 0.679027, 0.827273, 0.679753, 0.840230, 0.682062, 0.817986, 0.680569, 0.851403, 0.679753, 0.840230, 0.682062, 0.817986, 0.679753, 0.840230, 0.680569, 0.851403, 0.682062, 0.817986, 0.680569, 0.851403, 0.682062, 0.817986, 0.683319, 0.828196, 0.682062, 0.817986, 0.680569, 0.851403, 0.683319, 0.828196, 0.681475, 0.860791, 0.680569, 0.851403, 0.683319, 0.828196, 0.680569, 0.851403, 0.681475, 0.860791, 0.683319, 0.828196, 0.682470, 0.868395, 0.681475, 0.860791, 0.683319, 0.828196, 0.681475, 0.860791, 0.682470, 0.868395, 0.683319, 0.828196, 0.682470, 0.868395, 0.683319, 0.828196, 0.684759, 0.834322, 0.683319, 0.828196, 0.682470, 0.868395, 0.684759, 0.834322, 0.683560, 0.874268, 0.682470, 0.868395, 0.684759, 0.834322, 0.682470, 0.868395, 0.683560, 0.874268, 0.684759, 0.834322, 0.684752, 0.878462, 0.683560, 0.874268, 0.684759, 0.834322, 0.683560, 0.874268, 0.684752, 0.878462, 0.684759, 0.834322, 0.684752, 0.878462, 0.684759, 0.834322, 0.686382, 0.836364, 0.684759, 0.834322, 0.684752, 0.878462, 0.686382, 0.836364, 0.686045, 0.880979, 0.684752, 0.878462, 0.686382, 0.836364, 0.684752, 0.878462, 0.686045, 0.880979, 0.686382, 0.836364, 0.687440, 0.881818, 0.686045, 0.880979, 0.686382, 0.836364, 0.686045, 0.880979, 0.687440, 0.881818, 0.686382, 0.836364, 0.687440, 0.881818, 0.686382, 0.836364, 0.688006, 0.834322, 0.686382, 0.836364, 0.687440, 0.881818, 0.688006, 0.834322, 0.689724, 0.878924, 0.687440, 0.881818, 0.688006, 0.834322, 0.687440, 0.881818, 0.689724, 0.878924, 0.688006, 0.834322, 0.688006, 0.834322, 0.689446, 0.828196, 0.689724, 0.878924, 0.689446, 0.828196, 0.688006, 0.834322, 0.689724, 0.878924, 0.689724, 0.878924, 0.689446, 0.828196, 0.691784, 0.870242, 0.689446, 0.828196, 0.689724, 0.878924, 0.691784, 0.870242, 0.689446, 0.828196, 0.690703, 0.817986, 0.691784, 0.870242, 0.690703, 0.817986, 0.689446, 0.828196, 0.691784, 0.870242, 0.691784, 0.870242, 0.690703, 0.817986, 0.693619, 0.855771, 0.690703, 0.817986, 0.691784, 0.870242, 0.693619, 0.855771, 0.690703, 0.817986, 0.691775, 0.803693, 0.693619, 0.855771, 0.691775, 0.803693, 0.690703, 0.817986, 0.693619, 0.855771, 0.693619, 0.855771, 0.691775, 0.803693, 0.695230, 0.835511, 0.691775, 0.803693, 0.693619, 0.855771, 0.695230, 0.835511, 0.695230, 0.835511, 0.691775, 0.803693, 0.696541, 0.810529, 0.691775, 0.803693, 0.695230, 0.835511, 0.696541, 0.810529, 0.696541, 0.810529, 0.691775, 0.803693, 0.692634, 0.785813, 0.691775, 0.803693, 0.696541, 0.810529, 0.692634, 0.785813, 0.696541, 0.810529, 0.692634, 0.785813, 0.697477, 0.781889, 0.692634, 0.785813, 0.696541, 0.810529, 0.697477, 0.781889, 0.697477, 0.781889, 0.692634, 0.785813, 0.693247, 0.764844, 0.692634, 0.785813, 0.697477, 0.781889, 0.693247, 0.764844, 0.697477, 0.781889, 0.693247, 0.764844, 0.698039, 0.749592, 0.693247, 0.764844, 0.697477, 0.781889, 0.698039, 0.749592, 0.698039, 0.749592, 0.693247, 0.764844, 0.693615, 0.740785, 0.693247, 0.764844, 0.698039, 0.749592, 0.693615, 0.740785, 0.698039, 0.749592, 0.693615, 0.740785, 0.698226, 0.713636, 0.693615, 0.740785, 0.698039, 0.749592, 0.698226, 0.713636, 0.698226, 0.713636, 0.693615, 0.740785, 0.693738, 0.713636, 0.693615, 0.740785, 0.698226, 0.713636, 0.693738, 0.713636, 0.698226, 0.713636, 0.693738, 0.713636, 0.693615, 0.686488, 0.693738, 0.713636, 0.698226, 0.713636, 0.693615, 0.686488, 0.698226, 0.713636, 0.693615, 0.686488, 0.698039, 0.677681, 0.693615, 0.686488, 0.698226, 0.713636, 0.698039, 0.677681, 0.693615, 0.686488, 0.693247, 0.662429, 0.698039, 0.677681, 0.693247, 0.662429, 0.693615, 0.686488, 0.698039, 0.677681, 0.693247, 0.662429, 0.697477, 0.645383, 0.698039, 0.677681, 0.697477, 0.645383, 0.693247, 0.662429, 0.698039, 0.677681, 0.693247, 0.662429, 0.692634, 0.641459, 0.697477, 0.645383, 0.692634, 0.641459, 0.693247, 0.662429, 0.697477, 0.645383, 0.692634, 0.641459, 0.696541, 0.616744, 0.697477, 0.645383, 0.696541, 0.616744, 0.692634, 0.641459, 0.697477, 0.645383, 0.692634, 0.641459, 0.691775, 0.623580, 0.696541, 0.616744, 0.691775, 0.623580, 0.692634, 0.641459, 0.696541, 0.616744, 0.691775, 0.623580, 0.695230, 0.591761, 0.696541, 0.616744, 0.695230, 0.591761, 0.691775, 0.623580, 0.696541, 0.616744, 0.695230, 0.591761, 0.691775, 0.623580, 0.693619, 0.571502, 0.691775, 0.623580, 0.695230, 0.591761, 0.693619, 0.571502, 0.691775, 0.623580, 0.690703, 0.609286, 0.693619, 0.571502, 0.690703, 0.609286, 0.691775, 0.623580, 0.693619, 0.571502, 0.693619, 0.571502, 0.690703, 0.609286, 0.691784, 0.557031, 0.690703, 0.609286, 0.693619, 0.571502, 0.691784, 0.557031, 0.690703, 0.609286, 0.689446, 0.599077, 0.691784, 0.557031, 0.689446, 0.599077, 0.690703, 0.609286, 0.691784, 0.557031, 0.691784, 0.557031, 0.689446, 0.599077, 0.689724, 0.548349, 0.689446, 0.599077, 0.691784, 0.557031, 0.689724, 0.548349, 0.689446, 0.599077, 0.688006, 0.592951, 0.689724, 0.548349, 0.688006, 0.592951, 0.689446, 0.599077, 0.689724, 0.548349, 0.688006, 0.592951, 0.687440, 0.545455, 0.689724, 0.548349, 0.687440, 0.545455, 0.688006, 0.592951, 0.689724, 0.548349, 0.686382, 0.590909, 0.687440, 0.545455, 0.688006, 0.592951, 0.687440, 0.545455, 0.686382, 0.590909, 0.688006, 0.592951, 0.686382, 0.590909, 0.686045, 0.546293, 0.687440, 0.545455, 0.686045, 0.546293, 0.686382, 0.590909, 0.687440, 0.545455, 0.684752, 0.548810, 0.686045, 0.546293, 0.686382, 0.590909, 0.686045, 0.546293, 0.684752, 0.548810, 0.686382, 0.590909, 0.684759, 0.592951, 0.684752, 0.548810, 0.686382, 0.590909, 0.684752, 0.548810, 0.684759, 0.592951, 0.686382, 0.590909, 0.683560, 0.553005, 0.684752, 0.548810, 0.684759, 0.592951, 0.684752, 0.548810, 0.683560, 0.553005, 0.684759, 0.592951, 0.682470, 0.558878, 0.683560, 0.553005, 0.684759, 0.592951, 0.683560, 0.553005, 0.682470, 0.558878, 0.684759, 0.592951, 0.683319, 0.599077, 0.682470, 0.558878, 0.684759, 0.592951, 0.682470, 0.558878, 0.683319, 0.599077, 0.684759, 0.592951, 0.681475, 0.566482, 0.682470, 0.558878, 0.683319, 0.599077, 0.682470, 0.558878, 0.681475, 0.566482, 0.683319, 0.599077, 0.680569, 0.575870, 0.681475, 0.566482, 0.683319, 0.599077, 0.681475, 0.566482, 0.680569, 0.575870, 0.683319, 0.599077, 0.682062, 0.609286, 0.680569, 0.575870, 0.683319, 0.599077, 0.680569, 0.575870, 0.682062, 0.609286, 0.683319, 0.599077, 0.679753, 0.587043, 0.680569, 0.575870, 0.682062, 0.609286, 0.680569, 0.575870, 0.679753, 0.587043, 0.682062, 0.609286, 0.679027, 0.600000, 0.679753, 0.587043, 0.682062, 0.609286, 0.679753, 0.587043, 0.679027, 0.600000, 0.682062, 0.609286, 0.680989, 0.623580, 0.679027, 0.600000, 0.682062, 0.609286, 0.679027, 0.600000, 0.680989, 0.623580, 0.682062, 0.609286, 0.680131, 0.641459, 0.679027, 0.600000, 0.680989, 0.623580, 0.679027, 0.600000, 0.680131, 0.641459, 0.680989, 0.623580, 0.679518, 0.662429, 0.679027, 0.600000, 0.680131, 0.641459, 0.679027, 0.600000, 0.679518, 0.662429, 0.680131, 0.641459, 0.679518, 0.662429, 0.674680, 0.554545, 0.679027, 0.600000, 0.674680, 0.554545, 0.679518, 0.662429, 0.679027, 0.600000, 0.679518, 0.662429, 0.679150, 0.686488, 0.674680, 0.554545, 0.679150, 0.686488, 0.679518, 0.662429, 0.674680, 0.554545, 0.679027, 0.713636, 0.674680, 0.554545, 0.679150, 0.686488, 0.674680, 0.554545, 0.679027, 0.713636, 0.679150, 0.686488, 0.679150, 0.740785, 0.674680, 0.554545, 0.679027, 0.713636, 0.674680, 0.554545, 0.679150, 0.740785, 0.679027, 0.713636, 0.674680, 0.554545, 0.679027, 0.554545, 0.679027, 0.600000, 0.679027, 0.554545, 0.674680, 0.554545, 0.679027, 0.600000, 0.728187, 0.728409, 0.728187, 0.700000, 0.728187, 0.728409, 0.728187, 0.728409, 0.728187, 0.700000, 0.728187, 0.700000, 0.728187, 0.700000, 0.708307, 0.700000, 0.728187, 0.700000, 0.728187, 0.700000, 0.708307, 0.700000, 0.708307, 0.700000, 0.708307, 0.700000, 0.708581, 0.674951, 0.708307, 0.700000, 0.708307, 0.700000, 0.708581, 0.674951, 0.708581, 0.674951, 0.708581, 0.674951, 0.709121, 0.653072, 0.708581, 0.674951, 0.708581, 0.674951, 0.709121, 0.653072, 0.709121, 0.653072, 0.709121, 0.653072, 0.709926, 0.634362, 0.709121, 0.653072, 0.709121, 0.653072, 0.709926, 0.634362, 0.709926, 0.634362, 0.709926, 0.634362, 0.710998, 0.618821, 0.709926, 0.634362, 0.709926, 0.634362, 0.710998, 0.618821, 0.710998, 0.618821, 0.710998, 0.618821, 0.712320, 0.606610, 0.710998, 0.618821, 0.710998, 0.618821, 0.712320, 0.606610, 0.712320, 0.606610, 0.712320, 0.606610, 0.713879, 0.597887, 0.712320, 0.606610, 0.712320, 0.606610, 0.713879, 0.597887, 0.713879, 0.597887, 0.713879, 0.597887, 0.715675, 0.592654, 0.713879, 0.597887, 0.713879, 0.597887, 0.715675, 0.592654, 0.715675, 0.592654, 0.715675, 0.592654, 0.717707, 0.590909, 0.715675, 0.592654, 0.715675, 0.592654, 0.717707, 0.590909, 0.717707, 0.590909, 0.717707, 0.590909, 0.718942, 0.591335, 0.717707, 0.590909, 0.717707, 0.590909, 0.718942, 0.591335, 0.718942, 0.591335, 0.718942, 0.591335, 0.720159, 0.592614, 0.718942, 0.591335, 0.718942, 0.591335, 0.720159, 0.592614, 0.720159, 0.592614, 0.720159, 0.592614, 0.721357, 0.594744, 0.720159, 0.592614, 0.720159, 0.592614, 0.721357, 0.594744, 0.721357, 0.594744, 0.721357, 0.594744, 0.722535, 0.597727, 0.721357, 0.594744, 0.721357, 0.594744, 0.722535, 0.597727, 0.722535, 0.597727, 0.722535, 0.597727, 0.723702, 0.601563, 0.722535, 0.597727, 0.722535, 0.597727, 0.723702, 0.601563, 0.723702, 0.601563, 0.723702, 0.601563, 0.724865, 0.606250, 0.723702, 0.601563, 0.723702, 0.601563, 0.724865, 0.606250, 0.724865, 0.606250, 0.724865, 0.606250, 0.726023, 0.611790, 0.724865, 0.606250, 0.724865, 0.606250, 0.726023, 0.611790, 0.726023, 0.611790, 0.726023, 0.611790, 0.727177, 0.618182, 0.726023, 0.611790, 0.726023, 0.611790, 0.727177, 0.618182, 0.727177, 0.618182, 0.727177, 0.618182, 0.727177, 0.572727, 0.727177, 0.618182, 0.727177, 0.618182, 0.727177, 0.572727, 0.727177, 0.572727, 0.727177, 0.572727, 0.726006, 0.566451, 0.727177, 0.572727, 0.727177, 0.572727, 0.726006, 0.566451, 0.726006, 0.566451, 0.726006, 0.566451, 0.724821, 0.560973, 0.726006, 0.566451, 0.726006, 0.566451, 0.724821, 0.560973, 0.724821, 0.560973, 0.724821, 0.560973, 0.723621, 0.556294, 0.724821, 0.560973, 0.724821, 0.560973, 0.723621, 0.556294, 0.723621, 0.556294, 0.723621, 0.556294, 0.722406, 0.552415, 0.723621, 0.556294, 0.723621, 0.556294, 0.722406, 0.552415, 0.722406, 0.552415, 0.722406, 0.552415, 0.721180, 0.549370, 0.722406, 0.552415, 0.722406, 0.552415, 0.721180, 0.549370, 0.721180, 0.549370, 0.721180, 0.549370, 0.719945, 0.547195, 0.721180, 0.549370, 0.721180, 0.549370, 0.719945, 0.547195, 0.719945, 0.547195, 0.719945, 0.547195, 0.718701, 0.545890, 0.719945, 0.547195, 0.719945, 0.547195, 0.718701, 0.545890, 0.718701, 0.545890, 0.718701, 0.545890, 0.717448, 0.545455, 0.718701, 0.545890, 0.718701, 0.545890, 0.717448, 0.545455, 0.717448, 0.545455, 0.717448, 0.545455, 0.714463, 0.548242, 0.717448, 0.545455, 0.717448, 0.545455, 0.714463, 0.548242, 0.714463, 0.548242, 0.714463, 0.548242, 0.711805, 0.556605, 0.714463, 0.548242, 0.714463, 0.548242, 0.711805, 0.556605, 0.711805, 0.556605, 0.711805, 0.556605, 0.709475, 0.570543, 0.711805, 0.556605, 0.711805, 0.556605, 0.709475, 0.570543, 0.709475, 0.570543, 0.709475, 0.570543, 0.707473, 0.590057, 0.709475, 0.570543, 0.709475, 0.570543, 0.707473, 0.590057, 0.707473, 0.590057, 0.707473, 0.590057, 0.705864, 0.614329, 0.707473, 0.590057, 0.707473, 0.590057, 0.705864, 0.614329, 0.705864, 0.614329, 0.705864, 0.614329, 0.704714, 0.642543, 0.705864, 0.614329, 0.705864, 0.614329, 0.704714, 0.642543, 0.704714, 0.642543, 0.704714, 0.642543, 0.704025, 0.674698, 0.704714, 0.642543, 0.704714, 0.642543, 0.704025, 0.674698, 0.704025, 0.674698, 0.704025, 0.674698, 0.703795, 0.710795, 0.704025, 0.674698, 0.704025, 0.674698, 0.703795, 0.710795, 0.703795, 0.710795, 0.703795, 0.710795, 0.704013, 0.748109, 0.703795, 0.710795, 0.703795, 0.710795, 0.704013, 0.748109, 0.704013, 0.748109, 0.704013, 0.748109, 0.704668, 0.781357, 0.704013, 0.748109, 0.704013, 0.748109, 0.704668, 0.781357, 0.704668, 0.781357, 0.704668, 0.781357, 0.705758, 0.810538, 0.704668, 0.781357, 0.704668, 0.781357, 0.705758, 0.810538, 0.705758, 0.810538, 0.705758, 0.810538, 0.707285, 0.835653, 0.705758, 0.810538, 0.705758, 0.810538, 0.707285, 0.835653, 0.707285, 0.835653, 0.707285, 0.835653, 0.709182, 0.855850, 0.707285, 0.835653, 0.707285, 0.835653, 0.709182, 0.855850, 0.709182, 0.855850, 0.709182, 0.855850, 0.711382, 0.870277, 0.709182, 0.855850, 0.709182, 0.855850, 0.711382, 0.870277, 0.711382, 0.870277, 0.711382, 0.870277, 0.713887, 0.878933, 0.711382, 0.870277, 0.711382, 0.870277, 0.713887, 0.878933, 0.713887, 0.878933, 0.713887, 0.878933, 0.716696, 0.881818, 0.713887, 0.878933, 0.713887, 0.878933, 0.716696, 0.881818, 0.716696, 0.881818, 0.716696, 0.881818, 0.719213, 0.879239, 0.716696, 0.881818, 0.716696, 0.881818, 0.719213, 0.879239, 0.719213, 0.879239, 0.719213, 0.879239, 0.721452, 0.871502, 0.719213, 0.879239, 0.719213, 0.879239, 0.721452, 0.871502, 0.721452, 0.871502, 0.721452, 0.871502, 0.723413, 0.858607, 0.721452, 0.871502, 0.721452, 0.871502, 0.723413, 0.858607, 0.723413, 0.858607, 0.723413, 0.858607, 0.725097, 0.840554, 0.723413, 0.858607, 0.723413, 0.858607, 0.725097, 0.840554, 0.725097, 0.840554, 0.725097, 0.840554, 0.726449, 0.818071, 0.725097, 0.840554, 0.725097, 0.840554, 0.726449, 0.818071, 0.726449, 0.818071, 0.726449, 0.818071, 0.727414, 0.791886, 0.726449, 0.818071, 0.726449, 0.818071, 0.727414, 0.791886, 0.727414, 0.791886, 0.727414, 0.791886, 0.727994, 0.761998, 0.727414, 0.791886, 0.727414, 0.791886, 0.727994, 0.761998, 0.727994, 0.761998, 0.704013, 0.748109, 0.703795, 0.710795, 0.708448, 0.745170, 0.703795, 0.710795, 0.704013, 0.748109, 0.708448, 0.745170, 0.704668, 0.781357, 0.704013, 0.748109, 0.708448, 0.745170, 0.704013, 0.748109, 0.704668, 0.781357, 0.708448, 0.745170, 0.704668, 0.781357, 0.708448, 0.745170, 0.708728, 0.765678, 0.708448, 0.745170, 0.704668, 0.781357, 0.708728, 0.765678, 0.709238, 0.783736, 0.704668, 0.781357, 0.708728, 0.765678, 0.704668, 0.781357, 0.709238, 0.783736, 0.708728, 0.765678, 0.709238, 0.783736, 0.705758, 0.810538, 0.704668, 0.781357, 0.705758, 0.810538, 0.709238, 0.783736, 0.704668, 0.781357, 0.705758, 0.810538, 0.709238, 0.783736, 0.709979, 0.799343, 0.709238, 0.783736, 0.705758, 0.810538, 0.709979, 0.799343, 0.707285, 0.835653, 0.705758, 0.810538, 0.709979, 0.799343, 0.705758, 0.810538, 0.707285, 0.835653, 0.709979, 0.799343, 0.707285, 0.835653, 0.709979, 0.799343, 0.710951, 0.812500, 0.709979, 0.799343, 0.707285, 0.835653, 0.710951, 0.812500, 0.709182, 0.855850, 0.707285, 0.835653, 0.710951, 0.812500, 0.707285, 0.835653, 0.709182, 0.855850, 0.710951, 0.812500, 0.709182, 0.855850, 0.710951, 0.812500, 0.712128, 0.822940, 0.710951, 0.812500, 0.709182, 0.855850, 0.712128, 0.822940, 0.711382, 0.870277, 0.709182, 0.855850, 0.712128, 0.822940, 0.709182, 0.855850, 0.711382, 0.870277, 0.712128, 0.822940, 0.711382, 0.870277, 0.712128, 0.822940, 0.713486, 0.830398, 0.712128, 0.822940, 0.711382, 0.870277, 0.713486, 0.830398, 0.711382, 0.870277, 0.713486, 0.830398, 0.713887, 0.878933, 0.713486, 0.830398, 0.711382, 0.870277, 0.713887, 0.878933, 0.713887, 0.878933, 0.713486, 0.830398, 0.715024, 0.834872, 0.713486, 0.830398, 0.713887, 0.878933, 0.715024, 0.834872, 0.713887, 0.878933, 0.715024, 0.834872, 0.716696, 0.881818, 0.715024, 0.834872, 0.713887, 0.878933, 0.716696, 0.881818, 0.716696, 0.881818, 0.715024, 0.834872, 0.716743, 0.836364, 0.715024, 0.834872, 0.716696, 0.881818, 0.716743, 0.836364, 0.716696, 0.881818, 0.716743, 0.836364, 0.718262, 0.834819, 0.716743, 0.836364, 0.716696, 0.881818, 0.718262, 0.834819, 0.716696, 0.881818, 0.718262, 0.834819, 0.719213, 0.879239, 0.718262, 0.834819, 0.716696, 0.881818, 0.719213, 0.879239, 0.719213, 0.879239, 0.718262, 0.834819, 0.719625, 0.830185, 0.718262, 0.834819, 0.719213, 0.879239, 0.719625, 0.830185, 0.719213, 0.879239, 0.719625, 0.830185, 0.721452, 0.871502, 0.719625, 0.830185, 0.719213, 0.879239, 0.721452, 0.871502, 0.719625, 0.830185, 0.720830, 0.822461, 0.721452, 0.871502, 0.720830, 0.822461, 0.719625, 0.830185, 0.721452, 0.871502, 0.721452, 0.871502, 0.720830, 0.822461, 0.723413, 0.858607, 0.720830, 0.822461, 0.721452, 0.871502, 0.723413, 0.858607, 0.720830, 0.822461, 0.721878, 0.811648, 0.723413, 0.858607, 0.721878, 0.811648, 0.720830, 0.822461, 0.723413, 0.858607, 0.723413, 0.858607, 0.721878, 0.811648, 0.725097, 0.840554, 0.721878, 0.811648, 0.723413, 0.858607, 0.725097, 0.840554, 0.721878, 0.811648, 0.722729, 0.798242, 0.725097, 0.840554, 0.722729, 0.798242, 0.721878, 0.811648, 0.725097, 0.840554, 0.725097, 0.840554, 0.722729, 0.798242, 0.726449, 0.818071, 0.722729, 0.798242, 0.725097, 0.840554, 0.726449, 0.818071, 0.722729, 0.798242, 0.723343, 0.782741, 0.726449, 0.818071, 0.723343, 0.782741, 0.722729, 0.798242, 0.726449, 0.818071, 0.726449, 0.818071, 0.723343, 0.782741, 0.727414, 0.791886, 0.723343, 0.782741, 0.726449, 0.818071, 0.727414, 0.791886, 0.723343, 0.782741, 0.723722, 0.765146, 0.727414, 0.791886, 0.723722, 0.765146, 0.723343, 0.782741, 0.727414, 0.791886, 0.727414, 0.791886, 0.723722, 0.765146, 0.727994, 0.761998, 0.723722, 0.765146, 0.727414, 0.791886, 0.727994, 0.761998, 0.723722, 0.765146, 0.723863, 0.745455, 0.727994, 0.761998, 0.723863, 0.745455, 0.723722, 0.765146, 0.727994, 0.761998, 0.727994, 0.761998, 0.723863, 0.745455, 0.728187, 0.728409, 0.723863, 0.745455, 0.727994, 0.761998, 0.728187, 0.728409, 0.728187, 0.700000, 0.728187, 0.728409, 0.723863, 0.745455, 0.728187, 0.728409, 0.728187, 0.700000, 0.723863, 0.745455, 0.723863, 0.745455, 0.708307, 0.700000, 0.728187, 0.700000, 0.708307, 0.700000, 0.723863, 0.745455, 0.728187, 0.700000, 0.723863, 0.745455, 0.708448, 0.745170, 0.708307, 0.700000, 0.708448, 0.745170, 0.723863, 0.745455, 0.708307, 0.700000, 0.703795, 0.710795, 0.708307, 0.700000, 0.708448, 0.745170, 0.708307, 0.700000, 0.703795, 0.710795, 0.708448, 0.745170, 0.703795, 0.710795, 0.704025, 0.674698, 0.708307, 0.700000, 0.704025, 0.674698, 0.703795, 0.710795, 0.708307, 0.700000, 0.708307, 0.700000, 0.704025, 0.674698, 0.708581, 0.674951, 0.704025, 0.674698, 0.708307, 0.700000, 0.708581, 0.674951, 0.704025, 0.674698, 0.704714, 0.642543, 0.708581, 0.674951, 0.704714, 0.642543, 0.704025, 0.674698, 0.708581, 0.674951, 0.708581, 0.674951, 0.704714, 0.642543, 0.709121, 0.653072, 0.704714, 0.642543, 0.708581, 0.674951, 0.709121, 0.653072, 0.704714, 0.642543, 0.705864, 0.614329, 0.709121, 0.653072, 0.705864, 0.614329, 0.704714, 0.642543, 0.709121, 0.653072, 0.709121, 0.653072, 0.705864, 0.614329, 0.709926, 0.634362, 0.705864, 0.614329, 0.709121, 0.653072, 0.709926, 0.634362, 0.705864, 0.614329, 0.707473, 0.590057, 0.709926, 0.634362, 0.707473, 0.590057, 0.705864, 0.614329, 0.709926, 0.634362, 0.709926, 0.634362, 0.707473, 0.590057, 0.710998, 0.618821, 0.707473, 0.590057, 0.709926, 0.634362, 0.710998, 0.618821, 0.707473, 0.590057, 0.709475, 0.570543, 0.710998, 0.618821, 0.709475, 0.570543, 0.707473, 0.590057, 0.710998, 0.618821, 0.710998, 0.618821, 0.709475, 0.570543, 0.712320, 0.606610, 0.709475, 0.570543, 0.710998, 0.618821, 0.712320, 0.606610, 0.712320, 0.606610, 0.709475, 0.570543, 0.711805, 0.556605, 0.709475, 0.570543, 0.712320, 0.606610, 0.711805, 0.556605, 0.713879, 0.597887, 0.712320, 0.606610, 0.711805, 0.556605, 0.712320, 0.606610, 0.713879, 0.597887, 0.711805, 0.556605, 0.713879, 0.597887, 0.711805, 0.556605, 0.714463, 0.548242, 0.711805, 0.556605, 0.713879, 0.597887, 0.714463, 0.548242, 0.715675, 0.592654, 0.713879, 0.597887, 0.714463, 0.548242, 0.713879, 0.597887, 0.715675, 0.592654, 0.714463, 0.548242, 0.715675, 0.592654, 0.714463, 0.548242, 0.717448, 0.545455, 0.714463, 0.548242, 0.715675, 0.592654, 0.717448, 0.545455, 0.717707, 0.590909, 0.715675, 0.592654, 0.717448, 0.545455, 0.715675, 0.592654, 0.717707, 0.590909, 0.717448, 0.545455, 0.717707, 0.590909, 0.717448, 0.545455, 0.718701, 0.545890, 0.717448, 0.545455, 0.717707, 0.590909, 0.718701, 0.545890, 0.717707, 0.590909, 0.718701, 0.545890, 0.718942, 0.591335, 0.718701, 0.545890, 0.717707, 0.590909, 0.718942, 0.591335, 0.718942, 0.591335, 0.718701, 0.545890, 0.719945, 0.547195, 0.718701, 0.545890, 0.718942, 0.591335, 0.719945, 0.547195, 0.718942, 0.591335, 0.719945, 0.547195, 0.720159, 0.592614, 0.719945, 0.547195, 0.718942, 0.591335, 0.720159, 0.592614, 0.720159, 0.592614, 0.719945, 0.547195, 0.721180, 0.549370, 0.719945, 0.547195, 0.720159, 0.592614, 0.721180, 0.549370, 0.720159, 0.592614, 0.721180, 0.549370, 0.721357, 0.594744, 0.721180, 0.549370, 0.720159, 0.592614, 0.721357, 0.594744, 0.721357, 0.594744, 0.721180, 0.549370, 0.722406, 0.552415, 0.721180, 0.549370, 0.721357, 0.594744, 0.722406, 0.552415, 0.721357, 0.594744, 0.722406, 0.552415, 0.722535, 0.597727, 0.722406, 0.552415, 0.721357, 0.594744, 0.722535, 0.597727, 0.722535, 0.597727, 0.722406, 0.552415, 0.723621, 0.556294, 0.722406, 0.552415, 0.722535, 0.597727, 0.723621, 0.556294, 0.723702, 0.601563, 0.722535, 0.597727, 0.723621, 0.556294, 0.722535, 0.597727, 0.723702, 0.601563, 0.723621, 0.556294, 0.724821, 0.560973, 0.723702, 0.601563, 0.723621, 0.556294, 0.723702, 0.601563, 0.724821, 0.560973, 0.723621, 0.556294, 0.723702, 0.601563, 0.724821, 0.560973, 0.726006, 0.566451, 0.724821, 0.560973, 0.723702, 0.601563, 0.726006, 0.566451, 0.724865, 0.606250, 0.723702, 0.601563, 0.726006, 0.566451, 0.723702, 0.601563, 0.724865, 0.606250, 0.726006, 0.566451, 0.724865, 0.606250, 0.726006, 0.566451, 0.727177, 0.572727, 0.726006, 0.566451, 0.724865, 0.606250, 0.727177, 0.572727, 0.726023, 0.611790, 0.724865, 0.606250, 0.727177, 0.572727, 0.724865, 0.606250, 0.726023, 0.611790, 0.727177, 0.572727, 0.726023, 0.611790, 0.727177, 0.572727, 0.727177, 0.618182, 0.727177, 0.572727, 0.726023, 0.611790, 0.727177, 0.618182, 0.723863, 0.745455, 0.723722, 0.765146, 0.723863, 0.745455, 0.723863, 0.745455, 0.723722, 0.765146, 0.723722, 0.765146, 0.723722, 0.765146, 0.723343, 0.782741, 0.723722, 0.765146, 0.723722, 0.765146, 0.723343, 0.782741, 0.723343, 0.782741, 0.723343, 0.782741, 0.722729, 0.798242, 0.723343, 0.782741, 0.723343, 0.782741, 0.722729, 0.798242, 0.722729, 0.798242, 0.722729, 0.798242, 0.721878, 0.811648, 0.722729, 0.798242, 0.722729, 0.798242, 0.721878, 0.811648, 0.721878, 0.811648, 0.721878, 0.811648, 0.720830, 0.822461, 0.721878, 0.811648, 0.721878, 0.811648, 0.720830, 0.822461, 0.720830, 0.822461, 0.720830, 0.822461, 0.719625, 0.830185, 0.720830, 0.822461, 0.720830, 0.822461, 0.719625, 0.830185, 0.719625, 0.830185, 0.719625, 0.830185, 0.718262, 0.834819, 0.719625, 0.830185, 0.719625, 0.830185, 0.718262, 0.834819, 0.718262, 0.834819, 0.718262, 0.834819, 0.716743, 0.836364, 0.718262, 0.834819, 0.718262, 0.834819, 0.716743, 0.836364, 0.716743, 0.836364, 0.716743, 0.836364, 0.715024, 0.834872, 0.716743, 0.836364, 0.716743, 0.836364, 0.715024, 0.834872, 0.715024, 0.834872, 0.715024, 0.834872, 0.713486, 0.830398, 0.715024, 0.834872, 0.715024, 0.834872, 0.713486, 0.830398, 0.713486, 0.830398, 0.713486, 0.830398, 0.712128, 0.822940, 0.713486, 0.830398, 0.713486, 0.830398, 0.712128, 0.822940, 0.712128, 0.822940, 0.712128, 0.822940, 0.710951, 0.812500, 0.712128, 0.822940, 0.712128, 0.822940, 0.710951, 0.812500, 0.710951, 0.812500, 0.710951, 0.812500, 0.709979, 0.799343, 0.710951, 0.812500, 0.710951, 0.812500, 0.709979, 0.799343, 0.709979, 0.799343, 0.709979, 0.799343, 0.709238, 0.783736, 0.709979, 0.799343, 0.709979, 0.799343, 0.709238, 0.783736, 0.709238, 0.783736, 0.709238, 0.783736, 0.708728, 0.765678, 0.709238, 0.783736, 0.709238, 0.783736, 0.708728, 0.765678, 0.708728, 0.765678, 0.708728, 0.765678, 0.708448, 0.745170, 0.708728, 0.765678, 0.708728, 0.765678, 0.708448, 0.745170, 0.708448, 0.745170, 0.762002, 0.709091, 0.759583, 0.708230, 0.762002, 0.709091, 0.762002, 0.709091, 0.759583, 0.708230, 0.759583, 0.708230, 0.759583, 0.708230, 0.757567, 0.705646, 0.759583, 0.708230, 0.759583, 0.708230, 0.757567, 0.705646, 0.757567, 0.705646, 0.757567, 0.705646, 0.755953, 0.701341, 0.757567, 0.705646, 0.757567, 0.705646, 0.755953, 0.701341, 0.755953, 0.701341, 0.755953, 0.701341, 0.754741, 0.695313, 0.755953, 0.701341, 0.755953, 0.701341, 0.754741, 0.695313, 0.754741, 0.695313, 0.754741, 0.695313, 0.753857, 0.687216, 0.754741, 0.695313, 0.754741, 0.695313, 0.753857, 0.687216, 0.753857, 0.687216, 0.753857, 0.687216, 0.753225, 0.676705, 0.753857, 0.687216, 0.753857, 0.687216, 0.753225, 0.676705, 0.753225, 0.676705, 0.753225, 0.676705, 0.752846, 0.663778, 0.753225, 0.676705, 0.753225, 0.676705, 0.752846, 0.663778, 0.752846, 0.663778, 0.752846, 0.663778, 0.752720, 0.648437, 0.752846, 0.663778, 0.752846, 0.663778, 0.752720, 0.648437, 0.752720, 0.648437, 0.752720, 0.648437, 0.752815, 0.635849, 0.752720, 0.648437, 0.752720, 0.648437, 0.752815, 0.635849, 0.752815, 0.635849, 0.752815, 0.635849, 0.753099, 0.624645, 0.752815, 0.635849, 0.752815, 0.635849, 0.753099, 0.624645, 0.753099, 0.624645, 0.753099, 0.624645, 0.753573, 0.614826, 0.753099, 0.624645, 0.753099, 0.624645, 0.753573, 0.614826, 0.753573, 0.614826, 0.753573, 0.614826, 0.754236, 0.606392, 0.753573, 0.614826, 0.753573, 0.614826, 0.754236, 0.606392, 0.754236, 0.606392, 0.754236, 0.606392, 0.755062, 0.599618, 0.754236, 0.606392, 0.754236, 0.606392, 0.755062, 0.599618, 0.755062, 0.599618, 0.755062, 0.599618, 0.756025, 0.594780, 0.755062, 0.599618, 0.755062, 0.599618, 0.756025, 0.594780, 0.756025, 0.594780, 0.756025, 0.594780, 0.757124, 0.591877, 0.756025, 0.594780, 0.756025, 0.594780, 0.757124, 0.591877, 0.757124, 0.591877, 0.757124, 0.591877, 0.758360, 0.590909, 0.757124, 0.591877, 0.757124, 0.591877, 0.758360, 0.590909, 0.758360, 0.590909, 0.758360, 0.590909, 0.760069, 0.592733, 0.758360, 0.590909, 0.758360, 0.590909, 0.760069, 0.592733, 0.760069, 0.592733, 0.760069, 0.592733, 0.761600, 0.598207, 0.760069, 0.592733, 0.760069, 0.592733, 0.761600, 0.598207, 0.761600, 0.598207, 0.761600, 0.598207, 0.762953, 0.607329, 0.761600, 0.598207, 0.761600, 0.598207, 0.762953, 0.607329, 0.762953, 0.607329, 0.762953, 0.607329, 0.764129, 0.620099, 0.762953, 0.607329, 0.762953, 0.607329, 0.764129, 0.620099, 0.764129, 0.620099, 0.764129, 0.620099, 0.765080, 0.635898, 0.764129, 0.620099, 0.764129, 0.620099, 0.765080, 0.635898, 0.765080, 0.635898, 0.765080, 0.635898, 0.765759, 0.654102, 0.765080, 0.635898, 0.765080, 0.635898, 0.765759, 0.654102, 0.765759, 0.654102, 0.765759, 0.654102, 0.766167, 0.674711, 0.765759, 0.654102, 0.765759, 0.654102, 0.766167, 0.674711, 0.766167, 0.674711, 0.766167, 0.674711, 0.766302, 0.697727, 0.766167, 0.674711, 0.766167, 0.674711, 0.766302, 0.697727, 0.766302, 0.697727, 0.766302, 0.697727, 0.766302, 0.709091, 0.766302, 0.697727, 0.766302, 0.697727, 0.766302, 0.709091, 0.766302, 0.709091, 0.770626, 0.733097, 0.770626, 0.554545, 0.770626, 0.733097, 0.770626, 0.733097, 0.770626, 0.554545, 0.770626, 0.554545, 0.770626, 0.554545, 0.766302, 0.554545, 0.770626, 0.554545, 0.770626, 0.554545, 0.766302, 0.554545, 0.766302, 0.554545, 0.766302, 0.554545, 0.766302, 0.600000, 0.766302, 0.554545, 0.766302, 0.554545, 0.766302, 0.600000, 0.766302, 0.600000, 0.766302, 0.600000, 0.765517, 0.586927, 0.766302, 0.600000, 0.766302, 0.600000, 0.765517, 0.586927, 0.765517, 0.586927, 0.765517, 0.586927, 0.764640, 0.575692, 0.765517, 0.586927, 0.765517, 0.586927, 0.764640, 0.575692, 0.764640, 0.575692, 0.764640, 0.575692, 0.763672, 0.566295, 0.764640, 0.575692, 0.764640, 0.575692, 0.763672, 0.566295, 0.763672, 0.566295, 0.763672, 0.566295, 0.762613, 0.558736, 0.763672, 0.566295, 0.763672, 0.566295, 0.762613, 0.558736, 0.762613, 0.558736, 0.762613, 0.558736, 0.761447, 0.552925, 0.762613, 0.558736, 0.762613, 0.558736, 0.761447, 0.552925, 0.761447, 0.552925, 0.761447, 0.552925, 0.760157, 0.548775, 0.761447, 0.552925, 0.761447, 0.552925, 0.760157, 0.548775, 0.760157, 0.548775, 0.760157, 0.548775, 0.758745, 0.546285, 0.760157, 0.548775, 0.760157, 0.548775, 0.758745, 0.546285, 0.758745, 0.546285, 0.758745, 0.546285, 0.757208, 0.545455, 0.758745, 0.546285, 0.758745, 0.546285, 0.757208, 0.545455, 0.757208, 0.545455, 0.757208, 0.545455, 0.755291, 0.547168, 0.757208, 0.545455, 0.757208, 0.545455, 0.755291, 0.547168, 0.755291, 0.547168, 0.755291, 0.547168, 0.753581, 0.552308, 0.755291, 0.547168, 0.755291, 0.547168, 0.753581, 0.552308, 0.753581, 0.552308, 0.753581, 0.552308, 0.752077, 0.560875, 0.753581, 0.552308, 0.753581, 0.552308, 0.752077, 0.560875, 0.752077, 0.560875, 0.752077, 0.560875, 0.750781, 0.572869, 0.752077, 0.560875, 0.752077, 0.560875, 0.750781, 0.572869, 0.750781, 0.572869, 0.750781, 0.572869, 0.749738, 0.587740, 0.750781, 0.572869, 0.750781, 0.572869, 0.749738, 0.587740, 0.749738, 0.587740, 0.749738, 0.587740, 0.748992, 0.604936, 0.749738, 0.587740, 0.749738, 0.587740, 0.748992, 0.604936, 0.748992, 0.604936, 0.748992, 0.604936, 0.748545, 0.624458, 0.748992, 0.604936, 0.748992, 0.604936, 0.748545, 0.624458, 0.748545, 0.624458, 0.748545, 0.624458, 0.748396, 0.646307, 0.748545, 0.624458, 0.748545, 0.624458, 0.748396, 0.646307, 0.748396, 0.646307, 0.748396, 0.646307, 0.748582, 0.671502, 0.748396, 0.646307, 0.748396, 0.646307, 0.748582, 0.671502, 0.748582, 0.671502, 0.748582, 0.671502, 0.749139, 0.693395, 0.748582, 0.671502, 0.748582, 0.671502, 0.749139, 0.693395, 0.749139, 0.693395, 0.749139, 0.693395, 0.750068, 0.711985, 0.749139, 0.693395, 0.749139, 0.693395, 0.750068, 0.711985, 0.750068, 0.711985, 0.750068, 0.711985, 0.751369, 0.727273, 0.750068, 0.711985, 0.750068, 0.711985, 0.751369, 0.727273, 0.751369, 0.727273, 0.751369, 0.727273, 0.753038, 0.739205, 0.751369, 0.727273, 0.751369, 0.727273, 0.753038, 0.739205, 0.753038, 0.739205, 0.753038, 0.739205, 0.755073, 0.747727, 0.753038, 0.739205, 0.753038, 0.739205, 0.755073, 0.747727, 0.755073, 0.747727, 0.755073, 0.747727, 0.757473, 0.752841, 0.755073, 0.747727, 0.755073, 0.747727, 0.757473, 0.752841, 0.757473, 0.752841, 0.757473, 0.752841, 0.760240, 0.754545, 0.757473, 0.752841, 0.757473, 0.752841, 0.760240, 0.754545, 0.760240, 0.754545, 0.760240, 0.754545, 0.766302, 0.754545, 0.760240, 0.754545, 0.760240, 0.754545, 0.766302, 0.754545, 0.766302, 0.754545, 0.766302, 0.754545, 0.766302, 0.759659, 0.766302, 0.754545, 0.766302, 0.754545, 0.766302, 0.759659, 0.766302, 0.759659, 0.766302, 0.759659, 0.766180, 0.776944, 0.766302, 0.759659, 0.766302, 0.759659, 0.766180, 0.776944, 0.766180, 0.776944, 0.766180, 0.776944, 0.765812, 0.792152, 0.766180, 0.776944, 0.766180, 0.776944, 0.765812, 0.792152, 0.765812, 0.792152, 0.765812, 0.792152, 0.765199, 0.805282, 0.765812, 0.792152, 0.765812, 0.792152, 0.765199, 0.805282, 0.765199, 0.805282, 0.765199, 0.805282, 0.764340, 0.816335, 0.765199, 0.805282, 0.765199, 0.805282, 0.764340, 0.816335, 0.764340, 0.816335, 0.764340, 0.816335, 0.763260, 0.825098, 0.764340, 0.816335, 0.764340, 0.816335, 0.763260, 0.825098, 0.763260, 0.825098, 0.763260, 0.825098, 0.761982, 0.831357, 0.763260, 0.825098, 0.763260, 0.825098, 0.761982, 0.831357, 0.761982, 0.831357, 0.761982, 0.831357, 0.760505, 0.835112, 0.761982, 0.831357, 0.761982, 0.831357, 0.760505, 0.835112, 0.760505, 0.835112, 0.760505, 0.835112, 0.758830, 0.836364, 0.760505, 0.835112, 0.760505, 0.835112, 0.758830, 0.836364, 0.758830, 0.836364, 0.758830, 0.836364, 0.757709, 0.835937, 0.758830, 0.836364, 0.758830, 0.836364, 0.757709, 0.835937, 0.757709, 0.835937, 0.757709, 0.835937, 0.756603, 0.834659, 0.757709, 0.835937, 0.757709, 0.835937, 0.756603, 0.834659, 0.756603, 0.834659, 0.756603, 0.834659, 0.755512, 0.832528, 0.756603, 0.834659, 0.756603, 0.834659, 0.755512, 0.832528, 0.755512, 0.832528, 0.755512, 0.832528, 0.754435, 0.829545, 0.755512, 0.832528, 0.755512, 0.832528, 0.754435, 0.829545, 0.754435, 0.829545, 0.754435, 0.829545, 0.753376, 0.825710, 0.754435, 0.829545, 0.754435, 0.829545, 0.753376, 0.825710, 0.753376, 0.825710, 0.753376, 0.825710, 0.752338, 0.821023, 0.753376, 0.825710, 0.753376, 0.825710, 0.752338, 0.821023, 0.752338, 0.821023, 0.752338, 0.821023, 0.751320, 0.815483, 0.752338, 0.821023, 0.752338, 0.821023, 0.751320, 0.815483, 0.751320, 0.815483, 0.751320, 0.815483, 0.750323, 0.809091, 0.751320, 0.815483, 0.751320, 0.815483, 0.750323, 0.809091, 0.750323, 0.809091, 0.750323, 0.809091, 0.750323, 0.854545, 0.750323, 0.809091, 0.750323, 0.809091, 0.750323, 0.854545, 0.750323, 0.854545, 0.750323, 0.854545, 0.751501, 0.860937, 0.750323, 0.854545, 0.750323, 0.854545, 0.751501, 0.860937, 0.751501, 0.860937, 0.751501, 0.860937, 0.752661, 0.866477, 0.751501, 0.860937, 0.751501, 0.860937, 0.752661, 0.866477, 0.752661, 0.866477, 0.752661, 0.866477, 0.753804, 0.871165, 0.752661, 0.866477, 0.752661, 0.866477, 0.753804, 0.871165, 0.753804, 0.871165, 0.753804, 0.871165, 0.754929, 0.875000, 0.753804, 0.871165, 0.753804, 0.871165, 0.754929, 0.875000, 0.754929, 0.875000, 0.754929, 0.875000, 0.756038, 0.877983, 0.754929, 0.875000, 0.754929, 0.875000, 0.756038, 0.877983, 0.756038, 0.877983, 0.756038, 0.877983, 0.757132, 0.880114, 0.756038, 0.877983, 0.756038, 0.877983, 0.757132, 0.880114, 0.757132, 0.880114, 0.757132, 0.880114, 0.758211, 0.881392, 0.757132, 0.880114, 0.757132, 0.880114, 0.758211, 0.881392, 0.758211, 0.881392, 0.758211, 0.881392, 0.759276, 0.881818, 0.758211, 0.881392, 0.758211, 0.881392, 0.759276, 0.881818, 0.759276, 0.881818, 0.759276, 0.881818, 0.761951, 0.879514, 0.759276, 0.881818, 0.759276, 0.881818, 0.761951, 0.879514, 0.761951, 0.879514, 0.761951, 0.879514, 0.764264, 0.872603, 0.761951, 0.879514, 0.761951, 0.879514, 0.764264, 0.872603, 0.764264, 0.872603, 0.764264, 0.872603, 0.766216, 0.861084, 0.764264, 0.872603, 0.764264, 0.872603, 0.766216, 0.861084, 0.766216, 0.861084, 0.766216, 0.861084, 0.767806, 0.844957, 0.766216, 0.861084, 0.766216, 0.861084, 0.767806, 0.844957, 0.767806, 0.844957, 0.767806, 0.844957, 0.769040, 0.824143, 0.767806, 0.844957, 0.767806, 0.844957, 0.769040, 0.824143, 0.769040, 0.824143, 0.769040, 0.824143, 0.769921, 0.798562, 0.769040, 0.824143, 0.769040, 0.824143, 0.769921, 0.798562, 0.769921, 0.798562, 0.769921, 0.798562, 0.770450, 0.768213, 0.769921, 0.798562, 0.769921, 0.798562, 0.770450, 0.768213, 0.770450, 0.768213, 0.748582, 0.671502, 0.748396, 0.646307, 0.752720, 0.648437, 0.748396, 0.646307, 0.748582, 0.671502, 0.752720, 0.648437, 0.748582, 0.671502, 0.752720, 0.648437, 0.752846, 0.663778, 0.752720, 0.648437, 0.748582, 0.671502, 0.752846, 0.663778, 0.749139, 0.693395, 0.748582, 0.671502, 0.752846, 0.663778, 0.748582, 0.671502, 0.749139, 0.693395, 0.752846, 0.663778, 0.749139, 0.693395, 0.752846, 0.663778, 0.753225, 0.676705, 0.752846, 0.663778, 0.749139, 0.693395, 0.753225, 0.676705, 0.750068, 0.711985, 0.749139, 0.693395, 0.753225, 0.676705, 0.749139, 0.693395, 0.750068, 0.711985, 0.753225, 0.676705, 0.750068, 0.711985, 0.753225, 0.676705, 0.753857, 0.687216, 0.753225, 0.676705, 0.750068, 0.711985, 0.753857, 0.687216, 0.751369, 0.727273, 0.750068, 0.711985, 0.753857, 0.687216, 0.750068, 0.711985, 0.751369, 0.727273, 0.753857, 0.687216, 0.751369, 0.727273, 0.753857, 0.687216, 0.754741, 0.695313, 0.753857, 0.687216, 0.751369, 0.727273, 0.754741, 0.695313, 0.753038, 0.739205, 0.751369, 0.727273, 0.754741, 0.695313, 0.751369, 0.727273, 0.753038, 0.739205, 0.754741, 0.695313, 0.755953, 0.701341, 0.753038, 0.739205, 0.754741, 0.695313, 0.753038, 0.739205, 0.755953, 0.701341, 0.754741, 0.695313, 0.755073, 0.747727, 0.753038, 0.739205, 0.755953, 0.701341, 0.753038, 0.739205, 0.755073, 0.747727, 0.755953, 0.701341, 0.757567, 0.705646, 0.755073, 0.747727, 0.755953, 0.701341, 0.755073, 0.747727, 0.757567, 0.705646, 0.755953, 0.701341, 0.757473, 0.752841, 0.755073, 0.747727, 0.757567, 0.705646, 0.755073, 0.747727, 0.757473, 0.752841, 0.757567, 0.705646, 0.757473, 0.752841, 0.757567, 0.705646, 0.759583, 0.708230, 0.757567, 0.705646, 0.757473, 0.752841, 0.759583, 0.708230, 0.757473, 0.752841, 0.759583, 0.708230, 0.760240, 0.754545, 0.759583, 0.708230, 0.757473, 0.752841, 0.760240, 0.754545, 0.760240, 0.754545, 0.759583, 0.708230, 0.762002, 0.709091, 0.759583, 0.708230, 0.760240, 0.754545, 0.762002, 0.709091, 0.760240, 0.754545, 0.762002, 0.709091, 0.766302, 0.754545, 0.762002, 0.709091, 0.760240, 0.754545, 0.766302, 0.754545, 0.762002, 0.709091, 0.766302, 0.709091, 0.766302, 0.754545, 0.766302, 0.709091, 0.762002, 0.709091, 0.766302, 0.754545, 0.766302, 0.754545, 0.766302, 0.709091, 0.770626, 0.733097, 0.766302, 0.709091, 0.766302, 0.754545, 0.770626, 0.733097, 0.770626, 0.733097, 0.770450, 0.768213, 0.766302, 0.754545, 0.770450, 0.768213, 0.770626, 0.733097, 0.766302, 0.754545, 0.770450, 0.768213, 0.766302, 0.759659, 0.766302, 0.754545, 0.766302, 0.759659, 0.770450, 0.768213, 0.766302, 0.754545, 0.766180, 0.776944, 0.766302, 0.759659, 0.770450, 0.768213, 0.766302, 0.759659, 0.766180, 0.776944, 0.770450, 0.768213, 0.769921, 0.798562, 0.766180, 0.776944, 0.770450, 0.768213, 0.766180, 0.776944, 0.769921, 0.798562, 0.770450, 0.768213, 0.765812, 0.792152, 0.766180, 0.776944, 0.769921, 0.798562, 0.766180, 0.776944, 0.765812, 0.792152, 0.769921, 0.798562, 0.769040, 0.824143, 0.765812, 0.792152, 0.769921, 0.798562, 0.765812, 0.792152, 0.769040, 0.824143, 0.769921, 0.798562, 0.765199, 0.805282, 0.765812, 0.792152, 0.769040, 0.824143, 0.765812, 0.792152, 0.765199, 0.805282, 0.769040, 0.824143, 0.767806, 0.844957, 0.765199, 0.805282, 0.769040, 0.824143, 0.765199, 0.805282, 0.767806, 0.844957, 0.769040, 0.824143, 0.764340, 0.816335, 0.765199, 0.805282, 0.767806, 0.844957, 0.765199, 0.805282, 0.764340, 0.816335, 0.767806, 0.844957, 0.766216, 0.861084, 0.764340, 0.816335, 0.767806, 0.844957, 0.764340, 0.816335, 0.766216, 0.861084, 0.767806, 0.844957, 0.763260, 0.825098, 0.764340, 0.816335, 0.766216, 0.861084, 0.764340, 0.816335, 0.763260, 0.825098, 0.766216, 0.861084, 0.764264, 0.872603, 0.763260, 0.825098, 0.766216, 0.861084, 0.763260, 0.825098, 0.764264, 0.872603, 0.766216, 0.861084, 0.761982, 0.831357, 0.763260, 0.825098, 0.764264, 0.872603, 0.763260, 0.825098, 0.761982, 0.831357, 0.764264, 0.872603, 0.761951, 0.879514, 0.761982, 0.831357, 0.764264, 0.872603, 0.761982, 0.831357, 0.761951, 0.879514, 0.764264, 0.872603, 0.761951, 0.879514, 0.760505, 0.835112, 0.761982, 0.831357, 0.760505, 0.835112, 0.761951, 0.879514, 0.761982, 0.831357, 0.761951, 0.879514, 0.759276, 0.881818, 0.760505, 0.835112, 0.759276, 0.881818, 0.761951, 0.879514, 0.760505, 0.835112, 0.758830, 0.836364, 0.760505, 0.835112, 0.759276, 0.881818, 0.760505, 0.835112, 0.758830, 0.836364, 0.759276, 0.881818, 0.759276, 0.881818, 0.758211, 0.881392, 0.758830, 0.836364, 0.758211, 0.881392, 0.759276, 0.881818, 0.758830, 0.836364, 0.758211, 0.881392, 0.757709, 0.835937, 0.758830, 0.836364, 0.757709, 0.835937, 0.758211, 0.881392, 0.758830, 0.836364, 0.757132, 0.880114, 0.757709, 0.835937, 0.758211, 0.881392, 0.757709, 0.835937, 0.757132, 0.880114, 0.758211, 0.881392, 0.757132, 0.880114, 0.756603, 0.834659, 0.757709, 0.835937, 0.756603, 0.834659, 0.757132, 0.880114, 0.757709, 0.835937, 0.756038, 0.877983, 0.756603, 0.834659, 0.757132, 0.880114, 0.756603, 0.834659, 0.756038, 0.877983, 0.757132, 0.880114, 0.756038, 0.877983, 0.754929, 0.875000, 0.756603, 0.834659, 0.754929, 0.875000, 0.756038, 0.877983, 0.756603, 0.834659, 0.756603, 0.834659, 0.754929, 0.875000, 0.755512, 0.832528, 0.754929, 0.875000, 0.756603, 0.834659, 0.755512, 0.832528, 0.754929, 0.875000, 0.753804, 0.871165, 0.755512, 0.832528, 0.753804, 0.871165, 0.754929, 0.875000, 0.755512, 0.832528, 0.753804, 0.871165, 0.754435, 0.829545, 0.755512, 0.832528, 0.754435, 0.829545, 0.753804, 0.871165, 0.755512, 0.832528, 0.753804, 0.871165, 0.752661, 0.866477, 0.754435, 0.829545, 0.752661, 0.866477, 0.753804, 0.871165, 0.754435, 0.829545, 0.752661, 0.866477, 0.753376, 0.825710, 0.754435, 0.829545, 0.753376, 0.825710, 0.752661, 0.866477, 0.754435, 0.829545, 0.752661, 0.866477, 0.751501, 0.860937, 0.753376, 0.825710, 0.751501, 0.860937, 0.752661, 0.866477, 0.753376, 0.825710, 0.751501, 0.860937, 0.752338, 0.821023, 0.753376, 0.825710, 0.752338, 0.821023, 0.751501, 0.860937, 0.753376, 0.825710, 0.751501, 0.860937, 0.750323, 0.854545, 0.752338, 0.821023, 0.750323, 0.854545, 0.751501, 0.860937, 0.752338, 0.821023, 0.750323, 0.854545, 0.751320, 0.815483, 0.752338, 0.821023, 0.751320, 0.815483, 0.750323, 0.854545, 0.752338, 0.821023, 0.750323, 0.854545, 0.750323, 0.809091, 0.751320, 0.815483, 0.750323, 0.809091, 0.750323, 0.854545, 0.751320, 0.815483, 0.770626, 0.733097, 0.766302, 0.709091, 0.766302, 0.697727, 0.766302, 0.709091, 0.770626, 0.733097, 0.766302, 0.697727, 0.770626, 0.733097, 0.766302, 0.697727, 0.766167, 0.674711, 0.766302, 0.697727, 0.770626, 0.733097, 0.766167, 0.674711, 0.770626, 0.554545, 0.770626, 0.733097, 0.766167, 0.674711, 0.770626, 0.733097, 0.770626, 0.554545, 0.766167, 0.674711, 0.770626, 0.554545, 0.766167, 0.674711, 0.765759, 0.654102, 0.766167, 0.674711, 0.770626, 0.554545, 0.765759, 0.654102, 0.766302, 0.600000, 0.770626, 0.554545, 0.765759, 0.654102, 0.770626, 0.554545, 0.766302, 0.600000, 0.765759, 0.654102, 0.766302, 0.600000, 0.766302, 0.554545, 0.770626, 0.554545, 0.766302, 0.554545, 0.766302, 0.600000, 0.770626, 0.554545, 0.765759, 0.654102, 0.765080, 0.635898, 0.766302, 0.600000, 0.765080, 0.635898, 0.765759, 0.654102, 0.766302, 0.600000, 0.765080, 0.635898, 0.764129, 0.620099, 0.766302, 0.600000, 0.764129, 0.620099, 0.765080, 0.635898, 0.766302, 0.600000, 0.764129, 0.620099, 0.765517, 0.586927, 0.766302, 0.600000, 0.765517, 0.586927, 0.764129, 0.620099, 0.766302, 0.600000, 0.764129, 0.620099, 0.762953, 0.607329, 0.765517, 0.586927, 0.762953, 0.607329, 0.764129, 0.620099, 0.765517, 0.586927, 0.762953, 0.607329, 0.764640, 0.575692, 0.765517, 0.586927, 0.764640, 0.575692, 0.762953, 0.607329, 0.765517, 0.586927, 0.764640, 0.575692, 0.762953, 0.607329, 0.763672, 0.566295, 0.762953, 0.607329, 0.764640, 0.575692, 0.763672, 0.566295, 0.762953, 0.607329, 0.761600, 0.598207, 0.763672, 0.566295, 0.761600, 0.598207, 0.762953, 0.607329, 0.763672, 0.566295, 0.761600, 0.598207, 0.762613, 0.558736, 0.763672, 0.566295, 0.762613, 0.558736, 0.761600, 0.598207, 0.763672, 0.566295, 0.762613, 0.558736, 0.761600, 0.598207, 0.761447, 0.552925, 0.761600, 0.598207, 0.762613, 0.558736, 0.761447, 0.552925, 0.761600, 0.598207, 0.760069, 0.592733, 0.761447, 0.552925, 0.760069, 0.592733, 0.761600, 0.598207, 0.761447, 0.552925, 0.760069, 0.592733, 0.760157, 0.548775, 0.761447, 0.552925, 0.760157, 0.548775, 0.760069, 0.592733, 0.761447, 0.552925, 0.760069, 0.592733, 0.758360, 0.590909, 0.760157, 0.548775, 0.758360, 0.590909, 0.760069, 0.592733, 0.760157, 0.548775, 0.758360, 0.590909, 0.758745, 0.546285, 0.760157, 0.548775, 0.758745, 0.546285, 0.758360, 0.590909, 0.760157, 0.548775, 0.758360, 0.590909, 0.757208, 0.545455, 0.758745, 0.546285, 0.757208, 0.545455, 0.758360, 0.590909, 0.758745, 0.546285, 0.757124, 0.591877, 0.757208, 0.545455, 0.758360, 0.590909, 0.757208, 0.545455, 0.757124, 0.591877, 0.758360, 0.590909, 0.757124, 0.591877, 0.755291, 0.547168, 0.757208, 0.545455, 0.755291, 0.547168, 0.757124, 0.591877, 0.757208, 0.545455, 0.756025, 0.594780, 0.755291, 0.547168, 0.757124, 0.591877, 0.755291, 0.547168, 0.756025, 0.594780, 0.757124, 0.591877, 0.753581, 0.552308, 0.755291, 0.547168, 0.756025, 0.594780, 0.755291, 0.547168, 0.753581, 0.552308, 0.756025, 0.594780, 0.755062, 0.599618, 0.753581, 0.552308, 0.756025, 0.594780, 0.753581, 0.552308, 0.755062, 0.599618, 0.756025, 0.594780, 0.752077, 0.560875, 0.753581, 0.552308, 0.755062, 0.599618, 0.753581, 0.552308, 0.752077, 0.560875, 0.755062, 0.599618, 0.755062, 0.599618, 0.754236, 0.606392, 0.752077, 0.560875, 0.754236, 0.606392, 0.755062, 0.599618, 0.752077, 0.560875, 0.754236, 0.606392, 0.750781, 0.572869, 0.752077, 0.560875, 0.750781, 0.572869, 0.754236, 0.606392, 0.752077, 0.560875, 0.754236, 0.606392, 0.753573, 0.614826, 0.750781, 0.572869, 0.753573, 0.614826, 0.754236, 0.606392, 0.750781, 0.572869, 0.753573, 0.614826, 0.749738, 0.587740, 0.750781, 0.572869, 0.749738, 0.587740, 0.753573, 0.614826, 0.750781, 0.572869, 0.753573, 0.614826, 0.753099, 0.624645, 0.749738, 0.587740, 0.753099, 0.624645, 0.753573, 0.614826, 0.749738, 0.587740, 0.753099, 0.624645, 0.748992, 0.604936, 0.749738, 0.587740, 0.748992, 0.604936, 0.753099, 0.624645, 0.749738, 0.587740, 0.748545, 0.624458, 0.748992, 0.604936, 0.753099, 0.624645, 0.748992, 0.604936, 0.748545, 0.624458, 0.753099, 0.624645, 0.752815, 0.635849, 0.748545, 0.624458, 0.753099, 0.624645, 0.748545, 0.624458, 0.752815, 0.635849, 0.753099, 0.624645, 0.748396, 0.646307, 0.748545, 0.624458, 0.752815, 0.635849, 0.748545, 0.624458, 0.748396, 0.646307, 0.752815, 0.635849, 0.752720, 0.648437, 0.748396, 0.646307, 0.752815, 0.635849, 0.748396, 0.646307, 0.752720, 0.648437, 0.752815, 0.635849, 0.796146, 0.854545, 0.796146, 0.809091, 0.796146, 0.854545, 0.796146, 0.854545, 0.796146, 0.809091, 0.796146, 0.809091, 0.796146, 0.809091, 0.795221, 0.815483, 0.796146, 0.809091, 0.796146, 0.809091, 0.795221, 0.815483, 0.795221, 0.815483, 0.795221, 0.815483, 0.794278, 0.821023, 0.795221, 0.815483, 0.795221, 0.815483, 0.794278, 0.821023, 0.794278, 0.821023, 0.794278, 0.821023, 0.793317, 0.825710, 0.794278, 0.821023, 0.794278, 0.821023, 0.793317, 0.825710, 0.793317, 0.825710, 0.793317, 0.825710, 0.792339, 0.829545, 0.793317, 0.825710, 0.793317, 0.825710, 0.792339, 0.829545, 0.792339, 0.829545, 0.792339, 0.829545, 0.791344, 0.832528, 0.792339, 0.829545, 0.792339, 0.829545, 0.791344, 0.832528, 0.791344, 0.832528, 0.791344, 0.832528, 0.790330, 0.834659, 0.791344, 0.832528, 0.791344, 0.832528, 0.790330, 0.834659, 0.790330, 0.834659, 0.790330, 0.834659, 0.789299, 0.835937, 0.790330, 0.834659, 0.790330, 0.834659, 0.789299, 0.835937, 0.789299, 0.835937, 0.789299, 0.835937, 0.788251, 0.836364, 0.789299, 0.835937, 0.789299, 0.835937, 0.788251, 0.836364, 0.788251, 0.836364, 0.788251, 0.836364, 0.786741, 0.835591, 0.788251, 0.836364, 0.788251, 0.836364, 0.786741, 0.835591, 0.786741, 0.835591, 0.786741, 0.835591, 0.785434, 0.833274, 0.786741, 0.835591, 0.786741, 0.835591, 0.785434, 0.833274, 0.785434, 0.833274, 0.785434, 0.833274, 0.784327, 0.829412, 0.785434, 0.833274, 0.785434, 0.833274, 0.784327, 0.829412, 0.784327, 0.829412, 0.784327, 0.829412, 0.783421, 0.824006, 0.784327, 0.829412, 0.784327, 0.829412, 0.783421, 0.824006, 0.783421, 0.824006, 0.783421, 0.824006, 0.782717, 0.817054, 0.783421, 0.824006, 0.783421, 0.824006, 0.782717, 0.817054, 0.782717, 0.817054, 0.782717, 0.817054, 0.782214, 0.808558, 0.782717, 0.817054, 0.782717, 0.817054, 0.782214, 0.808558, 0.782214, 0.808558, 0.782214, 0.808558, 0.781912, 0.798517, 0.782214, 0.808558, 0.782214, 0.808558, 0.781912, 0.798517, 0.781912, 0.798517, 0.781912, 0.798517, 0.781812, 0.786932, 0.781912, 0.798517, 0.781912, 0.798517, 0.781812, 0.786932, 0.781812, 0.786932, 0.781812, 0.786932, 0.781884, 0.778054, 0.781812, 0.786932, 0.781812, 0.786932, 0.781884, 0.778054, 0.781884, 0.778054, 0.781884, 0.778054, 0.782100, 0.770170, 0.781884, 0.778054, 0.781884, 0.778054, 0.782100, 0.770170, 0.782100, 0.770170, 0.782100, 0.770170, 0.782459, 0.763281, 0.782100, 0.770170, 0.782100, 0.770170, 0.782459, 0.763281, 0.782459, 0.763281, 0.782459, 0.763281, 0.782963, 0.757386, 0.782459, 0.763281, 0.782459, 0.763281, 0.782963, 0.757386, 0.782963, 0.757386, 0.782963, 0.757386, 0.783684, 0.752060, 0.782963, 0.757386, 0.782963, 0.757386, 0.783684, 0.752060, 0.783684, 0.752060, 0.783684, 0.752060, 0.784696, 0.746875, 0.783684, 0.752060, 0.783684, 0.752060, 0.784696, 0.746875, 0.784696, 0.746875, 0.784696, 0.746875, 0.785999, 0.741832, 0.784696, 0.746875, 0.784696, 0.746875, 0.785999, 0.741832, 0.785999, 0.741832, 0.785999, 0.741832, 0.787592, 0.736932, 0.785999, 0.741832, 0.785999, 0.741832, 0.787592, 0.736932, 0.787592, 0.736932, 0.787592, 0.736932, 0.789073, 0.732955, 0.787592, 0.736932, 0.787592, 0.736932, 0.789073, 0.732955, 0.789073, 0.732955, 0.789073, 0.732955, 0.791209, 0.726460, 0.789073, 0.732955, 0.789073, 0.732955, 0.791209, 0.726460, 0.791209, 0.726460, 0.791209, 0.726460, 0.793012, 0.718768, 0.791209, 0.726460, 0.791209, 0.726460, 0.793012, 0.718768, 0.793012, 0.718768, 0.793012, 0.718768, 0.794481, 0.709877, 0.793012, 0.718768, 0.793012, 0.718768, 0.794481, 0.709877, 0.794481, 0.709877, 0.794481, 0.709877, 0.795617, 0.699787, 0.794481, 0.709877, 0.794481, 0.709877, 0.795617, 0.699787, 0.795617, 0.699787, 0.795617, 0.699787, 0.796466, 0.688037, 0.795617, 0.699787, 0.795617, 0.699787, 0.796466, 0.688037, 0.796466, 0.688037, 0.796466, 0.688037, 0.797071, 0.674165, 0.796466, 0.688037, 0.796466, 0.688037, 0.797071, 0.674165, 0.797071, 0.674165, 0.797071, 0.674165, 0.797435, 0.658172, 0.797071, 0.674165, 0.797071, 0.674165, 0.797435, 0.658172, 0.797435, 0.658172, 0.797435, 0.658172, 0.797556, 0.640057, 0.797435, 0.658172, 0.797435, 0.658172, 0.797556, 0.640057, 0.797556, 0.640057, 0.797556, 0.640057, 0.797375, 0.619385, 0.797556, 0.640057, 0.797556, 0.640057, 0.797375, 0.619385, 0.797375, 0.619385, 0.797375, 0.619385, 0.796831, 0.600977, 0.797375, 0.619385, 0.797375, 0.619385, 0.796831, 0.600977, 0.796831, 0.600977, 0.796831, 0.600977, 0.795924, 0.584832, 0.796831, 0.600977, 0.796831, 0.600977, 0.795924, 0.584832, 0.795924, 0.584832, 0.795924, 0.584832, 0.794654, 0.570952, 0.795924, 0.584832, 0.795924, 0.584832, 0.794654, 0.570952, 0.794654, 0.570952, 0.794654, 0.570952, 0.793067, 0.559797, 0.794654, 0.570952, 0.794654, 0.570952, 0.793067, 0.559797, 0.793067, 0.559797, 0.793067, 0.559797, 0.791208, 0.551829, 0.793067, 0.559797, 0.793067, 0.559797, 0.791208, 0.551829, 0.791208, 0.551829, 0.791208, 0.551829, 0.789078, 0.547048, 0.791208, 0.551829, 0.791208, 0.551829, 0.789078, 0.547048, 0.789078, 0.547048, 0.789078, 0.547048, 0.786676, 0.545455, 0.789078, 0.547048, 0.789078, 0.547048, 0.786676, 0.545455, 0.786676, 0.545455, 0.786676, 0.545455, 0.785608, 0.545739, 0.786676, 0.545455, 0.786676, 0.545455, 0.785608, 0.545739, 0.785608, 0.545739, 0.785608, 0.545739, 0.784517, 0.546591, 0.785608, 0.545739, 0.785608, 0.545739, 0.784517, 0.546591, 0.784517, 0.546591, 0.784517, 0.546591, 0.783404, 0.548011, 0.784517, 0.546591, 0.784517, 0.546591, 0.783404, 0.548011, 0.783404, 0.548011, 0.783404, 0.548011, 0.782270, 0.550000, 0.783404, 0.548011, 0.783404, 0.548011, 0.782270, 0.550000, 0.782270, 0.550000, 0.782270, 0.550000, 0.781109, 0.552557, 0.782270, 0.550000, 0.782270, 0.550000, 0.781109, 0.552557, 0.781109, 0.552557, 0.781109, 0.552557, 0.779917, 0.555682, 0.781109, 0.552557, 0.781109, 0.552557, 0.779917, 0.555682, 0.779917, 0.555682, 0.779917, 0.555682, 0.778695, 0.559375, 0.779917, 0.555682, 0.779917, 0.555682, 0.778695, 0.559375, 0.778695, 0.559375, 0.778695, 0.559375, 0.777441, 0.563636, 0.778695, 0.559375, 0.778695, 0.559375, 0.777441, 0.563636, 0.777441, 0.563636, 0.777441, 0.563636, 0.777441, 0.618182, 0.777441, 0.563636, 0.777441, 0.563636, 0.777441, 0.618182, 0.777441, 0.618182, 0.777441, 0.618182, 0.778635, 0.611790, 0.777441, 0.618182, 0.777441, 0.618182, 0.778635, 0.611790, 0.778635, 0.611790, 0.778635, 0.611790, 0.779820, 0.606250, 0.778635, 0.611790, 0.778635, 0.611790, 0.779820, 0.606250, 0.779820, 0.606250, 0.779820, 0.606250, 0.780997, 0.601563, 0.779820, 0.606250, 0.779820, 0.606250, 0.780997, 0.601563, 0.780997, 0.601563, 0.780997, 0.601563, 0.782164, 0.597727, 0.780997, 0.601563, 0.780997, 0.601563, 0.782164, 0.597727, 0.782164, 0.597727, 0.782164, 0.597727, 0.783325, 0.594744, 0.782164, 0.597727, 0.782164, 0.597727, 0.783325, 0.594744, 0.783325, 0.594744, 0.783325, 0.594744, 0.784479, 0.592614, 0.783325, 0.594744, 0.783325, 0.594744, 0.784479, 0.592614, 0.784479, 0.592614, 0.784479, 0.592614, 0.785627, 0.591335, 0.784479, 0.592614, 0.784479, 0.592614, 0.785627, 0.591335, 0.785627, 0.591335, 0.785627, 0.591335, 0.786770, 0.590909, 0.785627, 0.591335, 0.785627, 0.591335, 0.786770, 0.590909, 0.786770, 0.590909, 0.786770, 0.590909, 0.788209, 0.591668, 0.786770, 0.590909, 0.786770, 0.590909, 0.788209, 0.591668, 0.788209, 0.591668, 0.788209, 0.591668, 0.789472, 0.593945, 0.788209, 0.591668, 0.788209, 0.591668, 0.789472, 0.593945, 0.789472, 0.593945, 0.789472, 0.593945, 0.790559, 0.597741, 0.789472, 0.593945, 0.789472, 0.593945, 0.790559, 0.597741, 0.790559, 0.597741, 0.790559, 0.597741, 0.791470, 0.603054, 0.790559, 0.597741, 0.790559, 0.597741, 0.791470, 0.603054, 0.791470, 0.603054, 0.791470, 0.603054, 0.792190, 0.609752, 0.791470, 0.603054, 0.791470, 0.603054, 0.792190, 0.609752, 0.792190, 0.609752, 0.792190, 0.609752, 0.792704, 0.617702, 0.792190, 0.609752, 0.792190, 0.609752, 0.792704, 0.617702, 0.792704, 0.617702, 0.792704, 0.617702, 0.793012, 0.626904, 0.792704, 0.617702, 0.792704, 0.617702, 0.793012, 0.626904, 0.793012, 0.626904, 0.793012, 0.626904, 0.793115, 0.637358, 0.793012, 0.626904, 0.793012, 0.626904, 0.793115, 0.637358, 0.793115, 0.637358, 0.793115, 0.637358, 0.793041, 0.646990, 0.793115, 0.637358, 0.793115, 0.637358, 0.793041, 0.646990, 0.793041, 0.646990, 0.793041, 0.646990, 0.792818, 0.655433, 0.793041, 0.646990, 0.793041, 0.646990, 0.792818, 0.655433, 0.792818, 0.655433, 0.792818, 0.655433, 0.792447, 0.662686, 0.792818, 0.655433, 0.792818, 0.655433, 0.792447, 0.662686, 0.792447, 0.662686, 0.792447, 0.662686, 0.791928, 0.668750, 0.792447, 0.662686, 0.792447, 0.662686, 0.791928, 0.668750, 0.791928, 0.668750, 0.791928, 0.668750, 0.791158, 0.674165, 0.791928, 0.668750, 0.791928, 0.668750, 0.791158, 0.674165, 0.791158, 0.674165, 0.791158, 0.674165, 0.790033, 0.679474, 0.791158, 0.674165, 0.791158, 0.674165, 0.790033, 0.679474, 0.790033, 0.679474, 0.790033, 0.679474, 0.788555, 0.684677, 0.790033, 0.679474, 0.790033, 0.679474, 0.788555, 0.684677, 0.788555, 0.684677, 0.788555, 0.684677, 0.786723, 0.689773, 0.788555, 0.684677, 0.788555, 0.684677, 0.786723, 0.689773, 0.786723, 0.689773, 0.786723, 0.689773, 0.785219, 0.694034, 0.786723, 0.689773, 0.786723, 0.689773, 0.785219, 0.694034, 0.785219, 0.694034, 0.785219, 0.694034, 0.783349, 0.699987, 0.785219, 0.694034, 0.785219, 0.694034, 0.783349, 0.699987, 0.783349, 0.699987, 0.783349, 0.699987, 0.781759, 0.707333, 0.783349, 0.699987, 0.783349, 0.699987, 0.781759, 0.707333, 0.781759, 0.707333, 0.781759, 0.707333, 0.780447, 0.716073, 0.781759, 0.707333, 0.781759, 0.707333, 0.780447, 0.716073, 0.780447, 0.716073, 0.780447, 0.716073, 0.779415, 0.726207, 0.780447, 0.716073, 0.780447, 0.716073, 0.779415, 0.726207, 0.779415, 0.726207, 0.779415, 0.726207, 0.778634, 0.738055, 0.779415, 0.726207, 0.779415, 0.726207, 0.778634, 0.738055, 0.778634, 0.738055, 0.778634, 0.738055, 0.778075, 0.751935, 0.778634, 0.738055, 0.778634, 0.738055, 0.778075, 0.751935, 0.778075, 0.751935, 0.778075, 0.751935, 0.777741, 0.767849, 0.778075, 0.751935, 0.778075, 0.751935, 0.777741, 0.767849, 0.777741, 0.767849, 0.777741, 0.767849, 0.777629, 0.785795, 0.777741, 0.767849, 0.777741, 0.767849, 0.777629, 0.785795, 0.777629, 0.785795, 0.777629, 0.785795, 0.777793, 0.807435, 0.777629, 0.785795, 0.777629, 0.785795, 0.777793, 0.807435, 0.777793, 0.807435, 0.777793, 0.807435, 0.778287, 0.826474, 0.777793, 0.807435, 0.777793, 0.807435, 0.778287, 0.826474, 0.778287, 0.826474, 0.778287, 0.826474, 0.779109, 0.842911, 0.778287, 0.826474, 0.778287, 0.826474, 0.779109, 0.842911, 0.779109, 0.842911, 0.779109, 0.842911, 0.780261, 0.856747, 0.779109, 0.842911, 0.779109, 0.842911, 0.780261, 0.856747, 0.780261, 0.856747, 0.780261, 0.856747, 0.781715, 0.867716, 0.780261, 0.856747, 0.780261, 0.856747, 0.781715, 0.867716, 0.781715, 0.867716, 0.781715, 0.867716, 0.783445, 0.875550, 0.781715, 0.867716, 0.781715, 0.867716, 0.783445, 0.875550, 0.783445, 0.875550, 0.783445, 0.875550, 0.785451, 0.880251, 0.783445, 0.875550, 0.783445, 0.875550, 0.785451, 0.880251, 0.785451, 0.880251, 0.785451, 0.880251, 0.787733, 0.881818, 0.785451, 0.880251, 0.785451, 0.880251, 0.787733, 0.881818, 0.787733, 0.881818, 0.787733, 0.881818, 0.788914, 0.881392, 0.787733, 0.881818, 0.787733, 0.881818, 0.788914, 0.881392, 0.788914, 0.881392, 0.788914, 0.881392, 0.790060, 0.880114, 0.788914, 0.881392, 0.788914, 0.881392, 0.790060, 0.880114, 0.790060, 0.880114, 0.790060, 0.880114, 0.791170, 0.877983, 0.790060, 0.880114, 0.790060, 0.880114, 0.791170, 0.877983, 0.791170, 0.877983, 0.791170, 0.877983, 0.792245, 0.875000, 0.791170, 0.877983, 0.791170, 0.877983, 0.792245, 0.875000, 0.792245, 0.875000, 0.792245, 0.875000, 0.793282, 0.871165, 0.792245, 0.875000, 0.792245, 0.875000, 0.793282, 0.871165, 0.793282, 0.871165, 0.793282, 0.871165, 0.794278, 0.866477, 0.793282, 0.871165, 0.793282, 0.871165, 0.794278, 0.866477, 0.794278, 0.866477, 0.794278, 0.866477, 0.795233, 0.860937, 0.794278, 0.866477, 0.794278, 0.866477, 0.795233, 0.860937, 0.795233, 0.860937, 0.777441, 0.618182, 0.777441, 0.563636, 0.778635, 0.611790, 0.777441, 0.563636, 0.777441, 0.618182, 0.778635, 0.611790, 0.779820, 0.606250, 0.778635, 0.611790, 0.777441, 0.563636, 0.778635, 0.611790, 0.779820, 0.606250, 0.777441, 0.563636, 0.779820, 0.606250, 0.777441, 0.563636, 0.778695, 0.559375, 0.777441, 0.563636, 0.779820, 0.606250, 0.778695, 0.559375, 0.780997, 0.601563, 0.779820, 0.606250, 0.778695, 0.559375, 0.779820, 0.606250, 0.780997, 0.601563, 0.778695, 0.559375, 0.780997, 0.601563, 0.778695, 0.559375, 0.779917, 0.555682, 0.778695, 0.559375, 0.780997, 0.601563, 0.779917, 0.555682, 0.780997, 0.601563, 0.779917, 0.555682, 0.781109, 0.552557, 0.779917, 0.555682, 0.780997, 0.601563, 0.781109, 0.552557, 0.780997, 0.601563, 0.781109, 0.552557, 0.782164, 0.597727, 0.781109, 0.552557, 0.780997, 0.601563, 0.782164, 0.597727, 0.782164, 0.597727, 0.781109, 0.552557, 0.782270, 0.550000, 0.781109, 0.552557, 0.782164, 0.597727, 0.782270, 0.550000, 0.782164, 0.597727, 0.782270, 0.550000, 0.783325, 0.594744, 0.782270, 0.550000, 0.782164, 0.597727, 0.783325, 0.594744, 0.783325, 0.594744, 0.782270, 0.550000, 0.783404, 0.548011, 0.782270, 0.550000, 0.783325, 0.594744, 0.783404, 0.548011, 0.783325, 0.594744, 0.783404, 0.548011, 0.784479, 0.592614, 0.783404, 0.548011, 0.783325, 0.594744, 0.784479, 0.592614, 0.784479, 0.592614, 0.783404, 0.548011, 0.784517, 0.546591, 0.783404, 0.548011, 0.784479, 0.592614, 0.784517, 0.546591, 0.784479, 0.592614, 0.784517, 0.546591, 0.785627, 0.591335, 0.784517, 0.546591, 0.784479, 0.592614, 0.785627, 0.591335, 0.784517, 0.546591, 0.785608, 0.545739, 0.785627, 0.591335, 0.785608, 0.545739, 0.784517, 0.546591, 0.785627, 0.591335, 0.785627, 0.591335, 0.785608, 0.545739, 0.786770, 0.590909, 0.785608, 0.545739, 0.785627, 0.591335, 0.786770, 0.590909, 0.785608, 0.545739, 0.786676, 0.545455, 0.786770, 0.590909, 0.786676, 0.545455, 0.785608, 0.545739, 0.786770, 0.590909, 0.788209, 0.591668, 0.786770, 0.590909, 0.786676, 0.545455, 0.786770, 0.590909, 0.788209, 0.591668, 0.786676, 0.545455, 0.789078, 0.547048, 0.788209, 0.591668, 0.786676, 0.545455, 0.788209, 0.591668, 0.789078, 0.547048, 0.786676, 0.545455, 0.789472, 0.593945, 0.788209, 0.591668, 0.789078, 0.547048, 0.788209, 0.591668, 0.789472, 0.593945, 0.789078, 0.547048, 0.791208, 0.551829, 0.789472, 0.593945, 0.789078, 0.547048, 0.789472, 0.593945, 0.791208, 0.551829, 0.789078, 0.547048, 0.790559, 0.597741, 0.789472, 0.593945, 0.791208, 0.551829, 0.789472, 0.593945, 0.790559, 0.597741, 0.791208, 0.551829, 0.790559, 0.597741, 0.791208, 0.551829, 0.793067, 0.559797, 0.791208, 0.551829, 0.790559, 0.597741, 0.793067, 0.559797, 0.791470, 0.603054, 0.790559, 0.597741, 0.793067, 0.559797, 0.790559, 0.597741, 0.791470, 0.603054, 0.793067, 0.559797, 0.791470, 0.603054, 0.793067, 0.559797, 0.794654, 0.570952, 0.793067, 0.559797, 0.791470, 0.603054, 0.794654, 0.570952, 0.792190, 0.609752, 0.791470, 0.603054, 0.794654, 0.570952, 0.791470, 0.603054, 0.792190, 0.609752, 0.794654, 0.570952, 0.792190, 0.609752, 0.794654, 0.570952, 0.795924, 0.584832, 0.794654, 0.570952, 0.792190, 0.609752, 0.795924, 0.584832, 0.792704, 0.617702, 0.792190, 0.609752, 0.795924, 0.584832, 0.792190, 0.609752, 0.792704, 0.617702, 0.795924, 0.584832, 0.792704, 0.617702, 0.795924, 0.584832, 0.796831, 0.600977, 0.795924, 0.584832, 0.792704, 0.617702, 0.796831, 0.600977, 0.793012, 0.626904, 0.792704, 0.617702, 0.796831, 0.600977, 0.792704, 0.617702, 0.793012, 0.626904, 0.796831, 0.600977, 0.793012, 0.626904, 0.796831, 0.600977, 0.797375, 0.619385, 0.796831, 0.600977, 0.793012, 0.626904, 0.797375, 0.619385, 0.793115, 0.637358, 0.793012, 0.626904, 0.797375, 0.619385, 0.793012, 0.626904, 0.793115, 0.637358, 0.797375, 0.619385, 0.793115, 0.637358, 0.797375, 0.619385, 0.797556, 0.640057, 0.797375, 0.619385, 0.793115, 0.637358, 0.797556, 0.640057, 0.797556, 0.640057, 0.793041, 0.646990, 0.793115, 0.637358, 0.793041, 0.646990, 0.797556, 0.640057, 0.793115, 0.637358, 0.797435, 0.658172, 0.793041, 0.646990, 0.797556, 0.640057, 0.793041, 0.646990, 0.797435, 0.658172, 0.797556, 0.640057, 0.797071, 0.674165, 0.793041, 0.646990, 0.797435, 0.658172, 0.793041, 0.646990, 0.797071, 0.674165, 0.797435, 0.658172, 0.792818, 0.655433, 0.793041, 0.646990, 0.797071, 0.674165, 0.793041, 0.646990, 0.792818, 0.655433, 0.797071, 0.674165, 0.796466, 0.688037, 0.792818, 0.655433, 0.797071, 0.674165, 0.792818, 0.655433, 0.796466, 0.688037, 0.797071, 0.674165, 0.792447, 0.662686, 0.792818, 0.655433, 0.796466, 0.688037, 0.792818, 0.655433, 0.792447, 0.662686, 0.796466, 0.688037, 0.795617, 0.699787, 0.792447, 0.662686, 0.796466, 0.688037, 0.792447, 0.662686, 0.795617, 0.699787, 0.796466, 0.688037, 0.794481, 0.709877, 0.792447, 0.662686, 0.795617, 0.699787, 0.792447, 0.662686, 0.794481, 0.709877, 0.795617, 0.699787, 0.791928, 0.668750, 0.792447, 0.662686, 0.794481, 0.709877, 0.792447, 0.662686, 0.791928, 0.668750, 0.794481, 0.709877, 0.793012, 0.718768, 0.791928, 0.668750, 0.794481, 0.709877, 0.791928, 0.668750, 0.793012, 0.718768, 0.794481, 0.709877, 0.791158, 0.674165, 0.791928, 0.668750, 0.793012, 0.718768, 0.791928, 0.668750, 0.791158, 0.674165, 0.793012, 0.718768, 0.791209, 0.726460, 0.791158, 0.674165, 0.793012, 0.718768, 0.791158, 0.674165, 0.791209, 0.726460, 0.793012, 0.718768, 0.790033, 0.679474, 0.791158, 0.674165, 0.791209, 0.726460, 0.791158, 0.674165, 0.790033, 0.679474, 0.791209, 0.726460, 0.791209, 0.726460, 0.788555, 0.684677, 0.790033, 0.679474, 0.788555, 0.684677, 0.791209, 0.726460, 0.790033, 0.679474, 0.791209, 0.726460, 0.789073, 0.732955, 0.788555, 0.684677, 0.789073, 0.732955, 0.791209, 0.726460, 0.788555, 0.684677, 0.786723, 0.689773, 0.788555, 0.684677, 0.789073, 0.732955, 0.788555, 0.684677, 0.786723, 0.689773, 0.789073, 0.732955, 0.787592, 0.736932, 0.786723, 0.689773, 0.789073, 0.732955, 0.786723, 0.689773, 0.787592, 0.736932, 0.789073, 0.732955, 0.785219, 0.694034, 0.786723, 0.689773, 0.787592, 0.736932, 0.786723, 0.689773, 0.785219, 0.694034, 0.787592, 0.736932, 0.785999, 0.741832, 0.785219, 0.694034, 0.787592, 0.736932, 0.785219, 0.694034, 0.785999, 0.741832, 0.787592, 0.736932, 0.783349, 0.699987, 0.785219, 0.694034, 0.785999, 0.741832, 0.785219, 0.694034, 0.783349, 0.699987, 0.785999, 0.741832, 0.784696, 0.746875, 0.783349, 0.699987, 0.785999, 0.741832, 0.783349, 0.699987, 0.784696, 0.746875, 0.785999, 0.741832, 0.784696, 0.746875, 0.783684, 0.752060, 0.783349, 0.699987, 0.783684, 0.752060, 0.784696, 0.746875, 0.783349, 0.699987, 0.783684, 0.752060, 0.781759, 0.707333, 0.783349, 0.699987, 0.781759, 0.707333, 0.783684, 0.752060, 0.783349, 0.699987, 0.783684, 0.752060, 0.782963, 0.757386, 0.781759, 0.707333, 0.782963, 0.757386, 0.783684, 0.752060, 0.781759, 0.707333, 0.782963, 0.757386, 0.780447, 0.716073, 0.781759, 0.707333, 0.780447, 0.716073, 0.782963, 0.757386, 0.781759, 0.707333, 0.779415, 0.726207, 0.780447, 0.716073, 0.782963, 0.757386, 0.780447, 0.716073, 0.779415, 0.726207, 0.782963, 0.757386, 0.782963, 0.757386, 0.782459, 0.763281, 0.779415, 0.726207, 0.782459, 0.763281, 0.782963, 0.757386, 0.779415, 0.726207, 0.782459, 0.763281, 0.778634, 0.738055, 0.779415, 0.726207, 0.778634, 0.738055, 0.782459, 0.763281, 0.779415, 0.726207, 0.782459, 0.763281, 0.782100, 0.770170, 0.778634, 0.738055, 0.782100, 0.770170, 0.782459, 0.763281, 0.778634, 0.738055, 0.782100, 0.770170, 0.778075, 0.751935, 0.778634, 0.738055, 0.778075, 0.751935, 0.782100, 0.770170, 0.778634, 0.738055, 0.782100, 0.770170, 0.781884, 0.778054, 0.778075, 0.751935, 0.781884, 0.778054, 0.782100, 0.770170, 0.778075, 0.751935, 0.781884, 0.778054, 0.777741, 0.767849, 0.778075, 0.751935, 0.777741, 0.767849, 0.781884, 0.778054, 0.778075, 0.751935, 0.777629, 0.785795, 0.777741, 0.767849, 0.781884, 0.778054, 0.777741, 0.767849, 0.777629, 0.785795, 0.781884, 0.778054, 0.781812, 0.786932, 0.777629, 0.785795, 0.781884, 0.778054, 0.777629, 0.785795, 0.781812, 0.786932, 0.781884, 0.778054, 0.777793, 0.807435, 0.777629, 0.785795, 0.781812, 0.786932, 0.777629, 0.785795, 0.777793, 0.807435, 0.781812, 0.786932, 0.777793, 0.807435, 0.781812, 0.786932, 0.781912, 0.798517, 0.781812, 0.786932, 0.777793, 0.807435, 0.781912, 0.798517, 0.778287, 0.826474, 0.777793, 0.807435, 0.781912, 0.798517, 0.777793, 0.807435, 0.778287, 0.826474, 0.781912, 0.798517, 0.778287, 0.826474, 0.781912, 0.798517, 0.782214, 0.808558, 0.781912, 0.798517, 0.778287, 0.826474, 0.782214, 0.808558, 0.779109, 0.842911, 0.778287, 0.826474, 0.782214, 0.808558, 0.778287, 0.826474, 0.779109, 0.842911, 0.782214, 0.808558, 0.779109, 0.842911, 0.782214, 0.808558, 0.782717, 0.817054, 0.782214, 0.808558, 0.779109, 0.842911, 0.782717, 0.817054, 0.780261, 0.856747, 0.779109, 0.842911, 0.782717, 0.817054, 0.779109, 0.842911, 0.780261, 0.856747, 0.782717, 0.817054, 0.780261, 0.856747, 0.782717, 0.817054, 0.783421, 0.824006, 0.782717, 0.817054, 0.780261, 0.856747, 0.783421, 0.824006, 0.781715, 0.867716, 0.780261, 0.856747, 0.783421, 0.824006, 0.780261, 0.856747, 0.781715, 0.867716, 0.783421, 0.824006, 0.781715, 0.867716, 0.783421, 0.824006, 0.784327, 0.829412, 0.783421, 0.824006, 0.781715, 0.867716, 0.784327, 0.829412, 0.783445, 0.875550, 0.781715, 0.867716, 0.784327, 0.829412, 0.781715, 0.867716, 0.783445, 0.875550, 0.784327, 0.829412, 0.783445, 0.875550, 0.784327, 0.829412, 0.785434, 0.833274, 0.784327, 0.829412, 0.783445, 0.875550, 0.785434, 0.833274, 0.783445, 0.875550, 0.785434, 0.833274, 0.785451, 0.880251, 0.785434, 0.833274, 0.783445, 0.875550, 0.785451, 0.880251, 0.785451, 0.880251, 0.785434, 0.833274, 0.786741, 0.835591, 0.785434, 0.833274, 0.785451, 0.880251, 0.786741, 0.835591, 0.787733, 0.881818, 0.785451, 0.880251, 0.786741, 0.835591, 0.785451, 0.880251, 0.787733, 0.881818, 0.786741, 0.835591, 0.787733, 0.881818, 0.786741, 0.835591, 0.788251, 0.836364, 0.786741, 0.835591, 0.787733, 0.881818, 0.788251, 0.836364, 0.788251, 0.836364, 0.788914, 0.881392, 0.787733, 0.881818, 0.788914, 0.881392, 0.788251, 0.836364, 0.787733, 0.881818, 0.788914, 0.881392, 0.788251, 0.836364, 0.789299, 0.835937, 0.788251, 0.836364, 0.788914, 0.881392, 0.789299, 0.835937, 0.790060, 0.880114, 0.788914, 0.881392, 0.789299, 0.835937, 0.788914, 0.881392, 0.790060, 0.880114, 0.789299, 0.835937, 0.790060, 0.880114, 0.789299, 0.835937, 0.790330, 0.834659, 0.789299, 0.835937, 0.790060, 0.880114, 0.790330, 0.834659, 0.790060, 0.880114, 0.790330, 0.834659, 0.791170, 0.877983, 0.790330, 0.834659, 0.790060, 0.880114, 0.791170, 0.877983, 0.791170, 0.877983, 0.790330, 0.834659, 0.791344, 0.832528, 0.790330, 0.834659, 0.791170, 0.877983, 0.791344, 0.832528, 0.791170, 0.877983, 0.791344, 0.832528, 0.792245, 0.875000, 0.791344, 0.832528, 0.791170, 0.877983, 0.792245, 0.875000, 0.793282, 0.871165, 0.792245, 0.875000, 0.791344, 0.832528, 0.792245, 0.875000, 0.793282, 0.871165, 0.791344, 0.832528, 0.793282, 0.871165, 0.791344, 0.832528, 0.792339, 0.829545, 0.791344, 0.832528, 0.793282, 0.871165, 0.792339, 0.829545, 0.793282, 0.871165, 0.792339, 0.829545, 0.794278, 0.866477, 0.792339, 0.829545, 0.793282, 0.871165, 0.794278, 0.866477, 0.792339, 0.829545, 0.793317, 0.825710, 0.794278, 0.866477, 0.793317, 0.825710, 0.792339, 0.829545, 0.794278, 0.866477, 0.794278, 0.866477, 0.793317, 0.825710, 0.795233, 0.860937, 0.793317, 0.825710, 0.794278, 0.866477, 0.795233, 0.860937, 0.793317, 0.825710, 0.794278, 0.821023, 0.795233, 0.860937, 0.794278, 0.821023, 0.793317, 0.825710, 0.795233, 0.860937, 0.795233, 0.860937, 0.794278, 0.821023, 0.796146, 0.854545, 0.794278, 0.821023, 0.795233, 0.860937, 0.796146, 0.854545, 0.794278, 0.821023, 0.795221, 0.815483, 0.796146, 0.854545, 0.795221, 0.815483, 0.794278, 0.821023, 0.796146, 0.854545, 0.795221, 0.815483, 0.796146, 0.809091, 0.796146, 0.854545, 0.796146, 0.809091, 0.795221, 0.815483, 0.796146, 0.854545, 0.820961, 0.854545, 0.820961, 0.809091, 0.820961, 0.854545, 0.820961, 0.854545, 0.820961, 0.809091, 0.820961, 0.809091, 0.820961, 0.809091, 0.820036, 0.815483, 0.820961, 0.809091, 0.820961, 0.809091, 0.820036, 0.815483, 0.820036, 0.815483, 0.820036, 0.815483, 0.819093, 0.821023, 0.820036, 0.815483, 0.820036, 0.815483, 0.819093, 0.821023, 0.819093, 0.821023, 0.819093, 0.821023, 0.818132, 0.825710, 0.819093, 0.821023, 0.819093, 0.821023, 0.818132, 0.825710, 0.818132, 0.825710, 0.818132, 0.825710, 0.817154, 0.829545, 0.818132, 0.825710, 0.818132, 0.825710, 0.817154, 0.829545, 0.817154, 0.829545, 0.817154, 0.829545, 0.816158, 0.832528, 0.817154, 0.829545, 0.817154, 0.829545, 0.816158, 0.832528, 0.816158, 0.832528, 0.816158, 0.832528, 0.815145, 0.834659, 0.816158, 0.832528, 0.816158, 0.832528, 0.815145, 0.834659, 0.815145, 0.834659, 0.815145, 0.834659, 0.814114, 0.835937, 0.815145, 0.834659, 0.815145, 0.834659, 0.814114, 0.835937, 0.814114, 0.835937, 0.814114, 0.835937, 0.813065, 0.836364, 0.814114, 0.835937, 0.814114, 0.835937, 0.813065, 0.836364, 0.813065, 0.836364, 0.813065, 0.836364, 0.811556, 0.835591, 0.813065, 0.836364, 0.813065, 0.836364, 0.811556, 0.835591, 0.811556, 0.835591, 0.811556, 0.835591, 0.810248, 0.833274, 0.811556, 0.835591, 0.811556, 0.835591, 0.810248, 0.833274, 0.810248, 0.833274, 0.810248, 0.833274, 0.809142, 0.829412, 0.810248, 0.833274, 0.810248, 0.833274, 0.809142, 0.829412, 0.809142, 0.829412, 0.809142, 0.829412, 0.808236, 0.824006, 0.809142, 0.829412, 0.809142, 0.829412, 0.808236, 0.824006, 0.808236, 0.824006, 0.808236, 0.824006, 0.807532, 0.817054, 0.808236, 0.824006, 0.808236, 0.824006, 0.807532, 0.817054, 0.807532, 0.817054, 0.807532, 0.817054, 0.807029, 0.808558, 0.807532, 0.817054, 0.807532, 0.817054, 0.807029, 0.808558, 0.807029, 0.808558, 0.807029, 0.808558, 0.806727, 0.798517, 0.807029, 0.808558, 0.807029, 0.808558, 0.806727, 0.798517, 0.806727, 0.798517, 0.806727, 0.798517, 0.806627, 0.786932, 0.806727, 0.798517, 0.806727, 0.798517, 0.806627, 0.786932, 0.806627, 0.786932, 0.806627, 0.786932, 0.806699, 0.778054, 0.806627, 0.786932, 0.806627, 0.786932, 0.806699, 0.778054, 0.806699, 0.778054, 0.806699, 0.778054, 0.806915, 0.770170, 0.806699, 0.778054, 0.806699, 0.778054, 0.806915, 0.770170, 0.806915, 0.770170, 0.806915, 0.770170, 0.807274, 0.763281, 0.806915, 0.770170, 0.806915, 0.770170, 0.807274, 0.763281, 0.807274, 0.763281, 0.807274, 0.763281, 0.807778, 0.757386, 0.807274, 0.763281, 0.807274, 0.763281, 0.807778, 0.757386, 0.807778, 0.757386, 0.807778, 0.757386, 0.808499, 0.752060, 0.807778, 0.757386, 0.807778, 0.757386, 0.808499, 0.752060, 0.808499, 0.752060, 0.808499, 0.752060, 0.809511, 0.746875, 0.808499, 0.752060, 0.808499, 0.752060, 0.809511, 0.746875, 0.809511, 0.746875, 0.809511, 0.746875, 0.810814, 0.741832, 0.809511, 0.746875, 0.809511, 0.746875, 0.810814, 0.741832, 0.810814, 0.741832, 0.810814, 0.741832, 0.812407, 0.736932, 0.810814, 0.741832, 0.810814, 0.741832, 0.812407, 0.736932, 0.812407, 0.736932, 0.812407, 0.736932, 0.813888, 0.732955, 0.812407, 0.736932, 0.812407, 0.736932, 0.813888, 0.732955, 0.813888, 0.732955, 0.813888, 0.732955, 0.816024, 0.726460, 0.813888, 0.732955, 0.813888, 0.732955, 0.816024, 0.726460, 0.816024, 0.726460, 0.816024, 0.726460, 0.817827, 0.718768, 0.816024, 0.726460, 0.816024, 0.726460, 0.817827, 0.718768, 0.817827, 0.718768, 0.817827, 0.718768, 0.819296, 0.709877, 0.817827, 0.718768, 0.817827, 0.718768, 0.819296, 0.709877, 0.819296, 0.709877, 0.819296, 0.709877, 0.820432, 0.699787, 0.819296, 0.709877, 0.819296, 0.709877, 0.820432, 0.699787, 0.820432, 0.699787, 0.820432, 0.699787, 0.821281, 0.688037, 0.820432, 0.699787, 0.820432, 0.699787, 0.821281, 0.688037, 0.821281, 0.688037, 0.821281, 0.688037, 0.821886, 0.674165, 0.821281, 0.688037, 0.821281, 0.688037, 0.821886, 0.674165, 0.821886, 0.674165, 0.821886, 0.674165, 0.822250, 0.658172, 0.821886, 0.674165, 0.821886, 0.674165, 0.822250, 0.658172, 0.822250, 0.658172, 0.822250, 0.658172, 0.822371, 0.640057, 0.822250, 0.658172, 0.822250, 0.658172, 0.822371, 0.640057, 0.822371, 0.640057, 0.822371, 0.640057, 0.822190, 0.619385, 0.822371, 0.640057, 0.822371, 0.640057, 0.822190, 0.619385, 0.822190, 0.619385, 0.822190, 0.619385, 0.821645, 0.600977, 0.822190, 0.619385, 0.822190, 0.619385, 0.821645, 0.600977, 0.821645, 0.600977, 0.821645, 0.600977, 0.820739, 0.584832, 0.821645, 0.600977, 0.821645, 0.600977, 0.820739, 0.584832, 0.820739, 0.584832, 0.820739, 0.584832, 0.819469, 0.570952, 0.820739, 0.584832, 0.820739, 0.584832, 0.819469, 0.570952, 0.819469, 0.570952, 0.819469, 0.570952, 0.817882, 0.559797, 0.819469, 0.570952, 0.819469, 0.570952, 0.817882, 0.559797, 0.817882, 0.559797, 0.817882, 0.559797, 0.816023, 0.551829, 0.817882, 0.559797, 0.817882, 0.559797, 0.816023, 0.551829, 0.816023, 0.551829, 0.816023, 0.551829, 0.813893, 0.547048, 0.816023, 0.551829, 0.816023, 0.551829, 0.813893, 0.547048, 0.813893, 0.547048, 0.813893, 0.547048, 0.811491, 0.545455, 0.813893, 0.547048, 0.813893, 0.547048, 0.811491, 0.545455, 0.811491, 0.545455, 0.811491, 0.545455, 0.810423, 0.545739, 0.811491, 0.545455, 0.811491, 0.545455, 0.810423, 0.545739, 0.810423, 0.545739, 0.810423, 0.545739, 0.809332, 0.546591, 0.810423, 0.545739, 0.810423, 0.545739, 0.809332, 0.546591, 0.809332, 0.546591, 0.809332, 0.546591, 0.808219, 0.548011, 0.809332, 0.546591, 0.809332, 0.546591, 0.808219, 0.548011, 0.808219, 0.548011, 0.808219, 0.548011, 0.807085, 0.550000, 0.808219, 0.548011, 0.808219, 0.548011, 0.807085, 0.550000, 0.807085, 0.550000, 0.807085, 0.550000, 0.805924, 0.552557, 0.807085, 0.550000, 0.807085, 0.550000, 0.805924, 0.552557, 0.805924, 0.552557, 0.805924, 0.552557, 0.804732, 0.555682, 0.805924, 0.552557, 0.805924, 0.552557, 0.804732, 0.555682, 0.804732, 0.555682, 0.804732, 0.555682, 0.803509, 0.559375, 0.804732, 0.555682, 0.804732, 0.555682, 0.803509, 0.559375, 0.803509, 0.559375, 0.803509, 0.559375, 0.802256, 0.563636, 0.803509, 0.559375, 0.803509, 0.559375, 0.802256, 0.563636, 0.802256, 0.563636, 0.802256, 0.563636, 0.802256, 0.618182, 0.802256, 0.563636, 0.802256, 0.563636, 0.802256, 0.618182, 0.802256, 0.618182, 0.802256, 0.618182, 0.803450, 0.611790, 0.802256, 0.618182, 0.802256, 0.618182, 0.803450, 0.611790, 0.803450, 0.611790, 0.803450, 0.611790, 0.804635, 0.606250, 0.803450, 0.611790, 0.803450, 0.611790, 0.804635, 0.606250, 0.804635, 0.606250, 0.804635, 0.606250, 0.805812, 0.601563, 0.804635, 0.606250, 0.804635, 0.606250, 0.805812, 0.601563, 0.805812, 0.601563, 0.805812, 0.601563, 0.806979, 0.597727, 0.805812, 0.601563, 0.805812, 0.601563, 0.806979, 0.597727, 0.806979, 0.597727, 0.806979, 0.597727, 0.808139, 0.594744, 0.806979, 0.597727, 0.806979, 0.597727, 0.808139, 0.594744, 0.808139, 0.594744, 0.808139, 0.594744, 0.809294, 0.592614, 0.808139, 0.594744, 0.808139, 0.594744, 0.809294, 0.592614, 0.809294, 0.592614, 0.809294, 0.592614, 0.810442, 0.591335, 0.809294, 0.592614, 0.809294, 0.592614, 0.810442, 0.591335, 0.810442, 0.591335, 0.810442, 0.591335, 0.811585, 0.590909, 0.810442, 0.591335, 0.810442, 0.591335, 0.811585, 0.590909, 0.811585, 0.590909, 0.811585, 0.590909, 0.813024, 0.591668, 0.811585, 0.590909, 0.811585, 0.590909, 0.813024, 0.591668, 0.813024, 0.591668, 0.813024, 0.591668, 0.814287, 0.593945, 0.813024, 0.591668, 0.813024, 0.591668, 0.814287, 0.593945, 0.814287, 0.593945, 0.814287, 0.593945, 0.815374, 0.597741, 0.814287, 0.593945, 0.814287, 0.593945, 0.815374, 0.597741, 0.815374, 0.597741, 0.815374, 0.597741, 0.816285, 0.603054, 0.815374, 0.597741, 0.815374, 0.597741, 0.816285, 0.603054, 0.816285, 0.603054, 0.816285, 0.603054, 0.817005, 0.609752, 0.816285, 0.603054, 0.816285, 0.603054, 0.817005, 0.609752, 0.817005, 0.609752, 0.817005, 0.609752, 0.817518, 0.617702, 0.817005, 0.609752, 0.817005, 0.609752, 0.817518, 0.617702, 0.817518, 0.617702, 0.817518, 0.617702, 0.817827, 0.626904, 0.817518, 0.617702, 0.817518, 0.617702, 0.817827, 0.626904, 0.817827, 0.626904, 0.817827, 0.626904, 0.817930, 0.637358, 0.817827, 0.626904, 0.817827, 0.626904, 0.817930, 0.637358, 0.817930, 0.637358, 0.817930, 0.637358, 0.817856, 0.646990, 0.817930, 0.637358, 0.817930, 0.637358, 0.817856, 0.646990, 0.817856, 0.646990, 0.817856, 0.646990, 0.817633, 0.655433, 0.817856, 0.646990, 0.817856, 0.646990, 0.817633, 0.655433, 0.817633, 0.655433, 0.817633, 0.655433, 0.817262, 0.662686, 0.817633, 0.655433, 0.817633, 0.655433, 0.817262, 0.662686, 0.817262, 0.662686, 0.817262, 0.662686, 0.816743, 0.668750, 0.817262, 0.662686, 0.817262, 0.662686, 0.816743, 0.668750, 0.816743, 0.668750, 0.816743, 0.668750, 0.815973, 0.674165, 0.816743, 0.668750, 0.816743, 0.668750, 0.815973, 0.674165, 0.815973, 0.674165, 0.815973, 0.674165, 0.814848, 0.679474, 0.815973, 0.674165, 0.815973, 0.674165, 0.814848, 0.679474, 0.814848, 0.679474, 0.814848, 0.679474, 0.813370, 0.684677, 0.814848, 0.679474, 0.814848, 0.679474, 0.813370, 0.684677, 0.813370, 0.684677, 0.813370, 0.684677, 0.811538, 0.689773, 0.813370, 0.684677, 0.813370, 0.684677, 0.811538, 0.689773, 0.811538, 0.689773, 0.811538, 0.689773, 0.810034, 0.694034, 0.811538, 0.689773, 0.811538, 0.689773, 0.810034, 0.694034, 0.810034, 0.694034, 0.810034, 0.694034, 0.808164, 0.699987, 0.810034, 0.694034, 0.810034, 0.694034, 0.808164, 0.699987, 0.808164, 0.699987, 0.808164, 0.699987, 0.806574, 0.707333, 0.808164, 0.699987, 0.808164, 0.699987, 0.806574, 0.707333, 0.806574, 0.707333, 0.806574, 0.707333, 0.805262, 0.716073, 0.806574, 0.707333, 0.806574, 0.707333, 0.805262, 0.716073, 0.805262, 0.716073, 0.805262, 0.716073, 0.804230, 0.726207, 0.805262, 0.716073, 0.805262, 0.716073, 0.804230, 0.726207, 0.804230, 0.726207, 0.804230, 0.726207, 0.803448, 0.738055, 0.804230, 0.726207, 0.804230, 0.726207, 0.803448, 0.738055, 0.803448, 0.738055, 0.803448, 0.738055, 0.802890, 0.751935, 0.803448, 0.738055, 0.803448, 0.738055, 0.802890, 0.751935, 0.802890, 0.751935, 0.802890, 0.751935, 0.802555, 0.767849, 0.802890, 0.751935, 0.802890, 0.751935, 0.802555, 0.767849, 0.802555, 0.767849, 0.802555, 0.767849, 0.802444, 0.785795, 0.802555, 0.767849, 0.802555, 0.767849, 0.802444, 0.785795, 0.802444, 0.785795, 0.802444, 0.785795, 0.802608, 0.807435, 0.802444, 0.785795, 0.802444, 0.785795, 0.802608, 0.807435, 0.802608, 0.807435, 0.802608, 0.807435, 0.803102, 0.826474, 0.802608, 0.807435, 0.802608, 0.807435, 0.803102, 0.826474, 0.803102, 0.826474, 0.803102, 0.826474, 0.803924, 0.842911, 0.803102, 0.826474, 0.803102, 0.826474, 0.803924, 0.842911, 0.803924, 0.842911, 0.803924, 0.842911, 0.805076, 0.856747, 0.803924, 0.842911, 0.803924, 0.842911, 0.805076, 0.856747, 0.805076, 0.856747, 0.805076, 0.856747, 0.806530, 0.867716, 0.805076, 0.856747, 0.805076, 0.856747, 0.806530, 0.867716, 0.806530, 0.867716, 0.806530, 0.867716, 0.808260, 0.875550, 0.806530, 0.867716, 0.806530, 0.867716, 0.808260, 0.875550, 0.808260, 0.875550, 0.808260, 0.875550, 0.810266, 0.880251, 0.808260, 0.875550, 0.808260, 0.875550, 0.810266, 0.880251, 0.810266, 0.880251, 0.810266, 0.880251, 0.812548, 0.881818, 0.810266, 0.880251, 0.810266, 0.880251, 0.812548, 0.881818, 0.812548, 0.881818, 0.812548, 0.881818, 0.813729, 0.881392, 0.812548, 0.881818, 0.812548, 0.881818, 0.813729, 0.881392, 0.813729, 0.881392, 0.813729, 0.881392, 0.814875, 0.880114, 0.813729, 0.881392, 0.813729, 0.881392, 0.814875, 0.880114, 0.814875, 0.880114, 0.814875, 0.880114, 0.815985, 0.877983, 0.814875, 0.880114, 0.814875, 0.880114, 0.815985, 0.877983, 0.815985, 0.877983, 0.815985, 0.877983, 0.817060, 0.875000, 0.815985, 0.877983, 0.815985, 0.877983, 0.817060, 0.875000, 0.817060, 0.875000, 0.817060, 0.875000, 0.818097, 0.871165, 0.817060, 0.875000, 0.817060, 0.875000, 0.818097, 0.871165, 0.818097, 0.871165, 0.818097, 0.871165, 0.819093, 0.866477, 0.818097, 0.871165, 0.818097, 0.871165, 0.819093, 0.866477, 0.819093, 0.866477, 0.819093, 0.866477, 0.820048, 0.860937, 0.819093, 0.866477, 0.819093, 0.866477, 0.820048, 0.860937, 0.820048, 0.860937, 0.802256, 0.618182, 0.802256, 0.563636, 0.803450, 0.611790, 0.802256, 0.563636, 0.802256, 0.618182, 0.803450, 0.611790, 0.804635, 0.606250, 0.803450, 0.611790, 0.802256, 0.563636, 0.803450, 0.611790, 0.804635, 0.606250, 0.802256, 0.563636, 0.804635, 0.606250, 0.802256, 0.563636, 0.803509, 0.559375, 0.802256, 0.563636, 0.804635, 0.606250, 0.803509, 0.559375, 0.805812, 0.601563, 0.804635, 0.606250, 0.803509, 0.559375, 0.804635, 0.606250, 0.805812, 0.601563, 0.803509, 0.559375, 0.805812, 0.601563, 0.803509, 0.559375, 0.804732, 0.555682, 0.803509, 0.559375, 0.805812, 0.601563, 0.804732, 0.555682, 0.805812, 0.601563, 0.804732, 0.555682, 0.805924, 0.552557, 0.804732, 0.555682, 0.805812, 0.601563, 0.805924, 0.552557, 0.805812, 0.601563, 0.805924, 0.552557, 0.806979, 0.597727, 0.805924, 0.552557, 0.805812, 0.601563, 0.806979, 0.597727, 0.806979, 0.597727, 0.805924, 0.552557, 0.807085, 0.550000, 0.805924, 0.552557, 0.806979, 0.597727, 0.807085, 0.550000, 0.806979, 0.597727, 0.807085, 0.550000, 0.808139, 0.594744, 0.807085, 0.550000, 0.806979, 0.597727, 0.808139, 0.594744, 0.808139, 0.594744, 0.807085, 0.550000, 0.808219, 0.548011, 0.807085, 0.550000, 0.808139, 0.594744, 0.808219, 0.548011, 0.808139, 0.594744, 0.808219, 0.548011, 0.809294, 0.592614, 0.808219, 0.548011, 0.808139, 0.594744, 0.809294, 0.592614, 0.809294, 0.592614, 0.808219, 0.548011, 0.809332, 0.546591, 0.808219, 0.548011, 0.809294, 0.592614, 0.809332, 0.546591, 0.809294, 0.592614, 0.809332, 0.546591, 0.810442, 0.591335, 0.809332, 0.546591, 0.809294, 0.592614, 0.810442, 0.591335, 0.809332, 0.546591, 0.810423, 0.545739, 0.810442, 0.591335, 0.810423, 0.545739, 0.809332, 0.546591, 0.810442, 0.591335, 0.810442, 0.591335, 0.810423, 0.545739, 0.811585, 0.590909, 0.810423, 0.545739, 0.810442, 0.591335, 0.811585, 0.590909, 0.810423, 0.545739, 0.811491, 0.545455, 0.811585, 0.590909, 0.811491, 0.545455, 0.810423, 0.545739, 0.811585, 0.590909, 0.813024, 0.591668, 0.811585, 0.590909, 0.811491, 0.545455, 0.811585, 0.590909, 0.813024, 0.591668, 0.811491, 0.545455, 0.813893, 0.547048, 0.813024, 0.591668, 0.811491, 0.545455, 0.813024, 0.591668, 0.813893, 0.547048, 0.811491, 0.545455, 0.814287, 0.593945, 0.813024, 0.591668, 0.813893, 0.547048, 0.813024, 0.591668, 0.814287, 0.593945, 0.813893, 0.547048, 0.816023, 0.551829, 0.814287, 0.593945, 0.813893, 0.547048, 0.814287, 0.593945, 0.816023, 0.551829, 0.813893, 0.547048, 0.815374, 0.597741, 0.814287, 0.593945, 0.816023, 0.551829, 0.814287, 0.593945, 0.815374, 0.597741, 0.816023, 0.551829, 0.815374, 0.597741, 0.816023, 0.551829, 0.817882, 0.559797, 0.816023, 0.551829, 0.815374, 0.597741, 0.817882, 0.559797, 0.816285, 0.603054, 0.815374, 0.597741, 0.817882, 0.559797, 0.815374, 0.597741, 0.816285, 0.603054, 0.817882, 0.559797, 0.816285, 0.603054, 0.817882, 0.559797, 0.819469, 0.570952, 0.817882, 0.559797, 0.816285, 0.603054, 0.819469, 0.570952, 0.817005, 0.609752, 0.816285, 0.603054, 0.819469, 0.570952, 0.816285, 0.603054, 0.817005, 0.609752, 0.819469, 0.570952, 0.817005, 0.609752, 0.819469, 0.570952, 0.820739, 0.584832, 0.819469, 0.570952, 0.817005, 0.609752, 0.820739, 0.584832, 0.817518, 0.617702, 0.817005, 0.609752, 0.820739, 0.584832, 0.817005, 0.609752, 0.817518, 0.617702, 0.820739, 0.584832, 0.817518, 0.617702, 0.820739, 0.584832, 0.821645, 0.600977, 0.820739, 0.584832, 0.817518, 0.617702, 0.821645, 0.600977, 0.817827, 0.626904, 0.817518, 0.617702, 0.821645, 0.600977, 0.817518, 0.617702, 0.817827, 0.626904, 0.821645, 0.600977, 0.817827, 0.626904, 0.821645, 0.600977, 0.822190, 0.619385, 0.821645, 0.600977, 0.817827, 0.626904, 0.822190, 0.619385, 0.817930, 0.637358, 0.817827, 0.626904, 0.822190, 0.619385, 0.817827, 0.626904, 0.817930, 0.637358, 0.822190, 0.619385, 0.817930, 0.637358, 0.822190, 0.619385, 0.822371, 0.640057, 0.822190, 0.619385, 0.817930, 0.637358, 0.822371, 0.640057, 0.822371, 0.640057, 0.817856, 0.646990, 0.817930, 0.637358, 0.817856, 0.646990, 0.822371, 0.640057, 0.817930, 0.637358, 0.822250, 0.658172, 0.817856, 0.646990, 0.822371, 0.640057, 0.817856, 0.646990, 0.822250, 0.658172, 0.822371, 0.640057, 0.821886, 0.674165, 0.817856, 0.646990, 0.822250, 0.658172, 0.817856, 0.646990, 0.821886, 0.674165, 0.822250, 0.658172, 0.817633, 0.655433, 0.817856, 0.646990, 0.821886, 0.674165, 0.817856, 0.646990, 0.817633, 0.655433, 0.821886, 0.674165, 0.821281, 0.688037, 0.817633, 0.655433, 0.821886, 0.674165, 0.817633, 0.655433, 0.821281, 0.688037, 0.821886, 0.674165, 0.817262, 0.662686, 0.817633, 0.655433, 0.821281, 0.688037, 0.817633, 0.655433, 0.817262, 0.662686, 0.821281, 0.688037, 0.820432, 0.699787, 0.817262, 0.662686, 0.821281, 0.688037, 0.817262, 0.662686, 0.820432, 0.699787, 0.821281, 0.688037, 0.819296, 0.709877, 0.817262, 0.662686, 0.820432, 0.699787, 0.817262, 0.662686, 0.819296, 0.709877, 0.820432, 0.699787, 0.816743, 0.668750, 0.817262, 0.662686, 0.819296, 0.709877, 0.817262, 0.662686, 0.816743, 0.668750, 0.819296, 0.709877, 0.817827, 0.718768, 0.816743, 0.668750, 0.819296, 0.709877, 0.816743, 0.668750, 0.817827, 0.718768, 0.819296, 0.709877, 0.815973, 0.674165, 0.816743, 0.668750, 0.817827, 0.718768, 0.816743, 0.668750, 0.815973, 0.674165, 0.817827, 0.718768, 0.816024, 0.726460, 0.815973, 0.674165, 0.817827, 0.718768, 0.815973, 0.674165, 0.816024, 0.726460, 0.817827, 0.718768, 0.814848, 0.679474, 0.815973, 0.674165, 0.816024, 0.726460, 0.815973, 0.674165, 0.814848, 0.679474, 0.816024, 0.726460, 0.816024, 0.726460, 0.813370, 0.684677, 0.814848, 0.679474, 0.813370, 0.684677, 0.816024, 0.726460, 0.814848, 0.679474, 0.816024, 0.726460, 0.813888, 0.732955, 0.813370, 0.684677, 0.813888, 0.732955, 0.816024, 0.726460, 0.813370, 0.684677, 0.811538, 0.689773, 0.813370, 0.684677, 0.813888, 0.732955, 0.813370, 0.684677, 0.811538, 0.689773, 0.813888, 0.732955, 0.812407, 0.736932, 0.811538, 0.689773, 0.813888, 0.732955, 0.811538, 0.689773, 0.812407, 0.736932, 0.813888, 0.732955, 0.810034, 0.694034, 0.811538, 0.689773, 0.812407, 0.736932, 0.811538, 0.689773, 0.810034, 0.694034, 0.812407, 0.736932, 0.810814, 0.741832, 0.810034, 0.694034, 0.812407, 0.736932, 0.810034, 0.694034, 0.810814, 0.741832, 0.812407, 0.736932, 0.808164, 0.699987, 0.810034, 0.694034, 0.810814, 0.741832, 0.810034, 0.694034, 0.808164, 0.699987, 0.810814, 0.741832, 0.809511, 0.746875, 0.808164, 0.699987, 0.810814, 0.741832, 0.808164, 0.699987, 0.809511, 0.746875, 0.810814, 0.741832, 0.809511, 0.746875, 0.808499, 0.752060, 0.808164, 0.699987, 0.808499, 0.752060, 0.809511, 0.746875, 0.808164, 0.699987, 0.808499, 0.752060, 0.806574, 0.707333, 0.808164, 0.699987, 0.806574, 0.707333, 0.808499, 0.752060, 0.808164, 0.699987, 0.808499, 0.752060, 0.807778, 0.757386, 0.806574, 0.707333, 0.807778, 0.757386, 0.808499, 0.752060, 0.806574, 0.707333, 0.807778, 0.757386, 0.805262, 0.716073, 0.806574, 0.707333, 0.805262, 0.716073, 0.807778, 0.757386, 0.806574, 0.707333, 0.804230, 0.726207, 0.805262, 0.716073, 0.807778, 0.757386, 0.805262, 0.716073, 0.804230, 0.726207, 0.807778, 0.757386, 0.807778, 0.757386, 0.807274, 0.763281, 0.804230, 0.726207, 0.807274, 0.763281, 0.807778, 0.757386, 0.804230, 0.726207, 0.807274, 0.763281, 0.803448, 0.738055, 0.804230, 0.726207, 0.803448, 0.738055, 0.807274, 0.763281, 0.804230, 0.726207, 0.807274, 0.763281, 0.806915, 0.770170, 0.803448, 0.738055, 0.806915, 0.770170, 0.807274, 0.763281, 0.803448, 0.738055, 0.806915, 0.770170, 0.802890, 0.751935, 0.803448, 0.738055, 0.802890, 0.751935, 0.806915, 0.770170, 0.803448, 0.738055, 0.806915, 0.770170, 0.806699, 0.778054, 0.802890, 0.751935, 0.806699, 0.778054, 0.806915, 0.770170, 0.802890, 0.751935, 0.806699, 0.778054, 0.802555, 0.767849, 0.802890, 0.751935, 0.802555, 0.767849, 0.806699, 0.778054, 0.802890, 0.751935, 0.802444, 0.785795, 0.802555, 0.767849, 0.806699, 0.778054, 0.802555, 0.767849, 0.802444, 0.785795, 0.806699, 0.778054, 0.806627, 0.786932, 0.802444, 0.785795, 0.806699, 0.778054, 0.802444, 0.785795, 0.806627, 0.786932, 0.806699, 0.778054, 0.802608, 0.807435, 0.802444, 0.785795, 0.806627, 0.786932, 0.802444, 0.785795, 0.802608, 0.807435, 0.806627, 0.786932, 0.802608, 0.807435, 0.806627, 0.786932, 0.806727, 0.798517, 0.806627, 0.786932, 0.802608, 0.807435, 0.806727, 0.798517, 0.803102, 0.826474, 0.802608, 0.807435, 0.806727, 0.798517, 0.802608, 0.807435, 0.803102, 0.826474, 0.806727, 0.798517, 0.803102, 0.826474, 0.806727, 0.798517, 0.807029, 0.808558, 0.806727, 0.798517, 0.803102, 0.826474, 0.807029, 0.808558, 0.803924, 0.842911, 0.803102, 0.826474, 0.807029, 0.808558, 0.803102, 0.826474, 0.803924, 0.842911, 0.807029, 0.808558, 0.803924, 0.842911, 0.807029, 0.808558, 0.807532, 0.817054, 0.807029, 0.808558, 0.803924, 0.842911, 0.807532, 0.817054, 0.805076, 0.856747, 0.803924, 0.842911, 0.807532, 0.817054, 0.803924, 0.842911, 0.805076, 0.856747, 0.807532, 0.817054, 0.805076, 0.856747, 0.807532, 0.817054, 0.808236, 0.824006, 0.807532, 0.817054, 0.805076, 0.856747, 0.808236, 0.824006, 0.806530, 0.867716, 0.805076, 0.856747, 0.808236, 0.824006, 0.805076, 0.856747, 0.806530, 0.867716, 0.808236, 0.824006, 0.806530, 0.867716, 0.808236, 0.824006, 0.809142, 0.829412, 0.808236, 0.824006, 0.806530, 0.867716, 0.809142, 0.829412, 0.808260, 0.875550, 0.806530, 0.867716, 0.809142, 0.829412, 0.806530, 0.867716, 0.808260, 0.875550, 0.809142, 0.829412, 0.808260, 0.875550, 0.809142, 0.829412, 0.810248, 0.833274, 0.809142, 0.829412, 0.808260, 0.875550, 0.810248, 0.833274, 0.808260, 0.875550, 0.810248, 0.833274, 0.810266, 0.880251, 0.810248, 0.833274, 0.808260, 0.875550, 0.810266, 0.880251, 0.810266, 0.880251, 0.810248, 0.833274, 0.811556, 0.835591, 0.810248, 0.833274, 0.810266, 0.880251, 0.811556, 0.835591, 0.812548, 0.881818, 0.810266, 0.880251, 0.811556, 0.835591, 0.810266, 0.880251, 0.812548, 0.881818, 0.811556, 0.835591, 0.812548, 0.881818, 0.811556, 0.835591, 0.813065, 0.836364, 0.811556, 0.835591, 0.812548, 0.881818, 0.813065, 0.836364, 0.813065, 0.836364, 0.813729, 0.881392, 0.812548, 0.881818, 0.813729, 0.881392, 0.813065, 0.836364, 0.812548, 0.881818, 0.813729, 0.881392, 0.813065, 0.836364, 0.814114, 0.835937, 0.813065, 0.836364, 0.813729, 0.881392, 0.814114, 0.835937, 0.814875, 0.880114, 0.813729, 0.881392, 0.814114, 0.835937, 0.813729, 0.881392, 0.814875, 0.880114, 0.814114, 0.835937, 0.814875, 0.880114, 0.814114, 0.835937, 0.815145, 0.834659, 0.814114, 0.835937, 0.814875, 0.880114, 0.815145, 0.834659, 0.814875, 0.880114, 0.815145, 0.834659, 0.815985, 0.877983, 0.815145, 0.834659, 0.814875, 0.880114, 0.815985, 0.877983, 0.815985, 0.877983, 0.815145, 0.834659, 0.816158, 0.832528, 0.815145, 0.834659, 0.815985, 0.877983, 0.816158, 0.832528, 0.815985, 0.877983, 0.816158, 0.832528, 0.817060, 0.875000, 0.816158, 0.832528, 0.815985, 0.877983, 0.817060, 0.875000, 0.818097, 0.871165, 0.817060, 0.875000, 0.816158, 0.832528, 0.817060, 0.875000, 0.818097, 0.871165, 0.816158, 0.832528, 0.818097, 0.871165, 0.816158, 0.832528, 0.817154, 0.829545, 0.816158, 0.832528, 0.818097, 0.871165, 0.817154, 0.829545, 0.818097, 0.871165, 0.817154, 0.829545, 0.819093, 0.866477, 0.817154, 0.829545, 0.818097, 0.871165, 0.819093, 0.866477, 0.817154, 0.829545, 0.818132, 0.825710, 0.819093, 0.866477, 0.818132, 0.825710, 0.817154, 0.829545, 0.819093, 0.866477, 0.819093, 0.866477, 0.818132, 0.825710, 0.820048, 0.860937, 0.818132, 0.825710, 0.819093, 0.866477, 0.820048, 0.860937, 0.818132, 0.825710, 0.819093, 0.821023, 0.820048, 0.860937, 0.819093, 0.821023, 0.818132, 0.825710, 0.820048, 0.860937, 0.820048, 0.860937, 0.819093, 0.821023, 0.820961, 0.854545, 0.819093, 0.821023, 0.820048, 0.860937, 0.820961, 0.854545, 0.819093, 0.821023, 0.820036, 0.815483, 0.820961, 0.854545, 0.820036, 0.815483, 0.819093, 0.821023, 0.820961, 0.854545, 0.820036, 0.815483, 0.820961, 0.809091, 0.820961, 0.854545, 0.820961, 0.809091, 0.820036, 0.815483, 0.820961, 0.854545, 0.828998, 0.872727, 0.833322, 0.872727, 0.828998, 0.872727, 0.828998, 0.872727, 0.833322, 0.872727, 0.833322, 0.872727, 0.833322, 0.872727, 0.833322, 0.554545, 0.833322, 0.872727, 0.833322, 0.872727, 0.833322, 0.554545, 0.833322, 0.554545, 0.833322, 0.554545, 0.828998, 0.554545, 0.833322, 0.554545, 0.833322, 0.554545, 0.828998, 0.554545, 0.828998, 0.554545, 0.828998, 0.872727, 0.833322, 0.554545, 0.833322, 0.872727, 0.833322, 0.554545, 0.828998, 0.872727, 0.833322, 0.872727, 0.828998, 0.872727, 0.828998, 0.554545, 0.833322, 0.554545, 0.828998, 0.554545, 0.828998, 0.872727, 0.833322, 0.554545, 0.828998, 1, 0.833322, 1, 0.828998, 1, 0.828998, 1, 0.833322, 1, 0.833322, 1, 0.833322, 1, 0.833322, 0.936364, 0.833322, 1, 0.833322, 1, 0.833322, 0.936364, 0.833322, 0.936364, 0.833322, 0.936364, 0.828998, 0.936364, 0.833322, 0.936364, 0.833322, 0.936364, 0.828998, 0.936364, 0.828998, 0.936364, 0.828998, 1, 0.833322, 0.936364, 0.833322, 1, 0.833322, 0.936364, 0.828998, 1, 0.833322, 1, 0.828998, 1, 0.828998, 0.936364, 0.833322, 0.936364, 0.828998, 0.936364, 0.828998, 1, 0.833322, 0.936364, 0.863024, 0.815483, 0.863875, 0.831348, 0.863024, 0.815483, 0.863024, 0.815483, 0.863875, 0.831348, 0.863875, 0.831348, 0.863875, 0.831348, 0.864804, 0.844993, 0.863875, 0.831348, 0.863875, 0.831348, 0.864804, 0.844993, 0.864804, 0.844993, 0.864804, 0.844993, 0.865813, 0.856419, 0.864804, 0.844993, 0.864804, 0.844993, 0.865813, 0.856419, 0.865813, 0.856419, 0.865813, 0.856419, 0.866902, 0.865625, 0.865813, 0.856419, 0.865813, 0.856419, 0.866902, 0.865625, 0.866902, 0.865625, 0.866902, 0.865625, 0.868080, 0.872710, 0.866902, 0.865625, 0.866902, 0.865625, 0.868080, 0.872710, 0.868080, 0.872710, 0.868080, 0.872710, 0.869357, 0.877770, 0.868080, 0.872710, 0.868080, 0.872710, 0.869357, 0.877770, 0.869357, 0.877770, 0.869357, 0.877770, 0.870735, 0.880806, 0.869357, 0.877770, 0.869357, 0.877770, 0.870735, 0.880806, 0.870735, 0.880806, 0.870735, 0.880806, 0.872212, 0.881818, 0.870735, 0.880806, 0.870735, 0.880806, 0.872212, 0.881818, 0.872212, 0.881818, 0.872212, 0.881818, 0.874151, 0.879665, 0.872212, 0.881818, 0.872212, 0.881818, 0.874151, 0.879665, 0.874151, 0.879665, 0.874151, 0.879665, 0.875855, 0.873207, 0.874151, 0.879665, 0.874151, 0.879665, 0.875855, 0.873207, 0.875855, 0.873207, 0.875855, 0.873207, 0.877324, 0.862442, 0.875855, 0.873207, 0.875855, 0.873207, 0.877324, 0.862442, 0.877324, 0.862442, 0.877324, 0.862442, 0.878557, 0.847372, 0.877324, 0.862442, 0.877324, 0.862442, 0.878557, 0.847372, 0.878557, 0.847372, 0.878557, 0.847372, 0.879534, 0.828334, 0.878557, 0.847372, 0.878557, 0.847372, 0.879534, 0.828334, 0.879534, 0.828334, 0.879534, 0.828334, 0.880231, 0.805664, 0.879534, 0.828334, 0.879534, 0.828334, 0.880231, 0.805664, 0.880231, 0.805664, 0.880231, 0.805664, 0.880650, 0.779363, 0.880231, 0.805664, 0.880231, 0.805664, 0.880650, 0.779363, 0.880650, 0.779363, 0.880650, 0.779363, 0.880790, 0.749432, 0.880650, 0.779363, 0.880650, 0.779363, 0.880790, 0.749432, 0.880790, 0.749432, 0.880790, 0.749432, 0.880790, 0.554545, 0.880790, 0.749432, 0.880790, 0.749432, 0.880790, 0.554545, 0.880790, 0.554545, 0.880790, 0.554545, 0.876442, 0.554545, 0.880790, 0.554545, 0.880790, 0.554545, 0.876442, 0.554545, 0.876442, 0.554545, 0.876442, 0.554545, 0.876442, 0.747727, 0.876442, 0.554545, 0.876442, 0.554545, 0.876442, 0.747727, 0.876442, 0.747727, 0.876442, 0.747727, 0.876359, 0.768790, 0.876442, 0.747727, 0.876442, 0.747727, 0.876359, 0.768790, 0.876359, 0.768790, 0.876359, 0.768790, 0.876107, 0.786950, 0.876359, 0.768790, 0.876359, 0.768790, 0.876107, 0.786950, 0.876107, 0.786950, 0.876107, 0.786950, 0.875689, 0.802206, 0.876107, 0.786950, 0.876107, 0.786950, 0.875689, 0.802206, 0.875689, 0.802206, 0.875689, 0.802206, 0.875103, 0.814560, 0.875689, 0.802206, 0.875689, 0.802206, 0.875103, 0.814560, 0.875103, 0.814560, 0.875103, 0.814560, 0.874345, 0.824099, 0.875103, 0.814560, 0.875103, 0.814560, 0.874345, 0.824099, 0.874345, 0.824099, 0.874345, 0.824099, 0.873411, 0.830913, 0.874345, 0.824099, 0.874345, 0.824099, 0.873411, 0.830913, 0.873411, 0.830913, 0.873411, 0.830913, 0.872301, 0.835001, 0.873411, 0.830913, 0.873411, 0.830913, 0.872301, 0.835001, 0.872301, 0.835001, 0.872301, 0.835001, 0.871014, 0.836364, 0.872301, 0.835001, 0.872301, 0.835001, 0.871014, 0.836364, 0.871014, 0.836364, 0.871014, 0.836364, 0.869422, 0.834699, 0.871014, 0.836364, 0.871014, 0.836364, 0.869422, 0.834699, 0.869422, 0.834699, 0.869422, 0.834699, 0.868006, 0.829705, 0.869422, 0.834699, 0.869422, 0.834699, 0.868006, 0.829705, 0.868006, 0.829705, 0.868006, 0.829705, 0.866767, 0.821382, 0.868006, 0.829705, 0.868006, 0.829705, 0.866767, 0.821382, 0.866767, 0.821382, 0.866767, 0.821382, 0.865703, 0.809730, 0.866767, 0.821382, 0.866767, 0.821382, 0.865703, 0.809730, 0.865703, 0.809730, 0.865703, 0.809730, 0.864850, 0.795201, 0.865703, 0.809730, 0.865703, 0.809730, 0.864850, 0.795201, 0.864850, 0.795201, 0.864850, 0.795201, 0.864240, 0.778249, 0.864850, 0.795201, 0.864850, 0.795201, 0.864240, 0.778249, 0.864240, 0.778249, 0.864240, 0.778249, 0.863875, 0.758873, 0.864240, 0.778249, 0.864240, 0.778249, 0.863875, 0.758873, 0.863875, 0.758873, 0.863875, 0.758873, 0.863753, 0.737074, 0.863875, 0.758873, 0.863875, 0.758873, 0.863753, 0.737074, 0.863753, 0.737074, 0.863753, 0.737074, 0.863753, 0.554545, 0.863753, 0.737074, 0.863753, 0.737074, 0.863753, 0.554545, 0.863753, 0.554545, 0.863753, 0.554545, 0.859405, 0.554545, 0.863753, 0.554545, 0.863753, 0.554545, 0.859405, 0.554545, 0.859405, 0.554545, 0.859405, 0.554545, 0.859405, 0.747727, 0.859405, 0.554545, 0.859405, 0.554545, 0.859405, 0.747727, 0.859405, 0.747727, 0.859405, 0.747727, 0.859322, 0.768905, 0.859405, 0.747727, 0.859405, 0.747727, 0.859322, 0.768905, 0.859322, 0.768905, 0.859322, 0.768905, 0.859071, 0.787127, 0.859322, 0.768905, 0.859322, 0.768905, 0.859071, 0.787127, 0.859071, 0.787127, 0.859071, 0.787127, 0.858652, 0.802393, 0.859071, 0.787127, 0.859071, 0.787127, 0.858652, 0.802393, 0.858652, 0.802393, 0.858652, 0.802393, 0.858066, 0.814702, 0.858652, 0.802393, 0.858652, 0.802393, 0.858066, 0.814702, 0.858066, 0.814702, 0.858066, 0.814702, 0.857305, 0.824179, 0.858066, 0.814702, 0.858066, 0.814702, 0.857305, 0.824179, 0.857305, 0.824179, 0.857305, 0.824179, 0.856362, 0.830948, 0.857305, 0.824179, 0.857305, 0.824179, 0.856362, 0.830948, 0.856362, 0.830948, 0.856362, 0.830948, 0.855237, 0.835010, 0.856362, 0.830948, 0.856362, 0.830948, 0.855237, 0.835010, 0.855237, 0.835010, 0.855237, 0.835010, 0.853930, 0.836364, 0.855237, 0.835010, 0.855237, 0.835010, 0.853930, 0.836364, 0.853930, 0.836364, 0.853930, 0.836364, 0.852359, 0.834690, 0.853930, 0.836364, 0.853930, 0.836364, 0.852359, 0.834690, 0.852359, 0.834690, 0.852359, 0.834690, 0.850958, 0.829670, 0.852359, 0.834690, 0.852359, 0.834690, 0.850958, 0.829670, 0.850958, 0.829670, 0.850958, 0.829670, 0.849727, 0.821302, 0.850958, 0.829670, 0.850958, 0.829670, 0.849727, 0.821302, 0.849727, 0.821302, 0.849727, 0.821302, 0.848666, 0.809588, 0.849727, 0.821302, 0.849727, 0.821302, 0.848666, 0.809588, 0.848666, 0.809588, 0.848666, 0.809588, 0.847813, 0.795015, 0.848666, 0.809588, 0.848666, 0.809588, 0.847813, 0.795015, 0.847813, 0.795015, 0.847813, 0.795015, 0.847204, 0.778072, 0.847813, 0.795015, 0.847813, 0.795015, 0.847204, 0.778072, 0.847204, 0.778072, 0.847204, 0.778072, 0.846838, 0.758758, 0.847204, 0.778072, 0.847204, 0.778072, 0.846838, 0.758758, 0.846838, 0.758758, 0.846838, 0.758758, 0.846716, 0.737074, 0.846838, 0.758758, 0.846838, 0.758758, 0.846716, 0.737074, 0.846716, 0.737074, 0.846716, 0.737074, 0.846716, 0.554545, 0.846716, 0.737074, 0.846716, 0.737074, 0.846716, 0.554545, 0.846716, 0.554545, 0.846716, 0.554545, 0.842369, 0.554545, 0.846716, 0.554545, 0.846716, 0.554545, 0.842369, 0.554545, 0.842369, 0.554545, 0.842369, 0.554545, 0.842369, 0.872727, 0.842369, 0.554545, 0.842369, 0.554545, 0.842369, 0.872727, 0.842369, 0.872727, 0.842369, 0.872727, 0.846716, 0.872727, 0.842369, 0.872727, 0.842369, 0.872727, 0.846716, 0.872727, 0.846716, 0.872727, 0.846716, 0.872727, 0.846716, 0.827273, 0.846716, 0.872727, 0.846716, 0.872727, 0.846716, 0.827273, 0.846716, 0.827273, 0.846716, 0.827273, 0.847493, 0.840345, 0.846716, 0.827273, 0.846716, 0.827273, 0.847493, 0.840345, 0.847493, 0.840345, 0.847493, 0.840345, 0.848343, 0.851580, 0.847493, 0.840345, 0.847493, 0.840345, 0.848343, 0.851580, 0.848343, 0.851580, 0.848343, 0.851580, 0.849267, 0.860977, 0.848343, 0.851580, 0.848343, 0.851580, 0.849267, 0.860977, 0.849267, 0.860977, 0.849267, 0.860977, 0.850264, 0.868537, 0.849267, 0.860977, 0.849267, 0.860977, 0.850264, 0.868537, 0.850264, 0.868537, 0.850264, 0.868537, 0.851347, 0.874347, 0.850264, 0.868537, 0.850264, 0.868537, 0.851347, 0.874347, 0.851347, 0.874347, 0.851347, 0.874347, 0.852526, 0.878498, 0.851347, 0.874347, 0.851347, 0.874347, 0.852526, 0.878498, 0.852526, 0.878498, 0.852526, 0.878498, 0.853802, 0.880988, 0.852526, 0.878498, 0.852526, 0.878498, 0.853802, 0.880988, 0.853802, 0.880988, 0.853802, 0.880988, 0.855176, 0.881818, 0.853802, 0.880988, 0.853802, 0.880988, 0.855176, 0.881818, 0.855176, 0.881818, 0.855176, 0.881818, 0.856555, 0.880757, 0.855176, 0.881818, 0.855176, 0.881818, 0.856555, 0.880757, 0.856555, 0.880757, 0.856555, 0.880757, 0.857828, 0.877575, 0.856555, 0.880757, 0.856555, 0.880757, 0.857828, 0.877575, 0.857828, 0.877575, 0.857828, 0.877575, 0.858994, 0.872270, 0.857828, 0.877575, 0.857828, 0.877575, 0.858994, 0.872270, 0.858994, 0.872270, 0.858994, 0.872270, 0.860052, 0.864844, 0.858994, 0.872270, 0.858994, 0.872270, 0.860052, 0.864844, 0.860052, 0.864844, 0.860052, 0.864844, 0.860991, 0.855393, 0.860052, 0.864844, 0.860052, 0.864844, 0.860991, 0.855393, 0.860991, 0.855393, 0.860991, 0.855393, 0.861799, 0.844016, 0.860991, 0.855393, 0.860991, 0.855393, 0.861799, 0.844016, 0.861799, 0.844016, 0.861799, 0.844016, 0.862477, 0.830713, 0.861799, 0.844016, 0.861799, 0.844016, 0.862477, 0.830713, 0.862477, 0.830713, 0.846716, 0.872727, 0.842369, 0.872727, 0.846716, 0.827273, 0.842369, 0.872727, 0.846716, 0.872727, 0.846716, 0.827273, 0.847204, 0.778072, 0.846716, 0.827273, 0.842369, 0.872727, 0.846716, 0.827273, 0.847204, 0.778072, 0.842369, 0.872727, 0.846716, 0.827273, 0.847204, 0.778072, 0.847813, 0.795015, 0.847204, 0.778072, 0.846716, 0.827273, 0.847813, 0.795015, 0.846716, 0.827273, 0.847813, 0.795015, 0.848666, 0.809588, 0.847813, 0.795015, 0.846716, 0.827273, 0.848666, 0.809588, 0.847493, 0.840345, 0.846716, 0.827273, 0.848666, 0.809588, 0.846716, 0.827273, 0.847493, 0.840345, 0.848666, 0.809588, 0.847493, 0.840345, 0.848666, 0.809588, 0.849727, 0.821302, 0.848666, 0.809588, 0.847493, 0.840345, 0.849727, 0.821302, 0.848343, 0.851580, 0.847493, 0.840345, 0.849727, 0.821302, 0.847493, 0.840345, 0.848343, 0.851580, 0.849727, 0.821302, 0.848343, 0.851580, 0.849727, 0.821302, 0.850958, 0.829670, 0.849727, 0.821302, 0.848343, 0.851580, 0.850958, 0.829670, 0.849267, 0.860977, 0.848343, 0.851580, 0.850958, 0.829670, 0.848343, 0.851580, 0.849267, 0.860977, 0.850958, 0.829670, 0.850264, 0.868537, 0.849267, 0.860977, 0.850958, 0.829670, 0.849267, 0.860977, 0.850264, 0.868537, 0.850958, 0.829670, 0.850264, 0.868537, 0.850958, 0.829670, 0.852359, 0.834690, 0.850958, 0.829670, 0.850264, 0.868537, 0.852359, 0.834690, 0.851347, 0.874347, 0.850264, 0.868537, 0.852359, 0.834690, 0.850264, 0.868537, 0.851347, 0.874347, 0.852359, 0.834690, 0.851347, 0.874347, 0.852359, 0.834690, 0.852526, 0.878498, 0.852359, 0.834690, 0.851347, 0.874347, 0.852526, 0.878498, 0.852526, 0.878498, 0.852359, 0.834690, 0.853930, 0.836364, 0.852359, 0.834690, 0.852526, 0.878498, 0.853930, 0.836364, 0.853802, 0.880988, 0.852526, 0.878498, 0.853930, 0.836364, 0.852526, 0.878498, 0.853802, 0.880988, 0.853930, 0.836364, 0.855176, 0.881818, 0.853802, 0.880988, 0.853930, 0.836364, 0.853802, 0.880988, 0.855176, 0.881818, 0.853930, 0.836364, 0.855176, 0.881818, 0.853930, 0.836364, 0.855237, 0.835010, 0.853930, 0.836364, 0.855176, 0.881818, 0.855237, 0.835010, 0.856555, 0.880757, 0.855176, 0.881818, 0.855237, 0.835010, 0.855176, 0.881818, 0.856555, 0.880757, 0.855237, 0.835010, 0.856555, 0.880757, 0.855237, 0.835010, 0.856362, 0.830948, 0.855237, 0.835010, 0.856555, 0.880757, 0.856362, 0.830948, 0.857828, 0.877575, 0.856555, 0.880757, 0.856362, 0.830948, 0.856555, 0.880757, 0.857828, 0.877575, 0.856362, 0.830948, 0.857828, 0.877575, 0.856362, 0.830948, 0.858994, 0.872270, 0.856362, 0.830948, 0.857828, 0.877575, 0.858994, 0.872270, 0.856362, 0.830948, 0.857305, 0.824179, 0.858994, 0.872270, 0.857305, 0.824179, 0.856362, 0.830948, 0.858994, 0.872270, 0.858994, 0.872270, 0.857305, 0.824179, 0.860052, 0.864844, 0.857305, 0.824179, 0.858994, 0.872270, 0.860052, 0.864844, 0.860052, 0.864844, 0.857305, 0.824179, 0.860991, 0.855393, 0.857305, 0.824179, 0.860052, 0.864844, 0.860991, 0.855393, 0.857305, 0.824179, 0.858066, 0.814702, 0.860991, 0.855393, 0.858066, 0.814702, 0.857305, 0.824179, 0.860991, 0.855393, 0.860991, 0.855393, 0.858066, 0.814702, 0.861799, 0.844016, 0.858066, 0.814702, 0.860991, 0.855393, 0.861799, 0.844016, 0.861799, 0.844016, 0.858066, 0.814702, 0.862477, 0.830713, 0.858066, 0.814702, 0.861799, 0.844016, 0.862477, 0.830713, 0.858066, 0.814702, 0.858652, 0.802393, 0.862477, 0.830713, 0.858652, 0.802393, 0.858066, 0.814702, 0.862477, 0.830713, 0.862477, 0.830713, 0.858652, 0.802393, 0.863024, 0.815483, 0.858652, 0.802393, 0.862477, 0.830713, 0.863024, 0.815483, 0.858652, 0.802393, 0.859071, 0.787127, 0.863024, 0.815483, 0.859071, 0.787127, 0.858652, 0.802393, 0.863024, 0.815483, 0.863024, 0.815483, 0.859071, 0.787127, 0.864240, 0.778249, 0.859071, 0.787127, 0.863024, 0.815483, 0.864240, 0.778249, 0.863024, 0.815483, 0.864240, 0.778249, 0.864850, 0.795201, 0.864240, 0.778249, 0.863024, 0.815483, 0.864850, 0.795201, 0.863024, 0.815483, 0.864850, 0.795201, 0.865703, 0.809730, 0.864850, 0.795201, 0.863024, 0.815483, 0.865703, 0.809730, 0.863875, 0.831348, 0.863024, 0.815483, 0.865703, 0.809730, 0.863024, 0.815483, 0.863875, 0.831348, 0.865703, 0.809730, 0.866767, 0.821382, 0.863875, 0.831348, 0.865703, 0.809730, 0.863875, 0.831348, 0.866767, 0.821382, 0.865703, 0.809730, 0.866767, 0.821382, 0.864804, 0.844993, 0.863875, 0.831348, 0.864804, 0.844993, 0.866767, 0.821382, 0.863875, 0.831348, 0.865813, 0.856419, 0.864804, 0.844993, 0.866767, 0.821382, 0.864804, 0.844993, 0.865813, 0.856419, 0.866767, 0.821382, 0.865813, 0.856419, 0.866767, 0.821382, 0.868006, 0.829705, 0.866767, 0.821382, 0.865813, 0.856419, 0.868006, 0.829705, 0.866902, 0.865625, 0.865813, 0.856419, 0.868006, 0.829705, 0.865813, 0.856419, 0.866902, 0.865625, 0.868006, 0.829705, 0.866902, 0.865625, 0.868006, 0.829705, 0.868080, 0.872710, 0.868006, 0.829705, 0.866902, 0.865625, 0.868080, 0.872710, 0.868080, 0.872710, 0.868006, 0.829705, 0.869422, 0.834699, 0.868006, 0.829705, 0.868080, 0.872710, 0.869422, 0.834699, 0.869357, 0.877770, 0.868080, 0.872710, 0.869422, 0.834699, 0.868080, 0.872710, 0.869357, 0.877770, 0.869422, 0.834699, 0.869357, 0.877770, 0.869422, 0.834699, 0.871014, 0.836364, 0.869422, 0.834699, 0.869357, 0.877770, 0.871014, 0.836364, 0.870735, 0.880806, 0.869357, 0.877770, 0.871014, 0.836364, 0.869357, 0.877770, 0.870735, 0.880806, 0.871014, 0.836364, 0.872212, 0.881818, 0.870735, 0.880806, 0.871014, 0.836364, 0.870735, 0.880806, 0.872212, 0.881818, 0.871014, 0.836364, 0.872212, 0.881818, 0.871014, 0.836364, 0.872301, 0.835001, 0.871014, 0.836364, 0.872212, 0.881818, 0.872301, 0.835001, 0.874151, 0.879665, 0.872212, 0.881818, 0.872301, 0.835001, 0.872212, 0.881818, 0.874151, 0.879665, 0.872301, 0.835001, 0.872301, 0.835001, 0.873411, 0.830913, 0.874151, 0.879665, 0.873411, 0.830913, 0.872301, 0.835001, 0.874151, 0.879665, 0.874151, 0.879665, 0.873411, 0.830913, 0.875855, 0.873207, 0.873411, 0.830913, 0.874151, 0.879665, 0.875855, 0.873207, 0.873411, 0.830913, 0.874345, 0.824099, 0.875855, 0.873207, 0.874345, 0.824099, 0.873411, 0.830913, 0.875855, 0.873207, 0.875855, 0.873207, 0.874345, 0.824099, 0.877324, 0.862442, 0.874345, 0.824099, 0.875855, 0.873207, 0.877324, 0.862442, 0.874345, 0.824099, 0.875103, 0.814560, 0.877324, 0.862442, 0.875103, 0.814560, 0.874345, 0.824099, 0.877324, 0.862442, 0.877324, 0.862442, 0.875103, 0.814560, 0.878557, 0.847372, 0.875103, 0.814560, 0.877324, 0.862442, 0.878557, 0.847372, 0.878557, 0.847372, 0.875103, 0.814560, 0.879534, 0.828334, 0.875103, 0.814560, 0.878557, 0.847372, 0.879534, 0.828334, 0.879534, 0.828334, 0.875103, 0.814560, 0.875689, 0.802206, 0.875103, 0.814560, 0.879534, 0.828334, 0.875689, 0.802206, 0.879534, 0.828334, 0.875689, 0.802206, 0.880231, 0.805664, 0.875689, 0.802206, 0.879534, 0.828334, 0.880231, 0.805664, 0.875689, 0.802206, 0.876107, 0.786950, 0.880231, 0.805664, 0.876107, 0.786950, 0.875689, 0.802206, 0.880231, 0.805664, 0.880231, 0.805664, 0.876107, 0.786950, 0.880650, 0.779363, 0.876107, 0.786950, 0.880231, 0.805664, 0.880650, 0.779363, 0.876107, 0.786950, 0.876359, 0.768790, 0.880650, 0.779363, 0.876359, 0.768790, 0.876107, 0.786950, 0.880650, 0.779363, 0.880650, 0.779363, 0.876359, 0.768790, 0.880790, 0.749432, 0.876359, 0.768790, 0.880650, 0.779363, 0.880790, 0.749432, 0.880790, 0.749432, 0.876359, 0.768790, 0.876442, 0.747727, 0.876359, 0.768790, 0.880790, 0.749432, 0.876442, 0.747727, 0.876442, 0.747727, 0.880790, 0.554545, 0.880790, 0.749432, 0.880790, 0.554545, 0.876442, 0.747727, 0.880790, 0.749432, 0.876442, 0.747727, 0.876442, 0.554545, 0.880790, 0.554545, 0.876442, 0.554545, 0.876442, 0.747727, 0.880790, 0.554545, 0.859322, 0.768905, 0.864240, 0.778249, 0.859071, 0.787127, 0.864240, 0.778249, 0.859322, 0.768905, 0.859071, 0.787127, 0.864240, 0.778249, 0.859322, 0.768905, 0.863875, 0.758873, 0.859322, 0.768905, 0.864240, 0.778249, 0.863875, 0.758873, 0.859322, 0.768905, 0.859405, 0.747727, 0.863875, 0.758873, 0.859405, 0.747727, 0.859322, 0.768905, 0.863875, 0.758873, 0.859405, 0.747727, 0.863753, 0.737074, 0.863875, 0.758873, 0.863753, 0.737074, 0.859405, 0.747727, 0.863875, 0.758873, 0.863753, 0.737074, 0.859405, 0.747727, 0.859405, 0.554545, 0.859405, 0.747727, 0.863753, 0.737074, 0.859405, 0.554545, 0.859405, 0.554545, 0.863753, 0.554545, 0.863753, 0.737074, 0.863753, 0.554545, 0.859405, 0.554545, 0.863753, 0.737074, 0.847204, 0.778072, 0.842369, 0.872727, 0.846838, 0.758758, 0.842369, 0.872727, 0.847204, 0.778072, 0.846838, 0.758758, 0.846716, 0.737074, 0.846838, 0.758758, 0.842369, 0.872727, 0.846838, 0.758758, 0.846716, 0.737074, 0.842369, 0.872727, 0.846716, 0.737074, 0.842369, 0.872727, 0.842369, 0.554545, 0.842369, 0.872727, 0.846716, 0.737074, 0.842369, 0.554545, 0.842369, 0.554545, 0.846716, 0.554545, 0.846716, 0.737074, 0.846716, 0.554545, 0.842369, 0.554545, 0.846716, 0.737074, 0.889155, 0.872727, 0.893479, 0.872727, 0.889155, 0.872727, 0.889155, 0.872727, 0.893479, 0.872727, 0.893479, 0.872727, 0.893479, 0.872727, 0.893479, 0.554545, 0.893479, 0.872727, 0.893479, 0.872727, 0.893479, 0.554545, 0.893479, 0.554545, 0.893479, 0.554545, 0.889155, 0.554545, 0.893479, 0.554545, 0.893479, 0.554545, 0.889155, 0.554545, 0.889155, 0.554545, 0.889155, 0.872727, 0.893479, 0.554545, 0.893479, 0.872727, 0.893479, 0.554545, 0.889155, 0.872727, 0.893479, 0.872727, 0.889155, 0.872727, 0.889155, 0.554545, 0.893479, 0.554545, 0.889155, 0.554545, 0.889155, 0.872727, 0.893479, 0.554545, 0.889155, 1, 0.893479, 1, 0.889155, 1, 0.889155, 1, 0.893479, 1, 0.893479, 1, 0.893479, 1, 0.893479, 0.936364, 0.893479, 1, 0.893479, 1, 0.893479, 0.936364, 0.893479, 0.936364, 0.893479, 0.936364, 0.889155, 0.936364, 0.893479, 0.936364, 0.893479, 0.936364, 0.889155, 0.936364, 0.889155, 0.936364, 0.889155, 1, 0.893479, 0.936364, 0.893479, 1, 0.893479, 0.936364, 0.889155, 1, 0.893479, 1, 0.889155, 1, 0.889155, 0.936364, 0.893479, 0.936364, 0.889155, 0.936364, 0.889155, 1, 0.893479, 0.936364, 0.902691, 1, 0.907014, 1, 0.902691, 1, 0.902691, 1, 0.907014, 1, 0.907014, 1, 0.907014, 1, 0.907014, 0.554545, 0.907014, 1, 0.907014, 1, 0.907014, 0.554545, 0.907014, 0.554545, 0.907014, 0.554545, 0.902691, 0.554545, 0.907014, 0.554545, 0.907014, 0.554545, 0.902691, 0.554545, 0.902691, 0.554545, 0.902691, 1, 0.907014, 0.554545, 0.907014, 1, 0.907014, 0.554545, 0.902691, 1, 0.907014, 1, 0.902691, 1, 0.902691, 0.554545, 0.907014, 0.554545, 0.902691, 0.554545, 0.902691, 1, 0.907014, 0.554545, 0.928187, 0.709091, 0.925768, 0.708230, 0.928187, 0.709091, 0.928187, 0.709091, 0.925768, 0.708230, 0.925768, 0.708230, 0.925768, 0.708230, 0.923752, 0.705646, 0.925768, 0.708230, 0.925768, 0.708230, 0.923752, 0.705646, 0.923752, 0.705646, 0.923752, 0.705646, 0.922138, 0.701341, 0.923752, 0.705646, 0.923752, 0.705646, 0.922138, 0.701341, 0.922138, 0.701341, 0.922138, 0.701341, 0.920926, 0.695313, 0.922138, 0.701341, 0.922138, 0.701341, 0.920926, 0.695313, 0.920926, 0.695313, 0.920926, 0.695313, 0.920042, 0.687216, 0.920926, 0.695313, 0.920926, 0.695313, 0.920042, 0.687216, 0.920042, 0.687216, 0.920042, 0.687216, 0.919410, 0.676705, 0.920042, 0.687216, 0.920042, 0.687216, 0.919410, 0.676705, 0.919410, 0.676705, 0.919410, 0.676705, 0.919031, 0.663778, 0.919410, 0.676705, 0.919410, 0.676705, 0.919031, 0.663778, 0.919031, 0.663778, 0.919031, 0.663778, 0.918905, 0.648437, 0.919031, 0.663778, 0.919031, 0.663778, 0.918905, 0.648437, 0.918905, 0.648437, 0.918905, 0.648437, 0.919000, 0.635849, 0.918905, 0.648437, 0.918905, 0.648437, 0.919000, 0.635849, 0.919000, 0.635849, 0.919000, 0.635849, 0.919284, 0.624645, 0.919000, 0.635849, 0.919000, 0.635849, 0.919284, 0.624645, 0.919284, 0.624645, 0.919284, 0.624645, 0.919757, 0.614826, 0.919284, 0.624645, 0.919284, 0.624645, 0.919757, 0.614826, 0.919757, 0.614826, 0.919757, 0.614826, 0.920421, 0.606392, 0.919757, 0.614826, 0.919757, 0.614826, 0.920421, 0.606392, 0.920421, 0.606392, 0.920421, 0.606392, 0.921247, 0.599618, 0.920421, 0.606392, 0.920421, 0.606392, 0.921247, 0.599618, 0.921247, 0.599618, 0.921247, 0.599618, 0.922210, 0.594780, 0.921247, 0.599618, 0.921247, 0.599618, 0.922210, 0.594780, 0.922210, 0.594780, 0.922210, 0.594780, 0.923309, 0.591877, 0.922210, 0.594780, 0.922210, 0.594780, 0.923309, 0.591877, 0.923309, 0.591877, 0.923309, 0.591877, 0.924545, 0.590909, 0.923309, 0.591877, 0.923309, 0.591877, 0.924545, 0.590909, 0.924545, 0.590909, 0.924545, 0.590909, 0.926253, 0.592733, 0.924545, 0.590909, 0.924545, 0.590909, 0.926253, 0.592733, 0.926253, 0.592733, 0.926253, 0.592733, 0.927785, 0.598207, 0.926253, 0.592733, 0.926253, 0.592733, 0.927785, 0.598207, 0.927785, 0.598207, 0.927785, 0.598207, 0.929138, 0.607329, 0.927785, 0.598207, 0.927785, 0.598207, 0.929138, 0.607329, 0.929138, 0.607329, 0.929138, 0.607329, 0.930314, 0.620099, 0.929138, 0.607329, 0.929138, 0.607329, 0.930314, 0.620099, 0.930314, 0.620099, 0.930314, 0.620099, 0.931265, 0.635898, 0.930314, 0.620099, 0.930314, 0.620099, 0.931265, 0.635898, 0.931265, 0.635898, 0.931265, 0.635898, 0.931944, 0.654102, 0.931265, 0.635898, 0.931265, 0.635898, 0.931944, 0.654102, 0.931944, 0.654102, 0.931944, 0.654102, 0.932351, 0.674711, 0.931944, 0.654102, 0.931944, 0.654102, 0.932351, 0.674711, 0.932351, 0.674711, 0.932351, 0.674711, 0.932487, 0.697727, 0.932351, 0.674711, 0.932351, 0.674711, 0.932487, 0.697727, 0.932487, 0.697727, 0.932487, 0.697727, 0.932487, 0.709091, 0.932487, 0.697727, 0.932487, 0.697727, 0.932487, 0.709091, 0.932487, 0.709091, 0.936811, 0.733097, 0.936811, 0.554545, 0.936811, 0.733097, 0.936811, 0.733097, 0.936811, 0.554545, 0.936811, 0.554545, 0.936811, 0.554545, 0.932487, 0.554545, 0.936811, 0.554545, 0.936811, 0.554545, 0.932487, 0.554545, 0.932487, 0.554545, 0.932487, 0.554545, 0.932487, 0.600000, 0.932487, 0.554545, 0.932487, 0.554545, 0.932487, 0.600000, 0.932487, 0.600000, 0.932487, 0.600000, 0.931702, 0.586927, 0.932487, 0.600000, 0.932487, 0.600000, 0.931702, 0.586927, 0.931702, 0.586927, 0.931702, 0.586927, 0.930825, 0.575692, 0.931702, 0.586927, 0.931702, 0.586927, 0.930825, 0.575692, 0.930825, 0.575692, 0.930825, 0.575692, 0.929857, 0.566295, 0.930825, 0.575692, 0.930825, 0.575692, 0.929857, 0.566295, 0.929857, 0.566295, 0.929857, 0.566295, 0.928798, 0.558736, 0.929857, 0.566295, 0.929857, 0.566295, 0.928798, 0.558736, 0.928798, 0.558736, 0.928798, 0.558736, 0.927632, 0.552925, 0.928798, 0.558736, 0.928798, 0.558736, 0.927632, 0.552925, 0.927632, 0.552925, 0.927632, 0.552925, 0.926342, 0.548775, 0.927632, 0.552925, 0.927632, 0.552925, 0.926342, 0.548775, 0.926342, 0.548775, 0.926342, 0.548775, 0.924929, 0.546285, 0.926342, 0.548775, 0.926342, 0.548775, 0.924929, 0.546285, 0.924929, 0.546285, 0.924929, 0.546285, 0.923393, 0.545455, 0.924929, 0.546285, 0.924929, 0.546285, 0.923393, 0.545455, 0.923393, 0.545455, 0.923393, 0.545455, 0.921476, 0.547168, 0.923393, 0.545455, 0.923393, 0.545455, 0.921476, 0.547168, 0.921476, 0.547168, 0.921476, 0.547168, 0.919766, 0.552308, 0.921476, 0.547168, 0.921476, 0.547168, 0.919766, 0.552308, 0.919766, 0.552308, 0.919766, 0.552308, 0.918262, 0.560875, 0.919766, 0.552308, 0.919766, 0.552308, 0.918262, 0.560875, 0.918262, 0.560875, 0.918262, 0.560875, 0.916966, 0.572869, 0.918262, 0.560875, 0.918262, 0.560875, 0.916966, 0.572869, 0.916966, 0.572869, 0.916966, 0.572869, 0.915923, 0.587740, 0.916966, 0.572869, 0.916966, 0.572869, 0.915923, 0.587740, 0.915923, 0.587740, 0.915923, 0.587740, 0.915177, 0.604936, 0.915923, 0.587740, 0.915923, 0.587740, 0.915177, 0.604936, 0.915177, 0.604936, 0.915177, 0.604936, 0.914730, 0.624458, 0.915177, 0.604936, 0.915177, 0.604936, 0.914730, 0.624458, 0.914730, 0.624458, 0.914730, 0.624458, 0.914581, 0.646307, 0.914730, 0.624458, 0.914730, 0.624458, 0.914581, 0.646307, 0.914581, 0.646307, 0.914581, 0.646307, 0.914767, 0.671502, 0.914581, 0.646307, 0.914581, 0.646307, 0.914767, 0.671502, 0.914767, 0.671502, 0.914767, 0.671502, 0.915324, 0.693395, 0.914767, 0.671502, 0.914767, 0.671502, 0.915324, 0.693395, 0.915324, 0.693395, 0.915324, 0.693395, 0.916253, 0.711985, 0.915324, 0.693395, 0.915324, 0.693395, 0.916253, 0.711985, 0.916253, 0.711985, 0.916253, 0.711985, 0.917554, 0.727273, 0.916253, 0.711985, 0.916253, 0.711985, 0.917554, 0.727273, 0.917554, 0.727273, 0.917554, 0.727273, 0.919223, 0.739205, 0.917554, 0.727273, 0.917554, 0.727273, 0.919223, 0.739205, 0.919223, 0.739205, 0.919223, 0.739205, 0.921258, 0.747727, 0.919223, 0.739205, 0.919223, 0.739205, 0.921258, 0.747727, 0.921258, 0.747727, 0.921258, 0.747727, 0.923658, 0.752841, 0.921258, 0.747727, 0.921258, 0.747727, 0.923658, 0.752841, 0.923658, 0.752841, 0.923658, 0.752841, 0.926425, 0.754545, 0.923658, 0.752841, 0.923658, 0.752841, 0.926425, 0.754545, 0.926425, 0.754545, 0.926425, 0.754545, 0.932487, 0.754545, 0.926425, 0.754545, 0.926425, 0.754545, 0.932487, 0.754545, 0.932487, 0.754545, 0.932487, 0.754545, 0.932487, 0.759659, 0.932487, 0.754545, 0.932487, 0.754545, 0.932487, 0.759659, 0.932487, 0.759659, 0.932487, 0.759659, 0.932365, 0.776944, 0.932487, 0.759659, 0.932487, 0.759659, 0.932365, 0.776944, 0.932365, 0.776944, 0.932365, 0.776944, 0.931997, 0.792152, 0.932365, 0.776944, 0.932365, 0.776944, 0.931997, 0.792152, 0.931997, 0.792152, 0.931997, 0.792152, 0.931384, 0.805282, 0.931997, 0.792152, 0.931997, 0.792152, 0.931384, 0.805282, 0.931384, 0.805282, 0.931384, 0.805282, 0.930525, 0.816335, 0.931384, 0.805282, 0.931384, 0.805282, 0.930525, 0.816335, 0.930525, 0.816335, 0.930525, 0.816335, 0.929445, 0.825098, 0.930525, 0.816335, 0.930525, 0.816335, 0.929445, 0.825098, 0.929445, 0.825098, 0.929445, 0.825098, 0.928166, 0.831357, 0.929445, 0.825098, 0.929445, 0.825098, 0.928166, 0.831357, 0.928166, 0.831357, 0.928166, 0.831357, 0.926690, 0.835112, 0.928166, 0.831357, 0.928166, 0.831357, 0.926690, 0.835112, 0.926690, 0.835112, 0.926690, 0.835112, 0.925015, 0.836364, 0.926690, 0.835112, 0.926690, 0.835112, 0.925015, 0.836364, 0.925015, 0.836364, 0.925015, 0.836364, 0.923894, 0.835937, 0.925015, 0.836364, 0.925015, 0.836364, 0.923894, 0.835937, 0.923894, 0.835937, 0.923894, 0.835937, 0.922788, 0.834659, 0.923894, 0.835937, 0.923894, 0.835937, 0.922788, 0.834659, 0.922788, 0.834659, 0.922788, 0.834659, 0.921697, 0.832528, 0.922788, 0.834659, 0.922788, 0.834659, 0.921697, 0.832528, 0.921697, 0.832528, 0.921697, 0.832528, 0.920620, 0.829545, 0.921697, 0.832528, 0.921697, 0.832528, 0.920620, 0.829545, 0.920620, 0.829545, 0.920620, 0.829545, 0.919561, 0.825710, 0.920620, 0.829545, 0.920620, 0.829545, 0.919561, 0.825710, 0.919561, 0.825710, 0.919561, 0.825710, 0.918523, 0.821023, 0.919561, 0.825710, 0.919561, 0.825710, 0.918523, 0.821023, 0.918523, 0.821023, 0.918523, 0.821023, 0.917505, 0.815483, 0.918523, 0.821023, 0.918523, 0.821023, 0.917505, 0.815483, 0.917505, 0.815483, 0.917505, 0.815483, 0.916508, 0.809091, 0.917505, 0.815483, 0.917505, 0.815483, 0.916508, 0.809091, 0.916508, 0.809091, 0.916508, 0.809091, 0.916508, 0.854545, 0.916508, 0.809091, 0.916508, 0.809091, 0.916508, 0.854545, 0.916508, 0.854545, 0.916508, 0.854545, 0.917686, 0.860937, 0.916508, 0.854545, 0.916508, 0.854545, 0.917686, 0.860937, 0.917686, 0.860937, 0.917686, 0.860937, 0.918846, 0.866477, 0.917686, 0.860937, 0.917686, 0.860937, 0.918846, 0.866477, 0.918846, 0.866477, 0.918846, 0.866477, 0.919989, 0.871165, 0.918846, 0.866477, 0.918846, 0.866477, 0.919989, 0.871165, 0.919989, 0.871165, 0.919989, 0.871165, 0.921114, 0.875000, 0.919989, 0.871165, 0.919989, 0.871165, 0.921114, 0.875000, 0.921114, 0.875000, 0.921114, 0.875000, 0.922223, 0.877983, 0.921114, 0.875000, 0.921114, 0.875000, 0.922223, 0.877983, 0.922223, 0.877983, 0.922223, 0.877983, 0.923317, 0.880114, 0.922223, 0.877983, 0.922223, 0.877983, 0.923317, 0.880114, 0.923317, 0.880114, 0.923317, 0.880114, 0.924396, 0.881392, 0.923317, 0.880114, 0.923317, 0.880114, 0.924396, 0.881392, 0.924396, 0.881392, 0.924396, 0.881392, 0.925461, 0.881818, 0.924396, 0.881392, 0.924396, 0.881392, 0.925461, 0.881818, 0.925461, 0.881818, 0.925461, 0.881818, 0.928136, 0.879514, 0.925461, 0.881818, 0.925461, 0.881818, 0.928136, 0.879514, 0.928136, 0.879514, 0.928136, 0.879514, 0.930449, 0.872603, 0.928136, 0.879514, 0.928136, 0.879514, 0.930449, 0.872603, 0.930449, 0.872603, 0.930449, 0.872603, 0.932401, 0.861084, 0.930449, 0.872603, 0.930449, 0.872603, 0.932401, 0.861084, 0.932401, 0.861084, 0.932401, 0.861084, 0.933991, 0.844957, 0.932401, 0.861084, 0.932401, 0.861084, 0.933991, 0.844957, 0.933991, 0.844957, 0.933991, 0.844957, 0.935225, 0.824143, 0.933991, 0.844957, 0.933991, 0.844957, 0.935225, 0.824143, 0.935225, 0.824143, 0.935225, 0.824143, 0.936106, 0.798562, 0.935225, 0.824143, 0.935225, 0.824143, 0.936106, 0.798562, 0.936106, 0.798562, 0.936106, 0.798562, 0.936635, 0.768213, 0.936106, 0.798562, 0.936106, 0.798562, 0.936635, 0.768213, 0.936635, 0.768213, 0.914767, 0.671502, 0.914581, 0.646307, 0.918905, 0.648437, 0.914581, 0.646307, 0.914767, 0.671502, 0.918905, 0.648437, 0.914767, 0.671502, 0.918905, 0.648437, 0.919031, 0.663778, 0.918905, 0.648437, 0.914767, 0.671502, 0.919031, 0.663778, 0.915324, 0.693395, 0.914767, 0.671502, 0.919031, 0.663778, 0.914767, 0.671502, 0.915324, 0.693395, 0.919031, 0.663778, 0.915324, 0.693395, 0.919031, 0.663778, 0.919410, 0.676705, 0.919031, 0.663778, 0.915324, 0.693395, 0.919410, 0.676705, 0.916253, 0.711985, 0.915324, 0.693395, 0.919410, 0.676705, 0.915324, 0.693395, 0.916253, 0.711985, 0.919410, 0.676705, 0.916253, 0.711985, 0.919410, 0.676705, 0.920042, 0.687216, 0.919410, 0.676705, 0.916253, 0.711985, 0.920042, 0.687216, 0.917554, 0.727273, 0.916253, 0.711985, 0.920042, 0.687216, 0.916253, 0.711985, 0.917554, 0.727273, 0.920042, 0.687216, 0.917554, 0.727273, 0.920042, 0.687216, 0.920926, 0.695313, 0.920042, 0.687216, 0.917554, 0.727273, 0.920926, 0.695313, 0.919223, 0.739205, 0.917554, 0.727273, 0.920926, 0.695313, 0.917554, 0.727273, 0.919223, 0.739205, 0.920926, 0.695313, 0.922138, 0.701341, 0.919223, 0.739205, 0.920926, 0.695313, 0.919223, 0.739205, 0.922138, 0.701341, 0.920926, 0.695313, 0.921258, 0.747727, 0.919223, 0.739205, 0.922138, 0.701341, 0.919223, 0.739205, 0.921258, 0.747727, 0.922138, 0.701341, 0.923752, 0.705646, 0.921258, 0.747727, 0.922138, 0.701341, 0.921258, 0.747727, 0.923752, 0.705646, 0.922138, 0.701341, 0.923658, 0.752841, 0.921258, 0.747727, 0.923752, 0.705646, 0.921258, 0.747727, 0.923658, 0.752841, 0.923752, 0.705646, 0.923658, 0.752841, 0.923752, 0.705646, 0.925768, 0.708230, 0.923752, 0.705646, 0.923658, 0.752841, 0.925768, 0.708230, 0.923658, 0.752841, 0.925768, 0.708230, 0.926425, 0.754545, 0.925768, 0.708230, 0.923658, 0.752841, 0.926425, 0.754545, 0.926425, 0.754545, 0.925768, 0.708230, 0.928187, 0.709091, 0.925768, 0.708230, 0.926425, 0.754545, 0.928187, 0.709091, 0.926425, 0.754545, 0.928187, 0.709091, 0.932487, 0.754545, 0.928187, 0.709091, 0.926425, 0.754545, 0.932487, 0.754545, 0.928187, 0.709091, 0.932487, 0.709091, 0.932487, 0.754545, 0.932487, 0.709091, 0.928187, 0.709091, 0.932487, 0.754545, 0.932487, 0.754545, 0.932487, 0.709091, 0.936811, 0.733097, 0.932487, 0.709091, 0.932487, 0.754545, 0.936811, 0.733097, 0.936811, 0.733097, 0.936635, 0.768213, 0.932487, 0.754545, 0.936635, 0.768213, 0.936811, 0.733097, 0.932487, 0.754545, 0.936635, 0.768213, 0.932487, 0.759659, 0.932487, 0.754545, 0.932487, 0.759659, 0.936635, 0.768213, 0.932487, 0.754545, 0.932365, 0.776944, 0.932487, 0.759659, 0.936635, 0.768213, 0.932487, 0.759659, 0.932365, 0.776944, 0.936635, 0.768213, 0.936106, 0.798562, 0.932365, 0.776944, 0.936635, 0.768213, 0.932365, 0.776944, 0.936106, 0.798562, 0.936635, 0.768213, 0.931997, 0.792152, 0.932365, 0.776944, 0.936106, 0.798562, 0.932365, 0.776944, 0.931997, 0.792152, 0.936106, 0.798562, 0.935225, 0.824143, 0.931997, 0.792152, 0.936106, 0.798562, 0.931997, 0.792152, 0.935225, 0.824143, 0.936106, 0.798562, 0.931384, 0.805282, 0.931997, 0.792152, 0.935225, 0.824143, 0.931997, 0.792152, 0.931384, 0.805282, 0.935225, 0.824143, 0.933991, 0.844957, 0.931384, 0.805282, 0.935225, 0.824143, 0.931384, 0.805282, 0.933991, 0.844957, 0.935225, 0.824143, 0.930525, 0.816335, 0.931384, 0.805282, 0.933991, 0.844957, 0.931384, 0.805282, 0.930525, 0.816335, 0.933991, 0.844957, 0.932401, 0.861084, 0.930525, 0.816335, 0.933991, 0.844957, 0.930525, 0.816335, 0.932401, 0.861084, 0.933991, 0.844957, 0.929445, 0.825098, 0.930525, 0.816335, 0.932401, 0.861084, 0.930525, 0.816335, 0.929445, 0.825098, 0.932401, 0.861084, 0.930449, 0.872603, 0.929445, 0.825098, 0.932401, 0.861084, 0.929445, 0.825098, 0.930449, 0.872603, 0.932401, 0.861084, 0.928166, 0.831357, 0.929445, 0.825098, 0.930449, 0.872603, 0.929445, 0.825098, 0.928166, 0.831357, 0.930449, 0.872603, 0.928136, 0.879514, 0.928166, 0.831357, 0.930449, 0.872603, 0.928166, 0.831357, 0.928136, 0.879514, 0.930449, 0.872603, 0.928136, 0.879514, 0.926690, 0.835112, 0.928166, 0.831357, 0.926690, 0.835112, 0.928136, 0.879514, 0.928166, 0.831357, 0.928136, 0.879514, 0.925461, 0.881818, 0.926690, 0.835112, 0.925461, 0.881818, 0.928136, 0.879514, 0.926690, 0.835112, 0.925015, 0.836364, 0.926690, 0.835112, 0.925461, 0.881818, 0.926690, 0.835112, 0.925015, 0.836364, 0.925461, 0.881818, 0.925461, 0.881818, 0.924396, 0.881392, 0.925015, 0.836364, 0.924396, 0.881392, 0.925461, 0.881818, 0.925015, 0.836364, 0.924396, 0.881392, 0.923894, 0.835937, 0.925015, 0.836364, 0.923894, 0.835937, 0.924396, 0.881392, 0.925015, 0.836364, 0.923317, 0.880114, 0.923894, 0.835937, 0.924396, 0.881392, 0.923894, 0.835937, 0.923317, 0.880114, 0.924396, 0.881392, 0.923317, 0.880114, 0.922788, 0.834659, 0.923894, 0.835937, 0.922788, 0.834659, 0.923317, 0.880114, 0.923894, 0.835937, 0.922223, 0.877983, 0.922788, 0.834659, 0.923317, 0.880114, 0.922788, 0.834659, 0.922223, 0.877983, 0.923317, 0.880114, 0.922223, 0.877983, 0.921114, 0.875000, 0.922788, 0.834659, 0.921114, 0.875000, 0.922223, 0.877983, 0.922788, 0.834659, 0.922788, 0.834659, 0.921114, 0.875000, 0.921697, 0.832528, 0.921114, 0.875000, 0.922788, 0.834659, 0.921697, 0.832528, 0.921114, 0.875000, 0.919989, 0.871165, 0.921697, 0.832528, 0.919989, 0.871165, 0.921114, 0.875000, 0.921697, 0.832528, 0.919989, 0.871165, 0.920620, 0.829545, 0.921697, 0.832528, 0.920620, 0.829545, 0.919989, 0.871165, 0.921697, 0.832528, 0.919989, 0.871165, 0.918846, 0.866477, 0.920620, 0.829545, 0.918846, 0.866477, 0.919989, 0.871165, 0.920620, 0.829545, 0.918846, 0.866477, 0.919561, 0.825710, 0.920620, 0.829545, 0.919561, 0.825710, 0.918846, 0.866477, 0.920620, 0.829545, 0.918846, 0.866477, 0.917686, 0.860937, 0.919561, 0.825710, 0.917686, 0.860937, 0.918846, 0.866477, 0.919561, 0.825710, 0.917686, 0.860937, 0.918523, 0.821023, 0.919561, 0.825710, 0.918523, 0.821023, 0.917686, 0.860937, 0.919561, 0.825710, 0.917686, 0.860937, 0.916508, 0.854545, 0.918523, 0.821023, 0.916508, 0.854545, 0.917686, 0.860937, 0.918523, 0.821023, 0.916508, 0.854545, 0.917505, 0.815483, 0.918523, 0.821023, 0.917505, 0.815483, 0.916508, 0.854545, 0.918523, 0.821023, 0.916508, 0.854545, 0.916508, 0.809091, 0.917505, 0.815483, 0.916508, 0.809091, 0.916508, 0.854545, 0.917505, 0.815483, 0.936811, 0.733097, 0.932487, 0.709091, 0.932487, 0.697727, 0.932487, 0.709091, 0.936811, 0.733097, 0.932487, 0.697727, 0.936811, 0.733097, 0.932487, 0.697727, 0.932351, 0.674711, 0.932487, 0.697727, 0.936811, 0.733097, 0.932351, 0.674711, 0.936811, 0.554545, 0.936811, 0.733097, 0.932351, 0.674711, 0.936811, 0.733097, 0.936811, 0.554545, 0.932351, 0.674711, 0.936811, 0.554545, 0.932351, 0.674711, 0.931944, 0.654102, 0.932351, 0.674711, 0.936811, 0.554545, 0.931944, 0.654102, 0.932487, 0.600000, 0.936811, 0.554545, 0.931944, 0.654102, 0.936811, 0.554545, 0.932487, 0.600000, 0.931944, 0.654102, 0.932487, 0.600000, 0.932487, 0.554545, 0.936811, 0.554545, 0.932487, 0.554545, 0.932487, 0.600000, 0.936811, 0.554545, 0.931944, 0.654102, 0.931265, 0.635898, 0.932487, 0.600000, 0.931265, 0.635898, 0.931944, 0.654102, 0.932487, 0.600000, 0.931265, 0.635898, 0.930314, 0.620099, 0.932487, 0.600000, 0.930314, 0.620099, 0.931265, 0.635898, 0.932487, 0.600000, 0.930314, 0.620099, 0.931702, 0.586927, 0.932487, 0.600000, 0.931702, 0.586927, 0.930314, 0.620099, 0.932487, 0.600000, 0.930314, 0.620099, 0.929138, 0.607329, 0.931702, 0.586927, 0.929138, 0.607329, 0.930314, 0.620099, 0.931702, 0.586927, 0.929138, 0.607329, 0.930825, 0.575692, 0.931702, 0.586927, 0.930825, 0.575692, 0.929138, 0.607329, 0.931702, 0.586927, 0.930825, 0.575692, 0.929138, 0.607329, 0.929857, 0.566295, 0.929138, 0.607329, 0.930825, 0.575692, 0.929857, 0.566295, 0.929138, 0.607329, 0.927785, 0.598207, 0.929857, 0.566295, 0.927785, 0.598207, 0.929138, 0.607329, 0.929857, 0.566295, 0.927785, 0.598207, 0.928798, 0.558736, 0.929857, 0.566295, 0.928798, 0.558736, 0.927785, 0.598207, 0.929857, 0.566295, 0.928798, 0.558736, 0.927785, 0.598207, 0.927632, 0.552925, 0.927785, 0.598207, 0.928798, 0.558736, 0.927632, 0.552925, 0.927785, 0.598207, 0.926253, 0.592733, 0.927632, 0.552925, 0.926253, 0.592733, 0.927785, 0.598207, 0.927632, 0.552925, 0.926253, 0.592733, 0.926342, 0.548775, 0.927632, 0.552925, 0.926342, 0.548775, 0.926253, 0.592733, 0.927632, 0.552925, 0.926253, 0.592733, 0.924545, 0.590909, 0.926342, 0.548775, 0.924545, 0.590909, 0.926253, 0.592733, 0.926342, 0.548775, 0.924545, 0.590909, 0.924929, 0.546285, 0.926342, 0.548775, 0.924929, 0.546285, 0.924545, 0.590909, 0.926342, 0.548775, 0.924545, 0.590909, 0.923393, 0.545455, 0.924929, 0.546285, 0.923393, 0.545455, 0.924545, 0.590909, 0.924929, 0.546285, 0.923309, 0.591877, 0.923393, 0.545455, 0.924545, 0.590909, 0.923393, 0.545455, 0.923309, 0.591877, 0.924545, 0.590909, 0.923309, 0.591877, 0.921476, 0.547168, 0.923393, 0.545455, 0.921476, 0.547168, 0.923309, 0.591877, 0.923393, 0.545455, 0.922210, 0.594780, 0.921476, 0.547168, 0.923309, 0.591877, 0.921476, 0.547168, 0.922210, 0.594780, 0.923309, 0.591877, 0.919766, 0.552308, 0.921476, 0.547168, 0.922210, 0.594780, 0.921476, 0.547168, 0.919766, 0.552308, 0.922210, 0.594780, 0.921247, 0.599618, 0.919766, 0.552308, 0.922210, 0.594780, 0.919766, 0.552308, 0.921247, 0.599618, 0.922210, 0.594780, 0.918262, 0.560875, 0.919766, 0.552308, 0.921247, 0.599618, 0.919766, 0.552308, 0.918262, 0.560875, 0.921247, 0.599618, 0.921247, 0.599618, 0.920421, 0.606392, 0.918262, 0.560875, 0.920421, 0.606392, 0.921247, 0.599618, 0.918262, 0.560875, 0.920421, 0.606392, 0.916966, 0.572869, 0.918262, 0.560875, 0.916966, 0.572869, 0.920421, 0.606392, 0.918262, 0.560875, 0.920421, 0.606392, 0.919757, 0.614826, 0.916966, 0.572869, 0.919757, 0.614826, 0.920421, 0.606392, 0.916966, 0.572869, 0.919757, 0.614826, 0.915923, 0.587740, 0.916966, 0.572869, 0.915923, 0.587740, 0.919757, 0.614826, 0.916966, 0.572869, 0.919757, 0.614826, 0.919284, 0.624645, 0.915923, 0.587740, 0.919284, 0.624645, 0.919757, 0.614826, 0.915923, 0.587740, 0.919284, 0.624645, 0.915177, 0.604936, 0.915923, 0.587740, 0.915177, 0.604936, 0.919284, 0.624645, 0.915923, 0.587740, 0.914730, 0.624458, 0.915177, 0.604936, 0.919284, 0.624645, 0.915177, 0.604936, 0.914730, 0.624458, 0.919284, 0.624645, 0.919000, 0.635849, 0.914730, 0.624458, 0.919284, 0.624645, 0.914730, 0.624458, 0.919000, 0.635849, 0.919284, 0.624645, 0.914581, 0.646307, 0.914730, 0.624458, 0.919000, 0.635849, 0.914730, 0.624458, 0.914581, 0.646307, 0.919000, 0.635849, 0.918905, 0.648437, 0.914581, 0.646307, 0.919000, 0.635849, 0.914581, 0.646307, 0.918905, 0.648437, 0.919000, 0.635849, 0.949830, 0.963636, 0.949830, 0.872727, 0.949830, 0.963636, 0.949830, 0.963636, 0.949830, 0.872727, 0.949830, 0.872727, 0.949830, 0.872727, 0.958736, 0.872727, 0.949830, 0.872727, 0.949830, 0.872727, 0.958736, 0.872727, 0.958736, 0.872727, 0.958736, 0.872727, 0.958736, 0.827273, 0.958736, 0.872727, 0.958736, 0.872727, 0.958736, 0.827273, 0.958736, 0.827273, 0.958736, 0.827273, 0.949830, 0.827273, 0.958736, 0.827273, 0.958736, 0.827273, 0.949830, 0.827273, 0.949830, 0.827273, 0.949830, 0.827273, 0.949830, 0.659375, 0.949830, 0.827273, 0.949830, 0.827273, 0.949830, 0.659375, 0.949830, 0.659375, 0.949830, 0.659375, 0.949885, 0.642170, 0.949830, 0.659375, 0.949830, 0.659375, 0.949885, 0.642170, 0.949885, 0.642170, 0.949885, 0.642170, 0.950050, 0.628338, 0.949885, 0.642170, 0.949885, 0.642170, 0.950050, 0.628338, 0.950050, 0.628338, 0.950050, 0.628338, 0.950325, 0.617880, 0.950050, 0.628338, 0.950050, 0.628338, 0.950325, 0.617880, 0.950325, 0.617880, 0.950325, 0.617880, 0.950711, 0.610795, 0.950325, 0.617880, 0.950325, 0.617880, 0.950711, 0.610795, 0.950711, 0.610795, 0.950711, 0.610795, 0.951265, 0.606072, 0.950711, 0.610795, 0.950711, 0.610795, 0.951265, 0.606072, 0.951265, 0.606072, 0.951265, 0.606072, 0.952047, 0.602699, 0.951265, 0.606072, 0.951265, 0.606072, 0.952047, 0.602699, 0.952047, 0.602699, 0.952047, 0.602699, 0.953057, 0.600675, 0.952047, 0.602699, 0.952047, 0.602699, 0.953057, 0.600675, 0.953057, 0.600675, 0.953057, 0.600675, 0.954294, 0.600000, 0.953057, 0.600675, 0.953057, 0.600675, 0.954294, 0.600000, 0.954294, 0.600000, 0.954294, 0.600000, 0.958736, 0.600000, 0.954294, 0.600000, 0.954294, 0.600000, 0.958736, 0.600000, 0.958736, 0.600000, 0.958736, 0.600000, 0.958736, 0.554545, 0.958736, 0.600000, 0.958736, 0.600000, 0.958736, 0.554545, 0.958736, 0.554545, 0.958736, 0.554545, 0.954294, 0.554545, 0.958736, 0.554545, 0.958736, 0.554545, 0.954294, 0.554545, 0.954294, 0.554545, 0.954294, 0.554545, 0.951986, 0.555957, 0.954294, 0.554545, 0.954294, 0.554545, 0.951986, 0.555957, 0.951986, 0.555957, 0.951986, 0.555957, 0.950065, 0.560192, 0.951986, 0.555957, 0.951986, 0.555957, 0.950065, 0.560192, 0.950065, 0.560192, 0.950065, 0.560192, 0.948531, 0.567250, 0.950065, 0.560192, 0.950065, 0.560192, 0.948531, 0.567250, 0.948531, 0.567250, 0.948531, 0.567250, 0.947386, 0.577131, 0.948531, 0.567250, 0.948531, 0.567250, 0.947386, 0.577131, 0.947386, 0.577131, 0.947386, 0.577131, 0.946553, 0.590740, 0.947386, 0.577131, 0.947386, 0.577131, 0.946553, 0.590740, 0.946553, 0.590740, 0.946553, 0.590740, 0.945958, 0.608984, 0.946553, 0.590740, 0.946553, 0.590740, 0.945958, 0.608984, 0.945958, 0.608984, 0.945958, 0.608984, 0.945601, 0.631863, 0.945958, 0.608984, 0.945958, 0.608984, 0.945601, 0.631863, 0.945601, 0.631863, 0.945601, 0.631863, 0.945482, 0.659375, 0.945601, 0.631863, 0.945601, 0.631863, 0.945482, 0.659375, 0.945482, 0.659375, 0.945482, 0.659375, 0.945482, 0.827273, 0.945482, 0.659375, 0.945482, 0.659375, 0.945482, 0.827273, 0.945482, 0.827273, 0.945482, 0.827273, 0.942310, 0.827273, 0.945482, 0.827273, 0.945482, 0.827273, 0.942310, 0.827273, 0.942310, 0.827273, 0.942310, 0.827273, 0.942310, 0.872727, 0.942310, 0.827273, 0.942310, 0.827273, 0.942310, 0.872727, 0.942310, 0.872727, 0.942310, 0.872727, 0.945482, 0.872727, 0.942310, 0.872727, 0.942310, 0.872727, 0.945482, 0.872727, 0.945482, 0.872727, 0.945482, 0.872727, 0.945482, 0.963636, 0.945482, 0.872727, 0.945482, 0.872727, 0.945482, 0.963636, 0.945482, 0.963636, 0.942310, 0.872727, 0.945482, 0.827273, 0.945482, 0.872727, 0.945482, 0.827273, 0.942310, 0.872727, 0.945482, 0.872727, 0.942310, 0.872727, 0.942310, 0.827273, 0.945482, 0.827273, 0.942310, 0.827273, 0.942310, 0.872727, 0.945482, 0.827273, 0.945482, 0.872727, 0.945482, 0.827273, 0.949830, 0.827273, 0.945482, 0.827273, 0.945482, 0.872727, 0.949830, 0.827273, 0.945482, 0.872727, 0.949830, 0.827273, 0.949830, 0.872727, 0.949830, 0.827273, 0.945482, 0.872727, 0.949830, 0.872727, 0.945482, 0.963636, 0.945482, 0.872727, 0.949830, 0.872727, 0.945482, 0.872727, 0.945482, 0.963636, 0.949830, 0.872727, 0.945482, 0.963636, 0.949830, 0.872727, 0.949830, 0.963636, 0.949830, 0.872727, 0.945482, 0.963636, 0.949830, 0.963636, 0.949830, 0.872727, 0.949830, 0.827273, 0.958736, 0.827273, 0.949830, 0.827273, 0.949830, 0.872727, 0.958736, 0.827273, 0.949830, 0.872727, 0.958736, 0.827273, 0.958736, 0.872727, 0.958736, 0.827273, 0.949830, 0.872727, 0.958736, 0.872727, 0.945482, 0.827273, 0.949830, 0.659375, 0.949830, 0.827273, 0.949830, 0.659375, 0.945482, 0.827273, 0.949830, 0.827273, 0.945482, 0.827273, 0.945482, 0.659375, 0.949830, 0.659375, 0.945482, 0.659375, 0.945482, 0.827273, 0.949830, 0.659375, 0.949830, 0.659375, 0.945482, 0.659375, 0.949885, 0.642170, 0.945482, 0.659375, 0.949830, 0.659375, 0.949885, 0.642170, 0.945482, 0.659375, 0.945601, 0.631863, 0.949885, 0.642170, 0.945601, 0.631863, 0.945482, 0.659375, 0.949885, 0.642170, 0.949885, 0.642170, 0.945601, 0.631863, 0.950050, 0.628338, 0.945601, 0.631863, 0.949885, 0.642170, 0.950050, 0.628338, 0.945601, 0.631863, 0.945958, 0.608984, 0.950050, 0.628338, 0.945958, 0.608984, 0.945601, 0.631863, 0.950050, 0.628338, 0.950050, 0.628338, 0.945958, 0.608984, 0.950325, 0.617880, 0.945958, 0.608984, 0.950050, 0.628338, 0.950325, 0.617880, 0.950325, 0.617880, 0.945958, 0.608984, 0.946553, 0.590740, 0.945958, 0.608984, 0.950325, 0.617880, 0.946553, 0.590740, 0.950711, 0.610795, 0.950325, 0.617880, 0.946553, 0.590740, 0.950325, 0.617880, 0.950711, 0.610795, 0.946553, 0.590740, 0.946553, 0.590740, 0.947386, 0.577131, 0.950711, 0.610795, 0.947386, 0.577131, 0.946553, 0.590740, 0.950711, 0.610795, 0.950711, 0.610795, 0.947386, 0.577131, 0.948531, 0.567250, 0.947386, 0.577131, 0.950711, 0.610795, 0.948531, 0.567250, 0.951265, 0.606072, 0.950711, 0.610795, 0.948531, 0.567250, 0.950711, 0.610795, 0.951265, 0.606072, 0.948531, 0.567250, 0.948531, 0.567250, 0.950065, 0.560192, 0.951265, 0.606072, 0.950065, 0.560192, 0.948531, 0.567250, 0.951265, 0.606072, 0.951265, 0.606072, 0.950065, 0.560192, 0.952047, 0.602699, 0.950065, 0.560192, 0.951265, 0.606072, 0.952047, 0.602699, 0.950065, 0.560192, 0.951986, 0.555957, 0.952047, 0.602699, 0.951986, 0.555957, 0.950065, 0.560192, 0.952047, 0.602699, 0.952047, 0.602699, 0.951986, 0.555957, 0.953057, 0.600675, 0.951986, 0.555957, 0.952047, 0.602699, 0.953057, 0.600675, 0.953057, 0.600675, 0.951986, 0.555957, 0.954294, 0.554545, 0.951986, 0.555957, 0.953057, 0.600675, 0.954294, 0.554545, 0.954294, 0.600000, 0.953057, 0.600675, 0.954294, 0.554545, 0.953057, 0.600675, 0.954294, 0.600000, 0.954294, 0.554545, 0.954294, 0.600000, 0.954294, 0.554545, 0.958736, 0.554545, 0.954294, 0.554545, 0.954294, 0.600000, 0.958736, 0.554545, 0.954294, 0.600000, 0.958736, 0.554545, 0.958736, 0.600000, 0.958736, 0.554545, 0.954294, 0.600000, 0.958736, 0.600000, 0.964963, 0.627273, 0.969921, 0.627273, 0.964963, 0.627273, 0.964963, 0.627273, 0.969921, 0.627273, 0.969921, 0.627273, 0.969921, 0.627273, 0.969921, 0.554545, 0.969921, 0.627273, 0.969921, 0.627273, 0.969921, 0.554545, 0.969921, 0.554545, 0.969921, 0.554545, 0.964963, 0.554545, 0.969921, 0.554545, 0.969921, 0.554545, 0.964963, 0.554545, 0.964963, 0.554545, 0.964963, 0.627273, 0.969921, 0.554545, 0.969921, 0.627273, 0.969921, 0.554545, 0.964963, 0.627273, 0.969921, 0.627273, 0.964963, 0.627273, 0.964963, 0.554545, 0.969921, 0.554545, 0.964963, 0.554545, 0.964963, 0.627273, 0.969921, 0.554545, 0.980002, 0.627273, 0.984961, 0.627273, 0.980002, 0.627273, 0.980002, 0.627273, 0.984961, 0.627273, 0.984961, 0.627273, 0.984961, 0.627273, 0.984961, 0.554545, 0.984961, 0.627273, 0.984961, 0.627273, 0.984961, 0.554545, 0.984961, 0.554545, 0.984961, 0.554545, 0.980002, 0.554545, 0.984961, 0.554545, 0.984961, 0.554545, 0.980002, 0.554545, 0.980002, 0.554545, 0.980002, 0.627273, 0.984961, 0.554545, 0.984961, 0.627273, 0.984961, 0.554545, 0.980002, 0.627273, 0.984961, 0.627273, 0.980002, 0.627273, 0.980002, 0.554545, 0.984961, 0.554545, 0.980002, 0.554545, 0.980002, 0.627273, 0.984961, 0.554545, 0.995042, 0.627273, 1, 0.627273, 0.995042, 0.627273, 0.995042, 0.627273, 1, 0.627273, 1, 0.627273, 1, 0.627273, 1, 0.554545, 1, 0.627273, 1, 0.627273, 1, 0.554545, 1, 0.554545, 1, 0.554545, 0.995042, 0.554545, 1, 0.554545, 1, 0.554545, 0.995042, 0.554545, 0.995042, 0.554545, 0.995042, 0.627273, 1, 0.554545, 1, 0.627273, 1, 0.554545, 0.995042, 0.627273, 1, 0.627273, 0.995042, 0.627273, 0.995042, 0.554545, 1, 0.554545, 0.995042, 0.554545, 0.995042, 0.627273, 1, 0.554545, 0.025943, 0.067187, 0.025943, 0.181818, 0.025943, 0.067187, 0.025943, 0.067187, 0.025943, 0.181818, 0.025943, 0.181818, 0.025943, 0.181818, 0.018188, 0.181818, 0.025943, 0.181818, 0.025943, 0.181818, 0.018188, 0.181818, 0.018188, 0.181818, 0.018188, 0.181818, 0.018188, 0.227273, 0.018188, 0.181818, 0.018188, 0.181818, 0.018188, 0.227273, 0.018188, 0.227273, 0.018188, 0.227273, 0.030643, 0.227273, 0.018188, 0.227273, 0.018188, 0.227273, 0.030643, 0.227273, 0.030643, 0.227273, 0.030643, 0.227273, 0.030643, 0.046023, 0.030643, 0.227273, 0.030643, 0.227273, 0.030643, 0.046023, 0.030643, 0.046023, 0.030643, 0.046023, 0.029233, 0.035352, 0.030643, 0.046023, 0.030643, 0.046023, 0.029233, 0.035352, 0.029233, 0.035352, 0.029233, 0.035352, 0.027752, 0.026065, 0.029233, 0.035352, 0.029233, 0.035352, 0.027752, 0.026065, 0.027752, 0.026065, 0.027752, 0.026065, 0.026201, 0.018164, 0.027752, 0.026065, 0.027752, 0.026065, 0.026201, 0.018164, 0.026201, 0.018164, 0.026201, 0.018164, 0.024580, 0.011648, 0.026201, 0.018164, 0.026201, 0.018164, 0.024580, 0.011648, 0.024580, 0.011648, 0.024580, 0.011648, 0.022895, 0.006552, 0.024580, 0.011648, 0.024580, 0.011648, 0.022895, 0.006552, 0.022895, 0.006552, 0.022895, 0.006552, 0.021155, 0.002912, 0.022895, 0.006552, 0.022895, 0.006552, 0.021155, 0.002912, 0.021155, 0.002912, 0.021155, 0.002912, 0.019359, 0.000728, 0.021155, 0.002912, 0.021155, 0.002912, 0.019359, 0.000728, 0.019359, 0.000728, 0.019359, 0.000728, 0.017507, 0, 0.019359, 0.000728, 0.019359, 0.000728, 0.017507, 0, 0.017507, 0, 0.017507, 0, 0.013618, 0.003675, 0.017507, 0, 0.017507, 0, 0.013618, 0.003675, 0.013618, 0.003675, 0.013618, 0.003675, 0.010178, 0.014702, 0.013618, 0.003675, 0.013618, 0.003675, 0.010178, 0.014702, 0.010178, 0.014702, 0.010178, 0.014702, 0.007186, 0.033079, 0.010178, 0.014702, 0.010178, 0.014702, 0.007186, 0.033079, 0.007186, 0.033079, 0.007186, 0.033079, 0.004641, 0.058807, 0.007186, 0.033079, 0.007186, 0.033079, 0.004641, 0.058807, 0.004641, 0.058807, 0.004641, 0.058807, 0.002611, 0.091096, 0.004641, 0.058807, 0.004641, 0.058807, 0.002611, 0.091096, 0.002611, 0.091096, 0.002611, 0.091096, 0.001160, 0.129155, 0.002611, 0.091096, 0.002611, 0.091096, 0.001160, 0.129155, 0.001160, 0.129155, 0.001160, 0.129155, 0.000290, 0.172985, 0.001160, 0.129155, 0.001160, 0.129155, 0.000290, 0.172985, 0.000290, 0.172985, 0.000290, 0.172985, 0, 0.222585, 0.000290, 0.172985, 0.000290, 0.172985, 0, 0.222585, 0, 0.222585, 0, 0.222585, 0.000290, 0.272310, 0, 0.222585, 0, 0.222585, 0.000290, 0.272310, 0.000290, 0.272310, 0.000290, 0.272310, 0.001160, 0.316229, 0.000290, 0.272310, 0.000290, 0.272310, 0.001160, 0.316229, 0.001160, 0.316229, 0.001160, 0.316229, 0.002611, 0.354341, 0.001160, 0.316229, 0.001160, 0.316229, 0.002611, 0.354341, 0.002611, 0.354341, 0.002611, 0.354341, 0.004641, 0.386648, 0.002611, 0.354341, 0.002611, 0.354341, 0.004641, 0.386648, 0.004641, 0.386648, 0.004641, 0.386648, 0.007186, 0.412376, 0.004641, 0.386648, 0.004641, 0.386648, 0.007186, 0.412376, 0.007186, 0.412376, 0.007186, 0.412376, 0.010178, 0.430753, 0.007186, 0.412376, 0.007186, 0.412376, 0.010178, 0.430753, 0.010178, 0.430753, 0.010178, 0.430753, 0.013618, 0.441779, 0.010178, 0.430753, 0.010178, 0.430753, 0.013618, 0.441779, 0.013618, 0.441779, 0.013618, 0.441779, 0.017507, 0.445455, 0.013618, 0.441779, 0.013618, 0.441779, 0.017507, 0.445455, 0.017507, 0.445455, 0.017507, 0.445455, 0.019201, 0.444877, 0.017507, 0.445455, 0.017507, 0.445455, 0.019201, 0.444877, 0.019201, 0.444877, 0.019201, 0.444877, 0.020852, 0.443146, 0.019201, 0.444877, 0.019201, 0.444877, 0.020852, 0.443146, 0.020852, 0.443146, 0.020852, 0.443146, 0.022461, 0.440261, 0.020852, 0.443146, 0.020852, 0.443146, 0.022461, 0.440261, 0.022461, 0.440261, 0.022461, 0.440261, 0.024028, 0.436222, 0.022461, 0.440261, 0.022461, 0.440261, 0.024028, 0.436222, 0.024028, 0.436222, 0.024028, 0.436222, 0.025543, 0.431064, 0.024028, 0.436222, 0.024028, 0.436222, 0.025543, 0.431064, 0.025543, 0.431064, 0.025543, 0.431064, 0.026997, 0.424822, 0.025543, 0.431064, 0.025543, 0.431064, 0.026997, 0.424822, 0.026997, 0.424822, 0.026997, 0.424822, 0.028392, 0.417498, 0.026997, 0.424822, 0.026997, 0.424822, 0.028392, 0.417498, 0.028392, 0.417498, 0.028392, 0.417498, 0.029726, 0.409091, 0.028392, 0.417498, 0.028392, 0.417498, 0.029726, 0.409091, 0.029726, 0.409091, 0.029726, 0.409091, 0.029726, 0.345455, 0.029726, 0.409091, 0.029726, 0.409091, 0.029726, 0.345455, 0.029726, 0.345455, 0.029726, 0.345455, 0.028390, 0.358181, 0.029726, 0.345455, 0.029726, 0.345455, 0.028390, 0.358181, 0.028390, 0.358181, 0.028390, 0.358181, 0.027012, 0.369229, 0.028390, 0.358181, 0.028390, 0.358181, 0.027012, 0.369229, 0.027012, 0.369229, 0.027012, 0.369229, 0.025593, 0.378600, 0.027012, 0.369229, 0.027012, 0.369229, 0.025593, 0.378600, 0.025593, 0.378600, 0.025593, 0.378600, 0.024133, 0.386293, 0.025593, 0.378600, 0.025593, 0.378600, 0.024133, 0.386293, 0.024133, 0.386293, 0.024133, 0.386293, 0.022634, 0.392290, 0.024133, 0.386293, 0.024133, 0.386293, 0.022634, 0.392290, 0.022634, 0.392290, 0.022634, 0.392290, 0.021096, 0.396573, 0.022634, 0.392290, 0.022634, 0.392290, 0.021096, 0.396573, 0.021096, 0.396573, 0.021096, 0.396573, 0.019520, 0.399143, 0.021096, 0.396573, 0.021096, 0.396573, 0.019520, 0.399143, 0.019520, 0.399143, 0.019520, 0.399143, 0.017906, 0.400000, 0.019520, 0.399143, 0.019520, 0.399143, 0.017906, 0.400000, 0.017906, 0.400000, 0.017906, 0.400000, 0.014887, 0.397212, 0.017906, 0.400000, 0.017906, 0.400000, 0.014887, 0.397212, 0.014887, 0.397212, 0.014887, 0.397212, 0.012269, 0.388849, 0.014887, 0.397212, 0.014887, 0.397212, 0.012269, 0.388849, 0.012269, 0.388849, 0.012269, 0.388849, 0.010052, 0.374911, 0.012269, 0.388849, 0.012269, 0.388849, 0.010052, 0.374911, 0.010052, 0.374911, 0.010052, 0.374911, 0.008236, 0.355398, 0.010052, 0.374911, 0.010052, 0.374911, 0.008236, 0.355398, 0.008236, 0.355398, 0.008236, 0.355398, 0.006823, 0.330371, 0.008236, 0.355398, 0.008236, 0.355398, 0.006823, 0.330371, 0.006823, 0.330371, 0.006823, 0.330371, 0.005813, 0.299893, 0.006823, 0.330371, 0.006823, 0.330371, 0.005813, 0.299893, 0.005813, 0.299893, 0.005813, 0.299893, 0.005207, 0.263965, 0.005813, 0.299893, 0.005813, 0.299893, 0.005207, 0.263965, 0.005207, 0.263965, 0.005207, 0.263965, 0.005005, 0.222585, 0.005207, 0.263965, 0.005207, 0.263965, 0.005005, 0.222585, 0.005005, 0.222585, 0.005005, 0.222585, 0.005207, 0.181330, 0.005005, 0.222585, 0.005005, 0.222585, 0.005207, 0.181330, 0.005207, 0.181330, 0.005207, 0.181330, 0.005813, 0.145490, 0.005207, 0.181330, 0.005207, 0.181330, 0.005813, 0.145490, 0.005813, 0.145490, 0.005813, 0.145490, 0.006823, 0.115066, 0.005813, 0.145490, 0.005813, 0.145490, 0.006823, 0.115066, 0.006823, 0.115066, 0.006823, 0.115066, 0.008236, 0.090057, 0.006823, 0.115066, 0.006823, 0.115066, 0.008236, 0.090057, 0.008236, 0.090057, 0.008236, 0.090057, 0.010052, 0.070543, 0.008236, 0.090057, 0.008236, 0.090057, 0.010052, 0.070543, 0.010052, 0.070543, 0.010052, 0.070543, 0.012269, 0.056605, 0.010052, 0.070543, 0.010052, 0.070543, 0.012269, 0.056605, 0.012269, 0.056605, 0.012269, 0.056605, 0.014887, 0.048242, 0.012269, 0.056605, 0.012269, 0.056605, 0.014887, 0.048242, 0.014887, 0.048242, 0.014887, 0.048242, 0.017906, 0.045455, 0.014887, 0.048242, 0.014887, 0.048242, 0.017906, 0.045455, 0.017906, 0.045455, 0.017906, 0.045455, 0.019130, 0.045783, 0.017906, 0.045455, 0.017906, 0.045455, 0.019130, 0.045783, 0.019130, 0.045783, 0.019130, 0.045783, 0.020286, 0.046768, 0.019130, 0.045783, 0.019130, 0.045783, 0.020286, 0.046768, 0.020286, 0.046768, 0.020286, 0.046768, 0.021374, 0.048411, 0.020286, 0.046768, 0.020286, 0.046768, 0.021374, 0.048411, 0.021374, 0.048411, 0.021374, 0.048411, 0.022395, 0.050710, 0.021374, 0.048411, 0.021374, 0.048411, 0.022395, 0.050710, 0.022395, 0.050710, 0.022395, 0.050710, 0.023357, 0.053711, 0.022395, 0.050710, 0.022395, 0.050710, 0.023357, 0.053711, 0.023357, 0.053711, 0.023357, 0.053711, 0.024269, 0.057457, 0.023357, 0.053711, 0.023357, 0.053711, 0.024269, 0.057457, 0.024269, 0.057457, 0.024269, 0.057457, 0.025131, 0.061950, 0.024269, 0.057457, 0.024269, 0.057457, 0.025131, 0.061950, 0.025131, 0.061950, 0.005207, 0.263965, 0.000290, 0.272310, 0, 0.222585, 0.000290, 0.272310, 0.005207, 0.263965, 0, 0.222585, 0.001160, 0.316229, 0.000290, 0.272310, 0.005207, 0.263965, 0.000290, 0.272310, 0.001160, 0.316229, 0.005207, 0.263965, 0.001160, 0.316229, 0.005207, 0.263965, 0.005813, 0.299893, 0.005207, 0.263965, 0.001160, 0.316229, 0.005813, 0.299893, 0.002611, 0.354341, 0.001160, 0.316229, 0.005813, 0.299893, 0.001160, 0.316229, 0.002611, 0.354341, 0.005813, 0.299893, 0.002611, 0.354341, 0.005813, 0.299893, 0.006823, 0.330371, 0.005813, 0.299893, 0.002611, 0.354341, 0.006823, 0.330371, 0.004641, 0.386648, 0.002611, 0.354341, 0.006823, 0.330371, 0.002611, 0.354341, 0.004641, 0.386648, 0.006823, 0.330371, 0.004641, 0.386648, 0.006823, 0.330371, 0.008236, 0.355398, 0.006823, 0.330371, 0.004641, 0.386648, 0.008236, 0.355398, 0.007186, 0.412376, 0.004641, 0.386648, 0.008236, 0.355398, 0.004641, 0.386648, 0.007186, 0.412376, 0.008236, 0.355398, 0.007186, 0.412376, 0.008236, 0.355398, 0.010052, 0.374911, 0.008236, 0.355398, 0.007186, 0.412376, 0.010052, 0.374911, 0.007186, 0.412376, 0.010052, 0.374911, 0.010178, 0.430753, 0.010052, 0.374911, 0.007186, 0.412376, 0.010178, 0.430753, 0.010178, 0.430753, 0.010052, 0.374911, 0.012269, 0.388849, 0.010052, 0.374911, 0.010178, 0.430753, 0.012269, 0.388849, 0.010178, 0.430753, 0.012269, 0.388849, 0.013618, 0.441779, 0.012269, 0.388849, 0.010178, 0.430753, 0.013618, 0.441779, 0.013618, 0.441779, 0.012269, 0.388849, 0.014887, 0.397212, 0.012269, 0.388849, 0.013618, 0.441779, 0.014887, 0.397212, 0.017507, 0.445455, 0.013618, 0.441779, 0.014887, 0.397212, 0.013618, 0.441779, 0.017507, 0.445455, 0.014887, 0.397212, 0.014887, 0.397212, 0.017906, 0.400000, 0.017507, 0.445455, 0.017906, 0.400000, 0.014887, 0.397212, 0.017507, 0.445455, 0.019201, 0.444877, 0.017507, 0.445455, 0.017906, 0.400000, 0.017507, 0.445455, 0.019201, 0.444877, 0.017906, 0.400000, 0.019201, 0.444877, 0.017906, 0.400000, 0.019520, 0.399143, 0.017906, 0.400000, 0.019201, 0.444877, 0.019520, 0.399143, 0.019201, 0.444877, 0.019520, 0.399143, 0.020852, 0.443146, 0.019520, 0.399143, 0.019201, 0.444877, 0.020852, 0.443146, 0.020852, 0.443146, 0.019520, 0.399143, 0.021096, 0.396573, 0.019520, 0.399143, 0.020852, 0.443146, 0.021096, 0.396573, 0.020852, 0.443146, 0.021096, 0.396573, 0.022461, 0.440261, 0.021096, 0.396573, 0.020852, 0.443146, 0.022461, 0.440261, 0.022461, 0.440261, 0.021096, 0.396573, 0.022634, 0.392290, 0.021096, 0.396573, 0.022461, 0.440261, 0.022634, 0.392290, 0.022461, 0.440261, 0.022634, 0.392290, 0.024028, 0.436222, 0.022634, 0.392290, 0.022461, 0.440261, 0.024028, 0.436222, 0.024028, 0.436222, 0.022634, 0.392290, 0.024133, 0.386293, 0.022634, 0.392290, 0.024028, 0.436222, 0.024133, 0.386293, 0.024028, 0.436222, 0.024133, 0.386293, 0.025543, 0.431064, 0.024133, 0.386293, 0.024028, 0.436222, 0.025543, 0.431064, 0.025543, 0.431064, 0.024133, 0.386293, 0.026997, 0.424822, 0.024133, 0.386293, 0.025543, 0.431064, 0.026997, 0.424822, 0.024133, 0.386293, 0.025593, 0.378600, 0.026997, 0.424822, 0.025593, 0.378600, 0.024133, 0.386293, 0.026997, 0.424822, 0.026997, 0.424822, 0.025593, 0.378600, 0.028392, 0.417498, 0.025593, 0.378600, 0.026997, 0.424822, 0.028392, 0.417498, 0.025593, 0.378600, 0.027012, 0.369229, 0.028392, 0.417498, 0.027012, 0.369229, 0.025593, 0.378600, 0.028392, 0.417498, 0.028392, 0.417498, 0.027012, 0.369229, 0.029726, 0.409091, 0.027012, 0.369229, 0.028392, 0.417498, 0.029726, 0.409091, 0.027012, 0.369229, 0.028390, 0.358181, 0.029726, 0.409091, 0.028390, 0.358181, 0.027012, 0.369229, 0.029726, 0.409091, 0.028390, 0.358181, 0.029726, 0.345455, 0.029726, 0.409091, 0.029726, 0.345455, 0.028390, 0.358181, 0.029726, 0.409091, 0.005207, 0.263965, 0, 0.222585, 0.005005, 0.222585, 0, 0.222585, 0.005207, 0.263965, 0.005005, 0.222585, 0.005005, 0.222585, 0, 0.222585, 0.005207, 0.181330, 0, 0.222585, 0.005005, 0.222585, 0.005207, 0.181330, 0, 0.222585, 0.000290, 0.172985, 0.005207, 0.181330, 0.000290, 0.172985, 0, 0.222585, 0.005207, 0.181330, 0.005207, 0.181330, 0.000290, 0.172985, 0.001160, 0.129155, 0.000290, 0.172985, 0.005207, 0.181330, 0.001160, 0.129155, 0.005813, 0.145490, 0.005207, 0.181330, 0.001160, 0.129155, 0.005207, 0.181330, 0.005813, 0.145490, 0.001160, 0.129155, 0.005813, 0.145490, 0.001160, 0.129155, 0.002611, 0.091096, 0.001160, 0.129155, 0.005813, 0.145490, 0.002611, 0.091096, 0.006823, 0.115066, 0.005813, 0.145490, 0.002611, 0.091096, 0.005813, 0.145490, 0.006823, 0.115066, 0.002611, 0.091096, 0.002611, 0.091096, 0.004641, 0.058807, 0.006823, 0.115066, 0.004641, 0.058807, 0.002611, 0.091096, 0.006823, 0.115066, 0.006823, 0.115066, 0.004641, 0.058807, 0.008236, 0.090057, 0.004641, 0.058807, 0.006823, 0.115066, 0.008236, 0.090057, 0.004641, 0.058807, 0.007186, 0.033079, 0.008236, 0.090057, 0.007186, 0.033079, 0.004641, 0.058807, 0.008236, 0.090057, 0.008236, 0.090057, 0.007186, 0.033079, 0.010052, 0.070543, 0.007186, 0.033079, 0.008236, 0.090057, 0.010052, 0.070543, 0.010052, 0.070543, 0.007186, 0.033079, 0.010178, 0.014702, 0.007186, 0.033079, 0.010052, 0.070543, 0.010178, 0.014702, 0.010052, 0.070543, 0.010178, 0.014702, 0.012269, 0.056605, 0.010178, 0.014702, 0.010052, 0.070543, 0.012269, 0.056605, 0.012269, 0.056605, 0.010178, 0.014702, 0.013618, 0.003675, 0.010178, 0.014702, 0.012269, 0.056605, 0.013618, 0.003675, 0.014887, 0.048242, 0.012269, 0.056605, 0.013618, 0.003675, 0.012269, 0.056605, 0.014887, 0.048242, 0.013618, 0.003675, 0.014887, 0.048242, 0.013618, 0.003675, 0.017507, 0, 0.013618, 0.003675, 0.014887, 0.048242, 0.017507, 0, 0.017906, 0.045455, 0.014887, 0.048242, 0.017507, 0, 0.014887, 0.048242, 0.017906, 0.045455, 0.017507, 0, 0.017906, 0.045455, 0.017507, 0, 0.019359, 0.000728, 0.017507, 0, 0.017906, 0.045455, 0.019359, 0.000728, 0.019130, 0.045783, 0.017906, 0.045455, 0.019359, 0.000728, 0.017906, 0.045455, 0.019130, 0.045783, 0.019359, 0.000728, 0.020286, 0.046768, 0.019130, 0.045783, 0.019359, 0.000728, 0.019130, 0.045783, 0.020286, 0.046768, 0.019359, 0.000728, 0.021155, 0.002912, 0.020286, 0.046768, 0.019359, 0.000728, 0.020286, 0.046768, 0.021155, 0.002912, 0.019359, 0.000728, 0.020286, 0.046768, 0.021155, 0.002912, 0.021374, 0.048411, 0.021155, 0.002912, 0.020286, 0.046768, 0.021374, 0.048411, 0.021374, 0.048411, 0.021155, 0.002912, 0.022895, 0.006552, 0.021155, 0.002912, 0.021374, 0.048411, 0.022895, 0.006552, 0.022395, 0.050710, 0.021374, 0.048411, 0.022895, 0.006552, 0.021374, 0.048411, 0.022395, 0.050710, 0.022895, 0.006552, 0.022395, 0.050710, 0.022895, 0.006552, 0.024580, 0.011648, 0.022895, 0.006552, 0.022395, 0.050710, 0.024580, 0.011648, 0.023357, 0.053711, 0.022395, 0.050710, 0.024580, 0.011648, 0.022395, 0.050710, 0.023357, 0.053711, 0.024580, 0.011648, 0.024269, 0.057457, 0.023357, 0.053711, 0.024580, 0.011648, 0.023357, 0.053711, 0.024269, 0.057457, 0.024580, 0.011648, 0.026201, 0.018164, 0.024269, 0.057457, 0.024580, 0.011648, 0.024269, 0.057457, 0.026201, 0.018164, 0.024580, 0.011648, 0.025131, 0.061950, 0.024269, 0.057457, 0.026201, 0.018164, 0.024269, 0.057457, 0.025131, 0.061950, 0.026201, 0.018164, 0.025131, 0.061950, 0.026201, 0.018164, 0.027752, 0.026065, 0.026201, 0.018164, 0.025131, 0.061950, 0.027752, 0.026065, 0.025943, 0.067187, 0.025131, 0.061950, 0.027752, 0.026065, 0.025131, 0.061950, 0.025943, 0.067187, 0.027752, 0.026065, 0.025943, 0.067187, 0.027752, 0.026065, 0.029233, 0.035352, 0.027752, 0.026065, 0.025943, 0.067187, 0.029233, 0.035352, 0.030643, 0.046023, 0.025943, 0.067187, 0.029233, 0.035352, 0.025943, 0.067187, 0.030643, 0.046023, 0.029233, 0.035352, 0.030643, 0.046023, 0.025943, 0.181818, 0.025943, 0.067187, 0.025943, 0.181818, 0.030643, 0.046023, 0.025943, 0.067187, 0.025943, 0.181818, 0.030643, 0.046023, 0.030643, 0.227273, 0.030643, 0.046023, 0.025943, 0.181818, 0.030643, 0.227273, 0.018188, 0.227273, 0.025943, 0.181818, 0.030643, 0.227273, 0.025943, 0.181818, 0.018188, 0.227273, 0.030643, 0.227273, 0.018188, 0.227273, 0.018188, 0.181818, 0.025943, 0.181818, 0.018188, 0.181818, 0.018188, 0.227273, 0.025943, 0.181818, 0.061943, 0.182955, 0.061943, 0.154545, 0.061943, 0.182955, 0.061943, 0.182955, 0.061943, 0.154545, 0.061943, 0.154545, 0.061943, 0.154545, 0.042063, 0.154545, 0.061943, 0.154545, 0.061943, 0.154545, 0.042063, 0.154545, 0.042063, 0.154545, 0.042063, 0.154545, 0.042337, 0.129497, 0.042063, 0.154545, 0.042063, 0.154545, 0.042337, 0.129497, 0.042337, 0.129497, 0.042337, 0.129497, 0.042877, 0.107617, 0.042337, 0.129497, 0.042337, 0.129497, 0.042877, 0.107617, 0.042877, 0.107617, 0.042877, 0.107617, 0.043682, 0.088907, 0.042877, 0.107617, 0.042877, 0.107617, 0.043682, 0.088907, 0.043682, 0.088907, 0.043682, 0.088907, 0.044754, 0.073366, 0.043682, 0.088907, 0.043682, 0.088907, 0.044754, 0.073366, 0.044754, 0.073366, 0.044754, 0.073366, 0.046076, 0.061155, 0.044754, 0.073366, 0.044754, 0.073366, 0.046076, 0.061155, 0.046076, 0.061155, 0.046076, 0.061155, 0.047635, 0.052433, 0.046076, 0.061155, 0.046076, 0.061155, 0.047635, 0.052433, 0.047635, 0.052433, 0.047635, 0.052433, 0.049431, 0.047199, 0.047635, 0.052433, 0.047635, 0.052433, 0.049431, 0.047199, 0.049431, 0.047199, 0.049431, 0.047199, 0.051463, 0.045455, 0.049431, 0.047199, 0.049431, 0.047199, 0.051463, 0.045455, 0.051463, 0.045455, 0.051463, 0.045455, 0.052699, 0.045881, 0.051463, 0.045455, 0.051463, 0.045455, 0.052699, 0.045881, 0.052699, 0.045881, 0.052699, 0.045881, 0.053916, 0.047159, 0.052699, 0.045881, 0.052699, 0.045881, 0.053916, 0.047159, 0.053916, 0.047159, 0.053916, 0.047159, 0.055113, 0.049290, 0.053916, 0.047159, 0.053916, 0.047159, 0.055113, 0.049290, 0.055113, 0.049290, 0.055113, 0.049290, 0.056292, 0.052273, 0.055113, 0.049290, 0.055113, 0.049290, 0.056292, 0.052273, 0.056292, 0.052273, 0.056292, 0.052273, 0.057459, 0.056108, 0.056292, 0.052273, 0.056292, 0.052273, 0.057459, 0.056108, 0.057459, 0.056108, 0.057459, 0.056108, 0.058621, 0.060795, 0.057459, 0.056108, 0.057459, 0.056108, 0.058621, 0.060795, 0.058621, 0.060795, 0.058621, 0.060795, 0.059779, 0.066335, 0.058621, 0.060795, 0.058621, 0.060795, 0.059779, 0.066335, 0.059779, 0.066335, 0.059779, 0.066335, 0.060933, 0.072727, 0.059779, 0.066335, 0.059779, 0.066335, 0.060933, 0.072727, 0.060933, 0.072727, 0.060933, 0.072727, 0.060933, 0.027273, 0.060933, 0.072727, 0.060933, 0.072727, 0.060933, 0.027273, 0.060933, 0.027273, 0.060933, 0.027273, 0.059762, 0.020996, 0.060933, 0.027273, 0.060933, 0.027273, 0.059762, 0.020996, 0.059762, 0.020996, 0.059762, 0.020996, 0.058577, 0.015518, 0.059762, 0.020996, 0.059762, 0.020996, 0.058577, 0.015518, 0.058577, 0.015518, 0.058577, 0.015518, 0.057377, 0.010840, 0.058577, 0.015518, 0.058577, 0.015518, 0.057377, 0.010840, 0.057377, 0.010840, 0.057377, 0.010840, 0.056163, 0.006960, 0.057377, 0.010840, 0.057377, 0.010840, 0.056163, 0.006960, 0.056163, 0.006960, 0.056163, 0.006960, 0.054936, 0.003915, 0.056163, 0.006960, 0.056163, 0.006960, 0.054936, 0.003915, 0.054936, 0.003915, 0.054936, 0.003915, 0.053701, 0.001740, 0.054936, 0.003915, 0.054936, 0.003915, 0.053701, 0.001740, 0.053701, 0.001740, 0.053701, 0.001740, 0.052457, 0.000435, 0.053701, 0.001740, 0.053701, 0.001740, 0.052457, 0.000435, 0.052457, 0.000435, 0.052457, 0.000435, 0.051204, 0, 0.052457, 0.000435, 0.052457, 0.000435, 0.051204, 0, 0.051204, 0, 0.051204, 0, 0.048219, 0.002788, 0.051204, 0, 0.051204, 0, 0.048219, 0.002788, 0.048219, 0.002788, 0.048219, 0.002788, 0.045562, 0.011151, 0.048219, 0.002788, 0.048219, 0.002788, 0.045562, 0.011151, 0.045562, 0.011151, 0.045562, 0.011151, 0.043232, 0.025089, 0.045562, 0.011151, 0.045562, 0.011151, 0.043232, 0.025089, 0.043232, 0.025089, 0.043232, 0.025089, 0.041229, 0.044602, 0.043232, 0.025089, 0.043232, 0.025089, 0.041229, 0.044602, 0.041229, 0.044602, 0.041229, 0.044602, 0.039620, 0.068874, 0.041229, 0.044602, 0.041229, 0.044602, 0.039620, 0.068874, 0.039620, 0.068874, 0.039620, 0.068874, 0.038471, 0.097088, 0.039620, 0.068874, 0.039620, 0.068874, 0.038471, 0.097088, 0.038471, 0.097088, 0.038471, 0.097088, 0.037781, 0.129244, 0.038471, 0.097088, 0.038471, 0.097088, 0.037781, 0.129244, 0.037781, 0.129244, 0.037781, 0.129244, 0.037551, 0.165341, 0.037781, 0.129244, 0.037781, 0.129244, 0.037551, 0.165341, 0.037551, 0.165341, 0.037551, 0.165341, 0.037770, 0.202654, 0.037551, 0.165341, 0.037551, 0.165341, 0.037770, 0.202654, 0.037770, 0.202654, 0.037770, 0.202654, 0.038424, 0.235902, 0.037770, 0.202654, 0.037770, 0.202654, 0.038424, 0.235902, 0.038424, 0.235902, 0.038424, 0.235902, 0.039514, 0.265083, 0.038424, 0.235902, 0.038424, 0.235902, 0.039514, 0.265083, 0.039514, 0.265083, 0.039514, 0.265083, 0.041041, 0.290199, 0.039514, 0.265083, 0.039514, 0.265083, 0.041041, 0.290199, 0.041041, 0.290199, 0.041041, 0.290199, 0.042938, 0.310396, 0.041041, 0.290199, 0.041041, 0.290199, 0.042938, 0.310396, 0.042938, 0.310396, 0.042938, 0.310396, 0.045139, 0.324822, 0.042938, 0.310396, 0.042938, 0.310396, 0.045139, 0.324822, 0.045139, 0.324822, 0.045139, 0.324822, 0.047643, 0.333478, 0.045139, 0.324822, 0.045139, 0.324822, 0.047643, 0.333478, 0.047643, 0.333478, 0.047643, 0.333478, 0.050452, 0.336364, 0.047643, 0.333478, 0.047643, 0.333478, 0.050452, 0.336364, 0.050452, 0.336364, 0.050452, 0.336364, 0.052969, 0.333785, 0.050452, 0.336364, 0.050452, 0.336364, 0.052969, 0.333785, 0.052969, 0.333785, 0.052969, 0.333785, 0.055208, 0.326048, 0.052969, 0.333785, 0.052969, 0.333785, 0.055208, 0.326048, 0.055208, 0.326048, 0.055208, 0.326048, 0.057169, 0.313153, 0.055208, 0.326048, 0.055208, 0.326048, 0.057169, 0.313153, 0.057169, 0.313153, 0.057169, 0.313153, 0.058853, 0.295099, 0.057169, 0.313153, 0.057169, 0.313153, 0.058853, 0.295099, 0.058853, 0.295099, 0.058853, 0.295099, 0.060205, 0.272616, 0.058853, 0.295099, 0.058853, 0.295099, 0.060205, 0.272616, 0.060205, 0.272616, 0.060205, 0.272616, 0.061171, 0.246431, 0.060205, 0.272616, 0.060205, 0.272616, 0.061171, 0.246431, 0.061171, 0.246431, 0.061171, 0.246431, 0.061750, 0.216544, 0.061171, 0.246431, 0.061171, 0.246431, 0.061750, 0.216544, 0.061750, 0.216544, 0.037770, 0.202654, 0.037551, 0.165341, 0.042204, 0.199716, 0.037551, 0.165341, 0.037770, 0.202654, 0.042204, 0.199716, 0.038424, 0.235902, 0.037770, 0.202654, 0.042204, 0.199716, 0.037770, 0.202654, 0.038424, 0.235902, 0.042204, 0.199716, 0.038424, 0.235902, 0.042204, 0.199716, 0.042484, 0.220224, 0.042204, 0.199716, 0.038424, 0.235902, 0.042484, 0.220224, 0.042994, 0.238281, 0.038424, 0.235902, 0.042484, 0.220224, 0.038424, 0.235902, 0.042994, 0.238281, 0.042484, 0.220224, 0.042994, 0.238281, 0.039514, 0.265083, 0.038424, 0.235902, 0.039514, 0.265083, 0.042994, 0.238281, 0.038424, 0.235902, 0.039514, 0.265083, 0.042994, 0.238281, 0.043735, 0.253888, 0.042994, 0.238281, 0.039514, 0.265083, 0.043735, 0.253888, 0.041041, 0.290199, 0.039514, 0.265083, 0.043735, 0.253888, 0.039514, 0.265083, 0.041041, 0.290199, 0.043735, 0.253888, 0.041041, 0.290199, 0.043735, 0.253888, 0.044707, 0.267045, 0.043735, 0.253888, 0.041041, 0.290199, 0.044707, 0.267045, 0.042938, 0.310396, 0.041041, 0.290199, 0.044707, 0.267045, 0.041041, 0.290199, 0.042938, 0.310396, 0.044707, 0.267045, 0.042938, 0.310396, 0.044707, 0.267045, 0.045884, 0.277486, 0.044707, 0.267045, 0.042938, 0.310396, 0.045884, 0.277486, 0.045139, 0.324822, 0.042938, 0.310396, 0.045884, 0.277486, 0.042938, 0.310396, 0.045139, 0.324822, 0.045884, 0.277486, 0.045139, 0.324822, 0.045884, 0.277486, 0.047242, 0.284943, 0.045884, 0.277486, 0.045139, 0.324822, 0.047242, 0.284943, 0.045139, 0.324822, 0.047242, 0.284943, 0.047643, 0.333478, 0.047242, 0.284943, 0.045139, 0.324822, 0.047643, 0.333478, 0.047643, 0.333478, 0.047242, 0.284943, 0.048780, 0.289418, 0.047242, 0.284943, 0.047643, 0.333478, 0.048780, 0.289418, 0.047643, 0.333478, 0.048780, 0.289418, 0.050452, 0.336364, 0.048780, 0.289418, 0.047643, 0.333478, 0.050452, 0.336364, 0.050452, 0.336364, 0.048780, 0.289418, 0.050499, 0.290909, 0.048780, 0.289418, 0.050452, 0.336364, 0.050499, 0.290909, 0.050452, 0.336364, 0.050499, 0.290909, 0.052019, 0.289364, 0.050499, 0.290909, 0.050452, 0.336364, 0.052019, 0.289364, 0.050452, 0.336364, 0.052019, 0.289364, 0.052969, 0.333785, 0.052019, 0.289364, 0.050452, 0.336364, 0.052969, 0.333785, 0.052969, 0.333785, 0.052019, 0.289364, 0.053381, 0.284730, 0.052019, 0.289364, 0.052969, 0.333785, 0.053381, 0.284730, 0.052969, 0.333785, 0.053381, 0.284730, 0.055208, 0.326048, 0.053381, 0.284730, 0.052969, 0.333785, 0.055208, 0.326048, 0.053381, 0.284730, 0.054586, 0.277006, 0.055208, 0.326048, 0.054586, 0.277006, 0.053381, 0.284730, 0.055208, 0.326048, 0.055208, 0.326048, 0.054586, 0.277006, 0.057169, 0.313153, 0.054586, 0.277006, 0.055208, 0.326048, 0.057169, 0.313153, 0.054586, 0.277006, 0.055634, 0.266193, 0.057169, 0.313153, 0.055634, 0.266193, 0.054586, 0.277006, 0.057169, 0.313153, 0.057169, 0.313153, 0.055634, 0.266193, 0.058853, 0.295099, 0.055634, 0.266193, 0.057169, 0.313153, 0.058853, 0.295099, 0.055634, 0.266193, 0.056485, 0.252788, 0.058853, 0.295099, 0.056485, 0.252788, 0.055634, 0.266193, 0.058853, 0.295099, 0.058853, 0.295099, 0.056485, 0.252788, 0.060205, 0.272616, 0.056485, 0.252788, 0.058853, 0.295099, 0.060205, 0.272616, 0.056485, 0.252788, 0.057100, 0.237287, 0.060205, 0.272616, 0.057100, 0.237287, 0.056485, 0.252788, 0.060205, 0.272616, 0.060205, 0.272616, 0.057100, 0.237287, 0.061171, 0.246431, 0.057100, 0.237287, 0.060205, 0.272616, 0.061171, 0.246431, 0.057100, 0.237287, 0.057478, 0.219691, 0.061171, 0.246431, 0.057478, 0.219691, 0.057100, 0.237287, 0.061171, 0.246431, 0.061171, 0.246431, 0.057478, 0.219691, 0.061750, 0.216544, 0.057478, 0.219691, 0.061171, 0.246431, 0.061750, 0.216544, 0.057478, 0.219691, 0.057620, 0.200000, 0.061750, 0.216544, 0.057620, 0.200000, 0.057478, 0.219691, 0.061750, 0.216544, 0.061750, 0.216544, 0.057620, 0.200000, 0.061943, 0.182955, 0.057620, 0.200000, 0.061750, 0.216544, 0.061943, 0.182955, 0.061943, 0.154545, 0.061943, 0.182955, 0.057620, 0.200000, 0.061943, 0.182955, 0.061943, 0.154545, 0.057620, 0.200000, 0.057620, 0.200000, 0.042063, 0.154545, 0.061943, 0.154545, 0.042063, 0.154545, 0.057620, 0.200000, 0.061943, 0.154545, 0.057620, 0.200000, 0.042204, 0.199716, 0.042063, 0.154545, 0.042204, 0.199716, 0.057620, 0.200000, 0.042063, 0.154545, 0.037551, 0.165341, 0.042063, 0.154545, 0.042204, 0.199716, 0.042063, 0.154545, 0.037551, 0.165341, 0.042204, 0.199716, 0.037551, 0.165341, 0.037781, 0.129244, 0.042063, 0.154545, 0.037781, 0.129244, 0.037551, 0.165341, 0.042063, 0.154545, 0.042063, 0.154545, 0.037781, 0.129244, 0.042337, 0.129497, 0.037781, 0.129244, 0.042063, 0.154545, 0.042337, 0.129497, 0.037781, 0.129244, 0.038471, 0.097088, 0.042337, 0.129497, 0.038471, 0.097088, 0.037781, 0.129244, 0.042337, 0.129497, 0.042337, 0.129497, 0.038471, 0.097088, 0.042877, 0.107617, 0.038471, 0.097088, 0.042337, 0.129497, 0.042877, 0.107617, 0.038471, 0.097088, 0.039620, 0.068874, 0.042877, 0.107617, 0.039620, 0.068874, 0.038471, 0.097088, 0.042877, 0.107617, 0.042877, 0.107617, 0.039620, 0.068874, 0.043682, 0.088907, 0.039620, 0.068874, 0.042877, 0.107617, 0.043682, 0.088907, 0.039620, 0.068874, 0.041229, 0.044602, 0.043682, 0.088907, 0.041229, 0.044602, 0.039620, 0.068874, 0.043682, 0.088907, 0.043682, 0.088907, 0.041229, 0.044602, 0.044754, 0.073366, 0.041229, 0.044602, 0.043682, 0.088907, 0.044754, 0.073366, 0.041229, 0.044602, 0.043232, 0.025089, 0.044754, 0.073366, 0.043232, 0.025089, 0.041229, 0.044602, 0.044754, 0.073366, 0.044754, 0.073366, 0.043232, 0.025089, 0.046076, 0.061155, 0.043232, 0.025089, 0.044754, 0.073366, 0.046076, 0.061155, 0.046076, 0.061155, 0.043232, 0.025089, 0.045562, 0.011151, 0.043232, 0.025089, 0.046076, 0.061155, 0.045562, 0.011151, 0.047635, 0.052433, 0.046076, 0.061155, 0.045562, 0.011151, 0.046076, 0.061155, 0.047635, 0.052433, 0.045562, 0.011151, 0.047635, 0.052433, 0.045562, 0.011151, 0.048219, 0.002788, 0.045562, 0.011151, 0.047635, 0.052433, 0.048219, 0.002788, 0.049431, 0.047199, 0.047635, 0.052433, 0.048219, 0.002788, 0.047635, 0.052433, 0.049431, 0.047199, 0.048219, 0.002788, 0.049431, 0.047199, 0.048219, 0.002788, 0.051204, 0, 0.048219, 0.002788, 0.049431, 0.047199, 0.051204, 0, 0.051463, 0.045455, 0.049431, 0.047199, 0.051204, 0, 0.049431, 0.047199, 0.051463, 0.045455, 0.051204, 0, 0.051463, 0.045455, 0.051204, 0, 0.052457, 0.000435, 0.051204, 0, 0.051463, 0.045455, 0.052457, 0.000435, 0.051463, 0.045455, 0.052457, 0.000435, 0.052699, 0.045881, 0.052457, 0.000435, 0.051463, 0.045455, 0.052699, 0.045881, 0.052699, 0.045881, 0.052457, 0.000435, 0.053701, 0.001740, 0.052457, 0.000435, 0.052699, 0.045881, 0.053701, 0.001740, 0.052699, 0.045881, 0.053701, 0.001740, 0.053916, 0.047159, 0.053701, 0.001740, 0.052699, 0.045881, 0.053916, 0.047159, 0.053916, 0.047159, 0.053701, 0.001740, 0.054936, 0.003915, 0.053701, 0.001740, 0.053916, 0.047159, 0.054936, 0.003915, 0.053916, 0.047159, 0.054936, 0.003915, 0.055113, 0.049290, 0.054936, 0.003915, 0.053916, 0.047159, 0.055113, 0.049290, 0.055113, 0.049290, 0.054936, 0.003915, 0.056163, 0.006960, 0.054936, 0.003915, 0.055113, 0.049290, 0.056163, 0.006960, 0.055113, 0.049290, 0.056163, 0.006960, 0.056292, 0.052273, 0.056163, 0.006960, 0.055113, 0.049290, 0.056292, 0.052273, 0.056292, 0.052273, 0.056163, 0.006960, 0.057377, 0.010840, 0.056163, 0.006960, 0.056292, 0.052273, 0.057377, 0.010840, 0.057459, 0.056108, 0.056292, 0.052273, 0.057377, 0.010840, 0.056292, 0.052273, 0.057459, 0.056108, 0.057377, 0.010840, 0.058577, 0.015518, 0.057459, 0.056108, 0.057377, 0.010840, 0.057459, 0.056108, 0.058577, 0.015518, 0.057377, 0.010840, 0.057459, 0.056108, 0.058577, 0.015518, 0.059762, 0.020996, 0.058577, 0.015518, 0.057459, 0.056108, 0.059762, 0.020996, 0.058621, 0.060795, 0.057459, 0.056108, 0.059762, 0.020996, 0.057459, 0.056108, 0.058621, 0.060795, 0.059762, 0.020996, 0.058621, 0.060795, 0.059762, 0.020996, 0.060933, 0.027273, 0.059762, 0.020996, 0.058621, 0.060795, 0.060933, 0.027273, 0.059779, 0.066335, 0.058621, 0.060795, 0.060933, 0.027273, 0.058621, 0.060795, 0.059779, 0.066335, 0.060933, 0.027273, 0.059779, 0.066335, 0.060933, 0.027273, 0.060933, 0.072727, 0.060933, 0.027273, 0.059779, 0.066335, 0.060933, 0.072727, 0.057620, 0.200000, 0.057478, 0.219691, 0.057620, 0.200000, 0.057620, 0.200000, 0.057478, 0.219691, 0.057478, 0.219691, 0.057478, 0.219691, 0.057100, 0.237287, 0.057478, 0.219691, 0.057478, 0.219691, 0.057100, 0.237287, 0.057100, 0.237287, 0.057100, 0.237287, 0.056485, 0.252788, 0.057100, 0.237287, 0.057100, 0.237287, 0.056485, 0.252788, 0.056485, 0.252788, 0.056485, 0.252788, 0.055634, 0.266193, 0.056485, 0.252788, 0.056485, 0.252788, 0.055634, 0.266193, 0.055634, 0.266193, 0.055634, 0.266193, 0.054586, 0.277006, 0.055634, 0.266193, 0.055634, 0.266193, 0.054586, 0.277006, 0.054586, 0.277006, 0.054586, 0.277006, 0.053381, 0.284730, 0.054586, 0.277006, 0.054586, 0.277006, 0.053381, 0.284730, 0.053381, 0.284730, 0.053381, 0.284730, 0.052019, 0.289364, 0.053381, 0.284730, 0.053381, 0.284730, 0.052019, 0.289364, 0.052019, 0.289364, 0.052019, 0.289364, 0.050499, 0.290909, 0.052019, 0.289364, 0.052019, 0.289364, 0.050499, 0.290909, 0.050499, 0.290909, 0.050499, 0.290909, 0.048780, 0.289418, 0.050499, 0.290909, 0.050499, 0.290909, 0.048780, 0.289418, 0.048780, 0.289418, 0.048780, 0.289418, 0.047242, 0.284943, 0.048780, 0.289418, 0.048780, 0.289418, 0.047242, 0.284943, 0.047242, 0.284943, 0.047242, 0.284943, 0.045884, 0.277486, 0.047242, 0.284943, 0.047242, 0.284943, 0.045884, 0.277486, 0.045884, 0.277486, 0.045884, 0.277486, 0.044707, 0.267045, 0.045884, 0.277486, 0.045884, 0.277486, 0.044707, 0.267045, 0.044707, 0.267045, 0.044707, 0.267045, 0.043735, 0.253888, 0.044707, 0.267045, 0.044707, 0.267045, 0.043735, 0.253888, 0.043735, 0.253888, 0.043735, 0.253888, 0.042994, 0.238281, 0.043735, 0.253888, 0.043735, 0.253888, 0.042994, 0.238281, 0.042994, 0.238281, 0.042994, 0.238281, 0.042484, 0.220224, 0.042994, 0.238281, 0.042994, 0.238281, 0.042484, 0.220224, 0.042484, 0.220224, 0.042484, 0.220224, 0.042204, 0.199716, 0.042484, 0.220224, 0.042484, 0.220224, 0.042204, 0.199716, 0.042204, 0.199716, 0.090636, 0.203977, 0.090636, 0.009091, 0.090636, 0.203977, 0.090636, 0.203977, 0.090636, 0.009091, 0.090636, 0.009091, 0.090636, 0.009091, 0.086312, 0.009091, 0.090636, 0.009091, 0.090636, 0.009091, 0.086312, 0.009091, 0.086312, 0.009091, 0.086312, 0.009091, 0.086312, 0.202273, 0.086312, 0.009091, 0.086312, 0.009091, 0.086312, 0.202273, 0.086312, 0.202273, 0.086312, 0.202273, 0.086221, 0.223105, 0.086312, 0.202273, 0.086312, 0.202273, 0.086221, 0.223105, 0.086221, 0.223105, 0.086221, 0.223105, 0.085948, 0.241140, 0.086221, 0.223105, 0.086221, 0.223105, 0.085948, 0.241140, 0.085948, 0.241140, 0.085948, 0.241140, 0.085492, 0.256379, 0.085948, 0.241140, 0.085948, 0.241140, 0.085492, 0.256379, 0.085492, 0.256379, 0.085492, 0.256379, 0.084855, 0.268821, 0.085492, 0.256379, 0.085492, 0.256379, 0.084855, 0.268821, 0.084855, 0.268821, 0.084855, 0.268821, 0.084035, 0.278485, 0.084855, 0.268821, 0.084855, 0.268821, 0.084035, 0.278485, 0.084035, 0.278485, 0.084035, 0.278485, 0.083034, 0.285387, 0.084035, 0.278485, 0.084035, 0.278485, 0.083034, 0.285387, 0.083034, 0.285387, 0.083034, 0.285387, 0.081850, 0.289529, 0.083034, 0.285387, 0.083034, 0.285387, 0.081850, 0.289529, 0.081850, 0.289529, 0.081850, 0.289529, 0.080484, 0.290909, 0.081850, 0.289529, 0.081850, 0.289529, 0.080484, 0.290909, 0.080484, 0.290909, 0.080484, 0.290909, 0.078826, 0.289244, 0.080484, 0.290909, 0.080484, 0.290909, 0.078826, 0.289244, 0.078826, 0.289244, 0.078826, 0.289244, 0.077353, 0.284251, 0.078826, 0.289244, 0.078826, 0.289244, 0.077353, 0.284251, 0.077353, 0.284251, 0.077353, 0.284251, 0.076065, 0.275928, 0.077353, 0.284251, 0.077353, 0.284251, 0.076065, 0.275928, 0.076065, 0.275928, 0.076065, 0.275928, 0.074962, 0.264276, 0.076065, 0.275928, 0.076065, 0.275928, 0.074962, 0.264276, 0.074962, 0.264276, 0.074962, 0.264276, 0.074078, 0.249747, 0.074962, 0.264276, 0.074962, 0.264276, 0.074078, 0.249747, 0.074078, 0.249747, 0.074078, 0.249747, 0.073446, 0.232795, 0.074078, 0.249747, 0.074078, 0.249747, 0.073446, 0.232795, 0.073446, 0.232795, 0.073446, 0.232795, 0.073067, 0.213419, 0.073446, 0.232795, 0.073446, 0.232795, 0.073067, 0.213419, 0.073067, 0.213419, 0.073067, 0.213419, 0.072941, 0.191619, 0.073067, 0.213419, 0.073067, 0.213419, 0.072941, 0.191619, 0.072941, 0.191619, 0.072941, 0.191619, 0.072941, 0.009091, 0.072941, 0.191619, 0.072941, 0.191619, 0.072941, 0.009091, 0.072941, 0.009091, 0.072941, 0.009091, 0.068594, 0.009091, 0.072941, 0.009091, 0.072941, 0.009091, 0.068594, 0.009091, 0.068594, 0.009091, 0.068594, 0.009091, 0.068594, 0.327273, 0.068594, 0.009091, 0.068594, 0.009091, 0.068594, 0.327273, 0.068594, 0.327273, 0.068594, 0.327273, 0.072941, 0.327273, 0.068594, 0.327273, 0.068594, 0.327273, 0.072941, 0.327273, 0.072941, 0.327273, 0.072941, 0.327273, 0.072941, 0.281818, 0.072941, 0.327273, 0.072941, 0.327273, 0.072941, 0.281818, 0.072941, 0.281818, 0.072941, 0.281818, 0.073751, 0.294660, 0.072941, 0.281818, 0.072941, 0.281818, 0.073751, 0.294660, 0.073751, 0.294660, 0.073751, 0.294660, 0.074630, 0.305771, 0.073751, 0.294660, 0.073751, 0.294660, 0.074630, 0.305771, 0.074630, 0.305771, 0.074630, 0.305771, 0.075578, 0.315150, 0.074630, 0.305771, 0.074630, 0.305771, 0.075578, 0.315150, 0.075578, 0.315150, 0.075578, 0.315150, 0.076595, 0.322798, 0.075578, 0.315150, 0.075578, 0.315150, 0.076595, 0.322798, 0.076595, 0.322798, 0.076595, 0.322798, 0.077687, 0.328733, 0.076595, 0.322798, 0.076595, 0.322798, 0.077687, 0.328733, 0.077687, 0.328733, 0.077687, 0.328733, 0.078860, 0.332972, 0.077687, 0.328733, 0.077687, 0.328733, 0.078860, 0.332972, 0.078860, 0.332972, 0.078860, 0.332972, 0.080113, 0.335516, 0.078860, 0.332972, 0.078860, 0.332972, 0.080113, 0.335516, 0.080113, 0.335516, 0.080113, 0.335516, 0.081448, 0.336364, 0.080113, 0.335516, 0.080113, 0.335516, 0.081448, 0.336364, 0.081448, 0.336364, 0.081448, 0.336364, 0.083577, 0.334264, 0.081448, 0.336364, 0.081448, 0.336364, 0.083577, 0.334264, 0.083577, 0.334264, 0.083577, 0.334264, 0.085431, 0.327965, 0.083577, 0.334264, 0.083577, 0.334264, 0.085431, 0.327965, 0.085431, 0.327965, 0.085431, 0.327965, 0.087008, 0.317467, 0.085431, 0.327965, 0.085431, 0.327965, 0.087008, 0.317467, 0.087008, 0.317467, 0.087008, 0.317467, 0.088309, 0.302770, 0.087008, 0.317467, 0.087008, 0.317467, 0.088309, 0.302770, 0.088309, 0.302770, 0.088309, 0.302770, 0.089327, 0.283998, 0.088309, 0.302770, 0.088309, 0.302770, 0.089327, 0.283998, 0.089327, 0.283998, 0.089327, 0.283998, 0.090054, 0.261275, 0.089327, 0.283998, 0.089327, 0.283998, 0.090054, 0.261275, 0.090054, 0.261275, 0.090054, 0.261275, 0.090490, 0.234601, 0.090054, 0.261275, 0.090054, 0.261275, 0.090490, 0.234601, 0.090490, 0.234601, 0.072941, 0.327273, 0.068594, 0.327273, 0.072941, 0.281818, 0.068594, 0.327273, 0.072941, 0.327273, 0.072941, 0.281818, 0.068594, 0.327273, 0.073446, 0.232795, 0.072941, 0.281818, 0.073446, 0.232795, 0.068594, 0.327273, 0.072941, 0.281818, 0.073446, 0.232795, 0.068594, 0.327273, 0.073067, 0.213419, 0.068594, 0.327273, 0.073446, 0.232795, 0.073067, 0.213419, 0.073067, 0.213419, 0.068594, 0.327273, 0.072941, 0.191619, 0.068594, 0.327273, 0.073067, 0.213419, 0.072941, 0.191619, 0.068594, 0.327273, 0.068594, 0.009091, 0.072941, 0.191619, 0.068594, 0.009091, 0.068594, 0.327273, 0.072941, 0.191619, 0.068594, 0.009091, 0.072941, 0.009091, 0.072941, 0.191619, 0.072941, 0.009091, 0.068594, 0.009091, 0.072941, 0.191619, 0.072941, 0.281818, 0.073446, 0.232795, 0.074078, 0.249747, 0.073446, 0.232795, 0.072941, 0.281818, 0.074078, 0.249747, 0.072941, 0.281818, 0.074078, 0.249747, 0.074962, 0.264276, 0.074078, 0.249747, 0.072941, 0.281818, 0.074962, 0.264276, 0.073751, 0.294660, 0.072941, 0.281818, 0.074962, 0.264276, 0.072941, 0.281818, 0.073751, 0.294660, 0.074962, 0.264276, 0.073751, 0.294660, 0.074962, 0.264276, 0.076065, 0.275928, 0.074962, 0.264276, 0.073751, 0.294660, 0.076065, 0.275928, 0.074630, 0.305771, 0.073751, 0.294660, 0.076065, 0.275928, 0.073751, 0.294660, 0.074630, 0.305771, 0.076065, 0.275928, 0.077353, 0.284251, 0.074630, 0.305771, 0.076065, 0.275928, 0.074630, 0.305771, 0.077353, 0.284251, 0.076065, 0.275928, 0.077353, 0.284251, 0.075578, 0.315150, 0.074630, 0.305771, 0.075578, 0.315150, 0.077353, 0.284251, 0.074630, 0.305771, 0.076595, 0.322798, 0.075578, 0.315150, 0.077353, 0.284251, 0.075578, 0.315150, 0.076595, 0.322798, 0.077353, 0.284251, 0.076595, 0.322798, 0.077353, 0.284251, 0.078826, 0.289244, 0.077353, 0.284251, 0.076595, 0.322798, 0.078826, 0.289244, 0.077687, 0.328733, 0.076595, 0.322798, 0.078826, 0.289244, 0.076595, 0.322798, 0.077687, 0.328733, 0.078826, 0.289244, 0.077687, 0.328733, 0.078826, 0.289244, 0.078860, 0.332972, 0.078826, 0.289244, 0.077687, 0.328733, 0.078860, 0.332972, 0.078860, 0.332972, 0.078826, 0.289244, 0.080484, 0.290909, 0.078826, 0.289244, 0.078860, 0.332972, 0.080484, 0.290909, 0.080113, 0.335516, 0.078860, 0.332972, 0.080484, 0.290909, 0.078860, 0.332972, 0.080113, 0.335516, 0.080484, 0.290909, 0.081448, 0.336364, 0.080113, 0.335516, 0.080484, 0.290909, 0.080113, 0.335516, 0.081448, 0.336364, 0.080484, 0.290909, 0.081448, 0.336364, 0.080484, 0.290909, 0.081850, 0.289529, 0.080484, 0.290909, 0.081448, 0.336364, 0.081850, 0.289529, 0.083577, 0.334264, 0.081448, 0.336364, 0.081850, 0.289529, 0.081448, 0.336364, 0.083577, 0.334264, 0.081850, 0.289529, 0.081850, 0.289529, 0.083034, 0.285387, 0.083577, 0.334264, 0.083034, 0.285387, 0.081850, 0.289529, 0.083577, 0.334264, 0.083577, 0.334264, 0.083034, 0.285387, 0.085431, 0.327965, 0.083034, 0.285387, 0.083577, 0.334264, 0.085431, 0.327965, 0.083034, 0.285387, 0.084035, 0.278485, 0.085431, 0.327965, 0.084035, 0.278485, 0.083034, 0.285387, 0.085431, 0.327965, 0.085431, 0.327965, 0.084035, 0.278485, 0.087008, 0.317467, 0.084035, 0.278485, 0.085431, 0.327965, 0.087008, 0.317467, 0.084035, 0.278485, 0.084855, 0.268821, 0.087008, 0.317467, 0.084855, 0.268821, 0.084035, 0.278485, 0.087008, 0.317467, 0.087008, 0.317467, 0.084855, 0.268821, 0.088309, 0.302770, 0.084855, 0.268821, 0.087008, 0.317467, 0.088309, 0.302770, 0.088309, 0.302770, 0.084855, 0.268821, 0.089327, 0.283998, 0.084855, 0.268821, 0.088309, 0.302770, 0.089327, 0.283998, 0.089327, 0.283998, 0.084855, 0.268821, 0.085492, 0.256379, 0.084855, 0.268821, 0.089327, 0.283998, 0.085492, 0.256379, 0.089327, 0.283998, 0.085492, 0.256379, 0.090054, 0.261275, 0.085492, 0.256379, 0.089327, 0.283998, 0.090054, 0.261275, 0.085492, 0.256379, 0.085948, 0.241140, 0.090054, 0.261275, 0.085948, 0.241140, 0.085492, 0.256379, 0.090054, 0.261275, 0.090054, 0.261275, 0.085948, 0.241140, 0.090490, 0.234601, 0.085948, 0.241140, 0.090054, 0.261275, 0.090490, 0.234601, 0.085948, 0.241140, 0.086221, 0.223105, 0.090490, 0.234601, 0.086221, 0.223105, 0.085948, 0.241140, 0.090490, 0.234601, 0.090490, 0.234601, 0.086221, 0.223105, 0.090636, 0.203977, 0.086221, 0.223105, 0.090490, 0.234601, 0.090636, 0.203977, 0.090636, 0.203977, 0.086221, 0.223105, 0.086312, 0.202273, 0.086221, 0.223105, 0.090636, 0.203977, 0.086312, 0.202273, 0.086312, 0.202273, 0.090636, 0.009091, 0.090636, 0.203977, 0.090636, 0.009091, 0.086312, 0.202273, 0.090636, 0.203977, 0.086312, 0.202273, 0.086312, 0.009091, 0.090636, 0.009091, 0.086312, 0.009091, 0.086312, 0.202273, 0.090636, 0.009091, 0.122101, 0.182955, 0.122101, 0.154545, 0.122101, 0.182955, 0.122101, 0.182955, 0.122101, 0.154545, 0.122101, 0.154545, 0.122101, 0.154545, 0.102221, 0.154545, 0.122101, 0.154545, 0.122101, 0.154545, 0.102221, 0.154545, 0.102221, 0.154545, 0.102221, 0.154545, 0.102495, 0.129497, 0.102221, 0.154545, 0.102221, 0.154545, 0.102495, 0.129497, 0.102495, 0.129497, 0.102495, 0.129497, 0.103034, 0.107617, 0.102495, 0.129497, 0.102495, 0.129497, 0.103034, 0.107617, 0.103034, 0.107617, 0.103034, 0.107617, 0.103840, 0.088907, 0.103034, 0.107617, 0.103034, 0.107617, 0.103840, 0.088907, 0.103840, 0.088907, 0.103840, 0.088907, 0.104911, 0.073366, 0.103840, 0.088907, 0.103840, 0.088907, 0.104911, 0.073366, 0.104911, 0.073366, 0.104911, 0.073366, 0.106234, 0.061155, 0.104911, 0.073366, 0.104911, 0.073366, 0.106234, 0.061155, 0.106234, 0.061155, 0.106234, 0.061155, 0.107793, 0.052433, 0.106234, 0.061155, 0.106234, 0.061155, 0.107793, 0.052433, 0.107793, 0.052433, 0.107793, 0.052433, 0.109588, 0.047199, 0.107793, 0.052433, 0.107793, 0.052433, 0.109588, 0.047199, 0.109588, 0.047199, 0.109588, 0.047199, 0.111620, 0.045455, 0.109588, 0.047199, 0.109588, 0.047199, 0.111620, 0.045455, 0.111620, 0.045455, 0.111620, 0.045455, 0.112856, 0.045881, 0.111620, 0.045455, 0.111620, 0.045455, 0.112856, 0.045881, 0.112856, 0.045881, 0.112856, 0.045881, 0.114073, 0.047159, 0.112856, 0.045881, 0.112856, 0.045881, 0.114073, 0.047159, 0.114073, 0.047159, 0.114073, 0.047159, 0.115271, 0.049290, 0.114073, 0.047159, 0.114073, 0.047159, 0.115271, 0.049290, 0.115271, 0.049290, 0.115271, 0.049290, 0.116449, 0.052273, 0.115271, 0.049290, 0.115271, 0.049290, 0.116449, 0.052273, 0.116449, 0.052273, 0.116449, 0.052273, 0.117616, 0.056108, 0.116449, 0.052273, 0.116449, 0.052273, 0.117616, 0.056108, 0.117616, 0.056108, 0.117616, 0.056108, 0.118779, 0.060795, 0.117616, 0.056108, 0.117616, 0.056108, 0.118779, 0.060795, 0.118779, 0.060795, 0.118779, 0.060795, 0.119937, 0.066335, 0.118779, 0.060795, 0.118779, 0.060795, 0.119937, 0.066335, 0.119937, 0.066335, 0.119937, 0.066335, 0.121090, 0.072727, 0.119937, 0.066335, 0.119937, 0.066335, 0.121090, 0.072727, 0.121090, 0.072727, 0.121090, 0.072727, 0.121090, 0.027273, 0.121090, 0.072727, 0.121090, 0.072727, 0.121090, 0.027273, 0.121090, 0.027273, 0.121090, 0.027273, 0.119920, 0.020996, 0.121090, 0.027273, 0.121090, 0.027273, 0.119920, 0.020996, 0.119920, 0.020996, 0.119920, 0.020996, 0.118735, 0.015518, 0.119920, 0.020996, 0.119920, 0.020996, 0.118735, 0.015518, 0.118735, 0.015518, 0.118735, 0.015518, 0.117535, 0.010840, 0.118735, 0.015518, 0.118735, 0.015518, 0.117535, 0.010840, 0.117535, 0.010840, 0.117535, 0.010840, 0.116320, 0.006960, 0.117535, 0.010840, 0.117535, 0.010840, 0.116320, 0.006960, 0.116320, 0.006960, 0.116320, 0.006960, 0.115094, 0.003915, 0.116320, 0.006960, 0.116320, 0.006960, 0.115094, 0.003915, 0.115094, 0.003915, 0.115094, 0.003915, 0.113859, 0.001740, 0.115094, 0.003915, 0.115094, 0.003915, 0.113859, 0.001740, 0.113859, 0.001740, 0.113859, 0.001740, 0.112615, 0.000435, 0.113859, 0.001740, 0.113859, 0.001740, 0.112615, 0.000435, 0.112615, 0.000435, 0.112615, 0.000435, 0.111362, 0, 0.112615, 0.000435, 0.112615, 0.000435, 0.111362, 0, 0.111362, 0, 0.111362, 0, 0.108377, 0.002788, 0.111362, 0, 0.111362, 0, 0.108377, 0.002788, 0.108377, 0.002788, 0.108377, 0.002788, 0.105719, 0.011151, 0.108377, 0.002788, 0.108377, 0.002788, 0.105719, 0.011151, 0.105719, 0.011151, 0.105719, 0.011151, 0.103389, 0.025089, 0.105719, 0.011151, 0.105719, 0.011151, 0.103389, 0.025089, 0.103389, 0.025089, 0.103389, 0.025089, 0.101386, 0.044602, 0.103389, 0.025089, 0.103389, 0.025089, 0.101386, 0.044602, 0.101386, 0.044602, 0.101386, 0.044602, 0.099778, 0.068874, 0.101386, 0.044602, 0.101386, 0.044602, 0.099778, 0.068874, 0.099778, 0.068874, 0.099778, 0.068874, 0.098628, 0.097088, 0.099778, 0.068874, 0.099778, 0.068874, 0.098628, 0.097088, 0.098628, 0.097088, 0.098628, 0.097088, 0.097939, 0.129244, 0.098628, 0.097088, 0.098628, 0.097088, 0.097939, 0.129244, 0.097939, 0.129244, 0.097939, 0.129244, 0.097709, 0.165341, 0.097939, 0.129244, 0.097939, 0.129244, 0.097709, 0.165341, 0.097709, 0.165341, 0.097709, 0.165341, 0.097927, 0.202654, 0.097709, 0.165341, 0.097709, 0.165341, 0.097927, 0.202654, 0.097927, 0.202654, 0.097927, 0.202654, 0.098581, 0.235902, 0.097927, 0.202654, 0.097927, 0.202654, 0.098581, 0.235902, 0.098581, 0.235902, 0.098581, 0.235902, 0.099672, 0.265083, 0.098581, 0.235902, 0.098581, 0.235902, 0.099672, 0.265083, 0.099672, 0.265083, 0.099672, 0.265083, 0.101198, 0.290199, 0.099672, 0.265083, 0.099672, 0.265083, 0.101198, 0.290199, 0.101198, 0.290199, 0.101198, 0.290199, 0.103095, 0.310396, 0.101198, 0.290199, 0.101198, 0.290199, 0.103095, 0.310396, 0.103095, 0.310396, 0.103095, 0.310396, 0.105296, 0.324822, 0.103095, 0.310396, 0.103095, 0.310396, 0.105296, 0.324822, 0.105296, 0.324822, 0.105296, 0.324822, 0.107801, 0.333478, 0.105296, 0.324822, 0.105296, 0.324822, 0.107801, 0.333478, 0.107801, 0.333478, 0.107801, 0.333478, 0.110610, 0.336364, 0.107801, 0.333478, 0.107801, 0.333478, 0.110610, 0.336364, 0.110610, 0.336364, 0.110610, 0.336364, 0.113126, 0.333785, 0.110610, 0.336364, 0.110610, 0.336364, 0.113126, 0.333785, 0.113126, 0.333785, 0.113126, 0.333785, 0.115365, 0.326048, 0.113126, 0.333785, 0.113126, 0.333785, 0.115365, 0.326048, 0.115365, 0.326048, 0.115365, 0.326048, 0.117327, 0.313153, 0.115365, 0.326048, 0.115365, 0.326048, 0.117327, 0.313153, 0.117327, 0.313153, 0.117327, 0.313153, 0.119011, 0.295099, 0.117327, 0.313153, 0.117327, 0.313153, 0.119011, 0.295099, 0.119011, 0.295099, 0.119011, 0.295099, 0.120363, 0.272616, 0.119011, 0.295099, 0.119011, 0.295099, 0.120363, 0.272616, 0.120363, 0.272616, 0.120363, 0.272616, 0.121328, 0.246431, 0.120363, 0.272616, 0.120363, 0.272616, 0.121328, 0.246431, 0.121328, 0.246431, 0.121328, 0.246431, 0.121908, 0.216544, 0.121328, 0.246431, 0.121328, 0.246431, 0.121908, 0.216544, 0.121908, 0.216544, 0.097927, 0.202654, 0.097709, 0.165341, 0.102362, 0.199716, 0.097709, 0.165341, 0.097927, 0.202654, 0.102362, 0.199716, 0.098581, 0.235902, 0.097927, 0.202654, 0.102362, 0.199716, 0.097927, 0.202654, 0.098581, 0.235902, 0.102362, 0.199716, 0.098581, 0.235902, 0.102362, 0.199716, 0.102641, 0.220224, 0.102362, 0.199716, 0.098581, 0.235902, 0.102641, 0.220224, 0.103152, 0.238281, 0.098581, 0.235902, 0.102641, 0.220224, 0.098581, 0.235902, 0.103152, 0.238281, 0.102641, 0.220224, 0.103152, 0.238281, 0.099672, 0.265083, 0.098581, 0.235902, 0.099672, 0.265083, 0.103152, 0.238281, 0.098581, 0.235902, 0.099672, 0.265083, 0.103152, 0.238281, 0.103893, 0.253888, 0.103152, 0.238281, 0.099672, 0.265083, 0.103893, 0.253888, 0.101198, 0.290199, 0.099672, 0.265083, 0.103893, 0.253888, 0.099672, 0.265083, 0.101198, 0.290199, 0.103893, 0.253888, 0.101198, 0.290199, 0.103893, 0.253888, 0.104864, 0.267045, 0.103893, 0.253888, 0.101198, 0.290199, 0.104864, 0.267045, 0.103095, 0.310396, 0.101198, 0.290199, 0.104864, 0.267045, 0.101198, 0.290199, 0.103095, 0.310396, 0.104864, 0.267045, 0.103095, 0.310396, 0.104864, 0.267045, 0.106041, 0.277486, 0.104864, 0.267045, 0.103095, 0.310396, 0.106041, 0.277486, 0.105296, 0.324822, 0.103095, 0.310396, 0.106041, 0.277486, 0.103095, 0.310396, 0.105296, 0.324822, 0.106041, 0.277486, 0.105296, 0.324822, 0.106041, 0.277486, 0.107399, 0.284943, 0.106041, 0.277486, 0.105296, 0.324822, 0.107399, 0.284943, 0.105296, 0.324822, 0.107399, 0.284943, 0.107801, 0.333478, 0.107399, 0.284943, 0.105296, 0.324822, 0.107801, 0.333478, 0.107801, 0.333478, 0.107399, 0.284943, 0.108938, 0.289418, 0.107399, 0.284943, 0.107801, 0.333478, 0.108938, 0.289418, 0.107801, 0.333478, 0.108938, 0.289418, 0.110610, 0.336364, 0.108938, 0.289418, 0.107801, 0.333478, 0.110610, 0.336364, 0.110610, 0.336364, 0.108938, 0.289418, 0.110657, 0.290909, 0.108938, 0.289418, 0.110610, 0.336364, 0.110657, 0.290909, 0.110610, 0.336364, 0.110657, 0.290909, 0.112176, 0.289364, 0.110657, 0.290909, 0.110610, 0.336364, 0.112176, 0.289364, 0.110610, 0.336364, 0.112176, 0.289364, 0.113126, 0.333785, 0.112176, 0.289364, 0.110610, 0.336364, 0.113126, 0.333785, 0.113126, 0.333785, 0.112176, 0.289364, 0.113538, 0.284730, 0.112176, 0.289364, 0.113126, 0.333785, 0.113538, 0.284730, 0.113126, 0.333785, 0.113538, 0.284730, 0.115365, 0.326048, 0.113538, 0.284730, 0.113126, 0.333785, 0.115365, 0.326048, 0.113538, 0.284730, 0.114743, 0.277006, 0.115365, 0.326048, 0.114743, 0.277006, 0.113538, 0.284730, 0.115365, 0.326048, 0.115365, 0.326048, 0.114743, 0.277006, 0.117327, 0.313153, 0.114743, 0.277006, 0.115365, 0.326048, 0.117327, 0.313153, 0.114743, 0.277006, 0.115791, 0.266193, 0.117327, 0.313153, 0.115791, 0.266193, 0.114743, 0.277006, 0.117327, 0.313153, 0.117327, 0.313153, 0.115791, 0.266193, 0.119011, 0.295099, 0.115791, 0.266193, 0.117327, 0.313153, 0.119011, 0.295099, 0.115791, 0.266193, 0.116642, 0.252788, 0.119011, 0.295099, 0.116642, 0.252788, 0.115791, 0.266193, 0.119011, 0.295099, 0.119011, 0.295099, 0.116642, 0.252788, 0.120363, 0.272616, 0.116642, 0.252788, 0.119011, 0.295099, 0.120363, 0.272616, 0.116642, 0.252788, 0.117257, 0.237287, 0.120363, 0.272616, 0.117257, 0.237287, 0.116642, 0.252788, 0.120363, 0.272616, 0.120363, 0.272616, 0.117257, 0.237287, 0.121328, 0.246431, 0.117257, 0.237287, 0.120363, 0.272616, 0.121328, 0.246431, 0.117257, 0.237287, 0.117635, 0.219691, 0.121328, 0.246431, 0.117635, 0.219691, 0.117257, 0.237287, 0.121328, 0.246431, 0.121328, 0.246431, 0.117635, 0.219691, 0.121908, 0.216544, 0.117635, 0.219691, 0.121328, 0.246431, 0.121908, 0.216544, 0.117635, 0.219691, 0.117777, 0.200000, 0.121908, 0.216544, 0.117777, 0.200000, 0.117635, 0.219691, 0.121908, 0.216544, 0.121908, 0.216544, 0.117777, 0.200000, 0.122101, 0.182955, 0.117777, 0.200000, 0.121908, 0.216544, 0.122101, 0.182955, 0.122101, 0.154545, 0.122101, 0.182955, 0.117777, 0.200000, 0.122101, 0.182955, 0.122101, 0.154545, 0.117777, 0.200000, 0.117777, 0.200000, 0.102221, 0.154545, 0.122101, 0.154545, 0.102221, 0.154545, 0.117777, 0.200000, 0.122101, 0.154545, 0.117777, 0.200000, 0.102362, 0.199716, 0.102221, 0.154545, 0.102362, 0.199716, 0.117777, 0.200000, 0.102221, 0.154545, 0.097709, 0.165341, 0.102221, 0.154545, 0.102362, 0.199716, 0.102221, 0.154545, 0.097709, 0.165341, 0.102362, 0.199716, 0.097709, 0.165341, 0.097939, 0.129244, 0.102221, 0.154545, 0.097939, 0.129244, 0.097709, 0.165341, 0.102221, 0.154545, 0.102221, 0.154545, 0.097939, 0.129244, 0.102495, 0.129497, 0.097939, 0.129244, 0.102221, 0.154545, 0.102495, 0.129497, 0.097939, 0.129244, 0.098628, 0.097088, 0.102495, 0.129497, 0.098628, 0.097088, 0.097939, 0.129244, 0.102495, 0.129497, 0.102495, 0.129497, 0.098628, 0.097088, 0.103034, 0.107617, 0.098628, 0.097088, 0.102495, 0.129497, 0.103034, 0.107617, 0.098628, 0.097088, 0.099778, 0.068874, 0.103034, 0.107617, 0.099778, 0.068874, 0.098628, 0.097088, 0.103034, 0.107617, 0.103034, 0.107617, 0.099778, 0.068874, 0.103840, 0.088907, 0.099778, 0.068874, 0.103034, 0.107617, 0.103840, 0.088907, 0.099778, 0.068874, 0.101386, 0.044602, 0.103840, 0.088907, 0.101386, 0.044602, 0.099778, 0.068874, 0.103840, 0.088907, 0.103840, 0.088907, 0.101386, 0.044602, 0.104911, 0.073366, 0.101386, 0.044602, 0.103840, 0.088907, 0.104911, 0.073366, 0.101386, 0.044602, 0.103389, 0.025089, 0.104911, 0.073366, 0.103389, 0.025089, 0.101386, 0.044602, 0.104911, 0.073366, 0.104911, 0.073366, 0.103389, 0.025089, 0.106234, 0.061155, 0.103389, 0.025089, 0.104911, 0.073366, 0.106234, 0.061155, 0.106234, 0.061155, 0.103389, 0.025089, 0.105719, 0.011151, 0.103389, 0.025089, 0.106234, 0.061155, 0.105719, 0.011151, 0.107793, 0.052433, 0.106234, 0.061155, 0.105719, 0.011151, 0.106234, 0.061155, 0.107793, 0.052433, 0.105719, 0.011151, 0.107793, 0.052433, 0.105719, 0.011151, 0.108377, 0.002788, 0.105719, 0.011151, 0.107793, 0.052433, 0.108377, 0.002788, 0.109588, 0.047199, 0.107793, 0.052433, 0.108377, 0.002788, 0.107793, 0.052433, 0.109588, 0.047199, 0.108377, 0.002788, 0.109588, 0.047199, 0.108377, 0.002788, 0.111362, 0, 0.108377, 0.002788, 0.109588, 0.047199, 0.111362, 0, 0.111620, 0.045455, 0.109588, 0.047199, 0.111362, 0, 0.109588, 0.047199, 0.111620, 0.045455, 0.111362, 0, 0.111620, 0.045455, 0.111362, 0, 0.112615, 0.000435, 0.111362, 0, 0.111620, 0.045455, 0.112615, 0.000435, 0.111620, 0.045455, 0.112615, 0.000435, 0.112856, 0.045881, 0.112615, 0.000435, 0.111620, 0.045455, 0.112856, 0.045881, 0.112856, 0.045881, 0.112615, 0.000435, 0.113859, 0.001740, 0.112615, 0.000435, 0.112856, 0.045881, 0.113859, 0.001740, 0.112856, 0.045881, 0.113859, 0.001740, 0.114073, 0.047159, 0.113859, 0.001740, 0.112856, 0.045881, 0.114073, 0.047159, 0.114073, 0.047159, 0.113859, 0.001740, 0.115094, 0.003915, 0.113859, 0.001740, 0.114073, 0.047159, 0.115094, 0.003915, 0.114073, 0.047159, 0.115094, 0.003915, 0.115271, 0.049290, 0.115094, 0.003915, 0.114073, 0.047159, 0.115271, 0.049290, 0.115271, 0.049290, 0.115094, 0.003915, 0.116320, 0.006960, 0.115094, 0.003915, 0.115271, 0.049290, 0.116320, 0.006960, 0.115271, 0.049290, 0.116320, 0.006960, 0.116449, 0.052273, 0.116320, 0.006960, 0.115271, 0.049290, 0.116449, 0.052273, 0.116449, 0.052273, 0.116320, 0.006960, 0.117535, 0.010840, 0.116320, 0.006960, 0.116449, 0.052273, 0.117535, 0.010840, 0.117616, 0.056108, 0.116449, 0.052273, 0.117535, 0.010840, 0.116449, 0.052273, 0.117616, 0.056108, 0.117535, 0.010840, 0.118735, 0.015518, 0.117616, 0.056108, 0.117535, 0.010840, 0.117616, 0.056108, 0.118735, 0.015518, 0.117535, 0.010840, 0.117616, 0.056108, 0.118735, 0.015518, 0.119920, 0.020996, 0.118735, 0.015518, 0.117616, 0.056108, 0.119920, 0.020996, 0.118779, 0.060795, 0.117616, 0.056108, 0.119920, 0.020996, 0.117616, 0.056108, 0.118779, 0.060795, 0.119920, 0.020996, 0.118779, 0.060795, 0.119920, 0.020996, 0.121090, 0.027273, 0.119920, 0.020996, 0.118779, 0.060795, 0.121090, 0.027273, 0.119937, 0.066335, 0.118779, 0.060795, 0.121090, 0.027273, 0.118779, 0.060795, 0.119937, 0.066335, 0.121090, 0.027273, 0.119937, 0.066335, 0.121090, 0.027273, 0.121090, 0.072727, 0.121090, 0.027273, 0.119937, 0.066335, 0.121090, 0.072727, 0.117777, 0.200000, 0.117635, 0.219691, 0.117777, 0.200000, 0.117777, 0.200000, 0.117635, 0.219691, 0.117635, 0.219691, 0.117635, 0.219691, 0.117257, 0.237287, 0.117635, 0.219691, 0.117635, 0.219691, 0.117257, 0.237287, 0.117257, 0.237287, 0.117257, 0.237287, 0.116642, 0.252788, 0.117257, 0.237287, 0.117257, 0.237287, 0.116642, 0.252788, 0.116642, 0.252788, 0.116642, 0.252788, 0.115791, 0.266193, 0.116642, 0.252788, 0.116642, 0.252788, 0.115791, 0.266193, 0.115791, 0.266193, 0.115791, 0.266193, 0.114743, 0.277006, 0.115791, 0.266193, 0.115791, 0.266193, 0.114743, 0.277006, 0.114743, 0.277006, 0.114743, 0.277006, 0.113538, 0.284730, 0.114743, 0.277006, 0.114743, 0.277006, 0.113538, 0.284730, 0.113538, 0.284730, 0.113538, 0.284730, 0.112176, 0.289364, 0.113538, 0.284730, 0.113538, 0.284730, 0.112176, 0.289364, 0.112176, 0.289364, 0.112176, 0.289364, 0.110657, 0.290909, 0.112176, 0.289364, 0.112176, 0.289364, 0.110657, 0.290909, 0.110657, 0.290909, 0.110657, 0.290909, 0.108938, 0.289418, 0.110657, 0.290909, 0.110657, 0.290909, 0.108938, 0.289418, 0.108938, 0.289418, 0.108938, 0.289418, 0.107399, 0.284943, 0.108938, 0.289418, 0.108938, 0.289418, 0.107399, 0.284943, 0.107399, 0.284943, 0.107399, 0.284943, 0.106041, 0.277486, 0.107399, 0.284943, 0.107399, 0.284943, 0.106041, 0.277486, 0.106041, 0.277486, 0.106041, 0.277486, 0.104864, 0.267045, 0.106041, 0.277486, 0.106041, 0.277486, 0.104864, 0.267045, 0.104864, 0.267045, 0.104864, 0.267045, 0.103893, 0.253888, 0.104864, 0.267045, 0.104864, 0.267045, 0.103893, 0.253888, 0.103893, 0.253888, 0.103893, 0.253888, 0.103152, 0.238281, 0.103893, 0.253888, 0.103893, 0.253888, 0.103152, 0.238281, 0.103152, 0.238281, 0.103152, 0.238281, 0.102641, 0.220224, 0.103152, 0.238281, 0.103152, 0.238281, 0.102641, 0.220224, 0.102641, 0.220224, 0.102641, 0.220224, 0.102362, 0.199716, 0.102641, 0.220224, 0.102641, 0.220224, 0.102362, 0.199716, 0.102362, 0.199716, 0.144166, 0.281818, 0.143794, 0.284007, 0.144166, 0.281818, 0.144166, 0.281818, 0.143794, 0.284007, 0.143794, 0.284007, 0.143794, 0.284007, 0.143406, 0.285884, 0.143794, 0.284007, 0.143794, 0.284007, 0.143406, 0.285884, 0.143406, 0.285884, 0.143406, 0.285884, 0.143001, 0.287451, 0.143406, 0.285884, 0.143406, 0.285884, 0.143001, 0.287451, 0.143001, 0.287451, 0.143001, 0.287451, 0.142580, 0.288707, 0.143001, 0.287451, 0.143001, 0.287451, 0.142580, 0.288707, 0.142580, 0.288707, 0.142580, 0.288707, 0.142140, 0.289671, 0.142580, 0.288707, 0.142580, 0.288707, 0.142140, 0.289671, 0.142140, 0.289671, 0.142140, 0.289671, 0.141678, 0.290359, 0.142140, 0.289671, 0.142140, 0.289671, 0.141678, 0.290359, 0.141678, 0.290359, 0.141678, 0.290359, 0.141194, 0.290771, 0.141678, 0.290359, 0.141678, 0.290359, 0.141194, 0.290771, 0.141194, 0.290771, 0.141194, 0.290771, 0.140689, 0.290909, 0.141194, 0.290771, 0.141194, 0.290771, 0.140689, 0.290909, 0.140689, 0.290909, 0.140689, 0.290909, 0.138962, 0.289102, 0.140689, 0.290909, 0.140689, 0.290909, 0.138962, 0.289102, 0.138962, 0.289102, 0.138962, 0.289102, 0.137449, 0.283683, 0.138962, 0.289102, 0.138962, 0.289102, 0.137449, 0.283683, 0.137449, 0.283683, 0.137449, 0.283683, 0.136148, 0.274649, 0.137449, 0.283683, 0.137449, 0.283683, 0.136148, 0.274649, 0.136148, 0.274649, 0.136148, 0.274649, 0.135061, 0.262003, 0.136148, 0.274649, 0.136148, 0.274649, 0.135061, 0.262003, 0.135061, 0.262003, 0.135061, 0.262003, 0.134202, 0.245983, 0.135061, 0.262003, 0.135061, 0.262003, 0.134202, 0.245983, 0.134202, 0.245983, 0.134202, 0.245983, 0.133589, 0.226829, 0.134202, 0.245983, 0.134202, 0.245983, 0.133589, 0.226829, 0.133589, 0.226829, 0.133589, 0.226829, 0.133221, 0.204541, 0.133589, 0.226829, 0.133589, 0.226829, 0.133221, 0.204541, 0.133221, 0.204541, 0.133221, 0.204541, 0.133098, 0.179119, 0.133221, 0.204541, 0.133221, 0.204541, 0.133098, 0.179119, 0.133098, 0.179119, 0.133098, 0.179119, 0.133098, 0.009091, 0.133098, 0.179119, 0.133098, 0.179119, 0.133098, 0.009091, 0.133098, 0.009091, 0.133098, 0.009091, 0.128751, 0.009091, 0.133098, 0.009091, 0.133098, 0.009091, 0.128751, 0.009091, 0.128751, 0.009091, 0.128751, 0.009091, 0.128751, 0.327273, 0.128751, 0.009091, 0.128751, 0.009091, 0.128751, 0.327273, 0.128751, 0.327273, 0.128751, 0.327273, 0.133098, 0.327273, 0.128751, 0.327273, 0.128751, 0.327273, 0.133098, 0.327273, 0.133098, 0.327273, 0.133098, 0.327273, 0.133098, 0.281818, 0.133098, 0.327273, 0.133098, 0.327273, 0.133098, 0.281818, 0.133098, 0.281818, 0.133098, 0.281818, 0.133831, 0.294775, 0.133098, 0.281818, 0.133098, 0.281818, 0.133831, 0.294775, 0.133831, 0.294775, 0.133831, 0.294775, 0.134667, 0.305948, 0.133831, 0.294775, 0.133831, 0.294775, 0.134667, 0.305948, 0.134667, 0.305948, 0.134667, 0.305948, 0.135605, 0.315336, 0.134667, 0.305948, 0.134667, 0.305948, 0.135605, 0.315336, 0.135605, 0.315336, 0.135605, 0.315336, 0.136647, 0.322940, 0.135605, 0.315336, 0.135605, 0.315336, 0.136647, 0.322940, 0.136647, 0.322940, 0.136647, 0.322940, 0.137798, 0.328813, 0.136647, 0.322940, 0.136647, 0.322940, 0.137798, 0.328813, 0.137798, 0.328813, 0.137798, 0.328813, 0.139067, 0.333008, 0.137798, 0.328813, 0.137798, 0.328813, 0.139067, 0.333008, 0.139067, 0.333008, 0.139067, 0.333008, 0.140454, 0.335525, 0.139067, 0.333008, 0.139067, 0.333008, 0.140454, 0.335525, 0.140454, 0.335525, 0.140454, 0.335525, 0.141957, 0.336364, 0.140454, 0.335525, 0.140454, 0.335525, 0.141957, 0.336364, 0.141957, 0.336364, 0.141957, 0.336364, 0.142187, 0.336364, 0.141957, 0.336364, 0.141957, 0.336364, 0.142187, 0.336364, 0.142187, 0.336364, 0.142187, 0.336364, 0.142427, 0.336364, 0.142187, 0.336364, 0.142187, 0.336364, 0.142427, 0.336364, 0.142427, 0.336364, 0.142427, 0.336364, 0.142680, 0.336364, 0.142427, 0.336364, 0.142427, 0.336364, 0.142680, 0.336364, 0.142680, 0.336364, 0.142680, 0.336364, 0.142944, 0.336364, 0.142680, 0.336364, 0.142680, 0.336364, 0.142944, 0.336364, 0.142944, 0.336364, 0.142944, 0.336364, 0.143222, 0.336364, 0.142944, 0.336364, 0.142944, 0.336364, 0.143222, 0.336364, 0.143222, 0.336364, 0.143222, 0.336364, 0.143514, 0.336364, 0.143222, 0.336364, 0.143222, 0.336364, 0.143514, 0.336364, 0.143514, 0.336364, 0.143514, 0.336364, 0.143821, 0.336364, 0.143514, 0.336364, 0.143514, 0.336364, 0.143821, 0.336364, 0.143821, 0.336364, 0.143821, 0.336364, 0.144143, 0.336364, 0.143821, 0.336364, 0.143821, 0.336364, 0.144143, 0.336364, 0.144143, 0.336364, 0.133098, 0.327273, 0.128751, 0.327273, 0.133098, 0.281818, 0.128751, 0.327273, 0.133098, 0.327273, 0.133098, 0.281818, 0.133589, 0.226829, 0.133098, 0.281818, 0.128751, 0.327273, 0.133098, 0.281818, 0.133589, 0.226829, 0.128751, 0.327273, 0.133098, 0.281818, 0.133589, 0.226829, 0.134202, 0.245983, 0.133589, 0.226829, 0.133098, 0.281818, 0.134202, 0.245983, 0.133098, 0.281818, 0.134202, 0.245983, 0.135061, 0.262003, 0.134202, 0.245983, 0.133098, 0.281818, 0.135061, 0.262003, 0.133831, 0.294775, 0.133098, 0.281818, 0.135061, 0.262003, 0.133098, 0.281818, 0.133831, 0.294775, 0.135061, 0.262003, 0.133831, 0.294775, 0.135061, 0.262003, 0.136148, 0.274649, 0.135061, 0.262003, 0.133831, 0.294775, 0.136148, 0.274649, 0.134667, 0.305948, 0.133831, 0.294775, 0.136148, 0.274649, 0.133831, 0.294775, 0.134667, 0.305948, 0.136148, 0.274649, 0.134667, 0.305948, 0.136148, 0.274649, 0.137449, 0.283683, 0.136148, 0.274649, 0.134667, 0.305948, 0.137449, 0.283683, 0.135605, 0.315336, 0.134667, 0.305948, 0.137449, 0.283683, 0.134667, 0.305948, 0.135605, 0.315336, 0.137449, 0.283683, 0.136647, 0.322940, 0.135605, 0.315336, 0.137449, 0.283683, 0.135605, 0.315336, 0.136647, 0.322940, 0.137449, 0.283683, 0.136647, 0.322940, 0.137449, 0.283683, 0.138962, 0.289102, 0.137449, 0.283683, 0.136647, 0.322940, 0.138962, 0.289102, 0.137798, 0.328813, 0.136647, 0.322940, 0.138962, 0.289102, 0.136647, 0.322940, 0.137798, 0.328813, 0.138962, 0.289102, 0.137798, 0.328813, 0.138962, 0.289102, 0.139067, 0.333008, 0.138962, 0.289102, 0.137798, 0.328813, 0.139067, 0.333008, 0.139067, 0.333008, 0.138962, 0.289102, 0.140689, 0.290909, 0.138962, 0.289102, 0.139067, 0.333008, 0.140689, 0.290909, 0.140454, 0.335525, 0.139067, 0.333008, 0.140689, 0.290909, 0.139067, 0.333008, 0.140454, 0.335525, 0.140689, 0.290909, 0.140454, 0.335525, 0.140689, 0.290909, 0.141194, 0.290771, 0.140689, 0.290909, 0.140454, 0.335525, 0.141194, 0.290771, 0.141957, 0.336364, 0.140454, 0.335525, 0.141194, 0.290771, 0.140454, 0.335525, 0.141957, 0.336364, 0.141194, 0.290771, 0.141194, 0.290771, 0.141678, 0.290359, 0.141957, 0.336364, 0.141678, 0.290359, 0.141194, 0.290771, 0.141957, 0.336364, 0.142187, 0.336364, 0.141957, 0.336364, 0.141678, 0.290359, 0.141957, 0.336364, 0.142187, 0.336364, 0.141678, 0.290359, 0.142187, 0.336364, 0.141678, 0.290359, 0.142140, 0.289671, 0.141678, 0.290359, 0.142187, 0.336364, 0.142140, 0.289671, 0.142427, 0.336364, 0.142187, 0.336364, 0.142140, 0.289671, 0.142187, 0.336364, 0.142427, 0.336364, 0.142140, 0.289671, 0.142140, 0.289671, 0.142680, 0.336364, 0.142427, 0.336364, 0.142680, 0.336364, 0.142140, 0.289671, 0.142427, 0.336364, 0.142680, 0.336364, 0.142140, 0.289671, 0.142580, 0.288707, 0.142140, 0.289671, 0.142680, 0.336364, 0.142580, 0.288707, 0.142944, 0.336364, 0.142680, 0.336364, 0.142580, 0.288707, 0.142680, 0.336364, 0.142944, 0.336364, 0.142580, 0.288707, 0.142580, 0.288707, 0.143222, 0.336364, 0.142944, 0.336364, 0.143222, 0.336364, 0.142580, 0.288707, 0.142944, 0.336364, 0.143222, 0.336364, 0.142580, 0.288707, 0.143001, 0.287451, 0.142580, 0.288707, 0.143222, 0.336364, 0.143001, 0.287451, 0.143514, 0.336364, 0.143222, 0.336364, 0.143001, 0.287451, 0.143222, 0.336364, 0.143514, 0.336364, 0.143001, 0.287451, 0.143821, 0.336364, 0.143514, 0.336364, 0.143001, 0.287451, 0.143514, 0.336364, 0.143821, 0.336364, 0.143001, 0.287451, 0.143406, 0.285884, 0.143821, 0.336364, 0.143001, 0.287451, 0.143821, 0.336364, 0.143406, 0.285884, 0.143001, 0.287451, 0.144143, 0.336364, 0.143821, 0.336364, 0.143406, 0.285884, 0.143821, 0.336364, 0.144143, 0.336364, 0.143406, 0.285884, 0.143794, 0.284007, 0.144143, 0.336364, 0.143406, 0.285884, 0.144143, 0.336364, 0.143794, 0.284007, 0.143406, 0.285884, 0.144166, 0.281818, 0.144143, 0.336364, 0.143794, 0.284007, 0.144143, 0.336364, 0.144166, 0.281818, 0.143794, 0.284007, 0.133589, 0.226829, 0.128751, 0.327273, 0.133221, 0.204541, 0.128751, 0.327273, 0.133589, 0.226829, 0.133221, 0.204541, 0.133098, 0.179119, 0.133221, 0.204541, 0.128751, 0.327273, 0.133221, 0.204541, 0.133098, 0.179119, 0.128751, 0.327273, 0.133098, 0.179119, 0.128751, 0.327273, 0.128751, 0.009091, 0.128751, 0.327273, 0.133098, 0.179119, 0.128751, 0.009091, 0.128751, 0.009091, 0.133098, 0.009091, 0.133098, 0.179119, 0.133098, 0.009091, 0.128751, 0.009091, 0.133098, 0.179119, 0.160428, 0.163636, 0.158009, 0.162775, 0.160428, 0.163636, 0.160428, 0.163636, 0.158009, 0.162775, 0.158009, 0.162775, 0.158009, 0.162775, 0.155992, 0.160192, 0.158009, 0.162775, 0.158009, 0.162775, 0.155992, 0.160192, 0.155992, 0.160192, 0.155992, 0.160192, 0.154378, 0.155886, 0.155992, 0.160192, 0.155992, 0.160192, 0.154378, 0.155886, 0.154378, 0.155886, 0.154378, 0.155886, 0.153167, 0.149858, 0.154378, 0.155886, 0.154378, 0.155886, 0.153167, 0.149858, 0.153167, 0.149858, 0.153167, 0.149858, 0.152282, 0.141761, 0.153167, 0.149858, 0.153167, 0.149858, 0.152282, 0.141761, 0.152282, 0.141761, 0.152282, 0.141761, 0.151651, 0.131250, 0.152282, 0.141761, 0.152282, 0.141761, 0.151651, 0.131250, 0.151651, 0.131250, 0.151651, 0.131250, 0.151272, 0.118324, 0.151651, 0.131250, 0.151651, 0.131250, 0.151272, 0.118324, 0.151272, 0.118324, 0.151272, 0.118324, 0.151146, 0.102983, 0.151272, 0.118324, 0.151272, 0.118324, 0.151146, 0.102983, 0.151146, 0.102983, 0.151146, 0.102983, 0.151240, 0.090394, 0.151146, 0.102983, 0.151146, 0.102983, 0.151240, 0.090394, 0.151240, 0.090394, 0.151240, 0.090394, 0.151524, 0.079190, 0.151240, 0.090394, 0.151240, 0.090394, 0.151524, 0.079190, 0.151524, 0.079190, 0.151524, 0.079190, 0.151998, 0.069371, 0.151524, 0.079190, 0.151524, 0.079190, 0.151998, 0.069371, 0.151998, 0.069371, 0.151998, 0.069371, 0.152661, 0.060937, 0.151998, 0.069371, 0.151998, 0.069371, 0.152661, 0.060937, 0.152661, 0.060937, 0.152661, 0.060937, 0.153487, 0.054164, 0.152661, 0.060937, 0.152661, 0.060937, 0.153487, 0.054164, 0.153487, 0.054164, 0.153487, 0.054164, 0.154450, 0.049325, 0.153487, 0.054164, 0.153487, 0.054164, 0.154450, 0.049325, 0.154450, 0.049325, 0.154450, 0.049325, 0.155549, 0.046422, 0.154450, 0.049325, 0.154450, 0.049325, 0.155549, 0.046422, 0.155549, 0.046422, 0.155549, 0.046422, 0.156785, 0.045455, 0.155549, 0.046422, 0.155549, 0.046422, 0.156785, 0.045455, 0.156785, 0.045455, 0.156785, 0.045455, 0.158494, 0.047279, 0.156785, 0.045455, 0.156785, 0.045455, 0.158494, 0.047279, 0.158494, 0.047279, 0.158494, 0.047279, 0.160025, 0.052752, 0.158494, 0.047279, 0.158494, 0.047279, 0.160025, 0.052752, 0.160025, 0.052752, 0.160025, 0.052752, 0.161379, 0.061874, 0.160025, 0.052752, 0.160025, 0.052752, 0.161379, 0.061874, 0.161379, 0.061874, 0.161379, 0.061874, 0.162554, 0.074645, 0.161379, 0.061874, 0.161379, 0.061874, 0.162554, 0.074645, 0.162554, 0.074645, 0.162554, 0.074645, 0.163505, 0.090443, 0.162554, 0.074645, 0.162554, 0.074645, 0.163505, 0.090443, 0.163505, 0.090443, 0.163505, 0.090443, 0.164185, 0.108647, 0.163505, 0.090443, 0.163505, 0.090443, 0.164185, 0.108647, 0.164185, 0.108647, 0.164185, 0.108647, 0.164592, 0.129257, 0.164185, 0.108647, 0.164185, 0.108647, 0.164592, 0.129257, 0.164592, 0.129257, 0.164592, 0.129257, 0.164728, 0.152273, 0.164592, 0.129257, 0.164592, 0.129257, 0.164728, 0.152273, 0.164728, 0.152273, 0.164728, 0.152273, 0.164728, 0.163636, 0.164728, 0.152273, 0.164728, 0.152273, 0.164728, 0.163636, 0.164728, 0.163636, 0.169052, 0.187642, 0.169052, 0.009091, 0.169052, 0.187642, 0.169052, 0.187642, 0.169052, 0.009091, 0.169052, 0.009091, 0.169052, 0.009091, 0.164728, 0.009091, 0.169052, 0.009091, 0.169052, 0.009091, 0.164728, 0.009091, 0.164728, 0.009091, 0.164728, 0.009091, 0.164728, 0.054545, 0.164728, 0.009091, 0.164728, 0.009091, 0.164728, 0.054545, 0.164728, 0.054545, 0.164728, 0.054545, 0.163942, 0.041473, 0.164728, 0.054545, 0.164728, 0.054545, 0.163942, 0.041473, 0.163942, 0.041473, 0.163942, 0.041473, 0.163065, 0.030238, 0.163942, 0.041473, 0.163942, 0.041473, 0.163065, 0.030238, 0.163065, 0.030238, 0.163065, 0.030238, 0.162098, 0.020841, 0.163065, 0.030238, 0.163065, 0.030238, 0.162098, 0.020841, 0.162098, 0.020841, 0.162098, 0.020841, 0.161039, 0.013281, 0.162098, 0.020841, 0.162098, 0.020841, 0.161039, 0.013281, 0.161039, 0.013281, 0.161039, 0.013281, 0.159873, 0.007471, 0.161039, 0.013281, 0.161039, 0.013281, 0.159873, 0.007471, 0.159873, 0.007471, 0.159873, 0.007471, 0.158583, 0.003320, 0.159873, 0.007471, 0.159873, 0.007471, 0.158583, 0.003320, 0.158583, 0.003320, 0.158583, 0.003320, 0.157170, 0.000830, 0.158583, 0.003320, 0.158583, 0.003320, 0.157170, 0.000830, 0.157170, 0.000830, 0.157170, 0.000830, 0.155634, 0, 0.157170, 0.000830, 0.157170, 0.000830, 0.155634, 0, 0.155634, 0, 0.155634, 0, 0.153717, 0.001713, 0.155634, 0, 0.155634, 0, 0.153717, 0.001713, 0.153717, 0.001713, 0.153717, 0.001713, 0.152006, 0.006854, 0.153717, 0.001713, 0.153717, 0.001713, 0.152006, 0.006854, 0.152006, 0.006854, 0.152006, 0.006854, 0.150503, 0.015421, 0.152006, 0.006854, 0.152006, 0.006854, 0.150503, 0.015421, 0.150503, 0.015421, 0.150503, 0.015421, 0.149207, 0.027415, 0.150503, 0.015421, 0.150503, 0.015421, 0.149207, 0.027415, 0.149207, 0.027415, 0.149207, 0.027415, 0.148163, 0.042285, 0.149207, 0.027415, 0.149207, 0.027415, 0.148163, 0.042285, 0.148163, 0.042285, 0.148163, 0.042285, 0.147418, 0.059482, 0.148163, 0.042285, 0.148163, 0.042285, 0.147418, 0.059482, 0.147418, 0.059482, 0.147418, 0.059482, 0.146971, 0.079004, 0.147418, 0.059482, 0.147418, 0.059482, 0.146971, 0.079004, 0.146971, 0.079004, 0.146971, 0.079004, 0.146822, 0.100852, 0.146971, 0.079004, 0.146971, 0.079004, 0.146822, 0.100852, 0.146822, 0.100852, 0.146822, 0.100852, 0.147008, 0.126048, 0.146822, 0.100852, 0.146822, 0.100852, 0.147008, 0.126048, 0.147008, 0.126048, 0.147008, 0.126048, 0.147565, 0.147940, 0.147008, 0.126048, 0.147008, 0.126048, 0.147565, 0.147940, 0.147565, 0.147940, 0.147565, 0.147940, 0.148494, 0.166531, 0.147565, 0.147940, 0.147565, 0.147940, 0.148494, 0.166531, 0.148494, 0.166531, 0.148494, 0.166531, 0.149794, 0.181818, 0.148494, 0.166531, 0.148494, 0.166531, 0.149794, 0.181818, 0.149794, 0.181818, 0.149794, 0.181818, 0.151464, 0.193750, 0.149794, 0.181818, 0.149794, 0.181818, 0.151464, 0.193750, 0.151464, 0.193750, 0.151464, 0.193750, 0.153498, 0.202273, 0.151464, 0.193750, 0.151464, 0.193750, 0.153498, 0.202273, 0.153498, 0.202273, 0.153498, 0.202273, 0.155899, 0.207386, 0.153498, 0.202273, 0.153498, 0.202273, 0.155899, 0.207386, 0.155899, 0.207386, 0.155899, 0.207386, 0.158665, 0.209091, 0.155899, 0.207386, 0.155899, 0.207386, 0.158665, 0.209091, 0.158665, 0.209091, 0.158665, 0.209091, 0.164728, 0.209091, 0.158665, 0.209091, 0.158665, 0.209091, 0.164728, 0.209091, 0.164728, 0.209091, 0.164728, 0.209091, 0.164728, 0.214205, 0.164728, 0.209091, 0.164728, 0.209091, 0.164728, 0.214205, 0.164728, 0.214205, 0.164728, 0.214205, 0.164605, 0.231490, 0.164728, 0.214205, 0.164728, 0.214205, 0.164605, 0.231490, 0.164605, 0.231490, 0.164605, 0.231490, 0.164237, 0.246697, 0.164605, 0.231490, 0.164605, 0.231490, 0.164237, 0.246697, 0.164237, 0.246697, 0.164237, 0.246697, 0.163624, 0.259828, 0.164237, 0.246697, 0.164237, 0.246697, 0.163624, 0.259828, 0.163624, 0.259828, 0.163624, 0.259828, 0.162766, 0.270881, 0.163624, 0.259828, 0.163624, 0.259828, 0.162766, 0.270881, 0.162766, 0.270881, 0.162766, 0.270881, 0.161686, 0.279643, 0.162766, 0.270881, 0.162766, 0.270881, 0.161686, 0.279643, 0.161686, 0.279643, 0.161686, 0.279643, 0.160407, 0.285902, 0.161686, 0.279643, 0.161686, 0.279643, 0.160407, 0.285902, 0.160407, 0.285902, 0.160407, 0.285902, 0.158930, 0.289657, 0.160407, 0.285902, 0.160407, 0.285902, 0.158930, 0.289657, 0.158930, 0.289657, 0.158930, 0.289657, 0.157255, 0.290909, 0.158930, 0.289657, 0.158930, 0.289657, 0.157255, 0.290909, 0.157255, 0.290909, 0.157255, 0.290909, 0.156135, 0.290483, 0.157255, 0.290909, 0.157255, 0.290909, 0.156135, 0.290483, 0.156135, 0.290483, 0.156135, 0.290483, 0.155029, 0.289205, 0.156135, 0.290483, 0.156135, 0.290483, 0.155029, 0.289205, 0.155029, 0.289205, 0.155029, 0.289205, 0.153938, 0.287074, 0.155029, 0.289205, 0.155029, 0.289205, 0.153938, 0.287074, 0.153938, 0.287074, 0.153938, 0.287074, 0.152861, 0.284091, 0.153938, 0.287074, 0.153938, 0.287074, 0.152861, 0.284091, 0.152861, 0.284091, 0.152861, 0.284091, 0.151802, 0.280256, 0.152861, 0.284091, 0.152861, 0.284091, 0.151802, 0.280256, 0.151802, 0.280256, 0.151802, 0.280256, 0.150764, 0.275568, 0.151802, 0.280256, 0.151802, 0.280256, 0.150764, 0.275568, 0.150764, 0.275568, 0.150764, 0.275568, 0.149746, 0.270028, 0.150764, 0.275568, 0.150764, 0.275568, 0.149746, 0.270028, 0.149746, 0.270028, 0.149746, 0.270028, 0.148749, 0.263636, 0.149746, 0.270028, 0.149746, 0.270028, 0.148749, 0.263636, 0.148749, 0.263636, 0.148749, 0.263636, 0.148749, 0.309091, 0.148749, 0.263636, 0.148749, 0.263636, 0.148749, 0.309091, 0.148749, 0.309091, 0.148749, 0.309091, 0.149927, 0.315483, 0.148749, 0.309091, 0.148749, 0.309091, 0.149927, 0.315483, 0.149927, 0.315483, 0.149927, 0.315483, 0.151087, 0.321023, 0.149927, 0.315483, 0.149927, 0.315483, 0.151087, 0.321023, 0.151087, 0.321023, 0.151087, 0.321023, 0.152229, 0.325710, 0.151087, 0.321023, 0.151087, 0.321023, 0.152229, 0.325710, 0.152229, 0.325710, 0.152229, 0.325710, 0.153354, 0.329545, 0.152229, 0.325710, 0.152229, 0.325710, 0.153354, 0.329545, 0.153354, 0.329545, 0.153354, 0.329545, 0.154463, 0.332528, 0.153354, 0.329545, 0.153354, 0.329545, 0.154463, 0.332528, 0.154463, 0.332528, 0.154463, 0.332528, 0.155558, 0.334659, 0.154463, 0.332528, 0.154463, 0.332528, 0.155558, 0.334659, 0.155558, 0.334659, 0.155558, 0.334659, 0.156637, 0.335938, 0.155558, 0.334659, 0.155558, 0.334659, 0.156637, 0.335938, 0.156637, 0.335938, 0.156637, 0.335938, 0.157702, 0.336364, 0.156637, 0.335938, 0.156637, 0.335938, 0.157702, 0.336364, 0.157702, 0.336364, 0.157702, 0.336364, 0.160376, 0.334060, 0.157702, 0.336364, 0.157702, 0.336364, 0.160376, 0.334060, 0.160376, 0.334060, 0.160376, 0.334060, 0.162689, 0.327148, 0.160376, 0.334060, 0.160376, 0.334060, 0.162689, 0.327148, 0.162689, 0.327148, 0.162689, 0.327148, 0.164641, 0.315629, 0.162689, 0.327148, 0.162689, 0.327148, 0.164641, 0.315629, 0.164641, 0.315629, 0.164641, 0.315629, 0.166232, 0.299503, 0.164641, 0.315629, 0.164641, 0.315629, 0.166232, 0.299503, 0.166232, 0.299503, 0.166232, 0.299503, 0.167466, 0.278689, 0.166232, 0.299503, 0.166232, 0.299503, 0.167466, 0.278689, 0.167466, 0.278689, 0.167466, 0.278689, 0.168347, 0.253107, 0.167466, 0.278689, 0.167466, 0.278689, 0.168347, 0.253107, 0.168347, 0.253107, 0.168347, 0.253107, 0.168876, 0.222758, 0.168347, 0.253107, 0.168347, 0.253107, 0.168876, 0.222758, 0.168876, 0.222758, 0.147008, 0.126048, 0.146822, 0.100852, 0.151146, 0.102983, 0.146822, 0.100852, 0.147008, 0.126048, 0.151146, 0.102983, 0.147008, 0.126048, 0.151146, 0.102983, 0.151272, 0.118324, 0.151146, 0.102983, 0.147008, 0.126048, 0.151272, 0.118324, 0.147565, 0.147940, 0.147008, 0.126048, 0.151272, 0.118324, 0.147008, 0.126048, 0.147565, 0.147940, 0.151272, 0.118324, 0.147565, 0.147940, 0.151272, 0.118324, 0.151651, 0.131250, 0.151272, 0.118324, 0.147565, 0.147940, 0.151651, 0.131250, 0.148494, 0.166531, 0.147565, 0.147940, 0.151651, 0.131250, 0.147565, 0.147940, 0.148494, 0.166531, 0.151651, 0.131250, 0.148494, 0.166531, 0.151651, 0.131250, 0.152282, 0.141761, 0.151651, 0.131250, 0.148494, 0.166531, 0.152282, 0.141761, 0.149794, 0.181818, 0.148494, 0.166531, 0.152282, 0.141761, 0.148494, 0.166531, 0.149794, 0.181818, 0.152282, 0.141761, 0.149794, 0.181818, 0.152282, 0.141761, 0.153167, 0.149858, 0.152282, 0.141761, 0.149794, 0.181818, 0.153167, 0.149858, 0.151464, 0.193750, 0.149794, 0.181818, 0.153167, 0.149858, 0.149794, 0.181818, 0.151464, 0.193750, 0.153167, 0.149858, 0.154378, 0.155886, 0.151464, 0.193750, 0.153167, 0.149858, 0.151464, 0.193750, 0.154378, 0.155886, 0.153167, 0.149858, 0.153498, 0.202273, 0.151464, 0.193750, 0.154378, 0.155886, 0.151464, 0.193750, 0.153498, 0.202273, 0.154378, 0.155886, 0.155992, 0.160192, 0.153498, 0.202273, 0.154378, 0.155886, 0.153498, 0.202273, 0.155992, 0.160192, 0.154378, 0.155886, 0.155899, 0.207386, 0.153498, 0.202273, 0.155992, 0.160192, 0.153498, 0.202273, 0.155899, 0.207386, 0.155992, 0.160192, 0.155899, 0.207386, 0.155992, 0.160192, 0.158009, 0.162775, 0.155992, 0.160192, 0.155899, 0.207386, 0.158009, 0.162775, 0.155899, 0.207386, 0.158009, 0.162775, 0.158665, 0.209091, 0.158009, 0.162775, 0.155899, 0.207386, 0.158665, 0.209091, 0.158665, 0.209091, 0.158009, 0.162775, 0.160428, 0.163636, 0.158009, 0.162775, 0.158665, 0.209091, 0.160428, 0.163636, 0.158665, 0.209091, 0.160428, 0.163636, 0.164728, 0.209091, 0.160428, 0.163636, 0.158665, 0.209091, 0.164728, 0.209091, 0.160428, 0.163636, 0.164728, 0.163636, 0.164728, 0.209091, 0.164728, 0.163636, 0.160428, 0.163636, 0.164728, 0.209091, 0.164728, 0.209091, 0.164728, 0.163636, 0.169052, 0.187642, 0.164728, 0.163636, 0.164728, 0.209091, 0.169052, 0.187642, 0.169052, 0.187642, 0.168876, 0.222758, 0.164728, 0.209091, 0.168876, 0.222758, 0.169052, 0.187642, 0.164728, 0.209091, 0.168876, 0.222758, 0.164728, 0.214205, 0.164728, 0.209091, 0.164728, 0.214205, 0.168876, 0.222758, 0.164728, 0.209091, 0.164605, 0.231490, 0.164728, 0.214205, 0.168876, 0.222758, 0.164728, 0.214205, 0.164605, 0.231490, 0.168876, 0.222758, 0.168347, 0.253107, 0.164605, 0.231490, 0.168876, 0.222758, 0.164605, 0.231490, 0.168347, 0.253107, 0.168876, 0.222758, 0.164237, 0.246697, 0.164605, 0.231490, 0.168347, 0.253107, 0.164605, 0.231490, 0.164237, 0.246697, 0.168347, 0.253107, 0.167466, 0.278689, 0.164237, 0.246697, 0.168347, 0.253107, 0.164237, 0.246697, 0.167466, 0.278689, 0.168347, 0.253107, 0.163624, 0.259828, 0.164237, 0.246697, 0.167466, 0.278689, 0.164237, 0.246697, 0.163624, 0.259828, 0.167466, 0.278689, 0.166232, 0.299503, 0.163624, 0.259828, 0.167466, 0.278689, 0.163624, 0.259828, 0.166232, 0.299503, 0.167466, 0.278689, 0.162766, 0.270881, 0.163624, 0.259828, 0.166232, 0.299503, 0.163624, 0.259828, 0.162766, 0.270881, 0.166232, 0.299503, 0.164641, 0.315629, 0.162766, 0.270881, 0.166232, 0.299503, 0.162766, 0.270881, 0.164641, 0.315629, 0.166232, 0.299503, 0.161686, 0.279643, 0.162766, 0.270881, 0.164641, 0.315629, 0.162766, 0.270881, 0.161686, 0.279643, 0.164641, 0.315629, 0.162689, 0.327148, 0.161686, 0.279643, 0.164641, 0.315629, 0.161686, 0.279643, 0.162689, 0.327148, 0.164641, 0.315629, 0.160407, 0.285902, 0.161686, 0.279643, 0.162689, 0.327148, 0.161686, 0.279643, 0.160407, 0.285902, 0.162689, 0.327148, 0.160376, 0.334060, 0.160407, 0.285902, 0.162689, 0.327148, 0.160407, 0.285902, 0.160376, 0.334060, 0.162689, 0.327148, 0.160376, 0.334060, 0.158930, 0.289657, 0.160407, 0.285902, 0.158930, 0.289657, 0.160376, 0.334060, 0.160407, 0.285902, 0.160376, 0.334060, 0.157702, 0.336364, 0.158930, 0.289657, 0.157702, 0.336364, 0.160376, 0.334060, 0.158930, 0.289657, 0.157255, 0.290909, 0.158930, 0.289657, 0.157702, 0.336364, 0.158930, 0.289657, 0.157255, 0.290909, 0.157702, 0.336364, 0.157702, 0.336364, 0.156637, 0.335938, 0.157255, 0.290909, 0.156637, 0.335938, 0.157702, 0.336364, 0.157255, 0.290909, 0.156637, 0.335938, 0.156135, 0.290483, 0.157255, 0.290909, 0.156135, 0.290483, 0.156637, 0.335938, 0.157255, 0.290909, 0.155558, 0.334659, 0.156135, 0.290483, 0.156637, 0.335938, 0.156135, 0.290483, 0.155558, 0.334659, 0.156637, 0.335938, 0.155558, 0.334659, 0.155029, 0.289205, 0.156135, 0.290483, 0.155029, 0.289205, 0.155558, 0.334659, 0.156135, 0.290483, 0.154463, 0.332528, 0.155029, 0.289205, 0.155558, 0.334659, 0.155029, 0.289205, 0.154463, 0.332528, 0.155558, 0.334659, 0.154463, 0.332528, 0.153354, 0.329545, 0.155029, 0.289205, 0.153354, 0.329545, 0.154463, 0.332528, 0.155029, 0.289205, 0.155029, 0.289205, 0.153354, 0.329545, 0.153938, 0.287074, 0.153354, 0.329545, 0.155029, 0.289205, 0.153938, 0.287074, 0.153354, 0.329545, 0.152229, 0.325710, 0.153938, 0.287074, 0.152229, 0.325710, 0.153354, 0.329545, 0.153938, 0.287074, 0.152229, 0.325710, 0.152861, 0.284091, 0.153938, 0.287074, 0.152861, 0.284091, 0.152229, 0.325710, 0.153938, 0.287074, 0.152229, 0.325710, 0.151087, 0.321023, 0.152861, 0.284091, 0.151087, 0.321023, 0.152229, 0.325710, 0.152861, 0.284091, 0.151087, 0.321023, 0.151802, 0.280256, 0.152861, 0.284091, 0.151802, 0.280256, 0.151087, 0.321023, 0.152861, 0.284091, 0.151087, 0.321023, 0.149927, 0.315483, 0.151802, 0.280256, 0.149927, 0.315483, 0.151087, 0.321023, 0.151802, 0.280256, 0.149927, 0.315483, 0.150764, 0.275568, 0.151802, 0.280256, 0.150764, 0.275568, 0.149927, 0.315483, 0.151802, 0.280256, 0.149927, 0.315483, 0.148749, 0.309091, 0.150764, 0.275568, 0.148749, 0.309091, 0.149927, 0.315483, 0.150764, 0.275568, 0.148749, 0.309091, 0.149746, 0.270028, 0.150764, 0.275568, 0.149746, 0.270028, 0.148749, 0.309091, 0.150764, 0.275568, 0.148749, 0.309091, 0.148749, 0.263636, 0.149746, 0.270028, 0.148749, 0.263636, 0.148749, 0.309091, 0.149746, 0.270028, 0.169052, 0.187642, 0.164728, 0.163636, 0.164728, 0.152273, 0.164728, 0.163636, 0.169052, 0.187642, 0.164728, 0.152273, 0.169052, 0.187642, 0.164728, 0.152273, 0.164592, 0.129257, 0.164728, 0.152273, 0.169052, 0.187642, 0.164592, 0.129257, 0.169052, 0.009091, 0.169052, 0.187642, 0.164592, 0.129257, 0.169052, 0.187642, 0.169052, 0.009091, 0.164592, 0.129257, 0.169052, 0.009091, 0.164592, 0.129257, 0.164185, 0.108647, 0.164592, 0.129257, 0.169052, 0.009091, 0.164185, 0.108647, 0.164728, 0.054545, 0.169052, 0.009091, 0.164185, 0.108647, 0.169052, 0.009091, 0.164728, 0.054545, 0.164185, 0.108647, 0.164728, 0.054545, 0.164728, 0.009091, 0.169052, 0.009091, 0.164728, 0.009091, 0.164728, 0.054545, 0.169052, 0.009091, 0.164185, 0.108647, 0.163505, 0.090443, 0.164728, 0.054545, 0.163505, 0.090443, 0.164185, 0.108647, 0.164728, 0.054545, 0.163505, 0.090443, 0.162554, 0.074645, 0.164728, 0.054545, 0.162554, 0.074645, 0.163505, 0.090443, 0.164728, 0.054545, 0.162554, 0.074645, 0.163942, 0.041473, 0.164728, 0.054545, 0.163942, 0.041473, 0.162554, 0.074645, 0.164728, 0.054545, 0.162554, 0.074645, 0.161379, 0.061874, 0.163942, 0.041473, 0.161379, 0.061874, 0.162554, 0.074645, 0.163942, 0.041473, 0.161379, 0.061874, 0.163065, 0.030238, 0.163942, 0.041473, 0.163065, 0.030238, 0.161379, 0.061874, 0.163942, 0.041473, 0.163065, 0.030238, 0.161379, 0.061874, 0.162098, 0.020841, 0.161379, 0.061874, 0.163065, 0.030238, 0.162098, 0.020841, 0.161379, 0.061874, 0.160025, 0.052752, 0.162098, 0.020841, 0.160025, 0.052752, 0.161379, 0.061874, 0.162098, 0.020841, 0.160025, 0.052752, 0.161039, 0.013281, 0.162098, 0.020841, 0.161039, 0.013281, 0.160025, 0.052752, 0.162098, 0.020841, 0.161039, 0.013281, 0.160025, 0.052752, 0.159873, 0.007471, 0.160025, 0.052752, 0.161039, 0.013281, 0.159873, 0.007471, 0.160025, 0.052752, 0.158494, 0.047279, 0.159873, 0.007471, 0.158494, 0.047279, 0.160025, 0.052752, 0.159873, 0.007471, 0.158494, 0.047279, 0.158583, 0.003320, 0.159873, 0.007471, 0.158583, 0.003320, 0.158494, 0.047279, 0.159873, 0.007471, 0.158494, 0.047279, 0.156785, 0.045455, 0.158583, 0.003320, 0.156785, 0.045455, 0.158494, 0.047279, 0.158583, 0.003320, 0.156785, 0.045455, 0.157170, 0.000830, 0.158583, 0.003320, 0.157170, 0.000830, 0.156785, 0.045455, 0.158583, 0.003320, 0.156785, 0.045455, 0.155634, 0, 0.157170, 0.000830, 0.155634, 0, 0.156785, 0.045455, 0.157170, 0.000830, 0.155549, 0.046422, 0.155634, 0, 0.156785, 0.045455, 0.155634, 0, 0.155549, 0.046422, 0.156785, 0.045455, 0.155549, 0.046422, 0.153717, 0.001713, 0.155634, 0, 0.153717, 0.001713, 0.155549, 0.046422, 0.155634, 0, 0.154450, 0.049325, 0.153717, 0.001713, 0.155549, 0.046422, 0.153717, 0.001713, 0.154450, 0.049325, 0.155549, 0.046422, 0.152006, 0.006854, 0.153717, 0.001713, 0.154450, 0.049325, 0.153717, 0.001713, 0.152006, 0.006854, 0.154450, 0.049325, 0.153487, 0.054164, 0.152006, 0.006854, 0.154450, 0.049325, 0.152006, 0.006854, 0.153487, 0.054164, 0.154450, 0.049325, 0.150503, 0.015421, 0.152006, 0.006854, 0.153487, 0.054164, 0.152006, 0.006854, 0.150503, 0.015421, 0.153487, 0.054164, 0.153487, 0.054164, 0.152661, 0.060937, 0.150503, 0.015421, 0.152661, 0.060937, 0.153487, 0.054164, 0.150503, 0.015421, 0.152661, 0.060937, 0.149207, 0.027415, 0.150503, 0.015421, 0.149207, 0.027415, 0.152661, 0.060937, 0.150503, 0.015421, 0.152661, 0.060937, 0.151998, 0.069371, 0.149207, 0.027415, 0.151998, 0.069371, 0.152661, 0.060937, 0.149207, 0.027415, 0.151998, 0.069371, 0.148163, 0.042285, 0.149207, 0.027415, 0.148163, 0.042285, 0.151998, 0.069371, 0.149207, 0.027415, 0.151998, 0.069371, 0.151524, 0.079190, 0.148163, 0.042285, 0.151524, 0.079190, 0.151998, 0.069371, 0.148163, 0.042285, 0.151524, 0.079190, 0.147418, 0.059482, 0.148163, 0.042285, 0.147418, 0.059482, 0.151524, 0.079190, 0.148163, 0.042285, 0.146971, 0.079004, 0.147418, 0.059482, 0.151524, 0.079190, 0.147418, 0.059482, 0.146971, 0.079004, 0.151524, 0.079190, 0.151240, 0.090394, 0.146971, 0.079004, 0.151524, 0.079190, 0.146971, 0.079004, 0.151240, 0.090394, 0.151524, 0.079190, 0.146822, 0.100852, 0.146971, 0.079004, 0.151240, 0.090394, 0.146971, 0.079004, 0.146822, 0.100852, 0.151240, 0.090394, 0.151146, 0.102983, 0.146822, 0.100852, 0.151240, 0.090394, 0.146822, 0.100852, 0.151146, 0.102983, 0.151240, 0.090394, 0.177793, 0.454545, 0.182117, 0.454545, 0.177793, 0.454545, 0.177793, 0.454545, 0.182117, 0.454545, 0.182117, 0.454545, 0.182117, 0.454545, 0.182117, 0.009091, 0.182117, 0.454545, 0.182117, 0.454545, 0.182117, 0.009091, 0.182117, 0.009091, 0.182117, 0.009091, 0.177793, 0.009091, 0.182117, 0.009091, 0.182117, 0.009091, 0.177793, 0.009091, 0.177793, 0.009091, 0.177793, 0.454545, 0.182117, 0.009091, 0.182117, 0.454545, 0.182117, 0.009091, 0.177793, 0.454545, 0.182117, 0.454545, 0.177793, 0.454545, 0.177793, 0.009091, 0.182117, 0.009091, 0.177793, 0.009091, 0.177793, 0.454545, 0.182117, 0.009091, 0.211303, 0.390909, 0.211303, 0.227273, 0.211303, 0.390909, 0.211303, 0.390909, 0.211303, 0.227273, 0.211303, 0.227273, 0.211303, 0.227273, 0.217272, 0.227273, 0.211303, 0.227273, 0.211303, 0.227273, 0.217272, 0.227273, 0.217272, 0.227273, 0.217272, 0.227273, 0.218834, 0.228604, 0.217272, 0.227273, 0.217272, 0.227273, 0.218834, 0.228604, 0.218834, 0.228604, 0.218834, 0.228604, 0.220209, 0.232599, 0.218834, 0.228604, 0.218834, 0.228604, 0.220209, 0.232599, 0.220209, 0.232599, 0.220209, 0.232599, 0.221396, 0.239258, 0.220209, 0.232599, 0.220209, 0.232599, 0.221396, 0.239258, 0.221396, 0.239258, 0.221396, 0.239258, 0.222395, 0.248580, 0.221396, 0.239258, 0.221396, 0.239258, 0.222395, 0.248580, 0.222395, 0.248580, 0.222395, 0.248580, 0.223186, 0.260360, 0.222395, 0.248580, 0.222395, 0.248580, 0.223186, 0.260360, 0.223186, 0.260360, 0.223186, 0.260360, 0.223752, 0.274396, 0.223186, 0.260360, 0.223186, 0.260360, 0.223752, 0.274396, 0.223752, 0.274396, 0.223752, 0.274396, 0.224091, 0.290687, 0.223752, 0.274396, 0.223752, 0.274396, 0.224091, 0.290687, 0.224091, 0.290687, 0.224091, 0.290687, 0.224204, 0.309233, 0.224091, 0.290687, 0.224091, 0.290687, 0.224204, 0.309233, 0.224204, 0.309233, 0.224204, 0.309233, 0.224091, 0.327654, 0.224204, 0.309233, 0.224204, 0.309233, 0.224091, 0.327654, 0.224091, 0.327654, 0.224091, 0.327654, 0.223752, 0.343857, 0.224091, 0.327654, 0.224091, 0.327654, 0.223752, 0.343857, 0.223752, 0.343857, 0.223752, 0.343857, 0.223186, 0.357839, 0.223752, 0.343857, 0.223752, 0.343857, 0.223186, 0.357839, 0.223186, 0.357839, 0.223186, 0.357839, 0.222395, 0.369602, 0.223186, 0.357839, 0.223186, 0.357839, 0.222395, 0.369602, 0.222395, 0.369602, 0.222395, 0.369602, 0.221396, 0.378924, 0.222395, 0.369602, 0.222395, 0.369602, 0.221396, 0.378924, 0.221396, 0.378924, 0.221396, 0.378924, 0.220209, 0.385582, 0.221396, 0.378924, 0.221396, 0.378924, 0.220209, 0.385582, 0.220209, 0.385582, 0.220209, 0.385582, 0.218834, 0.389577, 0.220209, 0.385582, 0.220209, 0.385582, 0.218834, 0.389577, 0.218834, 0.389577, 0.218834, 0.389577, 0.217272, 0.390909, 0.218834, 0.389577, 0.218834, 0.389577, 0.217272, 0.390909, 0.217272, 0.390909, 0.206556, 0.436364, 0.217272, 0.436364, 0.206556, 0.436364, 0.206556, 0.436364, 0.217272, 0.436364, 0.217272, 0.436364, 0.217272, 0.436364, 0.220041, 0.434339, 0.217272, 0.436364, 0.217272, 0.436364, 0.220041, 0.434339, 0.220041, 0.434339, 0.220041, 0.434339, 0.222450, 0.428267, 0.220041, 0.434339, 0.220041, 0.434339, 0.222450, 0.428267, 0.222450, 0.428267, 0.222450, 0.428267, 0.224500, 0.418146, 0.222450, 0.428267, 0.222450, 0.428267, 0.224500, 0.418146, 0.224500, 0.418146, 0.224500, 0.418146, 0.226190, 0.403977, 0.224500, 0.418146, 0.224500, 0.418146, 0.226190, 0.403977, 0.226190, 0.403977, 0.226190, 0.403977, 0.227511, 0.385911, 0.226190, 0.403977, 0.226190, 0.403977, 0.227511, 0.385911, 0.227511, 0.385911, 0.227511, 0.385911, 0.228454, 0.364098, 0.227511, 0.385911, 0.227511, 0.385911, 0.228454, 0.364098, 0.228454, 0.364098, 0.228454, 0.364098, 0.229021, 0.338539, 0.228454, 0.364098, 0.228454, 0.364098, 0.229021, 0.338539, 0.229021, 0.338539, 0.229021, 0.338539, 0.229209, 0.309233, 0.229021, 0.338539, 0.229021, 0.338539, 0.229209, 0.309233, 0.229209, 0.309233, 0.229209, 0.309233, 0.229021, 0.279687, 0.229209, 0.309233, 0.229209, 0.309233, 0.229021, 0.279687, 0.229021, 0.279687, 0.229021, 0.279687, 0.228454, 0.253977, 0.229021, 0.279687, 0.229021, 0.279687, 0.228454, 0.253977, 0.228454, 0.253977, 0.228454, 0.253977, 0.227511, 0.232102, 0.228454, 0.253977, 0.228454, 0.253977, 0.227511, 0.232102, 0.227511, 0.232102, 0.227511, 0.232102, 0.226190, 0.214063, 0.227511, 0.232102, 0.227511, 0.232102, 0.226190, 0.214063, 0.226190, 0.214063, 0.226190, 0.214063, 0.224500, 0.199956, 0.226190, 0.214063, 0.226190, 0.214063, 0.224500, 0.199956, 0.224500, 0.199956, 0.224500, 0.199956, 0.222450, 0.189879, 0.224500, 0.199956, 0.224500, 0.199956, 0.222450, 0.189879, 0.222450, 0.189879, 0.222450, 0.189879, 0.220041, 0.183833, 0.222450, 0.189879, 0.222450, 0.189879, 0.220041, 0.183833, 0.220041, 0.183833, 0.220041, 0.183833, 0.217272, 0.181818, 0.220041, 0.183833, 0.220041, 0.183833, 0.217272, 0.181818, 0.217272, 0.181818, 0.217272, 0.181818, 0.211303, 0.181818, 0.217272, 0.181818, 0.217272, 0.181818, 0.211303, 0.181818, 0.211303, 0.181818, 0.211303, 0.181818, 0.211303, 0.009091, 0.211303, 0.181818, 0.211303, 0.181818, 0.211303, 0.009091, 0.211303, 0.009091, 0.211303, 0.009091, 0.206556, 0.009091, 0.211303, 0.009091, 0.211303, 0.009091, 0.206556, 0.009091, 0.206556, 0.009091, 0.206556, 0.436364, 0.211303, 0.390909, 0.217272, 0.436364, 0.211303, 0.390909, 0.206556, 0.436364, 0.217272, 0.436364, 0.211303, 0.390909, 0.206556, 0.436364, 0.211303, 0.227273, 0.206556, 0.436364, 0.211303, 0.390909, 0.211303, 0.227273, 0.206556, 0.436364, 0.206556, 0.009091, 0.211303, 0.227273, 0.206556, 0.009091, 0.206556, 0.436364, 0.211303, 0.227273, 0.206556, 0.009091, 0.211303, 0.181818, 0.211303, 0.227273, 0.211303, 0.181818, 0.206556, 0.009091, 0.211303, 0.227273, 0.206556, 0.009091, 0.211303, 0.009091, 0.211303, 0.181818, 0.211303, 0.009091, 0.206556, 0.009091, 0.211303, 0.181818, 0.211303, 0.227273, 0.211303, 0.181818, 0.217272, 0.181818, 0.211303, 0.181818, 0.211303, 0.227273, 0.217272, 0.181818, 0.211303, 0.227273, 0.217272, 0.181818, 0.217272, 0.227273, 0.217272, 0.181818, 0.211303, 0.227273, 0.217272, 0.227273, 0.218834, 0.228604, 0.217272, 0.227273, 0.217272, 0.181818, 0.217272, 0.227273, 0.218834, 0.228604, 0.217272, 0.181818, 0.220041, 0.183833, 0.218834, 0.228604, 0.217272, 0.181818, 0.218834, 0.228604, 0.220041, 0.183833, 0.217272, 0.181818, 0.218834, 0.228604, 0.220041, 0.183833, 0.220209, 0.232599, 0.220041, 0.183833, 0.218834, 0.228604, 0.220209, 0.232599, 0.220209, 0.232599, 0.220041, 0.183833, 0.222450, 0.189879, 0.220041, 0.183833, 0.220209, 0.232599, 0.222450, 0.189879, 0.221396, 0.239258, 0.220209, 0.232599, 0.222450, 0.189879, 0.220209, 0.232599, 0.221396, 0.239258, 0.222450, 0.189879, 0.221396, 0.239258, 0.222450, 0.189879, 0.224500, 0.199956, 0.222450, 0.189879, 0.221396, 0.239258, 0.224500, 0.199956, 0.222395, 0.248580, 0.221396, 0.239258, 0.224500, 0.199956, 0.221396, 0.239258, 0.222395, 0.248580, 0.224500, 0.199956, 0.222395, 0.248580, 0.224500, 0.199956, 0.226190, 0.214063, 0.224500, 0.199956, 0.222395, 0.248580, 0.226190, 0.214063, 0.223186, 0.260360, 0.222395, 0.248580, 0.226190, 0.214063, 0.222395, 0.248580, 0.223186, 0.260360, 0.226190, 0.214063, 0.223186, 0.260360, 0.226190, 0.214063, 0.227511, 0.232102, 0.226190, 0.214063, 0.223186, 0.260360, 0.227511, 0.232102, 0.223752, 0.274396, 0.223186, 0.260360, 0.227511, 0.232102, 0.223186, 0.260360, 0.223752, 0.274396, 0.227511, 0.232102, 0.228454, 0.253977, 0.223752, 0.274396, 0.227511, 0.232102, 0.223752, 0.274396, 0.228454, 0.253977, 0.227511, 0.232102, 0.223752, 0.274396, 0.228454, 0.253977, 0.229021, 0.279687, 0.228454, 0.253977, 0.223752, 0.274396, 0.229021, 0.279687, 0.224091, 0.290687, 0.223752, 0.274396, 0.229021, 0.279687, 0.223752, 0.274396, 0.224091, 0.290687, 0.229021, 0.279687, 0.229209, 0.309233, 0.224091, 0.290687, 0.229021, 0.279687, 0.224091, 0.290687, 0.229209, 0.309233, 0.229021, 0.279687, 0.229209, 0.309233, 0.224204, 0.309233, 0.224091, 0.290687, 0.224204, 0.309233, 0.229209, 0.309233, 0.224091, 0.290687, 0.229209, 0.309233, 0.224091, 0.327654, 0.224204, 0.309233, 0.224091, 0.327654, 0.229209, 0.309233, 0.224204, 0.309233, 0.229021, 0.338539, 0.224091, 0.327654, 0.229209, 0.309233, 0.224091, 0.327654, 0.229021, 0.338539, 0.229209, 0.309233, 0.229021, 0.338539, 0.223752, 0.343857, 0.224091, 0.327654, 0.223752, 0.343857, 0.229021, 0.338539, 0.224091, 0.327654, 0.228454, 0.364098, 0.223752, 0.343857, 0.229021, 0.338539, 0.223752, 0.343857, 0.228454, 0.364098, 0.229021, 0.338539, 0.227511, 0.385911, 0.223752, 0.343857, 0.228454, 0.364098, 0.223752, 0.343857, 0.227511, 0.385911, 0.228454, 0.364098, 0.223186, 0.357839, 0.223752, 0.343857, 0.227511, 0.385911, 0.223752, 0.343857, 0.223186, 0.357839, 0.227511, 0.385911, 0.226190, 0.403977, 0.223186, 0.357839, 0.227511, 0.385911, 0.223186, 0.357839, 0.226190, 0.403977, 0.227511, 0.385911, 0.222395, 0.369602, 0.223186, 0.357839, 0.226190, 0.403977, 0.223186, 0.357839, 0.222395, 0.369602, 0.226190, 0.403977, 0.224500, 0.418146, 0.222395, 0.369602, 0.226190, 0.403977, 0.222395, 0.369602, 0.224500, 0.418146, 0.226190, 0.403977, 0.221396, 0.378924, 0.222395, 0.369602, 0.224500, 0.418146, 0.222395, 0.369602, 0.221396, 0.378924, 0.224500, 0.418146, 0.222450, 0.428267, 0.221396, 0.378924, 0.224500, 0.418146, 0.221396, 0.378924, 0.222450, 0.428267, 0.224500, 0.418146, 0.220209, 0.385582, 0.221396, 0.378924, 0.222450, 0.428267, 0.221396, 0.378924, 0.220209, 0.385582, 0.222450, 0.428267, 0.220041, 0.434339, 0.220209, 0.385582, 0.222450, 0.428267, 0.220209, 0.385582, 0.220041, 0.434339, 0.222450, 0.428267, 0.220041, 0.434339, 0.218834, 0.389577, 0.220209, 0.385582, 0.218834, 0.389577, 0.220041, 0.434339, 0.220209, 0.385582, 0.217272, 0.436364, 0.218834, 0.389577, 0.220041, 0.434339, 0.218834, 0.389577, 0.217272, 0.436364, 0.220041, 0.434339, 0.217272, 0.436364, 0.217272, 0.390909, 0.218834, 0.389577, 0.217272, 0.390909, 0.217272, 0.436364, 0.218834, 0.389577, 0.211303, 0.390909, 0.217272, 0.390909, 0.217272, 0.436364, 0.217272, 0.390909, 0.211303, 0.390909, 0.217272, 0.436364, 0.250194, 0.281818, 0.249822, 0.284007, 0.250194, 0.281818, 0.250194, 0.281818, 0.249822, 0.284007, 0.249822, 0.284007, 0.249822, 0.284007, 0.249433, 0.285884, 0.249822, 0.284007, 0.249822, 0.284007, 0.249433, 0.285884, 0.249433, 0.285884, 0.249433, 0.285884, 0.249028, 0.287451, 0.249433, 0.285884, 0.249433, 0.285884, 0.249028, 0.287451, 0.249028, 0.287451, 0.249028, 0.287451, 0.248608, 0.288707, 0.249028, 0.287451, 0.249028, 0.287451, 0.248608, 0.288707, 0.248608, 0.288707, 0.248608, 0.288707, 0.248168, 0.289671, 0.248608, 0.288707, 0.248608, 0.288707, 0.248168, 0.289671, 0.248168, 0.289671, 0.248168, 0.289671, 0.247706, 0.290359, 0.248168, 0.289671, 0.248168, 0.289671, 0.247706, 0.290359, 0.247706, 0.290359, 0.247706, 0.290359, 0.247222, 0.290771, 0.247706, 0.290359, 0.247706, 0.290359, 0.247222, 0.290771, 0.247222, 0.290771, 0.247222, 0.290771, 0.246716, 0.290909, 0.247222, 0.290771, 0.247222, 0.290771, 0.246716, 0.290909, 0.246716, 0.290909, 0.246716, 0.290909, 0.244990, 0.289102, 0.246716, 0.290909, 0.246716, 0.290909, 0.244990, 0.289102, 0.244990, 0.289102, 0.244990, 0.289102, 0.243476, 0.283683, 0.244990, 0.289102, 0.244990, 0.289102, 0.243476, 0.283683, 0.243476, 0.283683, 0.243476, 0.283683, 0.242176, 0.274649, 0.243476, 0.283683, 0.243476, 0.283683, 0.242176, 0.274649, 0.242176, 0.274649, 0.242176, 0.274649, 0.241088, 0.262003, 0.242176, 0.274649, 0.242176, 0.274649, 0.241088, 0.262003, 0.241088, 0.262003, 0.241088, 0.262003, 0.240230, 0.245983, 0.241088, 0.262003, 0.241088, 0.262003, 0.240230, 0.245983, 0.240230, 0.245983, 0.240230, 0.245983, 0.239616, 0.226829, 0.240230, 0.245983, 0.240230, 0.245983, 0.239616, 0.226829, 0.239616, 0.226829, 0.239616, 0.226829, 0.239248, 0.204541, 0.239616, 0.226829, 0.239616, 0.226829, 0.239248, 0.204541, 0.239248, 0.204541, 0.239248, 0.204541, 0.239126, 0.179119, 0.239248, 0.204541, 0.239248, 0.204541, 0.239126, 0.179119, 0.239126, 0.179119, 0.239126, 0.179119, 0.239126, 0.009091, 0.239126, 0.179119, 0.239126, 0.179119, 0.239126, 0.009091, 0.239126, 0.009091, 0.239126, 0.009091, 0.234779, 0.009091, 0.239126, 0.009091, 0.239126, 0.009091, 0.234779, 0.009091, 0.234779, 0.009091, 0.234779, 0.009091, 0.234779, 0.327273, 0.234779, 0.009091, 0.234779, 0.009091, 0.234779, 0.327273, 0.234779, 0.327273, 0.234779, 0.327273, 0.239126, 0.327273, 0.234779, 0.327273, 0.234779, 0.327273, 0.239126, 0.327273, 0.239126, 0.327273, 0.239126, 0.327273, 0.239126, 0.281818, 0.239126, 0.327273, 0.239126, 0.327273, 0.239126, 0.281818, 0.239126, 0.281818, 0.239126, 0.281818, 0.239859, 0.294775, 0.239126, 0.281818, 0.239126, 0.281818, 0.239859, 0.294775, 0.239859, 0.294775, 0.239859, 0.294775, 0.240694, 0.305948, 0.239859, 0.294775, 0.239859, 0.294775, 0.240694, 0.305948, 0.240694, 0.305948, 0.240694, 0.305948, 0.241633, 0.315336, 0.240694, 0.305948, 0.240694, 0.305948, 0.241633, 0.315336, 0.241633, 0.315336, 0.241633, 0.315336, 0.242674, 0.322940, 0.241633, 0.315336, 0.241633, 0.315336, 0.242674, 0.322940, 0.242674, 0.322940, 0.242674, 0.322940, 0.243826, 0.328813, 0.242674, 0.322940, 0.242674, 0.322940, 0.243826, 0.328813, 0.243826, 0.328813, 0.243826, 0.328813, 0.245095, 0.333008, 0.243826, 0.328813, 0.243826, 0.328813, 0.245095, 0.333008, 0.245095, 0.333008, 0.245095, 0.333008, 0.246481, 0.335525, 0.245095, 0.333008, 0.245095, 0.333008, 0.246481, 0.335525, 0.246481, 0.335525, 0.246481, 0.335525, 0.247985, 0.336364, 0.246481, 0.335525, 0.246481, 0.335525, 0.247985, 0.336364, 0.247985, 0.336364, 0.247985, 0.336364, 0.248214, 0.336364, 0.247985, 0.336364, 0.247985, 0.336364, 0.248214, 0.336364, 0.248214, 0.336364, 0.248214, 0.336364, 0.248455, 0.336364, 0.248214, 0.336364, 0.248214, 0.336364, 0.248455, 0.336364, 0.248455, 0.336364, 0.248455, 0.336364, 0.248708, 0.336364, 0.248455, 0.336364, 0.248455, 0.336364, 0.248708, 0.336364, 0.248708, 0.336364, 0.248708, 0.336364, 0.248972, 0.336364, 0.248708, 0.336364, 0.248708, 0.336364, 0.248972, 0.336364, 0.248972, 0.336364, 0.248972, 0.336364, 0.249250, 0.336364, 0.248972, 0.336364, 0.248972, 0.336364, 0.249250, 0.336364, 0.249250, 0.336364, 0.249250, 0.336364, 0.249542, 0.336364, 0.249250, 0.336364, 0.249250, 0.336364, 0.249542, 0.336364, 0.249542, 0.336364, 0.249542, 0.336364, 0.249849, 0.336364, 0.249542, 0.336364, 0.249542, 0.336364, 0.249849, 0.336364, 0.249849, 0.336364, 0.249849, 0.336364, 0.250170, 0.336364, 0.249849, 0.336364, 0.249849, 0.336364, 0.250170, 0.336364, 0.250170, 0.336364, 0.239126, 0.327273, 0.234779, 0.327273, 0.239126, 0.281818, 0.234779, 0.327273, 0.239126, 0.327273, 0.239126, 0.281818, 0.239616, 0.226829, 0.239126, 0.281818, 0.234779, 0.327273, 0.239126, 0.281818, 0.239616, 0.226829, 0.234779, 0.327273, 0.239126, 0.281818, 0.239616, 0.226829, 0.240230, 0.245983, 0.239616, 0.226829, 0.239126, 0.281818, 0.240230, 0.245983, 0.239126, 0.281818, 0.240230, 0.245983, 0.241088, 0.262003, 0.240230, 0.245983, 0.239126, 0.281818, 0.241088, 0.262003, 0.239859, 0.294775, 0.239126, 0.281818, 0.241088, 0.262003, 0.239126, 0.281818, 0.239859, 0.294775, 0.241088, 0.262003, 0.239859, 0.294775, 0.241088, 0.262003, 0.242176, 0.274649, 0.241088, 0.262003, 0.239859, 0.294775, 0.242176, 0.274649, 0.240694, 0.305948, 0.239859, 0.294775, 0.242176, 0.274649, 0.239859, 0.294775, 0.240694, 0.305948, 0.242176, 0.274649, 0.240694, 0.305948, 0.242176, 0.274649, 0.243476, 0.283683, 0.242176, 0.274649, 0.240694, 0.305948, 0.243476, 0.283683, 0.241633, 0.315336, 0.240694, 0.305948, 0.243476, 0.283683, 0.240694, 0.305948, 0.241633, 0.315336, 0.243476, 0.283683, 0.242674, 0.322940, 0.241633, 0.315336, 0.243476, 0.283683, 0.241633, 0.315336, 0.242674, 0.322940, 0.243476, 0.283683, 0.242674, 0.322940, 0.243476, 0.283683, 0.244990, 0.289102, 0.243476, 0.283683, 0.242674, 0.322940, 0.244990, 0.289102, 0.243826, 0.328813, 0.242674, 0.322940, 0.244990, 0.289102, 0.242674, 0.322940, 0.243826, 0.328813, 0.244990, 0.289102, 0.243826, 0.328813, 0.244990, 0.289102, 0.245095, 0.333008, 0.244990, 0.289102, 0.243826, 0.328813, 0.245095, 0.333008, 0.245095, 0.333008, 0.244990, 0.289102, 0.246716, 0.290909, 0.244990, 0.289102, 0.245095, 0.333008, 0.246716, 0.290909, 0.246481, 0.335525, 0.245095, 0.333008, 0.246716, 0.290909, 0.245095, 0.333008, 0.246481, 0.335525, 0.246716, 0.290909, 0.246481, 0.335525, 0.246716, 0.290909, 0.247222, 0.290771, 0.246716, 0.290909, 0.246481, 0.335525, 0.247222, 0.290771, 0.247985, 0.336364, 0.246481, 0.335525, 0.247222, 0.290771, 0.246481, 0.335525, 0.247985, 0.336364, 0.247222, 0.290771, 0.247222, 0.290771, 0.247706, 0.290359, 0.247985, 0.336364, 0.247706, 0.290359, 0.247222, 0.290771, 0.247985, 0.336364, 0.248214, 0.336364, 0.247985, 0.336364, 0.247706, 0.290359, 0.247985, 0.336364, 0.248214, 0.336364, 0.247706, 0.290359, 0.248214, 0.336364, 0.247706, 0.290359, 0.248168, 0.289671, 0.247706, 0.290359, 0.248214, 0.336364, 0.248168, 0.289671, 0.248455, 0.336364, 0.248214, 0.336364, 0.248168, 0.289671, 0.248214, 0.336364, 0.248455, 0.336364, 0.248168, 0.289671, 0.248168, 0.289671, 0.248708, 0.336364, 0.248455, 0.336364, 0.248708, 0.336364, 0.248168, 0.289671, 0.248455, 0.336364, 0.248708, 0.336364, 0.248168, 0.289671, 0.248608, 0.288707, 0.248168, 0.289671, 0.248708, 0.336364, 0.248608, 0.288707, 0.248972, 0.336364, 0.248708, 0.336364, 0.248608, 0.288707, 0.248708, 0.336364, 0.248972, 0.336364, 0.248608, 0.288707, 0.248608, 0.288707, 0.249250, 0.336364, 0.248972, 0.336364, 0.249250, 0.336364, 0.248608, 0.288707, 0.248972, 0.336364, 0.249250, 0.336364, 0.248608, 0.288707, 0.249028, 0.287451, 0.248608, 0.288707, 0.249250, 0.336364, 0.249028, 0.287451, 0.249542, 0.336364, 0.249250, 0.336364, 0.249028, 0.287451, 0.249250, 0.336364, 0.249542, 0.336364, 0.249028, 0.287451, 0.249849, 0.336364, 0.249542, 0.336364, 0.249028, 0.287451, 0.249542, 0.336364, 0.249849, 0.336364, 0.249028, 0.287451, 0.249433, 0.285884, 0.249849, 0.336364, 0.249028, 0.287451, 0.249849, 0.336364, 0.249433, 0.285884, 0.249028, 0.287451, 0.250170, 0.336364, 0.249849, 0.336364, 0.249433, 0.285884, 0.249849, 0.336364, 0.250170, 0.336364, 0.249433, 0.285884, 0.249822, 0.284007, 0.250170, 0.336364, 0.249433, 0.285884, 0.250170, 0.336364, 0.249822, 0.284007, 0.249433, 0.285884, 0.250194, 0.281818, 0.250170, 0.336364, 0.249822, 0.284007, 0.250170, 0.336364, 0.250194, 0.281818, 0.249822, 0.284007, 0.239616, 0.226829, 0.234779, 0.327273, 0.239248, 0.204541, 0.234779, 0.327273, 0.239616, 0.226829, 0.239248, 0.204541, 0.239126, 0.179119, 0.239248, 0.204541, 0.234779, 0.327273, 0.239248, 0.204541, 0.239126, 0.179119, 0.234779, 0.327273, 0.239126, 0.179119, 0.234779, 0.327273, 0.234779, 0.009091, 0.234779, 0.327273, 0.239126, 0.179119, 0.234779, 0.009091, 0.234779, 0.009091, 0.239126, 0.009091, 0.239126, 0.179119, 0.239126, 0.009091, 0.234779, 0.009091, 0.239126, 0.179119, 0.263941, 0.290909, 0.262293, 0.288858, 0.263941, 0.290909, 0.263941, 0.290909, 0.262293, 0.288858, 0.262293, 0.288858, 0.262293, 0.288858, 0.260827, 0.282706, 0.262293, 0.288858, 0.262293, 0.288858, 0.260827, 0.282706, 0.260827, 0.282706, 0.260827, 0.282706, 0.259544, 0.272452, 0.260827, 0.282706, 0.260827, 0.282706, 0.259544, 0.272452, 0.259544, 0.272452, 0.259544, 0.272452, 0.258442, 0.258097, 0.259544, 0.272452, 0.259544, 0.272452, 0.258442, 0.258097, 0.258442, 0.258097, 0.258442, 0.258097, 0.257558, 0.240172, 0.258442, 0.258097, 0.258442, 0.258097, 0.257558, 0.240172, 0.257558, 0.240172, 0.257558, 0.240172, 0.256926, 0.219212, 0.257558, 0.240172, 0.257558, 0.240172, 0.256926, 0.219212, 0.256926, 0.219212, 0.256926, 0.219212, 0.256547, 0.195215, 0.256926, 0.219212, 0.256926, 0.219212, 0.256547, 0.195215, 0.256547, 0.195215, 0.256547, 0.195215, 0.256421, 0.168182, 0.256547, 0.195215, 0.256547, 0.195215, 0.256421, 0.168182, 0.256421, 0.168182, 0.256421, 0.168182, 0.256547, 0.141149, 0.256421, 0.168182, 0.256421, 0.168182, 0.256547, 0.141149, 0.256547, 0.141149, 0.256547, 0.141149, 0.256923, 0.117152, 0.256547, 0.141149, 0.256547, 0.141149, 0.256923, 0.117152, 0.256923, 0.117152, 0.256923, 0.117152, 0.257551, 0.096191, 0.256923, 0.117152, 0.256923, 0.117152, 0.257551, 0.096191, 0.257551, 0.096191, 0.257551, 0.096191, 0.258430, 0.078267, 0.257551, 0.096191, 0.257551, 0.096191, 0.258430, 0.078267, 0.258430, 0.078267, 0.258430, 0.078267, 0.259528, 0.063912, 0.258430, 0.078267, 0.258430, 0.078267, 0.259528, 0.063912, 0.259528, 0.063912, 0.259528, 0.063912, 0.260812, 0.053658, 0.259528, 0.063912, 0.259528, 0.063912, 0.260812, 0.053658, 0.260812, 0.053658, 0.260812, 0.053658, 0.262283, 0.047505, 0.260812, 0.053658, 0.260812, 0.053658, 0.262283, 0.047505, 0.262283, 0.047505, 0.262283, 0.047505, 0.263941, 0.045455, 0.262283, 0.047505, 0.262283, 0.047505, 0.263941, 0.045455, 0.263941, 0.045455, 0.263941, 0.045455, 0.265578, 0.047514, 0.263941, 0.045455, 0.263941, 0.045455, 0.265578, 0.047514, 0.265578, 0.047514, 0.265578, 0.047514, 0.267037, 0.053693, 0.265578, 0.047514, 0.265578, 0.047514, 0.267037, 0.053693, 0.267037, 0.053693, 0.267037, 0.053693, 0.268316, 0.063991, 0.267037, 0.053693, 0.267037, 0.053693, 0.268316, 0.063991, 0.268316, 0.063991, 0.268316, 0.063991, 0.269416, 0.078409, 0.268316, 0.063991, 0.268316, 0.063991, 0.269416, 0.078409, 0.269416, 0.078409, 0.269416, 0.078409, 0.270300, 0.096378, 0.269416, 0.078409, 0.269416, 0.078409, 0.270300, 0.096378, 0.270300, 0.096378, 0.270300, 0.096378, 0.270932, 0.117330, 0.270300, 0.096378, 0.270300, 0.096378, 0.270932, 0.117330, 0.270932, 0.117330, 0.270932, 0.117330, 0.271311, 0.141264, 0.270932, 0.117330, 0.270932, 0.117330, 0.271311, 0.141264, 0.271311, 0.141264, 0.271311, 0.141264, 0.271437, 0.168182, 0.271311, 0.141264, 0.271311, 0.141264, 0.271437, 0.168182, 0.271437, 0.168182, 0.271437, 0.168182, 0.271311, 0.194984, 0.271437, 0.168182, 0.271437, 0.168182, 0.271311, 0.194984, 0.271311, 0.194984, 0.271311, 0.194984, 0.270932, 0.218857, 0.271311, 0.194984, 0.271311, 0.194984, 0.270932, 0.218857, 0.270932, 0.218857, 0.270932, 0.218857, 0.270300, 0.239799, 0.270932, 0.218857, 0.270932, 0.218857, 0.270300, 0.239799, 0.270300, 0.239799, 0.270300, 0.239799, 0.269416, 0.257813, 0.270300, 0.239799, 0.270300, 0.239799, 0.269416, 0.257813, 0.269416, 0.257813, 0.269416, 0.257813, 0.268316, 0.272292, 0.269416, 0.257813, 0.269416, 0.257813, 0.268316, 0.272292, 0.268316, 0.272292, 0.268316, 0.272292, 0.267037, 0.282635, 0.268316, 0.272292, 0.268316, 0.272292, 0.267037, 0.282635, 0.267037, 0.282635, 0.267037, 0.282635, 0.265578, 0.288841, 0.267037, 0.282635, 0.267037, 0.282635, 0.265578, 0.288841, 0.265578, 0.288841, 0.263941, 0.336364, 0.266609, 0.333576, 0.263941, 0.336364, 0.263941, 0.336364, 0.266609, 0.333576, 0.266609, 0.333576, 0.266609, 0.333576, 0.268975, 0.325213, 0.266609, 0.333576, 0.266609, 0.333576, 0.268975, 0.325213, 0.268975, 0.325213, 0.268975, 0.325213, 0.271039, 0.311275, 0.268975, 0.325213, 0.268975, 0.325213, 0.271039, 0.311275, 0.271039, 0.311275, 0.271039, 0.311275, 0.272800, 0.291761, 0.271039, 0.311275, 0.271039, 0.311275, 0.272800, 0.291761, 0.272800, 0.291761, 0.272800, 0.291761, 0.274208, 0.267312, 0.272800, 0.291761, 0.272800, 0.291761, 0.274208, 0.267312, 0.274208, 0.267312, 0.274208, 0.267312, 0.275214, 0.238565, 0.274208, 0.267312, 0.274208, 0.267312, 0.275214, 0.238565, 0.275214, 0.238565, 0.275214, 0.238565, 0.275818, 0.205522, 0.275214, 0.238565, 0.275214, 0.238565, 0.275818, 0.205522, 0.275818, 0.205522, 0.275818, 0.205522, 0.276019, 0.168182, 0.275818, 0.205522, 0.275818, 0.205522, 0.276019, 0.168182, 0.276019, 0.168182, 0.276019, 0.168182, 0.275818, 0.130957, 0.276019, 0.168182, 0.276019, 0.168182, 0.275818, 0.130957, 0.275818, 0.130957, 0.275818, 0.130957, 0.275214, 0.097976, 0.275818, 0.130957, 0.275818, 0.130957, 0.275214, 0.097976, 0.275214, 0.097976, 0.275214, 0.097976, 0.274208, 0.069238, 0.275214, 0.097976, 0.275214, 0.097976, 0.274208, 0.069238, 0.274208, 0.069238, 0.274208, 0.069238, 0.272800, 0.044744, 0.274208, 0.069238, 0.274208, 0.069238, 0.272800, 0.044744, 0.272800, 0.044744, 0.272800, 0.044744, 0.271039, 0.025169, 0.272800, 0.044744, 0.272800, 0.044744, 0.271039, 0.025169, 0.271039, 0.025169, 0.271039, 0.025169, 0.268975, 0.011186, 0.271039, 0.025169, 0.271039, 0.025169, 0.268975, 0.011186, 0.268975, 0.011186, 0.268975, 0.011186, 0.266609, 0.002797, 0.268975, 0.011186, 0.268975, 0.011186, 0.266609, 0.002797, 0.266609, 0.002797, 0.266609, 0.002797, 0.263941, 0, 0.266609, 0.002797, 0.266609, 0.002797, 0.263941, 0, 0.263941, 0, 0.263941, 0, 0.261263, 0.002797, 0.263941, 0, 0.263941, 0, 0.261263, 0.002797, 0.261263, 0.002797, 0.261263, 0.002797, 0.258891, 0.011186, 0.261263, 0.002797, 0.261263, 0.002797, 0.258891, 0.011186, 0.258891, 0.011186, 0.258891, 0.011186, 0.256827, 0.025169, 0.258891, 0.011186, 0.258891, 0.011186, 0.256827, 0.025169, 0.256827, 0.025169, 0.256827, 0.025169, 0.255070, 0.044744, 0.256827, 0.025169, 0.256827, 0.025169, 0.255070, 0.044744, 0.255070, 0.044744, 0.255070, 0.044744, 0.253667, 0.069238, 0.255070, 0.044744, 0.255070, 0.044744, 0.253667, 0.069238, 0.253667, 0.069238, 0.253667, 0.069238, 0.252664, 0.097976, 0.253667, 0.069238, 0.253667, 0.069238, 0.252664, 0.097976, 0.252664, 0.097976, 0.252664, 0.097976, 0.252063, 0.130957, 0.252664, 0.097976, 0.252664, 0.097976, 0.252063, 0.130957, 0.252063, 0.130957, 0.252063, 0.130957, 0.251862, 0.168182, 0.252063, 0.130957, 0.252063, 0.130957, 0.251862, 0.168182, 0.251862, 0.168182, 0.251862, 0.168182, 0.252063, 0.205522, 0.251862, 0.168182, 0.251862, 0.168182, 0.252063, 0.205522, 0.252063, 0.205522, 0.252063, 0.205522, 0.252664, 0.238565, 0.252063, 0.205522, 0.252063, 0.205522, 0.252664, 0.238565, 0.252664, 0.238565, 0.252664, 0.238565, 0.253667, 0.267312, 0.252664, 0.238565, 0.252664, 0.238565, 0.253667, 0.267312, 0.253667, 0.267312, 0.253667, 0.267312, 0.255070, 0.291761, 0.253667, 0.267312, 0.253667, 0.267312, 0.255070, 0.291761, 0.255070, 0.291761, 0.255070, 0.291761, 0.256827, 0.311275, 0.255070, 0.291761, 0.255070, 0.291761, 0.256827, 0.311275, 0.256827, 0.311275, 0.256827, 0.311275, 0.258891, 0.325213, 0.256827, 0.311275, 0.256827, 0.311275, 0.258891, 0.325213, 0.258891, 0.325213, 0.258891, 0.325213, 0.261263, 0.333576, 0.258891, 0.325213, 0.258891, 0.325213, 0.261263, 0.333576, 0.261263, 0.333576, 0.256547, 0.195215, 0.252063, 0.205522, 0.251862, 0.168182, 0.252063, 0.205522, 0.256547, 0.195215, 0.251862, 0.168182, 0.256926, 0.219212, 0.252063, 0.205522, 0.256547, 0.195215, 0.252063, 0.205522, 0.256926, 0.219212, 0.256547, 0.195215, 0.256926, 0.219212, 0.252664, 0.238565, 0.252063, 0.205522, 0.252664, 0.238565, 0.256926, 0.219212, 0.252063, 0.205522, 0.252664, 0.238565, 0.256926, 0.219212, 0.257558, 0.240172, 0.256926, 0.219212, 0.252664, 0.238565, 0.257558, 0.240172, 0.253667, 0.267312, 0.252664, 0.238565, 0.257558, 0.240172, 0.252664, 0.238565, 0.253667, 0.267312, 0.257558, 0.240172, 0.258442, 0.258097, 0.253667, 0.267312, 0.257558, 0.240172, 0.253667, 0.267312, 0.258442, 0.258097, 0.257558, 0.240172, 0.258442, 0.258097, 0.255070, 0.291761, 0.253667, 0.267312, 0.255070, 0.291761, 0.258442, 0.258097, 0.253667, 0.267312, 0.256827, 0.311275, 0.255070, 0.291761, 0.258442, 0.258097, 0.255070, 0.291761, 0.256827, 0.311275, 0.258442, 0.258097, 0.256827, 0.311275, 0.258442, 0.258097, 0.259544, 0.272452, 0.258442, 0.258097, 0.256827, 0.311275, 0.259544, 0.272452, 0.258891, 0.325213, 0.256827, 0.311275, 0.259544, 0.272452, 0.256827, 0.311275, 0.258891, 0.325213, 0.259544, 0.272452, 0.258891, 0.325213, 0.259544, 0.272452, 0.260827, 0.282706, 0.259544, 0.272452, 0.258891, 0.325213, 0.260827, 0.282706, 0.258891, 0.325213, 0.260827, 0.282706, 0.261263, 0.333576, 0.260827, 0.282706, 0.258891, 0.325213, 0.261263, 0.333576, 0.261263, 0.333576, 0.260827, 0.282706, 0.262293, 0.288858, 0.260827, 0.282706, 0.261263, 0.333576, 0.262293, 0.288858, 0.261263, 0.333576, 0.262293, 0.288858, 0.263941, 0.336364, 0.262293, 0.288858, 0.261263, 0.333576, 0.263941, 0.336364, 0.262293, 0.288858, 0.263941, 0.290909, 0.263941, 0.336364, 0.263941, 0.290909, 0.262293, 0.288858, 0.263941, 0.336364, 0.263941, 0.336364, 0.263941, 0.290909, 0.265578, 0.288841, 0.263941, 0.290909, 0.263941, 0.336364, 0.265578, 0.288841, 0.263941, 0.336364, 0.265578, 0.288841, 0.266609, 0.333576, 0.265578, 0.288841, 0.263941, 0.336364, 0.266609, 0.333576, 0.266609, 0.333576, 0.265578, 0.288841, 0.267037, 0.282635, 0.265578, 0.288841, 0.266609, 0.333576, 0.267037, 0.282635, 0.266609, 0.333576, 0.267037, 0.282635, 0.268975, 0.325213, 0.267037, 0.282635, 0.266609, 0.333576, 0.268975, 0.325213, 0.267037, 0.282635, 0.268316, 0.272292, 0.268975, 0.325213, 0.268316, 0.272292, 0.267037, 0.282635, 0.268975, 0.325213, 0.268975, 0.325213, 0.268316, 0.272292, 0.271039, 0.311275, 0.268316, 0.272292, 0.268975, 0.325213, 0.271039, 0.311275, 0.268316, 0.272292, 0.269416, 0.257813, 0.271039, 0.311275, 0.269416, 0.257813, 0.268316, 0.272292, 0.271039, 0.311275, 0.271039, 0.311275, 0.269416, 0.257813, 0.272800, 0.291761, 0.269416, 0.257813, 0.271039, 0.311275, 0.272800, 0.291761, 0.272800, 0.291761, 0.269416, 0.257813, 0.274208, 0.267312, 0.269416, 0.257813, 0.272800, 0.291761, 0.274208, 0.267312, 0.274208, 0.267312, 0.269416, 0.257813, 0.270300, 0.239799, 0.269416, 0.257813, 0.274208, 0.267312, 0.270300, 0.239799, 0.274208, 0.267312, 0.270300, 0.239799, 0.275214, 0.238565, 0.270300, 0.239799, 0.274208, 0.267312, 0.275214, 0.238565, 0.270300, 0.239799, 0.270932, 0.218857, 0.275214, 0.238565, 0.270932, 0.218857, 0.270300, 0.239799, 0.275214, 0.238565, 0.275214, 0.238565, 0.270932, 0.218857, 0.275818, 0.205522, 0.270932, 0.218857, 0.275214, 0.238565, 0.275818, 0.205522, 0.270932, 0.218857, 0.271311, 0.194984, 0.275818, 0.205522, 0.271311, 0.194984, 0.270932, 0.218857, 0.275818, 0.205522, 0.275818, 0.205522, 0.271311, 0.194984, 0.276019, 0.168182, 0.271311, 0.194984, 0.275818, 0.205522, 0.276019, 0.168182, 0.276019, 0.168182, 0.271311, 0.194984, 0.271437, 0.168182, 0.271311, 0.194984, 0.276019, 0.168182, 0.271437, 0.168182, 0.276019, 0.168182, 0.271437, 0.168182, 0.271311, 0.141264, 0.271437, 0.168182, 0.276019, 0.168182, 0.271311, 0.141264, 0.276019, 0.168182, 0.271311, 0.141264, 0.275818, 0.130957, 0.271311, 0.141264, 0.276019, 0.168182, 0.275818, 0.130957, 0.271311, 0.141264, 0.270932, 0.117330, 0.275818, 0.130957, 0.270932, 0.117330, 0.271311, 0.141264, 0.275818, 0.130957, 0.270932, 0.117330, 0.275214, 0.097976, 0.275818, 0.130957, 0.275214, 0.097976, 0.270932, 0.117330, 0.275818, 0.130957, 0.270932, 0.117330, 0.270300, 0.096378, 0.275214, 0.097976, 0.270300, 0.096378, 0.270932, 0.117330, 0.275214, 0.097976, 0.270300, 0.096378, 0.274208, 0.069238, 0.275214, 0.097976, 0.274208, 0.069238, 0.270300, 0.096378, 0.275214, 0.097976, 0.270300, 0.096378, 0.269416, 0.078409, 0.274208, 0.069238, 0.269416, 0.078409, 0.270300, 0.096378, 0.274208, 0.069238, 0.269416, 0.078409, 0.272800, 0.044744, 0.274208, 0.069238, 0.272800, 0.044744, 0.269416, 0.078409, 0.274208, 0.069238, 0.272800, 0.044744, 0.269416, 0.078409, 0.271039, 0.025169, 0.269416, 0.078409, 0.272800, 0.044744, 0.271039, 0.025169, 0.269416, 0.078409, 0.268316, 0.063991, 0.271039, 0.025169, 0.268316, 0.063991, 0.269416, 0.078409, 0.271039, 0.025169, 0.271039, 0.025169, 0.268316, 0.063991, 0.268975, 0.011186, 0.268316, 0.063991, 0.271039, 0.025169, 0.268975, 0.011186, 0.268316, 0.063991, 0.267037, 0.053693, 0.268975, 0.011186, 0.267037, 0.053693, 0.268316, 0.063991, 0.268975, 0.011186, 0.268975, 0.011186, 0.267037, 0.053693, 0.266609, 0.002797, 0.267037, 0.053693, 0.268975, 0.011186, 0.266609, 0.002797, 0.267037, 0.053693, 0.265578, 0.047514, 0.266609, 0.002797, 0.265578, 0.047514, 0.267037, 0.053693, 0.266609, 0.002797, 0.266609, 0.002797, 0.265578, 0.047514, 0.263941, 0, 0.265578, 0.047514, 0.266609, 0.002797, 0.263941, 0, 0.265578, 0.047514, 0.263941, 0.045455, 0.263941, 0, 0.263941, 0.045455, 0.265578, 0.047514, 0.263941, 0, 0.263941, 0.045455, 0.262283, 0.047505, 0.263941, 0, 0.262283, 0.047505, 0.263941, 0.045455, 0.263941, 0, 0.262283, 0.047505, 0.261263, 0.002797, 0.263941, 0, 0.261263, 0.002797, 0.262283, 0.047505, 0.263941, 0, 0.262283, 0.047505, 0.260812, 0.053658, 0.261263, 0.002797, 0.260812, 0.053658, 0.262283, 0.047505, 0.261263, 0.002797, 0.260812, 0.053658, 0.258891, 0.011186, 0.261263, 0.002797, 0.258891, 0.011186, 0.260812, 0.053658, 0.261263, 0.002797, 0.260812, 0.053658, 0.259528, 0.063912, 0.258891, 0.011186, 0.259528, 0.063912, 0.260812, 0.053658, 0.258891, 0.011186, 0.259528, 0.063912, 0.256827, 0.025169, 0.258891, 0.011186, 0.256827, 0.025169, 0.259528, 0.063912, 0.258891, 0.011186, 0.259528, 0.063912, 0.258430, 0.078267, 0.256827, 0.025169, 0.258430, 0.078267, 0.259528, 0.063912, 0.256827, 0.025169, 0.258430, 0.078267, 0.255070, 0.044744, 0.256827, 0.025169, 0.255070, 0.044744, 0.258430, 0.078267, 0.256827, 0.025169, 0.253667, 0.069238, 0.255070, 0.044744, 0.258430, 0.078267, 0.255070, 0.044744, 0.253667, 0.069238, 0.258430, 0.078267, 0.257551, 0.096191, 0.253667, 0.069238, 0.258430, 0.078267, 0.253667, 0.069238, 0.257551, 0.096191, 0.258430, 0.078267, 0.252664, 0.097976, 0.253667, 0.069238, 0.257551, 0.096191, 0.253667, 0.069238, 0.252664, 0.097976, 0.257551, 0.096191, 0.256923, 0.117152, 0.252664, 0.097976, 0.257551, 0.096191, 0.252664, 0.097976, 0.256923, 0.117152, 0.257551, 0.096191, 0.252063, 0.130957, 0.252664, 0.097976, 0.256923, 0.117152, 0.252664, 0.097976, 0.252063, 0.130957, 0.256923, 0.117152, 0.256547, 0.141149, 0.252063, 0.130957, 0.256923, 0.117152, 0.252063, 0.130957, 0.256547, 0.141149, 0.256923, 0.117152, 0.251862, 0.168182, 0.252063, 0.130957, 0.256547, 0.141149, 0.252063, 0.130957, 0.251862, 0.168182, 0.256547, 0.141149, 0.256421, 0.168182, 0.251862, 0.168182, 0.256547, 0.141149, 0.251862, 0.168182, 0.256421, 0.168182, 0.256547, 0.141149, 0.256547, 0.195215, 0.251862, 0.168182, 0.256421, 0.168182, 0.251862, 0.168182, 0.256547, 0.195215, 0.256421, 0.168182, 0.287346, 0.418182, 0.287346, 0.327273, 0.287346, 0.418182, 0.287346, 0.418182, 0.287346, 0.327273, 0.287346, 0.327273, 0.287346, 0.327273, 0.296252, 0.327273, 0.287346, 0.327273, 0.287346, 0.327273, 0.296252, 0.327273, 0.296252, 0.327273, 0.296252, 0.327273, 0.296252, 0.281818, 0.296252, 0.327273, 0.296252, 0.327273, 0.296252, 0.281818, 0.296252, 0.281818, 0.296252, 0.281818, 0.287346, 0.281818, 0.296252, 0.281818, 0.296252, 0.281818, 0.287346, 0.281818, 0.287346, 0.281818, 0.287346, 0.281818, 0.287346, 0.113920, 0.287346, 0.281818, 0.287346, 0.281818, 0.287346, 0.113920, 0.287346, 0.113920, 0.287346, 0.113920, 0.287401, 0.096715, 0.287346, 0.113920, 0.287346, 0.113920, 0.287401, 0.096715, 0.287401, 0.096715, 0.287401, 0.096715, 0.287566, 0.082884, 0.287401, 0.096715, 0.287401, 0.096715, 0.287566, 0.082884, 0.287566, 0.082884, 0.287566, 0.082884, 0.287841, 0.072425, 0.287566, 0.082884, 0.287566, 0.082884, 0.287841, 0.072425, 0.287841, 0.072425, 0.287841, 0.072425, 0.288227, 0.065341, 0.287841, 0.072425, 0.287841, 0.072425, 0.288227, 0.065341, 0.288227, 0.065341, 0.288227, 0.065341, 0.288781, 0.060618, 0.288227, 0.065341, 0.288227, 0.065341, 0.288781, 0.060618, 0.288781, 0.060618, 0.288781, 0.060618, 0.289564, 0.057244, 0.288781, 0.060618, 0.288781, 0.060618, 0.289564, 0.057244, 0.289564, 0.057244, 0.289564, 0.057244, 0.290573, 0.055220, 0.289564, 0.057244, 0.289564, 0.057244, 0.290573, 0.055220, 0.290573, 0.055220, 0.290573, 0.055220, 0.291811, 0.054545, 0.290573, 0.055220, 0.290573, 0.055220, 0.291811, 0.054545, 0.291811, 0.054545, 0.291811, 0.054545, 0.296252, 0.054545, 0.291811, 0.054545, 0.291811, 0.054545, 0.296252, 0.054545, 0.296252, 0.054545, 0.296252, 0.054545, 0.296252, 0.009091, 0.296252, 0.054545, 0.296252, 0.054545, 0.296252, 0.009091, 0.296252, 0.009091, 0.296252, 0.009091, 0.291811, 0.009091, 0.296252, 0.009091, 0.296252, 0.009091, 0.291811, 0.009091, 0.291811, 0.009091, 0.291811, 0.009091, 0.289502, 0.010502, 0.291811, 0.009091, 0.291811, 0.009091, 0.289502, 0.010502, 0.289502, 0.010502, 0.289502, 0.010502, 0.287581, 0.014737, 0.289502, 0.010502, 0.289502, 0.010502, 0.287581, 0.014737, 0.287581, 0.014737, 0.287581, 0.014737, 0.286047, 0.021795, 0.287581, 0.014737, 0.287581, 0.014737, 0.286047, 0.021795, 0.286047, 0.021795, 0.286047, 0.021795, 0.284902, 0.031676, 0.286047, 0.021795, 0.286047, 0.021795, 0.284902, 0.031676, 0.284902, 0.031676, 0.284902, 0.031676, 0.284069, 0.045286, 0.284902, 0.031676, 0.284902, 0.031676, 0.284069, 0.045286, 0.284069, 0.045286, 0.284069, 0.045286, 0.283474, 0.063530, 0.284069, 0.045286, 0.284069, 0.045286, 0.283474, 0.063530, 0.283474, 0.063530, 0.283474, 0.063530, 0.283117, 0.086408, 0.283474, 0.063530, 0.283474, 0.063530, 0.283117, 0.086408, 0.283117, 0.086408, 0.283117, 0.086408, 0.282998, 0.113920, 0.283117, 0.086408, 0.283117, 0.086408, 0.282998, 0.113920, 0.282998, 0.113920, 0.282998, 0.113920, 0.282998, 0.281818, 0.282998, 0.113920, 0.282998, 0.113920, 0.282998, 0.281818, 0.282998, 0.281818, 0.282998, 0.281818, 0.279826, 0.281818, 0.282998, 0.281818, 0.282998, 0.281818, 0.279826, 0.281818, 0.279826, 0.281818, 0.279826, 0.281818, 0.279826, 0.327273, 0.279826, 0.281818, 0.279826, 0.281818, 0.279826, 0.327273, 0.279826, 0.327273, 0.279826, 0.327273, 0.282998, 0.327273, 0.279826, 0.327273, 0.279826, 0.327273, 0.282998, 0.327273, 0.282998, 0.327273, 0.282998, 0.327273, 0.282998, 0.418182, 0.282998, 0.327273, 0.282998, 0.327273, 0.282998, 0.418182, 0.282998, 0.418182, 0.279826, 0.327273, 0.282998, 0.281818, 0.282998, 0.327273, 0.282998, 0.281818, 0.279826, 0.327273, 0.282998, 0.327273, 0.279826, 0.327273, 0.279826, 0.281818, 0.282998, 0.281818, 0.279826, 0.281818, 0.279826, 0.327273, 0.282998, 0.281818, 0.282998, 0.327273, 0.282998, 0.281818, 0.287346, 0.281818, 0.282998, 0.281818, 0.282998, 0.327273, 0.287346, 0.281818, 0.282998, 0.327273, 0.287346, 0.281818, 0.287346, 0.327273, 0.287346, 0.281818, 0.282998, 0.327273, 0.287346, 0.327273, 0.282998, 0.418182, 0.282998, 0.327273, 0.287346, 0.327273, 0.282998, 0.327273, 0.282998, 0.418182, 0.287346, 0.327273, 0.282998, 0.418182, 0.287346, 0.327273, 0.287346, 0.418182, 0.287346, 0.327273, 0.282998, 0.418182, 0.287346, 0.418182, 0.287346, 0.327273, 0.287346, 0.281818, 0.296252, 0.281818, 0.287346, 0.281818, 0.287346, 0.327273, 0.296252, 0.281818, 0.287346, 0.327273, 0.296252, 0.281818, 0.296252, 0.327273, 0.296252, 0.281818, 0.287346, 0.327273, 0.296252, 0.327273, 0.282998, 0.281818, 0.287346, 0.113920, 0.287346, 0.281818, 0.287346, 0.113920, 0.282998, 0.281818, 0.287346, 0.281818, 0.282998, 0.281818, 0.282998, 0.113920, 0.287346, 0.113920, 0.282998, 0.113920, 0.282998, 0.281818, 0.287346, 0.113920, 0.287346, 0.113920, 0.282998, 0.113920, 0.287401, 0.096715, 0.282998, 0.113920, 0.287346, 0.113920, 0.287401, 0.096715, 0.282998, 0.113920, 0.283117, 0.086408, 0.287401, 0.096715, 0.283117, 0.086408, 0.282998, 0.113920, 0.287401, 0.096715, 0.287401, 0.096715, 0.283117, 0.086408, 0.287566, 0.082884, 0.283117, 0.086408, 0.287401, 0.096715, 0.287566, 0.082884, 0.283117, 0.086408, 0.283474, 0.063530, 0.287566, 0.082884, 0.283474, 0.063530, 0.283117, 0.086408, 0.287566, 0.082884, 0.287566, 0.082884, 0.283474, 0.063530, 0.287841, 0.072425, 0.283474, 0.063530, 0.287566, 0.082884, 0.287841, 0.072425, 0.287841, 0.072425, 0.283474, 0.063530, 0.284069, 0.045286, 0.283474, 0.063530, 0.287841, 0.072425, 0.284069, 0.045286, 0.288227, 0.065341, 0.287841, 0.072425, 0.284069, 0.045286, 0.287841, 0.072425, 0.288227, 0.065341, 0.284069, 0.045286, 0.284069, 0.045286, 0.284902, 0.031676, 0.288227, 0.065341, 0.284902, 0.031676, 0.284069, 0.045286, 0.288227, 0.065341, 0.288227, 0.065341, 0.284902, 0.031676, 0.286047, 0.021795, 0.284902, 0.031676, 0.288227, 0.065341, 0.286047, 0.021795, 0.288781, 0.060618, 0.288227, 0.065341, 0.286047, 0.021795, 0.288227, 0.065341, 0.288781, 0.060618, 0.286047, 0.021795, 0.286047, 0.021795, 0.287581, 0.014737, 0.288781, 0.060618, 0.287581, 0.014737, 0.286047, 0.021795, 0.288781, 0.060618, 0.288781, 0.060618, 0.287581, 0.014737, 0.289564, 0.057244, 0.287581, 0.014737, 0.288781, 0.060618, 0.289564, 0.057244, 0.287581, 0.014737, 0.289502, 0.010502, 0.289564, 0.057244, 0.289502, 0.010502, 0.287581, 0.014737, 0.289564, 0.057244, 0.289564, 0.057244, 0.289502, 0.010502, 0.290573, 0.055220, 0.289502, 0.010502, 0.289564, 0.057244, 0.290573, 0.055220, 0.290573, 0.055220, 0.289502, 0.010502, 0.291811, 0.009091, 0.289502, 0.010502, 0.290573, 0.055220, 0.291811, 0.009091, 0.291811, 0.054545, 0.290573, 0.055220, 0.291811, 0.009091, 0.290573, 0.055220, 0.291811, 0.054545, 0.291811, 0.009091, 0.291811, 0.054545, 0.291811, 0.009091, 0.296252, 0.009091, 0.291811, 0.009091, 0.291811, 0.054545, 0.296252, 0.009091, 0.291811, 0.054545, 0.296252, 0.009091, 0.296252, 0.054545, 0.296252, 0.009091, 0.291811, 0.054545, 0.296252, 0.054545, 0.324380, 0.182955, 0.324380, 0.154545, 0.324380, 0.182955, 0.324380, 0.182955, 0.324380, 0.154545, 0.324380, 0.154545, 0.324380, 0.154545, 0.304500, 0.154545, 0.324380, 0.154545, 0.324380, 0.154545, 0.304500, 0.154545, 0.304500, 0.154545, 0.304500, 0.154545, 0.304774, 0.129497, 0.304500, 0.154545, 0.304500, 0.154545, 0.304774, 0.129497, 0.304774, 0.129497, 0.304774, 0.129497, 0.305314, 0.107617, 0.304774, 0.129497, 0.304774, 0.129497, 0.305314, 0.107617, 0.305314, 0.107617, 0.305314, 0.107617, 0.306119, 0.088907, 0.305314, 0.107617, 0.305314, 0.107617, 0.306119, 0.088907, 0.306119, 0.088907, 0.306119, 0.088907, 0.307191, 0.073366, 0.306119, 0.088907, 0.306119, 0.088907, 0.307191, 0.073366, 0.307191, 0.073366, 0.307191, 0.073366, 0.308513, 0.061155, 0.307191, 0.073366, 0.307191, 0.073366, 0.308513, 0.061155, 0.308513, 0.061155, 0.308513, 0.061155, 0.310072, 0.052433, 0.308513, 0.061155, 0.308513, 0.061155, 0.310072, 0.052433, 0.310072, 0.052433, 0.310072, 0.052433, 0.311868, 0.047199, 0.310072, 0.052433, 0.310072, 0.052433, 0.311868, 0.047199, 0.311868, 0.047199, 0.311868, 0.047199, 0.313900, 0.045455, 0.311868, 0.047199, 0.311868, 0.047199, 0.313900, 0.045455, 0.313900, 0.045455, 0.313900, 0.045455, 0.315136, 0.045881, 0.313900, 0.045455, 0.313900, 0.045455, 0.315136, 0.045881, 0.315136, 0.045881, 0.315136, 0.045881, 0.316352, 0.047159, 0.315136, 0.045881, 0.315136, 0.045881, 0.316352, 0.047159, 0.316352, 0.047159, 0.316352, 0.047159, 0.317550, 0.049290, 0.316352, 0.047159, 0.316352, 0.047159, 0.317550, 0.049290, 0.317550, 0.049290, 0.317550, 0.049290, 0.318729, 0.052273, 0.317550, 0.049290, 0.317550, 0.049290, 0.318729, 0.052273, 0.318729, 0.052273, 0.318729, 0.052273, 0.319896, 0.056108, 0.318729, 0.052273, 0.318729, 0.052273, 0.319896, 0.056108, 0.319896, 0.056108, 0.319896, 0.056108, 0.321058, 0.060795, 0.319896, 0.056108, 0.319896, 0.056108, 0.321058, 0.060795, 0.321058, 0.060795, 0.321058, 0.060795, 0.322216, 0.066335, 0.321058, 0.060795, 0.321058, 0.060795, 0.322216, 0.066335, 0.322216, 0.066335, 0.322216, 0.066335, 0.323370, 0.072727, 0.322216, 0.066335, 0.322216, 0.066335, 0.323370, 0.072727, 0.323370, 0.072727, 0.323370, 0.072727, 0.323370, 0.027273, 0.323370, 0.072727, 0.323370, 0.072727, 0.323370, 0.027273, 0.323370, 0.027273, 0.323370, 0.027273, 0.322199, 0.020996, 0.323370, 0.027273, 0.323370, 0.027273, 0.322199, 0.020996, 0.322199, 0.020996, 0.322199, 0.020996, 0.321014, 0.015518, 0.322199, 0.020996, 0.322199, 0.020996, 0.321014, 0.015518, 0.321014, 0.015518, 0.321014, 0.015518, 0.319814, 0.010840, 0.321014, 0.015518, 0.321014, 0.015518, 0.319814, 0.010840, 0.319814, 0.010840, 0.319814, 0.010840, 0.318599, 0.006960, 0.319814, 0.010840, 0.319814, 0.010840, 0.318599, 0.006960, 0.318599, 0.006960, 0.318599, 0.006960, 0.317373, 0.003915, 0.318599, 0.006960, 0.318599, 0.006960, 0.317373, 0.003915, 0.317373, 0.003915, 0.317373, 0.003915, 0.316138, 0.001740, 0.317373, 0.003915, 0.317373, 0.003915, 0.316138, 0.001740, 0.316138, 0.001740, 0.316138, 0.001740, 0.314894, 0.000435, 0.316138, 0.001740, 0.316138, 0.001740, 0.314894, 0.000435, 0.314894, 0.000435, 0.314894, 0.000435, 0.313641, 0, 0.314894, 0.000435, 0.314894, 0.000435, 0.313641, 0, 0.313641, 0, 0.313641, 0, 0.310656, 0.002788, 0.313641, 0, 0.313641, 0, 0.310656, 0.002788, 0.310656, 0.002788, 0.310656, 0.002788, 0.307998, 0.011151, 0.310656, 0.002788, 0.310656, 0.002788, 0.307998, 0.011151, 0.307998, 0.011151, 0.307998, 0.011151, 0.305668, 0.025089, 0.307998, 0.011151, 0.307998, 0.011151, 0.305668, 0.025089, 0.305668, 0.025089, 0.305668, 0.025089, 0.303666, 0.044602, 0.305668, 0.025089, 0.305668, 0.025089, 0.303666, 0.044602, 0.303666, 0.044602, 0.303666, 0.044602, 0.302057, 0.068874, 0.303666, 0.044602, 0.303666, 0.044602, 0.302057, 0.068874, 0.302057, 0.068874, 0.302057, 0.068874, 0.300908, 0.097088, 0.302057, 0.068874, 0.302057, 0.068874, 0.300908, 0.097088, 0.300908, 0.097088, 0.300908, 0.097088, 0.300218, 0.129244, 0.300908, 0.097088, 0.300908, 0.097088, 0.300218, 0.129244, 0.300218, 0.129244, 0.300218, 0.129244, 0.299988, 0.165341, 0.300218, 0.129244, 0.300218, 0.129244, 0.299988, 0.165341, 0.299988, 0.165341, 0.299988, 0.165341, 0.300206, 0.202654, 0.299988, 0.165341, 0.299988, 0.165341, 0.300206, 0.202654, 0.300206, 0.202654, 0.300206, 0.202654, 0.300861, 0.235902, 0.300206, 0.202654, 0.300206, 0.202654, 0.300861, 0.235902, 0.300861, 0.235902, 0.300861, 0.235902, 0.301951, 0.265083, 0.300861, 0.235902, 0.300861, 0.235902, 0.301951, 0.265083, 0.301951, 0.265083, 0.301951, 0.265083, 0.303478, 0.290199, 0.301951, 0.265083, 0.301951, 0.265083, 0.303478, 0.290199, 0.303478, 0.290199, 0.303478, 0.290199, 0.305375, 0.310396, 0.303478, 0.290199, 0.303478, 0.290199, 0.305375, 0.310396, 0.305375, 0.310396, 0.305375, 0.310396, 0.307575, 0.324822, 0.305375, 0.310396, 0.305375, 0.310396, 0.307575, 0.324822, 0.307575, 0.324822, 0.307575, 0.324822, 0.310080, 0.333478, 0.307575, 0.324822, 0.307575, 0.324822, 0.310080, 0.333478, 0.310080, 0.333478, 0.310080, 0.333478, 0.312889, 0.336364, 0.310080, 0.333478, 0.310080, 0.333478, 0.312889, 0.336364, 0.312889, 0.336364, 0.312889, 0.336364, 0.315406, 0.333785, 0.312889, 0.336364, 0.312889, 0.336364, 0.315406, 0.333785, 0.315406, 0.333785, 0.315406, 0.333785, 0.317645, 0.326048, 0.315406, 0.333785, 0.315406, 0.333785, 0.317645, 0.326048, 0.317645, 0.326048, 0.317645, 0.326048, 0.319606, 0.313153, 0.317645, 0.326048, 0.317645, 0.326048, 0.319606, 0.313153, 0.319606, 0.313153, 0.319606, 0.313153, 0.321290, 0.295099, 0.319606, 0.313153, 0.319606, 0.313153, 0.321290, 0.295099, 0.321290, 0.295099, 0.321290, 0.295099, 0.322642, 0.272616, 0.321290, 0.295099, 0.321290, 0.295099, 0.322642, 0.272616, 0.322642, 0.272616, 0.322642, 0.272616, 0.323608, 0.246431, 0.322642, 0.272616, 0.322642, 0.272616, 0.323608, 0.246431, 0.323608, 0.246431, 0.323608, 0.246431, 0.324187, 0.216544, 0.323608, 0.246431, 0.323608, 0.246431, 0.324187, 0.216544, 0.324187, 0.216544, 0.300206, 0.202654, 0.299988, 0.165341, 0.304641, 0.199716, 0.299988, 0.165341, 0.300206, 0.202654, 0.304641, 0.199716, 0.300861, 0.235902, 0.300206, 0.202654, 0.304641, 0.199716, 0.300206, 0.202654, 0.300861, 0.235902, 0.304641, 0.199716, 0.300861, 0.235902, 0.304641, 0.199716, 0.304921, 0.220224, 0.304641, 0.199716, 0.300861, 0.235902, 0.304921, 0.220224, 0.305431, 0.238281, 0.300861, 0.235902, 0.304921, 0.220224, 0.300861, 0.235902, 0.305431, 0.238281, 0.304921, 0.220224, 0.305431, 0.238281, 0.301951, 0.265083, 0.300861, 0.235902, 0.301951, 0.265083, 0.305431, 0.238281, 0.300861, 0.235902, 0.301951, 0.265083, 0.305431, 0.238281, 0.306172, 0.253888, 0.305431, 0.238281, 0.301951, 0.265083, 0.306172, 0.253888, 0.303478, 0.290199, 0.301951, 0.265083, 0.306172, 0.253888, 0.301951, 0.265083, 0.303478, 0.290199, 0.306172, 0.253888, 0.303478, 0.290199, 0.306172, 0.253888, 0.307144, 0.267045, 0.306172, 0.253888, 0.303478, 0.290199, 0.307144, 0.267045, 0.305375, 0.310396, 0.303478, 0.290199, 0.307144, 0.267045, 0.303478, 0.290199, 0.305375, 0.310396, 0.307144, 0.267045, 0.305375, 0.310396, 0.307144, 0.267045, 0.308321, 0.277486, 0.307144, 0.267045, 0.305375, 0.310396, 0.308321, 0.277486, 0.307575, 0.324822, 0.305375, 0.310396, 0.308321, 0.277486, 0.305375, 0.310396, 0.307575, 0.324822, 0.308321, 0.277486, 0.307575, 0.324822, 0.308321, 0.277486, 0.309679, 0.284943, 0.308321, 0.277486, 0.307575, 0.324822, 0.309679, 0.284943, 0.307575, 0.324822, 0.309679, 0.284943, 0.310080, 0.333478, 0.309679, 0.284943, 0.307575, 0.324822, 0.310080, 0.333478, 0.310080, 0.333478, 0.309679, 0.284943, 0.311217, 0.289418, 0.309679, 0.284943, 0.310080, 0.333478, 0.311217, 0.289418, 0.310080, 0.333478, 0.311217, 0.289418, 0.312889, 0.336364, 0.311217, 0.289418, 0.310080, 0.333478, 0.312889, 0.336364, 0.312889, 0.336364, 0.311217, 0.289418, 0.312936, 0.290909, 0.311217, 0.289418, 0.312889, 0.336364, 0.312936, 0.290909, 0.312889, 0.336364, 0.312936, 0.290909, 0.314456, 0.289364, 0.312936, 0.290909, 0.312889, 0.336364, 0.314456, 0.289364, 0.312889, 0.336364, 0.314456, 0.289364, 0.315406, 0.333785, 0.314456, 0.289364, 0.312889, 0.336364, 0.315406, 0.333785, 0.315406, 0.333785, 0.314456, 0.289364, 0.315818, 0.284730, 0.314456, 0.289364, 0.315406, 0.333785, 0.315818, 0.284730, 0.315406, 0.333785, 0.315818, 0.284730, 0.317645, 0.326048, 0.315818, 0.284730, 0.315406, 0.333785, 0.317645, 0.326048, 0.315818, 0.284730, 0.317023, 0.277006, 0.317645, 0.326048, 0.317023, 0.277006, 0.315818, 0.284730, 0.317645, 0.326048, 0.317645, 0.326048, 0.317023, 0.277006, 0.319606, 0.313153, 0.317023, 0.277006, 0.317645, 0.326048, 0.319606, 0.313153, 0.317023, 0.277006, 0.318071, 0.266193, 0.319606, 0.313153, 0.318071, 0.266193, 0.317023, 0.277006, 0.319606, 0.313153, 0.319606, 0.313153, 0.318071, 0.266193, 0.321290, 0.295099, 0.318071, 0.266193, 0.319606, 0.313153, 0.321290, 0.295099, 0.318071, 0.266193, 0.318922, 0.252788, 0.321290, 0.295099, 0.318922, 0.252788, 0.318071, 0.266193, 0.321290, 0.295099, 0.321290, 0.295099, 0.318922, 0.252788, 0.322642, 0.272616, 0.318922, 0.252788, 0.321290, 0.295099, 0.322642, 0.272616, 0.318922, 0.252788, 0.319536, 0.237287, 0.322642, 0.272616, 0.319536, 0.237287, 0.318922, 0.252788, 0.322642, 0.272616, 0.322642, 0.272616, 0.319536, 0.237287, 0.323608, 0.246431, 0.319536, 0.237287, 0.322642, 0.272616, 0.323608, 0.246431, 0.319536, 0.237287, 0.319915, 0.219691, 0.323608, 0.246431, 0.319915, 0.219691, 0.319536, 0.237287, 0.323608, 0.246431, 0.323608, 0.246431, 0.319915, 0.219691, 0.324187, 0.216544, 0.319915, 0.219691, 0.323608, 0.246431, 0.324187, 0.216544, 0.319915, 0.219691, 0.320056, 0.200000, 0.324187, 0.216544, 0.320056, 0.200000, 0.319915, 0.219691, 0.324187, 0.216544, 0.324187, 0.216544, 0.320056, 0.200000, 0.324380, 0.182955, 0.320056, 0.200000, 0.324187, 0.216544, 0.324380, 0.182955, 0.324380, 0.154545, 0.324380, 0.182955, 0.320056, 0.200000, 0.324380, 0.182955, 0.324380, 0.154545, 0.320056, 0.200000, 0.320056, 0.200000, 0.304500, 0.154545, 0.324380, 0.154545, 0.304500, 0.154545, 0.320056, 0.200000, 0.324380, 0.154545, 0.320056, 0.200000, 0.304641, 0.199716, 0.304500, 0.154545, 0.304641, 0.199716, 0.320056, 0.200000, 0.304500, 0.154545, 0.299988, 0.165341, 0.304500, 0.154545, 0.304641, 0.199716, 0.304500, 0.154545, 0.299988, 0.165341, 0.304641, 0.199716, 0.299988, 0.165341, 0.300218, 0.129244, 0.304500, 0.154545, 0.300218, 0.129244, 0.299988, 0.165341, 0.304500, 0.154545, 0.304500, 0.154545, 0.300218, 0.129244, 0.304774, 0.129497, 0.300218, 0.129244, 0.304500, 0.154545, 0.304774, 0.129497, 0.300218, 0.129244, 0.300908, 0.097088, 0.304774, 0.129497, 0.300908, 0.097088, 0.300218, 0.129244, 0.304774, 0.129497, 0.304774, 0.129497, 0.300908, 0.097088, 0.305314, 0.107617, 0.300908, 0.097088, 0.304774, 0.129497, 0.305314, 0.107617, 0.300908, 0.097088, 0.302057, 0.068874, 0.305314, 0.107617, 0.302057, 0.068874, 0.300908, 0.097088, 0.305314, 0.107617, 0.305314, 0.107617, 0.302057, 0.068874, 0.306119, 0.088907, 0.302057, 0.068874, 0.305314, 0.107617, 0.306119, 0.088907, 0.302057, 0.068874, 0.303666, 0.044602, 0.306119, 0.088907, 0.303666, 0.044602, 0.302057, 0.068874, 0.306119, 0.088907, 0.306119, 0.088907, 0.303666, 0.044602, 0.307191, 0.073366, 0.303666, 0.044602, 0.306119, 0.088907, 0.307191, 0.073366, 0.303666, 0.044602, 0.305668, 0.025089, 0.307191, 0.073366, 0.305668, 0.025089, 0.303666, 0.044602, 0.307191, 0.073366, 0.307191, 0.073366, 0.305668, 0.025089, 0.308513, 0.061155, 0.305668, 0.025089, 0.307191, 0.073366, 0.308513, 0.061155, 0.308513, 0.061155, 0.305668, 0.025089, 0.307998, 0.011151, 0.305668, 0.025089, 0.308513, 0.061155, 0.307998, 0.011151, 0.310072, 0.052433, 0.308513, 0.061155, 0.307998, 0.011151, 0.308513, 0.061155, 0.310072, 0.052433, 0.307998, 0.011151, 0.310072, 0.052433, 0.307998, 0.011151, 0.310656, 0.002788, 0.307998, 0.011151, 0.310072, 0.052433, 0.310656, 0.002788, 0.311868, 0.047199, 0.310072, 0.052433, 0.310656, 0.002788, 0.310072, 0.052433, 0.311868, 0.047199, 0.310656, 0.002788, 0.311868, 0.047199, 0.310656, 0.002788, 0.313641, 0, 0.310656, 0.002788, 0.311868, 0.047199, 0.313641, 0, 0.313900, 0.045455, 0.311868, 0.047199, 0.313641, 0, 0.311868, 0.047199, 0.313900, 0.045455, 0.313641, 0, 0.313900, 0.045455, 0.313641, 0, 0.314894, 0.000435, 0.313641, 0, 0.313900, 0.045455, 0.314894, 0.000435, 0.313900, 0.045455, 0.314894, 0.000435, 0.315136, 0.045881, 0.314894, 0.000435, 0.313900, 0.045455, 0.315136, 0.045881, 0.315136, 0.045881, 0.314894, 0.000435, 0.316138, 0.001740, 0.314894, 0.000435, 0.315136, 0.045881, 0.316138, 0.001740, 0.315136, 0.045881, 0.316138, 0.001740, 0.316352, 0.047159, 0.316138, 0.001740, 0.315136, 0.045881, 0.316352, 0.047159, 0.316352, 0.047159, 0.316138, 0.001740, 0.317373, 0.003915, 0.316138, 0.001740, 0.316352, 0.047159, 0.317373, 0.003915, 0.316352, 0.047159, 0.317373, 0.003915, 0.317550, 0.049290, 0.317373, 0.003915, 0.316352, 0.047159, 0.317550, 0.049290, 0.317550, 0.049290, 0.317373, 0.003915, 0.318599, 0.006960, 0.317373, 0.003915, 0.317550, 0.049290, 0.318599, 0.006960, 0.317550, 0.049290, 0.318599, 0.006960, 0.318729, 0.052273, 0.318599, 0.006960, 0.317550, 0.049290, 0.318729, 0.052273, 0.318729, 0.052273, 0.318599, 0.006960, 0.319814, 0.010840, 0.318599, 0.006960, 0.318729, 0.052273, 0.319814, 0.010840, 0.319896, 0.056108, 0.318729, 0.052273, 0.319814, 0.010840, 0.318729, 0.052273, 0.319896, 0.056108, 0.319814, 0.010840, 0.321014, 0.015518, 0.319896, 0.056108, 0.319814, 0.010840, 0.319896, 0.056108, 0.321014, 0.015518, 0.319814, 0.010840, 0.319896, 0.056108, 0.321014, 0.015518, 0.322199, 0.020996, 0.321014, 0.015518, 0.319896, 0.056108, 0.322199, 0.020996, 0.321058, 0.060795, 0.319896, 0.056108, 0.322199, 0.020996, 0.319896, 0.056108, 0.321058, 0.060795, 0.322199, 0.020996, 0.321058, 0.060795, 0.322199, 0.020996, 0.323370, 0.027273, 0.322199, 0.020996, 0.321058, 0.060795, 0.323370, 0.027273, 0.322216, 0.066335, 0.321058, 0.060795, 0.323370, 0.027273, 0.321058, 0.060795, 0.322216, 0.066335, 0.323370, 0.027273, 0.322216, 0.066335, 0.323370, 0.027273, 0.323370, 0.072727, 0.323370, 0.027273, 0.322216, 0.066335, 0.323370, 0.072727, 0.320056, 0.200000, 0.319915, 0.219691, 0.320056, 0.200000, 0.320056, 0.200000, 0.319915, 0.219691, 0.319915, 0.219691, 0.319915, 0.219691, 0.319536, 0.237287, 0.319915, 0.219691, 0.319915, 0.219691, 0.319536, 0.237287, 0.319536, 0.237287, 0.319536, 0.237287, 0.318922, 0.252788, 0.319536, 0.237287, 0.319536, 0.237287, 0.318922, 0.252788, 0.318922, 0.252788, 0.318922, 0.252788, 0.318071, 0.266193, 0.318922, 0.252788, 0.318922, 0.252788, 0.318071, 0.266193, 0.318071, 0.266193, 0.318071, 0.266193, 0.317023, 0.277006, 0.318071, 0.266193, 0.318071, 0.266193, 0.317023, 0.277006, 0.317023, 0.277006, 0.317023, 0.277006, 0.315818, 0.284730, 0.317023, 0.277006, 0.317023, 0.277006, 0.315818, 0.284730, 0.315818, 0.284730, 0.315818, 0.284730, 0.314456, 0.289364, 0.315818, 0.284730, 0.315818, 0.284730, 0.314456, 0.289364, 0.314456, 0.289364, 0.314456, 0.289364, 0.312936, 0.290909, 0.314456, 0.289364, 0.314456, 0.289364, 0.312936, 0.290909, 0.312936, 0.290909, 0.312936, 0.290909, 0.311217, 0.289418, 0.312936, 0.290909, 0.312936, 0.290909, 0.311217, 0.289418, 0.311217, 0.289418, 0.311217, 0.289418, 0.309679, 0.284943, 0.311217, 0.289418, 0.311217, 0.289418, 0.309679, 0.284943, 0.309679, 0.284943, 0.309679, 0.284943, 0.308321, 0.277486, 0.309679, 0.284943, 0.309679, 0.284943, 0.308321, 0.277486, 0.308321, 0.277486, 0.308321, 0.277486, 0.307144, 0.267045, 0.308321, 0.277486, 0.308321, 0.277486, 0.307144, 0.267045, 0.307144, 0.267045, 0.307144, 0.267045, 0.306172, 0.253888, 0.307144, 0.267045, 0.307144, 0.267045, 0.306172, 0.253888, 0.306172, 0.253888, 0.306172, 0.253888, 0.305431, 0.238281, 0.306172, 0.253888, 0.306172, 0.253888, 0.305431, 0.238281, 0.305431, 0.238281, 0.305431, 0.238281, 0.304921, 0.220224, 0.305431, 0.238281, 0.305431, 0.238281, 0.304921, 0.220224, 0.304921, 0.220224, 0.304921, 0.220224, 0.304641, 0.199716, 0.304921, 0.220224, 0.304921, 0.220224, 0.304641, 0.199716, 0.304641, 0.199716, 0.350135, 0.309091, 0.350135, 0.263636, 0.350135, 0.309091, 0.350135, 0.309091, 0.350135, 0.263636, 0.350135, 0.263636, 0.350135, 0.263636, 0.349218, 0.270028, 0.350135, 0.263636, 0.350135, 0.263636, 0.349218, 0.270028, 0.349218, 0.270028, 0.349218, 0.270028, 0.348299, 0.275568, 0.349218, 0.270028, 0.349218, 0.270028, 0.348299, 0.275568, 0.348299, 0.275568, 0.348299, 0.275568, 0.347379, 0.280256, 0.348299, 0.275568, 0.348299, 0.275568, 0.347379, 0.280256, 0.347379, 0.280256, 0.347379, 0.280256, 0.346458, 0.284091, 0.347379, 0.280256, 0.347379, 0.280256, 0.346458, 0.284091, 0.346458, 0.284091, 0.346458, 0.284091, 0.345533, 0.287074, 0.346458, 0.284091, 0.346458, 0.284091, 0.345533, 0.287074, 0.345533, 0.287074, 0.345533, 0.287074, 0.344604, 0.289205, 0.345533, 0.287074, 0.345533, 0.287074, 0.344604, 0.289205, 0.344604, 0.289205, 0.344604, 0.289205, 0.343671, 0.290483, 0.344604, 0.289205, 0.344604, 0.289205, 0.343671, 0.290483, 0.343671, 0.290483, 0.343671, 0.290483, 0.342733, 0.290909, 0.343671, 0.290483, 0.343671, 0.290483, 0.342733, 0.290909, 0.342733, 0.290909, 0.342733, 0.290909, 0.340747, 0.288894, 0.342733, 0.290909, 0.342733, 0.290909, 0.340747, 0.288894, 0.340747, 0.288894, 0.340747, 0.288894, 0.338997, 0.282848, 0.340747, 0.288894, 0.340747, 0.288894, 0.338997, 0.282848, 0.338997, 0.282848, 0.338997, 0.282848, 0.337481, 0.272772, 0.338997, 0.282848, 0.338997, 0.282848, 0.337481, 0.272772, 0.337481, 0.272772, 0.337481, 0.272772, 0.336200, 0.258665, 0.337481, 0.272772, 0.337481, 0.272772, 0.336200, 0.258665, 0.336200, 0.258665, 0.336200, 0.258665, 0.335182, 0.240918, 0.336200, 0.258665, 0.336200, 0.258665, 0.335182, 0.240918, 0.335182, 0.240918, 0.335182, 0.240918, 0.334455, 0.219922, 0.335182, 0.240918, 0.335182, 0.240918, 0.334455, 0.219922, 0.334455, 0.219922, 0.334455, 0.219922, 0.334019, 0.195676, 0.334455, 0.219922, 0.334455, 0.219922, 0.334019, 0.195676, 0.334019, 0.195676, 0.334019, 0.195676, 0.333874, 0.168182, 0.334019, 0.195676, 0.334019, 0.195676, 0.333874, 0.168182, 0.333874, 0.168182, 0.333874, 0.168182, 0.334019, 0.140687, 0.333874, 0.168182, 0.333874, 0.168182, 0.334019, 0.140687, 0.334019, 0.140687, 0.334019, 0.140687, 0.334455, 0.116442, 0.334019, 0.140687, 0.334019, 0.140687, 0.334455, 0.116442, 0.334455, 0.116442, 0.334455, 0.116442, 0.335182, 0.095446, 0.334455, 0.116442, 0.334455, 0.116442, 0.335182, 0.095446, 0.335182, 0.095446, 0.335182, 0.095446, 0.336200, 0.077699, 0.335182, 0.095446, 0.335182, 0.095446, 0.336200, 0.077699, 0.336200, 0.077699, 0.336200, 0.077699, 0.337481, 0.063592, 0.336200, 0.077699, 0.336200, 0.077699, 0.337481, 0.063592, 0.337481, 0.063592, 0.337481, 0.063592, 0.338997, 0.053516, 0.337481, 0.063592, 0.337481, 0.063592, 0.338997, 0.053516, 0.338997, 0.053516, 0.338997, 0.053516, 0.340747, 0.047470, 0.338997, 0.053516, 0.338997, 0.053516, 0.340747, 0.047470, 0.340747, 0.047470, 0.340747, 0.047470, 0.342733, 0.045455, 0.340747, 0.047470, 0.340747, 0.047470, 0.342733, 0.045455, 0.342733, 0.045455, 0.342733, 0.045455, 0.343671, 0.045881, 0.342733, 0.045455, 0.342733, 0.045455, 0.343671, 0.045881, 0.343671, 0.045881, 0.343671, 0.045881, 0.344604, 0.047159, 0.343671, 0.045881, 0.343671, 0.045881, 0.344604, 0.047159, 0.344604, 0.047159, 0.344604, 0.047159, 0.345533, 0.049290, 0.344604, 0.047159, 0.344604, 0.047159, 0.345533, 0.049290, 0.345533, 0.049290, 0.345533, 0.049290, 0.346458, 0.052273, 0.345533, 0.049290, 0.345533, 0.049290, 0.346458, 0.052273, 0.346458, 0.052273, 0.346458, 0.052273, 0.347379, 0.056108, 0.346458, 0.052273, 0.346458, 0.052273, 0.347379, 0.056108, 0.347379, 0.056108, 0.347379, 0.056108, 0.348299, 0.060795, 0.347379, 0.056108, 0.347379, 0.056108, 0.348299, 0.060795, 0.348299, 0.060795, 0.348299, 0.060795, 0.349218, 0.066335, 0.348299, 0.060795, 0.348299, 0.060795, 0.349218, 0.066335, 0.349218, 0.066335, 0.349218, 0.066335, 0.350135, 0.072727, 0.349218, 0.066335, 0.349218, 0.066335, 0.350135, 0.072727, 0.350135, 0.072727, 0.350135, 0.072727, 0.350135, 0.027273, 0.350135, 0.072727, 0.350135, 0.072727, 0.350135, 0.027273, 0.350135, 0.027273, 0.350135, 0.027273, 0.349222, 0.020881, 0.350135, 0.027273, 0.350135, 0.027273, 0.349222, 0.020881, 0.349222, 0.020881, 0.349222, 0.020881, 0.348293, 0.015341, 0.349222, 0.020881, 0.349222, 0.020881, 0.348293, 0.015341, 0.348293, 0.015341, 0.348293, 0.015341, 0.347348, 0.010653, 0.348293, 0.015341, 0.348293, 0.015341, 0.347348, 0.010653, 0.347348, 0.010653, 0.347348, 0.010653, 0.346387, 0.006818, 0.347348, 0.010653, 0.347348, 0.010653, 0.346387, 0.006818, 0.346387, 0.006818, 0.346387, 0.006818, 0.345402, 0.003835, 0.346387, 0.006818, 0.346387, 0.006818, 0.345402, 0.003835, 0.345402, 0.003835, 0.345402, 0.003835, 0.344387, 0.001705, 0.345402, 0.003835, 0.345402, 0.003835, 0.344387, 0.001705, 0.344387, 0.001705, 0.344387, 0.001705, 0.343340, 0.000426, 0.344387, 0.001705, 0.344387, 0.001705, 0.343340, 0.000426, 0.343340, 0.000426, 0.343340, 0.000426, 0.342263, 0, 0.343340, 0.000426, 0.343340, 0.000426, 0.342263, 0, 0.342263, 0, 0.342263, 0, 0.339443, 0.002841, 0.342263, 0, 0.342263, 0, 0.339443, 0.002841, 0.339443, 0.002841, 0.339443, 0.002841, 0.336929, 0.011364, 0.339443, 0.002841, 0.339443, 0.002841, 0.336929, 0.011364, 0.336929, 0.011364, 0.336929, 0.011364, 0.334720, 0.025568, 0.336929, 0.011364, 0.336929, 0.011364, 0.334720, 0.025568, 0.334720, 0.025568, 0.334720, 0.025568, 0.332816, 0.045455, 0.334720, 0.025568, 0.334720, 0.025568, 0.332816, 0.045455, 0.332816, 0.045455, 0.332816, 0.045455, 0.331284, 0.070170, 0.332816, 0.045455, 0.332816, 0.045455, 0.331284, 0.070170, 0.331284, 0.070170, 0.331284, 0.070170, 0.330190, 0.098864, 0.331284, 0.070170, 0.331284, 0.070170, 0.330190, 0.098864, 0.330190, 0.098864, 0.330190, 0.098864, 0.329534, 0.131534, 0.330190, 0.098864, 0.330190, 0.098864, 0.329534, 0.131534, 0.329534, 0.131534, 0.329534, 0.131534, 0.329315, 0.168182, 0.329534, 0.131534, 0.329534, 0.131534, 0.329315, 0.168182, 0.329315, 0.168182, 0.329315, 0.168182, 0.329536, 0.205291, 0.329315, 0.168182, 0.329315, 0.168182, 0.329536, 0.205291, 0.329536, 0.205291, 0.329536, 0.205291, 0.330199, 0.238210, 0.329536, 0.205291, 0.329536, 0.205291, 0.330199, 0.238210, 0.330199, 0.238210, 0.330199, 0.238210, 0.331304, 0.266939, 0.330199, 0.238210, 0.330199, 0.238210, 0.331304, 0.266939, 0.331304, 0.266939, 0.331304, 0.266939, 0.332852, 0.291477, 0.331304, 0.266939, 0.331304, 0.266939, 0.332852, 0.291477, 0.332852, 0.291477, 0.332852, 0.291477, 0.334784, 0.311115, 0.332852, 0.291477, 0.332852, 0.291477, 0.334784, 0.311115, 0.334784, 0.311115, 0.334784, 0.311115, 0.337043, 0.325142, 0.334784, 0.311115, 0.334784, 0.311115, 0.337043, 0.325142, 0.337043, 0.325142, 0.337043, 0.325142, 0.339630, 0.333558, 0.337043, 0.325142, 0.337043, 0.325142, 0.339630, 0.333558, 0.339630, 0.333558, 0.339630, 0.333558, 0.342545, 0.336364, 0.339630, 0.333558, 0.339630, 0.333558, 0.342545, 0.336364, 0.342545, 0.336364, 0.342545, 0.336364, 0.343538, 0.335938, 0.342545, 0.336364, 0.342545, 0.336364, 0.343538, 0.335938, 0.343538, 0.335938, 0.343538, 0.335938, 0.344519, 0.334659, 0.343538, 0.335938, 0.343538, 0.335938, 0.344519, 0.334659, 0.344519, 0.334659, 0.344519, 0.334659, 0.345488, 0.332528, 0.344519, 0.334659, 0.344519, 0.334659, 0.345488, 0.332528, 0.345488, 0.332528, 0.345488, 0.332528, 0.346446, 0.329545, 0.345488, 0.332528, 0.345488, 0.332528, 0.346446, 0.329545, 0.346446, 0.329545, 0.346446, 0.329545, 0.347390, 0.325710, 0.346446, 0.329545, 0.346446, 0.329545, 0.347390, 0.325710, 0.347390, 0.325710, 0.347390, 0.325710, 0.348320, 0.321023, 0.347390, 0.325710, 0.347390, 0.325710, 0.348320, 0.321023, 0.348320, 0.321023, 0.348320, 0.321023, 0.349235, 0.315483, 0.348320, 0.321023, 0.348320, 0.321023, 0.349235, 0.315483, 0.349235, 0.315483, 0.334019, 0.195676, 0.329536, 0.205291, 0.329315, 0.168182, 0.329536, 0.205291, 0.334019, 0.195676, 0.329315, 0.168182, 0.334455, 0.219922, 0.329536, 0.205291, 0.334019, 0.195676, 0.329536, 0.205291, 0.334455, 0.219922, 0.334019, 0.195676, 0.334455, 0.219922, 0.330199, 0.238210, 0.329536, 0.205291, 0.330199, 0.238210, 0.334455, 0.219922, 0.329536, 0.205291, 0.335182, 0.240918, 0.330199, 0.238210, 0.334455, 0.219922, 0.330199, 0.238210, 0.335182, 0.240918, 0.334455, 0.219922, 0.335182, 0.240918, 0.331304, 0.266939, 0.330199, 0.238210, 0.331304, 0.266939, 0.335182, 0.240918, 0.330199, 0.238210, 0.332852, 0.291477, 0.331304, 0.266939, 0.335182, 0.240918, 0.331304, 0.266939, 0.332852, 0.291477, 0.335182, 0.240918, 0.332852, 0.291477, 0.335182, 0.240918, 0.336200, 0.258665, 0.335182, 0.240918, 0.332852, 0.291477, 0.336200, 0.258665, 0.334784, 0.311115, 0.332852, 0.291477, 0.336200, 0.258665, 0.332852, 0.291477, 0.334784, 0.311115, 0.336200, 0.258665, 0.334784, 0.311115, 0.336200, 0.258665, 0.337481, 0.272772, 0.336200, 0.258665, 0.334784, 0.311115, 0.337481, 0.272772, 0.337043, 0.325142, 0.334784, 0.311115, 0.337481, 0.272772, 0.334784, 0.311115, 0.337043, 0.325142, 0.337481, 0.272772, 0.337043, 0.325142, 0.337481, 0.272772, 0.338997, 0.282848, 0.337481, 0.272772, 0.337043, 0.325142, 0.338997, 0.282848, 0.337043, 0.325142, 0.338997, 0.282848, 0.339630, 0.333558, 0.338997, 0.282848, 0.337043, 0.325142, 0.339630, 0.333558, 0.339630, 0.333558, 0.338997, 0.282848, 0.340747, 0.288894, 0.338997, 0.282848, 0.339630, 0.333558, 0.340747, 0.288894, 0.342545, 0.336364, 0.339630, 0.333558, 0.340747, 0.288894, 0.339630, 0.333558, 0.342545, 0.336364, 0.340747, 0.288894, 0.340747, 0.288894, 0.342733, 0.290909, 0.342545, 0.336364, 0.342733, 0.290909, 0.340747, 0.288894, 0.342545, 0.336364, 0.343538, 0.335938, 0.342545, 0.336364, 0.342733, 0.290909, 0.342545, 0.336364, 0.343538, 0.335938, 0.342733, 0.290909, 0.343538, 0.335938, 0.342733, 0.290909, 0.343671, 0.290483, 0.342733, 0.290909, 0.343538, 0.335938, 0.343671, 0.290483, 0.343538, 0.335938, 0.343671, 0.290483, 0.344519, 0.334659, 0.343671, 0.290483, 0.343538, 0.335938, 0.344519, 0.334659, 0.344519, 0.334659, 0.343671, 0.290483, 0.344604, 0.289205, 0.343671, 0.290483, 0.344519, 0.334659, 0.344604, 0.289205, 0.345488, 0.332528, 0.344519, 0.334659, 0.344604, 0.289205, 0.344519, 0.334659, 0.345488, 0.332528, 0.344604, 0.289205, 0.345488, 0.332528, 0.344604, 0.289205, 0.345533, 0.287074, 0.344604, 0.289205, 0.345488, 0.332528, 0.345533, 0.287074, 0.345488, 0.332528, 0.345533, 0.287074, 0.346446, 0.329545, 0.345533, 0.287074, 0.345488, 0.332528, 0.346446, 0.329545, 0.347390, 0.325710, 0.346446, 0.329545, 0.345533, 0.287074, 0.346446, 0.329545, 0.347390, 0.325710, 0.345533, 0.287074, 0.347390, 0.325710, 0.345533, 0.287074, 0.346458, 0.284091, 0.345533, 0.287074, 0.347390, 0.325710, 0.346458, 0.284091, 0.347390, 0.325710, 0.346458, 0.284091, 0.348320, 0.321023, 0.346458, 0.284091, 0.347390, 0.325710, 0.348320, 0.321023, 0.346458, 0.284091, 0.347379, 0.280256, 0.348320, 0.321023, 0.347379, 0.280256, 0.346458, 0.284091, 0.348320, 0.321023, 0.348320, 0.321023, 0.347379, 0.280256, 0.349235, 0.315483, 0.347379, 0.280256, 0.348320, 0.321023, 0.349235, 0.315483, 0.347379, 0.280256, 0.348299, 0.275568, 0.349235, 0.315483, 0.348299, 0.275568, 0.347379, 0.280256, 0.349235, 0.315483, 0.349235, 0.315483, 0.348299, 0.275568, 0.350135, 0.309091, 0.348299, 0.275568, 0.349235, 0.315483, 0.350135, 0.309091, 0.348299, 0.275568, 0.349218, 0.270028, 0.350135, 0.309091, 0.349218, 0.270028, 0.348299, 0.275568, 0.350135, 0.309091, 0.349218, 0.270028, 0.350135, 0.263636, 0.350135, 0.309091, 0.350135, 0.263636, 0.349218, 0.270028, 0.350135, 0.309091, 0.334019, 0.195676, 0.329315, 0.168182, 0.333874, 0.168182, 0.329315, 0.168182, 0.334019, 0.195676, 0.333874, 0.168182, 0.333874, 0.168182, 0.329315, 0.168182, 0.334019, 0.140687, 0.329315, 0.168182, 0.333874, 0.168182, 0.334019, 0.140687, 0.329315, 0.168182, 0.329534, 0.131534, 0.334019, 0.140687, 0.329534, 0.131534, 0.329315, 0.168182, 0.334019, 0.140687, 0.334019, 0.140687, 0.329534, 0.131534, 0.334455, 0.116442, 0.329534, 0.131534, 0.334019, 0.140687, 0.334455, 0.116442, 0.329534, 0.131534, 0.330190, 0.098864, 0.334455, 0.116442, 0.330190, 0.098864, 0.329534, 0.131534, 0.334455, 0.116442, 0.334455, 0.116442, 0.330190, 0.098864, 0.331284, 0.070170, 0.330190, 0.098864, 0.334455, 0.116442, 0.331284, 0.070170, 0.335182, 0.095446, 0.334455, 0.116442, 0.331284, 0.070170, 0.334455, 0.116442, 0.335182, 0.095446, 0.331284, 0.070170, 0.335182, 0.095446, 0.331284, 0.070170, 0.332816, 0.045455, 0.331284, 0.070170, 0.335182, 0.095446, 0.332816, 0.045455, 0.336200, 0.077699, 0.335182, 0.095446, 0.332816, 0.045455, 0.335182, 0.095446, 0.336200, 0.077699, 0.332816, 0.045455, 0.332816, 0.045455, 0.334720, 0.025568, 0.336200, 0.077699, 0.334720, 0.025568, 0.332816, 0.045455, 0.336200, 0.077699, 0.336200, 0.077699, 0.334720, 0.025568, 0.337481, 0.063592, 0.334720, 0.025568, 0.336200, 0.077699, 0.337481, 0.063592, 0.337481, 0.063592, 0.334720, 0.025568, 0.336929, 0.011364, 0.334720, 0.025568, 0.337481, 0.063592, 0.336929, 0.011364, 0.338997, 0.053516, 0.337481, 0.063592, 0.336929, 0.011364, 0.337481, 0.063592, 0.338997, 0.053516, 0.336929, 0.011364, 0.338997, 0.053516, 0.336929, 0.011364, 0.339443, 0.002841, 0.336929, 0.011364, 0.338997, 0.053516, 0.339443, 0.002841, 0.340747, 0.047470, 0.338997, 0.053516, 0.339443, 0.002841, 0.338997, 0.053516, 0.340747, 0.047470, 0.339443, 0.002841, 0.340747, 0.047470, 0.339443, 0.002841, 0.342263, 0, 0.339443, 0.002841, 0.340747, 0.047470, 0.342263, 0, 0.342733, 0.045455, 0.340747, 0.047470, 0.342263, 0, 0.340747, 0.047470, 0.342733, 0.045455, 0.342263, 0, 0.342733, 0.045455, 0.342263, 0, 0.343340, 0.000426, 0.342263, 0, 0.342733, 0.045455, 0.343340, 0.000426, 0.342733, 0.045455, 0.343340, 0.000426, 0.343671, 0.045881, 0.343340, 0.000426, 0.342733, 0.045455, 0.343671, 0.045881, 0.343671, 0.045881, 0.343340, 0.000426, 0.344387, 0.001705, 0.343340, 0.000426, 0.343671, 0.045881, 0.344387, 0.001705, 0.344604, 0.047159, 0.343671, 0.045881, 0.344387, 0.001705, 0.343671, 0.045881, 0.344604, 0.047159, 0.344387, 0.001705, 0.345402, 0.003835, 0.344604, 0.047159, 0.344387, 0.001705, 0.344604, 0.047159, 0.345402, 0.003835, 0.344387, 0.001705, 0.345533, 0.049290, 0.344604, 0.047159, 0.345402, 0.003835, 0.344604, 0.047159, 0.345533, 0.049290, 0.345402, 0.003835, 0.346387, 0.006818, 0.345533, 0.049290, 0.345402, 0.003835, 0.345533, 0.049290, 0.346387, 0.006818, 0.345402, 0.003835, 0.345533, 0.049290, 0.346387, 0.006818, 0.347348, 0.010653, 0.346387, 0.006818, 0.345533, 0.049290, 0.347348, 0.010653, 0.346458, 0.052273, 0.345533, 0.049290, 0.347348, 0.010653, 0.345533, 0.049290, 0.346458, 0.052273, 0.347348, 0.010653, 0.346458, 0.052273, 0.347348, 0.010653, 0.348293, 0.015341, 0.347348, 0.010653, 0.346458, 0.052273, 0.348293, 0.015341, 0.347379, 0.056108, 0.346458, 0.052273, 0.348293, 0.015341, 0.346458, 0.052273, 0.347379, 0.056108, 0.348293, 0.015341, 0.347379, 0.056108, 0.348293, 0.015341, 0.349222, 0.020881, 0.348293, 0.015341, 0.347379, 0.056108, 0.349222, 0.020881, 0.348299, 0.060795, 0.347379, 0.056108, 0.349222, 0.020881, 0.347379, 0.056108, 0.348299, 0.060795, 0.349222, 0.020881, 0.348299, 0.060795, 0.349222, 0.020881, 0.350135, 0.027273, 0.349222, 0.020881, 0.348299, 0.060795, 0.350135, 0.027273, 0.349218, 0.066335, 0.348299, 0.060795, 0.350135, 0.027273, 0.348299, 0.060795, 0.349218, 0.066335, 0.350135, 0.027273, 0.349218, 0.066335, 0.350135, 0.027273, 0.350135, 0.072727, 0.350135, 0.027273, 0.349218, 0.066335, 0.350135, 0.072727, 0.361791, 0.418182, 0.361791, 0.327273, 0.361791, 0.418182, 0.361791, 0.418182, 0.361791, 0.327273, 0.361791, 0.327273, 0.361791, 0.327273, 0.370697, 0.327273, 0.361791, 0.327273, 0.361791, 0.327273, 0.370697, 0.327273, 0.370697, 0.327273, 0.370697, 0.327273, 0.370697, 0.281818, 0.370697, 0.327273, 0.370697, 0.327273, 0.370697, 0.281818, 0.370697, 0.281818, 0.370697, 0.281818, 0.361791, 0.281818, 0.370697, 0.281818, 0.370697, 0.281818, 0.361791, 0.281818, 0.361791, 0.281818, 0.361791, 0.281818, 0.361791, 0.113920, 0.361791, 0.281818, 0.361791, 0.281818, 0.361791, 0.113920, 0.361791, 0.113920, 0.361791, 0.113920, 0.361846, 0.096715, 0.361791, 0.113920, 0.361791, 0.113920, 0.361846, 0.096715, 0.361846, 0.096715, 0.361846, 0.096715, 0.362011, 0.082884, 0.361846, 0.096715, 0.361846, 0.096715, 0.362011, 0.082884, 0.362011, 0.082884, 0.362011, 0.082884, 0.362286, 0.072425, 0.362011, 0.082884, 0.362011, 0.082884, 0.362286, 0.072425, 0.362286, 0.072425, 0.362286, 0.072425, 0.362672, 0.065341, 0.362286, 0.072425, 0.362286, 0.072425, 0.362672, 0.065341, 0.362672, 0.065341, 0.362672, 0.065341, 0.363226, 0.060618, 0.362672, 0.065341, 0.362672, 0.065341, 0.363226, 0.060618, 0.363226, 0.060618, 0.363226, 0.060618, 0.364008, 0.057244, 0.363226, 0.060618, 0.363226, 0.060618, 0.364008, 0.057244, 0.364008, 0.057244, 0.364008, 0.057244, 0.365018, 0.055220, 0.364008, 0.057244, 0.364008, 0.057244, 0.365018, 0.055220, 0.365018, 0.055220, 0.365018, 0.055220, 0.366255, 0.054545, 0.365018, 0.055220, 0.365018, 0.055220, 0.366255, 0.054545, 0.366255, 0.054545, 0.366255, 0.054545, 0.370697, 0.054545, 0.366255, 0.054545, 0.366255, 0.054545, 0.370697, 0.054545, 0.370697, 0.054545, 0.370697, 0.054545, 0.370697, 0.009091, 0.370697, 0.054545, 0.370697, 0.054545, 0.370697, 0.009091, 0.370697, 0.009091, 0.370697, 0.009091, 0.366255, 0.009091, 0.370697, 0.009091, 0.370697, 0.009091, 0.366255, 0.009091, 0.366255, 0.009091, 0.366255, 0.009091, 0.363947, 0.010502, 0.366255, 0.009091, 0.366255, 0.009091, 0.363947, 0.010502, 0.363947, 0.010502, 0.363947, 0.010502, 0.362026, 0.014737, 0.363947, 0.010502, 0.363947, 0.010502, 0.362026, 0.014737, 0.362026, 0.014737, 0.362026, 0.014737, 0.360492, 0.021795, 0.362026, 0.014737, 0.362026, 0.014737, 0.360492, 0.021795, 0.360492, 0.021795, 0.360492, 0.021795, 0.359347, 0.031676, 0.360492, 0.021795, 0.360492, 0.021795, 0.359347, 0.031676, 0.359347, 0.031676, 0.359347, 0.031676, 0.358514, 0.045286, 0.359347, 0.031676, 0.359347, 0.031676, 0.358514, 0.045286, 0.358514, 0.045286, 0.358514, 0.045286, 0.357919, 0.063530, 0.358514, 0.045286, 0.358514, 0.045286, 0.357919, 0.063530, 0.357919, 0.063530, 0.357919, 0.063530, 0.357562, 0.086408, 0.357919, 0.063530, 0.357919, 0.063530, 0.357562, 0.086408, 0.357562, 0.086408, 0.357562, 0.086408, 0.357443, 0.113920, 0.357562, 0.086408, 0.357562, 0.086408, 0.357443, 0.113920, 0.357443, 0.113920, 0.357443, 0.113920, 0.357443, 0.281818, 0.357443, 0.113920, 0.357443, 0.113920, 0.357443, 0.281818, 0.357443, 0.281818, 0.357443, 0.281818, 0.354271, 0.281818, 0.357443, 0.281818, 0.357443, 0.281818, 0.354271, 0.281818, 0.354271, 0.281818, 0.354271, 0.281818, 0.354271, 0.327273, 0.354271, 0.281818, 0.354271, 0.281818, 0.354271, 0.327273, 0.354271, 0.327273, 0.354271, 0.327273, 0.357443, 0.327273, 0.354271, 0.327273, 0.354271, 0.327273, 0.357443, 0.327273, 0.357443, 0.327273, 0.357443, 0.327273, 0.357443, 0.418182, 0.357443, 0.327273, 0.357443, 0.327273, 0.357443, 0.418182, 0.357443, 0.418182, 0.354271, 0.327273, 0.357443, 0.281818, 0.357443, 0.327273, 0.357443, 0.281818, 0.354271, 0.327273, 0.357443, 0.327273, 0.354271, 0.327273, 0.354271, 0.281818, 0.357443, 0.281818, 0.354271, 0.281818, 0.354271, 0.327273, 0.357443, 0.281818, 0.357443, 0.327273, 0.357443, 0.281818, 0.361791, 0.281818, 0.357443, 0.281818, 0.357443, 0.327273, 0.361791, 0.281818, 0.357443, 0.327273, 0.361791, 0.281818, 0.361791, 0.327273, 0.361791, 0.281818, 0.357443, 0.327273, 0.361791, 0.327273, 0.357443, 0.418182, 0.357443, 0.327273, 0.361791, 0.327273, 0.357443, 0.327273, 0.357443, 0.418182, 0.361791, 0.327273, 0.357443, 0.418182, 0.361791, 0.327273, 0.361791, 0.418182, 0.361791, 0.327273, 0.357443, 0.418182, 0.361791, 0.418182, 0.361791, 0.327273, 0.361791, 0.281818, 0.370697, 0.281818, 0.361791, 0.281818, 0.361791, 0.327273, 0.370697, 0.281818, 0.361791, 0.327273, 0.370697, 0.281818, 0.370697, 0.327273, 0.370697, 0.281818, 0.361791, 0.327273, 0.370697, 0.327273, 0.357443, 0.281818, 0.361791, 0.113920, 0.361791, 0.281818, 0.361791, 0.113920, 0.357443, 0.281818, 0.361791, 0.281818, 0.357443, 0.281818, 0.357443, 0.113920, 0.361791, 0.113920, 0.357443, 0.113920, 0.357443, 0.281818, 0.361791, 0.113920, 0.361791, 0.113920, 0.357443, 0.113920, 0.361846, 0.096715, 0.357443, 0.113920, 0.361791, 0.113920, 0.361846, 0.096715, 0.357443, 0.113920, 0.357562, 0.086408, 0.361846, 0.096715, 0.357562, 0.086408, 0.357443, 0.113920, 0.361846, 0.096715, 0.361846, 0.096715, 0.357562, 0.086408, 0.362011, 0.082884, 0.357562, 0.086408, 0.361846, 0.096715, 0.362011, 0.082884, 0.357562, 0.086408, 0.357919, 0.063530, 0.362011, 0.082884, 0.357919, 0.063530, 0.357562, 0.086408, 0.362011, 0.082884, 0.362011, 0.082884, 0.357919, 0.063530, 0.362286, 0.072425, 0.357919, 0.063530, 0.362011, 0.082884, 0.362286, 0.072425, 0.362286, 0.072425, 0.357919, 0.063530, 0.358514, 0.045286, 0.357919, 0.063530, 0.362286, 0.072425, 0.358514, 0.045286, 0.362672, 0.065341, 0.362286, 0.072425, 0.358514, 0.045286, 0.362286, 0.072425, 0.362672, 0.065341, 0.358514, 0.045286, 0.358514, 0.045286, 0.359347, 0.031676, 0.362672, 0.065341, 0.359347, 0.031676, 0.358514, 0.045286, 0.362672, 0.065341, 0.362672, 0.065341, 0.359347, 0.031676, 0.360492, 0.021795, 0.359347, 0.031676, 0.362672, 0.065341, 0.360492, 0.021795, 0.363226, 0.060618, 0.362672, 0.065341, 0.360492, 0.021795, 0.362672, 0.065341, 0.363226, 0.060618, 0.360492, 0.021795, 0.360492, 0.021795, 0.362026, 0.014737, 0.363226, 0.060618, 0.362026, 0.014737, 0.360492, 0.021795, 0.363226, 0.060618, 0.363226, 0.060618, 0.362026, 0.014737, 0.364008, 0.057244, 0.362026, 0.014737, 0.363226, 0.060618, 0.364008, 0.057244, 0.362026, 0.014737, 0.363947, 0.010502, 0.364008, 0.057244, 0.363947, 0.010502, 0.362026, 0.014737, 0.364008, 0.057244, 0.364008, 0.057244, 0.363947, 0.010502, 0.365018, 0.055220, 0.363947, 0.010502, 0.364008, 0.057244, 0.365018, 0.055220, 0.365018, 0.055220, 0.363947, 0.010502, 0.366255, 0.009091, 0.363947, 0.010502, 0.365018, 0.055220, 0.366255, 0.009091, 0.366255, 0.054545, 0.365018, 0.055220, 0.366255, 0.009091, 0.365018, 0.055220, 0.366255, 0.054545, 0.366255, 0.009091, 0.366255, 0.054545, 0.366255, 0.009091, 0.370697, 0.009091, 0.366255, 0.009091, 0.366255, 0.054545, 0.370697, 0.009091, 0.366255, 0.054545, 0.370697, 0.009091, 0.370697, 0.054545, 0.370697, 0.009091, 0.366255, 0.054545, 0.370697, 0.054545, 0.376313, 0.327273, 0.380637, 0.327273, 0.376313, 0.327273, 0.376313, 0.327273, 0.380637, 0.327273, 0.380637, 0.327273, 0.380637, 0.327273, 0.380637, 0.009091, 0.380637, 0.327273, 0.380637, 0.327273, 0.380637, 0.009091, 0.380637, 0.009091, 0.380637, 0.009091, 0.376313, 0.009091, 0.380637, 0.009091, 0.380637, 0.009091, 0.376313, 0.009091, 0.376313, 0.009091, 0.376313, 0.327273, 0.380637, 0.009091, 0.380637, 0.327273, 0.380637, 0.009091, 0.376313, 0.327273, 0.380637, 0.327273, 0.376313, 0.327273, 0.376313, 0.009091, 0.380637, 0.009091, 0.376313, 0.009091, 0.376313, 0.327273, 0.380637, 0.009091, 0.376313, 0.454545, 0.380637, 0.454545, 0.376313, 0.454545, 0.376313, 0.454545, 0.380637, 0.454545, 0.380637, 0.454545, 0.380637, 0.454545, 0.380637, 0.390909, 0.380637, 0.454545, 0.380637, 0.454545, 0.380637, 0.390909, 0.380637, 0.390909, 0.380637, 0.390909, 0.376313, 0.390909, 0.380637, 0.390909, 0.380637, 0.390909, 0.376313, 0.390909, 0.376313, 0.390909, 0.376313, 0.454545, 0.380637, 0.390909, 0.380637, 0.454545, 0.380637, 0.390909, 0.376313, 0.454545, 0.380637, 0.454545, 0.376313, 0.454545, 0.376313, 0.390909, 0.380637, 0.390909, 0.376313, 0.390909, 0.376313, 0.454545, 0.380637, 0.390909, 0.400047, 0.290909, 0.398399, 0.288858, 0.400047, 0.290909, 0.400047, 0.290909, 0.398399, 0.288858, 0.398399, 0.288858, 0.398399, 0.288858, 0.396933, 0.282706, 0.398399, 0.288858, 0.398399, 0.288858, 0.396933, 0.282706, 0.396933, 0.282706, 0.396933, 0.282706, 0.395650, 0.272452, 0.396933, 0.282706, 0.396933, 0.282706, 0.395650, 0.272452, 0.395650, 0.272452, 0.395650, 0.272452, 0.394548, 0.258097, 0.395650, 0.272452, 0.395650, 0.272452, 0.394548, 0.258097, 0.394548, 0.258097, 0.394548, 0.258097, 0.393664, 0.240172, 0.394548, 0.258097, 0.394548, 0.258097, 0.393664, 0.240172, 0.393664, 0.240172, 0.393664, 0.240172, 0.393033, 0.219212, 0.393664, 0.240172, 0.393664, 0.240172, 0.393033, 0.219212, 0.393033, 0.219212, 0.393033, 0.219212, 0.392654, 0.195215, 0.393033, 0.219212, 0.393033, 0.219212, 0.392654, 0.195215, 0.392654, 0.195215, 0.392654, 0.195215, 0.392527, 0.168182, 0.392654, 0.195215, 0.392654, 0.195215, 0.392527, 0.168182, 0.392527, 0.168182, 0.392527, 0.168182, 0.392653, 0.141149, 0.392527, 0.168182, 0.392527, 0.168182, 0.392653, 0.141149, 0.392653, 0.141149, 0.392653, 0.141149, 0.393030, 0.117152, 0.392653, 0.141149, 0.392653, 0.141149, 0.393030, 0.117152, 0.393030, 0.117152, 0.393030, 0.117152, 0.393658, 0.096191, 0.393030, 0.117152, 0.393030, 0.117152, 0.393658, 0.096191, 0.393658, 0.096191, 0.393658, 0.096191, 0.394536, 0.078267, 0.393658, 0.096191, 0.393658, 0.096191, 0.394536, 0.078267, 0.394536, 0.078267, 0.394536, 0.078267, 0.395634, 0.063912, 0.394536, 0.078267, 0.394536, 0.078267, 0.395634, 0.063912, 0.395634, 0.063912, 0.395634, 0.063912, 0.396919, 0.053658, 0.395634, 0.063912, 0.395634, 0.063912, 0.396919, 0.053658, 0.396919, 0.053658, 0.396919, 0.053658, 0.398390, 0.047505, 0.396919, 0.053658, 0.396919, 0.053658, 0.398390, 0.047505, 0.398390, 0.047505, 0.398390, 0.047505, 0.400047, 0.045455, 0.398390, 0.047505, 0.398390, 0.047505, 0.400047, 0.045455, 0.400047, 0.045455, 0.400047, 0.045455, 0.401685, 0.047514, 0.400047, 0.045455, 0.400047, 0.045455, 0.401685, 0.047514, 0.401685, 0.047514, 0.401685, 0.047514, 0.403143, 0.053693, 0.401685, 0.047514, 0.401685, 0.047514, 0.403143, 0.053693, 0.403143, 0.053693, 0.403143, 0.053693, 0.404422, 0.063991, 0.403143, 0.053693, 0.403143, 0.053693, 0.404422, 0.063991, 0.404422, 0.063991, 0.404422, 0.063991, 0.405522, 0.078409, 0.404422, 0.063991, 0.404422, 0.063991, 0.405522, 0.078409, 0.405522, 0.078409, 0.405522, 0.078409, 0.406406, 0.096378, 0.405522, 0.078409, 0.405522, 0.078409, 0.406406, 0.096378, 0.406406, 0.096378, 0.406406, 0.096378, 0.407038, 0.117330, 0.406406, 0.096378, 0.406406, 0.096378, 0.407038, 0.117330, 0.407038, 0.117330, 0.407038, 0.117330, 0.407417, 0.141264, 0.407038, 0.117330, 0.407038, 0.117330, 0.407417, 0.141264, 0.407417, 0.141264, 0.407417, 0.141264, 0.407543, 0.168182, 0.407417, 0.141264, 0.407417, 0.141264, 0.407543, 0.168182, 0.407543, 0.168182, 0.407543, 0.168182, 0.407417, 0.194984, 0.407543, 0.168182, 0.407543, 0.168182, 0.407417, 0.194984, 0.407417, 0.194984, 0.407417, 0.194984, 0.407038, 0.218857, 0.407417, 0.194984, 0.407417, 0.194984, 0.407038, 0.218857, 0.407038, 0.218857, 0.407038, 0.218857, 0.406406, 0.239799, 0.407038, 0.218857, 0.407038, 0.218857, 0.406406, 0.239799, 0.406406, 0.239799, 0.406406, 0.239799, 0.405522, 0.257813, 0.406406, 0.239799, 0.406406, 0.239799, 0.405522, 0.257813, 0.405522, 0.257813, 0.405522, 0.257813, 0.404422, 0.272292, 0.405522, 0.257813, 0.405522, 0.257813, 0.404422, 0.272292, 0.404422, 0.272292, 0.404422, 0.272292, 0.403143, 0.282635, 0.404422, 0.272292, 0.404422, 0.272292, 0.403143, 0.282635, 0.403143, 0.282635, 0.403143, 0.282635, 0.401685, 0.288841, 0.403143, 0.282635, 0.403143, 0.282635, 0.401685, 0.288841, 0.401685, 0.288841, 0.400047, 0.336364, 0.402716, 0.333576, 0.400047, 0.336364, 0.400047, 0.336364, 0.402716, 0.333576, 0.402716, 0.333576, 0.402716, 0.333576, 0.405082, 0.325213, 0.402716, 0.333576, 0.402716, 0.333576, 0.405082, 0.325213, 0.405082, 0.325213, 0.405082, 0.325213, 0.407145, 0.311275, 0.405082, 0.325213, 0.405082, 0.325213, 0.407145, 0.311275, 0.407145, 0.311275, 0.407145, 0.311275, 0.408906, 0.291761, 0.407145, 0.311275, 0.407145, 0.311275, 0.408906, 0.291761, 0.408906, 0.291761, 0.408906, 0.291761, 0.410315, 0.267312, 0.408906, 0.291761, 0.408906, 0.291761, 0.410315, 0.267312, 0.410315, 0.267312, 0.410315, 0.267312, 0.411321, 0.238565, 0.410315, 0.267312, 0.410315, 0.267312, 0.411321, 0.238565, 0.411321, 0.238565, 0.411321, 0.238565, 0.411924, 0.205522, 0.411321, 0.238565, 0.411321, 0.238565, 0.411924, 0.205522, 0.411924, 0.205522, 0.411924, 0.205522, 0.412125, 0.168182, 0.411924, 0.205522, 0.411924, 0.205522, 0.412125, 0.168182, 0.412125, 0.168182, 0.412125, 0.168182, 0.411924, 0.130957, 0.412125, 0.168182, 0.412125, 0.168182, 0.411924, 0.130957, 0.411924, 0.130957, 0.411924, 0.130957, 0.411321, 0.097976, 0.411924, 0.130957, 0.411924, 0.130957, 0.411321, 0.097976, 0.411321, 0.097976, 0.411321, 0.097976, 0.410315, 0.069238, 0.411321, 0.097976, 0.411321, 0.097976, 0.410315, 0.069238, 0.410315, 0.069238, 0.410315, 0.069238, 0.408906, 0.044744, 0.410315, 0.069238, 0.410315, 0.069238, 0.408906, 0.044744, 0.408906, 0.044744, 0.408906, 0.044744, 0.407145, 0.025169, 0.408906, 0.044744, 0.408906, 0.044744, 0.407145, 0.025169, 0.407145, 0.025169, 0.407145, 0.025169, 0.405082, 0.011186, 0.407145, 0.025169, 0.407145, 0.025169, 0.405082, 0.011186, 0.405082, 0.011186, 0.405082, 0.011186, 0.402716, 0.002797, 0.405082, 0.011186, 0.405082, 0.011186, 0.402716, 0.002797, 0.402716, 0.002797, 0.402716, 0.002797, 0.400047, 0, 0.402716, 0.002797, 0.402716, 0.002797, 0.400047, 0, 0.400047, 0, 0.400047, 0, 0.397369, 0.002797, 0.400047, 0, 0.400047, 0, 0.397369, 0.002797, 0.397369, 0.002797, 0.397369, 0.002797, 0.394998, 0.011186, 0.397369, 0.002797, 0.397369, 0.002797, 0.394998, 0.011186, 0.394998, 0.011186, 0.394998, 0.011186, 0.392933, 0.025169, 0.394998, 0.011186, 0.394998, 0.011186, 0.392933, 0.025169, 0.392933, 0.025169, 0.392933, 0.025169, 0.391176, 0.044744, 0.392933, 0.025169, 0.392933, 0.025169, 0.391176, 0.044744, 0.391176, 0.044744, 0.391176, 0.044744, 0.389773, 0.069238, 0.391176, 0.044744, 0.391176, 0.044744, 0.389773, 0.069238, 0.389773, 0.069238, 0.389773, 0.069238, 0.388770, 0.097976, 0.389773, 0.069238, 0.389773, 0.069238, 0.388770, 0.097976, 0.388770, 0.097976, 0.388770, 0.097976, 0.388169, 0.130957, 0.388770, 0.097976, 0.388770, 0.097976, 0.388169, 0.130957, 0.388169, 0.130957, 0.388169, 0.130957, 0.387969, 0.168182, 0.388169, 0.130957, 0.388169, 0.130957, 0.387969, 0.168182, 0.387969, 0.168182, 0.387969, 0.168182, 0.388169, 0.205522, 0.387969, 0.168182, 0.387969, 0.168182, 0.388169, 0.205522, 0.388169, 0.205522, 0.388169, 0.205522, 0.388770, 0.238565, 0.388169, 0.205522, 0.388169, 0.205522, 0.388770, 0.238565, 0.388770, 0.238565, 0.388770, 0.238565, 0.389773, 0.267312, 0.388770, 0.238565, 0.388770, 0.238565, 0.389773, 0.267312, 0.389773, 0.267312, 0.389773, 0.267312, 0.391176, 0.291761, 0.389773, 0.267312, 0.389773, 0.267312, 0.391176, 0.291761, 0.391176, 0.291761, 0.391176, 0.291761, 0.392933, 0.311275, 0.391176, 0.291761, 0.391176, 0.291761, 0.392933, 0.311275, 0.392933, 0.311275, 0.392933, 0.311275, 0.394998, 0.325213, 0.392933, 0.311275, 0.392933, 0.311275, 0.394998, 0.325213, 0.394998, 0.325213, 0.394998, 0.325213, 0.397369, 0.333576, 0.394998, 0.325213, 0.394998, 0.325213, 0.397369, 0.333576, 0.397369, 0.333576, 0.392654, 0.195215, 0.388169, 0.205522, 0.387969, 0.168182, 0.388169, 0.205522, 0.392654, 0.195215, 0.387969, 0.168182, 0.393033, 0.219212, 0.388169, 0.205522, 0.392654, 0.195215, 0.388169, 0.205522, 0.393033, 0.219212, 0.392654, 0.195215, 0.393033, 0.219212, 0.388770, 0.238565, 0.388169, 0.205522, 0.388770, 0.238565, 0.393033, 0.219212, 0.388169, 0.205522, 0.388770, 0.238565, 0.393033, 0.219212, 0.393664, 0.240172, 0.393033, 0.219212, 0.388770, 0.238565, 0.393664, 0.240172, 0.389773, 0.267312, 0.388770, 0.238565, 0.393664, 0.240172, 0.388770, 0.238565, 0.389773, 0.267312, 0.393664, 0.240172, 0.394548, 0.258097, 0.389773, 0.267312, 0.393664, 0.240172, 0.389773, 0.267312, 0.394548, 0.258097, 0.393664, 0.240172, 0.394548, 0.258097, 0.391176, 0.291761, 0.389773, 0.267312, 0.391176, 0.291761, 0.394548, 0.258097, 0.389773, 0.267312, 0.392933, 0.311275, 0.391176, 0.291761, 0.394548, 0.258097, 0.391176, 0.291761, 0.392933, 0.311275, 0.394548, 0.258097, 0.392933, 0.311275, 0.394548, 0.258097, 0.395650, 0.272452, 0.394548, 0.258097, 0.392933, 0.311275, 0.395650, 0.272452, 0.394998, 0.325213, 0.392933, 0.311275, 0.395650, 0.272452, 0.392933, 0.311275, 0.394998, 0.325213, 0.395650, 0.272452, 0.394998, 0.325213, 0.395650, 0.272452, 0.396933, 0.282706, 0.395650, 0.272452, 0.394998, 0.325213, 0.396933, 0.282706, 0.394998, 0.325213, 0.396933, 0.282706, 0.397369, 0.333576, 0.396933, 0.282706, 0.394998, 0.325213, 0.397369, 0.333576, 0.397369, 0.333576, 0.396933, 0.282706, 0.398399, 0.288858, 0.396933, 0.282706, 0.397369, 0.333576, 0.398399, 0.288858, 0.397369, 0.333576, 0.398399, 0.288858, 0.400047, 0.336364, 0.398399, 0.288858, 0.397369, 0.333576, 0.400047, 0.336364, 0.398399, 0.288858, 0.400047, 0.290909, 0.400047, 0.336364, 0.400047, 0.290909, 0.398399, 0.288858, 0.400047, 0.336364, 0.400047, 0.336364, 0.400047, 0.290909, 0.401685, 0.288841, 0.400047, 0.290909, 0.400047, 0.336364, 0.401685, 0.288841, 0.400047, 0.336364, 0.401685, 0.288841, 0.402716, 0.333576, 0.401685, 0.288841, 0.400047, 0.336364, 0.402716, 0.333576, 0.402716, 0.333576, 0.401685, 0.288841, 0.403143, 0.282635, 0.401685, 0.288841, 0.402716, 0.333576, 0.403143, 0.282635, 0.402716, 0.333576, 0.403143, 0.282635, 0.405082, 0.325213, 0.403143, 0.282635, 0.402716, 0.333576, 0.405082, 0.325213, 0.403143, 0.282635, 0.404422, 0.272292, 0.405082, 0.325213, 0.404422, 0.272292, 0.403143, 0.282635, 0.405082, 0.325213, 0.405082, 0.325213, 0.404422, 0.272292, 0.407145, 0.311275, 0.404422, 0.272292, 0.405082, 0.325213, 0.407145, 0.311275, 0.404422, 0.272292, 0.405522, 0.257813, 0.407145, 0.311275, 0.405522, 0.257813, 0.404422, 0.272292, 0.407145, 0.311275, 0.407145, 0.311275, 0.405522, 0.257813, 0.408906, 0.291761, 0.405522, 0.257813, 0.407145, 0.311275, 0.408906, 0.291761, 0.408906, 0.291761, 0.405522, 0.257813, 0.410315, 0.267312, 0.405522, 0.257813, 0.408906, 0.291761, 0.410315, 0.267312, 0.410315, 0.267312, 0.405522, 0.257813, 0.406406, 0.239799, 0.405522, 0.257813, 0.410315, 0.267312, 0.406406, 0.239799, 0.410315, 0.267312, 0.406406, 0.239799, 0.411321, 0.238565, 0.406406, 0.239799, 0.410315, 0.267312, 0.411321, 0.238565, 0.406406, 0.239799, 0.407038, 0.218857, 0.411321, 0.238565, 0.407038, 0.218857, 0.406406, 0.239799, 0.411321, 0.238565, 0.411321, 0.238565, 0.407038, 0.218857, 0.411924, 0.205522, 0.407038, 0.218857, 0.411321, 0.238565, 0.411924, 0.205522, 0.407038, 0.218857, 0.407417, 0.194984, 0.411924, 0.205522, 0.407417, 0.194984, 0.407038, 0.218857, 0.411924, 0.205522, 0.411924, 0.205522, 0.407417, 0.194984, 0.412125, 0.168182, 0.407417, 0.194984, 0.411924, 0.205522, 0.412125, 0.168182, 0.412125, 0.168182, 0.407417, 0.194984, 0.407543, 0.168182, 0.407417, 0.194984, 0.412125, 0.168182, 0.407543, 0.168182, 0.412125, 0.168182, 0.407543, 0.168182, 0.407417, 0.141264, 0.407543, 0.168182, 0.412125, 0.168182, 0.407417, 0.141264, 0.412125, 0.168182, 0.407417, 0.141264, 0.411924, 0.130957, 0.407417, 0.141264, 0.412125, 0.168182, 0.411924, 0.130957, 0.407417, 0.141264, 0.407038, 0.117330, 0.411924, 0.130957, 0.407038, 0.117330, 0.407417, 0.141264, 0.411924, 0.130957, 0.407038, 0.117330, 0.411321, 0.097976, 0.411924, 0.130957, 0.411321, 0.097976, 0.407038, 0.117330, 0.411924, 0.130957, 0.407038, 0.117330, 0.406406, 0.096378, 0.411321, 0.097976, 0.406406, 0.096378, 0.407038, 0.117330, 0.411321, 0.097976, 0.406406, 0.096378, 0.410315, 0.069238, 0.411321, 0.097976, 0.410315, 0.069238, 0.406406, 0.096378, 0.411321, 0.097976, 0.406406, 0.096378, 0.405522, 0.078409, 0.410315, 0.069238, 0.405522, 0.078409, 0.406406, 0.096378, 0.410315, 0.069238, 0.405522, 0.078409, 0.408906, 0.044744, 0.410315, 0.069238, 0.408906, 0.044744, 0.405522, 0.078409, 0.410315, 0.069238, 0.408906, 0.044744, 0.405522, 0.078409, 0.407145, 0.025169, 0.405522, 0.078409, 0.408906, 0.044744, 0.407145, 0.025169, 0.405522, 0.078409, 0.404422, 0.063991, 0.407145, 0.025169, 0.404422, 0.063991, 0.405522, 0.078409, 0.407145, 0.025169, 0.407145, 0.025169, 0.404422, 0.063991, 0.405082, 0.011186, 0.404422, 0.063991, 0.407145, 0.025169, 0.405082, 0.011186, 0.404422, 0.063991, 0.403143, 0.053693, 0.405082, 0.011186, 0.403143, 0.053693, 0.404422, 0.063991, 0.405082, 0.011186, 0.405082, 0.011186, 0.403143, 0.053693, 0.402716, 0.002797, 0.403143, 0.053693, 0.405082, 0.011186, 0.402716, 0.002797, 0.403143, 0.053693, 0.401685, 0.047514, 0.402716, 0.002797, 0.401685, 0.047514, 0.403143, 0.053693, 0.402716, 0.002797, 0.402716, 0.002797, 0.401685, 0.047514, 0.400047, 0, 0.401685, 0.047514, 0.402716, 0.002797, 0.400047, 0, 0.401685, 0.047514, 0.400047, 0.045455, 0.400047, 0, 0.400047, 0.045455, 0.401685, 0.047514, 0.400047, 0, 0.400047, 0.045455, 0.398390, 0.047505, 0.400047, 0, 0.398390, 0.047505, 0.400047, 0.045455, 0.400047, 0, 0.398390, 0.047505, 0.397369, 0.002797, 0.400047, 0, 0.397369, 0.002797, 0.398390, 0.047505, 0.400047, 0, 0.398390, 0.047505, 0.396919, 0.053658, 0.397369, 0.002797, 0.396919, 0.053658, 0.398390, 0.047505, 0.397369, 0.002797, 0.396919, 0.053658, 0.394998, 0.011186, 0.397369, 0.002797, 0.394998, 0.011186, 0.396919, 0.053658, 0.397369, 0.002797, 0.396919, 0.053658, 0.395634, 0.063912, 0.394998, 0.011186, 0.395634, 0.063912, 0.396919, 0.053658, 0.394998, 0.011186, 0.395634, 0.063912, 0.392933, 0.025169, 0.394998, 0.011186, 0.392933, 0.025169, 0.395634, 0.063912, 0.394998, 0.011186, 0.395634, 0.063912, 0.394536, 0.078267, 0.392933, 0.025169, 0.394536, 0.078267, 0.395634, 0.063912, 0.392933, 0.025169, 0.394536, 0.078267, 0.391176, 0.044744, 0.392933, 0.025169, 0.391176, 0.044744, 0.394536, 0.078267, 0.392933, 0.025169, 0.389773, 0.069238, 0.391176, 0.044744, 0.394536, 0.078267, 0.391176, 0.044744, 0.389773, 0.069238, 0.394536, 0.078267, 0.393658, 0.096191, 0.389773, 0.069238, 0.394536, 0.078267, 0.389773, 0.069238, 0.393658, 0.096191, 0.394536, 0.078267, 0.388770, 0.097976, 0.389773, 0.069238, 0.393658, 0.096191, 0.389773, 0.069238, 0.388770, 0.097976, 0.393658, 0.096191, 0.393030, 0.117152, 0.388770, 0.097976, 0.393658, 0.096191, 0.388770, 0.097976, 0.393030, 0.117152, 0.393658, 0.096191, 0.388169, 0.130957, 0.388770, 0.097976, 0.393030, 0.117152, 0.388770, 0.097976, 0.388169, 0.130957, 0.393030, 0.117152, 0.392653, 0.141149, 0.388169, 0.130957, 0.393030, 0.117152, 0.388169, 0.130957, 0.392653, 0.141149, 0.393030, 0.117152, 0.387969, 0.168182, 0.388169, 0.130957, 0.392653, 0.141149, 0.388169, 0.130957, 0.387969, 0.168182, 0.392653, 0.141149, 0.392527, 0.168182, 0.387969, 0.168182, 0.392653, 0.141149, 0.387969, 0.168182, 0.392527, 0.168182, 0.392653, 0.141149, 0.392654, 0.195215, 0.387969, 0.168182, 0.392527, 0.168182, 0.387969, 0.168182, 0.392654, 0.195215, 0.392527, 0.168182, 0.441053, 0.203977, 0.441053, 0.009091, 0.441053, 0.203977, 0.441053, 0.203977, 0.441053, 0.009091, 0.441053, 0.009091, 0.441053, 0.009091, 0.436729, 0.009091, 0.441053, 0.009091, 0.441053, 0.009091, 0.436729, 0.009091, 0.436729, 0.009091, 0.436729, 0.009091, 0.436729, 0.202273, 0.436729, 0.009091, 0.436729, 0.009091, 0.436729, 0.202273, 0.436729, 0.202273, 0.436729, 0.202273, 0.436638, 0.223105, 0.436729, 0.202273, 0.436729, 0.202273, 0.436638, 0.223105, 0.436638, 0.223105, 0.436638, 0.223105, 0.436365, 0.241140, 0.436638, 0.223105, 0.436638, 0.223105, 0.436365, 0.241140, 0.436365, 0.241140, 0.436365, 0.241140, 0.435909, 0.256379, 0.436365, 0.241140, 0.436365, 0.241140, 0.435909, 0.256379, 0.435909, 0.256379, 0.435909, 0.256379, 0.435272, 0.268821, 0.435909, 0.256379, 0.435909, 0.256379, 0.435272, 0.268821, 0.435272, 0.268821, 0.435272, 0.268821, 0.434453, 0.278485, 0.435272, 0.268821, 0.435272, 0.268821, 0.434453, 0.278485, 0.434453, 0.278485, 0.434453, 0.278485, 0.433451, 0.285387, 0.434453, 0.278485, 0.434453, 0.278485, 0.433451, 0.285387, 0.433451, 0.285387, 0.433451, 0.285387, 0.432267, 0.289529, 0.433451, 0.285387, 0.433451, 0.285387, 0.432267, 0.289529, 0.432267, 0.289529, 0.432267, 0.289529, 0.430901, 0.290909, 0.432267, 0.289529, 0.432267, 0.289529, 0.430901, 0.290909, 0.430901, 0.290909, 0.430901, 0.290909, 0.429243, 0.289244, 0.430901, 0.290909, 0.430901, 0.290909, 0.429243, 0.289244, 0.429243, 0.289244, 0.429243, 0.289244, 0.427770, 0.284251, 0.429243, 0.289244, 0.429243, 0.289244, 0.427770, 0.284251, 0.427770, 0.284251, 0.427770, 0.284251, 0.426482, 0.275928, 0.427770, 0.284251, 0.427770, 0.284251, 0.426482, 0.275928, 0.426482, 0.275928, 0.426482, 0.275928, 0.425379, 0.264276, 0.426482, 0.275928, 0.426482, 0.275928, 0.425379, 0.264276, 0.425379, 0.264276, 0.425379, 0.264276, 0.424495, 0.249747, 0.425379, 0.264276, 0.425379, 0.264276, 0.424495, 0.249747, 0.424495, 0.249747, 0.424495, 0.249747, 0.423863, 0.232795, 0.424495, 0.249747, 0.424495, 0.249747, 0.423863, 0.232795, 0.423863, 0.232795, 0.423863, 0.232795, 0.423484, 0.213419, 0.423863, 0.232795, 0.423863, 0.232795, 0.423484, 0.213419, 0.423484, 0.213419, 0.423484, 0.213419, 0.423358, 0.191619, 0.423484, 0.213419, 0.423484, 0.213419, 0.423358, 0.191619, 0.423358, 0.191619, 0.423358, 0.191619, 0.423358, 0.009091, 0.423358, 0.191619, 0.423358, 0.191619, 0.423358, 0.009091, 0.423358, 0.009091, 0.423358, 0.009091, 0.419011, 0.009091, 0.423358, 0.009091, 0.423358, 0.009091, 0.419011, 0.009091, 0.419011, 0.009091, 0.419011, 0.009091, 0.419011, 0.327273, 0.419011, 0.009091, 0.419011, 0.009091, 0.419011, 0.327273, 0.419011, 0.327273, 0.419011, 0.327273, 0.423358, 0.327273, 0.419011, 0.327273, 0.419011, 0.327273, 0.423358, 0.327273, 0.423358, 0.327273, 0.423358, 0.327273, 0.423358, 0.281818, 0.423358, 0.327273, 0.423358, 0.327273, 0.423358, 0.281818, 0.423358, 0.281818, 0.423358, 0.281818, 0.424168, 0.294660, 0.423358, 0.281818, 0.423358, 0.281818, 0.424168, 0.294660, 0.424168, 0.294660, 0.424168, 0.294660, 0.425047, 0.305771, 0.424168, 0.294660, 0.424168, 0.294660, 0.425047, 0.305771, 0.425047, 0.305771, 0.425047, 0.305771, 0.425995, 0.315150, 0.425047, 0.305771, 0.425047, 0.305771, 0.425995, 0.315150, 0.425995, 0.315150, 0.425995, 0.315150, 0.427012, 0.322798, 0.425995, 0.315150, 0.425995, 0.315150, 0.427012, 0.322798, 0.427012, 0.322798, 0.427012, 0.322798, 0.428104, 0.328733, 0.427012, 0.322798, 0.427012, 0.322798, 0.428104, 0.328733, 0.428104, 0.328733, 0.428104, 0.328733, 0.429277, 0.332972, 0.428104, 0.328733, 0.428104, 0.328733, 0.429277, 0.332972, 0.429277, 0.332972, 0.429277, 0.332972, 0.430530, 0.335516, 0.429277, 0.332972, 0.429277, 0.332972, 0.430530, 0.335516, 0.430530, 0.335516, 0.430530, 0.335516, 0.431865, 0.336364, 0.430530, 0.335516, 0.430530, 0.335516, 0.431865, 0.336364, 0.431865, 0.336364, 0.431865, 0.336364, 0.433994, 0.334264, 0.431865, 0.336364, 0.431865, 0.336364, 0.433994, 0.334264, 0.433994, 0.334264, 0.433994, 0.334264, 0.435848, 0.327965, 0.433994, 0.334264, 0.433994, 0.334264, 0.435848, 0.327965, 0.435848, 0.327965, 0.435848, 0.327965, 0.437425, 0.317467, 0.435848, 0.327965, 0.435848, 0.327965, 0.437425, 0.317467, 0.437425, 0.317467, 0.437425, 0.317467, 0.438726, 0.302770, 0.437425, 0.317467, 0.437425, 0.317467, 0.438726, 0.302770, 0.438726, 0.302770, 0.438726, 0.302770, 0.439744, 0.283998, 0.438726, 0.302770, 0.438726, 0.302770, 0.439744, 0.283998, 0.439744, 0.283998, 0.439744, 0.283998, 0.440471, 0.261275, 0.439744, 0.283998, 0.439744, 0.283998, 0.440471, 0.261275, 0.440471, 0.261275, 0.440471, 0.261275, 0.440907, 0.234601, 0.440471, 0.261275, 0.440471, 0.261275, 0.440907, 0.234601, 0.440907, 0.234601, 0.423358, 0.327273, 0.419011, 0.327273, 0.423358, 0.281818, 0.419011, 0.327273, 0.423358, 0.327273, 0.423358, 0.281818, 0.419011, 0.327273, 0.423863, 0.232795, 0.423358, 0.281818, 0.423863, 0.232795, 0.419011, 0.327273, 0.423358, 0.281818, 0.423863, 0.232795, 0.419011, 0.327273, 0.423484, 0.213419, 0.419011, 0.327273, 0.423863, 0.232795, 0.423484, 0.213419, 0.423484, 0.213419, 0.419011, 0.327273, 0.423358, 0.191619, 0.419011, 0.327273, 0.423484, 0.213419, 0.423358, 0.191619, 0.419011, 0.327273, 0.419011, 0.009091, 0.423358, 0.191619, 0.419011, 0.009091, 0.419011, 0.327273, 0.423358, 0.191619, 0.419011, 0.009091, 0.423358, 0.009091, 0.423358, 0.191619, 0.423358, 0.009091, 0.419011, 0.009091, 0.423358, 0.191619, 0.423358, 0.281818, 0.423863, 0.232795, 0.424495, 0.249747, 0.423863, 0.232795, 0.423358, 0.281818, 0.424495, 0.249747, 0.423358, 0.281818, 0.424495, 0.249747, 0.425379, 0.264276, 0.424495, 0.249747, 0.423358, 0.281818, 0.425379, 0.264276, 0.424168, 0.294660, 0.423358, 0.281818, 0.425379, 0.264276, 0.423358, 0.281818, 0.424168, 0.294660, 0.425379, 0.264276, 0.424168, 0.294660, 0.425379, 0.264276, 0.426482, 0.275928, 0.425379, 0.264276, 0.424168, 0.294660, 0.426482, 0.275928, 0.425047, 0.305771, 0.424168, 0.294660, 0.426482, 0.275928, 0.424168, 0.294660, 0.425047, 0.305771, 0.426482, 0.275928, 0.427770, 0.284251, 0.425047, 0.305771, 0.426482, 0.275928, 0.425047, 0.305771, 0.427770, 0.284251, 0.426482, 0.275928, 0.427770, 0.284251, 0.425995, 0.315150, 0.425047, 0.305771, 0.425995, 0.315150, 0.427770, 0.284251, 0.425047, 0.305771, 0.427012, 0.322798, 0.425995, 0.315150, 0.427770, 0.284251, 0.425995, 0.315150, 0.427012, 0.322798, 0.427770, 0.284251, 0.427012, 0.322798, 0.427770, 0.284251, 0.429243, 0.289244, 0.427770, 0.284251, 0.427012, 0.322798, 0.429243, 0.289244, 0.428104, 0.328733, 0.427012, 0.322798, 0.429243, 0.289244, 0.427012, 0.322798, 0.428104, 0.328733, 0.429243, 0.289244, 0.428104, 0.328733, 0.429243, 0.289244, 0.429277, 0.332972, 0.429243, 0.289244, 0.428104, 0.328733, 0.429277, 0.332972, 0.429277, 0.332972, 0.429243, 0.289244, 0.430901, 0.290909, 0.429243, 0.289244, 0.429277, 0.332972, 0.430901, 0.290909, 0.430530, 0.335516, 0.429277, 0.332972, 0.430901, 0.290909, 0.429277, 0.332972, 0.430530, 0.335516, 0.430901, 0.290909, 0.431865, 0.336364, 0.430530, 0.335516, 0.430901, 0.290909, 0.430530, 0.335516, 0.431865, 0.336364, 0.430901, 0.290909, 0.431865, 0.336364, 0.430901, 0.290909, 0.432267, 0.289529, 0.430901, 0.290909, 0.431865, 0.336364, 0.432267, 0.289529, 0.433994, 0.334264, 0.431865, 0.336364, 0.432267, 0.289529, 0.431865, 0.336364, 0.433994, 0.334264, 0.432267, 0.289529, 0.432267, 0.289529, 0.433451, 0.285387, 0.433994, 0.334264, 0.433451, 0.285387, 0.432267, 0.289529, 0.433994, 0.334264, 0.433994, 0.334264, 0.433451, 0.285387, 0.435848, 0.327965, 0.433451, 0.285387, 0.433994, 0.334264, 0.435848, 0.327965, 0.433451, 0.285387, 0.434453, 0.278485, 0.435848, 0.327965, 0.434453, 0.278485, 0.433451, 0.285387, 0.435848, 0.327965, 0.435848, 0.327965, 0.434453, 0.278485, 0.437425, 0.317467, 0.434453, 0.278485, 0.435848, 0.327965, 0.437425, 0.317467, 0.434453, 0.278485, 0.435272, 0.268821, 0.437425, 0.317467, 0.435272, 0.268821, 0.434453, 0.278485, 0.437425, 0.317467, 0.437425, 0.317467, 0.435272, 0.268821, 0.438726, 0.302770, 0.435272, 0.268821, 0.437425, 0.317467, 0.438726, 0.302770, 0.438726, 0.302770, 0.435272, 0.268821, 0.439744, 0.283998, 0.435272, 0.268821, 0.438726, 0.302770, 0.439744, 0.283998, 0.439744, 0.283998, 0.435272, 0.268821, 0.435909, 0.256379, 0.435272, 0.268821, 0.439744, 0.283998, 0.435909, 0.256379, 0.439744, 0.283998, 0.435909, 0.256379, 0.440471, 0.261275, 0.435909, 0.256379, 0.439744, 0.283998, 0.440471, 0.261275, 0.435909, 0.256379, 0.436365, 0.241140, 0.440471, 0.261275, 0.436365, 0.241140, 0.435909, 0.256379, 0.440471, 0.261275, 0.440471, 0.261275, 0.436365, 0.241140, 0.440907, 0.234601, 0.436365, 0.241140, 0.440471, 0.261275, 0.440907, 0.234601, 0.436365, 0.241140, 0.436638, 0.223105, 0.440907, 0.234601, 0.436638, 0.223105, 0.436365, 0.241140, 0.440907, 0.234601, 0.440907, 0.234601, 0.436638, 0.223105, 0.441053, 0.203977, 0.436638, 0.223105, 0.440907, 0.234601, 0.441053, 0.203977, 0.441053, 0.203977, 0.436638, 0.223105, 0.436729, 0.202273, 0.436638, 0.223105, 0.441053, 0.203977, 0.436729, 0.202273, 0.436729, 0.202273, 0.441053, 0.009091, 0.441053, 0.203977, 0.441053, 0.009091, 0.436729, 0.202273, 0.441053, 0.203977, 0.436729, 0.202273, 0.436729, 0.009091, 0.441053, 0.009091, 0.436729, 0.009091, 0.436729, 0.202273, 0.441053, 0.009091, 0.465233, 0.436364, 0.485395, 0.436364, 0.465233, 0.436364, 0.465233, 0.436364, 0.485395, 0.436364, 0.485395, 0.436364, 0.485395, 0.436364, 0.485395, 0.390909, 0.485395, 0.436364, 0.485395, 0.436364, 0.485395, 0.390909, 0.485395, 0.390909, 0.485395, 0.390909, 0.469980, 0.390909, 0.485395, 0.390909, 0.485395, 0.390909, 0.469980, 0.390909, 0.469980, 0.390909, 0.469980, 0.390909, 0.469980, 0.263636, 0.469980, 0.390909, 0.469980, 0.390909, 0.469980, 0.263636, 0.469980, 0.263636, 0.469980, 0.263636, 0.483891, 0.263636, 0.469980, 0.263636, 0.469980, 0.263636, 0.483891, 0.263636, 0.483891, 0.263636, 0.483891, 0.263636, 0.483891, 0.218182, 0.483891, 0.263636, 0.483891, 0.263636, 0.483891, 0.218182, 0.483891, 0.218182, 0.483891, 0.218182, 0.469980, 0.218182, 0.483891, 0.218182, 0.483891, 0.218182, 0.469980, 0.218182, 0.469980, 0.218182, 0.469980, 0.218182, 0.469980, 0.009091, 0.469980, 0.218182, 0.469980, 0.218182, 0.469980, 0.009091, 0.469980, 0.009091, 0.469980, 0.009091, 0.465233, 0.009091, 0.469980, 0.009091, 0.469980, 0.009091, 0.465233, 0.009091, 0.465233, 0.009091, 0.465233, 0.436364, 0.469980, 0.390909, 0.485395, 0.436364, 0.469980, 0.390909, 0.465233, 0.436364, 0.485395, 0.436364, 0.469980, 0.390909, 0.465233, 0.436364, 0.469980, 0.263636, 0.465233, 0.436364, 0.469980, 0.390909, 0.469980, 0.263636, 0.469980, 0.263636, 0.465233, 0.436364, 0.469980, 0.218182, 0.465233, 0.436364, 0.469980, 0.263636, 0.469980, 0.218182, 0.469980, 0.263636, 0.469980, 0.218182, 0.483891, 0.218182, 0.469980, 0.218182, 0.469980, 0.263636, 0.483891, 0.218182, 0.469980, 0.263636, 0.483891, 0.218182, 0.483891, 0.263636, 0.483891, 0.218182, 0.469980, 0.263636, 0.483891, 0.263636, 0.465233, 0.436364, 0.465233, 0.009091, 0.469980, 0.218182, 0.465233, 0.009091, 0.465233, 0.436364, 0.469980, 0.218182, 0.465233, 0.009091, 0.469980, 0.009091, 0.469980, 0.218182, 0.469980, 0.009091, 0.465233, 0.009091, 0.469980, 0.218182, 0.469980, 0.390909, 0.485395, 0.390909, 0.485395, 0.436364, 0.485395, 0.390909, 0.469980, 0.390909, 0.485395, 0.436364, 0.500317, 0.163636, 0.497898, 0.162775, 0.500317, 0.163636, 0.500317, 0.163636, 0.497898, 0.162775, 0.497898, 0.162775, 0.497898, 0.162775, 0.495882, 0.160192, 0.497898, 0.162775, 0.497898, 0.162775, 0.495882, 0.160192, 0.495882, 0.160192, 0.495882, 0.160192, 0.494268, 0.155886, 0.495882, 0.160192, 0.495882, 0.160192, 0.494268, 0.155886, 0.494268, 0.155886, 0.494268, 0.155886, 0.493056, 0.149858, 0.494268, 0.155886, 0.494268, 0.155886, 0.493056, 0.149858, 0.493056, 0.149858, 0.493056, 0.149858, 0.492172, 0.141761, 0.493056, 0.149858, 0.493056, 0.149858, 0.492172, 0.141761, 0.492172, 0.141761, 0.492172, 0.141761, 0.491540, 0.131250, 0.492172, 0.141761, 0.492172, 0.141761, 0.491540, 0.131250, 0.491540, 0.131250, 0.491540, 0.131250, 0.491161, 0.118324, 0.491540, 0.131250, 0.491540, 0.131250, 0.491161, 0.118324, 0.491161, 0.118324, 0.491161, 0.118324, 0.491035, 0.102983, 0.491161, 0.118324, 0.491161, 0.118324, 0.491035, 0.102983, 0.491035, 0.102983, 0.491035, 0.102983, 0.491130, 0.090394, 0.491035, 0.102983, 0.491035, 0.102983, 0.491130, 0.090394, 0.491130, 0.090394, 0.491130, 0.090394, 0.491414, 0.079190, 0.491130, 0.090394, 0.491130, 0.090394, 0.491414, 0.079190, 0.491414, 0.079190, 0.491414, 0.079190, 0.491888, 0.069371, 0.491414, 0.079190, 0.491414, 0.079190, 0.491888, 0.069371, 0.491888, 0.069371, 0.491888, 0.069371, 0.492551, 0.060937, 0.491888, 0.069371, 0.491888, 0.069371, 0.492551, 0.060937, 0.492551, 0.060937, 0.492551, 0.060937, 0.493377, 0.054164, 0.492551, 0.060937, 0.492551, 0.060937, 0.493377, 0.054164, 0.493377, 0.054164, 0.493377, 0.054164, 0.494340, 0.049325, 0.493377, 0.054164, 0.493377, 0.054164, 0.494340, 0.049325, 0.494340, 0.049325, 0.494340, 0.049325, 0.495439, 0.046422, 0.494340, 0.049325, 0.494340, 0.049325, 0.495439, 0.046422, 0.495439, 0.046422, 0.495439, 0.046422, 0.496675, 0.045455, 0.495439, 0.046422, 0.495439, 0.046422, 0.496675, 0.045455, 0.496675, 0.045455, 0.496675, 0.045455, 0.498384, 0.047279, 0.496675, 0.045455, 0.496675, 0.045455, 0.498384, 0.047279, 0.498384, 0.047279, 0.498384, 0.047279, 0.499915, 0.052752, 0.498384, 0.047279, 0.498384, 0.047279, 0.499915, 0.052752, 0.499915, 0.052752, 0.499915, 0.052752, 0.501268, 0.061874, 0.499915, 0.052752, 0.499915, 0.052752, 0.501268, 0.061874, 0.501268, 0.061874, 0.501268, 0.061874, 0.502444, 0.074645, 0.501268, 0.061874, 0.501268, 0.061874, 0.502444, 0.074645, 0.502444, 0.074645, 0.502444, 0.074645, 0.503395, 0.090443, 0.502444, 0.074645, 0.502444, 0.074645, 0.503395, 0.090443, 0.503395, 0.090443, 0.503395, 0.090443, 0.504074, 0.108647, 0.503395, 0.090443, 0.503395, 0.090443, 0.504074, 0.108647, 0.504074, 0.108647, 0.504074, 0.108647, 0.504482, 0.129257, 0.504074, 0.108647, 0.504074, 0.108647, 0.504482, 0.129257, 0.504482, 0.129257, 0.504482, 0.129257, 0.504618, 0.152273, 0.504482, 0.129257, 0.504482, 0.129257, 0.504618, 0.152273, 0.504618, 0.152273, 0.504618, 0.152273, 0.504618, 0.163636, 0.504618, 0.152273, 0.504618, 0.152273, 0.504618, 0.163636, 0.504618, 0.163636, 0.508941, 0.187642, 0.508941, 0.009091, 0.508941, 0.187642, 0.508941, 0.187642, 0.508941, 0.009091, 0.508941, 0.009091, 0.508941, 0.009091, 0.504618, 0.009091, 0.508941, 0.009091, 0.508941, 0.009091, 0.504618, 0.009091, 0.504618, 0.009091, 0.504618, 0.009091, 0.504618, 0.054545, 0.504618, 0.009091, 0.504618, 0.009091, 0.504618, 0.054545, 0.504618, 0.054545, 0.504618, 0.054545, 0.503832, 0.041473, 0.504618, 0.054545, 0.504618, 0.054545, 0.503832, 0.041473, 0.503832, 0.041473, 0.503832, 0.041473, 0.502955, 0.030238, 0.503832, 0.041473, 0.503832, 0.041473, 0.502955, 0.030238, 0.502955, 0.030238, 0.502955, 0.030238, 0.501987, 0.020841, 0.502955, 0.030238, 0.502955, 0.030238, 0.501987, 0.020841, 0.501987, 0.020841, 0.501987, 0.020841, 0.500928, 0.013281, 0.501987, 0.020841, 0.501987, 0.020841, 0.500928, 0.013281, 0.500928, 0.013281, 0.500928, 0.013281, 0.499762, 0.007471, 0.500928, 0.013281, 0.500928, 0.013281, 0.499762, 0.007471, 0.499762, 0.007471, 0.499762, 0.007471, 0.498473, 0.003320, 0.499762, 0.007471, 0.499762, 0.007471, 0.498473, 0.003320, 0.498473, 0.003320, 0.498473, 0.003320, 0.497060, 0.000830, 0.498473, 0.003320, 0.498473, 0.003320, 0.497060, 0.000830, 0.497060, 0.000830, 0.497060, 0.000830, 0.495523, 0, 0.497060, 0.000830, 0.497060, 0.000830, 0.495523, 0, 0.495523, 0, 0.495523, 0, 0.493606, 0.001713, 0.495523, 0, 0.495523, 0, 0.493606, 0.001713, 0.493606, 0.001713, 0.493606, 0.001713, 0.491896, 0.006854, 0.493606, 0.001713, 0.493606, 0.001713, 0.491896, 0.006854, 0.491896, 0.006854, 0.491896, 0.006854, 0.490393, 0.015421, 0.491896, 0.006854, 0.491896, 0.006854, 0.490393, 0.015421, 0.490393, 0.015421, 0.490393, 0.015421, 0.489096, 0.027415, 0.490393, 0.015421, 0.490393, 0.015421, 0.489096, 0.027415, 0.489096, 0.027415, 0.489096, 0.027415, 0.488053, 0.042285, 0.489096, 0.027415, 0.489096, 0.027415, 0.488053, 0.042285, 0.488053, 0.042285, 0.488053, 0.042285, 0.487308, 0.059482, 0.488053, 0.042285, 0.488053, 0.042285, 0.487308, 0.059482, 0.487308, 0.059482, 0.487308, 0.059482, 0.486860, 0.079004, 0.487308, 0.059482, 0.487308, 0.059482, 0.486860, 0.079004, 0.486860, 0.079004, 0.486860, 0.079004, 0.486711, 0.100852, 0.486860, 0.079004, 0.486860, 0.079004, 0.486711, 0.100852, 0.486711, 0.100852, 0.486711, 0.100852, 0.486897, 0.126048, 0.486711, 0.100852, 0.486711, 0.100852, 0.486897, 0.126048, 0.486897, 0.126048, 0.486897, 0.126048, 0.487455, 0.147940, 0.486897, 0.126048, 0.486897, 0.126048, 0.487455, 0.147940, 0.487455, 0.147940, 0.487455, 0.147940, 0.488383, 0.166531, 0.487455, 0.147940, 0.487455, 0.147940, 0.488383, 0.166531, 0.488383, 0.166531, 0.488383, 0.166531, 0.489684, 0.181818, 0.488383, 0.166531, 0.488383, 0.166531, 0.489684, 0.181818, 0.489684, 0.181818, 0.489684, 0.181818, 0.491353, 0.193750, 0.489684, 0.181818, 0.489684, 0.181818, 0.491353, 0.193750, 0.491353, 0.193750, 0.491353, 0.193750, 0.493388, 0.202273, 0.491353, 0.193750, 0.491353, 0.193750, 0.493388, 0.202273, 0.493388, 0.202273, 0.493388, 0.202273, 0.495789, 0.207386, 0.493388, 0.202273, 0.493388, 0.202273, 0.495789, 0.207386, 0.495789, 0.207386, 0.495789, 0.207386, 0.498555, 0.209091, 0.495789, 0.207386, 0.495789, 0.207386, 0.498555, 0.209091, 0.498555, 0.209091, 0.498555, 0.209091, 0.504618, 0.209091, 0.498555, 0.209091, 0.498555, 0.209091, 0.504618, 0.209091, 0.504618, 0.209091, 0.504618, 0.209091, 0.504618, 0.214205, 0.504618, 0.209091, 0.504618, 0.209091, 0.504618, 0.214205, 0.504618, 0.214205, 0.504618, 0.214205, 0.504495, 0.231490, 0.504618, 0.214205, 0.504618, 0.214205, 0.504495, 0.231490, 0.504495, 0.231490, 0.504495, 0.231490, 0.504127, 0.246697, 0.504495, 0.231490, 0.504495, 0.231490, 0.504127, 0.246697, 0.504127, 0.246697, 0.504127, 0.246697, 0.503514, 0.259828, 0.504127, 0.246697, 0.504127, 0.246697, 0.503514, 0.259828, 0.503514, 0.259828, 0.503514, 0.259828, 0.502655, 0.270881, 0.503514, 0.259828, 0.503514, 0.259828, 0.502655, 0.270881, 0.502655, 0.270881, 0.502655, 0.270881, 0.501575, 0.279643, 0.502655, 0.270881, 0.502655, 0.270881, 0.501575, 0.279643, 0.501575, 0.279643, 0.501575, 0.279643, 0.500297, 0.285902, 0.501575, 0.279643, 0.501575, 0.279643, 0.500297, 0.285902, 0.500297, 0.285902, 0.500297, 0.285902, 0.498820, 0.289657, 0.500297, 0.285902, 0.500297, 0.285902, 0.498820, 0.289657, 0.498820, 0.289657, 0.498820, 0.289657, 0.497145, 0.290909, 0.498820, 0.289657, 0.498820, 0.289657, 0.497145, 0.290909, 0.497145, 0.290909, 0.497145, 0.290909, 0.496024, 0.290483, 0.497145, 0.290909, 0.497145, 0.290909, 0.496024, 0.290483, 0.496024, 0.290483, 0.496024, 0.290483, 0.494918, 0.289205, 0.496024, 0.290483, 0.496024, 0.290483, 0.494918, 0.289205, 0.494918, 0.289205, 0.494918, 0.289205, 0.493827, 0.287074, 0.494918, 0.289205, 0.494918, 0.289205, 0.493827, 0.287074, 0.493827, 0.287074, 0.493827, 0.287074, 0.492751, 0.284091, 0.493827, 0.287074, 0.493827, 0.287074, 0.492751, 0.284091, 0.492751, 0.284091, 0.492751, 0.284091, 0.491692, 0.280256, 0.492751, 0.284091, 0.492751, 0.284091, 0.491692, 0.280256, 0.491692, 0.280256, 0.491692, 0.280256, 0.490653, 0.275568, 0.491692, 0.280256, 0.491692, 0.280256, 0.490653, 0.275568, 0.490653, 0.275568, 0.490653, 0.275568, 0.489635, 0.270028, 0.490653, 0.275568, 0.490653, 0.275568, 0.489635, 0.270028, 0.489635, 0.270028, 0.489635, 0.270028, 0.488638, 0.263636, 0.489635, 0.270028, 0.489635, 0.270028, 0.488638, 0.263636, 0.488638, 0.263636, 0.488638, 0.263636, 0.488638, 0.309091, 0.488638, 0.263636, 0.488638, 0.263636, 0.488638, 0.309091, 0.488638, 0.309091, 0.488638, 0.309091, 0.489816, 0.315483, 0.488638, 0.309091, 0.488638, 0.309091, 0.489816, 0.315483, 0.489816, 0.315483, 0.489816, 0.315483, 0.490976, 0.321023, 0.489816, 0.315483, 0.489816, 0.315483, 0.490976, 0.321023, 0.490976, 0.321023, 0.490976, 0.321023, 0.492119, 0.325710, 0.490976, 0.321023, 0.490976, 0.321023, 0.492119, 0.325710, 0.492119, 0.325710, 0.492119, 0.325710, 0.493244, 0.329545, 0.492119, 0.325710, 0.492119, 0.325710, 0.493244, 0.329545, 0.493244, 0.329545, 0.493244, 0.329545, 0.494353, 0.332528, 0.493244, 0.329545, 0.493244, 0.329545, 0.494353, 0.332528, 0.494353, 0.332528, 0.494353, 0.332528, 0.495447, 0.334659, 0.494353, 0.332528, 0.494353, 0.332528, 0.495447, 0.334659, 0.495447, 0.334659, 0.495447, 0.334659, 0.496527, 0.335938, 0.495447, 0.334659, 0.495447, 0.334659, 0.496527, 0.335938, 0.496527, 0.335938, 0.496527, 0.335938, 0.497591, 0.336364, 0.496527, 0.335938, 0.496527, 0.335938, 0.497591, 0.336364, 0.497591, 0.336364, 0.497591, 0.336364, 0.500266, 0.334060, 0.497591, 0.336364, 0.497591, 0.336364, 0.500266, 0.334060, 0.500266, 0.334060, 0.500266, 0.334060, 0.502579, 0.327148, 0.500266, 0.334060, 0.500266, 0.334060, 0.502579, 0.327148, 0.502579, 0.327148, 0.502579, 0.327148, 0.504531, 0.315629, 0.502579, 0.327148, 0.502579, 0.327148, 0.504531, 0.315629, 0.504531, 0.315629, 0.504531, 0.315629, 0.506122, 0.299503, 0.504531, 0.315629, 0.504531, 0.315629, 0.506122, 0.299503, 0.506122, 0.299503, 0.506122, 0.299503, 0.507355, 0.278689, 0.506122, 0.299503, 0.506122, 0.299503, 0.507355, 0.278689, 0.507355, 0.278689, 0.507355, 0.278689, 0.508236, 0.253107, 0.507355, 0.278689, 0.507355, 0.278689, 0.508236, 0.253107, 0.508236, 0.253107, 0.508236, 0.253107, 0.508765, 0.222758, 0.508236, 0.253107, 0.508236, 0.253107, 0.508765, 0.222758, 0.508765, 0.222758, 0.486897, 0.126048, 0.486711, 0.100852, 0.491035, 0.102983, 0.486711, 0.100852, 0.486897, 0.126048, 0.491035, 0.102983, 0.486897, 0.126048, 0.491035, 0.102983, 0.491161, 0.118324, 0.491035, 0.102983, 0.486897, 0.126048, 0.491161, 0.118324, 0.487455, 0.147940, 0.486897, 0.126048, 0.491161, 0.118324, 0.486897, 0.126048, 0.487455, 0.147940, 0.491161, 0.118324, 0.487455, 0.147940, 0.491161, 0.118324, 0.491540, 0.131250, 0.491161, 0.118324, 0.487455, 0.147940, 0.491540, 0.131250, 0.488383, 0.166531, 0.487455, 0.147940, 0.491540, 0.131250, 0.487455, 0.147940, 0.488383, 0.166531, 0.491540, 0.131250, 0.488383, 0.166531, 0.491540, 0.131250, 0.492172, 0.141761, 0.491540, 0.131250, 0.488383, 0.166531, 0.492172, 0.141761, 0.489684, 0.181818, 0.488383, 0.166531, 0.492172, 0.141761, 0.488383, 0.166531, 0.489684, 0.181818, 0.492172, 0.141761, 0.489684, 0.181818, 0.492172, 0.141761, 0.493056, 0.149858, 0.492172, 0.141761, 0.489684, 0.181818, 0.493056, 0.149858, 0.491353, 0.193750, 0.489684, 0.181818, 0.493056, 0.149858, 0.489684, 0.181818, 0.491353, 0.193750, 0.493056, 0.149858, 0.494268, 0.155886, 0.491353, 0.193750, 0.493056, 0.149858, 0.491353, 0.193750, 0.494268, 0.155886, 0.493056, 0.149858, 0.493388, 0.202273, 0.491353, 0.193750, 0.494268, 0.155886, 0.491353, 0.193750, 0.493388, 0.202273, 0.494268, 0.155886, 0.495882, 0.160192, 0.493388, 0.202273, 0.494268, 0.155886, 0.493388, 0.202273, 0.495882, 0.160192, 0.494268, 0.155886, 0.495789, 0.207386, 0.493388, 0.202273, 0.495882, 0.160192, 0.493388, 0.202273, 0.495789, 0.207386, 0.495882, 0.160192, 0.495789, 0.207386, 0.495882, 0.160192, 0.497898, 0.162775, 0.495882, 0.160192, 0.495789, 0.207386, 0.497898, 0.162775, 0.495789, 0.207386, 0.497898, 0.162775, 0.498555, 0.209091, 0.497898, 0.162775, 0.495789, 0.207386, 0.498555, 0.209091, 0.498555, 0.209091, 0.497898, 0.162775, 0.500317, 0.163636, 0.497898, 0.162775, 0.498555, 0.209091, 0.500317, 0.163636, 0.498555, 0.209091, 0.500317, 0.163636, 0.504618, 0.209091, 0.500317, 0.163636, 0.498555, 0.209091, 0.504618, 0.209091, 0.500317, 0.163636, 0.504618, 0.163636, 0.504618, 0.209091, 0.504618, 0.163636, 0.500317, 0.163636, 0.504618, 0.209091, 0.504618, 0.209091, 0.504618, 0.163636, 0.508941, 0.187642, 0.504618, 0.163636, 0.504618, 0.209091, 0.508941, 0.187642, 0.508941, 0.187642, 0.508765, 0.222758, 0.504618, 0.209091, 0.508765, 0.222758, 0.508941, 0.187642, 0.504618, 0.209091, 0.508765, 0.222758, 0.504618, 0.214205, 0.504618, 0.209091, 0.504618, 0.214205, 0.508765, 0.222758, 0.504618, 0.209091, 0.504495, 0.231490, 0.504618, 0.214205, 0.508765, 0.222758, 0.504618, 0.214205, 0.504495, 0.231490, 0.508765, 0.222758, 0.508236, 0.253107, 0.504495, 0.231490, 0.508765, 0.222758, 0.504495, 0.231490, 0.508236, 0.253107, 0.508765, 0.222758, 0.504127, 0.246697, 0.504495, 0.231490, 0.508236, 0.253107, 0.504495, 0.231490, 0.504127, 0.246697, 0.508236, 0.253107, 0.507355, 0.278689, 0.504127, 0.246697, 0.508236, 0.253107, 0.504127, 0.246697, 0.507355, 0.278689, 0.508236, 0.253107, 0.503514, 0.259828, 0.504127, 0.246697, 0.507355, 0.278689, 0.504127, 0.246697, 0.503514, 0.259828, 0.507355, 0.278689, 0.506122, 0.299503, 0.503514, 0.259828, 0.507355, 0.278689, 0.503514, 0.259828, 0.506122, 0.299503, 0.507355, 0.278689, 0.502655, 0.270881, 0.503514, 0.259828, 0.506122, 0.299503, 0.503514, 0.259828, 0.502655, 0.270881, 0.506122, 0.299503, 0.504531, 0.315629, 0.502655, 0.270881, 0.506122, 0.299503, 0.502655, 0.270881, 0.504531, 0.315629, 0.506122, 0.299503, 0.501575, 0.279643, 0.502655, 0.270881, 0.504531, 0.315629, 0.502655, 0.270881, 0.501575, 0.279643, 0.504531, 0.315629, 0.502579, 0.327148, 0.501575, 0.279643, 0.504531, 0.315629, 0.501575, 0.279643, 0.502579, 0.327148, 0.504531, 0.315629, 0.500297, 0.285902, 0.501575, 0.279643, 0.502579, 0.327148, 0.501575, 0.279643, 0.500297, 0.285902, 0.502579, 0.327148, 0.500266, 0.334060, 0.500297, 0.285902, 0.502579, 0.327148, 0.500297, 0.285902, 0.500266, 0.334060, 0.502579, 0.327148, 0.500266, 0.334060, 0.498820, 0.289657, 0.500297, 0.285902, 0.498820, 0.289657, 0.500266, 0.334060, 0.500297, 0.285902, 0.500266, 0.334060, 0.497591, 0.336364, 0.498820, 0.289657, 0.497591, 0.336364, 0.500266, 0.334060, 0.498820, 0.289657, 0.497145, 0.290909, 0.498820, 0.289657, 0.497591, 0.336364, 0.498820, 0.289657, 0.497145, 0.290909, 0.497591, 0.336364, 0.497591, 0.336364, 0.496527, 0.335938, 0.497145, 0.290909, 0.496527, 0.335938, 0.497591, 0.336364, 0.497145, 0.290909, 0.496527, 0.335938, 0.496024, 0.290483, 0.497145, 0.290909, 0.496024, 0.290483, 0.496527, 0.335938, 0.497145, 0.290909, 0.495447, 0.334659, 0.496024, 0.290483, 0.496527, 0.335938, 0.496024, 0.290483, 0.495447, 0.334659, 0.496527, 0.335938, 0.495447, 0.334659, 0.494918, 0.289205, 0.496024, 0.290483, 0.494918, 0.289205, 0.495447, 0.334659, 0.496024, 0.290483, 0.494353, 0.332528, 0.494918, 0.289205, 0.495447, 0.334659, 0.494918, 0.289205, 0.494353, 0.332528, 0.495447, 0.334659, 0.494353, 0.332528, 0.493244, 0.329545, 0.494918, 0.289205, 0.493244, 0.329545, 0.494353, 0.332528, 0.494918, 0.289205, 0.494918, 0.289205, 0.493244, 0.329545, 0.493827, 0.287074, 0.493244, 0.329545, 0.494918, 0.289205, 0.493827, 0.287074, 0.493244, 0.329545, 0.492119, 0.325710, 0.493827, 0.287074, 0.492119, 0.325710, 0.493244, 0.329545, 0.493827, 0.287074, 0.492119, 0.325710, 0.492751, 0.284091, 0.493827, 0.287074, 0.492751, 0.284091, 0.492119, 0.325710, 0.493827, 0.287074, 0.492119, 0.325710, 0.490976, 0.321023, 0.492751, 0.284091, 0.490976, 0.321023, 0.492119, 0.325710, 0.492751, 0.284091, 0.490976, 0.321023, 0.491692, 0.280256, 0.492751, 0.284091, 0.491692, 0.280256, 0.490976, 0.321023, 0.492751, 0.284091, 0.490976, 0.321023, 0.489816, 0.315483, 0.491692, 0.280256, 0.489816, 0.315483, 0.490976, 0.321023, 0.491692, 0.280256, 0.489816, 0.315483, 0.490653, 0.275568, 0.491692, 0.280256, 0.490653, 0.275568, 0.489816, 0.315483, 0.491692, 0.280256, 0.489816, 0.315483, 0.488638, 0.309091, 0.490653, 0.275568, 0.488638, 0.309091, 0.489816, 0.315483, 0.490653, 0.275568, 0.488638, 0.309091, 0.489635, 0.270028, 0.490653, 0.275568, 0.489635, 0.270028, 0.488638, 0.309091, 0.490653, 0.275568, 0.488638, 0.309091, 0.488638, 0.263636, 0.489635, 0.270028, 0.488638, 0.263636, 0.488638, 0.309091, 0.489635, 0.270028, 0.508941, 0.187642, 0.504618, 0.163636, 0.504618, 0.152273, 0.504618, 0.163636, 0.508941, 0.187642, 0.504618, 0.152273, 0.508941, 0.187642, 0.504618, 0.152273, 0.504482, 0.129257, 0.504618, 0.152273, 0.508941, 0.187642, 0.504482, 0.129257, 0.508941, 0.009091, 0.508941, 0.187642, 0.504482, 0.129257, 0.508941, 0.187642, 0.508941, 0.009091, 0.504482, 0.129257, 0.508941, 0.009091, 0.504482, 0.129257, 0.504074, 0.108647, 0.504482, 0.129257, 0.508941, 0.009091, 0.504074, 0.108647, 0.504618, 0.054545, 0.508941, 0.009091, 0.504074, 0.108647, 0.508941, 0.009091, 0.504618, 0.054545, 0.504074, 0.108647, 0.504618, 0.054545, 0.504618, 0.009091, 0.508941, 0.009091, 0.504618, 0.009091, 0.504618, 0.054545, 0.508941, 0.009091, 0.504074, 0.108647, 0.503395, 0.090443, 0.504618, 0.054545, 0.503395, 0.090443, 0.504074, 0.108647, 0.504618, 0.054545, 0.503395, 0.090443, 0.502444, 0.074645, 0.504618, 0.054545, 0.502444, 0.074645, 0.503395, 0.090443, 0.504618, 0.054545, 0.502444, 0.074645, 0.503832, 0.041473, 0.504618, 0.054545, 0.503832, 0.041473, 0.502444, 0.074645, 0.504618, 0.054545, 0.502444, 0.074645, 0.501268, 0.061874, 0.503832, 0.041473, 0.501268, 0.061874, 0.502444, 0.074645, 0.503832, 0.041473, 0.501268, 0.061874, 0.502955, 0.030238, 0.503832, 0.041473, 0.502955, 0.030238, 0.501268, 0.061874, 0.503832, 0.041473, 0.502955, 0.030238, 0.501268, 0.061874, 0.501987, 0.020841, 0.501268, 0.061874, 0.502955, 0.030238, 0.501987, 0.020841, 0.501268, 0.061874, 0.499915, 0.052752, 0.501987, 0.020841, 0.499915, 0.052752, 0.501268, 0.061874, 0.501987, 0.020841, 0.499915, 0.052752, 0.500928, 0.013281, 0.501987, 0.020841, 0.500928, 0.013281, 0.499915, 0.052752, 0.501987, 0.020841, 0.500928, 0.013281, 0.499915, 0.052752, 0.499762, 0.007471, 0.499915, 0.052752, 0.500928, 0.013281, 0.499762, 0.007471, 0.499915, 0.052752, 0.498384, 0.047279, 0.499762, 0.007471, 0.498384, 0.047279, 0.499915, 0.052752, 0.499762, 0.007471, 0.498384, 0.047279, 0.498473, 0.003320, 0.499762, 0.007471, 0.498473, 0.003320, 0.498384, 0.047279, 0.499762, 0.007471, 0.498384, 0.047279, 0.496675, 0.045455, 0.498473, 0.003320, 0.496675, 0.045455, 0.498384, 0.047279, 0.498473, 0.003320, 0.496675, 0.045455, 0.497060, 0.000830, 0.498473, 0.003320, 0.497060, 0.000830, 0.496675, 0.045455, 0.498473, 0.003320, 0.496675, 0.045455, 0.495523, 0, 0.497060, 0.000830, 0.495523, 0, 0.496675, 0.045455, 0.497060, 0.000830, 0.495439, 0.046422, 0.495523, 0, 0.496675, 0.045455, 0.495523, 0, 0.495439, 0.046422, 0.496675, 0.045455, 0.495439, 0.046422, 0.493606, 0.001713, 0.495523, 0, 0.493606, 0.001713, 0.495439, 0.046422, 0.495523, 0, 0.494340, 0.049325, 0.493606, 0.001713, 0.495439, 0.046422, 0.493606, 0.001713, 0.494340, 0.049325, 0.495439, 0.046422, 0.491896, 0.006854, 0.493606, 0.001713, 0.494340, 0.049325, 0.493606, 0.001713, 0.491896, 0.006854, 0.494340, 0.049325, 0.493377, 0.054164, 0.491896, 0.006854, 0.494340, 0.049325, 0.491896, 0.006854, 0.493377, 0.054164, 0.494340, 0.049325, 0.490393, 0.015421, 0.491896, 0.006854, 0.493377, 0.054164, 0.491896, 0.006854, 0.490393, 0.015421, 0.493377, 0.054164, 0.493377, 0.054164, 0.492551, 0.060937, 0.490393, 0.015421, 0.492551, 0.060937, 0.493377, 0.054164, 0.490393, 0.015421, 0.492551, 0.060937, 0.489096, 0.027415, 0.490393, 0.015421, 0.489096, 0.027415, 0.492551, 0.060937, 0.490393, 0.015421, 0.492551, 0.060937, 0.491888, 0.069371, 0.489096, 0.027415, 0.491888, 0.069371, 0.492551, 0.060937, 0.489096, 0.027415, 0.491888, 0.069371, 0.488053, 0.042285, 0.489096, 0.027415, 0.488053, 0.042285, 0.491888, 0.069371, 0.489096, 0.027415, 0.491888, 0.069371, 0.491414, 0.079190, 0.488053, 0.042285, 0.491414, 0.079190, 0.491888, 0.069371, 0.488053, 0.042285, 0.491414, 0.079190, 0.487308, 0.059482, 0.488053, 0.042285, 0.487308, 0.059482, 0.491414, 0.079190, 0.488053, 0.042285, 0.486860, 0.079004, 0.487308, 0.059482, 0.491414, 0.079190, 0.487308, 0.059482, 0.486860, 0.079004, 0.491414, 0.079190, 0.491130, 0.090394, 0.486860, 0.079004, 0.491414, 0.079190, 0.486860, 0.079004, 0.491130, 0.090394, 0.491414, 0.079190, 0.486711, 0.100852, 0.486860, 0.079004, 0.491130, 0.090394, 0.486860, 0.079004, 0.486711, 0.100852, 0.491130, 0.090394, 0.491035, 0.102983, 0.486711, 0.100852, 0.491130, 0.090394, 0.486711, 0.100852, 0.491035, 0.102983, 0.491130, 0.090394, 0.517237, 0.132244, 0.517237, 0.327273, 0.517237, 0.132244, 0.517237, 0.132244, 0.517237, 0.327273, 0.517237, 0.327273, 0.517237, 0.327273, 0.521560, 0.327273, 0.517237, 0.327273, 0.517237, 0.327273, 0.521560, 0.327273, 0.521560, 0.327273, 0.521560, 0.327273, 0.521560, 0.134233, 0.521560, 0.327273, 0.521560, 0.327273, 0.521560, 0.134233, 0.521560, 0.134233, 0.521560, 0.134233, 0.521651, 0.113397, 0.521560, 0.134233, 0.521560, 0.134233, 0.521651, 0.113397, 0.521651, 0.113397, 0.521651, 0.113397, 0.521925, 0.095348, 0.521651, 0.113397, 0.521651, 0.113397, 0.521925, 0.095348, 0.521925, 0.095348, 0.521925, 0.095348, 0.522380, 0.080087, 0.521925, 0.095348, 0.521925, 0.095348, 0.522380, 0.080087, 0.522380, 0.080087, 0.522380, 0.080087, 0.523017, 0.067614, 0.522380, 0.080087, 0.522380, 0.080087, 0.523017, 0.067614, 0.523017, 0.067614, 0.523017, 0.067614, 0.523837, 0.057919, 0.523017, 0.067614, 0.523017, 0.067614, 0.523837, 0.057919, 0.523837, 0.057919, 0.523837, 0.057919, 0.524838, 0.050994, 0.523837, 0.057919, 0.523837, 0.057919, 0.524838, 0.050994, 0.524838, 0.050994, 0.524838, 0.050994, 0.526022, 0.046839, 0.524838, 0.050994, 0.524838, 0.050994, 0.526022, 0.046839, 0.526022, 0.046839, 0.526022, 0.046839, 0.527388, 0.045455, 0.526022, 0.046839, 0.526022, 0.046839, 0.527388, 0.045455, 0.527388, 0.045455, 0.527388, 0.045455, 0.529047, 0.047119, 0.527388, 0.045455, 0.527388, 0.045455, 0.529047, 0.047119, 0.529047, 0.047119, 0.529047, 0.047119, 0.530522, 0.052113, 0.529047, 0.047119, 0.529047, 0.047119, 0.530522, 0.052113, 0.530522, 0.052113, 0.530522, 0.052113, 0.531814, 0.060436, 0.530522, 0.052113, 0.530522, 0.052113, 0.531814, 0.060436, 0.531814, 0.060436, 0.531814, 0.060436, 0.532922, 0.072088, 0.531814, 0.060436, 0.531814, 0.060436, 0.532922, 0.072088, 0.532922, 0.072088, 0.532922, 0.072088, 0.533811, 0.086608, 0.532922, 0.072088, 0.532922, 0.072088, 0.533811, 0.086608, 0.533811, 0.086608, 0.533811, 0.086608, 0.534447, 0.103533, 0.533811, 0.086608, 0.533811, 0.086608, 0.534447, 0.103533, 0.534447, 0.103533, 0.534447, 0.103533, 0.534828, 0.122865, 0.534447, 0.103533, 0.534447, 0.103533, 0.534828, 0.122865, 0.534828, 0.122865, 0.534828, 0.122865, 0.534955, 0.144602, 0.534828, 0.122865, 0.534828, 0.122865, 0.534955, 0.144602, 0.534955, 0.144602, 0.534955, 0.144602, 0.534955, 0.327273, 0.534955, 0.144602, 0.534955, 0.144602, 0.534955, 0.327273, 0.534955, 0.327273, 0.534955, 0.327273, 0.539279, 0.327273, 0.534955, 0.327273, 0.534955, 0.327273, 0.539279, 0.327273, 0.539279, 0.327273, 0.539279, 0.327273, 0.539279, 0.009091, 0.539279, 0.327273, 0.539279, 0.327273, 0.539279, 0.009091, 0.539279, 0.009091, 0.539279, 0.009091, 0.534955, 0.009091, 0.539279, 0.009091, 0.539279, 0.009091, 0.534955, 0.009091, 0.534955, 0.009091, 0.534955, 0.009091, 0.534955, 0.054545, 0.534955, 0.009091, 0.534955, 0.009091, 0.534955, 0.054545, 0.534955, 0.054545, 0.534955, 0.054545, 0.534136, 0.041588, 0.534955, 0.054545, 0.534955, 0.054545, 0.534136, 0.041588, 0.534136, 0.041588, 0.534136, 0.041588, 0.533254, 0.030415, 0.534136, 0.041588, 0.534136, 0.041588, 0.533254, 0.030415, 0.533254, 0.030415, 0.533254, 0.030415, 0.532309, 0.021027, 0.533254, 0.030415, 0.533254, 0.030415, 0.532309, 0.021027, 0.532309, 0.021027, 0.532309, 0.021027, 0.531301, 0.013423, 0.532309, 0.021027, 0.532309, 0.021027, 0.531301, 0.013423, 0.531301, 0.013423, 0.531301, 0.013423, 0.530219, 0.007551, 0.531301, 0.013423, 0.531301, 0.013423, 0.530219, 0.007551, 0.530219, 0.007551, 0.530219, 0.007551, 0.529054, 0.003356, 0.530219, 0.007551, 0.530219, 0.007551, 0.529054, 0.003356, 0.529054, 0.003356, 0.529054, 0.003356, 0.527804, 0.000839, 0.529054, 0.003356, 0.529054, 0.003356, 0.527804, 0.000839, 0.527804, 0.000839, 0.527804, 0.000839, 0.526472, 0, 0.527804, 0.000839, 0.527804, 0.000839, 0.526472, 0, 0.526472, 0, 0.526472, 0, 0.524341, 0.002104, 0.526472, 0, 0.526472, 0, 0.524341, 0.002104, 0.524341, 0.002104, 0.524341, 0.002104, 0.522483, 0.008416, 0.524341, 0.002104, 0.524341, 0.002104, 0.522483, 0.008416, 0.522483, 0.008416, 0.522483, 0.008416, 0.520898, 0.018936, 0.522483, 0.008416, 0.522483, 0.008416, 0.520898, 0.018936, 0.520898, 0.018936, 0.520898, 0.018936, 0.519586, 0.033665, 0.520898, 0.018936, 0.520898, 0.018936, 0.519586, 0.033665, 0.519586, 0.033665, 0.519586, 0.033665, 0.518558, 0.052450, 0.519586, 0.033665, 0.519586, 0.033665, 0.518558, 0.052450, 0.518558, 0.052450, 0.518558, 0.052450, 0.517824, 0.075142, 0.518558, 0.052450, 0.518558, 0.052450, 0.517824, 0.075142, 0.517824, 0.075142, 0.517824, 0.075142, 0.517383, 0.101740, 0.517824, 0.075142, 0.517824, 0.075142, 0.517383, 0.101740, 0.517383, 0.101740, 0.517237, 0.327273, 0.521560, 0.134233, 0.521560, 0.327273, 0.521560, 0.134233, 0.517237, 0.327273, 0.521560, 0.327273, 0.517237, 0.327273, 0.517237, 0.132244, 0.521560, 0.134233, 0.517237, 0.132244, 0.517237, 0.327273, 0.521560, 0.134233, 0.521560, 0.134233, 0.517237, 0.132244, 0.521651, 0.113397, 0.517237, 0.132244, 0.521560, 0.134233, 0.521651, 0.113397, 0.517237, 0.132244, 0.517383, 0.101740, 0.521651, 0.113397, 0.517383, 0.101740, 0.517237, 0.132244, 0.521651, 0.113397, 0.521651, 0.113397, 0.517383, 0.101740, 0.521925, 0.095348, 0.517383, 0.101740, 0.521651, 0.113397, 0.521925, 0.095348, 0.517383, 0.101740, 0.517824, 0.075142, 0.521925, 0.095348, 0.517824, 0.075142, 0.517383, 0.101740, 0.521925, 0.095348, 0.521925, 0.095348, 0.517824, 0.075142, 0.522380, 0.080087, 0.517824, 0.075142, 0.521925, 0.095348, 0.522380, 0.080087, 0.517824, 0.075142, 0.518558, 0.052450, 0.522380, 0.080087, 0.518558, 0.052450, 0.517824, 0.075142, 0.522380, 0.080087, 0.522380, 0.080087, 0.518558, 0.052450, 0.523017, 0.067614, 0.518558, 0.052450, 0.522380, 0.080087, 0.523017, 0.067614, 0.518558, 0.052450, 0.519586, 0.033665, 0.523017, 0.067614, 0.519586, 0.033665, 0.518558, 0.052450, 0.523017, 0.067614, 0.523017, 0.067614, 0.519586, 0.033665, 0.520898, 0.018936, 0.519586, 0.033665, 0.523017, 0.067614, 0.520898, 0.018936, 0.523837, 0.057919, 0.523017, 0.067614, 0.520898, 0.018936, 0.523017, 0.067614, 0.523837, 0.057919, 0.520898, 0.018936, 0.523837, 0.057919, 0.520898, 0.018936, 0.522483, 0.008416, 0.520898, 0.018936, 0.523837, 0.057919, 0.522483, 0.008416, 0.524838, 0.050994, 0.523837, 0.057919, 0.522483, 0.008416, 0.523837, 0.057919, 0.524838, 0.050994, 0.522483, 0.008416, 0.522483, 0.008416, 0.524341, 0.002104, 0.524838, 0.050994, 0.524341, 0.002104, 0.522483, 0.008416, 0.524838, 0.050994, 0.524838, 0.050994, 0.524341, 0.002104, 0.526022, 0.046839, 0.524341, 0.002104, 0.524838, 0.050994, 0.526022, 0.046839, 0.526022, 0.046839, 0.524341, 0.002104, 0.526472, 0, 0.524341, 0.002104, 0.526022, 0.046839, 0.526472, 0, 0.526022, 0.046839, 0.526472, 0, 0.527388, 0.045455, 0.526472, 0, 0.526022, 0.046839, 0.527388, 0.045455, 0.527388, 0.045455, 0.526472, 0, 0.527804, 0.000839, 0.526472, 0, 0.527388, 0.045455, 0.527804, 0.000839, 0.527388, 0.045455, 0.527804, 0.000839, 0.529054, 0.003356, 0.527804, 0.000839, 0.527388, 0.045455, 0.529054, 0.003356, 0.529047, 0.047119, 0.527388, 0.045455, 0.529054, 0.003356, 0.527388, 0.045455, 0.529047, 0.047119, 0.529054, 0.003356, 0.529047, 0.047119, 0.529054, 0.003356, 0.530219, 0.007551, 0.529054, 0.003356, 0.529047, 0.047119, 0.530219, 0.007551, 0.529047, 0.047119, 0.530219, 0.007551, 0.531301, 0.013423, 0.530219, 0.007551, 0.529047, 0.047119, 0.531301, 0.013423, 0.530522, 0.052113, 0.529047, 0.047119, 0.531301, 0.013423, 0.529047, 0.047119, 0.530522, 0.052113, 0.531301, 0.013423, 0.530522, 0.052113, 0.531301, 0.013423, 0.532309, 0.021027, 0.531301, 0.013423, 0.530522, 0.052113, 0.532309, 0.021027, 0.530522, 0.052113, 0.532309, 0.021027, 0.533254, 0.030415, 0.532309, 0.021027, 0.530522, 0.052113, 0.533254, 0.030415, 0.531814, 0.060436, 0.530522, 0.052113, 0.533254, 0.030415, 0.530522, 0.052113, 0.531814, 0.060436, 0.533254, 0.030415, 0.531814, 0.060436, 0.533254, 0.030415, 0.534136, 0.041588, 0.533254, 0.030415, 0.531814, 0.060436, 0.534136, 0.041588, 0.532922, 0.072088, 0.531814, 0.060436, 0.534136, 0.041588, 0.531814, 0.060436, 0.532922, 0.072088, 0.534136, 0.041588, 0.532922, 0.072088, 0.534136, 0.041588, 0.534955, 0.054545, 0.534136, 0.041588, 0.532922, 0.072088, 0.534955, 0.054545, 0.533811, 0.086608, 0.532922, 0.072088, 0.534955, 0.054545, 0.532922, 0.072088, 0.533811, 0.086608, 0.534955, 0.054545, 0.534447, 0.103533, 0.533811, 0.086608, 0.534955, 0.054545, 0.533811, 0.086608, 0.534447, 0.103533, 0.534955, 0.054545, 0.534955, 0.054545, 0.539279, 0.009091, 0.534447, 0.103533, 0.539279, 0.009091, 0.534955, 0.054545, 0.534447, 0.103533, 0.534955, 0.054545, 0.534955, 0.009091, 0.539279, 0.009091, 0.534955, 0.009091, 0.534955, 0.054545, 0.539279, 0.009091, 0.539279, 0.009091, 0.534828, 0.122865, 0.534447, 0.103533, 0.534828, 0.122865, 0.539279, 0.009091, 0.534447, 0.103533, 0.534955, 0.144602, 0.534828, 0.122865, 0.539279, 0.009091, 0.534828, 0.122865, 0.534955, 0.144602, 0.539279, 0.009091, 0.539279, 0.327273, 0.534955, 0.144602, 0.539279, 0.009091, 0.534955, 0.144602, 0.539279, 0.327273, 0.539279, 0.009091, 0.539279, 0.327273, 0.534955, 0.327273, 0.534955, 0.144602, 0.534955, 0.327273, 0.539279, 0.327273, 0.534955, 0.144602, 0.548514, 0.454545, 0.552838, 0.454545, 0.548514, 0.454545, 0.548514, 0.454545, 0.552838, 0.454545, 0.552838, 0.454545, 0.552838, 0.454545, 0.552838, 0.009091, 0.552838, 0.454545, 0.552838, 0.454545, 0.552838, 0.009091, 0.552838, 0.009091, 0.552838, 0.009091, 0.548514, 0.009091, 0.552838, 0.009091, 0.552838, 0.009091, 0.548514, 0.009091, 0.548514, 0.009091, 0.548514, 0.454545, 0.552838, 0.009091, 0.552838, 0.454545, 0.552838, 0.009091, 0.548514, 0.454545, 0.552838, 0.454545, 0.548514, 0.454545, 0.548514, 0.009091, 0.552838, 0.009091, 0.548514, 0.009091, 0.548514, 0.454545, 0.552838, 0.009091, 0.566326, 0.418182, 0.566326, 0.327273, 0.566326, 0.418182, 0.566326, 0.418182, 0.566326, 0.327273, 0.566326, 0.327273, 0.566326, 0.327273, 0.575232, 0.327273, 0.566326, 0.327273, 0.566326, 0.327273, 0.575232, 0.327273, 0.575232, 0.327273, 0.575232, 0.327273, 0.575232, 0.281818, 0.575232, 0.327273, 0.575232, 0.327273, 0.575232, 0.281818, 0.575232, 0.281818, 0.575232, 0.281818, 0.566326, 0.281818, 0.575232, 0.281818, 0.575232, 0.281818, 0.566326, 0.281818, 0.566326, 0.281818, 0.566326, 0.281818, 0.566326, 0.113920, 0.566326, 0.281818, 0.566326, 0.281818, 0.566326, 0.113920, 0.566326, 0.113920, 0.566326, 0.113920, 0.566381, 0.096715, 0.566326, 0.113920, 0.566326, 0.113920, 0.566381, 0.096715, 0.566381, 0.096715, 0.566381, 0.096715, 0.566546, 0.082884, 0.566381, 0.096715, 0.566381, 0.096715, 0.566546, 0.082884, 0.566546, 0.082884, 0.566546, 0.082884, 0.566822, 0.072425, 0.566546, 0.082884, 0.566546, 0.082884, 0.566822, 0.072425, 0.566822, 0.072425, 0.566822, 0.072425, 0.567207, 0.065341, 0.566822, 0.072425, 0.566822, 0.072425, 0.567207, 0.065341, 0.567207, 0.065341, 0.567207, 0.065341, 0.567762, 0.060618, 0.567207, 0.065341, 0.567207, 0.065341, 0.567762, 0.060618, 0.567762, 0.060618, 0.567762, 0.060618, 0.568544, 0.057244, 0.567762, 0.060618, 0.567762, 0.060618, 0.568544, 0.057244, 0.568544, 0.057244, 0.568544, 0.057244, 0.569553, 0.055220, 0.568544, 0.057244, 0.568544, 0.057244, 0.569553, 0.055220, 0.569553, 0.055220, 0.569553, 0.055220, 0.570791, 0.054545, 0.569553, 0.055220, 0.569553, 0.055220, 0.570791, 0.054545, 0.570791, 0.054545, 0.570791, 0.054545, 0.575232, 0.054545, 0.570791, 0.054545, 0.570791, 0.054545, 0.575232, 0.054545, 0.575232, 0.054545, 0.575232, 0.054545, 0.575232, 0.009091, 0.575232, 0.054545, 0.575232, 0.054545, 0.575232, 0.009091, 0.575232, 0.009091, 0.575232, 0.009091, 0.570791, 0.009091, 0.575232, 0.009091, 0.575232, 0.009091, 0.570791, 0.009091, 0.570791, 0.009091, 0.570791, 0.009091, 0.568482, 0.010502, 0.570791, 0.009091, 0.570791, 0.009091, 0.568482, 0.010502, 0.568482, 0.010502, 0.568482, 0.010502, 0.566561, 0.014737, 0.568482, 0.010502, 0.568482, 0.010502, 0.566561, 0.014737, 0.566561, 0.014737, 0.566561, 0.014737, 0.565028, 0.021795, 0.566561, 0.014737, 0.566561, 0.014737, 0.565028, 0.021795, 0.565028, 0.021795, 0.565028, 0.021795, 0.563882, 0.031676, 0.565028, 0.021795, 0.565028, 0.021795, 0.563882, 0.031676, 0.563882, 0.031676, 0.563882, 0.031676, 0.563049, 0.045286, 0.563882, 0.031676, 0.563882, 0.031676, 0.563049, 0.045286, 0.563049, 0.045286, 0.563049, 0.045286, 0.562454, 0.063530, 0.563049, 0.045286, 0.563049, 0.045286, 0.562454, 0.063530, 0.562454, 0.063530, 0.562454, 0.063530, 0.562098, 0.086408, 0.562454, 0.063530, 0.562454, 0.063530, 0.562098, 0.086408, 0.562098, 0.086408, 0.562098, 0.086408, 0.561979, 0.113920, 0.562098, 0.086408, 0.562098, 0.086408, 0.561979, 0.113920, 0.561979, 0.113920, 0.561979, 0.113920, 0.561979, 0.281818, 0.561979, 0.113920, 0.561979, 0.113920, 0.561979, 0.281818, 0.561979, 0.281818, 0.561979, 0.281818, 0.558806, 0.281818, 0.561979, 0.281818, 0.561979, 0.281818, 0.558806, 0.281818, 0.558806, 0.281818, 0.558806, 0.281818, 0.558806, 0.327273, 0.558806, 0.281818, 0.558806, 0.281818, 0.558806, 0.327273, 0.558806, 0.327273, 0.558806, 0.327273, 0.561979, 0.327273, 0.558806, 0.327273, 0.558806, 0.327273, 0.561979, 0.327273, 0.561979, 0.327273, 0.561979, 0.327273, 0.561979, 0.418182, 0.561979, 0.327273, 0.561979, 0.327273, 0.561979, 0.418182, 0.561979, 0.418182, 0.558806, 0.327273, 0.561979, 0.281818, 0.561979, 0.327273, 0.561979, 0.281818, 0.558806, 0.327273, 0.561979, 0.327273, 0.558806, 0.327273, 0.558806, 0.281818, 0.561979, 0.281818, 0.558806, 0.281818, 0.558806, 0.327273, 0.561979, 0.281818, 0.561979, 0.327273, 0.561979, 0.281818, 0.566326, 0.281818, 0.561979, 0.281818, 0.561979, 0.327273, 0.566326, 0.281818, 0.561979, 0.327273, 0.566326, 0.281818, 0.566326, 0.327273, 0.566326, 0.281818, 0.561979, 0.327273, 0.566326, 0.327273, 0.561979, 0.418182, 0.561979, 0.327273, 0.566326, 0.327273, 0.561979, 0.327273, 0.561979, 0.418182, 0.566326, 0.327273, 0.561979, 0.418182, 0.566326, 0.327273, 0.566326, 0.418182, 0.566326, 0.327273, 0.561979, 0.418182, 0.566326, 0.418182, 0.566326, 0.327273, 0.566326, 0.281818, 0.575232, 0.281818, 0.566326, 0.281818, 0.566326, 0.327273, 0.575232, 0.281818, 0.566326, 0.327273, 0.575232, 0.281818, 0.575232, 0.327273, 0.575232, 0.281818, 0.566326, 0.327273, 0.575232, 0.327273, 0.561979, 0.281818, 0.566326, 0.113920, 0.566326, 0.281818, 0.566326, 0.113920, 0.561979, 0.281818, 0.566326, 0.281818, 0.561979, 0.281818, 0.561979, 0.113920, 0.566326, 0.113920, 0.561979, 0.113920, 0.561979, 0.281818, 0.566326, 0.113920, 0.566326, 0.113920, 0.561979, 0.113920, 0.566381, 0.096715, 0.561979, 0.113920, 0.566326, 0.113920, 0.566381, 0.096715, 0.561979, 0.113920, 0.562098, 0.086408, 0.566381, 0.096715, 0.562098, 0.086408, 0.561979, 0.113920, 0.566381, 0.096715, 0.566381, 0.096715, 0.562098, 0.086408, 0.566546, 0.082884, 0.562098, 0.086408, 0.566381, 0.096715, 0.566546, 0.082884, 0.562098, 0.086408, 0.562454, 0.063530, 0.566546, 0.082884, 0.562454, 0.063530, 0.562098, 0.086408, 0.566546, 0.082884, 0.566546, 0.082884, 0.562454, 0.063530, 0.566822, 0.072425, 0.562454, 0.063530, 0.566546, 0.082884, 0.566822, 0.072425, 0.566822, 0.072425, 0.562454, 0.063530, 0.563049, 0.045286, 0.562454, 0.063530, 0.566822, 0.072425, 0.563049, 0.045286, 0.567207, 0.065341, 0.566822, 0.072425, 0.563049, 0.045286, 0.566822, 0.072425, 0.567207, 0.065341, 0.563049, 0.045286, 0.563049, 0.045286, 0.563882, 0.031676, 0.567207, 0.065341, 0.563882, 0.031676, 0.563049, 0.045286, 0.567207, 0.065341, 0.567207, 0.065341, 0.563882, 0.031676, 0.565028, 0.021795, 0.563882, 0.031676, 0.567207, 0.065341, 0.565028, 0.021795, 0.567762, 0.060618, 0.567207, 0.065341, 0.565028, 0.021795, 0.567207, 0.065341, 0.567762, 0.060618, 0.565028, 0.021795, 0.565028, 0.021795, 0.566561, 0.014737, 0.567762, 0.060618, 0.566561, 0.014737, 0.565028, 0.021795, 0.567762, 0.060618, 0.567762, 0.060618, 0.566561, 0.014737, 0.568544, 0.057244, 0.566561, 0.014737, 0.567762, 0.060618, 0.568544, 0.057244, 0.566561, 0.014737, 0.568482, 0.010502, 0.568544, 0.057244, 0.568482, 0.010502, 0.566561, 0.014737, 0.568544, 0.057244, 0.568544, 0.057244, 0.568482, 0.010502, 0.569553, 0.055220, 0.568482, 0.010502, 0.568544, 0.057244, 0.569553, 0.055220, 0.569553, 0.055220, 0.568482, 0.010502, 0.570791, 0.009091, 0.568482, 0.010502, 0.569553, 0.055220, 0.570791, 0.009091, 0.570791, 0.054545, 0.569553, 0.055220, 0.570791, 0.009091, 0.569553, 0.055220, 0.570791, 0.054545, 0.570791, 0.009091, 0.570791, 0.054545, 0.570791, 0.009091, 0.575232, 0.009091, 0.570791, 0.009091, 0.570791, 0.054545, 0.575232, 0.009091, 0.570791, 0.054545, 0.575232, 0.009091, 0.575232, 0.054545, 0.575232, 0.009091, 0.570791, 0.054545, 0.575232, 0.054545, 0.595888, 0.327273, 0.600212, 0.327273, 0.595888, 0.327273, 0.595888, 0.327273, 0.600212, 0.327273, 0.600212, 0.327273, 0.600212, 0.327273, 0.600212, 0.009091, 0.600212, 0.327273, 0.600212, 0.327273, 0.600212, 0.009091, 0.600212, 0.009091, 0.600212, 0.009091, 0.595888, 0.009091, 0.600212, 0.009091, 0.600212, 0.009091, 0.595888, 0.009091, 0.595888, 0.009091, 0.595888, 0.327273, 0.600212, 0.009091, 0.600212, 0.327273, 0.600212, 0.009091, 0.595888, 0.327273, 0.600212, 0.327273, 0.595888, 0.327273, 0.595888, 0.009091, 0.600212, 0.009091, 0.595888, 0.009091, 0.595888, 0.327273, 0.600212, 0.009091, 0.595888, 0.454545, 0.600212, 0.454545, 0.595888, 0.454545, 0.595888, 0.454545, 0.600212, 0.454545, 0.600212, 0.454545, 0.600212, 0.454545, 0.600212, 0.390909, 0.600212, 0.454545, 0.600212, 0.454545, 0.600212, 0.390909, 0.600212, 0.390909, 0.600212, 0.390909, 0.595888, 0.390909, 0.600212, 0.390909, 0.600212, 0.390909, 0.595888, 0.390909, 0.595888, 0.390909, 0.595888, 0.454545, 0.600212, 0.390909, 0.600212, 0.454545, 0.600212, 0.390909, 0.595888, 0.454545, 0.600212, 0.454545, 0.595888, 0.454545, 0.595888, 0.390909, 0.600212, 0.390909, 0.595888, 0.390909, 0.595888, 0.454545, 0.600212, 0.390909, 0.631301, 0.203977, 0.631301, 0.009091, 0.631301, 0.203977, 0.631301, 0.203977, 0.631301, 0.009091, 0.631301, 0.009091, 0.631301, 0.009091, 0.626977, 0.009091, 0.631301, 0.009091, 0.631301, 0.009091, 0.626977, 0.009091, 0.626977, 0.009091, 0.626977, 0.009091, 0.626977, 0.202273, 0.626977, 0.009091, 0.626977, 0.009091, 0.626977, 0.202273, 0.626977, 0.202273, 0.626977, 0.202273, 0.626886, 0.223105, 0.626977, 0.202273, 0.626977, 0.202273, 0.626886, 0.223105, 0.626886, 0.223105, 0.626886, 0.223105, 0.626613, 0.241140, 0.626886, 0.223105, 0.626886, 0.223105, 0.626613, 0.241140, 0.626613, 0.241140, 0.626613, 0.241140, 0.626157, 0.256379, 0.626613, 0.241140, 0.626613, 0.241140, 0.626157, 0.256379, 0.626157, 0.256379, 0.626157, 0.256379, 0.625520, 0.268821, 0.626157, 0.256379, 0.626157, 0.256379, 0.625520, 0.268821, 0.625520, 0.268821, 0.625520, 0.268821, 0.624700, 0.278485, 0.625520, 0.268821, 0.625520, 0.268821, 0.624700, 0.278485, 0.624700, 0.278485, 0.624700, 0.278485, 0.623699, 0.285387, 0.624700, 0.278485, 0.624700, 0.278485, 0.623699, 0.285387, 0.623699, 0.285387, 0.623699, 0.285387, 0.622515, 0.289529, 0.623699, 0.285387, 0.623699, 0.285387, 0.622515, 0.289529, 0.622515, 0.289529, 0.622515, 0.289529, 0.621149, 0.290909, 0.622515, 0.289529, 0.622515, 0.289529, 0.621149, 0.290909, 0.621149, 0.290909, 0.621149, 0.290909, 0.619491, 0.289244, 0.621149, 0.290909, 0.621149, 0.290909, 0.619491, 0.289244, 0.619491, 0.289244, 0.619491, 0.289244, 0.618018, 0.284251, 0.619491, 0.289244, 0.619491, 0.289244, 0.618018, 0.284251, 0.618018, 0.284251, 0.618018, 0.284251, 0.616730, 0.275928, 0.618018, 0.284251, 0.618018, 0.284251, 0.616730, 0.275928, 0.616730, 0.275928, 0.616730, 0.275928, 0.615627, 0.264276, 0.616730, 0.275928, 0.616730, 0.275928, 0.615627, 0.264276, 0.615627, 0.264276, 0.615627, 0.264276, 0.614743, 0.249747, 0.615627, 0.264276, 0.615627, 0.264276, 0.614743, 0.249747, 0.614743, 0.249747, 0.614743, 0.249747, 0.614111, 0.232795, 0.614743, 0.249747, 0.614743, 0.249747, 0.614111, 0.232795, 0.614111, 0.232795, 0.614111, 0.232795, 0.613732, 0.213419, 0.614111, 0.232795, 0.614111, 0.232795, 0.613732, 0.213419, 0.613732, 0.213419, 0.613732, 0.213419, 0.613606, 0.191619, 0.613732, 0.213419, 0.613732, 0.213419, 0.613606, 0.191619, 0.613606, 0.191619, 0.613606, 0.191619, 0.613606, 0.009091, 0.613606, 0.191619, 0.613606, 0.191619, 0.613606, 0.009091, 0.613606, 0.009091, 0.613606, 0.009091, 0.609259, 0.009091, 0.613606, 0.009091, 0.613606, 0.009091, 0.609259, 0.009091, 0.609259, 0.009091, 0.609259, 0.009091, 0.609259, 0.327273, 0.609259, 0.009091, 0.609259, 0.009091, 0.609259, 0.327273, 0.609259, 0.327273, 0.609259, 0.327273, 0.613606, 0.327273, 0.609259, 0.327273, 0.609259, 0.327273, 0.613606, 0.327273, 0.613606, 0.327273, 0.613606, 0.327273, 0.613606, 0.281818, 0.613606, 0.327273, 0.613606, 0.327273, 0.613606, 0.281818, 0.613606, 0.281818, 0.613606, 0.281818, 0.614416, 0.294660, 0.613606, 0.281818, 0.613606, 0.281818, 0.614416, 0.294660, 0.614416, 0.294660, 0.614416, 0.294660, 0.615295, 0.305771, 0.614416, 0.294660, 0.614416, 0.294660, 0.615295, 0.305771, 0.615295, 0.305771, 0.615295, 0.305771, 0.616243, 0.315150, 0.615295, 0.305771, 0.615295, 0.305771, 0.616243, 0.315150, 0.616243, 0.315150, 0.616243, 0.315150, 0.617260, 0.322798, 0.616243, 0.315150, 0.616243, 0.315150, 0.617260, 0.322798, 0.617260, 0.322798, 0.617260, 0.322798, 0.618352, 0.328733, 0.617260, 0.322798, 0.617260, 0.322798, 0.618352, 0.328733, 0.618352, 0.328733, 0.618352, 0.328733, 0.619525, 0.332972, 0.618352, 0.328733, 0.618352, 0.328733, 0.619525, 0.332972, 0.619525, 0.332972, 0.619525, 0.332972, 0.620778, 0.335516, 0.619525, 0.332972, 0.619525, 0.332972, 0.620778, 0.335516, 0.620778, 0.335516, 0.620778, 0.335516, 0.622113, 0.336364, 0.620778, 0.335516, 0.620778, 0.335516, 0.622113, 0.336364, 0.622113, 0.336364, 0.622113, 0.336364, 0.624242, 0.334264, 0.622113, 0.336364, 0.622113, 0.336364, 0.624242, 0.334264, 0.624242, 0.334264, 0.624242, 0.334264, 0.626096, 0.327965, 0.624242, 0.334264, 0.624242, 0.334264, 0.626096, 0.327965, 0.626096, 0.327965, 0.626096, 0.327965, 0.627673, 0.317467, 0.626096, 0.327965, 0.626096, 0.327965, 0.627673, 0.317467, 0.627673, 0.317467, 0.627673, 0.317467, 0.628974, 0.302770, 0.627673, 0.317467, 0.627673, 0.317467, 0.628974, 0.302770, 0.628974, 0.302770, 0.628974, 0.302770, 0.629992, 0.283998, 0.628974, 0.302770, 0.628974, 0.302770, 0.629992, 0.283998, 0.629992, 0.283998, 0.629992, 0.283998, 0.630719, 0.261275, 0.629992, 0.283998, 0.629992, 0.283998, 0.630719, 0.261275, 0.630719, 0.261275, 0.630719, 0.261275, 0.631155, 0.234601, 0.630719, 0.261275, 0.630719, 0.261275, 0.631155, 0.234601, 0.631155, 0.234601, 0.613606, 0.327273, 0.609259, 0.327273, 0.613606, 0.281818, 0.609259, 0.327273, 0.613606, 0.327273, 0.613606, 0.281818, 0.609259, 0.327273, 0.614111, 0.232795, 0.613606, 0.281818, 0.614111, 0.232795, 0.609259, 0.327273, 0.613606, 0.281818, 0.614111, 0.232795, 0.609259, 0.327273, 0.613732, 0.213419, 0.609259, 0.327273, 0.614111, 0.232795, 0.613732, 0.213419, 0.613732, 0.213419, 0.609259, 0.327273, 0.613606, 0.191619, 0.609259, 0.327273, 0.613732, 0.213419, 0.613606, 0.191619, 0.609259, 0.327273, 0.609259, 0.009091, 0.613606, 0.191619, 0.609259, 0.009091, 0.609259, 0.327273, 0.613606, 0.191619, 0.609259, 0.009091, 0.613606, 0.009091, 0.613606, 0.191619, 0.613606, 0.009091, 0.609259, 0.009091, 0.613606, 0.191619, 0.613606, 0.281818, 0.614111, 0.232795, 0.614743, 0.249747, 0.614111, 0.232795, 0.613606, 0.281818, 0.614743, 0.249747, 0.613606, 0.281818, 0.614743, 0.249747, 0.615627, 0.264276, 0.614743, 0.249747, 0.613606, 0.281818, 0.615627, 0.264276, 0.614416, 0.294660, 0.613606, 0.281818, 0.615627, 0.264276, 0.613606, 0.281818, 0.614416, 0.294660, 0.615627, 0.264276, 0.614416, 0.294660, 0.615627, 0.264276, 0.616730, 0.275928, 0.615627, 0.264276, 0.614416, 0.294660, 0.616730, 0.275928, 0.615295, 0.305771, 0.614416, 0.294660, 0.616730, 0.275928, 0.614416, 0.294660, 0.615295, 0.305771, 0.616730, 0.275928, 0.618018, 0.284251, 0.615295, 0.305771, 0.616730, 0.275928, 0.615295, 0.305771, 0.618018, 0.284251, 0.616730, 0.275928, 0.618018, 0.284251, 0.616243, 0.315150, 0.615295, 0.305771, 0.616243, 0.315150, 0.618018, 0.284251, 0.615295, 0.305771, 0.617260, 0.322798, 0.616243, 0.315150, 0.618018, 0.284251, 0.616243, 0.315150, 0.617260, 0.322798, 0.618018, 0.284251, 0.617260, 0.322798, 0.618018, 0.284251, 0.619491, 0.289244, 0.618018, 0.284251, 0.617260, 0.322798, 0.619491, 0.289244, 0.618352, 0.328733, 0.617260, 0.322798, 0.619491, 0.289244, 0.617260, 0.322798, 0.618352, 0.328733, 0.619491, 0.289244, 0.618352, 0.328733, 0.619491, 0.289244, 0.619525, 0.332972, 0.619491, 0.289244, 0.618352, 0.328733, 0.619525, 0.332972, 0.619525, 0.332972, 0.619491, 0.289244, 0.621149, 0.290909, 0.619491, 0.289244, 0.619525, 0.332972, 0.621149, 0.290909, 0.620778, 0.335516, 0.619525, 0.332972, 0.621149, 0.290909, 0.619525, 0.332972, 0.620778, 0.335516, 0.621149, 0.290909, 0.622113, 0.336364, 0.620778, 0.335516, 0.621149, 0.290909, 0.620778, 0.335516, 0.622113, 0.336364, 0.621149, 0.290909, 0.622113, 0.336364, 0.621149, 0.290909, 0.622515, 0.289529, 0.621149, 0.290909, 0.622113, 0.336364, 0.622515, 0.289529, 0.624242, 0.334264, 0.622113, 0.336364, 0.622515, 0.289529, 0.622113, 0.336364, 0.624242, 0.334264, 0.622515, 0.289529, 0.622515, 0.289529, 0.623699, 0.285387, 0.624242, 0.334264, 0.623699, 0.285387, 0.622515, 0.289529, 0.624242, 0.334264, 0.624242, 0.334264, 0.623699, 0.285387, 0.626096, 0.327965, 0.623699, 0.285387, 0.624242, 0.334264, 0.626096, 0.327965, 0.623699, 0.285387, 0.624700, 0.278485, 0.626096, 0.327965, 0.624700, 0.278485, 0.623699, 0.285387, 0.626096, 0.327965, 0.626096, 0.327965, 0.624700, 0.278485, 0.627673, 0.317467, 0.624700, 0.278485, 0.626096, 0.327965, 0.627673, 0.317467, 0.624700, 0.278485, 0.625520, 0.268821, 0.627673, 0.317467, 0.625520, 0.268821, 0.624700, 0.278485, 0.627673, 0.317467, 0.627673, 0.317467, 0.625520, 0.268821, 0.628974, 0.302770, 0.625520, 0.268821, 0.627673, 0.317467, 0.628974, 0.302770, 0.628974, 0.302770, 0.625520, 0.268821, 0.629992, 0.283998, 0.625520, 0.268821, 0.628974, 0.302770, 0.629992, 0.283998, 0.629992, 0.283998, 0.625520, 0.268821, 0.626157, 0.256379, 0.625520, 0.268821, 0.629992, 0.283998, 0.626157, 0.256379, 0.629992, 0.283998, 0.626157, 0.256379, 0.630719, 0.261275, 0.626157, 0.256379, 0.629992, 0.283998, 0.630719, 0.261275, 0.626157, 0.256379, 0.626613, 0.241140, 0.630719, 0.261275, 0.626613, 0.241140, 0.626157, 0.256379, 0.630719, 0.261275, 0.630719, 0.261275, 0.626613, 0.241140, 0.631155, 0.234601, 0.626613, 0.241140, 0.630719, 0.261275, 0.631155, 0.234601, 0.626613, 0.241140, 0.626886, 0.223105, 0.631155, 0.234601, 0.626886, 0.223105, 0.626613, 0.241140, 0.631155, 0.234601, 0.631155, 0.234601, 0.626886, 0.223105, 0.631301, 0.203977, 0.626886, 0.223105, 0.631155, 0.234601, 0.631301, 0.203977, 0.631301, 0.203977, 0.626886, 0.223105, 0.626977, 0.202273, 0.626886, 0.223105, 0.631301, 0.203977, 0.626977, 0.202273, 0.626977, 0.202273, 0.631301, 0.009091, 0.631301, 0.203977, 0.631301, 0.009091, 0.626977, 0.202273, 0.631301, 0.203977, 0.626977, 0.202273, 0.626977, 0.009091, 0.631301, 0.009091, 0.626977, 0.009091, 0.626977, 0.202273, 0.631301, 0.009091, 0.655481, 0.436364, 0.662554, 0.436364, 0.655481, 0.436364, 0.655481, 0.436364, 0.662554, 0.436364, 0.662554, 0.436364, 0.662554, 0.436364, 0.671507, 0.145312, 0.662554, 0.436364, 0.662554, 0.436364, 0.671507, 0.145312, 0.671507, 0.145312, 0.671507, 0.145312, 0.680508, 0.436364, 0.671507, 0.145312, 0.671507, 0.145312, 0.680508, 0.436364, 0.680508, 0.436364, 0.680508, 0.436364, 0.687581, 0.436364, 0.680508, 0.436364, 0.680508, 0.436364, 0.687581, 0.436364, 0.687581, 0.436364, 0.687581, 0.436364, 0.687581, 0.009091, 0.687581, 0.436364, 0.687581, 0.436364, 0.687581, 0.009091, 0.687581, 0.009091, 0.687581, 0.009091, 0.682952, 0.009091, 0.687581, 0.009091, 0.687581, 0.009091, 0.682952, 0.009091, 0.682952, 0.009091, 0.682952, 0.009091, 0.682952, 0.384233, 0.682952, 0.009091, 0.682952, 0.009091, 0.682952, 0.384233, 0.682952, 0.384233, 0.682952, 0.384233, 0.673904, 0.090909, 0.682952, 0.384233, 0.682952, 0.384233, 0.673904, 0.090909, 0.673904, 0.090909, 0.673904, 0.090909, 0.669134, 0.090909, 0.673904, 0.090909, 0.673904, 0.090909, 0.669134, 0.090909, 0.669134, 0.090909, 0.669134, 0.090909, 0.660087, 0.384233, 0.669134, 0.090909, 0.669134, 0.090909, 0.660087, 0.384233, 0.660087, 0.384233, 0.660087, 0.384233, 0.660087, 0.009091, 0.660087, 0.384233, 0.660087, 0.384233, 0.660087, 0.009091, 0.660087, 0.009091, 0.660087, 0.009091, 0.655481, 0.009091, 0.660087, 0.009091, 0.660087, 0.009091, 0.655481, 0.009091, 0.655481, 0.009091, 0.655481, 0.436364, 0.660087, 0.384233, 0.662554, 0.436364, 0.660087, 0.384233, 0.655481, 0.436364, 0.662554, 0.436364, 0.655481, 0.436364, 0.655481, 0.009091, 0.660087, 0.384233, 0.655481, 0.009091, 0.655481, 0.436364, 0.660087, 0.384233, 0.660087, 0.009091, 0.660087, 0.384233, 0.655481, 0.009091, 0.660087, 0.384233, 0.660087, 0.009091, 0.655481, 0.009091, 0.662554, 0.436364, 0.660087, 0.384233, 0.671507, 0.145312, 0.660087, 0.384233, 0.662554, 0.436364, 0.671507, 0.145312, 0.660087, 0.384233, 0.669134, 0.090909, 0.671507, 0.145312, 0.669134, 0.090909, 0.660087, 0.384233, 0.671507, 0.145312, 0.671507, 0.145312, 0.669134, 0.090909, 0.673904, 0.090909, 0.669134, 0.090909, 0.671507, 0.145312, 0.673904, 0.090909, 0.671507, 0.145312, 0.673904, 0.090909, 0.682952, 0.384233, 0.673904, 0.090909, 0.671507, 0.145312, 0.682952, 0.384233, 0.682952, 0.384233, 0.680508, 0.436364, 0.671507, 0.145312, 0.680508, 0.436364, 0.682952, 0.384233, 0.671507, 0.145312, 0.680508, 0.436364, 0.682952, 0.384233, 0.687581, 0.436364, 0.682952, 0.384233, 0.680508, 0.436364, 0.687581, 0.436364, 0.682952, 0.384233, 0.687581, 0.009091, 0.687581, 0.436364, 0.687581, 0.009091, 0.682952, 0.384233, 0.687581, 0.436364, 0.687581, 0.009091, 0.682952, 0.384233, 0.682952, 0.009091, 0.682952, 0.384233, 0.687581, 0.009091, 0.682952, 0.009091, 0.717871, 0.427273, 0.717871, 0.372727, 0.717871, 0.427273, 0.717871, 0.427273, 0.717871, 0.372727, 0.717871, 0.372727, 0.717871, 0.372727, 0.716539, 0.379177, 0.717871, 0.372727, 0.717871, 0.372727, 0.716539, 0.379177, 0.716539, 0.379177, 0.716539, 0.379177, 0.715245, 0.384748, 0.716539, 0.379177, 0.716539, 0.379177, 0.715245, 0.384748, 0.715245, 0.384748, 0.715245, 0.384748, 0.713989, 0.389440, 0.715245, 0.384748, 0.715245, 0.384748, 0.713989, 0.389440, 0.713989, 0.389440, 0.713989, 0.389440, 0.712772, 0.393253, 0.713989, 0.389440, 0.713989, 0.389440, 0.712772, 0.393253, 0.712772, 0.393253, 0.712772, 0.393253, 0.711583, 0.396205, 0.712772, 0.393253, 0.712772, 0.393253, 0.711583, 0.396205, 0.711583, 0.396205, 0.711583, 0.396205, 0.710416, 0.398313, 0.711583, 0.396205, 0.711583, 0.396205, 0.710416, 0.398313, 0.710416, 0.398313, 0.710416, 0.398313, 0.709269, 0.399578, 0.710416, 0.398313, 0.710416, 0.398313, 0.709269, 0.399578, 0.709269, 0.399578, 0.709269, 0.399578, 0.708142, 0.400000, 0.709269, 0.399578, 0.709269, 0.399578, 0.708142, 0.400000, 0.708142, 0.400000, 0.708142, 0.400000, 0.706315, 0.398877, 0.708142, 0.400000, 0.708142, 0.400000, 0.706315, 0.398877, 0.706315, 0.398877, 0.706315, 0.398877, 0.704709, 0.395508, 0.706315, 0.398877, 0.706315, 0.398877, 0.704709, 0.395508, 0.704709, 0.395508, 0.704709, 0.395508, 0.703324, 0.389893, 0.704709, 0.395508, 0.704709, 0.395508, 0.703324, 0.389893, 0.703324, 0.389893, 0.703324, 0.389893, 0.702162, 0.382031, 0.703324, 0.389893, 0.703324, 0.389893, 0.702162, 0.382031, 0.702162, 0.382031, 0.702162, 0.382031, 0.701242, 0.372101, 0.702162, 0.382031, 0.702162, 0.382031, 0.701242, 0.372101, 0.701242, 0.372101, 0.701242, 0.372101, 0.700585, 0.360281, 0.701242, 0.372101, 0.701242, 0.372101, 0.700585, 0.360281, 0.700585, 0.360281, 0.700585, 0.360281, 0.700190, 0.346569, 0.700585, 0.360281, 0.700585, 0.360281, 0.700190, 0.346569, 0.700190, 0.346569, 0.700190, 0.346569, 0.700059, 0.330966, 0.700190, 0.346569, 0.700190, 0.346569, 0.700059, 0.330966, 0.700059, 0.330966, 0.700059, 0.330966, 0.700146, 0.317960, 0.700059, 0.330966, 0.700059, 0.330966, 0.700146, 0.317960, 0.700146, 0.317960, 0.700146, 0.317960, 0.700408, 0.306641, 0.700146, 0.317960, 0.700146, 0.317960, 0.700408, 0.306641, 0.700408, 0.306641, 0.700408, 0.306641, 0.700845, 0.297008, 0.700408, 0.306641, 0.700408, 0.306641, 0.700845, 0.297008, 0.700845, 0.297008, 0.700845, 0.297008, 0.701457, 0.289063, 0.700845, 0.297008, 0.700845, 0.297008, 0.701457, 0.289063, 0.701457, 0.289063, 0.701457, 0.289063, 0.702313, 0.282306, 0.701457, 0.289063, 0.701457, 0.289063, 0.702313, 0.282306, 0.702313, 0.282306, 0.702313, 0.282306, 0.703481, 0.276243, 0.702313, 0.282306, 0.702313, 0.282306, 0.703481, 0.276243, 0.703481, 0.276243, 0.703481, 0.276243, 0.704962, 0.270872, 0.703481, 0.276243, 0.703481, 0.276243, 0.704962, 0.270872, 0.704962, 0.270872, 0.704962, 0.270872, 0.706756, 0.266193, 0.704962, 0.270872, 0.704962, 0.270872, 0.706756, 0.266193, 0.706756, 0.266193, 0.706756, 0.266193, 0.709623, 0.258949, 0.706756, 0.266193, 0.706756, 0.266193, 0.709623, 0.258949, 0.709623, 0.258949, 0.709623, 0.258949, 0.712104, 0.251429, 0.709623, 0.258949, 0.709623, 0.258949, 0.712104, 0.251429, 0.712104, 0.251429, 0.712104, 0.251429, 0.714238, 0.241513, 0.712104, 0.251429, 0.712104, 0.251429, 0.714238, 0.241513, 0.714238, 0.241513, 0.714238, 0.241513, 0.716023, 0.229199, 0.714238, 0.241513, 0.714238, 0.241513, 0.716023, 0.229199, 0.716023, 0.229199, 0.716023, 0.229199, 0.717460, 0.214489, 0.716023, 0.229199, 0.716023, 0.229199, 0.717460, 0.214489, 0.717460, 0.214489, 0.717460, 0.214489, 0.718565, 0.197230, 0.717460, 0.214489, 0.717460, 0.214489, 0.718565, 0.197230, 0.718565, 0.197230, 0.718565, 0.197230, 0.719354, 0.177273, 0.718565, 0.197230, 0.718565, 0.197230, 0.719354, 0.177273, 0.719354, 0.177273, 0.719354, 0.177273, 0.719828, 0.154616, 0.719354, 0.177273, 0.719354, 0.177273, 0.719828, 0.154616, 0.719828, 0.154616, 0.719828, 0.154616, 0.719986, 0.129261, 0.719828, 0.154616, 0.719828, 0.154616, 0.719986, 0.129261, 0.719986, 0.129261, 0.719986, 0.129261, 0.719772, 0.099370, 0.719986, 0.129261, 0.719986, 0.129261, 0.719772, 0.099370, 0.719772, 0.099370, 0.719772, 0.099370, 0.719131, 0.073331, 0.719772, 0.099370, 0.719772, 0.099370, 0.719131, 0.073331, 0.719131, 0.073331, 0.719131, 0.073331, 0.718063, 0.051145, 0.719131, 0.073331, 0.719131, 0.073331, 0.718063, 0.051145, 0.718063, 0.051145, 0.718063, 0.051145, 0.716567, 0.032812, 0.718063, 0.051145, 0.718063, 0.051145, 0.716567, 0.032812, 0.716567, 0.032812, 0.716567, 0.032812, 0.714658, 0.018457, 0.716567, 0.032812, 0.716567, 0.032812, 0.714658, 0.018457, 0.714658, 0.018457, 0.714658, 0.018457, 0.712352, 0.008203, 0.714658, 0.018457, 0.714658, 0.018457, 0.712352, 0.008203, 0.712352, 0.008203, 0.712352, 0.008203, 0.709647, 0.002051, 0.712352, 0.008203, 0.712352, 0.008203, 0.709647, 0.002051, 0.709647, 0.002051, 0.709647, 0.002051, 0.706545, 0, 0.709647, 0.002051, 0.709647, 0.002051, 0.706545, 0, 0.706545, 0, 0.706545, 0, 0.705279, 0.000431, 0.706545, 0, 0.706545, 0, 0.705279, 0.000431, 0.705279, 0.000431, 0.705279, 0.000431, 0.703974, 0.001722, 0.705279, 0.000431, 0.705279, 0.000431, 0.703974, 0.001722, 0.703974, 0.001722, 0.703974, 0.001722, 0.702630, 0.003875, 0.703974, 0.001722, 0.703974, 0.001722, 0.702630, 0.003875, 0.702630, 0.003875, 0.702630, 0.003875, 0.701245, 0.006889, 0.702630, 0.003875, 0.702630, 0.003875, 0.701245, 0.006889, 0.701245, 0.006889, 0.701245, 0.006889, 0.699829, 0.010747, 0.701245, 0.006889, 0.701245, 0.006889, 0.699829, 0.010747, 0.699829, 0.010747, 0.699829, 0.010747, 0.698387, 0.015430, 0.699829, 0.010747, 0.699829, 0.010747, 0.698387, 0.015430, 0.698387, 0.015430, 0.698387, 0.015430, 0.696921, 0.020938, 0.698387, 0.015430, 0.698387, 0.015430, 0.696921, 0.020938, 0.696921, 0.020938, 0.696921, 0.020938, 0.695429, 0.027273, 0.696921, 0.020938, 0.696921, 0.020938, 0.695429, 0.027273, 0.695429, 0.027273, 0.695429, 0.027273, 0.695429, 0.090909, 0.695429, 0.027273, 0.695429, 0.027273, 0.695429, 0.090909, 0.695429, 0.090909, 0.695429, 0.090909, 0.696867, 0.080313, 0.695429, 0.090909, 0.695429, 0.090909, 0.696867, 0.080313, 0.696867, 0.080313, 0.696867, 0.080313, 0.698290, 0.071112, 0.696867, 0.080313, 0.696867, 0.080313, 0.698290, 0.071112, 0.698290, 0.071112, 0.698290, 0.071112, 0.699699, 0.063303, 0.698290, 0.071112, 0.698290, 0.071112, 0.699699, 0.063303, 0.699699, 0.063303, 0.699699, 0.063303, 0.701093, 0.056889, 0.699699, 0.063303, 0.699699, 0.063303, 0.701093, 0.056889, 0.701093, 0.056889, 0.701093, 0.056889, 0.702473, 0.051887, 0.701093, 0.056889, 0.701093, 0.056889, 0.702473, 0.051887, 0.702473, 0.051887, 0.702473, 0.051887, 0.703842, 0.048313, 0.702473, 0.051887, 0.702473, 0.051887, 0.703842, 0.048313, 0.703842, 0.048313, 0.703842, 0.048313, 0.705199, 0.046169, 0.703842, 0.048313, 0.703842, 0.048313, 0.705199, 0.046169, 0.705199, 0.046169, 0.705199, 0.046169, 0.706545, 0.045455, 0.705199, 0.046169, 0.705199, 0.046169, 0.706545, 0.045455, 0.706545, 0.045455, 0.706545, 0.045455, 0.708461, 0.046729, 0.706545, 0.045455, 0.706545, 0.045455, 0.708461, 0.046729, 0.708461, 0.046729, 0.708461, 0.046729, 0.710146, 0.050550, 0.708461, 0.046729, 0.708461, 0.046729, 0.710146, 0.050550, 0.710146, 0.050550, 0.710146, 0.050550, 0.711598, 0.056920, 0.710146, 0.050550, 0.710146, 0.050550, 0.711598, 0.056920, 0.711598, 0.056920, 0.711598, 0.056920, 0.712819, 0.065838, 0.711598, 0.056920, 0.711598, 0.056920, 0.712819, 0.065838, 0.712819, 0.065838, 0.712819, 0.065838, 0.713785, 0.077108, 0.712819, 0.065838, 0.712819, 0.065838, 0.713785, 0.077108, 0.713785, 0.077108, 0.713785, 0.077108, 0.714475, 0.090536, 0.713785, 0.077108, 0.713785, 0.077108, 0.714475, 0.090536, 0.714475, 0.090536, 0.714475, 0.090536, 0.714890, 0.106121, 0.714475, 0.090536, 0.714475, 0.090536, 0.714890, 0.106121, 0.714890, 0.106121, 0.714890, 0.106121, 0.715028, 0.123864, 0.714890, 0.106121, 0.714890, 0.106121, 0.715028, 0.123864, 0.715028, 0.123864, 0.715028, 0.123864, 0.714929, 0.139444, 0.715028, 0.123864, 0.715028, 0.123864, 0.714929, 0.139444, 0.714929, 0.139444, 0.714929, 0.139444, 0.714631, 0.153232, 0.714929, 0.139444, 0.714929, 0.139444, 0.714631, 0.153232, 0.714631, 0.153232, 0.714631, 0.153232, 0.714135, 0.165226, 0.714631, 0.153232, 0.714631, 0.153232, 0.714135, 0.165226, 0.714135, 0.165226, 0.714135, 0.165226, 0.713441, 0.175426, 0.714135, 0.165226, 0.714135, 0.165226, 0.713441, 0.175426, 0.713441, 0.175426, 0.713441, 0.175426, 0.712521, 0.184144, 0.713441, 0.175426, 0.713441, 0.175426, 0.712521, 0.184144, 0.712521, 0.184144, 0.712521, 0.184144, 0.711347, 0.191690, 0.712521, 0.184144, 0.712521, 0.184144, 0.711347, 0.191690, 0.711347, 0.191690, 0.711347, 0.191690, 0.709919, 0.198065, 0.711347, 0.191690, 0.711347, 0.191690, 0.709919, 0.198065, 0.709919, 0.198065, 0.709919, 0.198065, 0.708236, 0.203267, 0.709919, 0.198065, 0.709919, 0.198065, 0.708236, 0.203267, 0.708236, 0.203267, 0.708236, 0.203267, 0.705346, 0.210227, 0.708236, 0.203267, 0.708236, 0.203267, 0.705346, 0.210227, 0.705346, 0.210227, 0.705346, 0.210227, 0.702874, 0.217454, 0.705346, 0.210227, 0.705346, 0.210227, 0.702874, 0.217454, 0.702874, 0.217454, 0.702874, 0.217454, 0.700770, 0.226491, 0.702874, 0.217454, 0.702874, 0.217454, 0.700770, 0.226491, 0.700770, 0.226491, 0.700770, 0.226491, 0.699032, 0.237340, 0.700770, 0.226491, 0.700770, 0.226491, 0.699032, 0.237340, 0.699032, 0.237340, 0.699032, 0.237340, 0.697662, 0.250000, 0.699032, 0.237340, 0.699032, 0.237340, 0.697662, 0.250000, 0.697662, 0.250000, 0.697662, 0.250000, 0.696624, 0.264879, 0.697662, 0.250000, 0.697662, 0.250000, 0.696624, 0.264879, 0.696624, 0.264879, 0.696624, 0.264879, 0.695882, 0.282386, 0.696624, 0.264879, 0.696624, 0.264879, 0.695882, 0.282386, 0.695882, 0.282386, 0.695882, 0.282386, 0.695437, 0.302521, 0.695882, 0.282386, 0.695882, 0.282386, 0.695437, 0.302521, 0.695437, 0.302521, 0.695437, 0.302521, 0.695288, 0.325284, 0.695437, 0.302521, 0.695437, 0.302521, 0.695288, 0.325284, 0.695288, 0.325284, 0.695288, 0.325284, 0.695493, 0.351718, 0.695288, 0.325284, 0.695288, 0.325284, 0.695493, 0.351718, 0.695493, 0.351718, 0.695493, 0.351718, 0.696108, 0.375195, 0.695493, 0.351718, 0.695493, 0.351718, 0.696108, 0.375195, 0.696108, 0.375195, 0.696108, 0.375195, 0.697132, 0.395716, 0.696108, 0.375195, 0.696108, 0.375195, 0.697132, 0.395716, 0.697132, 0.395716, 0.697132, 0.395716, 0.698567, 0.413281, 0.697132, 0.395716, 0.697132, 0.395716, 0.698567, 0.413281, 0.698567, 0.413281, 0.698567, 0.413281, 0.700361, 0.427357, 0.698567, 0.413281, 0.698567, 0.413281, 0.700361, 0.427357, 0.700361, 0.427357, 0.700361, 0.427357, 0.702465, 0.437411, 0.700361, 0.427357, 0.700361, 0.427357, 0.702465, 0.437411, 0.702465, 0.437411, 0.702465, 0.437411, 0.704878, 0.443444, 0.702465, 0.437411, 0.702465, 0.437411, 0.704878, 0.443444, 0.704878, 0.443444, 0.704878, 0.443444, 0.707602, 0.445455, 0.704878, 0.443444, 0.704878, 0.443444, 0.707602, 0.445455, 0.707602, 0.445455, 0.707602, 0.445455, 0.708842, 0.445170, 0.707602, 0.445455, 0.707602, 0.445455, 0.708842, 0.445170, 0.708842, 0.445170, 0.708842, 0.445170, 0.710093, 0.444318, 0.708842, 0.445170, 0.708842, 0.445170, 0.710093, 0.444318, 0.710093, 0.444318, 0.710093, 0.444318, 0.711356, 0.442898, 0.710093, 0.444318, 0.710093, 0.444318, 0.711356, 0.442898, 0.711356, 0.442898, 0.711356, 0.442898, 0.712631, 0.440909, 0.711356, 0.442898, 0.711356, 0.442898, 0.712631, 0.440909, 0.712631, 0.440909, 0.712631, 0.440909, 0.713919, 0.438352, 0.712631, 0.440909, 0.712631, 0.440909, 0.713919, 0.438352, 0.713919, 0.438352, 0.713919, 0.438352, 0.715221, 0.435227, 0.713919, 0.438352, 0.713919, 0.438352, 0.715221, 0.435227, 0.715221, 0.435227, 0.715221, 0.435227, 0.716539, 0.431534, 0.715221, 0.435227, 0.715221, 0.435227, 0.716539, 0.431534, 0.716539, 0.431534, 0.695493, 0.351718, 0.695288, 0.325284, 0.700059, 0.330966, 0.695288, 0.325284, 0.695493, 0.351718, 0.700059, 0.330966, 0.695493, 0.351718, 0.700059, 0.330966, 0.700190, 0.346569, 0.700059, 0.330966, 0.695493, 0.351718, 0.700190, 0.346569, 0.696108, 0.375195, 0.695493, 0.351718, 0.700190, 0.346569, 0.695493, 0.351718, 0.696108, 0.375195, 0.700190, 0.346569, 0.696108, 0.375195, 0.700190, 0.346569, 0.700585, 0.360281, 0.700190, 0.346569, 0.696108, 0.375195, 0.700585, 0.360281, 0.697132, 0.395716, 0.696108, 0.375195, 0.700585, 0.360281, 0.696108, 0.375195, 0.697132, 0.395716, 0.700585, 0.360281, 0.697132, 0.395716, 0.700585, 0.360281, 0.701242, 0.372101, 0.700585, 0.360281, 0.697132, 0.395716, 0.701242, 0.372101, 0.698567, 0.413281, 0.697132, 0.395716, 0.701242, 0.372101, 0.697132, 0.395716, 0.698567, 0.413281, 0.701242, 0.372101, 0.698567, 0.413281, 0.701242, 0.372101, 0.702162, 0.382031, 0.701242, 0.372101, 0.698567, 0.413281, 0.702162, 0.382031, 0.700361, 0.427357, 0.698567, 0.413281, 0.702162, 0.382031, 0.698567, 0.413281, 0.700361, 0.427357, 0.702162, 0.382031, 0.700361, 0.427357, 0.702162, 0.382031, 0.703324, 0.389893, 0.702162, 0.382031, 0.700361, 0.427357, 0.703324, 0.389893, 0.702465, 0.437411, 0.700361, 0.427357, 0.703324, 0.389893, 0.700361, 0.427357, 0.702465, 0.437411, 0.703324, 0.389893, 0.702465, 0.437411, 0.703324, 0.389893, 0.704709, 0.395508, 0.703324, 0.389893, 0.702465, 0.437411, 0.704709, 0.395508, 0.702465, 0.437411, 0.704709, 0.395508, 0.704878, 0.443444, 0.704709, 0.395508, 0.702465, 0.437411, 0.704878, 0.443444, 0.704878, 0.443444, 0.704709, 0.395508, 0.706315, 0.398877, 0.704709, 0.395508, 0.704878, 0.443444, 0.706315, 0.398877, 0.707602, 0.445455, 0.704878, 0.443444, 0.706315, 0.398877, 0.704878, 0.443444, 0.707602, 0.445455, 0.706315, 0.398877, 0.707602, 0.445455, 0.706315, 0.398877, 0.708142, 0.400000, 0.706315, 0.398877, 0.707602, 0.445455, 0.708142, 0.400000, 0.708142, 0.400000, 0.708842, 0.445170, 0.707602, 0.445455, 0.708842, 0.445170, 0.708142, 0.400000, 0.707602, 0.445455, 0.708842, 0.445170, 0.708142, 0.400000, 0.709269, 0.399578, 0.708142, 0.400000, 0.708842, 0.445170, 0.709269, 0.399578, 0.708842, 0.445170, 0.709269, 0.399578, 0.710093, 0.444318, 0.709269, 0.399578, 0.708842, 0.445170, 0.710093, 0.444318, 0.710093, 0.444318, 0.709269, 0.399578, 0.710416, 0.398313, 0.709269, 0.399578, 0.710093, 0.444318, 0.710416, 0.398313, 0.711356, 0.442898, 0.710093, 0.444318, 0.710416, 0.398313, 0.710093, 0.444318, 0.711356, 0.442898, 0.710416, 0.398313, 0.711356, 0.442898, 0.710416, 0.398313, 0.711583, 0.396205, 0.710416, 0.398313, 0.711356, 0.442898, 0.711583, 0.396205, 0.711356, 0.442898, 0.711583, 0.396205, 0.712631, 0.440909, 0.711583, 0.396205, 0.711356, 0.442898, 0.712631, 0.440909, 0.712631, 0.440909, 0.711583, 0.396205, 0.712772, 0.393253, 0.711583, 0.396205, 0.712631, 0.440909, 0.712772, 0.393253, 0.712631, 0.440909, 0.712772, 0.393253, 0.713919, 0.438352, 0.712772, 0.393253, 0.712631, 0.440909, 0.713919, 0.438352, 0.713919, 0.438352, 0.712772, 0.393253, 0.713989, 0.389440, 0.712772, 0.393253, 0.713919, 0.438352, 0.713989, 0.389440, 0.713919, 0.438352, 0.713989, 0.389440, 0.715221, 0.435227, 0.713989, 0.389440, 0.713919, 0.438352, 0.715221, 0.435227, 0.715221, 0.435227, 0.713989, 0.389440, 0.715245, 0.384748, 0.713989, 0.389440, 0.715221, 0.435227, 0.715245, 0.384748, 0.715221, 0.435227, 0.715245, 0.384748, 0.716539, 0.431534, 0.715245, 0.384748, 0.715221, 0.435227, 0.716539, 0.431534, 0.716539, 0.431534, 0.715245, 0.384748, 0.716539, 0.379177, 0.715245, 0.384748, 0.716539, 0.431534, 0.716539, 0.379177, 0.717871, 0.427273, 0.716539, 0.431534, 0.716539, 0.379177, 0.716539, 0.431534, 0.717871, 0.427273, 0.716539, 0.379177, 0.716539, 0.379177, 0.717871, 0.372727, 0.717871, 0.427273, 0.717871, 0.372727, 0.716539, 0.379177, 0.717871, 0.427273, 0.700059, 0.330966, 0.695288, 0.325284, 0.700146, 0.317960, 0.695288, 0.325284, 0.700059, 0.330966, 0.700146, 0.317960, 0.695288, 0.325284, 0.695437, 0.302521, 0.700146, 0.317960, 0.695437, 0.302521, 0.695288, 0.325284, 0.700146, 0.317960, 0.700146, 0.317960, 0.695437, 0.302521, 0.700408, 0.306641, 0.695437, 0.302521, 0.700146, 0.317960, 0.700408, 0.306641, 0.695437, 0.302521, 0.695882, 0.282386, 0.700408, 0.306641, 0.695882, 0.282386, 0.695437, 0.302521, 0.700408, 0.306641, 0.700408, 0.306641, 0.695882, 0.282386, 0.700845, 0.297008, 0.695882, 0.282386, 0.700408, 0.306641, 0.700845, 0.297008, 0.695882, 0.282386, 0.696624, 0.264879, 0.700845, 0.297008, 0.696624, 0.264879, 0.695882, 0.282386, 0.700845, 0.297008, 0.700845, 0.297008, 0.696624, 0.264879, 0.697662, 0.250000, 0.696624, 0.264879, 0.700845, 0.297008, 0.697662, 0.250000, 0.701457, 0.289063, 0.700845, 0.297008, 0.697662, 0.250000, 0.700845, 0.297008, 0.701457, 0.289063, 0.697662, 0.250000, 0.701457, 0.289063, 0.697662, 0.250000, 0.699032, 0.237340, 0.697662, 0.250000, 0.701457, 0.289063, 0.699032, 0.237340, 0.702313, 0.282306, 0.701457, 0.289063, 0.699032, 0.237340, 0.701457, 0.289063, 0.702313, 0.282306, 0.699032, 0.237340, 0.699032, 0.237340, 0.700770, 0.226491, 0.702313, 0.282306, 0.700770, 0.226491, 0.699032, 0.237340, 0.702313, 0.282306, 0.702313, 0.282306, 0.700770, 0.226491, 0.703481, 0.276243, 0.700770, 0.226491, 0.702313, 0.282306, 0.703481, 0.276243, 0.700770, 0.226491, 0.702874, 0.217454, 0.703481, 0.276243, 0.702874, 0.217454, 0.700770, 0.226491, 0.703481, 0.276243, 0.703481, 0.276243, 0.702874, 0.217454, 0.704962, 0.270872, 0.702874, 0.217454, 0.703481, 0.276243, 0.704962, 0.270872, 0.704962, 0.270872, 0.702874, 0.217454, 0.705346, 0.210227, 0.702874, 0.217454, 0.704962, 0.270872, 0.705346, 0.210227, 0.704962, 0.270872, 0.705346, 0.210227, 0.706756, 0.266193, 0.705346, 0.210227, 0.704962, 0.270872, 0.706756, 0.266193, 0.706756, 0.266193, 0.705346, 0.210227, 0.708236, 0.203267, 0.705346, 0.210227, 0.706756, 0.266193, 0.708236, 0.203267, 0.706756, 0.266193, 0.708236, 0.203267, 0.709623, 0.258949, 0.708236, 0.203267, 0.706756, 0.266193, 0.709623, 0.258949, 0.709623, 0.258949, 0.708236, 0.203267, 0.709919, 0.198065, 0.708236, 0.203267, 0.709623, 0.258949, 0.709919, 0.198065, 0.709623, 0.258949, 0.709919, 0.198065, 0.712104, 0.251429, 0.709919, 0.198065, 0.709623, 0.258949, 0.712104, 0.251429, 0.709919, 0.198065, 0.711347, 0.191690, 0.712104, 0.251429, 0.711347, 0.191690, 0.709919, 0.198065, 0.712104, 0.251429, 0.712104, 0.251429, 0.711347, 0.191690, 0.714238, 0.241513, 0.711347, 0.191690, 0.712104, 0.251429, 0.714238, 0.241513, 0.711347, 0.191690, 0.712521, 0.184144, 0.714238, 0.241513, 0.712521, 0.184144, 0.711347, 0.191690, 0.714238, 0.241513, 0.714238, 0.241513, 0.712521, 0.184144, 0.716023, 0.229199, 0.712521, 0.184144, 0.714238, 0.241513, 0.716023, 0.229199, 0.712521, 0.184144, 0.713441, 0.175426, 0.716023, 0.229199, 0.713441, 0.175426, 0.712521, 0.184144, 0.716023, 0.229199, 0.716023, 0.229199, 0.713441, 0.175426, 0.717460, 0.214489, 0.713441, 0.175426, 0.716023, 0.229199, 0.717460, 0.214489, 0.713441, 0.175426, 0.714135, 0.165226, 0.717460, 0.214489, 0.714135, 0.165226, 0.713441, 0.175426, 0.717460, 0.214489, 0.717460, 0.214489, 0.714135, 0.165226, 0.718565, 0.197230, 0.714135, 0.165226, 0.717460, 0.214489, 0.718565, 0.197230, 0.718565, 0.197230, 0.714135, 0.165226, 0.719354, 0.177273, 0.714135, 0.165226, 0.718565, 0.197230, 0.719354, 0.177273, 0.719354, 0.177273, 0.714135, 0.165226, 0.714631, 0.153232, 0.714135, 0.165226, 0.719354, 0.177273, 0.714631, 0.153232, 0.719354, 0.177273, 0.714631, 0.153232, 0.719828, 0.154616, 0.714631, 0.153232, 0.719354, 0.177273, 0.719828, 0.154616, 0.719828, 0.154616, 0.714631, 0.153232, 0.714929, 0.139444, 0.714631, 0.153232, 0.719828, 0.154616, 0.714929, 0.139444, 0.719828, 0.154616, 0.714929, 0.139444, 0.719986, 0.129261, 0.714929, 0.139444, 0.719828, 0.154616, 0.719986, 0.129261, 0.714929, 0.139444, 0.715028, 0.123864, 0.719986, 0.129261, 0.715028, 0.123864, 0.714929, 0.139444, 0.719986, 0.129261, 0.715028, 0.123864, 0.719772, 0.099370, 0.719986, 0.129261, 0.719772, 0.099370, 0.715028, 0.123864, 0.719986, 0.129261, 0.715028, 0.123864, 0.714890, 0.106121, 0.719772, 0.099370, 0.714890, 0.106121, 0.715028, 0.123864, 0.719772, 0.099370, 0.719772, 0.099370, 0.714890, 0.106121, 0.719131, 0.073331, 0.714890, 0.106121, 0.719772, 0.099370, 0.719131, 0.073331, 0.714890, 0.106121, 0.714475, 0.090536, 0.719131, 0.073331, 0.714475, 0.090536, 0.714890, 0.106121, 0.719131, 0.073331, 0.714475, 0.090536, 0.718063, 0.051145, 0.719131, 0.073331, 0.718063, 0.051145, 0.714475, 0.090536, 0.719131, 0.073331, 0.714475, 0.090536, 0.713785, 0.077108, 0.718063, 0.051145, 0.713785, 0.077108, 0.714475, 0.090536, 0.718063, 0.051145, 0.713785, 0.077108, 0.716567, 0.032812, 0.718063, 0.051145, 0.716567, 0.032812, 0.713785, 0.077108, 0.718063, 0.051145, 0.713785, 0.077108, 0.712819, 0.065838, 0.716567, 0.032812, 0.712819, 0.065838, 0.713785, 0.077108, 0.716567, 0.032812, 0.712819, 0.065838, 0.714658, 0.018457, 0.716567, 0.032812, 0.714658, 0.018457, 0.712819, 0.065838, 0.716567, 0.032812, 0.712819, 0.065838, 0.711598, 0.056920, 0.714658, 0.018457, 0.711598, 0.056920, 0.712819, 0.065838, 0.714658, 0.018457, 0.711598, 0.056920, 0.712352, 0.008203, 0.714658, 0.018457, 0.712352, 0.008203, 0.711598, 0.056920, 0.714658, 0.018457, 0.711598, 0.056920, 0.710146, 0.050550, 0.712352, 0.008203, 0.710146, 0.050550, 0.711598, 0.056920, 0.712352, 0.008203, 0.710146, 0.050550, 0.709647, 0.002051, 0.712352, 0.008203, 0.709647, 0.002051, 0.710146, 0.050550, 0.712352, 0.008203, 0.708461, 0.046729, 0.709647, 0.002051, 0.710146, 0.050550, 0.709647, 0.002051, 0.708461, 0.046729, 0.710146, 0.050550, 0.709647, 0.002051, 0.708461, 0.046729, 0.706545, 0, 0.708461, 0.046729, 0.709647, 0.002051, 0.706545, 0, 0.708461, 0.046729, 0.706545, 0.045455, 0.706545, 0, 0.706545, 0.045455, 0.708461, 0.046729, 0.706545, 0, 0.705279, 0.000431, 0.706545, 0, 0.706545, 0.045455, 0.706545, 0, 0.705279, 0.000431, 0.706545, 0.045455, 0.705199, 0.046169, 0.705279, 0.000431, 0.706545, 0.045455, 0.705279, 0.000431, 0.705199, 0.046169, 0.706545, 0.045455, 0.705199, 0.046169, 0.703974, 0.001722, 0.705279, 0.000431, 0.703974, 0.001722, 0.705199, 0.046169, 0.705279, 0.000431, 0.703842, 0.048313, 0.703974, 0.001722, 0.705199, 0.046169, 0.703974, 0.001722, 0.703842, 0.048313, 0.705199, 0.046169, 0.703842, 0.048313, 0.702630, 0.003875, 0.703974, 0.001722, 0.702630, 0.003875, 0.703842, 0.048313, 0.703974, 0.001722, 0.702473, 0.051887, 0.702630, 0.003875, 0.703842, 0.048313, 0.702630, 0.003875, 0.702473, 0.051887, 0.703842, 0.048313, 0.702473, 0.051887, 0.701245, 0.006889, 0.702630, 0.003875, 0.701245, 0.006889, 0.702473, 0.051887, 0.702630, 0.003875, 0.701093, 0.056889, 0.701245, 0.006889, 0.702473, 0.051887, 0.701245, 0.006889, 0.701093, 0.056889, 0.702473, 0.051887, 0.701093, 0.056889, 0.699829, 0.010747, 0.701245, 0.006889, 0.699829, 0.010747, 0.701093, 0.056889, 0.701245, 0.006889, 0.699699, 0.063303, 0.699829, 0.010747, 0.701093, 0.056889, 0.699829, 0.010747, 0.699699, 0.063303, 0.701093, 0.056889, 0.699699, 0.063303, 0.698387, 0.015430, 0.699829, 0.010747, 0.698387, 0.015430, 0.699699, 0.063303, 0.699829, 0.010747, 0.699699, 0.063303, 0.696921, 0.020938, 0.698387, 0.015430, 0.696921, 0.020938, 0.699699, 0.063303, 0.698387, 0.015430, 0.699699, 0.063303, 0.698290, 0.071112, 0.696921, 0.020938, 0.698290, 0.071112, 0.699699, 0.063303, 0.696921, 0.020938, 0.695429, 0.027273, 0.696921, 0.020938, 0.698290, 0.071112, 0.696921, 0.020938, 0.695429, 0.027273, 0.698290, 0.071112, 0.696867, 0.080313, 0.695429, 0.027273, 0.698290, 0.071112, 0.695429, 0.027273, 0.696867, 0.080313, 0.698290, 0.071112, 0.695429, 0.090909, 0.695429, 0.027273, 0.696867, 0.080313, 0.695429, 0.027273, 0.695429, 0.090909, 0.696867, 0.080313, 0.732417, 0.209091, 0.732417, 0.054545, 0.732417, 0.209091, 0.732417, 0.209091, 0.732417, 0.054545, 0.732417, 0.054545, 0.732417, 0.054545, 0.740031, 0.054545, 0.732417, 0.054545, 0.732417, 0.054545, 0.740031, 0.054545, 0.740031, 0.054545, 0.740031, 0.054545, 0.741822, 0.055740, 0.740031, 0.054545, 0.740031, 0.054545, 0.741822, 0.055740, 0.741822, 0.055740, 0.741822, 0.055740, 0.743364, 0.059322, 0.741822, 0.055740, 0.741822, 0.055740, 0.743364, 0.059322, 0.743364, 0.059322, 0.743364, 0.059322, 0.744659, 0.065292, 0.743364, 0.059322, 0.743364, 0.059322, 0.744659, 0.065292, 0.744659, 0.065292, 0.744659, 0.065292, 0.745706, 0.073651, 0.744659, 0.065292, 0.744659, 0.065292, 0.745706, 0.073651, 0.745706, 0.073651, 0.745706, 0.073651, 0.746513, 0.084459, 0.745706, 0.073651, 0.745706, 0.073651, 0.746513, 0.084459, 0.746513, 0.084459, 0.746513, 0.084459, 0.747089, 0.097781, 0.746513, 0.084459, 0.746513, 0.084459, 0.747089, 0.097781, 0.747089, 0.097781, 0.747089, 0.097781, 0.747435, 0.113614, 0.747089, 0.097781, 0.747089, 0.097781, 0.747435, 0.113614, 0.747435, 0.113614, 0.747435, 0.113614, 0.747550, 0.131960, 0.747435, 0.113614, 0.747435, 0.113614, 0.747550, 0.131960, 0.747550, 0.131960, 0.747550, 0.131960, 0.747435, 0.150413, 0.747550, 0.131960, 0.747550, 0.131960, 0.747435, 0.150413, 0.747435, 0.150413, 0.747435, 0.150413, 0.747089, 0.166282, 0.747435, 0.150413, 0.747435, 0.150413, 0.747089, 0.166282, 0.747089, 0.166282, 0.747089, 0.166282, 0.746513, 0.179568, 0.747089, 0.166282, 0.747089, 0.166282, 0.746513, 0.179568, 0.746513, 0.179568, 0.746513, 0.179568, 0.745706, 0.190270, 0.746513, 0.179568, 0.746513, 0.179568, 0.745706, 0.190270, 0.745706, 0.190270, 0.745706, 0.190270, 0.744659, 0.198504, 0.745706, 0.190270, 0.745706, 0.190270, 0.744659, 0.198504, 0.744659, 0.198504, 0.744659, 0.198504, 0.743364, 0.204386, 0.744659, 0.198504, 0.744659, 0.198504, 0.743364, 0.204386, 0.743364, 0.204386, 0.743364, 0.204386, 0.741822, 0.207915, 0.743364, 0.204386, 0.743364, 0.204386, 0.741822, 0.207915, 0.741822, 0.207915, 0.741822, 0.207915, 0.740031, 0.209091, 0.741822, 0.207915, 0.741822, 0.207915, 0.740031, 0.209091, 0.740031, 0.209091, 0.732417, 0.390909, 0.732417, 0.254545, 0.732417, 0.390909, 0.732417, 0.390909, 0.732417, 0.254545, 0.732417, 0.254545, 0.732417, 0.254545, 0.739443, 0.254545, 0.732417, 0.254545, 0.732417, 0.254545, 0.739443, 0.254545, 0.739443, 0.254545, 0.739443, 0.254545, 0.741071, 0.255597, 0.739443, 0.254545, 0.739443, 0.254545, 0.741071, 0.255597, 0.741071, 0.255597, 0.741071, 0.255597, 0.742477, 0.258754, 0.741071, 0.255597, 0.741071, 0.255597, 0.742477, 0.258754, 0.742477, 0.258754, 0.742477, 0.258754, 0.743662, 0.264014, 0.742477, 0.258754, 0.742477, 0.258754, 0.743662, 0.264014, 0.743662, 0.264014, 0.743662, 0.264014, 0.744625, 0.271378, 0.743662, 0.264014, 0.743662, 0.264014, 0.744625, 0.271378, 0.744625, 0.271378, 0.744625, 0.271378, 0.745370, 0.280899, 0.744625, 0.271378, 0.744625, 0.271378, 0.745370, 0.280899, 0.745370, 0.280899, 0.745370, 0.280899, 0.745902, 0.292631, 0.745370, 0.280899, 0.745370, 0.280899, 0.745902, 0.292631, 0.745902, 0.292631, 0.745902, 0.292631, 0.746222, 0.306574, 0.745902, 0.292631, 0.745902, 0.292631, 0.746222, 0.306574, 0.746222, 0.306574, 0.746222, 0.306574, 0.746328, 0.322727, 0.746222, 0.306574, 0.746222, 0.306574, 0.746328, 0.322727, 0.746328, 0.322727, 0.746328, 0.322727, 0.746222, 0.338765, 0.746328, 0.322727, 0.746328, 0.322727, 0.746222, 0.338765, 0.746222, 0.338765, 0.746222, 0.338765, 0.745902, 0.352646, 0.746222, 0.338765, 0.746222, 0.338765, 0.745902, 0.352646, 0.745902, 0.352646, 0.745902, 0.352646, 0.745370, 0.364369, 0.745902, 0.352646, 0.745902, 0.352646, 0.745370, 0.364369, 0.745370, 0.364369, 0.745370, 0.364369, 0.744625, 0.373935, 0.745370, 0.364369, 0.745370, 0.364369, 0.744625, 0.373935, 0.744625, 0.373935, 0.744625, 0.373935, 0.743662, 0.381361, 0.744625, 0.373935, 0.744625, 0.373935, 0.743662, 0.381361, 0.743662, 0.381361, 0.743662, 0.381361, 0.742477, 0.386665, 0.743662, 0.381361, 0.743662, 0.381361, 0.742477, 0.386665, 0.742477, 0.386665, 0.742477, 0.386665, 0.741071, 0.389848, 0.742477, 0.386665, 0.742477, 0.386665, 0.741071, 0.389848, 0.741071, 0.389848, 0.741071, 0.389848, 0.739443, 0.390909, 0.741071, 0.389848, 0.741071, 0.389848, 0.739443, 0.390909, 0.739443, 0.390909, 0.727670, 0.436364, 0.739796, 0.436364, 0.727670, 0.436364, 0.727670, 0.436364, 0.739796, 0.436364, 0.739796, 0.436364, 0.739796, 0.436364, 0.742354, 0.434606, 0.739796, 0.436364, 0.739796, 0.436364, 0.742354, 0.434606, 0.742354, 0.434606, 0.742354, 0.434606, 0.744601, 0.429332, 0.742354, 0.434606, 0.742354, 0.434606, 0.744601, 0.429332, 0.744601, 0.429332, 0.744601, 0.429332, 0.746537, 0.420543, 0.744601, 0.429332, 0.744601, 0.429332, 0.746537, 0.420543, 0.746537, 0.420543, 0.746537, 0.420543, 0.748161, 0.408239, 0.746537, 0.420543, 0.746537, 0.420543, 0.748161, 0.408239, 0.748161, 0.408239, 0.748161, 0.408239, 0.749446, 0.392685, 0.748161, 0.408239, 0.748161, 0.408239, 0.749446, 0.392685, 0.749446, 0.392685, 0.749446, 0.392685, 0.750364, 0.374148, 0.749446, 0.392685, 0.749446, 0.392685, 0.750364, 0.374148, 0.750364, 0.374148, 0.750364, 0.374148, 0.750915, 0.352628, 0.750364, 0.374148, 0.750364, 0.374148, 0.750915, 0.352628, 0.750915, 0.352628, 0.750915, 0.352628, 0.751099, 0.328125, 0.750915, 0.352628, 0.750915, 0.352628, 0.751099, 0.328125, 0.751099, 0.328125, 0.751099, 0.328125, 0.751005, 0.309113, 0.751099, 0.328125, 0.751099, 0.328125, 0.751005, 0.309113, 0.751005, 0.309113, 0.751005, 0.309113, 0.750723, 0.292134, 0.751005, 0.309113, 0.751005, 0.309113, 0.750723, 0.292134, 0.750723, 0.292134, 0.750723, 0.292134, 0.750253, 0.277188, 0.750723, 0.292134, 0.750723, 0.292134, 0.750253, 0.277188, 0.750253, 0.277188, 0.750253, 0.277188, 0.749595, 0.264276, 0.750253, 0.277188, 0.750253, 0.277188, 0.749595, 0.264276, 0.749595, 0.264276, 0.749595, 0.264276, 0.748755, 0.253502, 0.749595, 0.264276, 0.749595, 0.264276, 0.748755, 0.253502, 0.748755, 0.253502, 0.748755, 0.253502, 0.747738, 0.244975, 0.748755, 0.253502, 0.748755, 0.253502, 0.747738, 0.244975, 0.747738, 0.244975, 0.747738, 0.244975, 0.746546, 0.238694, 0.747738, 0.244975, 0.747738, 0.244975, 0.746546, 0.238694, 0.746546, 0.238694, 0.746546, 0.238694, 0.745177, 0.234659, 0.746546, 0.238694, 0.746546, 0.238694, 0.745177, 0.234659, 0.745177, 0.234659, 0.745177, 0.234659, 0.746830, 0.228964, 0.745177, 0.234659, 0.745177, 0.234659, 0.746830, 0.228964, 0.746830, 0.228964, 0.746830, 0.228964, 0.748287, 0.220827, 0.746830, 0.228964, 0.746830, 0.228964, 0.748287, 0.220827, 0.748287, 0.220827, 0.748287, 0.220827, 0.749550, 0.210249, 0.748287, 0.220827, 0.748287, 0.220827, 0.749550, 0.210249, 0.749550, 0.210249, 0.749550, 0.210249, 0.750617, 0.197230, 0.749550, 0.210249, 0.749550, 0.210249, 0.750617, 0.197230, 0.750617, 0.197230, 0.750617, 0.197230, 0.751465, 0.182116, 0.750617, 0.197230, 0.750617, 0.197230, 0.751465, 0.182116, 0.751465, 0.182116, 0.751465, 0.182116, 0.752071, 0.165252, 0.751465, 0.182116, 0.751465, 0.182116, 0.752071, 0.165252, 0.752071, 0.165252, 0.752071, 0.165252, 0.752434, 0.146640, 0.752071, 0.165252, 0.752071, 0.165252, 0.752434, 0.146640, 0.752434, 0.146640, 0.752434, 0.146640, 0.752555, 0.126278, 0.752434, 0.146640, 0.752434, 0.146640, 0.752555, 0.126278, 0.752555, 0.126278, 0.752555, 0.126278, 0.752356, 0.099822, 0.752555, 0.126278, 0.752555, 0.126278, 0.752356, 0.099822, 0.752356, 0.099822, 0.752356, 0.099822, 0.751757, 0.076563, 0.752356, 0.099822, 0.752356, 0.099822, 0.751757, 0.076563, 0.751757, 0.076563, 0.751757, 0.076563, 0.750758, 0.056499, 0.751757, 0.076563, 0.751757, 0.076563, 0.750758, 0.056499, 0.750758, 0.056499, 0.750758, 0.056499, 0.749360, 0.039631, 0.750758, 0.056499, 0.750758, 0.056499, 0.749360, 0.039631, 0.749360, 0.039631, 0.749360, 0.039631, 0.747593, 0.026270, 0.749360, 0.039631, 0.749360, 0.039631, 0.747593, 0.026270, 0.747593, 0.026270, 0.747593, 0.026270, 0.745488, 0.016726, 0.747593, 0.026270, 0.747593, 0.026270, 0.745488, 0.016726, 0.745488, 0.016726, 0.745488, 0.016726, 0.743046, 0.011000, 0.745488, 0.016726, 0.745488, 0.016726, 0.743046, 0.011000, 0.743046, 0.011000, 0.743046, 0.011000, 0.740266, 0.009091, 0.743046, 0.011000, 0.743046, 0.011000, 0.740266, 0.009091, 0.740266, 0.009091, 0.740266, 0.009091, 0.727670, 0.009091, 0.740266, 0.009091, 0.740266, 0.009091, 0.727670, 0.009091, 0.727670, 0.009091, 0.727670, 0.436364, 0.732417, 0.390909, 0.739796, 0.436364, 0.732417, 0.390909, 0.727670, 0.436364, 0.739796, 0.436364, 0.732417, 0.390909, 0.727670, 0.436364, 0.732417, 0.254545, 0.727670, 0.436364, 0.732417, 0.390909, 0.732417, 0.254545, 0.732417, 0.254545, 0.727670, 0.436364, 0.732417, 0.209091, 0.727670, 0.436364, 0.732417, 0.254545, 0.732417, 0.209091, 0.739443, 0.254545, 0.732417, 0.254545, 0.732417, 0.209091, 0.732417, 0.254545, 0.739443, 0.254545, 0.732417, 0.209091, 0.740031, 0.209091, 0.739443, 0.254545, 0.732417, 0.209091, 0.739443, 0.254545, 0.740031, 0.209091, 0.732417, 0.209091, 0.739443, 0.254545, 0.740031, 0.209091, 0.741071, 0.255597, 0.740031, 0.209091, 0.739443, 0.254545, 0.741071, 0.255597, 0.741071, 0.255597, 0.740031, 0.209091, 0.741822, 0.207915, 0.740031, 0.209091, 0.741071, 0.255597, 0.741822, 0.207915, 0.741822, 0.207915, 0.742477, 0.258754, 0.741071, 0.255597, 0.742477, 0.258754, 0.741822, 0.207915, 0.741071, 0.255597, 0.745177, 0.234659, 0.742477, 0.258754, 0.741822, 0.207915, 0.742477, 0.258754, 0.745177, 0.234659, 0.741822, 0.207915, 0.743662, 0.264014, 0.742477, 0.258754, 0.745177, 0.234659, 0.742477, 0.258754, 0.743662, 0.264014, 0.745177, 0.234659, 0.744625, 0.271378, 0.743662, 0.264014, 0.745177, 0.234659, 0.743662, 0.264014, 0.744625, 0.271378, 0.745177, 0.234659, 0.744625, 0.271378, 0.745177, 0.234659, 0.746546, 0.238694, 0.745177, 0.234659, 0.744625, 0.271378, 0.746546, 0.238694, 0.745370, 0.280899, 0.744625, 0.271378, 0.746546, 0.238694, 0.744625, 0.271378, 0.745370, 0.280899, 0.746546, 0.238694, 0.745370, 0.280899, 0.746546, 0.238694, 0.747738, 0.244975, 0.746546, 0.238694, 0.745370, 0.280899, 0.747738, 0.244975, 0.745370, 0.280899, 0.747738, 0.244975, 0.748755, 0.253502, 0.747738, 0.244975, 0.745370, 0.280899, 0.748755, 0.253502, 0.745902, 0.292631, 0.745370, 0.280899, 0.748755, 0.253502, 0.745370, 0.280899, 0.745902, 0.292631, 0.748755, 0.253502, 0.745902, 0.292631, 0.748755, 0.253502, 0.749595, 0.264276, 0.748755, 0.253502, 0.745902, 0.292631, 0.749595, 0.264276, 0.745902, 0.292631, 0.749595, 0.264276, 0.750253, 0.277188, 0.749595, 0.264276, 0.745902, 0.292631, 0.750253, 0.277188, 0.746222, 0.306574, 0.745902, 0.292631, 0.750253, 0.277188, 0.745902, 0.292631, 0.746222, 0.306574, 0.750253, 0.277188, 0.746222, 0.306574, 0.750253, 0.277188, 0.750723, 0.292134, 0.750253, 0.277188, 0.746222, 0.306574, 0.750723, 0.292134, 0.746222, 0.306574, 0.750723, 0.292134, 0.751005, 0.309113, 0.750723, 0.292134, 0.746222, 0.306574, 0.751005, 0.309113, 0.746328, 0.322727, 0.746222, 0.306574, 0.751005, 0.309113, 0.746222, 0.306574, 0.746328, 0.322727, 0.751005, 0.309113, 0.746328, 0.322727, 0.751005, 0.309113, 0.751099, 0.328125, 0.751005, 0.309113, 0.746328, 0.322727, 0.751099, 0.328125, 0.746222, 0.338765, 0.746328, 0.322727, 0.751099, 0.328125, 0.746328, 0.322727, 0.746222, 0.338765, 0.751099, 0.328125, 0.750915, 0.352628, 0.746222, 0.338765, 0.751099, 0.328125, 0.746222, 0.338765, 0.750915, 0.352628, 0.751099, 0.328125, 0.750915, 0.352628, 0.745902, 0.352646, 0.746222, 0.338765, 0.745902, 0.352646, 0.750915, 0.352628, 0.746222, 0.338765, 0.750364, 0.374148, 0.745902, 0.352646, 0.750915, 0.352628, 0.745902, 0.352646, 0.750364, 0.374148, 0.750915, 0.352628, 0.749446, 0.392685, 0.745902, 0.352646, 0.750364, 0.374148, 0.745902, 0.352646, 0.749446, 0.392685, 0.750364, 0.374148, 0.745370, 0.364369, 0.745902, 0.352646, 0.749446, 0.392685, 0.745902, 0.352646, 0.745370, 0.364369, 0.749446, 0.392685, 0.748161, 0.408239, 0.745370, 0.364369, 0.749446, 0.392685, 0.745370, 0.364369, 0.748161, 0.408239, 0.749446, 0.392685, 0.744625, 0.373935, 0.745370, 0.364369, 0.748161, 0.408239, 0.745370, 0.364369, 0.744625, 0.373935, 0.748161, 0.408239, 0.746537, 0.420543, 0.744625, 0.373935, 0.748161, 0.408239, 0.744625, 0.373935, 0.746537, 0.420543, 0.748161, 0.408239, 0.743662, 0.381361, 0.744625, 0.373935, 0.746537, 0.420543, 0.744625, 0.373935, 0.743662, 0.381361, 0.746537, 0.420543, 0.744601, 0.429332, 0.743662, 0.381361, 0.746537, 0.420543, 0.743662, 0.381361, 0.744601, 0.429332, 0.746537, 0.420543, 0.742477, 0.386665, 0.743662, 0.381361, 0.744601, 0.429332, 0.743662, 0.381361, 0.742477, 0.386665, 0.744601, 0.429332, 0.742354, 0.434606, 0.742477, 0.386665, 0.744601, 0.429332, 0.742477, 0.386665, 0.742354, 0.434606, 0.744601, 0.429332, 0.742354, 0.434606, 0.741071, 0.389848, 0.742477, 0.386665, 0.741071, 0.389848, 0.742354, 0.434606, 0.742477, 0.386665, 0.739796, 0.436364, 0.741071, 0.389848, 0.742354, 0.434606, 0.741071, 0.389848, 0.739796, 0.436364, 0.742354, 0.434606, 0.739796, 0.436364, 0.739443, 0.390909, 0.741071, 0.389848, 0.739443, 0.390909, 0.739796, 0.436364, 0.741071, 0.389848, 0.732417, 0.390909, 0.739443, 0.390909, 0.739796, 0.436364, 0.739443, 0.390909, 0.732417, 0.390909, 0.739796, 0.436364, 0.745177, 0.234659, 0.741822, 0.207915, 0.743364, 0.204386, 0.741822, 0.207915, 0.745177, 0.234659, 0.743364, 0.204386, 0.743364, 0.204386, 0.744659, 0.198504, 0.745177, 0.234659, 0.744659, 0.198504, 0.743364, 0.204386, 0.745177, 0.234659, 0.745177, 0.234659, 0.744659, 0.198504, 0.746830, 0.228964, 0.744659, 0.198504, 0.745177, 0.234659, 0.746830, 0.228964, 0.744659, 0.198504, 0.745706, 0.190270, 0.746830, 0.228964, 0.745706, 0.190270, 0.744659, 0.198504, 0.746830, 0.228964, 0.746830, 0.228964, 0.745706, 0.190270, 0.748287, 0.220827, 0.745706, 0.190270, 0.746830, 0.228964, 0.748287, 0.220827, 0.745706, 0.190270, 0.746513, 0.179568, 0.748287, 0.220827, 0.746513, 0.179568, 0.745706, 0.190270, 0.748287, 0.220827, 0.748287, 0.220827, 0.746513, 0.179568, 0.749550, 0.210249, 0.746513, 0.179568, 0.748287, 0.220827, 0.749550, 0.210249, 0.749550, 0.210249, 0.746513, 0.179568, 0.750617, 0.197230, 0.746513, 0.179568, 0.749550, 0.210249, 0.750617, 0.197230, 0.746513, 0.179568, 0.747089, 0.166282, 0.750617, 0.197230, 0.747089, 0.166282, 0.746513, 0.179568, 0.750617, 0.197230, 0.750617, 0.197230, 0.747089, 0.166282, 0.751465, 0.182116, 0.747089, 0.166282, 0.750617, 0.197230, 0.751465, 0.182116, 0.751465, 0.182116, 0.747089, 0.166282, 0.747435, 0.150413, 0.747089, 0.166282, 0.751465, 0.182116, 0.747435, 0.150413, 0.752071, 0.165252, 0.751465, 0.182116, 0.747435, 0.150413, 0.751465, 0.182116, 0.752071, 0.165252, 0.747435, 0.150413, 0.752071, 0.165252, 0.747435, 0.150413, 0.752434, 0.146640, 0.747435, 0.150413, 0.752071, 0.165252, 0.752434, 0.146640, 0.747435, 0.150413, 0.747550, 0.131960, 0.752434, 0.146640, 0.747550, 0.131960, 0.747435, 0.150413, 0.752434, 0.146640, 0.752434, 0.146640, 0.747550, 0.131960, 0.752555, 0.126278, 0.747550, 0.131960, 0.752434, 0.146640, 0.752555, 0.126278, 0.747550, 0.131960, 0.747435, 0.113614, 0.752555, 0.126278, 0.747435, 0.113614, 0.747550, 0.131960, 0.752555, 0.126278, 0.747435, 0.113614, 0.752356, 0.099822, 0.752555, 0.126278, 0.752356, 0.099822, 0.747435, 0.113614, 0.752555, 0.126278, 0.747435, 0.113614, 0.747089, 0.097781, 0.752356, 0.099822, 0.747089, 0.097781, 0.747435, 0.113614, 0.752356, 0.099822, 0.747089, 0.097781, 0.751757, 0.076563, 0.752356, 0.099822, 0.751757, 0.076563, 0.747089, 0.097781, 0.752356, 0.099822, 0.747089, 0.097781, 0.750758, 0.056499, 0.751757, 0.076563, 0.750758, 0.056499, 0.747089, 0.097781, 0.751757, 0.076563, 0.747089, 0.097781, 0.746513, 0.084459, 0.750758, 0.056499, 0.746513, 0.084459, 0.747089, 0.097781, 0.750758, 0.056499, 0.746513, 0.084459, 0.749360, 0.039631, 0.750758, 0.056499, 0.749360, 0.039631, 0.746513, 0.084459, 0.750758, 0.056499, 0.746513, 0.084459, 0.745706, 0.073651, 0.749360, 0.039631, 0.745706, 0.073651, 0.746513, 0.084459, 0.749360, 0.039631, 0.745706, 0.073651, 0.747593, 0.026270, 0.749360, 0.039631, 0.747593, 0.026270, 0.745706, 0.073651, 0.749360, 0.039631, 0.745706, 0.073651, 0.744659, 0.065292, 0.747593, 0.026270, 0.744659, 0.065292, 0.745706, 0.073651, 0.747593, 0.026270, 0.744659, 0.065292, 0.745488, 0.016726, 0.747593, 0.026270, 0.745488, 0.016726, 0.744659, 0.065292, 0.747593, 0.026270, 0.744659, 0.065292, 0.743364, 0.059322, 0.745488, 0.016726, 0.743364, 0.059322, 0.744659, 0.065292, 0.745488, 0.016726, 0.743364, 0.059322, 0.743046, 0.011000, 0.745488, 0.016726, 0.743046, 0.011000, 0.743364, 0.059322, 0.745488, 0.016726, 0.741822, 0.055740, 0.743046, 0.011000, 0.743364, 0.059322, 0.743046, 0.011000, 0.741822, 0.055740, 0.743364, 0.059322, 0.743046, 0.011000, 0.741822, 0.055740, 0.740266, 0.009091, 0.741822, 0.055740, 0.743046, 0.011000, 0.740266, 0.009091, 0.741822, 0.055740, 0.740031, 0.054545, 0.740266, 0.009091, 0.740031, 0.054545, 0.741822, 0.055740, 0.740266, 0.009091, 0.740031, 0.054545, 0.732417, 0.054545, 0.740266, 0.009091, 0.732417, 0.054545, 0.740031, 0.054545, 0.740266, 0.009091, 0.732417, 0.054545, 0.727670, 0.009091, 0.740266, 0.009091, 0.727670, 0.009091, 0.732417, 0.054545, 0.740266, 0.009091, 0.727670, 0.009091, 0.732417, 0.054545, 0.732417, 0.209091, 0.732417, 0.054545, 0.727670, 0.009091, 0.732417, 0.209091, 0.727670, 0.436364, 0.727670, 0.009091, 0.732417, 0.209091, 0.727670, 0.009091, 0.727670, 0.436364, 0.732417, 0.209091, 0.760757, 0.436364, 0.765503, 0.436364, 0.760757, 0.436364, 0.760757, 0.436364, 0.765503, 0.436364, 0.765503, 0.436364, 0.765503, 0.436364, 0.765503, 0.009091, 0.765503, 0.436364, 0.765503, 0.436364, 0.765503, 0.009091, 0.765503, 0.009091, 0.765503, 0.009091, 0.760757, 0.009091, 0.765503, 0.009091, 0.765503, 0.009091, 0.760757, 0.009091, 0.760757, 0.009091, 0.760757, 0.436364, 0.765503, 0.009091, 0.765503, 0.436364, 0.765503, 0.009091, 0.760757, 0.436364, 0.765503, 0.436364, 0.760757, 0.436364, 0.760757, 0.009091, 0.765503, 0.009091, 0.760757, 0.009091, 0.760757, 0.436364, 0.765503, 0.009091, 0.775044, 0.436364, 0.779791, 0.436364, 0.775044, 0.436364, 0.775044, 0.436364, 0.779791, 0.436364, 0.779791, 0.436364, 0.779791, 0.436364, 0.779791, 0.054545, 0.779791, 0.436364, 0.779791, 0.436364, 0.779791, 0.054545, 0.779791, 0.054545, 0.779791, 0.054545, 0.796875, 0.054545, 0.779791, 0.054545, 0.779791, 0.054545, 0.796875, 0.054545, 0.796875, 0.054545, 0.796875, 0.054545, 0.796875, 0.009091, 0.796875, 0.054545, 0.796875, 0.054545, 0.796875, 0.009091, 0.796875, 0.009091, 0.796875, 0.009091, 0.775044, 0.009091, 0.796875, 0.009091, 0.796875, 0.009091, 0.775044, 0.009091, 0.775044, 0.009091, 0.775044, 0.436364, 0.779791, 0.054545, 0.779791, 0.436364, 0.779791, 0.054545, 0.775044, 0.436364, 0.779791, 0.436364, 0.775044, 0.436364, 0.775044, 0.009091, 0.779791, 0.054545, 0.775044, 0.009091, 0.775044, 0.436364, 0.779791, 0.054545, 0.779791, 0.054545, 0.775044, 0.009091, 0.796875, 0.009091, 0.775044, 0.009091, 0.779791, 0.054545, 0.796875, 0.009091, 0.779791, 0.054545, 0.796875, 0.009091, 0.796875, 0.054545, 0.796875, 0.009091, 0.779791, 0.054545, 0.796875, 0.054545, 0.802115, 0.436364, 0.806862, 0.436364, 0.802115, 0.436364, 0.802115, 0.436364, 0.806862, 0.436364, 0.806862, 0.436364, 0.806862, 0.436364, 0.806862, 0.054545, 0.806862, 0.436364, 0.806862, 0.436364, 0.806862, 0.054545, 0.806862, 0.054545, 0.806862, 0.054545, 0.823945, 0.054545, 0.806862, 0.054545, 0.806862, 0.054545, 0.823945, 0.054545, 0.823945, 0.054545, 0.823945, 0.054545, 0.823945, 0.009091, 0.823945, 0.054545, 0.823945, 0.054545, 0.823945, 0.009091, 0.823945, 0.009091, 0.823945, 0.009091, 0.802115, 0.009091, 0.823945, 0.009091, 0.823945, 0.009091, 0.802115, 0.009091, 0.802115, 0.009091, 0.802115, 0.436364, 0.806862, 0.054545, 0.806862, 0.436364, 0.806862, 0.054545, 0.802115, 0.436364, 0.806862, 0.436364, 0.802115, 0.436364, 0.802115, 0.009091, 0.806862, 0.054545, 0.802115, 0.009091, 0.802115, 0.436364, 0.806862, 0.054545, 0.806862, 0.054545, 0.802115, 0.009091, 0.823945, 0.009091, 0.802115, 0.009091, 0.806862, 0.054545, 0.823945, 0.009091, 0.806862, 0.054545, 0.823945, 0.009091, 0.823945, 0.054545, 0.823945, 0.009091, 0.806862, 0.054545, 0.823945, 0.054545, 0.843990, 0.242756, 0.845601, 0.236998, 0.843990, 0.242756, 0.843990, 0.242756, 0.845601, 0.236998, 0.845601, 0.236998, 0.845601, 0.236998, 0.847024, 0.228817, 0.845601, 0.236998, 0.845601, 0.236998, 0.847024, 0.228817, 0.847024, 0.228817, 0.847024, 0.228817, 0.848262, 0.218213, 0.847024, 0.228817, 0.847024, 0.228817, 0.848262, 0.218213, 0.848262, 0.218213, 0.848262, 0.218213, 0.849313, 0.205185, 0.848262, 0.218213, 0.848262, 0.218213, 0.849313, 0.205185, 0.849313, 0.205185, 0.849313, 0.205185, 0.850150, 0.190106, 0.849313, 0.205185, 0.849313, 0.205185, 0.850150, 0.190106, 0.850150, 0.190106, 0.850150, 0.190106, 0.850749, 0.173349, 0.850150, 0.190106, 0.850150, 0.190106, 0.850749, 0.173349, 0.850749, 0.173349, 0.850749, 0.173349, 0.851108, 0.154914, 0.850749, 0.173349, 0.850749, 0.173349, 0.851108, 0.154914, 0.851108, 0.154914, 0.851108, 0.154914, 0.851228, 0.134801, 0.851108, 0.154914, 0.851108, 0.154914, 0.851228, 0.134801, 0.851228, 0.134801, 0.851228, 0.134801, 0.851005, 0.104448, 0.851228, 0.134801, 0.851228, 0.134801, 0.851005, 0.104448, 0.851005, 0.104448, 0.851005, 0.104448, 0.850335, 0.077734, 0.851005, 0.104448, 0.851005, 0.104448, 0.850335, 0.077734, 0.850335, 0.077734, 0.850335, 0.077734, 0.849219, 0.054661, 0.850335, 0.077734, 0.850335, 0.077734, 0.849219, 0.054661, 0.849219, 0.054661, 0.849219, 0.054661, 0.847656, 0.035227, 0.849219, 0.054661, 0.849219, 0.054661, 0.847656, 0.035227, 0.847656, 0.035227, 0.847656, 0.035227, 0.845682, 0.019815, 0.847656, 0.035227, 0.847656, 0.035227, 0.845682, 0.019815, 0.845682, 0.019815, 0.845682, 0.019815, 0.843332, 0.008807, 0.845682, 0.019815, 0.845682, 0.019815, 0.843332, 0.008807, 0.843332, 0.008807, 0.843332, 0.008807, 0.840606, 0.002202, 0.843332, 0.008807, 0.843332, 0.008807, 0.840606, 0.002202, 0.840606, 0.002202, 0.840606, 0.002202, 0.837504, 0, 0.840606, 0.002202, 0.840606, 0.002202, 0.837504, 0, 0.837504, 0, 0.837504, 0, 0.836392, 0.000284, 0.837504, 0, 0.837504, 0, 0.836392, 0.000284, 0.836392, 0.000284, 0.836392, 0.000284, 0.835263, 0.001136, 0.836392, 0.000284, 0.836392, 0.000284, 0.835263, 0.001136, 0.835263, 0.001136, 0.835263, 0.001136, 0.834118, 0.002557, 0.835263, 0.001136, 0.835263, 0.001136, 0.834118, 0.002557, 0.834118, 0.002557, 0.834118, 0.002557, 0.832957, 0.004545, 0.834118, 0.002557, 0.834118, 0.002557, 0.832957, 0.004545, 0.832957, 0.004545, 0.832957, 0.004545, 0.831779, 0.007102, 0.832957, 0.004545, 0.832957, 0.004545, 0.831779, 0.007102, 0.831779, 0.007102, 0.831779, 0.007102, 0.830581, 0.010227, 0.831779, 0.007102, 0.831779, 0.007102, 0.830581, 0.010227, 0.830581, 0.010227, 0.830581, 0.010227, 0.829364, 0.013920, 0.830581, 0.010227, 0.830581, 0.010227, 0.829364, 0.013920, 0.829364, 0.013920, 0.829364, 0.013920, 0.828128, 0.018182, 0.829364, 0.013920, 0.829364, 0.013920, 0.828128, 0.018182, 0.828128, 0.018182, 0.828128, 0.018182, 0.828128, 0.072727, 0.828128, 0.018182, 0.828128, 0.018182, 0.828128, 0.072727, 0.828128, 0.072727, 0.828128, 0.072727, 0.829139, 0.066393, 0.828128, 0.072727, 0.828128, 0.072727, 0.829139, 0.066393, 0.829139, 0.066393, 0.829139, 0.066393, 0.830196, 0.060884, 0.829139, 0.066393, 0.829139, 0.066393, 0.830196, 0.060884, 0.830196, 0.060884, 0.830196, 0.060884, 0.831301, 0.056201, 0.830196, 0.060884, 0.830196, 0.060884, 0.831301, 0.056201, 0.831301, 0.056201, 0.831301, 0.056201, 0.832452, 0.052344, 0.831301, 0.056201, 0.831301, 0.056201, 0.832452, 0.052344, 0.832452, 0.052344, 0.832452, 0.052344, 0.833640, 0.049330, 0.832452, 0.052344, 0.832452, 0.052344, 0.833640, 0.049330, 0.833640, 0.049330, 0.833640, 0.049330, 0.834855, 0.047177, 0.833640, 0.049330, 0.833640, 0.049330, 0.834855, 0.047177, 0.834855, 0.047177, 0.834855, 0.047177, 0.836096, 0.045885, 0.834855, 0.047177, 0.834855, 0.047177, 0.836096, 0.045885, 0.836096, 0.045885, 0.836096, 0.045885, 0.837363, 0.045455, 0.836096, 0.045885, 0.836096, 0.045885, 0.837363, 0.045455, 0.837363, 0.045455, 0.837363, 0.045455, 0.839463, 0.046866, 0.837363, 0.045455, 0.837363, 0.045455, 0.839463, 0.046866, 0.839463, 0.046866, 0.839463, 0.046866, 0.841297, 0.051101, 0.839463, 0.046866, 0.839463, 0.046866, 0.841297, 0.051101, 0.841297, 0.051101, 0.841297, 0.051101, 0.842864, 0.058159, 0.841297, 0.051101, 0.841297, 0.051101, 0.842864, 0.058159, 0.842864, 0.058159, 0.842864, 0.058159, 0.844166, 0.068040, 0.842864, 0.058159, 0.842864, 0.058159, 0.844166, 0.068040, 0.844166, 0.068040, 0.844166, 0.068040, 0.845189, 0.080611, 0.844166, 0.068040, 0.844166, 0.068040, 0.845189, 0.080611, 0.845189, 0.080611, 0.845189, 0.080611, 0.845920, 0.095739, 0.845189, 0.080611, 0.845189, 0.080611, 0.845920, 0.095739, 0.845920, 0.095739, 0.845920, 0.095739, 0.846358, 0.113423, 0.845920, 0.095739, 0.845920, 0.095739, 0.846358, 0.113423, 0.846358, 0.113423, 0.846358, 0.113423, 0.846504, 0.133665, 0.846358, 0.113423, 0.846358, 0.113423, 0.846504, 0.133665, 0.846504, 0.133665, 0.846504, 0.133665, 0.846369, 0.152406, 0.846504, 0.133665, 0.846504, 0.133665, 0.846369, 0.152406, 0.846369, 0.152406, 0.846369, 0.152406, 0.845961, 0.168999, 0.846369, 0.152406, 0.846369, 0.152406, 0.845961, 0.168999, 0.845961, 0.168999, 0.845961, 0.168999, 0.845282, 0.183443, 0.845961, 0.168999, 0.845961, 0.168999, 0.845282, 0.183443, 0.845282, 0.183443, 0.845282, 0.183443, 0.844331, 0.195739, 0.845282, 0.183443, 0.845282, 0.183443, 0.844331, 0.195739, 0.844331, 0.195739, 0.844331, 0.195739, 0.843138, 0.205558, 0.844331, 0.195739, 0.844331, 0.195739, 0.843138, 0.205558, 0.843138, 0.205558, 0.843138, 0.205558, 0.841731, 0.212571, 0.843138, 0.205558, 0.843138, 0.205558, 0.841731, 0.212571, 0.841731, 0.212571, 0.841731, 0.212571, 0.840112, 0.216779, 0.841731, 0.212571, 0.841731, 0.212571, 0.840112, 0.216779, 0.840112, 0.216779, 0.840112, 0.216779, 0.838280, 0.218182, 0.840112, 0.216779, 0.840112, 0.216779, 0.838280, 0.218182, 0.838280, 0.218182, 0.838280, 0.218182, 0.834191, 0.218182, 0.838280, 0.218182, 0.838280, 0.218182, 0.834191, 0.218182, 0.834191, 0.218182, 0.834191, 0.218182, 0.834191, 0.263636, 0.834191, 0.218182, 0.834191, 0.218182, 0.834191, 0.263636, 0.834191, 0.263636, 0.834191, 0.263636, 0.838468, 0.263636, 0.834191, 0.263636, 0.834191, 0.263636, 0.838468, 0.263636, 0.838468, 0.263636, 0.838468, 0.263636, 0.840116, 0.264719, 0.838468, 0.263636, 0.838468, 0.263636, 0.840116, 0.264719, 0.840116, 0.264719, 0.840116, 0.264719, 0.841558, 0.267969, 0.840116, 0.264719, 0.840116, 0.264719, 0.841558, 0.267969, 0.841558, 0.267969, 0.841558, 0.267969, 0.842795, 0.273384, 0.841558, 0.267969, 0.841558, 0.267969, 0.842795, 0.273384, 0.842795, 0.273384, 0.842795, 0.273384, 0.843826, 0.280966, 0.842795, 0.273384, 0.842795, 0.273384, 0.843826, 0.280966, 0.843826, 0.280966, 0.843826, 0.280966, 0.844638, 0.290581, 0.843826, 0.280966, 0.843826, 0.280966, 0.844638, 0.290581, 0.844638, 0.290581, 0.844638, 0.290581, 0.845218, 0.302095, 0.844638, 0.290581, 0.844638, 0.290581, 0.845218, 0.302095, 0.845218, 0.302095, 0.845218, 0.302095, 0.845566, 0.315510, 0.845218, 0.302095, 0.845218, 0.302095, 0.845566, 0.315510, 0.845566, 0.315510, 0.845566, 0.315510, 0.845682, 0.330824, 0.845566, 0.315510, 0.845566, 0.315510, 0.845682, 0.330824, 0.845682, 0.330824, 0.845682, 0.330824, 0.845562, 0.346547, 0.845682, 0.330824, 0.845682, 0.330824, 0.845562, 0.346547, 0.845562, 0.346547, 0.845562, 0.346547, 0.845203, 0.360334, 0.845562, 0.346547, 0.845562, 0.346547, 0.845203, 0.360334, 0.845203, 0.360334, 0.845203, 0.360334, 0.844605, 0.372186, 0.845203, 0.360334, 0.845203, 0.360334, 0.844605, 0.372186, 0.844605, 0.372186, 0.844605, 0.372186, 0.843767, 0.382102, 0.844605, 0.372186, 0.844605, 0.372186, 0.843767, 0.382102, 0.843767, 0.382102, 0.843767, 0.382102, 0.842706, 0.389933, 0.843767, 0.382102, 0.843767, 0.382102, 0.842706, 0.389933, 0.842706, 0.389933, 0.842706, 0.389933, 0.841438, 0.395526, 0.842706, 0.389933, 0.842706, 0.389933, 0.841438, 0.395526, 0.841438, 0.395526, 0.841438, 0.395526, 0.839962, 0.398881, 0.841438, 0.395526, 0.841438, 0.395526, 0.839962, 0.398881, 0.839962, 0.398881, 0.839962, 0.398881, 0.838280, 0.400000, 0.839962, 0.398881, 0.839962, 0.398881, 0.838280, 0.400000, 0.838280, 0.400000, 0.838280, 0.400000, 0.837287, 0.399587, 0.838280, 0.400000, 0.838280, 0.400000, 0.837287, 0.399587, 0.837287, 0.399587, 0.837287, 0.399587, 0.836259, 0.398349, 0.837287, 0.399587, 0.837287, 0.399587, 0.836259, 0.398349, 0.836259, 0.398349, 0.836259, 0.398349, 0.835196, 0.396285, 0.836259, 0.398349, 0.836259, 0.398349, 0.835196, 0.396285, 0.835196, 0.396285, 0.835196, 0.396285, 0.834097, 0.393395, 0.835196, 0.396285, 0.835196, 0.396285, 0.834097, 0.393395, 0.834097, 0.393395, 0.834097, 0.393395, 0.832953, 0.389626, 0.834097, 0.393395, 0.834097, 0.393395, 0.832953, 0.389626, 0.832953, 0.389626, 0.832953, 0.389626, 0.831753, 0.384925, 0.832953, 0.389626, 0.832953, 0.389626, 0.831753, 0.384925, 0.831753, 0.384925, 0.831753, 0.384925, 0.830497, 0.379292, 0.831753, 0.384925, 0.831753, 0.384925, 0.830497, 0.379292, 0.830497, 0.379292, 0.830497, 0.379292, 0.829186, 0.372727, 0.830497, 0.379292, 0.830497, 0.379292, 0.829186, 0.372727, 0.829186, 0.372727, 0.829186, 0.372727, 0.829186, 0.427273, 0.829186, 0.372727, 0.829186, 0.372727, 0.829186, 0.427273, 0.829186, 0.427273, 0.829186, 0.427273, 0.830516, 0.431534, 0.829186, 0.427273, 0.829186, 0.427273, 0.830516, 0.431534, 0.830516, 0.431534, 0.830516, 0.431534, 0.831803, 0.435227, 0.830516, 0.431534, 0.830516, 0.431534, 0.831803, 0.435227, 0.831803, 0.435227, 0.831803, 0.435227, 0.833048, 0.438352, 0.831803, 0.435227, 0.831803, 0.435227, 0.833048, 0.438352, 0.833048, 0.438352, 0.833048, 0.438352, 0.834250, 0.440909, 0.833048, 0.438352, 0.833048, 0.438352, 0.834250, 0.440909, 0.834250, 0.440909, 0.834250, 0.440909, 0.835414, 0.442898, 0.834250, 0.440909, 0.834250, 0.440909, 0.835414, 0.442898, 0.835414, 0.442898, 0.835414, 0.442898, 0.836544, 0.444318, 0.835414, 0.442898, 0.835414, 0.442898, 0.836544, 0.444318, 0.836544, 0.444318, 0.836544, 0.444318, 0.837640, 0.445170, 0.836544, 0.444318, 0.836544, 0.444318, 0.837640, 0.445170, 0.837640, 0.445170, 0.837640, 0.445170, 0.838703, 0.445455, 0.837640, 0.445170, 0.837640, 0.445170, 0.838703, 0.445455, 0.838703, 0.445455, 0.838703, 0.445455, 0.841264, 0.443604, 0.838703, 0.445455, 0.838703, 0.445455, 0.841264, 0.443604, 0.841264, 0.443604, 0.841264, 0.443604, 0.843544, 0.438050, 0.841264, 0.443604, 0.841264, 0.443604, 0.843544, 0.438050, 0.843544, 0.438050, 0.843544, 0.438050, 0.845541, 0.428795, 0.843544, 0.438050, 0.843544, 0.438050, 0.845541, 0.428795, 0.845541, 0.428795, 0.845541, 0.428795, 0.847257, 0.415838, 0.845541, 0.428795, 0.845541, 0.428795, 0.847257, 0.415838, 0.847257, 0.415838, 0.847257, 0.415838, 0.848634, 0.399729, 0.847257, 0.415838, 0.847257, 0.415838, 0.848634, 0.399729, 0.848634, 0.399729, 0.848634, 0.399729, 0.849618, 0.381019, 0.848634, 0.399729, 0.848634, 0.399729, 0.849618, 0.381019, 0.849618, 0.381019, 0.849618, 0.381019, 0.850209, 0.359708, 0.849618, 0.381019, 0.849618, 0.381019, 0.850209, 0.359708, 0.850209, 0.359708, 0.850209, 0.359708, 0.850405, 0.335795, 0.850209, 0.359708, 0.850209, 0.359708, 0.850405, 0.335795, 0.850405, 0.335795, 0.850405, 0.335795, 0.850301, 0.318870, 0.850405, 0.335795, 0.850405, 0.335795, 0.850301, 0.318870, 0.850301, 0.318870, 0.850301, 0.318870, 0.849988, 0.303320, 0.850301, 0.318870, 0.850301, 0.318870, 0.849988, 0.303320, 0.849988, 0.303320, 0.849988, 0.303320, 0.849467, 0.289147, 0.849988, 0.303320, 0.849988, 0.303320, 0.849467, 0.289147, 0.849467, 0.289147, 0.849467, 0.289147, 0.848737, 0.276349, 0.849467, 0.289147, 0.849467, 0.289147, 0.848737, 0.276349, 0.848737, 0.276349, 0.848737, 0.276349, 0.847815, 0.265168, 0.848737, 0.276349, 0.848737, 0.276349, 0.847815, 0.265168, 0.847815, 0.265168, 0.847815, 0.265168, 0.846716, 0.255842, 0.847815, 0.265168, 0.847815, 0.265168, 0.846716, 0.255842, 0.846716, 0.255842, 0.846716, 0.255842, 0.845441, 0.248371, 0.846716, 0.255842, 0.846716, 0.255842, 0.845441, 0.248371, 0.845441, 0.248371, 0.828128, 0.072727, 0.828128, 0.018182, 0.829139, 0.066393, 0.828128, 0.018182, 0.828128, 0.072727, 0.829139, 0.066393, 0.829139, 0.066393, 0.828128, 0.018182, 0.830196, 0.060884, 0.828128, 0.018182, 0.829139, 0.066393, 0.830196, 0.060884, 0.828128, 0.018182, 0.829364, 0.013920, 0.830196, 0.060884, 0.829364, 0.013920, 0.828128, 0.018182, 0.830196, 0.060884, 0.830196, 0.060884, 0.829364, 0.013920, 0.831301, 0.056201, 0.829364, 0.013920, 0.830196, 0.060884, 0.831301, 0.056201, 0.829364, 0.013920, 0.830581, 0.010227, 0.831301, 0.056201, 0.830581, 0.010227, 0.829364, 0.013920, 0.831301, 0.056201, 0.831301, 0.056201, 0.830581, 0.010227, 0.832452, 0.052344, 0.830581, 0.010227, 0.831301, 0.056201, 0.832452, 0.052344, 0.830581, 0.010227, 0.831779, 0.007102, 0.832452, 0.052344, 0.831779, 0.007102, 0.830581, 0.010227, 0.832452, 0.052344, 0.832452, 0.052344, 0.831779, 0.007102, 0.833640, 0.049330, 0.831779, 0.007102, 0.832452, 0.052344, 0.833640, 0.049330, 0.831779, 0.007102, 0.832957, 0.004545, 0.833640, 0.049330, 0.832957, 0.004545, 0.831779, 0.007102, 0.833640, 0.049330, 0.833640, 0.049330, 0.832957, 0.004545, 0.834118, 0.002557, 0.832957, 0.004545, 0.833640, 0.049330, 0.834118, 0.002557, 0.833640, 0.049330, 0.834118, 0.002557, 0.834855, 0.047177, 0.834118, 0.002557, 0.833640, 0.049330, 0.834855, 0.047177, 0.834855, 0.047177, 0.834118, 0.002557, 0.835263, 0.001136, 0.834118, 0.002557, 0.834855, 0.047177, 0.835263, 0.001136, 0.834855, 0.047177, 0.835263, 0.001136, 0.836096, 0.045885, 0.835263, 0.001136, 0.834855, 0.047177, 0.836096, 0.045885, 0.836096, 0.045885, 0.835263, 0.001136, 0.836392, 0.000284, 0.835263, 0.001136, 0.836096, 0.045885, 0.836392, 0.000284, 0.836096, 0.045885, 0.836392, 0.000284, 0.837363, 0.045455, 0.836392, 0.000284, 0.836096, 0.045885, 0.837363, 0.045455, 0.837363, 0.045455, 0.836392, 0.000284, 0.837504, 0, 0.836392, 0.000284, 0.837363, 0.045455, 0.837504, 0, 0.839463, 0.046866, 0.837363, 0.045455, 0.837504, 0, 0.837363, 0.045455, 0.839463, 0.046866, 0.837504, 0, 0.840606, 0.002202, 0.839463, 0.046866, 0.837504, 0, 0.839463, 0.046866, 0.840606, 0.002202, 0.837504, 0, 0.839463, 0.046866, 0.840606, 0.002202, 0.841297, 0.051101, 0.840606, 0.002202, 0.839463, 0.046866, 0.841297, 0.051101, 0.841297, 0.051101, 0.840606, 0.002202, 0.843332, 0.008807, 0.840606, 0.002202, 0.841297, 0.051101, 0.843332, 0.008807, 0.842864, 0.058159, 0.841297, 0.051101, 0.843332, 0.008807, 0.841297, 0.051101, 0.842864, 0.058159, 0.843332, 0.008807, 0.842864, 0.058159, 0.843332, 0.008807, 0.845682, 0.019815, 0.843332, 0.008807, 0.842864, 0.058159, 0.845682, 0.019815, 0.844166, 0.068040, 0.842864, 0.058159, 0.845682, 0.019815, 0.842864, 0.058159, 0.844166, 0.068040, 0.845682, 0.019815, 0.844166, 0.068040, 0.845682, 0.019815, 0.847656, 0.035227, 0.845682, 0.019815, 0.844166, 0.068040, 0.847656, 0.035227, 0.845189, 0.080611, 0.844166, 0.068040, 0.847656, 0.035227, 0.844166, 0.068040, 0.845189, 0.080611, 0.847656, 0.035227, 0.845189, 0.080611, 0.847656, 0.035227, 0.849219, 0.054661, 0.847656, 0.035227, 0.845189, 0.080611, 0.849219, 0.054661, 0.845920, 0.095739, 0.845189, 0.080611, 0.849219, 0.054661, 0.845189, 0.080611, 0.845920, 0.095739, 0.849219, 0.054661, 0.845920, 0.095739, 0.849219, 0.054661, 0.850335, 0.077734, 0.849219, 0.054661, 0.845920, 0.095739, 0.850335, 0.077734, 0.846358, 0.113423, 0.845920, 0.095739, 0.850335, 0.077734, 0.845920, 0.095739, 0.846358, 0.113423, 0.850335, 0.077734, 0.851005, 0.104448, 0.846358, 0.113423, 0.850335, 0.077734, 0.846358, 0.113423, 0.851005, 0.104448, 0.850335, 0.077734, 0.846504, 0.133665, 0.846358, 0.113423, 0.851005, 0.104448, 0.846358, 0.113423, 0.846504, 0.133665, 0.851005, 0.104448, 0.846504, 0.133665, 0.851005, 0.104448, 0.851228, 0.134801, 0.851005, 0.104448, 0.846504, 0.133665, 0.851228, 0.134801, 0.851108, 0.154914, 0.846504, 0.133665, 0.851228, 0.134801, 0.846504, 0.133665, 0.851108, 0.154914, 0.851228, 0.134801, 0.846369, 0.152406, 0.846504, 0.133665, 0.851108, 0.154914, 0.846504, 0.133665, 0.846369, 0.152406, 0.851108, 0.154914, 0.850749, 0.173349, 0.846369, 0.152406, 0.851108, 0.154914, 0.846369, 0.152406, 0.850749, 0.173349, 0.851108, 0.154914, 0.845961, 0.168999, 0.846369, 0.152406, 0.850749, 0.173349, 0.846369, 0.152406, 0.845961, 0.168999, 0.850749, 0.173349, 0.850150, 0.190106, 0.845961, 0.168999, 0.850749, 0.173349, 0.845961, 0.168999, 0.850150, 0.190106, 0.850749, 0.173349, 0.849313, 0.205185, 0.845961, 0.168999, 0.850150, 0.190106, 0.845961, 0.168999, 0.849313, 0.205185, 0.850150, 0.190106, 0.845282, 0.183443, 0.845961, 0.168999, 0.849313, 0.205185, 0.845961, 0.168999, 0.845282, 0.183443, 0.849313, 0.205185, 0.848262, 0.218213, 0.845282, 0.183443, 0.849313, 0.205185, 0.845282, 0.183443, 0.848262, 0.218213, 0.849313, 0.205185, 0.848262, 0.218213, 0.847024, 0.228817, 0.845282, 0.183443, 0.847024, 0.228817, 0.848262, 0.218213, 0.845282, 0.183443, 0.847024, 0.228817, 0.844331, 0.195739, 0.845282, 0.183443, 0.844331, 0.195739, 0.847024, 0.228817, 0.845282, 0.183443, 0.845601, 0.236998, 0.844331, 0.195739, 0.847024, 0.228817, 0.844331, 0.195739, 0.845601, 0.236998, 0.847024, 0.228817, 0.843138, 0.205558, 0.844331, 0.195739, 0.845601, 0.236998, 0.844331, 0.195739, 0.843138, 0.205558, 0.845601, 0.236998, 0.843990, 0.242756, 0.843138, 0.205558, 0.845601, 0.236998, 0.843138, 0.205558, 0.843990, 0.242756, 0.845601, 0.236998, 0.841731, 0.212571, 0.843138, 0.205558, 0.843990, 0.242756, 0.843138, 0.205558, 0.841731, 0.212571, 0.843990, 0.242756, 0.843990, 0.242756, 0.841558, 0.267969, 0.841731, 0.212571, 0.841558, 0.267969, 0.843990, 0.242756, 0.841731, 0.212571, 0.842795, 0.273384, 0.841558, 0.267969, 0.843990, 0.242756, 0.841558, 0.267969, 0.842795, 0.273384, 0.843990, 0.242756, 0.843826, 0.280966, 0.842795, 0.273384, 0.843990, 0.242756, 0.842795, 0.273384, 0.843826, 0.280966, 0.843990, 0.242756, 0.843826, 0.280966, 0.843990, 0.242756, 0.845441, 0.248371, 0.843990, 0.242756, 0.843826, 0.280966, 0.845441, 0.248371, 0.846716, 0.255842, 0.843826, 0.280966, 0.845441, 0.248371, 0.843826, 0.280966, 0.846716, 0.255842, 0.845441, 0.248371, 0.846716, 0.255842, 0.844638, 0.290581, 0.843826, 0.280966, 0.844638, 0.290581, 0.846716, 0.255842, 0.843826, 0.280966, 0.844638, 0.290581, 0.846716, 0.255842, 0.847815, 0.265168, 0.846716, 0.255842, 0.844638, 0.290581, 0.847815, 0.265168, 0.845218, 0.302095, 0.844638, 0.290581, 0.847815, 0.265168, 0.844638, 0.290581, 0.845218, 0.302095, 0.847815, 0.265168, 0.845218, 0.302095, 0.847815, 0.265168, 0.848737, 0.276349, 0.847815, 0.265168, 0.845218, 0.302095, 0.848737, 0.276349, 0.849467, 0.289147, 0.845218, 0.302095, 0.848737, 0.276349, 0.845218, 0.302095, 0.849467, 0.289147, 0.848737, 0.276349, 0.849467, 0.289147, 0.845566, 0.315510, 0.845218, 0.302095, 0.845566, 0.315510, 0.849467, 0.289147, 0.845218, 0.302095, 0.845566, 0.315510, 0.849467, 0.289147, 0.849988, 0.303320, 0.849467, 0.289147, 0.845566, 0.315510, 0.849988, 0.303320, 0.850301, 0.318870, 0.845566, 0.315510, 0.849988, 0.303320, 0.845566, 0.315510, 0.850301, 0.318870, 0.849988, 0.303320, 0.850301, 0.318870, 0.845682, 0.330824, 0.845566, 0.315510, 0.845682, 0.330824, 0.850301, 0.318870, 0.845566, 0.315510, 0.845682, 0.330824, 0.850301, 0.318870, 0.850405, 0.335795, 0.850301, 0.318870, 0.845682, 0.330824, 0.850405, 0.335795, 0.850405, 0.335795, 0.845562, 0.346547, 0.845682, 0.330824, 0.845562, 0.346547, 0.850405, 0.335795, 0.845682, 0.330824, 0.850209, 0.359708, 0.845562, 0.346547, 0.850405, 0.335795, 0.845562, 0.346547, 0.850209, 0.359708, 0.850405, 0.335795, 0.849618, 0.381019, 0.845562, 0.346547, 0.850209, 0.359708, 0.845562, 0.346547, 0.849618, 0.381019, 0.850209, 0.359708, 0.845203, 0.360334, 0.845562, 0.346547, 0.849618, 0.381019, 0.845562, 0.346547, 0.845203, 0.360334, 0.849618, 0.381019, 0.848634, 0.399729, 0.845203, 0.360334, 0.849618, 0.381019, 0.845203, 0.360334, 0.848634, 0.399729, 0.849618, 0.381019, 0.844605, 0.372186, 0.845203, 0.360334, 0.848634, 0.399729, 0.845203, 0.360334, 0.844605, 0.372186, 0.848634, 0.399729, 0.847257, 0.415838, 0.844605, 0.372186, 0.848634, 0.399729, 0.844605, 0.372186, 0.847257, 0.415838, 0.848634, 0.399729, 0.843767, 0.382102, 0.844605, 0.372186, 0.847257, 0.415838, 0.844605, 0.372186, 0.843767, 0.382102, 0.847257, 0.415838, 0.845541, 0.428795, 0.843767, 0.382102, 0.847257, 0.415838, 0.843767, 0.382102, 0.845541, 0.428795, 0.847257, 0.415838, 0.842706, 0.389933, 0.843767, 0.382102, 0.845541, 0.428795, 0.843767, 0.382102, 0.842706, 0.389933, 0.845541, 0.428795, 0.843544, 0.438050, 0.842706, 0.389933, 0.845541, 0.428795, 0.842706, 0.389933, 0.843544, 0.438050, 0.845541, 0.428795, 0.841438, 0.395526, 0.842706, 0.389933, 0.843544, 0.438050, 0.842706, 0.389933, 0.841438, 0.395526, 0.843544, 0.438050, 0.841264, 0.443604, 0.841438, 0.395526, 0.843544, 0.438050, 0.841438, 0.395526, 0.841264, 0.443604, 0.843544, 0.438050, 0.841264, 0.443604, 0.839962, 0.398881, 0.841438, 0.395526, 0.839962, 0.398881, 0.841264, 0.443604, 0.841438, 0.395526, 0.841264, 0.443604, 0.838703, 0.445455, 0.839962, 0.398881, 0.838703, 0.445455, 0.841264, 0.443604, 0.839962, 0.398881, 0.838280, 0.400000, 0.839962, 0.398881, 0.838703, 0.445455, 0.839962, 0.398881, 0.838280, 0.400000, 0.838703, 0.445455, 0.838703, 0.445455, 0.837640, 0.445170, 0.838280, 0.400000, 0.837640, 0.445170, 0.838703, 0.445455, 0.838280, 0.400000, 0.837640, 0.445170, 0.837287, 0.399587, 0.838280, 0.400000, 0.837287, 0.399587, 0.837640, 0.445170, 0.838280, 0.400000, 0.836544, 0.444318, 0.837287, 0.399587, 0.837640, 0.445170, 0.837287, 0.399587, 0.836544, 0.444318, 0.837640, 0.445170, 0.836544, 0.444318, 0.836259, 0.398349, 0.837287, 0.399587, 0.836259, 0.398349, 0.836544, 0.444318, 0.837287, 0.399587, 0.835414, 0.442898, 0.836259, 0.398349, 0.836544, 0.444318, 0.836259, 0.398349, 0.835414, 0.442898, 0.836544, 0.444318, 0.835414, 0.442898, 0.835196, 0.396285, 0.836259, 0.398349, 0.835196, 0.396285, 0.835414, 0.442898, 0.836259, 0.398349, 0.834250, 0.440909, 0.835196, 0.396285, 0.835414, 0.442898, 0.835196, 0.396285, 0.834250, 0.440909, 0.835414, 0.442898, 0.834250, 0.440909, 0.834097, 0.393395, 0.835196, 0.396285, 0.834097, 0.393395, 0.834250, 0.440909, 0.835196, 0.396285, 0.833048, 0.438352, 0.834097, 0.393395, 0.834250, 0.440909, 0.834097, 0.393395, 0.833048, 0.438352, 0.834250, 0.440909, 0.833048, 0.438352, 0.832953, 0.389626, 0.834097, 0.393395, 0.832953, 0.389626, 0.833048, 0.438352, 0.834097, 0.393395, 0.831803, 0.435227, 0.832953, 0.389626, 0.833048, 0.438352, 0.832953, 0.389626, 0.831803, 0.435227, 0.833048, 0.438352, 0.832953, 0.389626, 0.831803, 0.435227, 0.831753, 0.384925, 0.831803, 0.435227, 0.832953, 0.389626, 0.831753, 0.384925, 0.831803, 0.435227, 0.830516, 0.431534, 0.831753, 0.384925, 0.830516, 0.431534, 0.831803, 0.435227, 0.831753, 0.384925, 0.831753, 0.384925, 0.830516, 0.431534, 0.830497, 0.379292, 0.830516, 0.431534, 0.831753, 0.384925, 0.830497, 0.379292, 0.830516, 0.431534, 0.829186, 0.427273, 0.830497, 0.379292, 0.829186, 0.427273, 0.830516, 0.431534, 0.830497, 0.379292, 0.829186, 0.427273, 0.829186, 0.372727, 0.830497, 0.379292, 0.829186, 0.372727, 0.829186, 0.427273, 0.830497, 0.379292, 0.841558, 0.267969, 0.840112, 0.216779, 0.841731, 0.212571, 0.840112, 0.216779, 0.841558, 0.267969, 0.841731, 0.212571, 0.840112, 0.216779, 0.841558, 0.267969, 0.840116, 0.264719, 0.841558, 0.267969, 0.840112, 0.216779, 0.840116, 0.264719, 0.840112, 0.216779, 0.840116, 0.264719, 0.838468, 0.263636, 0.840116, 0.264719, 0.840112, 0.216779, 0.838468, 0.263636, 0.838468, 0.263636, 0.838280, 0.218182, 0.840112, 0.216779, 0.838280, 0.218182, 0.838468, 0.263636, 0.840112, 0.216779, 0.834191, 0.263636, 0.838280, 0.218182, 0.838468, 0.263636, 0.838280, 0.218182, 0.834191, 0.263636, 0.838468, 0.263636, 0.834191, 0.263636, 0.834191, 0.218182, 0.838280, 0.218182, 0.834191, 0.218182, 0.834191, 0.263636, 0.838280, 0.218182, 0.864528, 0.054545, 0.881095, 0.054545, 0.864528, 0.054545, 0.864528, 0.054545, 0.881095, 0.054545, 0.881095, 0.054545, 0.881095, 0.054545, 0.881095, 0.009091, 0.881095, 0.054545, 0.881095, 0.054545, 0.881095, 0.009091, 0.881095, 0.009091, 0.881095, 0.009091, 0.858818, 0.009091, 0.881095, 0.009091, 0.881095, 0.009091, 0.858818, 0.009091, 0.858818, 0.009091, 0.858818, 0.009091, 0.858818, 0.054545, 0.858818, 0.009091, 0.858818, 0.009091, 0.858818, 0.054545, 0.858818, 0.054545, 0.858818, 0.054545, 0.860292, 0.073442, 0.858818, 0.054545, 0.858818, 0.054545, 0.860292, 0.073442, 0.860292, 0.073442, 0.860292, 0.073442, 0.862011, 0.095330, 0.860292, 0.073442, 0.860292, 0.073442, 0.862011, 0.095330, 0.862011, 0.095330, 0.862011, 0.095330, 0.863975, 0.120210, 0.862011, 0.095330, 0.862011, 0.095330, 0.863975, 0.120210, 0.863975, 0.120210, 0.863975, 0.120210, 0.866185, 0.148082, 0.863975, 0.120210, 0.863975, 0.120210, 0.866185, 0.148082, 0.866185, 0.148082, 0.866185, 0.148082, 0.868301, 0.174836, 0.866185, 0.148082, 0.866185, 0.148082, 0.868301, 0.174836, 0.868301, 0.174836, 0.868301, 0.174836, 0.869983, 0.196360, 0.868301, 0.174836, 0.868301, 0.174836, 0.869983, 0.196360, 0.869983, 0.196360, 0.869983, 0.196360, 0.871232, 0.212655, 0.869983, 0.196360, 0.869983, 0.196360, 0.871232, 0.212655, 0.871232, 0.212655, 0.871232, 0.212655, 0.872048, 0.223722, 0.871232, 0.212655, 0.871232, 0.212655, 0.872048, 0.223722, 0.872048, 0.223722, 0.872048, 0.223722, 0.873102, 0.239085, 0.872048, 0.223722, 0.872048, 0.223722, 0.873102, 0.239085, 0.873102, 0.239085, 0.873102, 0.239085, 0.873984, 0.253214, 0.873102, 0.239085, 0.873102, 0.239085, 0.873984, 0.253214, 0.873984, 0.253214, 0.873984, 0.253214, 0.874694, 0.266109, 0.873984, 0.253214, 0.873984, 0.253214, 0.874694, 0.266109, 0.874694, 0.266109, 0.874694, 0.266109, 0.875232, 0.277770, 0.874694, 0.266109, 0.874694, 0.266109, 0.875232, 0.277770, 0.875232, 0.277770, 0.875232, 0.277770, 0.875628, 0.288774, 0.875232, 0.277770, 0.875232, 0.277770, 0.875628, 0.288774, 0.875628, 0.288774, 0.875628, 0.288774, 0.875911, 0.299698, 0.875628, 0.288774, 0.875628, 0.288774, 0.875911, 0.299698, 0.875911, 0.299698, 0.875911, 0.299698, 0.876080, 0.310542, 0.875911, 0.299698, 0.875911, 0.299698, 0.876080, 0.310542, 0.876080, 0.310542, 0.876080, 0.310542, 0.876137, 0.321307, 0.876080, 0.310542, 0.876080, 0.310542, 0.876137, 0.321307, 0.876137, 0.321307, 0.876137, 0.321307, 0.876014, 0.337904, 0.876137, 0.321307, 0.876137, 0.321307, 0.876014, 0.337904, 0.876014, 0.337904, 0.876014, 0.337904, 0.875646, 0.352894, 0.876014, 0.337904, 0.876014, 0.337904, 0.875646, 0.352894, 0.875646, 0.352894, 0.875646, 0.352894, 0.875033, 0.366278, 0.875646, 0.352894, 0.875646, 0.352894, 0.875033, 0.366278, 0.875033, 0.366278, 0.875033, 0.366278, 0.874175, 0.378054, 0.875033, 0.366278, 0.875033, 0.366278, 0.874175, 0.378054, 0.874175, 0.378054, 0.874175, 0.378054, 0.873119, 0.387655, 0.874175, 0.378054, 0.874175, 0.378054, 0.873119, 0.387655, 0.873119, 0.387655, 0.873119, 0.387655, 0.871916, 0.394513, 0.873119, 0.387655, 0.873119, 0.387655, 0.871916, 0.394513, 0.871916, 0.394513, 0.871916, 0.394513, 0.870564, 0.398628, 0.871916, 0.394513, 0.871916, 0.394513, 0.870564, 0.398628, 0.870564, 0.398628, 0.870564, 0.398628, 0.869063, 0.400000, 0.870564, 0.398628, 0.870564, 0.398628, 0.869063, 0.400000, 0.869063, 0.400000, 0.869063, 0.400000, 0.867932, 0.399294, 0.869063, 0.400000, 0.869063, 0.400000, 0.867932, 0.399294, 0.867932, 0.399294, 0.867932, 0.399294, 0.866769, 0.397177, 0.867932, 0.399294, 0.867932, 0.399294, 0.866769, 0.397177, 0.866769, 0.397177, 0.866769, 0.397177, 0.865576, 0.393648, 0.866769, 0.397177, 0.866769, 0.397177, 0.865576, 0.393648, 0.865576, 0.393648, 0.865576, 0.393648, 0.864352, 0.388707, 0.865576, 0.393648, 0.865576, 0.393648, 0.864352, 0.388707, 0.864352, 0.388707, 0.864352, 0.388707, 0.863091, 0.382338, 0.864352, 0.388707, 0.864352, 0.388707, 0.863091, 0.382338, 0.863091, 0.382338, 0.863091, 0.382338, 0.861788, 0.374521, 0.863091, 0.382338, 0.863091, 0.382338, 0.861788, 0.374521, 0.861788, 0.374521, 0.861788, 0.374521, 0.860442, 0.365257, 0.861788, 0.374521, 0.861788, 0.374521, 0.860442, 0.365257, 0.860442, 0.365257, 0.860442, 0.365257, 0.859053, 0.354545, 0.860442, 0.365257, 0.860442, 0.365257, 0.859053, 0.354545, 0.859053, 0.354545, 0.859053, 0.354545, 0.859053, 0.418182, 0.859053, 0.354545, 0.859053, 0.354545, 0.859053, 0.418182, 0.859053, 0.418182, 0.859053, 0.418182, 0.860463, 0.424516, 0.859053, 0.418182, 0.859053, 0.418182, 0.860463, 0.424516, 0.860463, 0.424516, 0.860463, 0.424516, 0.861826, 0.430025, 0.860463, 0.424516, 0.860463, 0.424516, 0.861826, 0.430025, 0.861826, 0.430025, 0.861826, 0.430025, 0.863142, 0.434708, 0.861826, 0.430025, 0.861826, 0.430025, 0.863142, 0.434708, 0.863142, 0.434708, 0.863142, 0.434708, 0.864411, 0.438565, 0.863142, 0.434708, 0.863142, 0.434708, 0.864411, 0.438565, 0.864411, 0.438565, 0.864411, 0.438565, 0.865630, 0.441579, 0.864411, 0.438565, 0.864411, 0.438565, 0.865630, 0.441579, 0.865630, 0.441579, 0.865630, 0.441579, 0.866796, 0.443732, 0.865630, 0.441579, 0.865630, 0.441579, 0.866796, 0.443732, 0.866796, 0.443732, 0.866796, 0.443732, 0.867909, 0.445024, 0.866796, 0.443732, 0.866796, 0.443732, 0.867909, 0.445024, 0.867909, 0.445024, 0.867909, 0.445024, 0.868970, 0.445455, 0.867909, 0.445024, 0.867909, 0.445024, 0.868970, 0.445455, 0.868970, 0.445455, 0.868970, 0.445455, 0.871557, 0.443355, 0.868970, 0.445455, 0.868970, 0.445455, 0.871557, 0.443355, 0.871557, 0.443355, 0.871557, 0.443355, 0.873869, 0.437056, 0.871557, 0.443355, 0.871557, 0.443355, 0.873869, 0.437056, 0.873869, 0.437056, 0.873869, 0.437056, 0.875905, 0.426558, 0.873869, 0.437056, 0.873869, 0.437056, 0.875905, 0.426558, 0.875905, 0.426558, 0.875905, 0.426558, 0.877664, 0.411861, 0.875905, 0.426558, 0.875905, 0.426558, 0.877664, 0.411861, 0.877664, 0.411861, 0.877664, 0.411861, 0.879083, 0.393657, 0.877664, 0.411861, 0.877664, 0.411861, 0.879083, 0.393657, 0.879083, 0.393657, 0.879083, 0.393657, 0.880096, 0.372638, 0.879083, 0.393657, 0.879083, 0.393657, 0.880096, 0.372638, 0.880096, 0.372638, 0.880096, 0.372638, 0.880704, 0.348806, 0.880096, 0.372638, 0.880096, 0.372638, 0.880704, 0.348806, 0.880704, 0.348806, 0.880704, 0.348806, 0.880907, 0.322159, 0.880704, 0.348806, 0.880704, 0.348806, 0.880907, 0.322159, 0.880907, 0.322159, 0.880907, 0.322159, 0.880856, 0.308984, 0.880907, 0.322159, 0.880907, 0.322159, 0.880856, 0.308984, 0.880856, 0.308984, 0.880856, 0.308984, 0.880704, 0.296165, 0.880856, 0.308984, 0.880856, 0.308984, 0.880704, 0.296165, 0.880704, 0.296165, 0.880704, 0.296165, 0.880451, 0.283700, 0.880704, 0.296165, 0.880704, 0.296165, 0.880451, 0.283700, 0.880451, 0.283700, 0.880451, 0.283700, 0.880096, 0.271591, 0.880451, 0.283700, 0.880451, 0.283700, 0.880096, 0.271591, 0.880096, 0.271591, 0.880096, 0.271591, 0.879608, 0.259118, 0.880096, 0.271591, 0.880096, 0.271591, 0.879608, 0.259118, 0.879608, 0.259118, 0.879608, 0.259118, 0.878954, 0.245561, 0.879608, 0.259118, 0.879608, 0.259118, 0.878954, 0.245561, 0.878954, 0.245561, 0.878954, 0.245561, 0.878133, 0.230922, 0.878954, 0.245561, 0.878954, 0.245561, 0.878133, 0.230922, 0.878133, 0.230922, 0.878133, 0.230922, 0.877147, 0.215199, 0.878133, 0.230922, 0.878133, 0.230922, 0.877147, 0.215199, 0.877147, 0.215199, 0.877147, 0.215199, 0.876693, 0.209029, 0.877147, 0.215199, 0.877147, 0.215199, 0.876693, 0.209029, 0.876693, 0.209029, 0.876693, 0.209029, 0.875919, 0.198899, 0.876693, 0.209029, 0.876693, 0.209029, 0.875919, 0.198899, 0.875919, 0.198899, 0.875919, 0.198899, 0.874825, 0.184810, 0.875919, 0.198899, 0.875919, 0.198899, 0.874825, 0.184810, 0.874825, 0.184810, 0.874825, 0.184810, 0.873411, 0.166761, 0.874825, 0.184810, 0.874825, 0.184810, 0.873411, 0.166761, 0.873411, 0.166761, 0.873411, 0.166761, 0.871675, 0.144727, 0.873411, 0.166761, 0.873411, 0.166761, 0.871675, 0.144727, 0.871675, 0.144727, 0.871675, 0.144727, 0.869616, 0.118679, 0.871675, 0.144727, 0.871675, 0.144727, 0.869616, 0.118679, 0.869616, 0.118679, 0.869616, 0.118679, 0.867234, 0.088619, 0.869616, 0.118679, 0.869616, 0.118679, 0.867234, 0.088619, 0.867234, 0.088619, 0.860292, 0.073442, 0.858818, 0.054545, 0.864528, 0.054545, 0.858818, 0.054545, 0.860292, 0.073442, 0.864528, 0.054545, 0.862011, 0.095330, 0.860292, 0.073442, 0.864528, 0.054545, 0.860292, 0.073442, 0.862011, 0.095330, 0.864528, 0.054545, 0.862011, 0.095330, 0.864528, 0.054545, 0.867234, 0.088619, 0.864528, 0.054545, 0.862011, 0.095330, 0.867234, 0.088619, 0.863975, 0.120210, 0.862011, 0.095330, 0.867234, 0.088619, 0.862011, 0.095330, 0.863975, 0.120210, 0.867234, 0.088619, 0.866185, 0.148082, 0.863975, 0.120210, 0.867234, 0.088619, 0.863975, 0.120210, 0.866185, 0.148082, 0.867234, 0.088619, 0.866185, 0.148082, 0.867234, 0.088619, 0.869616, 0.118679, 0.867234, 0.088619, 0.866185, 0.148082, 0.869616, 0.118679, 0.868301, 0.174836, 0.866185, 0.148082, 0.869616, 0.118679, 0.866185, 0.148082, 0.868301, 0.174836, 0.869616, 0.118679, 0.868301, 0.174836, 0.869616, 0.118679, 0.871675, 0.144727, 0.869616, 0.118679, 0.868301, 0.174836, 0.871675, 0.144727, 0.869983, 0.196360, 0.868301, 0.174836, 0.871675, 0.144727, 0.868301, 0.174836, 0.869983, 0.196360, 0.871675, 0.144727, 0.869983, 0.196360, 0.871675, 0.144727, 0.873411, 0.166761, 0.871675, 0.144727, 0.869983, 0.196360, 0.873411, 0.166761, 0.871232, 0.212655, 0.869983, 0.196360, 0.873411, 0.166761, 0.869983, 0.196360, 0.871232, 0.212655, 0.873411, 0.166761, 0.871232, 0.212655, 0.873411, 0.166761, 0.874825, 0.184810, 0.873411, 0.166761, 0.871232, 0.212655, 0.874825, 0.184810, 0.872048, 0.223722, 0.871232, 0.212655, 0.874825, 0.184810, 0.871232, 0.212655, 0.872048, 0.223722, 0.874825, 0.184810, 0.872048, 0.223722, 0.874825, 0.184810, 0.875919, 0.198899, 0.874825, 0.184810, 0.872048, 0.223722, 0.875919, 0.198899, 0.873102, 0.239085, 0.872048, 0.223722, 0.875919, 0.198899, 0.872048, 0.223722, 0.873102, 0.239085, 0.875919, 0.198899, 0.873102, 0.239085, 0.875919, 0.198899, 0.876693, 0.209029, 0.875919, 0.198899, 0.873102, 0.239085, 0.876693, 0.209029, 0.877147, 0.215199, 0.873102, 0.239085, 0.876693, 0.209029, 0.873102, 0.239085, 0.877147, 0.215199, 0.876693, 0.209029, 0.877147, 0.215199, 0.873984, 0.253214, 0.873102, 0.239085, 0.873984, 0.253214, 0.877147, 0.215199, 0.873102, 0.239085, 0.873984, 0.253214, 0.877147, 0.215199, 0.878133, 0.230922, 0.877147, 0.215199, 0.873984, 0.253214, 0.878133, 0.230922, 0.874694, 0.266109, 0.873984, 0.253214, 0.878133, 0.230922, 0.873984, 0.253214, 0.874694, 0.266109, 0.878133, 0.230922, 0.874694, 0.266109, 0.878133, 0.230922, 0.878954, 0.245561, 0.878133, 0.230922, 0.874694, 0.266109, 0.878954, 0.245561, 0.875232, 0.277770, 0.874694, 0.266109, 0.878954, 0.245561, 0.874694, 0.266109, 0.875232, 0.277770, 0.878954, 0.245561, 0.875232, 0.277770, 0.878954, 0.245561, 0.879608, 0.259118, 0.878954, 0.245561, 0.875232, 0.277770, 0.879608, 0.259118, 0.875628, 0.288774, 0.875232, 0.277770, 0.879608, 0.259118, 0.875232, 0.277770, 0.875628, 0.288774, 0.879608, 0.259118, 0.875628, 0.288774, 0.879608, 0.259118, 0.880096, 0.271591, 0.879608, 0.259118, 0.875628, 0.288774, 0.880096, 0.271591, 0.875911, 0.299698, 0.875628, 0.288774, 0.880096, 0.271591, 0.875628, 0.288774, 0.875911, 0.299698, 0.880096, 0.271591, 0.875911, 0.299698, 0.880096, 0.271591, 0.880451, 0.283700, 0.880096, 0.271591, 0.875911, 0.299698, 0.880451, 0.283700, 0.875911, 0.299698, 0.880451, 0.283700, 0.880704, 0.296165, 0.880451, 0.283700, 0.875911, 0.299698, 0.880704, 0.296165, 0.876080, 0.310542, 0.875911, 0.299698, 0.880704, 0.296165, 0.875911, 0.299698, 0.876080, 0.310542, 0.880704, 0.296165, 0.876080, 0.310542, 0.880704, 0.296165, 0.880856, 0.308984, 0.880704, 0.296165, 0.876080, 0.310542, 0.880856, 0.308984, 0.876137, 0.321307, 0.876080, 0.310542, 0.880856, 0.308984, 0.876080, 0.310542, 0.876137, 0.321307, 0.880856, 0.308984, 0.876137, 0.321307, 0.880856, 0.308984, 0.880907, 0.322159, 0.880856, 0.308984, 0.876137, 0.321307, 0.880907, 0.322159, 0.880907, 0.322159, 0.876014, 0.337904, 0.876137, 0.321307, 0.876014, 0.337904, 0.880907, 0.322159, 0.876137, 0.321307, 0.880704, 0.348806, 0.876014, 0.337904, 0.880907, 0.322159, 0.876014, 0.337904, 0.880704, 0.348806, 0.880907, 0.322159, 0.880096, 0.372638, 0.876014, 0.337904, 0.880704, 0.348806, 0.876014, 0.337904, 0.880096, 0.372638, 0.880704, 0.348806, 0.875646, 0.352894, 0.876014, 0.337904, 0.880096, 0.372638, 0.876014, 0.337904, 0.875646, 0.352894, 0.880096, 0.372638, 0.879083, 0.393657, 0.875646, 0.352894, 0.880096, 0.372638, 0.875646, 0.352894, 0.879083, 0.393657, 0.880096, 0.372638, 0.875033, 0.366278, 0.875646, 0.352894, 0.879083, 0.393657, 0.875646, 0.352894, 0.875033, 0.366278, 0.879083, 0.393657, 0.877664, 0.411861, 0.875033, 0.366278, 0.879083, 0.393657, 0.875033, 0.366278, 0.877664, 0.411861, 0.879083, 0.393657, 0.874175, 0.378054, 0.875033, 0.366278, 0.877664, 0.411861, 0.875033, 0.366278, 0.874175, 0.378054, 0.877664, 0.411861, 0.875905, 0.426558, 0.874175, 0.378054, 0.877664, 0.411861, 0.874175, 0.378054, 0.875905, 0.426558, 0.877664, 0.411861, 0.873119, 0.387655, 0.874175, 0.378054, 0.875905, 0.426558, 0.874175, 0.378054, 0.873119, 0.387655, 0.875905, 0.426558, 0.873869, 0.437056, 0.873119, 0.387655, 0.875905, 0.426558, 0.873119, 0.387655, 0.873869, 0.437056, 0.875905, 0.426558, 0.871916, 0.394513, 0.873119, 0.387655, 0.873869, 0.437056, 0.873119, 0.387655, 0.871916, 0.394513, 0.873869, 0.437056, 0.871557, 0.443355, 0.871916, 0.394513, 0.873869, 0.437056, 0.871916, 0.394513, 0.871557, 0.443355, 0.873869, 0.437056, 0.871557, 0.443355, 0.870564, 0.398628, 0.871916, 0.394513, 0.870564, 0.398628, 0.871557, 0.443355, 0.871916, 0.394513, 0.870564, 0.398628, 0.871557, 0.443355, 0.868970, 0.445455, 0.871557, 0.443355, 0.870564, 0.398628, 0.868970, 0.445455, 0.868970, 0.445455, 0.869063, 0.400000, 0.870564, 0.398628, 0.869063, 0.400000, 0.868970, 0.445455, 0.870564, 0.398628, 0.868970, 0.445455, 0.867909, 0.445024, 0.869063, 0.400000, 0.867909, 0.445024, 0.868970, 0.445455, 0.869063, 0.400000, 0.867909, 0.445024, 0.867932, 0.399294, 0.869063, 0.400000, 0.867932, 0.399294, 0.867909, 0.445024, 0.869063, 0.400000, 0.867909, 0.445024, 0.866796, 0.443732, 0.867932, 0.399294, 0.866796, 0.443732, 0.867909, 0.445024, 0.867932, 0.399294, 0.866796, 0.443732, 0.866769, 0.397177, 0.867932, 0.399294, 0.866769, 0.397177, 0.866796, 0.443732, 0.867932, 0.399294, 0.865630, 0.441579, 0.866769, 0.397177, 0.866796, 0.443732, 0.866769, 0.397177, 0.865630, 0.441579, 0.866796, 0.443732, 0.865630, 0.441579, 0.865576, 0.393648, 0.866769, 0.397177, 0.865576, 0.393648, 0.865630, 0.441579, 0.866769, 0.397177, 0.864411, 0.438565, 0.865576, 0.393648, 0.865630, 0.441579, 0.865576, 0.393648, 0.864411, 0.438565, 0.865630, 0.441579, 0.865576, 0.393648, 0.864411, 0.438565, 0.864352, 0.388707, 0.864411, 0.438565, 0.865576, 0.393648, 0.864352, 0.388707, 0.864411, 0.438565, 0.863142, 0.434708, 0.864352, 0.388707, 0.863142, 0.434708, 0.864411, 0.438565, 0.864352, 0.388707, 0.863142, 0.434708, 0.861826, 0.430025, 0.864352, 0.388707, 0.861826, 0.430025, 0.863142, 0.434708, 0.864352, 0.388707, 0.861826, 0.430025, 0.863091, 0.382338, 0.864352, 0.388707, 0.863091, 0.382338, 0.861826, 0.430025, 0.864352, 0.388707, 0.861826, 0.430025, 0.860463, 0.424516, 0.863091, 0.382338, 0.860463, 0.424516, 0.861826, 0.430025, 0.863091, 0.382338, 0.860463, 0.424516, 0.861788, 0.374521, 0.863091, 0.382338, 0.861788, 0.374521, 0.860463, 0.424516, 0.863091, 0.382338, 0.860463, 0.424516, 0.859053, 0.418182, 0.861788, 0.374521, 0.859053, 0.418182, 0.860463, 0.424516, 0.861788, 0.374521, 0.859053, 0.418182, 0.860442, 0.365257, 0.861788, 0.374521, 0.860442, 0.365257, 0.859053, 0.418182, 0.861788, 0.374521, 0.859053, 0.418182, 0.859053, 0.354545, 0.860442, 0.365257, 0.859053, 0.354545, 0.859053, 0.418182, 0.860442, 0.365257, 0.864528, 0.054545, 0.858818, 0.054545, 0.858818, 0.009091, 0.858818, 0.054545, 0.864528, 0.054545, 0.858818, 0.009091, 0.881095, 0.009091, 0.864528, 0.054545, 0.858818, 0.009091, 0.864528, 0.054545, 0.881095, 0.009091, 0.858818, 0.009091, 0.864528, 0.054545, 0.881095, 0.009091, 0.881095, 0.054545, 0.881095, 0.009091, 0.864528, 0.054545, 0.881095, 0.054545, 0.891270, 0.081818, 0.896228, 0.081818, 0.891270, 0.081818, 0.891270, 0.081818, 0.896228, 0.081818, 0.896228, 0.081818, 0.896228, 0.081818, 0.896228, 0.009091, 0.896228, 0.081818, 0.896228, 0.081818, 0.896228, 0.009091, 0.896228, 0.009091, 0.896228, 0.009091, 0.891270, 0.009091, 0.896228, 0.009091, 0.896228, 0.009091, 0.891270, 0.009091, 0.891270, 0.009091, 0.891270, 0.081818, 0.896228, 0.009091, 0.896228, 0.081818, 0.896228, 0.009091, 0.891270, 0.081818, 0.896228, 0.081818, 0.891270, 0.081818, 0.891270, 0.009091, 0.896228, 0.009091, 0.891270, 0.009091, 0.891270, 0.081818, 0.896228, 0.009091, 0.910633, 0.390909, 0.910633, 0.054545, 0.910633, 0.390909, 0.910633, 0.390909, 0.910633, 0.054545, 0.910633, 0.054545, 0.910633, 0.054545, 0.916367, 0.054545, 0.910633, 0.054545, 0.910633, 0.054545, 0.916367, 0.054545, 0.916367, 0.054545, 0.916367, 0.054545, 0.919755, 0.057080, 0.916367, 0.054545, 0.916367, 0.054545, 0.919755, 0.057080, 0.919755, 0.057080, 0.919755, 0.057080, 0.922656, 0.064684, 0.919755, 0.057080, 0.919755, 0.057080, 0.922656, 0.064684, 0.922656, 0.064684, 0.922656, 0.064684, 0.925071, 0.077357, 0.922656, 0.064684, 0.922656, 0.064684, 0.925071, 0.077357, 0.925071, 0.077357, 0.925071, 0.077357, 0.927000, 0.095099, 0.925071, 0.077357, 0.925071, 0.077357, 0.927000, 0.095099, 0.927000, 0.095099, 0.927000, 0.095099, 0.928476, 0.118311, 0.927000, 0.095099, 0.927000, 0.095099, 0.928476, 0.118311, 0.928476, 0.118311, 0.928476, 0.118311, 0.929529, 0.147390, 0.928476, 0.118311, 0.928476, 0.118311, 0.929529, 0.147390, 0.929529, 0.147390, 0.929529, 0.147390, 0.930162, 0.182338, 0.929529, 0.147390, 0.929529, 0.147390, 0.930162, 0.182338, 0.930162, 0.182338, 0.930162, 0.182338, 0.930372, 0.223153, 0.930162, 0.182338, 0.930162, 0.182338, 0.930372, 0.223153, 0.930372, 0.223153, 0.930372, 0.223153, 0.930162, 0.263712, 0.930372, 0.223153, 0.930372, 0.223153, 0.930162, 0.263712, 0.930162, 0.263712, 0.930162, 0.263712, 0.929529, 0.298455, 0.930162, 0.263712, 0.930162, 0.263712, 0.929529, 0.298455, 0.929529, 0.298455, 0.929529, 0.298455, 0.928476, 0.327384, 0.929529, 0.298455, 0.929529, 0.298455, 0.928476, 0.327384, 0.928476, 0.327384, 0.928476, 0.327384, 0.927000, 0.350497, 0.928476, 0.327384, 0.928476, 0.327384, 0.927000, 0.350497, 0.927000, 0.350497, 0.927000, 0.350497, 0.925071, 0.368177, 0.927000, 0.350497, 0.927000, 0.350497, 0.925071, 0.368177, 0.925071, 0.368177, 0.925071, 0.368177, 0.922656, 0.380806, 0.925071, 0.368177, 0.925071, 0.368177, 0.922656, 0.380806, 0.922656, 0.380806, 0.922656, 0.380806, 0.919755, 0.388383, 0.922656, 0.380806, 0.922656, 0.380806, 0.919755, 0.388383, 0.919755, 0.388383, 0.919755, 0.388383, 0.916367, 0.390909, 0.919755, 0.388383, 0.919755, 0.388383, 0.916367, 0.390909, 0.916367, 0.390909, 0.905886, 0.436364, 0.915639, 0.436364, 0.905886, 0.436364, 0.905886, 0.436364, 0.915639, 0.436364, 0.915639, 0.436364, 0.915639, 0.436364, 0.920399, 0.433137, 0.915639, 0.436364, 0.915639, 0.436364, 0.920399, 0.433137, 0.920399, 0.433137, 0.920399, 0.433137, 0.924480, 0.423455, 0.920399, 0.433137, 0.920399, 0.433137, 0.924480, 0.423455, 0.924480, 0.423455, 0.924480, 0.423455, 0.927883, 0.407320, 0.924480, 0.423455, 0.924480, 0.423455, 0.927883, 0.407320, 0.927883, 0.407320, 0.927883, 0.407320, 0.930607, 0.384730, 0.927883, 0.407320, 0.927883, 0.407320, 0.930607, 0.384730, 0.930607, 0.384730, 0.930607, 0.384730, 0.932694, 0.355269, 0.930607, 0.384730, 0.930607, 0.384730, 0.932694, 0.355269, 0.932694, 0.355269, 0.932694, 0.355269, 0.934185, 0.318519, 0.932694, 0.355269, 0.932694, 0.355269, 0.934185, 0.318519, 0.934185, 0.318519, 0.934185, 0.318519, 0.935080, 0.274481, 0.934185, 0.318519, 0.934185, 0.318519, 0.935080, 0.274481, 0.935080, 0.274481, 0.935080, 0.274481, 0.935378, 0.223153, 0.935080, 0.274481, 0.935080, 0.274481, 0.935378, 0.223153, 0.935378, 0.223153, 0.935378, 0.223153, 0.935078, 0.171569, 0.935378, 0.223153, 0.935378, 0.223153, 0.935078, 0.171569, 0.935078, 0.171569, 0.935078, 0.171569, 0.934179, 0.127326, 0.935078, 0.171569, 0.935078, 0.171569, 0.934179, 0.127326, 0.934179, 0.127326, 0.934179, 0.127326, 0.932681, 0.090425, 0.934179, 0.127326, 0.934179, 0.127326, 0.932681, 0.090425, 0.932681, 0.090425, 0.932681, 0.090425, 0.930584, 0.060866, 0.932681, 0.090425, 0.932681, 0.090425, 0.930584, 0.060866, 0.930584, 0.060866, 0.930584, 0.060866, 0.927852, 0.038215, 0.930584, 0.060866, 0.930584, 0.060866, 0.927852, 0.038215, 0.927852, 0.038215, 0.927852, 0.038215, 0.924451, 0.022035, 0.927852, 0.038215, 0.927852, 0.038215, 0.924451, 0.022035, 0.924451, 0.022035, 0.924451, 0.022035, 0.920380, 0.012327, 0.924451, 0.022035, 0.924451, 0.022035, 0.920380, 0.012327, 0.920380, 0.012327, 0.920380, 0.012327, 0.915639, 0.009091, 0.920380, 0.012327, 0.920380, 0.012327, 0.915639, 0.009091, 0.915639, 0.009091, 0.915639, 0.009091, 0.905886, 0.009091, 0.915639, 0.009091, 0.915639, 0.009091, 0.905886, 0.009091, 0.905886, 0.009091, 0.905886, 0.436364, 0.910633, 0.390909, 0.915639, 0.436364, 0.910633, 0.390909, 0.905886, 0.436364, 0.915639, 0.436364, 0.905886, 0.436364, 0.905886, 0.009091, 0.910633, 0.390909, 0.905886, 0.009091, 0.905886, 0.436364, 0.910633, 0.390909, 0.905886, 0.009091, 0.910633, 0.054545, 0.910633, 0.390909, 0.910633, 0.054545, 0.905886, 0.009091, 0.910633, 0.390909, 0.910633, 0.054545, 0.905886, 0.009091, 0.915639, 0.009091, 0.905886, 0.009091, 0.910633, 0.054545, 0.915639, 0.009091, 0.910633, 0.054545, 0.915639, 0.009091, 0.916367, 0.054545, 0.915639, 0.009091, 0.910633, 0.054545, 0.916367, 0.054545, 0.916367, 0.054545, 0.915639, 0.009091, 0.920380, 0.012327, 0.915639, 0.009091, 0.916367, 0.054545, 0.920380, 0.012327, 0.919755, 0.057080, 0.916367, 0.054545, 0.920380, 0.012327, 0.916367, 0.054545, 0.919755, 0.057080, 0.920380, 0.012327, 0.922656, 0.064684, 0.919755, 0.057080, 0.920380, 0.012327, 0.919755, 0.057080, 0.922656, 0.064684, 0.920380, 0.012327, 0.924451, 0.022035, 0.922656, 0.064684, 0.920380, 0.012327, 0.922656, 0.064684, 0.924451, 0.022035, 0.920380, 0.012327, 0.925071, 0.077357, 0.922656, 0.064684, 0.924451, 0.022035, 0.922656, 0.064684, 0.925071, 0.077357, 0.924451, 0.022035, 0.927852, 0.038215, 0.925071, 0.077357, 0.924451, 0.022035, 0.925071, 0.077357, 0.927852, 0.038215, 0.924451, 0.022035, 0.927000, 0.095099, 0.925071, 0.077357, 0.927852, 0.038215, 0.925071, 0.077357, 0.927000, 0.095099, 0.927852, 0.038215, 0.927000, 0.095099, 0.927852, 0.038215, 0.930584, 0.060866, 0.927852, 0.038215, 0.927000, 0.095099, 0.930584, 0.060866, 0.928476, 0.118311, 0.927000, 0.095099, 0.930584, 0.060866, 0.927000, 0.095099, 0.928476, 0.118311, 0.930584, 0.060866, 0.928476, 0.118311, 0.930584, 0.060866, 0.932681, 0.090425, 0.930584, 0.060866, 0.928476, 0.118311, 0.932681, 0.090425, 0.929529, 0.147390, 0.928476, 0.118311, 0.932681, 0.090425, 0.928476, 0.118311, 0.929529, 0.147390, 0.932681, 0.090425, 0.934179, 0.127326, 0.929529, 0.147390, 0.932681, 0.090425, 0.929529, 0.147390, 0.934179, 0.127326, 0.932681, 0.090425, 0.929529, 0.147390, 0.934179, 0.127326, 0.935078, 0.171569, 0.934179, 0.127326, 0.929529, 0.147390, 0.935078, 0.171569, 0.930162, 0.182338, 0.929529, 0.147390, 0.935078, 0.171569, 0.929529, 0.147390, 0.930162, 0.182338, 0.935078, 0.171569, 0.935378, 0.223153, 0.930162, 0.182338, 0.935078, 0.171569, 0.930162, 0.182338, 0.935378, 0.223153, 0.935078, 0.171569, 0.935378, 0.223153, 0.930372, 0.223153, 0.930162, 0.182338, 0.930372, 0.223153, 0.935378, 0.223153, 0.930162, 0.182338, 0.935378, 0.223153, 0.930162, 0.263712, 0.930372, 0.223153, 0.930162, 0.263712, 0.935378, 0.223153, 0.930372, 0.223153, 0.935080, 0.274481, 0.930162, 0.263712, 0.935378, 0.223153, 0.930162, 0.263712, 0.935080, 0.274481, 0.935378, 0.223153, 0.935080, 0.274481, 0.929529, 0.298455, 0.930162, 0.263712, 0.929529, 0.298455, 0.935080, 0.274481, 0.930162, 0.263712, 0.934185, 0.318519, 0.929529, 0.298455, 0.935080, 0.274481, 0.929529, 0.298455, 0.934185, 0.318519, 0.935080, 0.274481, 0.932694, 0.355269, 0.929529, 0.298455, 0.934185, 0.318519, 0.929529, 0.298455, 0.932694, 0.355269, 0.934185, 0.318519, 0.928476, 0.327384, 0.929529, 0.298455, 0.932694, 0.355269, 0.929529, 0.298455, 0.928476, 0.327384, 0.932694, 0.355269, 0.930607, 0.384730, 0.928476, 0.327384, 0.932694, 0.355269, 0.928476, 0.327384, 0.930607, 0.384730, 0.932694, 0.355269, 0.927000, 0.350497, 0.928476, 0.327384, 0.930607, 0.384730, 0.928476, 0.327384, 0.927000, 0.350497, 0.930607, 0.384730, 0.927883, 0.407320, 0.927000, 0.350497, 0.930607, 0.384730, 0.927000, 0.350497, 0.927883, 0.407320, 0.930607, 0.384730, 0.925071, 0.368177, 0.927000, 0.350497, 0.927883, 0.407320, 0.927000, 0.350497, 0.925071, 0.368177, 0.927883, 0.407320, 0.924480, 0.423455, 0.925071, 0.368177, 0.927883, 0.407320, 0.925071, 0.368177, 0.924480, 0.423455, 0.927883, 0.407320, 0.924480, 0.423455, 0.922656, 0.380806, 0.925071, 0.368177, 0.922656, 0.380806, 0.924480, 0.423455, 0.925071, 0.368177, 0.920399, 0.433137, 0.922656, 0.380806, 0.924480, 0.423455, 0.922656, 0.380806, 0.920399, 0.433137, 0.924480, 0.423455, 0.920399, 0.433137, 0.919755, 0.388383, 0.922656, 0.380806, 0.919755, 0.388383, 0.920399, 0.433137, 0.922656, 0.380806, 0.920399, 0.433137, 0.916367, 0.390909, 0.919755, 0.388383, 0.916367, 0.390909, 0.920399, 0.433137, 0.919755, 0.388383, 0.920399, 0.433137, 0.915639, 0.436364, 0.916367, 0.390909, 0.915639, 0.436364, 0.920399, 0.433137, 0.916367, 0.390909, 0.915639, 0.436364, 0.910633, 0.390909, 0.916367, 0.390909, 0.910633, 0.390909, 0.915639, 0.436364, 0.916367, 0.390909, 0.942733, 0.436364, 0.947480, 0.436364, 0.942733, 0.436364, 0.942733, 0.436364, 0.947480, 0.436364, 0.947480, 0.436364, 0.947480, 0.436364, 0.947480, 0.054545, 0.947480, 0.436364, 0.947480, 0.436364, 0.947480, 0.054545, 0.947480, 0.054545, 0.947480, 0.054545, 0.964564, 0.054545, 0.947480, 0.054545, 0.947480, 0.054545, 0.964564, 0.054545, 0.964564, 0.054545, 0.964564, 0.054545, 0.964564, 0.009091, 0.964564, 0.054545, 0.964564, 0.054545, 0.964564, 0.009091, 0.964564, 0.009091, 0.964564, 0.009091, 0.942733, 0.009091, 0.964564, 0.009091, 0.964564, 0.009091, 0.942733, 0.009091, 0.942733, 0.009091, 0.942733, 0.436364, 0.947480, 0.054545, 0.947480, 0.436364, 0.947480, 0.054545, 0.942733, 0.436364, 0.947480, 0.436364, 0.942733, 0.436364, 0.942733, 0.009091, 0.947480, 0.054545, 0.942733, 0.009091, 0.942733, 0.436364, 0.947480, 0.054545, 0.947480, 0.054545, 0.942733, 0.009091, 0.964564, 0.009091, 0.942733, 0.009091, 0.947480, 0.054545, 0.964564, 0.009091, 0.947480, 0.054545, 0.964564, 0.009091, 0.964564, 0.054545, 0.964564, 0.009091, 0.947480, 0.054545, 0.964564, 0.054545, 0.969804, 0.436364, 0.974550, 0.436364, 0.969804, 0.436364, 0.969804, 0.436364, 0.974550, 0.436364, 0.974550, 0.436364, 0.974550, 0.436364, 0.974550, 0.054545, 0.974550, 0.436364, 0.974550, 0.436364, 0.974550, 0.054545, 0.974550, 0.054545, 0.974550, 0.054545, 0.991634, 0.054545, 0.974550, 0.054545, 0.974550, 0.054545, 0.991634, 0.054545, 0.991634, 0.054545, 0.991634, 0.054545, 0.991634, 0.009091, 0.991634, 0.054545, 0.991634, 0.054545, 0.991634, 0.009091, 0.991634, 0.009091, 0.991634, 0.009091, 0.969804, 0.009091, 0.991634, 0.009091, 0.991634, 0.009091, 0.969804, 0.009091, 0.969804, 0.009091, 0.969804, 0.436364, 0.974550, 0.054545, 0.974550, 0.436364, 0.974550, 0.054545, 0.969804, 0.436364, 0.974550, 0.436364, 0.969804, 0.436364, 0.969804, 0.009091, 0.974550, 0.054545, 0.969804, 0.009091, 0.969804, 0.436364, 0.974550, 0.054545, 0.974550, 0.054545, 0.969804, 0.009091, 0.991634, 0.009091, 0.969804, 0.009091, 0.974550, 0.054545, 0.991634, 0.009091, 0.974550, 0.054545, 0.991634, 0.009091, 0.991634, 0.054545, 0.991634, 0.009091, 0.974550, 0.054545, 0.991634, 0.054545 };
    TextureCoordinate7.point = m_point;
    TextureCoordinate7.point_length = sizeof(m_point) / sizeof(float);
    }
    Text1m_5.color = NULL;
    Text1m_5.coord = &Coordinate14;
    Text1m_5.normal = &Normal15;
    Text1m_5.texCoord = &TextureCoordinate7;
    Text1m_5.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    Text1m_5.colorIndex = m_colorIndex;
    Text1m_5.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    Text1m_5.colorPerVertex = false;
    Text1m_5.convex = false;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1, 5994, 5995, 5996, -1, 5997, 5998, 5999, -1, 6000, 6001, 6002, -1, 6003, 6004, 6005, -1, 6006, 6007, 6008, -1, 6009, 6010, 6011, -1, 6012, 6013, 6014, -1, 6015, 6016, 6017, -1, 6018, 6019, 6020, -1, 6021, 6022, 6023, -1, 6024, 6025, 6026, -1, 6027, 6028, 6029, -1, 6030, 6031, 6032, -1, 6033, 6034, 6035, -1, 6036, 6037, 6038, -1, 6039, 6040, 6041, -1, 6042, 6043, 6044, -1, 6045, 6046, 6047, -1, 6048, 6049, 6050, -1, 6051, 6052, 6053, -1, 6054, 6055, 6056, -1, 6057, 6058, 6059, -1, 6060, 6061, 6062, -1, 6063, 6064, 6065, -1, 6066, 6067, 6068, -1, 6069, 6070, 6071, -1, 6072, 6073, 6074, -1, 6075, 6076, 6077, -1, 6078, 6079, 6080, -1, 6081, 6082, 6083, -1, 6084, 6085, 6086, -1, 6087, 6088, 6089, -1, 6090, 6091, 6092, -1, 6093, 6094, 6095, -1, 6096, 6097, 6098, -1, 6099, 6100, 6101, -1, 6102, 6103, 6104, -1, 6105, 6106, 6107, -1, 6108, 6109, 6110, -1, 6111, 6112, 6113, -1, 6114, 6115, 6116, -1, 6117, 6118, 6119, -1, 6120, 6121, 6122, -1, 6123, 6124, 6125, -1, 6126, 6127, 6128, -1, 6129, 6130, 6131, -1, 6132, 6133, 6134, -1, 6135, 6136, 6137, -1, 6138, 6139, 6140, -1, 6141, 6142, 6143, -1, 6144, 6145, 6146, -1, 6147, 6148, 6149, -1, 6150, 6151, 6152, -1, 6153, 6154, 6155, -1, 6156, 6157, 6158, -1, 6159, 6160, 6161, -1, 6162, 6163, 6164, -1, 6165, 6166, 6167, -1, 6168, 6169, 6170, -1, 6171, 6172, 6173, -1, 6174, 6175, 6176, -1, 6177, 6178, 6179, -1, 6180, 6181, 6182, -1, 6183, 6184, 6185, -1, 6186, 6187, 6188, -1, 6189, 6190, 6191, -1, 6192, 6193, 6194, -1, 6195, 6196, 6197, -1, 6198, 6199, 6200, -1, 6201, 6202, 6203, -1, 6204, 6205, 6206, -1, 6207, 6208, 6209, -1, 6210, 6211, 6212, -1, 6213, 6214, 6215, -1, 6216, 6217, 6218, -1, 6219, 6220, 6221, -1, 6222, 6223, 6224, -1, 6225, 6226, 6227, -1, 6228, 6229, 6230, -1, 6231, 6232, 6233, -1, 6234, 6235, 6236, -1, 6237, 6238, 6239, -1, 6240, 6241, 6242, -1, 6243, 6244, 6245, -1, 6246, 6247, 6248, -1, 6249, 6250, 6251, -1, 6252, 6253, 6254, -1, 6255, 6256, 6257, -1, 6258, 6259, 6260, -1, 6261, 6262, 6263, -1, 6264, 6265, 6266, -1, 6267, 6268, 6269, -1, 6270, 6271, 6272, -1, 6273, 6274, 6275, -1, 6276, 6277, 6278, -1, 6279, 6280, 6281, -1, 6282, 6283, 6284, -1, 6285, 6286, 6287, -1, 6288, 6289, 6290, -1, 6291, 6292, 6293, -1, 6294, 6295, 6296, -1, 6297, 6298, 6299, -1, 6300, 6301, 6302, -1, 6303, 6304, 6305, -1, 6306, 6307, 6308, -1, 6309, 6310, 6311, -1, 6312, 6313, 6314, -1, 6315, 6316, 6317, -1, 6318, 6319, 6320, -1, 6321, 6322, 6323, -1, 6324, 6325, 6326, -1, 6327, 6328, 6329, -1, 6330, 6331, 6332, -1, 6333, 6334, 6335, -1, 6336, 6337, 6338, -1, 6339, 6340, 6341, -1, 6342, 6343, 6344, -1, 6345, 6346, 6347, -1, 6348, 6349, 6350, -1, 6351, 6352, 6353, -1, 6354, 6355, 6356, -1, 6357, 6358, 6359, -1, 6360, 6361, 6362, -1, 6363, 6364, 6365, -1, 6366, 6367, 6368, -1, 6369, 6370, 6371, -1, 6372, 6373, 6374, -1, 6375, 6376, 6377, -1, 6378, 6379, 6380, -1, 6381, 6382, 6383, -1, 6384, 6385, 6386, -1, 6387, 6388, 6389, -1, 6390, 6391, 6392, -1, 6393, 6394, 6395, -1, 6396, 6397, 6398, -1, 6399, 6400, 6401, -1, 6402, 6403, 6404, -1, 6405, 6406, 6407, -1, 6408, 6409, 6410, -1, 6411, 6412, 6413, -1, 6414, 6415, 6416, -1, 6417, 6418, 6419, -1, 6420, 6421, 6422, -1, 6423, 6424, 6425, -1, 6426, 6427, 6428, -1, 6429, 6430, 6431, -1, 6432, 6433, 6434, -1, 6435, 6436, 6437, -1, 6438, 6439, 6440, -1, 6441, 6442, 6443, -1, 6444, 6445, 6446, -1, 6447, 6448, 6449, -1, 6450, 6451, 6452, -1, 6453, 6454, 6455, -1, 6456, 6457, 6458, -1, 6459, 6460, 6461, -1, 6462, 6463, 6464, -1, 6465, 6466, 6467, -1, 6468, 6469, 6470, -1, 6471, 6472, 6473, -1, 6474, 6475, 6476, -1, 6477, 6478, 6479, -1, 6480, 6481, 6482, -1, 6483, 6484, 6485, -1, 6486, 6487, 6488, -1, 6489, 6490, 6491, -1, 6492, 6493, 6494, -1, 6495, 6496, 6497, -1, 6498, 6499, 6500, -1, 6501, 6502, 6503, -1, 6504, 6505, 6506, -1, 6507, 6508, 6509, -1, 6510, 6511, 6512, -1, 6513, 6514, 6515, -1, 6516, 6517, 6518, -1, 6519, 6520, 6521, -1, 6522, 6523, 6524, -1, 6525, 6526, 6527, -1, 6528, 6529, 6530, -1, 6531, 6532, 6533, -1, 6534, 6535, 6536, -1, 6537, 6538, 6539, -1, 6540, 6541, 6542, -1, 6543, 6544, 6545, -1, 6546, 6547, 6548, -1, 6549, 6550, 6551, -1, 6552, 6553, 6554, -1, 6555, 6556, 6557, -1, 6558, 6559, 6560, -1, 6561, 6562, 6563, -1, 6564, 6565, 6566, -1, 6567, 6568, 6569, -1, 6570, 6571, 6572, -1, 6573, 6574, 6575, -1, 6576, 6577, 6578, -1, 6579, 6580, 6581, -1, 6582, 6583, 6584, -1, 6585, 6586, 6587, -1, 6588, 6589, 6590, -1, 6591, 6592, 6593, -1, 6594, 6595, 6596, -1, 6597, 6598, 6599, -1, 6600, 6601, 6602, -1, 6603, 6604, 6605, -1, 6606, 6607, 6608, -1, 6609, 6610, 6611, -1, 6612, 6613, 6614, -1, 6615, 6616, 6617, -1, 6618, 6619, 6620, -1, 6621, 6622, 6623, -1, 6624, 6625, 6626, -1, 6627, 6628, 6629, -1, 6630, 6631, 6632, -1, 6633, 6634, 6635, -1, 6636, 6637, 6638, -1, 6639, 6640, 6641, -1, 6642, 6643, 6644, -1, 6645, 6646, 6647, -1, 6648, 6649, 6650, -1, 6651, 6652, 6653, -1, 6654, 6655, 6656, -1, 6657, 6658, 6659, -1, 6660, 6661, 6662, -1, 6663, 6664, 6665, -1, 6666, 6667, 6668, -1, 6669, 6670, 6671, -1, 6672, 6673, 6674, -1, 6675, 6676, 6677, -1, 6678, 6679, 6680, -1, 6681, 6682, 6683, -1, 6684, 6685, 6686, -1, 6687, 6688, 6689, -1, 6690, 6691, 6692, -1, 6693, 6694, 6695, -1, 6696, 6697, 6698, -1, 6699, 6700, 6701, -1, 6702, 6703, 6704, -1, 6705, 6706, 6707, -1, 6708, 6709, 6710, -1, 6711, 6712, 6713, -1, 6714, 6715, 6716, -1, 6717, 6718, 6719, -1, 6720, 6721, 6722, -1, 6723, 6724, 6725, -1, 6726, 6727, 6728, -1, 6729, 6730, 6731, -1, 6732, 6733, 6734, -1, 6735, 6736, 6737, -1, 6738, 6739, 6740, -1, 6741, 6742, 6743, -1, 6744, 6745, 6746, -1, 6747, 6748, 6749, -1, 6750, 6751, 6752, -1, 6753, 6754, 6755, -1, 6756, 6757, 6758, -1, 6759, 6760, 6761, -1, 6762, 6763, 6764, -1, 6765, 6766, 6767, -1, 6768, 6769, 6770, -1, 6771, 6772, 6773, -1, 6774, 6775, 6776, -1, 6777, 6778, 6779, -1, 6780, 6781, 6782, -1, 6783, 6784, 6785, -1, 6786, 6787, 6788, -1, 6789, 6790, 6791, -1, 6792, 6793, 6794, -1, 6795, 6796, 6797, -1, 6798, 6799, 6800, -1, 6801, 6802, 6803, -1, 6804, 6805, 6806, -1, 6807, 6808, 6809, -1, 6810, 6811, 6812, -1, 6813, 6814, 6815, -1, 6816, 6817, 6818, -1, 6819, 6820, 6821, -1, 6822, 6823, 6824, -1, 6825, 6826, 6827, -1, 6828, 6829, 6830, -1, 6831, 6832, 6833, -1, 6834, 6835, 6836, -1, 6837, 6838, 6839, -1, 6840, 6841, 6842, -1, 6843, 6844, 6845, -1, 6846, 6847, 6848, -1, 6849, 6850, 6851, -1, 6852, 6853, 6854, -1, 6855, 6856, 6857, -1, 6858, 6859, 6860, -1, 6861, 6862, 6863, -1, 6864, 6865, 6866, -1, 6867, 6868, 6869, -1, 6870, 6871, 6872, -1, 6873, 6874, 6875, -1, 6876, 6877, 6878, -1, 6879, 6880, 6881, -1, 6882, 6883, 6884, -1, 6885, 6886, 6887, -1, 6888, 6889, 6890, -1, 6891, 6892, 6893, -1, 6894, 6895, 6896, -1, 6897, 6898, 6899, -1, 6900, 6901, 6902, -1, 6903, 6904, 6905, -1, 6906, 6907, 6908, -1, 6909, 6910, 6911, -1, 6912, 6913, 6914, -1, 6915, 6916, 6917, -1, 6918, 6919, 6920, -1, 6921, 6922, 6923, -1, 6924, 6925, 6926, -1, 6927, 6928, 6929, -1, 6930, 6931, 6932, -1, 6933, 6934, 6935, -1, 6936, 6937, 6938, -1, 6939, 6940, 6941, -1, 6942, 6943, 6944, -1, 6945, 6946, 6947, -1, 6948, 6949, 6950, -1, 6951, 6952, 6953, -1, 6954, 6955, 6956, -1, 6957, 6958, 6959, -1, 6960, 6961, 6962, -1, 6963, 6964, 6965, -1, 6966, 6967, 6968, -1, 6969, 6970, 6971, -1, 6972, 6973, 6974, -1, 6975, 6976, 6977, -1, 6978, 6979, 6980, -1, 6981, 6982, 6983, -1, 6984, 6985, 6986, -1, 6987, 6988, 6989, -1, 6990, 6991, 6992, -1, 6993, 6994, 6995, -1, 6996, 6997, 6998, -1, 6999, 7000, 7001, -1, 7002, 7003, 7004, -1, 7005, 7006, 7007, -1, 7008, 7009, 7010, -1, 7011, 7012, 7013, -1, 7014, 7015, 7016, -1, 7017, 7018, 7019, -1, 7020, 7021, 7022, -1, 7023, 7024, 7025, -1, 7026, 7027, 7028, -1, 7029, 7030, 7031, -1, 7032, 7033, 7034, -1, 7035, 7036, 7037, -1, 7038, 7039, 7040, -1, 7041, 7042, 7043, -1, 7044, 7045, 7046, -1, 7047, 7048, 7049, -1, 7050, 7051, 7052, -1, 7053, 7054, 7055, -1, 7056, 7057, 7058, -1, 7059, 7060, 7061, -1, 7062, 7063, 7064, -1, 7065, 7066, 7067, -1, 7068, 7069, 7070, -1, 7071, 7072, 7073, -1, 7074, 7075, 7076, -1, 7077, 7078, 7079, -1, 7080, 7081, 7082, -1, 7083, 7084, 7085, -1, 7086, 7087, 7088, -1, 7089, 7090, 7091, -1, 7092, 7093, 7094, -1, 7095, 7096, 7097, -1, 7098, 7099, 7100, -1, 7101, 7102, 7103, -1, 7104, 7105, 7106, -1, 7107, 7108, 7109, -1, 7110, 7111, 7112, -1, 7113, 7114, 7115, -1, 7116, 7117, 7118, -1, 7119, 7120, 7121, -1, 7122, 7123, 7124, -1, 7125, 7126, 7127, -1, 7128, 7129, 7130, -1, 7131, 7132, 7133, -1, 7134, 7135, 7136, -1, 7137, 7138, 7139, -1, 7140, 7141, 7142, -1, 7143, 7144, 7145, -1, 7146, 7147, 7148, -1, 7149, 7150, 7151, -1, 7152, 7153, 7154, -1, 7155, 7156, 7157, -1, 7158, 7159, 7160, -1, 7161, 7162, 7163, -1, 7164, 7165, 7166, -1, 7167, 7168, 7169, -1, 7170, 7171, 7172, -1, 7173, 7174, 7175, -1, 7176, 7177, 7178, -1, 7179, 7180, 7181, -1, 7182, 7183, 7184, -1, 7185, 7186, 7187, -1, 7188, 7189, 7190, -1, 7191, 7192, 7193, -1, 7194, 7195, 7196, -1, 7197, 7198, 7199, -1, 7200, 7201, 7202, -1, 7203, 7204, 7205, -1, 7206, 7207, 7208, -1, 7209, 7210, 7211, -1, 7212, 7213, 7214, -1, 7215, 7216, 7217, -1, 7218, 7219, 7220, -1, 7221, 7222, 7223, -1, 7224, 7225, 7226, -1, 7227, 7228, 7229, -1, 7230, 7231, 7232, -1, 7233, 7234, 7235, -1, 7236, 7237, 7238, -1, 7239, 7240, 7241, -1, 7242, 7243, 7244, -1, 7245, 7246, 7247, -1, 7248, 7249, 7250, -1, 7251, 7252, 7253, -1, 7254, 7255, 7256, -1, 7257, 7258, 7259, -1, 7260, 7261, 7262, -1, 7263, 7264, 7265, -1, 7266, 7267, 7268, -1, 7269, 7270, 7271, -1, 7272, 7273, 7274, -1, 7275, 7276, 7277, -1, 7278, 7279, 7280, -1, 7281, 7282, 7283, -1, 7284, 7285, 7286, -1, 7287, 7288, 7289, -1, 7290, 7291, 7292, -1, 7293, 7294, 7295, -1, 7296, 7297, 7298, -1, 7299, 7300, 7301, -1, 7302, 7303, 7304, -1, 7305, 7306, 7307, -1, 7308, 7309, 7310, -1, 7311, 7312, 7313, -1, 7314, 7315, 7316, -1, 7317, 7318, 7319, -1, 7320, 7321, 7322, -1, 7323, 7324, 7325, -1, 7326, 7327, 7328, -1, 7329, 7330, 7331, -1, 7332, 7333, 7334, -1, 7335, 7336, 7337, -1, 7338, 7339, 7340, -1, 7341, 7342, 7343, -1, 7344, 7345, 7346, -1, 7347, 7348, 7349, -1, 7350, 7351, 7352, -1, 7353, 7354, 7355, -1, 7356, 7357, 7358, -1, 7359, 7360, 7361, -1, 7362, 7363, 7364, -1, 7365, 7366, 7367, -1, 7368, 7369, 7370, -1, 7371, 7372, 7373, -1, 7374, 7375, 7376, -1, 7377, 7378, 7379, -1, 7380, 7381, 7382, -1, 7383, 7384, 7385, -1, 7386, 7387, 7388, -1, 7389, 7390, 7391, -1, 7392, 7393, 7394, -1, 7395, 7396, 7397, -1, 7398, 7399, 7400, -1, 7401, 7402, 7403, -1, 7404, 7405, 7406, -1, 7407, 7408, 7409, -1, 7410, 7411, 7412, -1, 7413, 7414, 7415, -1, 7416, 7417, 7418, -1, 7419, 7420, 7421, -1, 7422, 7423, 7424, -1, 7425, 7426, 7427, -1, 7428, 7429, 7430, -1, 7431, 7432, 7433, -1, 7434, 7435, 7436, -1, 7437, 7438, 7439, -1, 7440, 7441, 7442, -1, 7443, 7444, 7445, -1, 7446, 7447, 7448, -1, 7449, 7450, 7451, -1, 7452, 7453, 7454, -1, 7455, 7456, 7457, -1, 7458, 7459, 7460, -1, 7461, 7462, 7463, -1, 7464, 7465, 7466, -1, 7467, 7468, 7469, -1, 7470, 7471, 7472, -1, 7473, 7474, 7475, -1, 7476, 7477, 7478, -1, 7479, 7480, 7481, -1, 7482, 7483, 7484, -1, 7485, 7486, 7487, -1, 7488, 7489, 7490, -1, 7491, 7492, 7493, -1, 7494, 7495, 7496, -1, 7497, 7498, 7499, -1, 7500, 7501, 7502, -1, 7503, 7504, 7505, -1, 7506, 7507, 7508, -1, 7509, 7510, 7511, -1, 7512, 7513, 7514, -1, 7515, 7516, 7517, -1, 7518, 7519, 7520, -1, 7521, 7522, 7523, -1, 7524, 7525, 7526, -1, 7527, 7528, 7529, -1, 7530, 7531, 7532, -1, 7533, 7534, 7535, -1, 7536, 7537, 7538, -1, 7539, 7540, 7541, -1, 7542, 7543, 7544, -1, 7545, 7546, 7547, -1, 7548, 7549, 7550, -1, 7551, 7552, 7553, -1, 7554, 7555, 7556, -1, 7557, 7558, 7559, -1, 7560, 7561, 7562, -1, 7563, 7564, 7565, -1, 7566, 7567, 7568, -1, 7569, 7570, 7571, -1, 7572, 7573, 7574, -1, 7575, 7576, 7577, -1, 7578, 7579, 7580, -1, 7581, 7582, 7583, -1, 7584, 7585, 7586, -1, 7587, 7588, 7589, -1, 7590, 7591, 7592, -1, 7593, 7594, 7595, -1, 7596, 7597, 7598, -1, 7599, 7600, 7601, -1, 7602, 7603, 7604, -1, 7605, 7606, 7607, -1, 7608, 7609, 7610, -1, 7611, 7612, 7613, -1, 7614, 7615, 7616, -1, 7617, 7618, 7619, -1, 7620, 7621, 7622, -1, 7623, 7624, 7625, -1, 7626, 7627, 7628, -1, 7629, 7630, 7631, -1, 7632, 7633, 7634, -1, 7635, 7636, 7637, -1, 7638, 7639, 7640, -1, 7641, 7642, 7643, -1, 7644, 7645, 7646, -1, 7647, 7648, 7649, -1, 7650, 7651, 7652, -1, 7653, 7654, 7655, -1, 7656, 7657, 7658, -1, 7659, 7660, 7661, -1, 7662, 7663, 7664, -1, 7665, 7666, 7667, -1, 7668, 7669, 7670, -1, 7671, 7672, 7673, -1, 7674, 7675, 7676, -1, 7677, 7678, 7679, -1, 7680, 7681, 7682, -1, 7683, 7684, 7685, -1, 7686, 7687, 7688, -1, 7689, 7690, 7691, -1, 7692, 7693, 7694, -1, 7695, 7696, 7697, -1, 7698, 7699, 7700, -1, 7701, 7702, 7703, -1, 7704, 7705, 7706, -1, 7707, 7708, 7709, -1, 7710, 7711, 7712, -1, 7713, 7714, 7715, -1, 7716, 7717, 7718, -1, 7719, 7720, 7721, -1, 7722, 7723, 7724, -1, 7725, 7726, 7727, -1, 7728, 7729, 7730, -1, 7731, 7732, 7733, -1, 7734, 7735, 7736, -1, 7737, 7738, 7739, -1, 7740, 7741, 7742, -1, 7743, 7744, 7745, -1, 7746, 7747, 7748, -1, 7749, 7750, 7751, -1, 7752, 7753, 7754, -1, 7755, 7756, 7757, -1, 7758, 7759, 7760, -1, 7761, 7762, 7763, -1, 7764, 7765, 7766, -1, 7767, 7768, 7769, -1, 7770, 7771, 7772, -1, 7773, 7774, 7775, -1, 7776, 7777, 7778, -1, 7779, 7780, 7781, -1, 7782, 7783, 7784, -1, 7785, 7786, 7787, -1, 7788, 7789, 7790, -1, 7791, 7792, 7793, -1, 7794, 7795, 7796, -1, 7797, 7798, 7799, -1, 7800, 7801, 7802, -1, 7803, 7804, 7805, -1, 7806, 7807, 7808, -1, 7809, 7810, 7811, -1, 7812, 7813, 7814, -1, 7815, 7816, 7817, -1, 7818, 7819, 7820, -1, 7821, 7822, 7823, -1, 7824, 7825, 7826, -1, 7827, 7828, 7829, -1, 7830, 7831, 7832, -1, 7833, 7834, 7835, -1, 7836, 7837, 7838, -1, 7839, 7840, 7841, -1, 7842, 7843, 7844, -1, 7845, 7846, 7847, -1, 7848, 7849, 7850, -1, 7851, 7852, 7853, -1, 7854, 7855, 7856, -1, 7857, 7858, 7859, -1, 7860, 7861, 7862, -1, 7863, 7864, 7865, -1, 7866, 7867, 7868, -1, 7869, 7870, 7871, -1, 7872, 7873, 7874, -1, 7875, 7876, 7877, -1, 7878, 7879, 7880, -1, 7881, 7882, 7883, -1, 7884, 7885, 7886, -1, 7887, 7888, 7889, -1, 7890, 7891, 7892, -1, 7893, 7894, 7895, -1, 7896, 7897, 7898, -1, 7899, 7900, 7901, -1, 7902, 7903, 7904, -1, 7905, 7906, 7907, -1, 7908, 7909, 7910, -1, 7911, 7912, 7913, -1, 7914, 7915, 7916, -1, 7917, 7918, 7919, -1, 7920, 7921, 7922, -1, 7923, 7924, 7925, -1, 7926, 7927, 7928, -1, 7929, 7930, 7931, -1, 7932, 7933, 7934, -1, 7935, 7936, 7937, -1, 7938, 7939, 7940, -1, 7941, 7942, 7943, -1, 7944, 7945, 7946, -1, 7947, 7948, 7949, -1, 7950, 7951, 7952, -1, 7953, 7954, 7955, -1, 7956, 7957, 7958, -1, 7959, 7960, 7961, -1, 7962, 7963, 7964, -1, 7965, 7966, 7967, -1, 7968, 7969, 7970, -1, 7971, 7972, 7973, -1, 7974, 7975, 7976, -1, 7977, 7978, 7979, -1, 7980, 7981, 7982, -1, 7983, 7984, 7985, -1, 7986, 7987, 7988, -1, 7989, 7990, 7991, -1, 7992, 7993, 7994, -1, 7995, 7996, 7997, -1, 7998, 7999, 8000, -1, 8001, 8002, 8003, -1, 8004, 8005, 8006, -1, 8007, 8008, 8009, -1, 8010, 8011, 8012, -1, 8013, 8014, 8015, -1, 8016, 8017, 8018, -1, 8019, 8020, 8021, -1, 8022, 8023, 8024, -1, 8025, 8026, 8027, -1, 8028, 8029, 8030, -1, 8031, 8032, 8033, -1, 8034, 8035, 8036, -1, 8037, 8038, 8039, -1, 8040, 8041, 8042, -1, 8043, 8044, 8045, -1, 8046, 8047, 8048, -1, 8049, 8050, 8051, -1, 8052, 8053, 8054, -1, 8055, 8056, 8057, -1, 8058, 8059, 8060, -1, 8061, 8062, 8063, -1, 8064, 8065, 8066, -1, 8067, 8068, 8069, -1, 8070, 8071, 8072, -1, 8073, 8074, 8075, -1, 8076, 8077, 8078, -1, 8079, 8080, 8081, -1, 8082, 8083, 8084, -1, 8085, 8086, 8087, -1, 8088, 8089, 8090, -1, 8091, 8092, 8093, -1, 8094, 8095, 8096, -1, 8097, 8098, 8099, -1, 8100, 8101, 8102, -1, 8103, 8104, 8105, -1, 8106, 8107, 8108, -1, 8109, 8110, 8111, -1, 8112, 8113, 8114, -1, 8115, 8116, 8117, -1, 8118, 8119, 8120, -1, 8121, 8122, 8123, -1, 8124, 8125, 8126, -1, 8127, 8128, 8129, -1, 8130, 8131, 8132, -1, 8133, 8134, 8135, -1, 8136, 8137, 8138, -1, 8139, 8140, 8141, -1, 8142, 8143, 8144, -1, 8145, 8146, 8147, -1, 8148, 8149, 8150, -1, 8151, 8152, 8153, -1, 8154, 8155, 8156, -1, 8157, 8158, 8159, -1, 8160, 8161, 8162, -1, 8163, 8164, 8165, -1, 8166, 8167, 8168, -1, 8169, 8170, 8171, -1, 8172, 8173, 8174, -1, 8175, 8176, 8177, -1, 8178, 8179, 8180, -1, 8181, 8182, 8183, -1, 8184, 8185, 8186, -1, 8187, 8188, 8189, -1, 8190, 8191, 8192, -1, 8193, 8194, 8195, -1, 8196, 8197, 8198, -1, 8199, 8200, 8201, -1, 8202, 8203, 8204, -1, 8205, 8206, 8207, -1, 8208, 8209, 8210, -1, 8211, 8212, 8213, -1, 8214, 8215, 8216, -1, 8217, 8218, 8219, -1, 8220, 8221, 8222, -1, 8223, 8224, 8225, -1, 8226, 8227, 8228, -1, 8229, 8230, 8231, -1, 8232, 8233, 8234, -1, 8235, 8236, 8237, -1, 8238, 8239, 8240, -1, 8241, 8242, 8243, -1, 8244, 8245, 8246, -1, 8247, 8248, 8249, -1, 8250, 8251, 8252, -1, 8253, 8254, 8255, -1, 8256, 8257, 8258, -1, 8259, 8260, 8261, -1, 8262, 8263, 8264, -1, 8265, 8266, 8267, -1, 8268, 8269, 8270, -1, 8271, 8272, 8273, -1, 8274, 8275, 8276, -1, 8277, 8278, 8279, -1, 8280, 8281, 8282, -1, 8283, 8284, 8285, -1, 8286, 8287, 8288, -1, 8289, 8290, 8291, -1, 8292, 8293, 8294, -1, 8295, 8296, 8297, -1, 8298, 8299, 8300, -1, 8301, 8302, 8303, -1, 8304, 8305, 8306, -1, 8307, 8308, 8309, -1, 8310, 8311, 8312, -1, 8313, 8314, 8315, -1, 8316, 8317, 8318, -1, 8319, 8320, 8321, -1, 8322, 8323, 8324, -1, 8325, 8326, 8327, -1, 8328, 8329, 8330, -1, 8331, 8332, 8333, -1, 8334, 8335, 8336, -1, 8337, 8338, 8339, -1, 8340, 8341, 8342, -1, 8343, 8344, 8345, -1, 8346, 8347, 8348, -1, 8349, 8350, 8351, -1, 8352, 8353, 8354, -1, 8355, 8356, 8357, -1, 8358, 8359, 8360, -1, 8361, 8362, 8363, -1, 8364, 8365, 8366, -1, 8367, 8368, 8369, -1, 8370, 8371, 8372, -1, 8373, 8374, 8375, -1, 8376, 8377, 8378, -1, 8379, 8380, 8381, -1, 8382, 8383, 8384, -1, 8385, 8386, 8387, -1, 8388, 8389, 8390, -1, 8391, 8392, 8393, -1, 8394, 8395, 8396, -1, 8397, 8398, 8399, -1, 8400, 8401, 8402, -1, 8403, 8404, 8405, -1, 8406, 8407, 8408, -1, 8409, 8410, 8411, -1, 8412, 8413, 8414, -1, 8415, 8416, 8417, -1, 8418, 8419, 8420, -1, 8421, 8422, 8423, -1, 8424, 8425, 8426, -1, 8427, 8428, 8429, -1, 8430, 8431, 8432, -1, 8433, 8434, 8435, -1, 8436, 8437, 8438, -1, 8439, 8440, 8441, -1, 8442, 8443, 8444, -1, 8445, 8446, 8447, -1, 8448, 8449, 8450, -1, 8451, 8452, 8453, -1, 8454, 8455, 8456, -1, 8457, 8458, 8459, -1, 8460, 8461, 8462, -1, 8463, 8464, 8465, -1, 8466, 8467, 8468, -1, 8469, 8470, 8471, -1, 8472, 8473, 8474, -1, 8475, 8476, 8477, -1, 8478, 8479, 8480, -1, 8481, 8482, 8483, -1, 8484, 8485, 8486, -1, 8487, 8488, 8489, -1, 8490, 8491, 8492, -1, 8493, 8494, 8495, -1, 8496, 8497, 8498, -1, 8499, 8500, 8501, -1, 8502, 8503, 8504, -1, 8505, 8506, 8507, -1, 8508, 8509, 8510, -1, 8511, 8512, 8513, -1, 8514, 8515, 8516, -1, 8517, 8518, 8519, -1, 8520, 8521, 8522, -1, 8523, 8524, 8525, -1, 8526, 8527, 8528, -1, 8529, 8530, 8531, -1, 8532, 8533, 8534, -1, 8535, 8536, 8537, -1, 8538, 8539, 8540, -1, 8541, 8542, 8543, -1, 8544, 8545, 8546, -1, 8547, 8548, 8549, -1, 8550, 8551, 8552, -1, 8553, 8554, 8555, -1, 8556, 8557, 8558, -1, 8559, 8560, 8561, -1, 8562, 8563, 8564, -1, 8565, 8566, 8567, -1, 8568, 8569, 8570, -1, 8571, 8572, 8573, -1, 8574, 8575, 8576, -1, 8577, 8578, 8579, -1, 8580, 8581, 8582, -1, 8583, 8584, 8585, -1, 8586, 8587, 8588, -1, 8589, 8590, 8591, -1, 8592, 8593, 8594, -1, 8595, 8596, 8597, -1, 8598, 8599, 8600, -1, 8601, 8602, 8603, -1, 8604, 8605, 8606, -1, 8607, 8608, 8609, -1, 8610, 8611, 8612, -1, 8613, 8614, 8615, -1, 8616, 8617, 8618, -1, 8619, 8620, 8621, -1, 8622, 8623, 8624, -1, 8625, 8626, 8627, -1, 8628, 8629, 8630, -1, 8631, 8632, 8633, -1, 8634, 8635, 8636, -1, 8637, 8638, 8639, -1, 8640, 8641, 8642, -1, 8643, 8644, 8645, -1, 8646, 8647, 8648, -1, 8649, 8650, 8651, -1, 8652, 8653, 8654, -1, 8655, 8656, 8657, -1, 8658, 8659, 8660, -1, 8661, 8662, 8663, -1, 8664, 8665, 8666, -1, 8667, 8668, 8669, -1, 8670, 8671, 8672, -1, 8673, 8674, 8675, -1, 8676, 8677, 8678, -1, 8679, 8680, 8681, -1, 8682, 8683, 8684, -1, 8685, 8686, 8687, -1, 8688, 8689, 8690, -1, 8691, 8692, 8693, -1, 8694, 8695, 8696, -1, 8697, 8698, 8699, -1, 8700, 8701, 8702, -1, 8703, 8704, 8705, -1, 8706, 8707, 8708, -1, 8709, 8710, 8711, -1, 8712, 8713, 8714, -1, 8715, 8716, 8717, -1, 8718, 8719, 8720, -1, 8721, 8722, 8723, -1, 8724, 8725, 8726, -1, 8727, 8728, 8729, -1, 8730, 8731, 8732, -1, 8733, 8734, 8735, -1, 8736, 8737, 8738, -1, 8739, 8740, 8741, -1, 8742, 8743, 8744, -1, 8745, 8746, 8747, -1, 8748, 8749, 8750, -1, 8751, 8752, 8753, -1, 8754, 8755, 8756, -1, 8757, 8758, 8759, -1, 8760, 8761, 8762, -1, 8763, 8764, 8765, -1, 8766, 8767, 8768, -1, 8769, 8770, 8771, -1, 8772, 8773, 8774, -1, 8775, 8776, 8777, -1, 8778, 8779, 8780, -1, 8781, 8782, 8783, -1, 8784, 8785, 8786, -1, 8787, 8788, 8789, -1, 8790, 8791, 8792, -1, 8793, 8794, 8795, -1, 8796, 8797, 8798, -1, 8799, 8800, 8801, -1, 8802, 8803, 8804, -1, 8805, 8806, 8807, -1, 8808, 8809, 8810, -1, 8811, 8812, 8813, -1, 8814, 8815, 8816, -1, 8817, 8818, 8819, -1, 8820, 8821, 8822, -1, 8823, 8824, 8825, -1, 8826, 8827, 8828, -1, 8829, 8830, 8831, -1, 8832, 8833, 8834, -1, 8835, 8836, 8837, -1, 8838, 8839, 8840, -1, 8841, 8842, 8843, -1, 8844, 8845, 8846, -1, 8847, 8848, 8849, -1, 8850, 8851, 8852, -1, 8853, 8854, 8855, -1, 8856, 8857, 8858, -1, 8859, 8860, 8861, -1, 8862, 8863, 8864, -1, 8865, 8866, 8867, -1, 8868, 8869, 8870, -1, 8871, 8872, 8873, -1, 8874, 8875, 8876, -1, 8877, 8878, 8879, -1, 8880, 8881, 8882, -1, 8883, 8884, 8885, -1, 8886, 8887, 8888, -1, 8889, 8890, 8891, -1, 8892, 8893, 8894, -1, 8895, 8896, 8897, -1, 8898, 8899, 8900, -1, 8901, 8902, 8903, -1, 8904, 8905, 8906, -1, 8907, 8908, 8909, -1, 8910, 8911, 8912, -1, 8913, 8914, 8915, -1, 8916, 8917, 8918, -1, 8919, 8920, 8921, -1, 8922, 8923, 8924, -1, 8925, 8926, 8927, -1, 8928, 8929, 8930, -1, 8931, 8932, 8933, -1, 8934, 8935, 8936, -1, 8937, 8938, 8939, -1, 8940, 8941, 8942, -1, 8943, 8944, 8945, -1, 8946, 8947, 8948, -1, 8949, 8950, 8951, -1, 8952, 8953, 8954, -1, 8955, 8956, 8957, -1, 8958, 8959, 8960, -1, 8961, 8962, 8963, -1, 8964, 8965, 8966, -1, 8967, 8968, 8969, -1, 8970, 8971, 8972, -1, 8973, 8974, 8975, -1, 8976, 8977, 8978, -1, 8979, 8980, 8981, -1, 8982, 8983, 8984, -1, 8985, 8986, 8987, -1, 8988, 8989, 8990, -1, 8991, 8992, 8993, -1, 8994, 8995, 8996, -1, 8997, 8998, 8999, -1, 9000, 9001, 9002, -1, 9003, 9004, 9005, -1, 9006, 9007, 9008, -1, 9009, 9010, 9011, -1, 9012, 9013, 9014, -1, 9015, 9016, 9017, -1, 9018, 9019, 9020, -1, 9021, 9022, 9023, -1, 9024, 9025, 9026, -1, 9027, 9028, 9029, -1, 9030, 9031, 9032, -1, 9033, 9034, 9035, -1, 9036, 9037, 9038, -1, 9039, 9040, 9041, -1, 9042, 9043, 9044, -1, 9045, 9046, 9047, -1, 9048, 9049, 9050, -1, 9051, 9052, 9053, -1, 9054, 9055, 9056, -1, 9057, 9058, 9059, -1, 9060, 9061, 9062, -1, 9063, 9064, 9065, -1, 9066, 9067, 9068, -1, 9069, 9070, 9071, -1, 9072, 9073, 9074, -1, 9075, 9076, 9077, -1, 9078, 9079, 9080, -1, 9081, 9082, 9083, -1, 9084, 9085, 9086, -1, 9087, 9088, 9089, -1, 9090, 9091, 9092, -1, 9093, 9094, 9095, -1, 9096, 9097, 9098, -1, 9099, 9100, 9101, -1, 9102, 9103, 9104, -1, 9105, 9106, 9107, -1, 9108, 9109, 9110, -1, 9111, 9112, 9113, -1, 9114, 9115, 9116, -1, 9117, 9118, 9119, -1, 9120, 9121, 9122, -1, 9123, 9124, 9125, -1, 9126, 9127, 9128, -1, 9129, 9130, 9131, -1, 9132, 9133, 9134, -1, 9135, 9136, 9137, -1, 9138, 9139, 9140, -1, 9141, 9142, 9143, -1, 9144, 9145, 9146, -1, 9147, 9148, 9149, -1, 9150, 9151, 9152, -1, 9153, 9154, 9155, -1, 9156, 9157, 9158, -1, 9159, 9160, 9161, -1, 9162, 9163, 9164, -1, 9165, 9166, 9167, -1, 9168, 9169, 9170, -1, 9171, 9172, 9173, -1, 9174, 9175, 9176, -1, 9177, 9178, 9179, -1, 9180, 9181, 9182, -1, 9183, 9184, 9185, -1, 9186, 9187, 9188, -1, 9189, 9190, 9191, -1, 9192, 9193, 9194, -1, 9195, 9196, 9197, -1, 9198, 9199, 9200, -1, 9201, 9202, 9203, -1, 9204, 9205, 9206, -1, 9207, 9208, 9209, -1, 9210, 9211, 9212, -1, 9213, 9214, 9215, -1, 9216, 9217, 9218, -1, 9219, 9220, 9221, -1, 9222, 9223, 9224, -1, 9225, 9226, 9227, -1, 9228, 9229, 9230, -1, 9231, 9232, 9233, -1, 9234, 9235, 9236, -1, 9237, 9238, 9239, -1, 9240, 9241, 9242, -1, 9243, 9244, 9245, -1, 9246, 9247, 9248, -1, 9249, 9250, 9251, -1, 9252, 9253, 9254, -1, 9255, 9256, 9257, -1, 9258, 9259, 9260, -1, 9261, 9262, 9263, -1, 9264, 9265, 9266, -1, 9267, 9268, 9269, -1, 9270, 9271, 9272, -1, 9273, 9274, 9275, -1, 9276, 9277, 9278, -1, 9279, 9280, 9281, -1, 9282, 9283, 9284, -1, 9285, 9286, 9287, -1, 9288, 9289, 9290, -1, 9291, 9292, 9293, -1, 9294, 9295, 9296, -1, 9297, 9298, 9299, -1, 9300, 9301, 9302, -1, 9303, 9304, 9305, -1, 9306, 9307, 9308, -1, 9309, 9310, 9311, -1, 9312, 9313, 9314, -1, 9315, 9316, 9317, -1, 9318, 9319, 9320, -1, 9321, 9322, 9323, -1, 9324, 9325, 9326, -1, 9327, 9328, 9329, -1, 9330, 9331, 9332, -1, 9333, 9334, 9335, -1, 9336, 9337, 9338, -1, 9339, 9340, 9341, -1, 9342, 9343, 9344, -1, 9345, 9346, 9347, -1, 9348, 9349, 9350, -1, 9351, 9352, 9353, -1, 9354, 9355, 9356, -1, 9357, 9358, 9359, -1, 9360, 9361, 9362, -1, 9363, 9364, 9365, -1, 9366, 9367, 9368, -1, 9369, 9370, 9371, -1, 9372, 9373, 9374, -1, 9375, 9376, 9377, -1, 9378, 9379, 9380, -1, 9381, 9382, 9383, -1, 9384, 9385, 9386, -1, 9387, 9388, 9389, -1, 9390, 9391, 9392, -1, 9393, 9394, 9395, -1, 9396, 9397, 9398, -1, 9399, 9400, 9401, -1, 9402, 9403, 9404, -1, 9405, 9406, 9407, -1, 9408, 9409, 9410, -1, 9411, 9412, 9413, -1, 9414, 9415, 9416, -1, 9417, 9418, 9419, -1, 9420, 9421, 9422, -1, 9423, 9424, 9425, -1, 9426, 9427, 9428, -1, 9429, 9430, 9431, -1, 9432, 9433, 9434, -1, 9435, 9436, 9437, -1, 9438, 9439, 9440, -1, 9441, 9442, 9443, -1, 9444, 9445, 9446, -1, 9447, 9448, 9449, -1, 9450, 9451, 9452, -1, 9453, 9454, 9455, -1, 9456, 9457, 9458, -1, 9459, 9460, 9461, -1, 9462, 9463, 9464, -1, 9465, 9466, 9467, -1, 9468, 9469, 9470, -1, 9471, 9472, 9473, -1, 9474, 9475, 9476, -1, 9477, 9478, 9479, -1, 9480, 9481, 9482, -1, 9483, 9484, 9485, -1, 9486, 9487, 9488, -1, 9489, 9490, 9491, -1, 9492, 9493, 9494, -1, 9495, 9496, 9497, -1, 9498, 9499, 9500, -1, 9501, 9502, 9503, -1, 9504, 9505, 9506, -1, 9507, 9508, 9509, -1, 9510, 9511, 9512, -1, 9513, 9514, 9515, -1, 9516, 9517, 9518, -1, 9519, 9520, 9521, -1, 9522, 9523, 9524, -1, 9525, 9526, 9527, -1, 9528, 9529, 9530, -1, 9531, 9532, 9533, -1, 9534, 9535, 9536, -1, 9537, 9538, 9539, -1, 9540, 9541, 9542, -1, 9543, 9544, 9545, -1, 9546, 9547, 9548, -1, 9549, 9550, 9551, -1, 9552, 9553, 9554, -1, 9555, 9556, 9557, -1, 9558, 9559, 9560, -1, 9561, 9562, 9563, -1, 9564, 9565, 9566, -1, 9567, 9568, 9569, -1, 9570, 9571, 9572, -1, 9573, 9574, 9575, -1, 9576, 9577, 9578, -1, 9579, 9580, 9581, -1, 9582, 9583, 9584, -1, 9585, 9586, 9587, -1, 9588, 9589, 9590, -1, 9591, 9592, 9593, -1, 9594, 9595, 9596, -1, 9597, 9598, 9599, -1, 9600, 9601, 9602, -1, 9603, 9604, 9605, -1, 9606, 9607, 9608, -1, 9609, 9610, 9611, -1, 9612, 9613, 9614, -1, 9615, 9616, 9617, -1, 9618, 9619, 9620, -1, 9621, 9622, 9623, -1, 9624, 9625, 9626, -1, 9627, 9628, 9629, -1, 9630, 9631, 9632, -1, 9633, 9634, 9635, -1, 9636, 9637, 9638, -1, 9639, 9640, 9641, -1, 9642, 9643, 9644, -1, 9645, 9646, 9647, -1, 9648, 9649, 9650, -1, 9651, 9652, 9653, -1, 9654, 9655, 9656, -1, 9657, 9658, 9659, -1, 9660, 9661, 9662, -1, 9663, 9664, 9665, -1, 9666, 9667, 9668, -1, 9669, 9670, 9671, -1, 9672, 9673, 9674, -1, 9675, 9676, 9677, -1, 9678, 9679, 9680, -1, 9681, 9682, 9683, -1, 9684, 9685, 9686, -1, 9687, 9688, 9689, -1, 9690, 9691, 9692, -1, 9693, 9694, 9695, -1, 9696, 9697, 9698, -1, 9699, 9700, 9701, -1, 9702, 9703, 9704, -1, 9705, 9706, 9707, -1, 9708, 9709, 9710, -1, 9711, 9712, 9713, -1, 9714, 9715, 9716, -1, 9717, 9718, 9719, -1, 9720, 9721, 9722, -1, 9723, 9724, 9725, -1, 9726, 9727, 9728, -1, 9729, 9730, 9731, -1, 9732, 9733, 9734, -1, 9735, 9736, 9737, -1, 9738, 9739, 9740, -1, 9741, 9742, 9743, -1, 9744, 9745, 9746, -1, 9747, 9748, 9749, -1, 9750, 9751, 9752, -1, 9753, 9754, 9755, -1, 9756, 9757, 9758, -1, 9759, 9760, 9761, -1, 9762, 9763, 9764, -1, 9765, 9766, 9767, -1, 9768, 9769, 9770, -1, 9771, 9772, 9773, -1, 9774, 9775, 9776, -1, 9777, 9778, 9779, -1, 9780, 9781, 9782, -1, 9783, 9784, 9785, -1, 9786, 9787, 9788, -1, 9789, 9790, 9791, -1, 9792, 9793, 9794, -1, 9795, 9796, 9797, -1, 9798, 9799, 9800, -1, 9801, 9802, 9803, -1, 9804, 9805, 9806, -1, 9807, 9808, 9809, -1, 9810, 9811, 9812, -1, 9813, 9814, 9815, -1, 9816, 9817, 9818, -1, 9819, 9820, 9821, -1, 9822, 9823, 9824, -1, 9825, 9826, 9827, -1, 9828, 9829, 9830, -1, 9831, 9832, 9833, -1, 9834, 9835, 9836, -1, 9837, 9838, 9839, -1, 9840, 9841, 9842, -1, 9843, 9844, 9845, -1, 9846, 9847, 9848, -1, 9849, 9850, 9851, -1, 9852, 9853, 9854, -1, 9855, 9856, 9857, -1, 9858, 9859, 9860, -1, 9861, 9862, 9863, -1, 9864, 9865, 9866, -1, 9867, 9868, 9869, -1, 9870, 9871, 9872, -1, 9873, 9874, 9875, -1, 9876, 9877, 9878, -1, 9879, 9880, 9881, -1, 9882, 9883, 9884, -1, 9885, 9886, 9887, -1, 9888, 9889, 9890, -1, 9891, 9892, 9893, -1, 9894, 9895, 9896, -1, 9897, 9898, 9899, -1, 9900, 9901, 9902, -1, 9903, 9904, 9905, -1, 9906, 9907, 9908, -1, 9909, 9910, 9911, -1, 9912, 9913, 9914, -1, 9915, 9916, 9917, -1, 9918, 9919, 9920, -1, 9921, 9922, 9923, -1, 9924, 9925, 9926, -1, 9927, 9928, 9929, -1, 9930, 9931, 9932, -1, 9933, 9934, 9935, -1, 9936, 9937, 9938, -1, 9939, 9940, 9941, -1, 9942, 9943, 9944, -1, 9945, 9946, 9947, -1, 9948, 9949, 9950, -1, 9951, 9952, 9953, -1, 9954, 9955, 9956, -1, 9957, 9958, 9959, -1, 9960, 9961, 9962, -1, 9963, 9964, 9965, -1, 9966, 9967, 9968, -1, 9969, 9970, 9971, -1, 9972, 9973, 9974, -1, 9975, 9976, 9977, -1, 9978, 9979, 9980, -1, 9981, 9982, 9983, -1, 9984, 9985, 9986, -1, 9987, 9988, 9989, -1, 9990, 9991, 9992, -1, 9993, 9994, 9995, -1, 9996, 9997, 9998, -1, 9999, 10000, 10001, -1, 10002, 10003, 10004, -1, 10005, 10006, 10007, -1, 10008, 10009, 10010, -1, 10011, 10012, 10013, -1, 10014, 10015, 10016, -1, 10017, 10018, 10019, -1, 10020, 10021, 10022, -1, 10023, 10024, 10025, -1, 10026, 10027, 10028, -1, 10029, 10030, 10031, -1, 10032, 10033, 10034, -1, 10035, 10036, 10037, -1, 10038, 10039, 10040, -1, 10041, 10042, 10043, -1, 10044, 10045, 10046, -1, 10047, 10048, 10049, -1, 10050, 10051, 10052, -1, 10053, 10054, 10055, -1, 10056, 10057, 10058, -1, 10059, 10060, 10061, -1, 10062, 10063, 10064, -1, 10065, 10066, 10067, -1, 10068, 10069, 10070, -1, 10071, 10072, 10073, -1, 10074, 10075, 10076, -1, 10077, 10078, 10079, -1, 10080, 10081, 10082, -1, 10083, 10084, 10085, -1, 10086, 10087, 10088, -1, 10089, 10090, 10091, -1, 10092, 10093, 10094, -1, 10095, 10096, 10097, -1, 10098, 10099, 10100, -1, 10101, 10102, 10103, -1, 10104, 10105, 10106, -1, 10107, 10108, 10109, -1, 10110, 10111, 10112, -1, 10113, 10114, 10115, -1, 10116, 10117, 10118, -1, 10119, 10120, 10121, -1, 10122, 10123, 10124, -1, 10125, 10126, 10127, -1, 10128, 10129, 10130, -1, 10131, 10132, 10133, -1, 10134, 10135, 10136, -1, 10137, 10138, 10139, -1, 10140, 10141, 10142, -1, 10143, 10144, 10145, -1, 10146, 10147, 10148, -1, 10149, 10150, 10151, -1, 10152, 10153, 10154, -1, 10155, 10156, 10157, -1, 10158, 10159, 10160, -1, 10161, 10162, 10163, -1, 10164, 10165, 10166, -1, 10167, 10168, 10169, -1, 10170, 10171, 10172, -1, 10173, 10174, 10175, -1, 10176, 10177, 10178, -1, 10179, 10180, 10181, -1, 10182, 10183, 10184, -1, 10185, 10186, 10187, -1, 10188, 10189, 10190, -1, 10191, 10192, 10193, -1, 10194, 10195, 10196, -1, 10197, 10198, 10199, -1, 10200, 10201, 10202, -1, 10203, 10204, 10205, -1, 10206, 10207, 10208, -1, 10209, 10210, 10211, -1, 10212, 10213, 10214, -1, 10215, 10216, 10217, -1, 10218, 10219, 10220, -1, 10221, 10222, 10223, -1, 10224, 10225, 10226, -1, 10227, 10228, 10229, -1, 10230, 10231, 10232, -1, 10233, 10234, 10235, -1, 10236, 10237, 10238, -1, 10239, 10240, 10241, -1, 10242, 10243, 10244, -1, 10245, 10246, 10247, -1, 10248, 10249, 10250, -1, 10251, 10252, 10253, -1, 10254, 10255, 10256, -1, 10257, 10258, 10259, -1, 10260, 10261, 10262, -1, 10263, 10264, 10265, -1, 10266, 10267, 10268, -1, 10269, 10270, 10271, -1, 10272, 10273, 10274, -1, 10275, 10276, 10277, -1, 10278, 10279, 10280, -1, 10281, 10282, 10283, -1, 10284, 10285, 10286, -1, 10287, 10288, 10289, -1, 10290, 10291, 10292, -1, 10293, 10294, 10295, -1, 10296, 10297, 10298, -1, 10299, 10300, 10301, -1, 10302, 10303, 10304, -1, 10305, 10306, 10307, -1, 10308, 10309, 10310, -1, 10311, 10312, 10313, -1, 10314, 10315, 10316, -1, 10317, 10318, 10319, -1, 10320, 10321, 10322, -1, 10323, 10324, 10325, -1, 10326, 10327, 10328, -1, 10329, 10330, 10331, -1, 10332, 10333, 10334, -1, 10335, 10336, 10337, -1, 10338, 10339, 10340, -1, 10341, 10342, 10343, -1, 10344, 10345, 10346, -1, 10347, 10348, 10349, -1, 10350, 10351, 10352, -1, 10353, 10354, 10355, -1, 10356, 10357, 10358, -1, 10359, 10360, 10361, -1, 10362, 10363, 10364, -1, 10365, 10366, 10367, -1, 10368, 10369, 10370, -1, 10371, 10372, 10373, -1, 10374, 10375, 10376, -1, 10377, 10378, 10379, -1, 10380, 10381, 10382, -1, 10383, 10384, 10385, -1, 10386, 10387, 10388, -1, 10389, 10390, 10391, -1, 10392, 10393, 10394, -1, 10395, 10396, 10397, -1, 10398, 10399, 10400, -1, 10401, 10402, 10403, -1, 10404, 10405, 10406, -1, 10407, 10408, 10409, -1, 10410, 10411, 10412, -1, 10413, 10414, 10415, -1, 10416, 10417, 10418, -1, 10419, 10420, 10421, -1, 10422, 10423, 10424, -1, 10425, 10426, 10427, -1, 10428, 10429, 10430, -1, 10431, 10432, 10433, -1, 10434, 10435, 10436, -1, 10437, 10438, 10439, -1, 10440, 10441, 10442, -1, 10443, 10444, 10445, -1, 10446, 10447, 10448, -1, 10449, 10450, 10451, -1, 10452, 10453, 10454, -1, 10455, 10456, 10457, -1, 10458, 10459, 10460, -1, 10461, 10462, 10463, -1, 10464, 10465, 10466, -1, 10467, 10468, 10469, -1, 10470, 10471, 10472, -1, 10473, 10474, 10475, -1, 10476, 10477, 10478, -1, 10479, 10480, 10481, -1, 10482, 10483, 10484, -1, 10485, 10486, 10487, -1, 10488, 10489, 10490, -1, 10491, 10492, 10493, -1, 10494, 10495, 10496, -1, 10497, 10498, 10499, -1, 10500, 10501, 10502, -1, 10503, 10504, 10505, -1, 10506, 10507, 10508, -1, 10509, 10510, 10511, -1, 10512, 10513, 10514, -1, 10515, 10516, 10517, -1, 10518, 10519, 10520, -1, 10521, 10522, 10523, -1, 10524, 10525, 10526, -1, 10527, 10528, 10529, -1, 10530, 10531, 10532, -1, 10533, 10534, 10535, -1, 10536, 10537, 10538, -1, 10539, 10540, 10541, -1, 10542, 10543, 10544, -1, 10545, 10546, 10547, -1, 10548, 10549, 10550, -1, 10551, 10552, 10553, -1, 10554, 10555, 10556, -1, 10557, 10558, 10559, -1, 10560, 10561, 10562, -1, 10563, 10564, 10565, -1, 10566, 10567, 10568, -1, 10569, 10570, 10571, -1, 10572, 10573, 10574, -1, 10575, 10576, 10577, -1, 10578, 10579, 10580, -1, 10581, 10582, 10583, -1, 10584, 10585, 10586, -1, 10587, 10588, 10589, -1, 10590, 10591, 10592, -1, 10593, 10594, 10595, -1, 10596, 10597, 10598, -1, 10599, 10600, 10601, -1, 10602, 10603, 10604, -1, 10605, 10606, 10607, -1, 10608, 10609, 10610, -1, 10611, 10612, 10613, -1, 10614, 10615, 10616, -1, 10617, 10618, 10619, -1, 10620, 10621, 10622, -1, 10623, 10624, 10625, -1, 10626, 10627, 10628, -1, 10629, 10630, 10631, -1, 10632, 10633, 10634, -1, 10635, 10636, 10637, -1, 10638, 10639, 10640, -1, 10641, 10642, 10643, -1, 10644, 10645, 10646, -1, 10647, 10648, 10649, -1, 10650, 10651, 10652, -1, 10653, 10654, 10655, -1, 10656, 10657, 10658, -1, 10659, 10660, 10661, -1, 10662, 10663, 10664, -1, 10665, 10666, 10667, -1, 10668, 10669, 10670, -1, 10671, 10672, 10673, -1, 10674, 10675, 10676, -1, 10677, 10678, 10679, -1, 10680, 10681, 10682, -1, 10683, 10684, 10685, -1, 10686, 10687, 10688, -1, 10689, 10690, 10691, -1, 10692, 10693, 10694, -1, 10695, 10696, 10697, -1, 10698, 10699, 10700, -1, 10701, 10702, 10703, -1, 10704, 10705, 10706, -1, 10707, 10708, 10709, -1, 10710, 10711, 10712, -1, 10713, 10714, 10715, -1, 10716, 10717, 10718, -1, 10719, 10720, 10721, -1, 10722, 10723, 10724, -1, 10725, 10726, 10727, -1, 10728, 10729, 10730, -1, 10731, 10732, 10733, -1, 10734, 10735, 10736, -1, 10737, 10738, 10739, -1, 10740, 10741, 10742, -1, 10743, 10744, 10745, -1, 10746, 10747, 10748, -1, 10749, 10750, 10751, -1, 10752, 10753, 10754, -1, 10755, 10756, 10757, -1, 10758, 10759, 10760, -1, 10761, 10762, 10763, -1, 10764, 10765, 10766, -1, 10767, 10768, 10769, -1, 10770, 10771, 10772, -1, 10773, 10774, 10775, -1, 10776, 10777, 10778, -1, 10779, 10780, 10781, -1, 10782, 10783, 10784, -1, 10785, 10786, 10787, -1, 10788, 10789, 10790, -1, 10791, 10792, 10793, -1, 10794, 10795, 10796, -1, 10797, 10798, 10799, -1, 10800, 10801, 10802, -1, 10803, 10804, 10805, -1, 10806, 10807, 10808, -1, 10809, 10810, 10811, -1, 10812, 10813, 10814, -1, 10815, 10816, 10817, -1, 10818, 10819, 10820, -1, 10821, 10822, 10823, -1, 10824, 10825, 10826, -1, 10827, 10828, 10829, -1, 10830, 10831, 10832, -1, 10833, 10834, 10835, -1, 10836, 10837, 10838, -1, 10839, 10840, 10841, -1, 10842, 10843, 10844, -1, 10845, 10846, 10847, -1, 10848, 10849, 10850, -1, 10851, 10852, 10853, -1, 10854, 10855, 10856, -1, 10857, 10858, 10859, -1, 10860, 10861, 10862, -1, 10863, 10864, 10865, -1, 10866, 10867, 10868, -1, 10869, 10870, 10871, -1, 10872, 10873, 10874, -1, 10875, 10876, 10877, -1, 10878, 10879, 10880, -1, 10881, 10882, 10883, -1, 10884, 10885, 10886, -1, 10887, 10888, 10889, -1, 10890, 10891, 10892, -1, 10893, 10894, 10895, -1, 10896, 10897, 10898, -1, 10899, 10900, 10901, -1, 10902, 10903, 10904, -1, 10905, 10906, 10907, -1, 10908, 10909, 10910, -1, 10911, 10912, 10913, -1, 10914, 10915, 10916, -1, 10917, 10918, 10919, -1, 10920, 10921, 10922, -1, 10923, 10924, 10925, -1, 10926, 10927, 10928, -1, 10929, 10930, 10931, -1, 10932, 10933, 10934, -1, 10935, 10936, 10937, -1, 10938, 10939, 10940, -1, 10941, 10942, 10943, -1, 10944, 10945, 10946, -1, 10947, 10948, 10949, -1, 10950, 10951, 10952, -1, 10953, 10954, 10955, -1, 10956, 10957, 10958, -1, 10959, 10960, 10961, -1, 10962, 10963, 10964, -1, 10965, 10966, 10967, -1, 10968, 10969, 10970, -1, 10971, 10972, 10973, -1, 10974, 10975, 10976, -1, 10977, 10978, 10979, -1, 10980, 10981, 10982, -1, 10983, 10984, 10985, -1, 10986, 10987, 10988, -1, 10989, 10990, 10991, -1, 10992, 10993, 10994, -1, 10995, 10996, 10997, -1, 10998, 10999, 11000, -1, 11001, 11002, 11003, -1, 11004, 11005, 11006, -1, 11007, 11008, 11009, -1, 11010, 11011, 11012, -1, 11013, 11014, 11015, -1, 11016, 11017, 11018, -1, 11019, 11020, 11021, -1, 11022, 11023, 11024, -1, 11025, 11026, 11027, -1, 11028, 11029, 11030, -1, 11031, 11032, 11033, -1, 11034, 11035, 11036, -1, 11037, 11038, 11039, -1, 11040, 11041, 11042, -1, 11043, 11044, 11045, -1, 11046, 11047, 11048, -1, 11049, 11050, 11051, -1, 11052, 11053, 11054, -1, 11055, 11056, 11057, -1, 11058, 11059, 11060, -1, 11061, 11062, 11063, -1, 11064, 11065, 11066, -1, 11067, 11068, 11069, -1, 11070, 11071, 11072, -1, 11073, 11074, 11075, -1, 11076, 11077, 11078, -1, 11079, 11080, 11081, -1, 11082, 11083, 11084, -1, 11085, 11086, 11087, -1, 11088, 11089, 11090, -1, 11091, 11092, 11093, -1, 11094, 11095, 11096, -1, 11097, 11098, 11099, -1, 11100, 11101, 11102, -1, 11103, 11104, 11105, -1, 11106, 11107, 11108, -1, 11109, 11110, 11111, -1, 11112, 11113, 11114, -1, 11115, 11116, 11117, -1, 11118, 11119, 11120, -1, 11121, 11122, 11123, -1, 11124, 11125, 11126, -1, 11127, 11128, 11129, -1, 11130, 11131, 11132, -1, 11133, 11134, 11135, -1, 11136, 11137, 11138, -1, 11139, 11140, 11141, -1, 11142, 11143, 11144, -1, 11145, 11146, 11147, -1, 11148, 11149, 11150, -1, 11151, 11152, 11153, -1, 11154, 11155, 11156, -1, 11157, 11158, 11159, -1, 11160, 11161, 11162, -1, 11163, 11164, 11165, -1, 11166, 11167, 11168, -1, 11169, 11170, 11171, -1, 11172, 11173, 11174, -1, 11175, 11176, 11177, -1, 11178, 11179, 11180, -1, 11181, 11182, 11183, -1, 11184, 11185, 11186, -1, 11187, 11188, 11189, -1, 11190, 11191, 11192, -1, 11193, 11194, 11195, -1, 11196, 11197, 11198, -1, 11199, 11200, 11201, -1, 11202, 11203, 11204, -1, 11205, 11206, 11207, -1, 11208, 11209, 11210, -1, 11211, 11212, 11213, -1, 11214, 11215, 11216, -1, 11217, 11218, 11219, -1, 11220, 11221, 11222, -1, 11223, 11224, 11225, -1, 11226, 11227, 11228, -1, 11229, 11230, 11231, -1, 11232, 11233, 11234, -1, 11235, 11236, 11237, -1, 11238, 11239, 11240, -1, 11241, 11242, 11243, -1, 11244, 11245, 11246, -1, 11247, 11248, 11249, -1, 11250, 11251, 11252, -1, 11253, 11254, 11255, -1, 11256, 11257, 11258, -1, 11259, 11260, 11261, -1, 11262, 11263, 11264, -1, 11265, 11266, 11267, -1, 11268, 11269, 11270, -1, 11271, 11272, 11273, -1, 11274, 11275, 11276, -1, 11277, 11278, 11279, -1, 11280, 11281, 11282, -1, 11283, 11284, 11285, -1, 11286, 11287, 11288, -1, 11289, 11290, 11291, -1, 11292, 11293, 11294, -1, 11295, 11296, 11297, -1, 11298, 11299, 11300, -1, 11301, 11302, 11303, -1, 11304, 11305, 11306, -1, 11307, 11308, 11309, -1, 11310, 11311, 11312, -1, 11313, 11314, 11315, -1, 11316, 11317, 11318, -1, 11319, 11320, 11321, -1, 11322, 11323, 11324, -1, 11325, 11326, 11327, -1, 11328, 11329, 11330, -1, 11331, 11332, 11333, -1, 11334, 11335, 11336, -1, 11337, 11338, 11339, -1, 11340, 11341, 11342, -1, 11343, 11344, 11345, -1, 11346, 11347, 11348, -1, 11349, 11350, 11351, -1, 11352, 11353, 11354, -1, 11355, 11356, 11357, -1, 11358, 11359, 11360, -1, 11361, 11362, 11363, -1, 11364, 11365, 11366, -1, 11367, 11368, 11369, -1, 11370, 11371, 11372, -1, 11373, 11374, 11375, -1, 11376, 11377, 11378, -1, 11379, 11380, 11381, -1, 11382, 11383, 11384, -1, 11385, 11386, 11387, -1, 11388, 11389, 11390, -1, 11391, 11392, 11393, -1, 11394, 11395, 11396, -1, 11397, 11398, 11399, -1, 11400, 11401, 11402, -1, 11403, 11404, 11405, -1, 11406, 11407, 11408, -1, 11409, 11410, 11411, -1, 11412, 11413, 11414, -1, 11415, 11416, 11417, -1, 11418, 11419, 11420, -1, 11421, 11422, 11423, -1, 11424, 11425, 11426, -1, 11427, 11428, 11429, -1, 11430, 11431, 11432, -1, 11433, 11434, 11435, -1, 11436, 11437, 11438, -1, 11439, 11440, 11441, -1, 11442, 11443, 11444, -1, 11445, 11446, 11447, -1, 11448, 11449, 11450, -1, 11451, 11452, 11453, -1, 11454, 11455, 11456, -1, 11457, 11458, 11459, -1, 11460, 11461, 11462, -1, 11463, 11464, 11465, -1, 11466, 11467, 11468, -1, 11469, 11470, 11471, -1, 11472, 11473, 11474, -1, 11475, 11476, 11477, -1, 11478, 11479, 11480, -1, 11481, 11482, 11483, -1, 11484, 11485, 11486, -1, 11487, 11488, 11489, -1, 11490, 11491, 11492, -1, 11493, 11494, 11495, -1, 11496, 11497, 11498, -1, 11499, 11500, 11501, -1, 11502, 11503, 11504, -1, 11505, 11506, 11507, -1, 11508, 11509, 11510, -1, 11511, 11512, 11513, -1, 11514, 11515, 11516, -1, 11517, 11518, 11519, -1, 11520, 11521, 11522, -1, 11523, 11524, 11525, -1, 11526, 11527, 11528, -1, 11529, 11530, 11531, -1, 11532, 11533, 11534, -1, 11535, 11536, 11537, -1, 11538, 11539, 11540, -1, 11541, 11542, 11543, -1, 11544, 11545, 11546, -1, 11547, 11548, 11549, -1, 11550, 11551, 11552, -1, 11553, 11554, 11555, -1, 11556, 11557, 11558, -1, 11559, 11560, 11561, -1, 11562, 11563, 11564, -1, 11565, 11566, 11567, -1, 11568, 11569, 11570, -1, 11571, 11572, 11573, -1, 11574, 11575, 11576, -1, 11577, 11578, 11579, -1, 11580, 11581, 11582, -1, 11583, 11584, 11585, -1, 11586, 11587, 11588, -1, 11589, 11590, 11591, -1, 11592, 11593, 11594, -1, 11595, 11596, 11597, -1, 11598, 11599, 11600, -1, 11601, 11602, 11603, -1, 11604, 11605, 11606, -1, 11607, 11608, 11609, -1, 11610, 11611, 11612, -1, 11613, 11614, 11615, -1, 11616, 11617, 11618, -1, 11619, 11620, 11621, -1, 11622, 11623, 11624, -1, 11625, 11626, 11627, -1, 11628, 11629, 11630, -1, 11631, 11632, 11633, -1, 11634, 11635, 11636, -1, 11637, 11638, 11639, -1, 11640, 11641, 11642, -1, 11643, 11644, 11645, -1, 11646, 11647, 11648, -1, 11649, 11650, 11651, -1, 11652, 11653, 11654, -1, 11655, 11656, 11657, -1, 11658, 11659, 11660, -1, 11661, 11662, 11663, -1, 11664, 11665, 11666, -1, 11667, 11668, 11669, -1, 11670, 11671, 11672, -1, 11673, 11674, 11675, -1, 11676, 11677, 11678, -1, 11679, 11680, 11681, -1, 11682, 11683, 11684, -1, 11685, 11686, 11687, -1, 11688, 11689, 11690, -1, 11691, 11692, 11693, -1, 11694, 11695, 11696, -1, 11697, 11698, 11699, -1, 11700, 11701, 11702, -1, 11703, 11704, 11705, -1, 11706, 11707, 11708, -1, 11709, 11710, 11711, -1, 11712, 11713, 11714, -1, 11715, 11716, 11717, -1, 11718, 11719, 11720, -1, 11721, 11722, 11723, -1, 11724, 11725, 11726, -1, 11727, 11728, 11729, -1, 11730, 11731, 11732, -1, 11733, 11734, 11735, -1, 11736, 11737, 11738, -1, 11739, 11740, 11741, -1, 11742, 11743, 11744, -1, 11745, 11746, 11747, -1, 11748, 11749, 11750, -1, 11751, 11752, 11753, -1, 11754, 11755, 11756, -1, 11757, 11758, 11759, -1, 11760, 11761, 11762, -1, 11763, 11764, 11765, -1, 11766, 11767, 11768, -1, 11769, 11770, 11771, -1, 11772, 11773, 11774, -1, 11775, 11776, 11777, -1, 11778, 11779, 11780, -1, 11781, 11782, 11783, -1, 11784, 11785, 11786, -1, 11787, 11788, 11789, -1, 11790, 11791, 11792, -1, 11793, 11794, 11795, -1, 11796, 11797, 11798, -1, 11799, 11800, 11801, -1, 11802, 11803, 11804, -1, 11805, 11806, 11807, -1, 11808, 11809, 11810, -1, 11811, 11812, 11813, -1, 11814, 11815, 11816, -1, 11817, 11818, 11819, -1, 11820, 11821, 11822, -1, 11823, 11824, 11825, -1, 11826, 11827, 11828, -1, 11829, 11830, 11831, -1, 11832, 11833, 11834, -1, 11835, 11836, 11837, -1, 11838, 11839, 11840, -1, 11841, 11842, 11843, -1, 11844, 11845, 11846, -1, 11847, 11848, 11849, -1, 11850, 11851, 11852, -1, 11853, 11854, 11855, -1, 11856, 11857, 11858, -1, 11859, 11860, 11861, -1, 11862, 11863, 11864, -1, 11865, 11866, 11867, -1, 11868, 11869, 11870, -1, 11871, 11872, 11873, -1, 11874, 11875, 11876, -1, 11877, 11878, 11879, -1, 11880, 11881, 11882, -1, 11883, 11884, 11885, -1, 11886, 11887, 11888, -1, 11889, 11890, 11891, -1, 11892, 11893, 11894, -1, 11895, 11896, 11897, -1, 11898, 11899, 11900, -1, 11901, 11902, 11903, -1, 11904, 11905, 11906, -1, 11907, 11908, 11909, -1, 11910, 11911, 11912, -1, 11913, 11914, 11915, -1, 11916, 11917, 11918, -1, 11919, 11920, 11921, -1, 11922, 11923, 11924, -1, 11925, 11926, 11927, -1, 11928, 11929, 11930, -1, 11931, 11932, 11933, -1, 11934, 11935, 11936, -1, 11937, 11938, 11939, -1, 11940, 11941, 11942, -1, 11943, 11944, 11945, -1, 11946, 11947, 11948, -1, 11949, 11950, 11951, -1, 11952, 11953, 11954, -1, 11955, 11956, 11957, -1, 11958, 11959, 11960, -1, 11961, 11962, 11963, -1, 11964, 11965, 11966, -1, 11967, 11968, 11969, -1, 11970, 11971, 11972, -1, 11973, 11974, 11975, -1, 11976, 11977, 11978, -1, 11979, 11980, 11981, -1, 11982, 11983, 11984, -1, 11985, 11986, 11987, -1, 11988, 11989, 11990, -1, 11991, 11992, 11993, -1, 11994, 11995, 11996, -1, 11997, 11998, 11999, -1, 12000, 12001, 12002, -1, 12003, 12004, 12005, -1, 12006, 12007, 12008, -1, 12009, 12010, 12011, -1, 12012, 12013, 12014, -1, 12015, 12016, 12017, -1, 12018, 12019, 12020, -1, 12021, 12022, 12023, -1, 12024, 12025, 12026, -1, 12027, 12028, 12029, -1, 12030, 12031, 12032, -1, 12033, 12034, 12035, -1, 12036, 12037, 12038, -1, 12039, 12040, 12041, -1, 12042, 12043, 12044, -1, 12045, 12046, 12047, -1, 12048, 12049, 12050, -1, 12051, 12052, 12053, -1, 12054, 12055, 12056, -1, 12057, 12058, 12059, -1, 12060, 12061, 12062, -1, 12063, 12064, 12065, -1, 12066, 12067, 12068, -1, 12069, 12070, 12071, -1, 12072, 12073, 12074, -1, 12075, 12076, 12077, -1, 12078, 12079, 12080, -1, 12081, 12082, 12083, -1, 12084, 12085, 12086, -1, 12087, 12088, 12089, -1, 12090, 12091, 12092, -1, 12093, 12094, 12095, -1, 12096, 12097, 12098, -1, 12099, 12100, 12101, -1, 12102, 12103, 12104, -1, 12105, 12106, 12107, -1, 12108, 12109, 12110, -1, 12111, 12112, 12113, -1, 12114, 12115, 12116, -1, 12117, 12118, 12119, -1, 12120, 12121, 12122, -1, 12123, 12124, 12125, -1, 12126, 12127, 12128, -1, 12129, 12130, 12131, -1, 12132, 12133, 12134, -1, 12135, 12136, 12137, -1, 12138, 12139, 12140, -1, 12141, 12142, 12143, -1, 12144, 12145, 12146, -1, 12147, 12148, 12149, -1, 12150, 12151, 12152, -1, 12153, 12154, 12155, -1, 12156, 12157, 12158, -1, 12159, 12160, 12161, -1, 12162, 12163, 12164, -1, 12165, 12166, 12167, -1, 12168, 12169, 12170, -1, 12171, 12172, 12173, -1, 12174, 12175, 12176, -1, 12177, 12178, 12179, -1, 12180, 12181, 12182, -1, 12183, 12184, 12185, -1, 12186, 12187, 12188, -1, 12189, 12190, 12191, -1, 12192, 12193, 12194, -1, 12195, 12196, 12197, -1, 12198, 12199, 12200, -1, 12201, 12202, 12203, -1, 12204, 12205, 12206, -1, 12207, 12208, 12209, -1, 12210, 12211, 12212, -1, 12213, 12214, 12215, -1, 12216, 12217, 12218, -1, 12219, 12220, 12221, -1, 12222, 12223, 12224, -1, 12225, 12226, 12227, -1, 12228, 12229, 12230, -1, 12231, 12232, 12233, -1, 12234, 12235, 12236, -1, 12237, 12238, 12239, -1, 12240, 12241, 12242, -1, 12243, 12244, 12245, -1, 12246, 12247, 12248, -1, 12249, 12250, 12251, -1, 12252, 12253, 12254, -1, 12255, 12256, 12257, -1, 12258, 12259, 12260, -1, 12261, 12262, 12263, -1, 12264, 12265, 12266, -1, 12267, 12268, 12269, -1, 12270, 12271, 12272, -1, 12273, 12274, 12275, -1, 12276, 12277, 12278, -1, 12279, 12280, 12281, -1, 12282, 12283, 12284, -1, 12285, 12286, 12287, -1, 12288, 12289, 12290, -1, 12291, 12292, 12293, -1, 12294, 12295, 12296, -1, 12297, 12298, 12299, -1, 12300, 12301, 12302, -1, 12303, 12304, 12305, -1, 12306, 12307, 12308, -1, 12309, 12310, 12311, -1, 12312, 12313, 12314, -1, 12315, 12316, 12317, -1, 12318, 12319, 12320, -1, 12321, 12322, 12323, -1, 12324, 12325, 12326, -1, 12327, 12328, 12329, -1, 12330, 12331, 12332, -1, 12333, 12334, 12335, -1, 12336, 12337, 12338, -1, 12339, 12340, 12341, -1, 12342, 12343, 12344, -1, 12345, 12346, 12347, -1, 12348, 12349, 12350, -1, 12351, 12352, 12353, -1, 12354, 12355, 12356, -1, 12357, 12358, 12359, -1, 12360, 12361, 12362, -1, 12363, 12364, 12365, -1, 12366, 12367, 12368, -1, 12369, 12370, 12371, -1, 12372, 12373, 12374, -1, 12375, 12376, 12377, -1, 12378, 12379, 12380, -1, 12381, 12382, 12383, -1, 12384, 12385, 12386, -1, 12387, 12388, 12389, -1, 12390, 12391, 12392, -1, 12393, 12394, 12395, -1, 12396, 12397, 12398, -1, 12399, 12400, 12401, -1, 12402, 12403, 12404, -1, 12405, 12406, 12407, -1, 12408, 12409, 12410, -1, 12411, 12412, 12413, -1, 12414, 12415, 12416, -1, 12417, 12418, 12419, -1, 12420, 12421, 12422, -1, 12423, 12424, 12425, -1, 12426, 12427, 12428, -1, 12429, 12430, 12431, -1, 12432, 12433, 12434, -1, 12435, 12436, 12437, -1, 12438, 12439, 12440, -1, 12441, 12442, 12443, -1, 12444, 12445, 12446, -1, 12447, 12448, 12449, -1, 12450, 12451, 12452, -1, 12453, 12454, 12455, -1, 12456, 12457, 12458, -1, 12459, 12460, 12461, -1, 12462, 12463, 12464, -1, 12465, 12466, 12467, -1, 12468, 12469, 12470, -1, 12471, 12472, 12473, -1, 12474, 12475, 12476, -1, 12477, 12478, 12479, -1, 12480, 12481, 12482, -1, 12483, 12484, 12485, -1, 12486, 12487, 12488, -1, 12489, 12490, 12491, -1, 12492, 12493, 12494, -1, 12495, 12496, 12497, -1, 12498, 12499, 12500, -1, 12501, 12502, 12503, -1, 12504, 12505, 12506, -1, 12507, 12508, 12509, -1, 12510, 12511, 12512, -1, 12513, 12514, 12515, -1, 12516, 12517, 12518, -1, 12519, 12520, 12521, -1, 12522, 12523, 12524, -1, 12525, 12526, 12527, -1, 12528, 12529, 12530, -1, 12531, 12532, 12533, -1, 12534, 12535, 12536, -1, 12537, 12538, 12539, -1, 12540, 12541, 12542, -1, 12543, 12544, 12545, -1, 12546, 12547, 12548, -1, 12549, 12550, 12551, -1, 12552, 12553, 12554, -1, 12555, 12556, 12557, -1, 12558, 12559, 12560, -1, 12561, 12562, 12563, -1, 12564, 12565, 12566, -1, 12567, 12568, 12569, -1, 12570, 12571, 12572, -1, 12573, 12574, 12575, -1, 12576, 12577, 12578, -1, 12579, 12580, 12581, -1, 12582, 12583, 12584, -1, 12585, 12586, 12587, -1, 12588, 12589, 12590, -1, 12591, 12592, 12593, -1, 12594, 12595, 12596, -1, 12597, 12598, 12599, -1, 12600, 12601, 12602, -1, 12603, 12604, 12605, -1, 12606, 12607, 12608, -1, 12609, 12610, 12611, -1, 12612, 12613, 12614, -1, 12615, 12616, 12617, -1, 12618, 12619, 12620, -1, 12621, 12622, 12623, -1, 12624, 12625, 12626, -1, 12627, 12628, 12629, -1, 12630, 12631, 12632, -1, 12633, 12634, 12635, -1, 12636, 12637, 12638, -1, 12639, 12640, 12641, -1, 12642, 12643, 12644, -1, 12645, 12646, 12647, -1, 12648, 12649, 12650, -1, 12651, 12652, 12653, -1, 12654, 12655, 12656, -1, 12657, 12658, 12659, -1, 12660, 12661, 12662, -1, 12663, 12664, 12665, -1, 12666, 12667, 12668, -1, 12669, 12670, 12671, -1, 12672, 12673, 12674, -1, 12675, 12676, 12677, -1, 12678, 12679, 12680, -1, 12681, 12682, 12683, -1, 12684, 12685, 12686, -1, 12687, 12688, 12689, -1, 12690, 12691, 12692, -1, 12693, 12694, 12695, -1, 12696, 12697, 12698, -1, 12699, 12700, 12701, -1, 12702, 12703, 12704, -1, 12705, 12706, 12707, -1, 12708, 12709, 12710, -1, 12711, 12712, 12713, -1, 12714, 12715, 12716, -1, 12717, 12718, 12719, -1, 12720, 12721, 12722, -1, 12723, 12724, 12725, -1, 12726, 12727, 12728, -1, 12729, 12730, 12731, -1, 12732, 12733, 12734, -1, 12735, 12736, 12737, -1, 12738, 12739, 12740, -1, 12741, 12742, 12743, -1, 12744, 12745, 12746, -1, 12747, 12748, 12749, -1, 12750, 12751, 12752, -1, 12753, 12754, 12755, -1, 12756, 12757, 12758, -1, 12759, 12760, 12761, -1, 12762, 12763, 12764, -1, 12765, 12766, 12767, -1, 12768, 12769, 12770, -1, 12771, 12772, 12773, -1, 12774, 12775, 12776, -1, 12777, 12778, 12779, -1, 12780, 12781, 12782, -1, 12783, 12784, 12785, -1, 12786, 12787, 12788, -1, 12789, 12790, 12791, -1, 12792, 12793, 12794, -1, 12795, 12796, 12797, -1, 12798, 12799, 12800, -1, 12801, 12802, 12803, -1, 12804, 12805, 12806, -1, 12807, 12808, 12809, -1, 12810, 12811, 12812, -1, 12813, 12814, 12815, -1, 12816, 12817, 12818, -1, 12819, 12820, 12821, -1, 12822, 12823, 12824, -1, 12825, 12826, 12827, -1, 12828, 12829, 12830, -1, 12831, 12832, 12833, -1, 12834, 12835, 12836, -1, 12837, 12838, 12839, -1, 12840, 12841, 12842, -1, 12843, 12844, 12845, -1, 12846, 12847, 12848, -1, 12849, 12850, 12851, -1, 12852, 12853, 12854, -1, 12855, 12856, 12857, -1, 12858, 12859, 12860, -1, 12861, 12862, 12863, -1, 12864, 12865, 12866, -1, 12867, 12868, 12869, -1, 12870, 12871, 12872, -1, 12873, 12874, 12875, -1, 12876, 12877, 12878, -1, 12879, 12880, 12881, -1, 12882, 12883, 12884, -1, 12885, 12886, 12887, -1, 12888, 12889, 12890, -1, 12891, 12892, 12893, -1, 12894, 12895, 12896, -1, 12897, 12898, 12899, -1, 12900, 12901, 12902, -1, 12903, 12904, 12905, -1, 12906, 12907, 12908, -1, 12909, 12910, 12911, -1, 12912, 12913, 12914, -1, 12915, 12916, 12917, -1, 12918, 12919, 12920, -1, 12921, 12922, 12923, -1, 12924, 12925, 12926, -1, 12927, 12928, 12929, -1, 12930, 12931, 12932, -1, 12933, 12934, 12935, -1, 12936, 12937, 12938, -1, 12939, 12940, 12941, -1, 12942, 12943, 12944, -1, 12945, 12946, 12947, -1, 12948, 12949, 12950, -1, 12951, 12952, 12953, -1, 12954, 12955, 12956, -1, 12957, 12958, 12959, -1, 12960, 12961, 12962, -1, 12963, 12964, 12965, -1, 12966, 12967, 12968, -1, 12969, 12970, 12971, -1, 12972, 12973, 12974, -1, 12975, 12976, 12977, -1, 12978, 12979, 12980, -1, 12981, 12982, 12983, -1, 12984, 12985, 12986, -1, 12987, 12988, 12989, -1, 12990, 12991, 12992, -1, 12993, 12994, 12995, -1, 12996, 12997, 12998, -1, 12999, 13000, 13001, -1, 13002, 13003, 13004, -1, 13005, 13006, 13007, -1, 13008, 13009, 13010, -1, 13011, 13012, 13013, -1, 13014, 13015, 13016, -1, 13017, 13018, 13019, -1, 13020, 13021, 13022, -1, 13023, 13024, 13025, -1, 13026, 13027, 13028, -1, 13029, 13030, 13031, -1, 13032, 13033, 13034, -1, 13035, 13036, 13037, -1, 13038, 13039, 13040, -1, 13041, 13042, 13043, -1, 13044, 13045, 13046, -1, 13047, 13048, 13049, -1, 13050, 13051, 13052, -1, 13053, 13054, 13055, -1, 13056, 13057, 13058, -1, 13059, 13060, 13061, -1, 13062, 13063, 13064, -1, 13065, 13066, 13067, -1, 13068, 13069, 13070, -1, 13071, 13072, 13073, -1, 13074, 13075, 13076, -1, 13077, 13078, 13079, -1, 13080, 13081, 13082, -1, 13083, 13084, 13085, -1, 13086, 13087, 13088, -1, 13089, 13090, 13091, -1, 13092, 13093, 13094, -1, 13095, 13096, 13097, -1, 13098, 13099, 13100, -1, 13101, 13102, 13103, -1, 13104, 13105, 13106, -1, 13107, 13108, 13109, -1, 13110, 13111, 13112, -1, 13113, 13114, 13115, -1, 13116, 13117, 13118, -1, 13119, 13120, 13121, -1, 13122, 13123, 13124, -1, 13125, 13126, 13127, -1, 13128, 13129, 13130, -1, 13131, 13132, 13133, -1, 13134, 13135, 13136, -1, 13137, 13138, 13139, -1, 13140, 13141, 13142, -1, 13143, 13144, 13145, -1, 13146, 13147, 13148, -1, 13149, 13150, 13151, -1, 13152, 13153, 13154, -1, 13155, 13156, 13157, -1, 13158, 13159, 13160, -1, 13161, 13162, 13163, -1, 13164, 13165, 13166, -1, 13167, 13168, 13169, -1, 13170, 13171, 13172, -1, 13173, 13174, 13175, -1, 13176, 13177, 13178, -1, 13179, 13180, 13181, -1, 13182, 13183, 13184, -1, 13185, 13186, 13187, -1, 13188, 13189, 13190, -1, 13191, 13192, 13193, -1, 13194, 13195, 13196, -1, 13197, 13198, 13199, -1, 13200, 13201, 13202, -1, 13203, 13204, 13205, -1, 13206, 13207, 13208, -1, 13209, 13210, 13211, -1, 13212, 13213, 13214, -1, 13215, 13216, 13217, -1, 13218, 13219, 13220, -1, 13221, 13222, 13223, -1, 13224, 13225, 13226, -1, 13227, 13228, 13229, -1, 13230, 13231, 13232, -1, 13233, 13234, 13235, -1, 13236, 13237, 13238, -1, 13239, 13240, 13241, -1, 13242, 13243, 13244, -1, 13245, 13246, 13247, -1, 13248, 13249, 13250, -1, 13251, 13252, 13253, -1, 13254, 13255, 13256, -1, 13257, 13258, 13259, -1, 13260, 13261, 13262, -1, 13263, 13264, 13265, -1, 13266, 13267, 13268, -1, 13269, 13270, 13271, -1, 13272, 13273, 13274, -1, 13275, 13276, 13277, -1, 13278, 13279, 13280, -1, 13281, 13282, 13283, -1, 13284, 13285, 13286, -1, 13287, 13288, 13289, -1, 13290, 13291, 13292, -1, 13293, 13294, 13295, -1, 13296, 13297, 13298, -1, 13299, 13300, 13301, -1, 13302, 13303, 13304, -1, 13305, 13306, 13307, -1, 13308, 13309, 13310, -1, 13311, 13312, 13313, -1, 13314, 13315, 13316, -1, 13317, 13318, 13319, -1, 13320, 13321, 13322, -1, 13323, 13324, 13325, -1, 13326, 13327, 13328, -1, 13329, 13330, 13331, -1, 13332, 13333, 13334, -1, 13335, 13336, 13337, -1, 13338, 13339, 13340, -1, 13341, 13342, 13343, -1, 13344, 13345, 13346, -1, 13347, 13348, 13349, -1, 13350, 13351, 13352, -1, 13353, 13354, 13355, -1, 13356, 13357, 13358, -1, 13359, 13360, 13361, -1, 13362, 13363, 13364, -1, 13365, 13366, 13367, -1, 13368, 13369, 13370, -1, 13371, 13372, 13373, -1, 13374, 13375, 13376, -1, 13377, 13378, 13379, -1, 13380, 13381, 13382, -1, 13383, 13384, 13385, -1, 13386, 13387, 13388, -1, 13389, 13390, 13391, -1, 13392, 13393, 13394, -1, 13395, 13396, 13397, -1, 13398, 13399, 13400, -1, 13401, 13402, 13403, -1, 13404, 13405, 13406, -1, 13407, 13408, 13409, -1, 13410, 13411, 13412, -1, 13413, 13414, 13415, -1, 13416, 13417, 13418, -1, 13419, 13420, 13421, -1, 13422, 13423, 13424, -1, 13425, 13426, 13427, -1, 13428, 13429, 13430, -1, 13431, 13432, 13433, -1, 13434, 13435, 13436, -1, 13437, 13438, 13439, -1, 13440, 13441, 13442, -1, 13443, 13444, 13445, -1, 13446, 13447, 13448, -1, 13449, 13450, 13451, -1, 13452, 13453, 13454, -1, 13455, 13456, 13457, -1, 13458, 13459, 13460, -1, 13461, 13462, 13463, -1, 13464, 13465, 13466, -1, 13467, 13468, 13469, -1, 13470, 13471, 13472, -1, 13473, 13474, 13475, -1, 13476, 13477, 13478, -1, 13479, 13480, 13481, -1, 13482, 13483, 13484, -1, 13485, 13486, 13487, -1, 13488, 13489, 13490, -1, 13491, 13492, 13493, -1, 13494, 13495, 13496, -1, 13497, 13498, 13499, -1, 13500, 13501, 13502, -1, 13503, 13504, 13505, -1, 13506, 13507, 13508, -1, 13509, 13510, 13511, -1, 13512, 13513, 13514, -1, 13515, 13516, 13517, -1, 13518, 13519, 13520, -1, 13521, 13522, 13523, -1, 13524, 13525, 13526, -1, 13527, 13528, 13529, -1, 13530, 13531, 13532, -1, 13533, 13534, 13535, -1, 13536, 13537, 13538, -1, 13539, 13540, 13541, -1, 13542, 13543, 13544, -1, 13545, 13546, 13547, -1, 13548, 13549, 13550, -1, 13551, 13552, 13553, -1, 13554, 13555, 13556, -1, 13557, 13558, 13559, -1, 13560, 13561, 13562, -1, 13563, 13564, 13565, -1, 13566, 13567, 13568, -1, 13569, 13570, 13571, -1, 13572, 13573, 13574, -1, 13575, 13576, 13577, -1, 13578, 13579, 13580, -1, 13581, 13582, 13583, -1, 13584, 13585, 13586, -1, 13587, 13588, 13589, -1, 13590, 13591, 13592, -1, 13593, 13594, 13595, -1, 13596, 13597, 13598, -1, 13599, 13600, 13601, -1, 13602, 13603, 13604, -1, 13605, 13606, 13607, -1, 13608, 13609, 13610, -1, 13611, 13612, 13613, -1, 13614, 13615, 13616, -1, 13617, 13618, 13619, -1, 13620, 13621, 13622, -1, 13623, 13624, 13625, -1, 13626, 13627, 13628, -1, 13629, 13630, 13631, -1, 13632, 13633, 13634, -1, 13635, 13636, 13637, -1, 13638, 13639, 13640, -1, 13641, 13642, 13643, -1, 13644, 13645, 13646, -1, 13647, 13648, 13649, -1, 13650, 13651, 13652, -1, 13653, 13654, 13655, -1, 13656, 13657, 13658, -1, 13659, 13660, 13661, -1, 13662, 13663, 13664, -1, 13665, 13666, 13667, -1, 13668, 13669, 13670, -1, 13671, 13672, 13673, -1, 13674, 13675, 13676, -1, 13677, 13678, 13679, -1, 13680, 13681, 13682, -1, 13683, 13684, 13685, -1, 13686, 13687, 13688, -1, 13689, 13690, 13691, -1, 13692, 13693, 13694, -1, 13695, 13696, 13697, -1, 13698, 13699, 13700, -1, 13701, 13702, 13703, -1, 13704, 13705, 13706, -1, 13707, 13708, 13709, -1, 13710, 13711, 13712, -1, 13713, 13714, 13715, -1, 13716, 13717, 13718, -1, 13719, 13720, 13721, -1, 13722, 13723, 13724, -1, 13725, 13726, 13727, -1, 13728, 13729, 13730, -1, 13731, 13732, 13733, -1, 13734, 13735, 13736, -1, 13737, 13738, 13739, -1, 13740, 13741, 13742, -1, 13743, 13744, 13745, -1, 13746, 13747, 13748, -1, 13749, 13750, 13751, -1, 13752, 13753, 13754, -1, 13755, 13756, 13757, -1, 13758, 13759, 13760, -1, 13761, 13762, 13763, -1, 13764, 13765, 13766, -1, 13767, 13768, 13769, -1, 13770, 13771, 13772, -1, 13773, 13774, 13775, -1, 13776, 13777, 13778, -1, 13779, 13780, 13781, -1, 13782, 13783, 13784, -1, 13785, 13786, 13787, -1, 13788, 13789, 13790, -1, 13791, 13792, 13793, -1, 13794, 13795, 13796, -1, 13797, 13798, 13799, -1, 13800, 13801, 13802, -1, 13803, 13804, 13805, -1, 13806, 13807, 13808, -1, 13809, 13810, 13811, -1, 13812, 13813, 13814, -1, 13815, 13816, 13817, -1, 13818, 13819, 13820, -1, 13821, 13822, 13823, -1, 13824, 13825, 13826, -1, 13827, 13828, 13829, -1, 13830, 13831, 13832, -1, 13833, 13834, 13835, -1, 13836, 13837, 13838, -1, 13839, 13840, 13841, -1, 13842, 13843, 13844, -1, 13845, 13846, 13847, -1, 13848, 13849, 13850, -1, 13851, 13852, 13853, -1, 13854, 13855, 13856, -1, 13857, 13858, 13859, -1, 13860, 13861, 13862, -1, 13863, 13864, 13865, -1, 13866, 13867, 13868, -1, 13869, 13870, 13871, -1, 13872, 13873, 13874, -1, 13875, 13876, 13877, -1, 13878, 13879, 13880, -1, 13881, 13882, 13883, -1, 13884, 13885, 13886, -1, 13887, 13888, 13889, -1, 13890, 13891, 13892, -1, 13893, 13894, 13895, -1, 13896, 13897, 13898, -1, 13899, 13900, 13901, -1, 13902, 13903, 13904, -1, 13905, 13906, 13907, -1, 13908, 13909, 13910, -1, 13911, 13912, 13913, -1, 13914, 13915, 13916, -1, 13917, 13918, 13919, -1, 13920, 13921, 13922, -1, 13923, 13924, 13925, -1, 13926, 13927, 13928, -1, 13929, 13930, 13931, -1, 13932, 13933, 13934, -1, 13935, 13936, 13937, -1, 13938, 13939, 13940, -1, 13941, 13942, 13943, -1, 13944, 13945, 13946, -1, 13947, 13948, 13949, -1, 13950, 13951, 13952, -1, 13953, 13954, 13955, -1, 13956, 13957, 13958, -1, 13959, 13960, 13961, -1, 13962, 13963, 13964, -1, 13965, 13966, 13967, -1, 13968, 13969, 13970, -1, 13971, 13972, 13973, -1, 13974, 13975, 13976, -1, 13977, 13978, 13979, -1, 13980, 13981, 13982, -1, 13983, 13984, 13985, -1, 13986, 13987, 13988, -1, 13989, 13990, 13991, -1, 13992, 13993, 13994, -1, 13995, 13996, 13997, -1, 13998, 13999, 14000, -1, 14001, 14002, 14003, -1, 14004, 14005, 14006, -1, 14007, 14008, 14009, -1, 14010, 14011, 14012, -1, 14013, 14014, 14015, -1, 14016, 14017, 14018, -1, 14019, 14020, 14021, -1, 14022, 14023, 14024, -1, 14025, 14026, 14027, -1, 14028, 14029, 14030, -1, 14031, 14032, 14033, -1, 14034, 14035, 14036, -1, 14037, 14038, 14039, -1, 14040, 14041, 14042, -1, 14043, 14044, 14045, -1, 14046, 14047, 14048, -1, 14049, 14050, 14051, -1, 14052, 14053, 14054, -1, 14055, 14056, 14057, -1, 14058, 14059, 14060, -1, 14061, 14062, 14063, -1, 14064, 14065, 14066, -1, 14067, 14068, 14069, -1, 14070, 14071, 14072, -1, 14073, 14074, 14075, -1, 14076, 14077, 14078, -1, 14079, 14080, 14081, -1, 14082, 14083, 14084, -1, 14085, 14086, 14087, -1, 14088, 14089, 14090, -1, 14091, 14092, 14093, -1, 14094, 14095, 14096, -1, 14097, 14098, 14099, -1, 14100, 14101, 14102, -1, 14103, 14104, 14105, -1, 14106, 14107, 14108, -1, 14109, 14110, 14111, -1, 14112, 14113, 14114, -1, 14115, 14116, 14117, -1, 14118, 14119, 14120, -1, 14121, 14122, 14123, -1, 14124, 14125, 14126, -1, 14127, 14128, 14129, -1, 14130, 14131, 14132, -1, 14133, 14134, 14135, -1, 14136, 14137, 14138, -1, 14139, 14140, 14141, -1, 14142, 14143, 14144, -1, 14145, 14146, 14147, -1, 14148, 14149, 14150, -1, 14151, 14152, 14153, -1, 14154, 14155, 14156, -1, 14157, 14158, 14159, -1, 14160, 14161, 14162, -1, 14163, 14164, 14165, -1, 14166, 14167, 14168, -1, 14169, 14170, 14171, -1, 14172, 14173, 14174, -1, 14175, 14176, 14177, -1, 14178, 14179, 14180, -1, 14181, 14182, 14183, -1, 14184, 14185, 14186, -1, 14187, 14188, 14189, -1, 14190, 14191, 14192, -1, 14193, 14194, 14195, -1, 14196, 14197, 14198, -1, 14199, 14200, 14201, -1, 14202, 14203, 14204, -1, 14205, 14206, 14207, -1, 14208, 14209, 14210, -1, 14211, 14212, 14213, -1, 14214, 14215, 14216, -1, 14217, 14218, 14219, -1, 14220, 14221, 14222, -1, 14223, 14224, 14225, -1, 14226, 14227, 14228, -1, 14229, 14230, 14231, -1, 14232, 14233, 14234, -1, 14235, 14236, 14237, -1, 14238, 14239, 14240, -1, 14241, 14242, 14243, -1, 14244, 14245, 14246, -1, 14247, 14248, 14249, -1, 14250, 14251, 14252, -1, 14253, 14254, 14255, -1, 14256, 14257, 14258, -1, 14259, 14260, 14261, -1, 14262, 14263, 14264, -1, 14265, 14266, 14267, -1, 14268, 14269, 14270, -1, 14271, 14272, 14273, -1, 14274, 14275, 14276, -1, 14277, 14278, 14279, -1, 14280, 14281, 14282, -1, 14283, 14284, 14285, -1, 14286, 14287, 14288, -1, 14289, 14290, 14291, -1, 14292, 14293, 14294, -1, 14295, 14296, 14297, -1, 14298, 14299, 14300, -1, 14301, 14302, 14303, -1, 14304, 14305, 14306, -1, 14307, 14308, 14309, -1, 14310, 14311, 14312, -1, 14313, 14314, 14315, -1, 14316, 14317, 14318, -1, 14319, 14320, 14321, -1, 14322, 14323, 14324, -1, 14325, 14326, 14327, -1, 14328, 14329, 14330, -1, 14331, 14332, 14333, -1, 14334, 14335, 14336, -1, 14337, 14338, 14339, -1, 14340, 14341, 14342, -1, 14343, 14344, 14345, -1, 14346, 14347, 14348, -1, 14349, 14350, 14351, -1, 14352, 14353, 14354, -1, 14355, 14356, 14357, -1, 14358, 14359, 14360, -1, 14361, 14362, 14363, -1, 14364, 14365, 14366, -1, 14367, 14368, 14369, -1, 14370, 14371, 14372, -1, 14373, 14374, 14375, -1, 14376, 14377, 14378, -1, 14379, 14380, 14381, -1, 14382, 14383, 14384, -1, 14385, 14386, 14387, -1, 14388, 14389, 14390, -1, 14391, 14392, 14393, -1, 14394, 14395, 14396, -1, 14397, 14398, 14399, -1, 14400, 14401, 14402, -1, 14403, 14404, 14405, -1, 14406, 14407, 14408, -1, 14409, 14410, 14411, -1, 14412, 14413, 14414, -1, 14415, 14416, 14417, -1, 14418, 14419, 14420, -1, 14421, 14422, 14423, -1, 14424, 14425, 14426, -1, 14427, 14428, 14429, -1, 14430, 14431, 14432, -1, 14433, 14434, 14435, -1, 14436, 14437, 14438, -1, 14439, 14440, 14441, -1, 14442, 14443, 14444, -1, 14445, 14446, 14447, -1, 14448, 14449, 14450, -1, 14451, 14452, 14453, -1, 14454, 14455, 14456, -1, 14457, 14458, 14459, -1, 14460, 14461, 14462, -1, 14463, 14464, 14465, -1, 14466, 14467, 14468, -1, 14469, 14470, 14471, -1, 14472, 14473, 14474, -1, 14475, 14476, 14477, -1, 14478, 14479, 14480, -1, 14481, 14482, 14483, -1, 14484, 14485, 14486, -1, 14487, 14488, 14489, -1, 14490, 14491, 14492, -1, 14493, 14494, 14495, -1, 14496, 14497, 14498, -1, 14499, 14500, 14501, -1, 14502, 14503, 14504, -1, 14505, 14506, 14507, -1, 14508, 14509, 14510, -1, 14511, 14512, 14513, -1, 14514, 14515, 14516, -1, 14517, 14518, 14519, -1, 14520, 14521, 14522, -1, 14523, 14524, 14525, -1, 14526, 14527, 14528, -1, 14529, 14530, 14531, -1, 14532, 14533, 14534, -1, 14535, 14536, 14537, -1, 14538, 14539, 14540, -1, 14541, 14542, 14543, -1, 14544, 14545, 14546, -1, 14547, 14548, 14549, -1, 14550, 14551, 14552, -1, 14553, 14554, 14555, -1, 14556, 14557, 14558, -1, 14559, 14560, 14561, -1, 14562, 14563, 14564, -1, 14565, 14566, 14567, -1, 14568, 14569, 14570, -1, 14571, 14572, 14573, -1, 14574, 14575, 14576, -1, 14577, 14578, 14579, -1, 14580, 14581, 14582, -1, 14583, 14584, 14585, -1, 14586, 14587, 14588, -1, 14589, 14590, 14591, -1, 14592, 14593, 14594, -1, 14595, 14596, 14597, -1, 14598, 14599, 14600, -1, 14601, 14602, 14603, -1, 14604, 14605, 14606, -1, 14607, 14608, 14609, -1, 14610, 14611, 14612, -1, 14613, 14614, 14615, -1, 14616, 14617, 14618, -1, 14619, 14620, 14621, -1, 14622, 14623, 14624, -1, 14625, 14626, 14627, -1, 14628, 14629, 14630, -1, 14631, 14632, 14633, -1, 14634, 14635, 14636, -1, 14637, 14638, 14639, -1, 14640, 14641, 14642, -1, 14643, 14644, 14645, -1, 14646, 14647, 14648, -1, 14649, 14650, 14651, -1, 14652, 14653, 14654, -1, 14655, 14656, 14657, -1, 14658, 14659, 14660, -1, 14661, 14662, 14663, -1, 14664, 14665, 14666, -1, 14667, 14668, 14669, -1, 14670, 14671, 14672, -1, 14673, 14674, 14675, -1, 14676, 14677, 14678, -1, 14679, 14680, 14681, -1, 14682, 14683, 14684, -1, 14685, 14686, 14687, -1, 14688, 14689, 14690, -1, 14691, 14692, 14693, -1, 14694, 14695, 14696, -1, 14697, 14698, 14699, -1, 14700, 14701, 14702, -1, 14703, 14704, 14705, -1, 14706, 14707, 14708, -1, 14709, 14710, 14711, -1, 14712, 14713, 14714, -1, 14715, 14716, 14717, -1, 14718, 14719, 14720, -1, 14721, 14722, 14723, -1, 14724, 14725, 14726, -1, 14727, 14728, 14729, -1, 14730, 14731, 14732, -1, 14733, 14734, 14735, -1, 14736, 14737, 14738, -1, 14739, 14740, 14741, -1, 14742, 14743, 14744, -1, 14745, 14746, 14747, -1, 14748, 14749, 14750, -1, 14751, 14752, 14753, -1, 14754, 14755, 14756, -1, 14757, 14758, 14759, -1, 14760, 14761, 14762, -1, 14763, 14764, 14765, -1, 14766, 14767, 14768, -1, 14769, 14770, 14771, -1, 14772, 14773, 14774, -1, 14775, 14776, 14777, -1, 14778, 14779, 14780, -1, 14781, 14782, 14783, -1, 14784, 14785, 14786, -1, 14787, 14788, 14789, -1, 14790, 14791, 14792, -1, 14793, 14794, 14795, -1, 14796, 14797, 14798, -1, 14799, 14800, 14801, -1, 14802, 14803, 14804, -1, 14805, 14806, 14807, -1, 14808, 14809, 14810, -1, 14811, 14812, 14813, -1, 14814, 14815, 14816, -1, 14817, 14818, 14819, -1, 14820, 14821, 14822, -1, 14823, 14824, 14825, -1, 14826, 14827, 14828, -1, 14829, 14830, 14831, -1, 14832, 14833, 14834, -1, 14835, 14836, 14837, -1, 14838, 14839, 14840, -1, 14841, 14842, 14843, -1, 14844, 14845, 14846, -1, 14847, 14848, 14849, -1, 14850, 14851, 14852, -1, 14853, 14854, 14855, -1, 14856, 14857, 14858, -1, 14859, 14860, 14861, -1, 14862, 14863, 14864, -1, 14865, 14866, 14867, -1, 14868, 14869, 14870, -1, 14871, 14872, 14873, -1, 14874, 14875, 14876, -1, 14877, 14878, 14879, -1, 14880, 14881, 14882, -1, 14883, 14884, 14885, -1, 14886, 14887, 14888, -1, 14889, 14890, 14891, -1, 14892, 14893, 14894, -1, 14895, 14896, 14897, -1, 14898, 14899, 14900, -1, 14901, 14902, 14903, -1, 14904, 14905, 14906, -1, 14907, 14908, 14909, -1, 14910, 14911, 14912, -1, 14913, 14914, 14915, -1, 14916, 14917, 14918, -1, 14919, 14920, 14921, -1, 14922, 14923, 14924, -1, 14925, 14926, 14927, -1, 14928, 14929, 14930, -1, 14931, 14932, 14933, -1, 14934, 14935, 14936, -1, 14937, 14938, 14939, -1, 14940, 14941, 14942, -1, 14943, 14944, 14945, -1, 14946, 14947, 14948, -1, 14949, 14950, 14951, -1, 14952, 14953, 14954, -1, 14955, 14956, 14957, -1, 14958, 14959, 14960, -1, 14961, 14962, 14963, -1, 14964, 14965, 14966, -1, 14967, 14968, 14969, -1, 14970, 14971, 14972, -1, 14973, 14974, 14975, -1, 14976, 14977, 14978, -1, 14979, 14980, 14981, -1, 14982, 14983, 14984, -1, 14985, 14986, 14987, -1, 14988, 14989, 14990, -1, 14991, 14992, 14993, -1, 14994, 14995, 14996, -1, 14997, 14998, 14999, -1, 15000, 15001, 15002, -1, 15003, 15004, 15005, -1, 15006, 15007, 15008, -1, 15009, 15010, 15011, -1, 15012, 15013, 15014, -1, 15015, 15016, 15017, -1, 15018, 15019, 15020, -1, 15021, 15022, 15023, -1, 15024, 15025, 15026, -1, 15027, 15028, 15029, -1, 15030, 15031, 15032, -1, 15033, 15034, 15035, -1, 15036, 15037, 15038, -1, 15039, 15040, 15041, -1, 15042, 15043, 15044, -1, 15045, 15046, 15047, -1, 15048, 15049, 15050, -1, 15051, 15052, 15053, -1, 15054, 15055, 15056, -1, 15057, 15058, 15059, -1, 15060, 15061, 15062, -1, 15063, 15064, 15065, -1, 15066, 15067, 15068, -1, 15069, 15070, 15071, -1, 15072, 15073, 15074, -1, 15075, 15076, 15077, -1, 15078, 15079, 15080, -1, 15081, 15082, 15083, -1, 15084, 15085, 15086, -1, 15087, 15088, 15089, -1, 15090, 15091, 15092, -1, 15093, 15094, 15095, -1, 15096, 15097, 15098, -1, 15099, 15100, 15101, -1, 15102, 15103, 15104, -1, 15105, 15106, 15107, -1, 15108, 15109, 15110, -1, 15111, 15112, 15113, -1, 15114, 15115, 15116, -1, 15117, 15118, 15119, -1, 15120, 15121, 15122, -1, 15123, 15124, 15125, -1, 15126, 15127, 15128, -1, 15129, 15130, 15131, -1, 15132, 15133, 15134, -1, 15135, 15136, 15137, -1, 15138, 15139, 15140, -1, 15141, 15142, 15143, -1, 15144, 15145, 15146, -1, 15147, 15148, 15149, -1, 15150, 15151, 15152, -1, 15153, 15154, 15155, -1, 15156, 15157, 15158, -1, 15159, 15160, 15161, -1, 15162, 15163, 15164, -1, 15165, 15166, 15167, -1, 15168, 15169, 15170, -1, 15171, 15172, 15173, -1, 15174, 15175, 15176, -1, 15177, 15178, 15179, -1, 15180, 15181, 15182, -1, 15183, 15184, 15185, -1, 15186, 15187, 15188, -1, 15189, 15190, 15191, -1, 15192, 15193, 15194, -1, 15195, 15196, 15197, -1, 15198, 15199, 15200, -1, 15201, 15202, 15203, -1, 15204, 15205, 15206, -1, 15207, 15208, 15209, -1, 15210, 15211, 15212, -1, 15213, 15214, 15215, -1, 15216, 15217, 15218, -1, 15219, 15220, 15221, -1, 15222, 15223, 15224, -1, 15225, 15226, 15227, -1, 15228, 15229, 15230, -1, 15231, 15232, 15233, -1, 15234, 15235, 15236, -1, 15237, 15238, 15239, -1, 15240, 15241, 15242, -1, 15243, 15244, 15245, -1, 15246, 15247, 15248, -1, 15249, 15250, 15251, -1, 15252, 15253, 15254, -1, 15255, 15256, 15257, -1, 15258, 15259, 15260, -1, 15261, 15262, 15263, -1, 15264, 15265, 15266, -1, 15267, 15268, 15269, -1, 15270, 15271, 15272, -1, 15273, 15274, 15275, -1, 15276, 15277, 15278, -1, 15279, 15280, 15281, -1, 15282, 15283, 15284, -1, 15285, 15286, 15287, -1, 15288, 15289, 15290, -1, 15291, 15292, 15293, -1, 15294, 15295, 15296, -1, 15297, 15298, 15299, -1, 15300, 15301, 15302, -1, 15303, 15304, 15305, -1, 15306, 15307, 15308, -1, 15309, 15310, 15311, -1, 15312, 15313, 15314, -1, 15315, 15316, 15317, -1, 15318, 15319, 15320, -1, 15321, 15322, 15323, -1, 15324, 15325, 15326, -1, 15327, 15328, 15329, -1, 15330, 15331, 15332, -1, 15333, 15334, 15335, -1, 15336, 15337, 15338, -1, 15339, 15340, 15341, -1, 15342, 15343, 15344, -1, 15345, 15346, 15347, -1, 15348, 15349, 15350, -1, 15351, 15352, 15353, -1, 15354, 15355, 15356, -1, 15357, 15358, 15359, -1, 15360, 15361, 15362, -1, 15363, 15364, 15365, -1, 15366, 15367, 15368, -1, 15369, 15370, 15371, -1, 15372, 15373, 15374, -1, 15375, 15376, 15377, -1, 15378, 15379, 15380, -1, 15381, 15382, 15383, -1, 15384, 15385, 15386, -1, 15387, 15388, 15389, -1, 15390, 15391, 15392, -1, 15393, 15394, 15395, -1, 15396, 15397, 15398, -1, 15399, 15400, 15401, -1, 15402, 15403, 15404, -1, 15405, 15406, 15407, -1, 15408, 15409, 15410, -1, 15411, 15412, 15413, -1, 15414, 15415, 15416, -1, 15417, 15418, 15419, -1, 15420, 15421, 15422, -1, 15423, 15424, 15425, -1, 15426, 15427, 15428, -1, 15429, 15430, 15431, -1, 15432, 15433, 15434, -1, 15435, 15436, 15437, -1, 15438, 15439, 15440, -1, 15441, 15442, 15443, -1, 15444, 15445, 15446, -1, 15447, 15448, 15449, -1, 15450, 15451, 15452, -1, 15453, 15454, 15455, -1, 15456, 15457, 15458, -1, 15459, 15460, 15461, -1, 15462, 15463, 15464, -1, 15465, 15466, 15467, -1, 15468, 15469, 15470, -1, 15471, 15472, 15473, -1, 15474, 15475, 15476, -1, 15477, 15478, 15479, -1, 15480, 15481, 15482, -1, 15483, 15484, 15485, -1, 15486, 15487, 15488, -1, 15489, 15490, 15491, -1, 15492, 15493, 15494, -1, 15495, 15496, 15497, -1, 15498, 15499, 15500, -1, 15501, 15502, 15503, -1, 15504, 15505, 15506, -1, 15507, 15508, 15509, -1, 15510, 15511, 15512, -1, 15513, 15514, 15515, -1, 15516, 15517, 15518, -1, 15519, 15520, 15521, -1, 15522, 15523, 15524, -1, 15525, 15526, 15527, -1, 15528, 15529, 15530, -1, 15531, 15532, 15533, -1, 15534, 15535, 15536, -1, 15537, 15538, 15539, -1, 15540, 15541, 15542, -1, 15543, 15544, 15545, -1, 15546, 15547, 15548, -1, 15549, 15550, 15551, -1, 15552, 15553, 15554, -1, 15555, 15556, 15557, -1, 15558, 15559, 15560, -1, 15561, 15562, 15563, -1, 15564, 15565, 15566, -1, 15567, 15568, 15569, -1, 15570, 15571, 15572, -1, 15573, 15574, 15575, -1, 15576, 15577, 15578, -1, 15579, 15580, 15581, -1, 15582, 15583, 15584, -1, 15585, 15586, 15587, -1, 15588, 15589, 15590, -1, 15591, 15592, 15593, -1, 15594, 15595, 15596, -1, 15597, 15598, 15599, -1, 15600, 15601, 15602, -1, 15603, 15604, 15605, -1, 15606, 15607, 15608, -1, 15609, 15610, 15611, -1, 15612, 15613, 15614, -1, 15615, 15616, 15617, -1, 15618, 15619, 15620, -1, 15621, 15622, 15623, -1, 15624, 15625, 15626, -1, 15627, 15628, 15629, -1, 15630, 15631, 15632, -1, 15633, 15634, 15635, -1, 15636, 15637, 15638, -1, 15639, 15640, 15641, -1, 15642, 15643, 15644, -1, 15645, 15646, 15647, -1, 15648, 15649, 15650, -1, 15651, 15652, 15653, -1, 15654, 15655, 15656, -1, 15657, 15658, 15659, -1, 15660, 15661, 15662, -1, 15663, 15664, 15665, -1, 15666, 15667, 15668, -1, 15669, 15670, 15671, -1, 15672, 15673, 15674, -1, 15675, 15676, 15677, -1, 15678, 15679, 15680, -1, 15681, 15682, 15683, -1, 15684, 15685, 15686, -1, 15687, 15688, 15689, -1, 15690, 15691, 15692, -1, 15693, 15694, 15695, -1, 15696, 15697, 15698, -1, 15699, 15700, 15701, -1, 15702, 15703, 15704, -1, 15705, 15706, 15707, -1, 15708, 15709, 15710, -1, 15711, 15712, 15713, -1, 15714, 15715, 15716, -1, 15717, 15718, 15719, -1, 15720, 15721, 15722, -1, 15723, 15724, 15725, -1, 15726, 15727, 15728, -1, 15729, 15730, 15731, -1, 15732, 15733, 15734, -1, 15735, 15736, 15737, -1, 15738, 15739, 15740, -1, 15741, 15742, 15743, -1, 15744, 15745, 15746, -1, 15747, 15748, 15749, -1, 15750, 15751, 15752, -1, 15753, 15754, 15755, -1, 15756, 15757, 15758, -1, 15759, 15760, 15761, -1, 15762, 15763, 15764, -1, 15765, 15766, 15767, -1, 15768, 15769, 15770, -1, 15771, 15772, 15773, -1, 15774, 15775, 15776, -1, 15777, 15778, 15779, -1, 15780, 15781, 15782, -1, 15783, 15784, 15785, -1, 15786, 15787, 15788, -1, 15789, 15790, 15791, -1, 15792, 15793, 15794, -1, 15795, 15796, 15797, -1, 15798, 15799, 15800, -1, 15801, 15802, 15803, -1, 15804, 15805, 15806, -1, 15807, 15808, 15809, -1, 15810, 15811, 15812, -1, 15813, 15814, 15815, -1, 15816, 15817, 15818, -1, 15819, 15820, 15821, -1, 15822, 15823, 15824, -1, 15825, 15826, 15827, -1, 15828, 15829, 15830, -1, 15831, 15832, 15833, -1, 15834, 15835, 15836, -1, 15837, 15838, 15839, -1, 15840, 15841, 15842, -1, 15843, 15844, 15845, -1, 15846, 15847, 15848, -1, 15849, 15850, 15851, -1, 15852, 15853, 15854, -1, 15855, 15856, 15857, -1, 15858, 15859, 15860, -1, 15861, 15862, 15863, -1, 15864, 15865, 15866, -1, 15867, 15868, 15869, -1, 15870, 15871, 15872, -1, 15873, 15874, 15875, -1, 15876, 15877, 15878, -1, 15879, 15880, 15881, -1, 15882, 15883, 15884, -1, 15885, 15886, 15887, -1, 15888, 15889, 15890, -1, 15891, 15892, 15893, -1, 15894, 15895, 15896, -1, 15897, 15898, 15899, -1, 15900, 15901, 15902, -1, 15903, 15904, 15905, -1, 15906, 15907, 15908, -1, 15909, 15910, 15911, -1, 15912, 15913, 15914, -1, 15915, 15916, 15917, -1, 15918, 15919, 15920, -1, 15921, 15922, 15923, -1, 15924, 15925, 15926, -1, 15927, 15928, 15929, -1, 15930, 15931, 15932, -1, 15933, 15934, 15935, -1, 15936, 15937, 15938, -1, 15939, 15940, 15941, -1, 15942, 15943, 15944, -1, 15945, 15946, 15947, -1, 15948, 15949, 15950, -1, 15951, 15952, 15953, -1, 15954, 15955, 15956, -1, 15957, 15958, 15959, -1, 15960, 15961, 15962, -1, 15963, 15964, 15965, -1, 15966, 15967, 15968, -1, 15969, 15970, 15971, -1, 15972, 15973, 15974, -1, 15975, 15976, 15977, -1, 15978, 15979, 15980, -1, 15981, 15982, 15983, -1, 15984, 15985, 15986, -1, 15987, 15988, 15989, -1, 15990, 15991, 15992, -1, 15993, 15994, 15995, -1, 15996, 15997, 15998, -1, 15999, 16000, 16001, -1, 16002, 16003, 16004, -1, 16005, 16006, 16007, -1, 16008, 16009, 16010, -1, 16011, 16012, 16013, -1, 16014, 16015, 16016, -1, 16017, 16018, 16019, -1, 16020, 16021, 16022, -1, 16023, 16024, 16025, -1, 16026, 16027, 16028, -1, 16029, 16030, 16031, -1, 16032, 16033, 16034, -1, 16035, 16036, 16037, -1, 16038, 16039, 16040, -1, 16041, 16042, 16043, -1, 16044, 16045, 16046, -1, 16047, 16048, 16049, -1, 16050, 16051, 16052, -1, 16053, 16054, 16055, -1, 16056, 16057, 16058, -1, 16059, 16060, 16061, -1, 16062, 16063, 16064, -1, 16065, 16066, 16067, -1, 16068, 16069, 16070, -1, 16071, 16072, 16073, -1, 16074, 16075, 16076, -1, 16077, 16078, 16079, -1, 16080, 16081, 16082, -1, 16083, 16084, 16085, -1, 16086, 16087, 16088, -1, 16089, 16090, 16091, -1, 16092, 16093, 16094, -1, 16095, 16096, 16097, -1, 16098, 16099, 16100, -1, 16101, 16102, 16103, -1, 16104, 16105, 16106, -1, 16107, 16108, 16109, -1, 16110, 16111, 16112, -1, 16113, 16114, 16115, -1, 16116, 16117, 16118, -1, 16119, 16120, 16121, -1, 16122, 16123, 16124, -1, 16125, 16126, 16127, -1, 16128, 16129, 16130, -1, 16131, 16132, 16133, -1, 16134, 16135, 16136, -1, 16137, 16138, 16139, -1, 16140, 16141, 16142, -1, 16143, 16144, 16145, -1, 16146, 16147, 16148, -1, 16149, 16150, 16151, -1, 16152, 16153, 16154, -1, 16155, 16156, 16157, -1, 16158, 16159, 16160, -1, 16161, 16162, 16163, -1, 16164, 16165, 16166, -1, 16167, 16168, 16169, -1, 16170, 16171, 16172, -1, 16173, 16174, 16175, -1, 16176, 16177, 16178, -1, 16179, 16180, 16181, -1, 16182, 16183, 16184, -1, 16185, 16186, 16187, -1, 16188, 16189, 16190, -1, 16191, 16192, 16193, -1, 16194, 16195, 16196, -1, 16197, 16198, 16199, -1, 16200, 16201, 16202, -1, 16203, 16204, 16205, -1, 16206, 16207, 16208, -1, 16209, 16210, 16211, -1, 16212, 16213, 16214, -1, 16215, 16216, 16217, -1, 16218, 16219, 16220, -1, 16221, 16222, 16223, -1, 16224, 16225, 16226, -1, 16227, 16228, 16229, -1, 16230, 16231, 16232, -1, 16233, 16234, 16235, -1, 16236, 16237, 16238, -1, 16239, 16240, 16241, -1, 16242, 16243, 16244, -1, 16245, 16246, 16247, -1, 16248, 16249, 16250, -1, 16251, 16252, 16253, -1, 16254, 16255, 16256, -1, 16257, 16258, 16259, -1, 16260, 16261, 16262, -1, 16263, 16264, 16265, -1, 16266, 16267, 16268, -1, 16269, 16270, 16271, -1, 16272, 16273, 16274, -1, 16275, 16276, 16277, -1, 16278, 16279, 16280, -1, 16281, 16282, 16283, -1, 16284, 16285, 16286, -1, 16287, 16288, 16289, -1, 16290, 16291, 16292, -1, 16293, 16294, 16295, -1, 16296, 16297, 16298, -1, 16299, 16300, 16301, -1, 16302, 16303, 16304, -1, 16305, 16306, 16307, -1, 16308, 16309, 16310, -1, 16311, 16312, 16313, -1, 16314, 16315, 16316, -1, 16317, 16318, 16319, -1, 16320, 16321, 16322, -1, 16323, 16324, 16325, -1, 16326, 16327, 16328, -1, 16329, 16330, 16331, -1, 16332, 16333, 16334, -1, 16335, 16336, 16337, -1, 16338, 16339, 16340, -1, 16341, 16342, 16343, -1, 16344, 16345, 16346, -1, 16347, 16348, 16349, -1, 16350, 16351, 16352, -1, 16353, 16354, 16355, -1, 16356, 16357, 16358, -1, 16359, 16360, 16361, -1, 16362, 16363, 16364, -1, 16365, 16366, 16367, -1, 16368, 16369, 16370, -1, 16371, 16372, 16373, -1, 16374, 16375, 16376, -1, 16377, 16378, 16379, -1, 16380, 16381, 16382, -1, 16383, 16384, 16385, -1, 16386, 16387, 16388, -1, 16389, 16390, 16391, -1, 16392, 16393, 16394, -1, 16395, 16396, 16397, -1, 16398, 16399, 16400, -1, 16401, 16402, 16403, -1, 16404, 16405, 16406, -1, 16407, 16408, 16409, -1, 16410, 16411, 16412, -1, 16413, 16414, 16415, -1, 16416, 16417, 16418, -1, 16419, 16420, 16421, -1, 16422, 16423, 16424, -1, 16425, 16426, 16427, -1, 16428, 16429, 16430, -1, 16431, 16432, 16433, -1, 16434, 16435, 16436, -1, 16437, 16438, 16439, -1, 16440, 16441, 16442, -1, 16443, 16444, 16445, -1, 16446, 16447, 16448, -1, 16449, 16450, 16451, -1, 16452, 16453, 16454, -1, 16455, 16456, 16457, -1, 16458, 16459, 16460, -1, 16461, 16462, 16463, -1, 16464, 16465, 16466, -1, 16467, 16468, 16469, -1, 16470, 16471, 16472, -1, 16473, 16474, 16475, -1, 16476, 16477, 16478, -1, 16479, 16480, 16481, -1, 16482, 16483, 16484, -1, 16485, 16486, 16487, -1, 16488, 16489, 16490, -1, 16491, 16492, 16493, -1, 16494, 16495, 16496, -1, 16497, 16498, 16499, -1, 16500, 16501, 16502, -1, 16503, 16504, 16505, -1, 16506, 16507, 16508, -1, 16509, 16510, 16511, -1, 16512, 16513, 16514, -1, 16515, 16516, 16517, -1, 16518, 16519, 16520, -1, 16521, 16522, 16523, -1, 16524, 16525, 16526, -1, 16527, 16528, 16529, -1, 16530, 16531, 16532, -1, 16533, 16534, 16535, -1, 16536, 16537, 16538, -1, 16539, 16540, 16541, -1, 16542, 16543, 16544, -1, 16545, 16546, 16547, -1, 16548, 16549, 16550, -1, 16551, 16552, 16553, -1, 16554, 16555, 16556, -1, 16557, 16558, 16559, -1, 16560, 16561, 16562, -1, 16563, 16564, 16565, -1, 16566, 16567, 16568, -1, 16569, 16570, 16571, -1, 16572, 16573, 16574, -1, 16575, 16576, 16577, -1, 16578, 16579, 16580, -1, 16581, 16582, 16583, -1, 16584, 16585, 16586, -1, 16587, 16588, 16589, -1, 16590, 16591, 16592, -1, 16593, 16594, 16595, -1, 16596, 16597, 16598, -1, 16599, 16600, 16601, -1, 16602, 16603, 16604, -1, 16605, 16606, 16607, -1, 16608, 16609, 16610, -1, 16611, 16612, 16613, -1, 16614, 16615, 16616, -1, 16617, 16618, 16619, -1, 16620, 16621, 16622, -1, 16623, 16624, 16625, -1, 16626, 16627, 16628, -1, 16629, 16630, 16631, -1, 16632, 16633, 16634, -1, 16635, 16636, 16637, -1, 16638, 16639, 16640, -1, 16641, 16642, 16643, -1, 16644, 16645, 16646, -1, 16647, 16648, 16649, -1, 16650, 16651, 16652, -1, 16653, 16654, 16655, -1, 16656, 16657, 16658, -1, 16659, 16660, 16661, -1, 16662, 16663, 16664, -1, 16665, 16666, 16667, -1, 16668, 16669, 16670, -1, 16671, 16672, 16673, -1, 16674, 16675, 16676, -1, 16677, 16678, 16679, -1, 16680, 16681, 16682, -1, 16683, 16684, 16685, -1, 16686, 16687, 16688, -1, 16689, 16690, 16691, -1, 16692, 16693, 16694, -1, 16695, 16696, 16697, -1, 16698, 16699, 16700, -1, 16701, 16702, 16703, -1, 16704, 16705, 16706, -1, 16707, 16708, 16709, -1, 16710, 16711, 16712, -1, 16713, 16714, 16715, -1, 16716, 16717, 16718, -1, 16719, 16720, 16721, -1, 16722, 16723, 16724, -1, 16725, 16726, 16727, -1, 16728, 16729, 16730, -1, 16731, 16732, 16733, -1, 16734, 16735, 16736, -1, 16737, 16738, 16739, -1, 16740, 16741, 16742, -1, 16743, 16744, 16745, -1, 16746, 16747, 16748, -1, 16749, 16750, 16751, -1, 16752, 16753, 16754, -1, 16755, 16756, 16757, -1, 16758, 16759, 16760, -1, 16761, 16762, 16763, -1, 16764, 16765, 16766, -1, 16767, 16768, 16769, -1, 16770, 16771, 16772, -1, 16773, 16774, 16775, -1, 16776, 16777, 16778, -1, 16779, 16780, 16781, -1, 16782, 16783, 16784, -1, 16785, 16786, 16787, -1, 16788, 16789, 16790, -1, 16791, 16792, 16793, -1, 16794, 16795, 16796, -1, 16797, 16798, 16799, -1, 16800, 16801, 16802, -1, 16803, 16804, 16805, -1, 16806, 16807, 16808, -1, 16809, 16810, 16811, -1, 16812, 16813, 16814, -1, 16815, 16816, 16817, -1, 16818, 16819, 16820, -1, 16821, 16822, 16823, -1, 16824, 16825, 16826, -1, 16827, 16828, 16829, -1, 16830, 16831, 16832, -1, 16833, 16834, 16835, -1, 16836, 16837, 16838, -1, 16839, 16840, 16841, -1, 16842, 16843, 16844, -1, 16845, 16846, 16847, -1, 16848, 16849, 16850, -1, 16851, 16852, 16853, -1, 16854, 16855, 16856, -1, 16857, 16858, 16859, -1, 16860, 16861, 16862, -1, 16863, 16864, 16865, -1, 16866, 16867, 16868, -1, 16869, 16870, 16871, -1, 16872, 16873, 16874, -1, 16875, 16876, 16877, -1, 16878, 16879, 16880, -1, 16881, 16882, 16883, -1, 16884, 16885, 16886, -1, 16887, 16888, 16889, -1, 16890, 16891, 16892, -1, 16893, 16894, 16895, -1, 16896, 16897, 16898, -1, 16899, 16900, 16901, -1, 16902, 16903, 16904, -1, 16905, 16906, 16907, -1, 16908, 16909, 16910, -1, 16911, 16912, 16913, -1, 16914, 16915, 16916, -1, 16917, 16918, 16919, -1, 16920, 16921, 16922, -1, 16923, 16924, 16925, -1, 16926, 16927, 16928, -1, 16929, 16930, 16931, -1, 16932, 16933, 16934, -1, 16935, 16936, 16937, -1, 16938, 16939, 16940, -1, 16941, 16942, 16943, -1, 16944, 16945, 16946, -1, 16947, 16948, 16949, -1, 16950, 16951, 16952, -1, 16953, 16954, 16955, -1, 16956, 16957, 16958, -1, 16959, 16960, 16961, -1, 16962, 16963, 16964, -1, 16965, 16966, 16967, -1, 16968, 16969, 16970, -1, 16971, 16972, 16973, -1, 16974, 16975, 16976, -1, 16977, 16978, 16979, -1, 16980, 16981, 16982, -1, 16983, 16984, 16985, -1, 16986, 16987, 16988, -1, 16989, 16990, 16991, -1, 16992, 16993, 16994, -1, 16995, 16996, 16997, -1, 16998, 16999, 17000, -1, 17001, 17002, 17003, -1, 17004, 17005, 17006, -1, 17007, 17008, 17009, -1, 17010, 17011, 17012, -1, 17013, 17014, 17015, -1, 17016, 17017, 17018, -1, 17019, 17020, 17021, -1, 17022, 17023, 17024, -1, 17025, 17026, 17027, -1, 17028, 17029, 17030, -1, 17031, 17032, 17033, -1, 17034, 17035, 17036, -1, 17037, 17038, 17039, -1, 17040, 17041, 17042, -1, 17043, 17044, 17045, -1, 17046, 17047, 17048, -1, 17049, 17050, 17051, -1, 17052, 17053, 17054, -1, 17055, 17056, 17057, -1, 17058, 17059, 17060, -1, 17061, 17062, 17063, -1, 17064, 17065, 17066, -1, 17067, 17068, 17069, -1, 17070, 17071, 17072, -1, 17073, 17074, 17075, -1, 17076, 17077, 17078, -1, 17079, 17080, 17081, -1, 17082, 17083, 17084, -1, 17085, 17086, 17087, -1, 17088, 17089, 17090, -1, 17091, 17092, 17093, -1, 17094, 17095, 17096, -1, 17097, 17098, 17099, -1, 17100, 17101, 17102, -1, 17103, 17104, 17105, -1, 17106, 17107, 17108, -1, 17109, 17110, 17111, -1, 17112, 17113, 17114, -1, 17115, 17116, 17117, -1, 17118, 17119, 17120, -1, 17121, 17122, 17123, -1, 17124, 17125, 17126, -1, 17127, 17128, 17129, -1, 17130, 17131, 17132, -1, 17133, 17134, 17135, -1, 17136, 17137, 17138, -1, 17139, 17140, 17141, -1, 17142, 17143, 17144, -1, 17145, 17146, 17147, -1, 17148, 17149, 17150, -1, 17151, 17152, 17153, -1, 17154, 17155, 17156, -1, 17157, 17158, 17159, -1, 17160, 17161, 17162, -1, 17163, 17164, 17165, -1, 17166, 17167, 17168, -1, 17169, 17170, 17171, -1, 17172, 17173, 17174, -1, 17175, 17176, 17177, -1, 17178, 17179, 17180, -1, 17181, 17182, 17183, -1, 17184, 17185, 17186, -1, 17187, 17188, 17189, -1, 17190, 17191, 17192, -1, 17193, 17194, 17195, -1, 17196, 17197, 17198, -1, 17199, 17200, 17201, -1, 17202, 17203, 17204, -1, 17205, 17206, 17207, -1, 17208, 17209, 17210, -1, 17211, 17212, 17213, -1, 17214, 17215, 17216, -1, 17217, 17218, 17219, -1, 17220, 17221, 17222, -1, 17223, 17224, 17225, -1, 17226, 17227, 17228, -1, 17229, 17230, 17231, -1, 17232, 17233, 17234, -1, 17235, 17236, 17237, -1, 17238, 17239, 17240, -1, 17241, 17242, 17243, -1, 17244, 17245, 17246, -1, 17247, 17248, 17249, -1, 17250, 17251, 17252, -1, 17253, 17254, 17255, -1, 17256, 17257, 17258, -1, 17259, 17260, 17261, -1, 17262, 17263, 17264, -1, 17265, 17266, 17267, -1, 17268, 17269, 17270, -1, 17271, 17272, 17273, -1, 17274, 17275, 17276, -1, 17277, 17278, 17279, -1, 17280, 17281, 17282, -1, 17283, 17284, 17285, -1, 17286, 17287, 17288, -1, 17289, 17290, 17291, -1, 17292, 17293, 17294, -1, 17295, 17296, 17297, -1, 17298, 17299, 17300, -1, 17301, 17302, 17303, -1, 17304, 17305, 17306, -1, 17307, 17308, 17309, -1, 17310, 17311, 17312, -1, 17313, 17314, 17315, -1, 17316, 17317, 17318, -1, 17319, 17320, 17321, -1, 17322, 17323, 17324, -1, 17325, 17326, 17327, -1, 17328, 17329, 17330, -1, 17331, 17332, 17333, -1, 17334, 17335, 17336, -1, 17337, 17338, 17339, -1, 17340, 17341, 17342, -1, 17343, 17344, 17345, -1, 17346, 17347, 17348, -1, 17349, 17350, 17351, -1, 17352, 17353, 17354, -1, 17355, 17356, 17357, -1, 17358, 17359, 17360, -1, 17361, 17362, 17363, -1, 17364, 17365, 17366, -1, 17367, 17368, 17369, -1, 17370, 17371, 17372, -1, 17373, 17374, 17375, -1, 17376, 17377, 17378, -1, 17379, 17380, 17381, -1, 17382, 17383, 17384, -1, 17385, 17386, 17387, -1, 17388, 17389, 17390, -1, 17391, 17392, 17393, -1, 17394, 17395, 17396, -1, 17397, 17398, 17399, -1, 17400, 17401, 17402, -1, 17403, 17404, 17405, -1, 17406, 17407, 17408, -1, 17409, 17410, 17411, -1, 17412, 17413, 17414, -1, 17415, 17416, 17417, -1, 17418, 17419, 17420, -1, 17421, 17422, 17423, -1, 17424, 17425, 17426, -1, 17427, 17428, 17429, -1, 17430, 17431, 17432, -1, 17433, 17434, 17435, -1, 17436, 17437, 17438, -1, 17439, 17440, 17441, -1, 17442, 17443, 17444, -1, 17445, 17446, 17447, -1, 17448, 17449, 17450, -1, 17451, 17452, 17453, -1, 17454, 17455, 17456, -1, 17457, 17458, 17459, -1, 17460, 17461, 17462, -1, 17463, 17464, 17465, -1, 17466, 17467, 17468, -1, 17469, 17470, 17471, -1, 17472, 17473, 17474, -1, 17475, 17476, 17477, -1, 17478, 17479, 17480, -1, 17481, 17482, 17483, -1, 17484, 17485, 17486, -1, 17487, 17488, 17489, -1, 17490, 17491, 17492, -1, 17493, 17494, 17495, -1, 17496, 17497, 17498, -1, 17499, 17500, 17501, -1, 17502, 17503, 17504, -1, 17505, 17506, 17507, -1, 17508, 17509, 17510, -1, 17511, 17512, 17513, -1, 17514, 17515, 17516, -1, 17517, 17518, 17519, -1, 17520, 17521, 17522, -1, 17523, 17524, 17525, -1, 17526, 17527, 17528, -1, 17529, 17530, 17531, -1, 17532, 17533, 17534, -1, 17535, 17536, 17537, -1, 17538, 17539, 17540, -1, 17541, 17542, 17543, -1, 17544, 17545, 17546, -1, 17547, 17548, 17549, -1, 17550, 17551, 17552, -1, 17553, 17554, 17555, -1, 17556, 17557, 17558, -1, 17559, 17560, 17561, -1, 17562, 17563, 17564, -1, 17565, 17566, 17567, -1, 17568, 17569, 17570, -1, 17571, 17572, 17573, -1, 17574, 17575, 17576, -1, 17577, 17578, 17579, -1, 17580, 17581, 17582, -1, 17583, 17584, 17585, -1, 17586, 17587, 17588, -1, 17589, 17590, 17591, -1, 17592, 17593, 17594, -1, 17595, 17596, 17597, -1, 17598, 17599, 17600, -1, 17601, 17602, 17603, -1, 17604, 17605, 17606, -1, 17607, 17608, 17609, -1, 17610, 17611, 17612, -1, 17613, 17614, 17615, -1, 17616, 17617, 17618, -1, 17619, 17620, 17621, -1, 17622, 17623, 17624, -1, 17625, 17626, 17627, -1, 17628, 17629, 17630, -1, 17631, 17632, 17633, -1, 17634, 17635, 17636, -1, 17637, 17638, 17639, -1, 17640, 17641, 17642, -1, 17643, 17644, 17645, -1, 17646, 17647, 17648, -1, 17649, 17650, 17651, -1, 17652, 17653, 17654, -1, 17655, 17656, 17657, -1, 17658, 17659, 17660, -1, 17661, 17662, 17663, -1, 17664, 17665, 17666, -1, 17667, 17668, 17669, -1, 17670, 17671, 17672, -1, 17673, 17674, 17675, -1, 17676, 17677, 17678, -1, 17679, 17680, 17681, -1, 17682, 17683, 17684, -1, 17685, 17686, 17687, -1, 17688, 17689, 17690, -1, 17691, 17692, 17693, -1, 17694, 17695, 17696, -1, 17697, 17698, 17699, -1, 17700, 17701, 17702, -1, 17703, 17704, 17705, -1, 17706, 17707, 17708, -1, 17709, 17710, 17711, -1, 17712, 17713, 17714, -1, 17715, 17716, 17717, -1, 17718, 17719, 17720, -1, 17721, 17722, 17723, -1, 17724, 17725, 17726, -1, 17727, 17728, 17729, -1, 17730, 17731, 17732, -1, 17733, 17734, 17735, -1, 17736, 17737, 17738, -1, 17739, 17740, 17741, -1, 17742, 17743, 17744, -1, 17745, 17746, 17747, -1, 17748, 17749, 17750, -1, 17751, 17752, 17753, -1, 17754, 17755, 17756, -1, 17757, 17758, 17759, -1, 17760, 17761, 17762, -1, 17763, 17764, 17765, -1, 17766, 17767, 17768, -1, 17769, 17770, 17771, -1, 17772, 17773, 17774, -1, 17775, 17776, 17777, -1, 17778, 17779, 17780, -1, 17781, 17782, 17783, -1, 17784, 17785, 17786, -1, 17787, 17788, 17789, -1, 17790, 17791, 17792, -1, 17793, 17794, 17795, -1, 17796, 17797, 17798, -1, 17799, 17800, 17801, -1, 17802, 17803, 17804, -1, 17805, 17806, 17807, -1, 17808, 17809, 17810, -1, 17811, 17812, 17813, -1, 17814, 17815, 17816, -1, 17817, 17818, 17819, -1, 17820, 17821, 17822, -1, 17823, 17824, 17825, -1, 17826, 17827, 17828, -1, 17829, 17830, 17831, -1, 17832, 17833, 17834, -1, 17835, 17836, 17837, -1, 17838, 17839, 17840, -1, 17841, 17842, 17843, -1, 17844, 17845, 17846, -1, 17847, 17848, 17849, -1, 17850, 17851, 17852, -1, 17853, 17854, 17855, -1, 17856, 17857, 17858, -1, 17859, 17860, 17861, -1, 17862, 17863, 17864, -1, 17865, 17866, 17867, -1, 17868, 17869, 17870, -1, 17871, 17872, 17873, -1, 17874, 17875, 17876, -1, 17877, 17878, 17879, -1, 17880, 17881, 17882, -1, 17883, 17884, 17885, -1, 17886, 17887, 17888, -1, 17889, 17890, 17891, -1, 17892, 17893, 17894, -1, 17895, 17896, 17897, -1, 17898, 17899, 17900, -1, 17901, 17902, 17903, -1, 17904, 17905, 17906, -1, 17907, 17908, 17909, -1, 17910, 17911, 17912, -1, 17913, 17914, 17915, -1, 17916, 17917, 17918, -1, 17919, 17920, 17921, -1, 17922, 17923, 17924, -1, 17925, 17926, 17927, -1, 17928, 17929, 17930, -1, 17931, 17932, 17933, -1, 17934, 17935, 17936, -1, 17937, 17938, 17939, -1, 17940, 17941, 17942, -1, 17943, 17944, 17945, -1, 17946, 17947, 17948, -1, 17949, 17950, 17951, -1, 17952, 17953, 17954, -1, 17955, 17956, 17957, -1, 17958, 17959, 17960, -1, 17961, 17962, 17963, -1, 17964, 17965, 17966, -1, 17967, 17968, 17969, -1, 17970, 17971, 17972, -1, 17973, 17974, 17975, -1, 17976, 17977, 17978, -1, 17979, 17980, 17981, -1, 17982, 17983, 17984, -1, 17985, 17986, 17987, -1, 17988, 17989, 17990, -1, 17991, 17992, 17993, -1, 17994, 17995, 17996, -1, 17997, 17998, 17999, -1, 18000, 18001, 18002, -1, 18003, 18004, 18005, -1, 18006, 18007, 18008, -1, 18009, 18010, 18011, -1, 18012, 18013, 18014, -1, 18015, 18016, 18017, -1, 18018, 18019, 18020, -1, 18021, 18022, 18023, -1, 18024, 18025, 18026, -1, 18027, 18028, 18029, -1, 18030, 18031, 18032, -1, 18033, 18034, 18035, -1, 18036, 18037, 18038, -1, 18039, 18040, 18041, -1, 18042, 18043, 18044, -1, 18045, 18046, 18047, -1, 18048, 18049, 18050, -1, 18051, 18052, 18053, -1, 18054, 18055, 18056, -1, 18057, 18058, 18059, -1, 18060, 18061, 18062, -1, 18063, 18064, 18065, -1, 18066, 18067, 18068, -1, 18069, 18070, 18071, -1, 18072, 18073, 18074, -1, 18075, 18076, 18077, -1, 18078, 18079, 18080, -1, 18081, 18082, 18083, -1, 18084, 18085, 18086, -1, 18087, 18088, 18089, -1, 18090, 18091, 18092, -1, 18093, 18094, 18095, -1, 18096, 18097, 18098, -1, 18099, 18100, 18101, -1, 18102, 18103, 18104, -1, 18105, 18106, 18107, -1, 18108, 18109, 18110, -1, 18111, 18112, 18113, -1, 18114, 18115, 18116, -1, 18117, 18118, 18119, -1, 18120, 18121, 18122, -1, 18123, 18124, 18125, -1, 18126, 18127, 18128, -1, 18129, 18130, 18131, -1, 18132, 18133, 18134, -1, 18135, 18136, 18137, -1, 18138, 18139, 18140, -1, 18141, 18142, 18143, -1, 18144, 18145, 18146, -1, 18147, 18148, 18149, -1, 18150, 18151, 18152, -1, 18153, 18154, 18155, -1, 18156, 18157, 18158, -1, 18159, 18160, 18161, -1, 18162, 18163, 18164, -1, 18165, 18166, 18167, -1, 18168, 18169, 18170, -1, 18171, 18172, 18173, -1, 18174, 18175, 18176, -1, 18177, 18178, 18179, -1, 18180, 18181, 18182, -1, 18183, 18184, 18185, -1, 18186, 18187, 18188, -1, 18189, 18190, 18191, -1, 18192, 18193, 18194, -1, 18195, 18196, 18197, -1, 18198, 18199, 18200, -1, 18201, 18202, 18203, -1, 18204, 18205, 18206, -1, 18207, 18208, 18209, -1, 18210, 18211, 18212, -1, 18213, 18214, 18215, -1, 18216, 18217, 18218, -1, 18219, 18220, 18221, -1, 18222, 18223, 18224, -1, 18225, 18226, 18227, -1, 18228, 18229, 18230, -1, 18231, 18232, 18233, -1, 18234, 18235, 18236, -1, 18237, 18238, 18239, -1, 18240, 18241, 18242, -1, 18243, 18244, 18245, -1, 18246, 18247, 18248, -1, 18249, 18250, 18251, -1, 18252, 18253, 18254, -1, 18255, 18256, 18257, -1, 18258, 18259, 18260, -1, 18261, 18262, 18263, -1, 18264, 18265, 18266, -1, 18267, 18268, 18269, -1, 18270, 18271, 18272, -1, 18273, 18274, 18275, -1, 18276, 18277, 18278, -1, 18279, 18280, 18281, -1, 18282, 18283, 18284, -1, 18285, 18286, 18287, -1, 18288, 18289, 18290, -1, 18291, 18292, 18293, -1, 18294, 18295, 18296, -1, 18297, 18298, 18299, -1, 18300, 18301, 18302, -1, 18303, 18304, 18305, -1, 18306, 18307, 18308, -1, 18309, 18310, 18311, -1, 18312, 18313, 18314, -1, 18315, 18316, 18317, -1, 18318, 18319, 18320, -1, 18321, 18322, 18323, -1, 18324, 18325, 18326, -1, 18327, 18328, 18329, -1, 18330, 18331, 18332, -1, 18333, 18334, 18335, -1, 18336, 18337, 18338, -1, 18339, 18340, 18341, -1, 18342, 18343, 18344, -1, 18345, 18346, 18347, -1, 18348, 18349, 18350, -1, 18351, 18352, 18353, -1, 18354, 18355, 18356, -1, 18357, 18358, 18359, -1, 18360, 18361, 18362, -1, 18363, 18364, 18365, -1, 18366, 18367, 18368, -1, 18369, 18370, 18371, -1, 18372, 18373, 18374, -1, 18375, 18376, 18377, -1, 18378, 18379, 18380, -1, 18381, 18382, 18383, -1, 18384, 18385, 18386, -1, 18387, 18388, 18389, -1, 18390, 18391, 18392, -1, 18393, 18394, 18395, -1, 18396, 18397, 18398, -1, 18399, 18400, 18401, -1, 18402, 18403, 18404, -1, 18405, 18406, 18407, -1, 18408, 18409, 18410, -1, 18411, 18412, 18413, -1, 18414, 18415, 18416, -1, 18417, 18418, 18419, -1, 18420, 18421, 18422, -1, 18423, 18424, 18425, -1, 18426, 18427, 18428, -1, 18429, 18430, 18431, -1, 18432, 18433, 18434, -1, 18435, 18436, 18437, -1, 18438, 18439, 18440, -1, 18441, 18442, 18443, -1, 18444, 18445, 18446, -1, 18447, 18448, 18449, -1, 18450, 18451, 18452, -1, 18453, 18454, 18455, -1, 18456, 18457, 18458, -1, 18459, 18460, 18461, -1, 18462, 18463, 18464, -1, 18465, 18466, 18467, -1, 18468, 18469, 18470, -1, 18471, 18472, 18473, -1, 18474, 18475, 18476, -1, 18477, 18478, 18479, -1, 18480, 18481, 18482, -1, 18483, 18484, 18485, -1, 18486, 18487, 18488, -1, 18489, 18490, 18491, -1, 18492, 18493, 18494, -1, 18495, 18496, 18497, -1, 18498, 18499, 18500, -1, 18501, 18502, 18503, -1, 18504, 18505, 18506, -1, 18507, 18508, 18509, -1, 18510, 18511, 18512, -1, 18513, 18514, 18515, -1, 18516, 18517, 18518, -1, 18519, 18520, 18521, -1, 18522, 18523, 18524, -1, 18525, 18526, 18527, -1, 18528, 18529, 18530, -1, 18531, 18532, 18533, -1, 18534, 18535, 18536, -1, 18537, 18538, 18539, -1, 18540, 18541, 18542, -1, 18543, 18544, 18545, -1, 18546, 18547, 18548, -1, 18549, 18550, 18551, -1, 18552, 18553, 18554, -1, 18555, 18556, 18557, -1, 18558, 18559, 18560, -1, 18561, 18562, 18563, -1, 18564, 18565, 18566, -1, 18567, 18568, 18569, -1, 18570, 18571, 18572, -1, 18573, 18574, 18575, -1, 18576, 18577, 18578, -1, 18579, 18580, 18581, -1, 18582, 18583, 18584, -1, 18585, 18586, 18587, -1, 18588, 18589, 18590, -1, 18591, 18592, 18593, -1, 18594, 18595, 18596, -1, 18597, 18598, 18599, -1, 18600, 18601, 18602, -1, 18603, 18604, 18605, -1, 18606, 18607, 18608, -1, 18609, 18610, 18611, -1, 18612, 18613, 18614, -1, 18615, 18616, 18617, -1, 18618, 18619, 18620, -1, 18621, 18622, 18623, -1, 18624, 18625, 18626, -1, 18627, 18628, 18629, -1, 18630, 18631, 18632, -1, 18633, 18634, 18635, -1, 18636, 18637, 18638, -1, 18639, 18640, 18641, -1, 18642, 18643, 18644, -1, 18645, 18646, 18647, -1, 18648, 18649, 18650, -1, 18651, 18652, 18653, -1, 18654, 18655, 18656, -1, 18657, 18658, 18659, -1, 18660, 18661, 18662, -1, 18663, 18664, 18665, -1, 18666, 18667, 18668, -1, 18669, 18670, 18671, -1, 18672, 18673, 18674, -1, 18675, 18676, 18677, -1, 18678, 18679, 18680, -1, 18681, 18682, 18683, -1, 18684, 18685, 18686, -1, 18687, 18688, 18689, -1, 18690, 18691, 18692, -1, 18693, 18694, 18695, -1, 18696, 18697, 18698, -1, 18699, 18700, 18701, -1, 18702, 18703, 18704, -1, 18705, 18706, 18707, -1, 18708, 18709, 18710, -1, 18711, 18712, 18713, -1, 18714, 18715, 18716, -1, 18717, 18718, 18719, -1, 18720, 18721, 18722, -1, 18723, 18724, 18725, -1, 18726, 18727, 18728, -1, 18729, 18730, 18731, -1, 18732, 18733, 18734, -1, 18735, 18736, 18737, -1, 18738, 18739, 18740, -1, 18741, 18742, 18743, -1, 18744, 18745, 18746, -1, 18747, 18748, 18749, -1, 18750, 18751, 18752, -1, 18753, 18754, 18755, -1, 18756, 18757, 18758, -1, 18759, 18760, 18761, -1, 18762, 18763, 18764, -1, 18765, 18766, 18767, -1, 18768, 18769, 18770, -1, 18771, 18772, 18773, -1, 18774, 18775, 18776, -1, 18777, 18778, 18779, -1, 18780, 18781, 18782, -1, 18783, 18784, 18785, -1, 18786, 18787, 18788, -1, 18789, 18790, 18791, -1, 18792, 18793, 18794, -1, 18795, 18796, 18797, -1, 18798, 18799, 18800, -1, 18801, 18802, 18803, -1, 18804, 18805, 18806, -1, 18807, 18808, 18809, -1, 18810, 18811, 18812, -1, 18813, 18814, 18815, -1, 18816, 18817, 18818, -1, 18819, 18820, 18821, -1, 18822, 18823, 18824, -1, 18825, 18826, 18827, -1, 18828, 18829, 18830, -1, 18831, 18832, 18833, -1, 18834, 18835, 18836, -1, 18837, 18838, 18839, -1, 18840, 18841, 18842, -1, 18843, 18844, 18845, -1, 18846, 18847, 18848, -1, 18849, 18850, 18851, -1, 18852, 18853, 18854, -1, 18855, 18856, 18857, -1, 18858, 18859, 18860, -1, 18861, 18862, 18863, -1, 18864, 18865, 18866, -1, 18867, 18868, 18869, -1, 18870, 18871, 18872, -1, 18873, 18874, 18875, -1, 18876, 18877, 18878, -1, 18879, 18880, 18881, -1, 18882, 18883, 18884, -1, 18885, 18886, 18887, -1, 18888, 18889, 18890, -1, 18891, 18892, 18893, -1, 18894, 18895, 18896, -1, 18897, 18898, 18899, -1, 18900, 18901, 18902, -1, 18903, 18904, 18905, -1, 18906, 18907, 18908, -1, 18909, 18910, 18911, -1, 18912, 18913, 18914, -1, 18915, 18916, 18917, -1, 18918, 18919, 18920, -1, 18921, 18922, 18923, -1, 18924, 18925, 18926, -1, 18927, 18928, 18929, -1, 18930, 18931, 18932, -1, 18933, 18934, 18935, -1, 18936, 18937, 18938, -1, 18939, 18940, 18941, -1, 18942, 18943, 18944, -1, 18945, 18946, 18947, -1, 18948, 18949, 18950, -1, 18951, 18952, 18953, -1, 18954, 18955, 18956, -1, 18957, 18958, 18959, -1, 18960, 18961, 18962, -1, 18963, 18964, 18965, -1, 18966, 18967, 18968, -1, 18969, 18970, 18971, -1, 18972, 18973, 18974, -1, 18975, 18976, 18977, -1, 18978, 18979, 18980, -1, 18981, 18982, 18983, -1, 18984, 18985, 18986, -1, 18987, 18988, 18989, -1, 18990, 18991, 18992, -1, 18993, 18994, 18995, -1, 18996, 18997, 18998, -1, 18999, 19000, 19001, -1, 19002, 19003, 19004, -1, 19005, 19006, 19007, -1, 19008, 19009, 19010, -1, 19011, 19012, 19013, -1, 19014, 19015, 19016, -1, 19017, 19018, 19019, -1, 19020, 19021, 19022, -1, 19023, 19024, 19025, -1, 19026, 19027, 19028, -1, 19029, 19030, 19031, -1, 19032, 19033, 19034, -1, 19035, 19036, 19037, -1, 19038, 19039, 19040, -1, 19041, 19042, 19043, -1, 19044, 19045, 19046, -1, 19047, 19048, 19049, -1, 19050, 19051, 19052, -1, 19053, 19054, 19055, -1, 19056, 19057, 19058, -1, 19059, 19060, 19061, -1, 19062, 19063, 19064, -1, 19065, 19066, 19067, -1, 19068, 19069, 19070, -1, 19071, 19072, 19073, -1, 19074, 19075, 19076, -1, 19077, 19078, 19079, -1, 19080, 19081, 19082, -1, 19083, 19084, 19085, -1, 19086, 19087, 19088, -1, 19089, 19090, 19091, -1, 19092, 19093, 19094, -1, 19095, 19096, 19097, -1, 19098, 19099, 19100, -1, 19101, 19102, 19103, -1, 19104, 19105, 19106, -1, 19107, 19108, 19109, -1, 19110, 19111, 19112, -1, 19113, 19114, 19115, -1, 19116, 19117, 19118, -1, 19119, 19120, 19121, -1, 19122, 19123, 19124, -1, 19125, 19126, 19127, -1, 19128, 19129, 19130, -1, 19131, 19132, 19133, -1, 19134, 19135, 19136, -1, 19137, 19138, 19139, -1, 19140, 19141, 19142, -1, 19143, 19144, 19145, -1, 19146, 19147, 19148, -1, 19149, 19150, 19151, -1, 19152, 19153, 19154, -1, 19155, 19156, 19157, -1, 19158, 19159, 19160, -1, 19161, 19162, 19163, -1, 19164, 19165, 19166, -1, 19167, 19168, 19169, -1, 19170, 19171, 19172, -1, 19173, 19174, 19175, -1, 19176, 19177, 19178, -1, 19179, 19180, 19181, -1, 19182, 19183, 19184, -1, 19185, 19186, 19187, -1, 19188, 19189, 19190, -1, 19191, 19192, 19193, -1, 19194, 19195, 19196, -1, 19197, 19198, 19199, -1, 19200, 19201, 19202, -1, 19203, 19204, 19205, -1, 19206, 19207, 19208, -1, 19209, 19210, 19211, -1, 19212, 19213, 19214, -1, 19215, 19216, 19217, -1, 19218, 19219, 19220, -1, 19221, 19222, 19223, -1, 19224, 19225, 19226, -1, 19227, 19228, 19229, -1, 19230, 19231, 19232, -1, 19233, 19234, 19235, -1, 19236, 19237, 19238, -1, 19239, 19240, 19241, -1, 19242, 19243, 19244, -1, 19245, 19246, 19247, -1, 19248, 19249, 19250, -1, 19251, 19252, 19253, -1, 19254, 19255, 19256, -1, 19257, 19258, 19259, -1, 19260, 19261, 19262, -1, 19263, 19264, 19265, -1, 19266, 19267, 19268, -1, 19269, 19270, 19271, -1, 19272, 19273, 19274, -1, 19275, 19276, 19277, -1, 19278, 19279, 19280, -1, 19281, 19282, 19283, -1, 19284, 19285, 19286, -1, 19287, 19288, 19289, -1, 19290, 19291, 19292, -1, 19293, 19294, 19295, -1, 19296, 19297, 19298, -1, 19299, 19300, 19301, -1, 19302, 19303, 19304, -1, 19305, 19306, 19307, -1, 19308, 19309, 19310, -1, 19311, 19312, 19313, -1, 19314, 19315, 19316, -1, 19317, 19318, 19319, -1, 19320, 19321, 19322, -1, 19323, 19324, 19325, -1, 19326, 19327, 19328, -1, 19329, 19330, 19331, -1, 19332, 19333, 19334, -1, 19335, 19336, 19337, -1, 19338, 19339, 19340, -1, 19341, 19342, 19343, -1, 19344, 19345, 19346, -1, 19347, 19348, 19349, -1, 19350, 19351, 19352, -1, 19353, 19354, 19355, -1, 19356, 19357, 19358, -1, 19359, 19360, 19361, -1, 19362, 19363, 19364, -1, 19365, 19366, 19367, -1, 19368, 19369, 19370, -1, 19371, 19372, 19373, -1, 19374, 19375, 19376, -1, 19377, 19378, 19379, -1, 19380, 19381, 19382, -1, 19383, 19384, 19385, -1, 19386, 19387, 19388, -1, 19389, 19390, 19391, -1, 19392, 19393, 19394, -1, 19395, 19396, 19397, -1, 19398, 19399, 19400, -1, 19401, 19402, 19403, -1, 19404, 19405, 19406, -1, 19407, 19408, 19409, -1, 19410, 19411, 19412, -1, 19413, 19414, 19415, -1, 19416, 19417, 19418, -1, 19419, 19420, 19421, -1, 19422, 19423, 19424, -1, 19425, 19426, 19427, -1, 19428, 19429, 19430, -1, 19431, 19432, 19433, -1, 19434, 19435, 19436, -1, 19437, 19438, 19439, -1, 19440, 19441, 19442, -1, 19443, 19444, 19445, -1, 19446, 19447, 19448, -1, 19449, 19450, 19451, -1, 19452, 19453, 19454, -1, 19455, 19456, 19457, -1, 19458, 19459, 19460, -1, 19461, 19462, 19463, -1, 19464, 19465, 19466, -1, 19467, 19468, 19469, -1, 19470, 19471, 19472, -1, 19473, 19474, 19475, -1, 19476, 19477, 19478, -1, 19479, 19480, 19481, -1, 19482, 19483, 19484, -1, 19485, 19486, 19487, -1, 19488, 19489, 19490, -1, 19491, 19492, 19493, -1, 19494, 19495, 19496, -1, 19497, 19498, 19499, -1, 19500, 19501, 19502, -1, 19503, 19504, 19505, -1, 19506, 19507, 19508, -1, 19509, 19510, 19511, -1, 19512, 19513, 19514, -1, 19515, 19516, 19517, -1, 19518, 19519, 19520, -1, 19521, 19522, 19523, -1, 19524, 19525, 19526, -1, 19527, 19528, 19529, -1, 19530, 19531, 19532, -1, 19533, 19534, 19535, -1, 19536, 19537, 19538, -1, 19539, 19540, 19541, -1, 19542, 19543, 19544, -1, 19545, 19546, 19547, -1, 19548, 19549, 19550, -1, 19551, 19552, 19553, -1, 19554, 19555, 19556, -1, 19557, 19558, 19559, -1, 19560, 19561, 19562, -1, 19563, 19564, 19565, -1, 19566, 19567, 19568, -1, 19569, 19570, 19571, -1, 19572, 19573, 19574, -1, 19575, 19576, 19577, -1, 19578, 19579, 19580, -1, 19581, 19582, 19583, -1, 19584, 19585, 19586, -1, 19587, 19588, 19589, -1, 19590, 19591, 19592, -1, 19593, 19594, 19595, -1, 19596, 19597, 19598, -1, 19599, 19600, 19601, -1, 19602, 19603, 19604, -1, 19605, 19606, 19607, -1, 19608, 19609, 19610, -1, 19611, 19612, 19613, -1, 19614, 19615, 19616, -1, 19617, 19618, 19619, -1, 19620, 19621, 19622, -1, 19623, 19624, 19625, -1, 19626, 19627, 19628, -1, 19629, 19630, 19631, -1, 19632, 19633, 19634, -1, 19635, 19636, 19637, -1, 19638, 19639, 19640, -1, 19641, 19642, 19643, -1, 19644, 19645, 19646, -1, 19647, 19648, 19649, -1, 19650, 19651, 19652, -1, 19653, 19654, 19655, -1, 19656, 19657, 19658, -1, 19659, 19660, 19661, -1, 19662, 19663, 19664, -1, 19665, 19666, 19667, -1, 19668, 19669, 19670, -1, 19671, 19672, 19673, -1, 19674, 19675, 19676, -1, 19677, 19678, 19679, -1, 19680, 19681, 19682, -1, 19683, 19684, 19685, -1, 19686, 19687, 19688, -1, 19689, 19690, 19691, -1, 19692, 19693, 19694, -1, 19695, 19696, 19697, -1, 19698, 19699, 19700, -1, 19701, 19702, 19703, -1, 19704, 19705, 19706, -1, 19707, 19708, 19709, -1, 19710, 19711, 19712, -1, 19713, 19714, 19715, -1, 19716, 19717, 19718, -1, 19719, 19720, 19721, -1, 19722, 19723, 19724, -1, 19725, 19726, 19727, -1, 19728, 19729, 19730, -1, 19731, 19732, 19733, -1, 19734, 19735, 19736, -1, 19737, 19738, 19739, -1, 19740, 19741, 19742, -1, 19743, 19744, 19745, -1, 19746, 19747, 19748, -1, 19749, 19750, 19751, -1, 19752, 19753, 19754, -1, 19755, 19756, 19757, -1, 19758, 19759, 19760, -1, 19761, 19762, 19763, -1, 19764, 19765, 19766, -1, 19767, 19768, 19769, -1, 19770, 19771, 19772, -1, 19773, 19774, 19775, -1, 19776, 19777, 19778, -1, 19779, 19780, 19781, -1, 19782, 19783, 19784, -1, 19785, 19786, 19787, -1, 19788, 19789, 19790, -1, 19791, 19792, 19793, -1, 19794, 19795, 19796, -1, 19797, 19798, 19799, -1, 19800, 19801, 19802, -1, 19803, 19804, 19805, -1, 19806, 19807, 19808, -1, 19809, 19810, 19811, -1, 19812, 19813, 19814, -1, 19815, 19816, 19817, -1, 19818, 19819, 19820, -1, 19821, 19822, 19823, -1, 19824, 19825, 19826, -1, 19827, 19828, 19829, -1, 19830, 19831, 19832, -1, 19833, 19834, 19835, -1, 19836, 19837, 19838, -1, 19839, 19840, 19841, -1, 19842, 19843, 19844, -1, 19845, 19846, 19847, -1, 19848, 19849, 19850, -1, 19851, 19852, 19853, -1, 19854, 19855, 19856, -1, 19857, 19858, 19859, -1, 19860, 19861, 19862, -1, 19863, 19864, 19865, -1, 19866, 19867, 19868, -1, 19869, 19870, 19871, -1, 19872, 19873, 19874, -1, 19875, 19876, 19877, -1, 19878, 19879, 19880, -1, 19881, 19882, 19883, -1, 19884, 19885, 19886, -1, 19887, 19888, 19889, -1, 19890, 19891, 19892, -1, 19893, 19894, 19895, -1, 19896, 19897, 19898, -1, 19899, 19900, 19901, -1, 19902, 19903, 19904, -1, 19905, 19906, 19907, -1, 19908, 19909, 19910, -1, 19911, 19912, 19913, -1, 19914, 19915, 19916, -1, 19917, 19918, 19919, -1, 19920, 19921, 19922, -1, 19923, 19924, 19925, -1, 19926, 19927, 19928, -1, 19929, 19930, 19931, -1, 19932, 19933, 19934, -1, 19935, 19936, 19937, -1, 19938, 19939, 19940, -1, 19941, 19942, 19943, -1, 19944, 19945, 19946, -1, 19947, 19948, 19949, -1, 19950, 19951, 19952, -1, 19953, 19954, 19955, -1, 19956, 19957, 19958, -1, 19959, 19960, 19961, -1, 19962, 19963, 19964, -1, 19965, 19966, 19967, -1, 19968, 19969, 19970, -1, 19971, 19972, 19973, -1, 19974, 19975, 19976, -1, 19977, 19978, 19979, -1, 19980, 19981, 19982, -1, 19983, 19984, 19985, -1, 19986, 19987, 19988, -1, 19989, 19990, 19991, -1, 19992, 19993, 19994, -1, 19995, 19996, 19997, -1, 19998, 19999, 20000, -1, 20001, 20002, 20003, -1, 20004, 20005, 20006, -1, 20007, 20008, 20009, -1, 20010, 20011, 20012, -1, 20013, 20014, 20015, -1, 20016, 20017, 20018, -1, 20019, 20020, 20021, -1, 20022, 20023, 20024, -1, 20025, 20026, 20027, -1, 20028, 20029, 20030, -1, 20031, 20032, 20033, -1, 20034, 20035, 20036, -1, 20037, 20038, 20039, -1, 20040, 20041, 20042, -1, 20043, 20044, 20045, -1, 20046, 20047, 20048, -1, 20049, 20050, 20051, -1, 20052, 20053, 20054, -1, 20055, 20056, 20057, -1, 20058, 20059, 20060, -1, 20061, 20062, 20063, -1, 20064, 20065, 20066, -1, 20067, 20068, 20069, -1, 20070, 20071, 20072, -1, 20073, 20074, 20075, -1, 20076, 20077, 20078, -1, 20079, 20080, 20081, -1, 20082, 20083, 20084, -1, 20085, 20086, 20087, -1, 20088, 20089, 20090, -1, 20091, 20092, 20093, -1, 20094, 20095, 20096, -1, 20097, 20098, 20099, -1, 20100, 20101, 20102, -1, 20103, 20104, 20105, -1, 20106, 20107, 20108, -1, 20109, 20110, 20111, -1, 20112, 20113, 20114, -1, 20115, 20116, 20117, -1, 20118, 20119, 20120, -1, 20121, 20122, 20123, -1, 20124, 20125, 20126, -1, 20127, 20128, 20129, -1, 20130, 20131, 20132, -1, 20133, 20134, 20135, -1, 20136, 20137, 20138, -1, 20139, 20140, 20141, -1, 20142, 20143, 20144, -1, 20145, 20146, 20147, -1, 20148, 20149, 20150, -1, 20151, 20152, 20153, -1, 20154, 20155, 20156, -1, 20157, 20158, 20159, -1, 20160, 20161, 20162, -1, 20163, 20164, 20165, -1, 20166, 20167, 20168, -1, 20169, 20170, 20171, -1, 20172, 20173, 20174, -1, 20175, 20176, 20177, -1, 20178, 20179, 20180, -1, 20181, 20182, 20183, -1, 20184, 20185, 20186, -1, 20187, 20188, 20189, -1, 20190, 20191, 20192, -1, 20193, 20194, 20195, -1, 20196, 20197, 20198, -1, 20199, 20200, 20201, -1, 20202, 20203, 20204, -1, 20205, 20206, 20207, -1, 20208, 20209, 20210, -1, 20211, 20212, 20213, -1, 20214, 20215, 20216, -1, 20217, 20218, 20219, -1, 20220, 20221, 20222, -1, 20223, 20224, 20225, -1, 20226, 20227, 20228, -1, 20229, 20230, 20231, -1, 20232, 20233, 20234, -1, 20235, 20236, 20237, -1, 20238, 20239, 20240, -1, 20241, 20242, 20243, -1, 20244, 20245, 20246, -1, 20247, 20248, 20249, -1, 20250, 20251, 20252, -1, 20253, 20254, 20255, -1, 20256, 20257, 20258, -1, 20259, 20260, 20261, -1, 20262, 20263, 20264, -1, 20265, 20266, 20267, -1, 20268, 20269, 20270, -1, 20271, 20272, 20273, -1, 20274, 20275, 20276, -1, 20277, 20278, 20279, -1, 20280, 20281, 20282, -1, 20283, 20284, 20285, -1, 20286, 20287, 20288, -1, 20289, 20290, 20291, -1, 20292, 20293, 20294, -1, 20295, 20296, 20297, -1, 20298, 20299, 20300, -1, 20301, 20302, 20303, -1, 20304, 20305, 20306, -1, 20307, 20308, 20309, -1, 20310, 20311, 20312, -1, 20313, 20314, 20315, -1, 20316, 20317, 20318, -1, 20319, 20320, 20321, -1, 20322, 20323, 20324, -1, 20325, 20326, 20327, -1, 20328, 20329, 20330, -1, 20331, 20332, 20333, -1, 20334, 20335, 20336, -1, 20337, 20338, 20339, -1, 20340, 20341, 20342, -1, 20343, 20344, 20345, -1, 20346, 20347, 20348, -1, 20349, 20350, 20351, -1, 20352, 20353, 20354, -1, 20355, 20356, 20357, -1, 20358, 20359, 20360, -1, 20361, 20362, 20363, -1, 20364, 20365, 20366, -1, 20367, 20368, 20369, -1, 20370, 20371, 20372, -1, 20373, 20374, 20375, -1, 20376, 20377, 20378, -1, 20379, 20380, 20381, -1, 20382, 20383, 20384, -1, 20385, 20386, 20387, -1, 20388, 20389, 20390, -1, 20391, 20392, 20393, -1, 20394, 20395, 20396, -1, 20397, 20398, 20399, -1, 20400, 20401, 20402, -1, 20403, 20404, 20405, -1, 20406, 20407, 20408, -1, 20409, 20410, 20411, -1, 20412, 20413, 20414, -1, 20415, 20416, 20417, -1, 20418, 20419, 20420, -1, 20421, 20422, 20423, -1, 20424, 20425, 20426, -1, 20427, 20428, 20429, -1, 20430, 20431, 20432, -1, 20433, 20434, 20435, -1, 20436, 20437, 20438, -1, 20439, 20440, 20441, -1, 20442, 20443, 20444, -1, 20445, 20446, 20447, -1, 20448, 20449, 20450, -1, 20451, 20452, 20453, -1, 20454, 20455, 20456, -1, 20457, 20458, 20459, -1, 20460, 20461, 20462, -1, 20463, 20464, 20465, -1, 20466, 20467, 20468, -1, 20469, 20470, 20471, -1, 20472, 20473, 20474, -1, 20475, 20476, 20477, -1, 20478, 20479, 20480, -1, 20481, 20482, 20483, -1, 20484, 20485, 20486, -1, 20487, 20488, 20489, -1, 20490, 20491, 20492, -1, 20493, 20494, 20495, -1, 20496, 20497, 20498, -1, 20499, 20500, 20501, -1, 20502, 20503, 20504, -1, 20505, 20506, 20507, -1, 20508, 20509, 20510, -1, 20511, 20512, 20513, -1, 20514, 20515, 20516, -1, 20517, 20518, 20519, -1, 20520, 20521, 20522, -1, 20523, 20524, 20525, -1, 20526, 20527, 20528, -1, 20529, 20530, 20531, -1, 20532, 20533, 20534, -1, 20535, 20536, 20537, -1, 20538, 20539, 20540, -1, 20541, 20542, 20543, -1, 20544, 20545, 20546, -1, 20547, 20548, 20549, -1, 20550, 20551, 20552, -1, 20553, 20554, 20555, -1, 20556, 20557, 20558, -1, 20559, 20560, 20561, -1, 20562, 20563, 20564, -1, 20565, 20566, 20567, -1, 20568, 20569, 20570, -1, 20571, 20572, 20573, -1, 20574, 20575, 20576, -1, 20577, 20578, 20579, -1, 20580, 20581, 20582, -1, 20583, 20584, 20585, -1, 20586, 20587, 20588, -1, 20589, 20590, 20591, -1, 20592, 20593, 20594, -1, 20595, 20596, 20597, -1, 20598, 20599, 20600, -1, 20601, 20602, 20603, -1, 20604, 20605, 20606, -1, 20607, 20608, 20609, -1, 20610, 20611, 20612, -1, 20613, 20614, 20615, -1, 20616, 20617, 20618, -1, 20619, 20620, 20621, -1, 20622, 20623, 20624, -1, 20625, 20626, 20627, -1, 20628, 20629, 20630, -1, 20631, 20632, 20633, -1, 20634, 20635, 20636, -1, 20637, 20638, 20639, -1, 20640, 20641, 20642, -1, 20643, 20644, 20645, -1, 20646, 20647, 20648, -1, 20649, 20650, 20651, -1, 20652, 20653, 20654, -1, 20655, 20656, 20657, -1, 20658, 20659, 20660, -1, 20661, 20662, 20663, -1, 20664, 20665, 20666, -1, 20667, 20668, 20669, -1, 20670, 20671, 20672, -1, 20673, 20674, 20675, -1, 20676, 20677, 20678, -1, 20679, 20680, 20681, -1, 20682, 20683, 20684, -1, 20685, 20686, 20687, -1, 20688, 20689, 20690, -1, 20691, 20692, 20693, -1, 20694, 20695, 20696, -1, 20697, 20698, 20699, -1, 20700, 20701, 20702, -1, 20703, 20704, 20705, -1, 20706, 20707, 20708, -1, 20709, 20710, 20711, -1, 20712, 20713, 20714, -1, 20715, 20716, 20717, -1, 20718, 20719, 20720, -1, 20721, 20722, 20723, -1, 20724, 20725, 20726, -1, 20727, 20728, 20729, -1, 20730, 20731, 20732, -1, 20733, 20734, 20735, -1, 20736, 20737, 20738, -1, 20739, 20740, 20741, -1, 20742, 20743, 20744, -1, 20745, 20746, 20747, -1, 20748, 20749, 20750, -1, 20751, 20752, 20753, -1, 20754, 20755, 20756, -1, 20757, 20758, 20759, -1, 20760, 20761, 20762, -1, 20763, 20764, 20765, -1, 20766, 20767, 20768, -1, 20769, 20770, 20771, -1, 20772, 20773, 20774, -1, 20775, 20776, 20777, -1, 20778, 20779, 20780, -1, 20781, 20782, 20783, -1, 20784, 20785, 20786, -1, 20787, 20788, 20789, -1, 20790, 20791, 20792, -1, 20793, 20794, 20795, -1, 20796, 20797, 20798, -1, 20799, 20800, 20801, -1, 20802, 20803, 20804, -1, 20805, 20806, 20807, -1, 20808, 20809, 20810, -1, 20811, 20812, 20813, -1, 20814, 20815, 20816, -1, 20817, 20818, 20819, -1, 20820, 20821, 20822, -1, 20823, 20824, 20825, -1, 20826, 20827, 20828, -1, 20829, 20830, 20831, -1, 20832, 20833, 20834, -1, 20835, 20836, 20837, -1, 20838, 20839, 20840, -1, 20841, 20842, 20843, -1, 20844, 20845, 20846, -1, 20847, 20848, 20849, -1, 20850, 20851, 20852, -1, 20853, 20854, 20855, -1, 20856, 20857, 20858, -1, 20859, 20860, 20861, -1, 20862, 20863, 20864, -1, 20865, 20866, 20867, -1, 20868, 20869, 20870, -1, 20871, 20872, 20873, -1, 20874, 20875, 20876, -1, 20877, 20878, 20879, -1, 20880, 20881, 20882, -1, 20883, 20884, 20885, -1, 20886, 20887, 20888, -1, 20889, 20890, 20891, -1, 20892, 20893, 20894, -1, 20895, 20896, 20897, -1, 20898, 20899, 20900, -1, 20901, 20902, 20903, -1, 20904, 20905, 20906, -1, 20907, 20908, 20909, -1, 20910, 20911, 20912, -1, 20913, 20914, 20915, -1, 20916, 20917, 20918, -1, 20919, 20920, 20921, -1, 20922, 20923, 20924, -1, 20925, 20926, 20927, -1, 20928, 20929, 20930, -1, 20931, 20932, 20933, -1, 20934, 20935, 20936, -1, 20937, 20938, 20939, -1, 20940, 20941, 20942, -1, 20943, 20944, 20945, -1, 20946, 20947, 20948, -1, 20949, 20950, 20951, -1, 20952, 20953, 20954, -1, 20955, 20956, 20957, -1, 20958, 20959, 20960, -1, 20961, 20962, 20963, -1, 20964, 20965, 20966, -1, 20967, 20968, 20969, -1, 20970, 20971, 20972, -1, 20973, 20974, 20975, -1, 20976, 20977, 20978, -1, 20979, 20980, 20981, -1, 20982, 20983, 20984, -1, 20985, 20986, 20987, -1, 20988, 20989, 20990, -1, 20991, 20992, 20993, -1, 20994, 20995, 20996, -1, 20997, 20998, 20999, -1, 21000, 21001, 21002, -1, 21003, 21004, 21005, -1, 21006, 21007, 21008, -1, 21009, 21010, 21011, -1, 21012, 21013, 21014, -1, 21015, 21016, 21017, -1, 21018, 21019, 21020, -1, 21021, 21022, 21023, -1, 21024, 21025, 21026, -1, 21027, 21028, 21029, -1, 21030, 21031, 21032, -1, 21033, 21034, 21035, -1, 21036, 21037, 21038, -1, 21039, 21040, 21041, -1, 21042, 21043, 21044, -1, 21045, 21046, 21047, -1, 21048, 21049, 21050, -1, 21051, 21052, 21053, -1, 21054, 21055, 21056, -1, 21057, 21058, 21059, -1, 21060, 21061, 21062, -1, 21063, 21064, 21065, -1, 21066, 21067, 21068, -1, 21069, 21070, 21071, -1, 21072, 21073, 21074, -1, 21075, 21076, 21077, -1, 21078, 21079, 21080, -1, 21081, 21082, 21083, -1, 21084, 21085, 21086, -1, 21087, 21088, 21089, -1, 21090, 21091, 21092, -1, 21093, 21094, 21095, -1, 21096, 21097, 21098, -1, 21099, 21100, 21101, -1, 21102, 21103, 21104, -1, 21105, 21106, 21107, -1, 21108, 21109, 21110, -1, 21111, 21112, 21113, -1, 21114, 21115, 21116, -1, 21117, 21118, 21119, -1, 21120, 21121, 21122, -1, 21123, 21124, 21125, -1, 21126, 21127, 21128, -1, 21129, 21130, 21131, -1, 21132, 21133, 21134, -1, 21135, 21136, 21137, -1, 21138, 21139, 21140, -1, 21141, 21142, 21143, -1, 21144, 21145, 21146, -1, 21147, 21148, 21149, -1, 21150, 21151, 21152, -1, 21153, 21154, 21155, -1, 21156, 21157, 21158, -1, 21159, 21160, 21161, -1, 21162, 21163, 21164, -1, 21165, 21166, 21167, -1, 21168, 21169, 21170, -1, 21171, 21172, 21173, -1, 21174, 21175, 21176, -1, 21177, 21178, 21179, -1, 21180, 21181, 21182, -1, 21183, 21184, 21185, -1, 21186, 21187, 21188, -1, 21189, 21190, 21191, -1, 21192, 21193, 21194, -1, 21195, 21196, 21197, -1, 21198, 21199, 21200, -1, 21201, 21202, 21203, -1, 21204, 21205, 21206, -1, 21207, 21208, 21209, -1, 21210, 21211, 21212, -1, 21213, 21214, 21215, -1, 21216, 21217, 21218, -1, 21219, 21220, 21221, -1, 21222, 21223, 21224, -1, 21225, 21226, 21227, -1, 21228, 21229, 21230, -1, 21231, 21232, 21233, -1, 21234, 21235, 21236, -1, 21237, 21238, 21239, -1, 21240, 21241, 21242, -1, 21243, 21244, 21245, -1, 21246, 21247, 21248, -1, 21249, 21250, 21251, -1, 21252, 21253, 21254, -1, 21255, 21256, 21257, -1, 21258, 21259, 21260, -1, 21261, 21262, 21263, -1, 21264, 21265, 21266, -1, 21267, 21268, 21269, -1, 21270, 21271, 21272, -1, 21273, 21274, 21275, -1, 21276, 21277, 21278, -1, 21279, 21280, 21281, -1, 21282, 21283, 21284, -1, 21285, 21286, 21287, -1, 21288, 21289, 21290, -1, 21291, 21292, 21293, -1, 21294, 21295, 21296, -1, 21297, 21298, 21299, -1, 21300, 21301, 21302, -1, 21303, 21304, 21305, -1, 21306, 21307, 21308, -1, 21309, 21310, 21311, -1, 21312, 21313, 21314, -1, 21315, 21316, 21317, -1, 21318, 21319, 21320, -1, 21321, 21322, 21323, -1, 21324, 21325, 21326, -1, 21327, 21328, 21329, -1, 21330, 21331, 21332, -1, 21333, 21334, 21335, -1, 21336, 21337, 21338, -1, 21339, 21340, 21341, -1, 21342, 21343, 21344, -1, 21345, 21346, 21347, -1, 21348, 21349, 21350, -1, 21351, 21352, 21353, -1, 21354, 21355, 21356, -1, 21357, 21358, 21359, -1, 21360, 21361, 21362, -1, 21363, 21364, 21365, -1, 21366, 21367, 21368, -1, 21369, 21370, 21371, -1, 21372, 21373, 21374, -1, 21375, 21376, 21377, -1, 21378, 21379, 21380, -1, 21381, 21382, 21383, -1, 21384, 21385, 21386, -1, 21387, 21388, 21389, -1, 21390, 21391, 21392, -1, 21393, 21394, 21395, -1, 21396, 21397, 21398, -1, 21399, 21400, 21401, -1, 21402, 21403, 21404, -1, 21405, 21406, 21407, -1, 21408, 21409, 21410, -1, 21411, 21412, 21413, -1, 21414, 21415, 21416, -1, 21417, 21418, 21419, -1, 21420, 21421, 21422, -1, 21423, 21424, 21425, -1, 21426, 21427, 21428, -1, 21429, 21430, 21431, -1, 21432, 21433, 21434, -1, 21435, 21436, 21437, -1, 21438, 21439, 21440, -1, 21441, 21442, 21443, -1, 21444, 21445, 21446, -1, 21447, 21448, 21449, -1, 21450, 21451, 21452, -1, 21453, 21454, 21455, -1, 21456, 21457, 21458, -1, 21459, 21460, 21461, -1, 21462, 21463, 21464, -1, 21465, 21466, 21467, -1, 21468, 21469, 21470, -1, 21471, 21472, 21473, -1, 21474, 21475, 21476, -1, 21477, 21478, 21479, -1, 21480, 21481, 21482, -1, 21483, 21484, 21485, -1, 21486, 21487, 21488, -1, 21489, 21490, 21491, -1, 21492, 21493, 21494, -1, 21495, 21496, 21497, -1, 21498, 21499, 21500, -1, 21501, 21502, 21503, -1, 21504, 21505, 21506, -1, 21507, 21508, 21509, -1, 21510, 21511, 21512, -1, 21513, 21514, 21515, -1, 21516, 21517, 21518, -1, 21519, 21520, 21521, -1, 21522, 21523, 21524, -1, 21525, 21526, 21527, -1, 21528, 21529, 21530, -1, 21531, 21532, 21533, -1, 21534, 21535, 21536, -1, 21537, 21538, 21539, -1, 21540, 21541, 21542, -1, 21543, 21544, 21545, -1, 21546, 21547, 21548, -1, 21549, 21550, 21551, -1, 21552, 21553, 21554, -1, 21555, 21556, 21557, -1, 21558, 21559, 21560, -1, 21561, 21562, 21563, -1, 21564, 21565, 21566, -1, 21567, 21568, 21569, -1, 21570, 21571, 21572, -1, 21573, 21574, 21575, -1, 21576, 21577, 21578, -1, 21579, 21580, 21581, -1, 21582, 21583, 21584, -1, 21585, 21586, 21587, -1, 21588, 21589, 21590, -1, 21591, 21592, 21593, -1, 21594, 21595, 21596, -1, 21597, 21598, 21599, -1, 21600, 21601, 21602, -1, 21603, 21604, 21605, -1, 21606, 21607, 21608, -1, 21609, 21610, 21611, -1, 21612, 21613, 21614, -1, 21615, 21616, 21617, -1, 21618, 21619, 21620, -1, 21621, 21622, 21623, -1, 21624, 21625, 21626, -1, 21627, 21628, 21629, -1, 21630, 21631, 21632, -1, 21633, 21634, 21635, -1, 21636, 21637, 21638, -1, 21639, 21640, 21641, -1, 21642, 21643, 21644, -1, 21645, 21646, 21647, -1, 21648, 21649, 21650, -1, 21651, 21652, 21653, -1, 21654, 21655, 21656, -1, 21657, 21658, 21659, -1, 21660, 21661, 21662, -1, 21663, 21664, 21665, -1, 21666, 21667, 21668, -1, 21669, 21670, 21671, -1, 21672, 21673, 21674, -1, 21675, 21676, 21677, -1, 21678, 21679, 21680, -1, 21681, 21682, 21683, -1, 21684, 21685, 21686, -1, 21687, 21688, 21689, -1, 21690, 21691, 21692, -1, 21693, 21694, 21695, -1, 21696, 21697, 21698, -1, 21699, 21700, 21701, -1, 21702, 21703, 21704, -1, 21705, 21706, 21707, -1, 21708, 21709, 21710, -1, 21711, 21712, 21713, -1, 21714, 21715, 21716, -1, 21717, 21718, 21719, -1, 21720, 21721, 21722, -1, 21723, 21724, 21725, -1, 21726, 21727, 21728, -1, 21729, 21730, 21731, -1, 21732, 21733, 21734, -1, 21735, 21736, 21737, -1, 21738, 21739, 21740, -1, 21741, 21742, 21743, -1, 21744, 21745, 21746, -1, 21747, 21748, 21749, -1, 21750, 21751, 21752, -1, 21753, 21754, 21755, -1, 21756, 21757, 21758, -1, 21759, 21760, 21761, -1, 21762, 21763, 21764, -1, 21765, 21766, 21767, -1, 21768, 21769, 21770, -1, 21771, 21772, 21773, -1, 21774, 21775, 21776, -1, 21777, 21778, 21779, -1, 21780, 21781, 21782, -1, 21783, 21784, 21785, -1, 21786, 21787, 21788, -1, 21789, 21790, 21791, -1, 21792, 21793, 21794, -1, 21795, 21796, 21797, -1, 21798, 21799, 21800, -1, 21801, 21802, 21803, -1, 21804, 21805, 21806, -1, 21807, 21808, 21809, -1, 21810, 21811, 21812, -1, 21813, 21814, 21815, -1, 21816, 21817, 21818, -1, 21819, 21820, 21821, -1, 21822, 21823, 21824, -1, 21825, 21826, 21827, -1, 21828, 21829, 21830, -1, 21831, 21832, 21833, -1, 21834, 21835, 21836, -1, 21837, 21838, 21839, -1, 21840, 21841, 21842, -1, 21843, 21844, 21845, -1, 21846, 21847, 21848, -1, 21849, 21850, 21851, -1, 21852, 21853, 21854, -1, 21855, 21856, 21857, -1, 21858, 21859, 21860, -1, 21861, 21862, 21863, -1, 21864, 21865, 21866, -1, 21867, 21868, 21869, -1, 21870, 21871, 21872, -1, 21873, 21874, 21875, -1, 21876, 21877, 21878, -1, 21879, 21880, 21881, -1, 21882, 21883, 21884, -1, 21885, 21886, 21887, -1, 21888, 21889, 21890, -1, 21891, 21892, 21893, -1, 21894, 21895, 21896, -1, 21897, 21898, 21899, -1, 21900, 21901, 21902, -1, 21903, 21904, 21905, -1, 21906, 21907, 21908, -1, 21909, 21910, 21911, -1, 21912, 21913, 21914, -1, 21915, 21916, 21917, -1, 21918, 21919, 21920, -1, 21921, 21922, 21923, -1, 21924, 21925, 21926, -1, 21927, 21928, 21929, -1, 21930, 21931, 21932, -1, 21933, 21934, 21935, -1, 21936, 21937, 21938, -1, 21939, 21940, 21941, -1, 21942, 21943, 21944, -1, 21945, 21946, 21947, -1, 21948, 21949, 21950, -1, 21951, 21952, 21953, -1, 21954, 21955, 21956, -1, 21957, 21958, 21959, -1, 21960, 21961, 21962, -1, 21963, 21964, 21965, -1, 21966, 21967, 21968, -1, 21969, 21970, 21971, -1, 21972, 21973, 21974, -1, 21975, 21976, 21977, -1, 21978, 21979, 21980, -1, 21981, 21982, 21983, -1, 21984, 21985, 21986, -1, 21987, 21988, 21989, -1, 21990, 21991, 21992, -1, 21993, 21994, 21995, -1, 21996, 21997, 21998, -1, 21999, 22000, 22001, -1, 22002, 22003, 22004, -1, 22005, 22006, 22007, -1, 22008, 22009, 22010, -1, 22011, 22012, 22013, -1, 22014, 22015, 22016, -1, 22017, 22018, 22019, -1, 22020, 22021, 22022, -1, 22023, 22024, 22025, -1, 22026, 22027, 22028, -1, 22029, 22030, 22031, -1, 22032, 22033, 22034, -1, 22035, 22036, 22037, -1, 22038, 22039, 22040, -1, 22041, 22042, 22043, -1, 22044, 22045, 22046, -1, 22047, 22048, 22049, -1, 22050, 22051, 22052, -1, 22053, 22054, 22055, -1, 22056, 22057, 22058, -1, 22059, 22060, 22061, -1, 22062, 22063, 22064, -1, 22065, 22066, 22067, -1, 22068, 22069, 22070, -1, 22071, 22072, 22073, -1, 22074, 22075, 22076, -1, 22077, 22078, 22079, -1, 22080, 22081, 22082, -1, 22083, 22084, 22085, -1, 22086, 22087, 22088, -1, 22089, 22090, 22091, -1, 22092, 22093, 22094, -1, 22095, 22096, 22097, -1, 22098, 22099, 22100, -1, 22101, 22102, 22103, -1, 22104, 22105, 22106, -1, 22107, 22108, 22109, -1, 22110, 22111, 22112, -1, 22113, 22114, 22115, -1, 22116, 22117, 22118, -1, 22119, 22120, 22121, -1, 22122, 22123, 22124, -1, 22125, 22126, 22127, -1, 22128, 22129, 22130, -1, 22131, 22132, 22133, -1, 22134, 22135, 22136, -1, 22137, 22138, 22139, -1, 22140, 22141, 22142, -1, 22143, 22144, 22145, -1, 22146, 22147, 22148, -1, 22149, 22150, 22151, -1, 22152, 22153, 22154, -1, 22155, 22156, 22157, -1, 22158, 22159, 22160, -1, 22161, 22162, 22163, -1, 22164, 22165, 22166, -1, 22167, 22168, 22169, -1, 22170, 22171, 22172, -1, 22173, 22174, 22175, -1, 22176, 22177, 22178, -1, 22179, 22180, 22181, -1, 22182, 22183, 22184, -1, 22185, 22186, 22187, -1, 22188, 22189, 22190, -1, 22191, 22192, 22193, -1, 22194, 22195, 22196, -1, 22197, 22198, 22199, -1, 22200, 22201, 22202, -1, 22203, 22204, 22205, -1, 22206, 22207, 22208, -1, 22209, 22210, 22211, -1, 22212, 22213, 22214, -1, 22215, 22216, 22217, -1, 22218, 22219, 22220, -1, 22221, 22222, 22223, -1, 22224, 22225, 22226, -1, 22227, 22228, 22229, -1, 22230, 22231, 22232, -1, 22233, 22234, 22235, -1, 22236, 22237, 22238, -1, 22239, 22240, 22241, -1, 22242, 22243, 22244, -1, 22245, 22246, 22247, -1, 22248, 22249, 22250, -1, 22251, 22252, 22253, -1, 22254, 22255, 22256, -1, 22257, 22258, 22259, -1, 22260, 22261, 22262, -1, 22263, 22264, 22265, -1, 22266, 22267, 22268, -1, 22269, 22270, 22271, -1, 22272, 22273, 22274, -1, 22275, 22276, 22277, -1, 22278, 22279, 22280, -1, 22281, 22282, 22283, -1, 22284, 22285, 22286, -1, 22287, 22288, 22289, -1, 22290, 22291, 22292, -1, 22293, 22294, 22295, -1, 22296, 22297, 22298, -1, 22299, 22300, 22301, -1, 22302, 22303, 22304, -1, 22305, 22306, 22307, -1, 22308, 22309, 22310, -1, 22311, 22312, 22313, -1, 22314, 22315, 22316, -1, 22317, 22318, 22319, -1, 22320, 22321, 22322, -1, 22323, 22324, 22325, -1, 22326, 22327, 22328, -1, 22329, 22330, 22331, -1, 22332, 22333, 22334, -1, 22335, 22336, 22337, -1, 22338, 22339, 22340, -1, 22341, 22342, 22343, -1, 22344, 22345, 22346, -1, 22347, 22348, 22349, -1, 22350, 22351, 22352, -1, 22353, 22354, 22355, -1, 22356, 22357, 22358, -1, 22359, 22360, 22361, -1, 22362, 22363, 22364, -1, 22365, 22366, 22367, -1, 22368, 22369, 22370, -1, 22371, 22372, 22373, -1, 22374, 22375, 22376, -1, 22377, 22378, 22379, -1, 22380, 22381, 22382, -1, 22383, 22384, 22385, -1, 22386, 22387, 22388, -1, 22389, 22390, 22391, -1, 22392, 22393, 22394, -1, 22395, 22396, 22397, -1, 22398, 22399, 22400, -1, 22401, 22402, 22403, -1, 22404, 22405, 22406, -1, 22407, 22408, 22409, -1, 22410, 22411, 22412, -1, 22413, 22414, 22415, -1, 22416, 22417, 22418, -1, 22419, 22420, 22421, -1, 22422, 22423, 22424, -1, 22425, 22426, 22427, -1, 22428, 22429, 22430, -1, 22431, 22432, 22433, -1, 22434, 22435, 22436, -1, 22437, 22438, 22439, -1, 22440, 22441, 22442, -1, 22443, 22444, 22445, -1, 22446, 22447, 22448, -1, 22449, 22450, 22451, -1, 22452, 22453, 22454, -1, 22455, 22456, 22457, -1, 22458, 22459, 22460, -1, 22461, 22462, 22463, -1, 22464, 22465, 22466, -1, 22467, 22468, 22469, -1, 22470, 22471, 22472, -1, 22473, 22474, 22475, -1, 22476, 22477, 22478, -1, 22479, 22480, 22481, -1, 22482, 22483, 22484, -1, 22485, 22486, 22487, -1, 22488, 22489, 22490, -1, 22491, 22492, 22493, -1, 22494, 22495, 22496, -1, 22497, 22498, 22499, -1, 22500, 22501, 22502, -1, 22503, 22504, 22505, -1, 22506, 22507, 22508, -1, 22509, 22510, 22511, -1, 22512, 22513, 22514, -1, 22515, 22516, 22517, -1, 22518, 22519, 22520, -1, 22521, 22522, 22523, -1, 22524, 22525, 22526, -1, 22527, 22528, 22529, -1, 22530, 22531, 22532, -1, 22533, 22534, 22535, -1, 22536, 22537, 22538, -1, 22539, 22540, 22541, -1, 22542, 22543, 22544, -1, 22545, 22546, 22547, -1, 22548, 22549, 22550, -1, 22551, 22552, 22553, -1, 22554, 22555, 22556, -1, 22557, 22558, 22559, -1, 22560, 22561, 22562, -1, 22563, 22564, 22565, -1, 22566, 22567, 22568, -1, 22569, 22570, 22571, -1, 22572, 22573, 22574, -1, 22575, 22576, 22577, -1, 22578, 22579, 22580, -1, 22581, 22582, 22583, -1, 22584, 22585, 22586, -1, 22587, 22588, 22589, -1, 22590, 22591, 22592, -1, 22593, 22594, 22595, -1, 22596, 22597, 22598, -1, 22599, 22600, 22601, -1, 22602, 22603, 22604, -1, 22605, 22606, 22607, -1, 22608, 22609, 22610, -1, 22611, 22612, 22613, -1, 22614, 22615, 22616, -1, 22617, 22618, 22619, -1, 22620, 22621, 22622, -1, 22623, 22624, 22625, -1, 22626, 22627, 22628, -1, 22629, 22630, 22631, -1, 22632, 22633, 22634, -1, 22635, 22636, 22637, -1, 22638, 22639, 22640, -1, 22641, 22642, 22643, -1, 22644, 22645, 22646, -1, 22647, 22648, 22649, -1, 22650, 22651, 22652, -1, 22653, 22654, 22655, -1, 22656, 22657, 22658, -1, 22659, 22660, 22661, -1, 22662, 22663, 22664, -1, 22665, 22666, 22667, -1, 22668, 22669, 22670, -1, 22671, 22672, 22673, -1, 22674, 22675, 22676, -1, 22677, 22678, 22679, -1, 22680, 22681, 22682, -1, 22683, 22684, 22685, -1, 22686, 22687, 22688, -1, 22689, 22690, 22691, -1, 22692, 22693, 22694, -1, 22695, 22696, 22697, -1, 22698, 22699, 22700, -1, 22701, 22702, 22703, -1, 22704, 22705, 22706, -1, 22707, 22708, 22709, -1, 22710, 22711, 22712, -1, 22713, 22714, 22715, -1, 22716, 22717, 22718, -1, 22719, 22720, 22721, -1, 22722, 22723, 22724, -1, 22725, 22726, 22727, -1, 22728, 22729, 22730, -1, 22731, 22732, 22733, -1, 22734, 22735, 22736, -1, 22737, 22738, 22739, -1, 22740, 22741, 22742, -1, 22743, 22744, 22745, -1, 22746, 22747, 22748, -1, 22749, 22750, 22751, -1, 22752, 22753, 22754, -1, 22755, 22756, 22757, -1, 22758, 22759, 22760, -1, 22761, 22762, 22763, -1, 22764, 22765, 22766, -1, 22767, 22768, 22769, -1, 22770, 22771, 22772, -1, 22773, 22774, 22775, -1, 22776, 22777, 22778, -1, 22779, 22780, 22781, -1, 22782, 22783, 22784, -1, 22785, 22786, 22787, -1, 22788, 22789, 22790, -1, 22791, 22792, 22793, -1, 22794, 22795, 22796, -1, 22797, 22798, 22799, -1, 22800, 22801, 22802, -1, 22803, 22804, 22805, -1, 22806, 22807, 22808, -1, 22809, 22810, 22811, -1, 22812, 22813, 22814, -1, 22815, 22816, 22817, -1, 22818, 22819, 22820, -1, 22821, 22822, 22823, -1, 22824, 22825, 22826, -1, 22827, 22828, 22829, -1, 22830, 22831, 22832, -1, 22833, 22834, 22835, -1, 22836, 22837, 22838, -1, 22839, 22840, 22841, -1, 22842, 22843, 22844, -1, 22845, 22846, 22847, -1, 22848, 22849, 22850, -1, 22851, 22852, 22853, -1, 22854, 22855, 22856, -1, 22857, 22858, 22859, -1, 22860, 22861, 22862, -1, 22863, 22864, 22865, -1, 22866, 22867, 22868, -1, 22869, 22870, 22871, -1, 22872, 22873, 22874, -1, 22875, 22876, 22877, -1, 22878, 22879, 22880, -1, 22881, 22882, 22883, -1, 22884, 22885, 22886, -1, 22887, 22888, 22889, -1, 22890, 22891, 22892, -1, 22893, 22894, 22895, -1, 22896, 22897, 22898, -1, 22899, 22900, 22901, -1, 22902, 22903, 22904, -1, 22905, 22906, 22907, -1, 22908, 22909, 22910, -1, 22911, 22912, 22913, -1, 22914, 22915, 22916, -1, 22917, 22918, 22919, -1, 22920, 22921, 22922, -1, 22923, 22924, 22925, -1, 22926, 22927, 22928, -1, 22929, 22930, 22931, -1, 22932, 22933, 22934, -1, 22935, 22936, 22937, -1, 22938, 22939, 22940, -1, 22941, 22942, 22943, -1, 22944, 22945, 22946, -1, 22947, 22948, 22949, -1, 22950, 22951, 22952, -1, 22953, 22954, 22955, -1, 22956, 22957, 22958, -1, 22959, 22960, 22961, -1, 22962, 22963, 22964, -1, 22965, 22966, 22967, -1, 22968, 22969, 22970, -1, 22971, 22972, 22973, -1, 22974, 22975, 22976, -1, 22977, 22978, 22979, -1, 22980, 22981, 22982, -1, 22983, 22984, 22985, -1, 22986, 22987, 22988, -1, 22989, 22990, 22991, -1, 22992, 22993, 22994, -1, 22995, 22996, 22997, -1, 22998, 22999, 23000, -1, 23001, 23002, 23003, -1, 23004, 23005, 23006, -1, 23007, 23008, 23009, -1, 23010, 23011, 23012, -1, 23013, 23014, 23015, -1, 23016, 23017, 23018, -1, 23019, 23020, 23021, -1, 23022, 23023, 23024, -1, 23025, 23026, 23027, -1, 23028, 23029, 23030, -1, 23031, 23032, 23033, -1, 23034, 23035, 23036, -1, 23037, 23038, 23039, -1, 23040, 23041, 23042, -1, 23043, 23044, 23045, -1, 23046, 23047, 23048, -1, 23049, 23050, 23051, -1, 23052, 23053, 23054, -1, 23055, 23056, 23057, -1, 23058, 23059, 23060, -1, 23061, 23062, 23063, -1, 23064, 23065, 23066, -1, 23067, 23068, 23069, -1, 23070, 23071, 23072, -1, 23073, 23074, 23075, -1, 23076, 23077, 23078, -1, 23079, 23080, 23081, -1, 23082, 23083, 23084, -1, 23085, 23086, 23087, -1, 23088, 23089, 23090, -1, 23091, 23092, 23093, -1, 23094, 23095, 23096, -1, 23097, 23098, 23099, -1, 23100, 23101, 23102, -1, 23103, 23104, 23105, -1, 23106, 23107, 23108, -1, 23109, 23110, 23111, -1, 23112, 23113, 23114, -1, 23115, 23116, 23117, -1, 23118, 23119, 23120, -1, 23121, 23122, 23123, -1, 23124, 23125, 23126, -1, 23127, 23128, 23129, -1, 23130, 23131, 23132, -1, 23133, 23134, 23135, -1, 23136, 23137, 23138, -1, 23139, 23140, 23141, -1, 23142, 23143, 23144, -1, 23145, 23146, 23147, -1, 23148, 23149, 23150, -1, 23151, 23152, 23153, -1, 23154, 23155, 23156, -1, 23157, 23158, 23159, -1, 23160, 23161, 23162, -1, 23163, 23164, 23165, -1, 23166, 23167, 23168, -1, 23169, 23170, 23171, -1, 23172, 23173, 23174, -1, 23175, 23176, 23177, -1, 23178, 23179, 23180, -1, 23181, 23182, 23183, -1, 23184, 23185, 23186, -1, 23187, 23188, 23189, -1, 23190, 23191, 23192, -1, 23193, 23194, 23195, -1, 23196, 23197, 23198, -1, 23199, 23200, 23201, -1, 23202, 23203, 23204, -1, 23205, 23206, 23207, -1, 23208, 23209, 23210, -1, 23211, 23212, 23213, -1, 23214, 23215, 23216, -1, 23217, 23218, 23219, -1, 23220, 23221, 23222, -1, 23223, 23224, 23225, -1, 23226, 23227, 23228, -1, 23229, 23230, 23231, -1, 23232, 23233, 23234, -1, 23235, 23236, 23237, -1, 23238, 23239, 23240, -1, 23241, 23242, 23243, -1, 23244, 23245, 23246, -1, 23247, 23248, 23249, -1, 23250, 23251, 23252, -1, 23253, 23254, 23255, -1, 23256, 23257, 23258, -1, 23259, 23260, 23261, -1, 23262, 23263, 23264, -1, 23265, 23266, 23267, -1, 23268, 23269, 23270, -1, 23271, 23272, 23273, -1, 23274, 23275, 23276, -1, 23277, 23278, 23279, -1, 23280, 23281, 23282, -1, 23283, 23284, 23285, -1, 23286, 23287, 23288, -1, 23289, 23290, 23291, -1, 23292, 23293, 23294, -1, 23295, 23296, 23297, -1, 23298, 23299, 23300, -1, 23301, 23302, 23303, -1, 23304, 23305, 23306, -1, 23307, 23308, 23309, -1, 23310, 23311, 23312, -1, 23313, 23314, 23315, -1, 23316, 23317, 23318, -1, 23319, 23320, 23321, -1, 23322, 23323, 23324, -1, 23325, 23326, 23327, -1, 23328, 23329, 23330, -1, 23331, 23332, 23333, -1, 23334, 23335, 23336, -1, 23337, 23338, 23339, -1, 23340, 23341, 23342, -1, 23343, 23344, 23345, -1, 23346, 23347, 23348, -1, 23349, 23350, 23351, -1, 23352, 23353, 23354, -1, 23355, 23356, 23357, -1, 23358, 23359, 23360, -1, 23361, 23362, 23363, -1, 23364, 23365, 23366, -1, 23367, 23368, 23369, -1, 23370, 23371, 23372, -1, 23373, 23374, 23375, -1, 23376, 23377, 23378, -1, 23379, 23380, 23381, -1, 23382, 23383, 23384, -1, 23385, 23386, 23387, -1, 23388, 23389, 23390, -1, 23391, 23392, 23393, -1, 23394, 23395, 23396, -1, 23397, 23398, 23399, -1, 23400, 23401, 23402, -1, 23403, 23404, 23405, -1, 23406, 23407, 23408, -1, 23409, 23410, 23411, -1, 23412, 23413, 23414, -1, 23415, 23416, 23417, -1, 23418, 23419, 23420, -1, 23421, 23422, 23423, -1, 23424, 23425, 23426, -1, 23427, 23428, 23429, -1, 23430, 23431, 23432, -1, 23433, 23434, 23435, -1, 23436, 23437, 23438, -1, 23439, 23440, 23441, -1, 23442, 23443, 23444, -1, 23445, 23446, 23447, -1, 23448, 23449, 23450, -1, 23451, 23452, 23453, -1, 23454, 23455, 23456, -1, 23457, 23458, 23459, -1, 23460, 23461, 23462, -1, 23463, 23464, 23465, -1, 23466, 23467, 23468, -1, 23469, 23470, 23471, -1, 23472, 23473, 23474, -1, 23475, 23476, 23477, -1, 23478, 23479, 23480, -1, 23481, 23482, 23483, -1, 23484, 23485, 23486, -1, 23487, 23488, 23489, -1, 23490, 23491, 23492, -1, 23493, 23494, 23495, -1, 23496, 23497, 23498, -1, 23499, 23500, 23501, -1, 23502, 23503, 23504, -1, 23505, 23506, 23507, -1, 23508, 23509, 23510, -1, 23511, 23512, 23513, -1, 23514, 23515, 23516, -1, 23517, 23518, 23519, -1, 23520, 23521, 23522, -1, 23523, 23524, 23525, -1, 23526, 23527, 23528, -1, 23529, 23530, 23531, -1, 23532, 23533, 23534, -1, 23535, 23536, 23537, -1, 23538, 23539, 23540, -1, 23541, 23542, 23543, -1, 23544, 23545, 23546, -1, 23547, 23548, 23549, -1, 23550, 23551, 23552, -1, 23553, 23554, 23555, -1, 23556, 23557, 23558, -1, 23559, 23560, 23561, -1, 23562, 23563, 23564, -1, 23565, 23566, 23567, -1, 23568, 23569, 23570, -1, 23571, 23572, 23573, -1, 23574, 23575, 23576, -1, 23577, 23578, 23579, -1, 23580, 23581, 23582, -1, 23583, 23584, 23585, -1, 23586, 23587, 23588, -1, 23589, 23590, 23591, -1, 23592, 23593, 23594, -1, 23595, 23596, 23597, -1, 23598, 23599, 23600, -1, 23601, 23602, 23603, -1, 23604, 23605, 23606, -1, 23607, 23608, 23609, -1, 23610, 23611, 23612, -1, 23613, 23614, 23615, -1, 23616, 23617, 23618, -1, 23619, 23620, 23621, -1, 23622, 23623, 23624, -1, 23625, 23626, 23627, -1, 23628, 23629, 23630, -1, 23631, 23632, 23633, -1, 23634, 23635, 23636, -1, 23637, 23638, 23639, -1, 23640, 23641, 23642, -1, 23643, 23644, 23645, -1, 23646, 23647, 23648, -1, 23649, 23650, 23651, -1, 23652, 23653, 23654, -1, 23655, 23656, 23657, -1, 23658, 23659, 23660, -1, 23661, 23662, 23663, -1, 23664, 23665, 23666, -1, 23667, 23668, 23669, -1, 23670, 23671, 23672, -1, 23673, 23674, 23675, -1, 23676, 23677, 23678, -1, 23679, 23680, 23681, -1, 23682, 23683, 23684, -1, 23685, 23686, 23687, -1, 23688, 23689, 23690, -1, 23691, 23692, 23693, -1, 23694, 23695, 23696, -1, 23697, 23698, 23699, -1, 23700, 23701, 23702, -1, 23703, 23704, 23705, -1, 23706, 23707, 23708, -1, 23709, 23710, 23711, -1, 23712, 23713, 23714, -1, 23715, 23716, 23717, -1, 23718, 23719, 23720, -1, 23721, 23722, 23723, -1, 23724, 23725, 23726, -1, 23727, 23728, 23729, -1, 23730, 23731, 23732, -1, 23733, 23734, 23735, -1, 23736, 23737, 23738, -1, 23739, 23740, 23741, -1, 23742, 23743, 23744, -1, 23745, 23746, 23747, -1, 23748, 23749, 23750, -1, 23751, 23752, 23753, -1, 23754, 23755, 23756, -1, 23757, 23758, 23759, -1, 23760, 23761, 23762, -1, 23763, 23764, 23765, -1, 23766, 23767, 23768, -1, 23769, 23770, 23771, -1, 23772, 23773, 23774, -1, 23775, 23776, 23777, -1, 23778, 23779, 23780, -1, 23781, 23782, 23783, -1, 23784, 23785, 23786, -1, 23787, 23788, 23789, -1, 23790, 23791, 23792, -1, 23793, 23794, 23795, -1, 23796, 23797, 23798, -1, 23799, 23800, 23801, -1, 23802, 23803, 23804, -1, 23805, 23806, 23807, -1, 23808, 23809, 23810, -1, 23811, 23812, 23813, -1, 23814, 23815, 23816, -1, 23817, 23818, 23819, -1, 23820, 23821, 23822, -1, 23823, 23824, 23825, -1, 23826, 23827, 23828, -1, 23829, 23830, 23831, -1, 23832, 23833, 23834, -1, 23835, 23836, 23837, -1, 23838, 23839, 23840, -1, 23841, 23842, 23843, -1, 23844, 23845, 23846, -1, 23847, 23848, 23849, -1, 23850, 23851, 23852, -1, 23853, 23854, 23855, -1, 23856, 23857, 23858, -1, 23859, 23860, 23861, -1, 23862, 23863, 23864, -1, 23865, 23866, 23867, -1, 23868, 23869, 23870, -1, 23871, 23872, 23873, -1, 23874, 23875, 23876, -1, 23877, 23878, 23879, -1, 23880, 23881, 23882, -1, 23883, 23884, 23885, -1, 23886, 23887, 23888, -1, 23889, 23890, 23891, -1, 23892, 23893, 23894, -1, 23895, 23896, 23897, -1, 23898, 23899, 23900, -1, 23901, 23902, 23903, -1, 23904, 23905, 23906, -1, 23907, 23908, 23909, -1, 23910, 23911, 23912, -1, 23913, 23914, 23915, -1, 23916, 23917, 23918, -1, 23919, 23920, 23921, -1, 23922, 23923, 23924, -1, 23925, 23926, 23927, -1, 23928, 23929, 23930, -1, 23931, 23932, 23933, -1, 23934, 23935, 23936, -1, 23937, 23938, 23939, -1, 23940, 23941, 23942, -1, 23943, 23944, 23945, -1, 23946, 23947, 23948, -1, 23949, 23950, 23951, -1, 23952, 23953, 23954, -1, 23955, 23956, 23957, -1, 23958, 23959, 23960, -1, 23961, 23962, 23963, -1, 23964, 23965, 23966, -1, 23967, 23968, 23969, -1, 23970, 23971, 23972, -1, 23973, 23974, 23975, -1, 23976, 23977, 23978, -1, 23979, 23980, 23981, -1, 23982, 23983, 23984, -1, 23985, 23986, 23987, -1, 23988, 23989, 23990, -1, 23991, 23992, 23993, -1, 23994, 23995, 23996, -1, 23997, 23998, 23999, -1, 24000, 24001, 24002, -1, 24003, 24004, 24005, -1, 24006, 24007, 24008, -1, 24009, 24010, 24011, -1, 24012, 24013, 24014, -1, 24015, 24016, 24017, -1, 24018, 24019, 24020, -1, 24021, 24022, 24023, -1, 24024, 24025, 24026, -1, 24027, 24028, 24029, -1, 24030, 24031, 24032, -1, 24033, 24034, 24035, -1, 24036, 24037, 24038, -1, 24039, 24040, 24041, -1, 24042, 24043, 24044, -1, 24045, 24046, 24047, -1, 24048, 24049, 24050, -1, 24051, 24052, 24053, -1, 24054, 24055, 24056, -1, 24057, 24058, 24059, -1, 24060, 24061, 24062, -1, 24063, 24064, 24065, -1, 24066, 24067, 24068, -1, 24069, 24070, 24071, -1, 24072, 24073, 24074, -1, 24075, 24076, 24077, -1, 24078, 24079, 24080, -1, 24081, 24082, 24083, -1, 24084, 24085, 24086, -1, 24087, 24088, 24089, -1, 24090, 24091, 24092, -1, 24093, 24094, 24095, -1, 24096, 24097, 24098, -1, 24099, 24100, 24101, -1, 24102, 24103, 24104, -1, 24105, 24106, 24107, -1, 24108, 24109, 24110, -1, 24111, 24112, 24113, -1, 24114, 24115, 24116, -1, 24117, 24118, 24119, -1, 24120, 24121, 24122, -1, 24123, 24124, 24125, -1, 24126, 24127, 24128, -1, 24129, 24130, 24131, -1, 24132, 24133, 24134, -1, 24135, 24136, 24137, -1, 24138, 24139, 24140, -1, 24141, 24142, 24143, -1, 24144, 24145, 24146, -1, 24147, 24148, 24149, -1, 24150, 24151, 24152, -1, 24153, 24154, 24155, -1, 24156, 24157, 24158, -1, 24159, 24160, 24161, -1, 24162, 24163, 24164, -1, 24165, 24166, 24167, -1, 24168, 24169, 24170, -1, 24171, 24172, 24173, -1, 24174, 24175, 24176, -1, 24177, 24178, 24179, -1, 24180, 24181, 24182, -1, 24183, 24184, 24185, -1, 24186, 24187, 24188, -1, 24189, 24190, 24191, -1, 24192, 24193, 24194, -1, 24195, 24196, 24197, -1, 24198, 24199, 24200, -1, 24201, 24202, 24203, -1, 24204, 24205, 24206, -1, 24207, 24208, 24209, -1, 24210, 24211, 24212, -1, 24213, 24214, 24215, -1, 24216, 24217, 24218, -1, 24219, 24220, 24221, -1, 24222, 24223, 24224, -1, 24225, 24226, 24227, -1, 24228, 24229, 24230, -1, 24231, 24232, 24233, -1, 24234, 24235, 24236, -1, 24237, 24238, 24239, -1, 24240, 24241, 24242, -1, 24243, 24244, 24245, -1, 24246, 24247, 24248, -1, 24249, 24250, 24251, -1, 24252, 24253, 24254, -1, 24255, 24256, 24257, -1, 24258, 24259, 24260, -1, 24261, 24262, 24263, -1, 24264, 24265, 24266, -1, 24267, 24268, 24269, -1, 24270, 24271, 24272, -1, 24273, 24274, 24275, -1, 24276, 24277, 24278, -1, 24279, 24280, 24281, -1, 24282, 24283, 24284, -1, 24285, 24286, 24287, -1, 24288, 24289, 24290, -1, 24291, 24292, 24293, -1, 24294, 24295, 24296, -1, 24297, 24298, 24299, -1, 24300, 24301, 24302, -1, 24303, 24304, 24305, -1, 24306, 24307, 24308, -1, 24309, 24310, 24311, -1, 24312, 24313, 24314, -1, 24315, 24316, 24317, -1, 24318, 24319, 24320, -1, 24321, 24322, 24323, -1, 24324, 24325, 24326, -1, 24327, 24328, 24329, -1, 24330, 24331, 24332, -1, 24333, 24334, 24335, -1, 24336, 24337, 24338, -1, 24339, 24340, 24341, -1, 24342, 24343, 24344, -1, 24345, 24346, 24347, -1, 24348, 24349, 24350, -1, 24351, 24352, 24353, -1, 24354, 24355, 24356, -1, 24357, 24358, 24359, -1, 24360, 24361, 24362, -1, 24363, 24364, 24365, -1, 24366, 24367, 24368, -1, 24369, 24370, 24371, -1, 24372, 24373, 24374, -1, 24375, 24376, 24377, -1, 24378, 24379, 24380, -1, 24381, 24382, 24383, -1, 24384, 24385, 24386, -1, 24387, 24388, 24389, -1, 24390, 24391, 24392, -1, 24393, 24394, 24395, -1, 24396, 24397, 24398, -1, 24399, 24400, 24401, -1, 24402, 24403, 24404, -1, 24405, 24406, 24407, -1, 24408, 24409, 24410, -1, 24411, 24412, 24413, -1, 24414, 24415, 24416, -1, 24417, 24418, 24419, -1, 24420, 24421, 24422, -1, 24423, 24424, 24425, -1, 24426, 24427, 24428, -1, 24429, 24430, 24431, -1, 24432, 24433, 24434, -1, 24435, 24436, 24437, -1, 24438, 24439, 24440, -1, 24441, 24442, 24443, -1, 24444, 24445, 24446, -1, 24447, 24448, 24449, -1, 24450, 24451, 24452, -1, 24453, 24454, 24455, -1, 24456, 24457, 24458, -1, 24459, 24460, 24461, -1, 24462, 24463, 24464, -1, 24465, 24466, 24467, -1, 24468, 24469, 24470, -1, 24471, 24472, 24473, -1, 24474, 24475, 24476, -1, 24477, 24478, 24479, -1, 24480, 24481, 24482, -1, 24483, 24484, 24485, -1, 24486, 24487, 24488, -1, 24489, 24490, 24491, -1, 24492, 24493, 24494, -1, 24495, 24496, 24497, -1, 24498, 24499, 24500, -1, 24501, 24502, 24503, -1, 24504, 24505, 24506, -1, 24507, 24508, 24509, -1, 24510, 24511, 24512, -1, 24513, 24514, 24515, -1, 24516, 24517, 24518, -1, 24519, 24520, 24521, -1, 24522, 24523, 24524, -1, 24525, 24526, 24527, -1, 24528, 24529, 24530, -1, 24531, 24532, 24533, -1, 24534, 24535, 24536, -1, 24537, 24538, 24539, -1, 24540, 24541, 24542, -1, 24543, 24544, 24545, -1, 24546, 24547, 24548, -1, 24549, 24550, 24551, -1, 24552, 24553, 24554, -1, 24555, 24556, 24557, -1, 24558, 24559, 24560, -1, 24561, 24562, 24563, -1, 24564, 24565, 24566, -1, 24567, 24568, 24569, -1, 24570, 24571, 24572, -1, 24573, 24574, 24575, -1, 24576, 24577, 24578, -1, 24579, 24580, 24581, -1, 24582, 24583, 24584, -1, 24585, 24586, 24587, -1, 24588, 24589, 24590, -1, 24591, 24592, 24593, -1, 24594, 24595, 24596, -1, 24597, 24598, 24599, -1, 24600, 24601, 24602, -1, 24603, 24604, 24605, -1, 24606, 24607, 24608, -1, 24609, 24610, 24611, -1, 24612, 24613, 24614, -1, 24615, 24616, 24617, -1, 24618, 24619, 24620, -1, 24621, 24622, 24623, -1, 24624, 24625, 24626, -1, 24627, 24628, 24629, -1, 24630, 24631, 24632, -1, 24633, 24634, 24635, -1, 24636, 24637, 24638, -1, 24639, 24640, 24641, -1, 24642, 24643, 24644, -1, 24645, 24646, 24647, -1, 24648, 24649, 24650, -1, 24651, 24652, 24653, -1, 24654, 24655, 24656, -1, 24657, 24658, 24659, -1, 24660, 24661, 24662, -1, 24663, 24664, 24665, -1, 24666, 24667, 24668, -1, 24669, 24670, 24671, -1, 24672, 24673, 24674, -1, 24675, 24676, 24677, -1, 24678, 24679, 24680, -1, 24681, 24682, 24683, -1, 24684, 24685, 24686, -1, 24687, 24688, 24689, -1, 24690, 24691, 24692, -1, 24693, 24694, 24695, -1, 24696, 24697, 24698, -1, 24699, 24700, 24701, -1, 24702, 24703, 24704, -1, 24705, 24706, 24707, -1, 24708, 24709, 24710, -1, 24711, 24712, 24713, -1, 24714, 24715, 24716, -1, 24717, 24718, 24719, -1, 24720, 24721, 24722, -1, 24723, 24724, 24725, -1, 24726, 24727, 24728, -1, 24729, 24730, 24731, -1, 24732, 24733, 24734, -1, 24735, 24736, 24737, -1, 24738, 24739, 24740, -1, 24741, 24742, 24743, -1, 24744, 24745, 24746, -1, 24747, 24748, 24749, -1, 24750, 24751, 24752, -1, 24753, 24754, 24755, -1, 24756, 24757, 24758, -1, 24759, 24760, 24761, -1, 24762, 24763, 24764, -1, 24765, 24766, 24767, -1, 24768, 24769, 24770, -1, 24771, 24772, 24773, -1, 24774, 24775, 24776, -1, 24777, 24778, 24779, -1, 24780, 24781, 24782, -1, 24783, 24784, 24785, -1, 24786, 24787, 24788, -1, 24789, 24790, 24791, -1, 24792, 24793, 24794, -1, 24795, 24796, 24797, -1, 24798, 24799, 24800, -1, 24801, 24802, 24803, -1, 24804, 24805, 24806, -1, 24807, 24808, 24809, -1, 24810, 24811, 24812, -1, 24813, 24814, 24815, -1, 24816, 24817, 24818, -1, 24819, 24820, 24821, -1, 24822, 24823, 24824, -1, 24825, 24826, 24827, -1, 24828, 24829, 24830, -1, 24831, 24832, 24833, -1, 24834, 24835, 24836, -1, 24837, 24838, 24839, -1, 24840, 24841, 24842, -1, 24843, 24844, 24845, -1, 24846, 24847, 24848, -1, 24849, 24850, 24851, -1, 24852, 24853, 24854, -1, 24855, 24856, 24857, -1, 24858, 24859, 24860, -1, 24861, 24862, 24863, -1, 24864, 24865, 24866, -1, 24867, 24868, 24869, -1, 24870, 24871, 24872, -1, 24873, 24874, 24875, -1, 24876, 24877, 24878, -1, 24879, 24880, 24881, -1, 24882, 24883, 24884, -1, 24885, 24886, 24887, -1, 24888, 24889, 24890, -1, 24891, 24892, 24893, -1, 24894, 24895, 24896, -1, 24897, 24898, 24899, -1, 24900, 24901, 24902, -1, 24903, 24904, 24905, -1, 24906, 24907, 24908, -1, 24909, 24910, 24911, -1, 24912, 24913, 24914, -1, 24915, 24916, 24917, -1, 24918, 24919, 24920, -1, 24921, 24922, 24923, -1, 24924, 24925, 24926, -1, 24927, 24928, 24929, -1, 24930, 24931, 24932, -1, 24933, 24934, 24935, -1, 24936, 24937, 24938, -1, 24939, 24940, 24941, -1, 24942, 24943, 24944, -1, 24945, 24946, 24947, -1, 24948, 24949, 24950, -1, 24951, 24952, 24953, -1, 24954, 24955, 24956, -1, 24957, 24958, 24959, -1, 24960, 24961, 24962, -1, 24963, 24964, 24965, -1, 24966, 24967, 24968, -1, 24969, 24970, 24971, -1, 24972, 24973, 24974, -1, 24975, 24976, 24977, -1, 24978, 24979, 24980, -1, 24981, 24982, 24983, -1, 24984, 24985, 24986, -1, 24987, 24988, 24989, -1, 24990, 24991, 24992, -1, 24993, 24994, 24995, -1, 24996, 24997, 24998, -1, 24999, 25000, 25001, -1, 25002, 25003, 25004, -1, 25005, 25006, 25007, -1, 25008, 25009, 25010, -1, 25011, 25012, 25013, -1, 25014, 25015, 25016, -1, 25017, 25018, 25019, -1, 25020, 25021, 25022, -1, 25023, 25024, 25025, -1, 25026, 25027, 25028, -1, 25029, 25030, 25031, -1, 25032, 25033, 25034, -1, 25035, 25036, 25037, -1, 25038, 25039, 25040, -1, 25041, 25042, 25043, -1, 25044, 25045, 25046, -1, 25047, 25048, 25049, -1, 25050, 25051, 25052, -1, 25053, 25054, 25055, -1, 25056, 25057, 25058, -1, 25059, 25060, 25061, -1, 25062, 25063, 25064, -1, 25065, 25066, 25067, -1, 25068, 25069, 25070, -1, 25071, 25072, 25073, -1, 25074, 25075, 25076, -1, 25077, 25078, 25079, -1, 25080, 25081, 25082, -1, 25083, 25084, 25085, -1, 25086, 25087, 25088, -1, 25089, 25090, 25091, -1, 25092, 25093, 25094, -1, 25095, 25096, 25097, -1, 25098, 25099, 25100, -1, 25101, 25102, 25103, -1, 25104, 25105, 25106, -1, 25107, 25108, 25109, -1, 25110, 25111, 25112, -1, 25113, 25114, 25115, -1, 25116, 25117, 25118, -1, 25119, 25120, 25121, -1, 25122, 25123, 25124, -1, 25125, 25126, 25127, -1, 25128, 25129, 25130, -1, 25131, 25132, 25133, -1, 25134, 25135, 25136, -1, 25137, 25138, 25139, -1, 25140, 25141, 25142, -1, 25143, 25144, 25145, -1, 25146, 25147, 25148, -1, 25149, 25150, 25151, -1, 25152, 25153, 25154, -1, 25155, 25156, 25157, -1, 25158, 25159, 25160, -1, 25161, 25162, 25163, -1, 25164, 25165, 25166, -1, 25167, 25168, 25169, -1, 25170, 25171, 25172, -1, 25173, 25174, 25175, -1, 25176, 25177, 25178, -1, 25179, 25180, 25181, -1, 25182, 25183, 25184, -1, 25185, 25186, 25187, -1, 25188, 25189, 25190, -1, 25191, 25192, 25193, -1, 25194, 25195, 25196, -1, 25197, 25198, 25199, -1, 25200, 25201, 25202, -1, 25203, 25204, 25205, -1, 25206, 25207, 25208, -1, 25209, 25210, 25211, -1, 25212, 25213, 25214, -1, 25215, 25216, 25217, -1, 25218, 25219, 25220, -1, 25221, 25222, 25223, -1, 25224, 25225, 25226, -1, 25227, 25228, 25229, -1, 25230, 25231, 25232, -1, 25233, 25234, 25235, -1, 25236, 25237, 25238, -1, 25239, 25240, 25241, -1, 25242, 25243, 25244, -1, 25245, 25246, 25247, -1, 25248, 25249, 25250, -1, 25251, 25252, 25253, -1, 25254, 25255, 25256, -1, 25257, 25258, 25259, -1, 25260, 25261, 25262, -1, 25263, 25264, 25265, -1, 25266, 25267, 25268, -1, 25269, 25270, 25271, -1, 25272, 25273, 25274, -1, 25275, 25276, 25277, -1, 25278, 25279, 25280, -1, 25281, 25282, 25283, -1, 25284, 25285, 25286, -1, 25287, 25288, 25289, -1, 25290, 25291, 25292, -1, 25293, 25294, 25295, -1, 25296, 25297, 25298, -1, 25299, 25300, 25301, -1, 25302, 25303, 25304, -1, 25305, 25306, 25307, -1, 25308, 25309, 25310, -1, 25311, 25312, 25313, -1, 25314, 25315, 25316, -1, 25317, 25318, 25319, -1, 25320, 25321, 25322, -1, 25323, 25324, 25325, -1, 25326, 25327, 25328, -1, 25329, 25330, 25331, -1, 25332, 25333, 25334, -1, 25335, 25336, 25337, -1, 25338, 25339, 25340, -1, 25341, 25342, 25343, -1, 25344, 25345, 25346, -1, 25347, 25348, 25349, -1, 25350, 25351, 25352, -1, 25353, 25354, 25355, -1, 25356, 25357, 25358, -1, 25359, 25360, 25361, -1, 25362, 25363, 25364, -1, 25365, 25366, 25367, -1, 25368, 25369, 25370, -1, 25371, 25372, 25373, -1, 25374, 25375, 25376, -1, 25377, 25378, 25379, -1, 25380, 25381, 25382, -1, 25383, 25384, 25385, -1, 25386, 25387, 25388, -1, 25389, 25390, 25391, -1, 25392, 25393, 25394, -1, 25395, 25396, 25397, -1, 25398, 25399, 25400, -1, 25401, 25402, 25403, -1, 25404, 25405, 25406, -1, 25407, 25408, 25409, -1, 25410, 25411, 25412, -1, 25413, 25414, 25415, -1, 25416, 25417, 25418, -1, 25419, 25420, 25421, -1, 25422, 25423, 25424, -1, 25425, 25426, 25427, -1, 25428, 25429, 25430, -1, 25431, 25432, 25433, -1, 25434, 25435, 25436, -1, 25437, 25438, 25439, -1, 25440, 25441, 25442, -1, 25443, 25444, 25445, -1, 25446, 25447, 25448, -1, 25449, 25450, 25451, -1, 25452, 25453, 25454, -1, 25455, 25456, 25457, -1, 25458, 25459, 25460, -1, 25461, 25462, 25463, -1, 25464, 25465, 25466, -1, 25467, 25468, 25469, -1, 25470, 25471, 25472, -1, 25473, 25474, 25475, -1, 25476, 25477, 25478, -1, 25479, 25480, 25481, -1, 25482, 25483, 25484, -1, 25485, 25486, 25487, -1, 25488, 25489, 25490, -1, 25491, 25492, 25493, -1, 25494, 25495, 25496, -1, 25497, 25498, 25499, -1, 25500, 25501, 25502, -1, 25503, 25504, 25505, -1, 25506, 25507, 25508, -1, 25509, 25510, 25511, -1, 25512, 25513, 25514, -1, 25515, 25516, 25517, -1, 25518, 25519, 25520, -1, 25521, 25522, 25523, -1, 25524, 25525, 25526, -1, 25527, 25528, 25529, -1, 25530, 25531, 25532, -1, 25533, 25534, 25535, -1, 25536, 25537, 25538, -1, 25539, 25540, 25541, -1, 25542, 25543, 25544, -1, 25545, 25546, 25547, -1, 25548, 25549, 25550, -1, 25551, 25552, 25553, -1, 25554, 25555, 25556, -1, 25557, 25558, 25559, -1, 25560, 25561, 25562, -1, 25563, 25564, 25565, -1, 25566, 25567, 25568, -1, 25569, 25570, 25571, -1, 25572, 25573, 25574, -1, 25575, 25576, 25577, -1, 25578, 25579, 25580, -1, 25581, 25582, 25583, -1, 25584, 25585, 25586, -1, 25587, 25588, 25589, -1, 25590, 25591, 25592, -1, 25593, 25594, 25595, -1, 25596, 25597, 25598, -1, 25599, 25600, 25601, -1, 25602, 25603, 25604, -1, 25605, 25606, 25607, -1, 25608, 25609, 25610, -1, 25611, 25612, 25613, -1, 25614, 25615, 25616, -1, 25617, 25618, 25619, -1, 25620, 25621, 25622, -1, 25623, 25624, 25625, -1, 25626, 25627, 25628, -1, 25629, 25630, 25631, -1, 25632, 25633, 25634, -1, 25635, 25636, 25637, -1, 25638, 25639, 25640, -1, 25641, 25642, 25643, -1, 25644, 25645, 25646, -1, 25647, 25648, 25649, -1, 25650, 25651, 25652, -1, 25653, 25654, 25655, -1, 25656, 25657, 25658, -1, 25659, 25660, 25661, -1, 25662, 25663, 25664, -1, 25665, 25666, 25667, -1, 25668, 25669, 25670, -1, 25671, 25672, 25673, -1, 25674, 25675, 25676, -1, 25677, 25678, 25679, -1, 25680, 25681, 25682, -1, 25683, 25684, 25685, -1, 25686, 25687, 25688, -1, 25689, 25690, 25691, -1, 25692, 25693, 25694, -1, 25695, 25696, 25697, -1, 25698, 25699, 25700, -1, 25701, 25702, 25703, -1, 25704, 25705, 25706, -1, 25707, 25708, 25709, -1, 25710, 25711, 25712, -1, 25713, 25714, 25715, -1, 25716, 25717, 25718, -1, 25719, 25720, 25721, -1, 25722, 25723, 25724, -1, 25725, 25726, 25727, -1, 25728, 25729, 25730, -1, 25731, 25732, 25733, -1, 25734, 25735, 25736, -1, 25737, 25738, 25739, -1, 25740, 25741, 25742, -1, 25743, 25744, 25745, -1, 25746, 25747, 25748, -1, 25749, 25750, 25751, -1, 25752, 25753, 25754, -1, 25755, 25756, 25757, -1, 25758, 25759, 25760, -1, 25761, 25762, 25763, -1, 25764, 25765, 25766, -1, 25767, 25768, 25769, -1, 25770, 25771, 25772, -1, 25773, 25774, 25775, -1, 25776, 25777, 25778, -1, 25779, 25780, 25781, -1, 25782, 25783, 25784, -1, 25785, 25786, 25787, -1, 25788, 25789, 25790, -1, 25791, 25792, 25793, -1, 25794, 25795, 25796, -1, 25797, 25798, 25799, -1, 25800, 25801, 25802, -1, 25803, 25804, 25805, -1, 25806, 25807, 25808, -1, 25809, 25810, 25811, -1, 25812, 25813, 25814, -1, 25815, 25816, 25817, -1, 25818, 25819, 25820, -1, 25821, 25822, 25823, -1, 25824, 25825, 25826, -1, 25827, 25828, 25829, -1, 25830, 25831, 25832, -1, 25833, 25834, 25835, -1, 25836, 25837, 25838, -1, 25839, 25840, 25841, -1, 25842, 25843, 25844, -1, 25845, 25846, 25847, -1, 25848, 25849, 25850, -1, 25851, 25852, 25853, -1, 25854, 25855, 25856, -1, 25857, 25858, 25859, -1, 25860, 25861, 25862, -1, 25863, 25864, 25865, -1, 25866, 25867, 25868, -1, 25869, 25870, 25871, -1, 25872, 25873, 25874, -1, 25875, 25876, 25877, -1, 25878, 25879, 25880, -1, 25881, 25882, 25883, -1, 25884, 25885, 25886, -1, 25887, 25888, 25889, -1, 25890, 25891, 25892, -1, 25893, 25894, 25895, -1, 25896, 25897, 25898, -1, 25899, 25900, 25901, -1, 25902, 25903, 25904, -1, 25905, 25906, 25907, -1, 25908, 25909, 25910, -1, 25911, 25912, 25913, -1, 25914, 25915, 25916, -1, 25917, 25918, 25919, -1, 25920, 25921, 25922, -1, 25923, 25924, 25925, -1, 25926, 25927, 25928, -1, 25929, 25930, 25931, -1, 25932, 25933, 25934, -1, 25935, 25936, 25937, -1, 25938, 25939, 25940, -1, 25941, 25942, 25943, -1, 25944, 25945, 25946, -1, 25947, 25948, 25949, -1, 25950, 25951, 25952, -1, 25953, 25954, 25955, -1, 25956, 25957, 25958, -1, 25959, 25960, 25961, -1, 25962, 25963, 25964, -1, 25965, 25966, 25967, -1, 25968, 25969, 25970, -1, 25971, 25972, 25973, -1, 25974, 25975, 25976, -1, 25977, 25978, 25979, -1, 25980, 25981, 25982, -1, 25983, 25984, 25985, -1, 25986, 25987, 25988, -1, 25989, 25990, 25991, -1, 25992, 25993, 25994, -1, 25995, 25996, 25997, -1, 25998, 25999, 26000, -1, 26001, 26002, 26003, -1, 26004, 26005, 26006, -1, 26007, 26008, 26009, -1, 26010, 26011, 26012, -1, 26013, 26014, 26015, -1, 26016, 26017, 26018, -1, 26019, 26020, 26021, -1, 26022, 26023, 26024, -1, 26025, 26026, 26027, -1, 26028, 26029, 26030, -1, 26031, 26032, 26033, -1, 26034, 26035, 26036, -1, 26037, 26038, 26039, -1, 26040, 26041, 26042, -1, 26043, 26044, 26045, -1, 26046, 26047, 26048, -1, 26049, 26050, 26051, -1, 26052, 26053, 26054, -1, 26055, 26056, 26057, -1, 26058, 26059, 26060, -1, 26061, 26062, 26063, -1, 26064, 26065, 26066, -1, 26067, 26068, 26069, -1, 26070, 26071, 26072, -1, 26073, 26074, 26075, -1, 26076, 26077, 26078, -1, 26079, 26080, 26081, -1, 26082, 26083, 26084, -1, 26085, 26086, 26087, -1, 26088, 26089, 26090, -1, 26091, 26092, 26093, -1, 26094, 26095, 26096, -1, 26097, 26098, 26099, -1, 26100, 26101, 26102, -1, 26103, 26104, 26105, -1, 26106, 26107, 26108, -1, 26109, 26110, 26111, -1, 26112, 26113, 26114, -1, 26115, 26116, 26117, -1, 26118, 26119, 26120, -1, 26121, 26122, 26123, -1, 26124, 26125, 26126, -1, 26127, 26128, 26129, -1, 26130, 26131, 26132, -1, 26133, 26134, 26135, -1, 26136, 26137, 26138, -1, 26139, 26140, 26141, -1, 26142, 26143, 26144, -1, 26145, 26146, 26147, -1, 26148, 26149, 26150, -1, 26151, 26152, 26153, -1, 26154, 26155, 26156, -1, 26157, 26158, 26159, -1, 26160, 26161, 26162, -1, 26163, 26164, 26165, -1, 26166, 26167, 26168, -1, 26169, 26170, 26171, -1, 26172, 26173, 26174, -1, 26175, 26176, 26177, -1, 26178, 26179, 26180, -1, 26181, 26182, 26183, -1, 26184, 26185, 26186, -1, 26187, 26188, 26189, -1, 26190, 26191, 26192, -1, 26193, 26194, 26195, -1, 26196, 26197, 26198, -1, 26199, 26200, 26201, -1, 26202, 26203, 26204, -1, 26205, 26206, 26207, -1, 26208, 26209, 26210, -1, 26211, 26212, 26213, -1, 26214, 26215, 26216, -1, 26217, 26218, 26219, -1, 26220, 26221, 26222, -1, 26223, 26224, 26225, -1, 26226, 26227, 26228, -1, 26229, 26230, 26231, -1, 26232, 26233, 26234, -1, 26235, 26236, 26237, -1, 26238, 26239, 26240, -1, 26241, 26242, 26243, -1, 26244, 26245, 26246, -1, 26247, 26248, 26249, -1, 26250, 26251, 26252, -1, 26253, 26254, 26255, -1, 26256, 26257, 26258, -1, 26259, 26260, 26261, -1, 26262, 26263, 26264, -1, 26265, 26266, 26267, -1, 26268, 26269, 26270, -1, 26271, 26272, 26273, -1, 26274, 26275, 26276, -1, 26277, 26278, 26279, -1, 26280, 26281, 26282, -1, 26283, 26284, 26285, -1, 26286, 26287, 26288, -1, 26289, 26290, 26291, -1, 26292, 26293, 26294, -1, 26295, 26296, 26297, -1, 26298, 26299, 26300, -1, 26301, 26302, 26303, -1, 26304, 26305, 26306, -1, 26307, 26308, 26309, -1, 26310, 26311, 26312, -1, 26313, 26314, 26315, -1, 26316, 26317, 26318, -1, 26319, 26320, 26321, -1, 26322, 26323, 26324, -1, 26325, 26326, 26327, -1, 26328, 26329, 26330, -1, 26331, 26332, 26333, -1, 26334, 26335, 26336, -1, 26337, 26338, 26339, -1, 26340, 26341, 26342, -1, 26343, 26344, 26345, -1, 26346, 26347, 26348, -1, 26349, 26350, 26351, -1, 26352, 26353, 26354, -1, 26355, 26356, 26357, -1, 26358, 26359, 26360, -1, 26361, 26362, 26363, -1, 26364, 26365, 26366, -1, 26367, 26368, 26369, -1, 26370, 26371, 26372, -1, 26373, 26374, 26375, -1, 26376, 26377, 26378, -1, 26379, 26380, 26381, -1, 26382, 26383, 26384, -1, 26385, 26386, 26387, -1, 26388, 26389, 26390, -1, 26391, 26392, 26393, -1, 26394, 26395, 26396, -1, 26397, 26398, 26399, -1, 26400, 26401, 26402, -1, 26403, 26404, 26405, -1, 26406, 26407, 26408, -1, 26409, 26410, 26411, -1, 26412, 26413, 26414, -1, 26415, 26416, 26417, -1, 26418, 26419, 26420, -1, 26421, 26422, 26423, -1, 26424, 26425, 26426, -1, 26427, 26428, 26429, -1, 26430, 26431, 26432, -1, 26433, 26434, 26435, -1, 26436, 26437, 26438, -1, 26439, 26440, 26441, -1, 26442, 26443, 26444, -1, 26445, 26446, 26447, -1, 26448, 26449, 26450, -1, 26451, 26452, 26453, -1, 26454, 26455, 26456, -1, 26457, 26458, 26459, -1, 26460, 26461, 26462, -1, 26463, 26464, 26465, -1, 26466, 26467, 26468, -1, 26469, 26470, 26471, -1, 26472, 26473, 26474, -1, 26475, 26476, 26477, -1, 26478, 26479, 26480, -1, 26481, 26482, 26483, -1, 26484, 26485, 26486, -1, 26487, 26488, 26489, -1, 26490, 26491, 26492, -1, 26493, 26494, 26495, -1, 26496, 26497, 26498, -1, 26499, 26500, 26501, -1, 26502, 26503, 26504, -1, 26505, 26506, 26507, -1, 26508, 26509, 26510, -1, 26511, 26512, 26513, -1, 26514, 26515, 26516, -1, 26517, 26518, 26519, -1, 26520, 26521, 26522, -1, 26523, 26524, 26525, -1, 26526, 26527, 26528, -1, 26529, 26530, 26531, -1, 26532, 26533, 26534, -1, 26535, 26536, 26537, -1, 26538, 26539, 26540, -1, 26541, 26542, 26543, -1, 26544, 26545, 26546, -1, 26547, 26548, 26549, -1, 26550, 26551, 26552, -1, 26553, 26554, 26555, -1, 26556, 26557, 26558, -1, 26559, 26560, 26561, -1, 26562, 26563, 26564, -1, 26565, 26566, 26567, -1, 26568, 26569, 26570, -1, 26571, 26572, 26573, -1, 26574, 26575, 26576, -1, 26577, 26578, 26579, -1, 26580, 26581, 26582, -1, 26583, 26584, 26585, -1, 26586, 26587, 26588, -1, 26589, 26590, 26591, -1, 26592, 26593, 26594, -1, 26595, 26596, 26597, -1, 26598, 26599, 26600, -1, 26601, 26602, 26603, -1, 26604, 26605, 26606, -1, 26607, 26608, 26609, -1, 26610, 26611, 26612, -1, 26613, 26614, 26615, -1, 26616, 26617, 26618, -1, 26619, 26620, 26621, -1, 26622, 26623, 26624, -1, 26625, 26626, 26627, -1, 26628, 26629, 26630, -1, 26631, 26632, 26633, -1, 26634, 26635, 26636, -1, 26637, 26638, 26639, -1, 26640, 26641, 26642, -1, 26643, 26644, 26645, -1, 26646, 26647, 26648, -1, 26649, 26650, 26651, -1, 26652, 26653, 26654, -1, 26655, 26656, 26657, -1, 26658, 26659, 26660, -1, 26661, 26662, 26663, -1, 26664, 26665, 26666, -1, 26667, 26668, 26669, -1, 26670, 26671, 26672, -1, 26673, 26674, 26675, -1, 26676, 26677, 26678, -1, 26679, 26680, 26681, -1, 26682, 26683, 26684, -1, 26685, 26686, 26687, -1, 26688, 26689, 26690, -1, 26691, 26692, 26693, -1, 26694, 26695, 26696, -1, 26697, 26698, 26699, -1, 26700, 26701, 26702, -1, 26703, 26704, 26705, -1, 26706, 26707, 26708, -1, 26709, 26710, 26711, -1, 26712, 26713, 26714, -1, 26715, 26716, 26717, -1, 26718, 26719, 26720, -1, 26721, 26722, 26723, -1, 26724, 26725, 26726, -1, 26727, 26728, 26729, -1, 26730, 26731, 26732, -1, 26733, 26734, 26735, -1, 26736, 26737, 26738, -1, 26739, 26740, 26741, -1, 26742, 26743, 26744, -1, 26745, 26746, 26747, -1, 26748, 26749, 26750, -1, 26751, 26752, 26753, -1, 26754, 26755, 26756, -1, 26757, 26758, 26759, -1, 26760, 26761, 26762, -1, 26763, 26764, 26765, -1, 26766, 26767, 26768, -1, 26769, 26770, 26771, -1, 26772, 26773, 26774, -1, 26775, 26776, 26777, -1, 26778, 26779, 26780, -1, 26781, 26782, 26783, -1, 26784, 26785, 26786, -1, 26787, 26788, 26789, -1, 26790, 26791, 26792, -1, 26793, 26794, 26795, -1, 26796, 26797, 26798, -1, 26799, 26800, 26801, -1, 26802, 26803, 26804, -1, 26805, 26806, 26807, -1, 26808, 26809, 26810, -1, 26811, 26812, 26813, -1, 26814, 26815, 26816, -1, 26817, 26818, 26819, -1, 26820, 26821, 26822, -1, 26823, 26824, 26825, -1, 26826, 26827, 26828, -1, 26829, 26830, 26831, -1, 26832, 26833, 26834, -1, 26835, 26836, 26837, -1, 26838, 26839, 26840, -1, 26841, 26842, 26843, -1, 26844, 26845, 26846, -1, 26847, 26848, 26849, -1, 26850, 26851, 26852, -1, 26853, 26854, 26855, -1, 26856, 26857, 26858, -1, 26859, 26860, 26861, -1, 26862, 26863, 26864, -1, 26865, 26866, 26867, -1, 26868, 26869, 26870, -1, 26871, 26872, 26873, -1, 26874, 26875, 26876, -1, 26877, 26878, 26879, -1, 26880, 26881, 26882, -1, 26883, 26884, 26885, -1, 26886, 26887, 26888, -1, 26889, 26890, 26891, -1, 26892, 26893, 26894, -1, 26895, 26896, 26897, -1, 26898, 26899, 26900, -1, 26901, 26902, 26903, -1, 26904, 26905, 26906, -1, 26907, 26908, 26909, -1, 26910, 26911, 26912, -1, 26913, 26914, 26915, -1, 26916, 26917, 26918, -1, 26919, 26920, 26921, -1, 26922, 26923, 26924, -1, 26925, 26926, 26927, -1, 26928, 26929, 26930, -1, 26931, 26932, 26933, -1, 26934, 26935, 26936, -1, 26937, 26938, 26939, -1, 26940, 26941, 26942, -1, 26943, 26944, 26945, -1, 26946, 26947, 26948, -1, 26949, 26950, 26951, -1, 26952, 26953, 26954, -1, 26955, 26956, 26957, -1, 26958, 26959, 26960, -1, 26961, 26962, 26963, -1, 26964, 26965, 26966, -1, 26967, 26968, 26969, -1, 26970, 26971, 26972, -1, 26973, 26974, 26975, -1, 26976, 26977, 26978, -1, 26979, 26980, 26981, -1, 26982, 26983, 26984, -1, 26985, 26986, 26987, -1, 26988, 26989, 26990, -1, 26991, 26992, 26993, -1, 26994, 26995, 26996, -1, 26997, 26998, 26999, -1, 27000, 27001, 27002, -1, 27003, 27004, 27005, -1, 27006, 27007, 27008, -1, 27009, 27010, 27011, -1, 27012, 27013, 27014, -1, 27015, 27016, 27017, -1, 27018, 27019, 27020, -1, 27021, 27022, 27023, -1, 27024, 27025, 27026, -1, 27027, 27028, 27029, -1, 27030, 27031, 27032, -1, 27033, 27034, 27035, -1, 27036, 27037, 27038, -1, 27039, 27040, 27041, -1, 27042, 27043, 27044, -1, 27045, 27046, 27047, -1, 27048, 27049, 27050, -1, 27051, 27052, 27053, -1, 27054, 27055, 27056, -1, 27057, 27058, 27059, -1, 27060, 27061, 27062, -1, 27063, 27064, 27065, -1, 27066, 27067, 27068, -1, 27069, 27070, 27071, -1, 27072, 27073, 27074, -1, 27075, 27076, 27077, -1, 27078, 27079, 27080, -1, 27081, 27082, 27083, -1, 27084, 27085, 27086, -1, 27087, 27088, 27089, -1, 27090, 27091, 27092, -1, 27093, 27094, 27095, -1, 27096, 27097, 27098, -1, 27099, 27100, 27101, -1, 27102, 27103, 27104, -1, 27105, 27106, 27107, -1, 27108, 27109, 27110, -1, 27111, 27112, 27113, -1, 27114, 27115, 27116, -1, 27117, 27118, 27119, -1, 27120, 27121, 27122, -1, 27123, 27124, 27125, -1, 27126, 27127, 27128, -1, 27129, 27130, 27131, -1, 27132, 27133, 27134, -1, 27135, 27136, 27137, -1, 27138, 27139, 27140, -1, 27141, 27142, 27143, -1, 27144, 27145, 27146, -1, 27147, 27148, 27149, -1, 27150, 27151, 27152, -1, 27153, 27154, 27155, -1, 27156, 27157, 27158, -1, 27159, 27160, 27161, -1, 27162, 27163, 27164, -1, 27165, 27166, 27167, -1, 27168, 27169, 27170, -1, 27171, 27172, 27173, -1, 27174, 27175, 27176, -1, 27177, 27178, 27179, -1, 27180, 27181, 27182, -1, 27183, 27184, 27185, -1, 27186, 27187, 27188, -1, 27189, 27190, 27191, -1, 27192, 27193, 27194, -1, 27195, 27196, 27197, -1, 27198, 27199, 27200, -1, 27201, 27202, 27203, -1, 27204, 27205, 27206, -1, 27207, 27208, 27209, -1, 27210, 27211, 27212, -1, 27213, 27214, 27215, -1, 27216, 27217, 27218, -1, 27219, 27220, 27221, -1, 27222, 27223, 27224, -1, 27225, 27226, 27227, -1, 27228, 27229, 27230, -1, 27231, 27232, 27233, -1, 27234, 27235, 27236, -1, 27237, 27238, 27239, -1, 27240, 27241, 27242, -1, 27243, 27244, 27245, -1, 27246, 27247, 27248, -1, 27249, 27250, 27251, -1, 27252, 27253, 27254, -1, 27255, 27256, 27257, -1, 27258, 27259, 27260, -1, 27261, 27262, 27263, -1, 27264, 27265, 27266, -1, 27267, 27268, 27269, -1, 27270, 27271, 27272, -1, 27273, 27274, 27275, -1, 27276, 27277, 27278, -1, 27279, 27280, 27281, -1, 27282, 27283, 27284, -1, 27285, 27286, 27287, -1, 27288, 27289, 27290, -1, 27291, 27292, 27293, -1, 27294, 27295, 27296, -1, 27297, 27298, 27299, -1, 27300, 27301, 27302, -1, 27303, 27304, 27305, -1, 27306, 27307, 27308, -1, 27309, 27310, 27311, -1, 27312, 27313, 27314, -1, 27315, 27316, 27317, -1, 27318, 27319, 27320, -1, 27321, 27322, 27323, -1, 27324, 27325, 27326, -1, 27327, 27328, 27329, -1, 27330, 27331, 27332, -1, 27333, 27334, 27335, -1, 27336, 27337, 27338, -1, 27339, 27340, 27341, -1, 27342, 27343, 27344, -1, 27345, 27346, 27347, -1, 27348, 27349, 27350, -1, 27351, 27352, 27353, -1, 27354, 27355, 27356, -1, 27357, 27358, 27359, -1, 27360, 27361, 27362, -1, 27363, 27364, 27365, -1, 27366, 27367, 27368, -1, 27369, 27370, 27371, -1, 27372, 27373, 27374, -1, 27375, 27376, 27377, -1, 27378, 27379, 27380, -1, 27381, 27382, 27383, -1, 27384, 27385, 27386, -1, 27387, 27388, 27389, -1, 27390, 27391, 27392, -1, 27393, 27394, 27395, -1, 27396, 27397, 27398, -1, 27399, 27400, 27401, -1, 27402, 27403, 27404, -1, 27405, 27406, 27407, -1, 27408, 27409, 27410, -1, 27411, 27412, 27413, -1, 27414, 27415, 27416, -1, 27417, 27418, 27419, -1, 27420, 27421, 27422, -1, 27423, 27424, 27425, -1, 27426, 27427, 27428, -1, 27429, 27430, 27431, -1, 27432, 27433, 27434, -1, 27435, 27436, 27437, -1, 27438, 27439, 27440, -1, 27441, 27442, 27443, -1, 27444, 27445, 27446, -1, 27447, 27448, 27449, -1, 27450, 27451, 27452, -1, 27453, 27454, 27455, -1, 27456, 27457, 27458, -1, 27459, 27460, 27461, -1, 27462, 27463, 27464, -1, 27465, 27466, 27467, -1, 27468, 27469, 27470, -1, 27471, 27472, 27473, -1, 27474, 27475, 27476, -1, 27477, 27478, 27479, -1, 27480, 27481, 27482, -1, 27483, 27484, 27485, -1, 27486, 27487, 27488, -1, 27489, 27490, 27491, -1, 27492, 27493, 27494, -1, 27495, 27496, 27497, -1, 27498, 27499, 27500, -1, 27501, 27502, 27503, -1, 27504, 27505, 27506, -1, 27507, 27508, 27509, -1, 27510, 27511, 27512, -1, 27513, 27514, 27515, -1, 27516, 27517, 27518, -1, 27519, 27520, 27521, -1, 27522, 27523, 27524, -1, 27525, 27526, 27527, -1, 27528, 27529, 27530, -1, 27531, 27532, 27533, -1, 27534, 27535, 27536, -1, 27537, 27538, 27539, -1, 27540, 27541, 27542, -1, 27543, 27544, 27545, -1, 27546, 27547, 27548, -1, 27549, 27550, 27551, -1, 27552, 27553, 27554, -1, 27555, 27556, 27557, -1, 27558, 27559, 27560, -1, 27561, 27562, 27563, -1, 27564, 27565, 27566, -1, 27567, 27568, 27569, -1, 27570, 27571, 27572, -1, 27573, 27574, 27575, -1, 27576, 27577, 27578, -1, 27579, 27580, 27581, -1, 27582, 27583, 27584, -1, 27585, 27586, 27587, -1, 27588, 27589, 27590, -1, 27591, 27592, 27593, -1, 27594, 27595, 27596, -1, 27597, 27598, 27599, -1, 27600, 27601, 27602, -1, 27603, 27604, 27605, -1, 27606, 27607, 27608, -1, 27609, 27610, 27611, -1, 27612, 27613, 27614, -1, 27615, 27616, 27617, -1, 27618, 27619, 27620, -1, 27621, 27622, 27623, -1, 27624, 27625, 27626, -1, 27627, 27628, 27629, -1, 27630, 27631, 27632, -1, 27633, 27634, 27635, -1, 27636, 27637, 27638, -1, 27639, 27640, 27641, -1, 27642, 27643, 27644, -1, 27645, 27646, 27647, -1, 27648, 27649, 27650, -1, 27651, 27652, 27653, -1, 27654, 27655, 27656, -1, 27657, 27658, 27659, -1, 27660, 27661, 27662, -1, 27663, 27664, 27665, -1, 27666, 27667, 27668, -1, 27669, 27670, 27671, -1, 27672, 27673, 27674, -1, 27675, 27676, 27677, -1, 27678, 27679, 27680, -1, 27681, 27682, 27683, -1, 27684, 27685, 27686, -1, 27687, 27688, 27689, -1, 27690, 27691, 27692, -1, 27693, 27694, 27695, -1, 27696, 27697, 27698, -1, 27699, 27700, 27701, -1, 27702, 27703, 27704, -1, 27705, 27706, 27707, -1, 27708, 27709, 27710, -1, 27711, 27712, 27713, -1, 27714, 27715, 27716, -1, 27717, 27718, 27719, -1, 27720, 27721, 27722, -1, 27723, 27724, 27725, -1, 27726, 27727, 27728, -1, 27729, 27730, 27731, -1, 27732, 27733, 27734, -1, 27735, 27736, 27737, -1, 27738, 27739, 27740, -1, 27741, 27742, 27743, -1, 27744, 27745, 27746, -1, 27747, 27748, 27749, -1, 27750, 27751, 27752, -1, 27753, 27754, 27755, -1, 27756, 27757, 27758, -1, 27759, 27760, 27761, -1, 27762, 27763, 27764, -1, 27765, 27766, 27767, -1, 27768, 27769, 27770, -1, 27771, 27772, 27773, -1, 27774, 27775, 27776, -1, 27777, 27778, 27779, -1, 27780, 27781, 27782, -1, 27783, 27784, 27785, -1, 27786, 27787, 27788, -1, 27789, 27790, 27791, -1, 27792, 27793, 27794, -1, 27795, 27796, 27797, -1, 27798, 27799, 27800, -1, 27801, 27802, 27803, -1, 27804, 27805, 27806, -1, 27807, 27808, 27809, -1, 27810, 27811, 27812, -1, 27813, 27814, 27815, -1, 27816, 27817, 27818, -1, 27819, 27820, 27821, -1, 27822, 27823, 27824, -1, 27825, 27826, 27827, -1, 27828, 27829, 27830, -1, 27831, 27832, 27833, -1, 27834, 27835, 27836, -1, 27837, 27838, 27839, -1, 27840, 27841, 27842, -1, 27843, 27844, 27845, -1, 27846, 27847, 27848, -1, 27849, 27850, 27851, -1, 27852, 27853, 27854, -1, 27855, 27856, 27857, -1, 27858, 27859, 27860, -1, 27861, 27862, 27863, -1, 27864, 27865, 27866, -1, 27867, 27868, 27869, -1, 27870, 27871, 27872, -1, 27873, 27874, 27875, -1, 27876, 27877, 27878, -1, 27879, 27880, 27881, -1, 27882, 27883, 27884, -1, 27885, 27886, 27887, -1, 27888, 27889, 27890, -1, 27891, 27892, 27893, -1, 27894, 27895, 27896, -1, 27897, 27898, 27899, -1, 27900, 27901, 27902, -1, 27903, 27904, 27905, -1, 27906, 27907, 27908, -1, 27909, 27910, 27911, -1, 27912, 27913, 27914, -1, 27915, 27916, 27917, -1, 27918, 27919, 27920, -1, 27921, 27922, 27923, -1, 27924, 27925, 27926, -1, 27927, 27928, 27929, -1, 27930, 27931, 27932, -1, 27933, 27934, 27935, -1, 27936, 27937, 27938, -1, 27939, 27940, 27941, -1, 27942, 27943, 27944, -1, 27945, 27946, 27947, -1, 27948, 27949, 27950, -1, 27951, 27952, 27953, -1, 27954, 27955, 27956, -1, 27957, 27958, 27959, -1, 27960, 27961, 27962, -1, 27963, 27964, 27965, -1, 27966, 27967, 27968, -1, 27969, 27970, 27971, -1, 27972, 27973, 27974, -1, 27975, 27976, 27977, -1, 27978, 27979, 27980, -1, 27981, 27982, 27983, -1, 27984, 27985, 27986, -1, 27987, 27988, 27989, -1, 27990, 27991, 27992, -1, 27993, 27994, 27995, -1, 27996, 27997, 27998, -1, 27999, 28000, 28001, -1, 28002, 28003, 28004, -1, 28005, 28006, 28007, -1, 28008, 28009, 28010, -1, 28011, 28012, 28013, -1, 28014, 28015, 28016, -1, 28017, 28018, 28019, -1, 28020, 28021, 28022, -1, 28023, 28024, 28025, -1, 28026, 28027, 28028, -1, 28029, 28030, 28031, -1, 28032, 28033, 28034, -1, 28035, 28036, 28037, -1, 28038, 28039, 28040, -1, 28041, 28042, 28043, -1, 28044, 28045, 28046, -1, 28047, 28048, 28049, -1, 28050, 28051, 28052, -1, 28053, 28054, 28055, -1, 28056, 28057, 28058, -1, 28059, 28060, 28061, -1, 28062, 28063, 28064, -1, 28065, 28066, 28067, -1, 28068, 28069, 28070, -1, 28071, 28072, 28073, -1, 28074, 28075, 28076, -1, 28077, 28078, 28079, -1, 28080, 28081, 28082, -1, 28083, 28084, 28085, -1, 28086, 28087, 28088, -1, 28089, 28090, 28091, -1, 28092, 28093, 28094, -1, 28095, 28096, 28097, -1, 28098, 28099, 28100, -1, 28101, 28102, 28103, -1, 28104, 28105, 28106, -1, 28107, 28108, 28109, -1, 28110, 28111, 28112, -1, 28113, 28114, 28115, -1, 28116, 28117, 28118, -1, 28119, 28120, 28121, -1, 28122, 28123, 28124, -1, 28125, 28126, 28127, -1, 28128, 28129, 28130, -1, 28131, 28132, 28133, -1, 28134, 28135, 28136, -1, 28137, 28138, 28139, -1, 28140, 28141, 28142, -1, 28143, 28144, 28145, -1, 28146, 28147, 28148, -1, 28149, 28150, 28151, -1, 28152, 28153, 28154, -1, 28155, 28156, 28157, -1, 28158, 28159, 28160, -1, 28161, 28162, 28163, -1, 28164, 28165, 28166, -1, 28167, 28168, 28169, -1, 28170, 28171, 28172, -1, 28173, 28174, 28175, -1, 28176, 28177, 28178, -1, 28179, 28180, 28181, -1, 28182, 28183, 28184, -1, 28185, 28186, 28187, -1, 28188, 28189, 28190, -1, 28191, 28192, 28193, -1, 28194, 28195, 28196, -1, 28197, 28198, 28199, -1, 28200, 28201, 28202, -1, 28203, 28204, 28205, -1, 28206, 28207, 28208, -1, 28209, 28210, 28211, -1, 28212, 28213, 28214, -1, 28215, 28216, 28217, -1, 28218, 28219, 28220, -1, 28221, 28222, 28223, -1, 28224, 28225, 28226, -1, 28227, 28228, 28229, -1, 28230, 28231, 28232, -1, 28233, 28234, 28235, -1, 28236, 28237, 28238, -1, 28239, 28240, 28241, -1, 28242, 28243, 28244, -1, 28245, 28246, 28247, -1, 28248, 28249, 28250, -1, 28251, 28252, 28253, -1, 28254, 28255, 28256, -1, 28257, 28258, 28259, -1, 28260, 28261, 28262, -1, 28263, 28264, 28265, -1, 28266, 28267, 28268, -1, 28269, 28270, 28271, -1, 28272, 28273, 28274, -1, 28275, 28276, 28277, -1, 28278, 28279, 28280, -1, 28281, 28282, 28283, -1, 28284, 28285, 28286, -1, 28287, 28288, 28289, -1, 28290, 28291, 28292, -1, 28293, 28294, 28295, -1, 28296, 28297, 28298, -1, 28299, 28300, 28301, -1, 28302, 28303, 28304, -1, 28305, 28306, 28307, -1, 28308, 28309, 28310, -1, 28311, 28312, 28313, -1, 28314, 28315, 28316, -1, 28317, 28318, 28319, -1, 28320, 28321, 28322, -1, 28323, 28324, 28325, -1, 28326, 28327, 28328, -1, 28329, 28330, 28331, -1, 28332, 28333, 28334, -1, 28335, 28336, 28337, -1, 28338, 28339, 28340, -1, 28341, 28342, 28343, -1, 28344, 28345, 28346, -1, 28347, 28348, 28349, -1, 28350, 28351, 28352, -1, 28353, 28354, 28355, -1, 28356, 28357, 28358, -1, 28359, 28360, 28361, -1, 28362, 28363, 28364, -1, 28365, 28366, 28367, -1, 28368, 28369, 28370, -1, 28371, 28372, 28373, -1, 28374, 28375, 28376, -1, 28377, 28378, 28379, -1, 28380, 28381, 28382, -1, 28383, 28384, 28385, -1, 28386, 28387, 28388, -1, 28389, 28390, 28391, -1, 28392, 28393, 28394, -1, 28395, 28396, 28397, -1, 28398, 28399, 28400, -1, 28401, 28402, 28403, -1, 28404, 28405, 28406, -1, 28407, 28408, 28409, -1, 28410, 28411, 28412, -1, 28413, 28414, 28415, -1, 28416, 28417, 28418, -1, 28419, 28420, 28421, -1, 28422, 28423, 28424, -1, 28425, 28426, 28427, -1, 28428, 28429, 28430, -1, 28431, 28432, 28433, -1, 28434, 28435, 28436, -1, 28437, 28438, 28439, -1, 28440, 28441, 28442, -1, 28443, 28444, 28445, -1, 28446, 28447, 28448, -1, 28449, 28450, 28451, -1, 28452, 28453, 28454, -1, 28455, 28456, 28457, -1, 28458, 28459, 28460, -1, 28461, 28462, 28463, -1, 28464, 28465, 28466, -1, 28467, 28468, 28469, -1, 28470, 28471, 28472, -1, 28473, 28474, 28475, -1, 28476, 28477, 28478, -1, 28479, 28480, 28481, -1, 28482, 28483, 28484, -1, 28485, 28486, 28487, -1, 28488, 28489, 28490, -1, 28491, 28492, 28493, -1, 28494, 28495, 28496, -1, 28497, 28498, 28499, -1, 28500, 28501, 28502, -1, 28503, 28504, 28505, -1, 28506, 28507, 28508, -1, 28509, 28510, 28511, -1, 28512, 28513, 28514, -1, 28515, 28516, 28517, -1, 28518, 28519, 28520, -1, 28521, 28522, 28523, -1, 28524, 28525, 28526, -1, 28527, 28528, 28529, -1, 28530, 28531, 28532, -1, 28533, 28534, 28535, -1, 28536, 28537, 28538, -1, 28539, 28540, 28541, -1, 28542, 28543, 28544, -1, 28545, 28546, 28547, -1, 28548, 28549, 28550, -1, 28551, 28552, 28553, -1, 28554, 28555, 28556, -1, 28557, 28558, 28559, -1, 28560, 28561, 28562, -1, 28563, 28564, 28565, -1, 28566, 28567, 28568, -1, 28569, 28570, 28571, -1, 28572, 28573, 28574, -1, 28575, 28576, 28577, -1, 28578, 28579, 28580, -1, 28581, 28582, 28583, -1, 28584, 28585, 28586, -1, 28587, 28588, 28589, -1, 28590, 28591, 28592, -1, 28593, 28594, 28595, -1, 28596, 28597, 28598, -1, 28599, 28600, 28601, -1, 28602, 28603, 28604, -1, 28605, 28606, 28607, -1, 28608, 28609, 28610, -1, 28611, 28612, 28613, -1, 28614, 28615, 28616, -1, 28617, 28618, 28619, -1, 28620, 28621, 28622, -1, 28623, 28624, 28625, -1, 28626, 28627, 28628, -1, 28629, 28630, 28631, -1, 28632, 28633, 28634, -1, 28635, 28636, 28637, -1, 28638, 28639, 28640, -1, 28641, 28642, 28643, -1, 28644, 28645, 28646, -1, 28647, 28648, 28649, -1, 28650, 28651, 28652, -1, 28653, 28654, 28655, -1, 28656, 28657, 28658, -1, 28659, 28660, 28661, -1, 28662, 28663, 28664, -1, 28665, 28666, 28667, -1, 28668, 28669, 28670, -1, 28671, 28672, 28673, -1, 28674, 28675, 28676, -1, 28677, 28678, 28679, -1, 28680, 28681, 28682, -1, 28683, 28684, 28685, -1, 28686, 28687, 28688, -1, 28689, 28690, 28691, -1, 28692, 28693, 28694, -1, 28695, 28696, 28697, -1, 28698, 28699, 28700, -1, 28701, 28702, 28703, -1, 28704, 28705, 28706, -1, 28707, 28708, 28709, -1, 28710, 28711, 28712, -1, 28713, 28714, 28715, -1, 28716, 28717, 28718, -1, 28719, 28720, 28721, -1, 28722, 28723, 28724, -1, 28725, 28726, 28727, -1, 28728, 28729, 28730, -1, 28731, 28732, 28733, -1, 28734, 28735, 28736, -1, 28737, 28738, 28739, -1, 28740, 28741, 28742, -1, 28743, 28744, 28745, -1, 28746, 28747, 28748, -1, 28749, 28750, 28751, -1, 28752, 28753, 28754, -1, 28755, 28756, 28757, -1, 28758, 28759, 28760, -1, 28761, 28762, 28763, -1, 28764, 28765, 28766, -1, 28767, 28768, 28769, -1, 28770, 28771, 28772, -1, 28773, 28774, 28775, -1, 28776, 28777, 28778, -1, 28779, 28780, 28781, -1, 28782, 28783, 28784, -1, 28785, 28786, 28787, -1, 28788, 28789, 28790, -1, 28791, 28792, 28793, -1, 28794, 28795, 28796, -1, 28797, 28798, 28799, -1, 28800, 28801, 28802, -1, 28803, 28804, 28805, -1, 28806, 28807, 28808, -1, 28809, 28810, 28811, -1, 28812, 28813, 28814, -1, 28815, 28816, 28817, -1, 28818, 28819, 28820, -1, 28821, 28822, 28823, -1, 28824, 28825, 28826, -1, 28827, 28828, 28829, -1, 28830, 28831, 28832, -1, 28833, 28834, 28835, -1, 28836, 28837, 28838, -1, 28839, 28840, 28841, -1, 28842, 28843, 28844, -1, 28845, 28846, 28847, -1, 28848, 28849, 28850, -1, 28851, 28852, 28853, -1, 28854, 28855, 28856, -1, 28857, 28858, 28859, -1, 28860, 28861, 28862, -1, 28863, 28864, 28865, -1, 28866, 28867, 28868, -1, 28869, 28870, 28871, -1, 28872, 28873, 28874, -1, 28875, 28876, 28877, -1, 28878, 28879, 28880, -1, 28881, 28882, 28883, -1, 28884, 28885, 28886, -1, 28887, 28888, 28889, -1, 28890, 28891, 28892, -1, 28893, 28894, 28895, -1, 28896, 28897, 28898, -1, 28899, 28900, 28901, -1, 28902, 28903, 28904, -1, 28905, 28906, 28907, -1, 28908, 28909, 28910, -1, 28911, 28912, 28913, -1, 28914, 28915, 28916, -1, 28917, 28918, 28919, -1, 28920, 28921, 28922, -1, 28923, 28924, 28925, -1, 28926, 28927, 28928, -1, 28929, 28930, 28931, -1, 28932, 28933, 28934, -1, 28935, 28936, 28937, -1, 28938, 28939, 28940, -1, 28941, 28942, 28943, -1, 28944, 28945, 28946, -1, 28947, 28948, 28949, -1, 28950, 28951, 28952, -1, 28953, 28954, 28955, -1, 28956, 28957, 28958, -1, 28959, 28960, 28961, -1, 28962, 28963, 28964, -1, 28965, 28966, 28967, -1, 28968, 28969, 28970, -1, 28971, 28972, 28973, -1, 28974, 28975, 28976, -1, 28977, 28978, 28979, -1, 28980, 28981, 28982, -1, 28983, 28984, 28985, -1, 28986, 28987, 28988, -1, 28989, 28990, 28991, -1, 28992, 28993, 28994, -1, 28995, 28996, 28997, -1, 28998, 28999, 29000, -1, 29001, 29002, 29003, -1, 29004, 29005, 29006, -1, 29007, 29008, 29009, -1, 29010, 29011, 29012, -1, 29013, 29014, 29015, -1, 29016, 29017, 29018, -1, 29019, 29020, 29021, -1, 29022, 29023, 29024, -1, 29025, 29026, 29027, -1, 29028, 29029, 29030, -1, 29031, 29032, 29033, -1, 29034, 29035, 29036, -1, 29037, 29038, 29039, -1, 29040, 29041, 29042, -1, 29043, 29044, 29045, -1, 29046, 29047, 29048, -1, 29049, 29050, 29051, -1, 29052, 29053, 29054, -1, 29055, 29056, 29057, -1, 29058, 29059, 29060, -1, 29061, 29062, 29063, -1, 29064, 29065, 29066, -1, 29067, 29068, 29069, -1, 29070, 29071, 29072, -1, 29073, 29074, 29075, -1, 29076, 29077, 29078, -1, 29079, 29080, 29081, -1, 29082, 29083, 29084, -1, 29085, 29086, 29087, -1, 29088, 29089, 29090, -1, 29091, 29092, 29093, -1, 29094, 29095, 29096, -1, 29097, 29098, 29099, -1, 29100, 29101, 29102, -1, 29103, 29104, 29105, -1, 29106, 29107, 29108, -1, 29109, 29110, 29111, -1, 29112, 29113, 29114, -1, 29115, 29116, 29117, -1, 29118, 29119, 29120, -1, 29121, 29122, 29123, -1, 29124, 29125, 29126, -1, 29127, 29128, 29129, -1, 29130, 29131, 29132, -1, 29133, 29134, 29135, -1, 29136, 29137, 29138, -1, 29139, 29140, 29141, -1, 29142, 29143, 29144, -1, 29145, 29146, 29147, -1, 29148, 29149, 29150, -1, 29151, 29152, 29153, -1, 29154, 29155, 29156, -1, 29157, 29158, 29159, -1, 29160, 29161, 29162, -1, 29163, 29164, 29165, -1, 29166, 29167, 29168, -1, 29169, 29170, 29171, -1, 29172, 29173, 29174, -1, 29175, 29176, 29177, -1, 29178, 29179, 29180, -1, 29181, 29182, 29183, -1, 29184, 29185, 29186, -1, 29187, 29188, 29189, -1, 29190, 29191, 29192, -1, 29193, 29194, 29195, -1, 29196, 29197, 29198, -1, 29199, 29200, 29201, -1, 29202, 29203, 29204, -1, 29205, 29206, 29207, -1, 29208, 29209, 29210, -1, 29211, 29212, 29213, -1, 29214, 29215, 29216, -1, 29217, 29218, 29219, -1, 29220, 29221, 29222, -1, 29223, 29224, 29225, -1, 29226, 29227, 29228, -1, 29229, 29230, 29231, -1, 29232, 29233, 29234, -1, 29235, 29236, 29237, -1, 29238, 29239, 29240, -1, 29241, 29242, 29243, -1, 29244, 29245, 29246, -1, 29247, 29248, 29249, -1, 29250, 29251, 29252, -1, 29253, 29254, 29255, -1, 29256, 29257, 29258, -1, 29259, 29260, 29261, -1, 29262, 29263, 29264, -1, 29265, 29266, 29267, -1, 29268, 29269, 29270, -1, 29271, 29272, 29273, -1, 29274, 29275, 29276, -1, 29277, 29278, 29279, -1, 29280, 29281, 29282, -1, 29283, 29284, 29285, -1, 29286, 29287, 29288, -1, 29289, 29290, 29291, -1, 29292, 29293, 29294, -1, 29295, 29296, 29297, -1, 29298, 29299, 29300, -1, 29301, 29302, 29303, -1, 29304, 29305, 29306, -1, 29307, 29308, 29309, -1, 29310, 29311, 29312, -1, 29313, 29314, 29315, -1, 29316, 29317, 29318, -1, 29319, 29320, 29321, -1, 29322, 29323, 29324, -1, 29325, 29326, 29327, -1, 29328, 29329, 29330, -1, 29331, 29332, 29333, -1, 29334, 29335, 29336, -1, 29337, 29338, 29339, -1, 29340, 29341, 29342, -1, 29343, 29344, 29345, -1, 29346, 29347, 29348, -1, 29349, 29350, 29351, -1, 29352, 29353, 29354, -1, 29355, 29356, 29357, -1, 29358, 29359, 29360, -1, 29361, 29362, 29363, -1, 29364, 29365, 29366, -1, 29367, 29368, 29369, -1, 29370, 29371, 29372, -1, 29373, 29374, 29375, -1, 29376, 29377, 29378, -1, 29379, 29380, 29381, -1, 29382, 29383, 29384, -1, 29385, 29386, 29387, -1, 29388, 29389, 29390, -1, 29391, 29392, 29393, -1, 29394, 29395, 29396, -1, 29397, 29398, 29399, -1, 29400, 29401, 29402, -1, 29403, 29404, 29405, -1, 29406, 29407, 29408, -1, 29409, 29410, 29411, -1, 29412, 29413, 29414, -1, 29415, 29416, 29417, -1, 29418, 29419, 29420, -1, 29421, 29422, 29423, -1, 29424, 29425, 29426, -1, 29427, 29428, 29429, -1, 29430, 29431, 29432, -1, 29433, 29434, 29435, -1, 29436, 29437, 29438, -1, 29439, 29440, 29441, -1, 29442, 29443, 29444, -1, 29445, 29446, 29447, -1, 29448, 29449, 29450, -1, 29451, 29452, 29453, -1, 29454, 29455, 29456, -1, 29457, 29458, 29459, -1, 29460, 29461, 29462, -1, 29463, 29464, 29465, -1, 29466, 29467, 29468, -1, 29469, 29470, 29471, -1, 29472, 29473, 29474, -1, 29475, 29476, 29477, -1, 29478, 29479, 29480, -1, 29481, 29482, 29483, -1, 29484, 29485, 29486, -1, 29487, 29488, 29489, -1, 29490, 29491, 29492, -1, 29493, 29494, 29495, -1, 29496, 29497, 29498, -1, 29499, 29500, 29501, -1, 29502, 29503, 29504, -1, 29505, 29506, 29507, -1, 29508, 29509, 29510, -1, 29511, 29512, 29513, -1, 29514, 29515, 29516, -1, 29517, 29518, 29519, -1, 29520, 29521, 29522, -1, 29523, 29524, 29525, -1, 29526, 29527, 29528, -1, 29529, 29530, 29531, -1, 29532, 29533, 29534, -1, 29535, 29536, 29537, -1, 29538, 29539, 29540, -1, 29541, 29542, 29543, -1, 29544, 29545, 29546, -1, 29547, 29548, 29549, -1, 29550, 29551, 29552, -1, 29553, 29554, 29555, -1, 29556, 29557, 29558, -1, 29559, 29560, 29561, -1, 29562, 29563, 29564, -1, 29565, 29566, 29567, -1, 29568, 29569, 29570, -1, 29571, 29572, 29573, -1, 29574, 29575, 29576, -1, 29577, 29578, 29579, -1, 29580, 29581, 29582, -1, 29583, 29584, 29585, -1, 29586, 29587, 29588, -1, 29589, 29590, 29591, -1, 29592, 29593, 29594, -1, 29595, 29596, 29597, -1, 29598, 29599, 29600, -1, 29601, 29602, 29603, -1, 29604, 29605, 29606, -1, 29607, 29608, 29609, -1, 29610, 29611, 29612, -1, 29613, 29614, 29615, -1, 29616, 29617, 29618, -1, 29619, 29620, 29621, -1, 29622, 29623, 29624, -1, 29625, 29626, 29627, -1, 29628, 29629, 29630, -1, 29631, 29632, 29633, -1, 29634, 29635, 29636, -1, 29637, 29638, 29639, -1, 29640, 29641, 29642, -1, 29643, 29644, 29645, -1, 29646, 29647, 29648, -1, 29649, 29650, 29651, -1, 29652, 29653, 29654, -1, 29655, 29656, 29657, -1, 29658, 29659, 29660, -1, 29661, 29662, 29663, -1, 29664, 29665, 29666, -1, 29667, 29668, 29669, -1, 29670, 29671, 29672, -1, 29673, 29674, 29675, -1, 29676, 29677, 29678, -1, 29679, 29680, 29681, -1, 29682, 29683, 29684, -1, 29685, 29686, 29687, -1, 29688, 29689, 29690, -1, 29691, 29692, 29693, -1, 29694, 29695, 29696, -1, 29697, 29698, 29699, -1, 29700, 29701, 29702, -1, 29703, 29704, 29705, -1, 29706, 29707, 29708, -1, 29709, 29710, 29711, -1, 29712, 29713, 29714, -1, 29715, 29716, 29717, -1, 29718, 29719, 29720, -1, 29721, 29722, 29723, -1, 29724, 29725, 29726, -1, 29727, 29728, 29729, -1, 29730, 29731, 29732, -1, 29733, 29734, 29735, -1, 29736, 29737, 29738, -1, 29739, 29740, 29741, -1, 29742, 29743, 29744, -1, 29745, 29746, 29747, -1, 29748, 29749, 29750, -1, 29751, 29752, 29753, -1, 29754, 29755, 29756, -1, 29757, 29758, 29759, -1, 29760, 29761, 29762, -1, 29763, 29764, 29765, -1, 29766, 29767, 29768, -1, 29769, 29770, 29771, -1, 29772, 29773, 29774, -1, 29775, 29776, 29777, -1, 29778, 29779, 29780, -1, 29781, 29782, 29783, -1, 29784, 29785, 29786, -1, 29787, 29788, 29789, -1, 29790, 29791, 29792, -1, 29793, 29794, 29795, -1, 29796, 29797, 29798, -1, 29799, 29800, 29801, -1, 29802, 29803, 29804, -1, 29805, 29806, 29807, -1, 29808, 29809, 29810, -1, 29811, 29812, 29813, -1, 29814, 29815, 29816, -1, 29817, 29818, 29819, -1, 29820, 29821, 29822, -1, 29823, 29824, 29825, -1, 29826, 29827, 29828, -1, 29829, 29830, 29831, -1, 29832, 29833, 29834, -1, 29835, 29836, 29837, -1, 29838, 29839, 29840, -1, 29841, 29842, 29843, -1, 29844, 29845, 29846, -1, 29847, 29848, 29849, -1, 29850, 29851, 29852, -1, 29853, 29854, 29855, -1, 29856, 29857, 29858, -1, 29859, 29860, 29861, -1, 29862, 29863, 29864, -1, 29865, 29866, 29867, -1, 29868, 29869, 29870, -1, 29871, 29872, 29873, -1, 29874, 29875, 29876, -1, 29877, 29878, 29879, -1, 29880, 29881, 29882, -1, 29883, 29884, 29885, -1, 29886, 29887, 29888, -1, 29889, 29890, 29891, -1, 29892, 29893, 29894, -1, 29895, 29896, 29897, -1, 29898, 29899, 29900, -1, 29901, 29902, 29903, -1, 29904, 29905, 29906, -1, 29907, 29908, 29909, -1, 29910, 29911, 29912, -1, 29913, 29914, 29915, -1, 29916, 29917, 29918, -1, 29919, 29920, 29921, -1, 29922, 29923, 29924, -1, 29925, 29926, 29927, -1, 29928, 29929, 29930, -1, 29931, 29932, 29933, -1, 29934, 29935, 29936, -1, 29937, 29938, 29939, -1, 29940, 29941, 29942, -1, 29943, 29944, 29945, -1, 29946, 29947, 29948, -1, 29949, 29950, 29951, -1, 29952, 29953, 29954, -1, 29955, 29956, 29957, -1, 29958, 29959, 29960, -1, 29961, 29962, 29963, -1, 29964, 29965, 29966, -1, 29967, 29968, 29969, -1, 29970, 29971, 29972, -1, 29973, 29974, 29975, -1, 29976, 29977, 29978, -1, 29979, 29980, 29981, -1, 29982, 29983, 29984, -1, 29985, 29986, 29987, -1, 29988, 29989, 29990, -1, 29991, 29992, 29993, -1, 29994, 29995, 29996, -1, 29997, 29998, 29999, -1, 30000, 30001, 30002, -1, 30003, 30004, 30005, -1, 30006, 30007, 30008, -1, 30009, 30010, 30011, -1, 30012, 30013, 30014, -1, 30015, 30016, 30017, -1, 30018, 30019, 30020, -1, 30021, 30022, 30023, -1, 30024, 30025, 30026, -1, 30027, 30028, 30029, -1, 30030, 30031, 30032, -1, 30033, 30034, 30035, -1, 30036, 30037, 30038, -1, 30039, 30040, 30041, -1, 30042, 30043, 30044, -1, 30045, 30046, 30047, -1, 30048, 30049, 30050, -1, 30051, 30052, 30053, -1, 30054, 30055, 30056, -1, 30057, 30058, 30059, -1, 30060, 30061, 30062, -1, 30063, 30064, 30065, -1, 30066, 30067, 30068, -1, 30069, 30070, 30071, -1, 30072, 30073, 30074, -1, 30075, 30076, 30077, -1, 30078, 30079, 30080, -1, 30081, 30082, 30083, -1, 30084, 30085, 30086, -1, 30087, 30088, 30089, -1, 30090, 30091, 30092, -1, 30093, 30094, 30095, -1, 30096, 30097, 30098, -1, 30099, 30100, 30101, -1, 30102, 30103, 30104, -1, 30105, 30106, 30107, -1, 30108, 30109, 30110, -1, 30111, 30112, 30113, -1, 30114, 30115, 30116, -1, 30117, 30118, 30119, -1, 30120, 30121, 30122, -1, 30123, 30124, 30125, -1, 30126, 30127, 30128, -1, 30129, 30130, 30131, -1, 30132, 30133, 30134, -1, 30135, 30136, 30137, -1, 30138, 30139, 30140, -1, 30141, 30142, 30143, -1, 30144, 30145, 30146, -1, 30147, 30148, 30149, -1, 30150, 30151, 30152, -1, 30153, 30154, 30155, -1, 30156, 30157, 30158, -1, 30159, 30160, 30161, -1, 30162, 30163, 30164, -1, 30165, 30166, 30167, -1, 30168, 30169, 30170, -1, 30171, 30172, 30173, -1, 30174, 30175, 30176, -1, 30177, 30178, 30179, -1, 30180, 30181, 30182, -1, 30183, 30184, 30185, -1, 30186, 30187, 30188, -1, 30189, 30190, 30191, -1, 30192, 30193, 30194, -1, 30195, 30196, 30197, -1, 30198, 30199, 30200, -1, 30201, 30202, 30203, -1, 30204, 30205, 30206, -1, 30207, 30208, 30209, -1, 30210, 30211, 30212, -1, 30213, 30214, 30215, -1, 30216, 30217, 30218, -1, 30219, 30220, 30221, -1, 30222, 30223, 30224, -1, 30225, 30226, 30227, -1, 30228, 30229, 30230, -1, 30231, 30232, 30233, -1, 30234, 30235, 30236, -1, 30237, 30238, 30239, -1, 30240, 30241, 30242, -1, 30243, 30244, 30245, -1, 30246, 30247, 30248, -1, 30249, 30250, 30251, -1, 30252, 30253, 30254, -1, 30255, 30256, 30257, -1, 30258, 30259, 30260, -1, 30261, 30262, 30263, -1, 30264, 30265, 30266, -1, 30267, 30268, 30269, -1, 30270, 30271, 30272, -1, 30273, 30274, 30275, -1, 30276, 30277, 30278, -1, 30279, 30280, 30281, -1, 30282, 30283, 30284, -1, 30285, 30286, 30287, -1, 30288, 30289, 30290, -1, 30291, 30292, 30293, -1, 30294, 30295, 30296, -1, 30297, 30298, 30299, -1, 30300, 30301, 30302, -1, 30303, 30304, 30305, -1, 30306, 30307, 30308, -1, 30309, 30310, 30311, -1, 30312, 30313, 30314, -1, 30315, 30316, 30317, -1, 30318, 30319, 30320, -1, 30321, 30322, 30323, -1, 30324, 30325, 30326, -1, 30327, 30328, 30329, -1, 30330, 30331, 30332, -1, 30333, 30334, 30335, -1, 30336, 30337, 30338, -1, 30339, 30340, 30341, -1, 30342, 30343, 30344, -1, 30345, 30346, 30347, -1, 30348, 30349, 30350, -1, 30351, 30352, 30353, -1, 30354, 30355, 30356, -1, 30357, 30358, 30359, -1, 30360, 30361, 30362, -1, 30363, 30364, 30365, -1, 30366, 30367, 30368, -1, 30369, 30370, 30371, -1, 30372, 30373, 30374, -1, 30375, 30376, 30377, -1, 30378, 30379, 30380, -1, 30381, 30382, 30383, -1, 30384, 30385, 30386, -1, 30387, 30388, 30389, -1, 30390, 30391, 30392, -1, 30393, 30394, 30395, -1, 30396, 30397, 30398, -1, 30399, 30400, 30401, -1, 30402, 30403, 30404, -1, 30405, 30406, 30407, -1, 30408, 30409, 30410, -1, 30411, 30412, 30413, -1, 30414, 30415, 30416, -1, 30417, 30418, 30419, -1, 30420, 30421, 30422, -1, 30423, 30424, 30425, -1, 30426, 30427, 30428, -1, 30429, 30430, 30431, -1, 30432, 30433, 30434, -1, 30435, 30436, 30437, -1, 30438, 30439, 30440, -1, 30441, 30442, 30443, -1, 30444, 30445, 30446, -1, 30447, 30448, 30449, -1, 30450, 30451, 30452, -1, 30453, 30454, 30455, -1, 30456, 30457, 30458, -1, 30459, 30460, 30461, -1, 30462, 30463, 30464, -1, 30465, 30466, 30467, -1, 30468, 30469, 30470, -1, 30471, 30472, 30473, -1, 30474, 30475, 30476, -1, 30477, 30478, 30479, -1, 30480, 30481, 30482, -1, 30483, 30484, 30485, -1, 30486, 30487, 30488, -1, 30489, 30490, 30491, -1, 30492, 30493, 30494, -1, 30495, 30496, 30497, -1, 30498, 30499, 30500, -1, 30501, 30502, 30503, -1, 30504, 30505, 30506, -1, 30507, 30508, 30509, -1, 30510, 30511, 30512, -1, 30513, 30514, 30515, -1, 30516, 30517, 30518, -1, 30519, 30520, 30521, -1, 30522, 30523, 30524, -1, 30525, 30526, 30527, -1, 30528, 30529, 30530, -1, 30531, 30532, 30533, -1, 30534, 30535, 30536, -1, 30537, 30538, 30539, -1, 30540, 30541, 30542, -1, 30543, 30544, 30545, -1, 30546, 30547, 30548, -1, 30549, 30550, 30551, -1, 30552, 30553, 30554, -1, 30555, 30556, 30557, -1, 30558, 30559, 30560, -1, 30561, 30562, 30563, -1, 30564, 30565, 30566, -1, 30567, 30568, 30569, -1, 30570, 30571, 30572, -1, 30573, 30574, 30575, -1, 30576, 30577, 30578, -1, 30579, 30580, 30581, -1, 30582, 30583, 30584, -1, 30585, 30586, 30587, -1, 30588, 30589, 30590, -1, 30591, 30592, 30593, -1, 30594, 30595, 30596, -1, 30597, 30598, 30599, -1, 30600, 30601, 30602, -1, 30603, 30604, 30605, -1, 30606, 30607, 30608, -1, 30609, 30610, 30611, -1, 30612, 30613, 30614, -1, 30615, 30616, 30617, -1, 30618, 30619, 30620, -1, 30621, 30622, 30623, -1, 30624, 30625, 30626, -1, 30627, 30628, 30629, -1, 30630, 30631, 30632, -1, 30633, 30634, 30635, -1, 30636, 30637, 30638, -1, 30639, 30640, 30641, -1, 30642, 30643, 30644, -1, 30645, 30646, 30647, -1, 30648, 30649, 30650, -1, 30651, 30652, 30653, -1, 30654, 30655, 30656, -1, 30657, 30658, 30659, -1, 30660, 30661, 30662, -1, 30663, 30664, 30665, -1, 30666, 30667, 30668, -1, 30669, 30670, 30671, -1, 30672, 30673, 30674, -1, 30675, 30676, 30677, -1, 30678, 30679, 30680, -1, 30681, 30682, 30683, -1, 30684, 30685, 30686, -1, 30687, 30688, 30689, -1, 30690, 30691, 30692, -1, 30693, 30694, 30695, -1, 30696, 30697, 30698, -1, 30699, 30700, 30701, -1, 30702, 30703, 30704, -1, 30705, 30706, 30707, -1, 30708, 30709, 30710, -1, 30711, 30712, 30713, -1, 30714, 30715, 30716, -1, 30717, 30718, 30719, -1, 30720, 30721, 30722, -1, 30723, 30724, 30725, -1, 30726, 30727, 30728, -1, 30729, 30730, 30731, -1, 30732, 30733, 30734, -1, 30735, 30736, 30737, -1, 30738, 30739, 30740, -1, 30741, 30742, 30743, -1, 30744, 30745, 30746, -1, 30747, 30748, 30749, -1, 30750, 30751, 30752, -1, 30753, 30754, 30755, -1, 30756, 30757, 30758, -1, 30759, 30760, 30761, -1, 30762, 30763, 30764, -1, 30765, 30766, 30767, -1, 30768, 30769, 30770, -1, 30771, 30772, 30773, -1, 30774, 30775, 30776, -1, 30777, 30778, 30779, -1, 30780, 30781, 30782, -1, 30783, 30784, 30785, -1, 30786, 30787, 30788, -1, 30789, 30790, 30791, -1, 30792, 30793, 30794, -1, 30795, 30796, 30797, -1, 30798, 30799, 30800, -1, 30801, 30802, 30803, -1, 30804, 30805, 30806, -1, 30807, 30808, 30809, -1, 30810, 30811, 30812, -1, 30813, 30814, 30815, -1, 30816, 30817, 30818, -1, 30819, 30820, 30821, -1, 30822, 30823, 30824, -1, 30825, 30826, 30827, -1, 30828, 30829, 30830, -1, 30831, 30832, 30833, -1, 30834, 30835, 30836, -1, 30837, 30838, 30839, -1, 30840, 30841, 30842, -1, 30843, 30844, 30845, -1, 30846, 30847, 30848, -1, 30849, 30850, 30851, -1, 30852, 30853, 30854, -1, 30855, 30856, 30857, -1, 30858, 30859, 30860, -1, 30861, 30862, 30863, -1, 30864, 30865, 30866, -1, 30867, 30868, 30869, -1, 30870, 30871, 30872, -1, 30873, 30874, 30875, -1, 30876, 30877, 30878, -1, 30879, 30880, 30881, -1, 30882, 30883, 30884, -1, 30885, 30886, 30887, -1, 30888, 30889, 30890, -1, 30891, 30892, 30893, -1, 30894, 30895, 30896, -1, 30897, 30898, 30899, -1, 30900, 30901, 30902, -1, 30903, 30904, 30905, -1, 30906, 30907, 30908, -1, 30909, 30910, 30911, -1, 30912, 30913, 30914, -1, 30915, 30916, 30917, -1, 30918, 30919, 30920, -1, 30921, 30922, 30923, -1, 30924, 30925, 30926, -1, 30927, 30928, 30929, -1, 30930, 30931, 30932, -1, 30933, 30934, 30935, -1, 30936, 30937, 30938, -1, 30939, 30940, 30941, -1, 30942, 30943, 30944, -1, 30945, 30946, 30947, -1, 30948, 30949, 30950, -1, 30951, 30952, 30953, -1, 30954, 30955, 30956, -1, 30957, 30958, 30959, -1, 30960, 30961, 30962, -1, 30963, 30964, 30965, -1, 30966, 30967, 30968, -1, 30969, 30970, 30971, -1, 30972, 30973, 30974, -1, 30975, 30976, 30977, -1, 30978, 30979, 30980, -1, 30981, 30982, 30983, -1, 30984, 30985, 30986, -1, 30987, 30988, 30989, -1, 30990, 30991, 30992, -1, 30993, 30994, 30995, -1, 30996, 30997, 30998, -1, 30999, 31000, 31001, -1, 31002, 31003, 31004, -1, 31005, 31006, 31007, -1, 31008, 31009, 31010, -1, 31011, 31012, 31013, -1, 31014, 31015, 31016, -1, 31017, 31018, 31019, -1, 31020, 31021, 31022, -1, 31023, 31024, 31025, -1, 31026, 31027, 31028, -1, 31029, 31030, 31031, -1, 31032, 31033, 31034, -1, 31035, 31036, 31037, -1, 31038, 31039, 31040, -1, 31041, 31042, 31043, -1, 31044, 31045, 31046, -1, 31047, 31048, 31049, -1, 31050, 31051, 31052, -1, 31053, 31054, 31055, -1, 31056, 31057, 31058, -1, 31059, 31060, 31061, -1, 31062, 31063, 31064, -1, 31065, 31066, 31067, -1, 31068, 31069, 31070, -1, 31071, 31072, 31073, -1, 31074, 31075, 31076, -1, 31077, 31078, 31079, -1, 31080, 31081, 31082, -1, 31083, 31084, 31085, -1, 31086, 31087, 31088, -1, 31089, 31090, 31091, -1, 31092, 31093, 31094, -1, 31095, 31096, 31097, -1, 31098, 31099, 31100, -1, 31101, 31102, 31103, -1, 31104, 31105, 31106, -1, 31107, 31108, 31109, -1, 31110, 31111, 31112, -1, 31113, 31114, 31115, -1, 31116, 31117, 31118, -1, 31119, 31120, 31121, -1, 31122, 31123, 31124, -1, 31125, 31126, 31127, -1, 31128, 31129, 31130, -1, 31131, 31132, 31133, -1, 31134, 31135, 31136, -1, 31137, 31138, 31139, -1, 31140, 31141, 31142, -1, 31143, 31144, 31145, -1, 31146, 31147, 31148, -1, 31149, 31150, 31151, -1, 31152, 31153, 31154, -1, 31155, 31156, 31157, -1, 31158, 31159, 31160, -1, 31161, 31162, 31163, -1, 31164, 31165, 31166, -1, 31167, 31168, 31169, -1, 31170, 31171, 31172, -1, 31173, 31174, 31175, -1, 31176, 31177, 31178, -1, 31179, 31180, 31181, -1, 31182, 31183, 31184, -1, 31185, 31186, 31187, -1, 31188, 31189, 31190, -1, 31191, 31192, 31193, -1, 31194, 31195, 31196, -1, 31197, 31198, 31199, -1, 31200, 31201, 31202, -1, 31203, 31204, 31205, -1, 31206, 31207, 31208, -1, 31209, 31210, 31211, -1, 31212, 31213, 31214, -1, 31215, 31216, 31217, -1, 31218, 31219, 31220, -1, 31221, 31222, 31223, -1, 31224, 31225, 31226, -1, 31227, 31228, 31229, -1, 31230, 31231, 31232, -1, 31233, 31234, 31235, -1, 31236, 31237, 31238, -1, 31239, 31240, 31241, -1, 31242, 31243, 31244, -1, 31245, 31246, 31247, -1, 31248, 31249, 31250, -1, 31251, 31252, 31253, -1, 31254, 31255, 31256, -1, 31257, 31258, 31259, -1, 31260, 31261, 31262, -1, 31263, 31264, 31265, -1, 31266, 31267, 31268, -1, 31269, 31270, 31271, -1, 31272, 31273, 31274, -1, 31275, 31276, 31277, -1, 31278, 31279, 31280, -1, 31281, 31282, 31283, -1, 31284, 31285, 31286, -1, 31287, 31288, 31289, -1, 31290, 31291, 31292, -1, 31293, 31294, 31295, -1, 31296, 31297, 31298, -1, 31299, 31300, 31301, -1, 31302, 31303, 31304, -1, 31305, 31306, 31307, -1, 31308, 31309, 31310, -1, 31311, 31312, 31313, -1, 31314, 31315, 31316, -1, 31317, 31318, 31319, -1, 31320, 31321, 31322, -1, 31323, 31324, 31325, -1, 31326, 31327, 31328, -1, 31329, 31330, 31331, -1, 31332, 31333, 31334, -1, 31335, 31336, 31337, -1, 31338, 31339, 31340, -1, 31341, 31342, 31343, -1, 31344, 31345, 31346, -1, 31347, 31348, 31349, -1, 31350, 31351, 31352, -1, 31353, 31354, 31355, -1, 31356, 31357, 31358, -1, 31359, 31360, 31361, -1, 31362, 31363, 31364, -1, 31365, 31366, 31367, -1, 31368, 31369, 31370, -1, 31371, 31372, 31373, -1, 31374, 31375, 31376, -1, 31377, 31378, 31379, -1, 31380, 31381, 31382, -1, 31383, 31384, 31385, -1, 31386, 31387, 31388, -1, 31389, 31390, 31391, -1, 31392, 31393, 31394, -1, 31395, 31396, 31397, -1, 31398, 31399, 31400, -1, 31401, 31402, 31403, -1, 31404, 31405, 31406, -1, 31407, 31408, 31409, -1, 31410, 31411, 31412, -1, 31413, 31414, 31415, -1, 31416, 31417, 31418, -1, 31419, 31420, 31421, -1, 31422, 31423, 31424, -1, 31425, 31426, 31427, -1, 31428, 31429, 31430, -1, 31431, 31432, 31433, -1, 31434, 31435, 31436, -1, 31437, 31438, 31439, -1, 31440, 31441, 31442, -1, 31443, 31444, 31445, -1, 31446, 31447, 31448, -1, 31449, 31450, 31451, -1, 31452, 31453, 31454, -1, 31455, 31456, 31457, -1, 31458, 31459, 31460, -1, 31461, 31462, 31463, -1, 31464, 31465, 31466, -1, 31467, 31468, 31469, -1, 31470, 31471, 31472, -1, 31473, 31474, 31475, -1, 31476, 31477, 31478, -1, 31479, 31480, 31481, -1, 31482, 31483, 31484, -1, 31485, 31486, 31487, -1, 31488, 31489, 31490, -1, 31491, 31492, 31493, -1, 31494, 31495, 31496, -1, 31497, 31498, 31499, -1, 31500, 31501, 31502, -1, 31503, 31504, 31505, -1, 31506, 31507, 31508, -1, 31509, 31510, 31511, -1, 31512, 31513, 31514, -1, 31515, 31516, 31517, -1, 31518, 31519, 31520, -1, 31521, 31522, 31523, -1, 31524, 31525, 31526, -1, 31527, 31528, 31529, -1, 31530, 31531, 31532, -1, 31533, 31534, 31535, -1, 31536, 31537, 31538, -1, 31539, 31540, 31541, -1, 31542, 31543, 31544, -1, 31545, 31546, 31547, -1, 31548, 31549, 31550, -1, 31551, 31552, 31553, -1, 31554, 31555, 31556, -1, 31557, 31558, 31559, -1, 31560, 31561, 31562, -1, 31563, 31564, 31565, -1, 31566, 31567, 31568, -1, 31569, 31570, 31571, -1, 31572, 31573, 31574, -1, 31575, 31576, 31577, -1, 31578, 31579, 31580, -1, 31581, 31582, 31583, -1, 31584, 31585, 31586, -1, 31587, 31588, 31589, -1, 31590, 31591, 31592, -1, 31593, 31594, 31595, -1, 31596, 31597, 31598, -1, 31599, 31600, 31601, -1, 31602, 31603, 31604, -1, 31605, 31606, 31607, -1, 31608, 31609, 31610, -1, 31611, 31612, 31613, -1, 31614, 31615, 31616, -1, 31617, 31618, 31619, -1, 31620, 31621, 31622, -1, 31623, 31624, 31625, -1, 31626, 31627, 31628, -1, 31629, 31630, 31631, -1, 31632, 31633, 31634, -1, 31635, 31636, 31637, -1, 31638, 31639, 31640, -1, 31641, 31642, 31643, -1, 31644, 31645, 31646, -1, 31647, 31648, 31649, -1, 31650, 31651, 31652, -1, 31653, 31654, 31655, -1, 31656, 31657, 31658, -1, 31659, 31660, 31661, -1, 31662, 31663, 31664, -1, 31665, 31666, 31667, -1, 31668, 31669, 31670, -1, 31671, 31672, 31673, -1, 31674, 31675, 31676, -1, 31677, 31678, 31679, -1, 31680, 31681, 31682, -1, 31683, 31684, 31685, -1, 31686, 31687, 31688, -1, 31689, 31690, 31691, -1, 31692, 31693, 31694, -1, 31695, 31696, 31697, -1, 31698, 31699, 31700, -1, 31701, 31702, 31703, -1, 31704, 31705, 31706, -1, 31707, 31708, 31709, -1, 31710, 31711, 31712, -1, 31713, 31714, 31715, -1, 31716, 31717, 31718, -1, 31719, 31720, 31721, -1, 31722, 31723, 31724, -1, 31725, 31726, 31727, -1, 31728, 31729, 31730, -1, 31731, 31732, 31733, -1, 31734, 31735, 31736, -1, 31737, 31738, 31739, -1, 31740, 31741, 31742, -1, 31743, 31744, 31745, -1, 31746, 31747, 31748, -1, 31749, 31750, 31751, -1, 31752, 31753, 31754, -1, 31755, 31756, 31757, -1, 31758, 31759, 31760, -1, 31761, 31762, 31763, -1, 31764, 31765, 31766, -1, 31767, 31768, 31769, -1, 31770, 31771, 31772, -1, 31773, 31774, 31775, -1, 31776, 31777, 31778, -1, 31779, 31780, 31781, -1, 31782, 31783, 31784, -1, 31785, 31786, 31787, -1, 31788, 31789, 31790, -1, 31791, 31792, 31793, -1, 31794, 31795, 31796, -1, 31797, 31798, 31799, -1, 31800, 31801, 31802, -1, 31803, 31804, 31805, -1, 31806, 31807, 31808, -1, 31809, 31810, 31811, -1, 31812, 31813, 31814, -1, 31815, 31816, 31817, -1, 31818, 31819, 31820, -1, 31821, 31822, 31823, -1, 31824, 31825, 31826, -1, 31827, 31828, 31829, -1, 31830, 31831, 31832, -1, 31833, 31834, 31835, -1, 31836, 31837, 31838, -1, 31839, 31840, 31841, -1, 31842, 31843, 31844, -1, 31845, 31846, 31847, -1, 31848, 31849, 31850, -1, 31851, 31852, 31853, -1, 31854, 31855, 31856, -1, 31857, 31858, 31859, -1, 31860, 31861, 31862, -1, 31863, 31864, 31865, -1, 31866, 31867, 31868, -1, 31869, 31870, 31871, -1, 31872, 31873, 31874, -1, 31875, 31876, 31877, -1, 31878, 31879, 31880, -1, 31881, 31882, 31883, -1, 31884, 31885, 31886, -1, 31887, 31888, 31889, -1, 31890, 31891, 31892, -1, 31893, 31894, 31895, -1, 31896, 31897, 31898, -1, 31899, 31900, 31901, -1, 31902, 31903, 31904, -1, 31905, 31906, 31907, -1, 31908, 31909, 31910, -1, 31911, 31912, 31913, -1, 31914, 31915, 31916, -1, 31917, 31918, 31919, -1, 31920, 31921, 31922, -1, 31923, 31924, 31925, -1, 31926, 31927, 31928, -1, 31929, 31930, 31931, -1, 31932, 31933, 31934, -1, 31935, 31936, 31937, -1, 31938, 31939, 31940, -1, 31941, 31942, 31943, -1, 31944, 31945, 31946, -1, 31947, 31948, 31949, -1, 31950, 31951, 31952, -1, 31953, 31954, 31955, -1, 31956, 31957, 31958, -1, 31959, 31960, 31961, -1, 31962, 31963, 31964, -1, 31965, 31966, 31967, -1, 31968, 31969, 31970, -1, 31971, 31972, 31973, -1, 31974, 31975, 31976, -1, 31977, 31978, 31979, -1, 31980, 31981, 31982, -1, 31983, 31984, 31985, -1, 31986, 31987, 31988, -1, 31989, 31990, 31991, -1, 31992, 31993, 31994, -1, 31995, 31996, 31997, -1, 31998, 31999, 32000, -1, 32001, 32002, 32003, -1, 32004, 32005, 32006, -1, 32007, 32008, 32009, -1, 32010, 32011, 32012, -1, 32013, 32014, 32015, -1, 32016, 32017, 32018, -1, 32019, 32020, 32021, -1, 32022, 32023, 32024, -1, 32025, 32026, 32027, -1, 32028, 32029, 32030, -1, 32031, 32032, 32033, -1, 32034, 32035, 32036, -1, 32037, 32038, 32039, -1, 32040, 32041, 32042, -1, 32043, 32044, 32045, -1, 32046, 32047, 32048, -1, 32049, 32050, 32051, -1, 32052, 32053, 32054, -1, 32055, 32056, 32057, -1, 32058, 32059, 32060, -1, 32061, 32062, 32063, -1, 32064, 32065, 32066, -1, 32067, 32068, 32069, -1, 32070, 32071, 32072, -1, 32073, 32074, 32075, -1, 32076, 32077, 32078, -1, 32079, 32080, 32081, -1, 32082, 32083, 32084, -1, 32085, 32086, 32087, -1, 32088, 32089, 32090, -1, 32091, 32092, 32093, -1, 32094, 32095, 32096, -1, 32097, 32098, 32099, -1, 32100, 32101, 32102, -1, 32103, 32104, 32105, -1, 32106, 32107, 32108, -1, 32109, 32110, 32111, -1, 32112, 32113, 32114, -1, 32115, 32116, 32117, -1, 32118, 32119, 32120, -1, 32121, 32122, 32123, -1, 32124, 32125, 32126, -1, 32127, 32128, 32129, -1, 32130, 32131, 32132, -1, 32133, 32134, 32135, -1, 32136, 32137, 32138, -1, 32139, 32140, 32141, -1, 32142, 32143, 32144, -1, 32145, 32146, 32147, -1, 32148, 32149, 32150, -1, 32151, 32152, 32153, -1, 32154, 32155, 32156, -1, 32157, 32158, 32159, -1, 32160, 32161, 32162, -1, 32163, 32164, 32165, -1, 32166, 32167, 32168, -1, 32169, 32170, 32171, -1, 32172, 32173, 32174, -1, 32175, 32176, 32177, -1, 32178, 32179, 32180, -1, 32181, 32182, 32183, -1, 32184, 32185, 32186, -1, 32187, 32188, 32189, -1, 32190, 32191, 32192, -1, 32193, 32194, 32195, -1, 32196, 32197, 32198, -1, 32199, 32200, 32201, -1, 32202, 32203, 32204, -1, 32205, 32206, 32207, -1, 32208, 32209, 32210, -1, 32211, 32212, 32213, -1, 32214, 32215, 32216, -1, 32217, 32218, 32219, -1, 32220, 32221, 32222, -1, 32223, 32224, 32225, -1, 32226, 32227, 32228, -1, 32229, 32230, 32231, -1, 32232, 32233, 32234, -1, 32235, 32236, 32237, -1, 32238, 32239, 32240, -1, 32241, 32242, 32243, -1, 32244, 32245, 32246, -1, 32247, 32248, 32249, -1, 32250, 32251, 32252, -1, 32253, 32254, 32255, -1, 32256, 32257, 32258, -1, 32259, 32260, 32261, -1, 32262, 32263, 32264, -1, 32265, 32266, 32267, -1, 32268, 32269, 32270, -1, 32271, 32272, 32273, -1, 32274, 32275, 32276, -1, 32277, 32278, 32279, -1, 32280, 32281, 32282, -1, 32283, 32284, 32285, -1, 32286, 32287, 32288, -1, 32289, 32290, 32291, -1, 32292, 32293, 32294, -1, 32295, 32296, 32297, -1, 32298, 32299, 32300, -1, 32301, 32302, 32303, -1, 32304, 32305, 32306, -1, 32307, 32308, 32309, -1, 32310, 32311, 32312, -1, 32313, 32314, 32315, -1, 32316, 32317, 32318, -1, 32319, 32320, 32321, -1, 32322, 32323, 32324, -1, 32325, 32326, 32327, -1, 32328, 32329, 32330, -1, 32331, 32332, 32333, -1, 32334, 32335, 32336, -1, 32337, 32338, 32339, -1, 32340, 32341, 32342, -1, 32343, 32344, 32345, -1, 32346, 32347, 32348, -1, 32349, 32350, 32351, -1, 32352, 32353, 32354, -1, 32355, 32356, 32357, -1, 32358, 32359, 32360, -1, 32361, 32362, 32363, -1, 32364, 32365, 32366, -1, 32367, 32368, 32369, -1, 32370, 32371, 32372, -1, 32373, 32374, 32375, -1, 32376, 32377, 32378, -1, 32379, 32380, 32381, -1, 32382, 32383, 32384, -1, 32385, 32386, 32387, -1, 32388, 32389, 32390, -1, 32391, 32392, 32393, -1, 32394, 32395, 32396, -1, 32397, 32398, 32399, -1, 32400, 32401, 32402, -1, 32403, 32404, 32405, -1, 32406, 32407, 32408, -1, 32409, 32410, 32411, -1, 32412, 32413, 32414, -1, 32415, 32416, 32417, -1, 32418, 32419, 32420, -1, 32421, 32422, 32423, -1, 32424, 32425, 32426, -1, 32427, 32428, 32429, -1, 32430, 32431, 32432, -1, 32433, 32434, 32435, -1, 32436, 32437, 32438, -1, 32439, 32440, 32441, -1, 32442, 32443, 32444, -1, 32445, 32446, 32447, -1, 32448, 32449, 32450, -1, 32451, 32452, 32453, -1, 32454, 32455, 32456, -1, 32457, 32458, 32459, -1, 32460, 32461, 32462, -1, 32463, 32464, 32465, -1, 32466, 32467, 32468, -1, 32469, 32470, 32471, -1, 32472, 32473, 32474, -1, 32475, 32476, 32477, -1, 32478, 32479, 32480, -1, 32481, 32482, 32483, -1, 32484, 32485, 32486, -1, 32487, 32488, 32489, -1, 32490, 32491, 32492, -1, 32493, 32494, 32495, -1, 32496, 32497, 32498, -1, 32499, 32500, 32501, -1, 32502, 32503, 32504, -1, 32505, 32506, 32507, -1, 32508, 32509, 32510, -1, 32511, 32512, 32513, -1, 32514, 32515, 32516, -1, 32517, 32518, 32519, -1, 32520, 32521, 32522, -1, 32523, 32524, 32525, -1, 32526, 32527, 32528, -1, 32529, 32530, 32531, -1, 32532, 32533, 32534, -1, 32535, 32536, 32537, -1, 32538, 32539, 32540, -1, 32541, 32542, 32543, -1, 32544, 32545, 32546, -1, 32547, 32548, 32549, -1, 32550, 32551, 32552, -1, 32553, 32554, 32555, -1, 32556, 32557, 32558, -1, 32559, 32560, 32561, -1, 32562, 32563, 32564, -1, 32565, 32566, 32567, -1, 32568, 32569, 32570, -1, 32571, 32572, 32573, -1, 32574, 32575, 32576, -1, 32577, 32578, 32579, -1, 32580, 32581, 32582, -1, 32583, 32584, 32585, -1, 32586, 32587, 32588, -1, 32589, 32590, 32591, -1, 32592, 32593, 32594, -1, 32595, 32596, 32597, -1, 32598, 32599, 32600, -1, 32601, 32602, 32603, -1, 32604, 32605, 32606, -1, 32607, 32608, 32609, -1, 32610, 32611, 32612, -1, 32613, 32614, 32615, -1, 32616, 32617, 32618, -1, 32619, 32620, 32621, -1, 32622, 32623, 32624, -1, 32625, 32626, 32627, -1, 32628, 32629, 32630, -1, 32631, 32632, 32633, -1, 32634, 32635, 32636, -1, 32637, 32638, 32639, -1, 32640, 32641, 32642, -1, 32643, 32644, 32645, -1, 32646, 32647, 32648, -1, 32649, 32650, 32651, -1, 32652, 32653, 32654, -1, 32655, 32656, 32657, -1, 32658, 32659, 32660, -1, 32661, 32662, 32663, -1, 32664, 32665, 32666, -1, 32667, 32668, 32669, -1, 32670, 32671, 32672, -1, 32673, 32674, 32675, -1, 32676, 32677, 32678, -1, 32679, 32680, 32681, -1, 32682, 32683, 32684, -1, 32685, 32686, 32687, -1, 32688, 32689, 32690, -1, 32691, 32692, 32693, -1, 32694, 32695, 32696, -1, 32697, 32698, 32699, -1, 32700, 32701, 32702, -1, 32703, 32704, 32705, -1, 32706, 32707, 32708, -1, 32709, 32710, 32711, -1, 32712, 32713, 32714, -1, 32715, 32716, 32717, -1, 32718, 32719, 32720, -1, 32721, 32722, 32723, -1, 32724, 32725, 32726, -1, 32727, 32728, 32729, -1, 32730, 32731, 32732, -1, 32733, 32734, 32735, -1, 32736, 32737, 32738, -1, 32739, 32740, 32741, -1, 32742, 32743, 32744, -1, 32745, 32746, 32747, -1, 32748, 32749, 32750, -1, 32751, 32752, 32753, -1, 32754, 32755, 32756, -1, 32757, 32758, 32759, -1, 32760, 32761, 32762, -1, 32763, 32764, 32765, -1, 32766, 32767, 32768, -1, 32769, 32770, 32771, -1, 32772, 32773, 32774, -1, 32775, 32776, 32777, -1, 32778, 32779, 32780, -1, 32781, 32782, 32783, -1, 32784, 32785, 32786, -1, 32787, 32788, 32789, -1, 32790, 32791, 32792, -1, 32793, 32794, 32795, -1, 32796, 32797, 32798, -1, 32799, 32800, 32801, -1, 32802, 32803, 32804, -1, 32805, 32806, 32807, -1, 32808, 32809, 32810, -1, 32811, 32812, 32813, -1, 32814, 32815, 32816, -1, 32817, 32818, 32819, -1, 32820, 32821, 32822, -1, 32823, 32824, 32825, -1, 32826, 32827, 32828, -1, 32829, 32830, 32831, -1, 32832, 32833, 32834, -1, 32835, 32836, 32837, -1, 32838, 32839, 32840, -1, 32841, 32842, 32843, -1, 32844, 32845, 32846, -1, 32847, 32848, 32849, -1, 32850, 32851, 32852, -1, 32853, 32854, 32855, -1, 32856, 32857, 32858, -1, 32859, 32860, 32861, -1, 32862, 32863, 32864, -1, 32865, 32866, 32867, -1, 32868, 32869, 32870, -1, 32871, 32872, 32873, -1, 32874, 32875, 32876, -1, 32877, 32878, 32879, -1, 32880, 32881, 32882, -1, 32883, 32884, 32885, -1, 32886, 32887, 32888, -1, 32889, 32890, 32891, -1, 32892, 32893, 32894, -1, 32895, 32896, 32897, -1, 32898, 32899, 32900, -1, 32901, 32902, 32903, -1, 32904, 32905, 32906, -1, 32907, 32908, 32909, -1, 32910, 32911, 32912, -1, 32913, 32914, 32915, -1, 32916, 32917, 32918, -1, 32919, 32920, 32921, -1, 32922, 32923, 32924, -1, 32925, 32926, 32927, -1, 32928, 32929, 32930, -1, 32931, 32932, 32933, -1, 32934, 32935, 32936, -1, 32937, 32938, 32939, -1, 32940, 32941, 32942, -1, 32943, 32944, 32945, -1, 32946, 32947, 32948, -1, 32949, 32950, 32951, -1, 32952, 32953, 32954, -1, 32955, 32956, 32957, -1, 32958, 32959, 32960, -1, 32961, 32962, 32963, -1, 32964, 32965, 32966, -1, 32967, 32968, 32969, -1, 32970, 32971, 32972, -1, 32973, 32974, 32975, -1, 32976, 32977, 32978, -1, 32979, 32980, 32981, -1, 32982, 32983, 32984, -1, 32985, 32986, 32987, -1, 32988, 32989, 32990, -1, 32991, 32992, 32993, -1, 32994, 32995, 32996, -1, 32997, 32998, 32999, -1, 33000, 33001, 33002, -1, 33003, 33004, 33005, -1, 33006, 33007, 33008, -1, 33009, 33010, 33011, -1, 33012, 33013, 33014, -1, 33015, 33016, 33017, -1, 33018, 33019, 33020, -1, 33021, 33022, 33023, -1, 33024, 33025, 33026, -1, 33027, 33028, 33029, -1, 33030, 33031, 33032, -1, 33033, 33034, 33035, -1, 33036, 33037, 33038, -1, 33039, 33040, 33041, -1, 33042, 33043, 33044, -1, 33045, 33046, 33047, -1, 33048, 33049, 33050, -1, 33051, 33052, 33053, -1, 33054, 33055, 33056, -1, 33057, 33058, 33059, -1, 33060, 33061, 33062, -1, 33063, 33064, 33065, -1, 33066, 33067, 33068, -1, 33069, 33070, 33071, -1, 33072, 33073, 33074, -1, 33075, 33076, 33077, -1, 33078, 33079, 33080, -1, 33081, 33082, 33083, -1, 33084, 33085, 33086, -1, 33087, 33088, 33089, -1, 33090, 33091, 33092, -1, 33093, 33094, 33095, -1, 33096, 33097, 33098, -1, 33099, 33100, 33101, -1, 33102, 33103, 33104, -1, 33105, 33106, 33107, -1, 33108, 33109, 33110, -1, 33111, 33112, 33113, -1, 33114, 33115, 33116, -1, 33117, 33118, 33119, -1, 33120, 33121, 33122, -1, 33123, 33124, 33125, -1, 33126, 33127, 33128, -1, 33129, 33130, 33131, -1, 33132, 33133, 33134, -1, 33135, 33136, 33137, -1, 33138, 33139, 33140, -1, 33141, 33142, 33143, -1, 33144, 33145, 33146, -1, 33147, 33148, 33149, -1, 33150, 33151, 33152, -1, 33153, 33154, 33155, -1, 33156, 33157, 33158, -1, 33159, 33160, 33161, -1, 33162, 33163, 33164, -1, 33165, 33166, 33167, -1, 33168, 33169, 33170, -1, 33171, 33172, 33173, -1, 33174, 33175, 33176, -1, 33177, 33178, 33179, -1, 33180, 33181, 33182, -1, 33183, 33184, 33185, -1, 33186, 33187, 33188, -1, 33189, 33190, 33191, -1, 33192, 33193, 33194, -1, 33195, 33196, 33197, -1, 33198, 33199, 33200, -1, 33201, 33202, 33203, -1, 33204, 33205, 33206, -1, 33207, 33208, 33209, -1, 33210, 33211, 33212, -1, 33213, 33214, 33215, -1, 33216, 33217, 33218, -1, 33219, 33220, 33221, -1, 33222, 33223, 33224, -1, 33225, 33226, 33227, -1, 33228, 33229, 33230, -1, 33231, 33232, 33233, -1, 33234, 33235, 33236, -1, 33237, 33238, 33239, -1, 33240, 33241, 33242, -1, 33243, 33244, 33245, -1, 33246, 33247, 33248, -1, 33249, 33250, 33251, -1, 33252, 33253, 33254, -1, 33255, 33256, 33257, -1, 33258, 33259, 33260, -1, 33261, 33262, 33263, -1, 33264, 33265, 33266, -1, 33267, 33268, 33269, -1, 33270, 33271, 33272, -1, 33273, 33274, 33275, -1, 33276, 33277, 33278, -1, 33279, 33280, 33281, -1, 33282, 33283, 33284, -1, 33285, 33286, 33287, -1, 33288, 33289, 33290, -1, 33291, 33292, 33293, -1, 33294, 33295, 33296, -1, 33297, 33298, 33299, -1, 33300, 33301, 33302, -1, 33303, 33304, 33305, -1, 33306, 33307, 33308, -1, 33309, 33310, 33311, -1, 33312, 33313, 33314, -1, 33315, 33316, 33317, -1, 33318, 33319, 33320, -1, 33321, 33322, 33323, -1, 33324, 33325, 33326, -1, 33327, 33328, 33329, -1, 33330, 33331, 33332, -1, 33333, 33334, 33335, -1, 33336, 33337, 33338, -1, 33339, 33340, 33341, -1, 33342, 33343, 33344, -1, 33345, 33346, 33347, -1, 33348, 33349, 33350, -1, 33351, 33352, 33353, -1, 33354, 33355, 33356, -1, 33357, 33358, 33359, -1, 33360, 33361, 33362, -1, 33363, 33364, 33365, -1, 33366, 33367, 33368, -1, 33369, 33370, 33371, -1, 33372, 33373, 33374, -1, 33375, 33376, 33377, -1, 33378, 33379, 33380, -1, 33381, 33382, 33383, -1, 33384, 33385, 33386, -1, 33387, 33388, 33389, -1, 33390, 33391, 33392, -1, 33393, 33394, 33395, -1, 33396, 33397, 33398, -1, 33399, 33400, 33401, -1, 33402, 33403, 33404, -1, 33405, 33406, 33407, -1, 33408, 33409, 33410, -1, 33411, 33412, 33413, -1, 33414, 33415, 33416, -1, 33417, 33418, 33419, -1, 33420, 33421, 33422, -1, 33423, 33424, 33425, -1, 33426, 33427, 33428, -1, 33429, 33430, 33431, -1, 33432, 33433, 33434, -1, 33435, 33436, 33437, -1, 33438, 33439, 33440, -1, 33441, 33442, 33443, -1, 33444, 33445, 33446, -1, 33447, 33448, 33449, -1, 33450, 33451, 33452, -1, 33453, 33454, 33455, -1, 33456, 33457, 33458, -1, 33459, 33460, 33461, -1, 33462, 33463, 33464, -1, 33465, 33466, 33467, -1, 33468, 33469, 33470, -1, 33471, 33472, 33473, -1, 33474, 33475, 33476, -1, 33477, 33478, 33479, -1, 33480, 33481, 33482, -1, 33483, 33484, 33485, -1, 33486, 33487, 33488, -1, 33489, 33490, 33491, -1, 33492, 33493, 33494, -1, 33495, 33496, 33497, -1, 33498, 33499, 33500, -1, 33501, 33502, 33503, -1, 33504, 33505, 33506, -1, 33507, 33508, 33509, -1, 33510, 33511, 33512, -1, 33513, 33514, 33515, -1, 33516, 33517, 33518, -1, 33519, 33520, 33521, -1, 33522, 33523, 33524, -1, 33525, 33526, 33527, -1, 33528, 33529, 33530, -1, 33531, 33532, 33533, -1, 33534, 33535, 33536, -1, 33537, 33538, 33539, -1, 33540, 33541, 33542, -1, 33543, 33544, 33545, -1, 33546, 33547, 33548, -1, 33549, 33550, 33551, -1, 33552, 33553, 33554, -1, 33555, 33556, 33557, -1, 33558, 33559, 33560, -1, 33561, 33562, 33563, -1, 33564, 33565, 33566, -1, 33567, 33568, 33569, -1, 33570, 33571, 33572, -1, 33573, 33574, 33575, -1, 33576, 33577, 33578, -1, 33579, 33580, 33581, -1, 33582, 33583, 33584, -1, 33585, 33586, 33587, -1, 33588, 33589, 33590, -1, 33591, 33592, 33593, -1, 33594, 33595, 33596, -1, 33597, 33598, 33599, -1, 33600, 33601, 33602, -1, 33603, 33604, 33605, -1, 33606, 33607, 33608, -1, 33609, 33610, 33611, -1, 33612, 33613, 33614, -1, 33615, 33616, 33617, -1, 33618, 33619, 33620, -1, 33621, 33622, 33623, -1, 33624, 33625, 33626, -1, 33627, 33628, 33629, -1, 33630, 33631, 33632, -1, 33633, 33634, 33635, -1, 33636, 33637, 33638, -1, 33639, 33640, 33641, -1, 33642, 33643, 33644, -1, 33645, 33646, 33647, -1, 33648, 33649, 33650, -1, 33651, 33652, 33653, -1, 33654, 33655, 33656, -1, 33657, 33658, 33659, -1, 33660, 33661, 33662, -1, 33663, 33664, 33665, -1, 33666, 33667, 33668, -1, 33669, 33670, 33671, -1, 33672, 33673, 33674, -1, 33675, 33676, 33677, -1, 33678, 33679, 33680, -1, 33681, 33682, 33683, -1, 33684, 33685, 33686, -1, 33687, 33688, 33689, -1, 33690, 33691, 33692, -1, 33693, 33694, 33695, -1, 33696, 33697, 33698, -1, 33699, 33700, 33701, -1, 33702, 33703, 33704, -1, 33705, 33706, 33707, -1, 33708, 33709, 33710, -1, 33711, 33712, 33713, -1, 33714, 33715, 33716, -1, 33717, 33718, 33719, -1, 33720, 33721, 33722, -1, 33723, 33724, 33725, -1, 33726, 33727, 33728, -1, 33729, 33730, 33731, -1, 33732, 33733, 33734, -1, 33735, 33736, 33737, -1, 33738, 33739, 33740, -1, 33741, 33742, 33743, -1, 33744, 33745, 33746, -1, 33747, 33748, 33749, -1, 33750, 33751, 33752, -1, 33753, 33754, 33755, -1, 33756, 33757, 33758, -1, 33759, 33760, 33761, -1, 33762, 33763, 33764, -1, 33765, 33766, 33767, -1, 33768, 33769, 33770, -1, 33771, 33772, 33773, -1, 33774, 33775, 33776, -1, 33777, 33778, 33779, -1, 33780, 33781, 33782, -1, 33783, 33784, 33785, -1, 33786, 33787, 33788, -1, 33789, 33790, 33791, -1, 33792, 33793, 33794, -1, 33795, 33796, 33797, -1, 33798, 33799, 33800, -1, 33801, 33802, 33803, -1, 33804, 33805, 33806, -1, 33807, 33808, 33809, -1, 33810, 33811, 33812, -1, 33813, 33814, 33815, -1, 33816, 33817, 33818, -1, 33819, 33820, 33821, -1, 33822, 33823, 33824, -1, 33825, 33826, 33827, -1, 33828, 33829, 33830, -1, 33831, 33832, 33833, -1, 33834, 33835, 33836, -1, 33837, 33838, 33839, -1, 33840, 33841, 33842, -1, 33843, 33844, 33845, -1, 33846, 33847, 33848, -1, 33849, 33850, 33851, -1, 33852, 33853, 33854, -1, 33855, 33856, 33857, -1, 33858, 33859, 33860, -1, 33861, 33862, 33863, -1, 33864, 33865, 33866, -1, 33867, 33868, 33869, -1, 33870, 33871, 33872, -1, 33873, 33874, 33875, -1, 33876, 33877, 33878, -1, 33879, 33880, 33881, -1, 33882, 33883, 33884, -1, 33885, 33886, 33887, -1, 33888, 33889, 33890, -1, 33891, 33892, 33893, -1, 33894, 33895, 33896, -1, 33897, 33898, 33899, -1, 33900, 33901, 33902, -1, 33903, 33904, 33905, -1, 33906, 33907, 33908, -1, 33909, 33910, 33911, -1, 33912, 33913, 33914, -1, 33915, 33916, 33917, -1, 33918, 33919, 33920, -1, 33921, 33922, 33923, -1, 33924, 33925, 33926, -1, 33927, 33928, 33929, -1, 33930, 33931, 33932, -1, 33933, 33934, 33935, -1, 33936, 33937, 33938, -1, 33939, 33940, 33941, -1, 33942, 33943, 33944, -1, 33945, 33946, 33947, -1, 33948, 33949, 33950, -1, 33951, 33952, 33953, -1, 33954, 33955, 33956, -1, 33957, 33958, 33959, -1, 33960, 33961, 33962, -1, 33963, 33964, 33965, -1, 33966, 33967, 33968, -1, 33969, 33970, 33971, -1, 33972, 33973, 33974, -1, 33975, 33976, 33977, -1, 33978, 33979, 33980, -1, 33981, 33982, 33983, -1, 33984, 33985, 33986, -1, 33987, 33988, 33989, -1, 33990, 33991, 33992, -1, 33993, 33994, 33995, -1, 33996, 33997, 33998, -1, 33999, 34000, 34001, -1, 34002, 34003, 34004, -1, 34005, 34006, 34007, -1, 34008, 34009, 34010, -1, 34011, 34012, 34013, -1, 34014, 34015, 34016, -1, 34017, 34018, 34019, -1, 34020, 34021, 34022, -1, 34023, 34024, 34025, -1, 34026, 34027, 34028, -1, 34029, 34030, 34031, -1, 34032, 34033, 34034, -1, 34035, 34036, 34037, -1, 34038, 34039, 34040, -1, 34041, 34042, 34043, -1, 34044, 34045, 34046, -1, 34047, 34048, 34049, -1, 34050, 34051, 34052, -1, 34053, 34054, 34055, -1, 34056, 34057, 34058, -1, 34059, 34060, 34061, -1, 34062, 34063, 34064, -1, 34065, 34066, 34067, -1, 34068, 34069, 34070, -1, 34071, 34072, 34073, -1, 34074, 34075, 34076, -1, 34077, 34078, 34079, -1, 34080, 34081, 34082, -1, 34083, 34084, 34085, -1, 34086, 34087, 34088, -1, 34089, 34090, 34091, -1, 34092, 34093, 34094, -1, 34095, 34096, 34097, -1, 34098, 34099, 34100, -1, 34101, 34102, 34103, -1, 34104, 34105, 34106, -1, 34107, 34108, 34109, -1, 34110, 34111, 34112, -1, 34113, 34114, 34115, -1, 34116, 34117, 34118, -1, 34119, 34120, 34121, -1, 34122, 34123, 34124, -1, 34125, 34126, 34127, -1, 34128, 34129, 34130, -1, 34131, 34132, 34133, -1, 34134, 34135, 34136, -1, 34137, 34138, 34139, -1, 34140, 34141, 34142, -1, 34143, 34144, 34145, -1, 34146, 34147, 34148, -1, 34149, 34150, 34151, -1, 34152, 34153, 34154, -1, 34155, 34156, 34157, -1, 34158, 34159, 34160, -1, 34161, 34162, 34163, -1, 34164, 34165, 34166, -1, 34167, 34168, 34169, -1, 34170, 34171, 34172, -1, 34173, 34174, 34175, -1, 34176, 34177, 34178, -1, 34179, 34180, 34181, -1, 34182, 34183, 34184, -1, 34185, 34186, 34187, -1, 34188, 34189, 34190, -1, 34191, 34192, 34193, -1, 34194, 34195, 34196, -1, 34197, 34198, 34199, -1, 34200, 34201, 34202, -1, 34203, 34204, 34205, -1, 34206, 34207, 34208, -1, 34209, 34210, 34211, -1, 34212, 34213, 34214, -1, 34215, 34216, 34217, -1, 34218, 34219, 34220, -1, 34221, 34222, 34223, -1, 34224, 34225, 34226, -1, 34227, 34228, 34229, -1, 34230, 34231, 34232, -1, 34233, 34234, 34235, -1, 34236, 34237, 34238, -1, 34239, 34240, 34241, -1, 34242, 34243, 34244, -1, 34245, 34246, 34247, -1, 34248, 34249, 34250, -1, 34251, 34252, 34253, -1, 34254, 34255, 34256, -1, 34257, 34258, 34259, -1, 34260, 34261, 34262, -1, 34263, 34264, 34265, -1, 34266, 34267, 34268, -1, 34269, 34270, 34271, -1, 34272, 34273, 34274, -1, 34275, 34276, 34277, -1, 34278, 34279, 34280, -1, 34281, 34282, 34283, -1, 34284, 34285, 34286, -1, 34287, 34288, 34289, -1, 34290, 34291, 34292, -1, 34293, 34294, 34295, -1, 34296, 34297, 34298, -1, 34299, 34300, 34301, -1, 34302, 34303, 34304, -1, 34305, 34306, 34307, -1, 34308, 34309, 34310, -1, 34311, 34312, 34313, -1, 34314, 34315, 34316, -1, 34317, 34318, 34319, -1, 34320, 34321, 34322, -1, 34323, 34324, 34325, -1, 34326, 34327, 34328, -1, 34329, 34330, 34331, -1, 34332, 34333, 34334, -1, 34335, 34336, 34337, -1, 34338, 34339, 34340, -1, 34341, 34342, 34343, -1, 34344, 34345, 34346, -1, 34347, 34348, 34349, -1, 34350, 34351, 34352, -1, 34353, 34354, 34355, -1, 34356, 34357, 34358, -1, 34359, 34360, 34361, -1, 34362, 34363, 34364, -1, 34365, 34366, 34367, -1, 34368, 34369, 34370, -1, 34371, 34372, 34373, -1, 34374, 34375, 34376, -1, 34377, 34378, 34379, -1, 34380, 34381, 34382, -1, 34383, 34384, 34385, -1, 34386, 34387, 34388, -1, 34389, 34390, 34391, -1, 34392, 34393, 34394, -1, 34395, 34396, 34397, -1, 34398, 34399, 34400, -1, 34401, 34402, 34403, -1, 34404, 34405, 34406, -1, 34407, 34408, 34409, -1, 34410, 34411, 34412, -1, 34413, 34414, 34415, -1, 34416, 34417, 34418, -1, 34419, 34420, 34421, -1, 34422, 34423, 34424, -1, 34425, 34426, 34427, -1, 34428, 34429, 34430, -1, 34431, 34432, 34433, -1, 34434, 34435, 34436, -1, 34437, 34438, 34439, -1, 34440, 34441, 34442, -1, 34443, 34444, 34445, -1, 34446, 34447, 34448, -1, 34449, 34450, 34451, -1, 34452, 34453, 34454, -1, 34455, 34456, 34457, -1, 34458, 34459, 34460, -1, 34461, 34462, 34463, -1, 34464, 34465, 34466, -1, 34467, 34468, 34469, -1, 34470, 34471, 34472, -1, 34473, 34474, 34475, -1, 34476, 34477, 34478, -1, 34479, 34480, 34481, -1, 34482, 34483, 34484, -1, 34485, 34486, 34487, -1, 34488, 34489, 34490, -1, 34491, 34492, 34493, -1, 34494, 34495, 34496, -1, 34497, 34498, 34499, -1, 34500, 34501, 34502, -1, 34503, 34504, 34505, -1, 34506, 34507, 34508, -1, 34509, 34510, 34511, -1, 34512, 34513, 34514, -1, 34515, 34516, 34517, -1, 34518, 34519, 34520, -1, 34521, 34522, 34523, -1, 34524, 34525, 34526, -1, 34527, 34528, 34529, -1, 34530, 34531, 34532, -1, 34533, 34534, 34535, -1, 34536, 34537, 34538, -1, 34539, 34540, 34541, -1, 34542, 34543, 34544, -1, 34545, 34546, 34547, -1, 34548, 34549, 34550, -1, 34551, 34552, 34553, -1, 34554, 34555, 34556, -1, 34557, 34558, 34559, -1, 34560, 34561, 34562, -1, 34563, 34564, 34565, -1, 34566, 34567, 34568, -1, 34569, 34570, 34571, -1, 34572, 34573, 34574, -1, 34575, 34576, 34577, -1, 34578, 34579, 34580, -1, 34581, 34582, 34583, -1, 34584, 34585, 34586, -1, 34587, 34588, 34589, -1, 34590, 34591, 34592, -1, 34593, 34594, 34595, -1, 34596, 34597, 34598, -1, 34599, 34600, 34601, -1, 34602, 34603, 34604, -1, 34605, 34606, 34607, -1, 34608, 34609, 34610, -1, 34611, 34612, 34613, -1, 34614, 34615, 34616, -1, 34617, 34618, 34619, -1, 34620, 34621, 34622, -1, 34623, 34624, 34625, -1, 34626, 34627, 34628, -1, 34629, 34630, 34631, -1, 34632, 34633, 34634, -1, 34635, 34636, 34637, -1, 34638, 34639, 34640, -1, 34641, 34642, 34643, -1, 34644, 34645, 34646, -1, 34647, 34648, 34649, -1, 34650, 34651, 34652, -1, 34653, 34654, 34655, -1, 34656, 34657, 34658, -1, 34659, 34660, 34661, -1, 34662, 34663, 34664, -1, 34665, 34666, 34667, -1, 34668, 34669, 34670, -1, 34671, 34672, 34673, -1, 34674, 34675, 34676, -1, 34677, 34678, 34679, -1, 34680, 34681, 34682, -1, 34683, 34684, 34685, -1, 34686, 34687, 34688, -1, 34689, 34690, 34691, -1, 34692, 34693, 34694, -1, 34695, 34696, 34697, -1, 34698, 34699, 34700, -1, 34701, 34702, 34703, -1, 34704, 34705, 34706, -1, 34707, 34708, 34709, -1, 34710, 34711, 34712, -1, 34713, 34714, 34715, -1, 34716, 34717, 34718, -1, 34719, 34720, 34721, -1, 34722, 34723, 34724, -1, 34725, 34726, 34727, -1, 34728, 34729, 34730, -1, 34731, 34732, 34733, -1, 34734, 34735, 34736, -1, 34737, 34738, 34739, -1, 34740, 34741, 34742, -1, 34743, 34744, 34745, -1, 34746, 34747, 34748, -1, 34749, 34750, 34751, -1, 34752, 34753, 34754, -1, 34755, 34756, 34757, -1, 34758, 34759, 34760, -1, 34761, 34762, 34763, -1, 34764, 34765, 34766, -1, 34767, 34768, 34769, -1, 34770, 34771, 34772, -1, 34773, 34774, 34775, -1, 34776, 34777, 34778, -1, 34779, 34780, 34781, -1, 34782, 34783, 34784, -1, 34785, 34786, 34787, -1, 34788, 34789, 34790, -1, 34791, 34792, 34793, -1, 34794, 34795, 34796, -1, 34797, 34798, 34799, -1, 34800, 34801, 34802, -1, 34803, 34804, 34805, -1, 34806, 34807, 34808, -1, 34809, 34810, 34811, -1, 34812, 34813, 34814, -1, 34815, 34816, 34817, -1, 34818, 34819, 34820, -1, 34821, 34822, 34823, -1, 34824, 34825, 34826, -1, 34827, 34828, 34829, -1, 34830, 34831, 34832, -1, 34833, 34834, 34835, -1, 34836, 34837, 34838, -1, 34839, 34840, 34841, -1, 34842, 34843, 34844, -1, 34845, 34846, 34847, -1, 34848, 34849, 34850, -1, 34851, 34852, 34853, -1, 34854, 34855, 34856, -1, 34857, 34858, 34859, -1, 34860, 34861, 34862, -1, 34863, 34864, 34865, -1, 34866, 34867, 34868, -1, 34869, 34870, 34871, -1, 34872, 34873, 34874, -1, 34875, 34876, 34877, -1, 34878, 34879, 34880, -1, 34881, 34882, 34883, -1, 34884, 34885, 34886, -1, 34887, 34888, 34889, -1, 34890, 34891, 34892, -1, 34893, 34894, 34895, -1, 34896, 34897, 34898, -1, 34899, 34900, 34901, -1, 34902, 34903, 34904, -1, 34905, 34906, 34907, -1, 34908, 34909, 34910, -1, 34911, 34912, 34913, -1, 34914, 34915, 34916, -1, 34917, 34918, 34919, -1, 34920, 34921, 34922, -1, 34923, 34924, 34925, -1, 34926, 34927, 34928, -1, 34929, 34930, 34931, -1, 34932, 34933, 34934, -1, 34935, 34936, 34937, -1, 34938, 34939, 34940, -1, 34941, 34942, 34943, -1, 34944, 34945, 34946, -1, 34947, 34948, 34949, -1, 34950, 34951, 34952, -1, 34953, 34954, 34955, -1, 34956, 34957, 34958, -1, 34959, 34960, 34961, -1, 34962, 34963, 34964, -1, 34965, 34966, 34967, -1, 34968, 34969, 34970, -1, 34971, 34972, 34973, -1, 34974, 34975, 34976, -1, 34977, 34978, 34979, -1, 34980, 34981, 34982, -1, 34983, 34984, 34985, -1, 34986, 34987, 34988, -1, 34989, 34990, 34991, -1, 34992, 34993, 34994, -1, 34995, 34996, 34997, -1, 34998, 34999, 35000, -1, 35001, 35002, 35003, -1, 35004, 35005, 35006, -1, 35007, 35008, 35009, -1, 35010, 35011, 35012, -1, 35013, 35014, 35015, -1, 35016, 35017, 35018, -1, 35019, 35020, 35021, -1, 35022, 35023, 35024, -1, 35025, 35026, 35027, -1, 35028, 35029, 35030, -1, 35031, 35032, 35033, -1, 35034, 35035, 35036, -1, 35037, 35038, 35039, -1, 35040, 35041, 35042, -1, 35043, 35044, 35045, -1, 35046, 35047, 35048, -1, 35049, 35050, 35051, -1, 35052, 35053, 35054, -1, 35055, 35056, 35057, -1, 35058, 35059, 35060, -1, 35061, 35062, 35063, -1, 35064, 35065, 35066, -1, 35067, 35068, 35069, -1, 35070, 35071, 35072, -1, 35073, 35074, 35075, -1, 35076, 35077, 35078, -1, 35079, 35080, 35081, -1, 35082, 35083, 35084, -1, 35085, 35086, 35087, -1, 35088, 35089, 35090, -1, 35091, 35092, 35093, -1, 35094, 35095, 35096, -1, 35097, 35098, 35099, -1, 35100, 35101, 35102, -1, 35103, 35104, 35105, -1, 35106, 35107, 35108, -1, 35109, 35110, 35111, -1, 35112, 35113, 35114, -1, 35115, 35116, 35117, -1, 35118, 35119, 35120, -1, 35121, 35122, 35123, -1, 35124, 35125, 35126, -1, 35127, 35128, 35129, -1, 35130, 35131, 35132, -1, 35133, 35134, 35135, -1, 35136, 35137, 35138, -1, 35139, 35140, 35141, -1, 35142, 35143, 35144, -1, 35145, 35146, 35147, -1, 35148, 35149, 35150, -1, 35151, 35152, 35153, -1, 35154, 35155, 35156, -1, 35157, 35158, 35159, -1, 35160, 35161, 35162, -1, 35163, 35164, 35165, -1, 35166, 35167, 35168, -1, 35169, 35170, 35171, -1, 35172, 35173, 35174, -1, 35175, 35176, 35177, -1, 35178, 35179, 35180, -1, 35181, 35182, 35183, -1, 35184, 35185, 35186, -1, 35187, 35188, 35189, -1, 35190, 35191, 35192, -1, 35193, 35194, 35195, -1, 35196, 35197, 35198, -1, 35199, 35200, 35201, -1, 35202, 35203, 35204, -1, 35205, 35206, 35207, -1, 35208, 35209, 35210, -1, 35211, 35212, 35213, -1, 35214, 35215, 35216, -1, 35217, 35218, 35219, -1, 35220, 35221, 35222, -1, 35223, 35224, 35225, -1, 35226, 35227, 35228, -1, 35229, 35230, 35231, -1, 35232, 35233, 35234, -1, 35235, 35236, 35237, -1, 35238, 35239, 35240, -1, 35241, 35242, 35243, -1, 35244, 35245, 35246, -1, 35247, 35248, 35249, -1, 35250, 35251, 35252, -1, 35253, 35254, 35255, -1, 35256, 35257, 35258, -1, 35259, 35260, 35261, -1, 35262, 35263, 35264, -1, 35265, 35266, 35267, -1, 35268, 35269, 35270, -1, 35271, 35272, 35273, -1, 35274, 35275, 35276, -1, 35277, 35278, 35279, -1, 35280, 35281, 35282, -1, 35283, 35284, 35285, -1, 35286, 35287, 35288, -1, 35289, 35290, 35291, -1, 35292, 35293, 35294, -1, 35295, 35296, 35297, -1, 35298, 35299, 35300, -1, 35301, 35302, 35303, -1, 35304, 35305, 35306, -1, 35307, 35308, 35309, -1, 35310, 35311, 35312, -1, 35313, 35314, 35315, -1, 35316, 35317, 35318, -1, 35319, 35320, 35321, -1, 35322, 35323, 35324, -1, 35325, 35326, 35327, -1, 35328, 35329, 35330, -1, 35331, 35332, 35333, -1, 35334, 35335, 35336, -1, 35337, 35338, 35339, -1, 35340, 35341, 35342, -1, 35343, 35344, 35345, -1, 35346, 35347, 35348, -1, 35349, 35350, 35351, -1, 35352, 35353, 35354, -1, 35355, 35356, 35357, -1, 35358, 35359, 35360, -1, 35361, 35362, 35363, -1, 35364, 35365, 35366, -1, 35367, 35368, 35369, -1, 35370, 35371, 35372, -1, 35373, 35374, 35375, -1, 35376, 35377, 35378, -1, 35379, 35380, 35381, -1, 35382, 35383, 35384, -1, 35385, 35386, 35387, -1, 35388, 35389, 35390, -1, 35391, 35392, 35393, -1, 35394, 35395, 35396, -1, 35397, 35398, 35399, -1, 35400, 35401, 35402, -1, 35403, 35404, 35405, -1, 35406, 35407, 35408, -1, 35409, 35410, 35411, -1, 35412, 35413, 35414, -1, 35415, 35416, 35417, -1, 35418, 35419, 35420, -1, 35421, 35422, 35423, -1, 35424, 35425, 35426, -1, 35427, 35428, 35429, -1, 35430, 35431, 35432, -1, 35433, 35434, 35435, -1, 35436, 35437, 35438, -1, 35439, 35440, 35441, -1, 35442, 35443, 35444, -1, 35445, 35446, 35447, -1, 35448, 35449, 35450, -1, 35451, 35452, 35453, -1, 35454, 35455, 35456, -1, 35457, 35458, 35459, -1, 35460, 35461, 35462, -1, 35463, 35464, 35465, -1, 35466, 35467, 35468, -1, 35469, 35470, 35471, -1, 35472, 35473, 35474, -1, 35475, 35476, 35477, -1, 35478, 35479, 35480, -1, 35481, 35482, 35483, -1, 35484, 35485, 35486, -1, 35487, 35488, 35489, -1, 35490, 35491, 35492, -1, 35493, 35494, 35495, -1, 35496, 35497, 35498, -1, 35499, 35500, 35501, -1, 35502, 35503, 35504, -1, 35505, 35506, 35507, -1, 35508, 35509, 35510, -1, 35511, 35512, 35513, -1, 35514, 35515, 35516, -1, 35517, 35518, 35519, -1, 35520, 35521, 35522, -1, 35523, 35524, 35525, -1, 35526, 35527, 35528, -1, 35529, 35530, 35531, -1, 35532, 35533, 35534, -1, 35535, 35536, 35537, -1, 35538, 35539, 35540, -1, 35541, 35542, 35543, -1, 35544, 35545, 35546, -1, 35547, 35548, 35549, -1, 35550, 35551, 35552, -1, 35553, 35554, 35555, -1, 35556, 35557, 35558, -1, 35559, 35560, 35561, -1, 35562, 35563, 35564, -1, 35565, 35566, 35567, -1, 35568, 35569, 35570, -1, 35571, 35572, 35573, -1, 35574, 35575, 35576, -1, 35577, 35578, 35579, -1, 35580, 35581, 35582, -1, 35583, 35584, 35585, -1, 35586, 35587, 35588, -1, 35589, 35590, 35591, -1, 35592, 35593, 35594, -1, 35595, 35596, 35597, -1, 35598, 35599, 35600, -1, 35601, 35602, 35603, -1, 35604, 35605, 35606, -1, 35607, 35608, 35609, -1, 35610, 35611, 35612, -1, 35613, 35614, 35615, -1, 35616, 35617, 35618, -1, 35619, 35620, 35621, -1, 35622, 35623, 35624, -1, 35625, 35626, 35627, -1, 35628, 35629, 35630, -1, 35631, 35632, 35633, -1, 35634, 35635, 35636, -1, 35637, 35638, 35639, -1, 35640, 35641, 35642, -1, 35643, 35644, 35645, -1, 35646, 35647, 35648, -1, 35649, 35650, 35651, -1, 35652, 35653, 35654, -1, 35655, 35656, 35657, -1, 35658, 35659, 35660, -1, 35661, 35662, 35663, -1, 35664, 35665, 35666, -1, 35667, 35668, 35669, -1, 35670, 35671, 35672, -1, 35673, 35674, 35675, -1, 35676, 35677, 35678, -1, 35679, 35680, 35681, -1, 35682, 35683, 35684, -1, 35685, 35686, 35687, -1, 35688, 35689, 35690, -1, 35691, 35692, 35693, -1, 35694, 35695, 35696, -1, 35697, 35698, 35699, -1, 35700, 35701, 35702, -1, 35703, 35704, 35705, -1, 35706, 35707, 35708, -1, 35709, 35710, 35711, -1, 35712, 35713, 35714, -1, 35715, 35716, 35717, -1, 35718, 35719, 35720, -1, 35721, 35722, 35723, -1, 35724, 35725, 35726, -1, 35727, 35728, 35729, -1, 35730, 35731, 35732, -1, 35733, 35734, 35735, -1, 35736, 35737, 35738, -1, 35739, 35740, 35741, -1, 35742, 35743, 35744, -1, 35745, 35746, 35747, -1, 35748, 35749, 35750, -1, 35751, 35752, 35753, -1, 35754, 35755, 35756, -1, 35757, 35758, 35759, -1, 35760, 35761, 35762, -1, 35763, 35764, 35765, -1, 35766, 35767, 35768, -1, 35769, 35770, 35771, -1, 35772, 35773, 35774, -1, 35775, 35776, 35777, -1, 35778, 35779, 35780, -1, 35781, 35782, 35783, -1, 35784, 35785, 35786, -1, 35787, 35788, 35789, -1, 35790, 35791, 35792, -1, 35793, 35794, 35795, -1, 35796, 35797, 35798, -1, 35799, 35800, 35801, -1, 35802, 35803, 35804, -1, 35805, 35806, 35807, -1, 35808, 35809, 35810, -1, 35811, 35812, 35813, -1, 35814, 35815, 35816, -1, 35817, 35818, 35819, -1, 35820, 35821, 35822, -1, 35823, 35824, 35825, -1, 35826, 35827, 35828, -1, 35829, 35830, 35831, -1, 35832, 35833, 35834, -1, 35835, 35836, 35837, -1, 35838, 35839, 35840, -1, 35841, 35842, 35843, -1, 35844, 35845, 35846, -1, 35847, 35848, 35849, -1, 35850, 35851, 35852, -1, 35853, 35854, 35855, -1, 35856, 35857, 35858, -1, 35859, 35860, 35861, -1, 35862, 35863, 35864, -1, 35865, 35866, 35867, -1, 35868, 35869, 35870, -1, 35871, 35872, 35873, -1, 35874, 35875, 35876, -1, 35877, 35878, 35879, -1, 35880, 35881, 35882, -1, 35883, 35884, 35885, -1, 35886, 35887, 35888, -1, 35889, 35890, 35891, -1, 35892, 35893, 35894, -1, 35895, 35896, 35897, -1, 35898, 35899, 35900, -1, 35901, 35902, 35903, -1, 35904, 35905, 35906, -1, 35907, 35908, 35909, -1, 35910, 35911, 35912, -1, 35913, 35914, 35915, -1, 35916, 35917, 35918, -1, 35919, 35920, 35921, -1, 35922, 35923, 35924, -1, 35925, 35926, 35927, -1, 35928, 35929, 35930, -1, 35931, 35932, 35933, -1, 35934, 35935, 35936, -1, 35937, 35938, 35939, -1, 35940, 35941, 35942, -1, 35943, 35944, 35945, -1, 35946, 35947, 35948, -1, 35949, 35950, 35951, -1, 35952, 35953, 35954, -1, 35955, 35956, 35957, -1, 35958, 35959, 35960, -1, 35961, 35962, 35963, -1, 35964, 35965, 35966, -1, 35967, 35968, 35969, -1, 35970, 35971, 35972, -1, 35973, 35974, 35975, -1, 35976, 35977, 35978, -1, 35979, 35980, 35981, -1, 35982, 35983, 35984, -1, 35985, 35986, 35987, -1, 35988, 35989, 35990, -1, 35991, 35992, 35993, -1, 35994, 35995, 35996, -1, 35997, 35998, 35999, -1, 36000, 36001, 36002, -1, 36003, 36004, 36005, -1, 36006, 36007, 36008, -1, 36009, 36010, 36011, -1, 36012, 36013, 36014, -1, 36015, 36016, 36017, -1, 36018, 36019, 36020, -1, 36021, 36022, 36023, -1, 36024, 36025, 36026, -1, 36027, 36028, 36029, -1, 36030, 36031, 36032, -1, 36033, 36034, 36035, -1, 36036, 36037, 36038, -1, 36039, 36040, 36041, -1, 36042, 36043, 36044, -1, 36045, 36046, 36047, -1, 36048, 36049, 36050, -1, 36051, 36052, 36053, -1, 36054, 36055, 36056, -1, 36057, 36058, 36059, -1, 36060, 36061, 36062, -1, 36063, 36064, 36065, -1, 36066, 36067, 36068, -1, 36069, 36070, 36071, -1, 36072, 36073, 36074, -1, 36075, 36076, 36077, -1, 36078, 36079, 36080, -1, 36081, 36082, 36083, -1, 36084, 36085, 36086, -1, 36087, 36088, 36089, -1, 36090, 36091, 36092, -1, 36093, 36094, 36095, -1, 36096, 36097, 36098, -1, 36099, 36100, 36101, -1, 36102, 36103, 36104, -1, 36105, 36106, 36107, -1, 36108, 36109, 36110, -1, 36111, 36112, 36113, -1, 36114, 36115, 36116, -1, 36117, 36118, 36119, -1, 36120, 36121, 36122, -1, 36123, 36124, 36125, -1, 36126, 36127, 36128, -1, 36129, 36130, 36131, -1, 36132, 36133, 36134, -1, 36135, 36136, 36137, -1, 36138, 36139, 36140, -1, 36141, 36142, 36143, -1, 36144, 36145, 36146, -1, 36147, 36148, 36149, -1, 36150, 36151, 36152, -1, 36153, 36154, 36155, -1, 36156, 36157, 36158, -1, 36159, 36160, 36161, -1, 36162, 36163, 36164, -1, 36165, 36166, 36167, -1, 36168, 36169, 36170, -1, 36171, 36172, 36173, -1, 36174, 36175, 36176, -1, 36177, 36178, 36179, -1, 36180, 36181, 36182, -1, 36183, 36184, 36185, -1, 36186, 36187, 36188, -1, 36189, 36190, 36191, -1, 36192, 36193, 36194, -1, 36195, 36196, 36197, -1, 36198, 36199, 36200, -1, 36201, 36202, 36203, -1, 36204, 36205, 36206, -1, 36207, 36208, 36209, -1, 36210, 36211, 36212, -1, 36213, 36214, 36215, -1, 36216, 36217, 36218, -1, 36219, 36220, 36221, -1, 36222, 36223, 36224, -1, 36225, 36226, 36227, -1, 36228, 36229, 36230, -1, 36231, 36232, 36233, -1, 36234, 36235, 36236, -1, 36237, 36238, 36239, -1, 36240, 36241, 36242, -1, 36243, 36244, 36245, -1, 36246, 36247, 36248, -1, 36249, 36250, 36251, -1, 36252, 36253, 36254, -1, 36255, 36256, 36257, -1, 36258, 36259, 36260, -1, 36261, 36262, 36263, -1, 36264, 36265, 36266, -1, 36267, 36268, 36269, -1, 36270, 36271, 36272, -1, 36273, 36274, 36275, -1, 36276, 36277, 36278, -1, 36279, 36280, 36281, -1, 36282, 36283, 36284, -1, 36285, 36286, 36287, -1, 36288, 36289, 36290, -1, 36291, 36292, 36293, -1, 36294, 36295, 36296, -1, 36297, 36298, 36299, -1, 36300, 36301, 36302, -1, 36303, 36304, 36305, -1, 36306, 36307, 36308, -1, 36309, 36310, 36311, -1, 36312, 36313, 36314, -1, 36315, 36316, 36317, -1, 36318, 36319, 36320, -1, 36321, 36322, 36323, -1, 36324, 36325, 36326, -1, 36327, 36328, 36329, -1, 36330, 36331, 36332, -1, 36333, 36334, 36335, -1, 36336, 36337, 36338, -1, 36339, 36340, 36341, -1, 36342, 36343, 36344, -1, 36345, 36346, 36347, -1, 36348, 36349, 36350, -1, 36351, 36352, 36353, -1, 36354, 36355, 36356, -1, 36357, 36358, 36359, -1, 36360, 36361, 36362, -1, 36363, 36364, 36365, -1, 36366, 36367, 36368, -1, 36369, 36370, 36371, -1, 36372, 36373, 36374, -1, 36375, 36376, 36377, -1, 36378, 36379, 36380, -1, 36381, 36382, 36383, -1, 36384, 36385, 36386, -1, 36387, 36388, 36389, -1, 36390, 36391, 36392, -1, 36393, 36394, 36395, -1, 36396, 36397, 36398, -1, 36399, 36400, 36401, -1, 36402, 36403, 36404, -1, 36405, 36406, 36407, -1, 36408, 36409, 36410, -1, 36411, 36412, 36413, -1, 36414, 36415, 36416, -1, 36417, 36418, 36419, -1, 36420, 36421, 36422, -1, 36423, 36424, 36425, -1, 36426, 36427, 36428, -1, 36429, 36430, 36431, -1, 36432, 36433, 36434, -1, 36435, 36436, 36437, -1, 36438, 36439, 36440, -1, 36441, 36442, 36443, -1, 36444, 36445, 36446, -1, 36447, 36448, 36449, -1, 36450, 36451, 36452, -1, 36453, 36454, 36455, -1, 36456, 36457, 36458, -1, 36459, 36460, 36461, -1, 36462, 36463, 36464, -1, 36465, 36466, 36467, -1, 36468, 36469, 36470, -1, 36471, 36472, 36473, -1, 36474, 36475, 36476, -1, 36477, 36478, 36479, -1, 36480, 36481, 36482, -1, 36483, 36484, 36485, -1, 36486, 36487, 36488, -1, 36489, 36490, 36491, -1, 36492, 36493, 36494, -1, 36495, 36496, 36497, -1, 36498, 36499, 36500, -1, 36501, 36502, 36503, -1, 36504, 36505, 36506, -1, 36507, 36508, 36509, -1, 36510, 36511, 36512, -1, 36513, 36514, 36515, -1, 36516, 36517, 36518, -1, 36519, 36520, 36521, -1, 36522, 36523, 36524, -1, 36525, 36526, 36527, -1, 36528, 36529, 36530, -1, 36531, 36532, 36533, -1, 36534, 36535, 36536, -1, 36537, 36538, 36539, -1, 36540, 36541, 36542, -1, 36543, 36544, 36545, -1, 36546, 36547, 36548, -1, 36549, 36550, 36551, -1, 36552, 36553, 36554, -1, 36555, 36556, 36557, -1, 36558, 36559, 36560, -1, 36561, 36562, 36563, -1, 36564, 36565, 36566, -1, 36567, 36568, 36569, -1, 36570, 36571, 36572, -1, 36573, 36574, 36575, -1, 36576, 36577, 36578, -1, 36579, 36580, 36581, -1, 36582, 36583, 36584, -1, 36585, 36586, 36587, -1, 36588, 36589, 36590, -1, 36591, 36592, 36593, -1, 36594, 36595, 36596, -1, 36597, 36598, 36599, -1, 36600, 36601, 36602, -1, 36603, 36604, 36605, -1, 36606, 36607, 36608, -1, 36609, 36610, 36611, -1, 36612, 36613, 36614, -1, 36615, 36616, 36617, -1, 36618, 36619, 36620, -1, 36621, 36622, 36623, -1, 36624, 36625, 36626, -1, 36627, 36628, 36629, -1, 36630, 36631, 36632, -1, 36633, 36634, 36635, -1, 36636, 36637, 36638, -1, 36639, 36640, 36641, -1, 36642, 36643, 36644, -1, 36645, 36646, 36647, -1, 36648, 36649, 36650, -1, 36651, 36652, 36653, -1, 36654, 36655, 36656, -1, 36657, 36658, 36659, -1, 36660, 36661, 36662, -1, 36663, 36664, 36665, -1, 36666, 36667, 36668, -1, 36669, 36670, 36671, -1, 36672, 36673, 36674, -1, 36675, 36676, 36677, -1, 36678, 36679, 36680, -1, 36681, 36682, 36683, -1, 36684, 36685, 36686, -1, 36687, 36688, 36689, -1, 36690, 36691, 36692, -1, 36693, 36694, 36695, -1, 36696, 36697, 36698, -1, 36699, 36700, 36701, -1, 36702, 36703, 36704, -1, 36705, 36706, 36707, -1, 36708, 36709, 36710, -1, 36711, 36712, 36713, -1, 36714, 36715, 36716, -1, 36717, 36718, 36719, -1, 36720, 36721, 36722, -1, 36723, 36724, 36725, -1, 36726, 36727, 36728, -1, 36729, 36730, 36731, -1, 36732, 36733, 36734, -1, 36735, 36736, 36737, -1, 36738, 36739, 36740, -1, 36741, 36742, 36743, -1, 36744, 36745, 36746, -1, 36747, 36748, 36749, -1, 36750, 36751, 36752, -1, 36753, 36754, 36755, -1, 36756, 36757, 36758, -1, 36759, 36760, 36761, -1, 36762, 36763, 36764, -1, 36765, 36766, 36767, -1, 36768, 36769, 36770, -1, 36771, 36772, 36773, -1, 36774, 36775, 36776, -1, 36777, 36778, 36779, -1, 36780, 36781, 36782, -1, 36783, 36784, 36785, -1 };
    Text1m_5.coordIndex = m_coordIndex;
    Text1m_5.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    Text1m_5.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1, 5994, 5995, 5996, -1, 5997, 5998, 5999, -1, 6000, 6001, 6002, -1, 6003, 6004, 6005, -1, 6006, 6007, 6008, -1, 6009, 6010, 6011, -1, 6012, 6013, 6014, -1, 6015, 6016, 6017, -1, 6018, 6019, 6020, -1, 6021, 6022, 6023, -1, 6024, 6025, 6026, -1, 6027, 6028, 6029, -1, 6030, 6031, 6032, -1, 6033, 6034, 6035, -1, 6036, 6037, 6038, -1, 6039, 6040, 6041, -1, 6042, 6043, 6044, -1, 6045, 6046, 6047, -1, 6048, 6049, 6050, -1, 6051, 6052, 6053, -1, 6054, 6055, 6056, -1, 6057, 6058, 6059, -1, 6060, 6061, 6062, -1, 6063, 6064, 6065, -1, 6066, 6067, 6068, -1, 6069, 6070, 6071, -1, 6072, 6073, 6074, -1, 6075, 6076, 6077, -1, 6078, 6079, 6080, -1, 6081, 6082, 6083, -1, 6084, 6085, 6086, -1, 6087, 6088, 6089, -1, 6090, 6091, 6092, -1, 6093, 6094, 6095, -1, 6096, 6097, 6098, -1, 6099, 6100, 6101, -1, 6102, 6103, 6104, -1, 6105, 6106, 6107, -1, 6108, 6109, 6110, -1, 6111, 6112, 6113, -1, 6114, 6115, 6116, -1, 6117, 6118, 6119, -1, 6120, 6121, 6122, -1, 6123, 6124, 6125, -1, 6126, 6127, 6128, -1, 6129, 6130, 6131, -1, 6132, 6133, 6134, -1, 6135, 6136, 6137, -1, 6138, 6139, 6140, -1, 6141, 6142, 6143, -1, 6144, 6145, 6146, -1, 6147, 6148, 6149, -1, 6150, 6151, 6152, -1, 6153, 6154, 6155, -1, 6156, 6157, 6158, -1, 6159, 6160, 6161, -1, 6162, 6163, 6164, -1, 6165, 6166, 6167, -1, 6168, 6169, 6170, -1, 6171, 6172, 6173, -1, 6174, 6175, 6176, -1, 6177, 6178, 6179, -1, 6180, 6181, 6182, -1, 6183, 6184, 6185, -1, 6186, 6187, 6188, -1, 6189, 6190, 6191, -1, 6192, 6193, 6194, -1, 6195, 6196, 6197, -1, 6198, 6199, 6200, -1, 6201, 6202, 6203, -1, 6204, 6205, 6206, -1, 6207, 6208, 6209, -1, 6210, 6211, 6212, -1, 6213, 6214, 6215, -1, 6216, 6217, 6218, -1, 6219, 6220, 6221, -1, 6222, 6223, 6224, -1, 6225, 6226, 6227, -1, 6228, 6229, 6230, -1, 6231, 6232, 6233, -1, 6234, 6235, 6236, -1, 6237, 6238, 6239, -1, 6240, 6241, 6242, -1, 6243, 6244, 6245, -1, 6246, 6247, 6248, -1, 6249, 6250, 6251, -1, 6252, 6253, 6254, -1, 6255, 6256, 6257, -1, 6258, 6259, 6260, -1, 6261, 6262, 6263, -1, 6264, 6265, 6266, -1, 6267, 6268, 6269, -1, 6270, 6271, 6272, -1, 6273, 6274, 6275, -1, 6276, 6277, 6278, -1, 6279, 6280, 6281, -1, 6282, 6283, 6284, -1, 6285, 6286, 6287, -1, 6288, 6289, 6290, -1, 6291, 6292, 6293, -1, 6294, 6295, 6296, -1, 6297, 6298, 6299, -1, 6300, 6301, 6302, -1, 6303, 6304, 6305, -1, 6306, 6307, 6308, -1, 6309, 6310, 6311, -1, 6312, 6313, 6314, -1, 6315, 6316, 6317, -1, 6318, 6319, 6320, -1, 6321, 6322, 6323, -1, 6324, 6325, 6326, -1, 6327, 6328, 6329, -1, 6330, 6331, 6332, -1, 6333, 6334, 6335, -1, 6336, 6337, 6338, -1, 6339, 6340, 6341, -1, 6342, 6343, 6344, -1, 6345, 6346, 6347, -1, 6348, 6349, 6350, -1, 6351, 6352, 6353, -1, 6354, 6355, 6356, -1, 6357, 6358, 6359, -1, 6360, 6361, 6362, -1, 6363, 6364, 6365, -1, 6366, 6367, 6368, -1, 6369, 6370, 6371, -1, 6372, 6373, 6374, -1, 6375, 6376, 6377, -1, 6378, 6379, 6380, -1, 6381, 6382, 6383, -1, 6384, 6385, 6386, -1, 6387, 6388, 6389, -1, 6390, 6391, 6392, -1, 6393, 6394, 6395, -1, 6396, 6397, 6398, -1, 6399, 6400, 6401, -1, 6402, 6403, 6404, -1, 6405, 6406, 6407, -1, 6408, 6409, 6410, -1, 6411, 6412, 6413, -1, 6414, 6415, 6416, -1, 6417, 6418, 6419, -1, 6420, 6421, 6422, -1, 6423, 6424, 6425, -1, 6426, 6427, 6428, -1, 6429, 6430, 6431, -1, 6432, 6433, 6434, -1, 6435, 6436, 6437, -1, 6438, 6439, 6440, -1, 6441, 6442, 6443, -1, 6444, 6445, 6446, -1, 6447, 6448, 6449, -1, 6450, 6451, 6452, -1, 6453, 6454, 6455, -1, 6456, 6457, 6458, -1, 6459, 6460, 6461, -1, 6462, 6463, 6464, -1, 6465, 6466, 6467, -1, 6468, 6469, 6470, -1, 6471, 6472, 6473, -1, 6474, 6475, 6476, -1, 6477, 6478, 6479, -1, 6480, 6481, 6482, -1, 6483, 6484, 6485, -1, 6486, 6487, 6488, -1, 6489, 6490, 6491, -1, 6492, 6493, 6494, -1, 6495, 6496, 6497, -1, 6498, 6499, 6500, -1, 6501, 6502, 6503, -1, 6504, 6505, 6506, -1, 6507, 6508, 6509, -1, 6510, 6511, 6512, -1, 6513, 6514, 6515, -1, 6516, 6517, 6518, -1, 6519, 6520, 6521, -1, 6522, 6523, 6524, -1, 6525, 6526, 6527, -1, 6528, 6529, 6530, -1, 6531, 6532, 6533, -1, 6534, 6535, 6536, -1, 6537, 6538, 6539, -1, 6540, 6541, 6542, -1, 6543, 6544, 6545, -1, 6546, 6547, 6548, -1, 6549, 6550, 6551, -1, 6552, 6553, 6554, -1, 6555, 6556, 6557, -1, 6558, 6559, 6560, -1, 6561, 6562, 6563, -1, 6564, 6565, 6566, -1, 6567, 6568, 6569, -1, 6570, 6571, 6572, -1, 6573, 6574, 6575, -1, 6576, 6577, 6578, -1, 6579, 6580, 6581, -1, 6582, 6583, 6584, -1, 6585, 6586, 6587, -1, 6588, 6589, 6590, -1, 6591, 6592, 6593, -1, 6594, 6595, 6596, -1, 6597, 6598, 6599, -1, 6600, 6601, 6602, -1, 6603, 6604, 6605, -1, 6606, 6607, 6608, -1, 6609, 6610, 6611, -1, 6612, 6613, 6614, -1, 6615, 6616, 6617, -1, 6618, 6619, 6620, -1, 6621, 6622, 6623, -1, 6624, 6625, 6626, -1, 6627, 6628, 6629, -1, 6630, 6631, 6632, -1, 6633, 6634, 6635, -1, 6636, 6637, 6638, -1, 6639, 6640, 6641, -1, 6642, 6643, 6644, -1, 6645, 6646, 6647, -1, 6648, 6649, 6650, -1, 6651, 6652, 6653, -1, 6654, 6655, 6656, -1, 6657, 6658, 6659, -1, 6660, 6661, 6662, -1, 6663, 6664, 6665, -1, 6666, 6667, 6668, -1, 6669, 6670, 6671, -1, 6672, 6673, 6674, -1, 6675, 6676, 6677, -1, 6678, 6679, 6680, -1, 6681, 6682, 6683, -1, 6684, 6685, 6686, -1, 6687, 6688, 6689, -1, 6690, 6691, 6692, -1, 6693, 6694, 6695, -1, 6696, 6697, 6698, -1, 6699, 6700, 6701, -1, 6702, 6703, 6704, -1, 6705, 6706, 6707, -1, 6708, 6709, 6710, -1, 6711, 6712, 6713, -1, 6714, 6715, 6716, -1, 6717, 6718, 6719, -1, 6720, 6721, 6722, -1, 6723, 6724, 6725, -1, 6726, 6727, 6728, -1, 6729, 6730, 6731, -1, 6732, 6733, 6734, -1, 6735, 6736, 6737, -1, 6738, 6739, 6740, -1, 6741, 6742, 6743, -1, 6744, 6745, 6746, -1, 6747, 6748, 6749, -1, 6750, 6751, 6752, -1, 6753, 6754, 6755, -1, 6756, 6757, 6758, -1, 6759, 6760, 6761, -1, 6762, 6763, 6764, -1, 6765, 6766, 6767, -1, 6768, 6769, 6770, -1, 6771, 6772, 6773, -1, 6774, 6775, 6776, -1, 6777, 6778, 6779, -1, 6780, 6781, 6782, -1, 6783, 6784, 6785, -1, 6786, 6787, 6788, -1, 6789, 6790, 6791, -1, 6792, 6793, 6794, -1, 6795, 6796, 6797, -1, 6798, 6799, 6800, -1, 6801, 6802, 6803, -1, 6804, 6805, 6806, -1, 6807, 6808, 6809, -1, 6810, 6811, 6812, -1, 6813, 6814, 6815, -1, 6816, 6817, 6818, -1, 6819, 6820, 6821, -1, 6822, 6823, 6824, -1, 6825, 6826, 6827, -1, 6828, 6829, 6830, -1, 6831, 6832, 6833, -1, 6834, 6835, 6836, -1, 6837, 6838, 6839, -1, 6840, 6841, 6842, -1, 6843, 6844, 6845, -1, 6846, 6847, 6848, -1, 6849, 6850, 6851, -1, 6852, 6853, 6854, -1, 6855, 6856, 6857, -1, 6858, 6859, 6860, -1, 6861, 6862, 6863, -1, 6864, 6865, 6866, -1, 6867, 6868, 6869, -1, 6870, 6871, 6872, -1, 6873, 6874, 6875, -1, 6876, 6877, 6878, -1, 6879, 6880, 6881, -1, 6882, 6883, 6884, -1, 6885, 6886, 6887, -1, 6888, 6889, 6890, -1, 6891, 6892, 6893, -1, 6894, 6895, 6896, -1, 6897, 6898, 6899, -1, 6900, 6901, 6902, -1, 6903, 6904, 6905, -1, 6906, 6907, 6908, -1, 6909, 6910, 6911, -1, 6912, 6913, 6914, -1, 6915, 6916, 6917, -1, 6918, 6919, 6920, -1, 6921, 6922, 6923, -1, 6924, 6925, 6926, -1, 6927, 6928, 6929, -1, 6930, 6931, 6932, -1, 6933, 6934, 6935, -1, 6936, 6937, 6938, -1, 6939, 6940, 6941, -1, 6942, 6943, 6944, -1, 6945, 6946, 6947, -1, 6948, 6949, 6950, -1, 6951, 6952, 6953, -1, 6954, 6955, 6956, -1, 6957, 6958, 6959, -1, 6960, 6961, 6962, -1, 6963, 6964, 6965, -1, 6966, 6967, 6968, -1, 6969, 6970, 6971, -1, 6972, 6973, 6974, -1, 6975, 6976, 6977, -1, 6978, 6979, 6980, -1, 6981, 6982, 6983, -1, 6984, 6985, 6986, -1, 6987, 6988, 6989, -1, 6990, 6991, 6992, -1, 6993, 6994, 6995, -1, 6996, 6997, 6998, -1, 6999, 7000, 7001, -1, 7002, 7003, 7004, -1, 7005, 7006, 7007, -1, 7008, 7009, 7010, -1, 7011, 7012, 7013, -1, 7014, 7015, 7016, -1, 7017, 7018, 7019, -1, 7020, 7021, 7022, -1, 7023, 7024, 7025, -1, 7026, 7027, 7028, -1, 7029, 7030, 7031, -1, 7032, 7033, 7034, -1, 7035, 7036, 7037, -1, 7038, 7039, 7040, -1, 7041, 7042, 7043, -1, 7044, 7045, 7046, -1, 7047, 7048, 7049, -1, 7050, 7051, 7052, -1, 7053, 7054, 7055, -1, 7056, 7057, 7058, -1, 7059, 7060, 7061, -1, 7062, 7063, 7064, -1, 7065, 7066, 7067, -1, 7068, 7069, 7070, -1, 7071, 7072, 7073, -1, 7074, 7075, 7076, -1, 7077, 7078, 7079, -1, 7080, 7081, 7082, -1, 7083, 7084, 7085, -1, 7086, 7087, 7088, -1, 7089, 7090, 7091, -1, 7092, 7093, 7094, -1, 7095, 7096, 7097, -1, 7098, 7099, 7100, -1, 7101, 7102, 7103, -1, 7104, 7105, 7106, -1, 7107, 7108, 7109, -1, 7110, 7111, 7112, -1, 7113, 7114, 7115, -1, 7116, 7117, 7118, -1, 7119, 7120, 7121, -1, 7122, 7123, 7124, -1, 7125, 7126, 7127, -1, 7128, 7129, 7130, -1, 7131, 7132, 7133, -1, 7134, 7135, 7136, -1, 7137, 7138, 7139, -1, 7140, 7141, 7142, -1, 7143, 7144, 7145, -1, 7146, 7147, 7148, -1, 7149, 7150, 7151, -1, 7152, 7153, 7154, -1, 7155, 7156, 7157, -1, 7158, 7159, 7160, -1, 7161, 7162, 7163, -1, 7164, 7165, 7166, -1, 7167, 7168, 7169, -1, 7170, 7171, 7172, -1, 7173, 7174, 7175, -1, 7176, 7177, 7178, -1, 7179, 7180, 7181, -1, 7182, 7183, 7184, -1, 7185, 7186, 7187, -1, 7188, 7189, 7190, -1, 7191, 7192, 7193, -1, 7194, 7195, 7196, -1, 7197, 7198, 7199, -1, 7200, 7201, 7202, -1, 7203, 7204, 7205, -1, 7206, 7207, 7208, -1, 7209, 7210, 7211, -1, 7212, 7213, 7214, -1, 7215, 7216, 7217, -1, 7218, 7219, 7220, -1, 7221, 7222, 7223, -1, 7224, 7225, 7226, -1, 7227, 7228, 7229, -1, 7230, 7231, 7232, -1, 7233, 7234, 7235, -1, 7236, 7237, 7238, -1, 7239, 7240, 7241, -1, 7242, 7243, 7244, -1, 7245, 7246, 7247, -1, 7248, 7249, 7250, -1, 7251, 7252, 7253, -1, 7254, 7255, 7256, -1, 7257, 7258, 7259, -1, 7260, 7261, 7262, -1, 7263, 7264, 7265, -1, 7266, 7267, 7268, -1, 7269, 7270, 7271, -1, 7272, 7273, 7274, -1, 7275, 7276, 7277, -1, 7278, 7279, 7280, -1, 7281, 7282, 7283, -1, 7284, 7285, 7286, -1, 7287, 7288, 7289, -1, 7290, 7291, 7292, -1, 7293, 7294, 7295, -1, 7296, 7297, 7298, -1, 7299, 7300, 7301, -1, 7302, 7303, 7304, -1, 7305, 7306, 7307, -1, 7308, 7309, 7310, -1, 7311, 7312, 7313, -1, 7314, 7315, 7316, -1, 7317, 7318, 7319, -1, 7320, 7321, 7322, -1, 7323, 7324, 7325, -1, 7326, 7327, 7328, -1, 7329, 7330, 7331, -1, 7332, 7333, 7334, -1, 7335, 7336, 7337, -1, 7338, 7339, 7340, -1, 7341, 7342, 7343, -1, 7344, 7345, 7346, -1, 7347, 7348, 7349, -1, 7350, 7351, 7352, -1, 7353, 7354, 7355, -1, 7356, 7357, 7358, -1, 7359, 7360, 7361, -1, 7362, 7363, 7364, -1, 7365, 7366, 7367, -1, 7368, 7369, 7370, -1, 7371, 7372, 7373, -1, 7374, 7375, 7376, -1, 7377, 7378, 7379, -1, 7380, 7381, 7382, -1, 7383, 7384, 7385, -1, 7386, 7387, 7388, -1, 7389, 7390, 7391, -1, 7392, 7393, 7394, -1, 7395, 7396, 7397, -1, 7398, 7399, 7400, -1, 7401, 7402, 7403, -1, 7404, 7405, 7406, -1, 7407, 7408, 7409, -1, 7410, 7411, 7412, -1, 7413, 7414, 7415, -1, 7416, 7417, 7418, -1, 7419, 7420, 7421, -1, 7422, 7423, 7424, -1, 7425, 7426, 7427, -1, 7428, 7429, 7430, -1, 7431, 7432, 7433, -1, 7434, 7435, 7436, -1, 7437, 7438, 7439, -1, 7440, 7441, 7442, -1, 7443, 7444, 7445, -1, 7446, 7447, 7448, -1, 7449, 7450, 7451, -1, 7452, 7453, 7454, -1, 7455, 7456, 7457, -1, 7458, 7459, 7460, -1, 7461, 7462, 7463, -1, 7464, 7465, 7466, -1, 7467, 7468, 7469, -1, 7470, 7471, 7472, -1, 7473, 7474, 7475, -1, 7476, 7477, 7478, -1, 7479, 7480, 7481, -1, 7482, 7483, 7484, -1, 7485, 7486, 7487, -1, 7488, 7489, 7490, -1, 7491, 7492, 7493, -1, 7494, 7495, 7496, -1, 7497, 7498, 7499, -1, 7500, 7501, 7502, -1, 7503, 7504, 7505, -1, 7506, 7507, 7508, -1, 7509, 7510, 7511, -1, 7512, 7513, 7514, -1, 7515, 7516, 7517, -1, 7518, 7519, 7520, -1, 7521, 7522, 7523, -1, 7524, 7525, 7526, -1, 7527, 7528, 7529, -1, 7530, 7531, 7532, -1, 7533, 7534, 7535, -1, 7536, 7537, 7538, -1, 7539, 7540, 7541, -1, 7542, 7543, 7544, -1, 7545, 7546, 7547, -1, 7548, 7549, 7550, -1, 7551, 7552, 7553, -1, 7554, 7555, 7556, -1, 7557, 7558, 7559, -1, 7560, 7561, 7562, -1, 7563, 7564, 7565, -1, 7566, 7567, 7568, -1, 7569, 7570, 7571, -1, 7572, 7573, 7574, -1, 7575, 7576, 7577, -1, 7578, 7579, 7580, -1, 7581, 7582, 7583, -1, 7584, 7585, 7586, -1, 7587, 7588, 7589, -1, 7590, 7591, 7592, -1, 7593, 7594, 7595, -1, 7596, 7597, 7598, -1, 7599, 7600, 7601, -1, 7602, 7603, 7604, -1, 7605, 7606, 7607, -1, 7608, 7609, 7610, -1, 7611, 7612, 7613, -1, 7614, 7615, 7616, -1, 7617, 7618, 7619, -1, 7620, 7621, 7622, -1, 7623, 7624, 7625, -1, 7626, 7627, 7628, -1, 7629, 7630, 7631, -1, 7632, 7633, 7634, -1, 7635, 7636, 7637, -1, 7638, 7639, 7640, -1, 7641, 7642, 7643, -1, 7644, 7645, 7646, -1, 7647, 7648, 7649, -1, 7650, 7651, 7652, -1, 7653, 7654, 7655, -1, 7656, 7657, 7658, -1, 7659, 7660, 7661, -1, 7662, 7663, 7664, -1, 7665, 7666, 7667, -1, 7668, 7669, 7670, -1, 7671, 7672, 7673, -1, 7674, 7675, 7676, -1, 7677, 7678, 7679, -1, 7680, 7681, 7682, -1, 7683, 7684, 7685, -1, 7686, 7687, 7688, -1, 7689, 7690, 7691, -1, 7692, 7693, 7694, -1, 7695, 7696, 7697, -1, 7698, 7699, 7700, -1, 7701, 7702, 7703, -1, 7704, 7705, 7706, -1, 7707, 7708, 7709, -1, 7710, 7711, 7712, -1, 7713, 7714, 7715, -1, 7716, 7717, 7718, -1, 7719, 7720, 7721, -1, 7722, 7723, 7724, -1, 7725, 7726, 7727, -1, 7728, 7729, 7730, -1, 7731, 7732, 7733, -1, 7734, 7735, 7736, -1, 7737, 7738, 7739, -1, 7740, 7741, 7742, -1, 7743, 7744, 7745, -1, 7746, 7747, 7748, -1, 7749, 7750, 7751, -1, 7752, 7753, 7754, -1, 7755, 7756, 7757, -1, 7758, 7759, 7760, -1, 7761, 7762, 7763, -1, 7764, 7765, 7766, -1, 7767, 7768, 7769, -1, 7770, 7771, 7772, -1, 7773, 7774, 7775, -1, 7776, 7777, 7778, -1, 7779, 7780, 7781, -1, 7782, 7783, 7784, -1, 7785, 7786, 7787, -1, 7788, 7789, 7790, -1, 7791, 7792, 7793, -1, 7794, 7795, 7796, -1, 7797, 7798, 7799, -1, 7800, 7801, 7802, -1, 7803, 7804, 7805, -1, 7806, 7807, 7808, -1, 7809, 7810, 7811, -1, 7812, 7813, 7814, -1, 7815, 7816, 7817, -1, 7818, 7819, 7820, -1, 7821, 7822, 7823, -1, 7824, 7825, 7826, -1, 7827, 7828, 7829, -1, 7830, 7831, 7832, -1, 7833, 7834, 7835, -1, 7836, 7837, 7838, -1, 7839, 7840, 7841, -1, 7842, 7843, 7844, -1, 7845, 7846, 7847, -1, 7848, 7849, 7850, -1, 7851, 7852, 7853, -1, 7854, 7855, 7856, -1, 7857, 7858, 7859, -1, 7860, 7861, 7862, -1, 7863, 7864, 7865, -1, 7866, 7867, 7868, -1, 7869, 7870, 7871, -1, 7872, 7873, 7874, -1, 7875, 7876, 7877, -1, 7878, 7879, 7880, -1, 7881, 7882, 7883, -1, 7884, 7885, 7886, -1, 7887, 7888, 7889, -1, 7890, 7891, 7892, -1, 7893, 7894, 7895, -1, 7896, 7897, 7898, -1, 7899, 7900, 7901, -1, 7902, 7903, 7904, -1, 7905, 7906, 7907, -1, 7908, 7909, 7910, -1, 7911, 7912, 7913, -1, 7914, 7915, 7916, -1, 7917, 7918, 7919, -1, 7920, 7921, 7922, -1, 7923, 7924, 7925, -1, 7926, 7927, 7928, -1, 7929, 7930, 7931, -1, 7932, 7933, 7934, -1, 7935, 7936, 7937, -1, 7938, 7939, 7940, -1, 7941, 7942, 7943, -1, 7944, 7945, 7946, -1, 7947, 7948, 7949, -1, 7950, 7951, 7952, -1, 7953, 7954, 7955, -1, 7956, 7957, 7958, -1, 7959, 7960, 7961, -1, 7962, 7963, 7964, -1, 7965, 7966, 7967, -1, 7968, 7969, 7970, -1, 7971, 7972, 7973, -1, 7974, 7975, 7976, -1, 7977, 7978, 7979, -1, 7980, 7981, 7982, -1, 7983, 7984, 7985, -1, 7986, 7987, 7988, -1, 7989, 7990, 7991, -1, 7992, 7993, 7994, -1, 7995, 7996, 7997, -1, 7998, 7999, 8000, -1, 8001, 8002, 8003, -1, 8004, 8005, 8006, -1, 8007, 8008, 8009, -1, 8010, 8011, 8012, -1, 8013, 8014, 8015, -1, 8016, 8017, 8018, -1, 8019, 8020, 8021, -1, 8022, 8023, 8024, -1, 8025, 8026, 8027, -1, 8028, 8029, 8030, -1, 8031, 8032, 8033, -1, 8034, 8035, 8036, -1, 8037, 8038, 8039, -1, 8040, 8041, 8042, -1, 8043, 8044, 8045, -1, 8046, 8047, 8048, -1, 8049, 8050, 8051, -1, 8052, 8053, 8054, -1, 8055, 8056, 8057, -1, 8058, 8059, 8060, -1, 8061, 8062, 8063, -1, 8064, 8065, 8066, -1, 8067, 8068, 8069, -1, 8070, 8071, 8072, -1, 8073, 8074, 8075, -1, 8076, 8077, 8078, -1, 8079, 8080, 8081, -1, 8082, 8083, 8084, -1, 8085, 8086, 8087, -1, 8088, 8089, 8090, -1, 8091, 8092, 8093, -1, 8094, 8095, 8096, -1, 8097, 8098, 8099, -1, 8100, 8101, 8102, -1, 8103, 8104, 8105, -1, 8106, 8107, 8108, -1, 8109, 8110, 8111, -1, 8112, 8113, 8114, -1, 8115, 8116, 8117, -1, 8118, 8119, 8120, -1, 8121, 8122, 8123, -1, 8124, 8125, 8126, -1, 8127, 8128, 8129, -1, 8130, 8131, 8132, -1, 8133, 8134, 8135, -1, 8136, 8137, 8138, -1, 8139, 8140, 8141, -1, 8142, 8143, 8144, -1, 8145, 8146, 8147, -1, 8148, 8149, 8150, -1, 8151, 8152, 8153, -1, 8154, 8155, 8156, -1, 8157, 8158, 8159, -1, 8160, 8161, 8162, -1, 8163, 8164, 8165, -1, 8166, 8167, 8168, -1, 8169, 8170, 8171, -1, 8172, 8173, 8174, -1, 8175, 8176, 8177, -1, 8178, 8179, 8180, -1, 8181, 8182, 8183, -1, 8184, 8185, 8186, -1, 8187, 8188, 8189, -1, 8190, 8191, 8192, -1, 8193, 8194, 8195, -1, 8196, 8197, 8198, -1, 8199, 8200, 8201, -1, 8202, 8203, 8204, -1, 8205, 8206, 8207, -1, 8208, 8209, 8210, -1, 8211, 8212, 8213, -1, 8214, 8215, 8216, -1, 8217, 8218, 8219, -1, 8220, 8221, 8222, -1, 8223, 8224, 8225, -1, 8226, 8227, 8228, -1, 8229, 8230, 8231, -1, 8232, 8233, 8234, -1, 8235, 8236, 8237, -1, 8238, 8239, 8240, -1, 8241, 8242, 8243, -1, 8244, 8245, 8246, -1, 8247, 8248, 8249, -1, 8250, 8251, 8252, -1, 8253, 8254, 8255, -1, 8256, 8257, 8258, -1, 8259, 8260, 8261, -1, 8262, 8263, 8264, -1, 8265, 8266, 8267, -1, 8268, 8269, 8270, -1, 8271, 8272, 8273, -1, 8274, 8275, 8276, -1, 8277, 8278, 8279, -1, 8280, 8281, 8282, -1, 8283, 8284, 8285, -1, 8286, 8287, 8288, -1, 8289, 8290, 8291, -1, 8292, 8293, 8294, -1, 8295, 8296, 8297, -1, 8298, 8299, 8300, -1, 8301, 8302, 8303, -1, 8304, 8305, 8306, -1, 8307, 8308, 8309, -1, 8310, 8311, 8312, -1, 8313, 8314, 8315, -1, 8316, 8317, 8318, -1, 8319, 8320, 8321, -1, 8322, 8323, 8324, -1, 8325, 8326, 8327, -1, 8328, 8329, 8330, -1, 8331, 8332, 8333, -1, 8334, 8335, 8336, -1, 8337, 8338, 8339, -1, 8340, 8341, 8342, -1, 8343, 8344, 8345, -1, 8346, 8347, 8348, -1, 8349, 8350, 8351, -1, 8352, 8353, 8354, -1, 8355, 8356, 8357, -1, 8358, 8359, 8360, -1, 8361, 8362, 8363, -1, 8364, 8365, 8366, -1, 8367, 8368, 8369, -1, 8370, 8371, 8372, -1, 8373, 8374, 8375, -1, 8376, 8377, 8378, -1, 8379, 8380, 8381, -1, 8382, 8383, 8384, -1, 8385, 8386, 8387, -1, 8388, 8389, 8390, -1, 8391, 8392, 8393, -1, 8394, 8395, 8396, -1, 8397, 8398, 8399, -1, 8400, 8401, 8402, -1, 8403, 8404, 8405, -1, 8406, 8407, 8408, -1, 8409, 8410, 8411, -1, 8412, 8413, 8414, -1, 8415, 8416, 8417, -1, 8418, 8419, 8420, -1, 8421, 8422, 8423, -1, 8424, 8425, 8426, -1, 8427, 8428, 8429, -1, 8430, 8431, 8432, -1, 8433, 8434, 8435, -1, 8436, 8437, 8438, -1, 8439, 8440, 8441, -1, 8442, 8443, 8444, -1, 8445, 8446, 8447, -1, 8448, 8449, 8450, -1, 8451, 8452, 8453, -1, 8454, 8455, 8456, -1, 8457, 8458, 8459, -1, 8460, 8461, 8462, -1, 8463, 8464, 8465, -1, 8466, 8467, 8468, -1, 8469, 8470, 8471, -1, 8472, 8473, 8474, -1, 8475, 8476, 8477, -1, 8478, 8479, 8480, -1, 8481, 8482, 8483, -1, 8484, 8485, 8486, -1, 8487, 8488, 8489, -1, 8490, 8491, 8492, -1, 8493, 8494, 8495, -1, 8496, 8497, 8498, -1, 8499, 8500, 8501, -1, 8502, 8503, 8504, -1, 8505, 8506, 8507, -1, 8508, 8509, 8510, -1, 8511, 8512, 8513, -1, 8514, 8515, 8516, -1, 8517, 8518, 8519, -1, 8520, 8521, 8522, -1, 8523, 8524, 8525, -1, 8526, 8527, 8528, -1, 8529, 8530, 8531, -1, 8532, 8533, 8534, -1, 8535, 8536, 8537, -1, 8538, 8539, 8540, -1, 8541, 8542, 8543, -1, 8544, 8545, 8546, -1, 8547, 8548, 8549, -1, 8550, 8551, 8552, -1, 8553, 8554, 8555, -1, 8556, 8557, 8558, -1, 8559, 8560, 8561, -1, 8562, 8563, 8564, -1, 8565, 8566, 8567, -1, 8568, 8569, 8570, -1, 8571, 8572, 8573, -1, 8574, 8575, 8576, -1, 8577, 8578, 8579, -1, 8580, 8581, 8582, -1, 8583, 8584, 8585, -1, 8586, 8587, 8588, -1, 8589, 8590, 8591, -1, 8592, 8593, 8594, -1, 8595, 8596, 8597, -1, 8598, 8599, 8600, -1, 8601, 8602, 8603, -1, 8604, 8605, 8606, -1, 8607, 8608, 8609, -1, 8610, 8611, 8612, -1, 8613, 8614, 8615, -1, 8616, 8617, 8618, -1, 8619, 8620, 8621, -1, 8622, 8623, 8624, -1, 8625, 8626, 8627, -1, 8628, 8629, 8630, -1, 8631, 8632, 8633, -1, 8634, 8635, 8636, -1, 8637, 8638, 8639, -1, 8640, 8641, 8642, -1, 8643, 8644, 8645, -1, 8646, 8647, 8648, -1, 8649, 8650, 8651, -1, 8652, 8653, 8654, -1, 8655, 8656, 8657, -1, 8658, 8659, 8660, -1, 8661, 8662, 8663, -1, 8664, 8665, 8666, -1, 8667, 8668, 8669, -1, 8670, 8671, 8672, -1, 8673, 8674, 8675, -1, 8676, 8677, 8678, -1, 8679, 8680, 8681, -1, 8682, 8683, 8684, -1, 8685, 8686, 8687, -1, 8688, 8689, 8690, -1, 8691, 8692, 8693, -1, 8694, 8695, 8696, -1, 8697, 8698, 8699, -1, 8700, 8701, 8702, -1, 8703, 8704, 8705, -1, 8706, 8707, 8708, -1, 8709, 8710, 8711, -1, 8712, 8713, 8714, -1, 8715, 8716, 8717, -1, 8718, 8719, 8720, -1, 8721, 8722, 8723, -1, 8724, 8725, 8726, -1, 8727, 8728, 8729, -1, 8730, 8731, 8732, -1, 8733, 8734, 8735, -1, 8736, 8737, 8738, -1, 8739, 8740, 8741, -1, 8742, 8743, 8744, -1, 8745, 8746, 8747, -1, 8748, 8749, 8750, -1, 8751, 8752, 8753, -1, 8754, 8755, 8756, -1, 8757, 8758, 8759, -1, 8760, 8761, 8762, -1, 8763, 8764, 8765, -1, 8766, 8767, 8768, -1, 8769, 8770, 8771, -1, 8772, 8773, 8774, -1, 8775, 8776, 8777, -1, 8778, 8779, 8780, -1, 8781, 8782, 8783, -1, 8784, 8785, 8786, -1, 8787, 8788, 8789, -1, 8790, 8791, 8792, -1, 8793, 8794, 8795, -1, 8796, 8797, 8798, -1, 8799, 8800, 8801, -1, 8802, 8803, 8804, -1, 8805, 8806, 8807, -1, 8808, 8809, 8810, -1, 8811, 8812, 8813, -1, 8814, 8815, 8816, -1, 8817, 8818, 8819, -1, 8820, 8821, 8822, -1, 8823, 8824, 8825, -1, 8826, 8827, 8828, -1, 8829, 8830, 8831, -1, 8832, 8833, 8834, -1, 8835, 8836, 8837, -1, 8838, 8839, 8840, -1, 8841, 8842, 8843, -1, 8844, 8845, 8846, -1, 8847, 8848, 8849, -1, 8850, 8851, 8852, -1, 8853, 8854, 8855, -1, 8856, 8857, 8858, -1, 8859, 8860, 8861, -1, 8862, 8863, 8864, -1, 8865, 8866, 8867, -1, 8868, 8869, 8870, -1, 8871, 8872, 8873, -1, 8874, 8875, 8876, -1, 8877, 8878, 8879, -1, 8880, 8881, 8882, -1, 8883, 8884, 8885, -1, 8886, 8887, 8888, -1, 8889, 8890, 8891, -1, 8892, 8893, 8894, -1, 8895, 8896, 8897, -1, 8898, 8899, 8900, -1, 8901, 8902, 8903, -1, 8904, 8905, 8906, -1, 8907, 8908, 8909, -1, 8910, 8911, 8912, -1, 8913, 8914, 8915, -1, 8916, 8917, 8918, -1, 8919, 8920, 8921, -1, 8922, 8923, 8924, -1, 8925, 8926, 8927, -1, 8928, 8929, 8930, -1, 8931, 8932, 8933, -1, 8934, 8935, 8936, -1, 8937, 8938, 8939, -1, 8940, 8941, 8942, -1, 8943, 8944, 8945, -1, 8946, 8947, 8948, -1, 8949, 8950, 8951, -1, 8952, 8953, 8954, -1, 8955, 8956, 8957, -1, 8958, 8959, 8960, -1, 8961, 8962, 8963, -1, 8964, 8965, 8966, -1, 8967, 8968, 8969, -1, 8970, 8971, 8972, -1, 8973, 8974, 8975, -1, 8976, 8977, 8978, -1, 8979, 8980, 8981, -1, 8982, 8983, 8984, -1, 8985, 8986, 8987, -1, 8988, 8989, 8990, -1, 8991, 8992, 8993, -1, 8994, 8995, 8996, -1, 8997, 8998, 8999, -1, 9000, 9001, 9002, -1, 9003, 9004, 9005, -1, 9006, 9007, 9008, -1, 9009, 9010, 9011, -1, 9012, 9013, 9014, -1, 9015, 9016, 9017, -1, 9018, 9019, 9020, -1, 9021, 9022, 9023, -1, 9024, 9025, 9026, -1, 9027, 9028, 9029, -1, 9030, 9031, 9032, -1, 9033, 9034, 9035, -1, 9036, 9037, 9038, -1, 9039, 9040, 9041, -1, 9042, 9043, 9044, -1, 9045, 9046, 9047, -1, 9048, 9049, 9050, -1, 9051, 9052, 9053, -1, 9054, 9055, 9056, -1, 9057, 9058, 9059, -1, 9060, 9061, 9062, -1, 9063, 9064, 9065, -1, 9066, 9067, 9068, -1, 9069, 9070, 9071, -1, 9072, 9073, 9074, -1, 9075, 9076, 9077, -1, 9078, 9079, 9080, -1, 9081, 9082, 9083, -1, 9084, 9085, 9086, -1, 9087, 9088, 9089, -1, 9090, 9091, 9092, -1, 9093, 9094, 9095, -1, 9096, 9097, 9098, -1, 9099, 9100, 9101, -1, 9102, 9103, 9104, -1, 9105, 9106, 9107, -1, 9108, 9109, 9110, -1, 9111, 9112, 9113, -1, 9114, 9115, 9116, -1, 9117, 9118, 9119, -1, 9120, 9121, 9122, -1, 9123, 9124, 9125, -1, 9126, 9127, 9128, -1, 9129, 9130, 9131, -1, 9132, 9133, 9134, -1, 9135, 9136, 9137, -1, 9138, 9139, 9140, -1, 9141, 9142, 9143, -1, 9144, 9145, 9146, -1, 9147, 9148, 9149, -1, 9150, 9151, 9152, -1, 9153, 9154, 9155, -1, 9156, 9157, 9158, -1, 9159, 9160, 9161, -1, 9162, 9163, 9164, -1, 9165, 9166, 9167, -1, 9168, 9169, 9170, -1, 9171, 9172, 9173, -1, 9174, 9175, 9176, -1, 9177, 9178, 9179, -1, 9180, 9181, 9182, -1, 9183, 9184, 9185, -1, 9186, 9187, 9188, -1, 9189, 9190, 9191, -1, 9192, 9193, 9194, -1, 9195, 9196, 9197, -1, 9198, 9199, 9200, -1, 9201, 9202, 9203, -1, 9204, 9205, 9206, -1, 9207, 9208, 9209, -1, 9210, 9211, 9212, -1, 9213, 9214, 9215, -1, 9216, 9217, 9218, -1, 9219, 9220, 9221, -1, 9222, 9223, 9224, -1, 9225, 9226, 9227, -1, 9228, 9229, 9230, -1, 9231, 9232, 9233, -1, 9234, 9235, 9236, -1, 9237, 9238, 9239, -1, 9240, 9241, 9242, -1, 9243, 9244, 9245, -1, 9246, 9247, 9248, -1, 9249, 9250, 9251, -1, 9252, 9253, 9254, -1, 9255, 9256, 9257, -1, 9258, 9259, 9260, -1, 9261, 9262, 9263, -1, 9264, 9265, 9266, -1, 9267, 9268, 9269, -1, 9270, 9271, 9272, -1, 9273, 9274, 9275, -1, 9276, 9277, 9278, -1, 9279, 9280, 9281, -1, 9282, 9283, 9284, -1, 9285, 9286, 9287, -1, 9288, 9289, 9290, -1, 9291, 9292, 9293, -1, 9294, 9295, 9296, -1, 9297, 9298, 9299, -1, 9300, 9301, 9302, -1, 9303, 9304, 9305, -1, 9306, 9307, 9308, -1, 9309, 9310, 9311, -1, 9312, 9313, 9314, -1, 9315, 9316, 9317, -1, 9318, 9319, 9320, -1, 9321, 9322, 9323, -1, 9324, 9325, 9326, -1, 9327, 9328, 9329, -1, 9330, 9331, 9332, -1, 9333, 9334, 9335, -1, 9336, 9337, 9338, -1, 9339, 9340, 9341, -1, 9342, 9343, 9344, -1, 9345, 9346, 9347, -1, 9348, 9349, 9350, -1, 9351, 9352, 9353, -1, 9354, 9355, 9356, -1, 9357, 9358, 9359, -1, 9360, 9361, 9362, -1, 9363, 9364, 9365, -1, 9366, 9367, 9368, -1, 9369, 9370, 9371, -1, 9372, 9373, 9374, -1, 9375, 9376, 9377, -1, 9378, 9379, 9380, -1, 9381, 9382, 9383, -1, 9384, 9385, 9386, -1, 9387, 9388, 9389, -1, 9390, 9391, 9392, -1, 9393, 9394, 9395, -1, 9396, 9397, 9398, -1, 9399, 9400, 9401, -1, 9402, 9403, 9404, -1, 9405, 9406, 9407, -1, 9408, 9409, 9410, -1, 9411, 9412, 9413, -1, 9414, 9415, 9416, -1, 9417, 9418, 9419, -1, 9420, 9421, 9422, -1, 9423, 9424, 9425, -1, 9426, 9427, 9428, -1, 9429, 9430, 9431, -1, 9432, 9433, 9434, -1, 9435, 9436, 9437, -1, 9438, 9439, 9440, -1, 9441, 9442, 9443, -1, 9444, 9445, 9446, -1, 9447, 9448, 9449, -1, 9450, 9451, 9452, -1, 9453, 9454, 9455, -1, 9456, 9457, 9458, -1, 9459, 9460, 9461, -1, 9462, 9463, 9464, -1, 9465, 9466, 9467, -1, 9468, 9469, 9470, -1, 9471, 9472, 9473, -1, 9474, 9475, 9476, -1, 9477, 9478, 9479, -1, 9480, 9481, 9482, -1, 9483, 9484, 9485, -1, 9486, 9487, 9488, -1, 9489, 9490, 9491, -1, 9492, 9493, 9494, -1, 9495, 9496, 9497, -1, 9498, 9499, 9500, -1, 9501, 9502, 9503, -1, 9504, 9505, 9506, -1, 9507, 9508, 9509, -1, 9510, 9511, 9512, -1, 9513, 9514, 9515, -1, 9516, 9517, 9518, -1, 9519, 9520, 9521, -1, 9522, 9523, 9524, -1, 9525, 9526, 9527, -1, 9528, 9529, 9530, -1, 9531, 9532, 9533, -1, 9534, 9535, 9536, -1, 9537, 9538, 9539, -1, 9540, 9541, 9542, -1, 9543, 9544, 9545, -1, 9546, 9547, 9548, -1, 9549, 9550, 9551, -1, 9552, 9553, 9554, -1, 9555, 9556, 9557, -1, 9558, 9559, 9560, -1, 9561, 9562, 9563, -1, 9564, 9565, 9566, -1, 9567, 9568, 9569, -1, 9570, 9571, 9572, -1, 9573, 9574, 9575, -1, 9576, 9577, 9578, -1, 9579, 9580, 9581, -1, 9582, 9583, 9584, -1, 9585, 9586, 9587, -1, 9588, 9589, 9590, -1, 9591, 9592, 9593, -1, 9594, 9595, 9596, -1, 9597, 9598, 9599, -1, 9600, 9601, 9602, -1, 9603, 9604, 9605, -1, 9606, 9607, 9608, -1, 9609, 9610, 9611, -1, 9612, 9613, 9614, -1, 9615, 9616, 9617, -1, 9618, 9619, 9620, -1, 9621, 9622, 9623, -1, 9624, 9625, 9626, -1, 9627, 9628, 9629, -1, 9630, 9631, 9632, -1, 9633, 9634, 9635, -1, 9636, 9637, 9638, -1, 9639, 9640, 9641, -1, 9642, 9643, 9644, -1, 9645, 9646, 9647, -1, 9648, 9649, 9650, -1, 9651, 9652, 9653, -1, 9654, 9655, 9656, -1, 9657, 9658, 9659, -1, 9660, 9661, 9662, -1, 9663, 9664, 9665, -1, 9666, 9667, 9668, -1, 9669, 9670, 9671, -1, 9672, 9673, 9674, -1, 9675, 9676, 9677, -1, 9678, 9679, 9680, -1, 9681, 9682, 9683, -1, 9684, 9685, 9686, -1, 9687, 9688, 9689, -1, 9690, 9691, 9692, -1, 9693, 9694, 9695, -1, 9696, 9697, 9698, -1, 9699, 9700, 9701, -1, 9702, 9703, 9704, -1, 9705, 9706, 9707, -1, 9708, 9709, 9710, -1, 9711, 9712, 9713, -1, 9714, 9715, 9716, -1, 9717, 9718, 9719, -1, 9720, 9721, 9722, -1, 9723, 9724, 9725, -1, 9726, 9727, 9728, -1, 9729, 9730, 9731, -1, 9732, 9733, 9734, -1, 9735, 9736, 9737, -1, 9738, 9739, 9740, -1, 9741, 9742, 9743, -1, 9744, 9745, 9746, -1, 9747, 9748, 9749, -1, 9750, 9751, 9752, -1, 9753, 9754, 9755, -1, 9756, 9757, 9758, -1, 9759, 9760, 9761, -1, 9762, 9763, 9764, -1, 9765, 9766, 9767, -1, 9768, 9769, 9770, -1, 9771, 9772, 9773, -1, 9774, 9775, 9776, -1, 9777, 9778, 9779, -1, 9780, 9781, 9782, -1, 9783, 9784, 9785, -1, 9786, 9787, 9788, -1, 9789, 9790, 9791, -1, 9792, 9793, 9794, -1, 9795, 9796, 9797, -1, 9798, 9799, 9800, -1, 9801, 9802, 9803, -1, 9804, 9805, 9806, -1, 9807, 9808, 9809, -1, 9810, 9811, 9812, -1, 9813, 9814, 9815, -1, 9816, 9817, 9818, -1, 9819, 9820, 9821, -1, 9822, 9823, 9824, -1, 9825, 9826, 9827, -1, 9828, 9829, 9830, -1, 9831, 9832, 9833, -1, 9834, 9835, 9836, -1, 9837, 9838, 9839, -1, 9840, 9841, 9842, -1, 9843, 9844, 9845, -1, 9846, 9847, 9848, -1, 9849, 9850, 9851, -1, 9852, 9853, 9854, -1, 9855, 9856, 9857, -1, 9858, 9859, 9860, -1, 9861, 9862, 9863, -1, 9864, 9865, 9866, -1, 9867, 9868, 9869, -1, 9870, 9871, 9872, -1, 9873, 9874, 9875, -1, 9876, 9877, 9878, -1, 9879, 9880, 9881, -1, 9882, 9883, 9884, -1, 9885, 9886, 9887, -1, 9888, 9889, 9890, -1, 9891, 9892, 9893, -1, 9894, 9895, 9896, -1, 9897, 9898, 9899, -1, 9900, 9901, 9902, -1, 9903, 9904, 9905, -1, 9906, 9907, 9908, -1, 9909, 9910, 9911, -1, 9912, 9913, 9914, -1, 9915, 9916, 9917, -1, 9918, 9919, 9920, -1, 9921, 9922, 9923, -1, 9924, 9925, 9926, -1, 9927, 9928, 9929, -1, 9930, 9931, 9932, -1, 9933, 9934, 9935, -1, 9936, 9937, 9938, -1, 9939, 9940, 9941, -1, 9942, 9943, 9944, -1, 9945, 9946, 9947, -1, 9948, 9949, 9950, -1, 9951, 9952, 9953, -1, 9954, 9955, 9956, -1, 9957, 9958, 9959, -1, 9960, 9961, 9962, -1, 9963, 9964, 9965, -1, 9966, 9967, 9968, -1, 9969, 9970, 9971, -1, 9972, 9973, 9974, -1, 9975, 9976, 9977, -1, 9978, 9979, 9980, -1, 9981, 9982, 9983, -1, 9984, 9985, 9986, -1, 9987, 9988, 9989, -1, 9990, 9991, 9992, -1, 9993, 9994, 9995, -1, 9996, 9997, 9998, -1, 9999, 10000, 10001, -1, 10002, 10003, 10004, -1, 10005, 10006, 10007, -1, 10008, 10009, 10010, -1, 10011, 10012, 10013, -1, 10014, 10015, 10016, -1, 10017, 10018, 10019, -1, 10020, 10021, 10022, -1, 10023, 10024, 10025, -1, 10026, 10027, 10028, -1, 10029, 10030, 10031, -1, 10032, 10033, 10034, -1, 10035, 10036, 10037, -1, 10038, 10039, 10040, -1, 10041, 10042, 10043, -1, 10044, 10045, 10046, -1, 10047, 10048, 10049, -1, 10050, 10051, 10052, -1, 10053, 10054, 10055, -1, 10056, 10057, 10058, -1, 10059, 10060, 10061, -1, 10062, 10063, 10064, -1, 10065, 10066, 10067, -1, 10068, 10069, 10070, -1, 10071, 10072, 10073, -1, 10074, 10075, 10076, -1, 10077, 10078, 10079, -1, 10080, 10081, 10082, -1, 10083, 10084, 10085, -1, 10086, 10087, 10088, -1, 10089, 10090, 10091, -1, 10092, 10093, 10094, -1, 10095, 10096, 10097, -1, 10098, 10099, 10100, -1, 10101, 10102, 10103, -1, 10104, 10105, 10106, -1, 10107, 10108, 10109, -1, 10110, 10111, 10112, -1, 10113, 10114, 10115, -1, 10116, 10117, 10118, -1, 10119, 10120, 10121, -1, 10122, 10123, 10124, -1, 10125, 10126, 10127, -1, 10128, 10129, 10130, -1, 10131, 10132, 10133, -1, 10134, 10135, 10136, -1, 10137, 10138, 10139, -1, 10140, 10141, 10142, -1, 10143, 10144, 10145, -1, 10146, 10147, 10148, -1, 10149, 10150, 10151, -1, 10152, 10153, 10154, -1, 10155, 10156, 10157, -1, 10158, 10159, 10160, -1, 10161, 10162, 10163, -1, 10164, 10165, 10166, -1, 10167, 10168, 10169, -1, 10170, 10171, 10172, -1, 10173, 10174, 10175, -1, 10176, 10177, 10178, -1, 10179, 10180, 10181, -1, 10182, 10183, 10184, -1, 10185, 10186, 10187, -1, 10188, 10189, 10190, -1, 10191, 10192, 10193, -1, 10194, 10195, 10196, -1, 10197, 10198, 10199, -1, 10200, 10201, 10202, -1, 10203, 10204, 10205, -1, 10206, 10207, 10208, -1, 10209, 10210, 10211, -1, 10212, 10213, 10214, -1, 10215, 10216, 10217, -1, 10218, 10219, 10220, -1, 10221, 10222, 10223, -1, 10224, 10225, 10226, -1, 10227, 10228, 10229, -1, 10230, 10231, 10232, -1, 10233, 10234, 10235, -1, 10236, 10237, 10238, -1, 10239, 10240, 10241, -1, 10242, 10243, 10244, -1, 10245, 10246, 10247, -1, 10248, 10249, 10250, -1, 10251, 10252, 10253, -1, 10254, 10255, 10256, -1, 10257, 10258, 10259, -1, 10260, 10261, 10262, -1, 10263, 10264, 10265, -1, 10266, 10267, 10268, -1, 10269, 10270, 10271, -1, 10272, 10273, 10274, -1, 10275, 10276, 10277, -1, 10278, 10279, 10280, -1, 10281, 10282, 10283, -1, 10284, 10285, 10286, -1, 10287, 10288, 10289, -1, 10290, 10291, 10292, -1, 10293, 10294, 10295, -1, 10296, 10297, 10298, -1, 10299, 10300, 10301, -1, 10302, 10303, 10304, -1, 10305, 10306, 10307, -1, 10308, 10309, 10310, -1, 10311, 10312, 10313, -1, 10314, 10315, 10316, -1, 10317, 10318, 10319, -1, 10320, 10321, 10322, -1, 10323, 10324, 10325, -1, 10326, 10327, 10328, -1, 10329, 10330, 10331, -1, 10332, 10333, 10334, -1, 10335, 10336, 10337, -1, 10338, 10339, 10340, -1, 10341, 10342, 10343, -1, 10344, 10345, 10346, -1, 10347, 10348, 10349, -1, 10350, 10351, 10352, -1, 10353, 10354, 10355, -1, 10356, 10357, 10358, -1, 10359, 10360, 10361, -1, 10362, 10363, 10364, -1, 10365, 10366, 10367, -1, 10368, 10369, 10370, -1, 10371, 10372, 10373, -1, 10374, 10375, 10376, -1, 10377, 10378, 10379, -1, 10380, 10381, 10382, -1, 10383, 10384, 10385, -1, 10386, 10387, 10388, -1, 10389, 10390, 10391, -1, 10392, 10393, 10394, -1, 10395, 10396, 10397, -1, 10398, 10399, 10400, -1, 10401, 10402, 10403, -1, 10404, 10405, 10406, -1, 10407, 10408, 10409, -1, 10410, 10411, 10412, -1, 10413, 10414, 10415, -1, 10416, 10417, 10418, -1, 10419, 10420, 10421, -1, 10422, 10423, 10424, -1, 10425, 10426, 10427, -1, 10428, 10429, 10430, -1, 10431, 10432, 10433, -1, 10434, 10435, 10436, -1, 10437, 10438, 10439, -1, 10440, 10441, 10442, -1, 10443, 10444, 10445, -1, 10446, 10447, 10448, -1, 10449, 10450, 10451, -1, 10452, 10453, 10454, -1, 10455, 10456, 10457, -1, 10458, 10459, 10460, -1, 10461, 10462, 10463, -1, 10464, 10465, 10466, -1, 10467, 10468, 10469, -1, 10470, 10471, 10472, -1, 10473, 10474, 10475, -1, 10476, 10477, 10478, -1, 10479, 10480, 10481, -1, 10482, 10483, 10484, -1, 10485, 10486, 10487, -1, 10488, 10489, 10490, -1, 10491, 10492, 10493, -1, 10494, 10495, 10496, -1, 10497, 10498, 10499, -1, 10500, 10501, 10502, -1, 10503, 10504, 10505, -1, 10506, 10507, 10508, -1, 10509, 10510, 10511, -1, 10512, 10513, 10514, -1, 10515, 10516, 10517, -1, 10518, 10519, 10520, -1, 10521, 10522, 10523, -1, 10524, 10525, 10526, -1, 10527, 10528, 10529, -1, 10530, 10531, 10532, -1, 10533, 10534, 10535, -1, 10536, 10537, 10538, -1, 10539, 10540, 10541, -1, 10542, 10543, 10544, -1, 10545, 10546, 10547, -1, 10548, 10549, 10550, -1, 10551, 10552, 10553, -1, 10554, 10555, 10556, -1, 10557, 10558, 10559, -1, 10560, 10561, 10562, -1, 10563, 10564, 10565, -1, 10566, 10567, 10568, -1, 10569, 10570, 10571, -1, 10572, 10573, 10574, -1, 10575, 10576, 10577, -1, 10578, 10579, 10580, -1, 10581, 10582, 10583, -1, 10584, 10585, 10586, -1, 10587, 10588, 10589, -1, 10590, 10591, 10592, -1, 10593, 10594, 10595, -1, 10596, 10597, 10598, -1, 10599, 10600, 10601, -1, 10602, 10603, 10604, -1, 10605, 10606, 10607, -1, 10608, 10609, 10610, -1, 10611, 10612, 10613, -1, 10614, 10615, 10616, -1, 10617, 10618, 10619, -1, 10620, 10621, 10622, -1, 10623, 10624, 10625, -1, 10626, 10627, 10628, -1, 10629, 10630, 10631, -1, 10632, 10633, 10634, -1, 10635, 10636, 10637, -1, 10638, 10639, 10640, -1, 10641, 10642, 10643, -1, 10644, 10645, 10646, -1, 10647, 10648, 10649, -1, 10650, 10651, 10652, -1, 10653, 10654, 10655, -1, 10656, 10657, 10658, -1, 10659, 10660, 10661, -1, 10662, 10663, 10664, -1, 10665, 10666, 10667, -1, 10668, 10669, 10670, -1, 10671, 10672, 10673, -1, 10674, 10675, 10676, -1, 10677, 10678, 10679, -1, 10680, 10681, 10682, -1, 10683, 10684, 10685, -1, 10686, 10687, 10688, -1, 10689, 10690, 10691, -1, 10692, 10693, 10694, -1, 10695, 10696, 10697, -1, 10698, 10699, 10700, -1, 10701, 10702, 10703, -1, 10704, 10705, 10706, -1, 10707, 10708, 10709, -1, 10710, 10711, 10712, -1, 10713, 10714, 10715, -1, 10716, 10717, 10718, -1, 10719, 10720, 10721, -1, 10722, 10723, 10724, -1, 10725, 10726, 10727, -1, 10728, 10729, 10730, -1, 10731, 10732, 10733, -1, 10734, 10735, 10736, -1, 10737, 10738, 10739, -1, 10740, 10741, 10742, -1, 10743, 10744, 10745, -1, 10746, 10747, 10748, -1, 10749, 10750, 10751, -1, 10752, 10753, 10754, -1, 10755, 10756, 10757, -1, 10758, 10759, 10760, -1, 10761, 10762, 10763, -1, 10764, 10765, 10766, -1, 10767, 10768, 10769, -1, 10770, 10771, 10772, -1, 10773, 10774, 10775, -1, 10776, 10777, 10778, -1, 10779, 10780, 10781, -1, 10782, 10783, 10784, -1, 10785, 10786, 10787, -1, 10788, 10789, 10790, -1, 10791, 10792, 10793, -1, 10794, 10795, 10796, -1, 10797, 10798, 10799, -1, 10800, 10801, 10802, -1, 10803, 10804, 10805, -1, 10806, 10807, 10808, -1, 10809, 10810, 10811, -1, 10812, 10813, 10814, -1, 10815, 10816, 10817, -1, 10818, 10819, 10820, -1, 10821, 10822, 10823, -1, 10824, 10825, 10826, -1, 10827, 10828, 10829, -1, 10830, 10831, 10832, -1, 10833, 10834, 10835, -1, 10836, 10837, 10838, -1, 10839, 10840, 10841, -1, 10842, 10843, 10844, -1, 10845, 10846, 10847, -1, 10848, 10849, 10850, -1, 10851, 10852, 10853, -1, 10854, 10855, 10856, -1, 10857, 10858, 10859, -1, 10860, 10861, 10862, -1, 10863, 10864, 10865, -1, 10866, 10867, 10868, -1, 10869, 10870, 10871, -1, 10872, 10873, 10874, -1, 10875, 10876, 10877, -1, 10878, 10879, 10880, -1, 10881, 10882, 10883, -1, 10884, 10885, 10886, -1, 10887, 10888, 10889, -1, 10890, 10891, 10892, -1, 10893, 10894, 10895, -1, 10896, 10897, 10898, -1, 10899, 10900, 10901, -1, 10902, 10903, 10904, -1, 10905, 10906, 10907, -1, 10908, 10909, 10910, -1, 10911, 10912, 10913, -1, 10914, 10915, 10916, -1, 10917, 10918, 10919, -1, 10920, 10921, 10922, -1, 10923, 10924, 10925, -1, 10926, 10927, 10928, -1, 10929, 10930, 10931, -1, 10932, 10933, 10934, -1, 10935, 10936, 10937, -1, 10938, 10939, 10940, -1, 10941, 10942, 10943, -1, 10944, 10945, 10946, -1, 10947, 10948, 10949, -1, 10950, 10951, 10952, -1, 10953, 10954, 10955, -1, 10956, 10957, 10958, -1, 10959, 10960, 10961, -1, 10962, 10963, 10964, -1, 10965, 10966, 10967, -1, 10968, 10969, 10970, -1, 10971, 10972, 10973, -1, 10974, 10975, 10976, -1, 10977, 10978, 10979, -1, 10980, 10981, 10982, -1, 10983, 10984, 10985, -1, 10986, 10987, 10988, -1, 10989, 10990, 10991, -1, 10992, 10993, 10994, -1, 10995, 10996, 10997, -1, 10998, 10999, 11000, -1, 11001, 11002, 11003, -1, 11004, 11005, 11006, -1, 11007, 11008, 11009, -1, 11010, 11011, 11012, -1, 11013, 11014, 11015, -1, 11016, 11017, 11018, -1, 11019, 11020, 11021, -1, 11022, 11023, 11024, -1, 11025, 11026, 11027, -1, 11028, 11029, 11030, -1, 11031, 11032, 11033, -1, 11034, 11035, 11036, -1, 11037, 11038, 11039, -1, 11040, 11041, 11042, -1, 11043, 11044, 11045, -1, 11046, 11047, 11048, -1, 11049, 11050, 11051, -1, 11052, 11053, 11054, -1, 11055, 11056, 11057, -1, 11058, 11059, 11060, -1, 11061, 11062, 11063, -1, 11064, 11065, 11066, -1, 11067, 11068, 11069, -1, 11070, 11071, 11072, -1, 11073, 11074, 11075, -1, 11076, 11077, 11078, -1, 11079, 11080, 11081, -1, 11082, 11083, 11084, -1, 11085, 11086, 11087, -1, 11088, 11089, 11090, -1, 11091, 11092, 11093, -1, 11094, 11095, 11096, -1, 11097, 11098, 11099, -1, 11100, 11101, 11102, -1, 11103, 11104, 11105, -1, 11106, 11107, 11108, -1, 11109, 11110, 11111, -1, 11112, 11113, 11114, -1, 11115, 11116, 11117, -1, 11118, 11119, 11120, -1, 11121, 11122, 11123, -1, 11124, 11125, 11126, -1, 11127, 11128, 11129, -1, 11130, 11131, 11132, -1, 11133, 11134, 11135, -1, 11136, 11137, 11138, -1, 11139, 11140, 11141, -1, 11142, 11143, 11144, -1, 11145, 11146, 11147, -1, 11148, 11149, 11150, -1, 11151, 11152, 11153, -1, 11154, 11155, 11156, -1, 11157, 11158, 11159, -1, 11160, 11161, 11162, -1, 11163, 11164, 11165, -1, 11166, 11167, 11168, -1, 11169, 11170, 11171, -1, 11172, 11173, 11174, -1, 11175, 11176, 11177, -1, 11178, 11179, 11180, -1, 11181, 11182, 11183, -1, 11184, 11185, 11186, -1, 11187, 11188, 11189, -1, 11190, 11191, 11192, -1, 11193, 11194, 11195, -1, 11196, 11197, 11198, -1, 11199, 11200, 11201, -1, 11202, 11203, 11204, -1, 11205, 11206, 11207, -1, 11208, 11209, 11210, -1, 11211, 11212, 11213, -1, 11214, 11215, 11216, -1, 11217, 11218, 11219, -1, 11220, 11221, 11222, -1, 11223, 11224, 11225, -1, 11226, 11227, 11228, -1, 11229, 11230, 11231, -1, 11232, 11233, 11234, -1, 11235, 11236, 11237, -1, 11238, 11239, 11240, -1, 11241, 11242, 11243, -1, 11244, 11245, 11246, -1, 11247, 11248, 11249, -1, 11250, 11251, 11252, -1, 11253, 11254, 11255, -1, 11256, 11257, 11258, -1, 11259, 11260, 11261, -1, 11262, 11263, 11264, -1, 11265, 11266, 11267, -1, 11268, 11269, 11270, -1, 11271, 11272, 11273, -1, 11274, 11275, 11276, -1, 11277, 11278, 11279, -1, 11280, 11281, 11282, -1, 11283, 11284, 11285, -1, 11286, 11287, 11288, -1, 11289, 11290, 11291, -1, 11292, 11293, 11294, -1, 11295, 11296, 11297, -1, 11298, 11299, 11300, -1, 11301, 11302, 11303, -1, 11304, 11305, 11306, -1, 11307, 11308, 11309, -1, 11310, 11311, 11312, -1, 11313, 11314, 11315, -1, 11316, 11317, 11318, -1, 11319, 11320, 11321, -1, 11322, 11323, 11324, -1, 11325, 11326, 11327, -1, 11328, 11329, 11330, -1, 11331, 11332, 11333, -1, 11334, 11335, 11336, -1, 11337, 11338, 11339, -1, 11340, 11341, 11342, -1, 11343, 11344, 11345, -1, 11346, 11347, 11348, -1, 11349, 11350, 11351, -1, 11352, 11353, 11354, -1, 11355, 11356, 11357, -1, 11358, 11359, 11360, -1, 11361, 11362, 11363, -1, 11364, 11365, 11366, -1, 11367, 11368, 11369, -1, 11370, 11371, 11372, -1, 11373, 11374, 11375, -1, 11376, 11377, 11378, -1, 11379, 11380, 11381, -1, 11382, 11383, 11384, -1, 11385, 11386, 11387, -1, 11388, 11389, 11390, -1, 11391, 11392, 11393, -1, 11394, 11395, 11396, -1, 11397, 11398, 11399, -1, 11400, 11401, 11402, -1, 11403, 11404, 11405, -1, 11406, 11407, 11408, -1, 11409, 11410, 11411, -1, 11412, 11413, 11414, -1, 11415, 11416, 11417, -1, 11418, 11419, 11420, -1, 11421, 11422, 11423, -1, 11424, 11425, 11426, -1, 11427, 11428, 11429, -1, 11430, 11431, 11432, -1, 11433, 11434, 11435, -1, 11436, 11437, 11438, -1, 11439, 11440, 11441, -1, 11442, 11443, 11444, -1, 11445, 11446, 11447, -1, 11448, 11449, 11450, -1, 11451, 11452, 11453, -1, 11454, 11455, 11456, -1, 11457, 11458, 11459, -1, 11460, 11461, 11462, -1, 11463, 11464, 11465, -1, 11466, 11467, 11468, -1, 11469, 11470, 11471, -1, 11472, 11473, 11474, -1, 11475, 11476, 11477, -1, 11478, 11479, 11480, -1, 11481, 11482, 11483, -1, 11484, 11485, 11486, -1, 11487, 11488, 11489, -1, 11490, 11491, 11492, -1, 11493, 11494, 11495, -1, 11496, 11497, 11498, -1, 11499, 11500, 11501, -1, 11502, 11503, 11504, -1, 11505, 11506, 11507, -1, 11508, 11509, 11510, -1, 11511, 11512, 11513, -1, 11514, 11515, 11516, -1, 11517, 11518, 11519, -1, 11520, 11521, 11522, -1, 11523, 11524, 11525, -1, 11526, 11527, 11528, -1, 11529, 11530, 11531, -1, 11532, 11533, 11534, -1, 11535, 11536, 11537, -1, 11538, 11539, 11540, -1, 11541, 11542, 11543, -1, 11544, 11545, 11546, -1, 11547, 11548, 11549, -1, 11550, 11551, 11552, -1, 11553, 11554, 11555, -1, 11556, 11557, 11558, -1, 11559, 11560, 11561, -1, 11562, 11563, 11564, -1, 11565, 11566, 11567, -1, 11568, 11569, 11570, -1, 11571, 11572, 11573, -1, 11574, 11575, 11576, -1, 11577, 11578, 11579, -1, 11580, 11581, 11582, -1, 11583, 11584, 11585, -1, 11586, 11587, 11588, -1, 11589, 11590, 11591, -1, 11592, 11593, 11594, -1, 11595, 11596, 11597, -1, 11598, 11599, 11600, -1, 11601, 11602, 11603, -1, 11604, 11605, 11606, -1, 11607, 11608, 11609, -1, 11610, 11611, 11612, -1, 11613, 11614, 11615, -1, 11616, 11617, 11618, -1, 11619, 11620, 11621, -1, 11622, 11623, 11624, -1, 11625, 11626, 11627, -1, 11628, 11629, 11630, -1, 11631, 11632, 11633, -1, 11634, 11635, 11636, -1, 11637, 11638, 11639, -1, 11640, 11641, 11642, -1, 11643, 11644, 11645, -1, 11646, 11647, 11648, -1, 11649, 11650, 11651, -1, 11652, 11653, 11654, -1, 11655, 11656, 11657, -1, 11658, 11659, 11660, -1, 11661, 11662, 11663, -1, 11664, 11665, 11666, -1, 11667, 11668, 11669, -1, 11670, 11671, 11672, -1, 11673, 11674, 11675, -1, 11676, 11677, 11678, -1, 11679, 11680, 11681, -1, 11682, 11683, 11684, -1, 11685, 11686, 11687, -1, 11688, 11689, 11690, -1, 11691, 11692, 11693, -1, 11694, 11695, 11696, -1, 11697, 11698, 11699, -1, 11700, 11701, 11702, -1, 11703, 11704, 11705, -1, 11706, 11707, 11708, -1, 11709, 11710, 11711, -1, 11712, 11713, 11714, -1, 11715, 11716, 11717, -1, 11718, 11719, 11720, -1, 11721, 11722, 11723, -1, 11724, 11725, 11726, -1, 11727, 11728, 11729, -1, 11730, 11731, 11732, -1, 11733, 11734, 11735, -1, 11736, 11737, 11738, -1, 11739, 11740, 11741, -1, 11742, 11743, 11744, -1, 11745, 11746, 11747, -1, 11748, 11749, 11750, -1, 11751, 11752, 11753, -1, 11754, 11755, 11756, -1, 11757, 11758, 11759, -1, 11760, 11761, 11762, -1, 11763, 11764, 11765, -1, 11766, 11767, 11768, -1, 11769, 11770, 11771, -1, 11772, 11773, 11774, -1, 11775, 11776, 11777, -1, 11778, 11779, 11780, -1, 11781, 11782, 11783, -1, 11784, 11785, 11786, -1, 11787, 11788, 11789, -1, 11790, 11791, 11792, -1, 11793, 11794, 11795, -1, 11796, 11797, 11798, -1, 11799, 11800, 11801, -1, 11802, 11803, 11804, -1, 11805, 11806, 11807, -1, 11808, 11809, 11810, -1, 11811, 11812, 11813, -1, 11814, 11815, 11816, -1, 11817, 11818, 11819, -1, 11820, 11821, 11822, -1, 11823, 11824, 11825, -1, 11826, 11827, 11828, -1, 11829, 11830, 11831, -1, 11832, 11833, 11834, -1, 11835, 11836, 11837, -1, 11838, 11839, 11840, -1, 11841, 11842, 11843, -1, 11844, 11845, 11846, -1, 11847, 11848, 11849, -1, 11850, 11851, 11852, -1, 11853, 11854, 11855, -1, 11856, 11857, 11858, -1, 11859, 11860, 11861, -1, 11862, 11863, 11864, -1, 11865, 11866, 11867, -1, 11868, 11869, 11870, -1, 11871, 11872, 11873, -1, 11874, 11875, 11876, -1, 11877, 11878, 11879, -1, 11880, 11881, 11882, -1, 11883, 11884, 11885, -1, 11886, 11887, 11888, -1, 11889, 11890, 11891, -1, 11892, 11893, 11894, -1, 11895, 11896, 11897, -1, 11898, 11899, 11900, -1, 11901, 11902, 11903, -1, 11904, 11905, 11906, -1, 11907, 11908, 11909, -1, 11910, 11911, 11912, -1, 11913, 11914, 11915, -1, 11916, 11917, 11918, -1, 11919, 11920, 11921, -1, 11922, 11923, 11924, -1, 11925, 11926, 11927, -1, 11928, 11929, 11930, -1, 11931, 11932, 11933, -1, 11934, 11935, 11936, -1, 11937, 11938, 11939, -1, 11940, 11941, 11942, -1, 11943, 11944, 11945, -1, 11946, 11947, 11948, -1, 11949, 11950, 11951, -1, 11952, 11953, 11954, -1, 11955, 11956, 11957, -1, 11958, 11959, 11960, -1, 11961, 11962, 11963, -1, 11964, 11965, 11966, -1, 11967, 11968, 11969, -1, 11970, 11971, 11972, -1, 11973, 11974, 11975, -1, 11976, 11977, 11978, -1, 11979, 11980, 11981, -1, 11982, 11983, 11984, -1, 11985, 11986, 11987, -1, 11988, 11989, 11990, -1, 11991, 11992, 11993, -1, 11994, 11995, 11996, -1, 11997, 11998, 11999, -1, 12000, 12001, 12002, -1, 12003, 12004, 12005, -1, 12006, 12007, 12008, -1, 12009, 12010, 12011, -1, 12012, 12013, 12014, -1, 12015, 12016, 12017, -1, 12018, 12019, 12020, -1, 12021, 12022, 12023, -1, 12024, 12025, 12026, -1, 12027, 12028, 12029, -1, 12030, 12031, 12032, -1, 12033, 12034, 12035, -1, 12036, 12037, 12038, -1, 12039, 12040, 12041, -1, 12042, 12043, 12044, -1, 12045, 12046, 12047, -1, 12048, 12049, 12050, -1, 12051, 12052, 12053, -1, 12054, 12055, 12056, -1, 12057, 12058, 12059, -1, 12060, 12061, 12062, -1, 12063, 12064, 12065, -1, 12066, 12067, 12068, -1, 12069, 12070, 12071, -1, 12072, 12073, 12074, -1, 12075, 12076, 12077, -1, 12078, 12079, 12080, -1, 12081, 12082, 12083, -1, 12084, 12085, 12086, -1, 12087, 12088, 12089, -1, 12090, 12091, 12092, -1, 12093, 12094, 12095, -1, 12096, 12097, 12098, -1, 12099, 12100, 12101, -1, 12102, 12103, 12104, -1, 12105, 12106, 12107, -1, 12108, 12109, 12110, -1, 12111, 12112, 12113, -1, 12114, 12115, 12116, -1, 12117, 12118, 12119, -1, 12120, 12121, 12122, -1, 12123, 12124, 12125, -1, 12126, 12127, 12128, -1, 12129, 12130, 12131, -1, 12132, 12133, 12134, -1, 12135, 12136, 12137, -1, 12138, 12139, 12140, -1, 12141, 12142, 12143, -1, 12144, 12145, 12146, -1, 12147, 12148, 12149, -1, 12150, 12151, 12152, -1, 12153, 12154, 12155, -1, 12156, 12157, 12158, -1, 12159, 12160, 12161, -1, 12162, 12163, 12164, -1, 12165, 12166, 12167, -1, 12168, 12169, 12170, -1, 12171, 12172, 12173, -1, 12174, 12175, 12176, -1, 12177, 12178, 12179, -1, 12180, 12181, 12182, -1, 12183, 12184, 12185, -1, 12186, 12187, 12188, -1, 12189, 12190, 12191, -1, 12192, 12193, 12194, -1, 12195, 12196, 12197, -1, 12198, 12199, 12200, -1, 12201, 12202, 12203, -1, 12204, 12205, 12206, -1, 12207, 12208, 12209, -1, 12210, 12211, 12212, -1, 12213, 12214, 12215, -1, 12216, 12217, 12218, -1, 12219, 12220, 12221, -1, 12222, 12223, 12224, -1, 12225, 12226, 12227, -1, 12228, 12229, 12230, -1, 12231, 12232, 12233, -1, 12234, 12235, 12236, -1, 12237, 12238, 12239, -1, 12240, 12241, 12242, -1, 12243, 12244, 12245, -1, 12246, 12247, 12248, -1, 12249, 12250, 12251, -1, 12252, 12253, 12254, -1, 12255, 12256, 12257, -1, 12258, 12259, 12260, -1, 12261, 12262, 12263, -1, 12264, 12265, 12266, -1, 12267, 12268, 12269, -1, 12270, 12271, 12272, -1, 12273, 12274, 12275, -1, 12276, 12277, 12278, -1, 12279, 12280, 12281, -1, 12282, 12283, 12284, -1, 12285, 12286, 12287, -1, 12288, 12289, 12290, -1, 12291, 12292, 12293, -1, 12294, 12295, 12296, -1, 12297, 12298, 12299, -1, 12300, 12301, 12302, -1, 12303, 12304, 12305, -1, 12306, 12307, 12308, -1, 12309, 12310, 12311, -1, 12312, 12313, 12314, -1, 12315, 12316, 12317, -1, 12318, 12319, 12320, -1, 12321, 12322, 12323, -1, 12324, 12325, 12326, -1, 12327, 12328, 12329, -1, 12330, 12331, 12332, -1, 12333, 12334, 12335, -1, 12336, 12337, 12338, -1, 12339, 12340, 12341, -1, 12342, 12343, 12344, -1, 12345, 12346, 12347, -1, 12348, 12349, 12350, -1, 12351, 12352, 12353, -1, 12354, 12355, 12356, -1, 12357, 12358, 12359, -1, 12360, 12361, 12362, -1, 12363, 12364, 12365, -1, 12366, 12367, 12368, -1, 12369, 12370, 12371, -1, 12372, 12373, 12374, -1, 12375, 12376, 12377, -1, 12378, 12379, 12380, -1, 12381, 12382, 12383, -1, 12384, 12385, 12386, -1, 12387, 12388, 12389, -1, 12390, 12391, 12392, -1, 12393, 12394, 12395, -1, 12396, 12397, 12398, -1, 12399, 12400, 12401, -1, 12402, 12403, 12404, -1, 12405, 12406, 12407, -1, 12408, 12409, 12410, -1, 12411, 12412, 12413, -1, 12414, 12415, 12416, -1, 12417, 12418, 12419, -1, 12420, 12421, 12422, -1, 12423, 12424, 12425, -1, 12426, 12427, 12428, -1, 12429, 12430, 12431, -1, 12432, 12433, 12434, -1, 12435, 12436, 12437, -1, 12438, 12439, 12440, -1, 12441, 12442, 12443, -1, 12444, 12445, 12446, -1, 12447, 12448, 12449, -1, 12450, 12451, 12452, -1, 12453, 12454, 12455, -1, 12456, 12457, 12458, -1, 12459, 12460, 12461, -1, 12462, 12463, 12464, -1, 12465, 12466, 12467, -1, 12468, 12469, 12470, -1, 12471, 12472, 12473, -1, 12474, 12475, 12476, -1, 12477, 12478, 12479, -1, 12480, 12481, 12482, -1, 12483, 12484, 12485, -1, 12486, 12487, 12488, -1, 12489, 12490, 12491, -1, 12492, 12493, 12494, -1, 12495, 12496, 12497, -1, 12498, 12499, 12500, -1, 12501, 12502, 12503, -1, 12504, 12505, 12506, -1, 12507, 12508, 12509, -1, 12510, 12511, 12512, -1, 12513, 12514, 12515, -1, 12516, 12517, 12518, -1, 12519, 12520, 12521, -1, 12522, 12523, 12524, -1, 12525, 12526, 12527, -1, 12528, 12529, 12530, -1, 12531, 12532, 12533, -1, 12534, 12535, 12536, -1, 12537, 12538, 12539, -1, 12540, 12541, 12542, -1, 12543, 12544, 12545, -1, 12546, 12547, 12548, -1, 12549, 12550, 12551, -1, 12552, 12553, 12554, -1, 12555, 12556, 12557, -1, 12558, 12559, 12560, -1, 12561, 12562, 12563, -1, 12564, 12565, 12566, -1, 12567, 12568, 12569, -1, 12570, 12571, 12572, -1, 12573, 12574, 12575, -1, 12576, 12577, 12578, -1, 12579, 12580, 12581, -1, 12582, 12583, 12584, -1, 12585, 12586, 12587, -1, 12588, 12589, 12590, -1, 12591, 12592, 12593, -1, 12594, 12595, 12596, -1, 12597, 12598, 12599, -1, 12600, 12601, 12602, -1, 12603, 12604, 12605, -1, 12606, 12607, 12608, -1, 12609, 12610, 12611, -1, 12612, 12613, 12614, -1, 12615, 12616, 12617, -1, 12618, 12619, 12620, -1, 12621, 12622, 12623, -1, 12624, 12625, 12626, -1, 12627, 12628, 12629, -1, 12630, 12631, 12632, -1, 12633, 12634, 12635, -1, 12636, 12637, 12638, -1, 12639, 12640, 12641, -1, 12642, 12643, 12644, -1, 12645, 12646, 12647, -1, 12648, 12649, 12650, -1, 12651, 12652, 12653, -1, 12654, 12655, 12656, -1, 12657, 12658, 12659, -1, 12660, 12661, 12662, -1, 12663, 12664, 12665, -1, 12666, 12667, 12668, -1, 12669, 12670, 12671, -1, 12672, 12673, 12674, -1, 12675, 12676, 12677, -1, 12678, 12679, 12680, -1, 12681, 12682, 12683, -1, 12684, 12685, 12686, -1, 12687, 12688, 12689, -1, 12690, 12691, 12692, -1, 12693, 12694, 12695, -1, 12696, 12697, 12698, -1, 12699, 12700, 12701, -1, 12702, 12703, 12704, -1, 12705, 12706, 12707, -1, 12708, 12709, 12710, -1, 12711, 12712, 12713, -1, 12714, 12715, 12716, -1, 12717, 12718, 12719, -1, 12720, 12721, 12722, -1, 12723, 12724, 12725, -1, 12726, 12727, 12728, -1, 12729, 12730, 12731, -1, 12732, 12733, 12734, -1, 12735, 12736, 12737, -1, 12738, 12739, 12740, -1, 12741, 12742, 12743, -1, 12744, 12745, 12746, -1, 12747, 12748, 12749, -1, 12750, 12751, 12752, -1, 12753, 12754, 12755, -1, 12756, 12757, 12758, -1, 12759, 12760, 12761, -1, 12762, 12763, 12764, -1, 12765, 12766, 12767, -1, 12768, 12769, 12770, -1, 12771, 12772, 12773, -1, 12774, 12775, 12776, -1, 12777, 12778, 12779, -1, 12780, 12781, 12782, -1, 12783, 12784, 12785, -1, 12786, 12787, 12788, -1, 12789, 12790, 12791, -1, 12792, 12793, 12794, -1, 12795, 12796, 12797, -1, 12798, 12799, 12800, -1, 12801, 12802, 12803, -1, 12804, 12805, 12806, -1, 12807, 12808, 12809, -1, 12810, 12811, 12812, -1, 12813, 12814, 12815, -1, 12816, 12817, 12818, -1, 12819, 12820, 12821, -1, 12822, 12823, 12824, -1, 12825, 12826, 12827, -1, 12828, 12829, 12830, -1, 12831, 12832, 12833, -1, 12834, 12835, 12836, -1, 12837, 12838, 12839, -1, 12840, 12841, 12842, -1, 12843, 12844, 12845, -1, 12846, 12847, 12848, -1, 12849, 12850, 12851, -1, 12852, 12853, 12854, -1, 12855, 12856, 12857, -1, 12858, 12859, 12860, -1, 12861, 12862, 12863, -1, 12864, 12865, 12866, -1, 12867, 12868, 12869, -1, 12870, 12871, 12872, -1, 12873, 12874, 12875, -1, 12876, 12877, 12878, -1, 12879, 12880, 12881, -1, 12882, 12883, 12884, -1, 12885, 12886, 12887, -1, 12888, 12889, 12890, -1, 12891, 12892, 12893, -1, 12894, 12895, 12896, -1, 12897, 12898, 12899, -1, 12900, 12901, 12902, -1, 12903, 12904, 12905, -1, 12906, 12907, 12908, -1, 12909, 12910, 12911, -1, 12912, 12913, 12914, -1, 12915, 12916, 12917, -1, 12918, 12919, 12920, -1, 12921, 12922, 12923, -1, 12924, 12925, 12926, -1, 12927, 12928, 12929, -1, 12930, 12931, 12932, -1, 12933, 12934, 12935, -1, 12936, 12937, 12938, -1, 12939, 12940, 12941, -1, 12942, 12943, 12944, -1, 12945, 12946, 12947, -1, 12948, 12949, 12950, -1, 12951, 12952, 12953, -1, 12954, 12955, 12956, -1, 12957, 12958, 12959, -1, 12960, 12961, 12962, -1, 12963, 12964, 12965, -1, 12966, 12967, 12968, -1, 12969, 12970, 12971, -1, 12972, 12973, 12974, -1, 12975, 12976, 12977, -1, 12978, 12979, 12980, -1, 12981, 12982, 12983, -1, 12984, 12985, 12986, -1, 12987, 12988, 12989, -1, 12990, 12991, 12992, -1, 12993, 12994, 12995, -1, 12996, 12997, 12998, -1, 12999, 13000, 13001, -1, 13002, 13003, 13004, -1, 13005, 13006, 13007, -1, 13008, 13009, 13010, -1, 13011, 13012, 13013, -1, 13014, 13015, 13016, -1, 13017, 13018, 13019, -1, 13020, 13021, 13022, -1, 13023, 13024, 13025, -1, 13026, 13027, 13028, -1, 13029, 13030, 13031, -1, 13032, 13033, 13034, -1, 13035, 13036, 13037, -1, 13038, 13039, 13040, -1, 13041, 13042, 13043, -1, 13044, 13045, 13046, -1, 13047, 13048, 13049, -1, 13050, 13051, 13052, -1, 13053, 13054, 13055, -1, 13056, 13057, 13058, -1, 13059, 13060, 13061, -1, 13062, 13063, 13064, -1, 13065, 13066, 13067, -1, 13068, 13069, 13070, -1, 13071, 13072, 13073, -1, 13074, 13075, 13076, -1, 13077, 13078, 13079, -1, 13080, 13081, 13082, -1, 13083, 13084, 13085, -1, 13086, 13087, 13088, -1, 13089, 13090, 13091, -1, 13092, 13093, 13094, -1, 13095, 13096, 13097, -1, 13098, 13099, 13100, -1, 13101, 13102, 13103, -1, 13104, 13105, 13106, -1, 13107, 13108, 13109, -1, 13110, 13111, 13112, -1, 13113, 13114, 13115, -1, 13116, 13117, 13118, -1, 13119, 13120, 13121, -1, 13122, 13123, 13124, -1, 13125, 13126, 13127, -1, 13128, 13129, 13130, -1, 13131, 13132, 13133, -1, 13134, 13135, 13136, -1, 13137, 13138, 13139, -1, 13140, 13141, 13142, -1, 13143, 13144, 13145, -1, 13146, 13147, 13148, -1, 13149, 13150, 13151, -1, 13152, 13153, 13154, -1, 13155, 13156, 13157, -1, 13158, 13159, 13160, -1, 13161, 13162, 13163, -1, 13164, 13165, 13166, -1, 13167, 13168, 13169, -1, 13170, 13171, 13172, -1, 13173, 13174, 13175, -1, 13176, 13177, 13178, -1, 13179, 13180, 13181, -1, 13182, 13183, 13184, -1, 13185, 13186, 13187, -1, 13188, 13189, 13190, -1, 13191, 13192, 13193, -1, 13194, 13195, 13196, -1, 13197, 13198, 13199, -1, 13200, 13201, 13202, -1, 13203, 13204, 13205, -1, 13206, 13207, 13208, -1, 13209, 13210, 13211, -1, 13212, 13213, 13214, -1, 13215, 13216, 13217, -1, 13218, 13219, 13220, -1, 13221, 13222, 13223, -1, 13224, 13225, 13226, -1, 13227, 13228, 13229, -1, 13230, 13231, 13232, -1, 13233, 13234, 13235, -1, 13236, 13237, 13238, -1, 13239, 13240, 13241, -1, 13242, 13243, 13244, -1, 13245, 13246, 13247, -1, 13248, 13249, 13250, -1, 13251, 13252, 13253, -1, 13254, 13255, 13256, -1, 13257, 13258, 13259, -1, 13260, 13261, 13262, -1, 13263, 13264, 13265, -1, 13266, 13267, 13268, -1, 13269, 13270, 13271, -1, 13272, 13273, 13274, -1, 13275, 13276, 13277, -1, 13278, 13279, 13280, -1, 13281, 13282, 13283, -1, 13284, 13285, 13286, -1, 13287, 13288, 13289, -1, 13290, 13291, 13292, -1, 13293, 13294, 13295, -1, 13296, 13297, 13298, -1, 13299, 13300, 13301, -1, 13302, 13303, 13304, -1, 13305, 13306, 13307, -1, 13308, 13309, 13310, -1, 13311, 13312, 13313, -1, 13314, 13315, 13316, -1, 13317, 13318, 13319, -1, 13320, 13321, 13322, -1, 13323, 13324, 13325, -1, 13326, 13327, 13328, -1, 13329, 13330, 13331, -1, 13332, 13333, 13334, -1, 13335, 13336, 13337, -1, 13338, 13339, 13340, -1, 13341, 13342, 13343, -1, 13344, 13345, 13346, -1, 13347, 13348, 13349, -1, 13350, 13351, 13352, -1, 13353, 13354, 13355, -1, 13356, 13357, 13358, -1, 13359, 13360, 13361, -1, 13362, 13363, 13364, -1, 13365, 13366, 13367, -1, 13368, 13369, 13370, -1, 13371, 13372, 13373, -1, 13374, 13375, 13376, -1, 13377, 13378, 13379, -1, 13380, 13381, 13382, -1, 13383, 13384, 13385, -1, 13386, 13387, 13388, -1, 13389, 13390, 13391, -1, 13392, 13393, 13394, -1, 13395, 13396, 13397, -1, 13398, 13399, 13400, -1, 13401, 13402, 13403, -1, 13404, 13405, 13406, -1, 13407, 13408, 13409, -1, 13410, 13411, 13412, -1, 13413, 13414, 13415, -1, 13416, 13417, 13418, -1, 13419, 13420, 13421, -1, 13422, 13423, 13424, -1, 13425, 13426, 13427, -1, 13428, 13429, 13430, -1, 13431, 13432, 13433, -1, 13434, 13435, 13436, -1, 13437, 13438, 13439, -1, 13440, 13441, 13442, -1, 13443, 13444, 13445, -1, 13446, 13447, 13448, -1, 13449, 13450, 13451, -1, 13452, 13453, 13454, -1, 13455, 13456, 13457, -1, 13458, 13459, 13460, -1, 13461, 13462, 13463, -1, 13464, 13465, 13466, -1, 13467, 13468, 13469, -1, 13470, 13471, 13472, -1, 13473, 13474, 13475, -1, 13476, 13477, 13478, -1, 13479, 13480, 13481, -1, 13482, 13483, 13484, -1, 13485, 13486, 13487, -1, 13488, 13489, 13490, -1, 13491, 13492, 13493, -1, 13494, 13495, 13496, -1, 13497, 13498, 13499, -1, 13500, 13501, 13502, -1, 13503, 13504, 13505, -1, 13506, 13507, 13508, -1, 13509, 13510, 13511, -1, 13512, 13513, 13514, -1, 13515, 13516, 13517, -1, 13518, 13519, 13520, -1, 13521, 13522, 13523, -1, 13524, 13525, 13526, -1, 13527, 13528, 13529, -1, 13530, 13531, 13532, -1, 13533, 13534, 13535, -1, 13536, 13537, 13538, -1, 13539, 13540, 13541, -1, 13542, 13543, 13544, -1, 13545, 13546, 13547, -1, 13548, 13549, 13550, -1, 13551, 13552, 13553, -1, 13554, 13555, 13556, -1, 13557, 13558, 13559, -1, 13560, 13561, 13562, -1, 13563, 13564, 13565, -1, 13566, 13567, 13568, -1, 13569, 13570, 13571, -1, 13572, 13573, 13574, -1, 13575, 13576, 13577, -1, 13578, 13579, 13580, -1, 13581, 13582, 13583, -1, 13584, 13585, 13586, -1, 13587, 13588, 13589, -1, 13590, 13591, 13592, -1, 13593, 13594, 13595, -1, 13596, 13597, 13598, -1, 13599, 13600, 13601, -1, 13602, 13603, 13604, -1, 13605, 13606, 13607, -1, 13608, 13609, 13610, -1, 13611, 13612, 13613, -1, 13614, 13615, 13616, -1, 13617, 13618, 13619, -1, 13620, 13621, 13622, -1, 13623, 13624, 13625, -1, 13626, 13627, 13628, -1, 13629, 13630, 13631, -1, 13632, 13633, 13634, -1, 13635, 13636, 13637, -1, 13638, 13639, 13640, -1, 13641, 13642, 13643, -1, 13644, 13645, 13646, -1, 13647, 13648, 13649, -1, 13650, 13651, 13652, -1, 13653, 13654, 13655, -1, 13656, 13657, 13658, -1, 13659, 13660, 13661, -1, 13662, 13663, 13664, -1, 13665, 13666, 13667, -1, 13668, 13669, 13670, -1, 13671, 13672, 13673, -1, 13674, 13675, 13676, -1, 13677, 13678, 13679, -1, 13680, 13681, 13682, -1, 13683, 13684, 13685, -1, 13686, 13687, 13688, -1, 13689, 13690, 13691, -1, 13692, 13693, 13694, -1, 13695, 13696, 13697, -1, 13698, 13699, 13700, -1, 13701, 13702, 13703, -1, 13704, 13705, 13706, -1, 13707, 13708, 13709, -1, 13710, 13711, 13712, -1, 13713, 13714, 13715, -1, 13716, 13717, 13718, -1, 13719, 13720, 13721, -1, 13722, 13723, 13724, -1, 13725, 13726, 13727, -1, 13728, 13729, 13730, -1, 13731, 13732, 13733, -1, 13734, 13735, 13736, -1, 13737, 13738, 13739, -1, 13740, 13741, 13742, -1, 13743, 13744, 13745, -1, 13746, 13747, 13748, -1, 13749, 13750, 13751, -1, 13752, 13753, 13754, -1, 13755, 13756, 13757, -1, 13758, 13759, 13760, -1, 13761, 13762, 13763, -1, 13764, 13765, 13766, -1, 13767, 13768, 13769, -1, 13770, 13771, 13772, -1, 13773, 13774, 13775, -1, 13776, 13777, 13778, -1, 13779, 13780, 13781, -1, 13782, 13783, 13784, -1, 13785, 13786, 13787, -1, 13788, 13789, 13790, -1, 13791, 13792, 13793, -1, 13794, 13795, 13796, -1, 13797, 13798, 13799, -1, 13800, 13801, 13802, -1, 13803, 13804, 13805, -1, 13806, 13807, 13808, -1, 13809, 13810, 13811, -1, 13812, 13813, 13814, -1, 13815, 13816, 13817, -1, 13818, 13819, 13820, -1, 13821, 13822, 13823, -1, 13824, 13825, 13826, -1, 13827, 13828, 13829, -1, 13830, 13831, 13832, -1, 13833, 13834, 13835, -1, 13836, 13837, 13838, -1, 13839, 13840, 13841, -1, 13842, 13843, 13844, -1, 13845, 13846, 13847, -1, 13848, 13849, 13850, -1, 13851, 13852, 13853, -1, 13854, 13855, 13856, -1, 13857, 13858, 13859, -1, 13860, 13861, 13862, -1, 13863, 13864, 13865, -1, 13866, 13867, 13868, -1, 13869, 13870, 13871, -1, 13872, 13873, 13874, -1, 13875, 13876, 13877, -1, 13878, 13879, 13880, -1, 13881, 13882, 13883, -1, 13884, 13885, 13886, -1, 13887, 13888, 13889, -1, 13890, 13891, 13892, -1, 13893, 13894, 13895, -1, 13896, 13897, 13898, -1, 13899, 13900, 13901, -1, 13902, 13903, 13904, -1, 13905, 13906, 13907, -1, 13908, 13909, 13910, -1, 13911, 13912, 13913, -1, 13914, 13915, 13916, -1, 13917, 13918, 13919, -1, 13920, 13921, 13922, -1, 13923, 13924, 13925, -1, 13926, 13927, 13928, -1, 13929, 13930, 13931, -1, 13932, 13933, 13934, -1, 13935, 13936, 13937, -1, 13938, 13939, 13940, -1, 13941, 13942, 13943, -1, 13944, 13945, 13946, -1, 13947, 13948, 13949, -1, 13950, 13951, 13952, -1, 13953, 13954, 13955, -1, 13956, 13957, 13958, -1, 13959, 13960, 13961, -1, 13962, 13963, 13964, -1, 13965, 13966, 13967, -1, 13968, 13969, 13970, -1, 13971, 13972, 13973, -1, 13974, 13975, 13976, -1, 13977, 13978, 13979, -1, 13980, 13981, 13982, -1, 13983, 13984, 13985, -1, 13986, 13987, 13988, -1, 13989, 13990, 13991, -1, 13992, 13993, 13994, -1, 13995, 13996, 13997, -1, 13998, 13999, 14000, -1, 14001, 14002, 14003, -1, 14004, 14005, 14006, -1, 14007, 14008, 14009, -1, 14010, 14011, 14012, -1, 14013, 14014, 14015, -1, 14016, 14017, 14018, -1, 14019, 14020, 14021, -1, 14022, 14023, 14024, -1, 14025, 14026, 14027, -1, 14028, 14029, 14030, -1, 14031, 14032, 14033, -1, 14034, 14035, 14036, -1, 14037, 14038, 14039, -1, 14040, 14041, 14042, -1, 14043, 14044, 14045, -1, 14046, 14047, 14048, -1, 14049, 14050, 14051, -1, 14052, 14053, 14054, -1, 14055, 14056, 14057, -1, 14058, 14059, 14060, -1, 14061, 14062, 14063, -1, 14064, 14065, 14066, -1, 14067, 14068, 14069, -1, 14070, 14071, 14072, -1, 14073, 14074, 14075, -1, 14076, 14077, 14078, -1, 14079, 14080, 14081, -1, 14082, 14083, 14084, -1, 14085, 14086, 14087, -1, 14088, 14089, 14090, -1, 14091, 14092, 14093, -1, 14094, 14095, 14096, -1, 14097, 14098, 14099, -1, 14100, 14101, 14102, -1, 14103, 14104, 14105, -1, 14106, 14107, 14108, -1, 14109, 14110, 14111, -1, 14112, 14113, 14114, -1, 14115, 14116, 14117, -1, 14118, 14119, 14120, -1, 14121, 14122, 14123, -1, 14124, 14125, 14126, -1, 14127, 14128, 14129, -1, 14130, 14131, 14132, -1, 14133, 14134, 14135, -1, 14136, 14137, 14138, -1, 14139, 14140, 14141, -1, 14142, 14143, 14144, -1, 14145, 14146, 14147, -1, 14148, 14149, 14150, -1, 14151, 14152, 14153, -1, 14154, 14155, 14156, -1, 14157, 14158, 14159, -1, 14160, 14161, 14162, -1, 14163, 14164, 14165, -1, 14166, 14167, 14168, -1, 14169, 14170, 14171, -1, 14172, 14173, 14174, -1, 14175, 14176, 14177, -1, 14178, 14179, 14180, -1, 14181, 14182, 14183, -1, 14184, 14185, 14186, -1, 14187, 14188, 14189, -1, 14190, 14191, 14192, -1, 14193, 14194, 14195, -1, 14196, 14197, 14198, -1, 14199, 14200, 14201, -1, 14202, 14203, 14204, -1, 14205, 14206, 14207, -1, 14208, 14209, 14210, -1, 14211, 14212, 14213, -1, 14214, 14215, 14216, -1, 14217, 14218, 14219, -1, 14220, 14221, 14222, -1, 14223, 14224, 14225, -1, 14226, 14227, 14228, -1, 14229, 14230, 14231, -1, 14232, 14233, 14234, -1, 14235, 14236, 14237, -1, 14238, 14239, 14240, -1, 14241, 14242, 14243, -1, 14244, 14245, 14246, -1, 14247, 14248, 14249, -1, 14250, 14251, 14252, -1, 14253, 14254, 14255, -1, 14256, 14257, 14258, -1, 14259, 14260, 14261, -1, 14262, 14263, 14264, -1, 14265, 14266, 14267, -1, 14268, 14269, 14270, -1, 14271, 14272, 14273, -1, 14274, 14275, 14276, -1, 14277, 14278, 14279, -1, 14280, 14281, 14282, -1, 14283, 14284, 14285, -1, 14286, 14287, 14288, -1, 14289, 14290, 14291, -1, 14292, 14293, 14294, -1, 14295, 14296, 14297, -1, 14298, 14299, 14300, -1, 14301, 14302, 14303, -1, 14304, 14305, 14306, -1, 14307, 14308, 14309, -1, 14310, 14311, 14312, -1, 14313, 14314, 14315, -1, 14316, 14317, 14318, -1, 14319, 14320, 14321, -1, 14322, 14323, 14324, -1, 14325, 14326, 14327, -1, 14328, 14329, 14330, -1, 14331, 14332, 14333, -1, 14334, 14335, 14336, -1, 14337, 14338, 14339, -1, 14340, 14341, 14342, -1, 14343, 14344, 14345, -1, 14346, 14347, 14348, -1, 14349, 14350, 14351, -1, 14352, 14353, 14354, -1, 14355, 14356, 14357, -1, 14358, 14359, 14360, -1, 14361, 14362, 14363, -1, 14364, 14365, 14366, -1, 14367, 14368, 14369, -1, 14370, 14371, 14372, -1, 14373, 14374, 14375, -1, 14376, 14377, 14378, -1, 14379, 14380, 14381, -1, 14382, 14383, 14384, -1, 14385, 14386, 14387, -1, 14388, 14389, 14390, -1, 14391, 14392, 14393, -1, 14394, 14395, 14396, -1, 14397, 14398, 14399, -1, 14400, 14401, 14402, -1, 14403, 14404, 14405, -1, 14406, 14407, 14408, -1, 14409, 14410, 14411, -1, 14412, 14413, 14414, -1, 14415, 14416, 14417, -1, 14418, 14419, 14420, -1, 14421, 14422, 14423, -1, 14424, 14425, 14426, -1, 14427, 14428, 14429, -1, 14430, 14431, 14432, -1, 14433, 14434, 14435, -1, 14436, 14437, 14438, -1, 14439, 14440, 14441, -1, 14442, 14443, 14444, -1, 14445, 14446, 14447, -1, 14448, 14449, 14450, -1, 14451, 14452, 14453, -1, 14454, 14455, 14456, -1, 14457, 14458, 14459, -1, 14460, 14461, 14462, -1, 14463, 14464, 14465, -1, 14466, 14467, 14468, -1, 14469, 14470, 14471, -1, 14472, 14473, 14474, -1, 14475, 14476, 14477, -1, 14478, 14479, 14480, -1, 14481, 14482, 14483, -1, 14484, 14485, 14486, -1, 14487, 14488, 14489, -1, 14490, 14491, 14492, -1, 14493, 14494, 14495, -1, 14496, 14497, 14498, -1, 14499, 14500, 14501, -1, 14502, 14503, 14504, -1, 14505, 14506, 14507, -1, 14508, 14509, 14510, -1, 14511, 14512, 14513, -1, 14514, 14515, 14516, -1, 14517, 14518, 14519, -1, 14520, 14521, 14522, -1, 14523, 14524, 14525, -1, 14526, 14527, 14528, -1, 14529, 14530, 14531, -1, 14532, 14533, 14534, -1, 14535, 14536, 14537, -1, 14538, 14539, 14540, -1, 14541, 14542, 14543, -1, 14544, 14545, 14546, -1, 14547, 14548, 14549, -1, 14550, 14551, 14552, -1, 14553, 14554, 14555, -1, 14556, 14557, 14558, -1, 14559, 14560, 14561, -1, 14562, 14563, 14564, -1, 14565, 14566, 14567, -1, 14568, 14569, 14570, -1, 14571, 14572, 14573, -1, 14574, 14575, 14576, -1, 14577, 14578, 14579, -1, 14580, 14581, 14582, -1, 14583, 14584, 14585, -1, 14586, 14587, 14588, -1, 14589, 14590, 14591, -1, 14592, 14593, 14594, -1, 14595, 14596, 14597, -1, 14598, 14599, 14600, -1, 14601, 14602, 14603, -1, 14604, 14605, 14606, -1, 14607, 14608, 14609, -1, 14610, 14611, 14612, -1, 14613, 14614, 14615, -1, 14616, 14617, 14618, -1, 14619, 14620, 14621, -1, 14622, 14623, 14624, -1, 14625, 14626, 14627, -1, 14628, 14629, 14630, -1, 14631, 14632, 14633, -1, 14634, 14635, 14636, -1, 14637, 14638, 14639, -1, 14640, 14641, 14642, -1, 14643, 14644, 14645, -1, 14646, 14647, 14648, -1, 14649, 14650, 14651, -1, 14652, 14653, 14654, -1, 14655, 14656, 14657, -1, 14658, 14659, 14660, -1, 14661, 14662, 14663, -1, 14664, 14665, 14666, -1, 14667, 14668, 14669, -1, 14670, 14671, 14672, -1, 14673, 14674, 14675, -1, 14676, 14677, 14678, -1, 14679, 14680, 14681, -1, 14682, 14683, 14684, -1, 14685, 14686, 14687, -1, 14688, 14689, 14690, -1, 14691, 14692, 14693, -1, 14694, 14695, 14696, -1, 14697, 14698, 14699, -1, 14700, 14701, 14702, -1, 14703, 14704, 14705, -1, 14706, 14707, 14708, -1, 14709, 14710, 14711, -1, 14712, 14713, 14714, -1, 14715, 14716, 14717, -1, 14718, 14719, 14720, -1, 14721, 14722, 14723, -1, 14724, 14725, 14726, -1, 14727, 14728, 14729, -1, 14730, 14731, 14732, -1, 14733, 14734, 14735, -1, 14736, 14737, 14738, -1, 14739, 14740, 14741, -1, 14742, 14743, 14744, -1, 14745, 14746, 14747, -1, 14748, 14749, 14750, -1, 14751, 14752, 14753, -1, 14754, 14755, 14756, -1, 14757, 14758, 14759, -1, 14760, 14761, 14762, -1, 14763, 14764, 14765, -1, 14766, 14767, 14768, -1, 14769, 14770, 14771, -1, 14772, 14773, 14774, -1, 14775, 14776, 14777, -1, 14778, 14779, 14780, -1, 14781, 14782, 14783, -1, 14784, 14785, 14786, -1, 14787, 14788, 14789, -1, 14790, 14791, 14792, -1, 14793, 14794, 14795, -1, 14796, 14797, 14798, -1, 14799, 14800, 14801, -1, 14802, 14803, 14804, -1, 14805, 14806, 14807, -1, 14808, 14809, 14810, -1, 14811, 14812, 14813, -1, 14814, 14815, 14816, -1, 14817, 14818, 14819, -1, 14820, 14821, 14822, -1, 14823, 14824, 14825, -1, 14826, 14827, 14828, -1, 14829, 14830, 14831, -1, 14832, 14833, 14834, -1, 14835, 14836, 14837, -1, 14838, 14839, 14840, -1, 14841, 14842, 14843, -1, 14844, 14845, 14846, -1, 14847, 14848, 14849, -1, 14850, 14851, 14852, -1, 14853, 14854, 14855, -1, 14856, 14857, 14858, -1, 14859, 14860, 14861, -1, 14862, 14863, 14864, -1, 14865, 14866, 14867, -1, 14868, 14869, 14870, -1, 14871, 14872, 14873, -1, 14874, 14875, 14876, -1, 14877, 14878, 14879, -1, 14880, 14881, 14882, -1, 14883, 14884, 14885, -1, 14886, 14887, 14888, -1, 14889, 14890, 14891, -1, 14892, 14893, 14894, -1, 14895, 14896, 14897, -1, 14898, 14899, 14900, -1, 14901, 14902, 14903, -1, 14904, 14905, 14906, -1, 14907, 14908, 14909, -1, 14910, 14911, 14912, -1, 14913, 14914, 14915, -1, 14916, 14917, 14918, -1, 14919, 14920, 14921, -1, 14922, 14923, 14924, -1, 14925, 14926, 14927, -1, 14928, 14929, 14930, -1, 14931, 14932, 14933, -1, 14934, 14935, 14936, -1, 14937, 14938, 14939, -1, 14940, 14941, 14942, -1, 14943, 14944, 14945, -1, 14946, 14947, 14948, -1, 14949, 14950, 14951, -1, 14952, 14953, 14954, -1, 14955, 14956, 14957, -1, 14958, 14959, 14960, -1, 14961, 14962, 14963, -1, 14964, 14965, 14966, -1, 14967, 14968, 14969, -1, 14970, 14971, 14972, -1, 14973, 14974, 14975, -1, 14976, 14977, 14978, -1, 14979, 14980, 14981, -1, 14982, 14983, 14984, -1, 14985, 14986, 14987, -1, 14988, 14989, 14990, -1, 14991, 14992, 14993, -1, 14994, 14995, 14996, -1, 14997, 14998, 14999, -1, 15000, 15001, 15002, -1, 15003, 15004, 15005, -1, 15006, 15007, 15008, -1, 15009, 15010, 15011, -1, 15012, 15013, 15014, -1, 15015, 15016, 15017, -1, 15018, 15019, 15020, -1, 15021, 15022, 15023, -1, 15024, 15025, 15026, -1, 15027, 15028, 15029, -1, 15030, 15031, 15032, -1, 15033, 15034, 15035, -1, 15036, 15037, 15038, -1, 15039, 15040, 15041, -1, 15042, 15043, 15044, -1, 15045, 15046, 15047, -1, 15048, 15049, 15050, -1, 15051, 15052, 15053, -1, 15054, 15055, 15056, -1, 15057, 15058, 15059, -1, 15060, 15061, 15062, -1, 15063, 15064, 15065, -1, 15066, 15067, 15068, -1, 15069, 15070, 15071, -1, 15072, 15073, 15074, -1, 15075, 15076, 15077, -1, 15078, 15079, 15080, -1, 15081, 15082, 15083, -1, 15084, 15085, 15086, -1, 15087, 15088, 15089, -1, 15090, 15091, 15092, -1, 15093, 15094, 15095, -1, 15096, 15097, 15098, -1, 15099, 15100, 15101, -1, 15102, 15103, 15104, -1, 15105, 15106, 15107, -1, 15108, 15109, 15110, -1, 15111, 15112, 15113, -1, 15114, 15115, 15116, -1, 15117, 15118, 15119, -1, 15120, 15121, 15122, -1, 15123, 15124, 15125, -1, 15126, 15127, 15128, -1, 15129, 15130, 15131, -1, 15132, 15133, 15134, -1, 15135, 15136, 15137, -1, 15138, 15139, 15140, -1, 15141, 15142, 15143, -1, 15144, 15145, 15146, -1, 15147, 15148, 15149, -1, 15150, 15151, 15152, -1, 15153, 15154, 15155, -1, 15156, 15157, 15158, -1, 15159, 15160, 15161, -1, 15162, 15163, 15164, -1, 15165, 15166, 15167, -1, 15168, 15169, 15170, -1, 15171, 15172, 15173, -1, 15174, 15175, 15176, -1, 15177, 15178, 15179, -1, 15180, 15181, 15182, -1, 15183, 15184, 15185, -1, 15186, 15187, 15188, -1, 15189, 15190, 15191, -1, 15192, 15193, 15194, -1, 15195, 15196, 15197, -1, 15198, 15199, 15200, -1, 15201, 15202, 15203, -1, 15204, 15205, 15206, -1, 15207, 15208, 15209, -1, 15210, 15211, 15212, -1, 15213, 15214, 15215, -1, 15216, 15217, 15218, -1, 15219, 15220, 15221, -1, 15222, 15223, 15224, -1, 15225, 15226, 15227, -1, 15228, 15229, 15230, -1, 15231, 15232, 15233, -1, 15234, 15235, 15236, -1, 15237, 15238, 15239, -1, 15240, 15241, 15242, -1, 15243, 15244, 15245, -1, 15246, 15247, 15248, -1, 15249, 15250, 15251, -1, 15252, 15253, 15254, -1, 15255, 15256, 15257, -1, 15258, 15259, 15260, -1, 15261, 15262, 15263, -1, 15264, 15265, 15266, -1, 15267, 15268, 15269, -1, 15270, 15271, 15272, -1, 15273, 15274, 15275, -1, 15276, 15277, 15278, -1, 15279, 15280, 15281, -1, 15282, 15283, 15284, -1, 15285, 15286, 15287, -1, 15288, 15289, 15290, -1, 15291, 15292, 15293, -1, 15294, 15295, 15296, -1, 15297, 15298, 15299, -1, 15300, 15301, 15302, -1, 15303, 15304, 15305, -1, 15306, 15307, 15308, -1, 15309, 15310, 15311, -1, 15312, 15313, 15314, -1, 15315, 15316, 15317, -1, 15318, 15319, 15320, -1, 15321, 15322, 15323, -1, 15324, 15325, 15326, -1, 15327, 15328, 15329, -1, 15330, 15331, 15332, -1, 15333, 15334, 15335, -1, 15336, 15337, 15338, -1, 15339, 15340, 15341, -1, 15342, 15343, 15344, -1, 15345, 15346, 15347, -1, 15348, 15349, 15350, -1, 15351, 15352, 15353, -1, 15354, 15355, 15356, -1, 15357, 15358, 15359, -1, 15360, 15361, 15362, -1, 15363, 15364, 15365, -1, 15366, 15367, 15368, -1, 15369, 15370, 15371, -1, 15372, 15373, 15374, -1, 15375, 15376, 15377, -1, 15378, 15379, 15380, -1, 15381, 15382, 15383, -1, 15384, 15385, 15386, -1, 15387, 15388, 15389, -1, 15390, 15391, 15392, -1, 15393, 15394, 15395, -1, 15396, 15397, 15398, -1, 15399, 15400, 15401, -1, 15402, 15403, 15404, -1, 15405, 15406, 15407, -1, 15408, 15409, 15410, -1, 15411, 15412, 15413, -1, 15414, 15415, 15416, -1, 15417, 15418, 15419, -1, 15420, 15421, 15422, -1, 15423, 15424, 15425, -1, 15426, 15427, 15428, -1, 15429, 15430, 15431, -1, 15432, 15433, 15434, -1, 15435, 15436, 15437, -1, 15438, 15439, 15440, -1, 15441, 15442, 15443, -1, 15444, 15445, 15446, -1, 15447, 15448, 15449, -1, 15450, 15451, 15452, -1, 15453, 15454, 15455, -1, 15456, 15457, 15458, -1, 15459, 15460, 15461, -1, 15462, 15463, 15464, -1, 15465, 15466, 15467, -1, 15468, 15469, 15470, -1, 15471, 15472, 15473, -1, 15474, 15475, 15476, -1, 15477, 15478, 15479, -1, 15480, 15481, 15482, -1, 15483, 15484, 15485, -1, 15486, 15487, 15488, -1, 15489, 15490, 15491, -1, 15492, 15493, 15494, -1, 15495, 15496, 15497, -1, 15498, 15499, 15500, -1, 15501, 15502, 15503, -1, 15504, 15505, 15506, -1, 15507, 15508, 15509, -1, 15510, 15511, 15512, -1, 15513, 15514, 15515, -1, 15516, 15517, 15518, -1, 15519, 15520, 15521, -1, 15522, 15523, 15524, -1, 15525, 15526, 15527, -1, 15528, 15529, 15530, -1, 15531, 15532, 15533, -1, 15534, 15535, 15536, -1, 15537, 15538, 15539, -1, 15540, 15541, 15542, -1, 15543, 15544, 15545, -1, 15546, 15547, 15548, -1, 15549, 15550, 15551, -1, 15552, 15553, 15554, -1, 15555, 15556, 15557, -1, 15558, 15559, 15560, -1, 15561, 15562, 15563, -1, 15564, 15565, 15566, -1, 15567, 15568, 15569, -1, 15570, 15571, 15572, -1, 15573, 15574, 15575, -1, 15576, 15577, 15578, -1, 15579, 15580, 15581, -1, 15582, 15583, 15584, -1, 15585, 15586, 15587, -1, 15588, 15589, 15590, -1, 15591, 15592, 15593, -1, 15594, 15595, 15596, -1, 15597, 15598, 15599, -1, 15600, 15601, 15602, -1, 15603, 15604, 15605, -1, 15606, 15607, 15608, -1, 15609, 15610, 15611, -1, 15612, 15613, 15614, -1, 15615, 15616, 15617, -1, 15618, 15619, 15620, -1, 15621, 15622, 15623, -1, 15624, 15625, 15626, -1, 15627, 15628, 15629, -1, 15630, 15631, 15632, -1, 15633, 15634, 15635, -1, 15636, 15637, 15638, -1, 15639, 15640, 15641, -1, 15642, 15643, 15644, -1, 15645, 15646, 15647, -1, 15648, 15649, 15650, -1, 15651, 15652, 15653, -1, 15654, 15655, 15656, -1, 15657, 15658, 15659, -1, 15660, 15661, 15662, -1, 15663, 15664, 15665, -1, 15666, 15667, 15668, -1, 15669, 15670, 15671, -1, 15672, 15673, 15674, -1, 15675, 15676, 15677, -1, 15678, 15679, 15680, -1, 15681, 15682, 15683, -1, 15684, 15685, 15686, -1, 15687, 15688, 15689, -1, 15690, 15691, 15692, -1, 15693, 15694, 15695, -1, 15696, 15697, 15698, -1, 15699, 15700, 15701, -1, 15702, 15703, 15704, -1, 15705, 15706, 15707, -1, 15708, 15709, 15710, -1, 15711, 15712, 15713, -1, 15714, 15715, 15716, -1, 15717, 15718, 15719, -1, 15720, 15721, 15722, -1, 15723, 15724, 15725, -1, 15726, 15727, 15728, -1, 15729, 15730, 15731, -1, 15732, 15733, 15734, -1, 15735, 15736, 15737, -1, 15738, 15739, 15740, -1, 15741, 15742, 15743, -1, 15744, 15745, 15746, -1, 15747, 15748, 15749, -1, 15750, 15751, 15752, -1, 15753, 15754, 15755, -1, 15756, 15757, 15758, -1, 15759, 15760, 15761, -1, 15762, 15763, 15764, -1, 15765, 15766, 15767, -1, 15768, 15769, 15770, -1, 15771, 15772, 15773, -1, 15774, 15775, 15776, -1, 15777, 15778, 15779, -1, 15780, 15781, 15782, -1, 15783, 15784, 15785, -1, 15786, 15787, 15788, -1, 15789, 15790, 15791, -1, 15792, 15793, 15794, -1, 15795, 15796, 15797, -1, 15798, 15799, 15800, -1, 15801, 15802, 15803, -1, 15804, 15805, 15806, -1, 15807, 15808, 15809, -1, 15810, 15811, 15812, -1, 15813, 15814, 15815, -1, 15816, 15817, 15818, -1, 15819, 15820, 15821, -1, 15822, 15823, 15824, -1, 15825, 15826, 15827, -1, 15828, 15829, 15830, -1, 15831, 15832, 15833, -1, 15834, 15835, 15836, -1, 15837, 15838, 15839, -1, 15840, 15841, 15842, -1, 15843, 15844, 15845, -1, 15846, 15847, 15848, -1, 15849, 15850, 15851, -1, 15852, 15853, 15854, -1, 15855, 15856, 15857, -1, 15858, 15859, 15860, -1, 15861, 15862, 15863, -1, 15864, 15865, 15866, -1, 15867, 15868, 15869, -1, 15870, 15871, 15872, -1, 15873, 15874, 15875, -1, 15876, 15877, 15878, -1, 15879, 15880, 15881, -1, 15882, 15883, 15884, -1, 15885, 15886, 15887, -1, 15888, 15889, 15890, -1, 15891, 15892, 15893, -1, 15894, 15895, 15896, -1, 15897, 15898, 15899, -1, 15900, 15901, 15902, -1, 15903, 15904, 15905, -1, 15906, 15907, 15908, -1, 15909, 15910, 15911, -1, 15912, 15913, 15914, -1, 15915, 15916, 15917, -1, 15918, 15919, 15920, -1, 15921, 15922, 15923, -1, 15924, 15925, 15926, -1, 15927, 15928, 15929, -1, 15930, 15931, 15932, -1, 15933, 15934, 15935, -1, 15936, 15937, 15938, -1, 15939, 15940, 15941, -1, 15942, 15943, 15944, -1, 15945, 15946, 15947, -1, 15948, 15949, 15950, -1, 15951, 15952, 15953, -1, 15954, 15955, 15956, -1, 15957, 15958, 15959, -1, 15960, 15961, 15962, -1, 15963, 15964, 15965, -1, 15966, 15967, 15968, -1, 15969, 15970, 15971, -1, 15972, 15973, 15974, -1, 15975, 15976, 15977, -1, 15978, 15979, 15980, -1, 15981, 15982, 15983, -1, 15984, 15985, 15986, -1, 15987, 15988, 15989, -1, 15990, 15991, 15992, -1, 15993, 15994, 15995, -1, 15996, 15997, 15998, -1, 15999, 16000, 16001, -1, 16002, 16003, 16004, -1, 16005, 16006, 16007, -1, 16008, 16009, 16010, -1, 16011, 16012, 16013, -1, 16014, 16015, 16016, -1, 16017, 16018, 16019, -1, 16020, 16021, 16022, -1, 16023, 16024, 16025, -1, 16026, 16027, 16028, -1, 16029, 16030, 16031, -1, 16032, 16033, 16034, -1, 16035, 16036, 16037, -1, 16038, 16039, 16040, -1, 16041, 16042, 16043, -1, 16044, 16045, 16046, -1, 16047, 16048, 16049, -1, 16050, 16051, 16052, -1, 16053, 16054, 16055, -1, 16056, 16057, 16058, -1, 16059, 16060, 16061, -1, 16062, 16063, 16064, -1, 16065, 16066, 16067, -1, 16068, 16069, 16070, -1, 16071, 16072, 16073, -1, 16074, 16075, 16076, -1, 16077, 16078, 16079, -1, 16080, 16081, 16082, -1, 16083, 16084, 16085, -1, 16086, 16087, 16088, -1, 16089, 16090, 16091, -1, 16092, 16093, 16094, -1, 16095, 16096, 16097, -1, 16098, 16099, 16100, -1, 16101, 16102, 16103, -1, 16104, 16105, 16106, -1, 16107, 16108, 16109, -1, 16110, 16111, 16112, -1, 16113, 16114, 16115, -1, 16116, 16117, 16118, -1, 16119, 16120, 16121, -1, 16122, 16123, 16124, -1, 16125, 16126, 16127, -1, 16128, 16129, 16130, -1, 16131, 16132, 16133, -1, 16134, 16135, 16136, -1, 16137, 16138, 16139, -1, 16140, 16141, 16142, -1, 16143, 16144, 16145, -1, 16146, 16147, 16148, -1, 16149, 16150, 16151, -1, 16152, 16153, 16154, -1, 16155, 16156, 16157, -1, 16158, 16159, 16160, -1, 16161, 16162, 16163, -1, 16164, 16165, 16166, -1, 16167, 16168, 16169, -1, 16170, 16171, 16172, -1, 16173, 16174, 16175, -1, 16176, 16177, 16178, -1, 16179, 16180, 16181, -1, 16182, 16183, 16184, -1, 16185, 16186, 16187, -1, 16188, 16189, 16190, -1, 16191, 16192, 16193, -1, 16194, 16195, 16196, -1, 16197, 16198, 16199, -1, 16200, 16201, 16202, -1, 16203, 16204, 16205, -1, 16206, 16207, 16208, -1, 16209, 16210, 16211, -1, 16212, 16213, 16214, -1, 16215, 16216, 16217, -1, 16218, 16219, 16220, -1, 16221, 16222, 16223, -1, 16224, 16225, 16226, -1, 16227, 16228, 16229, -1, 16230, 16231, 16232, -1, 16233, 16234, 16235, -1, 16236, 16237, 16238, -1, 16239, 16240, 16241, -1, 16242, 16243, 16244, -1, 16245, 16246, 16247, -1, 16248, 16249, 16250, -1, 16251, 16252, 16253, -1, 16254, 16255, 16256, -1, 16257, 16258, 16259, -1, 16260, 16261, 16262, -1, 16263, 16264, 16265, -1, 16266, 16267, 16268, -1, 16269, 16270, 16271, -1, 16272, 16273, 16274, -1, 16275, 16276, 16277, -1, 16278, 16279, 16280, -1, 16281, 16282, 16283, -1, 16284, 16285, 16286, -1, 16287, 16288, 16289, -1, 16290, 16291, 16292, -1, 16293, 16294, 16295, -1, 16296, 16297, 16298, -1, 16299, 16300, 16301, -1, 16302, 16303, 16304, -1, 16305, 16306, 16307, -1, 16308, 16309, 16310, -1, 16311, 16312, 16313, -1, 16314, 16315, 16316, -1, 16317, 16318, 16319, -1, 16320, 16321, 16322, -1, 16323, 16324, 16325, -1, 16326, 16327, 16328, -1, 16329, 16330, 16331, -1, 16332, 16333, 16334, -1, 16335, 16336, 16337, -1, 16338, 16339, 16340, -1, 16341, 16342, 16343, -1, 16344, 16345, 16346, -1, 16347, 16348, 16349, -1, 16350, 16351, 16352, -1, 16353, 16354, 16355, -1, 16356, 16357, 16358, -1, 16359, 16360, 16361, -1, 16362, 16363, 16364, -1, 16365, 16366, 16367, -1, 16368, 16369, 16370, -1, 16371, 16372, 16373, -1, 16374, 16375, 16376, -1, 16377, 16378, 16379, -1, 16380, 16381, 16382, -1, 16383, 16384, 16385, -1, 16386, 16387, 16388, -1, 16389, 16390, 16391, -1, 16392, 16393, 16394, -1, 16395, 16396, 16397, -1, 16398, 16399, 16400, -1, 16401, 16402, 16403, -1, 16404, 16405, 16406, -1, 16407, 16408, 16409, -1, 16410, 16411, 16412, -1, 16413, 16414, 16415, -1, 16416, 16417, 16418, -1, 16419, 16420, 16421, -1, 16422, 16423, 16424, -1, 16425, 16426, 16427, -1, 16428, 16429, 16430, -1, 16431, 16432, 16433, -1, 16434, 16435, 16436, -1, 16437, 16438, 16439, -1, 16440, 16441, 16442, -1, 16443, 16444, 16445, -1, 16446, 16447, 16448, -1, 16449, 16450, 16451, -1, 16452, 16453, 16454, -1, 16455, 16456, 16457, -1, 16458, 16459, 16460, -1, 16461, 16462, 16463, -1, 16464, 16465, 16466, -1, 16467, 16468, 16469, -1, 16470, 16471, 16472, -1, 16473, 16474, 16475, -1, 16476, 16477, 16478, -1, 16479, 16480, 16481, -1, 16482, 16483, 16484, -1, 16485, 16486, 16487, -1, 16488, 16489, 16490, -1, 16491, 16492, 16493, -1, 16494, 16495, 16496, -1, 16497, 16498, 16499, -1, 16500, 16501, 16502, -1, 16503, 16504, 16505, -1, 16506, 16507, 16508, -1, 16509, 16510, 16511, -1, 16512, 16513, 16514, -1, 16515, 16516, 16517, -1, 16518, 16519, 16520, -1, 16521, 16522, 16523, -1, 16524, 16525, 16526, -1, 16527, 16528, 16529, -1, 16530, 16531, 16532, -1, 16533, 16534, 16535, -1, 16536, 16537, 16538, -1, 16539, 16540, 16541, -1, 16542, 16543, 16544, -1, 16545, 16546, 16547, -1, 16548, 16549, 16550, -1, 16551, 16552, 16553, -1, 16554, 16555, 16556, -1, 16557, 16558, 16559, -1, 16560, 16561, 16562, -1, 16563, 16564, 16565, -1, 16566, 16567, 16568, -1, 16569, 16570, 16571, -1, 16572, 16573, 16574, -1, 16575, 16576, 16577, -1, 16578, 16579, 16580, -1, 16581, 16582, 16583, -1, 16584, 16585, 16586, -1, 16587, 16588, 16589, -1, 16590, 16591, 16592, -1, 16593, 16594, 16595, -1, 16596, 16597, 16598, -1, 16599, 16600, 16601, -1, 16602, 16603, 16604, -1, 16605, 16606, 16607, -1, 16608, 16609, 16610, -1, 16611, 16612, 16613, -1, 16614, 16615, 16616, -1, 16617, 16618, 16619, -1, 16620, 16621, 16622, -1, 16623, 16624, 16625, -1, 16626, 16627, 16628, -1, 16629, 16630, 16631, -1, 16632, 16633, 16634, -1, 16635, 16636, 16637, -1, 16638, 16639, 16640, -1, 16641, 16642, 16643, -1, 16644, 16645, 16646, -1, 16647, 16648, 16649, -1, 16650, 16651, 16652, -1, 16653, 16654, 16655, -1, 16656, 16657, 16658, -1, 16659, 16660, 16661, -1, 16662, 16663, 16664, -1, 16665, 16666, 16667, -1, 16668, 16669, 16670, -1, 16671, 16672, 16673, -1, 16674, 16675, 16676, -1, 16677, 16678, 16679, -1, 16680, 16681, 16682, -1, 16683, 16684, 16685, -1, 16686, 16687, 16688, -1, 16689, 16690, 16691, -1, 16692, 16693, 16694, -1, 16695, 16696, 16697, -1, 16698, 16699, 16700, -1, 16701, 16702, 16703, -1, 16704, 16705, 16706, -1, 16707, 16708, 16709, -1, 16710, 16711, 16712, -1, 16713, 16714, 16715, -1, 16716, 16717, 16718, -1, 16719, 16720, 16721, -1, 16722, 16723, 16724, -1, 16725, 16726, 16727, -1, 16728, 16729, 16730, -1, 16731, 16732, 16733, -1, 16734, 16735, 16736, -1, 16737, 16738, 16739, -1, 16740, 16741, 16742, -1, 16743, 16744, 16745, -1, 16746, 16747, 16748, -1, 16749, 16750, 16751, -1, 16752, 16753, 16754, -1, 16755, 16756, 16757, -1, 16758, 16759, 16760, -1, 16761, 16762, 16763, -1, 16764, 16765, 16766, -1, 16767, 16768, 16769, -1, 16770, 16771, 16772, -1, 16773, 16774, 16775, -1, 16776, 16777, 16778, -1, 16779, 16780, 16781, -1, 16782, 16783, 16784, -1, 16785, 16786, 16787, -1, 16788, 16789, 16790, -1, 16791, 16792, 16793, -1, 16794, 16795, 16796, -1, 16797, 16798, 16799, -1, 16800, 16801, 16802, -1, 16803, 16804, 16805, -1, 16806, 16807, 16808, -1, 16809, 16810, 16811, -1, 16812, 16813, 16814, -1, 16815, 16816, 16817, -1, 16818, 16819, 16820, -1, 16821, 16822, 16823, -1, 16824, 16825, 16826, -1, 16827, 16828, 16829, -1, 16830, 16831, 16832, -1, 16833, 16834, 16835, -1, 16836, 16837, 16838, -1, 16839, 16840, 16841, -1, 16842, 16843, 16844, -1, 16845, 16846, 16847, -1, 16848, 16849, 16850, -1, 16851, 16852, 16853, -1, 16854, 16855, 16856, -1, 16857, 16858, 16859, -1, 16860, 16861, 16862, -1, 16863, 16864, 16865, -1, 16866, 16867, 16868, -1, 16869, 16870, 16871, -1, 16872, 16873, 16874, -1, 16875, 16876, 16877, -1, 16878, 16879, 16880, -1, 16881, 16882, 16883, -1, 16884, 16885, 16886, -1, 16887, 16888, 16889, -1, 16890, 16891, 16892, -1, 16893, 16894, 16895, -1, 16896, 16897, 16898, -1, 16899, 16900, 16901, -1, 16902, 16903, 16904, -1, 16905, 16906, 16907, -1, 16908, 16909, 16910, -1, 16911, 16912, 16913, -1, 16914, 16915, 16916, -1, 16917, 16918, 16919, -1, 16920, 16921, 16922, -1, 16923, 16924, 16925, -1, 16926, 16927, 16928, -1, 16929, 16930, 16931, -1, 16932, 16933, 16934, -1, 16935, 16936, 16937, -1, 16938, 16939, 16940, -1, 16941, 16942, 16943, -1, 16944, 16945, 16946, -1, 16947, 16948, 16949, -1, 16950, 16951, 16952, -1, 16953, 16954, 16955, -1, 16956, 16957, 16958, -1, 16959, 16960, 16961, -1, 16962, 16963, 16964, -1, 16965, 16966, 16967, -1, 16968, 16969, 16970, -1, 16971, 16972, 16973, -1, 16974, 16975, 16976, -1, 16977, 16978, 16979, -1, 16980, 16981, 16982, -1, 16983, 16984, 16985, -1, 16986, 16987, 16988, -1, 16989, 16990, 16991, -1, 16992, 16993, 16994, -1, 16995, 16996, 16997, -1, 16998, 16999, 17000, -1, 17001, 17002, 17003, -1, 17004, 17005, 17006, -1, 17007, 17008, 17009, -1, 17010, 17011, 17012, -1, 17013, 17014, 17015, -1, 17016, 17017, 17018, -1, 17019, 17020, 17021, -1, 17022, 17023, 17024, -1, 17025, 17026, 17027, -1, 17028, 17029, 17030, -1, 17031, 17032, 17033, -1, 17034, 17035, 17036, -1, 17037, 17038, 17039, -1, 17040, 17041, 17042, -1, 17043, 17044, 17045, -1, 17046, 17047, 17048, -1, 17049, 17050, 17051, -1, 17052, 17053, 17054, -1, 17055, 17056, 17057, -1, 17058, 17059, 17060, -1, 17061, 17062, 17063, -1, 17064, 17065, 17066, -1, 17067, 17068, 17069, -1, 17070, 17071, 17072, -1, 17073, 17074, 17075, -1, 17076, 17077, 17078, -1, 17079, 17080, 17081, -1, 17082, 17083, 17084, -1, 17085, 17086, 17087, -1, 17088, 17089, 17090, -1, 17091, 17092, 17093, -1, 17094, 17095, 17096, -1, 17097, 17098, 17099, -1, 17100, 17101, 17102, -1, 17103, 17104, 17105, -1, 17106, 17107, 17108, -1, 17109, 17110, 17111, -1, 17112, 17113, 17114, -1, 17115, 17116, 17117, -1, 17118, 17119, 17120, -1, 17121, 17122, 17123, -1, 17124, 17125, 17126, -1, 17127, 17128, 17129, -1, 17130, 17131, 17132, -1, 17133, 17134, 17135, -1, 17136, 17137, 17138, -1, 17139, 17140, 17141, -1, 17142, 17143, 17144, -1, 17145, 17146, 17147, -1, 17148, 17149, 17150, -1, 17151, 17152, 17153, -1, 17154, 17155, 17156, -1, 17157, 17158, 17159, -1, 17160, 17161, 17162, -1, 17163, 17164, 17165, -1, 17166, 17167, 17168, -1, 17169, 17170, 17171, -1, 17172, 17173, 17174, -1, 17175, 17176, 17177, -1, 17178, 17179, 17180, -1, 17181, 17182, 17183, -1, 17184, 17185, 17186, -1, 17187, 17188, 17189, -1, 17190, 17191, 17192, -1, 17193, 17194, 17195, -1, 17196, 17197, 17198, -1, 17199, 17200, 17201, -1, 17202, 17203, 17204, -1, 17205, 17206, 17207, -1, 17208, 17209, 17210, -1, 17211, 17212, 17213, -1, 17214, 17215, 17216, -1, 17217, 17218, 17219, -1, 17220, 17221, 17222, -1, 17223, 17224, 17225, -1, 17226, 17227, 17228, -1, 17229, 17230, 17231, -1, 17232, 17233, 17234, -1, 17235, 17236, 17237, -1, 17238, 17239, 17240, -1, 17241, 17242, 17243, -1, 17244, 17245, 17246, -1, 17247, 17248, 17249, -1, 17250, 17251, 17252, -1, 17253, 17254, 17255, -1, 17256, 17257, 17258, -1, 17259, 17260, 17261, -1, 17262, 17263, 17264, -1, 17265, 17266, 17267, -1, 17268, 17269, 17270, -1, 17271, 17272, 17273, -1, 17274, 17275, 17276, -1, 17277, 17278, 17279, -1, 17280, 17281, 17282, -1, 17283, 17284, 17285, -1, 17286, 17287, 17288, -1, 17289, 17290, 17291, -1, 17292, 17293, 17294, -1, 17295, 17296, 17297, -1, 17298, 17299, 17300, -1, 17301, 17302, 17303, -1, 17304, 17305, 17306, -1, 17307, 17308, 17309, -1, 17310, 17311, 17312, -1, 17313, 17314, 17315, -1, 17316, 17317, 17318, -1, 17319, 17320, 17321, -1, 17322, 17323, 17324, -1, 17325, 17326, 17327, -1, 17328, 17329, 17330, -1, 17331, 17332, 17333, -1, 17334, 17335, 17336, -1, 17337, 17338, 17339, -1, 17340, 17341, 17342, -1, 17343, 17344, 17345, -1, 17346, 17347, 17348, -1, 17349, 17350, 17351, -1, 17352, 17353, 17354, -1, 17355, 17356, 17357, -1, 17358, 17359, 17360, -1, 17361, 17362, 17363, -1, 17364, 17365, 17366, -1, 17367, 17368, 17369, -1, 17370, 17371, 17372, -1, 17373, 17374, 17375, -1, 17376, 17377, 17378, -1, 17379, 17380, 17381, -1, 17382, 17383, 17384, -1, 17385, 17386, 17387, -1, 17388, 17389, 17390, -1, 17391, 17392, 17393, -1, 17394, 17395, 17396, -1, 17397, 17398, 17399, -1, 17400, 17401, 17402, -1, 17403, 17404, 17405, -1, 17406, 17407, 17408, -1, 17409, 17410, 17411, -1, 17412, 17413, 17414, -1, 17415, 17416, 17417, -1, 17418, 17419, 17420, -1, 17421, 17422, 17423, -1, 17424, 17425, 17426, -1, 17427, 17428, 17429, -1, 17430, 17431, 17432, -1, 17433, 17434, 17435, -1, 17436, 17437, 17438, -1, 17439, 17440, 17441, -1, 17442, 17443, 17444, -1, 17445, 17446, 17447, -1, 17448, 17449, 17450, -1, 17451, 17452, 17453, -1, 17454, 17455, 17456, -1, 17457, 17458, 17459, -1, 17460, 17461, 17462, -1, 17463, 17464, 17465, -1, 17466, 17467, 17468, -1, 17469, 17470, 17471, -1, 17472, 17473, 17474, -1, 17475, 17476, 17477, -1, 17478, 17479, 17480, -1, 17481, 17482, 17483, -1, 17484, 17485, 17486, -1, 17487, 17488, 17489, -1, 17490, 17491, 17492, -1, 17493, 17494, 17495, -1, 17496, 17497, 17498, -1, 17499, 17500, 17501, -1, 17502, 17503, 17504, -1, 17505, 17506, 17507, -1, 17508, 17509, 17510, -1, 17511, 17512, 17513, -1, 17514, 17515, 17516, -1, 17517, 17518, 17519, -1, 17520, 17521, 17522, -1, 17523, 17524, 17525, -1, 17526, 17527, 17528, -1, 17529, 17530, 17531, -1, 17532, 17533, 17534, -1, 17535, 17536, 17537, -1, 17538, 17539, 17540, -1, 17541, 17542, 17543, -1, 17544, 17545, 17546, -1, 17547, 17548, 17549, -1, 17550, 17551, 17552, -1, 17553, 17554, 17555, -1, 17556, 17557, 17558, -1, 17559, 17560, 17561, -1, 17562, 17563, 17564, -1, 17565, 17566, 17567, -1, 17568, 17569, 17570, -1, 17571, 17572, 17573, -1, 17574, 17575, 17576, -1, 17577, 17578, 17579, -1, 17580, 17581, 17582, -1, 17583, 17584, 17585, -1, 17586, 17587, 17588, -1, 17589, 17590, 17591, -1, 17592, 17593, 17594, -1, 17595, 17596, 17597, -1, 17598, 17599, 17600, -1, 17601, 17602, 17603, -1, 17604, 17605, 17606, -1, 17607, 17608, 17609, -1, 17610, 17611, 17612, -1, 17613, 17614, 17615, -1, 17616, 17617, 17618, -1, 17619, 17620, 17621, -1, 17622, 17623, 17624, -1, 17625, 17626, 17627, -1, 17628, 17629, 17630, -1, 17631, 17632, 17633, -1, 17634, 17635, 17636, -1, 17637, 17638, 17639, -1, 17640, 17641, 17642, -1, 17643, 17644, 17645, -1, 17646, 17647, 17648, -1, 17649, 17650, 17651, -1, 17652, 17653, 17654, -1, 17655, 17656, 17657, -1, 17658, 17659, 17660, -1, 17661, 17662, 17663, -1, 17664, 17665, 17666, -1, 17667, 17668, 17669, -1, 17670, 17671, 17672, -1, 17673, 17674, 17675, -1, 17676, 17677, 17678, -1, 17679, 17680, 17681, -1, 17682, 17683, 17684, -1, 17685, 17686, 17687, -1, 17688, 17689, 17690, -1, 17691, 17692, 17693, -1, 17694, 17695, 17696, -1, 17697, 17698, 17699, -1, 17700, 17701, 17702, -1, 17703, 17704, 17705, -1, 17706, 17707, 17708, -1, 17709, 17710, 17711, -1, 17712, 17713, 17714, -1, 17715, 17716, 17717, -1, 17718, 17719, 17720, -1, 17721, 17722, 17723, -1, 17724, 17725, 17726, -1, 17727, 17728, 17729, -1, 17730, 17731, 17732, -1, 17733, 17734, 17735, -1, 17736, 17737, 17738, -1, 17739, 17740, 17741, -1, 17742, 17743, 17744, -1, 17745, 17746, 17747, -1, 17748, 17749, 17750, -1, 17751, 17752, 17753, -1, 17754, 17755, 17756, -1, 17757, 17758, 17759, -1, 17760, 17761, 17762, -1, 17763, 17764, 17765, -1, 17766, 17767, 17768, -1, 17769, 17770, 17771, -1, 17772, 17773, 17774, -1, 17775, 17776, 17777, -1, 17778, 17779, 17780, -1, 17781, 17782, 17783, -1, 17784, 17785, 17786, -1, 17787, 17788, 17789, -1, 17790, 17791, 17792, -1, 17793, 17794, 17795, -1, 17796, 17797, 17798, -1, 17799, 17800, 17801, -1, 17802, 17803, 17804, -1, 17805, 17806, 17807, -1, 17808, 17809, 17810, -1, 17811, 17812, 17813, -1, 17814, 17815, 17816, -1, 17817, 17818, 17819, -1, 17820, 17821, 17822, -1, 17823, 17824, 17825, -1, 17826, 17827, 17828, -1, 17829, 17830, 17831, -1, 17832, 17833, 17834, -1, 17835, 17836, 17837, -1, 17838, 17839, 17840, -1, 17841, 17842, 17843, -1, 17844, 17845, 17846, -1, 17847, 17848, 17849, -1, 17850, 17851, 17852, -1, 17853, 17854, 17855, -1, 17856, 17857, 17858, -1, 17859, 17860, 17861, -1, 17862, 17863, 17864, -1, 17865, 17866, 17867, -1, 17868, 17869, 17870, -1, 17871, 17872, 17873, -1, 17874, 17875, 17876, -1, 17877, 17878, 17879, -1, 17880, 17881, 17882, -1, 17883, 17884, 17885, -1, 17886, 17887, 17888, -1, 17889, 17890, 17891, -1, 17892, 17893, 17894, -1, 17895, 17896, 17897, -1, 17898, 17899, 17900, -1, 17901, 17902, 17903, -1, 17904, 17905, 17906, -1, 17907, 17908, 17909, -1, 17910, 17911, 17912, -1, 17913, 17914, 17915, -1, 17916, 17917, 17918, -1, 17919, 17920, 17921, -1, 17922, 17923, 17924, -1, 17925, 17926, 17927, -1, 17928, 17929, 17930, -1, 17931, 17932, 17933, -1, 17934, 17935, 17936, -1, 17937, 17938, 17939, -1, 17940, 17941, 17942, -1, 17943, 17944, 17945, -1, 17946, 17947, 17948, -1, 17949, 17950, 17951, -1, 17952, 17953, 17954, -1, 17955, 17956, 17957, -1, 17958, 17959, 17960, -1, 17961, 17962, 17963, -1, 17964, 17965, 17966, -1, 17967, 17968, 17969, -1, 17970, 17971, 17972, -1, 17973, 17974, 17975, -1, 17976, 17977, 17978, -1, 17979, 17980, 17981, -1, 17982, 17983, 17984, -1, 17985, 17986, 17987, -1, 17988, 17989, 17990, -1, 17991, 17992, 17993, -1, 17994, 17995, 17996, -1, 17997, 17998, 17999, -1, 18000, 18001, 18002, -1, 18003, 18004, 18005, -1, 18006, 18007, 18008, -1, 18009, 18010, 18011, -1, 18012, 18013, 18014, -1, 18015, 18016, 18017, -1, 18018, 18019, 18020, -1, 18021, 18022, 18023, -1, 18024, 18025, 18026, -1, 18027, 18028, 18029, -1, 18030, 18031, 18032, -1, 18033, 18034, 18035, -1, 18036, 18037, 18038, -1, 18039, 18040, 18041, -1, 18042, 18043, 18044, -1, 18045, 18046, 18047, -1, 18048, 18049, 18050, -1, 18051, 18052, 18053, -1, 18054, 18055, 18056, -1, 18057, 18058, 18059, -1, 18060, 18061, 18062, -1, 18063, 18064, 18065, -1, 18066, 18067, 18068, -1, 18069, 18070, 18071, -1, 18072, 18073, 18074, -1, 18075, 18076, 18077, -1, 18078, 18079, 18080, -1, 18081, 18082, 18083, -1, 18084, 18085, 18086, -1, 18087, 18088, 18089, -1, 18090, 18091, 18092, -1, 18093, 18094, 18095, -1, 18096, 18097, 18098, -1, 18099, 18100, 18101, -1, 18102, 18103, 18104, -1, 18105, 18106, 18107, -1, 18108, 18109, 18110, -1, 18111, 18112, 18113, -1, 18114, 18115, 18116, -1, 18117, 18118, 18119, -1, 18120, 18121, 18122, -1, 18123, 18124, 18125, -1, 18126, 18127, 18128, -1, 18129, 18130, 18131, -1, 18132, 18133, 18134, -1, 18135, 18136, 18137, -1, 18138, 18139, 18140, -1, 18141, 18142, 18143, -1, 18144, 18145, 18146, -1, 18147, 18148, 18149, -1, 18150, 18151, 18152, -1, 18153, 18154, 18155, -1, 18156, 18157, 18158, -1, 18159, 18160, 18161, -1, 18162, 18163, 18164, -1, 18165, 18166, 18167, -1, 18168, 18169, 18170, -1, 18171, 18172, 18173, -1, 18174, 18175, 18176, -1, 18177, 18178, 18179, -1, 18180, 18181, 18182, -1, 18183, 18184, 18185, -1, 18186, 18187, 18188, -1, 18189, 18190, 18191, -1, 18192, 18193, 18194, -1, 18195, 18196, 18197, -1, 18198, 18199, 18200, -1, 18201, 18202, 18203, -1, 18204, 18205, 18206, -1, 18207, 18208, 18209, -1, 18210, 18211, 18212, -1, 18213, 18214, 18215, -1, 18216, 18217, 18218, -1, 18219, 18220, 18221, -1, 18222, 18223, 18224, -1, 18225, 18226, 18227, -1, 18228, 18229, 18230, -1, 18231, 18232, 18233, -1, 18234, 18235, 18236, -1, 18237, 18238, 18239, -1, 18240, 18241, 18242, -1, 18243, 18244, 18245, -1, 18246, 18247, 18248, -1, 18249, 18250, 18251, -1, 18252, 18253, 18254, -1, 18255, 18256, 18257, -1, 18258, 18259, 18260, -1, 18261, 18262, 18263, -1, 18264, 18265, 18266, -1, 18267, 18268, 18269, -1, 18270, 18271, 18272, -1, 18273, 18274, 18275, -1, 18276, 18277, 18278, -1, 18279, 18280, 18281, -1, 18282, 18283, 18284, -1, 18285, 18286, 18287, -1, 18288, 18289, 18290, -1, 18291, 18292, 18293, -1, 18294, 18295, 18296, -1, 18297, 18298, 18299, -1, 18300, 18301, 18302, -1, 18303, 18304, 18305, -1, 18306, 18307, 18308, -1, 18309, 18310, 18311, -1, 18312, 18313, 18314, -1, 18315, 18316, 18317, -1, 18318, 18319, 18320, -1, 18321, 18322, 18323, -1, 18324, 18325, 18326, -1, 18327, 18328, 18329, -1, 18330, 18331, 18332, -1, 18333, 18334, 18335, -1, 18336, 18337, 18338, -1, 18339, 18340, 18341, -1, 18342, 18343, 18344, -1, 18345, 18346, 18347, -1, 18348, 18349, 18350, -1, 18351, 18352, 18353, -1, 18354, 18355, 18356, -1, 18357, 18358, 18359, -1, 18360, 18361, 18362, -1, 18363, 18364, 18365, -1, 18366, 18367, 18368, -1, 18369, 18370, 18371, -1, 18372, 18373, 18374, -1, 18375, 18376, 18377, -1, 18378, 18379, 18380, -1, 18381, 18382, 18383, -1, 18384, 18385, 18386, -1, 18387, 18388, 18389, -1, 18390, 18391, 18392, -1, 18393, 18394, 18395, -1, 18396, 18397, 18398, -1, 18399, 18400, 18401, -1, 18402, 18403, 18404, -1, 18405, 18406, 18407, -1, 18408, 18409, 18410, -1, 18411, 18412, 18413, -1, 18414, 18415, 18416, -1, 18417, 18418, 18419, -1, 18420, 18421, 18422, -1, 18423, 18424, 18425, -1, 18426, 18427, 18428, -1, 18429, 18430, 18431, -1, 18432, 18433, 18434, -1, 18435, 18436, 18437, -1, 18438, 18439, 18440, -1, 18441, 18442, 18443, -1, 18444, 18445, 18446, -1, 18447, 18448, 18449, -1, 18450, 18451, 18452, -1, 18453, 18454, 18455, -1, 18456, 18457, 18458, -1, 18459, 18460, 18461, -1, 18462, 18463, 18464, -1, 18465, 18466, 18467, -1, 18468, 18469, 18470, -1, 18471, 18472, 18473, -1, 18474, 18475, 18476, -1, 18477, 18478, 18479, -1, 18480, 18481, 18482, -1, 18483, 18484, 18485, -1, 18486, 18487, 18488, -1, 18489, 18490, 18491, -1, 18492, 18493, 18494, -1, 18495, 18496, 18497, -1, 18498, 18499, 18500, -1, 18501, 18502, 18503, -1, 18504, 18505, 18506, -1, 18507, 18508, 18509, -1, 18510, 18511, 18512, -1, 18513, 18514, 18515, -1, 18516, 18517, 18518, -1, 18519, 18520, 18521, -1, 18522, 18523, 18524, -1, 18525, 18526, 18527, -1, 18528, 18529, 18530, -1, 18531, 18532, 18533, -1, 18534, 18535, 18536, -1, 18537, 18538, 18539, -1, 18540, 18541, 18542, -1, 18543, 18544, 18545, -1, 18546, 18547, 18548, -1, 18549, 18550, 18551, -1, 18552, 18553, 18554, -1, 18555, 18556, 18557, -1, 18558, 18559, 18560, -1, 18561, 18562, 18563, -1, 18564, 18565, 18566, -1, 18567, 18568, 18569, -1, 18570, 18571, 18572, -1, 18573, 18574, 18575, -1, 18576, 18577, 18578, -1, 18579, 18580, 18581, -1, 18582, 18583, 18584, -1, 18585, 18586, 18587, -1, 18588, 18589, 18590, -1, 18591, 18592, 18593, -1, 18594, 18595, 18596, -1, 18597, 18598, 18599, -1, 18600, 18601, 18602, -1, 18603, 18604, 18605, -1, 18606, 18607, 18608, -1, 18609, 18610, 18611, -1, 18612, 18613, 18614, -1, 18615, 18616, 18617, -1, 18618, 18619, 18620, -1, 18621, 18622, 18623, -1, 18624, 18625, 18626, -1, 18627, 18628, 18629, -1, 18630, 18631, 18632, -1, 18633, 18634, 18635, -1, 18636, 18637, 18638, -1, 18639, 18640, 18641, -1, 18642, 18643, 18644, -1, 18645, 18646, 18647, -1, 18648, 18649, 18650, -1, 18651, 18652, 18653, -1, 18654, 18655, 18656, -1, 18657, 18658, 18659, -1, 18660, 18661, 18662, -1, 18663, 18664, 18665, -1, 18666, 18667, 18668, -1, 18669, 18670, 18671, -1, 18672, 18673, 18674, -1, 18675, 18676, 18677, -1, 18678, 18679, 18680, -1, 18681, 18682, 18683, -1, 18684, 18685, 18686, -1, 18687, 18688, 18689, -1, 18690, 18691, 18692, -1, 18693, 18694, 18695, -1, 18696, 18697, 18698, -1, 18699, 18700, 18701, -1, 18702, 18703, 18704, -1, 18705, 18706, 18707, -1, 18708, 18709, 18710, -1, 18711, 18712, 18713, -1, 18714, 18715, 18716, -1, 18717, 18718, 18719, -1, 18720, 18721, 18722, -1, 18723, 18724, 18725, -1, 18726, 18727, 18728, -1, 18729, 18730, 18731, -1, 18732, 18733, 18734, -1, 18735, 18736, 18737, -1, 18738, 18739, 18740, -1, 18741, 18742, 18743, -1, 18744, 18745, 18746, -1, 18747, 18748, 18749, -1, 18750, 18751, 18752, -1, 18753, 18754, 18755, -1, 18756, 18757, 18758, -1, 18759, 18760, 18761, -1, 18762, 18763, 18764, -1, 18765, 18766, 18767, -1, 18768, 18769, 18770, -1, 18771, 18772, 18773, -1, 18774, 18775, 18776, -1, 18777, 18778, 18779, -1, 18780, 18781, 18782, -1, 18783, 18784, 18785, -1, 18786, 18787, 18788, -1, 18789, 18790, 18791, -1, 18792, 18793, 18794, -1, 18795, 18796, 18797, -1, 18798, 18799, 18800, -1, 18801, 18802, 18803, -1, 18804, 18805, 18806, -1, 18807, 18808, 18809, -1, 18810, 18811, 18812, -1, 18813, 18814, 18815, -1, 18816, 18817, 18818, -1, 18819, 18820, 18821, -1, 18822, 18823, 18824, -1, 18825, 18826, 18827, -1, 18828, 18829, 18830, -1, 18831, 18832, 18833, -1, 18834, 18835, 18836, -1, 18837, 18838, 18839, -1, 18840, 18841, 18842, -1, 18843, 18844, 18845, -1, 18846, 18847, 18848, -1, 18849, 18850, 18851, -1, 18852, 18853, 18854, -1, 18855, 18856, 18857, -1, 18858, 18859, 18860, -1, 18861, 18862, 18863, -1, 18864, 18865, 18866, -1, 18867, 18868, 18869, -1, 18870, 18871, 18872, -1, 18873, 18874, 18875, -1, 18876, 18877, 18878, -1, 18879, 18880, 18881, -1, 18882, 18883, 18884, -1, 18885, 18886, 18887, -1, 18888, 18889, 18890, -1, 18891, 18892, 18893, -1, 18894, 18895, 18896, -1, 18897, 18898, 18899, -1, 18900, 18901, 18902, -1, 18903, 18904, 18905, -1, 18906, 18907, 18908, -1, 18909, 18910, 18911, -1, 18912, 18913, 18914, -1, 18915, 18916, 18917, -1, 18918, 18919, 18920, -1, 18921, 18922, 18923, -1, 18924, 18925, 18926, -1, 18927, 18928, 18929, -1, 18930, 18931, 18932, -1, 18933, 18934, 18935, -1, 18936, 18937, 18938, -1, 18939, 18940, 18941, -1, 18942, 18943, 18944, -1, 18945, 18946, 18947, -1, 18948, 18949, 18950, -1, 18951, 18952, 18953, -1, 18954, 18955, 18956, -1, 18957, 18958, 18959, -1, 18960, 18961, 18962, -1, 18963, 18964, 18965, -1, 18966, 18967, 18968, -1, 18969, 18970, 18971, -1, 18972, 18973, 18974, -1, 18975, 18976, 18977, -1, 18978, 18979, 18980, -1, 18981, 18982, 18983, -1, 18984, 18985, 18986, -1, 18987, 18988, 18989, -1, 18990, 18991, 18992, -1, 18993, 18994, 18995, -1, 18996, 18997, 18998, -1, 18999, 19000, 19001, -1, 19002, 19003, 19004, -1, 19005, 19006, 19007, -1, 19008, 19009, 19010, -1, 19011, 19012, 19013, -1, 19014, 19015, 19016, -1, 19017, 19018, 19019, -1, 19020, 19021, 19022, -1, 19023, 19024, 19025, -1, 19026, 19027, 19028, -1, 19029, 19030, 19031, -1, 19032, 19033, 19034, -1, 19035, 19036, 19037, -1, 19038, 19039, 19040, -1, 19041, 19042, 19043, -1, 19044, 19045, 19046, -1, 19047, 19048, 19049, -1, 19050, 19051, 19052, -1, 19053, 19054, 19055, -1, 19056, 19057, 19058, -1, 19059, 19060, 19061, -1, 19062, 19063, 19064, -1, 19065, 19066, 19067, -1, 19068, 19069, 19070, -1, 19071, 19072, 19073, -1, 19074, 19075, 19076, -1, 19077, 19078, 19079, -1, 19080, 19081, 19082, -1, 19083, 19084, 19085, -1, 19086, 19087, 19088, -1, 19089, 19090, 19091, -1, 19092, 19093, 19094, -1, 19095, 19096, 19097, -1, 19098, 19099, 19100, -1, 19101, 19102, 19103, -1, 19104, 19105, 19106, -1, 19107, 19108, 19109, -1, 19110, 19111, 19112, -1, 19113, 19114, 19115, -1, 19116, 19117, 19118, -1, 19119, 19120, 19121, -1, 19122, 19123, 19124, -1, 19125, 19126, 19127, -1, 19128, 19129, 19130, -1, 19131, 19132, 19133, -1, 19134, 19135, 19136, -1, 19137, 19138, 19139, -1, 19140, 19141, 19142, -1, 19143, 19144, 19145, -1, 19146, 19147, 19148, -1, 19149, 19150, 19151, -1, 19152, 19153, 19154, -1, 19155, 19156, 19157, -1, 19158, 19159, 19160, -1, 19161, 19162, 19163, -1, 19164, 19165, 19166, -1, 19167, 19168, 19169, -1, 19170, 19171, 19172, -1, 19173, 19174, 19175, -1, 19176, 19177, 19178, -1, 19179, 19180, 19181, -1, 19182, 19183, 19184, -1, 19185, 19186, 19187, -1, 19188, 19189, 19190, -1, 19191, 19192, 19193, -1, 19194, 19195, 19196, -1, 19197, 19198, 19199, -1, 19200, 19201, 19202, -1, 19203, 19204, 19205, -1, 19206, 19207, 19208, -1, 19209, 19210, 19211, -1, 19212, 19213, 19214, -1, 19215, 19216, 19217, -1, 19218, 19219, 19220, -1, 19221, 19222, 19223, -1, 19224, 19225, 19226, -1, 19227, 19228, 19229, -1, 19230, 19231, 19232, -1, 19233, 19234, 19235, -1, 19236, 19237, 19238, -1, 19239, 19240, 19241, -1, 19242, 19243, 19244, -1, 19245, 19246, 19247, -1, 19248, 19249, 19250, -1, 19251, 19252, 19253, -1, 19254, 19255, 19256, -1, 19257, 19258, 19259, -1, 19260, 19261, 19262, -1, 19263, 19264, 19265, -1, 19266, 19267, 19268, -1, 19269, 19270, 19271, -1, 19272, 19273, 19274, -1, 19275, 19276, 19277, -1, 19278, 19279, 19280, -1, 19281, 19282, 19283, -1, 19284, 19285, 19286, -1, 19287, 19288, 19289, -1, 19290, 19291, 19292, -1, 19293, 19294, 19295, -1, 19296, 19297, 19298, -1, 19299, 19300, 19301, -1, 19302, 19303, 19304, -1, 19305, 19306, 19307, -1, 19308, 19309, 19310, -1, 19311, 19312, 19313, -1, 19314, 19315, 19316, -1, 19317, 19318, 19319, -1, 19320, 19321, 19322, -1, 19323, 19324, 19325, -1, 19326, 19327, 19328, -1, 19329, 19330, 19331, -1, 19332, 19333, 19334, -1, 19335, 19336, 19337, -1, 19338, 19339, 19340, -1, 19341, 19342, 19343, -1, 19344, 19345, 19346, -1, 19347, 19348, 19349, -1, 19350, 19351, 19352, -1, 19353, 19354, 19355, -1, 19356, 19357, 19358, -1, 19359, 19360, 19361, -1, 19362, 19363, 19364, -1, 19365, 19366, 19367, -1, 19368, 19369, 19370, -1, 19371, 19372, 19373, -1, 19374, 19375, 19376, -1, 19377, 19378, 19379, -1, 19380, 19381, 19382, -1, 19383, 19384, 19385, -1, 19386, 19387, 19388, -1, 19389, 19390, 19391, -1, 19392, 19393, 19394, -1, 19395, 19396, 19397, -1, 19398, 19399, 19400, -1, 19401, 19402, 19403, -1, 19404, 19405, 19406, -1, 19407, 19408, 19409, -1, 19410, 19411, 19412, -1, 19413, 19414, 19415, -1, 19416, 19417, 19418, -1, 19419, 19420, 19421, -1, 19422, 19423, 19424, -1, 19425, 19426, 19427, -1, 19428, 19429, 19430, -1, 19431, 19432, 19433, -1, 19434, 19435, 19436, -1, 19437, 19438, 19439, -1, 19440, 19441, 19442, -1, 19443, 19444, 19445, -1, 19446, 19447, 19448, -1, 19449, 19450, 19451, -1, 19452, 19453, 19454, -1, 19455, 19456, 19457, -1, 19458, 19459, 19460, -1, 19461, 19462, 19463, -1, 19464, 19465, 19466, -1, 19467, 19468, 19469, -1, 19470, 19471, 19472, -1, 19473, 19474, 19475, -1, 19476, 19477, 19478, -1, 19479, 19480, 19481, -1, 19482, 19483, 19484, -1, 19485, 19486, 19487, -1, 19488, 19489, 19490, -1, 19491, 19492, 19493, -1, 19494, 19495, 19496, -1, 19497, 19498, 19499, -1, 19500, 19501, 19502, -1, 19503, 19504, 19505, -1, 19506, 19507, 19508, -1, 19509, 19510, 19511, -1, 19512, 19513, 19514, -1, 19515, 19516, 19517, -1, 19518, 19519, 19520, -1, 19521, 19522, 19523, -1, 19524, 19525, 19526, -1, 19527, 19528, 19529, -1, 19530, 19531, 19532, -1, 19533, 19534, 19535, -1, 19536, 19537, 19538, -1, 19539, 19540, 19541, -1, 19542, 19543, 19544, -1, 19545, 19546, 19547, -1, 19548, 19549, 19550, -1, 19551, 19552, 19553, -1, 19554, 19555, 19556, -1, 19557, 19558, 19559, -1, 19560, 19561, 19562, -1, 19563, 19564, 19565, -1, 19566, 19567, 19568, -1, 19569, 19570, 19571, -1, 19572, 19573, 19574, -1, 19575, 19576, 19577, -1, 19578, 19579, 19580, -1, 19581, 19582, 19583, -1, 19584, 19585, 19586, -1, 19587, 19588, 19589, -1, 19590, 19591, 19592, -1, 19593, 19594, 19595, -1, 19596, 19597, 19598, -1, 19599, 19600, 19601, -1, 19602, 19603, 19604, -1, 19605, 19606, 19607, -1, 19608, 19609, 19610, -1, 19611, 19612, 19613, -1, 19614, 19615, 19616, -1, 19617, 19618, 19619, -1, 19620, 19621, 19622, -1, 19623, 19624, 19625, -1, 19626, 19627, 19628, -1, 19629, 19630, 19631, -1, 19632, 19633, 19634, -1, 19635, 19636, 19637, -1, 19638, 19639, 19640, -1, 19641, 19642, 19643, -1, 19644, 19645, 19646, -1, 19647, 19648, 19649, -1, 19650, 19651, 19652, -1, 19653, 19654, 19655, -1, 19656, 19657, 19658, -1, 19659, 19660, 19661, -1, 19662, 19663, 19664, -1, 19665, 19666, 19667, -1, 19668, 19669, 19670, -1, 19671, 19672, 19673, -1, 19674, 19675, 19676, -1, 19677, 19678, 19679, -1, 19680, 19681, 19682, -1, 19683, 19684, 19685, -1, 19686, 19687, 19688, -1, 19689, 19690, 19691, -1, 19692, 19693, 19694, -1, 19695, 19696, 19697, -1, 19698, 19699, 19700, -1, 19701, 19702, 19703, -1, 19704, 19705, 19706, -1, 19707, 19708, 19709, -1, 19710, 19711, 19712, -1, 19713, 19714, 19715, -1, 19716, 19717, 19718, -1, 19719, 19720, 19721, -1, 19722, 19723, 19724, -1, 19725, 19726, 19727, -1, 19728, 19729, 19730, -1, 19731, 19732, 19733, -1, 19734, 19735, 19736, -1, 19737, 19738, 19739, -1, 19740, 19741, 19742, -1, 19743, 19744, 19745, -1, 19746, 19747, 19748, -1, 19749, 19750, 19751, -1, 19752, 19753, 19754, -1, 19755, 19756, 19757, -1, 19758, 19759, 19760, -1, 19761, 19762, 19763, -1, 19764, 19765, 19766, -1, 19767, 19768, 19769, -1, 19770, 19771, 19772, -1, 19773, 19774, 19775, -1, 19776, 19777, 19778, -1, 19779, 19780, 19781, -1, 19782, 19783, 19784, -1, 19785, 19786, 19787, -1, 19788, 19789, 19790, -1, 19791, 19792, 19793, -1, 19794, 19795, 19796, -1, 19797, 19798, 19799, -1, 19800, 19801, 19802, -1, 19803, 19804, 19805, -1, 19806, 19807, 19808, -1, 19809, 19810, 19811, -1, 19812, 19813, 19814, -1, 19815, 19816, 19817, -1, 19818, 19819, 19820, -1, 19821, 19822, 19823, -1, 19824, 19825, 19826, -1, 19827, 19828, 19829, -1, 19830, 19831, 19832, -1, 19833, 19834, 19835, -1, 19836, 19837, 19838, -1, 19839, 19840, 19841, -1, 19842, 19843, 19844, -1, 19845, 19846, 19847, -1, 19848, 19849, 19850, -1, 19851, 19852, 19853, -1, 19854, 19855, 19856, -1, 19857, 19858, 19859, -1, 19860, 19861, 19862, -1, 19863, 19864, 19865, -1, 19866, 19867, 19868, -1, 19869, 19870, 19871, -1, 19872, 19873, 19874, -1, 19875, 19876, 19877, -1, 19878, 19879, 19880, -1, 19881, 19882, 19883, -1, 19884, 19885, 19886, -1, 19887, 19888, 19889, -1, 19890, 19891, 19892, -1, 19893, 19894, 19895, -1, 19896, 19897, 19898, -1, 19899, 19900, 19901, -1, 19902, 19903, 19904, -1, 19905, 19906, 19907, -1, 19908, 19909, 19910, -1, 19911, 19912, 19913, -1, 19914, 19915, 19916, -1, 19917, 19918, 19919, -1, 19920, 19921, 19922, -1, 19923, 19924, 19925, -1, 19926, 19927, 19928, -1, 19929, 19930, 19931, -1, 19932, 19933, 19934, -1, 19935, 19936, 19937, -1, 19938, 19939, 19940, -1, 19941, 19942, 19943, -1, 19944, 19945, 19946, -1, 19947, 19948, 19949, -1, 19950, 19951, 19952, -1, 19953, 19954, 19955, -1, 19956, 19957, 19958, -1, 19959, 19960, 19961, -1, 19962, 19963, 19964, -1, 19965, 19966, 19967, -1, 19968, 19969, 19970, -1, 19971, 19972, 19973, -1, 19974, 19975, 19976, -1, 19977, 19978, 19979, -1, 19980, 19981, 19982, -1, 19983, 19984, 19985, -1, 19986, 19987, 19988, -1, 19989, 19990, 19991, -1, 19992, 19993, 19994, -1, 19995, 19996, 19997, -1, 19998, 19999, 20000, -1, 20001, 20002, 20003, -1, 20004, 20005, 20006, -1, 20007, 20008, 20009, -1, 20010, 20011, 20012, -1, 20013, 20014, 20015, -1, 20016, 20017, 20018, -1, 20019, 20020, 20021, -1, 20022, 20023, 20024, -1, 20025, 20026, 20027, -1, 20028, 20029, 20030, -1, 20031, 20032, 20033, -1, 20034, 20035, 20036, -1, 20037, 20038, 20039, -1, 20040, 20041, 20042, -1, 20043, 20044, 20045, -1, 20046, 20047, 20048, -1, 20049, 20050, 20051, -1, 20052, 20053, 20054, -1, 20055, 20056, 20057, -1, 20058, 20059, 20060, -1, 20061, 20062, 20063, -1, 20064, 20065, 20066, -1, 20067, 20068, 20069, -1, 20070, 20071, 20072, -1, 20073, 20074, 20075, -1, 20076, 20077, 20078, -1, 20079, 20080, 20081, -1, 20082, 20083, 20084, -1, 20085, 20086, 20087, -1, 20088, 20089, 20090, -1, 20091, 20092, 20093, -1, 20094, 20095, 20096, -1, 20097, 20098, 20099, -1, 20100, 20101, 20102, -1, 20103, 20104, 20105, -1, 20106, 20107, 20108, -1, 20109, 20110, 20111, -1, 20112, 20113, 20114, -1, 20115, 20116, 20117, -1, 20118, 20119, 20120, -1, 20121, 20122, 20123, -1, 20124, 20125, 20126, -1, 20127, 20128, 20129, -1, 20130, 20131, 20132, -1, 20133, 20134, 20135, -1, 20136, 20137, 20138, -1, 20139, 20140, 20141, -1, 20142, 20143, 20144, -1, 20145, 20146, 20147, -1, 20148, 20149, 20150, -1, 20151, 20152, 20153, -1, 20154, 20155, 20156, -1, 20157, 20158, 20159, -1, 20160, 20161, 20162, -1, 20163, 20164, 20165, -1, 20166, 20167, 20168, -1, 20169, 20170, 20171, -1, 20172, 20173, 20174, -1, 20175, 20176, 20177, -1, 20178, 20179, 20180, -1, 20181, 20182, 20183, -1, 20184, 20185, 20186, -1, 20187, 20188, 20189, -1, 20190, 20191, 20192, -1, 20193, 20194, 20195, -1, 20196, 20197, 20198, -1, 20199, 20200, 20201, -1, 20202, 20203, 20204, -1, 20205, 20206, 20207, -1, 20208, 20209, 20210, -1, 20211, 20212, 20213, -1, 20214, 20215, 20216, -1, 20217, 20218, 20219, -1, 20220, 20221, 20222, -1, 20223, 20224, 20225, -1, 20226, 20227, 20228, -1, 20229, 20230, 20231, -1, 20232, 20233, 20234, -1, 20235, 20236, 20237, -1, 20238, 20239, 20240, -1, 20241, 20242, 20243, -1, 20244, 20245, 20246, -1, 20247, 20248, 20249, -1, 20250, 20251, 20252, -1, 20253, 20254, 20255, -1, 20256, 20257, 20258, -1, 20259, 20260, 20261, -1, 20262, 20263, 20264, -1, 20265, 20266, 20267, -1, 20268, 20269, 20270, -1, 20271, 20272, 20273, -1, 20274, 20275, 20276, -1, 20277, 20278, 20279, -1, 20280, 20281, 20282, -1, 20283, 20284, 20285, -1, 20286, 20287, 20288, -1, 20289, 20290, 20291, -1, 20292, 20293, 20294, -1, 20295, 20296, 20297, -1, 20298, 20299, 20300, -1, 20301, 20302, 20303, -1, 20304, 20305, 20306, -1, 20307, 20308, 20309, -1, 20310, 20311, 20312, -1, 20313, 20314, 20315, -1, 20316, 20317, 20318, -1, 20319, 20320, 20321, -1, 20322, 20323, 20324, -1, 20325, 20326, 20327, -1, 20328, 20329, 20330, -1, 20331, 20332, 20333, -1, 20334, 20335, 20336, -1, 20337, 20338, 20339, -1, 20340, 20341, 20342, -1, 20343, 20344, 20345, -1, 20346, 20347, 20348, -1, 20349, 20350, 20351, -1, 20352, 20353, 20354, -1, 20355, 20356, 20357, -1, 20358, 20359, 20360, -1, 20361, 20362, 20363, -1, 20364, 20365, 20366, -1, 20367, 20368, 20369, -1, 20370, 20371, 20372, -1, 20373, 20374, 20375, -1, 20376, 20377, 20378, -1, 20379, 20380, 20381, -1, 20382, 20383, 20384, -1, 20385, 20386, 20387, -1, 20388, 20389, 20390, -1, 20391, 20392, 20393, -1, 20394, 20395, 20396, -1, 20397, 20398, 20399, -1, 20400, 20401, 20402, -1, 20403, 20404, 20405, -1, 20406, 20407, 20408, -1, 20409, 20410, 20411, -1, 20412, 20413, 20414, -1, 20415, 20416, 20417, -1, 20418, 20419, 20420, -1, 20421, 20422, 20423, -1, 20424, 20425, 20426, -1, 20427, 20428, 20429, -1, 20430, 20431, 20432, -1, 20433, 20434, 20435, -1, 20436, 20437, 20438, -1, 20439, 20440, 20441, -1, 20442, 20443, 20444, -1, 20445, 20446, 20447, -1, 20448, 20449, 20450, -1, 20451, 20452, 20453, -1, 20454, 20455, 20456, -1, 20457, 20458, 20459, -1, 20460, 20461, 20462, -1, 20463, 20464, 20465, -1, 20466, 20467, 20468, -1, 20469, 20470, 20471, -1, 20472, 20473, 20474, -1, 20475, 20476, 20477, -1, 20478, 20479, 20480, -1, 20481, 20482, 20483, -1, 20484, 20485, 20486, -1, 20487, 20488, 20489, -1, 20490, 20491, 20492, -1, 20493, 20494, 20495, -1, 20496, 20497, 20498, -1, 20499, 20500, 20501, -1, 20502, 20503, 20504, -1, 20505, 20506, 20507, -1, 20508, 20509, 20510, -1, 20511, 20512, 20513, -1, 20514, 20515, 20516, -1, 20517, 20518, 20519, -1, 20520, 20521, 20522, -1, 20523, 20524, 20525, -1, 20526, 20527, 20528, -1, 20529, 20530, 20531, -1, 20532, 20533, 20534, -1, 20535, 20536, 20537, -1, 20538, 20539, 20540, -1, 20541, 20542, 20543, -1, 20544, 20545, 20546, -1, 20547, 20548, 20549, -1, 20550, 20551, 20552, -1, 20553, 20554, 20555, -1, 20556, 20557, 20558, -1, 20559, 20560, 20561, -1, 20562, 20563, 20564, -1, 20565, 20566, 20567, -1, 20568, 20569, 20570, -1, 20571, 20572, 20573, -1, 20574, 20575, 20576, -1, 20577, 20578, 20579, -1, 20580, 20581, 20582, -1, 20583, 20584, 20585, -1, 20586, 20587, 20588, -1, 20589, 20590, 20591, -1, 20592, 20593, 20594, -1, 20595, 20596, 20597, -1, 20598, 20599, 20600, -1, 20601, 20602, 20603, -1, 20604, 20605, 20606, -1, 20607, 20608, 20609, -1, 20610, 20611, 20612, -1, 20613, 20614, 20615, -1, 20616, 20617, 20618, -1, 20619, 20620, 20621, -1, 20622, 20623, 20624, -1, 20625, 20626, 20627, -1, 20628, 20629, 20630, -1, 20631, 20632, 20633, -1, 20634, 20635, 20636, -1, 20637, 20638, 20639, -1, 20640, 20641, 20642, -1, 20643, 20644, 20645, -1, 20646, 20647, 20648, -1, 20649, 20650, 20651, -1, 20652, 20653, 20654, -1, 20655, 20656, 20657, -1, 20658, 20659, 20660, -1, 20661, 20662, 20663, -1, 20664, 20665, 20666, -1, 20667, 20668, 20669, -1, 20670, 20671, 20672, -1, 20673, 20674, 20675, -1, 20676, 20677, 20678, -1, 20679, 20680, 20681, -1, 20682, 20683, 20684, -1, 20685, 20686, 20687, -1, 20688, 20689, 20690, -1, 20691, 20692, 20693, -1, 20694, 20695, 20696, -1, 20697, 20698, 20699, -1, 20700, 20701, 20702, -1, 20703, 20704, 20705, -1, 20706, 20707, 20708, -1, 20709, 20710, 20711, -1, 20712, 20713, 20714, -1, 20715, 20716, 20717, -1, 20718, 20719, 20720, -1, 20721, 20722, 20723, -1, 20724, 20725, 20726, -1, 20727, 20728, 20729, -1, 20730, 20731, 20732, -1, 20733, 20734, 20735, -1, 20736, 20737, 20738, -1, 20739, 20740, 20741, -1, 20742, 20743, 20744, -1, 20745, 20746, 20747, -1, 20748, 20749, 20750, -1, 20751, 20752, 20753, -1, 20754, 20755, 20756, -1, 20757, 20758, 20759, -1, 20760, 20761, 20762, -1, 20763, 20764, 20765, -1, 20766, 20767, 20768, -1, 20769, 20770, 20771, -1, 20772, 20773, 20774, -1, 20775, 20776, 20777, -1, 20778, 20779, 20780, -1, 20781, 20782, 20783, -1, 20784, 20785, 20786, -1, 20787, 20788, 20789, -1, 20790, 20791, 20792, -1, 20793, 20794, 20795, -1, 20796, 20797, 20798, -1, 20799, 20800, 20801, -1, 20802, 20803, 20804, -1, 20805, 20806, 20807, -1, 20808, 20809, 20810, -1, 20811, 20812, 20813, -1, 20814, 20815, 20816, -1, 20817, 20818, 20819, -1, 20820, 20821, 20822, -1, 20823, 20824, 20825, -1, 20826, 20827, 20828, -1, 20829, 20830, 20831, -1, 20832, 20833, 20834, -1, 20835, 20836, 20837, -1, 20838, 20839, 20840, -1, 20841, 20842, 20843, -1, 20844, 20845, 20846, -1, 20847, 20848, 20849, -1, 20850, 20851, 20852, -1, 20853, 20854, 20855, -1, 20856, 20857, 20858, -1, 20859, 20860, 20861, -1, 20862, 20863, 20864, -1, 20865, 20866, 20867, -1, 20868, 20869, 20870, -1, 20871, 20872, 20873, -1, 20874, 20875, 20876, -1, 20877, 20878, 20879, -1, 20880, 20881, 20882, -1, 20883, 20884, 20885, -1, 20886, 20887, 20888, -1, 20889, 20890, 20891, -1, 20892, 20893, 20894, -1, 20895, 20896, 20897, -1, 20898, 20899, 20900, -1, 20901, 20902, 20903, -1, 20904, 20905, 20906, -1, 20907, 20908, 20909, -1, 20910, 20911, 20912, -1, 20913, 20914, 20915, -1, 20916, 20917, 20918, -1, 20919, 20920, 20921, -1, 20922, 20923, 20924, -1, 20925, 20926, 20927, -1, 20928, 20929, 20930, -1, 20931, 20932, 20933, -1, 20934, 20935, 20936, -1, 20937, 20938, 20939, -1, 20940, 20941, 20942, -1, 20943, 20944, 20945, -1, 20946, 20947, 20948, -1, 20949, 20950, 20951, -1, 20952, 20953, 20954, -1, 20955, 20956, 20957, -1, 20958, 20959, 20960, -1, 20961, 20962, 20963, -1, 20964, 20965, 20966, -1, 20967, 20968, 20969, -1, 20970, 20971, 20972, -1, 20973, 20974, 20975, -1, 20976, 20977, 20978, -1, 20979, 20980, 20981, -1, 20982, 20983, 20984, -1, 20985, 20986, 20987, -1, 20988, 20989, 20990, -1, 20991, 20992, 20993, -1, 20994, 20995, 20996, -1, 20997, 20998, 20999, -1, 21000, 21001, 21002, -1, 21003, 21004, 21005, -1, 21006, 21007, 21008, -1, 21009, 21010, 21011, -1, 21012, 21013, 21014, -1, 21015, 21016, 21017, -1, 21018, 21019, 21020, -1, 21021, 21022, 21023, -1, 21024, 21025, 21026, -1, 21027, 21028, 21029, -1, 21030, 21031, 21032, -1, 21033, 21034, 21035, -1, 21036, 21037, 21038, -1, 21039, 21040, 21041, -1, 21042, 21043, 21044, -1, 21045, 21046, 21047, -1, 21048, 21049, 21050, -1, 21051, 21052, 21053, -1, 21054, 21055, 21056, -1, 21057, 21058, 21059, -1, 21060, 21061, 21062, -1, 21063, 21064, 21065, -1, 21066, 21067, 21068, -1, 21069, 21070, 21071, -1, 21072, 21073, 21074, -1, 21075, 21076, 21077, -1, 21078, 21079, 21080, -1, 21081, 21082, 21083, -1, 21084, 21085, 21086, -1, 21087, 21088, 21089, -1, 21090, 21091, 21092, -1, 21093, 21094, 21095, -1, 21096, 21097, 21098, -1, 21099, 21100, 21101, -1, 21102, 21103, 21104, -1, 21105, 21106, 21107, -1, 21108, 21109, 21110, -1, 21111, 21112, 21113, -1, 21114, 21115, 21116, -1, 21117, 21118, 21119, -1, 21120, 21121, 21122, -1, 21123, 21124, 21125, -1, 21126, 21127, 21128, -1, 21129, 21130, 21131, -1, 21132, 21133, 21134, -1, 21135, 21136, 21137, -1, 21138, 21139, 21140, -1, 21141, 21142, 21143, -1, 21144, 21145, 21146, -1, 21147, 21148, 21149, -1, 21150, 21151, 21152, -1, 21153, 21154, 21155, -1, 21156, 21157, 21158, -1, 21159, 21160, 21161, -1, 21162, 21163, 21164, -1, 21165, 21166, 21167, -1, 21168, 21169, 21170, -1, 21171, 21172, 21173, -1, 21174, 21175, 21176, -1, 21177, 21178, 21179, -1, 21180, 21181, 21182, -1, 21183, 21184, 21185, -1, 21186, 21187, 21188, -1, 21189, 21190, 21191, -1, 21192, 21193, 21194, -1, 21195, 21196, 21197, -1, 21198, 21199, 21200, -1, 21201, 21202, 21203, -1, 21204, 21205, 21206, -1, 21207, 21208, 21209, -1, 21210, 21211, 21212, -1, 21213, 21214, 21215, -1, 21216, 21217, 21218, -1, 21219, 21220, 21221, -1, 21222, 21223, 21224, -1, 21225, 21226, 21227, -1, 21228, 21229, 21230, -1, 21231, 21232, 21233, -1, 21234, 21235, 21236, -1, 21237, 21238, 21239, -1, 21240, 21241, 21242, -1, 21243, 21244, 21245, -1, 21246, 21247, 21248, -1, 21249, 21250, 21251, -1, 21252, 21253, 21254, -1, 21255, 21256, 21257, -1, 21258, 21259, 21260, -1, 21261, 21262, 21263, -1, 21264, 21265, 21266, -1, 21267, 21268, 21269, -1, 21270, 21271, 21272, -1, 21273, 21274, 21275, -1, 21276, 21277, 21278, -1, 21279, 21280, 21281, -1, 21282, 21283, 21284, -1, 21285, 21286, 21287, -1, 21288, 21289, 21290, -1, 21291, 21292, 21293, -1, 21294, 21295, 21296, -1, 21297, 21298, 21299, -1, 21300, 21301, 21302, -1, 21303, 21304, 21305, -1, 21306, 21307, 21308, -1, 21309, 21310, 21311, -1, 21312, 21313, 21314, -1, 21315, 21316, 21317, -1, 21318, 21319, 21320, -1, 21321, 21322, 21323, -1, 21324, 21325, 21326, -1, 21327, 21328, 21329, -1, 21330, 21331, 21332, -1, 21333, 21334, 21335, -1, 21336, 21337, 21338, -1, 21339, 21340, 21341, -1, 21342, 21343, 21344, -1, 21345, 21346, 21347, -1, 21348, 21349, 21350, -1, 21351, 21352, 21353, -1, 21354, 21355, 21356, -1, 21357, 21358, 21359, -1, 21360, 21361, 21362, -1, 21363, 21364, 21365, -1, 21366, 21367, 21368, -1, 21369, 21370, 21371, -1, 21372, 21373, 21374, -1, 21375, 21376, 21377, -1, 21378, 21379, 21380, -1, 21381, 21382, 21383, -1, 21384, 21385, 21386, -1, 21387, 21388, 21389, -1, 21390, 21391, 21392, -1, 21393, 21394, 21395, -1, 21396, 21397, 21398, -1, 21399, 21400, 21401, -1, 21402, 21403, 21404, -1, 21405, 21406, 21407, -1, 21408, 21409, 21410, -1, 21411, 21412, 21413, -1, 21414, 21415, 21416, -1, 21417, 21418, 21419, -1, 21420, 21421, 21422, -1, 21423, 21424, 21425, -1, 21426, 21427, 21428, -1, 21429, 21430, 21431, -1, 21432, 21433, 21434, -1, 21435, 21436, 21437, -1, 21438, 21439, 21440, -1, 21441, 21442, 21443, -1, 21444, 21445, 21446, -1, 21447, 21448, 21449, -1, 21450, 21451, 21452, -1, 21453, 21454, 21455, -1, 21456, 21457, 21458, -1, 21459, 21460, 21461, -1, 21462, 21463, 21464, -1, 21465, 21466, 21467, -1, 21468, 21469, 21470, -1, 21471, 21472, 21473, -1, 21474, 21475, 21476, -1, 21477, 21478, 21479, -1, 21480, 21481, 21482, -1, 21483, 21484, 21485, -1, 21486, 21487, 21488, -1, 21489, 21490, 21491, -1, 21492, 21493, 21494, -1, 21495, 21496, 21497, -1, 21498, 21499, 21500, -1, 21501, 21502, 21503, -1, 21504, 21505, 21506, -1, 21507, 21508, 21509, -1, 21510, 21511, 21512, -1, 21513, 21514, 21515, -1, 21516, 21517, 21518, -1, 21519, 21520, 21521, -1, 21522, 21523, 21524, -1, 21525, 21526, 21527, -1, 21528, 21529, 21530, -1, 21531, 21532, 21533, -1, 21534, 21535, 21536, -1, 21537, 21538, 21539, -1, 21540, 21541, 21542, -1, 21543, 21544, 21545, -1, 21546, 21547, 21548, -1, 21549, 21550, 21551, -1, 21552, 21553, 21554, -1, 21555, 21556, 21557, -1, 21558, 21559, 21560, -1, 21561, 21562, 21563, -1, 21564, 21565, 21566, -1, 21567, 21568, 21569, -1, 21570, 21571, 21572, -1, 21573, 21574, 21575, -1, 21576, 21577, 21578, -1, 21579, 21580, 21581, -1, 21582, 21583, 21584, -1, 21585, 21586, 21587, -1, 21588, 21589, 21590, -1, 21591, 21592, 21593, -1, 21594, 21595, 21596, -1, 21597, 21598, 21599, -1, 21600, 21601, 21602, -1, 21603, 21604, 21605, -1, 21606, 21607, 21608, -1, 21609, 21610, 21611, -1, 21612, 21613, 21614, -1, 21615, 21616, 21617, -1, 21618, 21619, 21620, -1, 21621, 21622, 21623, -1, 21624, 21625, 21626, -1, 21627, 21628, 21629, -1, 21630, 21631, 21632, -1, 21633, 21634, 21635, -1, 21636, 21637, 21638, -1, 21639, 21640, 21641, -1, 21642, 21643, 21644, -1, 21645, 21646, 21647, -1, 21648, 21649, 21650, -1, 21651, 21652, 21653, -1, 21654, 21655, 21656, -1, 21657, 21658, 21659, -1, 21660, 21661, 21662, -1, 21663, 21664, 21665, -1, 21666, 21667, 21668, -1, 21669, 21670, 21671, -1, 21672, 21673, 21674, -1, 21675, 21676, 21677, -1, 21678, 21679, 21680, -1, 21681, 21682, 21683, -1, 21684, 21685, 21686, -1, 21687, 21688, 21689, -1, 21690, 21691, 21692, -1, 21693, 21694, 21695, -1, 21696, 21697, 21698, -1, 21699, 21700, 21701, -1, 21702, 21703, 21704, -1, 21705, 21706, 21707, -1, 21708, 21709, 21710, -1, 21711, 21712, 21713, -1, 21714, 21715, 21716, -1, 21717, 21718, 21719, -1, 21720, 21721, 21722, -1, 21723, 21724, 21725, -1, 21726, 21727, 21728, -1, 21729, 21730, 21731, -1, 21732, 21733, 21734, -1, 21735, 21736, 21737, -1, 21738, 21739, 21740, -1, 21741, 21742, 21743, -1, 21744, 21745, 21746, -1, 21747, 21748, 21749, -1, 21750, 21751, 21752, -1, 21753, 21754, 21755, -1, 21756, 21757, 21758, -1, 21759, 21760, 21761, -1, 21762, 21763, 21764, -1, 21765, 21766, 21767, -1, 21768, 21769, 21770, -1, 21771, 21772, 21773, -1, 21774, 21775, 21776, -1, 21777, 21778, 21779, -1, 21780, 21781, 21782, -1, 21783, 21784, 21785, -1, 21786, 21787, 21788, -1, 21789, 21790, 21791, -1, 21792, 21793, 21794, -1, 21795, 21796, 21797, -1, 21798, 21799, 21800, -1, 21801, 21802, 21803, -1, 21804, 21805, 21806, -1, 21807, 21808, 21809, -1, 21810, 21811, 21812, -1, 21813, 21814, 21815, -1, 21816, 21817, 21818, -1, 21819, 21820, 21821, -1, 21822, 21823, 21824, -1, 21825, 21826, 21827, -1, 21828, 21829, 21830, -1, 21831, 21832, 21833, -1, 21834, 21835, 21836, -1, 21837, 21838, 21839, -1, 21840, 21841, 21842, -1, 21843, 21844, 21845, -1, 21846, 21847, 21848, -1, 21849, 21850, 21851, -1, 21852, 21853, 21854, -1, 21855, 21856, 21857, -1, 21858, 21859, 21860, -1, 21861, 21862, 21863, -1, 21864, 21865, 21866, -1, 21867, 21868, 21869, -1, 21870, 21871, 21872, -1, 21873, 21874, 21875, -1, 21876, 21877, 21878, -1, 21879, 21880, 21881, -1, 21882, 21883, 21884, -1, 21885, 21886, 21887, -1, 21888, 21889, 21890, -1, 21891, 21892, 21893, -1, 21894, 21895, 21896, -1, 21897, 21898, 21899, -1, 21900, 21901, 21902, -1, 21903, 21904, 21905, -1, 21906, 21907, 21908, -1, 21909, 21910, 21911, -1, 21912, 21913, 21914, -1, 21915, 21916, 21917, -1, 21918, 21919, 21920, -1, 21921, 21922, 21923, -1, 21924, 21925, 21926, -1, 21927, 21928, 21929, -1, 21930, 21931, 21932, -1, 21933, 21934, 21935, -1, 21936, 21937, 21938, -1, 21939, 21940, 21941, -1, 21942, 21943, 21944, -1, 21945, 21946, 21947, -1, 21948, 21949, 21950, -1, 21951, 21952, 21953, -1, 21954, 21955, 21956, -1, 21957, 21958, 21959, -1, 21960, 21961, 21962, -1, 21963, 21964, 21965, -1, 21966, 21967, 21968, -1, 21969, 21970, 21971, -1, 21972, 21973, 21974, -1, 21975, 21976, 21977, -1, 21978, 21979, 21980, -1, 21981, 21982, 21983, -1, 21984, 21985, 21986, -1, 21987, 21988, 21989, -1, 21990, 21991, 21992, -1, 21993, 21994, 21995, -1, 21996, 21997, 21998, -1, 21999, 22000, 22001, -1, 22002, 22003, 22004, -1, 22005, 22006, 22007, -1, 22008, 22009, 22010, -1, 22011, 22012, 22013, -1, 22014, 22015, 22016, -1, 22017, 22018, 22019, -1, 22020, 22021, 22022, -1, 22023, 22024, 22025, -1, 22026, 22027, 22028, -1, 22029, 22030, 22031, -1, 22032, 22033, 22034, -1, 22035, 22036, 22037, -1, 22038, 22039, 22040, -1, 22041, 22042, 22043, -1, 22044, 22045, 22046, -1, 22047, 22048, 22049, -1, 22050, 22051, 22052, -1, 22053, 22054, 22055, -1, 22056, 22057, 22058, -1, 22059, 22060, 22061, -1, 22062, 22063, 22064, -1, 22065, 22066, 22067, -1, 22068, 22069, 22070, -1, 22071, 22072, 22073, -1, 22074, 22075, 22076, -1, 22077, 22078, 22079, -1, 22080, 22081, 22082, -1, 22083, 22084, 22085, -1, 22086, 22087, 22088, -1, 22089, 22090, 22091, -1, 22092, 22093, 22094, -1, 22095, 22096, 22097, -1, 22098, 22099, 22100, -1, 22101, 22102, 22103, -1, 22104, 22105, 22106, -1, 22107, 22108, 22109, -1, 22110, 22111, 22112, -1, 22113, 22114, 22115, -1, 22116, 22117, 22118, -1, 22119, 22120, 22121, -1, 22122, 22123, 22124, -1, 22125, 22126, 22127, -1, 22128, 22129, 22130, -1, 22131, 22132, 22133, -1, 22134, 22135, 22136, -1, 22137, 22138, 22139, -1, 22140, 22141, 22142, -1, 22143, 22144, 22145, -1, 22146, 22147, 22148, -1, 22149, 22150, 22151, -1, 22152, 22153, 22154, -1, 22155, 22156, 22157, -1, 22158, 22159, 22160, -1, 22161, 22162, 22163, -1, 22164, 22165, 22166, -1, 22167, 22168, 22169, -1, 22170, 22171, 22172, -1, 22173, 22174, 22175, -1, 22176, 22177, 22178, -1, 22179, 22180, 22181, -1, 22182, 22183, 22184, -1, 22185, 22186, 22187, -1, 22188, 22189, 22190, -1, 22191, 22192, 22193, -1, 22194, 22195, 22196, -1, 22197, 22198, 22199, -1, 22200, 22201, 22202, -1, 22203, 22204, 22205, -1, 22206, 22207, 22208, -1, 22209, 22210, 22211, -1, 22212, 22213, 22214, -1, 22215, 22216, 22217, -1, 22218, 22219, 22220, -1, 22221, 22222, 22223, -1, 22224, 22225, 22226, -1, 22227, 22228, 22229, -1, 22230, 22231, 22232, -1, 22233, 22234, 22235, -1, 22236, 22237, 22238, -1, 22239, 22240, 22241, -1, 22242, 22243, 22244, -1, 22245, 22246, 22247, -1, 22248, 22249, 22250, -1, 22251, 22252, 22253, -1, 22254, 22255, 22256, -1, 22257, 22258, 22259, -1, 22260, 22261, 22262, -1, 22263, 22264, 22265, -1, 22266, 22267, 22268, -1, 22269, 22270, 22271, -1, 22272, 22273, 22274, -1, 22275, 22276, 22277, -1, 22278, 22279, 22280, -1, 22281, 22282, 22283, -1, 22284, 22285, 22286, -1, 22287, 22288, 22289, -1, 22290, 22291, 22292, -1, 22293, 22294, 22295, -1, 22296, 22297, 22298, -1, 22299, 22300, 22301, -1, 22302, 22303, 22304, -1, 22305, 22306, 22307, -1, 22308, 22309, 22310, -1, 22311, 22312, 22313, -1, 22314, 22315, 22316, -1, 22317, 22318, 22319, -1, 22320, 22321, 22322, -1, 22323, 22324, 22325, -1, 22326, 22327, 22328, -1, 22329, 22330, 22331, -1, 22332, 22333, 22334, -1, 22335, 22336, 22337, -1, 22338, 22339, 22340, -1, 22341, 22342, 22343, -1, 22344, 22345, 22346, -1, 22347, 22348, 22349, -1, 22350, 22351, 22352, -1, 22353, 22354, 22355, -1, 22356, 22357, 22358, -1, 22359, 22360, 22361, -1, 22362, 22363, 22364, -1, 22365, 22366, 22367, -1, 22368, 22369, 22370, -1, 22371, 22372, 22373, -1, 22374, 22375, 22376, -1, 22377, 22378, 22379, -1, 22380, 22381, 22382, -1, 22383, 22384, 22385, -1, 22386, 22387, 22388, -1, 22389, 22390, 22391, -1, 22392, 22393, 22394, -1, 22395, 22396, 22397, -1, 22398, 22399, 22400, -1, 22401, 22402, 22403, -1, 22404, 22405, 22406, -1, 22407, 22408, 22409, -1, 22410, 22411, 22412, -1, 22413, 22414, 22415, -1, 22416, 22417, 22418, -1, 22419, 22420, 22421, -1, 22422, 22423, 22424, -1, 22425, 22426, 22427, -1, 22428, 22429, 22430, -1, 22431, 22432, 22433, -1, 22434, 22435, 22436, -1, 22437, 22438, 22439, -1, 22440, 22441, 22442, -1, 22443, 22444, 22445, -1, 22446, 22447, 22448, -1, 22449, 22450, 22451, -1, 22452, 22453, 22454, -1, 22455, 22456, 22457, -1, 22458, 22459, 22460, -1, 22461, 22462, 22463, -1, 22464, 22465, 22466, -1, 22467, 22468, 22469, -1, 22470, 22471, 22472, -1, 22473, 22474, 22475, -1, 22476, 22477, 22478, -1, 22479, 22480, 22481, -1, 22482, 22483, 22484, -1, 22485, 22486, 22487, -1, 22488, 22489, 22490, -1, 22491, 22492, 22493, -1, 22494, 22495, 22496, -1, 22497, 22498, 22499, -1, 22500, 22501, 22502, -1, 22503, 22504, 22505, -1, 22506, 22507, 22508, -1, 22509, 22510, 22511, -1, 22512, 22513, 22514, -1, 22515, 22516, 22517, -1, 22518, 22519, 22520, -1, 22521, 22522, 22523, -1, 22524, 22525, 22526, -1, 22527, 22528, 22529, -1, 22530, 22531, 22532, -1, 22533, 22534, 22535, -1, 22536, 22537, 22538, -1, 22539, 22540, 22541, -1, 22542, 22543, 22544, -1, 22545, 22546, 22547, -1, 22548, 22549, 22550, -1, 22551, 22552, 22553, -1, 22554, 22555, 22556, -1, 22557, 22558, 22559, -1, 22560, 22561, 22562, -1, 22563, 22564, 22565, -1, 22566, 22567, 22568, -1, 22569, 22570, 22571, -1, 22572, 22573, 22574, -1, 22575, 22576, 22577, -1, 22578, 22579, 22580, -1, 22581, 22582, 22583, -1, 22584, 22585, 22586, -1, 22587, 22588, 22589, -1, 22590, 22591, 22592, -1, 22593, 22594, 22595, -1, 22596, 22597, 22598, -1, 22599, 22600, 22601, -1, 22602, 22603, 22604, -1, 22605, 22606, 22607, -1, 22608, 22609, 22610, -1, 22611, 22612, 22613, -1, 22614, 22615, 22616, -1, 22617, 22618, 22619, -1, 22620, 22621, 22622, -1, 22623, 22624, 22625, -1, 22626, 22627, 22628, -1, 22629, 22630, 22631, -1, 22632, 22633, 22634, -1, 22635, 22636, 22637, -1, 22638, 22639, 22640, -1, 22641, 22642, 22643, -1, 22644, 22645, 22646, -1, 22647, 22648, 22649, -1, 22650, 22651, 22652, -1, 22653, 22654, 22655, -1, 22656, 22657, 22658, -1, 22659, 22660, 22661, -1, 22662, 22663, 22664, -1, 22665, 22666, 22667, -1, 22668, 22669, 22670, -1, 22671, 22672, 22673, -1, 22674, 22675, 22676, -1, 22677, 22678, 22679, -1, 22680, 22681, 22682, -1, 22683, 22684, 22685, -1, 22686, 22687, 22688, -1, 22689, 22690, 22691, -1, 22692, 22693, 22694, -1, 22695, 22696, 22697, -1, 22698, 22699, 22700, -1, 22701, 22702, 22703, -1, 22704, 22705, 22706, -1, 22707, 22708, 22709, -1, 22710, 22711, 22712, -1, 22713, 22714, 22715, -1, 22716, 22717, 22718, -1, 22719, 22720, 22721, -1, 22722, 22723, 22724, -1, 22725, 22726, 22727, -1, 22728, 22729, 22730, -1, 22731, 22732, 22733, -1, 22734, 22735, 22736, -1, 22737, 22738, 22739, -1, 22740, 22741, 22742, -1, 22743, 22744, 22745, -1, 22746, 22747, 22748, -1, 22749, 22750, 22751, -1, 22752, 22753, 22754, -1, 22755, 22756, 22757, -1, 22758, 22759, 22760, -1, 22761, 22762, 22763, -1, 22764, 22765, 22766, -1, 22767, 22768, 22769, -1, 22770, 22771, 22772, -1, 22773, 22774, 22775, -1, 22776, 22777, 22778, -1, 22779, 22780, 22781, -1, 22782, 22783, 22784, -1, 22785, 22786, 22787, -1, 22788, 22789, 22790, -1, 22791, 22792, 22793, -1, 22794, 22795, 22796, -1, 22797, 22798, 22799, -1, 22800, 22801, 22802, -1, 22803, 22804, 22805, -1, 22806, 22807, 22808, -1, 22809, 22810, 22811, -1, 22812, 22813, 22814, -1, 22815, 22816, 22817, -1, 22818, 22819, 22820, -1, 22821, 22822, 22823, -1, 22824, 22825, 22826, -1, 22827, 22828, 22829, -1, 22830, 22831, 22832, -1, 22833, 22834, 22835, -1, 22836, 22837, 22838, -1, 22839, 22840, 22841, -1, 22842, 22843, 22844, -1, 22845, 22846, 22847, -1, 22848, 22849, 22850, -1, 22851, 22852, 22853, -1, 22854, 22855, 22856, -1, 22857, 22858, 22859, -1, 22860, 22861, 22862, -1, 22863, 22864, 22865, -1, 22866, 22867, 22868, -1, 22869, 22870, 22871, -1, 22872, 22873, 22874, -1, 22875, 22876, 22877, -1, 22878, 22879, 22880, -1, 22881, 22882, 22883, -1, 22884, 22885, 22886, -1, 22887, 22888, 22889, -1, 22890, 22891, 22892, -1, 22893, 22894, 22895, -1, 22896, 22897, 22898, -1, 22899, 22900, 22901, -1, 22902, 22903, 22904, -1, 22905, 22906, 22907, -1, 22908, 22909, 22910, -1, 22911, 22912, 22913, -1, 22914, 22915, 22916, -1, 22917, 22918, 22919, -1, 22920, 22921, 22922, -1, 22923, 22924, 22925, -1, 22926, 22927, 22928, -1, 22929, 22930, 22931, -1, 22932, 22933, 22934, -1, 22935, 22936, 22937, -1, 22938, 22939, 22940, -1, 22941, 22942, 22943, -1, 22944, 22945, 22946, -1, 22947, 22948, 22949, -1, 22950, 22951, 22952, -1, 22953, 22954, 22955, -1, 22956, 22957, 22958, -1, 22959, 22960, 22961, -1, 22962, 22963, 22964, -1, 22965, 22966, 22967, -1, 22968, 22969, 22970, -1, 22971, 22972, 22973, -1, 22974, 22975, 22976, -1, 22977, 22978, 22979, -1, 22980, 22981, 22982, -1, 22983, 22984, 22985, -1, 22986, 22987, 22988, -1, 22989, 22990, 22991, -1, 22992, 22993, 22994, -1, 22995, 22996, 22997, -1, 22998, 22999, 23000, -1, 23001, 23002, 23003, -1, 23004, 23005, 23006, -1, 23007, 23008, 23009, -1, 23010, 23011, 23012, -1, 23013, 23014, 23015, -1, 23016, 23017, 23018, -1, 23019, 23020, 23021, -1, 23022, 23023, 23024, -1, 23025, 23026, 23027, -1, 23028, 23029, 23030, -1, 23031, 23032, 23033, -1, 23034, 23035, 23036, -1, 23037, 23038, 23039, -1, 23040, 23041, 23042, -1, 23043, 23044, 23045, -1, 23046, 23047, 23048, -1, 23049, 23050, 23051, -1, 23052, 23053, 23054, -1, 23055, 23056, 23057, -1, 23058, 23059, 23060, -1, 23061, 23062, 23063, -1, 23064, 23065, 23066, -1, 23067, 23068, 23069, -1, 23070, 23071, 23072, -1, 23073, 23074, 23075, -1, 23076, 23077, 23078, -1, 23079, 23080, 23081, -1, 23082, 23083, 23084, -1, 23085, 23086, 23087, -1, 23088, 23089, 23090, -1, 23091, 23092, 23093, -1, 23094, 23095, 23096, -1, 23097, 23098, 23099, -1, 23100, 23101, 23102, -1, 23103, 23104, 23105, -1, 23106, 23107, 23108, -1, 23109, 23110, 23111, -1, 23112, 23113, 23114, -1, 23115, 23116, 23117, -1, 23118, 23119, 23120, -1, 23121, 23122, 23123, -1, 23124, 23125, 23126, -1, 23127, 23128, 23129, -1, 23130, 23131, 23132, -1, 23133, 23134, 23135, -1, 23136, 23137, 23138, -1, 23139, 23140, 23141, -1, 23142, 23143, 23144, -1, 23145, 23146, 23147, -1, 23148, 23149, 23150, -1, 23151, 23152, 23153, -1, 23154, 23155, 23156, -1, 23157, 23158, 23159, -1, 23160, 23161, 23162, -1, 23163, 23164, 23165, -1, 23166, 23167, 23168, -1, 23169, 23170, 23171, -1, 23172, 23173, 23174, -1, 23175, 23176, 23177, -1, 23178, 23179, 23180, -1, 23181, 23182, 23183, -1, 23184, 23185, 23186, -1, 23187, 23188, 23189, -1, 23190, 23191, 23192, -1, 23193, 23194, 23195, -1, 23196, 23197, 23198, -1, 23199, 23200, 23201, -1, 23202, 23203, 23204, -1, 23205, 23206, 23207, -1, 23208, 23209, 23210, -1, 23211, 23212, 23213, -1, 23214, 23215, 23216, -1, 23217, 23218, 23219, -1, 23220, 23221, 23222, -1, 23223, 23224, 23225, -1, 23226, 23227, 23228, -1, 23229, 23230, 23231, -1, 23232, 23233, 23234, -1, 23235, 23236, 23237, -1, 23238, 23239, 23240, -1, 23241, 23242, 23243, -1, 23244, 23245, 23246, -1, 23247, 23248, 23249, -1, 23250, 23251, 23252, -1, 23253, 23254, 23255, -1, 23256, 23257, 23258, -1, 23259, 23260, 23261, -1, 23262, 23263, 23264, -1, 23265, 23266, 23267, -1, 23268, 23269, 23270, -1, 23271, 23272, 23273, -1, 23274, 23275, 23276, -1, 23277, 23278, 23279, -1, 23280, 23281, 23282, -1, 23283, 23284, 23285, -1, 23286, 23287, 23288, -1, 23289, 23290, 23291, -1, 23292, 23293, 23294, -1, 23295, 23296, 23297, -1, 23298, 23299, 23300, -1, 23301, 23302, 23303, -1, 23304, 23305, 23306, -1, 23307, 23308, 23309, -1, 23310, 23311, 23312, -1, 23313, 23314, 23315, -1, 23316, 23317, 23318, -1, 23319, 23320, 23321, -1, 23322, 23323, 23324, -1, 23325, 23326, 23327, -1, 23328, 23329, 23330, -1, 23331, 23332, 23333, -1, 23334, 23335, 23336, -1, 23337, 23338, 23339, -1, 23340, 23341, 23342, -1, 23343, 23344, 23345, -1, 23346, 23347, 23348, -1, 23349, 23350, 23351, -1, 23352, 23353, 23354, -1, 23355, 23356, 23357, -1, 23358, 23359, 23360, -1, 23361, 23362, 23363, -1, 23364, 23365, 23366, -1, 23367, 23368, 23369, -1, 23370, 23371, 23372, -1, 23373, 23374, 23375, -1, 23376, 23377, 23378, -1, 23379, 23380, 23381, -1, 23382, 23383, 23384, -1, 23385, 23386, 23387, -1, 23388, 23389, 23390, -1, 23391, 23392, 23393, -1, 23394, 23395, 23396, -1, 23397, 23398, 23399, -1, 23400, 23401, 23402, -1, 23403, 23404, 23405, -1, 23406, 23407, 23408, -1, 23409, 23410, 23411, -1, 23412, 23413, 23414, -1, 23415, 23416, 23417, -1, 23418, 23419, 23420, -1, 23421, 23422, 23423, -1, 23424, 23425, 23426, -1, 23427, 23428, 23429, -1, 23430, 23431, 23432, -1, 23433, 23434, 23435, -1, 23436, 23437, 23438, -1, 23439, 23440, 23441, -1, 23442, 23443, 23444, -1, 23445, 23446, 23447, -1, 23448, 23449, 23450, -1, 23451, 23452, 23453, -1, 23454, 23455, 23456, -1, 23457, 23458, 23459, -1, 23460, 23461, 23462, -1, 23463, 23464, 23465, -1, 23466, 23467, 23468, -1, 23469, 23470, 23471, -1, 23472, 23473, 23474, -1, 23475, 23476, 23477, -1, 23478, 23479, 23480, -1, 23481, 23482, 23483, -1, 23484, 23485, 23486, -1, 23487, 23488, 23489, -1, 23490, 23491, 23492, -1, 23493, 23494, 23495, -1, 23496, 23497, 23498, -1, 23499, 23500, 23501, -1, 23502, 23503, 23504, -1, 23505, 23506, 23507, -1, 23508, 23509, 23510, -1, 23511, 23512, 23513, -1, 23514, 23515, 23516, -1, 23517, 23518, 23519, -1, 23520, 23521, 23522, -1, 23523, 23524, 23525, -1, 23526, 23527, 23528, -1, 23529, 23530, 23531, -1, 23532, 23533, 23534, -1, 23535, 23536, 23537, -1, 23538, 23539, 23540, -1, 23541, 23542, 23543, -1, 23544, 23545, 23546, -1, 23547, 23548, 23549, -1, 23550, 23551, 23552, -1, 23553, 23554, 23555, -1, 23556, 23557, 23558, -1, 23559, 23560, 23561, -1, 23562, 23563, 23564, -1, 23565, 23566, 23567, -1, 23568, 23569, 23570, -1, 23571, 23572, 23573, -1, 23574, 23575, 23576, -1, 23577, 23578, 23579, -1, 23580, 23581, 23582, -1, 23583, 23584, 23585, -1, 23586, 23587, 23588, -1, 23589, 23590, 23591, -1, 23592, 23593, 23594, -1, 23595, 23596, 23597, -1, 23598, 23599, 23600, -1, 23601, 23602, 23603, -1, 23604, 23605, 23606, -1, 23607, 23608, 23609, -1, 23610, 23611, 23612, -1, 23613, 23614, 23615, -1, 23616, 23617, 23618, -1, 23619, 23620, 23621, -1, 23622, 23623, 23624, -1, 23625, 23626, 23627, -1, 23628, 23629, 23630, -1, 23631, 23632, 23633, -1, 23634, 23635, 23636, -1, 23637, 23638, 23639, -1, 23640, 23641, 23642, -1, 23643, 23644, 23645, -1, 23646, 23647, 23648, -1, 23649, 23650, 23651, -1, 23652, 23653, 23654, -1, 23655, 23656, 23657, -1, 23658, 23659, 23660, -1, 23661, 23662, 23663, -1, 23664, 23665, 23666, -1, 23667, 23668, 23669, -1, 23670, 23671, 23672, -1, 23673, 23674, 23675, -1, 23676, 23677, 23678, -1, 23679, 23680, 23681, -1, 23682, 23683, 23684, -1, 23685, 23686, 23687, -1, 23688, 23689, 23690, -1, 23691, 23692, 23693, -1, 23694, 23695, 23696, -1, 23697, 23698, 23699, -1, 23700, 23701, 23702, -1, 23703, 23704, 23705, -1, 23706, 23707, 23708, -1, 23709, 23710, 23711, -1, 23712, 23713, 23714, -1, 23715, 23716, 23717, -1, 23718, 23719, 23720, -1, 23721, 23722, 23723, -1, 23724, 23725, 23726, -1, 23727, 23728, 23729, -1, 23730, 23731, 23732, -1, 23733, 23734, 23735, -1, 23736, 23737, 23738, -1, 23739, 23740, 23741, -1, 23742, 23743, 23744, -1, 23745, 23746, 23747, -1, 23748, 23749, 23750, -1, 23751, 23752, 23753, -1, 23754, 23755, 23756, -1, 23757, 23758, 23759, -1, 23760, 23761, 23762, -1, 23763, 23764, 23765, -1, 23766, 23767, 23768, -1, 23769, 23770, 23771, -1, 23772, 23773, 23774, -1, 23775, 23776, 23777, -1, 23778, 23779, 23780, -1, 23781, 23782, 23783, -1, 23784, 23785, 23786, -1, 23787, 23788, 23789, -1, 23790, 23791, 23792, -1, 23793, 23794, 23795, -1, 23796, 23797, 23798, -1, 23799, 23800, 23801, -1, 23802, 23803, 23804, -1, 23805, 23806, 23807, -1, 23808, 23809, 23810, -1, 23811, 23812, 23813, -1, 23814, 23815, 23816, -1, 23817, 23818, 23819, -1, 23820, 23821, 23822, -1, 23823, 23824, 23825, -1, 23826, 23827, 23828, -1, 23829, 23830, 23831, -1, 23832, 23833, 23834, -1, 23835, 23836, 23837, -1, 23838, 23839, 23840, -1, 23841, 23842, 23843, -1, 23844, 23845, 23846, -1, 23847, 23848, 23849, -1, 23850, 23851, 23852, -1, 23853, 23854, 23855, -1, 23856, 23857, 23858, -1, 23859, 23860, 23861, -1, 23862, 23863, 23864, -1, 23865, 23866, 23867, -1, 23868, 23869, 23870, -1, 23871, 23872, 23873, -1, 23874, 23875, 23876, -1, 23877, 23878, 23879, -1, 23880, 23881, 23882, -1, 23883, 23884, 23885, -1, 23886, 23887, 23888, -1, 23889, 23890, 23891, -1, 23892, 23893, 23894, -1, 23895, 23896, 23897, -1, 23898, 23899, 23900, -1, 23901, 23902, 23903, -1, 23904, 23905, 23906, -1, 23907, 23908, 23909, -1, 23910, 23911, 23912, -1, 23913, 23914, 23915, -1, 23916, 23917, 23918, -1, 23919, 23920, 23921, -1, 23922, 23923, 23924, -1, 23925, 23926, 23927, -1, 23928, 23929, 23930, -1, 23931, 23932, 23933, -1, 23934, 23935, 23936, -1, 23937, 23938, 23939, -1, 23940, 23941, 23942, -1, 23943, 23944, 23945, -1, 23946, 23947, 23948, -1, 23949, 23950, 23951, -1, 23952, 23953, 23954, -1, 23955, 23956, 23957, -1, 23958, 23959, 23960, -1, 23961, 23962, 23963, -1, 23964, 23965, 23966, -1, 23967, 23968, 23969, -1, 23970, 23971, 23972, -1, 23973, 23974, 23975, -1, 23976, 23977, 23978, -1, 23979, 23980, 23981, -1, 23982, 23983, 23984, -1, 23985, 23986, 23987, -1, 23988, 23989, 23990, -1, 23991, 23992, 23993, -1, 23994, 23995, 23996, -1, 23997, 23998, 23999, -1, 24000, 24001, 24002, -1, 24003, 24004, 24005, -1, 24006, 24007, 24008, -1, 24009, 24010, 24011, -1, 24012, 24013, 24014, -1, 24015, 24016, 24017, -1, 24018, 24019, 24020, -1, 24021, 24022, 24023, -1, 24024, 24025, 24026, -1, 24027, 24028, 24029, -1, 24030, 24031, 24032, -1, 24033, 24034, 24035, -1, 24036, 24037, 24038, -1, 24039, 24040, 24041, -1, 24042, 24043, 24044, -1, 24045, 24046, 24047, -1, 24048, 24049, 24050, -1, 24051, 24052, 24053, -1, 24054, 24055, 24056, -1, 24057, 24058, 24059, -1, 24060, 24061, 24062, -1, 24063, 24064, 24065, -1, 24066, 24067, 24068, -1, 24069, 24070, 24071, -1, 24072, 24073, 24074, -1, 24075, 24076, 24077, -1, 24078, 24079, 24080, -1, 24081, 24082, 24083, -1, 24084, 24085, 24086, -1, 24087, 24088, 24089, -1, 24090, 24091, 24092, -1, 24093, 24094, 24095, -1, 24096, 24097, 24098, -1, 24099, 24100, 24101, -1, 24102, 24103, 24104, -1, 24105, 24106, 24107, -1, 24108, 24109, 24110, -1, 24111, 24112, 24113, -1, 24114, 24115, 24116, -1, 24117, 24118, 24119, -1, 24120, 24121, 24122, -1, 24123, 24124, 24125, -1, 24126, 24127, 24128, -1, 24129, 24130, 24131, -1, 24132, 24133, 24134, -1, 24135, 24136, 24137, -1, 24138, 24139, 24140, -1, 24141, 24142, 24143, -1, 24144, 24145, 24146, -1, 24147, 24148, 24149, -1, 24150, 24151, 24152, -1, 24153, 24154, 24155, -1, 24156, 24157, 24158, -1, 24159, 24160, 24161, -1, 24162, 24163, 24164, -1, 24165, 24166, 24167, -1, 24168, 24169, 24170, -1, 24171, 24172, 24173, -1, 24174, 24175, 24176, -1, 24177, 24178, 24179, -1, 24180, 24181, 24182, -1, 24183, 24184, 24185, -1, 24186, 24187, 24188, -1, 24189, 24190, 24191, -1, 24192, 24193, 24194, -1, 24195, 24196, 24197, -1, 24198, 24199, 24200, -1, 24201, 24202, 24203, -1, 24204, 24205, 24206, -1, 24207, 24208, 24209, -1, 24210, 24211, 24212, -1, 24213, 24214, 24215, -1, 24216, 24217, 24218, -1, 24219, 24220, 24221, -1, 24222, 24223, 24224, -1, 24225, 24226, 24227, -1, 24228, 24229, 24230, -1, 24231, 24232, 24233, -1, 24234, 24235, 24236, -1, 24237, 24238, 24239, -1, 24240, 24241, 24242, -1, 24243, 24244, 24245, -1, 24246, 24247, 24248, -1, 24249, 24250, 24251, -1, 24252, 24253, 24254, -1, 24255, 24256, 24257, -1, 24258, 24259, 24260, -1, 24261, 24262, 24263, -1, 24264, 24265, 24266, -1, 24267, 24268, 24269, -1, 24270, 24271, 24272, -1, 24273, 24274, 24275, -1, 24276, 24277, 24278, -1, 24279, 24280, 24281, -1, 24282, 24283, 24284, -1, 24285, 24286, 24287, -1, 24288, 24289, 24290, -1, 24291, 24292, 24293, -1, 24294, 24295, 24296, -1, 24297, 24298, 24299, -1, 24300, 24301, 24302, -1, 24303, 24304, 24305, -1, 24306, 24307, 24308, -1, 24309, 24310, 24311, -1, 24312, 24313, 24314, -1, 24315, 24316, 24317, -1, 24318, 24319, 24320, -1, 24321, 24322, 24323, -1, 24324, 24325, 24326, -1, 24327, 24328, 24329, -1, 24330, 24331, 24332, -1, 24333, 24334, 24335, -1, 24336, 24337, 24338, -1, 24339, 24340, 24341, -1, 24342, 24343, 24344, -1, 24345, 24346, 24347, -1, 24348, 24349, 24350, -1, 24351, 24352, 24353, -1, 24354, 24355, 24356, -1, 24357, 24358, 24359, -1, 24360, 24361, 24362, -1, 24363, 24364, 24365, -1, 24366, 24367, 24368, -1, 24369, 24370, 24371, -1, 24372, 24373, 24374, -1, 24375, 24376, 24377, -1, 24378, 24379, 24380, -1, 24381, 24382, 24383, -1, 24384, 24385, 24386, -1, 24387, 24388, 24389, -1, 24390, 24391, 24392, -1, 24393, 24394, 24395, -1, 24396, 24397, 24398, -1, 24399, 24400, 24401, -1, 24402, 24403, 24404, -1, 24405, 24406, 24407, -1, 24408, 24409, 24410, -1, 24411, 24412, 24413, -1, 24414, 24415, 24416, -1, 24417, 24418, 24419, -1, 24420, 24421, 24422, -1, 24423, 24424, 24425, -1, 24426, 24427, 24428, -1, 24429, 24430, 24431, -1, 24432, 24433, 24434, -1, 24435, 24436, 24437, -1, 24438, 24439, 24440, -1, 24441, 24442, 24443, -1, 24444, 24445, 24446, -1, 24447, 24448, 24449, -1, 24450, 24451, 24452, -1, 24453, 24454, 24455, -1, 24456, 24457, 24458, -1, 24459, 24460, 24461, -1, 24462, 24463, 24464, -1, 24465, 24466, 24467, -1, 24468, 24469, 24470, -1, 24471, 24472, 24473, -1, 24474, 24475, 24476, -1, 24477, 24478, 24479, -1, 24480, 24481, 24482, -1, 24483, 24484, 24485, -1, 24486, 24487, 24488, -1, 24489, 24490, 24491, -1, 24492, 24493, 24494, -1, 24495, 24496, 24497, -1, 24498, 24499, 24500, -1, 24501, 24502, 24503, -1, 24504, 24505, 24506, -1, 24507, 24508, 24509, -1, 24510, 24511, 24512, -1, 24513, 24514, 24515, -1, 24516, 24517, 24518, -1, 24519, 24520, 24521, -1, 24522, 24523, 24524, -1, 24525, 24526, 24527, -1, 24528, 24529, 24530, -1, 24531, 24532, 24533, -1, 24534, 24535, 24536, -1, 24537, 24538, 24539, -1, 24540, 24541, 24542, -1, 24543, 24544, 24545, -1, 24546, 24547, 24548, -1, 24549, 24550, 24551, -1, 24552, 24553, 24554, -1, 24555, 24556, 24557, -1, 24558, 24559, 24560, -1, 24561, 24562, 24563, -1, 24564, 24565, 24566, -1, 24567, 24568, 24569, -1, 24570, 24571, 24572, -1, 24573, 24574, 24575, -1, 24576, 24577, 24578, -1, 24579, 24580, 24581, -1, 24582, 24583, 24584, -1, 24585, 24586, 24587, -1, 24588, 24589, 24590, -1, 24591, 24592, 24593, -1, 24594, 24595, 24596, -1, 24597, 24598, 24599, -1, 24600, 24601, 24602, -1, 24603, 24604, 24605, -1, 24606, 24607, 24608, -1, 24609, 24610, 24611, -1, 24612, 24613, 24614, -1, 24615, 24616, 24617, -1, 24618, 24619, 24620, -1, 24621, 24622, 24623, -1, 24624, 24625, 24626, -1, 24627, 24628, 24629, -1, 24630, 24631, 24632, -1, 24633, 24634, 24635, -1, 24636, 24637, 24638, -1, 24639, 24640, 24641, -1, 24642, 24643, 24644, -1, 24645, 24646, 24647, -1, 24648, 24649, 24650, -1, 24651, 24652, 24653, -1, 24654, 24655, 24656, -1, 24657, 24658, 24659, -1, 24660, 24661, 24662, -1, 24663, 24664, 24665, -1, 24666, 24667, 24668, -1, 24669, 24670, 24671, -1, 24672, 24673, 24674, -1, 24675, 24676, 24677, -1, 24678, 24679, 24680, -1, 24681, 24682, 24683, -1, 24684, 24685, 24686, -1, 24687, 24688, 24689, -1, 24690, 24691, 24692, -1, 24693, 24694, 24695, -1, 24696, 24697, 24698, -1, 24699, 24700, 24701, -1, 24702, 24703, 24704, -1, 24705, 24706, 24707, -1, 24708, 24709, 24710, -1, 24711, 24712, 24713, -1, 24714, 24715, 24716, -1, 24717, 24718, 24719, -1, 24720, 24721, 24722, -1, 24723, 24724, 24725, -1, 24726, 24727, 24728, -1, 24729, 24730, 24731, -1, 24732, 24733, 24734, -1, 24735, 24736, 24737, -1, 24738, 24739, 24740, -1, 24741, 24742, 24743, -1, 24744, 24745, 24746, -1, 24747, 24748, 24749, -1, 24750, 24751, 24752, -1, 24753, 24754, 24755, -1, 24756, 24757, 24758, -1, 24759, 24760, 24761, -1, 24762, 24763, 24764, -1, 24765, 24766, 24767, -1, 24768, 24769, 24770, -1, 24771, 24772, 24773, -1, 24774, 24775, 24776, -1, 24777, 24778, 24779, -1, 24780, 24781, 24782, -1, 24783, 24784, 24785, -1, 24786, 24787, 24788, -1, 24789, 24790, 24791, -1, 24792, 24793, 24794, -1, 24795, 24796, 24797, -1, 24798, 24799, 24800, -1, 24801, 24802, 24803, -1, 24804, 24805, 24806, -1, 24807, 24808, 24809, -1, 24810, 24811, 24812, -1, 24813, 24814, 24815, -1, 24816, 24817, 24818, -1, 24819, 24820, 24821, -1, 24822, 24823, 24824, -1, 24825, 24826, 24827, -1, 24828, 24829, 24830, -1, 24831, 24832, 24833, -1, 24834, 24835, 24836, -1, 24837, 24838, 24839, -1, 24840, 24841, 24842, -1, 24843, 24844, 24845, -1, 24846, 24847, 24848, -1, 24849, 24850, 24851, -1, 24852, 24853, 24854, -1, 24855, 24856, 24857, -1, 24858, 24859, 24860, -1, 24861, 24862, 24863, -1, 24864, 24865, 24866, -1, 24867, 24868, 24869, -1, 24870, 24871, 24872, -1, 24873, 24874, 24875, -1, 24876, 24877, 24878, -1, 24879, 24880, 24881, -1, 24882, 24883, 24884, -1, 24885, 24886, 24887, -1, 24888, 24889, 24890, -1, 24891, 24892, 24893, -1, 24894, 24895, 24896, -1, 24897, 24898, 24899, -1, 24900, 24901, 24902, -1, 24903, 24904, 24905, -1, 24906, 24907, 24908, -1, 24909, 24910, 24911, -1, 24912, 24913, 24914, -1, 24915, 24916, 24917, -1, 24918, 24919, 24920, -1, 24921, 24922, 24923, -1, 24924, 24925, 24926, -1, 24927, 24928, 24929, -1, 24930, 24931, 24932, -1, 24933, 24934, 24935, -1, 24936, 24937, 24938, -1, 24939, 24940, 24941, -1, 24942, 24943, 24944, -1, 24945, 24946, 24947, -1, 24948, 24949, 24950, -1, 24951, 24952, 24953, -1, 24954, 24955, 24956, -1, 24957, 24958, 24959, -1, 24960, 24961, 24962, -1, 24963, 24964, 24965, -1, 24966, 24967, 24968, -1, 24969, 24970, 24971, -1, 24972, 24973, 24974, -1, 24975, 24976, 24977, -1, 24978, 24979, 24980, -1, 24981, 24982, 24983, -1, 24984, 24985, 24986, -1, 24987, 24988, 24989, -1, 24990, 24991, 24992, -1, 24993, 24994, 24995, -1, 24996, 24997, 24998, -1, 24999, 25000, 25001, -1, 25002, 25003, 25004, -1, 25005, 25006, 25007, -1, 25008, 25009, 25010, -1, 25011, 25012, 25013, -1, 25014, 25015, 25016, -1, 25017, 25018, 25019, -1, 25020, 25021, 25022, -1, 25023, 25024, 25025, -1, 25026, 25027, 25028, -1, 25029, 25030, 25031, -1, 25032, 25033, 25034, -1, 25035, 25036, 25037, -1, 25038, 25039, 25040, -1, 25041, 25042, 25043, -1, 25044, 25045, 25046, -1, 25047, 25048, 25049, -1, 25050, 25051, 25052, -1, 25053, 25054, 25055, -1, 25056, 25057, 25058, -1, 25059, 25060, 25061, -1, 25062, 25063, 25064, -1, 25065, 25066, 25067, -1, 25068, 25069, 25070, -1, 25071, 25072, 25073, -1, 25074, 25075, 25076, -1, 25077, 25078, 25079, -1, 25080, 25081, 25082, -1, 25083, 25084, 25085, -1, 25086, 25087, 25088, -1, 25089, 25090, 25091, -1, 25092, 25093, 25094, -1, 25095, 25096, 25097, -1, 25098, 25099, 25100, -1, 25101, 25102, 25103, -1, 25104, 25105, 25106, -1, 25107, 25108, 25109, -1, 25110, 25111, 25112, -1, 25113, 25114, 25115, -1, 25116, 25117, 25118, -1, 25119, 25120, 25121, -1, 25122, 25123, 25124, -1, 25125, 25126, 25127, -1, 25128, 25129, 25130, -1, 25131, 25132, 25133, -1, 25134, 25135, 25136, -1, 25137, 25138, 25139, -1, 25140, 25141, 25142, -1, 25143, 25144, 25145, -1, 25146, 25147, 25148, -1, 25149, 25150, 25151, -1, 25152, 25153, 25154, -1, 25155, 25156, 25157, -1, 25158, 25159, 25160, -1, 25161, 25162, 25163, -1, 25164, 25165, 25166, -1, 25167, 25168, 25169, -1, 25170, 25171, 25172, -1, 25173, 25174, 25175, -1, 25176, 25177, 25178, -1, 25179, 25180, 25181, -1, 25182, 25183, 25184, -1, 25185, 25186, 25187, -1, 25188, 25189, 25190, -1, 25191, 25192, 25193, -1, 25194, 25195, 25196, -1, 25197, 25198, 25199, -1, 25200, 25201, 25202, -1, 25203, 25204, 25205, -1, 25206, 25207, 25208, -1, 25209, 25210, 25211, -1, 25212, 25213, 25214, -1, 25215, 25216, 25217, -1, 25218, 25219, 25220, -1, 25221, 25222, 25223, -1, 25224, 25225, 25226, -1, 25227, 25228, 25229, -1, 25230, 25231, 25232, -1, 25233, 25234, 25235, -1, 25236, 25237, 25238, -1, 25239, 25240, 25241, -1, 25242, 25243, 25244, -1, 25245, 25246, 25247, -1, 25248, 25249, 25250, -1, 25251, 25252, 25253, -1, 25254, 25255, 25256, -1, 25257, 25258, 25259, -1, 25260, 25261, 25262, -1, 25263, 25264, 25265, -1, 25266, 25267, 25268, -1, 25269, 25270, 25271, -1, 25272, 25273, 25274, -1, 25275, 25276, 25277, -1, 25278, 25279, 25280, -1, 25281, 25282, 25283, -1, 25284, 25285, 25286, -1, 25287, 25288, 25289, -1, 25290, 25291, 25292, -1, 25293, 25294, 25295, -1, 25296, 25297, 25298, -1, 25299, 25300, 25301, -1, 25302, 25303, 25304, -1, 25305, 25306, 25307, -1, 25308, 25309, 25310, -1, 25311, 25312, 25313, -1, 25314, 25315, 25316, -1, 25317, 25318, 25319, -1, 25320, 25321, 25322, -1, 25323, 25324, 25325, -1, 25326, 25327, 25328, -1, 25329, 25330, 25331, -1, 25332, 25333, 25334, -1, 25335, 25336, 25337, -1, 25338, 25339, 25340, -1, 25341, 25342, 25343, -1, 25344, 25345, 25346, -1, 25347, 25348, 25349, -1, 25350, 25351, 25352, -1, 25353, 25354, 25355, -1, 25356, 25357, 25358, -1, 25359, 25360, 25361, -1, 25362, 25363, 25364, -1, 25365, 25366, 25367, -1, 25368, 25369, 25370, -1, 25371, 25372, 25373, -1, 25374, 25375, 25376, -1, 25377, 25378, 25379, -1, 25380, 25381, 25382, -1, 25383, 25384, 25385, -1, 25386, 25387, 25388, -1, 25389, 25390, 25391, -1, 25392, 25393, 25394, -1, 25395, 25396, 25397, -1, 25398, 25399, 25400, -1, 25401, 25402, 25403, -1, 25404, 25405, 25406, -1, 25407, 25408, 25409, -1, 25410, 25411, 25412, -1, 25413, 25414, 25415, -1, 25416, 25417, 25418, -1, 25419, 25420, 25421, -1, 25422, 25423, 25424, -1, 25425, 25426, 25427, -1, 25428, 25429, 25430, -1, 25431, 25432, 25433, -1, 25434, 25435, 25436, -1, 25437, 25438, 25439, -1, 25440, 25441, 25442, -1, 25443, 25444, 25445, -1, 25446, 25447, 25448, -1, 25449, 25450, 25451, -1, 25452, 25453, 25454, -1, 25455, 25456, 25457, -1, 25458, 25459, 25460, -1, 25461, 25462, 25463, -1, 25464, 25465, 25466, -1, 25467, 25468, 25469, -1, 25470, 25471, 25472, -1, 25473, 25474, 25475, -1, 25476, 25477, 25478, -1, 25479, 25480, 25481, -1, 25482, 25483, 25484, -1, 25485, 25486, 25487, -1, 25488, 25489, 25490, -1, 25491, 25492, 25493, -1, 25494, 25495, 25496, -1, 25497, 25498, 25499, -1, 25500, 25501, 25502, -1, 25503, 25504, 25505, -1, 25506, 25507, 25508, -1, 25509, 25510, 25511, -1, 25512, 25513, 25514, -1, 25515, 25516, 25517, -1, 25518, 25519, 25520, -1, 25521, 25522, 25523, -1, 25524, 25525, 25526, -1, 25527, 25528, 25529, -1, 25530, 25531, 25532, -1, 25533, 25534, 25535, -1, 25536, 25537, 25538, -1, 25539, 25540, 25541, -1, 25542, 25543, 25544, -1, 25545, 25546, 25547, -1, 25548, 25549, 25550, -1, 25551, 25552, 25553, -1, 25554, 25555, 25556, -1, 25557, 25558, 25559, -1, 25560, 25561, 25562, -1, 25563, 25564, 25565, -1, 25566, 25567, 25568, -1, 25569, 25570, 25571, -1, 25572, 25573, 25574, -1, 25575, 25576, 25577, -1, 25578, 25579, 25580, -1, 25581, 25582, 25583, -1, 25584, 25585, 25586, -1, 25587, 25588, 25589, -1, 25590, 25591, 25592, -1, 25593, 25594, 25595, -1, 25596, 25597, 25598, -1, 25599, 25600, 25601, -1, 25602, 25603, 25604, -1, 25605, 25606, 25607, -1, 25608, 25609, 25610, -1, 25611, 25612, 25613, -1, 25614, 25615, 25616, -1, 25617, 25618, 25619, -1, 25620, 25621, 25622, -1, 25623, 25624, 25625, -1, 25626, 25627, 25628, -1, 25629, 25630, 25631, -1, 25632, 25633, 25634, -1, 25635, 25636, 25637, -1, 25638, 25639, 25640, -1, 25641, 25642, 25643, -1, 25644, 25645, 25646, -1, 25647, 25648, 25649, -1, 25650, 25651, 25652, -1, 25653, 25654, 25655, -1, 25656, 25657, 25658, -1, 25659, 25660, 25661, -1, 25662, 25663, 25664, -1, 25665, 25666, 25667, -1, 25668, 25669, 25670, -1, 25671, 25672, 25673, -1, 25674, 25675, 25676, -1, 25677, 25678, 25679, -1, 25680, 25681, 25682, -1, 25683, 25684, 25685, -1, 25686, 25687, 25688, -1, 25689, 25690, 25691, -1, 25692, 25693, 25694, -1, 25695, 25696, 25697, -1, 25698, 25699, 25700, -1, 25701, 25702, 25703, -1, 25704, 25705, 25706, -1, 25707, 25708, 25709, -1, 25710, 25711, 25712, -1, 25713, 25714, 25715, -1, 25716, 25717, 25718, -1, 25719, 25720, 25721, -1, 25722, 25723, 25724, -1, 25725, 25726, 25727, -1, 25728, 25729, 25730, -1, 25731, 25732, 25733, -1, 25734, 25735, 25736, -1, 25737, 25738, 25739, -1, 25740, 25741, 25742, -1, 25743, 25744, 25745, -1, 25746, 25747, 25748, -1, 25749, 25750, 25751, -1, 25752, 25753, 25754, -1, 25755, 25756, 25757, -1, 25758, 25759, 25760, -1, 25761, 25762, 25763, -1, 25764, 25765, 25766, -1, 25767, 25768, 25769, -1, 25770, 25771, 25772, -1, 25773, 25774, 25775, -1, 25776, 25777, 25778, -1, 25779, 25780, 25781, -1, 25782, 25783, 25784, -1, 25785, 25786, 25787, -1, 25788, 25789, 25790, -1, 25791, 25792, 25793, -1, 25794, 25795, 25796, -1, 25797, 25798, 25799, -1, 25800, 25801, 25802, -1, 25803, 25804, 25805, -1, 25806, 25807, 25808, -1, 25809, 25810, 25811, -1, 25812, 25813, 25814, -1, 25815, 25816, 25817, -1, 25818, 25819, 25820, -1, 25821, 25822, 25823, -1, 25824, 25825, 25826, -1, 25827, 25828, 25829, -1, 25830, 25831, 25832, -1, 25833, 25834, 25835, -1, 25836, 25837, 25838, -1, 25839, 25840, 25841, -1, 25842, 25843, 25844, -1, 25845, 25846, 25847, -1, 25848, 25849, 25850, -1, 25851, 25852, 25853, -1, 25854, 25855, 25856, -1, 25857, 25858, 25859, -1, 25860, 25861, 25862, -1, 25863, 25864, 25865, -1, 25866, 25867, 25868, -1, 25869, 25870, 25871, -1, 25872, 25873, 25874, -1, 25875, 25876, 25877, -1, 25878, 25879, 25880, -1, 25881, 25882, 25883, -1, 25884, 25885, 25886, -1, 25887, 25888, 25889, -1, 25890, 25891, 25892, -1, 25893, 25894, 25895, -1, 25896, 25897, 25898, -1, 25899, 25900, 25901, -1, 25902, 25903, 25904, -1, 25905, 25906, 25907, -1, 25908, 25909, 25910, -1, 25911, 25912, 25913, -1, 25914, 25915, 25916, -1, 25917, 25918, 25919, -1, 25920, 25921, 25922, -1, 25923, 25924, 25925, -1, 25926, 25927, 25928, -1, 25929, 25930, 25931, -1, 25932, 25933, 25934, -1, 25935, 25936, 25937, -1, 25938, 25939, 25940, -1, 25941, 25942, 25943, -1, 25944, 25945, 25946, -1, 25947, 25948, 25949, -1, 25950, 25951, 25952, -1, 25953, 25954, 25955, -1, 25956, 25957, 25958, -1, 25959, 25960, 25961, -1, 25962, 25963, 25964, -1, 25965, 25966, 25967, -1, 25968, 25969, 25970, -1, 25971, 25972, 25973, -1, 25974, 25975, 25976, -1, 25977, 25978, 25979, -1, 25980, 25981, 25982, -1, 25983, 25984, 25985, -1, 25986, 25987, 25988, -1, 25989, 25990, 25991, -1, 25992, 25993, 25994, -1, 25995, 25996, 25997, -1, 25998, 25999, 26000, -1, 26001, 26002, 26003, -1, 26004, 26005, 26006, -1, 26007, 26008, 26009, -1, 26010, 26011, 26012, -1, 26013, 26014, 26015, -1, 26016, 26017, 26018, -1, 26019, 26020, 26021, -1, 26022, 26023, 26024, -1, 26025, 26026, 26027, -1, 26028, 26029, 26030, -1, 26031, 26032, 26033, -1, 26034, 26035, 26036, -1, 26037, 26038, 26039, -1, 26040, 26041, 26042, -1, 26043, 26044, 26045, -1, 26046, 26047, 26048, -1, 26049, 26050, 26051, -1, 26052, 26053, 26054, -1, 26055, 26056, 26057, -1, 26058, 26059, 26060, -1, 26061, 26062, 26063, -1, 26064, 26065, 26066, -1, 26067, 26068, 26069, -1, 26070, 26071, 26072, -1, 26073, 26074, 26075, -1, 26076, 26077, 26078, -1, 26079, 26080, 26081, -1, 26082, 26083, 26084, -1, 26085, 26086, 26087, -1, 26088, 26089, 26090, -1, 26091, 26092, 26093, -1, 26094, 26095, 26096, -1, 26097, 26098, 26099, -1, 26100, 26101, 26102, -1, 26103, 26104, 26105, -1, 26106, 26107, 26108, -1, 26109, 26110, 26111, -1, 26112, 26113, 26114, -1, 26115, 26116, 26117, -1, 26118, 26119, 26120, -1, 26121, 26122, 26123, -1, 26124, 26125, 26126, -1, 26127, 26128, 26129, -1, 26130, 26131, 26132, -1, 26133, 26134, 26135, -1, 26136, 26137, 26138, -1, 26139, 26140, 26141, -1, 26142, 26143, 26144, -1, 26145, 26146, 26147, -1, 26148, 26149, 26150, -1, 26151, 26152, 26153, -1, 26154, 26155, 26156, -1, 26157, 26158, 26159, -1, 26160, 26161, 26162, -1, 26163, 26164, 26165, -1, 26166, 26167, 26168, -1, 26169, 26170, 26171, -1, 26172, 26173, 26174, -1, 26175, 26176, 26177, -1, 26178, 26179, 26180, -1, 26181, 26182, 26183, -1, 26184, 26185, 26186, -1, 26187, 26188, 26189, -1, 26190, 26191, 26192, -1, 26193, 26194, 26195, -1, 26196, 26197, 26198, -1, 26199, 26200, 26201, -1, 26202, 26203, 26204, -1, 26205, 26206, 26207, -1, 26208, 26209, 26210, -1, 26211, 26212, 26213, -1, 26214, 26215, 26216, -1, 26217, 26218, 26219, -1, 26220, 26221, 26222, -1, 26223, 26224, 26225, -1, 26226, 26227, 26228, -1, 26229, 26230, 26231, -1, 26232, 26233, 26234, -1, 26235, 26236, 26237, -1, 26238, 26239, 26240, -1, 26241, 26242, 26243, -1, 26244, 26245, 26246, -1, 26247, 26248, 26249, -1, 26250, 26251, 26252, -1, 26253, 26254, 26255, -1, 26256, 26257, 26258, -1, 26259, 26260, 26261, -1, 26262, 26263, 26264, -1, 26265, 26266, 26267, -1, 26268, 26269, 26270, -1, 26271, 26272, 26273, -1, 26274, 26275, 26276, -1, 26277, 26278, 26279, -1, 26280, 26281, 26282, -1, 26283, 26284, 26285, -1, 26286, 26287, 26288, -1, 26289, 26290, 26291, -1, 26292, 26293, 26294, -1, 26295, 26296, 26297, -1, 26298, 26299, 26300, -1, 26301, 26302, 26303, -1, 26304, 26305, 26306, -1, 26307, 26308, 26309, -1, 26310, 26311, 26312, -1, 26313, 26314, 26315, -1, 26316, 26317, 26318, -1, 26319, 26320, 26321, -1, 26322, 26323, 26324, -1, 26325, 26326, 26327, -1, 26328, 26329, 26330, -1, 26331, 26332, 26333, -1, 26334, 26335, 26336, -1, 26337, 26338, 26339, -1, 26340, 26341, 26342, -1, 26343, 26344, 26345, -1, 26346, 26347, 26348, -1, 26349, 26350, 26351, -1, 26352, 26353, 26354, -1, 26355, 26356, 26357, -1, 26358, 26359, 26360, -1, 26361, 26362, 26363, -1, 26364, 26365, 26366, -1, 26367, 26368, 26369, -1, 26370, 26371, 26372, -1, 26373, 26374, 26375, -1, 26376, 26377, 26378, -1, 26379, 26380, 26381, -1, 26382, 26383, 26384, -1, 26385, 26386, 26387, -1, 26388, 26389, 26390, -1, 26391, 26392, 26393, -1, 26394, 26395, 26396, -1, 26397, 26398, 26399, -1, 26400, 26401, 26402, -1, 26403, 26404, 26405, -1, 26406, 26407, 26408, -1, 26409, 26410, 26411, -1, 26412, 26413, 26414, -1, 26415, 26416, 26417, -1, 26418, 26419, 26420, -1, 26421, 26422, 26423, -1, 26424, 26425, 26426, -1, 26427, 26428, 26429, -1, 26430, 26431, 26432, -1, 26433, 26434, 26435, -1, 26436, 26437, 26438, -1, 26439, 26440, 26441, -1, 26442, 26443, 26444, -1, 26445, 26446, 26447, -1, 26448, 26449, 26450, -1, 26451, 26452, 26453, -1, 26454, 26455, 26456, -1, 26457, 26458, 26459, -1, 26460, 26461, 26462, -1, 26463, 26464, 26465, -1, 26466, 26467, 26468, -1, 26469, 26470, 26471, -1, 26472, 26473, 26474, -1, 26475, 26476, 26477, -1, 26478, 26479, 26480, -1, 26481, 26482, 26483, -1, 26484, 26485, 26486, -1, 26487, 26488, 26489, -1, 26490, 26491, 26492, -1, 26493, 26494, 26495, -1, 26496, 26497, 26498, -1, 26499, 26500, 26501, -1, 26502, 26503, 26504, -1, 26505, 26506, 26507, -1, 26508, 26509, 26510, -1, 26511, 26512, 26513, -1, 26514, 26515, 26516, -1, 26517, 26518, 26519, -1, 26520, 26521, 26522, -1, 26523, 26524, 26525, -1, 26526, 26527, 26528, -1, 26529, 26530, 26531, -1, 26532, 26533, 26534, -1, 26535, 26536, 26537, -1, 26538, 26539, 26540, -1, 26541, 26542, 26543, -1, 26544, 26545, 26546, -1, 26547, 26548, 26549, -1, 26550, 26551, 26552, -1, 26553, 26554, 26555, -1, 26556, 26557, 26558, -1, 26559, 26560, 26561, -1, 26562, 26563, 26564, -1, 26565, 26566, 26567, -1, 26568, 26569, 26570, -1, 26571, 26572, 26573, -1, 26574, 26575, 26576, -1, 26577, 26578, 26579, -1, 26580, 26581, 26582, -1, 26583, 26584, 26585, -1, 26586, 26587, 26588, -1, 26589, 26590, 26591, -1, 26592, 26593, 26594, -1, 26595, 26596, 26597, -1, 26598, 26599, 26600, -1, 26601, 26602, 26603, -1, 26604, 26605, 26606, -1, 26607, 26608, 26609, -1, 26610, 26611, 26612, -1, 26613, 26614, 26615, -1, 26616, 26617, 26618, -1, 26619, 26620, 26621, -1, 26622, 26623, 26624, -1, 26625, 26626, 26627, -1, 26628, 26629, 26630, -1, 26631, 26632, 26633, -1, 26634, 26635, 26636, -1, 26637, 26638, 26639, -1, 26640, 26641, 26642, -1, 26643, 26644, 26645, -1, 26646, 26647, 26648, -1, 26649, 26650, 26651, -1, 26652, 26653, 26654, -1, 26655, 26656, 26657, -1, 26658, 26659, 26660, -1, 26661, 26662, 26663, -1, 26664, 26665, 26666, -1, 26667, 26668, 26669, -1, 26670, 26671, 26672, -1, 26673, 26674, 26675, -1, 26676, 26677, 26678, -1, 26679, 26680, 26681, -1, 26682, 26683, 26684, -1, 26685, 26686, 26687, -1, 26688, 26689, 26690, -1, 26691, 26692, 26693, -1, 26694, 26695, 26696, -1, 26697, 26698, 26699, -1, 26700, 26701, 26702, -1, 26703, 26704, 26705, -1, 26706, 26707, 26708, -1, 26709, 26710, 26711, -1, 26712, 26713, 26714, -1, 26715, 26716, 26717, -1, 26718, 26719, 26720, -1, 26721, 26722, 26723, -1, 26724, 26725, 26726, -1, 26727, 26728, 26729, -1, 26730, 26731, 26732, -1, 26733, 26734, 26735, -1, 26736, 26737, 26738, -1, 26739, 26740, 26741, -1, 26742, 26743, 26744, -1, 26745, 26746, 26747, -1, 26748, 26749, 26750, -1, 26751, 26752, 26753, -1, 26754, 26755, 26756, -1, 26757, 26758, 26759, -1, 26760, 26761, 26762, -1, 26763, 26764, 26765, -1, 26766, 26767, 26768, -1, 26769, 26770, 26771, -1, 26772, 26773, 26774, -1, 26775, 26776, 26777, -1, 26778, 26779, 26780, -1, 26781, 26782, 26783, -1, 26784, 26785, 26786, -1, 26787, 26788, 26789, -1, 26790, 26791, 26792, -1, 26793, 26794, 26795, -1, 26796, 26797, 26798, -1, 26799, 26800, 26801, -1, 26802, 26803, 26804, -1, 26805, 26806, 26807, -1, 26808, 26809, 26810, -1, 26811, 26812, 26813, -1, 26814, 26815, 26816, -1, 26817, 26818, 26819, -1, 26820, 26821, 26822, -1, 26823, 26824, 26825, -1, 26826, 26827, 26828, -1, 26829, 26830, 26831, -1, 26832, 26833, 26834, -1, 26835, 26836, 26837, -1, 26838, 26839, 26840, -1, 26841, 26842, 26843, -1, 26844, 26845, 26846, -1, 26847, 26848, 26849, -1, 26850, 26851, 26852, -1, 26853, 26854, 26855, -1, 26856, 26857, 26858, -1, 26859, 26860, 26861, -1, 26862, 26863, 26864, -1, 26865, 26866, 26867, -1, 26868, 26869, 26870, -1, 26871, 26872, 26873, -1, 26874, 26875, 26876, -1, 26877, 26878, 26879, -1, 26880, 26881, 26882, -1, 26883, 26884, 26885, -1, 26886, 26887, 26888, -1, 26889, 26890, 26891, -1, 26892, 26893, 26894, -1, 26895, 26896, 26897, -1, 26898, 26899, 26900, -1, 26901, 26902, 26903, -1, 26904, 26905, 26906, -1, 26907, 26908, 26909, -1, 26910, 26911, 26912, -1, 26913, 26914, 26915, -1, 26916, 26917, 26918, -1, 26919, 26920, 26921, -1, 26922, 26923, 26924, -1, 26925, 26926, 26927, -1, 26928, 26929, 26930, -1, 26931, 26932, 26933, -1, 26934, 26935, 26936, -1, 26937, 26938, 26939, -1, 26940, 26941, 26942, -1, 26943, 26944, 26945, -1, 26946, 26947, 26948, -1, 26949, 26950, 26951, -1, 26952, 26953, 26954, -1, 26955, 26956, 26957, -1, 26958, 26959, 26960, -1, 26961, 26962, 26963, -1, 26964, 26965, 26966, -1, 26967, 26968, 26969, -1, 26970, 26971, 26972, -1, 26973, 26974, 26975, -1, 26976, 26977, 26978, -1, 26979, 26980, 26981, -1, 26982, 26983, 26984, -1, 26985, 26986, 26987, -1, 26988, 26989, 26990, -1, 26991, 26992, 26993, -1, 26994, 26995, 26996, -1, 26997, 26998, 26999, -1, 27000, 27001, 27002, -1, 27003, 27004, 27005, -1, 27006, 27007, 27008, -1, 27009, 27010, 27011, -1, 27012, 27013, 27014, -1, 27015, 27016, 27017, -1, 27018, 27019, 27020, -1, 27021, 27022, 27023, -1, 27024, 27025, 27026, -1, 27027, 27028, 27029, -1, 27030, 27031, 27032, -1, 27033, 27034, 27035, -1, 27036, 27037, 27038, -1, 27039, 27040, 27041, -1, 27042, 27043, 27044, -1, 27045, 27046, 27047, -1, 27048, 27049, 27050, -1, 27051, 27052, 27053, -1, 27054, 27055, 27056, -1, 27057, 27058, 27059, -1, 27060, 27061, 27062, -1, 27063, 27064, 27065, -1, 27066, 27067, 27068, -1, 27069, 27070, 27071, -1, 27072, 27073, 27074, -1, 27075, 27076, 27077, -1, 27078, 27079, 27080, -1, 27081, 27082, 27083, -1, 27084, 27085, 27086, -1, 27087, 27088, 27089, -1, 27090, 27091, 27092, -1, 27093, 27094, 27095, -1, 27096, 27097, 27098, -1, 27099, 27100, 27101, -1, 27102, 27103, 27104, -1, 27105, 27106, 27107, -1, 27108, 27109, 27110, -1, 27111, 27112, 27113, -1, 27114, 27115, 27116, -1, 27117, 27118, 27119, -1, 27120, 27121, 27122, -1, 27123, 27124, 27125, -1, 27126, 27127, 27128, -1, 27129, 27130, 27131, -1, 27132, 27133, 27134, -1, 27135, 27136, 27137, -1, 27138, 27139, 27140, -1, 27141, 27142, 27143, -1, 27144, 27145, 27146, -1, 27147, 27148, 27149, -1, 27150, 27151, 27152, -1, 27153, 27154, 27155, -1, 27156, 27157, 27158, -1, 27159, 27160, 27161, -1, 27162, 27163, 27164, -1, 27165, 27166, 27167, -1, 27168, 27169, 27170, -1, 27171, 27172, 27173, -1, 27174, 27175, 27176, -1, 27177, 27178, 27179, -1, 27180, 27181, 27182, -1, 27183, 27184, 27185, -1, 27186, 27187, 27188, -1, 27189, 27190, 27191, -1, 27192, 27193, 27194, -1, 27195, 27196, 27197, -1, 27198, 27199, 27200, -1, 27201, 27202, 27203, -1, 27204, 27205, 27206, -1, 27207, 27208, 27209, -1, 27210, 27211, 27212, -1, 27213, 27214, 27215, -1, 27216, 27217, 27218, -1, 27219, 27220, 27221, -1, 27222, 27223, 27224, -1, 27225, 27226, 27227, -1, 27228, 27229, 27230, -1, 27231, 27232, 27233, -1, 27234, 27235, 27236, -1, 27237, 27238, 27239, -1, 27240, 27241, 27242, -1, 27243, 27244, 27245, -1, 27246, 27247, 27248, -1, 27249, 27250, 27251, -1, 27252, 27253, 27254, -1, 27255, 27256, 27257, -1, 27258, 27259, 27260, -1, 27261, 27262, 27263, -1, 27264, 27265, 27266, -1, 27267, 27268, 27269, -1, 27270, 27271, 27272, -1, 27273, 27274, 27275, -1, 27276, 27277, 27278, -1, 27279, 27280, 27281, -1, 27282, 27283, 27284, -1, 27285, 27286, 27287, -1, 27288, 27289, 27290, -1, 27291, 27292, 27293, -1, 27294, 27295, 27296, -1, 27297, 27298, 27299, -1, 27300, 27301, 27302, -1, 27303, 27304, 27305, -1, 27306, 27307, 27308, -1, 27309, 27310, 27311, -1, 27312, 27313, 27314, -1, 27315, 27316, 27317, -1, 27318, 27319, 27320, -1, 27321, 27322, 27323, -1, 27324, 27325, 27326, -1, 27327, 27328, 27329, -1, 27330, 27331, 27332, -1, 27333, 27334, 27335, -1, 27336, 27337, 27338, -1, 27339, 27340, 27341, -1, 27342, 27343, 27344, -1, 27345, 27346, 27347, -1, 27348, 27349, 27350, -1, 27351, 27352, 27353, -1, 27354, 27355, 27356, -1, 27357, 27358, 27359, -1, 27360, 27361, 27362, -1, 27363, 27364, 27365, -1, 27366, 27367, 27368, -1, 27369, 27370, 27371, -1, 27372, 27373, 27374, -1, 27375, 27376, 27377, -1, 27378, 27379, 27380, -1, 27381, 27382, 27383, -1, 27384, 27385, 27386, -1, 27387, 27388, 27389, -1, 27390, 27391, 27392, -1, 27393, 27394, 27395, -1, 27396, 27397, 27398, -1, 27399, 27400, 27401, -1, 27402, 27403, 27404, -1, 27405, 27406, 27407, -1, 27408, 27409, 27410, -1, 27411, 27412, 27413, -1, 27414, 27415, 27416, -1, 27417, 27418, 27419, -1, 27420, 27421, 27422, -1, 27423, 27424, 27425, -1, 27426, 27427, 27428, -1, 27429, 27430, 27431, -1, 27432, 27433, 27434, -1, 27435, 27436, 27437, -1, 27438, 27439, 27440, -1, 27441, 27442, 27443, -1, 27444, 27445, 27446, -1, 27447, 27448, 27449, -1, 27450, 27451, 27452, -1, 27453, 27454, 27455, -1, 27456, 27457, 27458, -1, 27459, 27460, 27461, -1, 27462, 27463, 27464, -1, 27465, 27466, 27467, -1, 27468, 27469, 27470, -1, 27471, 27472, 27473, -1, 27474, 27475, 27476, -1, 27477, 27478, 27479, -1, 27480, 27481, 27482, -1, 27483, 27484, 27485, -1, 27486, 27487, 27488, -1, 27489, 27490, 27491, -1, 27492, 27493, 27494, -1, 27495, 27496, 27497, -1, 27498, 27499, 27500, -1, 27501, 27502, 27503, -1, 27504, 27505, 27506, -1, 27507, 27508, 27509, -1, 27510, 27511, 27512, -1, 27513, 27514, 27515, -1, 27516, 27517, 27518, -1, 27519, 27520, 27521, -1, 27522, 27523, 27524, -1, 27525, 27526, 27527, -1, 27528, 27529, 27530, -1, 27531, 27532, 27533, -1, 27534, 27535, 27536, -1, 27537, 27538, 27539, -1, 27540, 27541, 27542, -1, 27543, 27544, 27545, -1, 27546, 27547, 27548, -1, 27549, 27550, 27551, -1, 27552, 27553, 27554, -1, 27555, 27556, 27557, -1, 27558, 27559, 27560, -1, 27561, 27562, 27563, -1, 27564, 27565, 27566, -1, 27567, 27568, 27569, -1, 27570, 27571, 27572, -1, 27573, 27574, 27575, -1, 27576, 27577, 27578, -1, 27579, 27580, 27581, -1, 27582, 27583, 27584, -1, 27585, 27586, 27587, -1, 27588, 27589, 27590, -1, 27591, 27592, 27593, -1, 27594, 27595, 27596, -1, 27597, 27598, 27599, -1, 27600, 27601, 27602, -1, 27603, 27604, 27605, -1, 27606, 27607, 27608, -1, 27609, 27610, 27611, -1, 27612, 27613, 27614, -1, 27615, 27616, 27617, -1, 27618, 27619, 27620, -1, 27621, 27622, 27623, -1, 27624, 27625, 27626, -1, 27627, 27628, 27629, -1, 27630, 27631, 27632, -1, 27633, 27634, 27635, -1, 27636, 27637, 27638, -1, 27639, 27640, 27641, -1, 27642, 27643, 27644, -1, 27645, 27646, 27647, -1, 27648, 27649, 27650, -1, 27651, 27652, 27653, -1, 27654, 27655, 27656, -1, 27657, 27658, 27659, -1, 27660, 27661, 27662, -1, 27663, 27664, 27665, -1, 27666, 27667, 27668, -1, 27669, 27670, 27671, -1, 27672, 27673, 27674, -1, 27675, 27676, 27677, -1, 27678, 27679, 27680, -1, 27681, 27682, 27683, -1, 27684, 27685, 27686, -1, 27687, 27688, 27689, -1, 27690, 27691, 27692, -1, 27693, 27694, 27695, -1, 27696, 27697, 27698, -1, 27699, 27700, 27701, -1, 27702, 27703, 27704, -1, 27705, 27706, 27707, -1, 27708, 27709, 27710, -1, 27711, 27712, 27713, -1, 27714, 27715, 27716, -1, 27717, 27718, 27719, -1, 27720, 27721, 27722, -1, 27723, 27724, 27725, -1, 27726, 27727, 27728, -1, 27729, 27730, 27731, -1, 27732, 27733, 27734, -1, 27735, 27736, 27737, -1, 27738, 27739, 27740, -1, 27741, 27742, 27743, -1, 27744, 27745, 27746, -1, 27747, 27748, 27749, -1, 27750, 27751, 27752, -1, 27753, 27754, 27755, -1, 27756, 27757, 27758, -1, 27759, 27760, 27761, -1, 27762, 27763, 27764, -1, 27765, 27766, 27767, -1, 27768, 27769, 27770, -1, 27771, 27772, 27773, -1, 27774, 27775, 27776, -1, 27777, 27778, 27779, -1, 27780, 27781, 27782, -1, 27783, 27784, 27785, -1, 27786, 27787, 27788, -1, 27789, 27790, 27791, -1, 27792, 27793, 27794, -1, 27795, 27796, 27797, -1, 27798, 27799, 27800, -1, 27801, 27802, 27803, -1, 27804, 27805, 27806, -1, 27807, 27808, 27809, -1, 27810, 27811, 27812, -1, 27813, 27814, 27815, -1, 27816, 27817, 27818, -1, 27819, 27820, 27821, -1, 27822, 27823, 27824, -1, 27825, 27826, 27827, -1, 27828, 27829, 27830, -1, 27831, 27832, 27833, -1, 27834, 27835, 27836, -1, 27837, 27838, 27839, -1, 27840, 27841, 27842, -1, 27843, 27844, 27845, -1, 27846, 27847, 27848, -1, 27849, 27850, 27851, -1, 27852, 27853, 27854, -1, 27855, 27856, 27857, -1, 27858, 27859, 27860, -1, 27861, 27862, 27863, -1, 27864, 27865, 27866, -1, 27867, 27868, 27869, -1, 27870, 27871, 27872, -1, 27873, 27874, 27875, -1, 27876, 27877, 27878, -1, 27879, 27880, 27881, -1, 27882, 27883, 27884, -1, 27885, 27886, 27887, -1, 27888, 27889, 27890, -1, 27891, 27892, 27893, -1, 27894, 27895, 27896, -1, 27897, 27898, 27899, -1, 27900, 27901, 27902, -1, 27903, 27904, 27905, -1, 27906, 27907, 27908, -1, 27909, 27910, 27911, -1, 27912, 27913, 27914, -1, 27915, 27916, 27917, -1, 27918, 27919, 27920, -1, 27921, 27922, 27923, -1, 27924, 27925, 27926, -1, 27927, 27928, 27929, -1, 27930, 27931, 27932, -1, 27933, 27934, 27935, -1, 27936, 27937, 27938, -1, 27939, 27940, 27941, -1, 27942, 27943, 27944, -1, 27945, 27946, 27947, -1, 27948, 27949, 27950, -1, 27951, 27952, 27953, -1, 27954, 27955, 27956, -1, 27957, 27958, 27959, -1, 27960, 27961, 27962, -1, 27963, 27964, 27965, -1, 27966, 27967, 27968, -1, 27969, 27970, 27971, -1, 27972, 27973, 27974, -1, 27975, 27976, 27977, -1, 27978, 27979, 27980, -1, 27981, 27982, 27983, -1, 27984, 27985, 27986, -1, 27987, 27988, 27989, -1, 27990, 27991, 27992, -1, 27993, 27994, 27995, -1, 27996, 27997, 27998, -1, 27999, 28000, 28001, -1, 28002, 28003, 28004, -1, 28005, 28006, 28007, -1, 28008, 28009, 28010, -1, 28011, 28012, 28013, -1, 28014, 28015, 28016, -1, 28017, 28018, 28019, -1, 28020, 28021, 28022, -1, 28023, 28024, 28025, -1, 28026, 28027, 28028, -1, 28029, 28030, 28031, -1, 28032, 28033, 28034, -1, 28035, 28036, 28037, -1, 28038, 28039, 28040, -1, 28041, 28042, 28043, -1, 28044, 28045, 28046, -1, 28047, 28048, 28049, -1, 28050, 28051, 28052, -1, 28053, 28054, 28055, -1, 28056, 28057, 28058, -1, 28059, 28060, 28061, -1, 28062, 28063, 28064, -1, 28065, 28066, 28067, -1, 28068, 28069, 28070, -1, 28071, 28072, 28073, -1, 28074, 28075, 28076, -1, 28077, 28078, 28079, -1, 28080, 28081, 28082, -1, 28083, 28084, 28085, -1, 28086, 28087, 28088, -1, 28089, 28090, 28091, -1, 28092, 28093, 28094, -1, 28095, 28096, 28097, -1, 28098, 28099, 28100, -1, 28101, 28102, 28103, -1, 28104, 28105, 28106, -1, 28107, 28108, 28109, -1, 28110, 28111, 28112, -1, 28113, 28114, 28115, -1, 28116, 28117, 28118, -1, 28119, 28120, 28121, -1, 28122, 28123, 28124, -1, 28125, 28126, 28127, -1, 28128, 28129, 28130, -1, 28131, 28132, 28133, -1, 28134, 28135, 28136, -1, 28137, 28138, 28139, -1, 28140, 28141, 28142, -1, 28143, 28144, 28145, -1, 28146, 28147, 28148, -1, 28149, 28150, 28151, -1, 28152, 28153, 28154, -1, 28155, 28156, 28157, -1, 28158, 28159, 28160, -1, 28161, 28162, 28163, -1, 28164, 28165, 28166, -1, 28167, 28168, 28169, -1, 28170, 28171, 28172, -1, 28173, 28174, 28175, -1, 28176, 28177, 28178, -1, 28179, 28180, 28181, -1, 28182, 28183, 28184, -1, 28185, 28186, 28187, -1, 28188, 28189, 28190, -1, 28191, 28192, 28193, -1, 28194, 28195, 28196, -1, 28197, 28198, 28199, -1, 28200, 28201, 28202, -1, 28203, 28204, 28205, -1, 28206, 28207, 28208, -1, 28209, 28210, 28211, -1, 28212, 28213, 28214, -1, 28215, 28216, 28217, -1, 28218, 28219, 28220, -1, 28221, 28222, 28223, -1, 28224, 28225, 28226, -1, 28227, 28228, 28229, -1, 28230, 28231, 28232, -1, 28233, 28234, 28235, -1, 28236, 28237, 28238, -1, 28239, 28240, 28241, -1, 28242, 28243, 28244, -1, 28245, 28246, 28247, -1, 28248, 28249, 28250, -1, 28251, 28252, 28253, -1, 28254, 28255, 28256, -1, 28257, 28258, 28259, -1, 28260, 28261, 28262, -1, 28263, 28264, 28265, -1, 28266, 28267, 28268, -1, 28269, 28270, 28271, -1, 28272, 28273, 28274, -1, 28275, 28276, 28277, -1, 28278, 28279, 28280, -1, 28281, 28282, 28283, -1, 28284, 28285, 28286, -1, 28287, 28288, 28289, -1, 28290, 28291, 28292, -1, 28293, 28294, 28295, -1, 28296, 28297, 28298, -1, 28299, 28300, 28301, -1, 28302, 28303, 28304, -1, 28305, 28306, 28307, -1, 28308, 28309, 28310, -1, 28311, 28312, 28313, -1, 28314, 28315, 28316, -1, 28317, 28318, 28319, -1, 28320, 28321, 28322, -1, 28323, 28324, 28325, -1, 28326, 28327, 28328, -1, 28329, 28330, 28331, -1, 28332, 28333, 28334, -1, 28335, 28336, 28337, -1, 28338, 28339, 28340, -1, 28341, 28342, 28343, -1, 28344, 28345, 28346, -1, 28347, 28348, 28349, -1, 28350, 28351, 28352, -1, 28353, 28354, 28355, -1, 28356, 28357, 28358, -1, 28359, 28360, 28361, -1, 28362, 28363, 28364, -1, 28365, 28366, 28367, -1, 28368, 28369, 28370, -1, 28371, 28372, 28373, -1, 28374, 28375, 28376, -1, 28377, 28378, 28379, -1, 28380, 28381, 28382, -1, 28383, 28384, 28385, -1, 28386, 28387, 28388, -1, 28389, 28390, 28391, -1, 28392, 28393, 28394, -1, 28395, 28396, 28397, -1, 28398, 28399, 28400, -1, 28401, 28402, 28403, -1, 28404, 28405, 28406, -1, 28407, 28408, 28409, -1, 28410, 28411, 28412, -1, 28413, 28414, 28415, -1, 28416, 28417, 28418, -1, 28419, 28420, 28421, -1, 28422, 28423, 28424, -1, 28425, 28426, 28427, -1, 28428, 28429, 28430, -1, 28431, 28432, 28433, -1, 28434, 28435, 28436, -1, 28437, 28438, 28439, -1, 28440, 28441, 28442, -1, 28443, 28444, 28445, -1, 28446, 28447, 28448, -1, 28449, 28450, 28451, -1, 28452, 28453, 28454, -1, 28455, 28456, 28457, -1, 28458, 28459, 28460, -1, 28461, 28462, 28463, -1, 28464, 28465, 28466, -1, 28467, 28468, 28469, -1, 28470, 28471, 28472, -1, 28473, 28474, 28475, -1, 28476, 28477, 28478, -1, 28479, 28480, 28481, -1, 28482, 28483, 28484, -1, 28485, 28486, 28487, -1, 28488, 28489, 28490, -1, 28491, 28492, 28493, -1, 28494, 28495, 28496, -1, 28497, 28498, 28499, -1, 28500, 28501, 28502, -1, 28503, 28504, 28505, -1, 28506, 28507, 28508, -1, 28509, 28510, 28511, -1, 28512, 28513, 28514, -1, 28515, 28516, 28517, -1, 28518, 28519, 28520, -1, 28521, 28522, 28523, -1, 28524, 28525, 28526, -1, 28527, 28528, 28529, -1, 28530, 28531, 28532, -1, 28533, 28534, 28535, -1, 28536, 28537, 28538, -1, 28539, 28540, 28541, -1, 28542, 28543, 28544, -1, 28545, 28546, 28547, -1, 28548, 28549, 28550, -1, 28551, 28552, 28553, -1, 28554, 28555, 28556, -1, 28557, 28558, 28559, -1, 28560, 28561, 28562, -1, 28563, 28564, 28565, -1, 28566, 28567, 28568, -1, 28569, 28570, 28571, -1, 28572, 28573, 28574, -1, 28575, 28576, 28577, -1, 28578, 28579, 28580, -1, 28581, 28582, 28583, -1, 28584, 28585, 28586, -1, 28587, 28588, 28589, -1, 28590, 28591, 28592, -1, 28593, 28594, 28595, -1, 28596, 28597, 28598, -1, 28599, 28600, 28601, -1, 28602, 28603, 28604, -1, 28605, 28606, 28607, -1, 28608, 28609, 28610, -1, 28611, 28612, 28613, -1, 28614, 28615, 28616, -1, 28617, 28618, 28619, -1, 28620, 28621, 28622, -1, 28623, 28624, 28625, -1, 28626, 28627, 28628, -1, 28629, 28630, 28631, -1, 28632, 28633, 28634, -1, 28635, 28636, 28637, -1, 28638, 28639, 28640, -1, 28641, 28642, 28643, -1, 28644, 28645, 28646, -1, 28647, 28648, 28649, -1, 28650, 28651, 28652, -1, 28653, 28654, 28655, -1, 28656, 28657, 28658, -1, 28659, 28660, 28661, -1, 28662, 28663, 28664, -1, 28665, 28666, 28667, -1, 28668, 28669, 28670, -1, 28671, 28672, 28673, -1, 28674, 28675, 28676, -1, 28677, 28678, 28679, -1, 28680, 28681, 28682, -1, 28683, 28684, 28685, -1, 28686, 28687, 28688, -1, 28689, 28690, 28691, -1, 28692, 28693, 28694, -1, 28695, 28696, 28697, -1, 28698, 28699, 28700, -1, 28701, 28702, 28703, -1, 28704, 28705, 28706, -1, 28707, 28708, 28709, -1, 28710, 28711, 28712, -1, 28713, 28714, 28715, -1, 28716, 28717, 28718, -1, 28719, 28720, 28721, -1, 28722, 28723, 28724, -1, 28725, 28726, 28727, -1, 28728, 28729, 28730, -1, 28731, 28732, 28733, -1, 28734, 28735, 28736, -1, 28737, 28738, 28739, -1, 28740, 28741, 28742, -1, 28743, 28744, 28745, -1, 28746, 28747, 28748, -1, 28749, 28750, 28751, -1, 28752, 28753, 28754, -1, 28755, 28756, 28757, -1, 28758, 28759, 28760, -1, 28761, 28762, 28763, -1, 28764, 28765, 28766, -1, 28767, 28768, 28769, -1, 28770, 28771, 28772, -1, 28773, 28774, 28775, -1, 28776, 28777, 28778, -1, 28779, 28780, 28781, -1, 28782, 28783, 28784, -1, 28785, 28786, 28787, -1, 28788, 28789, 28790, -1, 28791, 28792, 28793, -1, 28794, 28795, 28796, -1, 28797, 28798, 28799, -1, 28800, 28801, 28802, -1, 28803, 28804, 28805, -1, 28806, 28807, 28808, -1, 28809, 28810, 28811, -1, 28812, 28813, 28814, -1, 28815, 28816, 28817, -1, 28818, 28819, 28820, -1, 28821, 28822, 28823, -1, 28824, 28825, 28826, -1, 28827, 28828, 28829, -1, 28830, 28831, 28832, -1, 28833, 28834, 28835, -1, 28836, 28837, 28838, -1, 28839, 28840, 28841, -1, 28842, 28843, 28844, -1, 28845, 28846, 28847, -1, 28848, 28849, 28850, -1, 28851, 28852, 28853, -1, 28854, 28855, 28856, -1, 28857, 28858, 28859, -1, 28860, 28861, 28862, -1, 28863, 28864, 28865, -1, 28866, 28867, 28868, -1, 28869, 28870, 28871, -1, 28872, 28873, 28874, -1, 28875, 28876, 28877, -1, 28878, 28879, 28880, -1, 28881, 28882, 28883, -1, 28884, 28885, 28886, -1, 28887, 28888, 28889, -1, 28890, 28891, 28892, -1, 28893, 28894, 28895, -1, 28896, 28897, 28898, -1, 28899, 28900, 28901, -1, 28902, 28903, 28904, -1, 28905, 28906, 28907, -1, 28908, 28909, 28910, -1, 28911, 28912, 28913, -1, 28914, 28915, 28916, -1, 28917, 28918, 28919, -1, 28920, 28921, 28922, -1, 28923, 28924, 28925, -1, 28926, 28927, 28928, -1, 28929, 28930, 28931, -1, 28932, 28933, 28934, -1, 28935, 28936, 28937, -1, 28938, 28939, 28940, -1, 28941, 28942, 28943, -1, 28944, 28945, 28946, -1, 28947, 28948, 28949, -1, 28950, 28951, 28952, -1, 28953, 28954, 28955, -1, 28956, 28957, 28958, -1, 28959, 28960, 28961, -1, 28962, 28963, 28964, -1, 28965, 28966, 28967, -1, 28968, 28969, 28970, -1, 28971, 28972, 28973, -1, 28974, 28975, 28976, -1, 28977, 28978, 28979, -1, 28980, 28981, 28982, -1, 28983, 28984, 28985, -1, 28986, 28987, 28988, -1, 28989, 28990, 28991, -1, 28992, 28993, 28994, -1, 28995, 28996, 28997, -1, 28998, 28999, 29000, -1, 29001, 29002, 29003, -1, 29004, 29005, 29006, -1, 29007, 29008, 29009, -1, 29010, 29011, 29012, -1, 29013, 29014, 29015, -1, 29016, 29017, 29018, -1, 29019, 29020, 29021, -1, 29022, 29023, 29024, -1, 29025, 29026, 29027, -1, 29028, 29029, 29030, -1, 29031, 29032, 29033, -1, 29034, 29035, 29036, -1, 29037, 29038, 29039, -1, 29040, 29041, 29042, -1, 29043, 29044, 29045, -1, 29046, 29047, 29048, -1, 29049, 29050, 29051, -1, 29052, 29053, 29054, -1, 29055, 29056, 29057, -1, 29058, 29059, 29060, -1, 29061, 29062, 29063, -1, 29064, 29065, 29066, -1, 29067, 29068, 29069, -1, 29070, 29071, 29072, -1, 29073, 29074, 29075, -1, 29076, 29077, 29078, -1, 29079, 29080, 29081, -1, 29082, 29083, 29084, -1, 29085, 29086, 29087, -1, 29088, 29089, 29090, -1, 29091, 29092, 29093, -1, 29094, 29095, 29096, -1, 29097, 29098, 29099, -1, 29100, 29101, 29102, -1, 29103, 29104, 29105, -1, 29106, 29107, 29108, -1, 29109, 29110, 29111, -1, 29112, 29113, 29114, -1, 29115, 29116, 29117, -1, 29118, 29119, 29120, -1, 29121, 29122, 29123, -1, 29124, 29125, 29126, -1, 29127, 29128, 29129, -1, 29130, 29131, 29132, -1, 29133, 29134, 29135, -1, 29136, 29137, 29138, -1, 29139, 29140, 29141, -1, 29142, 29143, 29144, -1, 29145, 29146, 29147, -1, 29148, 29149, 29150, -1, 29151, 29152, 29153, -1, 29154, 29155, 29156, -1, 29157, 29158, 29159, -1, 29160, 29161, 29162, -1, 29163, 29164, 29165, -1, 29166, 29167, 29168, -1, 29169, 29170, 29171, -1, 29172, 29173, 29174, -1, 29175, 29176, 29177, -1, 29178, 29179, 29180, -1, 29181, 29182, 29183, -1, 29184, 29185, 29186, -1, 29187, 29188, 29189, -1, 29190, 29191, 29192, -1, 29193, 29194, 29195, -1, 29196, 29197, 29198, -1, 29199, 29200, 29201, -1, 29202, 29203, 29204, -1, 29205, 29206, 29207, -1, 29208, 29209, 29210, -1, 29211, 29212, 29213, -1, 29214, 29215, 29216, -1, 29217, 29218, 29219, -1, 29220, 29221, 29222, -1, 29223, 29224, 29225, -1, 29226, 29227, 29228, -1, 29229, 29230, 29231, -1, 29232, 29233, 29234, -1, 29235, 29236, 29237, -1, 29238, 29239, 29240, -1, 29241, 29242, 29243, -1, 29244, 29245, 29246, -1, 29247, 29248, 29249, -1, 29250, 29251, 29252, -1, 29253, 29254, 29255, -1, 29256, 29257, 29258, -1, 29259, 29260, 29261, -1, 29262, 29263, 29264, -1, 29265, 29266, 29267, -1, 29268, 29269, 29270, -1, 29271, 29272, 29273, -1, 29274, 29275, 29276, -1, 29277, 29278, 29279, -1, 29280, 29281, 29282, -1, 29283, 29284, 29285, -1, 29286, 29287, 29288, -1, 29289, 29290, 29291, -1, 29292, 29293, 29294, -1, 29295, 29296, 29297, -1, 29298, 29299, 29300, -1, 29301, 29302, 29303, -1, 29304, 29305, 29306, -1, 29307, 29308, 29309, -1, 29310, 29311, 29312, -1, 29313, 29314, 29315, -1, 29316, 29317, 29318, -1, 29319, 29320, 29321, -1, 29322, 29323, 29324, -1, 29325, 29326, 29327, -1, 29328, 29329, 29330, -1, 29331, 29332, 29333, -1, 29334, 29335, 29336, -1, 29337, 29338, 29339, -1, 29340, 29341, 29342, -1, 29343, 29344, 29345, -1, 29346, 29347, 29348, -1, 29349, 29350, 29351, -1, 29352, 29353, 29354, -1, 29355, 29356, 29357, -1, 29358, 29359, 29360, -1, 29361, 29362, 29363, -1, 29364, 29365, 29366, -1, 29367, 29368, 29369, -1, 29370, 29371, 29372, -1, 29373, 29374, 29375, -1, 29376, 29377, 29378, -1, 29379, 29380, 29381, -1, 29382, 29383, 29384, -1, 29385, 29386, 29387, -1, 29388, 29389, 29390, -1, 29391, 29392, 29393, -1, 29394, 29395, 29396, -1, 29397, 29398, 29399, -1, 29400, 29401, 29402, -1, 29403, 29404, 29405, -1, 29406, 29407, 29408, -1, 29409, 29410, 29411, -1, 29412, 29413, 29414, -1, 29415, 29416, 29417, -1, 29418, 29419, 29420, -1, 29421, 29422, 29423, -1, 29424, 29425, 29426, -1, 29427, 29428, 29429, -1, 29430, 29431, 29432, -1, 29433, 29434, 29435, -1, 29436, 29437, 29438, -1, 29439, 29440, 29441, -1, 29442, 29443, 29444, -1, 29445, 29446, 29447, -1, 29448, 29449, 29450, -1, 29451, 29452, 29453, -1, 29454, 29455, 29456, -1, 29457, 29458, 29459, -1, 29460, 29461, 29462, -1, 29463, 29464, 29465, -1, 29466, 29467, 29468, -1, 29469, 29470, 29471, -1, 29472, 29473, 29474, -1, 29475, 29476, 29477, -1, 29478, 29479, 29480, -1, 29481, 29482, 29483, -1, 29484, 29485, 29486, -1, 29487, 29488, 29489, -1, 29490, 29491, 29492, -1, 29493, 29494, 29495, -1, 29496, 29497, 29498, -1, 29499, 29500, 29501, -1, 29502, 29503, 29504, -1, 29505, 29506, 29507, -1, 29508, 29509, 29510, -1, 29511, 29512, 29513, -1, 29514, 29515, 29516, -1, 29517, 29518, 29519, -1, 29520, 29521, 29522, -1, 29523, 29524, 29525, -1, 29526, 29527, 29528, -1, 29529, 29530, 29531, -1, 29532, 29533, 29534, -1, 29535, 29536, 29537, -1, 29538, 29539, 29540, -1, 29541, 29542, 29543, -1, 29544, 29545, 29546, -1, 29547, 29548, 29549, -1, 29550, 29551, 29552, -1, 29553, 29554, 29555, -1, 29556, 29557, 29558, -1, 29559, 29560, 29561, -1, 29562, 29563, 29564, -1, 29565, 29566, 29567, -1, 29568, 29569, 29570, -1, 29571, 29572, 29573, -1, 29574, 29575, 29576, -1, 29577, 29578, 29579, -1, 29580, 29581, 29582, -1, 29583, 29584, 29585, -1, 29586, 29587, 29588, -1, 29589, 29590, 29591, -1, 29592, 29593, 29594, -1, 29595, 29596, 29597, -1, 29598, 29599, 29600, -1, 29601, 29602, 29603, -1, 29604, 29605, 29606, -1, 29607, 29608, 29609, -1, 29610, 29611, 29612, -1, 29613, 29614, 29615, -1, 29616, 29617, 29618, -1, 29619, 29620, 29621, -1, 29622, 29623, 29624, -1, 29625, 29626, 29627, -1, 29628, 29629, 29630, -1, 29631, 29632, 29633, -1, 29634, 29635, 29636, -1, 29637, 29638, 29639, -1, 29640, 29641, 29642, -1, 29643, 29644, 29645, -1, 29646, 29647, 29648, -1, 29649, 29650, 29651, -1, 29652, 29653, 29654, -1, 29655, 29656, 29657, -1, 29658, 29659, 29660, -1, 29661, 29662, 29663, -1, 29664, 29665, 29666, -1, 29667, 29668, 29669, -1, 29670, 29671, 29672, -1, 29673, 29674, 29675, -1, 29676, 29677, 29678, -1, 29679, 29680, 29681, -1, 29682, 29683, 29684, -1, 29685, 29686, 29687, -1, 29688, 29689, 29690, -1, 29691, 29692, 29693, -1, 29694, 29695, 29696, -1, 29697, 29698, 29699, -1, 29700, 29701, 29702, -1, 29703, 29704, 29705, -1, 29706, 29707, 29708, -1, 29709, 29710, 29711, -1, 29712, 29713, 29714, -1, 29715, 29716, 29717, -1, 29718, 29719, 29720, -1, 29721, 29722, 29723, -1, 29724, 29725, 29726, -1, 29727, 29728, 29729, -1, 29730, 29731, 29732, -1, 29733, 29734, 29735, -1, 29736, 29737, 29738, -1, 29739, 29740, 29741, -1, 29742, 29743, 29744, -1, 29745, 29746, 29747, -1, 29748, 29749, 29750, -1, 29751, 29752, 29753, -1, 29754, 29755, 29756, -1, 29757, 29758, 29759, -1, 29760, 29761, 29762, -1, 29763, 29764, 29765, -1, 29766, 29767, 29768, -1, 29769, 29770, 29771, -1, 29772, 29773, 29774, -1, 29775, 29776, 29777, -1, 29778, 29779, 29780, -1, 29781, 29782, 29783, -1, 29784, 29785, 29786, -1, 29787, 29788, 29789, -1, 29790, 29791, 29792, -1, 29793, 29794, 29795, -1, 29796, 29797, 29798, -1, 29799, 29800, 29801, -1, 29802, 29803, 29804, -1, 29805, 29806, 29807, -1, 29808, 29809, 29810, -1, 29811, 29812, 29813, -1, 29814, 29815, 29816, -1, 29817, 29818, 29819, -1, 29820, 29821, 29822, -1, 29823, 29824, 29825, -1, 29826, 29827, 29828, -1, 29829, 29830, 29831, -1, 29832, 29833, 29834, -1, 29835, 29836, 29837, -1, 29838, 29839, 29840, -1, 29841, 29842, 29843, -1, 29844, 29845, 29846, -1, 29847, 29848, 29849, -1, 29850, 29851, 29852, -1, 29853, 29854, 29855, -1, 29856, 29857, 29858, -1, 29859, 29860, 29861, -1, 29862, 29863, 29864, -1, 29865, 29866, 29867, -1, 29868, 29869, 29870, -1, 29871, 29872, 29873, -1, 29874, 29875, 29876, -1, 29877, 29878, 29879, -1, 29880, 29881, 29882, -1, 29883, 29884, 29885, -1, 29886, 29887, 29888, -1, 29889, 29890, 29891, -1, 29892, 29893, 29894, -1, 29895, 29896, 29897, -1, 29898, 29899, 29900, -1, 29901, 29902, 29903, -1, 29904, 29905, 29906, -1, 29907, 29908, 29909, -1, 29910, 29911, 29912, -1, 29913, 29914, 29915, -1, 29916, 29917, 29918, -1, 29919, 29920, 29921, -1, 29922, 29923, 29924, -1, 29925, 29926, 29927, -1, 29928, 29929, 29930, -1, 29931, 29932, 29933, -1, 29934, 29935, 29936, -1, 29937, 29938, 29939, -1, 29940, 29941, 29942, -1, 29943, 29944, 29945, -1, 29946, 29947, 29948, -1, 29949, 29950, 29951, -1, 29952, 29953, 29954, -1, 29955, 29956, 29957, -1, 29958, 29959, 29960, -1, 29961, 29962, 29963, -1, 29964, 29965, 29966, -1, 29967, 29968, 29969, -1, 29970, 29971, 29972, -1, 29973, 29974, 29975, -1, 29976, 29977, 29978, -1, 29979, 29980, 29981, -1, 29982, 29983, 29984, -1, 29985, 29986, 29987, -1, 29988, 29989, 29990, -1, 29991, 29992, 29993, -1, 29994, 29995, 29996, -1, 29997, 29998, 29999, -1, 30000, 30001, 30002, -1, 30003, 30004, 30005, -1, 30006, 30007, 30008, -1, 30009, 30010, 30011, -1, 30012, 30013, 30014, -1, 30015, 30016, 30017, -1, 30018, 30019, 30020, -1, 30021, 30022, 30023, -1, 30024, 30025, 30026, -1, 30027, 30028, 30029, -1, 30030, 30031, 30032, -1, 30033, 30034, 30035, -1, 30036, 30037, 30038, -1, 30039, 30040, 30041, -1, 30042, 30043, 30044, -1, 30045, 30046, 30047, -1, 30048, 30049, 30050, -1, 30051, 30052, 30053, -1, 30054, 30055, 30056, -1, 30057, 30058, 30059, -1, 30060, 30061, 30062, -1, 30063, 30064, 30065, -1, 30066, 30067, 30068, -1, 30069, 30070, 30071, -1, 30072, 30073, 30074, -1, 30075, 30076, 30077, -1, 30078, 30079, 30080, -1, 30081, 30082, 30083, -1, 30084, 30085, 30086, -1, 30087, 30088, 30089, -1, 30090, 30091, 30092, -1, 30093, 30094, 30095, -1, 30096, 30097, 30098, -1, 30099, 30100, 30101, -1, 30102, 30103, 30104, -1, 30105, 30106, 30107, -1, 30108, 30109, 30110, -1, 30111, 30112, 30113, -1, 30114, 30115, 30116, -1, 30117, 30118, 30119, -1, 30120, 30121, 30122, -1, 30123, 30124, 30125, -1, 30126, 30127, 30128, -1, 30129, 30130, 30131, -1, 30132, 30133, 30134, -1, 30135, 30136, 30137, -1, 30138, 30139, 30140, -1, 30141, 30142, 30143, -1, 30144, 30145, 30146, -1, 30147, 30148, 30149, -1, 30150, 30151, 30152, -1, 30153, 30154, 30155, -1, 30156, 30157, 30158, -1, 30159, 30160, 30161, -1, 30162, 30163, 30164, -1, 30165, 30166, 30167, -1, 30168, 30169, 30170, -1, 30171, 30172, 30173, -1, 30174, 30175, 30176, -1, 30177, 30178, 30179, -1, 30180, 30181, 30182, -1, 30183, 30184, 30185, -1, 30186, 30187, 30188, -1, 30189, 30190, 30191, -1, 30192, 30193, 30194, -1, 30195, 30196, 30197, -1, 30198, 30199, 30200, -1, 30201, 30202, 30203, -1, 30204, 30205, 30206, -1, 30207, 30208, 30209, -1, 30210, 30211, 30212, -1, 30213, 30214, 30215, -1, 30216, 30217, 30218, -1, 30219, 30220, 30221, -1, 30222, 30223, 30224, -1, 30225, 30226, 30227, -1, 30228, 30229, 30230, -1, 30231, 30232, 30233, -1, 30234, 30235, 30236, -1, 30237, 30238, 30239, -1, 30240, 30241, 30242, -1, 30243, 30244, 30245, -1, 30246, 30247, 30248, -1, 30249, 30250, 30251, -1, 30252, 30253, 30254, -1, 30255, 30256, 30257, -1, 30258, 30259, 30260, -1, 30261, 30262, 30263, -1, 30264, 30265, 30266, -1, 30267, 30268, 30269, -1, 30270, 30271, 30272, -1, 30273, 30274, 30275, -1, 30276, 30277, 30278, -1, 30279, 30280, 30281, -1, 30282, 30283, 30284, -1, 30285, 30286, 30287, -1, 30288, 30289, 30290, -1, 30291, 30292, 30293, -1, 30294, 30295, 30296, -1, 30297, 30298, 30299, -1, 30300, 30301, 30302, -1, 30303, 30304, 30305, -1, 30306, 30307, 30308, -1, 30309, 30310, 30311, -1, 30312, 30313, 30314, -1, 30315, 30316, 30317, -1, 30318, 30319, 30320, -1, 30321, 30322, 30323, -1, 30324, 30325, 30326, -1, 30327, 30328, 30329, -1, 30330, 30331, 30332, -1, 30333, 30334, 30335, -1, 30336, 30337, 30338, -1, 30339, 30340, 30341, -1, 30342, 30343, 30344, -1, 30345, 30346, 30347, -1, 30348, 30349, 30350, -1, 30351, 30352, 30353, -1, 30354, 30355, 30356, -1, 30357, 30358, 30359, -1, 30360, 30361, 30362, -1, 30363, 30364, 30365, -1, 30366, 30367, 30368, -1, 30369, 30370, 30371, -1, 30372, 30373, 30374, -1, 30375, 30376, 30377, -1, 30378, 30379, 30380, -1, 30381, 30382, 30383, -1, 30384, 30385, 30386, -1, 30387, 30388, 30389, -1, 30390, 30391, 30392, -1, 30393, 30394, 30395, -1, 30396, 30397, 30398, -1, 30399, 30400, 30401, -1, 30402, 30403, 30404, -1, 30405, 30406, 30407, -1, 30408, 30409, 30410, -1, 30411, 30412, 30413, -1, 30414, 30415, 30416, -1, 30417, 30418, 30419, -1, 30420, 30421, 30422, -1, 30423, 30424, 30425, -1, 30426, 30427, 30428, -1, 30429, 30430, 30431, -1, 30432, 30433, 30434, -1, 30435, 30436, 30437, -1, 30438, 30439, 30440, -1, 30441, 30442, 30443, -1, 30444, 30445, 30446, -1, 30447, 30448, 30449, -1, 30450, 30451, 30452, -1, 30453, 30454, 30455, -1, 30456, 30457, 30458, -1, 30459, 30460, 30461, -1, 30462, 30463, 30464, -1, 30465, 30466, 30467, -1, 30468, 30469, 30470, -1, 30471, 30472, 30473, -1, 30474, 30475, 30476, -1, 30477, 30478, 30479, -1, 30480, 30481, 30482, -1, 30483, 30484, 30485, -1, 30486, 30487, 30488, -1, 30489, 30490, 30491, -1, 30492, 30493, 30494, -1, 30495, 30496, 30497, -1, 30498, 30499, 30500, -1, 30501, 30502, 30503, -1, 30504, 30505, 30506, -1, 30507, 30508, 30509, -1, 30510, 30511, 30512, -1, 30513, 30514, 30515, -1, 30516, 30517, 30518, -1, 30519, 30520, 30521, -1, 30522, 30523, 30524, -1, 30525, 30526, 30527, -1, 30528, 30529, 30530, -1, 30531, 30532, 30533, -1, 30534, 30535, 30536, -1, 30537, 30538, 30539, -1, 30540, 30541, 30542, -1, 30543, 30544, 30545, -1, 30546, 30547, 30548, -1, 30549, 30550, 30551, -1, 30552, 30553, 30554, -1, 30555, 30556, 30557, -1, 30558, 30559, 30560, -1, 30561, 30562, 30563, -1, 30564, 30565, 30566, -1, 30567, 30568, 30569, -1, 30570, 30571, 30572, -1, 30573, 30574, 30575, -1, 30576, 30577, 30578, -1, 30579, 30580, 30581, -1, 30582, 30583, 30584, -1, 30585, 30586, 30587, -1, 30588, 30589, 30590, -1, 30591, 30592, 30593, -1, 30594, 30595, 30596, -1, 30597, 30598, 30599, -1, 30600, 30601, 30602, -1, 30603, 30604, 30605, -1, 30606, 30607, 30608, -1, 30609, 30610, 30611, -1, 30612, 30613, 30614, -1, 30615, 30616, 30617, -1, 30618, 30619, 30620, -1, 30621, 30622, 30623, -1, 30624, 30625, 30626, -1, 30627, 30628, 30629, -1, 30630, 30631, 30632, -1, 30633, 30634, 30635, -1, 30636, 30637, 30638, -1, 30639, 30640, 30641, -1, 30642, 30643, 30644, -1, 30645, 30646, 30647, -1, 30648, 30649, 30650, -1, 30651, 30652, 30653, -1, 30654, 30655, 30656, -1, 30657, 30658, 30659, -1, 30660, 30661, 30662, -1, 30663, 30664, 30665, -1, 30666, 30667, 30668, -1, 30669, 30670, 30671, -1, 30672, 30673, 30674, -1, 30675, 30676, 30677, -1, 30678, 30679, 30680, -1, 30681, 30682, 30683, -1, 30684, 30685, 30686, -1, 30687, 30688, 30689, -1, 30690, 30691, 30692, -1, 30693, 30694, 30695, -1, 30696, 30697, 30698, -1, 30699, 30700, 30701, -1, 30702, 30703, 30704, -1, 30705, 30706, 30707, -1, 30708, 30709, 30710, -1, 30711, 30712, 30713, -1, 30714, 30715, 30716, -1, 30717, 30718, 30719, -1, 30720, 30721, 30722, -1, 30723, 30724, 30725, -1, 30726, 30727, 30728, -1, 30729, 30730, 30731, -1, 30732, 30733, 30734, -1, 30735, 30736, 30737, -1, 30738, 30739, 30740, -1, 30741, 30742, 30743, -1, 30744, 30745, 30746, -1, 30747, 30748, 30749, -1, 30750, 30751, 30752, -1, 30753, 30754, 30755, -1, 30756, 30757, 30758, -1, 30759, 30760, 30761, -1, 30762, 30763, 30764, -1, 30765, 30766, 30767, -1, 30768, 30769, 30770, -1, 30771, 30772, 30773, -1, 30774, 30775, 30776, -1, 30777, 30778, 30779, -1, 30780, 30781, 30782, -1, 30783, 30784, 30785, -1, 30786, 30787, 30788, -1, 30789, 30790, 30791, -1, 30792, 30793, 30794, -1, 30795, 30796, 30797, -1, 30798, 30799, 30800, -1, 30801, 30802, 30803, -1, 30804, 30805, 30806, -1, 30807, 30808, 30809, -1, 30810, 30811, 30812, -1, 30813, 30814, 30815, -1, 30816, 30817, 30818, -1, 30819, 30820, 30821, -1, 30822, 30823, 30824, -1, 30825, 30826, 30827, -1, 30828, 30829, 30830, -1, 30831, 30832, 30833, -1, 30834, 30835, 30836, -1, 30837, 30838, 30839, -1, 30840, 30841, 30842, -1, 30843, 30844, 30845, -1, 30846, 30847, 30848, -1, 30849, 30850, 30851, -1, 30852, 30853, 30854, -1, 30855, 30856, 30857, -1, 30858, 30859, 30860, -1, 30861, 30862, 30863, -1, 30864, 30865, 30866, -1, 30867, 30868, 30869, -1, 30870, 30871, 30872, -1, 30873, 30874, 30875, -1, 30876, 30877, 30878, -1, 30879, 30880, 30881, -1, 30882, 30883, 30884, -1, 30885, 30886, 30887, -1, 30888, 30889, 30890, -1, 30891, 30892, 30893, -1, 30894, 30895, 30896, -1, 30897, 30898, 30899, -1, 30900, 30901, 30902, -1, 30903, 30904, 30905, -1, 30906, 30907, 30908, -1, 30909, 30910, 30911, -1, 30912, 30913, 30914, -1, 30915, 30916, 30917, -1, 30918, 30919, 30920, -1, 30921, 30922, 30923, -1, 30924, 30925, 30926, -1, 30927, 30928, 30929, -1, 30930, 30931, 30932, -1, 30933, 30934, 30935, -1, 30936, 30937, 30938, -1, 30939, 30940, 30941, -1, 30942, 30943, 30944, -1, 30945, 30946, 30947, -1, 30948, 30949, 30950, -1, 30951, 30952, 30953, -1, 30954, 30955, 30956, -1, 30957, 30958, 30959, -1, 30960, 30961, 30962, -1, 30963, 30964, 30965, -1, 30966, 30967, 30968, -1, 30969, 30970, 30971, -1, 30972, 30973, 30974, -1, 30975, 30976, 30977, -1, 30978, 30979, 30980, -1, 30981, 30982, 30983, -1, 30984, 30985, 30986, -1, 30987, 30988, 30989, -1, 30990, 30991, 30992, -1, 30993, 30994, 30995, -1, 30996, 30997, 30998, -1, 30999, 31000, 31001, -1, 31002, 31003, 31004, -1, 31005, 31006, 31007, -1, 31008, 31009, 31010, -1, 31011, 31012, 31013, -1, 31014, 31015, 31016, -1, 31017, 31018, 31019, -1, 31020, 31021, 31022, -1, 31023, 31024, 31025, -1, 31026, 31027, 31028, -1, 31029, 31030, 31031, -1, 31032, 31033, 31034, -1, 31035, 31036, 31037, -1, 31038, 31039, 31040, -1, 31041, 31042, 31043, -1, 31044, 31045, 31046, -1, 31047, 31048, 31049, -1, 31050, 31051, 31052, -1, 31053, 31054, 31055, -1, 31056, 31057, 31058, -1, 31059, 31060, 31061, -1, 31062, 31063, 31064, -1, 31065, 31066, 31067, -1, 31068, 31069, 31070, -1, 31071, 31072, 31073, -1, 31074, 31075, 31076, -1, 31077, 31078, 31079, -1, 31080, 31081, 31082, -1, 31083, 31084, 31085, -1, 31086, 31087, 31088, -1, 31089, 31090, 31091, -1, 31092, 31093, 31094, -1, 31095, 31096, 31097, -1, 31098, 31099, 31100, -1, 31101, 31102, 31103, -1, 31104, 31105, 31106, -1, 31107, 31108, 31109, -1, 31110, 31111, 31112, -1, 31113, 31114, 31115, -1, 31116, 31117, 31118, -1, 31119, 31120, 31121, -1, 31122, 31123, 31124, -1, 31125, 31126, 31127, -1, 31128, 31129, 31130, -1, 31131, 31132, 31133, -1, 31134, 31135, 31136, -1, 31137, 31138, 31139, -1, 31140, 31141, 31142, -1, 31143, 31144, 31145, -1, 31146, 31147, 31148, -1, 31149, 31150, 31151, -1, 31152, 31153, 31154, -1, 31155, 31156, 31157, -1, 31158, 31159, 31160, -1, 31161, 31162, 31163, -1, 31164, 31165, 31166, -1, 31167, 31168, 31169, -1, 31170, 31171, 31172, -1, 31173, 31174, 31175, -1, 31176, 31177, 31178, -1, 31179, 31180, 31181, -1, 31182, 31183, 31184, -1, 31185, 31186, 31187, -1, 31188, 31189, 31190, -1, 31191, 31192, 31193, -1, 31194, 31195, 31196, -1, 31197, 31198, 31199, -1, 31200, 31201, 31202, -1, 31203, 31204, 31205, -1, 31206, 31207, 31208, -1, 31209, 31210, 31211, -1, 31212, 31213, 31214, -1, 31215, 31216, 31217, -1, 31218, 31219, 31220, -1, 31221, 31222, 31223, -1, 31224, 31225, 31226, -1, 31227, 31228, 31229, -1, 31230, 31231, 31232, -1, 31233, 31234, 31235, -1, 31236, 31237, 31238, -1, 31239, 31240, 31241, -1, 31242, 31243, 31244, -1, 31245, 31246, 31247, -1, 31248, 31249, 31250, -1, 31251, 31252, 31253, -1, 31254, 31255, 31256, -1, 31257, 31258, 31259, -1, 31260, 31261, 31262, -1, 31263, 31264, 31265, -1, 31266, 31267, 31268, -1, 31269, 31270, 31271, -1, 31272, 31273, 31274, -1, 31275, 31276, 31277, -1, 31278, 31279, 31280, -1, 31281, 31282, 31283, -1, 31284, 31285, 31286, -1, 31287, 31288, 31289, -1, 31290, 31291, 31292, -1, 31293, 31294, 31295, -1, 31296, 31297, 31298, -1, 31299, 31300, 31301, -1, 31302, 31303, 31304, -1, 31305, 31306, 31307, -1, 31308, 31309, 31310, -1, 31311, 31312, 31313, -1, 31314, 31315, 31316, -1, 31317, 31318, 31319, -1, 31320, 31321, 31322, -1, 31323, 31324, 31325, -1, 31326, 31327, 31328, -1, 31329, 31330, 31331, -1, 31332, 31333, 31334, -1, 31335, 31336, 31337, -1, 31338, 31339, 31340, -1, 31341, 31342, 31343, -1, 31344, 31345, 31346, -1, 31347, 31348, 31349, -1, 31350, 31351, 31352, -1, 31353, 31354, 31355, -1, 31356, 31357, 31358, -1, 31359, 31360, 31361, -1, 31362, 31363, 31364, -1, 31365, 31366, 31367, -1, 31368, 31369, 31370, -1, 31371, 31372, 31373, -1, 31374, 31375, 31376, -1, 31377, 31378, 31379, -1, 31380, 31381, 31382, -1, 31383, 31384, 31385, -1, 31386, 31387, 31388, -1, 31389, 31390, 31391, -1, 31392, 31393, 31394, -1, 31395, 31396, 31397, -1, 31398, 31399, 31400, -1, 31401, 31402, 31403, -1, 31404, 31405, 31406, -1, 31407, 31408, 31409, -1, 31410, 31411, 31412, -1, 31413, 31414, 31415, -1, 31416, 31417, 31418, -1, 31419, 31420, 31421, -1, 31422, 31423, 31424, -1, 31425, 31426, 31427, -1, 31428, 31429, 31430, -1, 31431, 31432, 31433, -1, 31434, 31435, 31436, -1, 31437, 31438, 31439, -1, 31440, 31441, 31442, -1, 31443, 31444, 31445, -1, 31446, 31447, 31448, -1, 31449, 31450, 31451, -1, 31452, 31453, 31454, -1, 31455, 31456, 31457, -1, 31458, 31459, 31460, -1, 31461, 31462, 31463, -1, 31464, 31465, 31466, -1, 31467, 31468, 31469, -1, 31470, 31471, 31472, -1, 31473, 31474, 31475, -1, 31476, 31477, 31478, -1, 31479, 31480, 31481, -1, 31482, 31483, 31484, -1, 31485, 31486, 31487, -1, 31488, 31489, 31490, -1, 31491, 31492, 31493, -1, 31494, 31495, 31496, -1, 31497, 31498, 31499, -1, 31500, 31501, 31502, -1, 31503, 31504, 31505, -1, 31506, 31507, 31508, -1, 31509, 31510, 31511, -1, 31512, 31513, 31514, -1, 31515, 31516, 31517, -1, 31518, 31519, 31520, -1, 31521, 31522, 31523, -1, 31524, 31525, 31526, -1, 31527, 31528, 31529, -1, 31530, 31531, 31532, -1, 31533, 31534, 31535, -1, 31536, 31537, 31538, -1, 31539, 31540, 31541, -1, 31542, 31543, 31544, -1, 31545, 31546, 31547, -1, 31548, 31549, 31550, -1, 31551, 31552, 31553, -1, 31554, 31555, 31556, -1, 31557, 31558, 31559, -1, 31560, 31561, 31562, -1, 31563, 31564, 31565, -1, 31566, 31567, 31568, -1, 31569, 31570, 31571, -1, 31572, 31573, 31574, -1, 31575, 31576, 31577, -1, 31578, 31579, 31580, -1, 31581, 31582, 31583, -1, 31584, 31585, 31586, -1, 31587, 31588, 31589, -1, 31590, 31591, 31592, -1, 31593, 31594, 31595, -1, 31596, 31597, 31598, -1, 31599, 31600, 31601, -1, 31602, 31603, 31604, -1, 31605, 31606, 31607, -1, 31608, 31609, 31610, -1, 31611, 31612, 31613, -1, 31614, 31615, 31616, -1, 31617, 31618, 31619, -1, 31620, 31621, 31622, -1, 31623, 31624, 31625, -1, 31626, 31627, 31628, -1, 31629, 31630, 31631, -1, 31632, 31633, 31634, -1, 31635, 31636, 31637, -1, 31638, 31639, 31640, -1, 31641, 31642, 31643, -1, 31644, 31645, 31646, -1, 31647, 31648, 31649, -1, 31650, 31651, 31652, -1, 31653, 31654, 31655, -1, 31656, 31657, 31658, -1, 31659, 31660, 31661, -1, 31662, 31663, 31664, -1, 31665, 31666, 31667, -1, 31668, 31669, 31670, -1, 31671, 31672, 31673, -1, 31674, 31675, 31676, -1, 31677, 31678, 31679, -1, 31680, 31681, 31682, -1, 31683, 31684, 31685, -1, 31686, 31687, 31688, -1, 31689, 31690, 31691, -1, 31692, 31693, 31694, -1, 31695, 31696, 31697, -1, 31698, 31699, 31700, -1, 31701, 31702, 31703, -1, 31704, 31705, 31706, -1, 31707, 31708, 31709, -1, 31710, 31711, 31712, -1, 31713, 31714, 31715, -1, 31716, 31717, 31718, -1, 31719, 31720, 31721, -1, 31722, 31723, 31724, -1, 31725, 31726, 31727, -1, 31728, 31729, 31730, -1, 31731, 31732, 31733, -1, 31734, 31735, 31736, -1, 31737, 31738, 31739, -1, 31740, 31741, 31742, -1, 31743, 31744, 31745, -1, 31746, 31747, 31748, -1, 31749, 31750, 31751, -1, 31752, 31753, 31754, -1, 31755, 31756, 31757, -1, 31758, 31759, 31760, -1, 31761, 31762, 31763, -1, 31764, 31765, 31766, -1, 31767, 31768, 31769, -1, 31770, 31771, 31772, -1, 31773, 31774, 31775, -1, 31776, 31777, 31778, -1, 31779, 31780, 31781, -1, 31782, 31783, 31784, -1, 31785, 31786, 31787, -1, 31788, 31789, 31790, -1, 31791, 31792, 31793, -1, 31794, 31795, 31796, -1, 31797, 31798, 31799, -1, 31800, 31801, 31802, -1, 31803, 31804, 31805, -1, 31806, 31807, 31808, -1, 31809, 31810, 31811, -1, 31812, 31813, 31814, -1, 31815, 31816, 31817, -1, 31818, 31819, 31820, -1, 31821, 31822, 31823, -1, 31824, 31825, 31826, -1, 31827, 31828, 31829, -1, 31830, 31831, 31832, -1, 31833, 31834, 31835, -1, 31836, 31837, 31838, -1, 31839, 31840, 31841, -1, 31842, 31843, 31844, -1, 31845, 31846, 31847, -1, 31848, 31849, 31850, -1, 31851, 31852, 31853, -1, 31854, 31855, 31856, -1, 31857, 31858, 31859, -1, 31860, 31861, 31862, -1, 31863, 31864, 31865, -1, 31866, 31867, 31868, -1, 31869, 31870, 31871, -1, 31872, 31873, 31874, -1, 31875, 31876, 31877, -1, 31878, 31879, 31880, -1, 31881, 31882, 31883, -1, 31884, 31885, 31886, -1, 31887, 31888, 31889, -1, 31890, 31891, 31892, -1, 31893, 31894, 31895, -1, 31896, 31897, 31898, -1, 31899, 31900, 31901, -1, 31902, 31903, 31904, -1, 31905, 31906, 31907, -1, 31908, 31909, 31910, -1, 31911, 31912, 31913, -1, 31914, 31915, 31916, -1, 31917, 31918, 31919, -1, 31920, 31921, 31922, -1, 31923, 31924, 31925, -1, 31926, 31927, 31928, -1, 31929, 31930, 31931, -1, 31932, 31933, 31934, -1, 31935, 31936, 31937, -1, 31938, 31939, 31940, -1, 31941, 31942, 31943, -1, 31944, 31945, 31946, -1, 31947, 31948, 31949, -1, 31950, 31951, 31952, -1, 31953, 31954, 31955, -1, 31956, 31957, 31958, -1, 31959, 31960, 31961, -1, 31962, 31963, 31964, -1, 31965, 31966, 31967, -1, 31968, 31969, 31970, -1, 31971, 31972, 31973, -1, 31974, 31975, 31976, -1, 31977, 31978, 31979, -1, 31980, 31981, 31982, -1, 31983, 31984, 31985, -1, 31986, 31987, 31988, -1, 31989, 31990, 31991, -1, 31992, 31993, 31994, -1, 31995, 31996, 31997, -1, 31998, 31999, 32000, -1, 32001, 32002, 32003, -1, 32004, 32005, 32006, -1, 32007, 32008, 32009, -1, 32010, 32011, 32012, -1, 32013, 32014, 32015, -1, 32016, 32017, 32018, -1, 32019, 32020, 32021, -1, 32022, 32023, 32024, -1, 32025, 32026, 32027, -1, 32028, 32029, 32030, -1, 32031, 32032, 32033, -1, 32034, 32035, 32036, -1, 32037, 32038, 32039, -1, 32040, 32041, 32042, -1, 32043, 32044, 32045, -1, 32046, 32047, 32048, -1, 32049, 32050, 32051, -1, 32052, 32053, 32054, -1, 32055, 32056, 32057, -1, 32058, 32059, 32060, -1, 32061, 32062, 32063, -1, 32064, 32065, 32066, -1, 32067, 32068, 32069, -1, 32070, 32071, 32072, -1, 32073, 32074, 32075, -1, 32076, 32077, 32078, -1, 32079, 32080, 32081, -1, 32082, 32083, 32084, -1, 32085, 32086, 32087, -1, 32088, 32089, 32090, -1, 32091, 32092, 32093, -1, 32094, 32095, 32096, -1, 32097, 32098, 32099, -1, 32100, 32101, 32102, -1, 32103, 32104, 32105, -1, 32106, 32107, 32108, -1, 32109, 32110, 32111, -1, 32112, 32113, 32114, -1, 32115, 32116, 32117, -1, 32118, 32119, 32120, -1, 32121, 32122, 32123, -1, 32124, 32125, 32126, -1, 32127, 32128, 32129, -1, 32130, 32131, 32132, -1, 32133, 32134, 32135, -1, 32136, 32137, 32138, -1, 32139, 32140, 32141, -1, 32142, 32143, 32144, -1, 32145, 32146, 32147, -1, 32148, 32149, 32150, -1, 32151, 32152, 32153, -1, 32154, 32155, 32156, -1, 32157, 32158, 32159, -1, 32160, 32161, 32162, -1, 32163, 32164, 32165, -1, 32166, 32167, 32168, -1, 32169, 32170, 32171, -1, 32172, 32173, 32174, -1, 32175, 32176, 32177, -1, 32178, 32179, 32180, -1, 32181, 32182, 32183, -1, 32184, 32185, 32186, -1, 32187, 32188, 32189, -1, 32190, 32191, 32192, -1, 32193, 32194, 32195, -1, 32196, 32197, 32198, -1, 32199, 32200, 32201, -1, 32202, 32203, 32204, -1, 32205, 32206, 32207, -1, 32208, 32209, 32210, -1, 32211, 32212, 32213, -1, 32214, 32215, 32216, -1, 32217, 32218, 32219, -1, 32220, 32221, 32222, -1, 32223, 32224, 32225, -1, 32226, 32227, 32228, -1, 32229, 32230, 32231, -1, 32232, 32233, 32234, -1, 32235, 32236, 32237, -1, 32238, 32239, 32240, -1, 32241, 32242, 32243, -1, 32244, 32245, 32246, -1, 32247, 32248, 32249, -1, 32250, 32251, 32252, -1, 32253, 32254, 32255, -1, 32256, 32257, 32258, -1, 32259, 32260, 32261, -1, 32262, 32263, 32264, -1, 32265, 32266, 32267, -1, 32268, 32269, 32270, -1, 32271, 32272, 32273, -1, 32274, 32275, 32276, -1, 32277, 32278, 32279, -1, 32280, 32281, 32282, -1, 32283, 32284, 32285, -1, 32286, 32287, 32288, -1, 32289, 32290, 32291, -1, 32292, 32293, 32294, -1, 32295, 32296, 32297, -1, 32298, 32299, 32300, -1, 32301, 32302, 32303, -1, 32304, 32305, 32306, -1, 32307, 32308, 32309, -1, 32310, 32311, 32312, -1, 32313, 32314, 32315, -1, 32316, 32317, 32318, -1, 32319, 32320, 32321, -1, 32322, 32323, 32324, -1, 32325, 32326, 32327, -1, 32328, 32329, 32330, -1, 32331, 32332, 32333, -1, 32334, 32335, 32336, -1, 32337, 32338, 32339, -1, 32340, 32341, 32342, -1, 32343, 32344, 32345, -1, 32346, 32347, 32348, -1, 32349, 32350, 32351, -1, 32352, 32353, 32354, -1, 32355, 32356, 32357, -1, 32358, 32359, 32360, -1, 32361, 32362, 32363, -1, 32364, 32365, 32366, -1, 32367, 32368, 32369, -1, 32370, 32371, 32372, -1, 32373, 32374, 32375, -1, 32376, 32377, 32378, -1, 32379, 32380, 32381, -1, 32382, 32383, 32384, -1, 32385, 32386, 32387, -1, 32388, 32389, 32390, -1, 32391, 32392, 32393, -1, 32394, 32395, 32396, -1, 32397, 32398, 32399, -1, 32400, 32401, 32402, -1, 32403, 32404, 32405, -1, 32406, 32407, 32408, -1, 32409, 32410, 32411, -1, 32412, 32413, 32414, -1, 32415, 32416, 32417, -1, 32418, 32419, 32420, -1, 32421, 32422, 32423, -1, 32424, 32425, 32426, -1, 32427, 32428, 32429, -1, 32430, 32431, 32432, -1, 32433, 32434, 32435, -1, 32436, 32437, 32438, -1, 32439, 32440, 32441, -1, 32442, 32443, 32444, -1, 32445, 32446, 32447, -1, 32448, 32449, 32450, -1, 32451, 32452, 32453, -1, 32454, 32455, 32456, -1, 32457, 32458, 32459, -1, 32460, 32461, 32462, -1, 32463, 32464, 32465, -1, 32466, 32467, 32468, -1, 32469, 32470, 32471, -1, 32472, 32473, 32474, -1, 32475, 32476, 32477, -1, 32478, 32479, 32480, -1, 32481, 32482, 32483, -1, 32484, 32485, 32486, -1, 32487, 32488, 32489, -1, 32490, 32491, 32492, -1, 32493, 32494, 32495, -1, 32496, 32497, 32498, -1, 32499, 32500, 32501, -1, 32502, 32503, 32504, -1, 32505, 32506, 32507, -1, 32508, 32509, 32510, -1, 32511, 32512, 32513, -1, 32514, 32515, 32516, -1, 32517, 32518, 32519, -1, 32520, 32521, 32522, -1, 32523, 32524, 32525, -1, 32526, 32527, 32528, -1, 32529, 32530, 32531, -1, 32532, 32533, 32534, -1, 32535, 32536, 32537, -1, 32538, 32539, 32540, -1, 32541, 32542, 32543, -1, 32544, 32545, 32546, -1, 32547, 32548, 32549, -1, 32550, 32551, 32552, -1, 32553, 32554, 32555, -1, 32556, 32557, 32558, -1, 32559, 32560, 32561, -1, 32562, 32563, 32564, -1, 32565, 32566, 32567, -1, 32568, 32569, 32570, -1, 32571, 32572, 32573, -1, 32574, 32575, 32576, -1, 32577, 32578, 32579, -1, 32580, 32581, 32582, -1, 32583, 32584, 32585, -1, 32586, 32587, 32588, -1, 32589, 32590, 32591, -1, 32592, 32593, 32594, -1, 32595, 32596, 32597, -1, 32598, 32599, 32600, -1, 32601, 32602, 32603, -1, 32604, 32605, 32606, -1, 32607, 32608, 32609, -1, 32610, 32611, 32612, -1, 32613, 32614, 32615, -1, 32616, 32617, 32618, -1, 32619, 32620, 32621, -1, 32622, 32623, 32624, -1, 32625, 32626, 32627, -1, 32628, 32629, 32630, -1, 32631, 32632, 32633, -1, 32634, 32635, 32636, -1, 32637, 32638, 32639, -1, 32640, 32641, 32642, -1, 32643, 32644, 32645, -1, 32646, 32647, 32648, -1, 32649, 32650, 32651, -1, 32652, 32653, 32654, -1, 32655, 32656, 32657, -1, 32658, 32659, 32660, -1, 32661, 32662, 32663, -1, 32664, 32665, 32666, -1, 32667, 32668, 32669, -1, 32670, 32671, 32672, -1, 32673, 32674, 32675, -1, 32676, 32677, 32678, -1, 32679, 32680, 32681, -1, 32682, 32683, 32684, -1, 32685, 32686, 32687, -1, 32688, 32689, 32690, -1, 32691, 32692, 32693, -1, 32694, 32695, 32696, -1, 32697, 32698, 32699, -1, 32700, 32701, 32702, -1, 32703, 32704, 32705, -1, 32706, 32707, 32708, -1, 32709, 32710, 32711, -1, 32712, 32713, 32714, -1, 32715, 32716, 32717, -1, 32718, 32719, 32720, -1, 32721, 32722, 32723, -1, 32724, 32725, 32726, -1, 32727, 32728, 32729, -1, 32730, 32731, 32732, -1, 32733, 32734, 32735, -1, 32736, 32737, 32738, -1, 32739, 32740, 32741, -1, 32742, 32743, 32744, -1, 32745, 32746, 32747, -1, 32748, 32749, 32750, -1, 32751, 32752, 32753, -1, 32754, 32755, 32756, -1, 32757, 32758, 32759, -1, 32760, 32761, 32762, -1, 32763, 32764, 32765, -1, 32766, 32767, 32768, -1, 32769, 32770, 32771, -1, 32772, 32773, 32774, -1, 32775, 32776, 32777, -1, 32778, 32779, 32780, -1, 32781, 32782, 32783, -1, 32784, 32785, 32786, -1, 32787, 32788, 32789, -1, 32790, 32791, 32792, -1, 32793, 32794, 32795, -1, 32796, 32797, 32798, -1, 32799, 32800, 32801, -1, 32802, 32803, 32804, -1, 32805, 32806, 32807, -1, 32808, 32809, 32810, -1, 32811, 32812, 32813, -1, 32814, 32815, 32816, -1, 32817, 32818, 32819, -1, 32820, 32821, 32822, -1, 32823, 32824, 32825, -1, 32826, 32827, 32828, -1, 32829, 32830, 32831, -1, 32832, 32833, 32834, -1, 32835, 32836, 32837, -1, 32838, 32839, 32840, -1, 32841, 32842, 32843, -1, 32844, 32845, 32846, -1, 32847, 32848, 32849, -1, 32850, 32851, 32852, -1, 32853, 32854, 32855, -1, 32856, 32857, 32858, -1, 32859, 32860, 32861, -1, 32862, 32863, 32864, -1, 32865, 32866, 32867, -1, 32868, 32869, 32870, -1, 32871, 32872, 32873, -1, 32874, 32875, 32876, -1, 32877, 32878, 32879, -1, 32880, 32881, 32882, -1, 32883, 32884, 32885, -1, 32886, 32887, 32888, -1, 32889, 32890, 32891, -1, 32892, 32893, 32894, -1, 32895, 32896, 32897, -1, 32898, 32899, 32900, -1, 32901, 32902, 32903, -1, 32904, 32905, 32906, -1, 32907, 32908, 32909, -1, 32910, 32911, 32912, -1, 32913, 32914, 32915, -1, 32916, 32917, 32918, -1, 32919, 32920, 32921, -1, 32922, 32923, 32924, -1, 32925, 32926, 32927, -1, 32928, 32929, 32930, -1, 32931, 32932, 32933, -1, 32934, 32935, 32936, -1, 32937, 32938, 32939, -1, 32940, 32941, 32942, -1, 32943, 32944, 32945, -1, 32946, 32947, 32948, -1, 32949, 32950, 32951, -1, 32952, 32953, 32954, -1, 32955, 32956, 32957, -1, 32958, 32959, 32960, -1, 32961, 32962, 32963, -1, 32964, 32965, 32966, -1, 32967, 32968, 32969, -1, 32970, 32971, 32972, -1, 32973, 32974, 32975, -1, 32976, 32977, 32978, -1, 32979, 32980, 32981, -1, 32982, 32983, 32984, -1, 32985, 32986, 32987, -1, 32988, 32989, 32990, -1, 32991, 32992, 32993, -1, 32994, 32995, 32996, -1, 32997, 32998, 32999, -1, 33000, 33001, 33002, -1, 33003, 33004, 33005, -1, 33006, 33007, 33008, -1, 33009, 33010, 33011, -1, 33012, 33013, 33014, -1, 33015, 33016, 33017, -1, 33018, 33019, 33020, -1, 33021, 33022, 33023, -1, 33024, 33025, 33026, -1, 33027, 33028, 33029, -1, 33030, 33031, 33032, -1, 33033, 33034, 33035, -1, 33036, 33037, 33038, -1, 33039, 33040, 33041, -1, 33042, 33043, 33044, -1, 33045, 33046, 33047, -1, 33048, 33049, 33050, -1, 33051, 33052, 33053, -1, 33054, 33055, 33056, -1, 33057, 33058, 33059, -1, 33060, 33061, 33062, -1, 33063, 33064, 33065, -1, 33066, 33067, 33068, -1, 33069, 33070, 33071, -1, 33072, 33073, 33074, -1, 33075, 33076, 33077, -1, 33078, 33079, 33080, -1, 33081, 33082, 33083, -1, 33084, 33085, 33086, -1, 33087, 33088, 33089, -1, 33090, 33091, 33092, -1, 33093, 33094, 33095, -1, 33096, 33097, 33098, -1, 33099, 33100, 33101, -1, 33102, 33103, 33104, -1, 33105, 33106, 33107, -1, 33108, 33109, 33110, -1, 33111, 33112, 33113, -1, 33114, 33115, 33116, -1, 33117, 33118, 33119, -1, 33120, 33121, 33122, -1, 33123, 33124, 33125, -1, 33126, 33127, 33128, -1, 33129, 33130, 33131, -1, 33132, 33133, 33134, -1, 33135, 33136, 33137, -1, 33138, 33139, 33140, -1, 33141, 33142, 33143, -1, 33144, 33145, 33146, -1, 33147, 33148, 33149, -1, 33150, 33151, 33152, -1, 33153, 33154, 33155, -1, 33156, 33157, 33158, -1, 33159, 33160, 33161, -1, 33162, 33163, 33164, -1, 33165, 33166, 33167, -1, 33168, 33169, 33170, -1, 33171, 33172, 33173, -1, 33174, 33175, 33176, -1, 33177, 33178, 33179, -1, 33180, 33181, 33182, -1, 33183, 33184, 33185, -1, 33186, 33187, 33188, -1, 33189, 33190, 33191, -1, 33192, 33193, 33194, -1, 33195, 33196, 33197, -1, 33198, 33199, 33200, -1, 33201, 33202, 33203, -1, 33204, 33205, 33206, -1, 33207, 33208, 33209, -1, 33210, 33211, 33212, -1, 33213, 33214, 33215, -1, 33216, 33217, 33218, -1, 33219, 33220, 33221, -1, 33222, 33223, 33224, -1, 33225, 33226, 33227, -1, 33228, 33229, 33230, -1, 33231, 33232, 33233, -1, 33234, 33235, 33236, -1, 33237, 33238, 33239, -1, 33240, 33241, 33242, -1, 33243, 33244, 33245, -1, 33246, 33247, 33248, -1, 33249, 33250, 33251, -1, 33252, 33253, 33254, -1, 33255, 33256, 33257, -1, 33258, 33259, 33260, -1, 33261, 33262, 33263, -1, 33264, 33265, 33266, -1, 33267, 33268, 33269, -1, 33270, 33271, 33272, -1, 33273, 33274, 33275, -1, 33276, 33277, 33278, -1, 33279, 33280, 33281, -1, 33282, 33283, 33284, -1, 33285, 33286, 33287, -1, 33288, 33289, 33290, -1, 33291, 33292, 33293, -1, 33294, 33295, 33296, -1, 33297, 33298, 33299, -1, 33300, 33301, 33302, -1, 33303, 33304, 33305, -1, 33306, 33307, 33308, -1, 33309, 33310, 33311, -1, 33312, 33313, 33314, -1, 33315, 33316, 33317, -1, 33318, 33319, 33320, -1, 33321, 33322, 33323, -1, 33324, 33325, 33326, -1, 33327, 33328, 33329, -1, 33330, 33331, 33332, -1, 33333, 33334, 33335, -1, 33336, 33337, 33338, -1, 33339, 33340, 33341, -1, 33342, 33343, 33344, -1, 33345, 33346, 33347, -1, 33348, 33349, 33350, -1, 33351, 33352, 33353, -1, 33354, 33355, 33356, -1, 33357, 33358, 33359, -1, 33360, 33361, 33362, -1, 33363, 33364, 33365, -1, 33366, 33367, 33368, -1, 33369, 33370, 33371, -1, 33372, 33373, 33374, -1, 33375, 33376, 33377, -1, 33378, 33379, 33380, -1, 33381, 33382, 33383, -1, 33384, 33385, 33386, -1, 33387, 33388, 33389, -1, 33390, 33391, 33392, -1, 33393, 33394, 33395, -1, 33396, 33397, 33398, -1, 33399, 33400, 33401, -1, 33402, 33403, 33404, -1, 33405, 33406, 33407, -1, 33408, 33409, 33410, -1, 33411, 33412, 33413, -1, 33414, 33415, 33416, -1, 33417, 33418, 33419, -1, 33420, 33421, 33422, -1, 33423, 33424, 33425, -1, 33426, 33427, 33428, -1, 33429, 33430, 33431, -1, 33432, 33433, 33434, -1, 33435, 33436, 33437, -1, 33438, 33439, 33440, -1, 33441, 33442, 33443, -1, 33444, 33445, 33446, -1, 33447, 33448, 33449, -1, 33450, 33451, 33452, -1, 33453, 33454, 33455, -1, 33456, 33457, 33458, -1, 33459, 33460, 33461, -1, 33462, 33463, 33464, -1, 33465, 33466, 33467, -1, 33468, 33469, 33470, -1, 33471, 33472, 33473, -1, 33474, 33475, 33476, -1, 33477, 33478, 33479, -1, 33480, 33481, 33482, -1, 33483, 33484, 33485, -1, 33486, 33487, 33488, -1, 33489, 33490, 33491, -1, 33492, 33493, 33494, -1, 33495, 33496, 33497, -1, 33498, 33499, 33500, -1, 33501, 33502, 33503, -1, 33504, 33505, 33506, -1, 33507, 33508, 33509, -1, 33510, 33511, 33512, -1, 33513, 33514, 33515, -1, 33516, 33517, 33518, -1, 33519, 33520, 33521, -1, 33522, 33523, 33524, -1, 33525, 33526, 33527, -1, 33528, 33529, 33530, -1, 33531, 33532, 33533, -1, 33534, 33535, 33536, -1, 33537, 33538, 33539, -1, 33540, 33541, 33542, -1, 33543, 33544, 33545, -1, 33546, 33547, 33548, -1, 33549, 33550, 33551, -1, 33552, 33553, 33554, -1, 33555, 33556, 33557, -1, 33558, 33559, 33560, -1, 33561, 33562, 33563, -1, 33564, 33565, 33566, -1, 33567, 33568, 33569, -1, 33570, 33571, 33572, -1, 33573, 33574, 33575, -1, 33576, 33577, 33578, -1, 33579, 33580, 33581, -1, 33582, 33583, 33584, -1, 33585, 33586, 33587, -1, 33588, 33589, 33590, -1, 33591, 33592, 33593, -1, 33594, 33595, 33596, -1, 33597, 33598, 33599, -1, 33600, 33601, 33602, -1, 33603, 33604, 33605, -1, 33606, 33607, 33608, -1, 33609, 33610, 33611, -1, 33612, 33613, 33614, -1, 33615, 33616, 33617, -1, 33618, 33619, 33620, -1, 33621, 33622, 33623, -1, 33624, 33625, 33626, -1, 33627, 33628, 33629, -1, 33630, 33631, 33632, -1, 33633, 33634, 33635, -1, 33636, 33637, 33638, -1, 33639, 33640, 33641, -1, 33642, 33643, 33644, -1, 33645, 33646, 33647, -1, 33648, 33649, 33650, -1, 33651, 33652, 33653, -1, 33654, 33655, 33656, -1, 33657, 33658, 33659, -1, 33660, 33661, 33662, -1, 33663, 33664, 33665, -1, 33666, 33667, 33668, -1, 33669, 33670, 33671, -1, 33672, 33673, 33674, -1, 33675, 33676, 33677, -1, 33678, 33679, 33680, -1, 33681, 33682, 33683, -1, 33684, 33685, 33686, -1, 33687, 33688, 33689, -1, 33690, 33691, 33692, -1, 33693, 33694, 33695, -1, 33696, 33697, 33698, -1, 33699, 33700, 33701, -1, 33702, 33703, 33704, -1, 33705, 33706, 33707, -1, 33708, 33709, 33710, -1, 33711, 33712, 33713, -1, 33714, 33715, 33716, -1, 33717, 33718, 33719, -1, 33720, 33721, 33722, -1, 33723, 33724, 33725, -1, 33726, 33727, 33728, -1, 33729, 33730, 33731, -1, 33732, 33733, 33734, -1, 33735, 33736, 33737, -1, 33738, 33739, 33740, -1, 33741, 33742, 33743, -1, 33744, 33745, 33746, -1, 33747, 33748, 33749, -1, 33750, 33751, 33752, -1, 33753, 33754, 33755, -1, 33756, 33757, 33758, -1, 33759, 33760, 33761, -1, 33762, 33763, 33764, -1, 33765, 33766, 33767, -1, 33768, 33769, 33770, -1, 33771, 33772, 33773, -1, 33774, 33775, 33776, -1, 33777, 33778, 33779, -1, 33780, 33781, 33782, -1, 33783, 33784, 33785, -1, 33786, 33787, 33788, -1, 33789, 33790, 33791, -1, 33792, 33793, 33794, -1, 33795, 33796, 33797, -1, 33798, 33799, 33800, -1, 33801, 33802, 33803, -1, 33804, 33805, 33806, -1, 33807, 33808, 33809, -1, 33810, 33811, 33812, -1, 33813, 33814, 33815, -1, 33816, 33817, 33818, -1, 33819, 33820, 33821, -1, 33822, 33823, 33824, -1, 33825, 33826, 33827, -1, 33828, 33829, 33830, -1, 33831, 33832, 33833, -1, 33834, 33835, 33836, -1, 33837, 33838, 33839, -1, 33840, 33841, 33842, -1, 33843, 33844, 33845, -1, 33846, 33847, 33848, -1, 33849, 33850, 33851, -1, 33852, 33853, 33854, -1, 33855, 33856, 33857, -1, 33858, 33859, 33860, -1, 33861, 33862, 33863, -1, 33864, 33865, 33866, -1, 33867, 33868, 33869, -1, 33870, 33871, 33872, -1, 33873, 33874, 33875, -1, 33876, 33877, 33878, -1, 33879, 33880, 33881, -1, 33882, 33883, 33884, -1, 33885, 33886, 33887, -1, 33888, 33889, 33890, -1, 33891, 33892, 33893, -1, 33894, 33895, 33896, -1, 33897, 33898, 33899, -1, 33900, 33901, 33902, -1, 33903, 33904, 33905, -1, 33906, 33907, 33908, -1, 33909, 33910, 33911, -1, 33912, 33913, 33914, -1, 33915, 33916, 33917, -1, 33918, 33919, 33920, -1, 33921, 33922, 33923, -1, 33924, 33925, 33926, -1, 33927, 33928, 33929, -1, 33930, 33931, 33932, -1, 33933, 33934, 33935, -1, 33936, 33937, 33938, -1, 33939, 33940, 33941, -1, 33942, 33943, 33944, -1, 33945, 33946, 33947, -1, 33948, 33949, 33950, -1, 33951, 33952, 33953, -1, 33954, 33955, 33956, -1, 33957, 33958, 33959, -1, 33960, 33961, 33962, -1, 33963, 33964, 33965, -1, 33966, 33967, 33968, -1, 33969, 33970, 33971, -1, 33972, 33973, 33974, -1, 33975, 33976, 33977, -1, 33978, 33979, 33980, -1, 33981, 33982, 33983, -1, 33984, 33985, 33986, -1, 33987, 33988, 33989, -1, 33990, 33991, 33992, -1, 33993, 33994, 33995, -1, 33996, 33997, 33998, -1, 33999, 34000, 34001, -1, 34002, 34003, 34004, -1, 34005, 34006, 34007, -1, 34008, 34009, 34010, -1, 34011, 34012, 34013, -1, 34014, 34015, 34016, -1, 34017, 34018, 34019, -1, 34020, 34021, 34022, -1, 34023, 34024, 34025, -1, 34026, 34027, 34028, -1, 34029, 34030, 34031, -1, 34032, 34033, 34034, -1, 34035, 34036, 34037, -1, 34038, 34039, 34040, -1, 34041, 34042, 34043, -1, 34044, 34045, 34046, -1, 34047, 34048, 34049, -1, 34050, 34051, 34052, -1, 34053, 34054, 34055, -1, 34056, 34057, 34058, -1, 34059, 34060, 34061, -1, 34062, 34063, 34064, -1, 34065, 34066, 34067, -1, 34068, 34069, 34070, -1, 34071, 34072, 34073, -1, 34074, 34075, 34076, -1, 34077, 34078, 34079, -1, 34080, 34081, 34082, -1, 34083, 34084, 34085, -1, 34086, 34087, 34088, -1, 34089, 34090, 34091, -1, 34092, 34093, 34094, -1, 34095, 34096, 34097, -1, 34098, 34099, 34100, -1, 34101, 34102, 34103, -1, 34104, 34105, 34106, -1, 34107, 34108, 34109, -1, 34110, 34111, 34112, -1, 34113, 34114, 34115, -1, 34116, 34117, 34118, -1, 34119, 34120, 34121, -1, 34122, 34123, 34124, -1, 34125, 34126, 34127, -1, 34128, 34129, 34130, -1, 34131, 34132, 34133, -1, 34134, 34135, 34136, -1, 34137, 34138, 34139, -1, 34140, 34141, 34142, -1, 34143, 34144, 34145, -1, 34146, 34147, 34148, -1, 34149, 34150, 34151, -1, 34152, 34153, 34154, -1, 34155, 34156, 34157, -1, 34158, 34159, 34160, -1, 34161, 34162, 34163, -1, 34164, 34165, 34166, -1, 34167, 34168, 34169, -1, 34170, 34171, 34172, -1, 34173, 34174, 34175, -1, 34176, 34177, 34178, -1, 34179, 34180, 34181, -1, 34182, 34183, 34184, -1, 34185, 34186, 34187, -1, 34188, 34189, 34190, -1, 34191, 34192, 34193, -1, 34194, 34195, 34196, -1, 34197, 34198, 34199, -1, 34200, 34201, 34202, -1, 34203, 34204, 34205, -1, 34206, 34207, 34208, -1, 34209, 34210, 34211, -1, 34212, 34213, 34214, -1, 34215, 34216, 34217, -1, 34218, 34219, 34220, -1, 34221, 34222, 34223, -1, 34224, 34225, 34226, -1, 34227, 34228, 34229, -1, 34230, 34231, 34232, -1, 34233, 34234, 34235, -1, 34236, 34237, 34238, -1, 34239, 34240, 34241, -1, 34242, 34243, 34244, -1, 34245, 34246, 34247, -1, 34248, 34249, 34250, -1, 34251, 34252, 34253, -1, 34254, 34255, 34256, -1, 34257, 34258, 34259, -1, 34260, 34261, 34262, -1, 34263, 34264, 34265, -1, 34266, 34267, 34268, -1, 34269, 34270, 34271, -1, 34272, 34273, 34274, -1, 34275, 34276, 34277, -1, 34278, 34279, 34280, -1, 34281, 34282, 34283, -1, 34284, 34285, 34286, -1, 34287, 34288, 34289, -1, 34290, 34291, 34292, -1, 34293, 34294, 34295, -1, 34296, 34297, 34298, -1, 34299, 34300, 34301, -1, 34302, 34303, 34304, -1, 34305, 34306, 34307, -1, 34308, 34309, 34310, -1, 34311, 34312, 34313, -1, 34314, 34315, 34316, -1, 34317, 34318, 34319, -1, 34320, 34321, 34322, -1, 34323, 34324, 34325, -1, 34326, 34327, 34328, -1, 34329, 34330, 34331, -1, 34332, 34333, 34334, -1, 34335, 34336, 34337, -1, 34338, 34339, 34340, -1, 34341, 34342, 34343, -1, 34344, 34345, 34346, -1, 34347, 34348, 34349, -1, 34350, 34351, 34352, -1, 34353, 34354, 34355, -1, 34356, 34357, 34358, -1, 34359, 34360, 34361, -1, 34362, 34363, 34364, -1, 34365, 34366, 34367, -1, 34368, 34369, 34370, -1, 34371, 34372, 34373, -1, 34374, 34375, 34376, -1, 34377, 34378, 34379, -1, 34380, 34381, 34382, -1, 34383, 34384, 34385, -1, 34386, 34387, 34388, -1, 34389, 34390, 34391, -1, 34392, 34393, 34394, -1, 34395, 34396, 34397, -1, 34398, 34399, 34400, -1, 34401, 34402, 34403, -1, 34404, 34405, 34406, -1, 34407, 34408, 34409, -1, 34410, 34411, 34412, -1, 34413, 34414, 34415, -1, 34416, 34417, 34418, -1, 34419, 34420, 34421, -1, 34422, 34423, 34424, -1, 34425, 34426, 34427, -1, 34428, 34429, 34430, -1, 34431, 34432, 34433, -1, 34434, 34435, 34436, -1, 34437, 34438, 34439, -1, 34440, 34441, 34442, -1, 34443, 34444, 34445, -1, 34446, 34447, 34448, -1, 34449, 34450, 34451, -1, 34452, 34453, 34454, -1, 34455, 34456, 34457, -1, 34458, 34459, 34460, -1, 34461, 34462, 34463, -1, 34464, 34465, 34466, -1, 34467, 34468, 34469, -1, 34470, 34471, 34472, -1, 34473, 34474, 34475, -1, 34476, 34477, 34478, -1, 34479, 34480, 34481, -1, 34482, 34483, 34484, -1, 34485, 34486, 34487, -1, 34488, 34489, 34490, -1, 34491, 34492, 34493, -1, 34494, 34495, 34496, -1, 34497, 34498, 34499, -1, 34500, 34501, 34502, -1, 34503, 34504, 34505, -1, 34506, 34507, 34508, -1, 34509, 34510, 34511, -1, 34512, 34513, 34514, -1, 34515, 34516, 34517, -1, 34518, 34519, 34520, -1, 34521, 34522, 34523, -1, 34524, 34525, 34526, -1, 34527, 34528, 34529, -1, 34530, 34531, 34532, -1, 34533, 34534, 34535, -1, 34536, 34537, 34538, -1, 34539, 34540, 34541, -1, 34542, 34543, 34544, -1, 34545, 34546, 34547, -1, 34548, 34549, 34550, -1, 34551, 34552, 34553, -1, 34554, 34555, 34556, -1, 34557, 34558, 34559, -1, 34560, 34561, 34562, -1, 34563, 34564, 34565, -1, 34566, 34567, 34568, -1, 34569, 34570, 34571, -1, 34572, 34573, 34574, -1, 34575, 34576, 34577, -1, 34578, 34579, 34580, -1, 34581, 34582, 34583, -1, 34584, 34585, 34586, -1, 34587, 34588, 34589, -1, 34590, 34591, 34592, -1, 34593, 34594, 34595, -1, 34596, 34597, 34598, -1, 34599, 34600, 34601, -1, 34602, 34603, 34604, -1, 34605, 34606, 34607, -1, 34608, 34609, 34610, -1, 34611, 34612, 34613, -1, 34614, 34615, 34616, -1, 34617, 34618, 34619, -1, 34620, 34621, 34622, -1, 34623, 34624, 34625, -1, 34626, 34627, 34628, -1, 34629, 34630, 34631, -1, 34632, 34633, 34634, -1, 34635, 34636, 34637, -1, 34638, 34639, 34640, -1, 34641, 34642, 34643, -1, 34644, 34645, 34646, -1, 34647, 34648, 34649, -1, 34650, 34651, 34652, -1, 34653, 34654, 34655, -1, 34656, 34657, 34658, -1, 34659, 34660, 34661, -1, 34662, 34663, 34664, -1, 34665, 34666, 34667, -1, 34668, 34669, 34670, -1, 34671, 34672, 34673, -1, 34674, 34675, 34676, -1, 34677, 34678, 34679, -1, 34680, 34681, 34682, -1, 34683, 34684, 34685, -1, 34686, 34687, 34688, -1, 34689, 34690, 34691, -1, 34692, 34693, 34694, -1, 34695, 34696, 34697, -1, 34698, 34699, 34700, -1, 34701, 34702, 34703, -1, 34704, 34705, 34706, -1, 34707, 34708, 34709, -1, 34710, 34711, 34712, -1, 34713, 34714, 34715, -1, 34716, 34717, 34718, -1, 34719, 34720, 34721, -1, 34722, 34723, 34724, -1, 34725, 34726, 34727, -1, 34728, 34729, 34730, -1, 34731, 34732, 34733, -1, 34734, 34735, 34736, -1, 34737, 34738, 34739, -1, 34740, 34741, 34742, -1, 34743, 34744, 34745, -1, 34746, 34747, 34748, -1, 34749, 34750, 34751, -1, 34752, 34753, 34754, -1, 34755, 34756, 34757, -1, 34758, 34759, 34760, -1, 34761, 34762, 34763, -1, 34764, 34765, 34766, -1, 34767, 34768, 34769, -1, 34770, 34771, 34772, -1, 34773, 34774, 34775, -1, 34776, 34777, 34778, -1, 34779, 34780, 34781, -1, 34782, 34783, 34784, -1, 34785, 34786, 34787, -1, 34788, 34789, 34790, -1, 34791, 34792, 34793, -1, 34794, 34795, 34796, -1, 34797, 34798, 34799, -1, 34800, 34801, 34802, -1, 34803, 34804, 34805, -1, 34806, 34807, 34808, -1, 34809, 34810, 34811, -1, 34812, 34813, 34814, -1, 34815, 34816, 34817, -1, 34818, 34819, 34820, -1, 34821, 34822, 34823, -1, 34824, 34825, 34826, -1, 34827, 34828, 34829, -1, 34830, 34831, 34832, -1, 34833, 34834, 34835, -1, 34836, 34837, 34838, -1, 34839, 34840, 34841, -1, 34842, 34843, 34844, -1, 34845, 34846, 34847, -1, 34848, 34849, 34850, -1, 34851, 34852, 34853, -1, 34854, 34855, 34856, -1, 34857, 34858, 34859, -1, 34860, 34861, 34862, -1, 34863, 34864, 34865, -1, 34866, 34867, 34868, -1, 34869, 34870, 34871, -1, 34872, 34873, 34874, -1, 34875, 34876, 34877, -1, 34878, 34879, 34880, -1, 34881, 34882, 34883, -1, 34884, 34885, 34886, -1, 34887, 34888, 34889, -1, 34890, 34891, 34892, -1, 34893, 34894, 34895, -1, 34896, 34897, 34898, -1, 34899, 34900, 34901, -1, 34902, 34903, 34904, -1, 34905, 34906, 34907, -1, 34908, 34909, 34910, -1, 34911, 34912, 34913, -1, 34914, 34915, 34916, -1, 34917, 34918, 34919, -1, 34920, 34921, 34922, -1, 34923, 34924, 34925, -1, 34926, 34927, 34928, -1, 34929, 34930, 34931, -1, 34932, 34933, 34934, -1, 34935, 34936, 34937, -1, 34938, 34939, 34940, -1, 34941, 34942, 34943, -1, 34944, 34945, 34946, -1, 34947, 34948, 34949, -1, 34950, 34951, 34952, -1, 34953, 34954, 34955, -1, 34956, 34957, 34958, -1, 34959, 34960, 34961, -1, 34962, 34963, 34964, -1, 34965, 34966, 34967, -1, 34968, 34969, 34970, -1, 34971, 34972, 34973, -1, 34974, 34975, 34976, -1, 34977, 34978, 34979, -1, 34980, 34981, 34982, -1, 34983, 34984, 34985, -1, 34986, 34987, 34988, -1, 34989, 34990, 34991, -1, 34992, 34993, 34994, -1, 34995, 34996, 34997, -1, 34998, 34999, 35000, -1, 35001, 35002, 35003, -1, 35004, 35005, 35006, -1, 35007, 35008, 35009, -1, 35010, 35011, 35012, -1, 35013, 35014, 35015, -1, 35016, 35017, 35018, -1, 35019, 35020, 35021, -1, 35022, 35023, 35024, -1, 35025, 35026, 35027, -1, 35028, 35029, 35030, -1, 35031, 35032, 35033, -1, 35034, 35035, 35036, -1, 35037, 35038, 35039, -1, 35040, 35041, 35042, -1, 35043, 35044, 35045, -1, 35046, 35047, 35048, -1, 35049, 35050, 35051, -1, 35052, 35053, 35054, -1, 35055, 35056, 35057, -1, 35058, 35059, 35060, -1, 35061, 35062, 35063, -1, 35064, 35065, 35066, -1, 35067, 35068, 35069, -1, 35070, 35071, 35072, -1, 35073, 35074, 35075, -1, 35076, 35077, 35078, -1, 35079, 35080, 35081, -1, 35082, 35083, 35084, -1, 35085, 35086, 35087, -1, 35088, 35089, 35090, -1, 35091, 35092, 35093, -1, 35094, 35095, 35096, -1, 35097, 35098, 35099, -1, 35100, 35101, 35102, -1, 35103, 35104, 35105, -1, 35106, 35107, 35108, -1, 35109, 35110, 35111, -1, 35112, 35113, 35114, -1, 35115, 35116, 35117, -1, 35118, 35119, 35120, -1, 35121, 35122, 35123, -1, 35124, 35125, 35126, -1, 35127, 35128, 35129, -1, 35130, 35131, 35132, -1, 35133, 35134, 35135, -1, 35136, 35137, 35138, -1, 35139, 35140, 35141, -1, 35142, 35143, 35144, -1, 35145, 35146, 35147, -1, 35148, 35149, 35150, -1, 35151, 35152, 35153, -1, 35154, 35155, 35156, -1, 35157, 35158, 35159, -1, 35160, 35161, 35162, -1, 35163, 35164, 35165, -1, 35166, 35167, 35168, -1, 35169, 35170, 35171, -1, 35172, 35173, 35174, -1, 35175, 35176, 35177, -1, 35178, 35179, 35180, -1, 35181, 35182, 35183, -1, 35184, 35185, 35186, -1, 35187, 35188, 35189, -1, 35190, 35191, 35192, -1, 35193, 35194, 35195, -1, 35196, 35197, 35198, -1, 35199, 35200, 35201, -1, 35202, 35203, 35204, -1, 35205, 35206, 35207, -1, 35208, 35209, 35210, -1, 35211, 35212, 35213, -1, 35214, 35215, 35216, -1, 35217, 35218, 35219, -1, 35220, 35221, 35222, -1, 35223, 35224, 35225, -1, 35226, 35227, 35228, -1, 35229, 35230, 35231, -1, 35232, 35233, 35234, -1, 35235, 35236, 35237, -1, 35238, 35239, 35240, -1, 35241, 35242, 35243, -1, 35244, 35245, 35246, -1, 35247, 35248, 35249, -1, 35250, 35251, 35252, -1, 35253, 35254, 35255, -1, 35256, 35257, 35258, -1, 35259, 35260, 35261, -1, 35262, 35263, 35264, -1, 35265, 35266, 35267, -1, 35268, 35269, 35270, -1, 35271, 35272, 35273, -1, 35274, 35275, 35276, -1, 35277, 35278, 35279, -1, 35280, 35281, 35282, -1, 35283, 35284, 35285, -1, 35286, 35287, 35288, -1, 35289, 35290, 35291, -1, 35292, 35293, 35294, -1, 35295, 35296, 35297, -1, 35298, 35299, 35300, -1, 35301, 35302, 35303, -1, 35304, 35305, 35306, -1, 35307, 35308, 35309, -1, 35310, 35311, 35312, -1, 35313, 35314, 35315, -1, 35316, 35317, 35318, -1, 35319, 35320, 35321, -1, 35322, 35323, 35324, -1, 35325, 35326, 35327, -1, 35328, 35329, 35330, -1, 35331, 35332, 35333, -1, 35334, 35335, 35336, -1, 35337, 35338, 35339, -1, 35340, 35341, 35342, -1, 35343, 35344, 35345, -1, 35346, 35347, 35348, -1, 35349, 35350, 35351, -1, 35352, 35353, 35354, -1, 35355, 35356, 35357, -1, 35358, 35359, 35360, -1, 35361, 35362, 35363, -1, 35364, 35365, 35366, -1, 35367, 35368, 35369, -1, 35370, 35371, 35372, -1, 35373, 35374, 35375, -1, 35376, 35377, 35378, -1, 35379, 35380, 35381, -1, 35382, 35383, 35384, -1, 35385, 35386, 35387, -1, 35388, 35389, 35390, -1, 35391, 35392, 35393, -1, 35394, 35395, 35396, -1, 35397, 35398, 35399, -1, 35400, 35401, 35402, -1, 35403, 35404, 35405, -1, 35406, 35407, 35408, -1, 35409, 35410, 35411, -1, 35412, 35413, 35414, -1, 35415, 35416, 35417, -1, 35418, 35419, 35420, -1, 35421, 35422, 35423, -1, 35424, 35425, 35426, -1, 35427, 35428, 35429, -1, 35430, 35431, 35432, -1, 35433, 35434, 35435, -1, 35436, 35437, 35438, -1, 35439, 35440, 35441, -1, 35442, 35443, 35444, -1, 35445, 35446, 35447, -1, 35448, 35449, 35450, -1, 35451, 35452, 35453, -1, 35454, 35455, 35456, -1, 35457, 35458, 35459, -1, 35460, 35461, 35462, -1, 35463, 35464, 35465, -1, 35466, 35467, 35468, -1, 35469, 35470, 35471, -1, 35472, 35473, 35474, -1, 35475, 35476, 35477, -1, 35478, 35479, 35480, -1, 35481, 35482, 35483, -1, 35484, 35485, 35486, -1, 35487, 35488, 35489, -1, 35490, 35491, 35492, -1, 35493, 35494, 35495, -1, 35496, 35497, 35498, -1, 35499, 35500, 35501, -1, 35502, 35503, 35504, -1, 35505, 35506, 35507, -1, 35508, 35509, 35510, -1, 35511, 35512, 35513, -1, 35514, 35515, 35516, -1, 35517, 35518, 35519, -1, 35520, 35521, 35522, -1, 35523, 35524, 35525, -1, 35526, 35527, 35528, -1, 35529, 35530, 35531, -1, 35532, 35533, 35534, -1, 35535, 35536, 35537, -1, 35538, 35539, 35540, -1, 35541, 35542, 35543, -1, 35544, 35545, 35546, -1, 35547, 35548, 35549, -1, 35550, 35551, 35552, -1, 35553, 35554, 35555, -1, 35556, 35557, 35558, -1, 35559, 35560, 35561, -1, 35562, 35563, 35564, -1, 35565, 35566, 35567, -1, 35568, 35569, 35570, -1, 35571, 35572, 35573, -1, 35574, 35575, 35576, -1, 35577, 35578, 35579, -1, 35580, 35581, 35582, -1, 35583, 35584, 35585, -1, 35586, 35587, 35588, -1, 35589, 35590, 35591, -1, 35592, 35593, 35594, -1, 35595, 35596, 35597, -1, 35598, 35599, 35600, -1, 35601, 35602, 35603, -1, 35604, 35605, 35606, -1, 35607, 35608, 35609, -1, 35610, 35611, 35612, -1, 35613, 35614, 35615, -1, 35616, 35617, 35618, -1, 35619, 35620, 35621, -1, 35622, 35623, 35624, -1, 35625, 35626, 35627, -1, 35628, 35629, 35630, -1, 35631, 35632, 35633, -1, 35634, 35635, 35636, -1, 35637, 35638, 35639, -1, 35640, 35641, 35642, -1, 35643, 35644, 35645, -1, 35646, 35647, 35648, -1, 35649, 35650, 35651, -1, 35652, 35653, 35654, -1, 35655, 35656, 35657, -1, 35658, 35659, 35660, -1, 35661, 35662, 35663, -1, 35664, 35665, 35666, -1, 35667, 35668, 35669, -1, 35670, 35671, 35672, -1, 35673, 35674, 35675, -1, 35676, 35677, 35678, -1, 35679, 35680, 35681, -1, 35682, 35683, 35684, -1, 35685, 35686, 35687, -1, 35688, 35689, 35690, -1, 35691, 35692, 35693, -1, 35694, 35695, 35696, -1, 35697, 35698, 35699, -1, 35700, 35701, 35702, -1, 35703, 35704, 35705, -1, 35706, 35707, 35708, -1, 35709, 35710, 35711, -1, 35712, 35713, 35714, -1, 35715, 35716, 35717, -1, 35718, 35719, 35720, -1, 35721, 35722, 35723, -1, 35724, 35725, 35726, -1, 35727, 35728, 35729, -1, 35730, 35731, 35732, -1, 35733, 35734, 35735, -1, 35736, 35737, 35738, -1, 35739, 35740, 35741, -1, 35742, 35743, 35744, -1, 35745, 35746, 35747, -1, 35748, 35749, 35750, -1, 35751, 35752, 35753, -1, 35754, 35755, 35756, -1, 35757, 35758, 35759, -1, 35760, 35761, 35762, -1, 35763, 35764, 35765, -1, 35766, 35767, 35768, -1, 35769, 35770, 35771, -1, 35772, 35773, 35774, -1, 35775, 35776, 35777, -1, 35778, 35779, 35780, -1, 35781, 35782, 35783, -1, 35784, 35785, 35786, -1, 35787, 35788, 35789, -1, 35790, 35791, 35792, -1, 35793, 35794, 35795, -1, 35796, 35797, 35798, -1, 35799, 35800, 35801, -1, 35802, 35803, 35804, -1, 35805, 35806, 35807, -1, 35808, 35809, 35810, -1, 35811, 35812, 35813, -1, 35814, 35815, 35816, -1, 35817, 35818, 35819, -1, 35820, 35821, 35822, -1, 35823, 35824, 35825, -1, 35826, 35827, 35828, -1, 35829, 35830, 35831, -1, 35832, 35833, 35834, -1, 35835, 35836, 35837, -1, 35838, 35839, 35840, -1, 35841, 35842, 35843, -1, 35844, 35845, 35846, -1, 35847, 35848, 35849, -1, 35850, 35851, 35852, -1, 35853, 35854, 35855, -1, 35856, 35857, 35858, -1, 35859, 35860, 35861, -1, 35862, 35863, 35864, -1, 35865, 35866, 35867, -1, 35868, 35869, 35870, -1, 35871, 35872, 35873, -1, 35874, 35875, 35876, -1, 35877, 35878, 35879, -1, 35880, 35881, 35882, -1, 35883, 35884, 35885, -1, 35886, 35887, 35888, -1, 35889, 35890, 35891, -1, 35892, 35893, 35894, -1, 35895, 35896, 35897, -1, 35898, 35899, 35900, -1, 35901, 35902, 35903, -1, 35904, 35905, 35906, -1, 35907, 35908, 35909, -1, 35910, 35911, 35912, -1, 35913, 35914, 35915, -1, 35916, 35917, 35918, -1, 35919, 35920, 35921, -1, 35922, 35923, 35924, -1, 35925, 35926, 35927, -1, 35928, 35929, 35930, -1, 35931, 35932, 35933, -1, 35934, 35935, 35936, -1, 35937, 35938, 35939, -1, 35940, 35941, 35942, -1, 35943, 35944, 35945, -1, 35946, 35947, 35948, -1, 35949, 35950, 35951, -1, 35952, 35953, 35954, -1, 35955, 35956, 35957, -1, 35958, 35959, 35960, -1, 35961, 35962, 35963, -1, 35964, 35965, 35966, -1, 35967, 35968, 35969, -1, 35970, 35971, 35972, -1, 35973, 35974, 35975, -1, 35976, 35977, 35978, -1, 35979, 35980, 35981, -1, 35982, 35983, 35984, -1, 35985, 35986, 35987, -1, 35988, 35989, 35990, -1, 35991, 35992, 35993, -1, 35994, 35995, 35996, -1, 35997, 35998, 35999, -1, 36000, 36001, 36002, -1, 36003, 36004, 36005, -1, 36006, 36007, 36008, -1, 36009, 36010, 36011, -1, 36012, 36013, 36014, -1, 36015, 36016, 36017, -1, 36018, 36019, 36020, -1, 36021, 36022, 36023, -1, 36024, 36025, 36026, -1, 36027, 36028, 36029, -1, 36030, 36031, 36032, -1, 36033, 36034, 36035, -1, 36036, 36037, 36038, -1, 36039, 36040, 36041, -1, 36042, 36043, 36044, -1, 36045, 36046, 36047, -1, 36048, 36049, 36050, -1, 36051, 36052, 36053, -1, 36054, 36055, 36056, -1, 36057, 36058, 36059, -1, 36060, 36061, 36062, -1, 36063, 36064, 36065, -1, 36066, 36067, 36068, -1, 36069, 36070, 36071, -1, 36072, 36073, 36074, -1, 36075, 36076, 36077, -1, 36078, 36079, 36080, -1, 36081, 36082, 36083, -1, 36084, 36085, 36086, -1, 36087, 36088, 36089, -1, 36090, 36091, 36092, -1, 36093, 36094, 36095, -1, 36096, 36097, 36098, -1, 36099, 36100, 36101, -1, 36102, 36103, 36104, -1, 36105, 36106, 36107, -1, 36108, 36109, 36110, -1, 36111, 36112, 36113, -1, 36114, 36115, 36116, -1, 36117, 36118, 36119, -1, 36120, 36121, 36122, -1, 36123, 36124, 36125, -1, 36126, 36127, 36128, -1, 36129, 36130, 36131, -1, 36132, 36133, 36134, -1, 36135, 36136, 36137, -1, 36138, 36139, 36140, -1, 36141, 36142, 36143, -1, 36144, 36145, 36146, -1, 36147, 36148, 36149, -1, 36150, 36151, 36152, -1, 36153, 36154, 36155, -1, 36156, 36157, 36158, -1, 36159, 36160, 36161, -1, 36162, 36163, 36164, -1, 36165, 36166, 36167, -1, 36168, 36169, 36170, -1, 36171, 36172, 36173, -1, 36174, 36175, 36176, -1, 36177, 36178, 36179, -1, 36180, 36181, 36182, -1, 36183, 36184, 36185, -1, 36186, 36187, 36188, -1, 36189, 36190, 36191, -1, 36192, 36193, 36194, -1, 36195, 36196, 36197, -1, 36198, 36199, 36200, -1, 36201, 36202, 36203, -1, 36204, 36205, 36206, -1, 36207, 36208, 36209, -1, 36210, 36211, 36212, -1, 36213, 36214, 36215, -1, 36216, 36217, 36218, -1, 36219, 36220, 36221, -1, 36222, 36223, 36224, -1, 36225, 36226, 36227, -1, 36228, 36229, 36230, -1, 36231, 36232, 36233, -1, 36234, 36235, 36236, -1, 36237, 36238, 36239, -1, 36240, 36241, 36242, -1, 36243, 36244, 36245, -1, 36246, 36247, 36248, -1, 36249, 36250, 36251, -1, 36252, 36253, 36254, -1, 36255, 36256, 36257, -1, 36258, 36259, 36260, -1, 36261, 36262, 36263, -1, 36264, 36265, 36266, -1, 36267, 36268, 36269, -1, 36270, 36271, 36272, -1, 36273, 36274, 36275, -1, 36276, 36277, 36278, -1, 36279, 36280, 36281, -1, 36282, 36283, 36284, -1, 36285, 36286, 36287, -1, 36288, 36289, 36290, -1, 36291, 36292, 36293, -1, 36294, 36295, 36296, -1, 36297, 36298, 36299, -1, 36300, 36301, 36302, -1, 36303, 36304, 36305, -1, 36306, 36307, 36308, -1, 36309, 36310, 36311, -1, 36312, 36313, 36314, -1, 36315, 36316, 36317, -1, 36318, 36319, 36320, -1, 36321, 36322, 36323, -1, 36324, 36325, 36326, -1, 36327, 36328, 36329, -1, 36330, 36331, 36332, -1, 36333, 36334, 36335, -1, 36336, 36337, 36338, -1, 36339, 36340, 36341, -1, 36342, 36343, 36344, -1, 36345, 36346, 36347, -1, 36348, 36349, 36350, -1, 36351, 36352, 36353, -1, 36354, 36355, 36356, -1, 36357, 36358, 36359, -1, 36360, 36361, 36362, -1, 36363, 36364, 36365, -1, 36366, 36367, 36368, -1, 36369, 36370, 36371, -1, 36372, 36373, 36374, -1, 36375, 36376, 36377, -1, 36378, 36379, 36380, -1, 36381, 36382, 36383, -1, 36384, 36385, 36386, -1, 36387, 36388, 36389, -1, 36390, 36391, 36392, -1, 36393, 36394, 36395, -1, 36396, 36397, 36398, -1, 36399, 36400, 36401, -1, 36402, 36403, 36404, -1, 36405, 36406, 36407, -1, 36408, 36409, 36410, -1, 36411, 36412, 36413, -1, 36414, 36415, 36416, -1, 36417, 36418, 36419, -1, 36420, 36421, 36422, -1, 36423, 36424, 36425, -1, 36426, 36427, 36428, -1, 36429, 36430, 36431, -1, 36432, 36433, 36434, -1, 36435, 36436, 36437, -1, 36438, 36439, 36440, -1, 36441, 36442, 36443, -1, 36444, 36445, 36446, -1, 36447, 36448, 36449, -1, 36450, 36451, 36452, -1, 36453, 36454, 36455, -1, 36456, 36457, 36458, -1, 36459, 36460, 36461, -1, 36462, 36463, 36464, -1, 36465, 36466, 36467, -1, 36468, 36469, 36470, -1, 36471, 36472, 36473, -1, 36474, 36475, 36476, -1, 36477, 36478, 36479, -1, 36480, 36481, 36482, -1, 36483, 36484, 36485, -1, 36486, 36487, 36488, -1, 36489, 36490, 36491, -1, 36492, 36493, 36494, -1, 36495, 36496, 36497, -1, 36498, 36499, 36500, -1, 36501, 36502, 36503, -1, 36504, 36505, 36506, -1, 36507, 36508, 36509, -1, 36510, 36511, 36512, -1, 36513, 36514, 36515, -1, 36516, 36517, 36518, -1, 36519, 36520, 36521, -1, 36522, 36523, 36524, -1, 36525, 36526, 36527, -1, 36528, 36529, 36530, -1, 36531, 36532, 36533, -1, 36534, 36535, 36536, -1, 36537, 36538, 36539, -1, 36540, 36541, 36542, -1, 36543, 36544, 36545, -1, 36546, 36547, 36548, -1, 36549, 36550, 36551, -1, 36552, 36553, 36554, -1, 36555, 36556, 36557, -1, 36558, 36559, 36560, -1, 36561, 36562, 36563, -1, 36564, 36565, 36566, -1, 36567, 36568, 36569, -1, 36570, 36571, 36572, -1, 36573, 36574, 36575, -1, 36576, 36577, 36578, -1, 36579, 36580, 36581, -1, 36582, 36583, 36584, -1, 36585, 36586, 36587, -1, 36588, 36589, 36590, -1, 36591, 36592, 36593, -1, 36594, 36595, 36596, -1, 36597, 36598, 36599, -1, 36600, 36601, 36602, -1, 36603, 36604, 36605, -1, 36606, 36607, 36608, -1, 36609, 36610, 36611, -1, 36612, 36613, 36614, -1, 36615, 36616, 36617, -1, 36618, 36619, 36620, -1, 36621, 36622, 36623, -1, 36624, 36625, 36626, -1, 36627, 36628, 36629, -1, 36630, 36631, 36632, -1, 36633, 36634, 36635, -1, 36636, 36637, 36638, -1, 36639, 36640, 36641, -1, 36642, 36643, 36644, -1, 36645, 36646, 36647, -1, 36648, 36649, 36650, -1, 36651, 36652, 36653, -1, 36654, 36655, 36656, -1, 36657, 36658, 36659, -1, 36660, 36661, 36662, -1, 36663, 36664, 36665, -1, 36666, 36667, 36668, -1, 36669, 36670, 36671, -1, 36672, 36673, 36674, -1, 36675, 36676, 36677, -1, 36678, 36679, 36680, -1, 36681, 36682, 36683, -1, 36684, 36685, 36686, -1, 36687, 36688, 36689, -1, 36690, 36691, 36692, -1, 36693, 36694, 36695, -1, 36696, 36697, 36698, -1, 36699, 36700, 36701, -1, 36702, 36703, 36704, -1, 36705, 36706, 36707, -1, 36708, 36709, 36710, -1, 36711, 36712, 36713, -1, 36714, 36715, 36716, -1, 36717, 36718, 36719, -1, 36720, 36721, 36722, -1, 36723, 36724, 36725, -1, 36726, 36727, 36728, -1, 36729, 36730, 36731, -1, 36732, 36733, 36734, -1, 36735, 36736, 36737, -1, 36738, 36739, 36740, -1, 36741, 36742, 36743, -1, 36744, 36745, 36746, -1, 36747, 36748, 36749, -1, 36750, 36751, 36752, -1, 36753, 36754, 36755, -1, 36756, 36757, 36758, -1, 36759, 36760, 36761, -1, 36762, 36763, 36764, -1, 36765, 36766, 36767, -1, 36768, 36769, 36770, -1, 36771, 36772, 36773, -1, 36774, 36775, 36776, -1, 36777, 36778, 36779, -1, 36780, 36781, 36782, -1, 36783, 36784, 36785, -1 };
    Text1m_5.normalIndex = m_normalIndex;
    Text1m_5.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    Text1m_5.normalPerVertex = true;
    Text1m_5.solid = false;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1, 5994, 5995, 5996, -1, 5997, 5998, 5999, -1, 6000, 6001, 6002, -1, 6003, 6004, 6005, -1, 6006, 6007, 6008, -1, 6009, 6010, 6011, -1, 6012, 6013, 6014, -1, 6015, 6016, 6017, -1, 6018, 6019, 6020, -1, 6021, 6022, 6023, -1, 6024, 6025, 6026, -1, 6027, 6028, 6029, -1, 6030, 6031, 6032, -1, 6033, 6034, 6035, -1, 6036, 6037, 6038, -1, 6039, 6040, 6041, -1, 6042, 6043, 6044, -1, 6045, 6046, 6047, -1, 6048, 6049, 6050, -1, 6051, 6052, 6053, -1, 6054, 6055, 6056, -1, 6057, 6058, 6059, -1, 6060, 6061, 6062, -1, 6063, 6064, 6065, -1, 6066, 6067, 6068, -1, 6069, 6070, 6071, -1, 6072, 6073, 6074, -1, 6075, 6076, 6077, -1, 6078, 6079, 6080, -1, 6081, 6082, 6083, -1, 6084, 6085, 6086, -1, 6087, 6088, 6089, -1, 6090, 6091, 6092, -1, 6093, 6094, 6095, -1, 6096, 6097, 6098, -1, 6099, 6100, 6101, -1, 6102, 6103, 6104, -1, 6105, 6106, 6107, -1, 6108, 6109, 6110, -1, 6111, 6112, 6113, -1, 6114, 6115, 6116, -1, 6117, 6118, 6119, -1, 6120, 6121, 6122, -1, 6123, 6124, 6125, -1, 6126, 6127, 6128, -1, 6129, 6130, 6131, -1, 6132, 6133, 6134, -1, 6135, 6136, 6137, -1, 6138, 6139, 6140, -1, 6141, 6142, 6143, -1, 6144, 6145, 6146, -1, 6147, 6148, 6149, -1, 6150, 6151, 6152, -1, 6153, 6154, 6155, -1, 6156, 6157, 6158, -1, 6159, 6160, 6161, -1, 6162, 6163, 6164, -1, 6165, 6166, 6167, -1, 6168, 6169, 6170, -1, 6171, 6172, 6173, -1, 6174, 6175, 6176, -1, 6177, 6178, 6179, -1, 6180, 6181, 6182, -1, 6183, 6184, 6185, -1, 6186, 6187, 6188, -1, 6189, 6190, 6191, -1, 6192, 6193, 6194, -1, 6195, 6196, 6197, -1, 6198, 6199, 6200, -1, 6201, 6202, 6203, -1, 6204, 6205, 6206, -1, 6207, 6208, 6209, -1, 6210, 6211, 6212, -1, 6213, 6214, 6215, -1, 6216, 6217, 6218, -1, 6219, 6220, 6221, -1, 6222, 6223, 6224, -1, 6225, 6226, 6227, -1, 6228, 6229, 6230, -1, 6231, 6232, 6233, -1, 6234, 6235, 6236, -1, 6237, 6238, 6239, -1, 6240, 6241, 6242, -1, 6243, 6244, 6245, -1, 6246, 6247, 6248, -1, 6249, 6250, 6251, -1, 6252, 6253, 6254, -1, 6255, 6256, 6257, -1, 6258, 6259, 6260, -1, 6261, 6262, 6263, -1, 6264, 6265, 6266, -1, 6267, 6268, 6269, -1, 6270, 6271, 6272, -1, 6273, 6274, 6275, -1, 6276, 6277, 6278, -1, 6279, 6280, 6281, -1, 6282, 6283, 6284, -1, 6285, 6286, 6287, -1, 6288, 6289, 6290, -1, 6291, 6292, 6293, -1, 6294, 6295, 6296, -1, 6297, 6298, 6299, -1, 6300, 6301, 6302, -1, 6303, 6304, 6305, -1, 6306, 6307, 6308, -1, 6309, 6310, 6311, -1, 6312, 6313, 6314, -1, 6315, 6316, 6317, -1, 6318, 6319, 6320, -1, 6321, 6322, 6323, -1, 6324, 6325, 6326, -1, 6327, 6328, 6329, -1, 6330, 6331, 6332, -1, 6333, 6334, 6335, -1, 6336, 6337, 6338, -1, 6339, 6340, 6341, -1, 6342, 6343, 6344, -1, 6345, 6346, 6347, -1, 6348, 6349, 6350, -1, 6351, 6352, 6353, -1, 6354, 6355, 6356, -1, 6357, 6358, 6359, -1, 6360, 6361, 6362, -1, 6363, 6364, 6365, -1, 6366, 6367, 6368, -1, 6369, 6370, 6371, -1, 6372, 6373, 6374, -1, 6375, 6376, 6377, -1, 6378, 6379, 6380, -1, 6381, 6382, 6383, -1, 6384, 6385, 6386, -1, 6387, 6388, 6389, -1, 6390, 6391, 6392, -1, 6393, 6394, 6395, -1, 6396, 6397, 6398, -1, 6399, 6400, 6401, -1, 6402, 6403, 6404, -1, 6405, 6406, 6407, -1, 6408, 6409, 6410, -1, 6411, 6412, 6413, -1, 6414, 6415, 6416, -1, 6417, 6418, 6419, -1, 6420, 6421, 6422, -1, 6423, 6424, 6425, -1, 6426, 6427, 6428, -1, 6429, 6430, 6431, -1, 6432, 6433, 6434, -1, 6435, 6436, 6437, -1, 6438, 6439, 6440, -1, 6441, 6442, 6443, -1, 6444, 6445, 6446, -1, 6447, 6448, 6449, -1, 6450, 6451, 6452, -1, 6453, 6454, 6455, -1, 6456, 6457, 6458, -1, 6459, 6460, 6461, -1, 6462, 6463, 6464, -1, 6465, 6466, 6467, -1, 6468, 6469, 6470, -1, 6471, 6472, 6473, -1, 6474, 6475, 6476, -1, 6477, 6478, 6479, -1, 6480, 6481, 6482, -1, 6483, 6484, 6485, -1, 6486, 6487, 6488, -1, 6489, 6490, 6491, -1, 6492, 6493, 6494, -1, 6495, 6496, 6497, -1, 6498, 6499, 6500, -1, 6501, 6502, 6503, -1, 6504, 6505, 6506, -1, 6507, 6508, 6509, -1, 6510, 6511, 6512, -1, 6513, 6514, 6515, -1, 6516, 6517, 6518, -1, 6519, 6520, 6521, -1, 6522, 6523, 6524, -1, 6525, 6526, 6527, -1, 6528, 6529, 6530, -1, 6531, 6532, 6533, -1, 6534, 6535, 6536, -1, 6537, 6538, 6539, -1, 6540, 6541, 6542, -1, 6543, 6544, 6545, -1, 6546, 6547, 6548, -1, 6549, 6550, 6551, -1, 6552, 6553, 6554, -1, 6555, 6556, 6557, -1, 6558, 6559, 6560, -1, 6561, 6562, 6563, -1, 6564, 6565, 6566, -1, 6567, 6568, 6569, -1, 6570, 6571, 6572, -1, 6573, 6574, 6575, -1, 6576, 6577, 6578, -1, 6579, 6580, 6581, -1, 6582, 6583, 6584, -1, 6585, 6586, 6587, -1, 6588, 6589, 6590, -1, 6591, 6592, 6593, -1, 6594, 6595, 6596, -1, 6597, 6598, 6599, -1, 6600, 6601, 6602, -1, 6603, 6604, 6605, -1, 6606, 6607, 6608, -1, 6609, 6610, 6611, -1, 6612, 6613, 6614, -1, 6615, 6616, 6617, -1, 6618, 6619, 6620, -1, 6621, 6622, 6623, -1, 6624, 6625, 6626, -1, 6627, 6628, 6629, -1, 6630, 6631, 6632, -1, 6633, 6634, 6635, -1, 6636, 6637, 6638, -1, 6639, 6640, 6641, -1, 6642, 6643, 6644, -1, 6645, 6646, 6647, -1, 6648, 6649, 6650, -1, 6651, 6652, 6653, -1, 6654, 6655, 6656, -1, 6657, 6658, 6659, -1, 6660, 6661, 6662, -1, 6663, 6664, 6665, -1, 6666, 6667, 6668, -1, 6669, 6670, 6671, -1, 6672, 6673, 6674, -1, 6675, 6676, 6677, -1, 6678, 6679, 6680, -1, 6681, 6682, 6683, -1, 6684, 6685, 6686, -1, 6687, 6688, 6689, -1, 6690, 6691, 6692, -1, 6693, 6694, 6695, -1, 6696, 6697, 6698, -1, 6699, 6700, 6701, -1, 6702, 6703, 6704, -1, 6705, 6706, 6707, -1, 6708, 6709, 6710, -1, 6711, 6712, 6713, -1, 6714, 6715, 6716, -1, 6717, 6718, 6719, -1, 6720, 6721, 6722, -1, 6723, 6724, 6725, -1, 6726, 6727, 6728, -1, 6729, 6730, 6731, -1, 6732, 6733, 6734, -1, 6735, 6736, 6737, -1, 6738, 6739, 6740, -1, 6741, 6742, 6743, -1, 6744, 6745, 6746, -1, 6747, 6748, 6749, -1, 6750, 6751, 6752, -1, 6753, 6754, 6755, -1, 6756, 6757, 6758, -1, 6759, 6760, 6761, -1, 6762, 6763, 6764, -1, 6765, 6766, 6767, -1, 6768, 6769, 6770, -1, 6771, 6772, 6773, -1, 6774, 6775, 6776, -1, 6777, 6778, 6779, -1, 6780, 6781, 6782, -1, 6783, 6784, 6785, -1, 6786, 6787, 6788, -1, 6789, 6790, 6791, -1, 6792, 6793, 6794, -1, 6795, 6796, 6797, -1, 6798, 6799, 6800, -1, 6801, 6802, 6803, -1, 6804, 6805, 6806, -1, 6807, 6808, 6809, -1, 6810, 6811, 6812, -1, 6813, 6814, 6815, -1, 6816, 6817, 6818, -1, 6819, 6820, 6821, -1, 6822, 6823, 6824, -1, 6825, 6826, 6827, -1, 6828, 6829, 6830, -1, 6831, 6832, 6833, -1, 6834, 6835, 6836, -1, 6837, 6838, 6839, -1, 6840, 6841, 6842, -1, 6843, 6844, 6845, -1, 6846, 6847, 6848, -1, 6849, 6850, 6851, -1, 6852, 6853, 6854, -1, 6855, 6856, 6857, -1, 6858, 6859, 6860, -1, 6861, 6862, 6863, -1, 6864, 6865, 6866, -1, 6867, 6868, 6869, -1, 6870, 6871, 6872, -1, 6873, 6874, 6875, -1, 6876, 6877, 6878, -1, 6879, 6880, 6881, -1, 6882, 6883, 6884, -1, 6885, 6886, 6887, -1, 6888, 6889, 6890, -1, 6891, 6892, 6893, -1, 6894, 6895, 6896, -1, 6897, 6898, 6899, -1, 6900, 6901, 6902, -1, 6903, 6904, 6905, -1, 6906, 6907, 6908, -1, 6909, 6910, 6911, -1, 6912, 6913, 6914, -1, 6915, 6916, 6917, -1, 6918, 6919, 6920, -1, 6921, 6922, 6923, -1, 6924, 6925, 6926, -1, 6927, 6928, 6929, -1, 6930, 6931, 6932, -1, 6933, 6934, 6935, -1, 6936, 6937, 6938, -1, 6939, 6940, 6941, -1, 6942, 6943, 6944, -1, 6945, 6946, 6947, -1, 6948, 6949, 6950, -1, 6951, 6952, 6953, -1, 6954, 6955, 6956, -1, 6957, 6958, 6959, -1, 6960, 6961, 6962, -1, 6963, 6964, 6965, -1, 6966, 6967, 6968, -1, 6969, 6970, 6971, -1, 6972, 6973, 6974, -1, 6975, 6976, 6977, -1, 6978, 6979, 6980, -1, 6981, 6982, 6983, -1, 6984, 6985, 6986, -1, 6987, 6988, 6989, -1, 6990, 6991, 6992, -1, 6993, 6994, 6995, -1, 6996, 6997, 6998, -1, 6999, 7000, 7001, -1, 7002, 7003, 7004, -1, 7005, 7006, 7007, -1, 7008, 7009, 7010, -1, 7011, 7012, 7013, -1, 7014, 7015, 7016, -1, 7017, 7018, 7019, -1, 7020, 7021, 7022, -1, 7023, 7024, 7025, -1, 7026, 7027, 7028, -1, 7029, 7030, 7031, -1, 7032, 7033, 7034, -1, 7035, 7036, 7037, -1, 7038, 7039, 7040, -1, 7041, 7042, 7043, -1, 7044, 7045, 7046, -1, 7047, 7048, 7049, -1, 7050, 7051, 7052, -1, 7053, 7054, 7055, -1, 7056, 7057, 7058, -1, 7059, 7060, 7061, -1, 7062, 7063, 7064, -1, 7065, 7066, 7067, -1, 7068, 7069, 7070, -1, 7071, 7072, 7073, -1, 7074, 7075, 7076, -1, 7077, 7078, 7079, -1, 7080, 7081, 7082, -1, 7083, 7084, 7085, -1, 7086, 7087, 7088, -1, 7089, 7090, 7091, -1, 7092, 7093, 7094, -1, 7095, 7096, 7097, -1, 7098, 7099, 7100, -1, 7101, 7102, 7103, -1, 7104, 7105, 7106, -1, 7107, 7108, 7109, -1, 7110, 7111, 7112, -1, 7113, 7114, 7115, -1, 7116, 7117, 7118, -1, 7119, 7120, 7121, -1, 7122, 7123, 7124, -1, 7125, 7126, 7127, -1, 7128, 7129, 7130, -1, 7131, 7132, 7133, -1, 7134, 7135, 7136, -1, 7137, 7138, 7139, -1, 7140, 7141, 7142, -1, 7143, 7144, 7145, -1, 7146, 7147, 7148, -1, 7149, 7150, 7151, -1, 7152, 7153, 7154, -1, 7155, 7156, 7157, -1, 7158, 7159, 7160, -1, 7161, 7162, 7163, -1, 7164, 7165, 7166, -1, 7167, 7168, 7169, -1, 7170, 7171, 7172, -1, 7173, 7174, 7175, -1, 7176, 7177, 7178, -1, 7179, 7180, 7181, -1, 7182, 7183, 7184, -1, 7185, 7186, 7187, -1, 7188, 7189, 7190, -1, 7191, 7192, 7193, -1, 7194, 7195, 7196, -1, 7197, 7198, 7199, -1, 7200, 7201, 7202, -1, 7203, 7204, 7205, -1, 7206, 7207, 7208, -1, 7209, 7210, 7211, -1, 7212, 7213, 7214, -1, 7215, 7216, 7217, -1, 7218, 7219, 7220, -1, 7221, 7222, 7223, -1, 7224, 7225, 7226, -1, 7227, 7228, 7229, -1, 7230, 7231, 7232, -1, 7233, 7234, 7235, -1, 7236, 7237, 7238, -1, 7239, 7240, 7241, -1, 7242, 7243, 7244, -1, 7245, 7246, 7247, -1, 7248, 7249, 7250, -1, 7251, 7252, 7253, -1, 7254, 7255, 7256, -1, 7257, 7258, 7259, -1, 7260, 7261, 7262, -1, 7263, 7264, 7265, -1, 7266, 7267, 7268, -1, 7269, 7270, 7271, -1, 7272, 7273, 7274, -1, 7275, 7276, 7277, -1, 7278, 7279, 7280, -1, 7281, 7282, 7283, -1, 7284, 7285, 7286, -1, 7287, 7288, 7289, -1, 7290, 7291, 7292, -1, 7293, 7294, 7295, -1, 7296, 7297, 7298, -1, 7299, 7300, 7301, -1, 7302, 7303, 7304, -1, 7305, 7306, 7307, -1, 7308, 7309, 7310, -1, 7311, 7312, 7313, -1, 7314, 7315, 7316, -1, 7317, 7318, 7319, -1, 7320, 7321, 7322, -1, 7323, 7324, 7325, -1, 7326, 7327, 7328, -1, 7329, 7330, 7331, -1, 7332, 7333, 7334, -1, 7335, 7336, 7337, -1, 7338, 7339, 7340, -1, 7341, 7342, 7343, -1, 7344, 7345, 7346, -1, 7347, 7348, 7349, -1, 7350, 7351, 7352, -1, 7353, 7354, 7355, -1, 7356, 7357, 7358, -1, 7359, 7360, 7361, -1, 7362, 7363, 7364, -1, 7365, 7366, 7367, -1, 7368, 7369, 7370, -1, 7371, 7372, 7373, -1, 7374, 7375, 7376, -1, 7377, 7378, 7379, -1, 7380, 7381, 7382, -1, 7383, 7384, 7385, -1, 7386, 7387, 7388, -1, 7389, 7390, 7391, -1, 7392, 7393, 7394, -1, 7395, 7396, 7397, -1, 7398, 7399, 7400, -1, 7401, 7402, 7403, -1, 7404, 7405, 7406, -1, 7407, 7408, 7409, -1, 7410, 7411, 7412, -1, 7413, 7414, 7415, -1, 7416, 7417, 7418, -1, 7419, 7420, 7421, -1, 7422, 7423, 7424, -1, 7425, 7426, 7427, -1, 7428, 7429, 7430, -1, 7431, 7432, 7433, -1, 7434, 7435, 7436, -1, 7437, 7438, 7439, -1, 7440, 7441, 7442, -1, 7443, 7444, 7445, -1, 7446, 7447, 7448, -1, 7449, 7450, 7451, -1, 7452, 7453, 7454, -1, 7455, 7456, 7457, -1, 7458, 7459, 7460, -1, 7461, 7462, 7463, -1, 7464, 7465, 7466, -1, 7467, 7468, 7469, -1, 7470, 7471, 7472, -1, 7473, 7474, 7475, -1, 7476, 7477, 7478, -1, 7479, 7480, 7481, -1, 7482, 7483, 7484, -1, 7485, 7486, 7487, -1, 7488, 7489, 7490, -1, 7491, 7492, 7493, -1, 7494, 7495, 7496, -1, 7497, 7498, 7499, -1, 7500, 7501, 7502, -1, 7503, 7504, 7505, -1, 7506, 7507, 7508, -1, 7509, 7510, 7511, -1, 7512, 7513, 7514, -1, 7515, 7516, 7517, -1, 7518, 7519, 7520, -1, 7521, 7522, 7523, -1, 7524, 7525, 7526, -1, 7527, 7528, 7529, -1, 7530, 7531, 7532, -1, 7533, 7534, 7535, -1, 7536, 7537, 7538, -1, 7539, 7540, 7541, -1, 7542, 7543, 7544, -1, 7545, 7546, 7547, -1, 7548, 7549, 7550, -1, 7551, 7552, 7553, -1, 7554, 7555, 7556, -1, 7557, 7558, 7559, -1, 7560, 7561, 7562, -1, 7563, 7564, 7565, -1, 7566, 7567, 7568, -1, 7569, 7570, 7571, -1, 7572, 7573, 7574, -1, 7575, 7576, 7577, -1, 7578, 7579, 7580, -1, 7581, 7582, 7583, -1, 7584, 7585, 7586, -1, 7587, 7588, 7589, -1, 7590, 7591, 7592, -1, 7593, 7594, 7595, -1, 7596, 7597, 7598, -1, 7599, 7600, 7601, -1, 7602, 7603, 7604, -1, 7605, 7606, 7607, -1, 7608, 7609, 7610, -1, 7611, 7612, 7613, -1, 7614, 7615, 7616, -1, 7617, 7618, 7619, -1, 7620, 7621, 7622, -1, 7623, 7624, 7625, -1, 7626, 7627, 7628, -1, 7629, 7630, 7631, -1, 7632, 7633, 7634, -1, 7635, 7636, 7637, -1, 7638, 7639, 7640, -1, 7641, 7642, 7643, -1, 7644, 7645, 7646, -1, 7647, 7648, 7649, -1, 7650, 7651, 7652, -1, 7653, 7654, 7655, -1, 7656, 7657, 7658, -1, 7659, 7660, 7661, -1, 7662, 7663, 7664, -1, 7665, 7666, 7667, -1, 7668, 7669, 7670, -1, 7671, 7672, 7673, -1, 7674, 7675, 7676, -1, 7677, 7678, 7679, -1, 7680, 7681, 7682, -1, 7683, 7684, 7685, -1, 7686, 7687, 7688, -1, 7689, 7690, 7691, -1, 7692, 7693, 7694, -1, 7695, 7696, 7697, -1, 7698, 7699, 7700, -1, 7701, 7702, 7703, -1, 7704, 7705, 7706, -1, 7707, 7708, 7709, -1, 7710, 7711, 7712, -1, 7713, 7714, 7715, -1, 7716, 7717, 7718, -1, 7719, 7720, 7721, -1, 7722, 7723, 7724, -1, 7725, 7726, 7727, -1, 7728, 7729, 7730, -1, 7731, 7732, 7733, -1, 7734, 7735, 7736, -1, 7737, 7738, 7739, -1, 7740, 7741, 7742, -1, 7743, 7744, 7745, -1, 7746, 7747, 7748, -1, 7749, 7750, 7751, -1, 7752, 7753, 7754, -1, 7755, 7756, 7757, -1, 7758, 7759, 7760, -1, 7761, 7762, 7763, -1, 7764, 7765, 7766, -1, 7767, 7768, 7769, -1, 7770, 7771, 7772, -1, 7773, 7774, 7775, -1, 7776, 7777, 7778, -1, 7779, 7780, 7781, -1, 7782, 7783, 7784, -1, 7785, 7786, 7787, -1, 7788, 7789, 7790, -1, 7791, 7792, 7793, -1, 7794, 7795, 7796, -1, 7797, 7798, 7799, -1, 7800, 7801, 7802, -1, 7803, 7804, 7805, -1, 7806, 7807, 7808, -1, 7809, 7810, 7811, -1, 7812, 7813, 7814, -1, 7815, 7816, 7817, -1, 7818, 7819, 7820, -1, 7821, 7822, 7823, -1, 7824, 7825, 7826, -1, 7827, 7828, 7829, -1, 7830, 7831, 7832, -1, 7833, 7834, 7835, -1, 7836, 7837, 7838, -1, 7839, 7840, 7841, -1, 7842, 7843, 7844, -1, 7845, 7846, 7847, -1, 7848, 7849, 7850, -1, 7851, 7852, 7853, -1, 7854, 7855, 7856, -1, 7857, 7858, 7859, -1, 7860, 7861, 7862, -1, 7863, 7864, 7865, -1, 7866, 7867, 7868, -1, 7869, 7870, 7871, -1, 7872, 7873, 7874, -1, 7875, 7876, 7877, -1, 7878, 7879, 7880, -1, 7881, 7882, 7883, -1, 7884, 7885, 7886, -1, 7887, 7888, 7889, -1, 7890, 7891, 7892, -1, 7893, 7894, 7895, -1, 7896, 7897, 7898, -1, 7899, 7900, 7901, -1, 7902, 7903, 7904, -1, 7905, 7906, 7907, -1, 7908, 7909, 7910, -1, 7911, 7912, 7913, -1, 7914, 7915, 7916, -1, 7917, 7918, 7919, -1, 7920, 7921, 7922, -1, 7923, 7924, 7925, -1, 7926, 7927, 7928, -1, 7929, 7930, 7931, -1, 7932, 7933, 7934, -1, 7935, 7936, 7937, -1, 7938, 7939, 7940, -1, 7941, 7942, 7943, -1, 7944, 7945, 7946, -1, 7947, 7948, 7949, -1, 7950, 7951, 7952, -1, 7953, 7954, 7955, -1, 7956, 7957, 7958, -1, 7959, 7960, 7961, -1, 7962, 7963, 7964, -1, 7965, 7966, 7967, -1, 7968, 7969, 7970, -1, 7971, 7972, 7973, -1, 7974, 7975, 7976, -1, 7977, 7978, 7979, -1, 7980, 7981, 7982, -1, 7983, 7984, 7985, -1, 7986, 7987, 7988, -1, 7989, 7990, 7991, -1, 7992, 7993, 7994, -1, 7995, 7996, 7997, -1, 7998, 7999, 8000, -1, 8001, 8002, 8003, -1, 8004, 8005, 8006, -1, 8007, 8008, 8009, -1, 8010, 8011, 8012, -1, 8013, 8014, 8015, -1, 8016, 8017, 8018, -1, 8019, 8020, 8021, -1, 8022, 8023, 8024, -1, 8025, 8026, 8027, -1, 8028, 8029, 8030, -1, 8031, 8032, 8033, -1, 8034, 8035, 8036, -1, 8037, 8038, 8039, -1, 8040, 8041, 8042, -1, 8043, 8044, 8045, -1, 8046, 8047, 8048, -1, 8049, 8050, 8051, -1, 8052, 8053, 8054, -1, 8055, 8056, 8057, -1, 8058, 8059, 8060, -1, 8061, 8062, 8063, -1, 8064, 8065, 8066, -1, 8067, 8068, 8069, -1, 8070, 8071, 8072, -1, 8073, 8074, 8075, -1, 8076, 8077, 8078, -1, 8079, 8080, 8081, -1, 8082, 8083, 8084, -1, 8085, 8086, 8087, -1, 8088, 8089, 8090, -1, 8091, 8092, 8093, -1, 8094, 8095, 8096, -1, 8097, 8098, 8099, -1, 8100, 8101, 8102, -1, 8103, 8104, 8105, -1, 8106, 8107, 8108, -1, 8109, 8110, 8111, -1, 8112, 8113, 8114, -1, 8115, 8116, 8117, -1, 8118, 8119, 8120, -1, 8121, 8122, 8123, -1, 8124, 8125, 8126, -1, 8127, 8128, 8129, -1, 8130, 8131, 8132, -1, 8133, 8134, 8135, -1, 8136, 8137, 8138, -1, 8139, 8140, 8141, -1, 8142, 8143, 8144, -1, 8145, 8146, 8147, -1, 8148, 8149, 8150, -1, 8151, 8152, 8153, -1, 8154, 8155, 8156, -1, 8157, 8158, 8159, -1, 8160, 8161, 8162, -1, 8163, 8164, 8165, -1, 8166, 8167, 8168, -1, 8169, 8170, 8171, -1, 8172, 8173, 8174, -1, 8175, 8176, 8177, -1, 8178, 8179, 8180, -1, 8181, 8182, 8183, -1, 8184, 8185, 8186, -1, 8187, 8188, 8189, -1, 8190, 8191, 8192, -1, 8193, 8194, 8195, -1, 8196, 8197, 8198, -1, 8199, 8200, 8201, -1, 8202, 8203, 8204, -1, 8205, 8206, 8207, -1, 8208, 8209, 8210, -1, 8211, 8212, 8213, -1, 8214, 8215, 8216, -1, 8217, 8218, 8219, -1, 8220, 8221, 8222, -1, 8223, 8224, 8225, -1, 8226, 8227, 8228, -1, 8229, 8230, 8231, -1, 8232, 8233, 8234, -1, 8235, 8236, 8237, -1, 8238, 8239, 8240, -1, 8241, 8242, 8243, -1, 8244, 8245, 8246, -1, 8247, 8248, 8249, -1, 8250, 8251, 8252, -1, 8253, 8254, 8255, -1, 8256, 8257, 8258, -1, 8259, 8260, 8261, -1, 8262, 8263, 8264, -1, 8265, 8266, 8267, -1, 8268, 8269, 8270, -1, 8271, 8272, 8273, -1, 8274, 8275, 8276, -1, 8277, 8278, 8279, -1, 8280, 8281, 8282, -1, 8283, 8284, 8285, -1, 8286, 8287, 8288, -1, 8289, 8290, 8291, -1, 8292, 8293, 8294, -1, 8295, 8296, 8297, -1, 8298, 8299, 8300, -1, 8301, 8302, 8303, -1, 8304, 8305, 8306, -1, 8307, 8308, 8309, -1, 8310, 8311, 8312, -1, 8313, 8314, 8315, -1, 8316, 8317, 8318, -1, 8319, 8320, 8321, -1, 8322, 8323, 8324, -1, 8325, 8326, 8327, -1, 8328, 8329, 8330, -1, 8331, 8332, 8333, -1, 8334, 8335, 8336, -1, 8337, 8338, 8339, -1, 8340, 8341, 8342, -1, 8343, 8344, 8345, -1, 8346, 8347, 8348, -1, 8349, 8350, 8351, -1, 8352, 8353, 8354, -1, 8355, 8356, 8357, -1, 8358, 8359, 8360, -1, 8361, 8362, 8363, -1, 8364, 8365, 8366, -1, 8367, 8368, 8369, -1, 8370, 8371, 8372, -1, 8373, 8374, 8375, -1, 8376, 8377, 8378, -1, 8379, 8380, 8381, -1, 8382, 8383, 8384, -1, 8385, 8386, 8387, -1, 8388, 8389, 8390, -1, 8391, 8392, 8393, -1, 8394, 8395, 8396, -1, 8397, 8398, 8399, -1, 8400, 8401, 8402, -1, 8403, 8404, 8405, -1, 8406, 8407, 8408, -1, 8409, 8410, 8411, -1, 8412, 8413, 8414, -1, 8415, 8416, 8417, -1, 8418, 8419, 8420, -1, 8421, 8422, 8423, -1, 8424, 8425, 8426, -1, 8427, 8428, 8429, -1, 8430, 8431, 8432, -1, 8433, 8434, 8435, -1, 8436, 8437, 8438, -1, 8439, 8440, 8441, -1, 8442, 8443, 8444, -1, 8445, 8446, 8447, -1, 8448, 8449, 8450, -1, 8451, 8452, 8453, -1, 8454, 8455, 8456, -1, 8457, 8458, 8459, -1, 8460, 8461, 8462, -1, 8463, 8464, 8465, -1, 8466, 8467, 8468, -1, 8469, 8470, 8471, -1, 8472, 8473, 8474, -1, 8475, 8476, 8477, -1, 8478, 8479, 8480, -1, 8481, 8482, 8483, -1, 8484, 8485, 8486, -1, 8487, 8488, 8489, -1, 8490, 8491, 8492, -1, 8493, 8494, 8495, -1, 8496, 8497, 8498, -1, 8499, 8500, 8501, -1, 8502, 8503, 8504, -1, 8505, 8506, 8507, -1, 8508, 8509, 8510, -1, 8511, 8512, 8513, -1, 8514, 8515, 8516, -1, 8517, 8518, 8519, -1, 8520, 8521, 8522, -1, 8523, 8524, 8525, -1, 8526, 8527, 8528, -1, 8529, 8530, 8531, -1, 8532, 8533, 8534, -1, 8535, 8536, 8537, -1, 8538, 8539, 8540, -1, 8541, 8542, 8543, -1, 8544, 8545, 8546, -1, 8547, 8548, 8549, -1, 8550, 8551, 8552, -1, 8553, 8554, 8555, -1, 8556, 8557, 8558, -1, 8559, 8560, 8561, -1, 8562, 8563, 8564, -1, 8565, 8566, 8567, -1, 8568, 8569, 8570, -1, 8571, 8572, 8573, -1, 8574, 8575, 8576, -1, 8577, 8578, 8579, -1, 8580, 8581, 8582, -1, 8583, 8584, 8585, -1, 8586, 8587, 8588, -1, 8589, 8590, 8591, -1, 8592, 8593, 8594, -1, 8595, 8596, 8597, -1, 8598, 8599, 8600, -1, 8601, 8602, 8603, -1, 8604, 8605, 8606, -1, 8607, 8608, 8609, -1, 8610, 8611, 8612, -1, 8613, 8614, 8615, -1, 8616, 8617, 8618, -1, 8619, 8620, 8621, -1, 8622, 8623, 8624, -1, 8625, 8626, 8627, -1, 8628, 8629, 8630, -1, 8631, 8632, 8633, -1, 8634, 8635, 8636, -1, 8637, 8638, 8639, -1, 8640, 8641, 8642, -1, 8643, 8644, 8645, -1, 8646, 8647, 8648, -1, 8649, 8650, 8651, -1, 8652, 8653, 8654, -1, 8655, 8656, 8657, -1, 8658, 8659, 8660, -1, 8661, 8662, 8663, -1, 8664, 8665, 8666, -1, 8667, 8668, 8669, -1, 8670, 8671, 8672, -1, 8673, 8674, 8675, -1, 8676, 8677, 8678, -1, 8679, 8680, 8681, -1, 8682, 8683, 8684, -1, 8685, 8686, 8687, -1, 8688, 8689, 8690, -1, 8691, 8692, 8693, -1, 8694, 8695, 8696, -1, 8697, 8698, 8699, -1, 8700, 8701, 8702, -1, 8703, 8704, 8705, -1, 8706, 8707, 8708, -1, 8709, 8710, 8711, -1, 8712, 8713, 8714, -1, 8715, 8716, 8717, -1, 8718, 8719, 8720, -1, 8721, 8722, 8723, -1, 8724, 8725, 8726, -1, 8727, 8728, 8729, -1, 8730, 8731, 8732, -1, 8733, 8734, 8735, -1, 8736, 8737, 8738, -1, 8739, 8740, 8741, -1, 8742, 8743, 8744, -1, 8745, 8746, 8747, -1, 8748, 8749, 8750, -1, 8751, 8752, 8753, -1, 8754, 8755, 8756, -1, 8757, 8758, 8759, -1, 8760, 8761, 8762, -1, 8763, 8764, 8765, -1, 8766, 8767, 8768, -1, 8769, 8770, 8771, -1, 8772, 8773, 8774, -1, 8775, 8776, 8777, -1, 8778, 8779, 8780, -1, 8781, 8782, 8783, -1, 8784, 8785, 8786, -1, 8787, 8788, 8789, -1, 8790, 8791, 8792, -1, 8793, 8794, 8795, -1, 8796, 8797, 8798, -1, 8799, 8800, 8801, -1, 8802, 8803, 8804, -1, 8805, 8806, 8807, -1, 8808, 8809, 8810, -1, 8811, 8812, 8813, -1, 8814, 8815, 8816, -1, 8817, 8818, 8819, -1, 8820, 8821, 8822, -1, 8823, 8824, 8825, -1, 8826, 8827, 8828, -1, 8829, 8830, 8831, -1, 8832, 8833, 8834, -1, 8835, 8836, 8837, -1, 8838, 8839, 8840, -1, 8841, 8842, 8843, -1, 8844, 8845, 8846, -1, 8847, 8848, 8849, -1, 8850, 8851, 8852, -1, 8853, 8854, 8855, -1, 8856, 8857, 8858, -1, 8859, 8860, 8861, -1, 8862, 8863, 8864, -1, 8865, 8866, 8867, -1, 8868, 8869, 8870, -1, 8871, 8872, 8873, -1, 8874, 8875, 8876, -1, 8877, 8878, 8879, -1, 8880, 8881, 8882, -1, 8883, 8884, 8885, -1, 8886, 8887, 8888, -1, 8889, 8890, 8891, -1, 8892, 8893, 8894, -1, 8895, 8896, 8897, -1, 8898, 8899, 8900, -1, 8901, 8902, 8903, -1, 8904, 8905, 8906, -1, 8907, 8908, 8909, -1, 8910, 8911, 8912, -1, 8913, 8914, 8915, -1, 8916, 8917, 8918, -1, 8919, 8920, 8921, -1, 8922, 8923, 8924, -1, 8925, 8926, 8927, -1, 8928, 8929, 8930, -1, 8931, 8932, 8933, -1, 8934, 8935, 8936, -1, 8937, 8938, 8939, -1, 8940, 8941, 8942, -1, 8943, 8944, 8945, -1, 8946, 8947, 8948, -1, 8949, 8950, 8951, -1, 8952, 8953, 8954, -1, 8955, 8956, 8957, -1, 8958, 8959, 8960, -1, 8961, 8962, 8963, -1, 8964, 8965, 8966, -1, 8967, 8968, 8969, -1, 8970, 8971, 8972, -1, 8973, 8974, 8975, -1, 8976, 8977, 8978, -1, 8979, 8980, 8981, -1, 8982, 8983, 8984, -1, 8985, 8986, 8987, -1, 8988, 8989, 8990, -1, 8991, 8992, 8993, -1, 8994, 8995, 8996, -1, 8997, 8998, 8999, -1, 9000, 9001, 9002, -1, 9003, 9004, 9005, -1, 9006, 9007, 9008, -1, 9009, 9010, 9011, -1, 9012, 9013, 9014, -1, 9015, 9016, 9017, -1, 9018, 9019, 9020, -1, 9021, 9022, 9023, -1, 9024, 9025, 9026, -1, 9027, 9028, 9029, -1, 9030, 9031, 9032, -1, 9033, 9034, 9035, -1, 9036, 9037, 9038, -1, 9039, 9040, 9041, -1, 9042, 9043, 9044, -1, 9045, 9046, 9047, -1, 9048, 9049, 9050, -1, 9051, 9052, 9053, -1, 9054, 9055, 9056, -1, 9057, 9058, 9059, -1, 9060, 9061, 9062, -1, 9063, 9064, 9065, -1, 9066, 9067, 9068, -1, 9069, 9070, 9071, -1, 9072, 9073, 9074, -1, 9075, 9076, 9077, -1, 9078, 9079, 9080, -1, 9081, 9082, 9083, -1, 9084, 9085, 9086, -1, 9087, 9088, 9089, -1, 9090, 9091, 9092, -1, 9093, 9094, 9095, -1, 9096, 9097, 9098, -1, 9099, 9100, 9101, -1, 9102, 9103, 9104, -1, 9105, 9106, 9107, -1, 9108, 9109, 9110, -1, 9111, 9112, 9113, -1, 9114, 9115, 9116, -1, 9117, 9118, 9119, -1, 9120, 9121, 9122, -1, 9123, 9124, 9125, -1, 9126, 9127, 9128, -1, 9129, 9130, 9131, -1, 9132, 9133, 9134, -1, 9135, 9136, 9137, -1, 9138, 9139, 9140, -1, 9141, 9142, 9143, -1, 9144, 9145, 9146, -1, 9147, 9148, 9149, -1, 9150, 9151, 9152, -1, 9153, 9154, 9155, -1, 9156, 9157, 9158, -1, 9159, 9160, 9161, -1, 9162, 9163, 9164, -1, 9165, 9166, 9167, -1, 9168, 9169, 9170, -1, 9171, 9172, 9173, -1, 9174, 9175, 9176, -1, 9177, 9178, 9179, -1, 9180, 9181, 9182, -1, 9183, 9184, 9185, -1, 9186, 9187, 9188, -1, 9189, 9190, 9191, -1, 9192, 9193, 9194, -1, 9195, 9196, 9197, -1, 9198, 9199, 9200, -1, 9201, 9202, 9203, -1, 9204, 9205, 9206, -1, 9207, 9208, 9209, -1, 9210, 9211, 9212, -1, 9213, 9214, 9215, -1, 9216, 9217, 9218, -1, 9219, 9220, 9221, -1, 9222, 9223, 9224, -1, 9225, 9226, 9227, -1, 9228, 9229, 9230, -1, 9231, 9232, 9233, -1, 9234, 9235, 9236, -1, 9237, 9238, 9239, -1, 9240, 9241, 9242, -1, 9243, 9244, 9245, -1, 9246, 9247, 9248, -1, 9249, 9250, 9251, -1, 9252, 9253, 9254, -1, 9255, 9256, 9257, -1, 9258, 9259, 9260, -1, 9261, 9262, 9263, -1, 9264, 9265, 9266, -1, 9267, 9268, 9269, -1, 9270, 9271, 9272, -1, 9273, 9274, 9275, -1, 9276, 9277, 9278, -1, 9279, 9280, 9281, -1, 9282, 9283, 9284, -1, 9285, 9286, 9287, -1, 9288, 9289, 9290, -1, 9291, 9292, 9293, -1, 9294, 9295, 9296, -1, 9297, 9298, 9299, -1, 9300, 9301, 9302, -1, 9303, 9304, 9305, -1, 9306, 9307, 9308, -1, 9309, 9310, 9311, -1, 9312, 9313, 9314, -1, 9315, 9316, 9317, -1, 9318, 9319, 9320, -1, 9321, 9322, 9323, -1, 9324, 9325, 9326, -1, 9327, 9328, 9329, -1, 9330, 9331, 9332, -1, 9333, 9334, 9335, -1, 9336, 9337, 9338, -1, 9339, 9340, 9341, -1, 9342, 9343, 9344, -1, 9345, 9346, 9347, -1, 9348, 9349, 9350, -1, 9351, 9352, 9353, -1, 9354, 9355, 9356, -1, 9357, 9358, 9359, -1, 9360, 9361, 9362, -1, 9363, 9364, 9365, -1, 9366, 9367, 9368, -1, 9369, 9370, 9371, -1, 9372, 9373, 9374, -1, 9375, 9376, 9377, -1, 9378, 9379, 9380, -1, 9381, 9382, 9383, -1, 9384, 9385, 9386, -1, 9387, 9388, 9389, -1, 9390, 9391, 9392, -1, 9393, 9394, 9395, -1, 9396, 9397, 9398, -1, 9399, 9400, 9401, -1, 9402, 9403, 9404, -1, 9405, 9406, 9407, -1, 9408, 9409, 9410, -1, 9411, 9412, 9413, -1, 9414, 9415, 9416, -1, 9417, 9418, 9419, -1, 9420, 9421, 9422, -1, 9423, 9424, 9425, -1, 9426, 9427, 9428, -1, 9429, 9430, 9431, -1, 9432, 9433, 9434, -1, 9435, 9436, 9437, -1, 9438, 9439, 9440, -1, 9441, 9442, 9443, -1, 9444, 9445, 9446, -1, 9447, 9448, 9449, -1, 9450, 9451, 9452, -1, 9453, 9454, 9455, -1, 9456, 9457, 9458, -1, 9459, 9460, 9461, -1, 9462, 9463, 9464, -1, 9465, 9466, 9467, -1, 9468, 9469, 9470, -1, 9471, 9472, 9473, -1, 9474, 9475, 9476, -1, 9477, 9478, 9479, -1, 9480, 9481, 9482, -1, 9483, 9484, 9485, -1, 9486, 9487, 9488, -1, 9489, 9490, 9491, -1, 9492, 9493, 9494, -1, 9495, 9496, 9497, -1, 9498, 9499, 9500, -1, 9501, 9502, 9503, -1, 9504, 9505, 9506, -1, 9507, 9508, 9509, -1, 9510, 9511, 9512, -1, 9513, 9514, 9515, -1, 9516, 9517, 9518, -1, 9519, 9520, 9521, -1, 9522, 9523, 9524, -1, 9525, 9526, 9527, -1, 9528, 9529, 9530, -1, 9531, 9532, 9533, -1, 9534, 9535, 9536, -1, 9537, 9538, 9539, -1, 9540, 9541, 9542, -1, 9543, 9544, 9545, -1, 9546, 9547, 9548, -1, 9549, 9550, 9551, -1, 9552, 9553, 9554, -1, 9555, 9556, 9557, -1, 9558, 9559, 9560, -1, 9561, 9562, 9563, -1, 9564, 9565, 9566, -1, 9567, 9568, 9569, -1, 9570, 9571, 9572, -1, 9573, 9574, 9575, -1, 9576, 9577, 9578, -1, 9579, 9580, 9581, -1, 9582, 9583, 9584, -1, 9585, 9586, 9587, -1, 9588, 9589, 9590, -1, 9591, 9592, 9593, -1, 9594, 9595, 9596, -1, 9597, 9598, 9599, -1, 9600, 9601, 9602, -1, 9603, 9604, 9605, -1, 9606, 9607, 9608, -1, 9609, 9610, 9611, -1, 9612, 9613, 9614, -1, 9615, 9616, 9617, -1, 9618, 9619, 9620, -1, 9621, 9622, 9623, -1, 9624, 9625, 9626, -1, 9627, 9628, 9629, -1, 9630, 9631, 9632, -1, 9633, 9634, 9635, -1, 9636, 9637, 9638, -1, 9639, 9640, 9641, -1, 9642, 9643, 9644, -1, 9645, 9646, 9647, -1, 9648, 9649, 9650, -1, 9651, 9652, 9653, -1, 9654, 9655, 9656, -1, 9657, 9658, 9659, -1, 9660, 9661, 9662, -1, 9663, 9664, 9665, -1, 9666, 9667, 9668, -1, 9669, 9670, 9671, -1, 9672, 9673, 9674, -1, 9675, 9676, 9677, -1, 9678, 9679, 9680, -1, 9681, 9682, 9683, -1, 9684, 9685, 9686, -1, 9687, 9688, 9689, -1, 9690, 9691, 9692, -1, 9693, 9694, 9695, -1, 9696, 9697, 9698, -1, 9699, 9700, 9701, -1, 9702, 9703, 9704, -1, 9705, 9706, 9707, -1, 9708, 9709, 9710, -1, 9711, 9712, 9713, -1, 9714, 9715, 9716, -1, 9717, 9718, 9719, -1, 9720, 9721, 9722, -1, 9723, 9724, 9725, -1, 9726, 9727, 9728, -1, 9729, 9730, 9731, -1, 9732, 9733, 9734, -1, 9735, 9736, 9737, -1, 9738, 9739, 9740, -1, 9741, 9742, 9743, -1, 9744, 9745, 9746, -1, 9747, 9748, 9749, -1, 9750, 9751, 9752, -1, 9753, 9754, 9755, -1, 9756, 9757, 9758, -1, 9759, 9760, 9761, -1, 9762, 9763, 9764, -1, 9765, 9766, 9767, -1, 9768, 9769, 9770, -1, 9771, 9772, 9773, -1, 9774, 9775, 9776, -1, 9777, 9778, 9779, -1, 9780, 9781, 9782, -1, 9783, 9784, 9785, -1, 9786, 9787, 9788, -1, 9789, 9790, 9791, -1, 9792, 9793, 9794, -1, 9795, 9796, 9797, -1, 9798, 9799, 9800, -1, 9801, 9802, 9803, -1, 9804, 9805, 9806, -1, 9807, 9808, 9809, -1, 9810, 9811, 9812, -1, 9813, 9814, 9815, -1, 9816, 9817, 9818, -1, 9819, 9820, 9821, -1, 9822, 9823, 9824, -1, 9825, 9826, 9827, -1, 9828, 9829, 9830, -1, 9831, 9832, 9833, -1, 9834, 9835, 9836, -1, 9837, 9838, 9839, -1, 9840, 9841, 9842, -1, 9843, 9844, 9845, -1, 9846, 9847, 9848, -1, 9849, 9850, 9851, -1, 9852, 9853, 9854, -1, 9855, 9856, 9857, -1, 9858, 9859, 9860, -1, 9861, 9862, 9863, -1, 9864, 9865, 9866, -1, 9867, 9868, 9869, -1, 9870, 9871, 9872, -1, 9873, 9874, 9875, -1, 9876, 9877, 9878, -1, 9879, 9880, 9881, -1, 9882, 9883, 9884, -1, 9885, 9886, 9887, -1, 9888, 9889, 9890, -1, 9891, 9892, 9893, -1, 9894, 9895, 9896, -1, 9897, 9898, 9899, -1, 9900, 9901, 9902, -1, 9903, 9904, 9905, -1, 9906, 9907, 9908, -1, 9909, 9910, 9911, -1, 9912, 9913, 9914, -1, 9915, 9916, 9917, -1, 9918, 9919, 9920, -1, 9921, 9922, 9923, -1, 9924, 9925, 9926, -1, 9927, 9928, 9929, -1, 9930, 9931, 9932, -1, 9933, 9934, 9935, -1, 9936, 9937, 9938, -1, 9939, 9940, 9941, -1, 9942, 9943, 9944, -1, 9945, 9946, 9947, -1, 9948, 9949, 9950, -1, 9951, 9952, 9953, -1, 9954, 9955, 9956, -1, 9957, 9958, 9959, -1, 9960, 9961, 9962, -1, 9963, 9964, 9965, -1, 9966, 9967, 9968, -1, 9969, 9970, 9971, -1, 9972, 9973, 9974, -1, 9975, 9976, 9977, -1, 9978, 9979, 9980, -1, 9981, 9982, 9983, -1, 9984, 9985, 9986, -1, 9987, 9988, 9989, -1, 9990, 9991, 9992, -1, 9993, 9994, 9995, -1, 9996, 9997, 9998, -1, 9999, 10000, 10001, -1, 10002, 10003, 10004, -1, 10005, 10006, 10007, -1, 10008, 10009, 10010, -1, 10011, 10012, 10013, -1, 10014, 10015, 10016, -1, 10017, 10018, 10019, -1, 10020, 10021, 10022, -1, 10023, 10024, 10025, -1, 10026, 10027, 10028, -1, 10029, 10030, 10031, -1, 10032, 10033, 10034, -1, 10035, 10036, 10037, -1, 10038, 10039, 10040, -1, 10041, 10042, 10043, -1, 10044, 10045, 10046, -1, 10047, 10048, 10049, -1, 10050, 10051, 10052, -1, 10053, 10054, 10055, -1, 10056, 10057, 10058, -1, 10059, 10060, 10061, -1, 10062, 10063, 10064, -1, 10065, 10066, 10067, -1, 10068, 10069, 10070, -1, 10071, 10072, 10073, -1, 10074, 10075, 10076, -1, 10077, 10078, 10079, -1, 10080, 10081, 10082, -1, 10083, 10084, 10085, -1, 10086, 10087, 10088, -1, 10089, 10090, 10091, -1, 10092, 10093, 10094, -1, 10095, 10096, 10097, -1, 10098, 10099, 10100, -1, 10101, 10102, 10103, -1, 10104, 10105, 10106, -1, 10107, 10108, 10109, -1, 10110, 10111, 10112, -1, 10113, 10114, 10115, -1, 10116, 10117, 10118, -1, 10119, 10120, 10121, -1, 10122, 10123, 10124, -1, 10125, 10126, 10127, -1, 10128, 10129, 10130, -1, 10131, 10132, 10133, -1, 10134, 10135, 10136, -1, 10137, 10138, 10139, -1, 10140, 10141, 10142, -1, 10143, 10144, 10145, -1, 10146, 10147, 10148, -1, 10149, 10150, 10151, -1, 10152, 10153, 10154, -1, 10155, 10156, 10157, -1, 10158, 10159, 10160, -1, 10161, 10162, 10163, -1, 10164, 10165, 10166, -1, 10167, 10168, 10169, -1, 10170, 10171, 10172, -1, 10173, 10174, 10175, -1, 10176, 10177, 10178, -1, 10179, 10180, 10181, -1, 10182, 10183, 10184, -1, 10185, 10186, 10187, -1, 10188, 10189, 10190, -1, 10191, 10192, 10193, -1, 10194, 10195, 10196, -1, 10197, 10198, 10199, -1, 10200, 10201, 10202, -1, 10203, 10204, 10205, -1, 10206, 10207, 10208, -1, 10209, 10210, 10211, -1, 10212, 10213, 10214, -1, 10215, 10216, 10217, -1, 10218, 10219, 10220, -1, 10221, 10222, 10223, -1, 10224, 10225, 10226, -1, 10227, 10228, 10229, -1, 10230, 10231, 10232, -1, 10233, 10234, 10235, -1, 10236, 10237, 10238, -1, 10239, 10240, 10241, -1, 10242, 10243, 10244, -1, 10245, 10246, 10247, -1, 10248, 10249, 10250, -1, 10251, 10252, 10253, -1, 10254, 10255, 10256, -1, 10257, 10258, 10259, -1, 10260, 10261, 10262, -1, 10263, 10264, 10265, -1, 10266, 10267, 10268, -1, 10269, 10270, 10271, -1, 10272, 10273, 10274, -1, 10275, 10276, 10277, -1, 10278, 10279, 10280, -1, 10281, 10282, 10283, -1, 10284, 10285, 10286, -1, 10287, 10288, 10289, -1, 10290, 10291, 10292, -1, 10293, 10294, 10295, -1, 10296, 10297, 10298, -1, 10299, 10300, 10301, -1, 10302, 10303, 10304, -1, 10305, 10306, 10307, -1, 10308, 10309, 10310, -1, 10311, 10312, 10313, -1, 10314, 10315, 10316, -1, 10317, 10318, 10319, -1, 10320, 10321, 10322, -1, 10323, 10324, 10325, -1, 10326, 10327, 10328, -1, 10329, 10330, 10331, -1, 10332, 10333, 10334, -1, 10335, 10336, 10337, -1, 10338, 10339, 10340, -1, 10341, 10342, 10343, -1, 10344, 10345, 10346, -1, 10347, 10348, 10349, -1, 10350, 10351, 10352, -1, 10353, 10354, 10355, -1, 10356, 10357, 10358, -1, 10359, 10360, 10361, -1, 10362, 10363, 10364, -1, 10365, 10366, 10367, -1, 10368, 10369, 10370, -1, 10371, 10372, 10373, -1, 10374, 10375, 10376, -1, 10377, 10378, 10379, -1, 10380, 10381, 10382, -1, 10383, 10384, 10385, -1, 10386, 10387, 10388, -1, 10389, 10390, 10391, -1, 10392, 10393, 10394, -1, 10395, 10396, 10397, -1, 10398, 10399, 10400, -1, 10401, 10402, 10403, -1, 10404, 10405, 10406, -1, 10407, 10408, 10409, -1, 10410, 10411, 10412, -1, 10413, 10414, 10415, -1, 10416, 10417, 10418, -1, 10419, 10420, 10421, -1, 10422, 10423, 10424, -1, 10425, 10426, 10427, -1, 10428, 10429, 10430, -1, 10431, 10432, 10433, -1, 10434, 10435, 10436, -1, 10437, 10438, 10439, -1, 10440, 10441, 10442, -1, 10443, 10444, 10445, -1, 10446, 10447, 10448, -1, 10449, 10450, 10451, -1, 10452, 10453, 10454, -1, 10455, 10456, 10457, -1, 10458, 10459, 10460, -1, 10461, 10462, 10463, -1, 10464, 10465, 10466, -1, 10467, 10468, 10469, -1, 10470, 10471, 10472, -1, 10473, 10474, 10475, -1, 10476, 10477, 10478, -1, 10479, 10480, 10481, -1, 10482, 10483, 10484, -1, 10485, 10486, 10487, -1, 10488, 10489, 10490, -1, 10491, 10492, 10493, -1, 10494, 10495, 10496, -1, 10497, 10498, 10499, -1, 10500, 10501, 10502, -1, 10503, 10504, 10505, -1, 10506, 10507, 10508, -1, 10509, 10510, 10511, -1, 10512, 10513, 10514, -1, 10515, 10516, 10517, -1, 10518, 10519, 10520, -1, 10521, 10522, 10523, -1, 10524, 10525, 10526, -1, 10527, 10528, 10529, -1, 10530, 10531, 10532, -1, 10533, 10534, 10535, -1, 10536, 10537, 10538, -1, 10539, 10540, 10541, -1, 10542, 10543, 10544, -1, 10545, 10546, 10547, -1, 10548, 10549, 10550, -1, 10551, 10552, 10553, -1, 10554, 10555, 10556, -1, 10557, 10558, 10559, -1, 10560, 10561, 10562, -1, 10563, 10564, 10565, -1, 10566, 10567, 10568, -1, 10569, 10570, 10571, -1, 10572, 10573, 10574, -1, 10575, 10576, 10577, -1, 10578, 10579, 10580, -1, 10581, 10582, 10583, -1, 10584, 10585, 10586, -1, 10587, 10588, 10589, -1, 10590, 10591, 10592, -1, 10593, 10594, 10595, -1, 10596, 10597, 10598, -1, 10599, 10600, 10601, -1, 10602, 10603, 10604, -1, 10605, 10606, 10607, -1, 10608, 10609, 10610, -1, 10611, 10612, 10613, -1, 10614, 10615, 10616, -1, 10617, 10618, 10619, -1, 10620, 10621, 10622, -1, 10623, 10624, 10625, -1, 10626, 10627, 10628, -1, 10629, 10630, 10631, -1, 10632, 10633, 10634, -1, 10635, 10636, 10637, -1, 10638, 10639, 10640, -1, 10641, 10642, 10643, -1, 10644, 10645, 10646, -1, 10647, 10648, 10649, -1, 10650, 10651, 10652, -1, 10653, 10654, 10655, -1, 10656, 10657, 10658, -1, 10659, 10660, 10661, -1, 10662, 10663, 10664, -1, 10665, 10666, 10667, -1, 10668, 10669, 10670, -1, 10671, 10672, 10673, -1, 10674, 10675, 10676, -1, 10677, 10678, 10679, -1, 10680, 10681, 10682, -1, 10683, 10684, 10685, -1, 10686, 10687, 10688, -1, 10689, 10690, 10691, -1, 10692, 10693, 10694, -1, 10695, 10696, 10697, -1, 10698, 10699, 10700, -1, 10701, 10702, 10703, -1, 10704, 10705, 10706, -1, 10707, 10708, 10709, -1, 10710, 10711, 10712, -1, 10713, 10714, 10715, -1, 10716, 10717, 10718, -1, 10719, 10720, 10721, -1, 10722, 10723, 10724, -1, 10725, 10726, 10727, -1, 10728, 10729, 10730, -1, 10731, 10732, 10733, -1, 10734, 10735, 10736, -1, 10737, 10738, 10739, -1, 10740, 10741, 10742, -1, 10743, 10744, 10745, -1, 10746, 10747, 10748, -1, 10749, 10750, 10751, -1, 10752, 10753, 10754, -1, 10755, 10756, 10757, -1, 10758, 10759, 10760, -1, 10761, 10762, 10763, -1, 10764, 10765, 10766, -1, 10767, 10768, 10769, -1, 10770, 10771, 10772, -1, 10773, 10774, 10775, -1, 10776, 10777, 10778, -1, 10779, 10780, 10781, -1, 10782, 10783, 10784, -1, 10785, 10786, 10787, -1, 10788, 10789, 10790, -1, 10791, 10792, 10793, -1, 10794, 10795, 10796, -1, 10797, 10798, 10799, -1, 10800, 10801, 10802, -1, 10803, 10804, 10805, -1, 10806, 10807, 10808, -1, 10809, 10810, 10811, -1, 10812, 10813, 10814, -1, 10815, 10816, 10817, -1, 10818, 10819, 10820, -1, 10821, 10822, 10823, -1, 10824, 10825, 10826, -1, 10827, 10828, 10829, -1, 10830, 10831, 10832, -1, 10833, 10834, 10835, -1, 10836, 10837, 10838, -1, 10839, 10840, 10841, -1, 10842, 10843, 10844, -1, 10845, 10846, 10847, -1, 10848, 10849, 10850, -1, 10851, 10852, 10853, -1, 10854, 10855, 10856, -1, 10857, 10858, 10859, -1, 10860, 10861, 10862, -1, 10863, 10864, 10865, -1, 10866, 10867, 10868, -1, 10869, 10870, 10871, -1, 10872, 10873, 10874, -1, 10875, 10876, 10877, -1, 10878, 10879, 10880, -1, 10881, 10882, 10883, -1, 10884, 10885, 10886, -1, 10887, 10888, 10889, -1, 10890, 10891, 10892, -1, 10893, 10894, 10895, -1, 10896, 10897, 10898, -1, 10899, 10900, 10901, -1, 10902, 10903, 10904, -1, 10905, 10906, 10907, -1, 10908, 10909, 10910, -1, 10911, 10912, 10913, -1, 10914, 10915, 10916, -1, 10917, 10918, 10919, -1, 10920, 10921, 10922, -1, 10923, 10924, 10925, -1, 10926, 10927, 10928, -1, 10929, 10930, 10931, -1, 10932, 10933, 10934, -1, 10935, 10936, 10937, -1, 10938, 10939, 10940, -1, 10941, 10942, 10943, -1, 10944, 10945, 10946, -1, 10947, 10948, 10949, -1, 10950, 10951, 10952, -1, 10953, 10954, 10955, -1, 10956, 10957, 10958, -1, 10959, 10960, 10961, -1, 10962, 10963, 10964, -1, 10965, 10966, 10967, -1, 10968, 10969, 10970, -1, 10971, 10972, 10973, -1, 10974, 10975, 10976, -1, 10977, 10978, 10979, -1, 10980, 10981, 10982, -1, 10983, 10984, 10985, -1, 10986, 10987, 10988, -1, 10989, 10990, 10991, -1, 10992, 10993, 10994, -1, 10995, 10996, 10997, -1, 10998, 10999, 11000, -1, 11001, 11002, 11003, -1, 11004, 11005, 11006, -1, 11007, 11008, 11009, -1, 11010, 11011, 11012, -1, 11013, 11014, 11015, -1, 11016, 11017, 11018, -1, 11019, 11020, 11021, -1, 11022, 11023, 11024, -1, 11025, 11026, 11027, -1, 11028, 11029, 11030, -1, 11031, 11032, 11033, -1, 11034, 11035, 11036, -1, 11037, 11038, 11039, -1, 11040, 11041, 11042, -1, 11043, 11044, 11045, -1, 11046, 11047, 11048, -1, 11049, 11050, 11051, -1, 11052, 11053, 11054, -1, 11055, 11056, 11057, -1, 11058, 11059, 11060, -1, 11061, 11062, 11063, -1, 11064, 11065, 11066, -1, 11067, 11068, 11069, -1, 11070, 11071, 11072, -1, 11073, 11074, 11075, -1, 11076, 11077, 11078, -1, 11079, 11080, 11081, -1, 11082, 11083, 11084, -1, 11085, 11086, 11087, -1, 11088, 11089, 11090, -1, 11091, 11092, 11093, -1, 11094, 11095, 11096, -1, 11097, 11098, 11099, -1, 11100, 11101, 11102, -1, 11103, 11104, 11105, -1, 11106, 11107, 11108, -1, 11109, 11110, 11111, -1, 11112, 11113, 11114, -1, 11115, 11116, 11117, -1, 11118, 11119, 11120, -1, 11121, 11122, 11123, -1, 11124, 11125, 11126, -1, 11127, 11128, 11129, -1, 11130, 11131, 11132, -1, 11133, 11134, 11135, -1, 11136, 11137, 11138, -1, 11139, 11140, 11141, -1, 11142, 11143, 11144, -1, 11145, 11146, 11147, -1, 11148, 11149, 11150, -1, 11151, 11152, 11153, -1, 11154, 11155, 11156, -1, 11157, 11158, 11159, -1, 11160, 11161, 11162, -1, 11163, 11164, 11165, -1, 11166, 11167, 11168, -1, 11169, 11170, 11171, -1, 11172, 11173, 11174, -1, 11175, 11176, 11177, -1, 11178, 11179, 11180, -1, 11181, 11182, 11183, -1, 11184, 11185, 11186, -1, 11187, 11188, 11189, -1, 11190, 11191, 11192, -1, 11193, 11194, 11195, -1, 11196, 11197, 11198, -1, 11199, 11200, 11201, -1, 11202, 11203, 11204, -1, 11205, 11206, 11207, -1, 11208, 11209, 11210, -1, 11211, 11212, 11213, -1, 11214, 11215, 11216, -1, 11217, 11218, 11219, -1, 11220, 11221, 11222, -1, 11223, 11224, 11225, -1, 11226, 11227, 11228, -1, 11229, 11230, 11231, -1, 11232, 11233, 11234, -1, 11235, 11236, 11237, -1, 11238, 11239, 11240, -1, 11241, 11242, 11243, -1, 11244, 11245, 11246, -1, 11247, 11248, 11249, -1, 11250, 11251, 11252, -1, 11253, 11254, 11255, -1, 11256, 11257, 11258, -1, 11259, 11260, 11261, -1, 11262, 11263, 11264, -1, 11265, 11266, 11267, -1, 11268, 11269, 11270, -1, 11271, 11272, 11273, -1, 11274, 11275, 11276, -1, 11277, 11278, 11279, -1, 11280, 11281, 11282, -1, 11283, 11284, 11285, -1, 11286, 11287, 11288, -1, 11289, 11290, 11291, -1, 11292, 11293, 11294, -1, 11295, 11296, 11297, -1, 11298, 11299, 11300, -1, 11301, 11302, 11303, -1, 11304, 11305, 11306, -1, 11307, 11308, 11309, -1, 11310, 11311, 11312, -1, 11313, 11314, 11315, -1, 11316, 11317, 11318, -1, 11319, 11320, 11321, -1, 11322, 11323, 11324, -1, 11325, 11326, 11327, -1, 11328, 11329, 11330, -1, 11331, 11332, 11333, -1, 11334, 11335, 11336, -1, 11337, 11338, 11339, -1, 11340, 11341, 11342, -1, 11343, 11344, 11345, -1, 11346, 11347, 11348, -1, 11349, 11350, 11351, -1, 11352, 11353, 11354, -1, 11355, 11356, 11357, -1, 11358, 11359, 11360, -1, 11361, 11362, 11363, -1, 11364, 11365, 11366, -1, 11367, 11368, 11369, -1, 11370, 11371, 11372, -1, 11373, 11374, 11375, -1, 11376, 11377, 11378, -1, 11379, 11380, 11381, -1, 11382, 11383, 11384, -1, 11385, 11386, 11387, -1, 11388, 11389, 11390, -1, 11391, 11392, 11393, -1, 11394, 11395, 11396, -1, 11397, 11398, 11399, -1, 11400, 11401, 11402, -1, 11403, 11404, 11405, -1, 11406, 11407, 11408, -1, 11409, 11410, 11411, -1, 11412, 11413, 11414, -1, 11415, 11416, 11417, -1, 11418, 11419, 11420, -1, 11421, 11422, 11423, -1, 11424, 11425, 11426, -1, 11427, 11428, 11429, -1, 11430, 11431, 11432, -1, 11433, 11434, 11435, -1, 11436, 11437, 11438, -1, 11439, 11440, 11441, -1, 11442, 11443, 11444, -1, 11445, 11446, 11447, -1, 11448, 11449, 11450, -1, 11451, 11452, 11453, -1, 11454, 11455, 11456, -1, 11457, 11458, 11459, -1, 11460, 11461, 11462, -1, 11463, 11464, 11465, -1, 11466, 11467, 11468, -1, 11469, 11470, 11471, -1, 11472, 11473, 11474, -1, 11475, 11476, 11477, -1, 11478, 11479, 11480, -1, 11481, 11482, 11483, -1, 11484, 11485, 11486, -1, 11487, 11488, 11489, -1, 11490, 11491, 11492, -1, 11493, 11494, 11495, -1, 11496, 11497, 11498, -1, 11499, 11500, 11501, -1, 11502, 11503, 11504, -1, 11505, 11506, 11507, -1, 11508, 11509, 11510, -1, 11511, 11512, 11513, -1, 11514, 11515, 11516, -1, 11517, 11518, 11519, -1, 11520, 11521, 11522, -1, 11523, 11524, 11525, -1, 11526, 11527, 11528, -1, 11529, 11530, 11531, -1, 11532, 11533, 11534, -1, 11535, 11536, 11537, -1, 11538, 11539, 11540, -1, 11541, 11542, 11543, -1, 11544, 11545, 11546, -1, 11547, 11548, 11549, -1, 11550, 11551, 11552, -1, 11553, 11554, 11555, -1, 11556, 11557, 11558, -1, 11559, 11560, 11561, -1, 11562, 11563, 11564, -1, 11565, 11566, 11567, -1, 11568, 11569, 11570, -1, 11571, 11572, 11573, -1, 11574, 11575, 11576, -1, 11577, 11578, 11579, -1, 11580, 11581, 11582, -1, 11583, 11584, 11585, -1, 11586, 11587, 11588, -1, 11589, 11590, 11591, -1, 11592, 11593, 11594, -1, 11595, 11596, 11597, -1, 11598, 11599, 11600, -1, 11601, 11602, 11603, -1, 11604, 11605, 11606, -1, 11607, 11608, 11609, -1, 11610, 11611, 11612, -1, 11613, 11614, 11615, -1, 11616, 11617, 11618, -1, 11619, 11620, 11621, -1, 11622, 11623, 11624, -1, 11625, 11626, 11627, -1, 11628, 11629, 11630, -1, 11631, 11632, 11633, -1, 11634, 11635, 11636, -1, 11637, 11638, 11639, -1, 11640, 11641, 11642, -1, 11643, 11644, 11645, -1, 11646, 11647, 11648, -1, 11649, 11650, 11651, -1, 11652, 11653, 11654, -1, 11655, 11656, 11657, -1, 11658, 11659, 11660, -1, 11661, 11662, 11663, -1, 11664, 11665, 11666, -1, 11667, 11668, 11669, -1, 11670, 11671, 11672, -1, 11673, 11674, 11675, -1, 11676, 11677, 11678, -1, 11679, 11680, 11681, -1, 11682, 11683, 11684, -1, 11685, 11686, 11687, -1, 11688, 11689, 11690, -1, 11691, 11692, 11693, -1, 11694, 11695, 11696, -1, 11697, 11698, 11699, -1, 11700, 11701, 11702, -1, 11703, 11704, 11705, -1, 11706, 11707, 11708, -1, 11709, 11710, 11711, -1, 11712, 11713, 11714, -1, 11715, 11716, 11717, -1, 11718, 11719, 11720, -1, 11721, 11722, 11723, -1, 11724, 11725, 11726, -1, 11727, 11728, 11729, -1, 11730, 11731, 11732, -1, 11733, 11734, 11735, -1, 11736, 11737, 11738, -1, 11739, 11740, 11741, -1, 11742, 11743, 11744, -1, 11745, 11746, 11747, -1, 11748, 11749, 11750, -1, 11751, 11752, 11753, -1, 11754, 11755, 11756, -1, 11757, 11758, 11759, -1, 11760, 11761, 11762, -1, 11763, 11764, 11765, -1, 11766, 11767, 11768, -1, 11769, 11770, 11771, -1, 11772, 11773, 11774, -1, 11775, 11776, 11777, -1, 11778, 11779, 11780, -1, 11781, 11782, 11783, -1, 11784, 11785, 11786, -1, 11787, 11788, 11789, -1, 11790, 11791, 11792, -1, 11793, 11794, 11795, -1, 11796, 11797, 11798, -1, 11799, 11800, 11801, -1, 11802, 11803, 11804, -1, 11805, 11806, 11807, -1, 11808, 11809, 11810, -1, 11811, 11812, 11813, -1, 11814, 11815, 11816, -1, 11817, 11818, 11819, -1, 11820, 11821, 11822, -1, 11823, 11824, 11825, -1, 11826, 11827, 11828, -1, 11829, 11830, 11831, -1, 11832, 11833, 11834, -1, 11835, 11836, 11837, -1, 11838, 11839, 11840, -1, 11841, 11842, 11843, -1, 11844, 11845, 11846, -1, 11847, 11848, 11849, -1, 11850, 11851, 11852, -1, 11853, 11854, 11855, -1, 11856, 11857, 11858, -1, 11859, 11860, 11861, -1, 11862, 11863, 11864, -1, 11865, 11866, 11867, -1, 11868, 11869, 11870, -1, 11871, 11872, 11873, -1, 11874, 11875, 11876, -1, 11877, 11878, 11879, -1, 11880, 11881, 11882, -1, 11883, 11884, 11885, -1, 11886, 11887, 11888, -1, 11889, 11890, 11891, -1, 11892, 11893, 11894, -1, 11895, 11896, 11897, -1, 11898, 11899, 11900, -1, 11901, 11902, 11903, -1, 11904, 11905, 11906, -1, 11907, 11908, 11909, -1, 11910, 11911, 11912, -1, 11913, 11914, 11915, -1, 11916, 11917, 11918, -1, 11919, 11920, 11921, -1, 11922, 11923, 11924, -1, 11925, 11926, 11927, -1, 11928, 11929, 11930, -1, 11931, 11932, 11933, -1, 11934, 11935, 11936, -1, 11937, 11938, 11939, -1, 11940, 11941, 11942, -1, 11943, 11944, 11945, -1, 11946, 11947, 11948, -1, 11949, 11950, 11951, -1, 11952, 11953, 11954, -1, 11955, 11956, 11957, -1, 11958, 11959, 11960, -1, 11961, 11962, 11963, -1, 11964, 11965, 11966, -1, 11967, 11968, 11969, -1, 11970, 11971, 11972, -1, 11973, 11974, 11975, -1, 11976, 11977, 11978, -1, 11979, 11980, 11981, -1, 11982, 11983, 11984, -1, 11985, 11986, 11987, -1, 11988, 11989, 11990, -1, 11991, 11992, 11993, -1, 11994, 11995, 11996, -1, 11997, 11998, 11999, -1, 12000, 12001, 12002, -1, 12003, 12004, 12005, -1, 12006, 12007, 12008, -1, 12009, 12010, 12011, -1, 12012, 12013, 12014, -1, 12015, 12016, 12017, -1, 12018, 12019, 12020, -1, 12021, 12022, 12023, -1, 12024, 12025, 12026, -1, 12027, 12028, 12029, -1, 12030, 12031, 12032, -1, 12033, 12034, 12035, -1, 12036, 12037, 12038, -1, 12039, 12040, 12041, -1, 12042, 12043, 12044, -1, 12045, 12046, 12047, -1, 12048, 12049, 12050, -1, 12051, 12052, 12053, -1, 12054, 12055, 12056, -1, 12057, 12058, 12059, -1, 12060, 12061, 12062, -1, 12063, 12064, 12065, -1, 12066, 12067, 12068, -1, 12069, 12070, 12071, -1, 12072, 12073, 12074, -1, 12075, 12076, 12077, -1, 12078, 12079, 12080, -1, 12081, 12082, 12083, -1, 12084, 12085, 12086, -1, 12087, 12088, 12089, -1, 12090, 12091, 12092, -1, 12093, 12094, 12095, -1, 12096, 12097, 12098, -1, 12099, 12100, 12101, -1, 12102, 12103, 12104, -1, 12105, 12106, 12107, -1, 12108, 12109, 12110, -1, 12111, 12112, 12113, -1, 12114, 12115, 12116, -1, 12117, 12118, 12119, -1, 12120, 12121, 12122, -1, 12123, 12124, 12125, -1, 12126, 12127, 12128, -1, 12129, 12130, 12131, -1, 12132, 12133, 12134, -1, 12135, 12136, 12137, -1, 12138, 12139, 12140, -1, 12141, 12142, 12143, -1, 12144, 12145, 12146, -1, 12147, 12148, 12149, -1, 12150, 12151, 12152, -1, 12153, 12154, 12155, -1, 12156, 12157, 12158, -1, 12159, 12160, 12161, -1, 12162, 12163, 12164, -1, 12165, 12166, 12167, -1, 12168, 12169, 12170, -1, 12171, 12172, 12173, -1, 12174, 12175, 12176, -1, 12177, 12178, 12179, -1, 12180, 12181, 12182, -1, 12183, 12184, 12185, -1, 12186, 12187, 12188, -1, 12189, 12190, 12191, -1, 12192, 12193, 12194, -1, 12195, 12196, 12197, -1, 12198, 12199, 12200, -1, 12201, 12202, 12203, -1, 12204, 12205, 12206, -1, 12207, 12208, 12209, -1, 12210, 12211, 12212, -1, 12213, 12214, 12215, -1, 12216, 12217, 12218, -1, 12219, 12220, 12221, -1, 12222, 12223, 12224, -1, 12225, 12226, 12227, -1, 12228, 12229, 12230, -1, 12231, 12232, 12233, -1, 12234, 12235, 12236, -1, 12237, 12238, 12239, -1, 12240, 12241, 12242, -1, 12243, 12244, 12245, -1, 12246, 12247, 12248, -1, 12249, 12250, 12251, -1, 12252, 12253, 12254, -1, 12255, 12256, 12257, -1, 12258, 12259, 12260, -1, 12261, 12262, 12263, -1, 12264, 12265, 12266, -1, 12267, 12268, 12269, -1, 12270, 12271, 12272, -1, 12273, 12274, 12275, -1, 12276, 12277, 12278, -1, 12279, 12280, 12281, -1, 12282, 12283, 12284, -1, 12285, 12286, 12287, -1, 12288, 12289, 12290, -1, 12291, 12292, 12293, -1, 12294, 12295, 12296, -1, 12297, 12298, 12299, -1, 12300, 12301, 12302, -1, 12303, 12304, 12305, -1, 12306, 12307, 12308, -1, 12309, 12310, 12311, -1, 12312, 12313, 12314, -1, 12315, 12316, 12317, -1, 12318, 12319, 12320, -1, 12321, 12322, 12323, -1, 12324, 12325, 12326, -1, 12327, 12328, 12329, -1, 12330, 12331, 12332, -1, 12333, 12334, 12335, -1, 12336, 12337, 12338, -1, 12339, 12340, 12341, -1, 12342, 12343, 12344, -1, 12345, 12346, 12347, -1, 12348, 12349, 12350, -1, 12351, 12352, 12353, -1, 12354, 12355, 12356, -1, 12357, 12358, 12359, -1, 12360, 12361, 12362, -1, 12363, 12364, 12365, -1, 12366, 12367, 12368, -1, 12369, 12370, 12371, -1, 12372, 12373, 12374, -1, 12375, 12376, 12377, -1, 12378, 12379, 12380, -1, 12381, 12382, 12383, -1, 12384, 12385, 12386, -1, 12387, 12388, 12389, -1, 12390, 12391, 12392, -1, 12393, 12394, 12395, -1, 12396, 12397, 12398, -1, 12399, 12400, 12401, -1, 12402, 12403, 12404, -1, 12405, 12406, 12407, -1, 12408, 12409, 12410, -1, 12411, 12412, 12413, -1, 12414, 12415, 12416, -1, 12417, 12418, 12419, -1, 12420, 12421, 12422, -1, 12423, 12424, 12425, -1, 12426, 12427, 12428, -1, 12429, 12430, 12431, -1, 12432, 12433, 12434, -1, 12435, 12436, 12437, -1, 12438, 12439, 12440, -1, 12441, 12442, 12443, -1, 12444, 12445, 12446, -1, 12447, 12448, 12449, -1, 12450, 12451, 12452, -1, 12453, 12454, 12455, -1, 12456, 12457, 12458, -1, 12459, 12460, 12461, -1, 12462, 12463, 12464, -1, 12465, 12466, 12467, -1, 12468, 12469, 12470, -1, 12471, 12472, 12473, -1, 12474, 12475, 12476, -1, 12477, 12478, 12479, -1, 12480, 12481, 12482, -1, 12483, 12484, 12485, -1, 12486, 12487, 12488, -1, 12489, 12490, 12491, -1, 12492, 12493, 12494, -1, 12495, 12496, 12497, -1, 12498, 12499, 12500, -1, 12501, 12502, 12503, -1, 12504, 12505, 12506, -1, 12507, 12508, 12509, -1, 12510, 12511, 12512, -1, 12513, 12514, 12515, -1, 12516, 12517, 12518, -1, 12519, 12520, 12521, -1, 12522, 12523, 12524, -1, 12525, 12526, 12527, -1, 12528, 12529, 12530, -1, 12531, 12532, 12533, -1, 12534, 12535, 12536, -1, 12537, 12538, 12539, -1, 12540, 12541, 12542, -1, 12543, 12544, 12545, -1, 12546, 12547, 12548, -1, 12549, 12550, 12551, -1, 12552, 12553, 12554, -1, 12555, 12556, 12557, -1, 12558, 12559, 12560, -1, 12561, 12562, 12563, -1, 12564, 12565, 12566, -1, 12567, 12568, 12569, -1, 12570, 12571, 12572, -1, 12573, 12574, 12575, -1, 12576, 12577, 12578, -1, 12579, 12580, 12581, -1, 12582, 12583, 12584, -1, 12585, 12586, 12587, -1, 12588, 12589, 12590, -1, 12591, 12592, 12593, -1, 12594, 12595, 12596, -1, 12597, 12598, 12599, -1, 12600, 12601, 12602, -1, 12603, 12604, 12605, -1, 12606, 12607, 12608, -1, 12609, 12610, 12611, -1, 12612, 12613, 12614, -1, 12615, 12616, 12617, -1, 12618, 12619, 12620, -1, 12621, 12622, 12623, -1, 12624, 12625, 12626, -1, 12627, 12628, 12629, -1, 12630, 12631, 12632, -1, 12633, 12634, 12635, -1, 12636, 12637, 12638, -1, 12639, 12640, 12641, -1, 12642, 12643, 12644, -1, 12645, 12646, 12647, -1, 12648, 12649, 12650, -1, 12651, 12652, 12653, -1, 12654, 12655, 12656, -1, 12657, 12658, 12659, -1, 12660, 12661, 12662, -1, 12663, 12664, 12665, -1, 12666, 12667, 12668, -1, 12669, 12670, 12671, -1, 12672, 12673, 12674, -1, 12675, 12676, 12677, -1, 12678, 12679, 12680, -1, 12681, 12682, 12683, -1, 12684, 12685, 12686, -1, 12687, 12688, 12689, -1, 12690, 12691, 12692, -1, 12693, 12694, 12695, -1, 12696, 12697, 12698, -1, 12699, 12700, 12701, -1, 12702, 12703, 12704, -1, 12705, 12706, 12707, -1, 12708, 12709, 12710, -1, 12711, 12712, 12713, -1, 12714, 12715, 12716, -1, 12717, 12718, 12719, -1, 12720, 12721, 12722, -1, 12723, 12724, 12725, -1, 12726, 12727, 12728, -1, 12729, 12730, 12731, -1, 12732, 12733, 12734, -1, 12735, 12736, 12737, -1, 12738, 12739, 12740, -1, 12741, 12742, 12743, -1, 12744, 12745, 12746, -1, 12747, 12748, 12749, -1, 12750, 12751, 12752, -1, 12753, 12754, 12755, -1, 12756, 12757, 12758, -1, 12759, 12760, 12761, -1, 12762, 12763, 12764, -1, 12765, 12766, 12767, -1, 12768, 12769, 12770, -1, 12771, 12772, 12773, -1, 12774, 12775, 12776, -1, 12777, 12778, 12779, -1, 12780, 12781, 12782, -1, 12783, 12784, 12785, -1, 12786, 12787, 12788, -1, 12789, 12790, 12791, -1, 12792, 12793, 12794, -1, 12795, 12796, 12797, -1, 12798, 12799, 12800, -1, 12801, 12802, 12803, -1, 12804, 12805, 12806, -1, 12807, 12808, 12809, -1, 12810, 12811, 12812, -1, 12813, 12814, 12815, -1, 12816, 12817, 12818, -1, 12819, 12820, 12821, -1, 12822, 12823, 12824, -1, 12825, 12826, 12827, -1, 12828, 12829, 12830, -1, 12831, 12832, 12833, -1, 12834, 12835, 12836, -1, 12837, 12838, 12839, -1, 12840, 12841, 12842, -1, 12843, 12844, 12845, -1, 12846, 12847, 12848, -1, 12849, 12850, 12851, -1, 12852, 12853, 12854, -1, 12855, 12856, 12857, -1, 12858, 12859, 12860, -1, 12861, 12862, 12863, -1, 12864, 12865, 12866, -1, 12867, 12868, 12869, -1, 12870, 12871, 12872, -1, 12873, 12874, 12875, -1, 12876, 12877, 12878, -1, 12879, 12880, 12881, -1, 12882, 12883, 12884, -1, 12885, 12886, 12887, -1, 12888, 12889, 12890, -1, 12891, 12892, 12893, -1, 12894, 12895, 12896, -1, 12897, 12898, 12899, -1, 12900, 12901, 12902, -1, 12903, 12904, 12905, -1, 12906, 12907, 12908, -1, 12909, 12910, 12911, -1, 12912, 12913, 12914, -1, 12915, 12916, 12917, -1, 12918, 12919, 12920, -1, 12921, 12922, 12923, -1, 12924, 12925, 12926, -1, 12927, 12928, 12929, -1, 12930, 12931, 12932, -1, 12933, 12934, 12935, -1, 12936, 12937, 12938, -1, 12939, 12940, 12941, -1, 12942, 12943, 12944, -1, 12945, 12946, 12947, -1, 12948, 12949, 12950, -1, 12951, 12952, 12953, -1, 12954, 12955, 12956, -1, 12957, 12958, 12959, -1, 12960, 12961, 12962, -1, 12963, 12964, 12965, -1, 12966, 12967, 12968, -1, 12969, 12970, 12971, -1, 12972, 12973, 12974, -1, 12975, 12976, 12977, -1, 12978, 12979, 12980, -1, 12981, 12982, 12983, -1, 12984, 12985, 12986, -1, 12987, 12988, 12989, -1, 12990, 12991, 12992, -1, 12993, 12994, 12995, -1, 12996, 12997, 12998, -1, 12999, 13000, 13001, -1, 13002, 13003, 13004, -1, 13005, 13006, 13007, -1, 13008, 13009, 13010, -1, 13011, 13012, 13013, -1, 13014, 13015, 13016, -1, 13017, 13018, 13019, -1, 13020, 13021, 13022, -1, 13023, 13024, 13025, -1, 13026, 13027, 13028, -1, 13029, 13030, 13031, -1, 13032, 13033, 13034, -1, 13035, 13036, 13037, -1, 13038, 13039, 13040, -1, 13041, 13042, 13043, -1, 13044, 13045, 13046, -1, 13047, 13048, 13049, -1, 13050, 13051, 13052, -1, 13053, 13054, 13055, -1, 13056, 13057, 13058, -1, 13059, 13060, 13061, -1, 13062, 13063, 13064, -1, 13065, 13066, 13067, -1, 13068, 13069, 13070, -1, 13071, 13072, 13073, -1, 13074, 13075, 13076, -1, 13077, 13078, 13079, -1, 13080, 13081, 13082, -1, 13083, 13084, 13085, -1, 13086, 13087, 13088, -1, 13089, 13090, 13091, -1, 13092, 13093, 13094, -1, 13095, 13096, 13097, -1, 13098, 13099, 13100, -1, 13101, 13102, 13103, -1, 13104, 13105, 13106, -1, 13107, 13108, 13109, -1, 13110, 13111, 13112, -1, 13113, 13114, 13115, -1, 13116, 13117, 13118, -1, 13119, 13120, 13121, -1, 13122, 13123, 13124, -1, 13125, 13126, 13127, -1, 13128, 13129, 13130, -1, 13131, 13132, 13133, -1, 13134, 13135, 13136, -1, 13137, 13138, 13139, -1, 13140, 13141, 13142, -1, 13143, 13144, 13145, -1, 13146, 13147, 13148, -1, 13149, 13150, 13151, -1, 13152, 13153, 13154, -1, 13155, 13156, 13157, -1, 13158, 13159, 13160, -1, 13161, 13162, 13163, -1, 13164, 13165, 13166, -1, 13167, 13168, 13169, -1, 13170, 13171, 13172, -1, 13173, 13174, 13175, -1, 13176, 13177, 13178, -1, 13179, 13180, 13181, -1, 13182, 13183, 13184, -1, 13185, 13186, 13187, -1, 13188, 13189, 13190, -1, 13191, 13192, 13193, -1, 13194, 13195, 13196, -1, 13197, 13198, 13199, -1, 13200, 13201, 13202, -1, 13203, 13204, 13205, -1, 13206, 13207, 13208, -1, 13209, 13210, 13211, -1, 13212, 13213, 13214, -1, 13215, 13216, 13217, -1, 13218, 13219, 13220, -1, 13221, 13222, 13223, -1, 13224, 13225, 13226, -1, 13227, 13228, 13229, -1, 13230, 13231, 13232, -1, 13233, 13234, 13235, -1, 13236, 13237, 13238, -1, 13239, 13240, 13241, -1, 13242, 13243, 13244, -1, 13245, 13246, 13247, -1, 13248, 13249, 13250, -1, 13251, 13252, 13253, -1, 13254, 13255, 13256, -1, 13257, 13258, 13259, -1, 13260, 13261, 13262, -1, 13263, 13264, 13265, -1, 13266, 13267, 13268, -1, 13269, 13270, 13271, -1, 13272, 13273, 13274, -1, 13275, 13276, 13277, -1, 13278, 13279, 13280, -1, 13281, 13282, 13283, -1, 13284, 13285, 13286, -1, 13287, 13288, 13289, -1, 13290, 13291, 13292, -1, 13293, 13294, 13295, -1, 13296, 13297, 13298, -1, 13299, 13300, 13301, -1, 13302, 13303, 13304, -1, 13305, 13306, 13307, -1, 13308, 13309, 13310, -1, 13311, 13312, 13313, -1, 13314, 13315, 13316, -1, 13317, 13318, 13319, -1, 13320, 13321, 13322, -1, 13323, 13324, 13325, -1, 13326, 13327, 13328, -1, 13329, 13330, 13331, -1, 13332, 13333, 13334, -1, 13335, 13336, 13337, -1, 13338, 13339, 13340, -1, 13341, 13342, 13343, -1, 13344, 13345, 13346, -1, 13347, 13348, 13349, -1, 13350, 13351, 13352, -1, 13353, 13354, 13355, -1, 13356, 13357, 13358, -1, 13359, 13360, 13361, -1, 13362, 13363, 13364, -1, 13365, 13366, 13367, -1, 13368, 13369, 13370, -1, 13371, 13372, 13373, -1, 13374, 13375, 13376, -1, 13377, 13378, 13379, -1, 13380, 13381, 13382, -1, 13383, 13384, 13385, -1, 13386, 13387, 13388, -1, 13389, 13390, 13391, -1, 13392, 13393, 13394, -1, 13395, 13396, 13397, -1, 13398, 13399, 13400, -1, 13401, 13402, 13403, -1, 13404, 13405, 13406, -1, 13407, 13408, 13409, -1, 13410, 13411, 13412, -1, 13413, 13414, 13415, -1, 13416, 13417, 13418, -1, 13419, 13420, 13421, -1, 13422, 13423, 13424, -1, 13425, 13426, 13427, -1, 13428, 13429, 13430, -1, 13431, 13432, 13433, -1, 13434, 13435, 13436, -1, 13437, 13438, 13439, -1, 13440, 13441, 13442, -1, 13443, 13444, 13445, -1, 13446, 13447, 13448, -1, 13449, 13450, 13451, -1, 13452, 13453, 13454, -1, 13455, 13456, 13457, -1, 13458, 13459, 13460, -1, 13461, 13462, 13463, -1, 13464, 13465, 13466, -1, 13467, 13468, 13469, -1, 13470, 13471, 13472, -1, 13473, 13474, 13475, -1, 13476, 13477, 13478, -1, 13479, 13480, 13481, -1, 13482, 13483, 13484, -1, 13485, 13486, 13487, -1, 13488, 13489, 13490, -1, 13491, 13492, 13493, -1, 13494, 13495, 13496, -1, 13497, 13498, 13499, -1, 13500, 13501, 13502, -1, 13503, 13504, 13505, -1, 13506, 13507, 13508, -1, 13509, 13510, 13511, -1, 13512, 13513, 13514, -1, 13515, 13516, 13517, -1, 13518, 13519, 13520, -1, 13521, 13522, 13523, -1, 13524, 13525, 13526, -1, 13527, 13528, 13529, -1, 13530, 13531, 13532, -1, 13533, 13534, 13535, -1, 13536, 13537, 13538, -1, 13539, 13540, 13541, -1, 13542, 13543, 13544, -1, 13545, 13546, 13547, -1, 13548, 13549, 13550, -1, 13551, 13552, 13553, -1, 13554, 13555, 13556, -1, 13557, 13558, 13559, -1, 13560, 13561, 13562, -1, 13563, 13564, 13565, -1, 13566, 13567, 13568, -1, 13569, 13570, 13571, -1, 13572, 13573, 13574, -1, 13575, 13576, 13577, -1, 13578, 13579, 13580, -1, 13581, 13582, 13583, -1, 13584, 13585, 13586, -1, 13587, 13588, 13589, -1, 13590, 13591, 13592, -1, 13593, 13594, 13595, -1, 13596, 13597, 13598, -1, 13599, 13600, 13601, -1, 13602, 13603, 13604, -1, 13605, 13606, 13607, -1, 13608, 13609, 13610, -1, 13611, 13612, 13613, -1, 13614, 13615, 13616, -1, 13617, 13618, 13619, -1, 13620, 13621, 13622, -1, 13623, 13624, 13625, -1, 13626, 13627, 13628, -1, 13629, 13630, 13631, -1, 13632, 13633, 13634, -1, 13635, 13636, 13637, -1, 13638, 13639, 13640, -1, 13641, 13642, 13643, -1, 13644, 13645, 13646, -1, 13647, 13648, 13649, -1, 13650, 13651, 13652, -1, 13653, 13654, 13655, -1, 13656, 13657, 13658, -1, 13659, 13660, 13661, -1, 13662, 13663, 13664, -1, 13665, 13666, 13667, -1, 13668, 13669, 13670, -1, 13671, 13672, 13673, -1, 13674, 13675, 13676, -1, 13677, 13678, 13679, -1, 13680, 13681, 13682, -1, 13683, 13684, 13685, -1, 13686, 13687, 13688, -1, 13689, 13690, 13691, -1, 13692, 13693, 13694, -1, 13695, 13696, 13697, -1, 13698, 13699, 13700, -1, 13701, 13702, 13703, -1, 13704, 13705, 13706, -1, 13707, 13708, 13709, -1, 13710, 13711, 13712, -1, 13713, 13714, 13715, -1, 13716, 13717, 13718, -1, 13719, 13720, 13721, -1, 13722, 13723, 13724, -1, 13725, 13726, 13727, -1, 13728, 13729, 13730, -1, 13731, 13732, 13733, -1, 13734, 13735, 13736, -1, 13737, 13738, 13739, -1, 13740, 13741, 13742, -1, 13743, 13744, 13745, -1, 13746, 13747, 13748, -1, 13749, 13750, 13751, -1, 13752, 13753, 13754, -1, 13755, 13756, 13757, -1, 13758, 13759, 13760, -1, 13761, 13762, 13763, -1, 13764, 13765, 13766, -1, 13767, 13768, 13769, -1, 13770, 13771, 13772, -1, 13773, 13774, 13775, -1, 13776, 13777, 13778, -1, 13779, 13780, 13781, -1, 13782, 13783, 13784, -1, 13785, 13786, 13787, -1, 13788, 13789, 13790, -1, 13791, 13792, 13793, -1, 13794, 13795, 13796, -1, 13797, 13798, 13799, -1, 13800, 13801, 13802, -1, 13803, 13804, 13805, -1, 13806, 13807, 13808, -1, 13809, 13810, 13811, -1, 13812, 13813, 13814, -1, 13815, 13816, 13817, -1, 13818, 13819, 13820, -1, 13821, 13822, 13823, -1, 13824, 13825, 13826, -1, 13827, 13828, 13829, -1, 13830, 13831, 13832, -1, 13833, 13834, 13835, -1, 13836, 13837, 13838, -1, 13839, 13840, 13841, -1, 13842, 13843, 13844, -1, 13845, 13846, 13847, -1, 13848, 13849, 13850, -1, 13851, 13852, 13853, -1, 13854, 13855, 13856, -1, 13857, 13858, 13859, -1, 13860, 13861, 13862, -1, 13863, 13864, 13865, -1, 13866, 13867, 13868, -1, 13869, 13870, 13871, -1, 13872, 13873, 13874, -1, 13875, 13876, 13877, -1, 13878, 13879, 13880, -1, 13881, 13882, 13883, -1, 13884, 13885, 13886, -1, 13887, 13888, 13889, -1, 13890, 13891, 13892, -1, 13893, 13894, 13895, -1, 13896, 13897, 13898, -1, 13899, 13900, 13901, -1, 13902, 13903, 13904, -1, 13905, 13906, 13907, -1, 13908, 13909, 13910, -1, 13911, 13912, 13913, -1, 13914, 13915, 13916, -1, 13917, 13918, 13919, -1, 13920, 13921, 13922, -1, 13923, 13924, 13925, -1, 13926, 13927, 13928, -1, 13929, 13930, 13931, -1, 13932, 13933, 13934, -1, 13935, 13936, 13937, -1, 13938, 13939, 13940, -1, 13941, 13942, 13943, -1, 13944, 13945, 13946, -1, 13947, 13948, 13949, -1, 13950, 13951, 13952, -1, 13953, 13954, 13955, -1, 13956, 13957, 13958, -1, 13959, 13960, 13961, -1, 13962, 13963, 13964, -1, 13965, 13966, 13967, -1, 13968, 13969, 13970, -1, 13971, 13972, 13973, -1, 13974, 13975, 13976, -1, 13977, 13978, 13979, -1, 13980, 13981, 13982, -1, 13983, 13984, 13985, -1, 13986, 13987, 13988, -1, 13989, 13990, 13991, -1, 13992, 13993, 13994, -1, 13995, 13996, 13997, -1, 13998, 13999, 14000, -1, 14001, 14002, 14003, -1, 14004, 14005, 14006, -1, 14007, 14008, 14009, -1, 14010, 14011, 14012, -1, 14013, 14014, 14015, -1, 14016, 14017, 14018, -1, 14019, 14020, 14021, -1, 14022, 14023, 14024, -1, 14025, 14026, 14027, -1, 14028, 14029, 14030, -1, 14031, 14032, 14033, -1, 14034, 14035, 14036, -1, 14037, 14038, 14039, -1, 14040, 14041, 14042, -1, 14043, 14044, 14045, -1, 14046, 14047, 14048, -1, 14049, 14050, 14051, -1, 14052, 14053, 14054, -1, 14055, 14056, 14057, -1, 14058, 14059, 14060, -1, 14061, 14062, 14063, -1, 14064, 14065, 14066, -1, 14067, 14068, 14069, -1, 14070, 14071, 14072, -1, 14073, 14074, 14075, -1, 14076, 14077, 14078, -1, 14079, 14080, 14081, -1, 14082, 14083, 14084, -1, 14085, 14086, 14087, -1, 14088, 14089, 14090, -1, 14091, 14092, 14093, -1, 14094, 14095, 14096, -1, 14097, 14098, 14099, -1, 14100, 14101, 14102, -1, 14103, 14104, 14105, -1, 14106, 14107, 14108, -1, 14109, 14110, 14111, -1, 14112, 14113, 14114, -1, 14115, 14116, 14117, -1, 14118, 14119, 14120, -1, 14121, 14122, 14123, -1, 14124, 14125, 14126, -1, 14127, 14128, 14129, -1, 14130, 14131, 14132, -1, 14133, 14134, 14135, -1, 14136, 14137, 14138, -1, 14139, 14140, 14141, -1, 14142, 14143, 14144, -1, 14145, 14146, 14147, -1, 14148, 14149, 14150, -1, 14151, 14152, 14153, -1, 14154, 14155, 14156, -1, 14157, 14158, 14159, -1, 14160, 14161, 14162, -1, 14163, 14164, 14165, -1, 14166, 14167, 14168, -1, 14169, 14170, 14171, -1, 14172, 14173, 14174, -1, 14175, 14176, 14177, -1, 14178, 14179, 14180, -1, 14181, 14182, 14183, -1, 14184, 14185, 14186, -1, 14187, 14188, 14189, -1, 14190, 14191, 14192, -1, 14193, 14194, 14195, -1, 14196, 14197, 14198, -1, 14199, 14200, 14201, -1, 14202, 14203, 14204, -1, 14205, 14206, 14207, -1, 14208, 14209, 14210, -1, 14211, 14212, 14213, -1, 14214, 14215, 14216, -1, 14217, 14218, 14219, -1, 14220, 14221, 14222, -1, 14223, 14224, 14225, -1, 14226, 14227, 14228, -1, 14229, 14230, 14231, -1, 14232, 14233, 14234, -1, 14235, 14236, 14237, -1, 14238, 14239, 14240, -1, 14241, 14242, 14243, -1, 14244, 14245, 14246, -1, 14247, 14248, 14249, -1, 14250, 14251, 14252, -1, 14253, 14254, 14255, -1, 14256, 14257, 14258, -1, 14259, 14260, 14261, -1, 14262, 14263, 14264, -1, 14265, 14266, 14267, -1, 14268, 14269, 14270, -1, 14271, 14272, 14273, -1, 14274, 14275, 14276, -1, 14277, 14278, 14279, -1, 14280, 14281, 14282, -1, 14283, 14284, 14285, -1, 14286, 14287, 14288, -1, 14289, 14290, 14291, -1, 14292, 14293, 14294, -1, 14295, 14296, 14297, -1, 14298, 14299, 14300, -1, 14301, 14302, 14303, -1, 14304, 14305, 14306, -1, 14307, 14308, 14309, -1, 14310, 14311, 14312, -1, 14313, 14314, 14315, -1, 14316, 14317, 14318, -1, 14319, 14320, 14321, -1, 14322, 14323, 14324, -1, 14325, 14326, 14327, -1, 14328, 14329, 14330, -1, 14331, 14332, 14333, -1, 14334, 14335, 14336, -1, 14337, 14338, 14339, -1, 14340, 14341, 14342, -1, 14343, 14344, 14345, -1, 14346, 14347, 14348, -1, 14349, 14350, 14351, -1, 14352, 14353, 14354, -1, 14355, 14356, 14357, -1, 14358, 14359, 14360, -1, 14361, 14362, 14363, -1, 14364, 14365, 14366, -1, 14367, 14368, 14369, -1, 14370, 14371, 14372, -1, 14373, 14374, 14375, -1, 14376, 14377, 14378, -1, 14379, 14380, 14381, -1, 14382, 14383, 14384, -1, 14385, 14386, 14387, -1, 14388, 14389, 14390, -1, 14391, 14392, 14393, -1, 14394, 14395, 14396, -1, 14397, 14398, 14399, -1, 14400, 14401, 14402, -1, 14403, 14404, 14405, -1, 14406, 14407, 14408, -1, 14409, 14410, 14411, -1, 14412, 14413, 14414, -1, 14415, 14416, 14417, -1, 14418, 14419, 14420, -1, 14421, 14422, 14423, -1, 14424, 14425, 14426, -1, 14427, 14428, 14429, -1, 14430, 14431, 14432, -1, 14433, 14434, 14435, -1, 14436, 14437, 14438, -1, 14439, 14440, 14441, -1, 14442, 14443, 14444, -1, 14445, 14446, 14447, -1, 14448, 14449, 14450, -1, 14451, 14452, 14453, -1, 14454, 14455, 14456, -1, 14457, 14458, 14459, -1, 14460, 14461, 14462, -1, 14463, 14464, 14465, -1, 14466, 14467, 14468, -1, 14469, 14470, 14471, -1, 14472, 14473, 14474, -1, 14475, 14476, 14477, -1, 14478, 14479, 14480, -1, 14481, 14482, 14483, -1, 14484, 14485, 14486, -1, 14487, 14488, 14489, -1, 14490, 14491, 14492, -1, 14493, 14494, 14495, -1, 14496, 14497, 14498, -1, 14499, 14500, 14501, -1, 14502, 14503, 14504, -1, 14505, 14506, 14507, -1, 14508, 14509, 14510, -1, 14511, 14512, 14513, -1, 14514, 14515, 14516, -1, 14517, 14518, 14519, -1, 14520, 14521, 14522, -1, 14523, 14524, 14525, -1, 14526, 14527, 14528, -1, 14529, 14530, 14531, -1, 14532, 14533, 14534, -1, 14535, 14536, 14537, -1, 14538, 14539, 14540, -1, 14541, 14542, 14543, -1, 14544, 14545, 14546, -1, 14547, 14548, 14549, -1, 14550, 14551, 14552, -1, 14553, 14554, 14555, -1, 14556, 14557, 14558, -1, 14559, 14560, 14561, -1, 14562, 14563, 14564, -1, 14565, 14566, 14567, -1, 14568, 14569, 14570, -1, 14571, 14572, 14573, -1, 14574, 14575, 14576, -1, 14577, 14578, 14579, -1, 14580, 14581, 14582, -1, 14583, 14584, 14585, -1, 14586, 14587, 14588, -1, 14589, 14590, 14591, -1, 14592, 14593, 14594, -1, 14595, 14596, 14597, -1, 14598, 14599, 14600, -1, 14601, 14602, 14603, -1, 14604, 14605, 14606, -1, 14607, 14608, 14609, -1, 14610, 14611, 14612, -1, 14613, 14614, 14615, -1, 14616, 14617, 14618, -1, 14619, 14620, 14621, -1, 14622, 14623, 14624, -1, 14625, 14626, 14627, -1, 14628, 14629, 14630, -1, 14631, 14632, 14633, -1, 14634, 14635, 14636, -1, 14637, 14638, 14639, -1, 14640, 14641, 14642, -1, 14643, 14644, 14645, -1, 14646, 14647, 14648, -1, 14649, 14650, 14651, -1, 14652, 14653, 14654, -1, 14655, 14656, 14657, -1, 14658, 14659, 14660, -1, 14661, 14662, 14663, -1, 14664, 14665, 14666, -1, 14667, 14668, 14669, -1, 14670, 14671, 14672, -1, 14673, 14674, 14675, -1, 14676, 14677, 14678, -1, 14679, 14680, 14681, -1, 14682, 14683, 14684, -1, 14685, 14686, 14687, -1, 14688, 14689, 14690, -1, 14691, 14692, 14693, -1, 14694, 14695, 14696, -1, 14697, 14698, 14699, -1, 14700, 14701, 14702, -1, 14703, 14704, 14705, -1, 14706, 14707, 14708, -1, 14709, 14710, 14711, -1, 14712, 14713, 14714, -1, 14715, 14716, 14717, -1, 14718, 14719, 14720, -1, 14721, 14722, 14723, -1, 14724, 14725, 14726, -1, 14727, 14728, 14729, -1, 14730, 14731, 14732, -1, 14733, 14734, 14735, -1, 14736, 14737, 14738, -1, 14739, 14740, 14741, -1, 14742, 14743, 14744, -1, 14745, 14746, 14747, -1, 14748, 14749, 14750, -1, 14751, 14752, 14753, -1, 14754, 14755, 14756, -1, 14757, 14758, 14759, -1, 14760, 14761, 14762, -1, 14763, 14764, 14765, -1, 14766, 14767, 14768, -1, 14769, 14770, 14771, -1, 14772, 14773, 14774, -1, 14775, 14776, 14777, -1, 14778, 14779, 14780, -1, 14781, 14782, 14783, -1, 14784, 14785, 14786, -1, 14787, 14788, 14789, -1, 14790, 14791, 14792, -1, 14793, 14794, 14795, -1, 14796, 14797, 14798, -1, 14799, 14800, 14801, -1, 14802, 14803, 14804, -1, 14805, 14806, 14807, -1, 14808, 14809, 14810, -1, 14811, 14812, 14813, -1, 14814, 14815, 14816, -1, 14817, 14818, 14819, -1, 14820, 14821, 14822, -1, 14823, 14824, 14825, -1, 14826, 14827, 14828, -1, 14829, 14830, 14831, -1, 14832, 14833, 14834, -1, 14835, 14836, 14837, -1, 14838, 14839, 14840, -1, 14841, 14842, 14843, -1, 14844, 14845, 14846, -1, 14847, 14848, 14849, -1, 14850, 14851, 14852, -1, 14853, 14854, 14855, -1, 14856, 14857, 14858, -1, 14859, 14860, 14861, -1, 14862, 14863, 14864, -1, 14865, 14866, 14867, -1, 14868, 14869, 14870, -1, 14871, 14872, 14873, -1, 14874, 14875, 14876, -1, 14877, 14878, 14879, -1, 14880, 14881, 14882, -1, 14883, 14884, 14885, -1, 14886, 14887, 14888, -1, 14889, 14890, 14891, -1, 14892, 14893, 14894, -1, 14895, 14896, 14897, -1, 14898, 14899, 14900, -1, 14901, 14902, 14903, -1, 14904, 14905, 14906, -1, 14907, 14908, 14909, -1, 14910, 14911, 14912, -1, 14913, 14914, 14915, -1, 14916, 14917, 14918, -1, 14919, 14920, 14921, -1, 14922, 14923, 14924, -1, 14925, 14926, 14927, -1, 14928, 14929, 14930, -1, 14931, 14932, 14933, -1, 14934, 14935, 14936, -1, 14937, 14938, 14939, -1, 14940, 14941, 14942, -1, 14943, 14944, 14945, -1, 14946, 14947, 14948, -1, 14949, 14950, 14951, -1, 14952, 14953, 14954, -1, 14955, 14956, 14957, -1, 14958, 14959, 14960, -1, 14961, 14962, 14963, -1, 14964, 14965, 14966, -1, 14967, 14968, 14969, -1, 14970, 14971, 14972, -1, 14973, 14974, 14975, -1, 14976, 14977, 14978, -1, 14979, 14980, 14981, -1, 14982, 14983, 14984, -1, 14985, 14986, 14987, -1, 14988, 14989, 14990, -1, 14991, 14992, 14993, -1, 14994, 14995, 14996, -1, 14997, 14998, 14999, -1, 15000, 15001, 15002, -1, 15003, 15004, 15005, -1, 15006, 15007, 15008, -1, 15009, 15010, 15011, -1, 15012, 15013, 15014, -1, 15015, 15016, 15017, -1, 15018, 15019, 15020, -1, 15021, 15022, 15023, -1, 15024, 15025, 15026, -1, 15027, 15028, 15029, -1, 15030, 15031, 15032, -1, 15033, 15034, 15035, -1, 15036, 15037, 15038, -1, 15039, 15040, 15041, -1, 15042, 15043, 15044, -1, 15045, 15046, 15047, -1, 15048, 15049, 15050, -1, 15051, 15052, 15053, -1, 15054, 15055, 15056, -1, 15057, 15058, 15059, -1, 15060, 15061, 15062, -1, 15063, 15064, 15065, -1, 15066, 15067, 15068, -1, 15069, 15070, 15071, -1, 15072, 15073, 15074, -1, 15075, 15076, 15077, -1, 15078, 15079, 15080, -1, 15081, 15082, 15083, -1, 15084, 15085, 15086, -1, 15087, 15088, 15089, -1, 15090, 15091, 15092, -1, 15093, 15094, 15095, -1, 15096, 15097, 15098, -1, 15099, 15100, 15101, -1, 15102, 15103, 15104, -1, 15105, 15106, 15107, -1, 15108, 15109, 15110, -1, 15111, 15112, 15113, -1, 15114, 15115, 15116, -1, 15117, 15118, 15119, -1, 15120, 15121, 15122, -1, 15123, 15124, 15125, -1, 15126, 15127, 15128, -1, 15129, 15130, 15131, -1, 15132, 15133, 15134, -1, 15135, 15136, 15137, -1, 15138, 15139, 15140, -1, 15141, 15142, 15143, -1, 15144, 15145, 15146, -1, 15147, 15148, 15149, -1, 15150, 15151, 15152, -1, 15153, 15154, 15155, -1, 15156, 15157, 15158, -1, 15159, 15160, 15161, -1, 15162, 15163, 15164, -1, 15165, 15166, 15167, -1, 15168, 15169, 15170, -1, 15171, 15172, 15173, -1, 15174, 15175, 15176, -1, 15177, 15178, 15179, -1, 15180, 15181, 15182, -1, 15183, 15184, 15185, -1, 15186, 15187, 15188, -1, 15189, 15190, 15191, -1, 15192, 15193, 15194, -1, 15195, 15196, 15197, -1, 15198, 15199, 15200, -1, 15201, 15202, 15203, -1, 15204, 15205, 15206, -1, 15207, 15208, 15209, -1, 15210, 15211, 15212, -1, 15213, 15214, 15215, -1, 15216, 15217, 15218, -1, 15219, 15220, 15221, -1, 15222, 15223, 15224, -1, 15225, 15226, 15227, -1, 15228, 15229, 15230, -1, 15231, 15232, 15233, -1, 15234, 15235, 15236, -1, 15237, 15238, 15239, -1, 15240, 15241, 15242, -1, 15243, 15244, 15245, -1, 15246, 15247, 15248, -1, 15249, 15250, 15251, -1, 15252, 15253, 15254, -1, 15255, 15256, 15257, -1, 15258, 15259, 15260, -1, 15261, 15262, 15263, -1, 15264, 15265, 15266, -1, 15267, 15268, 15269, -1, 15270, 15271, 15272, -1, 15273, 15274, 15275, -1, 15276, 15277, 15278, -1, 15279, 15280, 15281, -1, 15282, 15283, 15284, -1, 15285, 15286, 15287, -1, 15288, 15289, 15290, -1, 15291, 15292, 15293, -1, 15294, 15295, 15296, -1, 15297, 15298, 15299, -1, 15300, 15301, 15302, -1, 15303, 15304, 15305, -1, 15306, 15307, 15308, -1, 15309, 15310, 15311, -1, 15312, 15313, 15314, -1, 15315, 15316, 15317, -1, 15318, 15319, 15320, -1, 15321, 15322, 15323, -1, 15324, 15325, 15326, -1, 15327, 15328, 15329, -1, 15330, 15331, 15332, -1, 15333, 15334, 15335, -1, 15336, 15337, 15338, -1, 15339, 15340, 15341, -1, 15342, 15343, 15344, -1, 15345, 15346, 15347, -1, 15348, 15349, 15350, -1, 15351, 15352, 15353, -1, 15354, 15355, 15356, -1, 15357, 15358, 15359, -1, 15360, 15361, 15362, -1, 15363, 15364, 15365, -1, 15366, 15367, 15368, -1, 15369, 15370, 15371, -1, 15372, 15373, 15374, -1, 15375, 15376, 15377, -1, 15378, 15379, 15380, -1, 15381, 15382, 15383, -1, 15384, 15385, 15386, -1, 15387, 15388, 15389, -1, 15390, 15391, 15392, -1, 15393, 15394, 15395, -1, 15396, 15397, 15398, -1, 15399, 15400, 15401, -1, 15402, 15403, 15404, -1, 15405, 15406, 15407, -1, 15408, 15409, 15410, -1, 15411, 15412, 15413, -1, 15414, 15415, 15416, -1, 15417, 15418, 15419, -1, 15420, 15421, 15422, -1, 15423, 15424, 15425, -1, 15426, 15427, 15428, -1, 15429, 15430, 15431, -1, 15432, 15433, 15434, -1, 15435, 15436, 15437, -1, 15438, 15439, 15440, -1, 15441, 15442, 15443, -1, 15444, 15445, 15446, -1, 15447, 15448, 15449, -1, 15450, 15451, 15452, -1, 15453, 15454, 15455, -1, 15456, 15457, 15458, -1, 15459, 15460, 15461, -1, 15462, 15463, 15464, -1, 15465, 15466, 15467, -1, 15468, 15469, 15470, -1, 15471, 15472, 15473, -1, 15474, 15475, 15476, -1, 15477, 15478, 15479, -1, 15480, 15481, 15482, -1, 15483, 15484, 15485, -1, 15486, 15487, 15488, -1, 15489, 15490, 15491, -1, 15492, 15493, 15494, -1, 15495, 15496, 15497, -1, 15498, 15499, 15500, -1, 15501, 15502, 15503, -1, 15504, 15505, 15506, -1, 15507, 15508, 15509, -1, 15510, 15511, 15512, -1, 15513, 15514, 15515, -1, 15516, 15517, 15518, -1, 15519, 15520, 15521, -1, 15522, 15523, 15524, -1, 15525, 15526, 15527, -1, 15528, 15529, 15530, -1, 15531, 15532, 15533, -1, 15534, 15535, 15536, -1, 15537, 15538, 15539, -1, 15540, 15541, 15542, -1, 15543, 15544, 15545, -1, 15546, 15547, 15548, -1, 15549, 15550, 15551, -1, 15552, 15553, 15554, -1, 15555, 15556, 15557, -1, 15558, 15559, 15560, -1, 15561, 15562, 15563, -1, 15564, 15565, 15566, -1, 15567, 15568, 15569, -1, 15570, 15571, 15572, -1, 15573, 15574, 15575, -1, 15576, 15577, 15578, -1, 15579, 15580, 15581, -1, 15582, 15583, 15584, -1, 15585, 15586, 15587, -1, 15588, 15589, 15590, -1, 15591, 15592, 15593, -1, 15594, 15595, 15596, -1, 15597, 15598, 15599, -1, 15600, 15601, 15602, -1, 15603, 15604, 15605, -1, 15606, 15607, 15608, -1, 15609, 15610, 15611, -1, 15612, 15613, 15614, -1, 15615, 15616, 15617, -1, 15618, 15619, 15620, -1, 15621, 15622, 15623, -1, 15624, 15625, 15626, -1, 15627, 15628, 15629, -1, 15630, 15631, 15632, -1, 15633, 15634, 15635, -1, 15636, 15637, 15638, -1, 15639, 15640, 15641, -1, 15642, 15643, 15644, -1, 15645, 15646, 15647, -1, 15648, 15649, 15650, -1, 15651, 15652, 15653, -1, 15654, 15655, 15656, -1, 15657, 15658, 15659, -1, 15660, 15661, 15662, -1, 15663, 15664, 15665, -1, 15666, 15667, 15668, -1, 15669, 15670, 15671, -1, 15672, 15673, 15674, -1, 15675, 15676, 15677, -1, 15678, 15679, 15680, -1, 15681, 15682, 15683, -1, 15684, 15685, 15686, -1, 15687, 15688, 15689, -1, 15690, 15691, 15692, -1, 15693, 15694, 15695, -1, 15696, 15697, 15698, -1, 15699, 15700, 15701, -1, 15702, 15703, 15704, -1, 15705, 15706, 15707, -1, 15708, 15709, 15710, -1, 15711, 15712, 15713, -1, 15714, 15715, 15716, -1, 15717, 15718, 15719, -1, 15720, 15721, 15722, -1, 15723, 15724, 15725, -1, 15726, 15727, 15728, -1, 15729, 15730, 15731, -1, 15732, 15733, 15734, -1, 15735, 15736, 15737, -1, 15738, 15739, 15740, -1, 15741, 15742, 15743, -1, 15744, 15745, 15746, -1, 15747, 15748, 15749, -1, 15750, 15751, 15752, -1, 15753, 15754, 15755, -1, 15756, 15757, 15758, -1, 15759, 15760, 15761, -1, 15762, 15763, 15764, -1, 15765, 15766, 15767, -1, 15768, 15769, 15770, -1, 15771, 15772, 15773, -1, 15774, 15775, 15776, -1, 15777, 15778, 15779, -1, 15780, 15781, 15782, -1, 15783, 15784, 15785, -1, 15786, 15787, 15788, -1, 15789, 15790, 15791, -1, 15792, 15793, 15794, -1, 15795, 15796, 15797, -1, 15798, 15799, 15800, -1, 15801, 15802, 15803, -1, 15804, 15805, 15806, -1, 15807, 15808, 15809, -1, 15810, 15811, 15812, -1, 15813, 15814, 15815, -1, 15816, 15817, 15818, -1, 15819, 15820, 15821, -1, 15822, 15823, 15824, -1, 15825, 15826, 15827, -1, 15828, 15829, 15830, -1, 15831, 15832, 15833, -1, 15834, 15835, 15836, -1, 15837, 15838, 15839, -1, 15840, 15841, 15842, -1, 15843, 15844, 15845, -1, 15846, 15847, 15848, -1, 15849, 15850, 15851, -1, 15852, 15853, 15854, -1, 15855, 15856, 15857, -1, 15858, 15859, 15860, -1, 15861, 15862, 15863, -1, 15864, 15865, 15866, -1, 15867, 15868, 15869, -1, 15870, 15871, 15872, -1, 15873, 15874, 15875, -1, 15876, 15877, 15878, -1, 15879, 15880, 15881, -1, 15882, 15883, 15884, -1, 15885, 15886, 15887, -1, 15888, 15889, 15890, -1, 15891, 15892, 15893, -1, 15894, 15895, 15896, -1, 15897, 15898, 15899, -1, 15900, 15901, 15902, -1, 15903, 15904, 15905, -1, 15906, 15907, 15908, -1, 15909, 15910, 15911, -1, 15912, 15913, 15914, -1, 15915, 15916, 15917, -1, 15918, 15919, 15920, -1, 15921, 15922, 15923, -1, 15924, 15925, 15926, -1, 15927, 15928, 15929, -1, 15930, 15931, 15932, -1, 15933, 15934, 15935, -1, 15936, 15937, 15938, -1, 15939, 15940, 15941, -1, 15942, 15943, 15944, -1, 15945, 15946, 15947, -1, 15948, 15949, 15950, -1, 15951, 15952, 15953, -1, 15954, 15955, 15956, -1, 15957, 15958, 15959, -1, 15960, 15961, 15962, -1, 15963, 15964, 15965, -1, 15966, 15967, 15968, -1, 15969, 15970, 15971, -1, 15972, 15973, 15974, -1, 15975, 15976, 15977, -1, 15978, 15979, 15980, -1, 15981, 15982, 15983, -1, 15984, 15985, 15986, -1, 15987, 15988, 15989, -1, 15990, 15991, 15992, -1, 15993, 15994, 15995, -1, 15996, 15997, 15998, -1, 15999, 16000, 16001, -1, 16002, 16003, 16004, -1, 16005, 16006, 16007, -1, 16008, 16009, 16010, -1, 16011, 16012, 16013, -1, 16014, 16015, 16016, -1, 16017, 16018, 16019, -1, 16020, 16021, 16022, -1, 16023, 16024, 16025, -1, 16026, 16027, 16028, -1, 16029, 16030, 16031, -1, 16032, 16033, 16034, -1, 16035, 16036, 16037, -1, 16038, 16039, 16040, -1, 16041, 16042, 16043, -1, 16044, 16045, 16046, -1, 16047, 16048, 16049, -1, 16050, 16051, 16052, -1, 16053, 16054, 16055, -1, 16056, 16057, 16058, -1, 16059, 16060, 16061, -1, 16062, 16063, 16064, -1, 16065, 16066, 16067, -1, 16068, 16069, 16070, -1, 16071, 16072, 16073, -1, 16074, 16075, 16076, -1, 16077, 16078, 16079, -1, 16080, 16081, 16082, -1, 16083, 16084, 16085, -1, 16086, 16087, 16088, -1, 16089, 16090, 16091, -1, 16092, 16093, 16094, -1, 16095, 16096, 16097, -1, 16098, 16099, 16100, -1, 16101, 16102, 16103, -1, 16104, 16105, 16106, -1, 16107, 16108, 16109, -1, 16110, 16111, 16112, -1, 16113, 16114, 16115, -1, 16116, 16117, 16118, -1, 16119, 16120, 16121, -1, 16122, 16123, 16124, -1, 16125, 16126, 16127, -1, 16128, 16129, 16130, -1, 16131, 16132, 16133, -1, 16134, 16135, 16136, -1, 16137, 16138, 16139, -1, 16140, 16141, 16142, -1, 16143, 16144, 16145, -1, 16146, 16147, 16148, -1, 16149, 16150, 16151, -1, 16152, 16153, 16154, -1, 16155, 16156, 16157, -1, 16158, 16159, 16160, -1, 16161, 16162, 16163, -1, 16164, 16165, 16166, -1, 16167, 16168, 16169, -1, 16170, 16171, 16172, -1, 16173, 16174, 16175, -1, 16176, 16177, 16178, -1, 16179, 16180, 16181, -1, 16182, 16183, 16184, -1, 16185, 16186, 16187, -1, 16188, 16189, 16190, -1, 16191, 16192, 16193, -1, 16194, 16195, 16196, -1, 16197, 16198, 16199, -1, 16200, 16201, 16202, -1, 16203, 16204, 16205, -1, 16206, 16207, 16208, -1, 16209, 16210, 16211, -1, 16212, 16213, 16214, -1, 16215, 16216, 16217, -1, 16218, 16219, 16220, -1, 16221, 16222, 16223, -1, 16224, 16225, 16226, -1, 16227, 16228, 16229, -1, 16230, 16231, 16232, -1, 16233, 16234, 16235, -1, 16236, 16237, 16238, -1, 16239, 16240, 16241, -1, 16242, 16243, 16244, -1, 16245, 16246, 16247, -1, 16248, 16249, 16250, -1, 16251, 16252, 16253, -1, 16254, 16255, 16256, -1, 16257, 16258, 16259, -1, 16260, 16261, 16262, -1, 16263, 16264, 16265, -1, 16266, 16267, 16268, -1, 16269, 16270, 16271, -1, 16272, 16273, 16274, -1, 16275, 16276, 16277, -1, 16278, 16279, 16280, -1, 16281, 16282, 16283, -1, 16284, 16285, 16286, -1, 16287, 16288, 16289, -1, 16290, 16291, 16292, -1, 16293, 16294, 16295, -1, 16296, 16297, 16298, -1, 16299, 16300, 16301, -1, 16302, 16303, 16304, -1, 16305, 16306, 16307, -1, 16308, 16309, 16310, -1, 16311, 16312, 16313, -1, 16314, 16315, 16316, -1, 16317, 16318, 16319, -1, 16320, 16321, 16322, -1, 16323, 16324, 16325, -1, 16326, 16327, 16328, -1, 16329, 16330, 16331, -1, 16332, 16333, 16334, -1, 16335, 16336, 16337, -1, 16338, 16339, 16340, -1, 16341, 16342, 16343, -1, 16344, 16345, 16346, -1, 16347, 16348, 16349, -1, 16350, 16351, 16352, -1, 16353, 16354, 16355, -1, 16356, 16357, 16358, -1, 16359, 16360, 16361, -1, 16362, 16363, 16364, -1, 16365, 16366, 16367, -1, 16368, 16369, 16370, -1, 16371, 16372, 16373, -1, 16374, 16375, 16376, -1, 16377, 16378, 16379, -1, 16380, 16381, 16382, -1, 16383, 16384, 16385, -1, 16386, 16387, 16388, -1, 16389, 16390, 16391, -1, 16392, 16393, 16394, -1, 16395, 16396, 16397, -1, 16398, 16399, 16400, -1, 16401, 16402, 16403, -1, 16404, 16405, 16406, -1, 16407, 16408, 16409, -1, 16410, 16411, 16412, -1, 16413, 16414, 16415, -1, 16416, 16417, 16418, -1, 16419, 16420, 16421, -1, 16422, 16423, 16424, -1, 16425, 16426, 16427, -1, 16428, 16429, 16430, -1, 16431, 16432, 16433, -1, 16434, 16435, 16436, -1, 16437, 16438, 16439, -1, 16440, 16441, 16442, -1, 16443, 16444, 16445, -1, 16446, 16447, 16448, -1, 16449, 16450, 16451, -1, 16452, 16453, 16454, -1, 16455, 16456, 16457, -1, 16458, 16459, 16460, -1, 16461, 16462, 16463, -1, 16464, 16465, 16466, -1, 16467, 16468, 16469, -1, 16470, 16471, 16472, -1, 16473, 16474, 16475, -1, 16476, 16477, 16478, -1, 16479, 16480, 16481, -1, 16482, 16483, 16484, -1, 16485, 16486, 16487, -1, 16488, 16489, 16490, -1, 16491, 16492, 16493, -1, 16494, 16495, 16496, -1, 16497, 16498, 16499, -1, 16500, 16501, 16502, -1, 16503, 16504, 16505, -1, 16506, 16507, 16508, -1, 16509, 16510, 16511, -1, 16512, 16513, 16514, -1, 16515, 16516, 16517, -1, 16518, 16519, 16520, -1, 16521, 16522, 16523, -1, 16524, 16525, 16526, -1, 16527, 16528, 16529, -1, 16530, 16531, 16532, -1, 16533, 16534, 16535, -1, 16536, 16537, 16538, -1, 16539, 16540, 16541, -1, 16542, 16543, 16544, -1, 16545, 16546, 16547, -1, 16548, 16549, 16550, -1, 16551, 16552, 16553, -1, 16554, 16555, 16556, -1, 16557, 16558, 16559, -1, 16560, 16561, 16562, -1, 16563, 16564, 16565, -1, 16566, 16567, 16568, -1, 16569, 16570, 16571, -1, 16572, 16573, 16574, -1, 16575, 16576, 16577, -1, 16578, 16579, 16580, -1, 16581, 16582, 16583, -1, 16584, 16585, 16586, -1, 16587, 16588, 16589, -1, 16590, 16591, 16592, -1, 16593, 16594, 16595, -1, 16596, 16597, 16598, -1, 16599, 16600, 16601, -1, 16602, 16603, 16604, -1, 16605, 16606, 16607, -1, 16608, 16609, 16610, -1, 16611, 16612, 16613, -1, 16614, 16615, 16616, -1, 16617, 16618, 16619, -1, 16620, 16621, 16622, -1, 16623, 16624, 16625, -1, 16626, 16627, 16628, -1, 16629, 16630, 16631, -1, 16632, 16633, 16634, -1, 16635, 16636, 16637, -1, 16638, 16639, 16640, -1, 16641, 16642, 16643, -1, 16644, 16645, 16646, -1, 16647, 16648, 16649, -1, 16650, 16651, 16652, -1, 16653, 16654, 16655, -1, 16656, 16657, 16658, -1, 16659, 16660, 16661, -1, 16662, 16663, 16664, -1, 16665, 16666, 16667, -1, 16668, 16669, 16670, -1, 16671, 16672, 16673, -1, 16674, 16675, 16676, -1, 16677, 16678, 16679, -1, 16680, 16681, 16682, -1, 16683, 16684, 16685, -1, 16686, 16687, 16688, -1, 16689, 16690, 16691, -1, 16692, 16693, 16694, -1, 16695, 16696, 16697, -1, 16698, 16699, 16700, -1, 16701, 16702, 16703, -1, 16704, 16705, 16706, -1, 16707, 16708, 16709, -1, 16710, 16711, 16712, -1, 16713, 16714, 16715, -1, 16716, 16717, 16718, -1, 16719, 16720, 16721, -1, 16722, 16723, 16724, -1, 16725, 16726, 16727, -1, 16728, 16729, 16730, -1, 16731, 16732, 16733, -1, 16734, 16735, 16736, -1, 16737, 16738, 16739, -1, 16740, 16741, 16742, -1, 16743, 16744, 16745, -1, 16746, 16747, 16748, -1, 16749, 16750, 16751, -1, 16752, 16753, 16754, -1, 16755, 16756, 16757, -1, 16758, 16759, 16760, -1, 16761, 16762, 16763, -1, 16764, 16765, 16766, -1, 16767, 16768, 16769, -1, 16770, 16771, 16772, -1, 16773, 16774, 16775, -1, 16776, 16777, 16778, -1, 16779, 16780, 16781, -1, 16782, 16783, 16784, -1, 16785, 16786, 16787, -1, 16788, 16789, 16790, -1, 16791, 16792, 16793, -1, 16794, 16795, 16796, -1, 16797, 16798, 16799, -1, 16800, 16801, 16802, -1, 16803, 16804, 16805, -1, 16806, 16807, 16808, -1, 16809, 16810, 16811, -1, 16812, 16813, 16814, -1, 16815, 16816, 16817, -1, 16818, 16819, 16820, -1, 16821, 16822, 16823, -1, 16824, 16825, 16826, -1, 16827, 16828, 16829, -1, 16830, 16831, 16832, -1, 16833, 16834, 16835, -1, 16836, 16837, 16838, -1, 16839, 16840, 16841, -1, 16842, 16843, 16844, -1, 16845, 16846, 16847, -1, 16848, 16849, 16850, -1, 16851, 16852, 16853, -1, 16854, 16855, 16856, -1, 16857, 16858, 16859, -1, 16860, 16861, 16862, -1, 16863, 16864, 16865, -1, 16866, 16867, 16868, -1, 16869, 16870, 16871, -1, 16872, 16873, 16874, -1, 16875, 16876, 16877, -1, 16878, 16879, 16880, -1, 16881, 16882, 16883, -1, 16884, 16885, 16886, -1, 16887, 16888, 16889, -1, 16890, 16891, 16892, -1, 16893, 16894, 16895, -1, 16896, 16897, 16898, -1, 16899, 16900, 16901, -1, 16902, 16903, 16904, -1, 16905, 16906, 16907, -1, 16908, 16909, 16910, -1, 16911, 16912, 16913, -1, 16914, 16915, 16916, -1, 16917, 16918, 16919, -1, 16920, 16921, 16922, -1, 16923, 16924, 16925, -1, 16926, 16927, 16928, -1, 16929, 16930, 16931, -1, 16932, 16933, 16934, -1, 16935, 16936, 16937, -1, 16938, 16939, 16940, -1, 16941, 16942, 16943, -1, 16944, 16945, 16946, -1, 16947, 16948, 16949, -1, 16950, 16951, 16952, -1, 16953, 16954, 16955, -1, 16956, 16957, 16958, -1, 16959, 16960, 16961, -1, 16962, 16963, 16964, -1, 16965, 16966, 16967, -1, 16968, 16969, 16970, -1, 16971, 16972, 16973, -1, 16974, 16975, 16976, -1, 16977, 16978, 16979, -1, 16980, 16981, 16982, -1, 16983, 16984, 16985, -1, 16986, 16987, 16988, -1, 16989, 16990, 16991, -1, 16992, 16993, 16994, -1, 16995, 16996, 16997, -1, 16998, 16999, 17000, -1, 17001, 17002, 17003, -1, 17004, 17005, 17006, -1, 17007, 17008, 17009, -1, 17010, 17011, 17012, -1, 17013, 17014, 17015, -1, 17016, 17017, 17018, -1, 17019, 17020, 17021, -1, 17022, 17023, 17024, -1, 17025, 17026, 17027, -1, 17028, 17029, 17030, -1, 17031, 17032, 17033, -1, 17034, 17035, 17036, -1, 17037, 17038, 17039, -1, 17040, 17041, 17042, -1, 17043, 17044, 17045, -1, 17046, 17047, 17048, -1, 17049, 17050, 17051, -1, 17052, 17053, 17054, -1, 17055, 17056, 17057, -1, 17058, 17059, 17060, -1, 17061, 17062, 17063, -1, 17064, 17065, 17066, -1, 17067, 17068, 17069, -1, 17070, 17071, 17072, -1, 17073, 17074, 17075, -1, 17076, 17077, 17078, -1, 17079, 17080, 17081, -1, 17082, 17083, 17084, -1, 17085, 17086, 17087, -1, 17088, 17089, 17090, -1, 17091, 17092, 17093, -1, 17094, 17095, 17096, -1, 17097, 17098, 17099, -1, 17100, 17101, 17102, -1, 17103, 17104, 17105, -1, 17106, 17107, 17108, -1, 17109, 17110, 17111, -1, 17112, 17113, 17114, -1, 17115, 17116, 17117, -1, 17118, 17119, 17120, -1, 17121, 17122, 17123, -1, 17124, 17125, 17126, -1, 17127, 17128, 17129, -1, 17130, 17131, 17132, -1, 17133, 17134, 17135, -1, 17136, 17137, 17138, -1, 17139, 17140, 17141, -1, 17142, 17143, 17144, -1, 17145, 17146, 17147, -1, 17148, 17149, 17150, -1, 17151, 17152, 17153, -1, 17154, 17155, 17156, -1, 17157, 17158, 17159, -1, 17160, 17161, 17162, -1, 17163, 17164, 17165, -1, 17166, 17167, 17168, -1, 17169, 17170, 17171, -1, 17172, 17173, 17174, -1, 17175, 17176, 17177, -1, 17178, 17179, 17180, -1, 17181, 17182, 17183, -1, 17184, 17185, 17186, -1, 17187, 17188, 17189, -1, 17190, 17191, 17192, -1, 17193, 17194, 17195, -1, 17196, 17197, 17198, -1, 17199, 17200, 17201, -1, 17202, 17203, 17204, -1, 17205, 17206, 17207, -1, 17208, 17209, 17210, -1, 17211, 17212, 17213, -1, 17214, 17215, 17216, -1, 17217, 17218, 17219, -1, 17220, 17221, 17222, -1, 17223, 17224, 17225, -1, 17226, 17227, 17228, -1, 17229, 17230, 17231, -1, 17232, 17233, 17234, -1, 17235, 17236, 17237, -1, 17238, 17239, 17240, -1, 17241, 17242, 17243, -1, 17244, 17245, 17246, -1, 17247, 17248, 17249, -1, 17250, 17251, 17252, -1, 17253, 17254, 17255, -1, 17256, 17257, 17258, -1, 17259, 17260, 17261, -1, 17262, 17263, 17264, -1, 17265, 17266, 17267, -1, 17268, 17269, 17270, -1, 17271, 17272, 17273, -1, 17274, 17275, 17276, -1, 17277, 17278, 17279, -1, 17280, 17281, 17282, -1, 17283, 17284, 17285, -1, 17286, 17287, 17288, -1, 17289, 17290, 17291, -1, 17292, 17293, 17294, -1, 17295, 17296, 17297, -1, 17298, 17299, 17300, -1, 17301, 17302, 17303, -1, 17304, 17305, 17306, -1, 17307, 17308, 17309, -1, 17310, 17311, 17312, -1, 17313, 17314, 17315, -1, 17316, 17317, 17318, -1, 17319, 17320, 17321, -1, 17322, 17323, 17324, -1, 17325, 17326, 17327, -1, 17328, 17329, 17330, -1, 17331, 17332, 17333, -1, 17334, 17335, 17336, -1, 17337, 17338, 17339, -1, 17340, 17341, 17342, -1, 17343, 17344, 17345, -1, 17346, 17347, 17348, -1, 17349, 17350, 17351, -1, 17352, 17353, 17354, -1, 17355, 17356, 17357, -1, 17358, 17359, 17360, -1, 17361, 17362, 17363, -1, 17364, 17365, 17366, -1, 17367, 17368, 17369, -1, 17370, 17371, 17372, -1, 17373, 17374, 17375, -1, 17376, 17377, 17378, -1, 17379, 17380, 17381, -1, 17382, 17383, 17384, -1, 17385, 17386, 17387, -1, 17388, 17389, 17390, -1, 17391, 17392, 17393, -1, 17394, 17395, 17396, -1, 17397, 17398, 17399, -1, 17400, 17401, 17402, -1, 17403, 17404, 17405, -1, 17406, 17407, 17408, -1, 17409, 17410, 17411, -1, 17412, 17413, 17414, -1, 17415, 17416, 17417, -1, 17418, 17419, 17420, -1, 17421, 17422, 17423, -1, 17424, 17425, 17426, -1, 17427, 17428, 17429, -1, 17430, 17431, 17432, -1, 17433, 17434, 17435, -1, 17436, 17437, 17438, -1, 17439, 17440, 17441, -1, 17442, 17443, 17444, -1, 17445, 17446, 17447, -1, 17448, 17449, 17450, -1, 17451, 17452, 17453, -1, 17454, 17455, 17456, -1, 17457, 17458, 17459, -1, 17460, 17461, 17462, -1, 17463, 17464, 17465, -1, 17466, 17467, 17468, -1, 17469, 17470, 17471, -1, 17472, 17473, 17474, -1, 17475, 17476, 17477, -1, 17478, 17479, 17480, -1, 17481, 17482, 17483, -1, 17484, 17485, 17486, -1, 17487, 17488, 17489, -1, 17490, 17491, 17492, -1, 17493, 17494, 17495, -1, 17496, 17497, 17498, -1, 17499, 17500, 17501, -1, 17502, 17503, 17504, -1, 17505, 17506, 17507, -1, 17508, 17509, 17510, -1, 17511, 17512, 17513, -1, 17514, 17515, 17516, -1, 17517, 17518, 17519, -1, 17520, 17521, 17522, -1, 17523, 17524, 17525, -1, 17526, 17527, 17528, -1, 17529, 17530, 17531, -1, 17532, 17533, 17534, -1, 17535, 17536, 17537, -1, 17538, 17539, 17540, -1, 17541, 17542, 17543, -1, 17544, 17545, 17546, -1, 17547, 17548, 17549, -1, 17550, 17551, 17552, -1, 17553, 17554, 17555, -1, 17556, 17557, 17558, -1, 17559, 17560, 17561, -1, 17562, 17563, 17564, -1, 17565, 17566, 17567, -1, 17568, 17569, 17570, -1, 17571, 17572, 17573, -1, 17574, 17575, 17576, -1, 17577, 17578, 17579, -1, 17580, 17581, 17582, -1, 17583, 17584, 17585, -1, 17586, 17587, 17588, -1, 17589, 17590, 17591, -1, 17592, 17593, 17594, -1, 17595, 17596, 17597, -1, 17598, 17599, 17600, -1, 17601, 17602, 17603, -1, 17604, 17605, 17606, -1, 17607, 17608, 17609, -1, 17610, 17611, 17612, -1, 17613, 17614, 17615, -1, 17616, 17617, 17618, -1, 17619, 17620, 17621, -1, 17622, 17623, 17624, -1, 17625, 17626, 17627, -1, 17628, 17629, 17630, -1, 17631, 17632, 17633, -1, 17634, 17635, 17636, -1, 17637, 17638, 17639, -1, 17640, 17641, 17642, -1, 17643, 17644, 17645, -1, 17646, 17647, 17648, -1, 17649, 17650, 17651, -1, 17652, 17653, 17654, -1, 17655, 17656, 17657, -1, 17658, 17659, 17660, -1, 17661, 17662, 17663, -1, 17664, 17665, 17666, -1, 17667, 17668, 17669, -1, 17670, 17671, 17672, -1, 17673, 17674, 17675, -1, 17676, 17677, 17678, -1, 17679, 17680, 17681, -1, 17682, 17683, 17684, -1, 17685, 17686, 17687, -1, 17688, 17689, 17690, -1, 17691, 17692, 17693, -1, 17694, 17695, 17696, -1, 17697, 17698, 17699, -1, 17700, 17701, 17702, -1, 17703, 17704, 17705, -1, 17706, 17707, 17708, -1, 17709, 17710, 17711, -1, 17712, 17713, 17714, -1, 17715, 17716, 17717, -1, 17718, 17719, 17720, -1, 17721, 17722, 17723, -1, 17724, 17725, 17726, -1, 17727, 17728, 17729, -1, 17730, 17731, 17732, -1, 17733, 17734, 17735, -1, 17736, 17737, 17738, -1, 17739, 17740, 17741, -1, 17742, 17743, 17744, -1, 17745, 17746, 17747, -1, 17748, 17749, 17750, -1, 17751, 17752, 17753, -1, 17754, 17755, 17756, -1, 17757, 17758, 17759, -1, 17760, 17761, 17762, -1, 17763, 17764, 17765, -1, 17766, 17767, 17768, -1, 17769, 17770, 17771, -1, 17772, 17773, 17774, -1, 17775, 17776, 17777, -1, 17778, 17779, 17780, -1, 17781, 17782, 17783, -1, 17784, 17785, 17786, -1, 17787, 17788, 17789, -1, 17790, 17791, 17792, -1, 17793, 17794, 17795, -1, 17796, 17797, 17798, -1, 17799, 17800, 17801, -1, 17802, 17803, 17804, -1, 17805, 17806, 17807, -1, 17808, 17809, 17810, -1, 17811, 17812, 17813, -1, 17814, 17815, 17816, -1, 17817, 17818, 17819, -1, 17820, 17821, 17822, -1, 17823, 17824, 17825, -1, 17826, 17827, 17828, -1, 17829, 17830, 17831, -1, 17832, 17833, 17834, -1, 17835, 17836, 17837, -1, 17838, 17839, 17840, -1, 17841, 17842, 17843, -1, 17844, 17845, 17846, -1, 17847, 17848, 17849, -1, 17850, 17851, 17852, -1, 17853, 17854, 17855, -1, 17856, 17857, 17858, -1, 17859, 17860, 17861, -1, 17862, 17863, 17864, -1, 17865, 17866, 17867, -1, 17868, 17869, 17870, -1, 17871, 17872, 17873, -1, 17874, 17875, 17876, -1, 17877, 17878, 17879, -1, 17880, 17881, 17882, -1, 17883, 17884, 17885, -1, 17886, 17887, 17888, -1, 17889, 17890, 17891, -1, 17892, 17893, 17894, -1, 17895, 17896, 17897, -1, 17898, 17899, 17900, -1, 17901, 17902, 17903, -1, 17904, 17905, 17906, -1, 17907, 17908, 17909, -1, 17910, 17911, 17912, -1, 17913, 17914, 17915, -1, 17916, 17917, 17918, -1, 17919, 17920, 17921, -1, 17922, 17923, 17924, -1, 17925, 17926, 17927, -1, 17928, 17929, 17930, -1, 17931, 17932, 17933, -1, 17934, 17935, 17936, -1, 17937, 17938, 17939, -1, 17940, 17941, 17942, -1, 17943, 17944, 17945, -1, 17946, 17947, 17948, -1, 17949, 17950, 17951, -1, 17952, 17953, 17954, -1, 17955, 17956, 17957, -1, 17958, 17959, 17960, -1, 17961, 17962, 17963, -1, 17964, 17965, 17966, -1, 17967, 17968, 17969, -1, 17970, 17971, 17972, -1, 17973, 17974, 17975, -1, 17976, 17977, 17978, -1, 17979, 17980, 17981, -1, 17982, 17983, 17984, -1, 17985, 17986, 17987, -1, 17988, 17989, 17990, -1, 17991, 17992, 17993, -1, 17994, 17995, 17996, -1, 17997, 17998, 17999, -1, 18000, 18001, 18002, -1, 18003, 18004, 18005, -1, 18006, 18007, 18008, -1, 18009, 18010, 18011, -1, 18012, 18013, 18014, -1, 18015, 18016, 18017, -1, 18018, 18019, 18020, -1, 18021, 18022, 18023, -1, 18024, 18025, 18026, -1, 18027, 18028, 18029, -1, 18030, 18031, 18032, -1, 18033, 18034, 18035, -1, 18036, 18037, 18038, -1, 18039, 18040, 18041, -1, 18042, 18043, 18044, -1, 18045, 18046, 18047, -1, 18048, 18049, 18050, -1, 18051, 18052, 18053, -1, 18054, 18055, 18056, -1, 18057, 18058, 18059, -1, 18060, 18061, 18062, -1, 18063, 18064, 18065, -1, 18066, 18067, 18068, -1, 18069, 18070, 18071, -1, 18072, 18073, 18074, -1, 18075, 18076, 18077, -1, 18078, 18079, 18080, -1, 18081, 18082, 18083, -1, 18084, 18085, 18086, -1, 18087, 18088, 18089, -1, 18090, 18091, 18092, -1, 18093, 18094, 18095, -1, 18096, 18097, 18098, -1, 18099, 18100, 18101, -1, 18102, 18103, 18104, -1, 18105, 18106, 18107, -1, 18108, 18109, 18110, -1, 18111, 18112, 18113, -1, 18114, 18115, 18116, -1, 18117, 18118, 18119, -1, 18120, 18121, 18122, -1, 18123, 18124, 18125, -1, 18126, 18127, 18128, -1, 18129, 18130, 18131, -1, 18132, 18133, 18134, -1, 18135, 18136, 18137, -1, 18138, 18139, 18140, -1, 18141, 18142, 18143, -1, 18144, 18145, 18146, -1, 18147, 18148, 18149, -1, 18150, 18151, 18152, -1, 18153, 18154, 18155, -1, 18156, 18157, 18158, -1, 18159, 18160, 18161, -1, 18162, 18163, 18164, -1, 18165, 18166, 18167, -1, 18168, 18169, 18170, -1, 18171, 18172, 18173, -1, 18174, 18175, 18176, -1, 18177, 18178, 18179, -1, 18180, 18181, 18182, -1, 18183, 18184, 18185, -1, 18186, 18187, 18188, -1, 18189, 18190, 18191, -1, 18192, 18193, 18194, -1, 18195, 18196, 18197, -1, 18198, 18199, 18200, -1, 18201, 18202, 18203, -1, 18204, 18205, 18206, -1, 18207, 18208, 18209, -1, 18210, 18211, 18212, -1, 18213, 18214, 18215, -1, 18216, 18217, 18218, -1, 18219, 18220, 18221, -1, 18222, 18223, 18224, -1, 18225, 18226, 18227, -1, 18228, 18229, 18230, -1, 18231, 18232, 18233, -1, 18234, 18235, 18236, -1, 18237, 18238, 18239, -1, 18240, 18241, 18242, -1, 18243, 18244, 18245, -1, 18246, 18247, 18248, -1, 18249, 18250, 18251, -1, 18252, 18253, 18254, -1, 18255, 18256, 18257, -1, 18258, 18259, 18260, -1, 18261, 18262, 18263, -1, 18264, 18265, 18266, -1, 18267, 18268, 18269, -1, 18270, 18271, 18272, -1, 18273, 18274, 18275, -1, 18276, 18277, 18278, -1, 18279, 18280, 18281, -1, 18282, 18283, 18284, -1, 18285, 18286, 18287, -1, 18288, 18289, 18290, -1, 18291, 18292, 18293, -1, 18294, 18295, 18296, -1, 18297, 18298, 18299, -1, 18300, 18301, 18302, -1, 18303, 18304, 18305, -1, 18306, 18307, 18308, -1, 18309, 18310, 18311, -1, 18312, 18313, 18314, -1, 18315, 18316, 18317, -1, 18318, 18319, 18320, -1, 18321, 18322, 18323, -1, 18324, 18325, 18326, -1, 18327, 18328, 18329, -1, 18330, 18331, 18332, -1, 18333, 18334, 18335, -1, 18336, 18337, 18338, -1, 18339, 18340, 18341, -1, 18342, 18343, 18344, -1, 18345, 18346, 18347, -1, 18348, 18349, 18350, -1, 18351, 18352, 18353, -1, 18354, 18355, 18356, -1, 18357, 18358, 18359, -1, 18360, 18361, 18362, -1, 18363, 18364, 18365, -1, 18366, 18367, 18368, -1, 18369, 18370, 18371, -1, 18372, 18373, 18374, -1, 18375, 18376, 18377, -1, 18378, 18379, 18380, -1, 18381, 18382, 18383, -1, 18384, 18385, 18386, -1, 18387, 18388, 18389, -1, 18390, 18391, 18392, -1, 18393, 18394, 18395, -1, 18396, 18397, 18398, -1, 18399, 18400, 18401, -1, 18402, 18403, 18404, -1, 18405, 18406, 18407, -1, 18408, 18409, 18410, -1, 18411, 18412, 18413, -1, 18414, 18415, 18416, -1, 18417, 18418, 18419, -1, 18420, 18421, 18422, -1, 18423, 18424, 18425, -1, 18426, 18427, 18428, -1, 18429, 18430, 18431, -1, 18432, 18433, 18434, -1, 18435, 18436, 18437, -1, 18438, 18439, 18440, -1, 18441, 18442, 18443, -1, 18444, 18445, 18446, -1, 18447, 18448, 18449, -1, 18450, 18451, 18452, -1, 18453, 18454, 18455, -1, 18456, 18457, 18458, -1, 18459, 18460, 18461, -1, 18462, 18463, 18464, -1, 18465, 18466, 18467, -1, 18468, 18469, 18470, -1, 18471, 18472, 18473, -1, 18474, 18475, 18476, -1, 18477, 18478, 18479, -1, 18480, 18481, 18482, -1, 18483, 18484, 18485, -1, 18486, 18487, 18488, -1, 18489, 18490, 18491, -1, 18492, 18493, 18494, -1, 18495, 18496, 18497, -1, 18498, 18499, 18500, -1, 18501, 18502, 18503, -1, 18504, 18505, 18506, -1, 18507, 18508, 18509, -1, 18510, 18511, 18512, -1, 18513, 18514, 18515, -1, 18516, 18517, 18518, -1, 18519, 18520, 18521, -1, 18522, 18523, 18524, -1, 18525, 18526, 18527, -1, 18528, 18529, 18530, -1, 18531, 18532, 18533, -1, 18534, 18535, 18536, -1, 18537, 18538, 18539, -1, 18540, 18541, 18542, -1, 18543, 18544, 18545, -1, 18546, 18547, 18548, -1, 18549, 18550, 18551, -1, 18552, 18553, 18554, -1, 18555, 18556, 18557, -1, 18558, 18559, 18560, -1, 18561, 18562, 18563, -1, 18564, 18565, 18566, -1, 18567, 18568, 18569, -1, 18570, 18571, 18572, -1, 18573, 18574, 18575, -1, 18576, 18577, 18578, -1, 18579, 18580, 18581, -1, 18582, 18583, 18584, -1, 18585, 18586, 18587, -1, 18588, 18589, 18590, -1, 18591, 18592, 18593, -1, 18594, 18595, 18596, -1, 18597, 18598, 18599, -1, 18600, 18601, 18602, -1, 18603, 18604, 18605, -1, 18606, 18607, 18608, -1, 18609, 18610, 18611, -1, 18612, 18613, 18614, -1, 18615, 18616, 18617, -1, 18618, 18619, 18620, -1, 18621, 18622, 18623, -1, 18624, 18625, 18626, -1, 18627, 18628, 18629, -1, 18630, 18631, 18632, -1, 18633, 18634, 18635, -1, 18636, 18637, 18638, -1, 18639, 18640, 18641, -1, 18642, 18643, 18644, -1, 18645, 18646, 18647, -1, 18648, 18649, 18650, -1, 18651, 18652, 18653, -1, 18654, 18655, 18656, -1, 18657, 18658, 18659, -1, 18660, 18661, 18662, -1, 18663, 18664, 18665, -1, 18666, 18667, 18668, -1, 18669, 18670, 18671, -1, 18672, 18673, 18674, -1, 18675, 18676, 18677, -1, 18678, 18679, 18680, -1, 18681, 18682, 18683, -1, 18684, 18685, 18686, -1, 18687, 18688, 18689, -1, 18690, 18691, 18692, -1, 18693, 18694, 18695, -1, 18696, 18697, 18698, -1, 18699, 18700, 18701, -1, 18702, 18703, 18704, -1, 18705, 18706, 18707, -1, 18708, 18709, 18710, -1, 18711, 18712, 18713, -1, 18714, 18715, 18716, -1, 18717, 18718, 18719, -1, 18720, 18721, 18722, -1, 18723, 18724, 18725, -1, 18726, 18727, 18728, -1, 18729, 18730, 18731, -1, 18732, 18733, 18734, -1, 18735, 18736, 18737, -1, 18738, 18739, 18740, -1, 18741, 18742, 18743, -1, 18744, 18745, 18746, -1, 18747, 18748, 18749, -1, 18750, 18751, 18752, -1, 18753, 18754, 18755, -1, 18756, 18757, 18758, -1, 18759, 18760, 18761, -1, 18762, 18763, 18764, -1, 18765, 18766, 18767, -1, 18768, 18769, 18770, -1, 18771, 18772, 18773, -1, 18774, 18775, 18776, -1, 18777, 18778, 18779, -1, 18780, 18781, 18782, -1, 18783, 18784, 18785, -1, 18786, 18787, 18788, -1, 18789, 18790, 18791, -1, 18792, 18793, 18794, -1, 18795, 18796, 18797, -1, 18798, 18799, 18800, -1, 18801, 18802, 18803, -1, 18804, 18805, 18806, -1, 18807, 18808, 18809, -1, 18810, 18811, 18812, -1, 18813, 18814, 18815, -1, 18816, 18817, 18818, -1, 18819, 18820, 18821, -1, 18822, 18823, 18824, -1, 18825, 18826, 18827, -1, 18828, 18829, 18830, -1, 18831, 18832, 18833, -1, 18834, 18835, 18836, -1, 18837, 18838, 18839, -1, 18840, 18841, 18842, -1, 18843, 18844, 18845, -1, 18846, 18847, 18848, -1, 18849, 18850, 18851, -1, 18852, 18853, 18854, -1, 18855, 18856, 18857, -1, 18858, 18859, 18860, -1, 18861, 18862, 18863, -1, 18864, 18865, 18866, -1, 18867, 18868, 18869, -1, 18870, 18871, 18872, -1, 18873, 18874, 18875, -1, 18876, 18877, 18878, -1, 18879, 18880, 18881, -1, 18882, 18883, 18884, -1, 18885, 18886, 18887, -1, 18888, 18889, 18890, -1, 18891, 18892, 18893, -1, 18894, 18895, 18896, -1, 18897, 18898, 18899, -1, 18900, 18901, 18902, -1, 18903, 18904, 18905, -1, 18906, 18907, 18908, -1, 18909, 18910, 18911, -1, 18912, 18913, 18914, -1, 18915, 18916, 18917, -1, 18918, 18919, 18920, -1, 18921, 18922, 18923, -1, 18924, 18925, 18926, -1, 18927, 18928, 18929, -1, 18930, 18931, 18932, -1, 18933, 18934, 18935, -1, 18936, 18937, 18938, -1, 18939, 18940, 18941, -1, 18942, 18943, 18944, -1, 18945, 18946, 18947, -1, 18948, 18949, 18950, -1, 18951, 18952, 18953, -1, 18954, 18955, 18956, -1, 18957, 18958, 18959, -1, 18960, 18961, 18962, -1, 18963, 18964, 18965, -1, 18966, 18967, 18968, -1, 18969, 18970, 18971, -1, 18972, 18973, 18974, -1, 18975, 18976, 18977, -1, 18978, 18979, 18980, -1, 18981, 18982, 18983, -1, 18984, 18985, 18986, -1, 18987, 18988, 18989, -1, 18990, 18991, 18992, -1, 18993, 18994, 18995, -1, 18996, 18997, 18998, -1, 18999, 19000, 19001, -1, 19002, 19003, 19004, -1, 19005, 19006, 19007, -1, 19008, 19009, 19010, -1, 19011, 19012, 19013, -1, 19014, 19015, 19016, -1, 19017, 19018, 19019, -1, 19020, 19021, 19022, -1, 19023, 19024, 19025, -1, 19026, 19027, 19028, -1, 19029, 19030, 19031, -1, 19032, 19033, 19034, -1, 19035, 19036, 19037, -1, 19038, 19039, 19040, -1, 19041, 19042, 19043, -1, 19044, 19045, 19046, -1, 19047, 19048, 19049, -1, 19050, 19051, 19052, -1, 19053, 19054, 19055, -1, 19056, 19057, 19058, -1, 19059, 19060, 19061, -1, 19062, 19063, 19064, -1, 19065, 19066, 19067, -1, 19068, 19069, 19070, -1, 19071, 19072, 19073, -1, 19074, 19075, 19076, -1, 19077, 19078, 19079, -1, 19080, 19081, 19082, -1, 19083, 19084, 19085, -1, 19086, 19087, 19088, -1, 19089, 19090, 19091, -1, 19092, 19093, 19094, -1, 19095, 19096, 19097, -1, 19098, 19099, 19100, -1, 19101, 19102, 19103, -1, 19104, 19105, 19106, -1, 19107, 19108, 19109, -1, 19110, 19111, 19112, -1, 19113, 19114, 19115, -1, 19116, 19117, 19118, -1, 19119, 19120, 19121, -1, 19122, 19123, 19124, -1, 19125, 19126, 19127, -1, 19128, 19129, 19130, -1, 19131, 19132, 19133, -1, 19134, 19135, 19136, -1, 19137, 19138, 19139, -1, 19140, 19141, 19142, -1, 19143, 19144, 19145, -1, 19146, 19147, 19148, -1, 19149, 19150, 19151, -1, 19152, 19153, 19154, -1, 19155, 19156, 19157, -1, 19158, 19159, 19160, -1, 19161, 19162, 19163, -1, 19164, 19165, 19166, -1, 19167, 19168, 19169, -1, 19170, 19171, 19172, -1, 19173, 19174, 19175, -1, 19176, 19177, 19178, -1, 19179, 19180, 19181, -1, 19182, 19183, 19184, -1, 19185, 19186, 19187, -1, 19188, 19189, 19190, -1, 19191, 19192, 19193, -1, 19194, 19195, 19196, -1, 19197, 19198, 19199, -1, 19200, 19201, 19202, -1, 19203, 19204, 19205, -1, 19206, 19207, 19208, -1, 19209, 19210, 19211, -1, 19212, 19213, 19214, -1, 19215, 19216, 19217, -1, 19218, 19219, 19220, -1, 19221, 19222, 19223, -1, 19224, 19225, 19226, -1, 19227, 19228, 19229, -1, 19230, 19231, 19232, -1, 19233, 19234, 19235, -1, 19236, 19237, 19238, -1, 19239, 19240, 19241, -1, 19242, 19243, 19244, -1, 19245, 19246, 19247, -1, 19248, 19249, 19250, -1, 19251, 19252, 19253, -1, 19254, 19255, 19256, -1, 19257, 19258, 19259, -1, 19260, 19261, 19262, -1, 19263, 19264, 19265, -1, 19266, 19267, 19268, -1, 19269, 19270, 19271, -1, 19272, 19273, 19274, -1, 19275, 19276, 19277, -1, 19278, 19279, 19280, -1, 19281, 19282, 19283, -1, 19284, 19285, 19286, -1, 19287, 19288, 19289, -1, 19290, 19291, 19292, -1, 19293, 19294, 19295, -1, 19296, 19297, 19298, -1, 19299, 19300, 19301, -1, 19302, 19303, 19304, -1, 19305, 19306, 19307, -1, 19308, 19309, 19310, -1, 19311, 19312, 19313, -1, 19314, 19315, 19316, -1, 19317, 19318, 19319, -1, 19320, 19321, 19322, -1, 19323, 19324, 19325, -1, 19326, 19327, 19328, -1, 19329, 19330, 19331, -1, 19332, 19333, 19334, -1, 19335, 19336, 19337, -1, 19338, 19339, 19340, -1, 19341, 19342, 19343, -1, 19344, 19345, 19346, -1, 19347, 19348, 19349, -1, 19350, 19351, 19352, -1, 19353, 19354, 19355, -1, 19356, 19357, 19358, -1, 19359, 19360, 19361, -1, 19362, 19363, 19364, -1, 19365, 19366, 19367, -1, 19368, 19369, 19370, -1, 19371, 19372, 19373, -1, 19374, 19375, 19376, -1, 19377, 19378, 19379, -1, 19380, 19381, 19382, -1, 19383, 19384, 19385, -1, 19386, 19387, 19388, -1, 19389, 19390, 19391, -1, 19392, 19393, 19394, -1, 19395, 19396, 19397, -1, 19398, 19399, 19400, -1, 19401, 19402, 19403, -1, 19404, 19405, 19406, -1, 19407, 19408, 19409, -1, 19410, 19411, 19412, -1, 19413, 19414, 19415, -1, 19416, 19417, 19418, -1, 19419, 19420, 19421, -1, 19422, 19423, 19424, -1, 19425, 19426, 19427, -1, 19428, 19429, 19430, -1, 19431, 19432, 19433, -1, 19434, 19435, 19436, -1, 19437, 19438, 19439, -1, 19440, 19441, 19442, -1, 19443, 19444, 19445, -1, 19446, 19447, 19448, -1, 19449, 19450, 19451, -1, 19452, 19453, 19454, -1, 19455, 19456, 19457, -1, 19458, 19459, 19460, -1, 19461, 19462, 19463, -1, 19464, 19465, 19466, -1, 19467, 19468, 19469, -1, 19470, 19471, 19472, -1, 19473, 19474, 19475, -1, 19476, 19477, 19478, -1, 19479, 19480, 19481, -1, 19482, 19483, 19484, -1, 19485, 19486, 19487, -1, 19488, 19489, 19490, -1, 19491, 19492, 19493, -1, 19494, 19495, 19496, -1, 19497, 19498, 19499, -1, 19500, 19501, 19502, -1, 19503, 19504, 19505, -1, 19506, 19507, 19508, -1, 19509, 19510, 19511, -1, 19512, 19513, 19514, -1, 19515, 19516, 19517, -1, 19518, 19519, 19520, -1, 19521, 19522, 19523, -1, 19524, 19525, 19526, -1, 19527, 19528, 19529, -1, 19530, 19531, 19532, -1, 19533, 19534, 19535, -1, 19536, 19537, 19538, -1, 19539, 19540, 19541, -1, 19542, 19543, 19544, -1, 19545, 19546, 19547, -1, 19548, 19549, 19550, -1, 19551, 19552, 19553, -1, 19554, 19555, 19556, -1, 19557, 19558, 19559, -1, 19560, 19561, 19562, -1, 19563, 19564, 19565, -1, 19566, 19567, 19568, -1, 19569, 19570, 19571, -1, 19572, 19573, 19574, -1, 19575, 19576, 19577, -1, 19578, 19579, 19580, -1, 19581, 19582, 19583, -1, 19584, 19585, 19586, -1, 19587, 19588, 19589, -1, 19590, 19591, 19592, -1, 19593, 19594, 19595, -1, 19596, 19597, 19598, -1, 19599, 19600, 19601, -1, 19602, 19603, 19604, -1, 19605, 19606, 19607, -1, 19608, 19609, 19610, -1, 19611, 19612, 19613, -1, 19614, 19615, 19616, -1, 19617, 19618, 19619, -1, 19620, 19621, 19622, -1, 19623, 19624, 19625, -1, 19626, 19627, 19628, -1, 19629, 19630, 19631, -1, 19632, 19633, 19634, -1, 19635, 19636, 19637, -1, 19638, 19639, 19640, -1, 19641, 19642, 19643, -1, 19644, 19645, 19646, -1, 19647, 19648, 19649, -1, 19650, 19651, 19652, -1, 19653, 19654, 19655, -1, 19656, 19657, 19658, -1, 19659, 19660, 19661, -1, 19662, 19663, 19664, -1, 19665, 19666, 19667, -1, 19668, 19669, 19670, -1, 19671, 19672, 19673, -1, 19674, 19675, 19676, -1, 19677, 19678, 19679, -1, 19680, 19681, 19682, -1, 19683, 19684, 19685, -1, 19686, 19687, 19688, -1, 19689, 19690, 19691, -1, 19692, 19693, 19694, -1, 19695, 19696, 19697, -1, 19698, 19699, 19700, -1, 19701, 19702, 19703, -1, 19704, 19705, 19706, -1, 19707, 19708, 19709, -1, 19710, 19711, 19712, -1, 19713, 19714, 19715, -1, 19716, 19717, 19718, -1, 19719, 19720, 19721, -1, 19722, 19723, 19724, -1, 19725, 19726, 19727, -1, 19728, 19729, 19730, -1, 19731, 19732, 19733, -1, 19734, 19735, 19736, -1, 19737, 19738, 19739, -1, 19740, 19741, 19742, -1, 19743, 19744, 19745, -1, 19746, 19747, 19748, -1, 19749, 19750, 19751, -1, 19752, 19753, 19754, -1, 19755, 19756, 19757, -1, 19758, 19759, 19760, -1, 19761, 19762, 19763, -1, 19764, 19765, 19766, -1, 19767, 19768, 19769, -1, 19770, 19771, 19772, -1, 19773, 19774, 19775, -1, 19776, 19777, 19778, -1, 19779, 19780, 19781, -1, 19782, 19783, 19784, -1, 19785, 19786, 19787, -1, 19788, 19789, 19790, -1, 19791, 19792, 19793, -1, 19794, 19795, 19796, -1, 19797, 19798, 19799, -1, 19800, 19801, 19802, -1, 19803, 19804, 19805, -1, 19806, 19807, 19808, -1, 19809, 19810, 19811, -1, 19812, 19813, 19814, -1, 19815, 19816, 19817, -1, 19818, 19819, 19820, -1, 19821, 19822, 19823, -1, 19824, 19825, 19826, -1, 19827, 19828, 19829, -1, 19830, 19831, 19832, -1, 19833, 19834, 19835, -1, 19836, 19837, 19838, -1, 19839, 19840, 19841, -1, 19842, 19843, 19844, -1, 19845, 19846, 19847, -1, 19848, 19849, 19850, -1, 19851, 19852, 19853, -1, 19854, 19855, 19856, -1, 19857, 19858, 19859, -1, 19860, 19861, 19862, -1, 19863, 19864, 19865, -1, 19866, 19867, 19868, -1, 19869, 19870, 19871, -1, 19872, 19873, 19874, -1, 19875, 19876, 19877, -1, 19878, 19879, 19880, -1, 19881, 19882, 19883, -1, 19884, 19885, 19886, -1, 19887, 19888, 19889, -1, 19890, 19891, 19892, -1, 19893, 19894, 19895, -1, 19896, 19897, 19898, -1, 19899, 19900, 19901, -1, 19902, 19903, 19904, -1, 19905, 19906, 19907, -1, 19908, 19909, 19910, -1, 19911, 19912, 19913, -1, 19914, 19915, 19916, -1, 19917, 19918, 19919, -1, 19920, 19921, 19922, -1, 19923, 19924, 19925, -1, 19926, 19927, 19928, -1, 19929, 19930, 19931, -1, 19932, 19933, 19934, -1, 19935, 19936, 19937, -1, 19938, 19939, 19940, -1, 19941, 19942, 19943, -1, 19944, 19945, 19946, -1, 19947, 19948, 19949, -1, 19950, 19951, 19952, -1, 19953, 19954, 19955, -1, 19956, 19957, 19958, -1, 19959, 19960, 19961, -1, 19962, 19963, 19964, -1, 19965, 19966, 19967, -1, 19968, 19969, 19970, -1, 19971, 19972, 19973, -1, 19974, 19975, 19976, -1, 19977, 19978, 19979, -1, 19980, 19981, 19982, -1, 19983, 19984, 19985, -1, 19986, 19987, 19988, -1, 19989, 19990, 19991, -1, 19992, 19993, 19994, -1, 19995, 19996, 19997, -1, 19998, 19999, 20000, -1, 20001, 20002, 20003, -1, 20004, 20005, 20006, -1, 20007, 20008, 20009, -1, 20010, 20011, 20012, -1, 20013, 20014, 20015, -1, 20016, 20017, 20018, -1, 20019, 20020, 20021, -1, 20022, 20023, 20024, -1, 20025, 20026, 20027, -1, 20028, 20029, 20030, -1, 20031, 20032, 20033, -1, 20034, 20035, 20036, -1, 20037, 20038, 20039, -1, 20040, 20041, 20042, -1, 20043, 20044, 20045, -1, 20046, 20047, 20048, -1, 20049, 20050, 20051, -1, 20052, 20053, 20054, -1, 20055, 20056, 20057, -1, 20058, 20059, 20060, -1, 20061, 20062, 20063, -1, 20064, 20065, 20066, -1, 20067, 20068, 20069, -1, 20070, 20071, 20072, -1, 20073, 20074, 20075, -1, 20076, 20077, 20078, -1, 20079, 20080, 20081, -1, 20082, 20083, 20084, -1, 20085, 20086, 20087, -1, 20088, 20089, 20090, -1, 20091, 20092, 20093, -1, 20094, 20095, 20096, -1, 20097, 20098, 20099, -1, 20100, 20101, 20102, -1, 20103, 20104, 20105, -1, 20106, 20107, 20108, -1, 20109, 20110, 20111, -1, 20112, 20113, 20114, -1, 20115, 20116, 20117, -1, 20118, 20119, 20120, -1, 20121, 20122, 20123, -1, 20124, 20125, 20126, -1, 20127, 20128, 20129, -1, 20130, 20131, 20132, -1, 20133, 20134, 20135, -1, 20136, 20137, 20138, -1, 20139, 20140, 20141, -1, 20142, 20143, 20144, -1, 20145, 20146, 20147, -1, 20148, 20149, 20150, -1, 20151, 20152, 20153, -1, 20154, 20155, 20156, -1, 20157, 20158, 20159, -1, 20160, 20161, 20162, -1, 20163, 20164, 20165, -1, 20166, 20167, 20168, -1, 20169, 20170, 20171, -1, 20172, 20173, 20174, -1, 20175, 20176, 20177, -1, 20178, 20179, 20180, -1, 20181, 20182, 20183, -1, 20184, 20185, 20186, -1, 20187, 20188, 20189, -1, 20190, 20191, 20192, -1, 20193, 20194, 20195, -1, 20196, 20197, 20198, -1, 20199, 20200, 20201, -1, 20202, 20203, 20204, -1, 20205, 20206, 20207, -1, 20208, 20209, 20210, -1, 20211, 20212, 20213, -1, 20214, 20215, 20216, -1, 20217, 20218, 20219, -1, 20220, 20221, 20222, -1, 20223, 20224, 20225, -1, 20226, 20227, 20228, -1, 20229, 20230, 20231, -1, 20232, 20233, 20234, -1, 20235, 20236, 20237, -1, 20238, 20239, 20240, -1, 20241, 20242, 20243, -1, 20244, 20245, 20246, -1, 20247, 20248, 20249, -1, 20250, 20251, 20252, -1, 20253, 20254, 20255, -1, 20256, 20257, 20258, -1, 20259, 20260, 20261, -1, 20262, 20263, 20264, -1, 20265, 20266, 20267, -1, 20268, 20269, 20270, -1, 20271, 20272, 20273, -1, 20274, 20275, 20276, -1, 20277, 20278, 20279, -1, 20280, 20281, 20282, -1, 20283, 20284, 20285, -1, 20286, 20287, 20288, -1, 20289, 20290, 20291, -1, 20292, 20293, 20294, -1, 20295, 20296, 20297, -1, 20298, 20299, 20300, -1, 20301, 20302, 20303, -1, 20304, 20305, 20306, -1, 20307, 20308, 20309, -1, 20310, 20311, 20312, -1, 20313, 20314, 20315, -1, 20316, 20317, 20318, -1, 20319, 20320, 20321, -1, 20322, 20323, 20324, -1, 20325, 20326, 20327, -1, 20328, 20329, 20330, -1, 20331, 20332, 20333, -1, 20334, 20335, 20336, -1, 20337, 20338, 20339, -1, 20340, 20341, 20342, -1, 20343, 20344, 20345, -1, 20346, 20347, 20348, -1, 20349, 20350, 20351, -1, 20352, 20353, 20354, -1, 20355, 20356, 20357, -1, 20358, 20359, 20360, -1, 20361, 20362, 20363, -1, 20364, 20365, 20366, -1, 20367, 20368, 20369, -1, 20370, 20371, 20372, -1, 20373, 20374, 20375, -1, 20376, 20377, 20378, -1, 20379, 20380, 20381, -1, 20382, 20383, 20384, -1, 20385, 20386, 20387, -1, 20388, 20389, 20390, -1, 20391, 20392, 20393, -1, 20394, 20395, 20396, -1, 20397, 20398, 20399, -1, 20400, 20401, 20402, -1, 20403, 20404, 20405, -1, 20406, 20407, 20408, -1, 20409, 20410, 20411, -1, 20412, 20413, 20414, -1, 20415, 20416, 20417, -1, 20418, 20419, 20420, -1, 20421, 20422, 20423, -1, 20424, 20425, 20426, -1, 20427, 20428, 20429, -1, 20430, 20431, 20432, -1, 20433, 20434, 20435, -1, 20436, 20437, 20438, -1, 20439, 20440, 20441, -1, 20442, 20443, 20444, -1, 20445, 20446, 20447, -1, 20448, 20449, 20450, -1, 20451, 20452, 20453, -1, 20454, 20455, 20456, -1, 20457, 20458, 20459, -1, 20460, 20461, 20462, -1, 20463, 20464, 20465, -1, 20466, 20467, 20468, -1, 20469, 20470, 20471, -1, 20472, 20473, 20474, -1, 20475, 20476, 20477, -1, 20478, 20479, 20480, -1, 20481, 20482, 20483, -1, 20484, 20485, 20486, -1, 20487, 20488, 20489, -1, 20490, 20491, 20492, -1, 20493, 20494, 20495, -1, 20496, 20497, 20498, -1, 20499, 20500, 20501, -1, 20502, 20503, 20504, -1, 20505, 20506, 20507, -1, 20508, 20509, 20510, -1, 20511, 20512, 20513, -1, 20514, 20515, 20516, -1, 20517, 20518, 20519, -1, 20520, 20521, 20522, -1, 20523, 20524, 20525, -1, 20526, 20527, 20528, -1, 20529, 20530, 20531, -1, 20532, 20533, 20534, -1, 20535, 20536, 20537, -1, 20538, 20539, 20540, -1, 20541, 20542, 20543, -1, 20544, 20545, 20546, -1, 20547, 20548, 20549, -1, 20550, 20551, 20552, -1, 20553, 20554, 20555, -1, 20556, 20557, 20558, -1, 20559, 20560, 20561, -1, 20562, 20563, 20564, -1, 20565, 20566, 20567, -1, 20568, 20569, 20570, -1, 20571, 20572, 20573, -1, 20574, 20575, 20576, -1, 20577, 20578, 20579, -1, 20580, 20581, 20582, -1, 20583, 20584, 20585, -1, 20586, 20587, 20588, -1, 20589, 20590, 20591, -1, 20592, 20593, 20594, -1, 20595, 20596, 20597, -1, 20598, 20599, 20600, -1, 20601, 20602, 20603, -1, 20604, 20605, 20606, -1, 20607, 20608, 20609, -1, 20610, 20611, 20612, -1, 20613, 20614, 20615, -1, 20616, 20617, 20618, -1, 20619, 20620, 20621, -1, 20622, 20623, 20624, -1, 20625, 20626, 20627, -1, 20628, 20629, 20630, -1, 20631, 20632, 20633, -1, 20634, 20635, 20636, -1, 20637, 20638, 20639, -1, 20640, 20641, 20642, -1, 20643, 20644, 20645, -1, 20646, 20647, 20648, -1, 20649, 20650, 20651, -1, 20652, 20653, 20654, -1, 20655, 20656, 20657, -1, 20658, 20659, 20660, -1, 20661, 20662, 20663, -1, 20664, 20665, 20666, -1, 20667, 20668, 20669, -1, 20670, 20671, 20672, -1, 20673, 20674, 20675, -1, 20676, 20677, 20678, -1, 20679, 20680, 20681, -1, 20682, 20683, 20684, -1, 20685, 20686, 20687, -1, 20688, 20689, 20690, -1, 20691, 20692, 20693, -1, 20694, 20695, 20696, -1, 20697, 20698, 20699, -1, 20700, 20701, 20702, -1, 20703, 20704, 20705, -1, 20706, 20707, 20708, -1, 20709, 20710, 20711, -1, 20712, 20713, 20714, -1, 20715, 20716, 20717, -1, 20718, 20719, 20720, -1, 20721, 20722, 20723, -1, 20724, 20725, 20726, -1, 20727, 20728, 20729, -1, 20730, 20731, 20732, -1, 20733, 20734, 20735, -1, 20736, 20737, 20738, -1, 20739, 20740, 20741, -1, 20742, 20743, 20744, -1, 20745, 20746, 20747, -1, 20748, 20749, 20750, -1, 20751, 20752, 20753, -1, 20754, 20755, 20756, -1, 20757, 20758, 20759, -1, 20760, 20761, 20762, -1, 20763, 20764, 20765, -1, 20766, 20767, 20768, -1, 20769, 20770, 20771, -1, 20772, 20773, 20774, -1, 20775, 20776, 20777, -1, 20778, 20779, 20780, -1, 20781, 20782, 20783, -1, 20784, 20785, 20786, -1, 20787, 20788, 20789, -1, 20790, 20791, 20792, -1, 20793, 20794, 20795, -1, 20796, 20797, 20798, -1, 20799, 20800, 20801, -1, 20802, 20803, 20804, -1, 20805, 20806, 20807, -1, 20808, 20809, 20810, -1, 20811, 20812, 20813, -1, 20814, 20815, 20816, -1, 20817, 20818, 20819, -1, 20820, 20821, 20822, -1, 20823, 20824, 20825, -1, 20826, 20827, 20828, -1, 20829, 20830, 20831, -1, 20832, 20833, 20834, -1, 20835, 20836, 20837, -1, 20838, 20839, 20840, -1, 20841, 20842, 20843, -1, 20844, 20845, 20846, -1, 20847, 20848, 20849, -1, 20850, 20851, 20852, -1, 20853, 20854, 20855, -1, 20856, 20857, 20858, -1, 20859, 20860, 20861, -1, 20862, 20863, 20864, -1, 20865, 20866, 20867, -1, 20868, 20869, 20870, -1, 20871, 20872, 20873, -1, 20874, 20875, 20876, -1, 20877, 20878, 20879, -1, 20880, 20881, 20882, -1, 20883, 20884, 20885, -1, 20886, 20887, 20888, -1, 20889, 20890, 20891, -1, 20892, 20893, 20894, -1, 20895, 20896, 20897, -1, 20898, 20899, 20900, -1, 20901, 20902, 20903, -1, 20904, 20905, 20906, -1, 20907, 20908, 20909, -1, 20910, 20911, 20912, -1, 20913, 20914, 20915, -1, 20916, 20917, 20918, -1, 20919, 20920, 20921, -1, 20922, 20923, 20924, -1, 20925, 20926, 20927, -1, 20928, 20929, 20930, -1, 20931, 20932, 20933, -1, 20934, 20935, 20936, -1, 20937, 20938, 20939, -1, 20940, 20941, 20942, -1, 20943, 20944, 20945, -1, 20946, 20947, 20948, -1, 20949, 20950, 20951, -1, 20952, 20953, 20954, -1, 20955, 20956, 20957, -1, 20958, 20959, 20960, -1, 20961, 20962, 20963, -1, 20964, 20965, 20966, -1, 20967, 20968, 20969, -1, 20970, 20971, 20972, -1, 20973, 20974, 20975, -1, 20976, 20977, 20978, -1, 20979, 20980, 20981, -1, 20982, 20983, 20984, -1, 20985, 20986, 20987, -1, 20988, 20989, 20990, -1, 20991, 20992, 20993, -1, 20994, 20995, 20996, -1, 20997, 20998, 20999, -1, 21000, 21001, 21002, -1, 21003, 21004, 21005, -1, 21006, 21007, 21008, -1, 21009, 21010, 21011, -1, 21012, 21013, 21014, -1, 21015, 21016, 21017, -1, 21018, 21019, 21020, -1, 21021, 21022, 21023, -1, 21024, 21025, 21026, -1, 21027, 21028, 21029, -1, 21030, 21031, 21032, -1, 21033, 21034, 21035, -1, 21036, 21037, 21038, -1, 21039, 21040, 21041, -1, 21042, 21043, 21044, -1, 21045, 21046, 21047, -1, 21048, 21049, 21050, -1, 21051, 21052, 21053, -1, 21054, 21055, 21056, -1, 21057, 21058, 21059, -1, 21060, 21061, 21062, -1, 21063, 21064, 21065, -1, 21066, 21067, 21068, -1, 21069, 21070, 21071, -1, 21072, 21073, 21074, -1, 21075, 21076, 21077, -1, 21078, 21079, 21080, -1, 21081, 21082, 21083, -1, 21084, 21085, 21086, -1, 21087, 21088, 21089, -1, 21090, 21091, 21092, -1, 21093, 21094, 21095, -1, 21096, 21097, 21098, -1, 21099, 21100, 21101, -1, 21102, 21103, 21104, -1, 21105, 21106, 21107, -1, 21108, 21109, 21110, -1, 21111, 21112, 21113, -1, 21114, 21115, 21116, -1, 21117, 21118, 21119, -1, 21120, 21121, 21122, -1, 21123, 21124, 21125, -1, 21126, 21127, 21128, -1, 21129, 21130, 21131, -1, 21132, 21133, 21134, -1, 21135, 21136, 21137, -1, 21138, 21139, 21140, -1, 21141, 21142, 21143, -1, 21144, 21145, 21146, -1, 21147, 21148, 21149, -1, 21150, 21151, 21152, -1, 21153, 21154, 21155, -1, 21156, 21157, 21158, -1, 21159, 21160, 21161, -1, 21162, 21163, 21164, -1, 21165, 21166, 21167, -1, 21168, 21169, 21170, -1, 21171, 21172, 21173, -1, 21174, 21175, 21176, -1, 21177, 21178, 21179, -1, 21180, 21181, 21182, -1, 21183, 21184, 21185, -1, 21186, 21187, 21188, -1, 21189, 21190, 21191, -1, 21192, 21193, 21194, -1, 21195, 21196, 21197, -1, 21198, 21199, 21200, -1, 21201, 21202, 21203, -1, 21204, 21205, 21206, -1, 21207, 21208, 21209, -1, 21210, 21211, 21212, -1, 21213, 21214, 21215, -1, 21216, 21217, 21218, -1, 21219, 21220, 21221, -1, 21222, 21223, 21224, -1, 21225, 21226, 21227, -1, 21228, 21229, 21230, -1, 21231, 21232, 21233, -1, 21234, 21235, 21236, -1, 21237, 21238, 21239, -1, 21240, 21241, 21242, -1, 21243, 21244, 21245, -1, 21246, 21247, 21248, -1, 21249, 21250, 21251, -1, 21252, 21253, 21254, -1, 21255, 21256, 21257, -1, 21258, 21259, 21260, -1, 21261, 21262, 21263, -1, 21264, 21265, 21266, -1, 21267, 21268, 21269, -1, 21270, 21271, 21272, -1, 21273, 21274, 21275, -1, 21276, 21277, 21278, -1, 21279, 21280, 21281, -1, 21282, 21283, 21284, -1, 21285, 21286, 21287, -1, 21288, 21289, 21290, -1, 21291, 21292, 21293, -1, 21294, 21295, 21296, -1, 21297, 21298, 21299, -1, 21300, 21301, 21302, -1, 21303, 21304, 21305, -1, 21306, 21307, 21308, -1, 21309, 21310, 21311, -1, 21312, 21313, 21314, -1, 21315, 21316, 21317, -1, 21318, 21319, 21320, -1, 21321, 21322, 21323, -1, 21324, 21325, 21326, -1, 21327, 21328, 21329, -1, 21330, 21331, 21332, -1, 21333, 21334, 21335, -1, 21336, 21337, 21338, -1, 21339, 21340, 21341, -1, 21342, 21343, 21344, -1, 21345, 21346, 21347, -1, 21348, 21349, 21350, -1, 21351, 21352, 21353, -1, 21354, 21355, 21356, -1, 21357, 21358, 21359, -1, 21360, 21361, 21362, -1, 21363, 21364, 21365, -1, 21366, 21367, 21368, -1, 21369, 21370, 21371, -1, 21372, 21373, 21374, -1, 21375, 21376, 21377, -1, 21378, 21379, 21380, -1, 21381, 21382, 21383, -1, 21384, 21385, 21386, -1, 21387, 21388, 21389, -1, 21390, 21391, 21392, -1, 21393, 21394, 21395, -1, 21396, 21397, 21398, -1, 21399, 21400, 21401, -1, 21402, 21403, 21404, -1, 21405, 21406, 21407, -1, 21408, 21409, 21410, -1, 21411, 21412, 21413, -1, 21414, 21415, 21416, -1, 21417, 21418, 21419, -1, 21420, 21421, 21422, -1, 21423, 21424, 21425, -1, 21426, 21427, 21428, -1, 21429, 21430, 21431, -1, 21432, 21433, 21434, -1, 21435, 21436, 21437, -1, 21438, 21439, 21440, -1, 21441, 21442, 21443, -1, 21444, 21445, 21446, -1, 21447, 21448, 21449, -1, 21450, 21451, 21452, -1, 21453, 21454, 21455, -1, 21456, 21457, 21458, -1, 21459, 21460, 21461, -1, 21462, 21463, 21464, -1, 21465, 21466, 21467, -1, 21468, 21469, 21470, -1, 21471, 21472, 21473, -1, 21474, 21475, 21476, -1, 21477, 21478, 21479, -1, 21480, 21481, 21482, -1, 21483, 21484, 21485, -1, 21486, 21487, 21488, -1, 21489, 21490, 21491, -1, 21492, 21493, 21494, -1, 21495, 21496, 21497, -1, 21498, 21499, 21500, -1, 21501, 21502, 21503, -1, 21504, 21505, 21506, -1, 21507, 21508, 21509, -1, 21510, 21511, 21512, -1, 21513, 21514, 21515, -1, 21516, 21517, 21518, -1, 21519, 21520, 21521, -1, 21522, 21523, 21524, -1, 21525, 21526, 21527, -1, 21528, 21529, 21530, -1, 21531, 21532, 21533, -1, 21534, 21535, 21536, -1, 21537, 21538, 21539, -1, 21540, 21541, 21542, -1, 21543, 21544, 21545, -1, 21546, 21547, 21548, -1, 21549, 21550, 21551, -1, 21552, 21553, 21554, -1, 21555, 21556, 21557, -1, 21558, 21559, 21560, -1, 21561, 21562, 21563, -1, 21564, 21565, 21566, -1, 21567, 21568, 21569, -1, 21570, 21571, 21572, -1, 21573, 21574, 21575, -1, 21576, 21577, 21578, -1, 21579, 21580, 21581, -1, 21582, 21583, 21584, -1, 21585, 21586, 21587, -1, 21588, 21589, 21590, -1, 21591, 21592, 21593, -1, 21594, 21595, 21596, -1, 21597, 21598, 21599, -1, 21600, 21601, 21602, -1, 21603, 21604, 21605, -1, 21606, 21607, 21608, -1, 21609, 21610, 21611, -1, 21612, 21613, 21614, -1, 21615, 21616, 21617, -1, 21618, 21619, 21620, -1, 21621, 21622, 21623, -1, 21624, 21625, 21626, -1, 21627, 21628, 21629, -1, 21630, 21631, 21632, -1, 21633, 21634, 21635, -1, 21636, 21637, 21638, -1, 21639, 21640, 21641, -1, 21642, 21643, 21644, -1, 21645, 21646, 21647, -1, 21648, 21649, 21650, -1, 21651, 21652, 21653, -1, 21654, 21655, 21656, -1, 21657, 21658, 21659, -1, 21660, 21661, 21662, -1, 21663, 21664, 21665, -1, 21666, 21667, 21668, -1, 21669, 21670, 21671, -1, 21672, 21673, 21674, -1, 21675, 21676, 21677, -1, 21678, 21679, 21680, -1, 21681, 21682, 21683, -1, 21684, 21685, 21686, -1, 21687, 21688, 21689, -1, 21690, 21691, 21692, -1, 21693, 21694, 21695, -1, 21696, 21697, 21698, -1, 21699, 21700, 21701, -1, 21702, 21703, 21704, -1, 21705, 21706, 21707, -1, 21708, 21709, 21710, -1, 21711, 21712, 21713, -1, 21714, 21715, 21716, -1, 21717, 21718, 21719, -1, 21720, 21721, 21722, -1, 21723, 21724, 21725, -1, 21726, 21727, 21728, -1, 21729, 21730, 21731, -1, 21732, 21733, 21734, -1, 21735, 21736, 21737, -1, 21738, 21739, 21740, -1, 21741, 21742, 21743, -1, 21744, 21745, 21746, -1, 21747, 21748, 21749, -1, 21750, 21751, 21752, -1, 21753, 21754, 21755, -1, 21756, 21757, 21758, -1, 21759, 21760, 21761, -1, 21762, 21763, 21764, -1, 21765, 21766, 21767, -1, 21768, 21769, 21770, -1, 21771, 21772, 21773, -1, 21774, 21775, 21776, -1, 21777, 21778, 21779, -1, 21780, 21781, 21782, -1, 21783, 21784, 21785, -1, 21786, 21787, 21788, -1, 21789, 21790, 21791, -1, 21792, 21793, 21794, -1, 21795, 21796, 21797, -1, 21798, 21799, 21800, -1, 21801, 21802, 21803, -1, 21804, 21805, 21806, -1, 21807, 21808, 21809, -1, 21810, 21811, 21812, -1, 21813, 21814, 21815, -1, 21816, 21817, 21818, -1, 21819, 21820, 21821, -1, 21822, 21823, 21824, -1, 21825, 21826, 21827, -1, 21828, 21829, 21830, -1, 21831, 21832, 21833, -1, 21834, 21835, 21836, -1, 21837, 21838, 21839, -1, 21840, 21841, 21842, -1, 21843, 21844, 21845, -1, 21846, 21847, 21848, -1, 21849, 21850, 21851, -1, 21852, 21853, 21854, -1, 21855, 21856, 21857, -1, 21858, 21859, 21860, -1, 21861, 21862, 21863, -1, 21864, 21865, 21866, -1, 21867, 21868, 21869, -1, 21870, 21871, 21872, -1, 21873, 21874, 21875, -1, 21876, 21877, 21878, -1, 21879, 21880, 21881, -1, 21882, 21883, 21884, -1, 21885, 21886, 21887, -1, 21888, 21889, 21890, -1, 21891, 21892, 21893, -1, 21894, 21895, 21896, -1, 21897, 21898, 21899, -1, 21900, 21901, 21902, -1, 21903, 21904, 21905, -1, 21906, 21907, 21908, -1, 21909, 21910, 21911, -1, 21912, 21913, 21914, -1, 21915, 21916, 21917, -1, 21918, 21919, 21920, -1, 21921, 21922, 21923, -1, 21924, 21925, 21926, -1, 21927, 21928, 21929, -1, 21930, 21931, 21932, -1, 21933, 21934, 21935, -1, 21936, 21937, 21938, -1, 21939, 21940, 21941, -1, 21942, 21943, 21944, -1, 21945, 21946, 21947, -1, 21948, 21949, 21950, -1, 21951, 21952, 21953, -1, 21954, 21955, 21956, -1, 21957, 21958, 21959, -1, 21960, 21961, 21962, -1, 21963, 21964, 21965, -1, 21966, 21967, 21968, -1, 21969, 21970, 21971, -1, 21972, 21973, 21974, -1, 21975, 21976, 21977, -1, 21978, 21979, 21980, -1, 21981, 21982, 21983, -1, 21984, 21985, 21986, -1, 21987, 21988, 21989, -1, 21990, 21991, 21992, -1, 21993, 21994, 21995, -1, 21996, 21997, 21998, -1, 21999, 22000, 22001, -1, 22002, 22003, 22004, -1, 22005, 22006, 22007, -1, 22008, 22009, 22010, -1, 22011, 22012, 22013, -1, 22014, 22015, 22016, -1, 22017, 22018, 22019, -1, 22020, 22021, 22022, -1, 22023, 22024, 22025, -1, 22026, 22027, 22028, -1, 22029, 22030, 22031, -1, 22032, 22033, 22034, -1, 22035, 22036, 22037, -1, 22038, 22039, 22040, -1, 22041, 22042, 22043, -1, 22044, 22045, 22046, -1, 22047, 22048, 22049, -1, 22050, 22051, 22052, -1, 22053, 22054, 22055, -1, 22056, 22057, 22058, -1, 22059, 22060, 22061, -1, 22062, 22063, 22064, -1, 22065, 22066, 22067, -1, 22068, 22069, 22070, -1, 22071, 22072, 22073, -1, 22074, 22075, 22076, -1, 22077, 22078, 22079, -1, 22080, 22081, 22082, -1, 22083, 22084, 22085, -1, 22086, 22087, 22088, -1, 22089, 22090, 22091, -1, 22092, 22093, 22094, -1, 22095, 22096, 22097, -1, 22098, 22099, 22100, -1, 22101, 22102, 22103, -1, 22104, 22105, 22106, -1, 22107, 22108, 22109, -1, 22110, 22111, 22112, -1, 22113, 22114, 22115, -1, 22116, 22117, 22118, -1, 22119, 22120, 22121, -1, 22122, 22123, 22124, -1, 22125, 22126, 22127, -1, 22128, 22129, 22130, -1, 22131, 22132, 22133, -1, 22134, 22135, 22136, -1, 22137, 22138, 22139, -1, 22140, 22141, 22142, -1, 22143, 22144, 22145, -1, 22146, 22147, 22148, -1, 22149, 22150, 22151, -1, 22152, 22153, 22154, -1, 22155, 22156, 22157, -1, 22158, 22159, 22160, -1, 22161, 22162, 22163, -1, 22164, 22165, 22166, -1, 22167, 22168, 22169, -1, 22170, 22171, 22172, -1, 22173, 22174, 22175, -1, 22176, 22177, 22178, -1, 22179, 22180, 22181, -1, 22182, 22183, 22184, -1, 22185, 22186, 22187, -1, 22188, 22189, 22190, -1, 22191, 22192, 22193, -1, 22194, 22195, 22196, -1, 22197, 22198, 22199, -1, 22200, 22201, 22202, -1, 22203, 22204, 22205, -1, 22206, 22207, 22208, -1, 22209, 22210, 22211, -1, 22212, 22213, 22214, -1, 22215, 22216, 22217, -1, 22218, 22219, 22220, -1, 22221, 22222, 22223, -1, 22224, 22225, 22226, -1, 22227, 22228, 22229, -1, 22230, 22231, 22232, -1, 22233, 22234, 22235, -1, 22236, 22237, 22238, -1, 22239, 22240, 22241, -1, 22242, 22243, 22244, -1, 22245, 22246, 22247, -1, 22248, 22249, 22250, -1, 22251, 22252, 22253, -1, 22254, 22255, 22256, -1, 22257, 22258, 22259, -1, 22260, 22261, 22262, -1, 22263, 22264, 22265, -1, 22266, 22267, 22268, -1, 22269, 22270, 22271, -1, 22272, 22273, 22274, -1, 22275, 22276, 22277, -1, 22278, 22279, 22280, -1, 22281, 22282, 22283, -1, 22284, 22285, 22286, -1, 22287, 22288, 22289, -1, 22290, 22291, 22292, -1, 22293, 22294, 22295, -1, 22296, 22297, 22298, -1, 22299, 22300, 22301, -1, 22302, 22303, 22304, -1, 22305, 22306, 22307, -1, 22308, 22309, 22310, -1, 22311, 22312, 22313, -1, 22314, 22315, 22316, -1, 22317, 22318, 22319, -1, 22320, 22321, 22322, -1, 22323, 22324, 22325, -1, 22326, 22327, 22328, -1, 22329, 22330, 22331, -1, 22332, 22333, 22334, -1, 22335, 22336, 22337, -1, 22338, 22339, 22340, -1, 22341, 22342, 22343, -1, 22344, 22345, 22346, -1, 22347, 22348, 22349, -1, 22350, 22351, 22352, -1, 22353, 22354, 22355, -1, 22356, 22357, 22358, -1, 22359, 22360, 22361, -1, 22362, 22363, 22364, -1, 22365, 22366, 22367, -1, 22368, 22369, 22370, -1, 22371, 22372, 22373, -1, 22374, 22375, 22376, -1, 22377, 22378, 22379, -1, 22380, 22381, 22382, -1, 22383, 22384, 22385, -1, 22386, 22387, 22388, -1, 22389, 22390, 22391, -1, 22392, 22393, 22394, -1, 22395, 22396, 22397, -1, 22398, 22399, 22400, -1, 22401, 22402, 22403, -1, 22404, 22405, 22406, -1, 22407, 22408, 22409, -1, 22410, 22411, 22412, -1, 22413, 22414, 22415, -1, 22416, 22417, 22418, -1, 22419, 22420, 22421, -1, 22422, 22423, 22424, -1, 22425, 22426, 22427, -1, 22428, 22429, 22430, -1, 22431, 22432, 22433, -1, 22434, 22435, 22436, -1, 22437, 22438, 22439, -1, 22440, 22441, 22442, -1, 22443, 22444, 22445, -1, 22446, 22447, 22448, -1, 22449, 22450, 22451, -1, 22452, 22453, 22454, -1, 22455, 22456, 22457, -1, 22458, 22459, 22460, -1, 22461, 22462, 22463, -1, 22464, 22465, 22466, -1, 22467, 22468, 22469, -1, 22470, 22471, 22472, -1, 22473, 22474, 22475, -1, 22476, 22477, 22478, -1, 22479, 22480, 22481, -1, 22482, 22483, 22484, -1, 22485, 22486, 22487, -1, 22488, 22489, 22490, -1, 22491, 22492, 22493, -1, 22494, 22495, 22496, -1, 22497, 22498, 22499, -1, 22500, 22501, 22502, -1, 22503, 22504, 22505, -1, 22506, 22507, 22508, -1, 22509, 22510, 22511, -1, 22512, 22513, 22514, -1, 22515, 22516, 22517, -1, 22518, 22519, 22520, -1, 22521, 22522, 22523, -1, 22524, 22525, 22526, -1, 22527, 22528, 22529, -1, 22530, 22531, 22532, -1, 22533, 22534, 22535, -1, 22536, 22537, 22538, -1, 22539, 22540, 22541, -1, 22542, 22543, 22544, -1, 22545, 22546, 22547, -1, 22548, 22549, 22550, -1, 22551, 22552, 22553, -1, 22554, 22555, 22556, -1, 22557, 22558, 22559, -1, 22560, 22561, 22562, -1, 22563, 22564, 22565, -1, 22566, 22567, 22568, -1, 22569, 22570, 22571, -1, 22572, 22573, 22574, -1, 22575, 22576, 22577, -1, 22578, 22579, 22580, -1, 22581, 22582, 22583, -1, 22584, 22585, 22586, -1, 22587, 22588, 22589, -1, 22590, 22591, 22592, -1, 22593, 22594, 22595, -1, 22596, 22597, 22598, -1, 22599, 22600, 22601, -1, 22602, 22603, 22604, -1, 22605, 22606, 22607, -1, 22608, 22609, 22610, -1, 22611, 22612, 22613, -1, 22614, 22615, 22616, -1, 22617, 22618, 22619, -1, 22620, 22621, 22622, -1, 22623, 22624, 22625, -1, 22626, 22627, 22628, -1, 22629, 22630, 22631, -1, 22632, 22633, 22634, -1, 22635, 22636, 22637, -1, 22638, 22639, 22640, -1, 22641, 22642, 22643, -1, 22644, 22645, 22646, -1, 22647, 22648, 22649, -1, 22650, 22651, 22652, -1, 22653, 22654, 22655, -1, 22656, 22657, 22658, -1, 22659, 22660, 22661, -1, 22662, 22663, 22664, -1, 22665, 22666, 22667, -1, 22668, 22669, 22670, -1, 22671, 22672, 22673, -1, 22674, 22675, 22676, -1, 22677, 22678, 22679, -1, 22680, 22681, 22682, -1, 22683, 22684, 22685, -1, 22686, 22687, 22688, -1, 22689, 22690, 22691, -1, 22692, 22693, 22694, -1, 22695, 22696, 22697, -1, 22698, 22699, 22700, -1, 22701, 22702, 22703, -1, 22704, 22705, 22706, -1, 22707, 22708, 22709, -1, 22710, 22711, 22712, -1, 22713, 22714, 22715, -1, 22716, 22717, 22718, -1, 22719, 22720, 22721, -1, 22722, 22723, 22724, -1, 22725, 22726, 22727, -1, 22728, 22729, 22730, -1, 22731, 22732, 22733, -1, 22734, 22735, 22736, -1, 22737, 22738, 22739, -1, 22740, 22741, 22742, -1, 22743, 22744, 22745, -1, 22746, 22747, 22748, -1, 22749, 22750, 22751, -1, 22752, 22753, 22754, -1, 22755, 22756, 22757, -1, 22758, 22759, 22760, -1, 22761, 22762, 22763, -1, 22764, 22765, 22766, -1, 22767, 22768, 22769, -1, 22770, 22771, 22772, -1, 22773, 22774, 22775, -1, 22776, 22777, 22778, -1, 22779, 22780, 22781, -1, 22782, 22783, 22784, -1, 22785, 22786, 22787, -1, 22788, 22789, 22790, -1, 22791, 22792, 22793, -1, 22794, 22795, 22796, -1, 22797, 22798, 22799, -1, 22800, 22801, 22802, -1, 22803, 22804, 22805, -1, 22806, 22807, 22808, -1, 22809, 22810, 22811, -1, 22812, 22813, 22814, -1, 22815, 22816, 22817, -1, 22818, 22819, 22820, -1, 22821, 22822, 22823, -1, 22824, 22825, 22826, -1, 22827, 22828, 22829, -1, 22830, 22831, 22832, -1, 22833, 22834, 22835, -1, 22836, 22837, 22838, -1, 22839, 22840, 22841, -1, 22842, 22843, 22844, -1, 22845, 22846, 22847, -1, 22848, 22849, 22850, -1, 22851, 22852, 22853, -1, 22854, 22855, 22856, -1, 22857, 22858, 22859, -1, 22860, 22861, 22862, -1, 22863, 22864, 22865, -1, 22866, 22867, 22868, -1, 22869, 22870, 22871, -1, 22872, 22873, 22874, -1, 22875, 22876, 22877, -1, 22878, 22879, 22880, -1, 22881, 22882, 22883, -1, 22884, 22885, 22886, -1, 22887, 22888, 22889, -1, 22890, 22891, 22892, -1, 22893, 22894, 22895, -1, 22896, 22897, 22898, -1, 22899, 22900, 22901, -1, 22902, 22903, 22904, -1, 22905, 22906, 22907, -1, 22908, 22909, 22910, -1, 22911, 22912, 22913, -1, 22914, 22915, 22916, -1, 22917, 22918, 22919, -1, 22920, 22921, 22922, -1, 22923, 22924, 22925, -1, 22926, 22927, 22928, -1, 22929, 22930, 22931, -1, 22932, 22933, 22934, -1, 22935, 22936, 22937, -1, 22938, 22939, 22940, -1, 22941, 22942, 22943, -1, 22944, 22945, 22946, -1, 22947, 22948, 22949, -1, 22950, 22951, 22952, -1, 22953, 22954, 22955, -1, 22956, 22957, 22958, -1, 22959, 22960, 22961, -1, 22962, 22963, 22964, -1, 22965, 22966, 22967, -1, 22968, 22969, 22970, -1, 22971, 22972, 22973, -1, 22974, 22975, 22976, -1, 22977, 22978, 22979, -1, 22980, 22981, 22982, -1, 22983, 22984, 22985, -1, 22986, 22987, 22988, -1, 22989, 22990, 22991, -1, 22992, 22993, 22994, -1, 22995, 22996, 22997, -1, 22998, 22999, 23000, -1, 23001, 23002, 23003, -1, 23004, 23005, 23006, -1, 23007, 23008, 23009, -1, 23010, 23011, 23012, -1, 23013, 23014, 23015, -1, 23016, 23017, 23018, -1, 23019, 23020, 23021, -1, 23022, 23023, 23024, -1, 23025, 23026, 23027, -1, 23028, 23029, 23030, -1, 23031, 23032, 23033, -1, 23034, 23035, 23036, -1, 23037, 23038, 23039, -1, 23040, 23041, 23042, -1, 23043, 23044, 23045, -1, 23046, 23047, 23048, -1, 23049, 23050, 23051, -1, 23052, 23053, 23054, -1, 23055, 23056, 23057, -1, 23058, 23059, 23060, -1, 23061, 23062, 23063, -1, 23064, 23065, 23066, -1, 23067, 23068, 23069, -1, 23070, 23071, 23072, -1, 23073, 23074, 23075, -1, 23076, 23077, 23078, -1, 23079, 23080, 23081, -1, 23082, 23083, 23084, -1, 23085, 23086, 23087, -1, 23088, 23089, 23090, -1, 23091, 23092, 23093, -1, 23094, 23095, 23096, -1, 23097, 23098, 23099, -1, 23100, 23101, 23102, -1, 23103, 23104, 23105, -1, 23106, 23107, 23108, -1, 23109, 23110, 23111, -1, 23112, 23113, 23114, -1, 23115, 23116, 23117, -1, 23118, 23119, 23120, -1, 23121, 23122, 23123, -1, 23124, 23125, 23126, -1, 23127, 23128, 23129, -1, 23130, 23131, 23132, -1, 23133, 23134, 23135, -1, 23136, 23137, 23138, -1, 23139, 23140, 23141, -1, 23142, 23143, 23144, -1, 23145, 23146, 23147, -1, 23148, 23149, 23150, -1, 23151, 23152, 23153, -1, 23154, 23155, 23156, -1, 23157, 23158, 23159, -1, 23160, 23161, 23162, -1, 23163, 23164, 23165, -1, 23166, 23167, 23168, -1, 23169, 23170, 23171, -1, 23172, 23173, 23174, -1, 23175, 23176, 23177, -1, 23178, 23179, 23180, -1, 23181, 23182, 23183, -1, 23184, 23185, 23186, -1, 23187, 23188, 23189, -1, 23190, 23191, 23192, -1, 23193, 23194, 23195, -1, 23196, 23197, 23198, -1, 23199, 23200, 23201, -1, 23202, 23203, 23204, -1, 23205, 23206, 23207, -1, 23208, 23209, 23210, -1, 23211, 23212, 23213, -1, 23214, 23215, 23216, -1, 23217, 23218, 23219, -1, 23220, 23221, 23222, -1, 23223, 23224, 23225, -1, 23226, 23227, 23228, -1, 23229, 23230, 23231, -1, 23232, 23233, 23234, -1, 23235, 23236, 23237, -1, 23238, 23239, 23240, -1, 23241, 23242, 23243, -1, 23244, 23245, 23246, -1, 23247, 23248, 23249, -1, 23250, 23251, 23252, -1, 23253, 23254, 23255, -1, 23256, 23257, 23258, -1, 23259, 23260, 23261, -1, 23262, 23263, 23264, -1, 23265, 23266, 23267, -1, 23268, 23269, 23270, -1, 23271, 23272, 23273, -1, 23274, 23275, 23276, -1, 23277, 23278, 23279, -1, 23280, 23281, 23282, -1, 23283, 23284, 23285, -1, 23286, 23287, 23288, -1, 23289, 23290, 23291, -1, 23292, 23293, 23294, -1, 23295, 23296, 23297, -1, 23298, 23299, 23300, -1, 23301, 23302, 23303, -1, 23304, 23305, 23306, -1, 23307, 23308, 23309, -1, 23310, 23311, 23312, -1, 23313, 23314, 23315, -1, 23316, 23317, 23318, -1, 23319, 23320, 23321, -1, 23322, 23323, 23324, -1, 23325, 23326, 23327, -1, 23328, 23329, 23330, -1, 23331, 23332, 23333, -1, 23334, 23335, 23336, -1, 23337, 23338, 23339, -1, 23340, 23341, 23342, -1, 23343, 23344, 23345, -1, 23346, 23347, 23348, -1, 23349, 23350, 23351, -1, 23352, 23353, 23354, -1, 23355, 23356, 23357, -1, 23358, 23359, 23360, -1, 23361, 23362, 23363, -1, 23364, 23365, 23366, -1, 23367, 23368, 23369, -1, 23370, 23371, 23372, -1, 23373, 23374, 23375, -1, 23376, 23377, 23378, -1, 23379, 23380, 23381, -1, 23382, 23383, 23384, -1, 23385, 23386, 23387, -1, 23388, 23389, 23390, -1, 23391, 23392, 23393, -1, 23394, 23395, 23396, -1, 23397, 23398, 23399, -1, 23400, 23401, 23402, -1, 23403, 23404, 23405, -1, 23406, 23407, 23408, -1, 23409, 23410, 23411, -1, 23412, 23413, 23414, -1, 23415, 23416, 23417, -1, 23418, 23419, 23420, -1, 23421, 23422, 23423, -1, 23424, 23425, 23426, -1, 23427, 23428, 23429, -1, 23430, 23431, 23432, -1, 23433, 23434, 23435, -1, 23436, 23437, 23438, -1, 23439, 23440, 23441, -1, 23442, 23443, 23444, -1, 23445, 23446, 23447, -1, 23448, 23449, 23450, -1, 23451, 23452, 23453, -1, 23454, 23455, 23456, -1, 23457, 23458, 23459, -1, 23460, 23461, 23462, -1, 23463, 23464, 23465, -1, 23466, 23467, 23468, -1, 23469, 23470, 23471, -1, 23472, 23473, 23474, -1, 23475, 23476, 23477, -1, 23478, 23479, 23480, -1, 23481, 23482, 23483, -1, 23484, 23485, 23486, -1, 23487, 23488, 23489, -1, 23490, 23491, 23492, -1, 23493, 23494, 23495, -1, 23496, 23497, 23498, -1, 23499, 23500, 23501, -1, 23502, 23503, 23504, -1, 23505, 23506, 23507, -1, 23508, 23509, 23510, -1, 23511, 23512, 23513, -1, 23514, 23515, 23516, -1, 23517, 23518, 23519, -1, 23520, 23521, 23522, -1, 23523, 23524, 23525, -1, 23526, 23527, 23528, -1, 23529, 23530, 23531, -1, 23532, 23533, 23534, -1, 23535, 23536, 23537, -1, 23538, 23539, 23540, -1, 23541, 23542, 23543, -1, 23544, 23545, 23546, -1, 23547, 23548, 23549, -1, 23550, 23551, 23552, -1, 23553, 23554, 23555, -1, 23556, 23557, 23558, -1, 23559, 23560, 23561, -1, 23562, 23563, 23564, -1, 23565, 23566, 23567, -1, 23568, 23569, 23570, -1, 23571, 23572, 23573, -1, 23574, 23575, 23576, -1, 23577, 23578, 23579, -1, 23580, 23581, 23582, -1, 23583, 23584, 23585, -1, 23586, 23587, 23588, -1, 23589, 23590, 23591, -1, 23592, 23593, 23594, -1, 23595, 23596, 23597, -1, 23598, 23599, 23600, -1, 23601, 23602, 23603, -1, 23604, 23605, 23606, -1, 23607, 23608, 23609, -1, 23610, 23611, 23612, -1, 23613, 23614, 23615, -1, 23616, 23617, 23618, -1, 23619, 23620, 23621, -1, 23622, 23623, 23624, -1, 23625, 23626, 23627, -1, 23628, 23629, 23630, -1, 23631, 23632, 23633, -1, 23634, 23635, 23636, -1, 23637, 23638, 23639, -1, 23640, 23641, 23642, -1, 23643, 23644, 23645, -1, 23646, 23647, 23648, -1, 23649, 23650, 23651, -1, 23652, 23653, 23654, -1, 23655, 23656, 23657, -1, 23658, 23659, 23660, -1, 23661, 23662, 23663, -1, 23664, 23665, 23666, -1, 23667, 23668, 23669, -1, 23670, 23671, 23672, -1, 23673, 23674, 23675, -1, 23676, 23677, 23678, -1, 23679, 23680, 23681, -1, 23682, 23683, 23684, -1, 23685, 23686, 23687, -1, 23688, 23689, 23690, -1, 23691, 23692, 23693, -1, 23694, 23695, 23696, -1, 23697, 23698, 23699, -1, 23700, 23701, 23702, -1, 23703, 23704, 23705, -1, 23706, 23707, 23708, -1, 23709, 23710, 23711, -1, 23712, 23713, 23714, -1, 23715, 23716, 23717, -1, 23718, 23719, 23720, -1, 23721, 23722, 23723, -1, 23724, 23725, 23726, -1, 23727, 23728, 23729, -1, 23730, 23731, 23732, -1, 23733, 23734, 23735, -1, 23736, 23737, 23738, -1, 23739, 23740, 23741, -1, 23742, 23743, 23744, -1, 23745, 23746, 23747, -1, 23748, 23749, 23750, -1, 23751, 23752, 23753, -1, 23754, 23755, 23756, -1, 23757, 23758, 23759, -1, 23760, 23761, 23762, -1, 23763, 23764, 23765, -1, 23766, 23767, 23768, -1, 23769, 23770, 23771, -1, 23772, 23773, 23774, -1, 23775, 23776, 23777, -1, 23778, 23779, 23780, -1, 23781, 23782, 23783, -1, 23784, 23785, 23786, -1, 23787, 23788, 23789, -1, 23790, 23791, 23792, -1, 23793, 23794, 23795, -1, 23796, 23797, 23798, -1, 23799, 23800, 23801, -1, 23802, 23803, 23804, -1, 23805, 23806, 23807, -1, 23808, 23809, 23810, -1, 23811, 23812, 23813, -1, 23814, 23815, 23816, -1, 23817, 23818, 23819, -1, 23820, 23821, 23822, -1, 23823, 23824, 23825, -1, 23826, 23827, 23828, -1, 23829, 23830, 23831, -1, 23832, 23833, 23834, -1, 23835, 23836, 23837, -1, 23838, 23839, 23840, -1, 23841, 23842, 23843, -1, 23844, 23845, 23846, -1, 23847, 23848, 23849, -1, 23850, 23851, 23852, -1, 23853, 23854, 23855, -1, 23856, 23857, 23858, -1, 23859, 23860, 23861, -1, 23862, 23863, 23864, -1, 23865, 23866, 23867, -1, 23868, 23869, 23870, -1, 23871, 23872, 23873, -1, 23874, 23875, 23876, -1, 23877, 23878, 23879, -1, 23880, 23881, 23882, -1, 23883, 23884, 23885, -1, 23886, 23887, 23888, -1, 23889, 23890, 23891, -1, 23892, 23893, 23894, -1, 23895, 23896, 23897, -1, 23898, 23899, 23900, -1, 23901, 23902, 23903, -1, 23904, 23905, 23906, -1, 23907, 23908, 23909, -1, 23910, 23911, 23912, -1, 23913, 23914, 23915, -1, 23916, 23917, 23918, -1, 23919, 23920, 23921, -1, 23922, 23923, 23924, -1, 23925, 23926, 23927, -1, 23928, 23929, 23930, -1, 23931, 23932, 23933, -1, 23934, 23935, 23936, -1, 23937, 23938, 23939, -1, 23940, 23941, 23942, -1, 23943, 23944, 23945, -1, 23946, 23947, 23948, -1, 23949, 23950, 23951, -1, 23952, 23953, 23954, -1, 23955, 23956, 23957, -1, 23958, 23959, 23960, -1, 23961, 23962, 23963, -1, 23964, 23965, 23966, -1, 23967, 23968, 23969, -1, 23970, 23971, 23972, -1, 23973, 23974, 23975, -1, 23976, 23977, 23978, -1, 23979, 23980, 23981, -1, 23982, 23983, 23984, -1, 23985, 23986, 23987, -1, 23988, 23989, 23990, -1, 23991, 23992, 23993, -1, 23994, 23995, 23996, -1, 23997, 23998, 23999, -1, 24000, 24001, 24002, -1, 24003, 24004, 24005, -1, 24006, 24007, 24008, -1, 24009, 24010, 24011, -1, 24012, 24013, 24014, -1, 24015, 24016, 24017, -1, 24018, 24019, 24020, -1, 24021, 24022, 24023, -1, 24024, 24025, 24026, -1, 24027, 24028, 24029, -1, 24030, 24031, 24032, -1, 24033, 24034, 24035, -1, 24036, 24037, 24038, -1, 24039, 24040, 24041, -1, 24042, 24043, 24044, -1, 24045, 24046, 24047, -1, 24048, 24049, 24050, -1, 24051, 24052, 24053, -1, 24054, 24055, 24056, -1, 24057, 24058, 24059, -1, 24060, 24061, 24062, -1, 24063, 24064, 24065, -1, 24066, 24067, 24068, -1, 24069, 24070, 24071, -1, 24072, 24073, 24074, -1, 24075, 24076, 24077, -1, 24078, 24079, 24080, -1, 24081, 24082, 24083, -1, 24084, 24085, 24086, -1, 24087, 24088, 24089, -1, 24090, 24091, 24092, -1, 24093, 24094, 24095, -1, 24096, 24097, 24098, -1, 24099, 24100, 24101, -1, 24102, 24103, 24104, -1, 24105, 24106, 24107, -1, 24108, 24109, 24110, -1, 24111, 24112, 24113, -1, 24114, 24115, 24116, -1, 24117, 24118, 24119, -1, 24120, 24121, 24122, -1, 24123, 24124, 24125, -1, 24126, 24127, 24128, -1, 24129, 24130, 24131, -1, 24132, 24133, 24134, -1, 24135, 24136, 24137, -1, 24138, 24139, 24140, -1, 24141, 24142, 24143, -1, 24144, 24145, 24146, -1, 24147, 24148, 24149, -1, 24150, 24151, 24152, -1, 24153, 24154, 24155, -1, 24156, 24157, 24158, -1, 24159, 24160, 24161, -1, 24162, 24163, 24164, -1, 24165, 24166, 24167, -1, 24168, 24169, 24170, -1, 24171, 24172, 24173, -1, 24174, 24175, 24176, -1, 24177, 24178, 24179, -1, 24180, 24181, 24182, -1, 24183, 24184, 24185, -1, 24186, 24187, 24188, -1, 24189, 24190, 24191, -1, 24192, 24193, 24194, -1, 24195, 24196, 24197, -1, 24198, 24199, 24200, -1, 24201, 24202, 24203, -1, 24204, 24205, 24206, -1, 24207, 24208, 24209, -1, 24210, 24211, 24212, -1, 24213, 24214, 24215, -1, 24216, 24217, 24218, -1, 24219, 24220, 24221, -1, 24222, 24223, 24224, -1, 24225, 24226, 24227, -1, 24228, 24229, 24230, -1, 24231, 24232, 24233, -1, 24234, 24235, 24236, -1, 24237, 24238, 24239, -1, 24240, 24241, 24242, -1, 24243, 24244, 24245, -1, 24246, 24247, 24248, -1, 24249, 24250, 24251, -1, 24252, 24253, 24254, -1, 24255, 24256, 24257, -1, 24258, 24259, 24260, -1, 24261, 24262, 24263, -1, 24264, 24265, 24266, -1, 24267, 24268, 24269, -1, 24270, 24271, 24272, -1, 24273, 24274, 24275, -1, 24276, 24277, 24278, -1, 24279, 24280, 24281, -1, 24282, 24283, 24284, -1, 24285, 24286, 24287, -1, 24288, 24289, 24290, -1, 24291, 24292, 24293, -1, 24294, 24295, 24296, -1, 24297, 24298, 24299, -1, 24300, 24301, 24302, -1, 24303, 24304, 24305, -1, 24306, 24307, 24308, -1, 24309, 24310, 24311, -1, 24312, 24313, 24314, -1, 24315, 24316, 24317, -1, 24318, 24319, 24320, -1, 24321, 24322, 24323, -1, 24324, 24325, 24326, -1, 24327, 24328, 24329, -1, 24330, 24331, 24332, -1, 24333, 24334, 24335, -1, 24336, 24337, 24338, -1, 24339, 24340, 24341, -1, 24342, 24343, 24344, -1, 24345, 24346, 24347, -1, 24348, 24349, 24350, -1, 24351, 24352, 24353, -1, 24354, 24355, 24356, -1, 24357, 24358, 24359, -1, 24360, 24361, 24362, -1, 24363, 24364, 24365, -1, 24366, 24367, 24368, -1, 24369, 24370, 24371, -1, 24372, 24373, 24374, -1, 24375, 24376, 24377, -1, 24378, 24379, 24380, -1, 24381, 24382, 24383, -1, 24384, 24385, 24386, -1, 24387, 24388, 24389, -1, 24390, 24391, 24392, -1, 24393, 24394, 24395, -1, 24396, 24397, 24398, -1, 24399, 24400, 24401, -1, 24402, 24403, 24404, -1, 24405, 24406, 24407, -1, 24408, 24409, 24410, -1, 24411, 24412, 24413, -1, 24414, 24415, 24416, -1, 24417, 24418, 24419, -1, 24420, 24421, 24422, -1, 24423, 24424, 24425, -1, 24426, 24427, 24428, -1, 24429, 24430, 24431, -1, 24432, 24433, 24434, -1, 24435, 24436, 24437, -1, 24438, 24439, 24440, -1, 24441, 24442, 24443, -1, 24444, 24445, 24446, -1, 24447, 24448, 24449, -1, 24450, 24451, 24452, -1, 24453, 24454, 24455, -1, 24456, 24457, 24458, -1, 24459, 24460, 24461, -1, 24462, 24463, 24464, -1, 24465, 24466, 24467, -1, 24468, 24469, 24470, -1, 24471, 24472, 24473, -1, 24474, 24475, 24476, -1, 24477, 24478, 24479, -1, 24480, 24481, 24482, -1, 24483, 24484, 24485, -1, 24486, 24487, 24488, -1, 24489, 24490, 24491, -1, 24492, 24493, 24494, -1, 24495, 24496, 24497, -1, 24498, 24499, 24500, -1, 24501, 24502, 24503, -1, 24504, 24505, 24506, -1, 24507, 24508, 24509, -1, 24510, 24511, 24512, -1, 24513, 24514, 24515, -1, 24516, 24517, 24518, -1, 24519, 24520, 24521, -1, 24522, 24523, 24524, -1, 24525, 24526, 24527, -1, 24528, 24529, 24530, -1, 24531, 24532, 24533, -1, 24534, 24535, 24536, -1, 24537, 24538, 24539, -1, 24540, 24541, 24542, -1, 24543, 24544, 24545, -1, 24546, 24547, 24548, -1, 24549, 24550, 24551, -1, 24552, 24553, 24554, -1, 24555, 24556, 24557, -1, 24558, 24559, 24560, -1, 24561, 24562, 24563, -1, 24564, 24565, 24566, -1, 24567, 24568, 24569, -1, 24570, 24571, 24572, -1, 24573, 24574, 24575, -1, 24576, 24577, 24578, -1, 24579, 24580, 24581, -1, 24582, 24583, 24584, -1, 24585, 24586, 24587, -1, 24588, 24589, 24590, -1, 24591, 24592, 24593, -1, 24594, 24595, 24596, -1, 24597, 24598, 24599, -1, 24600, 24601, 24602, -1, 24603, 24604, 24605, -1, 24606, 24607, 24608, -1, 24609, 24610, 24611, -1, 24612, 24613, 24614, -1, 24615, 24616, 24617, -1, 24618, 24619, 24620, -1, 24621, 24622, 24623, -1, 24624, 24625, 24626, -1, 24627, 24628, 24629, -1, 24630, 24631, 24632, -1, 24633, 24634, 24635, -1, 24636, 24637, 24638, -1, 24639, 24640, 24641, -1, 24642, 24643, 24644, -1, 24645, 24646, 24647, -1, 24648, 24649, 24650, -1, 24651, 24652, 24653, -1, 24654, 24655, 24656, -1, 24657, 24658, 24659, -1, 24660, 24661, 24662, -1, 24663, 24664, 24665, -1, 24666, 24667, 24668, -1, 24669, 24670, 24671, -1, 24672, 24673, 24674, -1, 24675, 24676, 24677, -1, 24678, 24679, 24680, -1, 24681, 24682, 24683, -1, 24684, 24685, 24686, -1, 24687, 24688, 24689, -1, 24690, 24691, 24692, -1, 24693, 24694, 24695, -1, 24696, 24697, 24698, -1, 24699, 24700, 24701, -1, 24702, 24703, 24704, -1, 24705, 24706, 24707, -1, 24708, 24709, 24710, -1, 24711, 24712, 24713, -1, 24714, 24715, 24716, -1, 24717, 24718, 24719, -1, 24720, 24721, 24722, -1, 24723, 24724, 24725, -1, 24726, 24727, 24728, -1, 24729, 24730, 24731, -1, 24732, 24733, 24734, -1, 24735, 24736, 24737, -1, 24738, 24739, 24740, -1, 24741, 24742, 24743, -1, 24744, 24745, 24746, -1, 24747, 24748, 24749, -1, 24750, 24751, 24752, -1, 24753, 24754, 24755, -1, 24756, 24757, 24758, -1, 24759, 24760, 24761, -1, 24762, 24763, 24764, -1, 24765, 24766, 24767, -1, 24768, 24769, 24770, -1, 24771, 24772, 24773, -1, 24774, 24775, 24776, -1, 24777, 24778, 24779, -1, 24780, 24781, 24782, -1, 24783, 24784, 24785, -1, 24786, 24787, 24788, -1, 24789, 24790, 24791, -1, 24792, 24793, 24794, -1, 24795, 24796, 24797, -1, 24798, 24799, 24800, -1, 24801, 24802, 24803, -1, 24804, 24805, 24806, -1, 24807, 24808, 24809, -1, 24810, 24811, 24812, -1, 24813, 24814, 24815, -1, 24816, 24817, 24818, -1, 24819, 24820, 24821, -1, 24822, 24823, 24824, -1, 24825, 24826, 24827, -1, 24828, 24829, 24830, -1, 24831, 24832, 24833, -1, 24834, 24835, 24836, -1, 24837, 24838, 24839, -1, 24840, 24841, 24842, -1, 24843, 24844, 24845, -1, 24846, 24847, 24848, -1, 24849, 24850, 24851, -1, 24852, 24853, 24854, -1, 24855, 24856, 24857, -1, 24858, 24859, 24860, -1, 24861, 24862, 24863, -1, 24864, 24865, 24866, -1, 24867, 24868, 24869, -1, 24870, 24871, 24872, -1, 24873, 24874, 24875, -1, 24876, 24877, 24878, -1, 24879, 24880, 24881, -1, 24882, 24883, 24884, -1, 24885, 24886, 24887, -1, 24888, 24889, 24890, -1, 24891, 24892, 24893, -1, 24894, 24895, 24896, -1, 24897, 24898, 24899, -1, 24900, 24901, 24902, -1, 24903, 24904, 24905, -1, 24906, 24907, 24908, -1, 24909, 24910, 24911, -1, 24912, 24913, 24914, -1, 24915, 24916, 24917, -1, 24918, 24919, 24920, -1, 24921, 24922, 24923, -1, 24924, 24925, 24926, -1, 24927, 24928, 24929, -1, 24930, 24931, 24932, -1, 24933, 24934, 24935, -1, 24936, 24937, 24938, -1, 24939, 24940, 24941, -1, 24942, 24943, 24944, -1, 24945, 24946, 24947, -1, 24948, 24949, 24950, -1, 24951, 24952, 24953, -1, 24954, 24955, 24956, -1, 24957, 24958, 24959, -1, 24960, 24961, 24962, -1, 24963, 24964, 24965, -1, 24966, 24967, 24968, -1, 24969, 24970, 24971, -1, 24972, 24973, 24974, -1, 24975, 24976, 24977, -1, 24978, 24979, 24980, -1, 24981, 24982, 24983, -1, 24984, 24985, 24986, -1, 24987, 24988, 24989, -1, 24990, 24991, 24992, -1, 24993, 24994, 24995, -1, 24996, 24997, 24998, -1, 24999, 25000, 25001, -1, 25002, 25003, 25004, -1, 25005, 25006, 25007, -1, 25008, 25009, 25010, -1, 25011, 25012, 25013, -1, 25014, 25015, 25016, -1, 25017, 25018, 25019, -1, 25020, 25021, 25022, -1, 25023, 25024, 25025, -1, 25026, 25027, 25028, -1, 25029, 25030, 25031, -1, 25032, 25033, 25034, -1, 25035, 25036, 25037, -1, 25038, 25039, 25040, -1, 25041, 25042, 25043, -1, 25044, 25045, 25046, -1, 25047, 25048, 25049, -1, 25050, 25051, 25052, -1, 25053, 25054, 25055, -1, 25056, 25057, 25058, -1, 25059, 25060, 25061, -1, 25062, 25063, 25064, -1, 25065, 25066, 25067, -1, 25068, 25069, 25070, -1, 25071, 25072, 25073, -1, 25074, 25075, 25076, -1, 25077, 25078, 25079, -1, 25080, 25081, 25082, -1, 25083, 25084, 25085, -1, 25086, 25087, 25088, -1, 25089, 25090, 25091, -1, 25092, 25093, 25094, -1, 25095, 25096, 25097, -1, 25098, 25099, 25100, -1, 25101, 25102, 25103, -1, 25104, 25105, 25106, -1, 25107, 25108, 25109, -1, 25110, 25111, 25112, -1, 25113, 25114, 25115, -1, 25116, 25117, 25118, -1, 25119, 25120, 25121, -1, 25122, 25123, 25124, -1, 25125, 25126, 25127, -1, 25128, 25129, 25130, -1, 25131, 25132, 25133, -1, 25134, 25135, 25136, -1, 25137, 25138, 25139, -1, 25140, 25141, 25142, -1, 25143, 25144, 25145, -1, 25146, 25147, 25148, -1, 25149, 25150, 25151, -1, 25152, 25153, 25154, -1, 25155, 25156, 25157, -1, 25158, 25159, 25160, -1, 25161, 25162, 25163, -1, 25164, 25165, 25166, -1, 25167, 25168, 25169, -1, 25170, 25171, 25172, -1, 25173, 25174, 25175, -1, 25176, 25177, 25178, -1, 25179, 25180, 25181, -1, 25182, 25183, 25184, -1, 25185, 25186, 25187, -1, 25188, 25189, 25190, -1, 25191, 25192, 25193, -1, 25194, 25195, 25196, -1, 25197, 25198, 25199, -1, 25200, 25201, 25202, -1, 25203, 25204, 25205, -1, 25206, 25207, 25208, -1, 25209, 25210, 25211, -1, 25212, 25213, 25214, -1, 25215, 25216, 25217, -1, 25218, 25219, 25220, -1, 25221, 25222, 25223, -1, 25224, 25225, 25226, -1, 25227, 25228, 25229, -1, 25230, 25231, 25232, -1, 25233, 25234, 25235, -1, 25236, 25237, 25238, -1, 25239, 25240, 25241, -1, 25242, 25243, 25244, -1, 25245, 25246, 25247, -1, 25248, 25249, 25250, -1, 25251, 25252, 25253, -1, 25254, 25255, 25256, -1, 25257, 25258, 25259, -1, 25260, 25261, 25262, -1, 25263, 25264, 25265, -1, 25266, 25267, 25268, -1, 25269, 25270, 25271, -1, 25272, 25273, 25274, -1, 25275, 25276, 25277, -1, 25278, 25279, 25280, -1, 25281, 25282, 25283, -1, 25284, 25285, 25286, -1, 25287, 25288, 25289, -1, 25290, 25291, 25292, -1, 25293, 25294, 25295, -1, 25296, 25297, 25298, -1, 25299, 25300, 25301, -1, 25302, 25303, 25304, -1, 25305, 25306, 25307, -1, 25308, 25309, 25310, -1, 25311, 25312, 25313, -1, 25314, 25315, 25316, -1, 25317, 25318, 25319, -1, 25320, 25321, 25322, -1, 25323, 25324, 25325, -1, 25326, 25327, 25328, -1, 25329, 25330, 25331, -1, 25332, 25333, 25334, -1, 25335, 25336, 25337, -1, 25338, 25339, 25340, -1, 25341, 25342, 25343, -1, 25344, 25345, 25346, -1, 25347, 25348, 25349, -1, 25350, 25351, 25352, -1, 25353, 25354, 25355, -1, 25356, 25357, 25358, -1, 25359, 25360, 25361, -1, 25362, 25363, 25364, -1, 25365, 25366, 25367, -1, 25368, 25369, 25370, -1, 25371, 25372, 25373, -1, 25374, 25375, 25376, -1, 25377, 25378, 25379, -1, 25380, 25381, 25382, -1, 25383, 25384, 25385, -1, 25386, 25387, 25388, -1, 25389, 25390, 25391, -1, 25392, 25393, 25394, -1, 25395, 25396, 25397, -1, 25398, 25399, 25400, -1, 25401, 25402, 25403, -1, 25404, 25405, 25406, -1, 25407, 25408, 25409, -1, 25410, 25411, 25412, -1, 25413, 25414, 25415, -1, 25416, 25417, 25418, -1, 25419, 25420, 25421, -1, 25422, 25423, 25424, -1, 25425, 25426, 25427, -1, 25428, 25429, 25430, -1, 25431, 25432, 25433, -1, 25434, 25435, 25436, -1, 25437, 25438, 25439, -1, 25440, 25441, 25442, -1, 25443, 25444, 25445, -1, 25446, 25447, 25448, -1, 25449, 25450, 25451, -1, 25452, 25453, 25454, -1, 25455, 25456, 25457, -1, 25458, 25459, 25460, -1, 25461, 25462, 25463, -1, 25464, 25465, 25466, -1, 25467, 25468, 25469, -1, 25470, 25471, 25472, -1, 25473, 25474, 25475, -1, 25476, 25477, 25478, -1, 25479, 25480, 25481, -1, 25482, 25483, 25484, -1, 25485, 25486, 25487, -1, 25488, 25489, 25490, -1, 25491, 25492, 25493, -1, 25494, 25495, 25496, -1, 25497, 25498, 25499, -1, 25500, 25501, 25502, -1, 25503, 25504, 25505, -1, 25506, 25507, 25508, -1, 25509, 25510, 25511, -1, 25512, 25513, 25514, -1, 25515, 25516, 25517, -1, 25518, 25519, 25520, -1, 25521, 25522, 25523, -1, 25524, 25525, 25526, -1, 25527, 25528, 25529, -1, 25530, 25531, 25532, -1, 25533, 25534, 25535, -1, 25536, 25537, 25538, -1, 25539, 25540, 25541, -1, 25542, 25543, 25544, -1, 25545, 25546, 25547, -1, 25548, 25549, 25550, -1, 25551, 25552, 25553, -1, 25554, 25555, 25556, -1, 25557, 25558, 25559, -1, 25560, 25561, 25562, -1, 25563, 25564, 25565, -1, 25566, 25567, 25568, -1, 25569, 25570, 25571, -1, 25572, 25573, 25574, -1, 25575, 25576, 25577, -1, 25578, 25579, 25580, -1, 25581, 25582, 25583, -1, 25584, 25585, 25586, -1, 25587, 25588, 25589, -1, 25590, 25591, 25592, -1, 25593, 25594, 25595, -1, 25596, 25597, 25598, -1, 25599, 25600, 25601, -1, 25602, 25603, 25604, -1, 25605, 25606, 25607, -1, 25608, 25609, 25610, -1, 25611, 25612, 25613, -1, 25614, 25615, 25616, -1, 25617, 25618, 25619, -1, 25620, 25621, 25622, -1, 25623, 25624, 25625, -1, 25626, 25627, 25628, -1, 25629, 25630, 25631, -1, 25632, 25633, 25634, -1, 25635, 25636, 25637, -1, 25638, 25639, 25640, -1, 25641, 25642, 25643, -1, 25644, 25645, 25646, -1, 25647, 25648, 25649, -1, 25650, 25651, 25652, -1, 25653, 25654, 25655, -1, 25656, 25657, 25658, -1, 25659, 25660, 25661, -1, 25662, 25663, 25664, -1, 25665, 25666, 25667, -1, 25668, 25669, 25670, -1, 25671, 25672, 25673, -1, 25674, 25675, 25676, -1, 25677, 25678, 25679, -1, 25680, 25681, 25682, -1, 25683, 25684, 25685, -1, 25686, 25687, 25688, -1, 25689, 25690, 25691, -1, 25692, 25693, 25694, -1, 25695, 25696, 25697, -1, 25698, 25699, 25700, -1, 25701, 25702, 25703, -1, 25704, 25705, 25706, -1, 25707, 25708, 25709, -1, 25710, 25711, 25712, -1, 25713, 25714, 25715, -1, 25716, 25717, 25718, -1, 25719, 25720, 25721, -1, 25722, 25723, 25724, -1, 25725, 25726, 25727, -1, 25728, 25729, 25730, -1, 25731, 25732, 25733, -1, 25734, 25735, 25736, -1, 25737, 25738, 25739, -1, 25740, 25741, 25742, -1, 25743, 25744, 25745, -1, 25746, 25747, 25748, -1, 25749, 25750, 25751, -1, 25752, 25753, 25754, -1, 25755, 25756, 25757, -1, 25758, 25759, 25760, -1, 25761, 25762, 25763, -1, 25764, 25765, 25766, -1, 25767, 25768, 25769, -1, 25770, 25771, 25772, -1, 25773, 25774, 25775, -1, 25776, 25777, 25778, -1, 25779, 25780, 25781, -1, 25782, 25783, 25784, -1, 25785, 25786, 25787, -1, 25788, 25789, 25790, -1, 25791, 25792, 25793, -1, 25794, 25795, 25796, -1, 25797, 25798, 25799, -1, 25800, 25801, 25802, -1, 25803, 25804, 25805, -1, 25806, 25807, 25808, -1, 25809, 25810, 25811, -1, 25812, 25813, 25814, -1, 25815, 25816, 25817, -1, 25818, 25819, 25820, -1, 25821, 25822, 25823, -1, 25824, 25825, 25826, -1, 25827, 25828, 25829, -1, 25830, 25831, 25832, -1, 25833, 25834, 25835, -1, 25836, 25837, 25838, -1, 25839, 25840, 25841, -1, 25842, 25843, 25844, -1, 25845, 25846, 25847, -1, 25848, 25849, 25850, -1, 25851, 25852, 25853, -1, 25854, 25855, 25856, -1, 25857, 25858, 25859, -1, 25860, 25861, 25862, -1, 25863, 25864, 25865, -1, 25866, 25867, 25868, -1, 25869, 25870, 25871, -1, 25872, 25873, 25874, -1, 25875, 25876, 25877, -1, 25878, 25879, 25880, -1, 25881, 25882, 25883, -1, 25884, 25885, 25886, -1, 25887, 25888, 25889, -1, 25890, 25891, 25892, -1, 25893, 25894, 25895, -1, 25896, 25897, 25898, -1, 25899, 25900, 25901, -1, 25902, 25903, 25904, -1, 25905, 25906, 25907, -1, 25908, 25909, 25910, -1, 25911, 25912, 25913, -1, 25914, 25915, 25916, -1, 25917, 25918, 25919, -1, 25920, 25921, 25922, -1, 25923, 25924, 25925, -1, 25926, 25927, 25928, -1, 25929, 25930, 25931, -1, 25932, 25933, 25934, -1, 25935, 25936, 25937, -1, 25938, 25939, 25940, -1, 25941, 25942, 25943, -1, 25944, 25945, 25946, -1, 25947, 25948, 25949, -1, 25950, 25951, 25952, -1, 25953, 25954, 25955, -1, 25956, 25957, 25958, -1, 25959, 25960, 25961, -1, 25962, 25963, 25964, -1, 25965, 25966, 25967, -1, 25968, 25969, 25970, -1, 25971, 25972, 25973, -1, 25974, 25975, 25976, -1, 25977, 25978, 25979, -1, 25980, 25981, 25982, -1, 25983, 25984, 25985, -1, 25986, 25987, 25988, -1, 25989, 25990, 25991, -1, 25992, 25993, 25994, -1, 25995, 25996, 25997, -1, 25998, 25999, 26000, -1, 26001, 26002, 26003, -1, 26004, 26005, 26006, -1, 26007, 26008, 26009, -1, 26010, 26011, 26012, -1, 26013, 26014, 26015, -1, 26016, 26017, 26018, -1, 26019, 26020, 26021, -1, 26022, 26023, 26024, -1, 26025, 26026, 26027, -1, 26028, 26029, 26030, -1, 26031, 26032, 26033, -1, 26034, 26035, 26036, -1, 26037, 26038, 26039, -1, 26040, 26041, 26042, -1, 26043, 26044, 26045, -1, 26046, 26047, 26048, -1, 26049, 26050, 26051, -1, 26052, 26053, 26054, -1, 26055, 26056, 26057, -1, 26058, 26059, 26060, -1, 26061, 26062, 26063, -1, 26064, 26065, 26066, -1, 26067, 26068, 26069, -1, 26070, 26071, 26072, -1, 26073, 26074, 26075, -1, 26076, 26077, 26078, -1, 26079, 26080, 26081, -1, 26082, 26083, 26084, -1, 26085, 26086, 26087, -1, 26088, 26089, 26090, -1, 26091, 26092, 26093, -1, 26094, 26095, 26096, -1, 26097, 26098, 26099, -1, 26100, 26101, 26102, -1, 26103, 26104, 26105, -1, 26106, 26107, 26108, -1, 26109, 26110, 26111, -1, 26112, 26113, 26114, -1, 26115, 26116, 26117, -1, 26118, 26119, 26120, -1, 26121, 26122, 26123, -1, 26124, 26125, 26126, -1, 26127, 26128, 26129, -1, 26130, 26131, 26132, -1, 26133, 26134, 26135, -1, 26136, 26137, 26138, -1, 26139, 26140, 26141, -1, 26142, 26143, 26144, -1, 26145, 26146, 26147, -1, 26148, 26149, 26150, -1, 26151, 26152, 26153, -1, 26154, 26155, 26156, -1, 26157, 26158, 26159, -1, 26160, 26161, 26162, -1, 26163, 26164, 26165, -1, 26166, 26167, 26168, -1, 26169, 26170, 26171, -1, 26172, 26173, 26174, -1, 26175, 26176, 26177, -1, 26178, 26179, 26180, -1, 26181, 26182, 26183, -1, 26184, 26185, 26186, -1, 26187, 26188, 26189, -1, 26190, 26191, 26192, -1, 26193, 26194, 26195, -1, 26196, 26197, 26198, -1, 26199, 26200, 26201, -1, 26202, 26203, 26204, -1, 26205, 26206, 26207, -1, 26208, 26209, 26210, -1, 26211, 26212, 26213, -1, 26214, 26215, 26216, -1, 26217, 26218, 26219, -1, 26220, 26221, 26222, -1, 26223, 26224, 26225, -1, 26226, 26227, 26228, -1, 26229, 26230, 26231, -1, 26232, 26233, 26234, -1, 26235, 26236, 26237, -1, 26238, 26239, 26240, -1, 26241, 26242, 26243, -1, 26244, 26245, 26246, -1, 26247, 26248, 26249, -1, 26250, 26251, 26252, -1, 26253, 26254, 26255, -1, 26256, 26257, 26258, -1, 26259, 26260, 26261, -1, 26262, 26263, 26264, -1, 26265, 26266, 26267, -1, 26268, 26269, 26270, -1, 26271, 26272, 26273, -1, 26274, 26275, 26276, -1, 26277, 26278, 26279, -1, 26280, 26281, 26282, -1, 26283, 26284, 26285, -1, 26286, 26287, 26288, -1, 26289, 26290, 26291, -1, 26292, 26293, 26294, -1, 26295, 26296, 26297, -1, 26298, 26299, 26300, -1, 26301, 26302, 26303, -1, 26304, 26305, 26306, -1, 26307, 26308, 26309, -1, 26310, 26311, 26312, -1, 26313, 26314, 26315, -1, 26316, 26317, 26318, -1, 26319, 26320, 26321, -1, 26322, 26323, 26324, -1, 26325, 26326, 26327, -1, 26328, 26329, 26330, -1, 26331, 26332, 26333, -1, 26334, 26335, 26336, -1, 26337, 26338, 26339, -1, 26340, 26341, 26342, -1, 26343, 26344, 26345, -1, 26346, 26347, 26348, -1, 26349, 26350, 26351, -1, 26352, 26353, 26354, -1, 26355, 26356, 26357, -1, 26358, 26359, 26360, -1, 26361, 26362, 26363, -1, 26364, 26365, 26366, -1, 26367, 26368, 26369, -1, 26370, 26371, 26372, -1, 26373, 26374, 26375, -1, 26376, 26377, 26378, -1, 26379, 26380, 26381, -1, 26382, 26383, 26384, -1, 26385, 26386, 26387, -1, 26388, 26389, 26390, -1, 26391, 26392, 26393, -1, 26394, 26395, 26396, -1, 26397, 26398, 26399, -1, 26400, 26401, 26402, -1, 26403, 26404, 26405, -1, 26406, 26407, 26408, -1, 26409, 26410, 26411, -1, 26412, 26413, 26414, -1, 26415, 26416, 26417, -1, 26418, 26419, 26420, -1, 26421, 26422, 26423, -1, 26424, 26425, 26426, -1, 26427, 26428, 26429, -1, 26430, 26431, 26432, -1, 26433, 26434, 26435, -1, 26436, 26437, 26438, -1, 26439, 26440, 26441, -1, 26442, 26443, 26444, -1, 26445, 26446, 26447, -1, 26448, 26449, 26450, -1, 26451, 26452, 26453, -1, 26454, 26455, 26456, -1, 26457, 26458, 26459, -1, 26460, 26461, 26462, -1, 26463, 26464, 26465, -1, 26466, 26467, 26468, -1, 26469, 26470, 26471, -1, 26472, 26473, 26474, -1, 26475, 26476, 26477, -1, 26478, 26479, 26480, -1, 26481, 26482, 26483, -1, 26484, 26485, 26486, -1, 26487, 26488, 26489, -1, 26490, 26491, 26492, -1, 26493, 26494, 26495, -1, 26496, 26497, 26498, -1, 26499, 26500, 26501, -1, 26502, 26503, 26504, -1, 26505, 26506, 26507, -1, 26508, 26509, 26510, -1, 26511, 26512, 26513, -1, 26514, 26515, 26516, -1, 26517, 26518, 26519, -1, 26520, 26521, 26522, -1, 26523, 26524, 26525, -1, 26526, 26527, 26528, -1, 26529, 26530, 26531, -1, 26532, 26533, 26534, -1, 26535, 26536, 26537, -1, 26538, 26539, 26540, -1, 26541, 26542, 26543, -1, 26544, 26545, 26546, -1, 26547, 26548, 26549, -1, 26550, 26551, 26552, -1, 26553, 26554, 26555, -1, 26556, 26557, 26558, -1, 26559, 26560, 26561, -1, 26562, 26563, 26564, -1, 26565, 26566, 26567, -1, 26568, 26569, 26570, -1, 26571, 26572, 26573, -1, 26574, 26575, 26576, -1, 26577, 26578, 26579, -1, 26580, 26581, 26582, -1, 26583, 26584, 26585, -1, 26586, 26587, 26588, -1, 26589, 26590, 26591, -1, 26592, 26593, 26594, -1, 26595, 26596, 26597, -1, 26598, 26599, 26600, -1, 26601, 26602, 26603, -1, 26604, 26605, 26606, -1, 26607, 26608, 26609, -1, 26610, 26611, 26612, -1, 26613, 26614, 26615, -1, 26616, 26617, 26618, -1, 26619, 26620, 26621, -1, 26622, 26623, 26624, -1, 26625, 26626, 26627, -1, 26628, 26629, 26630, -1, 26631, 26632, 26633, -1, 26634, 26635, 26636, -1, 26637, 26638, 26639, -1, 26640, 26641, 26642, -1, 26643, 26644, 26645, -1, 26646, 26647, 26648, -1, 26649, 26650, 26651, -1, 26652, 26653, 26654, -1, 26655, 26656, 26657, -1, 26658, 26659, 26660, -1, 26661, 26662, 26663, -1, 26664, 26665, 26666, -1, 26667, 26668, 26669, -1, 26670, 26671, 26672, -1, 26673, 26674, 26675, -1, 26676, 26677, 26678, -1, 26679, 26680, 26681, -1, 26682, 26683, 26684, -1, 26685, 26686, 26687, -1, 26688, 26689, 26690, -1, 26691, 26692, 26693, -1, 26694, 26695, 26696, -1, 26697, 26698, 26699, -1, 26700, 26701, 26702, -1, 26703, 26704, 26705, -1, 26706, 26707, 26708, -1, 26709, 26710, 26711, -1, 26712, 26713, 26714, -1, 26715, 26716, 26717, -1, 26718, 26719, 26720, -1, 26721, 26722, 26723, -1, 26724, 26725, 26726, -1, 26727, 26728, 26729, -1, 26730, 26731, 26732, -1, 26733, 26734, 26735, -1, 26736, 26737, 26738, -1, 26739, 26740, 26741, -1, 26742, 26743, 26744, -1, 26745, 26746, 26747, -1, 26748, 26749, 26750, -1, 26751, 26752, 26753, -1, 26754, 26755, 26756, -1, 26757, 26758, 26759, -1, 26760, 26761, 26762, -1, 26763, 26764, 26765, -1, 26766, 26767, 26768, -1, 26769, 26770, 26771, -1, 26772, 26773, 26774, -1, 26775, 26776, 26777, -1, 26778, 26779, 26780, -1, 26781, 26782, 26783, -1, 26784, 26785, 26786, -1, 26787, 26788, 26789, -1, 26790, 26791, 26792, -1, 26793, 26794, 26795, -1, 26796, 26797, 26798, -1, 26799, 26800, 26801, -1, 26802, 26803, 26804, -1, 26805, 26806, 26807, -1, 26808, 26809, 26810, -1, 26811, 26812, 26813, -1, 26814, 26815, 26816, -1, 26817, 26818, 26819, -1, 26820, 26821, 26822, -1, 26823, 26824, 26825, -1, 26826, 26827, 26828, -1, 26829, 26830, 26831, -1, 26832, 26833, 26834, -1, 26835, 26836, 26837, -1, 26838, 26839, 26840, -1, 26841, 26842, 26843, -1, 26844, 26845, 26846, -1, 26847, 26848, 26849, -1, 26850, 26851, 26852, -1, 26853, 26854, 26855, -1, 26856, 26857, 26858, -1, 26859, 26860, 26861, -1, 26862, 26863, 26864, -1, 26865, 26866, 26867, -1, 26868, 26869, 26870, -1, 26871, 26872, 26873, -1, 26874, 26875, 26876, -1, 26877, 26878, 26879, -1, 26880, 26881, 26882, -1, 26883, 26884, 26885, -1, 26886, 26887, 26888, -1, 26889, 26890, 26891, -1, 26892, 26893, 26894, -1, 26895, 26896, 26897, -1, 26898, 26899, 26900, -1, 26901, 26902, 26903, -1, 26904, 26905, 26906, -1, 26907, 26908, 26909, -1, 26910, 26911, 26912, -1, 26913, 26914, 26915, -1, 26916, 26917, 26918, -1, 26919, 26920, 26921, -1, 26922, 26923, 26924, -1, 26925, 26926, 26927, -1, 26928, 26929, 26930, -1, 26931, 26932, 26933, -1, 26934, 26935, 26936, -1, 26937, 26938, 26939, -1, 26940, 26941, 26942, -1, 26943, 26944, 26945, -1, 26946, 26947, 26948, -1, 26949, 26950, 26951, -1, 26952, 26953, 26954, -1, 26955, 26956, 26957, -1, 26958, 26959, 26960, -1, 26961, 26962, 26963, -1, 26964, 26965, 26966, -1, 26967, 26968, 26969, -1, 26970, 26971, 26972, -1, 26973, 26974, 26975, -1, 26976, 26977, 26978, -1, 26979, 26980, 26981, -1, 26982, 26983, 26984, -1, 26985, 26986, 26987, -1, 26988, 26989, 26990, -1, 26991, 26992, 26993, -1, 26994, 26995, 26996, -1, 26997, 26998, 26999, -1, 27000, 27001, 27002, -1, 27003, 27004, 27005, -1, 27006, 27007, 27008, -1, 27009, 27010, 27011, -1, 27012, 27013, 27014, -1, 27015, 27016, 27017, -1, 27018, 27019, 27020, -1, 27021, 27022, 27023, -1, 27024, 27025, 27026, -1, 27027, 27028, 27029, -1, 27030, 27031, 27032, -1, 27033, 27034, 27035, -1, 27036, 27037, 27038, -1, 27039, 27040, 27041, -1, 27042, 27043, 27044, -1, 27045, 27046, 27047, -1, 27048, 27049, 27050, -1, 27051, 27052, 27053, -1, 27054, 27055, 27056, -1, 27057, 27058, 27059, -1, 27060, 27061, 27062, -1, 27063, 27064, 27065, -1, 27066, 27067, 27068, -1, 27069, 27070, 27071, -1, 27072, 27073, 27074, -1, 27075, 27076, 27077, -1, 27078, 27079, 27080, -1, 27081, 27082, 27083, -1, 27084, 27085, 27086, -1, 27087, 27088, 27089, -1, 27090, 27091, 27092, -1, 27093, 27094, 27095, -1, 27096, 27097, 27098, -1, 27099, 27100, 27101, -1, 27102, 27103, 27104, -1, 27105, 27106, 27107, -1, 27108, 27109, 27110, -1, 27111, 27112, 27113, -1, 27114, 27115, 27116, -1, 27117, 27118, 27119, -1, 27120, 27121, 27122, -1, 27123, 27124, 27125, -1, 27126, 27127, 27128, -1, 27129, 27130, 27131, -1, 27132, 27133, 27134, -1, 27135, 27136, 27137, -1, 27138, 27139, 27140, -1, 27141, 27142, 27143, -1, 27144, 27145, 27146, -1, 27147, 27148, 27149, -1, 27150, 27151, 27152, -1, 27153, 27154, 27155, -1, 27156, 27157, 27158, -1, 27159, 27160, 27161, -1, 27162, 27163, 27164, -1, 27165, 27166, 27167, -1, 27168, 27169, 27170, -1, 27171, 27172, 27173, -1, 27174, 27175, 27176, -1, 27177, 27178, 27179, -1, 27180, 27181, 27182, -1, 27183, 27184, 27185, -1, 27186, 27187, 27188, -1, 27189, 27190, 27191, -1, 27192, 27193, 27194, -1, 27195, 27196, 27197, -1, 27198, 27199, 27200, -1, 27201, 27202, 27203, -1, 27204, 27205, 27206, -1, 27207, 27208, 27209, -1, 27210, 27211, 27212, -1, 27213, 27214, 27215, -1, 27216, 27217, 27218, -1, 27219, 27220, 27221, -1, 27222, 27223, 27224, -1, 27225, 27226, 27227, -1, 27228, 27229, 27230, -1, 27231, 27232, 27233, -1, 27234, 27235, 27236, -1, 27237, 27238, 27239, -1, 27240, 27241, 27242, -1, 27243, 27244, 27245, -1, 27246, 27247, 27248, -1, 27249, 27250, 27251, -1, 27252, 27253, 27254, -1, 27255, 27256, 27257, -1, 27258, 27259, 27260, -1, 27261, 27262, 27263, -1, 27264, 27265, 27266, -1, 27267, 27268, 27269, -1, 27270, 27271, 27272, -1, 27273, 27274, 27275, -1, 27276, 27277, 27278, -1, 27279, 27280, 27281, -1, 27282, 27283, 27284, -1, 27285, 27286, 27287, -1, 27288, 27289, 27290, -1, 27291, 27292, 27293, -1, 27294, 27295, 27296, -1, 27297, 27298, 27299, -1, 27300, 27301, 27302, -1, 27303, 27304, 27305, -1, 27306, 27307, 27308, -1, 27309, 27310, 27311, -1, 27312, 27313, 27314, -1, 27315, 27316, 27317, -1, 27318, 27319, 27320, -1, 27321, 27322, 27323, -1, 27324, 27325, 27326, -1, 27327, 27328, 27329, -1, 27330, 27331, 27332, -1, 27333, 27334, 27335, -1, 27336, 27337, 27338, -1, 27339, 27340, 27341, -1, 27342, 27343, 27344, -1, 27345, 27346, 27347, -1, 27348, 27349, 27350, -1, 27351, 27352, 27353, -1, 27354, 27355, 27356, -1, 27357, 27358, 27359, -1, 27360, 27361, 27362, -1, 27363, 27364, 27365, -1, 27366, 27367, 27368, -1, 27369, 27370, 27371, -1, 27372, 27373, 27374, -1, 27375, 27376, 27377, -1, 27378, 27379, 27380, -1, 27381, 27382, 27383, -1, 27384, 27385, 27386, -1, 27387, 27388, 27389, -1, 27390, 27391, 27392, -1, 27393, 27394, 27395, -1, 27396, 27397, 27398, -1, 27399, 27400, 27401, -1, 27402, 27403, 27404, -1, 27405, 27406, 27407, -1, 27408, 27409, 27410, -1, 27411, 27412, 27413, -1, 27414, 27415, 27416, -1, 27417, 27418, 27419, -1, 27420, 27421, 27422, -1, 27423, 27424, 27425, -1, 27426, 27427, 27428, -1, 27429, 27430, 27431, -1, 27432, 27433, 27434, -1, 27435, 27436, 27437, -1, 27438, 27439, 27440, -1, 27441, 27442, 27443, -1, 27444, 27445, 27446, -1, 27447, 27448, 27449, -1, 27450, 27451, 27452, -1, 27453, 27454, 27455, -1, 27456, 27457, 27458, -1, 27459, 27460, 27461, -1, 27462, 27463, 27464, -1, 27465, 27466, 27467, -1, 27468, 27469, 27470, -1, 27471, 27472, 27473, -1, 27474, 27475, 27476, -1, 27477, 27478, 27479, -1, 27480, 27481, 27482, -1, 27483, 27484, 27485, -1, 27486, 27487, 27488, -1, 27489, 27490, 27491, -1, 27492, 27493, 27494, -1, 27495, 27496, 27497, -1, 27498, 27499, 27500, -1, 27501, 27502, 27503, -1, 27504, 27505, 27506, -1, 27507, 27508, 27509, -1, 27510, 27511, 27512, -1, 27513, 27514, 27515, -1, 27516, 27517, 27518, -1, 27519, 27520, 27521, -1, 27522, 27523, 27524, -1, 27525, 27526, 27527, -1, 27528, 27529, 27530, -1, 27531, 27532, 27533, -1, 27534, 27535, 27536, -1, 27537, 27538, 27539, -1, 27540, 27541, 27542, -1, 27543, 27544, 27545, -1, 27546, 27547, 27548, -1, 27549, 27550, 27551, -1, 27552, 27553, 27554, -1, 27555, 27556, 27557, -1, 27558, 27559, 27560, -1, 27561, 27562, 27563, -1, 27564, 27565, 27566, -1, 27567, 27568, 27569, -1, 27570, 27571, 27572, -1, 27573, 27574, 27575, -1, 27576, 27577, 27578, -1, 27579, 27580, 27581, -1, 27582, 27583, 27584, -1, 27585, 27586, 27587, -1, 27588, 27589, 27590, -1, 27591, 27592, 27593, -1, 27594, 27595, 27596, -1, 27597, 27598, 27599, -1, 27600, 27601, 27602, -1, 27603, 27604, 27605, -1, 27606, 27607, 27608, -1, 27609, 27610, 27611, -1, 27612, 27613, 27614, -1, 27615, 27616, 27617, -1, 27618, 27619, 27620, -1, 27621, 27622, 27623, -1, 27624, 27625, 27626, -1, 27627, 27628, 27629, -1, 27630, 27631, 27632, -1, 27633, 27634, 27635, -1, 27636, 27637, 27638, -1, 27639, 27640, 27641, -1, 27642, 27643, 27644, -1, 27645, 27646, 27647, -1, 27648, 27649, 27650, -1, 27651, 27652, 27653, -1, 27654, 27655, 27656, -1, 27657, 27658, 27659, -1, 27660, 27661, 27662, -1, 27663, 27664, 27665, -1, 27666, 27667, 27668, -1, 27669, 27670, 27671, -1, 27672, 27673, 27674, -1, 27675, 27676, 27677, -1, 27678, 27679, 27680, -1, 27681, 27682, 27683, -1, 27684, 27685, 27686, -1, 27687, 27688, 27689, -1, 27690, 27691, 27692, -1, 27693, 27694, 27695, -1, 27696, 27697, 27698, -1, 27699, 27700, 27701, -1, 27702, 27703, 27704, -1, 27705, 27706, 27707, -1, 27708, 27709, 27710, -1, 27711, 27712, 27713, -1, 27714, 27715, 27716, -1, 27717, 27718, 27719, -1, 27720, 27721, 27722, -1, 27723, 27724, 27725, -1, 27726, 27727, 27728, -1, 27729, 27730, 27731, -1, 27732, 27733, 27734, -1, 27735, 27736, 27737, -1, 27738, 27739, 27740, -1, 27741, 27742, 27743, -1, 27744, 27745, 27746, -1, 27747, 27748, 27749, -1, 27750, 27751, 27752, -1, 27753, 27754, 27755, -1, 27756, 27757, 27758, -1, 27759, 27760, 27761, -1, 27762, 27763, 27764, -1, 27765, 27766, 27767, -1, 27768, 27769, 27770, -1, 27771, 27772, 27773, -1, 27774, 27775, 27776, -1, 27777, 27778, 27779, -1, 27780, 27781, 27782, -1, 27783, 27784, 27785, -1, 27786, 27787, 27788, -1, 27789, 27790, 27791, -1, 27792, 27793, 27794, -1, 27795, 27796, 27797, -1, 27798, 27799, 27800, -1, 27801, 27802, 27803, -1, 27804, 27805, 27806, -1, 27807, 27808, 27809, -1, 27810, 27811, 27812, -1, 27813, 27814, 27815, -1, 27816, 27817, 27818, -1, 27819, 27820, 27821, -1, 27822, 27823, 27824, -1, 27825, 27826, 27827, -1, 27828, 27829, 27830, -1, 27831, 27832, 27833, -1, 27834, 27835, 27836, -1, 27837, 27838, 27839, -1, 27840, 27841, 27842, -1, 27843, 27844, 27845, -1, 27846, 27847, 27848, -1, 27849, 27850, 27851, -1, 27852, 27853, 27854, -1, 27855, 27856, 27857, -1, 27858, 27859, 27860, -1, 27861, 27862, 27863, -1, 27864, 27865, 27866, -1, 27867, 27868, 27869, -1, 27870, 27871, 27872, -1, 27873, 27874, 27875, -1, 27876, 27877, 27878, -1, 27879, 27880, 27881, -1, 27882, 27883, 27884, -1, 27885, 27886, 27887, -1, 27888, 27889, 27890, -1, 27891, 27892, 27893, -1, 27894, 27895, 27896, -1, 27897, 27898, 27899, -1, 27900, 27901, 27902, -1, 27903, 27904, 27905, -1, 27906, 27907, 27908, -1, 27909, 27910, 27911, -1, 27912, 27913, 27914, -1, 27915, 27916, 27917, -1, 27918, 27919, 27920, -1, 27921, 27922, 27923, -1, 27924, 27925, 27926, -1, 27927, 27928, 27929, -1, 27930, 27931, 27932, -1, 27933, 27934, 27935, -1, 27936, 27937, 27938, -1, 27939, 27940, 27941, -1, 27942, 27943, 27944, -1, 27945, 27946, 27947, -1, 27948, 27949, 27950, -1, 27951, 27952, 27953, -1, 27954, 27955, 27956, -1, 27957, 27958, 27959, -1, 27960, 27961, 27962, -1, 27963, 27964, 27965, -1, 27966, 27967, 27968, -1, 27969, 27970, 27971, -1, 27972, 27973, 27974, -1, 27975, 27976, 27977, -1, 27978, 27979, 27980, -1, 27981, 27982, 27983, -1, 27984, 27985, 27986, -1, 27987, 27988, 27989, -1, 27990, 27991, 27992, -1, 27993, 27994, 27995, -1, 27996, 27997, 27998, -1, 27999, 28000, 28001, -1, 28002, 28003, 28004, -1, 28005, 28006, 28007, -1, 28008, 28009, 28010, -1, 28011, 28012, 28013, -1, 28014, 28015, 28016, -1, 28017, 28018, 28019, -1, 28020, 28021, 28022, -1, 28023, 28024, 28025, -1, 28026, 28027, 28028, -1, 28029, 28030, 28031, -1, 28032, 28033, 28034, -1, 28035, 28036, 28037, -1, 28038, 28039, 28040, -1, 28041, 28042, 28043, -1, 28044, 28045, 28046, -1, 28047, 28048, 28049, -1, 28050, 28051, 28052, -1, 28053, 28054, 28055, -1, 28056, 28057, 28058, -1, 28059, 28060, 28061, -1, 28062, 28063, 28064, -1, 28065, 28066, 28067, -1, 28068, 28069, 28070, -1, 28071, 28072, 28073, -1, 28074, 28075, 28076, -1, 28077, 28078, 28079, -1, 28080, 28081, 28082, -1, 28083, 28084, 28085, -1, 28086, 28087, 28088, -1, 28089, 28090, 28091, -1, 28092, 28093, 28094, -1, 28095, 28096, 28097, -1, 28098, 28099, 28100, -1, 28101, 28102, 28103, -1, 28104, 28105, 28106, -1, 28107, 28108, 28109, -1, 28110, 28111, 28112, -1, 28113, 28114, 28115, -1, 28116, 28117, 28118, -1, 28119, 28120, 28121, -1, 28122, 28123, 28124, -1, 28125, 28126, 28127, -1, 28128, 28129, 28130, -1, 28131, 28132, 28133, -1, 28134, 28135, 28136, -1, 28137, 28138, 28139, -1, 28140, 28141, 28142, -1, 28143, 28144, 28145, -1, 28146, 28147, 28148, -1, 28149, 28150, 28151, -1, 28152, 28153, 28154, -1, 28155, 28156, 28157, -1, 28158, 28159, 28160, -1, 28161, 28162, 28163, -1, 28164, 28165, 28166, -1, 28167, 28168, 28169, -1, 28170, 28171, 28172, -1, 28173, 28174, 28175, -1, 28176, 28177, 28178, -1, 28179, 28180, 28181, -1, 28182, 28183, 28184, -1, 28185, 28186, 28187, -1, 28188, 28189, 28190, -1, 28191, 28192, 28193, -1, 28194, 28195, 28196, -1, 28197, 28198, 28199, -1, 28200, 28201, 28202, -1, 28203, 28204, 28205, -1, 28206, 28207, 28208, -1, 28209, 28210, 28211, -1, 28212, 28213, 28214, -1, 28215, 28216, 28217, -1, 28218, 28219, 28220, -1, 28221, 28222, 28223, -1, 28224, 28225, 28226, -1, 28227, 28228, 28229, -1, 28230, 28231, 28232, -1, 28233, 28234, 28235, -1, 28236, 28237, 28238, -1, 28239, 28240, 28241, -1, 28242, 28243, 28244, -1, 28245, 28246, 28247, -1, 28248, 28249, 28250, -1, 28251, 28252, 28253, -1, 28254, 28255, 28256, -1, 28257, 28258, 28259, -1, 28260, 28261, 28262, -1, 28263, 28264, 28265, -1, 28266, 28267, 28268, -1, 28269, 28270, 28271, -1, 28272, 28273, 28274, -1, 28275, 28276, 28277, -1, 28278, 28279, 28280, -1, 28281, 28282, 28283, -1, 28284, 28285, 28286, -1, 28287, 28288, 28289, -1, 28290, 28291, 28292, -1, 28293, 28294, 28295, -1, 28296, 28297, 28298, -1, 28299, 28300, 28301, -1, 28302, 28303, 28304, -1, 28305, 28306, 28307, -1, 28308, 28309, 28310, -1, 28311, 28312, 28313, -1, 28314, 28315, 28316, -1, 28317, 28318, 28319, -1, 28320, 28321, 28322, -1, 28323, 28324, 28325, -1, 28326, 28327, 28328, -1, 28329, 28330, 28331, -1, 28332, 28333, 28334, -1, 28335, 28336, 28337, -1, 28338, 28339, 28340, -1, 28341, 28342, 28343, -1, 28344, 28345, 28346, -1, 28347, 28348, 28349, -1, 28350, 28351, 28352, -1, 28353, 28354, 28355, -1, 28356, 28357, 28358, -1, 28359, 28360, 28361, -1, 28362, 28363, 28364, -1, 28365, 28366, 28367, -1, 28368, 28369, 28370, -1, 28371, 28372, 28373, -1, 28374, 28375, 28376, -1, 28377, 28378, 28379, -1, 28380, 28381, 28382, -1, 28383, 28384, 28385, -1, 28386, 28387, 28388, -1, 28389, 28390, 28391, -1, 28392, 28393, 28394, -1, 28395, 28396, 28397, -1, 28398, 28399, 28400, -1, 28401, 28402, 28403, -1, 28404, 28405, 28406, -1, 28407, 28408, 28409, -1, 28410, 28411, 28412, -1, 28413, 28414, 28415, -1, 28416, 28417, 28418, -1, 28419, 28420, 28421, -1, 28422, 28423, 28424, -1, 28425, 28426, 28427, -1, 28428, 28429, 28430, -1, 28431, 28432, 28433, -1, 28434, 28435, 28436, -1, 28437, 28438, 28439, -1, 28440, 28441, 28442, -1, 28443, 28444, 28445, -1, 28446, 28447, 28448, -1, 28449, 28450, 28451, -1, 28452, 28453, 28454, -1, 28455, 28456, 28457, -1, 28458, 28459, 28460, -1, 28461, 28462, 28463, -1, 28464, 28465, 28466, -1, 28467, 28468, 28469, -1, 28470, 28471, 28472, -1, 28473, 28474, 28475, -1, 28476, 28477, 28478, -1, 28479, 28480, 28481, -1, 28482, 28483, 28484, -1, 28485, 28486, 28487, -1, 28488, 28489, 28490, -1, 28491, 28492, 28493, -1, 28494, 28495, 28496, -1, 28497, 28498, 28499, -1, 28500, 28501, 28502, -1, 28503, 28504, 28505, -1, 28506, 28507, 28508, -1, 28509, 28510, 28511, -1, 28512, 28513, 28514, -1, 28515, 28516, 28517, -1, 28518, 28519, 28520, -1, 28521, 28522, 28523, -1, 28524, 28525, 28526, -1, 28527, 28528, 28529, -1, 28530, 28531, 28532, -1, 28533, 28534, 28535, -1, 28536, 28537, 28538, -1, 28539, 28540, 28541, -1, 28542, 28543, 28544, -1, 28545, 28546, 28547, -1, 28548, 28549, 28550, -1, 28551, 28552, 28553, -1, 28554, 28555, 28556, -1, 28557, 28558, 28559, -1, 28560, 28561, 28562, -1, 28563, 28564, 28565, -1, 28566, 28567, 28568, -1, 28569, 28570, 28571, -1, 28572, 28573, 28574, -1, 28575, 28576, 28577, -1, 28578, 28579, 28580, -1, 28581, 28582, 28583, -1, 28584, 28585, 28586, -1, 28587, 28588, 28589, -1, 28590, 28591, 28592, -1, 28593, 28594, 28595, -1, 28596, 28597, 28598, -1, 28599, 28600, 28601, -1, 28602, 28603, 28604, -1, 28605, 28606, 28607, -1, 28608, 28609, 28610, -1, 28611, 28612, 28613, -1, 28614, 28615, 28616, -1, 28617, 28618, 28619, -1, 28620, 28621, 28622, -1, 28623, 28624, 28625, -1, 28626, 28627, 28628, -1, 28629, 28630, 28631, -1, 28632, 28633, 28634, -1, 28635, 28636, 28637, -1, 28638, 28639, 28640, -1, 28641, 28642, 28643, -1, 28644, 28645, 28646, -1, 28647, 28648, 28649, -1, 28650, 28651, 28652, -1, 28653, 28654, 28655, -1, 28656, 28657, 28658, -1, 28659, 28660, 28661, -1, 28662, 28663, 28664, -1, 28665, 28666, 28667, -1, 28668, 28669, 28670, -1, 28671, 28672, 28673, -1, 28674, 28675, 28676, -1, 28677, 28678, 28679, -1, 28680, 28681, 28682, -1, 28683, 28684, 28685, -1, 28686, 28687, 28688, -1, 28689, 28690, 28691, -1, 28692, 28693, 28694, -1, 28695, 28696, 28697, -1, 28698, 28699, 28700, -1, 28701, 28702, 28703, -1, 28704, 28705, 28706, -1, 28707, 28708, 28709, -1, 28710, 28711, 28712, -1, 28713, 28714, 28715, -1, 28716, 28717, 28718, -1, 28719, 28720, 28721, -1, 28722, 28723, 28724, -1, 28725, 28726, 28727, -1, 28728, 28729, 28730, -1, 28731, 28732, 28733, -1, 28734, 28735, 28736, -1, 28737, 28738, 28739, -1, 28740, 28741, 28742, -1, 28743, 28744, 28745, -1, 28746, 28747, 28748, -1, 28749, 28750, 28751, -1, 28752, 28753, 28754, -1, 28755, 28756, 28757, -1, 28758, 28759, 28760, -1, 28761, 28762, 28763, -1, 28764, 28765, 28766, -1, 28767, 28768, 28769, -1, 28770, 28771, 28772, -1, 28773, 28774, 28775, -1, 28776, 28777, 28778, -1, 28779, 28780, 28781, -1, 28782, 28783, 28784, -1, 28785, 28786, 28787, -1, 28788, 28789, 28790, -1, 28791, 28792, 28793, -1, 28794, 28795, 28796, -1, 28797, 28798, 28799, -1, 28800, 28801, 28802, -1, 28803, 28804, 28805, -1, 28806, 28807, 28808, -1, 28809, 28810, 28811, -1, 28812, 28813, 28814, -1, 28815, 28816, 28817, -1, 28818, 28819, 28820, -1, 28821, 28822, 28823, -1, 28824, 28825, 28826, -1, 28827, 28828, 28829, -1, 28830, 28831, 28832, -1, 28833, 28834, 28835, -1, 28836, 28837, 28838, -1, 28839, 28840, 28841, -1, 28842, 28843, 28844, -1, 28845, 28846, 28847, -1, 28848, 28849, 28850, -1, 28851, 28852, 28853, -1, 28854, 28855, 28856, -1, 28857, 28858, 28859, -1, 28860, 28861, 28862, -1, 28863, 28864, 28865, -1, 28866, 28867, 28868, -1, 28869, 28870, 28871, -1, 28872, 28873, 28874, -1, 28875, 28876, 28877, -1, 28878, 28879, 28880, -1, 28881, 28882, 28883, -1, 28884, 28885, 28886, -1, 28887, 28888, 28889, -1, 28890, 28891, 28892, -1, 28893, 28894, 28895, -1, 28896, 28897, 28898, -1, 28899, 28900, 28901, -1, 28902, 28903, 28904, -1, 28905, 28906, 28907, -1, 28908, 28909, 28910, -1, 28911, 28912, 28913, -1, 28914, 28915, 28916, -1, 28917, 28918, 28919, -1, 28920, 28921, 28922, -1, 28923, 28924, 28925, -1, 28926, 28927, 28928, -1, 28929, 28930, 28931, -1, 28932, 28933, 28934, -1, 28935, 28936, 28937, -1, 28938, 28939, 28940, -1, 28941, 28942, 28943, -1, 28944, 28945, 28946, -1, 28947, 28948, 28949, -1, 28950, 28951, 28952, -1, 28953, 28954, 28955, -1, 28956, 28957, 28958, -1, 28959, 28960, 28961, -1, 28962, 28963, 28964, -1, 28965, 28966, 28967, -1, 28968, 28969, 28970, -1, 28971, 28972, 28973, -1, 28974, 28975, 28976, -1, 28977, 28978, 28979, -1, 28980, 28981, 28982, -1, 28983, 28984, 28985, -1, 28986, 28987, 28988, -1, 28989, 28990, 28991, -1, 28992, 28993, 28994, -1, 28995, 28996, 28997, -1, 28998, 28999, 29000, -1, 29001, 29002, 29003, -1, 29004, 29005, 29006, -1, 29007, 29008, 29009, -1, 29010, 29011, 29012, -1, 29013, 29014, 29015, -1, 29016, 29017, 29018, -1, 29019, 29020, 29021, -1, 29022, 29023, 29024, -1, 29025, 29026, 29027, -1, 29028, 29029, 29030, -1, 29031, 29032, 29033, -1, 29034, 29035, 29036, -1, 29037, 29038, 29039, -1, 29040, 29041, 29042, -1, 29043, 29044, 29045, -1, 29046, 29047, 29048, -1, 29049, 29050, 29051, -1, 29052, 29053, 29054, -1, 29055, 29056, 29057, -1, 29058, 29059, 29060, -1, 29061, 29062, 29063, -1, 29064, 29065, 29066, -1, 29067, 29068, 29069, -1, 29070, 29071, 29072, -1, 29073, 29074, 29075, -1, 29076, 29077, 29078, -1, 29079, 29080, 29081, -1, 29082, 29083, 29084, -1, 29085, 29086, 29087, -1, 29088, 29089, 29090, -1, 29091, 29092, 29093, -1, 29094, 29095, 29096, -1, 29097, 29098, 29099, -1, 29100, 29101, 29102, -1, 29103, 29104, 29105, -1, 29106, 29107, 29108, -1, 29109, 29110, 29111, -1, 29112, 29113, 29114, -1, 29115, 29116, 29117, -1, 29118, 29119, 29120, -1, 29121, 29122, 29123, -1, 29124, 29125, 29126, -1, 29127, 29128, 29129, -1, 29130, 29131, 29132, -1, 29133, 29134, 29135, -1, 29136, 29137, 29138, -1, 29139, 29140, 29141, -1, 29142, 29143, 29144, -1, 29145, 29146, 29147, -1, 29148, 29149, 29150, -1, 29151, 29152, 29153, -1, 29154, 29155, 29156, -1, 29157, 29158, 29159, -1, 29160, 29161, 29162, -1, 29163, 29164, 29165, -1, 29166, 29167, 29168, -1, 29169, 29170, 29171, -1, 29172, 29173, 29174, -1, 29175, 29176, 29177, -1, 29178, 29179, 29180, -1, 29181, 29182, 29183, -1, 29184, 29185, 29186, -1, 29187, 29188, 29189, -1, 29190, 29191, 29192, -1, 29193, 29194, 29195, -1, 29196, 29197, 29198, -1, 29199, 29200, 29201, -1, 29202, 29203, 29204, -1, 29205, 29206, 29207, -1, 29208, 29209, 29210, -1, 29211, 29212, 29213, -1, 29214, 29215, 29216, -1, 29217, 29218, 29219, -1, 29220, 29221, 29222, -1, 29223, 29224, 29225, -1, 29226, 29227, 29228, -1, 29229, 29230, 29231, -1, 29232, 29233, 29234, -1, 29235, 29236, 29237, -1, 29238, 29239, 29240, -1, 29241, 29242, 29243, -1, 29244, 29245, 29246, -1, 29247, 29248, 29249, -1, 29250, 29251, 29252, -1, 29253, 29254, 29255, -1, 29256, 29257, 29258, -1, 29259, 29260, 29261, -1, 29262, 29263, 29264, -1, 29265, 29266, 29267, -1, 29268, 29269, 29270, -1, 29271, 29272, 29273, -1, 29274, 29275, 29276, -1, 29277, 29278, 29279, -1, 29280, 29281, 29282, -1, 29283, 29284, 29285, -1, 29286, 29287, 29288, -1, 29289, 29290, 29291, -1, 29292, 29293, 29294, -1, 29295, 29296, 29297, -1, 29298, 29299, 29300, -1, 29301, 29302, 29303, -1, 29304, 29305, 29306, -1, 29307, 29308, 29309, -1, 29310, 29311, 29312, -1, 29313, 29314, 29315, -1, 29316, 29317, 29318, -1, 29319, 29320, 29321, -1, 29322, 29323, 29324, -1, 29325, 29326, 29327, -1, 29328, 29329, 29330, -1, 29331, 29332, 29333, -1, 29334, 29335, 29336, -1, 29337, 29338, 29339, -1, 29340, 29341, 29342, -1, 29343, 29344, 29345, -1, 29346, 29347, 29348, -1, 29349, 29350, 29351, -1, 29352, 29353, 29354, -1, 29355, 29356, 29357, -1, 29358, 29359, 29360, -1, 29361, 29362, 29363, -1, 29364, 29365, 29366, -1, 29367, 29368, 29369, -1, 29370, 29371, 29372, -1, 29373, 29374, 29375, -1, 29376, 29377, 29378, -1, 29379, 29380, 29381, -1, 29382, 29383, 29384, -1, 29385, 29386, 29387, -1, 29388, 29389, 29390, -1, 29391, 29392, 29393, -1, 29394, 29395, 29396, -1, 29397, 29398, 29399, -1, 29400, 29401, 29402, -1, 29403, 29404, 29405, -1, 29406, 29407, 29408, -1, 29409, 29410, 29411, -1, 29412, 29413, 29414, -1, 29415, 29416, 29417, -1, 29418, 29419, 29420, -1, 29421, 29422, 29423, -1, 29424, 29425, 29426, -1, 29427, 29428, 29429, -1, 29430, 29431, 29432, -1, 29433, 29434, 29435, -1, 29436, 29437, 29438, -1, 29439, 29440, 29441, -1, 29442, 29443, 29444, -1, 29445, 29446, 29447, -1, 29448, 29449, 29450, -1, 29451, 29452, 29453, -1, 29454, 29455, 29456, -1, 29457, 29458, 29459, -1, 29460, 29461, 29462, -1, 29463, 29464, 29465, -1, 29466, 29467, 29468, -1, 29469, 29470, 29471, -1, 29472, 29473, 29474, -1, 29475, 29476, 29477, -1, 29478, 29479, 29480, -1, 29481, 29482, 29483, -1, 29484, 29485, 29486, -1, 29487, 29488, 29489, -1, 29490, 29491, 29492, -1, 29493, 29494, 29495, -1, 29496, 29497, 29498, -1, 29499, 29500, 29501, -1, 29502, 29503, 29504, -1, 29505, 29506, 29507, -1, 29508, 29509, 29510, -1, 29511, 29512, 29513, -1, 29514, 29515, 29516, -1, 29517, 29518, 29519, -1, 29520, 29521, 29522, -1, 29523, 29524, 29525, -1, 29526, 29527, 29528, -1, 29529, 29530, 29531, -1, 29532, 29533, 29534, -1, 29535, 29536, 29537, -1, 29538, 29539, 29540, -1, 29541, 29542, 29543, -1, 29544, 29545, 29546, -1, 29547, 29548, 29549, -1, 29550, 29551, 29552, -1, 29553, 29554, 29555, -1, 29556, 29557, 29558, -1, 29559, 29560, 29561, -1, 29562, 29563, 29564, -1, 29565, 29566, 29567, -1, 29568, 29569, 29570, -1, 29571, 29572, 29573, -1, 29574, 29575, 29576, -1, 29577, 29578, 29579, -1, 29580, 29581, 29582, -1, 29583, 29584, 29585, -1, 29586, 29587, 29588, -1, 29589, 29590, 29591, -1, 29592, 29593, 29594, -1, 29595, 29596, 29597, -1, 29598, 29599, 29600, -1, 29601, 29602, 29603, -1, 29604, 29605, 29606, -1, 29607, 29608, 29609, -1, 29610, 29611, 29612, -1, 29613, 29614, 29615, -1, 29616, 29617, 29618, -1, 29619, 29620, 29621, -1, 29622, 29623, 29624, -1, 29625, 29626, 29627, -1, 29628, 29629, 29630, -1, 29631, 29632, 29633, -1, 29634, 29635, 29636, -1, 29637, 29638, 29639, -1, 29640, 29641, 29642, -1, 29643, 29644, 29645, -1, 29646, 29647, 29648, -1, 29649, 29650, 29651, -1, 29652, 29653, 29654, -1, 29655, 29656, 29657, -1, 29658, 29659, 29660, -1, 29661, 29662, 29663, -1, 29664, 29665, 29666, -1, 29667, 29668, 29669, -1, 29670, 29671, 29672, -1, 29673, 29674, 29675, -1, 29676, 29677, 29678, -1, 29679, 29680, 29681, -1, 29682, 29683, 29684, -1, 29685, 29686, 29687, -1, 29688, 29689, 29690, -1, 29691, 29692, 29693, -1, 29694, 29695, 29696, -1, 29697, 29698, 29699, -1, 29700, 29701, 29702, -1, 29703, 29704, 29705, -1, 29706, 29707, 29708, -1, 29709, 29710, 29711, -1, 29712, 29713, 29714, -1, 29715, 29716, 29717, -1, 29718, 29719, 29720, -1, 29721, 29722, 29723, -1, 29724, 29725, 29726, -1, 29727, 29728, 29729, -1, 29730, 29731, 29732, -1, 29733, 29734, 29735, -1, 29736, 29737, 29738, -1, 29739, 29740, 29741, -1, 29742, 29743, 29744, -1, 29745, 29746, 29747, -1, 29748, 29749, 29750, -1, 29751, 29752, 29753, -1, 29754, 29755, 29756, -1, 29757, 29758, 29759, -1, 29760, 29761, 29762, -1, 29763, 29764, 29765, -1, 29766, 29767, 29768, -1, 29769, 29770, 29771, -1, 29772, 29773, 29774, -1, 29775, 29776, 29777, -1, 29778, 29779, 29780, -1, 29781, 29782, 29783, -1, 29784, 29785, 29786, -1, 29787, 29788, 29789, -1, 29790, 29791, 29792, -1, 29793, 29794, 29795, -1, 29796, 29797, 29798, -1, 29799, 29800, 29801, -1, 29802, 29803, 29804, -1, 29805, 29806, 29807, -1, 29808, 29809, 29810, -1, 29811, 29812, 29813, -1, 29814, 29815, 29816, -1, 29817, 29818, 29819, -1, 29820, 29821, 29822, -1, 29823, 29824, 29825, -1, 29826, 29827, 29828, -1, 29829, 29830, 29831, -1, 29832, 29833, 29834, -1, 29835, 29836, 29837, -1, 29838, 29839, 29840, -1, 29841, 29842, 29843, -1, 29844, 29845, 29846, -1, 29847, 29848, 29849, -1, 29850, 29851, 29852, -1, 29853, 29854, 29855, -1, 29856, 29857, 29858, -1, 29859, 29860, 29861, -1, 29862, 29863, 29864, -1, 29865, 29866, 29867, -1, 29868, 29869, 29870, -1, 29871, 29872, 29873, -1, 29874, 29875, 29876, -1, 29877, 29878, 29879, -1, 29880, 29881, 29882, -1, 29883, 29884, 29885, -1, 29886, 29887, 29888, -1, 29889, 29890, 29891, -1, 29892, 29893, 29894, -1, 29895, 29896, 29897, -1, 29898, 29899, 29900, -1, 29901, 29902, 29903, -1, 29904, 29905, 29906, -1, 29907, 29908, 29909, -1, 29910, 29911, 29912, -1, 29913, 29914, 29915, -1, 29916, 29917, 29918, -1, 29919, 29920, 29921, -1, 29922, 29923, 29924, -1, 29925, 29926, 29927, -1, 29928, 29929, 29930, -1, 29931, 29932, 29933, -1, 29934, 29935, 29936, -1, 29937, 29938, 29939, -1, 29940, 29941, 29942, -1, 29943, 29944, 29945, -1, 29946, 29947, 29948, -1, 29949, 29950, 29951, -1, 29952, 29953, 29954, -1, 29955, 29956, 29957, -1, 29958, 29959, 29960, -1, 29961, 29962, 29963, -1, 29964, 29965, 29966, -1, 29967, 29968, 29969, -1, 29970, 29971, 29972, -1, 29973, 29974, 29975, -1, 29976, 29977, 29978, -1, 29979, 29980, 29981, -1, 29982, 29983, 29984, -1, 29985, 29986, 29987, -1, 29988, 29989, 29990, -1, 29991, 29992, 29993, -1, 29994, 29995, 29996, -1, 29997, 29998, 29999, -1, 30000, 30001, 30002, -1, 30003, 30004, 30005, -1, 30006, 30007, 30008, -1, 30009, 30010, 30011, -1, 30012, 30013, 30014, -1, 30015, 30016, 30017, -1, 30018, 30019, 30020, -1, 30021, 30022, 30023, -1, 30024, 30025, 30026, -1, 30027, 30028, 30029, -1, 30030, 30031, 30032, -1, 30033, 30034, 30035, -1, 30036, 30037, 30038, -1, 30039, 30040, 30041, -1, 30042, 30043, 30044, -1, 30045, 30046, 30047, -1, 30048, 30049, 30050, -1, 30051, 30052, 30053, -1, 30054, 30055, 30056, -1, 30057, 30058, 30059, -1, 30060, 30061, 30062, -1, 30063, 30064, 30065, -1, 30066, 30067, 30068, -1, 30069, 30070, 30071, -1, 30072, 30073, 30074, -1, 30075, 30076, 30077, -1, 30078, 30079, 30080, -1, 30081, 30082, 30083, -1, 30084, 30085, 30086, -1, 30087, 30088, 30089, -1, 30090, 30091, 30092, -1, 30093, 30094, 30095, -1, 30096, 30097, 30098, -1, 30099, 30100, 30101, -1, 30102, 30103, 30104, -1, 30105, 30106, 30107, -1, 30108, 30109, 30110, -1, 30111, 30112, 30113, -1, 30114, 30115, 30116, -1, 30117, 30118, 30119, -1, 30120, 30121, 30122, -1, 30123, 30124, 30125, -1, 30126, 30127, 30128, -1, 30129, 30130, 30131, -1, 30132, 30133, 30134, -1, 30135, 30136, 30137, -1, 30138, 30139, 30140, -1, 30141, 30142, 30143, -1, 30144, 30145, 30146, -1, 30147, 30148, 30149, -1, 30150, 30151, 30152, -1, 30153, 30154, 30155, -1, 30156, 30157, 30158, -1, 30159, 30160, 30161, -1, 30162, 30163, 30164, -1, 30165, 30166, 30167, -1, 30168, 30169, 30170, -1, 30171, 30172, 30173, -1, 30174, 30175, 30176, -1, 30177, 30178, 30179, -1, 30180, 30181, 30182, -1, 30183, 30184, 30185, -1, 30186, 30187, 30188, -1, 30189, 30190, 30191, -1, 30192, 30193, 30194, -1, 30195, 30196, 30197, -1, 30198, 30199, 30200, -1, 30201, 30202, 30203, -1, 30204, 30205, 30206, -1, 30207, 30208, 30209, -1, 30210, 30211, 30212, -1, 30213, 30214, 30215, -1, 30216, 30217, 30218, -1, 30219, 30220, 30221, -1, 30222, 30223, 30224, -1, 30225, 30226, 30227, -1, 30228, 30229, 30230, -1, 30231, 30232, 30233, -1, 30234, 30235, 30236, -1, 30237, 30238, 30239, -1, 30240, 30241, 30242, -1, 30243, 30244, 30245, -1, 30246, 30247, 30248, -1, 30249, 30250, 30251, -1, 30252, 30253, 30254, -1, 30255, 30256, 30257, -1, 30258, 30259, 30260, -1, 30261, 30262, 30263, -1, 30264, 30265, 30266, -1, 30267, 30268, 30269, -1, 30270, 30271, 30272, -1, 30273, 30274, 30275, -1, 30276, 30277, 30278, -1, 30279, 30280, 30281, -1, 30282, 30283, 30284, -1, 30285, 30286, 30287, -1, 30288, 30289, 30290, -1, 30291, 30292, 30293, -1, 30294, 30295, 30296, -1, 30297, 30298, 30299, -1, 30300, 30301, 30302, -1, 30303, 30304, 30305, -1, 30306, 30307, 30308, -1, 30309, 30310, 30311, -1, 30312, 30313, 30314, -1, 30315, 30316, 30317, -1, 30318, 30319, 30320, -1, 30321, 30322, 30323, -1, 30324, 30325, 30326, -1, 30327, 30328, 30329, -1, 30330, 30331, 30332, -1, 30333, 30334, 30335, -1, 30336, 30337, 30338, -1, 30339, 30340, 30341, -1, 30342, 30343, 30344, -1, 30345, 30346, 30347, -1, 30348, 30349, 30350, -1, 30351, 30352, 30353, -1, 30354, 30355, 30356, -1, 30357, 30358, 30359, -1, 30360, 30361, 30362, -1, 30363, 30364, 30365, -1, 30366, 30367, 30368, -1, 30369, 30370, 30371, -1, 30372, 30373, 30374, -1, 30375, 30376, 30377, -1, 30378, 30379, 30380, -1, 30381, 30382, 30383, -1, 30384, 30385, 30386, -1, 30387, 30388, 30389, -1, 30390, 30391, 30392, -1, 30393, 30394, 30395, -1, 30396, 30397, 30398, -1, 30399, 30400, 30401, -1, 30402, 30403, 30404, -1, 30405, 30406, 30407, -1, 30408, 30409, 30410, -1, 30411, 30412, 30413, -1, 30414, 30415, 30416, -1, 30417, 30418, 30419, -1, 30420, 30421, 30422, -1, 30423, 30424, 30425, -1, 30426, 30427, 30428, -1, 30429, 30430, 30431, -1, 30432, 30433, 30434, -1, 30435, 30436, 30437, -1, 30438, 30439, 30440, -1, 30441, 30442, 30443, -1, 30444, 30445, 30446, -1, 30447, 30448, 30449, -1, 30450, 30451, 30452, -1, 30453, 30454, 30455, -1, 30456, 30457, 30458, -1, 30459, 30460, 30461, -1, 30462, 30463, 30464, -1, 30465, 30466, 30467, -1, 30468, 30469, 30470, -1, 30471, 30472, 30473, -1, 30474, 30475, 30476, -1, 30477, 30478, 30479, -1, 30480, 30481, 30482, -1, 30483, 30484, 30485, -1, 30486, 30487, 30488, -1, 30489, 30490, 30491, -1, 30492, 30493, 30494, -1, 30495, 30496, 30497, -1, 30498, 30499, 30500, -1, 30501, 30502, 30503, -1, 30504, 30505, 30506, -1, 30507, 30508, 30509, -1, 30510, 30511, 30512, -1, 30513, 30514, 30515, -1, 30516, 30517, 30518, -1, 30519, 30520, 30521, -1, 30522, 30523, 30524, -1, 30525, 30526, 30527, -1, 30528, 30529, 30530, -1, 30531, 30532, 30533, -1, 30534, 30535, 30536, -1, 30537, 30538, 30539, -1, 30540, 30541, 30542, -1, 30543, 30544, 30545, -1, 30546, 30547, 30548, -1, 30549, 30550, 30551, -1, 30552, 30553, 30554, -1, 30555, 30556, 30557, -1, 30558, 30559, 30560, -1, 30561, 30562, 30563, -1, 30564, 30565, 30566, -1, 30567, 30568, 30569, -1, 30570, 30571, 30572, -1, 30573, 30574, 30575, -1, 30576, 30577, 30578, -1, 30579, 30580, 30581, -1, 30582, 30583, 30584, -1, 30585, 30586, 30587, -1, 30588, 30589, 30590, -1, 30591, 30592, 30593, -1, 30594, 30595, 30596, -1, 30597, 30598, 30599, -1, 30600, 30601, 30602, -1, 30603, 30604, 30605, -1, 30606, 30607, 30608, -1, 30609, 30610, 30611, -1, 30612, 30613, 30614, -1, 30615, 30616, 30617, -1, 30618, 30619, 30620, -1, 30621, 30622, 30623, -1, 30624, 30625, 30626, -1, 30627, 30628, 30629, -1, 30630, 30631, 30632, -1, 30633, 30634, 30635, -1, 30636, 30637, 30638, -1, 30639, 30640, 30641, -1, 30642, 30643, 30644, -1, 30645, 30646, 30647, -1, 30648, 30649, 30650, -1, 30651, 30652, 30653, -1, 30654, 30655, 30656, -1, 30657, 30658, 30659, -1, 30660, 30661, 30662, -1, 30663, 30664, 30665, -1, 30666, 30667, 30668, -1, 30669, 30670, 30671, -1, 30672, 30673, 30674, -1, 30675, 30676, 30677, -1, 30678, 30679, 30680, -1, 30681, 30682, 30683, -1, 30684, 30685, 30686, -1, 30687, 30688, 30689, -1, 30690, 30691, 30692, -1, 30693, 30694, 30695, -1, 30696, 30697, 30698, -1, 30699, 30700, 30701, -1, 30702, 30703, 30704, -1, 30705, 30706, 30707, -1, 30708, 30709, 30710, -1, 30711, 30712, 30713, -1, 30714, 30715, 30716, -1, 30717, 30718, 30719, -1, 30720, 30721, 30722, -1, 30723, 30724, 30725, -1, 30726, 30727, 30728, -1, 30729, 30730, 30731, -1, 30732, 30733, 30734, -1, 30735, 30736, 30737, -1, 30738, 30739, 30740, -1, 30741, 30742, 30743, -1, 30744, 30745, 30746, -1, 30747, 30748, 30749, -1, 30750, 30751, 30752, -1, 30753, 30754, 30755, -1, 30756, 30757, 30758, -1, 30759, 30760, 30761, -1, 30762, 30763, 30764, -1, 30765, 30766, 30767, -1, 30768, 30769, 30770, -1, 30771, 30772, 30773, -1, 30774, 30775, 30776, -1, 30777, 30778, 30779, -1, 30780, 30781, 30782, -1, 30783, 30784, 30785, -1, 30786, 30787, 30788, -1, 30789, 30790, 30791, -1, 30792, 30793, 30794, -1, 30795, 30796, 30797, -1, 30798, 30799, 30800, -1, 30801, 30802, 30803, -1, 30804, 30805, 30806, -1, 30807, 30808, 30809, -1, 30810, 30811, 30812, -1, 30813, 30814, 30815, -1, 30816, 30817, 30818, -1, 30819, 30820, 30821, -1, 30822, 30823, 30824, -1, 30825, 30826, 30827, -1, 30828, 30829, 30830, -1, 30831, 30832, 30833, -1, 30834, 30835, 30836, -1, 30837, 30838, 30839, -1, 30840, 30841, 30842, -1, 30843, 30844, 30845, -1, 30846, 30847, 30848, -1, 30849, 30850, 30851, -1, 30852, 30853, 30854, -1, 30855, 30856, 30857, -1, 30858, 30859, 30860, -1, 30861, 30862, 30863, -1, 30864, 30865, 30866, -1, 30867, 30868, 30869, -1, 30870, 30871, 30872, -1, 30873, 30874, 30875, -1, 30876, 30877, 30878, -1, 30879, 30880, 30881, -1, 30882, 30883, 30884, -1, 30885, 30886, 30887, -1, 30888, 30889, 30890, -1, 30891, 30892, 30893, -1, 30894, 30895, 30896, -1, 30897, 30898, 30899, -1, 30900, 30901, 30902, -1, 30903, 30904, 30905, -1, 30906, 30907, 30908, -1, 30909, 30910, 30911, -1, 30912, 30913, 30914, -1, 30915, 30916, 30917, -1, 30918, 30919, 30920, -1, 30921, 30922, 30923, -1, 30924, 30925, 30926, -1, 30927, 30928, 30929, -1, 30930, 30931, 30932, -1, 30933, 30934, 30935, -1, 30936, 30937, 30938, -1, 30939, 30940, 30941, -1, 30942, 30943, 30944, -1, 30945, 30946, 30947, -1, 30948, 30949, 30950, -1, 30951, 30952, 30953, -1, 30954, 30955, 30956, -1, 30957, 30958, 30959, -1, 30960, 30961, 30962, -1, 30963, 30964, 30965, -1, 30966, 30967, 30968, -1, 30969, 30970, 30971, -1, 30972, 30973, 30974, -1, 30975, 30976, 30977, -1, 30978, 30979, 30980, -1, 30981, 30982, 30983, -1, 30984, 30985, 30986, -1, 30987, 30988, 30989, -1, 30990, 30991, 30992, -1, 30993, 30994, 30995, -1, 30996, 30997, 30998, -1, 30999, 31000, 31001, -1, 31002, 31003, 31004, -1, 31005, 31006, 31007, -1, 31008, 31009, 31010, -1, 31011, 31012, 31013, -1, 31014, 31015, 31016, -1, 31017, 31018, 31019, -1, 31020, 31021, 31022, -1, 31023, 31024, 31025, -1, 31026, 31027, 31028, -1, 31029, 31030, 31031, -1, 31032, 31033, 31034, -1, 31035, 31036, 31037, -1, 31038, 31039, 31040, -1, 31041, 31042, 31043, -1, 31044, 31045, 31046, -1, 31047, 31048, 31049, -1, 31050, 31051, 31052, -1, 31053, 31054, 31055, -1, 31056, 31057, 31058, -1, 31059, 31060, 31061, -1, 31062, 31063, 31064, -1, 31065, 31066, 31067, -1, 31068, 31069, 31070, -1, 31071, 31072, 31073, -1, 31074, 31075, 31076, -1, 31077, 31078, 31079, -1, 31080, 31081, 31082, -1, 31083, 31084, 31085, -1, 31086, 31087, 31088, -1, 31089, 31090, 31091, -1, 31092, 31093, 31094, -1, 31095, 31096, 31097, -1, 31098, 31099, 31100, -1, 31101, 31102, 31103, -1, 31104, 31105, 31106, -1, 31107, 31108, 31109, -1, 31110, 31111, 31112, -1, 31113, 31114, 31115, -1, 31116, 31117, 31118, -1, 31119, 31120, 31121, -1, 31122, 31123, 31124, -1, 31125, 31126, 31127, -1, 31128, 31129, 31130, -1, 31131, 31132, 31133, -1, 31134, 31135, 31136, -1, 31137, 31138, 31139, -1, 31140, 31141, 31142, -1, 31143, 31144, 31145, -1, 31146, 31147, 31148, -1, 31149, 31150, 31151, -1, 31152, 31153, 31154, -1, 31155, 31156, 31157, -1, 31158, 31159, 31160, -1, 31161, 31162, 31163, -1, 31164, 31165, 31166, -1, 31167, 31168, 31169, -1, 31170, 31171, 31172, -1, 31173, 31174, 31175, -1, 31176, 31177, 31178, -1, 31179, 31180, 31181, -1, 31182, 31183, 31184, -1, 31185, 31186, 31187, -1, 31188, 31189, 31190, -1, 31191, 31192, 31193, -1, 31194, 31195, 31196, -1, 31197, 31198, 31199, -1, 31200, 31201, 31202, -1, 31203, 31204, 31205, -1, 31206, 31207, 31208, -1, 31209, 31210, 31211, -1, 31212, 31213, 31214, -1, 31215, 31216, 31217, -1, 31218, 31219, 31220, -1, 31221, 31222, 31223, -1, 31224, 31225, 31226, -1, 31227, 31228, 31229, -1, 31230, 31231, 31232, -1, 31233, 31234, 31235, -1, 31236, 31237, 31238, -1, 31239, 31240, 31241, -1, 31242, 31243, 31244, -1, 31245, 31246, 31247, -1, 31248, 31249, 31250, -1, 31251, 31252, 31253, -1, 31254, 31255, 31256, -1, 31257, 31258, 31259, -1, 31260, 31261, 31262, -1, 31263, 31264, 31265, -1, 31266, 31267, 31268, -1, 31269, 31270, 31271, -1, 31272, 31273, 31274, -1, 31275, 31276, 31277, -1, 31278, 31279, 31280, -1, 31281, 31282, 31283, -1, 31284, 31285, 31286, -1, 31287, 31288, 31289, -1, 31290, 31291, 31292, -1, 31293, 31294, 31295, -1, 31296, 31297, 31298, -1, 31299, 31300, 31301, -1, 31302, 31303, 31304, -1, 31305, 31306, 31307, -1, 31308, 31309, 31310, -1, 31311, 31312, 31313, -1, 31314, 31315, 31316, -1, 31317, 31318, 31319, -1, 31320, 31321, 31322, -1, 31323, 31324, 31325, -1, 31326, 31327, 31328, -1, 31329, 31330, 31331, -1, 31332, 31333, 31334, -1, 31335, 31336, 31337, -1, 31338, 31339, 31340, -1, 31341, 31342, 31343, -1, 31344, 31345, 31346, -1, 31347, 31348, 31349, -1, 31350, 31351, 31352, -1, 31353, 31354, 31355, -1, 31356, 31357, 31358, -1, 31359, 31360, 31361, -1, 31362, 31363, 31364, -1, 31365, 31366, 31367, -1, 31368, 31369, 31370, -1, 31371, 31372, 31373, -1, 31374, 31375, 31376, -1, 31377, 31378, 31379, -1, 31380, 31381, 31382, -1, 31383, 31384, 31385, -1, 31386, 31387, 31388, -1, 31389, 31390, 31391, -1, 31392, 31393, 31394, -1, 31395, 31396, 31397, -1, 31398, 31399, 31400, -1, 31401, 31402, 31403, -1, 31404, 31405, 31406, -1, 31407, 31408, 31409, -1, 31410, 31411, 31412, -1, 31413, 31414, 31415, -1, 31416, 31417, 31418, -1, 31419, 31420, 31421, -1, 31422, 31423, 31424, -1, 31425, 31426, 31427, -1, 31428, 31429, 31430, -1, 31431, 31432, 31433, -1, 31434, 31435, 31436, -1, 31437, 31438, 31439, -1, 31440, 31441, 31442, -1, 31443, 31444, 31445, -1, 31446, 31447, 31448, -1, 31449, 31450, 31451, -1, 31452, 31453, 31454, -1, 31455, 31456, 31457, -1, 31458, 31459, 31460, -1, 31461, 31462, 31463, -1, 31464, 31465, 31466, -1, 31467, 31468, 31469, -1, 31470, 31471, 31472, -1, 31473, 31474, 31475, -1, 31476, 31477, 31478, -1, 31479, 31480, 31481, -1, 31482, 31483, 31484, -1, 31485, 31486, 31487, -1, 31488, 31489, 31490, -1, 31491, 31492, 31493, -1, 31494, 31495, 31496, -1, 31497, 31498, 31499, -1, 31500, 31501, 31502, -1, 31503, 31504, 31505, -1, 31506, 31507, 31508, -1, 31509, 31510, 31511, -1, 31512, 31513, 31514, -1, 31515, 31516, 31517, -1, 31518, 31519, 31520, -1, 31521, 31522, 31523, -1, 31524, 31525, 31526, -1, 31527, 31528, 31529, -1, 31530, 31531, 31532, -1, 31533, 31534, 31535, -1, 31536, 31537, 31538, -1, 31539, 31540, 31541, -1, 31542, 31543, 31544, -1, 31545, 31546, 31547, -1, 31548, 31549, 31550, -1, 31551, 31552, 31553, -1, 31554, 31555, 31556, -1, 31557, 31558, 31559, -1, 31560, 31561, 31562, -1, 31563, 31564, 31565, -1, 31566, 31567, 31568, -1, 31569, 31570, 31571, -1, 31572, 31573, 31574, -1, 31575, 31576, 31577, -1, 31578, 31579, 31580, -1, 31581, 31582, 31583, -1, 31584, 31585, 31586, -1, 31587, 31588, 31589, -1, 31590, 31591, 31592, -1, 31593, 31594, 31595, -1, 31596, 31597, 31598, -1, 31599, 31600, 31601, -1, 31602, 31603, 31604, -1, 31605, 31606, 31607, -1, 31608, 31609, 31610, -1, 31611, 31612, 31613, -1, 31614, 31615, 31616, -1, 31617, 31618, 31619, -1, 31620, 31621, 31622, -1, 31623, 31624, 31625, -1, 31626, 31627, 31628, -1, 31629, 31630, 31631, -1, 31632, 31633, 31634, -1, 31635, 31636, 31637, -1, 31638, 31639, 31640, -1, 31641, 31642, 31643, -1, 31644, 31645, 31646, -1, 31647, 31648, 31649, -1, 31650, 31651, 31652, -1, 31653, 31654, 31655, -1, 31656, 31657, 31658, -1, 31659, 31660, 31661, -1, 31662, 31663, 31664, -1, 31665, 31666, 31667, -1, 31668, 31669, 31670, -1, 31671, 31672, 31673, -1, 31674, 31675, 31676, -1, 31677, 31678, 31679, -1, 31680, 31681, 31682, -1, 31683, 31684, 31685, -1, 31686, 31687, 31688, -1, 31689, 31690, 31691, -1, 31692, 31693, 31694, -1, 31695, 31696, 31697, -1, 31698, 31699, 31700, -1, 31701, 31702, 31703, -1, 31704, 31705, 31706, -1, 31707, 31708, 31709, -1, 31710, 31711, 31712, -1, 31713, 31714, 31715, -1, 31716, 31717, 31718, -1, 31719, 31720, 31721, -1, 31722, 31723, 31724, -1, 31725, 31726, 31727, -1, 31728, 31729, 31730, -1, 31731, 31732, 31733, -1, 31734, 31735, 31736, -1, 31737, 31738, 31739, -1, 31740, 31741, 31742, -1, 31743, 31744, 31745, -1, 31746, 31747, 31748, -1, 31749, 31750, 31751, -1, 31752, 31753, 31754, -1, 31755, 31756, 31757, -1, 31758, 31759, 31760, -1, 31761, 31762, 31763, -1, 31764, 31765, 31766, -1, 31767, 31768, 31769, -1, 31770, 31771, 31772, -1, 31773, 31774, 31775, -1, 31776, 31777, 31778, -1, 31779, 31780, 31781, -1, 31782, 31783, 31784, -1, 31785, 31786, 31787, -1, 31788, 31789, 31790, -1, 31791, 31792, 31793, -1, 31794, 31795, 31796, -1, 31797, 31798, 31799, -1, 31800, 31801, 31802, -1, 31803, 31804, 31805, -1, 31806, 31807, 31808, -1, 31809, 31810, 31811, -1, 31812, 31813, 31814, -1, 31815, 31816, 31817, -1, 31818, 31819, 31820, -1, 31821, 31822, 31823, -1, 31824, 31825, 31826, -1, 31827, 31828, 31829, -1, 31830, 31831, 31832, -1, 31833, 31834, 31835, -1, 31836, 31837, 31838, -1, 31839, 31840, 31841, -1, 31842, 31843, 31844, -1, 31845, 31846, 31847, -1, 31848, 31849, 31850, -1, 31851, 31852, 31853, -1, 31854, 31855, 31856, -1, 31857, 31858, 31859, -1, 31860, 31861, 31862, -1, 31863, 31864, 31865, -1, 31866, 31867, 31868, -1, 31869, 31870, 31871, -1, 31872, 31873, 31874, -1, 31875, 31876, 31877, -1, 31878, 31879, 31880, -1, 31881, 31882, 31883, -1, 31884, 31885, 31886, -1, 31887, 31888, 31889, -1, 31890, 31891, 31892, -1, 31893, 31894, 31895, -1, 31896, 31897, 31898, -1, 31899, 31900, 31901, -1, 31902, 31903, 31904, -1, 31905, 31906, 31907, -1, 31908, 31909, 31910, -1, 31911, 31912, 31913, -1, 31914, 31915, 31916, -1, 31917, 31918, 31919, -1, 31920, 31921, 31922, -1, 31923, 31924, 31925, -1, 31926, 31927, 31928, -1, 31929, 31930, 31931, -1, 31932, 31933, 31934, -1, 31935, 31936, 31937, -1, 31938, 31939, 31940, -1, 31941, 31942, 31943, -1, 31944, 31945, 31946, -1, 31947, 31948, 31949, -1, 31950, 31951, 31952, -1, 31953, 31954, 31955, -1, 31956, 31957, 31958, -1, 31959, 31960, 31961, -1, 31962, 31963, 31964, -1, 31965, 31966, 31967, -1, 31968, 31969, 31970, -1, 31971, 31972, 31973, -1, 31974, 31975, 31976, -1, 31977, 31978, 31979, -1, 31980, 31981, 31982, -1, 31983, 31984, 31985, -1, 31986, 31987, 31988, -1, 31989, 31990, 31991, -1, 31992, 31993, 31994, -1, 31995, 31996, 31997, -1, 31998, 31999, 32000, -1, 32001, 32002, 32003, -1, 32004, 32005, 32006, -1, 32007, 32008, 32009, -1, 32010, 32011, 32012, -1, 32013, 32014, 32015, -1, 32016, 32017, 32018, -1, 32019, 32020, 32021, -1, 32022, 32023, 32024, -1, 32025, 32026, 32027, -1, 32028, 32029, 32030, -1, 32031, 32032, 32033, -1, 32034, 32035, 32036, -1, 32037, 32038, 32039, -1, 32040, 32041, 32042, -1, 32043, 32044, 32045, -1, 32046, 32047, 32048, -1, 32049, 32050, 32051, -1, 32052, 32053, 32054, -1, 32055, 32056, 32057, -1, 32058, 32059, 32060, -1, 32061, 32062, 32063, -1, 32064, 32065, 32066, -1, 32067, 32068, 32069, -1, 32070, 32071, 32072, -1, 32073, 32074, 32075, -1, 32076, 32077, 32078, -1, 32079, 32080, 32081, -1, 32082, 32083, 32084, -1, 32085, 32086, 32087, -1, 32088, 32089, 32090, -1, 32091, 32092, 32093, -1, 32094, 32095, 32096, -1, 32097, 32098, 32099, -1, 32100, 32101, 32102, -1, 32103, 32104, 32105, -1, 32106, 32107, 32108, -1, 32109, 32110, 32111, -1, 32112, 32113, 32114, -1, 32115, 32116, 32117, -1, 32118, 32119, 32120, -1, 32121, 32122, 32123, -1, 32124, 32125, 32126, -1, 32127, 32128, 32129, -1, 32130, 32131, 32132, -1, 32133, 32134, 32135, -1, 32136, 32137, 32138, -1, 32139, 32140, 32141, -1, 32142, 32143, 32144, -1, 32145, 32146, 32147, -1, 32148, 32149, 32150, -1, 32151, 32152, 32153, -1, 32154, 32155, 32156, -1, 32157, 32158, 32159, -1, 32160, 32161, 32162, -1, 32163, 32164, 32165, -1, 32166, 32167, 32168, -1, 32169, 32170, 32171, -1, 32172, 32173, 32174, -1, 32175, 32176, 32177, -1, 32178, 32179, 32180, -1, 32181, 32182, 32183, -1, 32184, 32185, 32186, -1, 32187, 32188, 32189, -1, 32190, 32191, 32192, -1, 32193, 32194, 32195, -1, 32196, 32197, 32198, -1, 32199, 32200, 32201, -1, 32202, 32203, 32204, -1, 32205, 32206, 32207, -1, 32208, 32209, 32210, -1, 32211, 32212, 32213, -1, 32214, 32215, 32216, -1, 32217, 32218, 32219, -1, 32220, 32221, 32222, -1, 32223, 32224, 32225, -1, 32226, 32227, 32228, -1, 32229, 32230, 32231, -1, 32232, 32233, 32234, -1, 32235, 32236, 32237, -1, 32238, 32239, 32240, -1, 32241, 32242, 32243, -1, 32244, 32245, 32246, -1, 32247, 32248, 32249, -1, 32250, 32251, 32252, -1, 32253, 32254, 32255, -1, 32256, 32257, 32258, -1, 32259, 32260, 32261, -1, 32262, 32263, 32264, -1, 32265, 32266, 32267, -1, 32268, 32269, 32270, -1, 32271, 32272, 32273, -1, 32274, 32275, 32276, -1, 32277, 32278, 32279, -1, 32280, 32281, 32282, -1, 32283, 32284, 32285, -1, 32286, 32287, 32288, -1, 32289, 32290, 32291, -1, 32292, 32293, 32294, -1, 32295, 32296, 32297, -1, 32298, 32299, 32300, -1, 32301, 32302, 32303, -1, 32304, 32305, 32306, -1, 32307, 32308, 32309, -1, 32310, 32311, 32312, -1, 32313, 32314, 32315, -1, 32316, 32317, 32318, -1, 32319, 32320, 32321, -1, 32322, 32323, 32324, -1, 32325, 32326, 32327, -1, 32328, 32329, 32330, -1, 32331, 32332, 32333, -1, 32334, 32335, 32336, -1, 32337, 32338, 32339, -1, 32340, 32341, 32342, -1, 32343, 32344, 32345, -1, 32346, 32347, 32348, -1, 32349, 32350, 32351, -1, 32352, 32353, 32354, -1, 32355, 32356, 32357, -1, 32358, 32359, 32360, -1, 32361, 32362, 32363, -1, 32364, 32365, 32366, -1, 32367, 32368, 32369, -1, 32370, 32371, 32372, -1, 32373, 32374, 32375, -1, 32376, 32377, 32378, -1, 32379, 32380, 32381, -1, 32382, 32383, 32384, -1, 32385, 32386, 32387, -1, 32388, 32389, 32390, -1, 32391, 32392, 32393, -1, 32394, 32395, 32396, -1, 32397, 32398, 32399, -1, 32400, 32401, 32402, -1, 32403, 32404, 32405, -1, 32406, 32407, 32408, -1, 32409, 32410, 32411, -1, 32412, 32413, 32414, -1, 32415, 32416, 32417, -1, 32418, 32419, 32420, -1, 32421, 32422, 32423, -1, 32424, 32425, 32426, -1, 32427, 32428, 32429, -1, 32430, 32431, 32432, -1, 32433, 32434, 32435, -1, 32436, 32437, 32438, -1, 32439, 32440, 32441, -1, 32442, 32443, 32444, -1, 32445, 32446, 32447, -1, 32448, 32449, 32450, -1, 32451, 32452, 32453, -1, 32454, 32455, 32456, -1, 32457, 32458, 32459, -1, 32460, 32461, 32462, -1, 32463, 32464, 32465, -1, 32466, 32467, 32468, -1, 32469, 32470, 32471, -1, 32472, 32473, 32474, -1, 32475, 32476, 32477, -1, 32478, 32479, 32480, -1, 32481, 32482, 32483, -1, 32484, 32485, 32486, -1, 32487, 32488, 32489, -1, 32490, 32491, 32492, -1, 32493, 32494, 32495, -1, 32496, 32497, 32498, -1, 32499, 32500, 32501, -1, 32502, 32503, 32504, -1, 32505, 32506, 32507, -1, 32508, 32509, 32510, -1, 32511, 32512, 32513, -1, 32514, 32515, 32516, -1, 32517, 32518, 32519, -1, 32520, 32521, 32522, -1, 32523, 32524, 32525, -1, 32526, 32527, 32528, -1, 32529, 32530, 32531, -1, 32532, 32533, 32534, -1, 32535, 32536, 32537, -1, 32538, 32539, 32540, -1, 32541, 32542, 32543, -1, 32544, 32545, 32546, -1, 32547, 32548, 32549, -1, 32550, 32551, 32552, -1, 32553, 32554, 32555, -1, 32556, 32557, 32558, -1, 32559, 32560, 32561, -1, 32562, 32563, 32564, -1, 32565, 32566, 32567, -1, 32568, 32569, 32570, -1, 32571, 32572, 32573, -1, 32574, 32575, 32576, -1, 32577, 32578, 32579, -1, 32580, 32581, 32582, -1, 32583, 32584, 32585, -1, 32586, 32587, 32588, -1, 32589, 32590, 32591, -1, 32592, 32593, 32594, -1, 32595, 32596, 32597, -1, 32598, 32599, 32600, -1, 32601, 32602, 32603, -1, 32604, 32605, 32606, -1, 32607, 32608, 32609, -1, 32610, 32611, 32612, -1, 32613, 32614, 32615, -1, 32616, 32617, 32618, -1, 32619, 32620, 32621, -1, 32622, 32623, 32624, -1, 32625, 32626, 32627, -1, 32628, 32629, 32630, -1, 32631, 32632, 32633, -1, 32634, 32635, 32636, -1, 32637, 32638, 32639, -1, 32640, 32641, 32642, -1, 32643, 32644, 32645, -1, 32646, 32647, 32648, -1, 32649, 32650, 32651, -1, 32652, 32653, 32654, -1, 32655, 32656, 32657, -1, 32658, 32659, 32660, -1, 32661, 32662, 32663, -1, 32664, 32665, 32666, -1, 32667, 32668, 32669, -1, 32670, 32671, 32672, -1, 32673, 32674, 32675, -1, 32676, 32677, 32678, -1, 32679, 32680, 32681, -1, 32682, 32683, 32684, -1, 32685, 32686, 32687, -1, 32688, 32689, 32690, -1, 32691, 32692, 32693, -1, 32694, 32695, 32696, -1, 32697, 32698, 32699, -1, 32700, 32701, 32702, -1, 32703, 32704, 32705, -1, 32706, 32707, 32708, -1, 32709, 32710, 32711, -1, 32712, 32713, 32714, -1, 32715, 32716, 32717, -1, 32718, 32719, 32720, -1, 32721, 32722, 32723, -1, 32724, 32725, 32726, -1, 32727, 32728, 32729, -1, 32730, 32731, 32732, -1, 32733, 32734, 32735, -1, 32736, 32737, 32738, -1, 32739, 32740, 32741, -1, 32742, 32743, 32744, -1, 32745, 32746, 32747, -1, 32748, 32749, 32750, -1, 32751, 32752, 32753, -1, 32754, 32755, 32756, -1, 32757, 32758, 32759, -1, 32760, 32761, 32762, -1, 32763, 32764, 32765, -1, 32766, 32767, 32768, -1, 32769, 32770, 32771, -1, 32772, 32773, 32774, -1, 32775, 32776, 32777, -1, 32778, 32779, 32780, -1, 32781, 32782, 32783, -1, 32784, 32785, 32786, -1, 32787, 32788, 32789, -1, 32790, 32791, 32792, -1, 32793, 32794, 32795, -1, 32796, 32797, 32798, -1, 32799, 32800, 32801, -1, 32802, 32803, 32804, -1, 32805, 32806, 32807, -1, 32808, 32809, 32810, -1, 32811, 32812, 32813, -1, 32814, 32815, 32816, -1, 32817, 32818, 32819, -1, 32820, 32821, 32822, -1, 32823, 32824, 32825, -1, 32826, 32827, 32828, -1, 32829, 32830, 32831, -1, 32832, 32833, 32834, -1, 32835, 32836, 32837, -1, 32838, 32839, 32840, -1, 32841, 32842, 32843, -1, 32844, 32845, 32846, -1, 32847, 32848, 32849, -1, 32850, 32851, 32852, -1, 32853, 32854, 32855, -1, 32856, 32857, 32858, -1, 32859, 32860, 32861, -1, 32862, 32863, 32864, -1, 32865, 32866, 32867, -1, 32868, 32869, 32870, -1, 32871, 32872, 32873, -1, 32874, 32875, 32876, -1, 32877, 32878, 32879, -1, 32880, 32881, 32882, -1, 32883, 32884, 32885, -1, 32886, 32887, 32888, -1, 32889, 32890, 32891, -1, 32892, 32893, 32894, -1, 32895, 32896, 32897, -1, 32898, 32899, 32900, -1, 32901, 32902, 32903, -1, 32904, 32905, 32906, -1, 32907, 32908, 32909, -1, 32910, 32911, 32912, -1, 32913, 32914, 32915, -1, 32916, 32917, 32918, -1, 32919, 32920, 32921, -1, 32922, 32923, 32924, -1, 32925, 32926, 32927, -1, 32928, 32929, 32930, -1, 32931, 32932, 32933, -1, 32934, 32935, 32936, -1, 32937, 32938, 32939, -1, 32940, 32941, 32942, -1, 32943, 32944, 32945, -1, 32946, 32947, 32948, -1, 32949, 32950, 32951, -1, 32952, 32953, 32954, -1, 32955, 32956, 32957, -1, 32958, 32959, 32960, -1, 32961, 32962, 32963, -1, 32964, 32965, 32966, -1, 32967, 32968, 32969, -1, 32970, 32971, 32972, -1, 32973, 32974, 32975, -1, 32976, 32977, 32978, -1, 32979, 32980, 32981, -1, 32982, 32983, 32984, -1, 32985, 32986, 32987, -1, 32988, 32989, 32990, -1, 32991, 32992, 32993, -1, 32994, 32995, 32996, -1, 32997, 32998, 32999, -1, 33000, 33001, 33002, -1, 33003, 33004, 33005, -1, 33006, 33007, 33008, -1, 33009, 33010, 33011, -1, 33012, 33013, 33014, -1, 33015, 33016, 33017, -1, 33018, 33019, 33020, -1, 33021, 33022, 33023, -1, 33024, 33025, 33026, -1, 33027, 33028, 33029, -1, 33030, 33031, 33032, -1, 33033, 33034, 33035, -1, 33036, 33037, 33038, -1, 33039, 33040, 33041, -1, 33042, 33043, 33044, -1, 33045, 33046, 33047, -1, 33048, 33049, 33050, -1, 33051, 33052, 33053, -1, 33054, 33055, 33056, -1, 33057, 33058, 33059, -1, 33060, 33061, 33062, -1, 33063, 33064, 33065, -1, 33066, 33067, 33068, -1, 33069, 33070, 33071, -1, 33072, 33073, 33074, -1, 33075, 33076, 33077, -1, 33078, 33079, 33080, -1, 33081, 33082, 33083, -1, 33084, 33085, 33086, -1, 33087, 33088, 33089, -1, 33090, 33091, 33092, -1, 33093, 33094, 33095, -1, 33096, 33097, 33098, -1, 33099, 33100, 33101, -1, 33102, 33103, 33104, -1, 33105, 33106, 33107, -1, 33108, 33109, 33110, -1, 33111, 33112, 33113, -1, 33114, 33115, 33116, -1, 33117, 33118, 33119, -1, 33120, 33121, 33122, -1, 33123, 33124, 33125, -1, 33126, 33127, 33128, -1, 33129, 33130, 33131, -1, 33132, 33133, 33134, -1, 33135, 33136, 33137, -1, 33138, 33139, 33140, -1, 33141, 33142, 33143, -1, 33144, 33145, 33146, -1, 33147, 33148, 33149, -1, 33150, 33151, 33152, -1, 33153, 33154, 33155, -1, 33156, 33157, 33158, -1, 33159, 33160, 33161, -1, 33162, 33163, 33164, -1, 33165, 33166, 33167, -1, 33168, 33169, 33170, -1, 33171, 33172, 33173, -1, 33174, 33175, 33176, -1, 33177, 33178, 33179, -1, 33180, 33181, 33182, -1, 33183, 33184, 33185, -1, 33186, 33187, 33188, -1, 33189, 33190, 33191, -1, 33192, 33193, 33194, -1, 33195, 33196, 33197, -1, 33198, 33199, 33200, -1, 33201, 33202, 33203, -1, 33204, 33205, 33206, -1, 33207, 33208, 33209, -1, 33210, 33211, 33212, -1, 33213, 33214, 33215, -1, 33216, 33217, 33218, -1, 33219, 33220, 33221, -1, 33222, 33223, 33224, -1, 33225, 33226, 33227, -1, 33228, 33229, 33230, -1, 33231, 33232, 33233, -1, 33234, 33235, 33236, -1, 33237, 33238, 33239, -1, 33240, 33241, 33242, -1, 33243, 33244, 33245, -1, 33246, 33247, 33248, -1, 33249, 33250, 33251, -1, 33252, 33253, 33254, -1, 33255, 33256, 33257, -1, 33258, 33259, 33260, -1, 33261, 33262, 33263, -1, 33264, 33265, 33266, -1, 33267, 33268, 33269, -1, 33270, 33271, 33272, -1, 33273, 33274, 33275, -1, 33276, 33277, 33278, -1, 33279, 33280, 33281, -1, 33282, 33283, 33284, -1, 33285, 33286, 33287, -1, 33288, 33289, 33290, -1, 33291, 33292, 33293, -1, 33294, 33295, 33296, -1, 33297, 33298, 33299, -1, 33300, 33301, 33302, -1, 33303, 33304, 33305, -1, 33306, 33307, 33308, -1, 33309, 33310, 33311, -1, 33312, 33313, 33314, -1, 33315, 33316, 33317, -1, 33318, 33319, 33320, -1, 33321, 33322, 33323, -1, 33324, 33325, 33326, -1, 33327, 33328, 33329, -1, 33330, 33331, 33332, -1, 33333, 33334, 33335, -1, 33336, 33337, 33338, -1, 33339, 33340, 33341, -1, 33342, 33343, 33344, -1, 33345, 33346, 33347, -1, 33348, 33349, 33350, -1, 33351, 33352, 33353, -1, 33354, 33355, 33356, -1, 33357, 33358, 33359, -1, 33360, 33361, 33362, -1, 33363, 33364, 33365, -1, 33366, 33367, 33368, -1, 33369, 33370, 33371, -1, 33372, 33373, 33374, -1, 33375, 33376, 33377, -1, 33378, 33379, 33380, -1, 33381, 33382, 33383, -1, 33384, 33385, 33386, -1, 33387, 33388, 33389, -1, 33390, 33391, 33392, -1, 33393, 33394, 33395, -1, 33396, 33397, 33398, -1, 33399, 33400, 33401, -1, 33402, 33403, 33404, -1, 33405, 33406, 33407, -1, 33408, 33409, 33410, -1, 33411, 33412, 33413, -1, 33414, 33415, 33416, -1, 33417, 33418, 33419, -1, 33420, 33421, 33422, -1, 33423, 33424, 33425, -1, 33426, 33427, 33428, -1, 33429, 33430, 33431, -1, 33432, 33433, 33434, -1, 33435, 33436, 33437, -1, 33438, 33439, 33440, -1, 33441, 33442, 33443, -1, 33444, 33445, 33446, -1, 33447, 33448, 33449, -1, 33450, 33451, 33452, -1, 33453, 33454, 33455, -1, 33456, 33457, 33458, -1, 33459, 33460, 33461, -1, 33462, 33463, 33464, -1, 33465, 33466, 33467, -1, 33468, 33469, 33470, -1, 33471, 33472, 33473, -1, 33474, 33475, 33476, -1, 33477, 33478, 33479, -1, 33480, 33481, 33482, -1, 33483, 33484, 33485, -1, 33486, 33487, 33488, -1, 33489, 33490, 33491, -1, 33492, 33493, 33494, -1, 33495, 33496, 33497, -1, 33498, 33499, 33500, -1, 33501, 33502, 33503, -1, 33504, 33505, 33506, -1, 33507, 33508, 33509, -1, 33510, 33511, 33512, -1, 33513, 33514, 33515, -1, 33516, 33517, 33518, -1, 33519, 33520, 33521, -1, 33522, 33523, 33524, -1, 33525, 33526, 33527, -1, 33528, 33529, 33530, -1, 33531, 33532, 33533, -1, 33534, 33535, 33536, -1, 33537, 33538, 33539, -1, 33540, 33541, 33542, -1, 33543, 33544, 33545, -1, 33546, 33547, 33548, -1, 33549, 33550, 33551, -1, 33552, 33553, 33554, -1, 33555, 33556, 33557, -1, 33558, 33559, 33560, -1, 33561, 33562, 33563, -1, 33564, 33565, 33566, -1, 33567, 33568, 33569, -1, 33570, 33571, 33572, -1, 33573, 33574, 33575, -1, 33576, 33577, 33578, -1, 33579, 33580, 33581, -1, 33582, 33583, 33584, -1, 33585, 33586, 33587, -1, 33588, 33589, 33590, -1, 33591, 33592, 33593, -1, 33594, 33595, 33596, -1, 33597, 33598, 33599, -1, 33600, 33601, 33602, -1, 33603, 33604, 33605, -1, 33606, 33607, 33608, -1, 33609, 33610, 33611, -1, 33612, 33613, 33614, -1, 33615, 33616, 33617, -1, 33618, 33619, 33620, -1, 33621, 33622, 33623, -1, 33624, 33625, 33626, -1, 33627, 33628, 33629, -1, 33630, 33631, 33632, -1, 33633, 33634, 33635, -1, 33636, 33637, 33638, -1, 33639, 33640, 33641, -1, 33642, 33643, 33644, -1, 33645, 33646, 33647, -1, 33648, 33649, 33650, -1, 33651, 33652, 33653, -1, 33654, 33655, 33656, -1, 33657, 33658, 33659, -1, 33660, 33661, 33662, -1, 33663, 33664, 33665, -1, 33666, 33667, 33668, -1, 33669, 33670, 33671, -1, 33672, 33673, 33674, -1, 33675, 33676, 33677, -1, 33678, 33679, 33680, -1, 33681, 33682, 33683, -1, 33684, 33685, 33686, -1, 33687, 33688, 33689, -1, 33690, 33691, 33692, -1, 33693, 33694, 33695, -1, 33696, 33697, 33698, -1, 33699, 33700, 33701, -1, 33702, 33703, 33704, -1, 33705, 33706, 33707, -1, 33708, 33709, 33710, -1, 33711, 33712, 33713, -1, 33714, 33715, 33716, -1, 33717, 33718, 33719, -1, 33720, 33721, 33722, -1, 33723, 33724, 33725, -1, 33726, 33727, 33728, -1, 33729, 33730, 33731, -1, 33732, 33733, 33734, -1, 33735, 33736, 33737, -1, 33738, 33739, 33740, -1, 33741, 33742, 33743, -1, 33744, 33745, 33746, -1, 33747, 33748, 33749, -1, 33750, 33751, 33752, -1, 33753, 33754, 33755, -1, 33756, 33757, 33758, -1, 33759, 33760, 33761, -1, 33762, 33763, 33764, -1, 33765, 33766, 33767, -1, 33768, 33769, 33770, -1, 33771, 33772, 33773, -1, 33774, 33775, 33776, -1, 33777, 33778, 33779, -1, 33780, 33781, 33782, -1, 33783, 33784, 33785, -1, 33786, 33787, 33788, -1, 33789, 33790, 33791, -1, 33792, 33793, 33794, -1, 33795, 33796, 33797, -1, 33798, 33799, 33800, -1, 33801, 33802, 33803, -1, 33804, 33805, 33806, -1, 33807, 33808, 33809, -1, 33810, 33811, 33812, -1, 33813, 33814, 33815, -1, 33816, 33817, 33818, -1, 33819, 33820, 33821, -1, 33822, 33823, 33824, -1, 33825, 33826, 33827, -1, 33828, 33829, 33830, -1, 33831, 33832, 33833, -1, 33834, 33835, 33836, -1, 33837, 33838, 33839, -1, 33840, 33841, 33842, -1, 33843, 33844, 33845, -1, 33846, 33847, 33848, -1, 33849, 33850, 33851, -1, 33852, 33853, 33854, -1, 33855, 33856, 33857, -1, 33858, 33859, 33860, -1, 33861, 33862, 33863, -1, 33864, 33865, 33866, -1, 33867, 33868, 33869, -1, 33870, 33871, 33872, -1, 33873, 33874, 33875, -1, 33876, 33877, 33878, -1, 33879, 33880, 33881, -1, 33882, 33883, 33884, -1, 33885, 33886, 33887, -1, 33888, 33889, 33890, -1, 33891, 33892, 33893, -1, 33894, 33895, 33896, -1, 33897, 33898, 33899, -1, 33900, 33901, 33902, -1, 33903, 33904, 33905, -1, 33906, 33907, 33908, -1, 33909, 33910, 33911, -1, 33912, 33913, 33914, -1, 33915, 33916, 33917, -1, 33918, 33919, 33920, -1, 33921, 33922, 33923, -1, 33924, 33925, 33926, -1, 33927, 33928, 33929, -1, 33930, 33931, 33932, -1, 33933, 33934, 33935, -1, 33936, 33937, 33938, -1, 33939, 33940, 33941, -1, 33942, 33943, 33944, -1, 33945, 33946, 33947, -1, 33948, 33949, 33950, -1, 33951, 33952, 33953, -1, 33954, 33955, 33956, -1, 33957, 33958, 33959, -1, 33960, 33961, 33962, -1, 33963, 33964, 33965, -1, 33966, 33967, 33968, -1, 33969, 33970, 33971, -1, 33972, 33973, 33974, -1, 33975, 33976, 33977, -1, 33978, 33979, 33980, -1, 33981, 33982, 33983, -1, 33984, 33985, 33986, -1, 33987, 33988, 33989, -1, 33990, 33991, 33992, -1, 33993, 33994, 33995, -1, 33996, 33997, 33998, -1, 33999, 34000, 34001, -1, 34002, 34003, 34004, -1, 34005, 34006, 34007, -1, 34008, 34009, 34010, -1, 34011, 34012, 34013, -1, 34014, 34015, 34016, -1, 34017, 34018, 34019, -1, 34020, 34021, 34022, -1, 34023, 34024, 34025, -1, 34026, 34027, 34028, -1, 34029, 34030, 34031, -1, 34032, 34033, 34034, -1, 34035, 34036, 34037, -1, 34038, 34039, 34040, -1, 34041, 34042, 34043, -1, 34044, 34045, 34046, -1, 34047, 34048, 34049, -1, 34050, 34051, 34052, -1, 34053, 34054, 34055, -1, 34056, 34057, 34058, -1, 34059, 34060, 34061, -1, 34062, 34063, 34064, -1, 34065, 34066, 34067, -1, 34068, 34069, 34070, -1, 34071, 34072, 34073, -1, 34074, 34075, 34076, -1, 34077, 34078, 34079, -1, 34080, 34081, 34082, -1, 34083, 34084, 34085, -1, 34086, 34087, 34088, -1, 34089, 34090, 34091, -1, 34092, 34093, 34094, -1, 34095, 34096, 34097, -1, 34098, 34099, 34100, -1, 34101, 34102, 34103, -1, 34104, 34105, 34106, -1, 34107, 34108, 34109, -1, 34110, 34111, 34112, -1, 34113, 34114, 34115, -1, 34116, 34117, 34118, -1, 34119, 34120, 34121, -1, 34122, 34123, 34124, -1, 34125, 34126, 34127, -1, 34128, 34129, 34130, -1, 34131, 34132, 34133, -1, 34134, 34135, 34136, -1, 34137, 34138, 34139, -1, 34140, 34141, 34142, -1, 34143, 34144, 34145, -1, 34146, 34147, 34148, -1, 34149, 34150, 34151, -1, 34152, 34153, 34154, -1, 34155, 34156, 34157, -1, 34158, 34159, 34160, -1, 34161, 34162, 34163, -1, 34164, 34165, 34166, -1, 34167, 34168, 34169, -1, 34170, 34171, 34172, -1, 34173, 34174, 34175, -1, 34176, 34177, 34178, -1, 34179, 34180, 34181, -1, 34182, 34183, 34184, -1, 34185, 34186, 34187, -1, 34188, 34189, 34190, -1, 34191, 34192, 34193, -1, 34194, 34195, 34196, -1, 34197, 34198, 34199, -1, 34200, 34201, 34202, -1, 34203, 34204, 34205, -1, 34206, 34207, 34208, -1, 34209, 34210, 34211, -1, 34212, 34213, 34214, -1, 34215, 34216, 34217, -1, 34218, 34219, 34220, -1, 34221, 34222, 34223, -1, 34224, 34225, 34226, -1, 34227, 34228, 34229, -1, 34230, 34231, 34232, -1, 34233, 34234, 34235, -1, 34236, 34237, 34238, -1, 34239, 34240, 34241, -1, 34242, 34243, 34244, -1, 34245, 34246, 34247, -1, 34248, 34249, 34250, -1, 34251, 34252, 34253, -1, 34254, 34255, 34256, -1, 34257, 34258, 34259, -1, 34260, 34261, 34262, -1, 34263, 34264, 34265, -1, 34266, 34267, 34268, -1, 34269, 34270, 34271, -1, 34272, 34273, 34274, -1, 34275, 34276, 34277, -1, 34278, 34279, 34280, -1, 34281, 34282, 34283, -1, 34284, 34285, 34286, -1, 34287, 34288, 34289, -1, 34290, 34291, 34292, -1, 34293, 34294, 34295, -1, 34296, 34297, 34298, -1, 34299, 34300, 34301, -1, 34302, 34303, 34304, -1, 34305, 34306, 34307, -1, 34308, 34309, 34310, -1, 34311, 34312, 34313, -1, 34314, 34315, 34316, -1, 34317, 34318, 34319, -1, 34320, 34321, 34322, -1, 34323, 34324, 34325, -1, 34326, 34327, 34328, -1, 34329, 34330, 34331, -1, 34332, 34333, 34334, -1, 34335, 34336, 34337, -1, 34338, 34339, 34340, -1, 34341, 34342, 34343, -1, 34344, 34345, 34346, -1, 34347, 34348, 34349, -1, 34350, 34351, 34352, -1, 34353, 34354, 34355, -1, 34356, 34357, 34358, -1, 34359, 34360, 34361, -1, 34362, 34363, 34364, -1, 34365, 34366, 34367, -1, 34368, 34369, 34370, -1, 34371, 34372, 34373, -1, 34374, 34375, 34376, -1, 34377, 34378, 34379, -1, 34380, 34381, 34382, -1, 34383, 34384, 34385, -1, 34386, 34387, 34388, -1, 34389, 34390, 34391, -1, 34392, 34393, 34394, -1, 34395, 34396, 34397, -1, 34398, 34399, 34400, -1, 34401, 34402, 34403, -1, 34404, 34405, 34406, -1, 34407, 34408, 34409, -1, 34410, 34411, 34412, -1, 34413, 34414, 34415, -1, 34416, 34417, 34418, -1, 34419, 34420, 34421, -1, 34422, 34423, 34424, -1, 34425, 34426, 34427, -1, 34428, 34429, 34430, -1, 34431, 34432, 34433, -1, 34434, 34435, 34436, -1, 34437, 34438, 34439, -1, 34440, 34441, 34442, -1, 34443, 34444, 34445, -1, 34446, 34447, 34448, -1, 34449, 34450, 34451, -1, 34452, 34453, 34454, -1, 34455, 34456, 34457, -1, 34458, 34459, 34460, -1, 34461, 34462, 34463, -1, 34464, 34465, 34466, -1, 34467, 34468, 34469, -1, 34470, 34471, 34472, -1, 34473, 34474, 34475, -1, 34476, 34477, 34478, -1, 34479, 34480, 34481, -1, 34482, 34483, 34484, -1, 34485, 34486, 34487, -1, 34488, 34489, 34490, -1, 34491, 34492, 34493, -1, 34494, 34495, 34496, -1, 34497, 34498, 34499, -1, 34500, 34501, 34502, -1, 34503, 34504, 34505, -1, 34506, 34507, 34508, -1, 34509, 34510, 34511, -1, 34512, 34513, 34514, -1, 34515, 34516, 34517, -1, 34518, 34519, 34520, -1, 34521, 34522, 34523, -1, 34524, 34525, 34526, -1, 34527, 34528, 34529, -1, 34530, 34531, 34532, -1, 34533, 34534, 34535, -1, 34536, 34537, 34538, -1, 34539, 34540, 34541, -1, 34542, 34543, 34544, -1, 34545, 34546, 34547, -1, 34548, 34549, 34550, -1, 34551, 34552, 34553, -1, 34554, 34555, 34556, -1, 34557, 34558, 34559, -1, 34560, 34561, 34562, -1, 34563, 34564, 34565, -1, 34566, 34567, 34568, -1, 34569, 34570, 34571, -1, 34572, 34573, 34574, -1, 34575, 34576, 34577, -1, 34578, 34579, 34580, -1, 34581, 34582, 34583, -1, 34584, 34585, 34586, -1, 34587, 34588, 34589, -1, 34590, 34591, 34592, -1, 34593, 34594, 34595, -1, 34596, 34597, 34598, -1, 34599, 34600, 34601, -1, 34602, 34603, 34604, -1, 34605, 34606, 34607, -1, 34608, 34609, 34610, -1, 34611, 34612, 34613, -1, 34614, 34615, 34616, -1, 34617, 34618, 34619, -1, 34620, 34621, 34622, -1, 34623, 34624, 34625, -1, 34626, 34627, 34628, -1, 34629, 34630, 34631, -1, 34632, 34633, 34634, -1, 34635, 34636, 34637, -1, 34638, 34639, 34640, -1, 34641, 34642, 34643, -1, 34644, 34645, 34646, -1, 34647, 34648, 34649, -1, 34650, 34651, 34652, -1, 34653, 34654, 34655, -1, 34656, 34657, 34658, -1, 34659, 34660, 34661, -1, 34662, 34663, 34664, -1, 34665, 34666, 34667, -1, 34668, 34669, 34670, -1, 34671, 34672, 34673, -1, 34674, 34675, 34676, -1, 34677, 34678, 34679, -1, 34680, 34681, 34682, -1, 34683, 34684, 34685, -1, 34686, 34687, 34688, -1, 34689, 34690, 34691, -1, 34692, 34693, 34694, -1, 34695, 34696, 34697, -1, 34698, 34699, 34700, -1, 34701, 34702, 34703, -1, 34704, 34705, 34706, -1, 34707, 34708, 34709, -1, 34710, 34711, 34712, -1, 34713, 34714, 34715, -1, 34716, 34717, 34718, -1, 34719, 34720, 34721, -1, 34722, 34723, 34724, -1, 34725, 34726, 34727, -1, 34728, 34729, 34730, -1, 34731, 34732, 34733, -1, 34734, 34735, 34736, -1, 34737, 34738, 34739, -1, 34740, 34741, 34742, -1, 34743, 34744, 34745, -1, 34746, 34747, 34748, -1, 34749, 34750, 34751, -1, 34752, 34753, 34754, -1, 34755, 34756, 34757, -1, 34758, 34759, 34760, -1, 34761, 34762, 34763, -1, 34764, 34765, 34766, -1, 34767, 34768, 34769, -1, 34770, 34771, 34772, -1, 34773, 34774, 34775, -1, 34776, 34777, 34778, -1, 34779, 34780, 34781, -1, 34782, 34783, 34784, -1, 34785, 34786, 34787, -1, 34788, 34789, 34790, -1, 34791, 34792, 34793, -1, 34794, 34795, 34796, -1, 34797, 34798, 34799, -1, 34800, 34801, 34802, -1, 34803, 34804, 34805, -1, 34806, 34807, 34808, -1, 34809, 34810, 34811, -1, 34812, 34813, 34814, -1, 34815, 34816, 34817, -1, 34818, 34819, 34820, -1, 34821, 34822, 34823, -1, 34824, 34825, 34826, -1, 34827, 34828, 34829, -1, 34830, 34831, 34832, -1, 34833, 34834, 34835, -1, 34836, 34837, 34838, -1, 34839, 34840, 34841, -1, 34842, 34843, 34844, -1, 34845, 34846, 34847, -1, 34848, 34849, 34850, -1, 34851, 34852, 34853, -1, 34854, 34855, 34856, -1, 34857, 34858, 34859, -1, 34860, 34861, 34862, -1, 34863, 34864, 34865, -1, 34866, 34867, 34868, -1, 34869, 34870, 34871, -1, 34872, 34873, 34874, -1, 34875, 34876, 34877, -1, 34878, 34879, 34880, -1, 34881, 34882, 34883, -1, 34884, 34885, 34886, -1, 34887, 34888, 34889, -1, 34890, 34891, 34892, -1, 34893, 34894, 34895, -1, 34896, 34897, 34898, -1, 34899, 34900, 34901, -1, 34902, 34903, 34904, -1, 34905, 34906, 34907, -1, 34908, 34909, 34910, -1, 34911, 34912, 34913, -1, 34914, 34915, 34916, -1, 34917, 34918, 34919, -1, 34920, 34921, 34922, -1, 34923, 34924, 34925, -1, 34926, 34927, 34928, -1, 34929, 34930, 34931, -1, 34932, 34933, 34934, -1, 34935, 34936, 34937, -1, 34938, 34939, 34940, -1, 34941, 34942, 34943, -1, 34944, 34945, 34946, -1, 34947, 34948, 34949, -1, 34950, 34951, 34952, -1, 34953, 34954, 34955, -1, 34956, 34957, 34958, -1, 34959, 34960, 34961, -1, 34962, 34963, 34964, -1, 34965, 34966, 34967, -1, 34968, 34969, 34970, -1, 34971, 34972, 34973, -1, 34974, 34975, 34976, -1, 34977, 34978, 34979, -1, 34980, 34981, 34982, -1, 34983, 34984, 34985, -1, 34986, 34987, 34988, -1, 34989, 34990, 34991, -1, 34992, 34993, 34994, -1, 34995, 34996, 34997, -1, 34998, 34999, 35000, -1, 35001, 35002, 35003, -1, 35004, 35005, 35006, -1, 35007, 35008, 35009, -1, 35010, 35011, 35012, -1, 35013, 35014, 35015, -1, 35016, 35017, 35018, -1, 35019, 35020, 35021, -1, 35022, 35023, 35024, -1, 35025, 35026, 35027, -1, 35028, 35029, 35030, -1, 35031, 35032, 35033, -1, 35034, 35035, 35036, -1, 35037, 35038, 35039, -1, 35040, 35041, 35042, -1, 35043, 35044, 35045, -1, 35046, 35047, 35048, -1, 35049, 35050, 35051, -1, 35052, 35053, 35054, -1, 35055, 35056, 35057, -1, 35058, 35059, 35060, -1, 35061, 35062, 35063, -1, 35064, 35065, 35066, -1, 35067, 35068, 35069, -1, 35070, 35071, 35072, -1, 35073, 35074, 35075, -1, 35076, 35077, 35078, -1, 35079, 35080, 35081, -1, 35082, 35083, 35084, -1, 35085, 35086, 35087, -1, 35088, 35089, 35090, -1, 35091, 35092, 35093, -1, 35094, 35095, 35096, -1, 35097, 35098, 35099, -1, 35100, 35101, 35102, -1, 35103, 35104, 35105, -1, 35106, 35107, 35108, -1, 35109, 35110, 35111, -1, 35112, 35113, 35114, -1, 35115, 35116, 35117, -1, 35118, 35119, 35120, -1, 35121, 35122, 35123, -1, 35124, 35125, 35126, -1, 35127, 35128, 35129, -1, 35130, 35131, 35132, -1, 35133, 35134, 35135, -1, 35136, 35137, 35138, -1, 35139, 35140, 35141, -1, 35142, 35143, 35144, -1, 35145, 35146, 35147, -1, 35148, 35149, 35150, -1, 35151, 35152, 35153, -1, 35154, 35155, 35156, -1, 35157, 35158, 35159, -1, 35160, 35161, 35162, -1, 35163, 35164, 35165, -1, 35166, 35167, 35168, -1, 35169, 35170, 35171, -1, 35172, 35173, 35174, -1, 35175, 35176, 35177, -1, 35178, 35179, 35180, -1, 35181, 35182, 35183, -1, 35184, 35185, 35186, -1, 35187, 35188, 35189, -1, 35190, 35191, 35192, -1, 35193, 35194, 35195, -1, 35196, 35197, 35198, -1, 35199, 35200, 35201, -1, 35202, 35203, 35204, -1, 35205, 35206, 35207, -1, 35208, 35209, 35210, -1, 35211, 35212, 35213, -1, 35214, 35215, 35216, -1, 35217, 35218, 35219, -1, 35220, 35221, 35222, -1, 35223, 35224, 35225, -1, 35226, 35227, 35228, -1, 35229, 35230, 35231, -1, 35232, 35233, 35234, -1, 35235, 35236, 35237, -1, 35238, 35239, 35240, -1, 35241, 35242, 35243, -1, 35244, 35245, 35246, -1, 35247, 35248, 35249, -1, 35250, 35251, 35252, -1, 35253, 35254, 35255, -1, 35256, 35257, 35258, -1, 35259, 35260, 35261, -1, 35262, 35263, 35264, -1, 35265, 35266, 35267, -1, 35268, 35269, 35270, -1, 35271, 35272, 35273, -1, 35274, 35275, 35276, -1, 35277, 35278, 35279, -1, 35280, 35281, 35282, -1, 35283, 35284, 35285, -1, 35286, 35287, 35288, -1, 35289, 35290, 35291, -1, 35292, 35293, 35294, -1, 35295, 35296, 35297, -1, 35298, 35299, 35300, -1, 35301, 35302, 35303, -1, 35304, 35305, 35306, -1, 35307, 35308, 35309, -1, 35310, 35311, 35312, -1, 35313, 35314, 35315, -1, 35316, 35317, 35318, -1, 35319, 35320, 35321, -1, 35322, 35323, 35324, -1, 35325, 35326, 35327, -1, 35328, 35329, 35330, -1, 35331, 35332, 35333, -1, 35334, 35335, 35336, -1, 35337, 35338, 35339, -1, 35340, 35341, 35342, -1, 35343, 35344, 35345, -1, 35346, 35347, 35348, -1, 35349, 35350, 35351, -1, 35352, 35353, 35354, -1, 35355, 35356, 35357, -1, 35358, 35359, 35360, -1, 35361, 35362, 35363, -1, 35364, 35365, 35366, -1, 35367, 35368, 35369, -1, 35370, 35371, 35372, -1, 35373, 35374, 35375, -1, 35376, 35377, 35378, -1, 35379, 35380, 35381, -1, 35382, 35383, 35384, -1, 35385, 35386, 35387, -1, 35388, 35389, 35390, -1, 35391, 35392, 35393, -1, 35394, 35395, 35396, -1, 35397, 35398, 35399, -1, 35400, 35401, 35402, -1, 35403, 35404, 35405, -1, 35406, 35407, 35408, -1, 35409, 35410, 35411, -1, 35412, 35413, 35414, -1, 35415, 35416, 35417, -1, 35418, 35419, 35420, -1, 35421, 35422, 35423, -1, 35424, 35425, 35426, -1, 35427, 35428, 35429, -1, 35430, 35431, 35432, -1, 35433, 35434, 35435, -1, 35436, 35437, 35438, -1, 35439, 35440, 35441, -1, 35442, 35443, 35444, -1, 35445, 35446, 35447, -1, 35448, 35449, 35450, -1, 35451, 35452, 35453, -1, 35454, 35455, 35456, -1, 35457, 35458, 35459, -1, 35460, 35461, 35462, -1, 35463, 35464, 35465, -1, 35466, 35467, 35468, -1, 35469, 35470, 35471, -1, 35472, 35473, 35474, -1, 35475, 35476, 35477, -1, 35478, 35479, 35480, -1, 35481, 35482, 35483, -1, 35484, 35485, 35486, -1, 35487, 35488, 35489, -1, 35490, 35491, 35492, -1, 35493, 35494, 35495, -1, 35496, 35497, 35498, -1, 35499, 35500, 35501, -1, 35502, 35503, 35504, -1, 35505, 35506, 35507, -1, 35508, 35509, 35510, -1, 35511, 35512, 35513, -1, 35514, 35515, 35516, -1, 35517, 35518, 35519, -1, 35520, 35521, 35522, -1, 35523, 35524, 35525, -1, 35526, 35527, 35528, -1, 35529, 35530, 35531, -1, 35532, 35533, 35534, -1, 35535, 35536, 35537, -1, 35538, 35539, 35540, -1, 35541, 35542, 35543, -1, 35544, 35545, 35546, -1, 35547, 35548, 35549, -1, 35550, 35551, 35552, -1, 35553, 35554, 35555, -1, 35556, 35557, 35558, -1, 35559, 35560, 35561, -1, 35562, 35563, 35564, -1, 35565, 35566, 35567, -1, 35568, 35569, 35570, -1, 35571, 35572, 35573, -1, 35574, 35575, 35576, -1, 35577, 35578, 35579, -1, 35580, 35581, 35582, -1, 35583, 35584, 35585, -1, 35586, 35587, 35588, -1, 35589, 35590, 35591, -1, 35592, 35593, 35594, -1, 35595, 35596, 35597, -1, 35598, 35599, 35600, -1, 35601, 35602, 35603, -1, 35604, 35605, 35606, -1, 35607, 35608, 35609, -1, 35610, 35611, 35612, -1, 35613, 35614, 35615, -1, 35616, 35617, 35618, -1, 35619, 35620, 35621, -1, 35622, 35623, 35624, -1, 35625, 35626, 35627, -1, 35628, 35629, 35630, -1, 35631, 35632, 35633, -1, 35634, 35635, 35636, -1, 35637, 35638, 35639, -1, 35640, 35641, 35642, -1, 35643, 35644, 35645, -1, 35646, 35647, 35648, -1, 35649, 35650, 35651, -1, 35652, 35653, 35654, -1, 35655, 35656, 35657, -1, 35658, 35659, 35660, -1, 35661, 35662, 35663, -1, 35664, 35665, 35666, -1, 35667, 35668, 35669, -1, 35670, 35671, 35672, -1, 35673, 35674, 35675, -1, 35676, 35677, 35678, -1, 35679, 35680, 35681, -1, 35682, 35683, 35684, -1, 35685, 35686, 35687, -1, 35688, 35689, 35690, -1, 35691, 35692, 35693, -1, 35694, 35695, 35696, -1, 35697, 35698, 35699, -1, 35700, 35701, 35702, -1, 35703, 35704, 35705, -1, 35706, 35707, 35708, -1, 35709, 35710, 35711, -1, 35712, 35713, 35714, -1, 35715, 35716, 35717, -1, 35718, 35719, 35720, -1, 35721, 35722, 35723, -1, 35724, 35725, 35726, -1, 35727, 35728, 35729, -1, 35730, 35731, 35732, -1, 35733, 35734, 35735, -1, 35736, 35737, 35738, -1, 35739, 35740, 35741, -1, 35742, 35743, 35744, -1, 35745, 35746, 35747, -1, 35748, 35749, 35750, -1, 35751, 35752, 35753, -1, 35754, 35755, 35756, -1, 35757, 35758, 35759, -1, 35760, 35761, 35762, -1, 35763, 35764, 35765, -1, 35766, 35767, 35768, -1, 35769, 35770, 35771, -1, 35772, 35773, 35774, -1, 35775, 35776, 35777, -1, 35778, 35779, 35780, -1, 35781, 35782, 35783, -1, 35784, 35785, 35786, -1, 35787, 35788, 35789, -1, 35790, 35791, 35792, -1, 35793, 35794, 35795, -1, 35796, 35797, 35798, -1, 35799, 35800, 35801, -1, 35802, 35803, 35804, -1, 35805, 35806, 35807, -1, 35808, 35809, 35810, -1, 35811, 35812, 35813, -1, 35814, 35815, 35816, -1, 35817, 35818, 35819, -1, 35820, 35821, 35822, -1, 35823, 35824, 35825, -1, 35826, 35827, 35828, -1, 35829, 35830, 35831, -1, 35832, 35833, 35834, -1, 35835, 35836, 35837, -1, 35838, 35839, 35840, -1, 35841, 35842, 35843, -1, 35844, 35845, 35846, -1, 35847, 35848, 35849, -1, 35850, 35851, 35852, -1, 35853, 35854, 35855, -1, 35856, 35857, 35858, -1, 35859, 35860, 35861, -1, 35862, 35863, 35864, -1, 35865, 35866, 35867, -1, 35868, 35869, 35870, -1, 35871, 35872, 35873, -1, 35874, 35875, 35876, -1, 35877, 35878, 35879, -1, 35880, 35881, 35882, -1, 35883, 35884, 35885, -1, 35886, 35887, 35888, -1, 35889, 35890, 35891, -1, 35892, 35893, 35894, -1, 35895, 35896, 35897, -1, 35898, 35899, 35900, -1, 35901, 35902, 35903, -1, 35904, 35905, 35906, -1, 35907, 35908, 35909, -1, 35910, 35911, 35912, -1, 35913, 35914, 35915, -1, 35916, 35917, 35918, -1, 35919, 35920, 35921, -1, 35922, 35923, 35924, -1, 35925, 35926, 35927, -1, 35928, 35929, 35930, -1, 35931, 35932, 35933, -1, 35934, 35935, 35936, -1, 35937, 35938, 35939, -1, 35940, 35941, 35942, -1, 35943, 35944, 35945, -1, 35946, 35947, 35948, -1, 35949, 35950, 35951, -1, 35952, 35953, 35954, -1, 35955, 35956, 35957, -1, 35958, 35959, 35960, -1, 35961, 35962, 35963, -1, 35964, 35965, 35966, -1, 35967, 35968, 35969, -1, 35970, 35971, 35972, -1, 35973, 35974, 35975, -1, 35976, 35977, 35978, -1, 35979, 35980, 35981, -1, 35982, 35983, 35984, -1, 35985, 35986, 35987, -1, 35988, 35989, 35990, -1, 35991, 35992, 35993, -1, 35994, 35995, 35996, -1, 35997, 35998, 35999, -1, 36000, 36001, 36002, -1, 36003, 36004, 36005, -1, 36006, 36007, 36008, -1, 36009, 36010, 36011, -1, 36012, 36013, 36014, -1, 36015, 36016, 36017, -1, 36018, 36019, 36020, -1, 36021, 36022, 36023, -1, 36024, 36025, 36026, -1, 36027, 36028, 36029, -1, 36030, 36031, 36032, -1, 36033, 36034, 36035, -1, 36036, 36037, 36038, -1, 36039, 36040, 36041, -1, 36042, 36043, 36044, -1, 36045, 36046, 36047, -1, 36048, 36049, 36050, -1, 36051, 36052, 36053, -1, 36054, 36055, 36056, -1, 36057, 36058, 36059, -1, 36060, 36061, 36062, -1, 36063, 36064, 36065, -1, 36066, 36067, 36068, -1, 36069, 36070, 36071, -1, 36072, 36073, 36074, -1, 36075, 36076, 36077, -1, 36078, 36079, 36080, -1, 36081, 36082, 36083, -1, 36084, 36085, 36086, -1, 36087, 36088, 36089, -1, 36090, 36091, 36092, -1, 36093, 36094, 36095, -1, 36096, 36097, 36098, -1, 36099, 36100, 36101, -1, 36102, 36103, 36104, -1, 36105, 36106, 36107, -1, 36108, 36109, 36110, -1, 36111, 36112, 36113, -1, 36114, 36115, 36116, -1, 36117, 36118, 36119, -1, 36120, 36121, 36122, -1, 36123, 36124, 36125, -1, 36126, 36127, 36128, -1, 36129, 36130, 36131, -1, 36132, 36133, 36134, -1, 36135, 36136, 36137, -1, 36138, 36139, 36140, -1, 36141, 36142, 36143, -1, 36144, 36145, 36146, -1, 36147, 36148, 36149, -1, 36150, 36151, 36152, -1, 36153, 36154, 36155, -1, 36156, 36157, 36158, -1, 36159, 36160, 36161, -1, 36162, 36163, 36164, -1, 36165, 36166, 36167, -1, 36168, 36169, 36170, -1, 36171, 36172, 36173, -1, 36174, 36175, 36176, -1, 36177, 36178, 36179, -1, 36180, 36181, 36182, -1, 36183, 36184, 36185, -1, 36186, 36187, 36188, -1, 36189, 36190, 36191, -1, 36192, 36193, 36194, -1, 36195, 36196, 36197, -1, 36198, 36199, 36200, -1, 36201, 36202, 36203, -1, 36204, 36205, 36206, -1, 36207, 36208, 36209, -1, 36210, 36211, 36212, -1, 36213, 36214, 36215, -1, 36216, 36217, 36218, -1, 36219, 36220, 36221, -1, 36222, 36223, 36224, -1, 36225, 36226, 36227, -1, 36228, 36229, 36230, -1, 36231, 36232, 36233, -1, 36234, 36235, 36236, -1, 36237, 36238, 36239, -1, 36240, 36241, 36242, -1, 36243, 36244, 36245, -1, 36246, 36247, 36248, -1, 36249, 36250, 36251, -1, 36252, 36253, 36254, -1, 36255, 36256, 36257, -1, 36258, 36259, 36260, -1, 36261, 36262, 36263, -1, 36264, 36265, 36266, -1, 36267, 36268, 36269, -1, 36270, 36271, 36272, -1, 36273, 36274, 36275, -1, 36276, 36277, 36278, -1, 36279, 36280, 36281, -1, 36282, 36283, 36284, -1, 36285, 36286, 36287, -1, 36288, 36289, 36290, -1, 36291, 36292, 36293, -1, 36294, 36295, 36296, -1, 36297, 36298, 36299, -1, 36300, 36301, 36302, -1, 36303, 36304, 36305, -1, 36306, 36307, 36308, -1, 36309, 36310, 36311, -1, 36312, 36313, 36314, -1, 36315, 36316, 36317, -1, 36318, 36319, 36320, -1, 36321, 36322, 36323, -1, 36324, 36325, 36326, -1, 36327, 36328, 36329, -1, 36330, 36331, 36332, -1, 36333, 36334, 36335, -1, 36336, 36337, 36338, -1, 36339, 36340, 36341, -1, 36342, 36343, 36344, -1, 36345, 36346, 36347, -1, 36348, 36349, 36350, -1, 36351, 36352, 36353, -1, 36354, 36355, 36356, -1, 36357, 36358, 36359, -1, 36360, 36361, 36362, -1, 36363, 36364, 36365, -1, 36366, 36367, 36368, -1, 36369, 36370, 36371, -1, 36372, 36373, 36374, -1, 36375, 36376, 36377, -1, 36378, 36379, 36380, -1, 36381, 36382, 36383, -1, 36384, 36385, 36386, -1, 36387, 36388, 36389, -1, 36390, 36391, 36392, -1, 36393, 36394, 36395, -1, 36396, 36397, 36398, -1, 36399, 36400, 36401, -1, 36402, 36403, 36404, -1, 36405, 36406, 36407, -1, 36408, 36409, 36410, -1, 36411, 36412, 36413, -1, 36414, 36415, 36416, -1, 36417, 36418, 36419, -1, 36420, 36421, 36422, -1, 36423, 36424, 36425, -1, 36426, 36427, 36428, -1, 36429, 36430, 36431, -1, 36432, 36433, 36434, -1, 36435, 36436, 36437, -1, 36438, 36439, 36440, -1, 36441, 36442, 36443, -1, 36444, 36445, 36446, -1, 36447, 36448, 36449, -1, 36450, 36451, 36452, -1, 36453, 36454, 36455, -1, 36456, 36457, 36458, -1, 36459, 36460, 36461, -1, 36462, 36463, 36464, -1, 36465, 36466, 36467, -1, 36468, 36469, 36470, -1, 36471, 36472, 36473, -1, 36474, 36475, 36476, -1, 36477, 36478, 36479, -1, 36480, 36481, 36482, -1, 36483, 36484, 36485, -1, 36486, 36487, 36488, -1, 36489, 36490, 36491, -1, 36492, 36493, 36494, -1, 36495, 36496, 36497, -1, 36498, 36499, 36500, -1, 36501, 36502, 36503, -1, 36504, 36505, 36506, -1, 36507, 36508, 36509, -1, 36510, 36511, 36512, -1, 36513, 36514, 36515, -1, 36516, 36517, 36518, -1, 36519, 36520, 36521, -1, 36522, 36523, 36524, -1, 36525, 36526, 36527, -1, 36528, 36529, 36530, -1, 36531, 36532, 36533, -1, 36534, 36535, 36536, -1, 36537, 36538, 36539, -1, 36540, 36541, 36542, -1, 36543, 36544, 36545, -1, 36546, 36547, 36548, -1, 36549, 36550, 36551, -1, 36552, 36553, 36554, -1, 36555, 36556, 36557, -1, 36558, 36559, 36560, -1, 36561, 36562, 36563, -1, 36564, 36565, 36566, -1, 36567, 36568, 36569, -1, 36570, 36571, 36572, -1, 36573, 36574, 36575, -1, 36576, 36577, 36578, -1, 36579, 36580, 36581, -1, 36582, 36583, 36584, -1, 36585, 36586, 36587, -1, 36588, 36589, 36590, -1, 36591, 36592, 36593, -1, 36594, 36595, 36596, -1, 36597, 36598, 36599, -1, 36600, 36601, 36602, -1, 36603, 36604, 36605, -1, 36606, 36607, 36608, -1, 36609, 36610, 36611, -1, 36612, 36613, 36614, -1, 36615, 36616, 36617, -1, 36618, 36619, 36620, -1, 36621, 36622, 36623, -1, 36624, 36625, 36626, -1, 36627, 36628, 36629, -1, 36630, 36631, 36632, -1, 36633, 36634, 36635, -1, 36636, 36637, 36638, -1, 36639, 36640, 36641, -1, 36642, 36643, 36644, -1, 36645, 36646, 36647, -1, 36648, 36649, 36650, -1, 36651, 36652, 36653, -1, 36654, 36655, 36656, -1, 36657, 36658, 36659, -1, 36660, 36661, 36662, -1, 36663, 36664, 36665, -1, 36666, 36667, 36668, -1, 36669, 36670, 36671, -1, 36672, 36673, 36674, -1, 36675, 36676, 36677, -1, 36678, 36679, 36680, -1, 36681, 36682, 36683, -1, 36684, 36685, 36686, -1, 36687, 36688, 36689, -1, 36690, 36691, 36692, -1, 36693, 36694, 36695, -1, 36696, 36697, 36698, -1, 36699, 36700, 36701, -1, 36702, 36703, 36704, -1, 36705, 36706, 36707, -1, 36708, 36709, 36710, -1, 36711, 36712, 36713, -1, 36714, 36715, 36716, -1, 36717, 36718, 36719, -1, 36720, 36721, 36722, -1, 36723, 36724, 36725, -1, 36726, 36727, 36728, -1, 36729, 36730, 36731, -1, 36732, 36733, 36734, -1, 36735, 36736, 36737, -1, 36738, 36739, 36740, -1, 36741, 36742, 36743, -1, 36744, 36745, 36746, -1, 36747, 36748, 36749, -1, 36750, 36751, 36752, -1, 36753, 36754, 36755, -1, 36756, 36757, 36758, -1, 36759, 36760, 36761, -1, 36762, 36763, 36764, -1, 36765, 36766, 36767, -1, 36768, 36769, 36770, -1, 36771, 36772, 36773, -1, 36774, 36775, 36776, -1, 36777, 36778, 36779, -1, 36780, 36781, 36782, -1, 36783, 36784, 36785, -1 };
    Text1m_5.texCoordIndex = m_texCoordIndex;
    Text1m_5.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    Text1m_5.set_colorIndex = NULL;
    Text1m_5.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    Text1m_5.set_coordIndex = NULL;
    Text1m_5.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    Text1m_5.set_normalIndex = NULL;
    Text1m_5.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    Text1m_5.set_texCoordIndex = NULL;
    Text1m_5.set_texCoordIndex_length = 0;
    }
    Shape_84.appearance = &Appearance_85;
    Shape_84.geometry = &Text1m_5;
    DUMMYm_TEXT.m_parent = (X3dNode *)&Transform_83;
    DUMMYm_TEXT.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    DUMMYm_TEXT.hitNormal_changed = m_hitNormal_changed;
    DUMMYm_TEXT.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    DUMMYm_TEXT.hitPoint_changed = m_hitPoint_changed;
    DUMMYm_TEXT.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    DUMMYm_TEXT.hitTexCoord_changed = m_hitTexCoord_changed;
    DUMMYm_TEXT.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    DUMMYm_TEXT.isActive = false;
    DUMMYm_TEXT.isOver = false;
    DUMMYm_TEXT.touchTime = 0;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_83.rotation = m_rotation;
    Transform_83.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -10.787153, 0, -18.900000 };
    Transform_83.translation = m_translation;
    Transform_83.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_83.bboxCenter = m_bboxCenter;
    Transform_83.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_83.bboxSize = m_bboxSize;
    Transform_83.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_83.center = m_center;
    Transform_83.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_83.scale = m_scale;
    Transform_83.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_83.scaleOrientation = m_scaleOrientation;
    Transform_83.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_84, &DUMMYm_TEXT };
    Transform_83.children = m_children;
    Transform_83.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_83.addChildren = NULL;
    Transform_83.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_83.removeChildren = NULL;
    Transform_83.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_83 };
    Switch1.children = m_children;
    Switch1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    Switch1.whichChoice = -1;
    {
    static X3dNode* m_children[] = { &Switch1 };
    Group_82.children = m_children;
    Group_82.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_82.bboxCenter = m_bboxCenter;
    Group_82.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_82.bboxSize = m_bboxSize;
    Group_82.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_82.addChildren = NULL;
    Group_82.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_82.removeChildren = NULL;
    Group_82.removeChildren_length = 0;
    }
    Shape_87.m_parent = (X3dNode *)&Group_71;
    Appearance_88.m_parent = (X3dNode *)&Shape_87;
    Material_89.m_parent = (X3dNode *)&Appearance_88;
    Material_89.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0, 0, 1 };
    Material_89.diffuseColor = m_diffuseColor;
    Material_89.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_89.emissiveColor = m_emissiveColor;
    Material_89.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_89.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_89.specularColor = m_specularColor;
    Material_89.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_89.transparency = 0;
    Appearance_88.material = &Material_89;
    Appearance_88.texture = NULL;
    Appearance_88.textureTransform = NULL;
    IndexedFaceSet_29.setGlName(16);
    IndexedFaceSet_29.m_parent = (X3dNode *)&Shape_87;
    COORD_5.m_parent = (X3dNode *)&IndexedFaceSet_29;
    {
    static float m_point[] = { -600, -600, -35, 600, -600, -35, -600, 600, -35, 600, 600, -35 };
    COORD_5.point = m_point;
    COORD_5.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal16_5.m_parent = (X3dNode *)&IndexedFaceSet_29;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal16_5.vector = m_vector;
    Normal16_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_29.color = NULL;
    IndexedFaceSet_29.coord = &COORD_5;
    IndexedFaceSet_29.normal = &Normal16_5;
    IndexedFaceSet_29.texCoord = NULL;
    IndexedFaceSet_29.ccw = true;
    {
    static int m_colorIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_29.colorIndex = m_colorIndex;
    IndexedFaceSet_29.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_29.colorPerVertex = true;
    IndexedFaceSet_29.convex = true;
    {
    static int m_coordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_29.coordIndex = m_coordIndex;
    IndexedFaceSet_29.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_29.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_29.normalIndex = m_normalIndex;
    IndexedFaceSet_29.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_29.normalPerVertex = true;
    IndexedFaceSet_29.solid = true;
    {
    static int m_texCoordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_29.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_29.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_29.set_colorIndex = NULL;
    IndexedFaceSet_29.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_29.set_coordIndex = NULL;
    IndexedFaceSet_29.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_29.set_normalIndex = NULL;
    IndexedFaceSet_29.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_29.set_texCoordIndex = NULL;
    IndexedFaceSet_29.set_texCoordIndex_length = 0;
    }
    Shape_87.appearance = &Appearance_88;
    Shape_87.geometry = &IndexedFaceSet_29;
    BOX2m_HIDE.route_sources =  NULL;
    BOX2m_HIDE.num_route_source = 0;
    BOX2m_HIDE.m_parent = (X3dNode *)&Group_71;
    BOX2m_SWITCH.m_parent = (X3dNode *)&BOX2m_HIDE;
    BOX2m_MOVE.route_sources =  NULL;
    BOX2m_MOVE.num_route_source = 0;
    BOX2m_MOVE.m_parent = (X3dNode *)&BOX2m_SWITCH;
    BOX2m_SENSOR.m_parent = (X3dNode *)&BOX2m_MOVE;
    BOX2m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX2m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX2m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX2m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX2m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX2m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX2m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX2m_SENSOR.isActive = false;
    BOX2m_SENSOR.isOver = false;
    BOX2m_SENSOR.touchTime = 0;
    Inline_34.m_parent = (X3dNode *)&BOX2m_MOVE;
    Group5.m_parent = (X3dNode *)&Inline_34;
    Shape_185.m_parent = (X3dNode *)NULL;
    Appearance_186.m_parent = (X3dNode *)NULL;
    Material_187.m_parent = (X3dNode *)NULL;
    Material_187.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_187.diffuseColor = m_diffuseColor;
    Material_187.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_187.emissiveColor = m_emissiveColor;
    Material_187.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_187.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_187.specularColor = m_specularColor;
    Material_187.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_187.transparency = 0;
    Appearance_186.material = &Material_187;
    Appearance_186.texture = NULL;
    Appearance_186.textureTransform = NULL;
    IndexedFaceSet_30.setGlName(17);
    IndexedFaceSet_30.m_parent = (X3dNode *)NULL;
    Color5_4.m_parent = (X3dNode *)&IndexedFaceSet_30;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color5_4.color = m_color;
    Color5_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate15_9.m_parent = (X3dNode *)&IndexedFaceSet_30;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate15_9.point = m_point;
    Coordinate15_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal17.m_parent = (X3dNode *)&IndexedFaceSet_30;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal17.vector = m_vector;
    Normal17.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_30.color = &Color5_4;
    IndexedFaceSet_30.coord = &Coordinate15_9;
    IndexedFaceSet_30.normal = &Normal17;
    IndexedFaceSet_30.texCoord = NULL;
    IndexedFaceSet_30.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 1494924160, -1, 0, 0, 0, -1, 2999, 3951, 2998, -1, 0, 0, 0, -1, -1, 3955, 3953, -1, 0, 0, 0, -1, 4068, 4070, -1, -1, 0, 0, 0, -1, 4072, -1, 4071, -1, 0, 0, 0, -1, 4076, 4077, 4075, -1 };
    IndexedFaceSet_30.colorIndex = m_colorIndex;
    IndexedFaceSet_30.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_30.colorPerVertex = false;
    IndexedFaceSet_30.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_30.coordIndex = m_coordIndex;
    IndexedFaceSet_30.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_30.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet_30.normalIndex = m_normalIndex;
    IndexedFaceSet_30.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_30.normalPerVertex = true;
    IndexedFaceSet_30.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_30.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_30.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_30.set_colorIndex = NULL;
    IndexedFaceSet_30.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_30.set_coordIndex = NULL;
    IndexedFaceSet_30.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_30.set_normalIndex = NULL;
    IndexedFaceSet_30.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_30.set_texCoordIndex = NULL;
    IndexedFaceSet_30.set_texCoordIndex_length = 0;
    }
    Shape_185.appearance = &Appearance_186;
    Shape_185.geometry = &IndexedFaceSet_30;
    Shape_190.m_parent = (X3dNode *)NULL;
    Appearance_191.m_parent = (X3dNode *)NULL;
    Material_192.m_parent = (X3dNode *)NULL;
    Material_192.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_192.diffuseColor = m_diffuseColor;
    Material_192.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_192.emissiveColor = m_emissiveColor;
    Material_192.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_192.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_192.specularColor = m_specularColor;
    Material_192.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_192.transparency = 0;
    ImageTexture_193.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture_193.url = m_url;
    ImageTexture_193.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_193.repeatS = true;
    ImageTexture_193.repeatT = true;
    Appearance_191.material = &Material_192;
    Appearance_191.texture = &ImageTexture_193;
    Appearance_191.textureTransform = NULL;
    IndexedFaceSet_32.setGlName(18);
    IndexedFaceSet_32.m_parent = (X3dNode *)NULL;
    Coordinate16_9.m_parent = (X3dNode *)&IndexedFaceSet_32;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate16_9.point = m_point;
    Coordinate16_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal18.m_parent = (X3dNode *)&IndexedFaceSet_32;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal18.vector = m_vector;
    Normal18.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate8_16.m_parent = (X3dNode *)&IndexedFaceSet_32;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate8_16.point = m_point;
    TextureCoordinate8_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_32.color = NULL;
    IndexedFaceSet_32.coord = &Coordinate16_9;
    IndexedFaceSet_32.normal = &Normal18;
    IndexedFaceSet_32.texCoord = &TextureCoordinate8_16;
    IndexedFaceSet_32.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, -147544264, 0, -147544264, -1 };
    IndexedFaceSet_32.colorIndex = m_colorIndex;
    IndexedFaceSet_32.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_32.colorPerVertex = false;
    IndexedFaceSet_32.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_32.coordIndex = m_coordIndex;
    IndexedFaceSet_32.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_32.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_32.normalIndex = m_normalIndex;
    IndexedFaceSet_32.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_32.normalPerVertex = true;
    IndexedFaceSet_32.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_32.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_32.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_32.set_colorIndex = NULL;
    IndexedFaceSet_32.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_32.set_coordIndex = NULL;
    IndexedFaceSet_32.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_32.set_normalIndex = NULL;
    IndexedFaceSet_32.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_32.set_texCoordIndex = NULL;
    IndexedFaceSet_32.set_texCoordIndex_length = 0;
    }
    Shape_190.appearance = &Appearance_191;
    Shape_190.geometry = &IndexedFaceSet_32;
    {
    static X3dNode* m_children[] = { &Shape_185, &Shape_190 };
    Group5.children = m_children;
    Group5.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group5.bboxCenter = m_bboxCenter;
    Group5.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group5.bboxSize = m_bboxSize;
    Group5.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group5.addChildren = NULL;
    Group5.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group5.removeChildren = NULL;
    Group5.removeChildren_length = 0;
    }
    WorldInfo5.m_parent = (X3dNode *)&Inline_34;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo5.info = m_info;
    WorldInfo5.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo5.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group5, &WorldInfo5 };
    Inline_34.children = m_children;
    Inline_34.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_34.bboxCenter = m_bboxCenter;
    Inline_34.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_34.bboxSize = m_bboxSize;
    Inline_34.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_34.addChildren = NULL;
    Inline_34.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_34.removeChildren = NULL;
    Inline_34.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX2m_MOVE.rotation = m_rotation;
    BOX2m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX2m_MOVE.translation = m_translation;
    BOX2m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX2m_MOVE.bboxCenter = m_bboxCenter;
    BOX2m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX2m_MOVE.bboxSize = m_bboxSize;
    BOX2m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX2m_MOVE.center = m_center;
    BOX2m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX2m_MOVE.scale = m_scale;
    BOX2m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX2m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX2m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX2m_SENSOR, &Inline_34 };
    BOX2m_MOVE.children = m_children;
    BOX2m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX2m_MOVE.addChildren = NULL;
    BOX2m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX2m_MOVE.removeChildren = NULL;
    BOX2m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX2m_MOVE };
    BOX2m_SWITCH.children = m_children;
    BOX2m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX2m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX2m_HIDE.rotation = m_rotation;
    BOX2m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX2m_HIDE.translation = m_translation;
    BOX2m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX2m_HIDE.bboxCenter = m_bboxCenter;
    BOX2m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX2m_HIDE.bboxSize = m_bboxSize;
    BOX2m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX2m_HIDE.center = m_center;
    BOX2m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX2m_HIDE.scale = m_scale;
    BOX2m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX2m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX2m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX2m_SWITCH };
    BOX2m_HIDE.children = m_children;
    BOX2m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX2m_HIDE.addChildren = NULL;
    BOX2m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX2m_HIDE.removeChildren = NULL;
    BOX2m_HIDE.removeChildren_length = 0;
    }
    BOX2m_HITm_CHECK.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function initialize()\n         {\n         choice_out=0;\n         deleted_out=0;\n         }\n      function translation_in(val)  \n         {\n         if (!deleted)\n            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)\n                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)\n                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)\n               )\n               {\n               choice_out=1;\n               deleted_out=1;\n               deleted=1;\n               }\n            else\n               choice_out=0;\n         else\n            choice_out=1;\n         }\n         \n      function box_position(val)\n         {\n         box[0]=val[0];\n         box[1]=val[1];\n         box[2]=val[2];\n         }  \n      ", "c++:\n   X3dScript_BOX2_HIT_CHECK *self = (X3dScript_BOX2_HIT_CHECK *)node;\n   static bool BOX2_HIT_CHECK_init = false;\n   if (!BOX2_HIT_CHECK_init) {\n       self->choice_out = 0;\n       self->deleted_out = 0;\n       BOX2_HIT_CHECK_init = true;\n   } else if (!self->deleted) \n      {\n      self->box[0] = self->box_position[0];\n      self->box[1] = self->box_position[1];\n      self->box[2] = self->box_position[2];\n      if (\n          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && \n          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && \n          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && \n          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && \n          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && \n          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)\n         )\n         {\n         self->choice_out = 1;\n         self->deleted_out = 1;\n         self->deleted = 1;\n         }\n      else\n         self->choice_out = 0;\n   }\n   else\n      self->choice_out = 1;\n" };
    BOX2m_HITm_CHECK.url = m_url;
    BOX2m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_HITm_CHECK.directOutput = false;
    BOX2m_HITm_CHECK.mustEvaluate = false;
    BOX2m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX2m_HITm_CHECK.box = m_box;
    BOX2m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX2m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX2m_HITm_CHECK.translation_in = m_translation_in;
    BOX2m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX2m_HITm_CHECK.box_position = m_box_position;
    BOX2m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX2m_HITm_CHECK.deleted_out = 0;
    BOX2m_HITm_CHECK.choice_out = 0;
    BOX2m_MOVEm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function nextsteps()\n         {\n         step_translation[0]=(Math.random()-0.5)*0.04;\n         step_translation[1]=(Math.random()-0.5)*0.04;\n         step_translation[2]=(Math.random()-0.5)*0.04;\n         step=Math.random()*200+1;\n         }\n      function initialize()\n         {\n         translation_out[0]=Math.random()*8-4;\n         translation_out[1]=Math.random()*6-3;\n         translation_out[2]=Math.random()*-30-1;\n         nextsteps();\n         }\n      function translation_in(val)  \n         {\n         step--;\n         if (step<=0)\n            nextsteps();\n         if (   (translation_out[0]+step_translation[0]<4) \n             && (translation_out[0]+step_translation[0]>-4)) \n            translation_out[0]+=step_translation[0];\n         if (   (translation_out[1]+step_translation[1]<3) \n             && (translation_out[1]+step_translation[1]>-3)) \n             translation_out[1]+=step_translation[1];\n         if (   (translation_out[2]+step_translation[2]<-1) \n             && (translation_out[2]+step_translation[2]>-30)) \n            translation_out[2]+=step_translation[2];\n         }\n      ", "c++:\n   X3dScript_BOX2_MOVE_ON *self = (X3dScript_BOX2_MOVE_ON *)node;\n   static bool BOX2_MOVE_ON_init = false;\n   if (!BOX2_MOVE_ON_init) {\n       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;\n       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;\n       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;\n       BOX2_MOVE_ON_init = true;\n   }         \n   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   if ((self->translation_out[0] + self->step_translation[0] < 4) && \n       (self->translation_out[0] + self->step_translation[0] > -4)) \n       self->translation_out[0] += self->step_translation[0];\n   if ((self->translation_out[1] + self->step_translation[1] < 3) &&\n       (self->translation_out[1] + self->step_translation[1] > -3)) \n       self->translation_out[1] += self->step_translation[1];\n   if ((self->translation_out[2] + self->step_translation[2] < -1) && \n       (self->translation_out[2] + self->step_translation[2] > -30)) \n       self->translation_out[2] += self->step_translation[2];\n" };
    BOX2m_MOVEm_ON.url = m_url;
    BOX2m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_MOVEm_ON.directOutput = false;
    BOX2m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.step_translation = m_step_translation;
    BOX2m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX2m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.translation_in = m_translation_in;
    BOX2m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX2m_MOVEm_ON.translation_out = m_translation_out;
    BOX2m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX2m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=100;\n         }\n      else\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=0;\n         }\n      }    \n   ", "c++:\n   X3dScript_BOX2_HIDE_SCRIPT *self = (X3dScript_BOX2_HIDE_SCRIPT *)node;\n   if (self->isover_in) {             \n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 100;\n   } else {\n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 0;\n   }\n" };
    BOX2m_HIDEm_SCRIPT.url = m_url;
    BOX2m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX2m_HIDEm_SCRIPT.directOutput = false;
    BOX2m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX2m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX2m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX2m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX3m_HIDE.route_sources =  NULL;
    BOX3m_HIDE.num_route_source = 0;
    BOX3m_HIDE.m_parent = (X3dNode *)&Group_71;
    BOX3m_SWITCH.m_parent = (X3dNode *)&BOX3m_HIDE;
    BOX3m_MOVE.route_sources =  NULL;
    BOX3m_MOVE.num_route_source = 0;
    BOX3m_MOVE.m_parent = (X3dNode *)&BOX3m_SWITCH;
    BOX3m_SENSOR.m_parent = (X3dNode *)&BOX3m_MOVE;
    BOX3m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX3m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX3m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX3m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX3m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX3m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX3m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX3m_SENSOR.isActive = false;
    BOX3m_SENSOR.isOver = false;
    BOX3m_SENSOR.touchTime = 0;
    Inline_39.m_parent = (X3dNode *)&BOX3m_MOVE;
    Group6.m_parent = (X3dNode *)&Inline_39;
    Shape_196.m_parent = (X3dNode *)NULL;
    Appearance_197.m_parent = (X3dNode *)NULL;
    Material_198.m_parent = (X3dNode *)NULL;
    Material_198.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_198.diffuseColor = m_diffuseColor;
    Material_198.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_198.emissiveColor = m_emissiveColor;
    Material_198.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_198.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_198.specularColor = m_specularColor;
    Material_198.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_198.transparency = 0;
    Appearance_197.material = &Material_198;
    Appearance_197.texture = NULL;
    Appearance_197.textureTransform = NULL;
    IndexedFaceSet_35.setGlName(19);
    IndexedFaceSet_35.m_parent = (X3dNode *)NULL;
    Color6_4.m_parent = (X3dNode *)&IndexedFaceSet_35;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color6_4.color = m_color;
    Color6_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate17_9.m_parent = (X3dNode *)&IndexedFaceSet_35;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate17_9.point = m_point;
    Coordinate17_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal19.m_parent = (X3dNode *)&IndexedFaceSet_35;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal19.vector = m_vector;
    Normal19.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_35.color = &Color6_4;
    IndexedFaceSet_35.coord = &Coordinate17_9;
    IndexedFaceSet_35.normal = &Normal19;
    IndexedFaceSet_35.texCoord = NULL;
    IndexedFaceSet_35.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 1495005536, -1, 0, 0, 0, -1, 18257, 19350, 18256, -1, 0, 0, 0, -1, -1, 19354, 19352, -1, 0, 0, 0, -1, 19467, 19469, -1, -1, 0, 0, 0, -1, 19471, -1, 19470, -1, 0, 0, 0, -1, 19475, 19476, 19474, -1 };
    IndexedFaceSet_35.colorIndex = m_colorIndex;
    IndexedFaceSet_35.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_35.colorPerVertex = false;
    IndexedFaceSet_35.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_35.coordIndex = m_coordIndex;
    IndexedFaceSet_35.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_35.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet_35.normalIndex = m_normalIndex;
    IndexedFaceSet_35.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_35.normalPerVertex = true;
    IndexedFaceSet_35.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_35.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_35.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_35.set_colorIndex = NULL;
    IndexedFaceSet_35.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_35.set_coordIndex = NULL;
    IndexedFaceSet_35.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_35.set_normalIndex = NULL;
    IndexedFaceSet_35.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_35.set_texCoordIndex = NULL;
    IndexedFaceSet_35.set_texCoordIndex_length = 0;
    }
    Shape_196.appearance = &Appearance_197;
    Shape_196.geometry = &IndexedFaceSet_35;
    Shape_201.m_parent = (X3dNode *)NULL;
    Appearance_202.m_parent = (X3dNode *)NULL;
    Material_203.m_parent = (X3dNode *)NULL;
    Material_203.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_203.diffuseColor = m_diffuseColor;
    Material_203.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_203.emissiveColor = m_emissiveColor;
    Material_203.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_203.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_203.specularColor = m_specularColor;
    Material_203.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_203.transparency = 0;
    ImageTexture_204.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture_204.url = m_url;
    ImageTexture_204.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_204.repeatS = true;
    ImageTexture_204.repeatT = true;
    Appearance_202.material = &Material_203;
    Appearance_202.texture = &ImageTexture_204;
    Appearance_202.textureTransform = NULL;
    IndexedFaceSet_37.setGlName(20);
    IndexedFaceSet_37.m_parent = (X3dNode *)NULL;
    Coordinate18_9.m_parent = (X3dNode *)&IndexedFaceSet_37;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate18_9.point = m_point;
    Coordinate18_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal20.m_parent = (X3dNode *)&IndexedFaceSet_37;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal20.vector = m_vector;
    Normal20.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate9_16.m_parent = (X3dNode *)&IndexedFaceSet_37;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate9_16.point = m_point;
    TextureCoordinate9_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_37.color = NULL;
    IndexedFaceSet_37.coord = &Coordinate18_9;
    IndexedFaceSet_37.normal = &Normal20;
    IndexedFaceSet_37.texCoord = &TextureCoordinate9_16;
    IndexedFaceSet_37.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, -147544264, 0, -147544264, -1 };
    IndexedFaceSet_37.colorIndex = m_colorIndex;
    IndexedFaceSet_37.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_37.colorPerVertex = false;
    IndexedFaceSet_37.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_37.coordIndex = m_coordIndex;
    IndexedFaceSet_37.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_37.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_37.normalIndex = m_normalIndex;
    IndexedFaceSet_37.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_37.normalPerVertex = true;
    IndexedFaceSet_37.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_37.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_37.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_37.set_colorIndex = NULL;
    IndexedFaceSet_37.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_37.set_coordIndex = NULL;
    IndexedFaceSet_37.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_37.set_normalIndex = NULL;
    IndexedFaceSet_37.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_37.set_texCoordIndex = NULL;
    IndexedFaceSet_37.set_texCoordIndex_length = 0;
    }
    Shape_201.appearance = &Appearance_202;
    Shape_201.geometry = &IndexedFaceSet_37;
    {
    static X3dNode* m_children[] = { &Shape_196, &Shape_201 };
    Group6.children = m_children;
    Group6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group6.bboxCenter = m_bboxCenter;
    Group6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group6.bboxSize = m_bboxSize;
    Group6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group6.addChildren = NULL;
    Group6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group6.removeChildren = NULL;
    Group6.removeChildren_length = 0;
    }
    WorldInfo6.m_parent = (X3dNode *)&Inline_39;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo6.info = m_info;
    WorldInfo6.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo6.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group6, &WorldInfo6 };
    Inline_39.children = m_children;
    Inline_39.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_39.bboxCenter = m_bboxCenter;
    Inline_39.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_39.bboxSize = m_bboxSize;
    Inline_39.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_39.addChildren = NULL;
    Inline_39.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_39.removeChildren = NULL;
    Inline_39.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX3m_MOVE.rotation = m_rotation;
    BOX3m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX3m_MOVE.translation = m_translation;
    BOX3m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX3m_MOVE.bboxCenter = m_bboxCenter;
    BOX3m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX3m_MOVE.bboxSize = m_bboxSize;
    BOX3m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX3m_MOVE.center = m_center;
    BOX3m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX3m_MOVE.scale = m_scale;
    BOX3m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX3m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX3m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX3m_SENSOR, &Inline_39 };
    BOX3m_MOVE.children = m_children;
    BOX3m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX3m_MOVE.addChildren = NULL;
    BOX3m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX3m_MOVE.removeChildren = NULL;
    BOX3m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX3m_MOVE };
    BOX3m_SWITCH.children = m_children;
    BOX3m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX3m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX3m_HIDE.rotation = m_rotation;
    BOX3m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX3m_HIDE.translation = m_translation;
    BOX3m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX3m_HIDE.bboxCenter = m_bboxCenter;
    BOX3m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX3m_HIDE.bboxSize = m_bboxSize;
    BOX3m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX3m_HIDE.center = m_center;
    BOX3m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX3m_HIDE.scale = m_scale;
    BOX3m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX3m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX3m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX3m_SWITCH };
    BOX3m_HIDE.children = m_children;
    BOX3m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX3m_HIDE.addChildren = NULL;
    BOX3m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX3m_HIDE.removeChildren = NULL;
    BOX3m_HIDE.removeChildren_length = 0;
    }
    BOX3m_HITm_CHECK.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function initialize()\n         {\n         choice_out=0;\n         deleted_out=0;\n         }\n      function translation_in(val)  \n         {\n         if (!deleted)\n            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)\n                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)\n                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)\n               )\n               {\n               choice_out=1;\n               deleted_out=1;\n               deleted=1;\n               }\n            else\n               choice_out=0;\n         else\n            choice_out=1;\n         }\n         \n      function box_position(val)\n         {\n         box[0]=val[0];\n         box[1]=val[1];\n         box[2]=val[2];\n         }  \n      ", "c++:\n   X3dScript_BOX3_HIT_CHECK *self = (X3dScript_BOX3_HIT_CHECK *)node;\n   static bool BOX3_HIT_CHECK_init = false;\n   if (!BOX3_HIT_CHECK_init) {\n       self->choice_out = 0;\n       self->deleted_out = 9;\n       BOX3_HIT_CHECK_init = true;\n   } else if (!self->deleted) \n      {\n      self->box[0] = self->box_position[0];\n      self->box[1] = self->box_position[1];\n      self->box[2] = self->box_position[2];\n      if (\n          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && \n          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && \n          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && \n          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && \n          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && \n          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)\n         )\n         {\n         self->choice_out = 1;\n         self->deleted_out = 1;\n         self->deleted = 1;\n         }\n      else\n         self->choice_out = 0;\n   }\n   else\n      self->choice_out = 1;\n" };
    BOX3m_HITm_CHECK.url = m_url;
    BOX3m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_HITm_CHECK.directOutput = false;
    BOX3m_HITm_CHECK.mustEvaluate = false;
    BOX3m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX3m_HITm_CHECK.box = m_box;
    BOX3m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX3m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX3m_HITm_CHECK.translation_in = m_translation_in;
    BOX3m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX3m_HITm_CHECK.box_position = m_box_position;
    BOX3m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX3m_HITm_CHECK.deleted_out = 0;
    BOX3m_HITm_CHECK.choice_out = 0;
    BOX3m_MOVEm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function nextsteps()\n         {\n         step_translation[0]=(Math.random()-0.5)*0.04;\n         step_translation[1]=(Math.random()-0.5)*0.04;\n         step_translation[2]=(Math.random()-0.5)*0.04;\n         step=Math.random()*200+1;\n         }\n      function initialize()\n         {\n         translation_out[0]=Math.random()*8-4;\n         translation_out[1]=Math.random()*6-3;\n         translation_out[2]=Math.random()*-30-1;\n         nextsteps();\n         }\n      function translation_in(val)  \n         {\n         step--;\n         if (step<=0)\n            nextsteps();\n         if (   (translation_out[0]+step_translation[0]<4) \n             && (translation_out[0]+step_translation[0]>-4)) \n            translation_out[0]+=step_translation[0];\n         if (   (translation_out[1]+step_translation[1]<3) \n             && (translation_out[1]+step_translation[1]>-3)) \n             translation_out[1]+=step_translation[1];\n         if (   (translation_out[2]+step_translation[2]<-1) \n             && (translation_out[2]+step_translation[2]>-30)) \n            translation_out[2]+=step_translation[2];\n         }\n      ", "c++:\n   X3dScript_BOX3_MOVE_ON *self = (X3dScript_BOX3_MOVE_ON *)node;\n   static bool BOX3_MOVE_ON_init = false;\n   if (!BOX3_MOVE_ON_init) {\n       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;\n       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;\n       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;\n       BOX3_MOVE_ON_init = true;\n   }         \n   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   if ((self->translation_out[0] + self->step_translation[0] < 4) && \n       (self->translation_out[0] + self->step_translation[0] > -4)) \n       self->translation_out[0] += self->step_translation[0];\n   if ((self->translation_out[1] + self->step_translation[1] < 3) &&\n       (self->translation_out[1] + self->step_translation[1] > -3)) \n       self->translation_out[1] += self->step_translation[1];\n   if ((self->translation_out[2] + self->step_translation[2] < -1) && \n       (self->translation_out[2] + self->step_translation[2] > -30)) \n       self->translation_out[2] += self->step_translation[2];\n" };
    BOX3m_MOVEm_ON.url = m_url;
    BOX3m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_MOVEm_ON.directOutput = false;
    BOX3m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.step_translation = m_step_translation;
    BOX3m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX3m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.translation_in = m_translation_in;
    BOX3m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX3m_MOVEm_ON.translation_out = m_translation_out;
    BOX3m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX3m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=100;\n         }\n      else\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=0;\n         }\n      }    \n   ", "c++:\n   X3dScript_BOX3_HIDE_SCRIPT *self = (X3dScript_BOX3_HIDE_SCRIPT *)node;\n   if (self->isover_in) {             \n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 100;\n   } else {\n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 0;\n   }\n" };
    BOX3m_HIDEm_SCRIPT.url = m_url;
    BOX3m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX3m_HIDEm_SCRIPT.directOutput = false;
    BOX3m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX3m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX3m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX3m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX4m_HIDE.route_sources =  NULL;
    BOX4m_HIDE.num_route_source = 0;
    BOX4m_HIDE.m_parent = (X3dNode *)&Group_71;
    BOX4m_SWITCH.m_parent = (X3dNode *)&BOX4m_HIDE;
    BOX4m_MOVE.route_sources =  NULL;
    BOX4m_MOVE.num_route_source = 0;
    BOX4m_MOVE.m_parent = (X3dNode *)&BOX4m_SWITCH;
    BOX4m_SENSOR.m_parent = (X3dNode *)&BOX4m_MOVE;
    BOX4m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX4m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX4m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX4m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX4m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX4m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX4m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX4m_SENSOR.isActive = false;
    BOX4m_SENSOR.isOver = false;
    BOX4m_SENSOR.touchTime = 0;
    Inline_44.m_parent = (X3dNode *)&BOX4m_MOVE;
    Group7.m_parent = (X3dNode *)&Inline_44;
    Shape_207.m_parent = (X3dNode *)NULL;
    Appearance_208.m_parent = (X3dNode *)NULL;
    Material_209.m_parent = (X3dNode *)NULL;
    Material_209.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_209.diffuseColor = m_diffuseColor;
    Material_209.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_209.emissiveColor = m_emissiveColor;
    Material_209.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_209.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_209.specularColor = m_specularColor;
    Material_209.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_209.transparency = 0;
    Appearance_208.material = &Material_209;
    Appearance_208.texture = NULL;
    Appearance_208.textureTransform = NULL;
    IndexedFaceSet_40.setGlName(21);
    IndexedFaceSet_40.m_parent = (X3dNode *)NULL;
    Color7_4.m_parent = (X3dNode *)&IndexedFaceSet_40;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color7_4.color = m_color;
    Color7_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate19_9.m_parent = (X3dNode *)&IndexedFaceSet_40;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate19_9.point = m_point;
    Coordinate19_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal21.m_parent = (X3dNode *)&IndexedFaceSet_40;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal21.vector = m_vector;
    Normal21.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_40.color = &Color7_4;
    IndexedFaceSet_40.coord = &Coordinate19_9;
    IndexedFaceSet_40.normal = &Normal21;
    IndexedFaceSet_40.texCoord = NULL;
    IndexedFaceSet_40.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 1495087968, -1, 0, 0, 0, -1, 33713, 34806, 33712, -1, 0, 0, 0, -1, -1, 34810, 34808, -1, 0, 0, 0, -1, 34923, 34925, -1, -1, 0, 0, 0, -1, 34927, -1, 34926, -1, 0, 0, 0, -1, 34931, 34932, 34930, -1 };
    IndexedFaceSet_40.colorIndex = m_colorIndex;
    IndexedFaceSet_40.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_40.colorPerVertex = false;
    IndexedFaceSet_40.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_40.coordIndex = m_coordIndex;
    IndexedFaceSet_40.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_40.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet_40.normalIndex = m_normalIndex;
    IndexedFaceSet_40.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_40.normalPerVertex = true;
    IndexedFaceSet_40.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_40.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_40.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_40.set_colorIndex = NULL;
    IndexedFaceSet_40.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_40.set_coordIndex = NULL;
    IndexedFaceSet_40.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_40.set_normalIndex = NULL;
    IndexedFaceSet_40.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_40.set_texCoordIndex = NULL;
    IndexedFaceSet_40.set_texCoordIndex_length = 0;
    }
    Shape_207.appearance = &Appearance_208;
    Shape_207.geometry = &IndexedFaceSet_40;
    Shape_212.m_parent = (X3dNode *)NULL;
    Appearance_213.m_parent = (X3dNode *)NULL;
    Material_214.m_parent = (X3dNode *)NULL;
    Material_214.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_214.diffuseColor = m_diffuseColor;
    Material_214.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_214.emissiveColor = m_emissiveColor;
    Material_214.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_214.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_214.specularColor = m_specularColor;
    Material_214.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_214.transparency = 0;
    ImageTexture_215.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture_215.url = m_url;
    ImageTexture_215.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_215.repeatS = true;
    ImageTexture_215.repeatT = true;
    Appearance_213.material = &Material_214;
    Appearance_213.texture = &ImageTexture_215;
    Appearance_213.textureTransform = NULL;
    IndexedFaceSet_42.setGlName(22);
    IndexedFaceSet_42.m_parent = (X3dNode *)NULL;
    Coordinate20_9.m_parent = (X3dNode *)&IndexedFaceSet_42;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate20_9.point = m_point;
    Coordinate20_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal22.m_parent = (X3dNode *)&IndexedFaceSet_42;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal22.vector = m_vector;
    Normal22.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate10_16.m_parent = (X3dNode *)&IndexedFaceSet_42;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate10_16.point = m_point;
    TextureCoordinate10_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_42.color = NULL;
    IndexedFaceSet_42.coord = &Coordinate20_9;
    IndexedFaceSet_42.normal = &Normal22;
    IndexedFaceSet_42.texCoord = &TextureCoordinate10_16;
    IndexedFaceSet_42.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, -147544264, 0, -147544264, -1 };
    IndexedFaceSet_42.colorIndex = m_colorIndex;
    IndexedFaceSet_42.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_42.colorPerVertex = false;
    IndexedFaceSet_42.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_42.coordIndex = m_coordIndex;
    IndexedFaceSet_42.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_42.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_42.normalIndex = m_normalIndex;
    IndexedFaceSet_42.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_42.normalPerVertex = true;
    IndexedFaceSet_42.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_42.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_42.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_42.set_colorIndex = NULL;
    IndexedFaceSet_42.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_42.set_coordIndex = NULL;
    IndexedFaceSet_42.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_42.set_normalIndex = NULL;
    IndexedFaceSet_42.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_42.set_texCoordIndex = NULL;
    IndexedFaceSet_42.set_texCoordIndex_length = 0;
    }
    Shape_212.appearance = &Appearance_213;
    Shape_212.geometry = &IndexedFaceSet_42;
    {
    static X3dNode* m_children[] = { &Shape_207, &Shape_212 };
    Group7.children = m_children;
    Group7.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group7.bboxCenter = m_bboxCenter;
    Group7.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group7.bboxSize = m_bboxSize;
    Group7.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group7.addChildren = NULL;
    Group7.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group7.removeChildren = NULL;
    Group7.removeChildren_length = 0;
    }
    WorldInfo7.m_parent = (X3dNode *)&Inline_44;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo7.info = m_info;
    WorldInfo7.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo7.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group7, &WorldInfo7 };
    Inline_44.children = m_children;
    Inline_44.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_44.bboxCenter = m_bboxCenter;
    Inline_44.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_44.bboxSize = m_bboxSize;
    Inline_44.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_44.addChildren = NULL;
    Inline_44.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_44.removeChildren = NULL;
    Inline_44.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX4m_MOVE.rotation = m_rotation;
    BOX4m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX4m_MOVE.translation = m_translation;
    BOX4m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX4m_MOVE.bboxCenter = m_bboxCenter;
    BOX4m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX4m_MOVE.bboxSize = m_bboxSize;
    BOX4m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX4m_MOVE.center = m_center;
    BOX4m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX4m_MOVE.scale = m_scale;
    BOX4m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX4m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX4m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX4m_SENSOR, &Inline_44 };
    BOX4m_MOVE.children = m_children;
    BOX4m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX4m_MOVE.addChildren = NULL;
    BOX4m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX4m_MOVE.removeChildren = NULL;
    BOX4m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX4m_MOVE };
    BOX4m_SWITCH.children = m_children;
    BOX4m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX4m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX4m_HIDE.rotation = m_rotation;
    BOX4m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX4m_HIDE.translation = m_translation;
    BOX4m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX4m_HIDE.bboxCenter = m_bboxCenter;
    BOX4m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX4m_HIDE.bboxSize = m_bboxSize;
    BOX4m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX4m_HIDE.center = m_center;
    BOX4m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX4m_HIDE.scale = m_scale;
    BOX4m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX4m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX4m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX4m_SWITCH };
    BOX4m_HIDE.children = m_children;
    BOX4m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX4m_HIDE.addChildren = NULL;
    BOX4m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX4m_HIDE.removeChildren = NULL;
    BOX4m_HIDE.removeChildren_length = 0;
    }
    BOX4m_HITm_CHECK.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function initialize()\n         {\n         choice_out=0;\n         deleted_out=0;\n         }\n      function translation_in(val)  \n         {\n         if (!deleted)\n            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)\n                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)\n                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)\n               )\n               {\n               choice_out=1;\n               deleted_out=1;\n               deleted=1;\n               }\n            else\n               choice_out=0;\n         else\n            choice_out=1;\n         }\n         \n      function box_position(val)\n         {\n         box[0]=val[0];\n         box[1]=val[1];\n         box[2]=val[2];\n         }  \n      ", "c++:\n   X3dScript_BOX4_HIT_CHECK *self = (X3dScript_BOX4_HIT_CHECK *)node;\n   static bool BOX4_HIT_CHECK_init = false;\n   if (!BOX4_HIT_CHECK_init) {\n       self->choice_out = 0;\n       self->deleted_out = 0;\n       BOX4_HIT_CHECK_init = true;\n   } else if (!self->deleted) \n      {\n      self->box[0] = self->box_position[0];\n      self->box[1] = self->box_position[1];\n      self->box[2] = self->box_position[2];\n      if (\n          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && \n          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && \n          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && \n          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && \n          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && \n          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)\n         )\n         {\n         self->choice_out = 1;\n         self->deleted_out = 1;\n         self->deleted = 1;\n         }\n      else\n         self->choice_out = 0;\n   }\n   else\n      self->choice_out = 1;\n" };
    BOX4m_HITm_CHECK.url = m_url;
    BOX4m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_HITm_CHECK.directOutput = false;
    BOX4m_HITm_CHECK.mustEvaluate = false;
    BOX4m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX4m_HITm_CHECK.box = m_box;
    BOX4m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX4m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX4m_HITm_CHECK.translation_in = m_translation_in;
    BOX4m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX4m_HITm_CHECK.box_position = m_box_position;
    BOX4m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX4m_HITm_CHECK.deleted_out = 0;
    BOX4m_HITm_CHECK.choice_out = 0;
    BOX4m_MOVEm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function nextsteps()\n         {\n         step_translation[0]=(Math.random()-0.5)*0.04;\n         step_translation[1]=(Math.random()-0.5)*0.04;\n         step_translation[2]=(Math.random()-0.5)*0.04;\n         step=Math.random()*200+1;\n         }\n      function initialize()\n         {\n         translation_out[0]=Math.random()*8-4;\n         translation_out[1]=Math.random()*6-3;\n         translation_out[2]=Math.random()*-30-1;\n         nextsteps();\n         }\n      function translation_in(val)  \n         {\n         step--;\n         if (step<=0)\n            nextsteps();\n         if (   (translation_out[0]+step_translation[0]<4) \n             && (translation_out[0]+step_translation[0]>-4)) \n            translation_out[0]+=step_translation[0];\n         if (   (translation_out[1]+step_translation[1]<3) \n             && (translation_out[1]+step_translation[1]>-3)) \n             translation_out[1]+=step_translation[1];\n         if (   (translation_out[2]+step_translation[2]<-1) \n             && (translation_out[2]+step_translation[2]>-30)) \n            translation_out[2]+=step_translation[2];\n         }\n      ", "c++:\n   X3dScript_BOX4_MOVE_ON *self = (X3dScript_BOX4_MOVE_ON *)node;\n   static bool BOX4_MOVE_ON_init = false;\n   if (!BOX4_MOVE_ON_init) {\n       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;\n       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;\n       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;\n       BOX4_MOVE_ON_init = true;\n   }         \n   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   if ((self->translation_out[0] + self->step_translation[0] < 4) && \n       (self->translation_out[0] + self->step_translation[0] > -4)) \n       self->translation_out[0] += self->step_translation[0];\n   if ((self->translation_out[1] + self->step_translation[1] < 3) &&\n       (self->translation_out[1] + self->step_translation[1] > -3)) \n       self->translation_out[1] += self->step_translation[1];\n   if ((self->translation_out[2] + self->step_translation[2] < -1) && \n       (self->translation_out[2] + self->step_translation[2] > -30)) \n       self->translation_out[2] += self->step_translation[2];\n" };
    BOX4m_MOVEm_ON.url = m_url;
    BOX4m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_MOVEm_ON.directOutput = false;
    BOX4m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.step_translation = m_step_translation;
    BOX4m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX4m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.translation_in = m_translation_in;
    BOX4m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX4m_MOVEm_ON.translation_out = m_translation_out;
    BOX4m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX4m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=100;\n         }\n      else\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=0;\n         }\n      }    \n   ", "c++:\n   X3dScript_BOX4_HIDE_SCRIPT *self = (X3dScript_BOX4_HIDE_SCRIPT *)node;\n   if (self->isover_in) {             \n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 100;\n   } else {\n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 0;\n   }\n" };
    BOX4m_HIDEm_SCRIPT.url = m_url;
    BOX4m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX4m_HIDEm_SCRIPT.directOutput = false;
    BOX4m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX4m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX4m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX4m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    BOX5m_HIDE.route_sources =  NULL;
    BOX5m_HIDE.num_route_source = 0;
    BOX5m_HIDE.m_parent = (X3dNode *)&Group_71;
    BOX5m_SWITCH.m_parent = (X3dNode *)&BOX5m_HIDE;
    BOX5m_MOVE.route_sources =  NULL;
    BOX5m_MOVE.num_route_source = 0;
    BOX5m_MOVE.m_parent = (X3dNode *)&BOX5m_SWITCH;
    BOX5m_SENSOR.m_parent = (X3dNode *)&BOX5m_MOVE;
    BOX5m_SENSOR.enabled = true;
    {
    static float m_hitNormal_changed[] = { 0, 0, 0 };
    BOX5m_SENSOR.hitNormal_changed = m_hitNormal_changed;
    BOX5m_SENSOR.hitNormal_changed_length = sizeof(m_hitNormal_changed) / sizeof(float);
    }
    {
    static float m_hitPoint_changed[] = { 0, 0, 0 };
    BOX5m_SENSOR.hitPoint_changed = m_hitPoint_changed;
    BOX5m_SENSOR.hitPoint_changed_length = sizeof(m_hitPoint_changed) / sizeof(float);
    }
    {
    static float m_hitTexCoord_changed[2] = { 0, 0 };
    BOX5m_SENSOR.hitTexCoord_changed = m_hitTexCoord_changed;
    BOX5m_SENSOR.hitTexCoord_changed_length = sizeof(m_hitTexCoord_changed) / sizeof(float);
    }
    BOX5m_SENSOR.isActive = false;
    BOX5m_SENSOR.isOver = false;
    BOX5m_SENSOR.touchTime = 0;
    Inline_49.m_parent = (X3dNode *)&BOX5m_MOVE;
    Group8.m_parent = (X3dNode *)&Inline_49;
    Shape_218.m_parent = (X3dNode *)NULL;
    Appearance_219.m_parent = (X3dNode *)NULL;
    Material_220.m_parent = (X3dNode *)NULL;
    Material_220.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_220.diffuseColor = m_diffuseColor;
    Material_220.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_220.emissiveColor = m_emissiveColor;
    Material_220.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_220.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_220.specularColor = m_specularColor;
    Material_220.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_220.transparency = 0;
    Appearance_219.material = &Material_220;
    Appearance_219.texture = NULL;
    Appearance_219.textureTransform = NULL;
    IndexedFaceSet_45.setGlName(23);
    IndexedFaceSet_45.m_parent = (X3dNode *)NULL;
    Color8_4.m_parent = (X3dNode *)&IndexedFaceSet_45;
    {
    static float m_color[] = { 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000, 0.240000, 0.210000, 0.060000 };
    Color8_4.color = m_color;
    Color8_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate21_9.m_parent = (X3dNode *)&IndexedFaceSet_45;
    {
    static float m_point[] = { -0.500000, -0.500000, -0.500000, -0.500000, 0.500000, -0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, -0.500000, -0.500000, 0.400000, -0.500000, 0.500000, 0.400000, 0.500000, 0.500000, 0.400000, 0.500000, -0.500000, 0.400000 };
    Coordinate21_9.point = m_point;
    Coordinate21_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal23.m_parent = (X3dNode *)&IndexedFaceSet_45;
    {
    static float m_vector[] = { 0, 0, -1, -1, 0, 0, 0, -1, 0, 0, 0, -1, -1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, 1, 0, 0, 0, -1, 1, 0, 0, 0, -1, 0, 1, 0, 0, 0, 0, 1, 0, 1, 0, 1, 0, 0, 0, 0, 1, 0, -1, 0, 0, 0, 1, -1, 0, 0, 0, 1, 0, 0, 0, 1, -1, 0, 0, 0, -1, 0 };
    Normal23.vector = m_vector;
    Normal23.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_45.color = &Color8_4;
    IndexedFaceSet_45.coord = &Coordinate21_9;
    IndexedFaceSet_45.normal = &Normal23;
    IndexedFaceSet_45.texCoord = NULL;
    IndexedFaceSet_45.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 1495646016, -1, 0, 0, 0, -1, 11807, 609, 11806, -1, 0, 0, 0, -1, -1, 613, 611, -1, 0, 0, 0, -1, 726, 728, -1, -1, 0, 0, 0, -1, 730, -1, 729, -1, 0, 0, 0, -1, 734, 735, 733, -1 };
    IndexedFaceSet_45.colorIndex = m_colorIndex;
    IndexedFaceSet_45.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_45.colorPerVertex = false;
    IndexedFaceSet_45.convex = true;
    {
    static int m_coordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_45.coordIndex = m_coordIndex;
    IndexedFaceSet_45.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_45.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 3, 6, 9, -1, 3, 9, 0, -1, 15, 10, 7, -1, 15, 7, 12, -1, 13, 18, 21, -1, 13, 21, 16, -1, 19, 4, 1, -1, 19, 1, 22, -1, 20, 14, 8, -1, 20, 8, 5, -1, 2, 11, 17, -1, 2, 17, 23, -1 };
    IndexedFaceSet_45.normalIndex = m_normalIndex;
    IndexedFaceSet_45.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_45.normalPerVertex = true;
    IndexedFaceSet_45.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1, 7, 3, 2, -1, 7, 2, 6, -1, 6, 5, 4, -1, 6, 4, 7, -1, 5, 1, 0, -1, 5, 0, 4, -1, 5, 6, 2, -1, 5, 2, 1, -1, 0, 3, 7, -1, 0, 7, 4, -1 };
    IndexedFaceSet_45.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_45.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_45.set_colorIndex = NULL;
    IndexedFaceSet_45.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_45.set_coordIndex = NULL;
    IndexedFaceSet_45.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_45.set_normalIndex = NULL;
    IndexedFaceSet_45.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_45.set_texCoordIndex = NULL;
    IndexedFaceSet_45.set_texCoordIndex_length = 0;
    }
    Shape_218.appearance = &Appearance_219;
    Shape_218.geometry = &IndexedFaceSet_45;
    Shape_223.m_parent = (X3dNode *)NULL;
    Appearance_224.m_parent = (X3dNode *)NULL;
    Material_225.m_parent = (X3dNode *)NULL;
    Material_225.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_225.diffuseColor = m_diffuseColor;
    Material_225.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_225.emissiveColor = m_emissiveColor;
    Material_225.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_225.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_225.specularColor = m_specularColor;
    Material_225.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_225.transparency = 0;
    ImageTexture_226.m_parent = (X3dNode *)NULL;
    {
    static const char* m_url[] = { "bill.jpg" };
    ImageTexture_226.url = m_url;
    ImageTexture_226.url_length = sizeof(m_url) / sizeof(const char*);
    }
    ImageTexture_226.repeatS = true;
    ImageTexture_226.repeatT = true;
    Appearance_224.material = &Material_225;
    Appearance_224.texture = &ImageTexture_226;
    Appearance_224.textureTransform = NULL;
    IndexedFaceSet_47.setGlName(24);
    IndexedFaceSet_47.m_parent = (X3dNode *)NULL;
    Coordinate22_9.m_parent = (X3dNode *)&IndexedFaceSet_47;
    {
    static float m_point[] = { -0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, -0.500000, 0.500000, -0.500000, -0.500000, 0.500000 };
    Coordinate22_9.point = m_point;
    Coordinate22_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal24.m_parent = (X3dNode *)&IndexedFaceSet_47;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal24.vector = m_vector;
    Normal24.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate11_16.m_parent = (X3dNode *)&IndexedFaceSet_47;
    {
    static float m_point[] = { 0, 0, 1, 0, 1, 1, 0, 1 };
    TextureCoordinate11_16.point = m_point;
    TextureCoordinate11_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_47.color = NULL;
    IndexedFaceSet_47.coord = &Coordinate22_9;
    IndexedFaceSet_47.normal = &Normal24;
    IndexedFaceSet_47.texCoord = &TextureCoordinate11_16;
    IndexedFaceSet_47.ccw = true;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, -147544264, 0, -147544264, -1 };
    IndexedFaceSet_47.colorIndex = m_colorIndex;
    IndexedFaceSet_47.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_47.colorPerVertex = false;
    IndexedFaceSet_47.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 2, 0, 3, -1 };
    IndexedFaceSet_47.coordIndex = m_coordIndex;
    IndexedFaceSet_47.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_47.creaseAngle = 0.500000;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_47.normalIndex = m_normalIndex;
    IndexedFaceSet_47.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_47.normalPerVertex = true;
    IndexedFaceSet_47.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_47.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_47.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_47.set_colorIndex = NULL;
    IndexedFaceSet_47.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_47.set_coordIndex = NULL;
    IndexedFaceSet_47.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_47.set_normalIndex = NULL;
    IndexedFaceSet_47.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_47.set_texCoordIndex = NULL;
    IndexedFaceSet_47.set_texCoordIndex_length = 0;
    }
    Shape_223.appearance = &Appearance_224;
    Shape_223.geometry = &IndexedFaceSet_47;
    {
    static X3dNode* m_children[] = { &Shape_218, &Shape_223 };
    Group8.children = m_children;
    Group8.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group8.bboxCenter = m_bboxCenter;
    Group8.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group8.bboxSize = m_bboxSize;
    Group8.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group8.addChildren = NULL;
    Group8.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group8.removeChildren = NULL;
    Group8.removeChildren_length = 0;
    }
    WorldInfo8.m_parent = (X3dNode *)&Inline_49;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "bill.jpg is a modification of the program xbill by Brian Wellington", "and Matias Duarte (xbill is freeware)" };
    WorldInfo8.info = m_info;
    WorldInfo8.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo8.title = "Tux warbird";
    {
    static X3dNode* m_children[] = { &Group8, &WorldInfo8 };
    Inline_49.children = m_children;
    Inline_49.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_49.bboxCenter = m_bboxCenter;
    Inline_49.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_49.bboxSize = m_bboxSize;
    Inline_49.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_49.addChildren = NULL;
    Inline_49.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_49.removeChildren = NULL;
    Inline_49.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX5m_MOVE.rotation = m_rotation;
    BOX5m_MOVE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX5m_MOVE.translation = m_translation;
    BOX5m_MOVE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX5m_MOVE.bboxCenter = m_bboxCenter;
    BOX5m_MOVE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX5m_MOVE.bboxSize = m_bboxSize;
    BOX5m_MOVE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX5m_MOVE.center = m_center;
    BOX5m_MOVE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1.500000, 1.500000, 1.500000 };
    BOX5m_MOVE.scale = m_scale;
    BOX5m_MOVE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX5m_MOVE.scaleOrientation = m_scaleOrientation;
    BOX5m_MOVE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX5m_SENSOR, &Inline_49 };
    BOX5m_MOVE.children = m_children;
    BOX5m_MOVE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX5m_MOVE.addChildren = NULL;
    BOX5m_MOVE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX5m_MOVE.removeChildren = NULL;
    BOX5m_MOVE.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &BOX5m_MOVE };
    BOX5m_SWITCH.children = m_children;
    BOX5m_SWITCH.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    BOX5m_SWITCH.whichChoice = -1;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    BOX5m_HIDE.rotation = m_rotation;
    BOX5m_HIDE.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    BOX5m_HIDE.translation = m_translation;
    BOX5m_HIDE.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    BOX5m_HIDE.bboxCenter = m_bboxCenter;
    BOX5m_HIDE.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    BOX5m_HIDE.bboxSize = m_bboxSize;
    BOX5m_HIDE.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    BOX5m_HIDE.center = m_center;
    BOX5m_HIDE.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    BOX5m_HIDE.scale = m_scale;
    BOX5m_HIDE.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    BOX5m_HIDE.scaleOrientation = m_scaleOrientation;
    BOX5m_HIDE.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX5m_SWITCH };
    BOX5m_HIDE.children = m_children;
    BOX5m_HIDE.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    BOX5m_HIDE.addChildren = NULL;
    BOX5m_HIDE.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    BOX5m_HIDE.removeChildren = NULL;
    BOX5m_HIDE.removeChildren_length = 0;
    }
    BOX5m_HITm_CHECK.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function initialize()\n         {\n         choice_out=0;\n         deleted_out=0;\n         }\n      function translation_in(val)  \n         {\n         if (!deleted)\n            if (   (val[0]>=box[0]-boxsize/2) && (val[0]<=box[0]+boxsize/2)\n                && (val[1]>=box[1]-boxsize/2) && (val[1]<=box[1]+boxsize/2)\n                && (val[2]>=box[2]-boxsize/2) && (val[2]<=box[2]+boxsize/2)\n               )\n               {\n               choice_out=1;\n               deleted_out=1;\n               deleted=1;\n               }\n            else\n               choice_out=0;\n         else\n            choice_out=1;\n         }\n         \n      function box_position(val)\n         {\n         box[0]=val[0];\n         box[1]=val[1];\n         box[2]=val[2];\n         }  \n      ", "c++:\n   X3dScript_BOX5_HIT_CHECK *self = (X3dScript_BOX5_HIT_CHECK *)node;\n   static bool BOX5_HIT_CHECK_init = false;\n   if (!BOX5_HIT_CHECK_init) {\n       self->choice_out = 0;\n       self->deleted_out = 0;\n       BOX5_HIT_CHECK_init = true;\n   } else if (!self->deleted) \n      {\n      self->box[0] = self->box_position[0];\n      self->box[1] = self->box_position[1];\n      self->box[2] = self->box_position[2];\n      if (\n          (self->translation_in[0] >= self->box[0] - self->boxsize / 2) && \n          (self->translation_in[0] <= self->box[0] + self->boxsize / 2) && \n          (self->translation_in[1] >= self->box[1] - self->boxsize / 2) && \n          (self->translation_in[1] <= self->box[1] + self->boxsize / 2) && \n          (self->translation_in[2] >= self->box[2] - self->boxsize / 2) && \n          (self->translation_in[2] <= self->box[2] + self->boxsize / 2)\n         )\n         {\n         self->choice_out = 1;\n         self->deleted_out = 1;\n         self->deleted = 1;\n         }\n      else\n         self->choice_out = 0;\n   }\n   else\n      self->choice_out = 1;\n" };
    BOX5m_HITm_CHECK.url = m_url;
    BOX5m_HITm_CHECK.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_HITm_CHECK.directOutput = false;
    BOX5m_HITm_CHECK.mustEvaluate = false;
    BOX5m_HITm_CHECK.deleted = 0;
    {
    static float m_box[] = { 1e4, 1e4, 1e4 };
    BOX5m_HITm_CHECK.box = m_box;
    BOX5m_HITm_CHECK.box_length = sizeof(m_box) / sizeof(float);
    }
    BOX5m_HITm_CHECK.boxsize = 1.500000;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX5m_HITm_CHECK.translation_in = m_translation_in;
    BOX5m_HITm_CHECK.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_box_position[] = { 0, 0, 0 };
    BOX5m_HITm_CHECK.box_position = m_box_position;
    BOX5m_HITm_CHECK.box_position_length = sizeof(m_box_position) / sizeof(float);
    }
    BOX5m_HITm_CHECK.deleted_out = 0;
    BOX5m_HITm_CHECK.choice_out = 0;
    BOX5m_MOVEm_ON.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n      function nextsteps()\n         {\n         step_translation[0]=(Math.random()-0.5)*0.04;\n         step_translation[1]=(Math.random()-0.5)*0.04;\n         step_translation[2]=(Math.random()-0.5)*0.04;\n         step=Math.random()*200+1;\n         }\n      function initialize()\n         {\n         translation_out[0]=Math.random()*8-4;\n         translation_out[1]=Math.random()*6-3;\n         translation_out[2]=Math.random()*-30-1;\n         nextsteps();\n         }\n      function translation_in(val)  \n         {\n         step--;\n         if (step<=0)\n            nextsteps();\n         if (   (translation_out[0]+step_translation[0]<4) \n             && (translation_out[0]+step_translation[0]>-4)) \n            translation_out[0]+=step_translation[0];\n         if (   (translation_out[1]+step_translation[1]<3) \n             && (translation_out[1]+step_translation[1]>-3)) \n             translation_out[1]+=step_translation[1];\n         if (   (translation_out[2]+step_translation[2]<-1) \n             && (translation_out[2]+step_translation[2]>-30)) \n            translation_out[2]+=step_translation[2];\n         }\n      ", "c++:\n   X3dScript_BOX5_MOVE_ON *self = (X3dScript_BOX5_MOVE_ON *)node;\n   static bool BOX5_MOVE_ON_init = false;\n   if (!BOX5_MOVE_ON_init) {\n       self->translation_out[0] = ((float)rand()) / RAND_MAX * 8 - 4;\n       self->translation_out[1] = ((float)rand()) / RAND_MAX * 6 - 3;\n       self->translation_out[2] = ((float)rand()) / RAND_MAX * -30 - 1;\n       BOX5_MOVE_ON_init = true;\n   }         \n   self->step_translation[0] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[1] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   self->step_translation[2] = (((float)rand()) / RAND_MAX - 0.5) * 0.04;\n   if ((self->translation_out[0] + self->step_translation[0] < 4) && \n       (self->translation_out[0] + self->step_translation[0] > -4)) \n       self->translation_out[0] += self->step_translation[0];\n   if ((self->translation_out[1] + self->step_translation[1] < 3) &&\n       (self->translation_out[1] + self->step_translation[1] > -3)) \n       self->translation_out[1] += self->step_translation[1];\n   if ((self->translation_out[2] + self->step_translation[2] < -1) && \n       (self->translation_out[2] + self->step_translation[2] > -30)) \n       self->translation_out[2] += self->step_translation[2];\n" };
    BOX5m_MOVEm_ON.url = m_url;
    BOX5m_MOVEm_ON.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_MOVEm_ON.directOutput = false;
    BOX5m_MOVEm_ON.mustEvaluate = false;
    {
    static float m_step_translation[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.step_translation = m_step_translation;
    BOX5m_MOVEm_ON.step_translation_length = sizeof(m_step_translation) / sizeof(float);
    }
    BOX5m_MOVEm_ON.step = 0;
    {
    static float m_translation_in[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.translation_in = m_translation_in;
    BOX5m_MOVEm_ON.translation_in_length = sizeof(m_translation_in) / sizeof(float);
    }
    {
    static float m_translation_out[] = { 0, 0, 0 };
    BOX5m_MOVEm_ON.translation_out = m_translation_out;
    BOX5m_MOVEm_ON.translation_out_length = sizeof(m_translation_out) / sizeof(float);
    }
    BOX5m_HIDEm_SCRIPT.m_parent = (X3dNode *)&Group_71;
    {
    static const char* m_url[] = { "javascript:\n   function isover_in(val)            \n      {            \n      if (val)\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=100;\n         }\n      else\n         {\n         hide_out[0]=0;\n         hide_out[1]=0;\n         hide_out[2]=0;\n         }\n      }    \n   ", "c++:\n   X3dScript_BOX5_HIDE_SCRIPT *self = (X3dScript_BOX5_HIDE_SCRIPT *)node;\n   if (self->isover_in) {             \n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 100;\n   } else {\n       self->hide_out[0] = 0;\n       self->hide_out[1] = 0;\n       self->hide_out[2] = 0;\n   }\n" };
    BOX5m_HIDEm_SCRIPT.url = m_url;
    BOX5m_HIDEm_SCRIPT.url_length = sizeof(m_url) / sizeof(const char*);
    }
    BOX5m_HIDEm_SCRIPT.directOutput = false;
    BOX5m_HIDEm_SCRIPT.mustEvaluate = false;
    BOX5m_HIDEm_SCRIPT.isover_in = false;
    {
    static float m_hide_out[] = { 0, 0, 0 };
    BOX5m_HIDEm_SCRIPT.hide_out = m_hide_out;
    BOX5m_HIDEm_SCRIPT.hide_out_length = sizeof(m_hide_out) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &BOX1m_HIDE, &BOX1m_HITm_CHECK, &BOX1m_MOVEm_ON, &BOX1m_HIDEm_SCRIPT, &STEER, &WALLm_SENSOR, &WALLm_SENSOR2, &ROTm_TRANS1, &ROTm_TRANS2, &Background_72, &WALLm_FIRE, &THING, &BULLETm_SWITCH, &TIMER, &SOUND, &NavigationInfo_74, &ROLLm_TURN, &UPm_DOWNm_TURN, &FLYm_FORWARD, &FLYm_ON, &THINGm_SCRIPT, &Viewpoint_75, &Viewpoint_76, &WorldInfo_77, &Transform_78, &Group_82, &Shape_87, &BOX2m_HIDE, &BOX2m_HITm_CHECK, &BOX2m_MOVEm_ON, &BOX2m_HIDEm_SCRIPT, &BOX3m_HIDE, &BOX3m_HITm_CHECK, &BOX3m_MOVEm_ON, &BOX3m_HIDEm_SCRIPT, &BOX4m_HIDE, &BOX4m_HITm_CHECK, &BOX4m_MOVEm_ON, &BOX4m_HIDEm_SCRIPT, &BOX5m_HIDE, &BOX5m_HITm_CHECK, &BOX5m_MOVEm_ON, &BOX5m_HIDEm_SCRIPT };
    Group_71.children = m_children;
    Group_71.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_71.bboxCenter = m_bboxCenter;
    Group_71.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_71.bboxSize = m_bboxSize;
    Group_71.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_71.addChildren = NULL;
    Group_71.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_71.removeChildren = NULL;
    Group_71.removeChildren_length = 0;
    }
    Transform_91.route_sources =  NULL;
    Transform_91.num_route_source = 0;
    Transform_91.m_parent = (X3dNode *)&Switch2;
    Inline_70.m_parent = (X3dNode *)&Transform_91;
    Background1.m_parent = (X3dNode *)&Inline_70;
    {
    static float m_groundAngle[1];
    Background1.groundAngle = NULL;
    Background1.groundAngle_length = 0;
    }
    {
    static float m_groundColor[1];
    Background1.groundColor = NULL;
    Background1.groundColor_length = 0;
    }
    {
    static const char* m_backUrl[1];
    Background1.backUrl = NULL;
    Background1.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background1.rightUrl = NULL;
    Background1.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background1.frontUrl = NULL;
    Background1.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background1.leftUrl = NULL;
    Background1.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background1.topUrl = NULL;
    Background1.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background1.bottomUrl = NULL;
    Background1.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background1.skyAngle = NULL;
    Background1.skyAngle_length = 0;
    }
    Background1.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background1.skyColor = m_skyColor;
    Background1.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background1.set_bind = false;
    Background1.isBound = false;
    Anchor1.m_parent = (X3dNode *)&Inline_70;
    Billboard_229.m_parent = (X3dNode *)NULL;
    Shape_230.m_parent = (X3dNode *)NULL;
    Appearance_231.m_parent = (X3dNode *)NULL;
    Material_232.m_parent = (X3dNode *)NULL;
    Material_232.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0, 0, 1 };
    Material_232.diffuseColor = m_diffuseColor;
    Material_232.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_232.emissiveColor = m_emissiveColor;
    Material_232.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_232.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_232.specularColor = m_specularColor;
    Material_232.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_232.transparency = 0;
    Appearance_231.material = &Material_232;
    Appearance_231.texture = NULL;
    Appearance_231.textureTransform = NULL;
    IndexedFaceSet_50.setGlName(25);
    IndexedFaceSet_50.m_parent = (X3dNode *)NULL;
    COORD_8.m_parent = (X3dNode *)&IndexedFaceSet_50;
    {
    static float m_point[] = { -600, -600, -35, 600, -600, -35, -600, 600, -35, 600, 600, -35 };
    COORD_8.point = m_point;
    COORD_8.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal25.m_parent = (X3dNode *)&IndexedFaceSet_50;
    {
    static float m_vector[] = { 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal25.vector = m_vector;
    Normal25.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_50.color = NULL;
    IndexedFaceSet_50.coord = &COORD_8;
    IndexedFaceSet_50.normal = &Normal25;
    IndexedFaceSet_50.texCoord = NULL;
    IndexedFaceSet_50.ccw = true;
    {
    static int m_colorIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_50.colorIndex = m_colorIndex;
    IndexedFaceSet_50.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_50.colorPerVertex = true;
    IndexedFaceSet_50.convex = true;
    {
    static int m_coordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_50.coordIndex = m_coordIndex;
    IndexedFaceSet_50.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_50.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 1, 2, 3, -1, 1, 3, 0, -1 };
    IndexedFaceSet_50.normalIndex = m_normalIndex;
    IndexedFaceSet_50.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_50.normalPerVertex = true;
    IndexedFaceSet_50.solid = true;
    {
    static int m_texCoordIndex[] = { 1, 3, 2, -1, 1, 2, 0, -1 };
    IndexedFaceSet_50.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_50.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_50.set_colorIndex = NULL;
    IndexedFaceSet_50.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_50.set_coordIndex = NULL;
    IndexedFaceSet_50.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_50.set_normalIndex = NULL;
    IndexedFaceSet_50.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_50.set_texCoordIndex = NULL;
    IndexedFaceSet_50.set_texCoordIndex_length = 0;
    }
    Shape_230.appearance = &Appearance_231;
    Shape_230.geometry = &IndexedFaceSet_50;
    {
    static float m_axisOfRotation[] = { 0, 1, 0 };
    Billboard_229.axisOfRotation = m_axisOfRotation;
    Billboard_229.axisOfRotation_length = sizeof(m_axisOfRotation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_230 };
    Billboard_229.children = m_children;
    Billboard_229.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Billboard_229.bboxCenter = m_bboxCenter;
    Billboard_229.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Billboard_229.bboxSize = m_bboxSize;
    Billboard_229.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Billboard_229.addChildren = NULL;
    Billboard_229.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Billboard_229.removeChildren = NULL;
    Billboard_229.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Billboard_229 };
    Anchor1.children = m_children;
    Anchor1.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Anchor1.bboxCenter = m_bboxCenter;
    Anchor1.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Anchor1.bboxSize = m_bboxSize;
    Anchor1.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    Anchor1.description = "restart game";
    {
    static const char* m_parameter[1];
    Anchor1.parameter = NULL;
    Anchor1.parameter_length = 0;
    }
    {
    static const char* m_url[] = { "warbird.wrl" };
    Anchor1.url = m_url;
    Anchor1.url_length = sizeof(m_url) / sizeof(const char*);
    }
    {
    static X3dNode* m_addChildren[1];
    Anchor1.addChildren = NULL;
    Anchor1.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Anchor1.removeChildren = NULL;
    Anchor1.removeChildren_length = 0;
    }
    Anchor2.m_parent = (X3dNode *)&Inline_70;
    SCHRIFTm_6.route_sources =  NULL;
    SCHRIFTm_6.num_route_source = 0;
    SCHRIFTm_6.m_parent = (X3dNode *)NULL;
    Transform_233.route_sources =  NULL;
    Transform_233.num_route_source = 0;
    Transform_233.m_parent = (X3dNode *)NULL;
    Inline_68.m_parent = (X3dNode *)NULL;
    Transform4.route_sources =  NULL;
    Transform4.num_route_source = 0;
    Transform4.m_parent = (X3dNode *)&Inline_68;
    Inline_56.m_parent = (X3dNode *)NULL;
    Group9.m_parent = (X3dNode *)&Inline_56;
    Transform_234.route_sources =  NULL;
    Transform_234.num_route_source = 0;
    Transform_234.m_parent = (X3dNode *)NULL;
    Shape_235.m_parent = (X3dNode *)NULL;
    Appearance_236.m_parent = (X3dNode *)NULL;
    A0m_3.m_parent = (X3dNode *)NULL;
    A0m_3.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_3.diffuseColor = m_diffuseColor;
    A0m_3.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_3.emissiveColor = m_emissiveColor;
    A0m_3.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_3.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_3.specularColor = m_specularColor;
    A0m_3.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_3.transparency = 0;
    Appearance_236.material = &A0m_3;
    Appearance_236.texture = NULL;
    Appearance_236.textureTransform = NULL;
    IndexedFaceSet_52.setGlName(26);
    IndexedFaceSet_52.m_parent = (X3dNode *)NULL;
    Color9_4.m_parent = (X3dNode *)&IndexedFaceSet_52;
    {
    static float m_color[] = { 1, 0.793614, 0.087319, 1, 0.638174, 0.077397, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.793614, 0.087319, 1, 0.754465, 0.084820, 1, 0.754465, 0.084820, 1, 0.775771, 0.086180, 1, 0.756482, 0.084949, 1, 0.756482, 0.084949, 1, 0.777047, 0.086262 };
    Color9_4.color = m_color;
    Color9_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate23_9.m_parent = (X3dNode *)&IndexedFaceSet_52;
    {
    static float m_point[] = { 0.150260, 0.330511, 1.553120, 0.256526, 0.628258, 1.205410, 0.043506, 0.580147, 1.404830, 0.041913, 0.281121, 1.582420, -0.098818, 0.305979, 1.539120, -0.172704, 0.627974, 1.205410, -0.237476, 0.530979, 1.149410, -0.066433, 0.330228, 1.553120, 0.043506, 0.580146, 1.404830, 0.041913, 0.257624, 1.511120, 0.041913, 0.257624, 1.511120, 0.150260, 0.282014, 1.525120, 0.321306, 0.531263, 1.149410, 0.041913, 0.281121, 1.582420, -0.066433, 0.281730, 1.525120, 0.182646, 0.306263, 1.539120, 0.090713, 0.547383, 1.392800, 0.248053, 0.604517, 1.233140, 0.248850, 0.594803, 1.240080, 0.054951, 0.553382, 1.420730, 0.026772, 0.542105, 1.427400, -0.006386, 0.549079, 1.391450, -0.160300, 0.593221, 1.246000, -0.165083, 0.594759, 1.239830, 0.231697, 0.398694, 1.194280, 0.202474, 0.389779, 1.198610, 0.041913, 0.328005, 1.091790, -0.121436, 0.391855, 1.193190, 0.041913, 0.328657, 1.087900, -0.146267, 0.396108, 1.200800 };
    Coordinate23_9.point = m_point;
    Coordinate23_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal26_5.m_parent = (X3dNode *)&IndexedFaceSet_52;
    {
    static float m_vector[] = { 0.240520, -0.178855, 0.954024, 0.241302, -0.186810, 0.952300, 0.240520, -0.178855, 0.954024, 0.240520, -0.178855, 0.954024, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.383855, 0.446977, 0.808002, 0.293132, 0.577824, 0.761704, 0.293132, 0.577824, 0.761704, -0.000257, 0.510630, 0.859801, -0.540597, 0.550255, 0.636376, -0.540597, 0.550255, 0.636376, -0.627971, 0.485889, 0.607918, -0.741730, 0.393026, 0.543477, -0.741730, 0.393026, 0.543477, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.655967, -0.167242, 0.736029, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, -0.380785, 0.412824, 0.827393, 0.273372, -0.961689, -0.020544, 0.273372, -0.961689, -0.020544, 0.274033, -0.961490, -0.021035, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, 0.448301, -0.872153, 0.195894, -0.237959, -0.155430, 0.958758, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.240856, -0.184717, 0.952821, -0.001166, 0.510636, 0.859796, -0.278860, 0.577795, 0.767066, -0.278860, 0.577795, 0.767066, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.402447, -0.888923, 0.218751, -0.871407, -0.488674, 0.042975, -0.871407, -0.488674, 0.042975, -0.867993, -0.495110, 0.038144, -0.168427, -0.936187, 0.308521, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.640003, -0.070084, 0.765169, 0.947812, -0.246434, 0.202292, 0.949402, -0.233910, 0.209575, 0.949402, -0.233910, 0.209575, 0.947812, -0.246434, 0.202292, 0.539886, 0.550710, 0.636586, 0.539886, 0.550710, 0.636586, 0.765835, 0.369611, 0.526198, 0.765835, 0.369611, 0.526198, 0.765820, 0.369627, 0.526210, 0.741377, 0.393368, 0.543712, 0.741377, 0.393368, 0.543712, 0.502593, 0.598896, 0.623477, 0.502593, 0.598896, 0.623477, -0.489526, 0.601019, 0.631775, -0.489526, 0.601019, 0.631775, 0.578916, -0.803148, -0.140751, 0.578916, -0.803148, -0.140751, 0.344423, -0.915028, -0.209994, 0.344423, -0.915028, -0.209994, 0.439693, -0.885759, -0.148667, -0.438743, -0.886217, -0.148742, -0.242173, -0.932158, -0.269134, -0.438743, -0.886217, -0.148742, -0.829634, -0.558259, 0.007340, -0.242173, -0.932158, -0.269134 };
    Normal26_5.vector = m_vector;
    Normal26_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_52.color = &Color9_4;
    IndexedFaceSet_52.coord = &Coordinate23_9;
    IndexedFaceSet_52.normal = &Normal26_5;
    IndexedFaceSet_52.texCoord = NULL;
    IndexedFaceSet_52.ccw = true;
    {
    static int m_colorIndex[] = { 1, 0, 11, 0, -1, 16, 0, 19, 0, 0, -1, 1, 1, 0, -1, 1, 0, 13, 0, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 1, 0, 1, -1, 1, 0, 0, -1, 1, 0, 0, -1, 0, 0, 0, -1, 7, 0, 0, 9, 2, -1, 5, 0, 6, -1, 0, 0, 8, -1, 0, 3, 8, -1, 10, 0, 4, -1, 15, 0, 12, -1, 0, 14, 17, -1, 0, 18, 0, -1, 0, 23, 0, -1, 22, 0, 1, 25, -1, 1, 26, 24, -1, 1, 27, 29, -1, 0, 0, 21, -1, 28, 1, 0, 20, -1 };
    IndexedFaceSet_52.colorIndex = m_colorIndex;
    IndexedFaceSet_52.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_52.colorPerVertex = true;
    IndexedFaceSet_52.convex = true;
    {
    static int m_coordIndex[] = { 3, 0, 19, 2, -1, 22, 5, 23, 4, 7, -1, 3, 10, 11, -1, 13, 8, 20, 7, -1, 13, 7, 4, -1, 13, 4, 14, -1, 6, 14, 4, -1, 13, 14, 9, -1, 3, 15, 0, -1, 3, 11, 15, -1, 12, 15, 11, -1, 17, 0, 15, 18, 1, -1, 16, 0, 17, -1, 15, 12, 18, -1, 12, 1, 18, -1, 19, 0, 16, -1, 21, 7, 20, -1, 7, 21, 22, -1, 4, 23, 6, -1, 11, 24, 12, -1, 24, 11, 10, 25, -1, 10, 26, 25, -1, 9, 27, 28, -1, 14, 6, 29, -1, 27, 9, 14, 29, -1 };
    IndexedFaceSet_52.coordIndex = m_coordIndex;
    IndexedFaceSet_52.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_52.creaseAngle = 2.942970;
    {
    static int m_normalIndex[] = { 0, 4, 9, 11, -1, 12, 14, 15, 17, 22, -1, 1, 27, 30, -1, 35, 39, 40, 23, -1, 36, 24, 18, -1, 37, 19, 42, -1, 47, 43, 20, -1, 38, 44, 50, -1, 2, 53, 5, -1, 3, 31, 54, -1, 58, 55, 32, -1, 62, 6, 56, 64, 67, -1, 69, 7, 63, -1, 57, 59, 65, -1, 60, 68, 66, -1, 10, 8, 70, -1, 71, 25, 41, -1, 26, 72, 13, -1, 21, 16, 48, -1, 33, 73, 61, -1, 74, 34, 28, 75, -1, 29, 77, 76, -1, 51, 78, 80, -1, 45, 49, 81, -1, 79, 52, 46, 82, -1 };
    IndexedFaceSet_52.normalIndex = m_normalIndex;
    IndexedFaceSet_52.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_52.normalPerVertex = true;
    IndexedFaceSet_52.solid = true;
    {
    static int m_texCoordIndex[1];
    IndexedFaceSet_52.texCoordIndex = NULL;
    IndexedFaceSet_52.texCoordIndex_length = 0;
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_52.set_colorIndex = NULL;
    IndexedFaceSet_52.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_52.set_coordIndex = NULL;
    IndexedFaceSet_52.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_52.set_normalIndex = NULL;
    IndexedFaceSet_52.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_52.set_texCoordIndex = NULL;
    IndexedFaceSet_52.set_texCoordIndex_length = 0;
    }
    Shape_235.appearance = &Appearance_236;
    Shape_235.geometry = &IndexedFaceSet_52;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_234.rotation = m_rotation;
    Transform_234.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -2.980230e-8, 5.005030e-3, -3.011850e-2 };
    Transform_234.translation = m_translation;
    Transform_234.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_234.bboxCenter = m_bboxCenter;
    Transform_234.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_234.bboxSize = m_bboxSize;
    Transform_234.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_234.center = m_center;
    Transform_234.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_234.scale = m_scale;
    Transform_234.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_234.scaleOrientation = m_scaleOrientation;
    Transform_234.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_235 };
    Transform_234.children = m_children;
    Transform_234.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_234.addChildren = NULL;
    Transform_234.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_234.removeChildren = NULL;
    Transform_234.removeChildren_length = 0;
    }
    Transform_240.route_sources =  NULL;
    Transform_240.num_route_source = 0;
    Transform_240.m_parent = (X3dNode *)NULL;
    Shape_241.m_parent = (X3dNode *)NULL;
    Appearance_242.m_parent = (X3dNode *)NULL;
    A0m_4.m_parent = (X3dNode *)NULL;
    A0m_4.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    A0m_4.diffuseColor = m_diffuseColor;
    A0m_4.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    A0m_4.emissiveColor = m_emissiveColor;
    A0m_4.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    A0m_4.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    A0m_4.specularColor = m_specularColor;
    A0m_4.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    A0m_4.transparency = 0;
    Appearance_242.material = &A0m_4;
    Appearance_242.texture = NULL;
    Appearance_242.textureTransform = NULL;
    IndexedFaceSet_53.setGlName(27);
    IndexedFaceSet_53.m_parent = (X3dNode *)NULL;
    Color10_4.m_parent = (X3dNode *)&IndexedFaceSet_53;
    {
    static float m_color[] = { 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794, 1, 0.515564, 0.031721, 1, 0.963845, 0.064794 };
    Color10_4.color = m_color;
    Color10_4.color_length = sizeof(m_color) / sizeof(float);
    }
    Coordinate24_9.m_parent = (X3dNode *)&IndexedFaceSet_53;
    {
    static float m_point[] = { 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180, 0.041613, 0.179757, -1.951840, 0.275296, 0.046906, -1.173840, 0.607257, 0.161629, -1.835830, 0.041613, 0.179757, -1.951840, -0.524042, 0.158129, -1.836490, -0.192141, 0.045295, -1.174140, 0.041913, 0.190763, -1.968430, 0.275636, 0.190905, -1.179180, 0.607566, 0.190905, -1.851050, 0.041913, 0.190763, -1.968430, -0.523739, 0.190621, -1.851050, -0.191805, 0.190621, -1.179180 };
    Coordinate24_9.point = m_point;
    Coordinate24_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal27_5.m_parent = (X3dNode *)&IndexedFaceSet_53;
    {
    static float m_vector[] = { -0.958763, 0.012792, 0.283919, -0.181027, 0.455152, 0.871817, -0.110099, 0.301903, 0.946960, -0.110099, 0.301903, 0.946960, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.410009, -0.830193, 0.377720, -0.698330, -0.710853, 0.083806, -0.181027, 0.455152, 0.871817, -0.698330, -0.710853, 0.083806, 0.875466, 0.215616, 0.432514, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.181027, 0.455152, 0.871817, 0.733671, -0.642955, 0.219854, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, -0.186095, -0.400341, -0.897272, -0.896416, 0.017432, 0.442870, 0.003126, -0.985533, -0.169456, -0.186095, -0.400341, -0.897272, -0.958814, -0.007630, -0.283932, 0.003126, -0.985533, -0.169456, -0.896416, 0.017432, 0.442870, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.958814, -0.007630, -0.283932, -0.663988, -0.719595, 0.203230, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, -0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112, -0.000228, 1, 0.000112 };
    Normal27_5.vector = m_vector;
    Normal27_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_53.color = &Color10_4;
    IndexedFaceSet_53.coord = &Coordinate24_9;
    IndexedFaceSet_53.normal = &Normal27_5;
    IndexedFaceSet_53.texCoord = NULL;
    IndexedFaceSet_53.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 1, 1, 0, -1, 0, 1, 1, -1, 2, 3, 2, -1, 5, 4, 4, -1 };
    IndexedFaceSet_53.colorIndex = m_colorIndex;
    IndexedFaceSet_53.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_53.colorPerVertex = true;
    IndexedFaceSet_53.convex = true;
    {
    static int m_coordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet_53.coordIndex = m_coordIndex;
    IndexedFaceSet_53.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_53.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 2, 4, 7, -1, 2, 7, 0, -1, 3, 10, 12, -1, 3, 12, 5, -1, 11, 13, 8, -1, 11, 8, 1, -1, 17, 19, 22, -1, 17, 22, 15, -1, 25, 27, 20, -1, 25, 20, 18, -1, 26, 28, 23, -1, 26, 23, 16, -1, 14, 9, 6, -1, 29, 24, 21, -1, 30, 31, 32, -1, 33, 34, 35, -1 };
    IndexedFaceSet_53.normalIndex = m_normalIndex;
    IndexedFaceSet_53.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_53.normalPerVertex = true;
    IndexedFaceSet_53.solid = false;
    {
    static int m_texCoordIndex[] = { 1, 7, 6, -1, 1, 6, 0, -1, 1, 2, 8, -1, 1, 8, 7, -1, 2, 8, 6, -1, 2, 6, 0, -1, 4, 10, 9, -1, 4, 9, 3, -1, 5, 11, 10, -1, 5, 10, 4, -1, 5, 11, 9, -1, 5, 9, 3, -1, 8, 6, 7, -1, 11, 9, 10, -1, 14, 13, 12, -1, 17, 16, 15, -1 };
    IndexedFaceSet_53.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_53.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_53.set_colorIndex = NULL;
    IndexedFaceSet_53.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_53.set_coordIndex = NULL;
    IndexedFaceSet_53.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_53.set_normalIndex = NULL;
    IndexedFaceSet_53.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_53.set_texCoordIndex = NULL;
    IndexedFaceSet_53.set_texCoordIndex_length = 0;
    }
    Shape_241.appearance = &Appearance_242;
    Shape_241.geometry = &IndexedFaceSet_53;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_240.rotation = m_rotation;
    Transform_240.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0.179843, -9.580490e-2 };
    Transform_240.translation = m_translation;
    Transform_240.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_240.bboxCenter = m_bboxCenter;
    Transform_240.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_240.bboxSize = m_bboxSize;
    Transform_240.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_240.center = m_center;
    Transform_240.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_240.scale = m_scale;
    Transform_240.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_240.scaleOrientation = m_scaleOrientation;
    Transform_240.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_241 };
    Transform_240.children = m_children;
    Transform_240.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_240.addChildren = NULL;
    Transform_240.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_240.removeChildren = NULL;
    Transform_240.removeChildren_length = 0;
    }
    Transform_246.route_sources =  NULL;
    Transform_246.num_route_source = 0;
    Transform_246.m_parent = (X3dNode *)NULL;
    Shape_247.m_parent = (X3dNode *)NULL;
    Appearance_248.m_parent = (X3dNode *)NULL;
    Material_249.m_parent = (X3dNode *)NULL;
    Material_249.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_249.diffuseColor = m_diffuseColor;
    Material_249.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_249.emissiveColor = m_emissiveColor;
    Material_249.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_249.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_249.specularColor = m_specularColor;
    Material_249.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_249.transparency = 6.382980e-2;
    Appearance_248.material = &Material_249;
    Appearance_248.texture = NULL;
    Appearance_248.textureTransform = NULL;
    IndexedFaceSet_54.setGlName(28);
    IndexedFaceSet_54.m_parent = (X3dNode *)NULL;
    Coordinate25_9.m_parent = (X3dNode *)&IndexedFaceSet_54;
    {
    static float m_point[] = { 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.357975, 0.274623, 0.391110, 0.435755, 0.282992, -1.405890, 0.732023, 0.277202, -1.011950, 0.803608, 0.283498, -0.488598, 0.807795, 0.291706, -0.169135, 0.642283, 0.308596, 0.135106, -0.576839, 0.295092, 0.138182, -0.741419, 0.274624, -0.165225, -0.736057, 0.266519, -0.484712, -0.662406, 0.261798, -1.008430, -0.364349, 0.274033, -1.403870, 0.034173, 1.521740, 0.072271, 0.032379, 1.258290, 0.402776, 0.324052, 1.234020, 0.434518, 0.382959, 1.442650, 0.114314, 0.462972, 1.076210, 0.512842, 0.634192, 1.091710, 0.196483, 0.036047, 1.641430, -0.319814, 0.468123, 1.540110, -0.266019, 0.780864, 1.161780, -0.151701, 0.753362, 0.779925, 0.302416, 0.875071, 0.752977, 0.006324, 0.646152, 0.462049, 0.391531, 0.036992, 1.662050, -0.688097, 0.522545, 1.549060, -0.628109, 0.873437, 1.148710, -0.497152, 0.978519, 0.692729, -0.320894, 0.036779, 1.562450, -1.012960, 0.537744, 1.451000, -0.953720, 0.879328, 1.083370, -0.818803, 0.983199, 0.632646, -0.644566, 0.672590, 0.889258, -1.304530, 0.877021, 0.531470, -1.119920, 0.439314, 0.644779, -1.532240, 0.477913, 0.461302, -1.461330, 0.028006, 0.456359, -1.522590, 0.025463, 0.201297, -1.389500, -0.259292, 1.233740, 0.434518, -0.316266, 1.434790, 0.116078, -0.398215, 1.075930, 0.512842, -0.569430, 1.091430, 0.196483, -0.398142, 1.530440, -0.263833, -0.718144, 1.145240, -0.147918, -0.688602, 0.779642, 0.302416, -0.820289, 0.734310, 0.010603, -0.581395, 0.461765, 0.391532, -0.450917, 1.538220, -0.625652, -0.809539, 1.130170, -0.492904, -0.923472, 0.671821, -0.316093, -0.466519, 1.439820, -0.951186, -0.815262, 1.064720, -0.814525, -0.927883, 0.611639, -0.639742, -0.391565, 1.162260, -1.429880, -0.821517, 0.512768, -1.115630, -0.422165, 0.451259, -1.459060, 0.032379, 1.315590, 1.240890, 0.228842, 1.352260, 0.975027, 0.032379, 1.384890, 0.907439, 0.314142, 1.285900, 1.111910, 0.314142, 1.207010, 1.274610, 0.228842, 1.140650, 1.411480, 0.032379, 1.107740, 1.479070, 0.032379, 1.284680, 0.661614, 0.313972, 1.236840, 0.760583, 0.438872, 1.139660, 0.961005, 0.438872, 0.959292, 1.192490, 0.313972, 0.916734, 1.377380, 0.032379, 0.878853, 1.498640, 0.457846, 0.598657, 1.016000, 0.325246, 0.449644, 1.073590, -0.164085, 1.351980, 0.975027, -0.249383, 1.285620, 1.111910, -0.249383, 1.206730, 1.274610, -0.164085, 1.140370, 1.411480, -0.249218, 1.236560, 0.760583, -0.374117, 1.139380, 0.961005, -0.374117, 0.959003, 1.192490, -0.249218, 0.916450, 1.377380, -0.412717, 0.598373, 1.016000, -0.277239, 0.449360, 1.073590, 0.030998, 0.455945, -1.522540, 0.200163, 0.782040, -2.021660, 0.022803, 0.816558, -2.021660, -0.153194, 0.791256, -2.021660, 0.030997, 0.455945, -1.522540, -0.638224, 0.898115, -1.302900, -0.394524, 0.650450, -1.531170, 0.012304, 1.278610, -1.485710, 0.022803, 0.816558, -2.021660, 0.012304, 1.278610, -1.485710, 0.452747, 1.152960, -1.430970 };
    Coordinate25_9.point = m_point;
    Coordinate25_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal28_5.m_parent = (X3dNode *)&IndexedFaceSet_54;
    {
    static float m_vector[] = { -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, -0.004099, 0.827127, 0.562000, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.008263, 0.987786, 0.155599, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.228359, 0.866677, 0.443535, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.293875, 0.855095, 0.427143, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.760048, 0.560343, 0.329154, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, 0.724406, 0.568302, 0.390217, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, -0.009466, 0.966128, 0.257892, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.369082, 0.908212, 0.197305, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.755452, 0.591546, 0.281718, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.891556, 0.213729, 0.399308, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.940694, 0.149539, 0.304520, 0.940732, 0.149262, 0.304540, 0.940732, 0.149262, 0.304540, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.825417, -0.443539, 0.349229, 0.834986, -0.422306, 0.352783, 0.834986, -0.422306, 0.352783, 0.837532, -0.417196, 0.352827, 0.834986, -0.422306, 0.352783, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, -0.010026, 0.999925, 0.006972, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.402267, 0.911207, -0.088786, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824726, 0.564327, 0.036905, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.824714, 0.564344, 0.036901, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.983866, 0.093656, 0.152433, 0.983850, 0.093455, 0.152661, 0.983850, 0.093455, 0.152661, 0.919028, -0.327107, 0.219973, 0.919028, -0.327107, 0.219973, 0.919030, -0.327639, 0.219171, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, -0.010038, 0.946843, -0.321540, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.445253, 0.820330, -0.358900, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857347, 0.447616, -0.254158, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.857311, 0.447668, -0.254187, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.985578, 0.009544, -0.168950, 0.871918, -0.485933, -0.060240, 0.871918, -0.485933, -0.060240, 0.871819, -0.486088, -0.060419, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.850407, 0.221019, -0.477450, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.860237, -0.066562, -0.505531, 0.748933, -0.523293, -0.406527, 0.748933, -0.523293, -0.406527, 0.748998, -0.522697, -0.407174, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.682617, -0.274976, -0.677069, 0.687379, -0.282285, -0.669197, 0.682617, -0.274976, -0.677069, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.453666, -0.338832, -0.824245, 0.373882, -0.393399, -0.839911, 0.373882, -0.393399, -0.839911, 0.372714, -0.393753, -0.840264, 0.043708, -0.372748, -0.926903, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.028974, -0.374438, -0.926799, 0.001617, -0.462617, -0.886557, 0.001617, -0.462617, -0.886557, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.239979, 0.917130, 0.318249, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.387828, 0.795914, 0.464878, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.814762, 0.529720, 0.235710, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.732864, 0.551692, 0.398179, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.489185, 0.833403, 0.257174, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.775812, 0.539234, 0.327630, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.893675, 0.204296, 0.399510, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936814, 0.032493, 0.348315, -0.936815, 0.032489, 0.348312, -0.936814, 0.032493, 0.348315, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.818954, -0.429483, 0.380602, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.757930, -0.518425, 0.395951, -0.753712, -0.525849, 0.394210, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.524327, 0.851509, 0.003698, -0.858943, 0.498866, 0.115538, -0.859015, 0.498704, 0.115703, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.858943, 0.498866, 0.115538, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.968351, -0.027173, 0.248110, -0.909057, -0.352626, 0.221969, -0.909016, -0.354059, 0.219844, -0.909016, -0.354059, 0.219844, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.565011, 0.776068, -0.280146, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.891567, 0.419099, -0.171655, -0.972656, 0.230360, -0.029579, -0.917591, -0.388391, 0.084727, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.926292, 0.088999, -0.366146, -0.914069, 0.304729, -0.267615, -0.873613, -0.486616, -0.002317, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.578068, 0.628183, -0.520790, -0.854353, -0.512791, -0.084419, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.914069, 0.304729, -0.267615, -0.657343, -0.174205, -0.733180, -0.694040, -0.019618, -0.719669, -0.817564, -0.565257, -0.109879, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.572292, -0.601094, -0.557824, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.722505, -0.409231, -0.557239, -0.128081, -0.332729, -0.934284, -0.053473, -0.461524, -0.885515, -0.465415, -0.809935, 0.356924, -0.465415, -0.809935, 0.356924, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.240017, -0.786690, 0.568780, 0.301321, -0.764284, 0.570154, 0.512027, -0.788035, 0.341804, 0.512027, -0.788035, 0.341804, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.099946, 0.939540, 0.327528, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, 0.273139, 0.958043, -0.086883, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, -0.000691, 0.999890, 0.014820, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.557718, 0.819916, 0.129182, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.653166, 0.640655, 0.403653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.544984, 0.482569, 0.685653, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, 0.126179, 0.373689, 0.918932, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, -0.000529, 0.940308, -0.340325, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.357886, 0.919802, -0.160878, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.765749, 0.642641, 0.025330, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.949294, 0.158663, 0.271417, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.708759, -0.091641, 0.699473, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.229236, -0.205492, 0.951432, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.925509, 0.112398, 0.361662, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, 0.733555, -0.479943, 0.481198, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.281483, 0.958589, 0.043287, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.485875, 0.841887, 0.234845, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.536017, 0.701217, 0.470086, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.398428, 0.578529, 0.711730, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.481318, 0.860279, -0.168087, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.813503, 0.575653, 0.082686, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.898157, 0.199085, 0.392019, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.608505, -0.020281, 0.793291, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.920217, -0.024684, 0.390630, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, -0.650589, -0.467034, 0.598843, 0.115620, -0.804235, -0.582956, -0.370219, -0.828569, -0.420014, 0.115620, -0.804235, -0.582956, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.117789, 0.819323, -0.561103, -0.725452, 0.412604, -0.550888, -0.112111, 0.280660, -0.953237, -0.115216, 0.288185, -0.950618, -0.115216, 0.288185, -0.950618, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.859217, 0.200902, -0.470515, -0.802941, -0.336838, -0.491758, -0.370219, -0.828569, -0.420014, -0.657343, -0.174205, -0.733180, -0.128736, -0.321721, -0.938042, -0.694040, -0.019618, -0.719669, 0.113269, -0.848919, -0.516243, 0.113269, -0.848919, -0.516243, 0.109595, 0.280897, -0.953460, 0.110751, 0.283846, -0.952452, 0.109595, 0.280897, -0.953460, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.397003, -0.525904, -0.752205, 0.285260, 0.728679, -0.622619, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.493291, 0.657627, -0.569377, 0.509719, 0.651105, -0.562360, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.040934, 0.842844, -0.536599, -0.046990, 0.842110, -0.537256 };
    Normal28_5.vector = m_vector;
    Normal28_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_54.color = NULL;
    IndexedFaceSet_54.coord = &Coordinate25_9;
    IndexedFaceSet_54.normal = &Normal28_5;
    IndexedFaceSet_54.texCoord = NULL;
    IndexedFaceSet_54.ccw = true;
    {
    static int m_colorIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_54.colorIndex = m_colorIndex;
    IndexedFaceSet_54.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_54.colorPerVertex = true;
    IndexedFaceSet_54.convex = true;
    {
    static int m_coordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_54.coordIndex = m_coordIndex;
    IndexedFaceSet_54.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_54.creaseAngle = 3.104420;
    {
    static int m_normalIndex[] = { 0, 6, 12, -1, 13, 18, 1, -1, 19, 14, 24, -1, 25, 30, 20, -1, 36, 2, 21, -1, 22, 42, 37, -1, 43, 23, 31, -1, 32, 48, 44, -1, 49, 33, 54, -1, 55, 60, 50, -1, 61, 56, 66, -1, 67, 72, 62, -1, 76, 38, 45, -1, 46, 82, 77, -1, 83, 47, 51, -1, 52, 88, 84, -1, 89, 53, 63, -1, 64, 94, 90, -1, 95, 65, 73, -1, 74, 100, 96, -1, 103, 78, 85, -1, 86, 109, 104, -1, 110, 87, 91, -1, 92, 115, 111, -1, 116, 93, 97, -1, 98, 121, 117, -1, 122, 99, 101, -1, 102, 127, 123, -1, 130, 118, 124, -1, 125, 136, 131, -1, 137, 126, 128, -1, 129, 142, 138, -1, 145, 132, 139, -1, 140, 150, 146, -1, 151, 141, 143, -1, 144, 155, 152, -1, 158, 147, 153, -1, 159, 154, 156, -1, 160, 157, 164, -1, 3, 166, 7, -1, 167, 4, 172, -1, 173, 178, 168, -1, 179, 174, 184, -1, 39, 175, 5, -1, 176, 40, 190, -1, 191, 185, 177, -1, 186, 192, 196, -1, 197, 202, 187, -1, 203, 198, 208, -1, 209, 214, 204, -1, 215, 210, 220, -1, 79, 193, 41, -1, 194, 80, 224, -1, 225, 199, 195, -1, 200, 226, 230, -1, 231, 211, 201, -1, 212, 232, 236, -1, 237, 221, 213, -1, 222, 238, 242, -1, 105, 227, 81, -1, 228, 106, 245, -1, 246, 233, 229, -1, 234, 247, 251, -1, 252, 239, 235, -1, 240, 253, 257, -1, 258, 243, 241, -1, 244, 259, 263, -1, 266, 254, 248, -1, 272, 264, 260, -1, 265, 273, 278, -1, 281, 279, 274, -1, 280, 282, 286, -1, 161, 287, 283, -1, 162, 165, 288, -1, 223, 289, 216, -1, 291, 217, 290, -1, 75, 68, 297, -1, 299, 309, 314, -1, 300, 318, 310, -1, 301, 323, 319, -1, 302, 328, 324, -1, 303, 333, 329, -1, 339, 315, 311, -1, 312, 345, 340, -1, 346, 313, 320, -1, 321, 351, 347, -1, 352, 322, 325, -1, 326, 357, 353, -1, 358, 327, 330, -1, 331, 363, 359, -1, 364, 332, 334, -1, 335, 369, 365, -1, 8, 341, 348, -1, 349, 15, 9, -1, 16, 350, 354, -1, 355, 26, 17, -1, 27, 356, 360, -1, 361, 375, 28, -1, 376, 362, 366, -1, 367, 381, 377, -1, 382, 368, 370, -1, 371, 292, 383, -1, 34, 29, 378, -1, 379, 57, 35, -1, 58, 380, 384, -1, 385, 69, 59, -1, 70, 386, 293, -1, 304, 316, 387, -1, 305, 388, 392, -1, 306, 393, 397, -1, 307, 398, 402, -1, 308, 403, 336, -1, 342, 389, 317, -1, 390, 343, 407, -1, 408, 394, 391, -1, 395, 409, 413, -1, 414, 399, 396, -1, 400, 415, 419, -1, 420, 404, 401, -1, 405, 421, 425, -1, 426, 337, 406, -1, 338, 427, 372, -1, 10, 410, 344, -1, 411, 11, 169, -1, 170, 416, 412, -1, 417, 171, 180, -1, 181, 422, 418, -1, 423, 182, 431, -1, 432, 428, 424, -1, 429, 433, 437, -1, 438, 373, 430, -1, 374, 439, 294, -1, 188, 434, 183, -1, 435, 189, 205, -1, 206, 440, 436, -1, 441, 207, 218, -1, 219, 295, 442, -1, 71, 296, 298, -1, 443, 445, 450, -1, 453, 446, 459, -1, 444, 460, 447, -1, 464, 466, 469, -1, 467, 474, 475, -1, 476, 470, 468, -1, 112, 119, 477, -1, 120, 133, 478, -1, 471, 479, 134, -1, 255, 267, 454, -1, 261, 256, 455, -1, 275, 262, 456, -1, 107, 113, 481, -1, 249, 108, 482, -1, 268, 250, 483, -1, 114, 480, 484, -1, 451, 448, 269, -1, 449, 457, 270, -1, 485, 452, 271, -1, 284, 276, 461, -1, 163, 285, 462, -1, 277, 458, 463, -1, 465, 472, 148, -1, 473, 135, 149, -1 };
    IndexedFaceSet_54.normalIndex = m_normalIndex;
    IndexedFaceSet_54.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_54.normalPerVertex = true;
    IndexedFaceSet_54.solid = true;
    {
    static int m_texCoordIndex[] = { 13, 14, 15, -1, 15, 16, 13, -1, 16, 15, 17, -1, 17, 18, 16, -1, 19, 13, 16, -1, 16, 20, 19, -1, 20, 16, 18, -1, 18, 21, 20, -1, 21, 18, 22, -1, 22, 23, 21, -1, 23, 22, 24, -1, 24, 7, 23, -1, 25, 19, 20, -1, 20, 26, 25, -1, 26, 20, 21, -1, 21, 27, 26, -1, 27, 21, 23, -1, 23, 28, 27, -1, 28, 23, 7, -1, 7, 6, 28, -1, 29, 25, 26, -1, 26, 30, 29, -1, 30, 26, 27, -1, 27, 31, 30, -1, 31, 27, 28, -1, 28, 32, 31, -1, 32, 28, 6, -1, 6, 5, 32, -1, 33, 31, 32, -1, 32, 34, 33, -1, 34, 32, 5, -1, 5, 4, 34, -1, 35, 33, 34, -1, 34, 36, 35, -1, 36, 34, 4, -1, 4, 3, 36, -1, 37, 35, 36, -1, 37, 36, 3, -1, 37, 3, 38, -1, 13, 39, 14, -1, 39, 13, 40, -1, 40, 41, 39, -1, 41, 40, 42, -1, 19, 40, 13, -1, 40, 19, 43, -1, 43, 42, 40, -1, 42, 43, 44, -1, 44, 45, 42, -1, 45, 44, 46, -1, 46, 47, 45, -1, 47, 46, 8, -1, 25, 43, 19, -1, 43, 25, 48, -1, 48, 44, 43, -1, 44, 48, 49, -1, 49, 46, 44, -1, 46, 49, 50, -1, 50, 8, 46, -1, 8, 50, 9, -1, 29, 48, 25, -1, 48, 29, 51, -1, 51, 49, 48, -1, 49, 51, 52, -1, 52, 50, 49, -1, 50, 52, 53, -1, 53, 9, 50, -1, 9, 53, 10, -1, 54, 52, 51, -1, 55, 10, 53, -1, 10, 55, 11, -1, 56, 11, 55, -1, 11, 56, 12, -1, 37, 12, 56, -1, 37, 38, 12, -1, 8, 1, 47, -1, 0, 47, 1, -1, 7, 24, 2, -1, 57, 58, 59, -1, 57, 60, 58, -1, 57, 61, 60, -1, 57, 62, 61, -1, 57, 63, 62, -1, 64, 59, 58, -1, 58, 65, 64, -1, 65, 58, 60, -1, 60, 66, 65, -1, 66, 60, 61, -1, 61, 67, 66, -1, 67, 61, 62, -1, 62, 68, 67, -1, 68, 62, 63, -1, 63, 69, 68, -1, 14, 64, 65, -1, 65, 15, 14, -1, 15, 65, 66, -1, 66, 17, 15, -1, 17, 66, 67, -1, 67, 70, 17, -1, 70, 67, 68, -1, 68, 71, 70, -1, 71, 68, 69, -1, 69, 0, 71, -1, 18, 17, 70, -1, 70, 22, 18, -1, 22, 70, 71, -1, 71, 24, 22, -1, 24, 71, 0, -1, 57, 59, 72, -1, 57, 72, 73, -1, 57, 73, 74, -1, 57, 74, 75, -1, 57, 75, 63, -1, 64, 72, 59, -1, 72, 64, 76, -1, 76, 73, 72, -1, 73, 76, 77, -1, 77, 74, 73, -1, 74, 77, 78, -1, 78, 75, 74, -1, 75, 78, 79, -1, 79, 63, 75, -1, 63, 79, 69, -1, 14, 76, 64, -1, 76, 14, 39, -1, 39, 77, 76, -1, 77, 39, 41, -1, 41, 78, 77, -1, 78, 41, 80, -1, 80, 79, 78, -1, 79, 80, 81, -1, 81, 69, 79, -1, 69, 81, 0, -1, 42, 80, 41, -1, 80, 42, 45, -1, 45, 81, 80, -1, 81, 45, 47, -1, 47, 0, 81, -1, 24, 0, 2, -1, 86, 85, 84, -1, 87, 85, 88, -1, 86, 88, 85, -1, 82, 90, 83, -1, 90, 91, 92, -1, 92, 83, 90, -1, 30, 31, 92, -1, 31, 33, 92, -1, 83, 92, 33, -1, 52, 54, 87, -1, 53, 52, 87, -1, 55, 53, 87, -1, 29, 30, 89, -1, 51, 29, 89, -1, 54, 51, 89, -1, 30, 92, 89, -1, 84, 85, 54, -1, 85, 87, 54, -1, 89, 84, 54, -1, 56, 55, 88, -1, 37, 56, 88, -1, 55, 87, 88, -1, 82, 83, 35, -1, 83, 33, 35, -1 };
    IndexedFaceSet_54.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_54.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_54.set_colorIndex = NULL;
    IndexedFaceSet_54.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_54.set_coordIndex = NULL;
    IndexedFaceSet_54.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_54.set_normalIndex = NULL;
    IndexedFaceSet_54.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_54.set_texCoordIndex = NULL;
    IndexedFaceSet_54.set_texCoordIndex_length = 0;
    }
    Shape_247.appearance = &Appearance_248;
    Shape_247.geometry = &IndexedFaceSet_54;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_246.rotation = m_rotation;
    Transform_246.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform_246.translation = m_translation;
    Transform_246.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_246.bboxCenter = m_bboxCenter;
    Transform_246.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_246.bboxSize = m_bboxSize;
    Transform_246.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_246.center = m_center;
    Transform_246.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_246.scale = m_scale;
    Transform_246.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_246.scaleOrientation = m_scaleOrientation;
    Transform_246.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_247 };
    Transform_246.children = m_children;
    Transform_246.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_246.addChildren = NULL;
    Transform_246.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_246.removeChildren = NULL;
    Transform_246.removeChildren_length = 0;
    }
    Transform_252.route_sources =  NULL;
    Transform_252.num_route_source = 0;
    Transform_252.m_parent = (X3dNode *)NULL;
    Shape_253.m_parent = (X3dNode *)NULL;
    Appearance_254.m_parent = (X3dNode *)NULL;
    Material_255.m_parent = (X3dNode *)NULL;
    Material_255.ambientIntensity = 1;
    {
    static float m_diffuseColor[] = { 1, 1, 1 };
    Material_255.diffuseColor = m_diffuseColor;
    Material_255.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0.691489, 0.691489, 0.691489 };
    Material_255.emissiveColor = m_emissiveColor;
    Material_255.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_255.shininess = 1;
    {
    static float m_specularColor[] = { 0.765957, 0.765957, 0.765957 };
    Material_255.specularColor = m_specularColor;
    Material_255.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_255.transparency = 0.372340;
    Appearance_254.material = &Material_255;
    Appearance_254.texture = NULL;
    Appearance_254.textureTransform = NULL;
    IndexedFaceSet_55.setGlName(29);
    IndexedFaceSet_55.m_parent = (X3dNode *)NULL;
    Coordinate26_9.m_parent = (X3dNode *)&IndexedFaceSet_55;
    {
    static float m_point[] = { 0.023775, 0.125771, 0.096370, -0.736057, 0.266519, -0.484712, 0.019864, 0.155999, -0.238411, -0.741419, 0.274624, -0.165225, 0.041913, 0.353834, 1.370070, -0.255798, 0.274481, 0.391111, 0.033239, 0.290582, 0.404469, -0.576839, 0.295092, 0.138182, 0.357975, 0.274623, 0.391110, 0.642283, 0.308596, 0.135106, 0.026537, 0.145128, 0.290011, 0.807795, 0.291706, -0.169135, 0.803608, 0.283498, -0.488598, -0.662406, 0.261798, -1.008430, 0.019557, 0.169360, -0.585595, -0.364349, 0.274033, -1.403870, 0.034997, 0.201297, -1.394460, 0.732023, 0.277202, -1.011950, 0.435755, 0.282992, -1.405890 };
    Coordinate26_9.point = m_point;
    Coordinate26_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal29_5.m_parent = (X3dNode *)&IndexedFaceSet_55;
    {
    static float m_vector[] = { -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.073288, -0.997311, 0.000483, -0.115983, -0.989347, -0.087975, -0.197996, -0.979966, 0.021537, -0.141172, -0.989925, -0.010929, -0.115983, -0.989347, -0.087975, 0.131644, -0.987142, -0.090669, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, 0.127132, -0.991148, -0.038256, 0.157553, -0.987463, -0.009671, -0.228866, -0.968557, 0.097558, -0.229441, -0.968303, 0.098722, -0.229441, -0.968303, 0.098722, 0.003121, -0.997858, 0.065337, 0.003121, -0.997858, 0.065337, 0.057747, -0.997198, -0.047547, 0.057747, -0.997198, -0.047547, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, 0.014093, -0.992543, 0.121079, -0.193939, -0.928966, 0.315294, -0.193939, -0.928966, 0.315294, -0.194333, -0.929016, 0.314902, -0.046282, -0.996765, 0.065709, -0.056059, -0.979739, -0.192273, 0.204185, -0.934007, 0.293153, 0.205250, -0.934031, 0.292331, 0.205250, -0.934031, 0.292331, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.015524, -0.925758, 0.377798, 0.256610, -0.958616, 0.123319, 0.257004, -0.958385, 0.124286, 0.256610, -0.958616, 0.123319, 0.169604, -0.985103, -0.028410, 0.169720, -0.985078, -0.028591, 0.169604, -0.985103, -0.028410, -0.141172, -0.989925, -0.010929, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, -0.178067, -0.983112, -0.042216, -0.110975, -0.993092, -0.038120, -0.078733, -0.992819, -0.090062, 0.127132, -0.991148, -0.038256, 0.198666, -0.979446, -0.034880, 0.097470, -0.991352, -0.087874, -0.178067, -0.983112, -0.042216, -0.078733, -0.992819, -0.090062, 0.010487, -0.999174, -0.039251, 0.010487, -0.999174, -0.039251, 0.131753, -0.990507, -0.039208, 0.132081, -0.990482, -0.038739, 0.131753, -0.990507, -0.039208, 0.148313, -0.987028, -0.061478, 0.148313, -0.987028, -0.061478 };
    Normal29_5.vector = m_vector;
    Normal29_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_55.color = NULL;
    IndexedFaceSet_55.coord = &Coordinate26_9;
    IndexedFaceSet_55.normal = &Normal29_5;
    IndexedFaceSet_55.texCoord = NULL;
    IndexedFaceSet_55.ccw = true;
    {
    static int m_colorIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_55.colorIndex = m_colorIndex;
    IndexedFaceSet_55.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_55.colorPerVertex = true;
    IndexedFaceSet_55.convex = true;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_55.coordIndex = m_coordIndex;
    IndexedFaceSet_55.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_55.creaseAngle = 3.141590;
    {
    static int m_normalIndex[] = { 0, 6, 9, -1, 7, 1, 15, -1, 18, 20, 22, -1, 28, 23, 21, -1, 19, 24, 31, -1, 33, 32, 25, -1, 36, 16, 2, -1, 26, 37, 34, -1, 27, 29, 38, -1, 17, 39, 30, -1, 40, 3, 42, -1, 43, 35, 41, -1, 4, 10, 45, -1, 46, 44, 5, -1, 48, 11, 8, -1, 51, 57, 59, -1, 12, 49, 52, -1, 58, 53, 50, -1, 13, 54, 61, -1, 62, 47, 14, -1, 55, 60, 64, -1, 65, 63, 56, -1 };
    IndexedFaceSet_55.normalIndex = m_normalIndex;
    IndexedFaceSet_55.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_55.normalPerVertex = true;
    IndexedFaceSet_55.solid = true;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 1, 0, 3, -1, 4, 5, 6, -1, 7, 6, 5, -1, 4, 6, 8, -1, 9, 8, 6, -1, 10, 3, 0, -1, 6, 10, 9, -1, 6, 7, 10, -1, 3, 10, 7, -1, 10, 0, 11, -1, 11, 9, 10, -1, 0, 2, 12, -1, 12, 11, 0, -1, 13, 2, 1, -1, 14, 15, 16, -1, 2, 13, 14, -1, 15, 14, 13, -1, 2, 14, 17, -1, 17, 12, 2, -1, 14, 16, 18, -1, 18, 17, 14, -1 };
    IndexedFaceSet_55.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_55.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_55.set_colorIndex = NULL;
    IndexedFaceSet_55.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_55.set_coordIndex = NULL;
    IndexedFaceSet_55.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_55.set_normalIndex = NULL;
    IndexedFaceSet_55.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_55.set_texCoordIndex = NULL;
    IndexedFaceSet_55.set_texCoordIndex_length = 0;
    }
    Shape_253.appearance = &Appearance_254;
    Shape_253.geometry = &IndexedFaceSet_55;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_252.rotation = m_rotation;
    Transform_252.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -4.957230e-3 };
    Transform_252.translation = m_translation;
    Transform_252.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_252.bboxCenter = m_bboxCenter;
    Transform_252.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_252.bboxSize = m_bboxSize;
    Transform_252.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_252.center = m_center;
    Transform_252.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_252.scale = m_scale;
    Transform_252.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_252.scaleOrientation = m_scaleOrientation;
    Transform_252.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_253 };
    Transform_252.children = m_children;
    Transform_252.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_252.addChildren = NULL;
    Transform_252.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_252.removeChildren = NULL;
    Transform_252.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_234, &Transform_240, &Transform_246, &Transform_252 };
    Group9.children = m_children;
    Group9.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group9.bboxCenter = m_bboxCenter;
    Group9.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group9.bboxSize = m_bboxSize;
    Group9.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group9.addChildren = NULL;
    Group9.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group9.removeChildren = NULL;
    Group9.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group9 };
    Inline_56.children = m_children;
    Inline_56.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_56.bboxCenter = m_bboxCenter;
    Inline_56.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_56.bboxSize = m_bboxSize;
    Inline_56.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_56.addChildren = NULL;
    Inline_56.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_56.removeChildren = NULL;
    Inline_56.removeChildren_length = 0;
    }
    Transform_258.route_sources =  NULL;
    Transform_258.num_route_source = 0;
    Transform_258.m_parent = (X3dNode *)NULL;
    Transform_259.route_sources =  NULL;
    Transform_259.num_route_source = 0;
    Transform_259.m_parent = (X3dNode *)NULL;
    Transform_260.route_sources =  NULL;
    Transform_260.num_route_source = 0;
    Transform_260.m_parent = (X3dNode *)NULL;
    Shape_261.m_parent = (X3dNode *)NULL;
    Appearance_262.m_parent = (X3dNode *)NULL;
    Material_263.m_parent = (X3dNode *)NULL;
    Material_263.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_263.diffuseColor = m_diffuseColor;
    Material_263.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_263.emissiveColor = m_emissiveColor;
    Material_263.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_263.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_263.specularColor = m_specularColor;
    Material_263.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_263.transparency = 0;
    Appearance_262.material = &Material_263;
    Appearance_262.texture = NULL;
    Appearance_262.textureTransform = NULL;
    IndexedFaceSet_57.setGlName(30);
    IndexedFaceSet_57.m_parent = (X3dNode *)NULL;
    Coordinate27_9.m_parent = (X3dNode *)&IndexedFaceSet_57;
    {
    static float m_point[] = { 0, 0, -1, -0.382683, 0, -0.923880, -0.707107, 0, -0.707107, -0.923880, 0, -0.382683, -1, 0, 0, -0.923880, 0, 0.382684, -0.707107, 0, 0.707107, -0.382683, 0, 0.923880, 0, 0, 1, 0.382683, 0, 0.923880, 0.707107, 0, 0.707107, 0.923880, 0, 0.382683, 1, 0, 0, 0.923879, 0, -0.382684, 0.707107, 0, -0.707107, 0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, -0.363954, 0.309017, -0.878662, -0.672499, 0.309017, -0.672499, -0.878662, 0.309017, -0.363954, -0.951057, 0.309017, 0, -0.878662, 0.309017, 0.363954, -0.672499, 0.309017, 0.672499, -0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, 0.363954, 0.309017, 0.878662, 0.672499, 0.309017, 0.672498, 0.878662, 0.309017, 0.363953, 0.951057, 0.309017, 0, 0.878662, 0.309017, -0.363954, 0.672498, 0.309017, -0.672499, 0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, -0.309597, 0.587785, -0.747434, -0.572061, 0.587785, -0.572061, -0.747434, 0.587785, -0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, 0.309597, -0.572061, 0.587785, 0.572061, -0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, 0.309597, 0.587785, 0.747434, 0.572061, 0.587785, 0.572061, 0.747434, 0.587785, 0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, -0.309598, 0.572061, 0.587785, -0.572062, 0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, -0.224936, 0.809017, -0.543043, -0.415627, 0.809017, -0.415627, -0.543043, 0.809017, -0.224936, -0.587785, 0.809017, 0, -0.543043, 0.809017, 0.224936, -0.415627, 0.809017, 0.415627, -0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, 0.224936, 0.809017, 0.543043, 0.415627, 0.809017, 0.415627, 0.543043, 0.809017, 0.224935, 0.587785, 0.809017, 0, 0.543043, 0.809017, -0.224936, 0.415627, 0.809017, -0.415627, 0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, -0.118256, 0.951057, -0.285494, -0.218508, 0.951057, -0.218508, -0.285494, 0.951057, -0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, 0.118256, -0.218508, 0.951057, 0.218508, -0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, 0.118256, 0.951057, 0.285494, 0.218508, 0.951057, 0.218508, 0.285495, 0.951057, 0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, -0.118256, 0.218508, 0.951057, -0.218508, 0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate27_9.point = m_point;
    Coordinate27_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal30_5.m_parent = (X3dNode *)&IndexedFaceSet_57;
    {
    static float m_vector[] = { -0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, -0.185864, 0.303889, -0.934401, -0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377972, 0.156433, -0.912505, -0.377972, 0.156433, -0.912505, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.683039, 0.258682, -0.683039, -0.698401, 0.156433, -0.698401, -0.698401, 0.156433, -0.698401, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.892433, 0.258682, -0.369658, -0.912505, 0.156434, -0.377972, -0.912505, 0.156434, -0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987689, 0.156434, 0, -0.987689, 0.156434, 0, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.892433, 0.258682, 0.369658, -0.912505, 0.156434, 0.377972, -0.912505, 0.156434, 0.377972, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.683038, 0.258683, 0.683039, -0.698401, 0.156434, 0.698402, -0.698401, 0.156434, 0.698402, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377971, 0.156434, 0.912505, -0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377973, 0.156433, 0.912505, 0.377973, 0.156433, 0.912505, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.683039, 0.258683, 0.683038, 0.698402, 0.156434, 0.698400, 0.698402, 0.156434, 0.698400, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.892433, 0.258683, 0.369658, 0.912505, 0.156435, 0.377972, 0.912505, 0.156435, 0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987688, 0.156434, 0, 0.987688, 0.156434, 0, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.892433, 0.258681, -0.369658, 0.912505, 0.156433, -0.377973, 0.912505, 0.156433, -0.377973, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.683038, 0.258681, -0.683039, 0.698401, 0.156434, -0.698401, 0.698401, 0.156434, -0.698401, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377973, 0.156435, -0.912505, 0.377973, 0.156435, -0.912505, 0.104474, 0.325528, -0.939743, 0.105800, 0.318418, -0.942028, 0.104474, 0.325528, -0.939743, 0.104474, 0.325528, -0.939743, 0.097141, 0.233062, -0.967598, 0.097141, 0.233062, -0.967598, -0.158828, 0.580683, -0.798486, -0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.320943, 0.544651, -0.774823, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.593024, 0.544651, -0.593024, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.774823, 0.544651, -0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.774823, 0.544651, 0.320943, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.593024, 0.544651, 0.593024, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, -0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.320943, 0.544651, 0.774823, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.593025, 0.544650, 0.593024, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.774824, 0.544650, 0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.774824, 0.544650, -0.320943, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.593024, 0.544650, -0.593025, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.320942, 0.544650, -0.774824, 0.090247, 0.600810, -0.794282, 0.091515, 0.594767, -0.798672, 0.090247, 0.600810, -0.794282, 0.090247, 0.600810, -0.794282, -0.116055, 0.803815, -0.583449, -0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.581405, 0.777156, 0.240826, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.581406, 0.777156, 0.240825, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.067135, 0.818259, -0.570916, 0.068222, 0.813901, -0.576985, 0.067135, 0.818259, -0.570916, 0.067135, 0.818259, -0.570916, -0.061299, 0.949355, -0.308167, -0.061299, 0.949355, -0.308167, 0, 1, 0, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137070, 0.933532, -0.331255, -0.137550, 0.933532, -0.331056, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253402, 0.933532, -0.253585, -0.253769, 0.933532, -0.253218, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331155, 0.933532, -0.137310, -0.331354, 0.933532, -0.136830, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, -0.000130, -0.358494, 0.933532, 0.000389, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331255, 0.933532, 0.137070, -0.331056, 0.933532, 0.137550, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253585, 0.933532, 0.253402, -0.253218, 0.933532, 0.253769, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.137310, 0.933532, 0.331155, -0.136830, 0.933532, 0.331354, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, -0.000130, 0.933532, 0.358494, 0.000389, 0.933532, 0.358494, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137070, 0.933532, 0.331255, 0.137550, 0.933532, 0.331056, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253401, 0.933532, 0.253586, 0.253768, 0.933532, 0.253219, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331156, 0.933532, 0.137310, 0.331354, 0.933532, 0.136830, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, 0.000128, 0.358495, 0.933532, -0.000391, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331255, 0.933532, -0.137070, 0.331056, 0.933532, -0.137550, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253585, 0.933532, -0.253402, 0.253218, 0.933532, -0.253769, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.137310, 0.933532, -0.331155, 0.136830, 0.933532, -0.331354, 0.042033, 0.942951, -0.330266, 0.042048, 0.942867, -0.330504, 0.042033, 0.942951, -0.330266, 0, 1, 0, -0.031102, 0.987211, -0.156357, -0.088569, 0.987211, -0.132553, -0.132553, 0.987211, -0.088569, -0.156357, 0.987211, -0.031102, -0.156357, 0.987211, 0.031102, -0.132553, 0.987211, 0.088569, -0.088569, 0.987211, 0.132553, -0.031102, 0.987211, 0.156357, 0.031102, 0.987211, 0.156357, 0.088569, 0.987211, 0.132553, 0.132553, 0.987211, 0.088570, 0.156357, 0.987211, 0.031101, 0.156357, 0.987211, -0.031102, 0.132553, 0.987211, -0.088569, 0.088569, 0.987211, -0.132553, 0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal30_5.vector = m_vector;
    Normal30_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate12_16.m_parent = (X3dNode *)&IndexedFaceSet_57;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate12_16.point = m_point;
    TextureCoordinate12_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_57.color = NULL;
    IndexedFaceSet_57.coord = &Coordinate27_9;
    IndexedFaceSet_57.normal = &Normal30_5;
    IndexedFaceSet_57.texCoord = &TextureCoordinate12_16;
    IndexedFaceSet_57.ccw = false;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_57.colorIndex = m_colorIndex;
    IndexedFaceSet_57.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_57.colorPerVertex = true;
    IndexedFaceSet_57.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_57.coordIndex = m_coordIndex;
    IndexedFaceSet_57.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_57.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet_57.normalIndex = m_normalIndex;
    IndexedFaceSet_57.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_57.normalPerVertex = true;
    IndexedFaceSet_57.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_57.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_57.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_57.set_colorIndex = NULL;
    IndexedFaceSet_57.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_57.set_coordIndex = NULL;
    IndexedFaceSet_57.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_57.set_normalIndex = NULL;
    IndexedFaceSet_57.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_57.set_texCoordIndex = NULL;
    IndexedFaceSet_57.set_texCoordIndex_length = 0;
    }
    Shape_261.appearance = &Appearance_262;
    Shape_261.geometry = &IndexedFaceSet_57;
    Transform_267.route_sources =  NULL;
    Transform_267.num_route_source = 0;
    Transform_267.m_parent = (X3dNode *)NULL;
    Shape_268.m_parent = (X3dNode *)NULL;
    Appearance_269.m_parent = (X3dNode *)NULL;
    Material_270.m_parent = (X3dNode *)NULL;
    Material_270.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material_270.diffuseColor = m_diffuseColor;
    Material_270.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_270.emissiveColor = m_emissiveColor;
    Material_270.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_270.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material_270.specularColor = m_specularColor;
    Material_270.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_270.transparency = 0;
    Appearance_269.material = &Material_270;
    Appearance_269.texture = NULL;
    Appearance_269.textureTransform = NULL;
    IndexedFaceSet_58.setGlName(31);
    IndexedFaceSet_58.m_parent = (X3dNode *)NULL;
    Coordinate28_9.m_parent = (X3dNode *)&IndexedFaceSet_58;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate28_9.point = m_point;
    Coordinate28_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal31_5.m_parent = (X3dNode *)&IndexedFaceSet_58;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal31_5.vector = m_vector;
    Normal31_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate13_16.m_parent = (X3dNode *)&IndexedFaceSet_58;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate13_16.point = m_point;
    TextureCoordinate13_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_58.color = NULL;
    IndexedFaceSet_58.coord = &Coordinate28_9;
    IndexedFaceSet_58.normal = &Normal31_5;
    IndexedFaceSet_58.texCoord = &TextureCoordinate13_16;
    IndexedFaceSet_58.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_58.colorIndex = m_colorIndex;
    IndexedFaceSet_58.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_58.colorPerVertex = true;
    IndexedFaceSet_58.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_58.coordIndex = m_coordIndex;
    IndexedFaceSet_58.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_58.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet_58.normalIndex = m_normalIndex;
    IndexedFaceSet_58.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_58.normalPerVertex = true;
    IndexedFaceSet_58.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_58.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_58.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_58.set_colorIndex = NULL;
    IndexedFaceSet_58.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_58.set_coordIndex = NULL;
    IndexedFaceSet_58.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_58.set_normalIndex = NULL;
    IndexedFaceSet_58.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_58.set_texCoordIndex = NULL;
    IndexedFaceSet_58.set_texCoordIndex_length = 0;
    }
    Shape_268.appearance = &Appearance_269;
    Shape_268.geometry = &IndexedFaceSet_58;
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 0 };
    Transform_267.rotation = m_rotation;
    Transform_267.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.220000, 0.440000, 0.160000 };
    Transform_267.translation = m_translation;
    Transform_267.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_267.bboxCenter = m_bboxCenter;
    Transform_267.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_267.bboxSize = m_bboxSize;
    Transform_267.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_267.center = m_center;
    Transform_267.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform_267.scale = m_scale;
    Transform_267.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_267.scaleOrientation = m_scaleOrientation;
    Transform_267.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_268 };
    Transform_267.children = m_children;
    Transform_267.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_267.addChildren = NULL;
    Transform_267.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_267.removeChildren = NULL;
    Transform_267.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.991215, -0.104605, -8.093859e-2, 1.578764 };
    Transform_260.rotation = m_rotation;
    Transform_260.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_260.translation = m_translation;
    Transform_260.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_260.bboxCenter = m_bboxCenter;
    Transform_260.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_260.bboxSize = m_bboxSize;
    Transform_260.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_260.center = m_center;
    Transform_260.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_260.scale = m_scale;
    Transform_260.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_260.scaleOrientation = m_scaleOrientation;
    Transform_260.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_261, &Transform_267 };
    Transform_260.children = m_children;
    Transform_260.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_260.addChildren = NULL;
    Transform_260.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_260.removeChildren = NULL;
    Transform_260.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0.565552, -0.792356, -0.228743, 0.826199 };
    Transform_259.rotation = m_rotation;
    Transform_259.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.109765, 0.715337, 1.294240 };
    Transform_259.translation = m_translation;
    Transform_259.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_259.bboxCenter = m_bboxCenter;
    Transform_259.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_259.bboxSize = m_bboxSize;
    Transform_259.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_259.center = m_center;
    Transform_259.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform_259.scale = m_scale;
    Transform_259.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_259.scaleOrientation = m_scaleOrientation;
    Transform_259.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_260 };
    Transform_259.children = m_children;
    Transform_259.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_259.addChildren = NULL;
    Transform_259.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_259.removeChildren = NULL;
    Transform_259.removeChildren_length = 0;
    }
    Transform_274.route_sources =  NULL;
    Transform_274.num_route_source = 0;
    Transform_274.m_parent = (X3dNode *)NULL;
    Transform_275.route_sources =  NULL;
    Transform_275.num_route_source = 0;
    Transform_275.m_parent = (X3dNode *)NULL;
    Shape_276.m_parent = (X3dNode *)NULL;
    Appearance_277.m_parent = (X3dNode *)NULL;
    Material_278.m_parent = (X3dNode *)NULL;
    Material_278.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 0.800000, 0.800000, 0.800000 };
    Material_278.diffuseColor = m_diffuseColor;
    Material_278.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_278.emissiveColor = m_emissiveColor;
    Material_278.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_278.shininess = 0.200000;
    {
    static float m_specularColor[] = { 0, 0, 0 };
    Material_278.specularColor = m_specularColor;
    Material_278.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_278.transparency = 0;
    Appearance_277.material = &Material_278;
    Appearance_277.texture = NULL;
    Appearance_277.textureTransform = NULL;
    IndexedFaceSet_59.setGlName(32);
    IndexedFaceSet_59.m_parent = (X3dNode *)NULL;
    Coordinate29_9.m_parent = (X3dNode *)&IndexedFaceSet_59;
    {
    static float m_point[] = { 0, 0, -1, 0.382683, 0, -0.923880, 0.707107, 0, -0.707107, 0.923880, 0, -0.382683, 1, 0, 0, 0.923880, 0, 0.382684, 0.707107, 0, 0.707107, 0.382683, 0, 0.923880, 0, 0, 1, -0.382683, 0, 0.923880, -0.707107, 0, 0.707107, -0.923880, 0, 0.382683, -1, 0, 0, -0.923879, 0, -0.382684, -0.707107, 0, -0.707107, -0.382683, 0, -0.923880, 0, 0, -1, 0, 0.309017, -0.951057, 0.363954, 0.309017, -0.878662, 0.672499, 0.309017, -0.672499, 0.878662, 0.309017, -0.363954, 0.951057, 0.309017, 0, 0.878662, 0.309017, 0.363954, 0.672499, 0.309017, 0.672499, 0.363953, 0.309017, 0.878662, 0, 0.309017, 0.951057, -0.363954, 0.309017, 0.878662, -0.672499, 0.309017, 0.672498, -0.878662, 0.309017, 0.363953, -0.951057, 0.309017, 0, -0.878662, 0.309017, -0.363954, -0.672498, 0.309017, -0.672499, -0.363954, 0.309017, -0.878662, 0, 0.309017, -0.951057, 0, 0.587785, -0.809017, 0.309597, 0.587785, -0.747434, 0.572061, 0.587785, -0.572061, 0.747434, 0.587785, -0.309597, 0.809017, 0.587785, 0, 0.747434, 0.587785, 0.309597, 0.572061, 0.587785, 0.572061, 0.309597, 0.587785, 0.747434, 0, 0.587785, 0.809017, -0.309597, 0.587785, 0.747434, -0.572061, 0.587785, 0.572061, -0.747434, 0.587785, 0.309597, -0.809017, 0.587785, 0, -0.747434, 0.587785, -0.309598, -0.572061, 0.587785, -0.572062, -0.309597, 0.587785, -0.747434, 0, 0.587785, -0.809017, 0, 0.809017, -0.587785, 0.224936, 0.809017, -0.543043, 0.415627, 0.809017, -0.415627, 0.543043, 0.809017, -0.224936, 0.587785, 0.809017, 0, 0.543043, 0.809017, 0.224936, 0.415627, 0.809017, 0.415627, 0.224936, 0.809017, 0.543043, 0, 0.809017, 0.587785, -0.224936, 0.809017, 0.543043, -0.415627, 0.809017, 0.415627, -0.543043, 0.809017, 0.224935, -0.587785, 0.809017, 0, -0.543043, 0.809017, -0.224936, -0.415627, 0.809017, -0.415627, -0.224936, 0.809017, -0.543043, 0, 0.809017, -0.587785, 0, 0.951057, -0.309017, 0.118256, 0.951057, -0.285494, 0.218508, 0.951057, -0.218508, 0.285494, 0.951057, -0.118256, 0.309017, 0.951057, 0, 0.285494, 0.951057, 0.118256, 0.218508, 0.951057, 0.218508, 0.118256, 0.951057, 0.285494, 0, 0.951057, 0.309017, -0.118256, 0.951057, 0.285494, -0.218508, 0.951057, 0.218508, -0.285495, 0.951057, 0.118256, -0.309017, 0.951057, 0, -0.285494, 0.951057, -0.118256, -0.218508, 0.951057, -0.218508, -0.118256, 0.951057, -0.285494, 0, 0.951057, -0.309017, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate29_9.point = m_point;
    Coordinate29_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal32_5.m_parent = (X3dNode *)&IndexedFaceSet_59;
    {
    static float m_vector[] = { 0.192778, 0.153499, -0.969162, 0, 0.309017, -0.951057, 0.185864, 0.303889, -0.934401, 0.185864, 0.303889, -0.934401, 0, 0.587785, -0.809017, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.369658, 0.258682, -0.892433, 0.377972, 0.156433, -0.912505, 0.377972, 0.156433, -0.912505, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.683039, 0.258682, -0.683039, 0.698401, 0.156433, -0.698401, 0.698401, 0.156433, -0.698401, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.892433, 0.258682, -0.369658, 0.912505, 0.156434, -0.377972, 0.912505, 0.156434, -0.377972, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.965963, 0.258682, 0, 0.987689, 0.156434, 0, 0.987689, 0.156434, 0, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.892433, 0.258682, 0.369658, 0.912505, 0.156434, 0.377972, 0.912505, 0.156434, 0.377972, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.683038, 0.258683, 0.683039, 0.698401, 0.156434, 0.698402, 0.698401, 0.156434, 0.698402, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.369658, 0.258682, 0.892433, 0.377971, 0.156434, 0.912505, 0.377971, 0.156434, 0.912505, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.258682, 0.965963, 0, 0.156434, 0.987688, 0, 0.156434, 0.987688, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.369658, 0.258682, 0.892433, -0.377973, 0.156433, 0.912505, -0.377973, 0.156433, 0.912505, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.683039, 0.258683, 0.683038, -0.698402, 0.156434, 0.698400, -0.698402, 0.156434, 0.698400, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.892433, 0.258683, 0.369658, -0.912505, 0.156435, 0.377972, -0.912505, 0.156435, 0.377972, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.965963, 0.258682, 0, -0.987688, 0.156434, 0, -0.987688, 0.156434, 0, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.892433, 0.258681, -0.369658, -0.912505, 0.156433, -0.377973, -0.912505, 0.156433, -0.377973, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.683038, 0.258681, -0.683039, -0.698401, 0.156434, -0.698401, -0.698401, 0.156434, -0.698401, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.369658, 0.258682, -0.892433, -0.377973, 0.156435, -0.912505, -0.377973, 0.156435, -0.912505, -0.104474, 0.325528, -0.939743, -0.105800, 0.318418, -0.942028, -0.104474, 0.325528, -0.939743, -0.104474, 0.325528, -0.939743, -0.097141, 0.233062, -0.967598, -0.097141, 0.233062, -0.967598, 0.158828, 0.580683, -0.798486, 0.158828, 0.580683, -0.798486, 0, 0.809017, -0.587785, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.320943, 0.544651, -0.774823, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.593024, 0.544651, -0.593024, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.774823, 0.544651, -0.320943, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.838663, 0.544650, 0, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.774823, 0.544651, 0.320943, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.593024, 0.544651, 0.593024, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0.320943, 0.544651, 0.774824, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, 0, 0.544650, 0.838663, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.320943, 0.544651, 0.774823, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.593025, 0.544650, 0.593024, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.774824, 0.544650, 0.320943, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.838663, 0.544650, 0, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.774824, 0.544650, -0.320943, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.593024, 0.544650, -0.593025, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.320942, 0.544650, -0.774824, -0.090247, 0.600810, -0.794282, -0.091515, 0.594767, -0.798672, -0.090247, 0.600810, -0.794282, -0.090247, 0.600810, -0.794282, 0.116055, 0.803815, -0.583449, 0.116055, 0.803815, -0.583449, 0, 0.951057, -0.309017, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.240826, 0.777156, -0.581405, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.444988, 0.777156, -0.444988, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.581405, 0.777156, -0.240826, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.629308, 0.777156, 0, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.581405, 0.777156, 0.240826, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.444988, 0.777156, 0.444988, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0.240826, 0.777156, 0.581405, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, 0, 0.777156, 0.629308, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.240826, 0.777156, 0.581405, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.444988, 0.777156, 0.444988, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.581406, 0.777156, 0.240825, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.629308, 0.777156, 0, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.581405, 0.777156, -0.240826, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.444988, 0.777156, -0.444988, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.240826, 0.777156, -0.581405, -0.067135, 0.818259, -0.570916, -0.068222, 0.813901, -0.576985, -0.067135, 0.818259, -0.570916, -0.067135, 0.818259, -0.570916, 0.061299, 0.949355, -0.308167, 0.061299, 0.949355, -0.308167, 0, 1, 0, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137070, 0.933532, -0.331255, 0.137550, 0.933532, -0.331056, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253402, 0.933532, -0.253585, 0.253769, 0.933532, -0.253218, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331155, 0.933532, -0.137310, 0.331354, 0.933532, -0.136830, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, -0.000130, 0.358494, 0.933532, 0.000389, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331255, 0.933532, 0.137070, 0.331056, 0.933532, 0.137550, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253585, 0.933532, 0.253402, 0.253218, 0.933532, 0.253769, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.137310, 0.933532, 0.331155, 0.136830, 0.933532, 0.331354, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, 0.000130, 0.933532, 0.358494, -0.000389, 0.933532, 0.358494, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137070, 0.933532, 0.331255, -0.137550, 0.933532, 0.331056, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253401, 0.933532, 0.253586, -0.253768, 0.933532, 0.253219, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331156, 0.933532, 0.137310, -0.331354, 0.933532, 0.136830, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, 0.000128, -0.358495, 0.933532, -0.000391, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331255, 0.933532, -0.137070, -0.331056, 0.933532, -0.137550, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253585, 0.933532, -0.253402, -0.253218, 0.933532, -0.253769, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.137310, 0.933532, -0.331155, -0.136830, 0.933532, -0.331354, -0.042033, 0.942951, -0.330266, -0.042048, 0.942867, -0.330504, -0.042033, 0.942951, -0.330266, 0, 1, 0, 0.031102, 0.987211, -0.156357, 0.088569, 0.987211, -0.132553, 0.132553, 0.987211, -0.088569, 0.156357, 0.987211, -0.031102, 0.156357, 0.987211, 0.031102, 0.132553, 0.987211, 0.088569, 0.088569, 0.987211, 0.132553, 0.031102, 0.987211, 0.156357, -0.031102, 0.987211, 0.156357, -0.088569, 0.987211, 0.132553, -0.132553, 0.987211, 0.088570, -0.156357, 0.987211, 0.031101, -0.156357, 0.987211, -0.031102, -0.132553, 0.987211, -0.088569, -0.088569, 0.987211, -0.132553, -0.031102, 0.987211, -0.156357, 0, 1, 0 };
    Normal32_5.vector = m_vector;
    Normal32_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate14_16.m_parent = (X3dNode *)&IndexedFaceSet_59;
    {
    static float m_point[] = { 0.500000, 0, 0.308658, 0.038060, 0.146447, 0.146447, 0.038060, 0.308658, 0, 0.500000, 0.038060, 0.691342, 0.146447, 0.853553, 0.308658, 0.961940, 0.500000, 1, 0.691342, 0.961940, 0.853553, 0.853553, 0.961940, 0.691342, 1, 0.500000, 0.961940, 0.308658, 0.853553, 0.146447, 0.691342, 0.038060, 0.500000, 0, 0.500000, 0.024472, 0.318023, 0.060669, 0.163751, 0.163751, 0.060669, 0.318023, 0.024472, 0.500000, 0.060669, 0.681977, 0.163751, 0.836249, 0.318023, 0.939331, 0.500000, 0.975528, 0.681977, 0.939331, 0.836249, 0.836249, 0.939331, 0.681977, 0.975528, 0.500000, 0.939331, 0.318023, 0.836249, 0.163751, 0.681977, 0.060669, 0.500000, 0.024472, 0.500000, 0.095491, 0.345201, 0.126283, 0.213969, 0.213969, 0.126283, 0.345201, 0.095491, 0.500000, 0.126283, 0.654799, 0.213969, 0.786031, 0.345201, 0.873717, 0.500000, 0.904508, 0.654799, 0.873717, 0.786031, 0.786031, 0.873717, 0.654799, 0.904508, 0.500000, 0.873717, 0.345201, 0.786031, 0.213969, 0.654799, 0.126283, 0.500000, 0.095491, 0.500000, 0.206107, 0.387532, 0.228479, 0.292187, 0.292187, 0.228479, 0.387532, 0.206107, 0.500000, 0.228479, 0.612468, 0.292187, 0.707813, 0.387532, 0.771521, 0.500000, 0.793893, 0.612468, 0.771521, 0.707814, 0.707813, 0.771521, 0.612468, 0.793893, 0.500000, 0.771521, 0.387532, 0.707813, 0.292186, 0.612468, 0.228479, 0.500000, 0.206107, 0.500000, 0.345492, 0.440872, 0.357253, 0.390746, 0.390746, 0.357253, 0.440872, 0.345492, 0.500000, 0.357253, 0.559128, 0.390746, 0.609254, 0.440872, 0.642747, 0.500000, 0.654508, 0.559128, 0.642747, 0.609254, 0.609254, 0.642747, 0.559128, 0.654508, 0.500000, 0.642747, 0.440872, 0.609254, 0.390746, 0.559128, 0.357253, 0.500000, 0.345492, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate14_16.point = m_point;
    TextureCoordinate14_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_59.color = NULL;
    IndexedFaceSet_59.coord = &Coordinate29_9;
    IndexedFaceSet_59.normal = &Normal32_5;
    IndexedFaceSet_59.texCoord = &TextureCoordinate14_16;
    IndexedFaceSet_59.ccw = true;
    {
    static int m_colorIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_59.colorIndex = m_colorIndex;
    IndexedFaceSet_59.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_59.colorPerVertex = true;
    IndexedFaceSet_59.convex = false;
    {
    static int m_coordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_59.coordIndex = m_coordIndex;
    IndexedFaceSet_59.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_59.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 16, 12, 17, -1, 16, 17, 21, -1, 22, 18, 23, -1, 22, 23, 27, -1, 28, 24, 29, -1, 28, 29, 33, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 42, 47, 51, -1, 42, 51, 46, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 60, 65, 69, -1, 60, 69, 64, -1, 66, 71, 75, -1, 66, 75, 70, -1, 76, 72, 77, -1, 76, 77, 81, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 94, 90, 95, -1, 94, 95, 99, -1, 100, 96, 1, -1, 3, 101, 104, -1, 3, 104, 7, -1, 8, 105, 108, -1, 8, 108, 13, -1, 109, 112, 19, -1, 109, 19, 14, -1, 113, 116, 25, -1, 113, 25, 20, -1, 26, 117, 120, -1, 26, 120, 31, -1, 32, 121, 124, -1, 32, 124, 37, -1, 125, 128, 43, -1, 125, 43, 38, -1, 44, 129, 132, -1, 44, 132, 49, -1, 50, 133, 136, -1, 50, 136, 55, -1, 56, 137, 140, -1, 56, 140, 61, -1, 62, 141, 144, -1, 62, 144, 67, -1, 68, 145, 148, -1, 68, 148, 73, -1, 149, 152, 79, -1, 149, 79, 74, -1, 153, 156, 85, -1, 153, 85, 80, -1, 157, 160, 91, -1, 157, 91, 86, -1, 161, 164, 97, -1, 161, 97, 92, -1, 98, 165, 4, -1, 168, 171, 106, -1, 168, 106, 102, -1, 172, 175, 110, -1, 172, 110, 107, -1, 111, 176, 179, -1, 111, 179, 114, -1, 115, 180, 183, -1, 115, 183, 118, -1, 184, 187, 122, -1, 184, 122, 119, -1, 188, 191, 126, -1, 188, 126, 123, -1, 127, 192, 195, -1, 127, 195, 130, -1, 131, 196, 199, -1, 131, 199, 134, -1, 200, 203, 138, -1, 200, 138, 135, -1, 204, 207, 142, -1, 204, 142, 139, -1, 208, 211, 146, -1, 208, 146, 143, -1, 212, 215, 150, -1, 212, 150, 147, -1, 151, 216, 219, -1, 151, 219, 154, -1, 155, 220, 223, -1, 155, 223, 158, -1, 159, 224, 227, -1, 159, 227, 162, -1, 163, 228, 231, -1, 163, 231, 166, -1, 167, 232, 103, -1, 169, 235, 238, -1, 169, 238, 173, -1, 239, 242, 177, -1, 239, 177, 174, -1, 178, 243, 246, -1, 178, 246, 181, -1, 247, 250, 185, -1, 247, 185, 182, -1, 186, 251, 254, -1, 186, 254, 189, -1, 255, 258, 193, -1, 255, 193, 190, -1, 194, 259, 262, -1, 194, 262, 197, -1, 263, 266, 201, -1, 263, 201, 198, -1, 202, 267, 270, -1, 202, 270, 205, -1, 271, 274, 209, -1, 271, 209, 206, -1, 210, 275, 278, -1, 210, 278, 213, -1, 279, 282, 217, -1, 279, 217, 214, -1, 218, 283, 286, -1, 218, 286, 221, -1, 287, 290, 225, -1, 287, 225, 222, -1, 226, 291, 294, -1, 226, 294, 229, -1, 295, 298, 233, -1, 295, 233, 230, -1, 234, 299, 170, -1, 236, 302, 240, -1, 241, 303, 244, -1, 245, 304, 248, -1, 249, 305, 252, -1, 253, 306, 256, -1, 257, 307, 260, -1, 261, 308, 264, -1, 265, 309, 268, -1, 269, 310, 272, -1, 273, 311, 276, -1, 277, 312, 280, -1, 281, 313, 284, -1, 285, 314, 288, -1, 289, 315, 292, -1, 293, 316, 296, -1, 297, 317, 300, -1, 301, 318, 237, -1 };
    IndexedFaceSet_59.normalIndex = m_normalIndex;
    IndexedFaceSet_59.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_59.normalPerVertex = true;
    IndexedFaceSet_59.solid = false;
    {
    static int m_texCoordIndex[] = { 17, 18, 1, -1, 17, 1, 0, -1, 18, 19, 2, -1, 18, 2, 1, -1, 2, 19, 20, -1, 2, 20, 3, -1, 3, 20, 21, -1, 3, 21, 4, -1, 4, 21, 22, -1, 4, 22, 5, -1, 22, 23, 6, -1, 22, 6, 5, -1, 23, 24, 7, -1, 23, 7, 6, -1, 24, 25, 8, -1, 24, 8, 7, -1, 25, 26, 9, -1, 25, 9, 8, -1, 26, 27, 10, -1, 26, 10, 9, -1, 27, 28, 11, -1, 27, 11, 10, -1, 28, 29, 12, -1, 28, 12, 11, -1, 12, 29, 30, -1, 12, 30, 13, -1, 30, 31, 14, -1, 30, 14, 13, -1, 14, 31, 32, -1, 14, 32, 15, -1, 15, 32, 33, -1, 15, 33, 16, -1, 16, 33, 0, -1, 17, 34, 35, -1, 17, 35, 18, -1, 18, 35, 36, -1, 18, 36, 19, -1, 36, 37, 20, -1, 36, 20, 19, -1, 37, 38, 21, -1, 37, 21, 20, -1, 21, 38, 39, -1, 21, 39, 22, -1, 22, 39, 40, -1, 22, 40, 23, -1, 40, 41, 24, -1, 40, 24, 23, -1, 24, 41, 42, -1, 24, 42, 25, -1, 25, 42, 43, -1, 25, 43, 26, -1, 26, 43, 44, -1, 26, 44, 27, -1, 27, 44, 45, -1, 27, 45, 28, -1, 28, 45, 46, -1, 28, 46, 29, -1, 46, 47, 30, -1, 46, 30, 29, -1, 47, 48, 31, -1, 47, 31, 30, -1, 48, 49, 32, -1, 48, 32, 31, -1, 49, 50, 33, -1, 49, 33, 32, -1, 33, 50, 17, -1, 51, 52, 35, -1, 51, 35, 34, -1, 52, 53, 36, -1, 52, 36, 35, -1, 36, 53, 54, -1, 36, 54, 37, -1, 37, 54, 55, -1, 37, 55, 38, -1, 55, 56, 39, -1, 55, 39, 38, -1, 56, 57, 40, -1, 56, 40, 39, -1, 40, 57, 58, -1, 40, 58, 41, -1, 41, 58, 59, -1, 41, 59, 42, -1, 59, 60, 43, -1, 59, 43, 42, -1, 60, 61, 44, -1, 60, 44, 43, -1, 61, 62, 45, -1, 61, 45, 44, -1, 62, 63, 46, -1, 62, 46, 45, -1, 46, 63, 64, -1, 46, 64, 47, -1, 47, 64, 65, -1, 47, 65, 48, -1, 48, 65, 66, -1, 48, 66, 49, -1, 49, 66, 67, -1, 49, 67, 50, -1, 50, 67, 34, -1, 51, 68, 69, -1, 51, 69, 52, -1, 69, 70, 53, -1, 69, 53, 52, -1, 53, 70, 71, -1, 53, 71, 54, -1, 71, 72, 55, -1, 71, 55, 54, -1, 55, 72, 73, -1, 55, 73, 56, -1, 73, 74, 57, -1, 73, 57, 56, -1, 57, 74, 75, -1, 57, 75, 58, -1, 75, 76, 59, -1, 75, 59, 58, -1, 59, 76, 77, -1, 59, 77, 60, -1, 77, 78, 61, -1, 77, 61, 60, -1, 61, 78, 79, -1, 61, 79, 62, -1, 79, 80, 63, -1, 79, 63, 62, -1, 63, 80, 81, -1, 63, 81, 64, -1, 81, 82, 65, -1, 81, 65, 64, -1, 65, 82, 83, -1, 65, 83, 66, -1, 83, 84, 67, -1, 83, 67, 66, -1, 67, 84, 51, -1, 68, 85, 69, -1, 69, 86, 70, -1, 70, 87, 71, -1, 71, 88, 72, -1, 72, 89, 73, -1, 73, 90, 74, -1, 74, 91, 75, -1, 75, 92, 76, -1, 76, 93, 77, -1, 77, 94, 78, -1, 78, 95, 79, -1, 79, 96, 80, -1, 80, 97, 81, -1, 81, 98, 82, -1, 82, 99, 83, -1, 83, 100, 84, -1, 84, 101, 68, -1 };
    IndexedFaceSet_59.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_59.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_59.set_colorIndex = NULL;
    IndexedFaceSet_59.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_59.set_coordIndex = NULL;
    IndexedFaceSet_59.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_59.set_normalIndex = NULL;
    IndexedFaceSet_59.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_59.set_texCoordIndex = NULL;
    IndexedFaceSet_59.set_texCoordIndex_length = 0;
    }
    Shape_276.appearance = &Appearance_277;
    Shape_276.geometry = &IndexedFaceSet_59;
    Transform_282.route_sources =  NULL;
    Transform_282.num_route_source = 0;
    Transform_282.m_parent = (X3dNode *)NULL;
    Shape_283.m_parent = (X3dNode *)NULL;
    Appearance_284.m_parent = (X3dNode *)NULL;
    Material_285.m_parent = (X3dNode *)NULL;
    Material_285.ambientIntensity = 0.200000;
    {
    static float m_diffuseColor[] = { 8.627451e-2, 8.627451e-2, 8.627451e-2 };
    Material_285.diffuseColor = m_diffuseColor;
    Material_285.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_285.emissiveColor = m_emissiveColor;
    Material_285.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_285.shininess = 0.200000;
    {
    static float m_specularColor[] = { 1, 0.988235, 0.952941 };
    Material_285.specularColor = m_specularColor;
    Material_285.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_285.transparency = 0;
    Appearance_284.material = &Material_285;
    Appearance_284.texture = NULL;
    Appearance_284.textureTransform = NULL;
    IndexedFaceSet_60.setGlName(33);
    IndexedFaceSet_60.m_parent = (X3dNode *)NULL;
    Coordinate30_9.m_parent = (X3dNode *)&IndexedFaceSet_60;
    {
    static float m_point[] = { 0, 0, -1, -0.309017, 0, -0.951056, -0.587785, 0, -0.809017, -0.809017, 0, -0.587785, -0.951057, 0, -0.309017, -1, 0, 0, -0.951056, 0, 0.309017, -0.809017, 0, 0.587785, -0.587785, 0, 0.809017, -0.309017, 0, 0.951057, 0, 0, 1, 0.309017, 0, 0.951057, 0.587785, 0, 0.809017, 0.809017, 0, 0.587785, 0.951056, 0, 0.309017, 1, 0, 0, 0.951056, 0, -0.309017, 0.809017, 0, -0.587785, 0.587785, 0, -0.809017, 0.309017, 0, -0.951056, 0, 0, -1, 0, 0.156434, -0.987688, -0.305213, 0.156434, -0.939347, -0.580549, 0.156434, -0.799057, -0.799057, 0.156434, -0.580549, -0.939348, 0.156434, -0.305212, -0.987688, 0.156434, 0, -0.939347, 0.156434, 0.305213, -0.799057, 0.156434, 0.580549, -0.580549, 0.156434, 0.799057, -0.305212, 0.156434, 0.939348, 0, 0.156434, 0.987688, 0.305212, 0.156434, 0.939348, 0.580549, 0.156434, 0.799057, 0.799057, 0.156434, 0.580548, 0.939347, 0.156434, 0.305213, 0.987688, 0.156434, 0, 0.939347, 0.156434, -0.305213, 0.799057, 0.156434, -0.580549, 0.580548, 0.156434, -0.799057, 0.305213, 0.156434, -0.939347, 0, 0.156434, -0.987688, 0, 0.309017, -0.951057, -0.293893, 0.309017, -0.904508, -0.559017, 0.309017, -0.769421, -0.769421, 0.309017, -0.559017, -0.904509, 0.309017, -0.293893, -0.951057, 0.309017, 0, -0.904508, 0.309017, 0.293893, -0.769421, 0.309017, 0.559017, -0.559017, 0.309017, 0.769421, -0.293892, 0.309017, 0.904509, 0, 0.309017, 0.951057, 0.293893, 0.309017, 0.904509, 0.559017, 0.309017, 0.769421, 0.769421, 0.309017, 0.559017, 0.904508, 0.309017, 0.293893, 0.951057, 0.309017, 0, 0.904508, 0.309017, -0.293893, 0.769421, 0.309017, -0.559017, 0.559017, 0.309017, -0.769421, 0.293893, 0.309017, -0.904508, 0, 0.309017, -0.951057, 0, 0.453991, -0.891007, -0.275336, 0.453991, -0.847398, -0.523720, 0.453991, -0.720840, -0.720839, 0.453991, -0.523720, -0.847398, 0.453991, -0.275336, -0.891007, 0.453991, 0, -0.847398, 0.453991, 0.275336, -0.720840, 0.453991, 0.523720, -0.523721, 0.453991, 0.720839, -0.275336, 0.453991, 0.847398, 0, 0.453991, 0.891007, 0.275336, 0.453991, 0.847398, 0.523721, 0.453991, 0.720839, 0.720840, 0.453991, 0.523720, 0.847398, 0.453991, 0.275336, 0.891007, 0.453991, 0, 0.847398, 0.453991, -0.275336, 0.720839, 0.453991, -0.523721, 0.523720, 0.453991, -0.720840, 0.275336, 0.453991, -0.847398, 0, 0.453991, -0.891007, 0, 0.587785, -0.809017, -0.250000, 0.587785, -0.769421, -0.475528, 0.587785, -0.654509, -0.654509, 0.587785, -0.475528, -0.769421, 0.587785, -0.250000, -0.809017, 0.587785, 0, -0.769421, 0.587785, 0.250000, -0.654509, 0.587785, 0.475528, -0.475528, 0.587785, 0.654509, -0.250000, 0.587785, 0.769421, 0, 0.587785, 0.809017, 0.250000, 0.587785, 0.769421, 0.475528, 0.587785, 0.654508, 0.654509, 0.587785, 0.475528, 0.769421, 0.587785, 0.250000, 0.809017, 0.587785, 0, 0.769421, 0.587785, -0.250000, 0.654508, 0.587785, -0.475528, 0.475528, 0.587785, -0.654509, 0.250000, 0.587785, -0.769421, 0, 0.587785, -0.809017, 0, 0.707107, -0.707107, -0.218508, 0.707107, -0.672498, -0.415627, 0.707107, -0.572061, -0.572061, 0.707107, -0.415627, -0.672499, 0.707107, -0.218508, -0.707107, 0.707107, 0, -0.672498, 0.707107, 0.218508, -0.572061, 0.707107, 0.415627, -0.415627, 0.707107, 0.572061, -0.218508, 0.707107, 0.672499, 0, 0.707107, 0.707107, 0.218508, 0.707107, 0.672499, 0.415627, 0.707107, 0.572061, 0.572061, 0.707107, 0.415627, 0.672498, 0.707107, 0.218508, 0.707107, 0.707107, 0, 0.672498, 0.707107, -0.218508, 0.572061, 0.707107, -0.415627, 0.415627, 0.707107, -0.572062, 0.218508, 0.707107, -0.672498, 0, 0.707107, -0.707107, 0, 0.809017, -0.587785, -0.181636, 0.809017, -0.559017, -0.345491, 0.809017, -0.475528, -0.475528, 0.809017, -0.345491, -0.559017, 0.809017, -0.181636, -0.587785, 0.809017, 0, -0.559017, 0.809017, 0.181636, -0.475528, 0.809017, 0.345491, -0.345491, 0.809017, 0.475528, -0.181636, 0.809017, 0.559017, 0, 0.809017, 0.587785, 0.181636, 0.809017, 0.559017, 0.345492, 0.809017, 0.475528, 0.475528, 0.809017, 0.345491, 0.559017, 0.809017, 0.181636, 0.587785, 0.809017, 0, 0.559017, 0.809017, -0.181636, 0.475528, 0.809017, -0.345492, 0.345491, 0.809017, -0.475528, 0.181636, 0.809017, -0.559017, 0, 0.809017, -0.587785, 0, 0.891007, -0.453991, -0.140291, 0.891007, -0.431771, -0.266849, 0.891007, -0.367286, -0.367286, 0.891007, -0.266849, -0.431771, 0.891007, -0.140291, -0.453991, 0.891007, 0, -0.431771, 0.891007, 0.140291, -0.367286, 0.891007, 0.266849, -0.266849, 0.891007, 0.367286, -0.140291, 0.891007, 0.431771, 0, 0.891007, 0.453991, 0.140291, 0.891007, 0.431771, 0.266849, 0.891007, 0.367286, 0.367286, 0.891007, 0.266849, 0.431771, 0.891007, 0.140291, 0.453991, 0.891007, 0, 0.431771, 0.891007, -0.140291, 0.367286, 0.891007, -0.266849, 0.266849, 0.891007, -0.367286, 0.140291, 0.891007, -0.431771, 0, 0.891007, -0.453991, 0, 0.951057, -0.309017, -0.095492, 0.951057, -0.293893, -0.181636, 0.951057, -0.250000, -0.250000, 0.951057, -0.181636, -0.293893, 0.951057, -0.095491, -0.309017, 0.951057, 0, -0.293893, 0.951057, 0.095492, -0.250000, 0.951057, 0.181636, -0.181636, 0.951057, 0.250000, -0.095491, 0.951057, 0.293893, 0, 0.951057, 0.309017, 0.095491, 0.951057, 0.293893, 0.181636, 0.951057, 0.250000, 0.250000, 0.951057, 0.181636, 0.293893, 0.951057, 0.095492, 0.309017, 0.951057, 0, 0.293893, 0.951057, -0.095492, 0.250000, 0.951057, -0.181636, 0.181636, 0.951057, -0.250000, 0.095492, 0.951057, -0.293893, 0, 0.951057, -0.309017, 0, 0.987688, -0.156434, -0.048341, 0.987688, -0.148778, -0.091950, 0.987688, -0.126558, -0.126558, 0.987688, -0.091950, -0.148778, 0.987688, -0.048341, -0.156434, 0.987688, 0, -0.148778, 0.987688, 0.048341, -0.126558, 0.987688, 0.091950, -0.091950, 0.987688, 0.126558, -0.048341, 0.987688, 0.148778, 0, 0.987688, 0.156434, 0.048341, 0.987688, 0.148778, 0.091950, 0.987688, 0.126558, 0.126558, 0.987688, 0.091950, 0.148778, 0.987688, 0.048341, 0.156434, 0.987688, 0, 0.148778, 0.987688, -0.048341, 0.126558, 0.987688, -0.091950, 0.091950, 0.987688, -0.126558, 0.048341, 0.987688, -0.148778, 0, 0.987688, -0.156434, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0 };
    Coordinate30_9.point = m_point;
    Coordinate30_9.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal33_5.m_parent = (X3dNode *)&IndexedFaceSet_60;
    {
    static float m_vector[] = { -0.155964, 0.077501, -0.984718, 0, 0.156434, -0.987688, -0.154556, 0.154576, -0.975817, -0.154556, 0.154576, -0.975817, 0, 0.309017, -0.951057, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.306374, 0.130498, -0.942924, -0.308063, 0.078459, -0.948125, -0.308063, 0.078459, -0.948125, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.582758, 0.130498, -0.802099, -0.585973, 0.078456, -0.806524, -0.585973, 0.078456, -0.806524, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.802099, 0.130497, -0.582759, -0.806524, 0.078455, -0.585973, -0.806524, 0.078455, -0.585973, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.942924, 0.130498, -0.306374, -0.948125, 0.078458, -0.308063, -0.948125, 0.078458, -0.308063, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.991448, 0.130499, 0.000001, -0.996917, 0.078460, 0.000001, -0.996917, 0.078460, 0.000001, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.942924, 0.130498, 0.306374, -0.948125, 0.078459, 0.308063, -0.948125, 0.078459, 0.308063, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.802099, 0.130498, 0.582758, -0.806524, 0.078456, 0.585973, -0.806524, 0.078456, 0.585973, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.582759, 0.130497, 0.802099, -0.585973, 0.078455, 0.806524, -0.585973, 0.078455, 0.806524, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.306374, 0.130498, 0.942924, -0.308063, 0.078458, 0.948125, -0.308063, 0.078458, 0.948125, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.130499, 0.991448, 0, 0.078460, 0.996917, 0, 0.078460, 0.996917, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.306374, 0.130499, 0.942924, 0.308063, 0.078461, 0.948125, 0.308063, 0.078461, 0.948125, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.582760, 0.130498, 0.802098, 0.585974, 0.078458, 0.806523, 0.585974, 0.078458, 0.806523, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.802099, 0.130498, 0.582758, 0.806524, 0.078457, 0.585972, 0.806524, 0.078457, 0.585972, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.942924, 0.130498, 0.306375, 0.948125, 0.078458, 0.308064, 0.948125, 0.078458, 0.308064, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.991449, 0.130499, 0, 0.996917, 0.078460, 0, 0.996917, 0.078460, 0, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.942924, 0.130498, -0.306374, 0.948125, 0.078459, -0.308063, 0.948125, 0.078459, -0.308063, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.802099, 0.130498, -0.582759, 0.806523, 0.078456, -0.585974, 0.806523, 0.078456, -0.585974, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.582758, 0.130498, -0.802099, 0.585972, 0.078457, -0.806524, 0.585972, 0.078457, -0.806524, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.306375, 0.130498, -0.942924, 0.308064, 0.078458, -0.948125, 0.308064, 0.078458, -0.948125, 0.086114, 0.165318, -0.982474, 0.087179, 0.161619, -0.982995, 0.086114, 0.165318, -0.982474, 0.086114, 0.165318, -0.982474, 0.078282, 0.117415, -0.989993, 0.078282, 0.117415, -0.989993, -0.148952, 0.305608, -0.940434, -0.148952, 0.305608, -0.940434, 0, 0.453991, -0.891006, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.296293, 0.284004, -0.911895, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.563582, 0.284007, -0.775703, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.775703, 0.284009, -0.563582, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.911894, 0.284008, -0.296293, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.958823, 0.284005, 0.000001, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.911895, 0.284004, 0.296293, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.775704, 0.284007, 0.563581, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.563582, 0.284008, 0.775703, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, -0.296293, 0.284008, 0.911894, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0, 0.284005, 0.958823, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.296293, 0.284004, 0.911895, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.563582, 0.284007, 0.775703, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.775704, 0.284007, 0.563581, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.911894, 0.284006, 0.296293, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.958823, 0.284004, 0, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.911895, 0.284004, -0.296293, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.775703, 0.284007, -0.563581, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.563581, 0.284007, -0.775704, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.296293, 0.284006, -0.911894, 0.083183, 0.317053, -0.944753, 0.084235, 0.313483, -0.945850, 0.083183, 0.317053, -0.944753, 0.083183, 0.317053, -0.944753, -0.139733, 0.449592, -0.882237, -0.139733, 0.449592, -0.882237, 0, 0.587785, -0.809017, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.278913, 0.430517, -0.858407, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.530525, 0.430516, -0.730205, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.730205, 0.430517, -0.530525, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.858406, 0.430518, -0.278914, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.902582, 0.430518, 0, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.858407, 0.430517, 0.278913, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.730205, 0.430516, 0.530525, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.530525, 0.430517, 0.730205, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, -0.278913, 0.430517, 0.858407, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0, 0.430518, 0.902582, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.278914, 0.430518, 0.858406, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.530525, 0.430518, 0.730204, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.730205, 0.430517, 0.530525, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.858408, 0.430515, 0.278913, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.902583, 0.430516, 0, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.858407, 0.430517, -0.278914, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.730205, 0.430516, -0.530525, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.530525, 0.430517, -0.730205, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.278913, 0.430515, -0.858408, 0.078226, 0.461120, -0.883883, 0.079240, 0.457769, -0.885533, 0.078226, 0.461120, -0.883883, 0.078226, 0.461120, -0.883883, -0.127091, 0.583078, -0.802414, -0.127091, 0.583078, -0.802414, 0, 0.707107, -0.707107, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.254666, 0.566422, -0.783781, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.484403, 0.566421, -0.666724, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.666725, 0.566420, -0.484404, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.783782, 0.566420, -0.254666, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.824116, 0.566421, 0.000001, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.783781, 0.566422, 0.254666, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.666724, 0.566421, 0.484403, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.484404, 0.566420, 0.666724, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, -0.254666, 0.566421, 0.783781, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0, 0.566421, 0.824116, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.254667, 0.566421, 0.783781, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.484404, 0.566421, 0.666724, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.666723, 0.566422, 0.484403, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.783781, 0.566421, 0.254666, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.824115, 0.566422, 0, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.783780, 0.566422, -0.254667, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.666724, 0.566422, -0.484402, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.484404, 0.566421, -0.666724, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.254665, 0.566421, -0.783781, 0.071352, 0.594008, -0.801289, 0.072303, 0.590963, -0.803452, 0.071352, 0.594008, -0.801289, 0.071352, 0.594008, -0.801289, -0.111291, 0.702768, -0.702661, -0.111291, 0.702768, -0.702661, 0, 0.809017, -0.587785, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.224150, 0.688368, -0.689859, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.426356, 0.688369, -0.586829, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.586828, 0.688370, -0.426357, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.689859, 0.688369, -0.224149, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.725362, 0.688368, 0.000001, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.689859, 0.688368, 0.224150, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.586829, 0.688369, 0.426356, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.426357, 0.688370, 0.586828, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, -0.224149, 0.688369, 0.689859, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0, 0.688368, 0.725362, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.224150, 0.688368, 0.689860, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.426359, 0.688367, 0.586830, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.586829, 0.688368, 0.426357, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.689858, 0.688370, 0.224149, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.725361, 0.688369, 0, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.689859, 0.688368, -0.224149, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.586830, 0.688368, -0.426357, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.426357, 0.688368, -0.586830, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.224149, 0.688369, -0.689859, 0.062712, 0.712441, -0.698925, 0.063577, 0.709781, -0.701548, 0.062712, 0.712441, -0.698925, 0.062712, 0.712441, -0.698925, -0.092686, 0.805577, -0.585197, -0.092686, 0.805577, -0.585197, 0, 0.891006, -0.453991, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.188115, 0.793363, -0.578954, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.357815, 0.793363, -0.492488, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.492487, 0.793364, -0.357814, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.578954, 0.793364, -0.188114, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.608748, 0.793364, 0, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.578954, 0.793363, 0.188115, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.492488, 0.793363, 0.357815, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.357814, 0.793364, 0.492487, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, -0.188114, 0.793364, 0.578954, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0, 0.793364, 0.608748, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.188114, 0.793364, 0.578954, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.357815, 0.793363, 0.492487, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.492488, 0.793363, 0.357814, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.578955, 0.793363, 0.188114, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.608749, 0.793363, 0, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.578955, 0.793363, -0.188114, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.492488, 0.793363, -0.357815, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.357813, 0.793364, -0.492487, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.188114, 0.793364, -0.578954, 0.052500, 0.813468, -0.579235, 0.053258, 0.811263, -0.582251, 0.052500, 0.813468, -0.579235, 0.052500, 0.813468, -0.579235, -0.071711, 0.888740, -0.452768, -0.071711, 0.888740, -0.452768, 0, 0.951057, -0.309017, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.147449, 0.878819, -0.453801, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.280465, 0.878819, -0.386026, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.386026, 0.878819, -0.280465, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.453801, 0.878819, -0.147449, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.477155, 0.878819, 0, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.453801, 0.878819, 0.147449, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.386026, 0.878819, 0.280465, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.280465, 0.878819, 0.386026, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, -0.147449, 0.878819, 0.453801, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0, 0.878819, 0.477155, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.147449, 0.878819, 0.453801, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.280465, 0.878820, 0.386026, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.386025, 0.878820, 0.280465, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.453801, 0.878819, 0.147449, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.477155, 0.878819, 0, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.453801, 0.878819, -0.147449, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.386025, 0.878820, -0.280465, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.280465, 0.878820, -0.386025, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.147449, 0.878819, -0.453801, 0.040961, 0.894535, -0.445117, 0.041592, 0.892842, -0.448446, 0.040961, 0.894535, -0.445117, 0.040961, 0.894535, -0.445117, -0.048876, 0.949935, -0.308601, -0.048876, 0.949935, -0.308601, 0, 0.987688, -0.156434, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.103153, 0.942640, -0.317474, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.196211, 0.942639, -0.270060, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.270060, 0.942639, -0.196210, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.317475, 0.942639, -0.103153, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.333812, 0.942640, 0, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.317474, 0.942640, 0.103153, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.270060, 0.942639, 0.196211, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.196210, 0.942639, 0.270060, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, -0.103153, 0.942639, 0.317475, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0, 0.942640, 0.333812, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.103153, 0.942640, 0.317474, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.196210, 0.942640, 0.270060, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.270060, 0.942639, 0.196211, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.317475, 0.942640, 0.103153, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.333812, 0.942640, 0, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.317474, 0.942640, -0.103153, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.270060, 0.942639, -0.196211, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.196211, 0.942639, -0.270060, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.103153, 0.942640, -0.317475, 0.028374, 0.953563, -0.299856, 0.028862, 0.952422, -0.303414, 0.028374, 0.953563, -0.299856, 0.028374, 0.953563, -0.299856, -0.024764, 0.987390, -0.156361, -0.024764, 0.987390, -0.156361, 0, 1, 0, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056272, 0.983251, -0.173354, -0.056467, 0.983251, -0.173291, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107088, 0.983251, -0.147481, -0.107254, 0.983251, -0.147360, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147420, 0.983251, -0.107171, -0.147541, 0.983251, -0.107005, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173323, 0.983251, -0.056369, -0.173386, 0.983251, -0.056174, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, -0.000051, -0.182259, 0.983251, 0.000154, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173354, 0.983251, 0.056272, -0.173291, 0.983251, 0.056467, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147481, 0.983251, 0.107088, -0.147360, 0.983251, 0.107254, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107171, 0.983251, 0.147420, -0.107005, 0.983251, 0.147541, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056369, 0.983251, 0.173323, -0.056174, 0.983251, 0.173386, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, -0.000051, 0.983251, 0.182259, 0.000154, 0.983251, 0.182259, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056272, 0.983251, 0.173354, 0.056467, 0.983251, 0.173291, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107088, 0.983251, 0.147481, 0.107254, 0.983250, 0.147360, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147420, 0.983251, 0.107171, 0.147541, 0.983251, 0.107005, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173323, 0.983251, 0.056369, 0.173386, 0.983251, 0.056174, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, 0.000051, 0.182259, 0.983251, -0.000154, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173354, 0.983251, -0.056272, 0.173291, 0.983251, -0.056467, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147481, 0.983251, -0.107088, 0.147360, 0.983251, -0.107254, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107171, 0.983251, -0.147420, 0.107005, 0.983251, -0.147541, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056369, 0.983251, -0.173323, 0.056174, 0.983251, -0.173386, 0.017077, 0.985694, -0.167675, 0.017084, 0.985673, -0.167799, 0.017077, 0.985694, -0.167675, 0, 1, 0, -0.012425, 0.996840, -0.078455, -0.036062, 0.996840, -0.070775, -0.056168, 0.996840, -0.056168, -0.070775, 0.996840, -0.036062, -0.078455, 0.996840, -0.012425, -0.078455, 0.996840, 0.012425, -0.070775, 0.996840, 0.036062, -0.056168, 0.996840, 0.056168, -0.036062, 0.996840, 0.070775, -0.012425, 0.996840, 0.078455, 0.012425, 0.996840, 0.078455, 0.036062, 0.996840, 0.070775, 0.056168, 0.996840, 0.056168, 0.070775, 0.996840, 0.036062, 0.078455, 0.996840, 0.012425, 0.078455, 0.996840, -0.012425, 0.070775, 0.996840, -0.036062, 0.056168, 0.996840, -0.056168, 0.036062, 0.996840, -0.070775, 0.012425, 0.996840, -0.078455, 0, 1, 0 };
    Normal33_5.vector = m_vector;
    Normal33_5.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate15_16.m_parent = (X3dNode *)&IndexedFaceSet_60;
    {
    static float m_point[] = { 0.500000, 0, 0.345491, 0.024472, 0.206107, 0.095491, 0.095491, 0.206107, 0.024472, 0.345492, 0, 0.500000, 0.024472, 0.654509, 0.095491, 0.793893, 0.206107, 0.904508, 0.345492, 0.975528, 0.500000, 1, 0.654508, 0.975528, 0.793893, 0.904508, 0.904509, 0.793893, 0.975528, 0.654509, 1, 0.500000, 0.975528, 0.345491, 0.904508, 0.206107, 0.793893, 0.095491, 0.654509, 0.024472, 0.500000, 0, 0.500000, 0.006156, 0.347394, 0.030326, 0.209726, 0.100472, 0.100472, 0.209726, 0.030326, 0.347394, 0.006156, 0.500000, 0.030326, 0.652606, 0.100472, 0.790274, 0.209726, 0.899528, 0.347394, 0.969674, 0.500000, 0.993844, 0.652606, 0.969674, 0.790274, 0.899528, 0.899528, 0.790274, 0.969674, 0.652606, 0.993844, 0.500000, 0.969674, 0.347394, 0.899528, 0.209726, 0.790274, 0.100472, 0.652606, 0.030326, 0.500000, 0.006156, 0.500000, 0.024472, 0.353054, 0.047746, 0.220492, 0.115290, 0.115290, 0.220492, 0.047746, 0.353054, 0.024472, 0.500000, 0.047746, 0.646946, 0.115290, 0.779508, 0.220491, 0.884710, 0.353054, 0.952254, 0.500000, 0.975528, 0.646946, 0.952254, 0.779509, 0.884710, 0.884711, 0.779508, 0.952254, 0.646946, 0.975528, 0.500000, 0.952254, 0.353054, 0.884710, 0.220491, 0.779508, 0.115289, 0.646946, 0.047746, 0.500000, 0.024472, 0.500000, 0.054497, 0.362332, 0.076301, 0.238140, 0.139580, 0.139580, 0.238140, 0.076301, 0.362332, 0.054497, 0.500000, 0.076301, 0.637668, 0.139580, 0.761860, 0.238140, 0.860420, 0.362332, 0.923699, 0.500000, 0.945503, 0.637668, 0.923699, 0.761860, 0.860420, 0.860420, 0.761860, 0.923699, 0.637668, 0.945503, 0.500000, 0.923699, 0.362332, 0.860420, 0.238140, 0.761860, 0.139580, 0.637668, 0.076301, 0.500000, 0.054497, 0.500000, 0.095491, 0.375000, 0.115290, 0.262236, 0.172746, 0.172746, 0.262236, 0.115290, 0.375000, 0.095491, 0.500000, 0.115290, 0.625000, 0.172746, 0.737764, 0.262236, 0.827254, 0.375000, 0.884710, 0.500000, 0.904508, 0.625000, 0.884710, 0.737764, 0.827254, 0.827254, 0.737764, 0.884710, 0.625000, 0.904508, 0.500000, 0.884710, 0.375000, 0.827254, 0.262236, 0.737764, 0.172746, 0.625000, 0.115290, 0.500000, 0.095491, 0.500000, 0.146447, 0.390746, 0.163751, 0.292187, 0.213969, 0.213969, 0.292187, 0.163751, 0.390746, 0.146447, 0.500000, 0.163751, 0.609254, 0.213969, 0.707813, 0.292187, 0.786031, 0.390746, 0.836249, 0.500000, 0.853553, 0.609254, 0.836249, 0.707814, 0.786031, 0.786031, 0.707813, 0.836249, 0.609254, 0.853553, 0.500000, 0.836249, 0.390746, 0.786031, 0.292186, 0.707813, 0.213969, 0.609254, 0.163751, 0.500000, 0.146447, 0.500000, 0.206107, 0.409182, 0.220492, 0.327254, 0.262236, 0.262236, 0.327254, 0.220491, 0.409182, 0.206107, 0.500000, 0.220492, 0.590818, 0.262236, 0.672746, 0.327254, 0.737764, 0.409182, 0.779509, 0.500000, 0.793893, 0.590818, 0.779508, 0.672746, 0.737764, 0.737764, 0.672746, 0.779508, 0.590818, 0.793893, 0.500000, 0.779508, 0.409182, 0.737764, 0.327254, 0.672746, 0.262236, 0.590818, 0.220492, 0.500000, 0.206107, 0.500000, 0.273005, 0.429855, 0.284115, 0.366576, 0.316357, 0.316357, 0.366576, 0.284115, 0.429855, 0.273005, 0.500000, 0.284115, 0.570145, 0.316357, 0.633424, 0.366576, 0.683643, 0.429855, 0.715885, 0.500000, 0.726995, 0.570145, 0.715885, 0.633424, 0.683643, 0.683643, 0.633424, 0.715885, 0.570145, 0.726995, 0.500000, 0.715885, 0.429855, 0.683643, 0.366575, 0.633424, 0.316357, 0.570145, 0.284115, 0.500000, 0.273005, 0.500000, 0.345492, 0.452254, 0.353054, 0.409182, 0.375000, 0.375000, 0.409182, 0.353054, 0.452254, 0.345492, 0.500000, 0.353054, 0.547746, 0.375000, 0.590818, 0.409182, 0.625000, 0.452254, 0.646946, 0.500000, 0.654508, 0.547746, 0.646946, 0.590818, 0.625000, 0.625000, 0.590818, 0.646946, 0.547746, 0.654508, 0.500000, 0.646946, 0.452254, 0.625000, 0.409182, 0.590818, 0.375000, 0.547746, 0.353054, 0.500000, 0.345492, 0.500000, 0.421783, 0.475830, 0.425611, 0.454025, 0.436721, 0.436721, 0.454025, 0.425611, 0.475830, 0.421783, 0.500000, 0.425611, 0.524170, 0.436721, 0.545975, 0.454025, 0.563279, 0.475830, 0.574389, 0.500000, 0.578217, 0.524170, 0.574389, 0.545975, 0.563279, 0.563279, 0.545975, 0.574389, 0.524170, 0.578217, 0.500000, 0.574389, 0.475830, 0.563279, 0.454025, 0.545975, 0.436721, 0.524170, 0.425611, 0.500000, 0.421783, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000, 0.500000 };
    TextureCoordinate15_16.point = m_point;
    TextureCoordinate15_16.point_length = sizeof(m_point) / sizeof(float);
    }
    IndexedFaceSet_60.color = NULL;
    IndexedFaceSet_60.coord = &Coordinate30_9;
    IndexedFaceSet_60.normal = &Normal33_5;
    IndexedFaceSet_60.texCoord = &TextureCoordinate15_16;
    IndexedFaceSet_60.ccw = false;
    {
    static int m_colorIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_60.colorIndex = m_colorIndex;
    IndexedFaceSet_60.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_60.colorPerVertex = true;
    IndexedFaceSet_60.convex = false;
    {
    static int m_coordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_60.coordIndex = m_coordIndex;
    IndexedFaceSet_60.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_60.creaseAngle = 3;
    {
    static int m_normalIndex[] = { 2, 5, 9, -1, 2, 9, 0, -1, 6, 11, 15, -1, 6, 15, 10, -1, 12, 17, 21, -1, 12, 21, 16, -1, 18, 23, 27, -1, 18, 27, 22, -1, 24, 29, 33, -1, 24, 33, 28, -1, 30, 35, 39, -1, 30, 39, 34, -1, 36, 41, 45, -1, 36, 45, 40, -1, 46, 42, 47, -1, 46, 47, 51, -1, 48, 53, 57, -1, 48, 57, 52, -1, 54, 59, 63, -1, 54, 63, 58, -1, 64, 60, 65, -1, 64, 65, 69, -1, 70, 66, 71, -1, 70, 71, 75, -1, 72, 77, 81, -1, 72, 81, 76, -1, 78, 83, 87, -1, 78, 87, 82, -1, 88, 84, 89, -1, 88, 89, 93, -1, 90, 95, 99, -1, 90, 99, 94, -1, 96, 101, 105, -1, 96, 105, 100, -1, 102, 107, 111, -1, 102, 111, 106, -1, 108, 113, 117, -1, 108, 117, 112, -1, 118, 114, 119, -1, 118, 119, 123, -1, 124, 120, 1, -1, 3, 125, 128, -1, 3, 128, 7, -1, 8, 129, 132, -1, 8, 132, 13, -1, 133, 136, 19, -1, 133, 19, 14, -1, 20, 137, 140, -1, 20, 140, 25, -1, 26, 141, 144, -1, 26, 144, 31, -1, 32, 145, 148, -1, 32, 148, 37, -1, 38, 149, 152, -1, 38, 152, 43, -1, 153, 156, 49, -1, 153, 49, 44, -1, 157, 160, 55, -1, 157, 55, 50, -1, 161, 164, 61, -1, 161, 61, 56, -1, 165, 168, 67, -1, 165, 67, 62, -1, 169, 172, 73, -1, 169, 73, 68, -1, 74, 173, 176, -1, 74, 176, 79, -1, 80, 177, 180, -1, 80, 180, 85, -1, 181, 184, 91, -1, 181, 91, 86, -1, 92, 185, 188, -1, 92, 188, 97, -1, 98, 189, 192, -1, 98, 192, 103, -1, 104, 193, 196, -1, 104, 196, 109, -1, 110, 197, 200, -1, 110, 200, 115, -1, 201, 204, 121, -1, 201, 121, 116, -1, 122, 205, 4, -1, 126, 208, 211, -1, 126, 211, 130, -1, 131, 212, 215, -1, 131, 215, 134, -1, 135, 216, 219, -1, 135, 219, 138, -1, 220, 223, 142, -1, 220, 142, 139, -1, 224, 227, 146, -1, 224, 146, 143, -1, 147, 228, 231, -1, 147, 231, 150, -1, 151, 232, 235, -1, 151, 235, 154, -1, 155, 236, 239, -1, 155, 239, 158, -1, 159, 240, 243, -1, 159, 243, 162, -1, 163, 244, 247, -1, 163, 247, 166, -1, 167, 248, 251, -1, 167, 251, 170, -1, 252, 255, 174, -1, 252, 174, 171, -1, 256, 259, 178, -1, 256, 178, 175, -1, 260, 263, 182, -1, 260, 182, 179, -1, 264, 267, 186, -1, 264, 186, 183, -1, 187, 268, 271, -1, 187, 271, 190, -1, 191, 272, 275, -1, 191, 275, 194, -1, 276, 279, 198, -1, 276, 198, 195, -1, 280, 283, 202, -1, 280, 202, 199, -1, 284, 287, 206, -1, 284, 206, 203, -1, 207, 288, 127, -1, 291, 294, 213, -1, 291, 213, 209, -1, 295, 298, 217, -1, 295, 217, 214, -1, 299, 302, 221, -1, 299, 221, 218, -1, 222, 303, 306, -1, 222, 306, 225, -1, 226, 307, 310, -1, 226, 310, 229, -1, 311, 314, 233, -1, 311, 233, 230, -1, 315, 318, 237, -1, 315, 237, 234, -1, 319, 322, 241, -1, 319, 241, 238, -1, 323, 326, 245, -1, 323, 245, 242, -1, 246, 327, 330, -1, 246, 330, 249, -1, 331, 334, 253, -1, 331, 253, 250, -1, 335, 338, 257, -1, 335, 257, 254, -1, 258, 339, 342, -1, 258, 342, 261, -1, 262, 343, 346, -1, 262, 346, 265, -1, 266, 347, 350, -1, 266, 350, 269, -1, 351, 354, 273, -1, 351, 273, 270, -1, 355, 358, 277, -1, 355, 277, 274, -1, 278, 359, 362, -1, 278, 362, 281, -1, 282, 363, 366, -1, 282, 366, 285, -1, 286, 367, 370, -1, 286, 370, 289, -1, 290, 371, 210, -1, 374, 377, 296, -1, 374, 296, 292, -1, 378, 381, 300, -1, 378, 300, 297, -1, 382, 385, 304, -1, 382, 304, 301, -1, 305, 386, 389, -1, 305, 389, 308, -1, 390, 393, 312, -1, 390, 312, 309, -1, 394, 397, 316, -1, 394, 316, 313, -1, 398, 401, 320, -1, 398, 320, 317, -1, 402, 405, 324, -1, 402, 324, 321, -1, 325, 406, 409, -1, 325, 409, 328, -1, 410, 413, 332, -1, 410, 332, 329, -1, 333, 414, 417, -1, 333, 417, 336, -1, 418, 421, 340, -1, 418, 340, 337, -1, 341, 422, 425, -1, 341, 425, 344, -1, 345, 426, 429, -1, 345, 429, 348, -1, 349, 430, 433, -1, 349, 433, 352, -1, 434, 437, 356, -1, 434, 356, 353, -1, 438, 441, 360, -1, 438, 360, 357, -1, 361, 442, 445, -1, 361, 445, 364, -1, 446, 449, 368, -1, 446, 368, 365, -1, 369, 450, 453, -1, 369, 453, 372, -1, 373, 454, 293, -1, 457, 460, 379, -1, 457, 379, 375, -1, 380, 461, 464, -1, 380, 464, 383, -1, 465, 468, 387, -1, 465, 387, 384, -1, 388, 469, 472, -1, 388, 472, 391, -1, 392, 473, 476, -1, 392, 476, 395, -1, 477, 480, 399, -1, 477, 399, 396, -1, 400, 481, 484, -1, 400, 484, 403, -1, 485, 488, 407, -1, 485, 407, 404, -1, 408, 489, 492, -1, 408, 492, 411, -1, 412, 493, 496, -1, 412, 496, 415, -1, 497, 500, 419, -1, 497, 419, 416, -1, 501, 504, 423, -1, 501, 423, 420, -1, 505, 508, 427, -1, 505, 427, 424, -1, 509, 512, 431, -1, 509, 431, 428, -1, 432, 513, 516, -1, 432, 516, 435, -1, 517, 520, 439, -1, 517, 439, 436, -1, 521, 524, 443, -1, 521, 443, 440, -1, 525, 528, 447, -1, 525, 447, 444, -1, 529, 532, 451, -1, 529, 451, 448, -1, 452, 533, 536, -1, 452, 536, 455, -1, 456, 537, 376, -1, 458, 540, 543, -1, 458, 543, 462, -1, 544, 547, 466, -1, 544, 466, 463, -1, 548, 551, 470, -1, 548, 470, 467, -1, 471, 552, 555, -1, 471, 555, 474, -1, 556, 559, 478, -1, 556, 478, 475, -1, 479, 560, 563, -1, 479, 563, 482, -1, 564, 567, 486, -1, 564, 486, 483, -1, 568, 571, 490, -1, 568, 490, 487, -1, 491, 572, 575, -1, 491, 575, 494, -1, 576, 579, 498, -1, 576, 498, 495, -1, 499, 580, 583, -1, 499, 583, 502, -1, 503, 584, 587, -1, 503, 587, 506, -1, 507, 588, 591, -1, 507, 591, 510, -1, 511, 592, 595, -1, 511, 595, 514, -1, 596, 599, 518, -1, 596, 518, 515, -1, 519, 600, 603, -1, 519, 603, 522, -1, 523, 604, 607, -1, 523, 607, 526, -1, 527, 608, 611, -1, 527, 611, 530, -1, 531, 612, 615, -1, 531, 615, 534, -1, 616, 619, 538, -1, 616, 538, 535, -1, 539, 620, 459, -1, 541, 623, 626, -1, 541, 626, 545, -1, 627, 630, 549, -1, 627, 549, 546, -1, 631, 634, 553, -1, 631, 553, 550, -1, 554, 635, 638, -1, 554, 638, 557, -1, 639, 642, 561, -1, 639, 561, 558, -1, 562, 643, 646, -1, 562, 646, 565, -1, 647, 650, 569, -1, 647, 569, 566, -1, 651, 654, 573, -1, 651, 573, 570, -1, 574, 655, 658, -1, 574, 658, 577, -1, 659, 662, 581, -1, 659, 581, 578, -1, 582, 663, 666, -1, 582, 666, 585, -1, 667, 670, 589, -1, 667, 589, 586, -1, 671, 674, 593, -1, 671, 593, 590, -1, 594, 675, 678, -1, 594, 678, 597, -1, 679, 682, 601, -1, 679, 601, 598, -1, 602, 683, 686, -1, 602, 686, 605, -1, 687, 690, 609, -1, 687, 609, 606, -1, 691, 694, 613, -1, 691, 613, 610, -1, 614, 695, 698, -1, 614, 698, 617, -1, 699, 702, 621, -1, 699, 621, 618, -1, 622, 703, 542, -1, 624, 706, 709, -1, 624, 709, 628, -1, 710, 713, 632, -1, 710, 632, 629, -1, 714, 717, 636, -1, 714, 636, 633, -1, 637, 718, 721, -1, 637, 721, 640, -1, 722, 725, 644, -1, 722, 644, 641, -1, 645, 726, 729, -1, 645, 729, 648, -1, 730, 733, 652, -1, 730, 652, 649, -1, 734, 737, 656, -1, 734, 656, 653, -1, 657, 738, 741, -1, 657, 741, 660, -1, 742, 745, 664, -1, 742, 664, 661, -1, 665, 746, 749, -1, 665, 749, 668, -1, 750, 753, 672, -1, 750, 672, 669, -1, 754, 757, 676, -1, 754, 676, 673, -1, 677, 758, 761, -1, 677, 761, 680, -1, 762, 765, 684, -1, 762, 684, 681, -1, 685, 766, 769, -1, 685, 769, 688, -1, 770, 773, 692, -1, 770, 692, 689, -1, 774, 777, 696, -1, 774, 696, 693, -1, 697, 778, 781, -1, 697, 781, 700, -1, 782, 785, 704, -1, 782, 704, 701, -1, 705, 786, 625, -1, 707, 789, 711, -1, 712, 790, 715, -1, 716, 791, 719, -1, 720, 792, 723, -1, 724, 793, 727, -1, 728, 794, 731, -1, 732, 795, 735, -1, 736, 796, 739, -1, 740, 797, 743, -1, 744, 798, 747, -1, 748, 799, 751, -1, 752, 800, 755, -1, 756, 801, 759, -1, 760, 802, 763, -1, 764, 803, 767, -1, 768, 804, 771, -1, 772, 805, 775, -1, 776, 806, 779, -1, 780, 807, 783, -1, 784, 808, 787, -1, 788, 809, 708, -1 };
    IndexedFaceSet_60.normalIndex = m_normalIndex;
    IndexedFaceSet_60.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_60.normalPerVertex = true;
    IndexedFaceSet_60.solid = false;
    {
    static int m_texCoordIndex[] = { 21, 22, 1, -1, 21, 1, 0, -1, 22, 23, 2, -1, 22, 2, 1, -1, 23, 24, 3, -1, 23, 3, 2, -1, 24, 25, 4, -1, 24, 4, 3, -1, 25, 26, 5, -1, 25, 5, 4, -1, 26, 27, 6, -1, 26, 6, 5, -1, 27, 28, 7, -1, 27, 7, 6, -1, 7, 28, 29, -1, 7, 29, 8, -1, 29, 30, 9, -1, 29, 9, 8, -1, 30, 31, 10, -1, 30, 10, 9, -1, 10, 31, 32, -1, 10, 32, 11, -1, 11, 32, 33, -1, 11, 33, 12, -1, 33, 34, 13, -1, 33, 13, 12, -1, 34, 35, 14, -1, 34, 14, 13, -1, 14, 35, 36, -1, 14, 36, 15, -1, 36, 37, 16, -1, 36, 16, 15, -1, 37, 38, 17, -1, 37, 17, 16, -1, 38, 39, 18, -1, 38, 18, 17, -1, 39, 40, 19, -1, 39, 19, 18, -1, 19, 40, 41, -1, 19, 41, 20, -1, 20, 41, 0, -1, 21, 42, 43, -1, 21, 43, 22, -1, 22, 43, 44, -1, 22, 44, 23, -1, 44, 45, 24, -1, 44, 24, 23, -1, 24, 45, 46, -1, 24, 46, 25, -1, 25, 46, 47, -1, 25, 47, 26, -1, 26, 47, 48, -1, 26, 48, 27, -1, 27, 48, 49, -1, 27, 49, 28, -1, 49, 50, 29, -1, 49, 29, 28, -1, 50, 51, 30, -1, 50, 30, 29, -1, 51, 52, 31, -1, 51, 31, 30, -1, 52, 53, 32, -1, 52, 32, 31, -1, 53, 54, 33, -1, 53, 33, 32, -1, 33, 54, 55, -1, 33, 55, 34, -1, 34, 55, 56, -1, 34, 56, 35, -1, 56, 57, 36, -1, 56, 36, 35, -1, 36, 57, 58, -1, 36, 58, 37, -1, 37, 58, 59, -1, 37, 59, 38, -1, 38, 59, 60, -1, 38, 60, 39, -1, 39, 60, 61, -1, 39, 61, 40, -1, 61, 62, 41, -1, 61, 41, 40, -1, 41, 62, 21, -1, 42, 63, 64, -1, 42, 64, 43, -1, 43, 64, 65, -1, 43, 65, 44, -1, 44, 65, 66, -1, 44, 66, 45, -1, 66, 67, 46, -1, 66, 46, 45, -1, 67, 68, 47, -1, 67, 47, 46, -1, 47, 68, 69, -1, 47, 69, 48, -1, 48, 69, 70, -1, 48, 70, 49, -1, 49, 70, 71, -1, 49, 71, 50, -1, 50, 71, 72, -1, 50, 72, 51, -1, 51, 72, 73, -1, 51, 73, 52, -1, 52, 73, 74, -1, 52, 74, 53, -1, 74, 75, 54, -1, 74, 54, 53, -1, 75, 76, 55, -1, 75, 55, 54, -1, 76, 77, 56, -1, 76, 56, 55, -1, 77, 78, 57, -1, 77, 57, 56, -1, 57, 78, 79, -1, 57, 79, 58, -1, 58, 79, 80, -1, 58, 80, 59, -1, 80, 81, 60, -1, 80, 60, 59, -1, 81, 82, 61, -1, 81, 61, 60, -1, 82, 83, 62, -1, 82, 62, 61, -1, 62, 83, 42, -1, 84, 85, 64, -1, 84, 64, 63, -1, 85, 86, 65, -1, 85, 65, 64, -1, 86, 87, 66, -1, 86, 66, 65, -1, 66, 87, 88, -1, 66, 88, 67, -1, 67, 88, 89, -1, 67, 89, 68, -1, 89, 90, 69, -1, 89, 69, 68, -1, 90, 91, 70, -1, 90, 70, 69, -1, 91, 92, 71, -1, 91, 71, 70, -1, 92, 93, 72, -1, 92, 72, 71, -1, 72, 93, 94, -1, 72, 94, 73, -1, 94, 95, 74, -1, 94, 74, 73, -1, 95, 96, 75, -1, 95, 75, 74, -1, 75, 96, 97, -1, 75, 97, 76, -1, 76, 97, 98, -1, 76, 98, 77, -1, 77, 98, 99, -1, 77, 99, 78, -1, 99, 100, 79, -1, 99, 79, 78, -1, 100, 101, 80, -1, 100, 80, 79, -1, 80, 101, 102, -1, 80, 102, 81, -1, 81, 102, 103, -1, 81, 103, 82, -1, 82, 103, 104, -1, 82, 104, 83, -1, 83, 104, 63, -1, 105, 106, 85, -1, 105, 85, 84, -1, 106, 107, 86, -1, 106, 86, 85, -1, 107, 108, 87, -1, 107, 87, 86, -1, 87, 108, 109, -1, 87, 109, 88, -1, 109, 110, 89, -1, 109, 89, 88, -1, 110, 111, 90, -1, 110, 90, 89, -1, 111, 112, 91, -1, 111, 91, 90, -1, 112, 113, 92, -1, 112, 92, 91, -1, 92, 113, 114, -1, 92, 114, 93, -1, 114, 115, 94, -1, 114, 94, 93, -1, 94, 115, 116, -1, 94, 116, 95, -1, 116, 117, 96, -1, 116, 96, 95, -1, 96, 117, 118, -1, 96, 118, 97, -1, 97, 118, 119, -1, 97, 119, 98, -1, 98, 119, 120, -1, 98, 120, 99, -1, 120, 121, 100, -1, 120, 100, 99, -1, 121, 122, 101, -1, 121, 101, 100, -1, 101, 122, 123, -1, 101, 123, 102, -1, 123, 124, 103, -1, 123, 103, 102, -1, 103, 124, 125, -1, 103, 125, 104, -1, 104, 125, 84, -1, 126, 127, 106, -1, 126, 106, 105, -1, 106, 127, 128, -1, 106, 128, 107, -1, 128, 129, 108, -1, 128, 108, 107, -1, 108, 129, 130, -1, 108, 130, 109, -1, 109, 130, 131, -1, 109, 131, 110, -1, 131, 132, 111, -1, 131, 111, 110, -1, 111, 132, 133, -1, 111, 133, 112, -1, 133, 134, 113, -1, 133, 113, 112, -1, 113, 134, 135, -1, 113, 135, 114, -1, 114, 135, 136, -1, 114, 136, 115, -1, 136, 137, 116, -1, 136, 116, 115, -1, 137, 138, 117, -1, 137, 117, 116, -1, 138, 139, 118, -1, 138, 118, 117, -1, 139, 140, 119, -1, 139, 119, 118, -1, 119, 140, 141, -1, 119, 141, 120, -1, 141, 142, 121, -1, 141, 121, 120, -1, 142, 143, 122, -1, 142, 122, 121, -1, 143, 144, 123, -1, 143, 123, 122, -1, 144, 145, 124, -1, 144, 124, 123, -1, 124, 145, 146, -1, 124, 146, 125, -1, 125, 146, 105, -1, 126, 147, 148, -1, 126, 148, 127, -1, 148, 149, 128, -1, 148, 128, 127, -1, 149, 150, 129, -1, 149, 129, 128, -1, 129, 150, 151, -1, 129, 151, 130, -1, 151, 152, 131, -1, 151, 131, 130, -1, 131, 152, 153, -1, 131, 153, 132, -1, 153, 154, 133, -1, 153, 133, 132, -1, 154, 155, 134, -1, 154, 134, 133, -1, 134, 155, 156, -1, 134, 156, 135, -1, 156, 157, 136, -1, 156, 136, 135, -1, 136, 157, 158, -1, 136, 158, 137, -1, 137, 158, 159, -1, 137, 159, 138, -1, 138, 159, 160, -1, 138, 160, 139, -1, 139, 160, 161, -1, 139, 161, 140, -1, 161, 162, 141, -1, 161, 141, 140, -1, 141, 162, 163, -1, 141, 163, 142, -1, 142, 163, 164, -1, 142, 164, 143, -1, 143, 164, 165, -1, 143, 165, 144, -1, 144, 165, 166, -1, 144, 166, 145, -1, 166, 167, 146, -1, 166, 146, 145, -1, 146, 167, 126, -1, 147, 168, 169, -1, 147, 169, 148, -1, 169, 170, 149, -1, 169, 149, 148, -1, 170, 171, 150, -1, 170, 150, 149, -1, 150, 171, 172, -1, 150, 172, 151, -1, 172, 173, 152, -1, 172, 152, 151, -1, 152, 173, 174, -1, 152, 174, 153, -1, 174, 175, 154, -1, 174, 154, 153, -1, 175, 176, 155, -1, 175, 155, 154, -1, 155, 176, 177, -1, 155, 177, 156, -1, 177, 178, 157, -1, 177, 157, 156, -1, 157, 178, 179, -1, 157, 179, 158, -1, 179, 180, 159, -1, 179, 159, 158, -1, 180, 181, 160, -1, 180, 160, 159, -1, 160, 181, 182, -1, 160, 182, 161, -1, 182, 183, 162, -1, 182, 162, 161, -1, 162, 183, 184, -1, 162, 184, 163, -1, 184, 185, 164, -1, 184, 164, 163, -1, 185, 186, 165, -1, 185, 165, 164, -1, 165, 186, 187, -1, 165, 187, 166, -1, 187, 188, 167, -1, 187, 167, 166, -1, 167, 188, 147, -1, 168, 189, 190, -1, 168, 190, 169, -1, 190, 191, 170, -1, 190, 170, 169, -1, 191, 192, 171, -1, 191, 171, 170, -1, 171, 192, 193, -1, 171, 193, 172, -1, 193, 194, 173, -1, 193, 173, 172, -1, 173, 194, 195, -1, 173, 195, 174, -1, 195, 196, 175, -1, 195, 175, 174, -1, 196, 197, 176, -1, 196, 176, 175, -1, 176, 197, 198, -1, 176, 198, 177, -1, 198, 199, 178, -1, 198, 178, 177, -1, 178, 199, 200, -1, 178, 200, 179, -1, 200, 201, 180, -1, 200, 180, 179, -1, 201, 202, 181, -1, 201, 181, 180, -1, 181, 202, 203, -1, 181, 203, 182, -1, 203, 204, 183, -1, 203, 183, 182, -1, 183, 204, 205, -1, 183, 205, 184, -1, 205, 206, 185, -1, 205, 185, 184, -1, 206, 207, 186, -1, 206, 186, 185, -1, 186, 207, 208, -1, 186, 208, 187, -1, 208, 209, 188, -1, 208, 188, 187, -1, 188, 209, 168, -1, 189, 210, 190, -1, 190, 211, 191, -1, 191, 212, 192, -1, 192, 213, 193, -1, 193, 214, 194, -1, 194, 215, 195, -1, 195, 216, 196, -1, 196, 217, 197, -1, 197, 218, 198, -1, 198, 219, 199, -1, 199, 220, 200, -1, 200, 221, 201, -1, 201, 222, 202, -1, 202, 223, 203, -1, 203, 224, 204, -1, 204, 225, 205, -1, 205, 226, 206, -1, 206, 227, 207, -1, 207, 228, 208, -1, 208, 229, 209, -1, 209, 230, 189, -1 };
    IndexedFaceSet_60.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_60.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_60.set_colorIndex = NULL;
    IndexedFaceSet_60.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_60.set_coordIndex = NULL;
    IndexedFaceSet_60.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_60.set_normalIndex = NULL;
    IndexedFaceSet_60.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_60.set_texCoordIndex = NULL;
    IndexedFaceSet_60.set_texCoordIndex_length = 0;
    }
    Shape_283.appearance = &Appearance_284;
    Shape_283.geometry = &IndexedFaceSet_60;
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, 0 };
    Transform_282.rotation = m_rotation;
    Transform_282.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { -0.220000, 0.440000, 0.160000 };
    Transform_282.translation = m_translation;
    Transform_282.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_282.bboxCenter = m_bboxCenter;
    Transform_282.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_282.bboxSize = m_bboxSize;
    Transform_282.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_282.center = m_center;
    Transform_282.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.700000, 0.700000, 0.700000 };
    Transform_282.scale = m_scale;
    Transform_282.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_282.scaleOrientation = m_scaleOrientation;
    Transform_282.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_283 };
    Transform_282.children = m_children;
    Transform_282.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_282.addChildren = NULL;
    Transform_282.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_282.removeChildren = NULL;
    Transform_282.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.991215, -0.104605, -8.093859e-2, -1.578764 };
    Transform_275.rotation = m_rotation;
    Transform_275.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_275.translation = m_translation;
    Transform_275.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_275.bboxCenter = m_bboxCenter;
    Transform_275.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_275.bboxSize = m_bboxSize;
    Transform_275.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_275.center = m_center;
    Transform_275.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_275.scale = m_scale;
    Transform_275.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_275.scaleOrientation = m_scaleOrientation;
    Transform_275.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_276, &Transform_282 };
    Transform_275.children = m_children;
    Transform_275.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_275.addChildren = NULL;
    Transform_275.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_275.removeChildren = NULL;
    Transform_275.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { -0.565552, -0.792356, -0.228743, -0.826199 };
    Transform_274.rotation = m_rotation;
    Transform_274.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0.109765, 0.715337, 1.294240 };
    Transform_274.translation = m_translation;
    Transform_274.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_274.bboxCenter = m_bboxCenter;
    Transform_274.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_274.bboxSize = m_bboxSize;
    Transform_274.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_274.center = m_center;
    Transform_274.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 0.156146, 0.192396, 7.336500e-2 };
    Transform_274.scale = m_scale;
    Transform_274.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_274.scaleOrientation = m_scaleOrientation;
    Transform_274.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_275 };
    Transform_274.children = m_children;
    Transform_274.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_274.addChildren = NULL;
    Transform_274.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_274.removeChildren = NULL;
    Transform_274.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_258.rotation = m_rotation;
    Transform_258.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0.120000 };
    Transform_258.translation = m_translation;
    Transform_258.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_258.bboxCenter = m_bboxCenter;
    Transform_258.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_258.bboxSize = m_bboxSize;
    Transform_258.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_258.center = m_center;
    Transform_258.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_258.scale = m_scale;
    Transform_258.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_258.scaleOrientation = m_scaleOrientation;
    Transform_258.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_259, &Transform_274 };
    Transform_258.children = m_children;
    Transform_258.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_258.addChildren = NULL;
    Transform_258.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_258.removeChildren = NULL;
    Transform_258.removeChildren_length = 0;
    }
    Inline_67.m_parent = (X3dNode *)NULL;
    Inline_63.m_parent = (X3dNode *)NULL;
    Group10.m_parent = (X3dNode *)&Inline_63;
    Transform_289.route_sources =  NULL;
    Transform_289.num_route_source = 0;
    Transform_289.m_parent = (X3dNode *)NULL;
    CORINTm_7.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_7.key = m_key;
    CORINTm_7.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.619297, 0.959445, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.779760, 0.966057, -0.506980, 1.645980, 0.964696, -0.929632, 2.257560, 1.170100, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.900248, -0.506980, 1.593330, 0.898887, -0.929632, 2.269410, 1.064810, -0.674169, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.892521, -0.981481, 1.792920, 0.742307, -0.506980, 1.711780, 0.596166, -0.929632, 2.282570, 0.354070, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.833370, -0.294626, 0.986904, 0.813551, -0.981481, 1.699740, 0.578596, -0.506980, 1.632810, 0.416126, -0.929632, 2.295730, 0.196128, -0.779463, 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.807047, -0.294626, 1.000070, 0.760904, -0.981481, 1.727110, 0.439586, -0.506980, 1.620710, 0.340811, -1.021760, 2.216760, -0.132916, -0.976890, 0.633611, 0.893636, -0.216577, 0.516650, 0.893636, -0.903431, 1.076250, 0.780723, -0.294626, 0.950070, 0.734580, -0.981481, 1.650790, 0.307968, -0.506980, 1.530170, 0.280283, -1.021760, 2.021980, -0.277696, -0.976890 };
    CORINTm_7.keyValue = m_keyValue;
    CORINTm_7.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_7.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_7.value_changed = NULL;
    CORINTm_7.value_changed_length = 0;
    }
    TIMERm_6.m_parent = (X3dNode *)NULL;
    TIMERm_6.cycleInterval = 2;
    TIMERm_6.enabled = true;
    TIMERm_6.loop = true;
    TIMERm_6.startTime = 0;
    TIMERm_6.stopTime = 0;
    TIMERm_6.cycleTime = 0;
    TIMERm_6.fraction_changed = 0;
    TIMERm_6.isActive = false;
    TIMERm_6.time = 0;
    Group_290.m_parent = (X3dNode *)NULL;
    Background_291.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background_291.groundAngle = NULL;
    Background_291.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background_291.groundColor = m_groundColor;
    Background_291.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background_291.backUrl = NULL;
    Background_291.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background_291.rightUrl = NULL;
    Background_291.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background_291.frontUrl = NULL;
    Background_291.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background_291.leftUrl = NULL;
    Background_291.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background_291.topUrl = NULL;
    Background_291.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background_291.bottomUrl = NULL;
    Background_291.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background_291.skyAngle = NULL;
    Background_291.skyAngle_length = 0;
    }
    Background_291.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background_291.skyColor = m_skyColor;
    Background_291.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background_291.set_bind = false;
    Background_291.isBound = false;
    Shape_292.m_parent = (X3dNode *)NULL;
    Appearance_293.m_parent = (X3dNode *)NULL;
    Material_294.m_parent = (X3dNode *)NULL;
    Material_294.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_294.diffuseColor = m_diffuseColor;
    Material_294.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_294.emissiveColor = m_emissiveColor;
    Material_294.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_294.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_294.specularColor = m_specularColor;
    Material_294.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_294.transparency = 6.382980e-2;
    Appearance_293.material = &Material_294;
    Appearance_293.texture = NULL;
    Appearance_293.textureTransform = NULL;
    IndexedFaceSet_61.setGlName(34);
    IndexedFaceSet_61.m_parent = (X3dNode *)NULL;
    COORD1_12.m_parent = (X3dNode *)&IndexedFaceSet_61;
    {
    static float m_point[] = { 0.683611, 0.893636, -0.216577, 0.566650, 0.893636, -0.903431, 1.126250, 0.780723, -0.294626, 1.000070, 0.734580, -0.981481, 1.700790, 0.307968, -0.506980, 1.580170, 0.280283, -1.021760, 2.071980, -0.277696, -0.976890 };
    COORD1_12.point = m_point;
    COORD1_12.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal34.m_parent = (X3dNode *)&IndexedFaceSet_61;
    {
    static float m_vector[] = { 0.358806, 0.927849, -0.101757, 0.358806, 0.927849, -0.101757, 0.359718, 0.927470, -0.101994, 0.281824, 0.955978, -0.081744, 0.281824, 0.955978, -0.081744, 0.240191, 0.969864, -0.040901, 0.495155, 0.855218, -0.153047, 0.496190, 0.854566, -0.153340, 0.496190, 0.854566, -0.153340, 0.631845, 0.753624, -0.181169, 0.631845, 0.753624, -0.181169, 0.632281, 0.753233, -0.181277, 0.673713, 0.712478, -0.196177, 0.673713, 0.712478, -0.196177, 0.742001, 0.637272, -0.208134 };
    Normal34.vector = m_vector;
    Normal34.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_61.color = NULL;
    IndexedFaceSet_61.coord = &COORD1_12;
    IndexedFaceSet_61.normal = &Normal34;
    IndexedFaceSet_61.texCoord = NULL;
    IndexedFaceSet_61.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_61.colorIndex = m_colorIndex;
    IndexedFaceSet_61.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_61.colorPerVertex = true;
    IndexedFaceSet_61.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_61.coordIndex = m_coordIndex;
    IndexedFaceSet_61.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_61.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet_61.normalIndex = m_normalIndex;
    IndexedFaceSet_61.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_61.normalPerVertex = true;
    IndexedFaceSet_61.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_61.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_61.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_61.set_colorIndex = NULL;
    IndexedFaceSet_61.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_61.set_coordIndex = NULL;
    IndexedFaceSet_61.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_61.set_normalIndex = NULL;
    IndexedFaceSet_61.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_61.set_texCoordIndex = NULL;
    IndexedFaceSet_61.set_texCoordIndex_length = 0;
    }
    Shape_292.appearance = &Appearance_293;
    Shape_292.geometry = &IndexedFaceSet_61;
    {
    static X3dNode* m_children[] = { &Background_291, &Shape_292 };
    Group_290.children = m_children;
    Group_290.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_290.bboxCenter = m_bboxCenter;
    Group_290.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_290.bboxSize = m_bboxSize;
    Group_290.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_290.addChildren = NULL;
    Group_290.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_290.removeChildren = NULL;
    Group_290.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_289.rotation = m_rotation;
    Transform_289.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_289.translation = m_translation;
    Transform_289.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_289.bboxCenter = m_bboxCenter;
    Transform_289.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_289.bboxSize = m_bboxSize;
    Transform_289.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_289.center = m_center;
    Transform_289.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_289.scale = m_scale;
    Transform_289.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_289.scaleOrientation = m_scaleOrientation;
    Transform_289.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_7, &TIMERm_6, &Group_290 };
    Transform_289.children = m_children;
    Transform_289.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_289.addChildren = NULL;
    Transform_289.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_289.removeChildren = NULL;
    Transform_289.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_289 };
    Group10.children = m_children;
    Group10.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group10.bboxCenter = m_bboxCenter;
    Group10.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group10.bboxSize = m_bboxSize;
    Group10.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group10.addChildren = NULL;
    Group10.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group10.removeChildren = NULL;
    Group10.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group10 };
    Inline_63.children = m_children;
    Inline_63.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_63.bboxCenter = m_bboxCenter;
    Inline_63.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_63.bboxSize = m_bboxSize;
    Inline_63.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_63.addChildren = NULL;
    Inline_63.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_63.removeChildren = NULL;
    Inline_63.removeChildren_length = 0;
    }
    Inline_66.m_parent = (X3dNode *)NULL;
    Group11.m_parent = (X3dNode *)&Inline_66;
    Transform_295.route_sources =  NULL;
    Transform_295.num_route_source = 0;
    Transform_295.m_parent = (X3dNode *)NULL;
    CORINTm_8.m_parent = (X3dNode *)NULL;
    {
    static float m_key[] = { 0, 0.150000, 0.300000, 0.450000, 0.600000, 0.750000, 0.800000, 0.850000, 0.900000, 0.950000, 1 };
    CORINTm_8.key = m_key;
    CORINTm_8.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.619297, 0.959445, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.779760, 0.966057, -0.506980, -1.645980, 0.964696, -0.929632, -2.257560, 1.170100, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.900248, -0.506980, -1.593330, 0.898887, -0.929632, -2.269410, 1.064810, -0.674169, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.892521, -0.981481, -1.792920, 0.742307, -0.506980, -1.711780, 0.596166, -0.929632, -2.282570, 0.354070, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.833370, -0.294626, -0.986904, 0.813551, -0.981481, -1.699740, 0.578596, -0.506980, -1.632810, 0.416126, -0.929632, -2.295730, 0.196128, -0.779463, -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.807047, -0.294626, -1.000070, 0.760904, -0.981481, -1.727110, 0.439586, -0.506980, -1.620710, 0.340811, -1.021760, -2.216760, -0.132916, -0.976890, -0.633611, 0.893636, -0.216577, -0.516650, 0.893636, -0.903431, -1.076250, 0.780723, -0.294626, -0.950070, 0.734580, -0.981481, -1.650790, 0.307968, -0.506980, -1.530170, 0.280283, -1.021760, -2.021980, -0.277696, -0.976890 };
    CORINTm_8.keyValue = m_keyValue;
    CORINTm_8.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    CORINTm_8.set_fraction = 0;
    {
    static float m_value_changed[1];
    CORINTm_8.value_changed = NULL;
    CORINTm_8.value_changed_length = 0;
    }
    TIMERm_7.m_parent = (X3dNode *)NULL;
    TIMERm_7.cycleInterval = 2;
    TIMERm_7.enabled = true;
    TIMERm_7.loop = true;
    TIMERm_7.startTime = 0;
    TIMERm_7.stopTime = 0;
    TIMERm_7.cycleTime = 0;
    TIMERm_7.fraction_changed = 0;
    TIMERm_7.isActive = false;
    TIMERm_7.time = 0;
    Group_296.m_parent = (X3dNode *)NULL;
    Background_297.m_parent = (X3dNode *)NULL;
    {
    static float m_groundAngle[1];
    Background_297.groundAngle = NULL;
    Background_297.groundAngle_length = 0;
    }
    {
    static float m_groundColor[] = { 0, 0, 1 };
    Background_297.groundColor = m_groundColor;
    Background_297.groundColor_length = sizeof(m_groundColor) / sizeof(float);
    }
    {
    static const char* m_backUrl[1];
    Background_297.backUrl = NULL;
    Background_297.backUrl_length = 0;
    }
    {
    static const char* m_rightUrl[1];
    Background_297.rightUrl = NULL;
    Background_297.rightUrl_length = 0;
    }
    {
    static const char* m_frontUrl[1];
    Background_297.frontUrl = NULL;
    Background_297.frontUrl_length = 0;
    }
    {
    static const char* m_leftUrl[1];
    Background_297.leftUrl = NULL;
    Background_297.leftUrl_length = 0;
    }
    {
    static const char* m_topUrl[1];
    Background_297.topUrl = NULL;
    Background_297.topUrl_length = 0;
    }
    {
    static const char* m_bottomUrl[1];
    Background_297.bottomUrl = NULL;
    Background_297.bottomUrl_length = 0;
    }
    {
    static float m_skyAngle[1];
    Background_297.skyAngle = NULL;
    Background_297.skyAngle_length = 0;
    }
    Background_297.transparency = 0;
    {
    static float m_skyColor[] = { 0, 0, 1 };
    Background_297.skyColor = m_skyColor;
    Background_297.skyColor_length = sizeof(m_skyColor) / sizeof(float);
    }
    Background_297.set_bind = false;
    Background_297.isBound = false;
    Shape_298.m_parent = (X3dNode *)NULL;
    Appearance_299.m_parent = (X3dNode *)NULL;
    Material_300.m_parent = (X3dNode *)NULL;
    Material_300.ambientIntensity = 0;
    {
    static float m_diffuseColor[] = { 0, 0, 0 };
    Material_300.diffuseColor = m_diffuseColor;
    Material_300.diffuseColor_length = sizeof(m_diffuseColor) / sizeof(float);
    }
    {
    static float m_emissiveColor[] = { 0, 0, 0 };
    Material_300.emissiveColor = m_emissiveColor;
    Material_300.emissiveColor_length = sizeof(m_emissiveColor) / sizeof(float);
    }
    Material_300.shininess = 8.510640e-2;
    {
    static float m_specularColor[] = { 0.221631, 0.265957, 0.265957 };
    Material_300.specularColor = m_specularColor;
    Material_300.specularColor_length = sizeof(m_specularColor) / sizeof(float);
    }
    Material_300.transparency = 6.382980e-2;
    Appearance_299.material = &Material_300;
    Appearance_299.texture = NULL;
    Appearance_299.textureTransform = NULL;
    IndexedFaceSet_64.setGlName(35);
    IndexedFaceSet_64.m_parent = (X3dNode *)NULL;
    COORD1_15.m_parent = (X3dNode *)&IndexedFaceSet_64;
    {
    static float m_point[] = { -0.683611, 0.893636, -0.216577, -0.566650, 0.893636, -0.903431, -1.126250, 0.780723, -0.294626, -1.000070, 0.734580, -0.981481, -1.700790, 0.307968, -0.506980, -1.580170, 0.280283, -1.021760, -2.071980, -0.277696, -0.976890 };
    COORD1_15.point = m_point;
    COORD1_15.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal35.m_parent = (X3dNode *)&IndexedFaceSet_64;
    {
    static float m_vector[] = { 0.358806, -0.927849, 0.101757, 0.358806, -0.927849, 0.101757, 0.359718, -0.927470, 0.101994, 0.281824, -0.955978, 0.081744, 0.281824, -0.955978, 0.081744, 0.240191, -0.969864, 0.040901, 0.495155, -0.855218, 0.153047, 0.496190, -0.854566, 0.153340, 0.496190, -0.854566, 0.153340, 0.631845, -0.753624, 0.181169, 0.631845, -0.753624, 0.181169, 0.632281, -0.753233, 0.181277, 0.673713, -0.712478, 0.196177, 0.673713, -0.712478, 0.196177, 0.742001, -0.637272, 0.208134 };
    Normal35.vector = m_vector;
    Normal35.vector_length = sizeof(m_vector) / sizeof(float);
    }
    IndexedFaceSet_64.color = NULL;
    IndexedFaceSet_64.coord = &COORD1_15;
    IndexedFaceSet_64.normal = &Normal35;
    IndexedFaceSet_64.texCoord = NULL;
    IndexedFaceSet_64.ccw = true;
    {
    static int m_colorIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_64.colorIndex = m_colorIndex;
    IndexedFaceSet_64.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    IndexedFaceSet_64.colorPerVertex = true;
    IndexedFaceSet_64.convex = true;
    {
    static int m_coordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_64.coordIndex = m_coordIndex;
    IndexedFaceSet_64.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    IndexedFaceSet_64.creaseAngle = 1;
    {
    static int m_normalIndex[] = { 0, 3, 5, -1, 4, 1, 6, -1, 9, 7, 2, -1, 8, 10, 12, -1, 14, 13, 11, -1 };
    IndexedFaceSet_64.normalIndex = m_normalIndex;
    IndexedFaceSet_64.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    IndexedFaceSet_64.normalPerVertex = true;
    IndexedFaceSet_64.solid = false;
    {
    static int m_texCoordIndex[] = { 2, 1, 0, -1, 1, 2, 3, -1, 4, 3, 2, -1, 3, 4, 5, -1, 6, 5, 4, -1 };
    IndexedFaceSet_64.texCoordIndex = m_texCoordIndex;
    IndexedFaceSet_64.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    IndexedFaceSet_64.set_colorIndex = NULL;
    IndexedFaceSet_64.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    IndexedFaceSet_64.set_coordIndex = NULL;
    IndexedFaceSet_64.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    IndexedFaceSet_64.set_normalIndex = NULL;
    IndexedFaceSet_64.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    IndexedFaceSet_64.set_texCoordIndex = NULL;
    IndexedFaceSet_64.set_texCoordIndex_length = 0;
    }
    Shape_298.appearance = &Appearance_299;
    Shape_298.geometry = &IndexedFaceSet_64;
    {
    static X3dNode* m_children[] = { &Background_297, &Shape_298 };
    Group_296.children = m_children;
    Group_296.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group_296.bboxCenter = m_bboxCenter;
    Group_296.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group_296.bboxSize = m_bboxSize;
    Group_296.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group_296.addChildren = NULL;
    Group_296.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group_296.removeChildren = NULL;
    Group_296.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_295.rotation = m_rotation;
    Transform_295.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_295.translation = m_translation;
    Transform_295.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_295.bboxCenter = m_bboxCenter;
    Transform_295.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_295.bboxSize = m_bboxSize;
    Transform_295.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_295.center = m_center;
    Transform_295.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_295.scale = m_scale;
    Transform_295.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_295.scaleOrientation = m_scaleOrientation;
    Transform_295.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &CORINTm_8, &TIMERm_7, &Group_296 };
    Transform_295.children = m_children;
    Transform_295.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_295.addChildren = NULL;
    Transform_295.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_295.removeChildren = NULL;
    Transform_295.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Transform_295 };
    Group11.children = m_children;
    Group11.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Group11.bboxCenter = m_bboxCenter;
    Group11.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Group11.bboxSize = m_bboxSize;
    Group11.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Group11.addChildren = NULL;
    Group11.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Group11.removeChildren = NULL;
    Group11.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group11 };
    Inline_66.children = m_children;
    Inline_66.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_66.bboxCenter = m_bboxCenter;
    Inline_66.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_66.bboxSize = m_bboxSize;
    Inline_66.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_66.addChildren = NULL;
    Inline_66.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_66.removeChildren = NULL;
    Inline_66.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Inline_63, &Inline_66 };
    Inline_67.children = m_children;
    Inline_67.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_67.bboxCenter = m_bboxCenter;
    Inline_67.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_67.bboxSize = m_bboxSize;
    Inline_67.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_67.addChildren = NULL;
    Inline_67.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_67.removeChildren = NULL;
    Inline_67.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.141000 };
    Transform4.rotation = m_rotation;
    Transform4.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, -0.370000, 0 };
    Transform4.translation = m_translation;
    Transform4.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform4.bboxCenter = m_bboxCenter;
    Transform4.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform4.bboxSize = m_bboxSize;
    Transform4.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform4.center = m_center;
    Transform4.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform4.scale = m_scale;
    Transform4.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform4.scaleOrientation = m_scaleOrientation;
    Transform4.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_56, &Transform_258, &Inline_67 };
    Transform4.children = m_children;
    Transform4.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform4.addChildren = NULL;
    Transform4.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform4.removeChildren = NULL;
    Transform4.removeChildren_length = 0;
    }
    WorldInfo9.m_parent = (X3dNode *)&Inline_68;
    {
    static const char* m_info[] = { "(C) Copyright 1999 ICA University Stuttgart (modified by mufti@csv.ica.uni-stuttgart.de)", "This program is free software; you can redistribute it and/or modify", "it under the terms of the GNU General Public License as published by", "the Free Software Foundation; version 2 of the License              ", "This program is distributed in the hope that it will be useful,     ", "but WITHOUT ANY WARRANTY; without even the implied warranty of      ", "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the       ", "GNU General Public License for more details.                        ", "You should have received a copy of the GNU General Public License   ", "along with this program; if not, write to the Free Software         ", "Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA", "p_fly is a modification of the program tux_aqfh by Steve Baker", "tux_aqfh is GPL" };
    WorldInfo9.info = m_info;
    WorldInfo9.info_length = sizeof(m_info) / sizeof(const char*);
    }
    WorldInfo9.title = "Tux warbird 3D Game";
    {
    static X3dNode* m_children[] = { &Transform4, &WorldInfo9 };
    Inline_68.children = m_children;
    Inline_68.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_68.bboxCenter = m_bboxCenter;
    Inline_68.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_68.bboxSize = m_bboxSize;
    Inline_68.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_68.addChildren = NULL;
    Inline_68.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_68.removeChildren = NULL;
    Inline_68.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 1, 0, 3.140000 };
    Transform_233.rotation = m_rotation;
    Transform_233.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    Transform_233.translation = m_translation;
    Transform_233.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_233.bboxCenter = m_bboxCenter;
    Transform_233.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_233.bboxSize = m_bboxSize;
    Transform_233.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_233.center = m_center;
    Transform_233.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_233.scale = m_scale;
    Transform_233.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_233.scaleOrientation = m_scaleOrientation;
    Transform_233.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_68 };
    Transform_233.children = m_children;
    Transform_233.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_233.addChildren = NULL;
    Transform_233.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_233.removeChildren = NULL;
    Transform_233.removeChildren_length = 0;
    }
    Transform_301.route_sources =  NULL;
    Transform_301.num_route_source = 0;
    Transform_301.m_parent = (X3dNode *)NULL;
    Shape_302.m_parent = (X3dNode *)NULL;
    Text_69.setGlName(36);
    Text_69.m_parent = (X3dNode *)NULL;
    Coordinate31.m_parent = (X3dNode *)&Text_69;
    {
    static float m_point[] = { 0.484375, 0.299740, 0, 0.481689, 0.348519, 0, 0.484375, 0.299740, 0, 0.484375, 0.299740, 0, 0.481689, 0.348519, 0, 0.481689, 0.348519, 0, 0.481689, 0.348519, 0, 0.473633, 0.391471, 0, 0.481689, 0.348519, 0, 0.481689, 0.348519, 0, 0.473633, 0.391471, 0, 0.473633, 0.391471, 0, 0.473633, 0.391471, 0, 0.460205, 0.428597, 0, 0.473633, 0.391471, 0, 0.473633, 0.391471, 0, 0.460205, 0.428597, 0, 0.460205, 0.428597, 0, 0.460205, 0.428597, 0, 0.441406, 0.459896, 0, 0.460205, 0.428597, 0, 0.460205, 0.428597, 0, 0.441406, 0.459896, 0, 0.441406, 0.459896, 0, 0.441406, 0.459896, 0, 0.417757, 0.484733, 0, 0.441406, 0.459896, 0, 0.441406, 0.459896, 0, 0.417757, 0.484733, 0, 0.417757, 0.484733, 0, 0.417757, 0.484733, 0, 0.389779, 0.502474, 0, 0.417757, 0.484733, 0, 0.417757, 0.484733, 0, 0.389779, 0.502474, 0, 0.389779, 0.502474, 0, 0.389779, 0.502474, 0, 0.357471, 0.513119, 0, 0.389779, 0.502474, 0, 0.389779, 0.502474, 0, 0.357471, 0.513119, 0, 0.357471, 0.513119, 0, 0.357471, 0.513119, 0, 0.320833, 0.516667, 0, 0.357471, 0.513119, 0, 0.357471, 0.513119, 0, 0.320833, 0.516667, 0, 0.320833, 0.516667, 0, 0.320833, 0.516667, 0, 0.284424, 0.513119, 0, 0.320833, 0.516667, 0, 0.320833, 0.516667, 0, 0.284424, 0.513119, 0, 0.284424, 0.513119, 0, 0.284424, 0.513119, 0, 0.252279, 0.502474, 0, 0.284424, 0.513119, 0, 0.284424, 0.513119, 0, 0.252279, 0.502474, 0, 0.252279, 0.502474, 0, 0.252279, 0.502474, 0, 0.224398, 0.484733, 0, 0.252279, 0.502474, 0, 0.252279, 0.502474, 0, 0.224398, 0.484733, 0, 0.224398, 0.484733, 0, 0.224398, 0.484733, 0, 0.200781, 0.459896, 0, 0.224398, 0.484733, 0, 0.224398, 0.484733, 0, 0.200781, 0.459896, 0, 0.200781, 0.459896, 0, 0.200781, 0.459896, 0, 0.181982, 0.428597, 0, 0.200781, 0.459896, 0, 0.200781, 0.459896, 0, 0.181982, 0.428597, 0, 0.181982, 0.428597, 0, 0.181982, 0.428597, 0, 0.168555, 0.391471, 0, 0.181982, 0.428597, 0, 0.181982, 0.428597, 0, 0.168555, 0.391471, 0, 0.168555, 0.391471, 0, 0.168555, 0.391471, 0, 0.160498, 0.348519, 0, 0.168555, 0.391471, 0, 0.168555, 0.391471, 0, 0.160498, 0.348519, 0, 0.160498, 0.348519, 0, 0.160498, 0.348519, 0, 0.157813, 0.299740, 0, 0.160498, 0.348519, 0, 0.160498, 0.348519, 0, 0.157813, 0.299740, 0, 0.157813, 0.299740, 0, 0.157813, 0.299740, 0, 0.160498, 0.251188, 0, 0.157813, 0.299740, 0, 0.157813, 0.299740, 0, 0.160498, 0.251188, 0, 0.160498, 0.251188, 0, 0.160498, 0.251188, 0, 0.168555, 0.208398, 0, 0.160498, 0.251188, 0, 0.160498, 0.251188, 0, 0.168555, 0.208398, 0, 0.168555, 0.208398, 0, 0.168555, 0.208398, 0, 0.181982, 0.171370, 0, 0.168555, 0.208398, 0, 0.168555, 0.208398, 0, 0.181982, 0.171370, 0, 0.181982, 0.171370, 0, 0.181982, 0.171370, 0, 0.200781, 0.140104, 0, 0.181982, 0.171370, 0, 0.181982, 0.171370, 0, 0.200781, 0.140104, 0, 0.200781, 0.140104, 0, 0.200781, 0.140104, 0, 0.224398, 0.115267, 0, 0.200781, 0.140104, 0, 0.200781, 0.140104, 0, 0.224398, 0.115267, 0, 0.224398, 0.115267, 0, 0.224398, 0.115267, 0, 0.252279, 0.097526, 0, 0.224398, 0.115267, 0, 0.224398, 0.115267, 0, 0.252279, 0.097526, 0, 0.252279, 0.097526, 0, 0.252279, 0.097526, 0, 0.284424, 0.086882, 0, 0.252279, 0.097526, 0, 0.252279, 0.097526, 0, 0.284424, 0.086882, 0, 0.284424, 0.086882, 0, 0.284424, 0.086882, 0, 0.320833, 0.083333, 0, 0.284424, 0.086882, 0, 0.284424, 0.086882, 0, 0.320833, 0.083333, 0, 0.320833, 0.083333, 0, 0.320833, 0.083333, 0, 0.357471, 0.086882, 0, 0.320833, 0.083333, 0, 0.320833, 0.083333, 0, 0.357471, 0.086882, 0, 0.357471, 0.086882, 0, 0.357471, 0.086882, 0, 0.389779, 0.097526, 0, 0.357471, 0.086882, 0, 0.357471, 0.086882, 0, 0.389779, 0.097526, 0, 0.389779, 0.097526, 0, 0.389779, 0.097526, 0, 0.417757, 0.115267, 0, 0.389779, 0.097526, 0, 0.389779, 0.097526, 0, 0.417757, 0.115267, 0, 0.417757, 0.115267, 0, 0.417757, 0.115267, 0, 0.441406, 0.140104, 0, 0.417757, 0.115267, 0, 0.417757, 0.115267, 0, 0.441406, 0.140104, 0, 0.441406, 0.140104, 0, 0.441406, 0.140104, 0, 0.460205, 0.171370, 0, 0.441406, 0.140104, 0, 0.441406, 0.140104, 0, 0.460205, 0.171370, 0, 0.460205, 0.171370, 0, 0.460205, 0.171370, 0, 0.473633, 0.208398, 0, 0.460205, 0.171370, 0, 0.460205, 0.171370, 0, 0.473633, 0.208398, 0, 0.473633, 0.208398, 0, 0.473633, 0.208398, 0, 0.481689, 0.251188, 0, 0.473633, 0.208398, 0, 0.473633, 0.208398, 0, 0.481689, 0.251188, 0, 0.481689, 0.251188, 0, 0.580208, 0.075260, 0, 0.576074, 0.006152, 0, 0.580208, 0.075260, 0, 0.580208, 0.075260, 0, 0.576074, 0.006152, 0, 0.576074, 0.006152, 0, 0.576074, 0.006152, 0, 0.563672, -0.053516, 0, 0.576074, 0.006152, 0, 0.576074, 0.006152, 0, 0.563672, -0.053516, 0, 0.563672, -0.053516, 0, 0.563672, -0.053516, 0, 0.543001, -0.103743, 0, 0.563672, -0.053516, 0, 0.563672, -0.053516, 0, 0.543001, -0.103743, 0, 0.543001, -0.103743, 0, 0.543001, -0.103743, 0, 0.514063, -0.144531, 0, 0.543001, -0.103743, 0, 0.543001, -0.103743, 0, 0.514063, -0.144531, 0, 0.514063, -0.144531, 0, 0.514063, -0.144531, 0, 0.476595, -0.176090, 0, 0.514063, -0.144531, 0, 0.514063, -0.144531, 0, 0.476595, -0.176090, 0, 0.476595, -0.176090, 0, 0.476595, -0.176090, 0, 0.430339, -0.198633, 0, 0.476595, -0.176090, 0, 0.476595, -0.176090, 0, 0.430339, -0.198633, 0, 0.430339, -0.198633, 0, 0.430339, -0.198633, 0, 0.375293, -0.212158, 0, 0.430339, -0.198633, 0, 0.430339, -0.198633, 0, 0.375293, -0.212158, 0, 0.375293, -0.212158, 0, 0.375293, -0.212158, 0, 0.311458, -0.216667, 0, 0.375293, -0.212158, 0, 0.375293, -0.212158, 0, 0.311458, -0.216667, 0, 0.311458, -0.216667, 0, 0.311458, -0.216667, 0, 0.286556, -0.216154, 0, 0.311458, -0.216667, 0, 0.311458, -0.216667, 0, 0.286556, -0.216154, 0, 0.286556, -0.216154, 0, 0.286556, -0.216154, 0, 0.262370, -0.214616, 0, 0.286556, -0.216154, 0, 0.286556, -0.216154, 0, 0.262370, -0.214616, 0, 0.262370, -0.214616, 0, 0.262370, -0.214616, 0, 0.238900, -0.212052, 0, 0.262370, -0.214616, 0, 0.262370, -0.214616, 0, 0.238900, -0.212052, 0, 0.238900, -0.212052, 0, 0.238900, -0.212052, 0, 0.216146, -0.208464, 0, 0.238900, -0.212052, 0, 0.238900, -0.212052, 0, 0.216146, -0.208464, 0, 0.216146, -0.208464, 0, 0.216146, -0.208464, 0, 0.193913, -0.203817, 0, 0.216146, -0.208464, 0, 0.216146, -0.208464, 0, 0.193913, -0.203817, 0, 0.193913, -0.203817, 0, 0.193913, -0.203817, 0, 0.172005, -0.198079, 0, 0.193913, -0.203817, 0, 0.193913, -0.203817, 0, 0.172005, -0.198079, 0, 0.172005, -0.198079, 0, 0.172005, -0.198079, 0, 0.150423, -0.191252, 0, 0.172005, -0.198079, 0, 0.172005, -0.198079, 0, 0.150423, -0.191252, 0, 0.150423, -0.191252, 0, 0.150423, -0.191252, 0, 0.129167, -0.183333, 0, 0.150423, -0.191252, 0, 0.150423, -0.191252, 0, 0.129167, -0.183333, 0, 0.129167, -0.183333, 0, 0.129167, -0.183333, 0, 0.129167, -0.083333, 0, 0.129167, -0.183333, 0, 0.129167, -0.183333, 0, 0.129167, -0.083333, 0, 0.129167, -0.083333, 0, 0.129167, -0.083333, 0, 0.150195, -0.095264, 0, 0.129167, -0.083333, 0, 0.129167, -0.083333, 0, 0.150195, -0.095264, 0, 0.150195, -0.095264, 0, 0.150195, -0.095264, 0, 0.171094, -0.105534, 0, 0.150195, -0.095264, 0, 0.150195, -0.095264, 0, 0.171094, -0.105534, 0, 0.171094, -0.105534, 0, 0.171094, -0.105534, 0, 0.191862, -0.114144, 0, 0.171094, -0.105534, 0, 0.171094, -0.105534, 0, 0.191862, -0.114144, 0, 0.191862, -0.114144, 0, 0.191862, -0.114144, 0, 0.212500, -0.121094, 0, 0.191862, -0.114144, 0, 0.191862, -0.114144, 0, 0.212500, -0.121094, 0, 0.212500, -0.121094, 0, 0.212500, -0.121094, 0, 0.233171, -0.126449, 0, 0.212500, -0.121094, 0, 0.212500, -0.121094, 0, 0.233171, -0.126449, 0, 0.233171, -0.126449, 0, 0.233171, -0.126449, 0, 0.254036, -0.130273, 0, 0.233171, -0.126449, 0, 0.233171, -0.126449, 0, 0.254036, -0.130273, 0, 0.254036, -0.130273, 0, 0.254036, -0.130273, 0, 0.275098, -0.132568, 0, 0.254036, -0.130273, 0, 0.254036, -0.130273, 0, 0.275098, -0.132568, 0, 0.275098, -0.132568, 0, 0.275098, -0.132568, 0, 0.296354, -0.133333, 0, 0.275098, -0.132568, 0, 0.275098, -0.132568, 0, 0.296354, -0.133333, 0, 0.296354, -0.133333, 0, 0.296354, -0.133333, 0, 0.340527, -0.130371, 0, 0.296354, -0.133333, 0, 0.296354, -0.133333, 0, 0.340527, -0.130371, 0, 0.340527, -0.130371, 0, 0.340527, -0.130371, 0, 0.378776, -0.121484, 0, 0.340527, -0.130371, 0, 0.340527, -0.130371, 0, 0.378776, -0.121484, 0, 0.378776, -0.121484, 0, 0.378776, -0.121484, 0, 0.411100, -0.106673, 0, 0.378776, -0.121484, 0, 0.378776, -0.121484, 0, 0.411100, -0.106673, 0, 0.411100, -0.106673, 0, 0.411100, -0.106673, 0, 0.437500, -0.085938, 0, 0.411100, -0.106673, 0, 0.411100, -0.106673, 0, 0.437500, -0.085938, 0, 0.437500, -0.085938, 0, 0.437500, -0.085938, 0, 0.458008, -0.059212, 0, 0.437500, -0.085938, 0, 0.437500, -0.085938, 0, 0.458008, -0.059212, 0, 0.458008, -0.059212, 0, 0.458008, -0.059212, 0, 0.472656, -0.026432, 0, 0.458008, -0.059212, 0, 0.458008, -0.059212, 0, 0.472656, -0.026432, 0, 0.472656, -0.026432, 0, 0.472656, -0.026432, 0, 0.481445, 0.012402, 0, 0.472656, -0.026432, 0, 0.472656, -0.026432, 0, 0.481445, 0.012402, 0, 0.481445, 0.012402, 0, 0.481445, 0.012402, 0, 0.484375, 0.057292, 0, 0.481445, 0.012402, 0, 0.481445, 0.012402, 0, 0.484375, 0.057292, 0, 0.484375, 0.057292, 0, 0.484375, 0.057292, 0, 0.484375, 0.100000, 0, 0.484375, 0.057292, 0, 0.484375, 0.057292, 0, 0.484375, 0.100000, 0, 0.484375, 0.100000, 0, 0.484375, 0.100000, 0, 0.468490, 0.076457, 0, 0.484375, 0.100000, 0, 0.484375, 0.100000, 0, 0.468490, 0.076457, 0, 0.468490, 0.076457, 0, 0.468490, 0.076457, 0, 0.450521, 0.056087, 0, 0.468490, 0.076457, 0, 0.468490, 0.076457, 0, 0.450521, 0.056087, 0, 0.450521, 0.056087, 0, 0.450521, 0.056087, 0, 0.430469, 0.038892, 0, 0.450521, 0.056087, 0, 0.450521, 0.056087, 0, 0.430469, 0.038892, 0, 0.430469, 0.038892, 0, 0.430469, 0.038892, 0, 0.408333, 0.024870, 0, 0.430469, 0.038892, 0, 0.430469, 0.038892, 0, 0.408333, 0.024870, 0, 0.408333, 0.024870, 0, 0.408333, 0.024870, 0, 0.384017, 0.013989, 0, 0.408333, 0.024870, 0, 0.408333, 0.024870, 0, 0.384017, 0.013989, 0, 0.384017, 0.013989, 0, 0.384017, 0.013989, 0, 0.357422, 0.006217, 0, 0.384017, 0.013989, 0, 0.384017, 0.013989, 0, 0.357422, 0.006217, 0, 0.357422, 0.006217, 0, 0.357422, 0.006217, 0, 0.328548, 0.001554, 0, 0.357422, 0.006217, 0, 0.357422, 0.006217, 0, 0.328548, 0.001554, 0, 0.328548, 0.001554, 0, 0.328548, 0.001554, 0, 0.297396, 0, 0, 0.328548, 0.001554, 0, 0.328548, 0.001554, 0, 0.297396, 0, 0, 0.297396, 0, 0, 0.297396, 0, 0, 0.246354, 0.005135, 0, 0.297396, 0, 0, 0.297396, 0, 0, 0.246354, 0.005135, 0, 0.246354, 0.005135, 0, 0.246354, 0.005135, 0, 0.200521, 0.020540, 0, 0.246354, 0.005135, 0, 0.246354, 0.005135, 0, 0.200521, 0.020540, 0, 0.200521, 0.020540, 0, 0.200521, 0.020540, 0, 0.159896, 0.046216, 0, 0.200521, 0.020540, 0, 0.200521, 0.020540, 0, 0.159896, 0.046216, 0, 0.159896, 0.046216, 0, 0.159896, 0.046216, 0, 0.124479, 0.082161, 0, 0.159896, 0.046216, 0, 0.159896, 0.046216, 0, 0.124479, 0.082161, 0, 0.124479, 0.082161, 0, 0.124479, 0.082161, 0, 0.095768, 0.126571, 0, 0.124479, 0.082161, 0, 0.124479, 0.082161, 0, 0.095768, 0.126571, 0, 0.095768, 0.126571, 0, 0.095768, 0.126571, 0, 0.075260, 0.177637, 0, 0.095768, 0.126571, 0, 0.095768, 0.126571, 0, 0.075260, 0.177637, 0, 0.075260, 0.177637, 0, 0.075260, 0.177637, 0, 0.062956, 0.235360, 0, 0.075260, 0.177637, 0, 0.075260, 0.177637, 0, 0.062956, 0.235360, 0, 0.062956, 0.235360, 0, 0.062956, 0.235360, 0, 0.058854, 0.299740, 0, 0.062956, 0.235360, 0, 0.062956, 0.235360, 0, 0.058854, 0.299740, 0, 0.058854, 0.299740, 0, 0.058854, 0.299740, 0, 0.062956, 0.364347, 0, 0.058854, 0.299740, 0, 0.058854, 0.299740, 0, 0.062956, 0.364347, 0, 0.062956, 0.364347, 0, 0.062956, 0.364347, 0, 0.075260, 0.422233, 0, 0.062956, 0.364347, 0, 0.062956, 0.364347, 0, 0.075260, 0.422233, 0, 0.075260, 0.422233, 0, 0.075260, 0.422233, 0, 0.095768, 0.473397, 0, 0.075260, 0.422233, 0, 0.075260, 0.422233, 0, 0.095768, 0.473397, 0, 0.095768, 0.473397, 0, 0.095768, 0.473397, 0, 0.124479, 0.517839, 0, 0.095768, 0.473397, 0, 0.095768, 0.473397, 0, 0.124479, 0.517839, 0, 0.124479, 0.517839, 0, 0.124479, 0.517839, 0, 0.159896, 0.553784, 0, 0.124479, 0.517839, 0, 0.124479, 0.517839, 0, 0.159896, 0.553784, 0, 0.159896, 0.553784, 0, 0.159896, 0.553784, 0, 0.200521, 0.579460, 0, 0.159896, 0.553784, 0, 0.159896, 0.553784, 0, 0.200521, 0.579460, 0, 0.200521, 0.579460, 0, 0.200521, 0.579460, 0, 0.246354, 0.594865, 0, 0.200521, 0.579460, 0, 0.200521, 0.579460, 0, 0.246354, 0.594865, 0, 0.246354, 0.594865, 0, 0.246354, 0.594865, 0, 0.297396, 0.600000, 0, 0.246354, 0.594865, 0, 0.246354, 0.594865, 0, 0.297396, 0.600000, 0, 0.297396, 0.600000, 0, 0.297396, 0.600000, 0, 0.328548, 0.598446, 0, 0.297396, 0.600000, 0, 0.297396, 0.600000, 0, 0.328548, 0.598446, 0, 0.328548, 0.598446, 0, 0.328548, 0.598446, 0, 0.357422, 0.593783, 0, 0.328548, 0.598446, 0, 0.328548, 0.598446, 0, 0.357422, 0.593783, 0, 0.357422, 0.593783, 0, 0.357422, 0.593783, 0, 0.384017, 0.586011, 0, 0.357422, 0.593783, 0, 0.357422, 0.593783, 0, 0.384017, 0.586011, 0, 0.384017, 0.586011, 0, 0.384017, 0.586011, 0, 0.408333, 0.575130, 0, 0.384017, 0.586011, 0, 0.384017, 0.586011, 0, 0.408333, 0.575130, 0, 0.408333, 0.575130, 0, 0.408333, 0.575130, 0, 0.430469, 0.561108, 0, 0.408333, 0.575130, 0, 0.408333, 0.575130, 0, 0.430469, 0.561108, 0, 0.430469, 0.561108, 0, 0.430469, 0.561108, 0, 0.450521, 0.543913, 0, 0.430469, 0.561108, 0, 0.430469, 0.561108, 0, 0.450521, 0.543913, 0, 0.450521, 0.543913, 0, 0.450521, 0.543913, 0, 0.468490, 0.523543, 0, 0.450521, 0.543913, 0, 0.450521, 0.543913, 0, 0.468490, 0.523543, 0, 0.468490, 0.523543, 0, 0.468490, 0.523543, 0, 0.484375, 0.500000, 0, 0.468490, 0.523543, 0, 0.468490, 0.523543, 0, 0.484375, 0.500000, 0, 0.484375, 0.500000, 0, 0.484375, 0.500000, 0, 0.484375, 0.583333, 0, 0.484375, 0.500000, 0, 0.484375, 0.500000, 0, 0.484375, 0.583333, 0, 0.484375, 0.583333, 0, 0.484375, 0.583333, 0, 0.580208, 0.583333, 0, 0.484375, 0.583333, 0, 0.484375, 0.583333, 0, 0.580208, 0.583333, 0, 0.580208, 0.583333, 0, 0.160498, 0.348519, 0, 0.062956, 0.364347, 0, 0.058854, 0.299740, 0, 0.062956, 0.364347, 0, 0.160498, 0.348519, 0, 0.058854, 0.299740, 0, 0.168555, 0.391471, 0, 0.062956, 0.364347, 0, 0.160498, 0.348519, 0, 0.062956, 0.364347, 0, 0.168555, 0.391471, 0, 0.160498, 0.348519, 0, 0.168555, 0.391471, 0, 0.075260, 0.422233, 0, 0.062956, 0.364347, 0, 0.075260, 0.422233, 0, 0.168555, 0.391471, 0, 0.062956, 0.364347, 0, 0.181982, 0.428597, 0, 0.075260, 0.422233, 0, 0.168555, 0.391471, 0, 0.075260, 0.422233, 0, 0.181982, 0.428597, 0, 0.168555, 0.391471, 0, 0.181982, 0.428597, 0, 0.095768, 0.473397, 0, 0.075260, 0.422233, 0, 0.095768, 0.473397, 0, 0.181982, 0.428597, 0, 0.075260, 0.422233, 0, 0.200781, 0.459896, 0, 0.095768, 0.473397, 0, 0.181982, 0.428597, 0, 0.095768, 0.473397, 0, 0.200781, 0.459896, 0, 0.181982, 0.428597, 0, 0.200781, 0.459896, 0, 0.124479, 0.517839, 0, 0.095768, 0.473397, 0, 0.124479, 0.517839, 0, 0.200781, 0.459896, 0, 0.095768, 0.473397, 0, 0.159896, 0.553784, 0, 0.124479, 0.517839, 0, 0.200781, 0.459896, 0, 0.124479, 0.517839, 0, 0.159896, 0.553784, 0, 0.200781, 0.459896, 0, 0.159896, 0.553784, 0, 0.200781, 0.459896, 0, 0.224398, 0.484733, 0, 0.200781, 0.459896, 0, 0.159896, 0.553784, 0, 0.224398, 0.484733, 0, 0.200521, 0.579460, 0, 0.159896, 0.553784, 0, 0.224398, 0.484733, 0, 0.159896, 0.553784, 0, 0.200521, 0.579460, 0, 0.224398, 0.484733, 0, 0.200521, 0.579460, 0, 0.224398, 0.484733, 0, 0.252279, 0.502474, 0, 0.224398, 0.484733, 0, 0.200521, 0.579460, 0, 0.252279, 0.502474, 0, 0.246354, 0.594865, 0, 0.200521, 0.579460, 0, 0.252279, 0.502474, 0, 0.200521, 0.579460, 0, 0.246354, 0.594865, 0, 0.252279, 0.502474, 0, 0.246354, 0.594865, 0, 0.252279, 0.502474, 0, 0.284424, 0.513119, 0, 0.252279, 0.502474, 0, 0.246354, 0.594865, 0, 0.284424, 0.513119, 0, 0.284424, 0.513119, 0, 0.297396, 0.600000, 0, 0.246354, 0.594865, 0, 0.297396, 0.600000, 0, 0.284424, 0.513119, 0, 0.246354, 0.594865, 0, 0.284424, 0.513119, 0, 0.320833, 0.516667, 0, 0.297396, 0.600000, 0, 0.320833, 0.516667, 0, 0.284424, 0.513119, 0, 0.297396, 0.600000, 0, 0.320833, 0.516667, 0, 0.328548, 0.598446, 0, 0.297396, 0.600000, 0, 0.328548, 0.598446, 0, 0.320833, 0.516667, 0, 0.297396, 0.600000, 0, 0.357422, 0.593783, 0, 0.328548, 0.598446, 0, 0.320833, 0.516667, 0, 0.328548, 0.598446, 0, 0.357422, 0.593783, 0, 0.320833, 0.516667, 0, 0.357422, 0.593783, 0, 0.320833, 0.516667, 0, 0.357471, 0.513119, 0, 0.320833, 0.516667, 0, 0.357422, 0.593783, 0, 0.357471, 0.513119, 0, 0.357422, 0.593783, 0, 0.357471, 0.513119, 0, 0.384017, 0.586011, 0, 0.357471, 0.513119, 0, 0.357422, 0.593783, 0, 0.384017, 0.586011, 0, 0.384017, 0.586011, 0, 0.357471, 0.513119, 0, 0.408333, 0.575130, 0, 0.357471, 0.513119, 0, 0.384017, 0.586011, 0, 0.408333, 0.575130, 0, 0.357471, 0.513119, 0, 0.389779, 0.502474, 0, 0.408333, 0.575130, 0, 0.389779, 0.502474, 0, 0.357471, 0.513119, 0, 0.408333, 0.575130, 0, 0.408333, 0.575130, 0, 0.389779, 0.502474, 0, 0.430469, 0.561108, 0, 0.389779, 0.502474, 0, 0.408333, 0.575130, 0, 0.430469, 0.561108, 0, 0.430469, 0.561108, 0, 0.389779, 0.502474, 0, 0.450521, 0.543913, 0, 0.389779, 0.502474, 0, 0.430469, 0.561108, 0, 0.450521, 0.543913, 0, 0.389779, 0.502474, 0, 0.417757, 0.484733, 0, 0.450521, 0.543913, 0, 0.417757, 0.484733, 0, 0.389779, 0.502474, 0, 0.450521, 0.543913, 0, 0.450521, 0.543913, 0, 0.417757, 0.484733, 0, 0.468490, 0.523543, 0, 0.417757, 0.484733, 0, 0.450521, 0.543913, 0, 0.468490, 0.523543, 0, 0.468490, 0.523543, 0, 0.417757, 0.484733, 0, 0.484375, 0.500000, 0, 0.417757, 0.484733, 0, 0.468490, 0.523543, 0, 0.484375, 0.500000, 0, 0.484375, 0.500000, 0, 0.417757, 0.484733, 0, 0.441406, 0.459896, 0, 0.417757, 0.484733, 0, 0.484375, 0.500000, 0, 0.441406, 0.459896, 0, 0.441406, 0.459896, 0, 0.460205, 0.428597, 0, 0.484375, 0.500000, 0, 0.460205, 0.428597, 0, 0.441406, 0.459896, 0, 0.484375, 0.500000, 0, 0.460205, 0.428597, 0, 0.473633, 0.391471, 0, 0.484375, 0.500000, 0, 0.473633, 0.391471, 0, 0.460205, 0.428597, 0, 0.484375, 0.500000, 0, 0.484375, 0.500000, 0, 0.473633, 0.391471, 0, 0.580208, 0.583333, 0, 0.473633, 0.391471, 0, 0.484375, 0.500000, 0, 0.580208, 0.583333, 0, 0.580208, 0.583333, 0, 0.484375, 0.583333, 0, 0.484375, 0.500000, 0, 0.484375, 0.583333, 0, 0.580208, 0.583333, 0, 0.484375, 0.500000, 0, 0.481689, 0.348519, 0, 0.580208, 0.583333, 0, 0.473633, 0.391471, 0, 0.580208, 0.583333, 0, 0.481689, 0.348519, 0, 0.473633, 0.391471, 0, 0.484375, 0.299740, 0, 0.580208, 0.583333, 0, 0.481689, 0.348519, 0, 0.580208, 0.583333, 0, 0.484375, 0.299740, 0, 0.481689, 0.348519, 0, 0.484375, 0.299740, 0, 0.580208, 0.075260, 0, 0.580208, 0.583333, 0, 0.580208, 0.075260, 0, 0.484375, 0.299740, 0, 0.580208, 0.583333, 0, 0.484375, 0.299740, 0, 0.481689, 0.251188, 0, 0.580208, 0.075260, 0, 0.481689, 0.251188, 0, 0.484375, 0.299740, 0, 0.580208, 0.075260, 0, 0.580208, 0.075260, 0, 0.481689, 0.251188, 0, 0.473633, 0.208398, 0, 0.481689, 0.251188, 0, 0.580208, 0.075260, 0, 0.473633, 0.208398, 0, 0.484375, 0.100000, 0, 0.580208, 0.075260, 0, 0.473633, 0.208398, 0, 0.580208, 0.075260, 0, 0.484375, 0.100000, 0, 0.473633, 0.208398, 0, 0.484375, 0.100000, 0, 0.484375, 0.057292, 0, 0.580208, 0.075260, 0, 0.484375, 0.057292, 0, 0.484375, 0.100000, 0, 0.580208, 0.075260, 0, 0.484375, 0.057292, 0, 0.576074, 0.006152, 0, 0.580208, 0.075260, 0, 0.576074, 0.006152, 0, 0.484375, 0.057292, 0, 0.580208, 0.075260, 0, 0.484375, 0.057292, 0, 0.481445, 0.012402, 0, 0.576074, 0.006152, 0, 0.481445, 0.012402, 0, 0.484375, 0.057292, 0, 0.576074, 0.006152, 0, 0.576074, 0.006152, 0, 0.481445, 0.012402, 0, 0.563672, -0.053516, 0, 0.481445, 0.012402, 0, 0.576074, 0.006152, 0, 0.563672, -0.053516, 0, 0.481445, 0.012402, 0, 0.472656, -0.026432, 0, 0.563672, -0.053516, 0, 0.472656, -0.026432, 0, 0.481445, 0.012402, 0, 0.563672, -0.053516, 0, 0.563672, -0.053516, 0, 0.472656, -0.026432, 0, 0.543001, -0.103743, 0, 0.472656, -0.026432, 0, 0.563672, -0.053516, 0, 0.543001, -0.103743, 0, 0.472656, -0.026432, 0, 0.458008, -0.059212, 0, 0.543001, -0.103743, 0, 0.458008, -0.059212, 0, 0.472656, -0.026432, 0, 0.543001, -0.103743, 0, 0.543001, -0.103743, 0, 0.458008, -0.059212, 0, 0.514063, -0.144531, 0, 0.458008, -0.059212, 0, 0.543001, -0.103743, 0, 0.514063, -0.144531, 0, 0.458008, -0.059212, 0, 0.437500, -0.085938, 0, 0.514063, -0.144531, 0, 0.437500, -0.085938, 0, 0.458008, -0.059212, 0, 0.514063, -0.144531, 0, 0.437500, -0.085938, 0, 0.476595, -0.176090, 0, 0.514063, -0.144531, 0, 0.476595, -0.176090, 0, 0.437500, -0.085938, 0, 0.514063, -0.144531, 0, 0.437500, -0.085938, 0, 0.411100, -0.106673, 0, 0.476595, -0.176090, 0, 0.411100, -0.106673, 0, 0.437500, -0.085938, 0, 0.476595, -0.176090, 0, 0.411100, -0.106673, 0, 0.430339, -0.198633, 0, 0.476595, -0.176090, 0, 0.430339, -0.198633, 0, 0.411100, -0.106673, 0, 0.476595, -0.176090, 0, 0.411100, -0.106673, 0, 0.378776, -0.121484, 0, 0.430339, -0.198633, 0, 0.378776, -0.121484, 0, 0.411100, -0.106673, 0, 0.430339, -0.198633, 0, 0.378776, -0.121484, 0, 0.375293, -0.212158, 0, 0.430339, -0.198633, 0, 0.375293, -0.212158, 0, 0.378776, -0.121484, 0, 0.430339, -0.198633, 0, 0.340527, -0.130371, 0, 0.375293, -0.212158, 0, 0.378776, -0.121484, 0, 0.375293, -0.212158, 0, 0.340527, -0.130371, 0, 0.378776, -0.121484, 0, 0.340527, -0.130371, 0, 0.311458, -0.216667, 0, 0.375293, -0.212158, 0, 0.311458, -0.216667, 0, 0.340527, -0.130371, 0, 0.375293, -0.212158, 0, 0.296354, -0.133333, 0, 0.311458, -0.216667, 0, 0.340527, -0.130371, 0, 0.311458, -0.216667, 0, 0.296354, -0.133333, 0, 0.340527, -0.130371, 0, 0.296354, -0.133333, 0, 0.286556, -0.216154, 0, 0.311458, -0.216667, 0, 0.286556, -0.216154, 0, 0.296354, -0.133333, 0, 0.311458, -0.216667, 0, 0.275098, -0.132568, 0, 0.286556, -0.216154, 0, 0.296354, -0.133333, 0, 0.286556, -0.216154, 0, 0.275098, -0.132568, 0, 0.296354, -0.133333, 0, 0.275098, -0.132568, 0, 0.262370, -0.214616, 0, 0.286556, -0.216154, 0, 0.262370, -0.214616, 0, 0.275098, -0.132568, 0, 0.286556, -0.216154, 0, 0.254036, -0.130273, 0, 0.262370, -0.214616, 0, 0.275098, -0.132568, 0, 0.262370, -0.214616, 0, 0.254036, -0.130273, 0, 0.275098, -0.132568, 0, 0.254036, -0.130273, 0, 0.238900, -0.212052, 0, 0.262370, -0.214616, 0, 0.238900, -0.212052, 0, 0.254036, -0.130273, 0, 0.262370, -0.214616, 0, 0.233171, -0.126449, 0, 0.238900, -0.212052, 0, 0.254036, -0.130273, 0, 0.238900, -0.212052, 0, 0.233171, -0.126449, 0, 0.254036, -0.130273, 0, 0.233171, -0.126449, 0, 0.216146, -0.208464, 0, 0.238900, -0.212052, 0, 0.216146, -0.208464, 0, 0.233171, -0.126449, 0, 0.238900, -0.212052, 0, 0.193913, -0.203817, 0, 0.216146, -0.208464, 0, 0.233171, -0.126449, 0, 0.216146, -0.208464, 0, 0.193913, -0.203817, 0, 0.233171, -0.126449, 0, 0.212500, -0.121094, 0, 0.193913, -0.203817, 0, 0.233171, -0.126449, 0, 0.193913, -0.203817, 0, 0.212500, -0.121094, 0, 0.233171, -0.126449, 0, 0.172005, -0.198079, 0, 0.193913, -0.203817, 0, 0.212500, -0.121094, 0, 0.193913, -0.203817, 0, 0.172005, -0.198079, 0, 0.212500, -0.121094, 0, 0.191862, -0.114144, 0, 0.172005, -0.198079, 0, 0.212500, -0.121094, 0, 0.172005, -0.198079, 0, 0.191862, -0.114144, 0, 0.212500, -0.121094, 0, 0.150423, -0.191252, 0, 0.172005, -0.198079, 0, 0.191862, -0.114144, 0, 0.172005, -0.198079, 0, 0.150423, -0.191252, 0, 0.191862, -0.114144, 0, 0.171094, -0.105534, 0, 0.150423, -0.191252, 0, 0.191862, -0.114144, 0, 0.150423, -0.191252, 0, 0.171094, -0.105534, 0, 0.191862, -0.114144, 0, 0.129167, -0.183333, 0, 0.150423, -0.191252, 0, 0.171094, -0.105534, 0, 0.150423, -0.191252, 0, 0.129167, -0.183333, 0, 0.171094, -0.105534, 0, 0.150195, -0.095264, 0, 0.129167, -0.183333, 0, 0.171094, -0.105534, 0, 0.129167, -0.183333, 0, 0.150195, -0.095264, 0, 0.171094, -0.105534, 0, 0.129167, -0.083333, 0, 0.129167, -0.183333, 0, 0.150195, -0.095264, 0, 0.129167, -0.183333, 0, 0.129167, -0.083333, 0, 0.150195, -0.095264, 0, 0.473633, 0.208398, 0, 0.460205, 0.171370, 0, 0.484375, 0.100000, 0, 0.460205, 0.171370, 0, 0.473633, 0.208398, 0, 0.484375, 0.100000, 0, 0.460205, 0.171370, 0, 0.441406, 0.140104, 0, 0.484375, 0.100000, 0, 0.441406, 0.140104, 0, 0.460205, 0.171370, 0, 0.484375, 0.100000, 0, 0.441406, 0.140104, 0, 0.417757, 0.115267, 0, 0.484375, 0.100000, 0, 0.417757, 0.115267, 0, 0.441406, 0.140104, 0, 0.484375, 0.100000, 0, 0.417757, 0.115267, 0, 0.468490, 0.076457, 0, 0.484375, 0.100000, 0, 0.468490, 0.076457, 0, 0.417757, 0.115267, 0, 0.484375, 0.100000, 0, 0.417757, 0.115267, 0, 0.450521, 0.056087, 0, 0.468490, 0.076457, 0, 0.450521, 0.056087, 0, 0.417757, 0.115267, 0, 0.468490, 0.076457, 0, 0.417757, 0.115267, 0, 0.389779, 0.097526, 0, 0.450521, 0.056087, 0, 0.389779, 0.097526, 0, 0.417757, 0.115267, 0, 0.450521, 0.056087, 0, 0.389779, 0.097526, 0, 0.430469, 0.038892, 0, 0.450521, 0.056087, 0, 0.430469, 0.038892, 0, 0.389779, 0.097526, 0, 0.450521, 0.056087, 0, 0.389779, 0.097526, 0, 0.408333, 0.024870, 0, 0.430469, 0.038892, 0, 0.408333, 0.024870, 0, 0.389779, 0.097526, 0, 0.430469, 0.038892, 0, 0.389779, 0.097526, 0, 0.357471, 0.086882, 0, 0.408333, 0.024870, 0, 0.357471, 0.086882, 0, 0.389779, 0.097526, 0, 0.408333, 0.024870, 0, 0.357471, 0.086882, 0, 0.384017, 0.013989, 0, 0.408333, 0.024870, 0, 0.384017, 0.013989, 0, 0.357471, 0.086882, 0, 0.408333, 0.024870, 0, 0.357471, 0.086882, 0, 0.357422, 0.006217, 0, 0.384017, 0.013989, 0, 0.357422, 0.006217, 0, 0.357471, 0.086882, 0, 0.384017, 0.013989, 0, 0.320833, 0.083333, 0, 0.357422, 0.006217, 0, 0.357471, 0.086882, 0, 0.357422, 0.006217, 0, 0.320833, 0.083333, 0, 0.357471, 0.086882, 0, 0.320833, 0.083333, 0, 0.328548, 0.001554, 0, 0.357422, 0.006217, 0, 0.328548, 0.001554, 0, 0.320833, 0.083333, 0, 0.357422, 0.006217, 0, 0.320833, 0.083333, 0, 0.297396, 0, 0, 0.328548, 0.001554, 0, 0.297396, 0, 0, 0.320833, 0.083333, 0, 0.328548, 0.001554, 0, 0.284424, 0.086882, 0, 0.297396, 0, 0, 0.320833, 0.083333, 0, 0.297396, 0, 0, 0.284424, 0.086882, 0, 0.320833, 0.083333, 0, 0.284424, 0.086882, 0, 0.246354, 0.005135, 0, 0.297396, 0, 0, 0.246354, 0.005135, 0, 0.284424, 0.086882, 0, 0.297396, 0, 0, 0.284424, 0.086882, 0, 0.252279, 0.097526, 0, 0.246354, 0.005135, 0, 0.252279, 0.097526, 0, 0.284424, 0.086882, 0, 0.246354, 0.005135, 0, 0.252279, 0.097526, 0, 0.200521, 0.020540, 0, 0.246354, 0.005135, 0, 0.200521, 0.020540, 0, 0.252279, 0.097526, 0, 0.246354, 0.005135, 0, 0.252279, 0.097526, 0, 0.224398, 0.115267, 0, 0.200521, 0.020540, 0, 0.224398, 0.115267, 0, 0.252279, 0.097526, 0, 0.200521, 0.020540, 0, 0.224398, 0.115267, 0, 0.159896, 0.046216, 0, 0.200521, 0.020540, 0, 0.159896, 0.046216, 0, 0.224398, 0.115267, 0, 0.200521, 0.020540, 0, 0.224398, 0.115267, 0, 0.200781, 0.140104, 0, 0.159896, 0.046216, 0, 0.200781, 0.140104, 0, 0.224398, 0.115267, 0, 0.159896, 0.046216, 0, 0.200781, 0.140104, 0, 0.124479, 0.082161, 0, 0.159896, 0.046216, 0, 0.124479, 0.082161, 0, 0.200781, 0.140104, 0, 0.159896, 0.046216, 0, 0.095768, 0.126571, 0, 0.124479, 0.082161, 0, 0.200781, 0.140104, 0, 0.124479, 0.082161, 0, 0.095768, 0.126571, 0, 0.200781, 0.140104, 0, 0.181982, 0.171370, 0, 0.095768, 0.126571, 0, 0.200781, 0.140104, 0, 0.095768, 0.126571, 0, 0.181982, 0.171370, 0, 0.200781, 0.140104, 0, 0.075260, 0.177637, 0, 0.095768, 0.126571, 0, 0.181982, 0.171370, 0, 0.095768, 0.126571, 0, 0.075260, 0.177637, 0, 0.181982, 0.171370, 0, 0.168555, 0.208398, 0, 0.075260, 0.177637, 0, 0.181982, 0.171370, 0, 0.075260, 0.177637, 0, 0.168555, 0.208398, 0, 0.181982, 0.171370, 0, 0.062956, 0.235360, 0, 0.075260, 0.177637, 0, 0.168555, 0.208398, 0, 0.075260, 0.177637, 0, 0.062956, 0.235360, 0, 0.168555, 0.208398, 0, 0.160498, 0.251188, 0, 0.062956, 0.235360, 0, 0.168555, 0.208398, 0, 0.062956, 0.235360, 0, 0.160498, 0.251188, 0, 0.168555, 0.208398, 0, 0.058854, 0.299740, 0, 0.062956, 0.235360, 0, 0.160498, 0.251188, 0, 0.062956, 0.235360, 0, 0.058854, 0.299740, 0, 0.160498, 0.251188, 0, 0.157813, 0.299740, 0, 0.058854, 0.299740, 0, 0.160498, 0.251188, 0, 0.058854, 0.299740, 0, 0.157813, 0.299740, 0, 0.160498, 0.251188, 0, 0.160498, 0.348519, 0, 0.058854, 0.299740, 0, 0.157813, 0.299740, 0, 0.058854, 0.299740, 0, 0.160498, 0.348519, 0, 0.157813, 0.299740, 0, 1.048958, 0.283333, 0, 0.995345, 0.281755, 0, 1.048958, 0.283333, 0, 1.048958, 0.283333, 0, 0.995345, 0.281755, 0, 0.995345, 0.281755, 0, 0.995345, 0.281755, 0, 0.950651, 0.277018, 0, 0.995345, 0.281755, 0, 0.995345, 0.281755, 0, 0.950651, 0.277018, 0, 0.950651, 0.277018, 0, 0.950651, 0.277018, 0, 0.914876, 0.269124, 0, 0.950651, 0.277018, 0, 0.950651, 0.277018, 0, 0.914876, 0.269124, 0, 0.914876, 0.269124, 0, 0.914876, 0.269124, 0, 0.888021, 0.258073, 0, 0.914876, 0.269124, 0, 0.914876, 0.269124, 0, 0.888021, 0.258073, 0, 0.888021, 0.258073, 0, 0.888021, 0.258073, 0, 0.868425, 0.243229, 0, 0.888021, 0.258073, 0, 0.888021, 0.258073, 0, 0.868425, 0.243229, 0, 0.868425, 0.243229, 0, 0.868425, 0.243229, 0, 0.854427, 0.223958, 0, 0.868425, 0.243229, 0, 0.868425, 0.243229, 0, 0.854427, 0.223958, 0, 0.854427, 0.223958, 0, 0.854427, 0.223958, 0, 0.846029, 0.200260, 0, 0.854427, 0.223958, 0, 0.854427, 0.223958, 0, 0.846029, 0.200260, 0, 0.846029, 0.200260, 0, 0.846029, 0.200260, 0, 0.843229, 0.172135, 0, 0.846029, 0.200260, 0, 0.846029, 0.200260, 0, 0.843229, 0.172135, 0, 0.843229, 0.172135, 0, 0.843229, 0.172135, 0, 0.845329, 0.149056, 0, 0.843229, 0.172135, 0, 0.843229, 0.172135, 0, 0.845329, 0.149056, 0, 0.845329, 0.149056, 0, 0.845329, 0.149056, 0, 0.851628, 0.128516, 0, 0.845329, 0.149056, 0, 0.845329, 0.149056, 0, 0.851628, 0.128516, 0, 0.851628, 0.128516, 0, 0.851628, 0.128516, 0, 0.862126, 0.110514, 0, 0.851628, 0.128516, 0, 0.851628, 0.128516, 0, 0.862126, 0.110514, 0, 0.862126, 0.110514, 0, 0.862126, 0.110514, 0, 0.876823, 0.095052, 0, 0.862126, 0.110514, 0, 0.862126, 0.110514, 0, 0.876823, 0.095052, 0, 0.876823, 0.095052, 0, 0.876823, 0.095052, 0, 0.895133, 0.082633, 0, 0.876823, 0.095052, 0, 0.876823, 0.095052, 0, 0.895133, 0.082633, 0, 0.895133, 0.082633, 0, 0.895133, 0.082633, 0, 0.916471, 0.073763, 0, 0.895133, 0.082633, 0, 0.895133, 0.082633, 0, 0.916471, 0.073763, 0, 0.916471, 0.073763, 0, 0.916471, 0.073763, 0, 0.940837, 0.068441, 0, 0.916471, 0.073763, 0, 0.916471, 0.073763, 0, 0.940837, 0.068441, 0, 0.940837, 0.068441, 0, 0.940837, 0.068441, 0, 0.968229, 0.066667, 0, 0.940837, 0.068441, 0, 0.940837, 0.068441, 0, 0.968229, 0.066667, 0, 0.968229, 0.066667, 0, 0.968229, 0.066667, 0, 1.006104, 0.070011, 0, 0.968229, 0.066667, 0, 0.968229, 0.066667, 0, 1.006104, 0.070011, 0, 1.006104, 0.070011, 0, 1.006104, 0.070011, 0, 1.040039, 0.080046, 0, 1.006104, 0.070011, 0, 1.006104, 0.070011, 0, 1.040039, 0.080046, 0, 1.040039, 0.080046, 0, 1.040039, 0.080046, 0, 1.070036, 0.096769, 0, 1.040039, 0.080046, 0, 1.040039, 0.080046, 0, 1.070036, 0.096769, 0, 1.070036, 0.096769, 0, 1.070036, 0.096769, 0, 1.096094, 0.120182, 0, 1.070036, 0.096769, 0, 1.070036, 0.096769, 0, 1.096094, 0.120182, 0, 1.096094, 0.120182, 0, 1.096094, 0.120182, 0, 1.117171, 0.149146, 0, 1.096094, 0.120182, 0, 1.096094, 0.120182, 0, 1.117171, 0.149146, 0, 1.117171, 0.149146, 0, 1.117171, 0.149146, 0, 1.132227, 0.182520, 0, 1.117171, 0.149146, 0, 1.117171, 0.149146, 0, 1.132227, 0.182520, 0, 1.132227, 0.182520, 0, 1.132227, 0.182520, 0, 1.141260, 0.220304, 0, 1.132227, 0.182520, 0, 1.132227, 0.182520, 0, 1.141260, 0.220304, 0, 1.141260, 0.220304, 0, 1.141260, 0.220304, 0, 1.144271, 0.262500, 0, 1.141260, 0.220304, 0, 1.141260, 0.220304, 0, 1.144271, 0.262500, 0, 1.144271, 0.262500, 0, 1.144271, 0.262500, 0, 1.144271, 0.283333, 0, 1.144271, 0.262500, 0, 1.144271, 0.262500, 0, 1.144271, 0.283333, 0, 1.144271, 0.283333, 0, 1.240104, 0.327344, 0, 1.240104, 0, 0, 1.240104, 0.327344, 0, 1.240104, 0.327344, 0, 1.240104, 0, 0, 1.240104, 0, 0, 1.240104, 0, 0, 1.144271, 0, 0, 1.240104, 0, 0, 1.240104, 0, 0, 1.144271, 0, 0, 1.144271, 0, 0, 1.144271, 0, 0, 1.144271, 0.083333, 0, 1.144271, 0, 0, 1.144271, 0, 0, 1.144271, 0.083333, 0, 1.144271, 0.083333, 0, 1.144271, 0.083333, 0, 1.126855, 0.059367, 0, 1.144271, 0.083333, 0, 1.144271, 0.083333, 0, 1.126855, 0.059367, 0, 1.126855, 0.059367, 0, 1.126855, 0.059367, 0, 1.107422, 0.038770, 0, 1.126855, 0.059367, 0, 1.126855, 0.059367, 0, 1.107422, 0.038770, 0, 1.107422, 0.038770, 0, 1.107422, 0.038770, 0, 1.085970, 0.021541, 0, 1.107422, 0.038770, 0, 1.107422, 0.038770, 0, 1.085970, 0.021541, 0, 1.085970, 0.021541, 0, 1.085970, 0.021541, 0, 1.062500, 0.007682, 0, 1.085970, 0.021541, 0, 1.085970, 0.021541, 0, 1.062500, 0.007682, 0, 1.062500, 0.007682, 0, 1.062500, 0.007682, 0, 1.036654, -0.002970, 0, 1.062500, 0.007682, 0, 1.062500, 0.007682, 0, 1.036654, -0.002970, 0, 1.036654, -0.002970, 0, 1.036654, -0.002970, 0, 1.008073, -0.010579, 0, 1.036654, -0.002970, 0, 1.036654, -0.002970, 0, 1.008073, -0.010579, 0, 1.008073, -0.010579, 0, 1.008073, -0.010579, 0, 0.976758, -0.015145, 0, 1.008073, -0.010579, 0, 1.008073, -0.010579, 0, 0.976758, -0.015145, 0, 0.976758, -0.015145, 0, 0.976758, -0.015145, 0, 0.942708, -0.016667, 0, 0.976758, -0.015145, 0, 0.976758, -0.015145, 0, 0.942708, -0.016667, 0, 0.942708, -0.016667, 0, 0.942708, -0.016667, 0, 0.900212, -0.013525, 0, 0.942708, -0.016667, 0, 0.942708, -0.016667, 0, 0.900212, -0.013525, 0, 0.900212, -0.013525, 0, 0.900212, -0.013525, 0, 0.862305, -0.004102, 0, 0.900212, -0.013525, 0, 0.900212, -0.013525, 0, 0.862305, -0.004102, 0, 0.862305, -0.004102, 0, 0.862305, -0.004102, 0, 0.828988, 0.011605, 0, 0.862305, -0.004102, 0, 0.862305, -0.004102, 0, 0.828988, 0.011605, 0, 0.828988, 0.011605, 0, 0.828988, 0.011605, 0, 0.800260, 0.033594, 0, 0.828988, 0.011605, 0, 0.828988, 0.011605, 0, 0.800260, 0.033594, 0, 0.800260, 0.033594, 0, 0.800260, 0.033594, 0, 0.777132, 0.060856, 0, 0.800260, 0.033594, 0, 0.800260, 0.033594, 0, 0.777132, 0.060856, 0, 0.777132, 0.060856, 0, 0.777132, 0.060856, 0, 0.760612, 0.092383, 0, 0.777132, 0.060856, 0, 0.777132, 0.060856, 0, 0.760612, 0.092383, 0, 0.760612, 0.092383, 0, 0.760612, 0.092383, 0, 0.750700, 0.128174, 0, 0.760612, 0.092383, 0, 0.760612, 0.092383, 0, 0.750700, 0.128174, 0, 0.750700, 0.128174, 0, 0.750700, 0.128174, 0, 0.747396, 0.168229, 0, 0.750700, 0.128174, 0, 0.750700, 0.128174, 0, 0.747396, 0.168229, 0, 0.747396, 0.168229, 0, 0.747396, 0.168229, 0, 0.751514, 0.214421, 0, 0.747396, 0.168229, 0, 0.747396, 0.168229, 0, 0.751514, 0.214421, 0, 0.751514, 0.214421, 0, 0.751514, 0.214421, 0, 0.763867, 0.254557, 0, 0.751514, 0.214421, 0, 0.751514, 0.214421, 0, 0.763867, 0.254557, 0, 0.763867, 0.254557, 0, 0.763867, 0.254557, 0, 0.784456, 0.288639, 0, 0.763867, 0.254557, 0, 0.763867, 0.254557, 0, 0.784456, 0.288639, 0, 0.784456, 0.288639, 0, 0.784456, 0.288639, 0, 0.813281, 0.316667, 0, 0.784456, 0.288639, 0, 0.784456, 0.288639, 0, 0.813281, 0.316667, 0, 0.813281, 0.316667, 0, 0.813281, 0.316667, 0, 0.850277, 0.338542, 0, 0.813281, 0.316667, 0, 0.813281, 0.316667, 0, 0.850277, 0.338542, 0, 0.850277, 0.338542, 0, 0.850277, 0.338542, 0, 0.895378, 0.354167, 0, 0.850277, 0.338542, 0, 0.850277, 0.338542, 0, 0.895378, 0.354167, 0, 0.895378, 0.354167, 0, 0.895378, 0.354167, 0, 0.948584, 0.363542, 0, 0.895378, 0.354167, 0, 0.895378, 0.354167, 0, 0.948584, 0.363542, 0, 0.948584, 0.363542, 0, 0.948584, 0.363542, 0, 1.009896, 0.366667, 0, 0.948584, 0.363542, 0, 0.948584, 0.363542, 0, 1.009896, 0.366667, 0, 1.009896, 0.366667, 0, 1.009896, 0.366667, 0, 1.144271, 0.366667, 0, 1.009896, 0.366667, 0, 1.009896, 0.366667, 0, 1.144271, 0.366667, 0, 1.144271, 0.366667, 0, 1.144271, 0.366667, 0, 1.144271, 0.376042, 0, 1.144271, 0.366667, 0, 1.144271, 0.366667, 0, 1.144271, 0.376042, 0, 1.144271, 0.376042, 0, 1.144271, 0.376042, 0, 1.141553, 0.407731, 0, 1.144271, 0.376042, 0, 1.144271, 0.376042, 0, 1.141553, 0.407731, 0, 1.141553, 0.407731, 0, 1.141553, 0.407731, 0, 1.133398, 0.435612, 0, 1.141553, 0.407731, 0, 1.141553, 0.407731, 0, 1.133398, 0.435612, 0, 1.133398, 0.435612, 0, 1.133398, 0.435612, 0, 1.119808, 0.459684, 0, 1.133398, 0.435612, 0, 1.133398, 0.435612, 0, 1.119808, 0.459684, 0, 1.119808, 0.459684, 0, 1.119808, 0.459684, 0, 1.100781, 0.479948, 0, 1.119808, 0.459684, 0, 1.119808, 0.459684, 0, 1.100781, 0.479948, 0, 1.100781, 0.479948, 0, 1.100781, 0.479948, 0, 1.076839, 0.496012, 0, 1.100781, 0.479948, 0, 1.100781, 0.479948, 0, 1.076839, 0.496012, 0, 1.076839, 0.496012, 0, 1.076839, 0.496012, 0, 1.048503, 0.507487, 0, 1.076839, 0.496012, 0, 1.076839, 0.496012, 0, 1.048503, 0.507487, 0, 1.048503, 0.507487, 0, 1.048503, 0.507487, 0, 1.015772, 0.514372, 0, 1.048503, 0.507487, 0, 1.048503, 0.507487, 0, 1.015772, 0.514372, 0, 1.015772, 0.514372, 0, 1.015772, 0.514372, 0, 0.978646, 0.516667, 0, 1.015772, 0.514372, 0, 1.015772, 0.514372, 0, 0.978646, 0.516667, 0, 0.978646, 0.516667, 0, 0.978646, 0.516667, 0, 0.953809, 0.515885, 0, 0.978646, 0.516667, 0, 0.978646, 0.516667, 0, 0.953809, 0.515885, 0, 0.953809, 0.515885, 0, 0.953809, 0.515885, 0, 0.929297, 0.513542, 0, 0.953809, 0.515885, 0, 0.953809, 0.515885, 0, 0.929297, 0.513542, 0, 0.929297, 0.513542, 0, 0.929297, 0.513542, 0, 0.905111, 0.509635, 0, 0.929297, 0.513542, 0, 0.929297, 0.513542, 0, 0.905111, 0.509635, 0, 0.905111, 0.509635, 0, 0.905111, 0.509635, 0, 0.881250, 0.504167, 0, 0.905111, 0.509635, 0, 0.905111, 0.509635, 0, 0.881250, 0.504167, 0, 0.881250, 0.504167, 0, 0.881250, 0.504167, 0, 0.857780, 0.497135, 0, 0.881250, 0.504167, 0, 0.881250, 0.504167, 0, 0.857780, 0.497135, 0, 0.857780, 0.497135, 0, 0.857780, 0.497135, 0, 0.834766, 0.488542, 0, 0.857780, 0.497135, 0, 0.857780, 0.497135, 0, 0.834766, 0.488542, 0, 0.834766, 0.488542, 0, 0.834766, 0.488542, 0, 0.812207, 0.478385, 0, 0.834766, 0.488542, 0, 0.834766, 0.488542, 0, 0.812207, 0.478385, 0, 0.812207, 0.478385, 0, 0.812207, 0.478385, 0, 0.790104, 0.466667, 0, 0.812207, 0.478385, 0, 0.812207, 0.478385, 0, 0.790104, 0.466667, 0, 0.790104, 0.466667, 0, 0.790104, 0.466667, 0, 0.790104, 0.550000, 0, 0.790104, 0.466667, 0, 0.790104, 0.466667, 0, 0.790104, 0.550000, 0, 0.790104, 0.550000, 0, 0.790104, 0.550000, 0, 0.816211, 0.561719, 0, 0.790104, 0.550000, 0, 0.790104, 0.550000, 0, 0.816211, 0.561719, 0, 0.816211, 0.561719, 0, 0.816211, 0.561719, 0, 0.841927, 0.571875, 0, 0.816211, 0.561719, 0, 0.816211, 0.561719, 0, 0.841927, 0.571875, 0, 0.841927, 0.571875, 0, 0.841927, 0.571875, 0, 0.867253, 0.580469, 0, 0.841927, 0.571875, 0, 0.841927, 0.571875, 0, 0.867253, 0.580469, 0, 0.867253, 0.580469, 0, 0.867253, 0.580469, 0, 0.892188, 0.587500, 0, 0.867253, 0.580469, 0, 0.867253, 0.580469, 0, 0.892188, 0.587500, 0, 0.892188, 0.587500, 0, 0.892188, 0.587500, 0, 0.916764, 0.592969, 0, 0.892188, 0.587500, 0, 0.892188, 0.587500, 0, 0.916764, 0.592969, 0, 0.916764, 0.592969, 0, 0.916764, 0.592969, 0, 0.941016, 0.596875, 0, 0.916764, 0.592969, 0, 0.916764, 0.592969, 0, 0.941016, 0.596875, 0, 0.941016, 0.596875, 0, 0.941016, 0.596875, 0, 0.964941, 0.599219, 0, 0.941016, 0.596875, 0, 0.941016, 0.596875, 0, 0.964941, 0.599219, 0, 0.964941, 0.599219, 0, 0.964941, 0.599219, 0, 0.988542, 0.600000, 0, 0.964941, 0.599219, 0, 0.964941, 0.599219, 0, 0.988542, 0.600000, 0, 0.988542, 0.600000, 0, 0.988542, 0.600000, 0, 1.047819, 0.595776, 0, 0.988542, 0.600000, 0, 0.988542, 0.600000, 0, 1.047819, 0.595776, 0, 1.047819, 0.595776, 0, 1.047819, 0.595776, 0, 1.099089, 0.583106, 0, 1.047819, 0.595776, 0, 1.047819, 0.595776, 0, 1.099089, 0.583106, 0, 1.099089, 0.583106, 0, 1.099089, 0.583106, 0, 1.142350, 0.561987, 0, 1.099089, 0.583106, 0, 1.099089, 0.583106, 0, 1.142350, 0.561987, 0, 1.142350, 0.561987, 0, 1.142350, 0.561987, 0, 1.177604, 0.532422, 0, 1.142350, 0.561987, 0, 1.142350, 0.561987, 0, 1.177604, 0.532422, 0, 1.177604, 0.532422, 0, 1.177604, 0.532422, 0, 1.204948, 0.494263, 0, 1.177604, 0.532422, 0, 1.177604, 0.532422, 0, 1.204948, 0.494263, 0, 1.204948, 0.494263, 0, 1.204948, 0.494263, 0, 1.224479, 0.447363, 0, 1.204948, 0.494263, 0, 1.204948, 0.494263, 0, 1.224479, 0.447363, 0, 1.224479, 0.447363, 0, 1.224479, 0.447363, 0, 1.236198, 0.391724, 0, 1.224479, 0.447363, 0, 1.224479, 0.447363, 0, 1.236198, 0.391724, 0, 1.236198, 0.391724, 0, 0.751514, 0.214421, 0, 0.747396, 0.168229, 0, 0.843229, 0.172135, 0, 0.747396, 0.168229, 0, 0.751514, 0.214421, 0, 0.843229, 0.172135, 0, 0.751514, 0.214421, 0, 0.843229, 0.172135, 0, 0.846029, 0.200260, 0, 0.843229, 0.172135, 0, 0.751514, 0.214421, 0, 0.846029, 0.200260, 0, 0.763867, 0.254557, 0, 0.751514, 0.214421, 0, 0.846029, 0.200260, 0, 0.751514, 0.214421, 0, 0.763867, 0.254557, 0, 0.846029, 0.200260, 0, 0.763867, 0.254557, 0, 0.846029, 0.200260, 0, 0.854427, 0.223958, 0, 0.846029, 0.200260, 0, 0.763867, 0.254557, 0, 0.854427, 0.223958, 0, 0.784456, 0.288639, 0, 0.763867, 0.254557, 0, 0.854427, 0.223958, 0, 0.763867, 0.254557, 0, 0.784456, 0.288639, 0, 0.854427, 0.223958, 0, 0.784456, 0.288639, 0, 0.854427, 0.223958, 0, 0.868425, 0.243229, 0, 0.854427, 0.223958, 0, 0.784456, 0.288639, 0, 0.868425, 0.243229, 0, 0.813281, 0.316667, 0, 0.784456, 0.288639, 0, 0.868425, 0.243229, 0, 0.784456, 0.288639, 0, 0.813281, 0.316667, 0, 0.868425, 0.243229, 0, 0.813281, 0.316667, 0, 0.868425, 0.243229, 0, 0.888021, 0.258073, 0, 0.868425, 0.243229, 0, 0.813281, 0.316667, 0, 0.888021, 0.258073, 0, 0.850277, 0.338542, 0, 0.813281, 0.316667, 0, 0.888021, 0.258073, 0, 0.813281, 0.316667, 0, 0.850277, 0.338542, 0, 0.888021, 0.258073, 0, 0.914876, 0.269124, 0, 0.850277, 0.338542, 0, 0.888021, 0.258073, 0, 0.850277, 0.338542, 0, 0.914876, 0.269124, 0, 0.888021, 0.258073, 0, 0.895378, 0.354167, 0, 0.850277, 0.338542, 0, 0.914876, 0.269124, 0, 0.850277, 0.338542, 0, 0.895378, 0.354167, 0, 0.914876, 0.269124, 0, 0.950651, 0.277018, 0, 0.895378, 0.354167, 0, 0.914876, 0.269124, 0, 0.895378, 0.354167, 0, 0.950651, 0.277018, 0, 0.914876, 0.269124, 0, 0.948584, 0.363542, 0, 0.895378, 0.354167, 0, 0.950651, 0.277018, 0, 0.895378, 0.354167, 0, 0.948584, 0.363542, 0, 0.950651, 0.277018, 0, 0.948584, 0.363542, 0, 0.950651, 0.277018, 0, 0.995345, 0.281755, 0, 0.950651, 0.277018, 0, 0.948584, 0.363542, 0, 0.995345, 0.281755, 0, 0.948584, 0.363542, 0, 0.995345, 0.281755, 0, 1.009896, 0.366667, 0, 0.995345, 0.281755, 0, 0.948584, 0.363542, 0, 1.009896, 0.366667, 0, 1.009896, 0.366667, 0, 0.995345, 0.281755, 0, 1.048958, 0.283333, 0, 0.995345, 0.281755, 0, 1.009896, 0.366667, 0, 1.048958, 0.283333, 0, 1.009896, 0.366667, 0, 1.048958, 0.283333, 0, 1.144271, 0.366667, 0, 1.048958, 0.283333, 0, 1.009896, 0.366667, 0, 1.144271, 0.366667, 0, 1.048958, 0.283333, 0, 1.144271, 0.283333, 0, 1.144271, 0.366667, 0, 1.144271, 0.283333, 0, 1.048958, 0.283333, 0, 1.144271, 0.366667, 0, 1.144271, 0.366667, 0, 1.144271, 0.283333, 0, 1.240104, 0.327344, 0, 1.144271, 0.283333, 0, 1.144271, 0.366667, 0, 1.240104, 0.327344, 0, 1.240104, 0.327344, 0, 1.236198, 0.391724, 0, 1.144271, 0.366667, 0, 1.236198, 0.391724, 0, 1.240104, 0.327344, 0, 1.144271, 0.366667, 0, 1.236198, 0.391724, 0, 1.144271, 0.376042, 0, 1.144271, 0.366667, 0, 1.144271, 0.376042, 0, 1.236198, 0.391724, 0, 1.144271, 0.366667, 0, 1.141553, 0.407731, 0, 1.144271, 0.376042, 0, 1.236198, 0.391724, 0, 1.144271, 0.376042, 0, 1.141553, 0.407731, 0, 1.236198, 0.391724, 0, 1.224479, 0.447363, 0, 1.141553, 0.407731, 0, 1.236198, 0.391724, 0, 1.141553, 0.407731, 0, 1.224479, 0.447363, 0, 1.236198, 0.391724, 0, 1.133398, 0.435612, 0, 1.141553, 0.407731, 0, 1.224479, 0.447363, 0, 1.141553, 0.407731, 0, 1.133398, 0.435612, 0, 1.224479, 0.447363, 0, 1.204948, 0.494263, 0, 1.133398, 0.435612, 0, 1.224479, 0.447363, 0, 1.133398, 0.435612, 0, 1.204948, 0.494263, 0, 1.224479, 0.447363, 0, 1.119808, 0.459684, 0, 1.133398, 0.435612, 0, 1.204948, 0.494263, 0, 1.133398, 0.435612, 0, 1.119808, 0.459684, 0, 1.204948, 0.494263, 0, 1.177604, 0.532422, 0, 1.119808, 0.459684, 0, 1.204948, 0.494263, 0, 1.119808, 0.459684, 0, 1.177604, 0.532422, 0, 1.204948, 0.494263, 0, 1.100781, 0.479948, 0, 1.119808, 0.459684, 0, 1.177604, 0.532422, 0, 1.119808, 0.459684, 0, 1.100781, 0.479948, 0, 1.177604, 0.532422, 0, 1.142350, 0.561987, 0, 1.100781, 0.479948, 0, 1.177604, 0.532422, 0, 1.100781, 0.479948, 0, 1.142350, 0.561987, 0, 1.177604, 0.532422, 0, 1.076839, 0.496012, 0, 1.100781, 0.479948, 0, 1.142350, 0.561987, 0, 1.100781, 0.479948, 0, 1.076839, 0.496012, 0, 1.142350, 0.561987, 0, 1.099089, 0.583106, 0, 1.076839, 0.496012, 0, 1.142350, 0.561987, 0, 1.076839, 0.496012, 0, 1.099089, 0.583106, 0, 1.142350, 0.561987, 0, 1.048503, 0.507487, 0, 1.076839, 0.496012, 0, 1.099089, 0.583106, 0, 1.076839, 0.496012, 0, 1.048503, 0.507487, 0, 1.099089, 0.583106, 0, 1.047819, 0.595776, 0, 1.048503, 0.507487, 0, 1.099089, 0.583106, 0, 1.048503, 0.507487, 0, 1.047819, 0.595776, 0, 1.099089, 0.583106, 0, 1.047819, 0.595776, 0, 1.015772, 0.514372, 0, 1.048503, 0.507487, 0, 1.015772, 0.514372, 0, 1.047819, 0.595776, 0, 1.048503, 0.507487, 0, 1.047819, 0.595776, 0, 0.988542, 0.600000, 0, 1.015772, 0.514372, 0, 0.988542, 0.600000, 0, 1.047819, 0.595776, 0, 1.015772, 0.514372, 0, 0.978646, 0.516667, 0, 1.015772, 0.514372, 0, 0.988542, 0.600000, 0, 1.015772, 0.514372, 0, 0.978646, 0.516667, 0, 0.988542, 0.600000, 0, 0.988542, 0.600000, 0, 0.964941, 0.599219, 0, 0.978646, 0.516667, 0, 0.964941, 0.599219, 0, 0.988542, 0.600000, 0, 0.978646, 0.516667, 0, 0.964941, 0.599219, 0, 0.953809, 0.515885, 0, 0.978646, 0.516667, 0, 0.953809, 0.515885, 0, 0.964941, 0.599219, 0, 0.978646, 0.516667, 0, 0.941016, 0.596875, 0, 0.953809, 0.515885, 0, 0.964941, 0.599219, 0, 0.953809, 0.515885, 0, 0.941016, 0.596875, 0, 0.964941, 0.599219, 0, 0.941016, 0.596875, 0, 0.929297, 0.513542, 0, 0.953809, 0.515885, 0, 0.929297, 0.513542, 0, 0.941016, 0.596875, 0, 0.953809, 0.515885, 0, 0.916764, 0.592969, 0, 0.929297, 0.513542, 0, 0.941016, 0.596875, 0, 0.929297, 0.513542, 0, 0.916764, 0.592969, 0, 0.941016, 0.596875, 0, 0.916764, 0.592969, 0, 0.892188, 0.587500, 0, 0.929297, 0.513542, 0, 0.892188, 0.587500, 0, 0.916764, 0.592969, 0, 0.929297, 0.513542, 0, 0.929297, 0.513542, 0, 0.892188, 0.587500, 0, 0.905111, 0.509635, 0, 0.892188, 0.587500, 0, 0.929297, 0.513542, 0, 0.905111, 0.509635, 0, 0.892188, 0.587500, 0, 0.867253, 0.580469, 0, 0.905111, 0.509635, 0, 0.867253, 0.580469, 0, 0.892188, 0.587500, 0, 0.905111, 0.509635, 0, 0.867253, 0.580469, 0, 0.881250, 0.504167, 0, 0.905111, 0.509635, 0, 0.881250, 0.504167, 0, 0.867253, 0.580469, 0, 0.905111, 0.509635, 0, 0.867253, 0.580469, 0, 0.841927, 0.571875, 0, 0.881250, 0.504167, 0, 0.841927, 0.571875, 0, 0.867253, 0.580469, 0, 0.881250, 0.504167, 0, 0.841927, 0.571875, 0, 0.857780, 0.497135, 0, 0.881250, 0.504167, 0, 0.857780, 0.497135, 0, 0.841927, 0.571875, 0, 0.881250, 0.504167, 0, 0.841927, 0.571875, 0, 0.816211, 0.561719, 0, 0.857780, 0.497135, 0, 0.816211, 0.561719, 0, 0.841927, 0.571875, 0, 0.857780, 0.497135, 0, 0.816211, 0.561719, 0, 0.834766, 0.488542, 0, 0.857780, 0.497135, 0, 0.834766, 0.488542, 0, 0.816211, 0.561719, 0, 0.857780, 0.497135, 0, 0.816211, 0.561719, 0, 0.790104, 0.550000, 0, 0.834766, 0.488542, 0, 0.790104, 0.550000, 0, 0.816211, 0.561719, 0, 0.834766, 0.488542, 0, 0.790104, 0.550000, 0, 0.812207, 0.478385, 0, 0.834766, 0.488542, 0, 0.812207, 0.478385, 0, 0.790104, 0.550000, 0, 0.834766, 0.488542, 0, 0.790104, 0.550000, 0, 0.790104, 0.466667, 0, 0.812207, 0.478385, 0, 0.790104, 0.466667, 0, 0.790104, 0.550000, 0, 0.812207, 0.478385, 0, 1.240104, 0.327344, 0, 1.144271, 0.283333, 0, 1.144271, 0.262500, 0, 1.144271, 0.283333, 0, 1.240104, 0.327344, 0, 1.144271, 0.262500, 0, 1.240104, 0.327344, 0, 1.144271, 0.262500, 0, 1.141260, 0.220304, 0, 1.144271, 0.262500, 0, 1.240104, 0.327344, 0, 1.141260, 0.220304, 0, 1.240104, 0, 0, 1.240104, 0.327344, 0, 1.141260, 0.220304, 0, 1.240104, 0.327344, 0, 1.240104, 0, 0, 1.141260, 0.220304, 0, 1.240104, 0, 0, 1.141260, 0.220304, 0, 1.132227, 0.182520, 0, 1.141260, 0.220304, 0, 1.240104, 0, 0, 1.132227, 0.182520, 0, 1.144271, 0.083333, 0, 1.240104, 0, 0, 1.132227, 0.182520, 0, 1.240104, 0, 0, 1.144271, 0.083333, 0, 1.132227, 0.182520, 0, 1.144271, 0.083333, 0, 1.144271, 0, 0, 1.240104, 0, 0, 1.144271, 0, 0, 1.144271, 0.083333, 0, 1.240104, 0, 0, 1.132227, 0.182520, 0, 1.117171, 0.149146, 0, 1.144271, 0.083333, 0, 1.117171, 0.149146, 0, 1.132227, 0.182520, 0, 1.144271, 0.083333, 0, 1.117171, 0.149146, 0, 1.096094, 0.120182, 0, 1.144271, 0.083333, 0, 1.096094, 0.120182, 0, 1.117171, 0.149146, 0, 1.144271, 0.083333, 0, 1.096094, 0.120182, 0, 1.126855, 0.059367, 0, 1.144271, 0.083333, 0, 1.126855, 0.059367, 0, 1.096094, 0.120182, 0, 1.144271, 0.083333, 0, 1.096094, 0.120182, 0, 1.070036, 0.096769, 0, 1.126855, 0.059367, 0, 1.070036, 0.096769, 0, 1.096094, 0.120182, 0, 1.126855, 0.059367, 0, 1.070036, 0.096769, 0, 1.107422, 0.038770, 0, 1.126855, 0.059367, 0, 1.107422, 0.038770, 0, 1.070036, 0.096769, 0, 1.126855, 0.059367, 0, 1.107422, 0.038770, 0, 1.070036, 0.096769, 0, 1.085970, 0.021541, 0, 1.070036, 0.096769, 0, 1.107422, 0.038770, 0, 1.085970, 0.021541, 0, 1.070036, 0.096769, 0, 1.040039, 0.080046, 0, 1.085970, 0.021541, 0, 1.040039, 0.080046, 0, 1.070036, 0.096769, 0, 1.085970, 0.021541, 0, 1.040039, 0.080046, 0, 1.062500, 0.007682, 0, 1.085970, 0.021541, 0, 1.062500, 0.007682, 0, 1.040039, 0.080046, 0, 1.085970, 0.021541, 0, 1.062500, 0.007682, 0, 1.040039, 0.080046, 0, 1.036654, -0.002970, 0, 1.040039, 0.080046, 0, 1.062500, 0.007682, 0, 1.036654, -0.002970, 0, 1.040039, 0.080046, 0, 1.006104, 0.070011, 0, 1.036654, -0.002970, 0, 1.006104, 0.070011, 0, 1.040039, 0.080046, 0, 1.036654, -0.002970, 0, 1.006104, 0.070011, 0, 1.008073, -0.010579, 0, 1.036654, -0.002970, 0, 1.008073, -0.010579, 0, 1.006104, 0.070011, 0, 1.036654, -0.002970, 0, 1.006104, 0.070011, 0, 0.968229, 0.066667, 0, 1.008073, -0.010579, 0, 0.968229, 0.066667, 0, 1.006104, 0.070011, 0, 1.008073, -0.010579, 0, 0.968229, 0.066667, 0, 0.976758, -0.015145, 0, 1.008073, -0.010579, 0, 0.976758, -0.015145, 0, 0.968229, 0.066667, 0, 1.008073, -0.010579, 0, 0.968229, 0.066667, 0, 0.942708, -0.016667, 0, 0.976758, -0.015145, 0, 0.942708, -0.016667, 0, 0.968229, 0.066667, 0, 0.976758, -0.015145, 0, 0.940837, 0.068441, 0, 0.942708, -0.016667, 0, 0.968229, 0.066667, 0, 0.942708, -0.016667, 0, 0.940837, 0.068441, 0, 0.968229, 0.066667, 0, 0.940837, 0.068441, 0, 0.900212, -0.013525, 0, 0.942708, -0.016667, 0, 0.900212, -0.013525, 0, 0.940837, 0.068441, 0, 0.942708, -0.016667, 0, 0.916471, 0.073763, 0, 0.900212, -0.013525, 0, 0.940837, 0.068441, 0, 0.900212, -0.013525, 0, 0.916471, 0.073763, 0, 0.940837, 0.068441, 0, 0.862305, -0.004102, 0, 0.900212, -0.013525, 0, 0.916471, 0.073763, 0, 0.900212, -0.013525, 0, 0.862305, -0.004102, 0, 0.916471, 0.073763, 0, 0.895133, 0.082633, 0, 0.862305, -0.004102, 0, 0.916471, 0.073763, 0, 0.862305, -0.004102, 0, 0.895133, 0.082633, 0, 0.916471, 0.073763, 0, 0.828988, 0.011605, 0, 0.862305, -0.004102, 0, 0.895133, 0.082633, 0, 0.862305, -0.004102, 0, 0.828988, 0.011605, 0, 0.895133, 0.082633, 0, 0.895133, 0.082633, 0, 0.876823, 0.095052, 0, 0.828988, 0.011605, 0, 0.876823, 0.095052, 0, 0.895133, 0.082633, 0, 0.828988, 0.011605, 0, 0.876823, 0.095052, 0, 0.800260, 0.033594, 0, 0.828988, 0.011605, 0, 0.800260, 0.033594, 0, 0.876823, 0.095052, 0, 0.828988, 0.011605, 0, 0.876823, 0.095052, 0, 0.862126, 0.110514, 0, 0.800260, 0.033594, 0, 0.862126, 0.110514, 0, 0.876823, 0.095052, 0, 0.800260, 0.033594, 0, 0.862126, 0.110514, 0, 0.777132, 0.060856, 0, 0.800260, 0.033594, 0, 0.777132, 0.060856, 0, 0.862126, 0.110514, 0, 0.800260, 0.033594, 0, 0.862126, 0.110514, 0, 0.851628, 0.128516, 0, 0.777132, 0.060856, 0, 0.851628, 0.128516, 0, 0.862126, 0.110514, 0, 0.777132, 0.060856, 0, 0.851628, 0.128516, 0, 0.760612, 0.092383, 0, 0.777132, 0.060856, 0, 0.760612, 0.092383, 0, 0.851628, 0.128516, 0, 0.777132, 0.060856, 0, 0.750700, 0.128174, 0, 0.760612, 0.092383, 0, 0.851628, 0.128516, 0, 0.760612, 0.092383, 0, 0.750700, 0.128174, 0, 0.851628, 0.128516, 0, 0.845329, 0.149056, 0, 0.750700, 0.128174, 0, 0.851628, 0.128516, 0, 0.750700, 0.128174, 0, 0.845329, 0.149056, 0, 0.851628, 0.128516, 0, 0.747396, 0.168229, 0, 0.750700, 0.128174, 0, 0.845329, 0.149056, 0, 0.750700, 0.128174, 0, 0.747396, 0.168229, 0, 0.845329, 0.149056, 0, 0.843229, 0.172135, 0, 0.747396, 0.168229, 0, 0.845329, 0.149056, 0, 0.747396, 0.168229, 0, 0.843229, 0.172135, 0, 0.845329, 0.149056, 0, 1.888021, 0.478385, 0, 1.906869, 0.507471, 0, 1.888021, 0.478385, 0, 1.888021, 0.478385, 0, 1.906869, 0.507471, 0, 1.906869, 0.507471, 0, 1.906869, 0.507471, 0, 1.927474, 0.532487, 0, 1.906869, 0.507471, 0, 1.906869, 0.507471, 0, 1.927474, 0.532487, 0, 1.927474, 0.532487, 0, 1.927474, 0.532487, 0, 1.949837, 0.553434, 0, 1.927474, 0.532487, 0, 1.927474, 0.532487, 0, 1.949837, 0.553434, 0, 1.949837, 0.553434, 0, 1.949837, 0.553434, 0, 1.973958, 0.570313, 0, 1.949837, 0.553434, 0, 1.949837, 0.553434, 0, 1.973958, 0.570313, 0, 1.973958, 0.570313, 0, 1.973958, 0.570313, 0, 2.000065, 0.583301, 0, 1.973958, 0.570313, 0, 1.973958, 0.570313, 0, 2.000065, 0.583301, 0, 2.000065, 0.583301, 0, 2.000065, 0.583301, 0, 2.028385, 0.592578, 0, 2.000065, 0.583301, 0, 2.000065, 0.583301, 0, 2.028385, 0.592578, 0, 2.028385, 0.592578, 0, 2.028385, 0.592578, 0, 2.058919, 0.598145, 0, 2.028385, 0.592578, 0, 2.028385, 0.592578, 0, 2.058919, 0.598145, 0, 2.058919, 0.598145, 0, 2.058919, 0.598145, 0, 2.091667, 0.600000, 0, 2.058919, 0.598145, 0, 2.058919, 0.598145, 0, 2.091667, 0.600000, 0, 2.091667, 0.600000, 0, 2.091667, 0.600000, 0, 2.134635, 0.596053, 0, 2.091667, 0.600000, 0, 2.091667, 0.600000, 0, 2.134635, 0.596053, 0, 2.134635, 0.596053, 0, 2.134635, 0.596053, 0, 2.172396, 0.584212, 0, 2.134635, 0.596053, 0, 2.134635, 0.596053, 0, 2.172396, 0.584212, 0, 2.172396, 0.584212, 0, 2.172396, 0.584212, 0, 2.204948, 0.564478, 0, 2.172396, 0.584212, 0, 2.172396, 0.584212, 0, 2.204948, 0.564478, 0, 2.204948, 0.564478, 0, 2.204948, 0.564478, 0, 2.232292, 0.536849, 0, 2.204948, 0.564478, 0, 2.204948, 0.564478, 0, 2.232292, 0.536849, 0, 2.232292, 0.536849, 0, 2.232292, 0.536849, 0, 2.253939, 0.501945, 0, 2.232292, 0.536849, 0, 2.232292, 0.536849, 0, 2.253939, 0.501945, 0, 2.253939, 0.501945, 0, 2.253939, 0.501945, 0, 2.269401, 0.460384, 0, 2.253939, 0.501945, 0, 2.253939, 0.501945, 0, 2.269401, 0.460384, 0, 2.269401, 0.460384, 0, 2.269401, 0.460384, 0, 2.278678, 0.412166, 0, 2.269401, 0.460384, 0, 2.269401, 0.460384, 0, 2.278678, 0.412166, 0, 2.278678, 0.412166, 0, 2.278678, 0.412166, 0, 2.281771, 0.357292, 0, 2.278678, 0.412166, 0, 2.278678, 0.412166, 0, 2.281771, 0.357292, 0, 2.281771, 0.357292, 0, 2.281771, 0.357292, 0, 2.281771, 0, 0, 2.281771, 0.357292, 0, 2.281771, 0.357292, 0, 2.281771, 0, 0, 2.281771, 0, 0, 2.281771, 0, 0, 2.185417, 0, 0, 2.281771, 0, 0, 2.281771, 0, 0, 2.185417, 0, 0, 2.185417, 0, 0, 2.185417, 0, 0, 2.185417, 0.354167, 0, 2.185417, 0, 0, 2.185417, 0, 0, 2.185417, 0.354167, 0, 2.185417, 0.354167, 0, 2.185417, 0.354167, 0, 2.183561, 0.392782, 0, 2.185417, 0.354167, 0, 2.185417, 0.354167, 0, 2.183561, 0.392782, 0, 2.183561, 0.392782, 0, 2.183561, 0.392782, 0, 2.177995, 0.426074, 0, 2.183561, 0.392782, 0, 2.183561, 0.392782, 0, 2.177995, 0.426074, 0, 2.177995, 0.426074, 0, 2.177995, 0.426074, 0, 2.168717, 0.454045, 0, 2.177995, 0.426074, 0, 2.177995, 0.426074, 0, 2.168717, 0.454045, 0, 2.168717, 0.454045, 0, 2.168717, 0.454045, 0, 2.155729, 0.476693, 0, 2.168717, 0.454045, 0, 2.168717, 0.454045, 0, 2.155729, 0.476693, 0, 2.155729, 0.476693, 0, 2.155729, 0.476693, 0, 2.138932, 0.494181, 0, 2.155729, 0.476693, 0, 2.155729, 0.476693, 0, 2.138932, 0.494181, 0, 2.138932, 0.494181, 0, 2.138932, 0.494181, 0, 2.118229, 0.506673, 0, 2.138932, 0.494181, 0, 2.138932, 0.494181, 0, 2.118229, 0.506673, 0, 2.118229, 0.506673, 0, 2.118229, 0.506673, 0, 2.093620, 0.514168, 0, 2.118229, 0.506673, 0, 2.118229, 0.506673, 0, 2.093620, 0.514168, 0, 2.093620, 0.514168, 0, 2.093620, 0.514168, 0, 2.065104, 0.516667, 0, 2.093620, 0.514168, 0, 2.093620, 0.514168, 0, 2.065104, 0.516667, 0, 2.065104, 0.516667, 0, 2.065104, 0.516667, 0, 2.029818, 0.513615, 0, 2.065104, 0.516667, 0, 2.065104, 0.516667, 0, 2.029818, 0.513615, 0, 2.029818, 0.513615, 0, 2.029818, 0.513615, 0, 1.998438, 0.504460, 0, 2.029818, 0.513615, 0, 2.029818, 0.513615, 0, 1.998438, 0.504460, 0, 1.998438, 0.504460, 0, 1.998438, 0.504460, 0, 1.970963, 0.489201, 0, 1.998438, 0.504460, 0, 1.998438, 0.504460, 0, 1.970963, 0.489201, 0, 1.970963, 0.489201, 0, 1.970963, 0.489201, 0, 1.947396, 0.467839, 0, 1.970963, 0.489201, 0, 1.970963, 0.489201, 0, 1.947396, 0.467839, 0, 1.947396, 0.467839, 0, 1.947396, 0.467839, 0, 1.928483, 0.441203, 0, 1.947396, 0.467839, 0, 1.947396, 0.467839, 0, 1.928483, 0.441203, 0, 1.928483, 0.441203, 0, 1.928483, 0.441203, 0, 1.914974, 0.410124, 0, 1.928483, 0.441203, 0, 1.928483, 0.441203, 0, 1.914974, 0.410124, 0, 1.914974, 0.410124, 0, 1.914974, 0.410124, 0, 1.906869, 0.374601, 0, 1.914974, 0.410124, 0, 1.914974, 0.410124, 0, 1.906869, 0.374601, 0, 1.906869, 0.374601, 0, 1.906869, 0.374601, 0, 1.904167, 0.334635, 0, 1.906869, 0.374601, 0, 1.906869, 0.374601, 0, 1.904167, 0.334635, 0, 1.904167, 0.334635, 0, 1.904167, 0.334635, 0, 1.904167, 0, 0, 1.904167, 0.334635, 0, 1.904167, 0.334635, 0, 1.904167, 0, 0, 1.904167, 0, 0, 1.904167, 0, 0, 1.807813, 0, 0, 1.904167, 0, 0, 1.904167, 0, 0, 1.807813, 0, 0, 1.807813, 0, 0, 1.807813, 0, 0, 1.807813, 0.354167, 0, 1.807813, 0, 0, 1.807813, 0, 0, 1.807813, 0.354167, 0, 1.807813, 0.354167, 0, 1.807813, 0.354167, 0, 1.805957, 0.392993, 0, 1.807813, 0.354167, 0, 1.807813, 0.354167, 0, 1.805957, 0.392993, 0, 1.805957, 0.392993, 0, 1.805957, 0.392993, 0, 1.800391, 0.426400, 0, 1.805957, 0.392993, 0, 1.805957, 0.392993, 0, 1.800391, 0.426400, 0, 1.800391, 0.426400, 0, 1.800391, 0.426400, 0, 1.791113, 0.454386, 0, 1.800391, 0.426400, 0, 1.800391, 0.426400, 0, 1.791113, 0.454386, 0, 1.791113, 0.454386, 0, 1.791113, 0.454386, 0, 1.778125, 0.476953, 0, 1.791113, 0.454386, 0, 1.791113, 0.454386, 0, 1.778125, 0.476953, 0, 1.778125, 0.476953, 0, 1.778125, 0.476953, 0, 1.761263, 0.494328, 0, 1.778125, 0.476953, 0, 1.778125, 0.476953, 0, 1.761263, 0.494328, 0, 1.761263, 0.494328, 0, 1.761263, 0.494328, 0, 1.740365, 0.506738, 0, 1.761263, 0.494328, 0, 1.761263, 0.494328, 0, 1.740365, 0.506738, 0, 1.740365, 0.506738, 0, 1.740365, 0.506738, 0, 1.715430, 0.514185, 0, 1.740365, 0.506738, 0, 1.740365, 0.506738, 0, 1.715430, 0.514185, 0, 1.715430, 0.514185, 0, 1.715430, 0.514185, 0, 1.686458, 0.516667, 0, 1.715430, 0.514185, 0, 1.715430, 0.514185, 0, 1.686458, 0.516667, 0, 1.686458, 0.516667, 0, 1.686458, 0.516667, 0, 1.651628, 0.513599, 0, 1.686458, 0.516667, 0, 1.686458, 0.516667, 0, 1.651628, 0.513599, 0, 1.651628, 0.513599, 0, 1.651628, 0.513599, 0, 1.620573, 0.504395, 0, 1.651628, 0.513599, 0, 1.651628, 0.513599, 0, 1.620573, 0.504395, 0, 1.620573, 0.504395, 0, 1.620573, 0.504395, 0, 1.593294, 0.489054, 0, 1.620573, 0.504395, 0, 1.620573, 0.504395, 0, 1.593294, 0.489054, 0, 1.593294, 0.489054, 0, 1.593294, 0.489054, 0, 1.569792, 0.467578, 0, 1.593294, 0.489054, 0, 1.593294, 0.489054, 0, 1.569792, 0.467578, 0, 1.569792, 0.467578, 0, 1.569792, 0.467578, 0, 1.550879, 0.440861, 0, 1.569792, 0.467578, 0, 1.569792, 0.467578, 0, 1.550879, 0.440861, 0, 1.550879, 0.440861, 0, 1.550879, 0.440861, 0, 1.537370, 0.409798, 0, 1.550879, 0.440861, 0, 1.550879, 0.440861, 0, 1.537370, 0.409798, 0, 1.537370, 0.409798, 0, 1.537370, 0.409798, 0, 1.529264, 0.374390, 0, 1.537370, 0.409798, 0, 1.537370, 0.409798, 0, 1.529264, 0.374390, 0, 1.529264, 0.374390, 0, 1.529264, 0.374390, 0, 1.526563, 0.334635, 0, 1.529264, 0.374390, 0, 1.529264, 0.374390, 0, 1.526563, 0.334635, 0, 1.526563, 0.334635, 0, 1.526563, 0.334635, 0, 1.526563, 0, 0, 1.526563, 0.334635, 0, 1.526563, 0.334635, 0, 1.526563, 0, 0, 1.526563, 0, 0, 1.526563, 0, 0, 1.430208, 0, 0, 1.526563, 0, 0, 1.526563, 0, 0, 1.430208, 0, 0, 1.430208, 0, 0, 1.430208, 0, 0, 1.430208, 0.583333, 0, 1.430208, 0, 0, 1.430208, 0, 0, 1.430208, 0.583333, 0, 1.430208, 0.583333, 0, 1.430208, 0.583333, 0, 1.526563, 0.583333, 0, 1.430208, 0.583333, 0, 1.430208, 0.583333, 0, 1.526563, 0.583333, 0, 1.526563, 0.583333, 0, 1.526563, 0.583333, 0, 1.526563, 0.500000, 0, 1.526563, 0.583333, 0, 1.526563, 0.583333, 0, 1.526563, 0.500000, 0, 1.526563, 0.500000, 0, 1.526563, 0.500000, 0, 1.543783, 0.523966, 0, 1.526563, 0.500000, 0, 1.526563, 0.500000, 0, 1.543783, 0.523966, 0, 1.543783, 0.523966, 0, 1.543783, 0.523966, 0, 1.562630, 0.544564, 0, 1.543783, 0.523966, 0, 1.543783, 0.523966, 0, 1.562630, 0.544564, 0, 1.562630, 0.544564, 0, 1.562630, 0.544564, 0, 1.583105, 0.561792, 0, 1.562630, 0.544564, 0, 1.562630, 0.544564, 0, 1.583105, 0.561792, 0, 1.583105, 0.561792, 0, 1.583105, 0.561792, 0, 1.605208, 0.575651, 0, 1.583105, 0.561792, 0, 1.583105, 0.561792, 0, 1.605208, 0.575651, 0, 1.605208, 0.575651, 0, 1.605208, 0.575651, 0, 1.629199, 0.586304, 0, 1.605208, 0.575651, 0, 1.605208, 0.575651, 0, 1.629199, 0.586304, 0, 1.629199, 0.586304, 0, 1.629199, 0.586304, 0, 1.655339, 0.593913, 0, 1.629199, 0.586304, 0, 1.629199, 0.586304, 0, 1.655339, 0.593913, 0, 1.655339, 0.593913, 0, 1.655339, 0.593913, 0, 1.683626, 0.598478, 0, 1.655339, 0.593913, 0, 1.655339, 0.593913, 0, 1.683626, 0.598478, 0, 1.683626, 0.598478, 0, 1.683626, 0.598478, 0, 1.714063, 0.600000, 0, 1.683626, 0.598478, 0, 1.683626, 0.598478, 0, 1.714063, 0.600000, 0, 1.714063, 0.600000, 0, 1.714063, 0.600000, 0, 1.744645, 0.598055, 0, 1.714063, 0.600000, 0, 1.714063, 0.600000, 0, 1.744645, 0.598055, 0, 1.744645, 0.598055, 0, 1.744645, 0.598055, 0, 1.772852, 0.592220, 0, 1.744645, 0.598055, 0, 1.744645, 0.598055, 0, 1.772852, 0.592220, 0, 1.772852, 0.592220, 0, 1.772852, 0.592220, 0, 1.798682, 0.582495, 0, 1.772852, 0.592220, 0, 1.772852, 0.592220, 0, 1.798682, 0.582495, 0, 1.798682, 0.582495, 0, 1.798682, 0.582495, 0, 1.822135, 0.568880, 0, 1.798682, 0.582495, 0, 1.798682, 0.582495, 0, 1.822135, 0.568880, 0, 1.822135, 0.568880, 0, 1.822135, 0.568880, 0, 1.842953, 0.551554, 0, 1.822135, 0.568880, 0, 1.822135, 0.568880, 0, 1.842953, 0.551554, 0, 1.842953, 0.551554, 0, 1.842953, 0.551554, 0, 1.860873, 0.530697, 0, 1.842953, 0.551554, 0, 1.842953, 0.551554, 0, 1.860873, 0.530697, 0, 1.860873, 0.530697, 0, 1.860873, 0.530697, 0, 1.875895, 0.506307, 0, 1.860873, 0.530697, 0, 1.860873, 0.530697, 0, 1.875895, 0.506307, 0, 1.875895, 0.506307, 0, 1.526563, 0.583333, 0, 1.430208, 0.583333, 0, 1.526563, 0.500000, 0, 1.430208, 0.583333, 0, 1.526563, 0.583333, 0, 1.526563, 0.500000, 0, 1.537370, 0.409798, 0, 1.526563, 0.500000, 0, 1.430208, 0.583333, 0, 1.526563, 0.500000, 0, 1.537370, 0.409798, 0, 1.430208, 0.583333, 0, 1.526563, 0.500000, 0, 1.537370, 0.409798, 0, 1.550879, 0.440861, 0, 1.537370, 0.409798, 0, 1.526563, 0.500000, 0, 1.550879, 0.440861, 0, 1.526563, 0.500000, 0, 1.550879, 0.440861, 0, 1.569792, 0.467578, 0, 1.550879, 0.440861, 0, 1.526563, 0.500000, 0, 1.569792, 0.467578, 0, 1.543783, 0.523966, 0, 1.526563, 0.500000, 0, 1.569792, 0.467578, 0, 1.526563, 0.500000, 0, 1.543783, 0.523966, 0, 1.569792, 0.467578, 0, 1.543783, 0.523966, 0, 1.569792, 0.467578, 0, 1.593294, 0.489054, 0, 1.569792, 0.467578, 0, 1.543783, 0.523966, 0, 1.593294, 0.489054, 0, 1.562630, 0.544564, 0, 1.543783, 0.523966, 0, 1.593294, 0.489054, 0, 1.543783, 0.523966, 0, 1.562630, 0.544564, 0, 1.593294, 0.489054, 0, 1.562630, 0.544564, 0, 1.593294, 0.489054, 0, 1.620573, 0.504395, 0, 1.593294, 0.489054, 0, 1.562630, 0.544564, 0, 1.620573, 0.504395, 0, 1.583105, 0.561792, 0, 1.562630, 0.544564, 0, 1.620573, 0.504395, 0, 1.562630, 0.544564, 0, 1.583105, 0.561792, 0, 1.620573, 0.504395, 0, 1.605208, 0.575651, 0, 1.583105, 0.561792, 0, 1.620573, 0.504395, 0, 1.583105, 0.561792, 0, 1.605208, 0.575651, 0, 1.620573, 0.504395, 0, 1.605208, 0.575651, 0, 1.620573, 0.504395, 0, 1.651628, 0.513599, 0, 1.620573, 0.504395, 0, 1.605208, 0.575651, 0, 1.651628, 0.513599, 0, 1.629199, 0.586304, 0, 1.605208, 0.575651, 0, 1.651628, 0.513599, 0, 1.605208, 0.575651, 0, 1.629199, 0.586304, 0, 1.651628, 0.513599, 0, 1.629199, 0.586304, 0, 1.651628, 0.513599, 0, 1.655339, 0.593913, 0, 1.651628, 0.513599, 0, 1.629199, 0.586304, 0, 1.655339, 0.593913, 0, 1.655339, 0.593913, 0, 1.651628, 0.513599, 0, 1.686458, 0.516667, 0, 1.651628, 0.513599, 0, 1.655339, 0.593913, 0, 1.686458, 0.516667, 0, 1.683626, 0.598478, 0, 1.655339, 0.593913, 0, 1.686458, 0.516667, 0, 1.655339, 0.593913, 0, 1.683626, 0.598478, 0, 1.686458, 0.516667, 0, 1.714063, 0.600000, 0, 1.683626, 0.598478, 0, 1.686458, 0.516667, 0, 1.683626, 0.598478, 0, 1.714063, 0.600000, 0, 1.686458, 0.516667, 0, 1.714063, 0.600000, 0, 1.686458, 0.516667, 0, 1.715430, 0.514185, 0, 1.686458, 0.516667, 0, 1.714063, 0.600000, 0, 1.715430, 0.514185, 0, 1.744645, 0.598055, 0, 1.714063, 0.600000, 0, 1.715430, 0.514185, 0, 1.714063, 0.600000, 0, 1.744645, 0.598055, 0, 1.715430, 0.514185, 0, 1.744645, 0.598055, 0, 1.715430, 0.514185, 0, 1.740365, 0.506738, 0, 1.715430, 0.514185, 0, 1.744645, 0.598055, 0, 1.740365, 0.506738, 0, 1.772852, 0.592220, 0, 1.744645, 0.598055, 0, 1.740365, 0.506738, 0, 1.744645, 0.598055, 0, 1.772852, 0.592220, 0, 1.740365, 0.506738, 0, 1.772852, 0.592220, 0, 1.740365, 0.506738, 0, 1.798682, 0.582495, 0, 1.740365, 0.506738, 0, 1.772852, 0.592220, 0, 1.798682, 0.582495, 0, 1.740365, 0.506738, 0, 1.761263, 0.494328, 0, 1.798682, 0.582495, 0, 1.761263, 0.494328, 0, 1.740365, 0.506738, 0, 1.798682, 0.582495, 0, 1.798682, 0.582495, 0, 1.761263, 0.494328, 0, 1.822135, 0.568880, 0, 1.761263, 0.494328, 0, 1.798682, 0.582495, 0, 1.822135, 0.568880, 0, 1.822135, 0.568880, 0, 1.761263, 0.494328, 0, 1.842953, 0.551554, 0, 1.761263, 0.494328, 0, 1.822135, 0.568880, 0, 1.842953, 0.551554, 0, 1.761263, 0.494328, 0, 1.778125, 0.476953, 0, 1.842953, 0.551554, 0, 1.778125, 0.476953, 0, 1.761263, 0.494328, 0, 1.842953, 0.551554, 0, 1.842953, 0.551554, 0, 1.778125, 0.476953, 0, 1.860873, 0.530697, 0, 1.778125, 0.476953, 0, 1.842953, 0.551554, 0, 1.860873, 0.530697, 0, 1.860873, 0.530697, 0, 1.778125, 0.476953, 0, 1.875895, 0.506307, 0, 1.778125, 0.476953, 0, 1.860873, 0.530697, 0, 1.875895, 0.506307, 0, 1.778125, 0.476953, 0, 1.791113, 0.454386, 0, 1.875895, 0.506307, 0, 1.791113, 0.454386, 0, 1.778125, 0.476953, 0, 1.875895, 0.506307, 0, 1.875895, 0.506307, 0, 1.791113, 0.454386, 0, 1.888021, 0.478385, 0, 1.791113, 0.454386, 0, 1.875895, 0.506307, 0, 1.888021, 0.478385, 0, 1.791113, 0.454386, 0, 1.800391, 0.426400, 0, 1.888021, 0.478385, 0, 1.800391, 0.426400, 0, 1.791113, 0.454386, 0, 1.888021, 0.478385, 0, 1.888021, 0.478385, 0, 1.800391, 0.426400, 0, 1.914974, 0.410124, 0, 1.800391, 0.426400, 0, 1.888021, 0.478385, 0, 1.914974, 0.410124, 0, 1.888021, 0.478385, 0, 1.914974, 0.410124, 0, 1.928483, 0.441203, 0, 1.914974, 0.410124, 0, 1.888021, 0.478385, 0, 1.928483, 0.441203, 0, 1.888021, 0.478385, 0, 1.928483, 0.441203, 0, 1.947396, 0.467839, 0, 1.928483, 0.441203, 0, 1.888021, 0.478385, 0, 1.947396, 0.467839, 0, 1.906869, 0.507471, 0, 1.888021, 0.478385, 0, 1.947396, 0.467839, 0, 1.888021, 0.478385, 0, 1.906869, 0.507471, 0, 1.947396, 0.467839, 0, 1.970963, 0.489201, 0, 1.906869, 0.507471, 0, 1.947396, 0.467839, 0, 1.906869, 0.507471, 0, 1.970963, 0.489201, 0, 1.947396, 0.467839, 0, 1.970963, 0.489201, 0, 1.927474, 0.532487, 0, 1.906869, 0.507471, 0, 1.927474, 0.532487, 0, 1.970963, 0.489201, 0, 1.906869, 0.507471, 0, 1.949837, 0.553434, 0, 1.927474, 0.532487, 0, 1.970963, 0.489201, 0, 1.927474, 0.532487, 0, 1.949837, 0.553434, 0, 1.970963, 0.489201, 0, 1.949837, 0.553434, 0, 1.970963, 0.489201, 0, 1.998438, 0.504460, 0, 1.970963, 0.489201, 0, 1.949837, 0.553434, 0, 1.998438, 0.504460, 0, 1.973958, 0.570313, 0, 1.949837, 0.553434, 0, 1.998438, 0.504460, 0, 1.949837, 0.553434, 0, 1.973958, 0.570313, 0, 1.998438, 0.504460, 0, 1.973958, 0.570313, 0, 1.998438, 0.504460, 0, 2.000065, 0.583301, 0, 1.998438, 0.504460, 0, 1.973958, 0.570313, 0, 2.000065, 0.583301, 0, 2.000065, 0.583301, 0, 1.998438, 0.504460, 0, 2.029818, 0.513615, 0, 1.998438, 0.504460, 0, 2.000065, 0.583301, 0, 2.029818, 0.513615, 0, 2.028385, 0.592578, 0, 2.000065, 0.583301, 0, 2.029818, 0.513615, 0, 2.000065, 0.583301, 0, 2.028385, 0.592578, 0, 2.029818, 0.513615, 0, 2.028385, 0.592578, 0, 2.029818, 0.513615, 0, 2.065104, 0.516667, 0, 2.029818, 0.513615, 0, 2.028385, 0.592578, 0, 2.065104, 0.516667, 0, 2.058919, 0.598145, 0, 2.028385, 0.592578, 0, 2.065104, 0.516667, 0, 2.028385, 0.592578, 0, 2.058919, 0.598145, 0, 2.065104, 0.516667, 0, 2.091667, 0.600000, 0, 2.058919, 0.598145, 0, 2.065104, 0.516667, 0, 2.058919, 0.598145, 0, 2.091667, 0.600000, 0, 2.065104, 0.516667, 0, 2.091667, 0.600000, 0, 2.065104, 0.516667, 0, 2.093620, 0.514168, 0, 2.065104, 0.516667, 0, 2.091667, 0.600000, 0, 2.093620, 0.514168, 0, 2.134635, 0.596053, 0, 2.091667, 0.600000, 0, 2.093620, 0.514168, 0, 2.091667, 0.600000, 0, 2.134635, 0.596053, 0, 2.093620, 0.514168, 0, 2.093620, 0.514168, 0, 2.118229, 0.506673, 0, 2.134635, 0.596053, 0, 2.118229, 0.506673, 0, 2.093620, 0.514168, 0, 2.134635, 0.596053, 0, 2.134635, 0.596053, 0, 2.118229, 0.506673, 0, 2.172396, 0.584212, 0, 2.118229, 0.506673, 0, 2.134635, 0.596053, 0, 2.172396, 0.584212, 0, 2.118229, 0.506673, 0, 2.138932, 0.494181, 0, 2.172396, 0.584212, 0, 2.138932, 0.494181, 0, 2.118229, 0.506673, 0, 2.172396, 0.584212, 0, 2.172396, 0.584212, 0, 2.138932, 0.494181, 0, 2.204948, 0.564478, 0, 2.138932, 0.494181, 0, 2.172396, 0.584212, 0, 2.204948, 0.564478, 0, 2.138932, 0.494181, 0, 2.155729, 0.476693, 0, 2.204948, 0.564478, 0, 2.155729, 0.476693, 0, 2.138932, 0.494181, 0, 2.204948, 0.564478, 0, 2.204948, 0.564478, 0, 2.155729, 0.476693, 0, 2.232292, 0.536849, 0, 2.155729, 0.476693, 0, 2.204948, 0.564478, 0, 2.232292, 0.536849, 0, 2.232292, 0.536849, 0, 2.155729, 0.476693, 0, 2.253939, 0.501945, 0, 2.155729, 0.476693, 0, 2.232292, 0.536849, 0, 2.253939, 0.501945, 0, 2.253939, 0.501945, 0, 2.155729, 0.476693, 0, 2.168717, 0.454045, 0, 2.155729, 0.476693, 0, 2.253939, 0.501945, 0, 2.168717, 0.454045, 0, 2.253939, 0.501945, 0, 2.168717, 0.454045, 0, 2.269401, 0.460384, 0, 2.168717, 0.454045, 0, 2.253939, 0.501945, 0, 2.269401, 0.460384, 0, 2.168717, 0.454045, 0, 2.177995, 0.426074, 0, 2.269401, 0.460384, 0, 2.177995, 0.426074, 0, 2.168717, 0.454045, 0, 2.269401, 0.460384, 0, 2.269401, 0.460384, 0, 2.177995, 0.426074, 0, 2.278678, 0.412166, 0, 2.177995, 0.426074, 0, 2.269401, 0.460384, 0, 2.278678, 0.412166, 0, 2.177995, 0.426074, 0, 2.183561, 0.392782, 0, 2.278678, 0.412166, 0, 2.183561, 0.392782, 0, 2.177995, 0.426074, 0, 2.278678, 0.412166, 0, 2.278678, 0.412166, 0, 2.183561, 0.392782, 0, 2.281771, 0.357292, 0, 2.183561, 0.392782, 0, 2.278678, 0.412166, 0, 2.281771, 0.357292, 0, 2.281771, 0.357292, 0, 2.183561, 0.392782, 0, 2.185417, 0.354167, 0, 2.183561, 0.392782, 0, 2.281771, 0.357292, 0, 2.185417, 0.354167, 0, 2.185417, 0.354167, 0, 2.281771, 0, 0, 2.281771, 0.357292, 0, 2.281771, 0, 0, 2.185417, 0.354167, 0, 2.281771, 0.357292, 0, 2.185417, 0.354167, 0, 2.185417, 0, 0, 2.281771, 0, 0, 2.185417, 0, 0, 2.185417, 0.354167, 0, 2.281771, 0, 0, 1.805957, 0.392993, 0, 1.914974, 0.410124, 0, 1.800391, 0.426400, 0, 1.914974, 0.410124, 0, 1.805957, 0.392993, 0, 1.800391, 0.426400, 0, 1.914974, 0.410124, 0, 1.805957, 0.392993, 0, 1.906869, 0.374601, 0, 1.805957, 0.392993, 0, 1.914974, 0.410124, 0, 1.906869, 0.374601, 0, 1.805957, 0.392993, 0, 1.807813, 0.354167, 0, 1.906869, 0.374601, 0, 1.807813, 0.354167, 0, 1.805957, 0.392993, 0, 1.906869, 0.374601, 0, 1.807813, 0.354167, 0, 1.904167, 0.334635, 0, 1.906869, 0.374601, 0, 1.904167, 0.334635, 0, 1.807813, 0.354167, 0, 1.906869, 0.374601, 0, 1.904167, 0.334635, 0, 1.807813, 0.354167, 0, 1.807813, 0, 0, 1.807813, 0.354167, 0, 1.904167, 0.334635, 0, 1.807813, 0, 0, 1.807813, 0, 0, 1.904167, 0, 0, 1.904167, 0.334635, 0, 1.904167, 0, 0, 1.807813, 0, 0, 1.904167, 0.334635, 0, 1.537370, 0.409798, 0, 1.430208, 0.583333, 0, 1.529264, 0.374390, 0, 1.430208, 0.583333, 0, 1.537370, 0.409798, 0, 1.529264, 0.374390, 0, 1.526563, 0.334635, 0, 1.529264, 0.374390, 0, 1.430208, 0.583333, 0, 1.529264, 0.374390, 0, 1.526563, 0.334635, 0, 1.430208, 0.583333, 0, 1.526563, 0.334635, 0, 1.430208, 0.583333, 0, 1.430208, 0, 0, 1.430208, 0.583333, 0, 1.526563, 0.334635, 0, 1.430208, 0, 0, 1.430208, 0, 0, 1.526563, 0, 0, 1.526563, 0.334635, 0, 1.526563, 0, 0, 1.430208, 0, 0, 1.526563, 0.334635, 0, 2.966146, 0.318750, 0, 2.966146, 0.266667, 0, 2.966146, 0.318750, 0, 2.966146, 0.318750, 0, 2.966146, 0.266667, 0, 2.966146, 0.266667, 0, 2.966146, 0.266667, 0, 2.525521, 0.266667, 0, 2.966146, 0.266667, 0, 2.966146, 0.266667, 0, 2.525521, 0.266667, 0, 2.525521, 0.266667, 0, 2.525521, 0.266667, 0, 2.531592, 0.220744, 0, 2.525521, 0.266667, 0, 2.525521, 0.266667, 0, 2.531592, 0.220744, 0, 2.531592, 0.220744, 0, 2.531592, 0.220744, 0, 2.543555, 0.180632, 0, 2.531592, 0.220744, 0, 2.531592, 0.220744, 0, 2.543555, 0.180632, 0, 2.543555, 0.180632, 0, 2.543555, 0.180632, 0, 2.561410, 0.146330, 0, 2.543555, 0.180632, 0, 2.543555, 0.180632, 0, 2.561410, 0.146330, 0, 2.561410, 0.146330, 0, 2.561410, 0.146330, 0, 2.585156, 0.117839, 0, 2.561410, 0.146330, 0, 2.561410, 0.146330, 0, 2.585156, 0.117839, 0, 2.585156, 0.117839, 0, 2.585156, 0.117839, 0, 2.614469, 0.095451, 0, 2.585156, 0.117839, 0, 2.585156, 0.117839, 0, 2.614469, 0.095451, 0, 2.614469, 0.095451, 0, 2.614469, 0.095451, 0, 2.649024, 0.079460, 0, 2.614469, 0.095451, 0, 2.614469, 0.095451, 0, 2.649024, 0.079460, 0, 2.649024, 0.079460, 0, 2.649024, 0.079460, 0, 2.688818, 0.069865, 0, 2.649024, 0.079460, 0, 2.649024, 0.079460, 0, 2.688818, 0.069865, 0, 2.688818, 0.069865, 0, 2.688818, 0.069865, 0, 2.733854, 0.066667, 0, 2.688818, 0.069865, 0, 2.688818, 0.069865, 0, 2.733854, 0.066667, 0, 2.733854, 0.066667, 0, 2.733854, 0.066667, 0, 2.761247, 0.067448, 0, 2.733854, 0.066667, 0, 2.733854, 0.066667, 0, 2.761247, 0.067448, 0, 2.761247, 0.067448, 0, 2.761247, 0.067448, 0, 2.788216, 0.069792, 0, 2.761247, 0.067448, 0, 2.761247, 0.067448, 0, 2.788216, 0.069792, 0, 2.788216, 0.069792, 0, 2.788216, 0.069792, 0, 2.814762, 0.073698, 0, 2.788216, 0.069792, 0, 2.788216, 0.069792, 0, 2.814762, 0.073698, 0, 2.814762, 0.073698, 0, 2.814762, 0.073698, 0, 2.840885, 0.079167, 0, 2.814762, 0.073698, 0, 2.814762, 0.073698, 0, 2.840885, 0.079167, 0, 2.840885, 0.079167, 0, 2.840885, 0.079167, 0, 2.866748, 0.086198, 0, 2.840885, 0.079167, 0, 2.840885, 0.079167, 0, 2.866748, 0.086198, 0, 2.866748, 0.086198, 0, 2.866748, 0.086198, 0, 2.892513, 0.094792, 0, 2.866748, 0.086198, 0, 2.866748, 0.086198, 0, 2.892513, 0.094792, 0, 2.892513, 0.094792, 0, 2.892513, 0.094792, 0, 2.918180, 0.104948, 0, 2.892513, 0.094792, 0, 2.892513, 0.094792, 0, 2.918180, 0.104948, 0, 2.918180, 0.104948, 0, 2.918180, 0.104948, 0, 2.943750, 0.116667, 0, 2.918180, 0.104948, 0, 2.918180, 0.104948, 0, 2.943750, 0.116667, 0, 2.943750, 0.116667, 0, 2.943750, 0.116667, 0, 2.943750, 0.033333, 0, 2.943750, 0.116667, 0, 2.943750, 0.116667, 0, 2.943750, 0.033333, 0, 2.943750, 0.033333, 0, 2.943750, 0.033333, 0, 2.917806, 0.021826, 0, 2.943750, 0.033333, 0, 2.943750, 0.033333, 0, 2.917806, 0.021826, 0, 2.917806, 0.021826, 0, 2.917806, 0.021826, 0, 2.891536, 0.011784, 0, 2.917806, 0.021826, 0, 2.917806, 0.021826, 0, 2.891536, 0.011784, 0, 2.891536, 0.011784, 0, 2.891536, 0.011784, 0, 2.864941, 0.003206, 0, 2.891536, 0.011784, 0, 2.891536, 0.011784, 0, 2.864941, 0.003206, 0, 2.864941, 0.003206, 0, 2.864941, 0.003206, 0, 2.838021, -0.003906, 0, 2.864941, 0.003206, 0, 2.864941, 0.003206, 0, 2.838021, -0.003906, 0, 2.838021, -0.003906, 0, 2.838021, -0.003906, 0, 2.810840, -0.009489, 0, 2.838021, -0.003906, 0, 2.838021, -0.003906, 0, 2.810840, -0.009489, 0, 2.810840, -0.009489, 0, 2.810840, -0.009489, 0, 2.783463, -0.013477, 0, 2.810840, -0.009489, 0, 2.810840, -0.009489, 0, 2.783463, -0.013477, 0, 2.783463, -0.013477, 0, 2.783463, -0.013477, 0, 2.755892, -0.015869, 0, 2.783463, -0.013477, 0, 2.783463, -0.013477, 0, 2.755892, -0.015869, 0, 2.755892, -0.015869, 0, 2.755892, -0.015869, 0, 2.728125, -0.016667, 0, 2.755892, -0.015869, 0, 2.755892, -0.015869, 0, 2.728125, -0.016667, 0, 2.728125, -0.016667, 0, 2.728125, -0.016667, 0, 2.661963, -0.011556, 0, 2.728125, -0.016667, 0, 2.728125, -0.016667, 0, 2.661963, -0.011556, 0, 2.661963, -0.011556, 0, 2.661963, -0.011556, 0, 2.603060, 0.003776, 0, 2.661963, -0.011556, 0, 2.661963, -0.011556, 0, 2.603060, 0.003776, 0, 2.603060, 0.003776, 0, 2.603060, 0.003776, 0, 2.551416, 0.029329, 0, 2.603060, 0.003776, 0, 2.603060, 0.003776, 0, 2.551416, 0.029329, 0, 2.551416, 0.029329, 0, 2.551416, 0.029329, 0, 2.507031, 0.065104, 0, 2.551416, 0.029329, 0, 2.551416, 0.029329, 0, 2.507031, 0.065104, 0, 2.507031, 0.065104, 0, 2.507031, 0.065104, 0, 2.471370, 0.109603, 0, 2.507031, 0.065104, 0, 2.507031, 0.065104, 0, 2.471370, 0.109603, 0, 2.471370, 0.109603, 0, 2.471370, 0.109603, 0, 2.445899, 0.161328, 0, 2.471370, 0.109603, 0, 2.471370, 0.109603, 0, 2.445899, 0.161328, 0, 2.445899, 0.161328, 0, 2.445899, 0.161328, 0, 2.430615, 0.220280, 0, 2.445899, 0.161328, 0, 2.445899, 0.161328, 0, 2.430615, 0.220280, 0, 2.430615, 0.220280, 0, 2.430615, 0.220280, 0, 2.425521, 0.286458, 0, 2.430615, 0.220280, 0, 2.430615, 0.220280, 0, 2.425521, 0.286458, 0, 2.425521, 0.286458, 0, 2.425521, 0.286458, 0, 2.430355, 0.354867, 0, 2.425521, 0.286458, 0, 2.425521, 0.286458, 0, 2.430355, 0.354867, 0, 2.430355, 0.354867, 0, 2.430355, 0.354867, 0, 2.444857, 0.415820, 0, 2.430355, 0.354867, 0, 2.430355, 0.354867, 0, 2.444857, 0.415820, 0, 2.444857, 0.415820, 0, 2.444857, 0.415820, 0, 2.469027, 0.469320, 0, 2.444857, 0.415820, 0, 2.444857, 0.415820, 0, 2.469027, 0.469320, 0, 2.469027, 0.469320, 0, 2.469027, 0.469320, 0, 2.502865, 0.515365, 0, 2.469027, 0.469320, 0, 2.469027, 0.469320, 0, 2.502865, 0.515365, 0, 2.502865, 0.515365, 0, 2.502865, 0.515365, 0, 2.544906, 0.552393, 0, 2.502865, 0.515365, 0, 2.502865, 0.515365, 0, 2.544906, 0.552393, 0, 2.544906, 0.552393, 0, 2.544906, 0.552393, 0, 2.593685, 0.578841, 0, 2.544906, 0.552393, 0, 2.544906, 0.552393, 0, 2.593685, 0.578841, 0, 2.593685, 0.578841, 0, 2.593685, 0.578841, 0, 2.649203, 0.594710, 0, 2.593685, 0.578841, 0, 2.593685, 0.578841, 0, 2.649203, 0.594710, 0, 2.649203, 0.594710, 0, 2.649203, 0.594710, 0, 2.711458, 0.600000, 0, 2.649203, 0.594710, 0, 2.649203, 0.594710, 0, 2.711458, 0.600000, 0, 2.711458, 0.600000, 0, 2.711458, 0.600000, 0, 2.767236, 0.595272, 0, 2.711458, 0.600000, 0, 2.711458, 0.600000, 0, 2.767236, 0.595272, 0, 2.767236, 0.595272, 0, 2.767236, 0.595272, 0, 2.816862, 0.581087, 0, 2.767236, 0.595272, 0, 2.767236, 0.595272, 0, 2.816862, 0.581087, 0, 2.816862, 0.581087, 0, 2.816862, 0.581087, 0, 2.860335, 0.557446, 0, 2.816862, 0.581087, 0, 2.816862, 0.581087, 0, 2.860335, 0.557446, 0, 2.860335, 0.557446, 0, 2.860335, 0.557446, 0, 2.897656, 0.524349, 0, 2.860335, 0.557446, 0, 2.860335, 0.557446, 0, 2.897656, 0.524349, 0, 2.897656, 0.524349, 0, 2.897656, 0.524349, 0, 2.927621, 0.483130, 0, 2.897656, 0.524349, 0, 2.897656, 0.524349, 0, 2.927621, 0.483130, 0, 2.927621, 0.483130, 0, 2.927621, 0.483130, 0, 2.949023, 0.435124, 0, 2.927621, 0.483130, 0, 2.927621, 0.483130, 0, 2.949023, 0.435124, 0, 2.949023, 0.435124, 0, 2.949023, 0.435124, 0, 2.961865, 0.380330, 0, 2.949023, 0.435124, 0, 2.949023, 0.435124, 0, 2.961865, 0.380330, 0, 2.961865, 0.380330, 0, 2.430355, 0.354867, 0, 2.425521, 0.286458, 0, 2.528646, 0.349479, 0, 2.425521, 0.286458, 0, 2.430355, 0.354867, 0, 2.528646, 0.349479, 0, 2.444857, 0.415820, 0, 2.430355, 0.354867, 0, 2.528646, 0.349479, 0, 2.430355, 0.354867, 0, 2.444857, 0.415820, 0, 2.528646, 0.349479, 0, 2.444857, 0.415820, 0, 2.528646, 0.349479, 0, 2.534847, 0.387077, 0, 2.528646, 0.349479, 0, 2.444857, 0.415820, 0, 2.534847, 0.387077, 0, 2.546159, 0.420182, 0, 2.444857, 0.415820, 0, 2.534847, 0.387077, 0, 2.444857, 0.415820, 0, 2.546159, 0.420182, 0, 2.534847, 0.387077, 0, 2.546159, 0.420182, 0, 2.469027, 0.469320, 0, 2.444857, 0.415820, 0, 2.469027, 0.469320, 0, 2.546159, 0.420182, 0, 2.444857, 0.415820, 0, 2.469027, 0.469320, 0, 2.546159, 0.420182, 0, 2.562582, 0.448796, 0, 2.546159, 0.420182, 0, 2.469027, 0.469320, 0, 2.562582, 0.448796, 0, 2.502865, 0.515365, 0, 2.469027, 0.469320, 0, 2.562582, 0.448796, 0, 2.469027, 0.469320, 0, 2.502865, 0.515365, 0, 2.562582, 0.448796, 0, 2.502865, 0.515365, 0, 2.562582, 0.448796, 0, 2.584115, 0.472917, 0, 2.562582, 0.448796, 0, 2.502865, 0.515365, 0, 2.584115, 0.472917, 0, 2.544906, 0.552393, 0, 2.502865, 0.515365, 0, 2.584115, 0.472917, 0, 2.502865, 0.515365, 0, 2.544906, 0.552393, 0, 2.584115, 0.472917, 0, 2.544906, 0.552393, 0, 2.584115, 0.472917, 0, 2.610205, 0.492057, 0, 2.584115, 0.472917, 0, 2.544906, 0.552393, 0, 2.610205, 0.492057, 0, 2.593685, 0.578841, 0, 2.544906, 0.552393, 0, 2.610205, 0.492057, 0, 2.544906, 0.552393, 0, 2.593685, 0.578841, 0, 2.610205, 0.492057, 0, 2.593685, 0.578841, 0, 2.610205, 0.492057, 0, 2.640300, 0.505729, 0, 2.610205, 0.492057, 0, 2.593685, 0.578841, 0, 2.640300, 0.505729, 0, 2.593685, 0.578841, 0, 2.640300, 0.505729, 0, 2.649203, 0.594710, 0, 2.640300, 0.505729, 0, 2.593685, 0.578841, 0, 2.649203, 0.594710, 0, 2.649203, 0.594710, 0, 2.640300, 0.505729, 0, 2.674398, 0.513932, 0, 2.640300, 0.505729, 0, 2.649203, 0.594710, 0, 2.674398, 0.513932, 0, 2.649203, 0.594710, 0, 2.674398, 0.513932, 0, 2.711458, 0.600000, 0, 2.674398, 0.513932, 0, 2.649203, 0.594710, 0, 2.711458, 0.600000, 0, 2.711458, 0.600000, 0, 2.674398, 0.513932, 0, 2.712500, 0.516667, 0, 2.674398, 0.513932, 0, 2.711458, 0.600000, 0, 2.712500, 0.516667, 0, 2.711458, 0.600000, 0, 2.712500, 0.516667, 0, 2.746175, 0.513835, 0, 2.712500, 0.516667, 0, 2.711458, 0.600000, 0, 2.746175, 0.513835, 0, 2.711458, 0.600000, 0, 2.746175, 0.513835, 0, 2.767236, 0.595272, 0, 2.746175, 0.513835, 0, 2.711458, 0.600000, 0, 2.767236, 0.595272, 0, 2.767236, 0.595272, 0, 2.746175, 0.513835, 0, 2.776367, 0.505339, 0, 2.746175, 0.513835, 0, 2.767236, 0.595272, 0, 2.776367, 0.505339, 0, 2.767236, 0.595272, 0, 2.776367, 0.505339, 0, 2.816862, 0.581087, 0, 2.776367, 0.505339, 0, 2.767236, 0.595272, 0, 2.816862, 0.581087, 0, 2.776367, 0.505339, 0, 2.803076, 0.491178, 0, 2.816862, 0.581087, 0, 2.803076, 0.491178, 0, 2.776367, 0.505339, 0, 2.816862, 0.581087, 0, 2.816862, 0.581087, 0, 2.803076, 0.491178, 0, 2.860335, 0.557446, 0, 2.803076, 0.491178, 0, 2.816862, 0.581087, 0, 2.860335, 0.557446, 0, 2.803076, 0.491178, 0, 2.826302, 0.471354, 0, 2.860335, 0.557446, 0, 2.826302, 0.471354, 0, 2.803076, 0.491178, 0, 2.860335, 0.557446, 0, 2.860335, 0.557446, 0, 2.826302, 0.471354, 0, 2.897656, 0.524349, 0, 2.826302, 0.471354, 0, 2.860335, 0.557446, 0, 2.897656, 0.524349, 0, 2.826302, 0.471354, 0, 2.845166, 0.446777, 0, 2.897656, 0.524349, 0, 2.845166, 0.446777, 0, 2.826302, 0.471354, 0, 2.897656, 0.524349, 0, 2.897656, 0.524349, 0, 2.845166, 0.446777, 0, 2.927621, 0.483130, 0, 2.845166, 0.446777, 0, 2.897656, 0.524349, 0, 2.927621, 0.483130, 0, 2.845166, 0.446777, 0, 2.858789, 0.418359, 0, 2.927621, 0.483130, 0, 2.858789, 0.418359, 0, 2.845166, 0.446777, 0, 2.927621, 0.483130, 0, 2.927621, 0.483130, 0, 2.858789, 0.418359, 0, 2.949023, 0.435124, 0, 2.858789, 0.418359, 0, 2.927621, 0.483130, 0, 2.949023, 0.435124, 0, 2.858789, 0.418359, 0, 2.867171, 0.386100, 0, 2.949023, 0.435124, 0, 2.867171, 0.386100, 0, 2.858789, 0.418359, 0, 2.949023, 0.435124, 0, 2.949023, 0.435124, 0, 2.867171, 0.386100, 0, 2.961865, 0.380330, 0, 2.867171, 0.386100, 0, 2.949023, 0.435124, 0, 2.961865, 0.380330, 0, 2.867171, 0.386100, 0, 2.870312, 0.350000, 0, 2.961865, 0.380330, 0, 2.870312, 0.350000, 0, 2.867171, 0.386100, 0, 2.961865, 0.380330, 0, 2.961865, 0.380330, 0, 2.870312, 0.350000, 0, 2.966146, 0.318750, 0, 2.870312, 0.350000, 0, 2.961865, 0.380330, 0, 2.966146, 0.318750, 0, 2.966146, 0.266667, 0, 2.966146, 0.318750, 0, 2.870312, 0.350000, 0, 2.966146, 0.318750, 0, 2.966146, 0.266667, 0, 2.870312, 0.350000, 0, 2.870312, 0.350000, 0, 2.525521, 0.266667, 0, 2.966146, 0.266667, 0, 2.525521, 0.266667, 0, 2.870312, 0.350000, 0, 2.966146, 0.266667, 0, 2.870312, 0.350000, 0, 2.528646, 0.349479, 0, 2.525521, 0.266667, 0, 2.528646, 0.349479, 0, 2.870312, 0.350000, 0, 2.525521, 0.266667, 0, 2.425521, 0.286458, 0, 2.525521, 0.266667, 0, 2.528646, 0.349479, 0, 2.525521, 0.266667, 0, 2.425521, 0.286458, 0, 2.528646, 0.349479, 0, 2.425521, 0.286458, 0, 2.430615, 0.220280, 0, 2.525521, 0.266667, 0, 2.430615, 0.220280, 0, 2.425521, 0.286458, 0, 2.525521, 0.266667, 0, 2.525521, 0.266667, 0, 2.430615, 0.220280, 0, 2.531592, 0.220744, 0, 2.430615, 0.220280, 0, 2.525521, 0.266667, 0, 2.531592, 0.220744, 0, 2.430615, 0.220280, 0, 2.445899, 0.161328, 0, 2.531592, 0.220744, 0, 2.445899, 0.161328, 0, 2.430615, 0.220280, 0, 2.531592, 0.220744, 0, 2.531592, 0.220744, 0, 2.445899, 0.161328, 0, 2.543555, 0.180632, 0, 2.445899, 0.161328, 0, 2.531592, 0.220744, 0, 2.543555, 0.180632, 0, 2.445899, 0.161328, 0, 2.471370, 0.109603, 0, 2.543555, 0.180632, 0, 2.471370, 0.109603, 0, 2.445899, 0.161328, 0, 2.543555, 0.180632, 0, 2.543555, 0.180632, 0, 2.471370, 0.109603, 0, 2.561410, 0.146330, 0, 2.471370, 0.109603, 0, 2.543555, 0.180632, 0, 2.561410, 0.146330, 0, 2.471370, 0.109603, 0, 2.507031, 0.065104, 0, 2.561410, 0.146330, 0, 2.507031, 0.065104, 0, 2.471370, 0.109603, 0, 2.561410, 0.146330, 0, 2.561410, 0.146330, 0, 2.507031, 0.065104, 0, 2.585156, 0.117839, 0, 2.507031, 0.065104, 0, 2.561410, 0.146330, 0, 2.585156, 0.117839, 0, 2.507031, 0.065104, 0, 2.551416, 0.029329, 0, 2.585156, 0.117839, 0, 2.551416, 0.029329, 0, 2.507031, 0.065104, 0, 2.585156, 0.117839, 0, 2.585156, 0.117839, 0, 2.551416, 0.029329, 0, 2.614469, 0.095451, 0, 2.551416, 0.029329, 0, 2.585156, 0.117839, 0, 2.614469, 0.095451, 0, 2.614469, 0.095451, 0, 2.551416, 0.029329, 0, 2.603060, 0.003776, 0, 2.551416, 0.029329, 0, 2.614469, 0.095451, 0, 2.603060, 0.003776, 0, 2.649024, 0.079460, 0, 2.614469, 0.095451, 0, 2.603060, 0.003776, 0, 2.614469, 0.095451, 0, 2.649024, 0.079460, 0, 2.603060, 0.003776, 0, 2.649024, 0.079460, 0, 2.603060, 0.003776, 0, 2.661963, -0.011556, 0, 2.603060, 0.003776, 0, 2.649024, 0.079460, 0, 2.661963, -0.011556, 0, 2.688818, 0.069865, 0, 2.649024, 0.079460, 0, 2.661963, -0.011556, 0, 2.649024, 0.079460, 0, 2.688818, 0.069865, 0, 2.661963, -0.011556, 0, 2.688818, 0.069865, 0, 2.661963, -0.011556, 0, 2.728125, -0.016667, 0, 2.661963, -0.011556, 0, 2.688818, 0.069865, 0, 2.728125, -0.016667, 0, 2.733854, 0.066667, 0, 2.688818, 0.069865, 0, 2.728125, -0.016667, 0, 2.688818, 0.069865, 0, 2.733854, 0.066667, 0, 2.728125, -0.016667, 0, 2.733854, 0.066667, 0, 2.728125, -0.016667, 0, 2.755892, -0.015869, 0, 2.728125, -0.016667, 0, 2.733854, 0.066667, 0, 2.755892, -0.015869, 0, 2.733854, 0.066667, 0, 2.755892, -0.015869, 0, 2.761247, 0.067448, 0, 2.755892, -0.015869, 0, 2.733854, 0.066667, 0, 2.761247, 0.067448, 0, 2.761247, 0.067448, 0, 2.755892, -0.015869, 0, 2.783463, -0.013477, 0, 2.755892, -0.015869, 0, 2.761247, 0.067448, 0, 2.783463, -0.013477, 0, 2.761247, 0.067448, 0, 2.783463, -0.013477, 0, 2.788216, 0.069792, 0, 2.783463, -0.013477, 0, 2.761247, 0.067448, 0, 2.788216, 0.069792, 0, 2.788216, 0.069792, 0, 2.783463, -0.013477, 0, 2.810840, -0.009489, 0, 2.783463, -0.013477, 0, 2.788216, 0.069792, 0, 2.810840, -0.009489, 0, 2.788216, 0.069792, 0, 2.810840, -0.009489, 0, 2.814762, 0.073698, 0, 2.810840, -0.009489, 0, 2.788216, 0.069792, 0, 2.814762, 0.073698, 0, 2.814762, 0.073698, 0, 2.810840, -0.009489, 0, 2.838021, -0.003906, 0, 2.810840, -0.009489, 0, 2.814762, 0.073698, 0, 2.838021, -0.003906, 0, 2.814762, 0.073698, 0, 2.838021, -0.003906, 0, 2.840885, 0.079167, 0, 2.838021, -0.003906, 0, 2.814762, 0.073698, 0, 2.840885, 0.079167, 0, 2.840885, 0.079167, 0, 2.838021, -0.003906, 0, 2.864941, 0.003206, 0, 2.838021, -0.003906, 0, 2.840885, 0.079167, 0, 2.864941, 0.003206, 0, 2.866748, 0.086198, 0, 2.840885, 0.079167, 0, 2.864941, 0.003206, 0, 2.840885, 0.079167, 0, 2.866748, 0.086198, 0, 2.864941, 0.003206, 0, 2.891536, 0.011784, 0, 2.866748, 0.086198, 0, 2.864941, 0.003206, 0, 2.866748, 0.086198, 0, 2.891536, 0.011784, 0, 2.864941, 0.003206, 0, 2.866748, 0.086198, 0, 2.891536, 0.011784, 0, 2.917806, 0.021826, 0, 2.891536, 0.011784, 0, 2.866748, 0.086198, 0, 2.917806, 0.021826, 0, 2.892513, 0.094792, 0, 2.866748, 0.086198, 0, 2.917806, 0.021826, 0, 2.866748, 0.086198, 0, 2.892513, 0.094792, 0, 2.917806, 0.021826, 0, 2.892513, 0.094792, 0, 2.917806, 0.021826, 0, 2.943750, 0.033333, 0, 2.917806, 0.021826, 0, 2.892513, 0.094792, 0, 2.943750, 0.033333, 0, 2.918180, 0.104948, 0, 2.892513, 0.094792, 0, 2.943750, 0.033333, 0, 2.892513, 0.094792, 0, 2.918180, 0.104948, 0, 2.943750, 0.033333, 0, 2.918180, 0.104948, 0, 2.943750, 0.033333, 0, 2.943750, 0.116667, 0, 2.943750, 0.033333, 0, 2.918180, 0.104948, 0, 2.943750, 0.116667, 0, 2.870312, 0.350000, 0, 2.867171, 0.386100, 0, 2.870312, 0.350000, 0, 2.870312, 0.350000, 0, 2.867171, 0.386100, 0, 2.867171, 0.386100, 0, 2.867171, 0.386100, 0, 2.858789, 0.418359, 0, 2.867171, 0.386100, 0, 2.867171, 0.386100, 0, 2.858789, 0.418359, 0, 2.858789, 0.418359, 0, 2.858789, 0.418359, 0, 2.845166, 0.446777, 0, 2.858789, 0.418359, 0, 2.858789, 0.418359, 0, 2.845166, 0.446777, 0, 2.845166, 0.446777, 0, 2.845166, 0.446777, 0, 2.826302, 0.471354, 0, 2.845166, 0.446777, 0, 2.845166, 0.446777, 0, 2.826302, 0.471354, 0, 2.826302, 0.471354, 0, 2.826302, 0.471354, 0, 2.803076, 0.491178, 0, 2.826302, 0.471354, 0, 2.826302, 0.471354, 0, 2.803076, 0.491178, 0, 2.803076, 0.491178, 0, 2.803076, 0.491178, 0, 2.776367, 0.505339, 0, 2.803076, 0.491178, 0, 2.803076, 0.491178, 0, 2.776367, 0.505339, 0, 2.776367, 0.505339, 0, 2.776367, 0.505339, 0, 2.746175, 0.513835, 0, 2.776367, 0.505339, 0, 2.776367, 0.505339, 0, 2.746175, 0.513835, 0, 2.746175, 0.513835, 0, 2.746175, 0.513835, 0, 2.712500, 0.516667, 0, 2.746175, 0.513835, 0, 2.746175, 0.513835, 0, 2.712500, 0.516667, 0, 2.712500, 0.516667, 0, 2.712500, 0.516667, 0, 2.674398, 0.513932, 0, 2.712500, 0.516667, 0, 2.712500, 0.516667, 0, 2.674398, 0.513932, 0, 2.674398, 0.513932, 0, 2.674398, 0.513932, 0, 2.640300, 0.505729, 0, 2.674398, 0.513932, 0, 2.674398, 0.513932, 0, 2.640300, 0.505729, 0, 2.640300, 0.505729, 0, 2.640300, 0.505729, 0, 2.610205, 0.492057, 0, 2.640300, 0.505729, 0, 2.640300, 0.505729, 0, 2.610205, 0.492057, 0, 2.610205, 0.492057, 0, 2.610205, 0.492057, 0, 2.584115, 0.472917, 0, 2.610205, 0.492057, 0, 2.610205, 0.492057, 0, 2.584115, 0.472917, 0, 2.584115, 0.472917, 0, 2.584115, 0.472917, 0, 2.562582, 0.448796, 0, 2.584115, 0.472917, 0, 2.584115, 0.472917, 0, 2.562582, 0.448796, 0, 2.562582, 0.448796, 0, 2.562582, 0.448796, 0, 2.546159, 0.420182, 0, 2.562582, 0.448796, 0, 2.562582, 0.448796, 0, 2.546159, 0.420182, 0, 2.546159, 0.420182, 0, 2.546159, 0.420182, 0, 2.534847, 0.387077, 0, 2.546159, 0.420182, 0, 2.546159, 0.420182, 0, 2.534847, 0.387077, 0, 2.534847, 0.387077, 0, 2.534847, 0.387077, 0, 2.528646, 0.349479, 0, 2.534847, 0.387077, 0, 2.534847, 0.387077, 0, 2.528646, 0.349479, 0, 2.528646, 0.349479, 0, 3.676563, 0.516667, 0, 3.640039, 0.512907, 0, 3.676563, 0.516667, 0, 3.676563, 0.516667, 0, 3.640039, 0.512907, 0, 3.640039, 0.512907, 0, 3.640039, 0.512907, 0, 3.607552, 0.501628, 0, 3.640039, 0.512907, 0, 3.640039, 0.512907, 0, 3.607552, 0.501628, 0, 3.607552, 0.501628, 0, 3.607552, 0.501628, 0, 3.579102, 0.482829, 0, 3.607552, 0.501628, 0, 3.607552, 0.501628, 0, 3.579102, 0.482829, 0, 3.579102, 0.482829, 0, 3.579102, 0.482829, 0, 3.554688, 0.456510, 0, 3.579102, 0.482829, 0, 3.579102, 0.482829, 0, 3.554688, 0.456510, 0, 3.554688, 0.456510, 0, 3.554688, 0.456510, 0, 3.535091, 0.423649, 0, 3.554688, 0.456510, 0, 3.554688, 0.456510, 0, 3.535091, 0.423649, 0, 3.535091, 0.423649, 0, 3.535091, 0.423649, 0, 3.521094, 0.385221, 0, 3.535091, 0.423649, 0, 3.535091, 0.423649, 0, 3.521094, 0.385221, 0, 3.521094, 0.385221, 0, 3.521094, 0.385221, 0, 3.512695, 0.341227, 0, 3.521094, 0.385221, 0, 3.521094, 0.385221, 0, 3.512695, 0.341227, 0, 3.512695, 0.341227, 0, 3.512695, 0.341227, 0, 3.509896, 0.291667, 0, 3.512695, 0.341227, 0, 3.512695, 0.341227, 0, 3.509896, 0.291667, 0, 3.509896, 0.291667, 0, 3.509896, 0.291667, 0, 3.512679, 0.242106, 0, 3.509896, 0.291667, 0, 3.509896, 0.291667, 0, 3.512679, 0.242106, 0, 3.512679, 0.242106, 0, 3.512679, 0.242106, 0, 3.521029, 0.198112, 0, 3.512679, 0.242106, 0, 3.512679, 0.242106, 0, 3.521029, 0.198112, 0, 3.521029, 0.198112, 0, 3.521029, 0.198112, 0, 3.534945, 0.159684, 0, 3.521029, 0.198112, 0, 3.521029, 0.198112, 0, 3.534945, 0.159684, 0, 3.534945, 0.159684, 0, 3.534945, 0.159684, 0, 3.554427, 0.126823, 0, 3.534945, 0.159684, 0, 3.534945, 0.159684, 0, 3.554427, 0.126823, 0, 3.554427, 0.126823, 0, 3.554427, 0.126823, 0, 3.578760, 0.100505, 0, 3.554427, 0.126823, 0, 3.554427, 0.126823, 0, 3.578760, 0.100505, 0, 3.578760, 0.100505, 0, 3.578760, 0.100505, 0, 3.607227, 0.081706, 0, 3.578760, 0.100505, 0, 3.578760, 0.100505, 0, 3.607227, 0.081706, 0, 3.607227, 0.081706, 0, 3.607227, 0.081706, 0, 3.639827, 0.070426, 0, 3.607227, 0.081706, 0, 3.607227, 0.081706, 0, 3.639827, 0.070426, 0, 3.639827, 0.070426, 0, 3.639827, 0.070426, 0, 3.676563, 0.066667, 0, 3.639827, 0.070426, 0, 3.639827, 0.070426, 0, 3.676563, 0.066667, 0, 3.676563, 0.066667, 0, 3.676563, 0.066667, 0, 3.712858, 0.070443, 0, 3.676563, 0.066667, 0, 3.676563, 0.066667, 0, 3.712858, 0.070443, 0, 3.712858, 0.070443, 0, 3.712858, 0.070443, 0, 3.745183, 0.081771, 0, 3.712858, 0.070443, 0, 3.712858, 0.070443, 0, 3.745183, 0.081771, 0, 3.745183, 0.081771, 0, 3.745183, 0.081771, 0, 3.773535, 0.100651, 0, 3.745183, 0.081771, 0, 3.745183, 0.081771, 0, 3.773535, 0.100651, 0, 3.773535, 0.100651, 0, 3.773535, 0.100651, 0, 3.797917, 0.127083, 0, 3.773535, 0.100651, 0, 3.773535, 0.100651, 0, 3.797917, 0.127083, 0, 3.797917, 0.127083, 0, 3.797917, 0.127083, 0, 3.817513, 0.160026, 0, 3.797917, 0.127083, 0, 3.797917, 0.127083, 0, 3.817513, 0.160026, 0, 3.817513, 0.160026, 0, 3.817513, 0.160026, 0, 3.831511, 0.198437, 0, 3.817513, 0.160026, 0, 3.817513, 0.160026, 0, 3.831511, 0.198437, 0, 3.831511, 0.198437, 0, 3.831511, 0.198437, 0, 3.839909, 0.242318, 0, 3.831511, 0.198437, 0, 3.831511, 0.198437, 0, 3.839909, 0.242318, 0, 3.839909, 0.242318, 0, 3.839909, 0.242318, 0, 3.842708, 0.291667, 0, 3.839909, 0.242318, 0, 3.839909, 0.242318, 0, 3.842708, 0.291667, 0, 3.842708, 0.291667, 0, 3.842708, 0.291667, 0, 3.839909, 0.340804, 0, 3.842708, 0.291667, 0, 3.842708, 0.291667, 0, 3.839909, 0.340804, 0, 3.839909, 0.340804, 0, 3.839909, 0.340804, 0, 3.831511, 0.384570, 0, 3.839909, 0.340804, 0, 3.839909, 0.340804, 0, 3.831511, 0.384570, 0, 3.831511, 0.384570, 0, 3.831511, 0.384570, 0, 3.817513, 0.422965, 0, 3.831511, 0.384570, 0, 3.831511, 0.384570, 0, 3.817513, 0.422965, 0, 3.817513, 0.422965, 0, 3.817513, 0.422965, 0, 3.797917, 0.455990, 0, 3.817513, 0.422965, 0, 3.817513, 0.422965, 0, 3.797917, 0.455990, 0, 3.797917, 0.455990, 0, 3.797917, 0.455990, 0, 3.773535, 0.482536, 0, 3.797917, 0.455990, 0, 3.797917, 0.455990, 0, 3.773535, 0.482536, 0, 3.773535, 0.482536, 0, 3.773535, 0.482536, 0, 3.745183, 0.501497, 0, 3.773535, 0.482536, 0, 3.773535, 0.482536, 0, 3.745183, 0.501497, 0, 3.745183, 0.501497, 0, 3.745183, 0.501497, 0, 3.712858, 0.512874, 0, 3.745183, 0.501497, 0, 3.745183, 0.501497, 0, 3.712858, 0.512874, 0, 3.712858, 0.512874, 0, 3.676563, 0.600000, 0, 3.735710, 0.594889, 0, 3.676563, 0.600000, 0, 3.676563, 0.600000, 0, 3.735710, 0.594889, 0, 3.735710, 0.594889, 0, 3.735710, 0.594889, 0, 3.788151, 0.579557, 0, 3.735710, 0.594889, 0, 3.735710, 0.594889, 0, 3.788151, 0.579557, 0, 3.788151, 0.579557, 0, 3.788151, 0.579557, 0, 3.833887, 0.554004, 0, 3.788151, 0.579557, 0, 3.788151, 0.579557, 0, 3.833887, 0.554004, 0, 3.833887, 0.554004, 0, 3.833887, 0.554004, 0, 3.872917, 0.518229, 0, 3.833887, 0.554004, 0, 3.833887, 0.554004, 0, 3.872917, 0.518229, 0, 3.872917, 0.518229, 0, 3.872917, 0.518229, 0, 3.904134, 0.473405, 0, 3.872917, 0.518229, 0, 3.872917, 0.518229, 0, 3.904134, 0.473405, 0, 3.904134, 0.473405, 0, 3.904134, 0.473405, 0, 3.926432, 0.420703, 0, 3.904134, 0.473405, 0, 3.904134, 0.473405, 0, 3.926432, 0.420703, 0, 3.926432, 0.420703, 0, 3.926432, 0.420703, 0, 3.939811, 0.360124, 0, 3.926432, 0.420703, 0, 3.926432, 0.420703, 0, 3.939811, 0.360124, 0, 3.939811, 0.360124, 0, 3.939811, 0.360124, 0, 3.944271, 0.291667, 0, 3.939811, 0.360124, 0, 3.939811, 0.360124, 0, 3.944271, 0.291667, 0, 3.944271, 0.291667, 0, 3.944271, 0.291667, 0, 3.939811, 0.223421, 0, 3.944271, 0.291667, 0, 3.944271, 0.291667, 0, 3.939811, 0.223421, 0, 3.939811, 0.223421, 0, 3.939811, 0.223421, 0, 3.926432, 0.162956, 0, 3.939811, 0.223421, 0, 3.939811, 0.223421, 0, 3.926432, 0.162956, 0, 3.926432, 0.162956, 0, 3.926432, 0.162956, 0, 3.904134, 0.110270, 0, 3.926432, 0.162956, 0, 3.926432, 0.162956, 0, 3.904134, 0.110270, 0, 3.904134, 0.110270, 0, 3.904134, 0.110270, 0, 3.872917, 0.065365, 0, 3.904134, 0.110270, 0, 3.904134, 0.110270, 0, 3.872917, 0.065365, 0, 3.872917, 0.065365, 0, 3.872917, 0.065365, 0, 3.833887, 0.029476, 0, 3.872917, 0.065365, 0, 3.872917, 0.065365, 0, 3.833887, 0.029476, 0, 3.833887, 0.029476, 0, 3.833887, 0.029476, 0, 3.788151, 0.003841, 0, 3.833887, 0.029476, 0, 3.833887, 0.029476, 0, 3.788151, 0.003841, 0, 3.788151, 0.003841, 0, 3.788151, 0.003841, 0, 3.735710, -0.011540, 0, 3.788151, 0.003841, 0, 3.788151, 0.003841, 0, 3.735710, -0.011540, 0, 3.735710, -0.011540, 0, 3.735710, -0.011540, 0, 3.676563, -0.016667, 0, 3.735710, -0.011540, 0, 3.735710, -0.011540, 0, 3.676563, -0.016667, 0, 3.676563, -0.016667, 0, 3.676563, -0.016667, 0, 3.617204, -0.011540, 0, 3.676563, -0.016667, 0, 3.676563, -0.016667, 0, 3.617204, -0.011540, 0, 3.617204, -0.011540, 0, 3.617204, -0.011540, 0, 3.564648, 0.003841, 0, 3.617204, -0.011540, 0, 3.617204, -0.011540, 0, 3.564648, 0.003841, 0, 3.564648, 0.003841, 0, 3.564648, 0.003841, 0, 3.518897, 0.029476, 0, 3.564648, 0.003841, 0, 3.564648, 0.003841, 0, 3.518897, 0.029476, 0, 3.518897, 0.029476, 0, 3.518897, 0.029476, 0, 3.479948, 0.065365, 0, 3.518897, 0.029476, 0, 3.518897, 0.029476, 0, 3.479948, 0.065365, 0, 3.479948, 0.065365, 0, 3.479948, 0.065365, 0, 3.448845, 0.110270, 0, 3.479948, 0.065365, 0, 3.479948, 0.065365, 0, 3.448845, 0.110270, 0, 3.448845, 0.110270, 0, 3.448845, 0.110270, 0, 3.426628, 0.162956, 0, 3.448845, 0.110270, 0, 3.448845, 0.110270, 0, 3.426628, 0.162956, 0, 3.426628, 0.162956, 0, 3.426628, 0.162956, 0, 3.413298, 0.223421, 0, 3.426628, 0.162956, 0, 3.426628, 0.162956, 0, 3.413298, 0.223421, 0, 3.413298, 0.223421, 0, 3.413298, 0.223421, 0, 3.408854, 0.291667, 0, 3.413298, 0.223421, 0, 3.413298, 0.223421, 0, 3.408854, 0.291667, 0, 3.408854, 0.291667, 0, 3.408854, 0.291667, 0, 3.413298, 0.360124, 0, 3.408854, 0.291667, 0, 3.408854, 0.291667, 0, 3.413298, 0.360124, 0, 3.413298, 0.360124, 0, 3.413298, 0.360124, 0, 3.426628, 0.420703, 0, 3.413298, 0.360124, 0, 3.413298, 0.360124, 0, 3.426628, 0.420703, 0, 3.426628, 0.420703, 0, 3.426628, 0.420703, 0, 3.448845, 0.473405, 0, 3.426628, 0.420703, 0, 3.426628, 0.420703, 0, 3.448845, 0.473405, 0, 3.448845, 0.473405, 0, 3.448845, 0.473405, 0, 3.479948, 0.518229, 0, 3.448845, 0.473405, 0, 3.448845, 0.473405, 0, 3.479948, 0.518229, 0, 3.479948, 0.518229, 0, 3.479948, 0.518229, 0, 3.518897, 0.554004, 0, 3.479948, 0.518229, 0, 3.479948, 0.518229, 0, 3.518897, 0.554004, 0, 3.518897, 0.554004, 0, 3.518897, 0.554004, 0, 3.564648, 0.579557, 0, 3.518897, 0.554004, 0, 3.518897, 0.554004, 0, 3.564648, 0.579557, 0, 3.564648, 0.579557, 0, 3.564648, 0.579557, 0, 3.617204, 0.594889, 0, 3.564648, 0.579557, 0, 3.564648, 0.579557, 0, 3.617204, 0.594889, 0, 3.617204, 0.594889, 0, 3.512695, 0.341227, 0, 3.413298, 0.360124, 0, 3.408854, 0.291667, 0, 3.413298, 0.360124, 0, 3.512695, 0.341227, 0, 3.408854, 0.291667, 0, 3.521094, 0.385221, 0, 3.413298, 0.360124, 0, 3.512695, 0.341227, 0, 3.413298, 0.360124, 0, 3.521094, 0.385221, 0, 3.512695, 0.341227, 0, 3.521094, 0.385221, 0, 3.426628, 0.420703, 0, 3.413298, 0.360124, 0, 3.426628, 0.420703, 0, 3.521094, 0.385221, 0, 3.413298, 0.360124, 0, 3.426628, 0.420703, 0, 3.521094, 0.385221, 0, 3.535091, 0.423649, 0, 3.521094, 0.385221, 0, 3.426628, 0.420703, 0, 3.535091, 0.423649, 0, 3.448845, 0.473405, 0, 3.426628, 0.420703, 0, 3.535091, 0.423649, 0, 3.426628, 0.420703, 0, 3.448845, 0.473405, 0, 3.535091, 0.423649, 0, 3.554688, 0.456510, 0, 3.448845, 0.473405, 0, 3.535091, 0.423649, 0, 3.448845, 0.473405, 0, 3.554688, 0.456510, 0, 3.535091, 0.423649, 0, 3.554688, 0.456510, 0, 3.479948, 0.518229, 0, 3.448845, 0.473405, 0, 3.479948, 0.518229, 0, 3.554688, 0.456510, 0, 3.448845, 0.473405, 0, 3.518897, 0.554004, 0, 3.479948, 0.518229, 0, 3.554688, 0.456510, 0, 3.479948, 0.518229, 0, 3.518897, 0.554004, 0, 3.554688, 0.456510, 0, 3.518897, 0.554004, 0, 3.554688, 0.456510, 0, 3.579102, 0.482829, 0, 3.554688, 0.456510, 0, 3.518897, 0.554004, 0, 3.579102, 0.482829, 0, 3.564648, 0.579557, 0, 3.518897, 0.554004, 0, 3.579102, 0.482829, 0, 3.518897, 0.554004, 0, 3.564648, 0.579557, 0, 3.579102, 0.482829, 0, 3.564648, 0.579557, 0, 3.579102, 0.482829, 0, 3.607552, 0.501628, 0, 3.579102, 0.482829, 0, 3.564648, 0.579557, 0, 3.607552, 0.501628, 0, 3.564648, 0.579557, 0, 3.607552, 0.501628, 0, 3.617204, 0.594889, 0, 3.607552, 0.501628, 0, 3.564648, 0.579557, 0, 3.617204, 0.594889, 0, 3.617204, 0.594889, 0, 3.607552, 0.501628, 0, 3.640039, 0.512907, 0, 3.607552, 0.501628, 0, 3.617204, 0.594889, 0, 3.640039, 0.512907, 0, 3.617204, 0.594889, 0, 3.640039, 0.512907, 0, 3.676563, 0.600000, 0, 3.640039, 0.512907, 0, 3.617204, 0.594889, 0, 3.676563, 0.600000, 0, 3.640039, 0.512907, 0, 3.676563, 0.516667, 0, 3.676563, 0.600000, 0, 3.676563, 0.516667, 0, 3.640039, 0.512907, 0, 3.676563, 0.600000, 0, 3.676563, 0.600000, 0, 3.676563, 0.516667, 0, 3.712858, 0.512874, 0, 3.676563, 0.516667, 0, 3.676563, 0.600000, 0, 3.712858, 0.512874, 0, 3.676563, 0.600000, 0, 3.712858, 0.512874, 0, 3.735710, 0.594889, 0, 3.712858, 0.512874, 0, 3.676563, 0.600000, 0, 3.735710, 0.594889, 0, 3.735710, 0.594889, 0, 3.712858, 0.512874, 0, 3.745183, 0.501497, 0, 3.712858, 0.512874, 0, 3.735710, 0.594889, 0, 3.745183, 0.501497, 0, 3.735710, 0.594889, 0, 3.745183, 0.501497, 0, 3.788151, 0.579557, 0, 3.745183, 0.501497, 0, 3.735710, 0.594889, 0, 3.788151, 0.579557, 0, 3.745183, 0.501497, 0, 3.773535, 0.482536, 0, 3.788151, 0.579557, 0, 3.773535, 0.482536, 0, 3.745183, 0.501497, 0, 3.788151, 0.579557, 0, 3.788151, 0.579557, 0, 3.773535, 0.482536, 0, 3.833887, 0.554004, 0, 3.773535, 0.482536, 0, 3.788151, 0.579557, 0, 3.833887, 0.554004, 0, 3.773535, 0.482536, 0, 3.797917, 0.455990, 0, 3.833887, 0.554004, 0, 3.797917, 0.455990, 0, 3.773535, 0.482536, 0, 3.833887, 0.554004, 0, 3.833887, 0.554004, 0, 3.797917, 0.455990, 0, 3.872917, 0.518229, 0, 3.797917, 0.455990, 0, 3.833887, 0.554004, 0, 3.872917, 0.518229, 0, 3.872917, 0.518229, 0, 3.797917, 0.455990, 0, 3.904134, 0.473405, 0, 3.797917, 0.455990, 0, 3.872917, 0.518229, 0, 3.904134, 0.473405, 0, 3.904134, 0.473405, 0, 3.797917, 0.455990, 0, 3.817513, 0.422965, 0, 3.797917, 0.455990, 0, 3.904134, 0.473405, 0, 3.817513, 0.422965, 0, 3.904134, 0.473405, 0, 3.817513, 0.422965, 0, 3.926432, 0.420703, 0, 3.817513, 0.422965, 0, 3.904134, 0.473405, 0, 3.926432, 0.420703, 0, 3.817513, 0.422965, 0, 3.831511, 0.384570, 0, 3.926432, 0.420703, 0, 3.831511, 0.384570, 0, 3.817513, 0.422965, 0, 3.926432, 0.420703, 0, 3.926432, 0.420703, 0, 3.831511, 0.384570, 0, 3.939811, 0.360124, 0, 3.831511, 0.384570, 0, 3.926432, 0.420703, 0, 3.939811, 0.360124, 0, 3.831511, 0.384570, 0, 3.839909, 0.340804, 0, 3.939811, 0.360124, 0, 3.839909, 0.340804, 0, 3.831511, 0.384570, 0, 3.939811, 0.360124, 0, 3.939811, 0.360124, 0, 3.839909, 0.340804, 0, 3.944271, 0.291667, 0, 3.839909, 0.340804, 0, 3.939811, 0.360124, 0, 3.944271, 0.291667, 0, 3.944271, 0.291667, 0, 3.839909, 0.340804, 0, 3.842708, 0.291667, 0, 3.839909, 0.340804, 0, 3.944271, 0.291667, 0, 3.842708, 0.291667, 0, 3.944271, 0.291667, 0, 3.842708, 0.291667, 0, 3.839909, 0.242318, 0, 3.842708, 0.291667, 0, 3.944271, 0.291667, 0, 3.839909, 0.242318, 0, 3.944271, 0.291667, 0, 3.839909, 0.242318, 0, 3.939811, 0.223421, 0, 3.839909, 0.242318, 0, 3.944271, 0.291667, 0, 3.939811, 0.223421, 0, 3.839909, 0.242318, 0, 3.831511, 0.198437, 0, 3.939811, 0.223421, 0, 3.831511, 0.198437, 0, 3.839909, 0.242318, 0, 3.939811, 0.223421, 0, 3.831511, 0.198437, 0, 3.926432, 0.162956, 0, 3.939811, 0.223421, 0, 3.926432, 0.162956, 0, 3.831511, 0.198437, 0, 3.939811, 0.223421, 0, 3.831511, 0.198437, 0, 3.817513, 0.160026, 0, 3.926432, 0.162956, 0, 3.817513, 0.160026, 0, 3.831511, 0.198437, 0, 3.926432, 0.162956, 0, 3.817513, 0.160026, 0, 3.904134, 0.110270, 0, 3.926432, 0.162956, 0, 3.904134, 0.110270, 0, 3.817513, 0.160026, 0, 3.926432, 0.162956, 0, 3.817513, 0.160026, 0, 3.797917, 0.127083, 0, 3.904134, 0.110270, 0, 3.797917, 0.127083, 0, 3.817513, 0.160026, 0, 3.904134, 0.110270, 0, 3.797917, 0.127083, 0, 3.872917, 0.065365, 0, 3.904134, 0.110270, 0, 3.872917, 0.065365, 0, 3.797917, 0.127083, 0, 3.904134, 0.110270, 0, 3.872917, 0.065365, 0, 3.797917, 0.127083, 0, 3.833887, 0.029476, 0, 3.797917, 0.127083, 0, 3.872917, 0.065365, 0, 3.833887, 0.029476, 0, 3.797917, 0.127083, 0, 3.773535, 0.100651, 0, 3.833887, 0.029476, 0, 3.773535, 0.100651, 0, 3.797917, 0.127083, 0, 3.833887, 0.029476, 0, 3.833887, 0.029476, 0, 3.773535, 0.100651, 0, 3.788151, 0.003841, 0, 3.773535, 0.100651, 0, 3.833887, 0.029476, 0, 3.788151, 0.003841, 0, 3.773535, 0.100651, 0, 3.745183, 0.081771, 0, 3.788151, 0.003841, 0, 3.745183, 0.081771, 0, 3.773535, 0.100651, 0, 3.788151, 0.003841, 0, 3.788151, 0.003841, 0, 3.745183, 0.081771, 0, 3.735710, -0.011540, 0, 3.745183, 0.081771, 0, 3.788151, 0.003841, 0, 3.735710, -0.011540, 0, 3.745183, 0.081771, 0, 3.712858, 0.070443, 0, 3.735710, -0.011540, 0, 3.712858, 0.070443, 0, 3.745183, 0.081771, 0, 3.735710, -0.011540, 0, 3.735710, -0.011540, 0, 3.712858, 0.070443, 0, 3.676563, -0.016667, 0, 3.712858, 0.070443, 0, 3.735710, -0.011540, 0, 3.676563, -0.016667, 0, 3.712858, 0.070443, 0, 3.676563, 0.066667, 0, 3.676563, -0.016667, 0, 3.676563, 0.066667, 0, 3.712858, 0.070443, 0, 3.676563, -0.016667, 0, 3.676563, 0.066667, 0, 3.639827, 0.070426, 0, 3.676563, -0.016667, 0, 3.639827, 0.070426, 0, 3.676563, 0.066667, 0, 3.676563, -0.016667, 0, 3.639827, 0.070426, 0, 3.617204, -0.011540, 0, 3.676563, -0.016667, 0, 3.617204, -0.011540, 0, 3.639827, 0.070426, 0, 3.676563, -0.016667, 0, 3.639827, 0.070426, 0, 3.607227, 0.081706, 0, 3.617204, -0.011540, 0, 3.607227, 0.081706, 0, 3.639827, 0.070426, 0, 3.617204, -0.011540, 0, 3.607227, 0.081706, 0, 3.564648, 0.003841, 0, 3.617204, -0.011540, 0, 3.564648, 0.003841, 0, 3.607227, 0.081706, 0, 3.617204, -0.011540, 0, 3.607227, 0.081706, 0, 3.578760, 0.100505, 0, 3.564648, 0.003841, 0, 3.578760, 0.100505, 0, 3.607227, 0.081706, 0, 3.564648, 0.003841, 0, 3.578760, 0.100505, 0, 3.518897, 0.029476, 0, 3.564648, 0.003841, 0, 3.518897, 0.029476, 0, 3.578760, 0.100505, 0, 3.564648, 0.003841, 0, 3.578760, 0.100505, 0, 3.554427, 0.126823, 0, 3.518897, 0.029476, 0, 3.554427, 0.126823, 0, 3.578760, 0.100505, 0, 3.518897, 0.029476, 0, 3.554427, 0.126823, 0, 3.479948, 0.065365, 0, 3.518897, 0.029476, 0, 3.479948, 0.065365, 0, 3.554427, 0.126823, 0, 3.518897, 0.029476, 0, 3.448845, 0.110270, 0, 3.479948, 0.065365, 0, 3.554427, 0.126823, 0, 3.479948, 0.065365, 0, 3.448845, 0.110270, 0, 3.554427, 0.126823, 0, 3.534945, 0.159684, 0, 3.448845, 0.110270, 0, 3.554427, 0.126823, 0, 3.448845, 0.110270, 0, 3.534945, 0.159684, 0, 3.554427, 0.126823, 0, 3.426628, 0.162956, 0, 3.448845, 0.110270, 0, 3.534945, 0.159684, 0, 3.448845, 0.110270, 0, 3.426628, 0.162956, 0, 3.534945, 0.159684, 0, 3.521029, 0.198112, 0, 3.426628, 0.162956, 0, 3.534945, 0.159684, 0, 3.426628, 0.162956, 0, 3.521029, 0.198112, 0, 3.534945, 0.159684, 0, 3.413298, 0.223421, 0, 3.426628, 0.162956, 0, 3.521029, 0.198112, 0, 3.426628, 0.162956, 0, 3.413298, 0.223421, 0, 3.521029, 0.198112, 0, 3.512679, 0.242106, 0, 3.413298, 0.223421, 0, 3.521029, 0.198112, 0, 3.413298, 0.223421, 0, 3.512679, 0.242106, 0, 3.521029, 0.198112, 0, 3.408854, 0.291667, 0, 3.413298, 0.223421, 0, 3.512679, 0.242106, 0, 3.413298, 0.223421, 0, 3.408854, 0.291667, 0, 3.512679, 0.242106, 0, 3.509896, 0.291667, 0, 3.408854, 0.291667, 0, 3.512679, 0.242106, 0, 3.408854, 0.291667, 0, 3.509896, 0.291667, 0, 3.512679, 0.242106, 0, 3.512695, 0.341227, 0, 3.408854, 0.291667, 0, 3.509896, 0.291667, 0, 3.408854, 0.291667, 0, 3.512695, 0.341227, 0, 3.509896, 0.291667, 0, 4.031771, 0.583333, 0, 4.133333, 0.583333, 0, 4.031771, 0.583333, 0, 4.031771, 0.583333, 0, 4.133333, 0.583333, 0, 4.133333, 0.583333, 0, 4.133333, 0.583333, 0, 4.315625, 0.093750, 0, 4.133333, 0.583333, 0, 4.133333, 0.583333, 0, 4.315625, 0.093750, 0, 4.315625, 0.093750, 0, 4.315625, 0.093750, 0, 4.497917, 0.583333, 0, 4.315625, 0.093750, 0, 4.315625, 0.093750, 0, 4.497917, 0.583333, 0, 4.497917, 0.583333, 0, 4.497917, 0.583333, 0, 4.599479, 0.583333, 0, 4.497917, 0.583333, 0, 4.497917, 0.583333, 0, 4.599479, 0.583333, 0, 4.599479, 0.583333, 0, 4.599479, 0.583333, 0, 4.380729, 0, 0, 4.599479, 0.583333, 0, 4.599479, 0.583333, 0, 4.380729, 0, 0, 4.380729, 0, 0, 4.380729, 0, 0, 4.250521, 0, 0, 4.380729, 0, 0, 4.380729, 0, 0, 4.250521, 0, 0, 4.250521, 0, 0, 4.133333, 0.583333, 0, 4.031771, 0.583333, 0, 4.315625, 0.093750, 0, 4.031771, 0.583333, 0, 4.133333, 0.583333, 0, 4.315625, 0.093750, 0, 4.031771, 0.583333, 0, 4.250521, 0, 0, 4.315625, 0.093750, 0, 4.250521, 0, 0, 4.031771, 0.583333, 0, 4.315625, 0.093750, 0, 4.315625, 0.093750, 0, 4.250521, 0, 0, 4.380729, 0, 0, 4.250521, 0, 0, 4.315625, 0.093750, 0, 4.380729, 0, 0, 4.380729, 0, 0, 4.599479, 0.583333, 0, 4.315625, 0.093750, 0, 4.599479, 0.583333, 0, 4.380729, 0, 0, 4.315625, 0.093750, 0, 4.497917, 0.583333, 0, 4.315625, 0.093750, 0, 4.599479, 0.583333, 0, 4.315625, 0.093750, 0, 4.497917, 0.583333, 0, 4.599479, 0.583333, 0, 5.232812, 0.318750, 0, 5.232812, 0.266667, 0, 5.232812, 0.318750, 0, 5.232812, 0.318750, 0, 5.232812, 0.266667, 0, 5.232812, 0.266667, 0, 5.232812, 0.266667, 0, 4.792188, 0.266667, 0, 5.232812, 0.266667, 0, 5.232812, 0.266667, 0, 4.792188, 0.266667, 0, 4.792188, 0.266667, 0, 4.792188, 0.266667, 0, 4.798259, 0.220744, 0, 4.792188, 0.266667, 0, 4.792188, 0.266667, 0, 4.798259, 0.220744, 0, 4.798259, 0.220744, 0, 4.798259, 0.220744, 0, 4.810222, 0.180632, 0, 4.798259, 0.220744, 0, 4.798259, 0.220744, 0, 4.810222, 0.180632, 0, 4.810222, 0.180632, 0, 4.810222, 0.180632, 0, 4.828076, 0.146330, 0, 4.810222, 0.180632, 0, 4.810222, 0.180632, 0, 4.828076, 0.146330, 0, 4.828076, 0.146330, 0, 4.828076, 0.146330, 0, 4.851823, 0.117839, 0, 4.828076, 0.146330, 0, 4.828076, 0.146330, 0, 4.851823, 0.117839, 0, 4.851823, 0.117839, 0, 4.851823, 0.117839, 0, 4.881136, 0.095451, 0, 4.851823, 0.117839, 0, 4.851823, 0.117839, 0, 4.881136, 0.095451, 0, 4.881136, 0.095451, 0, 4.881136, 0.095451, 0, 4.915690, 0.079460, 0, 4.881136, 0.095451, 0, 4.881136, 0.095451, 0, 4.915690, 0.079460, 0, 4.915690, 0.079460, 0, 4.915690, 0.079460, 0, 4.955485, 0.069865, 0, 4.915690, 0.079460, 0, 4.915690, 0.079460, 0, 4.955485, 0.069865, 0, 4.955485, 0.069865, 0, 4.955485, 0.069865, 0, 5.000521, 0.066667, 0, 4.955485, 0.069865, 0, 4.955485, 0.069865, 0, 5.000521, 0.066667, 0, 5.000521, 0.066667, 0, 5.000521, 0.066667, 0, 5.027914, 0.067448, 0, 5.000521, 0.066667, 0, 5.000521, 0.066667, 0, 5.027914, 0.067448, 0, 5.027914, 0.067448, 0, 5.027914, 0.067448, 0, 5.054883, 0.069792, 0, 5.027914, 0.067448, 0, 5.027914, 0.067448, 0, 5.054883, 0.069792, 0, 5.054883, 0.069792, 0, 5.054883, 0.069792, 0, 5.081429, 0.073698, 0, 5.054883, 0.069792, 0, 5.054883, 0.069792, 0, 5.081429, 0.073698, 0, 5.081429, 0.073698, 0, 5.081429, 0.073698, 0, 5.107552, 0.079167, 0, 5.081429, 0.073698, 0, 5.081429, 0.073698, 0, 5.107552, 0.079167, 0, 5.107552, 0.079167, 0, 5.107552, 0.079167, 0, 5.133415, 0.086198, 0, 5.107552, 0.079167, 0, 5.107552, 0.079167, 0, 5.133415, 0.086198, 0, 5.133415, 0.086198, 0, 5.133415, 0.086198, 0, 5.159180, 0.094792, 0, 5.133415, 0.086198, 0, 5.133415, 0.086198, 0, 5.159180, 0.094792, 0, 5.159180, 0.094792, 0, 5.159180, 0.094792, 0, 5.184847, 0.104948, 0, 5.159180, 0.094792, 0, 5.159180, 0.094792, 0, 5.184847, 0.104948, 0, 5.184847, 0.104948, 0, 5.184847, 0.104948, 0, 5.210417, 0.116667, 0, 5.184847, 0.104948, 0, 5.184847, 0.104948, 0, 5.210417, 0.116667, 0, 5.210417, 0.116667, 0, 5.210417, 0.116667, 0, 5.210417, 0.033333, 0, 5.210417, 0.116667, 0, 5.210417, 0.116667, 0, 5.210417, 0.033333, 0, 5.210417, 0.033333, 0, 5.210417, 0.033333, 0, 5.184473, 0.021826, 0, 5.210417, 0.033333, 0, 5.210417, 0.033333, 0, 5.184473, 0.021826, 0, 5.184473, 0.021826, 0, 5.184473, 0.021826, 0, 5.158203, 0.011784, 0, 5.184473, 0.021826, 0, 5.184473, 0.021826, 0, 5.158203, 0.011784, 0, 5.158203, 0.011784, 0, 5.158203, 0.011784, 0, 5.131608, 0.003206, 0, 5.158203, 0.011784, 0, 5.158203, 0.011784, 0, 5.131608, 0.003206, 0, 5.131608, 0.003206, 0, 5.131608, 0.003206, 0, 5.104688, -0.003906, 0, 5.131608, 0.003206, 0, 5.131608, 0.003206, 0, 5.104688, -0.003906, 0, 5.104688, -0.003906, 0, 5.104688, -0.003906, 0, 5.077507, -0.009489, 0, 5.104688, -0.003906, 0, 5.104688, -0.003906, 0, 5.077507, -0.009489, 0, 5.077507, -0.009489, 0, 5.077507, -0.009489, 0, 5.050130, -0.013477, 0, 5.077507, -0.009489, 0, 5.077507, -0.009489, 0, 5.050130, -0.013477, 0, 5.050130, -0.013477, 0, 5.050130, -0.013477, 0, 5.022559, -0.015869, 0, 5.050130, -0.013477, 0, 5.050130, -0.013477, 0, 5.022559, -0.015869, 0, 5.022559, -0.015869, 0, 5.022559, -0.015869, 0, 4.994792, -0.016667, 0, 5.022559, -0.015869, 0, 5.022559, -0.015869, 0, 4.994792, -0.016667, 0, 4.994792, -0.016667, 0, 4.994792, -0.016667, 0, 4.928630, -0.011556, 0, 4.994792, -0.016667, 0, 4.994792, -0.016667, 0, 4.928630, -0.011556, 0, 4.928630, -0.011556, 0, 4.928630, -0.011556, 0, 4.869727, 0.003776, 0, 4.928630, -0.011556, 0, 4.928630, -0.011556, 0, 4.869727, 0.003776, 0, 4.869727, 0.003776, 0, 4.869727, 0.003776, 0, 4.818083, 0.029329, 0, 4.869727, 0.003776, 0, 4.869727, 0.003776, 0, 4.818083, 0.029329, 0, 4.818083, 0.029329, 0, 4.818083, 0.029329, 0, 4.773698, 0.065104, 0, 4.818083, 0.029329, 0, 4.818083, 0.029329, 0, 4.773698, 0.065104, 0, 4.773698, 0.065104, 0, 4.773698, 0.065104, 0, 4.738037, 0.109603, 0, 4.773698, 0.065104, 0, 4.773698, 0.065104, 0, 4.738037, 0.109603, 0, 4.738037, 0.109603, 0, 4.738037, 0.109603, 0, 4.712565, 0.161328, 0, 4.738037, 0.109603, 0, 4.738037, 0.109603, 0, 4.712565, 0.161328, 0, 4.712565, 0.161328, 0, 4.712565, 0.161328, 0, 4.697282, 0.220280, 0, 4.712565, 0.161328, 0, 4.712565, 0.161328, 0, 4.697282, 0.220280, 0, 4.697282, 0.220280, 0, 4.697282, 0.220280, 0, 4.692188, 0.286458, 0, 4.697282, 0.220280, 0, 4.697282, 0.220280, 0, 4.692188, 0.286458, 0, 4.692188, 0.286458, 0, 4.692188, 0.286458, 0, 4.697021, 0.354867, 0, 4.692188, 0.286458, 0, 4.692188, 0.286458, 0, 4.697021, 0.354867, 0, 4.697021, 0.354867, 0, 4.697021, 0.354867, 0, 4.711524, 0.415820, 0, 4.697021, 0.354867, 0, 4.697021, 0.354867, 0, 4.711524, 0.415820, 0, 4.711524, 0.415820, 0, 4.711524, 0.415820, 0, 4.735693, 0.469320, 0, 4.711524, 0.415820, 0, 4.711524, 0.415820, 0, 4.735693, 0.469320, 0, 4.735693, 0.469320, 0, 4.735693, 0.469320, 0, 4.769531, 0.515365, 0, 4.735693, 0.469320, 0, 4.735693, 0.469320, 0, 4.769531, 0.515365, 0, 4.769531, 0.515365, 0, 4.769531, 0.515365, 0, 4.811573, 0.552393, 0, 4.769531, 0.515365, 0, 4.769531, 0.515365, 0, 4.811573, 0.552393, 0, 4.811573, 0.552393, 0, 4.811573, 0.552393, 0, 4.860352, 0.578841, 0, 4.811573, 0.552393, 0, 4.811573, 0.552393, 0, 4.860352, 0.578841, 0, 4.860352, 0.578841, 0, 4.860352, 0.578841, 0, 4.915869, 0.594710, 0, 4.860352, 0.578841, 0, 4.860352, 0.578841, 0, 4.915869, 0.594710, 0, 4.915869, 0.594710, 0, 4.915869, 0.594710, 0, 4.978125, 0.600000, 0, 4.915869, 0.594710, 0, 4.915869, 0.594710, 0, 4.978125, 0.600000, 0, 4.978125, 0.600000, 0, 4.978125, 0.600000, 0, 5.033903, 0.595272, 0, 4.978125, 0.600000, 0, 4.978125, 0.600000, 0, 5.033903, 0.595272, 0, 5.033903, 0.595272, 0, 5.033903, 0.595272, 0, 5.083529, 0.581087, 0, 5.033903, 0.595272, 0, 5.033903, 0.595272, 0, 5.083529, 0.581087, 0, 5.083529, 0.581087, 0, 5.083529, 0.581087, 0, 5.127002, 0.557446, 0, 5.083529, 0.581087, 0, 5.083529, 0.581087, 0, 5.127002, 0.557446, 0, 5.127002, 0.557446, 0, 5.127002, 0.557446, 0, 5.164323, 0.524349, 0, 5.127002, 0.557446, 0, 5.127002, 0.557446, 0, 5.164323, 0.524349, 0, 5.164323, 0.524349, 0, 5.164323, 0.524349, 0, 5.194287, 0.483130, 0, 5.164323, 0.524349, 0, 5.164323, 0.524349, 0, 5.194287, 0.483130, 0, 5.194287, 0.483130, 0, 5.194287, 0.483130, 0, 5.215690, 0.435124, 0, 5.194287, 0.483130, 0, 5.194287, 0.483130, 0, 5.215690, 0.435124, 0, 5.215690, 0.435124, 0, 5.215690, 0.435124, 0, 5.228532, 0.380330, 0, 5.215690, 0.435124, 0, 5.215690, 0.435124, 0, 5.228532, 0.380330, 0, 5.228532, 0.380330, 0, 4.697021, 0.354867, 0, 4.692188, 0.286458, 0, 4.795312, 0.349479, 0, 4.692188, 0.286458, 0, 4.697021, 0.354867, 0, 4.795312, 0.349479, 0, 4.711524, 0.415820, 0, 4.697021, 0.354867, 0, 4.795312, 0.349479, 0, 4.697021, 0.354867, 0, 4.711524, 0.415820, 0, 4.795312, 0.349479, 0, 4.711524, 0.415820, 0, 4.795312, 0.349479, 0, 4.801514, 0.387077, 0, 4.795312, 0.349479, 0, 4.711524, 0.415820, 0, 4.801514, 0.387077, 0, 4.812826, 0.420182, 0, 4.711524, 0.415820, 0, 4.801514, 0.387077, 0, 4.711524, 0.415820, 0, 4.812826, 0.420182, 0, 4.801514, 0.387077, 0, 4.812826, 0.420182, 0, 4.735693, 0.469320, 0, 4.711524, 0.415820, 0, 4.735693, 0.469320, 0, 4.812826, 0.420182, 0, 4.711524, 0.415820, 0, 4.735693, 0.469320, 0, 4.812826, 0.420182, 0, 4.829248, 0.448796, 0, 4.812826, 0.420182, 0, 4.735693, 0.469320, 0, 4.829248, 0.448796, 0, 4.769531, 0.515365, 0, 4.735693, 0.469320, 0, 4.829248, 0.448796, 0, 4.735693, 0.469320, 0, 4.769531, 0.515365, 0, 4.829248, 0.448796, 0, 4.769531, 0.515365, 0, 4.829248, 0.448796, 0, 4.850781, 0.472917, 0, 4.829248, 0.448796, 0, 4.769531, 0.515365, 0, 4.850781, 0.472917, 0, 4.811573, 0.552393, 0, 4.769531, 0.515365, 0, 4.850781, 0.472917, 0, 4.769531, 0.515365, 0, 4.811573, 0.552393, 0, 4.850781, 0.472917, 0, 4.811573, 0.552393, 0, 4.850781, 0.472917, 0, 4.876872, 0.492057, 0, 4.850781, 0.472917, 0, 4.811573, 0.552393, 0, 4.876872, 0.492057, 0, 4.860352, 0.578841, 0, 4.811573, 0.552393, 0, 4.876872, 0.492057, 0, 4.811573, 0.552393, 0, 4.860352, 0.578841, 0, 4.876872, 0.492057, 0, 4.860352, 0.578841, 0, 4.876872, 0.492057, 0, 4.906966, 0.505729, 0, 4.876872, 0.492057, 0, 4.860352, 0.578841, 0, 4.906966, 0.505729, 0, 4.860352, 0.578841, 0, 4.906966, 0.505729, 0, 4.915869, 0.594710, 0, 4.906966, 0.505729, 0, 4.860352, 0.578841, 0, 4.915869, 0.594710, 0, 4.915869, 0.594710, 0, 4.906966, 0.505729, 0, 4.941064, 0.513932, 0, 4.906966, 0.505729, 0, 4.915869, 0.594710, 0, 4.941064, 0.513932, 0, 4.915869, 0.594710, 0, 4.941064, 0.513932, 0, 4.978125, 0.600000, 0, 4.941064, 0.513932, 0, 4.915869, 0.594710, 0, 4.978125, 0.600000, 0, 4.978125, 0.600000, 0, 4.941064, 0.513932, 0, 4.979167, 0.516667, 0, 4.941064, 0.513932, 0, 4.978125, 0.600000, 0, 4.979167, 0.516667, 0, 4.978125, 0.600000, 0, 4.979167, 0.516667, 0, 5.012842, 0.513835, 0, 4.979167, 0.516667, 0, 4.978125, 0.600000, 0, 5.012842, 0.513835, 0, 4.978125, 0.600000, 0, 5.012842, 0.513835, 0, 5.033903, 0.595272, 0, 5.012842, 0.513835, 0, 4.978125, 0.600000, 0, 5.033903, 0.595272, 0, 5.033903, 0.595272, 0, 5.012842, 0.513835, 0, 5.043034, 0.505339, 0, 5.012842, 0.513835, 0, 5.033903, 0.595272, 0, 5.043034, 0.505339, 0, 5.033903, 0.595272, 0, 5.043034, 0.505339, 0, 5.083529, 0.581087, 0, 5.043034, 0.505339, 0, 5.033903, 0.595272, 0, 5.083529, 0.581087, 0, 5.043034, 0.505339, 0, 5.069743, 0.491178, 0, 5.083529, 0.581087, 0, 5.069743, 0.491178, 0, 5.043034, 0.505339, 0, 5.083529, 0.581087, 0, 5.083529, 0.581087, 0, 5.069743, 0.491178, 0, 5.127002, 0.557446, 0, 5.069743, 0.491178, 0, 5.083529, 0.581087, 0, 5.127002, 0.557446, 0, 5.069743, 0.491178, 0, 5.092969, 0.471354, 0, 5.127002, 0.557446, 0, 5.092969, 0.471354, 0, 5.069743, 0.491178, 0, 5.127002, 0.557446, 0, 5.127002, 0.557446, 0, 5.092969, 0.471354, 0, 5.164323, 0.524349, 0, 5.092969, 0.471354, 0, 5.127002, 0.557446, 0, 5.164323, 0.524349, 0, 5.092969, 0.471354, 0, 5.111833, 0.446777, 0, 5.164323, 0.524349, 0, 5.111833, 0.446777, 0, 5.092969, 0.471354, 0, 5.164323, 0.524349, 0, 5.164323, 0.524349, 0, 5.111833, 0.446777, 0, 5.194287, 0.483130, 0, 5.111833, 0.446777, 0, 5.164323, 0.524349, 0, 5.194287, 0.483130, 0, 5.111833, 0.446777, 0, 5.125456, 0.418359, 0, 5.194287, 0.483130, 0, 5.125456, 0.418359, 0, 5.111833, 0.446777, 0, 5.194287, 0.483130, 0, 5.194287, 0.483130, 0, 5.125456, 0.418359, 0, 5.215690, 0.435124, 0, 5.125456, 0.418359, 0, 5.194287, 0.483130, 0, 5.215690, 0.435124, 0, 5.125456, 0.418359, 0, 5.133838, 0.386100, 0, 5.215690, 0.435124, 0, 5.133838, 0.386100, 0, 5.125456, 0.418359, 0, 5.215690, 0.435124, 0, 5.215690, 0.435124, 0, 5.133838, 0.386100, 0, 5.228532, 0.380330, 0, 5.133838, 0.386100, 0, 5.215690, 0.435124, 0, 5.228532, 0.380330, 0, 5.133838, 0.386100, 0, 5.136980, 0.350000, 0, 5.228532, 0.380330, 0, 5.136980, 0.350000, 0, 5.133838, 0.386100, 0, 5.228532, 0.380330, 0, 5.228532, 0.380330, 0, 5.136980, 0.350000, 0, 5.232812, 0.318750, 0, 5.136980, 0.350000, 0, 5.228532, 0.380330, 0, 5.232812, 0.318750, 0, 5.232812, 0.266667, 0, 5.232812, 0.318750, 0, 5.136980, 0.350000, 0, 5.232812, 0.318750, 0, 5.232812, 0.266667, 0, 5.136980, 0.350000, 0, 5.136980, 0.350000, 0, 4.792188, 0.266667, 0, 5.232812, 0.266667, 0, 4.792188, 0.266667, 0, 5.136980, 0.350000, 0, 5.232812, 0.266667, 0, 5.136980, 0.350000, 0, 4.795312, 0.349479, 0, 4.792188, 0.266667, 0, 4.795312, 0.349479, 0, 5.136980, 0.350000, 0, 4.792188, 0.266667, 0, 4.692188, 0.286458, 0, 4.792188, 0.266667, 0, 4.795312, 0.349479, 0, 4.792188, 0.266667, 0, 4.692188, 0.286458, 0, 4.795312, 0.349479, 0, 4.692188, 0.286458, 0, 4.697282, 0.220280, 0, 4.792188, 0.266667, 0, 4.697282, 0.220280, 0, 4.692188, 0.286458, 0, 4.792188, 0.266667, 0, 4.792188, 0.266667, 0, 4.697282, 0.220280, 0, 4.798259, 0.220744, 0, 4.697282, 0.220280, 0, 4.792188, 0.266667, 0, 4.798259, 0.220744, 0, 4.697282, 0.220280, 0, 4.712565, 0.161328, 0, 4.798259, 0.220744, 0, 4.712565, 0.161328, 0, 4.697282, 0.220280, 0, 4.798259, 0.220744, 0, 4.798259, 0.220744, 0, 4.712565, 0.161328, 0, 4.810222, 0.180632, 0, 4.712565, 0.161328, 0, 4.798259, 0.220744, 0, 4.810222, 0.180632, 0, 4.712565, 0.161328, 0, 4.738037, 0.109603, 0, 4.810222, 0.180632, 0, 4.738037, 0.109603, 0, 4.712565, 0.161328, 0, 4.810222, 0.180632, 0, 4.810222, 0.180632, 0, 4.738037, 0.109603, 0, 4.828076, 0.146330, 0, 4.738037, 0.109603, 0, 4.810222, 0.180632, 0, 4.828076, 0.146330, 0, 4.738037, 0.109603, 0, 4.773698, 0.065104, 0, 4.828076, 0.146330, 0, 4.773698, 0.065104, 0, 4.738037, 0.109603, 0, 4.828076, 0.146330, 0, 4.828076, 0.146330, 0, 4.773698, 0.065104, 0, 4.851823, 0.117839, 0, 4.773698, 0.065104, 0, 4.828076, 0.146330, 0, 4.851823, 0.117839, 0, 4.773698, 0.065104, 0, 4.818083, 0.029329, 0, 4.851823, 0.117839, 0, 4.818083, 0.029329, 0, 4.773698, 0.065104, 0, 4.851823, 0.117839, 0, 4.851823, 0.117839, 0, 4.818083, 0.029329, 0, 4.881136, 0.095451, 0, 4.818083, 0.029329, 0, 4.851823, 0.117839, 0, 4.881136, 0.095451, 0, 4.881136, 0.095451, 0, 4.818083, 0.029329, 0, 4.869727, 0.003776, 0, 4.818083, 0.029329, 0, 4.881136, 0.095451, 0, 4.869727, 0.003776, 0, 4.915690, 0.079460, 0, 4.881136, 0.095451, 0, 4.869727, 0.003776, 0, 4.881136, 0.095451, 0, 4.915690, 0.079460, 0, 4.869727, 0.003776, 0, 4.915690, 0.079460, 0, 4.869727, 0.003776, 0, 4.928630, -0.011556, 0, 4.869727, 0.003776, 0, 4.915690, 0.079460, 0, 4.928630, -0.011556, 0, 4.955485, 0.069865, 0, 4.915690, 0.079460, 0, 4.928630, -0.011556, 0, 4.915690, 0.079460, 0, 4.955485, 0.069865, 0, 4.928630, -0.011556, 0, 4.955485, 0.069865, 0, 4.928630, -0.011556, 0, 4.994792, -0.016667, 0, 4.928630, -0.011556, 0, 4.955485, 0.069865, 0, 4.994792, -0.016667, 0, 5.000521, 0.066667, 0, 4.955485, 0.069865, 0, 4.994792, -0.016667, 0, 4.955485, 0.069865, 0, 5.000521, 0.066667, 0, 4.994792, -0.016667, 0, 5.000521, 0.066667, 0, 4.994792, -0.016667, 0, 5.022559, -0.015869, 0, 4.994792, -0.016667, 0, 5.000521, 0.066667, 0, 5.022559, -0.015869, 0, 5.000521, 0.066667, 0, 5.022559, -0.015869, 0, 5.027914, 0.067448, 0, 5.022559, -0.015869, 0, 5.000521, 0.066667, 0, 5.027914, 0.067448, 0, 5.027914, 0.067448, 0, 5.022559, -0.015869, 0, 5.050130, -0.013477, 0, 5.022559, -0.015869, 0, 5.027914, 0.067448, 0, 5.050130, -0.013477, 0, 5.027914, 0.067448, 0, 5.050130, -0.013477, 0, 5.054883, 0.069792, 0, 5.050130, -0.013477, 0, 5.027914, 0.067448, 0, 5.054883, 0.069792, 0, 5.054883, 0.069792, 0, 5.050130, -0.013477, 0, 5.077507, -0.009489, 0, 5.050130, -0.013477, 0, 5.054883, 0.069792, 0, 5.077507, -0.009489, 0, 5.054883, 0.069792, 0, 5.077507, -0.009489, 0, 5.081429, 0.073698, 0, 5.077507, -0.009489, 0, 5.054883, 0.069792, 0, 5.081429, 0.073698, 0, 5.081429, 0.073698, 0, 5.077507, -0.009489, 0, 5.104688, -0.003906, 0, 5.077507, -0.009489, 0, 5.081429, 0.073698, 0, 5.104688, -0.003906, 0, 5.081429, 0.073698, 0, 5.104688, -0.003906, 0, 5.107552, 0.079167, 0, 5.104688, -0.003906, 0, 5.081429, 0.073698, 0, 5.107552, 0.079167, 0, 5.107552, 0.079167, 0, 5.104688, -0.003906, 0, 5.131608, 0.003206, 0, 5.104688, -0.003906, 0, 5.107552, 0.079167, 0, 5.131608, 0.003206, 0, 5.133415, 0.086198, 0, 5.107552, 0.079167, 0, 5.131608, 0.003206, 0, 5.107552, 0.079167, 0, 5.133415, 0.086198, 0, 5.131608, 0.003206, 0, 5.158203, 0.011784, 0, 5.133415, 0.086198, 0, 5.131608, 0.003206, 0, 5.133415, 0.086198, 0, 5.158203, 0.011784, 0, 5.131608, 0.003206, 0, 5.133415, 0.086198, 0, 5.158203, 0.011784, 0, 5.184473, 0.021826, 0, 5.158203, 0.011784, 0, 5.133415, 0.086198, 0, 5.184473, 0.021826, 0, 5.159180, 0.094792, 0, 5.133415, 0.086198, 0, 5.184473, 0.021826, 0, 5.133415, 0.086198, 0, 5.159180, 0.094792, 0, 5.184473, 0.021826, 0, 5.159180, 0.094792, 0, 5.184473, 0.021826, 0, 5.210417, 0.033333, 0, 5.184473, 0.021826, 0, 5.159180, 0.094792, 0, 5.210417, 0.033333, 0, 5.184847, 0.104948, 0, 5.159180, 0.094792, 0, 5.210417, 0.033333, 0, 5.159180, 0.094792, 0, 5.184847, 0.104948, 0, 5.210417, 0.033333, 0, 5.184847, 0.104948, 0, 5.210417, 0.033333, 0, 5.210417, 0.116667, 0, 5.210417, 0.033333, 0, 5.184847, 0.104948, 0, 5.210417, 0.116667, 0, 5.136980, 0.350000, 0, 5.133838, 0.386100, 0, 5.136980, 0.350000, 0, 5.136980, 0.350000, 0, 5.133838, 0.386100, 0, 5.133838, 0.386100, 0, 5.133838, 0.386100, 0, 5.125456, 0.418359, 0, 5.133838, 0.386100, 0, 5.133838, 0.386100, 0, 5.125456, 0.418359, 0, 5.125456, 0.418359, 0, 5.125456, 0.418359, 0, 5.111833, 0.446777, 0, 5.125456, 0.418359, 0, 5.125456, 0.418359, 0, 5.111833, 0.446777, 0, 5.111833, 0.446777, 0, 5.111833, 0.446777, 0, 5.092969, 0.471354, 0, 5.111833, 0.446777, 0, 5.111833, 0.446777, 0, 5.092969, 0.471354, 0, 5.092969, 0.471354, 0, 5.092969, 0.471354, 0, 5.069743, 0.491178, 0, 5.092969, 0.471354, 0, 5.092969, 0.471354, 0, 5.069743, 0.491178, 0, 5.069743, 0.491178, 0, 5.069743, 0.491178, 0, 5.043034, 0.505339, 0, 5.069743, 0.491178, 0, 5.069743, 0.491178, 0, 5.043034, 0.505339, 0, 5.043034, 0.505339, 0, 5.043034, 0.505339, 0, 5.012842, 0.513835, 0, 5.043034, 0.505339, 0, 5.043034, 0.505339, 0, 5.012842, 0.513835, 0, 5.012842, 0.513835, 0, 5.012842, 0.513835, 0, 4.979167, 0.516667, 0, 5.012842, 0.513835, 0, 5.012842, 0.513835, 0, 4.979167, 0.516667, 0, 4.979167, 0.516667, 0, 4.979167, 0.516667, 0, 4.941064, 0.513932, 0, 4.979167, 0.516667, 0, 4.979167, 0.516667, 0, 4.941064, 0.513932, 0, 4.941064, 0.513932, 0, 4.941064, 0.513932, 0, 4.906966, 0.505729, 0, 4.941064, 0.513932, 0, 4.941064, 0.513932, 0, 4.906966, 0.505729, 0, 4.906966, 0.505729, 0, 4.906966, 0.505729, 0, 4.876872, 0.492057, 0, 4.906966, 0.505729, 0, 4.906966, 0.505729, 0, 4.876872, 0.492057, 0, 4.876872, 0.492057, 0, 4.876872, 0.492057, 0, 4.850781, 0.472917, 0, 4.876872, 0.492057, 0, 4.876872, 0.492057, 0, 4.850781, 0.472917, 0, 4.850781, 0.472917, 0, 4.850781, 0.472917, 0, 4.829248, 0.448796, 0, 4.850781, 0.472917, 0, 4.850781, 0.472917, 0, 4.829248, 0.448796, 0, 4.829248, 0.448796, 0, 4.829248, 0.448796, 0, 4.812826, 0.420182, 0, 4.829248, 0.448796, 0, 4.829248, 0.448796, 0, 4.812826, 0.420182, 0, 4.812826, 0.420182, 0, 4.812826, 0.420182, 0, 4.801514, 0.387077, 0, 4.812826, 0.420182, 0, 4.812826, 0.420182, 0, 4.801514, 0.387077, 0, 4.801514, 0.387077, 0, 4.801514, 0.387077, 0, 4.795312, 0.349479, 0, 4.801514, 0.387077, 0, 4.801514, 0.387077, 0, 4.795312, 0.349479, 0, 4.795312, 0.349479, 0, 5.721875, 0.500000, 0, 5.713623, 0.504012, 0, 5.721875, 0.500000, 0, 5.721875, 0.500000, 0, 5.713623, 0.504012, 0, 5.713623, 0.504012, 0, 5.713623, 0.504012, 0, 5.705013, 0.507454, 0, 5.713623, 0.504012, 0, 5.713623, 0.504012, 0, 5.705013, 0.507454, 0, 5.705013, 0.507454, 0, 5.705013, 0.507454, 0, 5.696045, 0.510327, 0, 5.705013, 0.507454, 0, 5.705013, 0.507454, 0, 5.696045, 0.510327, 0, 5.696045, 0.510327, 0, 5.696045, 0.510327, 0, 5.686719, 0.512630, 0, 5.696045, 0.510327, 0, 5.696045, 0.510327, 0, 5.686719, 0.512630, 0, 5.686719, 0.512630, 0, 5.686719, 0.512630, 0, 5.676970, 0.514396, 0, 5.686719, 0.512630, 0, 5.686719, 0.512630, 0, 5.676970, 0.514396, 0, 5.676970, 0.514396, 0, 5.676970, 0.514396, 0, 5.666732, 0.515658, 0, 5.676970, 0.514396, 0, 5.676970, 0.514396, 0, 5.666732, 0.515658, 0, 5.666732, 0.515658, 0, 5.666732, 0.515658, 0, 5.656006, 0.516414, 0, 5.666732, 0.515658, 0, 5.666732, 0.515658, 0, 5.656006, 0.516414, 0, 5.656006, 0.516414, 0, 5.656006, 0.516414, 0, 5.644792, 0.516667, 0, 5.656006, 0.516414, 0, 5.656006, 0.516414, 0, 5.644792, 0.516667, 0, 5.644792, 0.516667, 0, 5.644792, 0.516667, 0, 5.606527, 0.513354, 0, 5.644792, 0.516667, 0, 5.644792, 0.516667, 0, 5.606527, 0.513354, 0, 5.606527, 0.513354, 0, 5.606527, 0.513354, 0, 5.572982, 0.503418, 0, 5.606527, 0.513354, 0, 5.606527, 0.513354, 0, 5.572982, 0.503418, 0, 5.572982, 0.503418, 0, 5.572982, 0.503418, 0, 5.544157, 0.486857, 0, 5.572982, 0.503418, 0, 5.572982, 0.503418, 0, 5.544157, 0.486857, 0, 5.544157, 0.486857, 0, 5.544157, 0.486857, 0, 5.520052, 0.463672, 0, 5.544157, 0.486857, 0, 5.544157, 0.486857, 0, 5.520052, 0.463672, 0, 5.520052, 0.463672, 0, 5.520052, 0.463672, 0, 5.501026, 0.434302, 0, 5.520052, 0.463672, 0, 5.520052, 0.463672, 0, 5.501026, 0.434302, 0, 5.501026, 0.434302, 0, 5.501026, 0.434302, 0, 5.487435, 0.399186, 0, 5.501026, 0.434302, 0, 5.501026, 0.434302, 0, 5.487435, 0.399186, 0, 5.487435, 0.399186, 0, 5.487435, 0.399186, 0, 5.479280, 0.358325, 0, 5.487435, 0.399186, 0, 5.487435, 0.399186, 0, 5.479280, 0.358325, 0, 5.479280, 0.358325, 0, 5.479280, 0.358325, 0, 5.476563, 0.311719, 0, 5.479280, 0.358325, 0, 5.479280, 0.358325, 0, 5.476563, 0.311719, 0, 5.476563, 0.311719, 0, 5.476563, 0.311719, 0, 5.476563, 0, 0, 5.476563, 0.311719, 0, 5.476563, 0.311719, 0, 5.476563, 0, 0, 5.476563, 0, 0, 5.476563, 0, 0, 5.380208, 0, 0, 5.476563, 0, 0, 5.476563, 0, 0, 5.380208, 0, 0, 5.380208, 0, 0, 5.380208, 0, 0, 5.380208, 0.583333, 0, 5.380208, 0, 0, 5.380208, 0, 0, 5.380208, 0.583333, 0, 5.380208, 0.583333, 0, 5.380208, 0.583333, 0, 5.476563, 0.583333, 0, 5.380208, 0.583333, 0, 5.380208, 0.583333, 0, 5.476563, 0.583333, 0, 5.476563, 0.583333, 0, 5.476563, 0.583333, 0, 5.476563, 0.500000, 0, 5.476563, 0.583333, 0, 5.476563, 0.583333, 0, 5.476563, 0.500000, 0, 5.476563, 0.500000, 0, 5.476563, 0.500000, 0, 5.492806, 0.523755, 0, 5.476563, 0.500000, 0, 5.476563, 0.500000, 0, 5.492806, 0.523755, 0, 5.492806, 0.523755, 0, 5.492806, 0.523755, 0, 5.511328, 0.544238, 0, 5.492806, 0.523755, 0, 5.492806, 0.523755, 0, 5.511328, 0.544238, 0, 5.511328, 0.544238, 0, 5.511328, 0.544238, 0, 5.532129, 0.561450, 0, 5.511328, 0.544238, 0, 5.511328, 0.544238, 0, 5.532129, 0.561450, 0, 5.532129, 0.561450, 0, 5.532129, 0.561450, 0, 5.555208, 0.575391, 0, 5.532129, 0.561450, 0, 5.532129, 0.561450, 0, 5.555208, 0.575391, 0, 5.555208, 0.575391, 0, 5.555208, 0.575391, 0, 5.580729, 0.586157, 0, 5.555208, 0.575391, 0, 5.555208, 0.575391, 0, 5.580729, 0.586157, 0, 5.580729, 0.586157, 0, 5.580729, 0.586157, 0, 5.608854, 0.593848, 0, 5.580729, 0.586157, 0, 5.580729, 0.586157, 0, 5.608854, 0.593848, 0, 5.608854, 0.593848, 0, 5.608854, 0.593848, 0, 5.639583, 0.598462, 0, 5.608854, 0.593848, 0, 5.608854, 0.593848, 0, 5.639583, 0.598462, 0, 5.639583, 0.598462, 0, 5.639583, 0.598462, 0, 5.672917, 0.600000, 0, 5.639583, 0.598462, 0, 5.639583, 0.598462, 0, 5.672917, 0.600000, 0, 5.672917, 0.600000, 0, 5.672917, 0.600000, 0, 5.677995, 0.600000, 0, 5.672917, 0.600000, 0, 5.672917, 0.600000, 0, 5.677995, 0.600000, 0, 5.677995, 0.600000, 0, 5.677995, 0.600000, 0, 5.683333, 0.600000, 0, 5.677995, 0.600000, 0, 5.677995, 0.600000, 0, 5.683333, 0.600000, 0, 5.683333, 0.600000, 0, 5.683333, 0.600000, 0, 5.688932, 0.600000, 0, 5.683333, 0.600000, 0, 5.683333, 0.600000, 0, 5.688932, 0.600000, 0, 5.688932, 0.600000, 0, 5.688932, 0.600000, 0, 5.694792, 0.600000, 0, 5.688932, 0.600000, 0, 5.688932, 0.600000, 0, 5.694792, 0.600000, 0, 5.694792, 0.600000, 0, 5.694792, 0.600000, 0, 5.700944, 0.600000, 0, 5.694792, 0.600000, 0, 5.694792, 0.600000, 0, 5.700944, 0.600000, 0, 5.700944, 0.600000, 0, 5.700944, 0.600000, 0, 5.707422, 0.600000, 0, 5.700944, 0.600000, 0, 5.700944, 0.600000, 0, 5.707422, 0.600000, 0, 5.707422, 0.600000, 0, 5.707422, 0.600000, 0, 5.714225, 0.600000, 0, 5.707422, 0.600000, 0, 5.707422, 0.600000, 0, 5.714225, 0.600000, 0, 5.714225, 0.600000, 0, 5.714225, 0.600000, 0, 5.721354, 0.600000, 0, 5.714225, 0.600000, 0, 5.714225, 0.600000, 0, 5.721354, 0.600000, 0, 5.721354, 0.600000, 0, 5.476563, 0.583333, 0, 5.380208, 0.583333, 0, 5.476563, 0.500000, 0, 5.380208, 0.583333, 0, 5.476563, 0.583333, 0, 5.476563, 0.500000, 0, 5.487435, 0.399186, 0, 5.476563, 0.500000, 0, 5.380208, 0.583333, 0, 5.476563, 0.500000, 0, 5.487435, 0.399186, 0, 5.380208, 0.583333, 0, 5.476563, 0.500000, 0, 5.487435, 0.399186, 0, 5.501026, 0.434302, 0, 5.487435, 0.399186, 0, 5.476563, 0.500000, 0, 5.501026, 0.434302, 0, 5.476563, 0.500000, 0, 5.501026, 0.434302, 0, 5.520052, 0.463672, 0, 5.501026, 0.434302, 0, 5.476563, 0.500000, 0, 5.520052, 0.463672, 0, 5.492806, 0.523755, 0, 5.476563, 0.500000, 0, 5.520052, 0.463672, 0, 5.476563, 0.500000, 0, 5.492806, 0.523755, 0, 5.520052, 0.463672, 0, 5.492806, 0.523755, 0, 5.520052, 0.463672, 0, 5.544157, 0.486857, 0, 5.520052, 0.463672, 0, 5.492806, 0.523755, 0, 5.544157, 0.486857, 0, 5.511328, 0.544238, 0, 5.492806, 0.523755, 0, 5.544157, 0.486857, 0, 5.492806, 0.523755, 0, 5.511328, 0.544238, 0, 5.544157, 0.486857, 0, 5.511328, 0.544238, 0, 5.544157, 0.486857, 0, 5.572982, 0.503418, 0, 5.544157, 0.486857, 0, 5.511328, 0.544238, 0, 5.572982, 0.503418, 0, 5.532129, 0.561450, 0, 5.511328, 0.544238, 0, 5.572982, 0.503418, 0, 5.511328, 0.544238, 0, 5.532129, 0.561450, 0, 5.572982, 0.503418, 0, 5.555208, 0.575391, 0, 5.532129, 0.561450, 0, 5.572982, 0.503418, 0, 5.532129, 0.561450, 0, 5.555208, 0.575391, 0, 5.572982, 0.503418, 0, 5.555208, 0.575391, 0, 5.572982, 0.503418, 0, 5.606527, 0.513354, 0, 5.572982, 0.503418, 0, 5.555208, 0.575391, 0, 5.606527, 0.513354, 0, 5.580729, 0.586157, 0, 5.555208, 0.575391, 0, 5.606527, 0.513354, 0, 5.555208, 0.575391, 0, 5.580729, 0.586157, 0, 5.606527, 0.513354, 0, 5.580729, 0.586157, 0, 5.606527, 0.513354, 0, 5.608854, 0.593848, 0, 5.606527, 0.513354, 0, 5.580729, 0.586157, 0, 5.608854, 0.593848, 0, 5.608854, 0.593848, 0, 5.606527, 0.513354, 0, 5.644792, 0.516667, 0, 5.606527, 0.513354, 0, 5.608854, 0.593848, 0, 5.644792, 0.516667, 0, 5.639583, 0.598462, 0, 5.608854, 0.593848, 0, 5.644792, 0.516667, 0, 5.608854, 0.593848, 0, 5.639583, 0.598462, 0, 5.644792, 0.516667, 0, 5.639583, 0.598462, 0, 5.644792, 0.516667, 0, 5.656006, 0.516414, 0, 5.644792, 0.516667, 0, 5.639583, 0.598462, 0, 5.656006, 0.516414, 0, 5.672917, 0.600000, 0, 5.639583, 0.598462, 0, 5.656006, 0.516414, 0, 5.639583, 0.598462, 0, 5.672917, 0.600000, 0, 5.656006, 0.516414, 0, 5.656006, 0.516414, 0, 5.666732, 0.515658, 0, 5.672917, 0.600000, 0, 5.666732, 0.515658, 0, 5.656006, 0.516414, 0, 5.672917, 0.600000, 0, 5.677995, 0.600000, 0, 5.672917, 0.600000, 0, 5.666732, 0.515658, 0, 5.672917, 0.600000, 0, 5.677995, 0.600000, 0, 5.666732, 0.515658, 0, 5.677995, 0.600000, 0, 5.666732, 0.515658, 0, 5.676970, 0.514396, 0, 5.666732, 0.515658, 0, 5.677995, 0.600000, 0, 5.676970, 0.514396, 0, 5.683333, 0.600000, 0, 5.677995, 0.600000, 0, 5.676970, 0.514396, 0, 5.677995, 0.600000, 0, 5.683333, 0.600000, 0, 5.676970, 0.514396, 0, 5.676970, 0.514396, 0, 5.688932, 0.600000, 0, 5.683333, 0.600000, 0, 5.688932, 0.600000, 0, 5.676970, 0.514396, 0, 5.683333, 0.600000, 0, 5.688932, 0.600000, 0, 5.676970, 0.514396, 0, 5.686719, 0.512630, 0, 5.676970, 0.514396, 0, 5.688932, 0.600000, 0, 5.686719, 0.512630, 0, 5.694792, 0.600000, 0, 5.688932, 0.600000, 0, 5.686719, 0.512630, 0, 5.688932, 0.600000, 0, 5.694792, 0.600000, 0, 5.686719, 0.512630, 0, 5.686719, 0.512630, 0, 5.700944, 0.600000, 0, 5.694792, 0.600000, 0, 5.700944, 0.600000, 0, 5.686719, 0.512630, 0, 5.694792, 0.600000, 0, 5.700944, 0.600000, 0, 5.686719, 0.512630, 0, 5.696045, 0.510327, 0, 5.686719, 0.512630, 0, 5.700944, 0.600000, 0, 5.696045, 0.510327, 0, 5.707422, 0.600000, 0, 5.700944, 0.600000, 0, 5.696045, 0.510327, 0, 5.700944, 0.600000, 0, 5.707422, 0.600000, 0, 5.696045, 0.510327, 0, 5.714225, 0.600000, 0, 5.707422, 0.600000, 0, 5.696045, 0.510327, 0, 5.707422, 0.600000, 0, 5.714225, 0.600000, 0, 5.696045, 0.510327, 0, 5.705013, 0.507454, 0, 5.714225, 0.600000, 0, 5.696045, 0.510327, 0, 5.714225, 0.600000, 0, 5.705013, 0.507454, 0, 5.696045, 0.510327, 0, 5.721354, 0.600000, 0, 5.714225, 0.600000, 0, 5.705013, 0.507454, 0, 5.714225, 0.600000, 0, 5.721354, 0.600000, 0, 5.705013, 0.507454, 0, 5.713623, 0.504012, 0, 5.721354, 0.600000, 0, 5.705013, 0.507454, 0, 5.721354, 0.600000, 0, 5.713623, 0.504012, 0, 5.705013, 0.507454, 0, 5.721875, 0.500000, 0, 5.721354, 0.600000, 0, 5.713623, 0.504012, 0, 5.721354, 0.600000, 0, 5.721875, 0.500000, 0, 5.713623, 0.504012, 0, 5.487435, 0.399186, 0, 5.380208, 0.583333, 0, 5.479280, 0.358325, 0, 5.380208, 0.583333, 0, 5.487435, 0.399186, 0, 5.479280, 0.358325, 0, 5.476563, 0.311719, 0, 5.479280, 0.358325, 0, 5.380208, 0.583333, 0, 5.479280, 0.358325, 0, 5.476563, 0.311719, 0, 5.380208, 0.583333, 0, 5.476563, 0.311719, 0, 5.380208, 0.583333, 0, 5.380208, 0, 0, 5.380208, 0.583333, 0, 5.476563, 0.311719, 0, 5.380208, 0, 0, 5.380208, 0, 0, 5.476563, 0, 0, 5.476563, 0.311719, 0, 5.476563, 0, 0, 5.380208, 0, 0, 5.476563, 0.311719, 0 };
    Coordinate31.point = m_point;
    Coordinate31.point_length = sizeof(m_point) / sizeof(float);
    }
    Normal36.m_parent = (X3dNode *)&Text_69;
    {
    static float m_vector[] = { 0.810174, 0.586189, 0, 0.810174, 0.586189, 0, 0.810174, 0.586189, 0, 0.810174, 0.586189, 0, 0.810174, 0.586189, 0, 0.810174, 0.586189, 0, 0.770794, 0.637084, 0, 0.770794, 0.637084, 0, 0.770794, 0.637084, 0, 0.770794, 0.637084, 0, 0.770794, 0.637084, 0, 0.770794, 0.637084, 0, 0.731791, 0.681529, 0, 0.731791, 0.681529, 0, 0.731791, 0.681529, 0, 0.731791, 0.681529, 0, 0.731791, 0.681529, 0, 0.731791, 0.681529, 0, 0.692455, 0.721461, 0, 0.692455, 0.721461, 0, 0.692455, 0.721461, 0, 0.692455, 0.721461, 0, 0.692455, 0.721461, 0, 0.692455, 0.721461, 0, 0.652835, 0.757500, 0, 0.652835, 0.757500, 0, 0.652835, 0.757500, 0, 0.652835, 0.757500, 0, 0.652835, 0.757500, 0, 0.652835, 0.757500, 0, 0.612927, 0.790140, 0, 0.612927, 0.790140, 0, 0.612927, 0.790140, 0, 0.612927, 0.790140, 0, 0.612927, 0.790140, 0, 0.612927, 0.790140, 0, 0.571623, 0.820516, 0, 0.571623, 0.820516, 0, 0.571623, 0.820516, 0, 0.571623, 0.820516, 0, 0.571623, 0.820516, 0, 0.571623, 0.820516, 0, 0.527533, 0.849534, 0, 0.527533, 0.849534, 0, 0.527533, 0.849534, 0, 0.527533, 0.849534, 0, 0.527533, 0.849534, 0, 0.527533, 0.849534, 0, 0.484807, 0.874621, 0, 0.484807, 0.874621, 0, 0.484807, 0.874621, 0, 0.484807, 0.874621, 0, 0.484807, 0.874621, 0, 0.484807, 0.874621, 0, 0.448695, 0.893685, 0, 0.448695, 0.893685, 0, 0.448695, 0.893685, 0, 0.448695, 0.893685, 0, 0.448695, 0.893685, 0, 0.448695, 0.893685, 0, 0.420099, 0.907478, 0, 0.420099, 0.907478, 0, 0.420099, 0.907478, 0, 0.420099, 0.907478, 0, 0.420099, 0.907478, 0, 0.420099, 0.907478, 0, 0.400111, 0.916467, 0, 0.400111, 0.916467, 0, 0.400111, 0.916467, 0, 0.400111, 0.916467, 0, 0.400111, 0.916467, 0, 0.400111, 0.916467, 0, 0.390829, 0.920463, 0, 0.390829, 0.920463, 0, 0.390829, 0.920463, 0, 0.390829, 0.920463, 0, 0.390829, 0.920463, 0, 0.390829, 0.920463, 0, 0.395467, 0.918480, 0, 0.395467, 0.918480, 0, 0.395467, 0.918480, 0, 0.395467, 0.918480, 0, 0.395467, 0.918480, 0, 0.395467, 0.918480, 0, 0.418291, 0.908313, 0, 0.418291, 0.908313, 0, 0.418291, 0.908313, 0, 0.418291, 0.908313, 0, 0.418291, 0.908313, 0, 0.418291, 0.908313, 0, 0.465873, 0.884852, 0, 0.465873, 0.884852, 0, 0.465873, 0.884852, 0, 0.465873, 0.884852, 0, 0.465873, 0.884852, 0, 0.465873, 0.884852, 0, 0.538429, 0.842671, 0, 0.538429, 0.842671, 0, 0.538429, 0.842671, 0, 0.538429, 0.842671, 0, 0.538429, 0.842671, 0, 0.538429, 0.842671, 0, 0.628863, 0.777516, 0, 0.628863, 0.777516, 0, 0.628863, 0.777516, 0, 0.628863, 0.777516, 0, 0.628863, 0.777516, 0, 0.628863, 0.777516, 0, 0.728015, 0.685562, 0, 0.728015, 0.685562, 0, 0.728015, 0.685562, 0, 0.728015, 0.685562, 0, 0.728015, 0.685562, 0, 0.728015, 0.685562, 0, 0.820081, 0.572248, 0, 0.820081, 0.572248, 0, 0.820081, 0.572248, 0, 0.820081, 0.572248, 0, 0.820081, 0.572248, 0, 0.820081, 0.572248, 0, 0.889510, 0.456917, 0, 0.889510, 0.456917, 0, 0.889510, 0.456917, 0, 0.889510, 0.456917, 0, 0.889510, 0.456917, 0, 0.889510, 0.456917, 0, 0.932730, 0.360575, 0, 0.932730, 0.360575, 0, 0.932730, 0.360575, 0, 0.932730, 0.360575, 0, 0.932730, 0.360575, 0, 0.932730, 0.360575, 0, 0.956376, 0.292139, 0, 0.956376, 0.292139, 0, 0.956376, 0.292139, 0, 0.956376, 0.292139, 0, 0.956376, 0.292139, 0, 0.956376, 0.292139, 0, 0.967884, 0.251398, 0, 0.967884, 0.251398, 0, 0.967884, 0.251398, 0, 0.967884, 0.251398, 0, 0.967884, 0.251398, 0, 0.967884, 0.251398, 0, 0.971712, 0.236170, 0, 0.971712, 0.236170, 0, 0.971712, 0.236170, 0, 0.971712, 0.236170, 0, 0.971712, 0.236170, 0, 0.971712, 0.236170, 0, 0.970095, 0.242726, 0, 0.970095, 0.242726, 0, 0.970095, 0.242726, 0, 0.970095, 0.242726, 0, 0.970095, 0.242726, 0, 0.970095, 0.242726, 0, 0.963979, 0.265980, 0, 0.963979, 0.265980, 0, 0.963979, 0.265980, 0, 0.963979, 0.265980, 0, 0.963979, 0.265980, 0, 0.963979, 0.265980, 0, 0.953140, 0.302530, 0, 0.953140, 0.302530, 0, 0.953140, 0.302530, 0, 0.953140, 0.302530, 0, 0.953140, 0.302530, 0, 0.953140, 0.302530, 0, 0.937134, 0.348969, 0, 0.937134, 0.348969, 0, 0.937134, 0.348969, 0, 0.937134, 0.348969, 0, 0.937134, 0.348969, 0, 0.937134, 0.348969, 0, 0.915315, 0.402739, 0, 0.915315, 0.402739, 0, 0.915315, 0.402739, 0, 0.915315, 0.402739, 0, 0.915315, 0.402739, 0, 0.915315, 0.402739, 0, 0.886682, 0.462381, 0, 0.886682, 0.462381, 0, 0.886682, 0.462381, 0, 0.886682, 0.462381, 0, 0.886682, 0.462381, 0, 0.886682, 0.462381, 0, 0.999943, 0.010679, 0, 0.999943, 0.010679, 0, 0.999943, 0.010679, 0, 0.999943, 0.010679, 0, 0.999943, 0.010679, 0, 0.999943, 0.010679, 0, 0.995523, -0.094516, 0, 0.995523, -0.094516, 0, 0.995523, -0.094516, 0, 0.995523, -0.094516, 0, 0.995523, -0.094516, 0, 0.995523, -0.094516, 0, 0.982234, -0.187661, 0, 0.982234, -0.187661, 0, 0.982234, -0.187661, 0, 0.982234, -0.187661, 0, 0.982234, -0.187661, 0, 0.982234, -0.187661, 0, 0.962675, -0.270661, 0, 0.962675, -0.270661, 0, 0.962675, -0.270661, 0, 0.962675, -0.270661, 0, 0.962675, -0.270661, 0, 0.962675, -0.270661, 0, 0.938022, -0.346577, 0, 0.938022, -0.346577, 0, 0.938022, -0.346577, 0, 0.938022, -0.346577, 0, 0.938022, -0.346577, 0, 0.938022, -0.346577, 0, 0.907947, -0.419084, 0, 0.907947, -0.419084, 0, 0.907947, -0.419084, 0, 0.907947, -0.419084, 0, 0.907947, -0.419084, 0, 0.907947, -0.419084, 0, 0.870527, -0.492121, 0, 0.870527, -0.492121, 0, 0.870527, -0.492121, 0, 0.870527, -0.492121, 0, 0.870527, -0.492121, 0, 0.870527, -0.492121, 0, 0.820905, -0.571065, 0, 0.820905, -0.571065, 0, 0.820905, -0.571065, 0, 0.820905, -0.571065, 0, 0.820905, -0.571065, 0, 0.820905, -0.571065, 0, 0.798343, -0.602203, 0, 0.798343, -0.602203, 0, 0.798343, -0.602203, 0, 0.798343, -0.602203, 0, 0.798343, -0.602203, 0, 0.798343, -0.602203, 0, 0.774030, -0.633149, 0, 0.774030, -0.633149, 0, 0.774030, -0.633149, 0, 0.774030, -0.633149, 0, 0.774030, -0.633149, 0, 0.774030, -0.633149, 0, 0.747880, -0.663834, 0, 0.747880, -0.663834, 0, 0.747880, -0.663834, 0, 0.747880, -0.663834, 0, 0.747880, -0.663834, 0, 0.747880, -0.663834, 0, 0.719782, -0.694200, 0, 0.719782, -0.694200, 0, 0.719782, -0.694200, 0, 0.719782, -0.694200, 0, 0.719782, -0.694200, 0, 0.719782, -0.694200, 0, 0.689285, -0.724490, 0, 0.689285, -0.724490, 0, 0.689285, -0.724490, 0, 0.689285, -0.724490, 0, 0.689285, -0.724490, 0, 0.689285, -0.724490, 0, 0.655662, -0.755054, 0, 0.655662, -0.755054, 0, 0.655662, -0.755054, 0, 0.655662, -0.755054, 0, 0.655662, -0.755054, 0, 0.655662, -0.755054, 0, 0.618213, -0.786011, 0, 0.618213, -0.786011, 0, 0.618213, -0.786011, 0, 0.618213, -0.786011, 0, 0.618213, -0.786011, 0, 0.618213, -0.786011, 0, 0.575954, -0.817482, 0, 0.575954, -0.817482, 0, 0.575954, -0.817482, 0, 0.575954, -0.817482, 0, 0.575954, -0.817482, 0, 0.575954, -0.817482, 0, 0.840297, -0.542127, 0, 0.840297, -0.542127, 0, 0.840297, -0.542127, 0, 0.840297, -0.542127, 0, 0.840297, -0.542127, 0, 0.840297, -0.542127, 0, 0.844463, -0.535614, 0, 0.844463, -0.535614, 0, 0.844463, -0.535614, 0, 0.844463, -0.535614, 0, 0.844463, -0.535614, 0, 0.844463, -0.535614, 0, 0.851113, -0.524983, 0, 0.851113, -0.524983, 0, 0.851113, -0.524983, 0, 0.851113, -0.524983, 0, 0.851113, -0.524983, 0, 0.851113, -0.524983, 0, 0.859410, -0.511286, 0, 0.859410, -0.511286, 0, 0.859410, -0.511286, 0, 0.859410, -0.511286, 0, 0.859410, -0.511286, 0, 0.859410, -0.511286, 0, 0.868832, -0.495107, 0, 0.868832, -0.495107, 0, 0.868832, -0.495107, 0, 0.868832, -0.495107, 0, 0.868832, -0.495107, 0, 0.868832, -0.495107, 0, 0.879058, -0.476714, 0, 0.879058, -0.476714, 0, 0.879058, -0.476714, 0, 0.879058, -0.476714, 0, 0.879058, -0.476714, 0, 0.879058, -0.476714, 0, 0.889820, -0.456312, 0, 0.889820, -0.456312, 0, 0.889820, -0.456312, 0, 0.889820, -0.456312, 0, 0.889820, -0.456312, 0, 0.889820, -0.456312, 0, 0.900856, -0.434118, 0, 0.900856, -0.434118, 0, 0.900856, -0.434118, 0, 0.900856, -0.434118, 0, 0.900856, -0.434118, 0, 0.900856, -0.434118, 0, 0.911952, -0.410298, 0, 0.911952, -0.410298, 0, 0.911952, -0.410298, 0, 0.911952, -0.410298, 0, 0.911952, -0.410298, 0, 0.911952, -0.410298, 0, 0.933897, -0.357543, 0, 0.933897, -0.357543, 0, 0.933897, -0.357543, 0, 0.933897, -0.357543, 0, 0.933897, -0.357543, 0, 0.933897, -0.357543, 0, 0.952223, -0.305405, 0, 0.952223, -0.305405, 0, 0.952223, -0.305405, 0, 0.952223, -0.305405, 0, 0.952223, -0.305405, 0, 0.952223, -0.305405, 0, 0.967944, -0.251164, 0, 0.967944, -0.251164, 0, 0.967944, -0.251164, 0, 0.967944, -0.251164, 0, 0.967944, -0.251164, 0, 0.967944, -0.251164, 0, 0.981249, -0.192745, 0, 0.981249, -0.192745, 0, 0.981249, -0.192745, 0, 0.981249, -0.192745, 0, 0.981249, -0.192745, 0, 0.981249, -0.192745, 0, 0.991746, -0.128215, 0, 0.991746, -0.128215, 0, 0.991746, -0.128215, 0, 0.991746, -0.128215, 0, 0.991746, -0.128215, 0, 0.991746, -0.128215, 0, 0.998440, -0.055836, 0, 0.998440, -0.055836, 0, 0.998440, -0.055836, 0, 0.998440, -0.055836, 0, 0.998440, -0.055836, 0, 0.998440, -0.055836, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.999668, 0.025752, 0, 0.993078, 0.117461, 0, 0.993078, 0.117461, 0, 0.993078, 0.117461, 0, 0.993078, 0.117461, 0, 0.993078, 0.117461, 0, 0.993078, 0.117461, 0, 0.979347, 0.202188, 0, 0.979347, 0.202188, 0, 0.979347, 0.202188, 0, 0.979347, 0.202188, 0, 0.979347, 0.202188, 0, 0.979347, 0.202188, 0, 0.986943, 0.161068, 0, 0.986943, 0.161068, 0, 0.986943, 0.161068, 0, 0.986943, 0.161068, 0, 0.986943, 0.161068, 0, 0.986943, 0.161068, 0, 0.992340, 0.123541, 0, 0.992340, 0.123541, 0, 0.992340, 0.123541, 0, 0.992340, 0.123541, 0, 0.992340, 0.123541, 0, 0.992340, 0.123541, 0, 0.995944, 0.089981, 0, 0.995944, 0.089981, 0, 0.995944, 0.089981, 0, 0.995944, 0.089981, 0, 0.995944, 0.089981, 0, 0.995944, 0.089981, 0, 0.998150, 0.060793, 0, 0.998150, 0.060793, 0, 0.998150, 0.060793, 0, 0.998150, 0.060793, 0, 0.998150, 0.060793, 0, 0.998150, 0.060793, 0, 0.999337, 0.036405, 0, 0.999337, 0.036405, 0, 0.999337, 0.036405, 0, 0.999337, 0.036405, 0, 0.999337, 0.036405, 0, 0.999337, 0.036405, 0, 0.999849, 0.017393, 0, 0.999849, 0.017393, 0, 0.999849, 0.017393, 0, 0.999849, 0.017393, 0, 0.999849, 0.017393, 0, 0.999849, 0.017393, 0, 0.999989, 0.004731, 0, 0.999989, 0.004731, 0, 0.999989, 0.004731, 0, 0.999989, 0.004731, 0, 0.999989, 0.004731, 0, 0.999989, 0.004731, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.999783, 0.020840, 0, 0.999783, 0.020840, 0, 0.999783, 0.020840, 0, 0.999783, 0.020840, 0, 0.999783, 0.020840, 0, 0.999783, 0.020840, 0, 0.994794, 0.101902, 0, 0.994794, 0.101902, 0, 0.994794, 0.101902, 0, 0.994794, 0.101902, 0, 0.994794, 0.101902, 0, 0.994794, 0.101902, 0, 0.960676, 0.277671, 0, 0.960676, 0.277671, 0, 0.960676, 0.277671, 0, 0.960676, 0.277671, 0, 0.960676, 0.277671, 0, 0.960676, 0.277671, 0, 0.834593, 0.550867, 0, 0.834593, 0.550867, 0, 0.834593, 0.550867, 0, 0.834593, 0.550867, 0, 0.834593, 0.550867, 0, 0.834593, 0.550867, 0, 0.603383, 0.797452, 0, 0.603383, 0.797452, 0, 0.603383, 0.797452, 0, 0.603383, 0.797452, 0, 0.603383, 0.797452, 0, 0.603383, 0.797452, 0, 0.390108, 0.920769, 0, 0.390108, 0.920769, 0, 0.390108, 0.920769, 0, 0.390108, 0.920769, 0, 0.390108, 0.920769, 0, 0.390108, 0.920769, 0, 0.258403, 0.966037, 0, 0.258403, 0.966037, 0, 0.258403, 0.966037, 0, 0.258403, 0.966037, 0, 0.258403, 0.966037, 0, 0.258403, 0.966037, 0, 0.192672, 0.981263, 0, 0.192672, 0.981263, 0, 0.192672, 0.981263, 0, 0.192672, 0.981263, 0, 0.192672, 0.981263, 0, 0.192672, 0.981263, 0, 0.170267, 0.985398, 0, 0.170267, 0.985398, 0, 0.170267, 0.985398, 0, 0.170267, 0.985398, 0, 0.170267, 0.985398, 0, 0.170267, 0.985398, 0, 0.175478, 0.984483, 0, 0.175478, 0.984483, 0, 0.175478, 0.984483, 0, 0.175478, 0.984483, 0, 0.175478, 0.984483, 0, 0.175478, 0.984483, 0, 0.198283, 0.980145, 0, 0.198283, 0.980145, 0, 0.198283, 0.980145, 0, 0.198283, 0.980145, 0, 0.198283, 0.980145, 0, 0.198283, 0.980145, 0, 0.233724, 0.972303, 0, 0.233724, 0.972303, 0, 0.233724, 0.972303, 0, 0.233724, 0.972303, 0, 0.233724, 0.972303, 0, 0.233724, 0.972303, 0, 0.277402, 0.960754, 0, 0.277402, 0.960754, 0, 0.277402, 0.960754, 0, 0.277402, 0.960754, 0, 0.277402, 0.960754, 0, 0.277402, 0.960754, 0, 0.327021, 0.945017, 0, 0.327021, 0.945017, 0, 0.327021, 0.945017, 0, 0.327021, 0.945017, 0, 0.327021, 0.945017, 0, 0.327021, 0.945017, 0, 0.382621, 0.923905, 0, 0.382621, 0.923905, 0, 0.382621, 0.923905, 0, 0.382621, 0.923905, 0, 0.382621, 0.923905, 0, 0.382621, 0.923905, 0, 0.444100, 0.895977, 0, 0.444100, 0.895977, 0, 0.444100, 0.895977, 0, 0.444100, 0.895977, 0, 0.444100, 0.895977, 0, 0.444100, 0.895977, 0, 0.481247, 0.876585, 0, 0.481247, 0.876585, 0, 0.481247, 0.876585, 0, 0.481247, 0.876585, 0, 0.481247, 0.876585, 0, 0.481247, 0.876585, 0, 0.515718, 0.856759, 0, 0.515718, 0.856759, 0, 0.515718, 0.856759, 0, 0.515718, 0.856759, 0, 0.515718, 0.856759, 0, 0.515718, 0.856759, 0, 0.548105, 0.836410, 0, 0.548105, 0.836410, 0, 0.548105, 0.836410, 0, 0.548105, 0.836410, 0, 0.548105, 0.836410, 0, 0.548105, 0.836410, 0, 0.578913, 0.815389, 0, 0.578913, 0.815389, 0, 0.578913, 0.815389, 0, 0.578913, 0.815389, 0, 0.578913, 0.815389, 0, 0.578913, 0.815389, 0, 0.608686, 0.793411, 0, 0.608686, 0.793411, 0, 0.608686, 0.793411, 0, 0.608686, 0.793411, 0, 0.608686, 0.793411, 0, 0.608686, 0.793411, 0, 0.637892, 0.770126, 0, 0.637892, 0.770126, 0, 0.637892, 0.770126, 0, 0.637892, 0.770126, 0, 0.637892, 0.770126, 0, 0.637892, 0.770126, 0, 0.666839, 0.745202, 0, 0.666839, 0.745202, 0, 0.666839, 0.745202, 0, 0.666839, 0.745202, 0, 0.666839, 0.745202, 0, 0.666839, 0.745202, 0, 0.695795, 0.718240, 0, 0.695795, 0.718240, 0, 0.695795, 0.718240, 0, 0.695795, 0.718240, 0, 0.695795, 0.718240, 0, 0.695795, 0.718240, 0, 0.638832, 0.769346, 0, 0.638832, 0.769346, 0, 0.638832, 0.769346, 0, 0.638832, 0.769346, 0, 0.638832, 0.769346, 0, 0.638832, 0.769346, 0, 0.705205, 0.709003, 0, 0.705205, 0.709003, 0, 0.705205, 0.709003, 0, 0.705205, 0.709003, 0, 0.705205, 0.709003, 0, 0.705205, 0.709003, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.962193, 0.272370, 0, 0.962193, 0.272370, 0, 0.962193, 0.272370, 0, 0.962193, 0.272370, 0, 0.962193, 0.272370, 0, 0.962193, 0.272370, 0, 0.960069, 0.279763, 0, 0.960069, 0.279763, 0, 0.960069, 0.279763, 0, 0.960069, 0.279763, 0, 0.960069, 0.279763, 0, 0.960069, 0.279763, 0, 0.959353, 0.282208, 0, 0.959353, 0.282208, 0, 0.959353, 0.282208, 0, 0.959353, 0.282208, 0, 0.959353, 0.282208, 0, 0.959353, 0.282208, 0, 0.960271, 0.279070, 0, 0.960271, 0.279070, 0, 0.960271, 0.279070, 0, 0.960271, 0.279070, 0, 0.960271, 0.279070, 0, 0.960271, 0.279070, 0, 0.962944, 0.269702, 0, 0.962944, 0.269702, 0, 0.962944, 0.269702, 0, 0.962944, 0.269702, 0, 0.962944, 0.269702, 0, 0.962944, 0.269702, 0, 0.967322, 0.253550, 0, 0.967322, 0.253550, 0, 0.967322, 0.253550, 0, 0.967322, 0.253550, 0, 0.967322, 0.253550, 0, 0.967322, 0.253550, 0, 0.973110, 0.230341, 0, 0.973110, 0.230341, 0, 0.973110, 0.230341, 0, 0.973110, 0.230341, 0, 0.973110, 0.230341, 0, 0.973110, 0.230341, 0, 0.979793, 0.200013, 0, 0.979793, 0.200013, 0, 0.979793, 0.200013, 0, 0.979793, 0.200013, 0, 0.979793, 0.200013, 0, 0.979793, 0.200013, 0, 0.984807, 0.173650, 0, 0.984807, 0.173650, 0, 0.984807, 0.173650, 0, 0.984807, 0.173650, 0, 0.984807, 0.173650, 0, 0.984807, 0.173650, 0, 0.988805, 0.149216, 0, 0.988805, 0.149216, 0, 0.988805, 0.149216, 0, 0.988805, 0.149216, 0, 0.988805, 0.149216, 0, 0.988805, 0.149216, 0, 0.991884, 0.127148, 0, 0.991884, 0.127148, 0, 0.991884, 0.127148, 0, 0.991884, 0.127148, 0, 0.991884, 0.127148, 0, 0.991884, 0.127148, 0, 0.994175, 0.107774, 0, 0.994175, 0.107774, 0, 0.994175, 0.107774, 0, 0.994175, 0.107774, 0, 0.994175, 0.107774, 0, 0.994175, 0.107774, 0, 0.995766, 0.091923, 0, 0.995766, 0.091923, 0, 0.995766, 0.091923, 0, 0.995766, 0.091923, 0, 0.995766, 0.091923, 0, 0.995766, 0.091923, 0, 0.996777, 0.080226, 0, 0.996777, 0.080226, 0, 0.996777, 0.080226, 0, 0.996777, 0.080226, 0, 0.996777, 0.080226, 0, 0.996777, 0.080226, 0, 0.997365, 0.072553, 0, 0.997365, 0.072553, 0, 0.997365, 0.072553, 0, 0.997365, 0.072553, 0, 0.997365, 0.072553, 0, 0.997365, 0.072553, 0, 0.997638, 0.068692, 0, 0.997638, 0.068692, 0, 0.997638, 0.068692, 0, 0.997638, 0.068692, 0, 0.997638, 0.068692, 0, 0.997638, 0.068692, 0, 0.997588, 0.069419, 0, 0.997588, 0.069419, 0, 0.997588, 0.069419, 0, 0.997588, 0.069419, 0, 0.997588, 0.069419, 0, 0.997588, 0.069419, 0, 0.997051, 0.076737, 0, 0.997051, 0.076737, 0, 0.997051, 0.076737, 0, 0.997051, 0.076737, 0, 0.997051, 0.076737, 0, 0.997051, 0.076737, 0, 0.995936, 0.090068, 0, 0.995936, 0.090068, 0, 0.995936, 0.090068, 0, 0.995936, 0.090068, 0, 0.995936, 0.090068, 0, 0.995936, 0.090068, 0, 0.994043, 0.108993, 0, 0.994043, 0.108993, 0, 0.994043, 0.108993, 0, 0.994043, 0.108993, 0, 0.994043, 0.108993, 0, 0.994043, 0.108993, 0, 0.991206, 0.132329, 0, 0.991206, 0.132329, 0, 0.991206, 0.132329, 0, 0.991206, 0.132329, 0, 0.991206, 0.132329, 0, 0.991206, 0.132329, 0, 0.987255, 0.159149, 0, 0.987255, 0.159149, 0, 0.987255, 0.159149, 0, 0.987255, 0.159149, 0, 0.987255, 0.159149, 0, 0.987255, 0.159149, 0, 0.981874, 0.189537, 0, 0.981874, 0.189537, 0, 0.981874, 0.189537, 0, 0.981874, 0.189537, 0, 0.981874, 0.189537, 0, 0.981874, 0.189537, 0, 0.974682, 0.223596, 0, 0.974682, 0.223596, 0, 0.974682, 0.223596, 0, 0.974682, 0.223596, 0, 0.974682, 0.223596, 0, 0.974682, 0.223596, 0, 0.970686, 0.240352, 0, 0.970686, 0.240352, 0, 0.970686, 0.240352, 0, 0.970686, 0.240352, 0, 0.970686, 0.240352, 0, 0.970686, 0.240352, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.997359, 0.072634, 0, 0.997359, 0.072634, 0, 0.997359, 0.072634, 0, 0.997359, 0.072634, 0, 0.997359, 0.072634, 0, 0.997359, 0.072634, 0, 0.998615, 0.052611, 0, 0.998615, 0.052611, 0, 0.998615, 0.052611, 0, 0.998615, 0.052611, 0, 0.998615, 0.052611, 0, 0.998615, 0.052611, 0, 0.999388, 0.034987, 0, 0.999388, 0.034987, 0, 0.999388, 0.034987, 0, 0.999388, 0.034987, 0, 0.999388, 0.034987, 0, 0.999388, 0.034987, 0, 0.999803, 0.019832, 0, 0.999803, 0.019832, 0, 0.999803, 0.019832, 0, 0.999803, 0.019832, 0, 0.999803, 0.019832, 0, 0.999803, 0.019832, 0, 0.999974, 0.007230, 0, 0.999974, 0.007230, 0, 0.999974, 0.007230, 0, 0.999974, 0.007230, 0, 0.999974, 0.007230, 0, 0.999974, 0.007230, 0, 0.999996, -0.002865, 0, 0.999996, -0.002865, 0, 0.999996, -0.002865, 0, 0.999996, -0.002865, 0, 0.999996, -0.002865, 0, 0.999996, -0.002865, 0, 0.999945, -0.010494, 0, 0.999945, -0.010494, 0, 0.999945, -0.010494, 0, 0.999945, -0.010494, 0, 0.999945, -0.010494, 0, 0.999945, -0.010494, 0, 0.999880, -0.015503, 0, 0.999880, -0.015503, 0, 0.999880, -0.015503, 0, 0.999880, -0.015503, 0, 0.999880, -0.015503, 0, 0.999880, -0.015503, 0, 0.999844, -0.017677, 0, 0.999844, -0.017677, 0, 0.999844, -0.017677, 0, 0.999844, -0.017677, 0, 0.999844, -0.017677, 0, 0.999844, -0.017677, 0, 0.999887, -0.015023, 0, 0.999887, -0.015023, 0, 0.999887, -0.015023, 0, 0.999887, -0.015023, 0, 0.999887, -0.015023, 0, 0.999887, -0.015023, 0, 0.999989, -0.004756, 0, 0.999989, -0.004756, 0, 0.999989, -0.004756, 0, 0.999989, -0.004756, 0, 0.999989, -0.004756, 0, 0.999989, -0.004756, 0, 0.999902, 0.013997, 0, 0.999902, 0.013997, 0, 0.999902, 0.013997, 0, 0.999902, 0.013997, 0, 0.999902, 0.013997, 0, 0.999902, 0.013997, 0, 0.999120, 0.041942, 0, 0.999120, 0.041942, 0, 0.999120, 0.041942, 0, 0.999120, 0.041942, 0, 0.999120, 0.041942, 0, 0.999120, 0.041942, 0, 0.996948, 0.078070, 0, 0.996948, 0.078070, 0, 0.996948, 0.078070, 0, 0.996948, 0.078070, 0, 0.996948, 0.078070, 0, 0.996948, 0.078070, 0, 0.992705, 0.120572, 0, 0.992705, 0.120572, 0, 0.992705, 0.120572, 0, 0.992705, 0.120572, 0, 0.992705, 0.120572, 0, 0.992705, 0.120572, 0, 0.985735, 0.168304, 0, 0.985735, 0.168304, 0, 0.985735, 0.168304, 0, 0.985735, 0.168304, 0, 0.985735, 0.168304, 0, 0.985735, 0.168304, 0, 0.975592, 0.219593, 0, 0.975592, 0.219593, 0, 0.975592, 0.219593, 0, 0.975592, 0.219593, 0, 0.975592, 0.219593, 0, 0.975592, 0.219593, 0, 0.961625, 0.274369, 0, 0.961625, 0.274369, 0, 0.961625, 0.274369, 0, 0.961625, 0.274369, 0, 0.961625, 0.274369, 0, 0.961625, 0.274369, 0, 0.948708, 0.316155, 0, 0.948708, 0.316155, 0, 0.948708, 0.316155, 0, 0.948708, 0.316155, 0, 0.948708, 0.316155, 0, 0.948708, 0.316155, 0, 0.938487, 0.345315, 0, 0.938487, 0.345315, 0, 0.938487, 0.345315, 0, 0.938487, 0.345315, 0, 0.938487, 0.345315, 0, 0.938487, 0.345315, 0, 0.931853, 0.362835, 0, 0.931853, 0.362835, 0, 0.931853, 0.362835, 0, 0.931853, 0.362835, 0, 0.931853, 0.362835, 0, 0.931853, 0.362835, 0, 0.929067, 0.369912, 0, 0.929067, 0.369912, 0, 0.929067, 0.369912, 0, 0.929067, 0.369912, 0, 0.929067, 0.369912, 0, 0.929067, 0.369912, 0, 0.929897, 0.367821, 0, 0.929897, 0.367821, 0, 0.929897, 0.367821, 0, 0.929897, 0.367821, 0, 0.929897, 0.367821, 0, 0.929897, 0.367821, 0, 0.933773, 0.357865, 0, 0.933773, 0.357865, 0, 0.933773, 0.357865, 0, 0.933773, 0.357865, 0, 0.933773, 0.357865, 0, 0.933773, 0.357865, 0, 0.939963, 0.341276, 0, 0.939963, 0.341276, 0, 0.939963, 0.341276, 0, 0.939963, 0.341276, 0, 0.939963, 0.341276, 0, 0.939963, 0.341276, 0, 0.952303, 0.305153, 0, 0.952303, 0.305153, 0, 0.952303, 0.305153, 0, 0.952303, 0.305153, 0, 0.952303, 0.305153, 0, 0.952303, 0.305153, 0, 0.950015, 0.312204, 0, 0.950015, 0.312204, 0, 0.950015, 0.312204, 0, 0.950015, 0.312204, 0, 0.950015, 0.312204, 0, 0.950015, 0.312204, 0, 0.941733, 0.336361, 0, 0.941733, 0.336361, 0, 0.941733, 0.336361, 0, 0.941733, 0.336361, 0, 0.941733, 0.336361, 0, 0.941733, 0.336361, 0, 0.933431, 0.358756, 0, 0.933431, 0.358756, 0, 0.933431, 0.358756, 0, 0.933431, 0.358756, 0, 0.933431, 0.358756, 0, 0.933431, 0.358756, 0, 0.925089, 0.379751, 0, 0.925089, 0.379751, 0, 0.925089, 0.379751, 0, 0.925089, 0.379751, 0, 0.925089, 0.379751, 0, 0.925089, 0.379751, 0, 0.916659, 0.399670, 0, 0.916659, 0.399670, 0, 0.916659, 0.399670, 0, 0.916659, 0.399670, 0, 0.916659, 0.399670, 0, 0.916659, 0.399670, 0, 0.908277, 0.418369, 0, 0.908277, 0.418369, 0, 0.908277, 0.418369, 0, 0.908277, 0.418369, 0, 0.908277, 0.418369, 0, 0.908277, 0.418369, 0, 0.900110, 0.435663, 0, 0.900110, 0.435663, 0, 0.900110, 0.435663, 0, 0.900110, 0.435663, 0, 0.900110, 0.435663, 0, 0.900110, 0.435663, 0, 0.892137, 0.451765, 0, 0.892137, 0.451765, 0, 0.892137, 0.451765, 0, 0.892137, 0.451765, 0, 0.892137, 0.451765, 0, 0.892137, 0.451765, 0, 0.884325, 0.466871, 0, 0.884325, 0.466871, 0, 0.884325, 0.466871, 0, 0.884325, 0.466871, 0, 0.884325, 0.466871, 0, 0.884325, 0.466871, 0, 0.879586, 0.475741, 0, 0.879586, 0.475741, 0, 0.879586, 0.475741, 0, 0.879586, 0.475741, 0, 0.879586, 0.475741, 0, 0.879586, 0.475741, 0, 0.875248, 0.483674, 0, 0.875248, 0.483674, 0, 0.875248, 0.483674, 0, 0.875248, 0.483674, 0, 0.875248, 0.483674, 0, 0.875248, 0.483674, 0, 0.871365, 0.490635, 0, 0.871365, 0.490635, 0, 0.871365, 0.490635, 0, 0.871365, 0.490635, 0, 0.871365, 0.490635, 0, 0.871365, 0.490635, 0, 0.867990, 0.496581, 0, 0.867990, 0.496581, 0, 0.867990, 0.496581, 0, 0.867990, 0.496581, 0, 0.867990, 0.496581, 0, 0.867990, 0.496581, 0, 0.865196, 0.501434, 0, 0.865196, 0.501434, 0, 0.865196, 0.501434, 0, 0.865196, 0.501434, 0, 0.865196, 0.501434, 0, 0.865196, 0.501434, 0, 0.863060, 0.505101, 0, 0.863060, 0.505101, 0, 0.863060, 0.505101, 0, 0.863060, 0.505101, 0, 0.863060, 0.505101, 0, 0.863060, 0.505101, 0, 0.861648, 0.507506, 0, 0.861648, 0.507506, 0, 0.861648, 0.507506, 0, 0.861648, 0.507506, 0, 0.861648, 0.507506, 0, 0.861648, 0.507506, 0, 0.861028, 0.508557, 0, 0.861028, 0.508557, 0, 0.861028, 0.508557, 0, 0.861028, 0.508557, 0, 0.861028, 0.508557, 0, 0.861028, 0.508557, 0, 0.820729, 0.571318, 0, 0.820729, 0.571318, 0, 0.820729, 0.571318, 0, 0.820729, 0.571318, 0, 0.820729, 0.571318, 0, 0.820729, 0.571318, 0, 0.823772, 0.566922, 0, 0.823772, 0.566922, 0, 0.823772, 0.566922, 0, 0.823772, 0.566922, 0, 0.823772, 0.566922, 0, 0.823772, 0.566922, 0, 0.827217, 0.561883, 0, 0.827217, 0.561883, 0, 0.827217, 0.561883, 0, 0.827217, 0.561883, 0, 0.827217, 0.561883, 0, 0.827217, 0.561883, 0, 0.831032, 0.556225, 0, 0.831032, 0.556225, 0, 0.831032, 0.556225, 0, 0.831032, 0.556225, 0, 0.831032, 0.556225, 0, 0.831032, 0.556225, 0, 0.835187, 0.549966, 0, 0.835187, 0.549966, 0, 0.835187, 0.549966, 0, 0.835187, 0.549966, 0, 0.835187, 0.549966, 0, 0.835187, 0.549966, 0, 0.839667, 0.543102, 0, 0.839667, 0.543102, 0, 0.839667, 0.543102, 0, 0.839667, 0.543102, 0, 0.839667, 0.543102, 0, 0.839667, 0.543102, 0, 0.844454, 0.535627, 0, 0.844454, 0.535627, 0, 0.844454, 0.535627, 0, 0.844454, 0.535627, 0, 0.844454, 0.535627, 0, 0.844454, 0.535627, 0, 0.849526, 0.527547, 0, 0.849526, 0.527547, 0, 0.849526, 0.527547, 0, 0.849526, 0.527547, 0, 0.849526, 0.527547, 0, 0.849526, 0.527547, 0, 0.854859, 0.518861, 0, 0.854859, 0.518861, 0, 0.854859, 0.518861, 0, 0.854859, 0.518861, 0, 0.854859, 0.518861, 0, 0.854859, 0.518861, 0, 0.869305, 0.494276, 0, 0.869305, 0.494276, 0, 0.869305, 0.494276, 0, 0.869305, 0.494276, 0, 0.869305, 0.494276, 0, 0.869305, 0.494276, 0, 0.883377, 0.468663, 0, 0.883377, 0.468663, 0, 0.883377, 0.468663, 0, 0.883377, 0.468663, 0, 0.883377, 0.468663, 0, 0.883377, 0.468663, 0, 0.897295, 0.441431, 0, 0.897295, 0.441431, 0, 0.897295, 0.441431, 0, 0.897295, 0.441431, 0, 0.897295, 0.441431, 0, 0.897295, 0.441431, 0, 0.911196, 0.411973, 0, 0.911196, 0.411973, 0, 0.911196, 0.411973, 0, 0.911196, 0.411973, 0, 0.911196, 0.411973, 0, 0.911196, 0.411973, 0, 0.925189, 0.379507, 0, 0.925189, 0.379507, 0, 0.925189, 0.379507, 0, 0.925189, 0.379507, 0, 0.925189, 0.379507, 0, 0.925189, 0.379507, 0, 0.939275, 0.343164, 0, 0.939275, 0.343164, 0, 0.939275, 0.343164, 0, 0.939275, 0.343164, 0, 0.939275, 0.343164, 0, 0.939275, 0.343164, 0, 0.953284, 0.302075, 0, 0.953284, 0.302075, 0, 0.953284, 0.302075, 0, 0.953284, 0.302075, 0, 0.953284, 0.302075, 0, 0.953284, 0.302075, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.966365, 0.257176, 0, 0.966365, 0.257176, 0, 0.966365, 0.257176, 0, 0.966365, 0.257176, 0, 0.966365, 0.257176, 0, 0.966365, 0.257176, 0, 0.963894, 0.266287, 0, 0.963894, 0.266287, 0, 0.963894, 0.266287, 0, 0.963894, 0.266287, 0, 0.963894, 0.266287, 0, 0.963894, 0.266287, 0, 0.962000, 0.273050, 0, 0.962000, 0.273050, 0, 0.962000, 0.273050, 0, 0.962000, 0.273050, 0, 0.962000, 0.273050, 0, 0.962000, 0.273050, 0, 0.960707, 0.277566, 0, 0.960707, 0.277566, 0, 0.960707, 0.277566, 0, 0.960707, 0.277566, 0, 0.960707, 0.277566, 0, 0.960707, 0.277566, 0, 0.960007, 0.279977, 0, 0.960007, 0.279977, 0, 0.960007, 0.279977, 0, 0.960007, 0.279977, 0, 0.960007, 0.279977, 0, 0.960007, 0.279977, 0, 0.959877, 0.280421, 0, 0.959877, 0.280421, 0, 0.959877, 0.280421, 0, 0.959877, 0.280421, 0, 0.959877, 0.280421, 0, 0.959877, 0.280421, 0, 0.960297, 0.278980, 0, 0.960297, 0.278980, 0, 0.960297, 0.278980, 0, 0.960297, 0.278980, 0, 0.960297, 0.278980, 0, 0.960297, 0.278980, 0, 0.961234, 0.275733, 0, 0.961234, 0.275733, 0, 0.961234, 0.275733, 0, 0.961234, 0.275733, 0, 0.961234, 0.275733, 0, 0.961234, 0.275733, 0, 0.963157, 0.268942, 0, 0.963157, 0.268942, 0, 0.963157, 0.268942, 0, 0.963157, 0.268942, 0, 0.963157, 0.268942, 0, 0.963157, 0.268942, 0, 0.965690, 0.259698, 0, 0.965690, 0.259698, 0, 0.965690, 0.259698, 0, 0.965690, 0.259698, 0, 0.965690, 0.259698, 0, 0.965690, 0.259698, 0, 0.968758, 0.248007, 0, 0.968758, 0.248007, 0, 0.968758, 0.248007, 0, 0.968758, 0.248007, 0, 0.968758, 0.248007, 0, 0.968758, 0.248007, 0, 0.972279, 0.233826, 0, 0.972279, 0.233826, 0, 0.972279, 0.233826, 0, 0.972279, 0.233826, 0, 0.972279, 0.233826, 0, 0.972279, 0.233826, 0, 0.976089, 0.217372, 0, 0.976089, 0.217372, 0, 0.976089, 0.217372, 0, 0.976089, 0.217372, 0, 0.976089, 0.217372, 0, 0.976089, 0.217372, 0, 0.980037, 0.198816, 0, 0.980037, 0.198816, 0, 0.980037, 0.198816, 0, 0.980037, 0.198816, 0, 0.980037, 0.198816, 0, 0.980037, 0.198816, 0, 0.984032, 0.177991, 0, 0.984032, 0.177991, 0, 0.984032, 0.177991, 0, 0.984032, 0.177991, 0, 0.984032, 0.177991, 0, 0.984032, 0.177991, 0, 0.987961, 0.154700, 0, 0.987961, 0.154700, 0, 0.987961, 0.154700, 0, 0.987961, 0.154700, 0, 0.987961, 0.154700, 0, 0.987961, 0.154700, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.987122, 0.159972, 0, 0.987122, 0.159972, 0, 0.987122, 0.159972, 0, 0.987122, 0.159972, 0, 0.987122, 0.159972, 0, 0.987122, 0.159972, 0, 0.984204, 0.177040, 0, 0.984204, 0.177040, 0, 0.984204, 0.177040, 0, 0.984204, 0.177040, 0, 0.984204, 0.177040, 0, 0.984204, 0.177040, 0, 0.981397, 0.191988, 0, 0.981397, 0.191988, 0, 0.981397, 0.191988, 0, 0.981397, 0.191988, 0, 0.981397, 0.191988, 0, 0.981397, 0.191988, 0, 0.978779, 0.204918, 0, 0.978779, 0.204918, 0, 0.978779, 0.204918, 0, 0.978779, 0.204918, 0, 0.978779, 0.204918, 0, 0.978779, 0.204918, 0, 0.976413, 0.215913, 0, 0.976413, 0.215913, 0, 0.976413, 0.215913, 0, 0.976413, 0.215913, 0, 0.976413, 0.215913, 0, 0.976413, 0.215913, 0, 0.974333, 0.225111, 0, 0.974333, 0.225111, 0, 0.974333, 0.225111, 0, 0.974333, 0.225111, 0, 0.974333, 0.225111, 0, 0.974333, 0.225111, 0, 0.972564, 0.232634, 0, 0.972564, 0.232634, 0, 0.972564, 0.232634, 0, 0.972564, 0.232634, 0, 0.972564, 0.232634, 0, 0.972564, 0.232634, 0, 0.971142, 0.238501, 0, 0.971142, 0.238501, 0, 0.971142, 0.238501, 0, 0.971142, 0.238501, 0, 0.971142, 0.238501, 0, 0.971142, 0.238501, 0, 0.970099, 0.242708, 0, 0.970099, 0.242708, 0, 0.970099, 0.242708, 0, 0.970099, 0.242708, 0, 0.970099, 0.242708, 0, 0.970099, 0.242708, 0, 0.969446, 0.245304, 0, 0.969446, 0.245304, 0, 0.969446, 0.245304, 0, 0.969446, 0.245304, 0, 0.969446, 0.245304, 0, 0.969446, 0.245304, 0, 0.969586, 0.244750, 0, 0.969586, 0.244750, 0, 0.969586, 0.244750, 0, 0.969586, 0.244750, 0, 0.969586, 0.244750, 0, 0.969586, 0.244750, 0, 0.970551, 0.240895, 0, 0.970551, 0.240895, 0, 0.970551, 0.240895, 0, 0.970551, 0.240895, 0, 0.970551, 0.240895, 0, 0.970551, 0.240895, 0, 0.972335, 0.233592, 0, 0.972335, 0.233592, 0, 0.972335, 0.233592, 0, 0.972335, 0.233592, 0, 0.972335, 0.233592, 0, 0.972335, 0.233592, 0, 0.974814, 0.223020, 0, 0.974814, 0.223020, 0, 0.974814, 0.223020, 0, 0.974814, 0.223020, 0, 0.974814, 0.223020, 0, 0.974814, 0.223020, 0, 0.977826, 0.209418, 0, 0.977826, 0.209418, 0, 0.977826, 0.209418, 0, 0.977826, 0.209418, 0, 0.977826, 0.209418, 0, 0.977826, 0.209418, 0, 0.981245, 0.192764, 0, 0.981245, 0.192764, 0, 0.981245, 0.192764, 0, 0.981245, 0.192764, 0, 0.981245, 0.192764, 0, 0.981245, 0.192764, 0, 0.984907, 0.173086, 0, 0.984907, 0.173086, 0, 0.984907, 0.173086, 0, 0.984907, 0.173086, 0, 0.984907, 0.173086, 0, 0.984907, 0.173086, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.981345, 0.192254, 0, 0.981345, 0.192254, 0, 0.981345, 0.192254, 0, 0.981345, 0.192254, 0, 0.981345, 0.192254, 0, 0.981345, 0.192254, 0, 0.977132, 0.212633, 0, 0.977132, 0.212633, 0, 0.977132, 0.212633, 0, 0.977132, 0.212633, 0, 0.977132, 0.212633, 0, 0.977132, 0.212633, 0, 0.973081, 0.230462, 0, 0.973081, 0.230462, 0, 0.973081, 0.230462, 0, 0.973081, 0.230462, 0, 0.973081, 0.230462, 0, 0.973081, 0.230462, 0, 0.969295, 0.245900, 0, 0.969295, 0.245900, 0, 0.969295, 0.245900, 0, 0.969295, 0.245900, 0, 0.969295, 0.245900, 0, 0.969295, 0.245900, 0, 0.965857, 0.259075, 0, 0.965857, 0.259075, 0, 0.965857, 0.259075, 0, 0.965857, 0.259075, 0, 0.965857, 0.259075, 0, 0.965857, 0.259075, 0, 0.962797, 0.270225, 0, 0.962797, 0.270225, 0, 0.962797, 0.270225, 0, 0.962797, 0.270225, 0, 0.962797, 0.270225, 0, 0.962797, 0.270225, 0, 0.960129, 0.279559, 0, 0.960129, 0.279559, 0, 0.960129, 0.279559, 0, 0.960129, 0.279559, 0, 0.960129, 0.279559, 0, 0.960129, 0.279559, 0, 0.957895, 0.287120, 0, 0.957895, 0.287120, 0, 0.957895, 0.287120, 0, 0.957895, 0.287120, 0, 0.957895, 0.287120, 0, 0.957895, 0.287120, 0, 0.956136, 0.292924, 0, 0.956136, 0.292924, 0, 0.956136, 0.292924, 0, 0.956136, 0.292924, 0, 0.956136, 0.292924, 0, 0.956136, 0.292924, 0, 0.954896, 0.296939, 0, 0.954896, 0.296939, 0, 0.954896, 0.296939, 0, 0.954896, 0.296939, 0, 0.954896, 0.296939, 0, 0.954896, 0.296939, 0, 0.954821, 0.297183, 0, 0.954821, 0.297183, 0, 0.954821, 0.297183, 0, 0.954821, 0.297183, 0, 0.954821, 0.297183, 0, 0.954821, 0.297183, 0, 0.955980, 0.293434, 0, 0.955980, 0.293434, 0, 0.955980, 0.293434, 0, 0.955980, 0.293434, 0, 0.955980, 0.293434, 0, 0.955980, 0.293434, 0, 0.958389, 0.285466, 0, 0.958389, 0.285466, 0, 0.958389, 0.285466, 0, 0.958389, 0.285466, 0, 0.958389, 0.285466, 0, 0.958389, 0.285466, 0, 0.961891, 0.273432, 0, 0.961891, 0.273432, 0, 0.961891, 0.273432, 0, 0.961891, 0.273432, 0, 0.961891, 0.273432, 0, 0.961891, 0.273432, 0, 0.966261, 0.257565, 0, 0.966261, 0.257565, 0, 0.966261, 0.257565, 0, 0.966261, 0.257565, 0, 0.966261, 0.257565, 0, 0.966261, 0.257565, 0, 0.971316, 0.237794, 0, 0.971316, 0.237794, 0, 0.971316, 0.237794, 0, 0.971316, 0.237794, 0, 0.971316, 0.237794, 0, 0.971316, 0.237794, 0, 0.976806, 0.214124, 0, 0.976806, 0.214124, 0, 0.976806, 0.214124, 0, 0.976806, 0.214124, 0, 0.976806, 0.214124, 0, 0.976806, 0.214124, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.925944, 0.377661, 0, 0.925944, 0.377661, 0, 0.925944, 0.377661, 0, 0.925944, 0.377661, 0, 0.925944, 0.377661, 0, 0.925944, 0.377661, 0, 0.934124, 0.356949, 0, 0.934124, 0.356949, 0, 0.934124, 0.356949, 0, 0.934124, 0.356949, 0, 0.934124, 0.356949, 0, 0.934124, 0.356949, 0, 0.950324, 0.311263, 0, 0.950324, 0.311263, 0, 0.950324, 0.311263, 0, 0.950324, 0.311263, 0, 0.950324, 0.311263, 0, 0.950324, 0.311263, 0, 0.946945, 0.321397, 0, 0.946945, 0.321397, 0, 0.946945, 0.321397, 0, 0.946945, 0.321397, 0, 0.946945, 0.321397, 0, 0.946945, 0.321397, 0, 0.944302, 0.329081, 0, 0.944302, 0.329081, 0, 0.944302, 0.329081, 0, 0.944302, 0.329081, 0, 0.944302, 0.329081, 0, 0.944302, 0.329081, 0, 0.942419, 0.334434, 0, 0.942419, 0.334434, 0, 0.942419, 0.334434, 0, 0.942419, 0.334434, 0, 0.942419, 0.334434, 0, 0.942419, 0.334434, 0, 0.941302, 0.337565, 0, 0.941302, 0.337565, 0, 0.941302, 0.337565, 0, 0.941302, 0.337565, 0, 0.941302, 0.337565, 0, 0.941302, 0.337565, 0, 0.940926, 0.338613, 0, 0.940926, 0.338613, 0, 0.940926, 0.338613, 0, 0.940926, 0.338613, 0, 0.940926, 0.338613, 0, 0.940926, 0.338613, 0, 0.941251, 0.337708, 0, 0.941251, 0.337708, 0, 0.941251, 0.337708, 0, 0.941251, 0.337708, 0, 0.941251, 0.337708, 0, 0.941251, 0.337708, 0, 0.942240, 0.334938, 0, 0.942240, 0.334938, 0, 0.942240, 0.334938, 0, 0.942240, 0.334938, 0, 0.942240, 0.334938, 0, 0.942240, 0.334938, 0, 0.943845, 0.330389, 0, 0.943845, 0.330389, 0, 0.943845, 0.330389, 0, 0.943845, 0.330389, 0, 0.943845, 0.330389, 0, 0.943845, 0.330389, 0, 0.945964, 0.324271, 0, 0.945964, 0.324271, 0, 0.945964, 0.324271, 0, 0.945964, 0.324271, 0, 0.945964, 0.324271, 0, 0.945964, 0.324271, 0, 0.948479, 0.316839, 0, 0.948479, 0.316839, 0, 0.948479, 0.316839, 0, 0.948479, 0.316839, 0, 0.948479, 0.316839, 0, 0.948479, 0.316839, 0, 0.951356, 0.308093, 0, 0.951356, 0.308093, 0, 0.951356, 0.308093, 0, 0.951356, 0.308093, 0, 0.951356, 0.308093, 0, 0.951356, 0.308093, 0, 0.954560, 0.298019, 0, 0.954560, 0.298019, 0, 0.954560, 0.298019, 0, 0.954560, 0.298019, 0, 0.954560, 0.298019, 0, 0.954560, 0.298019, 0, 0.958017, 0.286713, 0, 0.958017, 0.286713, 0, 0.958017, 0.286713, 0, 0.958017, 0.286713, 0, 0.958017, 0.286713, 0, 0.958017, 0.286713, 0, 0.961658, 0.274252, 0, 0.961658, 0.274252, 0, 0.961658, 0.274252, 0, 0.961658, 0.274252, 0, 0.961658, 0.274252, 0, 0.961658, 0.274252, 0, 0.965453, 0.260577, 0, 0.965453, 0.260577, 0, 0.965453, 0.260577, 0, 0.965453, 0.260577, 0, 0.965453, 0.260577, 0, 0.965453, 0.260577, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.995983, 0.089542, 0, 0.995983, 0.089542, 0, 0.995983, 0.089542, 0, 0.995983, 0.089542, 0, 0.995983, 0.089542, 0, 0.995983, 0.089542, 0, 0.994472, 0.105005, 0, 0.994472, 0.105005, 0, 0.994472, 0.105005, 0, 0.994472, 0.105005, 0, 0.994472, 0.105005, 0, 0.994472, 0.105005, 0, 0.996220, 0.086866, 0, 0.996220, 0.086866, 0, 0.996220, 0.086866, 0, 0.996220, 0.086866, 0, 0.996220, 0.086866, 0, 0.996220, 0.086866, 0, 0.997488, 0.070837, 0, 0.997488, 0.070837, 0, 0.997488, 0.070837, 0, 0.997488, 0.070837, 0, 0.997488, 0.070837, 0, 0.997488, 0.070837, 0, 0.998372, 0.057036, 0, 0.998372, 0.057036, 0, 0.998372, 0.057036, 0, 0.998372, 0.057036, 0, 0.998372, 0.057036, 0, 0.998372, 0.057036, 0, 0.998963, 0.045535, 0, 0.998963, 0.045535, 0, 0.998963, 0.045535, 0, 0.998963, 0.045535, 0, 0.998963, 0.045535, 0, 0.998963, 0.045535, 0, 0.999334, 0.036484, 0, 0.999334, 0.036484, 0, 0.999334, 0.036484, 0, 0.999334, 0.036484, 0, 0.999334, 0.036484, 0, 0.999334, 0.036484, 0, 0.999550, 0.029982, 0, 0.999550, 0.029982, 0, 0.999550, 0.029982, 0, 0.999550, 0.029982, 0, 0.999550, 0.029982, 0, 0.999550, 0.029982, 0, 0.999663, 0.025975, 0, 0.999663, 0.025975, 0, 0.999663, 0.025975, 0, 0.999663, 0.025975, 0, 0.999663, 0.025975, 0, 0.999663, 0.025975, 0, 0.999703, 0.024378, 0, 0.999703, 0.024378, 0, 0.999703, 0.024378, 0, 0.999703, 0.024378, 0, 0.999703, 0.024378, 0, 0.999703, 0.024378, 0, 0.999702, 0.024419, 0, 0.999702, 0.024419, 0, 0.999702, 0.024419, 0, 0.999702, 0.024419, 0, 0.999702, 0.024419, 0, 0.999702, 0.024419, 0, 0.999687, 0.025023, 0, 0.999687, 0.025023, 0, 0.999687, 0.025023, 0, 0.999687, 0.025023, 0, 0.999687, 0.025023, 0, 0.999687, 0.025023, 0, 0.999657, 0.026174, 0, 0.999657, 0.026174, 0, 0.999657, 0.026174, 0, 0.999657, 0.026174, 0, 0.999657, 0.026174, 0, 0.999657, 0.026174, 0, 0.999612, 0.027856, 0, 0.999612, 0.027856, 0, 0.999612, 0.027856, 0, 0.999612, 0.027856, 0, 0.999612, 0.027856, 0, 0.999612, 0.027856, 0, 0.999548, 0.030055, 0, 0.999548, 0.030055, 0, 0.999548, 0.030055, 0, 0.999548, 0.030055, 0, 0.999548, 0.030055, 0, 0.999548, 0.030055, 0, 0.999463, 0.032754, 0, 0.999463, 0.032754, 0, 0.999463, 0.032754, 0, 0.999463, 0.032754, 0, 0.999463, 0.032754, 0, 0.999463, 0.032754, 0, 0.999354, 0.035940, 0, 0.999354, 0.035940, 0, 0.999354, 0.035940, 0, 0.999354, 0.035940, 0, 0.999354, 0.035940, 0, 0.999354, 0.035940, 0, 0.999216, 0.039601, 0, 0.999216, 0.039601, 0, 0.999216, 0.039601, 0, 0.999216, 0.039601, 0, 0.999216, 0.039601, 0, 0.999216, 0.039601, 0, 0.999936, 0.011323, 0, 0.999936, 0.011323, 0, 0.999936, 0.011323, 0, 0.999936, 0.011323, 0, 0.999936, 0.011323, 0, 0.999936, 0.011323, 0, 0.999972, 0.007480, 0, 0.999972, 0.007480, 0, 0.999972, 0.007480, 0, 0.999972, 0.007480, 0, 0.999972, 0.007480, 0, 0.999972, 0.007480, 0, 0.999992, 0.004075, 0, 0.999992, 0.004075, 0, 0.999992, 0.004075, 0, 0.999992, 0.004075, 0, 0.999992, 0.004075, 0, 0.999992, 0.004075, 0, 0.999999, 0.001119, 0, 0.999999, 0.001119, 0, 0.999999, 0.001119, 0, 0.999999, 0.001119, 0, 0.999999, 0.001119, 0, 0.999999, 0.001119, 0, 0.999999, -0.001376, 0, 0.999999, -0.001376, 0, 0.999999, -0.001376, 0, 0.999999, -0.001376, 0, 0.999999, -0.001376, 0, 0.999999, -0.001376, 0, 0.999994, -0.003376, 0, 0.999994, -0.003376, 0, 0.999994, -0.003376, 0, 0.999994, -0.003376, 0, 0.999994, -0.003376, 0, 0.999994, -0.003376, 0, 0.999988, -0.004842, 0, 0.999988, -0.004842, 0, 0.999988, -0.004842, 0, 0.999988, -0.004842, 0, 0.999988, -0.004842, 0, 0.999988, -0.004842, 0, 0.999983, -0.005758, 0, 0.999983, -0.005758, 0, 0.999983, -0.005758, 0, 0.999983, -0.005758, 0, 0.999983, -0.005758, 0, 0.999983, -0.005758, 0, 0.999981, -0.006109, 0, 0.999981, -0.006109, 0, 0.999981, -0.006109, 0, 0.999981, -0.006109, 0, 0.999981, -0.006109, 0, 0.999981, -0.006109, 0, 0.999991, -0.004341, 0, 0.999991, -0.004341, 0, 0.999991, -0.004341, 0, 0.999991, -0.004341, 0, 0.999991, -0.004341, 0, 0.999991, -0.004341, 0, 0.999999, 0.001451, 0, 0.999999, 0.001451, 0, 0.999999, 0.001451, 0, 0.999999, 0.001451, 0, 0.999999, 0.001451, 0, 0.999999, 0.001451, 0, 0.999934, 0.011495, 0, 0.999934, 0.011495, 0, 0.999934, 0.011495, 0, 0.999934, 0.011495, 0, 0.999934, 0.011495, 0, 0.999934, 0.011495, 0, 0.999663, 0.025960, 0, 0.999663, 0.025960, 0, 0.999663, 0.025960, 0, 0.999663, 0.025960, 0, 0.999663, 0.025960, 0, 0.999663, 0.025960, 0, 0.999018, 0.044305, 0, 0.999018, 0.044305, 0, 0.999018, 0.044305, 0, 0.999018, 0.044305, 0, 0.999018, 0.044305, 0, 0.999018, 0.044305, 0, 0.997832, 0.065816, 0, 0.997832, 0.065816, 0, 0.997832, 0.065816, 0, 0.997832, 0.065816, 0, 0.997832, 0.065816, 0, 0.997832, 0.065816, 0, 0.995919, 0.090257, 0, 0.995919, 0.090257, 0, 0.995919, 0.090257, 0, 0.995919, 0.090257, 0, 0.995919, 0.090257, 0, 0.995919, 0.090257, 0, 0.993098, 0.117287, 0, 0.993098, 0.117287, 0, 0.993098, 0.117287, 0, 0.993098, 0.117287, 0, 0.993098, 0.117287, 0, 0.993098, 0.117287, 0, 0.989507, 0.144482, 0, 0.989507, 0.144482, 0, 0.989507, 0.144482, 0, 0.989507, 0.144482, 0, 0.989507, 0.144482, 0, 0.989507, 0.144482, 0, 0.985843, 0.167672, 0, 0.985843, 0.167672, 0, 0.985843, 0.167672, 0, 0.985843, 0.167672, 0, 0.985843, 0.167672, 0, 0.985843, 0.167672, 0, 0.982410, 0.186739, 0, 0.982410, 0.186739, 0, 0.982410, 0.186739, 0, 0.982410, 0.186739, 0, 0.982410, 0.186739, 0, 0.982410, 0.186739, 0, 0.979452, 0.201679, 0, 0.979452, 0.201679, 0, 0.979452, 0.201679, 0, 0.979452, 0.201679, 0, 0.979452, 0.201679, 0, 0.979452, 0.201679, 0, 0.977244, 0.212119, 0, 0.977244, 0.212119, 0, 0.977244, 0.212119, 0, 0.977244, 0.212119, 0, 0.977244, 0.212119, 0, 0.977244, 0.212119, 0, 0.975990, 0.217815, 0, 0.975990, 0.217815, 0, 0.975990, 0.217815, 0, 0.975990, 0.217815, 0, 0.975990, 0.217815, 0, 0.975990, 0.217815, 0, 0.975717, 0.219035, 0, 0.975717, 0.219035, 0, 0.975717, 0.219035, 0, 0.975717, 0.219035, 0, 0.975717, 0.219035, 0, 0.975717, 0.219035, 0, 0.976381, 0.216057, 0, 0.976381, 0.216057, 0, 0.976381, 0.216057, 0, 0.976381, 0.216057, 0, 0.976381, 0.216057, 0, 0.976381, 0.216057, 0, 0.977636, 0.210303, 0, 0.977636, 0.210303, 0, 0.977636, 0.210303, 0, 0.977636, 0.210303, 0, 0.977636, 0.210303, 0, 0.977636, 0.210303, 0, 0.979378, 0.202035, 0, 0.979378, 0.202035, 0, 0.979378, 0.202035, 0, 0.979378, 0.202035, 0, 0.979378, 0.202035, 0, 0.979378, 0.202035, 0, 0.981534, 0.191290, 0, 0.981534, 0.191290, 0, 0.981534, 0.191290, 0, 0.981534, 0.191290, 0, 0.981534, 0.191290, 0, 0.981534, 0.191290, 0, 0.984019, 0.178064, 0, 0.984019, 0.178064, 0, 0.984019, 0.178064, 0, 0.984019, 0.178064, 0, 0.984019, 0.178064, 0, 0.984019, 0.178064, 0, 0.986655, 0.162823, 0, 0.986655, 0.162823, 0, 0.986655, 0.162823, 0, 0.986655, 0.162823, 0, 0.986655, 0.162823, 0, 0.986655, 0.162823, 0, 0.989289, 0.145968, 0, 0.989289, 0.145968, 0, 0.989289, 0.145968, 0, 0.989289, 0.145968, 0, 0.989289, 0.145968, 0, 0.989289, 0.145968, 0, 0.991856, 0.127363, 0, 0.991856, 0.127363, 0, 0.991856, 0.127363, 0, 0.991856, 0.127363, 0, 0.991856, 0.127363, 0, 0.991856, 0.127363, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.991054, 0.133458, 0, 0.991054, 0.133458, 0, 0.991054, 0.133458, 0, 0.991054, 0.133458, 0, 0.991054, 0.133458, 0, 0.991054, 0.133458, 0, 0.989461, 0.144799, 0, 0.989461, 0.144799, 0, 0.989461, 0.144799, 0, 0.989461, 0.144799, 0, 0.989461, 0.144799, 0, 0.989461, 0.144799, 0, 0.987894, 0.155129, 0, 0.987894, 0.155129, 0, 0.987894, 0.155129, 0, 0.987894, 0.155129, 0, 0.987894, 0.155129, 0, 0.987894, 0.155129, 0, 0.986377, 0.164502, 0, 0.986377, 0.164502, 0, 0.986377, 0.164502, 0, 0.986377, 0.164502, 0, 0.986377, 0.164502, 0, 0.986377, 0.164502, 0, 0.984992, 0.172599, 0, 0.984992, 0.172599, 0, 0.984992, 0.172599, 0, 0.984992, 0.172599, 0, 0.984992, 0.172599, 0, 0.984992, 0.172599, 0, 0.983836, 0.179072, 0, 0.983836, 0.179072, 0, 0.983836, 0.179072, 0, 0.983836, 0.179072, 0, 0.983836, 0.179072, 0, 0.983836, 0.179072, 0, 0.982942, 0.183917, 0, 0.982942, 0.183917, 0, 0.982942, 0.183917, 0, 0.982942, 0.183917, 0, 0.982942, 0.183917, 0, 0.982942, 0.183917, 0, 0.982339, 0.187112, 0, 0.982339, 0.187112, 0, 0.982339, 0.187112, 0, 0.982339, 0.187112, 0, 0.982339, 0.187112, 0, 0.982339, 0.187112, 0, 0.982032, 0.188715, 0, 0.982032, 0.188715, 0, 0.982032, 0.188715, 0, 0.982032, 0.188715, 0, 0.982032, 0.188715, 0, 0.982032, 0.188715, 0, 0.982146, 0.188122, 0, 0.982146, 0.188122, 0, 0.982146, 0.188122, 0, 0.982146, 0.188122, 0, 0.982146, 0.188122, 0, 0.982146, 0.188122, 0, 0.982691, 0.185250, 0, 0.982691, 0.185250, 0, 0.982691, 0.185250, 0, 0.982691, 0.185250, 0, 0.982691, 0.185250, 0, 0.982691, 0.185250, 0, 0.983663, 0.180019, 0, 0.983663, 0.180019, 0, 0.983663, 0.180019, 0, 0.983663, 0.180019, 0, 0.983663, 0.180019, 0, 0.983663, 0.180019, 0, 0.985008, 0.172509, 0, 0.985008, 0.172509, 0, 0.985008, 0.172509, 0, 0.985008, 0.172509, 0, 0.985008, 0.172509, 0, 0.985008, 0.172509, 0, 0.986655, 0.162824, 0, 0.986655, 0.162824, 0, 0.986655, 0.162824, 0, 0.986655, 0.162824, 0, 0.986655, 0.162824, 0, 0.986655, 0.162824, 0, 0.988541, 0.150952, 0, 0.988541, 0.150952, 0, 0.988541, 0.150952, 0, 0.988541, 0.150952, 0, 0.988541, 0.150952, 0, 0.988541, 0.150952, 0, 0.990584, 0.136907, 0, 0.990584, 0.136907, 0, 0.990584, 0.136907, 0, 0.990584, 0.136907, 0, 0.990584, 0.136907, 0, 0.990584, 0.136907, 0, 0.990218, 0.139529, 0, 0.990218, 0.139529, 0, 0.990218, 0.139529, 0, 0.990218, 0.139529, 0, 0.990218, 0.139529, 0, 0.990218, 0.139529, 0, 0.990471, 0.137724, 0, 0.990471, 0.137724, 0, 0.990471, 0.137724, 0, 0.990471, 0.137724, 0, 0.990471, 0.137724, 0, 0.990471, 0.137724, 0, 0.991023, 0.133691, 0, 0.991023, 0.133691, 0, 0.991023, 0.133691, 0, 0.991023, 0.133691, 0, 0.991023, 0.133691, 0, 0.991023, 0.133691, 0, 0.991854, 0.127379, 0, 0.991854, 0.127379, 0, 0.991854, 0.127379, 0, 0.991854, 0.127379, 0, 0.991854, 0.127379, 0, 0.991854, 0.127379, 0, 0.992895, 0.118990, 0, 0.992895, 0.118990, 0, 0.992895, 0.118990, 0, 0.992895, 0.118990, 0, 0.992895, 0.118990, 0, 0.992895, 0.118990, 0, 0.994069, 0.108755, 0, 0.994069, 0.108755, 0, 0.994069, 0.108755, 0, 0.994069, 0.108755, 0, 0.994069, 0.108755, 0, 0.994069, 0.108755, 0, 0.995315, 0.096686, 0, 0.995315, 0.096686, 0, 0.995315, 0.096686, 0, 0.995315, 0.096686, 0, 0.995315, 0.096686, 0, 0.995315, 0.096686, 0, 0.996565, 0.082813, 0, 0.996565, 0.082813, 0, 0.996565, 0.082813, 0, 0.996565, 0.082813, 0, 0.996565, 0.082813, 0, 0.996565, 0.082813, 0, 0.997633, 0.068760, 0, 0.997633, 0.068760, 0, 0.997633, 0.068760, 0, 0.997633, 0.068760, 0, 0.997633, 0.068760, 0, 0.997633, 0.068760, 0, 0.998421, 0.056177, 0, 0.998421, 0.056177, 0, 0.998421, 0.056177, 0, 0.998421, 0.056177, 0, 0.998421, 0.056177, 0, 0.998421, 0.056177, 0, 0.998981, 0.045127, 0, 0.998981, 0.045127, 0, 0.998981, 0.045127, 0, 0.998981, 0.045127, 0, 0.998981, 0.045127, 0, 0.998981, 0.045127, 0, 0.999364, 0.035658, 0, 0.999364, 0.035658, 0, 0.999364, 0.035658, 0, 0.999364, 0.035658, 0, 0.999364, 0.035658, 0, 0.999364, 0.035658, 0, 0.999606, 0.028073, 0, 0.999606, 0.028073, 0, 0.999606, 0.028073, 0, 0.999606, 0.028073, 0, 0.999606, 0.028073, 0, 0.999606, 0.028073, 0, 0.999744, 0.022645, 0, 0.999744, 0.022645, 0, 0.999744, 0.022645, 0, 0.999744, 0.022645, 0, 0.999744, 0.022645, 0, 0.999744, 0.022645, 0, 0.999813, 0.019345, 0, 0.999813, 0.019345, 0, 0.999813, 0.019345, 0, 0.999813, 0.019345, 0, 0.999813, 0.019345, 0, 0.999813, 0.019345, 0, 0.999836, 0.018130, 0, 0.999836, 0.018130, 0, 0.999836, 0.018130, 0, 0.999836, 0.018130, 0, 0.999836, 0.018130, 0, 0.999836, 0.018130, 0, 0.999820, 0.018969, 0, 0.999820, 0.018969, 0, 0.999820, 0.018969, 0, 0.999820, 0.018969, 0, 0.999820, 0.018969, 0, 0.999820, 0.018969, 0, 0.999762, 0.021828, 0, 0.999762, 0.021828, 0, 0.999762, 0.021828, 0, 0.999762, 0.021828, 0, 0.999762, 0.021828, 0, 0.999762, 0.021828, 0, 0.999644, 0.026663, 0, 0.999644, 0.026663, 0, 0.999644, 0.026663, 0, 0.999644, 0.026663, 0, 0.999644, 0.026663, 0, 0.999644, 0.026663, 0, 0.999441, 0.033443, 0, 0.999441, 0.033443, 0, 0.999441, 0.033443, 0, 0.999441, 0.033443, 0, 0.999441, 0.033443, 0, 0.999441, 0.033443, 0, 0.999123, 0.041882, 0, 0.999123, 0.041882, 0, 0.999123, 0.041882, 0, 0.999123, 0.041882, 0, 0.999123, 0.041882, 0, 0.999123, 0.041882, 0, 0.998662, 0.051722, 0, 0.998662, 0.051722, 0, 0.998662, 0.051722, 0, 0.998662, 0.051722, 0, 0.998662, 0.051722, 0, 0.998662, 0.051722, 0, 0.998015, 0.062980, 0, 0.998015, 0.062980, 0, 0.998015, 0.062980, 0, 0.998015, 0.062980, 0, 0.998015, 0.062980, 0, 0.998015, 0.062980, 0, 0.997132, 0.075684, 0, 0.997132, 0.075684, 0, 0.997132, 0.075684, 0, 0.997132, 0.075684, 0, 0.997132, 0.075684, 0, 0.997132, 0.075684, 0, 0.996085, 0.088406, 0, 0.996085, 0.088406, 0, 0.996085, 0.088406, 0, 0.996085, 0.088406, 0, 0.996085, 0.088406, 0, 0.996085, 0.088406, 0, 0.995001, 0.099869, 0, 0.995001, 0.099869, 0, 0.995001, 0.099869, 0, 0.995001, 0.099869, 0, 0.995001, 0.099869, 0, 0.995001, 0.099869, 0, 0.993918, 0.110122, 0, 0.993918, 0.110122, 0, 0.993918, 0.110122, 0, 0.993918, 0.110122, 0, 0.993918, 0.110122, 0, 0.993918, 0.110122, 0, 0.992869, 0.119207, 0, 0.992869, 0.119207, 0, 0.992869, 0.119207, 0, 0.992869, 0.119207, 0, 0.992869, 0.119207, 0, 0.992869, 0.119207, 0, 0.991923, 0.126841, 0, 0.991923, 0.126841, 0, 0.991923, 0.126841, 0, 0.991923, 0.126841, 0, 0.991923, 0.126841, 0, 0.991923, 0.126841, 0, 0.991154, 0.132720, 0, 0.991154, 0.132720, 0, 0.991154, 0.132720, 0, 0.991154, 0.132720, 0, 0.991154, 0.132720, 0, 0.991154, 0.132720, 0, 0.990594, 0.136835, 0, 0.990594, 0.136835, 0, 0.990594, 0.136835, 0, 0.990594, 0.136835, 0, 0.990594, 0.136835, 0, 0.990594, 0.136835, 0, 0.987557, 0.157262, 0, 0.987557, 0.157262, 0, 0.987557, 0.157262, 0, 0.987557, 0.157262, 0, 0.987557, 0.157262, 0, 0.987557, 0.157262, 0, 0.988498, 0.151232, 0, 0.988498, 0.151232, 0, 0.988498, 0.151232, 0, 0.988498, 0.151232, 0, 0.988498, 0.151232, 0, 0.988498, 0.151232, 0, 0.989720, 0.143016, 0, 0.989720, 0.143016, 0, 0.989720, 0.143016, 0, 0.989720, 0.143016, 0, 0.989720, 0.143016, 0, 0.989720, 0.143016, 0, 0.991166, 0.132626, 0, 0.991166, 0.132626, 0, 0.991166, 0.132626, 0, 0.991166, 0.132626, 0, 0.991166, 0.132626, 0, 0.991166, 0.132626, 0, 0.992728, 0.120376, 0, 0.992728, 0.120376, 0, 0.992728, 0.120376, 0, 0.992728, 0.120376, 0, 0.992728, 0.120376, 0, 0.992728, 0.120376, 0, 0.994309, 0.106537, 0, 0.994309, 0.106537, 0, 0.994309, 0.106537, 0, 0.994309, 0.106537, 0, 0.994309, 0.106537, 0, 0.994309, 0.106537, 0, 0.995848, 0.091027, 0, 0.995848, 0.091027, 0, 0.995848, 0.091027, 0, 0.995848, 0.091027, 0, 0.995848, 0.091027, 0, 0.995848, 0.091027, 0, 0.997277, 0.073746, 0, 0.997277, 0.073746, 0, 0.997277, 0.073746, 0, 0.997277, 0.073746, 0, 0.997277, 0.073746, 0, 0.997277, 0.073746, 0, 0.998396, 0.056618, 0, 0.998396, 0.056618, 0, 0.998396, 0.056618, 0, 0.998396, 0.056618, 0, 0.998396, 0.056618, 0, 0.998396, 0.056618, 0, 0.999140, 0.041467, 0, 0.999140, 0.041467, 0, 0.999140, 0.041467, 0, 0.999140, 0.041467, 0, 0.999140, 0.041467, 0, 0.999140, 0.041467, 0, 0.999601, 0.028233, 0, 0.999601, 0.028233, 0, 0.999601, 0.028233, 0, 0.999601, 0.028233, 0, 0.999601, 0.028233, 0, 0.999601, 0.028233, 0, 0.999858, 0.016875, 0, 0.999858, 0.016875, 0, 0.999858, 0.016875, 0, 0.999858, 0.016875, 0, 0.999858, 0.016875, 0, 0.999858, 0.016875, 0, 0.999970, 0.007688, 0, 0.999970, 0.007688, 0, 0.999970, 0.007688, 0, 0.999970, 0.007688, 0, 0.999970, 0.007688, 0, 0.999970, 0.007688, 0, 1.000000, 0.001014, 0, 1.000000, 0.001014, 0, 1.000000, 0.001014, 0, 1.000000, 0.001014, 0, 1.000000, 0.001014, 0, 1.000000, 0.001014, 0, 0.999995, -0.003089, 0, 0.999995, -0.003089, 0, 0.999995, -0.003089, 0, 0.999995, -0.003089, 0, 0.999995, -0.003089, 0, 0.999995, -0.003089, 0, 0.999990, -0.004533, 0, 0.999990, -0.004533, 0, 0.999990, -0.004533, 0, 0.999990, -0.004533, 0, 0.999990, -0.004533, 0, 0.999990, -0.004533, 0, 0.999995, -0.003190, 0, 0.999995, -0.003190, 0, 0.999995, -0.003190, 0, 0.999995, -0.003190, 0, 0.999995, -0.003190, 0, 0.999995, -0.003190, 0, 0.999999, 0.001078, 0, 0.999999, 0.001078, 0, 0.999999, 0.001078, 0, 0.999999, 0.001078, 0, 0.999999, 0.001078, 0, 0.999999, 0.001078, 0, 0.999965, 0.008376, 0, 0.999965, 0.008376, 0, 0.999965, 0.008376, 0, 0.999965, 0.008376, 0, 0.999965, 0.008376, 0, 0.999965, 0.008376, 0, 0.999824, 0.018780, 0, 0.999824, 0.018780, 0, 0.999824, 0.018780, 0, 0.999824, 0.018780, 0, 0.999824, 0.018780, 0, 0.999824, 0.018780, 0, 0.999489, 0.031957, 0, 0.999489, 0.031957, 0, 0.999489, 0.031957, 0, 0.999489, 0.031957, 0, 0.999489, 0.031957, 0, 0.999489, 0.031957, 0, 0.998871, 0.047502, 0, 0.998871, 0.047502, 0, 0.998871, 0.047502, 0, 0.998871, 0.047502, 0, 0.998871, 0.047502, 0, 0.998871, 0.047502, 0, 0.997865, 0.065316, 0, 0.997865, 0.065316, 0, 0.997865, 0.065316, 0, 0.997865, 0.065316, 0, 0.997865, 0.065316, 0, 0.997865, 0.065316, 0, 0.996360, 0.085250, 0, 0.996360, 0.085250, 0, 0.996360, 0.085250, 0, 0.996360, 0.085250, 0, 0.996360, 0.085250, 0, 0.996360, 0.085250, 0, 0.994480, 0.104924, 0, 0.994480, 0.104924, 0, 0.994480, 0.104924, 0, 0.994480, 0.104924, 0, 0.994480, 0.104924, 0, 0.994480, 0.104924, 0, 0.992547, 0.121860, 0, 0.992547, 0.121860, 0, 0.992547, 0.121860, 0, 0.992547, 0.121860, 0, 0.992547, 0.121860, 0, 0.992547, 0.121860, 0, 0.990710, 0.135990, 0, 0.990710, 0.135990, 0, 0.990710, 0.135990, 0, 0.990710, 0.135990, 0, 0.990710, 0.135990, 0, 0.990710, 0.135990, 0, 0.989093, 0.147294, 0, 0.989093, 0.147294, 0, 0.989093, 0.147294, 0, 0.989093, 0.147294, 0, 0.989093, 0.147294, 0, 0.989093, 0.147294, 0, 0.987833, 0.155521, 0, 0.987833, 0.155521, 0, 0.987833, 0.155521, 0, 0.987833, 0.155521, 0, 0.987833, 0.155521, 0, 0.987833, 0.155521, 0, 0.987040, 0.160478, 0, 0.987040, 0.160478, 0, 0.987040, 0.160478, 0, 0.987040, 0.160478, 0, 0.987040, 0.160478, 0, 0.987040, 0.160478, 0, 0.986745, 0.162281, 0, 0.986745, 0.162281, 0, 0.986745, 0.162281, 0, 0.986745, 0.162281, 0, 0.986745, 0.162281, 0, 0.986745, 0.162281, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.990188, 0.139744, 0, 0.990188, 0.139744, 0, 0.990188, 0.139744, 0, 0.990188, 0.139744, 0, 0.990188, 0.139744, 0, 0.990188, 0.139744, 0, 0.999764, 0.021718, 0, 0.999764, 0.021718, 0, 0.999764, 0.021718, 0, 0.999764, 0.021718, 0, 0.999764, 0.021718, 0, 0.999764, 0.021718, 0, 0.991695, 0.128613, 0, 0.991695, 0.128613, 0, 0.991695, 0.128613, 0, 0.991695, 0.128613, 0, 0.991695, 0.128613, 0, 0.991695, 0.128613, 0, 0.992053, 0.125818, 0, 0.992053, 0.125818, 0, 0.992053, 0.125818, 0, 0.992053, 0.125818, 0, 0.992053, 0.125818, 0, 0.992053, 0.125818, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.998704, 0.050894, 0, 0.998704, 0.050894, 0, 0.998704, 0.050894, 0, 0.998704, 0.050894, 0, 0.998704, 0.050894, 0, 0.998704, 0.050894, 0, 0.998455, 0.055560, 0, 0.998455, 0.055560, 0, 0.998455, 0.055560, 0, 0.998455, 0.055560, 0, 0.998455, 0.055560, 0, 0.998455, 0.055560, 0, 0.998943, 0.045956, 0, 0.998943, 0.045956, 0, 0.998943, 0.045956, 0, 0.998943, 0.045956, 0, 0.998943, 0.045956, 0, 0.998943, 0.045956, 0, 0.999296, 0.037525, 0, 0.999296, 0.037525, 0, 0.999296, 0.037525, 0, 0.999296, 0.037525, 0, 0.999296, 0.037525, 0, 0.999296, 0.037525, 0, 0.999541, 0.030294, 0, 0.999541, 0.030294, 0, 0.999541, 0.030294, 0, 0.999541, 0.030294, 0, 0.999541, 0.030294, 0, 0.999541, 0.030294, 0, 0.999705, 0.024280, 0, 0.999705, 0.024280, 0, 0.999705, 0.024280, 0, 0.999705, 0.024280, 0, 0.999705, 0.024280, 0, 0.999705, 0.024280, 0, 0.999809, 0.019551, 0, 0.999809, 0.019551, 0, 0.999809, 0.019551, 0, 0.999809, 0.019551, 0, 0.999809, 0.019551, 0, 0.999809, 0.019551, 0, 0.999869, 0.016162, 0, 0.999869, 0.016162, 0, 0.999869, 0.016162, 0, 0.999869, 0.016162, 0, 0.999869, 0.016162, 0, 0.999869, 0.016162, 0, 0.999901, 0.014097, 0, 0.999901, 0.014097, 0, 0.999901, 0.014097, 0, 0.999901, 0.014097, 0, 0.999901, 0.014097, 0, 0.999901, 0.014097, 0, 0.999911, 0.013331, 0, 0.999911, 0.013331, 0, 0.999911, 0.013331, 0, 0.999911, 0.013331, 0, 0.999911, 0.013331, 0, 0.999911, 0.013331, 0, 0.999910, 0.013413, 0, 0.999910, 0.013413, 0, 0.999910, 0.013413, 0, 0.999910, 0.013413, 0, 0.999910, 0.013413, 0, 0.999910, 0.013413, 0, 0.999905, 0.013805, 0, 0.999905, 0.013805, 0, 0.999905, 0.013805, 0, 0.999905, 0.013805, 0, 0.999905, 0.013805, 0, 0.999905, 0.013805, 0, 0.999895, 0.014502, 0, 0.999895, 0.014502, 0, 0.999895, 0.014502, 0, 0.999895, 0.014502, 0, 0.999895, 0.014502, 0, 0.999895, 0.014502, 0, 0.999880, 0.015498, 0, 0.999880, 0.015498, 0, 0.999880, 0.015498, 0, 0.999880, 0.015498, 0, 0.999880, 0.015498, 0, 0.999880, 0.015498, 0, 0.999859, 0.016789, 0, 0.999859, 0.016789, 0, 0.999859, 0.016789, 0, 0.999859, 0.016789, 0, 0.999859, 0.016789, 0, 0.999859, 0.016789, 0, 0.999831, 0.018370, 0, 0.999831, 0.018370, 0, 0.999831, 0.018370, 0, 0.999831, 0.018370, 0, 0.999831, 0.018370, 0, 0.999831, 0.018370, 0, 0.999795, 0.020237, 0, 0.999795, 0.020237, 0, 0.999795, 0.020237, 0, 0.999795, 0.020237, 0, 0.999795, 0.020237, 0, 0.999795, 0.020237, 0, 0.999749, 0.022385, 0, 0.999749, 0.022385, 0, 0.999749, 0.022385, 0, 0.999749, 0.022385, 0, 0.999749, 0.022385, 0, 0.999749, 0.022385, 0, 0.999980, 0.006397, 0, 0.999980, 0.006397, 0, 0.999980, 0.006397, 0, 0.999980, 0.006397, 0, 0.999980, 0.006397, 0, 0.999980, 0.006397, 0, 0.999991, 0.004210, 0, 0.999991, 0.004210, 0, 0.999991, 0.004210, 0, 0.999991, 0.004210, 0, 0.999991, 0.004210, 0, 0.999991, 0.004210, 0, 0.999997, 0.002284, 0, 0.999997, 0.002284, 0, 0.999997, 0.002284, 0, 0.999997, 0.002284, 0, 0.999997, 0.002284, 0, 0.999997, 0.002284, 0, 1.000000, 0.000625, 0, 1.000000, 0.000625, 0, 1.000000, 0.000625, 0, 1.000000, 0.000625, 0, 1.000000, 0.000625, 0, 1.000000, 0.000625, 0, 1.000000, -0.000765, 0, 1.000000, -0.000765, 0, 1.000000, -0.000765, 0, 1.000000, -0.000765, 0, 1.000000, -0.000765, 0, 1.000000, -0.000765, 0, 0.999998, -0.001869, 0, 0.999998, -0.001869, 0, 0.999998, -0.001869, 0, 0.999998, -0.001869, 0, 0.999998, -0.001869, 0, 0.999998, -0.001869, 0, 0.999996, -0.002669, 0, 0.999996, -0.002669, 0, 0.999996, -0.002669, 0, 0.999996, -0.002669, 0, 0.999996, -0.002669, 0, 0.999996, -0.002669, 0, 0.999995, -0.003160, 0, 0.999995, -0.003160, 0, 0.999995, -0.003160, 0, 0.999995, -0.003160, 0, 0.999995, -0.003160, 0, 0.999995, -0.003160, 0, 0.999994, -0.003337, 0, 0.999994, -0.003337, 0, 0.999994, -0.003337, 0, 0.999994, -0.003337, 0, 0.999994, -0.003337, 0, 0.999994, -0.003337, 0, 0.999997, -0.002345, 0, 0.999997, -0.002345, 0, 0.999997, -0.002345, 0, 0.999997, -0.002345, 0, 0.999997, -0.002345, 0, 0.999997, -0.002345, 0, 1.000000, 0.000775, 0, 1.000000, 0.000775, 0, 1.000000, 0.000775, 0, 1.000000, 0.000775, 0, 1.000000, 0.000775, 0, 1.000000, 0.000775, 0, 0.999982, 0.006087, 0, 0.999982, 0.006087, 0, 0.999982, 0.006087, 0, 0.999982, 0.006087, 0, 0.999982, 0.006087, 0, 0.999982, 0.006087, 0, 0.999907, 0.013637, 0, 0.999907, 0.013637, 0, 0.999907, 0.013637, 0, 0.999907, 0.013637, 0, 0.999907, 0.013637, 0, 0.999907, 0.013637, 0, 0.999733, 0.023126, 0, 0.999733, 0.023126, 0, 0.999733, 0.023126, 0, 0.999733, 0.023126, 0, 0.999733, 0.023126, 0, 0.999733, 0.023126, 0, 0.999415, 0.034214, 0, 0.999415, 0.034214, 0, 0.999415, 0.034214, 0, 0.999415, 0.034214, 0, 0.999415, 0.034214, 0, 0.999415, 0.034214, 0, 0.998902, 0.046844, 0, 0.998902, 0.046844, 0, 0.998902, 0.046844, 0, 0.998902, 0.046844, 0, 0.998902, 0.046844, 0, 0.998902, 0.046844, 0, 0.998142, 0.060937, 0, 0.998142, 0.060937, 0, 0.998142, 0.060937, 0, 0.998142, 0.060937, 0, 0.998142, 0.060937, 0, 0.998142, 0.060937, 0, 0.997158, 0.075337, 0, 0.997158, 0.075337, 0, 0.997158, 0.075337, 0, 0.997158, 0.075337, 0, 0.997158, 0.075337, 0, 0.997158, 0.075337, 0, 0.996128, 0.087914, 0, 0.996128, 0.087914, 0, 0.996128, 0.087914, 0, 0.996128, 0.087914, 0, 0.996128, 0.087914, 0, 0.996128, 0.087914, 0, 0.995125, 0.098620, 0, 0.995125, 0.098620, 0, 0.995125, 0.098620, 0, 0.995125, 0.098620, 0, 0.995125, 0.098620, 0, 0.995125, 0.098620, 0, 0.994213, 0.107428, 0, 0.994213, 0.107428, 0, 0.994213, 0.107428, 0, 0.994213, 0.107428, 0, 0.994213, 0.107428, 0, 0.994213, 0.107428, 0, 0.993474, 0.114056, 0, 0.993474, 0.114056, 0, 0.993474, 0.114056, 0, 0.993474, 0.114056, 0, 0.993474, 0.114056, 0, 0.993474, 0.114056, 0, 0.992983, 0.118259, 0, 0.992983, 0.118259, 0, 0.992983, 0.118259, 0, 0.992983, 0.118259, 0, 0.992983, 0.118259, 0, 0.992983, 0.118259, 0, 0.992762, 0.120102, 0, 0.992762, 0.120102, 0, 0.992762, 0.120102, 0, 0.992762, 0.120102, 0, 0.992762, 0.120102, 0, 0.992762, 0.120102, 0, 0.992815, 0.119661, 0, 0.992815, 0.119661, 0, 0.992815, 0.119661, 0, 0.992815, 0.119661, 0, 0.992815, 0.119661, 0, 0.992815, 0.119661, 0, 0.993081, 0.117434, 0, 0.993081, 0.117434, 0, 0.993081, 0.117434, 0, 0.993081, 0.117434, 0, 0.993081, 0.117434, 0, 0.993081, 0.117434, 0, 0.993530, 0.113568, 0, 0.993530, 0.113568, 0, 0.993530, 0.113568, 0, 0.993530, 0.113568, 0, 0.993530, 0.113568, 0, 0.993530, 0.113568, 0, 0.994141, 0.108091, 0, 0.994141, 0.108091, 0, 0.994141, 0.108091, 0, 0.994141, 0.108091, 0, 0.994141, 0.108091, 0, 0.994141, 0.108091, 0, 0.994885, 0.101014, 0, 0.994885, 0.101014, 0, 0.994885, 0.101014, 0, 0.994885, 0.101014, 0, 0.994885, 0.101014, 0, 0.994885, 0.101014, 0, 0.995702, 0.092612, 0, 0.995702, 0.092612, 0, 0.995702, 0.092612, 0, 0.995702, 0.092612, 0, 0.995702, 0.092612, 0, 0.995702, 0.092612, 0, 0.996538, 0.083137, 0, 0.996538, 0.083137, 0, 0.996538, 0.083137, 0, 0.996538, 0.083137, 0, 0.996538, 0.083137, 0, 0.996538, 0.083137, 0, 0.997365, 0.072550, 0, 0.997365, 0.072550, 0, 0.997365, 0.072550, 0, 0.997365, 0.072550, 0, 0.997365, 0.072550, 0, 0.997365, 0.072550, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0.997184, 0.074995, 0, 0.997184, 0.074995, 0, 0.997184, 0.074995, 0, 0.997184, 0.074995, 0, 0.997184, 0.074995, 0, 0.997184, 0.074995, 0, 0.996685, 0.081353, 0, 0.996685, 0.081353, 0, 0.996685, 0.081353, 0, 0.996685, 0.081353, 0, 0.996685, 0.081353, 0, 0.996685, 0.081353, 0, 0.996202, 0.087069, 0, 0.996202, 0.087069, 0, 0.996202, 0.087069, 0, 0.996202, 0.087069, 0, 0.996202, 0.087069, 0, 0.996202, 0.087069, 0, 0.995745, 0.092156, 0, 0.995745, 0.092156, 0, 0.995745, 0.092156, 0, 0.995745, 0.092156, 0, 0.995745, 0.092156, 0, 0.995745, 0.092156, 0, 0.995340, 0.096433, 0, 0.995340, 0.096433, 0, 0.995340, 0.096433, 0, 0.995340, 0.096433, 0, 0.995340, 0.096433, 0, 0.995340, 0.096433, 0, 0.995017, 0.099706, 0, 0.995017, 0.099706, 0, 0.995017, 0.099706, 0, 0.995017, 0.099706, 0, 0.995017, 0.099706, 0, 0.995017, 0.099706, 0, 0.994788, 0.101969, 0, 0.994788, 0.101969, 0, 0.994788, 0.101969, 0, 0.994788, 0.101969, 0, 0.994788, 0.101969, 0, 0.994788, 0.101969, 0, 0.994659, 0.103212, 0, 0.994659, 0.103212, 0, 0.994659, 0.103212, 0, 0.994659, 0.103212, 0, 0.994659, 0.103212, 0, 0.994659, 0.103212, 0, 0.994634, 0.103454, 0, 0.994634, 0.103454, 0, 0.994634, 0.103454, 0, 0.994634, 0.103454, 0, 0.994634, 0.103454, 0, 0.994634, 0.103454, 0, 0.994731, 0.102520, 0, 0.994731, 0.102520, 0, 0.994731, 0.102520, 0, 0.994731, 0.102520, 0, 0.994731, 0.102520, 0, 0.994731, 0.102520, 0, 0.994949, 0.100386, 0, 0.994949, 0.100386, 0, 0.994949, 0.100386, 0, 0.994949, 0.100386, 0, 0.994949, 0.100386, 0, 0.994949, 0.100386, 0, 0.995281, 0.097033, 0, 0.995281, 0.097033, 0, 0.995281, 0.097033, 0, 0.995281, 0.097033, 0, 0.995281, 0.097033, 0, 0.995281, 0.097033, 0, 0.995710, 0.092534, 0, 0.995710, 0.092534, 0, 0.995710, 0.092534, 0, 0.995710, 0.092534, 0, 0.995710, 0.092534, 0, 0.995710, 0.092534, 0, 0.996211, 0.086974, 0, 0.996211, 0.086974, 0, 0.996211, 0.086974, 0, 0.996211, 0.086974, 0, 0.996211, 0.086974, 0, 0.996211, 0.086974, 0, 0.996766, 0.080355, 0, 0.996766, 0.080355, 0, 0.996766, 0.080355, 0, 0.996766, 0.080355, 0, 0.996766, 0.080355, 0, 0.996766, 0.080355, 0, 0.997355, 0.072687, 0, 0.997355, 0.072687, 0, 0.997355, 0.072687, 0, 0.997355, 0.072687, 0, 0.997355, 0.072687, 0, 0.997355, 0.072687, 0, 0.996132, 0.087871, 0, 0.996132, 0.087871, 0, 0.996132, 0.087871, 0, 0.996132, 0.087871, 0, 0.996132, 0.087871, 0, 0.996132, 0.087871, 0, 0.996067, 0.088599, 0, 0.996067, 0.088599, 0, 0.996067, 0.088599, 0, 0.996067, 0.088599, 0, 0.996067, 0.088599, 0, 0.996067, 0.088599, 0, 0.996011, 0.089236, 0, 0.996011, 0.089236, 0, 0.996011, 0.089236, 0, 0.996011, 0.089236, 0, 0.996011, 0.089236, 0, 0.996011, 0.089236, 0, 0.995962, 0.089781, 0, 0.995962, 0.089781, 0, 0.995962, 0.089781, 0, 0.995962, 0.089781, 0, 0.995962, 0.089781, 0, 0.995962, 0.089781, 0, 0.995920, 0.090241, 0, 0.995920, 0.090241, 0, 0.995920, 0.090241, 0, 0.995920, 0.090241, 0, 0.995920, 0.090241, 0, 0.995920, 0.090241, 0, 0.995885, 0.090623, 0, 0.995885, 0.090623, 0, 0.995885, 0.090623, 0, 0.995885, 0.090623, 0, 0.995885, 0.090623, 0, 0.995885, 0.090623, 0, 0.995858, 0.090926, 0, 0.995858, 0.090926, 0, 0.995858, 0.090926, 0, 0.995858, 0.090926, 0, 0.995858, 0.090926, 0, 0.995858, 0.090926, 0, 0.995837, 0.091149, 0, 0.995837, 0.091149, 0, 0.995837, 0.091149, 0, 0.995837, 0.091149, 0, 0.995837, 0.091149, 0, 0.995837, 0.091149, 0, 0.995834, 0.091182, 0, 0.995834, 0.091182, 0, 0.995834, 0.091182, 0, 0.995834, 0.091182, 0, 0.995834, 0.091182, 0, 0.995834, 0.091182, 0, 0.995945, 0.089966, 0, 0.995945, 0.089966, 0, 0.995945, 0.089966, 0, 0.995945, 0.089966, 0, 0.995945, 0.089966, 0, 0.995945, 0.089966, 0, 0.996166, 0.087478, 0, 0.996166, 0.087478, 0, 0.996166, 0.087478, 0, 0.996166, 0.087478, 0, 0.996166, 0.087478, 0, 0.996166, 0.087478, 0, 0.996491, 0.083703, 0, 0.996491, 0.083703, 0, 0.996491, 0.083703, 0, 0.996491, 0.083703, 0, 0.996491, 0.083703, 0, 0.996491, 0.083703, 0, 0.996898, 0.078704, 0, 0.996898, 0.078704, 0, 0.996898, 0.078704, 0, 0.996898, 0.078704, 0, 0.996898, 0.078704, 0, 0.996898, 0.078704, 0, 0.997365, 0.072554, 0, 0.997365, 0.072554, 0, 0.997365, 0.072554, 0, 0.997365, 0.072554, 0, 0.997365, 0.072554, 0, 0.997365, 0.072554, 0, 0.997869, 0.065257, 0, 0.997869, 0.065257, 0, 0.997869, 0.065257, 0, 0.997869, 0.065257, 0, 0.997869, 0.065257, 0, 0.997869, 0.065257, 0, 0.998384, 0.056827, 0, 0.998384, 0.056827, 0, 0.998384, 0.056827, 0, 0.998384, 0.056827, 0, 0.998384, 0.056827, 0, 0.998384, 0.056827, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, 0.994174, 0.107790, 0, 0.994174, 0.107790, 0, 0.994174, 0.107790, 0, 0.994174, 0.107790, 0, 0.994174, 0.107790, 0, 0.994174, 0.107790, 0, 0.994375, 0.105915, 0, 0.994375, 0.105915, 0, 0.994375, 0.105915, 0, 0.994375, 0.105915, 0, 0.994375, 0.105915, 0, 0.994375, 0.105915, 0, 0.995858, 0.090920, 0, 0.995858, 0.090920, 0, 0.995858, 0.090920, 0, 0.995858, 0.090920, 0, 0.995858, 0.090920, 0, 0.995858, 0.090920, 0, 0.995485, 0.094922, 0, 0.995485, 0.094922, 0, 0.995485, 0.094922, 0, 0.995485, 0.094922, 0, 0.995485, 0.094922, 0, 0.995485, 0.094922, 0, 0.995160, 0.098271, 0, 0.995160, 0.098271, 0, 0.995160, 0.098271, 0, 0.995160, 0.098271, 0, 0.995160, 0.098271, 0, 0.995160, 0.098271, 0, 0.994889, 0.100970, 0, 0.994889, 0.100970, 0, 0.994889, 0.100970, 0, 0.994889, 0.100970, 0, 0.994889, 0.100970, 0, 0.994889, 0.100970, 0, 0.994679, 0.103026, 0, 0.994679, 0.103026, 0, 0.994679, 0.103026, 0, 0.994679, 0.103026, 0, 0.994679, 0.103026, 0, 0.994679, 0.103026, 0, 0.994529, 0.104458, 0, 0.994529, 0.104458, 0, 0.994529, 0.104458, 0, 0.994529, 0.104458, 0, 0.994529, 0.104458, 0, 0.994529, 0.104458, 0, 0.994442, 0.105288, 0, 0.994442, 0.105288, 0, 0.994442, 0.105288, 0, 0.994442, 0.105288, 0, 0.994442, 0.105288, 0, 0.994442, 0.105288, 0, 0.994417, 0.105526, 0, 0.994417, 0.105526, 0, 0.994417, 0.105526, 0, 0.994417, 0.105526, 0, 0.994417, 0.105526, 0, 0.994417, 0.105526, 0, 0.994453, 0.105179, 0, 0.994453, 0.105179, 0, 0.994453, 0.105179, 0, 0.994453, 0.105179, 0, 0.994453, 0.105179, 0, 0.994453, 0.105179, 0, 0.994463, 0.105086, 0, 0.994463, 0.105086, 0, 0.994463, 0.105086, 0, 0.994463, 0.105086, 0, 0.994463, 0.105086, 0, 0.994463, 0.105086, 0, 0.994473, 0.104988, 0, 0.994473, 0.104988, 0, 0.994473, 0.104988, 0, 0.994473, 0.104988, 0, 0.994473, 0.104988, 0, 0.994473, 0.104988, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994484, 0.104886, 0, 0.994496, 0.104779, 0, 0.994496, 0.104779, 0, 0.994496, 0.104779, 0, 0.994496, 0.104779, 0, 0.994496, 0.104779, 0, 0.994496, 0.104779, 0, 0.994507, 0.104668, 0, 0.994507, 0.104668, 0, 0.994507, 0.104668, 0, 0.994507, 0.104668, 0, 0.994507, 0.104668, 0, 0.994507, 0.104668, 0, 0.994520, 0.104550, 0, 0.994520, 0.104550, 0, 0.994520, 0.104550, 0, 0.994520, 0.104550, 0, 0.994520, 0.104550, 0, 0.994520, 0.104550, 0, 0.994533, 0.104427, 0, 0.994533, 0.104427, 0, 0.994533, 0.104427, 0, 0.994533, 0.104427, 0, 0.994533, 0.104427, 0, 0.994533, 0.104427, 0, 0.994546, 0.104298, 0, 0.994546, 0.104298, 0, 0.994546, 0.104298, 0, 0.994546, 0.104298, 0, 0.994546, 0.104298, 0, 0.994546, 0.104298, 0, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1, 0, 0, -1, 0, 0, -1, 0, 0, -1, 0, 0, 1, 0, 0, 1, 0, 0, 1 };
    Normal36.vector = m_vector;
    Normal36.vector_length = sizeof(m_vector) / sizeof(float);
    }
    TextureCoordinate16.m_parent = (X3dNode *)&Text_69;
    {
    static float m_point[] = { 0.075140, 0.632334, 0.074666, 0.692064, 0.075140, 0.632334, 0.075140, 0.632334, 0.074666, 0.692064, 0.074666, 0.692064, 0.074666, 0.692064, 0.073243, 0.744659, 0.074666, 0.692064, 0.074666, 0.692064, 0.073243, 0.744659, 0.073243, 0.744659, 0.073243, 0.744659, 0.070872, 0.790119, 0.073243, 0.744659, 0.073243, 0.744659, 0.070872, 0.790119, 0.070872, 0.790119, 0.070872, 0.790119, 0.067553, 0.828444, 0.070872, 0.790119, 0.070872, 0.790119, 0.067553, 0.828444, 0.067553, 0.828444, 0.067553, 0.828444, 0.063377, 0.858857, 0.067553, 0.828444, 0.067553, 0.828444, 0.063377, 0.858857, 0.063377, 0.858857, 0.063377, 0.858857, 0.058436, 0.880580, 0.063377, 0.858857, 0.063377, 0.858857, 0.058436, 0.880580, 0.058436, 0.880580, 0.058436, 0.880580, 0.052731, 0.893614, 0.058436, 0.880580, 0.058436, 0.880580, 0.052731, 0.893614, 0.052731, 0.893614, 0.052731, 0.893614, 0.046261, 0.897959, 0.052731, 0.893614, 0.052731, 0.893614, 0.046261, 0.897959, 0.046261, 0.897959, 0.046261, 0.897959, 0.039832, 0.893614, 0.046261, 0.897959, 0.046261, 0.897959, 0.039832, 0.893614, 0.039832, 0.893614, 0.039832, 0.893614, 0.034156, 0.880580, 0.039832, 0.893614, 0.039832, 0.893614, 0.034156, 0.880580, 0.034156, 0.880580, 0.034156, 0.880580, 0.029232, 0.858857, 0.034156, 0.880580, 0.034156, 0.880580, 0.029232, 0.858857, 0.029232, 0.858857, 0.029232, 0.858857, 0.025062, 0.828444, 0.029232, 0.858857, 0.029232, 0.858857, 0.025062, 0.828444, 0.025062, 0.828444, 0.025062, 0.828444, 0.021743, 0.790119, 0.025062, 0.828444, 0.025062, 0.828444, 0.021743, 0.790119, 0.021743, 0.790119, 0.021743, 0.790119, 0.019371, 0.744659, 0.021743, 0.790119, 0.021743, 0.790119, 0.019371, 0.744659, 0.019371, 0.744659, 0.019371, 0.744659, 0.017949, 0.692064, 0.019371, 0.744659, 0.019371, 0.744659, 0.017949, 0.692064, 0.017949, 0.692064, 0.017949, 0.692064, 0.017474, 0.632334, 0.017949, 0.692064, 0.017949, 0.692064, 0.017474, 0.632334, 0.017474, 0.632334, 0.017474, 0.632334, 0.017949, 0.572883, 0.017474, 0.632334, 0.017474, 0.632334, 0.017949, 0.572883, 0.017949, 0.572883, 0.017949, 0.572883, 0.019371, 0.520488, 0.017949, 0.572883, 0.017949, 0.572883, 0.019371, 0.520488, 0.019371, 0.520488, 0.019371, 0.520488, 0.021743, 0.475147, 0.019371, 0.520488, 0.019371, 0.520488, 0.021743, 0.475147, 0.021743, 0.475147, 0.021743, 0.475147, 0.025062, 0.436862, 0.021743, 0.475147, 0.021743, 0.475147, 0.025062, 0.436862, 0.025062, 0.436862, 0.025062, 0.436862, 0.029232, 0.406449, 0.025062, 0.436862, 0.025062, 0.436862, 0.029232, 0.406449, 0.029232, 0.406449, 0.029232, 0.406449, 0.034156, 0.384726, 0.029232, 0.406449, 0.029232, 0.406449, 0.034156, 0.384726, 0.034156, 0.384726, 0.034156, 0.384726, 0.039832, 0.371692, 0.034156, 0.384726, 0.034156, 0.384726, 0.039832, 0.371692, 0.039832, 0.371692, 0.039832, 0.371692, 0.046261, 0.367347, 0.039832, 0.371692, 0.039832, 0.371692, 0.046261, 0.367347, 0.046261, 0.367347, 0.046261, 0.367347, 0.052731, 0.371692, 0.046261, 0.367347, 0.046261, 0.367347, 0.052731, 0.371692, 0.052731, 0.371692, 0.052731, 0.371692, 0.058436, 0.384726, 0.052731, 0.371692, 0.052731, 0.371692, 0.058436, 0.384726, 0.058436, 0.384726, 0.058436, 0.384726, 0.063377, 0.406449, 0.058436, 0.384726, 0.058436, 0.384726, 0.063377, 0.406449, 0.063377, 0.406449, 0.063377, 0.406449, 0.067553, 0.436862, 0.063377, 0.406449, 0.063377, 0.406449, 0.067553, 0.436862, 0.067553, 0.436862, 0.067553, 0.436862, 0.070872, 0.475147, 0.067553, 0.436862, 0.067553, 0.436862, 0.070872, 0.475147, 0.070872, 0.475147, 0.070872, 0.475147, 0.073243, 0.520488, 0.070872, 0.475147, 0.070872, 0.475147, 0.073243, 0.520488, 0.073243, 0.520488, 0.073243, 0.520488, 0.074666, 0.572883, 0.073243, 0.520488, 0.073243, 0.520488, 0.074666, 0.572883, 0.074666, 0.572883, 0.092063, 0.357462, 0.091333, 0.272840, 0.092063, 0.357462, 0.092063, 0.357462, 0.091333, 0.272840, 0.091333, 0.272840, 0.091333, 0.272840, 0.089143, 0.199777, 0.091333, 0.272840, 0.091333, 0.272840, 0.089143, 0.199777, 0.089143, 0.199777, 0.089143, 0.199777, 0.085493, 0.138273, 0.089143, 0.199777, 0.089143, 0.199777, 0.085493, 0.138273, 0.085493, 0.138273, 0.085493, 0.138273, 0.080383, 0.088329, 0.085493, 0.138273, 0.085493, 0.138273, 0.080383, 0.088329, 0.080383, 0.088329, 0.080383, 0.088329, 0.073766, 0.049685, 0.080383, 0.088329, 0.080383, 0.088329, 0.073766, 0.049685, 0.073766, 0.049685, 0.073766, 0.049685, 0.065598, 0.022082, 0.073766, 0.049685, 0.073766, 0.049685, 0.065598, 0.022082, 0.065598, 0.022082, 0.065598, 0.022082, 0.055878, 0.005521, 0.065598, 0.022082, 0.065598, 0.022082, 0.055878, 0.005521, 0.055878, 0.005521, 0.055878, 0.005521, 0.044606, 0, 0.055878, 0.005521, 0.055878, 0.005521, 0.044606, 0, 0.044606, 0, 0.044606, 0, 0.040209, 0.000628, 0.044606, 0, 0.044606, 0, 0.040209, 0.000628, 0.040209, 0.000628, 0.040209, 0.000628, 0.035938, 0.002511, 0.040209, 0.000628, 0.040209, 0.000628, 0.035938, 0.002511, 0.035938, 0.002511, 0.035938, 0.002511, 0.031793, 0.005650, 0.035938, 0.002511, 0.035938, 0.002511, 0.031793, 0.005650, 0.031793, 0.005650, 0.031793, 0.005650, 0.027775, 0.010045, 0.031793, 0.005650, 0.031793, 0.005650, 0.027775, 0.010045, 0.027775, 0.010045, 0.027775, 0.010045, 0.023849, 0.015735, 0.027775, 0.010045, 0.027775, 0.010045, 0.023849, 0.015735, 0.023849, 0.015735, 0.023849, 0.015735, 0.019981, 0.022760, 0.023849, 0.015735, 0.023849, 0.015735, 0.019981, 0.022760, 0.019981, 0.022760, 0.019981, 0.022760, 0.016170, 0.031120, 0.019981, 0.022760, 0.019981, 0.022760, 0.016170, 0.031120, 0.016170, 0.031120, 0.016170, 0.031120, 0.012416, 0.040816, 0.016170, 0.031120, 0.016170, 0.031120, 0.012416, 0.040816, 0.012416, 0.040816, 0.012416, 0.040816, 0.012416, 0.163265, 0.012416, 0.040816, 0.012416, 0.040816, 0.012416, 0.163265, 0.012416, 0.163265, 0.012416, 0.163265, 0.016129, 0.148657, 0.012416, 0.163265, 0.012416, 0.163265, 0.016129, 0.148657, 0.016129, 0.148657, 0.016129, 0.148657, 0.019820, 0.136081, 0.016129, 0.148657, 0.016129, 0.148657, 0.019820, 0.136081, 0.019820, 0.136081, 0.019820, 0.136081, 0.023487, 0.125538, 0.019820, 0.136081, 0.019820, 0.136081, 0.023487, 0.125538, 0.023487, 0.125538, 0.023487, 0.125538, 0.027131, 0.117028, 0.023487, 0.125538, 0.023487, 0.125538, 0.027131, 0.117028, 0.027131, 0.117028, 0.027131, 0.117028, 0.030782, 0.110471, 0.027131, 0.117028, 0.027131, 0.117028, 0.030782, 0.110471, 0.030782, 0.110471, 0.030782, 0.110471, 0.034466, 0.105788, 0.030782, 0.110471, 0.030782, 0.110471, 0.034466, 0.105788, 0.034466, 0.105788, 0.034466, 0.105788, 0.038185, 0.102978, 0.034466, 0.105788, 0.034466, 0.105788, 0.038185, 0.102978, 0.038185, 0.102978, 0.038185, 0.102978, 0.041939, 0.102041, 0.038185, 0.102978, 0.038185, 0.102978, 0.041939, 0.102041, 0.041939, 0.102041, 0.041939, 0.102041, 0.049739, 0.105668, 0.041939, 0.102041, 0.041939, 0.102041, 0.049739, 0.105668, 0.049739, 0.105668, 0.049739, 0.105668, 0.056493, 0.116550, 0.049739, 0.105668, 0.049739, 0.105668, 0.056493, 0.116550, 0.056493, 0.116550, 0.056493, 0.116550, 0.062201, 0.134686, 0.056493, 0.116550, 0.056493, 0.116550, 0.062201, 0.134686, 0.062201, 0.134686, 0.062201, 0.134686, 0.066863, 0.160077, 0.062201, 0.134686, 0.062201, 0.134686, 0.066863, 0.160077, 0.066863, 0.160077, 0.066863, 0.160077, 0.070484, 0.192801, 0.066863, 0.160077, 0.066863, 0.160077, 0.070484, 0.192801, 0.070484, 0.192801, 0.070484, 0.192801, 0.073071, 0.232940, 0.070484, 0.192801, 0.070484, 0.192801, 0.073071, 0.232940, 0.073071, 0.232940, 0.073071, 0.232940, 0.074623, 0.280493, 0.073071, 0.232940, 0.073071, 0.232940, 0.074623, 0.280493, 0.074623, 0.280493, 0.074623, 0.280493, 0.075140, 0.335459, 0.074623, 0.280493, 0.074623, 0.280493, 0.075140, 0.335459, 0.075140, 0.335459, 0.075140, 0.335459, 0.075140, 0.387755, 0.075140, 0.335459, 0.075140, 0.335459, 0.075140, 0.387755, 0.075140, 0.387755, 0.075140, 0.387755, 0.072335, 0.358927, 0.075140, 0.387755, 0.075140, 0.387755, 0.072335, 0.358927, 0.072335, 0.358927, 0.072335, 0.358927, 0.069162, 0.333984, 0.072335, 0.358927, 0.072335, 0.358927, 0.069162, 0.333984, 0.069162, 0.333984, 0.069162, 0.333984, 0.065621, 0.312928, 0.069162, 0.333984, 0.069162, 0.333984, 0.065621, 0.312928, 0.065621, 0.312928, 0.065621, 0.312928, 0.061713, 0.295759, 0.065621, 0.312928, 0.065621, 0.312928, 0.061713, 0.295759, 0.061713, 0.295759, 0.061713, 0.295759, 0.057419, 0.282436, 0.061713, 0.295759, 0.061713, 0.295759, 0.057419, 0.282436, 0.057419, 0.282436, 0.057419, 0.282436, 0.052722, 0.272919, 0.057419, 0.282436, 0.057419, 0.282436, 0.052722, 0.272919, 0.052722, 0.272919, 0.052722, 0.272919, 0.047624, 0.267209, 0.052722, 0.272919, 0.052722, 0.272919, 0.047624, 0.267209, 0.047624, 0.267209, 0.047624, 0.267209, 0.042123, 0.265306, 0.047624, 0.267209, 0.047624, 0.267209, 0.042123, 0.265306, 0.042123, 0.265306, 0.042123, 0.265306, 0.033110, 0.271594, 0.042123, 0.265306, 0.042123, 0.265306, 0.033110, 0.271594, 0.033110, 0.271594, 0.033110, 0.271594, 0.025016, 0.290458, 0.033110, 0.271594, 0.033110, 0.271594, 0.025016, 0.290458, 0.025016, 0.290458, 0.025016, 0.290458, 0.017842, 0.321897, 0.025016, 0.290458, 0.025016, 0.290458, 0.017842, 0.321897, 0.017842, 0.321897, 0.017842, 0.321897, 0.011588, 0.365912, 0.017842, 0.321897, 0.017842, 0.321897, 0.011588, 0.365912, 0.011588, 0.365912, 0.011588, 0.365912, 0.006518, 0.420291, 0.011588, 0.365912, 0.011588, 0.365912, 0.006518, 0.420291, 0.006518, 0.420291, 0.006518, 0.420291, 0.002897, 0.482820, 0.006518, 0.420291, 0.006518, 0.420291, 0.002897, 0.482820, 0.002897, 0.482820, 0.002897, 0.482820, 0.000724, 0.553502, 0.002897, 0.482820, 0.002897, 0.482820, 0.000724, 0.553502, 0.000724, 0.553502, 0.000724, 0.553502, 0, 0.632334, 0.000724, 0.553502, 0.000724, 0.553502, 0, 0.632334, 0, 0.632334, 0, 0.632334, 0.000724, 0.711446, 0, 0.632334, 0, 0.632334, 0.000724, 0.711446, 0.000724, 0.711446, 0.000724, 0.711446, 0.002897, 0.782326, 0.000724, 0.711446, 0.000724, 0.711446, 0.002897, 0.782326, 0.002897, 0.782326, 0.002897, 0.782326, 0.006518, 0.844976, 0.002897, 0.782326, 0.002897, 0.782326, 0.006518, 0.844976, 0.006518, 0.844976, 0.006518, 0.844976, 0.011588, 0.899394, 0.006518, 0.844976, 0.006518, 0.844976, 0.011588, 0.899394, 0.011588, 0.899394, 0.011588, 0.899394, 0.017842, 0.943409, 0.011588, 0.899394, 0.011588, 0.899394, 0.017842, 0.943409, 0.017842, 0.943409, 0.017842, 0.943409, 0.025016, 0.974848, 0.017842, 0.943409, 0.017842, 0.943409, 0.025016, 0.974848, 0.025016, 0.974848, 0.025016, 0.974848, 0.033110, 0.993712, 0.025016, 0.974848, 0.025016, 0.974848, 0.033110, 0.993712, 0.033110, 0.993712, 0.033110, 0.993712, 0.042123, 1, 0.033110, 0.993712, 0.033110, 0.993712, 0.042123, 1, 0.042123, 1, 0.042123, 1, 0.047624, 0.998097, 0.042123, 1, 0.042123, 1, 0.047624, 0.998097, 0.047624, 0.998097, 0.047624, 0.998097, 0.052722, 0.992387, 0.047624, 0.998097, 0.047624, 0.998097, 0.052722, 0.992387, 0.052722, 0.992387, 0.052722, 0.992387, 0.057419, 0.982870, 0.052722, 0.992387, 0.052722, 0.992387, 0.057419, 0.982870, 0.057419, 0.982870, 0.057419, 0.982870, 0.061713, 0.969547, 0.057419, 0.982870, 0.057419, 0.982870, 0.061713, 0.969547, 0.061713, 0.969547, 0.061713, 0.969547, 0.065621, 0.952378, 0.061713, 0.969547, 0.061713, 0.969547, 0.065621, 0.952378, 0.065621, 0.952378, 0.065621, 0.952378, 0.069162, 0.931322, 0.065621, 0.952378, 0.065621, 0.952378, 0.069162, 0.931322, 0.069162, 0.931322, 0.069162, 0.931322, 0.072335, 0.906380, 0.069162, 0.931322, 0.069162, 0.931322, 0.072335, 0.906380, 0.072335, 0.906380, 0.072335, 0.906380, 0.075140, 0.877551, 0.072335, 0.906380, 0.072335, 0.906380, 0.075140, 0.877551, 0.075140, 0.877551, 0.075140, 0.877551, 0.075140, 0.979592, 0.075140, 0.877551, 0.075140, 0.877551, 0.075140, 0.979592, 0.075140, 0.979592, 0.075140, 0.979592, 0.092063, 0.979592, 0.075140, 0.979592, 0.075140, 0.979592, 0.092063, 0.979592, 0.092063, 0.979592, 0.017949, 0.692064, 0.000724, 0.711446, 0, 0.632334, 0.000724, 0.711446, 0.017949, 0.692064, 0, 0.632334, 0.019371, 0.744659, 0.000724, 0.711446, 0.017949, 0.692064, 0.000724, 0.711446, 0.019371, 0.744659, 0.017949, 0.692064, 0.019371, 0.744659, 0.002897, 0.782326, 0.000724, 0.711446, 0.002897, 0.782326, 0.019371, 0.744659, 0.000724, 0.711446, 0.021743, 0.790119, 0.002897, 0.782326, 0.019371, 0.744659, 0.002897, 0.782326, 0.021743, 0.790119, 0.019371, 0.744659, 0.021743, 0.790119, 0.006518, 0.844976, 0.002897, 0.782326, 0.006518, 0.844976, 0.021743, 0.790119, 0.002897, 0.782326, 0.025062, 0.828444, 0.006518, 0.844976, 0.021743, 0.790119, 0.006518, 0.844976, 0.025062, 0.828444, 0.021743, 0.790119, 0.025062, 0.828444, 0.011588, 0.899394, 0.006518, 0.844976, 0.011588, 0.899394, 0.025062, 0.828444, 0.006518, 0.844976, 0.017842, 0.943409, 0.011588, 0.899394, 0.025062, 0.828444, 0.011588, 0.899394, 0.017842, 0.943409, 0.025062, 0.828444, 0.017842, 0.943409, 0.025062, 0.828444, 0.029232, 0.858857, 0.025062, 0.828444, 0.017842, 0.943409, 0.029232, 0.858857, 0.025016, 0.974848, 0.017842, 0.943409, 0.029232, 0.858857, 0.017842, 0.943409, 0.025016, 0.974848, 0.029232, 0.858857, 0.025016, 0.974848, 0.029232, 0.858857, 0.034156, 0.880580, 0.029232, 0.858857, 0.025016, 0.974848, 0.034156, 0.880580, 0.033110, 0.993712, 0.025016, 0.974848, 0.034156, 0.880580, 0.025016, 0.974848, 0.033110, 0.993712, 0.034156, 0.880580, 0.033110, 0.993712, 0.034156, 0.880580, 0.039832, 0.893614, 0.034156, 0.880580, 0.033110, 0.993712, 0.039832, 0.893614, 0.039832, 0.893614, 0.042123, 1, 0.033110, 0.993712, 0.042123, 1, 0.039832, 0.893614, 0.033110, 0.993712, 0.039832, 0.893614, 0.046261, 0.897959, 0.042123, 1, 0.046261, 0.897959, 0.039832, 0.893614, 0.042123, 1, 0.046261, 0.897959, 0.047624, 0.998097, 0.042123, 1, 0.047624, 0.998097, 0.046261, 0.897959, 0.042123, 1, 0.052722, 0.992387, 0.047624, 0.998097, 0.046261, 0.897959, 0.047624, 0.998097, 0.052722, 0.992387, 0.046261, 0.897959, 0.052722, 0.992387, 0.046261, 0.897959, 0.052731, 0.893614, 0.046261, 0.897959, 0.052722, 0.992387, 0.052731, 0.893614, 0.052722, 0.992387, 0.052731, 0.893614, 0.057419, 0.982870, 0.052731, 0.893614, 0.052722, 0.992387, 0.057419, 0.982870, 0.057419, 0.982870, 0.052731, 0.893614, 0.061713, 0.969547, 0.052731, 0.893614, 0.057419, 0.982870, 0.061713, 0.969547, 0.052731, 0.893614, 0.058436, 0.880580, 0.061713, 0.969547, 0.058436, 0.880580, 0.052731, 0.893614, 0.061713, 0.969547, 0.061713, 0.969547, 0.058436, 0.880580, 0.065621, 0.952378, 0.058436, 0.880580, 0.061713, 0.969547, 0.065621, 0.952378, 0.065621, 0.952378, 0.058436, 0.880580, 0.069162, 0.931322, 0.058436, 0.880580, 0.065621, 0.952378, 0.069162, 0.931322, 0.058436, 0.880580, 0.063377, 0.858857, 0.069162, 0.931322, 0.063377, 0.858857, 0.058436, 0.880580, 0.069162, 0.931322, 0.069162, 0.931322, 0.063377, 0.858857, 0.072335, 0.906380, 0.063377, 0.858857, 0.069162, 0.931322, 0.072335, 0.906380, 0.072335, 0.906380, 0.063377, 0.858857, 0.075140, 0.877551, 0.063377, 0.858857, 0.072335, 0.906380, 0.075140, 0.877551, 0.075140, 0.877551, 0.063377, 0.858857, 0.067553, 0.828444, 0.063377, 0.858857, 0.075140, 0.877551, 0.067553, 0.828444, 0.067553, 0.828444, 0.070872, 0.790119, 0.075140, 0.877551, 0.070872, 0.790119, 0.067553, 0.828444, 0.075140, 0.877551, 0.070872, 0.790119, 0.073243, 0.744659, 0.075140, 0.877551, 0.073243, 0.744659, 0.070872, 0.790119, 0.075140, 0.877551, 0.075140, 0.877551, 0.073243, 0.744659, 0.092063, 0.979592, 0.073243, 0.744659, 0.075140, 0.877551, 0.092063, 0.979592, 0.092063, 0.979592, 0.075140, 0.979592, 0.075140, 0.877551, 0.075140, 0.979592, 0.092063, 0.979592, 0.075140, 0.877551, 0.074666, 0.692064, 0.092063, 0.979592, 0.073243, 0.744659, 0.092063, 0.979592, 0.074666, 0.692064, 0.073243, 0.744659, 0.075140, 0.632334, 0.092063, 0.979592, 0.074666, 0.692064, 0.092063, 0.979592, 0.075140, 0.632334, 0.074666, 0.692064, 0.075140, 0.632334, 0.092063, 0.357462, 0.092063, 0.979592, 0.092063, 0.357462, 0.075140, 0.632334, 0.092063, 0.979592, 0.075140, 0.632334, 0.074666, 0.572883, 0.092063, 0.357462, 0.074666, 0.572883, 0.075140, 0.632334, 0.092063, 0.357462, 0.092063, 0.357462, 0.074666, 0.572883, 0.073243, 0.520488, 0.074666, 0.572883, 0.092063, 0.357462, 0.073243, 0.520488, 0.075140, 0.387755, 0.092063, 0.357462, 0.073243, 0.520488, 0.092063, 0.357462, 0.075140, 0.387755, 0.073243, 0.520488, 0.075140, 0.387755, 0.075140, 0.335459, 0.092063, 0.357462, 0.075140, 0.335459, 0.075140, 0.387755, 0.092063, 0.357462, 0.075140, 0.335459, 0.091333, 0.272840, 0.092063, 0.357462, 0.091333, 0.272840, 0.075140, 0.335459, 0.092063, 0.357462, 0.075140, 0.335459, 0.074623, 0.280493, 0.091333, 0.272840, 0.074623, 0.280493, 0.075140, 0.335459, 0.091333, 0.272840, 0.091333, 0.272840, 0.074623, 0.280493, 0.089143, 0.199777, 0.074623, 0.280493, 0.091333, 0.272840, 0.089143, 0.199777, 0.074623, 0.280493, 0.073071, 0.232940, 0.089143, 0.199777, 0.073071, 0.232940, 0.074623, 0.280493, 0.089143, 0.199777, 0.089143, 0.199777, 0.073071, 0.232940, 0.085493, 0.138273, 0.073071, 0.232940, 0.089143, 0.199777, 0.085493, 0.138273, 0.073071, 0.232940, 0.070484, 0.192801, 0.085493, 0.138273, 0.070484, 0.192801, 0.073071, 0.232940, 0.085493, 0.138273, 0.085493, 0.138273, 0.070484, 0.192801, 0.080383, 0.088329, 0.070484, 0.192801, 0.085493, 0.138273, 0.080383, 0.088329, 0.070484, 0.192801, 0.066863, 0.160077, 0.080383, 0.088329, 0.066863, 0.160077, 0.070484, 0.192801, 0.080383, 0.088329, 0.066863, 0.160077, 0.073766, 0.049685, 0.080383, 0.088329, 0.073766, 0.049685, 0.066863, 0.160077, 0.080383, 0.088329, 0.066863, 0.160077, 0.062201, 0.134686, 0.073766, 0.049685, 0.062201, 0.134686, 0.066863, 0.160077, 0.073766, 0.049685, 0.062201, 0.134686, 0.065598, 0.022082, 0.073766, 0.049685, 0.065598, 0.022082, 0.062201, 0.134686, 0.073766, 0.049685, 0.062201, 0.134686, 0.056493, 0.116550, 0.065598, 0.022082, 0.056493, 0.116550, 0.062201, 0.134686, 0.065598, 0.022082, 0.056493, 0.116550, 0.055878, 0.005521, 0.065598, 0.022082, 0.055878, 0.005521, 0.056493, 0.116550, 0.065598, 0.022082, 0.049739, 0.105668, 0.055878, 0.005521, 0.056493, 0.116550, 0.055878, 0.005521, 0.049739, 0.105668, 0.056493, 0.116550, 0.049739, 0.105668, 0.044606, 0, 0.055878, 0.005521, 0.044606, 0, 0.049739, 0.105668, 0.055878, 0.005521, 0.041939, 0.102041, 0.044606, 0, 0.049739, 0.105668, 0.044606, 0, 0.041939, 0.102041, 0.049739, 0.105668, 0.041939, 0.102041, 0.040209, 0.000628, 0.044606, 0, 0.040209, 0.000628, 0.041939, 0.102041, 0.044606, 0, 0.038185, 0.102978, 0.040209, 0.000628, 0.041939, 0.102041, 0.040209, 0.000628, 0.038185, 0.102978, 0.041939, 0.102041, 0.038185, 0.102978, 0.035938, 0.002511, 0.040209, 0.000628, 0.035938, 0.002511, 0.038185, 0.102978, 0.040209, 0.000628, 0.034466, 0.105788, 0.035938, 0.002511, 0.038185, 0.102978, 0.035938, 0.002511, 0.034466, 0.105788, 0.038185, 0.102978, 0.034466, 0.105788, 0.031793, 0.005650, 0.035938, 0.002511, 0.031793, 0.005650, 0.034466, 0.105788, 0.035938, 0.002511, 0.030782, 0.110471, 0.031793, 0.005650, 0.034466, 0.105788, 0.031793, 0.005650, 0.030782, 0.110471, 0.034466, 0.105788, 0.030782, 0.110471, 0.027775, 0.010045, 0.031793, 0.005650, 0.027775, 0.010045, 0.030782, 0.110471, 0.031793, 0.005650, 0.023849, 0.015735, 0.027775, 0.010045, 0.030782, 0.110471, 0.027775, 0.010045, 0.023849, 0.015735, 0.030782, 0.110471, 0.027131, 0.117028, 0.023849, 0.015735, 0.030782, 0.110471, 0.023849, 0.015735, 0.027131, 0.117028, 0.030782, 0.110471, 0.019981, 0.022760, 0.023849, 0.015735, 0.027131, 0.117028, 0.023849, 0.015735, 0.019981, 0.022760, 0.027131, 0.117028, 0.023487, 0.125538, 0.019981, 0.022760, 0.027131, 0.117028, 0.019981, 0.022760, 0.023487, 0.125538, 0.027131, 0.117028, 0.016170, 0.031120, 0.019981, 0.022760, 0.023487, 0.125538, 0.019981, 0.022760, 0.016170, 0.031120, 0.023487, 0.125538, 0.019820, 0.136081, 0.016170, 0.031120, 0.023487, 0.125538, 0.016170, 0.031120, 0.019820, 0.136081, 0.023487, 0.125538, 0.012416, 0.040816, 0.016170, 0.031120, 0.019820, 0.136081, 0.016170, 0.031120, 0.012416, 0.040816, 0.019820, 0.136081, 0.016129, 0.148657, 0.012416, 0.040816, 0.019820, 0.136081, 0.012416, 0.040816, 0.016129, 0.148657, 0.019820, 0.136081, 0.012416, 0.163265, 0.012416, 0.040816, 0.016129, 0.148657, 0.012416, 0.040816, 0.012416, 0.163265, 0.016129, 0.148657, 0.073243, 0.520488, 0.070872, 0.475147, 0.075140, 0.387755, 0.070872, 0.475147, 0.073243, 0.520488, 0.075140, 0.387755, 0.070872, 0.475147, 0.067553, 0.436862, 0.075140, 0.387755, 0.067553, 0.436862, 0.070872, 0.475147, 0.075140, 0.387755, 0.067553, 0.436862, 0.063377, 0.406449, 0.075140, 0.387755, 0.063377, 0.406449, 0.067553, 0.436862, 0.075140, 0.387755, 0.063377, 0.406449, 0.072335, 0.358927, 0.075140, 0.387755, 0.072335, 0.358927, 0.063377, 0.406449, 0.075140, 0.387755, 0.063377, 0.406449, 0.069162, 0.333984, 0.072335, 0.358927, 0.069162, 0.333984, 0.063377, 0.406449, 0.072335, 0.358927, 0.063377, 0.406449, 0.058436, 0.384726, 0.069162, 0.333984, 0.058436, 0.384726, 0.063377, 0.406449, 0.069162, 0.333984, 0.058436, 0.384726, 0.065621, 0.312928, 0.069162, 0.333984, 0.065621, 0.312928, 0.058436, 0.384726, 0.069162, 0.333984, 0.058436, 0.384726, 0.061713, 0.295759, 0.065621, 0.312928, 0.061713, 0.295759, 0.058436, 0.384726, 0.065621, 0.312928, 0.058436, 0.384726, 0.052731, 0.371692, 0.061713, 0.295759, 0.052731, 0.371692, 0.058436, 0.384726, 0.061713, 0.295759, 0.052731, 0.371692, 0.057419, 0.282436, 0.061713, 0.295759, 0.057419, 0.282436, 0.052731, 0.371692, 0.061713, 0.295759, 0.052731, 0.371692, 0.052722, 0.272919, 0.057419, 0.282436, 0.052722, 0.272919, 0.052731, 0.371692, 0.057419, 0.282436, 0.046261, 0.367347, 0.052722, 0.272919, 0.052731, 0.371692, 0.052722, 0.272919, 0.046261, 0.367347, 0.052731, 0.371692, 0.046261, 0.367347, 0.047624, 0.267209, 0.052722, 0.272919, 0.047624, 0.267209, 0.046261, 0.367347, 0.052722, 0.272919, 0.046261, 0.367347, 0.042123, 0.265306, 0.047624, 0.267209, 0.042123, 0.265306, 0.046261, 0.367347, 0.047624, 0.267209, 0.039832, 0.371692, 0.042123, 0.265306, 0.046261, 0.367347, 0.042123, 0.265306, 0.039832, 0.371692, 0.046261, 0.367347, 0.039832, 0.371692, 0.033110, 0.271594, 0.042123, 0.265306, 0.033110, 0.271594, 0.039832, 0.371692, 0.042123, 0.265306, 0.039832, 0.371692, 0.034156, 0.384726, 0.033110, 0.271594, 0.034156, 0.384726, 0.039832, 0.371692, 0.033110, 0.271594, 0.034156, 0.384726, 0.025016, 0.290458, 0.033110, 0.271594, 0.025016, 0.290458, 0.034156, 0.384726, 0.033110, 0.271594, 0.034156, 0.384726, 0.029232, 0.406449, 0.025016, 0.290458, 0.029232, 0.406449, 0.034156, 0.384726, 0.025016, 0.290458, 0.029232, 0.406449, 0.017842, 0.321897, 0.025016, 0.290458, 0.017842, 0.321897, 0.029232, 0.406449, 0.025016, 0.290458, 0.029232, 0.406449, 0.025062, 0.436862, 0.017842, 0.321897, 0.025062, 0.436862, 0.029232, 0.406449, 0.017842, 0.321897, 0.025062, 0.436862, 0.011588, 0.365912, 0.017842, 0.321897, 0.011588, 0.365912, 0.025062, 0.436862, 0.017842, 0.321897, 0.006518, 0.420291, 0.011588, 0.365912, 0.025062, 0.436862, 0.011588, 0.365912, 0.006518, 0.420291, 0.025062, 0.436862, 0.021743, 0.475147, 0.006518, 0.420291, 0.025062, 0.436862, 0.006518, 0.420291, 0.021743, 0.475147, 0.025062, 0.436862, 0.002897, 0.482820, 0.006518, 0.420291, 0.021743, 0.475147, 0.006518, 0.420291, 0.002897, 0.482820, 0.021743, 0.475147, 0.019371, 0.520488, 0.002897, 0.482820, 0.021743, 0.475147, 0.002897, 0.482820, 0.019371, 0.520488, 0.021743, 0.475147, 0.000724, 0.553502, 0.002897, 0.482820, 0.019371, 0.520488, 0.002897, 0.482820, 0.000724, 0.553502, 0.019371, 0.520488, 0.017949, 0.572883, 0.000724, 0.553502, 0.019371, 0.520488, 0.000724, 0.553502, 0.017949, 0.572883, 0.019371, 0.520488, 0, 0.632334, 0.000724, 0.553502, 0.017949, 0.572883, 0.000724, 0.553502, 0, 0.632334, 0.017949, 0.572883, 0.017474, 0.632334, 0, 0.632334, 0.017949, 0.572883, 0, 0.632334, 0.017474, 0.632334, 0.017949, 0.572883, 0.017949, 0.692064, 0, 0.632334, 0.017474, 0.632334, 0, 0.632334, 0.017949, 0.692064, 0.017474, 0.632334, 0.174837, 0.612245, 0.165369, 0.610312, 0.174837, 0.612245, 0.174837, 0.612245, 0.165369, 0.610312, 0.165369, 0.610312, 0.165369, 0.610312, 0.157477, 0.604512, 0.165369, 0.610312, 0.165369, 0.610312, 0.157477, 0.604512, 0.157477, 0.604512, 0.157477, 0.604512, 0.151160, 0.594846, 0.157477, 0.604512, 0.157477, 0.604512, 0.151160, 0.594846, 0.151160, 0.594846, 0.151160, 0.594846, 0.146418, 0.581314, 0.151160, 0.594846, 0.151160, 0.594846, 0.146418, 0.581314, 0.146418, 0.581314, 0.146418, 0.581314, 0.142957, 0.563138, 0.146418, 0.581314, 0.146418, 0.581314, 0.142957, 0.563138, 0.142957, 0.563138, 0.142957, 0.563138, 0.140486, 0.539541, 0.142957, 0.563138, 0.142957, 0.563138, 0.140486, 0.539541, 0.140486, 0.539541, 0.140486, 0.539541, 0.139003, 0.510523, 0.140486, 0.539541, 0.140486, 0.539541, 0.139003, 0.510523, 0.139003, 0.510523, 0.139003, 0.510523, 0.138508, 0.476084, 0.139003, 0.510523, 0.139003, 0.510523, 0.138508, 0.476084, 0.138508, 0.476084, 0.138508, 0.476084, 0.138879, 0.447824, 0.138508, 0.476084, 0.138508, 0.476084, 0.138879, 0.447824, 0.138879, 0.447824, 0.138879, 0.447824, 0.139991, 0.422672, 0.138879, 0.447824, 0.138879, 0.447824, 0.139991, 0.422672, 0.139991, 0.422672, 0.139991, 0.422672, 0.141845, 0.400630, 0.139991, 0.422672, 0.139991, 0.422672, 0.141845, 0.400630, 0.141845, 0.400630, 0.141845, 0.400630, 0.144440, 0.381696, 0.141845, 0.400630, 0.141845, 0.400630, 0.144440, 0.381696, 0.144440, 0.381696, 0.144440, 0.381696, 0.147674, 0.366490, 0.144440, 0.381696, 0.144440, 0.381696, 0.147674, 0.366490, 0.147674, 0.366490, 0.147674, 0.366490, 0.151442, 0.355628, 0.147674, 0.366490, 0.147674, 0.366490, 0.151442, 0.355628, 0.151442, 0.355628, 0.151442, 0.355628, 0.155744, 0.349111, 0.151442, 0.355628, 0.151442, 0.355628, 0.155744, 0.349111, 0.155744, 0.349111, 0.155744, 0.349111, 0.160581, 0.346939, 0.155744, 0.349111, 0.155744, 0.349111, 0.160581, 0.346939, 0.160581, 0.346939, 0.160581, 0.346939, 0.167269, 0.351034, 0.160581, 0.346939, 0.160581, 0.346939, 0.167269, 0.351034, 0.167269, 0.351034, 0.167269, 0.351034, 0.173262, 0.363321, 0.167269, 0.351034, 0.167269, 0.351034, 0.173262, 0.363321, 0.173262, 0.363321, 0.173262, 0.363321, 0.178559, 0.383799, 0.173262, 0.363321, 0.173262, 0.363321, 0.178559, 0.383799, 0.178559, 0.383799, 0.178559, 0.383799, 0.183160, 0.412468, 0.178559, 0.383799, 0.178559, 0.383799, 0.183160, 0.412468, 0.183160, 0.412468, 0.183160, 0.412468, 0.186882, 0.447933, 0.183160, 0.412468, 0.183160, 0.412468, 0.186882, 0.447933, 0.186882, 0.447933, 0.186882, 0.447933, 0.189541, 0.488799, 0.186882, 0.447933, 0.186882, 0.447933, 0.189541, 0.488799, 0.189541, 0.488799, 0.189541, 0.488799, 0.191136, 0.535067, 0.189541, 0.488799, 0.189541, 0.488799, 0.191136, 0.535067, 0.191136, 0.535067, 0.191136, 0.535067, 0.191667, 0.586735, 0.191136, 0.535067, 0.191136, 0.535067, 0.191667, 0.586735, 0.191667, 0.586735, 0.191667, 0.586735, 0.191667, 0.612245, 0.191667, 0.586735, 0.191667, 0.586735, 0.191667, 0.612245, 0.191667, 0.612245, 0.208590, 0.666135, 0.208590, 0.265306, 0.208590, 0.666135, 0.208590, 0.666135, 0.208590, 0.265306, 0.208590, 0.265306, 0.208590, 0.265306, 0.191667, 0.265306, 0.208590, 0.265306, 0.208590, 0.265306, 0.191667, 0.265306, 0.191667, 0.265306, 0.191667, 0.265306, 0.191667, 0.367347, 0.191667, 0.265306, 0.191667, 0.265306, 0.191667, 0.367347, 0.191667, 0.367347, 0.191667, 0.367347, 0.188592, 0.338000, 0.191667, 0.367347, 0.191667, 0.367347, 0.188592, 0.338000, 0.188592, 0.338000, 0.188592, 0.338000, 0.185160, 0.312779, 0.188592, 0.338000, 0.188592, 0.338000, 0.185160, 0.312779, 0.185160, 0.312779, 0.185160, 0.312779, 0.181372, 0.291683, 0.185160, 0.312779, 0.185160, 0.312779, 0.181372, 0.291683, 0.181372, 0.291683, 0.181372, 0.291683, 0.177228, 0.274713, 0.181372, 0.291683, 0.181372, 0.291683, 0.177228, 0.274713, 0.177228, 0.274713, 0.177228, 0.274713, 0.172664, 0.261669, 0.177228, 0.274713, 0.177228, 0.274713, 0.172664, 0.261669, 0.172664, 0.261669, 0.172664, 0.261669, 0.167617, 0.252352, 0.172664, 0.261669, 0.172664, 0.261669, 0.167617, 0.252352, 0.167617, 0.252352, 0.167617, 0.252352, 0.162087, 0.246761, 0.167617, 0.252352, 0.167617, 0.252352, 0.162087, 0.246761, 0.162087, 0.246761, 0.162087, 0.246761, 0.156075, 0.244898, 0.162087, 0.246761, 0.162087, 0.246761, 0.156075, 0.244898, 0.156075, 0.244898, 0.156075, 0.244898, 0.148570, 0.248744, 0.156075, 0.244898, 0.156075, 0.244898, 0.148570, 0.248744, 0.148570, 0.248744, 0.148570, 0.248744, 0.141877, 0.260284, 0.148570, 0.248744, 0.148570, 0.248744, 0.141877, 0.260284, 0.141877, 0.260284, 0.141877, 0.260284, 0.135993, 0.279516, 0.141877, 0.260284, 0.141877, 0.260284, 0.135993, 0.279516, 0.135993, 0.279516, 0.135993, 0.279516, 0.130921, 0.306441, 0.135993, 0.279516, 0.135993, 0.279516, 0.130921, 0.306441, 0.130921, 0.306441, 0.130921, 0.306441, 0.126837, 0.339824, 0.130921, 0.306441, 0.130921, 0.306441, 0.126837, 0.339824, 0.126837, 0.339824, 0.126837, 0.339824, 0.123919, 0.378428, 0.126837, 0.339824, 0.126837, 0.339824, 0.123919, 0.378428, 0.123919, 0.378428, 0.123919, 0.378428, 0.122169, 0.422254, 0.123919, 0.378428, 0.123919, 0.378428, 0.122169, 0.422254, 0.122169, 0.422254, 0.122169, 0.422254, 0.121586, 0.471301, 0.122169, 0.422254, 0.122169, 0.422254, 0.121586, 0.471301, 0.121586, 0.471301, 0.121586, 0.471301, 0.122313, 0.527862, 0.121586, 0.471301, 0.121586, 0.471301, 0.122313, 0.527862, 0.122313, 0.527862, 0.122313, 0.527862, 0.124494, 0.577009, 0.122313, 0.527862, 0.122313, 0.527862, 0.124494, 0.577009, 0.124494, 0.577009, 0.124494, 0.577009, 0.128130, 0.618742, 0.124494, 0.577009, 0.124494, 0.577009, 0.128130, 0.618742, 0.128130, 0.618742, 0.128130, 0.618742, 0.133220, 0.653061, 0.128130, 0.618742, 0.128130, 0.618742, 0.133220, 0.653061, 0.133220, 0.653061, 0.133220, 0.653061, 0.139753, 0.679847, 0.133220, 0.653061, 0.133220, 0.653061, 0.139753, 0.679847, 0.139753, 0.679847, 0.139753, 0.679847, 0.147717, 0.698980, 0.139753, 0.679847, 0.139753, 0.679847, 0.147717, 0.698980, 0.147717, 0.698980, 0.147717, 0.698980, 0.157112, 0.710459, 0.147717, 0.698980, 0.147717, 0.698980, 0.157112, 0.710459, 0.157112, 0.710459, 0.157112, 0.710459, 0.167939, 0.714286, 0.157112, 0.710459, 0.157112, 0.710459, 0.167939, 0.714286, 0.167939, 0.714286, 0.167939, 0.714286, 0.191667, 0.714286, 0.167939, 0.714286, 0.167939, 0.714286, 0.191667, 0.714286, 0.191667, 0.714286, 0.191667, 0.714286, 0.191667, 0.725765, 0.191667, 0.714286, 0.191667, 0.714286, 0.191667, 0.725765, 0.191667, 0.725765, 0.191667, 0.725765, 0.191187, 0.764569, 0.191667, 0.725765, 0.191667, 0.725765, 0.191187, 0.764569, 0.191187, 0.764569, 0.191187, 0.764569, 0.189748, 0.798708, 0.191187, 0.764569, 0.191187, 0.764569, 0.189748, 0.798708, 0.189748, 0.798708, 0.189748, 0.798708, 0.187348, 0.828185, 0.189748, 0.798708, 0.189748, 0.798708, 0.187348, 0.828185, 0.187348, 0.828185, 0.187348, 0.828185, 0.183988, 0.852997, 0.187348, 0.828185, 0.187348, 0.828185, 0.183988, 0.852997, 0.183988, 0.852997, 0.183988, 0.852997, 0.179760, 0.872668, 0.183988, 0.852997, 0.183988, 0.852997, 0.179760, 0.872668, 0.179760, 0.872668, 0.179760, 0.872668, 0.174756, 0.886719, 0.179760, 0.872668, 0.179760, 0.872668, 0.174756, 0.886719, 0.174756, 0.886719, 0.174756, 0.886719, 0.168976, 0.895149, 0.174756, 0.886719, 0.174756, 0.886719, 0.168976, 0.895149, 0.168976, 0.895149, 0.168976, 0.895149, 0.162421, 0.897959, 0.168976, 0.895149, 0.168976, 0.895149, 0.162421, 0.897959, 0.162421, 0.897959, 0.162421, 0.897959, 0.158035, 0.897003, 0.162421, 0.897959, 0.162421, 0.897959, 0.158035, 0.897003, 0.158035, 0.897003, 0.158035, 0.897003, 0.153706, 0.894133, 0.158035, 0.897003, 0.158035, 0.897003, 0.153706, 0.894133, 0.153706, 0.894133, 0.153706, 0.894133, 0.149436, 0.889349, 0.153706, 0.894133, 0.153706, 0.894133, 0.149436, 0.889349, 0.149436, 0.889349, 0.149436, 0.889349, 0.145222, 0.882653, 0.149436, 0.889349, 0.149436, 0.889349, 0.145222, 0.882653, 0.145222, 0.882653, 0.145222, 0.882653, 0.141078, 0.874043, 0.145222, 0.882653, 0.145222, 0.882653, 0.141078, 0.874043, 0.141078, 0.874043, 0.141078, 0.874043, 0.137014, 0.863520, 0.141078, 0.874043, 0.141078, 0.874043, 0.137014, 0.863520, 0.137014, 0.863520, 0.137014, 0.863520, 0.133030, 0.851084, 0.137014, 0.863520, 0.137014, 0.863520, 0.133030, 0.851084, 0.133030, 0.851084, 0.133030, 0.851084, 0.129127, 0.836735, 0.133030, 0.851084, 0.133030, 0.851084, 0.129127, 0.836735, 0.129127, 0.836735, 0.129127, 0.836735, 0.129127, 0.938775, 0.129127, 0.836735, 0.129127, 0.836735, 0.129127, 0.938775, 0.129127, 0.938775, 0.129127, 0.938775, 0.133737, 0.953125, 0.129127, 0.938775, 0.129127, 0.938775, 0.133737, 0.953125, 0.133737, 0.953125, 0.133737, 0.953125, 0.138278, 0.965561, 0.133737, 0.953125, 0.133737, 0.953125, 0.138278, 0.965561, 0.138278, 0.965561, 0.138278, 0.965561, 0.142750, 0.976084, 0.138278, 0.965561, 0.138278, 0.965561, 0.142750, 0.976084, 0.142750, 0.976084, 0.142750, 0.976084, 0.147153, 0.984694, 0.142750, 0.976084, 0.142750, 0.976084, 0.147153, 0.984694, 0.147153, 0.984694, 0.147153, 0.984694, 0.151493, 0.991390, 0.147153, 0.984694, 0.147153, 0.984694, 0.151493, 0.991390, 0.151493, 0.991390, 0.151493, 0.991390, 0.155776, 0.996173, 0.151493, 0.991390, 0.151493, 0.991390, 0.155776, 0.996173, 0.155776, 0.996173, 0.155776, 0.996173, 0.160001, 0.999043, 0.155776, 0.996173, 0.155776, 0.996173, 0.160001, 0.999043, 0.160001, 0.999043, 0.160001, 0.999043, 0.164168, 1, 0.160001, 0.999043, 0.160001, 0.999043, 0.164168, 1, 0.164168, 1, 0.164168, 1, 0.174636, 0.994828, 0.164168, 1, 0.164168, 1, 0.174636, 0.994828, 0.174636, 0.994828, 0.174636, 0.994828, 0.183689, 0.979313, 0.174636, 0.994828, 0.174636, 0.994828, 0.183689, 0.979313, 0.183689, 0.979313, 0.183689, 0.979313, 0.191328, 0.953454, 0.183689, 0.979313, 0.183689, 0.979313, 0.191328, 0.953454, 0.191328, 0.953454, 0.191328, 0.953454, 0.197554, 0.917251, 0.191328, 0.953454, 0.191328, 0.953454, 0.197554, 0.917251, 0.197554, 0.917251, 0.197554, 0.917251, 0.202382, 0.870526, 0.197554, 0.917251, 0.197554, 0.917251, 0.202382, 0.870526, 0.202382, 0.870526, 0.202382, 0.870526, 0.205831, 0.813098, 0.202382, 0.870526, 0.202382, 0.870526, 0.205831, 0.813098, 0.205831, 0.813098, 0.205831, 0.813098, 0.207900, 0.744968, 0.205831, 0.813098, 0.205831, 0.813098, 0.207900, 0.744968, 0.207900, 0.744968, 0.122313, 0.527862, 0.121586, 0.471301, 0.138508, 0.476084, 0.121586, 0.471301, 0.122313, 0.527862, 0.138508, 0.476084, 0.122313, 0.527862, 0.138508, 0.476084, 0.139003, 0.510523, 0.138508, 0.476084, 0.122313, 0.527862, 0.139003, 0.510523, 0.124494, 0.577009, 0.122313, 0.527862, 0.139003, 0.510523, 0.122313, 0.527862, 0.124494, 0.577009, 0.139003, 0.510523, 0.124494, 0.577009, 0.139003, 0.510523, 0.140486, 0.539541, 0.139003, 0.510523, 0.124494, 0.577009, 0.140486, 0.539541, 0.128130, 0.618742, 0.124494, 0.577009, 0.140486, 0.539541, 0.124494, 0.577009, 0.128130, 0.618742, 0.140486, 0.539541, 0.128130, 0.618742, 0.140486, 0.539541, 0.142957, 0.563138, 0.140486, 0.539541, 0.128130, 0.618742, 0.142957, 0.563138, 0.133220, 0.653061, 0.128130, 0.618742, 0.142957, 0.563138, 0.128130, 0.618742, 0.133220, 0.653061, 0.142957, 0.563138, 0.133220, 0.653061, 0.142957, 0.563138, 0.146418, 0.581314, 0.142957, 0.563138, 0.133220, 0.653061, 0.146418, 0.581314, 0.139753, 0.679847, 0.133220, 0.653061, 0.146418, 0.581314, 0.133220, 0.653061, 0.139753, 0.679847, 0.146418, 0.581314, 0.151160, 0.594846, 0.139753, 0.679847, 0.146418, 0.581314, 0.139753, 0.679847, 0.151160, 0.594846, 0.146418, 0.581314, 0.147717, 0.698980, 0.139753, 0.679847, 0.151160, 0.594846, 0.139753, 0.679847, 0.147717, 0.698980, 0.151160, 0.594846, 0.157477, 0.604512, 0.147717, 0.698980, 0.151160, 0.594846, 0.147717, 0.698980, 0.157477, 0.604512, 0.151160, 0.594846, 0.157112, 0.710459, 0.147717, 0.698980, 0.157477, 0.604512, 0.147717, 0.698980, 0.157112, 0.710459, 0.157477, 0.604512, 0.157112, 0.710459, 0.157477, 0.604512, 0.165369, 0.610312, 0.157477, 0.604512, 0.157112, 0.710459, 0.165369, 0.610312, 0.157112, 0.710459, 0.165369, 0.610312, 0.167939, 0.714286, 0.165369, 0.610312, 0.157112, 0.710459, 0.167939, 0.714286, 0.167939, 0.714286, 0.165369, 0.610312, 0.174837, 0.612245, 0.165369, 0.610312, 0.167939, 0.714286, 0.174837, 0.612245, 0.167939, 0.714286, 0.174837, 0.612245, 0.191667, 0.714286, 0.174837, 0.612245, 0.167939, 0.714286, 0.191667, 0.714286, 0.174837, 0.612245, 0.191667, 0.612245, 0.191667, 0.714286, 0.191667, 0.612245, 0.174837, 0.612245, 0.191667, 0.714286, 0.191667, 0.714286, 0.191667, 0.612245, 0.208590, 0.666135, 0.191667, 0.612245, 0.191667, 0.714286, 0.208590, 0.666135, 0.208590, 0.666135, 0.207900, 0.744968, 0.191667, 0.714286, 0.207900, 0.744968, 0.208590, 0.666135, 0.191667, 0.714286, 0.207900, 0.744968, 0.191667, 0.725765, 0.191667, 0.714286, 0.191667, 0.725765, 0.207900, 0.744968, 0.191667, 0.714286, 0.191187, 0.764569, 0.191667, 0.725765, 0.207900, 0.744968, 0.191667, 0.725765, 0.191187, 0.764569, 0.207900, 0.744968, 0.205831, 0.813098, 0.191187, 0.764569, 0.207900, 0.744968, 0.191187, 0.764569, 0.205831, 0.813098, 0.207900, 0.744968, 0.189748, 0.798708, 0.191187, 0.764569, 0.205831, 0.813098, 0.191187, 0.764569, 0.189748, 0.798708, 0.205831, 0.813098, 0.202382, 0.870526, 0.189748, 0.798708, 0.205831, 0.813098, 0.189748, 0.798708, 0.202382, 0.870526, 0.205831, 0.813098, 0.187348, 0.828185, 0.189748, 0.798708, 0.202382, 0.870526, 0.189748, 0.798708, 0.187348, 0.828185, 0.202382, 0.870526, 0.197554, 0.917251, 0.187348, 0.828185, 0.202382, 0.870526, 0.187348, 0.828185, 0.197554, 0.917251, 0.202382, 0.870526, 0.183988, 0.852997, 0.187348, 0.828185, 0.197554, 0.917251, 0.187348, 0.828185, 0.183988, 0.852997, 0.197554, 0.917251, 0.191328, 0.953454, 0.183988, 0.852997, 0.197554, 0.917251, 0.183988, 0.852997, 0.191328, 0.953454, 0.197554, 0.917251, 0.179760, 0.872668, 0.183988, 0.852997, 0.191328, 0.953454, 0.183988, 0.852997, 0.179760, 0.872668, 0.191328, 0.953454, 0.183689, 0.979313, 0.179760, 0.872668, 0.191328, 0.953454, 0.179760, 0.872668, 0.183689, 0.979313, 0.191328, 0.953454, 0.174756, 0.886719, 0.179760, 0.872668, 0.183689, 0.979313, 0.179760, 0.872668, 0.174756, 0.886719, 0.183689, 0.979313, 0.174636, 0.994828, 0.174756, 0.886719, 0.183689, 0.979313, 0.174756, 0.886719, 0.174636, 0.994828, 0.183689, 0.979313, 0.174636, 0.994828, 0.168976, 0.895149, 0.174756, 0.886719, 0.168976, 0.895149, 0.174636, 0.994828, 0.174756, 0.886719, 0.174636, 0.994828, 0.164168, 1, 0.168976, 0.895149, 0.164168, 1, 0.174636, 0.994828, 0.168976, 0.895149, 0.162421, 0.897959, 0.168976, 0.895149, 0.164168, 1, 0.168976, 0.895149, 0.162421, 0.897959, 0.164168, 1, 0.164168, 1, 0.160001, 0.999043, 0.162421, 0.897959, 0.160001, 0.999043, 0.164168, 1, 0.162421, 0.897959, 0.160001, 0.999043, 0.158035, 0.897003, 0.162421, 0.897959, 0.158035, 0.897003, 0.160001, 0.999043, 0.162421, 0.897959, 0.155776, 0.996173, 0.158035, 0.897003, 0.160001, 0.999043, 0.158035, 0.897003, 0.155776, 0.996173, 0.160001, 0.999043, 0.155776, 0.996173, 0.153706, 0.894133, 0.158035, 0.897003, 0.153706, 0.894133, 0.155776, 0.996173, 0.158035, 0.897003, 0.151493, 0.991390, 0.153706, 0.894133, 0.155776, 0.996173, 0.153706, 0.894133, 0.151493, 0.991390, 0.155776, 0.996173, 0.151493, 0.991390, 0.147153, 0.984694, 0.153706, 0.894133, 0.147153, 0.984694, 0.151493, 0.991390, 0.153706, 0.894133, 0.153706, 0.894133, 0.147153, 0.984694, 0.149436, 0.889349, 0.147153, 0.984694, 0.153706, 0.894133, 0.149436, 0.889349, 0.147153, 0.984694, 0.142750, 0.976084, 0.149436, 0.889349, 0.142750, 0.976084, 0.147153, 0.984694, 0.149436, 0.889349, 0.142750, 0.976084, 0.145222, 0.882653, 0.149436, 0.889349, 0.145222, 0.882653, 0.142750, 0.976084, 0.149436, 0.889349, 0.142750, 0.976084, 0.138278, 0.965561, 0.145222, 0.882653, 0.138278, 0.965561, 0.142750, 0.976084, 0.145222, 0.882653, 0.138278, 0.965561, 0.141078, 0.874043, 0.145222, 0.882653, 0.141078, 0.874043, 0.138278, 0.965561, 0.145222, 0.882653, 0.138278, 0.965561, 0.133737, 0.953125, 0.141078, 0.874043, 0.133737, 0.953125, 0.138278, 0.965561, 0.141078, 0.874043, 0.133737, 0.953125, 0.137014, 0.863520, 0.141078, 0.874043, 0.137014, 0.863520, 0.133737, 0.953125, 0.141078, 0.874043, 0.133737, 0.953125, 0.129127, 0.938775, 0.137014, 0.863520, 0.129127, 0.938775, 0.133737, 0.953125, 0.137014, 0.863520, 0.129127, 0.938775, 0.133030, 0.851084, 0.137014, 0.863520, 0.133030, 0.851084, 0.129127, 0.938775, 0.137014, 0.863520, 0.129127, 0.938775, 0.129127, 0.836735, 0.133030, 0.851084, 0.129127, 0.836735, 0.129127, 0.938775, 0.133030, 0.851084, 0.208590, 0.666135, 0.191667, 0.612245, 0.191667, 0.586735, 0.191667, 0.612245, 0.208590, 0.666135, 0.191667, 0.586735, 0.208590, 0.666135, 0.191667, 0.586735, 0.191136, 0.535067, 0.191667, 0.586735, 0.208590, 0.666135, 0.191136, 0.535067, 0.208590, 0.265306, 0.208590, 0.666135, 0.191136, 0.535067, 0.208590, 0.666135, 0.208590, 0.265306, 0.191136, 0.535067, 0.208590, 0.265306, 0.191136, 0.535067, 0.189541, 0.488799, 0.191136, 0.535067, 0.208590, 0.265306, 0.189541, 0.488799, 0.191667, 0.367347, 0.208590, 0.265306, 0.189541, 0.488799, 0.208590, 0.265306, 0.191667, 0.367347, 0.189541, 0.488799, 0.191667, 0.367347, 0.191667, 0.265306, 0.208590, 0.265306, 0.191667, 0.265306, 0.191667, 0.367347, 0.208590, 0.265306, 0.189541, 0.488799, 0.186882, 0.447933, 0.191667, 0.367347, 0.186882, 0.447933, 0.189541, 0.488799, 0.191667, 0.367347, 0.186882, 0.447933, 0.183160, 0.412468, 0.191667, 0.367347, 0.183160, 0.412468, 0.186882, 0.447933, 0.191667, 0.367347, 0.183160, 0.412468, 0.188592, 0.338000, 0.191667, 0.367347, 0.188592, 0.338000, 0.183160, 0.412468, 0.191667, 0.367347, 0.183160, 0.412468, 0.178559, 0.383799, 0.188592, 0.338000, 0.178559, 0.383799, 0.183160, 0.412468, 0.188592, 0.338000, 0.178559, 0.383799, 0.185160, 0.312779, 0.188592, 0.338000, 0.185160, 0.312779, 0.178559, 0.383799, 0.188592, 0.338000, 0.185160, 0.312779, 0.178559, 0.383799, 0.181372, 0.291683, 0.178559, 0.383799, 0.185160, 0.312779, 0.181372, 0.291683, 0.178559, 0.383799, 0.173262, 0.363321, 0.181372, 0.291683, 0.173262, 0.363321, 0.178559, 0.383799, 0.181372, 0.291683, 0.173262, 0.363321, 0.177228, 0.274713, 0.181372, 0.291683, 0.177228, 0.274713, 0.173262, 0.363321, 0.181372, 0.291683, 0.177228, 0.274713, 0.173262, 0.363321, 0.172664, 0.261669, 0.173262, 0.363321, 0.177228, 0.274713, 0.172664, 0.261669, 0.173262, 0.363321, 0.167269, 0.351034, 0.172664, 0.261669, 0.167269, 0.351034, 0.173262, 0.363321, 0.172664, 0.261669, 0.167269, 0.351034, 0.167617, 0.252352, 0.172664, 0.261669, 0.167617, 0.252352, 0.167269, 0.351034, 0.172664, 0.261669, 0.167269, 0.351034, 0.160581, 0.346939, 0.167617, 0.252352, 0.160581, 0.346939, 0.167269, 0.351034, 0.167617, 0.252352, 0.160581, 0.346939, 0.162087, 0.246761, 0.167617, 0.252352, 0.162087, 0.246761, 0.160581, 0.346939, 0.167617, 0.252352, 0.160581, 0.346939, 0.156075, 0.244898, 0.162087, 0.246761, 0.156075, 0.244898, 0.160581, 0.346939, 0.162087, 0.246761, 0.155744, 0.349111, 0.156075, 0.244898, 0.160581, 0.346939, 0.156075, 0.244898, 0.155744, 0.349111, 0.160581, 0.346939, 0.155744, 0.349111, 0.148570, 0.248744, 0.156075, 0.244898, 0.148570, 0.248744, 0.155744, 0.349111, 0.156075, 0.244898, 0.151442, 0.355628, 0.148570, 0.248744, 0.155744, 0.349111, 0.148570, 0.248744, 0.151442, 0.355628, 0.155744, 0.349111, 0.141877, 0.260284, 0.148570, 0.248744, 0.151442, 0.355628, 0.148570, 0.248744, 0.141877, 0.260284, 0.151442, 0.355628, 0.147674, 0.366490, 0.141877, 0.260284, 0.151442, 0.355628, 0.141877, 0.260284, 0.147674, 0.366490, 0.151442, 0.355628, 0.135993, 0.279516, 0.141877, 0.260284, 0.147674, 0.366490, 0.141877, 0.260284, 0.135993, 0.279516, 0.147674, 0.366490, 0.147674, 0.366490, 0.144440, 0.381696, 0.135993, 0.279516, 0.144440, 0.381696, 0.147674, 0.366490, 0.135993, 0.279516, 0.144440, 0.381696, 0.130921, 0.306441, 0.135993, 0.279516, 0.130921, 0.306441, 0.144440, 0.381696, 0.135993, 0.279516, 0.144440, 0.381696, 0.141845, 0.400630, 0.130921, 0.306441, 0.141845, 0.400630, 0.144440, 0.381696, 0.130921, 0.306441, 0.141845, 0.400630, 0.126837, 0.339824, 0.130921, 0.306441, 0.126837, 0.339824, 0.141845, 0.400630, 0.130921, 0.306441, 0.141845, 0.400630, 0.139991, 0.422672, 0.126837, 0.339824, 0.139991, 0.422672, 0.141845, 0.400630, 0.126837, 0.339824, 0.139991, 0.422672, 0.123919, 0.378428, 0.126837, 0.339824, 0.123919, 0.378428, 0.139991, 0.422672, 0.126837, 0.339824, 0.122169, 0.422254, 0.123919, 0.378428, 0.139991, 0.422672, 0.123919, 0.378428, 0.122169, 0.422254, 0.139991, 0.422672, 0.138879, 0.447824, 0.122169, 0.422254, 0.139991, 0.422672, 0.122169, 0.422254, 0.138879, 0.447824, 0.139991, 0.422672, 0.121586, 0.471301, 0.122169, 0.422254, 0.138879, 0.447824, 0.122169, 0.422254, 0.121586, 0.471301, 0.138879, 0.447824, 0.138508, 0.476084, 0.121586, 0.471301, 0.138879, 0.447824, 0.121586, 0.471301, 0.138508, 0.476084, 0.138879, 0.447824, 0.323002, 0.851084, 0.326330, 0.886699, 0.323002, 0.851084, 0.323002, 0.851084, 0.326330, 0.886699, 0.326330, 0.886699, 0.326330, 0.886699, 0.329969, 0.917331, 0.326330, 0.886699, 0.326330, 0.886699, 0.329969, 0.917331, 0.329969, 0.917331, 0.329969, 0.917331, 0.333918, 0.942981, 0.329969, 0.917331, 0.329969, 0.917331, 0.333918, 0.942981, 0.333918, 0.942981, 0.333918, 0.942981, 0.338177, 0.963648, 0.333918, 0.942981, 0.333918, 0.942981, 0.338177, 0.963648, 0.338177, 0.963648, 0.338177, 0.963648, 0.342787, 0.979552, 0.338177, 0.963648, 0.338177, 0.963648, 0.342787, 0.979552, 0.342787, 0.979552, 0.342787, 0.979552, 0.347788, 0.990912, 0.342787, 0.979552, 0.342787, 0.979552, 0.347788, 0.990912, 0.347788, 0.990912, 0.347788, 0.990912, 0.353180, 0.997728, 0.347788, 0.990912, 0.347788, 0.990912, 0.353180, 0.997728, 0.353180, 0.997728, 0.353180, 0.997728, 0.358963, 1, 0.353180, 0.997728, 0.353180, 0.997728, 0.358963, 1, 0.358963, 1, 0.358963, 1, 0.366550, 0.995167, 0.358963, 1, 0.358963, 1, 0.366550, 0.995167, 0.366550, 0.995167, 0.366550, 0.995167, 0.373218, 0.980668, 0.366550, 0.995167, 0.366550, 0.995167, 0.373218, 0.980668, 0.373218, 0.980668, 0.373218, 0.980668, 0.378966, 0.956503, 0.373218, 0.980668, 0.373218, 0.980668, 0.378966, 0.956503, 0.378966, 0.956503, 0.378966, 0.956503, 0.383795, 0.922672, 0.378966, 0.956503, 0.378966, 0.956503, 0.383795, 0.922672, 0.383795, 0.922672, 0.383795, 0.922672, 0.387617, 0.879933, 0.383795, 0.922672, 0.383795, 0.922672, 0.387617, 0.879933, 0.387617, 0.879933, 0.387617, 0.879933, 0.390348, 0.829042, 0.387617, 0.879933, 0.387617, 0.879933, 0.390348, 0.829042, 0.390348, 0.829042, 0.390348, 0.829042, 0.391986, 0.770000, 0.390348, 0.829042, 0.390348, 0.829042, 0.391986, 0.770000, 0.391986, 0.770000, 0.391986, 0.770000, 0.392532, 0.702806, 0.391986, 0.770000, 0.391986, 0.770000, 0.392532, 0.702806, 0.392532, 0.702806, 0.392532, 0.702806, 0.392532, 0.265306, 0.392532, 0.702806, 0.392532, 0.702806, 0.392532, 0.265306, 0.392532, 0.265306, 0.392532, 0.265306, 0.375517, 0.265306, 0.392532, 0.265306, 0.392532, 0.265306, 0.375517, 0.265306, 0.375517, 0.265306, 0.375517, 0.265306, 0.375517, 0.698980, 0.375517, 0.265306, 0.375517, 0.265306, 0.375517, 0.698980, 0.375517, 0.698980, 0.375517, 0.698980, 0.375190, 0.746263, 0.375517, 0.698980, 0.375517, 0.698980, 0.375190, 0.746263, 0.375190, 0.746263, 0.375190, 0.746263, 0.374207, 0.787030, 0.375190, 0.746263, 0.375190, 0.746263, 0.374207, 0.787030, 0.374207, 0.787030, 0.374207, 0.787030, 0.372568, 0.821279, 0.374207, 0.787030, 0.374207, 0.787030, 0.372568, 0.821279, 0.372568, 0.821279, 0.372568, 0.821279, 0.370275, 0.849011, 0.372568, 0.821279, 0.372568, 0.821279, 0.370275, 0.849011, 0.370275, 0.849011, 0.370275, 0.849011, 0.367309, 0.870426, 0.370275, 0.849011, 0.370275, 0.849011, 0.367309, 0.870426, 0.367309, 0.870426, 0.367309, 0.870426, 0.363653, 0.885722, 0.367309, 0.870426, 0.367309, 0.870426, 0.363653, 0.885722, 0.363653, 0.885722, 0.363653, 0.885722, 0.359307, 0.894900, 0.363653, 0.885722, 0.363653, 0.885722, 0.359307, 0.894900, 0.359307, 0.894900, 0.359307, 0.894900, 0.354272, 0.897959, 0.359307, 0.894900, 0.359307, 0.894900, 0.354272, 0.897959, 0.354272, 0.897959, 0.354272, 0.897959, 0.348041, 0.894222, 0.354272, 0.897959, 0.354272, 0.897959, 0.348041, 0.894222, 0.348041, 0.894222, 0.348041, 0.894222, 0.342500, 0.883012, 0.348041, 0.894222, 0.348041, 0.894222, 0.342500, 0.883012, 0.342500, 0.883012, 0.342500, 0.883012, 0.337648, 0.864328, 0.342500, 0.883012, 0.342500, 0.883012, 0.337648, 0.864328, 0.337648, 0.864328, 0.337648, 0.864328, 0.333487, 0.838170, 0.337648, 0.864328, 0.337648, 0.864328, 0.333487, 0.838170, 0.333487, 0.838170, 0.333487, 0.838170, 0.330147, 0.805554, 0.333487, 0.838170, 0.333487, 0.838170, 0.330147, 0.805554, 0.330147, 0.805554, 0.330147, 0.805554, 0.327761, 0.767498, 0.330147, 0.805554, 0.330147, 0.805554, 0.327761, 0.767498, 0.327761, 0.767498, 0.327761, 0.767498, 0.326330, 0.724002, 0.327761, 0.767498, 0.327761, 0.767498, 0.326330, 0.724002, 0.326330, 0.724002, 0.326330, 0.724002, 0.325853, 0.675064, 0.326330, 0.724002, 0.326330, 0.724002, 0.325853, 0.675064, 0.325853, 0.675064, 0.325853, 0.675064, 0.325853, 0.265306, 0.325853, 0.675064, 0.325853, 0.675064, 0.325853, 0.265306, 0.325853, 0.265306, 0.325853, 0.265306, 0.308838, 0.265306, 0.325853, 0.265306, 0.325853, 0.265306, 0.308838, 0.265306, 0.308838, 0.265306, 0.308838, 0.265306, 0.308838, 0.698980, 0.308838, 0.265306, 0.308838, 0.265306, 0.308838, 0.698980, 0.308838, 0.698980, 0.308838, 0.698980, 0.308511, 0.746522, 0.308838, 0.698980, 0.308838, 0.698980, 0.308511, 0.746522, 0.308511, 0.746522, 0.308511, 0.746522, 0.307528, 0.787428, 0.308511, 0.746522, 0.308511, 0.746522, 0.307528, 0.787428, 0.307528, 0.787428, 0.307528, 0.787428, 0.305890, 0.821698, 0.307528, 0.787428, 0.307528, 0.787428, 0.305890, 0.821698, 0.305890, 0.821698, 0.305890, 0.821698, 0.303596, 0.849330, 0.305890, 0.821698, 0.305890, 0.821698, 0.303596, 0.849330, 0.303596, 0.849330, 0.303596, 0.849330, 0.300618, 0.870605, 0.303596, 0.849330, 0.303596, 0.849330, 0.300618, 0.870605, 0.300618, 0.870605, 0.300618, 0.870605, 0.296928, 0.885802, 0.300618, 0.870605, 0.300618, 0.870605, 0.296928, 0.885802, 0.296928, 0.885802, 0.296928, 0.885802, 0.292525, 0.894920, 0.296928, 0.885802, 0.296928, 0.885802, 0.292525, 0.894920, 0.292525, 0.894920, 0.292525, 0.894920, 0.287409, 0.897959, 0.292525, 0.894920, 0.292525, 0.894920, 0.287409, 0.897959, 0.287409, 0.897959, 0.287409, 0.897959, 0.281259, 0.894202, 0.287409, 0.897959, 0.287409, 0.897959, 0.281259, 0.894202, 0.281259, 0.894202, 0.281259, 0.894202, 0.275775, 0.882932, 0.281259, 0.894202, 0.281259, 0.894202, 0.275775, 0.882932, 0.275775, 0.882932, 0.275775, 0.882932, 0.270958, 0.864148, 0.275775, 0.882932, 0.275775, 0.882932, 0.270958, 0.864148, 0.270958, 0.864148, 0.270958, 0.864148, 0.266808, 0.837851, 0.270958, 0.864148, 0.270958, 0.864148, 0.266808, 0.837851, 0.266808, 0.837851, 0.266808, 0.837851, 0.263468, 0.805136, 0.266808, 0.837851, 0.266808, 0.837851, 0.263468, 0.805136, 0.263468, 0.805136, 0.263468, 0.805136, 0.261083, 0.767100, 0.263468, 0.805136, 0.263468, 0.805136, 0.261083, 0.767100, 0.261083, 0.767100, 0.261083, 0.767100, 0.259651, 0.723742, 0.261083, 0.767100, 0.261083, 0.767100, 0.259651, 0.723742, 0.259651, 0.723742, 0.259651, 0.723742, 0.259174, 0.675064, 0.259651, 0.723742, 0.259651, 0.723742, 0.259174, 0.675064, 0.259174, 0.675064, 0.259174, 0.675064, 0.259174, 0.265306, 0.259174, 0.675064, 0.259174, 0.675064, 0.259174, 0.265306, 0.259174, 0.265306, 0.259174, 0.265306, 0.242159, 0.265306, 0.259174, 0.265306, 0.259174, 0.265306, 0.242159, 0.265306, 0.242159, 0.265306, 0.242159, 0.265306, 0.242159, 0.979592, 0.242159, 0.265306, 0.242159, 0.265306, 0.242159, 0.979592, 0.242159, 0.979592, 0.242159, 0.979592, 0.259174, 0.979592, 0.242159, 0.979592, 0.242159, 0.979592, 0.259174, 0.979592, 0.259174, 0.979592, 0.259174, 0.979592, 0.259174, 0.877551, 0.259174, 0.979592, 0.259174, 0.979592, 0.259174, 0.877551, 0.259174, 0.877551, 0.259174, 0.877551, 0.262215, 0.906898, 0.259174, 0.877551, 0.259174, 0.877551, 0.262215, 0.906898, 0.262215, 0.906898, 0.262215, 0.906898, 0.265543, 0.932119, 0.262215, 0.906898, 0.262215, 0.906898, 0.265543, 0.932119, 0.265543, 0.932119, 0.265543, 0.932119, 0.269159, 0.953215, 0.265543, 0.932119, 0.265543, 0.932119, 0.269159, 0.953215, 0.269159, 0.953215, 0.269159, 0.953215, 0.273062, 0.970185, 0.269159, 0.953215, 0.269159, 0.953215, 0.273062, 0.970185, 0.273062, 0.970185, 0.273062, 0.970185, 0.277298, 0.983229, 0.273062, 0.970185, 0.273062, 0.970185, 0.277298, 0.983229, 0.277298, 0.983229, 0.277298, 0.983229, 0.281914, 0.992546, 0.277298, 0.983229, 0.277298, 0.983229, 0.281914, 0.992546, 0.281914, 0.992546, 0.281914, 0.992546, 0.286909, 0.998137, 0.281914, 0.992546, 0.281914, 0.992546, 0.286909, 0.998137, 0.286909, 0.998137, 0.286909, 0.998137, 0.292284, 1, 0.286909, 0.998137, 0.286909, 0.998137, 0.292284, 1, 0.292284, 1, 0.292284, 1, 0.297684, 0.997618, 0.292284, 1, 0.292284, 1, 0.297684, 0.997618, 0.297684, 0.997618, 0.297684, 0.997618, 0.302665, 0.990474, 0.297684, 0.997618, 0.297684, 0.997618, 0.302665, 0.990474, 0.302665, 0.990474, 0.302665, 0.990474, 0.307226, 0.978565, 0.302665, 0.990474, 0.302665, 0.990474, 0.307226, 0.978565, 0.307226, 0.978565, 0.307226, 0.978565, 0.311368, 0.961894, 0.307226, 0.978565, 0.307226, 0.978565, 0.311368, 0.961894, 0.311368, 0.961894, 0.311368, 0.961894, 0.315044, 0.940679, 0.311368, 0.961894, 0.311368, 0.961894, 0.315044, 0.940679, 0.315044, 0.940679, 0.315044, 0.940679, 0.318208, 0.915139, 0.315044, 0.940679, 0.315044, 0.940679, 0.318208, 0.915139, 0.318208, 0.915139, 0.318208, 0.915139, 0.320861, 0.885274, 0.318208, 0.915139, 0.318208, 0.915139, 0.320861, 0.885274, 0.320861, 0.885274, 0.259174, 0.979592, 0.242159, 0.979592, 0.259174, 0.877551, 0.242159, 0.979592, 0.259174, 0.979592, 0.259174, 0.877551, 0.261083, 0.767100, 0.259174, 0.877551, 0.242159, 0.979592, 0.259174, 0.877551, 0.261083, 0.767100, 0.242159, 0.979592, 0.259174, 0.877551, 0.261083, 0.767100, 0.263468, 0.805136, 0.261083, 0.767100, 0.259174, 0.877551, 0.263468, 0.805136, 0.259174, 0.877551, 0.263468, 0.805136, 0.266808, 0.837851, 0.263468, 0.805136, 0.259174, 0.877551, 0.266808, 0.837851, 0.262215, 0.906898, 0.259174, 0.877551, 0.266808, 0.837851, 0.259174, 0.877551, 0.262215, 0.906898, 0.266808, 0.837851, 0.262215, 0.906898, 0.266808, 0.837851, 0.270958, 0.864148, 0.266808, 0.837851, 0.262215, 0.906898, 0.270958, 0.864148, 0.265543, 0.932119, 0.262215, 0.906898, 0.270958, 0.864148, 0.262215, 0.906898, 0.265543, 0.932119, 0.270958, 0.864148, 0.265543, 0.932119, 0.270958, 0.864148, 0.275775, 0.882932, 0.270958, 0.864148, 0.265543, 0.932119, 0.275775, 0.882932, 0.269159, 0.953215, 0.265543, 0.932119, 0.275775, 0.882932, 0.265543, 0.932119, 0.269159, 0.953215, 0.275775, 0.882932, 0.273062, 0.970185, 0.269159, 0.953215, 0.275775, 0.882932, 0.269159, 0.953215, 0.273062, 0.970185, 0.275775, 0.882932, 0.273062, 0.970185, 0.275775, 0.882932, 0.281259, 0.894202, 0.275775, 0.882932, 0.273062, 0.970185, 0.281259, 0.894202, 0.277298, 0.983229, 0.273062, 0.970185, 0.281259, 0.894202, 0.273062, 0.970185, 0.277298, 0.983229, 0.281259, 0.894202, 0.277298, 0.983229, 0.281259, 0.894202, 0.281914, 0.992546, 0.281259, 0.894202, 0.277298, 0.983229, 0.281914, 0.992546, 0.281914, 0.992546, 0.281259, 0.894202, 0.287409, 0.897959, 0.281259, 0.894202, 0.281914, 0.992546, 0.287409, 0.897959, 0.286909, 0.998137, 0.281914, 0.992546, 0.287409, 0.897959, 0.281914, 0.992546, 0.286909, 0.998137, 0.287409, 0.897959, 0.292284, 1, 0.286909, 0.998137, 0.287409, 0.897959, 0.286909, 0.998137, 0.292284, 1, 0.287409, 0.897959, 0.292284, 1, 0.287409, 0.897959, 0.292525, 0.894920, 0.287409, 0.897959, 0.292284, 1, 0.292525, 0.894920, 0.297684, 0.997618, 0.292284, 1, 0.292525, 0.894920, 0.292284, 1, 0.297684, 0.997618, 0.292525, 0.894920, 0.297684, 0.997618, 0.292525, 0.894920, 0.296928, 0.885802, 0.292525, 0.894920, 0.297684, 0.997618, 0.296928, 0.885802, 0.302665, 0.990474, 0.297684, 0.997618, 0.296928, 0.885802, 0.297684, 0.997618, 0.302665, 0.990474, 0.296928, 0.885802, 0.302665, 0.990474, 0.296928, 0.885802, 0.307226, 0.978565, 0.296928, 0.885802, 0.302665, 0.990474, 0.307226, 0.978565, 0.296928, 0.885802, 0.300618, 0.870605, 0.307226, 0.978565, 0.300618, 0.870605, 0.296928, 0.885802, 0.307226, 0.978565, 0.307226, 0.978565, 0.300618, 0.870605, 0.311368, 0.961894, 0.300618, 0.870605, 0.307226, 0.978565, 0.311368, 0.961894, 0.311368, 0.961894, 0.300618, 0.870605, 0.315044, 0.940679, 0.300618, 0.870605, 0.311368, 0.961894, 0.315044, 0.940679, 0.300618, 0.870605, 0.303596, 0.849330, 0.315044, 0.940679, 0.303596, 0.849330, 0.300618, 0.870605, 0.315044, 0.940679, 0.315044, 0.940679, 0.303596, 0.849330, 0.318208, 0.915139, 0.303596, 0.849330, 0.315044, 0.940679, 0.318208, 0.915139, 0.318208, 0.915139, 0.303596, 0.849330, 0.320861, 0.885274, 0.303596, 0.849330, 0.318208, 0.915139, 0.320861, 0.885274, 0.303596, 0.849330, 0.305890, 0.821698, 0.320861, 0.885274, 0.305890, 0.821698, 0.303596, 0.849330, 0.320861, 0.885274, 0.320861, 0.885274, 0.305890, 0.821698, 0.323002, 0.851084, 0.305890, 0.821698, 0.320861, 0.885274, 0.323002, 0.851084, 0.305890, 0.821698, 0.307528, 0.787428, 0.323002, 0.851084, 0.307528, 0.787428, 0.305890, 0.821698, 0.323002, 0.851084, 0.323002, 0.851084, 0.307528, 0.787428, 0.327761, 0.767498, 0.307528, 0.787428, 0.323002, 0.851084, 0.327761, 0.767498, 0.323002, 0.851084, 0.327761, 0.767498, 0.330147, 0.805554, 0.327761, 0.767498, 0.323002, 0.851084, 0.330147, 0.805554, 0.323002, 0.851084, 0.330147, 0.805554, 0.333487, 0.838170, 0.330147, 0.805554, 0.323002, 0.851084, 0.333487, 0.838170, 0.326330, 0.886699, 0.323002, 0.851084, 0.333487, 0.838170, 0.323002, 0.851084, 0.326330, 0.886699, 0.333487, 0.838170, 0.337648, 0.864328, 0.326330, 0.886699, 0.333487, 0.838170, 0.326330, 0.886699, 0.337648, 0.864328, 0.333487, 0.838170, 0.337648, 0.864328, 0.329969, 0.917331, 0.326330, 0.886699, 0.329969, 0.917331, 0.337648, 0.864328, 0.326330, 0.886699, 0.333918, 0.942981, 0.329969, 0.917331, 0.337648, 0.864328, 0.329969, 0.917331, 0.333918, 0.942981, 0.337648, 0.864328, 0.333918, 0.942981, 0.337648, 0.864328, 0.342500, 0.883012, 0.337648, 0.864328, 0.333918, 0.942981, 0.342500, 0.883012, 0.338177, 0.963648, 0.333918, 0.942981, 0.342500, 0.883012, 0.333918, 0.942981, 0.338177, 0.963648, 0.342500, 0.883012, 0.338177, 0.963648, 0.342500, 0.883012, 0.342787, 0.979552, 0.342500, 0.883012, 0.338177, 0.963648, 0.342787, 0.979552, 0.342787, 0.979552, 0.342500, 0.883012, 0.348041, 0.894222, 0.342500, 0.883012, 0.342787, 0.979552, 0.348041, 0.894222, 0.347788, 0.990912, 0.342787, 0.979552, 0.348041, 0.894222, 0.342787, 0.979552, 0.347788, 0.990912, 0.348041, 0.894222, 0.347788, 0.990912, 0.348041, 0.894222, 0.354272, 0.897959, 0.348041, 0.894222, 0.347788, 0.990912, 0.354272, 0.897959, 0.353180, 0.997728, 0.347788, 0.990912, 0.354272, 0.897959, 0.347788, 0.990912, 0.353180, 0.997728, 0.354272, 0.897959, 0.358963, 1, 0.353180, 0.997728, 0.354272, 0.897959, 0.353180, 0.997728, 0.358963, 1, 0.354272, 0.897959, 0.358963, 1, 0.354272, 0.897959, 0.359307, 0.894900, 0.354272, 0.897959, 0.358963, 1, 0.359307, 0.894900, 0.366550, 0.995167, 0.358963, 1, 0.359307, 0.894900, 0.358963, 1, 0.366550, 0.995167, 0.359307, 0.894900, 0.359307, 0.894900, 0.363653, 0.885722, 0.366550, 0.995167, 0.363653, 0.885722, 0.359307, 0.894900, 0.366550, 0.995167, 0.366550, 0.995167, 0.363653, 0.885722, 0.373218, 0.980668, 0.363653, 0.885722, 0.366550, 0.995167, 0.373218, 0.980668, 0.363653, 0.885722, 0.367309, 0.870426, 0.373218, 0.980668, 0.367309, 0.870426, 0.363653, 0.885722, 0.373218, 0.980668, 0.373218, 0.980668, 0.367309, 0.870426, 0.378966, 0.956503, 0.367309, 0.870426, 0.373218, 0.980668, 0.378966, 0.956503, 0.367309, 0.870426, 0.370275, 0.849011, 0.378966, 0.956503, 0.370275, 0.849011, 0.367309, 0.870426, 0.378966, 0.956503, 0.378966, 0.956503, 0.370275, 0.849011, 0.383795, 0.922672, 0.370275, 0.849011, 0.378966, 0.956503, 0.383795, 0.922672, 0.383795, 0.922672, 0.370275, 0.849011, 0.387617, 0.879933, 0.370275, 0.849011, 0.383795, 0.922672, 0.387617, 0.879933, 0.387617, 0.879933, 0.370275, 0.849011, 0.372568, 0.821279, 0.370275, 0.849011, 0.387617, 0.879933, 0.372568, 0.821279, 0.387617, 0.879933, 0.372568, 0.821279, 0.390348, 0.829042, 0.372568, 0.821279, 0.387617, 0.879933, 0.390348, 0.829042, 0.372568, 0.821279, 0.374207, 0.787030, 0.390348, 0.829042, 0.374207, 0.787030, 0.372568, 0.821279, 0.390348, 0.829042, 0.390348, 0.829042, 0.374207, 0.787030, 0.391986, 0.770000, 0.374207, 0.787030, 0.390348, 0.829042, 0.391986, 0.770000, 0.374207, 0.787030, 0.375190, 0.746263, 0.391986, 0.770000, 0.375190, 0.746263, 0.374207, 0.787030, 0.391986, 0.770000, 0.391986, 0.770000, 0.375190, 0.746263, 0.392532, 0.702806, 0.375190, 0.746263, 0.391986, 0.770000, 0.392532, 0.702806, 0.392532, 0.702806, 0.375190, 0.746263, 0.375517, 0.698980, 0.375190, 0.746263, 0.392532, 0.702806, 0.375517, 0.698980, 0.375517, 0.698980, 0.392532, 0.265306, 0.392532, 0.702806, 0.392532, 0.265306, 0.375517, 0.698980, 0.392532, 0.702806, 0.375517, 0.698980, 0.375517, 0.265306, 0.392532, 0.265306, 0.375517, 0.265306, 0.375517, 0.698980, 0.392532, 0.265306, 0.308511, 0.746522, 0.327761, 0.767498, 0.307528, 0.787428, 0.327761, 0.767498, 0.308511, 0.746522, 0.307528, 0.787428, 0.327761, 0.767498, 0.308511, 0.746522, 0.326330, 0.724002, 0.308511, 0.746522, 0.327761, 0.767498, 0.326330, 0.724002, 0.308511, 0.746522, 0.308838, 0.698980, 0.326330, 0.724002, 0.308838, 0.698980, 0.308511, 0.746522, 0.326330, 0.724002, 0.308838, 0.698980, 0.325853, 0.675064, 0.326330, 0.724002, 0.325853, 0.675064, 0.308838, 0.698980, 0.326330, 0.724002, 0.325853, 0.675064, 0.308838, 0.698980, 0.308838, 0.265306, 0.308838, 0.698980, 0.325853, 0.675064, 0.308838, 0.265306, 0.308838, 0.265306, 0.325853, 0.265306, 0.325853, 0.675064, 0.325853, 0.265306, 0.308838, 0.265306, 0.325853, 0.675064, 0.261083, 0.767100, 0.242159, 0.979592, 0.259651, 0.723742, 0.242159, 0.979592, 0.261083, 0.767100, 0.259651, 0.723742, 0.259174, 0.675064, 0.259651, 0.723742, 0.242159, 0.979592, 0.259651, 0.723742, 0.259174, 0.675064, 0.242159, 0.979592, 0.259174, 0.675064, 0.242159, 0.979592, 0.242159, 0.265306, 0.242159, 0.979592, 0.259174, 0.675064, 0.242159, 0.265306, 0.242159, 0.265306, 0.259174, 0.265306, 0.259174, 0.675064, 0.259174, 0.265306, 0.242159, 0.265306, 0.259174, 0.675064, 0.513382, 0.655612, 0.513382, 0.591837, 0.513382, 0.655612, 0.513382, 0.655612, 0.513382, 0.591837, 0.513382, 0.591837, 0.513382, 0.591837, 0.435574, 0.591837, 0.513382, 0.591837, 0.513382, 0.591837, 0.435574, 0.591837, 0.435574, 0.591837, 0.435574, 0.591837, 0.436646, 0.535605, 0.435574, 0.591837, 0.435574, 0.591837, 0.436646, 0.535605, 0.436646, 0.535605, 0.436646, 0.535605, 0.438759, 0.486488, 0.436646, 0.535605, 0.436646, 0.535605, 0.438759, 0.486488, 0.438759, 0.486488, 0.438759, 0.486488, 0.441912, 0.444485, 0.438759, 0.486488, 0.438759, 0.486488, 0.441912, 0.444485, 0.441912, 0.444485, 0.441912, 0.444485, 0.446105, 0.409598, 0.441912, 0.444485, 0.441912, 0.444485, 0.446105, 0.409598, 0.446105, 0.409598, 0.446105, 0.409598, 0.451281, 0.382185, 0.446105, 0.409598, 0.446105, 0.409598, 0.451281, 0.382185, 0.451281, 0.382185, 0.451281, 0.382185, 0.457383, 0.362604, 0.451281, 0.382185, 0.451281, 0.382185, 0.457383, 0.362604, 0.457383, 0.362604, 0.457383, 0.362604, 0.464410, 0.350855, 0.457383, 0.362604, 0.457383, 0.362604, 0.464410, 0.350855, 0.464410, 0.350855, 0.464410, 0.350855, 0.472363, 0.346939, 0.464410, 0.350855, 0.464410, 0.350855, 0.472363, 0.346939, 0.472363, 0.346939, 0.472363, 0.346939, 0.477200, 0.347895, 0.472363, 0.346939, 0.472363, 0.346939, 0.477200, 0.347895, 0.477200, 0.347895, 0.477200, 0.347895, 0.481962, 0.350765, 0.477200, 0.347895, 0.477200, 0.347895, 0.481962, 0.350765, 0.481962, 0.350765, 0.481962, 0.350765, 0.486650, 0.355548, 0.481962, 0.350765, 0.481962, 0.350765, 0.486650, 0.355548, 0.486650, 0.355548, 0.486650, 0.355548, 0.491263, 0.362245, 0.486650, 0.355548, 0.486650, 0.355548, 0.491263, 0.362245, 0.491263, 0.362245, 0.491263, 0.362245, 0.495830, 0.370855, 0.491263, 0.362245, 0.491263, 0.362245, 0.495830, 0.370855, 0.495830, 0.370855, 0.495830, 0.370855, 0.500379, 0.381378, 0.495830, 0.370855, 0.495830, 0.370855, 0.500379, 0.381378, 0.500379, 0.381378, 0.500379, 0.381378, 0.504912, 0.393814, 0.500379, 0.381378, 0.500379, 0.381378, 0.504912, 0.393814, 0.504912, 0.393814, 0.504912, 0.393814, 0.509427, 0.408163, 0.504912, 0.393814, 0.504912, 0.393814, 0.509427, 0.408163, 0.509427, 0.408163, 0.509427, 0.408163, 0.509427, 0.306122, 0.509427, 0.408163, 0.509427, 0.408163, 0.509427, 0.306122, 0.509427, 0.306122, 0.509427, 0.306122, 0.504846, 0.292032, 0.509427, 0.306122, 0.509427, 0.306122, 0.504846, 0.292032, 0.504846, 0.292032, 0.504846, 0.292032, 0.500207, 0.279735, 0.504846, 0.292032, 0.504846, 0.292032, 0.500207, 0.279735, 0.500207, 0.279735, 0.500207, 0.279735, 0.495511, 0.269232, 0.500207, 0.279735, 0.500207, 0.279735, 0.495511, 0.269232, 0.495511, 0.269232, 0.495511, 0.269232, 0.490757, 0.260523, 0.495511, 0.269232, 0.495511, 0.269232, 0.490757, 0.260523, 0.490757, 0.260523, 0.490757, 0.260523, 0.485957, 0.253687, 0.490757, 0.260523, 0.490757, 0.260523, 0.485957, 0.253687, 0.485957, 0.253687, 0.485957, 0.253687, 0.481123, 0.248804, 0.485957, 0.253687, 0.485957, 0.253687, 0.481123, 0.248804, 0.481123, 0.248804, 0.481123, 0.248804, 0.476254, 0.245875, 0.481123, 0.248804, 0.481123, 0.248804, 0.476254, 0.245875, 0.476254, 0.245875, 0.476254, 0.245875, 0.471351, 0.244898, 0.476254, 0.245875, 0.476254, 0.245875, 0.471351, 0.244898, 0.471351, 0.244898, 0.471351, 0.244898, 0.459668, 0.251156, 0.471351, 0.244898, 0.471351, 0.244898, 0.459668, 0.251156, 0.459668, 0.251156, 0.459668, 0.251156, 0.449267, 0.269930, 0.459668, 0.251156, 0.459668, 0.251156, 0.449267, 0.269930, 0.449267, 0.269930, 0.449267, 0.269930, 0.440147, 0.301220, 0.449267, 0.269930, 0.449267, 0.269930, 0.440147, 0.301220, 0.440147, 0.301220, 0.440147, 0.301220, 0.432309, 0.345026, 0.440147, 0.301220, 0.440147, 0.301220, 0.432309, 0.345026, 0.432309, 0.345026, 0.432309, 0.345026, 0.426012, 0.399514, 0.432309, 0.345026, 0.432309, 0.345026, 0.426012, 0.399514, 0.426012, 0.399514, 0.426012, 0.399514, 0.421514, 0.462851, 0.426012, 0.399514, 0.426012, 0.399514, 0.421514, 0.462851, 0.421514, 0.462851, 0.421514, 0.462851, 0.418816, 0.535037, 0.421514, 0.462851, 0.421514, 0.462851, 0.418816, 0.535037, 0.418816, 0.535037, 0.418816, 0.535037, 0.417916, 0.616071, 0.418816, 0.535037, 0.418816, 0.535037, 0.417916, 0.616071, 0.417916, 0.616071, 0.417916, 0.616071, 0.418770, 0.699837, 0.417916, 0.616071, 0.417916, 0.616071, 0.418770, 0.699837, 0.418770, 0.699837, 0.418770, 0.699837, 0.421330, 0.774474, 0.418770, 0.699837, 0.418770, 0.699837, 0.421330, 0.774474, 0.421330, 0.774474, 0.421330, 0.774474, 0.425598, 0.839983, 0.421330, 0.774474, 0.421330, 0.774474, 0.425598, 0.839983, 0.425598, 0.839983, 0.425598, 0.839983, 0.431574, 0.896365, 0.425598, 0.839983, 0.425598, 0.839983, 0.431574, 0.896365, 0.431574, 0.896365, 0.431574, 0.896365, 0.438997, 0.941705, 0.431574, 0.896365, 0.431574, 0.896365, 0.438997, 0.941705, 0.438997, 0.941705, 0.438997, 0.941705, 0.447611, 0.974091, 0.438997, 0.941705, 0.438997, 0.941705, 0.447611, 0.974091, 0.447611, 0.974091, 0.447611, 0.974091, 0.457415, 0.993523, 0.447611, 0.974091, 0.447611, 0.974091, 0.457415, 0.993523, 0.457415, 0.993523, 0.457415, 0.993523, 0.468408, 1, 0.457415, 0.993523, 0.457415, 0.993523, 0.468408, 1, 0.468408, 1, 0.468408, 1, 0.478258, 0.994210, 0.468408, 1, 0.468408, 1, 0.478258, 0.994210, 0.478258, 0.994210, 0.478258, 0.994210, 0.487021, 0.976842, 0.478258, 0.994210, 0.478258, 0.994210, 0.487021, 0.976842, 0.487021, 0.976842, 0.487021, 0.976842, 0.494697, 0.947893, 0.487021, 0.976842, 0.487021, 0.976842, 0.494697, 0.947893, 0.494697, 0.947893, 0.494697, 0.947893, 0.501288, 0.907366, 0.494697, 0.947893, 0.494697, 0.947893, 0.501288, 0.907366, 0.501288, 0.907366, 0.501288, 0.907366, 0.506579, 0.856894, 0.501288, 0.907366, 0.501288, 0.907366, 0.506579, 0.856894, 0.506579, 0.856894, 0.506579, 0.856894, 0.510358, 0.798111, 0.506579, 0.856894, 0.506579, 0.856894, 0.510358, 0.798111, 0.510358, 0.798111, 0.510358, 0.798111, 0.512626, 0.731017, 0.510358, 0.798111, 0.510358, 0.798111, 0.512626, 0.731017, 0.512626, 0.731017, 0.418770, 0.699837, 0.417916, 0.616071, 0.436126, 0.693240, 0.417916, 0.616071, 0.418770, 0.699837, 0.436126, 0.693240, 0.421330, 0.774474, 0.418770, 0.699837, 0.436126, 0.693240, 0.418770, 0.699837, 0.421330, 0.774474, 0.436126, 0.693240, 0.421330, 0.774474, 0.436126, 0.693240, 0.437221, 0.739278, 0.436126, 0.693240, 0.421330, 0.774474, 0.437221, 0.739278, 0.439219, 0.779815, 0.421330, 0.774474, 0.437221, 0.739278, 0.421330, 0.774474, 0.439219, 0.779815, 0.437221, 0.739278, 0.439219, 0.779815, 0.425598, 0.839983, 0.421330, 0.774474, 0.425598, 0.839983, 0.439219, 0.779815, 0.421330, 0.774474, 0.425598, 0.839983, 0.439219, 0.779815, 0.442119, 0.814852, 0.439219, 0.779815, 0.425598, 0.839983, 0.442119, 0.814852, 0.431574, 0.896365, 0.425598, 0.839983, 0.442119, 0.814852, 0.425598, 0.839983, 0.431574, 0.896365, 0.442119, 0.814852, 0.431574, 0.896365, 0.442119, 0.814852, 0.445921, 0.844388, 0.442119, 0.814852, 0.431574, 0.896365, 0.445921, 0.844388, 0.438997, 0.941705, 0.431574, 0.896365, 0.445921, 0.844388, 0.431574, 0.896365, 0.438997, 0.941705, 0.445921, 0.844388, 0.438997, 0.941705, 0.445921, 0.844388, 0.450528, 0.867825, 0.445921, 0.844388, 0.438997, 0.941705, 0.450528, 0.867825, 0.447611, 0.974091, 0.438997, 0.941705, 0.450528, 0.867825, 0.438997, 0.941705, 0.447611, 0.974091, 0.450528, 0.867825, 0.447611, 0.974091, 0.450528, 0.867825, 0.455842, 0.884566, 0.450528, 0.867825, 0.447611, 0.974091, 0.455842, 0.884566, 0.447611, 0.974091, 0.455842, 0.884566, 0.457415, 0.993523, 0.455842, 0.884566, 0.447611, 0.974091, 0.457415, 0.993523, 0.457415, 0.993523, 0.455842, 0.884566, 0.461864, 0.894611, 0.455842, 0.884566, 0.457415, 0.993523, 0.461864, 0.894611, 0.457415, 0.993523, 0.461864, 0.894611, 0.468408, 1, 0.461864, 0.894611, 0.457415, 0.993523, 0.468408, 1, 0.468408, 1, 0.461864, 0.894611, 0.468592, 0.897959, 0.461864, 0.894611, 0.468408, 1, 0.468592, 0.897959, 0.468408, 1, 0.468592, 0.897959, 0.474538, 0.894491, 0.468592, 0.897959, 0.468408, 1, 0.474538, 0.894491, 0.468408, 1, 0.474538, 0.894491, 0.478258, 0.994210, 0.474538, 0.894491, 0.468408, 1, 0.478258, 0.994210, 0.478258, 0.994210, 0.474538, 0.894491, 0.479870, 0.884088, 0.474538, 0.894491, 0.478258, 0.994210, 0.479870, 0.884088, 0.478258, 0.994210, 0.479870, 0.884088, 0.487021, 0.976842, 0.479870, 0.884088, 0.478258, 0.994210, 0.487021, 0.976842, 0.479870, 0.884088, 0.484586, 0.866749, 0.487021, 0.976842, 0.484586, 0.866749, 0.479870, 0.884088, 0.487021, 0.976842, 0.487021, 0.976842, 0.484586, 0.866749, 0.494697, 0.947893, 0.484586, 0.866749, 0.487021, 0.976842, 0.494697, 0.947893, 0.484586, 0.866749, 0.488688, 0.842474, 0.494697, 0.947893, 0.488688, 0.842474, 0.484586, 0.866749, 0.494697, 0.947893, 0.494697, 0.947893, 0.488688, 0.842474, 0.501288, 0.907366, 0.488688, 0.842474, 0.494697, 0.947893, 0.501288, 0.907366, 0.488688, 0.842474, 0.492019, 0.812380, 0.501288, 0.907366, 0.492019, 0.812380, 0.488688, 0.842474, 0.501288, 0.907366, 0.501288, 0.907366, 0.492019, 0.812380, 0.506579, 0.856894, 0.492019, 0.812380, 0.501288, 0.907366, 0.506579, 0.856894, 0.492019, 0.812380, 0.494424, 0.777583, 0.506579, 0.856894, 0.494424, 0.777583, 0.492019, 0.812380, 0.506579, 0.856894, 0.506579, 0.856894, 0.494424, 0.777583, 0.510358, 0.798111, 0.494424, 0.777583, 0.506579, 0.856894, 0.510358, 0.798111, 0.494424, 0.777583, 0.495904, 0.738082, 0.510358, 0.798111, 0.495904, 0.738082, 0.494424, 0.777583, 0.510358, 0.798111, 0.510358, 0.798111, 0.495904, 0.738082, 0.512626, 0.731017, 0.495904, 0.738082, 0.510358, 0.798111, 0.512626, 0.731017, 0.495904, 0.738082, 0.496459, 0.693878, 0.512626, 0.731017, 0.496459, 0.693878, 0.495904, 0.738082, 0.512626, 0.731017, 0.512626, 0.731017, 0.496459, 0.693878, 0.513382, 0.655612, 0.496459, 0.693878, 0.512626, 0.731017, 0.513382, 0.655612, 0.513382, 0.591837, 0.513382, 0.655612, 0.496459, 0.693878, 0.513382, 0.655612, 0.513382, 0.591837, 0.496459, 0.693878, 0.496459, 0.693878, 0.435574, 0.591837, 0.513382, 0.591837, 0.435574, 0.591837, 0.496459, 0.693878, 0.513382, 0.591837, 0.496459, 0.693878, 0.436126, 0.693240, 0.435574, 0.591837, 0.436126, 0.693240, 0.496459, 0.693878, 0.435574, 0.591837, 0.417916, 0.616071, 0.435574, 0.591837, 0.436126, 0.693240, 0.435574, 0.591837, 0.417916, 0.616071, 0.436126, 0.693240, 0.417916, 0.616071, 0.418816, 0.535037, 0.435574, 0.591837, 0.418816, 0.535037, 0.417916, 0.616071, 0.435574, 0.591837, 0.435574, 0.591837, 0.418816, 0.535037, 0.436646, 0.535605, 0.418816, 0.535037, 0.435574, 0.591837, 0.436646, 0.535605, 0.418816, 0.535037, 0.421514, 0.462851, 0.436646, 0.535605, 0.421514, 0.462851, 0.418816, 0.535037, 0.436646, 0.535605, 0.436646, 0.535605, 0.421514, 0.462851, 0.438759, 0.486488, 0.421514, 0.462851, 0.436646, 0.535605, 0.438759, 0.486488, 0.421514, 0.462851, 0.426012, 0.399514, 0.438759, 0.486488, 0.426012, 0.399514, 0.421514, 0.462851, 0.438759, 0.486488, 0.438759, 0.486488, 0.426012, 0.399514, 0.441912, 0.444485, 0.426012, 0.399514, 0.438759, 0.486488, 0.441912, 0.444485, 0.426012, 0.399514, 0.432309, 0.345026, 0.441912, 0.444485, 0.432309, 0.345026, 0.426012, 0.399514, 0.441912, 0.444485, 0.441912, 0.444485, 0.432309, 0.345026, 0.446105, 0.409598, 0.432309, 0.345026, 0.441912, 0.444485, 0.446105, 0.409598, 0.432309, 0.345026, 0.440147, 0.301220, 0.446105, 0.409598, 0.440147, 0.301220, 0.432309, 0.345026, 0.446105, 0.409598, 0.446105, 0.409598, 0.440147, 0.301220, 0.451281, 0.382185, 0.440147, 0.301220, 0.446105, 0.409598, 0.451281, 0.382185, 0.451281, 0.382185, 0.440147, 0.301220, 0.449267, 0.269930, 0.440147, 0.301220, 0.451281, 0.382185, 0.449267, 0.269930, 0.457383, 0.362604, 0.451281, 0.382185, 0.449267, 0.269930, 0.451281, 0.382185, 0.457383, 0.362604, 0.449267, 0.269930, 0.457383, 0.362604, 0.449267, 0.269930, 0.459668, 0.251156, 0.449267, 0.269930, 0.457383, 0.362604, 0.459668, 0.251156, 0.464410, 0.350855, 0.457383, 0.362604, 0.459668, 0.251156, 0.457383, 0.362604, 0.464410, 0.350855, 0.459668, 0.251156, 0.464410, 0.350855, 0.459668, 0.251156, 0.471351, 0.244898, 0.459668, 0.251156, 0.464410, 0.350855, 0.471351, 0.244898, 0.472363, 0.346939, 0.464410, 0.350855, 0.471351, 0.244898, 0.464410, 0.350855, 0.472363, 0.346939, 0.471351, 0.244898, 0.472363, 0.346939, 0.471351, 0.244898, 0.476254, 0.245875, 0.471351, 0.244898, 0.472363, 0.346939, 0.476254, 0.245875, 0.472363, 0.346939, 0.476254, 0.245875, 0.477200, 0.347895, 0.476254, 0.245875, 0.472363, 0.346939, 0.477200, 0.347895, 0.477200, 0.347895, 0.476254, 0.245875, 0.481123, 0.248804, 0.476254, 0.245875, 0.477200, 0.347895, 0.481123, 0.248804, 0.477200, 0.347895, 0.481123, 0.248804, 0.481962, 0.350765, 0.481123, 0.248804, 0.477200, 0.347895, 0.481962, 0.350765, 0.481962, 0.350765, 0.481123, 0.248804, 0.485957, 0.253687, 0.481123, 0.248804, 0.481962, 0.350765, 0.485957, 0.253687, 0.481962, 0.350765, 0.485957, 0.253687, 0.486650, 0.355548, 0.485957, 0.253687, 0.481962, 0.350765, 0.486650, 0.355548, 0.486650, 0.355548, 0.485957, 0.253687, 0.490757, 0.260523, 0.485957, 0.253687, 0.486650, 0.355548, 0.490757, 0.260523, 0.486650, 0.355548, 0.490757, 0.260523, 0.491263, 0.362245, 0.490757, 0.260523, 0.486650, 0.355548, 0.491263, 0.362245, 0.491263, 0.362245, 0.490757, 0.260523, 0.495511, 0.269232, 0.490757, 0.260523, 0.491263, 0.362245, 0.495511, 0.269232, 0.495830, 0.370855, 0.491263, 0.362245, 0.495511, 0.269232, 0.491263, 0.362245, 0.495830, 0.370855, 0.495511, 0.269232, 0.500207, 0.279735, 0.495830, 0.370855, 0.495511, 0.269232, 0.495830, 0.370855, 0.500207, 0.279735, 0.495511, 0.269232, 0.495830, 0.370855, 0.500207, 0.279735, 0.504846, 0.292032, 0.500207, 0.279735, 0.495830, 0.370855, 0.504846, 0.292032, 0.500379, 0.381378, 0.495830, 0.370855, 0.504846, 0.292032, 0.495830, 0.370855, 0.500379, 0.381378, 0.504846, 0.292032, 0.500379, 0.381378, 0.504846, 0.292032, 0.509427, 0.306122, 0.504846, 0.292032, 0.500379, 0.381378, 0.509427, 0.306122, 0.504912, 0.393814, 0.500379, 0.381378, 0.509427, 0.306122, 0.500379, 0.381378, 0.504912, 0.393814, 0.509427, 0.306122, 0.504912, 0.393814, 0.509427, 0.306122, 0.509427, 0.408163, 0.509427, 0.306122, 0.504912, 0.393814, 0.509427, 0.408163, 0.496459, 0.693878, 0.495904, 0.738082, 0.496459, 0.693878, 0.496459, 0.693878, 0.495904, 0.738082, 0.495904, 0.738082, 0.495904, 0.738082, 0.494424, 0.777583, 0.495904, 0.738082, 0.495904, 0.738082, 0.494424, 0.777583, 0.494424, 0.777583, 0.494424, 0.777583, 0.492019, 0.812380, 0.494424, 0.777583, 0.494424, 0.777583, 0.492019, 0.812380, 0.492019, 0.812380, 0.492019, 0.812380, 0.488688, 0.842474, 0.492019, 0.812380, 0.492019, 0.812380, 0.488688, 0.842474, 0.488688, 0.842474, 0.488688, 0.842474, 0.484586, 0.866749, 0.488688, 0.842474, 0.488688, 0.842474, 0.484586, 0.866749, 0.484586, 0.866749, 0.484586, 0.866749, 0.479870, 0.884088, 0.484586, 0.866749, 0.484586, 0.866749, 0.479870, 0.884088, 0.479870, 0.884088, 0.479870, 0.884088, 0.474538, 0.894491, 0.479870, 0.884088, 0.479870, 0.884088, 0.474538, 0.894491, 0.474538, 0.894491, 0.474538, 0.894491, 0.468592, 0.897959, 0.474538, 0.894491, 0.474538, 0.894491, 0.468592, 0.897959, 0.468592, 0.897959, 0.468592, 0.897959, 0.461864, 0.894611, 0.468592, 0.897959, 0.468592, 0.897959, 0.461864, 0.894611, 0.461864, 0.894611, 0.461864, 0.894611, 0.455842, 0.884566, 0.461864, 0.894611, 0.461864, 0.894611, 0.455842, 0.884566, 0.455842, 0.884566, 0.455842, 0.884566, 0.450528, 0.867825, 0.455842, 0.884566, 0.455842, 0.884566, 0.450528, 0.867825, 0.450528, 0.867825, 0.450528, 0.867825, 0.445921, 0.844388, 0.450528, 0.867825, 0.450528, 0.867825, 0.445921, 0.844388, 0.445921, 0.844388, 0.445921, 0.844388, 0.442119, 0.814852, 0.445921, 0.844388, 0.445921, 0.844388, 0.442119, 0.814852, 0.442119, 0.814852, 0.442119, 0.814852, 0.439219, 0.779815, 0.442119, 0.814852, 0.442119, 0.814852, 0.439219, 0.779815, 0.439219, 0.779815, 0.439219, 0.779815, 0.437221, 0.739278, 0.439219, 0.779815, 0.439219, 0.779815, 0.437221, 0.739278, 0.437221, 0.739278, 0.437221, 0.739278, 0.436126, 0.693240, 0.437221, 0.739278, 0.437221, 0.739278, 0.436126, 0.693240, 0.436126, 0.693240, 0.638830, 0.897959, 0.632381, 0.893355, 0.638830, 0.897959, 0.638830, 0.897959, 0.632381, 0.893355, 0.632381, 0.893355, 0.632381, 0.893355, 0.626644, 0.879544, 0.632381, 0.893355, 0.632381, 0.893355, 0.626644, 0.879544, 0.626644, 0.879544, 0.626644, 0.879544, 0.621620, 0.856525, 0.626644, 0.879544, 0.626644, 0.879544, 0.621620, 0.856525, 0.621620, 0.856525, 0.621620, 0.856525, 0.617309, 0.824298, 0.621620, 0.856525, 0.621620, 0.856525, 0.617309, 0.824298, 0.617309, 0.824298, 0.617309, 0.824298, 0.613849, 0.784060, 0.617309, 0.824298, 0.617309, 0.824298, 0.613849, 0.784060, 0.613849, 0.784060, 0.613849, 0.784060, 0.611377, 0.737006, 0.613849, 0.784060, 0.613849, 0.784060, 0.611377, 0.737006, 0.611377, 0.737006, 0.611377, 0.737006, 0.609894, 0.683135, 0.611377, 0.737006, 0.611377, 0.737006, 0.609894, 0.683135, 0.609894, 0.683135, 0.609894, 0.683135, 0.609399, 0.622449, 0.609894, 0.683135, 0.609894, 0.683135, 0.609399, 0.622449, 0.609399, 0.622449, 0.609399, 0.622449, 0.609891, 0.561763, 0.609399, 0.622449, 0.609399, 0.622449, 0.609891, 0.561763, 0.609891, 0.561763, 0.609891, 0.561763, 0.611365, 0.507892, 0.609891, 0.561763, 0.609891, 0.561763, 0.611365, 0.507892, 0.611365, 0.507892, 0.611365, 0.507892, 0.613823, 0.460838, 0.611365, 0.507892, 0.611365, 0.507892, 0.613823, 0.460838, 0.613823, 0.460838, 0.613823, 0.460838, 0.617263, 0.420599, 0.613823, 0.460838, 0.613823, 0.460838, 0.617263, 0.420599, 0.617263, 0.420599, 0.617263, 0.420599, 0.621560, 0.388373, 0.617263, 0.420599, 0.617263, 0.420599, 0.621560, 0.388373, 0.621560, 0.388373, 0.621560, 0.388373, 0.626586, 0.365354, 0.621560, 0.388373, 0.621560, 0.388373, 0.626586, 0.365354, 0.626586, 0.365354, 0.626586, 0.365354, 0.632343, 0.351543, 0.626586, 0.365354, 0.626586, 0.365354, 0.632343, 0.351543, 0.632343, 0.351543, 0.632343, 0.351543, 0.638830, 0.346939, 0.632343, 0.351543, 0.632343, 0.351543, 0.638830, 0.346939, 0.638830, 0.346939, 0.638830, 0.346939, 0.645239, 0.351562, 0.638830, 0.346939, 0.638830, 0.346939, 0.645239, 0.351562, 0.645239, 0.351562, 0.645239, 0.351562, 0.650947, 0.365434, 0.645239, 0.351562, 0.645239, 0.351562, 0.650947, 0.365434, 0.650947, 0.365434, 0.650947, 0.365434, 0.655954, 0.388552, 0.650947, 0.365434, 0.650947, 0.365434, 0.655954, 0.388552, 0.655954, 0.388552, 0.655954, 0.388552, 0.660259, 0.420918, 0.655954, 0.388552, 0.655954, 0.388552, 0.660259, 0.420918, 0.660259, 0.420918, 0.660259, 0.420918, 0.663720, 0.461256, 0.660259, 0.420918, 0.660259, 0.420918, 0.663720, 0.461256, 0.663720, 0.461256, 0.663720, 0.461256, 0.666191, 0.508291, 0.663720, 0.461256, 0.663720, 0.461256, 0.666191, 0.508291, 0.666191, 0.508291, 0.666191, 0.508291, 0.667675, 0.562022, 0.666191, 0.508291, 0.666191, 0.508291, 0.667675, 0.562022, 0.667675, 0.562022, 0.667675, 0.562022, 0.668169, 0.622449, 0.667675, 0.562022, 0.667675, 0.562022, 0.668169, 0.622449, 0.668169, 0.622449, 0.668169, 0.622449, 0.667675, 0.682617, 0.668169, 0.622449, 0.668169, 0.622449, 0.667675, 0.682617, 0.667675, 0.682617, 0.667675, 0.682617, 0.666191, 0.736208, 0.667675, 0.682617, 0.667675, 0.682617, 0.666191, 0.736208, 0.666191, 0.736208, 0.666191, 0.736208, 0.663720, 0.783223, 0.666191, 0.736208, 0.666191, 0.736208, 0.663720, 0.783223, 0.663720, 0.783223, 0.663720, 0.783223, 0.660259, 0.823661, 0.663720, 0.783223, 0.663720, 0.783223, 0.660259, 0.823661, 0.660259, 0.823661, 0.660259, 0.823661, 0.655954, 0.856166, 0.660259, 0.823661, 0.660259, 0.823661, 0.655954, 0.856166, 0.655954, 0.856166, 0.655954, 0.856166, 0.650947, 0.879385, 0.655954, 0.856166, 0.655954, 0.856166, 0.650947, 0.879385, 0.650947, 0.879385, 0.650947, 0.879385, 0.645239, 0.893316, 0.650947, 0.879385, 0.650947, 0.879385, 0.645239, 0.893316, 0.645239, 0.893316, 0.638830, 1, 0.649275, 0.993742, 0.638830, 1, 0.638830, 1, 0.649275, 0.993742, 0.649275, 0.993742, 0.649275, 0.993742, 0.658535, 0.974968, 0.649275, 0.993742, 0.649275, 0.993742, 0.658535, 0.974968, 0.658535, 0.974968, 0.658535, 0.974968, 0.666611, 0.943678, 0.658535, 0.974968, 0.658535, 0.974968, 0.666611, 0.943678, 0.666611, 0.943678, 0.666611, 0.943678, 0.673503, 0.899872, 0.666611, 0.943678, 0.666611, 0.943678, 0.673503, 0.899872, 0.673503, 0.899872, 0.673503, 0.899872, 0.679016, 0.844986, 0.673503, 0.899872, 0.673503, 0.899872, 0.679016, 0.844986, 0.679016, 0.844986, 0.679016, 0.844986, 0.682953, 0.780453, 0.679016, 0.844986, 0.679016, 0.844986, 0.682953, 0.780453, 0.682953, 0.780453, 0.682953, 0.780453, 0.685316, 0.706274, 0.682953, 0.780453, 0.682953, 0.780453, 0.685316, 0.706274, 0.685316, 0.706274, 0.685316, 0.706274, 0.686103, 0.622449, 0.685316, 0.706274, 0.685316, 0.706274, 0.686103, 0.622449, 0.686103, 0.622449, 0.686103, 0.622449, 0.685316, 0.538883, 0.686103, 0.622449, 0.686103, 0.622449, 0.685316, 0.538883, 0.685316, 0.538883, 0.685316, 0.538883, 0.682953, 0.464844, 0.685316, 0.538883, 0.685316, 0.538883, 0.682953, 0.464844, 0.682953, 0.464844, 0.682953, 0.464844, 0.679016, 0.400331, 0.682953, 0.464844, 0.682953, 0.464844, 0.679016, 0.400331, 0.679016, 0.400331, 0.679016, 0.400331, 0.673503, 0.345344, 0.679016, 0.400331, 0.679016, 0.400331, 0.673503, 0.345344, 0.673503, 0.345344, 0.673503, 0.345344, 0.666611, 0.301399, 0.673503, 0.345344, 0.673503, 0.345344, 0.666611, 0.301399, 0.666611, 0.301399, 0.666611, 0.301399, 0.658535, 0.270010, 0.666611, 0.301399, 0.666611, 0.301399, 0.658535, 0.270010, 0.658535, 0.270010, 0.658535, 0.270010, 0.649275, 0.251176, 0.658535, 0.270010, 0.658535, 0.270010, 0.649275, 0.251176, 0.649275, 0.251176, 0.649275, 0.251176, 0.638830, 0.244898, 0.649275, 0.251176, 0.649275, 0.251176, 0.638830, 0.244898, 0.638830, 0.244898, 0.638830, 0.244898, 0.628348, 0.251176, 0.638830, 0.244898, 0.638830, 0.244898, 0.628348, 0.251176, 0.628348, 0.251176, 0.628348, 0.251176, 0.619068, 0.270010, 0.628348, 0.251176, 0.628348, 0.251176, 0.619068, 0.270010, 0.619068, 0.270010, 0.619068, 0.270010, 0.610989, 0.301399, 0.619068, 0.270010, 0.619068, 0.270010, 0.610989, 0.301399, 0.610989, 0.301399, 0.610989, 0.301399, 0.604111, 0.345344, 0.610989, 0.301399, 0.610989, 0.301399, 0.604111, 0.345344, 0.604111, 0.345344, 0.604111, 0.345344, 0.598619, 0.400331, 0.604111, 0.345344, 0.604111, 0.345344, 0.598619, 0.400331, 0.598619, 0.400331, 0.598619, 0.400331, 0.594696, 0.464844, 0.598619, 0.400331, 0.598619, 0.400331, 0.594696, 0.464844, 0.594696, 0.464844, 0.594696, 0.464844, 0.592342, 0.538883, 0.594696, 0.464844, 0.594696, 0.464844, 0.592342, 0.538883, 0.592342, 0.538883, 0.592342, 0.538883, 0.591557, 0.622449, 0.592342, 0.538883, 0.592342, 0.538883, 0.591557, 0.622449, 0.591557, 0.622449, 0.591557, 0.622449, 0.592342, 0.706274, 0.591557, 0.622449, 0.591557, 0.622449, 0.592342, 0.706274, 0.592342, 0.706274, 0.592342, 0.706274, 0.594696, 0.780453, 0.592342, 0.706274, 0.592342, 0.706274, 0.594696, 0.780453, 0.594696, 0.780453, 0.594696, 0.780453, 0.598619, 0.844986, 0.594696, 0.780453, 0.594696, 0.780453, 0.598619, 0.844986, 0.598619, 0.844986, 0.598619, 0.844986, 0.604111, 0.899872, 0.598619, 0.844986, 0.598619, 0.844986, 0.604111, 0.899872, 0.604111, 0.899872, 0.604111, 0.899872, 0.610989, 0.943678, 0.604111, 0.899872, 0.604111, 0.899872, 0.610989, 0.943678, 0.610989, 0.943678, 0.610989, 0.943678, 0.619068, 0.974968, 0.610989, 0.943678, 0.610989, 0.943678, 0.619068, 0.974968, 0.619068, 0.974968, 0.619068, 0.974968, 0.628348, 0.993742, 0.619068, 0.974968, 0.619068, 0.974968, 0.628348, 0.993742, 0.628348, 0.993742, 0.609894, 0.683135, 0.592342, 0.706274, 0.591557, 0.622449, 0.592342, 0.706274, 0.609894, 0.683135, 0.591557, 0.622449, 0.611377, 0.737006, 0.592342, 0.706274, 0.609894, 0.683135, 0.592342, 0.706274, 0.611377, 0.737006, 0.609894, 0.683135, 0.611377, 0.737006, 0.594696, 0.780453, 0.592342, 0.706274, 0.594696, 0.780453, 0.611377, 0.737006, 0.592342, 0.706274, 0.594696, 0.780453, 0.611377, 0.737006, 0.613849, 0.784060, 0.611377, 0.737006, 0.594696, 0.780453, 0.613849, 0.784060, 0.598619, 0.844986, 0.594696, 0.780453, 0.613849, 0.784060, 0.594696, 0.780453, 0.598619, 0.844986, 0.613849, 0.784060, 0.617309, 0.824298, 0.598619, 0.844986, 0.613849, 0.784060, 0.598619, 0.844986, 0.617309, 0.824298, 0.613849, 0.784060, 0.617309, 0.824298, 0.604111, 0.899872, 0.598619, 0.844986, 0.604111, 0.899872, 0.617309, 0.824298, 0.598619, 0.844986, 0.610989, 0.943678, 0.604111, 0.899872, 0.617309, 0.824298, 0.604111, 0.899872, 0.610989, 0.943678, 0.617309, 0.824298, 0.610989, 0.943678, 0.617309, 0.824298, 0.621620, 0.856525, 0.617309, 0.824298, 0.610989, 0.943678, 0.621620, 0.856525, 0.619068, 0.974968, 0.610989, 0.943678, 0.621620, 0.856525, 0.610989, 0.943678, 0.619068, 0.974968, 0.621620, 0.856525, 0.619068, 0.974968, 0.621620, 0.856525, 0.626644, 0.879544, 0.621620, 0.856525, 0.619068, 0.974968, 0.626644, 0.879544, 0.619068, 0.974968, 0.626644, 0.879544, 0.628348, 0.993742, 0.626644, 0.879544, 0.619068, 0.974968, 0.628348, 0.993742, 0.628348, 0.993742, 0.626644, 0.879544, 0.632381, 0.893355, 0.626644, 0.879544, 0.628348, 0.993742, 0.632381, 0.893355, 0.628348, 0.993742, 0.632381, 0.893355, 0.638830, 1, 0.632381, 0.893355, 0.628348, 0.993742, 0.638830, 1, 0.632381, 0.893355, 0.638830, 0.897959, 0.638830, 1, 0.638830, 0.897959, 0.632381, 0.893355, 0.638830, 1, 0.638830, 1, 0.638830, 0.897959, 0.645239, 0.893316, 0.638830, 0.897959, 0.638830, 1, 0.645239, 0.893316, 0.638830, 1, 0.645239, 0.893316, 0.649275, 0.993742, 0.645239, 0.893316, 0.638830, 1, 0.649275, 0.993742, 0.649275, 0.993742, 0.645239, 0.893316, 0.650947, 0.879385, 0.645239, 0.893316, 0.649275, 0.993742, 0.650947, 0.879385, 0.649275, 0.993742, 0.650947, 0.879385, 0.658535, 0.974968, 0.650947, 0.879385, 0.649275, 0.993742, 0.658535, 0.974968, 0.650947, 0.879385, 0.655954, 0.856166, 0.658535, 0.974968, 0.655954, 0.856166, 0.650947, 0.879385, 0.658535, 0.974968, 0.658535, 0.974968, 0.655954, 0.856166, 0.666611, 0.943678, 0.655954, 0.856166, 0.658535, 0.974968, 0.666611, 0.943678, 0.655954, 0.856166, 0.660259, 0.823661, 0.666611, 0.943678, 0.660259, 0.823661, 0.655954, 0.856166, 0.666611, 0.943678, 0.666611, 0.943678, 0.660259, 0.823661, 0.673503, 0.899872, 0.660259, 0.823661, 0.666611, 0.943678, 0.673503, 0.899872, 0.673503, 0.899872, 0.660259, 0.823661, 0.679016, 0.844986, 0.660259, 0.823661, 0.673503, 0.899872, 0.679016, 0.844986, 0.679016, 0.844986, 0.660259, 0.823661, 0.663720, 0.783223, 0.660259, 0.823661, 0.679016, 0.844986, 0.663720, 0.783223, 0.679016, 0.844986, 0.663720, 0.783223, 0.682953, 0.780453, 0.663720, 0.783223, 0.679016, 0.844986, 0.682953, 0.780453, 0.663720, 0.783223, 0.666191, 0.736208, 0.682953, 0.780453, 0.666191, 0.736208, 0.663720, 0.783223, 0.682953, 0.780453, 0.682953, 0.780453, 0.666191, 0.736208, 0.685316, 0.706274, 0.666191, 0.736208, 0.682953, 0.780453, 0.685316, 0.706274, 0.666191, 0.736208, 0.667675, 0.682617, 0.685316, 0.706274, 0.667675, 0.682617, 0.666191, 0.736208, 0.685316, 0.706274, 0.685316, 0.706274, 0.667675, 0.682617, 0.686103, 0.622449, 0.667675, 0.682617, 0.685316, 0.706274, 0.686103, 0.622449, 0.686103, 0.622449, 0.667675, 0.682617, 0.668169, 0.622449, 0.667675, 0.682617, 0.686103, 0.622449, 0.668169, 0.622449, 0.686103, 0.622449, 0.668169, 0.622449, 0.667675, 0.562022, 0.668169, 0.622449, 0.686103, 0.622449, 0.667675, 0.562022, 0.686103, 0.622449, 0.667675, 0.562022, 0.685316, 0.538883, 0.667675, 0.562022, 0.686103, 0.622449, 0.685316, 0.538883, 0.667675, 0.562022, 0.666191, 0.508291, 0.685316, 0.538883, 0.666191, 0.508291, 0.667675, 0.562022, 0.685316, 0.538883, 0.666191, 0.508291, 0.682953, 0.464844, 0.685316, 0.538883, 0.682953, 0.464844, 0.666191, 0.508291, 0.685316, 0.538883, 0.666191, 0.508291, 0.663720, 0.461256, 0.682953, 0.464844, 0.663720, 0.461256, 0.666191, 0.508291, 0.682953, 0.464844, 0.663720, 0.461256, 0.679016, 0.400331, 0.682953, 0.464844, 0.679016, 0.400331, 0.663720, 0.461256, 0.682953, 0.464844, 0.663720, 0.461256, 0.660259, 0.420918, 0.679016, 0.400331, 0.660259, 0.420918, 0.663720, 0.461256, 0.679016, 0.400331, 0.660259, 0.420918, 0.673503, 0.345344, 0.679016, 0.400331, 0.673503, 0.345344, 0.660259, 0.420918, 0.679016, 0.400331, 0.673503, 0.345344, 0.660259, 0.420918, 0.666611, 0.301399, 0.660259, 0.420918, 0.673503, 0.345344, 0.666611, 0.301399, 0.660259, 0.420918, 0.655954, 0.388552, 0.666611, 0.301399, 0.655954, 0.388552, 0.660259, 0.420918, 0.666611, 0.301399, 0.666611, 0.301399, 0.655954, 0.388552, 0.658535, 0.270010, 0.655954, 0.388552, 0.666611, 0.301399, 0.658535, 0.270010, 0.655954, 0.388552, 0.650947, 0.365434, 0.658535, 0.270010, 0.650947, 0.365434, 0.655954, 0.388552, 0.658535, 0.270010, 0.658535, 0.270010, 0.650947, 0.365434, 0.649275, 0.251176, 0.650947, 0.365434, 0.658535, 0.270010, 0.649275, 0.251176, 0.650947, 0.365434, 0.645239, 0.351562, 0.649275, 0.251176, 0.645239, 0.351562, 0.650947, 0.365434, 0.649275, 0.251176, 0.649275, 0.251176, 0.645239, 0.351562, 0.638830, 0.244898, 0.645239, 0.351562, 0.649275, 0.251176, 0.638830, 0.244898, 0.645239, 0.351562, 0.638830, 0.346939, 0.638830, 0.244898, 0.638830, 0.346939, 0.645239, 0.351562, 0.638830, 0.244898, 0.638830, 0.346939, 0.632343, 0.351543, 0.638830, 0.244898, 0.632343, 0.351543, 0.638830, 0.346939, 0.638830, 0.244898, 0.632343, 0.351543, 0.628348, 0.251176, 0.638830, 0.244898, 0.628348, 0.251176, 0.632343, 0.351543, 0.638830, 0.244898, 0.632343, 0.351543, 0.626586, 0.365354, 0.628348, 0.251176, 0.626586, 0.365354, 0.632343, 0.351543, 0.628348, 0.251176, 0.626586, 0.365354, 0.619068, 0.270010, 0.628348, 0.251176, 0.619068, 0.270010, 0.626586, 0.365354, 0.628348, 0.251176, 0.626586, 0.365354, 0.621560, 0.388373, 0.619068, 0.270010, 0.621560, 0.388373, 0.626586, 0.365354, 0.619068, 0.270010, 0.621560, 0.388373, 0.610989, 0.301399, 0.619068, 0.270010, 0.610989, 0.301399, 0.621560, 0.388373, 0.619068, 0.270010, 0.621560, 0.388373, 0.617263, 0.420599, 0.610989, 0.301399, 0.617263, 0.420599, 0.621560, 0.388373, 0.610989, 0.301399, 0.617263, 0.420599, 0.604111, 0.345344, 0.610989, 0.301399, 0.604111, 0.345344, 0.617263, 0.420599, 0.610989, 0.301399, 0.598619, 0.400331, 0.604111, 0.345344, 0.617263, 0.420599, 0.604111, 0.345344, 0.598619, 0.400331, 0.617263, 0.420599, 0.613823, 0.460838, 0.598619, 0.400331, 0.617263, 0.420599, 0.598619, 0.400331, 0.613823, 0.460838, 0.617263, 0.420599, 0.594696, 0.464844, 0.598619, 0.400331, 0.613823, 0.460838, 0.598619, 0.400331, 0.594696, 0.464844, 0.613823, 0.460838, 0.611365, 0.507892, 0.594696, 0.464844, 0.613823, 0.460838, 0.594696, 0.464844, 0.611365, 0.507892, 0.613823, 0.460838, 0.592342, 0.538883, 0.594696, 0.464844, 0.611365, 0.507892, 0.594696, 0.464844, 0.592342, 0.538883, 0.611365, 0.507892, 0.609891, 0.561763, 0.592342, 0.538883, 0.611365, 0.507892, 0.592342, 0.538883, 0.609891, 0.561763, 0.611365, 0.507892, 0.591557, 0.622449, 0.592342, 0.538883, 0.609891, 0.561763, 0.592342, 0.538883, 0.591557, 0.622449, 0.609891, 0.561763, 0.609399, 0.622449, 0.591557, 0.622449, 0.609891, 0.561763, 0.591557, 0.622449, 0.609399, 0.622449, 0.609891, 0.561763, 0.609894, 0.683135, 0.591557, 0.622449, 0.609399, 0.622449, 0.591557, 0.622449, 0.609894, 0.683135, 0.609399, 0.622449, 0.701554, 0.979592, 0.719489, 0.979592, 0.701554, 0.979592, 0.701554, 0.979592, 0.719489, 0.979592, 0.719489, 0.979592, 0.719489, 0.979592, 0.751678, 0.380102, 0.719489, 0.979592, 0.719489, 0.979592, 0.751678, 0.380102, 0.751678, 0.380102, 0.751678, 0.380102, 0.783868, 0.979592, 0.751678, 0.380102, 0.751678, 0.380102, 0.783868, 0.979592, 0.783868, 0.979592, 0.783868, 0.979592, 0.801803, 0.979592, 0.783868, 0.979592, 0.783868, 0.979592, 0.801803, 0.979592, 0.801803, 0.979592, 0.801803, 0.979592, 0.763175, 0.265306, 0.801803, 0.979592, 0.801803, 0.979592, 0.763175, 0.265306, 0.763175, 0.265306, 0.763175, 0.265306, 0.740182, 0.265306, 0.763175, 0.265306, 0.763175, 0.265306, 0.740182, 0.265306, 0.740182, 0.265306, 0.719489, 0.979592, 0.701554, 0.979592, 0.751678, 0.380102, 0.701554, 0.979592, 0.719489, 0.979592, 0.751678, 0.380102, 0.701554, 0.979592, 0.740182, 0.265306, 0.751678, 0.380102, 0.740182, 0.265306, 0.701554, 0.979592, 0.751678, 0.380102, 0.751678, 0.380102, 0.740182, 0.265306, 0.763175, 0.265306, 0.740182, 0.265306, 0.751678, 0.380102, 0.763175, 0.265306, 0.763175, 0.265306, 0.801803, 0.979592, 0.751678, 0.380102, 0.801803, 0.979592, 0.763175, 0.265306, 0.751678, 0.380102, 0.783868, 0.979592, 0.751678, 0.380102, 0.801803, 0.979592, 0.751678, 0.380102, 0.783868, 0.979592, 0.801803, 0.979592, 0.913639, 0.655612, 0.913639, 0.591837, 0.913639, 0.655612, 0.913639, 0.655612, 0.913639, 0.591837, 0.913639, 0.591837, 0.913639, 0.591837, 0.835832, 0.591837, 0.913639, 0.591837, 0.913639, 0.591837, 0.835832, 0.591837, 0.835832, 0.591837, 0.835832, 0.591837, 0.836904, 0.535605, 0.835832, 0.591837, 0.835832, 0.591837, 0.836904, 0.535605, 0.836904, 0.535605, 0.836904, 0.535605, 0.839016, 0.486488, 0.836904, 0.535605, 0.836904, 0.535605, 0.839016, 0.486488, 0.839016, 0.486488, 0.839016, 0.486488, 0.842169, 0.444485, 0.839016, 0.486488, 0.839016, 0.486488, 0.842169, 0.444485, 0.842169, 0.444485, 0.842169, 0.444485, 0.846363, 0.409598, 0.842169, 0.444485, 0.842169, 0.444485, 0.846363, 0.409598, 0.846363, 0.409598, 0.846363, 0.409598, 0.851539, 0.382185, 0.846363, 0.409598, 0.846363, 0.409598, 0.851539, 0.382185, 0.851539, 0.382185, 0.851539, 0.382185, 0.857641, 0.362604, 0.851539, 0.382185, 0.851539, 0.382185, 0.857641, 0.362604, 0.857641, 0.362604, 0.857641, 0.362604, 0.864668, 0.350855, 0.857641, 0.362604, 0.857641, 0.362604, 0.864668, 0.350855, 0.864668, 0.350855, 0.864668, 0.350855, 0.872620, 0.346939, 0.864668, 0.350855, 0.864668, 0.350855, 0.872620, 0.346939, 0.872620, 0.346939, 0.872620, 0.346939, 0.877457, 0.347895, 0.872620, 0.346939, 0.872620, 0.346939, 0.877457, 0.347895, 0.877457, 0.347895, 0.877457, 0.347895, 0.882220, 0.350765, 0.877457, 0.347895, 0.877457, 0.347895, 0.882220, 0.350765, 0.882220, 0.350765, 0.882220, 0.350765, 0.886907, 0.355548, 0.882220, 0.350765, 0.882220, 0.350765, 0.886907, 0.355548, 0.886907, 0.355548, 0.886907, 0.355548, 0.891520, 0.362245, 0.886907, 0.355548, 0.886907, 0.355548, 0.891520, 0.362245, 0.891520, 0.362245, 0.891520, 0.362245, 0.896087, 0.370855, 0.891520, 0.362245, 0.891520, 0.362245, 0.896087, 0.370855, 0.896087, 0.370855, 0.896087, 0.370855, 0.900637, 0.381378, 0.896087, 0.370855, 0.896087, 0.370855, 0.900637, 0.381378, 0.900637, 0.381378, 0.900637, 0.381378, 0.905169, 0.393814, 0.900637, 0.381378, 0.900637, 0.381378, 0.905169, 0.393814, 0.905169, 0.393814, 0.905169, 0.393814, 0.909685, 0.408163, 0.905169, 0.393814, 0.905169, 0.393814, 0.909685, 0.408163, 0.909685, 0.408163, 0.909685, 0.408163, 0.909685, 0.306122, 0.909685, 0.408163, 0.909685, 0.408163, 0.909685, 0.306122, 0.909685, 0.306122, 0.909685, 0.306122, 0.905103, 0.292032, 0.909685, 0.306122, 0.909685, 0.306122, 0.905103, 0.292032, 0.905103, 0.292032, 0.905103, 0.292032, 0.900464, 0.279735, 0.905103, 0.292032, 0.905103, 0.292032, 0.900464, 0.279735, 0.900464, 0.279735, 0.900464, 0.279735, 0.895768, 0.269232, 0.900464, 0.279735, 0.900464, 0.279735, 0.895768, 0.269232, 0.895768, 0.269232, 0.895768, 0.269232, 0.891014, 0.260523, 0.895768, 0.269232, 0.895768, 0.269232, 0.891014, 0.260523, 0.891014, 0.260523, 0.891014, 0.260523, 0.886215, 0.253687, 0.891014, 0.260523, 0.891014, 0.260523, 0.886215, 0.253687, 0.886215, 0.253687, 0.886215, 0.253687, 0.881380, 0.248804, 0.886215, 0.253687, 0.886215, 0.253687, 0.881380, 0.248804, 0.881380, 0.248804, 0.881380, 0.248804, 0.876512, 0.245875, 0.881380, 0.248804, 0.881380, 0.248804, 0.876512, 0.245875, 0.876512, 0.245875, 0.876512, 0.245875, 0.871609, 0.244898, 0.876512, 0.245875, 0.876512, 0.245875, 0.871609, 0.244898, 0.871609, 0.244898, 0.871609, 0.244898, 0.859925, 0.251156, 0.871609, 0.244898, 0.871609, 0.244898, 0.859925, 0.251156, 0.859925, 0.251156, 0.859925, 0.251156, 0.849524, 0.269930, 0.859925, 0.251156, 0.859925, 0.251156, 0.849524, 0.269930, 0.849524, 0.269930, 0.849524, 0.269930, 0.840405, 0.301220, 0.849524, 0.269930, 0.849524, 0.269930, 0.840405, 0.301220, 0.840405, 0.301220, 0.840405, 0.301220, 0.832567, 0.345026, 0.840405, 0.301220, 0.840405, 0.301220, 0.832567, 0.345026, 0.832567, 0.345026, 0.832567, 0.345026, 0.826270, 0.399514, 0.832567, 0.345026, 0.832567, 0.345026, 0.826270, 0.399514, 0.826270, 0.399514, 0.826270, 0.399514, 0.821772, 0.462851, 0.826270, 0.399514, 0.826270, 0.399514, 0.821772, 0.462851, 0.821772, 0.462851, 0.821772, 0.462851, 0.819073, 0.535037, 0.821772, 0.462851, 0.821772, 0.462851, 0.819073, 0.535037, 0.819073, 0.535037, 0.819073, 0.535037, 0.818173, 0.616071, 0.819073, 0.535037, 0.819073, 0.535037, 0.818173, 0.616071, 0.818173, 0.616071, 0.818173, 0.616071, 0.819027, 0.699837, 0.818173, 0.616071, 0.818173, 0.616071, 0.819027, 0.699837, 0.819027, 0.699837, 0.819027, 0.699837, 0.821588, 0.774474, 0.819027, 0.699837, 0.819027, 0.699837, 0.821588, 0.774474, 0.821588, 0.774474, 0.821588, 0.774474, 0.825856, 0.839983, 0.821588, 0.774474, 0.821588, 0.774474, 0.825856, 0.839983, 0.825856, 0.839983, 0.825856, 0.839983, 0.831831, 0.896365, 0.825856, 0.839983, 0.825856, 0.839983, 0.831831, 0.896365, 0.831831, 0.896365, 0.831831, 0.896365, 0.839255, 0.941705, 0.831831, 0.896365, 0.831831, 0.896365, 0.839255, 0.941705, 0.839255, 0.941705, 0.839255, 0.941705, 0.847869, 0.974091, 0.839255, 0.941705, 0.839255, 0.941705, 0.847869, 0.974091, 0.847869, 0.974091, 0.847869, 0.974091, 0.857672, 0.993523, 0.847869, 0.974091, 0.847869, 0.974091, 0.857672, 0.993523, 0.857672, 0.993523, 0.857672, 0.993523, 0.868666, 1, 0.857672, 0.993523, 0.857672, 0.993523, 0.868666, 1, 0.868666, 1, 0.868666, 1, 0.878515, 0.994210, 0.868666, 1, 0.868666, 1, 0.878515, 0.994210, 0.878515, 0.994210, 0.878515, 0.994210, 0.887278, 0.976842, 0.878515, 0.994210, 0.878515, 0.994210, 0.887278, 0.976842, 0.887278, 0.976842, 0.887278, 0.976842, 0.894955, 0.947893, 0.887278, 0.976842, 0.887278, 0.976842, 0.894955, 0.947893, 0.894955, 0.947893, 0.894955, 0.947893, 0.901545, 0.907366, 0.894955, 0.947893, 0.894955, 0.947893, 0.901545, 0.907366, 0.901545, 0.907366, 0.901545, 0.907366, 0.906836, 0.856894, 0.901545, 0.907366, 0.901545, 0.907366, 0.906836, 0.856894, 0.906836, 0.856894, 0.906836, 0.856894, 0.910616, 0.798111, 0.906836, 0.856894, 0.906836, 0.856894, 0.910616, 0.798111, 0.910616, 0.798111, 0.910616, 0.798111, 0.912883, 0.731017, 0.910616, 0.798111, 0.910616, 0.798111, 0.912883, 0.731017, 0.912883, 0.731017, 0.819027, 0.699837, 0.818173, 0.616071, 0.836384, 0.693240, 0.818173, 0.616071, 0.819027, 0.699837, 0.836384, 0.693240, 0.821588, 0.774474, 0.819027, 0.699837, 0.836384, 0.693240, 0.819027, 0.699837, 0.821588, 0.774474, 0.836384, 0.693240, 0.821588, 0.774474, 0.836384, 0.693240, 0.837479, 0.739278, 0.836384, 0.693240, 0.821588, 0.774474, 0.837479, 0.739278, 0.839476, 0.779815, 0.821588, 0.774474, 0.837479, 0.739278, 0.821588, 0.774474, 0.839476, 0.779815, 0.837479, 0.739278, 0.839476, 0.779815, 0.825856, 0.839983, 0.821588, 0.774474, 0.825856, 0.839983, 0.839476, 0.779815, 0.821588, 0.774474, 0.825856, 0.839983, 0.839476, 0.779815, 0.842376, 0.814852, 0.839476, 0.779815, 0.825856, 0.839983, 0.842376, 0.814852, 0.831831, 0.896365, 0.825856, 0.839983, 0.842376, 0.814852, 0.825856, 0.839983, 0.831831, 0.896365, 0.842376, 0.814852, 0.831831, 0.896365, 0.842376, 0.814852, 0.846179, 0.844388, 0.842376, 0.814852, 0.831831, 0.896365, 0.846179, 0.844388, 0.839255, 0.941705, 0.831831, 0.896365, 0.846179, 0.844388, 0.831831, 0.896365, 0.839255, 0.941705, 0.846179, 0.844388, 0.839255, 0.941705, 0.846179, 0.844388, 0.850786, 0.867825, 0.846179, 0.844388, 0.839255, 0.941705, 0.850786, 0.867825, 0.847869, 0.974091, 0.839255, 0.941705, 0.850786, 0.867825, 0.839255, 0.941705, 0.847869, 0.974091, 0.850786, 0.867825, 0.847869, 0.974091, 0.850786, 0.867825, 0.856100, 0.884566, 0.850786, 0.867825, 0.847869, 0.974091, 0.856100, 0.884566, 0.847869, 0.974091, 0.856100, 0.884566, 0.857672, 0.993523, 0.856100, 0.884566, 0.847869, 0.974091, 0.857672, 0.993523, 0.857672, 0.993523, 0.856100, 0.884566, 0.862121, 0.894611, 0.856100, 0.884566, 0.857672, 0.993523, 0.862121, 0.894611, 0.857672, 0.993523, 0.862121, 0.894611, 0.868666, 1, 0.862121, 0.894611, 0.857672, 0.993523, 0.868666, 1, 0.868666, 1, 0.862121, 0.894611, 0.868849, 0.897959, 0.862121, 0.894611, 0.868666, 1, 0.868849, 0.897959, 0.868666, 1, 0.868849, 0.897959, 0.874796, 0.894491, 0.868849, 0.897959, 0.868666, 1, 0.874796, 0.894491, 0.868666, 1, 0.874796, 0.894491, 0.878515, 0.994210, 0.874796, 0.894491, 0.868666, 1, 0.878515, 0.994210, 0.878515, 0.994210, 0.874796, 0.894491, 0.880127, 0.884088, 0.874796, 0.894491, 0.878515, 0.994210, 0.880127, 0.884088, 0.878515, 0.994210, 0.880127, 0.884088, 0.887278, 0.976842, 0.880127, 0.884088, 0.878515, 0.994210, 0.887278, 0.976842, 0.880127, 0.884088, 0.884844, 0.866749, 0.887278, 0.976842, 0.884844, 0.866749, 0.880127, 0.884088, 0.887278, 0.976842, 0.887278, 0.976842, 0.884844, 0.866749, 0.894955, 0.947893, 0.884844, 0.866749, 0.887278, 0.976842, 0.894955, 0.947893, 0.884844, 0.866749, 0.888945, 0.842474, 0.894955, 0.947893, 0.888945, 0.842474, 0.884844, 0.866749, 0.894955, 0.947893, 0.894955, 0.947893, 0.888945, 0.842474, 0.901545, 0.907366, 0.888945, 0.842474, 0.894955, 0.947893, 0.901545, 0.907366, 0.888945, 0.842474, 0.892276, 0.812380, 0.901545, 0.907366, 0.892276, 0.812380, 0.888945, 0.842474, 0.901545, 0.907366, 0.901545, 0.907366, 0.892276, 0.812380, 0.906836, 0.856894, 0.892276, 0.812380, 0.901545, 0.907366, 0.906836, 0.856894, 0.892276, 0.812380, 0.894682, 0.777583, 0.906836, 0.856894, 0.894682, 0.777583, 0.892276, 0.812380, 0.906836, 0.856894, 0.906836, 0.856894, 0.894682, 0.777583, 0.910616, 0.798111, 0.894682, 0.777583, 0.906836, 0.856894, 0.910616, 0.798111, 0.894682, 0.777583, 0.896162, 0.738082, 0.910616, 0.798111, 0.896162, 0.738082, 0.894682, 0.777583, 0.910616, 0.798111, 0.910616, 0.798111, 0.896162, 0.738082, 0.912883, 0.731017, 0.896162, 0.738082, 0.910616, 0.798111, 0.912883, 0.731017, 0.896162, 0.738082, 0.896717, 0.693878, 0.912883, 0.731017, 0.896717, 0.693878, 0.896162, 0.738082, 0.912883, 0.731017, 0.912883, 0.731017, 0.896717, 0.693878, 0.913639, 0.655612, 0.896717, 0.693878, 0.912883, 0.731017, 0.913639, 0.655612, 0.913639, 0.591837, 0.913639, 0.655612, 0.896717, 0.693878, 0.913639, 0.655612, 0.913639, 0.591837, 0.896717, 0.693878, 0.896717, 0.693878, 0.835832, 0.591837, 0.913639, 0.591837, 0.835832, 0.591837, 0.896717, 0.693878, 0.913639, 0.591837, 0.896717, 0.693878, 0.836384, 0.693240, 0.835832, 0.591837, 0.836384, 0.693240, 0.896717, 0.693878, 0.835832, 0.591837, 0.818173, 0.616071, 0.835832, 0.591837, 0.836384, 0.693240, 0.835832, 0.591837, 0.818173, 0.616071, 0.836384, 0.693240, 0.818173, 0.616071, 0.819073, 0.535037, 0.835832, 0.591837, 0.819073, 0.535037, 0.818173, 0.616071, 0.835832, 0.591837, 0.835832, 0.591837, 0.819073, 0.535037, 0.836904, 0.535605, 0.819073, 0.535037, 0.835832, 0.591837, 0.836904, 0.535605, 0.819073, 0.535037, 0.821772, 0.462851, 0.836904, 0.535605, 0.821772, 0.462851, 0.819073, 0.535037, 0.836904, 0.535605, 0.836904, 0.535605, 0.821772, 0.462851, 0.839016, 0.486488, 0.821772, 0.462851, 0.836904, 0.535605, 0.839016, 0.486488, 0.821772, 0.462851, 0.826270, 0.399514, 0.839016, 0.486488, 0.826270, 0.399514, 0.821772, 0.462851, 0.839016, 0.486488, 0.839016, 0.486488, 0.826270, 0.399514, 0.842169, 0.444485, 0.826270, 0.399514, 0.839016, 0.486488, 0.842169, 0.444485, 0.826270, 0.399514, 0.832567, 0.345026, 0.842169, 0.444485, 0.832567, 0.345026, 0.826270, 0.399514, 0.842169, 0.444485, 0.842169, 0.444485, 0.832567, 0.345026, 0.846363, 0.409598, 0.832567, 0.345026, 0.842169, 0.444485, 0.846363, 0.409598, 0.832567, 0.345026, 0.840405, 0.301220, 0.846363, 0.409598, 0.840405, 0.301220, 0.832567, 0.345026, 0.846363, 0.409598, 0.846363, 0.409598, 0.840405, 0.301220, 0.851539, 0.382185, 0.840405, 0.301220, 0.846363, 0.409598, 0.851539, 0.382185, 0.851539, 0.382185, 0.840405, 0.301220, 0.849524, 0.269930, 0.840405, 0.301220, 0.851539, 0.382185, 0.849524, 0.269930, 0.857641, 0.362604, 0.851539, 0.382185, 0.849524, 0.269930, 0.851539, 0.382185, 0.857641, 0.362604, 0.849524, 0.269930, 0.857641, 0.362604, 0.849524, 0.269930, 0.859925, 0.251156, 0.849524, 0.269930, 0.857641, 0.362604, 0.859925, 0.251156, 0.864668, 0.350855, 0.857641, 0.362604, 0.859925, 0.251156, 0.857641, 0.362604, 0.864668, 0.350855, 0.859925, 0.251156, 0.864668, 0.350855, 0.859925, 0.251156, 0.871609, 0.244898, 0.859925, 0.251156, 0.864668, 0.350855, 0.871609, 0.244898, 0.872620, 0.346939, 0.864668, 0.350855, 0.871609, 0.244898, 0.864668, 0.350855, 0.872620, 0.346939, 0.871609, 0.244898, 0.872620, 0.346939, 0.871609, 0.244898, 0.876512, 0.245875, 0.871609, 0.244898, 0.872620, 0.346939, 0.876512, 0.245875, 0.872620, 0.346939, 0.876512, 0.245875, 0.877457, 0.347895, 0.876512, 0.245875, 0.872620, 0.346939, 0.877457, 0.347895, 0.877457, 0.347895, 0.876512, 0.245875, 0.881380, 0.248804, 0.876512, 0.245875, 0.877457, 0.347895, 0.881380, 0.248804, 0.877457, 0.347895, 0.881380, 0.248804, 0.882220, 0.350765, 0.881380, 0.248804, 0.877457, 0.347895, 0.882220, 0.350765, 0.882220, 0.350765, 0.881380, 0.248804, 0.886215, 0.253687, 0.881380, 0.248804, 0.882220, 0.350765, 0.886215, 0.253687, 0.882220, 0.350765, 0.886215, 0.253687, 0.886907, 0.355548, 0.886215, 0.253687, 0.882220, 0.350765, 0.886907, 0.355548, 0.886907, 0.355548, 0.886215, 0.253687, 0.891014, 0.260523, 0.886215, 0.253687, 0.886907, 0.355548, 0.891014, 0.260523, 0.886907, 0.355548, 0.891014, 0.260523, 0.891520, 0.362245, 0.891014, 0.260523, 0.886907, 0.355548, 0.891520, 0.362245, 0.891520, 0.362245, 0.891014, 0.260523, 0.895768, 0.269232, 0.891014, 0.260523, 0.891520, 0.362245, 0.895768, 0.269232, 0.896087, 0.370855, 0.891520, 0.362245, 0.895768, 0.269232, 0.891520, 0.362245, 0.896087, 0.370855, 0.895768, 0.269232, 0.900464, 0.279735, 0.896087, 0.370855, 0.895768, 0.269232, 0.896087, 0.370855, 0.900464, 0.279735, 0.895768, 0.269232, 0.896087, 0.370855, 0.900464, 0.279735, 0.905103, 0.292032, 0.900464, 0.279735, 0.896087, 0.370855, 0.905103, 0.292032, 0.900637, 0.381378, 0.896087, 0.370855, 0.905103, 0.292032, 0.896087, 0.370855, 0.900637, 0.381378, 0.905103, 0.292032, 0.900637, 0.381378, 0.905103, 0.292032, 0.909685, 0.306122, 0.905103, 0.292032, 0.900637, 0.381378, 0.909685, 0.306122, 0.905169, 0.393814, 0.900637, 0.381378, 0.909685, 0.306122, 0.900637, 0.381378, 0.905169, 0.393814, 0.909685, 0.306122, 0.905169, 0.393814, 0.909685, 0.306122, 0.909685, 0.408163, 0.909685, 0.306122, 0.905169, 0.393814, 0.909685, 0.408163, 0.896717, 0.693878, 0.896162, 0.738082, 0.896717, 0.693878, 0.896717, 0.693878, 0.896162, 0.738082, 0.896162, 0.738082, 0.896162, 0.738082, 0.894682, 0.777583, 0.896162, 0.738082, 0.896162, 0.738082, 0.894682, 0.777583, 0.894682, 0.777583, 0.894682, 0.777583, 0.892276, 0.812380, 0.894682, 0.777583, 0.894682, 0.777583, 0.892276, 0.812380, 0.892276, 0.812380, 0.892276, 0.812380, 0.888945, 0.842474, 0.892276, 0.812380, 0.892276, 0.812380, 0.888945, 0.842474, 0.888945, 0.842474, 0.888945, 0.842474, 0.884844, 0.866749, 0.888945, 0.842474, 0.888945, 0.842474, 0.884844, 0.866749, 0.884844, 0.866749, 0.884844, 0.866749, 0.880127, 0.884088, 0.884844, 0.866749, 0.884844, 0.866749, 0.880127, 0.884088, 0.880127, 0.884088, 0.880127, 0.884088, 0.874796, 0.894491, 0.880127, 0.884088, 0.880127, 0.884088, 0.874796, 0.894491, 0.874796, 0.894491, 0.874796, 0.894491, 0.868849, 0.897959, 0.874796, 0.894491, 0.874796, 0.894491, 0.868849, 0.897959, 0.868849, 0.897959, 0.868849, 0.897959, 0.862121, 0.894611, 0.868849, 0.897959, 0.868849, 0.897959, 0.862121, 0.894611, 0.862121, 0.894611, 0.862121, 0.894611, 0.856100, 0.884566, 0.862121, 0.894611, 0.862121, 0.894611, 0.856100, 0.884566, 0.856100, 0.884566, 0.856100, 0.884566, 0.850786, 0.867825, 0.856100, 0.884566, 0.856100, 0.884566, 0.850786, 0.867825, 0.850786, 0.867825, 0.850786, 0.867825, 0.846179, 0.844388, 0.850786, 0.867825, 0.850786, 0.867825, 0.846179, 0.844388, 0.846179, 0.844388, 0.846179, 0.844388, 0.842376, 0.814852, 0.846179, 0.844388, 0.846179, 0.844388, 0.842376, 0.814852, 0.842376, 0.814852, 0.842376, 0.814852, 0.839476, 0.779815, 0.842376, 0.814852, 0.842376, 0.814852, 0.839476, 0.779815, 0.839476, 0.779815, 0.839476, 0.779815, 0.837479, 0.739278, 0.839476, 0.779815, 0.839476, 0.779815, 0.837479, 0.739278, 0.837479, 0.739278, 0.837479, 0.739278, 0.836384, 0.693240, 0.837479, 0.739278, 0.837479, 0.739278, 0.836384, 0.693240, 0.836384, 0.693240, 1, 0.877551, 0.998543, 0.882464, 1, 0.877551, 1, 0.877551, 0.998543, 0.882464, 0.998543, 0.882464, 0.998543, 0.882464, 0.997022, 0.886679, 0.998543, 0.882464, 0.998543, 0.882464, 0.997022, 0.886679, 0.997022, 0.886679, 0.997022, 0.886679, 0.995439, 0.890197, 0.997022, 0.886679, 0.997022, 0.886679, 0.995439, 0.890197, 0.995439, 0.890197, 0.995439, 0.890197, 0.993792, 0.893017, 0.995439, 0.890197, 0.995439, 0.890197, 0.993792, 0.893017, 0.993792, 0.893017, 0.993792, 0.893017, 0.992070, 0.895179, 0.993792, 0.893017, 0.993792, 0.893017, 0.992070, 0.895179, 0.992070, 0.895179, 0.992070, 0.895179, 0.990263, 0.896724, 0.992070, 0.895179, 0.992070, 0.895179, 0.990263, 0.896724, 0.990263, 0.896724, 0.990263, 0.896724, 0.988369, 0.897650, 0.990263, 0.896724, 0.990263, 0.896724, 0.988369, 0.897650, 0.988369, 0.897650, 0.988369, 0.897650, 0.986388, 0.897959, 0.988369, 0.897650, 0.988369, 0.897650, 0.986388, 0.897959, 0.986388, 0.897959, 0.986388, 0.897959, 0.979631, 0.893903, 0.986388, 0.897959, 0.986388, 0.897959, 0.979631, 0.893903, 0.979631, 0.893903, 0.979631, 0.893903, 0.973708, 0.881736, 0.979631, 0.893903, 0.979631, 0.893903, 0.973708, 0.881736, 0.973708, 0.881736, 0.973708, 0.881736, 0.968618, 0.861458, 0.973708, 0.881736, 0.973708, 0.881736, 0.968618, 0.861458, 0.968618, 0.861458, 0.968618, 0.861458, 0.964361, 0.833068, 0.968618, 0.861458, 0.968618, 0.861458, 0.964361, 0.833068, 0.964361, 0.833068, 0.964361, 0.833068, 0.961001, 0.797104, 0.964361, 0.833068, 0.964361, 0.833068, 0.961001, 0.797104, 0.961001, 0.797104, 0.961001, 0.797104, 0.958602, 0.754106, 0.961001, 0.797104, 0.961001, 0.797104, 0.958602, 0.754106, 0.958602, 0.754106, 0.958602, 0.754106, 0.957162, 0.704072, 0.958602, 0.754106, 0.958602, 0.754106, 0.957162, 0.704072, 0.957162, 0.704072, 0.957162, 0.704072, 0.956682, 0.647003, 0.957162, 0.704072, 0.957162, 0.704072, 0.956682, 0.647003, 0.956682, 0.647003, 0.956682, 0.647003, 0.956682, 0.265306, 0.956682, 0.647003, 0.956682, 0.647003, 0.956682, 0.265306, 0.956682, 0.265306, 0.956682, 0.265306, 0.939667, 0.265306, 0.956682, 0.265306, 0.956682, 0.265306, 0.939667, 0.265306, 0.939667, 0.265306, 0.939667, 0.265306, 0.939667, 0.979592, 0.939667, 0.265306, 0.939667, 0.265306, 0.939667, 0.979592, 0.939667, 0.979592, 0.939667, 0.979592, 0.956682, 0.979592, 0.939667, 0.979592, 0.939667, 0.979592, 0.956682, 0.979592, 0.956682, 0.979592, 0.956682, 0.979592, 0.956682, 0.877551, 0.956682, 0.979592, 0.956682, 0.979592, 0.956682, 0.877551, 0.956682, 0.877551, 0.956682, 0.877551, 0.959550, 0.906639, 0.956682, 0.877551, 0.956682, 0.877551, 0.959550, 0.906639, 0.959550, 0.906639, 0.959550, 0.906639, 0.962821, 0.931720, 0.959550, 0.906639, 0.959550, 0.906639, 0.962821, 0.931720, 0.962821, 0.931720, 0.962821, 0.931720, 0.966494, 0.952796, 0.962821, 0.931720, 0.962821, 0.931720, 0.966494, 0.952796, 0.966494, 0.952796, 0.966494, 0.952796, 0.970569, 0.969866, 0.966494, 0.952796, 0.966494, 0.952796, 0.970569, 0.969866, 0.970569, 0.969866, 0.970569, 0.969866, 0.975076, 0.983050, 0.970569, 0.969866, 0.970569, 0.969866, 0.975076, 0.983050, 0.975076, 0.983050, 0.975076, 0.983050, 0.980042, 0.992467, 0.975076, 0.983050, 0.975076, 0.983050, 0.980042, 0.992467, 0.980042, 0.992467, 0.980042, 0.992467, 0.985469, 0.998117, 0.980042, 0.992467, 0.980042, 0.992467, 0.985469, 0.998117, 0.985469, 0.998117, 0.985469, 0.998117, 0.991355, 1, 0.985469, 0.998117, 0.985469, 0.998117, 0.991355, 1, 0.991355, 1, 0.991355, 1, 0.992252, 1, 0.991355, 1, 0.991355, 1, 0.992252, 1, 0.992252, 1, 0.992252, 1, 0.993194, 1, 0.992252, 1, 0.992252, 1, 0.993194, 1, 0.993194, 1, 0.993194, 1, 0.994183, 1, 0.993194, 1, 0.993194, 1, 0.994183, 1, 0.994183, 1, 0.994183, 1, 0.995218, 1, 0.994183, 1, 0.994183, 1, 0.995218, 1, 0.995218, 1, 0.995218, 1, 0.996304, 1, 0.995218, 1, 0.995218, 1, 0.996304, 1, 0.996304, 1, 0.996304, 1, 0.997448, 1, 0.996304, 1, 0.996304, 1, 0.997448, 1, 0.997448, 1, 0.997448, 1, 0.998649, 1, 0.997448, 1, 0.997448, 1, 0.998649, 1, 0.998649, 1, 0.998649, 1, 0.999908, 1, 0.998649, 1, 0.998649, 1, 0.999908, 1, 0.999908, 1, 0.956682, 0.979592, 0.939667, 0.979592, 0.956682, 0.877551, 0.939667, 0.979592, 0.956682, 0.979592, 0.956682, 0.877551, 0.958602, 0.754106, 0.956682, 0.877551, 0.939667, 0.979592, 0.956682, 0.877551, 0.958602, 0.754106, 0.939667, 0.979592, 0.956682, 0.877551, 0.958602, 0.754106, 0.961001, 0.797104, 0.958602, 0.754106, 0.956682, 0.877551, 0.961001, 0.797104, 0.956682, 0.877551, 0.961001, 0.797104, 0.964361, 0.833068, 0.961001, 0.797104, 0.956682, 0.877551, 0.964361, 0.833068, 0.959550, 0.906639, 0.956682, 0.877551, 0.964361, 0.833068, 0.956682, 0.877551, 0.959550, 0.906639, 0.964361, 0.833068, 0.959550, 0.906639, 0.964361, 0.833068, 0.968618, 0.861458, 0.964361, 0.833068, 0.959550, 0.906639, 0.968618, 0.861458, 0.962821, 0.931720, 0.959550, 0.906639, 0.968618, 0.861458, 0.959550, 0.906639, 0.962821, 0.931720, 0.968618, 0.861458, 0.962821, 0.931720, 0.968618, 0.861458, 0.973708, 0.881736, 0.968618, 0.861458, 0.962821, 0.931720, 0.973708, 0.881736, 0.966494, 0.952796, 0.962821, 0.931720, 0.973708, 0.881736, 0.962821, 0.931720, 0.966494, 0.952796, 0.973708, 0.881736, 0.970569, 0.969866, 0.966494, 0.952796, 0.973708, 0.881736, 0.966494, 0.952796, 0.970569, 0.969866, 0.973708, 0.881736, 0.970569, 0.969866, 0.973708, 0.881736, 0.979631, 0.893903, 0.973708, 0.881736, 0.970569, 0.969866, 0.979631, 0.893903, 0.975076, 0.983050, 0.970569, 0.969866, 0.979631, 0.893903, 0.970569, 0.969866, 0.975076, 0.983050, 0.979631, 0.893903, 0.975076, 0.983050, 0.979631, 0.893903, 0.980042, 0.992467, 0.979631, 0.893903, 0.975076, 0.983050, 0.980042, 0.992467, 0.980042, 0.992467, 0.979631, 0.893903, 0.986388, 0.897959, 0.979631, 0.893903, 0.980042, 0.992467, 0.986388, 0.897959, 0.985469, 0.998117, 0.980042, 0.992467, 0.986388, 0.897959, 0.980042, 0.992467, 0.985469, 0.998117, 0.986388, 0.897959, 0.985469, 0.998117, 0.986388, 0.897959, 0.988369, 0.897650, 0.986388, 0.897959, 0.985469, 0.998117, 0.988369, 0.897650, 0.991355, 1, 0.985469, 0.998117, 0.988369, 0.897650, 0.985469, 0.998117, 0.991355, 1, 0.988369, 0.897650, 0.988369, 0.897650, 0.990263, 0.896724, 0.991355, 1, 0.990263, 0.896724, 0.988369, 0.897650, 0.991355, 1, 0.992252, 1, 0.991355, 1, 0.990263, 0.896724, 0.991355, 1, 0.992252, 1, 0.990263, 0.896724, 0.992252, 1, 0.990263, 0.896724, 0.992070, 0.895179, 0.990263, 0.896724, 0.992252, 1, 0.992070, 0.895179, 0.993194, 1, 0.992252, 1, 0.992070, 0.895179, 0.992252, 1, 0.993194, 1, 0.992070, 0.895179, 0.992070, 0.895179, 0.994183, 1, 0.993194, 1, 0.994183, 1, 0.992070, 0.895179, 0.993194, 1, 0.994183, 1, 0.992070, 0.895179, 0.993792, 0.893017, 0.992070, 0.895179, 0.994183, 1, 0.993792, 0.893017, 0.995218, 1, 0.994183, 1, 0.993792, 0.893017, 0.994183, 1, 0.995218, 1, 0.993792, 0.893017, 0.993792, 0.893017, 0.996304, 1, 0.995218, 1, 0.996304, 1, 0.993792, 0.893017, 0.995218, 1, 0.996304, 1, 0.993792, 0.893017, 0.995439, 0.890197, 0.993792, 0.893017, 0.996304, 1, 0.995439, 0.890197, 0.997448, 1, 0.996304, 1, 0.995439, 0.890197, 0.996304, 1, 0.997448, 1, 0.995439, 0.890197, 0.998649, 1, 0.997448, 1, 0.995439, 0.890197, 0.997448, 1, 0.998649, 1, 0.995439, 0.890197, 0.997022, 0.886679, 0.998649, 1, 0.995439, 0.890197, 0.998649, 1, 0.997022, 0.886679, 0.995439, 0.890197, 0.999908, 1, 0.998649, 1, 0.997022, 0.886679, 0.998649, 1, 0.999908, 1, 0.997022, 0.886679, 0.998543, 0.882464, 0.999908, 1, 0.997022, 0.886679, 0.999908, 1, 0.998543, 0.882464, 0.997022, 0.886679, 1, 0.877551, 0.999908, 1, 0.998543, 0.882464, 0.999908, 1, 1, 0.877551, 0.998543, 0.882464, 0.958602, 0.754106, 0.939667, 0.979592, 0.957162, 0.704072, 0.939667, 0.979592, 0.958602, 0.754106, 0.957162, 0.704072, 0.956682, 0.647003, 0.957162, 0.704072, 0.939667, 0.979592, 0.957162, 0.704072, 0.956682, 0.647003, 0.939667, 0.979592, 0.956682, 0.647003, 0.939667, 0.979592, 0.939667, 0.265306, 0.939667, 0.979592, 0.956682, 0.647003, 0.939667, 0.265306, 0.939667, 0.265306, 0.956682, 0.265306, 0.956682, 0.647003, 0.956682, 0.265306, 0.939667, 0.265306, 0.956682, 0.647003 };
    TextureCoordinate16.point = m_point;
    TextureCoordinate16.point_length = sizeof(m_point) / sizeof(float);
    }
    Text_69.color = NULL;
    Text_69.coord = &Coordinate31;
    Text_69.normal = &Normal36;
    Text_69.texCoord = &TextureCoordinate16;
    Text_69.ccw = false;
    {
    static int m_colorIndex[] = { 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1, 0, 0, 0, -1 };
    Text_69.colorIndex = m_colorIndex;
    Text_69.colorIndex_length = sizeof(m_colorIndex) / sizeof(int);
    }
    Text_69.colorPerVertex = false;
    Text_69.convex = false;
    {
    static int m_coordIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1 };
    Text_69.coordIndex = m_coordIndex;
    Text_69.coordIndex_length = sizeof(m_coordIndex) / sizeof(int);
    }
    Text_69.creaseAngle = 0;
    {
    static int m_normalIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1 };
    Text_69.normalIndex = m_normalIndex;
    Text_69.normalIndex_length = sizeof(m_normalIndex) / sizeof(int);
    }
    Text_69.normalPerVertex = true;
    Text_69.solid = false;
    {
    static int m_texCoordIndex[] = { 0, 1, 2, -1, 3, 4, 5, -1, 6, 7, 8, -1, 9, 10, 11, -1, 12, 13, 14, -1, 15, 16, 17, -1, 18, 19, 20, -1, 21, 22, 23, -1, 24, 25, 26, -1, 27, 28, 29, -1, 30, 31, 32, -1, 33, 34, 35, -1, 36, 37, 38, -1, 39, 40, 41, -1, 42, 43, 44, -1, 45, 46, 47, -1, 48, 49, 50, -1, 51, 52, 53, -1, 54, 55, 56, -1, 57, 58, 59, -1, 60, 61, 62, -1, 63, 64, 65, -1, 66, 67, 68, -1, 69, 70, 71, -1, 72, 73, 74, -1, 75, 76, 77, -1, 78, 79, 80, -1, 81, 82, 83, -1, 84, 85, 86, -1, 87, 88, 89, -1, 90, 91, 92, -1, 93, 94, 95, -1, 96, 97, 98, -1, 99, 100, 101, -1, 102, 103, 104, -1, 105, 106, 107, -1, 108, 109, 110, -1, 111, 112, 113, -1, 114, 115, 116, -1, 117, 118, 119, -1, 120, 121, 122, -1, 123, 124, 125, -1, 126, 127, 128, -1, 129, 130, 131, -1, 132, 133, 134, -1, 135, 136, 137, -1, 138, 139, 140, -1, 141, 142, 143, -1, 144, 145, 146, -1, 147, 148, 149, -1, 150, 151, 152, -1, 153, 154, 155, -1, 156, 157, 158, -1, 159, 160, 161, -1, 162, 163, 164, -1, 165, 166, 167, -1, 168, 169, 170, -1, 171, 172, 173, -1, 174, 175, 176, -1, 177, 178, 179, -1, 180, 181, 182, -1, 183, 184, 185, -1, 186, 187, 188, -1, 189, 190, 191, -1, 192, 193, 194, -1, 195, 196, 197, -1, 198, 199, 200, -1, 201, 202, 203, -1, 204, 205, 206, -1, 207, 208, 209, -1, 210, 211, 212, -1, 213, 214, 215, -1, 216, 217, 218, -1, 219, 220, 221, -1, 222, 223, 224, -1, 225, 226, 227, -1, 228, 229, 230, -1, 231, 232, 233, -1, 234, 235, 236, -1, 237, 238, 239, -1, 240, 241, 242, -1, 243, 244, 245, -1, 246, 247, 248, -1, 249, 250, 251, -1, 252, 253, 254, -1, 255, 256, 257, -1, 258, 259, 260, -1, 261, 262, 263, -1, 264, 265, 266, -1, 267, 268, 269, -1, 270, 271, 272, -1, 273, 274, 275, -1, 276, 277, 278, -1, 279, 280, 281, -1, 282, 283, 284, -1, 285, 286, 287, -1, 288, 289, 290, -1, 291, 292, 293, -1, 294, 295, 296, -1, 297, 298, 299, -1, 300, 301, 302, -1, 303, 304, 305, -1, 306, 307, 308, -1, 309, 310, 311, -1, 312, 313, 314, -1, 315, 316, 317, -1, 318, 319, 320, -1, 321, 322, 323, -1, 324, 325, 326, -1, 327, 328, 329, -1, 330, 331, 332, -1, 333, 334, 335, -1, 336, 337, 338, -1, 339, 340, 341, -1, 342, 343, 344, -1, 345, 346, 347, -1, 348, 349, 350, -1, 351, 352, 353, -1, 354, 355, 356, -1, 357, 358, 359, -1, 360, 361, 362, -1, 363, 364, 365, -1, 366, 367, 368, -1, 369, 370, 371, -1, 372, 373, 374, -1, 375, 376, 377, -1, 378, 379, 380, -1, 381, 382, 383, -1, 384, 385, 386, -1, 387, 388, 389, -1, 390, 391, 392, -1, 393, 394, 395, -1, 396, 397, 398, -1, 399, 400, 401, -1, 402, 403, 404, -1, 405, 406, 407, -1, 408, 409, 410, -1, 411, 412, 413, -1, 414, 415, 416, -1, 417, 418, 419, -1, 420, 421, 422, -1, 423, 424, 425, -1, 426, 427, 428, -1, 429, 430, 431, -1, 432, 433, 434, -1, 435, 436, 437, -1, 438, 439, 440, -1, 441, 442, 443, -1, 444, 445, 446, -1, 447, 448, 449, -1, 450, 451, 452, -1, 453, 454, 455, -1, 456, 457, 458, -1, 459, 460, 461, -1, 462, 463, 464, -1, 465, 466, 467, -1, 468, 469, 470, -1, 471, 472, 473, -1, 474, 475, 476, -1, 477, 478, 479, -1, 480, 481, 482, -1, 483, 484, 485, -1, 486, 487, 488, -1, 489, 490, 491, -1, 492, 493, 494, -1, 495, 496, 497, -1, 498, 499, 500, -1, 501, 502, 503, -1, 504, 505, 506, -1, 507, 508, 509, -1, 510, 511, 512, -1, 513, 514, 515, -1, 516, 517, 518, -1, 519, 520, 521, -1, 522, 523, 524, -1, 525, 526, 527, -1, 528, 529, 530, -1, 531, 532, 533, -1, 534, 535, 536, -1, 537, 538, 539, -1, 540, 541, 542, -1, 543, 544, 545, -1, 546, 547, 548, -1, 549, 550, 551, -1, 552, 553, 554, -1, 555, 556, 557, -1, 558, 559, 560, -1, 561, 562, 563, -1, 564, 565, 566, -1, 567, 568, 569, -1, 570, 571, 572, -1, 573, 574, 575, -1, 576, 577, 578, -1, 579, 580, 581, -1, 582, 583, 584, -1, 585, 586, 587, -1, 588, 589, 590, -1, 591, 592, 593, -1, 594, 595, 596, -1, 597, 598, 599, -1, 600, 601, 602, -1, 603, 604, 605, -1, 606, 607, 608, -1, 609, 610, 611, -1, 612, 613, 614, -1, 615, 616, 617, -1, 618, 619, 620, -1, 621, 622, 623, -1, 624, 625, 626, -1, 627, 628, 629, -1, 630, 631, 632, -1, 633, 634, 635, -1, 636, 637, 638, -1, 639, 640, 641, -1, 642, 643, 644, -1, 645, 646, 647, -1, 648, 649, 650, -1, 651, 652, 653, -1, 654, 655, 656, -1, 657, 658, 659, -1, 660, 661, 662, -1, 663, 664, 665, -1, 666, 667, 668, -1, 669, 670, 671, -1, 672, 673, 674, -1, 675, 676, 677, -1, 678, 679, 680, -1, 681, 682, 683, -1, 684, 685, 686, -1, 687, 688, 689, -1, 690, 691, 692, -1, 693, 694, 695, -1, 696, 697, 698, -1, 699, 700, 701, -1, 702, 703, 704, -1, 705, 706, 707, -1, 708, 709, 710, -1, 711, 712, 713, -1, 714, 715, 716, -1, 717, 718, 719, -1, 720, 721, 722, -1, 723, 724, 725, -1, 726, 727, 728, -1, 729, 730, 731, -1, 732, 733, 734, -1, 735, 736, 737, -1, 738, 739, 740, -1, 741, 742, 743, -1, 744, 745, 746, -1, 747, 748, 749, -1, 750, 751, 752, -1, 753, 754, 755, -1, 756, 757, 758, -1, 759, 760, 761, -1, 762, 763, 764, -1, 765, 766, 767, -1, 768, 769, 770, -1, 771, 772, 773, -1, 774, 775, 776, -1, 777, 778, 779, -1, 780, 781, 782, -1, 783, 784, 785, -1, 786, 787, 788, -1, 789, 790, 791, -1, 792, 793, 794, -1, 795, 796, 797, -1, 798, 799, 800, -1, 801, 802, 803, -1, 804, 805, 806, -1, 807, 808, 809, -1, 810, 811, 812, -1, 813, 814, 815, -1, 816, 817, 818, -1, 819, 820, 821, -1, 822, 823, 824, -1, 825, 826, 827, -1, 828, 829, 830, -1, 831, 832, 833, -1, 834, 835, 836, -1, 837, 838, 839, -1, 840, 841, 842, -1, 843, 844, 845, -1, 846, 847, 848, -1, 849, 850, 851, -1, 852, 853, 854, -1, 855, 856, 857, -1, 858, 859, 860, -1, 861, 862, 863, -1, 864, 865, 866, -1, 867, 868, 869, -1, 870, 871, 872, -1, 873, 874, 875, -1, 876, 877, 878, -1, 879, 880, 881, -1, 882, 883, 884, -1, 885, 886, 887, -1, 888, 889, 890, -1, 891, 892, 893, -1, 894, 895, 896, -1, 897, 898, 899, -1, 900, 901, 902, -1, 903, 904, 905, -1, 906, 907, 908, -1, 909, 910, 911, -1, 912, 913, 914, -1, 915, 916, 917, -1, 918, 919, 920, -1, 921, 922, 923, -1, 924, 925, 926, -1, 927, 928, 929, -1, 930, 931, 932, -1, 933, 934, 935, -1, 936, 937, 938, -1, 939, 940, 941, -1, 942, 943, 944, -1, 945, 946, 947, -1, 948, 949, 950, -1, 951, 952, 953, -1, 954, 955, 956, -1, 957, 958, 959, -1, 960, 961, 962, -1, 963, 964, 965, -1, 966, 967, 968, -1, 969, 970, 971, -1, 972, 973, 974, -1, 975, 976, 977, -1, 978, 979, 980, -1, 981, 982, 983, -1, 984, 985, 986, -1, 987, 988, 989, -1, 990, 991, 992, -1, 993, 994, 995, -1, 996, 997, 998, -1, 999, 1000, 1001, -1, 1002, 1003, 1004, -1, 1005, 1006, 1007, -1, 1008, 1009, 1010, -1, 1011, 1012, 1013, -1, 1014, 1015, 1016, -1, 1017, 1018, 1019, -1, 1020, 1021, 1022, -1, 1023, 1024, 1025, -1, 1026, 1027, 1028, -1, 1029, 1030, 1031, -1, 1032, 1033, 1034, -1, 1035, 1036, 1037, -1, 1038, 1039, 1040, -1, 1041, 1042, 1043, -1, 1044, 1045, 1046, -1, 1047, 1048, 1049, -1, 1050, 1051, 1052, -1, 1053, 1054, 1055, -1, 1056, 1057, 1058, -1, 1059, 1060, 1061, -1, 1062, 1063, 1064, -1, 1065, 1066, 1067, -1, 1068, 1069, 1070, -1, 1071, 1072, 1073, -1, 1074, 1075, 1076, -1, 1077, 1078, 1079, -1, 1080, 1081, 1082, -1, 1083, 1084, 1085, -1, 1086, 1087, 1088, -1, 1089, 1090, 1091, -1, 1092, 1093, 1094, -1, 1095, 1096, 1097, -1, 1098, 1099, 1100, -1, 1101, 1102, 1103, -1, 1104, 1105, 1106, -1, 1107, 1108, 1109, -1, 1110, 1111, 1112, -1, 1113, 1114, 1115, -1, 1116, 1117, 1118, -1, 1119, 1120, 1121, -1, 1122, 1123, 1124, -1, 1125, 1126, 1127, -1, 1128, 1129, 1130, -1, 1131, 1132, 1133, -1, 1134, 1135, 1136, -1, 1137, 1138, 1139, -1, 1140, 1141, 1142, -1, 1143, 1144, 1145, -1, 1146, 1147, 1148, -1, 1149, 1150, 1151, -1, 1152, 1153, 1154, -1, 1155, 1156, 1157, -1, 1158, 1159, 1160, -1, 1161, 1162, 1163, -1, 1164, 1165, 1166, -1, 1167, 1168, 1169, -1, 1170, 1171, 1172, -1, 1173, 1174, 1175, -1, 1176, 1177, 1178, -1, 1179, 1180, 1181, -1, 1182, 1183, 1184, -1, 1185, 1186, 1187, -1, 1188, 1189, 1190, -1, 1191, 1192, 1193, -1, 1194, 1195, 1196, -1, 1197, 1198, 1199, -1, 1200, 1201, 1202, -1, 1203, 1204, 1205, -1, 1206, 1207, 1208, -1, 1209, 1210, 1211, -1, 1212, 1213, 1214, -1, 1215, 1216, 1217, -1, 1218, 1219, 1220, -1, 1221, 1222, 1223, -1, 1224, 1225, 1226, -1, 1227, 1228, 1229, -1, 1230, 1231, 1232, -1, 1233, 1234, 1235, -1, 1236, 1237, 1238, -1, 1239, 1240, 1241, -1, 1242, 1243, 1244, -1, 1245, 1246, 1247, -1, 1248, 1249, 1250, -1, 1251, 1252, 1253, -1, 1254, 1255, 1256, -1, 1257, 1258, 1259, -1, 1260, 1261, 1262, -1, 1263, 1264, 1265, -1, 1266, 1267, 1268, -1, 1269, 1270, 1271, -1, 1272, 1273, 1274, -1, 1275, 1276, 1277, -1, 1278, 1279, 1280, -1, 1281, 1282, 1283, -1, 1284, 1285, 1286, -1, 1287, 1288, 1289, -1, 1290, 1291, 1292, -1, 1293, 1294, 1295, -1, 1296, 1297, 1298, -1, 1299, 1300, 1301, -1, 1302, 1303, 1304, -1, 1305, 1306, 1307, -1, 1308, 1309, 1310, -1, 1311, 1312, 1313, -1, 1314, 1315, 1316, -1, 1317, 1318, 1319, -1, 1320, 1321, 1322, -1, 1323, 1324, 1325, -1, 1326, 1327, 1328, -1, 1329, 1330, 1331, -1, 1332, 1333, 1334, -1, 1335, 1336, 1337, -1, 1338, 1339, 1340, -1, 1341, 1342, 1343, -1, 1344, 1345, 1346, -1, 1347, 1348, 1349, -1, 1350, 1351, 1352, -1, 1353, 1354, 1355, -1, 1356, 1357, 1358, -1, 1359, 1360, 1361, -1, 1362, 1363, 1364, -1, 1365, 1366, 1367, -1, 1368, 1369, 1370, -1, 1371, 1372, 1373, -1, 1374, 1375, 1376, -1, 1377, 1378, 1379, -1, 1380, 1381, 1382, -1, 1383, 1384, 1385, -1, 1386, 1387, 1388, -1, 1389, 1390, 1391, -1, 1392, 1393, 1394, -1, 1395, 1396, 1397, -1, 1398, 1399, 1400, -1, 1401, 1402, 1403, -1, 1404, 1405, 1406, -1, 1407, 1408, 1409, -1, 1410, 1411, 1412, -1, 1413, 1414, 1415, -1, 1416, 1417, 1418, -1, 1419, 1420, 1421, -1, 1422, 1423, 1424, -1, 1425, 1426, 1427, -1, 1428, 1429, 1430, -1, 1431, 1432, 1433, -1, 1434, 1435, 1436, -1, 1437, 1438, 1439, -1, 1440, 1441, 1442, -1, 1443, 1444, 1445, -1, 1446, 1447, 1448, -1, 1449, 1450, 1451, -1, 1452, 1453, 1454, -1, 1455, 1456, 1457, -1, 1458, 1459, 1460, -1, 1461, 1462, 1463, -1, 1464, 1465, 1466, -1, 1467, 1468, 1469, -1, 1470, 1471, 1472, -1, 1473, 1474, 1475, -1, 1476, 1477, 1478, -1, 1479, 1480, 1481, -1, 1482, 1483, 1484, -1, 1485, 1486, 1487, -1, 1488, 1489, 1490, -1, 1491, 1492, 1493, -1, 1494, 1495, 1496, -1, 1497, 1498, 1499, -1, 1500, 1501, 1502, -1, 1503, 1504, 1505, -1, 1506, 1507, 1508, -1, 1509, 1510, 1511, -1, 1512, 1513, 1514, -1, 1515, 1516, 1517, -1, 1518, 1519, 1520, -1, 1521, 1522, 1523, -1, 1524, 1525, 1526, -1, 1527, 1528, 1529, -1, 1530, 1531, 1532, -1, 1533, 1534, 1535, -1, 1536, 1537, 1538, -1, 1539, 1540, 1541, -1, 1542, 1543, 1544, -1, 1545, 1546, 1547, -1, 1548, 1549, 1550, -1, 1551, 1552, 1553, -1, 1554, 1555, 1556, -1, 1557, 1558, 1559, -1, 1560, 1561, 1562, -1, 1563, 1564, 1565, -1, 1566, 1567, 1568, -1, 1569, 1570, 1571, -1, 1572, 1573, 1574, -1, 1575, 1576, 1577, -1, 1578, 1579, 1580, -1, 1581, 1582, 1583, -1, 1584, 1585, 1586, -1, 1587, 1588, 1589, -1, 1590, 1591, 1592, -1, 1593, 1594, 1595, -1, 1596, 1597, 1598, -1, 1599, 1600, 1601, -1, 1602, 1603, 1604, -1, 1605, 1606, 1607, -1, 1608, 1609, 1610, -1, 1611, 1612, 1613, -1, 1614, 1615, 1616, -1, 1617, 1618, 1619, -1, 1620, 1621, 1622, -1, 1623, 1624, 1625, -1, 1626, 1627, 1628, -1, 1629, 1630, 1631, -1, 1632, 1633, 1634, -1, 1635, 1636, 1637, -1, 1638, 1639, 1640, -1, 1641, 1642, 1643, -1, 1644, 1645, 1646, -1, 1647, 1648, 1649, -1, 1650, 1651, 1652, -1, 1653, 1654, 1655, -1, 1656, 1657, 1658, -1, 1659, 1660, 1661, -1, 1662, 1663, 1664, -1, 1665, 1666, 1667, -1, 1668, 1669, 1670, -1, 1671, 1672, 1673, -1, 1674, 1675, 1676, -1, 1677, 1678, 1679, -1, 1680, 1681, 1682, -1, 1683, 1684, 1685, -1, 1686, 1687, 1688, -1, 1689, 1690, 1691, -1, 1692, 1693, 1694, -1, 1695, 1696, 1697, -1, 1698, 1699, 1700, -1, 1701, 1702, 1703, -1, 1704, 1705, 1706, -1, 1707, 1708, 1709, -1, 1710, 1711, 1712, -1, 1713, 1714, 1715, -1, 1716, 1717, 1718, -1, 1719, 1720, 1721, -1, 1722, 1723, 1724, -1, 1725, 1726, 1727, -1, 1728, 1729, 1730, -1, 1731, 1732, 1733, -1, 1734, 1735, 1736, -1, 1737, 1738, 1739, -1, 1740, 1741, 1742, -1, 1743, 1744, 1745, -1, 1746, 1747, 1748, -1, 1749, 1750, 1751, -1, 1752, 1753, 1754, -1, 1755, 1756, 1757, -1, 1758, 1759, 1760, -1, 1761, 1762, 1763, -1, 1764, 1765, 1766, -1, 1767, 1768, 1769, -1, 1770, 1771, 1772, -1, 1773, 1774, 1775, -1, 1776, 1777, 1778, -1, 1779, 1780, 1781, -1, 1782, 1783, 1784, -1, 1785, 1786, 1787, -1, 1788, 1789, 1790, -1, 1791, 1792, 1793, -1, 1794, 1795, 1796, -1, 1797, 1798, 1799, -1, 1800, 1801, 1802, -1, 1803, 1804, 1805, -1, 1806, 1807, 1808, -1, 1809, 1810, 1811, -1, 1812, 1813, 1814, -1, 1815, 1816, 1817, -1, 1818, 1819, 1820, -1, 1821, 1822, 1823, -1, 1824, 1825, 1826, -1, 1827, 1828, 1829, -1, 1830, 1831, 1832, -1, 1833, 1834, 1835, -1, 1836, 1837, 1838, -1, 1839, 1840, 1841, -1, 1842, 1843, 1844, -1, 1845, 1846, 1847, -1, 1848, 1849, 1850, -1, 1851, 1852, 1853, -1, 1854, 1855, 1856, -1, 1857, 1858, 1859, -1, 1860, 1861, 1862, -1, 1863, 1864, 1865, -1, 1866, 1867, 1868, -1, 1869, 1870, 1871, -1, 1872, 1873, 1874, -1, 1875, 1876, 1877, -1, 1878, 1879, 1880, -1, 1881, 1882, 1883, -1, 1884, 1885, 1886, -1, 1887, 1888, 1889, -1, 1890, 1891, 1892, -1, 1893, 1894, 1895, -1, 1896, 1897, 1898, -1, 1899, 1900, 1901, -1, 1902, 1903, 1904, -1, 1905, 1906, 1907, -1, 1908, 1909, 1910, -1, 1911, 1912, 1913, -1, 1914, 1915, 1916, -1, 1917, 1918, 1919, -1, 1920, 1921, 1922, -1, 1923, 1924, 1925, -1, 1926, 1927, 1928, -1, 1929, 1930, 1931, -1, 1932, 1933, 1934, -1, 1935, 1936, 1937, -1, 1938, 1939, 1940, -1, 1941, 1942, 1943, -1, 1944, 1945, 1946, -1, 1947, 1948, 1949, -1, 1950, 1951, 1952, -1, 1953, 1954, 1955, -1, 1956, 1957, 1958, -1, 1959, 1960, 1961, -1, 1962, 1963, 1964, -1, 1965, 1966, 1967, -1, 1968, 1969, 1970, -1, 1971, 1972, 1973, -1, 1974, 1975, 1976, -1, 1977, 1978, 1979, -1, 1980, 1981, 1982, -1, 1983, 1984, 1985, -1, 1986, 1987, 1988, -1, 1989, 1990, 1991, -1, 1992, 1993, 1994, -1, 1995, 1996, 1997, -1, 1998, 1999, 2000, -1, 2001, 2002, 2003, -1, 2004, 2005, 2006, -1, 2007, 2008, 2009, -1, 2010, 2011, 2012, -1, 2013, 2014, 2015, -1, 2016, 2017, 2018, -1, 2019, 2020, 2021, -1, 2022, 2023, 2024, -1, 2025, 2026, 2027, -1, 2028, 2029, 2030, -1, 2031, 2032, 2033, -1, 2034, 2035, 2036, -1, 2037, 2038, 2039, -1, 2040, 2041, 2042, -1, 2043, 2044, 2045, -1, 2046, 2047, 2048, -1, 2049, 2050, 2051, -1, 2052, 2053, 2054, -1, 2055, 2056, 2057, -1, 2058, 2059, 2060, -1, 2061, 2062, 2063, -1, 2064, 2065, 2066, -1, 2067, 2068, 2069, -1, 2070, 2071, 2072, -1, 2073, 2074, 2075, -1, 2076, 2077, 2078, -1, 2079, 2080, 2081, -1, 2082, 2083, 2084, -1, 2085, 2086, 2087, -1, 2088, 2089, 2090, -1, 2091, 2092, 2093, -1, 2094, 2095, 2096, -1, 2097, 2098, 2099, -1, 2100, 2101, 2102, -1, 2103, 2104, 2105, -1, 2106, 2107, 2108, -1, 2109, 2110, 2111, -1, 2112, 2113, 2114, -1, 2115, 2116, 2117, -1, 2118, 2119, 2120, -1, 2121, 2122, 2123, -1, 2124, 2125, 2126, -1, 2127, 2128, 2129, -1, 2130, 2131, 2132, -1, 2133, 2134, 2135, -1, 2136, 2137, 2138, -1, 2139, 2140, 2141, -1, 2142, 2143, 2144, -1, 2145, 2146, 2147, -1, 2148, 2149, 2150, -1, 2151, 2152, 2153, -1, 2154, 2155, 2156, -1, 2157, 2158, 2159, -1, 2160, 2161, 2162, -1, 2163, 2164, 2165, -1, 2166, 2167, 2168, -1, 2169, 2170, 2171, -1, 2172, 2173, 2174, -1, 2175, 2176, 2177, -1, 2178, 2179, 2180, -1, 2181, 2182, 2183, -1, 2184, 2185, 2186, -1, 2187, 2188, 2189, -1, 2190, 2191, 2192, -1, 2193, 2194, 2195, -1, 2196, 2197, 2198, -1, 2199, 2200, 2201, -1, 2202, 2203, 2204, -1, 2205, 2206, 2207, -1, 2208, 2209, 2210, -1, 2211, 2212, 2213, -1, 2214, 2215, 2216, -1, 2217, 2218, 2219, -1, 2220, 2221, 2222, -1, 2223, 2224, 2225, -1, 2226, 2227, 2228, -1, 2229, 2230, 2231, -1, 2232, 2233, 2234, -1, 2235, 2236, 2237, -1, 2238, 2239, 2240, -1, 2241, 2242, 2243, -1, 2244, 2245, 2246, -1, 2247, 2248, 2249, -1, 2250, 2251, 2252, -1, 2253, 2254, 2255, -1, 2256, 2257, 2258, -1, 2259, 2260, 2261, -1, 2262, 2263, 2264, -1, 2265, 2266, 2267, -1, 2268, 2269, 2270, -1, 2271, 2272, 2273, -1, 2274, 2275, 2276, -1, 2277, 2278, 2279, -1, 2280, 2281, 2282, -1, 2283, 2284, 2285, -1, 2286, 2287, 2288, -1, 2289, 2290, 2291, -1, 2292, 2293, 2294, -1, 2295, 2296, 2297, -1, 2298, 2299, 2300, -1, 2301, 2302, 2303, -1, 2304, 2305, 2306, -1, 2307, 2308, 2309, -1, 2310, 2311, 2312, -1, 2313, 2314, 2315, -1, 2316, 2317, 2318, -1, 2319, 2320, 2321, -1, 2322, 2323, 2324, -1, 2325, 2326, 2327, -1, 2328, 2329, 2330, -1, 2331, 2332, 2333, -1, 2334, 2335, 2336, -1, 2337, 2338, 2339, -1, 2340, 2341, 2342, -1, 2343, 2344, 2345, -1, 2346, 2347, 2348, -1, 2349, 2350, 2351, -1, 2352, 2353, 2354, -1, 2355, 2356, 2357, -1, 2358, 2359, 2360, -1, 2361, 2362, 2363, -1, 2364, 2365, 2366, -1, 2367, 2368, 2369, -1, 2370, 2371, 2372, -1, 2373, 2374, 2375, -1, 2376, 2377, 2378, -1, 2379, 2380, 2381, -1, 2382, 2383, 2384, -1, 2385, 2386, 2387, -1, 2388, 2389, 2390, -1, 2391, 2392, 2393, -1, 2394, 2395, 2396, -1, 2397, 2398, 2399, -1, 2400, 2401, 2402, -1, 2403, 2404, 2405, -1, 2406, 2407, 2408, -1, 2409, 2410, 2411, -1, 2412, 2413, 2414, -1, 2415, 2416, 2417, -1, 2418, 2419, 2420, -1, 2421, 2422, 2423, -1, 2424, 2425, 2426, -1, 2427, 2428, 2429, -1, 2430, 2431, 2432, -1, 2433, 2434, 2435, -1, 2436, 2437, 2438, -1, 2439, 2440, 2441, -1, 2442, 2443, 2444, -1, 2445, 2446, 2447, -1, 2448, 2449, 2450, -1, 2451, 2452, 2453, -1, 2454, 2455, 2456, -1, 2457, 2458, 2459, -1, 2460, 2461, 2462, -1, 2463, 2464, 2465, -1, 2466, 2467, 2468, -1, 2469, 2470, 2471, -1, 2472, 2473, 2474, -1, 2475, 2476, 2477, -1, 2478, 2479, 2480, -1, 2481, 2482, 2483, -1, 2484, 2485, 2486, -1, 2487, 2488, 2489, -1, 2490, 2491, 2492, -1, 2493, 2494, 2495, -1, 2496, 2497, 2498, -1, 2499, 2500, 2501, -1, 2502, 2503, 2504, -1, 2505, 2506, 2507, -1, 2508, 2509, 2510, -1, 2511, 2512, 2513, -1, 2514, 2515, 2516, -1, 2517, 2518, 2519, -1, 2520, 2521, 2522, -1, 2523, 2524, 2525, -1, 2526, 2527, 2528, -1, 2529, 2530, 2531, -1, 2532, 2533, 2534, -1, 2535, 2536, 2537, -1, 2538, 2539, 2540, -1, 2541, 2542, 2543, -1, 2544, 2545, 2546, -1, 2547, 2548, 2549, -1, 2550, 2551, 2552, -1, 2553, 2554, 2555, -1, 2556, 2557, 2558, -1, 2559, 2560, 2561, -1, 2562, 2563, 2564, -1, 2565, 2566, 2567, -1, 2568, 2569, 2570, -1, 2571, 2572, 2573, -1, 2574, 2575, 2576, -1, 2577, 2578, 2579, -1, 2580, 2581, 2582, -1, 2583, 2584, 2585, -1, 2586, 2587, 2588, -1, 2589, 2590, 2591, -1, 2592, 2593, 2594, -1, 2595, 2596, 2597, -1, 2598, 2599, 2600, -1, 2601, 2602, 2603, -1, 2604, 2605, 2606, -1, 2607, 2608, 2609, -1, 2610, 2611, 2612, -1, 2613, 2614, 2615, -1, 2616, 2617, 2618, -1, 2619, 2620, 2621, -1, 2622, 2623, 2624, -1, 2625, 2626, 2627, -1, 2628, 2629, 2630, -1, 2631, 2632, 2633, -1, 2634, 2635, 2636, -1, 2637, 2638, 2639, -1, 2640, 2641, 2642, -1, 2643, 2644, 2645, -1, 2646, 2647, 2648, -1, 2649, 2650, 2651, -1, 2652, 2653, 2654, -1, 2655, 2656, 2657, -1, 2658, 2659, 2660, -1, 2661, 2662, 2663, -1, 2664, 2665, 2666, -1, 2667, 2668, 2669, -1, 2670, 2671, 2672, -1, 2673, 2674, 2675, -1, 2676, 2677, 2678, -1, 2679, 2680, 2681, -1, 2682, 2683, 2684, -1, 2685, 2686, 2687, -1, 2688, 2689, 2690, -1, 2691, 2692, 2693, -1, 2694, 2695, 2696, -1, 2697, 2698, 2699, -1, 2700, 2701, 2702, -1, 2703, 2704, 2705, -1, 2706, 2707, 2708, -1, 2709, 2710, 2711, -1, 2712, 2713, 2714, -1, 2715, 2716, 2717, -1, 2718, 2719, 2720, -1, 2721, 2722, 2723, -1, 2724, 2725, 2726, -1, 2727, 2728, 2729, -1, 2730, 2731, 2732, -1, 2733, 2734, 2735, -1, 2736, 2737, 2738, -1, 2739, 2740, 2741, -1, 2742, 2743, 2744, -1, 2745, 2746, 2747, -1, 2748, 2749, 2750, -1, 2751, 2752, 2753, -1, 2754, 2755, 2756, -1, 2757, 2758, 2759, -1, 2760, 2761, 2762, -1, 2763, 2764, 2765, -1, 2766, 2767, 2768, -1, 2769, 2770, 2771, -1, 2772, 2773, 2774, -1, 2775, 2776, 2777, -1, 2778, 2779, 2780, -1, 2781, 2782, 2783, -1, 2784, 2785, 2786, -1, 2787, 2788, 2789, -1, 2790, 2791, 2792, -1, 2793, 2794, 2795, -1, 2796, 2797, 2798, -1, 2799, 2800, 2801, -1, 2802, 2803, 2804, -1, 2805, 2806, 2807, -1, 2808, 2809, 2810, -1, 2811, 2812, 2813, -1, 2814, 2815, 2816, -1, 2817, 2818, 2819, -1, 2820, 2821, 2822, -1, 2823, 2824, 2825, -1, 2826, 2827, 2828, -1, 2829, 2830, 2831, -1, 2832, 2833, 2834, -1, 2835, 2836, 2837, -1, 2838, 2839, 2840, -1, 2841, 2842, 2843, -1, 2844, 2845, 2846, -1, 2847, 2848, 2849, -1, 2850, 2851, 2852, -1, 2853, 2854, 2855, -1, 2856, 2857, 2858, -1, 2859, 2860, 2861, -1, 2862, 2863, 2864, -1, 2865, 2866, 2867, -1, 2868, 2869, 2870, -1, 2871, 2872, 2873, -1, 2874, 2875, 2876, -1, 2877, 2878, 2879, -1, 2880, 2881, 2882, -1, 2883, 2884, 2885, -1, 2886, 2887, 2888, -1, 2889, 2890, 2891, -1, 2892, 2893, 2894, -1, 2895, 2896, 2897, -1, 2898, 2899, 2900, -1, 2901, 2902, 2903, -1, 2904, 2905, 2906, -1, 2907, 2908, 2909, -1, 2910, 2911, 2912, -1, 2913, 2914, 2915, -1, 2916, 2917, 2918, -1, 2919, 2920, 2921, -1, 2922, 2923, 2924, -1, 2925, 2926, 2927, -1, 2928, 2929, 2930, -1, 2931, 2932, 2933, -1, 2934, 2935, 2936, -1, 2937, 2938, 2939, -1, 2940, 2941, 2942, -1, 2943, 2944, 2945, -1, 2946, 2947, 2948, -1, 2949, 2950, 2951, -1, 2952, 2953, 2954, -1, 2955, 2956, 2957, -1, 2958, 2959, 2960, -1, 2961, 2962, 2963, -1, 2964, 2965, 2966, -1, 2967, 2968, 2969, -1, 2970, 2971, 2972, -1, 2973, 2974, 2975, -1, 2976, 2977, 2978, -1, 2979, 2980, 2981, -1, 2982, 2983, 2984, -1, 2985, 2986, 2987, -1, 2988, 2989, 2990, -1, 2991, 2992, 2993, -1, 2994, 2995, 2996, -1, 2997, 2998, 2999, -1, 3000, 3001, 3002, -1, 3003, 3004, 3005, -1, 3006, 3007, 3008, -1, 3009, 3010, 3011, -1, 3012, 3013, 3014, -1, 3015, 3016, 3017, -1, 3018, 3019, 3020, -1, 3021, 3022, 3023, -1, 3024, 3025, 3026, -1, 3027, 3028, 3029, -1, 3030, 3031, 3032, -1, 3033, 3034, 3035, -1, 3036, 3037, 3038, -1, 3039, 3040, 3041, -1, 3042, 3043, 3044, -1, 3045, 3046, 3047, -1, 3048, 3049, 3050, -1, 3051, 3052, 3053, -1, 3054, 3055, 3056, -1, 3057, 3058, 3059, -1, 3060, 3061, 3062, -1, 3063, 3064, 3065, -1, 3066, 3067, 3068, -1, 3069, 3070, 3071, -1, 3072, 3073, 3074, -1, 3075, 3076, 3077, -1, 3078, 3079, 3080, -1, 3081, 3082, 3083, -1, 3084, 3085, 3086, -1, 3087, 3088, 3089, -1, 3090, 3091, 3092, -1, 3093, 3094, 3095, -1, 3096, 3097, 3098, -1, 3099, 3100, 3101, -1, 3102, 3103, 3104, -1, 3105, 3106, 3107, -1, 3108, 3109, 3110, -1, 3111, 3112, 3113, -1, 3114, 3115, 3116, -1, 3117, 3118, 3119, -1, 3120, 3121, 3122, -1, 3123, 3124, 3125, -1, 3126, 3127, 3128, -1, 3129, 3130, 3131, -1, 3132, 3133, 3134, -1, 3135, 3136, 3137, -1, 3138, 3139, 3140, -1, 3141, 3142, 3143, -1, 3144, 3145, 3146, -1, 3147, 3148, 3149, -1, 3150, 3151, 3152, -1, 3153, 3154, 3155, -1, 3156, 3157, 3158, -1, 3159, 3160, 3161, -1, 3162, 3163, 3164, -1, 3165, 3166, 3167, -1, 3168, 3169, 3170, -1, 3171, 3172, 3173, -1, 3174, 3175, 3176, -1, 3177, 3178, 3179, -1, 3180, 3181, 3182, -1, 3183, 3184, 3185, -1, 3186, 3187, 3188, -1, 3189, 3190, 3191, -1, 3192, 3193, 3194, -1, 3195, 3196, 3197, -1, 3198, 3199, 3200, -1, 3201, 3202, 3203, -1, 3204, 3205, 3206, -1, 3207, 3208, 3209, -1, 3210, 3211, 3212, -1, 3213, 3214, 3215, -1, 3216, 3217, 3218, -1, 3219, 3220, 3221, -1, 3222, 3223, 3224, -1, 3225, 3226, 3227, -1, 3228, 3229, 3230, -1, 3231, 3232, 3233, -1, 3234, 3235, 3236, -1, 3237, 3238, 3239, -1, 3240, 3241, 3242, -1, 3243, 3244, 3245, -1, 3246, 3247, 3248, -1, 3249, 3250, 3251, -1, 3252, 3253, 3254, -1, 3255, 3256, 3257, -1, 3258, 3259, 3260, -1, 3261, 3262, 3263, -1, 3264, 3265, 3266, -1, 3267, 3268, 3269, -1, 3270, 3271, 3272, -1, 3273, 3274, 3275, -1, 3276, 3277, 3278, -1, 3279, 3280, 3281, -1, 3282, 3283, 3284, -1, 3285, 3286, 3287, -1, 3288, 3289, 3290, -1, 3291, 3292, 3293, -1, 3294, 3295, 3296, -1, 3297, 3298, 3299, -1, 3300, 3301, 3302, -1, 3303, 3304, 3305, -1, 3306, 3307, 3308, -1, 3309, 3310, 3311, -1, 3312, 3313, 3314, -1, 3315, 3316, 3317, -1, 3318, 3319, 3320, -1, 3321, 3322, 3323, -1, 3324, 3325, 3326, -1, 3327, 3328, 3329, -1, 3330, 3331, 3332, -1, 3333, 3334, 3335, -1, 3336, 3337, 3338, -1, 3339, 3340, 3341, -1, 3342, 3343, 3344, -1, 3345, 3346, 3347, -1, 3348, 3349, 3350, -1, 3351, 3352, 3353, -1, 3354, 3355, 3356, -1, 3357, 3358, 3359, -1, 3360, 3361, 3362, -1, 3363, 3364, 3365, -1, 3366, 3367, 3368, -1, 3369, 3370, 3371, -1, 3372, 3373, 3374, -1, 3375, 3376, 3377, -1, 3378, 3379, 3380, -1, 3381, 3382, 3383, -1, 3384, 3385, 3386, -1, 3387, 3388, 3389, -1, 3390, 3391, 3392, -1, 3393, 3394, 3395, -1, 3396, 3397, 3398, -1, 3399, 3400, 3401, -1, 3402, 3403, 3404, -1, 3405, 3406, 3407, -1, 3408, 3409, 3410, -1, 3411, 3412, 3413, -1, 3414, 3415, 3416, -1, 3417, 3418, 3419, -1, 3420, 3421, 3422, -1, 3423, 3424, 3425, -1, 3426, 3427, 3428, -1, 3429, 3430, 3431, -1, 3432, 3433, 3434, -1, 3435, 3436, 3437, -1, 3438, 3439, 3440, -1, 3441, 3442, 3443, -1, 3444, 3445, 3446, -1, 3447, 3448, 3449, -1, 3450, 3451, 3452, -1, 3453, 3454, 3455, -1, 3456, 3457, 3458, -1, 3459, 3460, 3461, -1, 3462, 3463, 3464, -1, 3465, 3466, 3467, -1, 3468, 3469, 3470, -1, 3471, 3472, 3473, -1, 3474, 3475, 3476, -1, 3477, 3478, 3479, -1, 3480, 3481, 3482, -1, 3483, 3484, 3485, -1, 3486, 3487, 3488, -1, 3489, 3490, 3491, -1, 3492, 3493, 3494, -1, 3495, 3496, 3497, -1, 3498, 3499, 3500, -1, 3501, 3502, 3503, -1, 3504, 3505, 3506, -1, 3507, 3508, 3509, -1, 3510, 3511, 3512, -1, 3513, 3514, 3515, -1, 3516, 3517, 3518, -1, 3519, 3520, 3521, -1, 3522, 3523, 3524, -1, 3525, 3526, 3527, -1, 3528, 3529, 3530, -1, 3531, 3532, 3533, -1, 3534, 3535, 3536, -1, 3537, 3538, 3539, -1, 3540, 3541, 3542, -1, 3543, 3544, 3545, -1, 3546, 3547, 3548, -1, 3549, 3550, 3551, -1, 3552, 3553, 3554, -1, 3555, 3556, 3557, -1, 3558, 3559, 3560, -1, 3561, 3562, 3563, -1, 3564, 3565, 3566, -1, 3567, 3568, 3569, -1, 3570, 3571, 3572, -1, 3573, 3574, 3575, -1, 3576, 3577, 3578, -1, 3579, 3580, 3581, -1, 3582, 3583, 3584, -1, 3585, 3586, 3587, -1, 3588, 3589, 3590, -1, 3591, 3592, 3593, -1, 3594, 3595, 3596, -1, 3597, 3598, 3599, -1, 3600, 3601, 3602, -1, 3603, 3604, 3605, -1, 3606, 3607, 3608, -1, 3609, 3610, 3611, -1, 3612, 3613, 3614, -1, 3615, 3616, 3617, -1, 3618, 3619, 3620, -1, 3621, 3622, 3623, -1, 3624, 3625, 3626, -1, 3627, 3628, 3629, -1, 3630, 3631, 3632, -1, 3633, 3634, 3635, -1, 3636, 3637, 3638, -1, 3639, 3640, 3641, -1, 3642, 3643, 3644, -1, 3645, 3646, 3647, -1, 3648, 3649, 3650, -1, 3651, 3652, 3653, -1, 3654, 3655, 3656, -1, 3657, 3658, 3659, -1, 3660, 3661, 3662, -1, 3663, 3664, 3665, -1, 3666, 3667, 3668, -1, 3669, 3670, 3671, -1, 3672, 3673, 3674, -1, 3675, 3676, 3677, -1, 3678, 3679, 3680, -1, 3681, 3682, 3683, -1, 3684, 3685, 3686, -1, 3687, 3688, 3689, -1, 3690, 3691, 3692, -1, 3693, 3694, 3695, -1, 3696, 3697, 3698, -1, 3699, 3700, 3701, -1, 3702, 3703, 3704, -1, 3705, 3706, 3707, -1, 3708, 3709, 3710, -1, 3711, 3712, 3713, -1, 3714, 3715, 3716, -1, 3717, 3718, 3719, -1, 3720, 3721, 3722, -1, 3723, 3724, 3725, -1, 3726, 3727, 3728, -1, 3729, 3730, 3731, -1, 3732, 3733, 3734, -1, 3735, 3736, 3737, -1, 3738, 3739, 3740, -1, 3741, 3742, 3743, -1, 3744, 3745, 3746, -1, 3747, 3748, 3749, -1, 3750, 3751, 3752, -1, 3753, 3754, 3755, -1, 3756, 3757, 3758, -1, 3759, 3760, 3761, -1, 3762, 3763, 3764, -1, 3765, 3766, 3767, -1, 3768, 3769, 3770, -1, 3771, 3772, 3773, -1, 3774, 3775, 3776, -1, 3777, 3778, 3779, -1, 3780, 3781, 3782, -1, 3783, 3784, 3785, -1, 3786, 3787, 3788, -1, 3789, 3790, 3791, -1, 3792, 3793, 3794, -1, 3795, 3796, 3797, -1, 3798, 3799, 3800, -1, 3801, 3802, 3803, -1, 3804, 3805, 3806, -1, 3807, 3808, 3809, -1, 3810, 3811, 3812, -1, 3813, 3814, 3815, -1, 3816, 3817, 3818, -1, 3819, 3820, 3821, -1, 3822, 3823, 3824, -1, 3825, 3826, 3827, -1, 3828, 3829, 3830, -1, 3831, 3832, 3833, -1, 3834, 3835, 3836, -1, 3837, 3838, 3839, -1, 3840, 3841, 3842, -1, 3843, 3844, 3845, -1, 3846, 3847, 3848, -1, 3849, 3850, 3851, -1, 3852, 3853, 3854, -1, 3855, 3856, 3857, -1, 3858, 3859, 3860, -1, 3861, 3862, 3863, -1, 3864, 3865, 3866, -1, 3867, 3868, 3869, -1, 3870, 3871, 3872, -1, 3873, 3874, 3875, -1, 3876, 3877, 3878, -1, 3879, 3880, 3881, -1, 3882, 3883, 3884, -1, 3885, 3886, 3887, -1, 3888, 3889, 3890, -1, 3891, 3892, 3893, -1, 3894, 3895, 3896, -1, 3897, 3898, 3899, -1, 3900, 3901, 3902, -1, 3903, 3904, 3905, -1, 3906, 3907, 3908, -1, 3909, 3910, 3911, -1, 3912, 3913, 3914, -1, 3915, 3916, 3917, -1, 3918, 3919, 3920, -1, 3921, 3922, 3923, -1, 3924, 3925, 3926, -1, 3927, 3928, 3929, -1, 3930, 3931, 3932, -1, 3933, 3934, 3935, -1, 3936, 3937, 3938, -1, 3939, 3940, 3941, -1, 3942, 3943, 3944, -1, 3945, 3946, 3947, -1, 3948, 3949, 3950, -1, 3951, 3952, 3953, -1, 3954, 3955, 3956, -1, 3957, 3958, 3959, -1, 3960, 3961, 3962, -1, 3963, 3964, 3965, -1, 3966, 3967, 3968, -1, 3969, 3970, 3971, -1, 3972, 3973, 3974, -1, 3975, 3976, 3977, -1, 3978, 3979, 3980, -1, 3981, 3982, 3983, -1, 3984, 3985, 3986, -1, 3987, 3988, 3989, -1, 3990, 3991, 3992, -1, 3993, 3994, 3995, -1, 3996, 3997, 3998, -1, 3999, 4000, 4001, -1, 4002, 4003, 4004, -1, 4005, 4006, 4007, -1, 4008, 4009, 4010, -1, 4011, 4012, 4013, -1, 4014, 4015, 4016, -1, 4017, 4018, 4019, -1, 4020, 4021, 4022, -1, 4023, 4024, 4025, -1, 4026, 4027, 4028, -1, 4029, 4030, 4031, -1, 4032, 4033, 4034, -1, 4035, 4036, 4037, -1, 4038, 4039, 4040, -1, 4041, 4042, 4043, -1, 4044, 4045, 4046, -1, 4047, 4048, 4049, -1, 4050, 4051, 4052, -1, 4053, 4054, 4055, -1, 4056, 4057, 4058, -1, 4059, 4060, 4061, -1, 4062, 4063, 4064, -1, 4065, 4066, 4067, -1, 4068, 4069, 4070, -1, 4071, 4072, 4073, -1, 4074, 4075, 4076, -1, 4077, 4078, 4079, -1, 4080, 4081, 4082, -1, 4083, 4084, 4085, -1, 4086, 4087, 4088, -1, 4089, 4090, 4091, -1, 4092, 4093, 4094, -1, 4095, 4096, 4097, -1, 4098, 4099, 4100, -1, 4101, 4102, 4103, -1, 4104, 4105, 4106, -1, 4107, 4108, 4109, -1, 4110, 4111, 4112, -1, 4113, 4114, 4115, -1, 4116, 4117, 4118, -1, 4119, 4120, 4121, -1, 4122, 4123, 4124, -1, 4125, 4126, 4127, -1, 4128, 4129, 4130, -1, 4131, 4132, 4133, -1, 4134, 4135, 4136, -1, 4137, 4138, 4139, -1, 4140, 4141, 4142, -1, 4143, 4144, 4145, -1, 4146, 4147, 4148, -1, 4149, 4150, 4151, -1, 4152, 4153, 4154, -1, 4155, 4156, 4157, -1, 4158, 4159, 4160, -1, 4161, 4162, 4163, -1, 4164, 4165, 4166, -1, 4167, 4168, 4169, -1, 4170, 4171, 4172, -1, 4173, 4174, 4175, -1, 4176, 4177, 4178, -1, 4179, 4180, 4181, -1, 4182, 4183, 4184, -1, 4185, 4186, 4187, -1, 4188, 4189, 4190, -1, 4191, 4192, 4193, -1, 4194, 4195, 4196, -1, 4197, 4198, 4199, -1, 4200, 4201, 4202, -1, 4203, 4204, 4205, -1, 4206, 4207, 4208, -1, 4209, 4210, 4211, -1, 4212, 4213, 4214, -1, 4215, 4216, 4217, -1, 4218, 4219, 4220, -1, 4221, 4222, 4223, -1, 4224, 4225, 4226, -1, 4227, 4228, 4229, -1, 4230, 4231, 4232, -1, 4233, 4234, 4235, -1, 4236, 4237, 4238, -1, 4239, 4240, 4241, -1, 4242, 4243, 4244, -1, 4245, 4246, 4247, -1, 4248, 4249, 4250, -1, 4251, 4252, 4253, -1, 4254, 4255, 4256, -1, 4257, 4258, 4259, -1, 4260, 4261, 4262, -1, 4263, 4264, 4265, -1, 4266, 4267, 4268, -1, 4269, 4270, 4271, -1, 4272, 4273, 4274, -1, 4275, 4276, 4277, -1, 4278, 4279, 4280, -1, 4281, 4282, 4283, -1, 4284, 4285, 4286, -1, 4287, 4288, 4289, -1, 4290, 4291, 4292, -1, 4293, 4294, 4295, -1, 4296, 4297, 4298, -1, 4299, 4300, 4301, -1, 4302, 4303, 4304, -1, 4305, 4306, 4307, -1, 4308, 4309, 4310, -1, 4311, 4312, 4313, -1, 4314, 4315, 4316, -1, 4317, 4318, 4319, -1, 4320, 4321, 4322, -1, 4323, 4324, 4325, -1, 4326, 4327, 4328, -1, 4329, 4330, 4331, -1, 4332, 4333, 4334, -1, 4335, 4336, 4337, -1, 4338, 4339, 4340, -1, 4341, 4342, 4343, -1, 4344, 4345, 4346, -1, 4347, 4348, 4349, -1, 4350, 4351, 4352, -1, 4353, 4354, 4355, -1, 4356, 4357, 4358, -1, 4359, 4360, 4361, -1, 4362, 4363, 4364, -1, 4365, 4366, 4367, -1, 4368, 4369, 4370, -1, 4371, 4372, 4373, -1, 4374, 4375, 4376, -1, 4377, 4378, 4379, -1, 4380, 4381, 4382, -1, 4383, 4384, 4385, -1, 4386, 4387, 4388, -1, 4389, 4390, 4391, -1, 4392, 4393, 4394, -1, 4395, 4396, 4397, -1, 4398, 4399, 4400, -1, 4401, 4402, 4403, -1, 4404, 4405, 4406, -1, 4407, 4408, 4409, -1, 4410, 4411, 4412, -1, 4413, 4414, 4415, -1, 4416, 4417, 4418, -1, 4419, 4420, 4421, -1, 4422, 4423, 4424, -1, 4425, 4426, 4427, -1, 4428, 4429, 4430, -1, 4431, 4432, 4433, -1, 4434, 4435, 4436, -1, 4437, 4438, 4439, -1, 4440, 4441, 4442, -1, 4443, 4444, 4445, -1, 4446, 4447, 4448, -1, 4449, 4450, 4451, -1, 4452, 4453, 4454, -1, 4455, 4456, 4457, -1, 4458, 4459, 4460, -1, 4461, 4462, 4463, -1, 4464, 4465, 4466, -1, 4467, 4468, 4469, -1, 4470, 4471, 4472, -1, 4473, 4474, 4475, -1, 4476, 4477, 4478, -1, 4479, 4480, 4481, -1, 4482, 4483, 4484, -1, 4485, 4486, 4487, -1, 4488, 4489, 4490, -1, 4491, 4492, 4493, -1, 4494, 4495, 4496, -1, 4497, 4498, 4499, -1, 4500, 4501, 4502, -1, 4503, 4504, 4505, -1, 4506, 4507, 4508, -1, 4509, 4510, 4511, -1, 4512, 4513, 4514, -1, 4515, 4516, 4517, -1, 4518, 4519, 4520, -1, 4521, 4522, 4523, -1, 4524, 4525, 4526, -1, 4527, 4528, 4529, -1, 4530, 4531, 4532, -1, 4533, 4534, 4535, -1, 4536, 4537, 4538, -1, 4539, 4540, 4541, -1, 4542, 4543, 4544, -1, 4545, 4546, 4547, -1, 4548, 4549, 4550, -1, 4551, 4552, 4553, -1, 4554, 4555, 4556, -1, 4557, 4558, 4559, -1, 4560, 4561, 4562, -1, 4563, 4564, 4565, -1, 4566, 4567, 4568, -1, 4569, 4570, 4571, -1, 4572, 4573, 4574, -1, 4575, 4576, 4577, -1, 4578, 4579, 4580, -1, 4581, 4582, 4583, -1, 4584, 4585, 4586, -1, 4587, 4588, 4589, -1, 4590, 4591, 4592, -1, 4593, 4594, 4595, -1, 4596, 4597, 4598, -1, 4599, 4600, 4601, -1, 4602, 4603, 4604, -1, 4605, 4606, 4607, -1, 4608, 4609, 4610, -1, 4611, 4612, 4613, -1, 4614, 4615, 4616, -1, 4617, 4618, 4619, -1, 4620, 4621, 4622, -1, 4623, 4624, 4625, -1, 4626, 4627, 4628, -1, 4629, 4630, 4631, -1, 4632, 4633, 4634, -1, 4635, 4636, 4637, -1, 4638, 4639, 4640, -1, 4641, 4642, 4643, -1, 4644, 4645, 4646, -1, 4647, 4648, 4649, -1, 4650, 4651, 4652, -1, 4653, 4654, 4655, -1, 4656, 4657, 4658, -1, 4659, 4660, 4661, -1, 4662, 4663, 4664, -1, 4665, 4666, 4667, -1, 4668, 4669, 4670, -1, 4671, 4672, 4673, -1, 4674, 4675, 4676, -1, 4677, 4678, 4679, -1, 4680, 4681, 4682, -1, 4683, 4684, 4685, -1, 4686, 4687, 4688, -1, 4689, 4690, 4691, -1, 4692, 4693, 4694, -1, 4695, 4696, 4697, -1, 4698, 4699, 4700, -1, 4701, 4702, 4703, -1, 4704, 4705, 4706, -1, 4707, 4708, 4709, -1, 4710, 4711, 4712, -1, 4713, 4714, 4715, -1, 4716, 4717, 4718, -1, 4719, 4720, 4721, -1, 4722, 4723, 4724, -1, 4725, 4726, 4727, -1, 4728, 4729, 4730, -1, 4731, 4732, 4733, -1, 4734, 4735, 4736, -1, 4737, 4738, 4739, -1, 4740, 4741, 4742, -1, 4743, 4744, 4745, -1, 4746, 4747, 4748, -1, 4749, 4750, 4751, -1, 4752, 4753, 4754, -1, 4755, 4756, 4757, -1, 4758, 4759, 4760, -1, 4761, 4762, 4763, -1, 4764, 4765, 4766, -1, 4767, 4768, 4769, -1, 4770, 4771, 4772, -1, 4773, 4774, 4775, -1, 4776, 4777, 4778, -1, 4779, 4780, 4781, -1, 4782, 4783, 4784, -1, 4785, 4786, 4787, -1, 4788, 4789, 4790, -1, 4791, 4792, 4793, -1, 4794, 4795, 4796, -1, 4797, 4798, 4799, -1, 4800, 4801, 4802, -1, 4803, 4804, 4805, -1, 4806, 4807, 4808, -1, 4809, 4810, 4811, -1, 4812, 4813, 4814, -1, 4815, 4816, 4817, -1, 4818, 4819, 4820, -1, 4821, 4822, 4823, -1, 4824, 4825, 4826, -1, 4827, 4828, 4829, -1, 4830, 4831, 4832, -1, 4833, 4834, 4835, -1, 4836, 4837, 4838, -1, 4839, 4840, 4841, -1, 4842, 4843, 4844, -1, 4845, 4846, 4847, -1, 4848, 4849, 4850, -1, 4851, 4852, 4853, -1, 4854, 4855, 4856, -1, 4857, 4858, 4859, -1, 4860, 4861, 4862, -1, 4863, 4864, 4865, -1, 4866, 4867, 4868, -1, 4869, 4870, 4871, -1, 4872, 4873, 4874, -1, 4875, 4876, 4877, -1, 4878, 4879, 4880, -1, 4881, 4882, 4883, -1, 4884, 4885, 4886, -1, 4887, 4888, 4889, -1, 4890, 4891, 4892, -1, 4893, 4894, 4895, -1, 4896, 4897, 4898, -1, 4899, 4900, 4901, -1, 4902, 4903, 4904, -1, 4905, 4906, 4907, -1, 4908, 4909, 4910, -1, 4911, 4912, 4913, -1, 4914, 4915, 4916, -1, 4917, 4918, 4919, -1, 4920, 4921, 4922, -1, 4923, 4924, 4925, -1, 4926, 4927, 4928, -1, 4929, 4930, 4931, -1, 4932, 4933, 4934, -1, 4935, 4936, 4937, -1, 4938, 4939, 4940, -1, 4941, 4942, 4943, -1, 4944, 4945, 4946, -1, 4947, 4948, 4949, -1, 4950, 4951, 4952, -1, 4953, 4954, 4955, -1, 4956, 4957, 4958, -1, 4959, 4960, 4961, -1, 4962, 4963, 4964, -1, 4965, 4966, 4967, -1, 4968, 4969, 4970, -1, 4971, 4972, 4973, -1, 4974, 4975, 4976, -1, 4977, 4978, 4979, -1, 4980, 4981, 4982, -1, 4983, 4984, 4985, -1, 4986, 4987, 4988, -1, 4989, 4990, 4991, -1, 4992, 4993, 4994, -1, 4995, 4996, 4997, -1, 4998, 4999, 5000, -1, 5001, 5002, 5003, -1, 5004, 5005, 5006, -1, 5007, 5008, 5009, -1, 5010, 5011, 5012, -1, 5013, 5014, 5015, -1, 5016, 5017, 5018, -1, 5019, 5020, 5021, -1, 5022, 5023, 5024, -1, 5025, 5026, 5027, -1, 5028, 5029, 5030, -1, 5031, 5032, 5033, -1, 5034, 5035, 5036, -1, 5037, 5038, 5039, -1, 5040, 5041, 5042, -1, 5043, 5044, 5045, -1, 5046, 5047, 5048, -1, 5049, 5050, 5051, -1, 5052, 5053, 5054, -1, 5055, 5056, 5057, -1, 5058, 5059, 5060, -1, 5061, 5062, 5063, -1, 5064, 5065, 5066, -1, 5067, 5068, 5069, -1, 5070, 5071, 5072, -1, 5073, 5074, 5075, -1, 5076, 5077, 5078, -1, 5079, 5080, 5081, -1, 5082, 5083, 5084, -1, 5085, 5086, 5087, -1, 5088, 5089, 5090, -1, 5091, 5092, 5093, -1, 5094, 5095, 5096, -1, 5097, 5098, 5099, -1, 5100, 5101, 5102, -1, 5103, 5104, 5105, -1, 5106, 5107, 5108, -1, 5109, 5110, 5111, -1, 5112, 5113, 5114, -1, 5115, 5116, 5117, -1, 5118, 5119, 5120, -1, 5121, 5122, 5123, -1, 5124, 5125, 5126, -1, 5127, 5128, 5129, -1, 5130, 5131, 5132, -1, 5133, 5134, 5135, -1, 5136, 5137, 5138, -1, 5139, 5140, 5141, -1, 5142, 5143, 5144, -1, 5145, 5146, 5147, -1, 5148, 5149, 5150, -1, 5151, 5152, 5153, -1, 5154, 5155, 5156, -1, 5157, 5158, 5159, -1, 5160, 5161, 5162, -1, 5163, 5164, 5165, -1, 5166, 5167, 5168, -1, 5169, 5170, 5171, -1, 5172, 5173, 5174, -1, 5175, 5176, 5177, -1, 5178, 5179, 5180, -1, 5181, 5182, 5183, -1, 5184, 5185, 5186, -1, 5187, 5188, 5189, -1, 5190, 5191, 5192, -1, 5193, 5194, 5195, -1, 5196, 5197, 5198, -1, 5199, 5200, 5201, -1, 5202, 5203, 5204, -1, 5205, 5206, 5207, -1, 5208, 5209, 5210, -1, 5211, 5212, 5213, -1, 5214, 5215, 5216, -1, 5217, 5218, 5219, -1, 5220, 5221, 5222, -1, 5223, 5224, 5225, -1, 5226, 5227, 5228, -1, 5229, 5230, 5231, -1, 5232, 5233, 5234, -1, 5235, 5236, 5237, -1, 5238, 5239, 5240, -1, 5241, 5242, 5243, -1, 5244, 5245, 5246, -1, 5247, 5248, 5249, -1, 5250, 5251, 5252, -1, 5253, 5254, 5255, -1, 5256, 5257, 5258, -1, 5259, 5260, 5261, -1, 5262, 5263, 5264, -1, 5265, 5266, 5267, -1, 5268, 5269, 5270, -1, 5271, 5272, 5273, -1, 5274, 5275, 5276, -1, 5277, 5278, 5279, -1, 5280, 5281, 5282, -1, 5283, 5284, 5285, -1, 5286, 5287, 5288, -1, 5289, 5290, 5291, -1, 5292, 5293, 5294, -1, 5295, 5296, 5297, -1, 5298, 5299, 5300, -1, 5301, 5302, 5303, -1, 5304, 5305, 5306, -1, 5307, 5308, 5309, -1, 5310, 5311, 5312, -1, 5313, 5314, 5315, -1, 5316, 5317, 5318, -1, 5319, 5320, 5321, -1, 5322, 5323, 5324, -1, 5325, 5326, 5327, -1, 5328, 5329, 5330, -1, 5331, 5332, 5333, -1, 5334, 5335, 5336, -1, 5337, 5338, 5339, -1, 5340, 5341, 5342, -1, 5343, 5344, 5345, -1, 5346, 5347, 5348, -1, 5349, 5350, 5351, -1, 5352, 5353, 5354, -1, 5355, 5356, 5357, -1, 5358, 5359, 5360, -1, 5361, 5362, 5363, -1, 5364, 5365, 5366, -1, 5367, 5368, 5369, -1, 5370, 5371, 5372, -1, 5373, 5374, 5375, -1, 5376, 5377, 5378, -1, 5379, 5380, 5381, -1, 5382, 5383, 5384, -1, 5385, 5386, 5387, -1, 5388, 5389, 5390, -1, 5391, 5392, 5393, -1, 5394, 5395, 5396, -1, 5397, 5398, 5399, -1, 5400, 5401, 5402, -1, 5403, 5404, 5405, -1, 5406, 5407, 5408, -1, 5409, 5410, 5411, -1, 5412, 5413, 5414, -1, 5415, 5416, 5417, -1, 5418, 5419, 5420, -1, 5421, 5422, 5423, -1, 5424, 5425, 5426, -1, 5427, 5428, 5429, -1, 5430, 5431, 5432, -1, 5433, 5434, 5435, -1, 5436, 5437, 5438, -1, 5439, 5440, 5441, -1, 5442, 5443, 5444, -1, 5445, 5446, 5447, -1, 5448, 5449, 5450, -1, 5451, 5452, 5453, -1, 5454, 5455, 5456, -1, 5457, 5458, 5459, -1, 5460, 5461, 5462, -1, 5463, 5464, 5465, -1, 5466, 5467, 5468, -1, 5469, 5470, 5471, -1, 5472, 5473, 5474, -1, 5475, 5476, 5477, -1, 5478, 5479, 5480, -1, 5481, 5482, 5483, -1, 5484, 5485, 5486, -1, 5487, 5488, 5489, -1, 5490, 5491, 5492, -1, 5493, 5494, 5495, -1, 5496, 5497, 5498, -1, 5499, 5500, 5501, -1, 5502, 5503, 5504, -1, 5505, 5506, 5507, -1, 5508, 5509, 5510, -1, 5511, 5512, 5513, -1, 5514, 5515, 5516, -1, 5517, 5518, 5519, -1, 5520, 5521, 5522, -1, 5523, 5524, 5525, -1, 5526, 5527, 5528, -1, 5529, 5530, 5531, -1, 5532, 5533, 5534, -1, 5535, 5536, 5537, -1, 5538, 5539, 5540, -1, 5541, 5542, 5543, -1, 5544, 5545, 5546, -1, 5547, 5548, 5549, -1, 5550, 5551, 5552, -1, 5553, 5554, 5555, -1, 5556, 5557, 5558, -1, 5559, 5560, 5561, -1, 5562, 5563, 5564, -1, 5565, 5566, 5567, -1, 5568, 5569, 5570, -1, 5571, 5572, 5573, -1, 5574, 5575, 5576, -1, 5577, 5578, 5579, -1, 5580, 5581, 5582, -1, 5583, 5584, 5585, -1, 5586, 5587, 5588, -1, 5589, 5590, 5591, -1, 5592, 5593, 5594, -1, 5595, 5596, 5597, -1, 5598, 5599, 5600, -1, 5601, 5602, 5603, -1, 5604, 5605, 5606, -1, 5607, 5608, 5609, -1, 5610, 5611, 5612, -1, 5613, 5614, 5615, -1, 5616, 5617, 5618, -1, 5619, 5620, 5621, -1, 5622, 5623, 5624, -1, 5625, 5626, 5627, -1, 5628, 5629, 5630, -1, 5631, 5632, 5633, -1, 5634, 5635, 5636, -1, 5637, 5638, 5639, -1, 5640, 5641, 5642, -1, 5643, 5644, 5645, -1, 5646, 5647, 5648, -1, 5649, 5650, 5651, -1, 5652, 5653, 5654, -1, 5655, 5656, 5657, -1, 5658, 5659, 5660, -1, 5661, 5662, 5663, -1, 5664, 5665, 5666, -1, 5667, 5668, 5669, -1, 5670, 5671, 5672, -1, 5673, 5674, 5675, -1, 5676, 5677, 5678, -1, 5679, 5680, 5681, -1, 5682, 5683, 5684, -1, 5685, 5686, 5687, -1, 5688, 5689, 5690, -1, 5691, 5692, 5693, -1, 5694, 5695, 5696, -1, 5697, 5698, 5699, -1, 5700, 5701, 5702, -1, 5703, 5704, 5705, -1, 5706, 5707, 5708, -1, 5709, 5710, 5711, -1, 5712, 5713, 5714, -1, 5715, 5716, 5717, -1, 5718, 5719, 5720, -1, 5721, 5722, 5723, -1, 5724, 5725, 5726, -1, 5727, 5728, 5729, -1, 5730, 5731, 5732, -1, 5733, 5734, 5735, -1, 5736, 5737, 5738, -1, 5739, 5740, 5741, -1, 5742, 5743, 5744, -1, 5745, 5746, 5747, -1, 5748, 5749, 5750, -1, 5751, 5752, 5753, -1, 5754, 5755, 5756, -1, 5757, 5758, 5759, -1, 5760, 5761, 5762, -1, 5763, 5764, 5765, -1, 5766, 5767, 5768, -1, 5769, 5770, 5771, -1, 5772, 5773, 5774, -1, 5775, 5776, 5777, -1, 5778, 5779, 5780, -1, 5781, 5782, 5783, -1, 5784, 5785, 5786, -1, 5787, 5788, 5789, -1, 5790, 5791, 5792, -1, 5793, 5794, 5795, -1, 5796, 5797, 5798, -1, 5799, 5800, 5801, -1, 5802, 5803, 5804, -1, 5805, 5806, 5807, -1, 5808, 5809, 5810, -1, 5811, 5812, 5813, -1, 5814, 5815, 5816, -1, 5817, 5818, 5819, -1, 5820, 5821, 5822, -1, 5823, 5824, 5825, -1, 5826, 5827, 5828, -1, 5829, 5830, 5831, -1, 5832, 5833, 5834, -1, 5835, 5836, 5837, -1, 5838, 5839, 5840, -1, 5841, 5842, 5843, -1, 5844, 5845, 5846, -1, 5847, 5848, 5849, -1, 5850, 5851, 5852, -1, 5853, 5854, 5855, -1, 5856, 5857, 5858, -1, 5859, 5860, 5861, -1, 5862, 5863, 5864, -1, 5865, 5866, 5867, -1, 5868, 5869, 5870, -1, 5871, 5872, 5873, -1, 5874, 5875, 5876, -1, 5877, 5878, 5879, -1, 5880, 5881, 5882, -1, 5883, 5884, 5885, -1, 5886, 5887, 5888, -1, 5889, 5890, 5891, -1, 5892, 5893, 5894, -1, 5895, 5896, 5897, -1, 5898, 5899, 5900, -1, 5901, 5902, 5903, -1, 5904, 5905, 5906, -1, 5907, 5908, 5909, -1, 5910, 5911, 5912, -1, 5913, 5914, 5915, -1, 5916, 5917, 5918, -1, 5919, 5920, 5921, -1, 5922, 5923, 5924, -1, 5925, 5926, 5927, -1, 5928, 5929, 5930, -1, 5931, 5932, 5933, -1, 5934, 5935, 5936, -1, 5937, 5938, 5939, -1, 5940, 5941, 5942, -1, 5943, 5944, 5945, -1, 5946, 5947, 5948, -1, 5949, 5950, 5951, -1, 5952, 5953, 5954, -1, 5955, 5956, 5957, -1, 5958, 5959, 5960, -1, 5961, 5962, 5963, -1, 5964, 5965, 5966, -1, 5967, 5968, 5969, -1, 5970, 5971, 5972, -1, 5973, 5974, 5975, -1, 5976, 5977, 5978, -1, 5979, 5980, 5981, -1, 5982, 5983, 5984, -1, 5985, 5986, 5987, -1, 5988, 5989, 5990, -1, 5991, 5992, 5993, -1 };
    Text_69.texCoordIndex = m_texCoordIndex;
    Text_69.texCoordIndex_length = sizeof(m_texCoordIndex) / sizeof(int);
    }
    {
    static int m_set_colorIndex[1];
    Text_69.set_colorIndex = NULL;
    Text_69.set_colorIndex_length = 0;
    }
    {
    static int m_set_coordIndex[1];
    Text_69.set_coordIndex = NULL;
    Text_69.set_coordIndex_length = 0;
    }
    {
    static int m_set_normalIndex[1];
    Text_69.set_normalIndex = NULL;
    Text_69.set_normalIndex_length = 0;
    }
    {
    static int m_set_texCoordIndex[1];
    Text_69.set_texCoordIndex = NULL;
    Text_69.set_texCoordIndex_length = 0;
    }
    Shape_302.appearance = NULL;
    Shape_302.geometry = &Text_69;
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_301.rotation = m_rotation;
    Transform_301.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 3 };
    Transform_301.translation = m_translation;
    Transform_301.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_301.bboxCenter = m_bboxCenter;
    Transform_301.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_301.bboxSize = m_bboxSize;
    Transform_301.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_301.center = m_center;
    Transform_301.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_301.scale = m_scale;
    Transform_301.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_301.scaleOrientation = m_scaleOrientation;
    Transform_301.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Shape_302 };
    Transform_301.children = m_children;
    Transform_301.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_301.addChildren = NULL;
    Transform_301.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_301.removeChildren = NULL;
    Transform_301.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    SCHRIFTm_6.rotation = m_rotation;
    SCHRIFTm_6.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, 0 };
    SCHRIFTm_6.translation = m_translation;
    SCHRIFTm_6.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    SCHRIFTm_6.bboxCenter = m_bboxCenter;
    SCHRIFTm_6.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    SCHRIFTm_6.bboxSize = m_bboxSize;
    SCHRIFTm_6.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    SCHRIFTm_6.center = m_center;
    SCHRIFTm_6.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    SCHRIFTm_6.scale = m_scale;
    SCHRIFTm_6.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    SCHRIFTm_6.scaleOrientation = m_scaleOrientation;
    SCHRIFTm_6.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Transform_233, &Transform_301 };
    SCHRIFTm_6.children = m_children;
    SCHRIFTm_6.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    SCHRIFTm_6.addChildren = NULL;
    SCHRIFTm_6.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    SCHRIFTm_6.removeChildren = NULL;
    SCHRIFTm_6.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &SCHRIFTm_6 };
    Anchor2.children = m_children;
    Anchor2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Anchor2.bboxCenter = m_bboxCenter;
    Anchor2.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Anchor2.bboxSize = m_bboxSize;
    Anchor2.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    Anchor2.description = "restart game";
    {
    static const char* m_parameter[1];
    Anchor2.parameter = NULL;
    Anchor2.parameter_length = 0;
    }
    {
    static const char* m_url[] = { "warbird.wrl" };
    Anchor2.url = m_url;
    Anchor2.url_length = sizeof(m_url) / sizeof(const char*);
    }
    {
    static X3dNode* m_addChildren[1];
    Anchor2.addChildren = NULL;
    Anchor2.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Anchor2.removeChildren = NULL;
    Anchor2.removeChildren_length = 0;
    }
    ROTATORm_6.m_parent = (X3dNode *)&Inline_70;
    {
    static float m_key[] = { 0, 0.500000, 1 };
    ROTATORm_6.key = m_key;
    ROTATORm_6.key_length = sizeof(m_key) / sizeof(float);
    }
    {
    static float m_keyValue[] = { 0, 1, 0, 0, 0, 1, 0, 3.140000, 0, 1, 0, 6.280000 };
    ROTATORm_6.keyValue = m_keyValue;
    ROTATORm_6.keyValue_length = sizeof(m_keyValue) / sizeof(float);
    }
    ROTATORm_6.set_fraction = 0;
    {
    static float m_value_changed[] = { 0, 0, 1, 0 };
    ROTATORm_6.value_changed = m_value_changed;
    ROTATORm_6.value_changed_length = sizeof(m_value_changed) / sizeof(float);
    }
    TIMERm_8.m_parent = (X3dNode *)&Inline_70;
    TIMERm_8.cycleInterval = 10;
    TIMERm_8.enabled = true;
    TIMERm_8.loop = true;
    TIMERm_8.startTime = 0;
    TIMERm_8.stopTime = 0;
    TIMERm_8.cycleTime = 0;
    TIMERm_8.fraction_changed = 0;
    TIMERm_8.isActive = false;
    TIMERm_8.time = 0;
    {
    static X3dNode* m_children[] = { &Background1, &Anchor1, &Anchor2, &ROTATORm_6, &TIMERm_8 };
    Inline_70.children = m_children;
    Inline_70.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Inline_70.bboxCenter = m_bboxCenter;
    Inline_70.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Inline_70.bboxSize = m_bboxSize;
    Inline_70.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    Inline_70.addChildren = NULL;
    Inline_70.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Inline_70.removeChildren = NULL;
    Inline_70.removeChildren_length = 0;
    }
    {
    static float m_rotation[] = { 0, 0, 1, 0 };
    Transform_91.rotation = m_rotation;
    Transform_91.rotation_length = sizeof(m_rotation) / sizeof(float);
    }
    {
    static float m_translation[] = { 0, 0, -10 };
    Transform_91.translation = m_translation;
    Transform_91.translation_length = sizeof(m_translation) / sizeof(float);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    Transform_91.bboxCenter = m_bboxCenter;
    Transform_91.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    Transform_91.bboxSize = m_bboxSize;
    Transform_91.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static float m_center[] = { 0, 0, 0 };
    Transform_91.center = m_center;
    Transform_91.center_length = sizeof(m_center) / sizeof(float);
    }
    {
    static float m_scale[] = { 1, 1, 1 };
    Transform_91.scale = m_scale;
    Transform_91.scale_length = sizeof(m_scale) / sizeof(float);
    }
    {
    static float m_scaleOrientation[] = { 0, 0, 1, 0 };
    Transform_91.scaleOrientation = m_scaleOrientation;
    Transform_91.scaleOrientation_length = sizeof(m_scaleOrientation) / sizeof(float);
    }
    {
    static X3dNode* m_children[] = { &Inline_70 };
    Transform_91.children = m_children;
    Transform_91.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static X3dNode* m_addChildren[1];
    Transform_91.addChildren = NULL;
    Transform_91.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    Transform_91.removeChildren = NULL;
    Transform_91.removeChildren_length = 0;
    }
    {
    static X3dNode* m_children[] = { &Group_71, &Transform_91 };
    Switch2.children = m_children;
    Switch2.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    Switch2.whichChoice = 0;
    LOADm_NEWm_WORLD.m_parent = (X3dNode *)&root;
    {
    static const char* m_url[] = { "javascript:\n\n // field MFInt32 box_in //\n\n // field SFInt32 deleted //\n\n // field SFInt32 waitticks //\n\n // field SFBool victory //\n\n // eventOut SFTime starttime_out //\n\n // eventOut SFInt32 plane_choice //\n\n // eventOut SFInt32 choice_out //\n\n // eventOut SFInt32 victory_out //\n\n   function initialize()\n      {\n      // insert your url at game end here \n      box_in[1]=0;\n      box_in[2]=0;\n      box_in[3]=0;\n      box_in[4]=0;\n      box_in[5]=0;\n      starttime_out=0;\n      plane_choice=0;\n      deleted=waitticks;\n      }\n   function starttime_in(val)\n      {\n      if (!victory)      \n         if (deleted>0)\n            {\n            if (deleted==waitticks)\n               {\n               starttime_out=val;\n               choice_out=0;\n               }\n            deleted--;\n            }         \n         else {\n            choice_out=-1;\n        }\n      }\n   function test_end()\n      {\n      deleted=waitticks;\n      if ((box_in[1]+box_in[2]+box_in[3]+box_in[4]+box_in[5])==5)\n         {\n         victory=TRUE;\n         victory_out=1; \n         }\n      }\n   function test_plane(val)\n      {\n      if (plane_choice==0)\n         plane_choice=1;\n      else\n         plane_choice=0;\n      }\n   function box1_in(val)\n      {\n      test_plane(val);\n      box_in[1]=val;\n      test_end();\n      }\n   function box2_in(val)\n      {\n      test_plane(val);\n      box_in[2]=val;\n      test_end();\n      }\n   function box3_in(val)\n      {\n      test_plane(val);\n      box_in[3]=val;\n      test_end();\n      }\n   function box4_in(val)\n      {\n      test_plane(val);\n      box_in[4]=val;\n      test_end();\n      }\n   function box5_in(val)\n      {\n      test_plane(val);\n      box_in[5]=val;\n      test_end();\n      }   \n", "c++:\n      X3dScript_LOAD_NEW_WORLD *self = (X3dScript_LOAD_NEW_WORLD *)node;\n      static bool LOAD_NEW_WORLD_init = false;\n      if (!LOAD_NEW_WORLD_init) {\n          self->box_in[0] = 0;\n          self->box_in[1] = 0;\n          self->box_in[2] = 0;\n          self->box_in[3] = 0;\n          self->box_in[4] = 0;\n          self->starttime_out = 0;\n          self->plane_choice = 0;\n          self->deleted = self->waitticks;\n          LOAD_NEW_WORLD_init = true;\n      }         \n      if (!self->victory)      \n         if (self->deleted > 0)\n            {\n            if (self->deleted == self->waitticks)\n               {\n//               starttime_out=val;\n               self->choice_out = 0;\n               }\n            self->deleted--;\n            }         \n         else\n            self->choice_out = -1;\n\n      if ((self->box1_in == 1) && (self->box_in[0] == 0))\n          {\n          self->deleted = self->waitticks;          \n          self->box_in[0] = self->box1_in;\n          if (self->plane_choice == 0)\n              self->plane_choice = 1;\n          else\n              self->plane_choice = 0;\n          }\n      if ((self->box2_in == 1) && (self->box_in[1] == 0))\n          {\n          self->deleted = self->waitticks;          \n          self->box_in[1] = self->box2_in;\n          if (self->plane_choice == 0)\n              self->plane_choice = 1;\n          else\n              self->plane_choice = 0;\n          }\n      if ((self->box3_in == 1) && (self->box_in[2] == 0))\n          {\n          self->deleted = self->waitticks;          \n          self->box_in[2] = self->box3_in;\n          if (self->plane_choice == 0)\n              self->plane_choice = 1;\n          else\n              self->plane_choice = 0;\n          }\n      if ((self->box4_in == 1) && (self->box_in[3] == 0))\n          {\n          self->deleted = self->waitticks;          \n          self->box_in[3] = self->box4_in;\n          if (self->plane_choice == 0)\n              self->plane_choice = 1;\n          else\n              self->plane_choice = 0;\n          }\n      if ((self->box5_in == 1) && (self->box_in[4] == 0))\n          {\n          self->deleted = self->waitticks;          \n          self->box_in[4] = self->box5_in;\n          if (self->plane_choice == 0)\n              self->plane_choice = 1;\n          else\n              self->plane_choice = 0;\n          }\n      if ((self->box_in[0] + \n           self->box_in[1] + \n           self->box_in[2] + \n           self->box_in[3] +\n           self->box_in[4]) == 5)\n          {\n          self->victory = true;\n          self->victory_out = 1;\n          }\n" };
    LOADm_NEWm_WORLD.url = m_url;
    LOADm_NEWm_WORLD.url_length = sizeof(m_url) / sizeof(const char*);
    }
    LOADm_NEWm_WORLD.directOutput = false;
    LOADm_NEWm_WORLD.mustEvaluate = false;
    {
    static int m_box_in[] = { 0, 0, 0, 0, 0 };
    LOADm_NEWm_WORLD.box_in = m_box_in;
    LOADm_NEWm_WORLD.box_in_length = sizeof(m_box_in) / sizeof(int);
    }
    {
    static const char* m_your_url[] = { "" };
    LOADm_NEWm_WORLD.your_url = m_your_url;
    LOADm_NEWm_WORLD.your_url_length = sizeof(m_your_url) / sizeof(const char*);
    }
    {
    static const char* m_param[] = { "" };
    LOADm_NEWm_WORLD.param = m_param;
    LOADm_NEWm_WORLD.param_length = sizeof(m_param) / sizeof(const char*);
    }
    LOADm_NEWm_WORLD.deleted = 0;
    LOADm_NEWm_WORLD.waitticks = 300;
    LOADm_NEWm_WORLD.victory = false;
    LOADm_NEWm_WORLD.starttime_in = 0;
    LOADm_NEWm_WORLD.box1_in = 0;
    LOADm_NEWm_WORLD.box2_in = 0;
    LOADm_NEWm_WORLD.box3_in = 0;
    LOADm_NEWm_WORLD.box4_in = 0;
    LOADm_NEWm_WORLD.box5_in = 0;
    LOADm_NEWm_WORLD.starttime_out = 0;
    LOADm_NEWm_WORLD.plane_choice = 0;
    LOADm_NEWm_WORLD.choice_out = 0;
    LOADm_NEWm_WORLD.victory_out = 0;
    {
    static X3dNode* m_children[] = { &Switch2, &LOADm_NEWm_WORLD };
    root.children = m_children;
    root.children_length = sizeof(m_children) / sizeof(X3dNode*);
    }
    {
    static float m_bboxCenter[] = { 0, 0, 0 };
    root.bboxCenter = m_bboxCenter;
    root.bboxCenter_length = sizeof(m_bboxCenter) / sizeof(float);
    }
    {
    static float m_bboxSize[] = { -1, -1, -1 };
    root.bboxSize = m_bboxSize;
    root.bboxSize_length = sizeof(m_bboxSize) / sizeof(float);
    }
    {
    static X3dNode* m_addChildren[1];
    root.addChildren = NULL;
    root.addChildren_length = 0;
    }
    {
    static X3dNode* m_removeChildren[1];
    root.removeChildren = NULL;
    root.removeChildren_length = 0;
    }
    X3dScript_BOX1_HIT_CHECK::processEventCallback = X3dScript_BOX1_HIT_CHECKSendEventsCallback;
    X3dScript_BOX1_MOVE_ON::processEventCallback = X3dScript_BOX1_MOVE_ONSendEventsCallback;
    X3dScript_BOX1_HIDE_SCRIPT::processEventCallback = X3dScript_BOX1_HIDE_SCRIPTSendEventsCallback;
    X3dScript_STEER::processEventCallback = X3dScript_STEERSendEventsCallback;
    X3dScript_ROT_TRANS1::processEventCallback = X3dScript_ROT_TRANS1SendEventsCallback;
    X3dScript_ROT_TRANS2::processEventCallback = X3dScript_ROT_TRANS2SendEventsCallback;
    X3dScript_WALL_FIRE::processEventCallback = X3dScript_WALL_FIRESendEventsCallback;
    X3dScript_ROLL_TURN::processEventCallback = X3dScript_ROLL_TURNSendEventsCallback;
    X3dScript_UP_DOWN_TURN::processEventCallback = X3dScript_UP_DOWN_TURNSendEventsCallback;
    X3dScript_FLY_FORWARD::processEventCallback = X3dScript_FLY_FORWARDSendEventsCallback;
    X3dScript_FLY_ON::processEventCallback = X3dScript_FLY_ONSendEventsCallback;
    X3dScript_THING_SCRIPT::processEventCallback = X3dScript_THING_SCRIPTSendEventsCallback;
    X3dScript_BOX2_HIT_CHECK::processEventCallback = X3dScript_BOX2_HIT_CHECKSendEventsCallback;
    X3dScript_BOX2_MOVE_ON::processEventCallback = X3dScript_BOX2_MOVE_ONSendEventsCallback;
    X3dScript_BOX2_HIDE_SCRIPT::processEventCallback = X3dScript_BOX2_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX3_HIT_CHECK::processEventCallback = X3dScript_BOX3_HIT_CHECKSendEventsCallback;
    X3dScript_BOX3_MOVE_ON::processEventCallback = X3dScript_BOX3_MOVE_ONSendEventsCallback;
    X3dScript_BOX3_HIDE_SCRIPT::processEventCallback = X3dScript_BOX3_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX4_HIT_CHECK::processEventCallback = X3dScript_BOX4_HIT_CHECKSendEventsCallback;
    X3dScript_BOX4_MOVE_ON::processEventCallback = X3dScript_BOX4_MOVE_ONSendEventsCallback;
    X3dScript_BOX4_HIDE_SCRIPT::processEventCallback = X3dScript_BOX4_HIDE_SCRIPTSendEventsCallback;
    X3dScript_BOX5_HIT_CHECK::processEventCallback = X3dScript_BOX5_HIT_CHECKSendEventsCallback;
    X3dScript_BOX5_MOVE_ON::processEventCallback = X3dScript_BOX5_MOVE_ONSendEventsCallback;
    X3dScript_BOX5_HIDE_SCRIPT::processEventCallback = X3dScript_BOX5_HIDE_SCRIPTSendEventsCallback;
    X3dScript_LOAD_NEW_WORLD::processEventCallback = X3dScript_LOAD_NEW_WORLDSendEventsCallback;
}
X3dNode *X3dSceneGraph::getNodeFromGlName(int glName) {
    switch (glName) {
       case 0:
         return &IndexedFaceSet_1;
       case 1:
         return &IndexedFaceSet_3;
       case 2:
         return &IndexedFaceSet_6;
       case 3:
         return &IndexedFaceSet_7;
       case 4:
         return &IndexedFaceSet_8;
       case 5:
         return &IndexedFaceSet_9;
       case 6:
         return &IndexedFaceSet_11;
       case 7:
         return &IndexedFaceSet_12;
       case 8:
         return &IndexedFaceSet_13;
       case 9:
         return &IndexedFaceSet_14;
       case 10:
         return &IndexedFaceSet_15;
       case 11:
         return &IndexedFaceSet_18;
       case 12:
         return &IndexedFaceSet_23;
       case 13:
         return &IndexedFaceSet_26;
       case 14:
         return &Text1;
       case 15:
         return &Text1m_5;
       case 16:
         return &IndexedFaceSet_29;
       case 17:
         return &IndexedFaceSet_30;
       case 18:
         return &IndexedFaceSet_32;
       case 19:
         return &IndexedFaceSet_35;
       case 20:
         return &IndexedFaceSet_37;
       case 21:
         return &IndexedFaceSet_40;
       case 22:
         return &IndexedFaceSet_42;
       case 23:
         return &IndexedFaceSet_45;
       case 24:
         return &IndexedFaceSet_47;
       case 25:
         return &IndexedFaceSet_50;
       case 26:
         return &IndexedFaceSet_52;
       case 27:
         return &IndexedFaceSet_53;
       case 28:
         return &IndexedFaceSet_54;
       case 29:
         return &IndexedFaceSet_55;
       case 30:
         return &IndexedFaceSet_57;
       case 31:
         return &IndexedFaceSet_58;
       case 32:
         return &IndexedFaceSet_59;
       case 33:
         return &IndexedFaceSet_60;
       case 34:
         return &IndexedFaceSet_61;
       case 35:
         return &IndexedFaceSet_64;
       case 36:
         return &Text_69;
    }
    return NULL;
}
void X3dMFBoolSendEvent(bool* target, bool* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFColorSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFColorRGBASendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFDoubleSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFFloatSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFInt32SendEvent(int* target, int* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFMatrix3fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFMatrix4fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFNodeSendEvent(X3dNode** target, X3dNode** source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFRotationSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFStringSendEvent(const char** target, const char** source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFTimeSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec2dSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec2fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec3dSendEvent(double* target, double* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec3fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dMFVec4fSendEvent(float* target, float* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dSFBoolSendEvent(bool* target, bool source) {
    *target = source;
}
void X3dSFColorSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dSFColorRGBASendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 4; i++)
        target[i] = source[i];
}
void X3dSFDoubleSendEvent(double* target, double source) {
    *target = source;
}
void X3dSFFloatSendEvent(float* target, float source) {
    *target = source;
}
void X3dSFImageSendEvent(int* target, int* source, int size) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < size; i++)
        target[i] = source[i];
}
void X3dSFInt32SendEvent(int* target, int source) {
    *target = source;
}
void X3dSFMatrix3fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 9; i++)
        target[i] = source[i];
}
void X3dSFMatrix4fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 9; i++)
        target[i] = source[i];
}
void X3dSFNodeSendEvent(X3dNode** target, X3dNode* source) {
    *target = source;
}
void X3dSFRotationSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 4; i++)
        target[i] = source[i];
}
void X3dSFStringSendEvent(const char** target, const char* source) {
    *target = source;
}
void X3dSFTimeSendEvent(double* target, double source) {
    *target = source;
}
void X3dSFVec2dSendEvent(double* target, double* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 2; i++)
        target[i] = source[i];
}
void X3dSFVec2fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 2; i++)
        target[i] = source[i];
}
void X3dSFVec3dSendEvent(double* target, double* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dSFVec3fSendEvent(float* target, float* source) {
    int i;
    if ((target == NULL) || (source == NULL)) return;
    for (i = 0; i < 3; i++)
        target[i] = source[i];
}
void X3dProcessEvents(X3dSceneGraph *self, void *data) {
    int nextEvent = 0;
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX1m_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dPlaneSensor::processEventCallback != NULL) {
        nextEvent = X3dPlaneSensor::processEventCallback(&self->WALLm_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->WALLm_SENSOR2, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->THINGm_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dTimeSensor::processEventCallback != NULL) {
        nextEvent = X3dTimeSensor::processEventCallback(&self->TIMER, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX2m_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX3m_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX4m_SENSOR, "", data);
        if (nextEvent) {
        }
}
    if (X3dTouchSensor::processEventCallback != NULL) {
        nextEvent = X3dTouchSensor::processEventCallback(&self->BOX5m_SENSOR, "", data);
        if (nextEvent) {
        }
}
}
